*========================PROGRAM: IQFMTS.SAS ================================;
*============================================================================;
*  Title:
*         Format Library for Inpatient Quality Indicators Program;
*
*  Description:
*          FORMAT LIB FOR AHRQ INPATIENT QUALITY INDICATORS;
*
*          >>>    VERSION 5.0 ICD-10-CM Enhanced Alpha, DECEMBER 2014     <<<
*
*          Note:  Codes valid from 10/1/1994 to 9/30/2015 (FY 2015)
*
*============================================================================;
FILENAME CONTROL 'C:\PATHNAME\CONTROL_IQI.SAS'; *<===USER MUST modify;

%INCLUDE CONTROL;


PROC FORMAT DDNAME=library;

 /* -------------------------------------------------------------- */
 /* --- FORMATS FOR STRATIFIERS ---------------------------------- */
 /* -------------------------------------------------------------- */

 /* --- FORMAT FOR SEX CATEGORY ---------------------------------- */
VALUE SEXCAT
 1 = 'Male'
 2 = 'Female'
  ;

 /* --- FORMAT FOR AGE CATEGORY ---------------------------------- */
VALUE AGECAT
 0 = '00 to 17'
 1 = '18 to 39'
 2 = '40 to 64'
 3 = '65 to 74'
 4 = '75+'
  ;

 /* --- FORMAT FOR RACE CATEGORY --------------------------------- */
VALUE RACECAT
 1 = 'White'
 2 = 'Black'
 3 = 'Hispanic'
 4 = 'Asian and NH/PI'
 5 = 'Amer Indian/AN'
 6 = 'Other'
  ;

 /* --- FORMAT FOR PAYER CATEGORY ---------------------------------- */
VALUE PAYCAT
 1 = 'Medicare'
 2 = 'Medicaid'
 3 = 'Private'
 4 = 'Self Pay'
 5 = 'No Charge'
 6 = 'Other'
  ;

 * -------------------------------------------------------------- ;
 * --- IQI DEFINING FORMATS ------------------------------------- ;
 * -------------------------------------------------------------- ;

 * -------------------------------------------------------------- ;
 * --- FORMATS - PROVIDER LEVEL MORTALITY INDICATORS ------------ ;
 * -------------------------------------------------------------- ;

  /* ----------------------------------------- */
 /* Palliative Care                           */
 /* ----------------------------------------- */

 VALUE $PALLIAD /* no mapping for this from UCD */
   other = '0'
   ;

 /* ----------------------------------------- */
 /* Mortality after Craniotomy                */
 /*   Mortality (post-op) Indicator / IQI #13 */
 /* ----------------------------------------- */

 /* Craniotomy DRGs  */
 /* CMS DRGs: */

VALUE $CRANIOTG
'001' = '1' /* CRANIOTOMY AGE >17 W CC */
'002' = '1' /* CRANIOTOMY AGE >17 W/O CC */
'528' = '1' /* INTRACRANIAL VASCULAR PROC W PDX HEMORRHAGE */
'529' = '1' /* VENTRICULAR SHUNT PROCEDURES W CC */
'530' = '1' /* VENTRICULAR SHUNT PROCEDURES W/O CC */
'543' = '1' /* CRANIOTOMY W MAJOR DEVICE IMPLANT OR ACUTE COMPLEX CNS PRINCIPAL DIAGNOSIS */
OTHER = '0' ;

 /* MS DRGs: */

VALUE $CRANIO2G
'020' = '1' /* Intracranial vascular procedures w PDX hemorrhage w MCC */
'021' = '1' /* Intracranial vascular procedures w PDX hemorrhage w CC */
'022' = '1' /* Intracranial vascular procedures w PDX hemorrhage w/o CC/MCC */
'023' = '1' /* Cranio w major dev impl/acute complex CNS PDX w MCC or chemo implant */
'024' = '1' /* Cranio w major dev impl/acute complex CNS PDX w/o MCC */
'025' = '1' /* Craniotomy & endovascular intracranial procedures w MCC */
'026' = '1' /* Craniotomy & endovascular intracranial procedures w CC */
'027' = '1' /* Craniotomy & endovascular intracranial procedures w/o CC/MCC */
'031' = '1' /* Ventricular shunt procedures w MCC */
'032' = '1' /* Ventricular shunt procedures w CC */
'033' = '1' /* Ventricular shunt procedures w/o CC/MCC */
OTHER = '0' ;

 /* EXCLUDE: HEAD TRAUMA */
 /* ICD-10-CM DIAGNOSIS CODES ( INCLUDES 4TH AND 5TH DIGITS): */

VALUE $HTRAUID
"S020XXA"  =  "1"  /* Fracture of vault of skull, initial encounter for closed fracture */
"S020XXB"  =  "1"  /* Fracture of vault of skull, initial encounter for open fracture */
"S0210XA"  =  "1"  /* Unspecified fracture of base of skull, initial encounter for closed fracture */
"S0210XB"  =  "1"  /* Unspecified fracture of base of skull, initial encounter for open fracture */
"S02110A"  =  "1"  /* Type I occipital condyle fracture, initial encounter for closed fracture */
"S02110B"  =  "1"  /* Type I occipital condyle fracture, initial encounter for open fracture */
"S02111A"  =  "1"  /* Type II occipital condyle fracture, initial encounter for closed fracture */
"S02111B"  =  "1"  /* Type II occipital condyle fracture, initial encounter for open fracture */
"S02112A"  =  "1"  /* Type III occipital condyle fracture, initial encounter for closed fracture */
"S02112B"  =  "1"  /* Type III occipital condyle fracture, initial encounter for open fracture */
"S02113A"  =  "1"  /* Unspecified occipital condyle fracture, initial encounter for closed fracture */
"S02113B"  =  "1"  /* Unspecified occipital condyle fracture, initial encounter for open fracture */
"S02118A"  =  "1"  /* Other fracture of occiput, initial encounter for closed fracture */
"S02118B"  =  "1"  /* Other fracture of occiput, initial encounter for open fracture */
"S02119A"  =  "1"  /* Unspecified fracture of occiput, initial encounter for closed fracture */
"S02119B"  =  "1"  /* Unspecified fracture of occiput, initial encounter for open fracture */
"S0219XA"  =  "1"  /* Other fracture of base of skull, initial encounter for closed fracture */
"S0219XB"  =  "1"  /* Other fracture of base of skull, initial encounter for open fracture */
"S022XXA"  =  "1"  /* Fracture of nasal bones, initial encounter for closed fracture */
"S022XXB"  =  "1"  /* Fracture of nasal bones, initial encounter for open fracture */
"S023XXA"  =  "1"  /* Fracture of orbital floor, initial encounter for closed fracture */
"S023XXB"  =  "1"  /* Fracture of orbital floor, initial encounter for open fracture */
"S02400A"  =  "1"  /* Malar fracture unspecified, initial encounter for closed fracture */
"S02400B"  =  "1"  /* Malar fracture unspecified, initial encounter for open fracture */
"S02401A"  =  "1"  /* Maxillary fracture, unspecified, initial encounter for closed fracture */
"S02401B"  =  "1"  /* Maxillary fracture, unspecified, initial encounter for open fracture */
"S02402A"  =  "1"  /* Zygomatic fracture, unspecified, initial encounter for closed fracture */
"S02402B"  =  "1"  /* Zygomatic fracture, unspecified, initial encounter for open fracture */
"S02411A"  =  "1"  /* LeFort I fracture, initial encounter for closed fracture */
"S02411B"  =  "1"  /* LeFort I fracture, initial encounter for open fracture */
"S02412A"  =  "1"  /* LeFort II fracture, initial encounter for closed fracture */
"S02412B"  =  "1"  /* LeFort II fracture, initial encounter for open fracture */
"S02413A"  =  "1"  /* LeFort III fracture, initial encounter for closed fracture */
"S02413B"  =  "1"  /* LeFort III fracture, initial encounter for open fracture */
"S0242XA"  =  "1"  /* Fracture of alveolus of maxilla, initial encounter for closed fracture */
"S0242XB"  =  "1"  /* Fracture of alveolus of maxilla, initial encounter for open fracture */
"S02600A"  =  "1"  /* Fracture of unspecified part of body of mandible, initial encounter for closed fracture */
"S02600B"  =  "1"  /* Fracture of unspecified part of body of mandible, initial encounter for open fracture */
"S02609A"  =  "1"  /* Fracture of mandible, unspecified, initial encounter for closed fracture */
"S02609B"  =  "1"  /* Fracture of mandible, unspecified, initial encounter for open fracture */
"S0261XA"  =  "1"  /* Fracture of condylar process of mandible, initial encounter for closed fracture */
"S0261XB"  =  "1"  /* Fracture of condylar process of mandible, initial encounter for open fracture */
"S0262XA"  =  "1"  /* Fracture of subcondylar process of mandible, initial encounter for closed fracture */
"S0262XB"  =  "1"  /* Fracture of subcondylar process of mandible, initial encounter for open fracture */
"S0263XA"  =  "1"  /* Fracture of coronoid process of mandible, initial encounter for closed fracture */
"S0263XB"  =  "1"  /* Fracture of coronoid process of mandible, initial encounter for open fracture */
"S0264XA"  =  "1"  /* Fracture of ramus of mandible, initial encounter for closed fracture */
"S0264XB"  =  "1"  /* Fracture of ramus of mandible, initial encounter for open fracture */
"S0265XA"  =  "1"  /* Fracture of angle of mandible, initial encounter for closed fracture */
"S0265XB"  =  "1"  /* Fracture of angle of mandible, initial encounter for open fracture */
"S0266XA"  =  "1"  /* Fracture of symphysis of mandible, initial encounter for closed fracture */
"S0266XB"  =  "1"  /* Fracture of symphysis of mandible, initial encounter for open fracture */
"S0267XA"  =  "1"  /* Fracture of alveolus of mandible, initial encounter for closed fracture */
"S0267XB"  =  "1"  /* Fracture of alveolus of mandible, initial encounter for open fracture */
"S0269XA"  =  "1"  /* Fracture of mandible of other specified site, initial encounter for closed fracture */
"S0269XB"  =  "1"  /* Fracture of mandible of other specified site, initial encounter for open fracture */
"S028XXA"  =  "1"  /* Fractures of other specified skull and facial bones, initial encounter for closed fracture */
"S028XXB"  =  "1"  /* Fractures of other specified skull and facial bones, initial encounter for open fracture */
"S0291XA"  =  "1"  /* Unspecified fracture of skull, initial encounter for closed fracture */
"S0291XB"  =  "1"  /* Unspecified fracture of skull, initial encounter for open fracture */
"S0292XA"  =  "1"  /* Unspecified fracture of facial bones, initial encounter for closed fracture */
"S0292XB"  =  "1"  /* Unspecified fracture of facial bones, initial encounter for open fracture */
"S04011A"  =  "1"  /* Injury of optic nerve, right eye, initial encounter */
"S04012A"  =  "1"  /* Injury of optic nerve, left eye, initial encounter */
"S04019A"  =  "1"  /* Injury of optic nerve, unspecified eye, initial encounter */
"S0402XA"  =  "1"  /* Injury of optic chiasm, initial encounter */
"S04031A"  =  "1"  /* Injury of optic tract and pathways, right eye, initial encounter */
"S04032A"  =  "1"  /* Injury of optic tract and pathways, left eye, initial encounter */
"S04039A"  =  "1"  /* Injury of optic tract and pathways, unspecified eye, initial encounter */
"S04041A"  =  "1"  /* Injury of visual cortex, right eye, initial encounter */
"S04042A"  =  "1"  /* Injury of visual cortex, left eye, initial encounter */
"S04049A"  =  "1"  /* Injury of visual cortex, unspecified eye, initial encounter */
"S0410XA"  =  "1"  /* Injury of oculomotor nerve, unspecified side, initial encounter */
"S0411XA"  =  "1"  /* Injury of oculomotor nerve, right side, initial encounter */
"S0412XA"  =  "1"  /* Injury of oculomotor nerve, left side, initial encounter */
"S0420XA"  =  "1"  /* Injury of trochlear nerve, unspecified side, initial encounter */
"S0421XA"  =  "1"  /* Injury of trochlear nerve, right side, initial encounter */
"S0422XA"  =  "1"  /* Injury of trochlear nerve, left side, initial encounter */
"S0430XA"  =  "1"  /* Injury of trigeminal nerve, unspecified side, initial encounter */
"S0431XA"  =  "1"  /* Injury of trigeminal nerve, right side, initial encounter */
"S0432XA"  =  "1"  /* Injury of trigeminal nerve, left side, initial encounter */
"S0440XA"  =  "1"  /* Injury of abducent nerve, unspecified side, initial encounter */
"S0441XA"  =  "1"  /* Injury of abducent nerve, right side, initial encounter */
"S0442XA"  =  "1"  /* Injury of abducent nerve, left side, initial encounter */
"S0450XA"  =  "1"  /* Injury of facial nerve, unspecified side, initial encounter */
"S0451XA"  =  "1"  /* Injury of facial nerve, right side, initial encounter */
"S0452XA"  =  "1"  /* Injury of facial nerve, left side, initial encounter */
"S0460XA"  =  "1"  /* Injury of acoustic nerve, unspecified side, initial encounter */
"S0461XA"  =  "1"  /* Injury of acoustic nerve, right side, initial encounter */
"S0462XA"  =  "1"  /* Injury of acoustic nerve, left side, initial encounter */
"S0470XA"  =  "1"  /* Injury of accessory nerve, unspecified side, initial encounter */
"S0471XA"  =  "1"  /* Injury of accessory nerve, right side, initial encounter */
"S0472XA"  =  "1"  /* Injury of accessory nerve, left side, initial encounter */
"S04811A"  =  "1"  /* Injury of olfactory [1st ] nerve, right side, initial encounter */
"S04812A"  =  "1"  /* Injury of olfactory [1st ] nerve, left side, initial encounter */
"S04819A"  =  "1"  /* Injury of olfactory [1st ] nerve, unspecified side, initial encounter */
"S04891A"  =  "1"  /* Injury of other cranial nerves, right side, initial encounter */
"S04892A"  =  "1"  /* Injury of other cranial nerves, left side, initial encounter */
"S04899A"  =  "1"  /* Injury of other cranial nerves, unspecified side, initial encounter */
"S049XXA"  =  "1"  /* Injury of unspecified cranial nerve, initial encounter */
"S060X0A"  =  "1"  /* Concussion without loss of consciousness, initial encounter */
"S060X1A"  =  "1"  /* Concussion with loss of consciousness of 30 minutes or less, initial encounter */
"S060X2A"  =  "1"  /* Concussion with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S060X3A"  =  "1"  /* Concussion with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter */
"S060X4A"  =  "1"  /* Concussion with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S060X5A"  =  "1"  /* Concussion with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S060X6A"  =  "1"  /* Concussion with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S060X7A"  =  "1"  /* Concussion with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S060X8A"  =  "1"  /* Concussion with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S060X9A"  =  "1"  /* Concussion with loss of consciousness of unspecified duration, initial encounter */
"S061X0A"  =  "1"  /* Traumatic cerebral edema without loss of consciousness, initial encounter */
"S061X1A"  =  "1"  /* Traumatic cerebral edema with loss of consciousness of 30 minutes or less, initial encounter */
"S061X2A"  =  "1"  /* Traumatic cerebral edema with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S061X3A"  =  "1"  /* Traumatic cerebral edema with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter */
"S061X4A"  =  "1"  /* Traumatic cerebral edema with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S061X5A"  =  "1"  /* Traumatic cerebral edema with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S061X6A"  =  "1"  /* Traumatic cerebral edema with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S061X7A"  =  "1"  /* Traumatic cerebral edema with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S061X8A"  =  "1"  /* Traumatic cerebral edema with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S061X9A"  =  "1"  /* Traumatic cerebral edema with loss of consciousness of unspecified duration, initial encounter */
"S062X0A"  =  "1"  /* Diffuse traumatic brain injury without loss of consciousness, initial encounter */
"S062X1A"  =  "1"  /* Diffuse traumatic brain injury with loss of consciousness of 30 minutes or less, initial encounter */
"S062X2A"  =  "1"  /* Diffuse traumatic brain injury with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S062X3A"  =  "1"  /* Diffuse traumatic brain injury with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter */
"S062X4A"  =  "1"  /* Diffuse traumatic brain injury with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S062X5A"  =  "1"  /* Diffuse traumatic brain injury with loss of consciousness greater than 24 hours with return to pre-existing conscious levels, initial encounter */
"S062X6A"  =  "1"  /* Diffuse traumatic brain injury with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S062X7A"  =  "1"  /* Diffuse traumatic brain injury with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S062X8A"  =  "1"  /* Diffuse traumatic brain injury with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S062X9A"  =  "1"  /* Diffuse traumatic brain injury with loss of consciousness of unspecified duration, initial encounter */
"S06300A"  =  "1"  /* Unspecified focal traumatic brain injury without loss of consciousness, initial encounter */
"S06301A"  =  "1"  /* Unspecified focal traumatic brain injury with loss of consciousness of 30 minutes or less, initial encounter */
"S06302A"  =  "1"  /* Unspecified focal traumatic brain injury with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S06303A"  =  "1"  /* Unspecified focal traumatic brain injury with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter */
"S06304A"  =  "1"  /* Unspecified focal traumatic brain injury with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S06305A"  =  "1"  /* Unspecified focal traumatic brain injury with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S06306A"  =  "1"  /* Unspecified focal traumatic brain injury with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S06307A"  =  "1"  /* Unspecified focal traumatic brain injury with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S06308A"  =  "1"  /* Unspecified focal traumatic brain injury with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S06309A"  =  "1"  /* Unspecified focal traumatic brain injury with loss of consciousness of unspecified duration, initial encounter */
"S06310A"  =  "1"  /* Contusion and laceration of right cerebrum without loss of consciousness, initial encounter */
"S06311A"  =  "1"  /* Contusion and laceration of right cerebrum with loss of consciousness of 30 minutes or less, initial encounter */
"S06312A"  =  "1"  /* Contusion and laceration of right cerebrum with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S06313A"  =  "1"  /* Contusion and laceration of right cerebrum with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter */
"S06314A"  =  "1"  /* Contusion and laceration of right cerebrum with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S06315A"  =  "1"  /* Contusion and laceration of right cerebrum with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S06316A"  =  "1"  /* Contusion and laceration of right cerebrum with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S06317A"  =  "1"  /* Contusion and laceration of right cerebrum with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S06318A"  =  "1"  /* Contusion and laceration of right cerebrum with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S06319A"  =  "1"  /* Contusion and laceration of right cerebrum with loss of consciousness of unspecified duration, initial encounter */
"S06320A"  =  "1"  /* Contusion and laceration of left cerebrum without loss of consciousness, initial encounter */
"S06321A"  =  "1"  /* Contusion and laceration of left cerebrum with loss of consciousness of 30 minutes or less, initial encounter */
"S06322A"  =  "1"  /* Contusion and laceration of left cerebrum with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S06323A"  =  "1"  /* Contusion and laceration of left cerebrum with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter */
"S06324A"  =  "1"  /* Contusion and laceration of left cerebrum with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S06325A"  =  "1"  /* Contusion and laceration of left cerebrum with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S06326A"  =  "1"  /* Contusion and laceration of left cerebrum with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S06327A"  =  "1"  /* Contusion and laceration of left cerebrum with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S06328A"  =  "1"  /* Contusion and laceration of left cerebrum with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S06329A"  =  "1"  /* Contusion and laceration of left cerebrum with loss of consciousness of unspecified duration, initial encounter */
"S06330A"  =  "1"  /* Contusion and laceration of cerebrum, unspecified, without loss of consciousness, initial encounter */
"S06331A"  =  "1"  /* Contusion and laceration of cerebrum, unspecified, with loss of consciousness of 30 minutes or less, initial encounter */
"S06332A"  =  "1"  /* Contusion and laceration of cerebrum, unspecified, with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S06333A"  =  "1"  /* Contusion and laceration of cerebrum, unspecified, with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter */
"S06334A"  =  "1"  /* Contusion and laceration of cerebrum, unspecified, with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S06335A"  =  "1"  /* Contusion and laceration of cerebrum, unspecified, with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S06336A"  =  "1"  /* Contusion and laceration of cerebrum, unspecified, with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S06337A"  =  "1"  /* Contusion and laceration of cerebrum, unspecified, with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S06338A"  =  "1"  /* Contusion and laceration of cerebrum, unspecified, with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S06339A"  =  "1"  /* Contusion and laceration of cerebrum, unspecified, with loss of consciousness of unspecified duration, initial encounter */
"S06340A"  =  "1"  /* Traumatic hemorrhage of right cerebrum without loss of consciousness, initial encounter */
"S06341A"  =  "1"  /* Traumatic hemorrhage of right cerebrum with loss of consciousness of 30 minutes or less, initial encounter */
"S06342A"  =  "1"  /* Traumatic hemorrhage of right cerebrum with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S06343A"  =  "1"  /* Traumatic hemorrhage of right cerebrum with loss of consciousness of 1 hours to 5 hours 59 minutes, initial encounter */
"S06344A"  =  "1"  /* Traumatic hemorrhage of right cerebrum with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S06345A"  =  "1"  /* Traumatic hemorrhage of right cerebrum with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S06346A"  =  "1"  /* Traumatic hemorrhage of right cerebrum with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S06347A"  =  "1"  /* Traumatic hemorrhage of right cerebrum with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S06348A"  =  "1"  /* Traumatic hemorrhage of right cerebrum with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S06349A"  =  "1"  /* Traumatic hemorrhage of right cerebrum with loss of consciousness of unspecified duration, initial encounter */
"S06350A"  =  "1"  /* Traumatic hemorrhage of left cerebrum without loss of consciousness, initial encounter */
"S06351A"  =  "1"  /* Traumatic hemorrhage of left cerebrum with loss of consciousness of 30 minutes or less, initial encounter */
"S06352A"  =  "1"  /* Traumatic hemorrhage of left cerebrum with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S06353A"  =  "1"  /* Traumatic hemorrhage of left cerebrum with loss of consciousness of 1 hours to 5 hours 59 minutes, initial encounter */
"S06354A"  =  "1"  /* Traumatic hemorrhage of left cerebrum with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S06355A"  =  "1"  /* Traumatic hemorrhage of left cerebrum with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S06356A"  =  "1"  /* Traumatic hemorrhage of left cerebrum with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S06357A"  =  "1"  /* Traumatic hemorrhage of left cerebrum with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S06358A"  =  "1"  /* Traumatic hemorrhage of left cerebrum with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S06359A"  =  "1"  /* Traumatic hemorrhage of left cerebrum with loss of consciousness of unspecified duration, initial encounter */
"S06360A"  =  "1"  /* Traumatic hemorrhage of cerebrum, unspecified, without loss of consciousness, initial encounter */
"S06361A"  =  "1"  /* Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of 30 minutes or less, initial encounter */
"S06362A"  =  "1"  /* Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S06363A"  =  "1"  /* Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of 1 hours to 5 hours 59 minutes, initial encounter */
"S06364A"  =  "1"  /* Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S06365A"  =  "1"  /* Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S06366A"  =  "1"  /* Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S06367A"  =  "1"  /* Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S06368A"  =  "1"  /* Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S06369A"  =  "1"  /* Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of unspecified duration, initial encounter */
"S06370A"  =  "1"  /* Contusion, laceration, and hemorrhage of cerebellum without loss of consciousness, initial encounter */
"S06371A"  =  "1"  /* Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness of 30 minutes or less, initial encounter */
"S06372A"  =  "1"  /* Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S06373A"  =  "1"  /* Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter */
"S06374A"  =  "1"  /* Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S06375A"  =  "1"  /* Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S06376A"  =  "1"  /* Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S06377A"  =  "1"  /* Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S06378A"  =  "1"  /* Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S06379A"  =  "1"  /* Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness of unspecified duration, initial encounter */
"S06380A"  =  "1"  /* Contusion, laceration, and hemorrhage of brainstem without loss of consciousness, initial encounter */
"S06381A"  =  "1"  /* Contusion, laceration, and hemorrhage of brainstem with loss of consciousness of 30 minutes or less, initial encounter */
"S06382A"  =  "1"  /* Contusion, laceration, and hemorrhage of brainstem with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S06383A"  =  "1"  /* Contusion, laceration, and hemorrhage of brainstem with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter */
"S06384A"  =  "1"  /* Contusion, laceration, and hemorrhage of brainstem with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S06385A"  =  "1"  /* Contusion, laceration, and hemorrhage of brainstem with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S06386A"  =  "1"  /* Contusion, laceration, and hemorrhage of brainstem with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S06387A"  =  "1"  /* Contusion, laceration, and hemorrhage of brainstem with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S06388A"  =  "1"  /* Contusion, laceration, and hemorrhage of brainstem with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S06389A"  =  "1"  /* Contusion, laceration, and hemorrhage of brainstem with loss of consciousness of unspecified duration, initial encounter */
"S064X0A"  =  "1"  /* Epidural hemorrhage without loss of consciousness, initial encounter */
"S064X1A"  =  "1"  /* Epidural hemorrhage with loss of consciousness of 30 minutes or less, initial encounter */
"S064X2A"  =  "1"  /* Epidural hemorrhage with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S064X3A"  =  "1"  /* Epidural hemorrhage with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter */
"S064X4A"  =  "1"  /* Epidural hemorrhage with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S064X5A"  =  "1"  /* Epidural hemorrhage with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S064X6A"  =  "1"  /* Epidural hemorrhage with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S064X7A"  =  "1"  /* Epidural hemorrhage with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S064X8A"  =  "1"  /* Epidural hemorrhage with loss of consciousness of any duration with death due to other causes prior to regaining consciousness, initial encounter */
"S064X9A"  =  "1"  /* Epidural hemorrhage with loss of consciousness of unspecified duration, initial encounter */
"S065X0A"  =  "1"  /* Traumatic subdural hemorrhage without loss of consciousness, initial encounter */
"S065X1A"  =  "1"  /* Traumatic subdural hemorrhage with loss of consciousness of 30 minutes or less, initial encounter */
"S065X2A"  =  "1"  /* Traumatic subdural hemorrhage with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S065X3A"  =  "1"  /* Traumatic subdural hemorrhage with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter */
"S065X4A"  =  "1"  /* Traumatic subdural hemorrhage with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S065X5A"  =  "1"  /* Traumatic subdural hemorrhage with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S065X6A"  =  "1"  /* Traumatic subdural hemorrhage with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S065X7A"  =  "1"  /* Traumatic subdural hemorrhage with loss of consciousness of any duration with death due to brain injury before regaining consciousness, initial encounter */
"S065X8A"  =  "1"  /* Traumatic subdural hemorrhage with loss of consciousness of any duration with death due to other cause before regaining consciousness, initial encounter */
"S065X9A"  =  "1"  /* Traumatic subdural hemorrhage with loss of consciousness of unspecified duration, initial encounter */
"S066X0A"  =  "1"  /* Traumatic subarachnoid hemorrhage without loss of consciousness, initial encounter */
"S066X1A"  =  "1"  /* Traumatic subarachnoid hemorrhage with loss of consciousness of 30 minutes or less, initial encounter */
"S066X2A"  =  "1"  /* Traumatic subarachnoid hemorrhage with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S066X3A"  =  "1"  /* Traumatic subarachnoid hemorrhage with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter */
"S066X4A"  =  "1"  /* Traumatic subarachnoid hemorrhage with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S066X5A"  =  "1"  /* Traumatic subarachnoid hemorrhage with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S066X6A"  =  "1"  /* Traumatic subarachnoid hemorrhage with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S066X7A"  =  "1"  /* Traumatic subarachnoid hemorrhage with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S066X8A"  =  "1"  /* Traumatic subarachnoid hemorrhage with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S066X9A"  =  "1"  /* Traumatic subarachnoid hemorrhage with loss of consciousness of unspecified duration, initial encounter */
"S06810A"  =  "1"  /* Injury of right internal carotid artery, intracranial portion, not elsewhere classified without loss of consciousness, initial encounter */
"S06811A"  =  "1"  /* Injury of right internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of 30 minutes or less, initial encounter */
"S06812A"  =  "1"  /* Injury of right internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S06813A"  =  "1"  /* Injury of right internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter */
"S06814A"  =  "1"  /* Injury of right internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S06815A"  =  "1"  /* Injury of right internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S06816A"  =  "1"  /* Injury of right internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S06817A"  =  "1"  /* Injury of right internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S06818A"  =  "1"  /* Injury of right internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S06819A"  =  "1"  /* Injury of right internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of unspecified duration, initial encounter */
"S06820A"  =  "1"  /* Injury of left internal carotid artery, intracranial portion, not elsewhere classified without loss of consciousness, initial encounter */
"S06821A"  =  "1"  /* Injury of left internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of 30 minutes or less, initial encounter */
"S06822A"  =  "1"  /* Injury of left internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S06823A"  =  "1"  /* Injury of left internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter */
"S06824A"  =  "1"  /* Injury of left internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S06825A"  =  "1"  /* Injury of left internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S06826A"  =  "1"  /* Injury of left internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S06827A"  =  "1"  /* Injury of left internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S06828A"  =  "1"  /* Injury of left internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S06829A"  =  "1"  /* Injury of left internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of unspecified duration, initial encounter */
"S06890A"  =  "1"  /* Other specified intracranial injury without loss of consciousness, initial encounter */
"S06891A"  =  "1"  /* Other specified intracranial injury with loss of consciousness of 30 minutes or less, initial encounter */
"S06892A"  =  "1"  /* Other specified intracranial injury with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S06893A"  =  "1"  /* Other specified intracranial injury with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter */
"S06894A"  =  "1"  /* Other specified intracranial injury with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S06895A"  =  "1"  /* Other specified intracranial injury with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S06896A"  =  "1"  /* Other specified intracranial injury with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S06897A"  =  "1"  /* Other specified intracranial injury with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S06898A"  =  "1"  /* Other specified intracranial injury with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S06899A"  =  "1"  /* Other specified intracranial injury with loss of consciousness of unspecified duration, initial encounter */
"S069X0A"  =  "1"  /* Unspecified intracranial injury without loss of consciousness, initial encounter */
"S069X1A"  =  "1"  /* Unspecified intracranial injury with loss of consciousness of 30 minutes or less, initial encounter */
"S069X2A"  =  "1"  /* Unspecified intracranial injury with loss of consciousness of 31 minutes to 59 minutes, initial encounter */
"S069X3A"  =  "1"  /* Unspecified intracranial injury with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter */
"S069X4A"  =  "1"  /* Unspecified intracranial injury with loss of consciousness of 6 hours to 24 hours, initial encounter */
"S069X5A"  =  "1"  /* Unspecified intracranial injury with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter */
"S069X6A"  =  "1"  /* Unspecified intracranial injury with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter */
"S069X7A"  =  "1"  /* Unspecified intracranial injury with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter */
"S069X8A"  =  "1"  /* Unspecified intracranial injury with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter */
"S069X9A"  =  "1"  /* Unspecified intracranial injury with loss of consciousness of unspecified duration, initial encounter */
"S12000A"  =  "1"  /* Unspecified displaced fracture of first cervical vertebra, initial encounter for closed fracture */
"S12000B"  =  "1"  /* Unspecified displaced fracture of first cervical vertebra, initial encounter for open fracture */
"S12001A"  =  "1"  /* Unspecified nondisplaced fracture of first cervical vertebra, initial encounter for closed fracture */
"S12001B"  =  "1"  /* Unspecified nondisplaced fracture of first cervical vertebra, initial encounter for open fracture */
"S1201XA"  =  "1"  /* Stable burst fracture of first cervical vertebra, initial encounter for closed fracture */
"S1201XB"  =  "1"  /* Stable burst fracture of first cervical vertebra, initial encounter for open fracture */
"S1202XA"  =  "1"  /* Unstable burst fracture of first cervical vertebra, initial encounter for closed fracture */
"S1202XB"  =  "1"  /* Unstable burst fracture of first cervical vertebra, initial encounter for open fracture */
"S12030A"  =  "1"  /* Displaced posterior arch fracture of first cervical vertebra, initial encounter for closed fracture */
"S12030B"  =  "1"  /* Displaced posterior arch fracture of first cervical vertebra, initial encounter for open fracture */
"S12031A"  =  "1"  /* Nondisplaced posterior arch fracture of first cervical vertebra, initial encounter for closed fracture */
"S12031B"  =  "1"  /* Nondisplaced posterior arch fracture of first cervical vertebra, initial encounter for open fracture */
"S12040A"  =  "1"  /* Displaced lateral mass fracture of first cervical vertebra, initial encounter for closed fracture */
"S12040B"  =  "1"  /* Displaced lateral mass fracture of first cervical vertebra, initial encounter for open fracture */
"S12041A"  =  "1"  /* Nondisplaced lateral mass fracture of first cervical vertebra, initial encounter for closed fracture */
"S12041B"  =  "1"  /* Nondisplaced lateral mass fracture of first cervical vertebra, initial encounter for open fracture */
"S12090A"  =  "1"  /* Other displaced fracture of first cervical vertebra, initial encounter for closed fracture */
"S12090B"  =  "1"  /* Other displaced fracture of first cervical vertebra, initial encounter for open fracture */
"S12091A"  =  "1"  /* Other nondisplaced fracture of first cervical vertebra, initial encounter for closed fracture */
"S12091B"  =  "1"  /* Other nondisplaced fracture of first cervical vertebra, initial encounter for open fracture */
"S12100A"  =  "1"  /* Unspecified displaced fracture of second cervical vertebra, initial encounter for closed fracture */
"S12100B"  =  "1"  /* Unspecified displaced fracture of second cervical vertebra, initial encounter for open fracture */
"S12101A"  =  "1"  /* Unspecified nondisplaced fracture of second cervical vertebra, initial encounter for closed fracture */
"S12101B"  =  "1"  /* Unspecified nondisplaced fracture of second cervical vertebra, initial encounter for open fracture */
"S12110A"  =  "1"  /* Anterior displaced Type II dens fracture, initial encounter for closed fracture */
"S12110B"  =  "1"  /* Anterior displaced Type II dens fracture, initial encounter for open fracture */
"S12111A"  =  "1"  /* Posterior displaced Type II dens fracture, initial encounter for closed fracture */
"S12111B"  =  "1"  /* Posterior displaced Type II dens fracture, initial encounter for open fracture */
"S12112A"  =  "1"  /* Nondisplaced Type II dens fracture, initial encounter for closed fracture */
"S12112B"  =  "1"  /* Nondisplaced Type II dens fracture, initial encounter for open fracture */
"S12120A"  =  "1"  /* Other displaced dens fracture, initial encounter for closed fracture */
"S12120B"  =  "1"  /* Other displaced dens fracture, initial encounter for open fracture */
"S12121A"  =  "1"  /* Other nondisplaced dens fracture, initial encounter for closed fracture */
"S12121B"  =  "1"  /* Other nondisplaced dens fracture, initial encounter for open fracture */
"S12130A"  =  "1"  /* Unspecified traumatic displaced spondylolisthesis of second cervical vertebra, initial encounter for closed fracture */
"S12130B"  =  "1"  /* Unspecified traumatic displaced spondylolisthesis of second cervical vertebra, initial encounter for open fracture */
"S12131A"  =  "1"  /* Unspecified traumatic nondisplaced spondylolisthesis of second cervical vertebra, initial encounter for closed fracture */
"S12131B"  =  "1"  /* Unspecified traumatic nondisplaced spondylolisthesis of second cervical vertebra, initial encounter for open fracture */
"S1214XA"  =  "1"  /* Type III traumatic spondylolisthesis of second cervical vertebra, initial encounter for closed fracture */
"S1214XB"  =  "1"  /* Type III traumatic spondylolisthesis of second cervical vertebra, initial encounter for open fracture */
"S12150A"  =  "1"  /* Other traumatic displaced spondylolisthesis of second cervical vertebra, initial encounter for closed fracture */
"S12150B"  =  "1"  /* Other traumatic displaced spondylolisthesis of second cervical vertebra, initial encounter for open fracture */
"S12151A"  =  "1"  /* Other traumatic nondisplaced spondylolisthesis of second cervical vertebra, initial encounter for closed fracture */
"S12151B"  =  "1"  /* Other traumatic nondisplaced spondylolisthesis of second cervical vertebra, initial encounter for open fracture */
"S12190A"  =  "1"  /* Other displaced fracture of second cervical vertebra, initial encounter for closed fracture */
"S12190B"  =  "1"  /* Other displaced fracture of second cervical vertebra, initial encounter for open fracture */
"S12191A"  =  "1"  /* Other nondisplaced fracture of second cervical vertebra, initial encounter for closed fracture */
"S12191B"  =  "1"  /* Other nondisplaced fracture of second cervical vertebra, initial encounter for open fracture */
"S12200A"  =  "1"  /* Unspecified displaced fracture of third cervical vertebra, initial encounter for closed fracture */
"S12200B"  =  "1"  /* Unspecified displaced fracture of third cervical vertebra, initial encounter for open fracture */
"S12201A"  =  "1"  /* Unspecified nondisplaced fracture of third cervical vertebra, initial encounter for closed fracture */
"S12201B"  =  "1"  /* Unspecified nondisplaced fracture of third cervical vertebra, initial encounter for open fracture */
"S12230A"  =  "1"  /* Unspecified traumatic displaced spondylolisthesis of third cervical vertebra, initial encounter for closed fracture */
"S12230B"  =  "1"  /* Unspecified traumatic displaced spondylolisthesis of third cervical vertebra, initial encounter for open fracture */
"S12231A"  =  "1"  /* Unspecified traumatic nondisplaced spondylolisthesis of third cervical vertebra, initial encounter for closed fracture */
"S12231B"  =  "1"  /* Unspecified traumatic nondisplaced spondylolisthesis of third cervical vertebra, initial encounter for open fracture */
"S1224XA"  =  "1"  /* Type III traumatic spondylolisthesis of third cervical vertebra, initial encounter for closed fracture */
"S1224XB"  =  "1"  /* Type III traumatic spondylolisthesis of third cervical vertebra, initial encounter for open fracture */
"S12250A"  =  "1"  /* Other traumatic displaced spondylolisthesis of third cervical vertebra, initial encounter for closed fracture */
"S12250B"  =  "1"  /* Other traumatic displaced spondylolisthesis of third cervical vertebra, initial encounter for open fracture */
"S12251A"  =  "1"  /* Other traumatic nondisplaced spondylolisthesis of third cervical vertebra, initial encounter for closed fracture */
"S12251B"  =  "1"  /* Other traumatic nondisplaced spondylolisthesis of third cervical vertebra, initial encounter for open fracture */
"S12290A"  =  "1"  /* Other displaced fracture of third cervical vertebra, initial encounter for closed fracture */
"S12290B"  =  "1"  /* Other displaced fracture of third cervical vertebra, initial encounter for open fracture */
"S12291A"  =  "1"  /* Other nondisplaced fracture of third cervical vertebra, initial encounter for closed fracture */
"S12291B"  =  "1"  /* Other nondisplaced fracture of third cervical vertebra, initial encounter for open fracture */
"S12300A"  =  "1"  /* Unspecified displaced fracture of fourth cervical vertebra, initial encounter for closed fracture */
"S12300B"  =  "1"  /* Unspecified displaced fracture of fourth cervical vertebra, initial encounter for open fracture */
"S12301A"  =  "1"  /* Unspecified nondisplaced fracture of fourth cervical vertebra, initial encounter for closed fracture */
"S12301B"  =  "1"  /* Unspecified nondisplaced fracture of fourth cervical vertebra, initial encounter for open fracture */
"S12330A"  =  "1"  /* Unspecified traumatic displaced spondylolisthesis of fourth cervical vertebra, initial encounter for closed fracture */
"S12330B"  =  "1"  /* Unspecified traumatic displaced spondylolisthesis of fourth cervical vertebra, initial encounter for open fracture */
"S12331A"  =  "1"  /* Unspecified traumatic nondisplaced spondylolisthesis of fourth cervical vertebra, initial encounter for closed fracture */
"S12331B"  =  "1"  /* Unspecified traumatic nondisplaced spondylolisthesis of fourth cervical vertebra, initial encounter for open fracture */
"S1234XA"  =  "1"  /* Type III traumatic spondylolisthesis of fourth cervical vertebra, initial encounter for closed fracture */
"S1234XB"  =  "1"  /* Type III traumatic spondylolisthesis of fourth cervical vertebra, initial encounter for open fracture */
"S12350A"  =  "1"  /* Other traumatic displaced spondylolisthesis of fourth cervical vertebra, initial encounter for closed fracture */
"S12350B"  =  "1"  /* Other traumatic displaced spondylolisthesis of fourth cervical vertebra, initial encounter for open fracture */
"S12351A"  =  "1"  /* Other traumatic nondisplaced spondylolisthesis of fourth cervical vertebra, initial encounter for closed fracture */
"S12351B"  =  "1"  /* Other traumatic nondisplaced spondylolisthesis of fourth cervical vertebra, initial encounter for open fracture */
"S12390A"  =  "1"  /* Other displaced fracture of fourth cervical vertebra, initial encounter for closed fracture */
"S12390B"  =  "1"  /* Other displaced fracture of fourth cervical vertebra, initial encounter for open fracture */
"S12391A"  =  "1"  /* Other nondisplaced fracture of fourth cervical vertebra, initial encounter for closed fracture */
"S12391B"  =  "1"  /* Other nondisplaced fracture of fourth cervical vertebra, initial encounter for open fracture */
"S12400A"  =  "1"  /* Unspecified displaced fracture of fifth cervical vertebra, initial encounter for closed fracture */
"S12400B"  =  "1"  /* Unspecified displaced fracture of fifth cervical vertebra, initial encounter for open fracture */
"S12401A"  =  "1"  /* Unspecified nondisplaced fracture of fifth cervical vertebra, initial encounter for closed fracture */
"S12401B"  =  "1"  /* Unspecified nondisplaced fracture of fifth cervical vertebra, initial encounter for open fracture */
"S12430A"  =  "1"  /* Unspecified traumatic displaced spondylolisthesis of fifth cervical vertebra, initial encounter for closed fracture */
"S12430B"  =  "1"  /* Unspecified traumatic displaced spondylolisthesis of fifth cervical vertebra, initial encounter for open fracture */
"S12431A"  =  "1"  /* Unspecified traumatic nondisplaced spondylolisthesis of fifth cervical vertebra, initial encounter for closed fracture */
"S12431B"  =  "1"  /* Unspecified traumatic nondisplaced spondylolisthesis of fifth cervical vertebra, initial encounter for open fracture */
"S1244XA"  =  "1"  /* Type III traumatic spondylolisthesis of fifth cervical vertebra, initial encounter for closed fracture */
"S1244XB"  =  "1"  /* Type III traumatic spondylolisthesis of fifth cervical vertebra, initial encounter for open fracture */
"S12450A"  =  "1"  /* Other traumatic displaced spondylolisthesis of fifth cervical vertebra, initial encounter for closed fracture */
"S12450B"  =  "1"  /* Other traumatic displaced spondylolisthesis of fifth cervical vertebra, initial encounter for open fracture */
"S12451A"  =  "1"  /* Other traumatic nondisplaced spondylolisthesis of fifth cervical vertebra, initial encounter for closed fracture */
"S12451B"  =  "1"  /* Other traumatic nondisplaced spondylolisthesis of fifth cervical vertebra, initial encounter for open fracture */
"S12490A"  =  "1"  /* Other displaced fracture of fifth cervical vertebra, initial encounter for closed fracture */
"S12490B"  =  "1"  /* Other displaced fracture of fifth cervical vertebra, initial encounter for open fracture */
"S12491A"  =  "1"  /* Other nondisplaced fracture of fifth cervical vertebra, initial encounter for closed fracture */
"S12491B"  =  "1"  /* Other nondisplaced fracture of fifth cervical vertebra, initial encounter for open fracture */
"S12500A"  =  "1"  /* Unspecified displaced fracture of sixth cervical vertebra, initial encounter for closed fracture */
"S12500B"  =  "1"  /* Unspecified displaced fracture of sixth cervical vertebra, initial encounter for open fracture */
"S12501A"  =  "1"  /* Unspecified nondisplaced fracture of sixth cervical vertebra, initial encounter for closed fracture */
"S12501B"  =  "1"  /* Unspecified nondisplaced fracture of sixth cervical vertebra, initial encounter for open fracture */
"S12530A"  =  "1"  /* Unspecified traumatic displaced spondylolisthesis of sixth cervical vertebra, initial encounter for closed fracture */
"S12530B"  =  "1"  /* Unspecified traumatic displaced spondylolisthesis of sixth cervical vertebra, initial encounter for open fracture */
"S12531A"  =  "1"  /* Unspecified traumatic nondisplaced spondylolisthesis of sixth cervical vertebra, initial encounter for closed fracture */
"S12531B"  =  "1"  /* Unspecified traumatic nondisplaced spondylolisthesis of sixth cervical vertebra, initial encounter for open fracture */
"S1254XA"  =  "1"  /* Type III traumatic spondylolisthesis of sixth cervical vertebra, initial encounter for closed fracture */
"S1254XB"  =  "1"  /* Type III traumatic spondylolisthesis of sixth cervical vertebra, initial encounter for open fracture */
"S12550A"  =  "1"  /* Other traumatic displaced spondylolisthesis of sixth cervical vertebra, initial encounter for closed fracture */
"S12550B"  =  "1"  /* Other traumatic displaced spondylolisthesis of sixth cervical vertebra, initial encounter for open fracture */
"S12551A"  =  "1"  /* Other traumatic nondisplaced spondylolisthesis of sixth cervical vertebra, initial encounter for closed fracture */
"S12551B"  =  "1"  /* Other traumatic nondisplaced spondylolisthesis of sixth cervical vertebra, initial encounter for open fracture */
"S12590A"  =  "1"  /* Other displaced fracture of sixth cervical vertebra, initial encounter for closed fracture */
"S12590B"  =  "1"  /* Other displaced fracture of sixth cervical vertebra, initial encounter for open fracture */
"S12591A"  =  "1"  /* Other nondisplaced fracture of sixth cervical vertebra, initial encounter for closed fracture */
"S12591B"  =  "1"  /* Other nondisplaced fracture of sixth cervical vertebra, initial encounter for open fracture */
"S12600A"  =  "1"  /* Unspecified displaced fracture of seventh cervical vertebra, initial encounter for closed fracture */
"S12600B"  =  "1"  /* Unspecified displaced fracture of seventh cervical vertebra, initial encounter for open fracture */
"S12601A"  =  "1"  /* Unspecified nondisplaced fracture of seventh cervical vertebra, initial encounter for closed fracture */
"S12601B"  =  "1"  /* Unspecified nondisplaced fracture of seventh cervical vertebra, initial encounter for open fracture */
"S12630A"  =  "1"  /* Unspecified traumatic displaced spondylolisthesis of seventh cervical vertebra, initial encounter for closed fracture */
"S12630B"  =  "1"  /* Unspecified traumatic displaced spondylolisthesis of seventh cervical vertebra, initial encounter for open fracture */
"S12631A"  =  "1"  /* Unspecified traumatic nondisplaced spondylolisthesis of seventh cervical vertebra, initial encounter for closed fracture */
"S12631B"  =  "1"  /* Unspecified traumatic nondisplaced spondylolisthesis of seventh cervical vertebra, initial encounter for open fracture */
"S1264XA"  =  "1"  /* Type III traumatic spondylolisthesis of seventh cervical vertebra, initial encounter for closed fracture */
"S1264XB"  =  "1"  /* Type III traumatic spondylolisthesis of seventh cervical vertebra, initial encounter for open fracture */
"S12650A"  =  "1"  /* Other traumatic displaced spondylolisthesis of seventh cervical vertebra, initial encounter for closed fracture */
"S12650B"  =  "1"  /* Other traumatic displaced spondylolisthesis of seventh cervical vertebra, initial encounter for open fracture */
"S12651A"  =  "1"  /* Other traumatic nondisplaced spondylolisthesis of seventh cervical vertebra, initial encounter for closed fracture */
"S12651B"  =  "1"  /* Other traumatic nondisplaced spondylolisthesis of seventh cervical vertebra, initial encounter for open fracture */
"S12690A"  =  "1"  /* Other displaced fracture of seventh cervical vertebra, initial encounter for closed fracture */
"S12690B"  =  "1"  /* Other displaced fracture of seventh cervical vertebra, initial encounter for open fracture */
"S12691A"  =  "1"  /* Other nondisplaced fracture of seventh cervical vertebra, initial encounter for closed fracture */
"S12691B"  =  "1"  /* Other nondisplaced fracture of seventh cervical vertebra, initial encounter for open fracture */
"S129XXA"  =  "1"  /* Fracture of neck, unspecified, initial encounter */
"S140XXA"  =  "1"  /* Concussion and edema of cervical spinal cord, initial encounter */
"S14101A"  =  "1"  /* Unspecified injury at C1 level of cervical spinal cord, initial encounter */
"S14102A"  =  "1"  /* Unspecified injury at C2 level of cervical spinal cord, initial encounter */
"S14103A"  =  "1"  /* Unspecified injury at C3 level of cervical spinal cord, initial encounter */
"S14104A"  =  "1"  /* Unspecified injury at C4 level of cervical spinal cord, initial encounter */
"S14105A"  =  "1"  /* Unspecified injury at C5 level of cervical spinal cord, initial encounter */
"S14106A"  =  "1"  /* Unspecified injury at C6 level of cervical spinal cord, initial encounter */
"S14107A"  =  "1"  /* Unspecified injury at C7 level of cervical spinal cord, initial encounter */
"S14108A"  =  "1"  /* Unspecified injury at C8 level of cervical spinal cord, initial encounter */
"S14111A"  =  "1"  /* Complete lesion at C1 level of cervical spinal cord, initial encounter */
"S14112A"  =  "1"  /* Complete lesion at C2 level of cervical spinal cord, initial encounter */
"S14113A"  =  "1"  /* Complete lesion at C3 level of cervical spinal cord, initial encounter */
"S14114A"  =  "1"  /* Complete lesion at C4 level of cervical spinal cord, initial encounter */
"S14115A"  =  "1"  /* Complete lesion at C5 level of cervical spinal cord, initial encounter */
"S14116A"  =  "1"  /* Complete lesion at C6 level of cervical spinal cord, initial encounter */
"S14117A"  =  "1"  /* Complete lesion at C7 level of cervical spinal cord, initial encounter */
"S14118A"  =  "1"  /* Complete lesion at C8 level of cervical spinal cord, initial encounter */
"S14121A"  =  "1"  /* Central cord syndrome at C1 level of cervical spinal cord, initial encounter */
"S14122A"  =  "1"  /* Central cord syndrome at C2 level of cervical spinal cord, initial encounter */
"S14123A"  =  "1"  /* Central cord syndrome at C3 level of cervical spinal cord, initial encounter */
"S14124A"  =  "1"  /* Central cord syndrome at C4 level of cervical spinal cord, initial encounter */
"S14125A"  =  "1"  /* Central cord syndrome at C5 level of cervical spinal cord, initial encounter */
"S14126A"  =  "1"  /* Central cord syndrome at C6 level of cervical spinal cord, initial encounter */
"S14127A"  =  "1"  /* Central cord syndrome at C7 level of cervical spinal cord, initial encounter */
"S14128A"  =  "1"  /* Central cord syndrome at C8 level of cervical spinal cord, initial encounter */
"S14131A"  =  "1"  /* Anterior cord syndrome at C1 level of cervical spinal cord, initial encounter */
"S14132A"  =  "1"  /* Anterior cord syndrome at C2 level of cervical spinal cord, initial encounter */
"S14133A"  =  "1"  /* Anterior cord syndrome at C3 level of cervical spinal cord, initial encounter */
"S14134A"  =  "1"  /* Anterior cord syndrome at C4 level of cervical spinal cord, initial encounter */
"S14135A"  =  "1"  /* Anterior cord syndrome at C5 level of cervical spinal cord, initial encounter */
"S14136A"  =  "1"  /* Anterior cord syndrome at C6 level of cervical spinal cord, initial encounter */
"S14137A"  =  "1"  /* Anterior cord syndrome at C7 level of cervical spinal cord, initial encounter */
"S14138A"  =  "1"  /* Anterior cord syndrome at C8 level of cervical spinal cord, initial encounter */
"S14141A"  =  "1"  /* Brown-Sequard syndrome at C1 level of cervical spinal cord, initial encounter */
"S14142A"  =  "1"  /* Brown-Sequard syndrome at C2 level of cervical spinal cord, initial encounter */
"S14143A"  =  "1"  /* Brown-Sequard syndrome at C3 level of cervical spinal cord, initial encounter */
"S14144A"  =  "1"  /* Brown-Sequard syndrome at C4 level of cervical spinal cord, initial encounter */
"S14145A"  =  "1"  /* Brown-Sequard syndrome at C5 level of cervical spinal cord, initial encounter */
"S14146A"  =  "1"  /* Brown-Sequard syndrome at C6 level of cervical spinal cord, initial encounter */
"S14147A"  =  "1"  /* Brown-Sequard syndrome at C7 level of cervical spinal cord, initial encounter */
"S14148A"  =  "1"  /* Brown-Sequard syndrome at C8 level of cervical spinal cord, initial encounter */
"S14151A"  =  "1"  /* Other incomplete lesion at C1 level of cervical spinal cord, initial encounter */
"S14152A"  =  "1"  /* Other incomplete lesion at C2 level of cervical spinal cord, initial encounter */
"S14153A"  =  "1"  /* Other incomplete lesion at C3 level of cervical spinal cord, initial encounter */
"S14154A"  =  "1"  /* Other incomplete lesion at C4 level of cervical spinal cord, initial encounter */
"S14155A"  =  "1"  /* Other incomplete lesion at C5 level of cervical spinal cord, initial encounter */
"S14156A"  =  "1"  /* Other incomplete lesion at C6 level of cervical spinal cord, initial encounter */
"S14157A"  =  "1"  /* Other incomplete lesion at C7 level of cervical spinal cord, initial encounter */
"S14158A"  =  "1"  /* Other incomplete lesion at C8 level of cervical spinal cord, initial encounter */
"S240XXA"  =  "1"  /* Concussion and edema of thoracic spinal cord, initial encounter */
"S24101A"  =  "1"  /* Unspecified injury at T1 level of thoracic spinal cord, initial encounter */
"S24102A"  =  "1"  /* Unspecified injury at T2-T6 level of thoracic spinal cord, initial encounter */
"S24103A"  =  "1"  /* Unspecified injury at T7-T10 level of thoracic spinal cord, initial encounter */
"S24104A"  =  "1"  /* Unspecified injury at T11-T12 level of thoracic spinal cord, initial encounter */
"S24111A"  =  "1"  /* Complete lesion at T1 level of thoracic spinal cord, initial encounter */
"S24112A"  =  "1"  /* Complete lesion at T2-T6 level of thoracic spinal cord, initial encounter */
"S24113A"  =  "1"  /* Complete lesion at T7-T10 level of thoracic spinal cord, initial encounter */
"S24114A"  =  "1"  /* Complete lesion at T11-T12 level of thoracic spinal cord, initial encounter */
"S24131A"  =  "1"  /* Anterior cord syndrome at T1 level of thoracic spinal cord, initial encounter */
"S24132A"  =  "1"  /* Anterior cord syndrome at T2-T6 level of thoracic spinal cord, initial encounter */
"S24133A"  =  "1"  /* Anterior cord syndrome at T7-T10 level of thoracic spinal cord, initial encounter */
"S24134A"  =  "1"  /* Anterior cord syndrome at T11-T12 level of thoracic spinal cord, initial encounter */
"S24141A"  =  "1"  /* Brown-Sequard syndrome at T1 level of thoracic spinal cord, initial encounter */
"S24142A"  =  "1"  /* Brown-Sequard syndrome at T2-T6 level of thoracic spinal cord, initial encounter */
"S24143A"  =  "1"  /* Brown-Sequard syndrome at T7-T10 level of thoracic spinal cord, initial encounter */
"S24144A"  =  "1"  /* Brown-Sequard syndrome at T11-T12 level of thoracic spinal cord, initial encounter */
"S24151A"  =  "1"  /* Other incomplete lesion at T1 level of thoracic spinal cord, initial encounter */
"S24152A"  =  "1"  /* Other incomplete lesion at T2-T6 level of thoracic spinal cord, initial encounter */
"S24153A"  =  "1"  /* Other incomplete lesion at T7-T10 level of thoracic spinal cord, initial encounter */
"S24154A"  =  "1"  /* Other incomplete lesion at T11-T12 level of thoracic spinal cord, initial encounter */
OTHER = '0' ;

 /* ---------------------------------------- */
 /* Esophageal Cancer Resection (PRESOP)     */
 /*   Volume Indicator / IQI #1              */
 /*   Mortality (post-op) Indicator / IQI #8 */
 /* ---------------------------------------- */

 /* Esophageal resection */
 /* ICD-10-CM Procedure Codes: */

VALUE $PRESOPP
"0D11074"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D11076"  =  "1"  /* Bypass Upper Esophagus to Stomach with Autologous Tissue Substitute, Open Approach */
"0D11079"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1107A"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1107B"  =  "1"  /* Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D110J4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Synthetic Substitute, Open Approach */
"0D110J6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Synthetic Substitute, Open Approach */
"0D110J9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D110JA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D110JB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D110K4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D110K6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach */
"0D110K9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D110KA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D110KB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D110Z4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous, Open Approach */
"0D110Z6"  =  "1"  /* Bypass Upper Esophagus to Stomach, Open Approach */
"0D110Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Open Approach */
"0D110ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Open Approach */
"0D110ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Open Approach */
"0D113J4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D11474"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D11476"  =  "1"  /* Bypass Upper Esophagus to Stomach with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D11479"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1147A"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1147B"  =  "1"  /* Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114J4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114J6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114J9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114JA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114JB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114K4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114K6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114K9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114KA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114KB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114Z4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous, Percutaneous Endoscopic Approach */
"0D114Z6"  =  "1"  /* Bypass Upper Esophagus to Stomach, Percutaneous Endoscopic Approach */
"0D114Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D114ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D114ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D11874"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D11876"  =  "1"  /* Bypass Upper Esophagus to Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D11879"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1187A"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1187B"  =  "1"  /* Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118J4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118J6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118J9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118JA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118JB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118K4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118K6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118K9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118KA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118KB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118Z4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D118Z6"  =  "1"  /* Bypass Upper Esophagus to Stomach, Via Natural or Artificial Opening Endoscopic */
"0D118Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D118ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D118ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D12074"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D12076"  =  "1"  /* Bypass Middle Esophagus to Stomach with Autologous Tissue Substitute, Open Approach */
"0D12079"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1207A"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1207B"  =  "1"  /* Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D120J4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Synthetic Substitute, Open Approach */
"0D120J6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Synthetic Substitute, Open Approach */
"0D120J9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D120JA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D120JB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D120K4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D120K6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach */
"0D120K9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D120KA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D120KB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D120Z4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous, Open Approach */
"0D120Z6"  =  "1"  /* Bypass Middle Esophagus to Stomach, Open Approach */
"0D120Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Open Approach */
"0D120ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Open Approach */
"0D120ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Open Approach */
"0D123J4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D12474"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D12476"  =  "1"  /* Bypass Middle Esophagus to Stomach with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D12479"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1247A"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1247B"  =  "1"  /* Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124J4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124J6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124J9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124JA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124JB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124K4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124K6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124K9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124KA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124KB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124Z4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous, Percutaneous Endoscopic Approach */
"0D124Z6"  =  "1"  /* Bypass Middle Esophagus to Stomach, Percutaneous Endoscopic Approach */
"0D124Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D124ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D124ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D12874"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D12876"  =  "1"  /* Bypass Middle Esophagus to Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D12879"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1287A"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1287B"  =  "1"  /* Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128J4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128J6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128J9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128JA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128JB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128K4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128K6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128K9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128KA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128KB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128Z4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D128Z6"  =  "1"  /* Bypass Middle Esophagus to Stomach, Via Natural or Artificial Opening Endoscopic */
"0D128Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D128ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D128ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D13074"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D13076"  =  "1"  /* Bypass Lower Esophagus to Stomach with Autologous Tissue Substitute, Open Approach */
"0D13079"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1307A"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1307B"  =  "1"  /* Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D130J4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Synthetic Substitute, Open Approach */
"0D130J6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Synthetic Substitute, Open Approach */
"0D130J9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D130JA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D130JB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D130K4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D130K6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach */
"0D130K9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D130KA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D130KB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D130Z4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous, Open Approach */
"0D130Z6"  =  "1"  /* Bypass Lower Esophagus to Stomach, Open Approach */
"0D130Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Open Approach */
"0D130ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Open Approach */
"0D130ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Open Approach */
"0D133J4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D13474"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D13476"  =  "1"  /* Bypass Lower Esophagus to Stomach with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D13479"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1347A"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1347B"  =  "1"  /* Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134J4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134J6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134J9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134JA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134JB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134K4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134K6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134K9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134KA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134KB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134Z4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous, Percutaneous Endoscopic Approach */
"0D134Z6"  =  "1"  /* Bypass Lower Esophagus to Stomach, Percutaneous Endoscopic Approach */
"0D134Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D134ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D134ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D13874"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D13876"  =  "1"  /* Bypass Lower Esophagus to Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D13879"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1387A"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1387B"  =  "1"  /* Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138J4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138J6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138J9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138JA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138JB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138K4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138K6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138K9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138KA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138KB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138Z4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D138Z6"  =  "1"  /* Bypass Lower Esophagus to Stomach, Via Natural or Artificial Opening Endoscopic */
"0D138Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D138ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D138ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D15074"  =  "1"  /* Bypass Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D15076"  =  "1"  /* Bypass Esophagus to Stomach with Autologous Tissue Substitute, Open Approach */
"0D15079"  =  "1"  /* Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1507A"  =  "1"  /* Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1507B"  =  "1"  /* Bypass Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D150J4"  =  "1"  /* Bypass Esophagus to Cutaneous with Synthetic Substitute, Open Approach */
"0D150J6"  =  "1"  /* Bypass Esophagus to Stomach with Synthetic Substitute, Open Approach */
"0D150J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D150JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D150JB"  =  "1"  /* Bypass Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D150K4"  =  "1"  /* Bypass Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D150K6"  =  "1"  /* Bypass Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach */
"0D150K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D150KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D150KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D150Z4"  =  "1"  /* Bypass Esophagus to Cutaneous, Open Approach */
"0D150Z6"  =  "1"  /* Bypass Esophagus to Stomach, Open Approach */
"0D150Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Open Approach */
"0D150ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Open Approach */
"0D150ZB"  =  "1"  /* Bypass Esophagus to Ileum, Open Approach */
"0D153J4"  =  "1"  /* Bypass Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D15474"  =  "1"  /* Bypass Esophagus to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D15476"  =  "1"  /* Bypass Esophagus to Stomach with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D15479"  =  "1"  /* Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1547A"  =  "1"  /* Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1547B"  =  "1"  /* Bypass Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154J4"  =  "1"  /* Bypass Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154J6"  =  "1"  /* Bypass Esophagus to Stomach with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154JB"  =  "1"  /* Bypass Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154K4"  =  "1"  /* Bypass Esophagus to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154K6"  =  "1"  /* Bypass Esophagus to Stomach with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154Z4"  =  "1"  /* Bypass Esophagus to Cutaneous, Percutaneous Endoscopic Approach */
"0D154Z6"  =  "1"  /* Bypass Esophagus to Stomach, Percutaneous Endoscopic Approach */
"0D154Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D154ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D154ZB"  =  "1"  /* Bypass Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D15874"  =  "1"  /* Bypass Esophagus to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D15876"  =  "1"  /* Bypass Esophagus to Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D15879"  =  "1"  /* Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1587A"  =  "1"  /* Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1587B"  =  "1"  /* Bypass Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158J4"  =  "1"  /* Bypass Esophagus to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158J6"  =  "1"  /* Bypass Esophagus to Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158JB"  =  "1"  /* Bypass Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158K4"  =  "1"  /* Bypass Esophagus to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158K6"  =  "1"  /* Bypass Esophagus to Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158Z4"  =  "1"  /* Bypass Esophagus to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D158Z6"  =  "1"  /* Bypass Esophagus to Stomach, Via Natural or Artificial Opening Endoscopic */
"0D158Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D158ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D158ZB"  =  "1"  /* Bypass Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0DB10ZZ"  =  "1"  /* Excision of Upper Esophagus, Open Approach */
"0DB13ZZ"  =  "1"  /* Excision of Upper Esophagus, Percutaneous Approach */
"0DB17ZZ"  =  "1"  /* Excision of Upper Esophagus, Via Natural or Artificial Opening */
"0DB20ZZ"  =  "1"  /* Excision of Middle Esophagus, Open Approach */
"0DB23ZZ"  =  "1"  /* Excision of Middle Esophagus, Percutaneous Approach */
"0DB27ZZ"  =  "1"  /* Excision of Middle Esophagus, Via Natural or Artificial Opening */
"0DB30ZZ"  =  "1"  /* Excision of Lower Esophagus, Open Approach */
"0DB33ZZ"  =  "1"  /* Excision of Lower Esophagus, Percutaneous Approach */
"0DB37ZZ"  =  "1"  /* Excision of Lower Esophagus, Via Natural or Artificial Opening */
"0DB50ZZ"  =  "1"  /* Excision of Esophagus, Open Approach */
"0DB53ZZ"  =  "1"  /* Excision of Esophagus, Percutaneous Approach */
"0DB57ZZ"  =  "1"  /* Excision of Esophagus, Via Natural or Artificial Opening */
"0DT10ZZ"  =  "1"  /* Resection of Upper Esophagus, Open Approach */
"0DT14ZZ"  =  "1"  /* Resection of Upper Esophagus, Percutaneous Endoscopic Approach */
"0DT17ZZ"  =  "1"  /* Resection of Upper Esophagus, Via Natural or Artificial Opening */
"0DT18ZZ"  =  "1"  /* Resection of Upper Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT20ZZ"  =  "1"  /* Resection of Middle Esophagus, Open Approach */
"0DT24ZZ"  =  "1"  /* Resection of Middle Esophagus, Percutaneous Endoscopic Approach */
"0DT27ZZ"  =  "1"  /* Resection of Middle Esophagus, Via Natural or Artificial Opening */
"0DT28ZZ"  =  "1"  /* Resection of Middle Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT30ZZ"  =  "1"  /* Resection of Lower Esophagus, Open Approach */
"0DT34ZZ"  =  "1"  /* Resection of Lower Esophagus, Percutaneous Endoscopic Approach */
"0DT37ZZ"  =  "1"  /* Resection of Lower Esophagus, Via Natural or Artificial Opening */
"0DT38ZZ"  =  "1"  /* Resection of Lower Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT50ZZ"  =  "1"  /* Resection of Esophagus, Open Approach */
"0DT54ZZ"  =  "1"  /* Resection of Esophagus, Percutaneous Endoscopic Approach */
"0DT57ZZ"  =  "1"  /* Resection of Esophagus, Via Natural or Artificial Opening */
"0DT58ZZ"  =  "1"  /* Resection of Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DX60Z5"  =  "1"  /* Transfer Stomach to Esophagus, Open Approach */
"0DX64Z5"  =  "1"  /* Transfer Stomach to Esophagus, Percutaneous Endoscopic Approach */
"0DX80Z5"  =  "1"  /* Transfer Small Intestine to Esophagus, Open Approach */
"0DX84Z5"  =  "1"  /* Transfer Small Intestine to Esophagus, Percutaneous Endoscopic Approach */
"0DXE0Z5"  =  "1"  /* Transfer Large Intestine to Esophagus, Open Approach */
"0DXE4Z5"  =  "1"  /* Transfer Large Intestine to Esophagus, Percutaneous Endoscopic Approach */
OTHER  = '0' ;

 /* Esophageal resection */
 /* ICD-10-CM Procedure Codes: */

VALUE $PRESO2P
"0DT60ZZ"  =  "1"  /* Resection of Stomach, Open Approach */
"0DT64ZZ"  =  "1"  /* Resection of Stomach, Percutaneous Endoscopic Approach */
"0DT67ZZ"  =  "1"  /* Resection of Stomach, Via Natural or Artificial Opening */
"0DT68ZZ"  =  "1"  /* Resection of Stomach, Via Natural or Artificial Opening Endoscopic */
OTHER  = '0' ;

 /* Include Only: Esophageal Cancer */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $PRESOPD
"C153"  =  "1"  /* Malignant neoplasm of upper third of esophagus */
"C154"  =  "1"  /* Malignant neoplasm of middle third of esophagus */
"C155"  =  "1"  /* Malignant neoplasm of lower third of esophagus */
"C158"  =  "1"  /* Malignant neoplasm of overlapping sites of esophagus */
"C159"  =  "1"  /* Malignant neoplasm of esophagus, unspecified */
OTHER  = '0' ;

 /* Include Only: Esophageal Cancer */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $PRESO2D
"C160"  =  "1"  /* Malignant neoplasm of cardia */
"C7880"  =  "1"  /* Secondary malignant neoplasm of unspecified digestive organ */
"C7889"  =  "1"  /* Secondary malignant neoplasm of other digestive organs */
"D001"  =  "1"  /* Carcinoma in situ of esophagus */
"D378"  =  "1"  /* Neoplasm of uncertain behavior of other specified digestive organs */
"D379"  =  "1"  /* Neoplasm of uncertain behavior of digestive organ, unspecified */
OTHER  = '0' ;

 /* ---------------------------------------- */
 /* Pancreatic Cancer Reseciton (PRPANC)     */
 /*   Volume Indicator / IQI #2              */
 /*   Mortality (post-op) Indicator / IQI #9 */
 /* ---------------------------------------- */

 /* Pancreatic Resection */
 /* ICD-10-CM Procedure Codes: */

VALUE $PRPAN3P
"0FBG0ZZ"  =  "1"  /* Excision of Pancreas, Open Approach */
"0FBG3ZZ"  =  "1"  /* Excision of Pancreas, Percutaneous Approach */
"0FBG4ZZ"  =  "1"  /* Excision of Pancreas, Percutaneous Endoscopic Approach */
OTHER  = '0' ;

VALUE $PRPANCP
"0FTG0ZZ"  =  "1"  /* Resection of Pancreas, Open Approach */
"0FTG4ZZ"  =  "1"  /* Resection of Pancreas, Percutaneous Endoscopic Approach */
OTHER  = '0' ;

 /* Include Only: Pancreatic Cancer */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $PRPANCD
"C170"  =  "1"  /* Malignant neoplasm of duodenum */
"C240"  =  "1"  /* Malignant neoplasm of extrahepatic bile duct */
"C241"  =  "1"  /* Malignant neoplasm of ampulla of Vater */
"C250"  =  "1"  /* Malignant neoplasm of head of pancreas */
"C251"  =  "1"  /* Malignant neoplasm of body of pancreas */
"C252"  =  "1"  /* Malignant neoplasm of tail of pancreas */
"C253"  =  "1"  /* Malignant neoplasm of pancreatic duct */
"C254"  =  "1"  /* Malignant neoplasm of endocrine pancreas */
"C257"  =  "1"  /* Malignant neoplasm of other parts of pancreas */
"C258"  =  "1"  /* Malignant neoplasm of overlapping sites of pancreas */
"C259"  =  "1"  /* Malignant neoplasm of pancreas, unspecified */
OTHER  = '0' ;

 /* Exclude: Acute Pancreatitis */

VALUE $PRPAN2D
"B252"  =  "1"  /* Cytomegaloviral pancreatitis */
"B263"  =  "1"  /* Mumps pancreatitis */
"K850"  =  "1"  /* Idiopathic acute pancreatitis */
"K851"  =  "1"  /* Biliary acute pancreatitis */
"K852"  =  "1"  /* Alcohol induced acute pancreatitis */
"K853"  =  "1"  /* Drug induced acute pancreatitis */
"K858"  =  "1"  /* Other acute pancreatitis */
"K859"  =  "1"  /* Acute pancreatitis, unspecified */
OTHER   = '0' ;



 /* ----------------------------------------- */
 /* Abdominal Aortic Aneurysm Repair (PRAAAR) */
 /*   Volume Indicator / IQI #4               */
 /*   Mortality (post-op) Indicator / IQI #11 */
 /* ----------------------------------------- */

 /* AAA Repair */
 /* ICD-10-CM Procedure Codes: */

VALUE $PRAAARP
"04500ZZ"  =  "1"  /* Destruction of Abdominal Aorta, Open Approach */
"04503ZZ"  =  "1"  /* Destruction of Abdominal Aorta, Percutaneous Approach */
"04504ZZ"  =  "1"  /* Destruction of Abdominal Aorta, Percutaneous Endoscopic Approach */
"04B00ZZ"  =  "1"  /* Excision of Abdominal Aorta, Open Approach */
"04B03ZZ"  =  "1"  /* Excision of Abdominal Aorta, Percutaneous Approach */
"04B04ZZ"  =  "1"  /* Excision of Abdominal Aorta, Percutaneous Endoscopic Approach */
"04R007Z"  =  "1"  /* Replacement of Abdominal Aorta with Autologous Tissue Substitute, Open Approach */
"04R00JZ"  =  "1"  /* Replacement of Abdominal Aorta with Synthetic Substitute, Open Approach */
"04R00KZ"  =  "1"  /* Replacement of Abdominal Aorta with Nonautologous Tissue Substitute, Open Approach */
"04R047Z"  =  "1"  /* Replacement of Abdominal Aorta with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R04JZ"  =  "1"  /* Replacement of Abdominal Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04R04KZ"  =  "1"  /* Replacement of Abdominal Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
OTHER  = '0' ;

VALUE $PRAAA2P
"04U03JZ"  =  "1"  /* Supplement Abdominal Aorta with Synthetic Substitute, Percutaneous Approach */
"04U04JZ"  =  "1"  /* Supplement Abdominal Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04V03DZ"  =  "1"  /* Restriction of Abdominal Aorta with Intraluminal Device, Percutaneous Approach */
"04V04DZ"  =  "1"  /* Restriction of Abdominal Aorta with Intraluminal Device, Percutaneous Endoscopic Approach */
OTHER  = '0' ;

 /* Include Only: AAA */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $PRAAARD
"I713"  =  "1"  /* Abdominal aortic aneurysm, ruptured */
OTHER  = '0' ;

VALUE $PRAAA2D
"I714"  =  "1"  /* Abdominal aortic aneurysm, without rupture */
OTHER  = '0' ;

 /* ----------------------------------------- */
 /* CABG (PRCABG)                             */
 /*   Volume Indicator / IQI #5               */
 /*   Mortality (post-op) Indicator / IQI #12 */
 /*   Utilization (area) / IQI #26            */
 /* ----------------------------------------- */

 /* CABG */
 /* ICD-10-CM Procedure Codes: */

VALUE $PRCABGP
"0210093"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Artery with Autologous Venous Tissue, Open Approach */
"0210098"  =  "1"  /* Bypass Coronary Artery, One Site from Right Internal Mammary with Autologous Venous Tissue, Open Approach */
"0210099"  =  "1"  /* Bypass Coronary Artery, One Site from Left Internal Mammary with Autologous Venous Tissue, Open Approach */
"021009C"  =  "1"  /* Bypass Coronary Artery, One Site from Thoracic Artery with Autologous Venous Tissue, Open Approach */
"021009F"  =  "1"  /* Bypass Coronary Artery, One Site from Abdominal Artery with Autologous Venous Tissue, Open Approach */
"021009W"  =  "1"  /* Bypass Coronary Artery, One Site from Aorta with Autologous Venous Tissue, Open Approach */
"02100A3"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Artery with Autologous Arterial Tissue, Open Approach */
"02100A8"  =  "1"  /* Bypass Coronary Artery, One Site from Right Internal Mammary with Autologous Arterial Tissue, Open Approach */
"02100A9"  =  "1"  /* Bypass Coronary Artery, One Site from Left Internal Mammary with Autologous Arterial Tissue, Open Approach */
"02100AC"  =  "1"  /* Bypass Coronary Artery, One Site from Thoracic Artery with Autologous Arterial Tissue, Open Approach */
"02100AF"  =  "1"  /* Bypass Coronary Artery, One Site from Abdominal Artery with Autologous Arterial Tissue, Open Approach */
"02100AW"  =  "1"  /* Bypass Coronary Artery, One Site from Aorta with Autologous Arterial Tissue, Open Approach */
"02100J3"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Artery with Synthetic Substitute, Open Approach */
"02100J8"  =  "1"  /* Bypass Coronary Artery, One Site from Right Internal Mammary with Synthetic Substitute, Open Approach */
"02100J9"  =  "1"  /* Bypass Coronary Artery, One Site from Left Internal Mammary with Synthetic Substitute, Open Approach */
"02100JC"  =  "1"  /* Bypass Coronary Artery, One Site from Thoracic Artery with Synthetic Substitute, Open Approach */
"02100JF"  =  "1"  /* Bypass Coronary Artery, One Site from Abdominal Artery with Synthetic Substitute, Open Approach */
"02100JW"  =  "1"  /* Bypass Coronary Artery, One Site from Aorta with Synthetic Substitute, Open Approach */
"02100K3"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Artery with Nonautologous Tissue Substitute, Open Approach */
"02100K8"  =  "1"  /* Bypass Coronary Artery, One Site from Right Internal Mammary with Nonautologous Tissue Substitute, Open Approach */
"02100K9"  =  "1"  /* Bypass Coronary Artery, One Site from Left Internal Mammary with Nonautologous Tissue Substitute, Open Approach */
"02100KC"  =  "1"  /* Bypass Coronary Artery, One Site from Thoracic Artery with Nonautologous Tissue Substitute, Open Approach */
"02100KF"  =  "1"  /* Bypass Coronary Artery, One Site from Abdominal Artery with Nonautologous Tissue Substitute, Open Approach */
"02100KW"  =  "1"  /* Bypass Coronary Artery, One Site from Aorta with Nonautologous Tissue Substitute, Open Approach */
"02100Z3"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Artery, Open Approach */
"02100Z8"  =  "1"  /* Bypass Coronary Artery, One Site from Right Internal Mammary, Open Approach */
"02100Z9"  =  "1"  /* Bypass Coronary Artery, One Site from Left Internal Mammary, Open Approach */
"02100ZC"  =  "1"  /* Bypass Coronary Artery, One Site from Thoracic Artery, Open Approach */
"02100ZF"  =  "1"  /* Bypass Coronary Artery, One Site from Abdominal Artery, Open Approach */
"0211093"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Artery with Autologous Venous Tissue, Open Approach */
"0211098"  =  "1"  /* Bypass Coronary Artery, Two Sites from Right Internal Mammary with Autologous Venous Tissue, Open Approach */
"0211099"  =  "1"  /* Bypass Coronary Artery, Two Sites from Left Internal Mammary with Autologous Venous Tissue, Open Approach */
"021109C"  =  "1"  /* Bypass Coronary Artery, Two Sites from Thoracic Artery with Autologous Venous Tissue, Open Approach */
"021109F"  =  "1"  /* Bypass Coronary Artery, Two Sites from Abdominal Artery with Autologous Venous Tissue, Open Approach */
"021109W"  =  "1"  /* Bypass Coronary Artery, Two Sites from Aorta with Autologous Venous Tissue, Open Approach */
"02110A3"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Artery with Autologous Arterial Tissue, Open Approach */
"02110A8"  =  "1"  /* Bypass Coronary Artery, Two Sites from Right Internal Mammary with Autologous Arterial Tissue, Open Approach */
"02110A9"  =  "1"  /* Bypass Coronary Artery, Two Sites from Left Internal Mammary with Autologous Arterial Tissue, Open Approach */
"02110AC"  =  "1"  /* Bypass Coronary Artery, Two Sites from Thoracic Artery with Autologous Arterial Tissue, Open Approach */
"02110AF"  =  "1"  /* Bypass Coronary Artery, Two Sites from Abdominal Artery with Autologous Arterial Tissue, Open Approach */
"02110AW"  =  "1"  /* Bypass Coronary Artery, Two Sites from Aorta with Autologous Arterial Tissue, Open Approach */
"02110J3"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Artery with Synthetic Substitute, Open Approach */
"02110J8"  =  "1"  /* Bypass Coronary Artery, Two Sites from Right Internal Mammary with Synthetic Substitute, Open Approach */
"02110J9"  =  "1"  /* Bypass Coronary Artery, Two Sites from Left Internal Mammary with Synthetic Substitute, Open Approach */
"02110JC"  =  "1"  /* Bypass Coronary Artery, Two Sites from Thoracic Artery with Synthetic Substitute, Open Approach */
"02110JF"  =  "1"  /* Bypass Coronary Artery, Two Sites from Abdominal Artery with Synthetic Substitute, Open Approach */
"02110JW"  =  "1"  /* Bypass Coronary Artery, Two Sites from Aorta with Synthetic Substitute, Open Approach */
"02110K3"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Artery with Nonautologous Tissue Substitute, Open Approach */
"02110K8"  =  "1"  /* Bypass Coronary Artery, Two Sites from Right Internal Mammary with Nonautologous Tissue Substitute, Open Approach */
"02110K9"  =  "1"  /* Bypass Coronary Artery, Two Sites from Left Internal Mammary with Nonautologous Tissue Substitute, Open Approach */
"02110KC"  =  "1"  /* Bypass Coronary Artery, Two Sites from Thoracic Artery with Nonautologous Tissue Substitute, Open Approach */
"02110KF"  =  "1"  /* Bypass Coronary Artery, Two Sites from Abdominal Artery with Nonautologous Tissue Substitute, Open Approach */
"02110KW"  =  "1"  /* Bypass Coronary Artery, Two Sites from Aorta with Nonautologous Tissue Substitute, Open Approach */
"02110Z3"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Artery, Open Approach */
"02110Z8"  =  "1"  /* Bypass Coronary Artery, Two Sites from Right Internal Mammary, Open Approach */
"02110Z9"  =  "1"  /* Bypass Coronary Artery, Two Sites from Left Internal Mammary, Open Approach */
"02110ZC"  =  "1"  /* Bypass Coronary Artery, Two Sites from Thoracic Artery, Open Approach */
"02110ZF"  =  "1"  /* Bypass Coronary Artery, Two Sites from Abdominal Artery, Open Approach */
"0212093"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Artery with Autologous Venous Tissue, Open Approach */
"0212098"  =  "1"  /* Bypass Coronary Artery, Three Sites from Right Internal Mammary with Autologous Venous Tissue, Open Approach */
"0212099"  =  "1"  /* Bypass Coronary Artery, Three Sites from Left Internal Mammary with Autologous Venous Tissue, Open Approach */
"021209C"  =  "1"  /* Bypass Coronary Artery, Three Sites from Thoracic Artery with Autologous Venous Tissue, Open Approach */
"021209F"  =  "1"  /* Bypass Coronary Artery, Three Sites from Abdominal Artery with Autologous Venous Tissue, Open Approach */
"021209W"  =  "1"  /* Bypass Coronary Artery, Three Sites from Aorta with Autologous Venous Tissue, Open Approach */
"02120A3"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Artery with Autologous Arterial Tissue, Open Approach */
"02120A8"  =  "1"  /* Bypass Coronary Artery, Three Sites from Right Internal Mammary with Autologous Arterial Tissue, Open Approach */
"02120A9"  =  "1"  /* Bypass Coronary Artery, Three Sites from Left Internal Mammary with Autologous Arterial Tissue, Open Approach */
"02120AC"  =  "1"  /* Bypass Coronary Artery, Three Sites from Thoracic Artery with Autologous Arterial Tissue, Open Approach */
"02120AF"  =  "1"  /* Bypass Coronary Artery, Three Sites from Abdominal Artery with Autologous Arterial Tissue, Open Approach */
"02120AW"  =  "1"  /* Bypass Coronary Artery, Three Sites from Aorta with Autologous Arterial Tissue, Open Approach */
"02120J3"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Artery with Synthetic Substitute, Open Approach */
"02120J8"  =  "1"  /* Bypass Coronary Artery, Three Sites from Right Internal Mammary with Synthetic Substitute, Open Approach */
"02120J9"  =  "1"  /* Bypass Coronary Artery, Three Sites from Left Internal Mammary with Synthetic Substitute, Open Approach */
"02120JC"  =  "1"  /* Bypass Coronary Artery, Three Sites from Thoracic Artery with Synthetic Substitute, Open Approach */
"02120JF"  =  "1"  /* Bypass Coronary Artery, Three Sites from Abdominal Artery with Synthetic Substitute, Open Approach */
"02120JW"  =  "1"  /* Bypass Coronary Artery, Three Sites from Aorta with Synthetic Substitute, Open Approach */
"02120K3"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Artery with Nonautologous Tissue Substitute, Open Approach */
"02120K8"  =  "1"  /* Bypass Coronary Artery, Three Sites from Right Internal Mammary with Nonautologous Tissue Substitute, Open Approach */
"02120K9"  =  "1"  /* Bypass Coronary Artery, Three Sites from Left Internal Mammary with Nonautologous Tissue Substitute, Open Approach */
"02120KC"  =  "1"  /* Bypass Coronary Artery, Three Sites from Thoracic Artery with Nonautologous Tissue Substitute, Open Approach */
"02120KF"  =  "1"  /* Bypass Coronary Artery, Three Sites from Abdominal Artery with Nonautologous Tissue Substitute, Open Approach */
"02120KW"  =  "1"  /* Bypass Coronary Artery, Three Sites from Aorta with Nonautologous Tissue Substitute, Open Approach */
"02120Z3"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Artery, Open Approach */
"02120Z8"  =  "1"  /* Bypass Coronary Artery, Three Sites from Right Internal Mammary, Open Approach */
"02120Z9"  =  "1"  /* Bypass Coronary Artery, Three Sites from Left Internal Mammary, Open Approach */
"02120ZC"  =  "1"  /* Bypass Coronary Artery, Three Sites from Thoracic Artery, Open Approach */
"02120ZF"  =  "1"  /* Bypass Coronary Artery, Three Sites from Abdominal Artery, Open Approach */
"0213093"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Artery with Autologous Venous Tissue, Open Approach */
"0213098"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Right Internal Mammary with Autologous Venous Tissue, Open Approach */
"0213099"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Left Internal Mammary with Autologous Venous Tissue, Open Approach */
"021309C"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Thoracic Artery with Autologous Venous Tissue, Open Approach */
"021309F"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Abdominal Artery with Autologous Venous Tissue, Open Approach */
"021309W"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Aorta with Autologous Venous Tissue, Open Approach */
"02130A3"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Artery with Autologous Arterial Tissue, Open Approach */
"02130A8"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Right Internal Mammary with Autologous Arterial Tissue, Open Approach */
"02130A9"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Left Internal Mammary with Autologous Arterial Tissue, Open Approach */
"02130AC"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Thoracic Artery with Autologous Arterial Tissue, Open Approach */
"02130AF"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Abdominal Artery with Autologous Arterial Tissue, Open Approach */
"02130AW"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Aorta with Autologous Arterial Tissue, Open Approach */
"02130J3"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Artery with Synthetic Substitute, Open Approach */
"02130J8"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Right Internal Mammary with Synthetic Substitute, Open Approach */
"02130J9"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Left Internal Mammary with Synthetic Substitute, Open Approach */
"02130JC"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Thoracic Artery with Synthetic Substitute, Open Approach */
"02130JF"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Abdominal Artery with Synthetic Substitute, Open Approach */
"02130JW"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Aorta with Synthetic Substitute, Open Approach */
"02130K3"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Artery with Nonautologous Tissue Substitute, Open Approach */
"02130K8"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Right Internal Mammary with Nonautologous Tissue Substitute, Open Approach */
"02130K9"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Left Internal Mammary with Nonautologous Tissue Substitute, Open Approach */
"02130KC"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Thoracic Artery with Nonautologous Tissue Substitute, Open Approach */
"02130KF"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Abdominal Artery with Nonautologous Tissue Substitute, Open Approach */
"02130KW"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Aorta with Nonautologous Tissue Substitute, Open Approach */
"02130Z3"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Artery, Open Approach */
"02130Z8"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Right Internal Mammary, Open Approach */
"02130Z9"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Left Internal Mammary, Open Approach */
"02130ZC"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Thoracic Artery, Open Approach */
"02130ZF"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Abdominal Artery, Open Approach */
OTHER  = '0';

 /* ---------------------------------------- */
 /* PTCA volume (PRPTCA)                     */
 /*   Volume Indicator / IQI #6              */
 /*   Utilization (area) Indicator / IQI #27 */
 /*   Mortality Indicator / IQI #30          */
 /* ---------------------------------------- */

 /* PTCA */
 /* ICD-10-CM Procedure Codes: */

VALUE $PRPTCAP
"0270346"  =  "1"  /* Dilation of Coronary Artery, One Site, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Approach */
"027034Z"  =  "1"  /* Dilation of Coronary Artery, One Site with Drug-eluting Intraluminal Device, Percutaneous Approach */
"02703D6"  =  "1"  /* Dilation of Coronary Artery, One Site, Bifurcation, with Intraluminal Device, Percutaneous Approach */
"02703DZ"  =  "1"  /* Dilation of Coronary Artery, One Site with Intraluminal Device, Percutaneous Approach */
"02703Z6"  =  "1"  /* Dilation of Coronary Artery, One Site, Bifurcation, Percutaneous Approach */
"02703ZZ"  =  "1"  /* Dilation of Coronary Artery, One Site, Percutaneous Approach */
"0270446"  =  "1"  /* Dilation of Coronary Artery, One Site, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"027044Z"  =  "1"  /* Dilation of Coronary Artery, One Site with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"02704D6"  =  "1"  /* Dilation of Coronary Artery, One Site, Bifurcation, with Intraluminal Device, Percutaneous Endoscopic Approach */
"02704DZ"  =  "1"  /* Dilation of Coronary Artery, One Site with Intraluminal Device, Percutaneous Endoscopic Approach */
"02704Z6"  =  "1"  /* Dilation of Coronary Artery, One Site, Bifurcation, Percutaneous Endoscopic Approach */
"02704ZZ"  =  "1"  /* Dilation of Coronary Artery, One Site, Percutaneous Endoscopic Approach */
"0271346"  =  "1"  /* Dilation of Coronary Artery, Two Sites, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Approach */
"027134Z"  =  "1"  /* Dilation of Coronary Artery, Two Sites with Drug-eluting Intraluminal Device, Percutaneous Approach */
"02713D6"  =  "1"  /* Dilation of Coronary Artery, Two Sites, Bifurcation, with Intraluminal Device, Percutaneous Approach */
"02713DZ"  =  "1"  /* Dilation of Coronary Artery, Two Sites with Intraluminal Device, Percutaneous Approach */
"02713Z6"  =  "1"  /* Dilation of Coronary Artery, Two Sites, Bifurcation, Percutaneous Approach */
"02713ZZ"  =  "1"  /* Dilation of Coronary Artery, Two Sites, Percutaneous Approach */
"0271446"  =  "1"  /* Dilation of Coronary Artery, Two Sites, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"027144Z"  =  "1"  /* Dilation of Coronary Artery, Two Sites with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"02714D6"  =  "1"  /* Dilation of Coronary Artery, Two Sites, Bifurcation, with Intraluminal Device, Percutaneous Endoscopic Approach */
"02714DZ"  =  "1"  /* Dilation of Coronary Artery, Two Sites with Intraluminal Device, Percutaneous Endoscopic Approach */
"02714Z6"  =  "1"  /* Dilation of Coronary Artery, Two Sites, Bifurcation, Percutaneous Endoscopic Approach */
"02714ZZ"  =  "1"  /* Dilation of Coronary Artery, Two Sites, Percutaneous Endoscopic Approach */
"0272346"  =  "1"  /* Dilation of Coronary Artery, Three Sites, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Approach */
"027234Z"  =  "1"  /* Dilation of Coronary Artery, Three Sites with Drug-eluting Intraluminal Device, Percutaneous Approach */
"02723D6"  =  "1"  /* Dilation of Coronary Artery, Three Sites, Bifurcation, with Intraluminal Device, Percutaneous Approach */
"02723DZ"  =  "1"  /* Dilation of Coronary Artery, Three Sites with Intraluminal Device, Percutaneous Approach */
"02723Z6"  =  "1"  /* Dilation of Coronary Artery, Three Sites, Bifurcation, Percutaneous Approach */
"02723ZZ"  =  "1"  /* Dilation of Coronary Artery, Three Sites, Percutaneous Approach */
"0272446"  =  "1"  /* Dilation of Coronary Artery, Three Sites, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"027244Z"  =  "1"  /* Dilation of Coronary Artery, Three Sites with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"02724D6"  =  "1"  /* Dilation of Coronary Artery, Three Sites, Bifurcation, with Intraluminal Device, Percutaneous Endoscopic Approach */
"02724DZ"  =  "1"  /* Dilation of Coronary Artery, Three Sites with Intraluminal Device, Percutaneous Endoscopic Approach */
"02724Z6"  =  "1"  /* Dilation of Coronary Artery, Three Sites, Bifurcation, Percutaneous Endoscopic Approach */
"02724ZZ"  =  "1"  /* Dilation of Coronary Artery, Three Sites, Percutaneous Endoscopic Approach */
"0273346"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Approach */
"027334Z"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites with Drug-eluting Intraluminal Device, Percutaneous Approach */
"02733D6"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites, Bifurcation, with Intraluminal Device, Percutaneous Approach */
"02733DZ"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites with Intraluminal Device, Percutaneous Approach */
"02733Z6"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites, Bifurcation, Percutaneous Approach */
"02733ZZ"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites, Percutaneous Approach */
"0273446"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"027344Z"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"02734D6"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites, Bifurcation, with Intraluminal Device, Percutaneous Endoscopic Approach */
"02734DZ"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites with Intraluminal Device, Percutaneous Endoscopic Approach */
"02734Z6"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites, Bifurcation, Percutaneous Endoscopic Approach */
"02734ZZ"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites, Percutaneous Endoscopic Approach */
OTHER  = '0' ;


 /* -------------------------------------- */
 /* Carotid Endarterectomy volume (PRCEAT) */
 /*   Volume Indicator / IQI #7            */
 /*   Mortality Indicator / IQI #31        */
 /* ---------------------------------------*/

 /* Carotid Endarterectomy */
 /* ICD-10-CM Procedure Codes: */

VALUE $PRCEATP
"03CH0ZZ"  =  "1"  /* Extirpation of Matter from Right Common Carotid Artery, Open Approach */
"03CJ0ZZ"  =  "1"  /* Extirpation of Matter from Left Common Carotid Artery, Open Approach */
"03CK0ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Carotid Artery, Open Approach */
"03CL0ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Carotid Artery, Open Approach */
OTHER  = '0' ;


 * -------------------------------------------------------------- ;
 * --- FORMATS - PROVIDER LEVEL MORTALITY INDICATORS ------------ ;
 * -------------------------------------------------------------- ;

 /* ----------------------------------------- */
 /* Mortality after Hip Replacement (MTHIPR)  */
 /*   Mortality (post-op) Indicator / IQI #14 */
 /* ----------------------------------------- */

 /* Hip Replacement */
 /* ICD-10-CM Procedure Codes: */

VALUE $MTHIPRP
"0SR9019"  =  "1"  /* Replacement of Right Hip Joint with Metal Synthetic Substitute, Cemented, Open Approach */
"0SR901A"  =  "1"  /* Replacement of Right Hip Joint with Metal Synthetic Substitute, Uncemented, Open Approach */
"0SR901Z"  =  "1"  /* Replacement of Right Hip Joint with Metal Synthetic Substitute, Open Approach */
"0SR9029"  =  "1"  /* Replacement of Right Hip Joint with Metal on Polyethylene Synthetic Substitute, Cemented, Open Approach */
"0SR902A"  =  "1"  /* Replacement of Right Hip Joint with Metal on Polyethylene Synthetic Substitute, Uncemented, Open Approach */
"0SR902Z"  =  "1"  /* Replacement of Right Hip Joint with Metal on Polyethylene Synthetic Substitute, Open Approach */
"0SR9039"  =  "1"  /* Replacement of Right Hip Joint with Ceramic Synthetic Substitute, Cemented, Open Approach */
"0SR903A"  =  "1"  /* Replacement of Right Hip Joint with Ceramic Synthetic Substitute, Uncemented, Open Approach */
"0SR903Z"  =  "1"  /* Replacement of Right Hip Joint with Ceramic Synthetic Substitute, Open Approach */
"0SR9049"  =  "1"  /* Replacement of Right Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Cemented, Open Approach */
"0SR904A"  =  "1"  /* Replacement of Right Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Uncemented, Open Approach */
"0SR904Z"  =  "1"  /* Replacement of Right Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Open Approach */
"0SR907Z"  =  "1"  /* Replacement of Right Hip Joint with Autologous Tissue Substitute, Open Approach */
"0SR90J9"  =  "1"  /* Replacement of Right Hip Joint with Synthetic Substitute, Cemented, Open Approach */
"0SR90JA"  =  "1"  /* Replacement of Right Hip Joint with Synthetic Substitute, Uncemented, Open Approach */
"0SR90JZ"  =  "1"  /* Replacement of Right Hip Joint with Synthetic Substitute, Open Approach */
"0SR90KZ"  =  "1"  /* Replacement of Right Hip Joint with Nonautologous Tissue Substitute, Open Approach */
"0SRA009"  =  "1"  /* Replacement of Right Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Cemented, Open Approach */
"0SRA00A"  =  "1"  /* Replacement of Right Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Uncemented, Open Approach */
"0SRA00Z"  =  "1"  /* Replacement of Right Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Open Approach */
"0SRA019"  =  "1"  /* Replacement of Right Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Cemented, Open Approach */
"0SRA01A"  =  "1"  /* Replacement of Right Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Uncemented, Open Approach */
"0SRA01Z"  =  "1"  /* Replacement of Right Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Open Approach */
"0SRA039"  =  "1"  /* Replacement of Right Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Cemented, Open Approach */
"0SRA03A"  =  "1"  /* Replacement of Right Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Uncemented, Open Approach */
"0SRA03Z"  =  "1"  /* Replacement of Right Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Open Approach */
"0SRA07Z"  =  "1"  /* Replacement of Right Hip Joint, Acetabular Surface with Autologous Tissue Substitute, Open Approach */
"0SRA0J9"  =  "1"  /* Replacement of Right Hip Joint, Acetabular Surface with Synthetic Substitute, Cemented, Open Approach */
"0SRA0JA"  =  "1"  /* Replacement of Right Hip Joint, Acetabular Surface with Synthetic Substitute, Uncemented, Open Approach */
"0SRA0JZ"  =  "1"  /* Replacement of Right Hip Joint, Acetabular Surface with Synthetic Substitute, Open Approach */
"0SRA0KZ"  =  "1"  /* Replacement of Right Hip Joint, Acetabular Surface with Nonautologous Tissue Substitute, Open Approach */
"0SRB019"  =  "1"  /* Replacement of Left Hip Joint with Metal Synthetic Substitute, Cemented, Open Approach */
"0SRB01A"  =  "1"  /* Replacement of Left Hip Joint with Metal Synthetic Substitute, Uncemented, Open Approach */
"0SRB01Z"  =  "1"  /* Replacement of Left Hip Joint with Metal Synthetic Substitute, Open Approach */
"0SRB029"  =  "1"  /* Replacement of Left Hip Joint with Metal on Polyethylene Synthetic Substitute, Cemented, Open Approach */
"0SRB02A"  =  "1"  /* Replacement of Left Hip Joint with Metal on Polyethylene Synthetic Substitute, Uncemented, Open Approach */
"0SRB02Z"  =  "1"  /* Replacement of Left Hip Joint with Metal on Polyethylene Synthetic Substitute, Open Approach */
"0SRB039"  =  "1"  /* Replacement of Left Hip Joint with Ceramic Synthetic Substitute, Cemented, Open Approach */
"0SRB03A"  =  "1"  /* Replacement of Left Hip Joint with Ceramic Synthetic Substitute, Uncemented, Open Approach */
"0SRB03Z"  =  "1"  /* Replacement of Left Hip Joint with Ceramic Synthetic Substitute, Open Approach */
"0SRB049"  =  "1"  /* Replacement of Left Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Cemented, Open Approach */
"0SRB04A"  =  "1"  /* Replacement of Left Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Uncemented, Open Approach */
"0SRB04Z"  =  "1"  /* Replacement of Left Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Open Approach */
"0SRB07Z"  =  "1"  /* Replacement of Left Hip Joint with Autologous Tissue Substitute, Open Approach */
"0SRB0J9"  =  "1"  /* Replacement of Left Hip Joint with Synthetic Substitute, Cemented, Open Approach */
"0SRB0JA"  =  "1"  /* Replacement of Left Hip Joint with Synthetic Substitute, Uncemented, Open Approach */
"0SRB0JZ"  =  "1"  /* Replacement of Left Hip Joint with Synthetic Substitute, Open Approach */
"0SRB0KZ"  =  "1"  /* Replacement of Left Hip Joint with Nonautologous Tissue Substitute, Open Approach */
"0SRE009"  =  "1"  /* Replacement of Left Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Cemented, Open Approach */
"0SRE00A"  =  "1"  /* Replacement of Left Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Uncemented, Open Approach */
"0SRE00Z"  =  "1"  /* Replacement of Left Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Open Approach */
"0SRE019"  =  "1"  /* Replacement of Left Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Cemented, Open Approach */
"0SRE01A"  =  "1"  /* Replacement of Left Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Uncemented, Open Approach */
"0SRE01Z"  =  "1"  /* Replacement of Left Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Open Approach */
"0SRE039"  =  "1"  /* Replacement of Left Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Cemented, Open Approach */
"0SRE03A"  =  "1"  /* Replacement of Left Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Uncemented, Open Approach */
"0SRE03Z"  =  "1"  /* Replacement of Left Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Open Approach */
"0SRE07Z"  =  "1"  /* Replacement of Left Hip Joint, Acetabular Surface with Autologous Tissue Substitute, Open Approach */
"0SRE0J9"  =  "1"  /* Replacement of Left Hip Joint, Acetabular Surface with Synthetic Substitute, Cemented, Open Approach */
"0SRE0JA"  =  "1"  /* Replacement of Left Hip Joint, Acetabular Surface with Synthetic Substitute, Uncemented, Open Approach */
"0SRE0JZ"  =  "1"  /* Replacement of Left Hip Joint, Acetabular Surface with Synthetic Substitute, Open Approach */
"0SRE0KZ"  =  "1"  /* Replacement of Left Hip Joint, Acetabular Surface with Nonautologous Tissue Substitute, Open Approach */
"0SRR019"  =  "1"  /* Replacement of Right Hip Joint, Femoral Surface with Metal Synthetic Substitute, Cemented, Open Approach */
"0SRR01A"  =  "1"  /* Replacement of Right Hip Joint, Femoral Surface with Metal Synthetic Substitute, Uncemented, Open Approach */
"0SRR01Z"  =  "1"  /* Replacement of Right Hip Joint, Femoral Surface with Metal Synthetic Substitute, Open Approach */
"0SRR039"  =  "1"  /* Replacement of Right Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Cemented, Open Approach */
"0SRR03A"  =  "1"  /* Replacement of Right Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Uncemented, Open Approach */
"0SRR03Z"  =  "1"  /* Replacement of Right Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Open Approach */
"0SRR07Z"  =  "1"  /* Replacement of Right Hip Joint, Femoral Surface with Autologous Tissue Substitute, Open Approach */
"0SRR0J9"  =  "1"  /* Replacement of Right Hip Joint, Femoral Surface with Synthetic Substitute, Cemented, Open Approach */
"0SRR0JA"  =  "1"  /* Replacement of Right Hip Joint, Femoral Surface with Synthetic Substitute, Uncemented, Open Approach */
"0SRR0JZ"  =  "1"  /* Replacement of Right Hip Joint, Femoral Surface with Synthetic Substitute, Open Approach */
"0SRR0KZ"  =  "1"  /* Replacement of Right Hip Joint, Femoral Surface with Nonautologous Tissue Substitute, Open Approach */
"0SRS019"  =  "1"  /* Replacement of Left Hip Joint, Femoral Surface with Metal Synthetic Substitute, Cemented, Open Approach */
"0SRS01A"  =  "1"  /* Replacement of Left Hip Joint, Femoral Surface with Metal Synthetic Substitute, Uncemented, Open Approach */
"0SRS01Z"  =  "1"  /* Replacement of Left Hip Joint, Femoral Surface with Metal Synthetic Substitute, Open Approach */
"0SRS039"  =  "1"  /* Replacement of Left Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Cemented, Open Approach */
"0SRS03A"  =  "1"  /* Replacement of Left Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Uncemented, Open Approach */
"0SRS03Z"  =  "1"  /* Replacement of Left Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Open Approach */
"0SRS07Z"  =  "1"  /* Replacement of Left Hip Joint, Femoral Surface with Autologous Tissue Substitute, Open Approach */
"0SRS0J9"  =  "1"  /* Replacement of Left Hip Joint, Femoral Surface with Synthetic Substitute, Cemented, Open Approach */
"0SRS0JA"  =  "1"  /* Replacement of Left Hip Joint, Femoral Surface with Synthetic Substitute, Uncemented, Open Approach */
"0SRS0JZ"  =  "1"  /* Replacement of Left Hip Joint, Femoral Surface with Synthetic Substitute, Open Approach */
"0SRS0KZ"  =  "1"  /* Replacement of Left Hip Joint, Femoral Surface with Nonautologous Tissue Substitute, Open Approach */
"0SU909Z"  =  "1"  /* Supplement Right Hip Joint with Liner, Open Approach */
"0SUA09Z"  =  "1"  /* Supplement Right Hip Joint, Acetabular Surface with Liner, Open Approach */
"0SUB09Z"  =  "1"  /* Supplement Left Hip Joint with Liner, Open Approach */
"0SUE09Z"  =  "1"  /* Supplement Left Hip Joint, Acetabular Surface with Liner, Open Approach */
"0SUR09Z"  =  "1"  /* Supplement Right Hip Joint, Femoral Surface with Liner, Open Approach */
"0SUS09Z"  =  "1"  /* Supplement Left Hip Joint, Femoral Surface with Liner, Open Approach */
"0SW90JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Hip Joint, Open Approach */
"0SWB0JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Hip Joint, Open Approach */
OTHER  = '0' ;

 /* Include Only: Osteoarthrosis of Hip */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $MTHIPRD
"M129"  =  "1"  /* Arthropathy, unspecified */
"M130"  =  "1"  /* Polyarthritis, unspecified */
"M1310"  =  "1"  /* Monoarthritis, not elsewhere classified, unspecified site */
"M13151"  =  "1"  /* Monoarthritis, not elsewhere classified, right hip */
"M13152"  =  "1"  /* Monoarthritis, not elsewhere classified, left hip */
"M13159"  =  "1"  /* Monoarthritis, not elsewhere classified, unspecified hip */
"M150"  =  "1"  /* Primary generalized (osteo)arthritis */
"M153"  =  "1"  /* Secondary multiple arthritis */
"M154"  =  "1"  /* Erosive (osteo)arthritis */
"M158"  =  "1"  /* Other polyosteoarthritis */
"M159"  =  "1"  /* Polyosteoarthritis, unspecified */
"M160"  =  "1"  /* Bilateral primary osteoarthritis of hip */
"M1610"  =  "1"  /* Unilateral primary osteoarthritis, unspecified hip */
"M1611"  =  "1"  /* Unilateral primary osteoarthritis, right hip */
"M1612"  =  "1"  /* Unilateral primary osteoarthritis, left hip */
"M162"  =  "1"  /* Bilateral osteoarthritis resulting from hip dysplasia */
"M1630"  =  "1"  /* Unilateral osteoarthritis resulting from hip dysplasia, unspecified hip */
"M1631"  =  "1"  /* Unilateral osteoarthritis resulting from hip dysplasia, right hip */
"M1632"  =  "1"  /* Unilateral osteoarthritis resulting from hip dysplasia, left hip */
"M164"  =  "1"  /* Bilateral post-traumatic osteoarthritis of hip */
"M1650"  =  "1"  /* Unilateral post-traumatic osteoarthritis, unspecified hip */
"M1651"  =  "1"  /* Unilateral post-traumatic osteoarthritis, right hip */
"M1652"  =  "1"  /* Unilateral post-traumatic osteoarthritis, left hip */
"M166"  =  "1"  /* Other bilateral secondary osteoarthritis of hip */
"M167"  =  "1"  /* Other unilateral secondary osteoarthritis of hip */
"M169"  =  "1"  /* Osteoarthritis of hip, unspecified */
"M1990"  =  "1"  /* Unspecified osteoarthritis, unspecified site */
"M1991"  =  "1"  /* Primary osteoarthritis, unspecified site */
"M1992"  =  "1"  /* Post-traumatic osteoarthritis, unspecified site */
"M1993"  =  "1"  /* Secondary osteoarthritis, unspecified site */
OTHER   = '0' ;

 /* ----------------------------------------------- */
 /* Acute Myocardial Infarction Mortality (MRTAMI)  */
 /*   Mortality Indicator / IQI #15                 */
 /* ----------------------------------------------- */

 /* Acute Myocardial Infarction */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $MRTAMID
"I2101"  =  "1"  /* ST elevation (STEMI) myocardial infarction involving left main coronary artery */
"I2102"  =  "1"  /* ST elevation (STEMI) myocardial infarction involving left anterior descending coronary artery */
"I2109"  =  "1"  /* ST elevation (STEMI) myocardial infarction involving other coronary artery of anterior wall */
"I2111"  =  "1"  /* ST elevation (STEMI) myocardial infarction involving right coronary artery */
"I2119"  =  "1"  /* ST elevation (STEMI) myocardial infarction involving other coronary artery of inferior wall */
"I2121"  =  "1"  /* ST elevation (STEMI) myocardial infarction involving left circumflex coronary artery */
"I2129"  =  "1"  /* ST elevation (STEMI) myocardial infarction involving other sites */
"I213"  =  "1"  /* ST elevation (STEMI) myocardial infarction of unspecified site */
"I214"  =  "1"  /* Non-ST elevation (NSTEMI) myocardial infarction */
"I220"  =  "1"  /* Subsequent ST elevation (STEMI) myocardial infarction of anterior wall */
"I221"  =  "1"  /* Subsequent ST elevation (STEMI) myocardial infarction of inferior wall */
"I222"  =  "1"  /* Subsequent non-ST elevation (NSTEMI) myocardial infarction */
"I228"  =  "1"  /* Subsequent ST elevation (STEMI) myocardial infarction of other sites */
"I229"  =  "1"  /* Subsequent ST elevation (STEMI) myocardial infarction of unspecified site */
OTHER  = '0' ;


 /* -------------------------------------- */
 /* Congestive Heart Failure rate (MRTCHF) */
 /*   Mortality Indicator / IQI #16        */
 /* -------------------------------------- */

 /* CHF */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $MRTCHFD
"I0981"  =  "1"  /* Rheumatic heart failure */
"I110"  =  "1"  /* Hypertensive heart disease with heart failure */
"I130"  =  "1"  /* Hypertensive heart and chronic kidney disease with heart failure and stage 1 through stage 4 chronic kidney disease, or unspecified chronic kidney disease */
"I132"  =  "1"  /* Hypertensive heart and chronic kidney disease with heart failure and with stage 5 chronic kidney disease, or end stage renal disease */
"I501"  =  "1"  /* Left ventricular failure */
"I5020"  =  "1"  /* Unspecified systolic (congestive) heart failure */
"I5021"  =  "1"  /* Acute systolic (congestive) heart failure */
"I5022"  =  "1"  /* Chronic systolic (congestive) heart failure */
"I5023"  =  "1"  /* Acute on chronic systolic (congestive) heart failure */
"I5030"  =  "1"  /* Unspecified diastolic (congestive) heart failure */
"I5031"  =  "1"  /* Acute diastolic (congestive) heart failure */
"I5032"  =  "1"  /* Chronic diastolic (congestive) heart failure */
"I5033"  =  "1"  /* Acute on chronic diastolic (congestive) heart failure */
"I5040"  =  "1"  /* Unspecified combined systolic (congestive) and diastolic (congestive) heart failure */
"I5041"  =  "1"  /* Acute combined systolic (congestive) and diastolic (congestive) heart failure */
"I5042"  =  "1"  /* Chronic combined systolic (congestive) and diastolic (congestive) heart failure */
"I5043"  =  "1"  /* Acute on chronic combined systolic (congestive) and diastolic (congestive) heart failure */
"I509"  =  "1"  /* Heart failure, unspecified */
OTHER   = '0' ;

 /* ------------------------------- */
 /* Stroke Mortality (MRTCVD)       */
 /*   Mortality Indicator / IQI #17 */
 /* ------------------------------- */

 /* Stroke */
 /* ICD-10-CM Diagnosis Codes: */


/* SUBARACHNOID HEMORRHAGE */
VALUE $MRTCV2A
"I6000"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from unspecified carotid siphon and bifurcation */
"I6001"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from right carotid siphon and bifurcation */
"I6002"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from left carotid siphon and bifurcation */
"I6010"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from unspecified middle cerebral artery */
"I6011"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from right middle cerebral artery */
"I6012"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from left middle cerebral artery */
"I6020"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from unspecified anterior communicating artery */
"I6021"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from right anterior communicating artery */
"I6022"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from left anterior communicating artery */
"I6030"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from unspecified posterior communicating artery */
"I6031"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from right posterior communicating artery */
"I6032"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from left posterior communicating artery */
"I604"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from basilar artery */
"I6050"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from unspecified vertebral artery */
"I6051"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from right vertebral artery */
"I6052"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from left vertebral artery */
"I606"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from other intracranial arteries */
"I607"  =  "1"  /* Nontraumatic subarachnoid hemorrhage from unspecified intracranial artery */
"I608"  =  "1"  /* Other nontraumatic subarachnoid hemorrhage */
"I609"  =  "1"  /* Nontraumatic subarachnoid hemorrhage, unspecified */
OTHER   = '0' ;

/* INTRACEREBRAL HEMORRHAGIC STROKE */

VALUE $MRTCV3D
"I610"  =  "1"  /* Nontraumatic intracerebral hemorrhage in hemisphere, subcortical */
"I611"  =  "1"  /* Nontraumatic intracerebral hemorrhage in hemisphere, cortical */
"I612"  =  "1"  /* Nontraumatic intracerebral hemorrhage in hemisphere, unspecified */
"I613"  =  "1"  /* Nontraumatic intracerebral hemorrhage in brain stem */
"I614"  =  "1"  /* Nontraumatic intracerebral hemorrhage in cerebellum */
"I615"  =  "1"  /* Nontraumatic intracerebral hemorrhage, intraventricular */
"I616"  =  "1"  /* Nontraumatic intracerebral hemorrhage, multiple localized */
"I618"  =  "1"  /* Other nontraumatic intracerebral hemorrhage */
"I619"  =  "1"  /* Nontraumatic intracerebral hemorrhage, unspecified */
"I6200"  =  "1"  /* Nontraumatic subdural hemorrhage, unspecified */
"I6201"  =  "1"  /* Nontraumatic acute subdural hemorrhage */
"I6202"  =  "1"  /* Nontraumatic subacute subdural hemorrhage */
"I6203"  =  "1"  /* Nontraumatic chronic subdural hemorrhage */
"I621"  =  "1"  /* Nontraumatic extradural hemorrhage */
"I629"  =  "1"  /* Nontraumatic intracranial hemorrhage, unspecified */
OTHER   = '0' ;

/* ISCHEMIC HEMORRHAGIC STROKE */

VALUE $MRTCV4D
"I6300"  =  "1"  /* Cerebral infarction due to thrombosis of unspecified precerebral artery */
"I63011"  =  "1"  /* Cerebral infarction due to thrombosis of right vertebral artery */
"I63012"  =  "1"  /* Cerebral infarction due to thrombosis of left vertebral artery */
"I63019"  =  "1"  /* Cerebral infarction due to thrombosis of unspecified vertebral artery */
"I6302"  =  "1"  /* Cerebral infarction due to thrombosis of basilar artery */
"I63031"  =  "1"  /* Cerebral infarction due to thrombosis of right carotid artery */
"I63032"  =  "1"  /* Cerebral infarction due to thrombosis of left carotid artery */
"I63039"  =  "1"  /* Cerebral infarction due to thrombosis of unspecified carotid artery */
"I6309"  =  "1"  /* Cerebral infarction due to thrombosis of other precerebral artery */
"I6310"  =  "1"  /* Cerebral infarction due to embolism of unspecified precerebral artery */
"I63111"  =  "1"  /* Cerebral infarction due to embolism of right vertebral artery */
"I63112"  =  "1"  /* Cerebral infarction due to embolism of left vertebral artery */
"I63119"  =  "1"  /* Cerebral infarction due to embolism of unspecified vertebral artery */
"I6312"  =  "1"  /* Cerebral infarction due to embolism of basilar artery */
"I63131"  =  "1"  /* Cerebral infarction due to embolism of right carotid artery */
"I63132"  =  "1"  /* Cerebral infarction due to embolism of left carotid artery */
"I63139"  =  "1"  /* Cerebral infarction due to embolism of unspecified carotid artery */
"I6319"  =  "1"  /* Cerebral infarction due to embolism of other precerebral artery */
"I6320"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of unspecified precerebral arteries */
"I63211"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of right vertebral arteries */
"I63212"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of left vertebral arteries */
"I63219"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of unspecified vertebral arteries */
"I6322"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of basilar arteries */
"I63231"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of right carotid arteries */
"I63232"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of left carotid arteries */
"I63239"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of unspecified carotid arteries */
"I6329"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of other precerebral arteries */
"I6330"  =  "1"  /* Cerebral infarction due to thrombosis of unspecified cerebral artery */
"I63311"  =  "1"  /* Cerebral infarction due to thrombosis of right middle cerebral artery */
"I63312"  =  "1"  /* Cerebral infarction due to thrombosis of left middle cerebral artery */
"I63319"  =  "1"  /* Cerebral infarction due to thrombosis of unspecified middle cerebral artery */
"I63321"  =  "1"  /* Cerebral infarction due to thrombosis of right anterior cerebral artery */
"I63322"  =  "1"  /* Cerebral infarction due to thrombosis of left anterior cerebral artery */
"I63329"  =  "1"  /* Cerebral infarction due to thrombosis of unspecified anterior cerebral artery */
"I63331"  =  "1"  /* Cerebral infarction due to thrombosis of right posterior cerebral artery */
"I63332"  =  "1"  /* Cerebral infarction due to thrombosis of left posterior cerebral artery */
"I63339"  =  "1"  /* Cerebral infarction due to thrombosis of unspecified posterior cerebral artery */
"I63341"  =  "1"  /* Cerebral infarction due to thrombosis of right cerebellar artery */
"I63342"  =  "1"  /* Cerebral infarction due to thrombosis of left cerebellar artery */
"I63349"  =  "1"  /* Cerebral infarction due to thrombosis of unspecified cerebellar artery */
"I6339"  =  "1"  /* Cerebral infarction due to thrombosis of other cerebral artery */
"I6340"  =  "1"  /* Cerebral infarction due to embolism of unspecified cerebral artery */
"I63411"  =  "1"  /* Cerebral infarction due to embolism of right middle cerebral artery */
"I63412"  =  "1"  /* Cerebral infarction due to embolism of left middle cerebral artery */
"I63419"  =  "1"  /* Cerebral infarction due to embolism of unspecified middle cerebral artery */
"I63421"  =  "1"  /* Cerebral infarction due to embolism of right anterior cerebral artery */
"I63422"  =  "1"  /* Cerebral infarction due to embolism of left anterior cerebral artery */
"I63429"  =  "1"  /* Cerebral infarction due to embolism of unspecified anterior cerebral artery */
"I63431"  =  "1"  /* Cerebral infarction due to embolism of right posterior cerebral artery */
"I63432"  =  "1"  /* Cerebral infarction due to embolism of left posterior cerebral artery */
"I63439"  =  "1"  /* Cerebral infarction due to embolism of unspecified posterior cerebral artery */
"I63441"  =  "1"  /* Cerebral infarction due to embolism of right cerebellar artery */
"I63442"  =  "1"  /* Cerebral infarction due to embolism of left cerebellar artery */
"I63449"  =  "1"  /* Cerebral infarction due to embolism of unspecified cerebellar artery */
"I6349"  =  "1"  /* Cerebral infarction due to embolism of other cerebral artery */
"I6350"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of unspecified cerebral artery */
"I63511"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of right middle cerebral artery */
"I63512"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of left middle cerebral artery */
"I63519"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of unspecified middle cerebral artery */
"I63521"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of right anterior cerebral artery */
"I63522"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of left anterior cerebral artery */
"I63529"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of unspecified anterior cerebral artery */
"I63531"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of right posterior cerebral artery */
"I63532"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of left posterior cerebral artery */
"I63539"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of unspecified posterior cerebral artery */
"I63541"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of right cerebellar artery */
"I63542"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of left cerebellar artery */
"I63549"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of unspecified cerebellar artery */
"I6359"  =  "1"  /* Cerebral infarction due to unspecified occlusion or stenosis of other cerebral artery */
"I636"  =  "1"  /* Cerebral infarction due to cerebral venous thrombosis, nonpyogenic */
"I638"  =  "1"  /* Other cerebral infarction */
"I639"  =  "1"  /* Cerebral infarction, unspecified */
OTHER   = '0' ;

 /* ---------------------------------------------- */
 /* Gastrointestinal Hemorrhage mortality (MRTGIH) */
 /*   Mortality Indicator / IQI #18                */
 /* ---------------------------------------------- */

 /* Gastrointestinal Hemorrhage */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $MRTGIHD
"I8501"  =  "1"  /* Esophageal varices with bleeding */
"K2211"  =  "1"  /* Ulcer of esophagus with bleeding */
"K226"  =  "1"  /* Gastro-esophageal laceration-hemorrhage syndrome */
"K250"  =  "1"  /* Acute gastric ulcer with hemorrhage */
"K252"  =  "1"  /* Acute gastric ulcer with both hemorrhage and perforation */
"K254"  =  "1"  /* Chronic or unspecified gastric ulcer with hemorrhage */
"K256"  =  "1"  /* Chronic or unspecified gastric ulcer with both hemorrhage and perforation */
"K260"  =  "1"  /* Acute duodenal ulcer with hemorrhage */
"K262"  =  "1"  /* Acute duodenal ulcer with both hemorrhage and perforation */
"K264"  =  "1"  /* Chronic or unspecified duodenal ulcer with hemorrhage */
"K266"  =  "1"  /* Chronic or unspecified duodenal ulcer with both hemorrhage and perforation */
"K270"  =  "1"  /* Acute peptic ulcer, site unspecified, with hemorrhage */
"K272"  =  "1"  /* Acute peptic ulcer, site unspecified, with both hemorrhage and perforation */
"K274"  =  "1"  /* Chronic or unspecified peptic ulcer, site unspecified, with hemorrhage */
"K276"  =  "1"  /* Chronic or unspecified peptic ulcer, site unspecified, with both hemorrhage and perforation */
"K280"  =  "1"  /* Acute gastrojejunal ulcer with hemorrhage */
"K282"  =  "1"  /* Acute gastrojejunal ulcer with both hemorrhage and perforation */
"K284"  =  "1"  /* Chronic or unspecified gastrojejunal ulcer with hemorrhage */
"K286"  =  "1"  /* Chronic or unspecified gastrojejunal ulcer with both hemorrhage and perforation */
"K2901"  =  "1"  /* Acute gastritis with bleeding */
"K2921"  =  "1"  /* Alcoholic gastritis with bleeding */
"K2931"  =  "1"  /* Chronic superficial gastritis with bleeding */
"K2941"  =  "1"  /* Chronic atrophic gastritis with bleeding */
"K2951"  =  "1"  /* Unspecified chronic gastritis with bleeding */
"K2961"  =  "1"  /* Other gastritis with bleeding */
"K2971"  =  "1"  /* Gastritis, unspecified, with bleeding */
"K2981"  =  "1"  /* Duodenitis with bleeding */
"K2991"  =  "1"  /* Gastroduodenitis, unspecified, with bleeding */
"K31811"  =  "1"  /* Angiodysplasia of stomach and duodenum with bleeding */
"K3182"  =  "1"  /* Dieulafoy lesion (hemorrhagic) of stomach and duodenum */
"K50011"  =  "1"  /* Crohn's disease of small intestine with rectal bleeding */
"K50111"  =  "1"  /* Crohn's disease of large intestine with rectal bleeding */
"K50811"  =  "1"  /* Crohn's disease of both small and large intestine with rectal bleeding */
"K50911"  =  "1"  /* Crohn's disease, unspecified, with rectal bleeding */
"K51011"  =  "1"  /* Ulcerative (chronic) pancolitis with rectal bleeding */
"K51211"  =  "1"  /* Ulcerative (chronic) proctitis with rectal bleeding */
"K51311"  =  "1"  /* Ulcerative (chronic) rectosigmoiditis with rectal bleeding */
"K51411"  =  "1"  /* Inflammatory polyps of colon with rectal bleeding */
"K51511"  =  "1"  /* Left sided colitis with rectal bleeding */
"K51811"  =  "1"  /* Other ulcerative colitis with rectal bleeding */
"K51911"  =  "1"  /* Ulcerative colitis, unspecified with rectal bleeding */
"K5521"  =  "1"  /* Angiodysplasia of colon with hemorrhage */
"K5701"  =  "1"  /* Diverticulitis of small intestine with perforation and abscess with bleeding */
"K5711"  =  "1"  /* Diverticulosis of small intestine without perforation or abscess with bleeding */
"K5713"  =  "1"  /* Diverticulitis of small intestine without perforation or abscess with bleeding */
"K5721"  =  "1"  /* Diverticulitis of large intestine with perforation and abscess with bleeding */
"K5731"  =  "1"  /* Diverticulosis of large intestine without perforation or abscess with bleeding */
"K5733"  =  "1"  /* Diverticulitis of large intestine without perforation or abscess with bleeding */
"K5741"  =  "1"  /* Diverticulitis of both small and large intestine with perforation and abscess with bleeding */
"K5751"  =  "1"  /* Diverticulosis of both small and large intestine without perforation or abscess with bleeding */
"K5753"  =  "1"  /* Diverticulitis of both small and large intestine without perforation or abscess with bleeding */
"K5781"  =  "1"  /* Diverticulitis of intestine, part unspecified, with perforation and abscess with bleeding */
"K5791"  =  "1"  /* Diverticulosis of intestine, part unspecified, without perforation or abscess with bleeding */
"K5793"  =  "1"  /* Diverticulitis of intestine, part unspecified, without perforation or abscess with bleeding */
"K625"  =  "1"  /* Hemorrhage of anus and rectum */
"K6381"  =  "1"  /* Dieulafoy lesion of intestine */
"K920"  =  "1"  /* Hematemesis */
"K921"  =  "1"  /* Melena */
"K922"  =  "1"  /* Gastrointestinal hemorrhage, unspecified */
OTHER   = '0' ;

 /* ------------------------------- */
 /* Hip Fracture Mortality (MTHIPF) */
 /*   Mortality Indicator / IQI #19 */
 /* ------------------------------- */

 /* Hip Fracture */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $MTHIPFD
"M80051"  =  "1"  /* Age-related osteoporosis with current pathological fracture, right femur */
"M80052"  =  "1"  /* Age-related osteoporosis with current pathological fracture, left femur */
"M80059"  =  "1"  /* Age-related osteoporosis with current pathological fracture, unspecified femur */
"M80851"  =  "1"  /* Other osteoporosis with current pathological fracture, right femur */
"M80852"  =  "1"  /* Other osteoporosis with current pathological fracture, left femur */
"M80859"  =  "1"  /* Other osteoporosis with current pathological fracture, unspecified femur */
"S72001A"  =  "1"  /* Fracture of unspecified part of neck of right femur, initial encounter for closed fracture */
"S72001B"  =  "1"  /* Fracture of unspecified part of neck of right femur, initial encounter for open fracture type I or II */
"S72001C"  =  "1"  /* Fracture of unspecified part of neck of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72002A"  =  "1"  /* Fracture of unspecified part of neck of left femur, initial encounter for closed fracture */
"S72002B"  =  "1"  /* Fracture of unspecified part of neck of left femur, initial encounter for open fracture type I or II */
"S72002C"  =  "1"  /* Fracture of unspecified part of neck of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72009A"  =  "1"  /* Fracture of unspecified part of neck of unspecified femur, initial encounter for closed fracture */
"S72009B"  =  "1"  /* Fracture of unspecified part of neck of unspecified femur, initial encounter for open fracture type I or II */
"S72009C"  =  "1"  /* Fracture of unspecified part of neck of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72011A"  =  "1"  /* Unspecified intracapsular fracture of right femur, initial encounter for closed fracture */
"S72011B"  =  "1"  /* Unspecified intracapsular fracture of right femur, initial encounter for open fracture type I or II */
"S72011C"  =  "1"  /* Unspecified intracapsular fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72012A"  =  "1"  /* Unspecified intracapsular fracture of left femur, initial encounter for closed fracture */
"S72012B"  =  "1"  /* Unspecified intracapsular fracture of left femur, initial encounter for open fracture type I or II */
"S72012C"  =  "1"  /* Unspecified intracapsular fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72019A"  =  "1"  /* Unspecified intracapsular fracture of unspecified femur, initial encounter for closed fracture */
"S72019B"  =  "1"  /* Unspecified intracapsular fracture of unspecified femur, initial encounter for open fracture type I or II */
"S72019C"  =  "1"  /* Unspecified intracapsular fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72031A"  =  "1"  /* Displaced midcervical fracture of right femur, initial encounter for closed fracture */
"S72031B"  =  "1"  /* Displaced midcervical fracture of right femur, initial encounter for open fracture type I or II */
"S72031C"  =  "1"  /* Displaced midcervical fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72032A"  =  "1"  /* Displaced midcervical fracture of left femur, initial encounter for closed fracture */
"S72032B"  =  "1"  /* Displaced midcervical fracture of left femur, initial encounter for open fracture type I or II */
"S72032C"  =  "1"  /* Displaced midcervical fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72033A"  =  "1"  /* Displaced midcervical fracture of unspecified femur, initial encounter for closed fracture */
"S72033B"  =  "1"  /* Displaced midcervical fracture of unspecified femur, initial encounter for open fracture type I or II */
"S72033C"  =  "1"  /* Displaced midcervical fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72034A"  =  "1"  /* Nondisplaced midcervical fracture of right femur, initial encounter for closed fracture */
"S72034B"  =  "1"  /* Nondisplaced midcervical fracture of right femur, initial encounter for open fracture type I or II */
"S72034C"  =  "1"  /* Nondisplaced midcervical fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72035A"  =  "1"  /* Nondisplaced midcervical fracture of left femur, initial encounter for closed fracture */
"S72035B"  =  "1"  /* Nondisplaced midcervical fracture of left femur, initial encounter for open fracture type I or II */
"S72035C"  =  "1"  /* Nondisplaced midcervical fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72036A"  =  "1"  /* Nondisplaced midcervical fracture of unspecified femur, initial encounter for closed fracture */
"S72036B"  =  "1"  /* Nondisplaced midcervical fracture of unspecified femur, initial encounter for open fracture type I or II */
"S72036C"  =  "1"  /* Nondisplaced midcervical fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72041A"  =  "1"  /* Displaced fracture of base of neck of right femur, initial encounter for closed fracture */
"S72041B"  =  "1"  /* Displaced fracture of base of neck of right femur, initial encounter for open fracture type I or II */
"S72041C"  =  "1"  /* Displaced fracture of base of neck of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72042A"  =  "1"  /* Displaced fracture of base of neck of left femur, initial encounter for closed fracture */
"S72042B"  =  "1"  /* Displaced fracture of base of neck of left femur, initial encounter for open fracture type I or II */
"S72042C"  =  "1"  /* Displaced fracture of base of neck of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72043A"  =  "1"  /* Displaced fracture of base of neck of unspecified femur, initial encounter for closed fracture */
"S72043B"  =  "1"  /* Displaced fracture of base of neck of unspecified femur, initial encounter for open fracture type I or II */
"S72043C"  =  "1"  /* Displaced fracture of base of neck of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72044A"  =  "1"  /* Nondisplaced fracture of base of neck of right femur, initial encounter for closed fracture */
"S72044B"  =  "1"  /* Nondisplaced fracture of base of neck of right femur, initial encounter for open fracture type I or II */
"S72044C"  =  "1"  /* Nondisplaced fracture of base of neck of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72045A"  =  "1"  /* Nondisplaced fracture of base of neck of left femur, initial encounter for closed fracture */
"S72045B"  =  "1"  /* Nondisplaced fracture of base of neck of left femur, initial encounter for open fracture type I or II */
"S72045C"  =  "1"  /* Nondisplaced fracture of base of neck of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72046A"  =  "1"  /* Nondisplaced fracture of base of neck of unspecified femur, initial encounter for closed fracture */
"S72046B"  =  "1"  /* Nondisplaced fracture of base of neck of unspecified femur, initial encounter for open fracture type I or II */
"S72046C"  =  "1"  /* Nondisplaced fracture of base of neck of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72051A"  =  "1"  /* Unspecified fracture of head of right femur, initial encounter for closed fracture */
"S72051B"  =  "1"  /* Unspecified fracture of head of right femur, initial encounter for open fracture type I or II */
"S72051C"  =  "1"  /* Unspecified fracture of head of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72052A"  =  "1"  /* Unspecified fracture of head of left femur, initial encounter for closed fracture */
"S72052B"  =  "1"  /* Unspecified fracture of head of left femur, initial encounter for open fracture type I or II */
"S72052C"  =  "1"  /* Unspecified fracture of head of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72059A"  =  "1"  /* Unspecified fracture of head of unspecified femur, initial encounter for closed fracture */
"S72059B"  =  "1"  /* Unspecified fracture of head of unspecified femur, initial encounter for open fracture type I or II */
"S72059C"  =  "1"  /* Unspecified fracture of head of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72061A"  =  "1"  /* Displaced articular fracture of head of right femur, initial encounter for closed fracture */
"S72061B"  =  "1"  /* Displaced articular fracture of head of right femur, initial encounter for open fracture type I or II */
"S72061C"  =  "1"  /* Displaced articular fracture of head of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72062A"  =  "1"  /* Displaced articular fracture of head of left femur, initial encounter for closed fracture */
"S72062B"  =  "1"  /* Displaced articular fracture of head of left femur, initial encounter for open fracture type I or II */
"S72062C"  =  "1"  /* Displaced articular fracture of head of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72063A"  =  "1"  /* Displaced articular fracture of head of unspecified femur, initial encounter for closed fracture */
"S72063B"  =  "1"  /* Displaced articular fracture of head of unspecified femur, initial encounter for open fracture type I or II */
"S72063C"  =  "1"  /* Displaced articular fracture of head of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72064A"  =  "1"  /* Nondisplaced articular fracture of head of right femur, initial encounter for closed fracture */
"S72064B"  =  "1"  /* Nondisplaced articular fracture of head of right femur, initial encounter for open fracture type I or II */
"S72064C"  =  "1"  /* Nondisplaced articular fracture of head of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72065A"  =  "1"  /* Nondisplaced articular fracture of head of left femur, initial encounter for closed fracture */
"S72065B"  =  "1"  /* Nondisplaced articular fracture of head of left femur, initial encounter for open fracture type I or II */
"S72065C"  =  "1"  /* Nondisplaced articular fracture of head of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72066A"  =  "1"  /* Nondisplaced articular fracture of head of unspecified femur, initial encounter for closed fracture */
"S72066B"  =  "1"  /* Nondisplaced articular fracture of head of unspecified femur, initial encounter for open fracture type I or II */
"S72066C"  =  "1"  /* Nondisplaced articular fracture of head of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72091A"  =  "1"  /* Other fracture of head and neck of right femur, initial encounter for closed fracture */
"S72091B"  =  "1"  /* Other fracture of head and neck of right femur, initial encounter for open fracture type I or II */
"S72091C"  =  "1"  /* Other fracture of head and neck of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72092A"  =  "1"  /* Other fracture of head and neck of left femur, initial encounter for closed fracture */
"S72092B"  =  "1"  /* Other fracture of head and neck of left femur, initial encounter for open fracture type I or II */
"S72092C"  =  "1"  /* Other fracture of head and neck of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72099A"  =  "1"  /* Other fracture of head and neck of unspecified femur, initial encounter for closed fracture */
"S72099B"  =  "1"  /* Other fracture of head and neck of unspecified femur, initial encounter for open fracture type I or II */
"S72099C"  =  "1"  /* Other fracture of head and neck of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72101A"  =  "1"  /* Unspecified trochanteric fracture of right femur, initial encounter for closed fracture */
"S72101B"  =  "1"  /* Unspecified trochanteric fracture of right femur, initial encounter for open fracture type I or II */
"S72101C"  =  "1"  /* Unspecified trochanteric fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72102A"  =  "1"  /* Unspecified trochanteric fracture of left femur, initial encounter for closed fracture */
"S72102B"  =  "1"  /* Unspecified trochanteric fracture of left femur, initial encounter for open fracture type I or II */
"S72102C"  =  "1"  /* Unspecified trochanteric fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72109A"  =  "1"  /* Unspecified trochanteric fracture of unspecified femur, initial encounter for closed fracture */
"S72109B"  =  "1"  /* Unspecified trochanteric fracture of unspecified femur, initial encounter for open fracture type I or II */
"S72109C"  =  "1"  /* Unspecified trochanteric fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72111A"  =  "1"  /* Displaced fracture of greater trochanter of right femur, initial encounter for closed fracture */
"S72111B"  =  "1"  /* Displaced fracture of greater trochanter of right femur, initial encounter for open fracture type I or II */
"S72111C"  =  "1"  /* Displaced fracture of greater trochanter of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72112A"  =  "1"  /* Displaced fracture of greater trochanter of left femur, initial encounter for closed fracture */
"S72112B"  =  "1"  /* Displaced fracture of greater trochanter of left femur, initial encounter for open fracture type I or II */
"S72112C"  =  "1"  /* Displaced fracture of greater trochanter of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72113A"  =  "1"  /* Displaced fracture of greater trochanter of unspecified femur, initial encounter for closed fracture */
"S72113B"  =  "1"  /* Displaced fracture of greater trochanter of unspecified femur, initial encounter for open fracture type I or II */
"S72113C"  =  "1"  /* Displaced fracture of greater trochanter of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72114A"  =  "1"  /* Nondisplaced fracture of greater trochanter of right femur, initial encounter for closed fracture */
"S72114B"  =  "1"  /* Nondisplaced fracture of greater trochanter of right femur, initial encounter for open fracture type I or II */
"S72114C"  =  "1"  /* Nondisplaced fracture of greater trochanter of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72115A"  =  "1"  /* Nondisplaced fracture of greater trochanter of left femur, initial encounter for closed fracture */
"S72115B"  =  "1"  /* Nondisplaced fracture of greater trochanter of left femur, initial encounter for open fracture type I or II */
"S72115C"  =  "1"  /* Nondisplaced fracture of greater trochanter of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72116A"  =  "1"  /* Nondisplaced fracture of greater trochanter of unspecified femur, initial encounter for closed fracture */
"S72116B"  =  "1"  /* Nondisplaced fracture of greater trochanter of unspecified femur, initial encounter for open fracture type I or II */
"S72116C"  =  "1"  /* Nondisplaced fracture of greater trochanter of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72121A"  =  "1"  /* Displaced fracture of lesser trochanter of right femur, initial encounter for closed fracture */
"S72121B"  =  "1"  /* Displaced fracture of lesser trochanter of right femur, initial encounter for open fracture type I or II */
"S72121C"  =  "1"  /* Displaced fracture of lesser trochanter of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72122A"  =  "1"  /* Displaced fracture of lesser trochanter of left femur, initial encounter for closed fracture */
"S72122B"  =  "1"  /* Displaced fracture of lesser trochanter of left femur, initial encounter for open fracture type I or II */
"S72122C"  =  "1"  /* Displaced fracture of lesser trochanter of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72123A"  =  "1"  /* Displaced fracture of lesser trochanter of unspecified femur, initial encounter for closed fracture */
"S72123B"  =  "1"  /* Displaced fracture of lesser trochanter of unspecified femur, initial encounter for open fracture type I or II */
"S72123C"  =  "1"  /* Displaced fracture of lesser trochanter of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72124A"  =  "1"  /* Nondisplaced fracture of lesser trochanter of right femur, initial encounter for closed fracture */
"S72124B"  =  "1"  /* Nondisplaced fracture of lesser trochanter of right femur, initial encounter for open fracture type I or II */
"S72124C"  =  "1"  /* Nondisplaced fracture of lesser trochanter of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72125A"  =  "1"  /* Nondisplaced fracture of lesser trochanter of left femur, initial encounter for closed fracture */
"S72125B"  =  "1"  /* Nondisplaced fracture of lesser trochanter of left femur, initial encounter for open fracture type I or II */
"S72125C"  =  "1"  /* Nondisplaced fracture of lesser trochanter of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72126A"  =  "1"  /* Nondisplaced fracture of lesser trochanter of unspecified femur, initial encounter for closed fracture */
"S72126B"  =  "1"  /* Nondisplaced fracture of lesser trochanter of unspecified femur, initial encounter for open fracture type I or II */
"S72126C"  =  "1"  /* Nondisplaced fracture of lesser trochanter of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72131A"  =  "1"  /* Displaced apophyseal fracture of right femur, initial encounter for closed fracture */
"S72131B"  =  "1"  /* Displaced apophyseal fracture of right femur, initial encounter for open fracture type I or II */
"S72131C"  =  "1"  /* Displaced apophyseal fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72132A"  =  "1"  /* Displaced apophyseal fracture of left femur, initial encounter for closed fracture */
"S72132B"  =  "1"  /* Displaced apophyseal fracture of left femur, initial encounter for open fracture type I or II */
"S72132C"  =  "1"  /* Displaced apophyseal fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72133A"  =  "1"  /* Displaced apophyseal fracture of unspecified femur, initial encounter for closed fracture */
"S72133B"  =  "1"  /* Displaced apophyseal fracture of unspecified femur, initial encounter for open fracture type I or II */
"S72133C"  =  "1"  /* Displaced apophyseal fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72134A"  =  "1"  /* Nondisplaced apophyseal fracture of right femur, initial encounter for closed fracture */
"S72134B"  =  "1"  /* Nondisplaced apophyseal fracture of right femur, initial encounter for open fracture type I or II */
"S72134C"  =  "1"  /* Nondisplaced apophyseal fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72135A"  =  "1"  /* Nondisplaced apophyseal fracture of left femur, initial encounter for closed fracture */
"S72135B"  =  "1"  /* Nondisplaced apophyseal fracture of left femur, initial encounter for open fracture type I or II */
"S72135C"  =  "1"  /* Nondisplaced apophyseal fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72136A"  =  "1"  /* Nondisplaced apophyseal fracture of unspecified femur, initial encounter for closed fracture */
"S72136B"  =  "1"  /* Nondisplaced apophyseal fracture of unspecified femur, initial encounter for open fracture type I or II */
"S72136C"  =  "1"  /* Nondisplaced apophyseal fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72141A"  =  "1"  /* Displaced intertrochanteric fracture of right femur, initial encounter for closed fracture */
"S72141B"  =  "1"  /* Displaced intertrochanteric fracture of right femur, initial encounter for open fracture type I or II */
"S72141C"  =  "1"  /* Displaced intertrochanteric fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72142A"  =  "1"  /* Displaced intertrochanteric fracture of left femur, initial encounter for closed fracture */
"S72142B"  =  "1"  /* Displaced intertrochanteric fracture of left femur, initial encounter for open fracture type I or II */
"S72142C"  =  "1"  /* Displaced intertrochanteric fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72143A"  =  "1"  /* Displaced intertrochanteric fracture of unspecified femur, initial encounter for closed fracture */
"S72143B"  =  "1"  /* Displaced intertrochanteric fracture of unspecified femur, initial encounter for open fracture type I or II */
"S72143C"  =  "1"  /* Displaced intertrochanteric fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72144A"  =  "1"  /* Nondisplaced intertrochanteric fracture of right femur, initial encounter for closed fracture */
"S72144B"  =  "1"  /* Nondisplaced intertrochanteric fracture of right femur, initial encounter for open fracture type I or II */
"S72144C"  =  "1"  /* Nondisplaced intertrochanteric fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72145A"  =  "1"  /* Nondisplaced intertrochanteric fracture of left femur, initial encounter for closed fracture */
"S72145B"  =  "1"  /* Nondisplaced intertrochanteric fracture of left femur, initial encounter for open fracture type I or II */
"S72145C"  =  "1"  /* Nondisplaced intertrochanteric fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S72146A"  =  "1"  /* Nondisplaced intertrochanteric fracture of unspecified femur, initial encounter for closed fracture */
"S72146B"  =  "1"  /* Nondisplaced intertrochanteric fracture of unspecified femur, initial encounter for open fracture type I or II */
"S72146C"  =  "1"  /* Nondisplaced intertrochanteric fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S7221XA"  =  "1"  /* Displaced subtrochanteric fracture of right femur, initial encounter for closed fracture */
"S7221XB"  =  "1"  /* Displaced subtrochanteric fracture of right femur, initial encounter for open fracture type I or II */
"S7221XC"  =  "1"  /* Displaced subtrochanteric fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S7222XA"  =  "1"  /* Displaced subtrochanteric fracture of left femur, initial encounter for closed fracture */
"S7222XB"  =  "1"  /* Displaced subtrochanteric fracture of left femur, initial encounter for open fracture type I or II */
"S7222XC"  =  "1"  /* Displaced subtrochanteric fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S7223XA"  =  "1"  /* Displaced subtrochanteric fracture of unspecified femur, initial encounter for closed fracture */
"S7223XB"  =  "1"  /* Displaced subtrochanteric fracture of unspecified femur, initial encounter for open fracture type I or II */
"S7223XC"  =  "1"  /* Displaced subtrochanteric fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S7224XA"  =  "1"  /* Nondisplaced subtrochanteric fracture of right femur, initial encounter for closed fracture */
"S7224XB"  =  "1"  /* Nondisplaced subtrochanteric fracture of right femur, initial encounter for open fracture type I or II */
"S7224XC"  =  "1"  /* Nondisplaced subtrochanteric fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S7225XA"  =  "1"  /* Nondisplaced subtrochanteric fracture of left femur, initial encounter for closed fracture */
"S7225XB"  =  "1"  /* Nondisplaced subtrochanteric fracture of left femur, initial encounter for open fracture type I or II */
"S7225XC"  =  "1"  /* Nondisplaced subtrochanteric fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
"S7226XA"  =  "1"  /* Nondisplaced subtrochanteric fracture of unspecified femur, initial encounter for closed fracture */
"S7226XB"  =  "1"  /* Nondisplaced subtrochanteric fracture of unspecified femur, initial encounter for open fracture type I or II */
"S7226XC"  =  "1"  /* Nondisplaced subtrochanteric fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC */
OTHER   = '0' ;

 /* Exclude: periprosthetic fracture */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $MTHIP2D
"T84040A"  =  "1"  /* Periprosthetic fracture around internal prosthetic right hip joint, initial encounter */
"T84041A"  =  "1"  /* Periprosthetic fracture around internal prosthetic left hip joint, initial encounter */
OTHER   = '0' ;

 /* ------------------------------- */
 /* Pneumonia mortality (MTPNEU)    */
 /*   Mortality Indicator / IQI #20 */
 /* ------------------------------- */

 /* Pneumonia */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $MTPNEUD
"A0222"  =  "1"  /* Salmonella pneumonia */
"A212"  =  "1"  /* Pulmonary tularemia */
"A221"  =  "1"  /* Pulmonary anthrax */
"A3701"  =  "1"  /* Whooping cough due to Bordetella pertussis with pneumonia */
"A3711"  =  "1"  /* Whooping cough due to Bordetella parapertussis with pneumonia */
"A3781"  =  "1"  /* Whooping cough due to other Bordetella species with pneumonia */
"A3791"  =  "1"  /* Whooping cough, unspecified species with pneumonia */
"A420"  =  "1"  /* Pulmonary actinomycosis */
"A430"  =  "1"  /* Pulmonary nocardiosis */
"A481"  =  "1"  /* Legionnaires' disease */
"B012"  =  "1"  /* Varicella pneumonia */
"B052"  =  "1"  /* Measles complicated by pneumonia */
"B250"  =  "1"  /* Cytomegaloviral pneumonitis */
"B371"  =  "1"  /* Pulmonary candidiasis */
"B380"  =  "1"  /* Acute pulmonary coccidioidomycosis */
"B381"  =  "1"  /* Chronic pulmonary coccidioidomycosis */
"B382"  =  "1"  /* Pulmonary coccidioidomycosis, unspecified */
"B390"  =  "1"  /* Acute pulmonary histoplasmosis capsulati */
"B391"  =  "1"  /* Chronic pulmonary histoplasmosis capsulati */
"B392"  =  "1"  /* Pulmonary histoplasmosis capsulati, unspecified */
"B440"  =  "1"  /* Invasive pulmonary aspergillosis */
"B583"  =  "1"  /* Pulmonary toxoplasmosis */
"B59"  =  "1"  /* Pneumocystosis */
"B7781"  =  "1"  /* Ascariasis pneumonia */
"J1000"  =  "1"  /* Influenza due to other identified influenza virus with unspecified type of pneumonia */
"J1008"  =  "1"  /* Influenza due to other identified influenza virus with other specified pneumonia */
"J1100"  =  "1"  /* Influenza due to unidentified influenza virus with unspecified type of pneumonia */
"J1108"  =  "1"  /* Influenza due to unidentified influenza virus with specified pneumonia */
"J120"  =  "1"  /* Adenoviral pneumonia */
"J121"  =  "1"  /* Respiratory syncytial virus pneumonia */
"J122"  =  "1"  /* Parainfluenza virus pneumonia */
"J123"  =  "1"  /* Human metapneumovirus pneumonia */
"J1281"  =  "1"  /* Pneumonia due to SARS-associated coronavirus */
"J1289"  =  "1"  /* Other viral pneumonia */
"J129"  =  "1"  /* Viral pneumonia, unspecified */
"J13"  =  "1"  /* Pneumonia due to Streptococcus pneumoniae */
"J14"  =  "1"  /* Pneumonia due to Hemophilus influenzae */
"J150"  =  "1"  /* Pneumonia due to Klebsiella pneumoniae */
"J151"  =  "1"  /* Pneumonia due to Pseudomonas */
"J1520"  =  "1"  /* Pneumonia due to staphylococcus, unspecified */
"J15211"  =  "1"  /* Pneumonia due to Methicillin susceptible Staphylococcus aureus */
"J15212"  =  "1"  /* Pneumonia due to Methicillin resistant Staphylococcus aureus */
"J1529"  =  "1"  /* Pneumonia due to other staphylococcus */
"J153"  =  "1"  /* Pneumonia due to streptococcus, group B */
"J154"  =  "1"  /* Pneumonia due to other streptococci */
"J155"  =  "1"  /* Pneumonia due to Escherichia coli */
"J156"  =  "1"  /* Pneumonia due to other aerobic Gram-negative bacteria */
"J157"  =  "1"  /* Pneumonia due to Mycoplasma pneumoniae */
"J158"  =  "1"  /* Pneumonia due to other specified bacteria */
"J159"  =  "1"  /* Unspecified bacterial pneumonia */
"J160"  =  "1"  /* Chlamydial pneumonia */
"J168"  =  "1"  /* Pneumonia due to other specified infectious organisms */
"J17"  =  "1"  /* Pneumonia in diseases classified elsewhere */
"J180"  =  "1"  /* Bronchopneumonia, unspecified organism */
"J181"  =  "1"  /* Lobar pneumonia, unspecified organism */
"J188"  =  "1"  /* Other pneumonia, unspecified organism */
"J189"  =  "1"  /* Pneumonia, unspecified organism */
OTHER  = '0' ;


 * -------------------------------------------------------------- ;
 * --- FORMATS - PROVIDER LEVEL UTILIZATION INDICATORS ---------- ;
 * -------------------------------------------------------------- ;

 /* -------------------------------------------------- */
 /* Cesarean Section rate (PRCSEC)                     */
 /*   Utilization (provider) Indicator / IQI #21       */
 /* -------------------------------------------------- */

 /* All Births (Population at Risk): */
 /* DRG Codes: */

VALUE $PRBRTHG
'370' = '1'  /* CESAREAN SECTION W CC */
'371' = '1'  /* CESAREAN SECTION W/O CC */
'372' = '1'  /* VAGINAL DELIVERY W COMPLICATING DIAGNOSES */
'373' = '1'  /* VAGINAL DELIVERY W/O COMPLICATING DIAGNOSES */
'374' = '1'  /* VAGINAL DELIVERY W STERILIZATION &/OR D&C */
'375' = '1'  /* VAGINAL DELIVERY W O.R. PROC EXCEPT STERIL &/OR D&C */
OTHER  = '0' ;

 /* All Births (Population at Risk): */
 /* MSDRG Codes: */

VALUE $PRBRT2G
'765' = '1'  /* Cesarean section w CC/MCC */
'766' = '1'  /* Cesarean section w/o CC/MCC */
'767' = '1'  /* Vaginal delivery w sterilization &/or D&C */
'768' = '1'  /* Vaginal delivery w O.R. proc except steril &/or D&C */
'774' = '1'  /* Vaginal delivery w complicating diagnoses */
'775' = '1'  /* Vaginal delivery w/o complicating diagnoses */
OTHER  = '0' ;

 /* C-Section (Outcome of Interest): */
 /* DRG Codes: */

VALUE $PRCSECG
'370' = '1'  /* CESAREAN SECTION W CC */
'371' = '1'  /* CESAREAN SECTION W/O CC */
OTHER  = '0' ;

 /* C-Section (Outcome of Interest): */
 /* MSDRG Codes: */

VALUE $PRCSE2G
'765' = '1'  /* Cesarean section w CC/MCC */
'766' = '1'  /* Cesarean section w/o CC/MCC */
OTHER  = '0' ;

 /* C-Section (Outcome of Interest): */
 /* ICD-10-CM Procedure Codes: */

VALUE $PRCSECP
"10D00Z0"  =  "1"  /* Extraction of Products of Conception, Classical, Open Approach */
"10D00Z1"  =  "1"  /* Extraction of Products of Conception, Low Cervical, Open Approach */
"10D00Z2"  =  "1"  /* Extraction of Products of Conception, Extraperitoneal, Open Approach */
OTHER  = '0' ;

VALUE $PRCSE2P
"10A00ZZ"  =  "1"  /* Abortion of Products of Conception, Open Approach */
"10A03ZZ"  =  "1"  /* Abortion of Products of Conception, Percutaneous Approach */
"10A04ZZ"  =  "1"  /* Abortion of Products of Conception, Percutaneous Endoscopic Approach */
OTHER  = '0' ;

 /* Exclude: Breech delivery, Preterm, Fetal Death, 
    Multiple Birth                                    */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $PRCSECD
"O30001"  =  "1"  /* Twin pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, first trimester */
"O30002"  =  "1"  /* Twin pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, second trimester */
"O30003"  =  "1"  /* Twin pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, third trimester */
"O30009"  =  "1"  /* Twin pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, unspecified trimester */
"O30011"  =  "1"  /* Twin pregnancy, monochorionic/monoamniotic, first trimester */
"O30012"  =  "1"  /* Twin pregnancy, monochorionic/monoamniotic, second trimester */
"O30013"  =  "1"  /* Twin pregnancy, monochorionic/monoamniotic, third trimester */
"O30019"  =  "1"  /* Twin pregnancy, monochorionic/monoamniotic, unspecified trimester */
"O30021"  =  "1"  /* Conjoined twin pregnancy, first trimester */
"O30022"  =  "1"  /* Conjoined twin pregnancy, second trimester */
"O30023"  =  "1"  /* Conjoined twin pregnancy, third trimester */
"O30029"  =  "1"  /* Conjoined twin pregnancy, unspecified trimester */
"O30031"  =  "1"  /* Twin pregnancy, monochorionic/diamniotic, first trimester */
"O30032"  =  "1"  /* Twin pregnancy, monochorionic/diamniotic, second trimester */
"O30033"  =  "1"  /* Twin pregnancy, monochorionic/diamniotic, third trimester */
"O30039"  =  "1"  /* Twin pregnancy, monochorionic/diamniotic, unspecified trimester */
"O30041"  =  "1"  /* Twin pregnancy, dichorionic/diamniotic, first trimester */
"O30042"  =  "1"  /* Twin pregnancy, dichorionic/diamniotic, second trimester */
"O30043"  =  "1"  /* Twin pregnancy, dichorionic/diamniotic, third trimester */
"O30049"  =  "1"  /* Twin pregnancy, dichorionic/diamniotic, unspecified trimester */
"O30091"  =  "1"  /* Twin pregnancy, unable to determine number of placenta and number of amniotic sacs, first trimester */
"O30092"  =  "1"  /* Twin pregnancy, unable to determine number of placenta and number of amniotic sacs, second trimester */
"O30093"  =  "1"  /* Twin pregnancy, unable to determine number of placenta and number of amniotic sacs, third trimester */
"O30099"  =  "1"  /* Twin pregnancy, unable to determine number of placenta and number of amniotic sacs, unspecified trimester */
"O30101"  =  "1"  /* Triplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, first trimester */
"O30102"  =  "1"  /* Triplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, second trimester */
"O30103"  =  "1"  /* Triplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, third trimester */
"O30109"  =  "1"  /* Triplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, unspecified trimester */
"O30111"  =  "1"  /* Triplet pregnancy with two or more monochorionic fetuses, first trimester */
"O30112"  =  "1"  /* Triplet pregnancy with two or more monochorionic fetuses, second trimester */
"O30113"  =  "1"  /* Triplet pregnancy with two or more monochorionic fetuses, third trimester */
"O30119"  =  "1"  /* Triplet pregnancy with two or more monochorionic fetuses, unspecified trimester */
"O30121"  =  "1"  /* Triplet pregnancy with two or more monoamniotic fetuses, first trimester */
"O30122"  =  "1"  /* Triplet pregnancy with two or more monoamniotic fetuses, second trimester */
"O30123"  =  "1"  /* Triplet pregnancy with two or more monoamniotic fetuses, third trimester */
"O30129"  =  "1"  /* Triplet pregnancy with two or more monoamniotic fetuses, unspecified trimester */
"O30191"  =  "1"  /* Triplet pregnancy, unable to determine number of placenta and number of amniotic sacs, first trimester */
"O30192"  =  "1"  /* Triplet pregnancy, unable to determine number of placenta and number of amniotic sacs, second trimester */
"O30193"  =  "1"  /* Triplet pregnancy, unable to determine number of placenta and number of amniotic sacs, third trimester */
"O30199"  =  "1"  /* Triplet pregnancy, unable to determine number of placenta and number of amniotic sacs, unspecified trimester */
"O30201"  =  "1"  /* Quadruplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, first trimester */
"O30202"  =  "1"  /* Quadruplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, second trimester */
"O30203"  =  "1"  /* Quadruplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, third trimester */
"O30209"  =  "1"  /* Quadruplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, unspecified trimester */
"O30211"  =  "1"  /* Quadruplet pregnancy with two or more monochorionic fetuses, first trimester */
"O30212"  =  "1"  /* Quadruplet pregnancy with two or more monochorionic fetuses, second trimester */
"O30213"  =  "1"  /* Quadruplet pregnancy with two or more monochorionic fetuses, third trimester */
"O30219"  =  "1"  /* Quadruplet pregnancy with two or more monochorionic fetuses, unspecified trimester */
"O30221"  =  "1"  /* Quadruplet pregnancy with two or more monoamniotic fetuses, first trimester */
"O30222"  =  "1"  /* Quadruplet pregnancy with two or more monoamniotic fetuses, second trimester */
"O30223"  =  "1"  /* Quadruplet pregnancy with two or more monoamniotic fetuses, third trimester */
"O30229"  =  "1"  /* Quadruplet pregnancy with two or more monoamniotic fetuses, unspecified trimester */
"O30291"  =  "1"  /* Quadruplet pregnancy, unable to determine number of placenta and number of amniotic sacs, first trimester */
"O30292"  =  "1"  /* Quadruplet pregnancy, unable to determine number of placenta and number of amniotic sacs, second trimester */
"O30293"  =  "1"  /* Quadruplet pregnancy, unable to determine number of placenta and number of amniotic sacs, third trimester */
"O30299"  =  "1"  /* Quadruplet pregnancy, unable to determine number of placenta and number of amniotic sacs, unspecified trimester */
"O30801"  =  "1"  /* Other specified multiple gestation, unspecified number of placenta and unspecified number of amniotic sacs, first trimester */
"O30802"  =  "1"  /* Other specified multiple gestation, unspecified number of placenta and unspecified number of amniotic sacs, second trimester */
"O30803"  =  "1"  /* Other specified multiple gestation, unspecified number of placenta and unspecified number of amniotic sacs, third trimester */
"O30809"  =  "1"  /* Other specified multiple gestation, unspecified number of placenta and unspecified number of amniotic sacs, unspecified trimester */
"O30811"  =  "1"  /* Other specified multiple gestation with two or more monochorionic fetuses, first trimester */
"O30812"  =  "1"  /* Other specified multiple gestation with two or more monochorionic fetuses, second trimester */
"O30813"  =  "1"  /* Other specified multiple gestation with two or more monochorionic fetuses, third trimester */
"O30819"  =  "1"  /* Other specified multiple gestation with two or more monochorionic fetuses, unspecified trimester */
"O30821"  =  "1"  /* Other specified multiple gestation with two or more monoamniotic fetuses, first trimester */
"O30822"  =  "1"  /* Other specified multiple gestation with two or more monoamniotic fetuses, second trimester */
"O30823"  =  "1"  /* Other specified multiple gestation with two or more monoamniotic fetuses, third trimester */
"O30829"  =  "1"  /* Other specified multiple gestation with two or more monoamniotic fetuses, unspecified trimester */
"O30891"  =  "1"  /* Other specified multiple gestation, unable to determine number of placenta and number of amniotic sacs, first trimester */
"O30892"  =  "1"  /* Other specified multiple gestation, unable to determine number of placenta and number of amniotic sacs, second trimester */
"O30893"  =  "1"  /* Other specified multiple gestation, unable to determine number of placenta and number of amniotic sacs, third trimester */
"O30899"  =  "1"  /* Other specified multiple gestation, unable to determine number of placenta and number of amniotic sacs, unspecified trimester */
"O3090"  =  "1"  /* Multiple gestation, unspecified, unspecified trimester */
"O3091"  =  "1"  /* Multiple gestation, unspecified, first trimester */
"O3092"  =  "1"  /* Multiple gestation, unspecified, second trimester */
"O3093"  =  "1"  /* Multiple gestation, unspecified, third trimester */
"O3110X0"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, not applicable or unspecified */
"O3110X1"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 1 */
"O3110X2"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 2 */
"O3110X3"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 3 */
"O3110X4"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 4 */
"O3110X5"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 5 */
"O3110X9"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, other fetus */
"O3111X0"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, not applicable or unspecified */
"O3111X1"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 1 */
"O3111X2"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 2 */
"O3111X3"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 3 */
"O3111X4"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 4 */
"O3111X5"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 5 */
"O3111X9"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, other fetus */
"O3112X0"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, not applicable or unspecified */
"O3112X1"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 1 */
"O3112X2"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 2 */
"O3112X3"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 3 */
"O3112X4"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 4 */
"O3112X5"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 5 */
"O3112X9"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, other fetus */
"O3113X0"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, not applicable or unspecified */
"O3113X1"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 1 */
"O3113X2"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 2 */
"O3113X3"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 3 */
"O3113X4"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 4 */
"O3113X5"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 5 */
"O3113X9"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, other fetus */
"O3120X0"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, not applicable or unspecified */
"O3120X1"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 1 */
"O3120X2"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 2 */
"O3120X3"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 3 */
"O3120X4"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 4 */
"O3120X5"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 5 */
"O3120X9"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, other fetus */
"O3121X0"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, first trimester, not applicable or unspecified */
"O3121X1"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 1 */
"O3121X2"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 2 */
"O3121X3"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 3 */
"O3121X4"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 4 */
"O3121X5"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 5 */
"O3121X9"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, first trimester, other fetus */
"O3122X0"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, second trimester, not applicable or unspecified */
"O3122X1"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 1 */
"O3122X2"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 2 */
"O3122X3"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 3 */
"O3122X4"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 4 */
"O3122X5"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 5 */
"O3122X9"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, second trimester, other fetus */
"O3123X0"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, third trimester, not applicable or unspecified */
"O3123X1"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 1 */
"O3123X2"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 2 */
"O3123X3"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 3 */
"O3123X4"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 4 */
"O3123X5"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 5 */
"O3123X9"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, third trimester, other fetus */
"O318X10"  =  "1"  /* Other complications specific to multiple gestation, first trimester, not applicable or unspecified */
"O318X11"  =  "1"  /* Other complications specific to multiple gestation, first trimester, fetus 1 */
"O318X12"  =  "1"  /* Other complications specific to multiple gestation, first trimester, fetus 2 */
"O318X13"  =  "1"  /* Other complications specific to multiple gestation, first trimester, fetus 3 */
"O318X14"  =  "1"  /* Other complications specific to multiple gestation, first trimester, fetus 4 */
"O318X15"  =  "1"  /* Other complications specific to multiple gestation, first trimester, fetus 5 */
"O318X19"  =  "1"  /* Other complications specific to multiple gestation, first trimester, other fetus */
"O318X20"  =  "1"  /* Other complications specific to multiple gestation, second trimester, not applicable or unspecified */
"O318X21"  =  "1"  /* Other complications specific to multiple gestation, second trimester, fetus 1 */
"O318X22"  =  "1"  /* Other complications specific to multiple gestation, second trimester, fetus 2 */
"O318X23"  =  "1"  /* Other complications specific to multiple gestation, second trimester, fetus 3 */
"O318X24"  =  "1"  /* Other complications specific to multiple gestation, second trimester, fetus 4 */
"O318X25"  =  "1"  /* Other complications specific to multiple gestation, second trimester, fetus 5 */
"O318X29"  =  "1"  /* Other complications specific to multiple gestation, second trimester, other fetus */
"O318X30"  =  "1"  /* Other complications specific to multiple gestation, third trimester, not applicable or unspecified */
"O318X31"  =  "1"  /* Other complications specific to multiple gestation, third trimester, fetus 1 */
"O318X32"  =  "1"  /* Other complications specific to multiple gestation, third trimester, fetus 2 */
"O318X33"  =  "1"  /* Other complications specific to multiple gestation, third trimester, fetus 3 */
"O318X34"  =  "1"  /* Other complications specific to multiple gestation, third trimester, fetus 4 */
"O318X35"  =  "1"  /* Other complications specific to multiple gestation, third trimester, fetus 5 */
"O318X39"  =  "1"  /* Other complications specific to multiple gestation, third trimester, other fetus */
"O318X90"  =  "1"  /* Other complications specific to multiple gestation, unspecified trimester, not applicable or unspecified */
"O318X91"  =  "1"  /* Other complications specific to multiple gestation, unspecified trimester, fetus 1 */
"O318X92"  =  "1"  /* Other complications specific to multiple gestation, unspecified trimester, fetus 2 */
"O318X93"  =  "1"  /* Other complications specific to multiple gestation, unspecified trimester, fetus 3 */
"O318X94"  =  "1"  /* Other complications specific to multiple gestation, unspecified trimester, fetus 4 */
"O318X95"  =  "1"  /* Other complications specific to multiple gestation, unspecified trimester, fetus 5 */
"O318X99"  =  "1"  /* Other complications specific to multiple gestation, unspecified trimester, other fetus */
"O321XX0"  =  "1"  /* Maternal care for breech presentation, not applicable or unspecified */
"O321XX1"  =  "1"  /* Maternal care for breech presentation, fetus 1 */
"O321XX2"  =  "1"  /* Maternal care for breech presentation, fetus 2 */
"O321XX3"  =  "1"  /* Maternal care for breech presentation, fetus 3 */
"O321XX4"  =  "1"  /* Maternal care for breech presentation, fetus 4 */
"O321XX5"  =  "1"  /* Maternal care for breech presentation, fetus 5 */
"O321XX9"  =  "1"  /* Maternal care for breech presentation, other fetus */
"O322XX0"  =  "1"  /* Maternal care for transverse and oblique lie, not applicable or unspecified */
"O322XX1"  =  "1"  /* Maternal care for transverse and oblique lie, fetus 1 */
"O322XX2"  =  "1"  /* Maternal care for transverse and oblique lie, fetus 2 */
"O322XX3"  =  "1"  /* Maternal care for transverse and oblique lie, fetus 3 */
"O322XX4"  =  "1"  /* Maternal care for transverse and oblique lie, fetus 4 */
"O322XX5"  =  "1"  /* Maternal care for transverse and oblique lie, fetus 5 */
"O322XX9"  =  "1"  /* Maternal care for transverse and oblique lie, other fetus */
"O323XX0"  =  "1"  /* Maternal care for face, brow and chin presentation, not applicable or unspecified */
"O323XX1"  =  "1"  /* Maternal care for face, brow and chin presentation, fetus 1 */
"O323XX2"  =  "1"  /* Maternal care for face, brow and chin presentation, fetus 2 */
"O323XX3"  =  "1"  /* Maternal care for face, brow and chin presentation, fetus 3 */
"O323XX4"  =  "1"  /* Maternal care for face, brow and chin presentation, fetus 4 */
"O323XX5"  =  "1"  /* Maternal care for face, brow and chin presentation, fetus 5 */
"O323XX9"  =  "1"  /* Maternal care for face, brow and chin presentation, other fetus */
"O329XX0"  =  "1"  /* Maternal care for malpresentation of fetus, unspecified, not applicable or unspecified */
"O329XX1"  =  "1"  /* Maternal care for malpresentation of fetus, unspecified, fetus 1 */
"O329XX2"  =  "1"  /* Maternal care for malpresentation of fetus, unspecified, fetus 2 */
"O329XX3"  =  "1"  /* Maternal care for malpresentation of fetus, unspecified, fetus 3 */
"O329XX4"  =  "1"  /* Maternal care for malpresentation of fetus, unspecified, fetus 4 */
"O329XX5"  =  "1"  /* Maternal care for malpresentation of fetus, unspecified, fetus 5 */
"O329XX9"  =  "1"  /* Maternal care for malpresentation of fetus, unspecified, other fetus */
"O364XX0"  =  "1"  /* Maternal care for intrauterine death, not applicable or unspecified */
"O364XX1"  =  "1"  /* Maternal care for intrauterine death, fetus 1 */
"O364XX2"  =  "1"  /* Maternal care for intrauterine death, fetus 2 */
"O364XX3"  =  "1"  /* Maternal care for intrauterine death, fetus 3 */
"O364XX4"  =  "1"  /* Maternal care for intrauterine death, fetus 4 */
"O364XX5"  =  "1"  /* Maternal care for intrauterine death, fetus 5 */
"O364XX9"  =  "1"  /* Maternal care for intrauterine death, other fetus */
"O6010X0"  =  "1"  /* Preterm labor with preterm delivery, unspecified trimester, not applicable or unspecified */
"O6010X1"  =  "1"  /* Preterm labor with preterm delivery, unspecified trimester, fetus 1 */
"O6010X2"  =  "1"  /* Preterm labor with preterm delivery, unspecified trimester, fetus 2 */
"O6010X3"  =  "1"  /* Preterm labor with preterm delivery, unspecified trimester, fetus 3 */
"O6010X4"  =  "1"  /* Preterm labor with preterm delivery, unspecified trimester, fetus 4 */
"O6010X5"  =  "1"  /* Preterm labor with preterm delivery, unspecified trimester, fetus 5 */
"O6010X9"  =  "1"  /* Preterm labor with preterm delivery, unspecified trimester, other fetus */
"O6012X0"  =  "1"  /* Preterm labor second trimester with preterm delivery second trimester, not applicable or unspecified */
"O6012X1"  =  "1"  /* Preterm labor second trimester with preterm delivery second trimester, fetus 1 */
"O6012X2"  =  "1"  /* Preterm labor second trimester with preterm delivery second trimester, fetus 2 */
"O6012X3"  =  "1"  /* Preterm labor second trimester with preterm delivery second trimester, fetus 3 */
"O6012X4"  =  "1"  /* Preterm labor second trimester with preterm delivery second trimester, fetus 4 */
"O6012X5"  =  "1"  /* Preterm labor second trimester with preterm delivery second trimester, fetus 5 */
"O6012X9"  =  "1"  /* Preterm labor second trimester with preterm delivery second trimester, other fetus */
"O6013X0"  =  "1"  /* Preterm labor second trimester with preterm delivery third trimester, not applicable or unspecified */
"O6013X1"  =  "1"  /* Preterm labor second trimester with preterm delivery third trimester, fetus 1 */
"O6013X2"  =  "1"  /* Preterm labor second trimester with preterm delivery third trimester, fetus 2 */
"O6013X3"  =  "1"  /* Preterm labor second trimester with preterm delivery third trimester, fetus 3 */
"O6013X4"  =  "1"  /* Preterm labor second trimester with preterm delivery third trimester, fetus 4 */
"O6013X5"  =  "1"  /* Preterm labor second trimester with preterm delivery third trimester, fetus 5 */
"O6013X9"  =  "1"  /* Preterm labor second trimester with preterm delivery third trimester, other fetus */
"O6014X0"  =  "1"  /* Preterm labor third trimester with preterm delivery third trimester, not applicable or unspecified */
"O6014X1"  =  "1"  /* Preterm labor third trimester with preterm delivery third trimester, fetus 1 */
"O6014X2"  =  "1"  /* Preterm labor third trimester with preterm delivery third trimester, fetus 2 */
"O6014X3"  =  "1"  /* Preterm labor third trimester with preterm delivery third trimester, fetus 3 */
"O6014X4"  =  "1"  /* Preterm labor third trimester with preterm delivery third trimester, fetus 4 */
"O6014X5"  =  "1"  /* Preterm labor third trimester with preterm delivery third trimester, fetus 5 */
"O6014X9"  =  "1"  /* Preterm labor third trimester with preterm delivery third trimester, other fetus */
"O632"  =  "1"  /* Delayed delivery of second twin, triplet, etc. */
"O641XX0"  =  "1"  /* Obstructed labor due to breech presentation, not applicable or unspecified */
"O641XX1"  =  "1"  /* Obstructed labor due to breech presentation, fetus 1 */
"O641XX2"  =  "1"  /* Obstructed labor due to breech presentation, fetus 2 */
"O641XX3"  =  "1"  /* Obstructed labor due to breech presentation, fetus 3 */
"O641XX4"  =  "1"  /* Obstructed labor due to breech presentation, fetus 4 */
"O641XX5"  =  "1"  /* Obstructed labor due to breech presentation, fetus 5 */
"O641XX9"  =  "1"  /* Obstructed labor due to breech presentation, other fetus */
"O642XX0"  =  "1"  /* Obstructed labor due to face presentation, not applicable or unspecified */
"O642XX1"  =  "1"  /* Obstructed labor due to face presentation, fetus 1 */
"O642XX2"  =  "1"  /* Obstructed labor due to face presentation, fetus 2 */
"O642XX3"  =  "1"  /* Obstructed labor due to face presentation, fetus 3 */
"O642XX4"  =  "1"  /* Obstructed labor due to face presentation, fetus 4 */
"O642XX5"  =  "1"  /* Obstructed labor due to face presentation, fetus 5 */
"O642XX9"  =  "1"  /* Obstructed labor due to face presentation, other fetus */
"O643XX0"  =  "1"  /* Obstructed labor due to brow presentation, not applicable or unspecified */
"O643XX1"  =  "1"  /* Obstructed labor due to brow presentation, fetus 1 */
"O643XX2"  =  "1"  /* Obstructed labor due to brow presentation, fetus 2 */
"O643XX3"  =  "1"  /* Obstructed labor due to brow presentation, fetus 3 */
"O643XX4"  =  "1"  /* Obstructed labor due to brow presentation, fetus 4 */
"O643XX5"  =  "1"  /* Obstructed labor due to brow presentation, fetus 5 */
"O643XX9"  =  "1"  /* Obstructed labor due to brow presentation, other fetus */
"O661"  =  "1"  /* Obstructed labor due to locked twins */
"O666"  =  "1"  /* Obstructed labor due to other multiple fetuses */
"Z371"  =  "1"  /* Single stillbirth */
"Z372"  =  "1"  /* Twins, both liveborn */
"Z373"  =  "1"  /* Twins, one liveborn and one stillborn */
"Z374"  =  "1"  /* Twins, both stillborn */
"Z3750"  =  "1"  /* Multiple births, unspecified, all liveborn */
"Z3751"  =  "1"  /* Triplets, all liveborn */
"Z3752"  =  "1"  /* Quadruplets, all liveborn */
"Z3753"  =  "1"  /* Quintuplets, all liveborn */
"Z3754"  =  "1"  /* Sextuplets, all liveborn */
"Z3759"  =  "1"  /* Other multiple births, all liveborn */
"Z3760"  =  "1"  /* Multiple births, unspecified, some liveborn */
"Z3761"  =  "1"  /* Triplets, some liveborn */
"Z3762"  =  "1"  /* Quadruplets, some liveborn */
"Z3763"  =  "1"  /* Quintuplets, some liveborn */
"Z3764"  =  "1"  /* Sextuplets, some liveborn */
"Z3769"  =  "1"  /* Other multiple births, some liveborn */
"Z377"  =  "1"  /* Other multiple births, all stillborn */
OTHER  = '0' ;

 /* ICD-10-CM Procedure Codes: */

VALUE $PRCSE3P /*per UCD - This setname Is not mappable to ICD-10-PCS because the ICD-10-PCS do not describe specific diagnoses.  The sole purpose of this setname is to capture breech presentation and delivery. */
   OTHER = '0'
   ;

 /* -------------------------------------------------- */
 /* Vaginal Birth after Cesarean Section rate (PRVBAC) */
 /*   Utilization (provider) Indicator / IQI #22       */
 /* -------------------------------------------------- */

 /* Previous C-Section (Population at Risk): */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $PRVBACD
"O3421"  =  "1"  /* Maternal care for scar from previous cesarean delivery */
"O6641"  =  "1"  /* Failed attempted vaginal birth after previous cesarean delivery  */
OTHER   = '0' ;

 /* Vaginal Birth (Outcome of Interest): */
 /* DRG Codes: */

VALUE $PRVAGBG
'372' = '1'  /* VAGINAL DELIVERY W COMPLICATING DIAGNOSES */
'373' = '1'  /* VAGINAL DELIVERY W/O COMPLICATING DIAGNOSES */
'374' = '1'  /* VAGINAL DELIVERY W STERILIZATION &/OR D&C */
'375' = '1'  /* VAGINAL DELIVERY W O.R. PROC EXCEPT STERIL &/OR D&C */
OTHER  = '0' ;

 /* Vaginal Birth (Outcome of Interest): */
 /* MSDRG Codes: */

VALUE $PRVAG2G
'767' = '1'  /* Vaginal delivery w sterilization &/or D&C */
'768' = '1'  /* Vaginal delivery w O.R. proc except steril &/or D&C */
'774' = '1'  /* Vaginal delivery w complicating diagnoses */
'775' = '1'  /* Vaginal delivery w/o complicating diagnoses */
OTHER  = '0' ;


 /* -------------------------------------------- */
 /* Laparoscopic Cholecystectomy rate (PRLAPO)   */
 /*   Utilization (provider) Indicator / IQI #23 */
 /* -------------------------------------------- */

 /* Laparoscopic Cholecystectomy (Outcome of Interest): */
 /* ICD-10-CM Procedure Code: */

VALUE $PRLAPOP
"0FT44ZZ"  =  "1"  /* Resection of Gallbladder, Percutaneous Endoscopic Approach */
OTHER  = '0' ;

 /* Cholecystectomy (Population at Risk): */
 /* ICD-10-CM Procedure Codes: */

VALUE $PRLAP2P
"0FT40ZZ"  =  "1"  /* Resection of Gallbladder, Open Approach */
"0FT44ZZ"  =  "1"  /* Resection of Gallbladder, Percutaneous Endoscopic Approach */
OTHER  = '0' ;

 /* Include Only: Uncomplicated Cholecystitis and/or Cholelithiasis */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $PRLAPOD
"K8000"  =  "1"  /* Calculus of gallbladder with acute cholecystitis without obstruction */
"K8001"  =  "1"  /* Calculus of gallbladder with acute cholecystitis with obstruction */
"K8010"  =  "1"  /* Calculus of gallbladder with chronic cholecystitis without obstruction */
"K8011"  =  "1"  /* Calculus of gallbladder with chronic cholecystitis with obstruction */
"K8012"  =  "1"  /* Calculus of gallbladder with acute and chronic cholecystitis without obstruction */
"K8013"  =  "1"  /* Calculus of gallbladder with acute and chronic cholecystitis with obstruction */
"K8018"  =  "1"  /* Calculus of gallbladder with other cholecystitis without obstruction */
"K8019"  =  "1"  /* Calculus of gallbladder with other cholecystitis with obstruction */
"K8020"  =  "1"  /* Calculus of gallbladder without cholecystitis without obstruction */
"K8021"  =  "1"  /* Calculus of gallbladder without cholecystitis with obstruction */
"K8080"  =  "1"  /* Other cholelithiasis without obstruction */
"K810"  =  "1"  /* Acute cholecystitis */
"K811"  =  "1"  /* Chronic cholecystitis */
"K812"  =  "1"  /* Acute cholecystitis with chronic cholecystitis */
"K819"  =  "1"  /* Cholecystitis, unspecified */
OTHER   = '0' ;

 /* -------------------------------------------- */
 /* Incidental Appendectomy rate (PRAPPN)        */
 /*   Utilization (provider) Indicator / IQI #24 */
 /* -------------------------------------------- */

 /* Incidental Appendectomy (Outcome of Interest): */
 /* ICD-10-CM Procedure Code: */

VALUE $PRAPPNP
"0DBJ0ZZ"  =  "1"  /* Excision of Appendix, Open Approach */
"0DBJ3ZZ"  =  "1"  /* Excision of Appendix, Percutaneous Approach */
"0DBJ4ZZ"  =  "1"  /* Excision of Appendix, Percutaneous Endoscopic Approach */
"0DBJ7ZZ"  =  "1"  /* Excision of Appendix, Via Natural or Artificial Opening */
"0DBJ8ZZ"  =  "1"  /* Excision of Appendix, Via Natural or Artificial Opening Endoscopic */
"0DTJ0ZZ"  =  "1"  /* Resection of Appendix, Open Approach */
"0DTJ4ZZ"  =  "1"  /* Resection of Appendix, Percutaneous Endoscopic Approach */
"0DTJ7ZZ"  =  "1"  /* Resection of Appendix, Via Natural or Artificial Opening */
"0DTJ8ZZ"  =  "1"  /* Resection of Appendix, Via Natural or Artificial Opening Endoscopic */
OTHER  = '0' ;

 /* Incidental Appendectomy (Population at Risk): */
 /* ICD-10-CM Procedure Code: */

VALUE $PRAPP2P
"008Q0ZZ"  =  "1"  /* Division of Vagus Nerve, Open Approach */
"008Q3ZZ"  =  "1"  /* Division of Vagus Nerve, Percutaneous Approach */
"008Q4ZZ"  =  "1"  /* Division of Vagus Nerve, Percutaneous Endoscopic Approach */
"015P0ZZ"  =  "1"  /* Destruction of Sacral Sympathetic Nerve, Open Approach */
"015P3ZZ"  =  "1"  /* Destruction of Sacral Sympathetic Nerve, Percutaneous Approach */
"015P4ZZ"  =  "1"  /* Destruction of Sacral Sympathetic Nerve, Percutaneous Endoscopic Approach */
"06L20ZZ"  =  "1"  /* Occlusion of Gastric Vein, Open Approach */
"06L23ZZ"  =  "1"  /* Occlusion of Gastric Vein, Percutaneous Approach */
"06L24ZZ"  =  "1"  /* Occlusion of Gastric Vein, Percutaneous Endoscopic Approach */
"06L30ZZ"  =  "1"  /* Occlusion of Esophageal Vein, Open Approach */
"06L33ZZ"  =  "1"  /* Occlusion of Esophageal Vein, Percutaneous Approach */
"06L34ZZ"  =  "1"  /* Occlusion of Esophageal Vein, Percutaneous Endoscopic Approach */
"075P0ZZ"  =  "1"  /* Destruction of Spleen, Open Approach */
"075P3ZZ"  =  "1"  /* Destruction of Spleen, Percutaneous Approach */
"075P4ZZ"  =  "1"  /* Destruction of Spleen, Percutaneous Endoscopic Approach */
"079P00Z"  =  "1"  /* Drainage of Spleen with Drainage Device, Open Approach */
"079P0ZX"  =  "1"  /* Drainage of Spleen, Open Approach, Diagnostic */
"079P0ZZ"  =  "1"  /* Drainage of Spleen, Open Approach */
"079P3ZZ"  =  "1"  /* Drainage of Spleen, Percutaneous Approach */
"079P4ZZ"  =  "1"  /* Drainage of Spleen, Percutaneous Endoscopic Approach */
"07BP0ZX"  =  "1"  /* Excision of Spleen, Open Approach, Diagnostic */
"07BP0ZZ"  =  "1"  /* Excision of Spleen, Open Approach */
"07BP3ZZ"  =  "1"  /* Excision of Spleen, Percutaneous Approach */
"07BP4ZZ"  =  "1"  /* Excision of Spleen, Percutaneous Endoscopic Approach */
"07CP0ZZ"  =  "1"  /* Extirpation of Matter from Spleen, Open Approach */
"07JP0ZZ"  =  "1"  /* Inspection of Spleen, Open Approach */
"07NP0ZZ"  =  "1"  /* Release Spleen, Open Approach */
"07NP3ZZ"  =  "1"  /* Release Spleen, Percutaneous Approach */
"07NP4ZZ"  =  "1"  /* Release Spleen, Percutaneous Endoscopic Approach */
"07PP00Z"  =  "1"  /* Removal of Drainage Device from Spleen, Open Approach */
"07PP03Z"  =  "1"  /* Removal of Infusion Device from Spleen, Open Approach */
"07PP30Z"  =  "1"  /* Removal of Drainage Device from Spleen, Percutaneous Approach */
"07PP33Z"  =  "1"  /* Removal of Infusion Device from Spleen, Percutaneous Approach */
"07PP40Z"  =  "1"  /* Removal of Drainage Device from Spleen, Percutaneous Endoscopic Approach */
"07PP43Z"  =  "1"  /* Removal of Infusion Device from Spleen, Percutaneous Endoscopic Approach */
"07QP0ZZ"  =  "1"  /* Repair Spleen, Open Approach */
"07QP3ZZ"  =  "1"  /* Repair Spleen, Percutaneous Approach */
"07QP4ZZ"  =  "1"  /* Repair Spleen, Percutaneous Endoscopic Approach */
"07SP0ZZ"  =  "1"  /* Reposition Spleen, Open Approach */
"07TP0ZZ"  =  "1"  /* Resection of Spleen, Open Approach */
"07TP4ZZ"  =  "1"  /* Resection of Spleen, Percutaneous Endoscopic Approach */
"07WP00Z"  =  "1"  /* Revision of Drainage Device in Spleen, Open Approach */
"07WP03Z"  =  "1"  /* Revision of Infusion Device in Spleen, Open Approach */
"07WP30Z"  =  "1"  /* Revision of Drainage Device in Spleen, Percutaneous Approach */
"07WP33Z"  =  "1"  /* Revision of Infusion Device in Spleen, Percutaneous Approach */
"07WP40Z"  =  "1"  /* Revision of Drainage Device in Spleen, Percutaneous Endoscopic Approach */
"07WP43Z"  =  "1"  /* Revision of Infusion Device in Spleen, Percutaneous Endoscopic Approach */
"07YP0Z0"  =  "1"  /* Transplantation of Spleen, Allogeneic, Open Approach */
"07YP0Z1"  =  "1"  /* Transplantation of Spleen, Syngeneic, Open Approach */
"07YP0Z2"  =  "1"  /* Transplantation of Spleen, Zooplastic, Open Approach */
"0BQR0ZZ"  =  "1"  /* Repair Right Diaphragm, Open Approach */
"0BQR3ZZ"  =  "1"  /* Repair Right Diaphragm, Percutaneous Approach */
"0BQR4ZZ"  =  "1"  /* Repair Right Diaphragm, Percutaneous Endoscopic Approach */
"0BQS0ZZ"  =  "1"  /* Repair Left Diaphragm, Open Approach */
"0BQS3ZZ"  =  "1"  /* Repair Left Diaphragm, Percutaneous Approach */
"0BQS4ZZ"  =  "1"  /* Repair Left Diaphragm, Percutaneous Endoscopic Approach */
"0BUR07Z"  =  "1"  /* Supplement Right Diaphragm with Autologous Tissue Substitute, Open Approach */
"0BUR0JZ"  =  "1"  /* Supplement Right Diaphragm with Synthetic Substitute, Open Approach */
"0BUR0KZ"  =  "1"  /* Supplement Right Diaphragm with Nonautologous Tissue Substitute, Open Approach */
"0BUR47Z"  =  "1"  /* Supplement Right Diaphragm with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BUR4JZ"  =  "1"  /* Supplement Right Diaphragm with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BUR4KZ"  =  "1"  /* Supplement Right Diaphragm with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BUS07Z"  =  "1"  /* Supplement Left Diaphragm with Autologous Tissue Substitute, Open Approach */
"0BUS0JZ"  =  "1"  /* Supplement Left Diaphragm with Synthetic Substitute, Open Approach */
"0BUS0KZ"  =  "1"  /* Supplement Left Diaphragm with Nonautologous Tissue Substitute, Open Approach */
"0BUS47Z"  =  "1"  /* Supplement Left Diaphragm with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BUS4JZ"  =  "1"  /* Supplement Left Diaphragm with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BUS4KZ"  =  "1"  /* Supplement Left Diaphragm with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D11079"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1107A"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1107B"  =  "1"  /* Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D110J9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D110JA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D110JB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D110K9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D110KA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D110KB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D110Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Open Approach */
"0D110ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Open Approach */
"0D110ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Open Approach */
"0D11479"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1147A"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1147B"  =  "1"  /* Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114J9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114JA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114JB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114K9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114KA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114KB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D114ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D114ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D11879"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1187A"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1187B"  =  "1"  /* Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118J9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118JA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118JB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118K9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118KA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118KB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D118ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D118ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D12079"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1207A"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1207B"  =  "1"  /* Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D120J9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D120JA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D120JB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D120K9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D120KA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D120KB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D120Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Open Approach */
"0D120ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Open Approach */
"0D120ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Open Approach */
"0D12479"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1247A"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1247B"  =  "1"  /* Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124J9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124JA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124JB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124K9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124KA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124KB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D124ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D124ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D12879"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1287A"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1287B"  =  "1"  /* Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128J9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128JA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128JB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128K9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128KA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128KB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D128ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D128ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D13079"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1307A"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1307B"  =  "1"  /* Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D130J9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D130JA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D130JB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D130K9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D130KA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D130KB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D130Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Open Approach */
"0D130ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Open Approach */
"0D130ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Open Approach */
"0D13479"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1347A"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1347B"  =  "1"  /* Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134J9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134JA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134JB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134K9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134KA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134KB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D134ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D134ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D13879"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1387A"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1387B"  =  "1"  /* Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138J9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138JA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138JB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138K9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138KA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138KB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D138ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D138ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D15079"  =  "1"  /* Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1507A"  =  "1"  /* Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1507B"  =  "1"  /* Bypass Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D150J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D150JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D150JB"  =  "1"  /* Bypass Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D150K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D150KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D150KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D150Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Open Approach */
"0D150ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Open Approach */
"0D150ZB"  =  "1"  /* Bypass Esophagus to Ileum, Open Approach */
"0D15479"  =  "1"  /* Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1547A"  =  "1"  /* Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1547B"  =  "1"  /* Bypass Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154JB"  =  "1"  /* Bypass Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D154ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D154ZB"  =  "1"  /* Bypass Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D15879"  =  "1"  /* Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1587A"  =  "1"  /* Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1587B"  =  "1"  /* Bypass Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158JB"  =  "1"  /* Bypass Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D158ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D158ZB"  =  "1"  /* Bypass Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D16079"  =  "1"  /* Bypass Stomach to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1607A"  =  "1"  /* Bypass Stomach to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1607B"  =  "1"  /* Bypass Stomach to Ileum with Autologous Tissue Substitute, Open Approach */
"0D1607L"  =  "1"  /* Bypass Stomach to Transverse Colon with Autologous Tissue Substitute, Open Approach */
"0D160J9"  =  "1"  /* Bypass Stomach to Duodenum with Synthetic Substitute, Open Approach */
"0D160JA"  =  "1"  /* Bypass Stomach to Jejunum with Synthetic Substitute, Open Approach */
"0D160JB"  =  "1"  /* Bypass Stomach to Ileum with Synthetic Substitute, Open Approach */
"0D160JL"  =  "1"  /* Bypass Stomach to Transverse Colon with Synthetic Substitute, Open Approach */
"0D160K9"  =  "1"  /* Bypass Stomach to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D160KA"  =  "1"  /* Bypass Stomach to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D160KB"  =  "1"  /* Bypass Stomach to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D160KL"  =  "1"  /* Bypass Stomach to Transverse Colon with Nonautologous Tissue Substitute, Open Approach */
"0D160Z9"  =  "1"  /* Bypass Stomach to Duodenum, Open Approach */
"0D160ZA"  =  "1"  /* Bypass Stomach to Jejunum, Open Approach */
"0D160ZB"  =  "1"  /* Bypass Stomach to Ileum, Open Approach */
"0D160ZL"  =  "1"  /* Bypass Stomach to Transverse Colon, Open Approach */
"0D16479"  =  "1"  /* Bypass Stomach to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1647A"  =  "1"  /* Bypass Stomach to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1647B"  =  "1"  /* Bypass Stomach to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1647L"  =  "1"  /* Bypass Stomach to Transverse Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D164J9"  =  "1"  /* Bypass Stomach to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D164JA"  =  "1"  /* Bypass Stomach to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D164JB"  =  "1"  /* Bypass Stomach to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D164JL"  =  "1"  /* Bypass Stomach to Transverse Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D164K9"  =  "1"  /* Bypass Stomach to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D164KA"  =  "1"  /* Bypass Stomach to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D164KB"  =  "1"  /* Bypass Stomach to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D164KL"  =  "1"  /* Bypass Stomach to Transverse Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D164Z9"  =  "1"  /* Bypass Stomach to Duodenum, Percutaneous Endoscopic Approach */
"0D164ZA"  =  "1"  /* Bypass Stomach to Jejunum, Percutaneous Endoscopic Approach */
"0D164ZB"  =  "1"  /* Bypass Stomach to Ileum, Percutaneous Endoscopic Approach */
"0D164ZL"  =  "1"  /* Bypass Stomach to Transverse Colon, Percutaneous Endoscopic Approach */
"0D16879"  =  "1"  /* Bypass Stomach to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1687A"  =  "1"  /* Bypass Stomach to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1687B"  =  "1"  /* Bypass Stomach to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1687L"  =  "1"  /* Bypass Stomach to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168J9"  =  "1"  /* Bypass Stomach to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168JA"  =  "1"  /* Bypass Stomach to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168JB"  =  "1"  /* Bypass Stomach to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168JL"  =  "1"  /* Bypass Stomach to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168K9"  =  "1"  /* Bypass Stomach to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168KA"  =  "1"  /* Bypass Stomach to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168KB"  =  "1"  /* Bypass Stomach to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168KL"  =  "1"  /* Bypass Stomach to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168Z9"  =  "1"  /* Bypass Stomach to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D168ZA"  =  "1"  /* Bypass Stomach to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D168ZB"  =  "1"  /* Bypass Stomach to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D168ZL"  =  "1"  /* Bypass Stomach to Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D19074"  =  "1"  /* Bypass Duodenum to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D19079"  =  "1"  /* Bypass Duodenum to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1907A"  =  "1"  /* Bypass Duodenum to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1907B"  =  "1"  /* Bypass Duodenum to Ileum with Autologous Tissue Substitute, Open Approach */
"0D1907L"  =  "1"  /* Bypass Duodenum to Transverse Colon with Autologous Tissue Substitute, Open Approach */
"0D190J4"  =  "1"  /* Bypass Duodenum to Cutaneous with Synthetic Substitute, Open Approach */
"0D190J9"  =  "1"  /* Bypass Duodenum to Duodenum with Synthetic Substitute, Open Approach */
"0D190JA"  =  "1"  /* Bypass Duodenum to Jejunum with Synthetic Substitute, Open Approach */
"0D190JB"  =  "1"  /* Bypass Duodenum to Ileum with Synthetic Substitute, Open Approach */
"0D190JL"  =  "1"  /* Bypass Duodenum to Transverse Colon with Synthetic Substitute, Open Approach */
"0D190K4"  =  "1"  /* Bypass Duodenum to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D190K9"  =  "1"  /* Bypass Duodenum to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D190KA"  =  "1"  /* Bypass Duodenum to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D190KB"  =  "1"  /* Bypass Duodenum to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D190KL"  =  "1"  /* Bypass Duodenum to Transverse Colon with Nonautologous Tissue Substitute, Open Approach */
"0D190Z4"  =  "1"  /* Bypass Duodenum to Cutaneous, Open Approach */
"0D190Z9"  =  "1"  /* Bypass Duodenum to Duodenum, Open Approach */
"0D190ZA"  =  "1"  /* Bypass Duodenum to Jejunum, Open Approach */
"0D190ZB"  =  "1"  /* Bypass Duodenum to Ileum, Open Approach */
"0D190ZL"  =  "1"  /* Bypass Duodenum to Transverse Colon, Open Approach */
"0D193J4"  =  "1"  /* Bypass Duodenum to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D19474"  =  "1"  /* Bypass Duodenum to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D19479"  =  "1"  /* Bypass Duodenum to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1947A"  =  "1"  /* Bypass Duodenum to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1947B"  =  "1"  /* Bypass Duodenum to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1947L"  =  "1"  /* Bypass Duodenum to Transverse Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D194J4"  =  "1"  /* Bypass Duodenum to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D194J9"  =  "1"  /* Bypass Duodenum to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D194JA"  =  "1"  /* Bypass Duodenum to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D194JB"  =  "1"  /* Bypass Duodenum to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D194JL"  =  "1"  /* Bypass Duodenum to Transverse Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D194K4"  =  "1"  /* Bypass Duodenum to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D194K9"  =  "1"  /* Bypass Duodenum to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D194KA"  =  "1"  /* Bypass Duodenum to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D194KB"  =  "1"  /* Bypass Duodenum to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D194KL"  =  "1"  /* Bypass Duodenum to Transverse Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D194Z4"  =  "1"  /* Bypass Duodenum to Cutaneous, Percutaneous Endoscopic Approach */
"0D194Z9"  =  "1"  /* Bypass Duodenum to Duodenum, Percutaneous Endoscopic Approach */
"0D194ZA"  =  "1"  /* Bypass Duodenum to Jejunum, Percutaneous Endoscopic Approach */
"0D194ZB"  =  "1"  /* Bypass Duodenum to Ileum, Percutaneous Endoscopic Approach */
"0D194ZL"  =  "1"  /* Bypass Duodenum to Transverse Colon, Percutaneous Endoscopic Approach */
"0D19874"  =  "1"  /* Bypass Duodenum to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D19879"  =  "1"  /* Bypass Duodenum to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1987A"  =  "1"  /* Bypass Duodenum to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1987B"  =  "1"  /* Bypass Duodenum to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1987L"  =  "1"  /* Bypass Duodenum to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198J4"  =  "1"  /* Bypass Duodenum to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198J9"  =  "1"  /* Bypass Duodenum to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198JA"  =  "1"  /* Bypass Duodenum to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198JB"  =  "1"  /* Bypass Duodenum to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198JL"  =  "1"  /* Bypass Duodenum to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198K4"  =  "1"  /* Bypass Duodenum to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198K9"  =  "1"  /* Bypass Duodenum to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198KA"  =  "1"  /* Bypass Duodenum to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198KB"  =  "1"  /* Bypass Duodenum to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198KL"  =  "1"  /* Bypass Duodenum to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198Z4"  =  "1"  /* Bypass Duodenum to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D198Z9"  =  "1"  /* Bypass Duodenum to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D198ZA"  =  "1"  /* Bypass Duodenum to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D198ZB"  =  "1"  /* Bypass Duodenum to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D198ZL"  =  "1"  /* Bypass Duodenum to Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D1A074"  =  "1"  /* Bypass Jejunum to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D1A07A"  =  "1"  /* Bypass Jejunum to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1A07B"  =  "1"  /* Bypass Jejunum to Ileum with Autologous Tissue Substitute, Open Approach */
"0D1A07H"  =  "1"  /* Bypass Jejunum to Cecum with Autologous Tissue Substitute, Open Approach */
"0D1A07K"  =  "1"  /* Bypass Jejunum to Ascending Colon with Autologous Tissue Substitute, Open Approach */
"0D1A07L"  =  "1"  /* Bypass Jejunum to Transverse Colon with Autologous Tissue Substitute, Open Approach */
"0D1A07M"  =  "1"  /* Bypass Jejunum to Descending Colon with Autologous Tissue Substitute, Open Approach */
"0D1A07N"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Autologous Tissue Substitute, Open Approach */
"0D1A07P"  =  "1"  /* Bypass Jejunum to Rectum with Autologous Tissue Substitute, Open Approach */
"0D1A07Q"  =  "1"  /* Bypass Jejunum to Anus with Autologous Tissue Substitute, Open Approach */
"0D1A0J4"  =  "1"  /* Bypass Jejunum to Cutaneous with Synthetic Substitute, Open Approach */
"0D1A0JA"  =  "1"  /* Bypass Jejunum to Jejunum with Synthetic Substitute, Open Approach */
"0D1A0JB"  =  "1"  /* Bypass Jejunum to Ileum with Synthetic Substitute, Open Approach */
"0D1A0JH"  =  "1"  /* Bypass Jejunum to Cecum with Synthetic Substitute, Open Approach */
"0D1A0JK"  =  "1"  /* Bypass Jejunum to Ascending Colon with Synthetic Substitute, Open Approach */
"0D1A0JL"  =  "1"  /* Bypass Jejunum to Transverse Colon with Synthetic Substitute, Open Approach */
"0D1A0JM"  =  "1"  /* Bypass Jejunum to Descending Colon with Synthetic Substitute, Open Approach */
"0D1A0JN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Synthetic Substitute, Open Approach */
"0D1A0JP"  =  "1"  /* Bypass Jejunum to Rectum with Synthetic Substitute, Open Approach */
"0D1A0JQ"  =  "1"  /* Bypass Jejunum to Anus with Synthetic Substitute, Open Approach */
"0D1A0K4"  =  "1"  /* Bypass Jejunum to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KA"  =  "1"  /* Bypass Jejunum to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KB"  =  "1"  /* Bypass Jejunum to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KH"  =  "1"  /* Bypass Jejunum to Cecum with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KK"  =  "1"  /* Bypass Jejunum to Ascending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KL"  =  "1"  /* Bypass Jejunum to Transverse Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KM"  =  "1"  /* Bypass Jejunum to Descending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KP"  =  "1"  /* Bypass Jejunum to Rectum with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KQ"  =  "1"  /* Bypass Jejunum to Anus with Nonautologous Tissue Substitute, Open Approach */
"0D1A0Z4"  =  "1"  /* Bypass Jejunum to Cutaneous, Open Approach */
"0D1A0ZA"  =  "1"  /* Bypass Jejunum to Jejunum, Open Approach */
"0D1A0ZB"  =  "1"  /* Bypass Jejunum to Ileum, Open Approach */
"0D1A0ZH"  =  "1"  /* Bypass Jejunum to Cecum, Open Approach */
"0D1A0ZK"  =  "1"  /* Bypass Jejunum to Ascending Colon, Open Approach */
"0D1A0ZL"  =  "1"  /* Bypass Jejunum to Transverse Colon, Open Approach */
"0D1A0ZM"  =  "1"  /* Bypass Jejunum to Descending Colon, Open Approach */
"0D1A0ZN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon, Open Approach */
"0D1A0ZP"  =  "1"  /* Bypass Jejunum to Rectum, Open Approach */
"0D1A0ZQ"  =  "1"  /* Bypass Jejunum to Anus, Open Approach */
"0D1A3J4"  =  "1"  /* Bypass Jejunum to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D1A474"  =  "1"  /* Bypass Jejunum to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47A"  =  "1"  /* Bypass Jejunum to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47B"  =  "1"  /* Bypass Jejunum to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47H"  =  "1"  /* Bypass Jejunum to Cecum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47K"  =  "1"  /* Bypass Jejunum to Ascending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47L"  =  "1"  /* Bypass Jejunum to Transverse Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47M"  =  "1"  /* Bypass Jejunum to Descending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47N"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47P"  =  "1"  /* Bypass Jejunum to Rectum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47Q"  =  "1"  /* Bypass Jejunum to Anus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4J4"  =  "1"  /* Bypass Jejunum to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JA"  =  "1"  /* Bypass Jejunum to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JB"  =  "1"  /* Bypass Jejunum to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JH"  =  "1"  /* Bypass Jejunum to Cecum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JK"  =  "1"  /* Bypass Jejunum to Ascending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JL"  =  "1"  /* Bypass Jejunum to Transverse Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JM"  =  "1"  /* Bypass Jejunum to Descending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JP"  =  "1"  /* Bypass Jejunum to Rectum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JQ"  =  "1"  /* Bypass Jejunum to Anus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4K4"  =  "1"  /* Bypass Jejunum to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KA"  =  "1"  /* Bypass Jejunum to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KB"  =  "1"  /* Bypass Jejunum to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KH"  =  "1"  /* Bypass Jejunum to Cecum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KK"  =  "1"  /* Bypass Jejunum to Ascending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KL"  =  "1"  /* Bypass Jejunum to Transverse Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KM"  =  "1"  /* Bypass Jejunum to Descending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KP"  =  "1"  /* Bypass Jejunum to Rectum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KQ"  =  "1"  /* Bypass Jejunum to Anus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4Z4"  =  "1"  /* Bypass Jejunum to Cutaneous, Percutaneous Endoscopic Approach */
"0D1A4ZA"  =  "1"  /* Bypass Jejunum to Jejunum, Percutaneous Endoscopic Approach */
"0D1A4ZB"  =  "1"  /* Bypass Jejunum to Ileum, Percutaneous Endoscopic Approach */
"0D1A4ZH"  =  "1"  /* Bypass Jejunum to Cecum, Percutaneous Endoscopic Approach */
"0D1A4ZK"  =  "1"  /* Bypass Jejunum to Ascending Colon, Percutaneous Endoscopic Approach */
"0D1A4ZL"  =  "1"  /* Bypass Jejunum to Transverse Colon, Percutaneous Endoscopic Approach */
"0D1A4ZM"  =  "1"  /* Bypass Jejunum to Descending Colon, Percutaneous Endoscopic Approach */
"0D1A4ZN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D1A4ZP"  =  "1"  /* Bypass Jejunum to Rectum, Percutaneous Endoscopic Approach */
"0D1A4ZQ"  =  "1"  /* Bypass Jejunum to Anus, Percutaneous Endoscopic Approach */
"0D1A874"  =  "1"  /* Bypass Jejunum to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87A"  =  "1"  /* Bypass Jejunum to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87B"  =  "1"  /* Bypass Jejunum to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87H"  =  "1"  /* Bypass Jejunum to Cecum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87K"  =  "1"  /* Bypass Jejunum to Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87L"  =  "1"  /* Bypass Jejunum to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87M"  =  "1"  /* Bypass Jejunum to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87N"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87P"  =  "1"  /* Bypass Jejunum to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87Q"  =  "1"  /* Bypass Jejunum to Anus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8J4"  =  "1"  /* Bypass Jejunum to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JA"  =  "1"  /* Bypass Jejunum to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JB"  =  "1"  /* Bypass Jejunum to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JH"  =  "1"  /* Bypass Jejunum to Cecum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JK"  =  "1"  /* Bypass Jejunum to Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JL"  =  "1"  /* Bypass Jejunum to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JM"  =  "1"  /* Bypass Jejunum to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JP"  =  "1"  /* Bypass Jejunum to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JQ"  =  "1"  /* Bypass Jejunum to Anus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8K4"  =  "1"  /* Bypass Jejunum to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KA"  =  "1"  /* Bypass Jejunum to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KB"  =  "1"  /* Bypass Jejunum to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KH"  =  "1"  /* Bypass Jejunum to Cecum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KK"  =  "1"  /* Bypass Jejunum to Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KL"  =  "1"  /* Bypass Jejunum to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KM"  =  "1"  /* Bypass Jejunum to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KP"  =  "1"  /* Bypass Jejunum to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KQ"  =  "1"  /* Bypass Jejunum to Anus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8Z4"  =  "1"  /* Bypass Jejunum to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZA"  =  "1"  /* Bypass Jejunum to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZB"  =  "1"  /* Bypass Jejunum to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZH"  =  "1"  /* Bypass Jejunum to Cecum, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZK"  =  "1"  /* Bypass Jejunum to Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZL"  =  "1"  /* Bypass Jejunum to Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZM"  =  "1"  /* Bypass Jejunum to Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZP"  =  "1"  /* Bypass Jejunum to Rectum, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZQ"  =  "1"  /* Bypass Jejunum to Anus, Via Natural or Artificial Opening Endoscopic */
"0D1B074"  =  "1"  /* Bypass Ileum to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D1B07B"  =  "1"  /* Bypass Ileum to Ileum with Autologous Tissue Substitute, Open Approach */
"0D1B07H"  =  "1"  /* Bypass Ileum to Cecum with Autologous Tissue Substitute, Open Approach */
"0D1B07K"  =  "1"  /* Bypass Ileum to Ascending Colon with Autologous Tissue Substitute, Open Approach */
"0D1B07L"  =  "1"  /* Bypass Ileum to Transverse Colon with Autologous Tissue Substitute, Open Approach */
"0D1B07M"  =  "1"  /* Bypass Ileum to Descending Colon with Autologous Tissue Substitute, Open Approach */
"0D1B07N"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Autologous Tissue Substitute, Open Approach */
"0D1B07P"  =  "1"  /* Bypass Ileum to Rectum with Autologous Tissue Substitute, Open Approach */
"0D1B07Q"  =  "1"  /* Bypass Ileum to Anus with Autologous Tissue Substitute, Open Approach */
"0D1B0J4"  =  "1"  /* Bypass Ileum to Cutaneous with Synthetic Substitute, Open Approach */
"0D1B0JB"  =  "1"  /* Bypass Ileum to Ileum with Synthetic Substitute, Open Approach */
"0D1B0JH"  =  "1"  /* Bypass Ileum to Cecum with Synthetic Substitute, Open Approach */
"0D1B0JK"  =  "1"  /* Bypass Ileum to Ascending Colon with Synthetic Substitute, Open Approach */
"0D1B0JL"  =  "1"  /* Bypass Ileum to Transverse Colon with Synthetic Substitute, Open Approach */
"0D1B0JM"  =  "1"  /* Bypass Ileum to Descending Colon with Synthetic Substitute, Open Approach */
"0D1B0JN"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Synthetic Substitute, Open Approach */
"0D1B0JP"  =  "1"  /* Bypass Ileum to Rectum with Synthetic Substitute, Open Approach */
"0D1B0JQ"  =  "1"  /* Bypass Ileum to Anus with Synthetic Substitute, Open Approach */
"0D1B0K4"  =  "1"  /* Bypass Ileum to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D1B0KB"  =  "1"  /* Bypass Ileum to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D1B0KH"  =  "1"  /* Bypass Ileum to Cecum with Nonautologous Tissue Substitute, Open Approach */
"0D1B0KK"  =  "1"  /* Bypass Ileum to Ascending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1B0KL"  =  "1"  /* Bypass Ileum to Transverse Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1B0KM"  =  "1"  /* Bypass Ileum to Descending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1B0KN"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1B0KP"  =  "1"  /* Bypass Ileum to Rectum with Nonautologous Tissue Substitute, Open Approach */
"0D1B0KQ"  =  "1"  /* Bypass Ileum to Anus with Nonautologous Tissue Substitute, Open Approach */
"0D1B0Z4"  =  "1"  /* Bypass Ileum to Cutaneous, Open Approach */
"0D1B0ZB"  =  "1"  /* Bypass Ileum to Ileum, Open Approach */
"0D1B0ZH"  =  "1"  /* Bypass Ileum to Cecum, Open Approach */
"0D1B0ZK"  =  "1"  /* Bypass Ileum to Ascending Colon, Open Approach */
"0D1B0ZL"  =  "1"  /* Bypass Ileum to Transverse Colon, Open Approach */
"0D1B0ZM"  =  "1"  /* Bypass Ileum to Descending Colon, Open Approach */
"0D1B0ZN"  =  "1"  /* Bypass Ileum to Sigmoid Colon, Open Approach */
"0D1B0ZP"  =  "1"  /* Bypass Ileum to Rectum, Open Approach */
"0D1B0ZQ"  =  "1"  /* Bypass Ileum to Anus, Open Approach */
"0D1B3J4"  =  "1"  /* Bypass Ileum to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D1B474"  =  "1"  /* Bypass Ileum to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B47B"  =  "1"  /* Bypass Ileum to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B47H"  =  "1"  /* Bypass Ileum to Cecum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B47K"  =  "1"  /* Bypass Ileum to Ascending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B47L"  =  "1"  /* Bypass Ileum to Transverse Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B47M"  =  "1"  /* Bypass Ileum to Descending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B47N"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B47P"  =  "1"  /* Bypass Ileum to Rectum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B47Q"  =  "1"  /* Bypass Ileum to Anus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4J4"  =  "1"  /* Bypass Ileum to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4JB"  =  "1"  /* Bypass Ileum to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4JH"  =  "1"  /* Bypass Ileum to Cecum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4JK"  =  "1"  /* Bypass Ileum to Ascending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4JL"  =  "1"  /* Bypass Ileum to Transverse Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4JM"  =  "1"  /* Bypass Ileum to Descending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4JN"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4JP"  =  "1"  /* Bypass Ileum to Rectum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4JQ"  =  "1"  /* Bypass Ileum to Anus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4K4"  =  "1"  /* Bypass Ileum to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4KB"  =  "1"  /* Bypass Ileum to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4KH"  =  "1"  /* Bypass Ileum to Cecum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4KK"  =  "1"  /* Bypass Ileum to Ascending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4KL"  =  "1"  /* Bypass Ileum to Transverse Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4KM"  =  "1"  /* Bypass Ileum to Descending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4KN"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4KP"  =  "1"  /* Bypass Ileum to Rectum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4KQ"  =  "1"  /* Bypass Ileum to Anus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4Z4"  =  "1"  /* Bypass Ileum to Cutaneous, Percutaneous Endoscopic Approach */
"0D1B4ZB"  =  "1"  /* Bypass Ileum to Ileum, Percutaneous Endoscopic Approach */
"0D1B4ZH"  =  "1"  /* Bypass Ileum to Cecum, Percutaneous Endoscopic Approach */
"0D1B4ZK"  =  "1"  /* Bypass Ileum to Ascending Colon, Percutaneous Endoscopic Approach */
"0D1B4ZL"  =  "1"  /* Bypass Ileum to Transverse Colon, Percutaneous Endoscopic Approach */
"0D1B4ZM"  =  "1"  /* Bypass Ileum to Descending Colon, Percutaneous Endoscopic Approach */
"0D1B4ZN"  =  "1"  /* Bypass Ileum to Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D1B4ZP"  =  "1"  /* Bypass Ileum to Rectum, Percutaneous Endoscopic Approach */
"0D1B4ZQ"  =  "1"  /* Bypass Ileum to Anus, Percutaneous Endoscopic Approach */
"0D1B874"  =  "1"  /* Bypass Ileum to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B87B"  =  "1"  /* Bypass Ileum to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B87H"  =  "1"  /* Bypass Ileum to Cecum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B87K"  =  "1"  /* Bypass Ileum to Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B87L"  =  "1"  /* Bypass Ileum to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B87M"  =  "1"  /* Bypass Ileum to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B87N"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B87P"  =  "1"  /* Bypass Ileum to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B87Q"  =  "1"  /* Bypass Ileum to Anus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8J4"  =  "1"  /* Bypass Ileum to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8JB"  =  "1"  /* Bypass Ileum to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8JH"  =  "1"  /* Bypass Ileum to Cecum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8JK"  =  "1"  /* Bypass Ileum to Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8JL"  =  "1"  /* Bypass Ileum to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8JM"  =  "1"  /* Bypass Ileum to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8JN"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8JP"  =  "1"  /* Bypass Ileum to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8JQ"  =  "1"  /* Bypass Ileum to Anus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8K4"  =  "1"  /* Bypass Ileum to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8KB"  =  "1"  /* Bypass Ileum to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8KH"  =  "1"  /* Bypass Ileum to Cecum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8KK"  =  "1"  /* Bypass Ileum to Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8KL"  =  "1"  /* Bypass Ileum to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8KM"  =  "1"  /* Bypass Ileum to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8KN"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8KP"  =  "1"  /* Bypass Ileum to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8KQ"  =  "1"  /* Bypass Ileum to Anus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8Z4"  =  "1"  /* Bypass Ileum to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D1B8ZB"  =  "1"  /* Bypass Ileum to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D1B8ZH"  =  "1"  /* Bypass Ileum to Cecum, Via Natural or Artificial Opening Endoscopic */
"0D1B8ZK"  =  "1"  /* Bypass Ileum to Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1B8ZL"  =  "1"  /* Bypass Ileum to Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D1B8ZM"  =  "1"  /* Bypass Ileum to Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1B8ZN"  =  "1"  /* Bypass Ileum to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D1B8ZP"  =  "1"  /* Bypass Ileum to Rectum, Via Natural or Artificial Opening Endoscopic */
"0D1B8ZQ"  =  "1"  /* Bypass Ileum to Anus, Via Natural or Artificial Opening Endoscopic */
"0D1H074"  =  "1"  /* Bypass Cecum to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D1H07H"  =  "1"  /* Bypass Cecum to Cecum with Autologous Tissue Substitute, Open Approach */
"0D1H07K"  =  "1"  /* Bypass Cecum to Ascending Colon with Autologous Tissue Substitute, Open Approach */
"0D1H07L"  =  "1"  /* Bypass Cecum to Transverse Colon with Autologous Tissue Substitute, Open Approach */
"0D1H07M"  =  "1"  /* Bypass Cecum to Descending Colon with Autologous Tissue Substitute, Open Approach */
"0D1H07N"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Autologous Tissue Substitute, Open Approach */
"0D1H07P"  =  "1"  /* Bypass Cecum to Rectum with Autologous Tissue Substitute, Open Approach */
"0D1H0J4"  =  "1"  /* Bypass Cecum to Cutaneous with Synthetic Substitute, Open Approach */
"0D1H0JH"  =  "1"  /* Bypass Cecum to Cecum with Synthetic Substitute, Open Approach */
"0D1H0JK"  =  "1"  /* Bypass Cecum to Ascending Colon with Synthetic Substitute, Open Approach */
"0D1H0JL"  =  "1"  /* Bypass Cecum to Transverse Colon with Synthetic Substitute, Open Approach */
"0D1H0JM"  =  "1"  /* Bypass Cecum to Descending Colon with Synthetic Substitute, Open Approach */
"0D1H0JN"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Synthetic Substitute, Open Approach */
"0D1H0JP"  =  "1"  /* Bypass Cecum to Rectum with Synthetic Substitute, Open Approach */
"0D1H0K4"  =  "1"  /* Bypass Cecum to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D1H0KH"  =  "1"  /* Bypass Cecum to Cecum with Nonautologous Tissue Substitute, Open Approach */
"0D1H0KK"  =  "1"  /* Bypass Cecum to Ascending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1H0KL"  =  "1"  /* Bypass Cecum to Transverse Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1H0KM"  =  "1"  /* Bypass Cecum to Descending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1H0KN"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1H0KP"  =  "1"  /* Bypass Cecum to Rectum with Nonautologous Tissue Substitute, Open Approach */
"0D1H0Z4"  =  "1"  /* Bypass Cecum to Cutaneous, Open Approach */
"0D1H0ZH"  =  "1"  /* Bypass Cecum to Cecum, Open Approach */
"0D1H0ZK"  =  "1"  /* Bypass Cecum to Ascending Colon, Open Approach */
"0D1H0ZL"  =  "1"  /* Bypass Cecum to Transverse Colon, Open Approach */
"0D1H0ZM"  =  "1"  /* Bypass Cecum to Descending Colon, Open Approach */
"0D1H0ZN"  =  "1"  /* Bypass Cecum to Sigmoid Colon, Open Approach */
"0D1H0ZP"  =  "1"  /* Bypass Cecum to Rectum, Open Approach */
"0D1H3J4"  =  "1"  /* Bypass Cecum to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D1H474"  =  "1"  /* Bypass Cecum to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H47H"  =  "1"  /* Bypass Cecum to Cecum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H47K"  =  "1"  /* Bypass Cecum to Ascending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H47L"  =  "1"  /* Bypass Cecum to Transverse Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H47M"  =  "1"  /* Bypass Cecum to Descending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H47N"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H47P"  =  "1"  /* Bypass Cecum to Rectum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H4J4"  =  "1"  /* Bypass Cecum to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1H4JH"  =  "1"  /* Bypass Cecum to Cecum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1H4JK"  =  "1"  /* Bypass Cecum to Ascending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1H4JL"  =  "1"  /* Bypass Cecum to Transverse Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1H4JM"  =  "1"  /* Bypass Cecum to Descending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1H4JN"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1H4JP"  =  "1"  /* Bypass Cecum to Rectum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1H4K4"  =  "1"  /* Bypass Cecum to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H4KH"  =  "1"  /* Bypass Cecum to Cecum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H4KK"  =  "1"  /* Bypass Cecum to Ascending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H4KL"  =  "1"  /* Bypass Cecum to Transverse Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H4KM"  =  "1"  /* Bypass Cecum to Descending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H4KN"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H4KP"  =  "1"  /* Bypass Cecum to Rectum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H4Z4"  =  "1"  /* Bypass Cecum to Cutaneous, Percutaneous Endoscopic Approach */
"0D1H4ZH"  =  "1"  /* Bypass Cecum to Cecum, Percutaneous Endoscopic Approach */
"0D1H4ZK"  =  "1"  /* Bypass Cecum to Ascending Colon, Percutaneous Endoscopic Approach */
"0D1H4ZL"  =  "1"  /* Bypass Cecum to Transverse Colon, Percutaneous Endoscopic Approach */
"0D1H4ZM"  =  "1"  /* Bypass Cecum to Descending Colon, Percutaneous Endoscopic Approach */
"0D1H4ZN"  =  "1"  /* Bypass Cecum to Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D1H4ZP"  =  "1"  /* Bypass Cecum to Rectum, Percutaneous Endoscopic Approach */
"0D1H874"  =  "1"  /* Bypass Cecum to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H87H"  =  "1"  /* Bypass Cecum to Cecum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H87K"  =  "1"  /* Bypass Cecum to Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H87L"  =  "1"  /* Bypass Cecum to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H87M"  =  "1"  /* Bypass Cecum to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H87N"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H87P"  =  "1"  /* Bypass Cecum to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8J4"  =  "1"  /* Bypass Cecum to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8JH"  =  "1"  /* Bypass Cecum to Cecum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8JK"  =  "1"  /* Bypass Cecum to Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8JL"  =  "1"  /* Bypass Cecum to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8JM"  =  "1"  /* Bypass Cecum to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8JN"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8JP"  =  "1"  /* Bypass Cecum to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8K4"  =  "1"  /* Bypass Cecum to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8KH"  =  "1"  /* Bypass Cecum to Cecum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8KK"  =  "1"  /* Bypass Cecum to Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8KL"  =  "1"  /* Bypass Cecum to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8KM"  =  "1"  /* Bypass Cecum to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8KN"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8KP"  =  "1"  /* Bypass Cecum to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8Z4"  =  "1"  /* Bypass Cecum to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D1H8ZH"  =  "1"  /* Bypass Cecum to Cecum, Via Natural or Artificial Opening Endoscopic */
"0D1H8ZK"  =  "1"  /* Bypass Cecum to Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1H8ZL"  =  "1"  /* Bypass Cecum to Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D1H8ZM"  =  "1"  /* Bypass Cecum to Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1H8ZN"  =  "1"  /* Bypass Cecum to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D1H8ZP"  =  "1"  /* Bypass Cecum to Rectum, Via Natural or Artificial Opening Endoscopic */
"0D1K074"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D1K07K"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Autologous Tissue Substitute, Open Approach */
"0D1K07L"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Autologous Tissue Substitute, Open Approach */
"0D1K07M"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Autologous Tissue Substitute, Open Approach */
"0D1K07N"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Autologous Tissue Substitute, Open Approach */
"0D1K07P"  =  "1"  /* Bypass Ascending Colon to Rectum with Autologous Tissue Substitute, Open Approach */
"0D1K0J4"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Synthetic Substitute, Open Approach */
"0D1K0JK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Synthetic Substitute, Open Approach */
"0D1K0JL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Synthetic Substitute, Open Approach */
"0D1K0JM"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Synthetic Substitute, Open Approach */
"0D1K0JN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Synthetic Substitute, Open Approach */
"0D1K0JP"  =  "1"  /* Bypass Ascending Colon to Rectum with Synthetic Substitute, Open Approach */
"0D1K0K4"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D1K0KK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1K0KL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1K0KM"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1K0KN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1K0KP"  =  "1"  /* Bypass Ascending Colon to Rectum with Nonautologous Tissue Substitute, Open Approach */
"0D1K0Z4"  =  "1"  /* Bypass Ascending Colon to Cutaneous, Open Approach */
"0D1K0ZK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon, Open Approach */
"0D1K0ZL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon, Open Approach */
"0D1K0ZM"  =  "1"  /* Bypass Ascending Colon to Descending Colon, Open Approach */
"0D1K0ZN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon, Open Approach */
"0D1K0ZP"  =  "1"  /* Bypass Ascending Colon to Rectum, Open Approach */
"0D1K3J4"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D1K474"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K47K"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K47L"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K47M"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K47N"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K47P"  =  "1"  /* Bypass Ascending Colon to Rectum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K4J4"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1K4JK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1K4JL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1K4JM"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1K4JN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1K4JP"  =  "1"  /* Bypass Ascending Colon to Rectum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1K4K4"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K4KK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K4KL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K4KM"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K4KN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K4KP"  =  "1"  /* Bypass Ascending Colon to Rectum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K4Z4"  =  "1"  /* Bypass Ascending Colon to Cutaneous, Percutaneous Endoscopic Approach */
"0D1K4ZK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon, Percutaneous Endoscopic Approach */
"0D1K4ZL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon, Percutaneous Endoscopic Approach */
"0D1K4ZM"  =  "1"  /* Bypass Ascending Colon to Descending Colon, Percutaneous Endoscopic Approach */
"0D1K4ZN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D1K4ZP"  =  "1"  /* Bypass Ascending Colon to Rectum, Percutaneous Endoscopic Approach */
"0D1K874"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K87K"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K87L"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K87M"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K87N"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K87P"  =  "1"  /* Bypass Ascending Colon to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8J4"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8JK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8JL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8JM"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8JN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8JP"  =  "1"  /* Bypass Ascending Colon to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8K4"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8KK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8KL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8KM"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8KN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8KP"  =  "1"  /* Bypass Ascending Colon to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8Z4"  =  "1"  /* Bypass Ascending Colon to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D1K8ZK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1K8ZL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D1K8ZM"  =  "1"  /* Bypass Ascending Colon to Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1K8ZN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D1K8ZP"  =  "1"  /* Bypass Ascending Colon to Rectum, Via Natural or Artificial Opening Endoscopic */
"0D1L074"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D1L07L"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Autologous Tissue Substitute, Open Approach */
"0D1L07M"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Autologous Tissue Substitute, Open Approach */
"0D1L07N"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Autologous Tissue Substitute, Open Approach */
"0D1L07P"  =  "1"  /* Bypass Transverse Colon to Rectum with Autologous Tissue Substitute, Open Approach */
"0D1L0J4"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Synthetic Substitute, Open Approach */
"0D1L0JL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Synthetic Substitute, Open Approach */
"0D1L0JM"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Synthetic Substitute, Open Approach */
"0D1L0JN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Synthetic Substitute, Open Approach */
"0D1L0JP"  =  "1"  /* Bypass Transverse Colon to Rectum with Synthetic Substitute, Open Approach */
"0D1L0K4"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D1L0KL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1L0KM"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1L0KN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1L0KP"  =  "1"  /* Bypass Transverse Colon to Rectum with Nonautologous Tissue Substitute, Open Approach */
"0D1L0Z4"  =  "1"  /* Bypass Transverse Colon to Cutaneous, Open Approach */
"0D1L0ZL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon, Open Approach */
"0D1L0ZM"  =  "1"  /* Bypass Transverse Colon to Descending Colon, Open Approach */
"0D1L0ZN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon, Open Approach */
"0D1L0ZP"  =  "1"  /* Bypass Transverse Colon to Rectum, Open Approach */
"0D1L3J4"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D1L474"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L47L"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L47M"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L47N"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L47P"  =  "1"  /* Bypass Transverse Colon to Rectum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L4J4"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1L4JL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1L4JM"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1L4JN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1L4JP"  =  "1"  /* Bypass Transverse Colon to Rectum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1L4K4"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L4KL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L4KM"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L4KN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L4KP"  =  "1"  /* Bypass Transverse Colon to Rectum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L4Z4"  =  "1"  /* Bypass Transverse Colon to Cutaneous, Percutaneous Endoscopic Approach */
"0D1L4ZL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon, Percutaneous Endoscopic Approach */
"0D1L4ZM"  =  "1"  /* Bypass Transverse Colon to Descending Colon, Percutaneous Endoscopic Approach */
"0D1L4ZN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D1L4ZP"  =  "1"  /* Bypass Transverse Colon to Rectum, Percutaneous Endoscopic Approach */
"0D1L874"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L87L"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L87M"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L87N"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L87P"  =  "1"  /* Bypass Transverse Colon to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8J4"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8JL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8JM"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8JN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8JP"  =  "1"  /* Bypass Transverse Colon to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8K4"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8KL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8KM"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8KN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8KP"  =  "1"  /* Bypass Transverse Colon to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8Z4"  =  "1"  /* Bypass Transverse Colon to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D1L8ZL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D1L8ZM"  =  "1"  /* Bypass Transverse Colon to Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1L8ZN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D1L8ZP"  =  "1"  /* Bypass Transverse Colon to Rectum, Via Natural or Artificial Opening Endoscopic */
"0D1M074"  =  "1"  /* Bypass Descending Colon to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D1M07M"  =  "1"  /* Bypass Descending Colon to Descending Colon with Autologous Tissue Substitute, Open Approach */
"0D1M07N"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Autologous Tissue Substitute, Open Approach */
"0D1M07P"  =  "1"  /* Bypass Descending Colon to Rectum with Autologous Tissue Substitute, Open Approach */
"0D1M0J4"  =  "1"  /* Bypass Descending Colon to Cutaneous with Synthetic Substitute, Open Approach */
"0D1M0JM"  =  "1"  /* Bypass Descending Colon to Descending Colon with Synthetic Substitute, Open Approach */
"0D1M0JN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Synthetic Substitute, Open Approach */
"0D1M0JP"  =  "1"  /* Bypass Descending Colon to Rectum with Synthetic Substitute, Open Approach */
"0D1M0K4"  =  "1"  /* Bypass Descending Colon to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D1M0KM"  =  "1"  /* Bypass Descending Colon to Descending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1M0KN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1M0KP"  =  "1"  /* Bypass Descending Colon to Rectum with Nonautologous Tissue Substitute, Open Approach */
"0D1M0Z4"  =  "1"  /* Bypass Descending Colon to Cutaneous, Open Approach */
"0D1M0ZM"  =  "1"  /* Bypass Descending Colon to Descending Colon, Open Approach */
"0D1M0ZN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon, Open Approach */
"0D1M0ZP"  =  "1"  /* Bypass Descending Colon to Rectum, Open Approach */
"0D1M3J4"  =  "1"  /* Bypass Descending Colon to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D1M474"  =  "1"  /* Bypass Descending Colon to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1M47M"  =  "1"  /* Bypass Descending Colon to Descending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1M47N"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1M47P"  =  "1"  /* Bypass Descending Colon to Rectum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1M4J4"  =  "1"  /* Bypass Descending Colon to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1M4JM"  =  "1"  /* Bypass Descending Colon to Descending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1M4JN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1M4JP"  =  "1"  /* Bypass Descending Colon to Rectum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1M4K4"  =  "1"  /* Bypass Descending Colon to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1M4KM"  =  "1"  /* Bypass Descending Colon to Descending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1M4KN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1M4KP"  =  "1"  /* Bypass Descending Colon to Rectum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1M4Z4"  =  "1"  /* Bypass Descending Colon to Cutaneous, Percutaneous Endoscopic Approach */
"0D1M4ZM"  =  "1"  /* Bypass Descending Colon to Descending Colon, Percutaneous Endoscopic Approach */
"0D1M4ZN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D1M4ZP"  =  "1"  /* Bypass Descending Colon to Rectum, Percutaneous Endoscopic Approach */
"0D1M874"  =  "1"  /* Bypass Descending Colon to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M87M"  =  "1"  /* Bypass Descending Colon to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M87N"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M87P"  =  "1"  /* Bypass Descending Colon to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8J4"  =  "1"  /* Bypass Descending Colon to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8JM"  =  "1"  /* Bypass Descending Colon to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8JN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8JP"  =  "1"  /* Bypass Descending Colon to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8K4"  =  "1"  /* Bypass Descending Colon to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8KM"  =  "1"  /* Bypass Descending Colon to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8KN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8KP"  =  "1"  /* Bypass Descending Colon to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8Z4"  =  "1"  /* Bypass Descending Colon to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D1M8ZM"  =  "1"  /* Bypass Descending Colon to Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1M8ZN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D1M8ZP"  =  "1"  /* Bypass Descending Colon to Rectum, Via Natural or Artificial Opening Endoscopic */
"0D1N074"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D1N07N"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Autologous Tissue Substitute, Open Approach */
"0D1N07P"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Autologous Tissue Substitute, Open Approach */
"0D1N0J4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Synthetic Substitute, Open Approach */
"0D1N0JN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Synthetic Substitute, Open Approach */
"0D1N0JP"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Synthetic Substitute, Open Approach */
"0D1N0K4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D1N0KN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1N0KP"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Nonautologous Tissue Substitute, Open Approach */
"0D1N0Z4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous, Open Approach */
"0D1N0ZN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon, Open Approach */
"0D1N0ZP"  =  "1"  /* Bypass Sigmoid Colon to Rectum, Open Approach */
"0D1N3J4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D1N474"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1N47N"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1N47P"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1N4J4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1N4JN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1N4JP"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1N4K4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1N4KN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1N4KP"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1N4Z4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous, Percutaneous Endoscopic Approach */
"0D1N4ZN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D1N4ZP"  =  "1"  /* Bypass Sigmoid Colon to Rectum, Percutaneous Endoscopic Approach */
"0D1N874"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N87N"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N87P"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N8J4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N8JN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N8JP"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N8K4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N8KN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N8KP"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N8Z4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D1N8ZN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D1N8ZP"  =  "1"  /* Bypass Sigmoid Colon to Rectum, Via Natural or Artificial Opening Endoscopic */
"0D560ZZ"  =  "1"  /* Destruction of Stomach, Open Approach */
"0D563ZZ"  =  "1"  /* Destruction of Stomach, Percutaneous Approach */
"0D567ZZ"  =  "1"  /* Destruction of Stomach, Via Natural or Artificial Opening */
"0D570ZZ"  =  "1"  /* Destruction of Stomach, Pylorus, Open Approach */
"0D573ZZ"  =  "1"  /* Destruction of Stomach, Pylorus, Percutaneous Approach */
"0D577ZZ"  =  "1"  /* Destruction of Stomach, Pylorus, Via Natural or Artificial Opening */
"0D580ZZ"  =  "1"  /* Destruction of Small Intestine, Open Approach */
"0D583ZZ"  =  "1"  /* Destruction of Small Intestine, Percutaneous Approach */
"0D584ZZ"  =  "1"  /* Destruction of Small Intestine, Percutaneous Endoscopic Approach */
"0D587ZZ"  =  "1"  /* Destruction of Small Intestine, Via Natural or Artificial Opening */
"0D588ZZ"  =  "1"  /* Destruction of Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0D590ZZ"  =  "1"  /* Destruction of Duodenum, Open Approach */
"0D593ZZ"  =  "1"  /* Destruction of Duodenum, Percutaneous Approach */
"0D597ZZ"  =  "1"  /* Destruction of Duodenum, Via Natural or Artificial Opening */
"0D5A0ZZ"  =  "1"  /* Destruction of Jejunum, Open Approach */
"0D5A3ZZ"  =  "1"  /* Destruction of Jejunum, Percutaneous Approach */
"0D5A4ZZ"  =  "1"  /* Destruction of Jejunum, Percutaneous Endoscopic Approach */
"0D5A7ZZ"  =  "1"  /* Destruction of Jejunum, Via Natural or Artificial Opening */
"0D5A8ZZ"  =  "1"  /* Destruction of Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D5B0ZZ"  =  "1"  /* Destruction of Ileum, Open Approach */
"0D5B3ZZ"  =  "1"  /* Destruction of Ileum, Percutaneous Approach */
"0D5B4ZZ"  =  "1"  /* Destruction of Ileum, Percutaneous Endoscopic Approach */
"0D5B7ZZ"  =  "1"  /* Destruction of Ileum, Via Natural or Artificial Opening */
"0D5B8ZZ"  =  "1"  /* Destruction of Ileum, Via Natural or Artificial Opening Endoscopic */
"0D5C0ZZ"  =  "1"  /* Destruction of Ileocecal Valve, Open Approach */
"0D5C3ZZ"  =  "1"  /* Destruction of Ileocecal Valve, Percutaneous Approach */
"0D5C4ZZ"  =  "1"  /* Destruction of Ileocecal Valve, Percutaneous Endoscopic Approach */
"0D5C7ZZ"  =  "1"  /* Destruction of Ileocecal Valve, Via Natural or Artificial Opening */
"0D5C8ZZ"  =  "1"  /* Destruction of Ileocecal Valve, Via Natural or Artificial Opening Endoscopic */
"0D5E0ZZ"  =  "1"  /* Destruction of Large Intestine, Open Approach */
"0D5E3ZZ"  =  "1"  /* Destruction of Large Intestine, Percutaneous Approach */
"0D5E7ZZ"  =  "1"  /* Destruction of Large Intestine, Via Natural or Artificial Opening */
"0D5F0ZZ"  =  "1"  /* Destruction of Right Large Intestine, Open Approach */
"0D5F3ZZ"  =  "1"  /* Destruction of Right Large Intestine, Percutaneous Approach */
"0D5F7ZZ"  =  "1"  /* Destruction of Right Large Intestine, Via Natural or Artificial Opening */
"0D5G0ZZ"  =  "1"  /* Destruction of Left Large Intestine, Open Approach */
"0D5G3ZZ"  =  "1"  /* Destruction of Left Large Intestine, Percutaneous Approach */
"0D5G7ZZ"  =  "1"  /* Destruction of Left Large Intestine, Via Natural or Artificial Opening */
"0D5H0ZZ"  =  "1"  /* Destruction of Cecum, Open Approach */
"0D5H3ZZ"  =  "1"  /* Destruction of Cecum, Percutaneous Approach */
"0D5H7ZZ"  =  "1"  /* Destruction of Cecum, Via Natural or Artificial Opening */
"0D5K0ZZ"  =  "1"  /* Destruction of Ascending Colon, Open Approach */
"0D5K3ZZ"  =  "1"  /* Destruction of Ascending Colon, Percutaneous Approach */
"0D5K7ZZ"  =  "1"  /* Destruction of Ascending Colon, Via Natural or Artificial Opening */
"0D5L0ZZ"  =  "1"  /* Destruction of Transverse Colon, Open Approach */
"0D5L3ZZ"  =  "1"  /* Destruction of Transverse Colon, Percutaneous Approach */
"0D5L7ZZ"  =  "1"  /* Destruction of Transverse Colon, Via Natural or Artificial Opening */
"0D5M0ZZ"  =  "1"  /* Destruction of Descending Colon, Open Approach */
"0D5M3ZZ"  =  "1"  /* Destruction of Descending Colon, Percutaneous Approach */
"0D5M7ZZ"  =  "1"  /* Destruction of Descending Colon, Via Natural or Artificial Opening */
"0D5N0ZZ"  =  "1"  /* Destruction of Sigmoid Colon, Open Approach */
"0D5N3ZZ"  =  "1"  /* Destruction of Sigmoid Colon, Percutaneous Approach */
"0D5N7ZZ"  =  "1"  /* Destruction of Sigmoid Colon, Via Natural or Artificial Opening */
"0D5S0ZZ"  =  "1"  /* Destruction of Greater Omentum, Open Approach */
"0D5S3ZZ"  =  "1"  /* Destruction of Greater Omentum, Percutaneous Approach */
"0D5S4ZZ"  =  "1"  /* Destruction of Greater Omentum, Percutaneous Endoscopic Approach */
"0D5T0ZZ"  =  "1"  /* Destruction of Lesser Omentum, Open Approach */
"0D5T3ZZ"  =  "1"  /* Destruction of Lesser Omentum, Percutaneous Approach */
"0D5T4ZZ"  =  "1"  /* Destruction of Lesser Omentum, Percutaneous Endoscopic Approach */
"0D5V0ZZ"  =  "1"  /* Destruction of Mesentery, Open Approach */
"0D5V3ZZ"  =  "1"  /* Destruction of Mesentery, Percutaneous Approach */
"0D5V4ZZ"  =  "1"  /* Destruction of Mesentery, Percutaneous Endoscopic Approach */
"0D5W0ZZ"  =  "1"  /* Destruction of Peritoneum, Open Approach */
"0D5W3ZZ"  =  "1"  /* Destruction of Peritoneum, Percutaneous Approach */
"0D5W4ZZ"  =  "1"  /* Destruction of Peritoneum, Percutaneous Endoscopic Approach */
"0D760DZ"  =  "1"  /* Dilation of Stomach with Intraluminal Device, Open Approach */
"0D760ZZ"  =  "1"  /* Dilation of Stomach, Open Approach */
"0D763DZ"  =  "1"  /* Dilation of Stomach with Intraluminal Device, Percutaneous Approach */
"0D763ZZ"  =  "1"  /* Dilation of Stomach, Percutaneous Approach */
"0D764DZ"  =  "1"  /* Dilation of Stomach with Intraluminal Device, Percutaneous Endoscopic Approach */
"0D764ZZ"  =  "1"  /* Dilation of Stomach, Percutaneous Endoscopic Approach */
"0D767DZ"  =  "1"  /* Dilation of Stomach with Intraluminal Device, Via Natural or Artificial Opening */
"0D767ZZ"  =  "1"  /* Dilation of Stomach, Via Natural or Artificial Opening */
"0D768DZ"  =  "1"  /* Dilation of Stomach with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0D768ZZ"  =  "1"  /* Dilation of Stomach, Via Natural or Artificial Opening Endoscopic */
"0D770DZ"  =  "1"  /* Dilation of Stomach, Pylorus with Intraluminal Device, Open Approach */
"0D770ZZ"  =  "1"  /* Dilation of Stomach, Pylorus, Open Approach */
"0D773DZ"  =  "1"  /* Dilation of Stomach, Pylorus with Intraluminal Device, Percutaneous Approach */
"0D773ZZ"  =  "1"  /* Dilation of Stomach, Pylorus, Percutaneous Approach */
"0D774ZZ"  =  "1"  /* Dilation of Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0D777DZ"  =  "1"  /* Dilation of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening */
"0D777ZZ"  =  "1"  /* Dilation of Stomach, Pylorus, Via Natural or Artificial Opening */
"0D778ZZ"  =  "1"  /* Dilation of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0D780ZZ"  =  "1"  /* Dilation of Small Intestine, Open Approach */
"0D783ZZ"  =  "1"  /* Dilation of Small Intestine, Percutaneous Approach */
"0D784ZZ"  =  "1"  /* Dilation of Small Intestine, Percutaneous Endoscopic Approach */
"0D790ZZ"  =  "1"  /* Dilation of Duodenum, Open Approach */
"0D793ZZ"  =  "1"  /* Dilation of Duodenum, Percutaneous Approach */
"0D794ZZ"  =  "1"  /* Dilation of Duodenum, Percutaneous Endoscopic Approach */
"0D7A0ZZ"  =  "1"  /* Dilation of Jejunum, Open Approach */
"0D7A3ZZ"  =  "1"  /* Dilation of Jejunum, Percutaneous Approach */
"0D7A4ZZ"  =  "1"  /* Dilation of Jejunum, Percutaneous Endoscopic Approach */
"0D7B0ZZ"  =  "1"  /* Dilation of Ileum, Open Approach */
"0D7B3ZZ"  =  "1"  /* Dilation of Ileum, Percutaneous Approach */
"0D7B4ZZ"  =  "1"  /* Dilation of Ileum, Percutaneous Endoscopic Approach */
"0D7C0ZZ"  =  "1"  /* Dilation of Ileocecal Valve, Open Approach */
"0D7C3ZZ"  =  "1"  /* Dilation of Ileocecal Valve, Percutaneous Approach */
"0D7C4ZZ"  =  "1"  /* Dilation of Ileocecal Valve, Percutaneous Endoscopic Approach */
"0D7E0ZZ"  =  "1"  /* Dilation of Large Intestine, Open Approach */
"0D7E3ZZ"  =  "1"  /* Dilation of Large Intestine, Percutaneous Approach */
"0D7E4ZZ"  =  "1"  /* Dilation of Large Intestine, Percutaneous Endoscopic Approach */
"0D7F0ZZ"  =  "1"  /* Dilation of Right Large Intestine, Open Approach */
"0D7F3ZZ"  =  "1"  /* Dilation of Right Large Intestine, Percutaneous Approach */
"0D7F4ZZ"  =  "1"  /* Dilation of Right Large Intestine, Percutaneous Endoscopic Approach */
"0D7G0ZZ"  =  "1"  /* Dilation of Left Large Intestine, Open Approach */
"0D7G3ZZ"  =  "1"  /* Dilation of Left Large Intestine, Percutaneous Approach */
"0D7G4ZZ"  =  "1"  /* Dilation of Left Large Intestine, Percutaneous Endoscopic Approach */
"0D7H0ZZ"  =  "1"  /* Dilation of Cecum, Open Approach */
"0D7H3ZZ"  =  "1"  /* Dilation of Cecum, Percutaneous Approach */
"0D7H4ZZ"  =  "1"  /* Dilation of Cecum, Percutaneous Endoscopic Approach */
"0D7K0ZZ"  =  "1"  /* Dilation of Ascending Colon, Open Approach */
"0D7K3ZZ"  =  "1"  /* Dilation of Ascending Colon, Percutaneous Approach */
"0D7K4ZZ"  =  "1"  /* Dilation of Ascending Colon, Percutaneous Endoscopic Approach */
"0D7L0ZZ"  =  "1"  /* Dilation of Transverse Colon, Open Approach */
"0D7L3ZZ"  =  "1"  /* Dilation of Transverse Colon, Percutaneous Approach */
"0D7L4ZZ"  =  "1"  /* Dilation of Transverse Colon, Percutaneous Endoscopic Approach */
"0D7M0ZZ"  =  "1"  /* Dilation of Descending Colon, Open Approach */
"0D7M3ZZ"  =  "1"  /* Dilation of Descending Colon, Percutaneous Approach */
"0D7M4ZZ"  =  "1"  /* Dilation of Descending Colon, Percutaneous Endoscopic Approach */
"0D7N0ZZ"  =  "1"  /* Dilation of Sigmoid Colon, Open Approach */
"0D7N3ZZ"  =  "1"  /* Dilation of Sigmoid Colon, Percutaneous Approach */
"0D7N4ZZ"  =  "1"  /* Dilation of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D7P0DZ"  =  "1"  /* Dilation of Rectum with Intraluminal Device, Open Approach */
"0D7P0ZZ"  =  "1"  /* Dilation of Rectum, Open Approach */
"0D7P3DZ"  =  "1"  /* Dilation of Rectum with Intraluminal Device, Percutaneous Approach */
"0D7P3ZZ"  =  "1"  /* Dilation of Rectum, Percutaneous Approach */
"0D7P4DZ"  =  "1"  /* Dilation of Rectum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0D7P4ZZ"  =  "1"  /* Dilation of Rectum, Percutaneous Endoscopic Approach */
"0D870ZZ"  =  "1"  /* Division of Stomach, Pylorus, Open Approach */
"0D873ZZ"  =  "1"  /* Division of Stomach, Pylorus, Percutaneous Approach */
"0D874ZZ"  =  "1"  /* Division of Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0D877ZZ"  =  "1"  /* Division of Stomach, Pylorus, Via Natural or Artificial Opening */
"0D878ZZ"  =  "1"  /* Division of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0D9600Z"  =  "1"  /* Drainage of Stomach with Drainage Device, Open Approach */
"0D960ZX"  =  "1"  /* Drainage of Stomach, Open Approach, Diagnostic */
"0D960ZZ"  =  "1"  /* Drainage of Stomach, Open Approach */
"0D9630Z"  =  "1"  /* Drainage of Stomach with Drainage Device, Percutaneous Approach */
"0D963ZZ"  =  "1"  /* Drainage of Stomach, Percutaneous Approach */
"0D9640Z"  =  "1"  /* Drainage of Stomach with Drainage Device, Percutaneous Endoscopic Approach */
"0D964ZZ"  =  "1"  /* Drainage of Stomach, Percutaneous Endoscopic Approach */
"0D967ZZ"  =  "1"  /* Drainage of Stomach, Via Natural or Artificial Opening */
"0D968ZZ"  =  "1"  /* Drainage of Stomach, Via Natural or Artificial Opening Endoscopic */
"0D9700Z"  =  "1"  /* Drainage of Stomach, Pylorus with Drainage Device, Open Approach */
"0D970ZX"  =  "1"  /* Drainage of Stomach, Pylorus, Open Approach, Diagnostic */
"0D970ZZ"  =  "1"  /* Drainage of Stomach, Pylorus, Open Approach */
"0D9730Z"  =  "1"  /* Drainage of Stomach, Pylorus with Drainage Device, Percutaneous Approach */
"0D973ZZ"  =  "1"  /* Drainage of Stomach, Pylorus, Percutaneous Approach */
"0D9740Z"  =  "1"  /* Drainage of Stomach, Pylorus with Drainage Device, Percutaneous Endoscopic Approach */
"0D974ZZ"  =  "1"  /* Drainage of Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0D977ZZ"  =  "1"  /* Drainage of Stomach, Pylorus, Via Natural or Artificial Opening */
"0D978ZZ"  =  "1"  /* Drainage of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0D9800Z"  =  "1"  /* Drainage of Small Intestine with Drainage Device, Open Approach */
"0D980ZX"  =  "1"  /* Drainage of Small Intestine, Open Approach, Diagnostic */
"0D980ZZ"  =  "1"  /* Drainage of Small Intestine, Open Approach */
"0D9830Z"  =  "1"  /* Drainage of Small Intestine with Drainage Device, Percutaneous Approach */
"0D983ZZ"  =  "1"  /* Drainage of Small Intestine, Percutaneous Approach */
"0D9840Z"  =  "1"  /* Drainage of Small Intestine with Drainage Device, Percutaneous Endoscopic Approach */
"0D984ZZ"  =  "1"  /* Drainage of Small Intestine, Percutaneous Endoscopic Approach */
"0D9870Z"  =  "1"  /* Drainage of Small Intestine with Drainage Device, Via Natural or Artificial Opening */
"0D987ZZ"  =  "1"  /* Drainage of Small Intestine, Via Natural or Artificial Opening */
"0D9880Z"  =  "1"  /* Drainage of Small Intestine with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D988ZZ"  =  "1"  /* Drainage of Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0D9900Z"  =  "1"  /* Drainage of Duodenum with Drainage Device, Open Approach */
"0D990ZX"  =  "1"  /* Drainage of Duodenum, Open Approach, Diagnostic */
"0D990ZZ"  =  "1"  /* Drainage of Duodenum, Open Approach */
"0D9930Z"  =  "1"  /* Drainage of Duodenum with Drainage Device, Percutaneous Approach */
"0D993ZZ"  =  "1"  /* Drainage of Duodenum, Percutaneous Approach */
"0D9940Z"  =  "1"  /* Drainage of Duodenum with Drainage Device, Percutaneous Endoscopic Approach */
"0D994ZZ"  =  "1"  /* Drainage of Duodenum, Percutaneous Endoscopic Approach */
"0D997ZZ"  =  "1"  /* Drainage of Duodenum, Via Natural or Artificial Opening */
"0D998ZZ"  =  "1"  /* Drainage of Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D9A00Z"  =  "1"  /* Drainage of Jejunum with Drainage Device, Open Approach */
"0D9A0ZX"  =  "1"  /* Drainage of Jejunum, Open Approach, Diagnostic */
"0D9A0ZZ"  =  "1"  /* Drainage of Jejunum, Open Approach */
"0D9A30Z"  =  "1"  /* Drainage of Jejunum with Drainage Device, Percutaneous Approach */
"0D9A3ZZ"  =  "1"  /* Drainage of Jejunum, Percutaneous Approach */
"0D9A40Z"  =  "1"  /* Drainage of Jejunum with Drainage Device, Percutaneous Endoscopic Approach */
"0D9A4ZZ"  =  "1"  /* Drainage of Jejunum, Percutaneous Endoscopic Approach */
"0D9A7ZZ"  =  "1"  /* Drainage of Jejunum, Via Natural or Artificial Opening */
"0D9A8ZZ"  =  "1"  /* Drainage of Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D9B00Z"  =  "1"  /* Drainage of Ileum with Drainage Device, Open Approach */
"0D9B0ZX"  =  "1"  /* Drainage of Ileum, Open Approach, Diagnostic */
"0D9B0ZZ"  =  "1"  /* Drainage of Ileum, Open Approach */
"0D9B30Z"  =  "1"  /* Drainage of Ileum with Drainage Device, Percutaneous Approach */
"0D9B3ZZ"  =  "1"  /* Drainage of Ileum, Percutaneous Approach */
"0D9B40Z"  =  "1"  /* Drainage of Ileum with Drainage Device, Percutaneous Endoscopic Approach */
"0D9B4ZZ"  =  "1"  /* Drainage of Ileum, Percutaneous Endoscopic Approach */
"0D9B7ZZ"  =  "1"  /* Drainage of Ileum, Via Natural or Artificial Opening */
"0D9B8ZZ"  =  "1"  /* Drainage of Ileum, Via Natural or Artificial Opening Endoscopic */
"0D9C00Z"  =  "1"  /* Drainage of Ileocecal Valve with Drainage Device, Open Approach */
"0D9C0ZX"  =  "1"  /* Drainage of Ileocecal Valve, Open Approach, Diagnostic */
"0D9C0ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Open Approach */
"0D9C30Z"  =  "1"  /* Drainage of Ileocecal Valve with Drainage Device, Percutaneous Approach */
"0D9C3ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Percutaneous Approach */
"0D9C40Z"  =  "1"  /* Drainage of Ileocecal Valve with Drainage Device, Percutaneous Endoscopic Approach */
"0D9C4ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Percutaneous Endoscopic Approach */
"0D9C70Z"  =  "1"  /* Drainage of Ileocecal Valve with Drainage Device, Via Natural or Artificial Opening */
"0D9C7ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Via Natural or Artificial Opening */
"0D9C80Z"  =  "1"  /* Drainage of Ileocecal Valve with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D9C8ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Via Natural or Artificial Opening Endoscopic */
"0D9E00Z"  =  "1"  /* Drainage of Large Intestine with Drainage Device, Open Approach */
"0D9E0ZX"  =  "1"  /* Drainage of Large Intestine, Open Approach, Diagnostic */
"0D9E0ZZ"  =  "1"  /* Drainage of Large Intestine, Open Approach */
"0D9E30Z"  =  "1"  /* Drainage of Large Intestine with Drainage Device, Percutaneous Approach */
"0D9E3ZZ"  =  "1"  /* Drainage of Large Intestine, Percutaneous Approach */
"0D9E40Z"  =  "1"  /* Drainage of Large Intestine with Drainage Device, Percutaneous Endoscopic Approach */
"0D9E4ZZ"  =  "1"  /* Drainage of Large Intestine, Percutaneous Endoscopic Approach */
"0D9E70Z"  =  "1"  /* Drainage of Large Intestine with Drainage Device, Via Natural or Artificial Opening */
"0D9E7ZZ"  =  "1"  /* Drainage of Large Intestine, Via Natural or Artificial Opening */
"0D9E80Z"  =  "1"  /* Drainage of Large Intestine with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D9E8ZZ"  =  "1"  /* Drainage of Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0D9F00Z"  =  "1"  /* Drainage of Right Large Intestine with Drainage Device, Open Approach */
"0D9F0ZX"  =  "1"  /* Drainage of Right Large Intestine, Open Approach, Diagnostic */
"0D9F0ZZ"  =  "1"  /* Drainage of Right Large Intestine, Open Approach */
"0D9F30Z"  =  "1"  /* Drainage of Right Large Intestine with Drainage Device, Percutaneous Approach */
"0D9F3ZZ"  =  "1"  /* Drainage of Right Large Intestine, Percutaneous Approach */
"0D9F40Z"  =  "1"  /* Drainage of Right Large Intestine with Drainage Device, Percutaneous Endoscopic Approach */
"0D9F4ZZ"  =  "1"  /* Drainage of Right Large Intestine, Percutaneous Endoscopic Approach */
"0D9F7ZZ"  =  "1"  /* Drainage of Right Large Intestine, Via Natural or Artificial Opening */
"0D9F8ZZ"  =  "1"  /* Drainage of Right Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0D9G00Z"  =  "1"  /* Drainage of Left Large Intestine with Drainage Device, Open Approach */
"0D9G0ZX"  =  "1"  /* Drainage of Left Large Intestine, Open Approach, Diagnostic */
"0D9G0ZZ"  =  "1"  /* Drainage of Left Large Intestine, Open Approach */
"0D9G30Z"  =  "1"  /* Drainage of Left Large Intestine with Drainage Device, Percutaneous Approach */
"0D9G3ZZ"  =  "1"  /* Drainage of Left Large Intestine, Percutaneous Approach */
"0D9G40Z"  =  "1"  /* Drainage of Left Large Intestine with Drainage Device, Percutaneous Endoscopic Approach */
"0D9G4ZZ"  =  "1"  /* Drainage of Left Large Intestine, Percutaneous Endoscopic Approach */
"0D9G7ZZ"  =  "1"  /* Drainage of Left Large Intestine, Via Natural or Artificial Opening */
"0D9G8ZZ"  =  "1"  /* Drainage of Left Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0D9H00Z"  =  "1"  /* Drainage of Cecum with Drainage Device, Open Approach */
"0D9H0ZX"  =  "1"  /* Drainage of Cecum, Open Approach, Diagnostic */
"0D9H0ZZ"  =  "1"  /* Drainage of Cecum, Open Approach */
"0D9H30Z"  =  "1"  /* Drainage of Cecum with Drainage Device, Percutaneous Approach */
"0D9H3ZZ"  =  "1"  /* Drainage of Cecum, Percutaneous Approach */
"0D9H40Z"  =  "1"  /* Drainage of Cecum with Drainage Device, Percutaneous Endoscopic Approach */
"0D9H4ZZ"  =  "1"  /* Drainage of Cecum, Percutaneous Endoscopic Approach */
"0D9H7ZZ"  =  "1"  /* Drainage of Cecum, Via Natural or Artificial Opening */
"0D9H8ZZ"  =  "1"  /* Drainage of Cecum, Via Natural or Artificial Opening Endoscopic */
"0D9K00Z"  =  "1"  /* Drainage of Ascending Colon with Drainage Device, Open Approach */
"0D9K0ZX"  =  "1"  /* Drainage of Ascending Colon, Open Approach, Diagnostic */
"0D9K0ZZ"  =  "1"  /* Drainage of Ascending Colon, Open Approach */
"0D9K30Z"  =  "1"  /* Drainage of Ascending Colon with Drainage Device, Percutaneous Approach */
"0D9K3ZZ"  =  "1"  /* Drainage of Ascending Colon, Percutaneous Approach */
"0D9K40Z"  =  "1"  /* Drainage of Ascending Colon with Drainage Device, Percutaneous Endoscopic Approach */
"0D9K4ZZ"  =  "1"  /* Drainage of Ascending Colon, Percutaneous Endoscopic Approach */
"0D9K7ZZ"  =  "1"  /* Drainage of Ascending Colon, Via Natural or Artificial Opening */
"0D9K8ZZ"  =  "1"  /* Drainage of Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0D9L00Z"  =  "1"  /* Drainage of Transverse Colon with Drainage Device, Open Approach */
"0D9L0ZX"  =  "1"  /* Drainage of Transverse Colon, Open Approach, Diagnostic */
"0D9L0ZZ"  =  "1"  /* Drainage of Transverse Colon, Open Approach */
"0D9L30Z"  =  "1"  /* Drainage of Transverse Colon with Drainage Device, Percutaneous Approach */
"0D9L3ZZ"  =  "1"  /* Drainage of Transverse Colon, Percutaneous Approach */
"0D9L40Z"  =  "1"  /* Drainage of Transverse Colon with Drainage Device, Percutaneous Endoscopic Approach */
"0D9L4ZZ"  =  "1"  /* Drainage of Transverse Colon, Percutaneous Endoscopic Approach */
"0D9L7ZZ"  =  "1"  /* Drainage of Transverse Colon, Via Natural or Artificial Opening */
"0D9L8ZZ"  =  "1"  /* Drainage of Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D9M00Z"  =  "1"  /* Drainage of Descending Colon with Drainage Device, Open Approach */
"0D9M0ZX"  =  "1"  /* Drainage of Descending Colon, Open Approach, Diagnostic */
"0D9M0ZZ"  =  "1"  /* Drainage of Descending Colon, Open Approach */
"0D9M30Z"  =  "1"  /* Drainage of Descending Colon with Drainage Device, Percutaneous Approach */
"0D9M3ZZ"  =  "1"  /* Drainage of Descending Colon, Percutaneous Approach */
"0D9M40Z"  =  "1"  /* Drainage of Descending Colon with Drainage Device, Percutaneous Endoscopic Approach */
"0D9M4ZZ"  =  "1"  /* Drainage of Descending Colon, Percutaneous Endoscopic Approach */
"0D9M7ZZ"  =  "1"  /* Drainage of Descending Colon, Via Natural or Artificial Opening */
"0D9M8ZZ"  =  "1"  /* Drainage of Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0D9N00Z"  =  "1"  /* Drainage of Sigmoid Colon with Drainage Device, Open Approach */
"0D9N0ZX"  =  "1"  /* Drainage of Sigmoid Colon, Open Approach, Diagnostic */
"0D9N0ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Open Approach */
"0D9N30Z"  =  "1"  /* Drainage of Sigmoid Colon with Drainage Device, Percutaneous Approach */
"0D9N3ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Percutaneous Approach */
"0D9N40Z"  =  "1"  /* Drainage of Sigmoid Colon with Drainage Device, Percutaneous Endoscopic Approach */
"0D9N4ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D9N7ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Via Natural or Artificial Opening */
"0D9N8ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D9P0ZX"  =  "1"  /* Drainage of Rectum, Open Approach, Diagnostic */
"0D9S00Z"  =  "1"  /* Drainage of Greater Omentum with Drainage Device, Open Approach */
"0D9S0ZX"  =  "1"  /* Drainage of Greater Omentum, Open Approach, Diagnostic */
"0D9S0ZZ"  =  "1"  /* Drainage of Greater Omentum, Open Approach */
"0D9S3ZX"  =  "1"  /* Drainage of Greater Omentum, Percutaneous Approach, Diagnostic */
"0D9S4ZX"  =  "1"  /* Drainage of Greater Omentum, Percutaneous Endoscopic Approach, Diagnostic */
"0D9T00Z"  =  "1"  /* Drainage of Lesser Omentum with Drainage Device, Open Approach */
"0D9T0ZX"  =  "1"  /* Drainage of Lesser Omentum, Open Approach, Diagnostic */
"0D9T0ZZ"  =  "1"  /* Drainage of Lesser Omentum, Open Approach */
"0D9T3ZX"  =  "1"  /* Drainage of Lesser Omentum, Percutaneous Approach, Diagnostic */
"0D9T4ZX"  =  "1"  /* Drainage of Lesser Omentum, Percutaneous Endoscopic Approach, Diagnostic */
"0D9V00Z"  =  "1"  /* Drainage of Mesentery with Drainage Device, Open Approach */
"0D9V0ZX"  =  "1"  /* Drainage of Mesentery, Open Approach, Diagnostic */
"0D9V0ZZ"  =  "1"  /* Drainage of Mesentery, Open Approach */
"0D9V3ZX"  =  "1"  /* Drainage of Mesentery, Percutaneous Approach, Diagnostic */
"0D9V4ZX"  =  "1"  /* Drainage of Mesentery, Percutaneous Endoscopic Approach, Diagnostic */
"0D9W00Z"  =  "1"  /* Drainage of Peritoneum with Drainage Device, Open Approach */
"0D9W0ZX"  =  "1"  /* Drainage of Peritoneum, Open Approach, Diagnostic */
"0D9W0ZZ"  =  "1"  /* Drainage of Peritoneum, Open Approach */
"0D9W3ZX"  =  "1"  /* Drainage of Peritoneum, Percutaneous Approach, Diagnostic */
"0D9W4ZX"  =  "1"  /* Drainage of Peritoneum, Percutaneous Endoscopic Approach, Diagnostic */
"0DB10ZZ"  =  "1"  /* Excision of Upper Esophagus, Open Approach */
"0DB13ZZ"  =  "1"  /* Excision of Upper Esophagus, Percutaneous Approach */
"0DB17ZZ"  =  "1"  /* Excision of Upper Esophagus, Via Natural or Artificial Opening */
"0DB20ZZ"  =  "1"  /* Excision of Middle Esophagus, Open Approach */
"0DB23ZZ"  =  "1"  /* Excision of Middle Esophagus, Percutaneous Approach */
"0DB27ZZ"  =  "1"  /* Excision of Middle Esophagus, Via Natural or Artificial Opening */
"0DB30ZZ"  =  "1"  /* Excision of Lower Esophagus, Open Approach */
"0DB33ZZ"  =  "1"  /* Excision of Lower Esophagus, Percutaneous Approach */
"0DB37ZZ"  =  "1"  /* Excision of Lower Esophagus, Via Natural or Artificial Opening */
"0DB40ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Open Approach */
"0DB43ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Percutaneous Approach */
"0DB44ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0DB47ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Via Natural or Artificial Opening */
"0DB50ZZ"  =  "1"  /* Excision of Esophagus, Open Approach */
"0DB53ZZ"  =  "1"  /* Excision of Esophagus, Percutaneous Approach */
"0DB57ZZ"  =  "1"  /* Excision of Esophagus, Via Natural or Artificial Opening */
"0DB60Z3"  =  "1"  /* Excision of Stomach, Open Approach, Vertical */
"0DB60ZX"  =  "1"  /* Excision of Stomach, Open Approach, Diagnostic */
"0DB60ZZ"  =  "1"  /* Excision of Stomach, Open Approach */
"0DB63Z3"  =  "1"  /* Excision of Stomach, Percutaneous Approach, Vertical */
"0DB63ZZ"  =  "1"  /* Excision of Stomach, Percutaneous Approach */
"0DB64Z3"  =  "1"  /* Excision of Stomach, Percutaneous Endoscopic Approach, Vertical */
"0DB64ZZ"  =  "1"  /* Excision of Stomach, Percutaneous Endoscopic Approach */
"0DB67Z3"  =  "1"  /* Excision of Stomach, Via Natural or Artificial Opening, Vertical */
"0DB67ZZ"  =  "1"  /* Excision of Stomach, Via Natural or Artificial Opening */
"0DB68Z3"  =  "1"  /* Excision of Stomach, Via Natural or Artificial Opening Endoscopic, Vertical */
"0DB68ZZ"  =  "1"  /* Excision of Stomach, Via Natural or Artificial Opening Endoscopic */
"0DB70ZX"  =  "1"  /* Excision of Stomach, Pylorus, Open Approach, Diagnostic */
"0DB70ZZ"  =  "1"  /* Excision of Stomach, Pylorus, Open Approach */
"0DB73ZZ"  =  "1"  /* Excision of Stomach, Pylorus, Percutaneous Approach */
"0DB77ZZ"  =  "1"  /* Excision of Stomach, Pylorus, Via Natural or Artificial Opening */
"0DB80ZX"  =  "1"  /* Excision of Small Intestine, Open Approach, Diagnostic */
"0DB80ZZ"  =  "1"  /* Excision of Small Intestine, Open Approach */
"0DB83ZZ"  =  "1"  /* Excision of Small Intestine, Percutaneous Approach */
"0DB84ZZ"  =  "1"  /* Excision of Small Intestine, Percutaneous Endoscopic Approach */
"0DB87ZZ"  =  "1"  /* Excision of Small Intestine, Via Natural or Artificial Opening */
"0DB88ZZ"  =  "1"  /* Excision of Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DB90ZX"  =  "1"  /* Excision of Duodenum, Open Approach, Diagnostic */
"0DB90ZZ"  =  "1"  /* Excision of Duodenum, Open Approach */
"0DB93ZZ"  =  "1"  /* Excision of Duodenum, Percutaneous Approach */
"0DB94ZZ"  =  "1"  /* Excision of Duodenum, Percutaneous Endoscopic Approach */
"0DB97ZZ"  =  "1"  /* Excision of Duodenum, Via Natural or Artificial Opening */
"0DB98ZZ"  =  "1"  /* Excision of Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DBA0ZX"  =  "1"  /* Excision of Jejunum, Open Approach, Diagnostic */
"0DBA0ZZ"  =  "1"  /* Excision of Jejunum, Open Approach */
"0DBA3ZZ"  =  "1"  /* Excision of Jejunum, Percutaneous Approach */
"0DBA4ZZ"  =  "1"  /* Excision of Jejunum, Percutaneous Endoscopic Approach */
"0DBA7ZZ"  =  "1"  /* Excision of Jejunum, Via Natural or Artificial Opening */
"0DBA8ZZ"  =  "1"  /* Excision of Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DBB0ZX"  =  "1"  /* Excision of Ileum, Open Approach, Diagnostic */
"0DBB0ZZ"  =  "1"  /* Excision of Ileum, Open Approach */
"0DBB3ZZ"  =  "1"  /* Excision of Ileum, Percutaneous Approach */
"0DBB4ZZ"  =  "1"  /* Excision of Ileum, Percutaneous Endoscopic Approach */
"0DBB7ZZ"  =  "1"  /* Excision of Ileum, Via Natural or Artificial Opening */
"0DBB8ZZ"  =  "1"  /* Excision of Ileum, Via Natural or Artificial Opening Endoscopic */
"0DBC0ZX"  =  "1"  /* Excision of Ileocecal Valve, Open Approach, Diagnostic */
"0DBC0ZZ"  =  "1"  /* Excision of Ileocecal Valve, Open Approach */
"0DBC3ZZ"  =  "1"  /* Excision of Ileocecal Valve, Percutaneous Approach */
"0DBC4ZZ"  =  "1"  /* Excision of Ileocecal Valve, Percutaneous Endoscopic Approach */
"0DBC7ZZ"  =  "1"  /* Excision of Ileocecal Valve, Via Natural or Artificial Opening */
"0DBC8ZZ"  =  "1"  /* Excision of Ileocecal Valve, Via Natural or Artificial Opening Endoscopic */
"0DBE0ZX"  =  "1"  /* Excision of Large Intestine, Open Approach, Diagnostic */
"0DBE0ZZ"  =  "1"  /* Excision of Large Intestine, Open Approach */
"0DBE3ZZ"  =  "1"  /* Excision of Large Intestine, Percutaneous Approach */
"0DBE7ZZ"  =  "1"  /* Excision of Large Intestine, Via Natural or Artificial Opening */
"0DBF0ZX"  =  "1"  /* Excision of Right Large Intestine, Open Approach, Diagnostic */
"0DBF0ZZ"  =  "1"  /* Excision of Right Large Intestine, Open Approach */
"0DBF3ZZ"  =  "1"  /* Excision of Right Large Intestine, Percutaneous Approach */
"0DBF7ZZ"  =  "1"  /* Excision of Right Large Intestine, Via Natural or Artificial Opening */
"0DBG0ZX"  =  "1"  /* Excision of Left Large Intestine, Open Approach, Diagnostic */
"0DBG0ZZ"  =  "1"  /* Excision of Left Large Intestine, Open Approach */
"0DBG3ZZ"  =  "1"  /* Excision of Left Large Intestine, Percutaneous Approach */
"0DBG7ZZ"  =  "1"  /* Excision of Left Large Intestine, Via Natural or Artificial Opening */
"0DBH0ZX"  =  "1"  /* Excision of Cecum, Open Approach, Diagnostic */
"0DBH0ZZ"  =  "1"  /* Excision of Cecum, Open Approach */
"0DBH3ZZ"  =  "1"  /* Excision of Cecum, Percutaneous Approach */
"0DBH7ZZ"  =  "1"  /* Excision of Cecum, Via Natural or Artificial Opening */
"0DBK0ZX"  =  "1"  /* Excision of Ascending Colon, Open Approach, Diagnostic */
"0DBK0ZZ"  =  "1"  /* Excision of Ascending Colon, Open Approach */
"0DBK3ZZ"  =  "1"  /* Excision of Ascending Colon, Percutaneous Approach */
"0DBK7ZZ"  =  "1"  /* Excision of Ascending Colon, Via Natural or Artificial Opening */
"0DBL0ZX"  =  "1"  /* Excision of Transverse Colon, Open Approach, Diagnostic */
"0DBL0ZZ"  =  "1"  /* Excision of Transverse Colon, Open Approach */
"0DBL3ZZ"  =  "1"  /* Excision of Transverse Colon, Percutaneous Approach */
"0DBL7ZZ"  =  "1"  /* Excision of Transverse Colon, Via Natural or Artificial Opening */
"0DBM0ZX"  =  "1"  /* Excision of Descending Colon, Open Approach, Diagnostic */
"0DBM0ZZ"  =  "1"  /* Excision of Descending Colon, Open Approach */
"0DBM3ZZ"  =  "1"  /* Excision of Descending Colon, Percutaneous Approach */
"0DBM7ZZ"  =  "1"  /* Excision of Descending Colon, Via Natural or Artificial Opening */
"0DBN0ZX"  =  "1"  /* Excision of Sigmoid Colon, Open Approach, Diagnostic */
"0DBN0ZZ"  =  "1"  /* Excision of Sigmoid Colon, Open Approach */
"0DBN3ZZ"  =  "1"  /* Excision of Sigmoid Colon, Percutaneous Approach */
"0DBN7ZZ"  =  "1"  /* Excision of Sigmoid Colon, Via Natural or Artificial Opening */
"0DBP0ZX"  =  "1"  /* Excision of Rectum, Open Approach, Diagnostic */
"0DBP7ZZ"  =  "1"  /* Excision of Rectum, Via Natural or Artificial Opening */
"0DBP8ZZ"  =  "1"  /* Excision of Rectum, Via Natural or Artificial Opening Endoscopic */
"0DBS0ZX"  =  "1"  /* Excision of Greater Omentum, Open Approach, Diagnostic */
"0DBS0ZZ"  =  "1"  /* Excision of Greater Omentum, Open Approach */
"0DBS3ZZ"  =  "1"  /* Excision of Greater Omentum, Percutaneous Approach */
"0DBS4ZZ"  =  "1"  /* Excision of Greater Omentum, Percutaneous Endoscopic Approach */
"0DBT0ZX"  =  "1"  /* Excision of Lesser Omentum, Open Approach, Diagnostic */
"0DBT0ZZ"  =  "1"  /* Excision of Lesser Omentum, Open Approach */
"0DBT3ZZ"  =  "1"  /* Excision of Lesser Omentum, Percutaneous Approach */
"0DBT4ZZ"  =  "1"  /* Excision of Lesser Omentum, Percutaneous Endoscopic Approach */
"0DBV0ZX"  =  "1"  /* Excision of Mesentery, Open Approach, Diagnostic */
"0DBV0ZZ"  =  "1"  /* Excision of Mesentery, Open Approach */
"0DBV3ZZ"  =  "1"  /* Excision of Mesentery, Percutaneous Approach */
"0DBV4ZZ"  =  "1"  /* Excision of Mesentery, Percutaneous Endoscopic Approach */
"0DBW0ZX"  =  "1"  /* Excision of Peritoneum, Open Approach, Diagnostic */
"0DBW0ZZ"  =  "1"  /* Excision of Peritoneum, Open Approach */
"0DBW3ZZ"  =  "1"  /* Excision of Peritoneum, Percutaneous Approach */
"0DBW4ZZ"  =  "1"  /* Excision of Peritoneum, Percutaneous Endoscopic Approach */
"0DC60ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Open Approach */
"0DC63ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Percutaneous Approach */
"0DC64ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Percutaneous Endoscopic Approach */
"0DC70ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Pylorus, Open Approach */
"0DC73ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Pylorus, Percutaneous Approach */
"0DC74ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0DC80ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Open Approach */
"0DC83ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Percutaneous Approach */
"0DC84ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Percutaneous Endoscopic Approach */
"0DC87ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Via Natural or Artificial Opening */
"0DC88ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DC90ZZ"  =  "1"  /* Extirpation of Matter from Duodenum, Open Approach */
"0DC93ZZ"  =  "1"  /* Extirpation of Matter from Duodenum, Percutaneous Approach */
"0DC94ZZ"  =  "1"  /* Extirpation of Matter from Duodenum, Percutaneous Endoscopic Approach */
"0DCA0ZZ"  =  "1"  /* Extirpation of Matter from Jejunum, Open Approach */
"0DCA3ZZ"  =  "1"  /* Extirpation of Matter from Jejunum, Percutaneous Approach */
"0DCA4ZZ"  =  "1"  /* Extirpation of Matter from Jejunum, Percutaneous Endoscopic Approach */
"0DCB0ZZ"  =  "1"  /* Extirpation of Matter from Ileum, Open Approach */
"0DCB3ZZ"  =  "1"  /* Extirpation of Matter from Ileum, Percutaneous Approach */
"0DCB4ZZ"  =  "1"  /* Extirpation of Matter from Ileum, Percutaneous Endoscopic Approach */
"0DCC0ZZ"  =  "1"  /* Extirpation of Matter from Ileocecal Valve, Open Approach */
"0DCC3ZZ"  =  "1"  /* Extirpation of Matter from Ileocecal Valve, Percutaneous Approach */
"0DCC4ZZ"  =  "1"  /* Extirpation of Matter from Ileocecal Valve, Percutaneous Endoscopic Approach */
"0DCE0ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Open Approach */
"0DCE3ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Percutaneous Approach */
"0DCE4ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Percutaneous Endoscopic Approach */
"0DCE7ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Via Natural or Artificial Opening */
"0DCE8ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DCF0ZZ"  =  "1"  /* Extirpation of Matter from Right Large Intestine, Open Approach */
"0DCF3ZZ"  =  "1"  /* Extirpation of Matter from Right Large Intestine, Percutaneous Approach */
"0DCF4ZZ"  =  "1"  /* Extirpation of Matter from Right Large Intestine, Percutaneous Endoscopic Approach */
"0DCG0ZZ"  =  "1"  /* Extirpation of Matter from Left Large Intestine, Open Approach */
"0DCG3ZZ"  =  "1"  /* Extirpation of Matter from Left Large Intestine, Percutaneous Approach */
"0DCG4ZZ"  =  "1"  /* Extirpation of Matter from Left Large Intestine, Percutaneous Endoscopic Approach */
"0DCH0ZZ"  =  "1"  /* Extirpation of Matter from Cecum, Open Approach */
"0DCH3ZZ"  =  "1"  /* Extirpation of Matter from Cecum, Percutaneous Approach */
"0DCH4ZZ"  =  "1"  /* Extirpation of Matter from Cecum, Percutaneous Endoscopic Approach */
"0DCK0ZZ"  =  "1"  /* Extirpation of Matter from Ascending Colon, Open Approach */
"0DCK3ZZ"  =  "1"  /* Extirpation of Matter from Ascending Colon, Percutaneous Approach */
"0DCK4ZZ"  =  "1"  /* Extirpation of Matter from Ascending Colon, Percutaneous Endoscopic Approach */
"0DCL0ZZ"  =  "1"  /* Extirpation of Matter from Transverse Colon, Open Approach */
"0DCL3ZZ"  =  "1"  /* Extirpation of Matter from Transverse Colon, Percutaneous Approach */
"0DCL4ZZ"  =  "1"  /* Extirpation of Matter from Transverse Colon, Percutaneous Endoscopic Approach */
"0DCM0ZZ"  =  "1"  /* Extirpation of Matter from Descending Colon, Open Approach */
"0DCM3ZZ"  =  "1"  /* Extirpation of Matter from Descending Colon, Percutaneous Approach */
"0DCM4ZZ"  =  "1"  /* Extirpation of Matter from Descending Colon, Percutaneous Endoscopic Approach */
"0DCN0ZZ"  =  "1"  /* Extirpation of Matter from Sigmoid Colon, Open Approach */
"0DCN3ZZ"  =  "1"  /* Extirpation of Matter from Sigmoid Colon, Percutaneous Approach */
"0DCN4ZZ"  =  "1"  /* Extirpation of Matter from Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DCS0ZZ"  =  "1"  /* Extirpation of Matter from Greater Omentum, Open Approach */
"0DCS3ZZ"  =  "1"  /* Extirpation of Matter from Greater Omentum, Percutaneous Approach */
"0DCS4ZZ"  =  "1"  /* Extirpation of Matter from Greater Omentum, Percutaneous Endoscopic Approach */
"0DCT0ZZ"  =  "1"  /* Extirpation of Matter from Lesser Omentum, Open Approach */
"0DCT3ZZ"  =  "1"  /* Extirpation of Matter from Lesser Omentum, Percutaneous Approach */
"0DCT4ZZ"  =  "1"  /* Extirpation of Matter from Lesser Omentum, Percutaneous Endoscopic Approach */
"0DCV0ZZ"  =  "1"  /* Extirpation of Matter from Mesentery, Open Approach */
"0DCV3ZZ"  =  "1"  /* Extirpation of Matter from Mesentery, Percutaneous Approach */
"0DCV4ZZ"  =  "1"  /* Extirpation of Matter from Mesentery, Percutaneous Endoscopic Approach */
"0DCW0ZZ"  =  "1"  /* Extirpation of Matter from Peritoneum, Open Approach */
"0DCW3ZZ"  =  "1"  /* Extirpation of Matter from Peritoneum, Percutaneous Approach */
"0DCW4ZZ"  =  "1"  /* Extirpation of Matter from Peritoneum, Percutaneous Endoscopic Approach */
"0DF60ZZ"  =  "1"  /* Fragmentation in Stomach, Open Approach */
"0DF63ZZ"  =  "1"  /* Fragmentation in Stomach, Percutaneous Approach */
"0DF64ZZ"  =  "1"  /* Fragmentation in Stomach, Percutaneous Endoscopic Approach */
"0DF67ZZ"  =  "1"  /* Fragmentation in Stomach, Via Natural or Artificial Opening */
"0DF68ZZ"  =  "1"  /* Fragmentation in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DF80ZZ"  =  "1"  /* Fragmentation in Small Intestine, Open Approach */
"0DF83ZZ"  =  "1"  /* Fragmentation in Small Intestine, Percutaneous Approach */
"0DF84ZZ"  =  "1"  /* Fragmentation in Small Intestine, Percutaneous Endoscopic Approach */
"0DF87ZZ"  =  "1"  /* Fragmentation in Small Intestine, Via Natural or Artificial Opening */
"0DF88ZZ"  =  "1"  /* Fragmentation in Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DF90ZZ"  =  "1"  /* Fragmentation in Duodenum, Open Approach */
"0DF93ZZ"  =  "1"  /* Fragmentation in Duodenum, Percutaneous Approach */
"0DF94ZZ"  =  "1"  /* Fragmentation in Duodenum, Percutaneous Endoscopic Approach */
"0DF97ZZ"  =  "1"  /* Fragmentation in Duodenum, Via Natural or Artificial Opening */
"0DF98ZZ"  =  "1"  /* Fragmentation in Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DFA0ZZ"  =  "1"  /* Fragmentation in Jejunum, Open Approach */
"0DFA3ZZ"  =  "1"  /* Fragmentation in Jejunum, Percutaneous Approach */
"0DFA4ZZ"  =  "1"  /* Fragmentation in Jejunum, Percutaneous Endoscopic Approach */
"0DFA7ZZ"  =  "1"  /* Fragmentation in Jejunum, Via Natural or Artificial Opening */
"0DFA8ZZ"  =  "1"  /* Fragmentation in Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DFB0ZZ"  =  "1"  /* Fragmentation in Ileum, Open Approach */
"0DFB3ZZ"  =  "1"  /* Fragmentation in Ileum, Percutaneous Approach */
"0DFB4ZZ"  =  "1"  /* Fragmentation in Ileum, Percutaneous Endoscopic Approach */
"0DFB7ZZ"  =  "1"  /* Fragmentation in Ileum, Via Natural or Artificial Opening */
"0DFB8ZZ"  =  "1"  /* Fragmentation in Ileum, Via Natural or Artificial Opening Endoscopic */
"0DFE0ZZ"  =  "1"  /* Fragmentation in Large Intestine, Open Approach */
"0DFE3ZZ"  =  "1"  /* Fragmentation in Large Intestine, Percutaneous Approach */
"0DFE4ZZ"  =  "1"  /* Fragmentation in Large Intestine, Percutaneous Endoscopic Approach */
"0DFE7ZZ"  =  "1"  /* Fragmentation in Large Intestine, Via Natural or Artificial Opening */
"0DFE8ZZ"  =  "1"  /* Fragmentation in Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DFF0ZZ"  =  "1"  /* Fragmentation in Right Large Intestine, Open Approach */
"0DFF3ZZ"  =  "1"  /* Fragmentation in Right Large Intestine, Percutaneous Approach */
"0DFF4ZZ"  =  "1"  /* Fragmentation in Right Large Intestine, Percutaneous Endoscopic Approach */
"0DFF7ZZ"  =  "1"  /* Fragmentation in Right Large Intestine, Via Natural or Artificial Opening */
"0DFF8ZZ"  =  "1"  /* Fragmentation in Right Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DFG0ZZ"  =  "1"  /* Fragmentation in Left Large Intestine, Open Approach */
"0DFG3ZZ"  =  "1"  /* Fragmentation in Left Large Intestine, Percutaneous Approach */
"0DFG4ZZ"  =  "1"  /* Fragmentation in Left Large Intestine, Percutaneous Endoscopic Approach */
"0DFG7ZZ"  =  "1"  /* Fragmentation in Left Large Intestine, Via Natural or Artificial Opening */
"0DFG8ZZ"  =  "1"  /* Fragmentation in Left Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DFH0ZZ"  =  "1"  /* Fragmentation in Cecum, Open Approach */
"0DFH3ZZ"  =  "1"  /* Fragmentation in Cecum, Percutaneous Approach */
"0DFH4ZZ"  =  "1"  /* Fragmentation in Cecum, Percutaneous Endoscopic Approach */
"0DFH7ZZ"  =  "1"  /* Fragmentation in Cecum, Via Natural or Artificial Opening */
"0DFH8ZZ"  =  "1"  /* Fragmentation in Cecum, Via Natural or Artificial Opening Endoscopic */
"0DFK0ZZ"  =  "1"  /* Fragmentation in Ascending Colon, Open Approach */
"0DFK3ZZ"  =  "1"  /* Fragmentation in Ascending Colon, Percutaneous Approach */
"0DFK4ZZ"  =  "1"  /* Fragmentation in Ascending Colon, Percutaneous Endoscopic Approach */
"0DFK7ZZ"  =  "1"  /* Fragmentation in Ascending Colon, Via Natural or Artificial Opening */
"0DFK8ZZ"  =  "1"  /* Fragmentation in Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0DFL0ZZ"  =  "1"  /* Fragmentation in Transverse Colon, Open Approach */
"0DFL3ZZ"  =  "1"  /* Fragmentation in Transverse Colon, Percutaneous Approach */
"0DFL4ZZ"  =  "1"  /* Fragmentation in Transverse Colon, Percutaneous Endoscopic Approach */
"0DFL7ZZ"  =  "1"  /* Fragmentation in Transverse Colon, Via Natural or Artificial Opening */
"0DFL8ZZ"  =  "1"  /* Fragmentation in Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0DFM0ZZ"  =  "1"  /* Fragmentation in Descending Colon, Open Approach */
"0DFM3ZZ"  =  "1"  /* Fragmentation in Descending Colon, Percutaneous Approach */
"0DFM4ZZ"  =  "1"  /* Fragmentation in Descending Colon, Percutaneous Endoscopic Approach */
"0DFM7ZZ"  =  "1"  /* Fragmentation in Descending Colon, Via Natural or Artificial Opening */
"0DFM8ZZ"  =  "1"  /* Fragmentation in Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0DFN0ZZ"  =  "1"  /* Fragmentation in Sigmoid Colon, Open Approach */
"0DFN3ZZ"  =  "1"  /* Fragmentation in Sigmoid Colon, Percutaneous Approach */
"0DFN4ZZ"  =  "1"  /* Fragmentation in Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DFN7ZZ"  =  "1"  /* Fragmentation in Sigmoid Colon, Via Natural or Artificial Opening */
"0DFN8ZZ"  =  "1"  /* Fragmentation in Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0DH602Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Open Approach */
"0DH603Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Open Approach */
"0DH60DZ"  =  "1"  /* Insertion of Intraluminal Device into Stomach, Open Approach */
"0DH60UZ"  =  "1"  /* Insertion of Feeding Device into Stomach, Open Approach */
"0DH632Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Percutaneous Approach */
"0DH633Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Percutaneous Approach */
"0DH63DZ"  =  "1"  /* Insertion of Intraluminal Device into Stomach, Percutaneous Approach */
"0DH642Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Percutaneous Endoscopic Approach */
"0DH643Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Percutaneous Endoscopic Approach */
"0DH64DZ"  =  "1"  /* Insertion of Intraluminal Device into Stomach, Percutaneous Endoscopic Approach */
"0DH672Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Via Natural or Artificial Opening */
"0DH673Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Via Natural or Artificial Opening */
"0DH67DZ"  =  "1"  /* Insertion of Intraluminal Device into Stomach, Via Natural or Artificial Opening */
"0DH67UZ"  =  "1"  /* Insertion of Feeding Device into Stomach, Via Natural or Artificial Opening */
"0DH682Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Via Natural or Artificial Opening Endoscopic */
"0DH683Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Via Natural or Artificial Opening Endoscopic */
"0DH68DZ"  =  "1"  /* Insertion of Intraluminal Device into Stomach, Via Natural or Artificial Opening Endoscopic */
"0DH68UZ"  =  "1"  /* Insertion of Feeding Device into Stomach, Via Natural or Artificial Opening Endoscopic */
"0DH802Z"  =  "1"  /* Insertion of Monitoring Device into Small Intestine, Open Approach */
"0DH803Z"  =  "1"  /* Insertion of Infusion Device into Small Intestine, Open Approach */
"0DH832Z"  =  "1"  /* Insertion of Monitoring Device into Small Intestine, Percutaneous Approach */
"0DH833Z"  =  "1"  /* Insertion of Infusion Device into Small Intestine, Percutaneous Approach */
"0DH842Z"  =  "1"  /* Insertion of Monitoring Device into Small Intestine, Percutaneous Endoscopic Approach */
"0DH843Z"  =  "1"  /* Insertion of Infusion Device into Small Intestine, Percutaneous Endoscopic Approach */
"0DH872Z"  =  "1"  /* Insertion of Monitoring Device into Small Intestine, Via Natural or Artificial Opening */
"0DH873Z"  =  "1"  /* Insertion of Infusion Device into Small Intestine, Via Natural or Artificial Opening */
"0DH882Z"  =  "1"  /* Insertion of Monitoring Device into Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DH883Z"  =  "1"  /* Insertion of Infusion Device into Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DH902Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Open Approach */
"0DH903Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Open Approach */
"0DH932Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Percutaneous Approach */
"0DH933Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Percutaneous Approach */
"0DH942Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Percutaneous Endoscopic Approach */
"0DH943Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Percutaneous Endoscopic Approach */
"0DH972Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Via Natural or Artificial Opening */
"0DH973Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Via Natural or Artificial Opening */
"0DH982Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DH983Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DHA02Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Open Approach */
"0DHA03Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Open Approach */
"0DHA32Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Percutaneous Approach */
"0DHA33Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Percutaneous Approach */
"0DHA42Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Percutaneous Endoscopic Approach */
"0DHA43Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Percutaneous Endoscopic Approach */
"0DHA72Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Via Natural or Artificial Opening */
"0DHA73Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Via Natural or Artificial Opening */
"0DHA82Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DHA83Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DHB02Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Open Approach */
"0DHB03Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Open Approach */
"0DHB32Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Percutaneous Approach */
"0DHB33Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Percutaneous Approach */
"0DHB42Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Percutaneous Endoscopic Approach */
"0DHB43Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Percutaneous Endoscopic Approach */
"0DHB72Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Via Natural or Artificial Opening */
"0DHB73Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Via Natural or Artificial Opening */
"0DHB82Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Via Natural or Artificial Opening Endoscopic */
"0DHB83Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Via Natural or Artificial Opening Endoscopic */
"0DJ00ZZ"  =  "1"  /* Inspection of Upper Intestinal Tract, Open Approach */
"0DJ60ZZ"  =  "1"  /* Inspection of Stomach, Open Approach */
"0DJ64ZZ"  =  "1"  /* Inspection of Stomach, Percutaneous Endoscopic Approach */
"0DJD0ZZ"  =  "1"  /* Inspection of Lower Intestinal Tract, Open Approach */
"0DJD4ZZ"  =  "1"  /* Inspection of Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DJU0ZZ"  =  "1"  /* Inspection of Omentum, Open Approach */
"0DJU3ZZ"  =  "1"  /* Inspection of Omentum, Percutaneous Approach */
"0DJU4ZZ"  =  "1"  /* Inspection of Omentum, Percutaneous Endoscopic Approach */
"0DJV0ZZ"  =  "1"  /* Inspection of Mesentery, Open Approach */
"0DJV3ZZ"  =  "1"  /* Inspection of Mesentery, Percutaneous Approach */
"0DJV4ZZ"  =  "1"  /* Inspection of Mesentery, Percutaneous Endoscopic Approach */
"0DJW0ZZ"  =  "1"  /* Inspection of Peritoneum, Open Approach */
"0DJW3ZZ"  =  "1"  /* Inspection of Peritoneum, Percutaneous Approach */
"0DJW4ZZ"  =  "1"  /* Inspection of Peritoneum, Percutaneous Endoscopic Approach */
"0DL60CZ"  =  "1"  /* Occlusion of Stomach with Extraluminal Device, Open Approach */
"0DL60DZ"  =  "1"  /* Occlusion of Stomach with Intraluminal Device, Open Approach */
"0DL60ZZ"  =  "1"  /* Occlusion of Stomach, Open Approach */
"0DL63CZ"  =  "1"  /* Occlusion of Stomach with Extraluminal Device, Percutaneous Approach */
"0DL63DZ"  =  "1"  /* Occlusion of Stomach with Intraluminal Device, Percutaneous Approach */
"0DL63ZZ"  =  "1"  /* Occlusion of Stomach, Percutaneous Approach */
"0DL64CZ"  =  "1"  /* Occlusion of Stomach with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DL64DZ"  =  "1"  /* Occlusion of Stomach with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DL64ZZ"  =  "1"  /* Occlusion of Stomach, Percutaneous Endoscopic Approach */
"0DL67DZ"  =  "1"  /* Occlusion of Stomach with Intraluminal Device, Via Natural or Artificial Opening */
"0DL67ZZ"  =  "1"  /* Occlusion of Stomach, Via Natural or Artificial Opening */
"0DL68DZ"  =  "1"  /* Occlusion of Stomach with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DL68ZZ"  =  "1"  /* Occlusion of Stomach, Via Natural or Artificial Opening Endoscopic */
"0DL70CZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Extraluminal Device, Open Approach */
"0DL70DZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Intraluminal Device, Open Approach */
"0DL70ZZ"  =  "1"  /* Occlusion of Stomach, Pylorus, Open Approach */
"0DL73CZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Extraluminal Device, Percutaneous Approach */
"0DL73DZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Intraluminal Device, Percutaneous Approach */
"0DL73ZZ"  =  "1"  /* Occlusion of Stomach, Pylorus, Percutaneous Approach */
"0DL74CZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DL74DZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DL74ZZ"  =  "1"  /* Occlusion of Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0DL77DZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening */
"0DL77ZZ"  =  "1"  /* Occlusion of Stomach, Pylorus, Via Natural or Artificial Opening */
"0DL78DZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DL78ZZ"  =  "1"  /* Occlusion of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0DL80CZ"  =  "1"  /* Occlusion of Small Intestine with Extraluminal Device, Open Approach */
"0DL80DZ"  =  "1"  /* Occlusion of Small Intestine with Intraluminal Device, Open Approach */
"0DL80ZZ"  =  "1"  /* Occlusion of Small Intestine, Open Approach */
"0DL83CZ"  =  "1"  /* Occlusion of Small Intestine with Extraluminal Device, Percutaneous Approach */
"0DL83DZ"  =  "1"  /* Occlusion of Small Intestine with Intraluminal Device, Percutaneous Approach */
"0DL83ZZ"  =  "1"  /* Occlusion of Small Intestine, Percutaneous Approach */
"0DL84CZ"  =  "1"  /* Occlusion of Small Intestine with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DL84DZ"  =  "1"  /* Occlusion of Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DL84ZZ"  =  "1"  /* Occlusion of Small Intestine, Percutaneous Endoscopic Approach */
"0DL87DZ"  =  "1"  /* Occlusion of Small Intestine with Intraluminal Device, Via Natural or Artificial Opening */
"0DL87ZZ"  =  "1"  /* Occlusion of Small Intestine, Via Natural or Artificial Opening */
"0DL88DZ"  =  "1"  /* Occlusion of Small Intestine with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DL88ZZ"  =  "1"  /* Occlusion of Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DL90CZ"  =  "1"  /* Occlusion of Duodenum with Extraluminal Device, Open Approach */
"0DL90DZ"  =  "1"  /* Occlusion of Duodenum with Intraluminal Device, Open Approach */
"0DL90ZZ"  =  "1"  /* Occlusion of Duodenum, Open Approach */
"0DL93CZ"  =  "1"  /* Occlusion of Duodenum with Extraluminal Device, Percutaneous Approach */
"0DL93DZ"  =  "1"  /* Occlusion of Duodenum with Intraluminal Device, Percutaneous Approach */
"0DL93ZZ"  =  "1"  /* Occlusion of Duodenum, Percutaneous Approach */
"0DL94CZ"  =  "1"  /* Occlusion of Duodenum with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DL94DZ"  =  "1"  /* Occlusion of Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DL94ZZ"  =  "1"  /* Occlusion of Duodenum, Percutaneous Endoscopic Approach */
"0DL97DZ"  =  "1"  /* Occlusion of Duodenum with Intraluminal Device, Via Natural or Artificial Opening */
"0DL97ZZ"  =  "1"  /* Occlusion of Duodenum, Via Natural or Artificial Opening */
"0DL98DZ"  =  "1"  /* Occlusion of Duodenum with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DL98ZZ"  =  "1"  /* Occlusion of Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DLA0CZ"  =  "1"  /* Occlusion of Jejunum with Extraluminal Device, Open Approach */
"0DLA0DZ"  =  "1"  /* Occlusion of Jejunum with Intraluminal Device, Open Approach */
"0DLA0ZZ"  =  "1"  /* Occlusion of Jejunum, Open Approach */
"0DLA3CZ"  =  "1"  /* Occlusion of Jejunum with Extraluminal Device, Percutaneous Approach */
"0DLA3DZ"  =  "1"  /* Occlusion of Jejunum with Intraluminal Device, Percutaneous Approach */
"0DLA3ZZ"  =  "1"  /* Occlusion of Jejunum, Percutaneous Approach */
"0DLA4CZ"  =  "1"  /* Occlusion of Jejunum with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLA4DZ"  =  "1"  /* Occlusion of Jejunum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLA4ZZ"  =  "1"  /* Occlusion of Jejunum, Percutaneous Endoscopic Approach */
"0DLA7DZ"  =  "1"  /* Occlusion of Jejunum with Intraluminal Device, Via Natural or Artificial Opening */
"0DLA7ZZ"  =  "1"  /* Occlusion of Jejunum, Via Natural or Artificial Opening */
"0DLA8DZ"  =  "1"  /* Occlusion of Jejunum with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLA8ZZ"  =  "1"  /* Occlusion of Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DLB0CZ"  =  "1"  /* Occlusion of Ileum with Extraluminal Device, Open Approach */
"0DLB0DZ"  =  "1"  /* Occlusion of Ileum with Intraluminal Device, Open Approach */
"0DLB0ZZ"  =  "1"  /* Occlusion of Ileum, Open Approach */
"0DLB3CZ"  =  "1"  /* Occlusion of Ileum with Extraluminal Device, Percutaneous Approach */
"0DLB3DZ"  =  "1"  /* Occlusion of Ileum with Intraluminal Device, Percutaneous Approach */
"0DLB3ZZ"  =  "1"  /* Occlusion of Ileum, Percutaneous Approach */
"0DLB4CZ"  =  "1"  /* Occlusion of Ileum with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLB4DZ"  =  "1"  /* Occlusion of Ileum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLB4ZZ"  =  "1"  /* Occlusion of Ileum, Percutaneous Endoscopic Approach */
"0DLB7DZ"  =  "1"  /* Occlusion of Ileum with Intraluminal Device, Via Natural or Artificial Opening */
"0DLB7ZZ"  =  "1"  /* Occlusion of Ileum, Via Natural or Artificial Opening */
"0DLB8DZ"  =  "1"  /* Occlusion of Ileum with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLB8ZZ"  =  "1"  /* Occlusion of Ileum, Via Natural or Artificial Opening Endoscopic */
"0DLC0CZ"  =  "1"  /* Occlusion of Ileocecal Valve with Extraluminal Device, Open Approach */
"0DLC0DZ"  =  "1"  /* Occlusion of Ileocecal Valve with Intraluminal Device, Open Approach */
"0DLC0ZZ"  =  "1"  /* Occlusion of Ileocecal Valve, Open Approach */
"0DLC3CZ"  =  "1"  /* Occlusion of Ileocecal Valve with Extraluminal Device, Percutaneous Approach */
"0DLC3DZ"  =  "1"  /* Occlusion of Ileocecal Valve with Intraluminal Device, Percutaneous Approach */
"0DLC3ZZ"  =  "1"  /* Occlusion of Ileocecal Valve, Percutaneous Approach */
"0DLC4CZ"  =  "1"  /* Occlusion of Ileocecal Valve with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLC4DZ"  =  "1"  /* Occlusion of Ileocecal Valve with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLC4ZZ"  =  "1"  /* Occlusion of Ileocecal Valve, Percutaneous Endoscopic Approach */
"0DLC7DZ"  =  "1"  /* Occlusion of Ileocecal Valve with Intraluminal Device, Via Natural or Artificial Opening */
"0DLC7ZZ"  =  "1"  /* Occlusion of Ileocecal Valve, Via Natural or Artificial Opening */
"0DLC8DZ"  =  "1"  /* Occlusion of Ileocecal Valve with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLC8ZZ"  =  "1"  /* Occlusion of Ileocecal Valve, Via Natural or Artificial Opening Endoscopic */
"0DLE0CZ"  =  "1"  /* Occlusion of Large Intestine with Extraluminal Device, Open Approach */
"0DLE0DZ"  =  "1"  /* Occlusion of Large Intestine with Intraluminal Device, Open Approach */
"0DLE0ZZ"  =  "1"  /* Occlusion of Large Intestine, Open Approach */
"0DLE3CZ"  =  "1"  /* Occlusion of Large Intestine with Extraluminal Device, Percutaneous Approach */
"0DLE3DZ"  =  "1"  /* Occlusion of Large Intestine with Intraluminal Device, Percutaneous Approach */
"0DLE3ZZ"  =  "1"  /* Occlusion of Large Intestine, Percutaneous Approach */
"0DLE4CZ"  =  "1"  /* Occlusion of Large Intestine with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLE4DZ"  =  "1"  /* Occlusion of Large Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLE4ZZ"  =  "1"  /* Occlusion of Large Intestine, Percutaneous Endoscopic Approach */
"0DLE7DZ"  =  "1"  /* Occlusion of Large Intestine with Intraluminal Device, Via Natural or Artificial Opening */
"0DLE7ZZ"  =  "1"  /* Occlusion of Large Intestine, Via Natural or Artificial Opening */
"0DLE8DZ"  =  "1"  /* Occlusion of Large Intestine with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLE8ZZ"  =  "1"  /* Occlusion of Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DLF0CZ"  =  "1"  /* Occlusion of Right Large Intestine with Extraluminal Device, Open Approach */
"0DLF0DZ"  =  "1"  /* Occlusion of Right Large Intestine with Intraluminal Device, Open Approach */
"0DLF0ZZ"  =  "1"  /* Occlusion of Right Large Intestine, Open Approach */
"0DLF3CZ"  =  "1"  /* Occlusion of Right Large Intestine with Extraluminal Device, Percutaneous Approach */
"0DLF3DZ"  =  "1"  /* Occlusion of Right Large Intestine with Intraluminal Device, Percutaneous Approach */
"0DLF3ZZ"  =  "1"  /* Occlusion of Right Large Intestine, Percutaneous Approach */
"0DLF4CZ"  =  "1"  /* Occlusion of Right Large Intestine with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLF4DZ"  =  "1"  /* Occlusion of Right Large Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLF4ZZ"  =  "1"  /* Occlusion of Right Large Intestine, Percutaneous Endoscopic Approach */
"0DLF7DZ"  =  "1"  /* Occlusion of Right Large Intestine with Intraluminal Device, Via Natural or Artificial Opening */
"0DLF7ZZ"  =  "1"  /* Occlusion of Right Large Intestine, Via Natural or Artificial Opening */
"0DLF8DZ"  =  "1"  /* Occlusion of Right Large Intestine with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLF8ZZ"  =  "1"  /* Occlusion of Right Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DLG0CZ"  =  "1"  /* Occlusion of Left Large Intestine with Extraluminal Device, Open Approach */
"0DLG0DZ"  =  "1"  /* Occlusion of Left Large Intestine with Intraluminal Device, Open Approach */
"0DLG0ZZ"  =  "1"  /* Occlusion of Left Large Intestine, Open Approach */
"0DLG3CZ"  =  "1"  /* Occlusion of Left Large Intestine with Extraluminal Device, Percutaneous Approach */
"0DLG3DZ"  =  "1"  /* Occlusion of Left Large Intestine with Intraluminal Device, Percutaneous Approach */
"0DLG3ZZ"  =  "1"  /* Occlusion of Left Large Intestine, Percutaneous Approach */
"0DLG4CZ"  =  "1"  /* Occlusion of Left Large Intestine with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLG4DZ"  =  "1"  /* Occlusion of Left Large Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLG4ZZ"  =  "1"  /* Occlusion of Left Large Intestine, Percutaneous Endoscopic Approach */
"0DLG7DZ"  =  "1"  /* Occlusion of Left Large Intestine with Intraluminal Device, Via Natural or Artificial Opening */
"0DLG7ZZ"  =  "1"  /* Occlusion of Left Large Intestine, Via Natural or Artificial Opening */
"0DLG8DZ"  =  "1"  /* Occlusion of Left Large Intestine with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLG8ZZ"  =  "1"  /* Occlusion of Left Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DLH0CZ"  =  "1"  /* Occlusion of Cecum with Extraluminal Device, Open Approach */
"0DLH0DZ"  =  "1"  /* Occlusion of Cecum with Intraluminal Device, Open Approach */
"0DLH0ZZ"  =  "1"  /* Occlusion of Cecum, Open Approach */
"0DLH3CZ"  =  "1"  /* Occlusion of Cecum with Extraluminal Device, Percutaneous Approach */
"0DLH3DZ"  =  "1"  /* Occlusion of Cecum with Intraluminal Device, Percutaneous Approach */
"0DLH3ZZ"  =  "1"  /* Occlusion of Cecum, Percutaneous Approach */
"0DLH4CZ"  =  "1"  /* Occlusion of Cecum with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLH4DZ"  =  "1"  /* Occlusion of Cecum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLH4ZZ"  =  "1"  /* Occlusion of Cecum, Percutaneous Endoscopic Approach */
"0DLH7DZ"  =  "1"  /* Occlusion of Cecum with Intraluminal Device, Via Natural or Artificial Opening */
"0DLH7ZZ"  =  "1"  /* Occlusion of Cecum, Via Natural or Artificial Opening */
"0DLH8DZ"  =  "1"  /* Occlusion of Cecum with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLH8ZZ"  =  "1"  /* Occlusion of Cecum, Via Natural or Artificial Opening Endoscopic */
"0DLK0CZ"  =  "1"  /* Occlusion of Ascending Colon with Extraluminal Device, Open Approach */
"0DLK0DZ"  =  "1"  /* Occlusion of Ascending Colon with Intraluminal Device, Open Approach */
"0DLK0ZZ"  =  "1"  /* Occlusion of Ascending Colon, Open Approach */
"0DLK3CZ"  =  "1"  /* Occlusion of Ascending Colon with Extraluminal Device, Percutaneous Approach */
"0DLK3DZ"  =  "1"  /* Occlusion of Ascending Colon with Intraluminal Device, Percutaneous Approach */
"0DLK3ZZ"  =  "1"  /* Occlusion of Ascending Colon, Percutaneous Approach */
"0DLK4CZ"  =  "1"  /* Occlusion of Ascending Colon with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLK4DZ"  =  "1"  /* Occlusion of Ascending Colon with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLK4ZZ"  =  "1"  /* Occlusion of Ascending Colon, Percutaneous Endoscopic Approach */
"0DLK7DZ"  =  "1"  /* Occlusion of Ascending Colon with Intraluminal Device, Via Natural or Artificial Opening */
"0DLK7ZZ"  =  "1"  /* Occlusion of Ascending Colon, Via Natural or Artificial Opening */
"0DLK8DZ"  =  "1"  /* Occlusion of Ascending Colon with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLK8ZZ"  =  "1"  /* Occlusion of Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0DLL0CZ"  =  "1"  /* Occlusion of Transverse Colon with Extraluminal Device, Open Approach */
"0DLL0DZ"  =  "1"  /* Occlusion of Transverse Colon with Intraluminal Device, Open Approach */
"0DLL0ZZ"  =  "1"  /* Occlusion of Transverse Colon, Open Approach */
"0DLL3CZ"  =  "1"  /* Occlusion of Transverse Colon with Extraluminal Device, Percutaneous Approach */
"0DLL3DZ"  =  "1"  /* Occlusion of Transverse Colon with Intraluminal Device, Percutaneous Approach */
"0DLL3ZZ"  =  "1"  /* Occlusion of Transverse Colon, Percutaneous Approach */
"0DLL4CZ"  =  "1"  /* Occlusion of Transverse Colon with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLL4DZ"  =  "1"  /* Occlusion of Transverse Colon with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLL4ZZ"  =  "1"  /* Occlusion of Transverse Colon, Percutaneous Endoscopic Approach */
"0DLL7DZ"  =  "1"  /* Occlusion of Transverse Colon with Intraluminal Device, Via Natural or Artificial Opening */
"0DLL7ZZ"  =  "1"  /* Occlusion of Transverse Colon, Via Natural or Artificial Opening */
"0DLL8DZ"  =  "1"  /* Occlusion of Transverse Colon with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLL8ZZ"  =  "1"  /* Occlusion of Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0DLM0CZ"  =  "1"  /* Occlusion of Descending Colon with Extraluminal Device, Open Approach */
"0DLM0DZ"  =  "1"  /* Occlusion of Descending Colon with Intraluminal Device, Open Approach */
"0DLM0ZZ"  =  "1"  /* Occlusion of Descending Colon, Open Approach */
"0DLM3CZ"  =  "1"  /* Occlusion of Descending Colon with Extraluminal Device, Percutaneous Approach */
"0DLM3DZ"  =  "1"  /* Occlusion of Descending Colon with Intraluminal Device, Percutaneous Approach */
"0DLM3ZZ"  =  "1"  /* Occlusion of Descending Colon, Percutaneous Approach */
"0DLM4CZ"  =  "1"  /* Occlusion of Descending Colon with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLM4DZ"  =  "1"  /* Occlusion of Descending Colon with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLM4ZZ"  =  "1"  /* Occlusion of Descending Colon, Percutaneous Endoscopic Approach */
"0DLM7DZ"  =  "1"  /* Occlusion of Descending Colon with Intraluminal Device, Via Natural or Artificial Opening */
"0DLM7ZZ"  =  "1"  /* Occlusion of Descending Colon, Via Natural or Artificial Opening */
"0DLM8DZ"  =  "1"  /* Occlusion of Descending Colon with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLM8ZZ"  =  "1"  /* Occlusion of Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0DLN0CZ"  =  "1"  /* Occlusion of Sigmoid Colon with Extraluminal Device, Open Approach */
"0DLN0DZ"  =  "1"  /* Occlusion of Sigmoid Colon with Intraluminal Device, Open Approach */
"0DLN0ZZ"  =  "1"  /* Occlusion of Sigmoid Colon, Open Approach */
"0DLN3CZ"  =  "1"  /* Occlusion of Sigmoid Colon with Extraluminal Device, Percutaneous Approach */
"0DLN3DZ"  =  "1"  /* Occlusion of Sigmoid Colon with Intraluminal Device, Percutaneous Approach */
"0DLN3ZZ"  =  "1"  /* Occlusion of Sigmoid Colon, Percutaneous Approach */
"0DLN4CZ"  =  "1"  /* Occlusion of Sigmoid Colon with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLN4DZ"  =  "1"  /* Occlusion of Sigmoid Colon with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLN4ZZ"  =  "1"  /* Occlusion of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DLN7DZ"  =  "1"  /* Occlusion of Sigmoid Colon with Intraluminal Device, Via Natural or Artificial Opening */
"0DLN7ZZ"  =  "1"  /* Occlusion of Sigmoid Colon, Via Natural or Artificial Opening */
"0DLN8DZ"  =  "1"  /* Occlusion of Sigmoid Colon with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLN8ZZ"  =  "1"  /* Occlusion of Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0DM60ZZ"  =  "1"  /* Reattachment of Stomach, Open Approach */
"0DM64ZZ"  =  "1"  /* Reattachment of Stomach, Percutaneous Endoscopic Approach */
"0DM80ZZ"  =  "1"  /* Reattachment of Small Intestine, Open Approach */
"0DM84ZZ"  =  "1"  /* Reattachment of Small Intestine, Percutaneous Endoscopic Approach */
"0DM90ZZ"  =  "1"  /* Reattachment of Duodenum, Open Approach */
"0DM94ZZ"  =  "1"  /* Reattachment of Duodenum, Percutaneous Endoscopic Approach */
"0DMA0ZZ"  =  "1"  /* Reattachment of Jejunum, Open Approach */
"0DMA4ZZ"  =  "1"  /* Reattachment of Jejunum, Percutaneous Endoscopic Approach */
"0DMB0ZZ"  =  "1"  /* Reattachment of Ileum, Open Approach */
"0DMB4ZZ"  =  "1"  /* Reattachment of Ileum, Percutaneous Endoscopic Approach */
"0DME0ZZ"  =  "1"  /* Reattachment of Large Intestine, Open Approach */
"0DME4ZZ"  =  "1"  /* Reattachment of Large Intestine, Percutaneous Endoscopic Approach */
"0DMF0ZZ"  =  "1"  /* Reattachment of Right Large Intestine, Open Approach */
"0DMF4ZZ"  =  "1"  /* Reattachment of Right Large Intestine, Percutaneous Endoscopic Approach */
"0DMG0ZZ"  =  "1"  /* Reattachment of Left Large Intestine, Open Approach */
"0DMG4ZZ"  =  "1"  /* Reattachment of Left Large Intestine, Percutaneous Endoscopic Approach */
"0DMH0ZZ"  =  "1"  /* Reattachment of Cecum, Open Approach */
"0DMH4ZZ"  =  "1"  /* Reattachment of Cecum, Percutaneous Endoscopic Approach */
"0DMK0ZZ"  =  "1"  /* Reattachment of Ascending Colon, Open Approach */
"0DMK4ZZ"  =  "1"  /* Reattachment of Ascending Colon, Percutaneous Endoscopic Approach */
"0DML0ZZ"  =  "1"  /* Reattachment of Transverse Colon, Open Approach */
"0DML4ZZ"  =  "1"  /* Reattachment of Transverse Colon, Percutaneous Endoscopic Approach */
"0DMM0ZZ"  =  "1"  /* Reattachment of Descending Colon, Open Approach */
"0DMM4ZZ"  =  "1"  /* Reattachment of Descending Colon, Percutaneous Endoscopic Approach */
"0DMN0ZZ"  =  "1"  /* Reattachment of Sigmoid Colon, Open Approach */
"0DMN4ZZ"  =  "1"  /* Reattachment of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DMP0ZZ"  =  "1"  /* Reattachment of Rectum, Open Approach */
"0DMP4ZZ"  =  "1"  /* Reattachment of Rectum, Percutaneous Endoscopic Approach */
"0DN60ZZ"  =  "1"  /* Release Stomach, Open Approach */
"0DN63ZZ"  =  "1"  /* Release Stomach, Percutaneous Approach */
"0DN64ZZ"  =  "1"  /* Release Stomach, Percutaneous Endoscopic Approach */
"0DN67ZZ"  =  "1"  /* Release Stomach, Via Natural or Artificial Opening */
"0DN68ZZ"  =  "1"  /* Release Stomach, Via Natural or Artificial Opening Endoscopic */
"0DN70ZZ"  =  "1"  /* Release Stomach, Pylorus, Open Approach */
"0DN73ZZ"  =  "1"  /* Release Stomach, Pylorus, Percutaneous Approach */
"0DN74ZZ"  =  "1"  /* Release Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0DN77ZZ"  =  "1"  /* Release Stomach, Pylorus, Via Natural or Artificial Opening */
"0DN78ZZ"  =  "1"  /* Release Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0DN80ZZ"  =  "1"  /* Release Small Intestine, Open Approach */
"0DN83ZZ"  =  "1"  /* Release Small Intestine, Percutaneous Approach */
"0DN84ZZ"  =  "1"  /* Release Small Intestine, Percutaneous Endoscopic Approach */
"0DN90ZZ"  =  "1"  /* Release Duodenum, Open Approach */
"0DN93ZZ"  =  "1"  /* Release Duodenum, Percutaneous Approach */
"0DN94ZZ"  =  "1"  /* Release Duodenum, Percutaneous Endoscopic Approach */
"0DNA0ZZ"  =  "1"  /* Release Jejunum, Open Approach */
"0DNA3ZZ"  =  "1"  /* Release Jejunum, Percutaneous Approach */
"0DNA4ZZ"  =  "1"  /* Release Jejunum, Percutaneous Endoscopic Approach */
"0DNB0ZZ"  =  "1"  /* Release Ileum, Open Approach */
"0DNB3ZZ"  =  "1"  /* Release Ileum, Percutaneous Approach */
"0DNB4ZZ"  =  "1"  /* Release Ileum, Percutaneous Endoscopic Approach */
"0DNC0ZZ"  =  "1"  /* Release Ileocecal Valve, Open Approach */
"0DNC3ZZ"  =  "1"  /* Release Ileocecal Valve, Percutaneous Approach */
"0DNC4ZZ"  =  "1"  /* Release Ileocecal Valve, Percutaneous Endoscopic Approach */
"0DNC7ZZ"  =  "1"  /* Release Ileocecal Valve, Via Natural or Artificial Opening */
"0DNC8ZZ"  =  "1"  /* Release Ileocecal Valve, Via Natural or Artificial Opening Endoscopic */
"0DNE0ZZ"  =  "1"  /* Release Large Intestine, Open Approach */
"0DNE3ZZ"  =  "1"  /* Release Large Intestine, Percutaneous Approach */
"0DNE4ZZ"  =  "1"  /* Release Large Intestine, Percutaneous Endoscopic Approach */
"0DNF0ZZ"  =  "1"  /* Release Right Large Intestine, Open Approach */
"0DNF3ZZ"  =  "1"  /* Release Right Large Intestine, Percutaneous Approach */
"0DNF4ZZ"  =  "1"  /* Release Right Large Intestine, Percutaneous Endoscopic Approach */
"0DNG0ZZ"  =  "1"  /* Release Left Large Intestine, Open Approach */
"0DNG3ZZ"  =  "1"  /* Release Left Large Intestine, Percutaneous Approach */
"0DNG4ZZ"  =  "1"  /* Release Left Large Intestine, Percutaneous Endoscopic Approach */
"0DNH0ZZ"  =  "1"  /* Release Cecum, Open Approach */
"0DNH3ZZ"  =  "1"  /* Release Cecum, Percutaneous Approach */
"0DNH4ZZ"  =  "1"  /* Release Cecum, Percutaneous Endoscopic Approach */
"0DNJ0ZZ"  =  "1"  /* Release Appendix, Open Approach */
"0DNJ3ZZ"  =  "1"  /* Release Appendix, Percutaneous Approach */
"0DNJ4ZZ"  =  "1"  /* Release Appendix, Percutaneous Endoscopic Approach */
"0DNK0ZZ"  =  "1"  /* Release Ascending Colon, Open Approach */
"0DNK3ZZ"  =  "1"  /* Release Ascending Colon, Percutaneous Approach */
"0DNK4ZZ"  =  "1"  /* Release Ascending Colon, Percutaneous Endoscopic Approach */
"0DNL0ZZ"  =  "1"  /* Release Transverse Colon, Open Approach */
"0DNL3ZZ"  =  "1"  /* Release Transverse Colon, Percutaneous Approach */
"0DNL4ZZ"  =  "1"  /* Release Transverse Colon, Percutaneous Endoscopic Approach */
"0DNM0ZZ"  =  "1"  /* Release Descending Colon, Open Approach */
"0DNM3ZZ"  =  "1"  /* Release Descending Colon, Percutaneous Approach */
"0DNM4ZZ"  =  "1"  /* Release Descending Colon, Percutaneous Endoscopic Approach */
"0DNN0ZZ"  =  "1"  /* Release Sigmoid Colon, Open Approach */
"0DNN3ZZ"  =  "1"  /* Release Sigmoid Colon, Percutaneous Approach */
"0DNN4ZZ"  =  "1"  /* Release Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DNS0ZZ"  =  "1"  /* Release Greater Omentum, Open Approach */
"0DNS3ZZ"  =  "1"  /* Release Greater Omentum, Percutaneous Approach */
"0DNS4ZZ"  =  "1"  /* Release Greater Omentum, Percutaneous Endoscopic Approach */
"0DNT0ZZ"  =  "1"  /* Release Lesser Omentum, Open Approach */
"0DNT3ZZ"  =  "1"  /* Release Lesser Omentum, Percutaneous Approach */
"0DNT4ZZ"  =  "1"  /* Release Lesser Omentum, Percutaneous Endoscopic Approach */
"0DNV0ZZ"  =  "1"  /* Release Mesentery, Open Approach */
"0DNV3ZZ"  =  "1"  /* Release Mesentery, Percutaneous Approach */
"0DNV4ZZ"  =  "1"  /* Release Mesentery, Percutaneous Endoscopic Approach */
"0DNW0ZZ"  =  "1"  /* Release Peritoneum, Open Approach */
"0DNW3ZZ"  =  "1"  /* Release Peritoneum, Percutaneous Approach */
"0DNW4ZZ"  =  "1"  /* Release Peritoneum, Percutaneous Endoscopic Approach */
"0DP000Z"  =  "1"  /* Removal of Drainage Device from Upper Intestinal Tract, Open Approach */
"0DP002Z"  =  "1"  /* Removal of Monitoring Device from Upper Intestinal Tract, Open Approach */
"0DP003Z"  =  "1"  /* Removal of Infusion Device from Upper Intestinal Tract, Open Approach */
"0DP007Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Open Approach */
"0DP00CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Intestinal Tract, Open Approach */
"0DP00DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Intestinal Tract, Open Approach */
"0DP00JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Intestinal Tract, Open Approach */
"0DP00KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Open Approach */
"0DP00UZ"  =  "1"  /* Removal of Feeding Device from Upper Intestinal Tract, Open Approach */
"0DP030Z"  =  "1"  /* Removal of Drainage Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP032Z"  =  "1"  /* Removal of Monitoring Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP033Z"  =  "1"  /* Removal of Infusion Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP037Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Percutaneous Approach */
"0DP03CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP03DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP03JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Intestinal Tract, Percutaneous Approach */
"0DP03KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Percutaneous Approach */
"0DP03UZ"  =  "1"  /* Removal of Feeding Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP040Z"  =  "1"  /* Removal of Drainage Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP042Z"  =  "1"  /* Removal of Monitoring Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP043Z"  =  "1"  /* Removal of Infusion Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP047Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP04CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP04DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP04JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP04KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP04UZ"  =  "1"  /* Removal of Feeding Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP070Z"  =  "1"  /* Removal of Drainage Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP072Z"  =  "1"  /* Removal of Monitoring Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP073Z"  =  "1"  /* Removal of Infusion Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP077Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP07CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP07DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP07JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP07KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP07UZ"  =  "1"  /* Removal of Feeding Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP080Z"  =  "1"  /* Removal of Drainage Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP082Z"  =  "1"  /* Removal of Monitoring Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP083Z"  =  "1"  /* Removal of Infusion Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP087Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP08CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP08DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP08JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP08KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP08UZ"  =  "1"  /* Removal of Feeding Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP600Z"  =  "1"  /* Removal of Drainage Device from Stomach, Open Approach */
"0DP602Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Open Approach */
"0DP603Z"  =  "1"  /* Removal of Infusion Device from Stomach, Open Approach */
"0DP607Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Stomach, Open Approach */
"0DP60CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Open Approach */
"0DP60DZ"  =  "1"  /* Removal of Intraluminal Device from Stomach, Open Approach */
"0DP60JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Open Approach */
"0DP60KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Stomach, Open Approach */
"0DP60UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Open Approach */
"0DP630Z"  =  "1"  /* Removal of Drainage Device from Stomach, Percutaneous Approach */
"0DP632Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Percutaneous Approach */
"0DP633Z"  =  "1"  /* Removal of Infusion Device from Stomach, Percutaneous Approach */
"0DP637Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Stomach, Percutaneous Approach */
"0DP63CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Percutaneous Approach */
"0DP63DZ"  =  "1"  /* Removal of Intraluminal Device from Stomach, Percutaneous Approach */
"0DP63JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Percutaneous Approach */
"0DP63KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Stomach, Percutaneous Approach */
"0DP63UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Percutaneous Approach */
"0DP640Z"  =  "1"  /* Removal of Drainage Device from Stomach, Percutaneous Endoscopic Approach */
"0DP642Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Percutaneous Endoscopic Approach */
"0DP643Z"  =  "1"  /* Removal of Infusion Device from Stomach, Percutaneous Endoscopic Approach */
"0DP647Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Stomach, Percutaneous Endoscopic Approach */
"0DP64CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Percutaneous Endoscopic Approach */
"0DP64DZ"  =  "1"  /* Removal of Intraluminal Device from Stomach, Percutaneous Endoscopic Approach */
"0DP64JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Percutaneous Endoscopic Approach */
"0DP64KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Stomach, Percutaneous Endoscopic Approach */
"0DP64UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Percutaneous Endoscopic Approach */
"0DP670Z"  =  "1"  /* Removal of Drainage Device from Stomach, Via Natural or Artificial Opening */
"0DP672Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Via Natural or Artificial Opening */
"0DP673Z"  =  "1"  /* Removal of Infusion Device from Stomach, Via Natural or Artificial Opening */
"0DP677Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Stomach, Via Natural or Artificial Opening */
"0DP67CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Via Natural or Artificial Opening */
"0DP67JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Via Natural or Artificial Opening */
"0DP67KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Stomach, Via Natural or Artificial Opening */
"0DP67UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Via Natural or Artificial Opening */
"0DP680Z"  =  "1"  /* Removal of Drainage Device from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP682Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP683Z"  =  "1"  /* Removal of Infusion Device from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP687Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP68CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP68JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP68KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP68UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DPD00Z"  =  "1"  /* Removal of Drainage Device from Lower Intestinal Tract, Open Approach */
"0DPD02Z"  =  "1"  /* Removal of Monitoring Device from Lower Intestinal Tract, Open Approach */
"0DPD03Z"  =  "1"  /* Removal of Infusion Device from Lower Intestinal Tract, Open Approach */
"0DPD07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Open Approach */
"0DPD0CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Intestinal Tract, Open Approach */
"0DPD0DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Intestinal Tract, Open Approach */
"0DPD0JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Intestinal Tract, Open Approach */
"0DPD0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Open Approach */
"0DPD0UZ"  =  "1"  /* Removal of Feeding Device from Lower Intestinal Tract, Open Approach */
"0DPD30Z"  =  "1"  /* Removal of Drainage Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD32Z"  =  "1"  /* Removal of Monitoring Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD33Z"  =  "1"  /* Removal of Infusion Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Percutaneous Approach */
"0DPD3CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD3DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD3JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Intestinal Tract, Percutaneous Approach */
"0DPD3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Percutaneous Approach */
"0DPD3UZ"  =  "1"  /* Removal of Feeding Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD40Z"  =  "1"  /* Removal of Drainage Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD42Z"  =  "1"  /* Removal of Monitoring Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD43Z"  =  "1"  /* Removal of Infusion Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD4CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD4DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD4JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD4UZ"  =  "1"  /* Removal of Feeding Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD70Z"  =  "1"  /* Removal of Drainage Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD72Z"  =  "1"  /* Removal of Monitoring Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD73Z"  =  "1"  /* Removal of Infusion Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD7CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD7DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD7JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD7UZ"  =  "1"  /* Removal of Feeding Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD80Z"  =  "1"  /* Removal of Drainage Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD82Z"  =  "1"  /* Removal of Monitoring Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD83Z"  =  "1"  /* Removal of Infusion Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD8CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD8DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD8JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD8UZ"  =  "1"  /* Removal of Feeding Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPU00Z"  =  "1"  /* Removal of Drainage Device from Omentum, Open Approach */
"0DPU01Z"  =  "1"  /* Removal of Radioactive Element from Omentum, Open Approach */
"0DPU07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Omentum, Open Approach */
"0DPU0JZ"  =  "1"  /* Removal of Synthetic Substitute from Omentum, Open Approach */
"0DPU0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Omentum, Open Approach */
"0DPU30Z"  =  "1"  /* Removal of Drainage Device from Omentum, Percutaneous Approach */
"0DPU31Z"  =  "1"  /* Removal of Radioactive Element from Omentum, Percutaneous Approach */
"0DPU37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Omentum, Percutaneous Approach */
"0DPU3JZ"  =  "1"  /* Removal of Synthetic Substitute from Omentum, Percutaneous Approach */
"0DPU3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Omentum, Percutaneous Approach */
"0DPU40Z"  =  "1"  /* Removal of Drainage Device from Omentum, Percutaneous Endoscopic Approach */
"0DPU41Z"  =  "1"  /* Removal of Radioactive Element from Omentum, Percutaneous Endoscopic Approach */
"0DPU47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Omentum, Percutaneous Endoscopic Approach */
"0DPU4JZ"  =  "1"  /* Removal of Synthetic Substitute from Omentum, Percutaneous Endoscopic Approach */
"0DPU4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Omentum, Percutaneous Endoscopic Approach */
"0DPV00Z"  =  "1"  /* Removal of Drainage Device from Mesentery, Open Approach */
"0DPV01Z"  =  "1"  /* Removal of Radioactive Element from Mesentery, Open Approach */
"0DPV07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mesentery, Open Approach */
"0DPV0JZ"  =  "1"  /* Removal of Synthetic Substitute from Mesentery, Open Approach */
"0DPV0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mesentery, Open Approach */
"0DPV30Z"  =  "1"  /* Removal of Drainage Device from Mesentery, Percutaneous Approach */
"0DPV31Z"  =  "1"  /* Removal of Radioactive Element from Mesentery, Percutaneous Approach */
"0DPV37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mesentery, Percutaneous Approach */
"0DPV3JZ"  =  "1"  /* Removal of Synthetic Substitute from Mesentery, Percutaneous Approach */
"0DPV3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mesentery, Percutaneous Approach */
"0DPV40Z"  =  "1"  /* Removal of Drainage Device from Mesentery, Percutaneous Endoscopic Approach */
"0DPV41Z"  =  "1"  /* Removal of Radioactive Element from Mesentery, Percutaneous Endoscopic Approach */
"0DPV47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mesentery, Percutaneous Endoscopic Approach */
"0DPV4JZ"  =  "1"  /* Removal of Synthetic Substitute from Mesentery, Percutaneous Endoscopic Approach */
"0DPV4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mesentery, Percutaneous Endoscopic Approach */
"0DPW00Z"  =  "1"  /* Removal of Drainage Device from Peritoneum, Open Approach */
"0DPW01Z"  =  "1"  /* Removal of Radioactive Element from Peritoneum, Open Approach */
"0DPW07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Peritoneum, Open Approach */
"0DPW0JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneum, Open Approach */
"0DPW0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Peritoneum, Open Approach */
"0DPW30Z"  =  "1"  /* Removal of Drainage Device from Peritoneum, Percutaneous Approach */
"0DPW31Z"  =  "1"  /* Removal of Radioactive Element from Peritoneum, Percutaneous Approach */
"0DPW37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Peritoneum, Percutaneous Approach */
"0DPW3JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneum, Percutaneous Approach */
"0DPW3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Peritoneum, Percutaneous Approach */
"0DPW40Z"  =  "1"  /* Removal of Drainage Device from Peritoneum, Percutaneous Endoscopic Approach */
"0DPW41Z"  =  "1"  /* Removal of Radioactive Element from Peritoneum, Percutaneous Endoscopic Approach */
"0DPW47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Peritoneum, Percutaneous Endoscopic Approach */
"0DPW4JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneum, Percutaneous Endoscopic Approach */
"0DPW4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Peritoneum, Percutaneous Endoscopic Approach */
"0DQ40ZZ"  =  "1"  /* Repair Esophagogastric Junction, Open Approach */
"0DQ43ZZ"  =  "1"  /* Repair Esophagogastric Junction, Percutaneous Approach */
"0DQ44ZZ"  =  "1"  /* Repair Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0DQ47ZZ"  =  "1"  /* Repair Esophagogastric Junction, Via Natural or Artificial Opening */
"0DQ48ZZ"  =  "1"  /* Repair Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic */
"0DQ60ZZ"  =  "1"  /* Repair Stomach, Open Approach */
"0DQ63ZZ"  =  "1"  /* Repair Stomach, Percutaneous Approach */
"0DQ64ZZ"  =  "1"  /* Repair Stomach, Percutaneous Endoscopic Approach */
"0DQ67ZZ"  =  "1"  /* Repair Stomach, Via Natural or Artificial Opening */
"0DQ68ZZ"  =  "1"  /* Repair Stomach, Via Natural or Artificial Opening Endoscopic */
"0DQ70ZZ"  =  "1"  /* Repair Stomach, Pylorus, Open Approach */
"0DQ73ZZ"  =  "1"  /* Repair Stomach, Pylorus, Percutaneous Approach */
"0DQ74ZZ"  =  "1"  /* Repair Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0DQ77ZZ"  =  "1"  /* Repair Stomach, Pylorus, Via Natural or Artificial Opening */
"0DQ78ZZ"  =  "1"  /* Repair Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0DQ80ZZ"  =  "1"  /* Repair Small Intestine, Open Approach */
"0DQ83ZZ"  =  "1"  /* Repair Small Intestine, Percutaneous Approach */
"0DQ84ZZ"  =  "1"  /* Repair Small Intestine, Percutaneous Endoscopic Approach */
"0DQ87ZZ"  =  "1"  /* Repair Small Intestine, Via Natural or Artificial Opening */
"0DQ88ZZ"  =  "1"  /* Repair Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DQ90ZZ"  =  "1"  /* Repair Duodenum, Open Approach */
"0DQ93ZZ"  =  "1"  /* Repair Duodenum, Percutaneous Approach */
"0DQ94ZZ"  =  "1"  /* Repair Duodenum, Percutaneous Endoscopic Approach */
"0DQ97ZZ"  =  "1"  /* Repair Duodenum, Via Natural or Artificial Opening */
"0DQ98ZZ"  =  "1"  /* Repair Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DQA0ZZ"  =  "1"  /* Repair Jejunum, Open Approach */
"0DQA3ZZ"  =  "1"  /* Repair Jejunum, Percutaneous Approach */
"0DQA4ZZ"  =  "1"  /* Repair Jejunum, Percutaneous Endoscopic Approach */
"0DQA7ZZ"  =  "1"  /* Repair Jejunum, Via Natural or Artificial Opening */
"0DQA8ZZ"  =  "1"  /* Repair Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DQB0ZZ"  =  "1"  /* Repair Ileum, Open Approach */
"0DQB3ZZ"  =  "1"  /* Repair Ileum, Percutaneous Approach */
"0DQB4ZZ"  =  "1"  /* Repair Ileum, Percutaneous Endoscopic Approach */
"0DQB7ZZ"  =  "1"  /* Repair Ileum, Via Natural or Artificial Opening */
"0DQB8ZZ"  =  "1"  /* Repair Ileum, Via Natural or Artificial Opening Endoscopic */
"0DQC0ZZ"  =  "1"  /* Repair Ileocecal Valve, Open Approach */
"0DQC3ZZ"  =  "1"  /* Repair Ileocecal Valve, Percutaneous Approach */
"0DQC4ZZ"  =  "1"  /* Repair Ileocecal Valve, Percutaneous Endoscopic Approach */
"0DQC7ZZ"  =  "1"  /* Repair Ileocecal Valve, Via Natural or Artificial Opening */
"0DQC8ZZ"  =  "1"  /* Repair Ileocecal Valve, Via Natural or Artificial Opening Endoscopic */
"0DQE0ZZ"  =  "1"  /* Repair Large Intestine, Open Approach */
"0DQE3ZZ"  =  "1"  /* Repair Large Intestine, Percutaneous Approach */
"0DQE4ZZ"  =  "1"  /* Repair Large Intestine, Percutaneous Endoscopic Approach */
"0DQE7ZZ"  =  "1"  /* Repair Large Intestine, Via Natural or Artificial Opening */
"0DQE8ZZ"  =  "1"  /* Repair Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DQF0ZZ"  =  "1"  /* Repair Right Large Intestine, Open Approach */
"0DQF3ZZ"  =  "1"  /* Repair Right Large Intestine, Percutaneous Approach */
"0DQF4ZZ"  =  "1"  /* Repair Right Large Intestine, Percutaneous Endoscopic Approach */
"0DQF7ZZ"  =  "1"  /* Repair Right Large Intestine, Via Natural or Artificial Opening */
"0DQF8ZZ"  =  "1"  /* Repair Right Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DQG0ZZ"  =  "1"  /* Repair Left Large Intestine, Open Approach */
"0DQG3ZZ"  =  "1"  /* Repair Left Large Intestine, Percutaneous Approach */
"0DQG4ZZ"  =  "1"  /* Repair Left Large Intestine, Percutaneous Endoscopic Approach */
"0DQG7ZZ"  =  "1"  /* Repair Left Large Intestine, Via Natural or Artificial Opening */
"0DQG8ZZ"  =  "1"  /* Repair Left Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DQH0ZZ"  =  "1"  /* Repair Cecum, Open Approach */
"0DQH3ZZ"  =  "1"  /* Repair Cecum, Percutaneous Approach */
"0DQH4ZZ"  =  "1"  /* Repair Cecum, Percutaneous Endoscopic Approach */
"0DQH7ZZ"  =  "1"  /* Repair Cecum, Via Natural or Artificial Opening */
"0DQH8ZZ"  =  "1"  /* Repair Cecum, Via Natural or Artificial Opening Endoscopic */
"0DQK0ZZ"  =  "1"  /* Repair Ascending Colon, Open Approach */
"0DQK3ZZ"  =  "1"  /* Repair Ascending Colon, Percutaneous Approach */
"0DQK4ZZ"  =  "1"  /* Repair Ascending Colon, Percutaneous Endoscopic Approach */
"0DQK7ZZ"  =  "1"  /* Repair Ascending Colon, Via Natural or Artificial Opening */
"0DQK8ZZ"  =  "1"  /* Repair Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0DQL0ZZ"  =  "1"  /* Repair Transverse Colon, Open Approach */
"0DQL3ZZ"  =  "1"  /* Repair Transverse Colon, Percutaneous Approach */
"0DQL4ZZ"  =  "1"  /* Repair Transverse Colon, Percutaneous Endoscopic Approach */
"0DQL7ZZ"  =  "1"  /* Repair Transverse Colon, Via Natural or Artificial Opening */
"0DQL8ZZ"  =  "1"  /* Repair Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0DQM0ZZ"  =  "1"  /* Repair Descending Colon, Open Approach */
"0DQM3ZZ"  =  "1"  /* Repair Descending Colon, Percutaneous Approach */
"0DQM4ZZ"  =  "1"  /* Repair Descending Colon, Percutaneous Endoscopic Approach */
"0DQM7ZZ"  =  "1"  /* Repair Descending Colon, Via Natural or Artificial Opening */
"0DQM8ZZ"  =  "1"  /* Repair Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0DQN0ZZ"  =  "1"  /* Repair Sigmoid Colon, Open Approach */
"0DQN3ZZ"  =  "1"  /* Repair Sigmoid Colon, Percutaneous Approach */
"0DQN4ZZ"  =  "1"  /* Repair Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DQN7ZZ"  =  "1"  /* Repair Sigmoid Colon, Via Natural or Artificial Opening */
"0DQN8ZZ"  =  "1"  /* Repair Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0DQP0ZZ"  =  "1"  /* Repair Rectum, Open Approach */
"0DQP3ZZ"  =  "1"  /* Repair Rectum, Percutaneous Approach */
"0DQP4ZZ"  =  "1"  /* Repair Rectum, Percutaneous Endoscopic Approach */
"0DQP7ZZ"  =  "1"  /* Repair Rectum, Via Natural or Artificial Opening */
"0DQP8ZZ"  =  "1"  /* Repair Rectum, Via Natural or Artificial Opening Endoscopic */
"0DQS0ZZ"  =  "1"  /* Repair Greater Omentum, Open Approach */
"0DQS3ZZ"  =  "1"  /* Repair Greater Omentum, Percutaneous Approach */
"0DQS4ZZ"  =  "1"  /* Repair Greater Omentum, Percutaneous Endoscopic Approach */
"0DQT0ZZ"  =  "1"  /* Repair Lesser Omentum, Open Approach */
"0DQT3ZZ"  =  "1"  /* Repair Lesser Omentum, Percutaneous Approach */
"0DQT4ZZ"  =  "1"  /* Repair Lesser Omentum, Percutaneous Endoscopic Approach */
"0DQV0ZZ"  =  "1"  /* Repair Mesentery, Open Approach */
"0DQV3ZZ"  =  "1"  /* Repair Mesentery, Percutaneous Approach */
"0DQV4ZZ"  =  "1"  /* Repair Mesentery, Percutaneous Endoscopic Approach */
"0DQW0ZZ"  =  "1"  /* Repair Peritoneum, Open Approach */
"0DQW3ZZ"  =  "1"  /* Repair Peritoneum, Percutaneous Approach */
"0DQW4ZZ"  =  "1"  /* Repair Peritoneum, Percutaneous Endoscopic Approach */
"0DRS07Z"  =  "1"  /* Replacement of Greater Omentum with Autologous Tissue Substitute, Open Approach */
"0DRS0JZ"  =  "1"  /* Replacement of Greater Omentum with Synthetic Substitute, Open Approach */
"0DRS0KZ"  =  "1"  /* Replacement of Greater Omentum with Nonautologous Tissue Substitute, Open Approach */
"0DRS47Z"  =  "1"  /* Replacement of Greater Omentum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DRS4JZ"  =  "1"  /* Replacement of Greater Omentum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DRS4KZ"  =  "1"  /* Replacement of Greater Omentum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DRT07Z"  =  "1"  /* Replacement of Lesser Omentum with Autologous Tissue Substitute, Open Approach */
"0DRT0JZ"  =  "1"  /* Replacement of Lesser Omentum with Synthetic Substitute, Open Approach */
"0DRT0KZ"  =  "1"  /* Replacement of Lesser Omentum with Nonautologous Tissue Substitute, Open Approach */
"0DRT47Z"  =  "1"  /* Replacement of Lesser Omentum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DRT4JZ"  =  "1"  /* Replacement of Lesser Omentum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DRT4KZ"  =  "1"  /* Replacement of Lesser Omentum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DRV07Z"  =  "1"  /* Replacement of Mesentery with Autologous Tissue Substitute, Open Approach */
"0DRV0JZ"  =  "1"  /* Replacement of Mesentery with Synthetic Substitute, Open Approach */
"0DRV0KZ"  =  "1"  /* Replacement of Mesentery with Nonautologous Tissue Substitute, Open Approach */
"0DRV47Z"  =  "1"  /* Replacement of Mesentery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DRV4JZ"  =  "1"  /* Replacement of Mesentery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DRV4KZ"  =  "1"  /* Replacement of Mesentery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DRW07Z"  =  "1"  /* Replacement of Peritoneum with Autologous Tissue Substitute, Open Approach */
"0DRW0JZ"  =  "1"  /* Replacement of Peritoneum with Synthetic Substitute, Open Approach */
"0DRW0KZ"  =  "1"  /* Replacement of Peritoneum with Nonautologous Tissue Substitute, Open Approach */
"0DRW47Z"  =  "1"  /* Replacement of Peritoneum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DRW4JZ"  =  "1"  /* Replacement of Peritoneum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DRW4KZ"  =  "1"  /* Replacement of Peritoneum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DS60ZZ"  =  "1"  /* Reposition Stomach, Open Approach */
"0DS64ZZ"  =  "1"  /* Reposition Stomach, Percutaneous Endoscopic Approach */
"0DS67ZZ"  =  "1"  /* Reposition Stomach, Via Natural or Artificial Opening */
"0DS68ZZ"  =  "1"  /* Reposition Stomach, Via Natural or Artificial Opening Endoscopic */
"0DS6XZZ"  =  "1"  /* Reposition Stomach, External Approach */
"0DS90ZZ"  =  "1"  /* Reposition Duodenum, Open Approach */
"0DS94ZZ"  =  "1"  /* Reposition Duodenum, Percutaneous Endoscopic Approach */
"0DS97ZZ"  =  "1"  /* Reposition Duodenum, Via Natural or Artificial Opening */
"0DS98ZZ"  =  "1"  /* Reposition Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DSA0ZZ"  =  "1"  /* Reposition Jejunum, Open Approach */
"0DSA4ZZ"  =  "1"  /* Reposition Jejunum, Percutaneous Endoscopic Approach */
"0DSA7ZZ"  =  "1"  /* Reposition Jejunum, Via Natural or Artificial Opening */
"0DSA8ZZ"  =  "1"  /* Reposition Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DSB0ZZ"  =  "1"  /* Reposition Ileum, Open Approach */
"0DSB4ZZ"  =  "1"  /* Reposition Ileum, Percutaneous Endoscopic Approach */
"0DSB7ZZ"  =  "1"  /* Reposition Ileum, Via Natural or Artificial Opening */
"0DSB8ZZ"  =  "1"  /* Reposition Ileum, Via Natural or Artificial Opening Endoscopic */
"0DSH0ZZ"  =  "1"  /* Reposition Cecum, Open Approach */
"0DSH4ZZ"  =  "1"  /* Reposition Cecum, Percutaneous Endoscopic Approach */
"0DSH7ZZ"  =  "1"  /* Reposition Cecum, Via Natural or Artificial Opening */
"0DSH8ZZ"  =  "1"  /* Reposition Cecum, Via Natural or Artificial Opening Endoscopic */
"0DSK0ZZ"  =  "1"  /* Reposition Ascending Colon, Open Approach */
"0DSK4ZZ"  =  "1"  /* Reposition Ascending Colon, Percutaneous Endoscopic Approach */
"0DSK7ZZ"  =  "1"  /* Reposition Ascending Colon, Via Natural or Artificial Opening */
"0DSK8ZZ"  =  "1"  /* Reposition Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0DSL0ZZ"  =  "1"  /* Reposition Transverse Colon, Open Approach */
"0DSL4ZZ"  =  "1"  /* Reposition Transverse Colon, Percutaneous Endoscopic Approach */
"0DSL7ZZ"  =  "1"  /* Reposition Transverse Colon, Via Natural or Artificial Opening */
"0DSL8ZZ"  =  "1"  /* Reposition Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0DSM0ZZ"  =  "1"  /* Reposition Descending Colon, Open Approach */
"0DSM4ZZ"  =  "1"  /* Reposition Descending Colon, Percutaneous Endoscopic Approach */
"0DSM7ZZ"  =  "1"  /* Reposition Descending Colon, Via Natural or Artificial Opening */
"0DSM8ZZ"  =  "1"  /* Reposition Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0DSN0ZZ"  =  "1"  /* Reposition Sigmoid Colon, Open Approach */
"0DSN4ZZ"  =  "1"  /* Reposition Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DSN7ZZ"  =  "1"  /* Reposition Sigmoid Colon, Via Natural or Artificial Opening */
"0DSN8ZZ"  =  "1"  /* Reposition Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0DSP0ZZ"  =  "1"  /* Reposition Rectum, Open Approach */
"0DSP4ZZ"  =  "1"  /* Reposition Rectum, Percutaneous Endoscopic Approach */
"0DT10ZZ"  =  "1"  /* Resection of Upper Esophagus, Open Approach */
"0DT14ZZ"  =  "1"  /* Resection of Upper Esophagus, Percutaneous Endoscopic Approach */
"0DT17ZZ"  =  "1"  /* Resection of Upper Esophagus, Via Natural or Artificial Opening */
"0DT18ZZ"  =  "1"  /* Resection of Upper Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT20ZZ"  =  "1"  /* Resection of Middle Esophagus, Open Approach */
"0DT24ZZ"  =  "1"  /* Resection of Middle Esophagus, Percutaneous Endoscopic Approach */
"0DT27ZZ"  =  "1"  /* Resection of Middle Esophagus, Via Natural or Artificial Opening */
"0DT28ZZ"  =  "1"  /* Resection of Middle Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT30ZZ"  =  "1"  /* Resection of Lower Esophagus, Open Approach */
"0DT34ZZ"  =  "1"  /* Resection of Lower Esophagus, Percutaneous Endoscopic Approach */
"0DT37ZZ"  =  "1"  /* Resection of Lower Esophagus, Via Natural or Artificial Opening */
"0DT38ZZ"  =  "1"  /* Resection of Lower Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT40ZZ"  =  "1"  /* Resection of Esophagogastric Junction, Open Approach */
"0DT44ZZ"  =  "1"  /* Resection of Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0DT47ZZ"  =  "1"  /* Resection of Esophagogastric Junction, Via Natural or Artificial Opening */
"0DT48ZZ"  =  "1"  /* Resection of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic */
"0DT50ZZ"  =  "1"  /* Resection of Esophagus, Open Approach */
"0DT54ZZ"  =  "1"  /* Resection of Esophagus, Percutaneous Endoscopic Approach */
"0DT57ZZ"  =  "1"  /* Resection of Esophagus, Via Natural or Artificial Opening */
"0DT58ZZ"  =  "1"  /* Resection of Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT60ZZ"  =  "1"  /* Resection of Stomach, Open Approach */
"0DT64ZZ"  =  "1"  /* Resection of Stomach, Percutaneous Endoscopic Approach */
"0DT67ZZ"  =  "1"  /* Resection of Stomach, Via Natural or Artificial Opening */
"0DT68ZZ"  =  "1"  /* Resection of Stomach, Via Natural or Artificial Opening Endoscopic */
"0DT70ZZ"  =  "1"  /* Resection of Stomach, Pylorus, Open Approach */
"0DT74ZZ"  =  "1"  /* Resection of Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0DT77ZZ"  =  "1"  /* Resection of Stomach, Pylorus, Via Natural or Artificial Opening */
"0DT78ZZ"  =  "1"  /* Resection of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0DT80ZZ"  =  "1"  /* Resection of Small Intestine, Open Approach */
"0DT84ZZ"  =  "1"  /* Resection of Small Intestine, Percutaneous Endoscopic Approach */
"0DT87ZZ"  =  "1"  /* Resection of Small Intestine, Via Natural or Artificial Opening */
"0DT88ZZ"  =  "1"  /* Resection of Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DT90ZZ"  =  "1"  /* Resection of Duodenum, Open Approach */
"0DT94ZZ"  =  "1"  /* Resection of Duodenum, Percutaneous Endoscopic Approach */
"0DT97ZZ"  =  "1"  /* Resection of Duodenum, Via Natural or Artificial Opening */
"0DT98ZZ"  =  "1"  /* Resection of Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DTA0ZZ"  =  "1"  /* Resection of Jejunum, Open Approach */
"0DTA4ZZ"  =  "1"  /* Resection of Jejunum, Percutaneous Endoscopic Approach */
"0DTA7ZZ"  =  "1"  /* Resection of Jejunum, Via Natural or Artificial Opening */
"0DTA8ZZ"  =  "1"  /* Resection of Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DTB0ZZ"  =  "1"  /* Resection of Ileum, Open Approach */
"0DTB4ZZ"  =  "1"  /* Resection of Ileum, Percutaneous Endoscopic Approach */
"0DTB7ZZ"  =  "1"  /* Resection of Ileum, Via Natural or Artificial Opening */
"0DTB8ZZ"  =  "1"  /* Resection of Ileum, Via Natural or Artificial Opening Endoscopic */
"0DTC0ZZ"  =  "1"  /* Resection of Ileocecal Valve, Open Approach */
"0DTC4ZZ"  =  "1"  /* Resection of Ileocecal Valve, Percutaneous Endoscopic Approach */
"0DTC7ZZ"  =  "1"  /* Resection of Ileocecal Valve, Via Natural or Artificial Opening */
"0DTC8ZZ"  =  "1"  /* Resection of Ileocecal Valve, Via Natural or Artificial Opening Endoscopic */
"0DTN0ZZ"  =  "1"  /* Resection of Sigmoid Colon, Open Approach */
"0DTP0ZZ"  =  "1"  /* Resection of Rectum, Open Approach */
"0DTP4ZZ"  =  "1"  /* Resection of Rectum, Percutaneous Endoscopic Approach */
"0DTP7ZZ"  =  "1"  /* Resection of Rectum, Via Natural or Artificial Opening */
"0DTP8ZZ"  =  "1"  /* Resection of Rectum, Via Natural or Artificial Opening Endoscopic */
"0DTS0ZZ"  =  "1"  /* Resection of Greater Omentum, Open Approach */
"0DTS4ZZ"  =  "1"  /* Resection of Greater Omentum, Percutaneous Endoscopic Approach */
"0DTT0ZZ"  =  "1"  /* Resection of Lesser Omentum, Open Approach */
"0DTT4ZZ"  =  "1"  /* Resection of Lesser Omentum, Percutaneous Endoscopic Approach */
"0DU407Z"  =  "1"  /* Supplement Esophagogastric Junction with Autologous Tissue Substitute, Open Approach */
"0DU40JZ"  =  "1"  /* Supplement Esophagogastric Junction with Synthetic Substitute, Open Approach */
"0DU40KZ"  =  "1"  /* Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Open Approach */
"0DU447Z"  =  "1"  /* Supplement Esophagogastric Junction with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU44JZ"  =  "1"  /* Supplement Esophagogastric Junction with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DU44KZ"  =  "1"  /* Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU477Z"  =  "1"  /* Supplement Esophagogastric Junction with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU47JZ"  =  "1"  /* Supplement Esophagogastric Junction with Synthetic Substitute, Via Natural or Artificial Opening */
"0DU47KZ"  =  "1"  /* Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU487Z"  =  "1"  /* Supplement Esophagogastric Junction with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU48JZ"  =  "1"  /* Supplement Esophagogastric Junction with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU48KZ"  =  "1"  /* Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU607Z"  =  "1"  /* Supplement Stomach with Autologous Tissue Substitute, Open Approach */
"0DU60JZ"  =  "1"  /* Supplement Stomach with Synthetic Substitute, Open Approach */
"0DU60KZ"  =  "1"  /* Supplement Stomach with Nonautologous Tissue Substitute, Open Approach */
"0DU647Z"  =  "1"  /* Supplement Stomach with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU64JZ"  =  "1"  /* Supplement Stomach with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DU64KZ"  =  "1"  /* Supplement Stomach with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU677Z"  =  "1"  /* Supplement Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU67JZ"  =  "1"  /* Supplement Stomach with Synthetic Substitute, Via Natural or Artificial Opening */
"0DU67KZ"  =  "1"  /* Supplement Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU687Z"  =  "1"  /* Supplement Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU68JZ"  =  "1"  /* Supplement Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU68KZ"  =  "1"  /* Supplement Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU707Z"  =  "1"  /* Supplement Stomach, Pylorus with Autologous Tissue Substitute, Open Approach */
"0DU70JZ"  =  "1"  /* Supplement Stomach, Pylorus with Synthetic Substitute, Open Approach */
"0DU70KZ"  =  "1"  /* Supplement Stomach, Pylorus with Nonautologous Tissue Substitute, Open Approach */
"0DU747Z"  =  "1"  /* Supplement Stomach, Pylorus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU74JZ"  =  "1"  /* Supplement Stomach, Pylorus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DU74KZ"  =  "1"  /* Supplement Stomach, Pylorus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU777Z"  =  "1"  /* Supplement Stomach, Pylorus with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU77JZ"  =  "1"  /* Supplement Stomach, Pylorus with Synthetic Substitute, Via Natural or Artificial Opening */
"0DU77KZ"  =  "1"  /* Supplement Stomach, Pylorus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU787Z"  =  "1"  /* Supplement Stomach, Pylorus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU78JZ"  =  "1"  /* Supplement Stomach, Pylorus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU78KZ"  =  "1"  /* Supplement Stomach, Pylorus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU807Z"  =  "1"  /* Supplement Small Intestine with Autologous Tissue Substitute, Open Approach */
"0DU80JZ"  =  "1"  /* Supplement Small Intestine with Synthetic Substitute, Open Approach */
"0DU80KZ"  =  "1"  /* Supplement Small Intestine with Nonautologous Tissue Substitute, Open Approach */
"0DU847Z"  =  "1"  /* Supplement Small Intestine with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU84JZ"  =  "1"  /* Supplement Small Intestine with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DU84KZ"  =  "1"  /* Supplement Small Intestine with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU877Z"  =  "1"  /* Supplement Small Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU87JZ"  =  "1"  /* Supplement Small Intestine with Synthetic Substitute, Via Natural or Artificial Opening */
"0DU87KZ"  =  "1"  /* Supplement Small Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU887Z"  =  "1"  /* Supplement Small Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU88JZ"  =  "1"  /* Supplement Small Intestine with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU88KZ"  =  "1"  /* Supplement Small Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU907Z"  =  "1"  /* Supplement Duodenum with Autologous Tissue Substitute, Open Approach */
"0DU90JZ"  =  "1"  /* Supplement Duodenum with Synthetic Substitute, Open Approach */
"0DU90KZ"  =  "1"  /* Supplement Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0DU947Z"  =  "1"  /* Supplement Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU94JZ"  =  "1"  /* Supplement Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DU94KZ"  =  "1"  /* Supplement Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU977Z"  =  "1"  /* Supplement Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU97JZ"  =  "1"  /* Supplement Duodenum with Synthetic Substitute, Via Natural or Artificial Opening */
"0DU97KZ"  =  "1"  /* Supplement Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU987Z"  =  "1"  /* Supplement Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU98JZ"  =  "1"  /* Supplement Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU98KZ"  =  "1"  /* Supplement Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUA07Z"  =  "1"  /* Supplement Jejunum with Autologous Tissue Substitute, Open Approach */
"0DUA0JZ"  =  "1"  /* Supplement Jejunum with Synthetic Substitute, Open Approach */
"0DUA0KZ"  =  "1"  /* Supplement Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0DUA47Z"  =  "1"  /* Supplement Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUA4JZ"  =  "1"  /* Supplement Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUA4KZ"  =  "1"  /* Supplement Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUA77Z"  =  "1"  /* Supplement Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUA7JZ"  =  "1"  /* Supplement Jejunum with Synthetic Substitute, Via Natural or Artificial Opening */
"0DUA7KZ"  =  "1"  /* Supplement Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUA87Z"  =  "1"  /* Supplement Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUA8JZ"  =  "1"  /* Supplement Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUA8KZ"  =  "1"  /* Supplement Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUB07Z"  =  "1"  /* Supplement Ileum with Autologous Tissue Substitute, Open Approach */
"0DUB0JZ"  =  "1"  /* Supplement Ileum with Synthetic Substitute, Open Approach */
"0DUB0KZ"  =  "1"  /* Supplement Ileum with Nonautologous Tissue Substitute, Open Approach */
"0DUB47Z"  =  "1"  /* Supplement Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUB4JZ"  =  "1"  /* Supplement Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUB4KZ"  =  "1"  /* Supplement Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUB77Z"  =  "1"  /* Supplement Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUB7JZ"  =  "1"  /* Supplement Ileum with Synthetic Substitute, Via Natural or Artificial Opening */
"0DUB7KZ"  =  "1"  /* Supplement Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUB87Z"  =  "1"  /* Supplement Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUB8JZ"  =  "1"  /* Supplement Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUB8KZ"  =  "1"  /* Supplement Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUC07Z"  =  "1"  /* Supplement Ileocecal Valve with Autologous Tissue Substitute, Open Approach */
"0DUC0JZ"  =  "1"  /* Supplement Ileocecal Valve with Synthetic Substitute, Open Approach */
"0DUC0KZ"  =  "1"  /* Supplement Ileocecal Valve with Nonautologous Tissue Substitute, Open Approach */
"0DUC47Z"  =  "1"  /* Supplement Ileocecal Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUC4JZ"  =  "1"  /* Supplement Ileocecal Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUC4KZ"  =  "1"  /* Supplement Ileocecal Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUC77Z"  =  "1"  /* Supplement Ileocecal Valve with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUC7JZ"  =  "1"  /* Supplement Ileocecal Valve with Synthetic Substitute, Via Natural or Artificial Opening */
"0DUC7KZ"  =  "1"  /* Supplement Ileocecal Valve with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUC87Z"  =  "1"  /* Supplement Ileocecal Valve with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUC8JZ"  =  "1"  /* Supplement Ileocecal Valve with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUC8KZ"  =  "1"  /* Supplement Ileocecal Valve with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUE07Z"  =  "1"  /* Supplement Large Intestine with Autologous Tissue Substitute, Open Approach */
"0DUE0JZ"  =  "1"  /* Supplement Large Intestine with Synthetic Substitute, Open Approach */
"0DUE0KZ"  =  "1"  /* Supplement Large Intestine with Nonautologous Tissue Substitute, Open Approach */
"0DUE47Z"  =  "1"  /* Supplement Large Intestine with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUE4JZ"  =  "1"  /* Supplement Large Intestine with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUE4KZ"  =  "1"  /* Supplement Large Intestine with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUE77Z"  =  "1"  /* Supplement Large Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUE7JZ"  =  "1"  /* Supplement Large Intestine with Synthetic Substitute, Via Natural or Artificial Opening */
"0DUE7KZ"  =  "1"  /* Supplement Large Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUE87Z"  =  "1"  /* Supplement Large Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUE8JZ"  =  "1"  /* Supplement Large Intestine with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUE8KZ"  =  "1"  /* Supplement Large Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUF07Z"  =  "1"  /* Supplement Right Large Intestine with Autologous Tissue Substitute, Open Approach */
"0DUF0JZ"  =  "1"  /* Supplement Right Large Intestine with Synthetic Substitute, Open Approach */
"0DUF0KZ"  =  "1"  /* Supplement Right Large Intestine with Nonautologous Tissue Substitute, Open Approach */
"0DUF47Z"  =  "1"  /* Supplement Right Large Intestine with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUF4JZ"  =  "1"  /* Supplement Right Large Intestine with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUF4KZ"  =  "1"  /* Supplement Right Large Intestine with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUF77Z"  =  "1"  /* Supplement Right Large Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUF7JZ"  =  "1"  /* Supplement Right Large Intestine with Synthetic Substitute, Via Natural or Artificial Opening */
"0DUF7KZ"  =  "1"  /* Supplement Right Large Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUF87Z"  =  "1"  /* Supplement Right Large Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUF8JZ"  =  "1"  /* Supplement Right Large Intestine with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUF8KZ"  =  "1"  /* Supplement Right Large Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUG07Z"  =  "1"  /* Supplement Left Large Intestine with Autologous Tissue Substitute, Open Approach */
"0DUG0JZ"  =  "1"  /* Supplement Left Large Intestine with Synthetic Substitute, Open Approach */
"0DUG0KZ"  =  "1"  /* Supplement Left Large Intestine with Nonautologous Tissue Substitute, Open Approach */
"0DUG47Z"  =  "1"  /* Supplement Left Large Intestine with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUG4JZ"  =  "1"  /* Supplement Left Large Intestine with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUG4KZ"  =  "1"  /* Supplement Left Large Intestine with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUG77Z"  =  "1"  /* Supplement Left Large Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUG7JZ"  =  "1"  /* Supplement Left Large Intestine with Synthetic Substitute, Via Natural or Artificial Opening */
"0DUG7KZ"  =  "1"  /* Supplement Left Large Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUG87Z"  =  "1"  /* Supplement Left Large Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUG8JZ"  =  "1"  /* Supplement Left Large Intestine with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUG8KZ"  =  "1"  /* Supplement Left Large Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUH07Z"  =  "1"  /* Supplement Cecum with Autologous Tissue Substitute, Open Approach */
"0DUH0JZ"  =  "1"  /* Supplement Cecum with Synthetic Substitute, Open Approach */
"0DUH0KZ"  =  "1"  /* Supplement Cecum with Nonautologous Tissue Substitute, Open Approach */
"0DUH47Z"  =  "1"  /* Supplement Cecum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUH4JZ"  =  "1"  /* Supplement Cecum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUH4KZ"  =  "1"  /* Supplement Cecum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUH77Z"  =  "1"  /* Supplement Cecum with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUH7JZ"  =  "1"  /* Supplement Cecum with Synthetic Substitute, Via Natural or Artificial Opening */
"0DUH7KZ"  =  "1"  /* Supplement Cecum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUH87Z"  =  "1"  /* Supplement Cecum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUH8JZ"  =  "1"  /* Supplement Cecum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUH8KZ"  =  "1"  /* Supplement Cecum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUK07Z"  =  "1"  /* Supplement Ascending Colon with Autologous Tissue Substitute, Open Approach */
"0DUK0JZ"  =  "1"  /* Supplement Ascending Colon with Synthetic Substitute, Open Approach */
"0DUK0KZ"  =  "1"  /* Supplement Ascending Colon with Nonautologous Tissue Substitute, Open Approach */
"0DUK47Z"  =  "1"  /* Supplement Ascending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUK4JZ"  =  "1"  /* Supplement Ascending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUK4KZ"  =  "1"  /* Supplement Ascending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUK77Z"  =  "1"  /* Supplement Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUK7JZ"  =  "1"  /* Supplement Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening */
"0DUK7KZ"  =  "1"  /* Supplement Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUK87Z"  =  "1"  /* Supplement Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUK8JZ"  =  "1"  /* Supplement Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUK8KZ"  =  "1"  /* Supplement Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUL07Z"  =  "1"  /* Supplement Transverse Colon with Autologous Tissue Substitute, Open Approach */
"0DUL0JZ"  =  "1"  /* Supplement Transverse Colon with Synthetic Substitute, Open Approach */
"0DUL0KZ"  =  "1"  /* Supplement Transverse Colon with Nonautologous Tissue Substitute, Open Approach */
"0DUL47Z"  =  "1"  /* Supplement Transverse Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUL4JZ"  =  "1"  /* Supplement Transverse Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUL4KZ"  =  "1"  /* Supplement Transverse Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUL77Z"  =  "1"  /* Supplement Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUL7JZ"  =  "1"  /* Supplement Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening */
"0DUL7KZ"  =  "1"  /* Supplement Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUL87Z"  =  "1"  /* Supplement Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUL8JZ"  =  "1"  /* Supplement Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUL8KZ"  =  "1"  /* Supplement Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUM07Z"  =  "1"  /* Supplement Descending Colon with Autologous Tissue Substitute, Open Approach */
"0DUM0JZ"  =  "1"  /* Supplement Descending Colon with Synthetic Substitute, Open Approach */
"0DUM0KZ"  =  "1"  /* Supplement Descending Colon with Nonautologous Tissue Substitute, Open Approach */
"0DUM47Z"  =  "1"  /* Supplement Descending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUM4JZ"  =  "1"  /* Supplement Descending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUM4KZ"  =  "1"  /* Supplement Descending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUM77Z"  =  "1"  /* Supplement Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUM7JZ"  =  "1"  /* Supplement Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening */
"0DUM7KZ"  =  "1"  /* Supplement Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUM87Z"  =  "1"  /* Supplement Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUM8JZ"  =  "1"  /* Supplement Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUM8KZ"  =  "1"  /* Supplement Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUN07Z"  =  "1"  /* Supplement Sigmoid Colon with Autologous Tissue Substitute, Open Approach */
"0DUN0JZ"  =  "1"  /* Supplement Sigmoid Colon with Synthetic Substitute, Open Approach */
"0DUN0KZ"  =  "1"  /* Supplement Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach */
"0DUN47Z"  =  "1"  /* Supplement Sigmoid Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUN4JZ"  =  "1"  /* Supplement Sigmoid Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUN4KZ"  =  "1"  /* Supplement Sigmoid Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUN77Z"  =  "1"  /* Supplement Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUN7JZ"  =  "1"  /* Supplement Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening */
"0DUN7KZ"  =  "1"  /* Supplement Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DUN87Z"  =  "1"  /* Supplement Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUN8JZ"  =  "1"  /* Supplement Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUN8KZ"  =  "1"  /* Supplement Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUS07Z"  =  "1"  /* Supplement Greater Omentum with Autologous Tissue Substitute, Open Approach */
"0DUS0JZ"  =  "1"  /* Supplement Greater Omentum with Synthetic Substitute, Open Approach */
"0DUS0KZ"  =  "1"  /* Supplement Greater Omentum with Nonautologous Tissue Substitute, Open Approach */
"0DUS47Z"  =  "1"  /* Supplement Greater Omentum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUS4JZ"  =  "1"  /* Supplement Greater Omentum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUS4KZ"  =  "1"  /* Supplement Greater Omentum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUT07Z"  =  "1"  /* Supplement Lesser Omentum with Autologous Tissue Substitute, Open Approach */
"0DUT0JZ"  =  "1"  /* Supplement Lesser Omentum with Synthetic Substitute, Open Approach */
"0DUT0KZ"  =  "1"  /* Supplement Lesser Omentum with Nonautologous Tissue Substitute, Open Approach */
"0DUT47Z"  =  "1"  /* Supplement Lesser Omentum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUT4JZ"  =  "1"  /* Supplement Lesser Omentum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUT4KZ"  =  "1"  /* Supplement Lesser Omentum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUV07Z"  =  "1"  /* Supplement Mesentery with Autologous Tissue Substitute, Open Approach */
"0DUV0JZ"  =  "1"  /* Supplement Mesentery with Synthetic Substitute, Open Approach */
"0DUV0KZ"  =  "1"  /* Supplement Mesentery with Nonautologous Tissue Substitute, Open Approach */
"0DUV47Z"  =  "1"  /* Supplement Mesentery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUV4JZ"  =  "1"  /* Supplement Mesentery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUV4KZ"  =  "1"  /* Supplement Mesentery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUW07Z"  =  "1"  /* Supplement Peritoneum with Autologous Tissue Substitute, Open Approach */
"0DUW0JZ"  =  "1"  /* Supplement Peritoneum with Synthetic Substitute, Open Approach */
"0DUW0KZ"  =  "1"  /* Supplement Peritoneum with Nonautologous Tissue Substitute, Open Approach */
"0DUW47Z"  =  "1"  /* Supplement Peritoneum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUW4JZ"  =  "1"  /* Supplement Peritoneum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUW4KZ"  =  "1"  /* Supplement Peritoneum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DV40CZ"  =  "1"  /* Restriction of Esophagogastric Junction with Extraluminal Device, Open Approach */
"0DV40DZ"  =  "1"  /* Restriction of Esophagogastric Junction with Intraluminal Device, Open Approach */
"0DV40ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Open Approach */
"0DV43CZ"  =  "1"  /* Restriction of Esophagogastric Junction with Extraluminal Device, Percutaneous Approach */
"0DV43DZ"  =  "1"  /* Restriction of Esophagogastric Junction with Intraluminal Device, Percutaneous Approach */
"0DV43ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Percutaneous Approach */
"0DV44CZ"  =  "1"  /* Restriction of Esophagogastric Junction with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DV44DZ"  =  "1"  /* Restriction of Esophagogastric Junction with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DV44ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0DV47DZ"  =  "1"  /* Restriction of Esophagogastric Junction with Intraluminal Device, Via Natural or Artificial Opening */
"0DV47ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Via Natural or Artificial Opening */
"0DV48DZ"  =  "1"  /* Restriction of Esophagogastric Junction with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DV48ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic */
"0DV60CZ"  =  "1"  /* Restriction of Stomach with Extraluminal Device, Open Approach */
"0DV60DZ"  =  "1"  /* Restriction of Stomach with Intraluminal Device, Open Approach */
"0DV60ZZ"  =  "1"  /* Restriction of Stomach, Open Approach */
"0DV63CZ"  =  "1"  /* Restriction of Stomach with Extraluminal Device, Percutaneous Approach */
"0DV63DZ"  =  "1"  /* Restriction of Stomach with Intraluminal Device, Percutaneous Approach */
"0DV63ZZ"  =  "1"  /* Restriction of Stomach, Percutaneous Approach */
"0DV64CZ"  =  "1"  /* Restriction of Stomach with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DV64DZ"  =  "1"  /* Restriction of Stomach with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DV64ZZ"  =  "1"  /* Restriction of Stomach, Percutaneous Endoscopic Approach */
"0DV67ZZ"  =  "1"  /* Restriction of Stomach, Via Natural or Artificial Opening */
"0DV68ZZ"  =  "1"  /* Restriction of Stomach, Via Natural or Artificial Opening Endoscopic */
"0DV70CZ"  =  "1"  /* Restriction of Stomach, Pylorus with Extraluminal Device, Open Approach */
"0DV70DZ"  =  "1"  /* Restriction of Stomach, Pylorus with Intraluminal Device, Open Approach */
"0DV70ZZ"  =  "1"  /* Restriction of Stomach, Pylorus, Open Approach */
"0DV73CZ"  =  "1"  /* Restriction of Stomach, Pylorus with Extraluminal Device, Percutaneous Approach */
"0DV73DZ"  =  "1"  /* Restriction of Stomach, Pylorus with Intraluminal Device, Percutaneous Approach */
"0DV73ZZ"  =  "1"  /* Restriction of Stomach, Pylorus, Percutaneous Approach */
"0DV74CZ"  =  "1"  /* Restriction of Stomach, Pylorus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DV74DZ"  =  "1"  /* Restriction of Stomach, Pylorus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DV74ZZ"  =  "1"  /* Restriction of Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0DV77DZ"  =  "1"  /* Restriction of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening */
"0DV77ZZ"  =  "1"  /* Restriction of Stomach, Pylorus, Via Natural or Artificial Opening */
"0DV78DZ"  =  "1"  /* Restriction of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DV78ZZ"  =  "1"  /* Restriction of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0DV80CZ"  =  "1"  /* Restriction of Small Intestine with Extraluminal Device, Open Approach */
"0DV80DZ"  =  "1"  /* Restriction of Small Intestine with Intraluminal Device, Open Approach */
"0DV80ZZ"  =  "1"  /* Restriction of Small Intestine, Open Approach */
"0DV83CZ"  =  "1"  /* Restriction of Small Intestine with Extraluminal Device, Percutaneous Approach */
"0DV83DZ"  =  "1"  /* Restriction of Small Intestine with Intraluminal Device, Percutaneous Approach */
"0DV83ZZ"  =  "1"  /* Restriction of Small Intestine, Percutaneous Approach */
"0DV84CZ"  =  "1"  /* Restriction of Small Intestine with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DV84DZ"  =  "1"  /* Restriction of Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DV84ZZ"  =  "1"  /* Restriction of Small Intestine, Percutaneous Endoscopic Approach */
"0DV87DZ"  =  "1"  /* Restriction of Small Intestine with Intraluminal Device, Via Natural or Artificial Opening */
"0DV87ZZ"  =  "1"  /* Restriction of Small Intestine, Via Natural or Artificial Opening */
"0DV88DZ"  =  "1"  /* Restriction of Small Intestine with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DV88ZZ"  =  "1"  /* Restriction of Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DV90CZ"  =  "1"  /* Restriction of Duodenum with Extraluminal Device, Open Approach */
"0DV90DZ"  =  "1"  /* Restriction of Duodenum with Intraluminal Device, Open Approach */
"0DV90ZZ"  =  "1"  /* Restriction of Duodenum, Open Approach */
"0DV93CZ"  =  "1"  /* Restriction of Duodenum with Extraluminal Device, Percutaneous Approach */
"0DV93DZ"  =  "1"  /* Restriction of Duodenum with Intraluminal Device, Percutaneous Approach */
"0DV93ZZ"  =  "1"  /* Restriction of Duodenum, Percutaneous Approach */
"0DV94CZ"  =  "1"  /* Restriction of Duodenum with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DV94DZ"  =  "1"  /* Restriction of Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DV94ZZ"  =  "1"  /* Restriction of Duodenum, Percutaneous Endoscopic Approach */
"0DV97DZ"  =  "1"  /* Restriction of Duodenum with Intraluminal Device, Via Natural or Artificial Opening */
"0DV97ZZ"  =  "1"  /* Restriction of Duodenum, Via Natural or Artificial Opening */
"0DV98DZ"  =  "1"  /* Restriction of Duodenum with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DV98ZZ"  =  "1"  /* Restriction of Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DVA0CZ"  =  "1"  /* Restriction of Jejunum with Extraluminal Device, Open Approach */
"0DVA0DZ"  =  "1"  /* Restriction of Jejunum with Intraluminal Device, Open Approach */
"0DVA0ZZ"  =  "1"  /* Restriction of Jejunum, Open Approach */
"0DVA3CZ"  =  "1"  /* Restriction of Jejunum with Extraluminal Device, Percutaneous Approach */
"0DVA3DZ"  =  "1"  /* Restriction of Jejunum with Intraluminal Device, Percutaneous Approach */
"0DVA3ZZ"  =  "1"  /* Restriction of Jejunum, Percutaneous Approach */
"0DVA4CZ"  =  "1"  /* Restriction of Jejunum with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DVA4DZ"  =  "1"  /* Restriction of Jejunum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DVA4ZZ"  =  "1"  /* Restriction of Jejunum, Percutaneous Endoscopic Approach */
"0DVA7DZ"  =  "1"  /* Restriction of Jejunum with Intraluminal Device, Via Natural or Artificial Opening */
"0DVA7ZZ"  =  "1"  /* Restriction of Jejunum, Via Natural or Artificial Opening */
"0DVA8DZ"  =  "1"  /* Restriction of Jejunum with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DVA8ZZ"  =  "1"  /* Restriction of Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DVB0CZ"  =  "1"  /* Restriction of Ileum with Extraluminal Device, Open Approach */
"0DVB0DZ"  =  "1"  /* Restriction of Ileum with Intraluminal Device, Open Approach */
"0DVB0ZZ"  =  "1"  /* Restriction of Ileum, Open Approach */
"0DVB3CZ"  =  "1"  /* Restriction of Ileum with Extraluminal Device, Percutaneous Approach */
"0DVB3DZ"  =  "1"  /* Restriction of Ileum with Intraluminal Device, Percutaneous Approach */
"0DVB3ZZ"  =  "1"  /* Restriction of Ileum, Percutaneous Approach */
"0DVB4CZ"  =  "1"  /* Restriction of Ileum with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DVB4DZ"  =  "1"  /* Restriction of Ileum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DVB4ZZ"  =  "1"  /* Restriction of Ileum, Percutaneous Endoscopic Approach */
"0DVB7DZ"  =  "1"  /* Restriction of Ileum with Intraluminal Device, Via Natural or Artificial Opening */
"0DVB7ZZ"  =  "1"  /* Restriction of Ileum, Via Natural or Artificial Opening */
"0DVB8DZ"  =  "1"  /* Restriction of Ileum with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DVB8ZZ"  =  "1"  /* Restriction of Ileum, Via Natural or Artificial Opening Endoscopic */
"0DVC0CZ"  =  "1"  /* Restriction of Ileocecal Valve with Extraluminal Device, Open Approach */
"0DVC0DZ"  =  "1"  /* Restriction of Ileocecal Valve with Intraluminal Device, Open Approach */
"0DVC0ZZ"  =  "1"  /* Restriction of Ileocecal Valve, Open Approach */
"0DVC3CZ"  =  "1"  /* Restriction of Ileocecal Valve with Extraluminal Device, Percutaneous Approach */
"0DVC3DZ"  =  "1"  /* Restriction of Ileocecal Valve with Intraluminal Device, Percutaneous Approach */
"0DVC3ZZ"  =  "1"  /* Restriction of Ileocecal Valve, Percutaneous Approach */
"0DVC4CZ"  =  "1"  /* Restriction of Ileocecal Valve with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DVC4DZ"  =  "1"  /* Restriction of Ileocecal Valve with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DVC4ZZ"  =  "1"  /* Restriction of Ileocecal Valve, Percutaneous Endoscopic Approach */
"0DVC7DZ"  =  "1"  /* Restriction of Ileocecal Valve with Intraluminal Device, Via Natural or Artificial Opening */
"0DVC7ZZ"  =  "1"  /* Restriction of Ileocecal Valve, Via Natural or Artificial Opening */
"0DVC8DZ"  =  "1"  /* Restriction of Ileocecal Valve with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DVC8ZZ"  =  "1"  /* Restriction of Ileocecal Valve, Via Natural or Artificial Opening Endoscopic */
"0DVE0CZ"  =  "1"  /* Restriction of Large Intestine with Extraluminal Device, Open Approach */
"0DVE0DZ"  =  "1"  /* Restriction of Large Intestine with Intraluminal Device, Open Approach */
"0DVE0ZZ"  =  "1"  /* Restriction of Large Intestine, Open Approach */
"0DVE3CZ"  =  "1"  /* Restriction of Large Intestine with Extraluminal Device, Percutaneous Approach */
"0DVE3DZ"  =  "1"  /* Restriction of Large Intestine with Intraluminal Device, Percutaneous Approach */
"0DVE3ZZ"  =  "1"  /* Restriction of Large Intestine, Percutaneous Approach */
"0DVE4CZ"  =  "1"  /* Restriction of Large Intestine with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DVE4DZ"  =  "1"  /* Restriction of Large Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DVE4ZZ"  =  "1"  /* Restriction of Large Intestine, Percutaneous Endoscopic Approach */
"0DVE7DZ"  =  "1"  /* Restriction of Large Intestine with Intraluminal Device, Via Natural or Artificial Opening */
"0DVE7ZZ"  =  "1"  /* Restriction of Large Intestine, Via Natural or Artificial Opening */
"0DVE8DZ"  =  "1"  /* Restriction of Large Intestine with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DVE8ZZ"  =  "1"  /* Restriction of Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DVF0CZ"  =  "1"  /* Restriction of Right Large Intestine with Extraluminal Device, Open Approach */
"0DVF0DZ"  =  "1"  /* Restriction of Right Large Intestine with Intraluminal Device, Open Approach */
"0DVF0ZZ"  =  "1"  /* Restriction of Right Large Intestine, Open Approach */
"0DVF3CZ"  =  "1"  /* Restriction of Right Large Intestine with Extraluminal Device, Percutaneous Approach */
"0DVF3DZ"  =  "1"  /* Restriction of Right Large Intestine with Intraluminal Device, Percutaneous Approach */
"0DVF3ZZ"  =  "1"  /* Restriction of Right Large Intestine, Percutaneous Approach */
"0DVF4CZ"  =  "1"  /* Restriction of Right Large Intestine with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DVF4DZ"  =  "1"  /* Restriction of Right Large Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DVF4ZZ"  =  "1"  /* Restriction of Right Large Intestine, Percutaneous Endoscopic Approach */
"0DVF7DZ"  =  "1"  /* Restriction of Right Large Intestine with Intraluminal Device, Via Natural or Artificial Opening */
"0DVF7ZZ"  =  "1"  /* Restriction of Right Large Intestine, Via Natural or Artificial Opening */
"0DVF8DZ"  =  "1"  /* Restriction of Right Large Intestine with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DVF8ZZ"  =  "1"  /* Restriction of Right Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DVG0CZ"  =  "1"  /* Restriction of Left Large Intestine with Extraluminal Device, Open Approach */
"0DVG0DZ"  =  "1"  /* Restriction of Left Large Intestine with Intraluminal Device, Open Approach */
"0DVG0ZZ"  =  "1"  /* Restriction of Left Large Intestine, Open Approach */
"0DVG3CZ"  =  "1"  /* Restriction of Left Large Intestine with Extraluminal Device, Percutaneous Approach */
"0DVG3DZ"  =  "1"  /* Restriction of Left Large Intestine with Intraluminal Device, Percutaneous Approach */
"0DVG3ZZ"  =  "1"  /* Restriction of Left Large Intestine, Percutaneous Approach */
"0DVG4CZ"  =  "1"  /* Restriction of Left Large Intestine with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DVG4DZ"  =  "1"  /* Restriction of Left Large Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DVG4ZZ"  =  "1"  /* Restriction of Left Large Intestine, Percutaneous Endoscopic Approach */
"0DVG7DZ"  =  "1"  /* Restriction of Left Large Intestine with Intraluminal Device, Via Natural or Artificial Opening */
"0DVG7ZZ"  =  "1"  /* Restriction of Left Large Intestine, Via Natural or Artificial Opening */
"0DVG8DZ"  =  "1"  /* Restriction of Left Large Intestine with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DVG8ZZ"  =  "1"  /* Restriction of Left Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DVH0CZ"  =  "1"  /* Restriction of Cecum with Extraluminal Device, Open Approach */
"0DVH0DZ"  =  "1"  /* Restriction of Cecum with Intraluminal Device, Open Approach */
"0DVH0ZZ"  =  "1"  /* Restriction of Cecum, Open Approach */
"0DVH3CZ"  =  "1"  /* Restriction of Cecum with Extraluminal Device, Percutaneous Approach */
"0DVH3DZ"  =  "1"  /* Restriction of Cecum with Intraluminal Device, Percutaneous Approach */
"0DVH3ZZ"  =  "1"  /* Restriction of Cecum, Percutaneous Approach */
"0DVH4CZ"  =  "1"  /* Restriction of Cecum with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DVH4DZ"  =  "1"  /* Restriction of Cecum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DVH4ZZ"  =  "1"  /* Restriction of Cecum, Percutaneous Endoscopic Approach */
"0DVH7DZ"  =  "1"  /* Restriction of Cecum with Intraluminal Device, Via Natural or Artificial Opening */
"0DVH7ZZ"  =  "1"  /* Restriction of Cecum, Via Natural or Artificial Opening */
"0DVH8DZ"  =  "1"  /* Restriction of Cecum with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DVH8ZZ"  =  "1"  /* Restriction of Cecum, Via Natural or Artificial Opening Endoscopic */
"0DVK0CZ"  =  "1"  /* Restriction of Ascending Colon with Extraluminal Device, Open Approach */
"0DVK0DZ"  =  "1"  /* Restriction of Ascending Colon with Intraluminal Device, Open Approach */
"0DVK0ZZ"  =  "1"  /* Restriction of Ascending Colon, Open Approach */
"0DVK3CZ"  =  "1"  /* Restriction of Ascending Colon with Extraluminal Device, Percutaneous Approach */
"0DVK3DZ"  =  "1"  /* Restriction of Ascending Colon with Intraluminal Device, Percutaneous Approach */
"0DVK3ZZ"  =  "1"  /* Restriction of Ascending Colon, Percutaneous Approach */
"0DVK4CZ"  =  "1"  /* Restriction of Ascending Colon with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DVK4DZ"  =  "1"  /* Restriction of Ascending Colon with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DVK4ZZ"  =  "1"  /* Restriction of Ascending Colon, Percutaneous Endoscopic Approach */
"0DVK7DZ"  =  "1"  /* Restriction of Ascending Colon with Intraluminal Device, Via Natural or Artificial Opening */
"0DVK7ZZ"  =  "1"  /* Restriction of Ascending Colon, Via Natural or Artificial Opening */
"0DVK8DZ"  =  "1"  /* Restriction of Ascending Colon with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DVK8ZZ"  =  "1"  /* Restriction of Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0DVL0CZ"  =  "1"  /* Restriction of Transverse Colon with Extraluminal Device, Open Approach */
"0DVL0DZ"  =  "1"  /* Restriction of Transverse Colon with Intraluminal Device, Open Approach */
"0DVL0ZZ"  =  "1"  /* Restriction of Transverse Colon, Open Approach */
"0DVL3CZ"  =  "1"  /* Restriction of Transverse Colon with Extraluminal Device, Percutaneous Approach */
"0DVL3DZ"  =  "1"  /* Restriction of Transverse Colon with Intraluminal Device, Percutaneous Approach */
"0DVL3ZZ"  =  "1"  /* Restriction of Transverse Colon, Percutaneous Approach */
"0DVL4CZ"  =  "1"  /* Restriction of Transverse Colon with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DVL4DZ"  =  "1"  /* Restriction of Transverse Colon with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DVL4ZZ"  =  "1"  /* Restriction of Transverse Colon, Percutaneous Endoscopic Approach */
"0DVL7DZ"  =  "1"  /* Restriction of Transverse Colon with Intraluminal Device, Via Natural or Artificial Opening */
"0DVL7ZZ"  =  "1"  /* Restriction of Transverse Colon, Via Natural or Artificial Opening */
"0DVL8DZ"  =  "1"  /* Restriction of Transverse Colon with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DVL8ZZ"  =  "1"  /* Restriction of Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0DVM0CZ"  =  "1"  /* Restriction of Descending Colon with Extraluminal Device, Open Approach */
"0DVM0DZ"  =  "1"  /* Restriction of Descending Colon with Intraluminal Device, Open Approach */
"0DVM0ZZ"  =  "1"  /* Restriction of Descending Colon, Open Approach */
"0DVM3CZ"  =  "1"  /* Restriction of Descending Colon with Extraluminal Device, Percutaneous Approach */
"0DVM3DZ"  =  "1"  /* Restriction of Descending Colon with Intraluminal Device, Percutaneous Approach */
"0DVM3ZZ"  =  "1"  /* Restriction of Descending Colon, Percutaneous Approach */
"0DVM4CZ"  =  "1"  /* Restriction of Descending Colon with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DVM4DZ"  =  "1"  /* Restriction of Descending Colon with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DVM4ZZ"  =  "1"  /* Restriction of Descending Colon, Percutaneous Endoscopic Approach */
"0DVM7DZ"  =  "1"  /* Restriction of Descending Colon with Intraluminal Device, Via Natural or Artificial Opening */
"0DVM7ZZ"  =  "1"  /* Restriction of Descending Colon, Via Natural or Artificial Opening */
"0DVM8DZ"  =  "1"  /* Restriction of Descending Colon with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DVM8ZZ"  =  "1"  /* Restriction of Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0DVN0CZ"  =  "1"  /* Restriction of Sigmoid Colon with Extraluminal Device, Open Approach */
"0DVN0DZ"  =  "1"  /* Restriction of Sigmoid Colon with Intraluminal Device, Open Approach */
"0DVN0ZZ"  =  "1"  /* Restriction of Sigmoid Colon, Open Approach */
"0DVN3CZ"  =  "1"  /* Restriction of Sigmoid Colon with Extraluminal Device, Percutaneous Approach */
"0DVN3DZ"  =  "1"  /* Restriction of Sigmoid Colon with Intraluminal Device, Percutaneous Approach */
"0DVN3ZZ"  =  "1"  /* Restriction of Sigmoid Colon, Percutaneous Approach */
"0DVN4CZ"  =  "1"  /* Restriction of Sigmoid Colon with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DVN4DZ"  =  "1"  /* Restriction of Sigmoid Colon with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DVN4ZZ"  =  "1"  /* Restriction of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DVN7DZ"  =  "1"  /* Restriction of Sigmoid Colon with Intraluminal Device, Via Natural or Artificial Opening */
"0DVN7ZZ"  =  "1"  /* Restriction of Sigmoid Colon, Via Natural or Artificial Opening */
"0DVN8DZ"  =  "1"  /* Restriction of Sigmoid Colon with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DVN8ZZ"  =  "1"  /* Restriction of Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0DW000Z"  =  "1"  /* Revision of Drainage Device in Upper Intestinal Tract, Open Approach */
"0DW002Z"  =  "1"  /* Revision of Monitoring Device in Upper Intestinal Tract, Open Approach */
"0DW003Z"  =  "1"  /* Revision of Infusion Device in Upper Intestinal Tract, Open Approach */
"0DW007Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Open Approach */
"0DW00CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Intestinal Tract, Open Approach */
"0DW00DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Intestinal Tract, Open Approach */
"0DW00JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Intestinal Tract, Open Approach */
"0DW00KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Open Approach */
"0DW00UZ"  =  "1"  /* Revision of Feeding Device in Upper Intestinal Tract, Open Approach */
"0DW030Z"  =  "1"  /* Revision of Drainage Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW032Z"  =  "1"  /* Revision of Monitoring Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW033Z"  =  "1"  /* Revision of Infusion Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW037Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Percutaneous Approach */
"0DW03CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW03DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW03JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Intestinal Tract, Percutaneous Approach */
"0DW03KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Percutaneous Approach */
"0DW03UZ"  =  "1"  /* Revision of Feeding Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW040Z"  =  "1"  /* Revision of Drainage Device in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW042Z"  =  "1"  /* Revision of Monitoring Device in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW043Z"  =  "1"  /* Revision of Infusion Device in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW047Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW04CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW04DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW04JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW04KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW070Z"  =  "1"  /* Revision of Drainage Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW072Z"  =  "1"  /* Revision of Monitoring Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW073Z"  =  "1"  /* Revision of Infusion Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW077Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW07CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW07DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW07JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW07KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW07UZ"  =  "1"  /* Revision of Feeding Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW080Z"  =  "1"  /* Revision of Drainage Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW082Z"  =  "1"  /* Revision of Monitoring Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW083Z"  =  "1"  /* Revision of Infusion Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW087Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW08CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW08DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW08JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW08KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW08UZ"  =  "1"  /* Revision of Feeding Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW600Z"  =  "1"  /* Revision of Drainage Device in Stomach, Open Approach */
"0DW602Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Open Approach */
"0DW603Z"  =  "1"  /* Revision of Infusion Device in Stomach, Open Approach */
"0DW607Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Stomach, Open Approach */
"0DW60CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Open Approach */
"0DW60DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Open Approach */
"0DW60JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Open Approach */
"0DW60KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Stomach, Open Approach */
"0DW60MZ"  =  "1"  /* Revision of Stimulator Lead in Stomach, Open Approach */
"0DW60UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Open Approach */
"0DW630Z"  =  "1"  /* Revision of Drainage Device in Stomach, Percutaneous Approach */
"0DW632Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Percutaneous Approach */
"0DW633Z"  =  "1"  /* Revision of Infusion Device in Stomach, Percutaneous Approach */
"0DW637Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Stomach, Percutaneous Approach */
"0DW63CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Percutaneous Approach */
"0DW63DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Percutaneous Approach */
"0DW63JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Percutaneous Approach */
"0DW63KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Stomach, Percutaneous Approach */
"0DW63MZ"  =  "1"  /* Revision of Stimulator Lead in Stomach, Percutaneous Approach */
"0DW63UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Percutaneous Approach */
"0DW640Z"  =  "1"  /* Revision of Drainage Device in Stomach, Percutaneous Endoscopic Approach */
"0DW642Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Percutaneous Endoscopic Approach */
"0DW643Z"  =  "1"  /* Revision of Infusion Device in Stomach, Percutaneous Endoscopic Approach */
"0DW647Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Stomach, Percutaneous Endoscopic Approach */
"0DW64CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Percutaneous Endoscopic Approach */
"0DW64DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Percutaneous Endoscopic Approach */
"0DW64JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Percutaneous Endoscopic Approach */
"0DW64KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Stomach, Percutaneous Endoscopic Approach */
"0DW64MZ"  =  "1"  /* Revision of Stimulator Lead in Stomach, Percutaneous Endoscopic Approach */
"0DW64UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Percutaneous Endoscopic Approach */
"0DW670Z"  =  "1"  /* Revision of Drainage Device in Stomach, Via Natural or Artificial Opening */
"0DW672Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Via Natural or Artificial Opening */
"0DW673Z"  =  "1"  /* Revision of Infusion Device in Stomach, Via Natural or Artificial Opening */
"0DW677Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Stomach, Via Natural or Artificial Opening */
"0DW67CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Via Natural or Artificial Opening */
"0DW67DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Via Natural or Artificial Opening */
"0DW67JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Via Natural or Artificial Opening */
"0DW67KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Stomach, Via Natural or Artificial Opening */
"0DW67UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Via Natural or Artificial Opening */
"0DW680Z"  =  "1"  /* Revision of Drainage Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW682Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW683Z"  =  "1"  /* Revision of Infusion Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW687Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW68CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW68DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW68JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW68KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW68UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW807Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Small Intestine, Open Approach */
"0DW80JZ"  =  "1"  /* Revision of Synthetic Substitute in Small Intestine, Open Approach */
"0DW80KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Small Intestine, Open Approach */
"0DW847Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Small Intestine, Percutaneous Endoscopic Approach */
"0DW84JZ"  =  "1"  /* Revision of Synthetic Substitute in Small Intestine, Percutaneous Endoscopic Approach */
"0DW84KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Small Intestine, Percutaneous Endoscopic Approach */
"0DW877Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Small Intestine, Via Natural or Artificial Opening */
"0DW87JZ"  =  "1"  /* Revision of Synthetic Substitute in Small Intestine, Via Natural or Artificial Opening */
"0DW87KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Small Intestine, Via Natural or Artificial Opening */
"0DW887Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DW88JZ"  =  "1"  /* Revision of Synthetic Substitute in Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DW88KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DWD00Z"  =  "1"  /* Revision of Drainage Device in Lower Intestinal Tract, Open Approach */
"0DWD02Z"  =  "1"  /* Revision of Monitoring Device in Lower Intestinal Tract, Open Approach */
"0DWD03Z"  =  "1"  /* Revision of Infusion Device in Lower Intestinal Tract, Open Approach */
"0DWD07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Open Approach */
"0DWD0CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Intestinal Tract, Open Approach */
"0DWD0DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Intestinal Tract, Open Approach */
"0DWD0JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Intestinal Tract, Open Approach */
"0DWD0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Open Approach */
"0DWD0UZ"  =  "1"  /* Revision of Feeding Device in Lower Intestinal Tract, Open Approach */
"0DWD30Z"  =  "1"  /* Revision of Drainage Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD32Z"  =  "1"  /* Revision of Monitoring Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD33Z"  =  "1"  /* Revision of Infusion Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Percutaneous Approach */
"0DWD3CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD3DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD3JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Intestinal Tract, Percutaneous Approach */
"0DWD3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Percutaneous Approach */
"0DWD3UZ"  =  "1"  /* Revision of Feeding Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD40Z"  =  "1"  /* Revision of Drainage Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD42Z"  =  "1"  /* Revision of Monitoring Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD43Z"  =  "1"  /* Revision of Infusion Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD4CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD4DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD4JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD4UZ"  =  "1"  /* Revision of Feeding Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD70Z"  =  "1"  /* Revision of Drainage Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD72Z"  =  "1"  /* Revision of Monitoring Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD73Z"  =  "1"  /* Revision of Infusion Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD7CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD7DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD7JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD7UZ"  =  "1"  /* Revision of Feeding Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD80Z"  =  "1"  /* Revision of Drainage Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD82Z"  =  "1"  /* Revision of Monitoring Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD83Z"  =  "1"  /* Revision of Infusion Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD8CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD8DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD8JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD8UZ"  =  "1"  /* Revision of Feeding Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWE07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Large Intestine, Open Approach */
"0DWE0JZ"  =  "1"  /* Revision of Synthetic Substitute in Large Intestine, Open Approach */
"0DWE0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Large Intestine, Open Approach */
"0DWE47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Large Intestine, Percutaneous Endoscopic Approach */
"0DWE4JZ"  =  "1"  /* Revision of Synthetic Substitute in Large Intestine, Percutaneous Endoscopic Approach */
"0DWE4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Large Intestine, Percutaneous Endoscopic Approach */
"0DWE77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Large Intestine, Via Natural or Artificial Opening */
"0DWE7JZ"  =  "1"  /* Revision of Synthetic Substitute in Large Intestine, Via Natural or Artificial Opening */
"0DWE7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Large Intestine, Via Natural or Artificial Opening */
"0DWE87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DWE8JZ"  =  "1"  /* Revision of Synthetic Substitute in Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DWE8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DWU07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Omentum, Open Approach */
"0DWU0JZ"  =  "1"  /* Revision of Synthetic Substitute in Omentum, Open Approach */
"0DWU0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Omentum, Open Approach */
"0DWU37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Omentum, Percutaneous Approach */
"0DWU3JZ"  =  "1"  /* Revision of Synthetic Substitute in Omentum, Percutaneous Approach */
"0DWU3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Omentum, Percutaneous Approach */
"0DWU47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Omentum, Percutaneous Endoscopic Approach */
"0DWU4JZ"  =  "1"  /* Revision of Synthetic Substitute in Omentum, Percutaneous Endoscopic Approach */
"0DWU4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Omentum, Percutaneous Endoscopic Approach */
"0DWV07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mesentery, Open Approach */
"0DWV0JZ"  =  "1"  /* Revision of Synthetic Substitute in Mesentery, Open Approach */
"0DWV0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mesentery, Open Approach */
"0DWV37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mesentery, Percutaneous Approach */
"0DWV3JZ"  =  "1"  /* Revision of Synthetic Substitute in Mesentery, Percutaneous Approach */
"0DWV3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mesentery, Percutaneous Approach */
"0DWV47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mesentery, Percutaneous Endoscopic Approach */
"0DWV4JZ"  =  "1"  /* Revision of Synthetic Substitute in Mesentery, Percutaneous Endoscopic Approach */
"0DWV4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mesentery, Percutaneous Endoscopic Approach */
"0DWW07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Peritoneum, Open Approach */
"0DWW0JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneum, Open Approach */
"0DWW0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Peritoneum, Open Approach */
"0DWW37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Peritoneum, Percutaneous Approach */
"0DWW3JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneum, Percutaneous Approach */
"0DWW3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Peritoneum, Percutaneous Approach */
"0DWW47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Peritoneum, Percutaneous Endoscopic Approach */
"0DWW4JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneum, Percutaneous Endoscopic Approach */
"0DWW4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Peritoneum, Percutaneous Endoscopic Approach */
"0DX80Z5"  =  "1"  /* Transfer Small Intestine to Esophagus, Open Approach */
"0DX84Z5"  =  "1"  /* Transfer Small Intestine to Esophagus, Percutaneous Endoscopic Approach */
"0DXE0Z5"  =  "1"  /* Transfer Large Intestine to Esophagus, Open Approach */
"0DXE4Z5"  =  "1"  /* Transfer Large Intestine to Esophagus, Percutaneous Endoscopic Approach */
"0DY60Z0"  =  "1"  /* Transplantation of Stomach, Allogeneic, Open Approach */
"0DY60Z1"  =  "1"  /* Transplantation of Stomach, Syngeneic, Open Approach */
"0DY60Z2"  =  "1"  /* Transplantation of Stomach, Zooplastic, Open Approach */
"0DY80Z0"  =  "1"  /* Transplantation of Small Intestine, Allogeneic, Open Approach */
"0DY80Z1"  =  "1"  /* Transplantation of Small Intestine, Syngeneic, Open Approach */
"0DY80Z2"  =  "1"  /* Transplantation of Small Intestine, Zooplastic, Open Approach */
"0DYE0Z0"  =  "1"  /* Transplantation of Large Intestine, Allogeneic, Open Approach */
"0DYE0Z1"  =  "1"  /* Transplantation of Large Intestine, Syngeneic, Open Approach */
"0DYE0Z2"  =  "1"  /* Transplantation of Large Intestine, Zooplastic, Open Approach */
"0F140D3"  =  "1"  /* Bypass Gallbladder to Duodenum with Intraluminal Device, Open Approach */
"0F140D4"  =  "1"  /* Bypass Gallbladder to Stomach with Intraluminal Device, Open Approach */
"0F140D5"  =  "1"  /* Bypass Gallbladder to Right Hepatic Duct with Intraluminal Device, Open Approach */
"0F140D6"  =  "1"  /* Bypass Gallbladder to Left Hepatic Duct with Intraluminal Device, Open Approach */
"0F140D7"  =  "1"  /* Bypass Gallbladder to Caudate Hepatic Duct with Intraluminal Device, Open Approach */
"0F140D8"  =  "1"  /* Bypass Gallbladder to Cystic Duct with Intraluminal Device, Open Approach */
"0F140D9"  =  "1"  /* Bypass Gallbladder to Common Bile Duct with Intraluminal Device, Open Approach */
"0F140DB"  =  "1"  /* Bypass Gallbladder to Small Intestine with Intraluminal Device, Open Approach */
"0F140Z3"  =  "1"  /* Bypass Gallbladder to Duodenum, Open Approach */
"0F140Z4"  =  "1"  /* Bypass Gallbladder to Stomach, Open Approach */
"0F140Z5"  =  "1"  /* Bypass Gallbladder to Right Hepatic Duct, Open Approach */
"0F140Z6"  =  "1"  /* Bypass Gallbladder to Left Hepatic Duct, Open Approach */
"0F140Z7"  =  "1"  /* Bypass Gallbladder to Caudate Hepatic Duct, Open Approach */
"0F140Z8"  =  "1"  /* Bypass Gallbladder to Cystic Duct, Open Approach */
"0F140Z9"  =  "1"  /* Bypass Gallbladder to Common Bile Duct, Open Approach */
"0F140ZB"  =  "1"  /* Bypass Gallbladder to Small Intestine, Open Approach */
"0F144D3"  =  "1"  /* Bypass Gallbladder to Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F144D4"  =  "1"  /* Bypass Gallbladder to Stomach with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F144D5"  =  "1"  /* Bypass Gallbladder to Right Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F144D6"  =  "1"  /* Bypass Gallbladder to Left Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F144D7"  =  "1"  /* Bypass Gallbladder to Caudate Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F144D8"  =  "1"  /* Bypass Gallbladder to Cystic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F144D9"  =  "1"  /* Bypass Gallbladder to Common Bile Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F144DB"  =  "1"  /* Bypass Gallbladder to Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F144Z3"  =  "1"  /* Bypass Gallbladder to Duodenum, Percutaneous Endoscopic Approach */
"0F144Z4"  =  "1"  /* Bypass Gallbladder to Stomach, Percutaneous Endoscopic Approach */
"0F144Z5"  =  "1"  /* Bypass Gallbladder to Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0F144Z6"  =  "1"  /* Bypass Gallbladder to Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0F144Z7"  =  "1"  /* Bypass Gallbladder to Caudate Hepatic Duct, Percutaneous Endoscopic Approach */
"0F144Z8"  =  "1"  /* Bypass Gallbladder to Cystic Duct, Percutaneous Endoscopic Approach */
"0F144Z9"  =  "1"  /* Bypass Gallbladder to Common Bile Duct, Percutaneous Endoscopic Approach */
"0F144ZB"  =  "1"  /* Bypass Gallbladder to Small Intestine, Percutaneous Endoscopic Approach */
"0F150D3"  =  "1"  /* Bypass Right Hepatic Duct to Duodenum with Intraluminal Device, Open Approach */
"0F150D4"  =  "1"  /* Bypass Right Hepatic Duct to Stomach with Intraluminal Device, Open Approach */
"0F150D5"  =  "1"  /* Bypass Right Hepatic Duct to Right Hepatic Duct with Intraluminal Device, Open Approach */
"0F150D6"  =  "1"  /* Bypass Right Hepatic Duct to Left Hepatic Duct with Intraluminal Device, Open Approach */
"0F150D7"  =  "1"  /* Bypass Right Hepatic Duct to Caudate Hepatic Duct with Intraluminal Device, Open Approach */
"0F150D8"  =  "1"  /* Bypass Right Hepatic Duct to Cystic Duct with Intraluminal Device, Open Approach */
"0F150D9"  =  "1"  /* Bypass Right Hepatic Duct to Common Bile Duct with Intraluminal Device, Open Approach */
"0F150DB"  =  "1"  /* Bypass Right Hepatic Duct to Small Intestine with Intraluminal Device, Open Approach */
"0F150Z3"  =  "1"  /* Bypass Right Hepatic Duct to Duodenum, Open Approach */
"0F150Z4"  =  "1"  /* Bypass Right Hepatic Duct to Stomach, Open Approach */
"0F150Z5"  =  "1"  /* Bypass Right Hepatic Duct to Right Hepatic Duct, Open Approach */
"0F150Z6"  =  "1"  /* Bypass Right Hepatic Duct to Left Hepatic Duct, Open Approach */
"0F150Z7"  =  "1"  /* Bypass Right Hepatic Duct to Caudate Hepatic Duct, Open Approach */
"0F150Z8"  =  "1"  /* Bypass Right Hepatic Duct to Cystic Duct, Open Approach */
"0F150Z9"  =  "1"  /* Bypass Right Hepatic Duct to Common Bile Duct, Open Approach */
"0F150ZB"  =  "1"  /* Bypass Right Hepatic Duct to Small Intestine, Open Approach */
"0F154D3"  =  "1"  /* Bypass Right Hepatic Duct to Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F154D4"  =  "1"  /* Bypass Right Hepatic Duct to Stomach with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F154D5"  =  "1"  /* Bypass Right Hepatic Duct to Right Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F154D6"  =  "1"  /* Bypass Right Hepatic Duct to Left Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F154D7"  =  "1"  /* Bypass Right Hepatic Duct to Caudate Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F154D8"  =  "1"  /* Bypass Right Hepatic Duct to Cystic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F154D9"  =  "1"  /* Bypass Right Hepatic Duct to Common Bile Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F154DB"  =  "1"  /* Bypass Right Hepatic Duct to Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F154Z3"  =  "1"  /* Bypass Right Hepatic Duct to Duodenum, Percutaneous Endoscopic Approach */
"0F154Z4"  =  "1"  /* Bypass Right Hepatic Duct to Stomach, Percutaneous Endoscopic Approach */
"0F154Z5"  =  "1"  /* Bypass Right Hepatic Duct to Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0F154Z6"  =  "1"  /* Bypass Right Hepatic Duct to Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0F154Z7"  =  "1"  /* Bypass Right Hepatic Duct to Caudate Hepatic Duct, Percutaneous Endoscopic Approach */
"0F154Z8"  =  "1"  /* Bypass Right Hepatic Duct to Cystic Duct, Percutaneous Endoscopic Approach */
"0F154Z9"  =  "1"  /* Bypass Right Hepatic Duct to Common Bile Duct, Percutaneous Endoscopic Approach */
"0F154ZB"  =  "1"  /* Bypass Right Hepatic Duct to Small Intestine, Percutaneous Endoscopic Approach */
"0F160D3"  =  "1"  /* Bypass Left Hepatic Duct to Duodenum with Intraluminal Device, Open Approach */
"0F160D4"  =  "1"  /* Bypass Left Hepatic Duct to Stomach with Intraluminal Device, Open Approach */
"0F160D5"  =  "1"  /* Bypass Left Hepatic Duct to Right Hepatic Duct with Intraluminal Device, Open Approach */
"0F160D6"  =  "1"  /* Bypass Left Hepatic Duct to Left Hepatic Duct with Intraluminal Device, Open Approach */
"0F160D7"  =  "1"  /* Bypass Left Hepatic Duct to Caudate Hepatic Duct with Intraluminal Device, Open Approach */
"0F160D8"  =  "1"  /* Bypass Left Hepatic Duct to Cystic Duct with Intraluminal Device, Open Approach */
"0F160D9"  =  "1"  /* Bypass Left Hepatic Duct to Common Bile Duct with Intraluminal Device, Open Approach */
"0F160DB"  =  "1"  /* Bypass Left Hepatic Duct to Small Intestine with Intraluminal Device, Open Approach */
"0F160Z3"  =  "1"  /* Bypass Left Hepatic Duct to Duodenum, Open Approach */
"0F160Z4"  =  "1"  /* Bypass Left Hepatic Duct to Stomach, Open Approach */
"0F160Z5"  =  "1"  /* Bypass Left Hepatic Duct to Right Hepatic Duct, Open Approach */
"0F160Z6"  =  "1"  /* Bypass Left Hepatic Duct to Left Hepatic Duct, Open Approach */
"0F160Z7"  =  "1"  /* Bypass Left Hepatic Duct to Caudate Hepatic Duct, Open Approach */
"0F160Z8"  =  "1"  /* Bypass Left Hepatic Duct to Cystic Duct, Open Approach */
"0F160Z9"  =  "1"  /* Bypass Left Hepatic Duct to Common Bile Duct, Open Approach */
"0F160ZB"  =  "1"  /* Bypass Left Hepatic Duct to Small Intestine, Open Approach */
"0F164D3"  =  "1"  /* Bypass Left Hepatic Duct to Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F164D4"  =  "1"  /* Bypass Left Hepatic Duct to Stomach with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F164D5"  =  "1"  /* Bypass Left Hepatic Duct to Right Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F164D6"  =  "1"  /* Bypass Left Hepatic Duct to Left Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F164D7"  =  "1"  /* Bypass Left Hepatic Duct to Caudate Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F164D8"  =  "1"  /* Bypass Left Hepatic Duct to Cystic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F164D9"  =  "1"  /* Bypass Left Hepatic Duct to Common Bile Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F164DB"  =  "1"  /* Bypass Left Hepatic Duct to Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F164Z3"  =  "1"  /* Bypass Left Hepatic Duct to Duodenum, Percutaneous Endoscopic Approach */
"0F164Z4"  =  "1"  /* Bypass Left Hepatic Duct to Stomach, Percutaneous Endoscopic Approach */
"0F164Z5"  =  "1"  /* Bypass Left Hepatic Duct to Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0F164Z6"  =  "1"  /* Bypass Left Hepatic Duct to Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0F164Z7"  =  "1"  /* Bypass Left Hepatic Duct to Caudate Hepatic Duct, Percutaneous Endoscopic Approach */
"0F164Z8"  =  "1"  /* Bypass Left Hepatic Duct to Cystic Duct, Percutaneous Endoscopic Approach */
"0F164Z9"  =  "1"  /* Bypass Left Hepatic Duct to Common Bile Duct, Percutaneous Endoscopic Approach */
"0F164ZB"  =  "1"  /* Bypass Left Hepatic Duct to Small Intestine, Percutaneous Endoscopic Approach */
"0F180D3"  =  "1"  /* Bypass Cystic Duct to Duodenum with Intraluminal Device, Open Approach */
"0F180D4"  =  "1"  /* Bypass Cystic Duct to Stomach with Intraluminal Device, Open Approach */
"0F180D5"  =  "1"  /* Bypass Cystic Duct to Right Hepatic Duct with Intraluminal Device, Open Approach */
"0F180D6"  =  "1"  /* Bypass Cystic Duct to Left Hepatic Duct with Intraluminal Device, Open Approach */
"0F180D7"  =  "1"  /* Bypass Cystic Duct to Caudate Hepatic Duct with Intraluminal Device, Open Approach */
"0F180D8"  =  "1"  /* Bypass Cystic Duct to Cystic Duct with Intraluminal Device, Open Approach */
"0F180D9"  =  "1"  /* Bypass Cystic Duct to Common Bile Duct with Intraluminal Device, Open Approach */
"0F180DB"  =  "1"  /* Bypass Cystic Duct to Small Intestine with Intraluminal Device, Open Approach */
"0F180Z3"  =  "1"  /* Bypass Cystic Duct to Duodenum, Open Approach */
"0F180Z4"  =  "1"  /* Bypass Cystic Duct to Stomach, Open Approach */
"0F180Z5"  =  "1"  /* Bypass Cystic Duct to Right Hepatic Duct, Open Approach */
"0F180Z6"  =  "1"  /* Bypass Cystic Duct to Left Hepatic Duct, Open Approach */
"0F180Z7"  =  "1"  /* Bypass Cystic Duct to Caudate Hepatic Duct, Open Approach */
"0F180Z8"  =  "1"  /* Bypass Cystic Duct to Cystic Duct, Open Approach */
"0F180Z9"  =  "1"  /* Bypass Cystic Duct to Common Bile Duct, Open Approach */
"0F180ZB"  =  "1"  /* Bypass Cystic Duct to Small Intestine, Open Approach */
"0F184D3"  =  "1"  /* Bypass Cystic Duct to Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F184D4"  =  "1"  /* Bypass Cystic Duct to Stomach with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F184D5"  =  "1"  /* Bypass Cystic Duct to Right Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F184D6"  =  "1"  /* Bypass Cystic Duct to Left Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F184D7"  =  "1"  /* Bypass Cystic Duct to Caudate Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F184D8"  =  "1"  /* Bypass Cystic Duct to Cystic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F184D9"  =  "1"  /* Bypass Cystic Duct to Common Bile Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F184DB"  =  "1"  /* Bypass Cystic Duct to Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F184Z3"  =  "1"  /* Bypass Cystic Duct to Duodenum, Percutaneous Endoscopic Approach */
"0F184Z4"  =  "1"  /* Bypass Cystic Duct to Stomach, Percutaneous Endoscopic Approach */
"0F184Z5"  =  "1"  /* Bypass Cystic Duct to Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0F184Z6"  =  "1"  /* Bypass Cystic Duct to Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0F184Z7"  =  "1"  /* Bypass Cystic Duct to Caudate Hepatic Duct, Percutaneous Endoscopic Approach */
"0F184Z8"  =  "1"  /* Bypass Cystic Duct to Cystic Duct, Percutaneous Endoscopic Approach */
"0F184Z9"  =  "1"  /* Bypass Cystic Duct to Common Bile Duct, Percutaneous Endoscopic Approach */
"0F184ZB"  =  "1"  /* Bypass Cystic Duct to Small Intestine, Percutaneous Endoscopic Approach */
"0F190D3"  =  "1"  /* Bypass Common Bile Duct to Duodenum with Intraluminal Device, Open Approach */
"0F190D4"  =  "1"  /* Bypass Common Bile Duct to Stomach with Intraluminal Device, Open Approach */
"0F190D5"  =  "1"  /* Bypass Common Bile Duct to Right Hepatic Duct with Intraluminal Device, Open Approach */
"0F190D6"  =  "1"  /* Bypass Common Bile Duct to Left Hepatic Duct with Intraluminal Device, Open Approach */
"0F190D7"  =  "1"  /* Bypass Common Bile Duct to Caudate Hepatic Duct with Intraluminal Device, Open Approach */
"0F190D8"  =  "1"  /* Bypass Common Bile Duct to Cystic Duct with Intraluminal Device, Open Approach */
"0F190D9"  =  "1"  /* Bypass Common Bile Duct to Common Bile Duct with Intraluminal Device, Open Approach */
"0F190DB"  =  "1"  /* Bypass Common Bile Duct to Small Intestine with Intraluminal Device, Open Approach */
"0F190Z3"  =  "1"  /* Bypass Common Bile Duct to Duodenum, Open Approach */
"0F190Z4"  =  "1"  /* Bypass Common Bile Duct to Stomach, Open Approach */
"0F190Z5"  =  "1"  /* Bypass Common Bile Duct to Right Hepatic Duct, Open Approach */
"0F190Z6"  =  "1"  /* Bypass Common Bile Duct to Left Hepatic Duct, Open Approach */
"0F190Z7"  =  "1"  /* Bypass Common Bile Duct to Caudate Hepatic Duct, Open Approach */
"0F190Z8"  =  "1"  /* Bypass Common Bile Duct to Cystic Duct, Open Approach */
"0F190Z9"  =  "1"  /* Bypass Common Bile Duct to Common Bile Duct, Open Approach */
"0F190ZB"  =  "1"  /* Bypass Common Bile Duct to Small Intestine, Open Approach */
"0F194D3"  =  "1"  /* Bypass Common Bile Duct to Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F194D4"  =  "1"  /* Bypass Common Bile Duct to Stomach with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F194D5"  =  "1"  /* Bypass Common Bile Duct to Right Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F194D6"  =  "1"  /* Bypass Common Bile Duct to Left Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F194D7"  =  "1"  /* Bypass Common Bile Duct to Caudate Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F194D8"  =  "1"  /* Bypass Common Bile Duct to Cystic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F194D9"  =  "1"  /* Bypass Common Bile Duct to Common Bile Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F194DB"  =  "1"  /* Bypass Common Bile Duct to Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F194Z3"  =  "1"  /* Bypass Common Bile Duct to Duodenum, Percutaneous Endoscopic Approach */
"0F194Z4"  =  "1"  /* Bypass Common Bile Duct to Stomach, Percutaneous Endoscopic Approach */
"0F194Z5"  =  "1"  /* Bypass Common Bile Duct to Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0F194Z6"  =  "1"  /* Bypass Common Bile Duct to Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0F194Z7"  =  "1"  /* Bypass Common Bile Duct to Caudate Hepatic Duct, Percutaneous Endoscopic Approach */
"0F194Z8"  =  "1"  /* Bypass Common Bile Duct to Cystic Duct, Percutaneous Endoscopic Approach */
"0F194Z9"  =  "1"  /* Bypass Common Bile Duct to Common Bile Duct, Percutaneous Endoscopic Approach */
"0F194ZB"  =  "1"  /* Bypass Common Bile Duct to Small Intestine, Percutaneous Endoscopic Approach */
"0F1D0D3"  =  "1"  /* Bypass Pancreatic Duct to Duodenum with Intraluminal Device, Open Approach */
"0F1D0DB"  =  "1"  /* Bypass Pancreatic Duct to Small Intestine with Intraluminal Device, Open Approach */
"0F1D0DC"  =  "1"  /* Bypass Pancreatic Duct to Large Intestine with Intraluminal Device, Open Approach */
"0F1D0Z3"  =  "1"  /* Bypass Pancreatic Duct to Duodenum, Open Approach */
"0F1D0ZB"  =  "1"  /* Bypass Pancreatic Duct to Small Intestine, Open Approach */
"0F1D0ZC"  =  "1"  /* Bypass Pancreatic Duct to Large Intestine, Open Approach */
"0F1D4D3"  =  "1"  /* Bypass Pancreatic Duct to Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1D4DB"  =  "1"  /* Bypass Pancreatic Duct to Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1D4DC"  =  "1"  /* Bypass Pancreatic Duct to Large Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1D4Z3"  =  "1"  /* Bypass Pancreatic Duct to Duodenum, Percutaneous Endoscopic Approach */
"0F1D4ZB"  =  "1"  /* Bypass Pancreatic Duct to Small Intestine, Percutaneous Endoscopic Approach */
"0F1D4ZC"  =  "1"  /* Bypass Pancreatic Duct to Large Intestine, Percutaneous Endoscopic Approach */
"0F1F0D3"  =  "1"  /* Bypass Accessory Pancreatic Duct to Duodenum with Intraluminal Device, Open Approach */
"0F1F0DB"  =  "1"  /* Bypass Accessory Pancreatic Duct to Small Intestine with Intraluminal Device, Open Approach */
"0F1F0DC"  =  "1"  /* Bypass Accessory Pancreatic Duct to Large Intestine with Intraluminal Device, Open Approach */
"0F1F0Z3"  =  "1"  /* Bypass Accessory Pancreatic Duct to Duodenum, Open Approach */
"0F1F0ZB"  =  "1"  /* Bypass Accessory Pancreatic Duct to Small Intestine, Open Approach */
"0F1F0ZC"  =  "1"  /* Bypass Accessory Pancreatic Duct to Large Intestine, Open Approach */
"0F1F4D3"  =  "1"  /* Bypass Accessory Pancreatic Duct to Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1F4DB"  =  "1"  /* Bypass Accessory Pancreatic Duct to Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1F4DC"  =  "1"  /* Bypass Accessory Pancreatic Duct to Large Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1F4Z3"  =  "1"  /* Bypass Accessory Pancreatic Duct to Duodenum, Percutaneous Endoscopic Approach */
"0F1F4ZB"  =  "1"  /* Bypass Accessory Pancreatic Duct to Small Intestine, Percutaneous Endoscopic Approach */
"0F1F4ZC"  =  "1"  /* Bypass Accessory Pancreatic Duct to Large Intestine, Percutaneous Endoscopic Approach */
"0F1G0D3"  =  "1"  /* Bypass Pancreas to Duodenum with Intraluminal Device, Open Approach */
"0F1G0DB"  =  "1"  /* Bypass Pancreas to Small Intestine with Intraluminal Device, Open Approach */
"0F1G0DC"  =  "1"  /* Bypass Pancreas to Large Intestine with Intraluminal Device, Open Approach */
"0F1G0Z3"  =  "1"  /* Bypass Pancreas to Duodenum, Open Approach */
"0F1G0ZB"  =  "1"  /* Bypass Pancreas to Small Intestine, Open Approach */
"0F1G0ZC"  =  "1"  /* Bypass Pancreas to Large Intestine, Open Approach */
"0F1G4D3"  =  "1"  /* Bypass Pancreas to Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1G4DB"  =  "1"  /* Bypass Pancreas to Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1G4DC"  =  "1"  /* Bypass Pancreas to Large Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1G4Z3"  =  "1"  /* Bypass Pancreas to Duodenum, Percutaneous Endoscopic Approach */
"0F1G4ZB"  =  "1"  /* Bypass Pancreas to Small Intestine, Percutaneous Endoscopic Approach */
"0F1G4ZC"  =  "1"  /* Bypass Pancreas to Large Intestine, Percutaneous Endoscopic Approach */
"0F500ZZ"  =  "1"  /* Destruction of Liver, Open Approach */
"0F503ZZ"  =  "1"  /* Destruction of Liver, Percutaneous Approach */
"0F504ZZ"  =  "1"  /* Destruction of Liver, Percutaneous Endoscopic Approach */
"0F510ZZ"  =  "1"  /* Destruction of Right Lobe Liver, Open Approach */
"0F513ZZ"  =  "1"  /* Destruction of Right Lobe Liver, Percutaneous Approach */
"0F514ZZ"  =  "1"  /* Destruction of Right Lobe Liver, Percutaneous Endoscopic Approach */
"0F520ZZ"  =  "1"  /* Destruction of Left Lobe Liver, Open Approach */
"0F523ZZ"  =  "1"  /* Destruction of Left Lobe Liver, Percutaneous Approach */
"0F524ZZ"  =  "1"  /* Destruction of Left Lobe Liver, Percutaneous Endoscopic Approach */
"0F540ZZ"  =  "1"  /* Destruction of Gallbladder, Open Approach */
"0F543ZZ"  =  "1"  /* Destruction of Gallbladder, Percutaneous Approach */
"0F544ZZ"  =  "1"  /* Destruction of Gallbladder, Percutaneous Endoscopic Approach */
"0F550ZZ"  =  "1"  /* Destruction of Right Hepatic Duct, Open Approach */
"0F553ZZ"  =  "1"  /* Destruction of Right Hepatic Duct, Percutaneous Approach */
"0F557ZZ"  =  "1"  /* Destruction of Right Hepatic Duct, Via Natural or Artificial Opening */
"0F560ZZ"  =  "1"  /* Destruction of Left Hepatic Duct, Open Approach */
"0F563ZZ"  =  "1"  /* Destruction of Left Hepatic Duct, Percutaneous Approach */
"0F567ZZ"  =  "1"  /* Destruction of Left Hepatic Duct, Via Natural or Artificial Opening */
"0F580ZZ"  =  "1"  /* Destruction of Cystic Duct, Open Approach */
"0F583ZZ"  =  "1"  /* Destruction of Cystic Duct, Percutaneous Approach */
"0F587ZZ"  =  "1"  /* Destruction of Cystic Duct, Via Natural or Artificial Opening */
"0F590ZZ"  =  "1"  /* Destruction of Common Bile Duct, Open Approach */
"0F593ZZ"  =  "1"  /* Destruction of Common Bile Duct, Percutaneous Approach */
"0F597ZZ"  =  "1"  /* Destruction of Common Bile Duct, Via Natural or Artificial Opening */
"0F5C0ZZ"  =  "1"  /* Destruction of Ampulla of Vater, Open Approach */
"0F5C3ZZ"  =  "1"  /* Destruction of Ampulla of Vater, Percutaneous Approach */
"0F5C7ZZ"  =  "1"  /* Destruction of Ampulla of Vater, Via Natural or Artificial Opening */
"0F5D0ZZ"  =  "1"  /* Destruction of Pancreatic Duct, Open Approach */
"0F5D3ZZ"  =  "1"  /* Destruction of Pancreatic Duct, Percutaneous Approach */
"0F5D7ZZ"  =  "1"  /* Destruction of Pancreatic Duct, Via Natural or Artificial Opening */
"0F5F0ZZ"  =  "1"  /* Destruction of Accessory Pancreatic Duct, Open Approach */
"0F5F3ZZ"  =  "1"  /* Destruction of Accessory Pancreatic Duct, Percutaneous Approach */
"0F5F7ZZ"  =  "1"  /* Destruction of Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0F5G0ZZ"  =  "1"  /* Destruction of Pancreas, Open Approach */
"0F5G3ZZ"  =  "1"  /* Destruction of Pancreas, Percutaneous Approach */
"0F750DZ"  =  "1"  /* Dilation of Right Hepatic Duct with Intraluminal Device, Open Approach */
"0F750ZZ"  =  "1"  /* Dilation of Right Hepatic Duct, Open Approach */
"0F757DZ"  =  "1"  /* Dilation of Right Hepatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0F757ZZ"  =  "1"  /* Dilation of Right Hepatic Duct, Via Natural or Artificial Opening */
"0F760DZ"  =  "1"  /* Dilation of Left Hepatic Duct with Intraluminal Device, Open Approach */
"0F760ZZ"  =  "1"  /* Dilation of Left Hepatic Duct, Open Approach */
"0F767DZ"  =  "1"  /* Dilation of Left Hepatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0F767ZZ"  =  "1"  /* Dilation of Left Hepatic Duct, Via Natural or Artificial Opening */
"0F780DZ"  =  "1"  /* Dilation of Cystic Duct with Intraluminal Device, Open Approach */
"0F780ZZ"  =  "1"  /* Dilation of Cystic Duct, Open Approach */
"0F787DZ"  =  "1"  /* Dilation of Cystic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0F787ZZ"  =  "1"  /* Dilation of Cystic Duct, Via Natural or Artificial Opening */
"0F790DZ"  =  "1"  /* Dilation of Common Bile Duct with Intraluminal Device, Open Approach */
"0F790ZZ"  =  "1"  /* Dilation of Common Bile Duct, Open Approach */
"0F797DZ"  =  "1"  /* Dilation of Common Bile Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0F797ZZ"  =  "1"  /* Dilation of Common Bile Duct, Via Natural or Artificial Opening */
"0F7C0DZ"  =  "1"  /* Dilation of Ampulla of Vater with Intraluminal Device, Open Approach */
"0F7C0ZZ"  =  "1"  /* Dilation of Ampulla of Vater, Open Approach */
"0F7C3DZ"  =  "1"  /* Dilation of Ampulla of Vater with Intraluminal Device, Percutaneous Approach */
"0F7C3ZZ"  =  "1"  /* Dilation of Ampulla of Vater, Percutaneous Approach */
"0F7C4DZ"  =  "1"  /* Dilation of Ampulla of Vater with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F7C4ZZ"  =  "1"  /* Dilation of Ampulla of Vater, Percutaneous Endoscopic Approach */
"0F7C7DZ"  =  "1"  /* Dilation of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening */
"0F7C7ZZ"  =  "1"  /* Dilation of Ampulla of Vater, Via Natural or Artificial Opening */
"0F7D0DZ"  =  "1"  /* Dilation of Pancreatic Duct with Intraluminal Device, Open Approach */
"0F7D0ZZ"  =  "1"  /* Dilation of Pancreatic Duct, Open Approach */
"0F7D3DZ"  =  "1"  /* Dilation of Pancreatic Duct with Intraluminal Device, Percutaneous Approach */
"0F7D3ZZ"  =  "1"  /* Dilation of Pancreatic Duct, Percutaneous Approach */
"0F7D7DZ"  =  "1"  /* Dilation of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0F7D7ZZ"  =  "1"  /* Dilation of Pancreatic Duct, Via Natural or Artificial Opening */
"0F7F0DZ"  =  "1"  /* Dilation of Accessory Pancreatic Duct with Intraluminal Device, Open Approach */
"0F7F0ZZ"  =  "1"  /* Dilation of Accessory Pancreatic Duct, Open Approach */
"0F7F3DZ"  =  "1"  /* Dilation of Accessory Pancreatic Duct with Intraluminal Device, Percutaneous Approach */
"0F7F3ZZ"  =  "1"  /* Dilation of Accessory Pancreatic Duct, Percutaneous Approach */
"0F7F7DZ"  =  "1"  /* Dilation of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0F7F7ZZ"  =  "1"  /* Dilation of Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0F8G0ZZ"  =  "1"  /* Division of Pancreas, Open Approach */
"0F8G3ZZ"  =  "1"  /* Division of Pancreas, Percutaneous Approach */
"0F8G4ZZ"  =  "1"  /* Division of Pancreas, Percutaneous Endoscopic Approach */
"0F9000Z"  =  "1"  /* Drainage of Liver with Drainage Device, Open Approach */
"0F900ZX"  =  "1"  /* Drainage of Liver, Open Approach, Diagnostic */
"0F900ZZ"  =  "1"  /* Drainage of Liver, Open Approach */
"0F903ZZ"  =  "1"  /* Drainage of Liver, Percutaneous Approach */
"0F904ZZ"  =  "1"  /* Drainage of Liver, Percutaneous Endoscopic Approach */
"0F9100Z"  =  "1"  /* Drainage of Right Lobe Liver with Drainage Device, Open Approach */
"0F910ZX"  =  "1"  /* Drainage of Right Lobe Liver, Open Approach, Diagnostic */
"0F910ZZ"  =  "1"  /* Drainage of Right Lobe Liver, Open Approach */
"0F9200Z"  =  "1"  /* Drainage of Left Lobe Liver with Drainage Device, Open Approach */
"0F920ZX"  =  "1"  /* Drainage of Left Lobe Liver, Open Approach, Diagnostic */
"0F920ZZ"  =  "1"  /* Drainage of Left Lobe Liver, Open Approach */
"0F9400Z"  =  "1"  /* Drainage of Gallbladder with Drainage Device, Open Approach */
"0F940ZX"  =  "1"  /* Drainage of Gallbladder, Open Approach, Diagnostic */
"0F940ZZ"  =  "1"  /* Drainage of Gallbladder, Open Approach */
"0F9430Z"  =  "1"  /* Drainage of Gallbladder with Drainage Device, Percutaneous Approach */
"0F9500Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drainage Device, Open Approach */
"0F950ZX"  =  "1"  /* Drainage of Right Hepatic Duct, Open Approach, Diagnostic */
"0F950ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Open Approach */
"0F9530Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drainage Device, Percutaneous Approach */
"0F953ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Percutaneous Approach */
"0F9540Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drainage Device, Percutaneous Endoscopic Approach */
"0F954ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0F9570Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drainage Device, Via Natural or Artificial Opening */
"0F957ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Via Natural or Artificial Opening */
"0F9580Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0F958ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0F9600Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drainage Device, Open Approach */
"0F960ZX"  =  "1"  /* Drainage of Left Hepatic Duct, Open Approach, Diagnostic */
"0F960ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Open Approach */
"0F9630Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drainage Device, Percutaneous Approach */
"0F963ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Percutaneous Approach */
"0F9640Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drainage Device, Percutaneous Endoscopic Approach */
"0F964ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0F9670Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drainage Device, Via Natural or Artificial Opening */
"0F967ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Via Natural or Artificial Opening */
"0F9680Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0F968ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0F9800Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Open Approach */
"0F980ZX"  =  "1"  /* Drainage of Cystic Duct, Open Approach, Diagnostic */
"0F980ZZ"  =  "1"  /* Drainage of Cystic Duct, Open Approach */
"0F9830Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Percutaneous Approach */
"0F983ZZ"  =  "1"  /* Drainage of Cystic Duct, Percutaneous Approach */
"0F9840Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Percutaneous Endoscopic Approach */
"0F984ZZ"  =  "1"  /* Drainage of Cystic Duct, Percutaneous Endoscopic Approach */
"0F9870Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Via Natural or Artificial Opening */
"0F987ZZ"  =  "1"  /* Drainage of Cystic Duct, Via Natural or Artificial Opening */
"0F9880Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0F988ZZ"  =  "1"  /* Drainage of Cystic Duct, Via Natural or Artificial Opening Endoscopic */
"0F9900Z"  =  "1"  /* Drainage of Common Bile Duct with Drainage Device, Open Approach */
"0F990ZX"  =  "1"  /* Drainage of Common Bile Duct, Open Approach, Diagnostic */
"0F990ZZ"  =  "1"  /* Drainage of Common Bile Duct, Open Approach */
"0F9930Z"  =  "1"  /* Drainage of Common Bile Duct with Drainage Device, Percutaneous Approach */
"0F9940Z"  =  "1"  /* Drainage of Common Bile Duct with Drainage Device, Percutaneous Endoscopic Approach */
"0F9970Z"  =  "1"  /* Drainage of Common Bile Duct with Drainage Device, Via Natural or Artificial Opening */
"0F9C00Z"  =  "1"  /* Drainage of Ampulla of Vater with Drainage Device, Open Approach */
"0F9C0ZX"  =  "1"  /* Drainage of Ampulla of Vater, Open Approach, Diagnostic */
"0F9C0ZZ"  =  "1"  /* Drainage of Ampulla of Vater, Open Approach */
"0F9C30Z"  =  "1"  /* Drainage of Ampulla of Vater with Drainage Device, Percutaneous Approach */
"0F9C3ZZ"  =  "1"  /* Drainage of Ampulla of Vater, Percutaneous Approach */
"0F9C70Z"  =  "1"  /* Drainage of Ampulla of Vater with Drainage Device, Via Natural or Artificial Opening */
"0F9C7ZZ"  =  "1"  /* Drainage of Ampulla of Vater, Via Natural or Artificial Opening */
"0F9D00Z"  =  "1"  /* Drainage of Pancreatic Duct with Drainage Device, Open Approach */
"0F9D0ZX"  =  "1"  /* Drainage of Pancreatic Duct, Open Approach, Diagnostic */
"0F9D0ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Open Approach */
"0F9D30Z"  =  "1"  /* Drainage of Pancreatic Duct with Drainage Device, Percutaneous Approach */
"0F9D3ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Percutaneous Approach */
"0F9D40Z"  =  "1"  /* Drainage of Pancreatic Duct with Drainage Device, Percutaneous Endoscopic Approach */
"0F9D4ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Percutaneous Endoscopic Approach */
"0F9D70Z"  =  "1"  /* Drainage of Pancreatic Duct with Drainage Device, Via Natural or Artificial Opening */
"0F9D7ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Via Natural or Artificial Opening */
"0F9D8ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0F9F00Z"  =  "1"  /* Drainage of Accessory Pancreatic Duct with Drainage Device, Open Approach */
"0F9F0ZX"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Open Approach, Diagnostic */
"0F9F0ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Open Approach */
"0F9F30Z"  =  "1"  /* Drainage of Accessory Pancreatic Duct with Drainage Device, Percutaneous Approach */
"0F9F3ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Percutaneous Approach */
"0F9F40Z"  =  "1"  /* Drainage of Accessory Pancreatic Duct with Drainage Device, Percutaneous Endoscopic Approach */
"0F9F4ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0F9F70Z"  =  "1"  /* Drainage of Accessory Pancreatic Duct with Drainage Device, Via Natural or Artificial Opening */
"0F9F7ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0F9F8ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0F9G00Z"  =  "1"  /* Drainage of Pancreas with Drainage Device, Open Approach */
"0F9G0ZX"  =  "1"  /* Drainage of Pancreas, Open Approach, Diagnostic */
"0F9G0ZZ"  =  "1"  /* Drainage of Pancreas, Open Approach */
"0F9G30Z"  =  "1"  /* Drainage of Pancreas with Drainage Device, Percutaneous Approach */
"0F9G3ZZ"  =  "1"  /* Drainage of Pancreas, Percutaneous Approach */
"0F9G40Z"  =  "1"  /* Drainage of Pancreas with Drainage Device, Percutaneous Endoscopic Approach */
"0F9G4ZZ"  =  "1"  /* Drainage of Pancreas, Percutaneous Endoscopic Approach */
"0FB00ZX"  =  "1"  /* Excision of Liver, Open Approach, Diagnostic */
"0FB00ZZ"  =  "1"  /* Excision of Liver, Open Approach */
"0FB03ZZ"  =  "1"  /* Excision of Liver, Percutaneous Approach */
"0FB04ZX"  =  "1"  /* Excision of Liver, Percutaneous Endoscopic Approach, Diagnostic */
"0FB04ZZ"  =  "1"  /* Excision of Liver, Percutaneous Endoscopic Approach */
"0FB10ZX"  =  "1"  /* Excision of Right Lobe Liver, Open Approach, Diagnostic */
"0FB10ZZ"  =  "1"  /* Excision of Right Lobe Liver, Open Approach */
"0FB13ZZ"  =  "1"  /* Excision of Right Lobe Liver, Percutaneous Approach */
"0FB14ZX"  =  "1"  /* Excision of Right Lobe Liver, Percutaneous Endoscopic Approach, Diagnostic */
"0FB14ZZ"  =  "1"  /* Excision of Right Lobe Liver, Percutaneous Endoscopic Approach */
"0FB20ZX"  =  "1"  /* Excision of Left Lobe Liver, Open Approach, Diagnostic */
"0FB20ZZ"  =  "1"  /* Excision of Left Lobe Liver, Open Approach */
"0FB23ZZ"  =  "1"  /* Excision of Left Lobe Liver, Percutaneous Approach */
"0FB24ZX"  =  "1"  /* Excision of Left Lobe Liver, Percutaneous Endoscopic Approach, Diagnostic */
"0FB24ZZ"  =  "1"  /* Excision of Left Lobe Liver, Percutaneous Endoscopic Approach */
"0FB40ZX"  =  "1"  /* Excision of Gallbladder, Open Approach, Diagnostic */
"0FB40ZZ"  =  "1"  /* Excision of Gallbladder, Open Approach */
"0FB43ZZ"  =  "1"  /* Excision of Gallbladder, Percutaneous Approach */
"0FB44ZZ"  =  "1"  /* Excision of Gallbladder, Percutaneous Endoscopic Approach */
"0FB50ZX"  =  "1"  /* Excision of Right Hepatic Duct, Open Approach, Diagnostic */
"0FB50ZZ"  =  "1"  /* Excision of Right Hepatic Duct, Open Approach */
"0FB53ZZ"  =  "1"  /* Excision of Right Hepatic Duct, Percutaneous Approach */
"0FB57ZZ"  =  "1"  /* Excision of Right Hepatic Duct, Via Natural or Artificial Opening */
"0FB60ZX"  =  "1"  /* Excision of Left Hepatic Duct, Open Approach, Diagnostic */
"0FB60ZZ"  =  "1"  /* Excision of Left Hepatic Duct, Open Approach */
"0FB63ZZ"  =  "1"  /* Excision of Left Hepatic Duct, Percutaneous Approach */
"0FB67ZZ"  =  "1"  /* Excision of Left Hepatic Duct, Via Natural or Artificial Opening */
"0FB80ZX"  =  "1"  /* Excision of Cystic Duct, Open Approach, Diagnostic */
"0FB80ZZ"  =  "1"  /* Excision of Cystic Duct, Open Approach */
"0FB83ZZ"  =  "1"  /* Excision of Cystic Duct, Percutaneous Approach */
"0FB87ZZ"  =  "1"  /* Excision of Cystic Duct, Via Natural or Artificial Opening */
"0FB90ZX"  =  "1"  /* Excision of Common Bile Duct, Open Approach, Diagnostic */
"0FB90ZZ"  =  "1"  /* Excision of Common Bile Duct, Open Approach */
"0FB93ZZ"  =  "1"  /* Excision of Common Bile Duct, Percutaneous Approach */
"0FB97ZZ"  =  "1"  /* Excision of Common Bile Duct, Via Natural or Artificial Opening */
"0FBC0ZX"  =  "1"  /* Excision of Ampulla of Vater, Open Approach, Diagnostic */
"0FBC0ZZ"  =  "1"  /* Excision of Ampulla of Vater, Open Approach */
"0FBC3ZZ"  =  "1"  /* Excision of Ampulla of Vater, Percutaneous Approach */
"0FBC7ZZ"  =  "1"  /* Excision of Ampulla of Vater, Via Natural or Artificial Opening */
"0FBD0ZX"  =  "1"  /* Excision of Pancreatic Duct, Open Approach, Diagnostic */
"0FBD0ZZ"  =  "1"  /* Excision of Pancreatic Duct, Open Approach */
"0FBD3ZZ"  =  "1"  /* Excision of Pancreatic Duct, Percutaneous Approach */
"0FBD7ZZ"  =  "1"  /* Excision of Pancreatic Duct, Via Natural or Artificial Opening */
"0FBF0ZX"  =  "1"  /* Excision of Accessory Pancreatic Duct, Open Approach, Diagnostic */
"0FBF0ZZ"  =  "1"  /* Excision of Accessory Pancreatic Duct, Open Approach */
"0FBF3ZZ"  =  "1"  /* Excision of Accessory Pancreatic Duct, Percutaneous Approach */
"0FBF7ZZ"  =  "1"  /* Excision of Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FBG0ZX"  =  "1"  /* Excision of Pancreas, Open Approach, Diagnostic */
"0FBG0ZZ"  =  "1"  /* Excision of Pancreas, Open Approach */
"0FBG3ZZ"  =  "1"  /* Excision of Pancreas, Percutaneous Approach */
"0FBG4ZZ"  =  "1"  /* Excision of Pancreas, Percutaneous Endoscopic Approach */
"0FC00ZZ"  =  "1"  /* Extirpation of Matter from Liver, Open Approach */
"0FC03ZZ"  =  "1"  /* Extirpation of Matter from Liver, Percutaneous Approach */
"0FC04ZZ"  =  "1"  /* Extirpation of Matter from Liver, Percutaneous Endoscopic Approach */
"0FC10ZZ"  =  "1"  /* Extirpation of Matter from Right Lobe Liver, Open Approach */
"0FC13ZZ"  =  "1"  /* Extirpation of Matter from Right Lobe Liver, Percutaneous Approach */
"0FC14ZZ"  =  "1"  /* Extirpation of Matter from Right Lobe Liver, Percutaneous Endoscopic Approach */
"0FC20ZZ"  =  "1"  /* Extirpation of Matter from Left Lobe Liver, Open Approach */
"0FC23ZZ"  =  "1"  /* Extirpation of Matter from Left Lobe Liver, Percutaneous Approach */
"0FC24ZZ"  =  "1"  /* Extirpation of Matter from Left Lobe Liver, Percutaneous Endoscopic Approach */
"0FC40ZZ"  =  "1"  /* Extirpation of Matter from Gallbladder, Open Approach */
"0FC43ZZ"  =  "1"  /* Extirpation of Matter from Gallbladder, Percutaneous Approach */
"0FC44ZZ"  =  "1"  /* Extirpation of Matter from Gallbladder, Percutaneous Endoscopic Approach */
"0FC50ZZ"  =  "1"  /* Extirpation of Matter from Right Hepatic Duct, Open Approach */
"0FC60ZZ"  =  "1"  /* Extirpation of Matter from Left Hepatic Duct, Open Approach */
"0FC80ZZ"  =  "1"  /* Extirpation of Matter from Cystic Duct, Open Approach */
"0FC90ZZ"  =  "1"  /* Extirpation of Matter from Common Bile Duct, Open Approach */
"0FCC0ZZ"  =  "1"  /* Extirpation of Matter from Ampulla of Vater, Open Approach */
"0FCC3ZZ"  =  "1"  /* Extirpation of Matter from Ampulla of Vater, Percutaneous Approach */
"0FCC7ZZ"  =  "1"  /* Extirpation of Matter from Ampulla of Vater, Via Natural or Artificial Opening */
"0FCD0ZZ"  =  "1"  /* Extirpation of Matter from Pancreatic Duct, Open Approach */
"0FCD7ZZ"  =  "1"  /* Extirpation of Matter from Pancreatic Duct, Via Natural or Artificial Opening */
"0FCF0ZZ"  =  "1"  /* Extirpation of Matter from Accessory Pancreatic Duct, Open Approach */
"0FCF7ZZ"  =  "1"  /* Extirpation of Matter from Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FCG0ZZ"  =  "1"  /* Extirpation of Matter from Pancreas, Open Approach */
"0FCG3ZZ"  =  "1"  /* Extirpation of Matter from Pancreas, Percutaneous Approach */
"0FCG4ZZ"  =  "1"  /* Extirpation of Matter from Pancreas, Percutaneous Endoscopic Approach */
"0FF40ZZ"  =  "1"  /* Fragmentation in Gallbladder, Open Approach */
"0FF43ZZ"  =  "1"  /* Fragmentation in Gallbladder, Percutaneous Approach */
"0FF44ZZ"  =  "1"  /* Fragmentation in Gallbladder, Percutaneous Endoscopic Approach */
"0FF47ZZ"  =  "1"  /* Fragmentation in Gallbladder, Via Natural or Artificial Opening */
"0FF50ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Open Approach */
"0FF53ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Percutaneous Approach */
"0FF54ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0FF57ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Via Natural or Artificial Opening */
"0FF60ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Open Approach */
"0FF63ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Percutaneous Approach */
"0FF64ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0FF67ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Via Natural or Artificial Opening */
"0FF80ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Open Approach */
"0FF83ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Percutaneous Approach */
"0FF84ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Percutaneous Endoscopic Approach */
"0FF87ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Via Natural or Artificial Opening */
"0FF90ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Open Approach */
"0FF93ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Percutaneous Approach */
"0FF94ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Percutaneous Endoscopic Approach */
"0FF97ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Via Natural or Artificial Opening */
"0FFC0ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Open Approach */
"0FFC3ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Percutaneous Approach */
"0FFC4ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FFC7ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Via Natural or Artificial Opening */
"0FFD0ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Open Approach */
"0FFD3ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Percutaneous Approach */
"0FFD4ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FFD7ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Via Natural or Artificial Opening */
"0FFD8ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FFF0ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Open Approach */
"0FFF3ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Percutaneous Approach */
"0FFF4ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FFF7ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FFF8ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FH002Z"  =  "1"  /* Insertion of Monitoring Device into Liver, Open Approach */
"0FH032Z"  =  "1"  /* Insertion of Monitoring Device into Liver, Percutaneous Approach */
"0FH042Z"  =  "1"  /* Insertion of Monitoring Device into Liver, Percutaneous Endoscopic Approach */
"0FH102Z"  =  "1"  /* Insertion of Monitoring Device into Right Lobe Liver, Open Approach */
"0FH132Z"  =  "1"  /* Insertion of Monitoring Device into Right Lobe Liver, Percutaneous Approach */
"0FH142Z"  =  "1"  /* Insertion of Monitoring Device into Right Lobe Liver, Percutaneous Endoscopic Approach */
"0FH202Z"  =  "1"  /* Insertion of Monitoring Device into Left Lobe Liver, Open Approach */
"0FH232Z"  =  "1"  /* Insertion of Monitoring Device into Left Lobe Liver, Percutaneous Approach */
"0FH242Z"  =  "1"  /* Insertion of Monitoring Device into Left Lobe Liver, Percutaneous Endoscopic Approach */
"0FH402Z"  =  "1"  /* Insertion of Monitoring Device into Gallbladder, Open Approach */
"0FH432Z"  =  "1"  /* Insertion of Monitoring Device into Gallbladder, Percutaneous Approach */
"0FH442Z"  =  "1"  /* Insertion of Monitoring Device into Gallbladder, Percutaneous Endoscopic Approach */
"0FHB02Z"  =  "1"  /* Insertion of Monitoring Device into Hepatobiliary Duct, Open Approach */
"0FHB0DZ"  =  "1"  /* Insertion of Intraluminal Device into Hepatobiliary Duct, Open Approach */
"0FHB32Z"  =  "1"  /* Insertion of Monitoring Device into Hepatobiliary Duct, Percutaneous Approach */
"0FHB3DZ"  =  "1"  /* Insertion of Intraluminal Device into Hepatobiliary Duct, Percutaneous Approach */
"0FHB42Z"  =  "1"  /* Insertion of Monitoring Device into Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FHB72Z"  =  "1"  /* Insertion of Monitoring Device into Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FHB7DZ"  =  "1"  /* Insertion of Intraluminal Device into Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FHB82Z"  =  "1"  /* Insertion of Monitoring Device into Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FHD02Z"  =  "1"  /* Insertion of Monitoring Device into Pancreatic Duct, Open Approach */
"0FHD0DZ"  =  "1"  /* Insertion of Intraluminal Device into Pancreatic Duct, Open Approach */
"0FHD32Z"  =  "1"  /* Insertion of Monitoring Device into Pancreatic Duct, Percutaneous Approach */
"0FHD3DZ"  =  "1"  /* Insertion of Intraluminal Device into Pancreatic Duct, Percutaneous Approach */
"0FHD42Z"  =  "1"  /* Insertion of Monitoring Device into Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FHD72Z"  =  "1"  /* Insertion of Monitoring Device into Pancreatic Duct, Via Natural or Artificial Opening */
"0FHD7DZ"  =  "1"  /* Insertion of Intraluminal Device into Pancreatic Duct, Via Natural or Artificial Opening */
"0FHD82Z"  =  "1"  /* Insertion of Monitoring Device into Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FHG02Z"  =  "1"  /* Insertion of Monitoring Device into Pancreas, Open Approach */
"0FHG32Z"  =  "1"  /* Insertion of Monitoring Device into Pancreas, Percutaneous Approach */
"0FHG42Z"  =  "1"  /* Insertion of Monitoring Device into Pancreas, Percutaneous Endoscopic Approach */
"0FJ00ZZ"  =  "1"  /* Inspection of Liver, Open Approach */
"0FJ03ZZ"  =  "1"  /* Inspection of Liver, Percutaneous Approach */
"0FJ04ZZ"  =  "1"  /* Inspection of Liver, Percutaneous Endoscopic Approach */
"0FJ40ZZ"  =  "1"  /* Inspection of Gallbladder, Open Approach */
"0FJ43ZZ"  =  "1"  /* Inspection of Gallbladder, Percutaneous Approach */
"0FJ44ZZ"  =  "1"  /* Inspection of Gallbladder, Percutaneous Endoscopic Approach */
"0FJB0ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Open Approach */
"0FJB3ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Percutaneous Approach */
"0FJB4ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FJB7ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FJB8ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FJD0ZZ"  =  "1"  /* Inspection of Pancreatic Duct, Open Approach */
"0FJD3ZZ"  =  "1"  /* Inspection of Pancreatic Duct, Percutaneous Approach */
"0FJD4ZZ"  =  "1"  /* Inspection of Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FJD7ZZ"  =  "1"  /* Inspection of Pancreatic Duct, Via Natural or Artificial Opening */
"0FJD8ZZ"  =  "1"  /* Inspection of Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FJG0ZZ"  =  "1"  /* Inspection of Pancreas, Open Approach */
"0FJG3ZZ"  =  "1"  /* Inspection of Pancreas, Percutaneous Approach */
"0FJG4ZZ"  =  "1"  /* Inspection of Pancreas, Percutaneous Endoscopic Approach */
"0FL50CZ"  =  "1"  /* Occlusion of Right Hepatic Duct with Extraluminal Device, Open Approach */
"0FL50DZ"  =  "1"  /* Occlusion of Right Hepatic Duct with Intraluminal Device, Open Approach */
"0FL50ZZ"  =  "1"  /* Occlusion of Right Hepatic Duct, Open Approach */
"0FL60CZ"  =  "1"  /* Occlusion of Left Hepatic Duct with Extraluminal Device, Open Approach */
"0FL60DZ"  =  "1"  /* Occlusion of Left Hepatic Duct with Intraluminal Device, Open Approach */
"0FL60ZZ"  =  "1"  /* Occlusion of Left Hepatic Duct, Open Approach */
"0FL80CZ"  =  "1"  /* Occlusion of Cystic Duct with Extraluminal Device, Open Approach */
"0FL80DZ"  =  "1"  /* Occlusion of Cystic Duct with Intraluminal Device, Open Approach */
"0FL80ZZ"  =  "1"  /* Occlusion of Cystic Duct, Open Approach */
"0FL90CZ"  =  "1"  /* Occlusion of Common Bile Duct with Extraluminal Device, Open Approach */
"0FL90DZ"  =  "1"  /* Occlusion of Common Bile Duct with Intraluminal Device, Open Approach */
"0FL90ZZ"  =  "1"  /* Occlusion of Common Bile Duct, Open Approach */
"0FLC0CZ"  =  "1"  /* Occlusion of Ampulla of Vater with Extraluminal Device, Open Approach */
"0FLC0DZ"  =  "1"  /* Occlusion of Ampulla of Vater with Intraluminal Device, Open Approach */
"0FLC0ZZ"  =  "1"  /* Occlusion of Ampulla of Vater, Open Approach */
"0FLC3CZ"  =  "1"  /* Occlusion of Ampulla of Vater with Extraluminal Device, Percutaneous Approach */
"0FLC3DZ"  =  "1"  /* Occlusion of Ampulla of Vater with Intraluminal Device, Percutaneous Approach */
"0FLC3ZZ"  =  "1"  /* Occlusion of Ampulla of Vater, Percutaneous Approach */
"0FLC4CZ"  =  "1"  /* Occlusion of Ampulla of Vater with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FLC4DZ"  =  "1"  /* Occlusion of Ampulla of Vater with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FLC4ZZ"  =  "1"  /* Occlusion of Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FLC7DZ"  =  "1"  /* Occlusion of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening */
"0FLC7ZZ"  =  "1"  /* Occlusion of Ampulla of Vater, Via Natural or Artificial Opening */
"0FLC8DZ"  =  "1"  /* Occlusion of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FLC8ZZ"  =  "1"  /* Occlusion of Ampulla of Vater, Via Natural or Artificial Opening Endoscopic */
"0FLD0CZ"  =  "1"  /* Occlusion of Pancreatic Duct with Extraluminal Device, Open Approach */
"0FLD0DZ"  =  "1"  /* Occlusion of Pancreatic Duct with Intraluminal Device, Open Approach */
"0FLD0ZZ"  =  "1"  /* Occlusion of Pancreatic Duct, Open Approach */
"0FLD3CZ"  =  "1"  /* Occlusion of Pancreatic Duct with Extraluminal Device, Percutaneous Approach */
"0FLD3DZ"  =  "1"  /* Occlusion of Pancreatic Duct with Intraluminal Device, Percutaneous Approach */
"0FLD3ZZ"  =  "1"  /* Occlusion of Pancreatic Duct, Percutaneous Approach */
"0FLD4CZ"  =  "1"  /* Occlusion of Pancreatic Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FLD4DZ"  =  "1"  /* Occlusion of Pancreatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FLD4ZZ"  =  "1"  /* Occlusion of Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FLD7DZ"  =  "1"  /* Occlusion of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0FLD7ZZ"  =  "1"  /* Occlusion of Pancreatic Duct, Via Natural or Artificial Opening */
"0FLD8DZ"  =  "1"  /* Occlusion of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FLD8ZZ"  =  "1"  /* Occlusion of Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FLF0CZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct with Extraluminal Device, Open Approach */
"0FLF0DZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct with Intraluminal Device, Open Approach */
"0FLF0ZZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct, Open Approach */
"0FLF3CZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct with Extraluminal Device, Percutaneous Approach */
"0FLF3DZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct with Intraluminal Device, Percutaneous Approach */
"0FLF3ZZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct, Percutaneous Approach */
"0FLF4CZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FLF4DZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FLF4ZZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FLF7DZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0FLF7ZZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FLF8DZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FLF8ZZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FM00ZZ"  =  "1"  /* Reattachment of Liver, Open Approach */
"0FM04ZZ"  =  "1"  /* Reattachment of Liver, Percutaneous Endoscopic Approach */
"0FM10ZZ"  =  "1"  /* Reattachment of Right Lobe Liver, Open Approach */
"0FM14ZZ"  =  "1"  /* Reattachment of Right Lobe Liver, Percutaneous Endoscopic Approach */
"0FM20ZZ"  =  "1"  /* Reattachment of Left Lobe Liver, Open Approach */
"0FM24ZZ"  =  "1"  /* Reattachment of Left Lobe Liver, Percutaneous Endoscopic Approach */
"0FM40ZZ"  =  "1"  /* Reattachment of Gallbladder, Open Approach */
"0FM50ZZ"  =  "1"  /* Reattachment of Right Hepatic Duct, Open Approach */
"0FM60ZZ"  =  "1"  /* Reattachment of Left Hepatic Duct, Open Approach */
"0FM80ZZ"  =  "1"  /* Reattachment of Cystic Duct, Open Approach */
"0FM90ZZ"  =  "1"  /* Reattachment of Common Bile Duct, Open Approach */
"0FMC0ZZ"  =  "1"  /* Reattachment of Ampulla of Vater, Open Approach */
"0FMC4ZZ"  =  "1"  /* Reattachment of Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FMD0ZZ"  =  "1"  /* Reattachment of Pancreatic Duct, Open Approach */
"0FMD4ZZ"  =  "1"  /* Reattachment of Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FMF0ZZ"  =  "1"  /* Reattachment of Accessory Pancreatic Duct, Open Approach */
"0FMF4ZZ"  =  "1"  /* Reattachment of Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FMG0ZZ"  =  "1"  /* Reattachment of Pancreas, Open Approach */
"0FMG4ZZ"  =  "1"  /* Reattachment of Pancreas, Percutaneous Endoscopic Approach */
"0FN00ZZ"  =  "1"  /* Release Liver, Open Approach */
"0FN03ZZ"  =  "1"  /* Release Liver, Percutaneous Approach */
"0FN04ZZ"  =  "1"  /* Release Liver, Percutaneous Endoscopic Approach */
"0FN10ZZ"  =  "1"  /* Release Right Lobe Liver, Open Approach */
"0FN13ZZ"  =  "1"  /* Release Right Lobe Liver, Percutaneous Approach */
"0FN14ZZ"  =  "1"  /* Release Right Lobe Liver, Percutaneous Endoscopic Approach */
"0FN20ZZ"  =  "1"  /* Release Left Lobe Liver, Open Approach */
"0FN23ZZ"  =  "1"  /* Release Left Lobe Liver, Percutaneous Approach */
"0FN24ZZ"  =  "1"  /* Release Left Lobe Liver, Percutaneous Endoscopic Approach */
"0FN40ZZ"  =  "1"  /* Release Gallbladder, Open Approach */
"0FN43ZZ"  =  "1"  /* Release Gallbladder, Percutaneous Approach */
"0FN44ZZ"  =  "1"  /* Release Gallbladder, Percutaneous Endoscopic Approach */
"0FN50ZZ"  =  "1"  /* Release Right Hepatic Duct, Open Approach */
"0FN53ZZ"  =  "1"  /* Release Right Hepatic Duct, Percutaneous Approach */
"0FN54ZZ"  =  "1"  /* Release Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0FN57ZZ"  =  "1"  /* Release Right Hepatic Duct, Via Natural or Artificial Opening */
"0FN58ZZ"  =  "1"  /* Release Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FN60ZZ"  =  "1"  /* Release Left Hepatic Duct, Open Approach */
"0FN63ZZ"  =  "1"  /* Release Left Hepatic Duct, Percutaneous Approach */
"0FN64ZZ"  =  "1"  /* Release Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0FN67ZZ"  =  "1"  /* Release Left Hepatic Duct, Via Natural or Artificial Opening */
"0FN68ZZ"  =  "1"  /* Release Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FN80ZZ"  =  "1"  /* Release Cystic Duct, Open Approach */
"0FN83ZZ"  =  "1"  /* Release Cystic Duct, Percutaneous Approach */
"0FN84ZZ"  =  "1"  /* Release Cystic Duct, Percutaneous Endoscopic Approach */
"0FN87ZZ"  =  "1"  /* Release Cystic Duct, Via Natural or Artificial Opening */
"0FN88ZZ"  =  "1"  /* Release Cystic Duct, Via Natural or Artificial Opening Endoscopic */
"0FN90ZZ"  =  "1"  /* Release Common Bile Duct, Open Approach */
"0FN93ZZ"  =  "1"  /* Release Common Bile Duct, Percutaneous Approach */
"0FN94ZZ"  =  "1"  /* Release Common Bile Duct, Percutaneous Endoscopic Approach */
"0FN97ZZ"  =  "1"  /* Release Common Bile Duct, Via Natural or Artificial Opening */
"0FN98ZZ"  =  "1"  /* Release Common Bile Duct, Via Natural or Artificial Opening Endoscopic */
"0FNC0ZZ"  =  "1"  /* Release Ampulla of Vater, Open Approach */
"0FNC3ZZ"  =  "1"  /* Release Ampulla of Vater, Percutaneous Approach */
"0FNC4ZZ"  =  "1"  /* Release Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FNC7ZZ"  =  "1"  /* Release Ampulla of Vater, Via Natural or Artificial Opening */
"0FNC8ZZ"  =  "1"  /* Release Ampulla of Vater, Via Natural or Artificial Opening Endoscopic */
"0FND0ZZ"  =  "1"  /* Release Pancreatic Duct, Open Approach */
"0FND3ZZ"  =  "1"  /* Release Pancreatic Duct, Percutaneous Approach */
"0FND4ZZ"  =  "1"  /* Release Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FND7ZZ"  =  "1"  /* Release Pancreatic Duct, Via Natural or Artificial Opening */
"0FND8ZZ"  =  "1"  /* Release Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FNF0ZZ"  =  "1"  /* Release Accessory Pancreatic Duct, Open Approach */
"0FNF3ZZ"  =  "1"  /* Release Accessory Pancreatic Duct, Percutaneous Approach */
"0FNF4ZZ"  =  "1"  /* Release Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FNF7ZZ"  =  "1"  /* Release Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FNF8ZZ"  =  "1"  /* Release Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FNG0ZZ"  =  "1"  /* Release Pancreas, Open Approach */
"0FNG3ZZ"  =  "1"  /* Release Pancreas, Percutaneous Approach */
"0FNG4ZZ"  =  "1"  /* Release Pancreas, Percutaneous Endoscopic Approach */
"0FP000Z"  =  "1"  /* Removal of Drainage Device from Liver, Open Approach */
"0FP002Z"  =  "1"  /* Removal of Monitoring Device from Liver, Open Approach */
"0FP003Z"  =  "1"  /* Removal of Infusion Device from Liver, Open Approach */
"0FP030Z"  =  "1"  /* Removal of Drainage Device from Liver, Percutaneous Approach */
"0FP032Z"  =  "1"  /* Removal of Monitoring Device from Liver, Percutaneous Approach */
"0FP033Z"  =  "1"  /* Removal of Infusion Device from Liver, Percutaneous Approach */
"0FP040Z"  =  "1"  /* Removal of Drainage Device from Liver, Percutaneous Endoscopic Approach */
"0FP042Z"  =  "1"  /* Removal of Monitoring Device from Liver, Percutaneous Endoscopic Approach */
"0FP043Z"  =  "1"  /* Removal of Infusion Device from Liver, Percutaneous Endoscopic Approach */
"0FP400Z"  =  "1"  /* Removal of Drainage Device from Gallbladder, Open Approach */
"0FP402Z"  =  "1"  /* Removal of Monitoring Device from Gallbladder, Open Approach */
"0FP403Z"  =  "1"  /* Removal of Infusion Device from Gallbladder, Open Approach */
"0FP40DZ"  =  "1"  /* Removal of Intraluminal Device from Gallbladder, Open Approach */
"0FP430Z"  =  "1"  /* Removal of Drainage Device from Gallbladder, Percutaneous Approach */
"0FP432Z"  =  "1"  /* Removal of Monitoring Device from Gallbladder, Percutaneous Approach */
"0FP433Z"  =  "1"  /* Removal of Infusion Device from Gallbladder, Percutaneous Approach */
"0FP43DZ"  =  "1"  /* Removal of Intraluminal Device from Gallbladder, Percutaneous Approach */
"0FP440Z"  =  "1"  /* Removal of Drainage Device from Gallbladder, Percutaneous Endoscopic Approach */
"0FP442Z"  =  "1"  /* Removal of Monitoring Device from Gallbladder, Percutaneous Endoscopic Approach */
"0FP443Z"  =  "1"  /* Removal of Infusion Device from Gallbladder, Percutaneous Endoscopic Approach */
"0FP44DZ"  =  "1"  /* Removal of Intraluminal Device from Gallbladder, Percutaneous Endoscopic Approach */
"0FPB00Z"  =  "1"  /* Removal of Drainage Device from Hepatobiliary Duct, Open Approach */
"0FPB01Z"  =  "1"  /* Removal of Radioactive Element from Hepatobiliary Duct, Open Approach */
"0FPB02Z"  =  "1"  /* Removal of Monitoring Device from Hepatobiliary Duct, Open Approach */
"0FPB03Z"  =  "1"  /* Removal of Infusion Device from Hepatobiliary Duct, Open Approach */
"0FPB07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Open Approach */
"0FPB0CZ"  =  "1"  /* Removal of Extraluminal Device from Hepatobiliary Duct, Open Approach */
"0FPB0DZ"  =  "1"  /* Removal of Intraluminal Device from Hepatobiliary Duct, Open Approach */
"0FPB0JZ"  =  "1"  /* Removal of Synthetic Substitute from Hepatobiliary Duct, Open Approach */
"0FPB0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Open Approach */
"0FPB30Z"  =  "1"  /* Removal of Drainage Device from Hepatobiliary Duct, Percutaneous Approach */
"0FPB31Z"  =  "1"  /* Removal of Radioactive Element from Hepatobiliary Duct, Percutaneous Approach */
"0FPB32Z"  =  "1"  /* Removal of Monitoring Device from Hepatobiliary Duct, Percutaneous Approach */
"0FPB33Z"  =  "1"  /* Removal of Infusion Device from Hepatobiliary Duct, Percutaneous Approach */
"0FPB37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Percutaneous Approach */
"0FPB3CZ"  =  "1"  /* Removal of Extraluminal Device from Hepatobiliary Duct, Percutaneous Approach */
"0FPB3DZ"  =  "1"  /* Removal of Intraluminal Device from Hepatobiliary Duct, Percutaneous Approach */
"0FPB3JZ"  =  "1"  /* Removal of Synthetic Substitute from Hepatobiliary Duct, Percutaneous Approach */
"0FPB3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Percutaneous Approach */
"0FPB40Z"  =  "1"  /* Removal of Drainage Device from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB41Z"  =  "1"  /* Removal of Radioactive Element from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB42Z"  =  "1"  /* Removal of Monitoring Device from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB43Z"  =  "1"  /* Removal of Infusion Device from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB4CZ"  =  "1"  /* Removal of Extraluminal Device from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB4DZ"  =  "1"  /* Removal of Intraluminal Device from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB4JZ"  =  "1"  /* Removal of Synthetic Substitute from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB70Z"  =  "1"  /* Removal of Drainage Device from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB71Z"  =  "1"  /* Removal of Radioactive Element from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB72Z"  =  "1"  /* Removal of Monitoring Device from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB73Z"  =  "1"  /* Removal of Infusion Device from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB7CZ"  =  "1"  /* Removal of Extraluminal Device from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB7DZ"  =  "1"  /* Removal of Intraluminal Device from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB7JZ"  =  "1"  /* Removal of Synthetic Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB80Z"  =  "1"  /* Removal of Drainage Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB81Z"  =  "1"  /* Removal of Radioactive Element from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB82Z"  =  "1"  /* Removal of Monitoring Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB83Z"  =  "1"  /* Removal of Infusion Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB8CZ"  =  "1"  /* Removal of Extraluminal Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB8DZ"  =  "1"  /* Removal of Intraluminal Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB8JZ"  =  "1"  /* Removal of Synthetic Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD00Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Open Approach */
"0FPD01Z"  =  "1"  /* Removal of Radioactive Element from Pancreatic Duct, Open Approach */
"0FPD02Z"  =  "1"  /* Removal of Monitoring Device from Pancreatic Duct, Open Approach */
"0FPD03Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Open Approach */
"0FPD07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Pancreatic Duct, Open Approach */
"0FPD0CZ"  =  "1"  /* Removal of Extraluminal Device from Pancreatic Duct, Open Approach */
"0FPD0DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreatic Duct, Open Approach */
"0FPD0JZ"  =  "1"  /* Removal of Synthetic Substitute from Pancreatic Duct, Open Approach */
"0FPD0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Open Approach */
"0FPD30Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Percutaneous Approach */
"0FPD31Z"  =  "1"  /* Removal of Radioactive Element from Pancreatic Duct, Percutaneous Approach */
"0FPD32Z"  =  "1"  /* Removal of Monitoring Device from Pancreatic Duct, Percutaneous Approach */
"0FPD33Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Percutaneous Approach */
"0FPD37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Pancreatic Duct, Percutaneous Approach */
"0FPD3CZ"  =  "1"  /* Removal of Extraluminal Device from Pancreatic Duct, Percutaneous Approach */
"0FPD3DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreatic Duct, Percutaneous Approach */
"0FPD3JZ"  =  "1"  /* Removal of Synthetic Substitute from Pancreatic Duct, Percutaneous Approach */
"0FPD3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Percutaneous Approach */
"0FPD40Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD41Z"  =  "1"  /* Removal of Radioactive Element from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD42Z"  =  "1"  /* Removal of Monitoring Device from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD43Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD4CZ"  =  "1"  /* Removal of Extraluminal Device from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD4DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD4JZ"  =  "1"  /* Removal of Synthetic Substitute from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD70Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD71Z"  =  "1"  /* Removal of Radioactive Element from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD72Z"  =  "1"  /* Removal of Monitoring Device from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD73Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD7CZ"  =  "1"  /* Removal of Extraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD7DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD7JZ"  =  "1"  /* Removal of Synthetic Substitute from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD80Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD81Z"  =  "1"  /* Removal of Radioactive Element from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD82Z"  =  "1"  /* Removal of Monitoring Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD83Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD8CZ"  =  "1"  /* Removal of Extraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD8DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD8JZ"  =  "1"  /* Removal of Synthetic Substitute from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPG00Z"  =  "1"  /* Removal of Drainage Device from Pancreas, Open Approach */
"0FPG02Z"  =  "1"  /* Removal of Monitoring Device from Pancreas, Open Approach */
"0FPG03Z"  =  "1"  /* Removal of Infusion Device from Pancreas, Open Approach */
"0FPG0DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreas, Open Approach */
"0FPG30Z"  =  "1"  /* Removal of Drainage Device from Pancreas, Percutaneous Approach */
"0FPG32Z"  =  "1"  /* Removal of Monitoring Device from Pancreas, Percutaneous Approach */
"0FPG33Z"  =  "1"  /* Removal of Infusion Device from Pancreas, Percutaneous Approach */
"0FPG3DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreas, Percutaneous Approach */
"0FPG40Z"  =  "1"  /* Removal of Drainage Device from Pancreas, Percutaneous Endoscopic Approach */
"0FPG42Z"  =  "1"  /* Removal of Monitoring Device from Pancreas, Percutaneous Endoscopic Approach */
"0FPG43Z"  =  "1"  /* Removal of Infusion Device from Pancreas, Percutaneous Endoscopic Approach */
"0FPG4DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreas, Percutaneous Endoscopic Approach */
"0FPGXDZ"  =  "1"  /* Removal of Intraluminal Device from Pancreas, External Approach */
"0FQ00ZZ"  =  "1"  /* Repair Liver, Open Approach */
"0FQ03ZZ"  =  "1"  /* Repair Liver, Percutaneous Approach */
"0FQ04ZZ"  =  "1"  /* Repair Liver, Percutaneous Endoscopic Approach */
"0FQ10ZZ"  =  "1"  /* Repair Right Lobe Liver, Open Approach */
"0FQ13ZZ"  =  "1"  /* Repair Right Lobe Liver, Percutaneous Approach */
"0FQ14ZZ"  =  "1"  /* Repair Right Lobe Liver, Percutaneous Endoscopic Approach */
"0FQ20ZZ"  =  "1"  /* Repair Left Lobe Liver, Open Approach */
"0FQ23ZZ"  =  "1"  /* Repair Left Lobe Liver, Percutaneous Approach */
"0FQ24ZZ"  =  "1"  /* Repair Left Lobe Liver, Percutaneous Endoscopic Approach */
"0FQ40ZZ"  =  "1"  /* Repair Gallbladder, Open Approach */
"0FQ43ZZ"  =  "1"  /* Repair Gallbladder, Percutaneous Approach */
"0FQ44ZZ"  =  "1"  /* Repair Gallbladder, Percutaneous Endoscopic Approach */
"0FQ50ZZ"  =  "1"  /* Repair Right Hepatic Duct, Open Approach */
"0FQ53ZZ"  =  "1"  /* Repair Right Hepatic Duct, Percutaneous Approach */
"0FQ54ZZ"  =  "1"  /* Repair Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0FQ57ZZ"  =  "1"  /* Repair Right Hepatic Duct, Via Natural or Artificial Opening */
"0FQ58ZZ"  =  "1"  /* Repair Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FQ60ZZ"  =  "1"  /* Repair Left Hepatic Duct, Open Approach */
"0FQ63ZZ"  =  "1"  /* Repair Left Hepatic Duct, Percutaneous Approach */
"0FQ64ZZ"  =  "1"  /* Repair Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0FQ67ZZ"  =  "1"  /* Repair Left Hepatic Duct, Via Natural or Artificial Opening */
"0FQ68ZZ"  =  "1"  /* Repair Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FQ80ZZ"  =  "1"  /* Repair Cystic Duct, Open Approach */
"0FQ83ZZ"  =  "1"  /* Repair Cystic Duct, Percutaneous Approach */
"0FQ84ZZ"  =  "1"  /* Repair Cystic Duct, Percutaneous Endoscopic Approach */
"0FQ87ZZ"  =  "1"  /* Repair Cystic Duct, Via Natural or Artificial Opening */
"0FQ88ZZ"  =  "1"  /* Repair Cystic Duct, Via Natural or Artificial Opening Endoscopic */
"0FQ90ZZ"  =  "1"  /* Repair Common Bile Duct, Open Approach */
"0FQ93ZZ"  =  "1"  /* Repair Common Bile Duct, Percutaneous Approach */
"0FQ94ZZ"  =  "1"  /* Repair Common Bile Duct, Percutaneous Endoscopic Approach */
"0FQ97ZZ"  =  "1"  /* Repair Common Bile Duct, Via Natural or Artificial Opening */
"0FQ98ZZ"  =  "1"  /* Repair Common Bile Duct, Via Natural or Artificial Opening Endoscopic */
"0FQC0ZZ"  =  "1"  /* Repair Ampulla of Vater, Open Approach */
"0FQC3ZZ"  =  "1"  /* Repair Ampulla of Vater, Percutaneous Approach */
"0FQC4ZZ"  =  "1"  /* Repair Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FQC7ZZ"  =  "1"  /* Repair Ampulla of Vater, Via Natural or Artificial Opening */
"0FQC8ZZ"  =  "1"  /* Repair Ampulla of Vater, Via Natural or Artificial Opening Endoscopic */
"0FQD0ZZ"  =  "1"  /* Repair Pancreatic Duct, Open Approach */
"0FQD3ZZ"  =  "1"  /* Repair Pancreatic Duct, Percutaneous Approach */
"0FQD4ZZ"  =  "1"  /* Repair Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FQD7ZZ"  =  "1"  /* Repair Pancreatic Duct, Via Natural or Artificial Opening */
"0FQD8ZZ"  =  "1"  /* Repair Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FQF0ZZ"  =  "1"  /* Repair Accessory Pancreatic Duct, Open Approach */
"0FQF3ZZ"  =  "1"  /* Repair Accessory Pancreatic Duct, Percutaneous Approach */
"0FQF4ZZ"  =  "1"  /* Repair Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FQF7ZZ"  =  "1"  /* Repair Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FQF8ZZ"  =  "1"  /* Repair Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FQG0ZZ"  =  "1"  /* Repair Pancreas, Open Approach */
"0FQG3ZZ"  =  "1"  /* Repair Pancreas, Percutaneous Approach */
"0FQG4ZZ"  =  "1"  /* Repair Pancreas, Percutaneous Endoscopic Approach */
"0FR507Z"  =  "1"  /* Replacement of Right Hepatic Duct with Autologous Tissue Substitute, Open Approach */
"0FR50JZ"  =  "1"  /* Replacement of Right Hepatic Duct with Synthetic Substitute, Open Approach */
"0FR50KZ"  =  "1"  /* Replacement of Right Hepatic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FR547Z"  =  "1"  /* Replacement of Right Hepatic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FR54JZ"  =  "1"  /* Replacement of Right Hepatic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FR54KZ"  =  "1"  /* Replacement of Right Hepatic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FR607Z"  =  "1"  /* Replacement of Left Hepatic Duct with Autologous Tissue Substitute, Open Approach */
"0FR60JZ"  =  "1"  /* Replacement of Left Hepatic Duct with Synthetic Substitute, Open Approach */
"0FR60KZ"  =  "1"  /* Replacement of Left Hepatic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FR647Z"  =  "1"  /* Replacement of Left Hepatic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FR64JZ"  =  "1"  /* Replacement of Left Hepatic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FR64KZ"  =  "1"  /* Replacement of Left Hepatic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FR807Z"  =  "1"  /* Replacement of Cystic Duct with Autologous Tissue Substitute, Open Approach */
"0FR80JZ"  =  "1"  /* Replacement of Cystic Duct with Synthetic Substitute, Open Approach */
"0FR80KZ"  =  "1"  /* Replacement of Cystic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FR847Z"  =  "1"  /* Replacement of Cystic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FR84JZ"  =  "1"  /* Replacement of Cystic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FR84KZ"  =  "1"  /* Replacement of Cystic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FR907Z"  =  "1"  /* Replacement of Common Bile Duct with Autologous Tissue Substitute, Open Approach */
"0FR90JZ"  =  "1"  /* Replacement of Common Bile Duct with Synthetic Substitute, Open Approach */
"0FR90KZ"  =  "1"  /* Replacement of Common Bile Duct with Nonautologous Tissue Substitute, Open Approach */
"0FR947Z"  =  "1"  /* Replacement of Common Bile Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FR94JZ"  =  "1"  /* Replacement of Common Bile Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FR94KZ"  =  "1"  /* Replacement of Common Bile Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FRC07Z"  =  "1"  /* Replacement of Ampulla of Vater with Autologous Tissue Substitute, Open Approach */
"0FRC0JZ"  =  "1"  /* Replacement of Ampulla of Vater with Synthetic Substitute, Open Approach */
"0FRC0KZ"  =  "1"  /* Replacement of Ampulla of Vater with Nonautologous Tissue Substitute, Open Approach */
"0FRC47Z"  =  "1"  /* Replacement of Ampulla of Vater with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FRC4JZ"  =  "1"  /* Replacement of Ampulla of Vater with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FRC4KZ"  =  "1"  /* Replacement of Ampulla of Vater with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FRD07Z"  =  "1"  /* Replacement of Pancreatic Duct with Autologous Tissue Substitute, Open Approach */
"0FRD0JZ"  =  "1"  /* Replacement of Pancreatic Duct with Synthetic Substitute, Open Approach */
"0FRD0KZ"  =  "1"  /* Replacement of Pancreatic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FRD47Z"  =  "1"  /* Replacement of Pancreatic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FRD4JZ"  =  "1"  /* Replacement of Pancreatic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FRD4KZ"  =  "1"  /* Replacement of Pancreatic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FRF07Z"  =  "1"  /* Replacement of Accessory Pancreatic Duct with Autologous Tissue Substitute, Open Approach */
"0FRF0JZ"  =  "1"  /* Replacement of Accessory Pancreatic Duct with Synthetic Substitute, Open Approach */
"0FRF0KZ"  =  "1"  /* Replacement of Accessory Pancreatic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FRF47Z"  =  "1"  /* Replacement of Accessory Pancreatic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FRF4JZ"  =  "1"  /* Replacement of Accessory Pancreatic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FRF4KZ"  =  "1"  /* Replacement of Accessory Pancreatic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FS00ZZ"  =  "1"  /* Reposition Liver, Open Approach */
"0FS04ZZ"  =  "1"  /* Reposition Liver, Percutaneous Endoscopic Approach */
"0FS40ZZ"  =  "1"  /* Reposition Gallbladder, Open Approach */
"0FS44ZZ"  =  "1"  /* Reposition Gallbladder, Percutaneous Endoscopic Approach */
"0FS50ZZ"  =  "1"  /* Reposition Right Hepatic Duct, Open Approach */
"0FS54ZZ"  =  "1"  /* Reposition Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0FS60ZZ"  =  "1"  /* Reposition Left Hepatic Duct, Open Approach */
"0FS64ZZ"  =  "1"  /* Reposition Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0FS80ZZ"  =  "1"  /* Reposition Cystic Duct, Open Approach */
"0FS84ZZ"  =  "1"  /* Reposition Cystic Duct, Percutaneous Endoscopic Approach */
"0FS90ZZ"  =  "1"  /* Reposition Common Bile Duct, Open Approach */
"0FS94ZZ"  =  "1"  /* Reposition Common Bile Duct, Percutaneous Endoscopic Approach */
"0FSC0ZZ"  =  "1"  /* Reposition Ampulla of Vater, Open Approach */
"0FSC4ZZ"  =  "1"  /* Reposition Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FSD0ZZ"  =  "1"  /* Reposition Pancreatic Duct, Open Approach */
"0FSD4ZZ"  =  "1"  /* Reposition Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FSF0ZZ"  =  "1"  /* Reposition Accessory Pancreatic Duct, Open Approach */
"0FSF4ZZ"  =  "1"  /* Reposition Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FSG0ZZ"  =  "1"  /* Reposition Pancreas, Open Approach */
"0FSG4ZZ"  =  "1"  /* Reposition Pancreas, Percutaneous Endoscopic Approach */
"0FT00ZZ"  =  "1"  /* Resection of Liver, Open Approach */
"0FT04ZZ"  =  "1"  /* Resection of Liver, Percutaneous Endoscopic Approach */
"0FT10ZZ"  =  "1"  /* Resection of Right Lobe Liver, Open Approach */
"0FT14ZZ"  =  "1"  /* Resection of Right Lobe Liver, Percutaneous Endoscopic Approach */
"0FT20ZZ"  =  "1"  /* Resection of Left Lobe Liver, Open Approach */
"0FT24ZZ"  =  "1"  /* Resection of Left Lobe Liver, Percutaneous Endoscopic Approach */
"0FT40ZZ"  =  "1"  /* Resection of Gallbladder, Open Approach */
"0FT44ZZ"  =  "1"  /* Resection of Gallbladder, Percutaneous Endoscopic Approach */
"0FT50ZZ"  =  "1"  /* Resection of Right Hepatic Duct, Open Approach */
"0FT54ZZ"  =  "1"  /* Resection of Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0FT57ZZ"  =  "1"  /* Resection of Right Hepatic Duct, Via Natural or Artificial Opening */
"0FT58ZZ"  =  "1"  /* Resection of Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FT60ZZ"  =  "1"  /* Resection of Left Hepatic Duct, Open Approach */
"0FT64ZZ"  =  "1"  /* Resection of Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0FT67ZZ"  =  "1"  /* Resection of Left Hepatic Duct, Via Natural or Artificial Opening */
"0FT68ZZ"  =  "1"  /* Resection of Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FT80ZZ"  =  "1"  /* Resection of Cystic Duct, Open Approach */
"0FT84ZZ"  =  "1"  /* Resection of Cystic Duct, Percutaneous Endoscopic Approach */
"0FT87ZZ"  =  "1"  /* Resection of Cystic Duct, Via Natural or Artificial Opening */
"0FT88ZZ"  =  "1"  /* Resection of Cystic Duct, Via Natural or Artificial Opening Endoscopic */
"0FT90ZZ"  =  "1"  /* Resection of Common Bile Duct, Open Approach */
"0FT94ZZ"  =  "1"  /* Resection of Common Bile Duct, Percutaneous Endoscopic Approach */
"0FT97ZZ"  =  "1"  /* Resection of Common Bile Duct, Via Natural or Artificial Opening */
"0FT98ZZ"  =  "1"  /* Resection of Common Bile Duct, Via Natural or Artificial Opening Endoscopic */
"0FTC0ZZ"  =  "1"  /* Resection of Ampulla of Vater, Open Approach */
"0FTC4ZZ"  =  "1"  /* Resection of Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FTC7ZZ"  =  "1"  /* Resection of Ampulla of Vater, Via Natural or Artificial Opening */
"0FTC8ZZ"  =  "1"  /* Resection of Ampulla of Vater, Via Natural or Artificial Opening Endoscopic */
"0FTD0ZZ"  =  "1"  /* Resection of Pancreatic Duct, Open Approach */
"0FTD7ZZ"  =  "1"  /* Resection of Pancreatic Duct, Via Natural or Artificial Opening */
"0FTF0ZZ"  =  "1"  /* Resection of Accessory Pancreatic Duct, Open Approach */
"0FTF7ZZ"  =  "1"  /* Resection of Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FTG0ZZ"  =  "1"  /* Resection of Pancreas, Open Approach */
"0FTG4ZZ"  =  "1"  /* Resection of Pancreas, Percutaneous Endoscopic Approach */
"0FU507Z"  =  "1"  /* Supplement Right Hepatic Duct with Autologous Tissue Substitute, Open Approach */
"0FU50JZ"  =  "1"  /* Supplement Right Hepatic Duct with Synthetic Substitute, Open Approach */
"0FU50KZ"  =  "1"  /* Supplement Right Hepatic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FU537Z"  =  "1"  /* Supplement Right Hepatic Duct with Autologous Tissue Substitute, Percutaneous Approach */
"0FU53JZ"  =  "1"  /* Supplement Right Hepatic Duct with Synthetic Substitute, Percutaneous Approach */
"0FU53KZ"  =  "1"  /* Supplement Right Hepatic Duct with Nonautologous Tissue Substitute, Percutaneous Approach */
"0FU547Z"  =  "1"  /* Supplement Right Hepatic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FU54JZ"  =  "1"  /* Supplement Right Hepatic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FU54KZ"  =  "1"  /* Supplement Right Hepatic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FU607Z"  =  "1"  /* Supplement Left Hepatic Duct with Autologous Tissue Substitute, Open Approach */
"0FU60JZ"  =  "1"  /* Supplement Left Hepatic Duct with Synthetic Substitute, Open Approach */
"0FU60KZ"  =  "1"  /* Supplement Left Hepatic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FU637Z"  =  "1"  /* Supplement Left Hepatic Duct with Autologous Tissue Substitute, Percutaneous Approach */
"0FU63JZ"  =  "1"  /* Supplement Left Hepatic Duct with Synthetic Substitute, Percutaneous Approach */
"0FU63KZ"  =  "1"  /* Supplement Left Hepatic Duct with Nonautologous Tissue Substitute, Percutaneous Approach */
"0FU647Z"  =  "1"  /* Supplement Left Hepatic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FU64JZ"  =  "1"  /* Supplement Left Hepatic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FU64KZ"  =  "1"  /* Supplement Left Hepatic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FU807Z"  =  "1"  /* Supplement Cystic Duct with Autologous Tissue Substitute, Open Approach */
"0FU80JZ"  =  "1"  /* Supplement Cystic Duct with Synthetic Substitute, Open Approach */
"0FU80KZ"  =  "1"  /* Supplement Cystic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FU837Z"  =  "1"  /* Supplement Cystic Duct with Autologous Tissue Substitute, Percutaneous Approach */
"0FU83JZ"  =  "1"  /* Supplement Cystic Duct with Synthetic Substitute, Percutaneous Approach */
"0FU83KZ"  =  "1"  /* Supplement Cystic Duct with Nonautologous Tissue Substitute, Percutaneous Approach */
"0FU847Z"  =  "1"  /* Supplement Cystic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FU84JZ"  =  "1"  /* Supplement Cystic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FU84KZ"  =  "1"  /* Supplement Cystic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FU907Z"  =  "1"  /* Supplement Common Bile Duct with Autologous Tissue Substitute, Open Approach */
"0FU90JZ"  =  "1"  /* Supplement Common Bile Duct with Synthetic Substitute, Open Approach */
"0FU90KZ"  =  "1"  /* Supplement Common Bile Duct with Nonautologous Tissue Substitute, Open Approach */
"0FU937Z"  =  "1"  /* Supplement Common Bile Duct with Autologous Tissue Substitute, Percutaneous Approach */
"0FU93JZ"  =  "1"  /* Supplement Common Bile Duct with Synthetic Substitute, Percutaneous Approach */
"0FU93KZ"  =  "1"  /* Supplement Common Bile Duct with Nonautologous Tissue Substitute, Percutaneous Approach */
"0FU947Z"  =  "1"  /* Supplement Common Bile Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FU94JZ"  =  "1"  /* Supplement Common Bile Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FU94KZ"  =  "1"  /* Supplement Common Bile Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FUC07Z"  =  "1"  /* Supplement Ampulla of Vater with Autologous Tissue Substitute, Open Approach */
"0FUC0JZ"  =  "1"  /* Supplement Ampulla of Vater with Synthetic Substitute, Open Approach */
"0FUC0KZ"  =  "1"  /* Supplement Ampulla of Vater with Nonautologous Tissue Substitute, Open Approach */
"0FUC37Z"  =  "1"  /* Supplement Ampulla of Vater with Autologous Tissue Substitute, Percutaneous Approach */
"0FUC3JZ"  =  "1"  /* Supplement Ampulla of Vater with Synthetic Substitute, Percutaneous Approach */
"0FUC3KZ"  =  "1"  /* Supplement Ampulla of Vater with Nonautologous Tissue Substitute, Percutaneous Approach */
"0FUC47Z"  =  "1"  /* Supplement Ampulla of Vater with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FUC4JZ"  =  "1"  /* Supplement Ampulla of Vater with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FUC4KZ"  =  "1"  /* Supplement Ampulla of Vater with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FUD07Z"  =  "1"  /* Supplement Pancreatic Duct with Autologous Tissue Substitute, Open Approach */
"0FUD0JZ"  =  "1"  /* Supplement Pancreatic Duct with Synthetic Substitute, Open Approach */
"0FUD0KZ"  =  "1"  /* Supplement Pancreatic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FUD37Z"  =  "1"  /* Supplement Pancreatic Duct with Autologous Tissue Substitute, Percutaneous Approach */
"0FUD3JZ"  =  "1"  /* Supplement Pancreatic Duct with Synthetic Substitute, Percutaneous Approach */
"0FUD3KZ"  =  "1"  /* Supplement Pancreatic Duct with Nonautologous Tissue Substitute, Percutaneous Approach */
"0FUD47Z"  =  "1"  /* Supplement Pancreatic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FUD4JZ"  =  "1"  /* Supplement Pancreatic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FUD4KZ"  =  "1"  /* Supplement Pancreatic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FUF07Z"  =  "1"  /* Supplement Accessory Pancreatic Duct with Autologous Tissue Substitute, Open Approach */
"0FUF0JZ"  =  "1"  /* Supplement Accessory Pancreatic Duct with Synthetic Substitute, Open Approach */
"0FUF0KZ"  =  "1"  /* Supplement Accessory Pancreatic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FUF37Z"  =  "1"  /* Supplement Accessory Pancreatic Duct with Autologous Tissue Substitute, Percutaneous Approach */
"0FUF3JZ"  =  "1"  /* Supplement Accessory Pancreatic Duct with Synthetic Substitute, Percutaneous Approach */
"0FUF3KZ"  =  "1"  /* Supplement Accessory Pancreatic Duct with Nonautologous Tissue Substitute, Percutaneous Approach */
"0FUF47Z"  =  "1"  /* Supplement Accessory Pancreatic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FUF4JZ"  =  "1"  /* Supplement Accessory Pancreatic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FUF4KZ"  =  "1"  /* Supplement Accessory Pancreatic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FV50CZ"  =  "1"  /* Restriction of Right Hepatic Duct with Extraluminal Device, Open Approach */
"0FV50DZ"  =  "1"  /* Restriction of Right Hepatic Duct with Intraluminal Device, Open Approach */
"0FV50ZZ"  =  "1"  /* Restriction of Right Hepatic Duct, Open Approach */
"0FV60CZ"  =  "1"  /* Restriction of Left Hepatic Duct with Extraluminal Device, Open Approach */
"0FV60DZ"  =  "1"  /* Restriction of Left Hepatic Duct with Intraluminal Device, Open Approach */
"0FV60ZZ"  =  "1"  /* Restriction of Left Hepatic Duct, Open Approach */
"0FV80CZ"  =  "1"  /* Restriction of Cystic Duct with Extraluminal Device, Open Approach */
"0FV80DZ"  =  "1"  /* Restriction of Cystic Duct with Intraluminal Device, Open Approach */
"0FV80ZZ"  =  "1"  /* Restriction of Cystic Duct, Open Approach */
"0FV90CZ"  =  "1"  /* Restriction of Common Bile Duct with Extraluminal Device, Open Approach */
"0FV90DZ"  =  "1"  /* Restriction of Common Bile Duct with Intraluminal Device, Open Approach */
"0FV90ZZ"  =  "1"  /* Restriction of Common Bile Duct, Open Approach */
"0FVC0CZ"  =  "1"  /* Restriction of Ampulla of Vater with Extraluminal Device, Open Approach */
"0FVC0DZ"  =  "1"  /* Restriction of Ampulla of Vater with Intraluminal Device, Open Approach */
"0FVC0ZZ"  =  "1"  /* Restriction of Ampulla of Vater, Open Approach */
"0FVC3CZ"  =  "1"  /* Restriction of Ampulla of Vater with Extraluminal Device, Percutaneous Approach */
"0FVC3DZ"  =  "1"  /* Restriction of Ampulla of Vater with Intraluminal Device, Percutaneous Approach */
"0FVC3ZZ"  =  "1"  /* Restriction of Ampulla of Vater, Percutaneous Approach */
"0FVC4CZ"  =  "1"  /* Restriction of Ampulla of Vater with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FVC4DZ"  =  "1"  /* Restriction of Ampulla of Vater with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FVC4ZZ"  =  "1"  /* Restriction of Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FVC7DZ"  =  "1"  /* Restriction of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening */
"0FVC7ZZ"  =  "1"  /* Restriction of Ampulla of Vater, Via Natural or Artificial Opening */
"0FVC8DZ"  =  "1"  /* Restriction of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FVC8ZZ"  =  "1"  /* Restriction of Ampulla of Vater, Via Natural or Artificial Opening Endoscopic */
"0FVD0CZ"  =  "1"  /* Restriction of Pancreatic Duct with Extraluminal Device, Open Approach */
"0FVD0DZ"  =  "1"  /* Restriction of Pancreatic Duct with Intraluminal Device, Open Approach */
"0FVD0ZZ"  =  "1"  /* Restriction of Pancreatic Duct, Open Approach */
"0FVD3CZ"  =  "1"  /* Restriction of Pancreatic Duct with Extraluminal Device, Percutaneous Approach */
"0FVD3DZ"  =  "1"  /* Restriction of Pancreatic Duct with Intraluminal Device, Percutaneous Approach */
"0FVD3ZZ"  =  "1"  /* Restriction of Pancreatic Duct, Percutaneous Approach */
"0FVD4CZ"  =  "1"  /* Restriction of Pancreatic Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FVD4DZ"  =  "1"  /* Restriction of Pancreatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FVD4ZZ"  =  "1"  /* Restriction of Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FVD7DZ"  =  "1"  /* Restriction of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0FVD7ZZ"  =  "1"  /* Restriction of Pancreatic Duct, Via Natural or Artificial Opening */
"0FVD8DZ"  =  "1"  /* Restriction of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FVD8ZZ"  =  "1"  /* Restriction of Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FVF0CZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct with Extraluminal Device, Open Approach */
"0FVF0DZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct with Intraluminal Device, Open Approach */
"0FVF0ZZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct, Open Approach */
"0FVF3CZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct with Extraluminal Device, Percutaneous Approach */
"0FVF3DZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct with Intraluminal Device, Percutaneous Approach */
"0FVF3ZZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct, Percutaneous Approach */
"0FVF4CZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FVF4DZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FVF4ZZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FVF7DZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0FVF7ZZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FVF8DZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FVF8ZZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FW000Z"  =  "1"  /* Revision of Drainage Device in Liver, Open Approach */
"0FW002Z"  =  "1"  /* Revision of Monitoring Device in Liver, Open Approach */
"0FW003Z"  =  "1"  /* Revision of Infusion Device in Liver, Open Approach */
"0FW030Z"  =  "1"  /* Revision of Drainage Device in Liver, Percutaneous Approach */
"0FW032Z"  =  "1"  /* Revision of Monitoring Device in Liver, Percutaneous Approach */
"0FW033Z"  =  "1"  /* Revision of Infusion Device in Liver, Percutaneous Approach */
"0FW040Z"  =  "1"  /* Revision of Drainage Device in Liver, Percutaneous Endoscopic Approach */
"0FW042Z"  =  "1"  /* Revision of Monitoring Device in Liver, Percutaneous Endoscopic Approach */
"0FW043Z"  =  "1"  /* Revision of Infusion Device in Liver, Percutaneous Endoscopic Approach */
"0FW400Z"  =  "1"  /* Revision of Drainage Device in Gallbladder, Open Approach */
"0FW402Z"  =  "1"  /* Revision of Monitoring Device in Gallbladder, Open Approach */
"0FW403Z"  =  "1"  /* Revision of Infusion Device in Gallbladder, Open Approach */
"0FW40DZ"  =  "1"  /* Revision of Intraluminal Device in Gallbladder, Open Approach */
"0FW430Z"  =  "1"  /* Revision of Drainage Device in Gallbladder, Percutaneous Approach */
"0FW432Z"  =  "1"  /* Revision of Monitoring Device in Gallbladder, Percutaneous Approach */
"0FW433Z"  =  "1"  /* Revision of Infusion Device in Gallbladder, Percutaneous Approach */
"0FW43DZ"  =  "1"  /* Revision of Intraluminal Device in Gallbladder, Percutaneous Approach */
"0FW440Z"  =  "1"  /* Revision of Drainage Device in Gallbladder, Percutaneous Endoscopic Approach */
"0FW442Z"  =  "1"  /* Revision of Monitoring Device in Gallbladder, Percutaneous Endoscopic Approach */
"0FW443Z"  =  "1"  /* Revision of Infusion Device in Gallbladder, Percutaneous Endoscopic Approach */
"0FW44DZ"  =  "1"  /* Revision of Intraluminal Device in Gallbladder, Percutaneous Endoscopic Approach */
"0FWB00Z"  =  "1"  /* Revision of Drainage Device in Hepatobiliary Duct, Open Approach */
"0FWB02Z"  =  "1"  /* Revision of Monitoring Device in Hepatobiliary Duct, Open Approach */
"0FWB03Z"  =  "1"  /* Revision of Infusion Device in Hepatobiliary Duct, Open Approach */
"0FWB07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Open Approach */
"0FWB0CZ"  =  "1"  /* Revision of Extraluminal Device in Hepatobiliary Duct, Open Approach */
"0FWB0DZ"  =  "1"  /* Revision of Intraluminal Device in Hepatobiliary Duct, Open Approach */
"0FWB0JZ"  =  "1"  /* Revision of Synthetic Substitute in Hepatobiliary Duct, Open Approach */
"0FWB0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Open Approach */
"0FWB30Z"  =  "1"  /* Revision of Drainage Device in Hepatobiliary Duct, Percutaneous Approach */
"0FWB32Z"  =  "1"  /* Revision of Monitoring Device in Hepatobiliary Duct, Percutaneous Approach */
"0FWB33Z"  =  "1"  /* Revision of Infusion Device in Hepatobiliary Duct, Percutaneous Approach */
"0FWB37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Percutaneous Approach */
"0FWB3CZ"  =  "1"  /* Revision of Extraluminal Device in Hepatobiliary Duct, Percutaneous Approach */
"0FWB3DZ"  =  "1"  /* Revision of Intraluminal Device in Hepatobiliary Duct, Percutaneous Approach */
"0FWB3JZ"  =  "1"  /* Revision of Synthetic Substitute in Hepatobiliary Duct, Percutaneous Approach */
"0FWB3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Percutaneous Approach */
"0FWB40Z"  =  "1"  /* Revision of Drainage Device in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB42Z"  =  "1"  /* Revision of Monitoring Device in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB43Z"  =  "1"  /* Revision of Infusion Device in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB4CZ"  =  "1"  /* Revision of Extraluminal Device in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB4DZ"  =  "1"  /* Revision of Intraluminal Device in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB4JZ"  =  "1"  /* Revision of Synthetic Substitute in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB70Z"  =  "1"  /* Revision of Drainage Device in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB72Z"  =  "1"  /* Revision of Monitoring Device in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB73Z"  =  "1"  /* Revision of Infusion Device in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB7CZ"  =  "1"  /* Revision of Extraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB7DZ"  =  "1"  /* Revision of Intraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB7JZ"  =  "1"  /* Revision of Synthetic Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB80Z"  =  "1"  /* Revision of Drainage Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB82Z"  =  "1"  /* Revision of Monitoring Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB83Z"  =  "1"  /* Revision of Infusion Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB8CZ"  =  "1"  /* Revision of Extraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB8DZ"  =  "1"  /* Revision of Intraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB8JZ"  =  "1"  /* Revision of Synthetic Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD00Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Open Approach */
"0FWD02Z"  =  "1"  /* Revision of Monitoring Device in Pancreatic Duct, Open Approach */
"0FWD03Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Open Approach */
"0FWD07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pancreatic Duct, Open Approach */
"0FWD0CZ"  =  "1"  /* Revision of Extraluminal Device in Pancreatic Duct, Open Approach */
"0FWD0DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreatic Duct, Open Approach */
"0FWD0JZ"  =  "1"  /* Revision of Synthetic Substitute in Pancreatic Duct, Open Approach */
"0FWD0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Open Approach */
"0FWD30Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Percutaneous Approach */
"0FWD32Z"  =  "1"  /* Revision of Monitoring Device in Pancreatic Duct, Percutaneous Approach */
"0FWD33Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Percutaneous Approach */
"0FWD37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pancreatic Duct, Percutaneous Approach */
"0FWD3CZ"  =  "1"  /* Revision of Extraluminal Device in Pancreatic Duct, Percutaneous Approach */
"0FWD3DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreatic Duct, Percutaneous Approach */
"0FWD3JZ"  =  "1"  /* Revision of Synthetic Substitute in Pancreatic Duct, Percutaneous Approach */
"0FWD3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Percutaneous Approach */
"0FWD40Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD42Z"  =  "1"  /* Revision of Monitoring Device in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD43Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD4CZ"  =  "1"  /* Revision of Extraluminal Device in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD4DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD4JZ"  =  "1"  /* Revision of Synthetic Substitute in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD70Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD72Z"  =  "1"  /* Revision of Monitoring Device in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD73Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD7CZ"  =  "1"  /* Revision of Extraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD7DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD7JZ"  =  "1"  /* Revision of Synthetic Substitute in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD80Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD82Z"  =  "1"  /* Revision of Monitoring Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD83Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD8CZ"  =  "1"  /* Revision of Extraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD8DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD8JZ"  =  "1"  /* Revision of Synthetic Substitute in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWG00Z"  =  "1"  /* Revision of Drainage Device in Pancreas, Open Approach */
"0FWG02Z"  =  "1"  /* Revision of Monitoring Device in Pancreas, Open Approach */
"0FWG03Z"  =  "1"  /* Revision of Infusion Device in Pancreas, Open Approach */
"0FWG0DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreas, Open Approach */
"0FWG30Z"  =  "1"  /* Revision of Drainage Device in Pancreas, Percutaneous Approach */
"0FWG32Z"  =  "1"  /* Revision of Monitoring Device in Pancreas, Percutaneous Approach */
"0FWG33Z"  =  "1"  /* Revision of Infusion Device in Pancreas, Percutaneous Approach */
"0FWG3DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreas, Percutaneous Approach */
"0FWG40Z"  =  "1"  /* Revision of Drainage Device in Pancreas, Percutaneous Endoscopic Approach */
"0FWG42Z"  =  "1"  /* Revision of Monitoring Device in Pancreas, Percutaneous Endoscopic Approach */
"0FWG43Z"  =  "1"  /* Revision of Infusion Device in Pancreas, Percutaneous Endoscopic Approach */
"0FWG4DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreas, Percutaneous Endoscopic Approach */
"0FY00Z0"  =  "1"  /* Transplantation of Liver, Allogeneic, Open Approach */
"0FY00Z1"  =  "1"  /* Transplantation of Liver, Syngeneic, Open Approach */
"0FY00Z2"  =  "1"  /* Transplantation of Liver, Zooplastic, Open Approach */
"0FYG0Z0"  =  "1"  /* Transplantation of Pancreas, Allogeneic, Open Approach */
"0FYG0Z1"  =  "1"  /* Transplantation of Pancreas, Syngeneic, Open Approach */
"0FYG0Z2"  =  "1"  /* Transplantation of Pancreas, Zooplastic, Open Approach */
"0T16078"  =  "1"  /* Bypass Right Ureter to Colon with Autologous Tissue Substitute, Open Approach */
"0T16079"  =  "1"  /* Bypass Right Ureter to Colocutaneous with Autologous Tissue Substitute, Open Approach */
"0T1607A"  =  "1"  /* Bypass Right Ureter to Ileum with Autologous Tissue Substitute, Open Approach */
"0T1607C"  =  "1"  /* Bypass Right Ureter to Ileocutaneous with Autologous Tissue Substitute, Open Approach */
"0T1607D"  =  "1"  /* Bypass Right Ureter to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0T160J8"  =  "1"  /* Bypass Right Ureter to Colon with Synthetic Substitute, Open Approach */
"0T160J9"  =  "1"  /* Bypass Right Ureter to Colocutaneous with Synthetic Substitute, Open Approach */
"0T160JA"  =  "1"  /* Bypass Right Ureter to Ileum with Synthetic Substitute, Open Approach */
"0T160JC"  =  "1"  /* Bypass Right Ureter to Ileocutaneous with Synthetic Substitute, Open Approach */
"0T160JD"  =  "1"  /* Bypass Right Ureter to Cutaneous with Synthetic Substitute, Open Approach */
"0T160K8"  =  "1"  /* Bypass Right Ureter to Colon with Nonautologous Tissue Substitute, Open Approach */
"0T160K9"  =  "1"  /* Bypass Right Ureter to Colocutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T160KA"  =  "1"  /* Bypass Right Ureter to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0T160KC"  =  "1"  /* Bypass Right Ureter to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T160KD"  =  "1"  /* Bypass Right Ureter to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T160Z8"  =  "1"  /* Bypass Right Ureter to Colon, Open Approach */
"0T160Z9"  =  "1"  /* Bypass Right Ureter to Colocutaneous, Open Approach */
"0T160ZA"  =  "1"  /* Bypass Right Ureter to Ileum, Open Approach */
"0T160ZC"  =  "1"  /* Bypass Right Ureter to Ileocutaneous, Open Approach */
"0T160ZD"  =  "1"  /* Bypass Right Ureter to Cutaneous, Open Approach */
"0T163JD"  =  "1"  /* Bypass Right Ureter to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0T16478"  =  "1"  /* Bypass Right Ureter to Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T16479"  =  "1"  /* Bypass Right Ureter to Colocutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1647A"  =  "1"  /* Bypass Right Ureter to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1647C"  =  "1"  /* Bypass Right Ureter to Ileocutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1647D"  =  "1"  /* Bypass Right Ureter to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T164J8"  =  "1"  /* Bypass Right Ureter to Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T164J9"  =  "1"  /* Bypass Right Ureter to Colocutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T164JA"  =  "1"  /* Bypass Right Ureter to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T164JC"  =  "1"  /* Bypass Right Ureter to Ileocutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T164JD"  =  "1"  /* Bypass Right Ureter to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T164K8"  =  "1"  /* Bypass Right Ureter to Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T164K9"  =  "1"  /* Bypass Right Ureter to Colocutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T164KA"  =  "1"  /* Bypass Right Ureter to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T164KC"  =  "1"  /* Bypass Right Ureter to Ileocutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T164KD"  =  "1"  /* Bypass Right Ureter to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T164Z8"  =  "1"  /* Bypass Right Ureter to Colon, Percutaneous Endoscopic Approach */
"0T164Z9"  =  "1"  /* Bypass Right Ureter to Colocutaneous, Percutaneous Endoscopic Approach */
"0T164ZA"  =  "1"  /* Bypass Right Ureter to Ileum, Percutaneous Endoscopic Approach */
"0T164ZC"  =  "1"  /* Bypass Right Ureter to Ileocutaneous, Percutaneous Endoscopic Approach */
"0T164ZD"  =  "1"  /* Bypass Right Ureter to Cutaneous, Percutaneous Endoscopic Approach */
"0T17078"  =  "1"  /* Bypass Left Ureter to Colon with Autologous Tissue Substitute, Open Approach */
"0T17079"  =  "1"  /* Bypass Left Ureter to Colocutaneous with Autologous Tissue Substitute, Open Approach */
"0T1707A"  =  "1"  /* Bypass Left Ureter to Ileum with Autologous Tissue Substitute, Open Approach */
"0T1707C"  =  "1"  /* Bypass Left Ureter to Ileocutaneous with Autologous Tissue Substitute, Open Approach */
"0T1707D"  =  "1"  /* Bypass Left Ureter to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0T170J8"  =  "1"  /* Bypass Left Ureter to Colon with Synthetic Substitute, Open Approach */
"0T170J9"  =  "1"  /* Bypass Left Ureter to Colocutaneous with Synthetic Substitute, Open Approach */
"0T170JA"  =  "1"  /* Bypass Left Ureter to Ileum with Synthetic Substitute, Open Approach */
"0T170JC"  =  "1"  /* Bypass Left Ureter to Ileocutaneous with Synthetic Substitute, Open Approach */
"0T170JD"  =  "1"  /* Bypass Left Ureter to Cutaneous with Synthetic Substitute, Open Approach */
"0T170K8"  =  "1"  /* Bypass Left Ureter to Colon with Nonautologous Tissue Substitute, Open Approach */
"0T170K9"  =  "1"  /* Bypass Left Ureter to Colocutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T170KA"  =  "1"  /* Bypass Left Ureter to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0T170KC"  =  "1"  /* Bypass Left Ureter to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T170KD"  =  "1"  /* Bypass Left Ureter to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T170Z8"  =  "1"  /* Bypass Left Ureter to Colon, Open Approach */
"0T170Z9"  =  "1"  /* Bypass Left Ureter to Colocutaneous, Open Approach */
"0T170ZA"  =  "1"  /* Bypass Left Ureter to Ileum, Open Approach */
"0T170ZC"  =  "1"  /* Bypass Left Ureter to Ileocutaneous, Open Approach */
"0T170ZD"  =  "1"  /* Bypass Left Ureter to Cutaneous, Open Approach */
"0T173JD"  =  "1"  /* Bypass Left Ureter to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0T17478"  =  "1"  /* Bypass Left Ureter to Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T17479"  =  "1"  /* Bypass Left Ureter to Colocutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1747A"  =  "1"  /* Bypass Left Ureter to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1747C"  =  "1"  /* Bypass Left Ureter to Ileocutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1747D"  =  "1"  /* Bypass Left Ureter to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T174J8"  =  "1"  /* Bypass Left Ureter to Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T174J9"  =  "1"  /* Bypass Left Ureter to Colocutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T174JA"  =  "1"  /* Bypass Left Ureter to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T174JC"  =  "1"  /* Bypass Left Ureter to Ileocutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T174JD"  =  "1"  /* Bypass Left Ureter to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T174K8"  =  "1"  /* Bypass Left Ureter to Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T174K9"  =  "1"  /* Bypass Left Ureter to Colocutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T174KA"  =  "1"  /* Bypass Left Ureter to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T174KC"  =  "1"  /* Bypass Left Ureter to Ileocutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T174KD"  =  "1"  /* Bypass Left Ureter to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T174Z8"  =  "1"  /* Bypass Left Ureter to Colon, Percutaneous Endoscopic Approach */
"0T174Z9"  =  "1"  /* Bypass Left Ureter to Colocutaneous, Percutaneous Endoscopic Approach */
"0T174ZA"  =  "1"  /* Bypass Left Ureter to Ileum, Percutaneous Endoscopic Approach */
"0T174ZC"  =  "1"  /* Bypass Left Ureter to Ileocutaneous, Percutaneous Endoscopic Approach */
"0T174ZD"  =  "1"  /* Bypass Left Ureter to Cutaneous, Percutaneous Endoscopic Approach */
"0T18078"  =  "1"  /* Bypass Bilateral Ureters to Colon with Autologous Tissue Substitute, Open Approach */
"0T18079"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous with Autologous Tissue Substitute, Open Approach */
"0T1807A"  =  "1"  /* Bypass Bilateral Ureters to Ileum with Autologous Tissue Substitute, Open Approach */
"0T1807C"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous with Autologous Tissue Substitute, Open Approach */
"0T1807D"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0T180J8"  =  "1"  /* Bypass Bilateral Ureters to Colon with Synthetic Substitute, Open Approach */
"0T180J9"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous with Synthetic Substitute, Open Approach */
"0T180JA"  =  "1"  /* Bypass Bilateral Ureters to Ileum with Synthetic Substitute, Open Approach */
"0T180JC"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous with Synthetic Substitute, Open Approach */
"0T180JD"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous with Synthetic Substitute, Open Approach */
"0T180K8"  =  "1"  /* Bypass Bilateral Ureters to Colon with Nonautologous Tissue Substitute, Open Approach */
"0T180K9"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T180KA"  =  "1"  /* Bypass Bilateral Ureters to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0T180KC"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T180KD"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T180Z8"  =  "1"  /* Bypass Bilateral Ureters to Colon, Open Approach */
"0T180Z9"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous, Open Approach */
"0T180ZA"  =  "1"  /* Bypass Bilateral Ureters to Ileum, Open Approach */
"0T180ZC"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous, Open Approach */
"0T180ZD"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous, Open Approach */
"0T183JD"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0T18478"  =  "1"  /* Bypass Bilateral Ureters to Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T18479"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1847A"  =  "1"  /* Bypass Bilateral Ureters to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1847C"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1847D"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T184J8"  =  "1"  /* Bypass Bilateral Ureters to Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T184J9"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T184JA"  =  "1"  /* Bypass Bilateral Ureters to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T184JC"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T184JD"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T184K8"  =  "1"  /* Bypass Bilateral Ureters to Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T184K9"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T184KA"  =  "1"  /* Bypass Bilateral Ureters to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T184KC"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T184KD"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T184Z8"  =  "1"  /* Bypass Bilateral Ureters to Colon, Percutaneous Endoscopic Approach */
"0T184Z9"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous, Percutaneous Endoscopic Approach */
"0T184ZA"  =  "1"  /* Bypass Bilateral Ureters to Ileum, Percutaneous Endoscopic Approach */
"0T184ZC"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous, Percutaneous Endoscopic Approach */
"0T184ZD"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous, Percutaneous Endoscopic Approach */
"0TQ60ZZ"  =  "1"  /* Repair Right Ureter, Open Approach */
"0TQ63ZZ"  =  "1"  /* Repair Right Ureter, Percutaneous Approach */
"0TQ64ZZ"  =  "1"  /* Repair Right Ureter, Percutaneous Endoscopic Approach */
"0TQ67ZZ"  =  "1"  /* Repair Right Ureter, Via Natural or Artificial Opening */
"0TQ68ZZ"  =  "1"  /* Repair Right Ureter, Via Natural or Artificial Opening Endoscopic */
"0TQ70ZZ"  =  "1"  /* Repair Left Ureter, Open Approach */
"0TQ73ZZ"  =  "1"  /* Repair Left Ureter, Percutaneous Approach */
"0TQ74ZZ"  =  "1"  /* Repair Left Ureter, Percutaneous Endoscopic Approach */
"0TQ77ZZ"  =  "1"  /* Repair Left Ureter, Via Natural or Artificial Opening */
"0TQ78ZZ"  =  "1"  /* Repair Left Ureter, Via Natural or Artificial Opening Endoscopic */
"0TRB07Z"  =  "1"  /* Replacement of Bladder with Autologous Tissue Substitute, Open Approach */
"0TTB0ZZ"  =  "1"  /* Resection of Bladder, Open Approach */
"0TTD0ZZ"  =  "1"  /* Resection of Urethra, Open Approach */
"0U15075"  =  "1"  /* Bypass Right Fallopian Tube to Right Fallopian Tube with Autologous Tissue Substitute, Open Approach */
"0U15076"  =  "1"  /* Bypass Right Fallopian Tube to Left Fallopian Tube with Autologous Tissue Substitute, Open Approach */
"0U15079"  =  "1"  /* Bypass Right Fallopian Tube to Uterus with Autologous Tissue Substitute, Open Approach */
"0U150J5"  =  "1"  /* Bypass Right Fallopian Tube to Right Fallopian Tube with Synthetic Substitute, Open Approach */
"0U150J6"  =  "1"  /* Bypass Right Fallopian Tube to Left Fallopian Tube with Synthetic Substitute, Open Approach */
"0U150J9"  =  "1"  /* Bypass Right Fallopian Tube to Uterus with Synthetic Substitute, Open Approach */
"0U150K5"  =  "1"  /* Bypass Right Fallopian Tube to Right Fallopian Tube with Nonautologous Tissue Substitute, Open Approach */
"0U150K6"  =  "1"  /* Bypass Right Fallopian Tube to Left Fallopian Tube with Nonautologous Tissue Substitute, Open Approach */
"0U150K9"  =  "1"  /* Bypass Right Fallopian Tube to Uterus with Nonautologous Tissue Substitute, Open Approach */
"0U150Z5"  =  "1"  /* Bypass Right Fallopian Tube to Right Fallopian Tube, Open Approach */
"0U150Z6"  =  "1"  /* Bypass Right Fallopian Tube to Left Fallopian Tube, Open Approach */
"0U150Z9"  =  "1"  /* Bypass Right Fallopian Tube to Uterus, Open Approach */
"0U15475"  =  "1"  /* Bypass Right Fallopian Tube to Right Fallopian Tube with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U15476"  =  "1"  /* Bypass Right Fallopian Tube to Left Fallopian Tube with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U15479"  =  "1"  /* Bypass Right Fallopian Tube to Uterus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U154J5"  =  "1"  /* Bypass Right Fallopian Tube to Right Fallopian Tube with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0U154J6"  =  "1"  /* Bypass Right Fallopian Tube to Left Fallopian Tube with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0U154J9"  =  "1"  /* Bypass Right Fallopian Tube to Uterus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0U154K5"  =  "1"  /* Bypass Right Fallopian Tube to Right Fallopian Tube with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U154K6"  =  "1"  /* Bypass Right Fallopian Tube to Left Fallopian Tube with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U154K9"  =  "1"  /* Bypass Right Fallopian Tube to Uterus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U154Z5"  =  "1"  /* Bypass Right Fallopian Tube to Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0U154Z6"  =  "1"  /* Bypass Right Fallopian Tube to Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0U154Z9"  =  "1"  /* Bypass Right Fallopian Tube to Uterus, Percutaneous Endoscopic Approach */
"0U16075"  =  "1"  /* Bypass Left Fallopian Tube to Right Fallopian Tube with Autologous Tissue Substitute, Open Approach */
"0U16076"  =  "1"  /* Bypass Left Fallopian Tube to Left Fallopian Tube with Autologous Tissue Substitute, Open Approach */
"0U16079"  =  "1"  /* Bypass Left Fallopian Tube to Uterus with Autologous Tissue Substitute, Open Approach */
"0U160J5"  =  "1"  /* Bypass Left Fallopian Tube to Right Fallopian Tube with Synthetic Substitute, Open Approach */
"0U160J6"  =  "1"  /* Bypass Left Fallopian Tube to Left Fallopian Tube with Synthetic Substitute, Open Approach */
"0U160J9"  =  "1"  /* Bypass Left Fallopian Tube to Uterus with Synthetic Substitute, Open Approach */
"0U160K5"  =  "1"  /* Bypass Left Fallopian Tube to Right Fallopian Tube with Nonautologous Tissue Substitute, Open Approach */
"0U160K6"  =  "1"  /* Bypass Left Fallopian Tube to Left Fallopian Tube with Nonautologous Tissue Substitute, Open Approach */
"0U160K9"  =  "1"  /* Bypass Left Fallopian Tube to Uterus with Nonautologous Tissue Substitute, Open Approach */
"0U160Z5"  =  "1"  /* Bypass Left Fallopian Tube to Right Fallopian Tube, Open Approach */
"0U160Z6"  =  "1"  /* Bypass Left Fallopian Tube to Left Fallopian Tube, Open Approach */
"0U160Z9"  =  "1"  /* Bypass Left Fallopian Tube to Uterus, Open Approach */
"0U16475"  =  "1"  /* Bypass Left Fallopian Tube to Right Fallopian Tube with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U16476"  =  "1"  /* Bypass Left Fallopian Tube to Left Fallopian Tube with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U16479"  =  "1"  /* Bypass Left Fallopian Tube to Uterus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U164J5"  =  "1"  /* Bypass Left Fallopian Tube to Right Fallopian Tube with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0U164J6"  =  "1"  /* Bypass Left Fallopian Tube to Left Fallopian Tube with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0U164J9"  =  "1"  /* Bypass Left Fallopian Tube to Uterus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0U164K5"  =  "1"  /* Bypass Left Fallopian Tube to Right Fallopian Tube with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U164K6"  =  "1"  /* Bypass Left Fallopian Tube to Left Fallopian Tube with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U164K9"  =  "1"  /* Bypass Left Fallopian Tube to Uterus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U164Z5"  =  "1"  /* Bypass Left Fallopian Tube to Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0U164Z6"  =  "1"  /* Bypass Left Fallopian Tube to Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0U164Z9"  =  "1"  /* Bypass Left Fallopian Tube to Uterus, Percutaneous Endoscopic Approach */
"0U500ZZ"  =  "1"  /* Destruction of Right Ovary, Open Approach */
"0U503ZZ"  =  "1"  /* Destruction of Right Ovary, Percutaneous Approach */
"0U504ZZ"  =  "1"  /* Destruction of Right Ovary, Percutaneous Endoscopic Approach */
"0U510ZZ"  =  "1"  /* Destruction of Left Ovary, Open Approach */
"0U513ZZ"  =  "1"  /* Destruction of Left Ovary, Percutaneous Approach */
"0U514ZZ"  =  "1"  /* Destruction of Left Ovary, Percutaneous Endoscopic Approach */
"0U520ZZ"  =  "1"  /* Destruction of Bilateral Ovaries, Open Approach */
"0U523ZZ"  =  "1"  /* Destruction of Bilateral Ovaries, Percutaneous Approach */
"0U524ZZ"  =  "1"  /* Destruction of Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0U540ZZ"  =  "1"  /* Destruction of Uterine Supporting Structure, Open Approach */
"0U543ZZ"  =  "1"  /* Destruction of Uterine Supporting Structure, Percutaneous Approach */
"0U544ZZ"  =  "1"  /* Destruction of Uterine Supporting Structure, Percutaneous Endoscopic Approach */
"0U550ZZ"  =  "1"  /* Destruction of Right Fallopian Tube, Open Approach */
"0U553ZZ"  =  "1"  /* Destruction of Right Fallopian Tube, Percutaneous Approach */
"0U554ZZ"  =  "1"  /* Destruction of Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0U560ZZ"  =  "1"  /* Destruction of Left Fallopian Tube, Open Approach */
"0U563ZZ"  =  "1"  /* Destruction of Left Fallopian Tube, Percutaneous Approach */
"0U564ZZ"  =  "1"  /* Destruction of Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0U570ZZ"  =  "1"  /* Destruction of Bilateral Fallopian Tubes, Open Approach */
"0U573ZZ"  =  "1"  /* Destruction of Bilateral Fallopian Tubes, Percutaneous Approach */
"0U574ZZ"  =  "1"  /* Destruction of Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0U590ZZ"  =  "1"  /* Destruction of Uterus, Open Approach */
"0U593ZZ"  =  "1"  /* Destruction of Uterus, Percutaneous Approach */
"0U594ZZ"  =  "1"  /* Destruction of Uterus, Percutaneous Endoscopic Approach */
"0U5B0ZZ"  =  "1"  /* Destruction of Endometrium, Open Approach */
"0U5B3ZZ"  =  "1"  /* Destruction of Endometrium, Percutaneous Approach */
"0U5B4ZZ"  =  "1"  /* Destruction of Endometrium, Percutaneous Endoscopic Approach */
"0U750DZ"  =  "1"  /* Dilation of Right Fallopian Tube with Intraluminal Device, Open Approach */
"0U750ZZ"  =  "1"  /* Dilation of Right Fallopian Tube, Open Approach */
"0U753DZ"  =  "1"  /* Dilation of Right Fallopian Tube with Intraluminal Device, Percutaneous Approach */
"0U753ZZ"  =  "1"  /* Dilation of Right Fallopian Tube, Percutaneous Approach */
"0U754DZ"  =  "1"  /* Dilation of Right Fallopian Tube with Intraluminal Device, Percutaneous Endoscopic Approach */
"0U754ZZ"  =  "1"  /* Dilation of Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0U760DZ"  =  "1"  /* Dilation of Left Fallopian Tube with Intraluminal Device, Open Approach */
"0U760ZZ"  =  "1"  /* Dilation of Left Fallopian Tube, Open Approach */
"0U763DZ"  =  "1"  /* Dilation of Left Fallopian Tube with Intraluminal Device, Percutaneous Approach */
"0U763ZZ"  =  "1"  /* Dilation of Left Fallopian Tube, Percutaneous Approach */
"0U764DZ"  =  "1"  /* Dilation of Left Fallopian Tube with Intraluminal Device, Percutaneous Endoscopic Approach */
"0U764ZZ"  =  "1"  /* Dilation of Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0U770DZ"  =  "1"  /* Dilation of Bilateral Fallopian Tubes with Intraluminal Device, Open Approach */
"0U770ZZ"  =  "1"  /* Dilation of Bilateral Fallopian Tubes, Open Approach */
"0U773DZ"  =  "1"  /* Dilation of Bilateral Fallopian Tubes with Intraluminal Device, Percutaneous Approach */
"0U773ZZ"  =  "1"  /* Dilation of Bilateral Fallopian Tubes, Percutaneous Approach */
"0U774DZ"  =  "1"  /* Dilation of Bilateral Fallopian Tubes with Intraluminal Device, Percutaneous Endoscopic Approach */
"0U774ZZ"  =  "1"  /* Dilation of Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0U790DZ"  =  "1"  /* Dilation of Uterus with Intraluminal Device, Open Approach */
"0U790ZZ"  =  "1"  /* Dilation of Uterus, Open Approach */
"0U793DZ"  =  "1"  /* Dilation of Uterus with Intraluminal Device, Percutaneous Approach */
"0U793ZZ"  =  "1"  /* Dilation of Uterus, Percutaneous Approach */
"0U794DZ"  =  "1"  /* Dilation of Uterus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0U794ZZ"  =  "1"  /* Dilation of Uterus, Percutaneous Endoscopic Approach */
"0U7K0DZ"  =  "1"  /* Dilation of Hymen with Intraluminal Device, Open Approach */
"0U7K0ZZ"  =  "1"  /* Dilation of Hymen, Open Approach */
"0U7K3DZ"  =  "1"  /* Dilation of Hymen with Intraluminal Device, Percutaneous Approach */
"0U7K3ZZ"  =  "1"  /* Dilation of Hymen, Percutaneous Approach */
"0U7K4DZ"  =  "1"  /* Dilation of Hymen with Intraluminal Device, Percutaneous Endoscopic Approach */
"0U7K4ZZ"  =  "1"  /* Dilation of Hymen, Percutaneous Endoscopic Approach */
"0U7KXDZ"  =  "1"  /* Dilation of Hymen with Intraluminal Device, External Approach */
"0U7KXZZ"  =  "1"  /* Dilation of Hymen, External Approach */
"0U800ZZ"  =  "1"  /* Division of Right Ovary, Open Approach */
"0U803ZZ"  =  "1"  /* Division of Right Ovary, Percutaneous Approach */
"0U804ZZ"  =  "1"  /* Division of Right Ovary, Percutaneous Endoscopic Approach */
"0U810ZZ"  =  "1"  /* Division of Left Ovary, Open Approach */
"0U813ZZ"  =  "1"  /* Division of Left Ovary, Percutaneous Approach */
"0U814ZZ"  =  "1"  /* Division of Left Ovary, Percutaneous Endoscopic Approach */
"0U820ZZ"  =  "1"  /* Division of Bilateral Ovaries, Open Approach */
"0U823ZZ"  =  "1"  /* Division of Bilateral Ovaries, Percutaneous Approach */
"0U824ZZ"  =  "1"  /* Division of Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0U840ZZ"  =  "1"  /* Division of Uterine Supporting Structure, Open Approach */
"0U843ZZ"  =  "1"  /* Division of Uterine Supporting Structure, Percutaneous Approach */
"0U844ZZ"  =  "1"  /* Division of Uterine Supporting Structure, Percutaneous Endoscopic Approach */
"0U9000Z"  =  "1"  /* Drainage of Right Ovary with Drainage Device, Open Approach */
"0U900ZX"  =  "1"  /* Drainage of Right Ovary, Open Approach, Diagnostic */
"0U900ZZ"  =  "1"  /* Drainage of Right Ovary, Open Approach */
"0U903ZZ"  =  "1"  /* Drainage of Right Ovary, Percutaneous Approach */
"0U9040Z"  =  "1"  /* Drainage of Right Ovary with Drainage Device, Percutaneous Endoscopic Approach */
"0U904ZZ"  =  "1"  /* Drainage of Right Ovary, Percutaneous Endoscopic Approach */
"0U90XZZ"  =  "1"  /* Drainage of Right Ovary, External Approach */
"0U9100Z"  =  "1"  /* Drainage of Left Ovary with Drainage Device, Open Approach */
"0U910ZX"  =  "1"  /* Drainage of Left Ovary, Open Approach, Diagnostic */
"0U910ZZ"  =  "1"  /* Drainage of Left Ovary, Open Approach */
"0U913ZZ"  =  "1"  /* Drainage of Left Ovary, Percutaneous Approach */
"0U9140Z"  =  "1"  /* Drainage of Left Ovary with Drainage Device, Percutaneous Endoscopic Approach */
"0U914ZZ"  =  "1"  /* Drainage of Left Ovary, Percutaneous Endoscopic Approach */
"0U91XZZ"  =  "1"  /* Drainage of Left Ovary, External Approach */
"0U9200Z"  =  "1"  /* Drainage of Bilateral Ovaries with Drainage Device, Open Approach */
"0U920ZX"  =  "1"  /* Drainage of Bilateral Ovaries, Open Approach, Diagnostic */
"0U920ZZ"  =  "1"  /* Drainage of Bilateral Ovaries, Open Approach */
"0U923ZZ"  =  "1"  /* Drainage of Bilateral Ovaries, Percutaneous Approach */
"0U9240Z"  =  "1"  /* Drainage of Bilateral Ovaries with Drainage Device, Percutaneous Endoscopic Approach */
"0U924ZZ"  =  "1"  /* Drainage of Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0U92XZZ"  =  "1"  /* Drainage of Bilateral Ovaries, External Approach */
"0U9400Z"  =  "1"  /* Drainage of Uterine Supporting Structure with Drainage Device, Open Approach */
"0U940ZX"  =  "1"  /* Drainage of Uterine Supporting Structure, Open Approach, Diagnostic */
"0U940ZZ"  =  "1"  /* Drainage of Uterine Supporting Structure, Open Approach */
"0U9430Z"  =  "1"  /* Drainage of Uterine Supporting Structure with Drainage Device, Percutaneous Approach */
"0U943ZZ"  =  "1"  /* Drainage of Uterine Supporting Structure, Percutaneous Approach */
"0U9440Z"  =  "1"  /* Drainage of Uterine Supporting Structure with Drainage Device, Percutaneous Endoscopic Approach */
"0U944ZZ"  =  "1"  /* Drainage of Uterine Supporting Structure, Percutaneous Endoscopic Approach */
"0U9500Z"  =  "1"  /* Drainage of Right Fallopian Tube with Drainage Device, Open Approach */
"0U950ZX"  =  "1"  /* Drainage of Right Fallopian Tube, Open Approach, Diagnostic */
"0U950ZZ"  =  "1"  /* Drainage of Right Fallopian Tube, Open Approach */
"0U9530Z"  =  "1"  /* Drainage of Right Fallopian Tube with Drainage Device, Percutaneous Approach */
"0U953ZX"  =  "1"  /* Drainage of Right Fallopian Tube, Percutaneous Approach, Diagnostic */
"0U9540Z"  =  "1"  /* Drainage of Right Fallopian Tube with Drainage Device, Percutaneous Endoscopic Approach */
"0U954ZX"  =  "1"  /* Drainage of Right Fallopian Tube, Percutaneous Endoscopic Approach, Diagnostic */
"0U9600Z"  =  "1"  /* Drainage of Left Fallopian Tube with Drainage Device, Open Approach */
"0U960ZX"  =  "1"  /* Drainage of Left Fallopian Tube, Open Approach, Diagnostic */
"0U960ZZ"  =  "1"  /* Drainage of Left Fallopian Tube, Open Approach */
"0U9630Z"  =  "1"  /* Drainage of Left Fallopian Tube with Drainage Device, Percutaneous Approach */
"0U963ZX"  =  "1"  /* Drainage of Left Fallopian Tube, Percutaneous Approach, Diagnostic */
"0U9640Z"  =  "1"  /* Drainage of Left Fallopian Tube with Drainage Device, Percutaneous Endoscopic Approach */
"0U964ZX"  =  "1"  /* Drainage of Left Fallopian Tube, Percutaneous Endoscopic Approach, Diagnostic */
"0U9700Z"  =  "1"  /* Drainage of Bilateral Fallopian Tubes with Drainage Device, Open Approach */
"0U970ZX"  =  "1"  /* Drainage of Bilateral Fallopian Tubes, Open Approach, Diagnostic */
"0U970ZZ"  =  "1"  /* Drainage of Bilateral Fallopian Tubes, Open Approach */
"0U9730Z"  =  "1"  /* Drainage of Bilateral Fallopian Tubes with Drainage Device, Percutaneous Approach */
"0U973ZX"  =  "1"  /* Drainage of Bilateral Fallopian Tubes, Percutaneous Approach, Diagnostic */
"0U9740Z"  =  "1"  /* Drainage of Bilateral Fallopian Tubes with Drainage Device, Percutaneous Endoscopic Approach */
"0U974ZX"  =  "1"  /* Drainage of Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach, Diagnostic */
"0U9900Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Open Approach */
"0U990ZX"  =  "1"  /* Drainage of Uterus, Open Approach, Diagnostic */
"0U990ZZ"  =  "1"  /* Drainage of Uterus, Open Approach */
"0U9930Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Percutaneous Approach */
"0U993ZZ"  =  "1"  /* Drainage of Uterus, Percutaneous Approach */
"0U9940Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Percutaneous Endoscopic Approach */
"0U994ZZ"  =  "1"  /* Drainage of Uterus, Percutaneous Endoscopic Approach */
"0UB00ZX"  =  "1"  /* Excision of Right Ovary, Open Approach, Diagnostic */
"0UB00ZZ"  =  "1"  /* Excision of Right Ovary, Open Approach */
"0UB03ZZ"  =  "1"  /* Excision of Right Ovary, Percutaneous Approach */
"0UB04ZZ"  =  "1"  /* Excision of Right Ovary, Percutaneous Endoscopic Approach */
"0UB10ZX"  =  "1"  /* Excision of Left Ovary, Open Approach, Diagnostic */
"0UB10ZZ"  =  "1"  /* Excision of Left Ovary, Open Approach */
"0UB13ZZ"  =  "1"  /* Excision of Left Ovary, Percutaneous Approach */
"0UB14ZZ"  =  "1"  /* Excision of Left Ovary, Percutaneous Endoscopic Approach */
"0UB20ZX"  =  "1"  /* Excision of Bilateral Ovaries, Open Approach, Diagnostic */
"0UB20ZZ"  =  "1"  /* Excision of Bilateral Ovaries, Open Approach */
"0UB23ZZ"  =  "1"  /* Excision of Bilateral Ovaries, Percutaneous Approach */
"0UB24ZZ"  =  "1"  /* Excision of Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0UB40ZX"  =  "1"  /* Excision of Uterine Supporting Structure, Open Approach, Diagnostic */
"0UB40ZZ"  =  "1"  /* Excision of Uterine Supporting Structure, Open Approach */
"0UB43ZZ"  =  "1"  /* Excision of Uterine Supporting Structure, Percutaneous Approach */
"0UB44ZZ"  =  "1"  /* Excision of Uterine Supporting Structure, Percutaneous Endoscopic Approach */
"0UB50ZX"  =  "1"  /* Excision of Right Fallopian Tube, Open Approach, Diagnostic */
"0UB50ZZ"  =  "1"  /* Excision of Right Fallopian Tube, Open Approach */
"0UB53ZX"  =  "1"  /* Excision of Right Fallopian Tube, Percutaneous Approach, Diagnostic */
"0UB53ZZ"  =  "1"  /* Excision of Right Fallopian Tube, Percutaneous Approach */
"0UB54ZX"  =  "1"  /* Excision of Right Fallopian Tube, Percutaneous Endoscopic Approach, Diagnostic */
"0UB54ZZ"  =  "1"  /* Excision of Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0UB60ZX"  =  "1"  /* Excision of Left Fallopian Tube, Open Approach, Diagnostic */
"0UB60ZZ"  =  "1"  /* Excision of Left Fallopian Tube, Open Approach */
"0UB63ZX"  =  "1"  /* Excision of Left Fallopian Tube, Percutaneous Approach, Diagnostic */
"0UB63ZZ"  =  "1"  /* Excision of Left Fallopian Tube, Percutaneous Approach */
"0UB64ZX"  =  "1"  /* Excision of Left Fallopian Tube, Percutaneous Endoscopic Approach, Diagnostic */
"0UB64ZZ"  =  "1"  /* Excision of Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UB70ZX"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Open Approach, Diagnostic */
"0UB70ZZ"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Open Approach */
"0UB73ZX"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Percutaneous Approach, Diagnostic */
"0UB73ZZ"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Percutaneous Approach */
"0UB74ZX"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach, Diagnostic */
"0UB74ZZ"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0UB90ZX"  =  "1"  /* Excision of Uterus, Open Approach, Diagnostic */
"0UB90ZZ"  =  "1"  /* Excision of Uterus, Open Approach */
"0UB93ZZ"  =  "1"  /* Excision of Uterus, Percutaneous Approach */
"0UB94ZZ"  =  "1"  /* Excision of Uterus, Percutaneous Endoscopic Approach */
"0UC00ZZ"  =  "1"  /* Extirpation of Matter from Right Ovary, Open Approach */
"0UC03ZZ"  =  "1"  /* Extirpation of Matter from Right Ovary, Percutaneous Approach */
"0UC04ZZ"  =  "1"  /* Extirpation of Matter from Right Ovary, Percutaneous Endoscopic Approach */
"0UC10ZZ"  =  "1"  /* Extirpation of Matter from Left Ovary, Open Approach */
"0UC13ZZ"  =  "1"  /* Extirpation of Matter from Left Ovary, Percutaneous Approach */
"0UC14ZZ"  =  "1"  /* Extirpation of Matter from Left Ovary, Percutaneous Endoscopic Approach */
"0UC20ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Ovaries, Open Approach */
"0UC23ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Ovaries, Percutaneous Approach */
"0UC24ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0UC40ZZ"  =  "1"  /* Extirpation of Matter from Uterine Supporting Structure, Open Approach */
"0UC43ZZ"  =  "1"  /* Extirpation of Matter from Uterine Supporting Structure, Percutaneous Approach */
"0UC44ZZ"  =  "1"  /* Extirpation of Matter from Uterine Supporting Structure, Percutaneous Endoscopic Approach */
"0UC50ZZ"  =  "1"  /* Extirpation of Matter from Right Fallopian Tube, Open Approach */
"0UC53ZZ"  =  "1"  /* Extirpation of Matter from Right Fallopian Tube, Percutaneous Approach */
"0UC54ZZ"  =  "1"  /* Extirpation of Matter from Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0UC60ZZ"  =  "1"  /* Extirpation of Matter from Left Fallopian Tube, Open Approach */
"0UC63ZZ"  =  "1"  /* Extirpation of Matter from Left Fallopian Tube, Percutaneous Approach */
"0UC64ZZ"  =  "1"  /* Extirpation of Matter from Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UC70ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Fallopian Tubes, Open Approach */
"0UC73ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Fallopian Tubes, Percutaneous Approach */
"0UC74ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0UC90ZZ"  =  "1"  /* Extirpation of Matter from Uterus, Open Approach */
"0UC93ZZ"  =  "1"  /* Extirpation of Matter from Uterus, Percutaneous Approach */
"0UC94ZZ"  =  "1"  /* Extirpation of Matter from Uterus, Percutaneous Endoscopic Approach */
"0UF50ZZ"  =  "1"  /* Fragmentation in Right Fallopian Tube, Open Approach */
"0UF53ZZ"  =  "1"  /* Fragmentation in Right Fallopian Tube, Percutaneous Approach */
"0UF54ZZ"  =  "1"  /* Fragmentation in Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0UF60ZZ"  =  "1"  /* Fragmentation in Left Fallopian Tube, Open Approach */
"0UF63ZZ"  =  "1"  /* Fragmentation in Left Fallopian Tube, Percutaneous Approach */
"0UF64ZZ"  =  "1"  /* Fragmentation in Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UF70ZZ"  =  "1"  /* Fragmentation in Bilateral Fallopian Tubes, Open Approach */
"0UF73ZZ"  =  "1"  /* Fragmentation in Bilateral Fallopian Tubes, Percutaneous Approach */
"0UF74ZZ"  =  "1"  /* Fragmentation in Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0UJ80ZZ"  =  "1"  /* Inspection of Fallopian Tube, Open Approach */
"0UJ83ZZ"  =  "1"  /* Inspection of Fallopian Tube, Percutaneous Approach */
"0UJ84ZZ"  =  "1"  /* Inspection of Fallopian Tube, Percutaneous Endoscopic Approach */
"0UJD0ZZ"  =  "1"  /* Inspection of Uterus and Cervix, Open Approach */
"0UJD3ZZ"  =  "1"  /* Inspection of Uterus and Cervix, Percutaneous Approach */
"0UJD4ZZ"  =  "1"  /* Inspection of Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UL50CZ"  =  "1"  /* Occlusion of Right Fallopian Tube with Extraluminal Device, Open Approach */
"0UL50DZ"  =  "1"  /* Occlusion of Right Fallopian Tube with Intraluminal Device, Open Approach */
"0UL50ZZ"  =  "1"  /* Occlusion of Right Fallopian Tube, Open Approach */
"0UL53CZ"  =  "1"  /* Occlusion of Right Fallopian Tube with Extraluminal Device, Percutaneous Approach */
"0UL53DZ"  =  "1"  /* Occlusion of Right Fallopian Tube with Intraluminal Device, Percutaneous Approach */
"0UL53ZZ"  =  "1"  /* Occlusion of Right Fallopian Tube, Percutaneous Approach */
"0UL54CZ"  =  "1"  /* Occlusion of Right Fallopian Tube with Extraluminal Device, Percutaneous Endoscopic Approach */
"0UL54DZ"  =  "1"  /* Occlusion of Right Fallopian Tube with Intraluminal Device, Percutaneous Endoscopic Approach */
"0UL54ZZ"  =  "1"  /* Occlusion of Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0UL60CZ"  =  "1"  /* Occlusion of Left Fallopian Tube with Extraluminal Device, Open Approach */
"0UL60DZ"  =  "1"  /* Occlusion of Left Fallopian Tube with Intraluminal Device, Open Approach */
"0UL60ZZ"  =  "1"  /* Occlusion of Left Fallopian Tube, Open Approach */
"0UL63CZ"  =  "1"  /* Occlusion of Left Fallopian Tube with Extraluminal Device, Percutaneous Approach */
"0UL63DZ"  =  "1"  /* Occlusion of Left Fallopian Tube with Intraluminal Device, Percutaneous Approach */
"0UL63ZZ"  =  "1"  /* Occlusion of Left Fallopian Tube, Percutaneous Approach */
"0UL64CZ"  =  "1"  /* Occlusion of Left Fallopian Tube with Extraluminal Device, Percutaneous Endoscopic Approach */
"0UL64DZ"  =  "1"  /* Occlusion of Left Fallopian Tube with Intraluminal Device, Percutaneous Endoscopic Approach */
"0UL64ZZ"  =  "1"  /* Occlusion of Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UL70CZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes with Extraluminal Device, Open Approach */
"0UL70DZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes with Intraluminal Device, Open Approach */
"0UL70ZZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes, Open Approach */
"0UL73CZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes with Extraluminal Device, Percutaneous Approach */
"0UL73DZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes with Intraluminal Device, Percutaneous Approach */
"0UL73ZZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes, Percutaneous Approach */
"0UL74ZZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0UM00ZZ"  =  "1"  /* Reattachment of Right Ovary, Open Approach */
"0UM04ZZ"  =  "1"  /* Reattachment of Right Ovary, Percutaneous Endoscopic Approach */
"0UM10ZZ"  =  "1"  /* Reattachment of Left Ovary, Open Approach */
"0UM14ZZ"  =  "1"  /* Reattachment of Left Ovary, Percutaneous Endoscopic Approach */
"0UM20ZZ"  =  "1"  /* Reattachment of Bilateral Ovaries, Open Approach */
"0UM24ZZ"  =  "1"  /* Reattachment of Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0UM40ZZ"  =  "1"  /* Reattachment of Uterine Supporting Structure, Open Approach */
"0UM44ZZ"  =  "1"  /* Reattachment of Uterine Supporting Structure, Percutaneous Endoscopic Approach */
"0UM50ZZ"  =  "1"  /* Reattachment of Right Fallopian Tube, Open Approach */
"0UM54ZZ"  =  "1"  /* Reattachment of Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0UM60ZZ"  =  "1"  /* Reattachment of Left Fallopian Tube, Open Approach */
"0UM64ZZ"  =  "1"  /* Reattachment of Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UM70ZZ"  =  "1"  /* Reattachment of Bilateral Fallopian Tubes, Open Approach */
"0UM74ZZ"  =  "1"  /* Reattachment of Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0UN00ZZ"  =  "1"  /* Release Right Ovary, Open Approach */
"0UN03ZZ"  =  "1"  /* Release Right Ovary, Percutaneous Approach */
"0UN04ZZ"  =  "1"  /* Release Right Ovary, Percutaneous Endoscopic Approach */
"0UN10ZZ"  =  "1"  /* Release Left Ovary, Open Approach */
"0UN13ZZ"  =  "1"  /* Release Left Ovary, Percutaneous Approach */
"0UN14ZZ"  =  "1"  /* Release Left Ovary, Percutaneous Endoscopic Approach */
"0UN20ZZ"  =  "1"  /* Release Bilateral Ovaries, Open Approach */
"0UN23ZZ"  =  "1"  /* Release Bilateral Ovaries, Percutaneous Approach */
"0UN24ZZ"  =  "1"  /* Release Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0UN40ZZ"  =  "1"  /* Release Uterine Supporting Structure, Open Approach */
"0UN43ZZ"  =  "1"  /* Release Uterine Supporting Structure, Percutaneous Approach */
"0UN44ZZ"  =  "1"  /* Release Uterine Supporting Structure, Percutaneous Endoscopic Approach */
"0UN50ZZ"  =  "1"  /* Release Right Fallopian Tube, Open Approach */
"0UN53ZZ"  =  "1"  /* Release Right Fallopian Tube, Percutaneous Approach */
"0UN54ZZ"  =  "1"  /* Release Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0UN60ZZ"  =  "1"  /* Release Left Fallopian Tube, Open Approach */
"0UN63ZZ"  =  "1"  /* Release Left Fallopian Tube, Percutaneous Approach */
"0UN64ZZ"  =  "1"  /* Release Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UN70ZZ"  =  "1"  /* Release Bilateral Fallopian Tubes, Open Approach */
"0UN73ZZ"  =  "1"  /* Release Bilateral Fallopian Tubes, Percutaneous Approach */
"0UN74ZZ"  =  "1"  /* Release Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0UP300Z"  =  "1"  /* Removal of Drainage Device from Ovary, Open Approach */
"0UP303Z"  =  "1"  /* Removal of Infusion Device from Ovary, Open Approach */
"0UP330Z"  =  "1"  /* Removal of Drainage Device from Ovary, Percutaneous Approach */
"0UP333Z"  =  "1"  /* Removal of Infusion Device from Ovary, Percutaneous Approach */
"0UP340Z"  =  "1"  /* Removal of Drainage Device from Ovary, Percutaneous Endoscopic Approach */
"0UP343Z"  =  "1"  /* Removal of Infusion Device from Ovary, Percutaneous Endoscopic Approach */
"0UP800Z"  =  "1"  /* Removal of Drainage Device from Fallopian Tube, Open Approach */
"0UP803Z"  =  "1"  /* Removal of Infusion Device from Fallopian Tube, Open Approach */
"0UP807Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Fallopian Tube, Open Approach */
"0UP80CZ"  =  "1"  /* Removal of Extraluminal Device from Fallopian Tube, Open Approach */
"0UP80DZ"  =  "1"  /* Removal of Intraluminal Device from Fallopian Tube, Open Approach */
"0UP80JZ"  =  "1"  /* Removal of Synthetic Substitute from Fallopian Tube, Open Approach */
"0UP80KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Fallopian Tube, Open Approach */
"0UP830Z"  =  "1"  /* Removal of Drainage Device from Fallopian Tube, Percutaneous Approach */
"0UP833Z"  =  "1"  /* Removal of Infusion Device from Fallopian Tube, Percutaneous Approach */
"0UP837Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Fallopian Tube, Percutaneous Approach */
"0UP83CZ"  =  "1"  /* Removal of Extraluminal Device from Fallopian Tube, Percutaneous Approach */
"0UP83DZ"  =  "1"  /* Removal of Intraluminal Device from Fallopian Tube, Percutaneous Approach */
"0UP83JZ"  =  "1"  /* Removal of Synthetic Substitute from Fallopian Tube, Percutaneous Approach */
"0UP83KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Fallopian Tube, Percutaneous Approach */
"0UP840Z"  =  "1"  /* Removal of Drainage Device from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP843Z"  =  "1"  /* Removal of Infusion Device from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP847Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP84CZ"  =  "1"  /* Removal of Extraluminal Device from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP84DZ"  =  "1"  /* Removal of Intraluminal Device from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP84JZ"  =  "1"  /* Removal of Synthetic Substitute from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP84KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UPD00Z"  =  "1"  /* Removal of Drainage Device from Uterus and Cervix, Open Approach */
"0UPD01Z"  =  "1"  /* Removal of Radioactive Element from Uterus and Cervix, Open Approach */
"0UPD03Z"  =  "1"  /* Removal of Infusion Device from Uterus and Cervix, Open Approach */
"0UPD07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Uterus and Cervix, Open Approach */
"0UPD0DZ"  =  "1"  /* Removal of Intraluminal Device from Uterus and Cervix, Open Approach */
"0UPD0HZ"  =  "1"  /* Removal of Contraceptive Device from Uterus and Cervix, Open Approach */
"0UPD0JZ"  =  "1"  /* Removal of Synthetic Substitute from Uterus and Cervix, Open Approach */
"0UPD0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Uterus and Cervix, Open Approach */
"0UPD30Z"  =  "1"  /* Removal of Drainage Device from Uterus and Cervix, Percutaneous Approach */
"0UPD31Z"  =  "1"  /* Removal of Radioactive Element from Uterus and Cervix, Percutaneous Approach */
"0UPD33Z"  =  "1"  /* Removal of Infusion Device from Uterus and Cervix, Percutaneous Approach */
"0UPD37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Uterus and Cervix, Percutaneous Approach */
"0UPD3DZ"  =  "1"  /* Removal of Intraluminal Device from Uterus and Cervix, Percutaneous Approach */
"0UPD3HZ"  =  "1"  /* Removal of Contraceptive Device from Uterus and Cervix, Percutaneous Approach */
"0UPD3JZ"  =  "1"  /* Removal of Synthetic Substitute from Uterus and Cervix, Percutaneous Approach */
"0UPD3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Uterus and Cervix, Percutaneous Approach */
"0UPD40Z"  =  "1"  /* Removal of Drainage Device from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD41Z"  =  "1"  /* Removal of Radioactive Element from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD43Z"  =  "1"  /* Removal of Infusion Device from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD4DZ"  =  "1"  /* Removal of Intraluminal Device from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD4HZ"  =  "1"  /* Removal of Contraceptive Device from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD4JZ"  =  "1"  /* Removal of Synthetic Substitute from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UQ00ZZ"  =  "1"  /* Repair Right Ovary, Open Approach */
"0UQ03ZZ"  =  "1"  /* Repair Right Ovary, Percutaneous Approach */
"0UQ04ZZ"  =  "1"  /* Repair Right Ovary, Percutaneous Endoscopic Approach */
"0UQ10ZZ"  =  "1"  /* Repair Left Ovary, Open Approach */
"0UQ13ZZ"  =  "1"  /* Repair Left Ovary, Percutaneous Approach */
"0UQ14ZZ"  =  "1"  /* Repair Left Ovary, Percutaneous Endoscopic Approach */
"0UQ20ZZ"  =  "1"  /* Repair Bilateral Ovaries, Open Approach */
"0UQ23ZZ"  =  "1"  /* Repair Bilateral Ovaries, Percutaneous Approach */
"0UQ24ZZ"  =  "1"  /* Repair Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0UQ40ZZ"  =  "1"  /* Repair Uterine Supporting Structure, Open Approach */
"0UQ43ZZ"  =  "1"  /* Repair Uterine Supporting Structure, Percutaneous Approach */
"0UQ44ZZ"  =  "1"  /* Repair Uterine Supporting Structure, Percutaneous Endoscopic Approach */
"0UQ50ZZ"  =  "1"  /* Repair Right Fallopian Tube, Open Approach */
"0UQ53ZZ"  =  "1"  /* Repair Right Fallopian Tube, Percutaneous Approach */
"0UQ54ZZ"  =  "1"  /* Repair Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0UQ60ZZ"  =  "1"  /* Repair Left Fallopian Tube, Open Approach */
"0UQ63ZZ"  =  "1"  /* Repair Left Fallopian Tube, Percutaneous Approach */
"0UQ64ZZ"  =  "1"  /* Repair Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UQ70ZZ"  =  "1"  /* Repair Bilateral Fallopian Tubes, Open Approach */
"0UQ73ZZ"  =  "1"  /* Repair Bilateral Fallopian Tubes, Percutaneous Approach */
"0UQ74ZZ"  =  "1"  /* Repair Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0UQ90ZZ"  =  "1"  /* Repair Uterus, Open Approach */
"0UQ93ZZ"  =  "1"  /* Repair Uterus, Percutaneous Approach */
"0UQ94ZZ"  =  "1"  /* Repair Uterus, Percutaneous Endoscopic Approach */
"0UQG0ZZ"  =  "1"  /* Repair Vagina, Open Approach */
"0UQG3ZZ"  =  "1"  /* Repair Vagina, Percutaneous Approach */
"0UQG4ZZ"  =  "1"  /* Repair Vagina, Percutaneous Endoscopic Approach */
"0US00ZZ"  =  "1"  /* Reposition Right Ovary, Open Approach */
"0US04ZZ"  =  "1"  /* Reposition Right Ovary, Percutaneous Endoscopic Approach */
"0US10ZZ"  =  "1"  /* Reposition Left Ovary, Open Approach */
"0US14ZZ"  =  "1"  /* Reposition Left Ovary, Percutaneous Endoscopic Approach */
"0US20ZZ"  =  "1"  /* Reposition Bilateral Ovaries, Open Approach */
"0US24ZZ"  =  "1"  /* Reposition Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0US40ZZ"  =  "1"  /* Reposition Uterine Supporting Structure, Open Approach */
"0US44ZZ"  =  "1"  /* Reposition Uterine Supporting Structure, Percutaneous Endoscopic Approach */
"0US50ZZ"  =  "1"  /* Reposition Right Fallopian Tube, Open Approach */
"0US54ZZ"  =  "1"  /* Reposition Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0US60ZZ"  =  "1"  /* Reposition Left Fallopian Tube, Open Approach */
"0US64ZZ"  =  "1"  /* Reposition Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0US70ZZ"  =  "1"  /* Reposition Bilateral Fallopian Tubes, Open Approach */
"0US74ZZ"  =  "1"  /* Reposition Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0US90ZZ"  =  "1"  /* Reposition Uterus, Open Approach */
"0US94ZZ"  =  "1"  /* Reposition Uterus, Percutaneous Endoscopic Approach */
"0USC0ZZ"  =  "1"  /* Reposition Cervix, Open Approach */
"0USC4ZZ"  =  "1"  /* Reposition Cervix, Percutaneous Endoscopic Approach */
"0UT00ZZ"  =  "1"  /* Resection of Right Ovary, Open Approach */
"0UT04ZZ"  =  "1"  /* Resection of Right Ovary, Percutaneous Endoscopic Approach */
"0UT10ZZ"  =  "1"  /* Resection of Left Ovary, Open Approach */
"0UT14ZZ"  =  "1"  /* Resection of Left Ovary, Percutaneous Endoscopic Approach */
"0UT20ZZ"  =  "1"  /* Resection of Bilateral Ovaries, Open Approach */
"0UT24ZZ"  =  "1"  /* Resection of Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0UT40ZZ"  =  "1"  /* Resection of Uterine Supporting Structure, Open Approach */
"0UT44ZZ"  =  "1"  /* Resection of Uterine Supporting Structure, Percutaneous Endoscopic Approach */
"0UT50ZZ"  =  "1"  /* Resection of Right Fallopian Tube, Open Approach */
"0UT54ZZ"  =  "1"  /* Resection of Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0UT60ZZ"  =  "1"  /* Resection of Left Fallopian Tube, Open Approach */
"0UT64ZZ"  =  "1"  /* Resection of Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UT70ZZ"  =  "1"  /* Resection of Bilateral Fallopian Tubes, Open Approach */
"0UT74ZZ"  =  "1"  /* Resection of Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0UT90ZZ"  =  "1"  /* Resection of Uterus, Open Approach */
"0UT94ZZ"  =  "1"  /* Resection of Uterus, Percutaneous Endoscopic Approach */
"0UTC0ZZ"  =  "1"  /* Resection of Cervix, Open Approach */
"0UTC4ZZ"  =  "1"  /* Resection of Cervix, Percutaneous Endoscopic Approach */
"0UTG0ZZ"  =  "1"  /* Resection of Vagina, Open Approach */
"0UU407Z"  =  "1"  /* Supplement Uterine Supporting Structure with Autologous Tissue Substitute, Open Approach */
"0UU40JZ"  =  "1"  /* Supplement Uterine Supporting Structure with Synthetic Substitute, Open Approach */
"0UU40KZ"  =  "1"  /* Supplement Uterine Supporting Structure with Nonautologous Tissue Substitute, Open Approach */
"0UU447Z"  =  "1"  /* Supplement Uterine Supporting Structure with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0UU44JZ"  =  "1"  /* Supplement Uterine Supporting Structure with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0UU44KZ"  =  "1"  /* Supplement Uterine Supporting Structure with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0UU507Z"  =  "1"  /* Supplement Right Fallopian Tube with Autologous Tissue Substitute, Open Approach */
"0UU50JZ"  =  "1"  /* Supplement Right Fallopian Tube with Synthetic Substitute, Open Approach */
"0UU50KZ"  =  "1"  /* Supplement Right Fallopian Tube with Nonautologous Tissue Substitute, Open Approach */
"0UU547Z"  =  "1"  /* Supplement Right Fallopian Tube with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0UU54JZ"  =  "1"  /* Supplement Right Fallopian Tube with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0UU54KZ"  =  "1"  /* Supplement Right Fallopian Tube with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0UU607Z"  =  "1"  /* Supplement Left Fallopian Tube with Autologous Tissue Substitute, Open Approach */
"0UU60JZ"  =  "1"  /* Supplement Left Fallopian Tube with Synthetic Substitute, Open Approach */
"0UU60KZ"  =  "1"  /* Supplement Left Fallopian Tube with Nonautologous Tissue Substitute, Open Approach */
"0UU647Z"  =  "1"  /* Supplement Left Fallopian Tube with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0UU64JZ"  =  "1"  /* Supplement Left Fallopian Tube with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0UU64KZ"  =  "1"  /* Supplement Left Fallopian Tube with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0UU707Z"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Autologous Tissue Substitute, Open Approach */
"0UU70JZ"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Synthetic Substitute, Open Approach */
"0UU70KZ"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Nonautologous Tissue Substitute, Open Approach */
"0UU747Z"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0UU74JZ"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0UU74KZ"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0UW300Z"  =  "1"  /* Revision of Drainage Device in Ovary, Open Approach */
"0UW303Z"  =  "1"  /* Revision of Infusion Device in Ovary, Open Approach */
"0UW330Z"  =  "1"  /* Revision of Drainage Device in Ovary, Percutaneous Approach */
"0UW333Z"  =  "1"  /* Revision of Infusion Device in Ovary, Percutaneous Approach */
"0UW340Z"  =  "1"  /* Revision of Drainage Device in Ovary, Percutaneous Endoscopic Approach */
"0UW343Z"  =  "1"  /* Revision of Infusion Device in Ovary, Percutaneous Endoscopic Approach */
"0UW800Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Open Approach */
"0UW803Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Open Approach */
"0UW807Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Fallopian Tube, Open Approach */
"0UW80CZ"  =  "1"  /* Revision of Extraluminal Device in Fallopian Tube, Open Approach */
"0UW80DZ"  =  "1"  /* Revision of Intraluminal Device in Fallopian Tube, Open Approach */
"0UW80JZ"  =  "1"  /* Revision of Synthetic Substitute in Fallopian Tube, Open Approach */
"0UW80KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Fallopian Tube, Open Approach */
"0UW830Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Percutaneous Approach */
"0UW833Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Percutaneous Approach */
"0UW837Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Fallopian Tube, Percutaneous Approach */
"0UW83CZ"  =  "1"  /* Revision of Extraluminal Device in Fallopian Tube, Percutaneous Approach */
"0UW83DZ"  =  "1"  /* Revision of Intraluminal Device in Fallopian Tube, Percutaneous Approach */
"0UW83JZ"  =  "1"  /* Revision of Synthetic Substitute in Fallopian Tube, Percutaneous Approach */
"0UW83KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Fallopian Tube, Percutaneous Approach */
"0UW840Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW843Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW847Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW84CZ"  =  "1"  /* Revision of Extraluminal Device in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW84DZ"  =  "1"  /* Revision of Intraluminal Device in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW84JZ"  =  "1"  /* Revision of Synthetic Substitute in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW84KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UWD00Z"  =  "1"  /* Revision of Drainage Device in Uterus and Cervix, Open Approach */
"0UWD01Z"  =  "1"  /* Revision of Radioactive Element in Uterus and Cervix, Open Approach */
"0UWD03Z"  =  "1"  /* Revision of Infusion Device in Uterus and Cervix, Open Approach */
"0UWD07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Uterus and Cervix, Open Approach */
"0UWD0DZ"  =  "1"  /* Revision of Intraluminal Device in Uterus and Cervix, Open Approach */
"0UWD0HZ"  =  "1"  /* Revision of Contraceptive Device in Uterus and Cervix, Open Approach */
"0UWD0JZ"  =  "1"  /* Revision of Synthetic Substitute in Uterus and Cervix, Open Approach */
"0UWD0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Uterus and Cervix, Open Approach */
"0UWD30Z"  =  "1"  /* Revision of Drainage Device in Uterus and Cervix, Percutaneous Approach */
"0UWD31Z"  =  "1"  /* Revision of Radioactive Element in Uterus and Cervix, Percutaneous Approach */
"0UWD33Z"  =  "1"  /* Revision of Infusion Device in Uterus and Cervix, Percutaneous Approach */
"0UWD37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Uterus and Cervix, Percutaneous Approach */
"0UWD3DZ"  =  "1"  /* Revision of Intraluminal Device in Uterus and Cervix, Percutaneous Approach */
"0UWD3HZ"  =  "1"  /* Revision of Contraceptive Device in Uterus and Cervix, Percutaneous Approach */
"0UWD3JZ"  =  "1"  /* Revision of Synthetic Substitute in Uterus and Cervix, Percutaneous Approach */
"0UWD3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Uterus and Cervix, Percutaneous Approach */
"0UWD40Z"  =  "1"  /* Revision of Drainage Device in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD41Z"  =  "1"  /* Revision of Radioactive Element in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD43Z"  =  "1"  /* Revision of Infusion Device in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD4DZ"  =  "1"  /* Revision of Intraluminal Device in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD4HZ"  =  "1"  /* Revision of Contraceptive Device in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD4JZ"  =  "1"  /* Revision of Synthetic Substitute in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UY00Z0"  =  "1"  /* Transplantation of Right Ovary, Allogeneic, Open Approach */
"0UY00Z1"  =  "1"  /* Transplantation of Right Ovary, Syngeneic, Open Approach */
"0UY00Z2"  =  "1"  /* Transplantation of Right Ovary, Zooplastic, Open Approach */
"0UY10Z0"  =  "1"  /* Transplantation of Left Ovary, Allogeneic, Open Approach */
"0UY10Z1"  =  "1"  /* Transplantation of Left Ovary, Syngeneic, Open Approach */
"0UY10Z2"  =  "1"  /* Transplantation of Left Ovary, Zooplastic, Open Approach */
"0W1G0J4"  =  "1"  /* Bypass Peritoneal Cavity to Cutaneous with Synthetic Substitute, Open Approach */
"0W1G0JY"  =  "1"  /* Bypass Peritoneal Cavity to Lower Vein with Synthetic Substitute, Open Approach */
"0W1G3J4"  =  "1"  /* Bypass Peritoneal Cavity to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0W1G4J4"  =  "1"  /* Bypass Peritoneal Cavity to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W1G4JY"  =  "1"  /* Bypass Peritoneal Cavity to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W3G0ZZ"  =  "1"  /* Control Bleeding in Peritoneal Cavity, Open Approach */
"0W3H0ZZ"  =  "1"  /* Control Bleeding in Retroperitoneum, Open Approach */
"0W3P0ZZ"  =  "1"  /* Control Bleeding in Gastrointestinal Tract, Open Approach */
"0W9F00Z"  =  "1"  /* Drainage of Abdominal Wall with Drainage Device, Open Approach */
"0W9F0ZX"  =  "1"  /* Drainage of Abdominal Wall, Open Approach, Diagnostic */
"0W9F0ZZ"  =  "1"  /* Drainage of Abdominal Wall, Open Approach */
"0W9F3ZX"  =  "1"  /* Drainage of Abdominal Wall, Percutaneous Approach, Diagnostic */
"0W9F4ZX"  =  "1"  /* Drainage of Abdominal Wall, Percutaneous Endoscopic Approach, Diagnostic */
"0W9G00Z"  =  "1"  /* Drainage of Peritoneal Cavity with Drainage Device, Open Approach */
"0W9G0ZX"  =  "1"  /* Drainage of Peritoneal Cavity, Open Approach, Diagnostic */
"0W9G0ZZ"  =  "1"  /* Drainage of Peritoneal Cavity, Open Approach */
"0W9G3ZX"  =  "1"  /* Drainage of Peritoneal Cavity, Percutaneous Approach, Diagnostic */
"0W9G4ZX"  =  "1"  /* Drainage of Peritoneal Cavity, Percutaneous Endoscopic Approach, Diagnostic */
"0W9H00Z"  =  "1"  /* Drainage of Retroperitoneum with Drainage Device, Open Approach */
"0W9H0ZX"  =  "1"  /* Drainage of Retroperitoneum, Open Approach, Diagnostic */
"0W9H0ZZ"  =  "1"  /* Drainage of Retroperitoneum, Open Approach */
"0W9H30Z"  =  "1"  /* Drainage of Retroperitoneum with Drainage Device, Percutaneous Approach */
"0W9H3ZX"  =  "1"  /* Drainage of Retroperitoneum, Percutaneous Approach, Diagnostic */
"0W9H3ZZ"  =  "1"  /* Drainage of Retroperitoneum, Percutaneous Approach */
"0W9H40Z"  =  "1"  /* Drainage of Retroperitoneum with Drainage Device, Percutaneous Endoscopic Approach */
"0W9H4ZX"  =  "1"  /* Drainage of Retroperitoneum, Percutaneous Endoscopic Approach, Diagnostic */
"0W9H4ZZ"  =  "1"  /* Drainage of Retroperitoneum, Percutaneous Endoscopic Approach */
"0W9J00Z"  =  "1"  /* Drainage of Pelvic Cavity with Drainage Device, Open Approach */
"0W9J0ZX"  =  "1"  /* Drainage of Pelvic Cavity, Open Approach, Diagnostic */
"0W9J0ZZ"  =  "1"  /* Drainage of Pelvic Cavity, Open Approach */
"0W9J3ZX"  =  "1"  /* Drainage of Pelvic Cavity, Percutaneous Approach, Diagnostic */
"0W9J40Z"  =  "1"  /* Drainage of Pelvic Cavity with Drainage Device, Percutaneous Endoscopic Approach */
"0W9J4ZX"  =  "1"  /* Drainage of Pelvic Cavity, Percutaneous Endoscopic Approach, Diagnostic */
"0W9J4ZZ"  =  "1"  /* Drainage of Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WBF0ZX"  =  "1"  /* Excision of Abdominal Wall, Open Approach, Diagnostic */
"0WBF0ZZ"  =  "1"  /* Excision of Abdominal Wall, Open Approach */
"0WBF3ZX"  =  "1"  /* Excision of Abdominal Wall, Percutaneous Approach, Diagnostic */
"0WBF3ZZ"  =  "1"  /* Excision of Abdominal Wall, Percutaneous Approach */
"0WBF4ZX"  =  "1"  /* Excision of Abdominal Wall, Percutaneous Endoscopic Approach, Diagnostic */
"0WBF4ZZ"  =  "1"  /* Excision of Abdominal Wall, Percutaneous Endoscopic Approach */
"0WBFXZ2"  =  "1"  /* Excision of Abdominal Wall, Stoma, External Approach */
"0WBFXZX"  =  "1"  /* Excision of Abdominal Wall, External Approach, Diagnostic */
"0WBFXZZ"  =  "1"  /* Excision of Abdominal Wall, External Approach */
"0WBH0ZX"  =  "1"  /* Excision of Retroperitoneum, Open Approach, Diagnostic */
"0WBH0ZZ"  =  "1"  /* Excision of Retroperitoneum, Open Approach */
"0WBH3ZZ"  =  "1"  /* Excision of Retroperitoneum, Percutaneous Approach */
"0WBH4ZZ"  =  "1"  /* Excision of Retroperitoneum, Percutaneous Endoscopic Approach */
"0WCG0ZZ"  =  "1"  /* Extirpation of Matter from Peritoneal Cavity, Open Approach */
"0WCG3ZZ"  =  "1"  /* Extirpation of Matter from Peritoneal Cavity, Percutaneous Approach */
"0WCG4ZZ"  =  "1"  /* Extirpation of Matter from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WCJ0ZZ"  =  "1"  /* Extirpation of Matter from Pelvic Cavity, Open Approach */
"0WCJ3ZZ"  =  "1"  /* Extirpation of Matter from Pelvic Cavity, Percutaneous Approach */
"0WCJ4ZZ"  =  "1"  /* Extirpation of Matter from Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WCP0ZZ"  =  "1"  /* Extirpation of Matter from Gastrointestinal Tract, Open Approach */
"0WCP3ZZ"  =  "1"  /* Extirpation of Matter from Gastrointestinal Tract, Percutaneous Approach */
"0WCP4ZZ"  =  "1"  /* Extirpation of Matter from Gastrointestinal Tract, Percutaneous Endoscopic Approach */
"0WCR0ZZ"  =  "1"  /* Extirpation of Matter from Genitourinary Tract, Open Approach */
"0WCR3ZZ"  =  "1"  /* Extirpation of Matter from Genitourinary Tract, Percutaneous Approach */
"0WCR4ZZ"  =  "1"  /* Extirpation of Matter from Genitourinary Tract, Percutaneous Endoscopic Approach */
"0WFG0ZZ"  =  "1"  /* Fragmentation in Peritoneal Cavity, Open Approach */
"0WFG3ZZ"  =  "1"  /* Fragmentation in Peritoneal Cavity, Percutaneous Approach */
"0WFG4ZZ"  =  "1"  /* Fragmentation in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WHF03Z"  =  "1"  /* Insertion of Infusion Device into Abdominal Wall, Open Approach */
"0WHF0YZ"  =  "1"  /* Insertion of Other Device into Abdominal Wall, Open Approach */
"0WHF33Z"  =  "1"  /* Insertion of Infusion Device into Abdominal Wall, Percutaneous Approach */
"0WHF3YZ"  =  "1"  /* Insertion of Other Device into Abdominal Wall, Percutaneous Approach */
"0WHF43Z"  =  "1"  /* Insertion of Infusion Device into Abdominal Wall, Percutaneous Endoscopic Approach */
"0WHF4YZ"  =  "1"  /* Insertion of Other Device into Abdominal Wall, Percutaneous Endoscopic Approach */
"0WHG03Z"  =  "1"  /* Insertion of Infusion Device into Peritoneal Cavity, Open Approach */
"0WHG0YZ"  =  "1"  /* Insertion of Other Device into Peritoneal Cavity, Open Approach */
"0WHG33Z"  =  "1"  /* Insertion of Infusion Device into Peritoneal Cavity, Percutaneous Approach */
"0WHG3YZ"  =  "1"  /* Insertion of Other Device into Peritoneal Cavity, Percutaneous Approach */
"0WHG43Z"  =  "1"  /* Insertion of Infusion Device into Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WHG4YZ"  =  "1"  /* Insertion of Other Device into Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WHH03Z"  =  "1"  /* Insertion of Infusion Device into Retroperitoneum, Open Approach */
"0WHH0YZ"  =  "1"  /* Insertion of Other Device into Retroperitoneum, Open Approach */
"0WHH33Z"  =  "1"  /* Insertion of Infusion Device into Retroperitoneum, Percutaneous Approach */
"0WHH3YZ"  =  "1"  /* Insertion of Other Device into Retroperitoneum, Percutaneous Approach */
"0WHH43Z"  =  "1"  /* Insertion of Infusion Device into Retroperitoneum, Percutaneous Endoscopic Approach */
"0WHH4YZ"  =  "1"  /* Insertion of Other Device into Retroperitoneum, Percutaneous Endoscopic Approach */
"0WHJ03Z"  =  "1"  /* Insertion of Infusion Device into Pelvic Cavity, Open Approach */
"0WHJ0YZ"  =  "1"  /* Insertion of Other Device into Pelvic Cavity, Open Approach */
"0WHJ33Z"  =  "1"  /* Insertion of Infusion Device into Pelvic Cavity, Percutaneous Approach */
"0WHJ3YZ"  =  "1"  /* Insertion of Other Device into Pelvic Cavity, Percutaneous Approach */
"0WHJ43Z"  =  "1"  /* Insertion of Infusion Device into Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WHJ4YZ"  =  "1"  /* Insertion of Other Device into Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WHP03Z"  =  "1"  /* Insertion of Infusion Device into Gastrointestinal Tract, Open Approach */
"0WJF0ZZ"  =  "1"  /* Inspection of Abdominal Wall, Open Approach */
"0WJF3ZZ"  =  "1"  /* Inspection of Abdominal Wall, Percutaneous Approach */
"0WJF4ZZ"  =  "1"  /* Inspection of Abdominal Wall, Percutaneous Endoscopic Approach */
"0WJG0ZZ"  =  "1"  /* Inspection of Peritoneal Cavity, Open Approach */
"0WJG3ZZ"  =  "1"  /* Inspection of Peritoneal Cavity, Percutaneous Approach */
"0WJG4ZZ"  =  "1"  /* Inspection of Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WJH0ZZ"  =  "1"  /* Inspection of Retroperitoneum, Open Approach */
"0WJH3ZZ"  =  "1"  /* Inspection of Retroperitoneum, Percutaneous Approach */
"0WJH4ZZ"  =  "1"  /* Inspection of Retroperitoneum, Percutaneous Endoscopic Approach */
"0WJJ0ZZ"  =  "1"  /* Inspection of Pelvic Cavity, Open Approach */
"0WJJ3ZZ"  =  "1"  /* Inspection of Pelvic Cavity, Percutaneous Approach */
"0WJJ4ZZ"  =  "1"  /* Inspection of Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WJP0ZZ"  =  "1"  /* Inspection of Gastrointestinal Tract, Open Approach */
"0WJP3ZZ"  =  "1"  /* Inspection of Gastrointestinal Tract, Percutaneous Approach */
"0WJP4ZZ"  =  "1"  /* Inspection of Gastrointestinal Tract, Percutaneous Endoscopic Approach */
"0WJP7ZZ"  =  "1"  /* Inspection of Gastrointestinal Tract, Via Natural or Artificial Opening Approach */
"0WJP8ZZ"  =  "1"  /* Inspection of Gastrointestinal Tract, Via Natural or Artificial Opening Endoscopic Approach */
"0WJR0ZZ"  =  "1"  /* Inspection of Genitourinary Tract, Open Approach */
"0WJR3ZZ"  =  "1"  /* Inspection of Genitourinary Tract, Percutaneous Approach */
"0WJR4ZZ"  =  "1"  /* Inspection of Genitourinary Tract, Percutaneous Endoscopic Approach */
"0WJR7ZZ"  =  "1"  /* Inspection of Genitourinary Tract, Via Natural or Artificial Opening Approach */
"0WJR8ZZ"  =  "1"  /* Inspection of Genitourinary Tract, Via Natural or Artificial Opening Endoscopic Approach */
"0WMF0ZZ"  =  "1"  /* Reattachment of Abdominal Wall, Open Approach */
"0WPF00Z"  =  "1"  /* Removal of Drainage Device from Abdominal Wall, Open Approach */
"0WPF01Z"  =  "1"  /* Removal of Radioactive Element from Abdominal Wall, Open Approach */
"0WPF03Z"  =  "1"  /* Removal of Infusion Device from Abdominal Wall, Open Approach */
"0WPF07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Abdominal Wall, Open Approach */
"0WPF0JZ"  =  "1"  /* Removal of Synthetic Substitute from Abdominal Wall, Open Approach */
"0WPF0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Abdominal Wall, Open Approach */
"0WPF0YZ"  =  "1"  /* Removal of Other Device from Abdominal Wall, Open Approach */
"0WPF30Z"  =  "1"  /* Removal of Drainage Device from Abdominal Wall, Percutaneous Approach */
"0WPF31Z"  =  "1"  /* Removal of Radioactive Element from Abdominal Wall, Percutaneous Approach */
"0WPF33Z"  =  "1"  /* Removal of Infusion Device from Abdominal Wall, Percutaneous Approach */
"0WPF37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Abdominal Wall, Percutaneous Approach */
"0WPF3JZ"  =  "1"  /* Removal of Synthetic Substitute from Abdominal Wall, Percutaneous Approach */
"0WPF3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Abdominal Wall, Percutaneous Approach */
"0WPF3YZ"  =  "1"  /* Removal of Other Device from Abdominal Wall, Percutaneous Approach */
"0WPF40Z"  =  "1"  /* Removal of Drainage Device from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF41Z"  =  "1"  /* Removal of Radioactive Element from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF43Z"  =  "1"  /* Removal of Infusion Device from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF4JZ"  =  "1"  /* Removal of Synthetic Substitute from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF4YZ"  =  "1"  /* Removal of Other Device from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPG00Z"  =  "1"  /* Removal of Drainage Device from Peritoneal Cavity, Open Approach */
"0WPG01Z"  =  "1"  /* Removal of Radioactive Element from Peritoneal Cavity, Open Approach */
"0WPG03Z"  =  "1"  /* Removal of Infusion Device from Peritoneal Cavity, Open Approach */
"0WPG0JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneal Cavity, Open Approach */
"0WPG0YZ"  =  "1"  /* Removal of Other Device from Peritoneal Cavity, Open Approach */
"0WPG30Z"  =  "1"  /* Removal of Drainage Device from Peritoneal Cavity, Percutaneous Approach */
"0WPG31Z"  =  "1"  /* Removal of Radioactive Element from Peritoneal Cavity, Percutaneous Approach */
"0WPG33Z"  =  "1"  /* Removal of Infusion Device from Peritoneal Cavity, Percutaneous Approach */
"0WPG3JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneal Cavity, Percutaneous Approach */
"0WPG3YZ"  =  "1"  /* Removal of Other Device from Peritoneal Cavity, Percutaneous Approach */
"0WPG40Z"  =  "1"  /* Removal of Drainage Device from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WPG41Z"  =  "1"  /* Removal of Radioactive Element from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WPG43Z"  =  "1"  /* Removal of Infusion Device from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WPG4JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WPG4YZ"  =  "1"  /* Removal of Other Device from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WPH00Z"  =  "1"  /* Removal of Drainage Device from Retroperitoneum, Open Approach */
"0WPH01Z"  =  "1"  /* Removal of Radioactive Element from Retroperitoneum, Open Approach */
"0WPH03Z"  =  "1"  /* Removal of Infusion Device from Retroperitoneum, Open Approach */
"0WPH0YZ"  =  "1"  /* Removal of Other Device from Retroperitoneum, Open Approach */
"0WPH30Z"  =  "1"  /* Removal of Drainage Device from Retroperitoneum, Percutaneous Approach */
"0WPH31Z"  =  "1"  /* Removal of Radioactive Element from Retroperitoneum, Percutaneous Approach */
"0WPH33Z"  =  "1"  /* Removal of Infusion Device from Retroperitoneum, Percutaneous Approach */
"0WPH3YZ"  =  "1"  /* Removal of Other Device from Retroperitoneum, Percutaneous Approach */
"0WPH40Z"  =  "1"  /* Removal of Drainage Device from Retroperitoneum, Percutaneous Endoscopic Approach */
"0WPH41Z"  =  "1"  /* Removal of Radioactive Element from Retroperitoneum, Percutaneous Endoscopic Approach */
"0WPH43Z"  =  "1"  /* Removal of Infusion Device from Retroperitoneum, Percutaneous Endoscopic Approach */
"0WPH4YZ"  =  "1"  /* Removal of Other Device from Retroperitoneum, Percutaneous Endoscopic Approach */
"0WPP01Z"  =  "1"  /* Removal of Radioactive Element from Gastrointestinal Tract, Open Approach */
"0WPP03Z"  =  "1"  /* Removal of Infusion Device from Gastrointestinal Tract, Open Approach */
"0WPP0YZ"  =  "1"  /* Removal of Other Device from Gastrointestinal Tract, Open Approach */
"0WQF0ZZ"  =  "1"  /* Repair Abdominal Wall, Open Approach */
"0WQF3ZZ"  =  "1"  /* Repair Abdominal Wall, Percutaneous Approach */
"0WQF4ZZ"  =  "1"  /* Repair Abdominal Wall, Percutaneous Endoscopic Approach */
"0WQFXZ2"  =  "1"  /* Repair Abdominal Wall, Stoma, External Approach */
"0WQFXZZ"  =  "1"  /* Repair Abdominal Wall, External Approach */
"0WUF07Z"  =  "1"  /* Supplement Abdominal Wall with Autologous Tissue Substitute, Open Approach */
"0WUF0JZ"  =  "1"  /* Supplement Abdominal Wall with Synthetic Substitute, Open Approach */
"0WUF0KZ"  =  "1"  /* Supplement Abdominal Wall with Nonautologous Tissue Substitute, Open Approach */
"0WUF47Z"  =  "1"  /* Supplement Abdominal Wall with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0WUF4JZ"  =  "1"  /* Supplement Abdominal Wall with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0WUF4KZ"  =  "1"  /* Supplement Abdominal Wall with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0WWF00Z"  =  "1"  /* Revision of Drainage Device in Abdominal Wall, Open Approach */
"0WWF01Z"  =  "1"  /* Revision of Radioactive Element in Abdominal Wall, Open Approach */
"0WWF03Z"  =  "1"  /* Revision of Infusion Device in Abdominal Wall, Open Approach */
"0WWF07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Abdominal Wall, Open Approach */
"0WWF0JZ"  =  "1"  /* Revision of Synthetic Substitute in Abdominal Wall, Open Approach */
"0WWF0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Abdominal Wall, Open Approach */
"0WWF0YZ"  =  "1"  /* Revision of Other Device in Abdominal Wall, Open Approach */
"0WWF30Z"  =  "1"  /* Revision of Drainage Device in Abdominal Wall, Percutaneous Approach */
"0WWF31Z"  =  "1"  /* Revision of Radioactive Element in Abdominal Wall, Percutaneous Approach */
"0WWF33Z"  =  "1"  /* Revision of Infusion Device in Abdominal Wall, Percutaneous Approach */
"0WWF37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Abdominal Wall, Percutaneous Approach */
"0WWF3JZ"  =  "1"  /* Revision of Synthetic Substitute in Abdominal Wall, Percutaneous Approach */
"0WWF3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Abdominal Wall, Percutaneous Approach */
"0WWF3YZ"  =  "1"  /* Revision of Other Device in Abdominal Wall, Percutaneous Approach */
"0WWF40Z"  =  "1"  /* Revision of Drainage Device in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF41Z"  =  "1"  /* Revision of Radioactive Element in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF43Z"  =  "1"  /* Revision of Infusion Device in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF4JZ"  =  "1"  /* Revision of Synthetic Substitute in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF4YZ"  =  "1"  /* Revision of Other Device in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWG00Z"  =  "1"  /* Revision of Drainage Device in Peritoneal Cavity, Open Approach */
"0WWG01Z"  =  "1"  /* Revision of Radioactive Element in Peritoneal Cavity, Open Approach */
"0WWG03Z"  =  "1"  /* Revision of Infusion Device in Peritoneal Cavity, Open Approach */
"0WWG0JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneal Cavity, Open Approach */
"0WWG0YZ"  =  "1"  /* Revision of Other Device in Peritoneal Cavity, Open Approach */
"0WWG30Z"  =  "1"  /* Revision of Drainage Device in Peritoneal Cavity, Percutaneous Approach */
"0WWG31Z"  =  "1"  /* Revision of Radioactive Element in Peritoneal Cavity, Percutaneous Approach */
"0WWG33Z"  =  "1"  /* Revision of Infusion Device in Peritoneal Cavity, Percutaneous Approach */
"0WWG3JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneal Cavity, Percutaneous Approach */
"0WWG3YZ"  =  "1"  /* Revision of Other Device in Peritoneal Cavity, Percutaneous Approach */
"0WWG40Z"  =  "1"  /* Revision of Drainage Device in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WWG41Z"  =  "1"  /* Revision of Radioactive Element in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WWG43Z"  =  "1"  /* Revision of Infusion Device in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WWG4JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WWG4YZ"  =  "1"  /* Revision of Other Device in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WWH00Z"  =  "1"  /* Revision of Drainage Device in Retroperitoneum, Open Approach */
"0WWH01Z"  =  "1"  /* Revision of Radioactive Element in Retroperitoneum, Open Approach */
"0WWH03Z"  =  "1"  /* Revision of Infusion Device in Retroperitoneum, Open Approach */
"0WWH0YZ"  =  "1"  /* Revision of Other Device in Retroperitoneum, Open Approach */
"0WWH30Z"  =  "1"  /* Revision of Drainage Device in Retroperitoneum, Percutaneous Approach */
"0WWH31Z"  =  "1"  /* Revision of Radioactive Element in Retroperitoneum, Percutaneous Approach */
"0WWH33Z"  =  "1"  /* Revision of Infusion Device in Retroperitoneum, Percutaneous Approach */
"0WWH3YZ"  =  "1"  /* Revision of Other Device in Retroperitoneum, Percutaneous Approach */
"0WWH40Z"  =  "1"  /* Revision of Drainage Device in Retroperitoneum, Percutaneous Endoscopic Approach */
"0WWH41Z"  =  "1"  /* Revision of Radioactive Element in Retroperitoneum, Percutaneous Endoscopic Approach */
"0WWH43Z"  =  "1"  /* Revision of Infusion Device in Retroperitoneum, Percutaneous Endoscopic Approach */
"0WWH4YZ"  =  "1"  /* Revision of Other Device in Retroperitoneum, Percutaneous Endoscopic Approach */
"0WWJ00Z"  =  "1"  /* Revision of Drainage Device in Pelvic Cavity, Open Approach */
"0WWJ01Z"  =  "1"  /* Revision of Radioactive Element in Pelvic Cavity, Open Approach */
"0WWJ03Z"  =  "1"  /* Revision of Infusion Device in Pelvic Cavity, Open Approach */
"0WWJ0JZ"  =  "1"  /* Revision of Synthetic Substitute in Pelvic Cavity, Open Approach */
"0WWJ0YZ"  =  "1"  /* Revision of Other Device in Pelvic Cavity, Open Approach */
"0WWJ30Z"  =  "1"  /* Revision of Drainage Device in Pelvic Cavity, Percutaneous Approach */
"0WWJ31Z"  =  "1"  /* Revision of Radioactive Element in Pelvic Cavity, Percutaneous Approach */
"0WWJ33Z"  =  "1"  /* Revision of Infusion Device in Pelvic Cavity, Percutaneous Approach */
"0WWJ3JZ"  =  "1"  /* Revision of Synthetic Substitute in Pelvic Cavity, Percutaneous Approach */
"0WWJ3YZ"  =  "1"  /* Revision of Other Device in Pelvic Cavity, Percutaneous Approach */
"0WWJ40Z"  =  "1"  /* Revision of Drainage Device in Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WWJ41Z"  =  "1"  /* Revision of Radioactive Element in Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WWJ43Z"  =  "1"  /* Revision of Infusion Device in Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WWJ4JZ"  =  "1"  /* Revision of Synthetic Substitute in Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WWJ4YZ"  =  "1"  /* Revision of Other Device in Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WWP01Z"  =  "1"  /* Revision of Radioactive Element in Gastrointestinal Tract, Open Approach */
"0WWP03Z"  =  "1"  /* Revision of Infusion Device in Gastrointestinal Tract, Open Approach */
"0WWP0YZ"  =  "1"  /* Revision of Other Device in Gastrointestinal Tract, Open Approach */
"0Y9500Z"  =  "1"  /* Drainage of Right Inguinal Region with Drainage Device, Open Approach */
"0Y950ZX"  =  "1"  /* Drainage of Right Inguinal Region, Open Approach, Diagnostic */
"0Y950ZZ"  =  "1"  /* Drainage of Right Inguinal Region, Open Approach */
"0Y9530Z"  =  "1"  /* Drainage of Right Inguinal Region with Drainage Device, Percutaneous Approach */
"0Y953ZX"  =  "1"  /* Drainage of Right Inguinal Region, Percutaneous Approach, Diagnostic */
"0Y953ZZ"  =  "1"  /* Drainage of Right Inguinal Region, Percutaneous Approach */
"0Y9540Z"  =  "1"  /* Drainage of Right Inguinal Region with Drainage Device, Percutaneous Endoscopic Approach */
"0Y954ZX"  =  "1"  /* Drainage of Right Inguinal Region, Percutaneous Endoscopic Approach, Diagnostic */
"0Y954ZZ"  =  "1"  /* Drainage of Right Inguinal Region, Percutaneous Endoscopic Approach */
"0Y9600Z"  =  "1"  /* Drainage of Left Inguinal Region with Drainage Device, Open Approach */
"0Y960ZX"  =  "1"  /* Drainage of Left Inguinal Region, Open Approach, Diagnostic */
"0Y960ZZ"  =  "1"  /* Drainage of Left Inguinal Region, Open Approach */
"0Y9630Z"  =  "1"  /* Drainage of Left Inguinal Region with Drainage Device, Percutaneous Approach */
"0Y963ZX"  =  "1"  /* Drainage of Left Inguinal Region, Percutaneous Approach, Diagnostic */
"0Y963ZZ"  =  "1"  /* Drainage of Left Inguinal Region, Percutaneous Approach */
"0Y9640Z"  =  "1"  /* Drainage of Left Inguinal Region with Drainage Device, Percutaneous Endoscopic Approach */
"0Y964ZX"  =  "1"  /* Drainage of Left Inguinal Region, Percutaneous Endoscopic Approach, Diagnostic */
"0Y964ZZ"  =  "1"  /* Drainage of Left Inguinal Region, Percutaneous Endoscopic Approach */
"0YB50ZX"  =  "1"  /* Excision of Right Inguinal Region, Open Approach, Diagnostic */
"0YB50ZZ"  =  "1"  /* Excision of Right Inguinal Region, Open Approach */
"0YB53ZX"  =  "1"  /* Excision of Right Inguinal Region, Percutaneous Approach, Diagnostic */
"0YB53ZZ"  =  "1"  /* Excision of Right Inguinal Region, Percutaneous Approach */
"0YB54ZX"  =  "1"  /* Excision of Right Inguinal Region, Percutaneous Endoscopic Approach, Diagnostic */
"0YB54ZZ"  =  "1"  /* Excision of Right Inguinal Region, Percutaneous Endoscopic Approach */
"0YB60ZX"  =  "1"  /* Excision of Left Inguinal Region, Open Approach, Diagnostic */
"0YB60ZZ"  =  "1"  /* Excision of Left Inguinal Region, Open Approach */
"0YB63ZX"  =  "1"  /* Excision of Left Inguinal Region, Percutaneous Approach, Diagnostic */
"0YB63ZZ"  =  "1"  /* Excision of Left Inguinal Region, Percutaneous Approach */
"0YB64ZX"  =  "1"  /* Excision of Left Inguinal Region, Percutaneous Endoscopic Approach, Diagnostic */
"0YB64ZZ"  =  "1"  /* Excision of Left Inguinal Region, Percutaneous Endoscopic Approach */
"0YB70ZX"  =  "1"  /* Excision of Right Femoral Region, Open Approach, Diagnostic */
"0YB70ZZ"  =  "1"  /* Excision of Right Femoral Region, Open Approach */
"0YB73ZX"  =  "1"  /* Excision of Right Femoral Region, Percutaneous Approach, Diagnostic */
"0YB73ZZ"  =  "1"  /* Excision of Right Femoral Region, Percutaneous Approach */
"0YB74ZX"  =  "1"  /* Excision of Right Femoral Region, Percutaneous Endoscopic Approach, Diagnostic */
"0YB74ZZ"  =  "1"  /* Excision of Right Femoral Region, Percutaneous Endoscopic Approach */
"0YB80ZX"  =  "1"  /* Excision of Left Femoral Region, Open Approach, Diagnostic */
"0YB80ZZ"  =  "1"  /* Excision of Left Femoral Region, Open Approach */
"0YB83ZX"  =  "1"  /* Excision of Left Femoral Region, Percutaneous Approach, Diagnostic */
"0YB83ZZ"  =  "1"  /* Excision of Left Femoral Region, Percutaneous Approach */
"0YB84ZX"  =  "1"  /* Excision of Left Femoral Region, Percutaneous Endoscopic Approach, Diagnostic */
"0YB84ZZ"  =  "1"  /* Excision of Left Femoral Region, Percutaneous Endoscopic Approach */
"0YJ50ZZ"  =  "1"  /* Inspection of Right Inguinal Region, Open Approach */
"0YJ53ZZ"  =  "1"  /* Inspection of Right Inguinal Region, Percutaneous Approach */
"0YJ54ZZ"  =  "1"  /* Inspection of Right Inguinal Region, Percutaneous Endoscopic Approach */
"0YJ60ZZ"  =  "1"  /* Inspection of Left Inguinal Region, Open Approach */
"0YJ63ZZ"  =  "1"  /* Inspection of Left Inguinal Region, Percutaneous Approach */
"0YJ64ZZ"  =  "1"  /* Inspection of Left Inguinal Region, Percutaneous Endoscopic Approach */
"0YJ70ZZ"  =  "1"  /* Inspection of Right Femoral Region, Open Approach */
"0YJ73ZZ"  =  "1"  /* Inspection of Right Femoral Region, Percutaneous Approach */
"0YJ74ZZ"  =  "1"  /* Inspection of Right Femoral Region, Percutaneous Endoscopic Approach */
"0YJ83ZZ"  =  "1"  /* Inspection of Left Femoral Region, Percutaneous Approach */
"0YJ84ZZ"  =  "1"  /* Inspection of Left Femoral Region, Percutaneous Endoscopic Approach */
"0YJA0ZZ"  =  "1"  /* Inspection of Bilateral Inguinal Region, Open Approach */
"0YJA3ZZ"  =  "1"  /* Inspection of Bilateral Inguinal Region, Percutaneous Approach */
"0YJA4ZZ"  =  "1"  /* Inspection of Bilateral Inguinal Region, Percutaneous Endoscopic Approach */
"0YJE3ZZ"  =  "1"  /* Inspection of Bilateral Femoral Region, Percutaneous Approach */
"0YJE4ZZ"  =  "1"  /* Inspection of Bilateral Femoral Region, Percutaneous Endoscopic Approach */
"0YQ50ZZ"  =  "1"  /* Repair Right Inguinal Region, Open Approach */
"0YQ53ZZ"  =  "1"  /* Repair Right Inguinal Region, Percutaneous Approach */
"0YQ54ZZ"  =  "1"  /* Repair Right Inguinal Region, Percutaneous Endoscopic Approach */
"0YQ60ZZ"  =  "1"  /* Repair Left Inguinal Region, Open Approach */
"0YQ63ZZ"  =  "1"  /* Repair Left Inguinal Region, Percutaneous Approach */
"0YQ64ZZ"  =  "1"  /* Repair Left Inguinal Region, Percutaneous Endoscopic Approach */
"0YQ70ZZ"  =  "1"  /* Repair Right Femoral Region, Open Approach */
"0YQ73ZZ"  =  "1"  /* Repair Right Femoral Region, Percutaneous Approach */
"0YQ74ZZ"  =  "1"  /* Repair Right Femoral Region, Percutaneous Endoscopic Approach */
"0YQ80ZZ"  =  "1"  /* Repair Left Femoral Region, Open Approach */
"0YQ83ZZ"  =  "1"  /* Repair Left Femoral Region, Percutaneous Approach */
"0YQ84ZZ"  =  "1"  /* Repair Left Femoral Region, Percutaneous Endoscopic Approach */
"0YQA0ZZ"  =  "1"  /* Repair Bilateral Inguinal Region, Open Approach */
"0YQA3ZZ"  =  "1"  /* Repair Bilateral Inguinal Region, Percutaneous Approach */
"0YQA4ZZ"  =  "1"  /* Repair Bilateral Inguinal Region, Percutaneous Endoscopic Approach */
"0YQE0ZZ"  =  "1"  /* Repair Bilateral Femoral Region, Open Approach */
"0YQE3ZZ"  =  "1"  /* Repair Bilateral Femoral Region, Percutaneous Approach */
"0YQE4ZZ"  =  "1"  /* Repair Bilateral Femoral Region, Percutaneous Endoscopic Approach */
"0YU507Z"  =  "1"  /* Supplement Right Inguinal Region with Autologous Tissue Substitute, Open Approach */
"0YU50JZ"  =  "1"  /* Supplement Right Inguinal Region with Synthetic Substitute, Open Approach */
"0YU50KZ"  =  "1"  /* Supplement Right Inguinal Region with Nonautologous Tissue Substitute, Open Approach */
"0YU547Z"  =  "1"  /* Supplement Right Inguinal Region with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0YU54JZ"  =  "1"  /* Supplement Right Inguinal Region with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0YU54KZ"  =  "1"  /* Supplement Right Inguinal Region with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0YU607Z"  =  "1"  /* Supplement Left Inguinal Region with Autologous Tissue Substitute, Open Approach */
"0YU60JZ"  =  "1"  /* Supplement Left Inguinal Region with Synthetic Substitute, Open Approach */
"0YU60KZ"  =  "1"  /* Supplement Left Inguinal Region with Nonautologous Tissue Substitute, Open Approach */
"0YU647Z"  =  "1"  /* Supplement Left Inguinal Region with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0YU64JZ"  =  "1"  /* Supplement Left Inguinal Region with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0YU64KZ"  =  "1"  /* Supplement Left Inguinal Region with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0YU707Z"  =  "1"  /* Supplement Right Femoral Region with Autologous Tissue Substitute, Open Approach */
"0YU70JZ"  =  "1"  /* Supplement Right Femoral Region with Synthetic Substitute, Open Approach */
"0YU70KZ"  =  "1"  /* Supplement Right Femoral Region with Nonautologous Tissue Substitute, Open Approach */
"0YU747Z"  =  "1"  /* Supplement Right Femoral Region with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0YU74JZ"  =  "1"  /* Supplement Right Femoral Region with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0YU74KZ"  =  "1"  /* Supplement Right Femoral Region with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0YU807Z"  =  "1"  /* Supplement Left Femoral Region with Autologous Tissue Substitute, Open Approach */
"0YU80JZ"  =  "1"  /* Supplement Left Femoral Region with Synthetic Substitute, Open Approach */
"0YU80KZ"  =  "1"  /* Supplement Left Femoral Region with Nonautologous Tissue Substitute, Open Approach */
"0YU847Z"  =  "1"  /* Supplement Left Femoral Region with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0YU84JZ"  =  "1"  /* Supplement Left Femoral Region with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0YU84KZ"  =  "1"  /* Supplement Left Femoral Region with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0YUA07Z"  =  "1"  /* Supplement Bilateral Inguinal Region with Autologous Tissue Substitute, Open Approach */
"0YUA0JZ"  =  "1"  /* Supplement Bilateral Inguinal Region with Synthetic Substitute, Open Approach */
"0YUA0KZ"  =  "1"  /* Supplement Bilateral Inguinal Region with Nonautologous Tissue Substitute, Open Approach */
"0YUA47Z"  =  "1"  /* Supplement Bilateral Inguinal Region with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0YUA4JZ"  =  "1"  /* Supplement Bilateral Inguinal Region with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0YUA4KZ"  =  "1"  /* Supplement Bilateral Inguinal Region with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0YUE07Z"  =  "1"  /* Supplement Bilateral Femoral Region with Autologous Tissue Substitute, Open Approach */
"0YUE0JZ"  =  "1"  /* Supplement Bilateral Femoral Region with Synthetic Substitute, Open Approach */
"0YUE0KZ"  =  "1"  /* Supplement Bilateral Femoral Region with Nonautologous Tissue Substitute, Open Approach */
"0YUE47Z"  =  "1"  /* Supplement Bilateral Femoral Region with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0YUE4JZ"  =  "1"  /* Supplement Bilateral Femoral Region with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0YUE4KZ"  =  "1"  /* Supplement Bilateral Femoral Region with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"10T20ZZ"  =  "1"  /* Resection of Products of Conception, Ectopic, Open Approach */
"10T23ZZ"  =  "1"  /* Resection of Products of Conception, Ectopic, Percutaneous Approach */
"10T24ZZ"  =  "1"  /* Resection of Products of Conception, Ectopic, Percutaneous Endoscopic Approach */
"4A0C35Z"  =  "1"  /* Measurement of Biliary Flow, Percutaneous Approach */
"4A0C3BZ"  =  "1"  /* Measurement of Biliary Pressure, Percutaneous Approach */
"4A0C45Z"  =  "1"  /* Measurement of Biliary Flow, Percutaneous Endoscopic Approach */
"4A0C4BZ"  =  "1"  /* Measurement of Biliary Pressure, Percutaneous Endoscopic Approach */
"4A0C75Z"  =  "1"  /* Measurement of Biliary Flow, Via Natural or Artificial Opening */
"4A0C7BZ"  =  "1"  /* Measurement of Biliary Pressure, Via Natural or Artificial Opening */
"4A0C85Z"  =  "1"  /* Measurement of Biliary Flow, Via Natural or Artificial Opening Endoscopic */
OTHER  = '0' ;

 /* Exclude: Colectomy or Pelvic Evisceration */
 /* ICD-10-CM Procedure Code: */

VALUE $PRAPP3P
"0DBE0ZZ"  =  "1"  /* Excision of Large Intestine, Open Approach */
"0DBE3ZZ"  =  "1"  /* Excision of Large Intestine, Percutaneous Approach */
"0DBE4ZZ"  =  "1"  /* Excision of Large Intestine, Percutaneous Endoscopic Approach */
"0DBE7ZZ"  =  "1"  /* Excision of Large Intestine, Via Natural or Artificial Opening */
"0DBE8ZZ"  =  "1"  /* Excision of Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DBF0ZZ"  =  "1"  /* Excision of Right Large Intestine, Open Approach */
"0DBF3ZZ"  =  "1"  /* Excision of Right Large Intestine, Percutaneous Approach */
"0DBF4ZZ"  =  "1"  /* Excision of Right Large Intestine, Percutaneous Endoscopic Approach */
"0DBF7ZZ"  =  "1"  /* Excision of Right Large Intestine, Via Natural or Artificial Opening */
"0DBF8ZZ"  =  "1"  /* Excision of Right Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DBG0ZZ"  =  "1"  /* Excision of Left Large Intestine, Open Approach */
"0DBG3ZZ"  =  "1"  /* Excision of Left Large Intestine, Percutaneous Approach */
"0DBG4ZZ"  =  "1"  /* Excision of Left Large Intestine, Percutaneous Endoscopic Approach */
"0DBG7ZZ"  =  "1"  /* Excision of Left Large Intestine, Via Natural or Artificial Opening */
"0DBG8ZZ"  =  "1"  /* Excision of Left Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DBH0ZZ"  =  "1"  /* Excision of Cecum, Open Approach */
"0DBH3ZZ"  =  "1"  /* Excision of Cecum, Percutaneous Approach */
"0DBH4ZZ"  =  "1"  /* Excision of Cecum, Percutaneous Endoscopic Approach */
"0DBH7ZZ"  =  "1"  /* Excision of Cecum, Via Natural or Artificial Opening */
"0DBH8ZZ"  =  "1"  /* Excision of Cecum, Via Natural or Artificial Opening Endoscopic */
"0DBK0ZZ"  =  "1"  /* Excision of Ascending Colon, Open Approach */
"0DBK3ZZ"  =  "1"  /* Excision of Ascending Colon, Percutaneous Approach */
"0DBK4ZZ"  =  "1"  /* Excision of Ascending Colon, Percutaneous Endoscopic Approach */
"0DBK7ZZ"  =  "1"  /* Excision of Ascending Colon, Via Natural or Artificial Opening */
"0DBK8ZZ"  =  "1"  /* Excision of Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0DBL0ZZ"  =  "1"  /* Excision of Transverse Colon, Open Approach */
"0DBL3ZZ"  =  "1"  /* Excision of Transverse Colon, Percutaneous Approach */
"0DBL4ZZ"  =  "1"  /* Excision of Transverse Colon, Percutaneous Endoscopic Approach */
"0DBL7ZZ"  =  "1"  /* Excision of Transverse Colon, Via Natural or Artificial Opening */
"0DBL8ZZ"  =  "1"  /* Excision of Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0DBM0ZZ"  =  "1"  /* Excision of Descending Colon, Open Approach */
"0DBM3ZZ"  =  "1"  /* Excision of Descending Colon, Percutaneous Approach */
"0DBM4ZZ"  =  "1"  /* Excision of Descending Colon, Percutaneous Endoscopic Approach */
"0DBM7ZZ"  =  "1"  /* Excision of Descending Colon, Via Natural or Artificial Opening */
"0DBM8ZZ"  =  "1"  /* Excision of Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0DBN0ZZ"  =  "1"  /* Excision of Sigmoid Colon, Open Approach */
"0DBN3ZZ"  =  "1"  /* Excision of Sigmoid Colon, Percutaneous Approach */
"0DBN4ZZ"  =  "1"  /* Excision of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DBN7ZZ"  =  "1"  /* Excision of Sigmoid Colon, Via Natural or Artificial Opening */
"0DBN8ZZ"  =  "1"  /* Excision of Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0DTE0ZZ"  =  "1"  /* Resection of Large Intestine, Open Approach */
"0DTE4ZZ"  =  "1"  /* Resection of Large Intestine, Percutaneous Endoscopic Approach */
"0DTE7ZZ"  =  "1"  /* Resection of Large Intestine, Via Natural or Artificial Opening */
"0DTE8ZZ"  =  "1"  /* Resection of Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DTF0ZZ"  =  "1"  /* Resection of Right Large Intestine, Open Approach */
"0DTF4ZZ"  =  "1"  /* Resection of Right Large Intestine, Percutaneous Endoscopic Approach */
"0DTF7ZZ"  =  "1"  /* Resection of Right Large Intestine, Via Natural or Artificial Opening */
"0DTF8ZZ"  =  "1"  /* Resection of Right Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DTG0ZZ"  =  "1"  /* Resection of Left Large Intestine, Open Approach */
"0DTG4ZZ"  =  "1"  /* Resection of Left Large Intestine, Percutaneous Endoscopic Approach */
"0DTG7ZZ"  =  "1"  /* Resection of Left Large Intestine, Via Natural or Artificial Opening */
"0DTG8ZZ"  =  "1"  /* Resection of Left Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DTH0ZZ"  =  "1"  /* Resection of Cecum, Open Approach */
"0DTH4ZZ"  =  "1"  /* Resection of Cecum, Percutaneous Endoscopic Approach */
"0DTH7ZZ"  =  "1"  /* Resection of Cecum, Via Natural or Artificial Opening */
"0DTH8ZZ"  =  "1"  /* Resection of Cecum, Via Natural or Artificial Opening Endoscopic */
"0DTK0ZZ"  =  "1"  /* Resection of Ascending Colon, Open Approach */
"0DTK4ZZ"  =  "1"  /* Resection of Ascending Colon, Percutaneous Endoscopic Approach */
"0DTK7ZZ"  =  "1"  /* Resection of Ascending Colon, Via Natural or Artificial Opening */
"0DTK8ZZ"  =  "1"  /* Resection of Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0DTL0ZZ"  =  "1"  /* Resection of Transverse Colon, Open Approach */
"0DTL4ZZ"  =  "1"  /* Resection of Transverse Colon, Percutaneous Endoscopic Approach */
"0DTL7ZZ"  =  "1"  /* Resection of Transverse Colon, Via Natural or Artificial Opening */
"0DTL8ZZ"  =  "1"  /* Resection of Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0DTM0ZZ"  =  "1"  /* Resection of Descending Colon, Open Approach */
"0DTM4ZZ"  =  "1"  /* Resection of Descending Colon, Percutaneous Endoscopic Approach */
"0DTM7ZZ"  =  "1"  /* Resection of Descending Colon, Via Natural or Artificial Opening */
"0DTM8ZZ"  =  "1"  /* Resection of Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0DTN0ZZ"  =  "1"  /* Resection of Sigmoid Colon, Open Approach */
"0DTN4ZZ"  =  "1"  /* Resection of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DTN7ZZ"  =  "1"  /* Resection of Sigmoid Colon, Via Natural or Artificial Opening */
"0DTN8ZZ"  =  "1"  /* Resection of Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
OTHER  = '0' ;

 /* Exclude: Cancer adj to appendix */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $PRAPPND
"C180"  =  "1"  /* Malignant neoplasm of cecum */
"C181"  =  "1"  /* Malignant neoplasm of appendix */
"C182"  =  "1"  /* Malignant neoplasm of ascending colon */
"C188"  =  "1"  /* Malignant neoplasm of overlapping sites of colon */
"C189"  =  "1"  /* Malignant neoplasm of colon, unspecified */
"C260"  =  "1"  /* Malignant neoplasm of intestinal tract, part unspecified */
"C269"  =  "1"  /* Malignant neoplasm of ill-defined sites within the digestive system */
"C451"  =  "1"  /* Mesothelioma of peritoneum */
"C481"  =  "1"  /* Malignant neoplasm of specified parts of peritoneum */
"C482"  =  "1"  /* Malignant neoplasm of peritoneum, unspecified */
"C488"  =  "1"  /* Malignant neoplasm of overlapping sites of retroperitoneum and peritoneum */
"C762"  =  "1"  /* Malignant neoplasm of abdomen */
"C785"  =  "1"  /* Secondary malignant neoplasm of large intestine and rectum */
"C786"  =  "1"  /* Secondary malignant neoplasm of retroperitoneum and peritoneum */
"C7B04"  =  "1"  /* Secondary carcinoid tumors of peritoneum */
OTHER  = '0' ;

 /* Intra-Abdominal Procedure (Population at Risk): */
 /* DRGs: */

VALUE PRAPPNG
146 = '1'     /* RECTAL RESECTION W CC */
147 = '1'     /* RECTAL RESECTION W/O CC */
148 = '1'     /* MAJ BOWEL PROC W CC */
149 = '1'     /* MAJ BOWEL PROC W/0 CC */
150 = '1'     /* PERITONEAL ADHES W CC */
151 = '1'     /* PERITONEAL ADHES W/O CC */
152 = '1'     /* MIN BOWEL PROC W CC */
153 = '1'     /* MIN BOWEL PROC W/O CC */
154 = '1'     /* UGI PROC AGE >17 W CC */
155 = '1'     /* UGI PROC AGE >17 W/O CC */
170 = '1'     /* OTH GI OR PROC W CC */
171 = '1'     /* OTH GI OR PROC W/O CC */
191 = '1'     /* PANC LVR SHNT PRC W CC */
192 = '1'     /* PANC LVR SHNT PRC W/O CC */
193 = '1'     /* BILIARY PROC W/ CC */
194 = '1'     /* BILIARY PROC W/O CC */
195 = '1'     /* CHOLE W/ CDE W/ CC */
196 = '1'     /* CHOLE W/ CDE W/O CC */
197 = '1'     /* CHOLE W/ CC */
198 = '1'     /* CHOLE W/O CC */
201 = '1'     /* OTH BILIARY/PANC PROC */
354 = '1'     /* UTER PROC MALIG W/ CC */
355 = '1'     /* UTER PROC MALIG W/O CC */
356 = '1'     /* FEMALE REPROD RECONSTR */
357 = '1'     /* UTER PROC OVARIAN MALIG */
358 = '1'     /* UTER PROC NONMALIG W/ CC */
359 = '1'     /* UTER PROC NONMALI W/O CC */
365 = '1'     /* OTH FEMAL REPROD PROC */
567 = '1'     /* STOMACH, ESOPHAGEAL & DUODENAL PROC AGE > 17 W CC W MAJOR GI DX */
568 = '1'     /* STOMACH, ESOPHAGEAL & DUODENAL PROCEDURES PROC AGE > 17 W CC W/O MAJOR GI DX */
569 = '1'     /* MAJOR SMALL & LARGE BOWEL PROCEDURES W CC W MAJOR GI DX */
570 = '1'     /* MAJOR SMALL & LARGE BOWEL PROCEDURES W CC W/O MAJOR GI DX */
OTHER = '0' ;

 /* -------------------------------------------- */
 /* Bi-lateral Catheterization rate (PRCATH)     */
 /*   Utilization (provider) Indicator / IQI #25 */
 /* -------------------------------------------- */

 /* Bi-lateral Catheterization (Outcome of Interest): */
 /* ICD-10-CM Procedure Code: */

VALUE $PRCATHP
"4A023N8"  =  "1"  /* Measurement of Cardiac Sampling and Pressure, Bilateral, Percutaneous Approach */
OTHER  = '0' ;

 /*  Catheterization (Population at Risk) */
 /* ICD-10-CM Procedure Code: */

VALUE $PRCAT2P
"4A023N7"  =  "1"  /* Measurement of Cardiac Sampling and Pressure, Left Heart, Percutaneous Approach */
"4A023N8"  =  "1"  /* Measurement of Cardiac Sampling and Pressure, Bilateral, Percutaneous Approach */
OTHER  = '0' ;

 /* Include only: Coronary artery disease */
 /* ICD-10-CM Diagnosis Code: */

VALUE $PRCATHD
"I200"  =  "1"  /* Unstable angina */
"I201"  =  "1"  /* Angina pectoris with documented spasm */
"I208"  =  "1"  /* Other forms of angina pectoris */
"I209"  =  "1"  /* Angina pectoris, unspecified */
"I2101"  =  "1"  /* ST elevation (STEMI) myocardial infarction involving left main coronary artery */
"I2102"  =  "1"  /* ST elevation (STEMI) myocardial infarction involving left anterior descending coronary artery */
"I2109"  =  "1"  /* ST elevation (STEMI) myocardial infarction involving other coronary artery of anterior wall */
"I2111"  =  "1"  /* ST elevation (STEMI) myocardial infarction involving right coronary artery */
"I2119"  =  "1"  /* ST elevation (STEMI) myocardial infarction involving other coronary artery of inferior wall */
"I2121"  =  "1"  /* ST elevation (STEMI) myocardial infarction involving left circumflex coronary artery */
"I2129"  =  "1"  /* ST elevation (STEMI) myocardial infarction involving other sites */
"I213"  =  "1"  /* ST elevation (STEMI) myocardial infarction of unspecified site */
"I214"  =  "1"  /* Non-ST elevation (NSTEMI) myocardial infarction */
"I220"  =  "1"  /* Subsequent ST elevation (STEMI) myocardial infarction of anterior wall */
"I221"  =  "1"  /* Subsequent ST elevation (STEMI) myocardial infarction of inferior wall */
"I222"  =  "1"  /* Subsequent non-ST elevation (NSTEMI) myocardial infarction */
"I228"  =  "1"  /* Subsequent ST elevation (STEMI) myocardial infarction of other sites */
"I229"  =  "1"  /* Subsequent ST elevation (STEMI) myocardial infarction of unspecified site */
"I240"  =  "1"  /* Acute coronary thrombosis not resulting in myocardial infarction */
"I241"  =  "1"  /* Dressler's syndrome */
"I248"  =  "1"  /* Other forms of acute ischemic heart disease */
"I249"  =  "1"  /* Acute ischemic heart disease, unspecified */
"I2510"  =  "1"  /* Atherosclerotic heart disease of native coronary artery without angina pectoris */
"I25110"  =  "1"  /* Atherosclerotic heart disease of native coronary artery with unstable angina pectoris */
"I25111"  =  "1"  /* Atherosclerotic heart disease of native coronary artery with angina pectoris with documented spasm */
"I25118"  =  "1"  /* Atherosclerotic heart disease of native coronary artery with other forms of angina pectoris */
"I25119"  =  "1"  /* Atherosclerotic heart disease of native coronary artery with unspecified angina pectoris */
"I252"  =  "1"  /* Old myocardial infarction */
"I253"  =  "1"  /* Aneurysm of heart */
"I2541"  =  "1"  /* Coronary artery aneurysm */
"I2542"  =  "1"  /* Coronary artery dissection */
"I255"  =  "1"  /* Ischemic cardiomyopathy */
"I256"  =  "1"  /* Silent myocardial ischemia */
"I25700"  =  "1"  /* Atherosclerosis of coronary artery bypass graft(s), unspecified, with unstable angina pectoris */
"I25701"  =  "1"  /* Atherosclerosis of coronary artery bypass graft(s), unspecified, with angina pectoris with documented spasm */
"I25708"  =  "1"  /* Atherosclerosis of coronary artery bypass graft(s), unspecified, with other forms of angina pectoris */
"I25709"  =  "1"  /* Atherosclerosis of coronary artery bypass graft(s), unspecified, with unspecified angina pectoris */
"I25710"  =  "1"  /* Atherosclerosis of autologous vein coronary artery bypass graft(s) with unstable angina pectoris */
"I25711"  =  "1"  /* Atherosclerosis of autologous vein coronary artery bypass graft(s) with angina pectoris with documented spasm */
"I25718"  =  "1"  /* Atherosclerosis of autologous vein coronary artery bypass graft(s) with other forms of angina pectoris */
"I25719"  =  "1"  /* Atherosclerosis of autologous vein coronary artery bypass graft(s) with unspecified angina pectoris */
"I25720"  =  "1"  /* Atherosclerosis of autologous artery coronary artery bypass graft(s) with unstable angina pectoris */
"I25721"  =  "1"  /* Atherosclerosis of autologous artery coronary artery bypass graft(s) with angina pectoris with documented spasm */
"I25728"  =  "1"  /* Atherosclerosis of autologous artery coronary artery bypass graft(s) with other forms of angina pectoris */
"I25729"  =  "1"  /* Atherosclerosis of autologous artery coronary artery bypass graft(s) with unspecified angina pectoris */
"I25730"  =  "1"  /* Atherosclerosis of nonautologous biological coronary artery bypass graft(s) with unstable angina pectoris */
"I25731"  =  "1"  /* Atherosclerosis of nonautologous biological coronary artery bypass graft(s) with angina pectoris with documented spasm */
"I25738"  =  "1"  /* Atherosclerosis of nonautologous biological coronary artery bypass graft(s) with other forms of angina pectoris */
"I25739"  =  "1"  /* Atherosclerosis of nonautologous biological coronary artery bypass graft(s) with unspecified angina pectoris */
"I25750"  =  "1"  /* Atherosclerosis of native coronary artery of transplanted heart with unstable angina */
"I25751"  =  "1"  /* Atherosclerosis of native coronary artery of transplanted heart with angina pectoris with documented spasm */
"I25758"  =  "1"  /* Atherosclerosis of native coronary artery of transplanted heart with other forms of angina pectoris */
"I25759"  =  "1"  /* Atherosclerosis of native coronary artery of transplanted heart with unspecified angina pectoris */
"I25760"  =  "1"  /* Atherosclerosis of bypass graft of coronary artery of transplanted heart with unstable angina */
"I25761"  =  "1"  /* Atherosclerosis of bypass graft of coronary artery of transplanted heart with angina pectoris with documented spasm */
"I25768"  =  "1"  /* Atherosclerosis of bypass graft of coronary artery of transplanted heart with other forms of angina pectoris */
"I25769"  =  "1"  /* Atherosclerosis of bypass graft of coronary artery of transplanted heart with unspecified angina pectoris */
"I25790"  =  "1"  /* Atherosclerosis of other coronary artery bypass graft(s) with unstable angina pectoris */
"I25791"  =  "1"  /* Atherosclerosis of other coronary artery bypass graft(s) with angina pectoris with documented spasm */
"I25798"  =  "1"  /* Atherosclerosis of other coronary artery bypass graft(s) with other forms of angina pectoris */
"I25799"  =  "1"  /* Atherosclerosis of other coronary artery bypass graft(s) with unspecified angina pectoris */
"I25810"  =  "1"  /* Atherosclerosis of coronary artery bypass graft(s) without angina pectoris */
"I25811"  =  "1"  /* Atherosclerosis of native coronary artery of transplanted heart without angina pectoris */
"I25812"  =  "1"  /* Atherosclerosis of bypass graft of coronary artery of transplanted heart without angina pectoris */
"I2583"  =  "1"  /* Coronary atherosclerosis due to lipid rich plaque */
"I2584"  =  "1"  /* Coronary atherosclerosis due to calcified coronary lesion */
"I2589"  =  "1"  /* Other forms of chronic ischemic heart disease */
"I259"  =  "1"  /* Chronic ischemic heart disease, unspecified */
OTHER  = '0' ;

 /* Exclude: Indications for right-sided catherterization diagnosis */
 /* ICD-10-CM Diagnosis Code: */

VALUE $PRCAT2D
"A1884"  =  "1"  /* Tuberculosis of heart */
"I010"  =  "1"  /* Acute rheumatic pericarditis */
"I011"  =  "1"  /* Acute rheumatic endocarditis */
"I012"  =  "1"  /* Acute rheumatic myocarditis */
"I018"  =  "1"  /* Other acute rheumatic heart disease */
"I019"  =  "1"  /* Acute rheumatic heart disease, unspecified */
"I020"  =  "1"  /* Rheumatic chorea with heart involvement */
"I050"  =  "1"  /* Rheumatic mitral stenosis */
"I051"  =  "1"  /* Rheumatic mitral insufficiency */
"I052"  =  "1"  /* Rheumatic mitral stenosis with insufficiency */
"I058"  =  "1"  /* Other rheumatic mitral valve diseases */
"I059"  =  "1"  /* Rheumatic mitral valve disease, unspecified */
"I070"  =  "1"  /* Rheumatic tricuspid stenosis */
"I071"  =  "1"  /* Rheumatic tricuspid insufficiency */
"I072"  =  "1"  /* Rheumatic tricuspid stenosis and insufficiency */
"I078"  =  "1"  /* Other rheumatic tricuspid valve diseases */
"I079"  =  "1"  /* Rheumatic tricuspid valve disease, unspecified */
"I080"  =  "1"  /* Rheumatic disorders of both mitral and aortic valves */
"I081"  =  "1"  /* Rheumatic disorders of both mitral and tricuspid valves */
"I082"  =  "1"  /* Rheumatic disorders of both aortic and tricuspid valves */
"I083"  =  "1"  /* Combined rheumatic disorders of mitral, aortic and tricuspid valves */
"I088"  =  "1"  /* Other rheumatic multiple valve diseases */
"I089"  =  "1"  /* Rheumatic multiple valve disease, unspecified */
"I090"  =  "1"  /* Rheumatic myocarditis */
"I091"  =  "1"  /* Rheumatic diseases of endocardium, valve unspecified */
"I092"  =  "1"  /* Chronic rheumatic pericarditis */
"I0981"  =  "1"  /* Rheumatic heart failure */
"I0989"  =  "1"  /* Other specified rheumatic heart diseases */
"I099"  =  "1"  /* Rheumatic heart disease, unspecified */
"I110"  =  "1"  /* Hypertensive heart disease with heart failure */
"I119"  =  "1"  /* Hypertensive heart disease without heart failure */
"I130"  =  "1"  /* Hypertensive heart and chronic kidney disease with heart failure and stage 1 through stage 4 chronic kidney disease, or unspecified chronic kidney disease */
"I1310"  =  "1"  /* Hypertensive heart and chronic kidney disease without heart failure, with stage 1 through stage 4 chronic kidney disease, or unspecified chronic kidney disease */
"I1311"  =  "1"  /* Hypertensive heart and chronic kidney disease without heart failure, with stage 5 chronic kidney disease, or end stage renal disease */
"I132"  =  "1"  /* Hypertensive heart and chronic kidney disease with heart failure and with stage 5 chronic kidney disease, or end stage renal disease */
"I2601"  =  "1"  /* Septic pulmonary embolism with acute cor pulmonale */
"I2602"  =  "1"  /* Saddle embolus of pulmonary artery with acute cor pulmonale */
"I2609"  =  "1"  /* Other pulmonary embolism with acute cor pulmonale */
"I2690"  =  "1"  /* Septic pulmonary embolism without acute cor pulmonale */
"I2692"  =  "1"  /* Saddle embolus of pulmonary artery without acute cor pulmonale */
"I2699"  =  "1"  /* Other pulmonary embolism without acute cor pulmonale */
"I270"  =  "1"  /* Primary pulmonary hypertension */
"I271"  =  "1"  /* Kyphoscoliotic heart disease */
"I272"  =  "1"  /* Other secondary pulmonary hypertension */
"I2781"  =  "1"  /* Cor pulmonale (chronic) */
"I2789"  =  "1"  /* Other specified pulmonary heart diseases */
"I279"  =  "1"  /* Pulmonary heart disease, unspecified */
"I280"  =  "1"  /* Arteriovenous fistula of pulmonary vessels */
"I281"  =  "1"  /* Aneurysm of pulmonary artery */
"I288"  =  "1"  /* Other diseases of pulmonary vessels */
"I289"  =  "1"  /* Disease of pulmonary vessels, unspecified */
"I300"  =  "1"  /* Acute nonspecific idiopathic pericarditis */
"I301"  =  "1"  /* Infective pericarditis */
"I308"  =  "1"  /* Other forms of acute pericarditis */
"I309"  =  "1"  /* Acute pericarditis, unspecified */
"I310"  =  "1"  /* Chronic adhesive pericarditis */
"I311"  =  "1"  /* Chronic constrictive pericarditis */
"I312"  =  "1"  /* Hemopericardium, not elsewhere classified */
"I313"  =  "1"  /* Pericardial effusion (noninflammatory) */
"I314"  =  "1"  /* Cardiac tamponade */
"I318"  =  "1"  /* Other specified diseases of pericardium */
"I319"  =  "1"  /* Disease of pericardium, unspecified */
"I32"  =  "1"  /* Pericarditis in diseases classified elsewhere */
"I330"  =  "1"  /* Acute and subacute infective endocarditis */
"I339"  =  "1"  /* Acute and subacute endocarditis, unspecified */
"I340"  =  "1"  /* Nonrheumatic mitral (valve) insufficiency */
"I341"  =  "1"  /* Nonrheumatic mitral (valve) prolapse */
"I342"  =  "1"  /* Nonrheumatic mitral (valve) stenosis */
"I348"  =  "1"  /* Other nonrheumatic mitral valve disorders */
"I349"  =  "1"  /* Nonrheumatic mitral valve disorder, unspecified */
"I350"  =  "1"  /* Nonrheumatic aortic (valve) stenosis */
"I351"  =  "1"  /* Nonrheumatic aortic (valve) insufficiency */
"I352"  =  "1"  /* Nonrheumatic aortic (valve) stenosis with insufficiency */
"I358"  =  "1"  /* Other nonrheumatic aortic valve disorders */
"I359"  =  "1"  /* Nonrheumatic aortic valve disorder, unspecified */
"I360"  =  "1"  /* Nonrheumatic tricuspid (valve) stenosis */
"I361"  =  "1"  /* Nonrheumatic tricuspid (valve) insufficiency */
"I362"  =  "1"  /* Nonrheumatic tricuspid (valve) stenosis with insufficiency */
"I368"  =  "1"  /* Other nonrheumatic tricuspid valve disorders */
"I369"  =  "1"  /* Nonrheumatic tricuspid valve disorder, unspecified */
"I370"  =  "1"  /* Nonrheumatic pulmonary valve stenosis */
"I371"  =  "1"  /* Nonrheumatic pulmonary valve insufficiency */
"I372"  =  "1"  /* Nonrheumatic pulmonary valve stenosis with insufficiency */
"I378"  =  "1"  /* Other nonrheumatic pulmonary valve disorders */
"I379"  =  "1"  /* Nonrheumatic pulmonary valve disorder, unspecified */
"I38"  =  "1"  /* Endocarditis, valve unspecified */
"I39"  =  "1"  /* Endocarditis and heart valve disorders in diseases classified elsewhere */
"I400"  =  "1"  /* Infective myocarditis */
"I401"  =  "1"  /* Isolated myocarditis */
"I408"  =  "1"  /* Other acute myocarditis */
"I409"  =  "1"  /* Acute myocarditis, unspecified */
"I41"  =  "1"  /* Myocarditis in diseases classified elsewhere */
"I420"  =  "1"  /* Dilated cardiomyopathy */
"I421"  =  "1"  /* Obstructive hypertrophic cardiomyopathy */
"I422"  =  "1"  /* Other hypertrophic cardiomyopathy */
"I423"  =  "1"  /* Endomyocardial (eosinophilic) disease */
"I424"  =  "1"  /* Endocardial fibroelastosis */
"I425"  =  "1"  /* Other restrictive cardiomyopathy */
"I426"  =  "1"  /* Alcoholic cardiomyopathy */
"I427"  =  "1"  /* Cardiomyopathy due to drug and external agent */
"I428"  =  "1"  /* Other cardiomyopathies */
"I429"  =  "1"  /* Cardiomyopathy, unspecified */
"I43"  =  "1"  /* Cardiomyopathy in diseases classified elsewhere */
"I501"  =  "1"  /* Left ventricular failure */
"I5020"  =  "1"  /* Unspecified systolic (congestive) heart failure */
"I5021"  =  "1"  /* Acute systolic (congestive) heart failure */
"I5022"  =  "1"  /* Chronic systolic (congestive) heart failure */
"I5023"  =  "1"  /* Acute on chronic systolic (congestive) heart failure */
"I5030"  =  "1"  /* Unspecified diastolic (congestive) heart failure */
"I5031"  =  "1"  /* Acute diastolic (congestive) heart failure */
"I5032"  =  "1"  /* Chronic diastolic (congestive) heart failure */
"I5033"  =  "1"  /* Acute on chronic diastolic (congestive) heart failure */
"I5040"  =  "1"  /* Unspecified combined systolic (congestive) and diastolic (congestive) heart failure */
"I5041"  =  "1"  /* Acute combined systolic (congestive) and diastolic (congestive) heart failure */
"I5042"  =  "1"  /* Chronic combined systolic (congestive) and diastolic (congestive) heart failure */
"I5043"  =  "1"  /* Acute on chronic combined systolic (congestive) and diastolic (congestive) heart failure */
"I509"  =  "1"  /* Heart failure, unspecified */
"M3211"  =  "1"  /* Endocarditis in systemic lupus erythematosus */
"M3212"  =  "1"  /* Pericarditis in systemic lupus erythematosus */
"P293"  =  "1"  /* Persistent fetal circulation */
"Q200"  =  "1"  /* Common arterial trunk */
"Q201"  =  "1"  /* Double outlet right ventricle */
"Q202"  =  "1"  /* Double outlet left ventricle */
"Q203"  =  "1"  /* Discordant ventriculoarterial connection */
"Q204"  =  "1"  /* Double inlet ventricle */
"Q205"  =  "1"  /* Discordant atrioventricular connection */
"Q206"  =  "1"  /* Isomerism of atrial appendages */
"Q208"  =  "1"  /* Other congenital malformations of cardiac chambers and connections */
"Q209"  =  "1"  /* Congenital malformation of cardiac chambers and connections, unspecified */
"Q210"  =  "1"  /* Ventricular septal defect */
"Q211"  =  "1"  /* Atrial septal defect */
"Q212"  =  "1"  /* Atrioventricular septal defect */
"Q213"  =  "1"  /* Tetralogy of Fallot */
"Q214"  =  "1"  /* Aortopulmonary septal defect */
"Q218"  =  "1"  /* Other congenital malformations of cardiac septa */
"Q219"  =  "1"  /* Congenital malformation of cardiac septum, unspecified */
"Q220"  =  "1"  /* Pulmonary valve atresia */
"Q221"  =  "1"  /* Congenital pulmonary valve stenosis */
"Q222"  =  "1"  /* Congenital pulmonary valve insufficiency */
"Q223"  =  "1"  /* Other congenital malformations of pulmonary valve */
"Q224"  =  "1"  /* Congenital tricuspid stenosis */
"Q225"  =  "1"  /* Ebstein's anomaly */
"Q226"  =  "1"  /* Hypoplastic right heart syndrome */
"Q228"  =  "1"  /* Other congenital malformations of tricuspid valve */
"Q229"  =  "1"  /* Congenital malformation of tricuspid valve, unspecified */
"Q230"  =  "1"  /* Congenital stenosis of aortic valve */
"Q231"  =  "1"  /* Congenital insufficiency of aortic valve */
"Q232"  =  "1"  /* Congenital mitral stenosis */
"Q233"  =  "1"  /* Congenital mitral insufficiency */
"Q234"  =  "1"  /* Hypoplastic left heart syndrome */
"Q238"  =  "1"  /* Other congenital malformations of aortic and mitral valves */
"Q239"  =  "1"  /* Congenital malformation of aortic and mitral valves, unspecified */
"Q240"  =  "1"  /* Dextrocardia */
"Q241"  =  "1"  /* Levocardia */
"Q242"  =  "1"  /* Cor triatriatum */
"Q243"  =  "1"  /* Pulmonary infundibular stenosis */
"Q244"  =  "1"  /* Congenital subaortic stenosis */
"Q245"  =  "1"  /* Malformation of coronary vessels */
"Q246"  =  "1"  /* Congenital heart block */
"Q248"  =  "1"  /* Other specified congenital malformations of heart */
"Q249"  =  "1"  /* Congenital malformation of heart, unspecified */
"Q250"  =  "1"  /* Patent ductus arteriosus */
"Q251"  =  "1"  /* Coarctation of aorta */
"Q252"  =  "1"  /* Atresia of aorta */
"Q253"  =  "1"  /* Supravalvular aortic stenosis */
"Q254"  =  "1"  /* Other congenital malformations of aorta */
"Q255"  =  "1"  /* Atresia of pulmonary artery */
"Q2571"  =  "1"  /* Coarctation of pulmonary artery */
"Q258"  =  "1"  /* Other congenital malformations of other great arteries */
"Q259"  =  "1"  /* Congenital malformation of great arteries, unspecified */
"Q260"  =  "1"  /* Congenital stenosis of vena cava */
"Q261"  =  "1"  /* Persistent left superior vena cava */
"Q262"  =  "1"  /* Total anomalous pulmonary venous connection */
"Q263"  =  "1"  /* Partial anomalous pulmonary venous connection */
"Q264"  =  "1"  /* Anomalous pulmonary venous connection, unspecified */
"Q265"  =  "1"  /* Anomalous portal venous connection */
"Q266"  =  "1"  /* Portal vein-hepatic artery fistula */
"Q268"  =  "1"  /* Other congenital malformations of great veins */
"Q269"  =  "1"  /* Congenital malformation of great vein, unspecified */
"Q270"  =  "1"  /* Congenital absence and hypoplasia of umbilical artery */
"Q271"  =  "1"  /* Congenital renal artery stenosis */
"Q272"  =  "1"  /* Other congenital malformations of renal artery */
"Q2730"  =  "1"  /* Arteriovenous malformation, site unspecified */
"Q2731"  =  "1"  /* Arteriovenous malformation of vessel of upper limb */
"Q2732"  =  "1"  /* Arteriovenous malformation of vessel of lower limb */
"Q2733"  =  "1"  /* Arteriovenous malformation of digestive system vessel */
"Q2734"  =  "1"  /* Arteriovenous malformation of renal vessel */
"Q2739"  =  "1"  /* Arteriovenous malformation, other site */
"Q274"  =  "1"  /* Congenital phlebectasia */
"Q278"  =  "1"  /* Other specified congenital malformations of peripheral vascular system */
"Q279"  =  "1"  /* Congenital malformation of peripheral vascular system, unspecified */
"Q280"  =  "1"  /* Arteriovenous malformation of precerebral vessels */
"Q281"  =  "1"  /* Other malformations of precerebral vessels */
"Q282"  =  "1"  /* Arteriovenous malformation of cerebral vessels */
"Q283"  =  "1"  /* Other malformations of cerebral vessels */
"Q288"  =  "1"  /* Other specified congenital malformations of circulatory system */
"Q289"  =  "1"  /* Congenital malformation of circulatory system, unspecified */
OTHER  = '0' ;

 * -------------------------------------------------------------- ;
 * --- FORMATS - AREA LEVEL UTILIZATION INDICATORS -------------- ;
 * -------------------------------------------------------------- ;

 /* ---------------------------------------- */
 /* Hysterectomy rate (PRHYST)               */
 /*   Utilization (area) Indicator / IQI #28 */
 /* ---------------------------------------- */

 /* Hysterectomy */
 /* ICD-10-CM Procedure Codes: */

VALUE $PRHYSTP
"0UT90ZZ"  =  "1"  /* Resection of Uterus, Open Approach */
"0UT94ZZ"  =  "1"  /* Resection of Uterus, Percutaneous Endoscopic Approach */
"0UT97ZZ"  =  "1"  /* Resection of Uterus, Via Natural or Artificial Opening */
"0UT98ZZ"  =  "1"  /* Resection of Uterus, Via Natural or Artificial Opening Endoscopic */
"0UT9FZZ"  =  "1"  /* Resection of Uterus, Via Natural or Artificial Opening With Percutaneous Endoscopic Assistance */
OTHER  = '0' ;

 /* Exclude: Female Genital Cancer */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $PRHYS1D
"C510"  =  "1"  /* Malignant neoplasm of labium majus */
"C511"  =  "1"  /* Malignant neoplasm of labium minus */
"C512"  =  "1"  /* Malignant neoplasm of clitoris */
"C518"  =  "1"  /* Malignant neoplasm of overlapping sites of vulva */
"C519"  =  "1"  /* Malignant neoplasm of vulva, unspecified */
"C52"  =  "1"  /* Malignant neoplasm of vagina */
"C530"  =  "1"  /* Malignant neoplasm of endocervix */
"C531"  =  "1"  /* Malignant neoplasm of exocervix */
"C538"  =  "1"  /* Malignant neoplasm of overlapping sites of cervix uteri */
"C539"  =  "1"  /* Malignant neoplasm of cervix uteri, unspecified */
"C540"  =  "1"  /* Malignant neoplasm of isthmus uteri */
"C541"  =  "1"  /* Malignant neoplasm of endometrium */
"C542"  =  "1"  /* Malignant neoplasm of myometrium */
"C543"  =  "1"  /* Malignant neoplasm of fundus uteri */
"C548"  =  "1"  /* Malignant neoplasm of overlapping sites of corpus uteri */
"C549"  =  "1"  /* Malignant neoplasm of corpus uteri, unspecified */
"C55"  =  "1"  /* Malignant neoplasm of uterus, part unspecified */
"C561"  =  "1"  /* Malignant neoplasm of right ovary */
"C562"  =  "1"  /* Malignant neoplasm of left ovary */
"C569"  =  "1"  /* Malignant neoplasm of unspecified ovary */
"C5700"  =  "1"  /* Malignant neoplasm of unspecified fallopian tube */
"C5701"  =  "1"  /* Malignant neoplasm of right fallopian tube */
"C5702"  =  "1"  /* Malignant neoplasm of left fallopian tube */
"C5710"  =  "1"  /* Malignant neoplasm of unspecified broad ligament */
"C5711"  =  "1"  /* Malignant neoplasm of right broad ligament */
"C5712"  =  "1"  /* Malignant neoplasm of left broad ligament */
"C5720"  =  "1"  /* Malignant neoplasm of unspecified round ligament */
"C5721"  =  "1"  /* Malignant neoplasm of right round ligament */
"C5722"  =  "1"  /* Malignant neoplasm of left round ligament */
"C573"  =  "1"  /* Malignant neoplasm of parametrium */
"C574"  =  "1"  /* Malignant neoplasm of uterine adnexa, unspecified */
"C577"  =  "1"  /* Malignant neoplasm of other specified female genital organs */
"C578"  =  "1"  /* Malignant neoplasm of overlapping sites of female genital organs */
"C579"  =  "1"  /* Malignant neoplasm of female genital organ, unspecified */
"C58"  =  "1"  /* Malignant neoplasm of placenta */
"D060"  =  "1"  /* Carcinoma in situ of endocervix */
"D061"  =  "1"  /* Carcinoma in situ of exocervix */
"D067"  =  "1"  /* Carcinoma in situ of other parts of cervix */
"D069"  =  "1"  /* Carcinoma in situ of cervix, unspecified */
"D070"  =  "1"  /* Carcinoma in situ of endometrium */
"D071"  =  "1"  /* Carcinoma in situ of vulva */
"D072"  =  "1"  /* Carcinoma in situ of vagina */
"D0730"  =  "1"  /* Carcinoma in situ of unspecified female genital organs */
"D0739"  =  "1"  /* Carcinoma in situ of other female genital organs */
"D390"  =  "1"  /* Neoplasm of uncertain behavior of uterus */
"D3910"  =  "1"  /* Neoplasm of uncertain behavior of unspecified ovary */
"D3911"  =  "1"  /* Neoplasm of uncertain behavior of right ovary */
"D3912"  =  "1"  /* Neoplasm of uncertain behavior of left ovary */
"D392"  =  "1"  /* Neoplasm of uncertain behavior of placenta */
"D398"  =  "1"  /* Neoplasm of uncertain behavior of other specified female genital organs */
"D399"  =  "1"  /* Neoplasm of uncertain behavior of female genital organ, unspecified */
OTHER  = '0' ;

 /* Exclude: Pelvic or Lower Abdominal Trauma */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $PRHYS2D
"S31000S"  =  "1"  /* Unspecified open wound of lower back and pelvis without penetration into retroperitoneum, sequela */
"S31001A"  =  "1"  /* Unspecified open wound of lower back and pelvis with penetration into retroperitoneum, initial encounter */
"S31001S"  =  "1"  /* Unspecified open wound of lower back and pelvis with penetration into retroperitoneum, sequela */
"S31010A"  =  "1"  /* Laceration without foreign body of lower back and pelvis without penetration into retroperitoneum, initial encounter */
"S31010S"  =  "1"  /* Laceration without foreign body of lower back and pelvis without penetration into retroperitoneum, sequela */
"S31011A"  =  "1"  /* Laceration without foreign body of lower back and pelvis with penetration into retroperitoneum, initial encounter */
"S31011S"  =  "1"  /* Laceration without foreign body of lower back and pelvis with penetration into retroperitoneum, sequela */
"S31020A"  =  "1"  /* Laceration with foreign body of lower back and pelvis without penetration into retroperitoneum, initial encounter */
"S31020S"  =  "1"  /* Laceration with foreign body of lower back and pelvis without penetration into retroperitoneum, sequela */
"S31021A"  =  "1"  /* Laceration with foreign body of lower back and pelvis with penetration into retroperitoneum, initial encounter */
"S31021S"  =  "1"  /* Laceration with foreign body of lower back and pelvis with penetration into retroperitoneum, sequela */
"S31030A"  =  "1"  /* Puncture wound without foreign body of lower back and pelvis without penetration into retroperitoneum, initial encounter */
"S31030S"  =  "1"  /* Puncture wound without foreign body of lower back and pelvis without penetration into retroperitoneum, sequela */
"S31031A"  =  "1"  /* Puncture wound without foreign body of lower back and pelvis with penetration into retroperitoneum, initial encounter */
"S31031S"  =  "1"  /* Puncture wound without foreign body of lower back and pelvis with penetration into retroperitoneum, sequela */
"S31040A"  =  "1"  /* Puncture wound with foreign body of lower back and pelvis without penetration into retroperitoneum, initial encounter */
"S31040S"  =  "1"  /* Puncture wound with foreign body of lower back and pelvis without penetration into retroperitoneum, sequela */
"S31041A"  =  "1"  /* Puncture wound with foreign body of lower back and pelvis with penetration into retroperitoneum, initial encounter */
"S31041S"  =  "1"  /* Puncture wound with foreign body of lower back and pelvis with penetration into retroperitoneum, sequela */
"S31050A"  =  "1"  /* Open bite of lower back and pelvis without penetration into retroperitoneum, initial encounter */
"S31050S"  =  "1"  /* Open bite of lower back and pelvis without penetration into retroperitoneum, sequela */
"S31051A"  =  "1"  /* Open bite of lower back and pelvis with penetration into retroperitoneum, initial encounter */
"S31051S"  =  "1"  /* Open bite of lower back and pelvis with penetration into retroperitoneum, sequela */
"S31600A"  =  "1"  /* Unspecified open wound of abdominal wall, right upper quadrant with penetration into peritoneal cavity, initial encounter */
"S31600S"  =  "1"  /* Unspecified open wound of abdominal wall, right upper quadrant with penetration into peritoneal cavity, sequela */
"S31601A"  =  "1"  /* Unspecified open wound of abdominal wall, left upper quadrant with penetration into peritoneal cavity, initial encounter */
"S31601S"  =  "1"  /* Unspecified open wound of abdominal wall, left upper quadrant with penetration into peritoneal cavity, sequela */
"S31602A"  =  "1"  /* Unspecified open wound of abdominal wall, epigastric region with penetration into peritoneal cavity, initial encounter */
"S31602S"  =  "1"  /* Unspecified open wound of abdominal wall, epigastric region with penetration into peritoneal cavity, sequela */
"S31603A"  =  "1"  /* Unspecified open wound of abdominal wall, right lower quadrant with penetration into peritoneal cavity, initial encounter */
"S31603S"  =  "1"  /* Unspecified open wound of abdominal wall, right lower quadrant with penetration into peritoneal cavity, sequela */
"S31604A"  =  "1"  /* Unspecified open wound of abdominal wall, left lower quadrant with penetration into peritoneal cavity, initial encounter */
"S31604S"  =  "1"  /* Unspecified open wound of abdominal wall, left lower quadrant with penetration into peritoneal cavity, sequela */
"S31605A"  =  "1"  /* Unspecified open wound of abdominal wall, periumbilic region with penetration into peritoneal cavity, initial encounter */
"S31605S"  =  "1"  /* Unspecified open wound of abdominal wall, periumbilic region with penetration into peritoneal cavity, sequela */
"S31609A"  =  "1"  /* Unspecified open wound of abdominal wall, unspecified quadrant with penetration into peritoneal cavity, initial encounter */
"S31609S"  =  "1"  /* Unspecified open wound of abdominal wall, unspecified quadrant with penetration into peritoneal cavity, sequela */
"S31610A"  =  "1"  /* Laceration without foreign body of abdominal wall, right upper quadrant with penetration into peritoneal cavity, initial encounter */
"S31610S"  =  "1"  /* Laceration without foreign body of abdominal wall, right upper quadrant with penetration into peritoneal cavity, sequela */
"S31611A"  =  "1"  /* Laceration without foreign body of abdominal wall, left upper quadrant with penetration into peritoneal cavity, initial encounter */
"S31611S"  =  "1"  /* Laceration without foreign body of abdominal wall, left upper quadrant with penetration into peritoneal cavity, sequela */
"S31612A"  =  "1"  /* Laceration without foreign body of abdominal wall, epigastric region with penetration into peritoneal cavity, initial encounter */
"S31612S"  =  "1"  /* Laceration without foreign body of abdominal wall, epigastric region with penetration into peritoneal cavity, sequela */
"S31613A"  =  "1"  /* Laceration without foreign body of abdominal wall, right lower quadrant with penetration into peritoneal cavity, initial encounter */
"S31613S"  =  "1"  /* Laceration without foreign body of abdominal wall, right lower quadrant with penetration into peritoneal cavity, sequela */
"S31614A"  =  "1"  /* Laceration without foreign body of abdominal wall, left lower quadrant with penetration into peritoneal cavity, initial encounter */
"S31614S"  =  "1"  /* Laceration without foreign body of abdominal wall, left lower quadrant with penetration into peritoneal cavity, sequela */
"S31615A"  =  "1"  /* Laceration without foreign body of abdominal wall, periumbilic region with penetration into peritoneal cavity, initial encounter */
"S31615S"  =  "1"  /* Laceration without foreign body of abdominal wall, periumbilic region with penetration into peritoneal cavity, sequela */
"S31619A"  =  "1"  /* Laceration without foreign body of abdominal wall, unspecified quadrant with penetration into peritoneal cavity, initial encounter */
"S31619S"  =  "1"  /* Laceration without foreign body of abdominal wall, unspecified quadrant with penetration into peritoneal cavity, sequela */
"S31620A"  =  "1"  /* Laceration with foreign body of abdominal wall, right upper quadrant with penetration into peritoneal cavity, initial encounter */
"S31620S"  =  "1"  /* Laceration with foreign body of abdominal wall, right upper quadrant with penetration into peritoneal cavity, sequela */
"S31621A"  =  "1"  /* Laceration with foreign body of abdominal wall, left upper quadrant with penetration into peritoneal cavity, initial encounter */
"S31621S"  =  "1"  /* Laceration with foreign body of abdominal wall, left upper quadrant with penetration into peritoneal cavity, sequela */
"S31622A"  =  "1"  /* Laceration with foreign body of abdominal wall, epigastric region with penetration into peritoneal cavity, initial encounter */
"S31622S"  =  "1"  /* Laceration with foreign body of abdominal wall, epigastric region with penetration into peritoneal cavity, sequela */
"S31623A"  =  "1"  /* Laceration with foreign body of abdominal wall, right lower quadrant with penetration into peritoneal cavity, initial encounter */
"S31623S"  =  "1"  /* Laceration with foreign body of abdominal wall, right lower quadrant with penetration into peritoneal cavity, sequela */
"S31624A"  =  "1"  /* Laceration with foreign body of abdominal wall, left lower quadrant with penetration into peritoneal cavity, initial encounter */
"S31624S"  =  "1"  /* Laceration with foreign body of abdominal wall, left lower quadrant with penetration into peritoneal cavity, sequela */
"S31625A"  =  "1"  /* Laceration with foreign body of abdominal wall, periumbilic region with penetration into peritoneal cavity, initial encounter */
"S31625S"  =  "1"  /* Laceration with foreign body of abdominal wall, periumbilic region with penetration into peritoneal cavity, sequela */
"S31629A"  =  "1"  /* Laceration with foreign body of abdominal wall, unspecified quadrant with penetration into peritoneal cavity, initial encounter */
"S31629S"  =  "1"  /* Laceration with foreign body of abdominal wall, unspecified quadrant with penetration into peritoneal cavity, sequela */
"S31630A"  =  "1"  /* Puncture wound without foreign body of abdominal wall, right upper quadrant with penetration into peritoneal cavity, initial encounter */
"S31630S"  =  "1"  /* Puncture wound without foreign body of abdominal wall, right upper quadrant with penetration into peritoneal cavity, sequela */
"S31631A"  =  "1"  /* Puncture wound without foreign body of abdominal wall, left upper quadrant with penetration into peritoneal cavity, initial encounter */
"S31631S"  =  "1"  /* Puncture wound without foreign body of abdominal wall, left upper quadrant with penetration into peritoneal cavity, sequela */
"S31632A"  =  "1"  /* Puncture wound without foreign body of abdominal wall, epigastric region with penetration into peritoneal cavity, initial encounter */
"S31632S"  =  "1"  /* Puncture wound without foreign body of abdominal wall, epigastric region with penetration into peritoneal cavity, sequela */
"S31633A"  =  "1"  /* Puncture wound without foreign body of abdominal wall, right lower quadrant with penetration into peritoneal cavity, initial encounter */
"S31633S"  =  "1"  /* Puncture wound without foreign body of abdominal wall, right lower quadrant with penetration into peritoneal cavity, sequela */
"S31634A"  =  "1"  /* Puncture wound without foreign body of abdominal wall, left lower quadrant with penetration into peritoneal cavity, initial encounter */
"S31634S"  =  "1"  /* Puncture wound without foreign body of abdominal wall, left lower quadrant with penetration into peritoneal cavity, sequela */
"S31635A"  =  "1"  /* Puncture wound without foreign body of abdominal wall, periumbilic region with penetration into peritoneal cavity, initial encounter */
"S31635S"  =  "1"  /* Puncture wound without foreign body of abdominal wall, periumbilic region with penetration into peritoneal cavity, sequela */
"S31639A"  =  "1"  /* Puncture wound without foreign body of abdominal wall, unspecified quadrant with penetration into peritoneal cavity, initial encounter */
"S31639S"  =  "1"  /* Puncture wound without foreign body of abdominal wall, unspecified quadrant with penetration into peritoneal cavity, sequela */
"S31640A"  =  "1"  /* Puncture wound with foreign body of abdominal wall, right upper quadrant with penetration into peritoneal cavity, initial encounter */
"S31640S"  =  "1"  /* Puncture wound with foreign body of abdominal wall, right upper quadrant with penetration into peritoneal cavity, sequela */
"S31641A"  =  "1"  /* Puncture wound with foreign body of abdominal wall, left upper quadrant with penetration into peritoneal cavity, initial encounter */
"S31641S"  =  "1"  /* Puncture wound with foreign body of abdominal wall, left upper quadrant with penetration into peritoneal cavity, sequela */
"S31642A"  =  "1"  /* Puncture wound with foreign body of abdominal wall, epigastric region with penetration into peritoneal cavity, initial encounter */
"S31642S"  =  "1"  /* Puncture wound with foreign body of abdominal wall, epigastric region with penetration into peritoneal cavity, sequela */
"S31643A"  =  "1"  /* Puncture wound with foreign body of abdominal wall, right lower quadrant with penetration into peritoneal cavity, initial encounter */
"S31643S"  =  "1"  /* Puncture wound with foreign body of abdominal wall, right lower quadrant with penetration into peritoneal cavity, sequela */
"S31644A"  =  "1"  /* Puncture wound with foreign body of abdominal wall, left lower quadrant with penetration into peritoneal cavity, initial encounter */
"S31644S"  =  "1"  /* Puncture wound with foreign body of abdominal wall, left lower quadrant with penetration into peritoneal cavity, sequela */
"S31645A"  =  "1"  /* Puncture wound with foreign body of abdominal wall, periumbilic region with penetration into peritoneal cavity, initial encounter */
"S31645S"  =  "1"  /* Puncture wound with foreign body of abdominal wall, periumbilic region with penetration into peritoneal cavity, sequela */
"S31649A"  =  "1"  /* Puncture wound with foreign body of abdominal wall, unspecified quadrant with penetration into peritoneal cavity, initial encounter */
"S31649S"  =  "1"  /* Puncture wound with foreign body of abdominal wall, unspecified quadrant with penetration into peritoneal cavity, sequela */
"S31650A"  =  "1"  /* Open bite of abdominal wall, right upper quadrant with penetration into peritoneal cavity, initial encounter */
"S31650S"  =  "1"  /* Open bite of abdominal wall, right upper quadrant with penetration into peritoneal cavity, sequela */
"S31651A"  =  "1"  /* Open bite of abdominal wall, left upper quadrant with penetration into peritoneal cavity, initial encounter */
"S31651S"  =  "1"  /* Open bite of abdominal wall, left upper quadrant with penetration into peritoneal cavity, sequela */
"S31652A"  =  "1"  /* Open bite of abdominal wall, epigastric region with penetration into peritoneal cavity, initial encounter */
"S31652S"  =  "1"  /* Open bite of abdominal wall, epigastric region with penetration into peritoneal cavity, sequela */
"S31653A"  =  "1"  /* Open bite of abdominal wall, right lower quadrant with penetration into peritoneal cavity, initial encounter */
"S31653S"  =  "1"  /* Open bite of abdominal wall, right lower quadrant with penetration into peritoneal cavity, sequela */
"S31654A"  =  "1"  /* Open bite of abdominal wall, left lower quadrant with penetration into peritoneal cavity, initial encounter */
"S31654S"  =  "1"  /* Open bite of abdominal wall, left lower quadrant with penetration into peritoneal cavity, sequela */
"S31655A"  =  "1"  /* Open bite of abdominal wall, periumbilic region with penetration into peritoneal cavity, initial encounter */
"S31655S"  =  "1"  /* Open bite of abdominal wall, periumbilic region with penetration into peritoneal cavity, sequela */
"S31659A"  =  "1"  /* Open bite of abdominal wall, unspecified quadrant with penetration into peritoneal cavity, initial encounter */
"S31659S"  =  "1"  /* Open bite of abdominal wall, unspecified quadrant with penetration into peritoneal cavity, sequela */
"S3681XA"  =  "1"  /* Injury of peritoneum, initial encounter */
"S3681XS"  =  "1"  /* Injury of peritoneum, sequela */
"S36892A"  =  "1"  /* Contusion of other intra-abdominal organs, initial encounter */
"S36892S"  =  "1"  /* Contusion of other intra-abdominal organs, sequela */
"S36893A"  =  "1"  /* Laceration of other intra-abdominal organs, initial encounter */
"S36893S"  =  "1"  /* Laceration of other intra-abdominal organs, sequela */
"S36898A"  =  "1"  /* Other injury of other intra-abdominal organs, initial encounter */
"S36898S"  =  "1"  /* Other injury of other intra-abdominal organs, sequela */
"S36899A"  =  "1"  /* Unspecified injury of other intra-abdominal organs, initial encounter */
"S36899S"  =  "1"  /* Unspecified injury of other intra-abdominal organs, sequela */
"S3690XA"  =  "1"  /* Unspecified injury of unspecified intra-abdominal organ, initial encounter */
"S3690XS"  =  "1"  /* Unspecified injury of unspecified intra-abdominal organ, sequela */
"S3692XA"  =  "1"  /* Contusion of unspecified intra-abdominal organ, initial encounter */
"S3692XS"  =  "1"  /* Contusion of unspecified intra-abdominal organ, sequela */
"S3693XA"  =  "1"  /* Laceration of unspecified intra-abdominal organ, initial encounter */
"S3693XS"  =  "1"  /* Laceration of unspecified intra-abdominal organ, sequela */
"S3699XA"  =  "1"  /* Other injury of unspecified intra-abdominal organ, initial encounter */
"S3699XS"  =  "1"  /* Other injury of unspecified intra-abdominal organ, sequela */
"S37401A"  =  "1"  /* Unspecified injury of ovary, unilateral, initial encounter */
"S37401S"  =  "1"  /* Unspecified injury of ovary, unilateral, sequela */
"S37402A"  =  "1"  /* Unspecified injury of ovary, bilateral, initial encounter */
"S37402S"  =  "1"  /* Unspecified injury of ovary, bilateral, sequela */
"S37409A"  =  "1"  /* Unspecified injury of ovary, unspecified, initial encounter */
"S37409S"  =  "1"  /* Unspecified injury of ovary, unspecified, sequela */
"S37421A"  =  "1"  /* Contusion of ovary, unilateral, initial encounter */
"S37421S"  =  "1"  /* Contusion of ovary, unilateral, sequela */
"S37422A"  =  "1"  /* Contusion of ovary, bilateral, initial encounter */
"S37422S"  =  "1"  /* Contusion of ovary, bilateral, sequela */
"S37429A"  =  "1"  /* Contusion of ovary, unspecified, initial encounter */
"S37429S"  =  "1"  /* Contusion of ovary, unspecified, sequela */
"S37431A"  =  "1"  /* Laceration of ovary, unilateral, initial encounter */
"S37431S"  =  "1"  /* Laceration of ovary, unilateral, sequela */
"S37432A"  =  "1"  /* Laceration of ovary, bilateral, initial encounter */
"S37432S"  =  "1"  /* Laceration of ovary, bilateral, sequela */
"S37439A"  =  "1"  /* Laceration of ovary, unspecified, initial encounter */
"S37439S"  =  "1"  /* Laceration of ovary, unspecified, sequela */
"S37491A"  =  "1"  /* Other injury of ovary, unilateral, initial encounter */
"S37491S"  =  "1"  /* Other injury of ovary, unilateral, sequela */
"S37492A"  =  "1"  /* Other injury of ovary, bilateral, initial encounter */
"S37492S"  =  "1"  /* Other injury of ovary, bilateral, sequela */
"S37499A"  =  "1"  /* Other injury of ovary, unspecified, initial encounter */
"S37499S"  =  "1"  /* Other injury of ovary, unspecified, sequela */
"S37501A"  =  "1"  /* Unspecified injury of fallopian tube, unilateral, initial encounter */
"S37501S"  =  "1"  /* Unspecified injury of fallopian tube, unilateral, sequela */
"S37502A"  =  "1"  /* Unspecified injury of fallopian tube, bilateral, initial encounter */
"S37502S"  =  "1"  /* Unspecified injury of fallopian tube, bilateral, sequela */
"S37509A"  =  "1"  /* Unspecified injury of fallopian tube, unspecified, initial encounter */
"S37509S"  =  "1"  /* Unspecified injury of fallopian tube, unspecified, sequela */
"S37511A"  =  "1"  /* Primary blast injury of fallopian tube, unilateral, initial encounter */
"S37511S"  =  "1"  /* Primary blast injury of fallopian tube, unilateral, sequela */
"S37512A"  =  "1"  /* Primary blast injury of fallopian tube, bilateral, initial encounter */
"S37512S"  =  "1"  /* Primary blast injury of fallopian tube, bilateral, sequela */
"S37519A"  =  "1"  /* Primary blast injury of fallopian tube, unspecified, initial encounter */
"S37519S"  =  "1"  /* Primary blast injury of fallopian tube, unspecified, sequela */
"S37521A"  =  "1"  /* Contusion of fallopian tube, unilateral, initial encounter */
"S37521S"  =  "1"  /* Contusion of fallopian tube, unilateral, sequela */
"S37522A"  =  "1"  /* Contusion of fallopian tube, bilateral, initial encounter */
"S37522S"  =  "1"  /* Contusion of fallopian tube, bilateral, sequela */
"S37529A"  =  "1"  /* Contusion of fallopian tube, unspecified, initial encounter */
"S37529S"  =  "1"  /* Contusion of fallopian tube, unspecified, sequela */
"S37531A"  =  "1"  /* Laceration of fallopian tube, unilateral, initial encounter */
"S37531S"  =  "1"  /* Laceration of fallopian tube, unilateral, sequela */
"S37532A"  =  "1"  /* Laceration of fallopian tube, bilateral, initial encounter */
"S37532S"  =  "1"  /* Laceration of fallopian tube, bilateral, sequela */
"S37539A"  =  "1"  /* Laceration of fallopian tube, unspecified, initial encounter */
"S37539S"  =  "1"  /* Laceration of fallopian tube, unspecified, sequela */
"S37591A"  =  "1"  /* Other injury of fallopian tube, unilateral, initial encounter */
"S37591S"  =  "1"  /* Other injury of fallopian tube, unilateral, sequela */
"S37592A"  =  "1"  /* Other injury of fallopian tube, bilateral, initial encounter */
"S37592S"  =  "1"  /* Other injury of fallopian tube, bilateral, sequela */
"S37599A"  =  "1"  /* Other injury of fallopian tube, unspecified, initial encounter */
"S37599S"  =  "1"  /* Other injury of fallopian tube, unspecified, sequela */
"S3760XA"  =  "1"  /* Unspecified injury of uterus, initial encounter */
"S3760XS"  =  "1"  /* Unspecified injury of uterus, sequela */
"S3762XA"  =  "1"  /* Contusion of uterus, initial encounter */
"S3762XS"  =  "1"  /* Contusion of uterus, sequela */
"S3763XA"  =  "1"  /* Laceration of uterus, initial encounter */
"S3763XS"  =  "1"  /* Laceration of uterus, sequela */
"S3769XA"  =  "1"  /* Other injury of uterus, initial encounter */
"S3769XS"  =  "1"  /* Other injury of uterus, sequela */
"S37892A"  =  "1"  /* Contusion of other urinary and pelvic organ, initial encounter */
"S37892S"  =  "1"  /* Contusion of other urinary and pelvic organ, sequela */
"S37893A"  =  "1"  /* Laceration of other urinary and pelvic organ, initial encounter */
"S37893S"  =  "1"  /* Laceration of other urinary and pelvic organ, sequela */
"S37898A"  =  "1"  /* Other injury of other urinary and pelvic organ, initial encounter */
"S37898S"  =  "1"  /* Other injury of other urinary and pelvic organ, sequela */
"S37899A"  =  "1"  /* Unspecified injury of other urinary and pelvic organ, initial encounter */
"S37899S"  =  "1"  /* Unspecified injury of other urinary and pelvic organ, sequela */
"S3790XA"  =  "1"  /* Unspecified injury of unspecified urinary and pelvic organ, initial encounter */
"S3790XS"  =  "1"  /* Unspecified injury of unspecified urinary and pelvic organ, sequela */
"S3792XA"  =  "1"  /* Contusion of unspecified urinary and pelvic organ, initial encounter */
"S3792XS"  =  "1"  /* Contusion of unspecified urinary and pelvic organ, sequela */
"S3793XA"  =  "1"  /* Laceration of unspecified urinary and pelvic organ, initial encounter */
"S3793XS"  =  "1"  /* Laceration of unspecified urinary and pelvic organ, sequela */
"S3799XA"  =  "1"  /* Other injury of unspecified urinary and pelvic organ, initial encounter */
"S3799XS"  =  "1"  /* Other injury of unspecified urinary and pelvic organ, sequela */
"S38211S"  =  "1"  /* Complete traumatic amputation of female external genital organs, sequela */
"S38212S"  =  "1"  /* Partial traumatic amputation of female external genital organs, sequela */
"S383XXA"  =  "1"  /* Transection (partial) of abdomen, initial encounter */
"S383XXS"  =  "1"  /* Transection (partial) of abdomen, sequela */
"S39021A"  =  "1"  /* Laceration of muscle, fascia and tendon of abdomen, initial encounter */
"S39021S"  =  "1"  /* Laceration of muscle, fascia and tendon of abdomen, sequela */
"S39022A"  =  "1"  /* Laceration of muscle, fascia and tendon of lower back, initial encounter */
"S39022S"  =  "1"  /* Laceration of muscle, fascia and tendon of lower back, sequela */
"S39023A"  =  "1"  /* Laceration of muscle, fascia and tendon of pelvis, initial encounter */
"S39023S"  =  "1"  /* Laceration of muscle, fascia and tendon of pelvis, sequela */
"T193XXA"  =  "1"  /* Foreign body in uterus, initial encounter */
"T283XXA"  =  "1"  /* Burn of internal genitourinary organs, initial encounter */
"T288XXA"  =  "1"  /* Corrosion of internal genitourinary organs, initial encounter */
OTHER   = '0' ;

 /* ---------------------------------------------- */
 /* Laminectomy and/or spinal fusion rate (PRLAMI) */
 /*   Utilization (area) Indicator / IQI #29       */
 /* ---------------------------------------------- */

 /* Laminectomy and/or Spinal Fusion */
 /* ICD-10-CM Procedure Codes: */

VALUE $PRLAMIP
"00NW0ZZ"  =  "1"  /* Release Cervical Spinal Cord, Open Approach */
"00NW3ZZ"  =  "1"  /* Release Cervical Spinal Cord, Percutaneous Approach */
"00NW4ZZ"  =  "1"  /* Release Cervical Spinal Cord, Percutaneous Endoscopic Approach */
"00NX0ZZ"  =  "1"  /* Release Thoracic Spinal Cord, Open Approach */
"00NX3ZZ"  =  "1"  /* Release Thoracic Spinal Cord, Percutaneous Approach */
"00NX4ZZ"  =  "1"  /* Release Thoracic Spinal Cord, Percutaneous Endoscopic Approach */
"00NY0ZZ"  =  "1"  /* Release Lumbar Spinal Cord, Open Approach */
"00NY3ZZ"  =  "1"  /* Release Lumbar Spinal Cord, Percutaneous Approach */
"00NY4ZZ"  =  "1"  /* Release Lumbar Spinal Cord, Percutaneous Endoscopic Approach */
"0R530ZZ"  =  "1"  /* Destruction of Cervical Vertebral Disc, Open Approach */
"0R533ZZ"  =  "1"  /* Destruction of Cervical Vertebral Disc, Percutaneous Approach */
"0R534ZZ"  =  "1"  /* Destruction of Cervical Vertebral Disc, Percutaneous Endoscopic Approach */
"0R550ZZ"  =  "1"  /* Destruction of Cervicothoracic Vertebral Disc, Open Approach */
"0R553ZZ"  =  "1"  /* Destruction of Cervicothoracic Vertebral Disc, Percutaneous Approach */
"0R554ZZ"  =  "1"  /* Destruction of Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach */
"0R590ZZ"  =  "1"  /* Destruction of Thoracic Vertebral Disc, Open Approach */
"0R593ZZ"  =  "1"  /* Destruction of Thoracic Vertebral Disc, Percutaneous Approach */
"0R594ZZ"  =  "1"  /* Destruction of Thoracic Vertebral Disc, Percutaneous Endoscopic Approach */
"0R5B0ZZ"  =  "1"  /* Destruction of Thoracolumbar Vertebral Disc, Open Approach */
"0R5B3ZZ"  =  "1"  /* Destruction of Thoracolumbar Vertebral Disc, Percutaneous Approach */
"0R5B4ZZ"  =  "1"  /* Destruction of Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach */
"0RB00ZZ"  =  "1"  /* Excision of Occipital-cervical Joint, Open Approach */
"0RB03ZZ"  =  "1"  /* Excision of Occipital-cervical Joint, Percutaneous Approach */
"0RB04ZZ"  =  "1"  /* Excision of Occipital-cervical Joint, Percutaneous Endoscopic Approach */
"0RB10ZZ"  =  "1"  /* Excision of Cervical Vertebral Joint, Open Approach */
"0RB13ZZ"  =  "1"  /* Excision of Cervical Vertebral Joint, Percutaneous Approach */
"0RB14ZZ"  =  "1"  /* Excision of Cervical Vertebral Joint, Percutaneous Endoscopic Approach */
"0RB30ZZ"  =  "1"  /* Excision of Cervical Vertebral Disc, Open Approach */
"0RB33ZZ"  =  "1"  /* Excision of Cervical Vertebral Disc, Percutaneous Approach */
"0RB34ZZ"  =  "1"  /* Excision of Cervical Vertebral Disc, Percutaneous Endoscopic Approach */
"0RB40ZZ"  =  "1"  /* Excision of Cervicothoracic Vertebral Joint, Open Approach */
"0RB43ZZ"  =  "1"  /* Excision of Cervicothoracic Vertebral Joint, Percutaneous Approach */
"0RB44ZZ"  =  "1"  /* Excision of Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RB50ZZ"  =  "1"  /* Excision of Cervicothoracic Vertebral Disc, Open Approach */
"0RB53ZZ"  =  "1"  /* Excision of Cervicothoracic Vertebral Disc, Percutaneous Approach */
"0RB54ZZ"  =  "1"  /* Excision of Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach */
"0RB60ZZ"  =  "1"  /* Excision of Thoracic Vertebral Joint, Open Approach */
"0RB63ZZ"  =  "1"  /* Excision of Thoracic Vertebral Joint, Percutaneous Approach */
"0RB64ZZ"  =  "1"  /* Excision of Thoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RB90ZZ"  =  "1"  /* Excision of Thoracic Vertebral Disc, Open Approach */
"0RB93ZZ"  =  "1"  /* Excision of Thoracic Vertebral Disc, Percutaneous Approach */
"0RB94ZZ"  =  "1"  /* Excision of Thoracic Vertebral Disc, Percutaneous Endoscopic Approach */
"0RBA0ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Joint, Open Approach */
"0RBA3ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Joint, Percutaneous Approach */
"0RBA4ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0RBB0ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Disc, Open Approach */
"0RBB3ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Disc, Percutaneous Approach */
"0RBB4ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach */
"0RG0070"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG0071"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG007J"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG00A0"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG00A1"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0RG00AJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0RG00J0"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG00J1"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG00JJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG00K0"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG00K1"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG00KJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG00Z0"  =  "1"  /* Fusion of Occipital-cervical Joint, Anterior Approach, Anterior Column, Open Approach */
"0RG00Z1"  =  "1"  /* Fusion of Occipital-cervical Joint, Posterior Approach, Posterior Column, Open Approach */
"0RG00ZJ"  =  "1"  /* Fusion of Occipital-cervical Joint, Posterior Approach, Anterior Column, Open Approach */
"0RG0370"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG0371"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG037J"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG03A0"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG03A1"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG03AJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG03J0"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG03J1"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG03JJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG03K0"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG03K1"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG03KJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG03Z0"  =  "1"  /* Fusion of Occipital-cervical Joint, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG03Z1"  =  "1"  /* Fusion of Occipital-cervical Joint, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG03ZJ"  =  "1"  /* Fusion of Occipital-cervical Joint, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG0470"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG0471"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG047J"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG04A0"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG04A1"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG04AJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG04J0"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG04J1"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG04JJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG04K0"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG04K1"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG04KJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG04Z0"  =  "1"  /* Fusion of Occipital-cervical Joint, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG04Z1"  =  "1"  /* Fusion of Occipital-cervical Joint, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG04ZJ"  =  "1"  /* Fusion of Occipital-cervical Joint, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG1070"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG1071"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG107J"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG10A0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG10A1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0RG10AJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0RG10J0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG10J1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG10JJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG10K0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG10K1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG10KJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG10Z0"  =  "1"  /* Fusion of Cervical Vertebral Joint, Anterior Approach, Anterior Column, Open Approach */
"0RG10Z1"  =  "1"  /* Fusion of Cervical Vertebral Joint, Posterior Approach, Posterior Column, Open Approach */
"0RG10ZJ"  =  "1"  /* Fusion of Cervical Vertebral Joint, Posterior Approach, Anterior Column, Open Approach */
"0RG1370"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG1371"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG137J"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG13A0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG13A1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG13AJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG13J0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG13J1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG13JJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG13K0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG13K1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG13KJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG13Z0"  =  "1"  /* Fusion of Cervical Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG13Z1"  =  "1"  /* Fusion of Cervical Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG13ZJ"  =  "1"  /* Fusion of Cervical Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG1470"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG1471"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG147J"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG14A0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG14A1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG14AJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG14J0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG14J1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG14JJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG14K0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG14K1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG14KJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG14Z0"  =  "1"  /* Fusion of Cervical Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG14Z1"  =  "1"  /* Fusion of Cervical Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG14ZJ"  =  "1"  /* Fusion of Cervical Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG2070"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG2071"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG207J"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG20A0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG20A1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0RG20AJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0RG20J0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG20J1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG20JJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG20K0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG20K1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG20KJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG20Z0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Anterior Approach, Anterior Column, Open Approach */
"0RG20Z1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Posterior Approach, Posterior Column, Open Approach */
"0RG20ZJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Posterior Approach, Anterior Column, Open Approach */
"0RG2370"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG2371"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG237J"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG23A0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG23A1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG23AJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG23J0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG23J1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG23JJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG23K0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG23K1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG23KJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG23Z0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG23Z1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG23ZJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG2470"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG2471"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG247J"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG24A0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG24A1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG24AJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG24J0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG24J1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG24JJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG24K0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG24K1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG24KJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG24Z0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG24Z1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG24ZJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG4070"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG4071"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG407J"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG40A0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG40A1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0RG40AJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0RG40J0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG40J1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG40JJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG40K0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG40K1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG40KJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG40Z0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Anterior Approach, Anterior Column, Open Approach */
"0RG40Z1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Posterior Approach, Posterior Column, Open Approach */
"0RG40ZJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Posterior Approach, Anterior Column, Open Approach */
"0RG4370"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG4371"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG437J"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG43A0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG43A1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG43AJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG43J0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG43J1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG43JJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG43K0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG43K1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG43KJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG43Z0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG43Z1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG43ZJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG4470"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG4471"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG447J"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG44A0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG44A1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG44AJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG44J0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG44J1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG44JJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG44K0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG44K1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG44KJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG44Z0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG44Z1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG44ZJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG6070"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG6071"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG607J"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG60A0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG60A1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0RG60AJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0RG60J0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG60J1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG60JJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG60K0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG60K1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG60KJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG60Z0"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Anterior Approach, Anterior Column, Open Approach */
"0RG60Z1"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Posterior Approach, Posterior Column, Open Approach */
"0RG60ZJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Posterior Approach, Anterior Column, Open Approach */
"0RG6370"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG6371"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG637J"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG63A0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG63A1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG63AJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG63J0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG63J1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG63JJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG63K0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG63K1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG63KJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG63Z0"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG63Z1"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG63ZJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG6470"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG6471"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG647J"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64A0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64A1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG64AJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64J0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64J1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG64JJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64K0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64K1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG64KJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64Z0"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64Z1"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG64ZJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG7070"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG7071"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG707J"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG70A0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG70A1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0RG70AJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0RG70J0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG70J1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG70JJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG70K0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG70K1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG70KJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG70Z0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Open Approach */
"0RG70Z1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Posterior Approach, Posterior Column, Open Approach */
"0RG70ZJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Posterior Approach, Anterior Column, Open Approach */
"0RG7370"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG7371"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG737J"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG73A0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG73A1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG73AJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG73J0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG73J1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG73JJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG73K0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG73K1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG73KJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG73Z0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG73Z1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG73ZJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG7470"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG7471"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG747J"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74A0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74A1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG74AJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74J0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74J1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG74JJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74K0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74K1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG74KJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74Z0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74Z1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG74ZJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG8070"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG8071"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG807J"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG80A0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG80A1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0RG80AJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0RG80J0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG80J1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG80JJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG80K0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG80K1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG80KJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG80Z0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Open Approach */
"0RG80Z1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Posterior Approach, Posterior Column, Open Approach */
"0RG80ZJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Posterior Approach, Anterior Column, Open Approach */
"0RG8370"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG8371"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG837J"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG83A0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG83A1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG83AJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG83J0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG83J1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG83JJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG83K0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG83K1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG83KJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG83Z0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG83Z1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG83ZJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG8470"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG8471"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG847J"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84A0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84A1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG84AJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84J0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84J1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG84JJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84K0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84K1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG84KJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84Z0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84Z1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG84ZJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA070"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RGA071"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RGA07J"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RGA0A0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RGA0A1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0RGA0AJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0RGA0J0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RGA0J1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RGA0JJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RGA0K0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RGA0K1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RGA0KJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RGA0Z0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Anterior Approach, Anterior Column, Open Approach */
"0RGA0Z1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Posterior Approach, Posterior Column, Open Approach */
"0RGA0ZJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Posterior Approach, Anterior Column, Open Approach */
"0RGA370"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RGA371"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RGA37J"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3A0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3A1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RGA3AJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3J0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3J1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RGA3JJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3K0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3K1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RGA3KJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3Z0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3Z1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RGA3ZJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RGA470"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA471"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RGA47J"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4A0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4A1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RGA4AJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4J0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4J1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RGA4JJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4K0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4K1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RGA4KJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4Z0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4Z1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RGA4ZJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RT30ZZ"  =  "1"  /* Resection of Cervical Vertebral Disc, Open Approach */
"0RT40ZZ"  =  "1"  /* Resection of Cervicothoracic Vertebral Joint, Open Approach */
"0RT50ZZ"  =  "1"  /* Resection of Cervicothoracic Vertebral Disc, Open Approach */
"0RT90ZZ"  =  "1"  /* Resection of Thoracic Vertebral Disc, Open Approach */
"0RTB0ZZ"  =  "1"  /* Resection of Thoracolumbar Vertebral Disc, Open Approach */
"0S520ZZ"  =  "1"  /* Destruction of Lumbar Vertebral Disc, Open Approach */
"0S523ZZ"  =  "1"  /* Destruction of Lumbar Vertebral Disc, Percutaneous Approach */
"0S524ZZ"  =  "1"  /* Destruction of Lumbar Vertebral Disc, Percutaneous Endoscopic Approach */
"0S540ZZ"  =  "1"  /* Destruction of Lumbosacral Disc, Open Approach */
"0S543ZZ"  =  "1"  /* Destruction of Lumbosacral Disc, Percutaneous Approach */
"0S544ZZ"  =  "1"  /* Destruction of Lumbosacral Disc, Percutaneous Endoscopic Approach */
"0SB00ZZ"  =  "1"  /* Excision of Lumbar Vertebral Joint, Open Approach */
"0SB03ZZ"  =  "1"  /* Excision of Lumbar Vertebral Joint, Percutaneous Approach */
"0SB04ZZ"  =  "1"  /* Excision of Lumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0SB20ZZ"  =  "1"  /* Excision of Lumbar Vertebral Disc, Open Approach */
"0SB23ZZ"  =  "1"  /* Excision of Lumbar Vertebral Disc, Percutaneous Approach */
"0SB24ZZ"  =  "1"  /* Excision of Lumbar Vertebral Disc, Percutaneous Endoscopic Approach */
"0SB30ZZ"  =  "1"  /* Excision of Lumbosacral Joint, Open Approach */
"0SB33ZZ"  =  "1"  /* Excision of Lumbosacral Joint, Percutaneous Approach */
"0SB34ZZ"  =  "1"  /* Excision of Lumbosacral Joint, Percutaneous Endoscopic Approach */
"0SB40ZZ"  =  "1"  /* Excision of Lumbosacral Disc, Open Approach */
"0SB43ZZ"  =  "1"  /* Excision of Lumbosacral Disc, Percutaneous Approach */
"0SB44ZZ"  =  "1"  /* Excision of Lumbosacral Disc, Percutaneous Endoscopic Approach */
"0SG0070"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG0071"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG007J"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG00A0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0SG00A1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0SG00AJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0SG00J0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG00J1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG00JJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG00K0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG00K1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG00KJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG00Z0"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Anterior Approach, Anterior Column, Open Approach */
"0SG00Z1"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Posterior Approach, Posterior Column, Open Approach */
"0SG00ZJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Posterior Approach, Anterior Column, Open Approach */
"0SG0370"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG0371"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG037J"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG03A0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG03A1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG03AJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG03J0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG03J1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG03JJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG03K0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG03K1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG03KJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG03Z0"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG03Z1"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG03ZJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG0470"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG0471"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG047J"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG04A0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG04A1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG04AJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG04J0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG04J1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG04JJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG04K0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG04K1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG04KJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG04Z0"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG04Z1"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG04ZJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG1070"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG1071"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG107J"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG10A0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0SG10A1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0SG10AJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0SG10J0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG10J1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG10JJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG10K0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG10K1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG10KJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG10Z0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Anterior Approach, Anterior Column, Open Approach */
"0SG10Z1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Posterior Approach, Posterior Column, Open Approach */
"0SG10ZJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Posterior Approach, Anterior Column, Open Approach */
"0SG1370"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG1371"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG137J"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG13A0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG13A1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG13AJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG13J0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG13J1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG13JJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG13K0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG13K1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG13KJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG13Z0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG13Z1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG13ZJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG1470"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG1471"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG147J"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG14A0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG14A1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG14AJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG14J0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG14J1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG14JJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG14K0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG14K1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG14KJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG14Z0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG14Z1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG14ZJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG3070"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG3071"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG307J"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG30A0"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0SG30A1"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0SG30AJ"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0SG30J0"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG30J1"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG30JJ"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG30K0"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG30K1"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG30KJ"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG30Z0"  =  "1"  /* Fusion of Lumbosacral Joint, Anterior Approach, Anterior Column, Open Approach */
"0SG30Z1"  =  "1"  /* Fusion of Lumbosacral Joint, Posterior Approach, Posterior Column, Open Approach */
"0SG30ZJ"  =  "1"  /* Fusion of Lumbosacral Joint, Posterior Approach, Anterior Column, Open Approach */
"0SG3370"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG3371"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG337J"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG33A0"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG33A1"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG33AJ"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG33J0"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG33J1"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG33JJ"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG33K0"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG33K1"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG33KJ"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG33Z0"  =  "1"  /* Fusion of Lumbosacral Joint, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG33Z1"  =  "1"  /* Fusion of Lumbosacral Joint, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG33ZJ"  =  "1"  /* Fusion of Lumbosacral Joint, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG3470"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG3471"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG347J"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG34A0"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG34A1"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG34AJ"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG34J0"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG34J1"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG34JJ"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG34K0"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG34K1"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG34KJ"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG34Z0"  =  "1"  /* Fusion of Lumbosacral Joint, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG34Z1"  =  "1"  /* Fusion of Lumbosacral Joint, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG34ZJ"  =  "1"  /* Fusion of Lumbosacral Joint, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0ST20ZZ"  =  "1"  /* Resection of Lumbar Vertebral Disc, Open Approach */
"0ST40ZZ"  =  "1"  /* Resection of Lumbosacral Disc, Open Approach */
OTHER  = '0' ;

 * ----------------------------------------------------- ;
 * ---   DRG FORMATS      ------------------------------ ;
 * ----------------------------------------------------- ;

 /* CMS-DRG (V24) TO MDC */;

VALUE MDCFMT
001 = 01
002 = 01
003 = 01
004 = 01
005 = 01
006 = 01
007 = 01
008 = 01
009 = 01
010 = 01
011 = 01
012 = 01
013 = 01
014 = 01
015 = 01
016 = 01
017 = 01
018 = 01
019 = 01
020 = 01
021 = 01
022 = 01
023 = 01
024 = 01
025 = 01
026 = 01
027 = 01
028 = 01
029 = 01
030 = 01
031 = 01
032 = 01
033 = 01
034 = 01
035 = 01
036 = 02
037 = 02
038 = 02
039 = 02
040 = 02
041 = 02
042 = 02
043 = 02
044 = 02
045 = 02
046 = 02
047 = 02
048 = 02
049 = 03
050 = 03
051 = 03
052 = 03
053 = 03
054 = 03
055 = 03
056 = 03
057 = 03
058 = 03
059 = 03
060 = 03
061 = 03
062 = 03
063 = 03
064 = 03
065 = 03
066 = 03
067 = 03
068 = 03
069 = 03
070 = 03
071 = 03
072 = 03
073 = 03
074 = 03
075 = 04
076 = 04
077 = 04
078 = 04
079 = 04
080 = 04
081 = 04
082 = 04
083 = 04
084 = 04
085 = 04
086 = 04
087 = 04
088 = 04
089 = 04
090 = 04
091 = 04
092 = 04
093 = 04
094 = 04
095 = 04
096 = 04
097 = 04
098 = 04
099 = 04
100 = 04
101 = 04
102 = 04
103 = 77
104 = 05
105 = 05
106 = 05
107 = 05
108 = 05
109 = 05
110 = 05
111 = 05
112 = 05
113 = 05
114 = 05
115 = 05
116 = 05
117 = 05
118 = 05
119 = 05
120 = 05
121 = 05
122 = 05
123 = 05
124 = 05
125 = 05
126 = 05
127 = 05
128 = 05
129 = 05
130 = 05
131 = 05
132 = 05
133 = 05
134 = 05
135 = 05
136 = 05
137 = 05
138 = 05
139 = 05
140 = 05
141 = 05
142 = 05
143 = 05
144 = 05
145 = 05
146 = 06
147 = 06
148 = 06
149 = 06
150 = 06
151 = 06
152 = 06
153 = 06
154 = 06
155 = 06
156 = 06
157 = 06
158 = 06
159 = 06
160 = 06
161 = 06
162 = 06
163 = 06
164 = 06
165 = 06
166 = 06
167 = 06
168 = 03
169 = 03
170 = 06
171 = 06
172 = 06
173 = 06
174 = 06
175 = 06
176 = 06
177 = 06
178 = 06
179 = 06
180 = 06
181 = 06
182 = 06
183 = 06
184 = 06
185 = 03
186 = 03
187 = 03
188 = 06
189 = 06
190 = 06
191 = 07
192 = 07
193 = 07
194 = 07
195 = 07
196 = 07
197 = 07
198 = 07
199 = 07
200 = 07
201 = 07
202 = 07
203 = 07
204 = 07
205 = 07
206 = 07
207 = 07
208 = 07
209 = 08
210 = 08
211 = 08
212 = 08
213 = 08
214 = 08
215 = 08
216 = 08
217 = 08
218 = 08
219 = 08
220 = 08
221 = 08
222 = 08
223 = 08
224 = 08
225 = 08
226 = 08
227 = 08
228 = 08
229 = 08
230 = 08
231 = 08
232 = 08
233 = 08
234 = 08
235 = 08
236 = 08
237 = 08
238 = 08
239 = 08
240 = 08
241 = 08
242 = 08
243 = 08
244 = 08
245 = 08
246 = 08
247 = 08
248 = 08
249 = 08
250 = 08
251 = 08
252 = 08
253 = 08
254 = 08
255 = 08
256 = 08
257 = 09
258 = 09
259 = 09
260 = 09
261 = 09
262 = 09
263 = 09
264 = 09
265 = 09
266 = 09
267 = 09
268 = 09
269 = 09
270 = 09
271 = 09
272 = 09
273 = 09
274 = 09
275 = 09
276 = 09
277 = 09
278 = 09
279 = 09
280 = 09
281 = 09
282 = 09
283 = 09
284 = 09
285 = 10
286 = 10
287 = 10
288 = 10
289 = 10
290 = 10
291 = 10
292 = 10
293 = 10
294 = 10
295 = 10
296 = 10
297 = 10
298 = 10
299 = 10
300 = 10
301 = 10
302 = 11
303 = 11
304 = 11
305 = 11
306 = 11
307 = 11
308 = 11
309 = 11
310 = 11
311 = 11
312 = 11
313 = 11
314 = 11
315 = 11
316 = 11
317 = 11
318 = 11
319 = 11
320 = 11
321 = 11
322 = 11
323 = 11
324 = 11
325 = 11
326 = 11
327 = 11
328 = 11
329 = 11
330 = 11
331 = 11
332 = 11
333 = 11
334 = 12
335 = 12
336 = 12
337 = 12
338 = 12
339 = 12
340 = 12
341 = 12
342 = 12
343 = 12
344 = 12
345 = 12
346 = 12
347 = 12
348 = 12
349 = 12
350 = 12
351 = 12
352 = 12
353 = 13
354 = 13
355 = 13
356 = 13
357 = 13
358 = 13
359 = 13
360 = 13
361 = 13
362 = 13
363 = 13
364 = 13
365 = 13
366 = 13
367 = 13
368 = 13
369 = 13
370 = 14
371 = 14
372 = 14
373 = 14
374 = 14
375 = 14
376 = 14
377 = 14
378 = 14
379 = 14
380 = 14
381 = 14
382 = 14
383 = 14
384 = 14
385 = 15
386 = 15
387 = 15
388 = 15
389 = 15
390 = 15
391 = 15
392 = 16
393 = 16
394 = 16
395 = 16
396 = 16
397 = 16
398 = 16
399 = 16
400 = 17
401 = 17
402 = 17
403 = 17
404 = 17
405 = 17
406 = 17
407 = 17
408 = 17
409 = 17
410 = 17
411 = 17
412 = 17
413 = 17
414 = 17
415 = 18
416 = 18
417 = 18
418 = 18
419 = 18
420 = 18
421 = 18
422 = 18
423 = 18
424 = 19
425 = 19
426 = 19
427 = 19
428 = 19
429 = 19
430 = 19
431 = 19
432 = 19
433 = 20
434 = 20
435 = 20
436 = 20
437 = 20
438 = 20
439 = 21
440 = 21
441 = 21
442 = 21
443 = 21
444 = 21
445 = 21
446 = 21
447 = 21
448 = 21
449 = 21
450 = 21
451 = 21
452 = 21
453 = 21
454 = 21
455 = 21
456 = 22
457 = 22
458 = 22
459 = 22
460 = 22
461 = 23
462 = 23
463 = 23
464 = 23
465 = 23
466 = 23
467 = 23
468 = 88
469 = 88
470 = 88
471 = 08
472 = 22
473 = 17
474 = 04
475 = 04
476 = 88
477 = 88
478 = 05
479 = 05
480 = 77
481 = 77
482 = 77
483 = 77
484 = 24
485 = 24
486 = 24
487 = 24
488 = 25
489 = 25
490 = 25
491 = 08
492 = 17
493 = 07
494 = 07
495 = 77
496 = 08
497 = 08
498 = 08
499 = 08
500 = 08
501 = 08
502 = 08
503 = 08
504 = 22
505 = 22
506 = 22
507 = 22
508 = 22
509 = 22
510 = 22
511 = 22
512 = 77
513 = 77
514 = 05
515 = 05
516 = 05
517 = 05
518 = 05
519 = 08
520 = 08
521 = 20
522 = 20
523 = 20
524 = 01
525 = 05
526 = 05
527 = 05
528 = 01
529 = 01
530 = 01
531 = 01
532 = 01
533 = 01
534 = 01
535 = 05
536 = 05
537 = 08
538 = 08
539 = 17
540 = 17
541 = 77
542 = 77
543 = 01
544 = 08
545 = 08
546 = 08
547 = 05
548 = 05
549 = 05
550 = 05
551 = 05
552 = 05
553 = 05
554 = 05
555 = 05
556 = 05
557 = 05
558 = 05
559 = 01
560 = 01
561 = 01
562 = 01
563 = 01
564 = 01
565 = 04
566 = 04
567 = 06
568 = 06
569 = 06
570 = 06
571 = 06
572 = 06
573 = 11
574 = 16
575 = 18
576 = 18
577 = 01
578 = 18
579 = 18
OTHER  = 99 ;

 /* MS-DRG (V25) TO MDC */;

VALUE MDCF2T
001 = 77
002 = 77
003 = 77
004 = 77
005 = 77
006 = 77
007 = 77
008 = 77
009 = 77
010 = 77
011 = 77
012 = 77
013 = 77
014 = 77   
015 = 77  
016 = 77
017 = 77
020 = 01
021 = 01
022 = 01
023 = 01
024 = 01
025 = 01
026 = 01
027 = 01
028 = 01
029 = 01
030 = 01
031 = 01
032 = 01
033 = 01
034 = 01
035 = 01
036 = 01
037 = 01
038 = 01
039 = 01
040 = 01
041 = 01
042 = 01
052 = 01
053 = 01
054 = 01
055 = 01
056 = 01
057 = 01
058 = 01
059 = 01
060 = 01
061 = 01
062 = 01
063 = 01
064 = 01
065 = 01
066 = 01
067 = 01
068 = 01
069 = 01
070 = 01
071 = 01
072 = 01
073 = 01
074 = 01
075 = 01
076 = 01
077 = 01
078 = 01
079 = 01
080 = 01
081 = 01
082 = 01
083 = 01
084 = 01
085 = 01
086 = 01
087 = 01
088 = 01
089 = 01
090 = 01
091 = 01
092 = 01
093 = 01
094 = 01
095 = 01
096 = 01
097 = 01
098 = 01
099 = 01
100 = 01
101 = 01
102 = 01
103 = 01
113 = 02
114 = 02
115 = 02
116 = 02
117 = 02
121 = 02
122 = 02
123 = 02
124 = 02
125 = 02
129 = 03
130 = 03
131 = 03
132 = 03
133 = 03
134 = 03
135 = 03
136 = 03
137 = 03
138 = 03
139 = 03
146 = 03
147 = 03
148 = 03
149 = 03
150 = 03
151 = 03
152 = 03
153 = 03
154 = 03
155 = 03
156 = 03
157 = 03
158 = 03
159 = 03
163 = 04
164 = 04
165 = 04
166 = 04
167 = 04
168 = 04
175 = 04
176 = 04
177 = 04
178 = 04
179 = 04
180 = 04
181 = 04
182 = 04
183 = 04
184 = 04
185 = 04
186 = 04
187 = 04
188 = 04
189 = 04
190 = 04
191 = 04
192 = 04
193 = 04
194 = 04
195 = 04
196 = 04
197 = 04
198 = 04
199 = 04
200 = 04
201 = 04
202 = 04
203 = 04
204 = 04
205 = 04
206 = 04
207 = 04
208 = 04
215 = 05
216 = 05
217 = 05
218 = 05
219 = 05
220 = 05
221 = 05
222 = 05
223 = 05
224 = 05
225 = 05
226 = 05
227 = 05
228 = 05
229 = 05
230 = 05
231 = 05
232 = 05
233 = 05
234 = 05
235 = 05
236 = 05
237 = 05
238 = 05
239 = 05
240 = 05
241 = 05
242 = 05
243 = 05
244 = 05
245 = 05
246 = 05
247 = 05
248 = 05
249 = 05
250 = 05
251 = 05
252 = 05
253 = 05
254 = 05
255 = 05
256 = 05
257 = 05
258 = 05
259 = 05
260 = 05
261 = 05
262 = 05
263 = 05
264 = 05
265 = 05 
266 = 05
267 = 05
280 = 05
281 = 05
282 = 05
283 = 05
284 = 05
285 = 05
286 = 05
287 = 05
288 = 05
289 = 05
290 = 05
291 = 05
292 = 05
293 = 05
294 = 05
295 = 05
296 = 05
297 = 05
298 = 05
299 = 05
300 = 05
301 = 05
302 = 05
303 = 05
304 = 05
305 = 05
306 = 05
307 = 05
308 = 05
309 = 05
310 = 05
311 = 05
312 = 05
313 = 05
314 = 05
315 = 05
316 = 05
326 = 06
327 = 06
328 = 06
329 = 06
330 = 06
331 = 06
332 = 06
333 = 06
334 = 06
335 = 06
336 = 06
337 = 06
338 = 06
339 = 06
340 = 06
341 = 06
342 = 06
343 = 06
344 = 06
345 = 06
346 = 06
347 = 06
348 = 06
349 = 06
350 = 06
351 = 06
352 = 06
353 = 06
354 = 06
355 = 06
356 = 06
357 = 06
358 = 06
368 = 06
369 = 06
370 = 06
371 = 06
372 = 06
373 = 06
374 = 06
375 = 06
376 = 06
377 = 06
378 = 06
379 = 06
380 = 06
381 = 06
382 = 06
383 = 06
384 = 06
385 = 06
386 = 06
387 = 06
388 = 06
389 = 06
390 = 06
391 = 06
392 = 06
393 = 06
394 = 06
395 = 06
405 = 07
406 = 07
407 = 07
408 = 07
409 = 07
410 = 07
411 = 07
412 = 07
413 = 07
414 = 07
415 = 07
416 = 07
417 = 07
418 = 07
419 = 07
420 = 07
421 = 07
422 = 07
423 = 07
424 = 07
425 = 07
432 = 07
433 = 07
434 = 07
435 = 07
436 = 07
437 = 07
438 = 07
439 = 07
440 = 07
441 = 07
442 = 07
443 = 07
444 = 07
445 = 07
446 = 07
453 = 08
454 = 08
455 = 08
456 = 08
457 = 08
458 = 08
459 = 08
460 = 08
461 = 08
462 = 08
463 = 08
464 = 08
465 = 08
466 = 08
467 = 08
468 = 08
469 = 08
470 = 08
471 = 08
472 = 08
473 = 08
474 = 08
475 = 08
476 = 08
477 = 08
478 = 08
479 = 08
480 = 08
481 = 08
482 = 08
483 = 08
484 = 08
485 = 08
486 = 08
487 = 08
488 = 08
489 = 08
490 = 08
491 = 08
492 = 08
493 = 08
494 = 08
495 = 08
496 = 08
497 = 08
498 = 08
499 = 08
500 = 08
501 = 08
502 = 08
503 = 08
504 = 08
505 = 08
506 = 08
507 = 08
508 = 08
509 = 08
510 = 08
511 = 08
512 = 08
513 = 08
514 = 08
515 = 08
516 = 08
517 = 08
518 = 08
519 = 08
520 = 08
533 = 08
534 = 08
535 = 08
536 = 08
537 = 08
538 = 08
539 = 08
540 = 08
541 = 08
542 = 08
543 = 08
544 = 08
545 = 08
546 = 08
547 = 08
548 = 08
549 = 08
550 = 08
551 = 08
552 = 08
553 = 08
554 = 08
555 = 08
556 = 08
557 = 08
558 = 08
559 = 08
560 = 08
561 = 08
562 = 08
563 = 08
564 = 08
565 = 08
566 = 08
570 = 09
571 = 09
572 = 09
573 = 09
574 = 09
575 = 09
576 = 09
577 = 09
578 = 09
579 = 09
580 = 09
581 = 09
582 = 09
583 = 09
584 = 09
585 = 09
592 = 09
593 = 09
594 = 09
595 = 09
596 = 09
597 = 09
598 = 09
599 = 09
600 = 09
601 = 09
602 = 09
603 = 09
604 = 09
605 = 09
606 = 09
607 = 09
614 = 10
615 = 10
616 = 10
617 = 10
618 = 10
619 = 10
620 = 10
621 = 10
622 = 10
623 = 10
624 = 10
625 = 10
626 = 10
627 = 10
628 = 10
629 = 10
630 = 10
637 = 10
638 = 10
639 = 10
640 = 10
641 = 10
642 = 10
643 = 10
644 = 10
645 = 10
652 = 11
653 = 11
654 = 11
655 = 11
656 = 11
657 = 11
658 = 11
659 = 11
660 = 11
661 = 11
662 = 11
663 = 11
664 = 11
665 = 11
666 = 11
667 = 11
668 = 11
669 = 11
670 = 11
671 = 11
672 = 11
673 = 11
674 = 11
675 = 11
682 = 11
683 = 11
684 = 11
685 = 11
686 = 11
687 = 11
688 = 11
689 = 11
690 = 11
691 = 11
692 = 11
693 = 11
694 = 11
695 = 11
696 = 11
697 = 11
698 = 11
699 = 11
700 = 11
707 = 12
708 = 12
709 = 12
710 = 12
711 = 12
712 = 12
713 = 12
714 = 12
715 = 12
716 = 12
717 = 12
718 = 12
722 = 12
723 = 12
724 = 12
725 = 12
726 = 12
727 = 12
728 = 12
729 = 12
730 = 12
734 = 13
735 = 13
736 = 13
737 = 13
738 = 13
739 = 13
740 = 13
741 = 13
742 = 13
743 = 13
744 = 13
745 = 13
746 = 13
747 = 13
748 = 13
749 = 13
750 = 13
754 = 13
755 = 13
756 = 13
757 = 13
758 = 13
759 = 13
760 = 13
761 = 13
765 = 14
766 = 14
767 = 14
768 = 14
769 = 14
770 = 14
774 = 14
775 = 14
776 = 14
777 = 14
778 = 14
779 = 14
780 = 14
781 = 14
782 = 14
789 = 15
790 = 15
791 = 15
792 = 15
793 = 15
794 = 15
795 = 15
799 = 16
800 = 16
801 = 16
802 = 16
803 = 16
804 = 16
808 = 16
809 = 16
810 = 16
811 = 16
812 = 16
813 = 16
814 = 16
815 = 16
816 = 16
820 = 17
821 = 17
822 = 17
823 = 17
824 = 17
825 = 17
826 = 17
827 = 17
828 = 17
829 = 17
830 = 17
834 = 17
835 = 17
836 = 17
837 = 17
838 = 17
839 = 17
840 = 17
841 = 17
842 = 17
843 = 17
844 = 17
845 = 17
846 = 17
847 = 17
848 = 17
849 = 17
853 = 18
854 = 18
855 = 18
856 = 18
857 = 18
858 = 18
862 = 18
863 = 18
864 = 18
865 = 18
866 = 18
867 = 18
868 = 18
869 = 18
870 = 18
871 = 18
872 = 18
876 = 19
880 = 19
881 = 19
882 = 19
883 = 19
884 = 19
885 = 19
886 = 19
887 = 19
894 = 20
895 = 20
896 = 20
897 = 20
901 = 21
902 = 21
903 = 21
904 = 21
905 = 21
906 = 21
907 = 21
908 = 21
909 = 21
913 = 21
914 = 21
915 = 21
916 = 21
917 = 21
918 = 21
919 = 21
920 = 21
921 = 21
922 = 21
923 = 21
927 = 22
928 = 22
929 = 22
933 = 22
934 = 22
935 = 22
939 = 23
940 = 23
941 = 23
945 = 23
946 = 23
947 = 23
948 = 23
949 = 23
950 = 23
951 = 23
955 = 24
956 = 24
957 = 24
958 = 24
959 = 24
963 = 24
964 = 24
965 = 24
969 = 25
970 = 25
974 = 25
975 = 25
976 = 25
977 = 25
981 = 88
982 = 88
983 = 88
984 = 88
985 = 88
986 = 88
987 = 88
988 = 88
989 = 88
998 = 88
999 = 88
OTHER  = 99 ;

 * ----------------------------------------------------- ;
 * ---   AREA FORMATS     ------------------------------ ;
 * ---   Translates FIPS State and County Code (SSCCC)   ;
 * ---   to MSA (Metropolitan Statistical Area)          ;
 * ----------------------------------------------------- ;

/* MODIFIED FIPS */

VALUE $M1AREA
'02013' = '02001'
'02016' = '02001'
'02020' = '02001'
'02050' = '02001'
'02060' = '02001'
'02068' = '02001'
'02070' = '02001'
'02090' = '02001'
'02100' = '02001'
'02110' = '02001'
'02122' = '02001'
'02130' = '02001'
'02150' = '02001'
'02164' = '02001'
'02170' = '02001'
'02180' = '02001'
'02185' = '02001'
'02188' = '02001'
'02201' = '02001'
'02220' = '02001'
'02232' = '02001'
'02240' = '02001'
'02261' = '02001'
'02270' = '02001'
'02280' = '02001'
'02282' = '02001'
'02290' = '02001'
'15005' = '15009'
'24510' = '24007'
'29510' = '29191'
'32510' = '32025'
'51515' = '51019'
'51520' = '51119'
'51530' = '51163'
'51540' = '51003'
'51560' = '51005'
'51570' = '51041'
'51580' = '51005'
'51590' = '51143'
'51595' = '51081'
'51600' = '51059'
'51610' = '51059'
'51620' = '51175'
'51630' = '51177'
'51640' = '51077'
'51660' = '51165'
'51670' = '51149'
'51678' = '51163'
'51680' = '51031'
'51683' = '51153'
'51685' = '51153'
'51690' = '51089'
'51710' = '51129'
'51720' = '51195'
'51730' = '51053'
'51735' = '51199'
'51740' = '51129'
'51750' = '51121'
'51760' = '51087'
'51770' = '51161'
'51775' = '51161'
'51780' = '51083'
'51790' = '51015'
'51800' = '51123'
'51820' = '51015'
'51830' = '51095'
'51840' = '51069'
;

/* OMB 1999 METRO AREA */

VALUE $M2AREA
'01001' = '5240 '
'01003' = '5160 '
'01009' = '1000 '
'01015' = '0450 '
'01033' = '2650 '
'01045' = '2180 '
'01051' = '5240 '
'01055' = '2880 '
'01069' = '2180 '
'01073' = '1000 '
'01077' = '2650 '
'01079' = '2030 '
'01081' = '0580 '
'01083' = '3440 '
'01089' = '3440 '
'01097' = '5160 '
'01101' = '5240 '
'01103' = '2030 '
'01113' = '1800 '
'01115' = '1000 '
'01117' = '1000 '
'01125' = '8600 '
'02020' = '0380 '
'04005' = '2620 '
'04013' = '6200 '
'04015' = '4120 '
'04019' = '8520 '
'04021' = '6200 '
'04027' = '9360 '
'05007' = '2580 '
'05031' = '3700 '
'05033' = '2720 '
'05035' = '4920 '
'05045' = '4400 '
'05069' = '6240 '
'05085' = '4400 '
'05091' = '8360 '
'05119' = '4400 '
'05125' = '4400 '
'05131' = '2720 '
'05143' = '2580 '
'06001' = '5775 '
'06007' = '1620 '
'06013' = '5775 '
'06017' = '6920 '
'06019' = '2840 '
'06029' = '0680 '
'06037' = '4480 '
'06039' = '2840 '
'06041' = '7360 '
'06047' = '4940 '
'06053' = '7120 '
'06055' = '8720 '
'06059' = '5945 '
'06061' = '6920 '
'06065' = '6780 '
'06067' = '6920 '
'06071' = '6780 '
'06073' = '7320 '
'06075' = '7360 '
'06077' = '8120 '
'06079' = '7460 '
'06081' = '7360 '
'06083' = '7480 '
'06085' = '7400 '
'06087' = '7485 '
'06089' = '6690 '
'06095' = '8720 '
'06097' = '7500 '
'06099' = '5170 '
'06101' = '9340 '
'06107' = '8780 '
'06111' = '8735 '
'06113' = '9270 '
'06115' = '9340 '
'08001' = '2080 '
'08005' = '2080 '
'08013' = '1125 '
'08031' = '2080 '
'08035' = '2080 '
'08041' = '1720 '
'08059' = '2080 '
'08069' = '2670 '
'08077' = '2995 '
'08101' = '6560 '
'08123' = '3060 '
'09001' = '5483 '
'09003' = '3283 '
'09007' = '3283 '
'09009' = '5483 '
'09011' = '5523 '
'09013' = '3283 '
'10001' = '2190 '
'10003' = '9160 '
'11001' = '8840 '
'12001' = '2900 '
'12005' = '6015 '
'12009' = '4900 '
'12011' = '2680 '
'12015' = '6580 '
'12019' = '3600 '
'12021' = '5345 '
'12031' = '3600 '
'12033' = '6080 '
'12035' = '2020 '
'12039' = '8240 '
'12053' = '8280 '
'12057' = '8280 '
'12069' = '5960 '
'12071' = '2700 '
'12073' = '8240 '
'12081' = '7510 '
'12083' = '5790 '
'12085' = '2710 '
'12086' = '5000 '
'12089' = '3600 '
'12091' = '2750 '
'12095' = '5960 '
'12097' = '5960 '
'12099' = '8960 '
'12101' = '8280 '
'12103' = '8280 '
'12105' = '3980 '
'12109' = '3600 '
'12111' = '2710 '
'12113' = '6080 '
'12115' = '7510 '
'12117' = '5960 '
'12127' = '2020 '
'13013' = '0520 '
'13015' = '0520 '
'13021' = '4680 '
'13029' = '7520 '
'13045' = '0520 '
'13047' = '1560 '
'13051' = '7520 '
'13053' = '1800 '
'13057' = '0520 '
'13059' = '0500 '
'13063' = '0520 '
'13067' = '0520 '
'13073' = '0600 '
'13077' = '0520 '
'13083' = '1560 '
'13089' = '0520 '
'13095' = '0120 '
'13097' = '0520 '
'13103' = '7520 '
'13113' = '0520 '
'13117' = '0520 '
'13121' = '0520 '
'13135' = '0520 '
'13145' = '1800 '
'13151' = '0520 '
'13153' = '4680 '
'13169' = '4680 '
'13177' = '0120 '
'13189' = '0600 '
'13195' = '0500 '
'13215' = '1800 '
'13217' = '0520 '
'13219' = '0500 '
'13223' = '0520 '
'13225' = '4680 '
'13227' = '0520 '
'13245' = '0600 '
'13247' = '0520 '
'13255' = '0520 '
'13289' = '4680 '
'13295' = '1560 '
'13297' = '0520 '
'15003' = '3320 '
'16001' = '1080 '
'16005' = '6340 '
'16027' = '1080 '
'17007' = '6880 '
'17019' = '1400 '
'17027' = '7040 '
'17031' = '1600 '
'17037' = '1600 '
'17043' = '1600 '
'17063' = '1600 '
'17073' = '1960 '
'17083' = '7040 '
'17089' = '1600 '
'17091' = '3740 '
'17093' = '1600 '
'17097' = '1600 '
'17111' = '1600 '
'17113' = '1040 '
'17115' = '2040 '
'17119' = '7040 '
'17129' = '7880 '
'17133' = '7040 '
'17141' = '6880 '
'17143' = '6120 '
'17161' = '1960 '
'17163' = '7040 '
'17167' = '7880 '
'17179' = '6120 '
'17197' = '1600 '
'17201' = '6880 '
'17203' = '6120 '
'18001' = '2760 '
'18003' = '2760 '
'18011' = '3480 '
'18019' = '4520 '
'18021' = '8320 '
'18023' = '3920 '
'18029' = '1640 '
'18033' = '2760 '
'18035' = '5280 '
'18039' = '2330 '
'18043' = '4520 '
'18057' = '3480 '
'18059' = '3480 '
'18061' = '4520 '
'18063' = '3480 '
'18067' = '3850 '
'18069' = '2760 '
'18081' = '3480 '
'18089' = '2960 '
'18095' = '3480 '
'18097' = '3480 '
'18105' = '1020 '
'18109' = '3480 '
'18115' = '1640 '
'18127' = '2960 '
'18129' = '2440 '
'18141' = '7800 '
'18143' = '4520 '
'18145' = '3480 '
'18157' = '3920 '
'18159' = '3850 '
'18163' = '2440 '
'18165' = '8320 '
'18167' = '8320 '
'18173' = '2440 '
'18179' = '2760 '
'18183' = '2760 '
'19013' = '8920 '
'19049' = '2120 '
'19061' = '2200 '
'19103' = '3500 '
'19113' = '1360 '
'19153' = '2120 '
'19155' = '5920 '
'19163' = '1960 '
'19181' = '2120 '
'19193' = '7720 '
'20015' = '9040 '
'20045' = '4150 '
'20079' = '9040 '
'20091' = '3760 '
'20103' = '3760 '
'20121' = '3760 '
'20173' = '9040 '
'20177' = '8440 '
'20209' = '3760 '
'21015' = '1640 '
'21017' = '4280 '
'21019' = '3400 '
'21029' = '4520 '
'21037' = '1640 '
'21043' = '3400 '
'21047' = '1660 '
'21049' = '4280 '
'21059' = '5990 '
'21067' = '4280 '
'21077' = '1640 '
'21081' = '1640 '
'21089' = '3400 '
'21101' = '2440 '
'21111' = '4520 '
'21113' = '4280 '
'21117' = '1640 '
'21151' = '4280 '
'21185' = '4520 '
'21191' = '1640 '
'21209' = '4280 '
'21239' = '4280 '
'22001' = '3880 '
'22005' = '0760 '
'22015' = '7680 '
'22017' = '7680 '
'22019' = '3960 '
'22033' = '0760 '
'22051' = '5560 '
'22055' = '3880 '
'22057' = '3350 '
'22063' = '0760 '
'22071' = '5560 '
'22073' = '5200 '
'22075' = '5560 '
'22079' = '0220 '
'22087' = '5560 '
'22089' = '5560 '
'22093' = '5560 '
'22095' = '5560 '
'22097' = '3880 '
'22099' = '3880 '
'22103' = '5560 '
'22109' = '3350 '
'22119' = '7680 '
'22121' = '0760 '
'23001' = '4243 '
'23005' = '6403 '
'23019' = '0733 '
'24001' = '1900 '
'24003' = '0720 '
'24005' = '0720 '
'24009' = '8840 '
'24013' = '0720 '
'24015' = '9160 '
'24017' = '8840 '
'24021' = '8840 '
'24025' = '0720 '
'24027' = '0720 '
'24031' = '8840 '
'24033' = '8840 '
'24035' = '0720 '
'24043' = '3180 '
'24510' = '0720 '
'25001' = '0743 '
'25003' = '6323 '
'25005' = '1123 '
'25009' = '1123 '
'25013' = '8003 '
'25015' = '8003 '
'25017' = '1123 '
'25021' = '1123 '
'25023' = '1123 '
'25025' = '1123 '
'25027' = '1123 '
'26005' = '3000 '
'26017' = '6960 '
'26021' = '0870 '
'26025' = '3720 '
'26037' = '4040 '
'26045' = '4040 '
'26049' = '2640 '
'26065' = '4040 '
'26075' = '3520 '
'26077' = '3720 '
'26081' = '3000 '
'26087' = '2160 '
'26091' = '0440 '
'26093' = '0440 '
'26099' = '2160 '
'26111' = '6960 '
'26115' = '2160 '
'26121' = '3000 '
'26125' = '2160 '
'26139' = '3000 '
'26145' = '6960 '
'26147' = '2160 '
'26159' = '3720 '
'26161' = '0440 '
'26163' = '2160 '
'27003' = '5120 '
'27009' = '6980 '
'27019' = '5120 '
'27025' = '5120 '
'27027' = '2520 '
'27037' = '5120 '
'27053' = '5120 '
'27055' = '3870 '
'27059' = '5120 '
'27109' = '6820 '
'27119' = '2985 '
'27123' = '5120 '
'27137' = '2240 '
'27139' = '5120 '
'27141' = '5120 '
'27145' = '6980 '
'27163' = '5120 '
'27171' = '5120 '
'28033' = '4920 '
'28035' = '3285 '
'28045' = '0920 '
'28047' = '0920 '
'28049' = '3560 '
'28059' = '0920 '
'28073' = '3285 '
'28089' = '3560 '
'28121' = '3560 '
'29003' = '7000 '
'29019' = '1740 '
'29021' = '7000 '
'29037' = '3760 '
'29043' = '7920 '
'29047' = '3760 '
'29049' = '3760 '
'29071' = '7040 '
'29077' = '7920 '
'29095' = '3760 '
'29097' = '3710 '
'29099' = '7040 '
'29107' = '3760 '
'29113' = '7040 '
'29145' = '3710 '
'29165' = '3760 '
'29177' = '3760 '
'29183' = '7040 '
'29189' = '7040 '
'29219' = '7040 '
'29225' = '7920 '
'29510' = '7040 '
'30013' = '3040 '
'30063' = '5140 '
'30111' = '0880 '
'31025' = '5920 '
'31043' = '7720 '
'31055' = '5920 '
'31109' = '4360 '
'31153' = '5920 '
'31177' = '5920 '
'32003' = '4120 '
'32023' = '4120 '
'32031' = '6720 '
'33011' = '1123 '
'33015' = '1123 '
'33017' = '1123 '
'34001' = '0560 '
'34003' = '0875 '
'34005' = '6160 '
'34007' = '6160 '
'34009' = '0560 '
'34011' = '8760 '
'34013' = '5640 '
'34015' = '6160 '
'34017' = '3640 '
'34019' = '5015 '
'34021' = '8480 '
'34023' = '5015 '
'34025' = '5190 '
'34027' = '5640 '
'34029' = '5190 '
'34031' = '0875 '
'34033' = '6160 '
'34035' = '5015 '
'34037' = '5640 '
'34039' = '5640 '
'34041' = '5640 '
'35001' = '0200 '
'35013' = '4100 '
'35028' = '7490 '
'35043' = '0200 '
'35049' = '7490 '
'35061' = '0200 '
'36001' = '0160 '
'36005' = '5600 '
'36007' = '0960 '
'36011' = '8160 '
'36013' = '3610 '
'36015' = '2335 '
'36027' = '2281 '
'36029' = '1280 '
'36037' = '6840 '
'36043' = '8680 '
'36047' = '5600 '
'36051' = '6840 '
'36053' = '8160 '
'36055' = '6840 '
'36057' = '0160 '
'36059' = '5380 '
'36061' = '5600 '
'36063' = '1280 '
'36065' = '8680 '
'36067' = '8160 '
'36069' = '6840 '
'36071' = '5660 '
'36073' = '6840 '
'36075' = '8160 '
'36079' = '5600 '
'36081' = '5600 '
'36083' = '0160 '
'36085' = '5600 '
'36087' = '5600 '
'36091' = '0160 '
'36093' = '0160 '
'36095' = '0160 '
'36103' = '5380 '
'36107' = '0960 '
'36113' = '2975 '
'36115' = '2975 '
'36117' = '6840 '
'36119' = '5600 '
'37001' = '3120 '
'37003' = '3290 '
'37019' = '9200 '
'37021' = '0480 '
'37023' = '3290 '
'37025' = '1520 '
'37027' = '3290 '
'37035' = '3290 '
'37037' = '6640 '
'37051' = '2560 '
'37053' = '5720 '
'37057' = '3120 '
'37059' = '3120 '
'37063' = '6640 '
'37065' = '6895 '
'37067' = '3120 '
'37069' = '6640 '
'37071' = '1520 '
'37081' = '3120 '
'37101' = '6640 '
'37109' = '1520 '
'37115' = '0480 '
'37119' = '1520 '
'37127' = '6895 '
'37129' = '9200 '
'37133' = '3605 '
'37135' = '6640 '
'37147' = '3150 '
'37151' = '3120 '
'37159' = '1520 '
'37169' = '3120 '
'37179' = '1520 '
'37183' = '6640 '
'37191' = '2980 '
'37197' = '3120 '
'38015' = '1010 '
'38017' = '2520 '
'38035' = '2985 '
'38059' = '1010 '
'39003' = '4320 '
'39007' = '1680 '
'39011' = '4320 '
'39013' = '9000 '
'39015' = '1640 '
'39017' = '3200 '
'39019' = '1320 '
'39023' = '2000 '
'39025' = '1640 '
'39029' = '9320 '
'39033' = '4800 '
'39035' = '1680 '
'39041' = '1840 '
'39045' = '1840 '
'39049' = '1840 '
'39051' = '8400 '
'39055' = '1680 '
'39057' = '2000 '
'39061' = '1640 '
'39081' = '8080 '
'39085' = '1680 '
'39087' = '3400 '
'39089' = '1840 '
'39093' = '1680 '
'39095' = '8400 '
'39097' = '1840 '
'39099' = '9320 '
'39103' = '1680 '
'39109' = '2000 '
'39113' = '2000 '
'39129' = '1840 '
'39133' = '0080 '
'39139' = '4800 '
'39151' = '1320 '
'39153' = '0080 '
'39155' = '9320 '
'39165' = '1640 '
'39167' = '6020 '
'39173' = '8400 '
'40017' = '5880 '
'40027' = '5880 '
'40031' = '4200 '
'40037' = '8560 '
'40047' = '2340 '
'40083' = '5880 '
'40087' = '5880 '
'40109' = '5880 '
'40113' = '8560 '
'40125' = '5880 '
'40131' = '8560 '
'40135' = '2720 '
'40143' = '8560 '
'40145' = '8560 '
'41003' = '1890 '
'41005' = '6440 '
'41009' = '6440 '
'41029' = '4890 '
'41039' = '2400 '
'41047' = '7080 '
'41051' = '6440 '
'41053' = '7080 '
'41067' = '6440 '
'41071' = '6440 '
'42003' = '6280 '
'42007' = '6280 '
'42011' = '6680 '
'42013' = '0280 '
'42017' = '6160 '
'42019' = '6280 '
'42021' = '3680 '
'42025' = '0240 '
'42027' = '8050 '
'42029' = '6160 '
'42037' = '7560 '
'42041' = '3240 '
'42043' = '3240 '
'42045' = '6160 '
'42049' = '2360 '
'42051' = '6280 '
'42069' = '7560 '
'42071' = '4000 '
'42075' = '3240 '
'42077' = '0240 '
'42079' = '7560 '
'42081' = '9140 '
'42085' = '7610 '
'42091' = '6160 '
'42095' = '0240 '
'42099' = '3240 '
'42101' = '6160 '
'42103' = '5660 '
'42111' = '3680 '
'42125' = '6280 '
'42129' = '6280 '
'42131' = '7560 '
'42133' = '9280 '
'44001' = '6483 '
'44003' = '6483 '
'44007' = '6483 '
'44009' = '6483 '
'45003' = '0600 '
'45007' = '3160 '
'45015' = '1440 '
'45019' = '1440 '
'45021' = '3160 '
'45035' = '1440 '
'45037' = '0600 '
'45041' = '2655 '
'45045' = '3160 '
'45051' = '5330 '
'45063' = '1760 '
'45077' = '3160 '
'45079' = '1760 '
'45083' = '3160 '
'45085' = '8140 '
'45091' = '1520 '
'46083' = '7760 '
'46099' = '7760 '
'46103' = '6660 '
'47001' = '3840 '
'47009' = '3840 '
'47019' = '3660 '
'47021' = '5360 '
'47023' = '3580 '
'47037' = '5360 '
'47043' = '5360 '
'47047' = '4920 '
'47065' = '1560 '
'47073' = '3660 '
'47093' = '3840 '
'47105' = '3840 '
'47113' = '3580 '
'47115' = '1560 '
'47125' = '1660 '
'47147' = '5360 '
'47149' = '5360 '
'47155' = '3840 '
'47157' = '4920 '
'47163' = '3660 '
'47165' = '5360 '
'47167' = '4920 '
'47171' = '3660 '
'47173' = '3840 '
'47179' = '3660 '
'47187' = '5360 '
'47189' = '5360 '
'48009' = '9080 '
'48021' = '0640 '
'48027' = '3810 '
'48029' = '7240 '
'48037' = '8360 '
'48039' = '1145 '
'48041' = '1260 '
'48055' = '0640 '
'48061' = '1240 '
'48071' = '3360 '
'48085' = '1920 '
'48091' = '7240 '
'48099' = '3810 '
'48113' = '1920 '
'48121' = '1920 '
'48135' = '5800 '
'48139' = '1920 '
'48141' = '2320 '
'48157' = '3360 '
'48167' = '2920 '
'48181' = '7640 '
'48183' = '4420 '
'48187' = '7240 '
'48199' = '0840 '
'48201' = '3360 '
'48203' = '4420 '
'48209' = '0640 '
'48213' = '1920 '
'48215' = '4880 '
'48221' = '2800 '
'48231' = '1920 '
'48245' = '0840 '
'48251' = '2800 '
'48257' = '1920 '
'48291' = '3360 '
'48303' = '4600 '
'48309' = '8800 '
'48329' = '5800 '
'48339' = '3360 '
'48355' = '1880 '
'48361' = '0840 '
'48367' = '2800 '
'48375' = '0320 '
'48381' = '0320 '
'48397' = '1920 '
'48409' = '1880 '
'48423' = '8640 '
'48439' = '2800 '
'48441' = '0040 '
'48451' = '7200 '
'48453' = '0640 '
'48459' = '4420 '
'48469' = '8750 '
'48473' = '3360 '
'48479' = '4080 '
'48485' = '9080 '
'48491' = '0640 '
'48493' = '7240 '
'49011' = '7160 '
'49025' = '2620 '
'49035' = '7160 '
'49049' = '6520 '
'49057' = '7160 '
'50007' = '1303 '
'50011' = '1303 '
'50013' = '1303 '
'51003' = '1540 '
'51009' = '4640 '
'51013' = '8840 '
'51019' = '4640 '
'51023' = '6800 '
'51031' = '4640 '
'51036' = '6760 '
'51041' = '6760 '
'51043' = '8840 '
'51047' = '8840 '
'51053' = '6760 '
'51059' = '8840 '
'51061' = '8840 '
'51065' = '1540 '
'51073' = '5720 '
'51075' = '6760 '
'51079' = '1540 '
'51085' = '6760 '
'51087' = '6760 '
'51093' = '5720 '
'51095' = '5720 '
'51099' = '8840 '
'51107' = '8840 '
'51115' = '5720 '
'51127' = '6760 '
'51143' = '1950 '
'51145' = '6760 '
'51149' = '6760 '
'51153' = '8840 '
'51161' = '6800 '
'51169' = '3660 '
'51177' = '8840 '
'51179' = '8840 '
'51187' = '8840 '
'51191' = '3660 '
'51199' = '5720 '
'51510' = '8840 '
'51515' = '4640 '
'51520' = '3660 '
'51540' = '1540 '
'51550' = '5720 '
'51570' = '6760 '
'51590' = '1950 '
'51600' = '8840 '
'51610' = '8840 '
'51630' = '8840 '
'51650' = '5720 '
'51670' = '6760 '
'51680' = '4640 '
'51683' = '8840 '
'51685' = '8840 '
'51700' = '5720 '
'51710' = '5720 '
'51730' = '6760 '
'51735' = '5720 '
'51740' = '5720 '
'51760' = '6760 '
'51770' = '6800 '
'51775' = '6800 '
'51800' = '5720 '
'51810' = '5720 '
'51830' = '5720 '
'53005' = '6740 '
'53011' = '6440 '
'53021' = '6740 '
'53029' = '7600 '
'53033' = '7600 '
'53035' = '1150 '
'53053' = '8200 '
'53061' = '7600 '
'53063' = '7840 '
'53067' = '5910 '
'53073' = '0860 '
'53077' = '9260 '
'54003' = '8840 '
'54009' = '8080 '
'54011' = '3400 '
'54029' = '8080 '
'54037' = '8840 '
'54039' = '1480 '
'54051' = '9000 '
'54057' = '1900 '
'54069' = '9000 '
'54079' = '1480 '
'54099' = '3400 '
'54107' = '6020 '
'55009' = '3080 '
'55015' = '0460 '
'55017' = '2290 '
'55025' = '4720 '
'55031' = '2240 '
'55035' = '2290 '
'55059' = '3800 '
'55063' = '3870 '
'55073' = '8940 '
'55079' = '5080 '
'55087' = '0460 '
'55089' = '5080 '
'55093' = '5120 '
'55101' = '6600 '
'55105' = '3620 '
'55109' = '5120 '
'55117' = '7620 '
'55131' = '5080 '
'55133' = '5080 '
'55139' = '0460 '
'56021' = '1580 '
'56025' = '1350 '
;

/* OMB 2003 METRO AREA */

VALUE $M3AREA
'01001' = '33860'
'01003' = '19300'
'01005' = '21640'
'01007' = '13820'
'01009' = '13820'
'01015' = '11500'
'01017' = '46740'
'01021' = '13820'
'01031' = '21460'
'01033' = '22520'
'01037' = '10760'
'01043' = '18980'
'01045' = '21460'
'01047' = '42820'
'01049' = '22840'
'01051' = '33860'
'01055' = '23460'
'01061' = '20020'
'01063' = '46220'
'01065' = '46220'
'01067' = '20020'
'01069' = '20020'
'01071' = '42460'
'01073' = '13820'
'01077' = '22520'
'01079' = '19460'
'01081' = '12220'
'01083' = '26620'
'01085' = '33860'
'01087' = '46260'
'01089' = '26620'
'01095' = '10700'
'01097' = '33660'
'01101' = '33860'
'01103' = '19460'
'01109' = '45980'
'01113' = '17980'
'01115' = '13820'
'01117' = '13820'
'01121' = '45180'
'01123' = '10760'
'01125' = '46220'
'01127' = '13820'
'02020' = '11260'
'02090' = '21820'
'02110' = '27940'
'02130' = '28540'
'02150' = '28980'
'02170' = '11260'
'04003' = '43420'
'04005' = '22380'
'04007' = '37740'
'04009' = '40940'
'04011' = '40940'
'04013' = '38060'
'04015' = '29420'
'04019' = '46060'
'04021' = '38060'
'04023' = '35700'
'04025' = '39140'
'04027' = '49740'
'05005' = '34260'
'05007' = '22220'
'05009' = '25460'
'05013' = '15780'
'05019' = '11660'
'05025' = '38220'
'05027' = '31620'
'05031' = '27860'
'05033' = '22900'
'05035' = '32820'
'05045' = '30780'
'05047' = '22900'
'05051' = '26300'
'05053' = '30780'
'05055' = '37500'
'05057' = '26260'
'05063' = '12900'
'05069' = '38220'
'05079' = '38220'
'05085' = '30780'
'05087' = '22220'
'05091' = '45500'
'05093' = '14180'
'05099' = '26260'
'05101' = '25460'
'05103' = '15780'
'05105' = '30780'
'05107' = '48340'
'05111' = '27860'
'05115' = '40780'
'05119' = '30780'
'05123' = '22620'
'05125' = '30780'
'05131' = '22900'
'05139' = '20980'
'05143' = '22220'
'05145' = '42620'
'05149' = '40780'
'06001' = '41860'
'06007' = '17020'
'06013' = '41860'
'06015' = '18860'
'06017' = '40900'
'06019' = '23420'
'06023' = '21700'
'06025' = '20940'
'06027' = '13860'
'06029' = '12540'
'06031' = '25260'
'06033' = '17340'
'06035' = '45000'
'06037' = '31100'
'06039' = '31460'
'06041' = '41860'
'06045' = '46380'
'06047' = '32900'
'06053' = '41500'
'06055' = '34900'
'06057' = '46020'
'06059' = '31100'
'06061' = '40900'
'06065' = '40140'
'06067' = '40900'
'06069' = '41940'
'06071' = '40140'
'06073' = '41740'
'06075' = '41860'
'06077' = '44700'
'06079' = '42020'
'06081' = '41860'
'06083' = '42060'
'06085' = '41940'
'06087' = '42100'
'06089' = '39820'
'06095' = '46700'
'06097' = '42220'
'06099' = '33700'
'06101' = '49700'
'06103' = '39780'
'06107' = '47300'
'06109' = '38020'
'06111' = '37100'
'06113' = '40900'
'06115' = '49700'
'08001' = '19740'
'08005' = '19740'
'08013' = '14500'
'08014' = '19740'
'08019' = '19740'
'08031' = '19740'
'08035' = '19740'
'08037' = '20780'
'08039' = '19740'
'08041' = '17820'
'08043' = '15860'
'08047' = '19740'
'08059' = '19740'
'08065' = '20780'
'08067' = '20420'
'08069' = '22660'
'08075' = '44540'
'08077' = '24300'
'08085' = '33940'
'08087' = '22820'
'08093' = '19740'
'08101' = '39380'
'08117' = '43540'
'08119' = '17820'
'08123' = '24540'
'09001' = '14860'
'09003' = '25540'
'09005' = '45860'
'09007' = '25540'
'09009' = '35300'
'09011' = '35980'
'09013' = '25540'
'09015' = '48740'
'10001' = '20100'
'10003' = '37980'
'10005' = '42580'
'11001' = '47900'
'12001' = '23540'
'12003' = '27260'
'12005' = '37460'
'12009' = '37340'
'12011' = '33100'
'12015' = '39460'
'12017' = '26140'
'12019' = '27260'
'12021' = '34940'
'12023' = '29380'
'12027' = '11580'
'12031' = '27260'
'12033' = '37860'
'12035' = '37380'
'12039' = '45220'
'12041' = '23540'
'12049' = '48100'
'12051' = '17500'
'12053' = '45300'
'12055' = '42700'
'12057' = '45300'
'12061' = '46940'
'12065' = '45220'
'12069' = '36740'
'12071' = '15980'
'12073' = '45220'
'12081' = '42260'
'12083' = '36100'
'12085' = '38940'
'12086' = '33100'
'12087' = '28580'
'12089' = '27260'
'12091' = '23020'
'12093' = '36380'
'12095' = '36740'
'12097' = '36740'
'12099' = '33100'
'12101' = '45300'
'12103' = '45300'
'12105' = '29460'
'12107' = '37260'
'12109' = '27260'
'12111' = '38940'
'12113' = '37860'
'12115' = '42260'
'12117' = '36740'
'12119' = '45540'
'12127' = '19660'
'12129' = '45220'
'13003' = '20060'
'13007' = '10500'
'13009' = '33300'
'13013' = '12060'
'13015' = '12060'
'13017' = '22340'
'13021' = '31420'
'13025' = '15260'
'13027' = '46660'
'13029' = '42340'
'13031' = '44340'
'13033' = '12260'
'13035' = '12060'
'13039' = '41220'
'13045' = '12060'
'13047' = '16860'
'13051' = '42340'
'13053' = '17980'
'13055' = '44900'
'13057' = '12060'
'13059' = '12020'
'13063' = '12060'
'13067' = '12060'
'13069' = '20060'
'13071' = '34220'
'13073' = '12260'
'13077' = '12060'
'13079' = '31420'
'13081' = '18380'
'13083' = '16860'
'13085' = '12060'
'13087' = '12460'
'13089' = '12060'
'13095' = '10500'
'13097' = '12060'
'13101' = '46660'
'13103' = '42340'
'13113' = '12060'
'13115' = '40660'
'13117' = '12060'
'13121' = '12060'
'13127' = '15260'
'13129' = '15660'
'13135' = '12060'
'13137' = '18460'
'13139' = '23580'
'13141' = '33300'
'13143' = '12060'
'13145' = '17980'
'13149' = '12060'
'13151' = '12060'
'13153' = '47580'
'13155' = '22340'
'13159' = '12060'
'13167' = '20140'
'13169' = '31420'
'13171' = '12060'
'13173' = '46660'
'13175' = '20140'
'13177' = '10500'
'13179' = '25980'
'13183' = '25980'
'13185' = '46660'
'13189' = '12260'
'13191' = '15260'
'13195' = '12020'
'13197' = '17980'
'13199' = '12060'
'13207' = '31420'
'13209' = '47080'
'13213' = '19140'
'13215' = '17980'
'13217' = '12060'
'13219' = '12020'
'13221' = '12020'
'13223' = '12060'
'13225' = '22980'
'13227' = '12060'
'13229' = '48180'
'13231' = '12060'
'13233' = '16340'
'13239' = '21640'
'13245' = '12260'
'13247' = '12060'
'13249' = '11140'
'13255' = '12060'
'13257' = '45740'
'13261' = '11140'
'13273' = '10500'
'13275' = '45620'
'13277' = '45700'
'13279' = '47080'
'13285' = '29300'
'13289' = '31420'
'13293' = '45580'
'13295' = '16860'
'13297' = '12060'
'13299' = '48180'
'13305' = '27700'
'13313' = '19140'
'13321' = '10500'
'15001' = '25900'
'15003' = '26180'
'15007' = '28180'
'15009' = '27980'
'16001' = '14260'
'16005' = '38540'
'16011' = '13940'
'16015' = '14260'
'16019' = '26820'
'16027' = '14260'
'16031' = '15420'
'16039' = '34300'
'16041' = '30860'
'16043' = '39940'
'16045' = '14260'
'16051' = '26820'
'16053' = '46300'
'16055' = '17660'
'16057' = '34140'
'16065' = '39940'
'16067' = '15420'
'16069' = '30300'
'16073' = '14260'
'16075' = '36620'
'16077' = '38540'
'16081' = '27220'
'16083' = '46300'
'17001' = '39500'
'17003' = '16020'
'17005' = '41180'
'17007' = '40420'
'17011' = '36860'
'17013' = '41180'
'17019' = '16580'
'17021' = '45380'
'17027' = '41180'
'17029' = '16660'
'17031' = '16980'
'17035' = '16660'
'17037' = '16980'
'17043' = '16980'
'17049' = '20820'
'17053' = '16580'
'17057' = '15900'
'17063' = '16980'
'17065' = '34500'
'17071' = '15460'
'17073' = '19340'
'17077' = '16060'
'17081' = '34500'
'17083' = '41180'
'17089' = '16980'
'17091' = '28100'
'17093' = '16980'
'17095' = '23660'
'17097' = '16980'
'17099' = '36860'
'17103' = '19940'
'17105' = '38700'
'17107' = '30660'
'17109' = '31380'
'17111' = '16980'
'17113' = '14060'
'17115' = '19500'
'17117' = '41180'
'17119' = '41180'
'17121' = '16460'
'17123' = '37900'
'17127' = '37140'
'17129' = '44100'
'17131' = '19340'
'17133' = '41180'
'17137' = '27300'
'17141' = '40300'
'17143' = '37900'
'17147' = '16580'
'17155' = '36860'
'17161' = '19340'
'17163' = '41180'
'17165' = '25380'
'17167' = '44100'
'17171' = '27300'
'17175' = '37900'
'17177' = '23300'
'17179' = '37900'
'17183' = '19180'
'17187' = '23660'
'17195' = '44580'
'17197' = '16980'
'17199' = '32060'
'17201' = '40420'
'17203' = '37900'
'18001' = '19540'
'18003' = '23060'
'18005' = '18020'
'18007' = '29140'
'18011' = '26900'
'18013' = '26900'
'18015' = '29140'
'18017' = '30900'
'18019' = '31140'
'18021' = '45460'
'18023' = '23140'
'18027' = '47780'
'18029' = '17140'
'18031' = '24700'
'18033' = '12140'
'18035' = '34620'
'18037' = '27540'
'18039' = '21140'
'18041' = '18220'
'18043' = '31140'
'18047' = '17140'
'18051' = '21780'
'18053' = '31980'
'18055' = '14020'
'18057' = '26900'
'18059' = '26900'
'18061' = '31140'
'18063' = '26900'
'18065' = '35220'
'18067' = '29020'
'18069' = '26540'
'18071' = '42980'
'18073' = '16980'
'18077' = '31500'
'18079' = '35860'
'18081' = '26900'
'18083' = '47180'
'18085' = '47700'
'18089' = '16980'
'18091' = '33140'
'18093' = '13260'
'18095' = '11300'
'18097' = '26900'
'18099' = '38500'
'18103' = '37940'
'18105' = '14020'
'18107' = '18820'
'18109' = '26900'
'18111' = '16980'
'18113' = '28340'
'18115' = '17140'
'18119' = '14020'
'18125' = '27540'
'18127' = '16980'
'18129' = '21780'
'18133' = '26900'
'18141' = '43780'
'18143' = '42500'
'18145' = '26900'
'18151' = '11420'
'18153' = '45460'
'18157' = '29140'
'18159' = '29020'
'18163' = '21780'
'18165' = '45460'
'18167' = '45460'
'18169' = '47340'
'18173' = '21780'
'18175' = '31140'
'18177' = '39980'
'18179' = '23060'
'18183' = '23060'
'19011' = '16300'
'19013' = '47940'
'19015' = '14340'
'19017' = '47940'
'19021' = '44740'
'19033' = '32380'
'19041' = '43980'
'19045' = '17540'
'19049' = '19780'
'19057' = '15460'
'19059' = '44020'
'19061' = '20220'
'19075' = '47940'
'19077' = '19780'
'19085' = '36540'
'19099' = '35500'
'19103' = '26980'
'19105' = '16300'
'19111' = '28460'
'19113' = '16300'
'19115' = '34700'
'19121' = '19780'
'19123' = '36820'
'19125' = '37800'
'19127' = '32260'
'19129' = '36540'
'19139' = '34700'
'19153' = '19780'
'19155' = '36540'
'19163' = '19340'
'19169' = '11180'
'19179' = '36900'
'19181' = '19780'
'19183' = '26980'
'19187' = '22700'
'19193' = '43580'
'19195' = '32380'
'20005' = '11860'
'20009' = '24460'
'20015' = '48620'
'20017' = '21380'
'20035' = '49060'
'20037' = '38260'
'20043' = '41140'
'20045' = '29940'
'20051' = '25700'
'20055' = '23780'
'20057' = '19980'
'20059' = '28140'
'20061' = '31740'
'20079' = '48620'
'20085' = '45820'
'20087' = '45820'
'20091' = '28140'
'20099' = '37660'
'20103' = '28140'
'20107' = '28140'
'20111' = '21380'
'20113' = '32700'
'20121' = '28140'
'20125' = '17700'
'20139' = '45820'
'20143' = '41460'
'20149' = '31740'
'20155' = '26740'
'20161' = '31740'
'20169' = '41460'
'20173' = '48620'
'20175' = '30580'
'20177' = '45820'
'20191' = '48620'
'20197' = '45820'
'20209' = '28140'
'21005' = '23180'
'21007' = '37140'
'21009' = '23980'
'21011' = '34460'
'21013' = '33180'
'21015' = '17140'
'21017' = '30460'
'21019' = '26580'
'21021' = '19220'
'21023' = '17140'
'21029' = '31140'
'21035' = '34660'
'21037' = '17140'
'21047' = '17300'
'21049' = '30460'
'21059' = '36980'
'21061' = '14540'
'21067' = '30460'
'21073' = '23180'
'21075' = '46460'
'21077' = '17140'
'21081' = '17140'
'21083' = '32460'
'21089' = '26580'
'21091' = '36980'
'21093' = '21060'
'21101' = '21780'
'21103' = '31140'
'21107' = '31580'
'21111' = '31140'
'21113' = '30460'
'21117' = '17140'
'21123' = '21060'
'21125' = '30940'
'21135' = '32500'
'21137' = '19220'
'21139' = '37140'
'21145' = '37140'
'21149' = '36980'
'21151' = '40080'
'21161' = '32500'
'21163' = '31140'
'21165' = '34460'
'21169' = '23980'
'21173' = '34460'
'21177' = '16420'
'21179' = '31140'
'21185' = '31140'
'21191' = '17140'
'21199' = '43700'
'21203' = '40080'
'21209' = '30460'
'21211' = '31140'
'21215' = '31140'
'21217' = '15820'
'21221' = '17300'
'21223' = '31140'
'21227' = '14540'
'21233' = '21780'
'21235' = '18340'
'21239' = '30460'
'22001' = '18940'
'22005' = '12940'
'22007' = '38200'
'22011' = '19760'
'22015' = '43340'
'22017' = '43340'
'22019' = '29340'
'22023' = '29340'
'22029' = '35020'
'22031' = '43340'
'22033' = '12940'
'22037' = '12940'
'22043' = '10780'
'22045' = '35340'
'22047' = '12940'
'22049' = '40820'
'22051' = '35380'
'22053' = '27660'
'22055' = '29180'
'22057' = '26380'
'22061' = '40820'
'22063' = '12940'
'22065' = '45260'
'22067' = '12820'
'22069' = '35060'
'22071' = '35380'
'22073' = '33740'
'22075' = '35380'
'22077' = '12940'
'22079' = '10780'
'22087' = '35380'
'22089' = '35380'
'22091' = '12940'
'22095' = '35380'
'22097' = '36660'
'22099' = '29180'
'22101' = '34020'
'22103' = '35380'
'22105' = '25220'
'22109' = '26380'
'22111' = '33740'
'22113' = '10020'
'22115' = '22860'
'22117' = '14220'
'22119' = '33380'
'22121' = '12940'
'22125' = '12940'
'23001' = '30340'
'23005' = '38860'
'23011' = '12300'
'23013' = '40500'
'23019' = '12620'
'23023' = '38860'
'23031' = '38860'
'24001' = '19060'
'24003' = '12580'
'24005' = '12580'
'24009' = '47900'
'24013' = '12580'
'24015' = '37980'
'24017' = '47900'
'24019' = '15700'
'24021' = '47900'
'24025' = '12580'
'24027' = '12580'
'24031' = '47900'
'24033' = '47900'
'24035' = '12580'
'24037' = '30500'
'24039' = '41540'
'24041' = '20660'
'24043' = '25180'
'24045' = '41540'
'24047' = '36180'
'24510' = '12580'
'25001' = '12700'
'25003' = '38340'
'25005' = '39300'
'25009' = '14460'
'25011' = '44140'
'25013' = '44140'
'25015' = '44140'
'25017' = '14460'
'25021' = '14460'
'25023' = '14460'
'25025' = '14460'
'25027' = '49340'
'26005' = '10880'
'26007' = '10980'
'26015' = '24340'
'26017' = '13020'
'26019' = '45900'
'26021' = '35660'
'26023' = '17740'
'26025' = '12980'
'26027' = '43780'
'26033' = '42300'
'26037' = '29620'
'26041' = '21540'
'26043' = '27020'
'26045' = '29620'
'26049' = '22420'
'26055' = '45900'
'26057' = '10940'
'26061' = '26340'
'26065' = '29620'
'26067' = '24340'
'26073' = '34380'
'26075' = '27100'
'26077' = '28020'
'26079' = '45900'
'26081' = '24340'
'26083' = '26340'
'26087' = '19820'
'26089' = '45900'
'26091' = '10300'
'26093' = '19820'
'26099' = '19820'
'26103' = '32100'
'26107' = '13660'
'26109' = '31940'
'26111' = '33220'
'26113' = '15620'
'26115' = '33780'
'26121' = '34740'
'26123' = '24340'
'26125' = '19820'
'26139' = '26100'
'26145' = '40980'
'26147' = '19820'
'26149' = '44780'
'26155' = '37020'
'26159' = '28020'
'26161' = '11460'
'26163' = '19820'
'26165' = '15620'
'27003' = '33460'
'27007' = '13420'
'27009' = '41060'
'27013' = '31860'
'27015' = '35580'
'27017' = '20260'
'27019' = '33460'
'27021' = '14660'
'27025' = '33460'
'27027' = '22020'
'27035' = '14660'
'27037' = '33460'
'27039' = '40340'
'27041' = '10820'
'27047' = '10660'
'27049' = '39860'
'27053' = '33460'
'27055' = '29100'
'27059' = '33460'
'27067' = '48820'
'27083' = '32140'
'27085' = '26780'
'27091' = '21860'
'27099' = '12380'
'27103' = '31860'
'27105' = '49380'
'27109' = '40340'
'27111' = '22260'
'27119' = '24220'
'27123' = '33460'
'27131' = '22060'
'27137' = '20260'
'27139' = '33460'
'27141' = '33460'
'27145' = '41060'
'27147' = '36940'
'27157' = '40340'
'27163' = '33460'
'27167' = '47420'
'27169' = '49100'
'27171' = '33460'
'28001' = '35020'
'28003' = '18420'
'28005' = '32620'
'28011' = '17380'
'28015' = '24900'
'28023' = '32940'
'28025' = '48500'
'28027' = '17260'
'28029' = '27140'
'28033' = '32820'
'28035' = '25620'
'28039' = '37700'
'28043' = '24980'
'28045' = '25060'
'28047' = '25060'
'28049' = '27140'
'28057' = '46180'
'28059' = '37700'
'28061' = '29860'
'28067' = '29860'
'28069' = '32940'
'28071' = '37060'
'28073' = '25620'
'28075' = '32940'
'28081' = '46180'
'28083' = '24900'
'28085' = '15020'
'28087' = '18060'
'28089' = '27140'
'28093' = '32820'
'28105' = '44260'
'28109' = '38100'
'28111' = '25620'
'28113' = '32620'
'28115' = '46180'
'28121' = '27140'
'28127' = '27140'
'28131' = '25060'
'28133' = '26940'
'28137' = '32820'
'28143' = '32820'
'28149' = '46980'
'28151' = '24740'
'28163' = '49540'
'29001' = '28860'
'29003' = '41140'
'29007' = '33020'
'29013' = '28140'
'29017' = '16020'
'29019' = '17860'
'29021' = '41140'
'29023' = '38740'
'29025' = '28140'
'29027' = '27620'
'29031' = '16020'
'29037' = '28140'
'29043' = '44180'
'29045' = '28460'
'29047' = '28140'
'29049' = '28140'
'29051' = '27620'
'29059' = '44180'
'29063' = '41140'
'29069' = '28380'
'29071' = '41180'
'29077' = '44180'
'29089' = '17860'
'29091' = '48460'
'29095' = '28140'
'29097' = '27900'
'29099' = '41180'
'29101' = '47660'
'29105' = '30060'
'29107' = '28140'
'29111' = '39500'
'29113' = '41180'
'29119' = '22220'
'29127' = '25300'
'29135' = '27620'
'29145' = '27900'
'29147' = '32340'
'29151' = '27620'
'29159' = '42740'
'29161' = '40620'
'29165' = '28140'
'29167' = '44180'
'29169' = '22780'
'29173' = '25300'
'29175' = '33620'
'29177' = '28140'
'29183' = '41180'
'29187' = '22100'
'29189' = '41180'
'29195' = '32180'
'29197' = '28860'
'29201' = '43460'
'29209' = '14700'
'29213' = '14700'
'29219' = '41180'
'29221' = '41180'
'29225' = '44180'
'29510' = '41180'
'30009' = '13740'
'30013' = '24500'
'30029' = '28060'
'30031' = '14580'
'30041' = '25660'
'30043' = '25740'
'30049' = '25740'
'30063' = '33540'
'30093' = '15580'
'30111' = '13740'
'31001' = '25580'
'31007' = '42420'
'31019' = '28260'
'31025' = '36540'
'31035' = '25580'
'31043' = '43580'
'31047' = '30420'
'31051' = '43580'
'31053' = '23340'
'31055' = '36540'
'31067' = '13100'
'31073' = '30420'
'31079' = '24260'
'31093' = '24260'
'31099' = '28260'
'31109' = '30700'
'31111' = '35820'
'31113' = '35820'
'31117' = '35820'
'31119' = '35740'
'31121' = '24260'
'31139' = '35740'
'31141' = '18100'
'31153' = '36540'
'31155' = '36540'
'31157' = '42420'
'31159' = '30700'
'31167' = '35740'
'31177' = '36540'
'32001' = '21980'
'32003' = '29820'
'32005' = '23820'
'32007' = '21220'
'32011' = '21220'
'32023' = '37220'
'32029' = '39900'
'32031' = '39900'
'32510' = '16180'
'33001' = '29060'
'33005' = '28300'
'33007' = '13620'
'33009' = '30100'
'33011' = '31700'
'33013' = '18180'
'33015' = '14460'
'33017' = '14460'
'33019' = '17200'
'34001' = '12100'
'34003' = '35620'
'34005' = '37980'
'34007' = '37980'
'34009' = '36140'
'34011' = '47220'
'34013' = '35620'
'34015' = '37980'
'34017' = '35620'
'34019' = '35620'
'34021' = '45940'
'34023' = '35620'
'34025' = '35620'
'34027' = '35620'
'34029' = '35620'
'34031' = '35620'
'34033' = '37980'
'34035' = '35620'
'34037' = '35620'
'34039' = '35620'
'34041' = '10900'
'35001' = '10740'
'35005' = '40740'
'35006' = '24380'
'35009' = '17580'
'35013' = '29740'
'35015' = '16100'
'35017' = '43500'
'35025' = '26020'
'35028' = '31060'
'35029' = '19700'
'35031' = '23700'
'35035' = '10460'
'35039' = '21580'
'35041' = '38780'
'35043' = '10740'
'35045' = '22140'
'35047' = '29780'
'35049' = '42140'
'35055' = '45340'
'35057' = '10740'
'35061' = '10740'
'36001' = '10580'
'36005' = '35620'
'36007' = '13780'
'36009' = '36460'
'36011' = '12180'
'36013' = '27460'
'36015' = '21300'
'36019' = '38460'
'36021' = '26460'
'36023' = '18660'
'36027' = '39100'
'36029' = '15380'
'36033' = '31660'
'36035' = '24100'
'36037' = '12860'
'36043' = '46540'
'36045' = '48060'
'36047' = '35620'
'36051' = '40380'
'36053' = '45060'
'36055' = '40380'
'36057' = '11220'
'36059' = '35620'
'36061' = '35620'
'36063' = '15380'
'36065' = '46540'
'36067' = '45060'
'36069' = '40380'
'36071' = '39100'
'36073' = '40380'
'36075' = '45060'
'36077' = '36580'
'36079' = '35620'
'36081' = '35620'
'36083' = '10580'
'36085' = '35620'
'36087' = '35620'
'36089' = '36300'
'36091' = '10580'
'36093' = '10580'
'36095' = '10580'
'36099' = '42900'
'36101' = '18500'
'36103' = '35620'
'36107' = '13780'
'36109' = '27060'
'36111' = '28740'
'36113' = '24020'
'36115' = '24020'
'36117' = '40380'
'36119' = '35620'
'37001' = '15500'
'37003' = '25860'
'37007' = '16740'
'37013' = '47820'
'37019' = '48900'
'37021' = '11700'
'37023' = '25860'
'37025' = '16740'
'37027' = '25860'
'37029' = '21020'
'37031' = '33980'
'37035' = '25860'
'37037' = '20500'
'37045' = '43140'
'37049' = '35100'
'37051' = '22180'
'37053' = '47260'
'37055' = '28620'
'37057' = '45640'
'37059' = '49180'
'37063' = '20500'
'37065' = '40580'
'37067' = '49180'
'37069' = '39580'
'37071' = '16740'
'37079' = '24780'
'37081' = '24660'
'37083' = '40260'
'37085' = '20380'
'37087' = '11700'
'37089' = '11700'
'37093' = '22180'
'37097' = '44380'
'37101' = '39580'
'37103' = '35100'
'37105' = '41820'
'37107' = '28820'
'37109' = '30740'
'37115' = '11700'
'37119' = '16740'
'37125' = '43860'
'37127' = '40580'
'37129' = '48900'
'37131' = '40260'
'37133' = '27340'
'37135' = '20500'
'37137' = '35100'
'37139' = '21020'
'37141' = '48900'
'37143' = '21020'
'37145' = '20500'
'37147' = '24780'
'37151' = '24660'
'37153' = '40460'
'37155' = '31300'
'37157' = '24660'
'37159' = '41580'
'37161' = '22580'
'37165' = '29900'
'37167' = '10620'
'37169' = '49180'
'37171' = '34340'
'37175' = '14820'
'37179' = '16740'
'37181' = '25780'
'37183' = '39580'
'37189' = '14380'
'37191' = '24140'
'37193' = '35900'
'37195' = '48980'
'37197' = '49180'
'38007' = '19860'
'38015' = '13900'
'38017' = '22020'
'38035' = '24220'
'38049' = '33500'
'38059' = '13900'
'38075' = '33500'
'38077' = '47420'
'38089' = '19860'
'38093' = '27420'
'38101' = '33500'
'38105' = '48780'
'39003' = '30620'
'39005' = '11740'
'39007' = '11780'
'39009' = '11900'
'39011' = '47540'
'39013' = '48540'
'39015' = '17140'
'39017' = '17140'
'39019' = '15940'
'39021' = '46500'
'39023' = '44220'
'39025' = '17140'
'39027' = '48940'
'39029' = '20620'
'39031' = '18740'
'39033' = '15340'
'39035' = '17460'
'39037' = '24820'
'39039' = '19580'
'39041' = '18140'
'39043' = '41780'
'39045' = '18140'
'39047' = '47860'
'39049' = '18140'
'39051' = '45780'
'39053' = '38580'
'39055' = '17460'
'39057' = '19380'
'39059' = '15740'
'39061' = '17140'
'39063' = '22300'
'39077' = '35940'
'39081' = '48260'
'39083' = '34540'
'39085' = '17460'
'39087' = '26580'
'39089' = '18140'
'39091' = '13340'
'39093' = '17460'
'39095' = '45780'
'39097' = '18140'
'39099' = '49660'
'39101' = '32020'
'39103' = '17460'
'39107' = '16380'
'39109' = '19380'
'39113' = '19380'
'39117' = '18140'
'39119' = '49780'
'39123' = '45780'
'39129' = '18140'
'39133' = '10420'
'39135' = '19380'
'39139' = '31900'
'39141' = '17060'
'39143' = '23380'
'39145' = '39020'
'39147' = '45660'
'39149' = '43380'
'39151' = '15940'
'39153' = '10420'
'39155' = '49660'
'39157' = '35420'
'39159' = '18140'
'39161' = '46780'
'39165' = '17140'
'39167' = '37620'
'39169' = '49300'
'39173' = '45780'
'40009' = '21120'
'40013' = '20460'
'40017' = '36420'
'40019' = '11620'
'40021' = '45140'
'40027' = '36420'
'40031' = '30020'
'40037' = '46140'
'40047' = '21420'
'40051' = '36420'
'40065' = '11060'
'40071' = '38620'
'40079' = '22900'
'40081' = '36420'
'40083' = '36420'
'40085' = '11620'
'40087' = '36420'
'40101' = '34780'
'40109' = '36420'
'40111' = '46140'
'40113' = '46140'
'40115' = '33060'
'40117' = '46140'
'40119' = '44660'
'40121' = '32540'
'40123' = '10220'
'40125' = '43060'
'40131' = '46140'
'40135' = '22900'
'40137' = '20340'
'40139' = '25100'
'40143' = '46140'
'40145' = '46140'
'40147' = '12780'
'40153' = '49260'
'41003' = '18700'
'41005' = '38900'
'41007' = '11820'
'41009' = '38900'
'41011' = '18300'
'41013' = '39260'
'41015' = '15060'
'41017' = '13460'
'41019' = '40700'
'41027' = '26220'
'41029' = '32780'
'41033' = '24420'
'41035' = '28900'
'41039' = '21660'
'41043' = '10540'
'41045' = '36620'
'41047' = '41420'
'41049' = '37820'
'41051' = '38900'
'41053' = '41420'
'41059' = '37820'
'41061' = '29260'
'41065' = '17180'
'41067' = '38900'
'41071' = '38900'
'42001' = '23900'
'42003' = '38300'
'42005' = '38300'
'42007' = '38300'
'42011' = '39740'
'42013' = '11020'
'42015' = '42380'
'42017' = '37980'
'42019' = '38300'
'42021' = '27780'
'42025' = '10900'
'42027' = '44300'
'42029' = '37980'
'42033' = '20180'
'42035' = '30820'
'42037' = '14100'
'42039' = '32740'
'42041' = '25420'
'42043' = '25420'
'42045' = '37980'
'42047' = '41260'
'42049' = '21500'
'42051' = '38300'
'42055' = '16540'
'42061' = '26500'
'42063' = '26860'
'42069' = '42540'
'42071' = '29540'
'42073' = '35260'
'42075' = '30140'
'42077' = '10900'
'42079' = '42540'
'42081' = '48700'
'42083' = '14620'
'42085' = '49660'
'42087' = '30380'
'42089' = '20700'
'42091' = '37980'
'42093' = '14100'
'42095' = '10900'
'42097' = '44980'
'42099' = '25420'
'42101' = '37980'
'42103' = '35620'
'42107' = '39060'
'42109' = '42780'
'42111' = '43740'
'42119' = '30260'
'42121' = '36340'
'42123' = '47620'
'42125' = '38300'
'42129' = '38300'
'42131' = '42540'
'42133' = '49620'
'44001' = '39300'
'44003' = '39300'
'44005' = '39300'
'44007' = '39300'
'44009' = '39300'
'45003' = '12260'
'45007' = '11340'
'45013' = '25940'
'45015' = '16700'
'45017' = '17900'
'45019' = '16700'
'45021' = '23500'
'45023' = '16900'
'45029' = '47500'
'45031' = '22500'
'45033' = '19900'
'45035' = '16700'
'45037' = '12260'
'45039' = '17900'
'45041' = '22500'
'45043' = '23860'
'45045' = '24860'
'45047' = '24940'
'45051' = '34820'
'45053' = '25940'
'45055' = '17900'
'45057' = '29580'
'45059' = '24860'
'45063' = '17900'
'45069' = '13500'
'45071' = '35140'
'45073' = '42860'
'45075' = '36700'
'45077' = '24860'
'45079' = '17900'
'45081' = '17900'
'45083' = '43900'
'45085' = '44940'
'45087' = '46420'
'45091' = '16740'
'46005' = '26700'
'46011' = '15100'
'46013' = '10100'
'46027' = '46820'
'46029' = '47980'
'46035' = '33580'
'46045' = '10100'
'46057' = '47980'
'46061' = '33580'
'46065' = '38180'
'46081' = '43940'
'46083' = '43620'
'46087' = '43620'
'46093' = '39660'
'46099' = '43620'
'46103' = '39660'
'46117' = '38180'
'46125' = '43620'
'46127' = '43580'
'46135' = '49460'
'47001' = '28940'
'47003' = '43180'
'47009' = '28940'
'47011' = '17420'
'47013' = '29220'
'47015' = '34980'
'47019' = '27740'
'47021' = '34980'
'47023' = '27180'
'47029' = '35460'
'47031' = '46100'
'47035' = '18900'
'47037' = '34980'
'47043' = '34980'
'47045' = '20540'
'47047' = '32820'
'47051' = '46100'
'47053' = '26480'
'47057' = '34100'
'47059' = '24620'
'47063' = '34100'
'47065' = '16860'
'47073' = '28700'
'47075' = '15140'
'47079' = '37540'
'47081' = '34980'
'47087' = '18260'
'47089' = '34100'
'47093' = '28940'
'47099' = '29980'
'47105' = '28940'
'47107' = '11940'
'47111' = '34980'
'47113' = '27180'
'47115' = '16860'
'47117' = '30280'
'47119' = '17940'
'47125' = '17300'
'47127' = '46100'
'47131' = '46460'
'47133' = '18260'
'47139' = '17420'
'47141' = '18260'
'47145' = '25340'
'47147' = '34980'
'47149' = '34980'
'47153' = '16860'
'47155' = '42940'
'47157' = '32820'
'47159' = '34980'
'47161' = '17300'
'47163' = '28700'
'47165' = '34980'
'47167' = '32820'
'47169' = '34980'
'47171' = '27740'
'47173' = '28940'
'47177' = '32660'
'47179' = '27740'
'47183' = '32280'
'47187' = '34980'
'47189' = '34980'
'48001' = '37300'
'48003' = '11380'
'48005' = '31260'
'48007' = '18580'
'48009' = '48660'
'48011' = '11100'
'48013' = '41700'
'48015' = '26420'
'48019' = '41700'
'48021' = '12420'
'48025' = '13300'
'48027' = '28660'
'48029' = '41700'
'48037' = '45500'
'48039' = '26420'
'48041' = '17780'
'48049' = '15220'
'48051' = '17780'
'48055' = '12420'
'48057' = '47020'
'48059' = '10180'
'48061' = '15180'
'48065' = '11100'
'48071' = '26420'
'48073' = '27380'
'48077' = '48660'
'48085' = '19100'
'48091' = '41700'
'48097' = '23620'
'48099' = '28660'
'48107' = '31180'
'48113' = '19100'
'48115' = '29500'
'48117' = '25820'
'48119' = '19100'
'48121' = '19100'
'48135' = '36220'
'48139' = '19100'
'48141' = '21340'
'48143' = '44500'
'48147' = '14300'
'48157' = '26420'
'48167' = '26420'
'48175' = '47020'
'48179' = '37420'
'48181' = '43300'
'48183' = '30980'
'48187' = '41700'
'48189' = '38380'
'48199' = '13140'
'48201' = '26420'
'48203' = '32220'
'48209' = '12420'
'48213' = '11980'
'48215' = '32580'
'48219' = '30220'
'48221' = '24180'
'48223' = '44860'
'48227' = '13700'
'48231' = '19100'
'48233' = '14420'
'48235' = '41660'
'48245' = '13140'
'48249' = '10860'
'48251' = '19100'
'48253' = '10180'
'48257' = '19100'
'48259' = '41700'
'48261' = '28780'
'48265' = '28500'
'48273' = '28780'
'48277' = '37580'
'48281' = '28660'
'48291' = '26420'
'48303' = '31180'
'48309' = '47380'
'48321' = '13060'
'48323' = '20580'
'48325' = '41700'
'48329' = '33260'
'48339' = '26420'
'48341' = '20300'
'48347' = '34860'
'48349' = '18620'
'48353' = '45020'
'48355' = '18580'
'48361' = '13140'
'48363' = '33420'
'48367' = '19100'
'48375' = '11100'
'48381' = '11100'
'48389' = '37780'
'48393' = '37420'
'48395' = '17780'
'48397' = '19100'
'48401' = '30980'
'48407' = '26420'
'48409' = '18580'
'48415' = '43660'
'48423' = '46340'
'48425' = '24180'
'48427' = '40100'
'48439' = '19100'
'48441' = '10180'
'48449' = '34420'
'48451' = '41660'
'48453' = '12420'
'48459' = '30980'
'48463' = '46620'
'48465' = '19620'
'48469' = '47020'
'48471' = '26660'
'48473' = '26420'
'48477' = '14780'
'48479' = '29700'
'48481' = '20900'
'48485' = '48660'
'48487' = '46900'
'48489' = '39700'
'48491' = '12420'
'48493' = '41700'
'48497' = '19100'
'49003' = '14940'
'49005' = '30860'
'49007' = '39220'
'49011' = '36260'
'49021' = '16260'
'49023' = '39340'
'49029' = '36260'
'49035' = '41620'
'49043' = '41620'
'49045' = '41620'
'49047' = '46860'
'49049' = '39340'
'49051' = '25720'
'49053' = '41100'
'49057' = '36260'
'50003' = '13540'
'50007' = '15540'
'50009' = '13620'
'50011' = '15540'
'50013' = '15540'
'50017' = '30100'
'50021' = '40860'
'50023' = '12740'
'50027' = '30100'
'51003' = '16820'
'51007' = '40060'
'51009' = '31340'
'51011' = '31340'
'51013' = '47900'
'51015' = '44420'
'51019' = '31340'
'51023' = '40220'
'51031' = '31340'
'51033' = '40060'
'51036' = '40060'
'51041' = '40060'
'51043' = '47900'
'51045' = '40220'
'51047' = '19020'
'51049' = '40060'
'51053' = '40060'
'51059' = '47900'
'51061' = '47900'
'51065' = '16820'
'51067' = '40220'
'51069' = '49020'
'51071' = '13980'
'51073' = '47260'
'51075' = '40060'
'51079' = '16820'
'51085' = '40060'
'51087' = '40060'
'51089' = '32300'
'51093' = '47260'
'51095' = '47260'
'51097' = '40060'
'51101' = '40060'
'51107' = '47900'
'51109' = '40060'
'51115' = '47260'
'51121' = '13980'
'51125' = '16820'
'51127' = '40060'
'51143' = '19260'
'51145' = '40060'
'51149' = '40060'
'51153' = '47900'
'51155' = '13980'
'51161' = '40220'
'51165' = '25500'
'51169' = '28700'
'51177' = '47900'
'51179' = '47900'
'51181' = '47260'
'51183' = '40060'
'51185' = '14140'
'51187' = '47900'
'51191' = '28700'
'51199' = '47260'
'51510' = '47900'
'51515' = '31340'
'51520' = '28700'
'51540' = '16820'
'51550' = '47260'
'51570' = '40060'
'51590' = '19260'
'51600' = '47900'
'51610' = '47900'
'51630' = '47900'
'51650' = '47260'
'51660' = '25500'
'51670' = '40060'
'51680' = '31340'
'51683' = '47900'
'51685' = '47900'
'51690' = '32300'
'51700' = '47260'
'51710' = '47260'
'51730' = '40060'
'51735' = '47260'
'51740' = '47260'
'51750' = '13980'
'51760' = '40060'
'51770' = '40220'
'51775' = '40220'
'51790' = '44420'
'51800' = '47260'
'51810' = '47260'
'51820' = '44420'
'51830' = '47260'
'51840' = '49020'
'53003' = '30300'
'53005' = '28420'
'53007' = '48300'
'53009' = '38820'
'53011' = '38900'
'53015' = '31020'
'53017' = '48300'
'53021' = '28420'
'53025' = '34180'
'53027' = '10140'
'53029' = '36020'
'53033' = '42660'
'53035' = '14740'
'53037' = '21260'
'53041' = '16500'
'53045' = '43220'
'53053' = '42660'
'53057' = '34580'
'53059' = '38900'
'53061' = '42660'
'53063' = '44060'
'53067' = '36500'
'53071' = '47460'
'53073' = '13380'
'53075' = '39420'
'53077' = '49420'
'54003' = '25180'
'54005' = '16620'
'54009' = '48260'
'54011' = '26580'
'54015' = '16620'
'54017' = '17220'
'54019' = '36060'
'54027' = '49020'
'54029' = '48260'
'54033' = '17220'
'54037' = '47900'
'54039' = '16620'
'54043' = '16620'
'54049' = '21900'
'54051' = '48540'
'54053' = '38580'
'54055' = '14140'
'54057' = '19060'
'54061' = '34060'
'54065' = '25180'
'54069' = '48540'
'54073' = '37620'
'54077' = '34060'
'54079' = '16620'
'54081' = '13220'
'54091' = '17220'
'54099' = '26580'
'54105' = '37620'
'54107' = '37620'
'55009' = '24580'
'55015' = '11540'
'55017' = '20740'
'55021' = '31540'
'55025' = '31540'
'55027' = '13180'
'55031' = '20260'
'55033' = '32860'
'55035' = '20740'
'55037' = '27020'
'55039' = '22540'
'55043' = '38420'
'55045' = '33820'
'55049' = '31540'
'55055' = '48020'
'55059' = '16980'
'55061' = '24580'
'55063' = '29100'
'55069' = '32980'
'55071' = '31820'
'55073' = '48140'
'55075' = '31940'
'55079' = '33340'
'55083' = '24580'
'55087' = '11540'
'55089' = '33340'
'55093' = '33460'
'55097' = '44620'
'55101' = '39540'
'55105' = '27500'
'55109' = '33460'
'55111' = '12660'
'55117' = '43100'
'55127' = '48580'
'55131' = '33340'
'55133' = '33340'
'55139' = '36780'
'55141' = '49220'
'56001' = '29660'
'56005' = '23940'
'56013' = '40180'
'56021' = '16940'
'56025' = '16220'
'56033' = '43260'
'56037' = '40540'
'56039' = '27220'
'56041' = '21740'
;

/*============================================================================;
*  Title:
*          Format Library for Inpatient Quality Indicators
*
*  Description:
*          COVARIATE FORMAT LIB FOR AHRQ INPATIENT QUALITY INDICATORS;
*
*                 >>>  VERSION 2.1  <<<
*
*============================================================================;*/

 /* -------------------------------------------------------------- */
 /* ---   FORMAT FOR AGE STRATIFIERS                           --- */
 /* ---   MODULE 3. ASSIGNS ARRAY INDEX FOR AGE CATEGORIES     --- */
 /* -------------------------------------------------------------- */

VALUE AGEFMT
low - 4 = 1
 5 -  9 = 2
10 - 14 = 3
15 - 17 = 4
18 - 24 = 5
25 - 29 = 6
30 - 34 = 7
35 - 39 = 8
40 - 44 = 9
45 - 49 = 10
50 - 54 = 11
55 - 59 = 12
60 - 64 = 13
65 - 69 = 14
70 - 74 = 15
75 - 79 = 16
80 - 84 = 17
85 - high = 18;

 /* -------------------------------------------------------------- */
 /* ---   FORMATS FOR SEX STRATIFIERS                          --- */
 /* ---   ONE FORMAT PER QI.  NUMBER REFERS TO N VARIABLE IN   --- */
 /* ---   MODULE 3. ASSIGNS ARRAY INDEX FOR SEX CATEGORIES     --- */
 /* -------------------------------------------------------------- */

Value S8IQFMT
1 = 0
2 = 0
other = 0 
; 

Value S9IQFMT
1 = 0
2 = 1
other = 0 
; 

Value S11IQFMT
1 = 0
2 = 1
other = 0 
; 

Value S12IQFMT
1 = 0
2 = 1
other = 0 
; 

Value S13IQFMT
1 = 0
2 = 0
other = 0 
; 

Value S14IQFMT
1 = 0
2 = 1
other = 0 
; 

Value S15IQFMT
1 = 0
2 = 0
other = 0 
; 

Value S16IQFMT
1 = 0
2 = 1
other = 0 
; 

Value S17IQFMT
1 = 0
2 = 1
other = 0 
; 

Value S18IQFMT
1 = 0
2 = 0
other = 0 
; 

Value S19IQFMT
1 = 0
2 = 1
other = 0 
; 

Value S20IQFMT
1 = 0
2 = 1
other = 0 
; 

Value S30IQFMT
1 = 0
2 = 1
other = 0 
; 

Value S31IQFMT
1 = 0
2 = 0
other = 0 
; 

Value S32IQFMT
1 = 0
2 = 0
other = 0 
; 

 /* -------------------------------------------------------------- */
 /* ---   FORMATS FOR AGE STRATIFIERS                          --- */
 /* ---   ONE FORMAT PER QI.  NUMBER REFERS TO N VARIABLE IN   --- */
 /* ---   MODULE 3. ASSIGNS ARRAY INDEX FOR AGE CATEGORIES     --- */
 /* -------------------------------------------------------------- */

Value A8IQFMT
1 = 0
2 = 0
3 = 0
4 = 0
5 = 0
6 = 0
7 = 0
8 = 0
9 = 0
10 = 0
11 = 0
12 = 0
13 = 0
14 = 1
15 = 1
16 = 2
17 = 2
18 = 2
other = 0 
;

Value A9IQFMT
1 = 0
2 = 0
3 = 0
4 = 0
5 = 0
6 = 0
7 = 0
8 = 0
9 = 0
10 = 0
11 = 0
12 = 0
13 = 0
14 = 2
15 = 2
16 = 3
17 = 3
18 = 3
other = 0 
;

Value A11IQFMT
1 = 0
2 = 0
3 = 0
4 = 0
5 = 0
6 = 0
7 = 0
8 = 0
9 = 0
10 = 0
11 = 0
12 = 0
13 = 0
14 = 2
15 = 2
16 = 3
17 = 4
18 = 5
other = 0 
;

Value A12IQFMT
1 = 0
2 = 0
3 = 0
4 = 0
5 = 0
6 = 0
7 = 0
8 = 0
9 = 2
10 = 2
11 = 3
12 = 4
13 = 0
14 = 5
15 = 5
16 = 5
17 = 5
18 = 6
other = 0 
;

Value A13IQFMT
1 = 0
2 = 0
3 = 0
4 = 0
5 = 1
6 = 2
7 = 2
8 = 2
9 = 2
10 = 2
11 = 2
12 = 2
13 = 0
14 = 3
15 = 3
16 = 3
17 = 3
18 = 3
other = 0 
;

Value A14IQFMT
1 = 0
2 = 0
3 = 0
4 = 0
5 = 2
6 = 2
7 = 2
8 = 2
9 = 2
10 = 2
11 = 2
12 = 2
13 = 0
14 = 3
15 = 3
16 = 3
17 = 3
18 = 3
other = 0 
;

Value A15IQFMT
1 = 0
2 = 0
3 = 0
4 = 0
5 = 1
6 = 1
7 = 1
8 = 1
9 = 2
10 = 3
11 = 4
12 = 5
13 = 0
14 = 6
15 = 6
16 = 6
17 = 7
18 = 8
other = 0 
;

Value A16IQFMT
1 = 0
2 = 0
3 = 0
4 = 0
5 = 2
6 = 2
7 = 2
8 = 2
9 = 2
10 = 2
11 = 3
12 = 4
13 = 0
14 = 5
15 = 5
16 = 5
17 = 5
18 = 6
other = 0 
;

Value A17IQFMT
1 = 0
2 = 0
3 = 0
4 = 0
5 = 2
6 = 2
7 = 2
8 = 2
9 = 2
10 = 2
11 = 2
12 = 2
13 = 0
14 = 3
15 = 3
16 = 3
17 = 3
18 = 4
other = 0 
;

Value A18IQFMT
1 = 0
2 = 0
3 = 0
4 = 0
5 = 1
6 = 1
7 = 1
8 = 1
9 = 1
10 = 1
11 = 1
12 = 1
13 = 0
14 = 2
15 = 2
16 = 2
17 = 2
18 = 2
other = 0 
;

Value A19IQFMT
1 = 0
2 = 0
3 = 0
4 = 0
5 = 0
6 = 0
7 = 0
8 = 0
9 = 0
10 = 0
11 = 0
12 = 0
13 = 0
14 = 0
15 = 2
16 = 2
17 = 2
18 = 3
other = 0 
;

Value A20IQFMT
1 = 0
2 = 0
3 = 0
4 = 0
5 = 2
6 = 3
7 = 4
8 = 5
9 = 6
10 = 7
11 = 8
12 = 9
13 = 0
14 = 10
15 = 10
16 = 10
17 = 10
18 = 11
other = 0 
;

Value A30IQFMT
1 = 0
2 = 0
3 = 0
4 = 0
5 = 0
6 = 0
7 = 0
8 = 0
9 = 2
10 = 2
11 = 2
12 = 2
13 = 0
14 = 3
15 = 3
16 = 4
17 = 5
18 = 6
other = 0 
;

Value A31IQFMT
1 = 0
2 = 0
3 = 0
4 = 0
5 = 1
6 = 1
7 = 1
8 = 1
9 = 1
10 = 1
11 = 1
12 = 1
13 = 0
14 = 2
15 = 2
16 = 2
17 = 2
18 = 2
other = 0 
;

Value A32IQFMT
1 = 0
2 = 0
3 = 0
4 = 0
5 = 1
6 = 1
7 = 1
8 = 1
9 = 2
10 = 3
11 = 4
12 = 5
13 = 0
14 = 6
15 = 6
16 = 6
17 = 6
18 = 7
other = 0 
;

 /* -------------------------------------------------------------- */
 /* --- FORMATS FOR APR-DRG STRATIFIERS     ---------------------- */
 /* ---   ONE FORMAT PER IQI.  NUMBER REFERS TO N VARIABLE IN  --- */
 /* ---   PROGRAM P3. ASSIGNS ARRAY INDEX FOR APR-DRG          --- */
 /* ---   CATEGORIES (PROVIDER ONLY).                          --- */
 /* -------------------------------------------------------------- */

Value $C8IQFM
'2201' = 0  
'2202' = 0  
'2203' = 3  
'2204' = 3  
other = -1;

Value $C9IQFM
'2601' = 0
'2602' = 0
'2603' = 4
'2604' = 4
other = -1;

Value $C11IQFM
'1691' = 6
'1692' = 6
'1693' = 7
'1694' = 7
'1731' = 0
'1732' = 0
'1733' = 8
'1734' = 8
other = -1;

Value $C12IQFM
'1611' = 7
'1612' = 7
'1613' = 8
'1614' = 9
'1621' = 10
'1622' = 10
'1623' = 11
'1624' = 12
'1631' = 13
'1632' = 13
'1633' = 14
'1634' = 15
'1651' = 0
'1652' = 16
'1653' = 17
'1654' = 18
'1661' = 19
'1662' = 19
'1663' = 20
'1664' = 21
other = -1;

Value $C13IQFM
'0211' = 0
'0212' = 4
'0213' = 5
'0214' = 6
'0221' = 7
'0222' = 7
'0223' = 8
'0224' = 9
'0231' = 10
'0232' = 10
'0233' = 11
'0234' = 11
'0241' = 12
'0242' = 12
'0243' = 13
'0244' = 14
other = -1;

Value $C14IQFM
'3011' = 0
'3012' = 0
'3013' = 4
'3014' = 4
other = -1;

Value $C15IQFM
'1611' = 9
'1612' = 9
'1613' = 10
'1614' = 10
'1621' = 11
'1622' = 11
'1623' = 12
'1624' = 13
'1651' = 14
'1652' = 14
'1653' = 15
'1654' = 16
'1731' = 17
'1732' = 17
'1733' = 17
'1734' = 17
'1741' = 0
'1742' = 18
'1743' = 19
'1744' = 20
'1901' = 21
'1902' = 22
'1903' = 23
'1904' = 24
other = -1;

Value $C16IQFM
'1611' = 7
'1612' = 8
'1613' = 9
'1614' = 10
'1621' = 11
'1622' = 11
'1623' = 12
'1624' = 13
'1751' = 14
'1752' = 14
'1753' = 14
'1754' = 15
'1801' = 16
'1802' = 17
'1803' = 18
'1804' = 19
'1911' = 20
'1912' = 20
'1913' = 21
'1914' = 22
'1941' = 0
'1942' = 23
'1943' = 24
'1944' = 25
other = -1;

Value $C17IQFM
'0211' = 5
'0212' = 6
'0213' = 7
'0214' = 8
'0221' = 9
'0222' = 10
'0223' = 11
'0224' = 11
'0231' = 12
'0232' = 12
'0233' = 13
'0234' = 14
'0241' = 15
'0242' = 16
'0243' = 17
'0244' = 18
'0261' = 19
'0262' = 19
'0263' = 19
'0264' = 20
'0441' = 21
'0442' = 22
'0443' = 23
'0444' = 24
'0451' = 0
'0452' = 25
'0453' = 26
'0454' = 27
other = -1;

Value $C18IQFM
'2201' = 3
'2202' = 4
'2203' = 5
'2204' = 6
'2211' = 7
'2212' = 8
'2213' = 9
'2214' = 10
'2411' = 11
'2412' = 11
'2413' = 11
'2414' = 12
'2421' = 13
'2422' = 13
'2423' = 13
'2424' = 14
'2441' = 15
'2442' = 15
'2443' = 16
'2444' = 17
'2531' = 0
'2532' = 18
'2533' = 19
'2534' = 20
'2541' = 21
'2542' = 21
'2543' = 21
'2544' = 22
other = -1;

Value $C19IQFM
'3011' = 4
'3012' = 4
'3013' = 5
'3014' = 6
'3081' = 0
'3082' = 7
'3083' = 8
'3084' = 9
'3401' = 10
'3402' = 11
'3403' = 12
'3404' = 13
other = -1;

Value $C20IQFM
'1211' = 12
'1212' = 13
'1213' = 14
'1214' = 15
'1301' = 16
'1302' = 17
'1303' = 18
'1304' = 18
'1371' = 19
'1372' = 20
'1373' = 21
'1374' = 22
'1391' = 0
'1392' = 23
'1393' = 24
'1394' = 25
other = -1;

Value $C30IQFM
'1651' = 0
'1652' = 0
'1653' = 7
'1654' = 7
'1741' = 8
'1742' = 9
'1743' = 10
'1744' = 11
'1751' = 0
'1752' = 12
'1753' = 13
'1754' = 14
other = -1;

Value $C31IQFM
'0241' = 0
'0242' = 3
'0243' = 3
'0244' = 3
other = -1;

Value $C32IQFM
'1611' = 8
'1612' = 8
'1613' = 8
'1614' = 8
'1621' = 9
'1622' = 9
'1623' = 10
'1624' = 11
'1651' = 12
'1652' = 12
'1653' = 13
'1654' = 14
'1731' = 15
'1732' = 15
'1733' = 15
'1734' = 15
'1741' = 0
'1742' = 16
'1743' = 17
'1744' = 18
'1901' = 19
'1902' = 20
'1903' = 21
'1904' = 22
other = -1;

 /* -------------------------------------------------------------- */
 /* --- FORMATS FOR MDC STRATIFIERS         ---------------------- */
 /* -------------------------------------------------------------- */

VALUE M8IQFMT
6 = 4
OTHER = 5
;

VALUE M9IQFMT
7 = 5
OTHER = 6
;

VALUE M11IQFMT
5 = 9
OTHER = 10
;

VALUE M12IQFMT
5 = 22
OTHER = 23
;

VALUE M13IQFMT
1 = 15
OTHER = 0
;

VALUE M14IQFMT
8 = 5
OTHER = 6
;

VALUE M15IQFMT
5 = 25
OTHER = 0
;

VALUE M16IQFMT
OTHER = 26
;

VALUE M17IQFMT
OTHER = 28
;

VALUE M18IQFMT
OTHER = 23
;

VALUE M19IQFMT
8 =  14
24 = 15
OTHER = 0
;

VALUE M20IQFMT
4 = 26
25 = 27
OTHER = 0
;

VALUE M30IQFMT
4 = 15
5 = 16
8 = 17
18 = 18
OTHER = 19
;

VALUE M31IQFMT
1 = 4
5 = 5
OTHER = 6
;

VALUE M32IQFMT
5 = 23
OTHER = 0
;

 /* -------------------------------------------------------------- */

RUN;
