*============================================================================;
* Title:
*     Format Library for Pediatric Quality Indicators Program;
*
* Description:
*     FORMAT LIB FOR AHRQ PEDIATRIC QUALITY INDICATORS;
*
*     >>>     VERSION 5.0 ICD-10-CM Enhanced Alpha, DECEMBER 2014      <<<
*
*     Note:  Codes valid from 10/1/1994 to 9/30/2015 (FY 2015)
*============================================================================;
FILENAME CONTROL "C:\PATHNAME\CONTROL_PDI.SAS";    *<==USER MUST modify;

%INCLUDE CONTROL;


PROC FORMAT DDNAME=LIBRARY;

 /* -------------------------------------------------------------- */
 /* --- FORMATS FOR STRATIFIERS ---------------------------------- */
 /* -------------------------------------------------------------- */

 /* --- FORMAT FOR SEX CATEGORY ---------------------------------- */
VALUE SEXCAT
 1 = 'Male'
 2 = 'Female'
  ;

 /* --- FORMAT FOR AGE CATEGORY ---------------------------------- */
VALUE PAGECAT
 1 = "<1"
 2 = "1 TO 2"
 3 = "3 TO 5"
 4 = "6 TO 12"
 5 = "13 TO 17"
 ;

 /* --- FORMAT FOR AGEDAY CATEGORY ------------------------------- */
VALUE AGEDCAT
 0 = "NA"
 1 = "0 TO 28"
 2 = "29 TO 60"
 3 = "61 TO 90"
 4 = "91+"
 ;

 /* --- FORMAT FOR BIRTH WEIGHT CATEGORY ------------------------- */
VALUE BWHTCAT
 0 = "NA"
 1 = "0 TO 499g"
 2 = "500 TO 749g"
 3 = "750 TO 999g"
 4 = "1000 TO 1249g"
 5 = "1250 TO 1499g"
 6 = "1500 TO 1749g"
 7 = "1750 TO 1999g"
 8 = "2000 TO 2499g"
 9 = "2500g or more"
 ;

 /* --- FORMAT FOR RACE CATEGORY --------------------------------- */
VALUE RACECAT
 1 = 'White'
 2 = 'Black'
 3 = 'Hispanic'
 4 = 'Asian and NH/PI'
 5 = 'Amer Indian/AN'
 6 = 'Other'
  ;

 /* --- FORMAT FOR PAYER CATEGORY ---------------------------------- */
VALUE PAYCAT
 1 = 'Medicare'
 2 = 'Medicaid'
 3 = 'Private'
 4 = 'Self Pay'
 5 = 'No Charge'
 6 = 'Other'
  ;

 /* --- FORMAT FOR POPULATION CATEGORY ----------------------------- */
VALUE POPCAT
 1 = '0 to 4'
 2 = '5 to 9'
 3 = '10 to 14'
 4 = '15 to 17'
  ;

/* -------------------------------------------------------------- */
 /* --- FORMATS FOR stratification variables--------------------- */
 /* -------------------------------------------------------------- */

  /* ---ACCIDENTAL PUNCTURE AND LACERATION------------- */

  VALUE GP01CAT
  1='Eye, ear, nose, mouthm throat, skin, breast and other low-risk procedures'
  2='Thoracic, cardiovascular, and specified neoplastic procedures'
  3='Kidney, and male/female reproductive procedures'
  4='Infectious, immunological, hematological, and ungroupable procedures'
  5='Trauma, orthopedic, and neurologic procedures'
  6='Gastrointestinal, hepatobiliary, and endocrine procedures';


  /* --- DECUBITUS ULCER------------- */

  VALUE GP02CAT
  1='Low risk'
  2='High risk';


  /* --- POSTOPERATIVE HEMORRHAGE OR HEMATOMA------------- */

  VALUE GP08CAT
  1='Low risk'
  2='High risk';

  /* --- POSTOPERATIVE SEPSIS------------- */

  VALUE GP10CAT
  1='Clean, elective'
  2='Clean, non-elective'
  3='Contaminated, elective'
  4='Contaminated, non-elective'
  5='Dirty';


  /* --- POSTOPERATIVE WOUND DEHISCENCE------------- */

  VALUE GP11CAT
  1='Clean, elective'
  2='Clean, non-elective'
  3='Contaminated, elective'
  4='Contaminated, non-elective'
  5='Dirty';


   /* --- SELECTED INFECTION DUE TO MEDICAL CARE----- */

 VALUE GP12CAT
  1='Low risk immunodeficient'
  2='Intermediate risk immunodeficient'
  3='High risk immunodeficient';


 * -------------------------------------------------------------- ;
 * --- PDI DEFINING FORMATS ------------------------------------- ;
 * -------------------------------------------------------------- ;

 * -------------------------------------------------------------- ;
 * --- FORMATS - PROVIDER AND AREA LEVEL INDICATORS ------------- ;
 * -------------------------------------------------------------- ;
 /* ----------------------------------------- */
 /* Palliative Care                           */
 /* ----------------------------------------- */

/* VALUE $PALLIAD no mapping for this from UCD */

 /* ------------------------------ */
 /* NEONATE AND NEWBORN            */
 /* ------------------------------ */

/* VALUE $V29D  according to UCD no acceptable ICD-10 code */

 /* ---------------------------------------------------------------- */
 /* ADULT DRGS                                                       */
 /* ---------------------------------------------------------------- */

 /* DIAGNOSTIC RELATED GROUPS (DRGS): */

VALUE $ADULTDR
"001" = "1" /* CRANIOTOMY, AGE GREATER THAN 17 EXCEPT FOR TRAUMA */
"002" = "1" /* CRANIOTOMY FOR TRAUMA, AGE GREATER THAN 17 */
"024" = "1" /* SEIZURE AND HEADACHE, AGE GREATER THAN 17 WITH CC */
"025" = "1" /* SEIZURE AND HEADACHE, AGE GREATER THAN 17 WITHOUT CC */
"028" = "1" /* TRAUMATIC STUPOR AND COMA, COMA LESS THAN ONE HOUR, AGE GREATER THAN 17 WITH CC */
"029" = "1" /* TRAUMATIC STUPOR AND COMA, COMA LESS THAN ONE HOUR, AGE GREATER THAN 17 WITHOUT CC */
"031" = "1" /* CONCUSSION, AGE GREATER THAN 17 WITH CC */
"032" = "1" /* CONCUSSION, AGE GREATER THAN 17 WITHOUT CC */
"040" = "1" /* EXTRAOCULAR PROCEDURES EXCEPT ORBIT, AGE GREATER THAN 17 */
"046" = "1" /* OTHER DISORDERS OF THE EYE, AGE GREATER THAN 17 WITH CC */
"047" = "1" /* OTHER DISORDER OF THE EYE, AGE GREATER THAN 17 WITHOUT CC */
"053" = "1" /* SINUS AND MASTOID PROCEDURES, AGE GREATER THAN 17 */
"057" = "1" /* TONSILLECTOMY AND ADENOIDECTOMY PROCEDURES EXCEPT TONSILLECTOMY AND/OR ADENOIDECTOMY ONLY, AGE GREATER THAN 17 */
"059" = "1" /* TONSILLECTOMY AND/OR ADENOIDECTOMY ONLY, AGE GREATER THAN 17 */
"061" = "1" /* MYRINGOTOMY WITH TUBE INSERTION, AGE GREATER THAN 17 */
"068" = "1" /* OTITIS MEDIA AND URI, AGE GREATER THAN 17 WITH CC */
"069" = "1" /* OTITIS MEDIA AND URI, AGE GREATER THAN 17 WITHOUT CC */
"073" = "1" /* OTHER EAR, NOSE, MOUTH AND THROAT DIAGNOSES, AGE GREATER THAN 17 */
"079" = "1" /* RESPIRATORY INFECTIONS AND INFLAMMATIONS, AGE GREATER THAN 17 WITH CC */
"080" = "1" /* RESPIRATORY INFECTIONS AND INFLAMMATIONS, AGE GREATER THAN 17 WITHOUT CC */
"089" = "1" /* SIMPLE PNEUMONIA AND PLEURISY, AGE GREATER THAN 17 WITH CC */
"090" = "1" /* SIMPLE PNEUMONIA AND PLEURISY, AGE GREATER THAN 17 WITHOUT CC */
"096" = "1" /* BRONCHITIS AND ASTHMA, AGE GREATER THAN 17 WITH CC */
"097" = "1" /* BRONCHITIS AND ASTHMA, AGE GREATER THAN 17 WITHOUT CC */
"135" = "1" /* CARDIAC CONGENITAL AND VALVULAR DISORDERS, AGE GREATER THAN 17 WITH CC */
"136" = "1" /* CARDIAC CONGENITAL AND VALVULAR DISORDERS, AGE GREATER THAN 17 WITHOUT CC */ 
"154" = "1" /* STOMACH, ESOPHAGEAL AND DUODENAL PROCEDURES, AGE GREATER THAN 17 WITH CC */
"155" = "1" /* STOMACH, ESOPHAGEAL AND DUODENAL PROCEDURES, AGE GREATER THAN 17 WIHOUT CC*/
"159" = "1" /* HERMIA PROCEDURES EXCEPT INGUINAL AND FEMORAL , AGE GREATER THAN 17 WITH CC */
"160" = "1" /* HERNIA PROCEDURES EXCEPT INGUINAL AND FEMORAL, AGE GREATER THAN 17 WITHOUT CC */
"161" = "1" /* INGUINAL AND FEMORAL HERNIA PROCEDURES, AGE GREATER THAN 17 WITH CC */
"162" = "1" /* INGUINAL AND FEMORAL HERNIA PROCEDURES, AGE GREATER THAN 17 WITHOUT CC */
"182" = "1" /* ESOPHAGITIS, GASTROENTERITIS AND MISCELLANEOUS DIGESTIVE DISORDERS, AGE GREATER THAN 17 WITH CC */
"183" = "1" /* ESOPHAGITIS, GASTROENTERITIS AND MISCELLANEOUS DIGESTIVE DISORDERS, AGE GREATER THAN 17 WITHOUT CC */
"185" = "1" /* DENTAL AND ORAL DISEASES EXCEPT EXTRACTIONS AND RESTORATIONS, AGE GREATER THAN 17 */
"188" = "1" /* OTHER DIGESTIVE SYSTEM DIAGNOSES, AGE GREATER THAN 17 WITH CC */
"189" = "1" /* OTHER DIGESTIVE SYSTEM DIAGNOSES, AGE GREATER THAN 17 WITHOUT CC */
"210" = "1" /* HIP AND FEMUR PROCEDURES EXCEPT MAJOR JOINT PROCEDURES, AGE GREATER THAN 17 WITH CC */
"211" = "1" /* HIP AND FEMUR PROCEDURES EXCEPT MAJOR JOINT PROCEDURES, AGE GREATER THAN 17 WITHOUT CC */
"218" = "1" /* LOWER EXTREMITY AND HUMERUS PROCEURES EXCEPT HIP, FOOT AND FEMUR, AGE GREATER THAN 17 WITH CC */
"219" = "1" /* LOWER EXTREMITY AND HUMERUS PROCEDURES EXCEPT HIP, FOOT AND FEMUR, AGE GREATER THAN 17 WITHOUT CC */
"250" = "1" /* FRACTURES, SPRAINS, STRAINS AND DISLOCATIONS OF FOREARM, HAND AND FOOT, AGE GREATER THAN 17 WITH CC */
"251" = "1" /* FRACTURES, SPRAINS, STRAINS AND DISLOCATIONS OF FOREARM, HAND AND FOOT, AGE GREATER THAN 17 WITHOUT CC */
"253" = "1" /* FRACTURES, SPRAINS, STRAINS AND DISLOCATIONS OF UPPER ARM AND LOWER LEG EXCEPT FOOT, AGE GREATER THAN 17 WITH CC */
"254" = "1" /* FRACTURES, SPRAINS, STRAINS AND DISLOCATIONS OF UPPER ARM AND LOWER LEG EXCEPT FOOT, AGE GREATER THAN 17 WITHOUT CC */
"277" = "1" /* CELLULITIS, AGE GREATER THAN 17 WITH CC */
"278" = "1" /* CELLULTIIS, AGE GREATER THAN 17 WITHOUT CC */
"280" = "1" /* TRAUMA TO SKIN, SUBCUTANEOUS TISSUE AND BREAST, AGE GREATER THAN 17 WITH CC */
"281" = "1" /* TRAUMA TO SKIN, SUBCUTANEOUS TISSUE AND BREAST, AGE GREATER THAN 17 WITHOUT CC */
"294" = "1" /* DIABETES, AGE GREATER THAN 35 */
"296" = "1" /* NUTRITIONAL AND MISCELLANEOUS METABOLIC DISORDERS, AGE GREATER THAN 17 WITH CC */
"297" = "1" /* NUTRITIONAL AND MISCELLANEOUS METABOLIC DISORDERS, AGE GREATER THAN 17 WITHOUT CC */
"312" = "1" /* URETHRAL PROCEDURES, AGE GREATER THAN 17 WITH CC */
"313" = "1" /* URETHRAL PROCEDURES, AGE GREATER THAN 17 WITHOUT CC */
"320" = "1" /* KIDNEY AND URINARY TRACT INFECTIONS, AGE GREATER THAN 17 WITH CC */
"321" = "1" /* KIDNEY AND URINARY TRACT INFECTIONS, AGE GREATER THAN 17 WITHOUT CC */
"325" = "1" /* KIDNEY AND URINARY TRACT SIGNS AND SYMPTOMS, AGE GREATER THAN 17 WITH CC */
"326" = "1" /* KIDNEY AND URINARY TRACT SIGNS AND SYMPTOMS, AGE GREATER THAN 17 WITHOUT CC */
"328" = "1" /* URETHRAL STRICTURE, AGE GREATER THAN 17 WITH CC */
"329" = "1" /* URETHRAL STRICTURE, AGE GREATER THAN 17 WITHOUT CC */
"331" = "1" /* OTHER KIDNEY AND URINARY TRACT DIAGNOSES, AGE GREATER THAN 17 WITH CC */
"332" = "1" /* OTHER KIDNEY AND URINARY TRACT DIAGNOSES, AGE GREATER THAN 17 WITHOUT CC */
"339" = "1" /* TESTES PROCEDURES FOR NONMALIGNANCY, AGE GREATER THAN 17 */
"342" = "1" /* CIRCUMCISION, AGE GREATER THAN 17 */
"392" = "1" /* SPLENECTOMY, AGE GREATER THAN 17 */
"395" = "1" /* RED BLOOD CELL DISORDERS, AGE GREATER THAN 17 */
"416" = "1" /* SEPTICEMIA, AGE GREATER THAN 17 */
"419" = "1" /* FEVER OF UNKNOWN ORIGIN, AGE GREATER THAN 17 WITH CC */
"420" = "1" /* FEVER OF UNKNOWN ORIGIN, AGE GREATER THAN 17 WITHOUT CC */
"421" = "1" /* VIRAL ILLNESS, AGE GREATER THAN 17 */
"444" = "1" /* TRAUMATIC INJURY, AGE GREATER THAN 17 WITH CC */
"445" = "1" /* TRAUMATIC INJURY, AGE GREATER THAN 17 WITHOUT CC */
"447" = "1" /* ALLERGIC REACTIONS, AGE GREATER THAN 17 */
"449" = "1" /* POISONING AND TOXIC EFFECTS OF DRUGS, AGE GREATER THAN 17 WITH CC */
"450" = "1" /* POISONING AND TOXIC EFFECTS OF DRUGS, AGE GREATER THAN 17 WITHOUT CC */
"473" = "1" /* ACUTE LEUKEMIA WITHOUT MAJOR OR PROCEDURE, AGE GREATER THAN 17 */
"562" = "1" /* SEIZURE AGE > 17 W CC */
"563" = "1" /* SEIZURE AGE > 17 W/O CC */
"564" = "1" /* HEADACHES AGE >17 */
"567" = "1" /* STOMACH, ESOPHAGEAL & DUODENAL PROC AGE > 17 W CC W MAJOR GI DX */
"568" = "1" /* STOMACH, ESOPHAGEAL & DUODENAL PROCEDURES PROC AGE > 17 W CC W/O MAJOR GI DX */
"575" = "1" /* SEPTICEMIA W MV96+ HOURS  AGE  >17 */
"576" = "1" /* SEPTICEMIA W/O MV96+ HOURS AGE  >17 */
OTHER  = "0" ;

 /* ---------------------------------------------------------------- */
 /* MEDICAL DRGS (deleted Adult DRGs)                                */
 /* ---------------------------------------------------------------- */

 /* DIAGNOSTIC RELATED GROUPS (DRGS): */

VALUE $MEDICDR
"009" = "1" /* SPINAL DISORDERS AND INJURIES */
"010" = "1" /* NERVOUS SYSTEM NEOPLASMS WITH CC */
"011" = "1" /* NERVOUS SYSTEM NEOPLASMS WITH CC */
"012" = "1" /* DEGENERATIVE NERVOUS SYSTEM DISORDERS */
"013" = "1" /* MULTIPLE SCLEROSIS AND CEREBELLAR ATAXIA */
"014" = "1" /* SPECIFIC CEREBROVASCULAR DISORDERS EXCEPT TRANSIENT ISCHEMIC ATTACK */
"015" = "1" /* TRANSIENT ISCHEMIC ATTACK AND PRECEREBRAL OCCLUSIONS */
"016" = "1" /* NONSPECIFIC CEREBROVASCULAR DISORDERS WITH CC */
"017" = "1" /* NONSPECIFIC CEREBROVASCULAR DISORDERS WITHOUT CC */
"018" = "1" /* CRANIAL AND PERIPHERAL NERVE DISORDERS WITH CC */
"019" = "1" /* CRANIAL AND PERIPHERAL NERVE DISORDERS WITHOUT CC */
"020" = "1" /* NERVOUS SYSTEM INFECTION EXCEPT VIRAL MENINGITIS */
"021" = "1" /* VIRAL MENINGITIS */
"022" = "1" /* HYPERTENSIVE ENCEPHALOPATHY */
"023" = "1" /* NONTRAUMATIC STUPOR AND COMA */
"026" = "1" /* SEIZURE AND HEADACHE, AGE 0-17 */
"027" = "1" /* TRAUMATIC STUPOR AND COMA, COMA GREATER THAN ONE HOUR */
"030" = "1" /* TRAUMATIC STUPOR AND COMA, COMA LESS THAN ONE HOUR, AGE 0-17 */
"033" = "1" /* CONCUSSION, AGE 0-17 */
"034" = "1" /* OTHER DISORDERS OF NERVOUS SYSTEM WITH CC */
"035" = "1" /* OTHER DISORDERS OF NERVOUS SYSTEM WITHOUT CC */
"043" = "1" /* HYPHEMA */
"044" = "1" /* ACUTE MAJOR EYE INFECTIONS */
"045" = "1" /* NEUROLOGICAL EYE DISORDERS */
"048" = "1" /* OTHER DISORDERS OF THE EYE, AGE 0-17 */
"064" = "1" /* EAR, NOSE, MOUTH AND THROAT MALIGNANCY */
"065" = "1" /* DISEQUILIBRIA */
"066" = "1" /* EPISTAXIS */
"067" = "1" /* EPIGLOTTITIS */
"070" = "1" /* OTITIS MEDIA AND URI, AGE 0-17 */
"071" = "1" /* LARYNGOTRACHEITIS */
"072" = "1" /* NASAL TRAUMA AND DEFORMITY */
"074" = "1" /* OTHER EAR, NOSE, MOUTH AND THROAT DIAGNOSES, AGE 0-17 */
"078" = "1" /* PULMONARY EMBOLISM */
"081" = "1" /* RESPIRATORY INFECTIONS AND INFLAMMATIONS, AGE 0-17 */
"082" = "1" /* RESPIRATORY NEOPLASMS */
"083" = "1" /* MAJOR CHEST TRAUMA WITH CC */
"084" = "1" /* MAJOR CHEST TRAUMA WITHOUT CC*/
"085" = "1" /* PLEURAL EFFUSION WITH CC */
"086" = "1" /* PLEURAL EFFUSION WITHOUT CC */
"087" = "1" /* PULMONARY EDEMA AND RESPIRATORY FAILURE */
"088" = "1" /* CHRONIC OBSTRUCTIVE PULMONARY DISEASE */
"091" = "1" /* SIMPLE PNEUMONIA AND PLEURISY, AGE 0-17 */
"092" = "1" /* INTERSTITIAL LUNG DISEASE WITH CC */
"093" = "1" /* INTERSTITIAL LUNG DISEASE WITHOUT CC */
"094" = "1" /* PNEUMOTHORAX WITH CC */
"095" = "1" /* PNEUMOTHORAX WITHOUT CC */
"098" = "1" /* BRONCHITIS AND ASTHMA, AGE 0-17 */
"099" = "1" /* RESPIRATORY SIGNS AND SYMPTOMS WITH CC */
"100" = "1" /* RESPIRATORY SIGNS AND SYMPTOMS WITHOUT CC */
"101" = "1" /* OTHER RESPIRATORY SYSTEM DIAGNOSES WITH CC */
"102" = "1" /* OTHER RESPIRATORY SYSTEM DIAGNOSES WITHOUT CC */
"121" = "1" /* CIRCULATORY DISORDERS WITH ACUTE MYOCARDIAL INFARCTION AND MAJOR COMPLICATION, DISCHARGED ALIVE */
"122" = "1" /* CIRCULATORY DISORDERS WITH ACUTE MYOCARDIAL INFARCTION WITHOUT MAJOR COMPLICATION, DISCHARGED ALIVE */
"123" = "1" /* CIRCULATORY DISORDERS WITH ACUTE MYOCARDIAL INFARCTION, EXPIRED */
"124" = "1" /* CIRCULATORY DISORDERS EXCEPT ACUTE MYOCARDIAL INFARCTION WITH CARDIAC CATHETERIZATION AND COMPLEX DIAGNOSIS */
"125" = "1" /* CIRCULATORY DISORDERS EXCEPT ACUTE MYOCARDIAL INFARCTION WITH CARDIAC CATHETERIZATION WITHOUT COMPLEX DIAGNOSIS */
"126" = "1" /* ACUTE AND SUB ACUTE ENDOCARDITIS */
"127" = "1" /* HEART FAILURE AND SHOCK */
"128" = "1" /* DEEP VEIN THROMBOPHLEBITIS */
"129" = "1" /* CARDIAC ARREST, UNEXPLAINED */
"130" = "1" /* PERIPHERAL VASCULAR DISORDERS WITH CC */
"131" = "1" /* PERIPHERAL VASCULAR DISORDERS WITHOUT CC */
"132" = "1" /* ATHEROSCLEROSIS WITH CC */
"133" = "1" /* ATHEROSCLEROSIS WITHOUT CC */
"134" = "1" /* HYPERTENSION */
"137" = "1" /* CARDIAC CONGENITAL AND VALVULAR DISORDERS, AGE 0 - 17*/
"138" = "1" /* CARDIAC ARRHYTHMIA AND CONDUCTION DISORDERS WITH CC */
"139" = "1" /* CARDIAC ARRHYTHMIA AND CONDUCTION DISORDERS WITHOUT CC */
"140" = "1" /* ANGINA PECTORIS*/
"141" = "1" /* SYNCOPE AND COLLAPSE WITH CC */
"142" = "1" /* SYNCOPE AND COLLAPSE WITHOUT CC */
"143" = "1" /* CHEST PAIN */
"144" = "1" /* OTHER CIRCULATORY SYSTEM DIAGNOSES WITH CC*/
"145" = "1" /* OTHER CIRCULATORY SYSTEM DIAGNOSES WITHOUT CC */
"172" = "1" /* DIGESTIVE MALIGNANCY WITH CC */
"173" = "1" /* DIGESTIVE MALIGNANCY WITHOUT CC */
"174" = "1" /* GI HEMORRHAGE WITH CC*/
"175" = "1" /* GI HEMORRHAGE WITHOUT CC */
"176" = "1" /* COMPLICATED PEPTIC ULCER */
"177" = "1" /* UNCOMPLICATED PEPTIC ULCER WITH CC */
"178" = "1" /* UNCOMPLICATED PEPTIC ULCER WITHOUT CC*/
"179" = "1" /* INFLAMMATORY BOWEL DISEASE */
"180" = "1" /* GI OBSTRUCTION WITH CC */
"181" = "1" /* GI OBSTRUCTION WITHOUT CC */
"184" = "1" /* ESOPHAGITIS, GASTROENTERITIS AND MISCELLANESOU DIGESTIVE DISORDERS, AGE 0-17 */
"186" = "1" /* DENTAL AND ORAL DISEASED EXCEPT EXTRACTIONS AND RESTORATIONS, AGE 0-17 */
"187" = "1" /* DENTAL EXTRACTIONS AND RESTORATIONS */
"190" = "1" /* OTHER DIGESTIVE SYSTEM DIAGNOSES, AGE 0-17 */
"202" = "1" /* CIRRHOSIS AND ALCOHOLIC HEPATITIS */
"203" = "1" /* MALIGNANCY OF HEPATOBILIARY SYSTEM OR PANCREAS */
"204" = "1" /* DISORDERS OF PANCREAS EXCEPT MALIGNANCY */
"205" = "1" /* DISORDERS OF LIVER EXCEPT MALIGNANCY, CIRRHOSIS AND ALCOHOLIC HEPATITIS WITH CC */
"206" = "1" /* DISORDERS OF LIVER EXCEPT MALIGNANCY, CIRRHOSIS AND ALCOHOLIC HEPATITIS WITHOUT CC */
"207" = "1" /* DISORDERS OF THE BILIARY TRACT WITH CC */
"208" = "1" /* DISORDERS OF THE BILIARY TRACT WITHOUT CC */
"235" = "1" /* FRACTURES OF FEMUR */
"236" = "1" /* FRACTURES OF HIP AND PELVIS */
"237" = "1" /* SPRAINS, STRAINS AND DISLOCATIONS OF HIP, PELVIS AND THIGH */
"238" = "1" /* OSTEOMYELITIS */
"239" = "1" /* PATHOLOGICAL FRACTURES AND MUSCULOSKELETAL AND CONNECTIVE TISSUE MALIGNANCY */
"240" = "1" /* CONNECTIVE TISSUE DISORDERS WITH CC */
"241" = "1" /* CONNECTIVE TISSUE DISORDERS WITHOUT CC */
"242" = "1" /* SEPTIC ARTHRITIS */
"243" = "1" /* MEDICAL BACK PROBLEMS */
"244" = "1" /* BONE DISEASES AND SPECIFIC ARTHROPATHIES WITH CC */
"245" = "1" /* BONE DISEASES AND SPECIFIC ARTHROPATHIES WITHOUT CC */
"246" = "1" /* NONSPECIFIC ARTHROPATHIES */
"247" = "1" /* SIGNS AND SYMPTOMS OF MUSCULOSKELETAL SYSTEM AND CONNECTIVE TISSUE */
"248" = "1" /* TENDONITIS, MYOSITIS AND BURSITIS */
"249" = "1" /* AFTERCARE, MUSCULOSKELETAL SYSTEM AND CONNECTIVE TISSUE */
"252" = "1" /* FRACTURES, SPRAINS, STRAINS AND DISLOCATIONS OF FOREARM, HAND AND FOOT, AGE 0-17 */
"255" = "1" /* FRACTURES, SPRAINS, STRAINS AND DISLOCATIONS OF UPPER ARM AND LOWER LEG EXCEPT FOOT, AGE 0-17 */
"256" = "1" /* OTHER MUSCULOSKELETAL SYSTEM AND CONNECTIVE TISSUE DIAGNOSES */
"271" = "1" /* SKIN ULCERS */
"272" = "1" /* MAJOR SKIN DISORDERS WITH CC */
"273" = "1" /* MAJOR SKIN DISORDERS WITHOUT CC */
"274" = "1" /* MALIGNANT BREAST DISORDERS WITH CC */
"275" = "1" /* MALIGNANT BREAST DISORDERS WITHOUT CC */
"276" = "1" /* NONMALIGNANT BREAST DISORDERS */
"279" = "1" /* CELLULITIS, AGE 0-17 */
"282" = "1" /* TRAUMA TO SKIN, SUBCUTANEOUS TISSUE AND BREAST, AGE 0-17 */
"283" = "1" /* MINOR SKIN DISORDERS WITH CC */
"284" = "1" /* MINOR SKIN DISORDERS WITHOUT CC */
"295" = "1" /* DIABETES, AGE 0-35 */
"298" = "1" /* NUTRITIONAL AND MISCELLANEOUS METABOLIC DISORDERS, AGE 0-17 */
"299" = "1" /* INBORN ERRORS OF METABOLISM */
"300" = "1" /* ENDOCRINE DISORDERS WITH CC */
"301" = "1" /* ENDOCRINE DISORDERS WITHOUT CC */
"316" = "1" /* RENAL FAILURE */
"317" = "1" /* ADMISSION FOR RENAL DIALYSIS */
"318" = "1" /* KIDNEY AND URINARY TRACT NEOPLASMS WITH CC */
"319" = "1" /* KIDNEY AND URINARY TRACT NEOPLASMS WITHOUT CC */
"322" = "1" /* KIDNEY AND URINARY TRACT INFECTION, AGE 0-17 */
"323" = "1" /* URINARY STONES WITH CC AND/ OR ESW LITHOTRIPSY */
"324" = "1" /* URINARY STONES WITHOUT CC */
"327" = "1" /* KIDNEY AND URINARY TRACT SIGNS AND SYMPTOMS, AGE 0-17 */
"330" = "1" /* URETHRAL STRICTURE, AGE 0-17 */
"333" = "1" /* OTHER KIDNEY AND URINARY TRACT DIAGNOSES, AGE 0-17 */
"346" = "1" /* MALIGNANCY OF MALE REPRODUCTIVE SYSTEM WITH CC */
"347" = "1" /* MALIGNANCY OF MALE REPRODUCTIVE SYSTEM WITHOUT CC */
"348" = "1" /* BENIGN PROSTATIC HYPERTROPHY WITH CC */
"349" = "1" /* BENIGN PROSTATIC HYPERTROPHY WITHOUT CC */
"350" = "1" /* INFLAMMATION OF THE MALE REPRODUCTIVE SYSTEM */
"351" = "1" /* STERILIZATION, MALE */
"352" = "1" /* OTHER MALE REPRODUCTIVE SYSTEM DIAGNOSES */
"366" = "1" /* MALIGNANCY OF FEMALE REPRODUCTIVE SYSTEM WITH CC */
"367" = "1" /* MALIGNANCY OF FEMALE REPRODUCTIVE SYSTEM WITHOUT CC */
"368" = "1" /* INFECTIONS OF FEMALE REPRODUCTIVE SYSTEM */
"369" = "1" /* MENSTRUAL AND OTHER FEMALE REPRODUCTIVE SYSTEM DISORDERS */
"372" = "1" /* VAGINAL DELIVERY WITH COMPLICATING DIAGNOSES */
"373" = "1" /* VAGINAL DELIVERY WITHOUT COMPLICATING DIAGNOSES */
"376" = "1" /* POSTPARTUM AND POSTABORTION DIAGNOSES WITHOUT OR PROCEDURE */
"378" = "1" /* ENTOPIC PREGNANCY */
"379" = "1" /* THREATENED ABORTION */
"380" = "1" /* ABORTION WITHOUT D AND G */
"382" = "1" /* FALSE LABOR */
"383" = "1" /* OTHER ANTEPARTUM DIAGNOSES WITH MEDICAL COMPLICATIONS */
"384" = "1" /* OTHER ANTEPARTUM DIAGNOSES WITHOUT MEDICAL COMPLICATIONS */
"385" = "1" /* NEONATES DIED OR TRANSFERRED TO ANOTHER ACUTE FACILITY */
"386" = "1" /* EXTREME IMMATURITY OR RESPIRATORY DISTRESS SYNDROME NEONATE */
"387" = "1" /* PREMATURITY W MAJOR PROBLEMS */
"388" = "1" /* PREMATURITY W/O MAJOR PROBLEMS */
"389" = "1" /* FULL TERM NEONATE W MAJOR PROBLEMS */
"390" = "1" /* NEONATE W OTHER SIGNIFICANT PROBLEMS */
"391" = "1" /* NORMAL NEWBORN */
"396" = "1" /* RED BLOOD CELL DISORDERS, AGE 0-17 */
"397" = "1" /* COAGULATION DISORDERS */
"398" = "1" /* RETICULOENDOTHELIAL AND IMMUNITY DISORDERS WITH CC */
"399" = "1" /* RETICULOENDOTHELIAL AND IMMUNITY DISORDERS WITHOUT CC */
"403" = "1" /* LYMPHOMA AND NONACUTE LEUKEMIA WITH CC */
"404" = "1" /* LYMPHOMA AND NONACUTE LEUKEMIA WITHOUT CC */
"405" = "1" /* ACUTE LEUKEMIA WITHOUT MAJOR OR PROCEDURE, AGE 0-17 */
"409" = "1" /* RADIOTHERAPY */
"410" = "1" /* CHEMOTHERAPY WITHOUT ACUTE LEUKEMIA AS SECONDARY DIAGNOSIS */
"411" = "1" /* HISTORY OF MALIGNANCY WITHOUT ENDOSCOPY */
"412" = "1" /* HISTORY OF MALIGNANCY WITH ENDOSCOPY */
"413" = "1" /* OTHER MYELOPROLIFERATIVE DISORDERS OR POORLY DIFFERENTIATED NEOPLASM DIAGNOSES WITH CC */
"414" = "1" /* OTHER MYELOPROLIFERATIVE DISORDERS OR POORLY DIFFERENTIATED NEOPLASM DIAGNOSES WITHOUT CC */
"417" = "1" /* SEPTICEMIA, AGE 0-17 */
"418" = "1" /* POSTOPERATIVE AND POSTTRAUMATIC INFECTIONS */
"422" = "1" /* VIRAL ILLNESS AND FEVER OF UNKNOWN ORIGIN, AGE 0-17 */
"423" = "1" /* OTHER INFECTIOUS AND PARASITIC DISEASES DIAGNOSES */
"425" = "1" /* ACUTE ADJUSTMENT REACTIONS AND DISTURBANCES OF PSYCHOSOCIAL DYSFUNCTION */
"426" = "1" /* DEPRESSIVE NEUROSES */
"427" = "1" /* NEUROSES EXCEPT DEPRESSIVE */
"428" = "1" /* DISORDERS OF PERSONALITY AND IMPULSE CONTROL */
"429" = "1" /* ORGANIC DISTURBANCES AND MENTAL RETARDATION */
"430" = "1" /* PSYCHOSES */
"431" = "1" /* CHILDHOOD MENTAL DISORDERS */
"432" = "1" /* OTHER MENTAL DISORDER DIAGNOSES */
"433" = "1" /* ALCOHOL/DRUG ABUSE OR DEPENDENCE, LEFT AGAINST MEDICAL ADVICE */
"434" = "1" /* ALCOHOL/DRUG ABUSE OR DEPENDENCE, DETOXIFICATION OR OTHER SYMPTOMATIC TREATMENT WITH CC */
"435" = "1" /* ALCOHOL/DRUG ABUSE OR DEPENDENCE, DETOXIFICATION OR OTHER SYMPTOMATIC TREATMENT WITHOUT CC */
"436" = "1" /* ALCOHOL/DRUG DEPENDENCE WITH REHABILITATION THERAPY */
"437" = "1" /* ALCOHOL DRUG DEPENDENCE WITH COMBINED REHABILITATION AND DETOXIFICATION THERAPY */
"446" = "1" /* TRAUMATIC INJURY, AGE 0-17 */
"448" = "1" /* ALLERGIC REACTIONS, AGE 0-17 */
"451" = "1" /* POISONING AND TOXIC EFFECTS OF DRUGS, AGE 0-17 */
"452" = "1" /* COMPLICATIONS OF TREATMENT WITH CC */
"453" = "1" /* COMPLICATIONS OF TREATMENT WITHOUT CC */
"454" = "1" /* OTHER INJURY, POISONING AND TOXIC EFFECT DIAGNOSES WITH CC */
"455" = "1" /* OTHER INJURY, POISONING AND TOXIC EFFECT DIAGNOSES WITHOUT CC */
"456" = "1" /* BURNS, TRANSFERRED TO ANOTHER ACUTE CARE FACILITY */
"457" = "1" /* EXTENSIVE BURNS W/O O.R. PROCEDUR */
"460" = "1" /* NON-EXTENSIVE BURNS W/O O.R. PROCEDURE */
"462" = "1" /* REHABILITATION */
"463" = "1" /* SIGNS AND SYMPTOMS WITH CC */
"464" = "1" /* SIGNS AND SYMPTOMS WITHOUT CC */
"465" = "1" /* AFTERCARE WITH HISTORY OF MALIGNANCY AS SECONDARY DIAGNOSIS */
"466" = "1" /* AFTERCARE WITHOUT HISTORY OF MALIGNANCY AS SECONDARY DIAGNOSIS */
"467" = "1" /* OTHER FACTORS INFLUENCING HEALTH STATUS */
"475" = "1" /* RESPIRATORY SYSTEM DIAGNOSIS WITH VENTILATOR SUPPORT */
"487" = "1" /* OTHER MULTIPLE SIGNIFICANT TRAUMA */
"489" = "1" /* HIV WITH MAJOR RELATED CONDITION */
"490" = "1" /* HIV WITH OR WITHOUT OTHER RELATED CONDITION */
"492" = "1" /* CHEMOTHERAPY WITH ACUTE LEUKEMIA AS SECONDARY DIAGNOSIS */
"505" = "1" /* EXTENSIVE 3RD DEGREE BURNS W/O SKIN GRAFT */
"508" = "1" /* FULL THICKNESS BURN W/O SKIN GRFT OR INHAL INJ W CC OR SIG TRAUMA */
"509" = "1" /* FULL THICKNESS BURN W/O SKIN GRFT OR INH INJ W/O CC OR SIG TRAUMA */
"510" = "1" /* NON-EXTENSIVE BURNS W CC OR SIGNIFICANT TRAUMA */
"511" = "1" /* NON-EXTENSIVE BURNS W/O CC OR SIGNIFICANT TRAUMA */
"521" = "1" /* ALCOHOL/DRUG ABUSE OR DEPENDENCE W CC */
"522" = "1" /* ALC/DRUG ABUSE OR DEPEND W REHABILITATION THERAPY W/O CC */
"523" = "1" /* ALC/DRUG ABUSE OR DEPEND W/O REHABILITATION THERAPY W/O CC */
"524" = "1" /* TRANSIENT ISCHEMIA */
"559" = "1" /* ACUTE ISCHEMIC STROKE WITH USE OF THROMBOLYTIC AGENT */
"560" = "1" /* BACTERIAL & TUBERCULOUS INFECTIONS OF NERVOUS SYSTEM */
"561" = "1" /* NON-BACTERIAL INFECTIONS OF NERVOUS SYSTEM EXCEPT VIRAL MENINGITIS */
"565" = "1" /* RESPIRATORY SYSTEM DIAGNOSIS WITH VENTILATOR SUPPORT 96+ HOURS */
"566" = "1" /* RESPIRATORY SYSTEM DIAGNOSIS WITH VENTILATOR SUPPORT < 96 HOURS */
"571" = "1" /* MAJOR ESOPHAGEAL DISORDERS */
"572" = "1" /* MAJOR GASTROINTESTINAL DISORDERS AND PERITONEAL INFECTIONS */
"574" = "1" /* MAJOR HEMATOLOGIC/IMMUNOLOGIC DIAG EXC SICKLE CELL CRISIS & COAGUL */
OTHER  = "0" ;

VALUE $MEDIC2R
'052' = '1' /* Spinal disorders & injuries w CC/MCC */
'053' = '1' /* Spinal disorders & injuries w/o CC/MCC */
'054' = '1' /* Nervous system neoplasms w MCC */
'055' = '1' /* Nervous system neoplasms w/o MCC */
'056' = '1' /* Degenerative nervous system disorders w MCC */
'057' = '1' /* Degenerative nervous system disorders w/o MCC */
'058' = '1' /* Multiple sclerosis & cerebellar ataxia w MCC */
'059' = '1' /* Multiple sclerosis & cerebellar ataxia w CC */
'060' = '1' /* Multiple sclerosis & cerebellar ataxia w/o CC/MCC */
'061' = '1' /* Acute ischemic stroke w use of thrombolytic agent w MCC */
'062' = '1' /* Acute ischemic stroke w use of thrombolytic agent w CC */
'063' = '1' /* Acute ischemic stroke w use of thrombolytic agent w/o CC/MCC */
'064' = '1' /* Intracranial hemorrhage or cerebral infarction w MCC */
'065' = '1' /* Intracranial hemorrhage or cerebral infarction w CC */
'066' = '1' /* Intracranial hemorrhage or cerebral infarction w/o CC/MCC */
'067' = '1' /* Nonspecific cva & precerebral occlusion w/o infarct w MCC */
'068' = '1' /* Nonspecific cva & precerebral occlusion w/o infarct w/o MCC */
'069' = '1' /* Transient ischemia */
'070' = '1' /* Nonspecific cerebrovascular disorders w MCC */
'071' = '1' /* Nonspecific cerebrovascular disorders w CC */
'072' = '1' /* Nonspecific cerebrovascular disorders w/o CC/MCC */
'073' = '1' /* Cranial & peripheral nerve disorders w MCC */
'074' = '1' /* Cranial & peripheral nerve disorders w/o MCC */
'075' = '1' /* Viral meningitis w CC/MCC */
'076' = '1' /* Viral meningitis w/o CC/MCC */
'077' = '1' /* Hypertensive encephalopathy w MCC */
'078' = '1' /* Hypertensive encephalopathy w CC */
'079' = '1' /* Hypertensive encephalopathy w/o CC/MCC */
'080' = '1' /* Nontraumatic stupor & coma w MCC */
'081' = '1' /* Nontraumatic stupor & coma w/o MCC */
'082' = '1' /* Traumatic stupor & coma, coma >1 hr w MCC */
'083' = '1' /* Traumatic stupor & coma, coma >1 hr w CC */
'084' = '1' /* Traumatic stupor & coma, coma >1 hr w/o CC/MCC */
'085' = '1' /* Traumatic stupor & coma, coma <1 hr w MCC */
'086' = '1' /* Traumatic stupor & coma, coma <1 hr w CC */
'087' = '1' /* Traumatic stupor & coma, coma <1 hr w/o CC/MCC */
'088' = '1' /* Concussion w MCC */
'089' = '1' /* Concussion w CC */
'090' = '1' /* Concussion w/o CC/MCC */
'091' = '1' /* Other disorders of nervous system w MCC */
'092' = '1' /* Other disorders of nervous system w CC */
'093' = '1' /* Other disorders of nervous system w/o CC/MCC */
'094' = '1' /* Bacterial & tuberculous infections of nervous system w MCC */
'095' = '1' /* Bacterial & tuberculous infections of nervous system w CC */
'096' = '1' /* Bacterial & tuberculous infections of nervous system w/o CC/MCC */
'097' = '1' /* Non-bacterial infect of nervous sys exc viral meningitis w MCC */
'098' = '1' /* Non-bacterial infect of nervous sys exc viral meningitis w CC */
'099' = '1' /* Non-bacterial infect of nervous sys exc viral meningitis w/o CC/MCC */
'100' = '1' /* Seizures w MCC */
'101' = '1' /* Seizures w/o MCC */
'102' = '1' /* Headaches w MCC */
'103' = '1' /* Headaches w/o MCC */
'121' = '1' /* Acute major eye infections w CC/MCC */
'122' = '1' /* Acute major eye infections w/o CC/MCC */
'123' = '1' /* Neurological eye disorders */
'124' = '1' /* Other disorders of the eye w MCC */
'125' = '1' /* Other disorders of the eye w/o MCC */
'146' = '1' /* Ear, nose, mouth & throat malignancy w MCC */
'147' = '1' /* Ear, nose, mouth & throat malignancy w CC */
'148' = '1' /* Ear, nose, mouth & throat malignancy w/o CC/MCC */
'149' = '1' /* Dysequilibrium */
'150' = '1' /* Epistaxis w MCC */
'151' = '1' /* Epistaxis w/o MCC */
'152' = '1' /* Otitis media & URI w MCC */
'153' = '1' /* Otitis media & URI w/o MCC */
'154' = '1' /* Nasal trauma & deformity w MCC */
'155' = '1' /* Nasal trauma & deformity w CC */
'156' = '1' /* Nasal trauma & deformity w/o CC/MCC */
'157' = '1' /* Dental & Oral Diseases w MCC */
'158' = '1' /* Dental & Oral Diseases w CC */
'159' = '1' /* Dental & Oral Diseases w/o CC/MCC */
'175' = '1' /* Pulmonary embolism w MCC */
'176' = '1' /* Pulmonary embolism w/o MCC */
'177' = '1' /* Respiratory infections & inflammations w MCC */
'178' = '1' /* Respiratory infections & inflammations w CC */
'179' = '1' /* Respiratory infections & inflammations w/o CC/MCC */
'180' = '1' /* Respiratory neoplasms w MCC */
'181' = '1' /* Respiratory neoplasms w CC */
'182' = '1' /* Respiratory neoplasms w/o CC/MCC */
'183' = '1' /* Major chest trauma w MCC */
'184' = '1' /* Major chest trauma w CC */
'185' = '1' /* Major chest trauma w/o CC/MCC */
'186' = '1' /* Pleural effusion w MCC */
'187' = '1' /* Pleural effusion w CC */
'188' = '1' /* Pleural effusion w/o CC/MCC */
'189' = '1' /* Pulmonary edema & respiratory failure */
'190' = '1' /* Chronic obstructive pulmonary disease w MCC */
'191' = '1' /* Chronic obstructive pulmonary disease w CC */
'192' = '1' /* Chronic obstructive pulmonary disease w/o CC/MCC */
'193' = '1' /* Simple pneumonia & pleurisy w MCC */
'194' = '1' /* Simple pneumonia & pleurisy w CC */
'195' = '1' /* Simple pneumonia & pleurisy w/o CC/MCC */
'196' = '1' /* Interstitial lung disease w MCC */
'197' = '1' /* Interstitial lung disease w CC */
'198' = '1' /* Interstitial lung disease w/o CC/MCC */
'199' = '1' /* Pneumothorax w MCC */
'200' = '1' /* Pneumothorax w CC */
'201' = '1' /* Pneumothorax w/o CC/MCC */
'202' = '1' /* Bronchitis & asthma w CC/MCC */
'203' = '1' /* Bronchitis & asthma w/o CC/MCC */
'204' = '1' /* Respiratory signs & symptoms */
'205' = '1' /* Other respiratory system diagnoses w MCC */
'206' = '1' /* Other respiratory system diagnoses w/o MCC */
'207' = '1' /* Respiratory system diagnosis w ventilator support 96+ hours */
'208' = '1' /* Respiratory system diagnosis w ventilator support <96 hours */
'280' = '1' /* Acute myocardial infarction, discharged alive w MCC */
'281' = '1' /* Acute myocardial infarction, discharged alive w CC */
'282' = '1' /* Acute myocardia infarction, discharged alive w/o CC/MCC */
'283' = '1' /* Acute myocardial infarction, expired w MCC */
'284' = '1' /* Acute myocardial infarction, expired w CC */
'285' = '1' /* Acute myocardial infarction, expired w/o CC/MCC */
'286' = '1' /* Circulatory disorders except AMI, w card cath w MCC */
'287' = '1' /* Circulatory disorders except AMI, w card cath w/o MCC */
'288' = '1' /* Acute & subacute endocarditis w MCC */
'289' = '1' /* Acute & subacute endocarditis w CC */
'290' = '1' /* Acute & subacute endocarditis w/o CC/MCC */
'291' = '1' /* Heart failure & shock w MCC */
'292' = '1' /* Heart failure & shock w CC */
'293' = '1' /* Heart failure & shock w/o CC/MCC */
'294' = '1' /* Deep vein thrombophlebitis w CC/MCC */
'295' = '1' /* Deep vein thrombophlebitis w/o CC/MCC */
'296' = '1' /* Cardiac arrest, unexplained w MCC */
'297' = '1' /* Cardiac arrest, unexplained w CC */
'298' = '1' /* Cardiac arrest, unexplained w/o CC/MCC */
'299' = '1' /* Peripheral vascular disorders w MCC */
'300' = '1' /* Peripheral vascular disorders w CC */
'301' = '1' /* Peripheral vascular disorders w/o CC/MCC */
'302' = '1' /* Atherosclerosis w MCC */
'303' = '1' /* Atherosclerosis w/o MCC */
'304' = '1' /* Hypertension w MCC */
'305' = '1' /* Hypertension w/o MCC */
'306' = '1' /* Cardiac congenital & valvular disorders w MCC */
'307' = '1' /* Cardiac congenital & valvular disorders w/o MCC */
'308' = '1' /* Cardiac arrhythmia & conduction disorders w MCC */
'309' = '1' /* Cardiac arrhythmia & conduction disorders w CC */
'310' = '1' /* Cardiac arrhythmia & conduction disorders w/o CC/MCC */
'311' = '1' /* Angina pectoris */
'312' = '1' /* Syncope & collapse */
'313' = '1' /* Chest pain */
'314' = '1' /* Other circulatory system diagnoses w MCC */
'315' = '1' /* Other circulatory system diagnoses w CC */
'316' = '1' /* Other circulatory system diagnoses w/o CC/MCC */
'368' = '1' /* Major esophageal disorders w MCC */
'369' = '1' /* Major esophageal disorders w CC */
'370' = '1' /* Major esophageal disorders w/o CC/MCC */
'371' = '1' /* Major gastrointestinal disorders & peritoneal infections w MCC */
'372' = '1' /* Major gastrointestinal disorders & peritoneal infections w CC */
'373' = '1' /* Major gastrointestinal disorders & peritoneal infections w/o CC/MCC */
'374' = '1' /* Digestive malignancy w MCC */
'375' = '1' /* Digestive malignancy w CC */
'376' = '1' /* Digestive malignancy w/o CC/MCC */
'377' = '1' /* G.I. hemorrhage w MCC */
'378' = '1' /* G.I. hemorrhage w CC */
'379' = '1' /* G.I. hemorrhage w/o CC/MCC */
'380' = '1' /* Complicated peptic ulcer w MCC */
'381' = '1' /* Complicated peptic ulcer w CC */
'382' = '1' /* Complicated peptic ulcer w/o CC/MCC */
'383' = '1' /* Uncomplicated peptic ulcer w MCC */
'384' = '1' /* Uncomplicated peptic ulcer w/o MCC */
'385' = '1' /* Inflammatory bowel disease w MCC */
'386' = '1' /* Inflammatory bowel disease w CC */
'387' = '1' /* Inflammatory bowel disease w/o CC/MCC */
'388' = '1' /* G.I. obstruction w MCC */
'389' = '1' /* G.I. obstruction w CC */
'390' = '1' /* G.I. obstruction w/o CC/MCC */
'391' = '1' /* Esophagitis, gastroent & misc digest disorders w MCC */
'392' = '1' /* Esophagitis, gastroent & misc digest disorders w/o MCC */
'393' = '1' /* Other digestive system diagnoses w MCC */
'394' = '1' /* Other digestive system diagnoses w CC */
'395' = '1' /* Other digestive system diagnoses w/o CC/MCC */
'432' = '1' /* Cirrhosis & alcoholic hepatitis w MCC */
'433' = '1' /* Cirrhosis & alcoholic hepatitis w CC */
'434' = '1' /* Cirrhosis & alcoholic hepatitis w/o CC/MCC */
'435' = '1' /* Malignancy of hepatobiliary system or pancreas w MCC */
'436' = '1' /* Malignancy of hepatobiliary system or pancreas w CC */
'437' = '1' /* Malignancy of hepatobiliary system or pancreas w/o CC/MCC */
'438' = '1' /* Disorders of pancreas except malignancy w MCC */
'439' = '1' /* Disorders of pancreas except malignancy w CC */
'440' = '1' /* Disorders of pancreas except malignancy w/o CC/MCC */
'441' = '1' /* Disorders of liver except malig,cirr,alc hepa w MCC */
'442' = '1' /* Disorders of liver except malig,cirr,alc hepa w CC */
'443' = '1' /* Disorders of liver except malig,cirr,alc hepa w/o CC/MCC */
'444' = '1' /* Disorders of the biliary tract w MCC */
'445' = '1' /* Disorders of the biliary tract w CC */
'446' = '1' /* Disorders of the biliary tract w/o CC/MCC */
'533' = '1' /* Fractures of femur w MCC */
'534' = '1' /* Fractures of femur w/o MCC */
'535' = '1' /* Fractures of hip & pelvis w MCC */
'536' = '1' /* Fractures of hip & pelvis w/o MCC */
'537' = '1' /* Sprains, strains, & dislocations of hip, pelvis & thigh w CC/MCC */
'538' = '1' /* Sprains, strains, & dislocations of hip, pelvis & thigh w/o CC/MCC */
'539' = '1' /* Osteomyelitis w MCC */
'540' = '1' /* Osteomyelitis w CC */
'541' = '1' /* Osteomyelitis w/o CC/MCC */
'542' = '1' /* Pathological fractures & musculoskelet & conn tiss malig w MCC */
'543' = '1' /* Pathological fractures & musculoskelet & conn tiss malig w CC */
'544' = '1' /* Pathological fractures & musculoskelet & conn tiss malig w/o CC/MCC */
'545' = '1' /* Connective tissue disorders w MCC */
'546' = '1' /* Connective tissue disorders w CC */
'547' = '1' /* Connective tissue disorders w/o CC/MCC */
'548' = '1' /* Septic arthritis w MCC */
'549' = '1' /* Septic arthritis w CC */
'550' = '1' /* Septic arthritis w/o CC/MCC */
'551' = '1' /* Medical back problems w MCC */
'552' = '1' /* Medical back problems w/o MCC */
'553' = '1' /* Bone diseases & arthropathies w MCC */
'554' = '1' /* Bone diseases & arthropathies w/o MCC */
'555' = '1' /* Signs & symptoms of musculoskeletal system & conn tissue w MCC */
'556' = '1' /* Signs & symptoms of musculoskeletal system & conn tissue w/o MCC */
'557' = '1' /* Tendonitis, myositis & bursitis w MCC */
'558' = '1' /* Tendonitis, myositis & bursitis w/o MCC */
'559' = '1' /* Aftercare, musculoskeletal system & connective tissue w MCC */
'560' = '1' /* Aftercare, musculoskeletal system & connective tissue w CC */
'561' = '1' /* Aftercare, musculoskeletal system & connective tissue w/o CC/MCC */
'562' = '1' /* Fx, sprn, strn & disl except femur, hip, pelvis & thigh w MCC */
'563' = '1' /* Fx, sprn, strn & disl except femur, hip, pelvis & thigh w/o MCC */
'564' = '1' /* Other musculoskeletal sys & connective tissue diagnoses w MCC */
'565' = '1' /* Other musculoskeletal sys & connective tissue diagnoses w CC */
'566' = '1' /* Other musculoskeletal sys & connective tissue diagnoses w/o CC/MCC */
'592' = '1' /* Skin ulcers w MCC */
'593' = '1' /* Skin ulcers w CC */
'594' = '1' /* Skin ulcers w/o CC/MCC */
'595' = '1' /* Major skin disorders w MCC */
'596' = '1' /* Major skin disorders w/o MCC */
'597' = '1' /* Malignant breast disorders w MCC */
'598' = '1' /* Malignant breast disorders w CC */
'599' = '1' /* Malignant breast disorders w/o CC/MCC */
'600' = '1' /* Non-malignant breast disorders w CC/MCC */
'601' = '1' /* Non-malignant breast disorders w/o CC/MCC */
'602' = '1' /* Cellulitis w MCC */
'603' = '1' /* Cellulitis w/o MCC */
'604' = '1' /* Trauma to the skin, subcut tiss & breast w MCC */
'605' = '1' /* Trauma to the skin, subcut tiss & breast w/o MCC */
'606' = '1' /* Minor skin disorders w MCC */
'607' = '1' /* Minor skin disorders w/o MCC */
'637' = '1' /* Diabetes w MCC */
'638' = '1' /* Diabetes w CC */
'639' = '1' /* Diabetes w/o CC/MCC */
'640' = '1' /* Nutritional & misc metabolic disorders w MCC */
'641' = '1' /* Nutritional & misc metabolic disorders w/o MCC */
'642' = '1' /* Inborn errors of metabolism */
'643' = '1' /* Endocrine disorders w MCC */
'644' = '1' /* Endocrine disorders w CC */
'645' = '1' /* Endocrine disorders w/o CC/MCC */
'682' = '1' /* Renal failure w MCC */
'683' = '1' /* Renal failure w CC */
'684' = '1' /* Renal failure w/o CC/MCC */
'685' = '1' /* Admit for renal dialysis */
'686' = '1' /* Kidney & urinary tract neoplasms w MCC */
'687' = '1' /* Kidney & urinary tract neoplasms w CC */
'688' = '1' /* Kidney & urinary tract neoplasms w/o CC/MCC */
'689' = '1' /* Kidney & urinary tract infections w MCC */
'690' = '1' /* Kidney & urinary tract infections w/o MCC */
'691' = '1' /* Urinary stones w esw lithotripsy w CC/MCC */
'692' = '1' /* Urinary stones w esw lithotripsy w/o CC/MCC */
'693' = '1' /* Urinary stones w/o esw lithotripsy w MCC */
'694' = '1' /* Urinary stones w/o esw lithotripsy w/o MCC */
'695' = '1' /* Kidney & urinary tract signs & symptoms w MCC */
'696' = '1' /* Kidney & urinary tract signs & symptoms w/o MCC */
'697' = '1' /* Urethral stricture */
'698' = '1' /* Other kidney & urinary tract diagnoses w MCC */
'699' = '1' /* Other kidney & urinary tract diagnoses w CC */
'700' = '1' /* Other kidney & urinary tract diagnoses w/o CC/MCC */
'722' = '1' /* Malignancy, male reproductive system w MCC */
'723' = '1' /* Malignancy, male reproductive system w CC */
'724' = '1' /* Malignancy, male reproductive system w/o CC/MCC */
'725' = '1' /* Benign prostatic hypertrophy w MCC */
'726' = '1' /* Benign prostatic hypertrophy w/o MCC */
'727' = '1' /* Inflammation of the male reproductive system w MCC */
'728' = '1' /* Inflammation of the male reproductive system w/o MCC */
'729' = '1' /* Other male reproductive system diagnoses w CC/MCC */
'730' = '1' /* Other male reproductive system diagnoses w/o CC/MCC */
'754' = '1' /* Malignancy, female reproductive system w MCC */
'755' = '1' /* Malignancy, female reproductive system w CC */
'756' = '1' /* Malignancy, female reproductive system w/o CC/MCC */
'757' = '1' /* Infections, female reproductive system w MCC */
'758' = '1' /* Infections, female reproductive system w CC */
'759' = '1' /* Infections, female reproductive system w/o CC/MCC */
'760' = '1' /* Menstrual & other female reproductive system disorders w CC/MCC */
'761' = '1' /* Menstrual & other female reproductive system disorders w/o CC/MCC */
'774' = '1' /* Vaginal delivery w complicating diagnoses */
'775' = '1' /* Vaginal delivery w/o complicating diagnoses */
'776' = '1' /* Postpartum & post abortion diagnoses w/o O.R. procedure */
'777' = '1' /* Ectopic pregnancy */
'778' = '1' /* Threatened abortion */
'779' = '1' /* Abortion w/o D&C */
'780' = '1' /* False labor */
'781' = '1' /* Other antepartum diagnoses w medical complications */
'782' = '1' /* Other antepartum diagnoses w/o medical complications */
'789' = '1' /* Neonates, died or transferred to another acute care facility */
'790' = '1' /* Extreme immaturity or respiratory distress syndrome, neonate */
'791' = '1' /* Prematurity w major problems */
'792' = '1' /* Prematurity w/o major problems */
'793' = '1' /* Full term neonate w major problems */
'794' = '1' /* Neonate w other significant problems */
'795' = '1' /* Normal newborn */
'808' = '1' /* Major hematol/immun diag exc sickle cell crisis & coagul w MCC */
'809' = '1' /* Major hematol/immun diag exc sickle cell crisis & coagul w CC */
'810' = '1' /* Major hematol/immun diag exc sickle cell crisis & coagul w/o CC/MCC */
'811' = '1' /* Red blood cell disorders w MCC */
'812' = '1' /* Red blood cell disorders w/o MCC */
'813' = '1' /* Coagulation disorders */
'814' = '1' /* Reticuloendothelial & immunity disorders w MCC */
'815' = '1' /* Reticuloendothelial & immunity disorders w CC */
'816' = '1' /* Reticuloendothelial & immunity disorders w/o CC/MCC */
'834' = '1' /* Acute leukemia w/o major O.R. procedure w MCC */
'835' = '1' /* Acute leukemia w/o major O.R. procedure w CC */
'836' = '1' /* Acute leukemia w/o major O.R. procedure w/o CC/MCC */
'837' = '1' /* Chemo w acute leukemia as sdx or w high dose chemo agent w MCC */
'838' = '1' /* Chemo w acute leukemia as sdx w CC or high dose chemo agent */
'839' = '1' /* Chemo w acute leukemia as sdx w/o CC/MCC */
'840' = '1' /* Lymphoma & non-acute leukemia w MCC */
'841' = '1' /* Lymphoma & non-acute leukemia w CC */
'842' = '1' /* Lymphoma & non-acute leukemia w/o CC/MCC */
'843' = '1' /* Other myeloprolif dis or poorly diff neopl diag w MCC */
'844' = '1' /* Other myeloprolif dis or poorly diff neopl diag w CC */
'845' = '1' /* Other myeloprolif dis or poorly diff neopl diag w/o CC/MCC */
'846' = '1' /* Chemotherapy w/o acute leukemia as secondary diagnosis w MCC */
'847' = '1' /* Chemotherapy w/o acute leukemia as secondary diagnosis w CC */
'848' = '1' /* Chemotherapy w/o acute leukemia as secondary diagnosis w/o CC/MCC */
'849' = '1' /* Radiotherapy */
'862' = '1' /* Postoperative & post-traumatic infections w MCC */
'863' = '1' /* Postoperative & post-traumatic infections w/o MCC */
'864' = '1' /* Fever of unknown origin */
'865' = '1' /* Viral illness w MCC */
'866' = '1' /* Viral illness w/o MCC */
'867' = '1' /* Other infectious & parasitic diseases diagnoses w MCC */
'868' = '1' /* Other infectious & parasitic diseases diagnoses w CC */
'869' = '1' /* Other infectious & parasitic diseases diagnoses w/o CC/MCC */
'870' = '1' /* Septicemia w MV 96+ hours */
'871' = '1' /* Septicemia w/o MV 96+ hours w MCC */
'872' = '1' /* Septicemia w/o MV 96+ hours w/o MCC */
'880' = '1' /* Acute adjustment reaction & psychosocial dysfunction */
'881' = '1' /* Depressive neuroses */
'882' = '1' /* Neuroses except depressive */
'883' = '1' /* Disorders of personality & impulse control */
'884' = '1' /* Organic disturbances & mental retardation */
'885' = '1' /* Psychoses */
'886' = '1' /* Behavioral & developmental disorders */
'887' = '1' /* Other mental disorder diagnoses */
'894' = '1' /* Alcohol/drug abuse or dependence, left ama */
'895' = '1' /* Alcohol/drug abuse or dependence w rehabilitation therapy */
'896' = '1' /* Alcohol/drug abuse or dependence w/o rehabilitation therapy w MCC */
'897' = '1' /* Alcohol/drug abuse or dependence w/o rehabilitation therapy w/o MCC */
'913' = '1' /* Traumatic injury w MCC */
'914' = '1' /* Traumatic injury w/o MCC */
'915' = '1' /* Allergic reactions w MCC */
'916' = '1' /* Allergic reactions w/o MCC */
'917' = '1' /* Poisoning & toxic effects of drugs w MCC */
'918' = '1' /* Poisoning & toxic effects of drugs w/o MCC */
'919' = '1' /* Complications of treatment w MCC */
'920' = '1' /* Complications of treatment w CC */
'921' = '1' /* Complications of treatment w/o CC/MCC */
'922' = '1' /* Other injury, poisoning & toxic effect diag w MCC */
'923' = '1' /* Other injury, poisoning & toxic effect diag w/o MCC */
'933' = '1' /* Extensive burns or full thickness burns w MV 96+ hrs w/o skin graft */
'934' = '1' /* Full thickness burn w/o skin grft or inhal inj */
'935' = '1' /* Non-extensive burns */
'945' = '1' /* Rehabilitation w CC/MCC */
'946' = '1' /* Rehabilitation w/o CC/MCC */
'947' = '1' /* Signs & symptoms w MCC */
'948' = '1' /* Signs & symptoms w/o MCC */
'949' = '1' /* Aftercare w CC/MCC */
'950' = '1' /* Aftercare w/o CC/MCC */
'951' = '1' /* Other factors influencing health status */
'963' = '1' /* Other multiple significant trauma w MCC */
'964' = '1' /* Other multiple significant trauma w CC */
'965' = '1' /* Other multiple significant trauma w/o CC/MCC */
'974' = '1' /* HIV w major related condition w MCC */
'975' = '1' /* HIV w major related condition w CC */
'976' = '1' /* HIV w major related condition w/o CC/MCC */
'977' = '1' /* HIV w or w/o other related condition */
OTHER = '0' ;

 /* ---------------------------------------------------------------- */
 /* SURGICAL DRGS  (deleted Adult DRGs)                              */
 /* ---------------------------------------------------------------- */

 /* DIAGNOSTIC RELATED GROUPS (DRGS): */

VALUE $SURGIDR
"001" = "1" /* CRANIOTOMY, AGE GREATER THAN 17 EXCEPT FOR TRAUMA */
"002" = "1" /* CRANIOTOMY FOR TRAUMA, AGE GREATER THAN 17 */
"003" = "1" /* CRANIOTOMY, AGE 0-17 */
"004" = "1" /* SPINAL PROCEDURES */
"005" = "1" /* EXTRACRANIAL VASCULAR PROCEDURES */
"006" = "1" /* CARPAL TUNNEL RELEASE */
"007" = "1" /* PERIPHERAL AND CRANIAL NERVE AND OTHER NERVOUS SYSTEM PROCEDURES WITH CC */
"008" = "1" /* PERIPHERAL AND CRANIAL NERVE AND OTHER NERVOUS SYSTEM PROCEDURES WITHOUT CC */
"036" = "1" /* RETINAL PROCEDURES */
"037" = "1" /* ORBITAL PROCEDURES */
"038" = "1" /* PRIMARY IRIS PROCEDURES */
"039" = "1" /* LENS PROCEDURES WITH OR WITHOUT VITRECTOMY */
"040" = "1" /* EXTRAOCULAR PROCEDURES EXCEPT ORBIT, AGE GREATER THAN 17 */
"041" = "1" /* EXTRAOCULAR PROCEDURES EXCEPT ORBIT, AGE 0-17 */
"042" = "1" /* INTRAOCULAR PROCEDURES EXCEPT RETINA, IRIS AND LENS */
"049" = "1" /* MAJOR HEAD AND NECK PROCEDURES */
"050" = "1" /* SIALOADENECTOMY */
"051" = "1" /* SALIVARY GLAND PROCEDURES EXCEPT SIALOADENECTOMY */
"052" = "1" /* CLEFT LIP AND PALATE REPAIR */
"053" = "1" /* SINUS AND MASTOID PROCEDURES, AGE GREATER THAN 17 */
"054" = "1" /* SINUS AND MASTOID PROCEDURES, AGE 0-17 */
"055" = "1" /* MISCELLANEOUS EAR, NOSE, MOUTH AND THROAT PROCEDURES */
"056" = "1" /* RHINOPLASTY */
"058" = "1" /* TONSILLECTOMY AND ADNOIDECTOMY PROCEDURES EXCEPT TONSILLECTOMY AND/OR ADENOIDECTOMY ONLY, AGE 0-17 */
"059" = "1" /* TONSILLECTOMY AND/OR ADENOIDECTOMY ONLY, AGE GREATER THAN 17 */
"060" = "1" /* TONSILLECTOMY AND/OR ADENOIDECTOMY ONLY, AGE 0 - 17 */
"061" = "1" /* MYRINGOTOMY WITH TUBE INSERTION, AGE GREATER THAN 17 */
"062" = "1" /* MYRINGOTOMY WITH TUBE INSERTION, AGE 0-17 */
"063" = "1" /* OTHER EAR, NOSE, MOUTH AND THROAT OR PROCEDURES */
"075" = "1" /* MAJOR CHEST PROCEDURES */
"076" = "1" /* OTHER RESPIRATORY SYSTEM OR PROCEDURES WITH CC */
"077" = "1" /* OTHER RESPIRATORY SYSTEM OR PROCEDURES WITHOUT CC */
"103" = "1" /* HEART TRANSPLANT */
"104" = "1" /* CARDIAC VALVE AND OTHER MAJOR CARDIOTHORACIC PROCEDURES WITH CARDIAC CATHETERIZATION */
"105" = "1" /* CARDIAC VALVE AND OTHER MAJOR CARDIOTHORACIC PROCEDURES WITHOUT CARDIAC CATHETERIZATION */
"106" = "1" /* CORONARY BYPASS WITH PTCA */
"107" = "1" /* CORONARY BYPASS WITH CARDIAC CATHETERIZATION */
"108" = "1" /* OTHER CARDIOTHORACIC PROCEDURES */
"109" = "1" /* CORONARY BYPASS WITHOUT CARDIAC CATHETERIZATION */
"110" = "1" /* MAJOR CARDIOVASCULAR PROCEDURES WITH CC */
"111" = "1" /* MAJOR CARDIOVASCULAR PROCEDURES WITHOUT CC */
"112" = "1" /* PERCUTANEOUS CARDIOVASCULAR PROCEDURES */
"113" = "1" /* AMPUTATION FOR CIRCULATORY SYSTEM DISORDERS EXCEPT UPPER LIMB AND TOE */
"114" = "1" /* UPPER LIMB AND TOES AMPUTATION FOR CIRCULATORY SITE */
"115" = "1" /* PERMANENT CARDIAC PACEMAKER IMPLANT WITH ACUTE MYOCARDIAL INFARCTION, HEART FAILURE OR SHOCK OR ACID LEAD OR GENERATOR PROCEDURE */
"116" = "1" /* OTHER PERMANENT CARDIAC PACEMAKER IMPLANT OR PTCA WITH CORONARY ARTERIAL STENT */
"117" = "1" /* CARDIAC PACEMAKER REVISION EXCEPT DEVICE REPLACEMENT */
"118" = "1" /* CARDIAC PACEMAKER DEVICE REPLACEMENT */
"119" = "1" /* VEIN LIGATION AND STRIPPING */
"120" = "1" /* OTHER CIRCULATORY SYSTEM OR PROCEDURES */
"146" = "1" /* RECTAL RESECTION WITH CC */
"147" = "1" /* RECTAL RESECTION WITHOUT CC */
"148" = "1" /* MAJOR SMALL AND LARGE BOWEL PROCEDURES WITH CC */
"149" = "1" /* MAJOR SMALL AND LARGE BOWEL PROCEDURES WITHOUT CC */
"150" = "1" /* PERITONEAL ADHESIOLYSIS WITH CC */
"151" = "1" /* PERITONEAL ADHESIOLYSIS WITHOUT CC */
"152" = "1" /* MINOR SMALL AND LARGE BOWEL PROCEDURES WITH CC */
"153" = "1" /* MINOR SMALL AND LARGE BOWEL PROCEDURES WITHOUT CC */
"154" = "1" /* STOMACH, ESOPHAGEAL AND DUODENAL PROCEDURES, AGE GREATER THAN 17 WITH CC */
"155" = "1" /* STOMACH, ESOPHAGEAL AND DUODENAL PROCEDURES, AGE GREATER THAN 17 WIHOUT CC*/
"156" = "1" /* STOMACH, ESOPHAGEAL AND DUODENAL PROCEDURES, AGE 0-17 */
"157" = "1" /* ANAL AND STOMAL PROCEDURES WITH CC */
"158" = "1" /* ANAL AND STOMAL PROCEDURES WITHOUT CC */
"159" = "1" /* HERMIA PROCEDURES EXCEPT INGUINAL AND FEMORAL , AGE GREATER THAN 17 WITH CC */
"160" = "1" /* HERNIA PROCEDURES EXCEPT INGUINAL AND FEMORAL, AGE GREATER THAN 17 WITHOUT CC */
"161" = "1" /* INGUINAL AND FEMORAL HERNIA PROCEDURES, AGE GREATER THAN 17 WITH CC */
"162" = "1" /* INGUINAL AND FEMORAL HERNIA PROCEDURES, AGE GREATER THAN 17 WITHOUT CC */
"163" = "1" /* HERNIA PROCEDURES, AGE 0-17 */
"164" = "1" /* APPENDECTOMY WITH COMPLICATED PRINCIPAL DIAGNOSIS WITH CC */
"165" = "1" /* APPENDECTOMY WITH COMPLICATED PRINCIPAL DIAGNOSIS WIHTOUT CC */
"166" = "1" /* APPENDECTOMY WITHOUT COMPLICATED PRINCIPAL IAGNOSIS WITH CC */
"167" = "1" /* APPENDECTOMY WITHOUT COMPLICATED PRINCIPAL DIAGNOSIS WITHOUT CC */
"168" = "1" /* MOUTH PROCEDURES WITH CC */
"169" = "1" /* MOUTH PROCEDURES WITHOUT CC */
"170" = "1" /* OTHER DIGESTIVE SYSTEM OR PROCEDURES WITH CC */
"171" = "1" /* OTHER DIGESTIVE SYSTEM OR PROCEDURES WITHOUT CC */
"191" = "1" /* PANCREAS, LIVER AND SHUNT PROCEDURES WITH CC */
"192" = "1" /* PANCREAS, LIVER AND SHUNT PROCEDURES WITHOUT CC */
"193" = "1" /* BILIARY TRACT PROCEDURES EXCEPT ONLY CHOLECYSTECTOIMY WITH OR WITHOUT COMMON DUCT EXPLORATION WITH CC */
"194" = "1" /* BILIARY TRACT PROCEDURES EXCEPT ONLY CHOLECYSTECTOMY WITH OR WITHOUT COMMON DUCT EXPLORATION WITHOUT CC */
"195" = "1" /* CHOLECYSTECTOMY WITH COMMON DUCT EXPLORATION WITH CC */
"196" = "1" /* CHOLECYSTECTOMY WITH COMMON DUCT EXPLORATION WITHOUT CC */
"197" = "1" /* CHOLECYSTECTOMY EXCEPT BY LAPAROSCOPE WITHOUT COMMON DUCT EXPLORATION WITH CC */
"198" = "1" /* CHOLECYSTECTOMY EXCEPT BY LAPAROSCOPE WITHOUT COMMON DUCT EXPORTATION WITHOUT CC */
"199" = "1" /* HEPATOBILIARY DIAGNOSTIC PROCEDURE FOR MALIGNANCY */
"200" = "1" /* HEPATOBILIARY DIAGNOSTIC PROCEDURE FOR NONMALIGNANCY */
"201" = "1" /* OTHER HEPATOBILIARY OR PANCREAS OR PROCEDURES */
"209" = "1" /* MAJOR JOINT AND LIMB REATTACHMENT PROCEDURES OF LOWER EXTREMITY */
"210" = "1" /* HIP AND FEMUR PROCEDURES EXCEPT MAJOR JOINT PROCEDURES, AGE GREATER THAN 17 WITH CC */
"211" = "1" /* HIP AND FEMUR PROCEDURES EXCEPT MAJOR JOINT PROCEDURES, AGE GREATER THAN 17 WITHOUT CC */
"212" = "1" /* HIP AND FEMUR PROCEDURES EXCEPT MAJOR JOINT PROCEDURE, AGE 0-17 */
"213" = "1" /* AMPUTATION FOR MUSCULOSKELETAL SYSTEM AND CONNECTIVE TISSUE DISORDERS */
"214" = "1" /* BACK & NECK PROCEDURES WITH CC */
"215" = "1" /* BACK & NECK PROCEDURES WITHOUT CC */
"216" = "1" /* BIOPSIES OF MUSCULOSKELETAL SYSTEM AND CONNECTIVE TISSUE */
"217" = "1" /* WOUND DEBRIDEMENT AND SKIN GRAFT EXCEPT HAND FOR MUSCULOSKELETAL AND CONNECTIVE TISSUE DISORDERS */
"218" = "1" /* LOWER EXTREMITY AND HUMERUS PROCEURES EXCEPT HIP, FOOT AND FEMUR, AGE GREATER THAN 17 WITH CC */
"219" = "1" /* LOWER EXTREMITY AND HUMERUS PROCEDURES EXCEPT HIP, FOOT AND FEMUR, AGE GREATER THAN 17 WITHOUT CC */
"220" = "1" /* LOWER EXTREMITY AND HUMERUS PROCEDURES EXCEPT HIP, FOOT AND FEMUR, AGE 0-17 */
"221" = "1" /* KNEE PROCEDURES WITH CC */
"222" = "1" /* KNEE PROCEDURES WITHOUT CC */
"223" = "1" /* MAJOR SHOULDER/ELBOW PROCEDURES OR OTHER UPPER EXTREMITY PROCEDURES WITH CC */
"224" = "1" /* SHOULDER, ELBOW OR FOREARM PROCEDURES EXCEPT MAJOR JOINT PROCEDURES WITHOUT CC */
"225" = "1" /* FOOT PROCEDURES*/
"226" = "1" /* SOFT TISSUE PROCEDURES WITH CC */
"227" = "1" /* SOFT TISSUE PROCEDURES WITHOUT CC */
"228" = "1" /* MAJOR THUMB OR JOINT PROCEDURES OR OTHER HAND OR WRIST PROCEDURES WITH CC */
"229" = "1" /* HAND OR WRIST PROCEDURES EXCEPT MAJOR JOINT PROCEDURES WITHOUT CC */
"230" = "1" /* LOCAL EXCISION AND REMOVAL OF INTERNAL FIXATION DEVICES OF HIP AND FEMUR */
"231" = "1" /* LOCAL EXCISION AND REMOVAL OF INTERNAL FIXATION DEVICES EXCEPT HIP AND FEMUR */
"232" = "1" /* ARTHROSCOPY */
"233" = "1" /* OTHER MUSCULOSKELETAL SYSTEM AND CONNECTIVE TISSUE OR PROCEDURES WITH CC */
"234" = "1" /* OTHER MUSCULOSKELETAL SYSTEM AND CONNECTIVE TISSUE OR PROCEDURES WITHOUT CC */
"257" = "1" /* TOTAL MASTECTOMY FOR MALIGNANCY WITH CC */
"258" = "1" /* TOTAL MASTECTOMY FOR MALIGNANCY WITHOUT CC */
"259" = "1" /* SUBTOTAL MASTECTOMY FOR MALIGNANCY WITH CC */
"260" = "1" /* SUBTOTAL MASTECTOMY FOR MALIGNANCY WITHOUT CC */
"261" = "1" /* BREAST PROCEDURE FOR NONMALIGNANCY EXCEPT BIOPSY AND LOCAL EXCISION */
"262" = "1" /* BREAST BIOPSY AND LOCAL EXCISION FOR NONMALIGNANCY */
"263" = "1" /* SKIN GRAFT AND/OR DEBRIDEMENT FOR SKIN ULCER OR CELLULITIS WITH CC */
"264" = "1" /* SKIN GRAFT AND OR DEBRIDEMENT FOR SKIN ULCER OR CELLULITIS WITHOUT CC */
"265" = "1" /* SKIN GRAFT AND OR DEBRIDEMENT EXCEPT FOR SKIN ULCER OR CELLULITIS WITH CC */
"266" = "1" /* SKIN GRAFT AND/OR DEBRIDEMENT EXCEPT FOR SKIN ULCER OR CELLUTLITIES WITHOUT CC */
"267" = "1" /* PERIANAL AND PILONIDAL PROCEDURES */
"268" = "1" /* SKIN, SUBCUTANEOUS TISSUE AND BREAST PLASTIC PROCEDURES */
"269" = "1" /* OTHER SKIN, SUBCUTANEOUS TISSUE AND BREAST PROCEDURES WITH CC */
"270" = "1" /* OTHER SKIN, SUBCUTANEOUS TISSUE AND BREAST PROCEDURS WITHOUT CC */
"285" = "1" /* AMPUTAETION OF LOWER LIMB FOR ENDOCRINE, NUTRITIONAL AND METABOLIC DISORDERS */
"286" = "1" /* ADRENAL AND PITUITARY PROCEDURES */
"287" = "1" /* SKIN GRAFTS AND WOUND DEBRIDEMENTS FOR ENDOCRINE, NUTRITIONAL AND METABOLIC DISORDERS */
"288" = "1" /* OR PROCEDURES FOR OBESITY */
"289" = "1" /* PARATHYROID PROCEDURES */
"290" = "1" /* THYROID PROCEDURES */
"291" = "1" /* THYROGLOSSAL PROCEDURES */
"292" = "1" /* OTHER ENDOCRINE, NUTRITIONAL AND METABOLIC OR PROCEDURES WITH CC */
"293" = "1" /* OTHER ENDOCRINE, NUTRITIONAL AND METABOLIC OR PROCEDURES WITHOUT CC */
"302" = "1" /* KIDNEY TRANSPLANT */
"303" = "1" /* KIDNEY, URETER AND MAJOR BLADDER PROCEDURES FOR NEOPLASM */
"304" = "1" /* KIDNEY, URETER AND MAJOR BLADDER PROCEDURES FOR NONNEOPLASMS WITH CC */
"305" = "1" /* KIDNEY, URETER AND MAJOR BLADDER PROCEDURES FOR NONEOPLSMS WITHOUT CC */
"306" = "1" /* PROSTATECTOMY WITH CC */
"307" = "1" /* PROSTATECTOMY WITHOUT CC */
"308" = "1" /* MINOR BLADDER PROCEDURES WITH CC */
"309" = "1" /* MINOR BLADDER PROCEDURES WITHOUT CC */
"310" = "1" /* TRANSURETHRAL PROCEDURES WITH CC */
"311" = "1" /* TRANSURETHRAL PROCEDURES WITHOUT CC */
"312" = "1" /* URETHRAL PROCEDURES, AGE GREATER THAN 17 WITH CC */
"313" = "1" /* URETHRAL PROCEDURES, AGE GREATER THAN 17 WITHOUT CC */
"314" = "1" /* URETHRAL PROCEDURES, AGE 0-17 */
"315" = "1" /* OTHER KIDNEY AND URINARY TRACT OR PROCEDURES */
"334" = "1" /* MAJOR MALE PELVIC PROCEDURES WITH CC */
"335" = "1" /* MAJOR MALE PELVIC PROCEDURES WITHOUT CC */
"336" = "1" /* TRANSURETHRAL PROSTATECTOMY WITH CC */
"337" = "1" /* TRANSURETHRAL PROSTATECTOMY WITHOUT CC */
"338" = "1" /* TESTES PROCEDURES FOR MALIGNANCY */
"339" = "1" /* TESTES PROCEDURES FOR NONMALIGNANCY, AGE GREATER THAN 17 */
"340" = "1" /* TESTES PROCEDURES FOR NONMALIGNANCY, AGE 0-17 */
"341" = "1" /* PENIS PROCEDURES */
"342" = "1" /* CIRCUMCISION, AGE GREATER THAN 17 */
"343" = "1" /* CIRCUMCISION, AGE 0-17 */
"344" = "1" /* OTHER MALE REPRODUCTIVE SYSTEM OR PROCEDURES FOR MALIGNANCY */
"345" = "1" /* OTHER MALE REPRODUCTIVE SYSTEM OR PROCEDURES EXCEPT FOR MALIGNANCY */
"353" = "1" /* PELVIC EVISCERATION, RADICAL HYSTERECTOMY AND RADICAL VULVECTOMY */
"354" = "1" /* UTERINE AND ADNEXA PROCEDURES FOR NONOVARIAN/ADNEXAL MALIGNANCY WITH CC */
"355" = "1" /* UTERINE AND ADNEXA PROCEDURES FOR NONOVARIAN/ADNEXA PROCEDURES WITHOUT CC*/
"356" = "1" /* FEMALE REPRODUCTIVE SYSTEM RECONSTRUCTIVE PROCEDURES */
"357" = "1" /* UTERINE AND ADNEXA PROCEDURES FOR OVARIAN OR ADNEXAL MALIGNANCY */
"358" = "1" /* UTERINE AND ADNEXA PROCEDURES FOR NONMALIGNANCY WITH CC */
"359" = "1" /* UTERINE AND ADNEXA PROCEDURES FOR NONMALIGNANCY WITHOUT CC */
"360" = "1" /* VAGINA, CERVIX AND VULVA PROCEDURES */
"361" = "1" /* LAPAROSCOPY AND INCISIONAL TUBAL INTERRUPTION */
"362" = "1" /* ENDOSCOPIC TUBAL INTERRUPTION */
"363" = "1" /* D AND C, CONIZATION AND RADIOIMPLANT FOR MALIGNANCY */
"364" = "1" /* D AND C, CONIZATION EXCEPT FOR MALIGNANCY */
"365" = "1" /* OTHER FEMALE REPRODUCTIVE SYSTEM OR PROCEDURES */
"370" = "1" /* CESAREAN SECTION WITH CC */
"371" = "1" /* CESAREAN SECTION WITHOUT CC */
"374" = "1" /* VAGINAL DELIVERY WITH STERILIZATION AND/OR D AND C */
"375" = "1" /* VAGINAL DELIVERY WITH OR PROCEDURE EXCEPT STERILIZATION AND/OR D AND C */
"377" = "1" /* POSTPARTUM AND POSTABORTION DIAGNOSES WITH OR PROCEDURE */
"381" = "1" /* ABORTION WITH D AND C ASPIRATION CURETTAGE OR HYSTERECTOMY */
"392" = "1" /* SPLENECTOMY, AGE GREATER THAN 17 */
"393" = "1" /* SPLENECTOMY, AGE 0-17 */
"394" = "1" /* OTHER OR PROCEDURES OF THE BLOOD AND BLOOD-FORMING ORGANS */
"400" = "1" /* LYMPHOMA AND LEUKEMIA WITH MAJOR OR PROCEDURES */
"401" = "1" /* LYMPHOMA AND NONACUTE LEUKEMIA WITH OTHER OR PROCEDURE WITH CC */
"402" = "1" /* LYMPHOMA AND NONACUTE LEUKEMIA WITH OTHER OR PROCEDURE WITHOUT CC */
"406" = "1" /* MYELOPROLIFERATIVE DISORDERS OR POORLY DIFFERENTIATED NEOPLASMS WITH MAJOR OR PROCEDURES WITH CC */
"407" = "1" /* MYELOPROLIFERATIVE DISORDERS OR POORLY DIFFERENTIATED NEOPLASMS WITH MAJOR OR PROCEDURES WITHOUT CC */
"408" = "1" /* MYELOPROLIFERATIVE DISORDERS OR POORLY DIFFERENTIATED NEOPLASMS WITH OTHER OR PROCEDURES */
"415" = "1" /* OR PROCEDURE FOR INFECTIOUS AND PARASITIC DISEASES */
"424" = "1" /* OR PROCEDURES WITH PRINCIPAL DIAGNOSIS OF MENTAL ILLNESS */
"439" = "1" /* SKIN GRAFTS FOR INJURIES */
"440" = "1" /* WOUND DEBRIDEMENTS FOR INJURIES */
"441" = "1" /* HAND PROCEDURES FOR INJURIES*/
"442" = "1" /* OTHER OR PROCEDURES FOR INJURIES WITH CC */
"443" = "1" /* OTHER OR PROCEDURES FOR INJURIES WITHOUT CC */
"458" = "1" /* NON-EXTENSIVE BURNS W SKIN GRAFT */
"459" = "1" /* NON-EXTENSIVE BURNS W WOUND DEBRIDEMENT OR OTHER O.R. PROC */
"461" = "1" /* OR PROCEDURES WITH DIAGNOSES OF OTHER CONTACT WITH HEALTH SERVICES */
"468" = "1" /* EXTENSIVE OR PROCEDURE UNRELATED TO PRINCIPAL DIAGNOSIS */
"471" = "1" /* BILATERAL OR MULTIPLE MAJOR JOINT PROCEDURES OF LOWER EXTREMITY */
"472" = "1" /* EXTENSIVE BURNS W O.R. PROCEDURE */
"476" = "1" /* PROSTATIC OR PROCEDURE UNRELATED TO PRINCIPAL DIAGNOSIS */
"477" = "1" /* NONEXTENSIVE OR PROCEDURE UNRELATED TO PRINCIPAL DIAGNOSIS */
"478" = "1" /* OTHER VASCULAR PROCEDURES WITH CC */
"479" = "1" /* OTHER VASCULAR PROCEDURES WITHOUT CC */
"480" = "1" /* LIVER TRANSPLANT */
"481" = "1" /* BONE MARROW TRANSPLANT */
"482" = "1" /* TRACHEOSTOMY FOR FACE, MOUTH AND NECK DIAGNOSES */
"483" = "1" /* TRACHEOSTOMY EXCEPT FOR FACE, MOUTH AND NECK DIAGNOSES */
"484" = "1" /* CRANIOTOMY FOR MULTIPLE SIGNIFICANT TRAUMA */
"485" = "1" /* LIMB REATTACHMENT, HIP AND FEMUR PROCEDURES FOR MULTIPLE SIGNIFICANT TRAUMA */
"486" = "1" /* OTHER OR PROCEDURES FOR MULTIPLE SIGNIFICANT TRAUMA */
"488" = "1" /* HIV WITH EXTENSIVE OR PROCEDURE */
"491" = "1" /* MAJOR JOINT AND LIMB REATTACHMENT PROCEDURES OF UPPER EXTREMITY */
"493" = "1" /* LAPAROSCOPIC CHOLECYSTECTOMY WITHOUT COMMON DUCT EXPLORATION WITH CC */
"494" = "1" /* LAPAROSCOPIC CHOLECYSTECTOMY WITHOUT COMMON DUCT EXPLORATION WITHOUT CC */
"495" = "1" /* LUNG TRANSPLANT */
"496" = "1" /* COMBINED ANTERIOR/POSTERIOR SPINAL FUSION */
"497" = "1" /* SPINAL FUSION WITH CC */
"498" = "1" /* SPINAL FUSION WITHOUT CC */
"499" = "1" /* BACK AND NECK PROCEDURES EXCEPT SPINAL FUSION WITH CC */
"500" = "1" /* BACK AND NECK PROCEDURES EXCEPT SPINAL FUSION WITHOUT CC */
"501" = "1" /* KNEE PROCEDURES WITH PRINCIPAL DIAGNOSIS OF INFECTION, WITH CC */
"502" = "1" /* KNEE PROCEDURES WITH PRINCIPAL DIAGNOSIS OF INFECTION, WITHOUT CC */
"503" = "1" /* KNEE PROCEDURES WITHOUT PRINCIPAL DIAGNOSIS OF INFECTION */
"504" = "1" /* EXTENSIVE 3RD DEGREE BURNS W SKIN GRAFT */
"506" = "1" /* FULL THICKNESS BURN W SKIN GRAFT OR INHAL INJ W CC OR SIG TRAUMA */
"507" = "1" /* FULL THICKNESS BURN W SKIN GRFT OR INHAL INJ W/O CC OR SIG TRAUMA */
"512" = "1" /* SIMULTANEOUS PANCREAS/KIDNEY TRANSPLANT */
"513" = "1" /* PANCREAS TRANSPLANT */
"514" = "1" /* CARDIAC DEFIBRILLATOR IMPLANT W CARDIAC CATH */
"515" = "1" /* CARDIAC DEFIBRILLATOR IMPLANT WITHOUT CARDIAC CATH */
"516" = "1" /* PERCUTANEOUS CARDIOVASC PROC W AMI */
"517" = "1" /* PERC CARDIO PROC W NON-DRUG ELUTING STENT WITHOUT AMI */
"518" = "1" /* PERC CARDIO PROC WITHOUT CORONARY ARTERY STENT OR AMI */
"519" = "1" /* CERVICAL SPINAL FUSION W CC */
"520" = "1" /* CERVICAL SPINAL FUSION WITHOUT CC */
"525" = "1" /* HEART ASSIST SYSTEM IMPLANT */
"526" = "1" /* PERCUTNEOUS CARDIOVASULAR PROC W DRUG ELUTING STENT W AMI */
"527" = "1" /* PERCUTNEOUS CARDIOVASULAR PROC W DRUG ELUTING STENT WITHOUT AMI */
"528" = "1" /* INTRACRANIAL VASCULAR PROC W PDX HEMORRHAGE */
"529" = "1" /* VENTRICULAR SHUNT PROCEDURES W CC */
"530" = "1" /* VENTRICULAR SHUNT PROCEDURES W/O CC */
"531" = "1" /* SPINAL PROCEDURES W CC */
"532" = "1" /* SPINAL PROCEDURES W/O CC */
"533" = "1" /* EXTRACRANIAL PROCEDURES W CC */
"534" = "1" /* EXTRACRANIAL PROCEDURES W/O CC */
"535" = "1" /* CARDIAC DEFIB IMPLANT W CARDIAC CATH W AMI/HF/SHOCK */
"536" = "1" /* CARDIAC DEFIB IMPLANT W CARDIAC CATH W/O AMI/HF/SHOCK */
"537" = "1" /* LOCAL EXCIS & REMOV OF INT FIX DEV EXCEPT HIP & FEMUR W CC */
"538" = "1" /* LOCAL EXCIS & REMOV OF INT FIX DEV EXCEPT HIP & FEMUR W/O CC */
"539" = "1" /* LYMPHOMA & LEUKEMIA W MAJOR OR PROCEDURE W CC */
"540" = "1" /* LYMPHOMA & LEUKEMIA W MAJOR OR PROCEDURE W/O CC */
"541" = "1" /* TRACH W MV 96+HRS OR PDX EXC FACE, MTH, FACE & NECK DX W/MAJ OR */
"542" = "1" /* TRACH W MV 96+HRS OR PDX EXC FACE, MTH, FACE & NECK DX W/O MJ OR */
"543" = "1" /* CRANIOTOMY WITH IMPLANTATION OF CHEMOTHERAPEUTIC AGENT OR ACUTE 
               COMPLEX CENTRAL NERVOUS SYSTEM PRINCIPAL DIAGNOSIS */
"544" = "1" /* MAJOR JOINT REPLACEMENT OR REATTACHMENT OF LOWER EXTREMITY */
"545" = "1" /* REVISION OF HIP OR KNEE REPLACEMENT */
"546" = "1" /* SPINAL FUSION EXC CERV WITH CURVATURE OF THE SPINE OR MALIG */
"547" = "1" /* CORONARY BYPASS W CARDIAC CATH W MAJOR CV DX */
"548" = "1" /* CORONARY BYPASS W CARDIAC CATH W/O MAJOR CV DX */
"549" = "1" /* CORONARY BYPASS W/O CARDIAC CATH W MAJOR CV DX */
"550" = "1" /* CORONARY BYPASS W/O CARDIAC CATH W/O MAJOR CV DX */
"551" = "1" /* PERMANENT CARDIAC PACEMAKER IMPL W MAJ CV DX OR AICD LEAD OR GNRTR */
"552" = "1" /* OTHER PERMANENT CARDIAC PACEMAKER IMPLANT W/O MAJOR CV DX */
"553" = "1" /* OTHER VASCULAR PROCEDURES W CC W MAJOR CV DX */
"554" = "1" /* OTHER VASCULAR PROCEDURES W CC W/O MAJOR CV DX */
"555" = "1" /* PERCUTANEOUS CARDIOVASCULAR PROC W MAJOR CV DX */
"556" = "1" /* PERCUTANEOUS CARDIOVASC PROC W NON-DRUG-ELUTING STENT W/O MAJ CV DX */
"557" = "1" /* PERCUTANEOUS CARDIOVASCULAR PROC W DRUG-ELUTING STENT W MAJOR CV DX */
"558" = "1" /* PERCUTANEOUS CARDIOVASCULAR PROC W DRUG-ELUTING STENT W/O MAJ CV DX */
"567" = "1" /* STOMACH, ESOPHAGEAL & DUODENAL PROC AGE > 17 W CC W MAJOR GI DX */
"568" = "1" /* STOMACH, ESOPHAGEAL & DUODENAL PROCEDURES PROC AGE > 17 W CC W/O MAJOR GI DX */
"569" = "1" /* MAJOR SMALL & LARGE BOWEL PROCEDURES W CC W MAJOR GI DX */
"570" = "1" /* MAJOR SMALL & LARGE BOWEL PROCEDURES W CC W/O MAJOR GI DX */
"573" = "1" /* MAJOR BLADDER PROCEDURES */
"577" = "1" /* CAROTID ARTERY STENT PROCEDURE */
"578" = "1" /* INFECTIOUS & PARASITIC DISEASES W OR PROCEDURE */
"579" = "1" /* POSTOPERATIVE OR POST-TRAUMATIC INFECTIONS W OR PROCEDURE */
OTHER = "0" ;

VALUE $SURGI2R
'001' = '1' /* Heart transplant or implant of heart assist system w MCC */
'002' = '1' /* Heart transplant or implant of heart assist system w/o MCC */
'003' = '1' /* ECMO or trach w MV 96+ hrs or PDX exc face, mouth & neck w maj O.R. */
'004' = '1' /* Trach w MV 96+ hrs or PDX exc face, mouth & neck w/o maj O.R. */
'005' = '1' /* Liver transplant w MCC or intestinal transplant */
'006' = '1' /* Liver transplant w/o MCC */
'007' = '1' /* Lung transplant */
'008' = '1' /* Simultaneous pancreas/kidney transplant */
'009' = '1' /* Bone marrow transplant */
'010' = '1' /* Pancreas transplant */
'011' = '1' /* Tracheostomy for face,mouth & neck diagnoses w MCC */
'012' = '1' /* Tracheostomy for face,mouth & neck diagnoses w CC */
'013' = '1' /* Tracheostomy for face,mouth & neck diagnoses w/o CC/MCC */
'014' = '1' /* ALLOGENEIC BONE MARROW TRANSPLANT */
'015' = '1' /* AUTOLOGOUS BONE MARROW TRANSPLANT */
'016' = '1' /* AUTOLOGOUS BONE MARROW TRANSPLANT W CC/MCC */
'017' = '1' /* AUTOLOGOUS BONE MARROW TRANSPLANT W/O CC/MCC */
'020' = '1' /* Intracranial vascular procedures w PDX hemorrhage w MCC */
'021' = '1' /* Intracranial vascular procedures w PDX hemorrhage w CC */
'022' = '1' /* Intracranial vascular procedures w PDX hemorrhage w/o CC/MCC */
'023' = '1' /* Cranio w major dev impl/acute complex CNS PDX w MCC or chemo implant */
'024' = '1' /* Cranio w major dev impl/acute complex CNS PDX w/o MCC */
'025' = '1' /* Craniotomy & endovascular intracranial procedures w MCC */
'026' = '1' /* Craniotomy & endovascular intracranial procedures w CC */
'027' = '1' /* Craniotomy & endovascular intracranial procedures w/o CC/MCC */
'028' = '1' /* Spinal procedures w MCC */
'029' = '1' /* Spinal procedures w CC or spinal neurostimulators */
'030' = '1' /* Spinal procedures w/o CC/MCC */
'031' = '1' /* Ventricular shunt procedures w MCC */
'032' = '1' /* Ventricular shunt procedures w CC */
'033' = '1' /* Ventricular shunt procedures w/o CC/MCC */
'034' = '1' /* Carotid artery stent procedure w MCC */
'035' = '1' /* Carotid artery stent procedure w CC */
'036' = '1' /* Carotid artery stent procedure w/o CC/MCC */
'037' = '1' /* Extracranial procedures w MCC */
'038' = '1' /* Extracranial procedures w CC */
'039' = '1' /* Extracranial procedures w/o CC/MCC */
'040' = '1' /* Periph/cranial nerve & other nerv syst proc w MCC */
'041' = '1' /* Periph/cranial nerve & other nerv syst proc w CC or periph neurostim */
'042' = '1' /* Periph/cranial nerve & other nerv syst proc w/o CC/MCC */
'113' = '1' /* Orbital procedures w CC/MCC */
'114' = '1' /* Orbital procedures w/o CC/MCC */
'115' = '1' /* Extraocular procedures except orbit */
'116' = '1' /* Intraocular procedures w CC/MCC */
'117' = '1' /* Intraocular procedures w/o CC/MCC */
'129' = '1' /* Major head & neck procedures w CC/MCC or major device */
'130' = '1' /* Major head & neck procedures w/o CC/MCC */
'131' = '1' /* Cranial/facial procedures w CC/MCC */
'132' = '1' /* Cranial/facial procedures w/o CC/MCC */
'133' = '1' /* Other ear, nose, mouth & throat O.R. procedures w CC/MCC */
'134' = '1' /* Other ear, nose, mouth & throat O.R. procedures w/o CC/MCC */
'135' = '1' /* Sinus & mastoid procedures w CC/MCC */
'136' = '1' /* Sinus & mastoid procedures w/o CC/MCC */
'137' = '1' /* Mouth procedures w CC/MCC */
'138' = '1' /* Mouth procedures w/o CC/MCC */
'139' = '1' /* Salivary gland procedures */
'163' = '1' /* Major chest procedures w MCC */
'164' = '1' /* Major chest procedures w CC */
'165' = '1' /* Major chest procedures w/o CC/MCC */
'166' = '1' /* Other resp system O.R. procedures w MCC */
'167' = '1' /* Other resp system O.R. procedures w CC */
'168' = '1' /* Other resp system O.R. procedures w/o CC/MCC */
'215' = '1' /* Other heart assist system implant */
'216' = '1' /* Cardiac valve & oth maj cardiothoracic proc w card cath w MCC */
'217' = '1' /* Cardiac valve & oth maj cardiothoracic proc w card cath w CC */
'218' = '1' /* Cardiac valve & oth maj cardiothoracic proc w card cath w/o CC/MCC */
'219' = '1' /* Cardiac valve & oth maj cardiothoracic proc w/o card cath w MCC */
'220' = '1' /* Cardiac valve & oth maj cardiothoracic proc w/o card cath w CC */
'221' = '1' /* Cardiac valve & oth maj cardiothoracic proc w/o card cath w/o CC/MCC */
'222' = '1' /* Cardiac defib implant w cardiac cath w AMI/HF/shock w MCC */
'223' = '1' /* Cardiac defib implant w cardiac cath w AMI/HF/shock w/o MCC */
'224' = '1' /* Cardiac defib implant w cardiac cath w/o AMI/HF/shock w MCC */
'225' = '1' /* Cardiac defib implant w cardiac cath w/o AMI/HF/shock w/o MCC */
'226' = '1' /* Cardiac defibrillator implant w/o cardiac cath w MCC */
'227' = '1' /* Cardiac defibrillator implant w/o cardiac cath w/o MCC */
'228' = '1' /* Other cardiothoracic procedures w MCC */
'229' = '1' /* Other cardiothoracic procedures w CC */
'230' = '1' /* Other cardiothoracic procedures w/o CC/MCC */
'231' = '1' /* Coronary bypass w PTCA w MCC */
'232' = '1' /* Coronary bypass w PTCA w/o MCC */
'233' = '1' /* Coronary bypass w cardiac cath w MCC */
'234' = '1' /* Coronary bypass w cardiac cath w/o MCC */
'235' = '1' /* Coronary bypass w/o cardiac cath w MCC */
'236' = '1' /* Coronary bypass w/o cardiac cath w/o MCC */
'237' = '1' /* Major cardiovasc procedures w MCC or thoracic aortic anuerysm repair */
'238' = '1' /* Major cardiovasc procedures w/o MCC */
'239' = '1' /* Amputation for circ sys disorders exc upper limb & toe w MCC */
'240' = '1' /* Amputation for circ sys disorders exc upper limb & toe w CC */
'241' = '1' /* Amputation for circ sys disorders exc upper limb & toe w/o CC/MCC */
'242' = '1' /* Permanent cardiac pacemaker implant w MCC */
'243' = '1' /* Permanent cardiac pacemaker implant w CC */
'244' = '1' /* Permanent cardiac pacemaker implant w/o CC/MCC */
'245' = '1' /* AICD lead & generator procedures */
'246' = '1' /* Perc cardiovasc proc w drug-eluting stent w MCC or 4+ vessels/stents */
'247' = '1' /* Perc cardiovasc proc w drug-eluting stent w/o MCC */
'248' = '1' /* Perc cardiovasc proc w non-drug-eluting stent w MCC or 4+ ves/stents */
'249' = '1' /* Perc cardiovasc proc w non-drug-eluting stent w/o MCC */
'250' = '1' /* Perc cardiovasc proc w/o coronary artery stent or AMI w MCC */
'251' = '1' /* Perc cardiovasc proc w/o coronary artery stent or AMI w/o MCC */
'252' = '1' /* Other vascular procedures w MCC */
'253' = '1' /* Other vascular procedures w CC */
'254' = '1' /* Other vascular procedures w/o CC/MCC */
'255' = '1' /* Upper limb & toe amputation for circ system disorders w MCC */
'256' = '1' /* Upper limb & toe amputation for circ system disorders w CC */
'257' = '1' /* Upper limb & toe amputation for circ system disorders w/o CC/MCC */
'258' = '1' /* Cardiac pacemaker device replacement w MCC */
'259' = '1' /* Cardiac pacemaker device replacement w/o MCC */
'260' = '1' /* Cardiac pacemaker revision except device replacement w MCC */
'261' = '1' /* Cardiac pacemaker revision except device replacement w CC */
'262' = '1' /* Cardiac pacemaker revision except device replacement w/o CC/MCC */
'263' = '1' /* Vein ligation & stripping */
'264' = '1' /* Other circulatory system O.R. procedures */
'265' = '1' /* AICD LEAD PROCEDURES */
'266' = '1' /* Endovascular Cardiac Valve Replacement w MCC */
'267' = '1' /* Endovascular Cardiac Valve Replacement w/o MCC */
'326' = '1' /* Stomach, esophageal & duodenal proc w MCC */
'327' = '1' /* Stomach, esophageal & duodenal proc w CC */
'328' = '1' /* Stomach, esophageal & duodenal proc w/o CC/MCC */
'329' = '1' /* Major small & large bowel procedures w MCC */
'330' = '1' /* Major small & large bowel procedures w CC */
'331' = '1' /* Major small & large bowel procedures w/o CC/MCC */
'332' = '1' /* Rectal resection w MCC */
'333' = '1' /* Rectal resection w CC */
'334' = '1' /* Rectal resection w/o CC/MCC */
'335' = '1' /* Peritoneal adhesiolysis w MCC */
'336' = '1' /* Peritoneal adhesiolysis w CC */
'337' = '1' /* Peritoneal adhesiolysis w/o CC/MCC */
'338' = '1' /* Appendectomy w complicated principal diag w MCC */
'339' = '1' /* Appendectomy w complicated principal diag w CC */
'340' = '1' /* Appendectomy w complicated principal diag w/o CC/MCC */
'341' = '1' /* Appendectomy w/o complicated principal diag w MCC */
'342' = '1' /* Appendectomy w/o complicated principal diag w CC */
'343' = '1' /* Appendectomy w/o complicated principal diag w/o CC/MCC */
'344' = '1' /* Minor small & large bowel procedures w MCC */
'345' = '1' /* Minor small & large bowel procedures w CC */
'346' = '1' /* Minor small & large bowel procedures w/o CC/MCC */
'347' = '1' /* Anal & stomal procedures w MCC */
'348' = '1' /* Anal & stomal procedures w CC */
'349' = '1' /* Anal & stomal procedures w/o CC/MCC */
'350' = '1' /* Inguinal & femoral hernia procedures w MCC */
'351' = '1' /* Inguinal & femoral hernia procedures w CC */
'352' = '1' /* Inguinal & femoral hernia procedures w/o CC/MCC */
'353' = '1' /* Hernia procedures except inguinal & femoral w MCC */
'354' = '1' /* Hernia procedures except inguinal & femoral w CC */
'355' = '1' /* Hernia procedures except inguinal & femoral w/o CC/MCC */
'356' = '1' /* Other digestive system O.R. procedures w MCC */
'357' = '1' /* Other digestive system O.R. procedures w CC */
'358' = '1' /* Other digestive system O.R. procedures w/o CC/MCC */
'405' = '1' /* Pancreas, liver & shunt procedures w MCC */
'406' = '1' /* Pancreas, liver & shunt procedures w CC */
'407' = '1' /* Pancreas, liver & shunt procedures w/o CC/MCC */
'408' = '1' /* Biliary tract proc except only cholecyst w or w/o c.d.e. w MCC */
'409' = '1' /* Biliary tract proc except only cholecyst w or w/o c.d.e. w CC */
'410' = '1' /* Biliary tract proc except only cholecyst w or w/o c.d.e. w/o CC/MCC */
'411' = '1' /* Cholecystectomy w c.d.e. w MCC */
'412' = '1' /* Cholecystectomy w c.d.e. w CC */
'413' = '1' /* Cholecystectomy w c.d.e. w/o CC/MCC */
'414' = '1' /* Cholecystectomy except by laparoscope w/o c.d.e. w MCC */
'415' = '1' /* Cholecystectomy except by laparoscope w/o c.d.e. w CC */
'416' = '1' /* Cholecystectomy except by laparoscope w/o c.d.e. w/o CC/MCC */
'417' = '1' /* Laparoscopic cholecystectomy w/o c.d.e. w MCC */
'418' = '1' /* Laparoscopic cholecystectomy w/o c.d.e. w CC */
'419' = '1' /* Laparoscopic cholecystectomy w/o c.d.e. w/o CC/MCC */
'420' = '1' /* Hepatobiliary diagnostic procedures w MCC */
'421' = '1' /* Hepatobiliary diagnostic procedures w CC */
'422' = '1' /* Hepatobiliary diagnostic procedures w/o CC/MCC */
'423' = '1' /* Other hepatobiliary or pancreas O.R. procedures w MCC */
'424' = '1' /* Other hepatobiliary or pancreas O.R. procedures w CC */
'425' = '1' /* Other hepatobiliary or pancreas O.R. procedures w/o CC/MCC */
'453' = '1' /* Combined anterior/posterior spinal fusion w MCC */
'454' = '1' /* Combined anterior/posterior spinal fusion w CC */
'455' = '1' /* Combined anterior/posterior spinal fusion w/o CC/MCC */
'456' = '1' /* Spinal fus exc cerv w spinal curv/malig/infec or 9+ fus w MCC */
'457' = '1' /* Spinal fus exc cerv w spinal curv/malig/infec or 9+ fus w CC */
'458' = '1' /* Spinal fus exc cerv w spinal curv/malig/infec or 9+ fus w/o CC/MCC */
'459' = '1' /* Spinal fusion except cervical w MCC */
'460' = '1' /* Spinal fusion except cervical w/o MCC */
'461' = '1' /* Bilateral or multiple major joint procs of lower extremity w MCC */
'462' = '1' /* Bilateral or multiple major joint procs of lower extremity w/o MCC */
'463' = '1' /* Wnd debrid & skn grft exc hand, for musculo-conn tiss dis w MCC */
'464' = '1' /* Wnd debrid & skn grft exc hand, for musculo-conn tiss dis w CC */
'465' = '1' /* Wnd debrid & skn grft exc hand, for musculo-conn tiss dis w/o CC/MCC */
'466' = '1' /* Revision of hip or knee replacement w MCC */
'467' = '1' /* Revision of hip or knee replacement w CC */
'468' = '1' /* Revision of hip or knee replacement w/o CC/MCC */
'469' = '1' /* Major joint replacement or reattachment of lower extremity w MCC */
'470' = '1' /* Major joint replacement or reattachment of lower extremity w/o MCC */
'471' = '1' /* Cervical spinal fusion w MCC */
'472' = '1' /* Cervical spinal fusion w CC */
'473' = '1' /* Cervical spinal fusion w/o CC/MCC */
'474' = '1' /* Amputation for musculoskeletal sys & conn tissue dis w MCC */
'475' = '1' /* Amputation for musculoskeletal sys & conn tissue dis w CC */
'476' = '1' /* Amputation for musculoskeletal sys & conn tissue dis w/o CC/MCC */
'477' = '1' /* Biopsies of musculoskeletal system & connective tissue w MCC */
'478' = '1' /* Biopsies of musculoskeletal system & connective tissue w CC */
'479' = '1' /* Biopsies of musculoskeletal system & connective tissue w/o CC/MCC */
'480' = '1' /* Hip & femur procedures except major joint w MCC */
'481' = '1' /* Hip & femur procedures except major joint w CC */
'482' = '1' /* Hip & femur procedures except major joint w/o CC/MCC */
'483' = '1' /* Major joint & limb reattachment proc of upper extremity w CC/MCC */
'484' = '1' /* Major joint & limb reattachment proc of upper extremity w/o CC/MCC */
'485' = '1' /* Knee procedures w pdx of infection w MCC */
'486' = '1' /* Knee procedures w pdx of infection w CC */
'487' = '1' /* Knee procedures w pdx of infection w/o CC/MCC */
'488' = '1' /* Knee procedures w/o pdx of infection w CC/MCC */
'489' = '1' /* Knee procedures w/o pdx of infection w/o CC/MCC */
'490' = '1' /* Back & neck proc exc spinal fusion w CC/MCC or disc device/neurostim */
'491' = '1' /* Back & neck proc exc spinal fusion w/o CC/MCC */
'492' = '1' /* Lower extrem & humer proc except hip,foot,femur w MCC */
'493' = '1' /* Lower extrem & humer proc except hip,foot,femur w CC */
'494' = '1' /* Lower extrem & humer proc except hip,foot,femur w/o CC/MCC */
'495' = '1' /* Local excision & removal int fix devices exc hip & femur w MCC */
'496' = '1' /* Local excision & removal int fix devices exc hip & femur w CC */
'497' = '1' /* Local excision & removal int fix devices exc hip & femur w/o CC/MCC */
'498' = '1' /* Local excision & removal int fix devices of hip & femur w CC/MCC */
'499' = '1' /* Local excision & removal int fix devices of hip & femur w/o CC/MCC */
'500' = '1' /* Soft tissue procedures w MCC */
'501' = '1' /* Soft tissue procedures w CC */
'502' = '1' /* Soft tissue procedures w/o CC/MCC */
'503' = '1' /* Foot procedures w MCC */
'504' = '1' /* Foot procedures w CC */
'505' = '1' /* Foot procedures w/o CC/MCC */
'506' = '1' /* Major thumb or joint procedures */
'507' = '1' /* Major shoulder or elbow joint procedures w CC/MCC */
'508' = '1' /* Major shoulder or elbow joint procedures w/o CC/MCC */
'509' = '1' /* Arthroscopy */
'510' = '1' /* Shoulder,elbow or forearm proc,exc major joint proc w MCC */
'511' = '1' /* Shoulder,elbow or forearm proc,exc major joint proc w CC */
'512' = '1' /* Shoulder,elbow or forearm proc,exc major joint proc w/o CC/MCC */
'513' = '1' /* Hand or wrist proc, except major thumb or joint proc w CC/MCC */
'514' = '1' /* Hand or wrist proc, except major thumb or joint proc w/o CC/MCC */
'515' = '1' /* Other musculoskelet sys & conn tiss O.R. proc w MCC */
'516' = '1' /* Other musculoskelet sys & conn tiss O.R. proc w CC */
'517' = '1' /* Other musculoskelet sys & conn tiss O.R. proc w/o CC/MCC */
'518' = '1' /* Back & Neck Procedure excluding spinal fusion w MCC or disc device/neurostimulator */
'519' = '1' /* Back & Neck Procedure excluding spinal fusion w CC */
'520' = '1' /* Back & Neck Procedure excluding spinal fusion w/o CC/MCC */
'570' = '1' /* SKIN DEBRIDEMENT W MCC */
'571' = '1' /* SKIN DEBRIDEMENT W CC */
'572' = '1' /* SKIN DEBRIDEMENT W/O CC/MCC */
'573' = '1' /* Skin graft for skn ulcer or cellulitis w MCC */
'574' = '1' /* Skin graft for skn ulcer or cellulitis w CC */
'575' = '1' /* Skin graft for skn ulcer or cellulitis w/o CC/MCC */
'576' = '1' /* Skin graft exc for skin ulcer or cellulitis w MCC */
'577' = '1' /* Skin graft exc for skin ulcer or cellulitis w CC */
'578' = '1' /* Skin graft exc for skin ulcer or cellulitis w/o CC/MCC */
'579' = '1' /* Other skin, subcut tiss & breast proc w MCC */
'580' = '1' /* Other skin, subcut tiss & breast proc w CC */
'581' = '1' /* Other skin, subcut tiss & breast proc w/o CC/MCC */
'582' = '1' /* Mastectomy for malignancy w CC/MCC */
'583' = '1' /* Mastectomy for malignancy w/o CC/MCC */
'584' = '1' /* Breast biopsy, local excision & other breast procedures w CC/MCC */
'585' = '1' /* Breast biopsy, local excision & other breast procedures w/o CC/MCC */
'614' = '1' /* Adrenal & pituitary procedures w CC/MCC */
'615' = '1' /* Adrenal & pituitary procedures w/o CC/MCC */
'616' = '1' /* Amputat of lower limb for endocrine,nutrit,& metabol dis w MCC */
'617' = '1' /* Amputat of lower limb for endocrine,nutrit,& metabol dis w CC */
'618' = '1' /* Amputat of lower limb for endocrine,nutrit,& metabol dis w/o CC/MCC */
'619' = '1' /* O.R. procedures for obesity w MCC */
'620' = '1' /* O.R. procedures for obesity w CC */
'621' = '1' /* O.R. procedures for obesity w/o CC/MCC */
'622' = '1' /* Skin grafts & wound debrid for endoc, nutrit & metab dis w MCC */
'623' = '1' /* Skin grafts & wound debrid for endoc, nutrit & metab dis w CC */
'624' = '1' /* Skin grafts & wound debrid for endoc, nutrit & metab dis w/o CC/MCC */
'625' = '1' /* Thyroid, parathyroid & thyroglossal procedures w MCC */
'626' = '1' /* Thyroid, parathyroid & thyroglossal procedures w CC */
'627' = '1' /* Thyroid, parathyroid & thyroglossal procedures w/o CC/MCC */
'628' = '1' /* Other endocrine, nutrit & metab O.R. proc w MCC */
'629' = '1' /* Other endocrine, nutrit & metab O.R. proc w CC */
'630' = '1' /* Other endocrine, nutrit & metab O.R. proc w/o CC/MCC */
'652' = '1' /* Kidney transplant */
'653' = '1' /* Major bladder procedures w MCC */
'654' = '1' /* Major bladder procedures w CC */
'655' = '1' /* Major bladder procedures w/o CC/MCC */
'656' = '1' /* Kidney & ureter procedures for neoplasm w MCC */
'657' = '1' /* Kidney & ureter procedures for neoplasm w CC */
'658' = '1' /* Kidney & ureter procedures for neoplasm w/o CC/MCC */
'659' = '1' /* Kidney & ureter procedures for non-neoplasm w MCC */
'660' = '1' /* Kidney & ureter procedures for non-neoplasm w CC */
'661' = '1' /* Kidney & ureter procedures for non-neoplasm w/o CC/MCC */
'662' = '1' /* Minor bladder procedures w MCC */
'663' = '1' /* Minor bladder procedures w CC */
'664' = '1' /* Minor bladder procedures w/o CC/MCC */
'665' = '1' /* Prostatectomy w MCC */
'666' = '1' /* Prostatectomy w CC */
'667' = '1' /* Prostatectomy w/o CC/MCC */
'668' = '1' /* Transurethral procedures w MCC */
'669' = '1' /* Transurethral procedures w CC */
'670' = '1' /* Transurethral procedures w/o CC/MCC */
'671' = '1' /* Urethral procedures w CC/MCC */
'672' = '1' /* Urethral procedures w/o CC/MCC */
'673' = '1' /* Other kidney & urinary tract procedures w MCC */
'674' = '1' /* Other kidney & urinary tract procedures w CC */
'675' = '1' /* Other kidney & urinary tract procedures w/o CC/MCC */
'707' = '1' /* Major male pelvic procedures w CC/MCC */
'708' = '1' /* Major male pelvic procedures w/o CC/MCC */
'709' = '1' /* Penis procedures w CC/MCC */
'710' = '1' /* Penis procedures w/o CC/MCC */
'711' = '1' /* Testes procedures w CC/MCC */
'712' = '1' /* Testes procedures w/o CC/MCC */
'713' = '1' /* Transurethral prostatectomy w CC/MCC */
'714' = '1' /* Transurethral prostatectomy w/o CC/MCC */
'715' = '1' /* Other male reproductive system O.R. proc for malignancy w CC/MCC */
'716' = '1' /* Other male reproductive system O.R. proc for malignancy w/o CC/MCC */
'717' = '1' /* Other male reproductive system O.R. proc exc malignancy w CC/MCC */
'718' = '1' /* Other male reproductive system O.R. proc exc malignancy w/o CC/MCC */
'734' = '1' /* Pelvic evisceration, rad hysterectomy & rad vulvectomy w CC/MCC */
'735' = '1' /* Pelvic evisceration, rad hysterectomy & rad vulvectomy w/o CC/MCC */
'736' = '1' /* Uterine & adnexa proc for ovarian or adnexal malignancy w MCC */
'737' = '1' /* Uterine & adnexa proc for ovarian or adnexal malignancy w CC */
'738' = '1' /* Uterine & adnexa proc for ovarian or adnexal malignancy w/o CC/MCC */
'739' = '1' /* Uterine,adnexa proc for non-ovarian/adnexal malig w MCC */
'740' = '1' /* Uterine,adnexa proc for non-ovarian/adnexal malig w CC */
'741' = '1' /* Uterine,adnexa proc for non-ovarian/adnexal malig w/o CC/MCC */
'742' = '1' /* Uterine & adnexa proc for non-malignancy w CC/MCC */
'743' = '1' /* Uterine & adnexa proc for non-malignancy w/o CC/MCC */
'744' = '1' /* D&C, conization, laparascopy & tubal interruption w CC/MCC */
'745' = '1' /* D&C, conization, laparascopy & tubal interruption w/o CC/MCC */
'746' = '1' /* Vagina, cervix & vulva procedures w CC/MCC */
'747' = '1' /* Vagina, cervix & vulva procedures w/o CC/MCC */
'748' = '1' /* Female reproductive system reconstructive procedures */
'749' = '1' /* Other female reproductive system O.R. procedures w CC/MCC */
'750' = '1' /* Other female reproductive system O.R. procedures w/o CC/MCC */
'765' = '1' /* Cesarean section w CC/MCC */
'766' = '1' /* Cesarean section w/o CC/MCC */
'767' = '1' /* Vaginal delivery w sterilization &/or D&C */
'768' = '1' /* Vaginal delivery w O.R. proc except steril &/or D&C */
'769' = '1' /* Postpartum & post abortion diagnoses w O.R. procedure */
'770' = '1' /* Abortion w D&C, aspiration curettage or hysterotomy */
'799' = '1' /* Splenectomy w MCC */
'800' = '1' /* Splenectomy w CC */
'801' = '1' /* Splenectomy w/o CC/MCC */
'802' = '1' /* Other O.R. proc of the blood & blood forming organs w MCC */
'803' = '1' /* Other O.R. proc of the blood & blood forming organs w CC */
'804' = '1' /* Other O.R. proc of the blood & blood forming organs w/o CC/MCC */
'820' = '1' /* Lymphoma & leukemia w major O.R. procedure w MCC */
'821' = '1' /* Lymphoma & leukemia w major O.R. procedure w CC */
'822' = '1' /* Lymphoma & leukemia w major O.R. procedure w/o CC/MCC */
'823' = '1' /* Lymphoma & non-acute leukemia w other O.R. proc w MCC */
'824' = '1' /* Lymphoma & non-acute leukemia w other O.R. proc w CC */
'825' = '1' /* Lymphoma & non-acute leukemia w other O.R. proc w/o CC/MCC */
'826' = '1' /* Myeloprolif disord or poorly diff neopl w maj O.R. proc w MCC */
'827' = '1' /* Myeloprolif disord or poorly diff neopl w maj O.R. proc w CC */
'828' = '1' /* Myeloprolif disord or poorly diff neopl w maj O.R. proc w/o CC/MCC */
'829' = '1' /* Myeloprolif disord or poorly diff neopl w other O.R. proc w CC/MCC */
'830' = '1' /* Myeloprolif disord or poorly diff neopl w other O.R. proc w/o CC/MCC */
'853' = '1' /* Infectious & parasitic diseases w O.R. procedure w MCC */
'854' = '1' /* Infectious & parasitic diseases w O.R. procedure w CC */
'855' = '1' /* Infectious & parasitic diseases w O.R. procedure w/o CC/MCC */
'856' = '1' /* Postoperative or post-traumatic infections w O.R. proc w MCC */
'857' = '1' /* Postoperative or post-traumatic infections w O.R. proc w CC */
'858' = '1' /* Postoperative or post-traumatic infections w O.R. proc w/o CC/MCC */
'876' = '1' /* O.R. procedure w principal diagnoses of mental illness */
'901' = '1' /* Wound debridements for injuries w MCC */
'902' = '1' /* Wound debridements for injuries w CC */
'903' = '1' /* Wound debridements for injuries w/o CC/MCC */
'904' = '1' /* Skin grafts for injuries w CC/MCC */
'905' = '1' /* Skin grafts for injuries w/o CC/MCC */
'906' = '1' /* Hand procedures for injuries */
'907' = '1' /* Other O.R. procedures for injuries w MCC */
'908' = '1' /* Other O.R. procedures for injuries w CC */
'909' = '1' /* Other O.R. procedures for injuries w/o CC/MCC */
'927' = '1' /* Extensive burns or full thickness burns w MV 96+ hrs w skin graft */
'928' = '1' /* Full thickness burn w skin graft or inhal inj w CC/MCC */
'929' = '1' /* Full thickness burn w skin graft or inhal inj w/o CC/MCC */
'939' = '1' /* O.R. proc w diagnoses of other contact w health services w MCC */
'940' = '1' /* O.R. proc w diagnoses of other contact w health services w CC */
'941' = '1' /* O.R. proc w diagnoses of other contact w health services w/o CC/MCC */
'955' = '1' /* Craniotomy for multiple significant trauma */
'956' = '1' /* Limb reattachment, hip & femur proc for multiple significant trauma */
'957' = '1' /* Other O.R. procedures for multiple significant trauma w MCC */
'958' = '1' /* Other O.R. procedures for multiple significant trauma w CC */
'959' = '1' /* Other O.R. procedures for multiple significant trauma w/o CC/MCC */
'969' = '1' /* HIV w extensive O.R. procedure w MCC */
'970' = '1' /* HIV w extensive O.R. procedure w/o MCC */
'981' = '1' /* Extensive O.R. procedure unrelated to principal diagnosis w MCC */
'982' = '1' /* Extensive O.R. procedure unrelated to principal diagnosis w CC */
'983' = '1' /* Extensive O.R. procedure unrelated to principal diagnosis w/o CC/MCC */
'984' = '1' /* Prostatic O.R. procedure unrelated to principal diagnosis w MCC */
'985' = '1' /* Prostatic O.R. procedure unrelated to principal diagnosis w CC */
'986' = '1' /* Prostatic O.R. procedure unrelated to principal diagnosis w/o CC/MCC */
'987' = '1' /* Non-extensive O.R. proc unrelated to principal diagnosis w MCC */
'988' = '1' /* Non-extensive O.R. proc unrelated to principal diagnosis w CC */
'989' = '1' /* Non-extensive O.R. proc unrelated to principal diagnosis w/o CC/MCC */
OTHER = '0' ;

 /* ---------------------------------------------------------------- */
 /* LOW BIRTH WEIGHT                                                 */
 /* ---------------------------------------------------------------- */

 /* ICD-10-CM DIAGNOSIS CODES: */

VALUE $LW500G
"P0501"  =  "1"  /* Newborn light for gestational age, less than 500 grams */
"P0511"  =  "1"  /* Newborn small for gestational age, less than 500 grams */
"P0701"  =  "1"  /* Extremely low birth weight newborn, less than 500 grams */
OTHER = "0" ;

VALUE $LW750G
"P0502"  =  "1"  /* Newborn light for gestational age, 500-749 grams */
"P0512"  =  "1"  /* Newborn small for gestational age, 500-749 grams */
"P0702"  =  "1"  /* Extremely low birth weight newborn, 500-749 grams */
OTHER   = "0";

VALUE $LW1000G
"P0503"  =  "1"  /* Newborn light for gestational age, 750-999 grams */
"P0513"  =  "1"  /* Newborn small for gestational age, 750-999 grams */
"P0703"  =  "1"  /* Extremely low birth weight newborn, 750-999 grams */
OTHER   = "0";

VALUE $LW1250G
"P0504"  =  "1"  /* Newborn light for gestational age, 1000-1249 grams */
"P0514"  =  "1"  /* Newborn small for gestational age, 1000-1249 grams */
"P0714"  =  "1"  /* Other low birth weight newborn, 1000-1249 grams */
OTHER   = "0";

VALUE $LW1500G
"P0505"  =  "1"  /* Newborn light for gestational age, 1250-1499 grams */
"P0515"  =  "1"  /* Newborn small for gestational age, 1250-1499 grams */
"P0715"  =  "1"  /* Other low birth weight newborn, 1250-1499 grams */
OTHER   = "0";

VALUE $LW1750G
"P0506"  =  "1"  /* Newborn light for gestational age, 1500-1749 grams */
"P0516"  =  "1"  /* Newborn small for gestational age, 1500-1749 grams */
"P0716"  =  "1"  /* Other low birth weight newborn, 1500-1749 grams */
OTHER   = "0";

VALUE $LW2000G
"P0507"  =  "1"  /* Newborn light for gestational age, 1750-1999 grams */
"P0517"  =  "1"  /* Newborn small for gestational age, 1750-1999 grams */
"P0717"  =  "1"  /* Other low birth weight newborn, 1750-1999 grams */
OTHER   = "0";

VALUE $LW2500G
"P0508"  =  "1"  /* Newborn light for gestational age, 2000-2499 grams */
"P0518"  =  "1"  /* Newborn small for gestational age, 2000-2499 grams */
"P0718"  =  "1"  /* Other low birth weight newborn, 2000-2499 grams */
OTHER   = "0";

 /* VALUE $LW2500P according to UCD None of the ICD-10-CM codes apply */


 /* ------------------------------------ */
 /* ACCIDENTAL PUNCTURE OR LACERATION   */
 /*   PROVIDER INDICATOR / PSI #  1     */
 /*   AREA     INDICATOR / PSI # TBD    */
 /* ----------------------------------- */

 /* ICD-10-CM DIAGNOSIS CODES: */

VALUE $TECHNID
"D7811"  =  "1"  /* Accidental puncture and laceration of the spleen during a procedure on the spleen */
"D7812"  =  "1"  /* Accidental puncture and laceration of the spleen during other procedure */
"E3611"  =  "1"  /* Accidental puncture and laceration of an endocrine system organ or structure during an endocrine system procedure */
"E3612"  =  "1"  /* Accidental puncture and laceration of an endocrine system organ or structure during other procedure */
"G9748"  =  "1"  /* Accidental puncture and laceration of other nervous system organ or structure during a nervous system procedure */
"G9749"  =  "1"  /* Accidental puncture and laceration of other nervous system organ or structure during other procedure */
"H59211"  =  "1"  /* Accidental puncture and laceration of right eye and adnexa during an ophthalmic procedure */
"H59212"  =  "1"  /* Accidental puncture and laceration of left eye and adnexa during an ophthalmic procedure */
"H59213"  =  "1"  /* Accidental puncture and laceration of eye and adnexa during an ophthalmic procedure, bilateral */
"H59219"  =  "1"  /* Accidental puncture and laceration of unspecified eye and adnexa during an ophthalmic procedure */
"H59221"  =  "1"  /* Accidental puncture and laceration of right eye and adnexa during other procedure */
"H59222"  =  "1"  /* Accidental puncture and laceration of left eye and adnexa during other procedure */
"H59223"  =  "1"  /* Accidental puncture and laceration of eye and adnexa during other procedure, bilateral */
"H59229"  =  "1"  /* Accidental puncture and laceration of unspecified eye and adnexa during other procedure */
"H9531"  =  "1"  /* Accidental puncture and laceration of the ear and mastoid process during a procedure on the ear and mastoid process */
"H9532"  =  "1"  /* Accidental puncture and laceration of the ear and mastoid process during other procedure */
"I9751"  =  "1"  /* Accidental puncture and laceration of a circulatory system organ or structure during a circulatory system procedure */
"I9752"  =  "1"  /* Accidental puncture and laceration of a circulatory system organ or structure during other procedure */
"J9571"  =  "1"  /* Accidental puncture and laceration of a respiratory system organ or structure during a respiratory system procedure */
"J9572"  =  "1"  /* Accidental puncture and laceration of a respiratory system organ or structure during other procedure */
"K9171"  =  "1"  /* Accidental puncture and laceration of a digestive system organ or structure during a digestive system procedure */
"K9172"  =  "1"  /* Accidental puncture and laceration of a digestive system organ or structure during other procedure */
"L7611"  =  "1"  /* Accidental puncture and laceration of skin and subcutaneous tissue during a dermatologic procedure */
"L7612"  =  "1"  /* Accidental puncture and laceration of skin and subcutaneous tissue during other procedure */
"M96820"  =  "1"  /* Accidental puncture and laceration of a musculoskeletal structure during a musculoskeletal system procedure */
"M96821"  =  "1"  /* Accidental puncture and laceration of a musculoskeletal structure during other procedure */
"N9971"  =  "1"  /* Accidental puncture and laceration of a genitourinary system organ or structure during a genitourinary system procedure */
"N9972"  =  "1"  /* Accidental puncture and laceration of a genitourinary system organ or structure during other procedure */
OTHER   = "0" ;


VALUE $INFECID
"A000"  =  "1"  /* Cholera due to Vibrio cholerae 01, biovar cholerae */
"A001"  =  "1"  /* Cholera due to Vibrio cholerae 01, biovar eltor */
"A009"  =  "1"  /* Cholera, unspecified */
"A0100"  =  "1"  /* Typhoid fever, unspecified */
"A0101"  =  "1"  /* Typhoid meningitis */
"A0102"  =  "1"  /* Typhoid fever with heart involvement */
"A0103"  =  "1"  /* Typhoid pneumonia */
"A0104"  =  "1"  /* Typhoid arthritis */
"A0105"  =  "1"  /* Typhoid osteomyelitis */
"A0109"  =  "1"  /* Typhoid fever with other complications */
"A011"  =  "1"  /* Paratyphoid fever A */
"A012"  =  "1"  /* Paratyphoid fever B */
"A013"  =  "1"  /* Paratyphoid fever C */
"A014"  =  "1"  /* Paratyphoid fever, unspecified */
"A020"  =  "1"  /* Salmonella enteritis */
"A021"  =  "1"  /* Salmonella sepsis */
"A0220"  =  "1"  /* Localized salmonella infection, unspecified */
"A0221"  =  "1"  /* Salmonella meningitis */
"A0222"  =  "1"  /* Salmonella pneumonia */
"A0223"  =  "1"  /* Salmonella arthritis */
"A0224"  =  "1"  /* Salmonella osteomyelitis */
"A0225"  =  "1"  /* Salmonella pyelonephritis */
"A0229"  =  "1"  /* Salmonella with other localized infection */
"A028"  =  "1"  /* Other specified salmonella infections */
"A029"  =  "1"  /* Salmonella infection, unspecified */
"A030"  =  "1"  /* Shigellosis due to Shigella dysenteriae */
"A031"  =  "1"  /* Shigellosis due to Shigella flexneri */
"A032"  =  "1"  /* Shigellosis due to Shigella boydii */
"A033"  =  "1"  /* Shigellosis due to Shigella sonnei */
"A038"  =  "1"  /* Other shigellosis */
"A039"  =  "1"  /* Shigellosis, unspecified */
"A040"  =  "1"  /* Enteropathogenic Escherichia coli infection */
"A041"  =  "1"  /* Enterotoxigenic Escherichia coli infection */
"A042"  =  "1"  /* Enteroinvasive Escherichia coli infection */
"A043"  =  "1"  /* Enterohemorrhagic Escherichia coli infection */
"A044"  =  "1"  /* Other intestinal Escherichia coli infections */
"A045"  =  "1"  /* Campylobacter enteritis */
"A046"  =  "1"  /* Enteritis due to Yersinia enterocolitica */
"A047"  =  "1"  /* Enterocolitis due to Clostridium difficile */
"A048"  =  "1"  /* Other specified bacterial intestinal infections */
"A049"  =  "1"  /* Bacterial intestinal infection, unspecified */
"A050"  =  "1"  /* Foodborne staphylococcal intoxication */
"A051"  =  "1"  /* Botulism food poisoning */
"A052"  =  "1"  /* Foodborne Clostridium perfringens [Clostridium welchii] intoxication */
"A053"  =  "1"  /* Foodborne Vibrio parahaemolyticus intoxication */
"A054"  =  "1"  /* Foodborne Bacillus cereus intoxication */
"A055"  =  "1"  /* Foodborne Vibrio vulnificus intoxication */
"A058"  =  "1"  /* Other specified bacterial foodborne intoxications */
"A059"  =  "1"  /* Bacterial foodborne intoxication, unspecified */
"A1801"  =  "1"  /* Tuberculosis of spine */
"A1803"  =  "1"  /* Tuberculosis of other bones */
"A1814"  =  "1"  /* Tuberculosis of prostate */
"A1884"  =  "1"  /* Tuberculosis of heart */
"A200"  =  "1"  /* Bubonic plague */
"A201"  =  "1"  /* Cellulocutaneous plague */
"A202"  =  "1"  /* Pneumonic plague */
"A203"  =  "1"  /* Plague meningitis */
"A207"  =  "1"  /* Septicemic plague */
"A208"  =  "1"  /* Other forms of plague */
"A209"  =  "1"  /* Plague, unspecified */
"A210"  =  "1"  /* Ulceroglandular tularemia */
"A211"  =  "1"  /* Oculoglandular tularemia */
"A212"  =  "1"  /* Pulmonary tularemia */
"A213"  =  "1"  /* Gastrointestinal tularemia */
"A217"  =  "1"  /* Generalized tularemia */
"A218"  =  "1"  /* Other forms of tularemia */
"A219"  =  "1"  /* Tularemia, unspecified */
"A220"  =  "1"  /* Cutaneous anthrax */
"A221"  =  "1"  /* Pulmonary anthrax */
"A222"  =  "1"  /* Gastrointestinal anthrax */
"A227"  =  "1"  /* Anthrax sepsis */
"A228"  =  "1"  /* Other forms of anthrax */
"A229"  =  "1"  /* Anthrax, unspecified */
"A230"  =  "1"  /* Brucellosis due to Brucella melitensis */
"A231"  =  "1"  /* Brucellosis due to Brucella abortus */
"A232"  =  "1"  /* Brucellosis due to Brucella suis */
"A233"  =  "1"  /* Brucellosis due to Brucella canis */
"A238"  =  "1"  /* Other brucellosis */
"A239"  =  "1"  /* Brucellosis, unspecified */
"A240"  =  "1"  /* Glanders */
"A241"  =  "1"  /* Acute and fulminating melioidosis */
"A242"  =  "1"  /* Subacute and chronic melioidosis */
"A243"  =  "1"  /* Other melioidosis */
"A249"  =  "1"  /* Melioidosis, unspecified */
"A250"  =  "1"  /* Spirillosis */
"A251"  =  "1"  /* Streptobacillosis */
"A259"  =  "1"  /* Rat-bite fever, unspecified */
"A260"  =  "1"  /* Cutaneous erysipeloid */
"A267"  =  "1"  /* Erysipelothrix sepsis */
"A268"  =  "1"  /* Other forms of erysipeloid */
"A269"  =  "1"  /* Erysipeloid, unspecified */
"A280"  =  "1"  /* Pasteurellosis */
"A281"  =  "1"  /* Cat-scratch disease */
"A282"  =  "1"  /* Extraintestinal yersiniosis */
"A288"  =  "1"  /* Other specified zoonotic bacterial diseases, not elsewhere classified */
"A289"  =  "1"  /* Zoonotic bacterial disease, unspecified */
"A320"  =  "1"  /* Cutaneous listeriosis */
"A3211"  =  "1"  /* Listerial meningitis */
"A3212"  =  "1"  /* Listerial meningoencephalitis */
"A327"  =  "1"  /* Listerial sepsis */
"A3281"  =  "1"  /* Oculoglandular listeriosis */
"A3282"  =  "1"  /* Listerial endocarditis */
"A3289"  =  "1"  /* Other forms of listeriosis */
"A329"  =  "1"  /* Listeriosis, unspecified */
"A33"  =  "1"  /* Tetanus neonatorum */
"A34"  =  "1"  /* Obstetrical tetanus */
"A35"  =  "1"  /* Other tetanus */
"A360"  =  "1"  /* Pharyngeal diphtheria */
"A361"  =  "1"  /* Nasopharyngeal diphtheria */
"A362"  =  "1"  /* Laryngeal diphtheria */
"A363"  =  "1"  /* Cutaneous diphtheria */
"A3681"  =  "1"  /* Diphtheritic cardiomyopathy */
"A3682"  =  "1"  /* Diphtheritic radiculomyelitis */
"A3683"  =  "1"  /* Diphtheritic polyneuritis */
"A3684"  =  "1"  /* Diphtheritic tubulo-interstitial nephropathy */
"A3685"  =  "1"  /* Diphtheritic cystitis */
"A3686"  =  "1"  /* Diphtheritic conjunctivitis */
"A3689"  =  "1"  /* Other diphtheritic complications */
"A369"  =  "1"  /* Diphtheria, unspecified */
"A3700"  =  "1"  /* Whooping cough due to Bordetella pertussis without pneumonia */
"A3701"  =  "1"  /* Whooping cough due to Bordetella pertussis with pneumonia */
"A3710"  =  "1"  /* Whooping cough due to Bordetella parapertussis without pneumonia */
"A3711"  =  "1"  /* Whooping cough due to Bordetella parapertussis with pneumonia */
"A3780"  =  "1"  /* Whooping cough due to other Bordetella species without pneumonia */
"A3781"  =  "1"  /* Whooping cough due to other Bordetella species with pneumonia */
"A3790"  =  "1"  /* Whooping cough, unspecified species without pneumonia */
"A3791"  =  "1"  /* Whooping cough, unspecified species with pneumonia */
"A380"  =  "1"  /* Scarlet fever with otitis media */
"A381"  =  "1"  /* Scarlet fever with myocarditis */
"A388"  =  "1"  /* Scarlet fever with other complications */
"A389"  =  "1"  /* Scarlet fever, uncomplicated */
"A390"  =  "1"  /* Meningococcal meningitis */
"A391"  =  "1"  /* Waterhouse-Friderichsen syndrome */
"A392"  =  "1"  /* Acute meningococcemia */
"A393"  =  "1"  /* Chronic meningococcemia */
"A394"  =  "1"  /* Meningococcemia, unspecified */
"A3950"  =  "1"  /* Meningococcal carditis, unspecified */
"A3951"  =  "1"  /* Meningococcal endocarditis */
"A3952"  =  "1"  /* Meningococcal myocarditis */
"A3953"  =  "1"  /* Meningococcal pericarditis */
"A3981"  =  "1"  /* Meningococcal encephalitis */
"A3982"  =  "1"  /* Meningococcal retrobulbar neuritis */
"A3983"  =  "1"  /* Meningococcal arthritis */
"A3984"  =  "1"  /* Postmeningococcal arthritis */
"A3989"  =  "1"  /* Other meningococcal infections */
"A399"  =  "1"  /* Meningococcal infection, unspecified */
"A400"  =  "1"  /* Sepsis due to streptococcus, group A */
"A401"  =  "1"  /* Sepsis due to streptococcus, group B */
"A403"  =  "1"  /* Sepsis due to Streptococcus pneumoniae */
"A408"  =  "1"  /* Other streptococcal sepsis */
"A409"  =  "1"  /* Streptococcal sepsis, unspecified */
"A4101"  =  "1"  /* Sepsis due to Methicillin susceptible Staphylococcus aureus */
"A4102"  =  "1"  /* Sepsis due to Methicillin resistant Staphylococcus aureus */
"A411"  =  "1"  /* Sepsis due to other specified staphylococcus */
"A412"  =  "1"  /* Sepsis due to unspecified staphylococcus */
"A413"  =  "1"  /* Sepsis due to Hemophilus influenzae */
"A414"  =  "1"  /* Sepsis due to anaerobes */
"A4150"  =  "1"  /* Gram-negative sepsis, unspecified */
"A4151"  =  "1"  /* Sepsis due to Escherichia coli [E. coli] */
"A4152"  =  "1"  /* Sepsis due to Pseudomonas */
"A4153"  =  "1"  /* Sepsis due to Serratia */
"A4159"  =  "1"  /* Other Gram-negative sepsis */
"A4181"  =  "1"  /* Sepsis due to Enterococcus */
"A4189"  =  "1"  /* Other specified sepsis */
"A419"  =  "1"  /* Sepsis, unspecified organism */
"A420"  =  "1"  /* Pulmonary actinomycosis */
"A421"  =  "1"  /* Abdominal actinomycosis */
"A422"  =  "1"  /* Cervicofacial actinomycosis */
"A427"  =  "1"  /* Actinomycotic sepsis */
"A4281"  =  "1"  /* Actinomycotic meningitis */
"A4282"  =  "1"  /* Actinomycotic encephalitis */
"A4289"  =  "1"  /* Other forms of actinomycosis */
"A429"  =  "1"  /* Actinomycosis, unspecified */
"A430"  =  "1"  /* Pulmonary nocardiosis */
"A431"  =  "1"  /* Cutaneous nocardiosis */
"A438"  =  "1"  /* Other forms of nocardiosis */
"A439"  =  "1"  /* Nocardiosis, unspecified */
"A46"  =  "1"  /* Erysipelas */
"A480"  =  "1"  /* Gas gangrene */
"A481"  =  "1"  /* Legionnaires' disease */
"A482"  =  "1"  /* Nonpneumonic Legionnaires' disease [Pontiac fever] */
"A483"  =  "1"  /* Toxic shock syndrome */
"A484"  =  "1"  /* Brazilian purpuric fever */
"A4851"  =  "1"  /* Infant botulism */
"A4852"  =  "1"  /* Wound botulism */
"A488"  =  "1"  /* Other specified bacterial diseases */
"A4901"  =  "1"  /* Methicillin susceptible Staphylococcus aureus infection, unspecified site */
"A4902"  =  "1"  /* Methicillin resistant Staphylococcus aureus infection, unspecified site */
"A491"  =  "1"  /* Streptococcal infection, unspecified site */
"A492"  =  "1"  /* Hemophilus influenzae infection, unspecified site */
"A498"  =  "1"  /* Other bacterial infections of unspecified site */
"A499"  =  "1"  /* Bacterial infection, unspecified */
"A5400"  =  "1"  /* Gonococcal infection of lower genitourinary tract, unspecified */
"A5401"  =  "1"  /* Gonococcal cystitis and urethritis, unspecified */
"A5402"  =  "1"  /* Gonococcal vulvovaginitis, unspecified */
"A5403"  =  "1"  /* Gonococcal cervicitis, unspecified */
"A5409"  =  "1"  /* Other gonococcal infection of lower genitourinary tract */
"A541"  =  "1"  /* Gonococcal infection of lower genitourinary tract with periurethral and accessory gland abscess */
"A5421"  =  "1"  /* Gonococcal infection of kidney and ureter */
"A5422"  =  "1"  /* Gonococcal prostatitis */
"A5423"  =  "1"  /* Gonococcal infection of other male genital organs */
"A5424"  =  "1"  /* Gonococcal female pelvic inflammatory disease */
"A5429"  =  "1"  /* Other gonococcal genitourinary infections */
"A5430"  =  "1"  /* Gonococcal infection of eye, unspecified */
"A5431"  =  "1"  /* Gonococcal conjunctivitis */
"A5432"  =  "1"  /* Gonococcal iridocyclitis */
"A5433"  =  "1"  /* Gonococcal keratitis */
"A5439"  =  "1"  /* Other gonococcal eye infection */
"A5440"  =  "1"  /* Gonococcal infection of musculoskeletal system, unspecified */
"A5441"  =  "1"  /* Gonococcal spondylopathy */
"A5442"  =  "1"  /* Gonococcal arthritis */
"A5443"  =  "1"  /* Gonococcal osteomyelitis */
"A5449"  =  "1"  /* Gonococcal infection of other musculoskeletal tissue */
"A545"  =  "1"  /* Gonococcal pharyngitis */
"A546"  =  "1"  /* Gonococcal infection of anus and rectum */
"A5481"  =  "1"  /* Gonococcal meningitis */
"A5482"  =  "1"  /* Gonococcal brain abscess */
"A5483"  =  "1"  /* Gonococcal heart infection */
"A5484"  =  "1"  /* Gonococcal pneumonia */
"A5485"  =  "1"  /* Gonococcal peritonitis */
"A5486"  =  "1"  /* Gonococcal sepsis */
"A5489"  =  "1"  /* Other gonococcal infections */
"A549"  =  "1"  /* Gonococcal infection, unspecified */
"A5601"  =  "1"  /* Chlamydial cystitis and urethritis */
"A5611"  =  "1"  /* Chlamydial female pelvic inflammatory disease */
"B252"  =  "1"  /* Cytomegaloviral pancreatitis */
"B377"  =  "1"  /* Candidal sepsis */
"B471"  =  "1"  /* Actinomycetoma */
"B479"  =  "1"  /* Mycetoma, unspecified */
"B950"  =  "1"  /* Streptococcus, group A, as the cause of diseases classified elsewhere */
"B951"  =  "1"  /* Streptococcus, group B, as the cause of diseases classified elsewhere */
"B952"  =  "1"  /* Enterococcus as the cause of diseases classified elsewhere */
"B953"  =  "1"  /* Streptococcus pneumoniae as the cause of diseases classified elsewhere */
"B954"  =  "1"  /* Other streptococcus as the cause of diseases classified elsewhere */
"B955"  =  "1"  /* Unspecified streptococcus as the cause of diseases classified elsewhere */
"B9561"  =  "1"  /* Methicillin susceptible Staphylococcus aureus infection as the cause of diseases classified elsewhere */
"B9562"  =  "1"  /* Methicillin resistant Staphylococcus aureus infection as the cause of diseases classified elsewhere */
"B957"  =  "1"  /* Other staphylococcus as the cause of diseases classified elsewhere */
"B958"  =  "1"  /* Unspecified staphylococcus as the cause of diseases classified elsewhere */
"B961"  =  "1"  /* Klebsiella pneumoniae [K. pneumoniae] as the cause of diseases classified elsewhere */
"B9620"  =  "1"  /* Unspecified Escherichia coli [E. coli] as the cause of diseases classified elsewhere */
"B9621"  =  "1"  /* Shiga toxin-producing Escherichia coli [E. coli] (STEC) O157 as the cause of diseases classified elsewhere */
"B9622"  =  "1"  /* Other specified Shiga toxin-producing Escherichia coli [E. coli] (STEC) as the cause of diseases classified elsewhere */
"B9623"  =  "1"  /* Unspecified Shiga toxin-producing Escherichia coli [E. coli] (STEC) as the cause of diseases classified elsewhere */
"B9629"  =  "1"  /* Other Escherichia coli [E. coli] as the cause of diseases classified elsewhere */
"B963"  =  "1"  /* Hemophilus influenzae [H. influenzae] as the cause of diseases classified elsewhere */
"B964"  =  "1"  /* Proteus (mirabilis) (morganii) as the cause of diseases classified elsewhere */
"B965"  =  "1"  /* Pseudomonas (aeruginosa) (mallei) (pseudomallei) as the cause of diseases classified elsewhere */
"B966"  =  "1"  /* Bacteroides fragilis [B. fragilis] as the cause of diseases classified elsewhere */
"B967"  =  "1"  /* Clostridium perfringens [C. perfringens] as the cause of diseases classified elsewhere */
"B9681"  =  "1"  /* Helicobacter pylori [H. pylori] as the cause of diseases classified elsewhere */
"B9682"  =  "1"  /* Vibrio vulnificus as the cause of diseases classified elsewhere */
"B9689"  =  "1"  /* Other specified bacterial agents as the cause of diseases classified elsewhere */
"E0852"  =  "1"  /* Diabetes mellitus due to underlying condition with diabetic peripheral angiopathy with gangrene */
"E0952"  =  "1"  /* Drug or chemical induced diabetes mellitus with diabetic peripheral angiopathy with gangrene */
"E1052"  =  "1"  /* Type 1 diabetes mellitus with diabetic peripheral angiopathy with gangrene */
"E1152"  =  "1"  /* Type 2 diabetes mellitus with diabetic peripheral angiopathy with gangrene */
"E1352"  =  "1"  /* Other specified diabetes mellitus with diabetic peripheral angiopathy with gangrene */
"G000"  =  "1"  /* Hemophilus meningitis */
"G001"  =  "1"  /* Pneumococcal meningitis */
"G002"  =  "1"  /* Streptococcal meningitis */
"G003"  =  "1"  /* Staphylococcal meningitis */
"G008"  =  "1"  /* Other bacterial meningitis */
"G009"  =  "1"  /* Bacterial meningitis, unspecified */
"G01"  =  "1"  /* Meningitis in bacterial diseases classified elsewhere */
"G039"  =  "1"  /* Meningitis, unspecified */
"G042"  =  "1"  /* Bacterial meningoencephalitis and meningomyelitis, not elsewhere classified */
"G060"  =  "1"  /* Intracranial abscess and granuloma */
"G061"  =  "1"  /* Intraspinal abscess and granuloma */
"G062"  =  "1"  /* Extradural and subdural abscess, unspecified */
"G07"  =  "1"  /* Intracranial and intraspinal abscess and granuloma in diseases classified elsewhere */
"H00011"  =  "1"  /* Hordeolum externum right upper eyelid */
"H00012"  =  "1"  /* Hordeolum externum right lower eyelid */
"H00013"  =  "1"  /* Hordeolum externum right eye, unspecified eyelid */
"H00014"  =  "1"  /* Hordeolum externum left upper eyelid */
"H00015"  =  "1"  /* Hordeolum externum left lower eyelid */
"H00016"  =  "1"  /* Hordeolum externum left eye, unspecified eyelid */
"H00019"  =  "1"  /* Hordeolum externum unspecified eye, unspecified eyelid */
"H00021"  =  "1"  /* Hordeolum internum right upper eyelid */
"H00022"  =  "1"  /* Hordeolum internum right lower eyelid */
"H00023"  =  "1"  /* Hordeolum internum right eye, unspecified eyelid */
"H00024"  =  "1"  /* Hordeolum internum left upper eyelid */
"H00025"  =  "1"  /* Hordeolum internum left lower eyelid */
"H00026"  =  "1"  /* Hordeolum internum left eye, unspecified eyelid */
"H00029"  =  "1"  /* Hordeolum internum unspecified eye, unspecified eyelid */
"H00031"  =  "1"  /* Abscess of right upper eyelid */
"H00032"  =  "1"  /* Abscess of right lower eyelid */
"H00033"  =  "1"  /* Abscess of eyelid right eye, unspecified eyelid */
"H00034"  =  "1"  /* Abscess of left upper eyelid */
"H00035"  =  "1"  /* Abscess of left lower eyelid */
"H00036"  =  "1"  /* Abscess of eyelid left eye, unspecified eyelid */
"H00039"  =  "1"  /* Abscess of eyelid unspecified eye, unspecified eyelid */
"H01001"  =  "1"  /* Unspecified blepharitis right upper eyelid */
"H01002"  =  "1"  /* Unspecified blepharitis right lower eyelid */
"H01003"  =  "1"  /* Unspecified blepharitis right eye, unspecified eyelid */
"H01004"  =  "1"  /* Unspecified blepharitis left upper eyelid */
"H01005"  =  "1"  /* Unspecified blepharitis left lower eyelid */
"H01006"  =  "1"  /* Unspecified blepharitis left eye, unspecified eyelid */
"H01009"  =  "1"  /* Unspecified blepharitis unspecified eye, unspecified eyelid */
"H01011"  =  "1"  /* Ulcerative blepharitis right upper eyelid */
"H01012"  =  "1"  /* Ulcerative blepharitis right lower eyelid */
"H01013"  =  "1"  /* Ulcerative blepharitis right eye, unspecified eyelid */
"H01014"  =  "1"  /* Ulcerative blepharitis left upper eyelid */
"H01015"  =  "1"  /* Ulcerative blepharitis left lower eyelid */
"H01016"  =  "1"  /* Ulcerative blepharitis left eye, unspecified eyelid */
"H01019"  =  "1"  /* Ulcerative blepharitis unspecified eye, unspecified eyelid */
"H04001"  =  "1"  /* Unspecified dacryoadenitis, right lacrimal gland */
"H04002"  =  "1"  /* Unspecified dacryoadenitis, left lacrimal gland */
"H04003"  =  "1"  /* Unspecified dacryoadenitis, bilateral lacrimal glands */
"H04009"  =  "1"  /* Unspecified dacryoadenitis, unspecified lacrimal gland */
"H04011"  =  "1"  /* Acute dacryoadenitis, right lacrimal gland */
"H04012"  =  "1"  /* Acute dacryoadenitis, left lacrimal gland */
"H04013"  =  "1"  /* Acute dacryoadenitis, bilateral lacrimal glands */
"H04019"  =  "1"  /* Acute dacryoadenitis, unspecified lacrimal gland */
"H04301"  =  "1"  /* Unspecified dacryocystitis of right lacrimal passage */
"H04302"  =  "1"  /* Unspecified dacryocystitis of left lacrimal passage */
"H04303"  =  "1"  /* Unspecified dacryocystitis of bilateral lacrimal passages */
"H04309"  =  "1"  /* Unspecified dacryocystitis of unspecified lacrimal passage */
"H04321"  =  "1"  /* Acute dacryocystitis of right lacrimal passage */
"H04322"  =  "1"  /* Acute dacryocystitis of left lacrimal passage */
"H04323"  =  "1"  /* Acute dacryocystitis of bilateral lacrimal passages */
"H04329"  =  "1"  /* Acute dacryocystitis of unspecified lacrimal passage */
"H04331"  =  "1"  /* Acute lacrimal canaliculitis of right lacrimal passage */
"H04332"  =  "1"  /* Acute lacrimal canaliculitis of left lacrimal passage */
"H04333"  =  "1"  /* Acute lacrimal canaliculitis of bilateral lacrimal passages */
"H04339"  =  "1"  /* Acute lacrimal canaliculitis of unspecified lacrimal passage */
"H05011"  =  "1"  /* Cellulitis of right orbit */
"H05012"  =  "1"  /* Cellulitis of left orbit */
"H05013"  =  "1"  /* Cellulitis of bilateral orbits */
"H05019"  =  "1"  /* Cellulitis of unspecified orbit */
"H05021"  =  "1"  /* Osteomyelitis of right orbit */
"H05022"  =  "1"  /* Osteomyelitis of left orbit */
"H05023"  =  "1"  /* Osteomyelitis of bilateral orbits */
"H05029"  =  "1"  /* Osteomyelitis of unspecified orbit */
"H05031"  =  "1"  /* Periostitis of right orbit */
"H05032"  =  "1"  /* Periostitis of left orbit */
"H05033"  =  "1"  /* Periostitis of bilateral orbits */
"H05039"  =  "1"  /* Periostitis of unspecified orbit */
"H05041"  =  "1"  /* Tenonitis of right orbit */
"H05042"  =  "1"  /* Tenonitis of left orbit */
"H05043"  =  "1"  /* Tenonitis of bilateral orbits */
"H05049"  =  "1"  /* Tenonitis of unspecified orbit */
"H10021"  =  "1"  /* Other mucopurulent conjunctivitis, right eye */
"H10022"  =  "1"  /* Other mucopurulent conjunctivitis, left eye */
"H10023"  =  "1"  /* Other mucopurulent conjunctivitis, bilateral */
"H10029"  =  "1"  /* Other mucopurulent conjunctivitis, unspecified eye */
"H10221"  =  "1"  /* Pseudomembranous conjunctivitis, right eye */
"H10222"  =  "1"  /* Pseudomembranous conjunctivitis, left eye */
"H10223"  =  "1"  /* Pseudomembranous conjunctivitis, bilateral */
"H10229"  =  "1"  /* Pseudomembranous conjunctivitis, unspecified eye */
"H1030"  =  "1"  /* Unspecified acute conjunctivitis, unspecified eye */
"H1031"  =  "1"  /* Unspecified acute conjunctivitis, right eye */
"H1032"  =  "1"  /* Unspecified acute conjunctivitis, left eye */
"H1033"  =  "1"  /* Unspecified acute conjunctivitis, bilateral */
"H10501"  =  "1"  /* Unspecified blepharoconjunctivitis, right eye */
"H10502"  =  "1"  /* Unspecified blepharoconjunctivitis, left eye */
"H10503"  =  "1"  /* Unspecified blepharoconjunctivitis, bilateral */
"H10509"  =  "1"  /* Unspecified blepharoconjunctivitis, unspecified eye */
"H10521"  =  "1"  /* Angular blepharoconjunctivitis, right eye */
"H10522"  =  "1"  /* Angular blepharoconjunctivitis, left eye */
"H10523"  =  "1"  /* Angular blepharoconjunctivitis, bilateral */
"H10529"  =  "1"  /* Angular blepharoconjunctivitis, unspecified eye */
"H109"  =  "1"  /* Unspecified conjunctivitis */
"H16311"  =  "1"  /* Corneal abscess, right eye */
"H16312"  =  "1"  /* Corneal abscess, left eye */
"H16313"  =  "1"  /* Corneal abscess, bilateral */
"H16319"  =  "1"  /* Corneal abscess, unspecified eye */
"H44001"  =  "1"  /* Unspecified purulent endophthalmitis, right eye */
"H44002"  =  "1"  /* Unspecified purulent endophthalmitis, left eye */
"H44003"  =  "1"  /* Unspecified purulent endophthalmitis, bilateral */
"H44009"  =  "1"  /* Unspecified purulent endophthalmitis, unspecified eye */
"H44011"  =  "1"  /* Panophthalmitis (acute), right eye */
"H44012"  =  "1"  /* Panophthalmitis (acute), left eye */
"H44013"  =  "1"  /* Panophthalmitis (acute), bilateral */
"H44019"  =  "1"  /* Panophthalmitis (acute), unspecified eye */
"H44021"  =  "1"  /* Vitreous abscess (chronic), right eye */
"H44022"  =  "1"  /* Vitreous abscess (chronic), left eye */
"H44023"  =  "1"  /* Vitreous abscess (chronic), bilateral */
"H44029"  =  "1"  /* Vitreous abscess (chronic), unspecified eye */
"H6000"  =  "1"  /* Abscess of external ear, unspecified ear */
"H6001"  =  "1"  /* Abscess of right external ear */
"H6002"  =  "1"  /* Abscess of left external ear */
"H6003"  =  "1"  /* Abscess of external ear, bilateral */
"H6010"  =  "1"  /* Cellulitis of external ear, unspecified ear */
"H6011"  =  "1"  /* Cellulitis of right external ear */
"H6012"  =  "1"  /* Cellulitis of left external ear */
"H6013"  =  "1"  /* Cellulitis of external ear, bilateral */
"H6020"  =  "1"  /* Malignant otitis externa, unspecified ear */
"H6021"  =  "1"  /* Malignant otitis externa, right ear */
"H6022"  =  "1"  /* Malignant otitis externa, left ear */
"H6023"  =  "1"  /* Malignant otitis externa, bilateral */
"H60311"  =  "1"  /* Diffuse otitis externa, right ear */
"H60312"  =  "1"  /* Diffuse otitis externa, left ear */
"H60313"  =  "1"  /* Diffuse otitis externa, bilateral */
"H60319"  =  "1"  /* Diffuse otitis externa, unspecified ear */
"H60321"  =  "1"  /* Hemorrhagic otitis externa, right ear */
"H60322"  =  "1"  /* Hemorrhagic otitis externa, left ear */
"H60323"  =  "1"  /* Hemorrhagic otitis externa, bilateral */
"H60329"  =  "1"  /* Hemorrhagic otitis externa, unspecified ear */
"H60331"  =  "1"  /* Swimmer's ear, right ear */
"H60332"  =  "1"  /* Swimmer's ear, left ear */
"H60333"  =  "1"  /* Swimmer's ear, bilateral */
"H60339"  =  "1"  /* Swimmer's ear, unspecified ear */
"H60391"  =  "1"  /* Other infective otitis externa, right ear */
"H60392"  =  "1"  /* Other infective otitis externa, left ear */
"H60393"  =  "1"  /* Other infective otitis externa, bilateral */
"H60399"  =  "1"  /* Other infective otitis externa, unspecified ear */
"H6240"  =  "1"  /* Otitis externa in other diseases classified elsewhere, unspecified ear */
"H6241"  =  "1"  /* Otitis externa in other diseases classified elsewhere, right ear */
"H6242"  =  "1"  /* Otitis externa in other diseases classified elsewhere, left ear */
"H6243"  =  "1"  /* Otitis externa in other diseases classified elsewhere, bilateral */
"H66001"  =  "1"  /* Acute suppurative otitis media without spontaneous rupture of ear drum, right ear */
"H66002"  =  "1"  /* Acute suppurative otitis media without spontaneous rupture of ear drum, left ear */
"H66003"  =  "1"  /* Acute suppurative otitis media without spontaneous rupture of ear drum, bilateral */
"H66004"  =  "1"  /* Acute suppurative otitis media without spontaneous rupture of ear drum, recurrent, right ear */
"H66005"  =  "1"  /* Acute suppurative otitis media without spontaneous rupture of ear drum, recurrent, left ear */
"H66006"  =  "1"  /* Acute suppurative otitis media without spontaneous rupture of ear drum, recurrent, bilateral */
"H66007"  =  "1"  /* Acute suppurative otitis media without spontaneous rupture of ear drum, recurrent, unspecified ear */
"H66009"  =  "1"  /* Acute suppurative otitis media without spontaneous rupture of ear drum, unspecified ear */
"H66011"  =  "1"  /* Acute suppurative otitis media with spontaneous rupture of ear drum, right ear */
"H66012"  =  "1"  /* Acute suppurative otitis media with spontaneous rupture of ear drum, left ear */
"H66013"  =  "1"  /* Acute suppurative otitis media with spontaneous rupture of ear drum, bilateral */
"H66014"  =  "1"  /* Acute suppurative otitis media with spontaneous rupture of ear drum, recurrent, right ear */
"H66015"  =  "1"  /* Acute suppurative otitis media with spontaneous rupture of ear drum, recurrent, left ear */
"H66016"  =  "1"  /* Acute suppurative otitis media with spontaneous rupture of ear drum, recurrent, bilateral */
"H66017"  =  "1"  /* Acute suppurative otitis media with spontaneous rupture of ear drum, recurrent, unspecified ear */
"H66019"  =  "1"  /* Acute suppurative otitis media with spontaneous rupture of ear drum, unspecified ear */
"H6610"  =  "1"  /* Chronic tubotympanic suppurative otitis media, unspecified */
"H6611"  =  "1"  /* Chronic tubotympanic suppurative otitis media, right ear */
"H6612"  =  "1"  /* Chronic tubotympanic suppurative otitis media, left ear */
"H6613"  =  "1"  /* Chronic tubotympanic suppurative otitis media, bilateral */
"H6620"  =  "1"  /* Chronic atticoantral suppurative otitis media, unspecified ear */
"H6621"  =  "1"  /* Chronic atticoantral suppurative otitis media, right ear */
"H6622"  =  "1"  /* Chronic atticoantral suppurative otitis media, left ear */
"H6623"  =  "1"  /* Chronic atticoantral suppurative otitis media, bilateral */
"H663X1"  =  "1"  /* Other chronic suppurative otitis media, right ear */
"H663X2"  =  "1"  /* Other chronic suppurative otitis media, left ear */
"H663X3"  =  "1"  /* Other chronic suppurative otitis media, bilateral */
"H663X9"  =  "1"  /* Other chronic suppurative otitis media, unspecified ear */
"H6640"  =  "1"  /* Suppurative otitis media, unspecified, unspecified ear */
"H6641"  =  "1"  /* Suppurative otitis media, unspecified, right ear */
"H6642"  =  "1"  /* Suppurative otitis media, unspecified, left ear */
"H6643"  =  "1"  /* Suppurative otitis media, unspecified, bilateral */
"H6690"  =  "1"  /* Otitis media, unspecified, unspecified ear */
"H6691"  =  "1"  /* Otitis media, unspecified, right ear */
"H6692"  =  "1"  /* Otitis media, unspecified, left ear */
"H6693"  =  "1"  /* Otitis media, unspecified, bilateral */
"H671"  =  "1"  /* Otitis media in diseases classified elsewhere, right ear */
"H672"  =  "1"  /* Otitis media in diseases classified elsewhere, left ear */
"H673"  =  "1"  /* Otitis media in diseases classified elsewhere, bilateral */
"H679"  =  "1"  /* Otitis media in diseases classified elsewhere, unspecified ear */
"H68001"  =  "1"  /* Unspecified Eustachian salpingitis, right ear */
"H68002"  =  "1"  /* Unspecified Eustachian salpingitis, left ear */
"H68003"  =  "1"  /* Unspecified Eustachian salpingitis, bilateral */
"H68009"  =  "1"  /* Unspecified Eustachian salpingitis, unspecified ear */
"H68011"  =  "1"  /* Acute Eustachian salpingitis, right ear */
"H68012"  =  "1"  /* Acute Eustachian salpingitis, left ear */
"H68013"  =  "1"  /* Acute Eustachian salpingitis, bilateral */
"H68019"  =  "1"  /* Acute Eustachian salpingitis, unspecified ear */
"H70001"  =  "1"  /* Acute mastoiditis without complications, right ear */
"H70002"  =  "1"  /* Acute mastoiditis without complications, left ear */
"H70003"  =  "1"  /* Acute mastoiditis without complications, bilateral */
"H70009"  =  "1"  /* Acute mastoiditis without complications, unspecified ear */
"H70011"  =  "1"  /* Subperiosteal abscess of mastoid, right ear */
"H70012"  =  "1"  /* Subperiosteal abscess of mastoid, left ear */
"H70013"  =  "1"  /* Subperiosteal abscess of mastoid, bilateral */
"H70019"  =  "1"  /* Subperiosteal abscess of mastoid, unspecified ear */
"H70091"  =  "1"  /* Acute mastoiditis with other complications, right ear */
"H70092"  =  "1"  /* Acute mastoiditis with other complications, left ear */
"H70093"  =  "1"  /* Acute mastoiditis with other complications, bilateral */
"H70099"  =  "1"  /* Acute mastoiditis with other complications, unspecified ear */
"H70201"  =  "1"  /* Unspecified petrositis, right ear */
"H70202"  =  "1"  /* Unspecified petrositis, left ear */
"H70203"  =  "1"  /* Unspecified petrositis, bilateral */
"H70209"  =  "1"  /* Unspecified petrositis, unspecified ear */
"H70211"  =  "1"  /* Acute petrositis, right ear */
"H70212"  =  "1"  /* Acute petrositis, left ear */
"H70213"  =  "1"  /* Acute petrositis, bilateral */
"H70219"  =  "1"  /* Acute petrositis, unspecified ear */
"H73001"  =  "1"  /* Acute myringitis, right ear */
"H73002"  =  "1"  /* Acute myringitis, left ear */
"H73003"  =  "1"  /* Acute myringitis, bilateral */
"H73009"  =  "1"  /* Acute myringitis, unspecified ear */
"I301"  =  "1"  /* Infective pericarditis */
"I308"  =  "1"  /* Other forms of acute pericarditis */
"I309"  =  "1"  /* Acute pericarditis, unspecified */
"I32"  =  "1"  /* Pericarditis in diseases classified elsewhere */
"I330"  =  "1"  /* Acute and subacute infective endocarditis */
"I339"  =  "1"  /* Acute and subacute endocarditis, unspecified */
"I39"  =  "1"  /* Endocarditis and heart valve disorders in diseases classified elsewhere */
"I400"  =  "1"  /* Infective myocarditis */
"I70361"  =  "1"  /* Atherosclerosis of unspecified type of bypass graft(s) of the extremities with gangrene, right leg */
"I70362"  =  "1"  /* Atherosclerosis of unspecified type of bypass graft(s) of the extremities with gangrene, left leg */
"I70363"  =  "1"  /* Atherosclerosis of unspecified type of bypass graft(s) of the extremities with gangrene, bilateral legs */
"I70368"  =  "1"  /* Atherosclerosis of unspecified type of bypass graft(s) of the extremities with gangrene, other extremity */
"I70369"  =  "1"  /* Atherosclerosis of unspecified type of bypass graft(s) of the extremities with gangrene, unspecified extremity */
"I70461"  =  "1"  /* Atherosclerosis of autologous vein bypass graft(s) of the extremities with gangrene, right leg */
"I70462"  =  "1"  /* Atherosclerosis of autologous vein bypass graft(s) of the extremities with gangrene, left leg */
"I70463"  =  "1"  /* Atherosclerosis of autologous vein bypass graft(s) of the extremities with gangrene, bilateral legs */
"I70468"  =  "1"  /* Atherosclerosis of autologous vein bypass graft(s) of the extremities with gangrene, other extremity */
"I70469"  =  "1"  /* Atherosclerosis of autologous vein bypass graft(s) of the extremities with gangrene, unspecified extremity */
"I70561"  =  "1"  /* Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with gangrene, right leg */
"I70562"  =  "1"  /* Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with gangrene, left leg */
"I70563"  =  "1"  /* Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with gangrene, bilateral legs */
"I70568"  =  "1"  /* Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with gangrene, other extremity */
"I70569"  =  "1"  /* Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with gangrene, unspecified extremity */
"I70661"  =  "1"  /* Atherosclerosis of nonbiological bypass graft(s) of the extremities with gangrene, right leg */
"I70662"  =  "1"  /* Atherosclerosis of nonbiological bypass graft(s) of the extremities with gangrene, left leg */
"I70663"  =  "1"  /* Atherosclerosis of nonbiological bypass graft(s) of the extremities with gangrene, bilateral legs */
"I70668"  =  "1"  /* Atherosclerosis of nonbiological bypass graft(s) of the extremities with gangrene, other extremity */
"I70669"  =  "1"  /* Atherosclerosis of nonbiological bypass graft(s) of the extremities with gangrene, unspecified extremity */
"I70761"  =  "1"  /* Atherosclerosis of other type of bypass graft(s) of the extremities with gangrene, right leg */
"I70762"  =  "1"  /* Atherosclerosis of other type of bypass graft(s) of the extremities with gangrene, left leg */
"I70763"  =  "1"  /* Atherosclerosis of other type of bypass graft(s) of the extremities with gangrene, bilateral legs */
"I70768"  =  "1"  /* Atherosclerosis of other type of bypass graft(s) of the extremities with gangrene, other extremity */
"I70769"  =  "1"  /* Atherosclerosis of other type of bypass graft(s) of the extremities with gangrene, unspecified extremity */
"I7301"  =  "1"  /* Raynaud's syndrome with gangrene */
"I96"  =  "1"  /* Gangrene, not elsewhere classified */
"J0100"  =  "1"  /* Acute maxillary sinusitis, unspecified */
"J0101"  =  "1"  /* Acute recurrent maxillary sinusitis */
"J0110"  =  "1"  /* Acute frontal sinusitis, unspecified */
"J0111"  =  "1"  /* Acute recurrent frontal sinusitis */
"J0120"  =  "1"  /* Acute ethmoidal sinusitis, unspecified */
"J0121"  =  "1"  /* Acute recurrent ethmoidal sinusitis */
"J0130"  =  "1"  /* Acute sphenoidal sinusitis, unspecified */
"J0131"  =  "1"  /* Acute recurrent sphenoidal sinusitis */
"J0140"  =  "1"  /* Acute pansinusitis, unspecified */
"J0141"  =  "1"  /* Acute recurrent pansinusitis */
"J0180"  =  "1"  /* Other acute sinusitis */
"J0181"  =  "1"  /* Other acute recurrent sinusitis */
"J0190"  =  "1"  /* Acute sinusitis, unspecified */
"J0191"  =  "1"  /* Acute recurrent sinusitis, unspecified */
"J020"  =  "1"  /* Streptococcal pharyngitis */
"J028"  =  "1"  /* Acute pharyngitis due to other specified organisms */
"J029"  =  "1"  /* Acute pharyngitis, unspecified */
"J0300"  =  "1"  /* Acute streptococcal tonsillitis, unspecified */
"J0301"  =  "1"  /* Acute recurrent streptococcal tonsillitis */
"J0380"  =  "1"  /* Acute tonsillitis due to other specified organisms */
"J0381"  =  "1"  /* Acute recurrent tonsillitis due to other specified organisms */
"J0390"  =  "1"  /* Acute tonsillitis, unspecified */
"J0391"  =  "1"  /* Acute recurrent tonsillitis, unspecified */
"J0510"  =  "1"  /* Acute epiglottitis without obstruction */
"J0511"  =  "1"  /* Acute epiglottitis with obstruction */
"J13"  =  "1"  /* Pneumonia due to Streptococcus pneumoniae */
"J14"  =  "1"  /* Pneumonia due to Hemophilus influenzae */
"J150"  =  "1"  /* Pneumonia due to Klebsiella pneumoniae */
"J151"  =  "1"  /* Pneumonia due to Pseudomonas */
"J1520"  =  "1"  /* Pneumonia due to staphylococcus, unspecified */
"J15211"  =  "1"  /* Pneumonia due to Methicillin susceptible Staphylococcus aureus */
"J15212"  =  "1"  /* Pneumonia due to Methicillin resistant Staphylococcus aureus */
"J1529"  =  "1"  /* Pneumonia due to other staphylococcus */
"J153"  =  "1"  /* Pneumonia due to streptococcus, group B */
"J154"  =  "1"  /* Pneumonia due to other streptococci */
"J155"  =  "1"  /* Pneumonia due to Escherichia coli */
"J156"  =  "1"  /* Pneumonia due to other aerobic Gram-negative bacteria */
"J158"  =  "1"  /* Pneumonia due to other specified bacteria */
"J159"  =  "1"  /* Unspecified bacterial pneumonia */
"J17"  =  "1"  /* Pneumonia in diseases classified elsewhere */
"J180"  =  "1"  /* Bronchopneumonia, unspecified organism */
"J181"  =  "1"  /* Lobar pneumonia, unspecified organism */
"J188"  =  "1"  /* Other pneumonia, unspecified organism */
"J189"  =  "1"  /* Pneumonia, unspecified organism */
"J200"  =  "1"  /* Acute bronchitis due to Mycoplasma pneumoniae */
"J201"  =  "1"  /* Acute bronchitis due to Hemophilus influenzae */
"J202"  =  "1"  /* Acute bronchitis due to streptococcus */
"J203"  =  "1"  /* Acute bronchitis due to coxsackievirus */
"J204"  =  "1"  /* Acute bronchitis due to parainfluenza virus */
"J205"  =  "1"  /* Acute bronchitis due to respiratory syncytial virus */
"J206"  =  "1"  /* Acute bronchitis due to rhinovirus */
"J207"  =  "1"  /* Acute bronchitis due to echovirus */
"J208"  =  "1"  /* Acute bronchitis due to other specified organisms */
"J209"  =  "1"  /* Acute bronchitis, unspecified */
"J36"  =  "1"  /* Peritonsillar abscess */
"J390"  =  "1"  /* Retropharyngeal and parapharyngeal abscess */
"J40"  =  "1"  /* Bronchitis, not specified as acute or chronic */
"J440"  =  "1"  /* Chronic obstructive pulmonary disease with acute lower respiratory infection */
"J470"  =  "1"  /* Bronchiectasis with acute lower respiratory infection */
"J471"  =  "1"  /* Bronchiectasis with (acute) exacerbation */
"J850"  =  "1"  /* Gangrene and necrosis of lung */
"J851"  =  "1"  /* Abscess of lung with pneumonia */
"J852"  =  "1"  /* Abscess of lung without pneumonia */
"J853"  =  "1"  /* Abscess of mediastinum */
"J860"  =  "1"  /* Pyothorax with fistula */
"J869"  =  "1"  /* Pyothorax without fistula */
"J9502"  =  "1"  /* Infection of tracheostomy stoma */
"K040"  =  "1"  /* Pulpitis */
"K046"  =  "1"  /* Periapical abscess with sinus */
"K047"  =  "1"  /* Periapical abscess without sinus */
"K0500"  =  "1"  /* Acute gingivitis, plaque induced */
"K0501"  =  "1"  /* Acute gingivitis, non-plaque induced */
"K0520"  =  "1"  /* Aggressive periodontitis, unspecified */
"K0521"  =  "1"  /* Aggressive periodontitis, localized */
"K0522"  =  "1"  /* Aggressive periodontitis, generalized */
"K113"  =  "1"  /* Abscess of salivary gland */
"K122"  =  "1"  /* Cellulitis and abscess of mouth */
"K352"  =  "1"  /* Acute appendicitis with generalized peritonitis */
"K353"  =  "1"  /* Acute appendicitis with localized peritonitis */
"K3580"  =  "1"  /* Unspecified acute appendicitis */
"K3589"  =  "1"  /* Other acute appendicitis */
"K36"  =  "1"  /* Other appendicitis */
"K37"  =  "1"  /* Unspecified appendicitis */
"K50014"  =  "1"  /* Crohn's disease of small intestine with abscess */
"K50114"  =  "1"  /* Crohn's disease of large intestine with abscess */
"K50814"  =  "1"  /* Crohn's disease of both small and large intestine with abscess */
"K50914"  =  "1"  /* Crohn's disease, unspecified, with abscess */
"K51014"  =  "1"  /* Ulcerative (chronic) pancolitis with abscess */
"K51214"  =  "1"  /* Ulcerative (chronic) proctitis with abscess */
"K51314"  =  "1"  /* Ulcerative (chronic) rectosigmoiditis with abscess */
"K51414"  =  "1"  /* Inflammatory polyps of colon with abscess */
"K51514"  =  "1"  /* Left sided colitis with abscess */
"K51814"  =  "1"  /* Other ulcerative colitis with abscess */
"K51914"  =  "1"  /* Ulcerative colitis, unspecified with abscess */
"K5700"  =  "1"  /* Diverticulitis of small intestine with perforation and abscess without bleeding */
"K5701"  =  "1"  /* Diverticulitis of small intestine with perforation and abscess with bleeding */
"K5712"  =  "1"  /* Diverticulitis of small intestine without perforation or abscess without bleeding */
"K5713"  =  "1"  /* Diverticulitis of small intestine without perforation or abscess with bleeding */
"K5720"  =  "1"  /* Diverticulitis of large intestine with perforation and abscess without bleeding */
"K5721"  =  "1"  /* Diverticulitis of large intestine with perforation and abscess with bleeding */
"K5732"  =  "1"  /* Diverticulitis of large intestine without perforation or abscess without bleeding */
"K5733"  =  "1"  /* Diverticulitis of large intestine without perforation or abscess with bleeding */
"K5740"  =  "1"  /* Diverticulitis of both small and large intestine with perforation and abscess without bleeding */
"K5741"  =  "1"  /* Diverticulitis of both small and large intestine with perforation and abscess with bleeding */
"K5752"  =  "1"  /* Diverticulitis of both small and large intestine without perforation or abscess without bleeding */
"K5753"  =  "1"  /* Diverticulitis of both small and large intestine without perforation or abscess with bleeding */
"K5780"  =  "1"  /* Diverticulitis of intestine, part unspecified, with perforation and abscess without bleeding */
"K5781"  =  "1"  /* Diverticulitis of intestine, part unspecified, with perforation and abscess with bleeding */
"K5792"  =  "1"  /* Diverticulitis of intestine, part unspecified, without perforation or abscess without bleeding */
"K5793"  =  "1"  /* Diverticulitis of intestine, part unspecified, without perforation or abscess with bleeding */
"K610"  =  "1"  /* Anal abscess */
"K611"  =  "1"  /* Rectal abscess */
"K612"  =  "1"  /* Anorectal abscess */
"K613"  =  "1"  /* Ischiorectal abscess */
"K614"  =  "1"  /* Intrasphincteric abscess */
"K630"  =  "1"  /* Abscess of intestine */
"K650"  =  "1"  /* Generalized (acute) peritonitis */
"K651"  =  "1"  /* Peritoneal abscess */
"K652"  =  "1"  /* Spontaneous bacterial peritonitis */
"K653"  =  "1"  /* Choleperitonitis */
"K654"  =  "1"  /* Sclerosing mesenteritis */
"K658"  =  "1"  /* Other peritonitis */
"K659"  =  "1"  /* Peritonitis, unspecified */
"K67"  =  "1"  /* Disorders of peritoneum in infectious diseases classified elsewhere */
"K6811"  =  "1"  /* Postprocedural retroperitoneal abscess */
"K6812"  =  "1"  /* Psoas muscle abscess */
"K6819"  =  "1"  /* Other retroperitoneal abscess */
"K750"  =  "1"  /* Abscess of liver */
"K751"  =  "1"  /* Phlebitis of portal vein */
"K8000"  =  "1"  /* Calculus of gallbladder with acute cholecystitis without obstruction */
"K8001"  =  "1"  /* Calculus of gallbladder with acute cholecystitis with obstruction */
"K8012"  =  "1"  /* Calculus of gallbladder with acute and chronic cholecystitis without obstruction */
"K8013"  =  "1"  /* Calculus of gallbladder with acute and chronic cholecystitis with obstruction */
"K8030"  =  "1"  /* Calculus of bile duct with cholangitis, unspecified, without obstruction */
"K8031"  =  "1"  /* Calculus of bile duct with cholangitis, unspecified, with obstruction */
"K8032"  =  "1"  /* Calculus of bile duct with acute cholangitis without obstruction */
"K8033"  =  "1"  /* Calculus of bile duct with acute cholangitis with obstruction */
"K8034"  =  "1"  /* Calculus of bile duct with chronic cholangitis without obstruction */
"K8035"  =  "1"  /* Calculus of bile duct with chronic cholangitis with obstruction */
"K8036"  =  "1"  /* Calculus of bile duct with acute and chronic cholangitis without obstruction */
"K8037"  =  "1"  /* Calculus of bile duct with acute and chronic cholangitis with obstruction */
"K8042"  =  "1"  /* Calculus of bile duct with acute cholecystitis without obstruction */
"K8043"  =  "1"  /* Calculus of bile duct with acute cholecystitis with obstruction */
"K8046"  =  "1"  /* Calculus of bile duct with acute and chronic cholecystitis without obstruction */
"K8047"  =  "1"  /* Calculus of bile duct with acute and chronic cholecystitis with obstruction */
"K8062"  =  "1"  /* Calculus of gallbladder and bile duct with acute cholecystitis without obstruction */
"K8063"  =  "1"  /* Calculus of gallbladder and bile duct with acute cholecystitis with obstruction */
"K8066"  =  "1"  /* Calculus of gallbladder and bile duct with acute and chronic cholecystitis without obstruction */
"K8067"  =  "1"  /* Calculus of gallbladder and bile duct with acute and chronic cholecystitis with obstruction */
"K810"  =  "1"  /* Acute cholecystitis */
"K812"  =  "1"  /* Acute cholecystitis with chronic cholecystitis */
"K819"  =  "1"  /* Cholecystitis, unspecified */
"K822"  =  "1"  /* Perforation of gallbladder */
"K830"  =  "1"  /* Cholangitis */
"K832"  =  "1"  /* Perforation of bile duct */
"K850"  =  "1"  /* Idiopathic acute pancreatitis */
"K851"  =  "1"  /* Biliary acute pancreatitis */
"K858"  =  "1"  /* Other acute pancreatitis */
"K859"  =  "1"  /* Acute pancreatitis, unspecified */
"K9081"  =  "1"  /* Whipple's disease */
"K9402"  =  "1"  /* Colostomy infection */
"K9412"  =  "1"  /* Enterostomy infection */
"K9422"  =  "1"  /* Gastrostomy infection */
"K9501"  =  "1"  /* Infection due to gastric band procedure */
"K9581"  =  "1"  /* Infection due to other bariatric procedure */
"L00"  =  "1"  /* Staphylococcal scalded skin syndrome */
"L0100"  =  "1"  /* Impetigo, unspecified */
"L0101"  =  "1"  /* Non-bullous impetigo */
"L0102"  =  "1"  /* Bockhart's impetigo */
"L0103"  =  "1"  /* Bullous impetigo */
"L0109"  =  "1"  /* Other impetigo */
"L011"  =  "1"  /* Impetiginization of other dermatoses */
"L0201"  =  "1"  /* Cutaneous abscess of face */
"L0202"  =  "1"  /* Furuncle of face */
"L0203"  =  "1"  /* Carbuncle of face */
"L0211"  =  "1"  /* Cutaneous abscess of neck */
"L0212"  =  "1"  /* Furuncle of neck */
"L0213"  =  "1"  /* Carbuncle of neck */
"L02211"  =  "1"  /* Cutaneous abscess of abdominal wall */
"L02212"  =  "1"  /* Cutaneous abscess of back [any part, except buttock] */
"L02213"  =  "1"  /* Cutaneous abscess of chest wall */
"L02214"  =  "1"  /* Cutaneous abscess of groin */
"L02215"  =  "1"  /* Cutaneous abscess of perineum */
"L02216"  =  "1"  /* Cutaneous abscess of umbilicus */
"L02219"  =  "1"  /* Cutaneous abscess of trunk, unspecified */
"L02221"  =  "1"  /* Furuncle of abdominal wall */
"L02222"  =  "1"  /* Furuncle of back [any part, except buttock] */
"L02223"  =  "1"  /* Furuncle of chest wall */
"L02224"  =  "1"  /* Furuncle of groin */
"L02225"  =  "1"  /* Furuncle of perineum */
"L02226"  =  "1"  /* Furuncle of umbilicus */
"L02229"  =  "1"  /* Furuncle of trunk, unspecified */
"L02231"  =  "1"  /* Carbuncle of abdominal wall */
"L02232"  =  "1"  /* Carbuncle of back [any part, except buttock] */
"L02233"  =  "1"  /* Carbuncle of chest wall */
"L02234"  =  "1"  /* Carbuncle of groin */
"L02235"  =  "1"  /* Carbuncle of perineum */
"L02236"  =  "1"  /* Carbuncle of umbilicus */
"L02239"  =  "1"  /* Carbuncle of trunk, unspecified */
"L0231"  =  "1"  /* Cutaneous abscess of buttock */
"L0232"  =  "1"  /* Furuncle of buttock */
"L0233"  =  "1"  /* Carbuncle of buttock */
"L02411"  =  "1"  /* Cutaneous abscess of right axilla */
"L02412"  =  "1"  /* Cutaneous abscess of left axilla */
"L02413"  =  "1"  /* Cutaneous abscess of right upper limb */
"L02414"  =  "1"  /* Cutaneous abscess of left upper limb */
"L02415"  =  "1"  /* Cutaneous abscess of right lower limb */
"L02416"  =  "1"  /* Cutaneous abscess of left lower limb */
"L02419"  =  "1"  /* Cutaneous abscess of limb, unspecified */
"L02421"  =  "1"  /* Furuncle of right axilla */
"L02422"  =  "1"  /* Furuncle of left axilla */
"L02423"  =  "1"  /* Furuncle of right upper limb */
"L02424"  =  "1"  /* Furuncle of left upper limb */
"L02425"  =  "1"  /* Furuncle of right lower limb */
"L02426"  =  "1"  /* Furuncle of left lower limb */
"L02429"  =  "1"  /* Furuncle of limb, unspecified */
"L02431"  =  "1"  /* Carbuncle of right axilla */
"L02432"  =  "1"  /* Carbuncle of left axilla */
"L02433"  =  "1"  /* Carbuncle of right upper limb */
"L02434"  =  "1"  /* Carbuncle of left upper limb */
"L02435"  =  "1"  /* Carbuncle of right lower limb */
"L02436"  =  "1"  /* Carbuncle of left lower limb */
"L02439"  =  "1"  /* Carbuncle of limb, unspecified */
"L02511"  =  "1"  /* Cutaneous abscess of right hand */
"L02512"  =  "1"  /* Cutaneous abscess of left hand */
"L02519"  =  "1"  /* Cutaneous abscess of unspecified hand */
"L02521"  =  "1"  /* Furuncle right hand */
"L02522"  =  "1"  /* Furuncle left hand */
"L02529"  =  "1"  /* Furuncle unspecified hand */
"L02531"  =  "1"  /* Carbuncle of right hand */
"L02532"  =  "1"  /* Carbuncle of left hand */
"L02539"  =  "1"  /* Carbuncle of unspecified hand */
"L02611"  =  "1"  /* Cutaneous abscess of right foot */
"L02612"  =  "1"  /* Cutaneous abscess of left foot */
"L02619"  =  "1"  /* Cutaneous abscess of unspecified foot */
"L02621"  =  "1"  /* Furuncle of right foot */
"L02622"  =  "1"  /* Furuncle of left foot */
"L02629"  =  "1"  /* Furuncle of unspecified foot */
"L02631"  =  "1"  /* Carbuncle of right foot */
"L02632"  =  "1"  /* Carbuncle of left foot */
"L02639"  =  "1"  /* Carbuncle of unspecified foot */
"L02811"  =  "1"  /* Cutaneous abscess of head [any part, except face] */
"L02818"  =  "1"  /* Cutaneous abscess of other sites */
"L02821"  =  "1"  /* Furuncle of head [any part, except face] */
"L02828"  =  "1"  /* Furuncle of other sites */
"L02831"  =  "1"  /* Carbuncle of head [any part, except face] */
"L02838"  =  "1"  /* Carbuncle of other sites */
"L0291"  =  "1"  /* Cutaneous abscess, unspecified */
"L0292"  =  "1"  /* Furuncle, unspecified */
"L0293"  =  "1"  /* Carbuncle, unspecified */
"L03011"  =  "1"  /* Cellulitis of right finger */
"L03012"  =  "1"  /* Cellulitis of left finger */
"L03019"  =  "1"  /* Cellulitis of unspecified finger */
"L03021"  =  "1"  /* Acute lymphangitis of right finger */
"L03022"  =  "1"  /* Acute lymphangitis of left finger */
"L03029"  =  "1"  /* Acute lymphangitis of unspecified finger */
"L03031"  =  "1"  /* Cellulitis of right toe */
"L03032"  =  "1"  /* Cellulitis of left toe */
"L03039"  =  "1"  /* Cellulitis of unspecified toe */
"L03041"  =  "1"  /* Acute lymphangitis of right toe */
"L03042"  =  "1"  /* Acute lymphangitis of left toe */
"L03049"  =  "1"  /* Acute lymphangitis of unspecified toe */
"L03111"  =  "1"  /* Cellulitis of right axilla */
"L03112"  =  "1"  /* Cellulitis of left axilla */
"L03113"  =  "1"  /* Cellulitis of right upper limb */
"L03114"  =  "1"  /* Cellulitis of left upper limb */
"L03115"  =  "1"  /* Cellulitis of right lower limb */
"L03116"  =  "1"  /* Cellulitis of left lower limb */
"L03119"  =  "1"  /* Cellulitis of unspecified part of limb */
"L03121"  =  "1"  /* Acute lymphangitis of right axilla */
"L03122"  =  "1"  /* Acute lymphangitis of left axilla */
"L03123"  =  "1"  /* Acute lymphangitis of right upper limb */
"L03124"  =  "1"  /* Acute lymphangitis of left upper limb */
"L03125"  =  "1"  /* Acute lymphangitis of right lower limb */
"L03126"  =  "1"  /* Acute lymphangitis of left lower limb */
"L03129"  =  "1"  /* Acute lymphangitis of unspecified part of limb */
"L03211"  =  "1"  /* Cellulitis of face */
"L03212"  =  "1"  /* Acute lymphangitis of face */
"L03221"  =  "1"  /* Cellulitis of neck */
"L03222"  =  "1"  /* Acute lymphangitis of neck */
"L03311"  =  "1"  /* Cellulitis of abdominal wall */
"L03312"  =  "1"  /* Cellulitis of back [any part except buttock] */
"L03313"  =  "1"  /* Cellulitis of chest wall */
"L03314"  =  "1"  /* Cellulitis of groin */
"L03315"  =  "1"  /* Cellulitis of perineum */
"L03316"  =  "1"  /* Cellulitis of umbilicus */
"L03317"  =  "1"  /* Cellulitis of buttock */
"L03319"  =  "1"  /* Cellulitis of trunk, unspecified */
"L03321"  =  "1"  /* Acute lymphangitis of abdominal wall */
"L03322"  =  "1"  /* Acute lymphangitis of back [any part except buttock] */
"L03323"  =  "1"  /* Acute lymphangitis of chest wall */
"L03324"  =  "1"  /* Acute lymphangitis of groin */
"L03325"  =  "1"  /* Acute lymphangitis of perineum */
"L03326"  =  "1"  /* Acute lymphangitis of umbilicus */
"L03327"  =  "1"  /* Acute lymphangitis of buttock */
"L03329"  =  "1"  /* Acute lymphangitis of trunk, unspecified */
"L03811"  =  "1"  /* Cellulitis of head [any part, except face] */
"L03818"  =  "1"  /* Cellulitis of other sites */
"L03891"  =  "1"  /* Acute lymphangitis of head [any part, except face] */
"L03898"  =  "1"  /* Acute lymphangitis of other sites */
"L0390"  =  "1"  /* Cellulitis, unspecified */
"L0391"  =  "1"  /* Acute lymphangitis, unspecified */
"L040"  =  "1"  /* Acute lymphadenitis of face, head and neck */
"L041"  =  "1"  /* Acute lymphadenitis of trunk */
"L042"  =  "1"  /* Acute lymphadenitis of upper limb */
"L043"  =  "1"  /* Acute lymphadenitis of lower limb */
"L048"  =  "1"  /* Acute lymphadenitis of other sites */
"L049"  =  "1"  /* Acute lymphadenitis, unspecified */
"L080"  =  "1"  /* Pyoderma */
"L081"  =  "1"  /* Erythrasma */
"L0881"  =  "1"  /* Pyoderma vegetans */
"L0882"  =  "1"  /* Omphalitis not of newborn */
"L0889"  =  "1"  /* Other specified local infections of the skin and subcutaneous tissue */
"L089"  =  "1"  /* Local infection of the skin and subcutaneous tissue, unspecified */
"L89002"  =  "1"  /* Pressure ulcer of unspecified elbow, stage 2 */
"L89003"  =  "1"  /* Pressure ulcer of unspecified elbow, stage 3 */
"L89004"  =  "1"  /* Pressure ulcer of unspecified elbow, stage 4 */
"L89009"  =  "1"  /* Pressure ulcer of unspecified elbow, unspecified stage */
"L89012"  =  "1"  /* Pressure ulcer of right elbow, stage 2 */
"L89013"  =  "1"  /* Pressure ulcer of right elbow, stage 3 */
"L89014"  =  "1"  /* Pressure ulcer of right elbow, stage 4 */
"L89019"  =  "1"  /* Pressure ulcer of right elbow, unspecified stage */
"L89022"  =  "1"  /* Pressure ulcer of left elbow, stage 2 */
"L89023"  =  "1"  /* Pressure ulcer of left elbow, stage 3 */
"L89024"  =  "1"  /* Pressure ulcer of left elbow, stage 4 */
"L89029"  =  "1"  /* Pressure ulcer of left elbow, unspecified stage */
"L89102"  =  "1"  /* Pressure ulcer of unspecified part of back, stage 2 */
"L89103"  =  "1"  /* Pressure ulcer of unspecified part of back, stage 3 */
"L89104"  =  "1"  /* Pressure ulcer of unspecified part of back, stage 4 */
"L89109"  =  "1"  /* Pressure ulcer of unspecified part of back, unspecified stage */
"L89112"  =  "1"  /* Pressure ulcer of right upper back, stage 2 */
"L89113"  =  "1"  /* Pressure ulcer of right upper back, stage 3 */
"L89114"  =  "1"  /* Pressure ulcer of right upper back, stage 4 */
"L89119"  =  "1"  /* Pressure ulcer of right upper back, unspecified stage */
"L89122"  =  "1"  /* Pressure ulcer of left upper back, stage 2 */
"L89123"  =  "1"  /* Pressure ulcer of left upper back, stage 3 */
"L89124"  =  "1"  /* Pressure ulcer of left upper back, stage 4 */
"L89129"  =  "1"  /* Pressure ulcer of left upper back, unspecified stage */
"L89132"  =  "1"  /* Pressure ulcer of right lower back, stage 2 */
"L89133"  =  "1"  /* Pressure ulcer of right lower back, stage 3 */
"L89134"  =  "1"  /* Pressure ulcer of right lower back, stage 4 */
"L89139"  =  "1"  /* Pressure ulcer of right lower back, unspecified stage */
"L89142"  =  "1"  /* Pressure ulcer of left lower back, stage 2 */
"L89143"  =  "1"  /* Pressure ulcer of left lower back, stage 3 */
"L89144"  =  "1"  /* Pressure ulcer of left lower back, stage 4 */
"L89149"  =  "1"  /* Pressure ulcer of left lower back, unspecified stage */
"L89152"  =  "1"  /* Pressure ulcer of sacral region, stage 2 */
"L89153"  =  "1"  /* Pressure ulcer of sacral region, stage 3 */
"L89154"  =  "1"  /* Pressure ulcer of sacral region, stage 4 */
"L89159"  =  "1"  /* Pressure ulcer of sacral region, unspecified stage */
"L89202"  =  "1"  /* Pressure ulcer of unspecified hip, stage 2 */
"L89203"  =  "1"  /* Pressure ulcer of unspecified hip, stage 3 */
"L89204"  =  "1"  /* Pressure ulcer of unspecified hip, stage 4 */
"L89209"  =  "1"  /* Pressure ulcer of unspecified hip, unspecified stage */
"L89212"  =  "1"  /* Pressure ulcer of right hip, stage 2 */
"L89213"  =  "1"  /* Pressure ulcer of right hip, stage 3 */
"L89214"  =  "1"  /* Pressure ulcer of right hip, stage 4 */
"L89219"  =  "1"  /* Pressure ulcer of right hip, unspecified stage */
"L89222"  =  "1"  /* Pressure ulcer of left hip, stage 2 */
"L89223"  =  "1"  /* Pressure ulcer of left hip, stage 3 */
"L89224"  =  "1"  /* Pressure ulcer of left hip, stage 4 */
"L89229"  =  "1"  /* Pressure ulcer of left hip, unspecified stage */
"L89302"  =  "1"  /* Pressure ulcer of unspecified buttock, stage 2 */
"L89303"  =  "1"  /* Pressure ulcer of unspecified buttock, stage 3 */
"L89304"  =  "1"  /* Pressure ulcer of unspecified buttock, stage 4 */
"L89309"  =  "1"  /* Pressure ulcer of unspecified buttock, unspecified stage */
"L89312"  =  "1"  /* Pressure ulcer of right buttock, stage 2 */
"L89313"  =  "1"  /* Pressure ulcer of right buttock, stage 3 */
"L89314"  =  "1"  /* Pressure ulcer of right buttock, stage 4 */
"L89319"  =  "1"  /* Pressure ulcer of right buttock, unspecified stage */
"L89322"  =  "1"  /* Pressure ulcer of left buttock, stage 2 */
"L89323"  =  "1"  /* Pressure ulcer of left buttock, stage 3 */
"L89324"  =  "1"  /* Pressure ulcer of left buttock, stage 4 */
"L89329"  =  "1"  /* Pressure ulcer of left buttock, unspecified stage */
"L8940"  =  "1"  /* Pressure ulcer of contiguous site of back, buttock and hip, unspecified stage */
"L8942"  =  "1"  /* Pressure ulcer of contiguous site of back, buttock and hip, stage 2 */
"L8943"  =  "1"  /* Pressure ulcer of contiguous site of back, buttock and hip, stage 3 */
"L8944"  =  "1"  /* Pressure ulcer of contiguous site of back, buttock and hip, stage 4 */
"L89502"  =  "1"  /* Pressure ulcer of unspecified ankle, stage 2 */
"L89503"  =  "1"  /* Pressure ulcer of unspecified ankle, stage 3 */
"L89504"  =  "1"  /* Pressure ulcer of unspecified ankle, stage 4 */
"L89509"  =  "1"  /* Pressure ulcer of unspecified ankle, unspecified stage */
"L89512"  =  "1"  /* Pressure ulcer of right ankle, stage 2 */
"L89513"  =  "1"  /* Pressure ulcer of right ankle, stage 3 */
"L89514"  =  "1"  /* Pressure ulcer of right ankle, stage 4 */
"L89519"  =  "1"  /* Pressure ulcer of right ankle, unspecified stage */
"L89522"  =  "1"  /* Pressure ulcer of left ankle, stage 2 */
"L89523"  =  "1"  /* Pressure ulcer of left ankle, stage 3 */
"L89524"  =  "1"  /* Pressure ulcer of left ankle, stage 4 */
"L89529"  =  "1"  /* Pressure ulcer of left ankle, unspecified stage */
"L89602"  =  "1"  /* Pressure ulcer of unspecified heel, stage 2 */
"L89603"  =  "1"  /* Pressure ulcer of unspecified heel, stage 3 */
"L89604"  =  "1"  /* Pressure ulcer of unspecified heel, stage 4 */
"L89609"  =  "1"  /* Pressure ulcer of unspecified heel, unspecified stage */
"L89612"  =  "1"  /* Pressure ulcer of right heel, stage 2 */
"L89613"  =  "1"  /* Pressure ulcer of right heel, stage 3 */
"L89614"  =  "1"  /* Pressure ulcer of right heel, stage 4 */
"L89619"  =  "1"  /* Pressure ulcer of right heel, unspecified stage */
"L89622"  =  "1"  /* Pressure ulcer of left heel, stage 2 */
"L89623"  =  "1"  /* Pressure ulcer of left heel, stage 3 */
"L89624"  =  "1"  /* Pressure ulcer of left heel, stage 4 */
"L89629"  =  "1"  /* Pressure ulcer of left heel, unspecified stage */
"L89812"  =  "1"  /* Pressure ulcer of head, stage 2 */
"L89813"  =  "1"  /* Pressure ulcer of head, stage 3 */
"L89814"  =  "1"  /* Pressure ulcer of head, stage 4 */
"L89819"  =  "1"  /* Pressure ulcer of head, unspecified stage */
"L89892"  =  "1"  /* Pressure ulcer of other site, stage 2 */
"L89893"  =  "1"  /* Pressure ulcer of other site, stage 3 */
"L89894"  =  "1"  /* Pressure ulcer of other site, stage 4 */
"L89899"  =  "1"  /* Pressure ulcer of other site, unspecified stage */
"L8990"  =  "1"  /* Pressure ulcer of unspecified site, unspecified stage */
"L8992"  =  "1"  /* Pressure ulcer of unspecified site, stage 2 */
"L8993"  =  "1"  /* Pressure ulcer of unspecified site, stage 3 */
"L8994"  =  "1"  /* Pressure ulcer of unspecified site, stage 4 */
"L983"  =  "1"  /* Eosinophilic cellulitis [Wells] */
"M0000"  =  "1"  /* Staphylococcal arthritis, unspecified joint */
"M00011"  =  "1"  /* Staphylococcal arthritis, right shoulder */
"M00012"  =  "1"  /* Staphylococcal arthritis, left shoulder */
"M00019"  =  "1"  /* Staphylococcal arthritis, unspecified shoulder */
"M00021"  =  "1"  /* Staphylococcal arthritis, right elbow */
"M00022"  =  "1"  /* Staphylococcal arthritis, left elbow */
"M00029"  =  "1"  /* Staphylococcal arthritis, unspecified elbow */
"M00031"  =  "1"  /* Staphylococcal arthritis, right wrist */
"M00032"  =  "1"  /* Staphylococcal arthritis, left wrist */
"M00039"  =  "1"  /* Staphylococcal arthritis, unspecified wrist */
"M00041"  =  "1"  /* Staphylococcal arthritis, right hand */
"M00042"  =  "1"  /* Staphylococcal arthritis, left hand */
"M00049"  =  "1"  /* Staphylococcal arthritis, unspecified hand */
"M00051"  =  "1"  /* Staphylococcal arthritis, right hip */
"M00052"  =  "1"  /* Staphylococcal arthritis, left hip */
"M00059"  =  "1"  /* Staphylococcal arthritis, unspecified hip */
"M00061"  =  "1"  /* Staphylococcal arthritis, right knee */
"M00062"  =  "1"  /* Staphylococcal arthritis, left knee */
"M00069"  =  "1"  /* Staphylococcal arthritis, unspecified knee */
"M00071"  =  "1"  /* Staphylococcal arthritis, right ankle and foot */
"M00072"  =  "1"  /* Staphylococcal arthritis, left ankle and foot */
"M00079"  =  "1"  /* Staphylococcal arthritis, unspecified ankle and foot */
"M0008"  =  "1"  /* Staphylococcal arthritis, vertebrae */
"M0009"  =  "1"  /* Staphylococcal polyarthritis */
"M0010"  =  "1"  /* Pneumococcal arthritis, unspecified joint */
"M00111"  =  "1"  /* Pneumococcal arthritis, right shoulder */
"M00112"  =  "1"  /* Pneumococcal arthritis, left shoulder */
"M00119"  =  "1"  /* Pneumococcal arthritis, unspecified shoulder */
"M00121"  =  "1"  /* Pneumococcal arthritis, right elbow */
"M00122"  =  "1"  /* Pneumococcal arthritis, left elbow */
"M00129"  =  "1"  /* Pneumococcal arthritis, unspecified elbow */
"M00131"  =  "1"  /* Pneumococcal arthritis, right wrist */
"M00132"  =  "1"  /* Pneumococcal arthritis, left wrist */
"M00139"  =  "1"  /* Pneumococcal arthritis, unspecified wrist */
"M00141"  =  "1"  /* Pneumococcal arthritis, right hand */
"M00142"  =  "1"  /* Pneumococcal arthritis, left hand */
"M00149"  =  "1"  /* Pneumococcal arthritis, unspecified hand */
"M00151"  =  "1"  /* Pneumococcal arthritis, right hip */
"M00152"  =  "1"  /* Pneumococcal arthritis, left hip */
"M00159"  =  "1"  /* Pneumococcal arthritis, unspecified hip */
"M00161"  =  "1"  /* Pneumococcal arthritis, right knee */
"M00162"  =  "1"  /* Pneumococcal arthritis, left knee */
"M00169"  =  "1"  /* Pneumococcal arthritis, unspecified knee */
"M00171"  =  "1"  /* Pneumococcal arthritis, right ankle and foot */
"M00172"  =  "1"  /* Pneumococcal arthritis, left ankle and foot */
"M00179"  =  "1"  /* Pneumococcal arthritis, unspecified ankle and foot */
"M0018"  =  "1"  /* Pneumococcal arthritis, vertebrae */
"M0019"  =  "1"  /* Pneumococcal polyarthritis */
"M0020"  =  "1"  /* Other streptococcal arthritis, unspecified joint */
"M00211"  =  "1"  /* Other streptococcal arthritis, right shoulder */
"M00212"  =  "1"  /* Other streptococcal arthritis, left shoulder */
"M00219"  =  "1"  /* Other streptococcal arthritis, unspecified shoulder */
"M00221"  =  "1"  /* Other streptococcal arthritis, right elbow */
"M00222"  =  "1"  /* Other streptococcal arthritis, left elbow */
"M00229"  =  "1"  /* Other streptococcal arthritis, unspecified elbow */
"M00231"  =  "1"  /* Other streptococcal arthritis, right wrist */
"M00232"  =  "1"  /* Other streptococcal arthritis, left wrist */
"M00239"  =  "1"  /* Other streptococcal arthritis, unspecified wrist */
"M00241"  =  "1"  /* Other streptococcal arthritis, right hand */
"M00242"  =  "1"  /* Other streptococcal arthritis, left hand */
"M00249"  =  "1"  /* Other streptococcal arthritis, unspecified hand */
"M00251"  =  "1"  /* Other streptococcal arthritis, right hip */
"M00252"  =  "1"  /* Other streptococcal arthritis, left hip */
"M00259"  =  "1"  /* Other streptococcal arthritis, unspecified hip */
"M00261"  =  "1"  /* Other streptococcal arthritis, right knee */
"M00262"  =  "1"  /* Other streptococcal arthritis, left knee */
"M00269"  =  "1"  /* Other streptococcal arthritis, unspecified knee */
"M00271"  =  "1"  /* Other streptococcal arthritis, right ankle and foot */
"M00272"  =  "1"  /* Other streptococcal arthritis, left ankle and foot */
"M00279"  =  "1"  /* Other streptococcal arthritis, unspecified ankle and foot */
"M0028"  =  "1"  /* Other streptococcal arthritis, vertebrae */
"M0029"  =  "1"  /* Other streptococcal polyarthritis */
"M0080"  =  "1"  /* Arthritis due to other bacteria, unspecified joint */
"M00811"  =  "1"  /* Arthritis due to other bacteria, right shoulder */
"M00812"  =  "1"  /* Arthritis due to other bacteria, left shoulder */
"M00819"  =  "1"  /* Arthritis due to other bacteria, unspecified shoulder */
"M00821"  =  "1"  /* Arthritis due to other bacteria, right elbow */
"M00822"  =  "1"  /* Arthritis due to other bacteria, left elbow */
"M00829"  =  "1"  /* Arthritis due to other bacteria, unspecified elbow */
"M00831"  =  "1"  /* Arthritis due to other bacteria, right wrist */
"M00832"  =  "1"  /* Arthritis due to other bacteria, left wrist */
"M00839"  =  "1"  /* Arthritis due to other bacteria, unspecified wrist */
"M00841"  =  "1"  /* Arthritis due to other bacteria, right hand */
"M00842"  =  "1"  /* Arthritis due to other bacteria, left hand */
"M00849"  =  "1"  /* Arthritis due to other bacteria, unspecified hand */
"M00851"  =  "1"  /* Arthritis due to other bacteria, right hip */
"M00852"  =  "1"  /* Arthritis due to other bacteria, left hip */
"M00859"  =  "1"  /* Arthritis due to other bacteria, unspecified hip */
"M00861"  =  "1"  /* Arthritis due to other bacteria, right knee */
"M00862"  =  "1"  /* Arthritis due to other bacteria, left knee */
"M00869"  =  "1"  /* Arthritis due to other bacteria, unspecified knee */
"M00871"  =  "1"  /* Arthritis due to other bacteria, right ankle and foot */
"M00872"  =  "1"  /* Arthritis due to other bacteria, left ankle and foot */
"M00879"  =  "1"  /* Arthritis due to other bacteria, unspecified ankle and foot */
"M0088"  =  "1"  /* Arthritis due to other bacteria, vertebrae */
"M0089"  =  "1"  /* Polyarthritis due to other bacteria */
"M009"  =  "1"  /* Pyogenic arthritis, unspecified */
"M01X0"  =  "1"  /* Direct infection of unspecified joint in infectious and parasitic diseases classified elsewhere */
"M01X19"  =  "1"  /* Direct infection of unspecified shoulder in infectious and parasitic diseases classified elsewhere */
"M01X29"  =  "1"  /* Direct infection of unspecified elbow in infectious and parasitic diseases classified elsewhere */
"M01X39"  =  "1"  /* Direct infection of unspecified wrist in infectious and parasitic diseases classified elsewhere */
"M01X49"  =  "1"  /* Direct infection of unspecified hand in infectious and parasitic diseases classified elsewhere */
"M01X59"  =  "1"  /* Direct infection of unspecified hip in infectious and parasitic diseases classified elsewhere */
"M01X69"  =  "1"  /* Direct infection of unspecified knee in infectious and parasitic diseases classified elsewhere */
"M01X79"  =  "1"  /* Direct infection of unspecified ankle and foot in infectious and parasitic diseases classified elsewhere */
"M01X8"  =  "1"  /* Direct infection of vertebrae in infectious and parasitic diseases classified elsewhere */
"M01X9"  =  "1"  /* Direct infection of multiple joints in infectious and parasitic diseases classified elsewhere */
"M272"  =  "1"  /* Inflammatory conditions of jaws */
"M3212"  =  "1"  /* Pericarditis in systemic lupus erythematosus */
"M4620"  =  "1"  /* Osteomyelitis of vertebra, site unspecified */
"M4621"  =  "1"  /* Osteomyelitis of vertebra, occipito-atlanto-axial region */
"M4622"  =  "1"  /* Osteomyelitis of vertebra, cervical region */
"M4623"  =  "1"  /* Osteomyelitis of vertebra, cervicothoracic region */
"M4624"  =  "1"  /* Osteomyelitis of vertebra, thoracic region */
"M4625"  =  "1"  /* Osteomyelitis of vertebra, thoracolumbar region */
"M4626"  =  "1"  /* Osteomyelitis of vertebra, lumbar region */
"M4627"  =  "1"  /* Osteomyelitis of vertebra, lumbosacral region */
"M4628"  =  "1"  /* Osteomyelitis of vertebra, sacral and sacrococcygeal region */
"M4630"  =  "1"  /* Infection of intervertebral disc (pyogenic), site unspecified */
"M4631"  =  "1"  /* Infection of intervertebral disc (pyogenic), occipito-atlanto-axial region */
"M4632"  =  "1"  /* Infection of intervertebral disc (pyogenic), cervical region */
"M4633"  =  "1"  /* Infection of intervertebral disc (pyogenic), cervicothoracic region */
"M4634"  =  "1"  /* Infection of intervertebral disc (pyogenic), thoracic region */
"M4635"  =  "1"  /* Infection of intervertebral disc (pyogenic), thoracolumbar region */
"M4636"  =  "1"  /* Infection of intervertebral disc (pyogenic), lumbar region */
"M4637"  =  "1"  /* Infection of intervertebral disc (pyogenic), lumbosacral region */
"M4638"  =  "1"  /* Infection of intervertebral disc (pyogenic), sacral and sacrococcygeal region */
"M4639"  =  "1"  /* Infection of intervertebral disc (pyogenic), multiple sites in spine */
"M60000"  =  "1"  /* Infective myositis, unspecified right arm */
"M60001"  =  "1"  /* Infective myositis, unspecified left arm */
"M60002"  =  "1"  /* Infective myositis, unspecified arm */
"M60003"  =  "1"  /* Infective myositis, unspecified right leg */
"M60004"  =  "1"  /* Infective myositis, unspecified left leg */
"M60005"  =  "1"  /* Infective myositis, unspecified leg */
"M60009"  =  "1"  /* Infective myositis, unspecified site */
"M60011"  =  "1"  /* Infective myositis, right shoulder */
"M60012"  =  "1"  /* Infective myositis, left shoulder */
"M60019"  =  "1"  /* Infective myositis, unspecified shoulder */
"M60021"  =  "1"  /* Infective myositis, right upper arm */
"M60022"  =  "1"  /* Infective myositis, left upper arm */
"M60029"  =  "1"  /* Infective myositis, unspecified upper arm */
"M60031"  =  "1"  /* Infective myositis, right forearm */
"M60032"  =  "1"  /* Infective myositis, left forearm */
"M60039"  =  "1"  /* Infective myositis, unspecified forearm */
"M60041"  =  "1"  /* Infective myositis, right hand */
"M60042"  =  "1"  /* Infective myositis, left hand */
"M60043"  =  "1"  /* Infective myositis, unspecified hand */
"M60044"  =  "1"  /* Infective myositis, right finger(s) */
"M60045"  =  "1"  /* Infective myositis, left finger(s) */
"M60046"  =  "1"  /* Infective myositis, unspecified finger(s) */
"M60051"  =  "1"  /* Infective myositis, right thigh */
"M60052"  =  "1"  /* Infective myositis, left thigh */
"M60059"  =  "1"  /* Infective myositis, unspecified thigh */
"M60061"  =  "1"  /* Infective myositis, right lower leg */
"M60062"  =  "1"  /* Infective myositis, left lower leg */
"M60069"  =  "1"  /* Infective myositis, unspecified lower leg */
"M60070"  =  "1"  /* Infective myositis, right ankle */
"M60071"  =  "1"  /* Infective myositis, left ankle */
"M60072"  =  "1"  /* Infective myositis, unspecified ankle */
"M60073"  =  "1"  /* Infective myositis, right foot */
"M60074"  =  "1"  /* Infective myositis, left foot */
"M60075"  =  "1"  /* Infective myositis, unspecified foot */
"M60076"  =  "1"  /* Infective myositis, right toe(s) */
"M60077"  =  "1"  /* Infective myositis, left toe(s) */
"M60078"  =  "1"  /* Infective myositis, unspecified toe(s) */
"M6008"  =  "1"  /* Infective myositis, other site */
"M6009"  =  "1"  /* Infective myositis, multiple sites */
"M726"  =  "1"  /* Necrotizing fasciitis */
"M8600"  =  "1"  /* Acute hematogenous osteomyelitis, unspecified site */
"M86011"  =  "1"  /* Acute hematogenous osteomyelitis, right shoulder */
"M86012"  =  "1"  /* Acute hematogenous osteomyelitis, left shoulder */
"M86019"  =  "1"  /* Acute hematogenous osteomyelitis, unspecified shoulder */
"M86021"  =  "1"  /* Acute hematogenous osteomyelitis, right humerus */
"M86022"  =  "1"  /* Acute hematogenous osteomyelitis, left humerus */
"M86029"  =  "1"  /* Acute hematogenous osteomyelitis, unspecified humerus */
"M86031"  =  "1"  /* Acute hematogenous osteomyelitis, right radius and ulna */
"M86032"  =  "1"  /* Acute hematogenous osteomyelitis, left radius and ulna */
"M86039"  =  "1"  /* Acute hematogenous osteomyelitis, unspecified radius and ulna */
"M86041"  =  "1"  /* Acute hematogenous osteomyelitis, right hand */
"M86042"  =  "1"  /* Acute hematogenous osteomyelitis, left hand */
"M86049"  =  "1"  /* Acute hematogenous osteomyelitis, unspecified hand */
"M86051"  =  "1"  /* Acute hematogenous osteomyelitis, right femur */
"M86052"  =  "1"  /* Acute hematogenous osteomyelitis, left femur */
"M86059"  =  "1"  /* Acute hematogenous osteomyelitis, unspecified femur */
"M86061"  =  "1"  /* Acute hematogenous osteomyelitis, right tibia and fibula */
"M86062"  =  "1"  /* Acute hematogenous osteomyelitis, left tibia and fibula */
"M86069"  =  "1"  /* Acute hematogenous osteomyelitis, unspecified tibia and fibula */
"M86071"  =  "1"  /* Acute hematogenous osteomyelitis, right ankle and foot */
"M86072"  =  "1"  /* Acute hematogenous osteomyelitis, left ankle and foot */
"M86079"  =  "1"  /* Acute hematogenous osteomyelitis, unspecified ankle and foot */
"M8608"  =  "1"  /* Acute hematogenous osteomyelitis, other sites */
"M8609"  =  "1"  /* Acute hematogenous osteomyelitis, multiple sites */
"M8610"  =  "1"  /* Other acute osteomyelitis, unspecified site */
"M86111"  =  "1"  /* Other acute osteomyelitis, right shoulder */
"M86112"  =  "1"  /* Other acute osteomyelitis, left shoulder */
"M86119"  =  "1"  /* Other acute osteomyelitis, unspecified shoulder */
"M86121"  =  "1"  /* Other acute osteomyelitis, right humerus */
"M86122"  =  "1"  /* Other acute osteomyelitis, left humerus */
"M86129"  =  "1"  /* Other acute osteomyelitis, unspecified humerus */
"M86131"  =  "1"  /* Other acute osteomyelitis, right radius and ulna */
"M86132"  =  "1"  /* Other acute osteomyelitis, left radius and ulna */
"M86139"  =  "1"  /* Other acute osteomyelitis, unspecified radius and ulna */
"M86141"  =  "1"  /* Other acute osteomyelitis, right hand */
"M86142"  =  "1"  /* Other acute osteomyelitis, left hand */
"M86149"  =  "1"  /* Other acute osteomyelitis, unspecified hand */
"M86151"  =  "1"  /* Other acute osteomyelitis, right femur */
"M86152"  =  "1"  /* Other acute osteomyelitis, left femur */
"M86159"  =  "1"  /* Other acute osteomyelitis, unspecified femur */
"M86161"  =  "1"  /* Other acute osteomyelitis, right tibia and fibula */
"M86162"  =  "1"  /* Other acute osteomyelitis, left tibia and fibula */
"M86169"  =  "1"  /* Other acute osteomyelitis, unspecified tibia and fibula */
"M86171"  =  "1"  /* Other acute osteomyelitis, right ankle and foot */
"M86172"  =  "1"  /* Other acute osteomyelitis, left ankle and foot */
"M86179"  =  "1"  /* Other acute osteomyelitis, unspecified ankle and foot */
"M8618"  =  "1"  /* Other acute osteomyelitis, other site */
"M8619"  =  "1"  /* Other acute osteomyelitis, multiple sites */
"M8620"  =  "1"  /* Subacute osteomyelitis, unspecified site */
"M86211"  =  "1"  /* Subacute osteomyelitis, right shoulder */
"M86212"  =  "1"  /* Subacute osteomyelitis, left shoulder */
"M86219"  =  "1"  /* Subacute osteomyelitis, unspecified shoulder */
"M86221"  =  "1"  /* Subacute osteomyelitis, right humerus */
"M86222"  =  "1"  /* Subacute osteomyelitis, left humerus */
"M86229"  =  "1"  /* Subacute osteomyelitis, unspecified humerus */
"M86231"  =  "1"  /* Subacute osteomyelitis, right radius and ulna */
"M86232"  =  "1"  /* Subacute osteomyelitis, left radius and ulna */
"M86239"  =  "1"  /* Subacute osteomyelitis, unspecified radius and ulna */
"M86241"  =  "1"  /* Subacute osteomyelitis, right hand */
"M86242"  =  "1"  /* Subacute osteomyelitis, left hand */
"M86249"  =  "1"  /* Subacute osteomyelitis, unspecified hand */
"M86251"  =  "1"  /* Subacute osteomyelitis, right femur */
"M86252"  =  "1"  /* Subacute osteomyelitis, left femur */
"M86259"  =  "1"  /* Subacute osteomyelitis, unspecified femur */
"M86261"  =  "1"  /* Subacute osteomyelitis, right tibia and fibula */
"M86262"  =  "1"  /* Subacute osteomyelitis, left tibia and fibula */
"M86269"  =  "1"  /* Subacute osteomyelitis, unspecified tibia and fibula */
"M86271"  =  "1"  /* Subacute osteomyelitis, right ankle and foot */
"M86272"  =  "1"  /* Subacute osteomyelitis, left ankle and foot */
"M86279"  =  "1"  /* Subacute osteomyelitis, unspecified ankle and foot */
"M8628"  =  "1"  /* Subacute osteomyelitis, other site */
"M8629"  =  "1"  /* Subacute osteomyelitis, multiple sites */
"M8630"  =  "1"  /* Chronic multifocal osteomyelitis, unspecified site */
"M86311"  =  "1"  /* Chronic multifocal osteomyelitis, right shoulder */
"M86312"  =  "1"  /* Chronic multifocal osteomyelitis, left shoulder */
"M86319"  =  "1"  /* Chronic multifocal osteomyelitis, unspecified shoulder */
"M86321"  =  "1"  /* Chronic multifocal osteomyelitis, right humerus */
"M86322"  =  "1"  /* Chronic multifocal osteomyelitis, left humerus */
"M86329"  =  "1"  /* Chronic multifocal osteomyelitis, unspecified humerus */
"M86331"  =  "1"  /* Chronic multifocal osteomyelitis, right radius and ulna */
"M86332"  =  "1"  /* Chronic multifocal osteomyelitis, left radius and ulna */
"M86339"  =  "1"  /* Chronic multifocal osteomyelitis, unspecified radius and ulna */
"M86341"  =  "1"  /* Chronic multifocal osteomyelitis, right hand */
"M86342"  =  "1"  /* Chronic multifocal osteomyelitis, left hand */
"M86349"  =  "1"  /* Chronic multifocal osteomyelitis, unspecified hand */
"M86351"  =  "1"  /* Chronic multifocal osteomyelitis, right femur */
"M86352"  =  "1"  /* Chronic multifocal osteomyelitis, left femur */
"M86359"  =  "1"  /* Chronic multifocal osteomyelitis, unspecified femur */
"M86361"  =  "1"  /* Chronic multifocal osteomyelitis, right tibia and fibula */
"M86362"  =  "1"  /* Chronic multifocal osteomyelitis, left tibia and fibula */
"M86369"  =  "1"  /* Chronic multifocal osteomyelitis, unspecified tibia and fibula */
"M86371"  =  "1"  /* Chronic multifocal osteomyelitis, right ankle and foot */
"M86372"  =  "1"  /* Chronic multifocal osteomyelitis, left ankle and foot */
"M86379"  =  "1"  /* Chronic multifocal osteomyelitis, unspecified ankle and foot */
"M8638"  =  "1"  /* Chronic multifocal osteomyelitis, other site */
"M8639"  =  "1"  /* Chronic multifocal osteomyelitis, multiple sites */
"M8640"  =  "1"  /* Chronic osteomyelitis with draining sinus, unspecified site */
"M86411"  =  "1"  /* Chronic osteomyelitis with draining sinus, right shoulder */
"M86412"  =  "1"  /* Chronic osteomyelitis with draining sinus, left shoulder */
"M86419"  =  "1"  /* Chronic osteomyelitis with draining sinus, unspecified shoulder */
"M86421"  =  "1"  /* Chronic osteomyelitis with draining sinus, right humerus */
"M86422"  =  "1"  /* Chronic osteomyelitis with draining sinus, left humerus */
"M86429"  =  "1"  /* Chronic osteomyelitis with draining sinus, unspecified humerus */
"M86431"  =  "1"  /* Chronic osteomyelitis with draining sinus, right radius and ulna */
"M86432"  =  "1"  /* Chronic osteomyelitis with draining sinus, left radius and ulna */
"M86439"  =  "1"  /* Chronic osteomyelitis with draining sinus, unspecified radius and ulna */
"M86441"  =  "1"  /* Chronic osteomyelitis with draining sinus, right hand */
"M86442"  =  "1"  /* Chronic osteomyelitis with draining sinus, left hand */
"M86449"  =  "1"  /* Chronic osteomyelitis with draining sinus, unspecified hand */
"M86451"  =  "1"  /* Chronic osteomyelitis with draining sinus, right femur */
"M86452"  =  "1"  /* Chronic osteomyelitis with draining sinus, left femur */
"M86459"  =  "1"  /* Chronic osteomyelitis with draining sinus, unspecified femur */
"M86461"  =  "1"  /* Chronic osteomyelitis with draining sinus, right tibia and fibula */
"M86462"  =  "1"  /* Chronic osteomyelitis with draining sinus, left tibia and fibula */
"M86469"  =  "1"  /* Chronic osteomyelitis with draining sinus, unspecified tibia and fibula */
"M86471"  =  "1"  /* Chronic osteomyelitis with draining sinus, right ankle and foot */
"M86472"  =  "1"  /* Chronic osteomyelitis with draining sinus, left ankle and foot */
"M86479"  =  "1"  /* Chronic osteomyelitis with draining sinus, unspecified ankle and foot */
"M8648"  =  "1"  /* Chronic osteomyelitis with draining sinus, other site */
"M8649"  =  "1"  /* Chronic osteomyelitis with draining sinus, multiple sites */
"M8650"  =  "1"  /* Other chronic hematogenous osteomyelitis, unspecified site */
"M86511"  =  "1"  /* Other chronic hematogenous osteomyelitis, right shoulder */
"M86512"  =  "1"  /* Other chronic hematogenous osteomyelitis, left shoulder */
"M86519"  =  "1"  /* Other chronic hematogenous osteomyelitis, unspecified shoulder */
"M86521"  =  "1"  /* Other chronic hematogenous osteomyelitis, right humerus */
"M86522"  =  "1"  /* Other chronic hematogenous osteomyelitis, left humerus */
"M86529"  =  "1"  /* Other chronic hematogenous osteomyelitis, unspecified humerus */
"M86531"  =  "1"  /* Other chronic hematogenous osteomyelitis, right radius and ulna */
"M86532"  =  "1"  /* Other chronic hematogenous osteomyelitis, left radius and ulna */
"M86539"  =  "1"  /* Other chronic hematogenous osteomyelitis, unspecified radius and ulna */
"M86541"  =  "1"  /* Other chronic hematogenous osteomyelitis, right hand */
"M86542"  =  "1"  /* Other chronic hematogenous osteomyelitis, left hand */
"M86549"  =  "1"  /* Other chronic hematogenous osteomyelitis, unspecified hand */
"M86551"  =  "1"  /* Other chronic hematogenous osteomyelitis, right femur */
"M86552"  =  "1"  /* Other chronic hematogenous osteomyelitis, left femur */
"M86559"  =  "1"  /* Other chronic hematogenous osteomyelitis, unspecified femur */
"M86561"  =  "1"  /* Other chronic hematogenous osteomyelitis, right tibia and fibula */
"M86562"  =  "1"  /* Other chronic hematogenous osteomyelitis, left tibia and fibula */
"M86569"  =  "1"  /* Other chronic hematogenous osteomyelitis, unspecified tibia and fibula */
"M86571"  =  "1"  /* Other chronic hematogenous osteomyelitis, right ankle and foot */
"M86572"  =  "1"  /* Other chronic hematogenous osteomyelitis, left ankle and foot */
"M86579"  =  "1"  /* Other chronic hematogenous osteomyelitis, unspecified ankle and foot */
"M8658"  =  "1"  /* Other chronic hematogenous osteomyelitis, other site */
"M8659"  =  "1"  /* Other chronic hematogenous osteomyelitis, multiple sites */
"M8660"  =  "1"  /* Other chronic osteomyelitis, unspecified site */
"M86611"  =  "1"  /* Other chronic osteomyelitis, right shoulder */
"M86612"  =  "1"  /* Other chronic osteomyelitis, left shoulder */
"M86619"  =  "1"  /* Other chronic osteomyelitis, unspecified shoulder */
"M86621"  =  "1"  /* Other chronic osteomyelitis, right  humerus */
"M86622"  =  "1"  /* Other chronic osteomyelitis, left  humerus */
"M86629"  =  "1"  /* Other chronic osteomyelitis, unspecified  humerus */
"M86631"  =  "1"  /* Other chronic osteomyelitis, right radius and ulna */
"M86632"  =  "1"  /* Other chronic osteomyelitis, left radius and ulna */
"M86639"  =  "1"  /* Other chronic osteomyelitis, unspecified radius and ulna */
"M86641"  =  "1"  /* Other chronic osteomyelitis, right hand */
"M86642"  =  "1"  /* Other chronic osteomyelitis, left hand */
"M86649"  =  "1"  /* Other chronic osteomyelitis, unspecified hand */
"M86651"  =  "1"  /* Other chronic osteomyelitis, right thigh */
"M86652"  =  "1"  /* Other chronic osteomyelitis, left thigh */
"M86659"  =  "1"  /* Other chronic osteomyelitis, unspecified thigh */
"M86661"  =  "1"  /* Other chronic osteomyelitis, right tibia and fibula */
"M86662"  =  "1"  /* Other chronic osteomyelitis, left tibia and fibula */
"M86669"  =  "1"  /* Other chronic osteomyelitis, unspecified tibia and fibula */
"M86671"  =  "1"  /* Other chronic osteomyelitis, right ankle and foot */
"M86672"  =  "1"  /* Other chronic osteomyelitis, left ankle and foot */
"M86679"  =  "1"  /* Other chronic osteomyelitis, unspecified ankle and foot */
"M8668"  =  "1"  /* Other chronic osteomyelitis, other site */
"M8669"  =  "1"  /* Other chronic osteomyelitis, multiple sites */
"M868X0"  =  "1"  /* Other osteomyelitis, multiple sites */
"M868X1"  =  "1"  /* Other osteomyelitis, shoulder */
"M868X2"  =  "1"  /* Other osteomyelitis, upper arm */
"M868X3"  =  "1"  /* Other osteomyelitis, forearm */
"M868X4"  =  "1"  /* Other osteomyelitis, hand */
"M868X5"  =  "1"  /* Other osteomyelitis, thigh */
"M868X6"  =  "1"  /* Other osteomyelitis, lower leg */
"M868X7"  =  "1"  /* Other osteomyelitis, ankle and foot */
"M868X8"  =  "1"  /* Other osteomyelitis, other site */
"M868X9"  =  "1"  /* Other osteomyelitis, unspecified sites */
"M869"  =  "1"  /* Osteomyelitis, unspecified */
"N10"  =  "1"  /* Acute tubulo-interstitial nephritis */
"N119"  =  "1"  /* Chronic tubulo-interstitial nephritis, unspecified */
"N12"  =  "1"  /* Tubulo-interstitial nephritis, not specified as acute or chronic */
"N136"  =  "1"  /* Pyonephrosis */
"N151"  =  "1"  /* Renal and perinephric abscess */
"N159"  =  "1"  /* Renal tubulo-interstitial disease, unspecified */
"N16"  =  "1"  /* Renal tubulo-interstitial disorders in diseases classified elsewhere */
"N2884"  =  "1"  /* Pyelitis cystica */
"N2885"  =  "1"  /* Pyeloureteritis cystica */
"N2886"  =  "1"  /* Ureteritis cystica */
"N3000"  =  "1"  /* Acute cystitis without hematuria */
"N3001"  =  "1"  /* Acute cystitis with hematuria */
"N3080"  =  "1"  /* Other cystitis without hematuria */
"N3081"  =  "1"  /* Other cystitis with hematuria */
"N3090"  =  "1"  /* Cystitis, unspecified without hematuria */
"N3091"  =  "1"  /* Cystitis, unspecified with hematuria */
"N340"  =  "1"  /* Urethral abscess */
"N390"  =  "1"  /* Urinary tract infection, site not specified */
"N410"  =  "1"  /* Acute prostatitis */
"N412"  =  "1"  /* Abscess of prostate */
"N413"  =  "1"  /* Prostatocystitis */
"N414"  =  "1"  /* Granulomatous prostatitis */
"N418"  =  "1"  /* Other inflammatory diseases of prostate */
"N419"  =  "1"  /* Inflammatory disease of prostate, unspecified */
"N431"  =  "1"  /* Infected hydrocele */
"N451"  =  "1"  /* Epididymitis */
"N452"  =  "1"  /* Orchitis */
"N453"  =  "1"  /* Epididymo-orchitis */
"N454"  =  "1"  /* Abscess of epididymis or testis */
"N476"  =  "1"  /* Balanoposthitis */
"N481"  =  "1"  /* Balanitis */
"N4821"  =  "1"  /* Abscess of corpus cavernosum and penis */
"N4822"  =  "1"  /* Cellulitis of corpus cavernosum and penis */
"N4829"  =  "1"  /* Other inflammatory disorders of penis */
"N490"  =  "1"  /* Inflammatory disorders of seminal vesicle */
"N491"  =  "1"  /* Inflammatory disorders of spermatic cord, tunica vaginalis and vas deferens */
"N492"  =  "1"  /* Inflammatory disorders of scrotum */
"N493"  =  "1"  /* Fournier gangrene */
"N498"  =  "1"  /* Inflammatory disorders of other specified male genital organs */
"N499"  =  "1"  /* Inflammatory disorder of unspecified male genital organ */
"N61"  =  "1"  /* Inflammatory disorders of breast */
"N7001"  =  "1"  /* Acute salpingitis */
"N7002"  =  "1"  /* Acute oophoritis */
"N7003"  =  "1"  /* Acute salpingitis and oophoritis */
"N7011"  =  "1"  /* Chronic salpingitis */
"N7012"  =  "1"  /* Chronic oophoritis */
"N7013"  =  "1"  /* Chronic salpingitis and oophoritis */
"N7091"  =  "1"  /* Salpingitis, unspecified */
"N7092"  =  "1"  /* Oophoritis, unspecified */
"N7093"  =  "1"  /* Salpingitis and oophoritis, unspecified */
"N710"  =  "1"  /* Acute inflammatory disease of uterus */
"N719"  =  "1"  /* Inflammatory disease of uterus, unspecified */
"N72"  =  "1"  /* Inflammatory disease of cervix uteri */
"N730"  =  "1"  /* Acute parametritis and pelvic cellulitis */
"N731"  =  "1"  /* Chronic parametritis and pelvic cellulitis */
"N732"  =  "1"  /* Unspecified parametritis and pelvic cellulitis */
"N733"  =  "1"  /* Female acute pelvic peritonitis */
"N735"  =  "1"  /* Female pelvic peritonitis, unspecified */
"N739"  =  "1"  /* Female pelvic inflammatory disease, unspecified */
"N751"  =  "1"  /* Abscess of Bartholin's gland */
"N760"  =  "1"  /* Acute vaginitis */
"N761"  =  "1"  /* Subacute and chronic vaginitis */
"N762"  =  "1"  /* Acute vulvitis */
"N763"  =  "1"  /* Subacute and chronic vulvitis */
"N764"  =  "1"  /* Abscess of vulva */
"N99511"  =  "1"  /* Cystostomy infection */
"O030"  =  "1"  /* Genital tract and pelvic infection following incomplete spontaneous abortion */
"O0337"  =  "1"  /* Sepsis following incomplete spontaneous abortion */
"O035"  =  "1"  /* Genital tract and pelvic infection following complete or unspecified spontaneous abortion */
"O0387"  =  "1"  /* Sepsis following complete or unspecified spontaneous abortion */
"O045"  =  "1"  /* Genital tract and pelvic infection following (induced) termination of pregnancy */
"O0487"  =  "1"  /* Sepsis following (induced) termination of pregnancy */
"O070"  =  "1"  /* Genital tract and pelvic infection following failed attempted termination of pregnancy */
"O0737"  =  "1"  /* Sepsis following failed attempted termination of pregnancy */
"O080"  =  "1"  /* Genital tract and pelvic infection following ectopic and molar pregnancy */
"O0882"  =  "1"  /* Sepsis following ectopic and molar pregnancy */
"O2300"  =  "1"  /* Infections of kidney in pregnancy, unspecified trimester */
"O2301"  =  "1"  /* Infections of kidney in pregnancy, first trimester */
"O2302"  =  "1"  /* Infections of kidney in pregnancy, second trimester */
"O2303"  =  "1"  /* Infections of kidney in pregnancy, third trimester */
"O2310"  =  "1"  /* Infections of bladder in pregnancy, unspecified trimester */
"O2311"  =  "1"  /* Infections of bladder in pregnancy, first trimester */
"O2312"  =  "1"  /* Infections of bladder in pregnancy, second trimester */
"O2313"  =  "1"  /* Infections of bladder in pregnancy, third trimester */
"O2320"  =  "1"  /* Infections of urethra in pregnancy, unspecified trimester */
"O2321"  =  "1"  /* Infections of urethra in pregnancy, first trimester */
"O2322"  =  "1"  /* Infections of urethra in pregnancy, second trimester */
"O2323"  =  "1"  /* Infections of urethra in pregnancy, third trimester */
"O2330"  =  "1"  /* Infections of other parts of urinary tract in pregnancy, unspecified trimester */
"O2331"  =  "1"  /* Infections of other parts of urinary tract in pregnancy, first trimester */
"O2332"  =  "1"  /* Infections of other parts of urinary tract in pregnancy, second trimester */
"O2333"  =  "1"  /* Infections of other parts of urinary tract in pregnancy, third trimester */
"O2340"  =  "1"  /* Unspecified infection of urinary tract in pregnancy, unspecified trimester */
"O2341"  =  "1"  /* Unspecified infection of urinary tract in pregnancy, first trimester */
"O2342"  =  "1"  /* Unspecified infection of urinary tract in pregnancy, second trimester */
"O2343"  =  "1"  /* Unspecified infection of urinary tract in pregnancy, third trimester */
"O23511"  =  "1"  /* Infections of cervix in pregnancy, first trimester */
"O23512"  =  "1"  /* Infections of cervix in pregnancy, second trimester */
"O23513"  =  "1"  /* Infections of cervix in pregnancy, third trimester */
"O23519"  =  "1"  /* Infections of cervix in pregnancy, unspecified trimester */
"O23521"  =  "1"  /* Salpingo-oophoritis in pregnancy, first trimester */
"O23522"  =  "1"  /* Salpingo-oophoritis in pregnancy, second trimester */
"O23523"  =  "1"  /* Salpingo-oophoritis in pregnancy, third trimester */
"O23529"  =  "1"  /* Salpingo-oophoritis in pregnancy, unspecified trimester */
"O23591"  =  "1"  /* Infection of other part of genital tract in pregnancy, first trimester */
"O23592"  =  "1"  /* Infection of other part of genital tract in pregnancy, second trimester */
"O23593"  =  "1"  /* Infection of other part of genital tract in pregnancy, third trimester */
"O23599"  =  "1"  /* Infection of other part of genital tract in pregnancy, unspecified trimester */
"O2390"  =  "1"  /* Unspecified genitourinary tract infection in pregnancy, unspecified trimester */
"O2391"  =  "1"  /* Unspecified genitourinary tract infection in pregnancy, first trimester */
"O2392"  =  "1"  /* Unspecified genitourinary tract infection in pregnancy, second trimester */
"O2393"  =  "1"  /* Unspecified genitourinary tract infection in pregnancy, third trimester */
"O411010"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, first trimester, not applicable or unspecified */
"O411011"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, first trimester, fetus 1 */
"O411012"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, first trimester, fetus 2 */
"O411013"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, first trimester, fetus 3 */
"O411014"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, first trimester, fetus 4 */
"O411015"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, first trimester, fetus 5 */
"O411019"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, first trimester, other fetus */
"O411020"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, second trimester, not applicable or unspecified */
"O411021"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, second trimester, fetus 1 */
"O411022"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, second trimester, fetus 2 */
"O411023"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, second trimester, fetus 3 */
"O411024"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, second trimester, fetus 4 */
"O411025"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, second trimester, fetus 5 */
"O411029"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, second trimester, other fetus */
"O411030"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, third trimester, not applicable or unspecified */
"O411031"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, third trimester, fetus 1 */
"O411032"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, third trimester, fetus 2 */
"O411033"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, third trimester, fetus 3 */
"O411034"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, third trimester, fetus 4 */
"O411035"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, third trimester, fetus 5 */
"O411039"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, third trimester, other fetus */
"O411090"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, unspecified trimester, not applicable or unspecified */
"O411091"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, unspecified trimester, fetus 1 */
"O411092"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, unspecified trimester, fetus 2 */
"O411093"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, unspecified trimester, fetus 3 */
"O411094"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, unspecified trimester, fetus 4 */
"O411095"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, unspecified trimester, fetus 5 */
"O411099"  =  "1"  /* Infection of amniotic sac and membranes, unspecified, unspecified trimester, other fetus */
"O411210"  =  "1"  /* Chorioamnionitis, first trimester, not applicable or unspecified */
"O411211"  =  "1"  /* Chorioamnionitis, first trimester, fetus 1 */
"O411212"  =  "1"  /* Chorioamnionitis, first trimester, fetus 2 */
"O411213"  =  "1"  /* Chorioamnionitis, first trimester, fetus 3 */
"O411214"  =  "1"  /* Chorioamnionitis, first trimester, fetus 4 */
"O411215"  =  "1"  /* Chorioamnionitis, first trimester, fetus 5 */
"O411219"  =  "1"  /* Chorioamnionitis, first trimester, other fetus */
"O411220"  =  "1"  /* Chorioamnionitis, second trimester, not applicable or unspecified */
"O411221"  =  "1"  /* Chorioamnionitis, second trimester, fetus 1 */
"O411222"  =  "1"  /* Chorioamnionitis, second trimester, fetus 2 */
"O411223"  =  "1"  /* Chorioamnionitis, second trimester, fetus 3 */
"O411224"  =  "1"  /* Chorioamnionitis, second trimester, fetus 4 */
"O411225"  =  "1"  /* Chorioamnionitis, second trimester, fetus 5 */
"O411229"  =  "1"  /* Chorioamnionitis, second trimester, other fetus */
"O411230"  =  "1"  /* Chorioamnionitis, third trimester, not applicable or unspecified */
"O411231"  =  "1"  /* Chorioamnionitis, third trimester, fetus 1 */
"O411232"  =  "1"  /* Chorioamnionitis, third trimester, fetus 2 */
"O411233"  =  "1"  /* Chorioamnionitis, third trimester, fetus 3 */
"O411234"  =  "1"  /* Chorioamnionitis, third trimester, fetus 4 */
"O411235"  =  "1"  /* Chorioamnionitis, third trimester, fetus 5 */
"O411239"  =  "1"  /* Chorioamnionitis, third trimester, other fetus */
"O411290"  =  "1"  /* Chorioamnionitis, unspecified trimester, not applicable or unspecified */
"O411291"  =  "1"  /* Chorioamnionitis, unspecified trimester, fetus 1 */
"O411292"  =  "1"  /* Chorioamnionitis, unspecified trimester, fetus 2 */
"O411293"  =  "1"  /* Chorioamnionitis, unspecified trimester, fetus 3 */
"O411294"  =  "1"  /* Chorioamnionitis, unspecified trimester, fetus 4 */
"O411295"  =  "1"  /* Chorioamnionitis, unspecified trimester, fetus 5 */
"O411299"  =  "1"  /* Chorioamnionitis, unspecified trimester, other fetus */
"O411410"  =  "1"  /* Placentitis, first trimester, not applicable or unspecified */
"O411411"  =  "1"  /* Placentitis, first trimester, fetus 1 */
"O411412"  =  "1"  /* Placentitis, first trimester, fetus 2 */
"O411413"  =  "1"  /* Placentitis, first trimester, fetus 3 */
"O411414"  =  "1"  /* Placentitis, first trimester, fetus 4 */
"O411415"  =  "1"  /* Placentitis, first trimester, fetus 5 */
"O411419"  =  "1"  /* Placentitis, first trimester, other fetus */
"O411420"  =  "1"  /* Placentitis, second trimester, not applicable or unspecified */
"O411421"  =  "1"  /* Placentitis, second trimester, fetus 1 */
"O411422"  =  "1"  /* Placentitis, second trimester, fetus 2 */
"O411423"  =  "1"  /* Placentitis, second trimester, fetus 3 */
"O411424"  =  "1"  /* Placentitis, second trimester, fetus 4 */
"O411425"  =  "1"  /* Placentitis, second trimester, fetus 5 */
"O411429"  =  "1"  /* Placentitis, second trimester, other fetus */
"O411430"  =  "1"  /* Placentitis, third trimester, not applicable or unspecified */
"O411431"  =  "1"  /* Placentitis, third trimester, fetus 1 */
"O411432"  =  "1"  /* Placentitis, third trimester, fetus 2 */
"O411433"  =  "1"  /* Placentitis, third trimester, fetus 3 */
"O411434"  =  "1"  /* Placentitis, third trimester, fetus 4 */
"O411435"  =  "1"  /* Placentitis, third trimester, fetus 5 */
"O411439"  =  "1"  /* Placentitis, third trimester, other fetus */
"O411490"  =  "1"  /* Placentitis, unspecified trimester, not applicable or unspecified */
"O411491"  =  "1"  /* Placentitis, unspecified trimester, fetus 1 */
"O411492"  =  "1"  /* Placentitis, unspecified trimester, fetus 2 */
"O411493"  =  "1"  /* Placentitis, unspecified trimester, fetus 3 */
"O411494"  =  "1"  /* Placentitis, unspecified trimester, fetus 4 */
"O411495"  =  "1"  /* Placentitis, unspecified trimester, fetus 5 */
"O411499"  =  "1"  /* Placentitis, unspecified trimester, other fetus */
"O85"  =  "1"  /* Puerperal sepsis */
"O8611"  =  "1"  /* Cervicitis following delivery */
"O8612"  =  "1"  /* Endometritis following delivery */
"O8613"  =  "1"  /* Vaginitis following delivery */
"O8619"  =  "1"  /* Other infection of genital tract following delivery */
"O8620"  =  "1"  /* Urinary tract infection following delivery, unspecified */
"O8621"  =  "1"  /* Infection of kidney following delivery */
"O8622"  =  "1"  /* Infection of bladder following delivery */
"O8629"  =  "1"  /* Other urinary tract infection following delivery */
"O8681"  =  "1"  /* Puerperal septic thrombophlebitis */
"O8689"  =  "1"  /* Other specified puerperal infections */
"O9089"  =  "1"  /* Other complications of the puerperium, not elsewhere classified */
"O91011"  =  "1"  /* Infection of nipple associated with pregnancy, first trimester */
"O91012"  =  "1"  /* Infection of nipple associated with pregnancy, second trimester */
"O91013"  =  "1"  /* Infection of nipple associated with pregnancy, third trimester */
"O91019"  =  "1"  /* Infection of nipple associated with pregnancy, unspecified trimester */
"O9102"  =  "1"  /* Infection of nipple associated with the puerperium */
"O9103"  =  "1"  /* Infection of nipple associated with lactation */
"O91111"  =  "1"  /* Abscess of breast associated with pregnancy, first trimester */
"O91112"  =  "1"  /* Abscess of breast associated with pregnancy, second trimester */
"O91113"  =  "1"  /* Abscess of breast associated with pregnancy, third trimester */
"O91119"  =  "1"  /* Abscess of breast associated with pregnancy, unspecified trimester */
"O9112"  =  "1"  /* Abscess of breast associated with the puerperium */
"O9113"  =  "1"  /* Abscess of breast associated with lactation */
"O9123"  =  "1"  /* Nonpurulent mastitis associated with lactation */
"O98211"  =  "1"  /* Gonorrhea complicating pregnancy, first trimester */
"O98212"  =  "1"  /* Gonorrhea complicating pregnancy, second trimester */
"O98213"  =  "1"  /* Gonorrhea complicating pregnancy, third trimester */
"O98219"  =  "1"  /* Gonorrhea complicating pregnancy, unspecified trimester */
"O9822"  =  "1"  /* Gonorrhea complicating childbirth */
"O9823"  =  "1"  /* Gonorrhea complicating the puerperium */
"O98611"  =  "1"  /* Protozoal diseases complicating pregnancy, first trimester */
"O98612"  =  "1"  /* Protozoal diseases complicating pregnancy, second trimester */
"O98613"  =  "1"  /* Protozoal diseases complicating pregnancy, third trimester */
"O98811"  =  "1"  /* Other maternal infectious and parasitic diseases complicating pregnancy, first trimester */
"O98812"  =  "1"  /* Other maternal infectious and parasitic diseases complicating pregnancy, second trimester */
"O98813"  =  "1"  /* Other maternal infectious and parasitic diseases complicating pregnancy, third trimester */
"O98819"  =  "1"  /* Other maternal infectious and parasitic diseases complicating pregnancy, unspecified trimester */
"O9882"  =  "1"  /* Other maternal infectious and parasitic diseases complicating childbirth */
"O9883"  =  "1"  /* Other maternal infectious and parasitic diseases complicating the puerperium */
"O98911"  =  "1"  /* Unspecified maternal infectious and parasitic disease complicating pregnancy, first trimester */
"O98912"  =  "1"  /* Unspecified maternal infectious and parasitic disease complicating pregnancy, second trimester */
"O98913"  =  "1"  /* Unspecified maternal infectious and parasitic disease complicating pregnancy, third trimester */
"O98919"  =  "1"  /* Unspecified maternal infectious and parasitic disease complicating pregnancy, unspecified trimester */
"O9892"  =  "1"  /* Unspecified maternal infectious and parasitic disease complicating childbirth */
"O9893"  =  "1"  /* Unspecified maternal infectious and parasitic disease complicating the puerperium */
"O99830"  =  "1"  /* Other infection carrier state complicating pregnancy */
"O99834"  =  "1"  /* Other infection carrier state complicating childbirth */
"O99835"  =  "1"  /* Other infection carrier state complicating the puerperium */
"P360"  =  "1"  /* Sepsis of newborn due to streptococcus, group B */
"P3610"  =  "1"  /* Sepsis of newborn due to unspecified streptococci */
"P3619"  =  "1"  /* Sepsis of newborn due to other streptococci */
"P362"  =  "1"  /* Sepsis of newborn due to Staphylococcus aureus */
"P3630"  =  "1"  /* Sepsis of newborn due to unspecified staphylococci */
"P3639"  =  "1"  /* Sepsis of newborn due to other staphylococci */
"P364"  =  "1"  /* Sepsis of newborn due to Escherichia coli */
"P365"  =  "1"  /* Sepsis of newborn due to anaerobes */
"P368"  =  "1"  /* Other bacterial sepsis of newborn */
"P369"  =  "1"  /* Bacterial sepsis of newborn, unspecified */
"P381"  =  "1"  /* Omphalitis with mild hemorrhage */
"P389"  =  "1"  /* Omphalitis without hemorrhage */
"P390"  =  "1"  /* Neonatal infective mastitis */
"P392"  =  "1"  /* Intra-amniotic infection affecting newborn, not elsewhere classified */
"P393"  =  "1"  /* Neonatal urinary tract infection */
"P394"  =  "1"  /* Neonatal skin infection */
"P398"  =  "1"  /* Other specified infections specific to the perinatal period */
"P399"  =  "1"  /* Infection specific to the perinatal period, unspecified */
"P771"  =  "1"  /* Stage 1 necrotizing enterocolitis in newborn */
"P772"  =  "1"  /* Stage 2 necrotizing enterocolitis in newborn */
"P773"  =  "1"  /* Stage 3 necrotizing enterocolitis in newborn */
"P779"  =  "1"  /* Necrotizing enterocolitis in newborn, unspecified */
"R6510"  =  "1"  /* Systemic inflammatory response syndrome (SIRS) of non-infectious origin without acute organ dysfunction */
"R6520"  =  "1"  /* Severe sepsis without septic shock */
"R6521"  =  "1"  /* Severe sepsis with septic shock */
"R7881"  =  "1"  /* Bacteremia */
"T80211A"  =  "1"  /* Bloodstream infection due to central venous catheter, initial encounter */
"T80212A"  =  "1"  /* Local infection due to central venous catheter, initial encounter */
"T80218A"  =  "1"  /* Other infection due to central venous catheter, initial encounter */
"T80219A"  =  "1"  /* Unspecified infection due to central venous catheter, initial encounter */
"T8022XA"  =  "1"  /* Acute infection following transfusion, infusion, or injection of blood and blood products, initial encounter */
"T8112XA"  =  "1"  /* Postprocedural septic shock, initial encounter */
"T814XXA"  =  "1"  /* Infection following a procedure, initial encounter */
"T826XXA"  =  "1"  /* Infection and inflammatory reaction due to cardiac valve prosthesis, initial encounter */
"T827XXA"  =  "1"  /* Infection and inflammatory reaction due to other cardiac and vascular devices, implants and grafts, initial encounter */
"T8351XA"  =  "1"  /* Infection and inflammatory reaction due to indwelling urinary catheter, initial encounter */
"T8359XA"  =  "1"  /* Infection and inflammatory reaction due to prosthetic device, implant and graft in urinary system, initial encounter */
"T836XXA"  =  "1"  /* Infection and inflammatory reaction due to prosthetic device, implant and graft in genital tract, initial encounter */
"T8450XA"  =  "1"  /* Infection and inflammatory reaction due to unspecified internal joint prosthesis, initial encounter */
"T8451XA"  =  "1"  /* Infection and inflammatory reaction due to internal right hip prosthesis, initial encounter */
"T8452XA"  =  "1"  /* Infection and inflammatory reaction due to internal left hip prosthesis, initial encounter */
"T8453XA"  =  "1"  /* Infection and inflammatory reaction due to internal right knee prosthesis, initial encounter */
"T8454XA"  =  "1"  /* Infection and inflammatory reaction due to internal left knee prosthesis, initial encounter */
"T8459XA"  =  "1"  /* Infection and inflammatory reaction due to other internal joint prosthesis, initial encounter */
"T8460XA"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of unspecified site, initial encounter */
"T84610A"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of right humerus, initial encounter */
"T84611A"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of left humerus, initial encounter */
"T84612A"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of right radius, initial encounter */
"T84613A"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of left radius, initial encounter */
"T84614A"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of right ulna, initial encounter */
"T84615A"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of left ulna, initial encounter */
"T84619A"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of unspecified bone of arm, initial encounter */
"T84620A"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of right femur, initial encounter */
"T84621A"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of left femur, initial encounter */
"T84622A"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of right tibia, initial encounter */
"T84623A"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of left tibia, initial encounter */
"T84624A"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of right fibula, initial encounter */
"T84625A"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of left fibula, initial encounter */
"T84629A"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of unspecified bone of leg, initial encounter */
"T8463XA"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of spine, initial encounter */
"T8469XA"  =  "1"  /* Infection and inflammatory reaction due to internal fixation device of other site, initial encounter */
"T847XXA"  =  "1"  /* Infection and inflammatory reaction due to other internal orthopedic prosthetic devices, implants and grafts, initial encounter */
"T8572XA"  =  "1"  /* Infection and inflammatory reaction due to insulin pump, initial encounter */
"T8579XA"  =  "1"  /* Infection and inflammatory reaction due to other internal prosthetic devices, implants and grafts, initial encounter */
"T86842"  =  "1"  /* Corneal transplant infection */
"T8740"  =  "1"  /* Infection of amputation stump, unspecified extremity */
"T8741"  =  "1"  /* Infection of amputation stump, right upper extremity */
"T8742"  =  "1"  /* Infection of amputation stump, left upper extremity */
"T8743"  =  "1"  /* Infection of amputation stump, right lower extremity */
"T8744"  =  "1"  /* Infection of amputation stump, left lower extremity */
OTHER   = "0" ;


 /* ICD-10-CM PROCEDURE CODES: */

VALUE $SPINEP
"009T00Z"  =  "1"  /* Drainage of Spinal Meninges with Drainage Device, Open Approach */
"009T0ZZ"  =  "1"  /* Drainage of Spinal Meninges, Open Approach */
"009T30Z"  =  "1"  /* Drainage of Spinal Meninges with Drainage Device, Percutaneous Approach */
"009T3ZZ"  =  "1"  /* Drainage of Spinal Meninges, Percutaneous Approach */
"009T40Z"  =  "1"  /* Drainage of Spinal Meninges with Drainage Device, Percutaneous Endoscopic Approach */
"009T4ZZ"  =  "1"  /* Drainage of Spinal Meninges, Percutaneous Endoscopic Approach */
"009U00Z"  =  "1"  /* Drainage of Spinal Canal with Drainage Device, Open Approach */
"009U0ZZ"  =  "1"  /* Drainage of Spinal Canal, Open Approach */
"009W00Z"  =  "1"  /* Drainage of Cervical Spinal Cord with Drainage Device, Open Approach */
"009W0ZZ"  =  "1"  /* Drainage of Cervical Spinal Cord, Open Approach */
"009W30Z"  =  "1"  /* Drainage of Cervical Spinal Cord with Drainage Device, Percutaneous Approach */
"009W3ZZ"  =  "1"  /* Drainage of Cervical Spinal Cord, Percutaneous Approach */
"009W40Z"  =  "1"  /* Drainage of Cervical Spinal Cord with Drainage Device, Percutaneous Endoscopic Approach */
"009W4ZZ"  =  "1"  /* Drainage of Cervical Spinal Cord, Percutaneous Endoscopic Approach */
"009X00Z"  =  "1"  /* Drainage of Thoracic Spinal Cord with Drainage Device, Open Approach */
"009X0ZZ"  =  "1"  /* Drainage of Thoracic Spinal Cord, Open Approach */
"009X30Z"  =  "1"  /* Drainage of Thoracic Spinal Cord with Drainage Device, Percutaneous Approach */
"009X3ZZ"  =  "1"  /* Drainage of Thoracic Spinal Cord, Percutaneous Approach */
"009X40Z"  =  "1"  /* Drainage of Thoracic Spinal Cord with Drainage Device, Percutaneous Endoscopic Approach */
"009X4ZZ"  =  "1"  /* Drainage of Thoracic Spinal Cord, Percutaneous Endoscopic Approach */
"009Y00Z"  =  "1"  /* Drainage of Lumbar Spinal Cord with Drainage Device, Open Approach */
"009Y0ZZ"  =  "1"  /* Drainage of Lumbar Spinal Cord, Open Approach */
"009Y30Z"  =  "1"  /* Drainage of Lumbar Spinal Cord with Drainage Device, Percutaneous Approach */
"009Y3ZZ"  =  "1"  /* Drainage of Lumbar Spinal Cord, Percutaneous Approach */
"009Y40Z"  =  "1"  /* Drainage of Lumbar Spinal Cord with Drainage Device, Percutaneous Endoscopic Approach */
"009Y4ZZ"  =  "1"  /* Drainage of Lumbar Spinal Cord, Percutaneous Endoscopic Approach */
"00C30ZZ"  =  "1"  /* Extirpation of Matter from Epidural Space, Open Approach */
"00C33ZZ"  =  "1"  /* Extirpation of Matter from Epidural Space, Percutaneous Approach */
"00C34ZZ"  =  "1"  /* Extirpation of Matter from Epidural Space, Percutaneous Endoscopic Approach */
"00CT0ZZ"  =  "1"  /* Extirpation of Matter from Spinal Meninges, Open Approach */
"00CT3ZZ"  =  "1"  /* Extirpation of Matter from Spinal Meninges, Percutaneous Approach */
"00CT4ZZ"  =  "1"  /* Extirpation of Matter from Spinal Meninges, Percutaneous Endoscopic Approach */
"00CW0ZZ"  =  "1"  /* Extirpation of Matter from Cervical Spinal Cord, Open Approach */
"00CW3ZZ"  =  "1"  /* Extirpation of Matter from Cervical Spinal Cord, Percutaneous Approach */
"00CW4ZZ"  =  "1"  /* Extirpation of Matter from Cervical Spinal Cord, Percutaneous Endoscopic Approach */
"00CX0ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Spinal Cord, Open Approach */
"00CX3ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Spinal Cord, Percutaneous Approach */
"00CX4ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Spinal Cord, Percutaneous Endoscopic Approach */
"00CY0ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Spinal Cord, Open Approach */
"00CY3ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Spinal Cord, Percutaneous Approach */
"00CY4ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Spinal Cord, Percutaneous Endoscopic Approach */
"00JU0ZZ"  =  "1"  /* Inspection of Spinal Canal, Open Approach */
"00JV0ZZ"  =  "1"  /* Inspection of Spinal Cord, Open Approach */
"00NW0ZZ"  =  "1"  /* Release Cervical Spinal Cord, Open Approach */
"00NW3ZZ"  =  "1"  /* Release Cervical Spinal Cord, Percutaneous Approach */
"00NW4ZZ"  =  "1"  /* Release Cervical Spinal Cord, Percutaneous Endoscopic Approach */
"00NX0ZZ"  =  "1"  /* Release Thoracic Spinal Cord, Open Approach */
"00NX3ZZ"  =  "1"  /* Release Thoracic Spinal Cord, Percutaneous Approach */
"00NX4ZZ"  =  "1"  /* Release Thoracic Spinal Cord, Percutaneous Endoscopic Approach */
"00NY0ZZ"  =  "1"  /* Release Lumbar Spinal Cord, Open Approach */
"00NY3ZZ"  =  "1"  /* Release Lumbar Spinal Cord, Percutaneous Approach */
"00NY4ZZ"  =  "1"  /* Release Lumbar Spinal Cord, Percutaneous Endoscopic Approach */
"0PS304Z"  =  "1"  /* Reposition Cervical Vertebra with Internal Fixation Device, Open Approach */
"0PS30ZZ"  =  "1"  /* Reposition Cervical Vertebra, Open Approach */
"0PS334Z"  =  "1"  /* Reposition Cervical Vertebra with Internal Fixation Device, Percutaneous Approach */
"0PS33ZZ"  =  "1"  /* Reposition Cervical Vertebra, Percutaneous Approach */
"0PS344Z"  =  "1"  /* Reposition Cervical Vertebra with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0PS34ZZ"  =  "1"  /* Reposition Cervical Vertebra, Percutaneous Endoscopic Approach */
"0PS3XZZ"  =  "1"  /* Reposition Cervical Vertebra, External Approach */
"0PS404Z"  =  "1"  /* Reposition Thoracic Vertebra with Internal Fixation Device, Open Approach */
"0PS40ZZ"  =  "1"  /* Reposition Thoracic Vertebra, Open Approach */
"0PS434Z"  =  "1"  /* Reposition Thoracic Vertebra with Internal Fixation Device, Percutaneous Approach */
"0PS43ZZ"  =  "1"  /* Reposition Thoracic Vertebra, Percutaneous Approach */
"0PS444Z"  =  "1"  /* Reposition Thoracic Vertebra with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0PS44ZZ"  =  "1"  /* Reposition Thoracic Vertebra, Percutaneous Endoscopic Approach */
"0PS4XZZ"  =  "1"  /* Reposition Thoracic Vertebra, External Approach */
"0PU33JZ"  =  "1"  /* Supplement Cervical Vertebra with Synthetic Substitute, Percutaneous Approach */
"0PU34JZ"  =  "1"  /* Supplement Cervical Vertebra with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0PU43JZ"  =  "1"  /* Supplement Thoracic Vertebra with Synthetic Substitute, Percutaneous Approach */
"0PU44JZ"  =  "1"  /* Supplement Thoracic Vertebra with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0QS004Z"  =  "1"  /* Reposition Lumbar Vertebra with Internal Fixation Device, Open Approach */
"0QS00ZZ"  =  "1"  /* Reposition Lumbar Vertebra, Open Approach */
"0QS034Z"  =  "1"  /* Reposition Lumbar Vertebra with Internal Fixation Device, Percutaneous Approach */
"0QS03ZZ"  =  "1"  /* Reposition Lumbar Vertebra, Percutaneous Approach */
"0QS044Z"  =  "1"  /* Reposition Lumbar Vertebra with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0QS04ZZ"  =  "1"  /* Reposition Lumbar Vertebra, Percutaneous Endoscopic Approach */
"0QS0XZZ"  =  "1"  /* Reposition Lumbar Vertebra, External Approach */
"0QU03JZ"  =  "1"  /* Supplement Lumbar Vertebra with Synthetic Substitute, Percutaneous Approach */
"0QU04JZ"  =  "1"  /* Supplement Lumbar Vertebra with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0RB00ZZ"  =  "1"  /* Excision of Occipital-cervical Joint, Open Approach */
"0RB03ZZ"  =  "1"  /* Excision of Occipital-cervical Joint, Percutaneous Approach */
"0RB04ZZ"  =  "1"  /* Excision of Occipital-cervical Joint, Percutaneous Endoscopic Approach */
"0RB10ZZ"  =  "1"  /* Excision of Cervical Vertebral Joint, Open Approach */
"0RB13ZZ"  =  "1"  /* Excision of Cervical Vertebral Joint, Percutaneous Approach */
"0RB14ZZ"  =  "1"  /* Excision of Cervical Vertebral Joint, Percutaneous Endoscopic Approach */
"0RB40ZZ"  =  "1"  /* Excision of Cervicothoracic Vertebral Joint, Open Approach */
"0RB43ZZ"  =  "1"  /* Excision of Cervicothoracic Vertebral Joint, Percutaneous Approach */
"0RB44ZZ"  =  "1"  /* Excision of Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RB60ZZ"  =  "1"  /* Excision of Thoracic Vertebral Joint, Open Approach */
"0RB63ZZ"  =  "1"  /* Excision of Thoracic Vertebral Joint, Percutaneous Approach */
"0RB64ZZ"  =  "1"  /* Excision of Thoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RBA0ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Joint, Open Approach */
"0RBA3ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Joint, Percutaneous Approach */
"0RBA4ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0RG0070"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG0071"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG007J"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG00A0"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG00A1"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0RG00AJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0RG00J0"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG00J1"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG00JJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG00K0"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG00K1"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG00KJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG00Z0"  =  "1"  /* Fusion of Occipital-cervical Joint, Anterior Approach, Anterior Column, Open Approach */
"0RG00Z1"  =  "1"  /* Fusion of Occipital-cervical Joint, Posterior Approach, Posterior Column, Open Approach */
"0RG00ZJ"  =  "1"  /* Fusion of Occipital-cervical Joint, Posterior Approach, Anterior Column, Open Approach */
"0RG0370"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG0371"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG037J"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG03A0"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG03A1"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG03AJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG03J0"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG03J1"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG03JJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG03K0"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG03K1"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG03KJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG03Z0"  =  "1"  /* Fusion of Occipital-cervical Joint, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG03Z1"  =  "1"  /* Fusion of Occipital-cervical Joint, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG03ZJ"  =  "1"  /* Fusion of Occipital-cervical Joint, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG0470"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG0471"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG047J"  =  "1"  /* Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG04A0"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG04A1"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG04AJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG04J0"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG04J1"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG04JJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG04K0"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG04K1"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG04KJ"  =  "1"  /* Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG04Z0"  =  "1"  /* Fusion of Occipital-cervical Joint, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG04Z1"  =  "1"  /* Fusion of Occipital-cervical Joint, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG04ZJ"  =  "1"  /* Fusion of Occipital-cervical Joint, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG1070"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG1071"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG107J"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG10A0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG10A1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0RG10AJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0RG10J0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG10J1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG10JJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG10K0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG10K1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG10KJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG10Z0"  =  "1"  /* Fusion of Cervical Vertebral Joint, Anterior Approach, Anterior Column, Open Approach */
"0RG10Z1"  =  "1"  /* Fusion of Cervical Vertebral Joint, Posterior Approach, Posterior Column, Open Approach */
"0RG10ZJ"  =  "1"  /* Fusion of Cervical Vertebral Joint, Posterior Approach, Anterior Column, Open Approach */
"0RG1370"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG1371"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG137J"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG13A0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG13A1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG13AJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG13J0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG13J1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG13JJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG13K0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG13K1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG13KJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG13Z0"  =  "1"  /* Fusion of Cervical Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG13Z1"  =  "1"  /* Fusion of Cervical Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG13ZJ"  =  "1"  /* Fusion of Cervical Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG1470"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG1471"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG147J"  =  "1"  /* Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG14A0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG14A1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG14AJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG14J0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG14J1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG14JJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG14K0"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG14K1"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG14KJ"  =  "1"  /* Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG14Z0"  =  "1"  /* Fusion of Cervical Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG14Z1"  =  "1"  /* Fusion of Cervical Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG14ZJ"  =  "1"  /* Fusion of Cervical Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG2070"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG2071"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG207J"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG20A0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG20A1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0RG20AJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0RG20J0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG20J1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG20JJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG20K0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG20K1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG20KJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG20Z0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Anterior Approach, Anterior Column, Open Approach */
"0RG20Z1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Posterior Approach, Posterior Column, Open Approach */
"0RG20ZJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Posterior Approach, Anterior Column, Open Approach */
"0RG2370"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG2371"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG237J"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG23A0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG23A1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG23AJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG23J0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG23J1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG23JJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG23K0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG23K1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG23KJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG23Z0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG23Z1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG23ZJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG2470"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG2471"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG247J"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG24A0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG24A1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG24AJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG24J0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG24J1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG24JJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG24K0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG24K1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG24KJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG24Z0"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG24Z1"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG24ZJ"  =  "1"  /* Fusion of 2 or more Cervical Vertebral Joints, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG4070"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG4071"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG407J"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG40A0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG40A1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0RG40AJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0RG40J0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG40J1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG40JJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG40K0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG40K1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG40KJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG40Z0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Anterior Approach, Anterior Column, Open Approach */
"0RG40Z1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Posterior Approach, Posterior Column, Open Approach */
"0RG40ZJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Posterior Approach, Anterior Column, Open Approach */
"0RG4370"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG4371"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG437J"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG43A0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG43A1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG43AJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG43J0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG43J1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG43JJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG43K0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG43K1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG43KJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG43Z0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG43Z1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG43ZJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG4470"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG4471"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG447J"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG44A0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG44A1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG44AJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG44J0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG44J1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG44JJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG44K0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG44K1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG44KJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG44Z0"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG44Z1"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG44ZJ"  =  "1"  /* Fusion of Cervicothoracic Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG6070"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG6071"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG607J"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG60A0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG60A1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0RG60AJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0RG60J0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG60J1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG60JJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG60K0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG60K1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG60KJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG60Z0"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Anterior Approach, Anterior Column, Open Approach */
"0RG60Z1"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Posterior Approach, Posterior Column, Open Approach */
"0RG60ZJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Posterior Approach, Anterior Column, Open Approach */
"0RG6370"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG6371"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG637J"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG63A0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG63A1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG63AJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG63J0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG63J1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG63JJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG63K0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG63K1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG63KJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG63Z0"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG63Z1"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG63ZJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG6470"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG6471"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG647J"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64A0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64A1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG64AJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64J0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64J1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG64JJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64K0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64K1"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG64KJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64Z0"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64Z1"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG64ZJ"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG7070"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG7071"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG707J"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG70A0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG70A1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0RG70AJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0RG70J0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG70J1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG70JJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG70K0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG70K1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG70KJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG70Z0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Open Approach */
"0RG70Z1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Posterior Approach, Posterior Column, Open Approach */
"0RG70ZJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Posterior Approach, Anterior Column, Open Approach */
"0RG7370"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG7371"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG737J"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG73A0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG73A1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG73AJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG73J0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG73J1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG73JJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG73K0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG73K1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG73KJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG73Z0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG73Z1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG73ZJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG7470"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG7471"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG747J"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74A0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74A1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG74AJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74J0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74J1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG74JJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74K0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74K1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG74KJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74Z0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74Z1"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG74ZJ"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG8070"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG8071"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG807J"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG80A0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG80A1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0RG80AJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0RG80J0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG80J1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG80JJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG80K0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG80K1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RG80KJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RG80Z0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Open Approach */
"0RG80Z1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Posterior Approach, Posterior Column, Open Approach */
"0RG80ZJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Posterior Approach, Anterior Column, Open Approach */
"0RG8370"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG8371"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG837J"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG83A0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG83A1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG83AJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG83J0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG83J1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG83JJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG83K0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG83K1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG83KJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG83Z0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG83Z1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RG83ZJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RG8470"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG8471"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG847J"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84A0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84A1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG84AJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84J0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84J1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG84JJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84K0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84K1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG84KJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84Z0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84Z1"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RG84ZJ"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA070"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RGA071"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RGA07J"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RGA0A0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RGA0A1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0RGA0AJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0RGA0J0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RGA0J1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RGA0JJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RGA0K0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RGA0K1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0RGA0KJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0RGA0Z0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Anterior Approach, Anterior Column, Open Approach */
"0RGA0Z1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Posterior Approach, Posterior Column, Open Approach */
"0RGA0ZJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Posterior Approach, Anterior Column, Open Approach */
"0RGA370"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RGA371"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RGA37J"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3A0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3A1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RGA3AJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3J0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3J1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RGA3JJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3K0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3K1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RGA3KJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3Z0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3Z1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Approach */
"0RGA3ZJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Approach */
"0RGA470"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA471"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RGA47J"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4A0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4A1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RGA4AJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4J0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4J1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RGA4JJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4K0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4K1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RGA4KJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4Z0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4Z1"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0RGA4ZJ"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RH00BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Occipital-cervical Joint, Open Approach */
"0RH00CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Occipital-cervical Joint, Open Approach */
"0RH00DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Occipital-cervical Joint, Open Approach */
"0RH03BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Occipital-cervical Joint, Percutaneous Approach */
"0RH03CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Occipital-cervical Joint, Percutaneous Approach */
"0RH03DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Occipital-cervical Joint, Percutaneous Approach */
"0RH04BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Occipital-cervical Joint, Percutaneous Endoscopic Approach */
"0RH04CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Occipital-cervical Joint, Percutaneous Endoscopic Approach */
"0RH04DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Occipital-cervical Joint, Percutaneous Endoscopic Approach */
"0RH10BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Cervical Vertebral Joint, Open Approach */
"0RH10CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Cervical Vertebral Joint, Open Approach */
"0RH10DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Cervical Vertebral Joint, Open Approach */
"0RH13BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Cervical Vertebral Joint, Percutaneous Approach */
"0RH13CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Cervical Vertebral Joint, Percutaneous Approach */
"0RH13DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Cervical Vertebral Joint, Percutaneous Approach */
"0RH14BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Cervical Vertebral Joint, Percutaneous Endoscopic Approach */
"0RH14CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Cervical Vertebral Joint, Percutaneous Endoscopic Approach */
"0RH14DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Cervical Vertebral Joint, Percutaneous Endoscopic Approach */
"0RH40BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Open Approach */
"0RH40CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Open Approach */
"0RH40DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Open Approach */
"0RH43BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Percutaneous Approach */
"0RH43CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Percutaneous Approach */
"0RH43DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Percutaneous Approach */
"0RH44BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RH44CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RH44DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RH60BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Thoracic Vertebral Joint, Open Approach */
"0RH60CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Thoracic Vertebral Joint, Open Approach */
"0RH60DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Thoracic Vertebral Joint, Open Approach */
"0RH63BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Thoracic Vertebral Joint, Percutaneous Approach */
"0RH63CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Thoracic Vertebral Joint, Percutaneous Approach */
"0RH63DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Thoracic Vertebral Joint, Percutaneous Approach */
"0RH64BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Thoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RH64CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Thoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RH64DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Thoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RHA0BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Open Approach */
"0RHA0CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Open Approach */
"0RHA0DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Open Approach */
"0RHA3BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Percutaneous Approach */
"0RHA3CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Percutaneous Approach */
"0RHA3DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Percutaneous Approach */
"0RHA4BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0RHA4CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0RHA4DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0RP004Z"  =  "1"  /* Removal of Internal Fixation Device from Occipital-cervical Joint, Open Approach */
"0RP034Z"  =  "1"  /* Removal of Internal Fixation Device from Occipital-cervical Joint, Percutaneous Approach */
"0RP044Z"  =  "1"  /* Removal of Internal Fixation Device from Occipital-cervical Joint, Percutaneous Endoscopic Approach */
"0RP104Z"  =  "1"  /* Removal of Internal Fixation Device from Cervical Vertebral Joint, Open Approach */
"0RP134Z"  =  "1"  /* Removal of Internal Fixation Device from Cervical Vertebral Joint, Percutaneous Approach */
"0RP144Z"  =  "1"  /* Removal of Internal Fixation Device from Cervical Vertebral Joint, Percutaneous Endoscopic Approach */
"0RP30JZ"  =  "1"  /* Removal of Synthetic Substitute from Cervical Vertebral Disc, Open Approach */
"0RP404Z"  =  "1"  /* Removal of Internal Fixation Device from Cervicothoracic Vertebral Joint, Open Approach */
"0RP434Z"  =  "1"  /* Removal of Internal Fixation Device from Cervicothoracic Vertebral Joint, Percutaneous Approach */
"0RP444Z"  =  "1"  /* Removal of Internal Fixation Device from Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RP50JZ"  =  "1"  /* Removal of Synthetic Substitute from Cervicothoracic Vertebral Disc, Open Approach */
"0RP604Z"  =  "1"  /* Removal of Internal Fixation Device from Thoracic Vertebral Joint, Open Approach */
"0RP634Z"  =  "1"  /* Removal of Internal Fixation Device from Thoracic Vertebral Joint, Percutaneous Approach */
"0RP644Z"  =  "1"  /* Removal of Internal Fixation Device from Thoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RP90JZ"  =  "1"  /* Removal of Synthetic Substitute from Thoracic Vertebral Disc, Open Approach */
"0RPA04Z"  =  "1"  /* Removal of Internal Fixation Device from Thoracolumbar Vertebral Joint, Open Approach */
"0RPA34Z"  =  "1"  /* Removal of Internal Fixation Device from Thoracolumbar Vertebral Joint, Percutaneous Approach */
"0RPA44Z"  =  "1"  /* Removal of Internal Fixation Device from Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0RPB0JZ"  =  "1"  /* Removal of Synthetic Substitute from Thoracolumbar Vertebral Disc, Open Approach */
"0RQ30ZZ"  =  "1"  /* Repair Cervical Vertebral Disc, Open Approach */
"0RQ90ZZ"  =  "1"  /* Repair Thoracic Vertebral Disc, Open Approach */
"0RQB0ZZ"  =  "1"  /* Repair Thoracolumbar Vertebral Disc, Open Approach */
"0RR30JZ"  =  "1"  /* Replacement of Cervical Vertebral Disc with Synthetic Substitute, Open Approach */
"0RR50JZ"  =  "1"  /* Replacement of Cervicothoracic Vertebral Disc with Synthetic Substitute, Open Approach */
"0RR90JZ"  =  "1"  /* Replacement of Thoracic Vertebral Disc with Synthetic Substitute, Open Approach */
"0RRB0JZ"  =  "1"  /* Replacement of Thoracolumbar Vertebral Disc with Synthetic Substitute, Open Approach */
"0RU00JZ"  =  "1"  /* Supplement Occipital-cervical Joint with Synthetic Substitute, Open Approach */
"0RU03JZ"  =  "1"  /* Supplement Occipital-cervical Joint with Synthetic Substitute, Percutaneous Approach */
"0RU04JZ"  =  "1"  /* Supplement Occipital-cervical Joint with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0RU10JZ"  =  "1"  /* Supplement Cervical Vertebral Joint with Synthetic Substitute, Open Approach */
"0RU13JZ"  =  "1"  /* Supplement Cervical Vertebral Joint with Synthetic Substitute, Percutaneous Approach */
"0RU14JZ"  =  "1"  /* Supplement Cervical Vertebral Joint with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0RU307Z"  =  "1"  /* Supplement Cervical Vertebral Disc with Autologous Tissue Substitute, Open Approach */
"0RU30JZ"  =  "1"  /* Supplement Cervical Vertebral Disc with Synthetic Substitute, Open Approach */
"0RU30KZ"  =  "1"  /* Supplement Cervical Vertebral Disc with Nonautologous Tissue Substitute, Open Approach */
"0RU337Z"  =  "1"  /* Supplement Cervical Vertebral Disc with Autologous Tissue Substitute, Percutaneous Approach */
"0RU33JZ"  =  "1"  /* Supplement Cervical Vertebral Disc with Synthetic Substitute, Percutaneous Approach */
"0RU33KZ"  =  "1"  /* Supplement Cervical Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Approach */
"0RU347Z"  =  "1"  /* Supplement Cervical Vertebral Disc with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0RU34JZ"  =  "1"  /* Supplement Cervical Vertebral Disc with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0RU34KZ"  =  "1"  /* Supplement Cervical Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0RU40JZ"  =  "1"  /* Supplement Cervicothoracic Vertebral Joint with Synthetic Substitute, Open Approach */
"0RU43JZ"  =  "1"  /* Supplement Cervicothoracic Vertebral Joint with Synthetic Substitute, Percutaneous Approach */
"0RU44JZ"  =  "1"  /* Supplement Cervicothoracic Vertebral Joint with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0RU50JZ"  =  "1"  /* Supplement Cervicothoracic Vertebral Disc with Synthetic Substitute, Open Approach */
"0RU53JZ"  =  "1"  /* Supplement Cervicothoracic Vertebral Disc with Synthetic Substitute, Percutaneous Approach */
"0RU54JZ"  =  "1"  /* Supplement Cervicothoracic Vertebral Disc with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0RU60JZ"  =  "1"  /* Supplement Thoracic Vertebral Joint with Synthetic Substitute, Open Approach */
"0RU63JZ"  =  "1"  /* Supplement Thoracic Vertebral Joint with Synthetic Substitute, Percutaneous Approach */
"0RU64JZ"  =  "1"  /* Supplement Thoracic Vertebral Joint with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0RU907Z"  =  "1"  /* Supplement Thoracic Vertebral Disc with Autologous Tissue Substitute, Open Approach */
"0RU90JZ"  =  "1"  /* Supplement Thoracic Vertebral Disc with Synthetic Substitute, Open Approach */
"0RU90KZ"  =  "1"  /* Supplement Thoracic Vertebral Disc with Nonautologous Tissue Substitute, Open Approach */
"0RU937Z"  =  "1"  /* Supplement Thoracic Vertebral Disc with Autologous Tissue Substitute, Percutaneous Approach */
"0RU93JZ"  =  "1"  /* Supplement Thoracic Vertebral Disc with Synthetic Substitute, Percutaneous Approach */
"0RU93KZ"  =  "1"  /* Supplement Thoracic Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Approach */
"0RU947Z"  =  "1"  /* Supplement Thoracic Vertebral Disc with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0RU94JZ"  =  "1"  /* Supplement Thoracic Vertebral Disc with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0RU94KZ"  =  "1"  /* Supplement Thoracic Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0RUA0JZ"  =  "1"  /* Supplement Thoracolumbar Vertebral Joint with Synthetic Substitute, Open Approach */
"0RUA3JZ"  =  "1"  /* Supplement Thoracolumbar Vertebral Joint with Synthetic Substitute, Percutaneous Approach */
"0RUA4JZ"  =  "1"  /* Supplement Thoracolumbar Vertebral Joint with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0RUB07Z"  =  "1"  /* Supplement Thoracolumbar Vertebral Disc with Autologous Tissue Substitute, Open Approach */
"0RUB0JZ"  =  "1"  /* Supplement Thoracolumbar Vertebral Disc with Synthetic Substitute, Open Approach */
"0RUB0KZ"  =  "1"  /* Supplement Thoracolumbar Vertebral Disc with Nonautologous Tissue Substitute, Open Approach */
"0RUB37Z"  =  "1"  /* Supplement Thoracolumbar Vertebral Disc with Autologous Tissue Substitute, Percutaneous Approach */
"0RUB3JZ"  =  "1"  /* Supplement Thoracolumbar Vertebral Disc with Synthetic Substitute, Percutaneous Approach */
"0RUB3KZ"  =  "1"  /* Supplement Thoracolumbar Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Approach */
"0RUB47Z"  =  "1"  /* Supplement Thoracolumbar Vertebral Disc with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0RUB4JZ"  =  "1"  /* Supplement Thoracolumbar Vertebral Disc with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0RUB4KZ"  =  "1"  /* Supplement Thoracolumbar Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0RW004Z"  =  "1"  /* Revision of Internal Fixation Device in Occipital-cervical Joint, Open Approach */
"0RW00JZ"  =  "1"  /* Revision of Synthetic Substitute in Occipital-cervical Joint, Open Approach */
"0RW034Z"  =  "1"  /* Revision of Internal Fixation Device in Occipital-cervical Joint, Percutaneous Approach */
"0RW03JZ"  =  "1"  /* Revision of Synthetic Substitute in Occipital-cervical Joint, Percutaneous Approach */
"0RW044Z"  =  "1"  /* Revision of Internal Fixation Device in Occipital-cervical Joint, Percutaneous Endoscopic Approach */
"0RW04JZ"  =  "1"  /* Revision of Synthetic Substitute in Occipital-cervical Joint, Percutaneous Endoscopic Approach */
"0RW104Z"  =  "1"  /* Revision of Internal Fixation Device in Cervical Vertebral Joint, Open Approach */
"0RW10JZ"  =  "1"  /* Revision of Synthetic Substitute in Cervical Vertebral Joint, Open Approach */
"0RW134Z"  =  "1"  /* Revision of Internal Fixation Device in Cervical Vertebral Joint, Percutaneous Approach */
"0RW13JZ"  =  "1"  /* Revision of Synthetic Substitute in Cervical Vertebral Joint, Percutaneous Approach */
"0RW144Z"  =  "1"  /* Revision of Internal Fixation Device in Cervical Vertebral Joint, Percutaneous Endoscopic Approach */
"0RW14JZ"  =  "1"  /* Revision of Synthetic Substitute in Cervical Vertebral Joint, Percutaneous Endoscopic Approach */
"0RW30JZ"  =  "1"  /* Revision of Synthetic Substitute in Cervical Vertebral Disc, Open Approach */
"0RW33JZ"  =  "1"  /* Revision of Synthetic Substitute in Cervical Vertebral Disc, Percutaneous Approach */
"0RW34JZ"  =  "1"  /* Revision of Synthetic Substitute in Cervical Vertebral Disc, Percutaneous Endoscopic Approach */
"0RW404Z"  =  "1"  /* Revision of Internal Fixation Device in Cervicothoracic Vertebral Joint, Open Approach */
"0RW40JZ"  =  "1"  /* Revision of Synthetic Substitute in Cervicothoracic Vertebral Joint, Open Approach */
"0RW434Z"  =  "1"  /* Revision of Internal Fixation Device in Cervicothoracic Vertebral Joint, Percutaneous Approach */
"0RW43JZ"  =  "1"  /* Revision of Synthetic Substitute in Cervicothoracic Vertebral Joint, Percutaneous Approach */
"0RW444Z"  =  "1"  /* Revision of Internal Fixation Device in Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RW44JZ"  =  "1"  /* Revision of Synthetic Substitute in Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RW50JZ"  =  "1"  /* Revision of Synthetic Substitute in Cervicothoracic Vertebral Disc, Open Approach */
"0RW53JZ"  =  "1"  /* Revision of Synthetic Substitute in Cervicothoracic Vertebral Disc, Percutaneous Approach */
"0RW54JZ"  =  "1"  /* Revision of Synthetic Substitute in Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach */
"0RW604Z"  =  "1"  /* Revision of Internal Fixation Device in Thoracic Vertebral Joint, Open Approach */
"0RW60JZ"  =  "1"  /* Revision of Synthetic Substitute in Thoracic Vertebral Joint, Open Approach */
"0RW634Z"  =  "1"  /* Revision of Internal Fixation Device in Thoracic Vertebral Joint, Percutaneous Approach */
"0RW63JZ"  =  "1"  /* Revision of Synthetic Substitute in Thoracic Vertebral Joint, Percutaneous Approach */
"0RW644Z"  =  "1"  /* Revision of Internal Fixation Device in Thoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RW64JZ"  =  "1"  /* Revision of Synthetic Substitute in Thoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RW90JZ"  =  "1"  /* Revision of Synthetic Substitute in Thoracic Vertebral Disc, Open Approach */
"0RW93JZ"  =  "1"  /* Revision of Synthetic Substitute in Thoracic Vertebral Disc, Percutaneous Approach */
"0RW94JZ"  =  "1"  /* Revision of Synthetic Substitute in Thoracic Vertebral Disc, Percutaneous Endoscopic Approach */
"0RWA04Z"  =  "1"  /* Revision of Internal Fixation Device in Thoracolumbar Vertebral Joint, Open Approach */
"0RWA0JZ"  =  "1"  /* Revision of Synthetic Substitute in Thoracolumbar Vertebral Joint, Open Approach */
"0RWA34Z"  =  "1"  /* Revision of Internal Fixation Device in Thoracolumbar Vertebral Joint, Percutaneous Approach */
"0RWA3JZ"  =  "1"  /* Revision of Synthetic Substitute in Thoracolumbar Vertebral Joint, Percutaneous Approach */
"0RWA44Z"  =  "1"  /* Revision of Internal Fixation Device in Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0RWA4JZ"  =  "1"  /* Revision of Synthetic Substitute in Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0RWB0JZ"  =  "1"  /* Revision of Synthetic Substitute in Thoracolumbar Vertebral Disc, Open Approach */
"0RWB3JZ"  =  "1"  /* Revision of Synthetic Substitute in Thoracolumbar Vertebral Disc, Percutaneous Approach */
"0RWB4JZ"  =  "1"  /* Revision of Synthetic Substitute in Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach */
"0SB00ZZ"  =  "1"  /* Excision of Lumbar Vertebral Joint, Open Approach */
"0SB03ZZ"  =  "1"  /* Excision of Lumbar Vertebral Joint, Percutaneous Approach */
"0SB04ZZ"  =  "1"  /* Excision of Lumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0SB30ZZ"  =  "1"  /* Excision of Lumbosacral Joint, Open Approach */
"0SB33ZZ"  =  "1"  /* Excision of Lumbosacral Joint, Percutaneous Approach */
"0SB34ZZ"  =  "1"  /* Excision of Lumbosacral Joint, Percutaneous Endoscopic Approach */
"0SG0070"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG0071"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG007J"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG00A0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0SG00A1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0SG00AJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0SG00J0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG00J1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG00JJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG00K0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG00K1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG00KJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG00Z0"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Anterior Approach, Anterior Column, Open Approach */
"0SG00Z1"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Posterior Approach, Posterior Column, Open Approach */
"0SG00ZJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Posterior Approach, Anterior Column, Open Approach */
"0SG0370"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG0371"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG037J"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG03A0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG03A1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG03AJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG03J0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG03J1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG03JJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG03K0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG03K1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG03KJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG03Z0"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG03Z1"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG03ZJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG0470"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG0471"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG047J"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG04A0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG04A1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG04AJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG04J0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG04J1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG04JJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG04K0"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG04K1"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG04KJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG04Z0"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG04Z1"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG04ZJ"  =  "1"  /* Fusion of Lumbar Vertebral Joint, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG1070"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG1071"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG107J"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG10A0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0SG10A1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0SG10AJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0SG10J0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG10J1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG10JJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG10K0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG10K1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG10KJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG10Z0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Anterior Approach, Anterior Column, Open Approach */
"0SG10Z1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Posterior Approach, Posterior Column, Open Approach */
"0SG10ZJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Posterior Approach, Anterior Column, Open Approach */
"0SG1370"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG1371"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG137J"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG13A0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG13A1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG13AJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG13J0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG13J1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG13JJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG13K0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG13K1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG13KJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG13Z0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG13Z1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG13ZJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG1470"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG1471"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG147J"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG14A0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG14A1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG14AJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG14J0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG14J1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG14JJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG14K0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG14K1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG14KJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG14Z0"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG14Z1"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG14ZJ"  =  "1"  /* Fusion of 2 or more Lumbar Vertebral Joints, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG3070"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG3071"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG307J"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG30A0"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0SG30A1"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Open Approach */
"0SG30AJ"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach */
"0SG30J0"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG30J1"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG30JJ"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG30K0"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0SG30K1"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach */
"0SG30KJ"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach */
"0SG30Z0"  =  "1"  /* Fusion of Lumbosacral Joint, Anterior Approach, Anterior Column, Open Approach */
"0SG30Z1"  =  "1"  /* Fusion of Lumbosacral Joint, Posterior Approach, Posterior Column, Open Approach */
"0SG30ZJ"  =  "1"  /* Fusion of Lumbosacral Joint, Posterior Approach, Anterior Column, Open Approach */
"0SG3370"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG3371"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG337J"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG33A0"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG33A1"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG33AJ"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG33J0"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG33J1"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG33JJ"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG33K0"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG33K1"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG33KJ"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG33Z0"  =  "1"  /* Fusion of Lumbosacral Joint, Anterior Approach, Anterior Column, Percutaneous Approach */
"0SG33Z1"  =  "1"  /* Fusion of Lumbosacral Joint, Posterior Approach, Posterior Column, Percutaneous Approach */
"0SG33ZJ"  =  "1"  /* Fusion of Lumbosacral Joint, Posterior Approach, Anterior Column, Percutaneous Approach */
"0SG3470"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG3471"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG347J"  =  "1"  /* Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG34A0"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG34A1"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG34AJ"  =  "1"  /* Fusion of Lumbosacral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG34J0"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG34J1"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG34JJ"  =  "1"  /* Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG34K0"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG34K1"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG34KJ"  =  "1"  /* Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG34Z0"  =  "1"  /* Fusion of Lumbosacral Joint, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SG34Z1"  =  "1"  /* Fusion of Lumbosacral Joint, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach */
"0SG34ZJ"  =  "1"  /* Fusion of Lumbosacral Joint, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0SH00BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Lumbar Vertebral Joint, Open Approach */
"0SH00CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Lumbar Vertebral Joint, Open Approach */
"0SH00DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Lumbar Vertebral Joint, Open Approach */
"0SH03BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Lumbar Vertebral Joint, Percutaneous Approach */
"0SH03CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Lumbar Vertebral Joint, Percutaneous Approach */
"0SH03DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Lumbar Vertebral Joint, Percutaneous Approach */
"0SH04BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Lumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0SH04CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Lumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0SH04DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Lumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0SH30BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Lumbosacral Joint, Open Approach */
"0SH30CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Lumbosacral Joint, Open Approach */
"0SH30DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Lumbosacral Joint, Open Approach */
"0SH33BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Lumbosacral Joint, Percutaneous Approach */
"0SH33CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Lumbosacral Joint, Percutaneous Approach */
"0SH33DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Lumbosacral Joint, Percutaneous Approach */
"0SH34BZ"  =  "1"  /* Insertion of Interspinous Process Spinal Stabilization Device into Lumbosacral Joint, Percutaneous Endoscopic Approach */
"0SH34CZ"  =  "1"  /* Insertion of Pedicle-Based Spinal Stabilization Device into Lumbosacral Joint, Percutaneous Endoscopic Approach */
"0SH34DZ"  =  "1"  /* Insertion of Facet Replacement Spinal Stabilization Device into Lumbosacral Joint, Percutaneous Endoscopic Approach */
"0SP004Z"  =  "1"  /* Removal of Internal Fixation Device from Lumbar Vertebral Joint, Open Approach */
"0SP034Z"  =  "1"  /* Removal of Internal Fixation Device from Lumbar Vertebral Joint, Percutaneous Approach */
"0SP044Z"  =  "1"  /* Removal of Internal Fixation Device from Lumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0SP20JZ"  =  "1"  /* Removal of Synthetic Substitute from Lumbar Vertebral Disc, Open Approach */
"0SP304Z"  =  "1"  /* Removal of Internal Fixation Device from Lumbosacral Joint, Open Approach */
"0SP334Z"  =  "1"  /* Removal of Internal Fixation Device from Lumbosacral Joint, Percutaneous Approach */
"0SP344Z"  =  "1"  /* Removal of Internal Fixation Device from Lumbosacral Joint, Percutaneous Endoscopic Approach */
"0SP40JZ"  =  "1"  /* Removal of Synthetic Substitute from Lumbosacral Disc, Open Approach */
"0SQ20ZZ"  =  "1"  /* Repair Lumbar Vertebral Disc, Open Approach */
"0SQ40ZZ"  =  "1"  /* Repair Lumbosacral Disc, Open Approach */
"0SR20JZ"  =  "1"  /* Replacement of Lumbar Vertebral Disc with Synthetic Substitute, Open Approach */
"0SR40JZ"  =  "1"  /* Replacement of Lumbosacral Disc with Synthetic Substitute, Open Approach */
"0SU00JZ"  =  "1"  /* Supplement Lumbar Vertebral Joint with Synthetic Substitute, Open Approach */
"0SU03JZ"  =  "1"  /* Supplement Lumbar Vertebral Joint with Synthetic Substitute, Percutaneous Approach */
"0SU04JZ"  =  "1"  /* Supplement Lumbar Vertebral Joint with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0SU207Z"  =  "1"  /* Supplement Lumbar Vertebral Disc with Autologous Tissue Substitute, Open Approach */
"0SU20JZ"  =  "1"  /* Supplement Lumbar Vertebral Disc with Synthetic Substitute, Open Approach */
"0SU20KZ"  =  "1"  /* Supplement Lumbar Vertebral Disc with Nonautologous Tissue Substitute, Open Approach */
"0SU237Z"  =  "1"  /* Supplement Lumbar Vertebral Disc with Autologous Tissue Substitute, Percutaneous Approach */
"0SU23JZ"  =  "1"  /* Supplement Lumbar Vertebral Disc with Synthetic Substitute, Percutaneous Approach */
"0SU23KZ"  =  "1"  /* Supplement Lumbar Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Approach */
"0SU247Z"  =  "1"  /* Supplement Lumbar Vertebral Disc with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0SU24JZ"  =  "1"  /* Supplement Lumbar Vertebral Disc with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0SU24KZ"  =  "1"  /* Supplement Lumbar Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0SU30JZ"  =  "1"  /* Supplement Lumbosacral Joint with Synthetic Substitute, Open Approach */
"0SU33JZ"  =  "1"  /* Supplement Lumbosacral Joint with Synthetic Substitute, Percutaneous Approach */
"0SU34JZ"  =  "1"  /* Supplement Lumbosacral Joint with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0SU407Z"  =  "1"  /* Supplement Lumbosacral Disc with Autologous Tissue Substitute, Open Approach */
"0SU40JZ"  =  "1"  /* Supplement Lumbosacral Disc with Synthetic Substitute, Open Approach */
"0SU40KZ"  =  "1"  /* Supplement Lumbosacral Disc with Nonautologous Tissue Substitute, Open Approach */
"0SU437Z"  =  "1"  /* Supplement Lumbosacral Disc with Autologous Tissue Substitute, Percutaneous Approach */
"0SU43JZ"  =  "1"  /* Supplement Lumbosacral Disc with Synthetic Substitute, Percutaneous Approach */
"0SU43KZ"  =  "1"  /* Supplement Lumbosacral Disc with Nonautologous Tissue Substitute, Percutaneous Approach */
"0SU447Z"  =  "1"  /* Supplement Lumbosacral Disc with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0SU44JZ"  =  "1"  /* Supplement Lumbosacral Disc with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0SU44KZ"  =  "1"  /* Supplement Lumbosacral Disc with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0SW004Z"  =  "1"  /* Revision of Internal Fixation Device in Lumbar Vertebral Joint, Open Approach */
"0SW00JZ"  =  "1"  /* Revision of Synthetic Substitute in Lumbar Vertebral Joint, Open Approach */
"0SW034Z"  =  "1"  /* Revision of Internal Fixation Device in Lumbar Vertebral Joint, Percutaneous Approach */
"0SW03JZ"  =  "1"  /* Revision of Synthetic Substitute in Lumbar Vertebral Joint, Percutaneous Approach */
"0SW044Z"  =  "1"  /* Revision of Internal Fixation Device in Lumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0SW04JZ"  =  "1"  /* Revision of Synthetic Substitute in Lumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0SW20JZ"  =  "1"  /* Revision of Synthetic Substitute in Lumbar Vertebral Disc, Open Approach */
"0SW23JZ"  =  "1"  /* Revision of Synthetic Substitute in Lumbar Vertebral Disc, Percutaneous Approach */
"0SW24JZ"  =  "1"  /* Revision of Synthetic Substitute in Lumbar Vertebral Disc, Percutaneous Endoscopic Approach */
"0SW304Z"  =  "1"  /* Revision of Internal Fixation Device in Lumbosacral Joint, Open Approach */
"0SW30JZ"  =  "1"  /* Revision of Synthetic Substitute in Lumbosacral Joint, Open Approach */
"0SW334Z"  =  "1"  /* Revision of Internal Fixation Device in Lumbosacral Joint, Percutaneous Approach */
"0SW33JZ"  =  "1"  /* Revision of Synthetic Substitute in Lumbosacral Joint, Percutaneous Approach */
"0SW344Z"  =  "1"  /* Revision of Internal Fixation Device in Lumbosacral Joint, Percutaneous Endoscopic Approach */
"0SW34JZ"  =  "1"  /* Revision of Synthetic Substitute in Lumbosacral Joint, Percutaneous Endoscopic Approach */
"0SW40JZ"  =  "1"  /* Revision of Synthetic Substitute in Lumbosacral Disc, Open Approach */
"0SW43JZ"  =  "1"  /* Revision of Synthetic Substitute in Lumbosacral Disc, Percutaneous Approach */
"0SW44JZ"  =  "1"  /* Revision of Synthetic Substitute in Lumbosacral Disc, Percutaneous Endoscopic Approach */
OTHER   = "0" ;


 /* ------------------------------ */
 /* PRESSURE ULCER                 */
 /*   PROVIDER INDICATOR / PDI # 2 */
 /* ------------------------------ */

 /* ICD-10-CM DIAGNOSIS CODES: */

/* VALUE $DECUBID  merged with DECUBVD */

 /* EXCLUDE: STAGE I OR STAGE II */
 /* ICD-10-CM DIAGNOSIS CODES: */

VALUE $DECUBVD
"L89000"  =  "1"  /* Pressure ulcer of unspecified elbow, unstageable */
"L89003"  =  "1"  /* Pressure ulcer of unspecified elbow, stage 3 */
"L89004"  =  "1"  /* Pressure ulcer of unspecified elbow, stage 4 */
"L89010"  =  "1"  /* Pressure ulcer of right elbow, unstageable */
"L89013"  =  "1"  /* Pressure ulcer of right elbow, stage 3 */
"L89014"  =  "1"  /* Pressure ulcer of right elbow, stage 4 */
"L89020"  =  "1"  /* Pressure ulcer of left elbow, unstageable */
"L89023"  =  "1"  /* Pressure ulcer of left elbow, stage 3 */
"L89024"  =  "1"  /* Pressure ulcer of left elbow, stage 4 */
"L89100"  =  "1"  /* Pressure ulcer of unspecified part of back, unstageable */
"L89103"  =  "1"  /* Pressure ulcer of unspecified part of back, stage 3 */
"L89104"  =  "1"  /* Pressure ulcer of unspecified part of back, stage 4 */
"L89110"  =  "1"  /* Pressure ulcer of right upper back, unstageable */
"L89113"  =  "1"  /* Pressure ulcer of right upper back, stage 3 */
"L89114"  =  "1"  /* Pressure ulcer of right upper back, stage 4 */
"L89120"  =  "1"  /* Pressure ulcer of left upper back, unstageable */
"L89123"  =  "1"  /* Pressure ulcer of left upper back, stage 3 */
"L89124"  =  "1"  /* Pressure ulcer of left upper back, stage 4 */
"L89130"  =  "1"  /* Pressure ulcer of right lower back, unstageable */
"L89133"  =  "1"  /* Pressure ulcer of right lower back, stage 3 */
"L89134"  =  "1"  /* Pressure ulcer of right lower back, stage 4 */
"L89140"  =  "1"  /* Pressure ulcer of left lower back, unstageable */
"L89143"  =  "1"  /* Pressure ulcer of left lower back, stage 3 */
"L89144"  =  "1"  /* Pressure ulcer of left lower back, stage 4 */
"L89150"  =  "1"  /* Pressure ulcer of sacral region, unstageable */
"L89153"  =  "1"  /* Pressure ulcer of sacral region, stage 3 */
"L89154"  =  "1"  /* Pressure ulcer of sacral region, stage 4 */
"L89200"  =  "1"  /* Pressure ulcer of unspecified hip, unstageable */
"L89203"  =  "1"  /* Pressure ulcer of unspecified hip, stage 3 */
"L89204"  =  "1"  /* Pressure ulcer of unspecified hip, stage 4 */
"L89210"  =  "1"  /* Pressure ulcer of right hip, unstageable */
"L89213"  =  "1"  /* Pressure ulcer of right hip, stage 3 */
"L89214"  =  "1"  /* Pressure ulcer of right hip, stage 4 */
"L89220"  =  "1"  /* Pressure ulcer of left hip, unstageable */
"L89223"  =  "1"  /* Pressure ulcer of left hip, stage 3 */
"L89224"  =  "1"  /* Pressure ulcer of left hip, stage 4 */
"L89300"  =  "1"  /* Pressure ulcer of unspecified buttock, unstageable */
"L89303"  =  "1"  /* Pressure ulcer of unspecified buttock, stage 3 */
"L89304"  =  "1"  /* Pressure ulcer of unspecified buttock, stage 4 */
"L89310"  =  "1"  /* Pressure ulcer of right buttock, unstageable */
"L89313"  =  "1"  /* Pressure ulcer of right buttock, stage 3 */
"L89314"  =  "1"  /* Pressure ulcer of right buttock, stage 4 */
"L89320"  =  "1"  /* Pressure ulcer of left buttock, unstageable */
"L89323"  =  "1"  /* Pressure ulcer of left buttock, stage 3 */
"L89324"  =  "1"  /* Pressure ulcer of left buttock, stage 4 */
"L8943"  =  "1"  /* Pressure ulcer of contiguous site of back, buttock and hip, stage 3 */
"L8944"  =  "1"  /* Pressure ulcer of contiguous site of back, buttock and hip, stage 4 */
"L8945"  =  "1"  /* Pressure ulcer of contiguous site of back, buttock and hip, unstageable */
"L89500"  =  "1"  /* Pressure ulcer of unspecified ankle, unstageable */
"L89503"  =  "1"  /* Pressure ulcer of unspecified ankle, stage 3 */
"L89504"  =  "1"  /* Pressure ulcer of unspecified ankle, stage 4 */
"L89510"  =  "1"  /* Pressure ulcer of right ankle, unstageable */
"L89513"  =  "1"  /* Pressure ulcer of right ankle, stage 3 */
"L89514"  =  "1"  /* Pressure ulcer of right ankle, stage 4 */
"L89520"  =  "1"  /* Pressure ulcer of left ankle, unstageable */
"L89523"  =  "1"  /* Pressure ulcer of left ankle, stage 3 */
"L89524"  =  "1"  /* Pressure ulcer of left ankle, stage 4 */
"L89600"  =  "1"  /* Pressure ulcer of unspecified heel, unstageable */
"L89603"  =  "1"  /* Pressure ulcer of unspecified heel, stage 3 */
"L89604"  =  "1"  /* Pressure ulcer of unspecified heel, stage 4 */
"L89610"  =  "1"  /* Pressure ulcer of right heel, unstageable */
"L89613"  =  "1"  /* Pressure ulcer of right heel, stage 3 */
"L89614"  =  "1"  /* Pressure ulcer of right heel, stage 4 */
"L89620"  =  "1"  /* Pressure ulcer of left heel, unstageable */
"L89623"  =  "1"  /* Pressure ulcer of left heel, stage 3 */
"L89624"  =  "1"  /* Pressure ulcer of left heel, stage 4 */
"L89810"  =  "1"  /* Pressure ulcer of head, unstageable */
"L89813"  =  "1"  /* Pressure ulcer of head, stage 3 */
"L89814"  =  "1"  /* Pressure ulcer of head, stage 4 */
"L89890"  =  "1"  /* Pressure ulcer of other site, unstageable */
"L89893"  =  "1"  /* Pressure ulcer of other site, stage 3 */
"L89894"  =  "1"  /* Pressure ulcer of other site, stage 4 */
"L8993"  =  "1"  /* Pressure ulcer of unspecified site, stage 3 */
"L8994"  =  "1"  /* Pressure ulcer of unspecified site, stage 4 */
"L8995"  =  "1"  /* Pressure ulcer of unspecified site, unstageable */
OTHER   = "0" ;

 /* EXCLUDE: DEBRIDEMENT OR PEDICLE GRAFT */
 /* ICD-10-CM PROCEDURE CODES: */

VALUE $DEBRIDP
"0H80XZZ"  =  "1"  /* Division of Scalp Skin, External Approach */
"0H81XZZ"  =  "1"  /* Division of Face Skin, External Approach */
"0H84XZZ"  =  "1"  /* Division of Neck Skin, External Approach */
"0H85XZZ"  =  "1"  /* Division of Chest Skin, External Approach */
"0H86XZZ"  =  "1"  /* Division of Back Skin, External Approach */
"0H87XZZ"  =  "1"  /* Division of Abdomen Skin, External Approach */
"0H88XZZ"  =  "1"  /* Division of Buttock Skin, External Approach */
"0H8AXZZ"  =  "1"  /* Division of Genitalia Skin, External Approach */
"0H8BXZZ"  =  "1"  /* Division of Right Upper Arm Skin, External Approach */
"0H8CXZZ"  =  "1"  /* Division of Left Upper Arm Skin, External Approach */
"0H8DXZZ"  =  "1"  /* Division of Right Lower Arm Skin, External Approach */
"0H8EXZZ"  =  "1"  /* Division of Left Lower Arm Skin, External Approach */
"0H8FXZZ"  =  "1"  /* Division of Right Hand Skin, External Approach */
"0H8GXZZ"  =  "1"  /* Division of Left Hand Skin, External Approach */
"0H8HXZZ"  =  "1"  /* Division of Right Upper Leg Skin, External Approach */
"0H8JXZZ"  =  "1"  /* Division of Left Upper Leg Skin, External Approach */
"0H8KXZZ"  =  "1"  /* Division of Right Lower Leg Skin, External Approach */
"0H8LXZZ"  =  "1"  /* Division of Left Lower Leg Skin, External Approach */
"0H8MXZZ"  =  "1"  /* Division of Right Foot Skin, External Approach */
"0H8NXZZ"  =  "1"  /* Division of Left Foot Skin, External Approach */
"0HB0XZZ"  =  "1"  /* Excision of Scalp Skin, External Approach */
"0HB1XZZ"  =  "1"  /* Excision of Face Skin, External Approach */
"0HB4XZZ"  =  "1"  /* Excision of Neck Skin, External Approach */
"0HB5XZZ"  =  "1"  /* Excision of Chest Skin, External Approach */
"0HB6XZZ"  =  "1"  /* Excision of Back Skin, External Approach */
"0HB7XZZ"  =  "1"  /* Excision of Abdomen Skin, External Approach */
"0HB8XZZ"  =  "1"  /* Excision of Buttock Skin, External Approach */
"0HBAXZZ"  =  "1"  /* Excision of Genitalia Skin, External Approach */
"0HBBXZZ"  =  "1"  /* Excision of Right Upper Arm Skin, External Approach */
"0HBCXZZ"  =  "1"  /* Excision of Left Upper Arm Skin, External Approach */
"0HBDXZZ"  =  "1"  /* Excision of Right Lower Arm Skin, External Approach */
"0HBEXZZ"  =  "1"  /* Excision of Left Lower Arm Skin, External Approach */
"0HBFXZZ"  =  "1"  /* Excision of Right Hand Skin, External Approach */
"0HBGXZZ"  =  "1"  /* Excision of Left Hand Skin, External Approach */
"0HBHXZZ"  =  "1"  /* Excision of Right Upper Leg Skin, External Approach */
"0HBJXZZ"  =  "1"  /* Excision of Left Upper Leg Skin, External Approach */
"0HBKXZZ"  =  "1"  /* Excision of Right Lower Leg Skin, External Approach */
"0HBLXZZ"  =  "1"  /* Excision of Left Lower Leg Skin, External Approach */
"0HBMXZZ"  =  "1"  /* Excision of Right Foot Skin, External Approach */
"0HBNXZZ"  =  "1"  /* Excision of Left Foot Skin, External Approach */
"0HD0XZZ"  =  "1"  /* Extraction of Scalp Skin, External Approach */
"0HD1XZZ"  =  "1"  /* Extraction of Face Skin, External Approach */
"0HD4XZZ"  =  "1"  /* Extraction of Neck Skin, External Approach */
"0HD5XZZ"  =  "1"  /* Extraction of Chest Skin, External Approach */
"0HD6XZZ"  =  "1"  /* Extraction of Back Skin, External Approach */
"0HD7XZZ"  =  "1"  /* Extraction of Abdomen Skin, External Approach */
"0HD8XZZ"  =  "1"  /* Extraction of Buttock Skin, External Approach */
"0HD9XZZ"  =  "1"  /* Extraction of Perineum Skin, External Approach */
"0HDAXZZ"  =  "1"  /* Extraction of Genitalia Skin, External Approach */
"0HDBXZZ"  =  "1"  /* Extraction of Right Upper Arm Skin, External Approach */
"0HDCXZZ"  =  "1"  /* Extraction of Left Upper Arm Skin, External Approach */
"0HDDXZZ"  =  "1"  /* Extraction of Right Lower Arm Skin, External Approach */
"0HDEXZZ"  =  "1"  /* Extraction of Left Lower Arm Skin, External Approach */
"0HDFXZZ"  =  "1"  /* Extraction of Right Hand Skin, External Approach */
"0HDGXZZ"  =  "1"  /* Extraction of Left Hand Skin, External Approach */
"0HDHXZZ"  =  "1"  /* Extraction of Right Upper Leg Skin, External Approach */
"0HDJXZZ"  =  "1"  /* Extraction of Left Upper Leg Skin, External Approach */
"0HDKXZZ"  =  "1"  /* Extraction of Right Lower Leg Skin, External Approach */
"0HDLXZZ"  =  "1"  /* Extraction of Left Lower Leg Skin, External Approach */
"0HDMXZZ"  =  "1"  /* Extraction of Right Foot Skin, External Approach */
"0HDNXZZ"  =  "1"  /* Extraction of Left Foot Skin, External Approach */
"0HX0XZZ"  =  "1"  /* Transfer Scalp Skin, External Approach */
"0HX1XZZ"  =  "1"  /* Transfer Face Skin, External Approach */
"0HX4XZZ"  =  "1"  /* Transfer Neck Skin, External Approach */
"0HX5XZZ"  =  "1"  /* Transfer Chest Skin, External Approach */
"0HX6XZZ"  =  "1"  /* Transfer Back Skin, External Approach */
"0HX7XZZ"  =  "1"  /* Transfer Abdomen Skin, External Approach */
"0HX8XZZ"  =  "1"  /* Transfer Buttock Skin, External Approach */
"0HX9XZZ"  =  "1"  /* Transfer Perineum Skin, External Approach */
"0HXAXZZ"  =  "1"  /* Transfer Genitalia Skin, External Approach */
"0HXBXZZ"  =  "1"  /* Transfer Right Upper Arm Skin, External Approach */
"0HXCXZZ"  =  "1"  /* Transfer Left Upper Arm Skin, External Approach */
"0HXDXZZ"  =  "1"  /* Transfer Right Lower Arm Skin, External Approach */
"0HXEXZZ"  =  "1"  /* Transfer Left Lower Arm Skin, External Approach */
"0HXFXZZ"  =  "1"  /* Transfer Right Hand Skin, External Approach */
"0HXGXZZ"  =  "1"  /* Transfer Left Hand Skin, External Approach */
"0HXHXZZ"  =  "1"  /* Transfer Right Upper Leg Skin, External Approach */
"0HXJXZZ"  =  "1"  /* Transfer Left Upper Leg Skin, External Approach */
"0HXKXZZ"  =  "1"  /* Transfer Right Lower Leg Skin, External Approach */
"0HXLXZZ"  =  "1"  /* Transfer Left Lower Leg Skin, External Approach */
"0HXMXZZ"  =  "1"  /* Transfer Right Foot Skin, External Approach */
"0HXNXZZ"  =  "1"  /* Transfer Left Foot Skin, External Approach */
"0JB00ZZ"  =  "1"  /* Excision of Scalp Subcutaneous Tissue and Fascia, Open Approach */
"0JB10ZZ"  =  "1"  /* Excision of Face Subcutaneous Tissue and Fascia, Open Approach */
"0JB40ZZ"  =  "1"  /* Excision of Anterior Neck Subcutaneous Tissue and Fascia, Open Approach */
"0JB50ZZ"  =  "1"  /* Excision of Posterior Neck Subcutaneous Tissue and Fascia, Open Approach */
"0JB60ZZ"  =  "1"  /* Excision of Chest Subcutaneous Tissue and Fascia, Open Approach */
"0JB70ZZ"  =  "1"  /* Excision of Back Subcutaneous Tissue and Fascia, Open Approach */
"0JB80ZZ"  =  "1"  /* Excision of Abdomen Subcutaneous Tissue and Fascia, Open Approach */
"0JB90ZZ"  =  "1"  /* Excision of Buttock Subcutaneous Tissue and Fascia, Open Approach */
"0JBB0ZZ"  =  "1"  /* Excision of Perineum Subcutaneous Tissue and Fascia, Open Approach */
"0JBC0ZZ"  =  "1"  /* Excision of Pelvic Region Subcutaneous Tissue and Fascia, Open Approach */
"0JBD0ZZ"  =  "1"  /* Excision of Right Upper Arm Subcutaneous Tissue and Fascia, Open Approach */
"0JBF0ZZ"  =  "1"  /* Excision of Left Upper Arm Subcutaneous Tissue and Fascia, Open Approach */
"0JBG0ZZ"  =  "1"  /* Excision of Right Lower Arm Subcutaneous Tissue and Fascia, Open Approach */
"0JBH0ZZ"  =  "1"  /* Excision of Left Lower Arm Subcutaneous Tissue and Fascia, Open Approach */
"0JBL0ZZ"  =  "1"  /* Excision of Right Upper Leg Subcutaneous Tissue and Fascia, Open Approach */
"0JBM0ZZ"  =  "1"  /* Excision of Left Upper Leg Subcutaneous Tissue and Fascia, Open Approach */
"0JBN0ZZ"  =  "1"  /* Excision of Right Lower Leg Subcutaneous Tissue and Fascia, Open Approach */
"0JBP0ZZ"  =  "1"  /* Excision of Left Lower Leg Subcutaneous Tissue and Fascia, Open Approach */
"0JBQ0ZZ"  =  "1"  /* Excision of Right Foot Subcutaneous Tissue and Fascia, Open Approach */
"0JBR0ZZ"  =  "1"  /* Excision of Left Foot Subcutaneous Tissue and Fascia, Open Approach */
"0JD00ZZ"  =  "1"  /* Extraction of Scalp Subcutaneous Tissue and Fascia, Open Approach */
"0JD10ZZ"  =  "1"  /* Extraction of Face Subcutaneous Tissue and Fascia, Open Approach */
"0JD40ZZ"  =  "1"  /* Extraction of Anterior Neck Subcutaneous Tissue and Fascia, Open Approach */
"0JD50ZZ"  =  "1"  /* Extraction of Posterior Neck Subcutaneous Tissue and Fascia, Open Approach */
"0JD60ZZ"  =  "1"  /* Extraction of Chest Subcutaneous Tissue and Fascia, Open Approach */
"0JD70ZZ"  =  "1"  /* Extraction of Back Subcutaneous Tissue and Fascia, Open Approach */
"0JD80ZZ"  =  "1"  /* Extraction of Abdomen Subcutaneous Tissue and Fascia, Open Approach */
"0JD90ZZ"  =  "1"  /* Extraction of Buttock Subcutaneous Tissue and Fascia, Open Approach */
"0JDB0ZZ"  =  "1"  /* Extraction of Perineum Subcutaneous Tissue and Fascia, Open Approach */
"0JDC0ZZ"  =  "1"  /* Extraction of Pelvic Region Subcutaneous Tissue and Fascia, Open Approach */
"0JDD0ZZ"  =  "1"  /* Extraction of Right Upper Arm Subcutaneous Tissue and Fascia, Open Approach */
"0JDF0ZZ"  =  "1"  /* Extraction of Left Upper Arm Subcutaneous Tissue and Fascia, Open Approach */
"0JDG0ZZ"  =  "1"  /* Extraction of Right Lower Arm Subcutaneous Tissue and Fascia, Open Approach */
"0JDH0ZZ"  =  "1"  /* Extraction of Left Lower Arm Subcutaneous Tissue and Fascia, Open Approach */
"0JDJ0ZZ"  =  "1"  /* Extraction of Right Hand Subcutaneous Tissue and Fascia, Open Approach */
"0JDK0ZZ"  =  "1"  /* Extraction of Left Hand Subcutaneous Tissue and Fascia, Open Approach */
"0JDL0ZZ"  =  "1"  /* Extraction of Right Upper Leg Subcutaneous Tissue and Fascia, Open Approach */
"0JDM0ZZ"  =  "1"  /* Extraction of Left Upper Leg Subcutaneous Tissue and Fascia, Open Approach */
"0JDN0ZZ"  =  "1"  /* Extraction of Right Lower Leg Subcutaneous Tissue and Fascia, Open Approach */
"0JDP0ZZ"  =  "1"  /* Extraction of Left Lower Leg Subcutaneous Tissue and Fascia, Open Approach */
"0JDQ0ZZ"  =  "1"  /* Extraction of Right Foot Subcutaneous Tissue and Fascia, Open Approach */
"0JDR0ZZ"  =  "1"  /* Extraction of Left Foot Subcutaneous Tissue and Fascia, Open Approach */
"0JX00ZB"  =  "1"  /* Transfer Scalp Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JX00ZC"  =  "1"  /* Transfer Scalp Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JX03ZB"  =  "1"  /* Transfer Scalp Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JX03ZC"  =  "1"  /* Transfer Scalp Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JX10ZB"  =  "1"  /* Transfer Face Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JX10ZC"  =  "1"  /* Transfer Face Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JX13ZB"  =  "1"  /* Transfer Face Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JX13ZC"  =  "1"  /* Transfer Face Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JX40ZB"  =  "1"  /* Transfer Anterior Neck Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JX40ZC"  =  "1"  /* Transfer Anterior Neck Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JX43ZB"  =  "1"  /* Transfer Anterior Neck Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JX43ZC"  =  "1"  /* Transfer Anterior Neck Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JX50ZB"  =  "1"  /* Transfer Posterior Neck Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JX50ZC"  =  "1"  /* Transfer Posterior Neck Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JX53ZB"  =  "1"  /* Transfer Posterior Neck Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JX53ZC"  =  "1"  /* Transfer Posterior Neck Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JX60ZB"  =  "1"  /* Transfer Chest Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JX60ZC"  =  "1"  /* Transfer Chest Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JX63ZB"  =  "1"  /* Transfer Chest Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JX63ZC"  =  "1"  /* Transfer Chest Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JX70ZB"  =  "1"  /* Transfer Back Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JX70ZC"  =  "1"  /* Transfer Back Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JX73ZB"  =  "1"  /* Transfer Back Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JX73ZC"  =  "1"  /* Transfer Back Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JX80ZB"  =  "1"  /* Transfer Abdomen Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JX80ZC"  =  "1"  /* Transfer Abdomen Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JX83ZB"  =  "1"  /* Transfer Abdomen Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JX83ZC"  =  "1"  /* Transfer Abdomen Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JX90ZB"  =  "1"  /* Transfer Buttock Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JX90ZC"  =  "1"  /* Transfer Buttock Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JX93ZB"  =  "1"  /* Transfer Buttock Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JX93ZC"  =  "1"  /* Transfer Buttock Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JXB0ZB"  =  "1"  /* Transfer Perineum Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JXB0ZC"  =  "1"  /* Transfer Perineum Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JXB3ZB"  =  "1"  /* Transfer Perineum Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JXB3ZC"  =  "1"  /* Transfer Perineum Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JXC0ZB"  =  "1"  /* Transfer Pelvic Region Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JXC0ZC"  =  "1"  /* Transfer Pelvic Region Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JXC3ZB"  =  "1"  /* Transfer Pelvic Region Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JXC3ZC"  =  "1"  /* Transfer Pelvic Region Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JXD0ZB"  =  "1"  /* Transfer Right Upper Arm Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JXD0ZC"  =  "1"  /* Transfer Right Upper Arm Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JXD3ZB"  =  "1"  /* Transfer Right Upper Arm Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JXD3ZC"  =  "1"  /* Transfer Right Upper Arm Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JXF0ZB"  =  "1"  /* Transfer Left Upper Arm Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JXF0ZC"  =  "1"  /* Transfer Left Upper Arm Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JXF3ZB"  =  "1"  /* Transfer Left Upper Arm Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JXF3ZC"  =  "1"  /* Transfer Left Upper Arm Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JXG0ZB"  =  "1"  /* Transfer Right Lower Arm Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JXG0ZC"  =  "1"  /* Transfer Right Lower Arm Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JXG3ZB"  =  "1"  /* Transfer Right Lower Arm Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JXG3ZC"  =  "1"  /* Transfer Right Lower Arm Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JXH0ZB"  =  "1"  /* Transfer Left Lower Arm Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JXH0ZC"  =  "1"  /* Transfer Left Lower Arm Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JXH3ZB"  =  "1"  /* Transfer Left Lower Arm Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JXH3ZC"  =  "1"  /* Transfer Left Lower Arm Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JXL0ZB"  =  "1"  /* Transfer Right Upper Leg Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JXL0ZC"  =  "1"  /* Transfer Right Upper Leg Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JXL3ZB"  =  "1"  /* Transfer Right Upper Leg Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JXL3ZC"  =  "1"  /* Transfer Right Upper Leg Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JXM0ZB"  =  "1"  /* Transfer Left Upper Leg Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JXM0ZC"  =  "1"  /* Transfer Left Upper Leg Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JXM3ZB"  =  "1"  /* Transfer Left Upper Leg Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JXM3ZC"  =  "1"  /* Transfer Left Upper Leg Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JXN0ZB"  =  "1"  /* Transfer Right Lower Leg Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JXN0ZC"  =  "1"  /* Transfer Right Lower Leg Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JXN3ZB"  =  "1"  /* Transfer Right Lower Leg Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JXN3ZC"  =  "1"  /* Transfer Right Lower Leg Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JXP0ZB"  =  "1"  /* Transfer Left Lower Leg Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JXP0ZC"  =  "1"  /* Transfer Left Lower Leg Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JXP3ZB"  =  "1"  /* Transfer Left Lower Leg Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JXP3ZC"  =  "1"  /* Transfer Left Lower Leg Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JXQ0ZB"  =  "1"  /* Transfer Right Foot Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JXQ0ZC"  =  "1"  /* Transfer Right Foot Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JXQ3ZB"  =  "1"  /* Transfer Right Foot Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JXQ3ZC"  =  "1"  /* Transfer Right Foot Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JXR0ZB"  =  "1"  /* Transfer Left Foot Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Open Approach */
"0JXR0ZC"  =  "1"  /* Transfer Left Foot Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Open Approach */
"0JXR3ZB"  =  "1"  /* Transfer Left Foot Subcutaneous Tissue and Fascia with Skin and Subcutaneous Tissue, Percutaneous Approach */
"0JXR3ZC"  =  "1"  /* Transfer Left Foot Subcutaneous Tissue and Fascia with Skin, Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0KB00ZZ"  =  "1"  /* Excision of Head Muscle, Open Approach */
"0KB03ZZ"  =  "1"  /* Excision of Head Muscle, Percutaneous Approach */
"0KB04ZZ"  =  "1"  /* Excision of Head Muscle, Percutaneous Endoscopic Approach */
"0KB10ZZ"  =  "1"  /* Excision of Facial Muscle, Open Approach */
"0KB13ZZ"  =  "1"  /* Excision of Facial Muscle, Percutaneous Approach */
"0KB14ZZ"  =  "1"  /* Excision of Facial Muscle, Percutaneous Endoscopic Approach */
"0KB20ZZ"  =  "1"  /* Excision of Right Neck Muscle, Open Approach */
"0KB23ZZ"  =  "1"  /* Excision of Right Neck Muscle, Percutaneous Approach */
"0KB24ZZ"  =  "1"  /* Excision of Right Neck Muscle, Percutaneous Endoscopic Approach */
"0KB30ZZ"  =  "1"  /* Excision of Left Neck Muscle, Open Approach */
"0KB33ZZ"  =  "1"  /* Excision of Left Neck Muscle, Percutaneous Approach */
"0KB34ZZ"  =  "1"  /* Excision of Left Neck Muscle, Percutaneous Endoscopic Approach */
"0KB40ZZ"  =  "1"  /* Excision of Tongue, Palate, Pharynx Muscle, Open Approach */
"0KB43ZZ"  =  "1"  /* Excision of Tongue, Palate, Pharynx Muscle, Percutaneous Approach */
"0KB44ZZ"  =  "1"  /* Excision of Tongue, Palate, Pharynx Muscle, Percutaneous Endoscopic Approach */
"0KB50ZZ"  =  "1"  /* Excision of Right Shoulder Muscle, Open Approach */
"0KB53ZZ"  =  "1"  /* Excision of Right Shoulder Muscle, Percutaneous Approach */
"0KB54ZZ"  =  "1"  /* Excision of Right Shoulder Muscle, Percutaneous Endoscopic Approach */
"0KB60ZZ"  =  "1"  /* Excision of Left Shoulder Muscle, Open Approach */
"0KB63ZZ"  =  "1"  /* Excision of Left Shoulder Muscle, Percutaneous Approach */
"0KB64ZZ"  =  "1"  /* Excision of Left Shoulder Muscle, Percutaneous Endoscopic Approach */
"0KB70ZZ"  =  "1"  /* Excision of Right Upper Arm Muscle, Open Approach */
"0KB73ZZ"  =  "1"  /* Excision of Right Upper Arm Muscle, Percutaneous Approach */
"0KB74ZZ"  =  "1"  /* Excision of Right Upper Arm Muscle, Percutaneous Endoscopic Approach */
"0KB80ZZ"  =  "1"  /* Excision of Left Upper Arm Muscle, Open Approach */
"0KB83ZZ"  =  "1"  /* Excision of Left Upper Arm Muscle, Percutaneous Approach */
"0KB84ZZ"  =  "1"  /* Excision of Left Upper Arm Muscle, Percutaneous Endoscopic Approach */
"0KB90ZZ"  =  "1"  /* Excision of Right Lower Arm and Wrist Muscle, Open Approach */
"0KB93ZZ"  =  "1"  /* Excision of Right Lower Arm and Wrist Muscle, Percutaneous Approach */
"0KB94ZZ"  =  "1"  /* Excision of Right Lower Arm and Wrist Muscle, Percutaneous Endoscopic Approach */
"0KBB0ZZ"  =  "1"  /* Excision of Left Lower Arm and Wrist Muscle, Open Approach */
"0KBB3ZZ"  =  "1"  /* Excision of Left Lower Arm and Wrist Muscle, Percutaneous Approach */
"0KBB4ZZ"  =  "1"  /* Excision of Left Lower Arm and Wrist Muscle, Percutaneous Endoscopic Approach */
"0KBF0ZZ"  =  "1"  /* Excision of Right Trunk Muscle, Open Approach */
"0KBF3ZZ"  =  "1"  /* Excision of Right Trunk Muscle, Percutaneous Approach */
"0KBF4ZZ"  =  "1"  /* Excision of Right Trunk Muscle, Percutaneous Endoscopic Approach */
"0KBG0ZZ"  =  "1"  /* Excision of Left Trunk Muscle, Open Approach */
"0KBG3ZZ"  =  "1"  /* Excision of Left Trunk Muscle, Percutaneous Approach */
"0KBG4ZZ"  =  "1"  /* Excision of Left Trunk Muscle, Percutaneous Endoscopic Approach */
"0KBH0ZZ"  =  "1"  /* Excision of Right Thorax Muscle, Open Approach */
"0KBH3ZZ"  =  "1"  /* Excision of Right Thorax Muscle, Percutaneous Approach */
"0KBH4ZZ"  =  "1"  /* Excision of Right Thorax Muscle, Percutaneous Endoscopic Approach */
"0KBJ0ZZ"  =  "1"  /* Excision of Left Thorax Muscle, Open Approach */
"0KBJ3ZZ"  =  "1"  /* Excision of Left Thorax Muscle, Percutaneous Approach */
"0KBJ4ZZ"  =  "1"  /* Excision of Left Thorax Muscle, Percutaneous Endoscopic Approach */
"0KBK0ZZ"  =  "1"  /* Excision of Right Abdomen Muscle, Open Approach */
"0KBK3ZZ"  =  "1"  /* Excision of Right Abdomen Muscle, Percutaneous Approach */
"0KBK4ZZ"  =  "1"  /* Excision of Right Abdomen Muscle, Percutaneous Endoscopic Approach */
"0KBL0ZZ"  =  "1"  /* Excision of Left Abdomen Muscle, Open Approach */
"0KBL3ZZ"  =  "1"  /* Excision of Left Abdomen Muscle, Percutaneous Approach */
"0KBL4ZZ"  =  "1"  /* Excision of Left Abdomen Muscle, Percutaneous Endoscopic Approach */
"0KBM0ZZ"  =  "1"  /* Excision of Perineum Muscle, Open Approach */
"0KBM3ZZ"  =  "1"  /* Excision of Perineum Muscle, Percutaneous Approach */
"0KBM4ZZ"  =  "1"  /* Excision of Perineum Muscle, Percutaneous Endoscopic Approach */
"0KBN0ZZ"  =  "1"  /* Excision of Right Hip Muscle, Open Approach */
"0KBN3ZZ"  =  "1"  /* Excision of Right Hip Muscle, Percutaneous Approach */
"0KBN4ZZ"  =  "1"  /* Excision of Right Hip Muscle, Percutaneous Endoscopic Approach */
"0KBP0ZZ"  =  "1"  /* Excision of Left Hip Muscle, Open Approach */
"0KBP3ZZ"  =  "1"  /* Excision of Left Hip Muscle, Percutaneous Approach */
"0KBP4ZZ"  =  "1"  /* Excision of Left Hip Muscle, Percutaneous Endoscopic Approach */
"0KBQ0ZZ"  =  "1"  /* Excision of Right Upper Leg Muscle, Open Approach */
"0KBQ3ZZ"  =  "1"  /* Excision of Right Upper Leg Muscle, Percutaneous Approach */
"0KBQ4ZZ"  =  "1"  /* Excision of Right Upper Leg Muscle, Percutaneous Endoscopic Approach */
"0KBR0ZZ"  =  "1"  /* Excision of Left Upper Leg Muscle, Open Approach */
"0KBR3ZZ"  =  "1"  /* Excision of Left Upper Leg Muscle, Percutaneous Approach */
"0KBR4ZZ"  =  "1"  /* Excision of Left Upper Leg Muscle, Percutaneous Endoscopic Approach */
"0KBS0ZZ"  =  "1"  /* Excision of Right Lower Leg Muscle, Open Approach */
"0KBS3ZZ"  =  "1"  /* Excision of Right Lower Leg Muscle, Percutaneous Approach */
"0KBS4ZZ"  =  "1"  /* Excision of Right Lower Leg Muscle, Percutaneous Endoscopic Approach */
"0KBT0ZZ"  =  "1"  /* Excision of Left Lower Leg Muscle, Open Approach */
"0KBT3ZZ"  =  "1"  /* Excision of Left Lower Leg Muscle, Percutaneous Approach */
"0KBT4ZZ"  =  "1"  /* Excision of Left Lower Leg Muscle, Percutaneous Endoscopic Approach */
"0KBV0ZZ"  =  "1"  /* Excision of Right Foot Muscle, Open Approach */
"0KBV3ZZ"  =  "1"  /* Excision of Right Foot Muscle, Percutaneous Approach */
"0KBV4ZZ"  =  "1"  /* Excision of Right Foot Muscle, Percutaneous Endoscopic Approach */
"0KBW0ZZ"  =  "1"  /* Excision of Left Foot Muscle, Open Approach */
"0KBW3ZZ"  =  "1"  /* Excision of Left Foot Muscle, Percutaneous Approach */
"0KBW4ZZ"  =  "1"  /* Excision of Left Foot Muscle, Percutaneous Endoscopic Approach */
"0KT00ZZ"  =  "1"  /* Resection of Head Muscle, Open Approach */
"0KT04ZZ"  =  "1"  /* Resection of Head Muscle, Percutaneous Endoscopic Approach */
"0KT10ZZ"  =  "1"  /* Resection of Facial Muscle, Open Approach */
"0KT14ZZ"  =  "1"  /* Resection of Facial Muscle, Percutaneous Endoscopic Approach */
"0KT20ZZ"  =  "1"  /* Resection of Right Neck Muscle, Open Approach */
"0KT24ZZ"  =  "1"  /* Resection of Right Neck Muscle, Percutaneous Endoscopic Approach */
"0KT30ZZ"  =  "1"  /* Resection of Left Neck Muscle, Open Approach */
"0KT34ZZ"  =  "1"  /* Resection of Left Neck Muscle, Percutaneous Endoscopic Approach */
"0KT40ZZ"  =  "1"  /* Resection of Tongue, Palate, Pharynx Muscle, Open Approach */
"0KT44ZZ"  =  "1"  /* Resection of Tongue, Palate, Pharynx Muscle, Percutaneous Endoscopic Approach */
"0KT50ZZ"  =  "1"  /* Resection of Right Shoulder Muscle, Open Approach */
"0KT54ZZ"  =  "1"  /* Resection of Right Shoulder Muscle, Percutaneous Endoscopic Approach */
"0KT60ZZ"  =  "1"  /* Resection of Left Shoulder Muscle, Open Approach */
"0KT64ZZ"  =  "1"  /* Resection of Left Shoulder Muscle, Percutaneous Endoscopic Approach */
"0KT70ZZ"  =  "1"  /* Resection of Right Upper Arm Muscle, Open Approach */
"0KT74ZZ"  =  "1"  /* Resection of Right Upper Arm Muscle, Percutaneous Endoscopic Approach */
"0KT80ZZ"  =  "1"  /* Resection of Left Upper Arm Muscle, Open Approach */
"0KT84ZZ"  =  "1"  /* Resection of Left Upper Arm Muscle, Percutaneous Endoscopic Approach */
"0KT90ZZ"  =  "1"  /* Resection of Right Lower Arm and Wrist Muscle, Open Approach */
"0KT94ZZ"  =  "1"  /* Resection of Right Lower Arm and Wrist Muscle, Percutaneous Endoscopic Approach */
"0KTB0ZZ"  =  "1"  /* Resection of Left Lower Arm and Wrist Muscle, Open Approach */
"0KTB4ZZ"  =  "1"  /* Resection of Left Lower Arm and Wrist Muscle, Percutaneous Endoscopic Approach */
"0KTF0ZZ"  =  "1"  /* Resection of Right Trunk Muscle, Open Approach */
"0KTF4ZZ"  =  "1"  /* Resection of Right Trunk Muscle, Percutaneous Endoscopic Approach */
"0KTG0ZZ"  =  "1"  /* Resection of Left Trunk Muscle, Open Approach */
"0KTG4ZZ"  =  "1"  /* Resection of Left Trunk Muscle, Percutaneous Endoscopic Approach */
"0KTH0ZZ"  =  "1"  /* Resection of Right Thorax Muscle, Open Approach */
"0KTH4ZZ"  =  "1"  /* Resection of Right Thorax Muscle, Percutaneous Endoscopic Approach */
"0KTJ0ZZ"  =  "1"  /* Resection of Left Thorax Muscle, Open Approach */
"0KTJ4ZZ"  =  "1"  /* Resection of Left Thorax Muscle, Percutaneous Endoscopic Approach */
"0KTK0ZZ"  =  "1"  /* Resection of Right Abdomen Muscle, Open Approach */
"0KTK4ZZ"  =  "1"  /* Resection of Right Abdomen Muscle, Percutaneous Endoscopic Approach */
"0KTL0ZZ"  =  "1"  /* Resection of Left Abdomen Muscle, Open Approach */
"0KTL4ZZ"  =  "1"  /* Resection of Left Abdomen Muscle, Percutaneous Endoscopic Approach */
"0KTM0ZZ"  =  "1"  /* Resection of Perineum Muscle, Open Approach */
"0KTM4ZZ"  =  "1"  /* Resection of Perineum Muscle, Percutaneous Endoscopic Approach */
"0KTN0ZZ"  =  "1"  /* Resection of Right Hip Muscle, Open Approach */
"0KTN4ZZ"  =  "1"  /* Resection of Right Hip Muscle, Percutaneous Endoscopic Approach */
"0KTP0ZZ"  =  "1"  /* Resection of Left Hip Muscle, Open Approach */
"0KTP4ZZ"  =  "1"  /* Resection of Left Hip Muscle, Percutaneous Endoscopic Approach */
"0KTQ0ZZ"  =  "1"  /* Resection of Right Upper Leg Muscle, Open Approach */
"0KTQ4ZZ"  =  "1"  /* Resection of Right Upper Leg Muscle, Percutaneous Endoscopic Approach */
"0KTR0ZZ"  =  "1"  /* Resection of Left Upper Leg Muscle, Open Approach */
"0KTR4ZZ"  =  "1"  /* Resection of Left Upper Leg Muscle, Percutaneous Endoscopic Approach */
"0KTS0ZZ"  =  "1"  /* Resection of Right Lower Leg Muscle, Open Approach */
"0KTS4ZZ"  =  "1"  /* Resection of Right Lower Leg Muscle, Percutaneous Endoscopic Approach */
"0KTT0ZZ"  =  "1"  /* Resection of Left Lower Leg Muscle, Open Approach */
"0KTT4ZZ"  =  "1"  /* Resection of Left Lower Leg Muscle, Percutaneous Endoscopic Approach */
"0KTV0ZZ"  =  "1"  /* Resection of Right Foot Muscle, Open Approach */
"0KTV4ZZ"  =  "1"  /* Resection of Right Foot Muscle, Percutaneous Endoscopic Approach */
"0KTW0ZZ"  =  "1"  /* Resection of Left Foot Muscle, Open Approach */
"0KTW4ZZ"  =  "1"  /* Resection of Left Foot Muscle, Percutaneous Endoscopic Approach */
"3E1038Z"  =  "1"  /* Irrigation of Skin and Mucous Membranes using Irrigating Substance, Percutaneous Approach */
"3E10X8Z"  =  "1"  /* Irrigation of Skin and Mucous Membranes using Irrigating Substance */
OTHER   = "0" ;

 /* STRATIFIED BY: HEMIPLEGIA, PARAPLEGIA, OR QUADRIPLEGIA */
 /* ICD-10-CM DIAGNOSIS CODES: */

VALUE $HEMIPID
"G041"  =  "1"  /* Tropical spastic paraplegia */
"G114"  =  "1"  /* Hereditary spastic paraplegia */
"G800"  =  "1"  /* Spastic quadriplegic cerebral palsy */
"G801"  =  "1"  /* Spastic diplegic cerebral palsy */
"G802"  =  "1"  /* Spastic hemiplegic cerebral palsy */
"G803"  =  "1"  /* Athetoid cerebral palsy */
"G804"  =  "1"  /* Ataxic cerebral palsy */
"G808"  =  "1"  /* Other cerebral palsy */
"G809"  =  "1"  /* Cerebral palsy, unspecified */
"G8100"  =  "1"  /* Flaccid hemiplegia affecting unspecified side */
"G8101"  =  "1"  /* Flaccid hemiplegia affecting right dominant side */
"G8102"  =  "1"  /* Flaccid hemiplegia affecting left dominant side */
"G8103"  =  "1"  /* Flaccid hemiplegia affecting right nondominant side */
"G8104"  =  "1"  /* Flaccid hemiplegia affecting left nondominant side */
"G8110"  =  "1"  /* Spastic hemiplegia affecting unspecified side */
"G8111"  =  "1"  /* Spastic hemiplegia affecting right dominant side */
"G8112"  =  "1"  /* Spastic hemiplegia affecting left dominant side */
"G8113"  =  "1"  /* Spastic hemiplegia affecting right nondominant side */
"G8114"  =  "1"  /* Spastic hemiplegia affecting left nondominant side */
"G8190"  =  "1"  /* Hemiplegia, unspecified affecting unspecified side */
"G8191"  =  "1"  /* Hemiplegia, unspecified affecting right dominant side */
"G8192"  =  "1"  /* Hemiplegia, unspecified affecting left dominant side */
"G8193"  =  "1"  /* Hemiplegia, unspecified affecting right nondominant side */
"G8194"  =  "1"  /* Hemiplegia, unspecified affecting left nondominant side */
"G8220"  =  "1"  /* Paraplegia, unspecified */
"G8221"  =  "1"  /* Paraplegia, complete */
"G8222"  =  "1"  /* Paraplegia, incomplete */
"G8250"  =  "1"  /* Quadriplegia, unspecified */
"G8251"  =  "1"  /* Quadriplegia, C1-C4 complete */
"G8252"  =  "1"  /* Quadriplegia, C1-C4 incomplete */
"G8253"  =  "1"  /* Quadriplegia, C5-C7 complete */
"G8254"  =  "1"  /* Quadriplegia, C5-C7 incomplete */
"G830"  =  "1"  /* Diplegia of upper limbs */
"G8310"  =  "1"  /* Monoplegia of lower limb affecting unspecified side */
"G8311"  =  "1"  /* Monoplegia of lower limb affecting right dominant side */
"G8312"  =  "1"  /* Monoplegia of lower limb affecting left dominant side */
"G8313"  =  "1"  /* Monoplegia of lower limb affecting right nondominant side */
"G8314"  =  "1"  /* Monoplegia of lower limb affecting left nondominant side */
"G8320"  =  "1"  /* Monoplegia of upper limb affecting unspecified side */
"G8321"  =  "1"  /* Monoplegia of upper limb affecting right dominant side */
"G8322"  =  "1"  /* Monoplegia of upper limb affecting left dominant side */
"G8323"  =  "1"  /* Monoplegia of upper limb affecting right nondominant side */
"G8324"  =  "1"  /* Monoplegia of upper limb affecting left nondominant side */
"G8330"  =  "1"  /* Monoplegia, unspecified affecting unspecified side */
"G8331"  =  "1"  /* Monoplegia, unspecified affecting right dominant side */
"G8332"  =  "1"  /* Monoplegia, unspecified affecting left dominant side */
"G8333"  =  "1"  /* Monoplegia, unspecified affecting right nondominant side */
"G8334"  =  "1"  /* Monoplegia, unspecified affecting left nondominant side */
"G834"  =  "1"  /* Cauda equina syndrome */
"G835"  =  "1"  /* Locked-in state */
"G8381"  =  "1"  /* Brown-Sequard syndrome */
"G8382"  =  "1"  /* Anterior cord syndrome */
"G8383"  =  "1"  /* Posterior cord syndrome */
"G8384"  =  "1"  /* Todd's paralysis (postepileptic) */
"G8389"  =  "1"  /* Other specified paralytic syndromes */
"G839"  =  "1"  /* Paralytic syndrome, unspecified */
"I69031"  =  "1"  /* Monoplegia of upper limb following nontraumatic subarachnoid hemorrhage affecting right dominant side */
"I69032"  =  "1"  /* Monoplegia of upper limb following nontraumatic subarachnoid hemorrhage affecting left dominant side */
"I69033"  =  "1"  /* Monoplegia of upper limb following nontraumatic subarachnoid hemorrhage affecting right non-dominant side */
"I69034"  =  "1"  /* Monoplegia of upper limb following nontraumatic subarachnoid hemorrhage affecting left non-dominant side */
"I69039"  =  "1"  /* Monoplegia of upper limb following nontraumatic subarachnoid hemorrhage affecting unspecified side */
"I69041"  =  "1"  /* Monoplegia of lower limb following nontraumatic subarachnoid hemorrhage affecting right dominant side */
"I69042"  =  "1"  /* Monoplegia of lower limb following nontraumatic subarachnoid hemorrhage affecting left dominant side */
"I69043"  =  "1"  /* Monoplegia of lower limb following nontraumatic subarachnoid hemorrhage affecting right non-dominant side */
"I69044"  =  "1"  /* Monoplegia of lower limb following nontraumatic subarachnoid hemorrhage affecting left non-dominant side */
"I69049"  =  "1"  /* Monoplegia of lower limb following nontraumatic subarachnoid hemorrhage affecting unspecified side */
"I69051"  =  "1"  /* Hemiplegia and hemiparesis following nontraumatic subarachnoid hemorrhage affecting right dominant side */
"I69052"  =  "1"  /* Hemiplegia and hemiparesis following nontraumatic subarachnoid hemorrhage affecting left dominant side */
"I69053"  =  "1"  /* Hemiplegia and hemiparesis following nontraumatic subarachnoid hemorrhage affecting right non-dominant side */
"I69054"  =  "1"  /* Hemiplegia and hemiparesis following nontraumatic subarachnoid hemorrhage affecting left non-dominant side */
"I69059"  =  "1"  /* Hemiplegia and hemiparesis following nontraumatic subarachnoid hemorrhage affecting unspecified side */
"I69061"  =  "1"  /* Other paralytic syndrome following nontraumatic subarachnoid hemorrhage affecting right dominant side */
"I69062"  =  "1"  /* Other paralytic syndrome following nontraumatic subarachnoid hemorrhage affecting left dominant side */
"I69063"  =  "1"  /* Other paralytic syndrome following nontraumatic subarachnoid hemorrhage affecting right non-dominant side */
"I69064"  =  "1"  /* Other paralytic syndrome following nontraumatic subarachnoid hemorrhage affecting left non-dominant side */
"I69065"  =  "1"  /* Other paralytic syndrome following nontraumatic subarachnoid hemorrhage, bilateral */
"I69069"  =  "1"  /* Other paralytic syndrome following nontraumatic subarachnoid hemorrhage affecting unspecified side */
"I69131"  =  "1"  /* Monoplegia of upper limb following nontraumatic intracerebral hemorrhage affecting right dominant side */
"I69132"  =  "1"  /* Monoplegia of upper limb following nontraumatic intracerebral hemorrhage affecting left dominant side */
"I69133"  =  "1"  /* Monoplegia of upper limb following nontraumatic intracerebral hemorrhage affecting right non-dominant side */
"I69134"  =  "1"  /* Monoplegia of upper limb following nontraumatic intracerebral hemorrhage affecting left non-dominant side */
"I69139"  =  "1"  /* Monoplegia of upper limb following nontraumatic intracerebral hemorrhage affecting unspecified side */
"I69141"  =  "1"  /* Monoplegia of lower limb following nontraumatic intracerebral hemorrhage affecting right dominant side */
"I69142"  =  "1"  /* Monoplegia of lower limb following nontraumatic intracerebral hemorrhage affecting left dominant side */
"I69143"  =  "1"  /* Monoplegia of lower limb following nontraumatic intracerebral hemorrhage affecting right non-dominant side */
"I69144"  =  "1"  /* Monoplegia of lower limb following nontraumatic intracerebral hemorrhage affecting left non-dominant side */
"I69149"  =  "1"  /* Monoplegia of lower limb following nontraumatic intracerebral hemorrhage affecting unspecified side */
"I69151"  =  "1"  /* Hemiplegia and hemiparesis following nontraumatic intracerebral hemorrhage affecting right dominant side */
"I69152"  =  "1"  /* Hemiplegia and hemiparesis following nontraumatic intracerebral hemorrhage affecting left dominant side */
"I69153"  =  "1"  /* Hemiplegia and hemiparesis following nontraumatic intracerebral hemorrhage affecting right non-dominant side */
"I69154"  =  "1"  /* Hemiplegia and hemiparesis following nontraumatic intracerebral hemorrhage affecting left non-dominant side */
"I69159"  =  "1"  /* Hemiplegia and hemiparesis following nontraumatic intracerebral hemorrhage affecting unspecified side */
"I69161"  =  "1"  /* Other paralytic syndrome following nontraumatic intracerebral hemorrhage affecting right dominant side */
"I69162"  =  "1"  /* Other paralytic syndrome following nontraumatic intracerebral hemorrhage affecting left dominant side */
"I69163"  =  "1"  /* Other paralytic syndrome following nontraumatic intracerebral hemorrhage affecting right non-dominant side */
"I69164"  =  "1"  /* Other paralytic syndrome following nontraumatic intracerebral hemorrhage affecting left non-dominant side */
"I69165"  =  "1"  /* Other paralytic syndrome following nontraumatic intracerebral hemorrhage, bilateral */
"I69169"  =  "1"  /* Other paralytic syndrome following nontraumatic intracerebral hemorrhage affecting unspecified side */
"I69231"  =  "1"  /* Monoplegia of upper limb following other nontraumatic intracranial hemorrhage affecting right dominant side */
"I69232"  =  "1"  /* Monoplegia of upper limb following other nontraumatic intracranial hemorrhage affecting left dominant side */
"I69233"  =  "1"  /* Monoplegia of upper limb following other nontraumatic intracranial hemorrhage affecting right non-dominant side */
"I69234"  =  "1"  /* Monoplegia of upper limb following other nontraumatic intracranial hemorrhage affecting left non-dominant side */
"I69239"  =  "1"  /* Monoplegia of upper limb following other nontraumatic intracranial hemorrhage affecting unspecified side */
"I69241"  =  "1"  /* Monoplegia of lower limb following other nontraumatic intracranial hemorrhage affecting right dominant side */
"I69242"  =  "1"  /* Monoplegia of lower limb following other nontraumatic intracranial hemorrhage affecting left dominant side */
"I69243"  =  "1"  /* Monoplegia of lower limb following other nontraumatic intracranial hemorrhage affecting right non-dominant side */
"I69244"  =  "1"  /* Monoplegia of lower limb following other nontraumatic intracranial hemorrhage affecting left non-dominant side */
"I69249"  =  "1"  /* Monoplegia of lower limb following other nontraumatic intracranial hemorrhage affecting unspecified side */
"I69251"  =  "1"  /* Hemiplegia and hemiparesis following other nontraumatic intracranial hemorrhage affecting right dominant side */
"I69252"  =  "1"  /* Hemiplegia and hemiparesis following other nontraumatic intracranial hemorrhage affecting left dominant side */
"I69253"  =  "1"  /* Hemiplegia and hemiparesis following other nontraumatic intracranial hemorrhage affecting right non-dominant side */
"I69254"  =  "1"  /* Hemiplegia and hemiparesis following other nontraumatic intracranial hemorrhage affecting left non-dominant side */
"I69259"  =  "1"  /* Hemiplegia and hemiparesis following other nontraumatic intracranial hemorrhage affecting unspecified side */
"I69261"  =  "1"  /* Other paralytic syndrome following other nontraumatic intracranial hemorrhage affecting right dominant side */
"I69262"  =  "1"  /* Other paralytic syndrome following other nontraumatic intracranial hemorrhage affecting left dominant side */
"I69263"  =  "1"  /* Other paralytic syndrome following other nontraumatic intracranial hemorrhage affecting right non-dominant side */
"I69264"  =  "1"  /* Other paralytic syndrome following other nontraumatic intracranial hemorrhage affecting left non-dominant side */
"I69265"  =  "1"  /* Other paralytic syndrome following other nontraumatic intracranial hemorrhage, bilateral */
"I69269"  =  "1"  /* Other paralytic syndrome following other nontraumatic intracranial hemorrhage affecting unspecified side */
"I69331"  =  "1"  /* Monoplegia of upper limb following cerebral infarction affecting right dominant side */
"I69332"  =  "1"  /* Monoplegia of upper limb following cerebral infarction affecting left dominant side */
"I69333"  =  "1"  /* Monoplegia of upper limb following cerebral infarction affecting right non-dominant side */
"I69334"  =  "1"  /* Monoplegia of upper limb following cerebral infarction affecting left non-dominant side */
"I69339"  =  "1"  /* Monoplegia of upper limb following cerebral infarction affecting unspecified side */
"I69341"  =  "1"  /* Monoplegia of lower limb following cerebral infarction affecting right dominant side */
"I69342"  =  "1"  /* Monoplegia of lower limb following cerebral infarction affecting left dominant side */
"I69343"  =  "1"  /* Monoplegia of lower limb following cerebral infarction affecting right non-dominant side */
"I69344"  =  "1"  /* Monoplegia of lower limb following cerebral infarction affecting left non-dominant side */
"I69349"  =  "1"  /* Monoplegia of lower limb following cerebral infarction affecting unspecified side */
"I69351"  =  "1"  /* Hemiplegia and hemiparesis following cerebral infarction affecting right dominant side */
"I69352"  =  "1"  /* Hemiplegia and hemiparesis following cerebral infarction affecting left dominant side */
"I69353"  =  "1"  /* Hemiplegia and hemiparesis following cerebral infarction affecting right non-dominant side */
"I69354"  =  "1"  /* Hemiplegia and hemiparesis following cerebral infarction affecting left non-dominant side */
"I69359"  =  "1"  /* Hemiplegia and hemiparesis following cerebral infarction affecting unspecified side */
"I69361"  =  "1"  /* Other paralytic syndrome following cerebral infarction affecting right dominant side */
"I69362"  =  "1"  /* Other paralytic syndrome following cerebral infarction affecting left dominant side */
"I69363"  =  "1"  /* Other paralytic syndrome following cerebral infarction affecting right non-dominant side */
"I69364"  =  "1"  /* Other paralytic syndrome following cerebral infarction affecting left non-dominant side */
"I69365"  =  "1"  /* Other paralytic syndrome following cerebral infarction, bilateral */
"I69369"  =  "1"  /* Other paralytic syndrome following cerebral infarction affecting unspecified side */
"I69831"  =  "1"  /* Monoplegia of upper limb following other cerebrovascular disease affecting right dominant side */
"I69832"  =  "1"  /* Monoplegia of upper limb following other cerebrovascular disease affecting left dominant side */
"I69833"  =  "1"  /* Monoplegia of upper limb following other cerebrovascular disease affecting right non-dominant side */
"I69834"  =  "1"  /* Monoplegia of upper limb following other cerebrovascular disease affecting left non-dominant side */
"I69839"  =  "1"  /* Monoplegia of upper limb following other cerebrovascular disease affecting unspecified side */
"I69841"  =  "1"  /* Monoplegia of lower limb following other cerebrovascular disease affecting right dominant side */
"I69842"  =  "1"  /* Monoplegia of lower limb following other cerebrovascular disease affecting left dominant side */
"I69843"  =  "1"  /* Monoplegia of lower limb following other cerebrovascular disease affecting right non-dominant side */
"I69844"  =  "1"  /* Monoplegia of lower limb following other cerebrovascular disease affecting left non-dominant side */
"I69849"  =  "1"  /* Monoplegia of lower limb following other cerebrovascular disease affecting unspecified side */
"I69851"  =  "1"  /* Hemiplegia and hemiparesis following other cerebrovascular disease affecting right dominant side */
"I69852"  =  "1"  /* Hemiplegia and hemiparesis following other cerebrovascular disease affecting left dominant side */
"I69853"  =  "1"  /* Hemiplegia and hemiparesis following other cerebrovascular disease affecting right non-dominant side */
"I69854"  =  "1"  /* Hemiplegia and hemiparesis following other cerebrovascular disease affecting left non-dominant side */
"I69859"  =  "1"  /* Hemiplegia and hemiparesis following other cerebrovascular disease affecting unspecified side */
"I69861"  =  "1"  /* Other paralytic syndrome following other cerebrovascular disease affecting right dominant side */
"I69862"  =  "1"  /* Other paralytic syndrome following other cerebrovascular disease affecting left dominant side */
"I69863"  =  "1"  /* Other paralytic syndrome following other cerebrovascular disease affecting right non-dominant side */
"I69864"  =  "1"  /* Other paralytic syndrome following other cerebrovascular disease affecting left non-dominant side */
"I69865"  =  "1"  /* Other paralytic syndrome following other cerebrovascular disease, bilateral */
"I69869"  =  "1"  /* Other paralytic syndrome following other cerebrovascular disease affecting unspecified side */
"I69931"  =  "1"  /* Monoplegia of upper limb following unspecified cerebrovascular disease affecting right dominant side */
"I69932"  =  "1"  /* Monoplegia of upper limb following unspecified cerebrovascular disease affecting left dominant side */
"I69933"  =  "1"  /* Monoplegia of upper limb following unspecified cerebrovascular disease affecting right non-dominant side */
"I69934"  =  "1"  /* Monoplegia of upper limb following unspecified cerebrovascular disease affecting left non-dominant side */
"I69939"  =  "1"  /* Monoplegia of upper limb following unspecified cerebrovascular disease affecting unspecified side */
"I69941"  =  "1"  /* Monoplegia of lower limb following unspecified cerebrovascular disease affecting right dominant side */
"I69942"  =  "1"  /* Monoplegia of lower limb following unspecified cerebrovascular disease affecting left dominant side */
"I69943"  =  "1"  /* Monoplegia of lower limb following unspecified cerebrovascular disease affecting right non-dominant side */
"I69944"  =  "1"  /* Monoplegia of lower limb following unspecified cerebrovascular disease affecting left non-dominant side */
"I69949"  =  "1"  /* Monoplegia of lower limb following unspecified cerebrovascular disease affecting unspecified side */
"I69951"  =  "1"  /* Hemiplegia and hemiparesis following unspecified cerebrovascular disease affecting right dominant side */
"I69952"  =  "1"  /* Hemiplegia and hemiparesis following unspecified cerebrovascular disease affecting left dominant side */
"I69953"  =  "1"  /* Hemiplegia and hemiparesis following unspecified cerebrovascular disease affecting right non-dominant side */
"I69954"  =  "1"  /* Hemiplegia and hemiparesis following unspecified cerebrovascular disease affecting left non-dominant side */
"I69959"  =  "1"  /* Hemiplegia and hemiparesis following unspecified cerebrovascular disease affecting unspecified side */
"I69961"  =  "1"  /* Other paralytic syndrome following unspecified cerebrovascular disease affecting right dominant side */
"I69962"  =  "1"  /* Other paralytic syndrome following unspecified cerebrovascular disease affecting left dominant side */
"I69963"  =  "1"  /* Other paralytic syndrome following unspecified cerebrovascular disease affecting right non-dominant side */
"I69964"  =  "1"  /* Other paralytic syndrome following unspecified cerebrovascular disease affecting left non-dominant side */
"I69965"  =  "1"  /* Other paralytic syndrome following unspecified cerebrovascular disease, bilateral */
"I69969"  =  "1"  /* Other paralytic syndrome following unspecified cerebrovascular disease affecting unspecified side */
"P9160"  =  "1"  /* Hypoxic ischemic encephalopathy [HIE], unspecified */
"P9162"  =  "1"  /* Moderate hypoxic ischemic encephalopathy [HIE] */
"P9163"  =  "1"  /* Severe hypoxic ischemic encephalopathy [HIE] */
OTHER   = "0" ;


 /* STRATIFIED BY: SPINA BIFIDA */
 /* ICD-10-CM DIAGNOSIS CODES: */

/* VALUE $SPIBIFD  This concept was replaced by another concept for "Spinal bifida or Anoxic brain damage" $SPINABD */
VALUE $SPINABD
"G931"  =  "1"  /* Anoxic brain damage, not elsewhere classified */
"Q050"  =  "1"  /* Cervical spina bifida with hydrocephalus */
"Q051"  =  "1"  /* Thoracic spina bifida with hydrocephalus */
"Q052"  =  "1"  /* Lumbar spina bifida with hydrocephalus */
"Q053"  =  "1"  /* Sacral spina bifida with hydrocephalus */
"Q054"  =  "1"  /* Unspecified spina bifida with hydrocephalus */
"Q055"  =  "1"  /* Cervical spina bifida without hydrocephalus */
"Q056"  =  "1"  /* Thoracic spina bifida without hydrocephalus */
"Q057"  =  "1"  /* Lumbar spina bifida without hydrocephalus */
"Q058"  =  "1"  /* Sacral spina bifida without hydrocephalus */
"Q059"  =  "1"  /* Spina bifida, unspecified */
"Q0701"  =  "1"  /* Arnold-Chiari syndrome with spina bifida */
"Q0703"  =  "1"  /* Arnold-Chiari syndrome with spina bifida and hydrocephalus */
OTHER   = "0" ;


 /* STRATIFIED BY: Anoxic Brain */
 /* ICD-10-CM DIAGNOSIS CODES: */

VALUE $ANOXBD
"G931"  =  "1"  /* Anoxic brain damage, not elsewhere classified */
OTHER   = "0" ;

 /* STRATIFIED BY: other continuous mechanical ventilation code for 96 or more consecutive hours */
 /* ICD-10-CM PROCEDURE CODES: */

VALUE $CMVENP
"5A1955Z"  =  "1"  /* Respiratory Ventilation, Greater than 96 Consecutive Hours */
OTHER   = "0" ;


 /* -------------------------------------- */
 /* FOREIGN BODY LEFT IN DURING PROCEDURE */
 /*   PROVIDER INDICATOR / PDI #  3       */
 /*   AREA     INDICATOR / PDI # TBD      */
 /* ------------------------------------- */

 /* ICD-10-CM DIAGNOSIS CODES: */

VALUE $FOREIID
"T81500A"  =  "1"  /* Unspecified complication of foreign body accidentally left in body following surgical operation, initial encounter */
"T81501A"  =  "1"  /* Unspecified complication of foreign body accidentally left in body following infusion or transfusion, initial encounter */
"T81502A"  =  "1"  /* Unspecified complication of foreign body accidentally left in body following kidney dialysis, initial encounter */
"T81503A"  =  "1"  /* Unspecified complication of foreign body accidentally left in body following injection or immunization, initial encounter */
"T81504A"  =  "1"  /* Unspecified complication of foreign body accidentally left in body following endoscopic examination, initial encounter */
"T81505A"  =  "1"  /* Unspecified complication of foreign body accidentally left in body following heart catheterization, initial encounter */
"T81506A"  =  "1"  /* Unspecified complication of foreign body accidentally left in body following aspiration, puncture or other catheterization, initial encounter */
"T81507A"  =  "1"  /* Unspecified complication of foreign body accidentally left in body following removal of catheter or packing, initial encounter */
"T81508A"  =  "1"  /* Unspecified complication of foreign body accidentally left in body following other procedure, initial encounter */
"T81509A"  =  "1"  /* Unspecified complication of foreign body accidentally left in body following unspecified procedure, initial encounter */
"T81510A"  =  "1"  /* Adhesions due to foreign body accidentally left in body following surgical operation, initial encounter */
"T81511A"  =  "1"  /* Adhesions due to foreign body accidentally left in body following infusion or transfusion, initial encounter */
"T81512A"  =  "1"  /* Adhesions due to foreign body accidentally left in body following kidney dialysis, initial encounter */
"T81513A"  =  "1"  /* Adhesions due to foreign body accidentally left in body following injection or immunization, initial encounter */
"T81514A"  =  "1"  /* Adhesions due to foreign body accidentally left in body following endoscopic examination, initial encounter */
"T81515A"  =  "1"  /* Adhesions due to foreign body accidentally left in body following heart catheterization, initial encounter */
"T81516A"  =  "1"  /* Adhesions due to foreign body accidentally left in body following aspiration, puncture or other catheterization, initial encounter */
"T81517A"  =  "1"  /* Adhesions due to foreign body accidentally left in body following removal of catheter or packing, initial encounter */
"T81518A"  =  "1"  /* Adhesions due to foreign body accidentally left in body following other procedure, initial encounter */
"T81519A"  =  "1"  /* Adhesions due to foreign body accidentally left in body following unspecified procedure, initial encounter */
"T81520A"  =  "1"  /* Obstruction due to foreign body accidentally left in body following surgical operation, initial encounter */
"T81521A"  =  "1"  /* Obstruction due to foreign body accidentally left in body following infusion or transfusion, initial encounter */
"T81522A"  =  "1"  /* Obstruction due to foreign body accidentally left in body following kidney dialysis, initial encounter */
"T81523A"  =  "1"  /* Obstruction due to foreign body accidentally left in body following injection or immunization, initial encounter */
"T81524A"  =  "1"  /* Obstruction due to foreign body accidentally left in body following endoscopic examination, initial encounter */
"T81525A"  =  "1"  /* Obstruction due to foreign body accidentally left in body following heart catheterization, initial encounter */
"T81526A"  =  "1"  /* Obstruction due to foreign body accidentally left in body following aspiration, puncture or other catheterization, initial encounter */
"T81527A"  =  "1"  /* Obstruction due to foreign body accidentally left in body following removal of catheter or packing, initial encounter */
"T81528A"  =  "1"  /* Obstruction due to foreign body accidentally left in body following other procedure, initial encounter */
"T81529A"  =  "1"  /* Obstruction due to foreign body accidentally left in body following unspecified procedure, initial encounter */
"T81530A"  =  "1"  /* Perforation due to foreign body accidentally left in body following surgical operation, initial encounter */
"T81531A"  =  "1"  /* Perforation due to foreign body accidentally left in body following infusion or transfusion, initial encounter */
"T81532A"  =  "1"  /* Perforation due to foreign body accidentally left in body following kidney dialysis, initial encounter */
"T81533A"  =  "1"  /* Perforation due to foreign body accidentally left in body following injection or immunization, initial encounter */
"T81534A"  =  "1"  /* Perforation due to foreign body accidentally left in body following endoscopic examination, initial encounter */
"T81535A"  =  "1"  /* Perforation due to foreign body accidentally left in body following heart catheterization, initial encounter */
"T81536A"  =  "1"  /* Perforation due to foreign body accidentally left in body following aspiration, puncture or other catheterization, initial encounter */
"T81537A"  =  "1"  /* Perforation due to foreign body accidentally left in body following removal of catheter or packing, initial encounter */
"T81538A"  =  "1"  /* Perforation due to foreign body accidentally left in body following other procedure, initial encounter */
"T81539A"  =  "1"  /* Perforation due to foreign body accidentally left in body following unspecified procedure, initial encounter */
"T81590A"  =  "1"  /* Other complications of foreign body accidentally left in body following surgical operation, initial encounter */
"T81591A"  =  "1"  /* Other complications of foreign body accidentally left in body following infusion or transfusion, initial encounter */
"T81592A"  =  "1"  /* Other complications of foreign body accidentally left in body following kidney dialysis, initial encounter */
"T81593A"  =  "1"  /* Other complications of foreign body accidentally left in body following injection or immunization, initial encounter */
"T81594A"  =  "1"  /* Other complications of foreign body accidentally left in body following endoscopic examination, initial encounter */
"T81595A"  =  "1"  /* Other complications of foreign body accidentally left in body following heart catheterization, initial encounter */
"T81596A"  =  "1"  /* Other complications of foreign body accidentally left in body following aspiration, puncture or other catheterization, initial encounter */
"T81597A"  =  "1"  /* Other complications of foreign body accidentally left in body following removal of catheter or packing, initial encounter */
"T81598A"  =  "1"  /* Other complications of foreign body accidentally left in body following other procedure, initial encounter */
"T81599A"  =  "1"  /* Other complications of foreign body accidentally left in body following unspecified procedure, initial encounter */
"T8160XA"  =  "1"  /* Unspecified acute reaction to foreign substance accidentally left during a procedure, initial encounter */
"T8161XA"  =  "1"  /* Aseptic peritonitis due to foreign substance accidentally left during a procedure, initial encounter */
"T8169XA"  =  "1"  /* Other acute reaction to foreign substance accidentally left during a procedure, initial encounter */
OTHER   = "0" ;


 /* ------------------------------- */
 /* IATROGENIC PNEUMOTHORAX         */
 /*   PROVIDER INDICATOR / PDI #4&5 */
 /*   AREA     INDICATOR / PDI #TBD */
 /* ------------------------------- */

 /* ICD-10-CM DIAGNOSIS CODES: */

VALUE $IATROID
"J95811"  =  "1"  /* Postprocedural pneumothorax */
OTHER   = "0" ;

/* EXCLUDE: CHEST TRAUMA */
/* ICD-10-CM DIAGNOSIS CODES: */

VALUE $CTRAUMD
"S128XXA"  =  "1"  /* Fracture of other parts of neck, initial encounter */
"S2000XA"  =  "1"  /* Contusion of breast, unspecified breast, initial encounter */
"S2001XA"  =  "1"  /* Contusion of right breast, initial encounter */
"S2002XA"  =  "1"  /* Contusion of left breast, initial encounter */
"S20101A"  =  "1"  /* Unspecified superficial injuries of breast, right breast, initial encounter */
"S20102A"  =  "1"  /* Unspecified superficial injuries of breast, left breast, initial encounter */
"S20109A"  =  "1"  /* Unspecified superficial injuries of breast, unspecified breast, initial encounter */
"S20111A"  =  "1"  /* Abrasion of breast, right breast, initial encounter */
"S20112A"  =  "1"  /* Abrasion of breast, left breast, initial encounter */
"S20119A"  =  "1"  /* Abrasion of breast, unspecified breast, initial encounter */
"S20141A"  =  "1"  /* External constriction of part of breast, right breast, initial encounter */
"S20142A"  =  "1"  /* External constriction of part of breast, left breast, initial encounter */
"S20149A"  =  "1"  /* External constriction of part of breast, unspecified breast, initial encounter */
"S20171A"  =  "1"  /* Other superficial bite of breast, right breast, initial encounter */
"S20172A"  =  "1"  /* Other superficial bite of breast, left breast, initial encounter */
"S20179A"  =  "1"  /* Other superficial bite of breast, unspecified breast, initial encounter */
"S2020XA"  =  "1"  /* Contusion of thorax, unspecified, initial encounter */
"S20211A"  =  "1"  /* Contusion of right front wall of thorax, initial encounter */
"S20212A"  =  "1"  /* Contusion of left front wall of thorax, initial encounter */
"S20219A"  =  "1"  /* Contusion of unspecified front wall of thorax, initial encounter */
"S20221A"  =  "1"  /* Contusion of right back wall of thorax, initial encounter */
"S20222A"  =  "1"  /* Contusion of left back wall of thorax, initial encounter */
"S20229A"  =  "1"  /* Contusion of unspecified back wall of thorax, initial encounter */
"S20301A"  =  "1"  /* Unspecified superficial injuries of right front wall of thorax, initial encounter */
"S20302A"  =  "1"  /* Unspecified superficial injuries of left front wall of thorax, initial encounter */
"S20309A"  =  "1"  /* Unspecified superficial injuries of unspecified front wall of thorax, initial encounter */
"S20311A"  =  "1"  /* Abrasion of right front wall of thorax, initial encounter */
"S20312A"  =  "1"  /* Abrasion of left front wall of thorax, initial encounter */
"S20319A"  =  "1"  /* Abrasion of unspecified front wall of thorax, initial encounter */
"S20341A"  =  "1"  /* External constriction of right front wall of thorax, initial encounter */
"S20342A"  =  "1"  /* External constriction of left front wall of thorax, initial encounter */
"S20349A"  =  "1"  /* External constriction of unspecified front wall of thorax, initial encounter */
"S20371A"  =  "1"  /* Other superficial bite of right front wall of thorax, initial encounter */
"S20372A"  =  "1"  /* Other superficial bite of left front wall of thorax, initial encounter */
"S20379A"  =  "1"  /* Other superficial bite of unspecified front wall of thorax, initial encounter */
"S20401A"  =  "1"  /* Unspecified superficial injuries of right back wall of thorax, initial encounter */
"S20402A"  =  "1"  /* Unspecified superficial injuries of left back wall of thorax, initial encounter */
"S20409A"  =  "1"  /* Unspecified superficial injuries of unspecified back wall of thorax, initial encounter */
"S20411A"  =  "1"  /* Abrasion of right back wall of thorax, initial encounter */
"S20412A"  =  "1"  /* Abrasion of left back wall of thorax, initial encounter */
"S20419A"  =  "1"  /* Abrasion of unspecified back wall of thorax, initial encounter */
"S20441A"  =  "1"  /* External constriction of right back wall of thorax, initial encounter */
"S20442A"  =  "1"  /* External constriction of left back wall of thorax, initial encounter */
"S20449A"  =  "1"  /* External constriction of unspecified back wall of thorax, initial encounter */
"S20471A"  =  "1"  /* Other superficial bite of right back wall of thorax, initial encounter */
"S20472A"  =  "1"  /* Other superficial bite of left back wall of thorax, initial encounter */
"S20479A"  =  "1"  /* Other superficial bite of unspecified back wall of thorax, initial encounter */
"S2090XA"  =  "1"  /* Unspecified superficial injury of unspecified parts of thorax, initial encounter */
"S2091XA"  =  "1"  /* Abrasion of unspecified parts of thorax, initial encounter */
"S2094XA"  =  "1"  /* External constriction of unspecified parts of thorax, initial encounter */
"S2097XA"  =  "1"  /* Other superficial bite of unspecified parts of thorax, initial encounter */
"S21101A"  =  "1"  /* Unspecified open wound of right front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21102A"  =  "1"  /* Unspecified open wound of left front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21109A"  =  "1"  /* Unspecified open wound of unspecified front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21111A"  =  "1"  /* Laceration without foreign body of right front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21112A"  =  "1"  /* Laceration without foreign body of left front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21119A"  =  "1"  /* Laceration without foreign body of unspecified front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21121A"  =  "1"  /* Laceration with foreign body of right front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21122A"  =  "1"  /* Laceration with foreign body of left front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21129A"  =  "1"  /* Laceration with foreign body of unspecified front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21131A"  =  "1"  /* Puncture wound without foreign body of right front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21132A"  =  "1"  /* Puncture wound without foreign body of left front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21139A"  =  "1"  /* Puncture wound without foreign body of unspecified front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21141A"  =  "1"  /* Puncture wound with foreign body of right front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21142A"  =  "1"  /* Puncture wound with foreign body of left front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21149A"  =  "1"  /* Puncture wound with foreign body of unspecified front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21151A"  =  "1"  /* Open bite of right front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21152A"  =  "1"  /* Open bite of left front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21159A"  =  "1"  /* Open bite of unspecified front wall of thorax without penetration into thoracic cavity, initial encounter */
"S21201A"  =  "1"  /* Unspecified open wound of right back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21202A"  =  "1"  /* Unspecified open wound of left back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21209A"  =  "1"  /* Unspecified open wound of unspecified back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21211A"  =  "1"  /* Laceration without foreign body of right back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21212A"  =  "1"  /* Laceration without foreign body of left back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21219A"  =  "1"  /* Laceration without foreign body of unspecified back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21221A"  =  "1"  /* Laceration with foreign body of right back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21222A"  =  "1"  /* Laceration with foreign body of left back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21229A"  =  "1"  /* Laceration with foreign body of unspecified back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21231A"  =  "1"  /* Puncture wound without foreign body of right back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21232A"  =  "1"  /* Puncture wound without foreign body of left back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21239A"  =  "1"  /* Puncture wound without foreign body of unspecified back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21241A"  =  "1"  /* Puncture wound with foreign body of right back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21242A"  =  "1"  /* Puncture wound with foreign body of left back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21249A"  =  "1"  /* Puncture wound with foreign body of unspecified back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21251A"  =  "1"  /* Open bite of right back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21252A"  =  "1"  /* Open bite of left back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21259A"  =  "1"  /* Open bite of unspecified back wall of thorax without penetration into thoracic cavity, initial encounter */
"S21301A"  =  "1"  /* Unspecified open wound of right front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21302A"  =  "1"  /* Unspecified open wound of left front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21309A"  =  "1"  /* Unspecified open wound of unspecified front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21311A"  =  "1"  /* Laceration without foreign body of right front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21312A"  =  "1"  /* Laceration without foreign body of left front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21319A"  =  "1"  /* Laceration without foreign body of unspecified front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21321A"  =  "1"  /* Laceration with foreign body of right front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21322A"  =  "1"  /* Laceration with foreign body of left front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21329A"  =  "1"  /* Laceration with foreign body of unspecified front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21331A"  =  "1"  /* Puncture wound without foreign body of right front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21332A"  =  "1"  /* Puncture wound without foreign body of left front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21339A"  =  "1"  /* Puncture wound without foreign body of unspecified front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21341A"  =  "1"  /* Puncture wound with foreign body of right front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21342A"  =  "1"  /* Puncture wound with foreign body of left front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21349A"  =  "1"  /* Puncture wound with foreign body of unspecified front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21351A"  =  "1"  /* Open bite of right front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21352A"  =  "1"  /* Open bite of left front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21359A"  =  "1"  /* Open bite of unspecified front wall of thorax with penetration into thoracic cavity, initial encounter */
"S21401A"  =  "1"  /* Unspecified open wound of right back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21402A"  =  "1"  /* Unspecified open wound of left back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21409A"  =  "1"  /* Unspecified open wound of unspecified back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21411A"  =  "1"  /* Laceration without foreign body of right back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21412A"  =  "1"  /* Laceration without foreign body of left back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21419A"  =  "1"  /* Laceration without foreign body of unspecified back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21421A"  =  "1"  /* Laceration with foreign body of right back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21422A"  =  "1"  /* Laceration with foreign body of left back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21429A"  =  "1"  /* Laceration with foreign body of unspecified back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21431A"  =  "1"  /* Puncture wound without foreign body of right back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21432A"  =  "1"  /* Puncture wound without foreign body of left back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21439A"  =  "1"  /* Puncture wound without foreign body of unspecified back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21441A"  =  "1"  /* Puncture wound with foreign body of right back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21442A"  =  "1"  /* Puncture wound with foreign body of left back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21449A"  =  "1"  /* Puncture wound with foreign body of unspecified back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21451A"  =  "1"  /* Open bite of right back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21452A"  =  "1"  /* Open bite of left back wall of thorax with penetration into thoracic cavity, initial encounter */
"S21459A"  =  "1"  /* Open bite of unspecified back wall of thorax with penetration into thoracic cavity, initial encounter */
"S2190XA"  =  "1"  /* Unspecified open wound of unspecified part of thorax, initial encounter */
"S2191XA"  =  "1"  /* Laceration without foreign body of unspecified part of thorax, initial encounter */
"S2192XA"  =  "1"  /* Laceration with foreign body of unspecified part of thorax, initial encounter */
"S2193XA"  =  "1"  /* Puncture wound without foreign body of unspecified part of thorax, initial encounter */
"S2194XA"  =  "1"  /* Puncture wound with foreign body of unspecified part of thorax, initial encounter */
"S2195XA"  =  "1"  /* Open bite of unspecified part of thorax, initial encounter */
"S2220XA"  =  "1"  /* Unspecified fracture of sternum, initial encounter for closed fracture */
"S2220XB"  =  "1"  /* Unspecified fracture of sternum, initial encounter for open fracture */
"S2221XA"  =  "1"  /* Fracture of manubrium, initial encounter for closed fracture */
"S2221XB"  =  "1"  /* Fracture of manubrium, initial encounter for open fracture */
"S2222XA"  =  "1"  /* Fracture of body of sternum, initial encounter for closed fracture */
"S2222XB"  =  "1"  /* Fracture of body of sternum, initial encounter for open fracture */
"S2223XA"  =  "1"  /* Sternal manubrial dissociation, initial encounter for closed fracture */
"S2223XB"  =  "1"  /* Sternal manubrial dissociation, initial encounter for open fracture */
"S2224XA"  =  "1"  /* Fracture of xiphoid process, initial encounter for closed fracture */
"S2224XB"  =  "1"  /* Fracture of xiphoid process, initial encounter for open fracture */
"S2231XA"  =  "1"  /* Fracture of one rib, right side, initial encounter for closed fracture */
"S2231XB"  =  "1"  /* Fracture of one rib, right side, initial encounter for open fracture */
"S2232XA"  =  "1"  /* Fracture of one rib, left side, initial encounter for closed fracture */
"S2232XB"  =  "1"  /* Fracture of one rib, left side, initial encounter for open fracture */
"S2239XA"  =  "1"  /* Fracture of one rib, unspecified side, initial encounter for closed fracture */
"S2239XB"  =  "1"  /* Fracture of one rib, unspecified side, initial encounter for open fracture */
"S2241XA"  =  "1"  /* Multiple fractures of ribs, right side, initial encounter for closed fracture */
"S2241XB"  =  "1"  /* Multiple fractures of ribs, right side, initial encounter for open fracture */
"S2242XA"  =  "1"  /* Multiple fractures of ribs, left side, initial encounter for closed fracture */
"S2242XB"  =  "1"  /* Multiple fractures of ribs, left side, initial encounter for open fracture */
"S2243XA"  =  "1"  /* Multiple fractures of ribs, bilateral, initial encounter for closed fracture */
"S2243XB"  =  "1"  /* Multiple fractures of ribs, bilateral, initial encounter for open fracture */
"S2249XA"  =  "1"  /* Multiple fractures of ribs, unspecified side, initial encounter for closed fracture */
"S2249XB"  =  "1"  /* Multiple fractures of ribs, unspecified side, initial encounter for open fracture */
"S225XXA"  =  "1"  /* Flail chest, initial encounter for closed fracture */
"S225XXB"  =  "1"  /* Flail chest, initial encounter for open fracture */
"S229XXA"  =  "1"  /* Fracture of bony thorax, part unspecified, initial encounter for closed fracture */
"S229XXB"  =  "1"  /* Fracture of bony thorax, part unspecified, initial encounter for open fracture */
"S243XXA"  =  "1"  /* Injury of peripheral nerves of thorax, initial encounter */
"S244XXA"  =  "1"  /* Injury of thoracic sympathetic nervous system, initial encounter */
"S248XXA"  =  "1"  /* Injury of other specified nerves of thorax, initial encounter */
"S249XXA"  =  "1"  /* Injury of unspecified nerve of thorax, initial encounter */
"S2500XA"  =  "1"  /* Unspecified injury of thoracic aorta, initial encounter */
"S2501XA"  =  "1"  /* Minor laceration of thoracic aorta, initial encounter */
"S2502XA"  =  "1"  /* Major laceration of thoracic aorta, initial encounter */
"S2509XA"  =  "1"  /* Other specified injury of thoracic aorta, initial encounter */
"S25101A"  =  "1"  /* Unspecified injury of right innominate or subclavian artery, initial encounter */
"S25102A"  =  "1"  /* Unspecified injury of left innominate or subclavian artery, initial encounter */
"S25109A"  =  "1"  /* Unspecified injury of unspecified innominate or subclavian artery, initial encounter */
"S25111A"  =  "1"  /* Minor laceration of right innominate or subclavian artery, initial encounter */
"S25112A"  =  "1"  /* Minor laceration of left innominate or subclavian artery, initial encounter */
"S25119A"  =  "1"  /* Minor laceration of unspecified innominate or subclavian artery, initial encounter */
"S25121A"  =  "1"  /* Major laceration of right innominate or subclavian artery, initial encounter */
"S25122A"  =  "1"  /* Major laceration of left innominate or subclavian artery, initial encounter */
"S25129A"  =  "1"  /* Major laceration of unspecified innominate or subclavian artery, initial encounter */
"S25191A"  =  "1"  /* Other specified injury of right innominate or subclavian artery, initial encounter */
"S25192A"  =  "1"  /* Other specified injury of left innominate or subclavian artery, initial encounter */
"S25199A"  =  "1"  /* Other specified injury of unspecified innominate or subclavian artery, initial encounter */
"S2520XA"  =  "1"  /* Unspecified injury of superior vena cava, initial encounter */
"S2521XA"  =  "1"  /* Minor laceration of superior vena cava, initial encounter */
"S2522XA"  =  "1"  /* Major laceration of superior vena cava, initial encounter */
"S2529XA"  =  "1"  /* Other specified injury of superior vena cava, initial encounter */
"S25301A"  =  "1"  /* Unspecified injury of right innominate or subclavian vein, initial encounter */
"S25302A"  =  "1"  /* Unspecified injury of left innominate or subclavian vein, initial encounter */
"S25309A"  =  "1"  /* Unspecified injury of unspecified innominate or subclavian vein, initial encounter */
"S25311A"  =  "1"  /* Minor laceration of right innominate or subclavian vein, initial encounter */
"S25312A"  =  "1"  /* Minor laceration of left innominate or subclavian vein, initial encounter */
"S25319A"  =  "1"  /* Minor laceration of unspecified innominate or subclavian vein, initial encounter */
"S25321A"  =  "1"  /* Major laceration of right innominate or subclavian vein, initial encounter */
"S25322A"  =  "1"  /* Major laceration of left innominate or subclavian vein, initial encounter */
"S25329A"  =  "1"  /* Major laceration of unspecified innominate or subclavian vein, initial encounter */
"S25391A"  =  "1"  /* Other specified injury of right innominate or subclavian vein, initial encounter */
"S25392A"  =  "1"  /* Other specified injury of left innominate or subclavian vein, initial encounter */
"S25399A"  =  "1"  /* Other specified injury of unspecified innominate or subclavian vein, initial encounter */
"S25401A"  =  "1"  /* Unspecified injury of right pulmonary blood vessels, initial encounter */
"S25402A"  =  "1"  /* Unspecified injury of left pulmonary blood vessels, initial encounter */
"S25409A"  =  "1"  /* Unspecified injury of unspecified pulmonary blood vessels, initial encounter */
"S25411A"  =  "1"  /* Minor laceration of right pulmonary blood vessels, initial encounter */
"S25412A"  =  "1"  /* Minor laceration of left pulmonary blood vessels, initial encounter */
"S25419A"  =  "1"  /* Minor laceration of unspecified pulmonary blood vessels, initial encounter */
"S25421A"  =  "1"  /* Major laceration of right pulmonary blood vessels, initial encounter */
"S25422A"  =  "1"  /* Major laceration of left pulmonary blood vessels, initial encounter */
"S25429A"  =  "1"  /* Major laceration of unspecified pulmonary blood vessels, initial encounter */
"S25491A"  =  "1"  /* Other specified injury of right pulmonary blood vessels, initial encounter */
"S25492A"  =  "1"  /* Other specified injury of left pulmonary blood vessels, initial encounter */
"S25499A"  =  "1"  /* Other specified injury of unspecified pulmonary blood vessels, initial encounter */
"S25501A"  =  "1"  /* Unspecified injury of intercostal blood vessels, right side, initial encounter */
"S25502A"  =  "1"  /* Unspecified injury of intercostal blood vessels, left side, initial encounter */
"S25509A"  =  "1"  /* Unspecified injury of intercostal blood vessels, unspecified side, initial encounter */
"S25511A"  =  "1"  /* Laceration of intercostal blood vessels, right side, initial encounter */
"S25512A"  =  "1"  /* Laceration of intercostal blood vessels, left side, initial encounter */
"S25519A"  =  "1"  /* Laceration of intercostal blood vessels, unspecified side, initial encounter */
"S25591A"  =  "1"  /* Other specified injury of intercostal blood vessels, right side, initial encounter */
"S25592A"  =  "1"  /* Other specified injury of intercostal blood vessels, left side, initial encounter */
"S25599A"  =  "1"  /* Other specified injury of intercostal blood vessels, unspecified side, initial encounter */
"S25801A"  =  "1"  /* Unspecified injury of other blood vessels of thorax, right side, initial encounter */
"S25802A"  =  "1"  /* Unspecified injury of other blood vessels of thorax, left side, initial encounter */
"S25809A"  =  "1"  /* Unspecified injury of other blood vessels of thorax, unspecified side, initial encounter */
"S25811A"  =  "1"  /* Laceration of other blood vessels of thorax, right side, initial encounter */
"S25812A"  =  "1"  /* Laceration of other blood vessels of thorax, left side, initial encounter */
"S25819A"  =  "1"  /* Laceration of other blood vessels of thorax, unspecified side, initial encounter */
"S25891A"  =  "1"  /* Other specified injury of other blood vessels of thorax, right side, initial encounter */
"S25892A"  =  "1"  /* Other specified injury of other blood vessels of thorax, left side, initial encounter */
"S25899A"  =  "1"  /* Other specified injury of other blood vessels of thorax, unspecified side, initial encounter */
"S2590XA"  =  "1"  /* Unspecified injury of unspecified blood vessel of thorax, initial encounter */
"S2591XA"  =  "1"  /* Laceration of unspecified blood vessel of thorax, initial encounter */
"S2599XA"  =  "1"  /* Other specified injury of unspecified blood vessel of thorax, initial encounter */
"S2600XA"  =  "1"  /* Unspecified injury of heart with hemopericardium, initial encounter */
"S2601XA"  =  "1"  /* Contusion of heart with hemopericardium, initial encounter */
"S26020A"  =  "1"  /* Mild laceration of heart with hemopericardium, initial encounter */
"S26021A"  =  "1"  /* Moderate laceration of heart with hemopericardium, initial encounter */
"S26022A"  =  "1"  /* Major laceration of heart with hemopericardium, initial encounter */
"S2609XA"  =  "1"  /* Other injury of heart with hemopericardium, initial encounter */
"S2610XA"  =  "1"  /* Unspecified injury of heart without hemopericardium, initial encounter */
"S2611XA"  =  "1"  /* Contusion of heart without hemopericardium, initial encounter */
"S2612XA"  =  "1"  /* Laceration of heart without hemopericardium, initial encounter */
"S2619XA"  =  "1"  /* Other injury of heart without hemopericardium, initial encounter */
"S2690XA"  =  "1"  /* Unspecified injury of heart, unspecified with or without hemopericardium, initial encounter */
"S2691XA"  =  "1"  /* Contusion of heart, unspecified with or without hemopericardium, initial encounter */
"S2692XA"  =  "1"  /* Laceration of heart, unspecified with or without hemopericardium, initial encounter */
"S2699XA"  =  "1"  /* Other injury of heart, unspecified with or without hemopericardium, initial encounter */
"S270XXA"  =  "1"  /* Traumatic pneumothorax, initial encounter */
"S271XXA"  =  "1"  /* Traumatic hemothorax, initial encounter */
"S272XXA"  =  "1"  /* Traumatic hemopneumothorax, initial encounter */
"S27301A"  =  "1"  /* Unspecified injury of lung, unilateral, initial encounter */
"S27302A"  =  "1"  /* Unspecified injury of lung, bilateral, initial encounter */
"S27309A"  =  "1"  /* Unspecified injury of lung, unspecified, initial encounter */
"S27311A"  =  "1"  /* Primary blast injury of lung, unilateral, initial encounter */
"S27312A"  =  "1"  /* Primary blast injury of lung, bilateral, initial encounter */
"S27319A"  =  "1"  /* Primary blast injury of lung, unspecified, initial encounter */
"S27321A"  =  "1"  /* Contusion of lung, unilateral, initial encounter */
"S27322A"  =  "1"  /* Contusion of lung, bilateral, initial encounter */
"S27329A"  =  "1"  /* Contusion of lung, unspecified, initial encounter */
"S27331A"  =  "1"  /* Laceration of lung, unilateral, initial encounter */
"S27332A"  =  "1"  /* Laceration of lung, bilateral, initial encounter */
"S27339A"  =  "1"  /* Laceration of lung, unspecified, initial encounter */
"S27391A"  =  "1"  /* Other injuries of lung, unilateral, initial encounter */
"S27392A"  =  "1"  /* Other injuries of lung, bilateral, initial encounter */
"S27399A"  =  "1"  /* Other injuries of lung, unspecified, initial encounter */
"S27401A"  =  "1"  /* Unspecified injury of bronchus, unilateral, initial encounter */
"S27402A"  =  "1"  /* Unspecified injury of bronchus, bilateral, initial encounter */
"S27409A"  =  "1"  /* Unspecified injury of bronchus, unspecified, initial encounter */
"S27411A"  =  "1"  /* Primary blast injury of bronchus, unilateral, initial encounter */
"S27412A"  =  "1"  /* Primary blast injury of bronchus, bilateral, initial encounter */
"S27419A"  =  "1"  /* Primary blast injury of bronchus, unspecified, initial encounter */
"S27421A"  =  "1"  /* Contusion of bronchus, unilateral, initial encounter */
"S27422A"  =  "1"  /* Contusion of bronchus, bilateral, initial encounter */
"S27429A"  =  "1"  /* Contusion of bronchus, unspecified, initial encounter */
"S27431A"  =  "1"  /* Laceration of bronchus, unilateral, initial encounter */
"S27432A"  =  "1"  /* Laceration of bronchus, bilateral, initial encounter */
"S27439A"  =  "1"  /* Laceration of bronchus, unspecified, initial encounter */
"S27491A"  =  "1"  /* Other injury of bronchus, unilateral, initial encounter */
"S27492A"  =  "1"  /* Other injury of bronchus, bilateral, initial encounter */
"S27499A"  =  "1"  /* Other injury of bronchus, unspecified, initial encounter */
"S2750XA"  =  "1"  /* Unspecified injury of thoracic trachea, initial encounter */
"S2751XA"  =  "1"  /* Primary blast injury of thoracic trachea, initial encounter */
"S2752XA"  =  "1"  /* Contusion of thoracic trachea, initial encounter */
"S2753XA"  =  "1"  /* Laceration of thoracic trachea, initial encounter */
"S2759XA"  =  "1"  /* Other injury of thoracic trachea, initial encounter */
"S2760XA"  =  "1"  /* Unspecified injury of pleura, initial encounter */
"S2763XA"  =  "1"  /* Laceration of pleura, initial encounter */
"S2769XA"  =  "1"  /* Other injury of pleura, initial encounter */
"S27802A"  =  "1"  /* Contusion of diaphragm, initial encounter */
"S27803A"  =  "1"  /* Laceration of diaphragm, initial encounter */
"S27808A"  =  "1"  /* Other injury of diaphragm, initial encounter */
"S27809A"  =  "1"  /* Unspecified injury of diaphragm, initial encounter */
"S27812A"  =  "1"  /* Contusion of esophagus (thoracic part), initial encounter */
"S27813A"  =  "1"  /* Laceration of esophagus (thoracic part), initial encounter */
"S27818A"  =  "1"  /* Other injury of esophagus (thoracic part), initial encounter */
"S27819A"  =  "1"  /* Unspecified injury of esophagus (thoracic part), initial encounter */
"S27892A"  =  "1"  /* Contusion of other specified intrathoracic organs, initial encounter */
"S27893A"  =  "1"  /* Laceration of other specified intrathoracic organs, initial encounter */
"S27898A"  =  "1"  /* Other injury of other specified intrathoracic organs, initial encounter */
"S27899A"  =  "1"  /* Unspecified injury of other specified intrathoracic organs, initial encounter */
"S279XXA"  =  "1"  /* Injury of unspecified intrathoracic organ, initial encounter */
"S280XXA"  =  "1"  /* Crushed chest, initial encounter */
"S281XXA"  =  "1"  /* Traumatic amputation (partial) of part of thorax, except breast, initial encounter */
"S29001A"  =  "1"  /* Unspecified injury of muscle and tendon of front wall of thorax, initial encounter */
"S29002A"  =  "1"  /* Unspecified injury of muscle and tendon of back wall of thorax, initial encounter */
"S29009A"  =  "1"  /* Unspecified injury of muscle and tendon of unspecified wall of thorax, initial encounter */
"S29021A"  =  "1"  /* Laceration of muscle and tendon of front wall of thorax, initial encounter */
"S29022A"  =  "1"  /* Laceration of muscle and tendon of back wall of thorax, initial encounter */
"S29029A"  =  "1"  /* Laceration of muscle and tendon of unspecified wall of thorax, initial encounter */
"S29091A"  =  "1"  /* Other injury of muscle and tendon of front wall of thorax, initial encounter */
"S29092A"  =  "1"  /* Other injury of muscle and tendon of back wall of thorax, initial encounter */
"S29099A"  =  "1"  /* Other injury of muscle and tendon of unspecified wall of thorax, initial encounter */
"S298XXA"  =  "1"  /* Other specified injuries of thorax, initial encounter */
"S299XXA"  =  "1"  /* Unspecified injury of thorax, initial encounter */
OTHER   = "0" ;

 /* EXCLUDE: PLEURAL EFFUSION */
 /* ICD-10-CM DIAGNOSIS CODES: */;

VALUE $PLEURAD
"A150"  =  "1"  /* Tuberculosis of lung */
"A156"  =  "1"  /* Tuberculous pleurisy */
"C782"  =  "1"  /* Secondary malignant neoplasm of pleura */
"J90"  =  "1"  /* Pleural effusion, not elsewhere classified */
"J910"  =  "1"  /* Malignant pleural effusion */
"J918"  =  "1"  /* Pleural effusion in other conditions classified elsewhere */
"J940"  =  "1"  /* Chylous effusion */
"J942"  =  "1"  /* Hemothorax */
"J948"  =  "1"  /* Other specified pleural conditions */
OTHER    = "0" ;

 /* EXCLUDE: THORACIC SURGERY */
 /* ICD-10-CM PROCEDURE CODES: */

VALUE $THORAIP
"01520ZZ"  =  "1"  /* Destruction of Phrenic Nerve, Open Approach */
"01523ZZ"  =  "1"  /* Destruction of Phrenic Nerve, Percutaneous Approach */
"01524ZZ"  =  "1"  /* Destruction of Phrenic Nerve, Percutaneous Endoscopic Approach */
"015K0ZZ"  =  "1"  /* Destruction of Head and Neck Sympathetic Nerve, Open Approach */
"015K3ZZ"  =  "1"  /* Destruction of Head and Neck Sympathetic Nerve, Percutaneous Approach */
"015K4ZZ"  =  "1"  /* Destruction of Head and Neck Sympathetic Nerve, Percutaneous Endoscopic Approach */
"015L0ZZ"  =  "1"  /* Destruction of Thoracic Sympathetic Nerve, Open Approach */
"015L3ZZ"  =  "1"  /* Destruction of Thoracic Sympathetic Nerve, Percutaneous Approach */
"015L4ZZ"  =  "1"  /* Destruction of Thoracic Sympathetic Nerve, Percutaneous Endoscopic Approach */
"015M0ZZ"  =  "1"  /* Destruction of Abdominal Sympathetic Nerve, Open Approach */
"015M3ZZ"  =  "1"  /* Destruction of Abdominal Sympathetic Nerve, Percutaneous Approach */
"015M4ZZ"  =  "1"  /* Destruction of Abdominal Sympathetic Nerve, Percutaneous Endoscopic Approach */
"015N0ZZ"  =  "1"  /* Destruction of Lumbar Sympathetic Nerve, Open Approach */
"015N3ZZ"  =  "1"  /* Destruction of Lumbar Sympathetic Nerve, Percutaneous Approach */
"015N4ZZ"  =  "1"  /* Destruction of Lumbar Sympathetic Nerve, Percutaneous Endoscopic Approach */
"01BK0ZZ"  =  "1"  /* Excision of Head and Neck Sympathetic Nerve, Open Approach */
"01BK3ZZ"  =  "1"  /* Excision of Head and Neck Sympathetic Nerve, Percutaneous Approach */
"01BK4ZZ"  =  "1"  /* Excision of Head and Neck Sympathetic Nerve, Percutaneous Endoscopic Approach */
"01BL0ZZ"  =  "1"  /* Excision of Thoracic Sympathetic Nerve, Open Approach */
"01BL3ZZ"  =  "1"  /* Excision of Thoracic Sympathetic Nerve, Percutaneous Approach */
"01BL4ZZ"  =  "1"  /* Excision of Thoracic Sympathetic Nerve, Percutaneous Endoscopic Approach */
"01BM0ZZ"  =  "1"  /* Excision of Abdominal Sympathetic Nerve, Open Approach */
"01BM3ZZ"  =  "1"  /* Excision of Abdominal Sympathetic Nerve, Percutaneous Approach */
"01BM4ZZ"  =  "1"  /* Excision of Abdominal Sympathetic Nerve, Percutaneous Endoscopic Approach */
"01BN0ZZ"  =  "1"  /* Excision of Lumbar Sympathetic Nerve, Open Approach */
"01BN3ZZ"  =  "1"  /* Excision of Lumbar Sympathetic Nerve, Percutaneous Approach */
"01BN4ZZ"  =  "1"  /* Excision of Lumbar Sympathetic Nerve, Percutaneous Endoscopic Approach */
"01DK0ZZ"  =  "1"  /* Extraction of Head and Neck Sympathetic Nerve, Open Approach */
"01DK3ZZ"  =  "1"  /* Extraction of Head and Neck Sympathetic Nerve, Percutaneous Approach */
"01DK4ZZ"  =  "1"  /* Extraction of Head and Neck Sympathetic Nerve, Percutaneous Endoscopic Approach */
"01DL0ZZ"  =  "1"  /* Extraction of Thoracic Sympathetic Nerve, Open Approach */
"01DL3ZZ"  =  "1"  /* Extraction of Thoracic Sympathetic Nerve, Percutaneous Approach */
"01DL4ZZ"  =  "1"  /* Extraction of Thoracic Sympathetic Nerve, Percutaneous Endoscopic Approach */
"01DM0ZZ"  =  "1"  /* Extraction of Abdominal Sympathetic Nerve, Open Approach */
"01DM3ZZ"  =  "1"  /* Extraction of Abdominal Sympathetic Nerve, Percutaneous Approach */
"01DM4ZZ"  =  "1"  /* Extraction of Abdominal Sympathetic Nerve, Percutaneous Endoscopic Approach */
"01DN0ZZ"  =  "1"  /* Extraction of Lumbar Sympathetic Nerve, Open Approach */
"01DN3ZZ"  =  "1"  /* Extraction of Lumbar Sympathetic Nerve, Percutaneous Approach */
"01DN4ZZ"  =  "1"  /* Extraction of Lumbar Sympathetic Nerve, Percutaneous Endoscopic Approach */
"02JA0ZZ"  =  "1"  /* Inspection of Heart, Open Approach */
"02JA4ZZ"  =  "1"  /* Inspection of Heart, Percutaneous Endoscopic Approach */
"02JY0ZZ"  =  "1"  /* Inspection of Great Vessel, Open Approach */
"02JY4ZZ"  =  "1"  /* Inspection of Great Vessel, Percutaneous Endoscopic Approach */
"02YA0Z0"  =  "1"  /* Transplantation of Heart, Allogeneic, Open Approach */
"02YA0Z1"  =  "1"  /* Transplantation of Heart, Syngeneic, Open Approach */
"02YA0Z2"  =  "1"  /* Transplantation of Heart, Zooplastic, Open Approach */
"075K0ZZ"  =  "1"  /* Destruction of Thoracic Duct, Open Approach */
"075K3ZZ"  =  "1"  /* Destruction of Thoracic Duct, Percutaneous Approach */
"075K4ZZ"  =  "1"  /* Destruction of Thoracic Duct, Percutaneous Endoscopic Approach */
"075L0ZZ"  =  "1"  /* Destruction of Cisterna Chyli, Open Approach */
"075L3ZZ"  =  "1"  /* Destruction of Cisterna Chyli, Percutaneous Approach */
"075L4ZZ"  =  "1"  /* Destruction of Cisterna Chyli, Percutaneous Endoscopic Approach */
"075M0ZZ"  =  "1"  /* Destruction of Thymus, Open Approach */
"075M3ZZ"  =  "1"  /* Destruction of Thymus, Percutaneous Approach */
"075M4ZZ"  =  "1"  /* Destruction of Thymus, Percutaneous Endoscopic Approach */
"079K00Z"  =  "1"  /* Drainage of Thoracic Duct with Drainage Device, Open Approach */
"079K0ZZ"  =  "1"  /* Drainage of Thoracic Duct, Open Approach */
"079K30Z"  =  "1"  /* Drainage of Thoracic Duct with Drainage Device, Percutaneous Approach */
"079K3ZZ"  =  "1"  /* Drainage of Thoracic Duct, Percutaneous Approach */
"079K40Z"  =  "1"  /* Drainage of Thoracic Duct with Drainage Device, Percutaneous Endoscopic Approach */
"079K4ZZ"  =  "1"  /* Drainage of Thoracic Duct, Percutaneous Endoscopic Approach */
"079L00Z"  =  "1"  /* Drainage of Cisterna Chyli with Drainage Device, Open Approach */
"079L0ZZ"  =  "1"  /* Drainage of Cisterna Chyli, Open Approach */
"079L30Z"  =  "1"  /* Drainage of Cisterna Chyli with Drainage Device, Percutaneous Approach */
"079L3ZZ"  =  "1"  /* Drainage of Cisterna Chyli, Percutaneous Approach */
"079L40Z"  =  "1"  /* Drainage of Cisterna Chyli with Drainage Device, Percutaneous Endoscopic Approach */
"079L4ZZ"  =  "1"  /* Drainage of Cisterna Chyli, Percutaneous Endoscopic Approach */
"07BK0ZZ"  =  "1"  /* Excision of Thoracic Duct, Open Approach */
"07BK3ZZ"  =  "1"  /* Excision of Thoracic Duct, Percutaneous Approach */
"07BK4ZZ"  =  "1"  /* Excision of Thoracic Duct, Percutaneous Endoscopic Approach */
"07BL0ZZ"  =  "1"  /* Excision of Cisterna Chyli, Open Approach */
"07BL3ZZ"  =  "1"  /* Excision of Cisterna Chyli, Percutaneous Approach */
"07BL4ZZ"  =  "1"  /* Excision of Cisterna Chyli, Percutaneous Endoscopic Approach */
"07BM0ZZ"  =  "1"  /* Excision of Thymus, Open Approach */
"07BM3ZZ"  =  "1"  /* Excision of Thymus, Percutaneous Approach */
"07BM4ZZ"  =  "1"  /* Excision of Thymus, Percutaneous Endoscopic Approach */
"07LK0CZ"  =  "1"  /* Occlusion of Thoracic Duct with Extraluminal Device, Open Approach */
"07LK0DZ"  =  "1"  /* Occlusion of Thoracic Duct with Intraluminal Device, Open Approach */
"07LK0ZZ"  =  "1"  /* Occlusion of Thoracic Duct, Open Approach */
"07LK3CZ"  =  "1"  /* Occlusion of Thoracic Duct with Extraluminal Device, Percutaneous Approach */
"07LK3DZ"  =  "1"  /* Occlusion of Thoracic Duct with Intraluminal Device, Percutaneous Approach */
"07LK3ZZ"  =  "1"  /* Occlusion of Thoracic Duct, Percutaneous Approach */
"07LK4CZ"  =  "1"  /* Occlusion of Thoracic Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"07LK4DZ"  =  "1"  /* Occlusion of Thoracic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"07LK4ZZ"  =  "1"  /* Occlusion of Thoracic Duct, Percutaneous Endoscopic Approach */
"07LL0CZ"  =  "1"  /* Occlusion of Cisterna Chyli with Extraluminal Device, Open Approach */
"07LL0DZ"  =  "1"  /* Occlusion of Cisterna Chyli with Intraluminal Device, Open Approach */
"07LL0ZZ"  =  "1"  /* Occlusion of Cisterna Chyli, Open Approach */
"07LL3CZ"  =  "1"  /* Occlusion of Cisterna Chyli with Extraluminal Device, Percutaneous Approach */
"07LL3DZ"  =  "1"  /* Occlusion of Cisterna Chyli with Intraluminal Device, Percutaneous Approach */
"07LL3ZZ"  =  "1"  /* Occlusion of Cisterna Chyli, Percutaneous Approach */
"07LL4CZ"  =  "1"  /* Occlusion of Cisterna Chyli with Extraluminal Device, Percutaneous Endoscopic Approach */
"07LL4DZ"  =  "1"  /* Occlusion of Cisterna Chyli with Intraluminal Device, Percutaneous Endoscopic Approach */
"07LL4ZZ"  =  "1"  /* Occlusion of Cisterna Chyli, Percutaneous Endoscopic Approach */
"07NK0ZZ"  =  "1"  /* Release Thoracic Duct, Open Approach */
"07NK3ZZ"  =  "1"  /* Release Thoracic Duct, Percutaneous Approach */
"07NK4ZZ"  =  "1"  /* Release Thoracic Duct, Percutaneous Endoscopic Approach */
"07NL0ZZ"  =  "1"  /* Release Cisterna Chyli, Open Approach */
"07NL3ZZ"  =  "1"  /* Release Cisterna Chyli, Percutaneous Approach */
"07NL4ZZ"  =  "1"  /* Release Cisterna Chyli, Percutaneous Endoscopic Approach */
"07PK00Z"  =  "1"  /* Removal of Drainage Device from Thoracic Duct, Open Approach */
"07PK03Z"  =  "1"  /* Removal of Infusion Device from Thoracic Duct, Open Approach */
"07PK0CZ"  =  "1"  /* Removal of Extraluminal Device from Thoracic Duct, Open Approach */
"07PK0DZ"  =  "1"  /* Removal of Intraluminal Device from Thoracic Duct, Open Approach */
"07PK30Z"  =  "1"  /* Removal of Drainage Device from Thoracic Duct, Percutaneous Approach */
"07PK33Z"  =  "1"  /* Removal of Infusion Device from Thoracic Duct, Percutaneous Approach */
"07PK3CZ"  =  "1"  /* Removal of Extraluminal Device from Thoracic Duct, Percutaneous Approach */
"07PK3DZ"  =  "1"  /* Removal of Intraluminal Device from Thoracic Duct, Percutaneous Approach */
"07PK40Z"  =  "1"  /* Removal of Drainage Device from Thoracic Duct, Percutaneous Endoscopic Approach */
"07PK43Z"  =  "1"  /* Removal of Infusion Device from Thoracic Duct, Percutaneous Endoscopic Approach */
"07PK4CZ"  =  "1"  /* Removal of Extraluminal Device from Thoracic Duct, Percutaneous Endoscopic Approach */
"07PK4DZ"  =  "1"  /* Removal of Intraluminal Device from Thoracic Duct, Percutaneous Endoscopic Approach */
"07PL00Z"  =  "1"  /* Removal of Drainage Device from Cisterna Chyli, Open Approach */
"07PL03Z"  =  "1"  /* Removal of Infusion Device from Cisterna Chyli, Open Approach */
"07PL0CZ"  =  "1"  /* Removal of Extraluminal Device from Cisterna Chyli, Open Approach */
"07PL0DZ"  =  "1"  /* Removal of Intraluminal Device from Cisterna Chyli, Open Approach */
"07PL30Z"  =  "1"  /* Removal of Drainage Device from Cisterna Chyli, Percutaneous Approach */
"07PL33Z"  =  "1"  /* Removal of Infusion Device from Cisterna Chyli, Percutaneous Approach */
"07PL3CZ"  =  "1"  /* Removal of Extraluminal Device from Cisterna Chyli, Percutaneous Approach */
"07PL3DZ"  =  "1"  /* Removal of Intraluminal Device from Cisterna Chyli, Percutaneous Approach */
"07PL40Z"  =  "1"  /* Removal of Drainage Device from Cisterna Chyli, Percutaneous Endoscopic Approach */
"07PL43Z"  =  "1"  /* Removal of Infusion Device from Cisterna Chyli, Percutaneous Endoscopic Approach */
"07PL4CZ"  =  "1"  /* Removal of Extraluminal Device from Cisterna Chyli, Percutaneous Endoscopic Approach */
"07PL4DZ"  =  "1"  /* Removal of Intraluminal Device from Cisterna Chyli, Percutaneous Endoscopic Approach */
"07QK0ZZ"  =  "1"  /* Repair Thoracic Duct, Open Approach */
"07QK3ZZ"  =  "1"  /* Repair Thoracic Duct, Percutaneous Approach */
"07QK4ZZ"  =  "1"  /* Repair Thoracic Duct, Percutaneous Endoscopic Approach */
"07QL0ZZ"  =  "1"  /* Repair Cisterna Chyli, Open Approach */
"07QL3ZZ"  =  "1"  /* Repair Cisterna Chyli, Percutaneous Approach */
"07QL4ZZ"  =  "1"  /* Repair Cisterna Chyli, Percutaneous Endoscopic Approach */
"07TK0ZZ"  =  "1"  /* Resection of Thoracic Duct, Open Approach */
"07TK4ZZ"  =  "1"  /* Resection of Thoracic Duct, Percutaneous Endoscopic Approach */
"07TL0ZZ"  =  "1"  /* Resection of Cisterna Chyli, Open Approach */
"07TL4ZZ"  =  "1"  /* Resection of Cisterna Chyli, Percutaneous Endoscopic Approach */
"07TM0ZZ"  =  "1"  /* Resection of Thymus, Open Approach */
"07TM4ZZ"  =  "1"  /* Resection of Thymus, Percutaneous Endoscopic Approach */
"07UK07Z"  =  "1"  /* Supplement Thoracic Duct with Autologous Tissue Substitute, Open Approach */
"07UK0JZ"  =  "1"  /* Supplement Thoracic Duct with Synthetic Substitute, Open Approach */
"07UK0KZ"  =  "1"  /* Supplement Thoracic Duct with Nonautologous Tissue Substitute, Open Approach */
"07UK47Z"  =  "1"  /* Supplement Thoracic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"07UK4JZ"  =  "1"  /* Supplement Thoracic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"07UK4KZ"  =  "1"  /* Supplement Thoracic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"07UL07Z"  =  "1"  /* Supplement Cisterna Chyli with Autologous Tissue Substitute, Open Approach */
"07UL0JZ"  =  "1"  /* Supplement Cisterna Chyli with Synthetic Substitute, Open Approach */
"07UL0KZ"  =  "1"  /* Supplement Cisterna Chyli with Nonautologous Tissue Substitute, Open Approach */
"07UL47Z"  =  "1"  /* Supplement Cisterna Chyli with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"07UL4JZ"  =  "1"  /* Supplement Cisterna Chyli with Synthetic Substitute, Percutaneous Endoscopic Approach */
"07UL4KZ"  =  "1"  /* Supplement Cisterna Chyli with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"07VK0CZ"  =  "1"  /* Restriction of Thoracic Duct with Extraluminal Device, Open Approach */
"07VK0DZ"  =  "1"  /* Restriction of Thoracic Duct with Intraluminal Device, Open Approach */
"07VK0ZZ"  =  "1"  /* Restriction of Thoracic Duct, Open Approach */
"07VK3CZ"  =  "1"  /* Restriction of Thoracic Duct with Extraluminal Device, Percutaneous Approach */
"07VK3DZ"  =  "1"  /* Restriction of Thoracic Duct with Intraluminal Device, Percutaneous Approach */
"07VK3ZZ"  =  "1"  /* Restriction of Thoracic Duct, Percutaneous Approach */
"07VK4CZ"  =  "1"  /* Restriction of Thoracic Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"07VK4DZ"  =  "1"  /* Restriction of Thoracic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"07VK4ZZ"  =  "1"  /* Restriction of Thoracic Duct, Percutaneous Endoscopic Approach */
"07VL0CZ"  =  "1"  /* Restriction of Cisterna Chyli with Extraluminal Device, Open Approach */
"07VL0DZ"  =  "1"  /* Restriction of Cisterna Chyli with Intraluminal Device, Open Approach */
"07VL0ZZ"  =  "1"  /* Restriction of Cisterna Chyli, Open Approach */
"07VL3CZ"  =  "1"  /* Restriction of Cisterna Chyli with Extraluminal Device, Percutaneous Approach */
"07VL3DZ"  =  "1"  /* Restriction of Cisterna Chyli with Intraluminal Device, Percutaneous Approach */
"07VL3ZZ"  =  "1"  /* Restriction of Cisterna Chyli, Percutaneous Approach */
"07VL4CZ"  =  "1"  /* Restriction of Cisterna Chyli with Extraluminal Device, Percutaneous Endoscopic Approach */
"07VL4DZ"  =  "1"  /* Restriction of Cisterna Chyli with Intraluminal Device, Percutaneous Endoscopic Approach */
"07VL4ZZ"  =  "1"  /* Restriction of Cisterna Chyli, Percutaneous Endoscopic Approach */
"07WK00Z"  =  "1"  /* Revision of Drainage Device in Thoracic Duct, Open Approach */
"07WK03Z"  =  "1"  /* Revision of Infusion Device in Thoracic Duct, Open Approach */
"07WK0CZ"  =  "1"  /* Revision of Extraluminal Device in Thoracic Duct, Open Approach */
"07WK0DZ"  =  "1"  /* Revision of Intraluminal Device in Thoracic Duct, Open Approach */
"07WK30Z"  =  "1"  /* Revision of Drainage Device in Thoracic Duct, Percutaneous Approach */
"07WK33Z"  =  "1"  /* Revision of Infusion Device in Thoracic Duct, Percutaneous Approach */
"07WK3CZ"  =  "1"  /* Revision of Extraluminal Device in Thoracic Duct, Percutaneous Approach */
"07WK3DZ"  =  "1"  /* Revision of Intraluminal Device in Thoracic Duct, Percutaneous Approach */
"07WK40Z"  =  "1"  /* Revision of Drainage Device in Thoracic Duct, Percutaneous Endoscopic Approach */
"07WK43Z"  =  "1"  /* Revision of Infusion Device in Thoracic Duct, Percutaneous Endoscopic Approach */
"07WK4CZ"  =  "1"  /* Revision of Extraluminal Device in Thoracic Duct, Percutaneous Endoscopic Approach */
"07WK4DZ"  =  "1"  /* Revision of Intraluminal Device in Thoracic Duct, Percutaneous Endoscopic Approach */
"07WL00Z"  =  "1"  /* Revision of Drainage Device in Cisterna Chyli, Open Approach */
"07WL03Z"  =  "1"  /* Revision of Infusion Device in Cisterna Chyli, Open Approach */
"07WL0CZ"  =  "1"  /* Revision of Extraluminal Device in Cisterna Chyli, Open Approach */
"07WL0DZ"  =  "1"  /* Revision of Intraluminal Device in Cisterna Chyli, Open Approach */
"07WL30Z"  =  "1"  /* Revision of Drainage Device in Cisterna Chyli, Percutaneous Approach */
"07WL33Z"  =  "1"  /* Revision of Infusion Device in Cisterna Chyli, Percutaneous Approach */
"07WL3CZ"  =  "1"  /* Revision of Extraluminal Device in Cisterna Chyli, Percutaneous Approach */
"07WL3DZ"  =  "1"  /* Revision of Intraluminal Device in Cisterna Chyli, Percutaneous Approach */
"07WL40Z"  =  "1"  /* Revision of Drainage Device in Cisterna Chyli, Percutaneous Endoscopic Approach */
"07WL43Z"  =  "1"  /* Revision of Infusion Device in Cisterna Chyli, Percutaneous Endoscopic Approach */
"07WL4CZ"  =  "1"  /* Revision of Extraluminal Device in Cisterna Chyli, Percutaneous Endoscopic Approach */
"07WL4DZ"  =  "1"  /* Revision of Intraluminal Device in Cisterna Chyli, Percutaneous Endoscopic Approach */
"0B110F4"  =  "1"  /* Bypass Trachea to Cutaneous with Tracheostomy Device, Open Approach */
"0B110Z4"  =  "1"  /* Bypass Trachea to Cutaneous, Open Approach */
"0B113F4"  =  "1"  /* Bypass Trachea to Cutaneous with Tracheostomy Device, Percutaneous Approach */
"0B113Z4"  =  "1"  /* Bypass Trachea to Cutaneous, Percutaneous Approach */
"0B114F4"  =  "1"  /* Bypass Trachea to Cutaneous with Tracheostomy Device, Percutaneous Endoscopic Approach */
"0B114Z4"  =  "1"  /* Bypass Trachea to Cutaneous, Percutaneous Endoscopic Approach */
"0B530ZZ"  =  "1"  /* Destruction of Right Main Bronchus, Open Approach */
"0B533ZZ"  =  "1"  /* Destruction of Right Main Bronchus, Percutaneous Approach */
"0B537ZZ"  =  "1"  /* Destruction of Right Main Bronchus, Via Natural or Artificial Opening */
"0B538ZZ"  =  "1"  /* Destruction of Right Main Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B540ZZ"  =  "1"  /* Destruction of Right Upper Lobe Bronchus, Open Approach */
"0B543ZZ"  =  "1"  /* Destruction of Right Upper Lobe Bronchus, Percutaneous Approach */
"0B547ZZ"  =  "1"  /* Destruction of Right Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0B548ZZ"  =  "1"  /* Destruction of Right Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B550ZZ"  =  "1"  /* Destruction of Right Middle Lobe Bronchus, Open Approach */
"0B553ZZ"  =  "1"  /* Destruction of Right Middle Lobe Bronchus, Percutaneous Approach */
"0B557ZZ"  =  "1"  /* Destruction of Right Middle Lobe Bronchus, Via Natural or Artificial Opening */
"0B558ZZ"  =  "1"  /* Destruction of Right Middle Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B560ZZ"  =  "1"  /* Destruction of Right Lower Lobe Bronchus, Open Approach */
"0B563ZZ"  =  "1"  /* Destruction of Right Lower Lobe Bronchus, Percutaneous Approach */
"0B567ZZ"  =  "1"  /* Destruction of Right Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0B568ZZ"  =  "1"  /* Destruction of Right Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B570ZZ"  =  "1"  /* Destruction of Left Main Bronchus, Open Approach */
"0B573ZZ"  =  "1"  /* Destruction of Left Main Bronchus, Percutaneous Approach */
"0B577ZZ"  =  "1"  /* Destruction of Left Main Bronchus, Via Natural or Artificial Opening */
"0B578ZZ"  =  "1"  /* Destruction of Left Main Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B580ZZ"  =  "1"  /* Destruction of Left Upper Lobe Bronchus, Open Approach */
"0B583ZZ"  =  "1"  /* Destruction of Left Upper Lobe Bronchus, Percutaneous Approach */
"0B587ZZ"  =  "1"  /* Destruction of Left Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0B588ZZ"  =  "1"  /* Destruction of Left Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B590ZZ"  =  "1"  /* Destruction of Lingula Bronchus, Open Approach */
"0B593ZZ"  =  "1"  /* Destruction of Lingula Bronchus, Percutaneous Approach */
"0B597ZZ"  =  "1"  /* Destruction of Lingula Bronchus, Via Natural or Artificial Opening */
"0B598ZZ"  =  "1"  /* Destruction of Lingula Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B5B0ZZ"  =  "1"  /* Destruction of Left Lower Lobe Bronchus, Open Approach */
"0B5B3ZZ"  =  "1"  /* Destruction of Left Lower Lobe Bronchus, Percutaneous Approach */
"0B5B7ZZ"  =  "1"  /* Destruction of Left Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0B5B8ZZ"  =  "1"  /* Destruction of Left Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B5C0ZZ"  =  "1"  /* Destruction of Right Upper Lung Lobe, Open Approach */
"0B5C3ZZ"  =  "1"  /* Destruction of Right Upper Lung Lobe, Percutaneous Approach */
"0B5C4ZZ"  =  "1"  /* Destruction of Right Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0B5C7ZZ"  =  "1"  /* Destruction of Right Upper Lung Lobe, Via Natural or Artificial Opening */
"0B5C8ZZ"  =  "1"  /* Destruction of Right Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0B5D0ZZ"  =  "1"  /* Destruction of Right Middle Lung Lobe, Open Approach */
"0B5D3ZZ"  =  "1"  /* Destruction of Right Middle Lung Lobe, Percutaneous Approach */
"0B5D4ZZ"  =  "1"  /* Destruction of Right Middle Lung Lobe, Percutaneous Endoscopic Approach */
"0B5D7ZZ"  =  "1"  /* Destruction of Right Middle Lung Lobe, Via Natural or Artificial Opening */
"0B5D8ZZ"  =  "1"  /* Destruction of Right Middle Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0B5F0ZZ"  =  "1"  /* Destruction of Right Lower Lung Lobe, Open Approach */
"0B5F3ZZ"  =  "1"  /* Destruction of Right Lower Lung Lobe, Percutaneous Approach */
"0B5F4ZZ"  =  "1"  /* Destruction of Right Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0B5F7ZZ"  =  "1"  /* Destruction of Right Lower Lung Lobe, Via Natural or Artificial Opening */
"0B5F8ZZ"  =  "1"  /* Destruction of Right Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0B5G0ZZ"  =  "1"  /* Destruction of Left Upper Lung Lobe, Open Approach */
"0B5G3ZZ"  =  "1"  /* Destruction of Left Upper Lung Lobe, Percutaneous Approach */
"0B5G4ZZ"  =  "1"  /* Destruction of Left Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0B5G7ZZ"  =  "1"  /* Destruction of Left Upper Lung Lobe, Via Natural or Artificial Opening */
"0B5G8ZZ"  =  "1"  /* Destruction of Left Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0B5H0ZZ"  =  "1"  /* Destruction of Lung Lingula, Open Approach */
"0B5H3ZZ"  =  "1"  /* Destruction of Lung Lingula, Percutaneous Approach */
"0B5H4ZZ"  =  "1"  /* Destruction of Lung Lingula, Percutaneous Endoscopic Approach */
"0B5H7ZZ"  =  "1"  /* Destruction of Lung Lingula, Via Natural or Artificial Opening */
"0B5H8ZZ"  =  "1"  /* Destruction of Lung Lingula, Via Natural or Artificial Opening Endoscopic */
"0B5J0ZZ"  =  "1"  /* Destruction of Left Lower Lung Lobe, Open Approach */
"0B5J3ZZ"  =  "1"  /* Destruction of Left Lower Lung Lobe, Percutaneous Approach */
"0B5J4ZZ"  =  "1"  /* Destruction of Left Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0B5J7ZZ"  =  "1"  /* Destruction of Left Lower Lung Lobe, Via Natural or Artificial Opening */
"0B5J8ZZ"  =  "1"  /* Destruction of Left Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0B5K0ZZ"  =  "1"  /* Destruction of Right Lung, Open Approach */
"0B5K3ZZ"  =  "1"  /* Destruction of Right Lung, Percutaneous Approach */
"0B5K4ZZ"  =  "1"  /* Destruction of Right Lung, Percutaneous Endoscopic Approach */
"0B5K7ZZ"  =  "1"  /* Destruction of Right Lung, Via Natural or Artificial Opening */
"0B5K8ZZ"  =  "1"  /* Destruction of Right Lung, Via Natural or Artificial Opening Endoscopic */
"0B5L0ZZ"  =  "1"  /* Destruction of Left Lung, Open Approach */
"0B5L3ZZ"  =  "1"  /* Destruction of Left Lung, Percutaneous Approach */
"0B5L4ZZ"  =  "1"  /* Destruction of Left Lung, Percutaneous Endoscopic Approach */
"0B5L7ZZ"  =  "1"  /* Destruction of Left Lung, Via Natural or Artificial Opening */
"0B5L8ZZ"  =  "1"  /* Destruction of Left Lung, Via Natural or Artificial Opening Endoscopic */
"0B5M0ZZ"  =  "1"  /* Destruction of Bilateral Lungs, Open Approach */
"0B5M3ZZ"  =  "1"  /* Destruction of Bilateral Lungs, Percutaneous Approach */
"0B5M4ZZ"  =  "1"  /* Destruction of Bilateral Lungs, Percutaneous Endoscopic Approach */
"0B5M7ZZ"  =  "1"  /* Destruction of Bilateral Lungs, Via Natural or Artificial Opening */
"0B5M8ZZ"  =  "1"  /* Destruction of Bilateral Lungs, Via Natural or Artificial Opening Endoscopic */
"0B5R0ZZ"  =  "1"  /* Destruction of Right Diaphragm, Open Approach */
"0B5R3ZZ"  =  "1"  /* Destruction of Right Diaphragm, Percutaneous Approach */
"0B5R4ZZ"  =  "1"  /* Destruction of Right Diaphragm, Percutaneous Endoscopic Approach */
"0B5S0ZZ"  =  "1"  /* Destruction of Left Diaphragm, Open Approach */
"0B5S3ZZ"  =  "1"  /* Destruction of Left Diaphragm, Percutaneous Approach */
"0B5S4ZZ"  =  "1"  /* Destruction of Left Diaphragm, Percutaneous Endoscopic Approach */
"0B710DZ"  =  "1"  /* Dilation of Trachea with Intraluminal Device, Open Approach */
"0B710ZZ"  =  "1"  /* Dilation of Trachea, Open Approach */
"0B713DZ"  =  "1"  /* Dilation of Trachea with Intraluminal Device, Percutaneous Approach */
"0B713ZZ"  =  "1"  /* Dilation of Trachea, Percutaneous Approach */
"0B714DZ"  =  "1"  /* Dilation of Trachea with Intraluminal Device, Percutaneous Endoscopic Approach */
"0B714ZZ"  =  "1"  /* Dilation of Trachea, Percutaneous Endoscopic Approach */
"0B717DZ"  =  "1"  /* Dilation of Trachea with Intraluminal Device, Via Natural or Artificial Opening */
"0B717ZZ"  =  "1"  /* Dilation of Trachea, Via Natural or Artificial Opening */
"0B718DZ"  =  "1"  /* Dilation of Trachea with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0B718ZZ"  =  "1"  /* Dilation of Trachea, Via Natural or Artificial Opening Endoscopic */
"0B720DZ"  =  "1"  /* Dilation of Carina with Intraluminal Device, Open Approach */
"0B720ZZ"  =  "1"  /* Dilation of Carina, Open Approach */
"0B723DZ"  =  "1"  /* Dilation of Carina with Intraluminal Device, Percutaneous Approach */
"0B723ZZ"  =  "1"  /* Dilation of Carina, Percutaneous Approach */
"0B724DZ"  =  "1"  /* Dilation of Carina with Intraluminal Device, Percutaneous Endoscopic Approach */
"0B724ZZ"  =  "1"  /* Dilation of Carina, Percutaneous Endoscopic Approach */
"0B727DZ"  =  "1"  /* Dilation of Carina with Intraluminal Device, Via Natural or Artificial Opening */
"0B727ZZ"  =  "1"  /* Dilation of Carina, Via Natural or Artificial Opening */
"0B728DZ"  =  "1"  /* Dilation of Carina with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0B728ZZ"  =  "1"  /* Dilation of Carina, Via Natural or Artificial Opening Endoscopic */
"0B910ZX"  =  "1"  /* Drainage of Trachea, Open Approach, Diagnostic */
"0B920ZX"  =  "1"  /* Drainage of Carina, Open Approach, Diagnostic */
"0B9300Z"  =  "1"  /* Drainage of Right Main Bronchus with Drainage Device, Open Approach */
"0B930ZX"  =  "1"  /* Drainage of Right Main Bronchus, Open Approach, Diagnostic */
"0B930ZZ"  =  "1"  /* Drainage of Right Main Bronchus, Open Approach */
"0B9330Z"  =  "1"  /* Drainage of Right Main Bronchus with Drainage Device, Percutaneous Approach */
"0B933ZZ"  =  "1"  /* Drainage of Right Main Bronchus, Percutaneous Approach */
"0B9340Z"  =  "1"  /* Drainage of Right Main Bronchus with Drainage Device, Percutaneous Endoscopic Approach */
"0B934ZZ"  =  "1"  /* Drainage of Right Main Bronchus, Percutaneous Endoscopic Approach */
"0B9370Z"  =  "1"  /* Drainage of Right Main Bronchus with Drainage Device, Via Natural or Artificial Opening */
"0B937ZZ"  =  "1"  /* Drainage of Right Main Bronchus, Via Natural or Artificial Opening */
"0B9380Z"  =  "1"  /* Drainage of Right Main Bronchus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B938ZZ"  =  "1"  /* Drainage of Right Main Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B9400Z"  =  "1"  /* Drainage of Right Upper Lobe Bronchus with Drainage Device, Open Approach */
"0B940ZX"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Open Approach, Diagnostic */
"0B940ZZ"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Open Approach */
"0B9430Z"  =  "1"  /* Drainage of Right Upper Lobe Bronchus with Drainage Device, Percutaneous Approach */
"0B943ZZ"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Percutaneous Approach */
"0B9440Z"  =  "1"  /* Drainage of Right Upper Lobe Bronchus with Drainage Device, Percutaneous Endoscopic Approach */
"0B944ZZ"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0B9470Z"  =  "1"  /* Drainage of Right Upper Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening */
"0B947ZZ"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0B9480Z"  =  "1"  /* Drainage of Right Upper Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B948ZZ"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B9500Z"  =  "1"  /* Drainage of Right Middle Lobe Bronchus with Drainage Device, Open Approach */
"0B950ZX"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Open Approach, Diagnostic */
"0B950ZZ"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Open Approach */
"0B9530Z"  =  "1"  /* Drainage of Right Middle Lobe Bronchus with Drainage Device, Percutaneous Approach */
"0B953ZZ"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Percutaneous Approach */
"0B9540Z"  =  "1"  /* Drainage of Right Middle Lobe Bronchus with Drainage Device, Percutaneous Endoscopic Approach */
"0B954ZZ"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Percutaneous Endoscopic Approach */
"0B9570Z"  =  "1"  /* Drainage of Right Middle Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening */
"0B957ZZ"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Via Natural or Artificial Opening */
"0B9580Z"  =  "1"  /* Drainage of Right Middle Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B958ZZ"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B9600Z"  =  "1"  /* Drainage of Right Lower Lobe Bronchus with Drainage Device, Open Approach */
"0B960ZX"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Open Approach, Diagnostic */
"0B960ZZ"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Open Approach */
"0B9630Z"  =  "1"  /* Drainage of Right Lower Lobe Bronchus with Drainage Device, Percutaneous Approach */
"0B963ZZ"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Percutaneous Approach */
"0B9640Z"  =  "1"  /* Drainage of Right Lower Lobe Bronchus with Drainage Device, Percutaneous Endoscopic Approach */
"0B964ZZ"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0B9670Z"  =  "1"  /* Drainage of Right Lower Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening */
"0B967ZZ"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0B9680Z"  =  "1"  /* Drainage of Right Lower Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B968ZZ"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B9700Z"  =  "1"  /* Drainage of Left Main Bronchus with Drainage Device, Open Approach */
"0B970ZX"  =  "1"  /* Drainage of Left Main Bronchus, Open Approach, Diagnostic */
"0B970ZZ"  =  "1"  /* Drainage of Left Main Bronchus, Open Approach */
"0B9730Z"  =  "1"  /* Drainage of Left Main Bronchus with Drainage Device, Percutaneous Approach */
"0B973ZZ"  =  "1"  /* Drainage of Left Main Bronchus, Percutaneous Approach */
"0B9740Z"  =  "1"  /* Drainage of Left Main Bronchus with Drainage Device, Percutaneous Endoscopic Approach */
"0B974ZZ"  =  "1"  /* Drainage of Left Main Bronchus, Percutaneous Endoscopic Approach */
"0B9770Z"  =  "1"  /* Drainage of Left Main Bronchus with Drainage Device, Via Natural or Artificial Opening */
"0B977ZZ"  =  "1"  /* Drainage of Left Main Bronchus, Via Natural or Artificial Opening */
"0B9780Z"  =  "1"  /* Drainage of Left Main Bronchus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B978ZZ"  =  "1"  /* Drainage of Left Main Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B9800Z"  =  "1"  /* Drainage of Left Upper Lobe Bronchus with Drainage Device, Open Approach */
"0B980ZX"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Open Approach, Diagnostic */
"0B980ZZ"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Open Approach */
"0B9830Z"  =  "1"  /* Drainage of Left Upper Lobe Bronchus with Drainage Device, Percutaneous Approach */
"0B983ZZ"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Percutaneous Approach */
"0B9840Z"  =  "1"  /* Drainage of Left Upper Lobe Bronchus with Drainage Device, Percutaneous Endoscopic Approach */
"0B984ZZ"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0B9870Z"  =  "1"  /* Drainage of Left Upper Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening */
"0B987ZZ"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0B9880Z"  =  "1"  /* Drainage of Left Upper Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B988ZZ"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B9900Z"  =  "1"  /* Drainage of Lingula Bronchus with Drainage Device, Open Approach */
"0B990ZX"  =  "1"  /* Drainage of Lingula Bronchus, Open Approach, Diagnostic */
"0B990ZZ"  =  "1"  /* Drainage of Lingula Bronchus, Open Approach */
"0B9930Z"  =  "1"  /* Drainage of Lingula Bronchus with Drainage Device, Percutaneous Approach */
"0B993ZZ"  =  "1"  /* Drainage of Lingula Bronchus, Percutaneous Approach */
"0B9940Z"  =  "1"  /* Drainage of Lingula Bronchus with Drainage Device, Percutaneous Endoscopic Approach */
"0B994ZZ"  =  "1"  /* Drainage of Lingula Bronchus, Percutaneous Endoscopic Approach */
"0B9970Z"  =  "1"  /* Drainage of Lingula Bronchus with Drainage Device, Via Natural or Artificial Opening */
"0B997ZZ"  =  "1"  /* Drainage of Lingula Bronchus, Via Natural or Artificial Opening */
"0B9980Z"  =  "1"  /* Drainage of Lingula Bronchus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B998ZZ"  =  "1"  /* Drainage of Lingula Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B9B00Z"  =  "1"  /* Drainage of Left Lower Lobe Bronchus with Drainage Device, Open Approach */
"0B9B0ZX"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Open Approach, Diagnostic */
"0B9B0ZZ"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Open Approach */
"0B9B30Z"  =  "1"  /* Drainage of Left Lower Lobe Bronchus with Drainage Device, Percutaneous Approach */
"0B9B3ZZ"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Percutaneous Approach */
"0B9B40Z"  =  "1"  /* Drainage of Left Lower Lobe Bronchus with Drainage Device, Percutaneous Endoscopic Approach */
"0B9B4ZZ"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0B9B70Z"  =  "1"  /* Drainage of Left Lower Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening */
"0B9B7ZZ"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0B9B80Z"  =  "1"  /* Drainage of Left Lower Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B9B8ZZ"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B9C00Z"  =  "1"  /* Drainage of Right Upper Lung Lobe with Drainage Device, Open Approach */
"0B9C0ZZ"  =  "1"  /* Drainage of Right Upper Lung Lobe, Open Approach */
"0B9C30Z"  =  "1"  /* Drainage of Right Upper Lung Lobe with Drainage Device, Percutaneous Approach */
"0B9C3ZZ"  =  "1"  /* Drainage of Right Upper Lung Lobe, Percutaneous Approach */
"0B9C40Z"  =  "1"  /* Drainage of Right Upper Lung Lobe with Drainage Device, Percutaneous Endoscopic Approach */
"0B9C4ZZ"  =  "1"  /* Drainage of Right Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0B9C70Z"  =  "1"  /* Drainage of Right Upper Lung Lobe with Drainage Device, Via Natural or Artificial Opening */
"0B9C7ZZ"  =  "1"  /* Drainage of Right Upper Lung Lobe, Via Natural or Artificial Opening */
"0B9C80Z"  =  "1"  /* Drainage of Right Upper Lung Lobe with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B9C8ZX"  =  "1"  /* Drainage of Right Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B9C8ZZ"  =  "1"  /* Drainage of Right Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0B9D00Z"  =  "1"  /* Drainage of Right Middle Lung Lobe with Drainage Device, Open Approach */
"0B9D0ZZ"  =  "1"  /* Drainage of Right Middle Lung Lobe, Open Approach */
"0B9D30Z"  =  "1"  /* Drainage of Right Middle Lung Lobe with Drainage Device, Percutaneous Approach */
"0B9D3ZZ"  =  "1"  /* Drainage of Right Middle Lung Lobe, Percutaneous Approach */
"0B9D40Z"  =  "1"  /* Drainage of Right Middle Lung Lobe with Drainage Device, Percutaneous Endoscopic Approach */
"0B9D4ZZ"  =  "1"  /* Drainage of Right Middle Lung Lobe, Percutaneous Endoscopic Approach */
"0B9D70Z"  =  "1"  /* Drainage of Right Middle Lung Lobe with Drainage Device, Via Natural or Artificial Opening */
"0B9D7ZZ"  =  "1"  /* Drainage of Right Middle Lung Lobe, Via Natural or Artificial Opening */
"0B9D80Z"  =  "1"  /* Drainage of Right Middle Lung Lobe with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B9D8ZX"  =  "1"  /* Drainage of Right Middle Lung Lobe, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B9D8ZZ"  =  "1"  /* Drainage of Right Middle Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0B9F00Z"  =  "1"  /* Drainage of Right Lower Lung Lobe with Drainage Device, Open Approach */
"0B9F0ZZ"  =  "1"  /* Drainage of Right Lower Lung Lobe, Open Approach */
"0B9F30Z"  =  "1"  /* Drainage of Right Lower Lung Lobe with Drainage Device, Percutaneous Approach */
"0B9F3ZZ"  =  "1"  /* Drainage of Right Lower Lung Lobe, Percutaneous Approach */
"0B9F40Z"  =  "1"  /* Drainage of Right Lower Lung Lobe with Drainage Device, Percutaneous Endoscopic Approach */
"0B9F4ZZ"  =  "1"  /* Drainage of Right Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0B9F70Z"  =  "1"  /* Drainage of Right Lower Lung Lobe with Drainage Device, Via Natural or Artificial Opening */
"0B9F7ZZ"  =  "1"  /* Drainage of Right Lower Lung Lobe, Via Natural or Artificial Opening */
"0B9F80Z"  =  "1"  /* Drainage of Right Lower Lung Lobe with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B9F8ZX"  =  "1"  /* Drainage of Right Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B9F8ZZ"  =  "1"  /* Drainage of Right Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0B9G00Z"  =  "1"  /* Drainage of Left Upper Lung Lobe with Drainage Device, Open Approach */
"0B9G0ZZ"  =  "1"  /* Drainage of Left Upper Lung Lobe, Open Approach */
"0B9G30Z"  =  "1"  /* Drainage of Left Upper Lung Lobe with Drainage Device, Percutaneous Approach */
"0B9G3ZZ"  =  "1"  /* Drainage of Left Upper Lung Lobe, Percutaneous Approach */
"0B9G40Z"  =  "1"  /* Drainage of Left Upper Lung Lobe with Drainage Device, Percutaneous Endoscopic Approach */
"0B9G4ZZ"  =  "1"  /* Drainage of Left Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0B9G70Z"  =  "1"  /* Drainage of Left Upper Lung Lobe with Drainage Device, Via Natural or Artificial Opening */
"0B9G7ZZ"  =  "1"  /* Drainage of Left Upper Lung Lobe, Via Natural or Artificial Opening */
"0B9G80Z"  =  "1"  /* Drainage of Left Upper Lung Lobe with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B9G8ZX"  =  "1"  /* Drainage of Left Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B9G8ZZ"  =  "1"  /* Drainage of Left Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0B9H00Z"  =  "1"  /* Drainage of Lung Lingula with Drainage Device, Open Approach */
"0B9H0ZZ"  =  "1"  /* Drainage of Lung Lingula, Open Approach */
"0B9H30Z"  =  "1"  /* Drainage of Lung Lingula with Drainage Device, Percutaneous Approach */
"0B9H3ZZ"  =  "1"  /* Drainage of Lung Lingula, Percutaneous Approach */
"0B9H40Z"  =  "1"  /* Drainage of Lung Lingula with Drainage Device, Percutaneous Endoscopic Approach */
"0B9H4ZZ"  =  "1"  /* Drainage of Lung Lingula, Percutaneous Endoscopic Approach */
"0B9H70Z"  =  "1"  /* Drainage of Lung Lingula with Drainage Device, Via Natural or Artificial Opening */
"0B9H7ZZ"  =  "1"  /* Drainage of Lung Lingula, Via Natural or Artificial Opening */
"0B9H80Z"  =  "1"  /* Drainage of Lung Lingula with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B9H8ZX"  =  "1"  /* Drainage of Lung Lingula, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B9H8ZZ"  =  "1"  /* Drainage of Lung Lingula, Via Natural or Artificial Opening Endoscopic */
"0B9J00Z"  =  "1"  /* Drainage of Left Lower Lung Lobe with Drainage Device, Open Approach */
"0B9J0ZZ"  =  "1"  /* Drainage of Left Lower Lung Lobe, Open Approach */
"0B9J30Z"  =  "1"  /* Drainage of Left Lower Lung Lobe with Drainage Device, Percutaneous Approach */
"0B9J3ZZ"  =  "1"  /* Drainage of Left Lower Lung Lobe, Percutaneous Approach */
"0B9J40Z"  =  "1"  /* Drainage of Left Lower Lung Lobe with Drainage Device, Percutaneous Endoscopic Approach */
"0B9J4ZZ"  =  "1"  /* Drainage of Left Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0B9J70Z"  =  "1"  /* Drainage of Left Lower Lung Lobe with Drainage Device, Via Natural or Artificial Opening */
"0B9J7ZZ"  =  "1"  /* Drainage of Left Lower Lung Lobe, Via Natural or Artificial Opening */
"0B9J80Z"  =  "1"  /* Drainage of Left Lower Lung Lobe with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B9J8ZX"  =  "1"  /* Drainage of Left Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B9J8ZZ"  =  "1"  /* Drainage of Left Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0B9K00Z"  =  "1"  /* Drainage of Right Lung with Drainage Device, Open Approach */
"0B9K0ZZ"  =  "1"  /* Drainage of Right Lung, Open Approach */
"0B9K30Z"  =  "1"  /* Drainage of Right Lung with Drainage Device, Percutaneous Approach */
"0B9K3ZZ"  =  "1"  /* Drainage of Right Lung, Percutaneous Approach */
"0B9K40Z"  =  "1"  /* Drainage of Right Lung with Drainage Device, Percutaneous Endoscopic Approach */
"0B9K4ZZ"  =  "1"  /* Drainage of Right Lung, Percutaneous Endoscopic Approach */
"0B9K70Z"  =  "1"  /* Drainage of Right Lung with Drainage Device, Via Natural or Artificial Opening */
"0B9K7ZZ"  =  "1"  /* Drainage of Right Lung, Via Natural or Artificial Opening */
"0B9K80Z"  =  "1"  /* Drainage of Right Lung with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B9K8ZX"  =  "1"  /* Drainage of Right Lung, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B9K8ZZ"  =  "1"  /* Drainage of Right Lung, Via Natural or Artificial Opening Endoscopic */
"0B9L00Z"  =  "1"  /* Drainage of Left Lung with Drainage Device, Open Approach */
"0B9L0ZZ"  =  "1"  /* Drainage of Left Lung, Open Approach */
"0B9L30Z"  =  "1"  /* Drainage of Left Lung with Drainage Device, Percutaneous Approach */
"0B9L3ZZ"  =  "1"  /* Drainage of Left Lung, Percutaneous Approach */
"0B9L40Z"  =  "1"  /* Drainage of Left Lung with Drainage Device, Percutaneous Endoscopic Approach */
"0B9L4ZZ"  =  "1"  /* Drainage of Left Lung, Percutaneous Endoscopic Approach */
"0B9L70Z"  =  "1"  /* Drainage of Left Lung with Drainage Device, Via Natural or Artificial Opening */
"0B9L7ZZ"  =  "1"  /* Drainage of Left Lung, Via Natural or Artificial Opening */
"0B9L80Z"  =  "1"  /* Drainage of Left Lung with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B9L8ZX"  =  "1"  /* Drainage of Left Lung, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B9L8ZZ"  =  "1"  /* Drainage of Left Lung, Via Natural or Artificial Opening Endoscopic */
"0B9M00Z"  =  "1"  /* Drainage of Bilateral Lungs with Drainage Device, Open Approach */
"0B9M0ZZ"  =  "1"  /* Drainage of Bilateral Lungs, Open Approach */
"0B9M30Z"  =  "1"  /* Drainage of Bilateral Lungs with Drainage Device, Percutaneous Approach */
"0B9M3ZZ"  =  "1"  /* Drainage of Bilateral Lungs, Percutaneous Approach */
"0B9M40Z"  =  "1"  /* Drainage of Bilateral Lungs with Drainage Device, Percutaneous Endoscopic Approach */
"0B9M4ZZ"  =  "1"  /* Drainage of Bilateral Lungs, Percutaneous Endoscopic Approach */
"0B9M70Z"  =  "1"  /* Drainage of Bilateral Lungs with Drainage Device, Via Natural or Artificial Opening */
"0B9M7ZZ"  =  "1"  /* Drainage of Bilateral Lungs, Via Natural or Artificial Opening */
"0B9M80Z"  =  "1"  /* Drainage of Bilateral Lungs with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B9M8ZX"  =  "1"  /* Drainage of Bilateral Lungs, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B9M8ZZ"  =  "1"  /* Drainage of Bilateral Lungs, Via Natural or Artificial Opening Endoscopic */
"0B9N00Z"  =  "1"  /* Drainage of Right Pleura with Drainage Device, Open Approach */
"0B9N0ZZ"  =  "1"  /* Drainage of Right Pleura, Open Approach */
"0B9N30Z"  =  "1"  /* Drainage of Right Pleura with Drainage Device, Percutaneous Approach */
"0B9P00Z"  =  "1"  /* Drainage of Left Pleura with Drainage Device, Open Approach */
"0B9P0ZZ"  =  "1"  /* Drainage of Left Pleura, Open Approach */
"0B9P30Z"  =  "1"  /* Drainage of Left Pleura with Drainage Device, Percutaneous Approach */
"0B9R00Z"  =  "1"  /* Drainage of Right Diaphragm with Drainage Device, Open Approach */
"0B9R0ZX"  =  "1"  /* Drainage of Right Diaphragm, Open Approach, Diagnostic */
"0B9R0ZZ"  =  "1"  /* Drainage of Right Diaphragm, Open Approach */
"0B9R30Z"  =  "1"  /* Drainage of Right Diaphragm with Drainage Device, Percutaneous Approach */
"0B9R3ZX"  =  "1"  /* Drainage of Right Diaphragm, Percutaneous Approach, Diagnostic */
"0B9R3ZZ"  =  "1"  /* Drainage of Right Diaphragm, Percutaneous Approach */
"0B9R40Z"  =  "1"  /* Drainage of Right Diaphragm with Drainage Device, Percutaneous Endoscopic Approach */
"0B9R4ZX"  =  "1"  /* Drainage of Right Diaphragm, Percutaneous Endoscopic Approach, Diagnostic */
"0B9R4ZZ"  =  "1"  /* Drainage of Right Diaphragm, Percutaneous Endoscopic Approach */
"0B9S00Z"  =  "1"  /* Drainage of Left Diaphragm with Drainage Device, Open Approach */
"0B9S0ZX"  =  "1"  /* Drainage of Left Diaphragm, Open Approach, Diagnostic */
"0B9S0ZZ"  =  "1"  /* Drainage of Left Diaphragm, Open Approach */
"0B9S30Z"  =  "1"  /* Drainage of Left Diaphragm with Drainage Device, Percutaneous Approach */
"0B9S3ZX"  =  "1"  /* Drainage of Left Diaphragm, Percutaneous Approach, Diagnostic */
"0B9S3ZZ"  =  "1"  /* Drainage of Left Diaphragm, Percutaneous Approach */
"0B9S40Z"  =  "1"  /* Drainage of Left Diaphragm with Drainage Device, Percutaneous Endoscopic Approach */
"0B9S4ZX"  =  "1"  /* Drainage of Left Diaphragm, Percutaneous Endoscopic Approach, Diagnostic */
"0B9S4ZZ"  =  "1"  /* Drainage of Left Diaphragm, Percutaneous Endoscopic Approach */
"0BB10ZX"  =  "1"  /* Excision of Trachea, Open Approach, Diagnostic */
"0BB20ZX"  =  "1"  /* Excision of Carina, Open Approach, Diagnostic */
"0BB30ZX"  =  "1"  /* Excision of Right Main Bronchus, Open Approach, Diagnostic */
"0BB30ZZ"  =  "1"  /* Excision of Right Main Bronchus, Open Approach */
"0BB33ZZ"  =  "1"  /* Excision of Right Main Bronchus, Percutaneous Approach */
"0BB37ZZ"  =  "1"  /* Excision of Right Main Bronchus, Via Natural or Artificial Opening */
"0BB40ZX"  =  "1"  /* Excision of Right Upper Lobe Bronchus, Open Approach, Diagnostic */
"0BB40ZZ"  =  "1"  /* Excision of Right Upper Lobe Bronchus, Open Approach */
"0BB43ZZ"  =  "1"  /* Excision of Right Upper Lobe Bronchus, Percutaneous Approach */
"0BB47ZZ"  =  "1"  /* Excision of Right Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0BB50ZX"  =  "1"  /* Excision of Right Middle Lobe Bronchus, Open Approach, Diagnostic */
"0BB50ZZ"  =  "1"  /* Excision of Right Middle Lobe Bronchus, Open Approach */
"0BB53ZZ"  =  "1"  /* Excision of Right Middle Lobe Bronchus, Percutaneous Approach */
"0BB57ZZ"  =  "1"  /* Excision of Right Middle Lobe Bronchus, Via Natural or Artificial Opening */
"0BB60ZX"  =  "1"  /* Excision of Right Lower Lobe Bronchus, Open Approach, Diagnostic */
"0BB60ZZ"  =  "1"  /* Excision of Right Lower Lobe Bronchus, Open Approach */
"0BB63ZZ"  =  "1"  /* Excision of Right Lower Lobe Bronchus, Percutaneous Approach */
"0BB67ZZ"  =  "1"  /* Excision of Right Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0BB70ZX"  =  "1"  /* Excision of Left Main Bronchus, Open Approach, Diagnostic */
"0BB70ZZ"  =  "1"  /* Excision of Left Main Bronchus, Open Approach */
"0BB73ZZ"  =  "1"  /* Excision of Left Main Bronchus, Percutaneous Approach */
"0BB77ZZ"  =  "1"  /* Excision of Left Main Bronchus, Via Natural or Artificial Opening */
"0BB80ZX"  =  "1"  /* Excision of Left Upper Lobe Bronchus, Open Approach, Diagnostic */
"0BB80ZZ"  =  "1"  /* Excision of Left Upper Lobe Bronchus, Open Approach */
"0BB83ZZ"  =  "1"  /* Excision of Left Upper Lobe Bronchus, Percutaneous Approach */
"0BB87ZZ"  =  "1"  /* Excision of Left Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0BB90ZX"  =  "1"  /* Excision of Lingula Bronchus, Open Approach, Diagnostic */
"0BB90ZZ"  =  "1"  /* Excision of Lingula Bronchus, Open Approach */
"0BB93ZZ"  =  "1"  /* Excision of Lingula Bronchus, Percutaneous Approach */
"0BB97ZZ"  =  "1"  /* Excision of Lingula Bronchus, Via Natural or Artificial Opening */
"0BBB0ZX"  =  "1"  /* Excision of Left Lower Lobe Bronchus, Open Approach, Diagnostic */
"0BBB0ZZ"  =  "1"  /* Excision of Left Lower Lobe Bronchus, Open Approach */
"0BBB3ZZ"  =  "1"  /* Excision of Left Lower Lobe Bronchus, Percutaneous Approach */
"0BBB7ZZ"  =  "1"  /* Excision of Left Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0BBC0ZZ"  =  "1"  /* Excision of Right Upper Lung Lobe, Open Approach */
"0BBC3ZZ"  =  "1"  /* Excision of Right Upper Lung Lobe, Percutaneous Approach */
"0BBC4ZX"  =  "1"  /* Excision of Right Upper Lung Lobe, Percutaneous Endoscopic Approach, Diagnostic */
"0BBC4ZZ"  =  "1"  /* Excision of Right Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0BBC7ZX"  =  "1"  /* Excision of Right Upper Lung Lobe, Via Natural or Artificial Opening, Diagnostic */
"0BBC7ZZ"  =  "1"  /* Excision of Right Upper Lung Lobe, Via Natural or Artificial Opening */
"0BBC8ZX"  =  "1"  /* Excision of Right Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BBC8ZZ"  =  "1"  /* Excision of Right Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BBD0ZZ"  =  "1"  /* Excision of Right Middle Lung Lobe, Open Approach */
"0BBD3ZZ"  =  "1"  /* Excision of Right Middle Lung Lobe, Percutaneous Approach */
"0BBD4ZX"  =  "1"  /* Excision of Right Middle Lung Lobe, Percutaneous Endoscopic Approach, Diagnostic */
"0BBD4ZZ"  =  "1"  /* Excision of Right Middle Lung Lobe, Percutaneous Endoscopic Approach */
"0BBD7ZX"  =  "1"  /* Excision of Right Middle Lung Lobe, Via Natural or Artificial Opening, Diagnostic */
"0BBD7ZZ"  =  "1"  /* Excision of Right Middle Lung Lobe, Via Natural or Artificial Opening */
"0BBD8ZX"  =  "1"  /* Excision of Right Middle Lung Lobe, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BBD8ZZ"  =  "1"  /* Excision of Right Middle Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BBF0ZZ"  =  "1"  /* Excision of Right Lower Lung Lobe, Open Approach */
"0BBF3ZZ"  =  "1"  /* Excision of Right Lower Lung Lobe, Percutaneous Approach */
"0BBF4ZX"  =  "1"  /* Excision of Right Lower Lung Lobe, Percutaneous Endoscopic Approach, Diagnostic */
"0BBF4ZZ"  =  "1"  /* Excision of Right Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0BBF7ZX"  =  "1"  /* Excision of Right Lower Lung Lobe, Via Natural or Artificial Opening, Diagnostic */
"0BBF7ZZ"  =  "1"  /* Excision of Right Lower Lung Lobe, Via Natural or Artificial Opening */
"0BBF8ZX"  =  "1"  /* Excision of Right Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BBF8ZZ"  =  "1"  /* Excision of Right Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BBG0ZZ"  =  "1"  /* Excision of Left Upper Lung Lobe, Open Approach */
"0BBG3ZZ"  =  "1"  /* Excision of Left Upper Lung Lobe, Percutaneous Approach */
"0BBG4ZX"  =  "1"  /* Excision of Left Upper Lung Lobe, Percutaneous Endoscopic Approach, Diagnostic */
"0BBG4ZZ"  =  "1"  /* Excision of Left Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0BBG7ZX"  =  "1"  /* Excision of Left Upper Lung Lobe, Via Natural or Artificial Opening, Diagnostic */
"0BBG7ZZ"  =  "1"  /* Excision of Left Upper Lung Lobe, Via Natural or Artificial Opening */
"0BBG8ZX"  =  "1"  /* Excision of Left Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BBG8ZZ"  =  "1"  /* Excision of Left Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BBH0ZZ"  =  "1"  /* Excision of Lung Lingula, Open Approach */
"0BBH3ZZ"  =  "1"  /* Excision of Lung Lingula, Percutaneous Approach */
"0BBH4ZX"  =  "1"  /* Excision of Lung Lingula, Percutaneous Endoscopic Approach, Diagnostic */
"0BBH4ZZ"  =  "1"  /* Excision of Lung Lingula, Percutaneous Endoscopic Approach */
"0BBH7ZX"  =  "1"  /* Excision of Lung Lingula, Via Natural or Artificial Opening, Diagnostic */
"0BBH7ZZ"  =  "1"  /* Excision of Lung Lingula, Via Natural or Artificial Opening */
"0BBH8ZX"  =  "1"  /* Excision of Lung Lingula, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BBH8ZZ"  =  "1"  /* Excision of Lung Lingula, Via Natural or Artificial Opening Endoscopic */
"0BBJ0ZZ"  =  "1"  /* Excision of Left Lower Lung Lobe, Open Approach */
"0BBJ3ZZ"  =  "1"  /* Excision of Left Lower Lung Lobe, Percutaneous Approach */
"0BBJ4ZX"  =  "1"  /* Excision of Left Lower Lung Lobe, Percutaneous Endoscopic Approach, Diagnostic */
"0BBJ4ZZ"  =  "1"  /* Excision of Left Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0BBJ7ZX"  =  "1"  /* Excision of Left Lower Lung Lobe, Via Natural or Artificial Opening, Diagnostic */
"0BBJ7ZZ"  =  "1"  /* Excision of Left Lower Lung Lobe, Via Natural or Artificial Opening */
"0BBJ8ZX"  =  "1"  /* Excision of Left Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BBJ8ZZ"  =  "1"  /* Excision of Left Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BBK0ZZ"  =  "1"  /* Excision of Right Lung, Open Approach */
"0BBK3ZZ"  =  "1"  /* Excision of Right Lung, Percutaneous Approach */
"0BBK4ZX"  =  "1"  /* Excision of Right Lung, Percutaneous Endoscopic Approach, Diagnostic */
"0BBK4ZZ"  =  "1"  /* Excision of Right Lung, Percutaneous Endoscopic Approach */
"0BBK7ZX"  =  "1"  /* Excision of Right Lung, Via Natural or Artificial Opening, Diagnostic */
"0BBK7ZZ"  =  "1"  /* Excision of Right Lung, Via Natural or Artificial Opening */
"0BBK8ZX"  =  "1"  /* Excision of Right Lung, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BBK8ZZ"  =  "1"  /* Excision of Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BBL0ZZ"  =  "1"  /* Excision of Left Lung, Open Approach */
"0BBL3ZZ"  =  "1"  /* Excision of Left Lung, Percutaneous Approach */
"0BBL4ZX"  =  "1"  /* Excision of Left Lung, Percutaneous Endoscopic Approach, Diagnostic */
"0BBL4ZZ"  =  "1"  /* Excision of Left Lung, Percutaneous Endoscopic Approach */
"0BBL7ZX"  =  "1"  /* Excision of Left Lung, Via Natural or Artificial Opening, Diagnostic */
"0BBL7ZZ"  =  "1"  /* Excision of Left Lung, Via Natural or Artificial Opening */
"0BBL8ZX"  =  "1"  /* Excision of Left Lung, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BBL8ZZ"  =  "1"  /* Excision of Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BBM0ZZ"  =  "1"  /* Excision of Bilateral Lungs, Open Approach */
"0BBM3ZZ"  =  "1"  /* Excision of Bilateral Lungs, Percutaneous Approach */
"0BBM4ZX"  =  "1"  /* Excision of Bilateral Lungs, Percutaneous Endoscopic Approach, Diagnostic */
"0BBM4ZZ"  =  "1"  /* Excision of Bilateral Lungs, Percutaneous Endoscopic Approach */
"0BBM7ZX"  =  "1"  /* Excision of Bilateral Lungs, Via Natural or Artificial Opening, Diagnostic */
"0BBM7ZZ"  =  "1"  /* Excision of Bilateral Lungs, Via Natural or Artificial Opening */
"0BBM8ZX"  =  "1"  /* Excision of Bilateral Lungs, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BBM8ZZ"  =  "1"  /* Excision of Bilateral Lungs, Via Natural or Artificial Opening Endoscopic */
"0BBN0ZZ"  =  "1"  /* Excision of Right Pleura, Open Approach */
"0BBN3ZZ"  =  "1"  /* Excision of Right Pleura, Percutaneous Approach */
"0BBN4ZX"  =  "1"  /* Excision of Right Pleura, Percutaneous Endoscopic Approach, Diagnostic */
"0BBN4ZZ"  =  "1"  /* Excision of Right Pleura, Percutaneous Endoscopic Approach */
"0BBP0ZZ"  =  "1"  /* Excision of Left Pleura, Open Approach */
"0BBP3ZZ"  =  "1"  /* Excision of Left Pleura, Percutaneous Approach */
"0BBP4ZX"  =  "1"  /* Excision of Left Pleura, Percutaneous Endoscopic Approach, Diagnostic */
"0BBP4ZZ"  =  "1"  /* Excision of Left Pleura, Percutaneous Endoscopic Approach */
"0BBR0ZX"  =  "1"  /* Excision of Right Diaphragm, Open Approach, Diagnostic */
"0BBR0ZZ"  =  "1"  /* Excision of Right Diaphragm, Open Approach */
"0BBR3ZX"  =  "1"  /* Excision of Right Diaphragm, Percutaneous Approach, Diagnostic */
"0BBR3ZZ"  =  "1"  /* Excision of Right Diaphragm, Percutaneous Approach */
"0BBR4ZX"  =  "1"  /* Excision of Right Diaphragm, Percutaneous Endoscopic Approach, Diagnostic */
"0BBR4ZZ"  =  "1"  /* Excision of Right Diaphragm, Percutaneous Endoscopic Approach */
"0BBS0ZX"  =  "1"  /* Excision of Left Diaphragm, Open Approach, Diagnostic */
"0BBS0ZZ"  =  "1"  /* Excision of Left Diaphragm, Open Approach */
"0BBS3ZX"  =  "1"  /* Excision of Left Diaphragm, Percutaneous Approach, Diagnostic */
"0BBS3ZZ"  =  "1"  /* Excision of Left Diaphragm, Percutaneous Approach */
"0BBS4ZX"  =  "1"  /* Excision of Left Diaphragm, Percutaneous Endoscopic Approach, Diagnostic */
"0BBS4ZZ"  =  "1"  /* Excision of Left Diaphragm, Percutaneous Endoscopic Approach */
"0BC30ZZ"  =  "1"  /* Extirpation of Matter from Right Main Bronchus, Open Approach */
"0BC33ZZ"  =  "1"  /* Extirpation of Matter from Right Main Bronchus, Percutaneous Approach */
"0BC34ZZ"  =  "1"  /* Extirpation of Matter from Right Main Bronchus, Percutaneous Endoscopic Approach */
"0BC40ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lobe Bronchus, Open Approach */
"0BC43ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lobe Bronchus, Percutaneous Approach */
"0BC44ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BC50ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lobe Bronchus, Open Approach */
"0BC53ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lobe Bronchus, Percutaneous Approach */
"0BC54ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BC60ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lobe Bronchus, Open Approach */
"0BC63ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lobe Bronchus, Percutaneous Approach */
"0BC64ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BC70ZZ"  =  "1"  /* Extirpation of Matter from Left Main Bronchus, Open Approach */
"0BC73ZZ"  =  "1"  /* Extirpation of Matter from Left Main Bronchus, Percutaneous Approach */
"0BC74ZZ"  =  "1"  /* Extirpation of Matter from Left Main Bronchus, Percutaneous Endoscopic Approach */
"0BC80ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lobe Bronchus, Open Approach */
"0BC83ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lobe Bronchus, Percutaneous Approach */
"0BC84ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BC90ZZ"  =  "1"  /* Extirpation of Matter from Lingula Bronchus, Open Approach */
"0BC93ZZ"  =  "1"  /* Extirpation of Matter from Lingula Bronchus, Percutaneous Approach */
"0BC94ZZ"  =  "1"  /* Extirpation of Matter from Lingula Bronchus, Percutaneous Endoscopic Approach */
"0BCB0ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lobe Bronchus, Open Approach */
"0BCB3ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lobe Bronchus, Percutaneous Approach */
"0BCB4ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BCC0ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lung Lobe, Open Approach */
"0BCC3ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lung Lobe, Percutaneous Approach */
"0BCC4ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0BCC7ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lung Lobe, Via Natural or Artificial Opening */
"0BCC8ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BCD0ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lung Lobe, Open Approach */
"0BCD3ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lung Lobe, Percutaneous Approach */
"0BCD4ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lung Lobe, Percutaneous Endoscopic Approach */
"0BCD7ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lung Lobe, Via Natural or Artificial Opening */
"0BCD8ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BCF0ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lung Lobe, Open Approach */
"0BCF3ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lung Lobe, Percutaneous Approach */
"0BCF4ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0BCF7ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lung Lobe, Via Natural or Artificial Opening */
"0BCF8ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BCG0ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lung Lobe, Open Approach */
"0BCG3ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lung Lobe, Percutaneous Approach */
"0BCG4ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0BCG7ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lung Lobe, Via Natural or Artificial Opening */
"0BCG8ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BCH0ZZ"  =  "1"  /* Extirpation of Matter from Lung Lingula, Open Approach */
"0BCH3ZZ"  =  "1"  /* Extirpation of Matter from Lung Lingula, Percutaneous Approach */
"0BCH4ZZ"  =  "1"  /* Extirpation of Matter from Lung Lingula, Percutaneous Endoscopic Approach */
"0BCH7ZZ"  =  "1"  /* Extirpation of Matter from Lung Lingula, Via Natural or Artificial Opening */
"0BCH8ZZ"  =  "1"  /* Extirpation of Matter from Lung Lingula, Via Natural or Artificial Opening Endoscopic */
"0BCJ0ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lung Lobe, Open Approach */
"0BCJ3ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lung Lobe, Percutaneous Approach */
"0BCJ4ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0BCJ7ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lung Lobe, Via Natural or Artificial Opening */
"0BCJ8ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BCK0ZZ"  =  "1"  /* Extirpation of Matter from Right Lung, Open Approach */
"0BCK3ZZ"  =  "1"  /* Extirpation of Matter from Right Lung, Percutaneous Approach */
"0BCK4ZZ"  =  "1"  /* Extirpation of Matter from Right Lung, Percutaneous Endoscopic Approach */
"0BCK7ZZ"  =  "1"  /* Extirpation of Matter from Right Lung, Via Natural or Artificial Opening */
"0BCK8ZZ"  =  "1"  /* Extirpation of Matter from Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BCL0ZZ"  =  "1"  /* Extirpation of Matter from Left Lung, Open Approach */
"0BCL3ZZ"  =  "1"  /* Extirpation of Matter from Left Lung, Percutaneous Approach */
"0BCL4ZZ"  =  "1"  /* Extirpation of Matter from Left Lung, Percutaneous Endoscopic Approach */
"0BCL7ZZ"  =  "1"  /* Extirpation of Matter from Left Lung, Via Natural or Artificial Opening */
"0BCL8ZZ"  =  "1"  /* Extirpation of Matter from Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BCM0ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Lungs, Open Approach */
"0BCM3ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Lungs, Percutaneous Approach */
"0BCM4ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Lungs, Percutaneous Endoscopic Approach */
"0BCM7ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Lungs, Via Natural or Artificial Opening */
"0BCM8ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Lungs, Via Natural or Artificial Opening Endoscopic */
"0BCN0ZZ"  =  "1"  /* Extirpation of Matter from Right Pleura, Open Approach */
"0BCN3ZZ"  =  "1"  /* Extirpation of Matter from Right Pleura, Percutaneous Approach */
"0BCN4ZZ"  =  "1"  /* Extirpation of Matter from Right Pleura, Percutaneous Endoscopic Approach */
"0BCP0ZZ"  =  "1"  /* Extirpation of Matter from Left Pleura, Open Approach */
"0BCP3ZZ"  =  "1"  /* Extirpation of Matter from Left Pleura, Percutaneous Approach */
"0BCP4ZZ"  =  "1"  /* Extirpation of Matter from Left Pleura, Percutaneous Endoscopic Approach */
"0BCR0ZZ"  =  "1"  /* Extirpation of Matter from Right Diaphragm, Open Approach */
"0BCR3ZZ"  =  "1"  /* Extirpation of Matter from Right Diaphragm, Percutaneous Approach */
"0BCR4ZZ"  =  "1"  /* Extirpation of Matter from Right Diaphragm, Percutaneous Endoscopic Approach */
"0BCS0ZZ"  =  "1"  /* Extirpation of Matter from Left Diaphragm, Open Approach */
"0BCS3ZZ"  =  "1"  /* Extirpation of Matter from Left Diaphragm, Percutaneous Approach */
"0BCS4ZZ"  =  "1"  /* Extirpation of Matter from Left Diaphragm, Percutaneous Endoscopic Approach */
"0BDN0ZX"  =  "1"  /* Extraction of Right Pleura, Open Approach, Diagnostic */
"0BDN0ZZ"  =  "1"  /* Extraction of Right Pleura, Open Approach */
"0BDN3ZX"  =  "1"  /* Extraction of Right Pleura, Percutaneous Approach, Diagnostic */
"0BDN3ZZ"  =  "1"  /* Extraction of Right Pleura, Percutaneous Approach */
"0BDN4ZX"  =  "1"  /* Extraction of Right Pleura, Percutaneous Endoscopic Approach, Diagnostic */
"0BDN4ZZ"  =  "1"  /* Extraction of Right Pleura, Percutaneous Endoscopic Approach */
"0BDP0ZX"  =  "1"  /* Extraction of Left Pleura, Open Approach, Diagnostic */
"0BDP0ZZ"  =  "1"  /* Extraction of Left Pleura, Open Approach */
"0BDP3ZX"  =  "1"  /* Extraction of Left Pleura, Percutaneous Approach, Diagnostic */
"0BDP3ZZ"  =  "1"  /* Extraction of Left Pleura, Percutaneous Approach */
"0BDP4ZX"  =  "1"  /* Extraction of Left Pleura, Percutaneous Endoscopic Approach, Diagnostic */
"0BDP4ZZ"  =  "1"  /* Extraction of Left Pleura, Percutaneous Endoscopic Approach */
"0BF10ZZ"  =  "1"  /* Fragmentation in Trachea, Open Approach */
"0BF13ZZ"  =  "1"  /* Fragmentation in Trachea, Percutaneous Approach */
"0BF14ZZ"  =  "1"  /* Fragmentation in Trachea, Percutaneous Endoscopic Approach */
"0BF17ZZ"  =  "1"  /* Fragmentation in Trachea, Via Natural or Artificial Opening */
"0BF18ZZ"  =  "1"  /* Fragmentation in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BF20ZZ"  =  "1"  /* Fragmentation in Carina, Open Approach */
"0BF23ZZ"  =  "1"  /* Fragmentation in Carina, Percutaneous Approach */
"0BF24ZZ"  =  "1"  /* Fragmentation in Carina, Percutaneous Endoscopic Approach */
"0BF27ZZ"  =  "1"  /* Fragmentation in Carina, Via Natural or Artificial Opening */
"0BF28ZZ"  =  "1"  /* Fragmentation in Carina, Via Natural or Artificial Opening Endoscopic */
"0BF30ZZ"  =  "1"  /* Fragmentation in Right Main Bronchus, Open Approach */
"0BF33ZZ"  =  "1"  /* Fragmentation in Right Main Bronchus, Percutaneous Approach */
"0BF34ZZ"  =  "1"  /* Fragmentation in Right Main Bronchus, Percutaneous Endoscopic Approach */
"0BF37ZZ"  =  "1"  /* Fragmentation in Right Main Bronchus, Via Natural or Artificial Opening */
"0BF38ZZ"  =  "1"  /* Fragmentation in Right Main Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BF40ZZ"  =  "1"  /* Fragmentation in Right Upper Lobe Bronchus, Open Approach */
"0BF43ZZ"  =  "1"  /* Fragmentation in Right Upper Lobe Bronchus, Percutaneous Approach */
"0BF44ZZ"  =  "1"  /* Fragmentation in Right Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BF47ZZ"  =  "1"  /* Fragmentation in Right Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0BF48ZZ"  =  "1"  /* Fragmentation in Right Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BF50ZZ"  =  "1"  /* Fragmentation in Right Middle Lobe Bronchus, Open Approach */
"0BF53ZZ"  =  "1"  /* Fragmentation in Right Middle Lobe Bronchus, Percutaneous Approach */
"0BF54ZZ"  =  "1"  /* Fragmentation in Right Middle Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BF57ZZ"  =  "1"  /* Fragmentation in Right Middle Lobe Bronchus, Via Natural or Artificial Opening */
"0BF58ZZ"  =  "1"  /* Fragmentation in Right Middle Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BF60ZZ"  =  "1"  /* Fragmentation in Right Lower Lobe Bronchus, Open Approach */
"0BF63ZZ"  =  "1"  /* Fragmentation in Right Lower Lobe Bronchus, Percutaneous Approach */
"0BF64ZZ"  =  "1"  /* Fragmentation in Right Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BF67ZZ"  =  "1"  /* Fragmentation in Right Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0BF68ZZ"  =  "1"  /* Fragmentation in Right Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BF70ZZ"  =  "1"  /* Fragmentation in Left Main Bronchus, Open Approach */
"0BF73ZZ"  =  "1"  /* Fragmentation in Left Main Bronchus, Percutaneous Approach */
"0BF74ZZ"  =  "1"  /* Fragmentation in Left Main Bronchus, Percutaneous Endoscopic Approach */
"0BF77ZZ"  =  "1"  /* Fragmentation in Left Main Bronchus, Via Natural or Artificial Opening */
"0BF78ZZ"  =  "1"  /* Fragmentation in Left Main Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BF80ZZ"  =  "1"  /* Fragmentation in Left Upper Lobe Bronchus, Open Approach */
"0BF83ZZ"  =  "1"  /* Fragmentation in Left Upper Lobe Bronchus, Percutaneous Approach */
"0BF84ZZ"  =  "1"  /* Fragmentation in Left Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BF87ZZ"  =  "1"  /* Fragmentation in Left Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0BF88ZZ"  =  "1"  /* Fragmentation in Left Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BF90ZZ"  =  "1"  /* Fragmentation in Lingula Bronchus, Open Approach */
"0BF93ZZ"  =  "1"  /* Fragmentation in Lingula Bronchus, Percutaneous Approach */
"0BF94ZZ"  =  "1"  /* Fragmentation in Lingula Bronchus, Percutaneous Endoscopic Approach */
"0BF97ZZ"  =  "1"  /* Fragmentation in Lingula Bronchus, Via Natural or Artificial Opening */
"0BF98ZZ"  =  "1"  /* Fragmentation in Lingula Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BFB0ZZ"  =  "1"  /* Fragmentation in Left Lower Lobe Bronchus, Open Approach */
"0BFB3ZZ"  =  "1"  /* Fragmentation in Left Lower Lobe Bronchus, Percutaneous Approach */
"0BFB4ZZ"  =  "1"  /* Fragmentation in Left Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BFB7ZZ"  =  "1"  /* Fragmentation in Left Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0BFB8ZZ"  =  "1"  /* Fragmentation in Left Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BH002Z"  =  "1"  /* Insertion of Monitoring Device into Tracheobronchial Tree, Open Approach */
"0BH003Z"  =  "1"  /* Insertion of Infusion Device into Tracheobronchial Tree, Open Approach */
"0BH00DZ"  =  "1"  /* Insertion of Intraluminal Device into Tracheobronchial Tree, Open Approach */
"0BH032Z"  =  "1"  /* Insertion of Monitoring Device into Tracheobronchial Tree, Percutaneous Approach */
"0BH033Z"  =  "1"  /* Insertion of Infusion Device into Tracheobronchial Tree, Percutaneous Approach */
"0BH03DZ"  =  "1"  /* Insertion of Intraluminal Device into Tracheobronchial Tree, Percutaneous Approach */
"0BH042Z"  =  "1"  /* Insertion of Monitoring Device into Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BH043Z"  =  "1"  /* Insertion of Infusion Device into Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BH04DZ"  =  "1"  /* Insertion of Intraluminal Device into Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BH30GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Main Bronchus, Open Approach */
"0BH33GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Main Bronchus, Percutaneous Approach */
"0BH34GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Main Bronchus, Percutaneous Endoscopic Approach */
"0BH37GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Main Bronchus, Via Natural or Artificial Opening */
"0BH40GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Upper Lobe Bronchus, Open Approach */
"0BH43GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Upper Lobe Bronchus, Percutaneous Approach */
"0BH44GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BH47GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0BH50GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Middle Lobe Bronchus, Open Approach */
"0BH53GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Middle Lobe Bronchus, Percutaneous Approach */
"0BH54GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Middle Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BH57GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Middle Lobe Bronchus, Via Natural or Artificial Opening */
"0BH60GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Lower Lobe Bronchus, Open Approach */
"0BH63GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Lower Lobe Bronchus, Percutaneous Approach */
"0BH64GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BH67GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0BH70GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Main Bronchus, Open Approach */
"0BH73GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Main Bronchus, Percutaneous Approach */
"0BH74GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Main Bronchus, Percutaneous Endoscopic Approach */
"0BH77GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Main Bronchus, Via Natural or Artificial Opening */
"0BH80GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Upper Lobe Bronchus, Open Approach */
"0BH83GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Upper Lobe Bronchus, Percutaneous Approach */
"0BH84GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BH87GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0BH90GZ"  =  "1"  /* Insertion of Endobronchial Valve into Lingula Bronchus, Open Approach */
"0BH93GZ"  =  "1"  /* Insertion of Endobronchial Valve into Lingula Bronchus, Percutaneous Approach */
"0BH94GZ"  =  "1"  /* Insertion of Endobronchial Valve into Lingula Bronchus, Percutaneous Endoscopic Approach */
"0BH97GZ"  =  "1"  /* Insertion of Endobronchial Valve into Lingula Bronchus, Via Natural or Artificial Opening */
"0BHB0GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Lower Lobe Bronchus, Open Approach */
"0BHB3GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Lower Lobe Bronchus, Percutaneous Approach */
"0BHB4GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BHB7GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0BHK02Z"  =  "1"  /* Insertion of Monitoring Device into Right Lung, Open Approach */
"0BHK03Z"  =  "1"  /* Insertion of Infusion Device into Right Lung, Open Approach */
"0BHK32Z"  =  "1"  /* Insertion of Monitoring Device into Right Lung, Percutaneous Approach */
"0BHK33Z"  =  "1"  /* Insertion of Infusion Device into Right Lung, Percutaneous Approach */
"0BHK42Z"  =  "1"  /* Insertion of Monitoring Device into Right Lung, Percutaneous Endoscopic Approach */
"0BHK43Z"  =  "1"  /* Insertion of Infusion Device into Right Lung, Percutaneous Endoscopic Approach */
"0BHK72Z"  =  "1"  /* Insertion of Monitoring Device into Right Lung, Via Natural or Artificial Opening */
"0BHK73Z"  =  "1"  /* Insertion of Infusion Device into Right Lung, Via Natural or Artificial Opening */
"0BHK82Z"  =  "1"  /* Insertion of Monitoring Device into Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BHK83Z"  =  "1"  /* Insertion of Infusion Device into Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BHL02Z"  =  "1"  /* Insertion of Monitoring Device into Left Lung, Open Approach */
"0BHL03Z"  =  "1"  /* Insertion of Infusion Device into Left Lung, Open Approach */
"0BHL32Z"  =  "1"  /* Insertion of Monitoring Device into Left Lung, Percutaneous Approach */
"0BHL33Z"  =  "1"  /* Insertion of Infusion Device into Left Lung, Percutaneous Approach */
"0BHL42Z"  =  "1"  /* Insertion of Monitoring Device into Left Lung, Percutaneous Endoscopic Approach */
"0BHL43Z"  =  "1"  /* Insertion of Infusion Device into Left Lung, Percutaneous Endoscopic Approach */
"0BHL72Z"  =  "1"  /* Insertion of Monitoring Device into Left Lung, Via Natural or Artificial Opening */
"0BHL73Z"  =  "1"  /* Insertion of Infusion Device into Left Lung, Via Natural or Artificial Opening */
"0BHL82Z"  =  "1"  /* Insertion of Monitoring Device into Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BHL83Z"  =  "1"  /* Insertion of Infusion Device into Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BHR02Z"  =  "1"  /* Insertion of Monitoring Device into Right Diaphragm, Open Approach */
"0BHR0MZ"  =  "1"  /* Insertion of Diaphragmatic Pacemaker Lead into Right Diaphragm, Open Approach */
"0BHR32Z"  =  "1"  /* Insertion of Monitoring Device into Right Diaphragm, Percutaneous Approach */
"0BHR3MZ"  =  "1"  /* Insertion of Diaphragmatic Pacemaker Lead into Right Diaphragm, Percutaneous Approach */
"0BHR42Z"  =  "1"  /* Insertion of Monitoring Device into Right Diaphragm, Percutaneous Endoscopic Approach */
"0BHR4MZ"  =  "1"  /* Insertion of Diaphragmatic Pacemaker Lead into Right Diaphragm, Percutaneous Endoscopic Approach */
"0BHS02Z"  =  "1"  /* Insertion of Monitoring Device into Left Diaphragm, Open Approach */
"0BHS0MZ"  =  "1"  /* Insertion of Diaphragmatic Pacemaker Lead into Left Diaphragm, Open Approach */
"0BHS32Z"  =  "1"  /* Insertion of Monitoring Device into Left Diaphragm, Percutaneous Approach */
"0BHS3MZ"  =  "1"  /* Insertion of Diaphragmatic Pacemaker Lead into Left Diaphragm, Percutaneous Approach */
"0BHS42Z"  =  "1"  /* Insertion of Monitoring Device into Left Diaphragm, Percutaneous Endoscopic Approach */
"0BHS4MZ"  =  "1"  /* Insertion of Diaphragmatic Pacemaker Lead into Left Diaphragm, Percutaneous Endoscopic Approach */
"0BJ00ZZ"  =  "1"  /* Inspection of Tracheobronchial Tree, Open Approach */
"0BJ03ZZ"  =  "1"  /* Inspection of Tracheobronchial Tree, Percutaneous Approach */
"0BJ04ZZ"  =  "1"  /* Inspection of Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BJ07ZZ"  =  "1"  /* Inspection of Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BJ0XZZ"  =  "1"  /* Inspection of Tracheobronchial Tree, External Approach */
"0BJ10ZZ"  =  "1"  /* Inspection of Trachea, Open Approach */
"0BJK0ZZ"  =  "1"  /* Inspection of Right Lung, Open Approach */
"0BJK3ZZ"  =  "1"  /* Inspection of Right Lung, Percutaneous Approach */
"0BJK4ZZ"  =  "1"  /* Inspection of Right Lung, Percutaneous Endoscopic Approach */
"0BJK7ZZ"  =  "1"  /* Inspection of Right Lung, Via Natural or Artificial Opening */
"0BJKXZZ"  =  "1"  /* Inspection of Right Lung, External Approach */
"0BJL0ZZ"  =  "1"  /* Inspection of Left Lung, Open Approach */
"0BJL3ZZ"  =  "1"  /* Inspection of Left Lung, Percutaneous Approach */
"0BJL4ZZ"  =  "1"  /* Inspection of Left Lung, Percutaneous Endoscopic Approach */
"0BJL7ZZ"  =  "1"  /* Inspection of Left Lung, Via Natural or Artificial Opening */
"0BJLXZZ"  =  "1"  /* Inspection of Left Lung, External Approach */
"0BJQ0ZZ"  =  "1"  /* Inspection of Pleura, Open Approach */
"0BJQ3ZZ"  =  "1"  /* Inspection of Pleura, Percutaneous Approach */
"0BJQ4ZZ"  =  "1"  /* Inspection of Pleura, Percutaneous Endoscopic Approach */
"0BJQ7ZZ"  =  "1"  /* Inspection of Pleura, Via Natural or Artificial Opening */
"0BJQ8ZZ"  =  "1"  /* Inspection of Pleura, Via Natural or Artificial Opening Endoscopic */
"0BJQXZZ"  =  "1"  /* Inspection of Pleura, External Approach */
"0BJT0ZZ"  =  "1"  /* Inspection of Diaphragm, Open Approach */
"0BJT3ZZ"  =  "1"  /* Inspection of Diaphragm, Percutaneous Approach */
"0BJT4ZZ"  =  "1"  /* Inspection of Diaphragm, Percutaneous Endoscopic Approach */
"0BJT7ZZ"  =  "1"  /* Inspection of Diaphragm, Via Natural or Artificial Opening */
"0BJT8ZZ"  =  "1"  /* Inspection of Diaphragm, Via Natural or Artificial Opening Endoscopic */
"0BJTXZZ"  =  "1"  /* Inspection of Diaphragm, External Approach */
"0BL10CZ"  =  "1"  /* Occlusion of Trachea with Extraluminal Device, Open Approach */
"0BL10DZ"  =  "1"  /* Occlusion of Trachea with Intraluminal Device, Open Approach */
"0BL10ZZ"  =  "1"  /* Occlusion of Trachea, Open Approach */
"0BL13CZ"  =  "1"  /* Occlusion of Trachea with Extraluminal Device, Percutaneous Approach */
"0BL13DZ"  =  "1"  /* Occlusion of Trachea with Intraluminal Device, Percutaneous Approach */
"0BL13ZZ"  =  "1"  /* Occlusion of Trachea, Percutaneous Approach */
"0BL14CZ"  =  "1"  /* Occlusion of Trachea with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BL14DZ"  =  "1"  /* Occlusion of Trachea with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BL14ZZ"  =  "1"  /* Occlusion of Trachea, Percutaneous Endoscopic Approach */
"0BL17DZ"  =  "1"  /* Occlusion of Trachea with Intraluminal Device, Via Natural or Artificial Opening */
"0BL17ZZ"  =  "1"  /* Occlusion of Trachea, Via Natural or Artificial Opening */
"0BL18DZ"  =  "1"  /* Occlusion of Trachea with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BL18ZZ"  =  "1"  /* Occlusion of Trachea, Via Natural or Artificial Opening Endoscopic */
"0BL20CZ"  =  "1"  /* Occlusion of Carina with Extraluminal Device, Open Approach */
"0BL20DZ"  =  "1"  /* Occlusion of Carina with Intraluminal Device, Open Approach */
"0BL20ZZ"  =  "1"  /* Occlusion of Carina, Open Approach */
"0BL23CZ"  =  "1"  /* Occlusion of Carina with Extraluminal Device, Percutaneous Approach */
"0BL23DZ"  =  "1"  /* Occlusion of Carina with Intraluminal Device, Percutaneous Approach */
"0BL23ZZ"  =  "1"  /* Occlusion of Carina, Percutaneous Approach */
"0BL24CZ"  =  "1"  /* Occlusion of Carina with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BL24DZ"  =  "1"  /* Occlusion of Carina with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BL24ZZ"  =  "1"  /* Occlusion of Carina, Percutaneous Endoscopic Approach */
"0BL27DZ"  =  "1"  /* Occlusion of Carina with Intraluminal Device, Via Natural or Artificial Opening */
"0BL27ZZ"  =  "1"  /* Occlusion of Carina, Via Natural or Artificial Opening */
"0BL28DZ"  =  "1"  /* Occlusion of Carina with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BL28ZZ"  =  "1"  /* Occlusion of Carina, Via Natural or Artificial Opening Endoscopic */
"0BL30CZ"  =  "1"  /* Occlusion of Right Main Bronchus with Extraluminal Device, Open Approach */
"0BL30DZ"  =  "1"  /* Occlusion of Right Main Bronchus with Intraluminal Device, Open Approach */
"0BL30ZZ"  =  "1"  /* Occlusion of Right Main Bronchus, Open Approach */
"0BL33CZ"  =  "1"  /* Occlusion of Right Main Bronchus with Extraluminal Device, Percutaneous Approach */
"0BL33DZ"  =  "1"  /* Occlusion of Right Main Bronchus with Intraluminal Device, Percutaneous Approach */
"0BL33ZZ"  =  "1"  /* Occlusion of Right Main Bronchus, Percutaneous Approach */
"0BL34CZ"  =  "1"  /* Occlusion of Right Main Bronchus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BL34DZ"  =  "1"  /* Occlusion of Right Main Bronchus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BL34ZZ"  =  "1"  /* Occlusion of Right Main Bronchus, Percutaneous Endoscopic Approach */
"0BL37DZ"  =  "1"  /* Occlusion of Right Main Bronchus with Intraluminal Device, Via Natural or Artificial Opening */
"0BL37ZZ"  =  "1"  /* Occlusion of Right Main Bronchus, Via Natural or Artificial Opening */
"0BL38DZ"  =  "1"  /* Occlusion of Right Main Bronchus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BL38ZZ"  =  "1"  /* Occlusion of Right Main Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BL40CZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus with Extraluminal Device, Open Approach */
"0BL40DZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus with Intraluminal Device, Open Approach */
"0BL40ZZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus, Open Approach */
"0BL43CZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus with Extraluminal Device, Percutaneous Approach */
"0BL43DZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus with Intraluminal Device, Percutaneous Approach */
"0BL43ZZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus, Percutaneous Approach */
"0BL44CZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BL44DZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BL44ZZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BL47DZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening */
"0BL47ZZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0BL48DZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BL48ZZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BL50CZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus with Extraluminal Device, Open Approach */
"0BL50DZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus with Intraluminal Device, Open Approach */
"0BL50ZZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus, Open Approach */
"0BL53CZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus with Extraluminal Device, Percutaneous Approach */
"0BL53DZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus with Intraluminal Device, Percutaneous Approach */
"0BL53ZZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus, Percutaneous Approach */
"0BL54CZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BL54DZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BL54ZZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BL57DZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening */
"0BL57ZZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus, Via Natural or Artificial Opening */
"0BL58DZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BL58ZZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BL60CZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus with Extraluminal Device, Open Approach */
"0BL60DZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus with Intraluminal Device, Open Approach */
"0BL60ZZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus, Open Approach */
"0BL63CZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus with Extraluminal Device, Percutaneous Approach */
"0BL63DZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus with Intraluminal Device, Percutaneous Approach */
"0BL63ZZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus, Percutaneous Approach */
"0BL64CZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BL64DZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BL64ZZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BL67DZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening */
"0BL67ZZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0BL68DZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BL68ZZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BL70CZ"  =  "1"  /* Occlusion of Left Main Bronchus with Extraluminal Device, Open Approach */
"0BL70DZ"  =  "1"  /* Occlusion of Left Main Bronchus with Intraluminal Device, Open Approach */
"0BL70ZZ"  =  "1"  /* Occlusion of Left Main Bronchus, Open Approach */
"0BL73CZ"  =  "1"  /* Occlusion of Left Main Bronchus with Extraluminal Device, Percutaneous Approach */
"0BL73DZ"  =  "1"  /* Occlusion of Left Main Bronchus with Intraluminal Device, Percutaneous Approach */
"0BL73ZZ"  =  "1"  /* Occlusion of Left Main Bronchus, Percutaneous Approach */
"0BL74CZ"  =  "1"  /* Occlusion of Left Main Bronchus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BL74DZ"  =  "1"  /* Occlusion of Left Main Bronchus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BL74ZZ"  =  "1"  /* Occlusion of Left Main Bronchus, Percutaneous Endoscopic Approach */
"0BL77DZ"  =  "1"  /* Occlusion of Left Main Bronchus with Intraluminal Device, Via Natural or Artificial Opening */
"0BL77ZZ"  =  "1"  /* Occlusion of Left Main Bronchus, Via Natural or Artificial Opening */
"0BL78DZ"  =  "1"  /* Occlusion of Left Main Bronchus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BL78ZZ"  =  "1"  /* Occlusion of Left Main Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BL80CZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus with Extraluminal Device, Open Approach */
"0BL80DZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus with Intraluminal Device, Open Approach */
"0BL80ZZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus, Open Approach */
"0BL83CZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus with Extraluminal Device, Percutaneous Approach */
"0BL83DZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus with Intraluminal Device, Percutaneous Approach */
"0BL83ZZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus, Percutaneous Approach */
"0BL84CZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BL84DZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BL84ZZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BL87DZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening */
"0BL87ZZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0BL88DZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BL88ZZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BL90CZ"  =  "1"  /* Occlusion of Lingula Bronchus with Extraluminal Device, Open Approach */
"0BL90DZ"  =  "1"  /* Occlusion of Lingula Bronchus with Intraluminal Device, Open Approach */
"0BL90ZZ"  =  "1"  /* Occlusion of Lingula Bronchus, Open Approach */
"0BL93CZ"  =  "1"  /* Occlusion of Lingula Bronchus with Extraluminal Device, Percutaneous Approach */
"0BL93DZ"  =  "1"  /* Occlusion of Lingula Bronchus with Intraluminal Device, Percutaneous Approach */
"0BL93ZZ"  =  "1"  /* Occlusion of Lingula Bronchus, Percutaneous Approach */
"0BL94CZ"  =  "1"  /* Occlusion of Lingula Bronchus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BL94DZ"  =  "1"  /* Occlusion of Lingula Bronchus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BL94ZZ"  =  "1"  /* Occlusion of Lingula Bronchus, Percutaneous Endoscopic Approach */
"0BL97DZ"  =  "1"  /* Occlusion of Lingula Bronchus with Intraluminal Device, Via Natural or Artificial Opening */
"0BL97ZZ"  =  "1"  /* Occlusion of Lingula Bronchus, Via Natural or Artificial Opening */
"0BL98DZ"  =  "1"  /* Occlusion of Lingula Bronchus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BL98ZZ"  =  "1"  /* Occlusion of Lingula Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BLB0CZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus with Extraluminal Device, Open Approach */
"0BLB0DZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus with Intraluminal Device, Open Approach */
"0BLB0ZZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus, Open Approach */
"0BLB3CZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus with Extraluminal Device, Percutaneous Approach */
"0BLB3DZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus with Intraluminal Device, Percutaneous Approach */
"0BLB3ZZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus, Percutaneous Approach */
"0BLB4CZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BLB4DZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BLB4ZZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BLB7DZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening */
"0BLB7ZZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0BLB8DZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BLB8ZZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BM10ZZ"  =  "1"  /* Reattachment of Trachea, Open Approach */
"0BM20ZZ"  =  "1"  /* Reattachment of Carina, Open Approach */
"0BM30ZZ"  =  "1"  /* Reattachment of Right Main Bronchus, Open Approach */
"0BM40ZZ"  =  "1"  /* Reattachment of Right Upper Lobe Bronchus, Open Approach */
"0BM50ZZ"  =  "1"  /* Reattachment of Right Middle Lobe Bronchus, Open Approach */
"0BM60ZZ"  =  "1"  /* Reattachment of Right Lower Lobe Bronchus, Open Approach */
"0BM70ZZ"  =  "1"  /* Reattachment of Left Main Bronchus, Open Approach */
"0BM80ZZ"  =  "1"  /* Reattachment of Left Upper Lobe Bronchus, Open Approach */
"0BM90ZZ"  =  "1"  /* Reattachment of Lingula Bronchus, Open Approach */
"0BMB0ZZ"  =  "1"  /* Reattachment of Left Lower Lobe Bronchus, Open Approach */
"0BMC0ZZ"  =  "1"  /* Reattachment of Right Upper Lung Lobe, Open Approach */
"0BMD0ZZ"  =  "1"  /* Reattachment of Right Middle Lung Lobe, Open Approach */
"0BMF0ZZ"  =  "1"  /* Reattachment of Right Lower Lung Lobe, Open Approach */
"0BMG0ZZ"  =  "1"  /* Reattachment of Left Upper Lung Lobe, Open Approach */
"0BMH0ZZ"  =  "1"  /* Reattachment of Lung Lingula, Open Approach */
"0BMJ0ZZ"  =  "1"  /* Reattachment of Left Lower Lung Lobe, Open Approach */
"0BMK0ZZ"  =  "1"  /* Reattachment of Right Lung, Open Approach */
"0BML0ZZ"  =  "1"  /* Reattachment of Left Lung, Open Approach */
"0BMR0ZZ"  =  "1"  /* Reattachment of Right Diaphragm, Open Approach */
"0BMS0ZZ"  =  "1"  /* Reattachment of Left Diaphragm, Open Approach */
"0BN30ZZ"  =  "1"  /* Release Right Main Bronchus, Open Approach */
"0BN33ZZ"  =  "1"  /* Release Right Main Bronchus, Percutaneous Approach */
"0BN34ZZ"  =  "1"  /* Release Right Main Bronchus, Percutaneous Endoscopic Approach */
"0BN37ZZ"  =  "1"  /* Release Right Main Bronchus, Via Natural or Artificial Opening */
"0BN38ZZ"  =  "1"  /* Release Right Main Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BN40ZZ"  =  "1"  /* Release Right Upper Lobe Bronchus, Open Approach */
"0BN43ZZ"  =  "1"  /* Release Right Upper Lobe Bronchus, Percutaneous Approach */
"0BN44ZZ"  =  "1"  /* Release Right Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BN47ZZ"  =  "1"  /* Release Right Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0BN48ZZ"  =  "1"  /* Release Right Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BN50ZZ"  =  "1"  /* Release Right Middle Lobe Bronchus, Open Approach */
"0BN53ZZ"  =  "1"  /* Release Right Middle Lobe Bronchus, Percutaneous Approach */
"0BN54ZZ"  =  "1"  /* Release Right Middle Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BN57ZZ"  =  "1"  /* Release Right Middle Lobe Bronchus, Via Natural or Artificial Opening */
"0BN58ZZ"  =  "1"  /* Release Right Middle Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BN60ZZ"  =  "1"  /* Release Right Lower Lobe Bronchus, Open Approach */
"0BN63ZZ"  =  "1"  /* Release Right Lower Lobe Bronchus, Percutaneous Approach */
"0BN64ZZ"  =  "1"  /* Release Right Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BN67ZZ"  =  "1"  /* Release Right Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0BN68ZZ"  =  "1"  /* Release Right Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BN70ZZ"  =  "1"  /* Release Left Main Bronchus, Open Approach */
"0BN73ZZ"  =  "1"  /* Release Left Main Bronchus, Percutaneous Approach */
"0BN74ZZ"  =  "1"  /* Release Left Main Bronchus, Percutaneous Endoscopic Approach */
"0BN77ZZ"  =  "1"  /* Release Left Main Bronchus, Via Natural or Artificial Opening */
"0BN78ZZ"  =  "1"  /* Release Left Main Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BN80ZZ"  =  "1"  /* Release Left Upper Lobe Bronchus, Open Approach */
"0BN83ZZ"  =  "1"  /* Release Left Upper Lobe Bronchus, Percutaneous Approach */
"0BN84ZZ"  =  "1"  /* Release Left Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BN87ZZ"  =  "1"  /* Release Left Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0BN88ZZ"  =  "1"  /* Release Left Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BN90ZZ"  =  "1"  /* Release Lingula Bronchus, Open Approach */
"0BN93ZZ"  =  "1"  /* Release Lingula Bronchus, Percutaneous Approach */
"0BN94ZZ"  =  "1"  /* Release Lingula Bronchus, Percutaneous Endoscopic Approach */
"0BN97ZZ"  =  "1"  /* Release Lingula Bronchus, Via Natural or Artificial Opening */
"0BN98ZZ"  =  "1"  /* Release Lingula Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BNB0ZZ"  =  "1"  /* Release Left Lower Lobe Bronchus, Open Approach */
"0BNB3ZZ"  =  "1"  /* Release Left Lower Lobe Bronchus, Percutaneous Approach */
"0BNB4ZZ"  =  "1"  /* Release Left Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BNB7ZZ"  =  "1"  /* Release Left Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0BNB8ZZ"  =  "1"  /* Release Left Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BNC0ZZ"  =  "1"  /* Release Right Upper Lung Lobe, Open Approach */
"0BNC3ZZ"  =  "1"  /* Release Right Upper Lung Lobe, Percutaneous Approach */
"0BNC4ZZ"  =  "1"  /* Release Right Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0BNC7ZZ"  =  "1"  /* Release Right Upper Lung Lobe, Via Natural or Artificial Opening */
"0BNC8ZZ"  =  "1"  /* Release Right Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BND0ZZ"  =  "1"  /* Release Right Middle Lung Lobe, Open Approach */
"0BND3ZZ"  =  "1"  /* Release Right Middle Lung Lobe, Percutaneous Approach */
"0BND4ZZ"  =  "1"  /* Release Right Middle Lung Lobe, Percutaneous Endoscopic Approach */
"0BND7ZZ"  =  "1"  /* Release Right Middle Lung Lobe, Via Natural or Artificial Opening */
"0BND8ZZ"  =  "1"  /* Release Right Middle Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BNF0ZZ"  =  "1"  /* Release Right Lower Lung Lobe, Open Approach */
"0BNF3ZZ"  =  "1"  /* Release Right Lower Lung Lobe, Percutaneous Approach */
"0BNF4ZZ"  =  "1"  /* Release Right Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0BNF7ZZ"  =  "1"  /* Release Right Lower Lung Lobe, Via Natural or Artificial Opening */
"0BNF8ZZ"  =  "1"  /* Release Right Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BNG0ZZ"  =  "1"  /* Release Left Upper Lung Lobe, Open Approach */
"0BNG3ZZ"  =  "1"  /* Release Left Upper Lung Lobe, Percutaneous Approach */
"0BNG4ZZ"  =  "1"  /* Release Left Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0BNG7ZZ"  =  "1"  /* Release Left Upper Lung Lobe, Via Natural or Artificial Opening */
"0BNG8ZZ"  =  "1"  /* Release Left Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BNH0ZZ"  =  "1"  /* Release Lung Lingula, Open Approach */
"0BNH3ZZ"  =  "1"  /* Release Lung Lingula, Percutaneous Approach */
"0BNH4ZZ"  =  "1"  /* Release Lung Lingula, Percutaneous Endoscopic Approach */
"0BNH7ZZ"  =  "1"  /* Release Lung Lingula, Via Natural or Artificial Opening */
"0BNH8ZZ"  =  "1"  /* Release Lung Lingula, Via Natural or Artificial Opening Endoscopic */
"0BNJ0ZZ"  =  "1"  /* Release Left Lower Lung Lobe, Open Approach */
"0BNJ3ZZ"  =  "1"  /* Release Left Lower Lung Lobe, Percutaneous Approach */
"0BNJ4ZZ"  =  "1"  /* Release Left Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0BNJ7ZZ"  =  "1"  /* Release Left Lower Lung Lobe, Via Natural or Artificial Opening */
"0BNJ8ZZ"  =  "1"  /* Release Left Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BNK0ZZ"  =  "1"  /* Release Right Lung, Open Approach */
"0BNK3ZZ"  =  "1"  /* Release Right Lung, Percutaneous Approach */
"0BNK4ZZ"  =  "1"  /* Release Right Lung, Percutaneous Endoscopic Approach */
"0BNK7ZZ"  =  "1"  /* Release Right Lung, Via Natural or Artificial Opening */
"0BNK8ZZ"  =  "1"  /* Release Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BNL0ZZ"  =  "1"  /* Release Left Lung, Open Approach */
"0BNL3ZZ"  =  "1"  /* Release Left Lung, Percutaneous Approach */
"0BNL4ZZ"  =  "1"  /* Release Left Lung, Percutaneous Endoscopic Approach */
"0BNL7ZZ"  =  "1"  /* Release Left Lung, Via Natural or Artificial Opening */
"0BNL8ZZ"  =  "1"  /* Release Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BNM0ZZ"  =  "1"  /* Release Bilateral Lungs, Open Approach */
"0BNM3ZZ"  =  "1"  /* Release Bilateral Lungs, Percutaneous Approach */
"0BNM4ZZ"  =  "1"  /* Release Bilateral Lungs, Percutaneous Endoscopic Approach */
"0BNM7ZZ"  =  "1"  /* Release Bilateral Lungs, Via Natural or Artificial Opening */
"0BNM8ZZ"  =  "1"  /* Release Bilateral Lungs, Via Natural or Artificial Opening Endoscopic */
"0BNN0ZZ"  =  "1"  /* Release Right Pleura, Open Approach */
"0BNN3ZZ"  =  "1"  /* Release Right Pleura, Percutaneous Approach */
"0BNN4ZZ"  =  "1"  /* Release Right Pleura, Percutaneous Endoscopic Approach */
"0BNP0ZZ"  =  "1"  /* Release Left Pleura, Open Approach */
"0BNP3ZZ"  =  "1"  /* Release Left Pleura, Percutaneous Approach */
"0BNP4ZZ"  =  "1"  /* Release Left Pleura, Percutaneous Endoscopic Approach */
"0BNR0ZZ"  =  "1"  /* Release Right Diaphragm, Open Approach */
"0BNR3ZZ"  =  "1"  /* Release Right Diaphragm, Percutaneous Approach */
"0BNR4ZZ"  =  "1"  /* Release Right Diaphragm, Percutaneous Endoscopic Approach */
"0BNS0ZZ"  =  "1"  /* Release Left Diaphragm, Open Approach */
"0BNS3ZZ"  =  "1"  /* Release Left Diaphragm, Percutaneous Approach */
"0BNS4ZZ"  =  "1"  /* Release Left Diaphragm, Percutaneous Endoscopic Approach */
"0BP000Z"  =  "1"  /* Removal of Drainage Device from Tracheobronchial Tree, Open Approach */
"0BP001Z"  =  "1"  /* Removal of Radioactive Element from Tracheobronchial Tree, Open Approach */
"0BP002Z"  =  "1"  /* Removal of Monitoring Device from Tracheobronchial Tree, Open Approach */
"0BP00CZ"  =  "1"  /* Removal of Extraluminal Device from Tracheobronchial Tree, Open Approach */
"0BP00DZ"  =  "1"  /* Removal of Intraluminal Device from Tracheobronchial Tree, Open Approach */
"0BP00JZ"  =  "1"  /* Removal of Synthetic Substitute from Tracheobronchial Tree, Open Approach */
"0BP00KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Tracheobronchial Tree, Open Approach */
"0BP030Z"  =  "1"  /* Removal of Drainage Device from Tracheobronchial Tree, Percutaneous Approach */
"0BP031Z"  =  "1"  /* Removal of Radioactive Element from Tracheobronchial Tree, Percutaneous Approach */
"0BP032Z"  =  "1"  /* Removal of Monitoring Device from Tracheobronchial Tree, Percutaneous Approach */
"0BP03CZ"  =  "1"  /* Removal of Extraluminal Device from Tracheobronchial Tree, Percutaneous Approach */
"0BP03DZ"  =  "1"  /* Removal of Intraluminal Device from Tracheobronchial Tree, Percutaneous Approach */
"0BP03JZ"  =  "1"  /* Removal of Synthetic Substitute from Tracheobronchial Tree, Percutaneous Approach */
"0BP03KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Tracheobronchial Tree, Percutaneous Approach */
"0BP040Z"  =  "1"  /* Removal of Drainage Device from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP041Z"  =  "1"  /* Removal of Radioactive Element from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP042Z"  =  "1"  /* Removal of Monitoring Device from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP04CZ"  =  "1"  /* Removal of Extraluminal Device from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP04DZ"  =  "1"  /* Removal of Intraluminal Device from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP04JZ"  =  "1"  /* Removal of Synthetic Substitute from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP04KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP070Z"  =  "1"  /* Removal of Drainage Device from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP071Z"  =  "1"  /* Removal of Radioactive Element from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP072Z"  =  "1"  /* Removal of Monitoring Device from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP07DZ"  =  "1"  /* Removal of Intraluminal Device from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP080Z"  =  "1"  /* Removal of Drainage Device from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP081Z"  =  "1"  /* Removal of Radioactive Element from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP082Z"  =  "1"  /* Removal of Monitoring Device from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP08DZ"  =  "1"  /* Removal of Intraluminal Device from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BPK00Z"  =  "1"  /* Removal of Drainage Device from Right Lung, Open Approach */
"0BPK01Z"  =  "1"  /* Removal of Radioactive Element from Right Lung, Open Approach */
"0BPK02Z"  =  "1"  /* Removal of Monitoring Device from Right Lung, Open Approach */
"0BPK03Z"  =  "1"  /* Removal of Infusion Device from Right Lung, Open Approach */
"0BPK30Z"  =  "1"  /* Removal of Drainage Device from Right Lung, Percutaneous Approach */
"0BPK31Z"  =  "1"  /* Removal of Radioactive Element from Right Lung, Percutaneous Approach */
"0BPK32Z"  =  "1"  /* Removal of Monitoring Device from Right Lung, Percutaneous Approach */
"0BPK33Z"  =  "1"  /* Removal of Infusion Device from Right Lung, Percutaneous Approach */
"0BPK40Z"  =  "1"  /* Removal of Drainage Device from Right Lung, Percutaneous Endoscopic Approach */
"0BPK41Z"  =  "1"  /* Removal of Radioactive Element from Right Lung, Percutaneous Endoscopic Approach */
"0BPK42Z"  =  "1"  /* Removal of Monitoring Device from Right Lung, Percutaneous Endoscopic Approach */
"0BPK43Z"  =  "1"  /* Removal of Infusion Device from Right Lung, Percutaneous Endoscopic Approach */
"0BPK70Z"  =  "1"  /* Removal of Drainage Device from Right Lung, Via Natural or Artificial Opening */
"0BPK71Z"  =  "1"  /* Removal of Radioactive Element from Right Lung, Via Natural or Artificial Opening */
"0BPK72Z"  =  "1"  /* Removal of Monitoring Device from Right Lung, Via Natural or Artificial Opening */
"0BPK73Z"  =  "1"  /* Removal of Infusion Device from Right Lung, Via Natural or Artificial Opening */
"0BPK80Z"  =  "1"  /* Removal of Drainage Device from Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BPK81Z"  =  "1"  /* Removal of Radioactive Element from Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BPK82Z"  =  "1"  /* Removal of Monitoring Device from Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BPK83Z"  =  "1"  /* Removal of Infusion Device from Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BPL00Z"  =  "1"  /* Removal of Drainage Device from Left Lung, Open Approach */
"0BPL01Z"  =  "1"  /* Removal of Radioactive Element from Left Lung, Open Approach */
"0BPL02Z"  =  "1"  /* Removal of Monitoring Device from Left Lung, Open Approach */
"0BPL03Z"  =  "1"  /* Removal of Infusion Device from Left Lung, Open Approach */
"0BPL30Z"  =  "1"  /* Removal of Drainage Device from Left Lung, Percutaneous Approach */
"0BPL31Z"  =  "1"  /* Removal of Radioactive Element from Left Lung, Percutaneous Approach */
"0BPL32Z"  =  "1"  /* Removal of Monitoring Device from Left Lung, Percutaneous Approach */
"0BPL33Z"  =  "1"  /* Removal of Infusion Device from Left Lung, Percutaneous Approach */
"0BPL40Z"  =  "1"  /* Removal of Drainage Device from Left Lung, Percutaneous Endoscopic Approach */
"0BPL41Z"  =  "1"  /* Removal of Radioactive Element from Left Lung, Percutaneous Endoscopic Approach */
"0BPL42Z"  =  "1"  /* Removal of Monitoring Device from Left Lung, Percutaneous Endoscopic Approach */
"0BPL43Z"  =  "1"  /* Removal of Infusion Device from Left Lung, Percutaneous Endoscopic Approach */
"0BPL70Z"  =  "1"  /* Removal of Drainage Device from Left Lung, Via Natural or Artificial Opening */
"0BPL71Z"  =  "1"  /* Removal of Radioactive Element from Left Lung, Via Natural or Artificial Opening */
"0BPL72Z"  =  "1"  /* Removal of Monitoring Device from Left Lung, Via Natural or Artificial Opening */
"0BPL73Z"  =  "1"  /* Removal of Infusion Device from Left Lung, Via Natural or Artificial Opening */
"0BPL80Z"  =  "1"  /* Removal of Drainage Device from Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BPL81Z"  =  "1"  /* Removal of Radioactive Element from Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BPL82Z"  =  "1"  /* Removal of Monitoring Device from Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BPL83Z"  =  "1"  /* Removal of Infusion Device from Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BPQ00Z"  =  "1"  /* Removal of Drainage Device from Pleura, Open Approach */
"0BPQ01Z"  =  "1"  /* Removal of Radioactive Element from Pleura, Open Approach */
"0BPQ02Z"  =  "1"  /* Removal of Monitoring Device from Pleura, Open Approach */
"0BPQ30Z"  =  "1"  /* Removal of Drainage Device from Pleura, Percutaneous Approach */
"0BPQ31Z"  =  "1"  /* Removal of Radioactive Element from Pleura, Percutaneous Approach */
"0BPQ32Z"  =  "1"  /* Removal of Monitoring Device from Pleura, Percutaneous Approach */
"0BPQ40Z"  =  "1"  /* Removal of Drainage Device from Pleura, Percutaneous Endoscopic Approach */
"0BPQ41Z"  =  "1"  /* Removal of Radioactive Element from Pleura, Percutaneous Endoscopic Approach */
"0BPQ42Z"  =  "1"  /* Removal of Monitoring Device from Pleura, Percutaneous Endoscopic Approach */
"0BPQ70Z"  =  "1"  /* Removal of Drainage Device from Pleura, Via Natural or Artificial Opening */
"0BPQ71Z"  =  "1"  /* Removal of Radioactive Element from Pleura, Via Natural or Artificial Opening */
"0BPQ72Z"  =  "1"  /* Removal of Monitoring Device from Pleura, Via Natural or Artificial Opening */
"0BPQ80Z"  =  "1"  /* Removal of Drainage Device from Pleura, Via Natural or Artificial Opening Endoscopic */
"0BPQ81Z"  =  "1"  /* Removal of Radioactive Element from Pleura, Via Natural or Artificial Opening Endoscopic */
"0BPQ82Z"  =  "1"  /* Removal of Monitoring Device from Pleura, Via Natural or Artificial Opening Endoscopic */
"0BPT00Z"  =  "1"  /* Removal of Drainage Device from Diaphragm, Open Approach */
"0BPT02Z"  =  "1"  /* Removal of Monitoring Device from Diaphragm, Open Approach */
"0BPT07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Diaphragm, Open Approach */
"0BPT0JZ"  =  "1"  /* Removal of Synthetic Substitute from Diaphragm, Open Approach */
"0BPT0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Diaphragm, Open Approach */
"0BPT0MZ"  =  "1"  /* Removal of Diaphragmatic Pacemaker Lead from Diaphragm, Open Approach */
"0BPT30Z"  =  "1"  /* Removal of Drainage Device from Diaphragm, Percutaneous Approach */
"0BPT32Z"  =  "1"  /* Removal of Monitoring Device from Diaphragm, Percutaneous Approach */
"0BPT37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Diaphragm, Percutaneous Approach */
"0BPT3JZ"  =  "1"  /* Removal of Synthetic Substitute from Diaphragm, Percutaneous Approach */
"0BPT3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Diaphragm, Percutaneous Approach */
"0BPT3MZ"  =  "1"  /* Removal of Diaphragmatic Pacemaker Lead from Diaphragm, Percutaneous Approach */
"0BPT40Z"  =  "1"  /* Removal of Drainage Device from Diaphragm, Percutaneous Endoscopic Approach */
"0BPT42Z"  =  "1"  /* Removal of Monitoring Device from Diaphragm, Percutaneous Endoscopic Approach */
"0BPT47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Diaphragm, Percutaneous Endoscopic Approach */
"0BPT4JZ"  =  "1"  /* Removal of Synthetic Substitute from Diaphragm, Percutaneous Endoscopic Approach */
"0BPT4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Diaphragm, Percutaneous Endoscopic Approach */
"0BPT4MZ"  =  "1"  /* Removal of Diaphragmatic Pacemaker Lead from Diaphragm, Percutaneous Endoscopic Approach */
"0BPT70Z"  =  "1"  /* Removal of Drainage Device from Diaphragm, Via Natural or Artificial Opening */
"0BPT72Z"  =  "1"  /* Removal of Monitoring Device from Diaphragm, Via Natural or Artificial Opening */
"0BPT77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Diaphragm, Via Natural or Artificial Opening */
"0BPT7JZ"  =  "1"  /* Removal of Synthetic Substitute from Diaphragm, Via Natural or Artificial Opening */
"0BPT7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Diaphragm, Via Natural or Artificial Opening */
"0BPT7MZ"  =  "1"  /* Removal of Diaphragmatic Pacemaker Lead from Diaphragm, Via Natural or Artificial Opening */
"0BPT80Z"  =  "1"  /* Removal of Drainage Device from Diaphragm, Via Natural or Artificial Opening Endoscopic */
"0BPT82Z"  =  "1"  /* Removal of Monitoring Device from Diaphragm, Via Natural or Artificial Opening Endoscopic */
"0BPT87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Diaphragm, Via Natural or Artificial Opening Endoscopic */
"0BPT8JZ"  =  "1"  /* Removal of Synthetic Substitute from Diaphragm, Via Natural or Artificial Opening Endoscopic */
"0BPT8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Diaphragm, Via Natural or Artificial Opening Endoscopic */
"0BPT8MZ"  =  "1"  /* Removal of Diaphragmatic Pacemaker Lead from Diaphragm, Via Natural or Artificial Opening Endoscopic */
"0BQ10ZZ"  =  "1"  /* Repair Trachea, Open Approach */
"0BQ13ZZ"  =  "1"  /* Repair Trachea, Percutaneous Approach */
"0BQ14ZZ"  =  "1"  /* Repair Trachea, Percutaneous Endoscopic Approach */
"0BQ17ZZ"  =  "1"  /* Repair Trachea, Via Natural or Artificial Opening */
"0BQ18ZZ"  =  "1"  /* Repair Trachea, Via Natural or Artificial Opening Endoscopic */
"0BQ20ZZ"  =  "1"  /* Repair Carina, Open Approach */
"0BQ23ZZ"  =  "1"  /* Repair Carina, Percutaneous Approach */
"0BQ24ZZ"  =  "1"  /* Repair Carina, Percutaneous Endoscopic Approach */
"0BQ27ZZ"  =  "1"  /* Repair Carina, Via Natural or Artificial Opening */
"0BQ28ZZ"  =  "1"  /* Repair Carina, Via Natural or Artificial Opening Endoscopic */
"0BQ30ZZ"  =  "1"  /* Repair Right Main Bronchus, Open Approach */
"0BQ33ZZ"  =  "1"  /* Repair Right Main Bronchus, Percutaneous Approach */
"0BQ34ZZ"  =  "1"  /* Repair Right Main Bronchus, Percutaneous Endoscopic Approach */
"0BQ37ZZ"  =  "1"  /* Repair Right Main Bronchus, Via Natural or Artificial Opening */
"0BQ38ZZ"  =  "1"  /* Repair Right Main Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BQ40ZZ"  =  "1"  /* Repair Right Upper Lobe Bronchus, Open Approach */
"0BQ43ZZ"  =  "1"  /* Repair Right Upper Lobe Bronchus, Percutaneous Approach */
"0BQ44ZZ"  =  "1"  /* Repair Right Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BQ47ZZ"  =  "1"  /* Repair Right Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0BQ48ZZ"  =  "1"  /* Repair Right Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BQ50ZZ"  =  "1"  /* Repair Right Middle Lobe Bronchus, Open Approach */
"0BQ53ZZ"  =  "1"  /* Repair Right Middle Lobe Bronchus, Percutaneous Approach */
"0BQ54ZZ"  =  "1"  /* Repair Right Middle Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BQ57ZZ"  =  "1"  /* Repair Right Middle Lobe Bronchus, Via Natural or Artificial Opening */
"0BQ58ZZ"  =  "1"  /* Repair Right Middle Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BQ60ZZ"  =  "1"  /* Repair Right Lower Lobe Bronchus, Open Approach */
"0BQ63ZZ"  =  "1"  /* Repair Right Lower Lobe Bronchus, Percutaneous Approach */
"0BQ64ZZ"  =  "1"  /* Repair Right Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BQ67ZZ"  =  "1"  /* Repair Right Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0BQ68ZZ"  =  "1"  /* Repair Right Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BQ70ZZ"  =  "1"  /* Repair Left Main Bronchus, Open Approach */
"0BQ73ZZ"  =  "1"  /* Repair Left Main Bronchus, Percutaneous Approach */
"0BQ74ZZ"  =  "1"  /* Repair Left Main Bronchus, Percutaneous Endoscopic Approach */
"0BQ77ZZ"  =  "1"  /* Repair Left Main Bronchus, Via Natural or Artificial Opening */
"0BQ78ZZ"  =  "1"  /* Repair Left Main Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BQ80ZZ"  =  "1"  /* Repair Left Upper Lobe Bronchus, Open Approach */
"0BQ83ZZ"  =  "1"  /* Repair Left Upper Lobe Bronchus, Percutaneous Approach */
"0BQ84ZZ"  =  "1"  /* Repair Left Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BQ87ZZ"  =  "1"  /* Repair Left Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0BQ88ZZ"  =  "1"  /* Repair Left Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BQ90ZZ"  =  "1"  /* Repair Lingula Bronchus, Open Approach */
"0BQ93ZZ"  =  "1"  /* Repair Lingula Bronchus, Percutaneous Approach */
"0BQ94ZZ"  =  "1"  /* Repair Lingula Bronchus, Percutaneous Endoscopic Approach */
"0BQ97ZZ"  =  "1"  /* Repair Lingula Bronchus, Via Natural or Artificial Opening */
"0BQ98ZZ"  =  "1"  /* Repair Lingula Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BQB0ZZ"  =  "1"  /* Repair Left Lower Lobe Bronchus, Open Approach */
"0BQB3ZZ"  =  "1"  /* Repair Left Lower Lobe Bronchus, Percutaneous Approach */
"0BQB4ZZ"  =  "1"  /* Repair Left Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BQB7ZZ"  =  "1"  /* Repair Left Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0BQB8ZZ"  =  "1"  /* Repair Left Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BQC0ZZ"  =  "1"  /* Repair Right Upper Lung Lobe, Open Approach */
"0BQC3ZZ"  =  "1"  /* Repair Right Upper Lung Lobe, Percutaneous Approach */
"0BQC4ZZ"  =  "1"  /* Repair Right Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0BQC7ZZ"  =  "1"  /* Repair Right Upper Lung Lobe, Via Natural or Artificial Opening */
"0BQC8ZZ"  =  "1"  /* Repair Right Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BQD0ZZ"  =  "1"  /* Repair Right Middle Lung Lobe, Open Approach */
"0BQD3ZZ"  =  "1"  /* Repair Right Middle Lung Lobe, Percutaneous Approach */
"0BQD4ZZ"  =  "1"  /* Repair Right Middle Lung Lobe, Percutaneous Endoscopic Approach */
"0BQD7ZZ"  =  "1"  /* Repair Right Middle Lung Lobe, Via Natural or Artificial Opening */
"0BQD8ZZ"  =  "1"  /* Repair Right Middle Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BQF0ZZ"  =  "1"  /* Repair Right Lower Lung Lobe, Open Approach */
"0BQF3ZZ"  =  "1"  /* Repair Right Lower Lung Lobe, Percutaneous Approach */
"0BQF4ZZ"  =  "1"  /* Repair Right Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0BQF7ZZ"  =  "1"  /* Repair Right Lower Lung Lobe, Via Natural or Artificial Opening */
"0BQF8ZZ"  =  "1"  /* Repair Right Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BQG0ZZ"  =  "1"  /* Repair Left Upper Lung Lobe, Open Approach */
"0BQG3ZZ"  =  "1"  /* Repair Left Upper Lung Lobe, Percutaneous Approach */
"0BQG4ZZ"  =  "1"  /* Repair Left Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0BQG7ZZ"  =  "1"  /* Repair Left Upper Lung Lobe, Via Natural or Artificial Opening */
"0BQG8ZZ"  =  "1"  /* Repair Left Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BQH0ZZ"  =  "1"  /* Repair Lung Lingula, Open Approach */
"0BQH3ZZ"  =  "1"  /* Repair Lung Lingula, Percutaneous Approach */
"0BQH4ZZ"  =  "1"  /* Repair Lung Lingula, Percutaneous Endoscopic Approach */
"0BQH7ZZ"  =  "1"  /* Repair Lung Lingula, Via Natural or Artificial Opening */
"0BQH8ZZ"  =  "1"  /* Repair Lung Lingula, Via Natural or Artificial Opening Endoscopic */
"0BQJ0ZZ"  =  "1"  /* Repair Left Lower Lung Lobe, Open Approach */
"0BQJ3ZZ"  =  "1"  /* Repair Left Lower Lung Lobe, Percutaneous Approach */
"0BQJ4ZZ"  =  "1"  /* Repair Left Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0BQJ7ZZ"  =  "1"  /* Repair Left Lower Lung Lobe, Via Natural or Artificial Opening */
"0BQJ8ZZ"  =  "1"  /* Repair Left Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BQK0ZZ"  =  "1"  /* Repair Right Lung, Open Approach */
"0BQK3ZZ"  =  "1"  /* Repair Right Lung, Percutaneous Approach */
"0BQK4ZZ"  =  "1"  /* Repair Right Lung, Percutaneous Endoscopic Approach */
"0BQK7ZZ"  =  "1"  /* Repair Right Lung, Via Natural or Artificial Opening */
"0BQK8ZZ"  =  "1"  /* Repair Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BQL0ZZ"  =  "1"  /* Repair Left Lung, Open Approach */
"0BQL3ZZ"  =  "1"  /* Repair Left Lung, Percutaneous Approach */
"0BQL4ZZ"  =  "1"  /* Repair Left Lung, Percutaneous Endoscopic Approach */
"0BQL7ZZ"  =  "1"  /* Repair Left Lung, Via Natural or Artificial Opening */
"0BQL8ZZ"  =  "1"  /* Repair Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BQM0ZZ"  =  "1"  /* Repair Bilateral Lungs, Open Approach */
"0BQM3ZZ"  =  "1"  /* Repair Bilateral Lungs, Percutaneous Approach */
"0BQM4ZZ"  =  "1"  /* Repair Bilateral Lungs, Percutaneous Endoscopic Approach */
"0BQM7ZZ"  =  "1"  /* Repair Bilateral Lungs, Via Natural or Artificial Opening */
"0BQM8ZZ"  =  "1"  /* Repair Bilateral Lungs, Via Natural or Artificial Opening Endoscopic */
"0BQN0ZZ"  =  "1"  /* Repair Right Pleura, Open Approach */
"0BQN3ZZ"  =  "1"  /* Repair Right Pleura, Percutaneous Approach */
"0BQN4ZZ"  =  "1"  /* Repair Right Pleura, Percutaneous Endoscopic Approach */
"0BQP0ZZ"  =  "1"  /* Repair Left Pleura, Open Approach */
"0BQP3ZZ"  =  "1"  /* Repair Left Pleura, Percutaneous Approach */
"0BQP4ZZ"  =  "1"  /* Repair Left Pleura, Percutaneous Endoscopic Approach */
"0BQR0ZZ"  =  "1"  /* Repair Right Diaphragm, Open Approach */
"0BQR3ZZ"  =  "1"  /* Repair Right Diaphragm, Percutaneous Approach */
"0BQR4ZZ"  =  "1"  /* Repair Right Diaphragm, Percutaneous Endoscopic Approach */
"0BQS0ZZ"  =  "1"  /* Repair Left Diaphragm, Open Approach */
"0BQS3ZZ"  =  "1"  /* Repair Left Diaphragm, Percutaneous Approach */
"0BQS4ZZ"  =  "1"  /* Repair Left Diaphragm, Percutaneous Endoscopic Approach */
"0BS10ZZ"  =  "1"  /* Reposition Trachea, Open Approach */
"0BS20ZZ"  =  "1"  /* Reposition Carina, Open Approach */
"0BS30ZZ"  =  "1"  /* Reposition Right Main Bronchus, Open Approach */
"0BS40ZZ"  =  "1"  /* Reposition Right Upper Lobe Bronchus, Open Approach */
"0BS50ZZ"  =  "1"  /* Reposition Right Middle Lobe Bronchus, Open Approach */
"0BS60ZZ"  =  "1"  /* Reposition Right Lower Lobe Bronchus, Open Approach */
"0BS70ZZ"  =  "1"  /* Reposition Left Main Bronchus, Open Approach */
"0BS80ZZ"  =  "1"  /* Reposition Left Upper Lobe Bronchus, Open Approach */
"0BS90ZZ"  =  "1"  /* Reposition Lingula Bronchus, Open Approach */
"0BSB0ZZ"  =  "1"  /* Reposition Left Lower Lobe Bronchus, Open Approach */
"0BSC0ZZ"  =  "1"  /* Reposition Right Upper Lung Lobe, Open Approach */
"0BSD0ZZ"  =  "1"  /* Reposition Right Middle Lung Lobe, Open Approach */
"0BSF0ZZ"  =  "1"  /* Reposition Right Lower Lung Lobe, Open Approach */
"0BSG0ZZ"  =  "1"  /* Reposition Left Upper Lung Lobe, Open Approach */
"0BSH0ZZ"  =  "1"  /* Reposition Lung Lingula, Open Approach */
"0BSJ0ZZ"  =  "1"  /* Reposition Left Lower Lung Lobe, Open Approach */
"0BSK0ZZ"  =  "1"  /* Reposition Right Lung, Open Approach */
"0BSL0ZZ"  =  "1"  /* Reposition Left Lung, Open Approach */
"0BSR0ZZ"  =  "1"  /* Reposition Right Diaphragm, Open Approach */
"0BSS0ZZ"  =  "1"  /* Reposition Left Diaphragm, Open Approach */
"0BT10ZZ"  =  "1"  /* Resection of Trachea, Open Approach */
"0BT14ZZ"  =  "1"  /* Resection of Trachea, Percutaneous Endoscopic Approach */
"0BT20ZZ"  =  "1"  /* Resection of Carina, Open Approach */
"0BT24ZZ"  =  "1"  /* Resection of Carina, Percutaneous Endoscopic Approach */
"0BT30ZZ"  =  "1"  /* Resection of Right Main Bronchus, Open Approach */
"0BT34ZZ"  =  "1"  /* Resection of Right Main Bronchus, Percutaneous Endoscopic Approach */
"0BT40ZZ"  =  "1"  /* Resection of Right Upper Lobe Bronchus, Open Approach */
"0BT44ZZ"  =  "1"  /* Resection of Right Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BT50ZZ"  =  "1"  /* Resection of Right Middle Lobe Bronchus, Open Approach */
"0BT54ZZ"  =  "1"  /* Resection of Right Middle Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BT60ZZ"  =  "1"  /* Resection of Right Lower Lobe Bronchus, Open Approach */
"0BT64ZZ"  =  "1"  /* Resection of Right Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BT70ZZ"  =  "1"  /* Resection of Left Main Bronchus, Open Approach */
"0BT74ZZ"  =  "1"  /* Resection of Left Main Bronchus, Percutaneous Endoscopic Approach */
"0BT80ZZ"  =  "1"  /* Resection of Left Upper Lobe Bronchus, Open Approach */
"0BT84ZZ"  =  "1"  /* Resection of Left Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BT90ZZ"  =  "1"  /* Resection of Lingula Bronchus, Open Approach */
"0BT94ZZ"  =  "1"  /* Resection of Lingula Bronchus, Percutaneous Endoscopic Approach */
"0BTB0ZZ"  =  "1"  /* Resection of Left Lower Lobe Bronchus, Open Approach */
"0BTB4ZZ"  =  "1"  /* Resection of Left Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BTC0ZZ"  =  "1"  /* Resection of Right Upper Lung Lobe, Open Approach */
"0BTC4ZZ"  =  "1"  /* Resection of Right Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0BTD0ZZ"  =  "1"  /* Resection of Right Middle Lung Lobe, Open Approach */
"0BTD4ZZ"  =  "1"  /* Resection of Right Middle Lung Lobe, Percutaneous Endoscopic Approach */
"0BTF0ZZ"  =  "1"  /* Resection of Right Lower Lung Lobe, Open Approach */
"0BTF4ZZ"  =  "1"  /* Resection of Right Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0BTG0ZZ"  =  "1"  /* Resection of Left Upper Lung Lobe, Open Approach */
"0BTG4ZZ"  =  "1"  /* Resection of Left Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0BTH0ZZ"  =  "1"  /* Resection of Lung Lingula, Open Approach */
"0BTH4ZZ"  =  "1"  /* Resection of Lung Lingula, Percutaneous Endoscopic Approach */
"0BTJ0ZZ"  =  "1"  /* Resection of Left Lower Lung Lobe, Open Approach */
"0BTJ4ZZ"  =  "1"  /* Resection of Left Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0BTK0ZZ"  =  "1"  /* Resection of Right Lung, Open Approach */
"0BTK4ZZ"  =  "1"  /* Resection of Right Lung, Percutaneous Endoscopic Approach */
"0BTL0ZZ"  =  "1"  /* Resection of Left Lung, Open Approach */
"0BTL4ZZ"  =  "1"  /* Resection of Left Lung, Percutaneous Endoscopic Approach */
"0BTM0ZZ"  =  "1"  /* Resection of Bilateral Lungs, Open Approach */
"0BTM4ZZ"  =  "1"  /* Resection of Bilateral Lungs, Percutaneous Endoscopic Approach */
"0BTR0ZZ"  =  "1"  /* Resection of Right Diaphragm, Open Approach */
"0BTR4ZZ"  =  "1"  /* Resection of Right Diaphragm, Percutaneous Endoscopic Approach */
"0BTS0ZZ"  =  "1"  /* Resection of Left Diaphragm, Open Approach */
"0BTS4ZZ"  =  "1"  /* Resection of Left Diaphragm, Percutaneous Endoscopic Approach */
"0BU107Z"  =  "1"  /* Supplement Trachea with Autologous Tissue Substitute, Open Approach */
"0BU10JZ"  =  "1"  /* Supplement Trachea with Synthetic Substitute, Open Approach */
"0BU10KZ"  =  "1"  /* Supplement Trachea with Nonautologous Tissue Substitute, Open Approach */
"0BU147Z"  =  "1"  /* Supplement Trachea with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU14JZ"  =  "1"  /* Supplement Trachea with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BU14KZ"  =  "1"  /* Supplement Trachea with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU207Z"  =  "1"  /* Supplement Carina with Autologous Tissue Substitute, Open Approach */
"0BU20JZ"  =  "1"  /* Supplement Carina with Synthetic Substitute, Open Approach */
"0BU20KZ"  =  "1"  /* Supplement Carina with Nonautologous Tissue Substitute, Open Approach */
"0BU247Z"  =  "1"  /* Supplement Carina with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU24JZ"  =  "1"  /* Supplement Carina with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BU24KZ"  =  "1"  /* Supplement Carina with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU307Z"  =  "1"  /* Supplement Right Main Bronchus with Autologous Tissue Substitute, Open Approach */
"0BU30JZ"  =  "1"  /* Supplement Right Main Bronchus with Synthetic Substitute, Open Approach */
"0BU30KZ"  =  "1"  /* Supplement Right Main Bronchus with Nonautologous Tissue Substitute, Open Approach */
"0BU347Z"  =  "1"  /* Supplement Right Main Bronchus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU34JZ"  =  "1"  /* Supplement Right Main Bronchus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BU34KZ"  =  "1"  /* Supplement Right Main Bronchus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU407Z"  =  "1"  /* Supplement Right Upper Lobe Bronchus with Autologous Tissue Substitute, Open Approach */
"0BU40JZ"  =  "1"  /* Supplement Right Upper Lobe Bronchus with Synthetic Substitute, Open Approach */
"0BU40KZ"  =  "1"  /* Supplement Right Upper Lobe Bronchus with Nonautologous Tissue Substitute, Open Approach */
"0BU447Z"  =  "1"  /* Supplement Right Upper Lobe Bronchus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU44JZ"  =  "1"  /* Supplement Right Upper Lobe Bronchus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BU44KZ"  =  "1"  /* Supplement Right Upper Lobe Bronchus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU507Z"  =  "1"  /* Supplement Right Middle Lobe Bronchus with Autologous Tissue Substitute, Open Approach */
"0BU50JZ"  =  "1"  /* Supplement Right Middle Lobe Bronchus with Synthetic Substitute, Open Approach */
"0BU50KZ"  =  "1"  /* Supplement Right Middle Lobe Bronchus with Nonautologous Tissue Substitute, Open Approach */
"0BU547Z"  =  "1"  /* Supplement Right Middle Lobe Bronchus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU54JZ"  =  "1"  /* Supplement Right Middle Lobe Bronchus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BU54KZ"  =  "1"  /* Supplement Right Middle Lobe Bronchus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU607Z"  =  "1"  /* Supplement Right Lower Lobe Bronchus with Autologous Tissue Substitute, Open Approach */
"0BU60JZ"  =  "1"  /* Supplement Right Lower Lobe Bronchus with Synthetic Substitute, Open Approach */
"0BU60KZ"  =  "1"  /* Supplement Right Lower Lobe Bronchus with Nonautologous Tissue Substitute, Open Approach */
"0BU647Z"  =  "1"  /* Supplement Right Lower Lobe Bronchus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU64JZ"  =  "1"  /* Supplement Right Lower Lobe Bronchus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BU64KZ"  =  "1"  /* Supplement Right Lower Lobe Bronchus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU707Z"  =  "1"  /* Supplement Left Main Bronchus with Autologous Tissue Substitute, Open Approach */
"0BU70JZ"  =  "1"  /* Supplement Left Main Bronchus with Synthetic Substitute, Open Approach */
"0BU70KZ"  =  "1"  /* Supplement Left Main Bronchus with Nonautologous Tissue Substitute, Open Approach */
"0BU747Z"  =  "1"  /* Supplement Left Main Bronchus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU74JZ"  =  "1"  /* Supplement Left Main Bronchus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BU74KZ"  =  "1"  /* Supplement Left Main Bronchus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU807Z"  =  "1"  /* Supplement Left Upper Lobe Bronchus with Autologous Tissue Substitute, Open Approach */
"0BU80JZ"  =  "1"  /* Supplement Left Upper Lobe Bronchus with Synthetic Substitute, Open Approach */
"0BU80KZ"  =  "1"  /* Supplement Left Upper Lobe Bronchus with Nonautologous Tissue Substitute, Open Approach */
"0BU847Z"  =  "1"  /* Supplement Left Upper Lobe Bronchus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU84JZ"  =  "1"  /* Supplement Left Upper Lobe Bronchus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BU84KZ"  =  "1"  /* Supplement Left Upper Lobe Bronchus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU907Z"  =  "1"  /* Supplement Lingula Bronchus with Autologous Tissue Substitute, Open Approach */
"0BU90JZ"  =  "1"  /* Supplement Lingula Bronchus with Synthetic Substitute, Open Approach */
"0BU90KZ"  =  "1"  /* Supplement Lingula Bronchus with Nonautologous Tissue Substitute, Open Approach */
"0BU947Z"  =  "1"  /* Supplement Lingula Bronchus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU94JZ"  =  "1"  /* Supplement Lingula Bronchus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BU94KZ"  =  "1"  /* Supplement Lingula Bronchus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BUB07Z"  =  "1"  /* Supplement Left Lower Lobe Bronchus with Autologous Tissue Substitute, Open Approach */
"0BUB0JZ"  =  "1"  /* Supplement Left Lower Lobe Bronchus with Synthetic Substitute, Open Approach */
"0BUB0KZ"  =  "1"  /* Supplement Left Lower Lobe Bronchus with Nonautologous Tissue Substitute, Open Approach */
"0BUB47Z"  =  "1"  /* Supplement Left Lower Lobe Bronchus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BUB4JZ"  =  "1"  /* Supplement Left Lower Lobe Bronchus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BUB4KZ"  =  "1"  /* Supplement Left Lower Lobe Bronchus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BV10CZ"  =  "1"  /* Restriction of Trachea with Extraluminal Device, Open Approach */
"0BV10DZ"  =  "1"  /* Restriction of Trachea with Intraluminal Device, Open Approach */
"0BV10ZZ"  =  "1"  /* Restriction of Trachea, Open Approach */
"0BV13CZ"  =  "1"  /* Restriction of Trachea with Extraluminal Device, Percutaneous Approach */
"0BV13DZ"  =  "1"  /* Restriction of Trachea with Intraluminal Device, Percutaneous Approach */
"0BV13ZZ"  =  "1"  /* Restriction of Trachea, Percutaneous Approach */
"0BV14CZ"  =  "1"  /* Restriction of Trachea with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BV14DZ"  =  "1"  /* Restriction of Trachea with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BV14ZZ"  =  "1"  /* Restriction of Trachea, Percutaneous Endoscopic Approach */
"0BV17DZ"  =  "1"  /* Restriction of Trachea with Intraluminal Device, Via Natural or Artificial Opening */
"0BV17ZZ"  =  "1"  /* Restriction of Trachea, Via Natural or Artificial Opening */
"0BV18DZ"  =  "1"  /* Restriction of Trachea with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BV18ZZ"  =  "1"  /* Restriction of Trachea, Via Natural or Artificial Opening Endoscopic */
"0BV20CZ"  =  "1"  /* Restriction of Carina with Extraluminal Device, Open Approach */
"0BV20DZ"  =  "1"  /* Restriction of Carina with Intraluminal Device, Open Approach */
"0BV20ZZ"  =  "1"  /* Restriction of Carina, Open Approach */
"0BV23CZ"  =  "1"  /* Restriction of Carina with Extraluminal Device, Percutaneous Approach */
"0BV23DZ"  =  "1"  /* Restriction of Carina with Intraluminal Device, Percutaneous Approach */
"0BV23ZZ"  =  "1"  /* Restriction of Carina, Percutaneous Approach */
"0BV24CZ"  =  "1"  /* Restriction of Carina with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BV24DZ"  =  "1"  /* Restriction of Carina with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BV24ZZ"  =  "1"  /* Restriction of Carina, Percutaneous Endoscopic Approach */
"0BV27DZ"  =  "1"  /* Restriction of Carina with Intraluminal Device, Via Natural or Artificial Opening */
"0BV27ZZ"  =  "1"  /* Restriction of Carina, Via Natural or Artificial Opening */
"0BV28DZ"  =  "1"  /* Restriction of Carina with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BV28ZZ"  =  "1"  /* Restriction of Carina, Via Natural or Artificial Opening Endoscopic */
"0BV30CZ"  =  "1"  /* Restriction of Right Main Bronchus with Extraluminal Device, Open Approach */
"0BV30DZ"  =  "1"  /* Restriction of Right Main Bronchus with Intraluminal Device, Open Approach */
"0BV30ZZ"  =  "1"  /* Restriction of Right Main Bronchus, Open Approach */
"0BV33CZ"  =  "1"  /* Restriction of Right Main Bronchus with Extraluminal Device, Percutaneous Approach */
"0BV33DZ"  =  "1"  /* Restriction of Right Main Bronchus with Intraluminal Device, Percutaneous Approach */
"0BV33ZZ"  =  "1"  /* Restriction of Right Main Bronchus, Percutaneous Approach */
"0BV34CZ"  =  "1"  /* Restriction of Right Main Bronchus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BV34DZ"  =  "1"  /* Restriction of Right Main Bronchus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BV34ZZ"  =  "1"  /* Restriction of Right Main Bronchus, Percutaneous Endoscopic Approach */
"0BV37DZ"  =  "1"  /* Restriction of Right Main Bronchus with Intraluminal Device, Via Natural or Artificial Opening */
"0BV37ZZ"  =  "1"  /* Restriction of Right Main Bronchus, Via Natural or Artificial Opening */
"0BV38DZ"  =  "1"  /* Restriction of Right Main Bronchus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BV38ZZ"  =  "1"  /* Restriction of Right Main Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BV40CZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus with Extraluminal Device, Open Approach */
"0BV40DZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus with Intraluminal Device, Open Approach */
"0BV40ZZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus, Open Approach */
"0BV43CZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus with Extraluminal Device, Percutaneous Approach */
"0BV43DZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus with Intraluminal Device, Percutaneous Approach */
"0BV43ZZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus, Percutaneous Approach */
"0BV44CZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BV44DZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BV44ZZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BV47DZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening */
"0BV47ZZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0BV48DZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BV48ZZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BV50CZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus with Extraluminal Device, Open Approach */
"0BV50DZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus with Intraluminal Device, Open Approach */
"0BV50ZZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus, Open Approach */
"0BV53CZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus with Extraluminal Device, Percutaneous Approach */
"0BV53DZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus with Intraluminal Device, Percutaneous Approach */
"0BV53ZZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus, Percutaneous Approach */
"0BV54CZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BV54DZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BV54ZZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BV57DZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening */
"0BV57ZZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus, Via Natural or Artificial Opening */
"0BV58DZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BV58ZZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BV60CZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus with Extraluminal Device, Open Approach */
"0BV60DZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus with Intraluminal Device, Open Approach */
"0BV60ZZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus, Open Approach */
"0BV63CZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus with Extraluminal Device, Percutaneous Approach */
"0BV63DZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus with Intraluminal Device, Percutaneous Approach */
"0BV63ZZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus, Percutaneous Approach */
"0BV64CZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BV64DZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BV64ZZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BV67DZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening */
"0BV67ZZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0BV68DZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BV68ZZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BV70CZ"  =  "1"  /* Restriction of Left Main Bronchus with Extraluminal Device, Open Approach */
"0BV70DZ"  =  "1"  /* Restriction of Left Main Bronchus with Intraluminal Device, Open Approach */
"0BV70ZZ"  =  "1"  /* Restriction of Left Main Bronchus, Open Approach */
"0BV73CZ"  =  "1"  /* Restriction of Left Main Bronchus with Extraluminal Device, Percutaneous Approach */
"0BV73DZ"  =  "1"  /* Restriction of Left Main Bronchus with Intraluminal Device, Percutaneous Approach */
"0BV73ZZ"  =  "1"  /* Restriction of Left Main Bronchus, Percutaneous Approach */
"0BV74CZ"  =  "1"  /* Restriction of Left Main Bronchus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BV74DZ"  =  "1"  /* Restriction of Left Main Bronchus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BV74ZZ"  =  "1"  /* Restriction of Left Main Bronchus, Percutaneous Endoscopic Approach */
"0BV77DZ"  =  "1"  /* Restriction of Left Main Bronchus with Intraluminal Device, Via Natural or Artificial Opening */
"0BV77ZZ"  =  "1"  /* Restriction of Left Main Bronchus, Via Natural or Artificial Opening */
"0BV78DZ"  =  "1"  /* Restriction of Left Main Bronchus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BV78ZZ"  =  "1"  /* Restriction of Left Main Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BV80CZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus with Extraluminal Device, Open Approach */
"0BV80DZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus with Intraluminal Device, Open Approach */
"0BV80ZZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus, Open Approach */
"0BV83CZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus with Extraluminal Device, Percutaneous Approach */
"0BV83DZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus with Intraluminal Device, Percutaneous Approach */
"0BV83ZZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus, Percutaneous Approach */
"0BV84CZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BV84DZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BV84ZZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BV87DZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening */
"0BV87ZZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0BV88DZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BV88ZZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BV90CZ"  =  "1"  /* Restriction of Lingula Bronchus with Extraluminal Device, Open Approach */
"0BV90DZ"  =  "1"  /* Restriction of Lingula Bronchus with Intraluminal Device, Open Approach */
"0BV90ZZ"  =  "1"  /* Restriction of Lingula Bronchus, Open Approach */
"0BV93CZ"  =  "1"  /* Restriction of Lingula Bronchus with Extraluminal Device, Percutaneous Approach */
"0BV93DZ"  =  "1"  /* Restriction of Lingula Bronchus with Intraluminal Device, Percutaneous Approach */
"0BV93ZZ"  =  "1"  /* Restriction of Lingula Bronchus, Percutaneous Approach */
"0BV94CZ"  =  "1"  /* Restriction of Lingula Bronchus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BV94DZ"  =  "1"  /* Restriction of Lingula Bronchus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BV94ZZ"  =  "1"  /* Restriction of Lingula Bronchus, Percutaneous Endoscopic Approach */
"0BV97DZ"  =  "1"  /* Restriction of Lingula Bronchus with Intraluminal Device, Via Natural or Artificial Opening */
"0BV97ZZ"  =  "1"  /* Restriction of Lingula Bronchus, Via Natural or Artificial Opening */
"0BV98DZ"  =  "1"  /* Restriction of Lingula Bronchus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BV98ZZ"  =  "1"  /* Restriction of Lingula Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BVB0CZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus with Extraluminal Device, Open Approach */
"0BVB0DZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus with Intraluminal Device, Open Approach */
"0BVB0ZZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus, Open Approach */
"0BVB3CZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus with Extraluminal Device, Percutaneous Approach */
"0BVB3DZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus with Intraluminal Device, Percutaneous Approach */
"0BVB3ZZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus, Percutaneous Approach */
"0BVB4CZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BVB4DZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BVB4ZZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BVB7DZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening */
"0BVB7ZZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0BVB8DZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BVB8ZZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0BW000Z"  =  "1"  /* Revision of Drainage Device in Tracheobronchial Tree, Open Approach */
"0BW002Z"  =  "1"  /* Revision of Monitoring Device in Tracheobronchial Tree, Open Approach */
"0BW00CZ"  =  "1"  /* Revision of Extraluminal Device in Tracheobronchial Tree, Open Approach */
"0BW00DZ"  =  "1"  /* Revision of Intraluminal Device in Tracheobronchial Tree, Open Approach */
"0BW00JZ"  =  "1"  /* Revision of Synthetic Substitute in Tracheobronchial Tree, Open Approach */
"0BW00KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Tracheobronchial Tree, Open Approach */
"0BW030Z"  =  "1"  /* Revision of Drainage Device in Tracheobronchial Tree, Percutaneous Approach */
"0BW032Z"  =  "1"  /* Revision of Monitoring Device in Tracheobronchial Tree, Percutaneous Approach */
"0BW03CZ"  =  "1"  /* Revision of Extraluminal Device in Tracheobronchial Tree, Percutaneous Approach */
"0BW03DZ"  =  "1"  /* Revision of Intraluminal Device in Tracheobronchial Tree, Percutaneous Approach */
"0BW03JZ"  =  "1"  /* Revision of Synthetic Substitute in Tracheobronchial Tree, Percutaneous Approach */
"0BW03KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Tracheobronchial Tree, Percutaneous Approach */
"0BW040Z"  =  "1"  /* Revision of Drainage Device in Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BW042Z"  =  "1"  /* Revision of Monitoring Device in Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BW04CZ"  =  "1"  /* Revision of Extraluminal Device in Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BW04DZ"  =  "1"  /* Revision of Intraluminal Device in Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BW04JZ"  =  "1"  /* Revision of Synthetic Substitute in Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BW04KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BW070Z"  =  "1"  /* Revision of Drainage Device in Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BW072Z"  =  "1"  /* Revision of Monitoring Device in Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BW07DZ"  =  "1"  /* Revision of Intraluminal Device in Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BW080Z"  =  "1"  /* Revision of Drainage Device in Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BW082Z"  =  "1"  /* Revision of Monitoring Device in Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BW08DZ"  =  "1"  /* Revision of Intraluminal Device in Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BWK00Z"  =  "1"  /* Revision of Drainage Device in Right Lung, Open Approach */
"0BWK02Z"  =  "1"  /* Revision of Monitoring Device in Right Lung, Open Approach */
"0BWK03Z"  =  "1"  /* Revision of Infusion Device in Right Lung, Open Approach */
"0BWK30Z"  =  "1"  /* Revision of Drainage Device in Right Lung, Percutaneous Approach */
"0BWK32Z"  =  "1"  /* Revision of Monitoring Device in Right Lung, Percutaneous Approach */
"0BWK33Z"  =  "1"  /* Revision of Infusion Device in Right Lung, Percutaneous Approach */
"0BWK40Z"  =  "1"  /* Revision of Drainage Device in Right Lung, Percutaneous Endoscopic Approach */
"0BWK42Z"  =  "1"  /* Revision of Monitoring Device in Right Lung, Percutaneous Endoscopic Approach */
"0BWK43Z"  =  "1"  /* Revision of Infusion Device in Right Lung, Percutaneous Endoscopic Approach */
"0BWK70Z"  =  "1"  /* Revision of Drainage Device in Right Lung, Via Natural or Artificial Opening */
"0BWK72Z"  =  "1"  /* Revision of Monitoring Device in Right Lung, Via Natural or Artificial Opening */
"0BWK73Z"  =  "1"  /* Revision of Infusion Device in Right Lung, Via Natural or Artificial Opening */
"0BWK80Z"  =  "1"  /* Revision of Drainage Device in Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BWK82Z"  =  "1"  /* Revision of Monitoring Device in Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BWK83Z"  =  "1"  /* Revision of Infusion Device in Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BWL00Z"  =  "1"  /* Revision of Drainage Device in Left Lung, Open Approach */
"0BWL02Z"  =  "1"  /* Revision of Monitoring Device in Left Lung, Open Approach */
"0BWL03Z"  =  "1"  /* Revision of Infusion Device in Left Lung, Open Approach */
"0BWL30Z"  =  "1"  /* Revision of Drainage Device in Left Lung, Percutaneous Approach */
"0BWL32Z"  =  "1"  /* Revision of Monitoring Device in Left Lung, Percutaneous Approach */
"0BWL33Z"  =  "1"  /* Revision of Infusion Device in Left Lung, Percutaneous Approach */
"0BWL40Z"  =  "1"  /* Revision of Drainage Device in Left Lung, Percutaneous Endoscopic Approach */
"0BWL42Z"  =  "1"  /* Revision of Monitoring Device in Left Lung, Percutaneous Endoscopic Approach */
"0BWL43Z"  =  "1"  /* Revision of Infusion Device in Left Lung, Percutaneous Endoscopic Approach */
"0BWL70Z"  =  "1"  /* Revision of Drainage Device in Left Lung, Via Natural or Artificial Opening */
"0BWL72Z"  =  "1"  /* Revision of Monitoring Device in Left Lung, Via Natural or Artificial Opening */
"0BWL73Z"  =  "1"  /* Revision of Infusion Device in Left Lung, Via Natural or Artificial Opening */
"0BWL80Z"  =  "1"  /* Revision of Drainage Device in Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BWL82Z"  =  "1"  /* Revision of Monitoring Device in Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BWL83Z"  =  "1"  /* Revision of Infusion Device in Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BWQ00Z"  =  "1"  /* Revision of Drainage Device in Pleura, Open Approach */
"0BWQ02Z"  =  "1"  /* Revision of Monitoring Device in Pleura, Open Approach */
"0BWQ30Z"  =  "1"  /* Revision of Drainage Device in Pleura, Percutaneous Approach */
"0BWQ32Z"  =  "1"  /* Revision of Monitoring Device in Pleura, Percutaneous Approach */
"0BWQ40Z"  =  "1"  /* Revision of Drainage Device in Pleura, Percutaneous Endoscopic Approach */
"0BWQ42Z"  =  "1"  /* Revision of Monitoring Device in Pleura, Percutaneous Endoscopic Approach */
"0BWQ70Z"  =  "1"  /* Revision of Drainage Device in Pleura, Via Natural or Artificial Opening */
"0BWQ72Z"  =  "1"  /* Revision of Monitoring Device in Pleura, Via Natural or Artificial Opening */
"0BWQ80Z"  =  "1"  /* Revision of Drainage Device in Pleura, Via Natural or Artificial Opening Endoscopic */
"0BWQ82Z"  =  "1"  /* Revision of Monitoring Device in Pleura, Via Natural or Artificial Opening Endoscopic */
"0BWT00Z"  =  "1"  /* Revision of Drainage Device in Diaphragm, Open Approach */
"0BWT02Z"  =  "1"  /* Revision of Monitoring Device in Diaphragm, Open Approach */
"0BWT07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Diaphragm, Open Approach */
"0BWT0JZ"  =  "1"  /* Revision of Synthetic Substitute in Diaphragm, Open Approach */
"0BWT0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Diaphragm, Open Approach */
"0BWT0MZ"  =  "1"  /* Revision of Diaphragmatic Pacemaker Lead in Diaphragm, Open Approach */
"0BWT30Z"  =  "1"  /* Revision of Drainage Device in Diaphragm, Percutaneous Approach */
"0BWT32Z"  =  "1"  /* Revision of Monitoring Device in Diaphragm, Percutaneous Approach */
"0BWT37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Diaphragm, Percutaneous Approach */
"0BWT3JZ"  =  "1"  /* Revision of Synthetic Substitute in Diaphragm, Percutaneous Approach */
"0BWT3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Diaphragm, Percutaneous Approach */
"0BWT3MZ"  =  "1"  /* Revision of Diaphragmatic Pacemaker Lead in Diaphragm, Percutaneous Approach */
"0BWT40Z"  =  "1"  /* Revision of Drainage Device in Diaphragm, Percutaneous Endoscopic Approach */
"0BWT42Z"  =  "1"  /* Revision of Monitoring Device in Diaphragm, Percutaneous Endoscopic Approach */
"0BWT47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Diaphragm, Percutaneous Endoscopic Approach */
"0BWT4JZ"  =  "1"  /* Revision of Synthetic Substitute in Diaphragm, Percutaneous Endoscopic Approach */
"0BWT4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Diaphragm, Percutaneous Endoscopic Approach */
"0BWT4MZ"  =  "1"  /* Revision of Diaphragmatic Pacemaker Lead in Diaphragm, Percutaneous Endoscopic Approach */
"0BWT70Z"  =  "1"  /* Revision of Drainage Device in Diaphragm, Via Natural or Artificial Opening */
"0BWT72Z"  =  "1"  /* Revision of Monitoring Device in Diaphragm, Via Natural or Artificial Opening */
"0BWT77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Diaphragm, Via Natural or Artificial Opening */
"0BWT7JZ"  =  "1"  /* Revision of Synthetic Substitute in Diaphragm, Via Natural or Artificial Opening */
"0BWT7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Diaphragm, Via Natural or Artificial Opening */
"0BWT7MZ"  =  "1"  /* Revision of Diaphragmatic Pacemaker Lead in Diaphragm, Via Natural or Artificial Opening */
"0BWT80Z"  =  "1"  /* Revision of Drainage Device in Diaphragm, Via Natural or Artificial Opening Endoscopic */
"0BWT82Z"  =  "1"  /* Revision of Monitoring Device in Diaphragm, Via Natural or Artificial Opening Endoscopic */
"0BWT87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Diaphragm, Via Natural or Artificial Opening Endoscopic */
"0BWT8JZ"  =  "1"  /* Revision of Synthetic Substitute in Diaphragm, Via Natural or Artificial Opening Endoscopic */
"0BWT8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Diaphragm, Via Natural or Artificial Opening Endoscopic */
"0BWT8MZ"  =  "1"  /* Revision of Diaphragmatic Pacemaker Lead in Diaphragm, Via Natural or Artificial Opening Endoscopic */
"0BYC0Z0"  =  "1"  /* Transplantation of Right Upper Lung Lobe, Allogeneic, Open Approach */
"0BYC0Z1"  =  "1"  /* Transplantation of Right Upper Lung Lobe, Syngeneic, Open Approach */
"0BYC0Z2"  =  "1"  /* Transplantation of Right Upper Lung Lobe, Zooplastic, Open Approach */
"0BYD0Z0"  =  "1"  /* Transplantation of Right Middle Lung Lobe, Allogeneic, Open Approach */
"0BYD0Z1"  =  "1"  /* Transplantation of Right Middle Lung Lobe, Syngeneic, Open Approach */
"0BYD0Z2"  =  "1"  /* Transplantation of Right Middle Lung Lobe, Zooplastic, Open Approach */
"0BYF0Z0"  =  "1"  /* Transplantation of Right Lower Lung Lobe, Allogeneic, Open Approach */
"0BYF0Z1"  =  "1"  /* Transplantation of Right Lower Lung Lobe, Syngeneic, Open Approach */
"0BYF0Z2"  =  "1"  /* Transplantation of Right Lower Lung Lobe, Zooplastic, Open Approach */
"0BYG0Z0"  =  "1"  /* Transplantation of Left Upper Lung Lobe, Allogeneic, Open Approach */
"0BYG0Z1"  =  "1"  /* Transplantation of Left Upper Lung Lobe, Syngeneic, Open Approach */
"0BYG0Z2"  =  "1"  /* Transplantation of Left Upper Lung Lobe, Zooplastic, Open Approach */
"0BYH0Z0"  =  "1"  /* Transplantation of Lung Lingula, Allogeneic, Open Approach */
"0BYH0Z1"  =  "1"  /* Transplantation of Lung Lingula, Syngeneic, Open Approach */
"0BYH0Z2"  =  "1"  /* Transplantation of Lung Lingula, Zooplastic, Open Approach */
"0BYJ0Z0"  =  "1"  /* Transplantation of Left Lower Lung Lobe, Allogeneic, Open Approach */
"0BYJ0Z1"  =  "1"  /* Transplantation of Left Lower Lung Lobe, Syngeneic, Open Approach */
"0BYJ0Z2"  =  "1"  /* Transplantation of Left Lower Lung Lobe, Zooplastic, Open Approach */
"0BYK0Z0"  =  "1"  /* Transplantation of Right Lung, Allogeneic, Open Approach */
"0BYK0Z1"  =  "1"  /* Transplantation of Right Lung, Syngeneic, Open Approach */
"0BYK0Z2"  =  "1"  /* Transplantation of Right Lung, Zooplastic, Open Approach */
"0BYL0Z0"  =  "1"  /* Transplantation of Left Lung, Allogeneic, Open Approach */
"0BYL0Z1"  =  "1"  /* Transplantation of Left Lung, Syngeneic, Open Approach */
"0BYL0Z2"  =  "1"  /* Transplantation of Left Lung, Zooplastic, Open Approach */
"0BYM0Z0"  =  "1"  /* Transplantation of Bilateral Lungs, Allogeneic, Open Approach */
"0BYM0Z1"  =  "1"  /* Transplantation of Bilateral Lungs, Syngeneic, Open Approach */
"0BYM0Z2"  =  "1"  /* Transplantation of Bilateral Lungs, Zooplastic, Open Approach */
"0C9R0ZX"  =  "1"  /* Drainage of Epiglottis, Open Approach, Diagnostic */
"0C9S0ZX"  =  "1"  /* Drainage of Larynx, Open Approach, Diagnostic */
"0C9T0ZX"  =  "1"  /* Drainage of Right Vocal Cord, Open Approach, Diagnostic */
"0C9V0ZX"  =  "1"  /* Drainage of Left Vocal Cord, Open Approach, Diagnostic */
"0CBR0ZX"  =  "1"  /* Excision of Epiglottis, Open Approach, Diagnostic */
"0CBS0ZX"  =  "1"  /* Excision of Larynx, Open Approach, Diagnostic */
"0CBT0ZX"  =  "1"  /* Excision of Right Vocal Cord, Open Approach, Diagnostic */
"0CBV0ZX"  =  "1"  /* Excision of Left Vocal Cord, Open Approach, Diagnostic */
"0D11074"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D11076"  =  "1"  /* Bypass Upper Esophagus to Stomach with Autologous Tissue Substitute, Open Approach */
"0D11079"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1107A"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1107B"  =  "1"  /* Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D110J4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Synthetic Substitute, Open Approach */
"0D110J6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Synthetic Substitute, Open Approach */
"0D110J9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D110JA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D110JB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D110K4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D110K6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach */
"0D110K9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D110KA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D110KB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D110Z4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous, Open Approach */
"0D110Z6"  =  "1"  /* Bypass Upper Esophagus to Stomach, Open Approach */
"0D110Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Open Approach */
"0D110ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Open Approach */
"0D110ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Open Approach */
"0D113J4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D11474"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D11476"  =  "1"  /* Bypass Upper Esophagus to Stomach with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D11479"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1147A"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1147B"  =  "1"  /* Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114J4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114J6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114J9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114JA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114JB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114K4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114K6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114K9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114KA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114KB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114Z4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous, Percutaneous Endoscopic Approach */
"0D114Z6"  =  "1"  /* Bypass Upper Esophagus to Stomach, Percutaneous Endoscopic Approach */
"0D114Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D114ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D114ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D11874"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D11876"  =  "1"  /* Bypass Upper Esophagus to Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D11879"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1187A"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1187B"  =  "1"  /* Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118J4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118J6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118J9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118JA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118JB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118K4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118K6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118K9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118KA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118KB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118Z4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D118Z6"  =  "1"  /* Bypass Upper Esophagus to Stomach, Via Natural or Artificial Opening Endoscopic */
"0D118Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D118ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D118ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D12074"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D12076"  =  "1"  /* Bypass Middle Esophagus to Stomach with Autologous Tissue Substitute, Open Approach */
"0D12079"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1207A"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1207B"  =  "1"  /* Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D120J4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Synthetic Substitute, Open Approach */
"0D120J6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Synthetic Substitute, Open Approach */
"0D120J9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D120JA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D120JB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D120K4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D120K6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach */
"0D120K9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D120KA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D120KB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D120Z4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous, Open Approach */
"0D120Z6"  =  "1"  /* Bypass Middle Esophagus to Stomach, Open Approach */
"0D120Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Open Approach */
"0D120ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Open Approach */
"0D120ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Open Approach */
"0D123J4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D12474"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D12476"  =  "1"  /* Bypass Middle Esophagus to Stomach with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D12479"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1247A"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1247B"  =  "1"  /* Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124J4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124J6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124J9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124JA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124JB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124K4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124K6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124K9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124KA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124KB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124Z4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous, Percutaneous Endoscopic Approach */
"0D124Z6"  =  "1"  /* Bypass Middle Esophagus to Stomach, Percutaneous Endoscopic Approach */
"0D124Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D124ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D124ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D12874"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D12876"  =  "1"  /* Bypass Middle Esophagus to Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D12879"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1287A"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1287B"  =  "1"  /* Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128J4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128J6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128J9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128JA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128JB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128K4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128K6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128K9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128KA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128KB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128Z4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D128Z6"  =  "1"  /* Bypass Middle Esophagus to Stomach, Via Natural or Artificial Opening Endoscopic */
"0D128Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D128ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D128ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D13074"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D13076"  =  "1"  /* Bypass Lower Esophagus to Stomach with Autologous Tissue Substitute, Open Approach */
"0D13079"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1307A"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1307B"  =  "1"  /* Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D130J4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Synthetic Substitute, Open Approach */
"0D130J6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Synthetic Substitute, Open Approach */
"0D130J9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D130JA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D130JB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D130K4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D130K6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach */
"0D130K9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D130KA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D130KB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D130Z4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous, Open Approach */
"0D130Z6"  =  "1"  /* Bypass Lower Esophagus to Stomach, Open Approach */
"0D130Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Open Approach */
"0D130ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Open Approach */
"0D130ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Open Approach */
"0D133J4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D13474"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D13476"  =  "1"  /* Bypass Lower Esophagus to Stomach with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D13479"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1347A"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1347B"  =  "1"  /* Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134J4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134J6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134J9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134JA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134JB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134K4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134K6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134K9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134KA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134KB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134Z4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous, Percutaneous Endoscopic Approach */
"0D134Z6"  =  "1"  /* Bypass Lower Esophagus to Stomach, Percutaneous Endoscopic Approach */
"0D134Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D134ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D134ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D13874"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D13876"  =  "1"  /* Bypass Lower Esophagus to Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D13879"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1387A"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1387B"  =  "1"  /* Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138J4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138J6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138J9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138JA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138JB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138K4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138K6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138K9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138KA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138KB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138Z4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D138Z6"  =  "1"  /* Bypass Lower Esophagus to Stomach, Via Natural or Artificial Opening Endoscopic */
"0D138Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D138ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D138ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D15074"  =  "1"  /* Bypass Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D15076"  =  "1"  /* Bypass Esophagus to Stomach with Autologous Tissue Substitute, Open Approach */
"0D15079"  =  "1"  /* Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1507A"  =  "1"  /* Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1507B"  =  "1"  /* Bypass Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D150J4"  =  "1"  /* Bypass Esophagus to Cutaneous with Synthetic Substitute, Open Approach */
"0D150J6"  =  "1"  /* Bypass Esophagus to Stomach with Synthetic Substitute, Open Approach */
"0D150J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D150JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D150JB"  =  "1"  /* Bypass Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D150K4"  =  "1"  /* Bypass Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D150K6"  =  "1"  /* Bypass Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach */
"0D150K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D150KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D150KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D150Z4"  =  "1"  /* Bypass Esophagus to Cutaneous, Open Approach */
"0D150Z6"  =  "1"  /* Bypass Esophagus to Stomach, Open Approach */
"0D150Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Open Approach */
"0D150ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Open Approach */
"0D150ZB"  =  "1"  /* Bypass Esophagus to Ileum, Open Approach */
"0D153J4"  =  "1"  /* Bypass Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D15474"  =  "1"  /* Bypass Esophagus to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D15476"  =  "1"  /* Bypass Esophagus to Stomach with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D15479"  =  "1"  /* Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1547A"  =  "1"  /* Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1547B"  =  "1"  /* Bypass Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154J4"  =  "1"  /* Bypass Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154J6"  =  "1"  /* Bypass Esophagus to Stomach with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154JB"  =  "1"  /* Bypass Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154K4"  =  "1"  /* Bypass Esophagus to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154K6"  =  "1"  /* Bypass Esophagus to Stomach with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154Z4"  =  "1"  /* Bypass Esophagus to Cutaneous, Percutaneous Endoscopic Approach */
"0D154Z6"  =  "1"  /* Bypass Esophagus to Stomach, Percutaneous Endoscopic Approach */
"0D154Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D154ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D154ZB"  =  "1"  /* Bypass Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D15874"  =  "1"  /* Bypass Esophagus to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D15876"  =  "1"  /* Bypass Esophagus to Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D15879"  =  "1"  /* Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1587A"  =  "1"  /* Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1587B"  =  "1"  /* Bypass Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158J4"  =  "1"  /* Bypass Esophagus to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158J6"  =  "1"  /* Bypass Esophagus to Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158JB"  =  "1"  /* Bypass Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158K4"  =  "1"  /* Bypass Esophagus to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158K6"  =  "1"  /* Bypass Esophagus to Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158Z4"  =  "1"  /* Bypass Esophagus to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D158Z6"  =  "1"  /* Bypass Esophagus to Stomach, Via Natural or Artificial Opening Endoscopic */
"0D158Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D158ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D158ZB"  =  "1"  /* Bypass Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D510ZZ"  =  "1"  /* Destruction of Upper Esophagus, Open Approach */
"0D513ZZ"  =  "1"  /* Destruction of Upper Esophagus, Percutaneous Approach */
"0D517ZZ"  =  "1"  /* Destruction of Upper Esophagus, Via Natural or Artificial Opening */
"0D520ZZ"  =  "1"  /* Destruction of Middle Esophagus, Open Approach */
"0D523ZZ"  =  "1"  /* Destruction of Middle Esophagus, Percutaneous Approach */
"0D527ZZ"  =  "1"  /* Destruction of Middle Esophagus, Via Natural or Artificial Opening */
"0D530ZZ"  =  "1"  /* Destruction of Lower Esophagus, Open Approach */
"0D533ZZ"  =  "1"  /* Destruction of Lower Esophagus, Percutaneous Approach */
"0D537ZZ"  =  "1"  /* Destruction of Lower Esophagus, Via Natural or Artificial Opening */
"0D540ZZ"  =  "1"  /* Destruction of Esophagogastric Junction, Open Approach */
"0D543ZZ"  =  "1"  /* Destruction of Esophagogastric Junction, Percutaneous Approach */
"0D547ZZ"  =  "1"  /* Destruction of Esophagogastric Junction, Via Natural or Artificial Opening */
"0D550ZZ"  =  "1"  /* Destruction of Esophagus, Open Approach */
"0D553ZZ"  =  "1"  /* Destruction of Esophagus, Percutaneous Approach */
"0D557ZZ"  =  "1"  /* Destruction of Esophagus, Via Natural or Artificial Opening */
"0D710DZ"  =  "1"  /* Dilation of Upper Esophagus with Intraluminal Device, Open Approach */
"0D710ZZ"  =  "1"  /* Dilation of Upper Esophagus, Open Approach */
"0D713DZ"  =  "1"  /* Dilation of Upper Esophagus with Intraluminal Device, Percutaneous Approach */
"0D713ZZ"  =  "1"  /* Dilation of Upper Esophagus, Percutaneous Approach */
"0D714DZ"  =  "1"  /* Dilation of Upper Esophagus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0D714ZZ"  =  "1"  /* Dilation of Upper Esophagus, Percutaneous Endoscopic Approach */
"0D720DZ"  =  "1"  /* Dilation of Middle Esophagus with Intraluminal Device, Open Approach */
"0D720ZZ"  =  "1"  /* Dilation of Middle Esophagus, Open Approach */
"0D723DZ"  =  "1"  /* Dilation of Middle Esophagus with Intraluminal Device, Percutaneous Approach */
"0D723ZZ"  =  "1"  /* Dilation of Middle Esophagus, Percutaneous Approach */
"0D724DZ"  =  "1"  /* Dilation of Middle Esophagus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0D724ZZ"  =  "1"  /* Dilation of Middle Esophagus, Percutaneous Endoscopic Approach */
"0D730DZ"  =  "1"  /* Dilation of Lower Esophagus with Intraluminal Device, Open Approach */
"0D730ZZ"  =  "1"  /* Dilation of Lower Esophagus, Open Approach */
"0D733DZ"  =  "1"  /* Dilation of Lower Esophagus with Intraluminal Device, Percutaneous Approach */
"0D733ZZ"  =  "1"  /* Dilation of Lower Esophagus, Percutaneous Approach */
"0D734DZ"  =  "1"  /* Dilation of Lower Esophagus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0D734ZZ"  =  "1"  /* Dilation of Lower Esophagus, Percutaneous Endoscopic Approach */
"0D740DZ"  =  "1"  /* Dilation of Esophagogastric Junction with Intraluminal Device, Open Approach */
"0D740ZZ"  =  "1"  /* Dilation of Esophagogastric Junction, Open Approach */
"0D743DZ"  =  "1"  /* Dilation of Esophagogastric Junction with Intraluminal Device, Percutaneous Approach */
"0D743ZZ"  =  "1"  /* Dilation of Esophagogastric Junction, Percutaneous Approach */
"0D744DZ"  =  "1"  /* Dilation of Esophagogastric Junction with Intraluminal Device, Percutaneous Endoscopic Approach */
"0D744ZZ"  =  "1"  /* Dilation of Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0D750DZ"  =  "1"  /* Dilation of Esophagus with Intraluminal Device, Open Approach */
"0D750ZZ"  =  "1"  /* Dilation of Esophagus, Open Approach */
"0D753DZ"  =  "1"  /* Dilation of Esophagus with Intraluminal Device, Percutaneous Approach */
"0D753ZZ"  =  "1"  /* Dilation of Esophagus, Percutaneous Approach */
"0D754DZ"  =  "1"  /* Dilation of Esophagus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0D754ZZ"  =  "1"  /* Dilation of Esophagus, Percutaneous Endoscopic Approach */
"0D840ZZ"  =  "1"  /* Division of Esophagogastric Junction, Open Approach */
"0D843ZZ"  =  "1"  /* Division of Esophagogastric Junction, Percutaneous Approach */
"0D844ZZ"  =  "1"  /* Division of Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0D847ZZ"  =  "1"  /* Division of Esophagogastric Junction, Via Natural or Artificial Opening */
"0D848ZZ"  =  "1"  /* Division of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic */
"0D9100Z"  =  "1"  /* Drainage of Upper Esophagus with Drainage Device, Open Approach */
"0D910ZX"  =  "1"  /* Drainage of Upper Esophagus, Open Approach, Diagnostic */
"0D910ZZ"  =  "1"  /* Drainage of Upper Esophagus, Open Approach */
"0D9130Z"  =  "1"  /* Drainage of Upper Esophagus with Drainage Device, Percutaneous Approach */
"0D913ZZ"  =  "1"  /* Drainage of Upper Esophagus, Percutaneous Approach */
"0D9140Z"  =  "1"  /* Drainage of Upper Esophagus with Drainage Device, Percutaneous Endoscopic Approach */
"0D914ZZ"  =  "1"  /* Drainage of Upper Esophagus, Percutaneous Endoscopic Approach */
"0D9170Z"  =  "1"  /* Drainage of Upper Esophagus with Drainage Device, Via Natural or Artificial Opening */
"0D917ZZ"  =  "1"  /* Drainage of Upper Esophagus, Via Natural or Artificial Opening */
"0D9180Z"  =  "1"  /* Drainage of Upper Esophagus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D918ZZ"  =  "1"  /* Drainage of Upper Esophagus, Via Natural or Artificial Opening Endoscopic */
"0D9200Z"  =  "1"  /* Drainage of Middle Esophagus with Drainage Device, Open Approach */
"0D920ZX"  =  "1"  /* Drainage of Middle Esophagus, Open Approach, Diagnostic */
"0D920ZZ"  =  "1"  /* Drainage of Middle Esophagus, Open Approach */
"0D9230Z"  =  "1"  /* Drainage of Middle Esophagus with Drainage Device, Percutaneous Approach */
"0D923ZZ"  =  "1"  /* Drainage of Middle Esophagus, Percutaneous Approach */
"0D9240Z"  =  "1"  /* Drainage of Middle Esophagus with Drainage Device, Percutaneous Endoscopic Approach */
"0D924ZZ"  =  "1"  /* Drainage of Middle Esophagus, Percutaneous Endoscopic Approach */
"0D9270Z"  =  "1"  /* Drainage of Middle Esophagus with Drainage Device, Via Natural or Artificial Opening */
"0D927ZZ"  =  "1"  /* Drainage of Middle Esophagus, Via Natural or Artificial Opening */
"0D9280Z"  =  "1"  /* Drainage of Middle Esophagus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D928ZZ"  =  "1"  /* Drainage of Middle Esophagus, Via Natural or Artificial Opening Endoscopic */
"0D9300Z"  =  "1"  /* Drainage of Lower Esophagus with Drainage Device, Open Approach */
"0D930ZX"  =  "1"  /* Drainage of Lower Esophagus, Open Approach, Diagnostic */
"0D930ZZ"  =  "1"  /* Drainage of Lower Esophagus, Open Approach */
"0D9330Z"  =  "1"  /* Drainage of Lower Esophagus with Drainage Device, Percutaneous Approach */
"0D933ZZ"  =  "1"  /* Drainage of Lower Esophagus, Percutaneous Approach */
"0D9340Z"  =  "1"  /* Drainage of Lower Esophagus with Drainage Device, Percutaneous Endoscopic Approach */
"0D934ZZ"  =  "1"  /* Drainage of Lower Esophagus, Percutaneous Endoscopic Approach */
"0D9370Z"  =  "1"  /* Drainage of Lower Esophagus with Drainage Device, Via Natural or Artificial Opening */
"0D937ZZ"  =  "1"  /* Drainage of Lower Esophagus, Via Natural or Artificial Opening */
"0D9380Z"  =  "1"  /* Drainage of Lower Esophagus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D938ZZ"  =  "1"  /* Drainage of Lower Esophagus, Via Natural or Artificial Opening Endoscopic */
"0D9400Z"  =  "1"  /* Drainage of Esophagogastric Junction with Drainage Device, Open Approach */
"0D940ZX"  =  "1"  /* Drainage of Esophagogastric Junction, Open Approach, Diagnostic */
"0D940ZZ"  =  "1"  /* Drainage of Esophagogastric Junction, Open Approach */
"0D9430Z"  =  "1"  /* Drainage of Esophagogastric Junction with Drainage Device, Percutaneous Approach */
"0D943ZZ"  =  "1"  /* Drainage of Esophagogastric Junction, Percutaneous Approach */
"0D9440Z"  =  "1"  /* Drainage of Esophagogastric Junction with Drainage Device, Percutaneous Endoscopic Approach */
"0D944ZZ"  =  "1"  /* Drainage of Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0D9470Z"  =  "1"  /* Drainage of Esophagogastric Junction with Drainage Device, Via Natural or Artificial Opening */
"0D947ZZ"  =  "1"  /* Drainage of Esophagogastric Junction, Via Natural or Artificial Opening */
"0D9480Z"  =  "1"  /* Drainage of Esophagogastric Junction with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D948ZZ"  =  "1"  /* Drainage of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic */
"0D9500Z"  =  "1"  /* Drainage of Esophagus with Drainage Device, Open Approach */
"0D950ZX"  =  "1"  /* Drainage of Esophagus, Open Approach, Diagnostic */
"0D950ZZ"  =  "1"  /* Drainage of Esophagus, Open Approach */
"0D9530Z"  =  "1"  /* Drainage of Esophagus with Drainage Device, Percutaneous Approach */
"0D953ZZ"  =  "1"  /* Drainage of Esophagus, Percutaneous Approach */
"0D9540Z"  =  "1"  /* Drainage of Esophagus with Drainage Device, Percutaneous Endoscopic Approach */
"0D954ZZ"  =  "1"  /* Drainage of Esophagus, Percutaneous Endoscopic Approach */
"0D9570Z"  =  "1"  /* Drainage of Esophagus with Drainage Device, Via Natural or Artificial Opening */
"0D957ZZ"  =  "1"  /* Drainage of Esophagus, Via Natural or Artificial Opening */
"0D9580Z"  =  "1"  /* Drainage of Esophagus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D958ZZ"  =  "1"  /* Drainage of Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DB10ZX"  =  "1"  /* Excision of Upper Esophagus, Open Approach, Diagnostic */
"0DB10ZZ"  =  "1"  /* Excision of Upper Esophagus, Open Approach */
"0DB13ZZ"  =  "1"  /* Excision of Upper Esophagus, Percutaneous Approach */
"0DB17ZZ"  =  "1"  /* Excision of Upper Esophagus, Via Natural or Artificial Opening */
"0DB20ZX"  =  "1"  /* Excision of Middle Esophagus, Open Approach, Diagnostic */
"0DB20ZZ"  =  "1"  /* Excision of Middle Esophagus, Open Approach */
"0DB23ZZ"  =  "1"  /* Excision of Middle Esophagus, Percutaneous Approach */
"0DB27ZZ"  =  "1"  /* Excision of Middle Esophagus, Via Natural or Artificial Opening */
"0DB30ZX"  =  "1"  /* Excision of Lower Esophagus, Open Approach, Diagnostic */
"0DB30ZZ"  =  "1"  /* Excision of Lower Esophagus, Open Approach */
"0DB33ZZ"  =  "1"  /* Excision of Lower Esophagus, Percutaneous Approach */
"0DB37ZZ"  =  "1"  /* Excision of Lower Esophagus, Via Natural or Artificial Opening */
"0DB40ZX"  =  "1"  /* Excision of Esophagogastric Junction, Open Approach, Diagnostic */
"0DB40ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Open Approach */
"0DB43ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Percutaneous Approach */
"0DB44ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0DB47ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Via Natural or Artificial Opening */
"0DB50ZX"  =  "1"  /* Excision of Esophagus, Open Approach, Diagnostic */
"0DB50ZZ"  =  "1"  /* Excision of Esophagus, Open Approach */
"0DB53ZZ"  =  "1"  /* Excision of Esophagus, Percutaneous Approach */
"0DB57ZZ"  =  "1"  /* Excision of Esophagus, Via Natural or Artificial Opening */
"0DC10ZZ"  =  "1"  /* Extirpation of Matter from Upper Esophagus, Open Approach */
"0DC13ZZ"  =  "1"  /* Extirpation of Matter from Upper Esophagus, Percutaneous Approach */
"0DC14ZZ"  =  "1"  /* Extirpation of Matter from Upper Esophagus, Percutaneous Endoscopic Approach */
"0DC20ZZ"  =  "1"  /* Extirpation of Matter from Middle Esophagus, Open Approach */
"0DC23ZZ"  =  "1"  /* Extirpation of Matter from Middle Esophagus, Percutaneous Approach */
"0DC24ZZ"  =  "1"  /* Extirpation of Matter from Middle Esophagus, Percutaneous Endoscopic Approach */
"0DC30ZZ"  =  "1"  /* Extirpation of Matter from Lower Esophagus, Open Approach */
"0DC33ZZ"  =  "1"  /* Extirpation of Matter from Lower Esophagus, Percutaneous Approach */
"0DC34ZZ"  =  "1"  /* Extirpation of Matter from Lower Esophagus, Percutaneous Endoscopic Approach */
"0DC40ZZ"  =  "1"  /* Extirpation of Matter from Esophagogastric Junction, Open Approach */
"0DC43ZZ"  =  "1"  /* Extirpation of Matter from Esophagogastric Junction, Percutaneous Approach */
"0DC44ZZ"  =  "1"  /* Extirpation of Matter from Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0DC50ZZ"  =  "1"  /* Extirpation of Matter from Esophagus, Open Approach */
"0DC53ZZ"  =  "1"  /* Extirpation of Matter from Esophagus, Percutaneous Approach */
"0DC54ZZ"  =  "1"  /* Extirpation of Matter from Esophagus, Percutaneous Endoscopic Approach */
"0DF50ZZ"  =  "1"  /* Fragmentation in Esophagus, Open Approach */
"0DF53ZZ"  =  "1"  /* Fragmentation in Esophagus, Percutaneous Approach */
"0DF54ZZ"  =  "1"  /* Fragmentation in Esophagus, Percutaneous Endoscopic Approach */
"0DF57ZZ"  =  "1"  /* Fragmentation in Esophagus, Via Natural or Artificial Opening */
"0DF58ZZ"  =  "1"  /* Fragmentation in Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DH502Z"  =  "1"  /* Insertion of Monitoring Device into Esophagus, Open Approach */
"0DH503Z"  =  "1"  /* Insertion of Infusion Device into Esophagus, Open Approach */
"0DH50DZ"  =  "1"  /* Insertion of Intraluminal Device into Esophagus, Open Approach */
"0DH50UZ"  =  "1"  /* Insertion of Feeding Device into Esophagus, Open Approach */
"0DH532Z"  =  "1"  /* Insertion of Monitoring Device into Esophagus, Percutaneous Approach */
"0DH533Z"  =  "1"  /* Insertion of Infusion Device into Esophagus, Percutaneous Approach */
"0DH53DZ"  =  "1"  /* Insertion of Intraluminal Device into Esophagus, Percutaneous Approach */
"0DH53UZ"  =  "1"  /* Insertion of Feeding Device into Esophagus, Percutaneous Approach */
"0DH542Z"  =  "1"  /* Insertion of Monitoring Device into Esophagus, Percutaneous Endoscopic Approach */
"0DH543Z"  =  "1"  /* Insertion of Infusion Device into Esophagus, Percutaneous Endoscopic Approach */
"0DH54DZ"  =  "1"  /* Insertion of Intraluminal Device into Esophagus, Percutaneous Endoscopic Approach */
"0DH54UZ"  =  "1"  /* Insertion of Feeding Device into Esophagus, Percutaneous Endoscopic Approach */
"0DH572Z"  =  "1"  /* Insertion of Monitoring Device into Esophagus, Via Natural or Artificial Opening */
"0DH573Z"  =  "1"  /* Insertion of Infusion Device into Esophagus, Via Natural or Artificial Opening */
"0DH57DZ"  =  "1"  /* Insertion of Intraluminal Device into Esophagus, Via Natural or Artificial Opening */
"0DH57UZ"  =  "1"  /* Insertion of Feeding Device into Esophagus, Via Natural or Artificial Opening */
"0DH582Z"  =  "1"  /* Insertion of Monitoring Device into Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DH583Z"  =  "1"  /* Insertion of Infusion Device into Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DH58DZ"  =  "1"  /* Insertion of Intraluminal Device into Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DH58UZ"  =  "1"  /* Insertion of Feeding Device into Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DJ04ZZ"  =  "1"  /* Inspection of Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DM50ZZ"  =  "1"  /* Reattachment of Esophagus, Open Approach */
"0DM54ZZ"  =  "1"  /* Reattachment of Esophagus, Percutaneous Endoscopic Approach */
"0DN10ZZ"  =  "1"  /* Release Upper Esophagus, Open Approach */
"0DN13ZZ"  =  "1"  /* Release Upper Esophagus, Percutaneous Approach */
"0DN14ZZ"  =  "1"  /* Release Upper Esophagus, Percutaneous Endoscopic Approach */
"0DN17ZZ"  =  "1"  /* Release Upper Esophagus, Via Natural or Artificial Opening */
"0DN18ZZ"  =  "1"  /* Release Upper Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DN20ZZ"  =  "1"  /* Release Middle Esophagus, Open Approach */
"0DN23ZZ"  =  "1"  /* Release Middle Esophagus, Percutaneous Approach */
"0DN24ZZ"  =  "1"  /* Release Middle Esophagus, Percutaneous Endoscopic Approach */
"0DN27ZZ"  =  "1"  /* Release Middle Esophagus, Via Natural or Artificial Opening */
"0DN28ZZ"  =  "1"  /* Release Middle Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DN30ZZ"  =  "1"  /* Release Lower Esophagus, Open Approach */
"0DN33ZZ"  =  "1"  /* Release Lower Esophagus, Percutaneous Approach */
"0DN34ZZ"  =  "1"  /* Release Lower Esophagus, Percutaneous Endoscopic Approach */
"0DN37ZZ"  =  "1"  /* Release Lower Esophagus, Via Natural or Artificial Opening */
"0DN38ZZ"  =  "1"  /* Release Lower Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DN40ZZ"  =  "1"  /* Release Esophagogastric Junction, Open Approach */
"0DN43ZZ"  =  "1"  /* Release Esophagogastric Junction, Percutaneous Approach */
"0DN44ZZ"  =  "1"  /* Release Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0DN47ZZ"  =  "1"  /* Release Esophagogastric Junction, Via Natural or Artificial Opening */
"0DN48ZZ"  =  "1"  /* Release Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic */
"0DN50ZZ"  =  "1"  /* Release Esophagus, Open Approach */
"0DN53ZZ"  =  "1"  /* Release Esophagus, Percutaneous Approach */
"0DN54ZZ"  =  "1"  /* Release Esophagus, Percutaneous Endoscopic Approach */
"0DN57ZZ"  =  "1"  /* Release Esophagus, Via Natural or Artificial Opening */
"0DN58ZZ"  =  "1"  /* Release Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DP501Z"  =  "1"  /* Removal of Radioactive Element from Esophagus, Open Approach */
"0DP502Z"  =  "1"  /* Removal of Monitoring Device from Esophagus, Open Approach */
"0DP503Z"  =  "1"  /* Removal of Infusion Device from Esophagus, Open Approach */
"0DP50UZ"  =  "1"  /* Removal of Feeding Device from Esophagus, Open Approach */
"0DP531Z"  =  "1"  /* Removal of Radioactive Element from Esophagus, Percutaneous Approach */
"0DP532Z"  =  "1"  /* Removal of Monitoring Device from Esophagus, Percutaneous Approach */
"0DP533Z"  =  "1"  /* Removal of Infusion Device from Esophagus, Percutaneous Approach */
"0DP53UZ"  =  "1"  /* Removal of Feeding Device from Esophagus, Percutaneous Approach */
"0DP541Z"  =  "1"  /* Removal of Radioactive Element from Esophagus, Percutaneous Endoscopic Approach */
"0DP542Z"  =  "1"  /* Removal of Monitoring Device from Esophagus, Percutaneous Endoscopic Approach */
"0DP543Z"  =  "1"  /* Removal of Infusion Device from Esophagus, Percutaneous Endoscopic Approach */
"0DP54UZ"  =  "1"  /* Removal of Feeding Device from Esophagus, Percutaneous Endoscopic Approach */
"0DP57DZ"  =  "1"  /* Removal of Intraluminal Device from Esophagus, Via Natural or Artificial Opening */
"0DP58DZ"  =  "1"  /* Removal of Intraluminal Device from Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DQ10ZZ"  =  "1"  /* Repair Upper Esophagus, Open Approach */
"0DQ13ZZ"  =  "1"  /* Repair Upper Esophagus, Percutaneous Approach */
"0DQ14ZZ"  =  "1"  /* Repair Upper Esophagus, Percutaneous Endoscopic Approach */
"0DQ17ZZ"  =  "1"  /* Repair Upper Esophagus, Via Natural or Artificial Opening */
"0DQ18ZZ"  =  "1"  /* Repair Upper Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DQ20ZZ"  =  "1"  /* Repair Middle Esophagus, Open Approach */
"0DQ23ZZ"  =  "1"  /* Repair Middle Esophagus, Percutaneous Approach */
"0DQ24ZZ"  =  "1"  /* Repair Middle Esophagus, Percutaneous Endoscopic Approach */
"0DQ27ZZ"  =  "1"  /* Repair Middle Esophagus, Via Natural or Artificial Opening */
"0DQ28ZZ"  =  "1"  /* Repair Middle Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DQ30ZZ"  =  "1"  /* Repair Lower Esophagus, Open Approach */
"0DQ33ZZ"  =  "1"  /* Repair Lower Esophagus, Percutaneous Approach */
"0DQ34ZZ"  =  "1"  /* Repair Lower Esophagus, Percutaneous Endoscopic Approach */
"0DQ37ZZ"  =  "1"  /* Repair Lower Esophagus, Via Natural or Artificial Opening */
"0DQ38ZZ"  =  "1"  /* Repair Lower Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DQ40ZZ"  =  "1"  /* Repair Esophagogastric Junction, Open Approach */
"0DQ43ZZ"  =  "1"  /* Repair Esophagogastric Junction, Percutaneous Approach */
"0DQ44ZZ"  =  "1"  /* Repair Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0DQ47ZZ"  =  "1"  /* Repair Esophagogastric Junction, Via Natural or Artificial Opening */
"0DQ48ZZ"  =  "1"  /* Repair Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic */
"0DQ50ZZ"  =  "1"  /* Repair Esophagus, Open Approach */
"0DQ53ZZ"  =  "1"  /* Repair Esophagus, Percutaneous Approach */
"0DQ54ZZ"  =  "1"  /* Repair Esophagus, Percutaneous Endoscopic Approach */
"0DQ57ZZ"  =  "1"  /* Repair Esophagus, Via Natural or Artificial Opening */
"0DQ58ZZ"  =  "1"  /* Repair Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DR507Z"  =  "1"  /* Replacement of Esophagus with Autologous Tissue Substitute, Open Approach */
"0DR50JZ"  =  "1"  /* Replacement of Esophagus with Synthetic Substitute, Open Approach */
"0DR50KZ"  =  "1"  /* Replacement of Esophagus with Nonautologous Tissue Substitute, Open Approach */
"0DR547Z"  =  "1"  /* Replacement of Esophagus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DR54JZ"  =  "1"  /* Replacement of Esophagus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DR54KZ"  =  "1"  /* Replacement of Esophagus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DR577Z"  =  "1"  /* Replacement of Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DR57JZ"  =  "1"  /* Replacement of Esophagus with Synthetic Substitute, Via Natural or Artificial Opening */
"0DR57KZ"  =  "1"  /* Replacement of Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DR587Z"  =  "1"  /* Replacement of Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DR58JZ"  =  "1"  /* Replacement of Esophagus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DR58KZ"  =  "1"  /* Replacement of Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DS50ZZ"  =  "1"  /* Reposition Esophagus, Open Approach */
"0DS54ZZ"  =  "1"  /* Reposition Esophagus, Percutaneous Endoscopic Approach */
"0DS57ZZ"  =  "1"  /* Reposition Esophagus, Via Natural or Artificial Opening */
"0DS58ZZ"  =  "1"  /* Reposition Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DS5XZZ"  =  "1"  /* Reposition Esophagus, External Approach */
"0DT10ZZ"  =  "1"  /* Resection of Upper Esophagus, Open Approach */
"0DT14ZZ"  =  "1"  /* Resection of Upper Esophagus, Percutaneous Endoscopic Approach */
"0DT17ZZ"  =  "1"  /* Resection of Upper Esophagus, Via Natural or Artificial Opening */
"0DT18ZZ"  =  "1"  /* Resection of Upper Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT20ZZ"  =  "1"  /* Resection of Middle Esophagus, Open Approach */
"0DT24ZZ"  =  "1"  /* Resection of Middle Esophagus, Percutaneous Endoscopic Approach */
"0DT27ZZ"  =  "1"  /* Resection of Middle Esophagus, Via Natural or Artificial Opening */
"0DT28ZZ"  =  "1"  /* Resection of Middle Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT30ZZ"  =  "1"  /* Resection of Lower Esophagus, Open Approach */
"0DT34ZZ"  =  "1"  /* Resection of Lower Esophagus, Percutaneous Endoscopic Approach */
"0DT37ZZ"  =  "1"  /* Resection of Lower Esophagus, Via Natural or Artificial Opening */
"0DT38ZZ"  =  "1"  /* Resection of Lower Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT40ZZ"  =  "1"  /* Resection of Esophagogastric Junction, Open Approach */
"0DT44ZZ"  =  "1"  /* Resection of Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0DT47ZZ"  =  "1"  /* Resection of Esophagogastric Junction, Via Natural or Artificial Opening */
"0DT48ZZ"  =  "1"  /* Resection of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic */
"0DT50ZZ"  =  "1"  /* Resection of Esophagus, Open Approach */
"0DT54ZZ"  =  "1"  /* Resection of Esophagus, Percutaneous Endoscopic Approach */
"0DT57ZZ"  =  "1"  /* Resection of Esophagus, Via Natural or Artificial Opening */
"0DT58ZZ"  =  "1"  /* Resection of Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT60ZZ"  =  "1"  /* Resection of Stomach, Open Approach */
"0DT64ZZ"  =  "1"  /* Resection of Stomach, Percutaneous Endoscopic Approach */
"0DT67ZZ"  =  "1"  /* Resection of Stomach, Via Natural or Artificial Opening */
"0DT68ZZ"  =  "1"  /* Resection of Stomach, Via Natural or Artificial Opening Endoscopic */
"0DU107Z"  =  "1"  /* Supplement Upper Esophagus with Autologous Tissue Substitute, Open Approach */
"0DU10JZ"  =  "1"  /* Supplement Upper Esophagus with Synthetic Substitute, Open Approach */
"0DU10KZ"  =  "1"  /* Supplement Upper Esophagus with Nonautologous Tissue Substitute, Open Approach */
"0DU147Z"  =  "1"  /* Supplement Upper Esophagus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU14JZ"  =  "1"  /* Supplement Upper Esophagus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DU14KZ"  =  "1"  /* Supplement Upper Esophagus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU177Z"  =  "1"  /* Supplement Upper Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU17JZ"  =  "1"  /* Supplement Upper Esophagus with Synthetic Substitute, Via Natural or Artificial Opening */
"0DU17KZ"  =  "1"  /* Supplement Upper Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU187Z"  =  "1"  /* Supplement Upper Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU18JZ"  =  "1"  /* Supplement Upper Esophagus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU18KZ"  =  "1"  /* Supplement Upper Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU207Z"  =  "1"  /* Supplement Middle Esophagus with Autologous Tissue Substitute, Open Approach */
"0DU20JZ"  =  "1"  /* Supplement Middle Esophagus with Synthetic Substitute, Open Approach */
"0DU20KZ"  =  "1"  /* Supplement Middle Esophagus with Nonautologous Tissue Substitute, Open Approach */
"0DU247Z"  =  "1"  /* Supplement Middle Esophagus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU24JZ"  =  "1"  /* Supplement Middle Esophagus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DU24KZ"  =  "1"  /* Supplement Middle Esophagus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU277Z"  =  "1"  /* Supplement Middle Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU27JZ"  =  "1"  /* Supplement Middle Esophagus with Synthetic Substitute, Via Natural or Artificial Opening */
"0DU27KZ"  =  "1"  /* Supplement Middle Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU287Z"  =  "1"  /* Supplement Middle Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU28JZ"  =  "1"  /* Supplement Middle Esophagus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU28KZ"  =  "1"  /* Supplement Middle Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU307Z"  =  "1"  /* Supplement Lower Esophagus with Autologous Tissue Substitute, Open Approach */
"0DU30JZ"  =  "1"  /* Supplement Lower Esophagus with Synthetic Substitute, Open Approach */
"0DU30KZ"  =  "1"  /* Supplement Lower Esophagus with Nonautologous Tissue Substitute, Open Approach */
"0DU347Z"  =  "1"  /* Supplement Lower Esophagus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU34JZ"  =  "1"  /* Supplement Lower Esophagus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DU34KZ"  =  "1"  /* Supplement Lower Esophagus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU377Z"  =  "1"  /* Supplement Lower Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU37JZ"  =  "1"  /* Supplement Lower Esophagus with Synthetic Substitute, Via Natural or Artificial Opening */
"0DU37KZ"  =  "1"  /* Supplement Lower Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU387Z"  =  "1"  /* Supplement Lower Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU38JZ"  =  "1"  /* Supplement Lower Esophagus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU38KZ"  =  "1"  /* Supplement Lower Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU407Z"  =  "1"  /* Supplement Esophagogastric Junction with Autologous Tissue Substitute, Open Approach */
"0DU40JZ"  =  "1"  /* Supplement Esophagogastric Junction with Synthetic Substitute, Open Approach */
"0DU40KZ"  =  "1"  /* Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Open Approach */
"0DU447Z"  =  "1"  /* Supplement Esophagogastric Junction with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU44JZ"  =  "1"  /* Supplement Esophagogastric Junction with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DU44KZ"  =  "1"  /* Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU477Z"  =  "1"  /* Supplement Esophagogastric Junction with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU47JZ"  =  "1"  /* Supplement Esophagogastric Junction with Synthetic Substitute, Via Natural or Artificial Opening */
"0DU47KZ"  =  "1"  /* Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU487Z"  =  "1"  /* Supplement Esophagogastric Junction with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU48JZ"  =  "1"  /* Supplement Esophagogastric Junction with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU48KZ"  =  "1"  /* Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU507Z"  =  "1"  /* Supplement Esophagus with Autologous Tissue Substitute, Open Approach */
"0DU50JZ"  =  "1"  /* Supplement Esophagus with Synthetic Substitute, Open Approach */
"0DU50KZ"  =  "1"  /* Supplement Esophagus with Nonautologous Tissue Substitute, Open Approach */
"0DU547Z"  =  "1"  /* Supplement Esophagus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU54JZ"  =  "1"  /* Supplement Esophagus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DU54KZ"  =  "1"  /* Supplement Esophagus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU577Z"  =  "1"  /* Supplement Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU57JZ"  =  "1"  /* Supplement Esophagus with Synthetic Substitute, Via Natural or Artificial Opening */
"0DU57KZ"  =  "1"  /* Supplement Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU587Z"  =  "1"  /* Supplement Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU58JZ"  =  "1"  /* Supplement Esophagus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU58KZ"  =  "1"  /* Supplement Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DV10CZ"  =  "1"  /* Restriction of Upper Esophagus with Extraluminal Device, Open Approach */
"0DV10DZ"  =  "1"  /* Restriction of Upper Esophagus with Intraluminal Device, Open Approach */
"0DV10ZZ"  =  "1"  /* Restriction of Upper Esophagus, Open Approach */
"0DV13CZ"  =  "1"  /* Restriction of Upper Esophagus with Extraluminal Device, Percutaneous Approach */
"0DV13DZ"  =  "1"  /* Restriction of Upper Esophagus with Intraluminal Device, Percutaneous Approach */
"0DV13ZZ"  =  "1"  /* Restriction of Upper Esophagus, Percutaneous Approach */
"0DV14CZ"  =  "1"  /* Restriction of Upper Esophagus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DV14DZ"  =  "1"  /* Restriction of Upper Esophagus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DV14ZZ"  =  "1"  /* Restriction of Upper Esophagus, Percutaneous Endoscopic Approach */
"0DV17DZ"  =  "1"  /* Restriction of Upper Esophagus with Intraluminal Device, Via Natural or Artificial Opening */
"0DV17ZZ"  =  "1"  /* Restriction of Upper Esophagus, Via Natural or Artificial Opening */
"0DV18DZ"  =  "1"  /* Restriction of Upper Esophagus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DV18ZZ"  =  "1"  /* Restriction of Upper Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DV20CZ"  =  "1"  /* Restriction of Middle Esophagus with Extraluminal Device, Open Approach */
"0DV20DZ"  =  "1"  /* Restriction of Middle Esophagus with Intraluminal Device, Open Approach */
"0DV20ZZ"  =  "1"  /* Restriction of Middle Esophagus, Open Approach */
"0DV23CZ"  =  "1"  /* Restriction of Middle Esophagus with Extraluminal Device, Percutaneous Approach */
"0DV23DZ"  =  "1"  /* Restriction of Middle Esophagus with Intraluminal Device, Percutaneous Approach */
"0DV23ZZ"  =  "1"  /* Restriction of Middle Esophagus, Percutaneous Approach */
"0DV24CZ"  =  "1"  /* Restriction of Middle Esophagus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DV24DZ"  =  "1"  /* Restriction of Middle Esophagus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DV24ZZ"  =  "1"  /* Restriction of Middle Esophagus, Percutaneous Endoscopic Approach */
"0DV27DZ"  =  "1"  /* Restriction of Middle Esophagus with Intraluminal Device, Via Natural or Artificial Opening */
"0DV27ZZ"  =  "1"  /* Restriction of Middle Esophagus, Via Natural or Artificial Opening */
"0DV28DZ"  =  "1"  /* Restriction of Middle Esophagus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DV28ZZ"  =  "1"  /* Restriction of Middle Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DV30CZ"  =  "1"  /* Restriction of Lower Esophagus with Extraluminal Device, Open Approach */
"0DV30DZ"  =  "1"  /* Restriction of Lower Esophagus with Intraluminal Device, Open Approach */
"0DV30ZZ"  =  "1"  /* Restriction of Lower Esophagus, Open Approach */
"0DV33CZ"  =  "1"  /* Restriction of Lower Esophagus with Extraluminal Device, Percutaneous Approach */
"0DV33DZ"  =  "1"  /* Restriction of Lower Esophagus with Intraluminal Device, Percutaneous Approach */
"0DV33ZZ"  =  "1"  /* Restriction of Lower Esophagus, Percutaneous Approach */
"0DV34CZ"  =  "1"  /* Restriction of Lower Esophagus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DV34DZ"  =  "1"  /* Restriction of Lower Esophagus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DV34ZZ"  =  "1"  /* Restriction of Lower Esophagus, Percutaneous Endoscopic Approach */
"0DV37DZ"  =  "1"  /* Restriction of Lower Esophagus with Intraluminal Device, Via Natural or Artificial Opening */
"0DV37ZZ"  =  "1"  /* Restriction of Lower Esophagus, Via Natural or Artificial Opening */
"0DV38DZ"  =  "1"  /* Restriction of Lower Esophagus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DV38ZZ"  =  "1"  /* Restriction of Lower Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DV40CZ"  =  "1"  /* Restriction of Esophagogastric Junction with Extraluminal Device, Open Approach */
"0DV40DZ"  =  "1"  /* Restriction of Esophagogastric Junction with Intraluminal Device, Open Approach */
"0DV40ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Open Approach */
"0DV43CZ"  =  "1"  /* Restriction of Esophagogastric Junction with Extraluminal Device, Percutaneous Approach */
"0DV43DZ"  =  "1"  /* Restriction of Esophagogastric Junction with Intraluminal Device, Percutaneous Approach */
"0DV43ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Percutaneous Approach */
"0DV44CZ"  =  "1"  /* Restriction of Esophagogastric Junction with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DV44DZ"  =  "1"  /* Restriction of Esophagogastric Junction with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DV44ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0DV47DZ"  =  "1"  /* Restriction of Esophagogastric Junction with Intraluminal Device, Via Natural or Artificial Opening */
"0DV47ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Via Natural or Artificial Opening */
"0DV48DZ"  =  "1"  /* Restriction of Esophagogastric Junction with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DV48ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic */
"0DV50CZ"  =  "1"  /* Restriction of Esophagus with Extraluminal Device, Open Approach */
"0DV50DZ"  =  "1"  /* Restriction of Esophagus with Intraluminal Device, Open Approach */
"0DV50ZZ"  =  "1"  /* Restriction of Esophagus, Open Approach */
"0DV53CZ"  =  "1"  /* Restriction of Esophagus with Extraluminal Device, Percutaneous Approach */
"0DV53DZ"  =  "1"  /* Restriction of Esophagus with Intraluminal Device, Percutaneous Approach */
"0DV53ZZ"  =  "1"  /* Restriction of Esophagus, Percutaneous Approach */
"0DV54CZ"  =  "1"  /* Restriction of Esophagus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DV54DZ"  =  "1"  /* Restriction of Esophagus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DV54ZZ"  =  "1"  /* Restriction of Esophagus, Percutaneous Endoscopic Approach */
"0DV57DZ"  =  "1"  /* Restriction of Esophagus with Intraluminal Device, Via Natural or Artificial Opening */
"0DV57ZZ"  =  "1"  /* Restriction of Esophagus, Via Natural or Artificial Opening */
"0DV58DZ"  =  "1"  /* Restriction of Esophagus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DV58ZZ"  =  "1"  /* Restriction of Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DW57DZ"  =  "1"  /* Revision of Intraluminal Device in Esophagus, Via Natural or Artificial Opening */
"0DW58DZ"  =  "1"  /* Revision of Intraluminal Device in Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DX60Z5"  =  "1"  /* Transfer Stomach to Esophagus, Open Approach */
"0DX64Z5"  =  "1"  /* Transfer Stomach to Esophagus, Percutaneous Endoscopic Approach */
"0DX80Z5"  =  "1"  /* Transfer Small Intestine to Esophagus, Open Approach */
"0DX84Z5"  =  "1"  /* Transfer Small Intestine to Esophagus, Percutaneous Endoscopic Approach */
"0DXE0Z5"  =  "1"  /* Transfer Large Intestine to Esophagus, Open Approach */
"0DXE4Z5"  =  "1"  /* Transfer Large Intestine to Esophagus, Percutaneous Endoscopic Approach */
"0JH604Z"  =  "1"  /* Insertion of Pacemaker, Single Chamber into Chest Subcutaneous Tissue and Fascia, Open Approach */
"0JH634Z"  =  "1"  /* Insertion of Pacemaker, Single Chamber into Chest Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JH804Z"  =  "1"  /* Insertion of Pacemaker, Single Chamber into Abdomen Subcutaneous Tissue and Fascia, Open Approach */
"0JH834Z"  =  "1"  /* Insertion of Pacemaker, Single Chamber into Abdomen Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0PB00ZZ"  =  "1"  /* Excision of Sternum, Open Approach */
"0PB03ZZ"  =  "1"  /* Excision of Sternum, Percutaneous Approach */
"0PB04ZZ"  =  "1"  /* Excision of Sternum, Percutaneous Endoscopic Approach */
"0PB10ZZ"  =  "1"  /* Excision of Right Rib, Open Approach */
"0PB13ZZ"  =  "1"  /* Excision of Right Rib, Percutaneous Approach */
"0PB14ZZ"  =  "1"  /* Excision of Right Rib, Percutaneous Endoscopic Approach */
"0PB20ZZ"  =  "1"  /* Excision of Left Rib, Open Approach */
"0PB23ZZ"  =  "1"  /* Excision of Left Rib, Percutaneous Approach */
"0PB24ZZ"  =  "1"  /* Excision of Left Rib, Percutaneous Endoscopic Approach */
"0PT00ZZ"  =  "1"  /* Resection of Sternum, Open Approach */
"0PT10ZZ"  =  "1"  /* Resection of Right Rib, Open Approach */
"0PT20ZZ"  =  "1"  /* Resection of Left Rib, Open Approach */
"0RG6070"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG60A0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG60J0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG60K0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG60Z0"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Anterior Approach, Anterior Column, Open Approach */
"0RG6370"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG63A0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG63J0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG63K0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG63Z0"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG6470"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64A0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64J0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64K0"  =  "1"  /* Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG64Z0"  =  "1"  /* Fusion of Thoracic Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG7070"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG70A0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG70J0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG70K0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG70Z0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Open Approach */
"0RG7370"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG73A0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG73J0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG73K0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG73Z0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG7470"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74A0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74J0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74K0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG74Z0"  =  "1"  /* Fusion of 2 to 7 Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG8070"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG80A0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RG80J0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG80K0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RG80Z0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Open Approach */
"0RG8370"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG83A0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG83J0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG83K0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG83Z0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RG8470"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84A0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84J0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84K0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RG84Z0"  =  "1"  /* Fusion of 8 or more Thoracic Vertebral Joints, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA070"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RGA0A0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach */
"0RGA0J0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RGA0K0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach */
"0RGA0Z0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Anterior Approach, Anterior Column, Open Approach */
"0RGA370"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3A0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3J0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3K0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RGA3Z0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Approach */
"0RGA470"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4A0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4J0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4K0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0RGA4Z0"  =  "1"  /* Fusion of Thoracolumbar Vertebral Joint, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach */
"0W190J4"  =  "1"  /* Bypass Right Pleural Cavity to Cutaneous with Synthetic Substitute, Open Approach */
"0W190J9"  =  "1"  /* Bypass Right Pleural Cavity to Right Pleural Cavity with Synthetic Substitute, Open Approach */
"0W190JB"  =  "1"  /* Bypass Right Pleural Cavity to Left Pleural Cavity with Synthetic Substitute, Open Approach */
"0W190JG"  =  "1"  /* Bypass Right Pleural Cavity to Peritoneal Cavity with Synthetic Substitute, Open Approach */
"0W190JJ"  =  "1"  /* Bypass Right Pleural Cavity to Pelvic Cavity with Synthetic Substitute, Open Approach */
"0W190JY"  =  "1"  /* Bypass Right Pleural Cavity to Lower Vein with Synthetic Substitute, Open Approach */
"0W193J4"  =  "1"  /* Bypass Right Pleural Cavity to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0W194J4"  =  "1"  /* Bypass Right Pleural Cavity to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W194J9"  =  "1"  /* Bypass Right Pleural Cavity to Right Pleural Cavity with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W194JB"  =  "1"  /* Bypass Right Pleural Cavity to Left Pleural Cavity with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W194JG"  =  "1"  /* Bypass Right Pleural Cavity to Peritoneal Cavity with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W194JJ"  =  "1"  /* Bypass Right Pleural Cavity to Pelvic Cavity with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W194JY"  =  "1"  /* Bypass Right Pleural Cavity to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W1B0J4"  =  "1"  /* Bypass Left Pleural Cavity to Cutaneous with Synthetic Substitute, Open Approach */
"0W1B0J9"  =  "1"  /* Bypass Left Pleural Cavity to Right Pleural Cavity with Synthetic Substitute, Open Approach */
"0W1B0JB"  =  "1"  /* Bypass Left Pleural Cavity to Left Pleural Cavity with Synthetic Substitute, Open Approach */
"0W1B0JG"  =  "1"  /* Bypass Left Pleural Cavity to Peritoneal Cavity with Synthetic Substitute, Open Approach */
"0W1B0JJ"  =  "1"  /* Bypass Left Pleural Cavity to Pelvic Cavity with Synthetic Substitute, Open Approach */
"0W1B0JY"  =  "1"  /* Bypass Left Pleural Cavity to Lower Vein with Synthetic Substitute, Open Approach */
"0W1B3J4"  =  "1"  /* Bypass Left Pleural Cavity to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0W1B4J4"  =  "1"  /* Bypass Left Pleural Cavity to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W1B4J9"  =  "1"  /* Bypass Left Pleural Cavity to Right Pleural Cavity with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W1B4JB"  =  "1"  /* Bypass Left Pleural Cavity to Left Pleural Cavity with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W1B4JG"  =  "1"  /* Bypass Left Pleural Cavity to Peritoneal Cavity with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W1B4JJ"  =  "1"  /* Bypass Left Pleural Cavity to Pelvic Cavity with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W1B4JY"  =  "1"  /* Bypass Left Pleural Cavity to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W390ZZ"  =  "1"  /* Control Bleeding in Right Pleural Cavity, Open Approach */
"0W3B0ZZ"  =  "1"  /* Control Bleeding in Left Pleural Cavity, Open Approach */
"0W3D0ZZ"  =  "1"  /* Control Bleeding in Pericardial Cavity, Open Approach */
"0W3Q0ZZ"  =  "1"  /* Control Bleeding in Respiratory Tract, Open Approach */
"0W9800Z"  =  "1"  /* Drainage of Chest Wall with Drainage Device, Open Approach */
"0W980ZX"  =  "1"  /* Drainage of Chest Wall, Open Approach, Diagnostic */
"0W980ZZ"  =  "1"  /* Drainage of Chest Wall, Open Approach */
"0W9830Z"  =  "1"  /* Drainage of Chest Wall with Drainage Device, Percutaneous Approach */
"0W983ZX"  =  "1"  /* Drainage of Chest Wall, Percutaneous Approach, Diagnostic */
"0W983ZZ"  =  "1"  /* Drainage of Chest Wall, Percutaneous Approach */
"0W9840Z"  =  "1"  /* Drainage of Chest Wall with Drainage Device, Percutaneous Endoscopic Approach */
"0W984ZX"  =  "1"  /* Drainage of Chest Wall, Percutaneous Endoscopic Approach, Diagnostic */
"0W984ZZ"  =  "1"  /* Drainage of Chest Wall, Percutaneous Endoscopic Approach */
"0W9900Z"  =  "1"  /* Drainage of Right Pleural Cavity with Drainage Device, Open Approach */
"0W990ZZ"  =  "1"  /* Drainage of Right Pleural Cavity, Open Approach */
"0W994ZZ"  =  "1"  /* Drainage of Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0W9B00Z"  =  "1"  /* Drainage of Left Pleural Cavity with Drainage Device, Open Approach */
"0W9B0ZZ"  =  "1"  /* Drainage of Left Pleural Cavity, Open Approach */
"0W9B4ZZ"  =  "1"  /* Drainage of Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0W9C00Z"  =  "1"  /* Drainage of Mediastinum with Drainage Device, Open Approach */
"0W9C0ZX"  =  "1"  /* Drainage of Mediastinum, Open Approach, Diagnostic */
"0W9C0ZZ"  =  "1"  /* Drainage of Mediastinum, Open Approach */
"0W9C30Z"  =  "1"  /* Drainage of Mediastinum with Drainage Device, Percutaneous Approach */
"0W9C3ZX"  =  "1"  /* Drainage of Mediastinum, Percutaneous Approach, Diagnostic */
"0W9C3ZZ"  =  "1"  /* Drainage of Mediastinum, Percutaneous Approach */
"0W9C40Z"  =  "1"  /* Drainage of Mediastinum with Drainage Device, Percutaneous Endoscopic Approach */
"0W9C4ZX"  =  "1"  /* Drainage of Mediastinum, Percutaneous Endoscopic Approach, Diagnostic */
"0W9C4ZZ"  =  "1"  /* Drainage of Mediastinum, Percutaneous Endoscopic Approach */
"0WB80ZX"  =  "1"  /* Excision of Chest Wall, Open Approach, Diagnostic */
"0WB80ZZ"  =  "1"  /* Excision of Chest Wall, Open Approach */
"0WB83ZX"  =  "1"  /* Excision of Chest Wall, Percutaneous Approach, Diagnostic */
"0WB83ZZ"  =  "1"  /* Excision of Chest Wall, Percutaneous Approach */
"0WB84ZX"  =  "1"  /* Excision of Chest Wall, Percutaneous Endoscopic Approach, Diagnostic */
"0WB84ZZ"  =  "1"  /* Excision of Chest Wall, Percutaneous Endoscopic Approach */
"0WB8XZX"  =  "1"  /* Excision of Chest Wall, External Approach, Diagnostic */
"0WB8XZZ"  =  "1"  /* Excision of Chest Wall, External Approach */
"0WBC0ZX"  =  "1"  /* Excision of Mediastinum, Open Approach, Diagnostic */
"0WBC0ZZ"  =  "1"  /* Excision of Mediastinum, Open Approach */
"0WBC3ZX"  =  "1"  /* Excision of Mediastinum, Percutaneous Approach, Diagnostic */
"0WBC3ZZ"  =  "1"  /* Excision of Mediastinum, Percutaneous Approach */
"0WBC4ZX"  =  "1"  /* Excision of Mediastinum, Percutaneous Endoscopic Approach, Diagnostic */
"0WBC4ZZ"  =  "1"  /* Excision of Mediastinum, Percutaneous Endoscopic Approach */
"0WC90ZZ"  =  "1"  /* Extirpation of Matter from Right Pleural Cavity, Open Approach */
"0WC93ZZ"  =  "1"  /* Extirpation of Matter from Right Pleural Cavity, Percutaneous Approach */
"0WC94ZZ"  =  "1"  /* Extirpation of Matter from Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WCB0ZZ"  =  "1"  /* Extirpation of Matter from Left Pleural Cavity, Open Approach */
"0WCB3ZZ"  =  "1"  /* Extirpation of Matter from Left Pleural Cavity, Percutaneous Approach */
"0WCB4ZZ"  =  "1"  /* Extirpation of Matter from Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WCC0ZZ"  =  "1"  /* Extirpation of Matter from Mediastinum, Open Approach */
"0WCC3ZZ"  =  "1"  /* Extirpation of Matter from Mediastinum, Percutaneous Approach */
"0WCC4ZZ"  =  "1"  /* Extirpation of Matter from Mediastinum, Percutaneous Endoscopic Approach */
"0WCQ0ZZ"  =  "1"  /* Extirpation of Matter from Respiratory Tract, Open Approach */
"0WCQ3ZZ"  =  "1"  /* Extirpation of Matter from Respiratory Tract, Percutaneous Approach */
"0WCQ4ZZ"  =  "1"  /* Extirpation of Matter from Respiratory Tract, Percutaneous Endoscopic Approach */
"0WCQ7ZZ"  =  "1"  /* Extirpation of Matter from Respiratory Tract, Via Natural or Artificial Opening */
"0WCQ8ZZ"  =  "1"  /* Extirpation of Matter from Respiratory Tract, Via Natural or Artificial Opening Endoscopic */
"0WF90ZZ"  =  "1"  /* Fragmentation in Right Pleural Cavity, Open Approach */
"0WF93ZZ"  =  "1"  /* Fragmentation in Right Pleural Cavity, Percutaneous Approach */
"0WF94ZZ"  =  "1"  /* Fragmentation in Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WFB0ZZ"  =  "1"  /* Fragmentation in Left Pleural Cavity, Open Approach */
"0WFB3ZZ"  =  "1"  /* Fragmentation in Left Pleural Cavity, Percutaneous Approach */
"0WFB4ZZ"  =  "1"  /* Fragmentation in Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WFC0ZZ"  =  "1"  /* Fragmentation in Mediastinum, Open Approach */
"0WFC3ZZ"  =  "1"  /* Fragmentation in Mediastinum, Percutaneous Approach */
"0WFC4ZZ"  =  "1"  /* Fragmentation in Mediastinum, Percutaneous Endoscopic Approach */
"0WFQ0ZZ"  =  "1"  /* Fragmentation in Respiratory Tract, Open Approach */
"0WFQ3ZZ"  =  "1"  /* Fragmentation in Respiratory Tract, Percutaneous Approach */
"0WFQ4ZZ"  =  "1"  /* Fragmentation in Respiratory Tract, Percutaneous Endoscopic Approach */
"0WFQ7ZZ"  =  "1"  /* Fragmentation in Respiratory Tract, Via Natural or Artificial Opening */
"0WFQ8ZZ"  =  "1"  /* Fragmentation in Respiratory Tract, Via Natural or Artificial Opening Endoscopic */
"0WH803Z"  =  "1"  /* Insertion of Infusion Device into Chest Wall, Open Approach */
"0WH80YZ"  =  "1"  /* Insertion of Other Device into Chest Wall, Open Approach */
"0WH833Z"  =  "1"  /* Insertion of Infusion Device into Chest Wall, Percutaneous Approach */
"0WH83YZ"  =  "1"  /* Insertion of Other Device into Chest Wall, Percutaneous Approach */
"0WH843Z"  =  "1"  /* Insertion of Infusion Device into Chest Wall, Percutaneous Endoscopic Approach */
"0WH84YZ"  =  "1"  /* Insertion of Other Device into Chest Wall, Percutaneous Endoscopic Approach */
"0WH903Z"  =  "1"  /* Insertion of Infusion Device into Right Pleural Cavity, Open Approach */
"0WH90YZ"  =  "1"  /* Insertion of Other Device into Right Pleural Cavity, Open Approach */
"0WH933Z"  =  "1"  /* Insertion of Infusion Device into Right Pleural Cavity, Percutaneous Approach */
"0WH93YZ"  =  "1"  /* Insertion of Other Device into Right Pleural Cavity, Percutaneous Approach */
"0WH943Z"  =  "1"  /* Insertion of Infusion Device into Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WH94YZ"  =  "1"  /* Insertion of Other Device into Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WHB03Z"  =  "1"  /* Insertion of Infusion Device into Left Pleural Cavity, Open Approach */
"0WHB0YZ"  =  "1"  /* Insertion of Other Device into Left Pleural Cavity, Open Approach */
"0WHB33Z"  =  "1"  /* Insertion of Infusion Device into Left Pleural Cavity, Percutaneous Approach */
"0WHB3YZ"  =  "1"  /* Insertion of Other Device into Left Pleural Cavity, Percutaneous Approach */
"0WHB43Z"  =  "1"  /* Insertion of Infusion Device into Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WHB4YZ"  =  "1"  /* Insertion of Other Device into Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WHC03Z"  =  "1"  /* Insertion of Infusion Device into Mediastinum, Open Approach */
"0WHC0YZ"  =  "1"  /* Insertion of Other Device into Mediastinum, Open Approach */
"0WHC33Z"  =  "1"  /* Insertion of Infusion Device into Mediastinum, Percutaneous Approach */
"0WHC3YZ"  =  "1"  /* Insertion of Other Device into Mediastinum, Percutaneous Approach */
"0WHC43Z"  =  "1"  /* Insertion of Infusion Device into Mediastinum, Percutaneous Endoscopic Approach */
"0WHC4YZ"  =  "1"  /* Insertion of Other Device into Mediastinum, Percutaneous Endoscopic Approach */
"0WHQ03Z"  =  "1"  /* Insertion of Infusion Device into Respiratory Tract, Open Approach */
"0WHQ0YZ"  =  "1"  /* Insertion of Other Device into Respiratory Tract, Open Approach */
"0WHQ33Z"  =  "1"  /* Insertion of Infusion Device into Respiratory Tract, Percutaneous Approach */
"0WHQ3YZ"  =  "1"  /* Insertion of Other Device into Respiratory Tract, Percutaneous Approach */
"0WHQ43Z"  =  "1"  /* Insertion of Infusion Device into Respiratory Tract, Percutaneous Endoscopic Approach */
"0WHQ4YZ"  =  "1"  /* Insertion of Other Device into Respiratory Tract, Percutaneous Endoscopic Approach */
"0WJ80ZZ"  =  "1"  /* Inspection of Chest Wall, Open Approach */
"0WJ83ZZ"  =  "1"  /* Inspection of Chest Wall, Percutaneous Approach */
"0WJ84ZZ"  =  "1"  /* Inspection of Chest Wall, Percutaneous Endoscopic Approach */
"0WJ90ZZ"  =  "1"  /* Inspection of Right Pleural Cavity, Open Approach */
"0WJ93ZZ"  =  "1"  /* Inspection of Right Pleural Cavity, Percutaneous Approach */
"0WJ94ZZ"  =  "1"  /* Inspection of Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WJB0ZZ"  =  "1"  /* Inspection of Left Pleural Cavity, Open Approach */
"0WJB3ZZ"  =  "1"  /* Inspection of Left Pleural Cavity, Percutaneous Approach */
"0WJB4ZZ"  =  "1"  /* Inspection of Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WJC0ZZ"  =  "1"  /* Inspection of Mediastinum, Open Approach */
"0WJC3ZZ"  =  "1"  /* Inspection of Mediastinum, Percutaneous Approach */
"0WJC4ZZ"  =  "1"  /* Inspection of Mediastinum, Percutaneous Endoscopic Approach */
"0WJD4ZZ"  =  "1"  /* Inspection of Pericardial Cavity, Percutaneous Endoscopic Approach */
"0WJQ0ZZ"  =  "1"  /* Inspection of Respiratory Tract, Open Approach */
"0WJQ3ZZ"  =  "1"  /* Inspection of Respiratory Tract, Percutaneous Approach */
"0WJQ4ZZ"  =  "1"  /* Inspection of Respiratory Tract, Percutaneous Endoscopic Approach */
"0WJQ7ZZ"  =  "1"  /* Inspection of Respiratory Tract, Via Natural or Artificial Opening Approach */
"0WJQ8ZZ"  =  "1"  /* Inspection of Respiratory Tract, Via Natural or Artificial Opening Endoscopic Approach */
"0WM80ZZ"  =  "1"  /* Reattachment of Chest Wall, Open Approach */
"0WP800Z"  =  "1"  /* Removal of Drainage Device from Chest Wall, Open Approach */
"0WP801Z"  =  "1"  /* Removal of Radioactive Element from Chest Wall, Open Approach */
"0WP803Z"  =  "1"  /* Removal of Infusion Device from Chest Wall, Open Approach */
"0WP807Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Chest Wall, Open Approach */
"0WP80JZ"  =  "1"  /* Removal of Synthetic Substitute from Chest Wall, Open Approach */
"0WP80KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Chest Wall, Open Approach */
"0WP80YZ"  =  "1"  /* Removal of Other Device from Chest Wall, Open Approach */
"0WP830Z"  =  "1"  /* Removal of Drainage Device from Chest Wall, Percutaneous Approach */
"0WP831Z"  =  "1"  /* Removal of Radioactive Element from Chest Wall, Percutaneous Approach */
"0WP833Z"  =  "1"  /* Removal of Infusion Device from Chest Wall, Percutaneous Approach */
"0WP837Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Chest Wall, Percutaneous Approach */
"0WP83JZ"  =  "1"  /* Removal of Synthetic Substitute from Chest Wall, Percutaneous Approach */
"0WP83KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Chest Wall, Percutaneous Approach */
"0WP83YZ"  =  "1"  /* Removal of Other Device from Chest Wall, Percutaneous Approach */
"0WP840Z"  =  "1"  /* Removal of Drainage Device from Chest Wall, Percutaneous Endoscopic Approach */
"0WP841Z"  =  "1"  /* Removal of Radioactive Element from Chest Wall, Percutaneous Endoscopic Approach */
"0WP843Z"  =  "1"  /* Removal of Infusion Device from Chest Wall, Percutaneous Endoscopic Approach */
"0WP847Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Chest Wall, Percutaneous Endoscopic Approach */
"0WP84JZ"  =  "1"  /* Removal of Synthetic Substitute from Chest Wall, Percutaneous Endoscopic Approach */
"0WP84KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Chest Wall, Percutaneous Endoscopic Approach */
"0WP84YZ"  =  "1"  /* Removal of Other Device from Chest Wall, Percutaneous Endoscopic Approach */
"0WP900Z"  =  "1"  /* Removal of Drainage Device from Right Pleural Cavity, Open Approach */
"0WP901Z"  =  "1"  /* Removal of Radioactive Element from Right Pleural Cavity, Open Approach */
"0WP903Z"  =  "1"  /* Removal of Infusion Device from Right Pleural Cavity, Open Approach */
"0WP90JZ"  =  "1"  /* Removal of Synthetic Substitute from Right Pleural Cavity, Open Approach */
"0WP90YZ"  =  "1"  /* Removal of Other Device from Right Pleural Cavity, Open Approach */
"0WP930Z"  =  "1"  /* Removal of Drainage Device from Right Pleural Cavity, Percutaneous Approach */
"0WP931Z"  =  "1"  /* Removal of Radioactive Element from Right Pleural Cavity, Percutaneous Approach */
"0WP933Z"  =  "1"  /* Removal of Infusion Device from Right Pleural Cavity, Percutaneous Approach */
"0WP93JZ"  =  "1"  /* Removal of Synthetic Substitute from Right Pleural Cavity, Percutaneous Approach */
"0WP93YZ"  =  "1"  /* Removal of Other Device from Right Pleural Cavity, Percutaneous Approach */
"0WP940Z"  =  "1"  /* Removal of Drainage Device from Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WP941Z"  =  "1"  /* Removal of Radioactive Element from Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WP943Z"  =  "1"  /* Removal of Infusion Device from Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WP94JZ"  =  "1"  /* Removal of Synthetic Substitute from Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WP94YZ"  =  "1"  /* Removal of Other Device from Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WPB00Z"  =  "1"  /* Removal of Drainage Device from Left Pleural Cavity, Open Approach */
"0WPB01Z"  =  "1"  /* Removal of Radioactive Element from Left Pleural Cavity, Open Approach */
"0WPB03Z"  =  "1"  /* Removal of Infusion Device from Left Pleural Cavity, Open Approach */
"0WPB0JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Pleural Cavity, Open Approach */
"0WPB0YZ"  =  "1"  /* Removal of Other Device from Left Pleural Cavity, Open Approach */
"0WPB30Z"  =  "1"  /* Removal of Drainage Device from Left Pleural Cavity, Percutaneous Approach */
"0WPB31Z"  =  "1"  /* Removal of Radioactive Element from Left Pleural Cavity, Percutaneous Approach */
"0WPB33Z"  =  "1"  /* Removal of Infusion Device from Left Pleural Cavity, Percutaneous Approach */
"0WPB3JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Pleural Cavity, Percutaneous Approach */
"0WPB3YZ"  =  "1"  /* Removal of Other Device from Left Pleural Cavity, Percutaneous Approach */
"0WPB40Z"  =  "1"  /* Removal of Drainage Device from Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WPB41Z"  =  "1"  /* Removal of Radioactive Element from Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WPB43Z"  =  "1"  /* Removal of Infusion Device from Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WPB4JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WPB4YZ"  =  "1"  /* Removal of Other Device from Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WPC00Z"  =  "1"  /* Removal of Drainage Device from Mediastinum, Open Approach */
"0WPC01Z"  =  "1"  /* Removal of Radioactive Element from Mediastinum, Open Approach */
"0WPC03Z"  =  "1"  /* Removal of Infusion Device from Mediastinum, Open Approach */
"0WPC07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mediastinum, Open Approach */
"0WPC0JZ"  =  "1"  /* Removal of Synthetic Substitute from Mediastinum, Open Approach */
"0WPC0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mediastinum, Open Approach */
"0WPC0YZ"  =  "1"  /* Removal of Other Device from Mediastinum, Open Approach */
"0WPC30Z"  =  "1"  /* Removal of Drainage Device from Mediastinum, Percutaneous Approach */
"0WPC31Z"  =  "1"  /* Removal of Radioactive Element from Mediastinum, Percutaneous Approach */
"0WPC33Z"  =  "1"  /* Removal of Infusion Device from Mediastinum, Percutaneous Approach */
"0WPC37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mediastinum, Percutaneous Approach */
"0WPC3JZ"  =  "1"  /* Removal of Synthetic Substitute from Mediastinum, Percutaneous Approach */
"0WPC3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mediastinum, Percutaneous Approach */
"0WPC3YZ"  =  "1"  /* Removal of Other Device from Mediastinum, Percutaneous Approach */
"0WPC40Z"  =  "1"  /* Removal of Drainage Device from Mediastinum, Percutaneous Endoscopic Approach */
"0WPC41Z"  =  "1"  /* Removal of Radioactive Element from Mediastinum, Percutaneous Endoscopic Approach */
"0WPC43Z"  =  "1"  /* Removal of Infusion Device from Mediastinum, Percutaneous Endoscopic Approach */
"0WPC47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mediastinum, Percutaneous Endoscopic Approach */
"0WPC4JZ"  =  "1"  /* Removal of Synthetic Substitute from Mediastinum, Percutaneous Endoscopic Approach */
"0WPC4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mediastinum, Percutaneous Endoscopic Approach */
"0WPC4YZ"  =  "1"  /* Removal of Other Device from Mediastinum, Percutaneous Endoscopic Approach */
"0WPQ01Z"  =  "1"  /* Removal of Radioactive Element from Respiratory Tract, Open Approach */
"0WPQ03Z"  =  "1"  /* Removal of Infusion Device from Respiratory Tract, Open Approach */
"0WPQ0YZ"  =  "1"  /* Removal of Other Device from Respiratory Tract, Open Approach */
"0WPQ31Z"  =  "1"  /* Removal of Radioactive Element from Respiratory Tract, Percutaneous Approach */
"0WPQ33Z"  =  "1"  /* Removal of Infusion Device from Respiratory Tract, Percutaneous Approach */
"0WPQ3YZ"  =  "1"  /* Removal of Other Device from Respiratory Tract, Percutaneous Approach */
"0WPQ41Z"  =  "1"  /* Removal of Radioactive Element from Respiratory Tract, Percutaneous Endoscopic Approach */
"0WPQ43Z"  =  "1"  /* Removal of Infusion Device from Respiratory Tract, Percutaneous Endoscopic Approach */
"0WPQ4YZ"  =  "1"  /* Removal of Other Device from Respiratory Tract, Percutaneous Endoscopic Approach */
"0WPQ71Z"  =  "1"  /* Removal of Radioactive Element from Respiratory Tract, Via Natural or Artificial Opening */
"0WPQ73Z"  =  "1"  /* Removal of Infusion Device from Respiratory Tract, Via Natural or Artificial Opening */
"0WPQ7YZ"  =  "1"  /* Removal of Other Device from Respiratory Tract, Via Natural or Artificial Opening */
"0WPQ81Z"  =  "1"  /* Removal of Radioactive Element from Respiratory Tract, Via Natural or Artificial Opening Endoscopic */
"0WQ6XZ2"  =  "1"  /* Repair Neck, Stoma, External Approach */
"0WQ80ZZ"  =  "1"  /* Repair Chest Wall, Open Approach */
"0WQ83ZZ"  =  "1"  /* Repair Chest Wall, Percutaneous Approach */
"0WQ84ZZ"  =  "1"  /* Repair Chest Wall, Percutaneous Endoscopic Approach */
"0WQ8XZZ"  =  "1"  /* Repair Chest Wall, External Approach */
"0WQC0ZZ"  =  "1"  /* Repair Mediastinum, Open Approach */
"0WQC3ZZ"  =  "1"  /* Repair Mediastinum, Percutaneous Approach */
"0WQC4ZZ"  =  "1"  /* Repair Mediastinum, Percutaneous Endoscopic Approach */
"0WU807Z"  =  "1"  /* Supplement Chest Wall with Autologous Tissue Substitute, Open Approach */
"0WU80JZ"  =  "1"  /* Supplement Chest Wall with Synthetic Substitute, Open Approach */
"0WU80KZ"  =  "1"  /* Supplement Chest Wall with Nonautologous Tissue Substitute, Open Approach */
"0WU847Z"  =  "1"  /* Supplement Chest Wall with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0WU84JZ"  =  "1"  /* Supplement Chest Wall with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0WU84KZ"  =  "1"  /* Supplement Chest Wall with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0WUC07Z"  =  "1"  /* Supplement Mediastinum with Autologous Tissue Substitute, Open Approach */
"0WUC0JZ"  =  "1"  /* Supplement Mediastinum with Synthetic Substitute, Open Approach */
"0WUC0KZ"  =  "1"  /* Supplement Mediastinum with Nonautologous Tissue Substitute, Open Approach */
"0WUC47Z"  =  "1"  /* Supplement Mediastinum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0WUC4JZ"  =  "1"  /* Supplement Mediastinum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0WUC4KZ"  =  "1"  /* Supplement Mediastinum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0WW800Z"  =  "1"  /* Revision of Drainage Device in Chest Wall, Open Approach */
"0WW801Z"  =  "1"  /* Revision of Radioactive Element in Chest Wall, Open Approach */
"0WW803Z"  =  "1"  /* Revision of Infusion Device in Chest Wall, Open Approach */
"0WW807Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Chest Wall, Open Approach */
"0WW80JZ"  =  "1"  /* Revision of Synthetic Substitute in Chest Wall, Open Approach */
"0WW80KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Chest Wall, Open Approach */
"0WW80YZ"  =  "1"  /* Revision of Other Device in Chest Wall, Open Approach */
"0WW830Z"  =  "1"  /* Revision of Drainage Device in Chest Wall, Percutaneous Approach */
"0WW831Z"  =  "1"  /* Revision of Radioactive Element in Chest Wall, Percutaneous Approach */
"0WW833Z"  =  "1"  /* Revision of Infusion Device in Chest Wall, Percutaneous Approach */
"0WW837Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Chest Wall, Percutaneous Approach */
"0WW83JZ"  =  "1"  /* Revision of Synthetic Substitute in Chest Wall, Percutaneous Approach */
"0WW83KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Chest Wall, Percutaneous Approach */
"0WW83YZ"  =  "1"  /* Revision of Other Device in Chest Wall, Percutaneous Approach */
"0WW840Z"  =  "1"  /* Revision of Drainage Device in Chest Wall, Percutaneous Endoscopic Approach */
"0WW841Z"  =  "1"  /* Revision of Radioactive Element in Chest Wall, Percutaneous Endoscopic Approach */
"0WW843Z"  =  "1"  /* Revision of Infusion Device in Chest Wall, Percutaneous Endoscopic Approach */
"0WW847Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Chest Wall, Percutaneous Endoscopic Approach */
"0WW84JZ"  =  "1"  /* Revision of Synthetic Substitute in Chest Wall, Percutaneous Endoscopic Approach */
"0WW84KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Chest Wall, Percutaneous Endoscopic Approach */
"0WW84YZ"  =  "1"  /* Revision of Other Device in Chest Wall, Percutaneous Endoscopic Approach */
"0WW900Z"  =  "1"  /* Revision of Drainage Device in Right Pleural Cavity, Open Approach */
"0WW901Z"  =  "1"  /* Revision of Radioactive Element in Right Pleural Cavity, Open Approach */
"0WW903Z"  =  "1"  /* Revision of Infusion Device in Right Pleural Cavity, Open Approach */
"0WW90JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Pleural Cavity, Open Approach */
"0WW90YZ"  =  "1"  /* Revision of Other Device in Right Pleural Cavity, Open Approach */
"0WW930Z"  =  "1"  /* Revision of Drainage Device in Right Pleural Cavity, Percutaneous Approach */
"0WW931Z"  =  "1"  /* Revision of Radioactive Element in Right Pleural Cavity, Percutaneous Approach */
"0WW933Z"  =  "1"  /* Revision of Infusion Device in Right Pleural Cavity, Percutaneous Approach */
"0WW93JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Pleural Cavity, Percutaneous Approach */
"0WW93YZ"  =  "1"  /* Revision of Other Device in Right Pleural Cavity, Percutaneous Approach */
"0WW940Z"  =  "1"  /* Revision of Drainage Device in Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WW941Z"  =  "1"  /* Revision of Radioactive Element in Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WW943Z"  =  "1"  /* Revision of Infusion Device in Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WW94JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WW94YZ"  =  "1"  /* Revision of Other Device in Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WWB00Z"  =  "1"  /* Revision of Drainage Device in Left Pleural Cavity, Open Approach */
"0WWB01Z"  =  "1"  /* Revision of Radioactive Element in Left Pleural Cavity, Open Approach */
"0WWB03Z"  =  "1"  /* Revision of Infusion Device in Left Pleural Cavity, Open Approach */
"0WWB0JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Pleural Cavity, Open Approach */
"0WWB0YZ"  =  "1"  /* Revision of Other Device in Left Pleural Cavity, Open Approach */
"0WWB30Z"  =  "1"  /* Revision of Drainage Device in Left Pleural Cavity, Percutaneous Approach */
"0WWB31Z"  =  "1"  /* Revision of Radioactive Element in Left Pleural Cavity, Percutaneous Approach */
"0WWB33Z"  =  "1"  /* Revision of Infusion Device in Left Pleural Cavity, Percutaneous Approach */
"0WWB3JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Pleural Cavity, Percutaneous Approach */
"0WWB3YZ"  =  "1"  /* Revision of Other Device in Left Pleural Cavity, Percutaneous Approach */
"0WWB40Z"  =  "1"  /* Revision of Drainage Device in Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WWB41Z"  =  "1"  /* Revision of Radioactive Element in Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WWB43Z"  =  "1"  /* Revision of Infusion Device in Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WWB4JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WWB4YZ"  =  "1"  /* Revision of Other Device in Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WWC00Z"  =  "1"  /* Revision of Drainage Device in Mediastinum, Open Approach */
"0WWC01Z"  =  "1"  /* Revision of Radioactive Element in Mediastinum, Open Approach */
"0WWC03Z"  =  "1"  /* Revision of Infusion Device in Mediastinum, Open Approach */
"0WWC07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mediastinum, Open Approach */
"0WWC0JZ"  =  "1"  /* Revision of Synthetic Substitute in Mediastinum, Open Approach */
"0WWC0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mediastinum, Open Approach */
"0WWC0YZ"  =  "1"  /* Revision of Other Device in Mediastinum, Open Approach */
"0WWC30Z"  =  "1"  /* Revision of Drainage Device in Mediastinum, Percutaneous Approach */
"0WWC31Z"  =  "1"  /* Revision of Radioactive Element in Mediastinum, Percutaneous Approach */
"0WWC33Z"  =  "1"  /* Revision of Infusion Device in Mediastinum, Percutaneous Approach */
"0WWC37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mediastinum, Percutaneous Approach */
"0WWC3JZ"  =  "1"  /* Revision of Synthetic Substitute in Mediastinum, Percutaneous Approach */
"0WWC3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mediastinum, Percutaneous Approach */
"0WWC3YZ"  =  "1"  /* Revision of Other Device in Mediastinum, Percutaneous Approach */
"0WWC40Z"  =  "1"  /* Revision of Drainage Device in Mediastinum, Percutaneous Endoscopic Approach */
"0WWC41Z"  =  "1"  /* Revision of Radioactive Element in Mediastinum, Percutaneous Endoscopic Approach */
"0WWC43Z"  =  "1"  /* Revision of Infusion Device in Mediastinum, Percutaneous Endoscopic Approach */
"0WWC47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mediastinum, Percutaneous Endoscopic Approach */
"0WWC4JZ"  =  "1"  /* Revision of Synthetic Substitute in Mediastinum, Percutaneous Endoscopic Approach */
"0WWC4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mediastinum, Percutaneous Endoscopic Approach */
"0WWC4YZ"  =  "1"  /* Revision of Other Device in Mediastinum, Percutaneous Endoscopic Approach */
"0WWQ01Z"  =  "1"  /* Revision of Radioactive Element in Respiratory Tract, Open Approach */
"0WWQ03Z"  =  "1"  /* Revision of Infusion Device in Respiratory Tract, Open Approach */
"0WWQ0YZ"  =  "1"  /* Revision of Other Device in Respiratory Tract, Open Approach */
"0WWQ31Z"  =  "1"  /* Revision of Radioactive Element in Respiratory Tract, Percutaneous Approach */
"0WWQ33Z"  =  "1"  /* Revision of Infusion Device in Respiratory Tract, Percutaneous Approach */
"0WWQ3YZ"  =  "1"  /* Revision of Other Device in Respiratory Tract, Percutaneous Approach */
"0WWQ41Z"  =  "1"  /* Revision of Radioactive Element in Respiratory Tract, Percutaneous Endoscopic Approach */
"0WWQ43Z"  =  "1"  /* Revision of Infusion Device in Respiratory Tract, Percutaneous Endoscopic Approach */
"0WWQ4YZ"  =  "1"  /* Revision of Other Device in Respiratory Tract, Percutaneous Endoscopic Approach */
"0WWQ71Z"  =  "1"  /* Revision of Radioactive Element in Respiratory Tract, Via Natural or Artificial Opening */
"0WWQ73Z"  =  "1"  /* Revision of Infusion Device in Respiratory Tract, Via Natural or Artificial Opening */
"0WWQ7YZ"  =  "1"  /* Revision of Other Device in Respiratory Tract, Via Natural or Artificial Opening */
"0WWQ81Z"  =  "1"  /* Revision of Radioactive Element in Respiratory Tract, Via Natural or Artificial Opening Endoscopic */
"0WWQ83Z"  =  "1"  /* Revision of Infusion Device in Respiratory Tract, Via Natural or Artificial Opening Endoscopic */
"0WWQ8YZ"  =  "1"  /* Revision of Other Device in Respiratory Tract, Via Natural or Artificial Opening Endoscopic */
"3E0L3SF"  =  "1"  /* Introduction of Other Gas into Pleural Cavity, Percutaneous Approach */
"3E0L7SF"  =  "1"  /* Introduction of Other Gas into Pleural Cavity, Via Natural or Artificial Opening */
"3E0M3SF"  =  "1"  /* Introduction of Other Gas into Peritoneal Cavity, Percutaneous Approach */
OTHER    = "0" ;

 /* EXCLUDE: LUNG CANCER PROCEDURE */
 /* ICD-10-CM PROCEDURE CODES: */

VALUE $LUNGCIP
"0BBC4ZZ"  =  "1"  /* Excision of Right Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0BBD4ZZ"  =  "1"  /* Excision of Right Middle Lung Lobe, Percutaneous Endoscopic Approach */
"0BBF4ZZ"  =  "1"  /* Excision of Right Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0BBG4ZZ"  =  "1"  /* Excision of Left Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0BBH4ZZ"  =  "1"  /* Excision of Lung Lingula, Percutaneous Endoscopic Approach */
"0BBJ4ZZ"  =  "1"  /* Excision of Left Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0BBK0ZZ"  =  "1"  /* Excision of Right Lung, Open Approach */
"0BBK3ZZ"  =  "1"  /* Excision of Right Lung, Percutaneous Approach */
"0BBK4ZZ"  =  "1"  /* Excision of Right Lung, Percutaneous Endoscopic Approach */
"0BBK7ZZ"  =  "1"  /* Excision of Right Lung, Via Natural or Artificial Opening */
"0BBL0ZZ"  =  "1"  /* Excision of Left Lung, Open Approach */
"0BBL3ZZ"  =  "1"  /* Excision of Left Lung, Percutaneous Approach */
"0BBL4ZZ"  =  "1"  /* Excision of Left Lung, Percutaneous Endoscopic Approach */
"0BBL7ZZ"  =  "1"  /* Excision of Left Lung, Via Natural or Artificial Opening */
"0BTC0ZZ"  =  "1"  /* Resection of Right Upper Lung Lobe, Open Approach */
"0BTC4ZZ"  =  "1"  /* Resection of Right Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0BTD0ZZ"  =  "1"  /* Resection of Right Middle Lung Lobe, Open Approach */
"0BTD4ZZ"  =  "1"  /* Resection of Right Middle Lung Lobe, Percutaneous Endoscopic Approach */
"0BTF0ZZ"  =  "1"  /* Resection of Right Lower Lung Lobe, Open Approach */
"0BTF4ZZ"  =  "1"  /* Resection of Right Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0BTG0ZZ"  =  "1"  /* Resection of Left Upper Lung Lobe, Open Approach */
"0BTG4ZZ"  =  "1"  /* Resection of Left Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0BTH0ZZ"  =  "1"  /* Resection of Lung Lingula, Open Approach */
"0BTH4ZZ"  =  "1"  /* Resection of Lung Lingula, Percutaneous Endoscopic Approach */
"0BTJ0ZZ"  =  "1"  /* Resection of Left Lower Lung Lobe, Open Approach */
"0BTJ4ZZ"  =  "1"  /* Resection of Left Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0BTK0ZZ"  =  "1"  /* Resection of Right Lung, Open Approach */
"0BTK4ZZ"  =  "1"  /* Resection of Right Lung, Percutaneous Endoscopic Approach */
"0BTL0ZZ"  =  "1"  /* Resection of Left Lung, Open Approach */
"0BTL4ZZ"  =  "1"  /* Resection of Left Lung, Percutaneous Endoscopic Approach */
"0BTM0ZZ"  =  "1"  /* Resection of Bilateral Lungs, Open Approach */
"0BTM4ZZ"  =  "1"  /* Resection of Bilateral Lungs, Percutaneous Endoscopic Approach */
OTHER  = "0" ;


 /* EXCLUDE: LUNG OR PLEURAL BIOPSY */
 /* ICD-10-CM PROCEDURE CODES: */

VALUE $LUNGBIP
"0B9C0ZX"  =  "1"  /* Drainage of Right Upper Lung Lobe, Open Approach, Diagnostic */
"0B9C3ZX"  =  "1"  /* Drainage of Right Upper Lung Lobe, Percutaneous Approach, Diagnostic */
"0B9C4ZX"  =  "1"  /* Drainage of Right Upper Lung Lobe, Percutaneous Endoscopic Approach, Diagnostic */
"0B9C7ZX"  =  "1"  /* Drainage of Right Upper Lung Lobe, Via Natural or Artificial Opening, Diagnostic */
"0B9D0ZX"  =  "1"  /* Drainage of Right Middle Lung Lobe, Open Approach, Diagnostic */
"0B9D3ZX"  =  "1"  /* Drainage of Right Middle Lung Lobe, Percutaneous Approach, Diagnostic */
"0B9D4ZX"  =  "1"  /* Drainage of Right Middle Lung Lobe, Percutaneous Endoscopic Approach, Diagnostic */
"0B9D7ZX"  =  "1"  /* Drainage of Right Middle Lung Lobe, Via Natural or Artificial Opening, Diagnostic */
"0B9F0ZX"  =  "1"  /* Drainage of Right Lower Lung Lobe, Open Approach, Diagnostic */
"0B9F3ZX"  =  "1"  /* Drainage of Right Lower Lung Lobe, Percutaneous Approach, Diagnostic */
"0B9F4ZX"  =  "1"  /* Drainage of Right Lower Lung Lobe, Percutaneous Endoscopic Approach, Diagnostic */
"0B9F7ZX"  =  "1"  /* Drainage of Right Lower Lung Lobe, Via Natural or Artificial Opening, Diagnostic */
"0B9G0ZX"  =  "1"  /* Drainage of Left Upper Lung Lobe, Open Approach, Diagnostic */
"0B9G3ZX"  =  "1"  /* Drainage of Left Upper Lung Lobe, Percutaneous Approach, Diagnostic */
"0B9G4ZX"  =  "1"  /* Drainage of Left Upper Lung Lobe, Percutaneous Endoscopic Approach, Diagnostic */
"0B9G7ZX"  =  "1"  /* Drainage of Left Upper Lung Lobe, Via Natural or Artificial Opening, Diagnostic */
"0B9H0ZX"  =  "1"  /* Drainage of Lung Lingula, Open Approach, Diagnostic */
"0B9H3ZX"  =  "1"  /* Drainage of Lung Lingula, Percutaneous Approach, Diagnostic */
"0B9H4ZX"  =  "1"  /* Drainage of Lung Lingula, Percutaneous Endoscopic Approach, Diagnostic */
"0B9H7ZX"  =  "1"  /* Drainage of Lung Lingula, Via Natural or Artificial Opening, Diagnostic */
"0B9J0ZX"  =  "1"  /* Drainage of Left Lower Lung Lobe, Open Approach, Diagnostic */
"0B9J3ZX"  =  "1"  /* Drainage of Left Lower Lung Lobe, Percutaneous Approach, Diagnostic */
"0B9J4ZX"  =  "1"  /* Drainage of Left Lower Lung Lobe, Percutaneous Endoscopic Approach, Diagnostic */
"0B9J7ZX"  =  "1"  /* Drainage of Left Lower Lung Lobe, Via Natural or Artificial Opening, Diagnostic */
"0B9K0ZX"  =  "1"  /* Drainage of Right Lung, Open Approach, Diagnostic */
"0B9K3ZX"  =  "1"  /* Drainage of Right Lung, Percutaneous Approach, Diagnostic */
"0B9K4ZX"  =  "1"  /* Drainage of Right Lung, Percutaneous Endoscopic Approach, Diagnostic */
"0B9K7ZX"  =  "1"  /* Drainage of Right Lung, Via Natural or Artificial Opening, Diagnostic */
"0B9L0ZX"  =  "1"  /* Drainage of Left Lung, Open Approach, Diagnostic */
"0B9L3ZX"  =  "1"  /* Drainage of Left Lung, Percutaneous Approach, Diagnostic */
"0B9L4ZX"  =  "1"  /* Drainage of Left Lung, Percutaneous Endoscopic Approach, Diagnostic */
"0B9L7ZX"  =  "1"  /* Drainage of Left Lung, Via Natural or Artificial Opening, Diagnostic */
"0B9M0ZX"  =  "1"  /* Drainage of Bilateral Lungs, Open Approach, Diagnostic */
"0B9M3ZX"  =  "1"  /* Drainage of Bilateral Lungs, Percutaneous Approach, Diagnostic */
"0B9M4ZX"  =  "1"  /* Drainage of Bilateral Lungs, Percutaneous Endoscopic Approach, Diagnostic */
"0B9M7ZX"  =  "1"  /* Drainage of Bilateral Lungs, Via Natural or Artificial Opening, Diagnostic */
"0B9N0ZX"  =  "1"  /* Drainage of Right Pleura, Open Approach, Diagnostic */
"0B9N3ZX"  =  "1"  /* Drainage of Right Pleura, Percutaneous Approach, Diagnostic */
"0B9N4ZX"  =  "1"  /* Drainage of Right Pleura, Percutaneous Endoscopic Approach, Diagnostic */
"0B9P0ZX"  =  "1"  /* Drainage of Left Pleura, Open Approach, Diagnostic */
"0B9P3ZX"  =  "1"  /* Drainage of Left Pleura, Percutaneous Approach, Diagnostic */
"0B9P4ZX"  =  "1"  /* Drainage of Left Pleura, Percutaneous Endoscopic Approach, Diagnostic */
"0BBC0ZX"  =  "1"  /* Excision of Right Upper Lung Lobe, Open Approach, Diagnostic */
"0BBC3ZX"  =  "1"  /* Excision of Right Upper Lung Lobe, Percutaneous Approach, Diagnostic */
"0BBD0ZX"  =  "1"  /* Excision of Right Middle Lung Lobe, Open Approach, Diagnostic */
"0BBD3ZX"  =  "1"  /* Excision of Right Middle Lung Lobe, Percutaneous Approach, Diagnostic */
"0BBF0ZX"  =  "1"  /* Excision of Right Lower Lung Lobe, Open Approach, Diagnostic */
"0BBF3ZX"  =  "1"  /* Excision of Right Lower Lung Lobe, Percutaneous Approach, Diagnostic */
"0BBG0ZX"  =  "1"  /* Excision of Left Upper Lung Lobe, Open Approach, Diagnostic */
"0BBG3ZX"  =  "1"  /* Excision of Left Upper Lung Lobe, Percutaneous Approach, Diagnostic */
"0BBH0ZX"  =  "1"  /* Excision of Lung Lingula, Open Approach, Diagnostic */
"0BBH3ZX"  =  "1"  /* Excision of Lung Lingula, Percutaneous Approach, Diagnostic */
"0BBJ0ZX"  =  "1"  /* Excision of Left Lower Lung Lobe, Open Approach, Diagnostic */
"0BBJ3ZX"  =  "1"  /* Excision of Left Lower Lung Lobe, Percutaneous Approach, Diagnostic */
"0BBK0ZX"  =  "1"  /* Excision of Right Lung, Open Approach, Diagnostic */
"0BBK3ZX"  =  "1"  /* Excision of Right Lung, Percutaneous Approach, Diagnostic */
"0BBL0ZX"  =  "1"  /* Excision of Left Lung, Open Approach, Diagnostic */
"0BBL3ZX"  =  "1"  /* Excision of Left Lung, Percutaneous Approach, Diagnostic */
"0BBM0ZX"  =  "1"  /* Excision of Bilateral Lungs, Open Approach, Diagnostic */
"0BBM3ZX"  =  "1"  /* Excision of Bilateral Lungs, Percutaneous Approach, Diagnostic */
"0BBN0ZX"  =  "1"  /* Excision of Right Pleura, Open Approach, Diagnostic */
"0BBN3ZX"  =  "1"  /* Excision of Right Pleura, Percutaneous Approach, Diagnostic */
"0BBP0ZX"  =  "1"  /* Excision of Left Pleura, Open Approach, Diagnostic */
"0BBP3ZX"  =  "1"  /* Excision of Left Pleura, Percutaneous Approach, Diagnostic */
"0W990ZX"  =  "1"  /* Drainage of Right Pleural Cavity, Open Approach, Diagnostic */
"0W993ZX"  =  "1"  /* Drainage of Right Pleural Cavity, Percutaneous Approach, Diagnostic */
"0W994ZX"  =  "1"  /* Drainage of Right Pleural Cavity, Percutaneous Endoscopic Approach, Diagnostic */
"0W9B0ZX"  =  "1"  /* Drainage of Left Pleural Cavity, Open Approach, Diagnostic */
"0W9B3ZX"  =  "1"  /* Drainage of Left Pleural Cavity, Percutaneous Approach, Diagnostic */
"0W9B4ZX"  =  "1"  /* Drainage of Left Pleural Cavity, Percutaneous Endoscopic Approach, Diagnostic */
OTHER  = "0" ;


 /* EXCLUDE: CARDIAC PROCEDURE */
 /* ICD-10-CM PROCEDURE CODES: */

VALUE $CARDSIP
"0210093"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Artery with Autologous Venous Tissue, Open Approach */
"0210098"  =  "1"  /* Bypass Coronary Artery, One Site from Right Internal Mammary with Autologous Venous Tissue, Open Approach */
"0210099"  =  "1"  /* Bypass Coronary Artery, One Site from Left Internal Mammary with Autologous Venous Tissue, Open Approach */
"021009C"  =  "1"  /* Bypass Coronary Artery, One Site from Thoracic Artery with Autologous Venous Tissue, Open Approach */
"021009F"  =  "1"  /* Bypass Coronary Artery, One Site from Abdominal Artery with Autologous Venous Tissue, Open Approach */
"021009W"  =  "1"  /* Bypass Coronary Artery, One Site from Aorta with Autologous Venous Tissue, Open Approach */
"02100A3"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Artery with Autologous Arterial Tissue, Open Approach */
"02100A8"  =  "1"  /* Bypass Coronary Artery, One Site from Right Internal Mammary with Autologous Arterial Tissue, Open Approach */
"02100A9"  =  "1"  /* Bypass Coronary Artery, One Site from Left Internal Mammary with Autologous Arterial Tissue, Open Approach */
"02100AC"  =  "1"  /* Bypass Coronary Artery, One Site from Thoracic Artery with Autologous Arterial Tissue, Open Approach */
"02100AF"  =  "1"  /* Bypass Coronary Artery, One Site from Abdominal Artery with Autologous Arterial Tissue, Open Approach */
"02100AW"  =  "1"  /* Bypass Coronary Artery, One Site from Aorta with Autologous Arterial Tissue, Open Approach */
"02100J3"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Artery with Synthetic Substitute, Open Approach */
"02100J8"  =  "1"  /* Bypass Coronary Artery, One Site from Right Internal Mammary with Synthetic Substitute, Open Approach */
"02100J9"  =  "1"  /* Bypass Coronary Artery, One Site from Left Internal Mammary with Synthetic Substitute, Open Approach */
"02100JC"  =  "1"  /* Bypass Coronary Artery, One Site from Thoracic Artery with Synthetic Substitute, Open Approach */
"02100JF"  =  "1"  /* Bypass Coronary Artery, One Site from Abdominal Artery with Synthetic Substitute, Open Approach */
"02100JW"  =  "1"  /* Bypass Coronary Artery, One Site from Aorta with Synthetic Substitute, Open Approach */
"02100K3"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Artery with Nonautologous Tissue Substitute, Open Approach */
"02100K8"  =  "1"  /* Bypass Coronary Artery, One Site from Right Internal Mammary with Nonautologous Tissue Substitute, Open Approach */
"02100K9"  =  "1"  /* Bypass Coronary Artery, One Site from Left Internal Mammary with Nonautologous Tissue Substitute, Open Approach */
"02100KC"  =  "1"  /* Bypass Coronary Artery, One Site from Thoracic Artery with Nonautologous Tissue Substitute, Open Approach */
"02100KF"  =  "1"  /* Bypass Coronary Artery, One Site from Abdominal Artery with Nonautologous Tissue Substitute, Open Approach */
"02100KW"  =  "1"  /* Bypass Coronary Artery, One Site from Aorta with Nonautologous Tissue Substitute, Open Approach */
"02100Z3"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Artery, Open Approach */
"02100Z8"  =  "1"  /* Bypass Coronary Artery, One Site from Right Internal Mammary, Open Approach */
"02100Z9"  =  "1"  /* Bypass Coronary Artery, One Site from Left Internal Mammary, Open Approach */
"02100ZC"  =  "1"  /* Bypass Coronary Artery, One Site from Thoracic Artery, Open Approach */
"02100ZF"  =  "1"  /* Bypass Coronary Artery, One Site from Abdominal Artery, Open Approach */
"0210444"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"0210493"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0210498"  =  "1"  /* Bypass Coronary Artery, One Site from Right Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0210499"  =  "1"  /* Bypass Coronary Artery, One Site from Left Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021049C"  =  "1"  /* Bypass Coronary Artery, One Site from Thoracic Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021049F"  =  "1"  /* Bypass Coronary Artery, One Site from Abdominal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021049W"  =  "1"  /* Bypass Coronary Artery, One Site from Aorta with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"02104A3"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02104A8"  =  "1"  /* Bypass Coronary Artery, One Site from Right Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02104A9"  =  "1"  /* Bypass Coronary Artery, One Site from Left Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02104AC"  =  "1"  /* Bypass Coronary Artery, One Site from Thoracic Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02104AF"  =  "1"  /* Bypass Coronary Artery, One Site from Abdominal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02104AW"  =  "1"  /* Bypass Coronary Artery, One Site from Aorta with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02104D4"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"02104J3"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02104J8"  =  "1"  /* Bypass Coronary Artery, One Site from Right Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02104J9"  =  "1"  /* Bypass Coronary Artery, One Site from Left Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02104JC"  =  "1"  /* Bypass Coronary Artery, One Site from Thoracic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02104JF"  =  "1"  /* Bypass Coronary Artery, One Site from Abdominal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02104JW"  =  "1"  /* Bypass Coronary Artery, One Site from Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02104K3"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02104K8"  =  "1"  /* Bypass Coronary Artery, One Site from Right Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02104K9"  =  "1"  /* Bypass Coronary Artery, One Site from Left Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02104KC"  =  "1"  /* Bypass Coronary Artery, One Site from Thoracic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02104KF"  =  "1"  /* Bypass Coronary Artery, One Site from Abdominal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02104KW"  =  "1"  /* Bypass Coronary Artery, One Site from Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02104Z3"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Artery, Percutaneous Endoscopic Approach */
"02104Z8"  =  "1"  /* Bypass Coronary Artery, One Site from Right Internal Mammary, Percutaneous Endoscopic Approach */
"02104Z9"  =  "1"  /* Bypass Coronary Artery, One Site from Left Internal Mammary, Percutaneous Endoscopic Approach */
"02104ZC"  =  "1"  /* Bypass Coronary Artery, One Site from Thoracic Artery, Percutaneous Endoscopic Approach */
"02104ZF"  =  "1"  /* Bypass Coronary Artery, One Site from Abdominal Artery, Percutaneous Endoscopic Approach */
"0211093"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Artery with Autologous Venous Tissue, Open Approach */
"0211098"  =  "1"  /* Bypass Coronary Artery, Two Sites from Right Internal Mammary with Autologous Venous Tissue, Open Approach */
"0211099"  =  "1"  /* Bypass Coronary Artery, Two Sites from Left Internal Mammary with Autologous Venous Tissue, Open Approach */
"021109C"  =  "1"  /* Bypass Coronary Artery, Two Sites from Thoracic Artery with Autologous Venous Tissue, Open Approach */
"021109F"  =  "1"  /* Bypass Coronary Artery, Two Sites from Abdominal Artery with Autologous Venous Tissue, Open Approach */
"021109W"  =  "1"  /* Bypass Coronary Artery, Two Sites from Aorta with Autologous Venous Tissue, Open Approach */
"02110A3"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Artery with Autologous Arterial Tissue, Open Approach */
"02110A8"  =  "1"  /* Bypass Coronary Artery, Two Sites from Right Internal Mammary with Autologous Arterial Tissue, Open Approach */
"02110A9"  =  "1"  /* Bypass Coronary Artery, Two Sites from Left Internal Mammary with Autologous Arterial Tissue, Open Approach */
"02110AC"  =  "1"  /* Bypass Coronary Artery, Two Sites from Thoracic Artery with Autologous Arterial Tissue, Open Approach */
"02110AF"  =  "1"  /* Bypass Coronary Artery, Two Sites from Abdominal Artery with Autologous Arterial Tissue, Open Approach */
"02110AW"  =  "1"  /* Bypass Coronary Artery, Two Sites from Aorta with Autologous Arterial Tissue, Open Approach */
"02110J3"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Artery with Synthetic Substitute, Open Approach */
"02110J8"  =  "1"  /* Bypass Coronary Artery, Two Sites from Right Internal Mammary with Synthetic Substitute, Open Approach */
"02110J9"  =  "1"  /* Bypass Coronary Artery, Two Sites from Left Internal Mammary with Synthetic Substitute, Open Approach */
"02110JC"  =  "1"  /* Bypass Coronary Artery, Two Sites from Thoracic Artery with Synthetic Substitute, Open Approach */
"02110JF"  =  "1"  /* Bypass Coronary Artery, Two Sites from Abdominal Artery with Synthetic Substitute, Open Approach */
"02110JW"  =  "1"  /* Bypass Coronary Artery, Two Sites from Aorta with Synthetic Substitute, Open Approach */
"02110K3"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Artery with Nonautologous Tissue Substitute, Open Approach */
"02110K8"  =  "1"  /* Bypass Coronary Artery, Two Sites from Right Internal Mammary with Nonautologous Tissue Substitute, Open Approach */
"02110K9"  =  "1"  /* Bypass Coronary Artery, Two Sites from Left Internal Mammary with Nonautologous Tissue Substitute, Open Approach */
"02110KC"  =  "1"  /* Bypass Coronary Artery, Two Sites from Thoracic Artery with Nonautologous Tissue Substitute, Open Approach */
"02110KF"  =  "1"  /* Bypass Coronary Artery, Two Sites from Abdominal Artery with Nonautologous Tissue Substitute, Open Approach */
"02110KW"  =  "1"  /* Bypass Coronary Artery, Two Sites from Aorta with Nonautologous Tissue Substitute, Open Approach */
"02110Z3"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Artery, Open Approach */
"02110Z8"  =  "1"  /* Bypass Coronary Artery, Two Sites from Right Internal Mammary, Open Approach */
"02110Z9"  =  "1"  /* Bypass Coronary Artery, Two Sites from Left Internal Mammary, Open Approach */
"02110ZC"  =  "1"  /* Bypass Coronary Artery, Two Sites from Thoracic Artery, Open Approach */
"02110ZF"  =  "1"  /* Bypass Coronary Artery, Two Sites from Abdominal Artery, Open Approach */
"0211444"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"0211493"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0211498"  =  "1"  /* Bypass Coronary Artery, Two Sites from Right Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0211499"  =  "1"  /* Bypass Coronary Artery, Two Sites from Left Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021149C"  =  "1"  /* Bypass Coronary Artery, Two Sites from Thoracic Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021149F"  =  "1"  /* Bypass Coronary Artery, Two Sites from Abdominal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021149W"  =  "1"  /* Bypass Coronary Artery, Two Sites from Aorta with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"02114A3"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02114A8"  =  "1"  /* Bypass Coronary Artery, Two Sites from Right Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02114A9"  =  "1"  /* Bypass Coronary Artery, Two Sites from Left Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02114AC"  =  "1"  /* Bypass Coronary Artery, Two Sites from Thoracic Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02114AF"  =  "1"  /* Bypass Coronary Artery, Two Sites from Abdominal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02114AW"  =  "1"  /* Bypass Coronary Artery, Two Sites from Aorta with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02114D4"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"02114J3"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02114J8"  =  "1"  /* Bypass Coronary Artery, Two Sites from Right Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02114J9"  =  "1"  /* Bypass Coronary Artery, Two Sites from Left Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02114JC"  =  "1"  /* Bypass Coronary Artery, Two Sites from Thoracic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02114JF"  =  "1"  /* Bypass Coronary Artery, Two Sites from Abdominal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02114JW"  =  "1"  /* Bypass Coronary Artery, Two Sites from Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02114K3"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02114K8"  =  "1"  /* Bypass Coronary Artery, Two Sites from Right Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02114K9"  =  "1"  /* Bypass Coronary Artery, Two Sites from Left Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02114KC"  =  "1"  /* Bypass Coronary Artery, Two Sites from Thoracic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02114KF"  =  "1"  /* Bypass Coronary Artery, Two Sites from Abdominal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02114KW"  =  "1"  /* Bypass Coronary Artery, Two Sites from Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02114Z3"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Artery, Percutaneous Endoscopic Approach */
"02114Z8"  =  "1"  /* Bypass Coronary Artery, Two Sites from Right Internal Mammary, Percutaneous Endoscopic Approach */
"02114Z9"  =  "1"  /* Bypass Coronary Artery, Two Sites from Left Internal Mammary, Percutaneous Endoscopic Approach */
"02114ZC"  =  "1"  /* Bypass Coronary Artery, Two Sites from Thoracic Artery, Percutaneous Endoscopic Approach */
"02114ZF"  =  "1"  /* Bypass Coronary Artery, Two Sites from Abdominal Artery, Percutaneous Endoscopic Approach */
"0212093"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Artery with Autologous Venous Tissue, Open Approach */
"0212098"  =  "1"  /* Bypass Coronary Artery, Three Sites from Right Internal Mammary with Autologous Venous Tissue, Open Approach */
"0212099"  =  "1"  /* Bypass Coronary Artery, Three Sites from Left Internal Mammary with Autologous Venous Tissue, Open Approach */
"021209C"  =  "1"  /* Bypass Coronary Artery, Three Sites from Thoracic Artery with Autologous Venous Tissue, Open Approach */
"021209F"  =  "1"  /* Bypass Coronary Artery, Three Sites from Abdominal Artery with Autologous Venous Tissue, Open Approach */
"021209W"  =  "1"  /* Bypass Coronary Artery, Three Sites from Aorta with Autologous Venous Tissue, Open Approach */
"02120A3"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Artery with Autologous Arterial Tissue, Open Approach */
"02120A8"  =  "1"  /* Bypass Coronary Artery, Three Sites from Right Internal Mammary with Autologous Arterial Tissue, Open Approach */
"02120A9"  =  "1"  /* Bypass Coronary Artery, Three Sites from Left Internal Mammary with Autologous Arterial Tissue, Open Approach */
"02120AC"  =  "1"  /* Bypass Coronary Artery, Three Sites from Thoracic Artery with Autologous Arterial Tissue, Open Approach */
"02120AF"  =  "1"  /* Bypass Coronary Artery, Three Sites from Abdominal Artery with Autologous Arterial Tissue, Open Approach */
"02120AW"  =  "1"  /* Bypass Coronary Artery, Three Sites from Aorta with Autologous Arterial Tissue, Open Approach */
"02120J3"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Artery with Synthetic Substitute, Open Approach */
"02120J8"  =  "1"  /* Bypass Coronary Artery, Three Sites from Right Internal Mammary with Synthetic Substitute, Open Approach */
"02120J9"  =  "1"  /* Bypass Coronary Artery, Three Sites from Left Internal Mammary with Synthetic Substitute, Open Approach */
"02120JC"  =  "1"  /* Bypass Coronary Artery, Three Sites from Thoracic Artery with Synthetic Substitute, Open Approach */
"02120JF"  =  "1"  /* Bypass Coronary Artery, Three Sites from Abdominal Artery with Synthetic Substitute, Open Approach */
"02120JW"  =  "1"  /* Bypass Coronary Artery, Three Sites from Aorta with Synthetic Substitute, Open Approach */
"02120K3"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Artery with Nonautologous Tissue Substitute, Open Approach */
"02120K8"  =  "1"  /* Bypass Coronary Artery, Three Sites from Right Internal Mammary with Nonautologous Tissue Substitute, Open Approach */
"02120K9"  =  "1"  /* Bypass Coronary Artery, Three Sites from Left Internal Mammary with Nonautologous Tissue Substitute, Open Approach */
"02120KC"  =  "1"  /* Bypass Coronary Artery, Three Sites from Thoracic Artery with Nonautologous Tissue Substitute, Open Approach */
"02120KF"  =  "1"  /* Bypass Coronary Artery, Three Sites from Abdominal Artery with Nonautologous Tissue Substitute, Open Approach */
"02120KW"  =  "1"  /* Bypass Coronary Artery, Three Sites from Aorta with Nonautologous Tissue Substitute, Open Approach */
"02120Z3"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Artery, Open Approach */
"02120Z8"  =  "1"  /* Bypass Coronary Artery, Three Sites from Right Internal Mammary, Open Approach */
"02120Z9"  =  "1"  /* Bypass Coronary Artery, Three Sites from Left Internal Mammary, Open Approach */
"02120ZC"  =  "1"  /* Bypass Coronary Artery, Three Sites from Thoracic Artery, Open Approach */
"02120ZF"  =  "1"  /* Bypass Coronary Artery, Three Sites from Abdominal Artery, Open Approach */
"0212444"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"0212493"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0212498"  =  "1"  /* Bypass Coronary Artery, Three Sites from Right Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0212499"  =  "1"  /* Bypass Coronary Artery, Three Sites from Left Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021249C"  =  "1"  /* Bypass Coronary Artery, Three Sites from Thoracic Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021249F"  =  "1"  /* Bypass Coronary Artery, Three Sites from Abdominal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021249W"  =  "1"  /* Bypass Coronary Artery, Three Sites from Aorta with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"02124A3"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02124A8"  =  "1"  /* Bypass Coronary Artery, Three Sites from Right Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02124A9"  =  "1"  /* Bypass Coronary Artery, Three Sites from Left Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02124AC"  =  "1"  /* Bypass Coronary Artery, Three Sites from Thoracic Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02124AF"  =  "1"  /* Bypass Coronary Artery, Three Sites from Abdominal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02124AW"  =  "1"  /* Bypass Coronary Artery, Three Sites from Aorta with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02124D4"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"02124J3"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02124J8"  =  "1"  /* Bypass Coronary Artery, Three Sites from Right Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02124J9"  =  "1"  /* Bypass Coronary Artery, Three Sites from Left Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02124JC"  =  "1"  /* Bypass Coronary Artery, Three Sites from Thoracic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02124JF"  =  "1"  /* Bypass Coronary Artery, Three Sites from Abdominal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02124JW"  =  "1"  /* Bypass Coronary Artery, Three Sites from Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02124K3"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02124K8"  =  "1"  /* Bypass Coronary Artery, Three Sites from Right Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02124K9"  =  "1"  /* Bypass Coronary Artery, Three Sites from Left Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02124KC"  =  "1"  /* Bypass Coronary Artery, Three Sites from Thoracic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02124KF"  =  "1"  /* Bypass Coronary Artery, Three Sites from Abdominal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02124KW"  =  "1"  /* Bypass Coronary Artery, Three Sites from Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02124Z3"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Artery, Percutaneous Endoscopic Approach */
"02124Z8"  =  "1"  /* Bypass Coronary Artery, Three Sites from Right Internal Mammary, Percutaneous Endoscopic Approach */
"02124Z9"  =  "1"  /* Bypass Coronary Artery, Three Sites from Left Internal Mammary, Percutaneous Endoscopic Approach */
"02124ZC"  =  "1"  /* Bypass Coronary Artery, Three Sites from Thoracic Artery, Percutaneous Endoscopic Approach */
"02124ZF"  =  "1"  /* Bypass Coronary Artery, Three Sites from Abdominal Artery, Percutaneous Endoscopic Approach */
"0213093"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Artery with Autologous Venous Tissue, Open Approach */
"0213098"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Right Internal Mammary with Autologous Venous Tissue, Open Approach */
"0213099"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Left Internal Mammary with Autologous Venous Tissue, Open Approach */
"021309C"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Thoracic Artery with Autologous Venous Tissue, Open Approach */
"021309F"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Abdominal Artery with Autologous Venous Tissue, Open Approach */
"021309W"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Aorta with Autologous Venous Tissue, Open Approach */
"02130A3"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Artery with Autologous Arterial Tissue, Open Approach */
"02130A8"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Right Internal Mammary with Autologous Arterial Tissue, Open Approach */
"02130A9"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Left Internal Mammary with Autologous Arterial Tissue, Open Approach */
"02130AC"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Thoracic Artery with Autologous Arterial Tissue, Open Approach */
"02130AF"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Abdominal Artery with Autologous Arterial Tissue, Open Approach */
"02130AW"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Aorta with Autologous Arterial Tissue, Open Approach */
"02130J3"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Artery with Synthetic Substitute, Open Approach */
"02130J8"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Right Internal Mammary with Synthetic Substitute, Open Approach */
"02130J9"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Left Internal Mammary with Synthetic Substitute, Open Approach */
"02130JC"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Thoracic Artery with Synthetic Substitute, Open Approach */
"02130JF"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Abdominal Artery with Synthetic Substitute, Open Approach */
"02130JW"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Aorta with Synthetic Substitute, Open Approach */
"02130K3"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Artery with Nonautologous Tissue Substitute, Open Approach */
"02130K8"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Right Internal Mammary with Nonautologous Tissue Substitute, Open Approach */
"02130K9"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Left Internal Mammary with Nonautologous Tissue Substitute, Open Approach */
"02130KC"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Thoracic Artery with Nonautologous Tissue Substitute, Open Approach */
"02130KF"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Abdominal Artery with Nonautologous Tissue Substitute, Open Approach */
"02130KW"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Aorta with Nonautologous Tissue Substitute, Open Approach */
"02130Z3"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Artery, Open Approach */
"02130Z8"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Right Internal Mammary, Open Approach */
"02130Z9"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Left Internal Mammary, Open Approach */
"02130ZC"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Thoracic Artery, Open Approach */
"02130ZF"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Abdominal Artery, Open Approach */
"0213444"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"0213493"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0213498"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Right Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0213499"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Left Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021349C"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Thoracic Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021349F"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Abdominal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021349W"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Aorta with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"02134A3"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02134A8"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Right Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02134A9"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Left Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02134AC"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Thoracic Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02134AF"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Abdominal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02134AW"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Aorta with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02134D4"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"02134J3"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02134J8"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Right Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02134J9"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Left Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02134JC"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Thoracic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02134JF"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Abdominal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02134JW"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02134K3"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02134K8"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Right Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02134K9"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Left Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02134KC"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Thoracic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02134KF"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Abdominal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02134KW"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02134Z3"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Artery, Percutaneous Endoscopic Approach */
"02134Z8"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Right Internal Mammary, Percutaneous Endoscopic Approach */
"02134Z9"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Left Internal Mammary, Percutaneous Endoscopic Approach */
"02134ZC"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Thoracic Artery, Percutaneous Endoscopic Approach */
"02134ZF"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Abdominal Artery, Percutaneous Endoscopic Approach */
"021609P"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk with Autologous Venous Tissue, Open Approach */
"021609Q"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021609R"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"02160AP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach */
"02160AQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"02160AR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"02160JP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk with Synthetic Substitute, Open Approach */
"02160JQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"02160JR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"02160KP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach */
"02160KQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"02160KR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"02160Z7"  =  "1"  /* Bypass Right Atrium to Left Atrium, Open Approach */
"02160ZP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk, Open Approach */
"02160ZQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery, Open Approach */
"02160ZR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery, Open Approach */
"021649P"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021649Q"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021649R"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"02164AP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02164AQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02164AR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02164JP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02164JQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02164JR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02164KP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02164KQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02164KR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02164Z7"  =  "1"  /* Bypass Right Atrium to Left Atrium, Percutaneous Endoscopic Approach */
"02164ZP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk, Percutaneous Endoscopic Approach */
"02164ZQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"02164ZR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"021709P"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk with Autologous Venous Tissue, Open Approach */
"021709Q"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021709R"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"02170AP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach */
"02170AQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"02170AR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"02170JP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk with Synthetic Substitute, Open Approach */
"02170JQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"02170JR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"02170KP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach */
"02170KQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"02170KR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"02170ZP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk, Open Approach */
"02170ZQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery, Open Approach */
"02170ZR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery, Open Approach */
"021749P"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021749Q"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021749R"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"02174AP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02174AQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02174AR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02174JP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02174JQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02174JR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02174KP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02174KQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02174KR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02174ZP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk, Percutaneous Endoscopic Approach */
"02174ZQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"02174ZR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"021K09P"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk with Autologous Venous Tissue, Open Approach */
"021K09Q"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021K09R"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021K0AP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach */
"021K0AQ"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"021K0AR"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"021K0JP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk with Synthetic Substitute, Open Approach */
"021K0JQ"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"021K0JR"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"021K0KP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach */
"021K0KQ"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"021K0KR"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"021K0Z5"  =  "1"  /* Bypass Right Ventricle to Coronary Circulation, Open Approach */
"021K0Z8"  =  "1"  /* Bypass Right Ventricle to Right Internal Mammary, Open Approach */
"021K0Z9"  =  "1"  /* Bypass Right Ventricle to Left Internal Mammary, Open Approach */
"021K0ZC"  =  "1"  /* Bypass Right Ventricle to Thoracic Artery, Open Approach */
"021K0ZF"  =  "1"  /* Bypass Right Ventricle to Abdominal Artery, Open Approach */
"021K0ZP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk, Open Approach */
"021K0ZQ"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery, Open Approach */
"021K0ZR"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery, Open Approach */
"021K0ZW"  =  "1"  /* Bypass Right Ventricle to Aorta, Open Approach */
"021K49P"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021K49Q"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021K49R"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021K4AP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021K4AQ"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021K4AR"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021K4JP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021K4JQ"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021K4JR"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021K4KP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021K4KQ"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021K4KR"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021K4Z5"  =  "1"  /* Bypass Right Ventricle to Coronary Circulation, Percutaneous Endoscopic Approach */
"021K4Z8"  =  "1"  /* Bypass Right Ventricle to Right Internal Mammary, Percutaneous Endoscopic Approach */
"021K4Z9"  =  "1"  /* Bypass Right Ventricle to Left Internal Mammary, Percutaneous Endoscopic Approach */
"021K4ZC"  =  "1"  /* Bypass Right Ventricle to Thoracic Artery, Percutaneous Endoscopic Approach */
"021K4ZF"  =  "1"  /* Bypass Right Ventricle to Abdominal Artery, Percutaneous Endoscopic Approach */
"021K4ZP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk, Percutaneous Endoscopic Approach */
"021K4ZQ"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"021K4ZR"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"021K4ZW"  =  "1"  /* Bypass Right Ventricle to Aorta, Percutaneous Endoscopic Approach */
"021L09P"  =  "1"  /* Bypass Left Ventricle to Pulmonary Trunk with Autologous Venous Tissue, Open Approach */
"021L09Q"  =  "1"  /* Bypass Left Ventricle to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021L09R"  =  "1"  /* Bypass Left Ventricle to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021L0AP"  =  "1"  /* Bypass Left Ventricle to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach */
"021L0AQ"  =  "1"  /* Bypass Left Ventricle to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"021L0AR"  =  "1"  /* Bypass Left Ventricle to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"021L0JP"  =  "1"  /* Bypass Left Ventricle to Pulmonary Trunk with Synthetic Substitute, Open Approach */
"021L0JQ"  =  "1"  /* Bypass Left Ventricle to Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"021L0JR"  =  "1"  /* Bypass Left Ventricle to Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"021L0KP"  =  "1"  /* Bypass Left Ventricle to Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach */
"021L0KQ"  =  "1"  /* Bypass Left Ventricle to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"021L0KR"  =  "1"  /* Bypass Left Ventricle to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"021L0Z5"  =  "1"  /* Bypass Left Ventricle to Coronary Circulation, Open Approach */
"021L0Z8"  =  "1"  /* Bypass Left Ventricle to Right Internal Mammary, Open Approach */
"021L0Z9"  =  "1"  /* Bypass Left Ventricle to Left Internal Mammary, Open Approach */
"021L0ZC"  =  "1"  /* Bypass Left Ventricle to Thoracic Artery, Open Approach */
"021L0ZF"  =  "1"  /* Bypass Left Ventricle to Abdominal Artery, Open Approach */
"021L0ZP"  =  "1"  /* Bypass Left Ventricle to Pulmonary Trunk, Open Approach */
"021L0ZQ"  =  "1"  /* Bypass Left Ventricle to Right Pulmonary Artery, Open Approach */
"021L0ZR"  =  "1"  /* Bypass Left Ventricle to Left Pulmonary Artery, Open Approach */
"021L0ZW"  =  "1"  /* Bypass Left Ventricle to Aorta, Open Approach */
"021L49P"  =  "1"  /* Bypass Left Ventricle to Pulmonary Trunk with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021L49Q"  =  "1"  /* Bypass Left Ventricle to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021L49R"  =  "1"  /* Bypass Left Ventricle to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021L4AP"  =  "1"  /* Bypass Left Ventricle to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021L4AQ"  =  "1"  /* Bypass Left Ventricle to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021L4AR"  =  "1"  /* Bypass Left Ventricle to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021L4JP"  =  "1"  /* Bypass Left Ventricle to Pulmonary Trunk with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021L4JQ"  =  "1"  /* Bypass Left Ventricle to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021L4JR"  =  "1"  /* Bypass Left Ventricle to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021L4KP"  =  "1"  /* Bypass Left Ventricle to Pulmonary Trunk with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021L4KQ"  =  "1"  /* Bypass Left Ventricle to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021L4KR"  =  "1"  /* Bypass Left Ventricle to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021L4Z5"  =  "1"  /* Bypass Left Ventricle to Coronary Circulation, Percutaneous Endoscopic Approach */
"021L4Z8"  =  "1"  /* Bypass Left Ventricle to Right Internal Mammary, Percutaneous Endoscopic Approach */
"021L4Z9"  =  "1"  /* Bypass Left Ventricle to Left Internal Mammary, Percutaneous Endoscopic Approach */
"021L4ZC"  =  "1"  /* Bypass Left Ventricle to Thoracic Artery, Percutaneous Endoscopic Approach */
"021L4ZF"  =  "1"  /* Bypass Left Ventricle to Abdominal Artery, Percutaneous Endoscopic Approach */
"021L4ZP"  =  "1"  /* Bypass Left Ventricle to Pulmonary Trunk, Percutaneous Endoscopic Approach */
"021L4ZQ"  =  "1"  /* Bypass Left Ventricle to Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"021L4ZR"  =  "1"  /* Bypass Left Ventricle to Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"021L4ZW"  =  "1"  /* Bypass Left Ventricle to Aorta, Percutaneous Endoscopic Approach */
"021V09P"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk with Autologous Venous Tissue, Open Approach */
"021V09Q"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021V09R"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021V0AP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach */
"021V0AQ"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"021V0AR"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"021V0JP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk with Synthetic Substitute, Open Approach */
"021V0JQ"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"021V0JR"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"021V0KP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach */
"021V0KQ"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"021V0KR"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"021V0ZP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk, Open Approach */
"021V0ZQ"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery, Open Approach */
"021V0ZR"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery, Open Approach */
"021V49P"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021V49Q"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021V49R"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021V4AP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021V4AQ"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021V4AR"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021V4JP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021V4JQ"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021V4JR"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021V4KP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021V4KQ"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021V4KR"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021V4ZP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk, Percutaneous Endoscopic Approach */
"021V4ZQ"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"021V4ZR"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"021W09B"  =  "1"  /* Bypass Thoracic Aorta to Subclavian with Autologous Venous Tissue, Open Approach */
"021W09D"  =  "1"  /* Bypass Thoracic Aorta to Carotid with Autologous Venous Tissue, Open Approach */
"021W09P"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk with Autologous Venous Tissue, Open Approach */
"021W09Q"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021W09R"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021W0AB"  =  "1"  /* Bypass Thoracic Aorta to Subclavian with Autologous Arterial Tissue, Open Approach */
"021W0AD"  =  "1"  /* Bypass Thoracic Aorta to Carotid with Autologous Arterial Tissue, Open Approach */
"021W0AP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach */
"021W0AQ"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"021W0AR"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"021W0JB"  =  "1"  /* Bypass Thoracic Aorta to Subclavian with Synthetic Substitute, Open Approach */
"021W0JD"  =  "1"  /* Bypass Thoracic Aorta to Carotid with Synthetic Substitute, Open Approach */
"021W0JP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk with Synthetic Substitute, Open Approach */
"021W0JQ"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"021W0JR"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"021W0KB"  =  "1"  /* Bypass Thoracic Aorta to Subclavian with Nonautologous Tissue Substitute, Open Approach */
"021W0KD"  =  "1"  /* Bypass Thoracic Aorta to Carotid with Nonautologous Tissue Substitute, Open Approach */
"021W0KP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach */
"021W0KQ"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"021W0KR"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"021W0ZB"  =  "1"  /* Bypass Thoracic Aorta to Subclavian, Open Approach */
"021W0ZD"  =  "1"  /* Bypass Thoracic Aorta to Carotid, Open Approach */
"021W0ZP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk, Open Approach */
"021W0ZQ"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery, Open Approach */
"021W0ZR"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery, Open Approach */
"021W49B"  =  "1"  /* Bypass Thoracic Aorta to Subclavian with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021W49D"  =  "1"  /* Bypass Thoracic Aorta to Carotid with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021W49P"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021W49Q"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021W49R"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021W4AB"  =  "1"  /* Bypass Thoracic Aorta to Subclavian with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021W4AD"  =  "1"  /* Bypass Thoracic Aorta to Carotid with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021W4AP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021W4AQ"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021W4AR"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021W4JB"  =  "1"  /* Bypass Thoracic Aorta to Subclavian with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021W4JD"  =  "1"  /* Bypass Thoracic Aorta to Carotid with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021W4JP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021W4JQ"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021W4JR"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021W4KB"  =  "1"  /* Bypass Thoracic Aorta to Subclavian with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021W4KD"  =  "1"  /* Bypass Thoracic Aorta to Carotid with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021W4KP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021W4KQ"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021W4KR"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021W4ZB"  =  "1"  /* Bypass Thoracic Aorta to Subclavian, Percutaneous Endoscopic Approach */
"021W4ZD"  =  "1"  /* Bypass Thoracic Aorta to Carotid, Percutaneous Endoscopic Approach */
"021W4ZP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk, Percutaneous Endoscopic Approach */
"021W4ZQ"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"021W4ZR"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"02540ZZ"  =  "1"  /* Destruction of Coronary Vein, Open Approach */
"02544ZZ"  =  "1"  /* Destruction of Coronary Vein, Percutaneous Endoscopic Approach */
"02550ZZ"  =  "1"  /* Destruction of Atrial Septum, Open Approach */
"02554ZZ"  =  "1"  /* Destruction of Atrial Septum, Percutaneous Endoscopic Approach */
"02560ZZ"  =  "1"  /* Destruction of Right Atrium, Open Approach */
"02564ZZ"  =  "1"  /* Destruction of Right Atrium, Percutaneous Endoscopic Approach */
"02570ZK"  =  "1"  /* Destruction of Left Atrial Appendage, Open Approach */
"02570ZZ"  =  "1"  /* Destruction of Left Atrium, Open Approach */
"02574ZK"  =  "1"  /* Destruction of Left Atrial Appendage, Percutaneous Endoscopic Approach */
"02574ZZ"  =  "1"  /* Destruction of Left Atrium, Percutaneous Endoscopic Approach */
"02580ZZ"  =  "1"  /* Destruction of Conduction Mechanism, Open Approach */
"02584ZZ"  =  "1"  /* Destruction of Conduction Mechanism, Percutaneous Endoscopic Approach */
"02590ZZ"  =  "1"  /* Destruction of Chordae Tendineae, Open Approach */
"02594ZZ"  =  "1"  /* Destruction of Chordae Tendineae, Percutaneous Endoscopic Approach */
"025D0ZZ"  =  "1"  /* Destruction of Papillary Muscle, Open Approach */
"025D4ZZ"  =  "1"  /* Destruction of Papillary Muscle, Percutaneous Endoscopic Approach */
"025F0ZZ"  =  "1"  /* Destruction of Aortic Valve, Open Approach */
"025F4ZZ"  =  "1"  /* Destruction of Aortic Valve, Percutaneous Endoscopic Approach */
"025G0ZZ"  =  "1"  /* Destruction of Mitral Valve, Open Approach */
"025G4ZZ"  =  "1"  /* Destruction of Mitral Valve, Percutaneous Endoscopic Approach */
"025H0ZZ"  =  "1"  /* Destruction of Pulmonary Valve, Open Approach */
"025H4ZZ"  =  "1"  /* Destruction of Pulmonary Valve, Percutaneous Endoscopic Approach */
"025J0ZZ"  =  "1"  /* Destruction of Tricuspid Valve, Open Approach */
"025J4ZZ"  =  "1"  /* Destruction of Tricuspid Valve, Percutaneous Endoscopic Approach */
"025K0ZZ"  =  "1"  /* Destruction of Right Ventricle, Open Approach */
"025K4ZZ"  =  "1"  /* Destruction of Right Ventricle, Percutaneous Endoscopic Approach */
"025L0ZZ"  =  "1"  /* Destruction of Left Ventricle, Open Approach */
"025L4ZZ"  =  "1"  /* Destruction of Left Ventricle, Percutaneous Endoscopic Approach */
"025M0ZZ"  =  "1"  /* Destruction of Ventricular Septum, Open Approach */
"025M4ZZ"  =  "1"  /* Destruction of Ventricular Septum, Percutaneous Endoscopic Approach */
"025N0ZZ"  =  "1"  /* Destruction of Pericardium, Open Approach */
"025N3ZZ"  =  "1"  /* Destruction of Pericardium, Percutaneous Approach */
"025N4ZZ"  =  "1"  /* Destruction of Pericardium, Percutaneous Endoscopic Approach */
"025P0ZZ"  =  "1"  /* Destruction of Pulmonary Trunk, Open Approach */
"025P4ZZ"  =  "1"  /* Destruction of Pulmonary Trunk, Percutaneous Endoscopic Approach */
"025Q0ZZ"  =  "1"  /* Destruction of Right Pulmonary Artery, Open Approach */
"025Q4ZZ"  =  "1"  /* Destruction of Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"025R0ZZ"  =  "1"  /* Destruction of Left Pulmonary Artery, Open Approach */
"025R4ZZ"  =  "1"  /* Destruction of Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"025S0ZZ"  =  "1"  /* Destruction of Right Pulmonary Vein, Open Approach */
"025S4ZZ"  =  "1"  /* Destruction of Right Pulmonary Vein, Percutaneous Endoscopic Approach */
"025T0ZZ"  =  "1"  /* Destruction of Left Pulmonary Vein, Open Approach */
"025T4ZZ"  =  "1"  /* Destruction of Left Pulmonary Vein, Percutaneous Endoscopic Approach */
"025V0ZZ"  =  "1"  /* Destruction of Superior Vena Cava, Open Approach */
"025V4ZZ"  =  "1"  /* Destruction of Superior Vena Cava, Percutaneous Endoscopic Approach */
"025W0ZZ"  =  "1"  /* Destruction of Thoracic Aorta, Open Approach */
"025W4ZZ"  =  "1"  /* Destruction of Thoracic Aorta, Percutaneous Endoscopic Approach */
"0270046"  =  "1"  /* Dilation of Coronary Artery, One Site, Bifurcation, with Drug-eluting Intraluminal Device, Open Approach */
"027004Z"  =  "1"  /* Dilation of Coronary Artery, One Site with Drug-eluting Intraluminal Device, Open Approach */
"02700D6"  =  "1"  /* Dilation of Coronary Artery, One Site, Bifurcation, with Intraluminal Device, Open Approach */
"02700DZ"  =  "1"  /* Dilation of Coronary Artery, One Site with Intraluminal Device, Open Approach */
"02700T6"  =  "1"  /* Dilation of Coronary Artery, One Site, Bifurcation, with Radioactive Intraluminal Device, Open Approach */
"02700TZ"  =  "1"  /* Dilation of Coronary Artery, One Site with Radioactive Intraluminal Device, Open Approach */
"02700Z6"  =  "1"  /* Dilation of Coronary Artery, One Site, Bifurcation, Open Approach */
"02700ZZ"  =  "1"  /* Dilation of Coronary Artery, One Site, Open Approach */
"0271046"  =  "1"  /* Dilation of Coronary Artery, Two Sites, Bifurcation, with Drug-eluting Intraluminal Device, Open Approach */
"027104Z"  =  "1"  /* Dilation of Coronary Artery, Two Sites with Drug-eluting Intraluminal Device, Open Approach */
"02710D6"  =  "1"  /* Dilation of Coronary Artery, Two Sites, Bifurcation, with Intraluminal Device, Open Approach */
"02710DZ"  =  "1"  /* Dilation of Coronary Artery, Two Sites with Intraluminal Device, Open Approach */
"02710T6"  =  "1"  /* Dilation of Coronary Artery, Two Sites, Bifurcation, with Radioactive Intraluminal Device, Open Approach */
"02710TZ"  =  "1"  /* Dilation of Coronary Artery, Two Sites with Radioactive Intraluminal Device, Open Approach */
"02710Z6"  =  "1"  /* Dilation of Coronary Artery, Two Sites, Bifurcation, Open Approach */
"02710ZZ"  =  "1"  /* Dilation of Coronary Artery, Two Sites, Open Approach */
"0272046"  =  "1"  /* Dilation of Coronary Artery, Three Sites, Bifurcation, with Drug-eluting Intraluminal Device, Open Approach */
"027204Z"  =  "1"  /* Dilation of Coronary Artery, Three Sites with Drug-eluting Intraluminal Device, Open Approach */
"02720D6"  =  "1"  /* Dilation of Coronary Artery, Three Sites, Bifurcation, with Intraluminal Device, Open Approach */
"02720DZ"  =  "1"  /* Dilation of Coronary Artery, Three Sites with Intraluminal Device, Open Approach */
"02720T6"  =  "1"  /* Dilation of Coronary Artery, Three Sites, Bifurcation, with Radioactive Intraluminal Device, Open Approach */
"02720TZ"  =  "1"  /* Dilation of Coronary Artery, Three Sites with Radioactive Intraluminal Device, Open Approach */
"02720Z6"  =  "1"  /* Dilation of Coronary Artery, Three Sites, Bifurcation, Open Approach */
"02720ZZ"  =  "1"  /* Dilation of Coronary Artery, Three Sites, Open Approach */
"0273046"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites, Bifurcation, with Drug-eluting Intraluminal Device, Open Approach */
"027304Z"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites with Drug-eluting Intraluminal Device, Open Approach */
"02730D6"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites, Bifurcation, with Intraluminal Device, Open Approach */
"02730DZ"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites with Intraluminal Device, Open Approach */
"02730T6"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites, Bifurcation, with Radioactive Intraluminal Device, Open Approach */
"02730TZ"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites with Radioactive Intraluminal Device, Open Approach */
"02730Z6"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites, Bifurcation, Open Approach */
"02730ZZ"  =  "1"  /* Dilation of Coronary Artery, Four or More Sites, Open Approach */
"027F04Z"  =  "1"  /* Dilation of Aortic Valve with Drug-eluting Intraluminal Device, Open Approach */
"027F0DZ"  =  "1"  /* Dilation of Aortic Valve with Intraluminal Device, Open Approach */
"027F0ZZ"  =  "1"  /* Dilation of Aortic Valve, Open Approach */
"027G04Z"  =  "1"  /* Dilation of Mitral Valve with Drug-eluting Intraluminal Device, Open Approach */
"027G0DZ"  =  "1"  /* Dilation of Mitral Valve with Intraluminal Device, Open Approach */
"027G0ZZ"  =  "1"  /* Dilation of Mitral Valve, Open Approach */
"027H04Z"  =  "1"  /* Dilation of Pulmonary Valve with Drug-eluting Intraluminal Device, Open Approach */
"027H0DZ"  =  "1"  /* Dilation of Pulmonary Valve with Intraluminal Device, Open Approach */
"027H0ZZ"  =  "1"  /* Dilation of Pulmonary Valve, Open Approach */
"027J04Z"  =  "1"  /* Dilation of Tricuspid Valve with Drug-eluting Intraluminal Device, Open Approach */
"027J0DZ"  =  "1"  /* Dilation of Tricuspid Valve with Intraluminal Device, Open Approach */
"027J0ZZ"  =  "1"  /* Dilation of Tricuspid Valve, Open Approach */
"027K04Z"  =  "1"  /* Dilation of Right Ventricle with Drug-eluting Intraluminal Device, Open Approach */
"027K0DZ"  =  "1"  /* Dilation of Right Ventricle with Intraluminal Device, Open Approach */
"027K0ZZ"  =  "1"  /* Dilation of Right Ventricle, Open Approach */
"027K44Z"  =  "1"  /* Dilation of Right Ventricle with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"027K4DZ"  =  "1"  /* Dilation of Right Ventricle with Intraluminal Device, Percutaneous Endoscopic Approach */
"027K4ZZ"  =  "1"  /* Dilation of Right Ventricle, Percutaneous Endoscopic Approach */
"027R04T"  =  "1"  /* Dilation of Ductus Arteriosus with Drug-eluting Intraluminal Device, Open Approach */
"027R0DT"  =  "1"  /* Dilation of Ductus Arteriosus with Intraluminal Device, Open Approach */
"027R0ZT"  =  "1"  /* Dilation of Ductus Arteriosus, Open Approach */
"027R44T"  =  "1"  /* Dilation of Ductus Arteriosus with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"027R4DT"  =  "1"  /* Dilation of Ductus Arteriosus with Intraluminal Device, Percutaneous Endoscopic Approach */
"027R4ZT"  =  "1"  /* Dilation of Ductus Arteriosus, Percutaneous Endoscopic Approach */
"02890ZZ"  =  "1"  /* Division of Chordae Tendineae, Open Approach */
"02894ZZ"  =  "1"  /* Division of Chordae Tendineae, Percutaneous Endoscopic Approach */
"028D0ZZ"  =  "1"  /* Division of Papillary Muscle, Open Approach */
"028D4ZZ"  =  "1"  /* Division of Papillary Muscle, Percutaneous Endoscopic Approach */
"02B40ZZ"  =  "1"  /* Excision of Coronary Vein, Open Approach */
"02B44ZZ"  =  "1"  /* Excision of Coronary Vein, Percutaneous Endoscopic Approach */
"02B50ZZ"  =  "1"  /* Excision of Atrial Septum, Open Approach */
"02B54ZZ"  =  "1"  /* Excision of Atrial Septum, Percutaneous Endoscopic Approach */
"02B60ZZ"  =  "1"  /* Excision of Right Atrium, Open Approach */
"02B64ZZ"  =  "1"  /* Excision of Right Atrium, Percutaneous Endoscopic Approach */
"02B70ZK"  =  "1"  /* Excision of Left Atrial Appendage, Open Approach */
"02B70ZZ"  =  "1"  /* Excision of Left Atrium, Open Approach */
"02B74ZK"  =  "1"  /* Excision of Left Atrial Appendage, Percutaneous Endoscopic Approach */
"02B74ZZ"  =  "1"  /* Excision of Left Atrium, Percutaneous Endoscopic Approach */
"02B80ZZ"  =  "1"  /* Excision of Conduction Mechanism, Open Approach */
"02B84ZZ"  =  "1"  /* Excision of Conduction Mechanism, Percutaneous Endoscopic Approach */
"02B90ZZ"  =  "1"  /* Excision of Chordae Tendineae, Open Approach */
"02B94ZZ"  =  "1"  /* Excision of Chordae Tendineae, Percutaneous Endoscopic Approach */
"02BD0ZZ"  =  "1"  /* Excision of Papillary Muscle, Open Approach */
"02BD4ZZ"  =  "1"  /* Excision of Papillary Muscle, Percutaneous Endoscopic Approach */
"02BF0ZZ"  =  "1"  /* Excision of Aortic Valve, Open Approach */
"02BF4ZZ"  =  "1"  /* Excision of Aortic Valve, Percutaneous Endoscopic Approach */
"02BG0ZZ"  =  "1"  /* Excision of Mitral Valve, Open Approach */
"02BG4ZZ"  =  "1"  /* Excision of Mitral Valve, Percutaneous Endoscopic Approach */
"02BH0ZZ"  =  "1"  /* Excision of Pulmonary Valve, Open Approach */
"02BH4ZZ"  =  "1"  /* Excision of Pulmonary Valve, Percutaneous Endoscopic Approach */
"02BJ0ZZ"  =  "1"  /* Excision of Tricuspid Valve, Open Approach */
"02BJ4ZZ"  =  "1"  /* Excision of Tricuspid Valve, Percutaneous Endoscopic Approach */
"02BK0ZZ"  =  "1"  /* Excision of Right Ventricle, Open Approach */
"02BK4ZZ"  =  "1"  /* Excision of Right Ventricle, Percutaneous Endoscopic Approach */
"02BL0ZZ"  =  "1"  /* Excision of Left Ventricle, Open Approach */
"02BL4ZZ"  =  "1"  /* Excision of Left Ventricle, Percutaneous Endoscopic Approach */
"02BM0ZZ"  =  "1"  /* Excision of Ventricular Septum, Open Approach */
"02BM4ZZ"  =  "1"  /* Excision of Ventricular Septum, Percutaneous Endoscopic Approach */
"02BN0ZZ"  =  "1"  /* Excision of Pericardium, Open Approach */
"02BN3ZZ"  =  "1"  /* Excision of Pericardium, Percutaneous Approach */
"02BN4ZZ"  =  "1"  /* Excision of Pericardium, Percutaneous Endoscopic Approach */
"02BP0ZZ"  =  "1"  /* Excision of Pulmonary Trunk, Open Approach */
"02BP4ZZ"  =  "1"  /* Excision of Pulmonary Trunk, Percutaneous Endoscopic Approach */
"02BQ0ZZ"  =  "1"  /* Excision of Right Pulmonary Artery, Open Approach */
"02BQ4ZZ"  =  "1"  /* Excision of Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"02BR0ZZ"  =  "1"  /* Excision of Left Pulmonary Artery, Open Approach */
"02BR4ZZ"  =  "1"  /* Excision of Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"02BS0ZZ"  =  "1"  /* Excision of Right Pulmonary Vein, Open Approach */
"02BS4ZZ"  =  "1"  /* Excision of Right Pulmonary Vein, Percutaneous Endoscopic Approach */
"02BT0ZZ"  =  "1"  /* Excision of Left Pulmonary Vein, Open Approach */
"02BT4ZZ"  =  "1"  /* Excision of Left Pulmonary Vein, Percutaneous Endoscopic Approach */
"02BV0ZZ"  =  "1"  /* Excision of Superior Vena Cava, Open Approach */
"02BV4ZZ"  =  "1"  /* Excision of Superior Vena Cava, Percutaneous Endoscopic Approach */
"02C00ZZ"  =  "1"  /* Extirpation of Matter from Coronary Artery, One Site, Open Approach */
"02C10ZZ"  =  "1"  /* Extirpation of Matter from Coronary Artery, Two Sites, Open Approach */
"02C20ZZ"  =  "1"  /* Extirpation of Matter from Coronary Artery, Three Sites, Open Approach */
"02C30ZZ"  =  "1"  /* Extirpation of Matter from Coronary Artery, Four or More Sites, Open Approach */
"02C40ZZ"  =  "1"  /* Extirpation of Matter from Coronary Vein, Open Approach */
"02C44ZZ"  =  "1"  /* Extirpation of Matter from Coronary Vein, Percutaneous Endoscopic Approach */
"02C50ZZ"  =  "1"  /* Extirpation of Matter from Atrial Septum, Open Approach */
"02C54ZZ"  =  "1"  /* Extirpation of Matter from Atrial Septum, Percutaneous Endoscopic Approach */
"02C60ZZ"  =  "1"  /* Extirpation of Matter from Right Atrium, Open Approach */
"02C64ZZ"  =  "1"  /* Extirpation of Matter from Right Atrium, Percutaneous Endoscopic Approach */
"02C70ZZ"  =  "1"  /* Extirpation of Matter from Left Atrium, Open Approach */
"02C74ZZ"  =  "1"  /* Extirpation of Matter from Left Atrium, Percutaneous Endoscopic Approach */
"02C80ZZ"  =  "1"  /* Extirpation of Matter from Conduction Mechanism, Open Approach */
"02C84ZZ"  =  "1"  /* Extirpation of Matter from Conduction Mechanism, Percutaneous Endoscopic Approach */
"02C90ZZ"  =  "1"  /* Extirpation of Matter from Chordae Tendineae, Open Approach */
"02C94ZZ"  =  "1"  /* Extirpation of Matter from Chordae Tendineae, Percutaneous Endoscopic Approach */
"02CD0ZZ"  =  "1"  /* Extirpation of Matter from Papillary Muscle, Open Approach */
"02CD4ZZ"  =  "1"  /* Extirpation of Matter from Papillary Muscle, Percutaneous Endoscopic Approach */
"02CF0ZZ"  =  "1"  /* Extirpation of Matter from Aortic Valve, Open Approach */
"02CG0ZZ"  =  "1"  /* Extirpation of Matter from Mitral Valve, Open Approach */
"02CH0ZZ"  =  "1"  /* Extirpation of Matter from Pulmonary Valve, Open Approach */
"02CJ0ZZ"  =  "1"  /* Extirpation of Matter from Tricuspid Valve, Open Approach */
"02CK0ZZ"  =  "1"  /* Extirpation of Matter from Right Ventricle, Open Approach */
"02CK4ZZ"  =  "1"  /* Extirpation of Matter from Right Ventricle, Percutaneous Endoscopic Approach */
"02CL0ZZ"  =  "1"  /* Extirpation of Matter from Left Ventricle, Open Approach */
"02CL4ZZ"  =  "1"  /* Extirpation of Matter from Left Ventricle, Percutaneous Endoscopic Approach */
"02CM0ZZ"  =  "1"  /* Extirpation of Matter from Ventricular Septum, Open Approach */
"02CM4ZZ"  =  "1"  /* Extirpation of Matter from Ventricular Septum, Percutaneous Endoscopic Approach */
"02CN0ZZ"  =  "1"  /* Extirpation of Matter from Pericardium, Open Approach */
"02CN3ZZ"  =  "1"  /* Extirpation of Matter from Pericardium, Percutaneous Approach */
"02CN4ZZ"  =  "1"  /* Extirpation of Matter from Pericardium, Percutaneous Endoscopic Approach */
"02CP0ZZ"  =  "1"  /* Extirpation of Matter from Pulmonary Trunk, Open Approach */
"02CP4ZZ"  =  "1"  /* Extirpation of Matter from Pulmonary Trunk, Percutaneous Endoscopic Approach */
"02CQ0ZZ"  =  "1"  /* Extirpation of Matter from Right Pulmonary Artery, Open Approach */
"02CQ4ZZ"  =  "1"  /* Extirpation of Matter from Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"02CR0ZZ"  =  "1"  /* Extirpation of Matter from Left Pulmonary Artery, Open Approach */
"02CR4ZZ"  =  "1"  /* Extirpation of Matter from Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"02CS0ZZ"  =  "1"  /* Extirpation of Matter from Right Pulmonary Vein, Open Approach */
"02CS4ZZ"  =  "1"  /* Extirpation of Matter from Right Pulmonary Vein, Percutaneous Endoscopic Approach */
"02CT0ZZ"  =  "1"  /* Extirpation of Matter from Left Pulmonary Vein, Open Approach */
"02CT4ZZ"  =  "1"  /* Extirpation of Matter from Left Pulmonary Vein, Percutaneous Endoscopic Approach */
"02CV0ZZ"  =  "1"  /* Extirpation of Matter from Superior Vena Cava, Open Approach */
"02CV4ZZ"  =  "1"  /* Extirpation of Matter from Superior Vena Cava, Percutaneous Endoscopic Approach */
"02CW0ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Aorta, Open Approach */
"02CW4ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Aorta, Percutaneous Endoscopic Approach */
"02FN0ZZ"  =  "1"  /* Fragmentation in Pericardium, Open Approach */
"02FN3ZZ"  =  "1"  /* Fragmentation in Pericardium, Percutaneous Approach */
"02FN4ZZ"  =  "1"  /* Fragmentation in Pericardium, Percutaneous Endoscopic Approach */
"02H402Z"  =  "1"  /* Insertion of Monitoring Device into Coronary Vein, Open Approach */
"02H403Z"  =  "1"  /* Insertion of Infusion Device into Coronary Vein, Open Approach */
"02H40DZ"  =  "1"  /* Insertion of Intraluminal Device into Coronary Vein, Open Approach */
"02H442Z"  =  "1"  /* Insertion of Monitoring Device into Coronary Vein, Percutaneous Endoscopic Approach */
"02H443Z"  =  "1"  /* Insertion of Infusion Device into Coronary Vein, Percutaneous Endoscopic Approach */
"02H44DZ"  =  "1"  /* Insertion of Intraluminal Device into Coronary Vein, Percutaneous Endoscopic Approach */
"02H602Z"  =  "1"  /* Insertion of Monitoring Device into Right Atrium, Open Approach */
"02H603Z"  =  "1"  /* Insertion of Infusion Device into Right Atrium, Open Approach */
"02H60DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Atrium, Open Approach */
"02H642Z"  =  "1"  /* Insertion of Monitoring Device into Right Atrium, Percutaneous Endoscopic Approach */
"02H643Z"  =  "1"  /* Insertion of Infusion Device into Right Atrium, Percutaneous Endoscopic Approach */
"02H64DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Atrium, Percutaneous Endoscopic Approach */
"02H702Z"  =  "1"  /* Insertion of Monitoring Device into Left Atrium, Open Approach */
"02H703Z"  =  "1"  /* Insertion of Infusion Device into Left Atrium, Open Approach */
"02H70DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Atrium, Open Approach */
"02H742Z"  =  "1"  /* Insertion of Monitoring Device into Left Atrium, Percutaneous Endoscopic Approach */
"02H743Z"  =  "1"  /* Insertion of Infusion Device into Left Atrium, Percutaneous Endoscopic Approach */
"02H74DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Atrium, Percutaneous Endoscopic Approach */
"02HA0QZ"  =  "1"  /* Insertion of Implantable Heart Assist System into Heart, Open Approach */
"02HA0RS"  =  "1"  /* Insertion of Biventricular External Heart Assist System into Heart, Open Approach */
"02HA0RZ"  =  "1"  /* Insertion of External Heart Assist System into Heart, Open Approach */
"02HA4QZ"  =  "1"  /* Insertion of Implantable Heart Assist System into Heart, Percutaneous Endoscopic Approach */
"02HA4RS"  =  "1"  /* Insertion of Biventricular External Heart Assist System into Heart, Percutaneous Endoscopic Approach */
"02HA4RZ"  =  "1"  /* Insertion of External Heart Assist System into Heart, Percutaneous Endoscopic Approach */
"02HK03Z"  =  "1"  /* Insertion of Infusion Device into Right Ventricle, Open Approach */
"02HK0DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Ventricle, Open Approach */
"02HK43Z"  =  "1"  /* Insertion of Infusion Device into Right Ventricle, Percutaneous Endoscopic Approach */
"02HK4DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Ventricle, Percutaneous Endoscopic Approach */
"02HL02Z"  =  "1"  /* Insertion of Monitoring Device into Left Ventricle, Open Approach */
"02HL03Z"  =  "1"  /* Insertion of Infusion Device into Left Ventricle, Open Approach */
"02HL0DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Ventricle, Open Approach */
"02HL42Z"  =  "1"  /* Insertion of Monitoring Device into Left Ventricle, Percutaneous Endoscopic Approach */
"02HL43Z"  =  "1"  /* Insertion of Infusion Device into Left Ventricle, Percutaneous Endoscopic Approach */
"02HL4DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Ventricle, Percutaneous Endoscopic Approach */
"02HN00Z"  =  "1"  /* Insertion of Pressure Sensor Monitoring Device into Pericardium, Open Approach */
"02HN02Z"  =  "1"  /* Insertion of Monitoring Device into Pericardium, Open Approach */
"02HN0MZ"  =  "1"  /* Insertion of Cardiac Lead into Pericardium, Open Approach */
"02HN30Z"  =  "1"  /* Insertion of Pressure Sensor Monitoring Device into Pericardium, Percutaneous Approach */
"02HN32Z"  =  "1"  /* Insertion of Monitoring Device into Pericardium, Percutaneous Approach */
"02HN3MZ"  =  "1"  /* Insertion of Cardiac Lead into Pericardium, Percutaneous Approach */
"02HN40Z"  =  "1"  /* Insertion of Pressure Sensor Monitoring Device into Pericardium, Percutaneous Endoscopic Approach */
"02HN42Z"  =  "1"  /* Insertion of Monitoring Device into Pericardium, Percutaneous Endoscopic Approach */
"02HN4MZ"  =  "1"  /* Insertion of Cardiac Lead into Pericardium, Percutaneous Endoscopic Approach */
"02HP0DZ"  =  "1"  /* Insertion of Intraluminal Device into Pulmonary Trunk, Open Approach */
"02HP4DZ"  =  "1"  /* Insertion of Intraluminal Device into Pulmonary Trunk, Percutaneous Endoscopic Approach */
"02HQ0DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Pulmonary Artery, Open Approach */
"02HQ4DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"02HR0DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Pulmonary Artery, Open Approach */
"02HR4DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"02HS02Z"  =  "1"  /* Insertion of Monitoring Device into Right Pulmonary Vein, Open Approach */
"02HS0DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Pulmonary Vein, Open Approach */
"02HS42Z"  =  "1"  /* Insertion of Monitoring Device into Right Pulmonary Vein, Percutaneous Endoscopic Approach */
"02HS4DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Pulmonary Vein, Percutaneous Endoscopic Approach */
"02HT02Z"  =  "1"  /* Insertion of Monitoring Device into Left Pulmonary Vein, Open Approach */
"02HT0DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Pulmonary Vein, Open Approach */
"02HT42Z"  =  "1"  /* Insertion of Monitoring Device into Left Pulmonary Vein, Percutaneous Endoscopic Approach */
"02HT4DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Pulmonary Vein, Percutaneous Endoscopic Approach */
"02HW02Z"  =  "1"  /* Insertion of Monitoring Device into Thoracic Aorta, Open Approach */
"02HW0DZ"  =  "1"  /* Insertion of Intraluminal Device into Thoracic Aorta, Open Approach */
"02HW42Z"  =  "1"  /* Insertion of Monitoring Device into Thoracic Aorta, Percutaneous Endoscopic Approach */
"02HW4DZ"  =  "1"  /* Insertion of Intraluminal Device into Thoracic Aorta, Percutaneous Endoscopic Approach */
"02L70CK"  =  "1"  /* Occlusion of Left Atrial Appendage with Extraluminal Device, Open Approach */
"02L70ZK"  =  "1"  /* Occlusion of Left Atrial Appendage, Open Approach */
"02L73CK"  =  "1"  /* Occlusion of Left Atrial Appendage with Extraluminal Device, Percutaneous Approach */
"02L73ZK"  =  "1"  /* Occlusion of Left Atrial Appendage, Percutaneous Approach */
"02L74CK"  =  "1"  /* Occlusion of Left Atrial Appendage with Extraluminal Device, Percutaneous Endoscopic Approach */
"02L74ZK"  =  "1"  /* Occlusion of Left Atrial Appendage, Percutaneous Endoscopic Approach */
"02LR0CT"  =  "1"  /* Occlusion of Ductus Arteriosus with Extraluminal Device, Open Approach */
"02LR0ZT"  =  "1"  /* Occlusion of Ductus Arteriosus, Open Approach */
"02LR3CT"  =  "1"  /* Occlusion of Ductus Arteriosus with Extraluminal Device, Percutaneous Approach */
"02LR3ZT"  =  "1"  /* Occlusion of Ductus Arteriosus, Percutaneous Approach */
"02LR4CT"  =  "1"  /* Occlusion of Ductus Arteriosus with Extraluminal Device, Percutaneous Endoscopic Approach */
"02LR4ZT"  =  "1"  /* Occlusion of Ductus Arteriosus, Percutaneous Endoscopic Approach */
"02LS0CZ"  =  "1"  /* Occlusion of Right Pulmonary Vein with Extraluminal Device, Open Approach */
"02LS0ZZ"  =  "1"  /* Occlusion of Right Pulmonary Vein, Open Approach */
"02LS3CZ"  =  "1"  /* Occlusion of Right Pulmonary Vein with Extraluminal Device, Percutaneous Approach */
"02LS3ZZ"  =  "1"  /* Occlusion of Right Pulmonary Vein, Percutaneous Approach */
"02LS4CZ"  =  "1"  /* Occlusion of Right Pulmonary Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"02LS4ZZ"  =  "1"  /* Occlusion of Right Pulmonary Vein, Percutaneous Endoscopic Approach */
"02LT0CZ"  =  "1"  /* Occlusion of Left Pulmonary Vein with Extraluminal Device, Open Approach */
"02LT0ZZ"  =  "1"  /* Occlusion of Left Pulmonary Vein, Open Approach */
"02LT3CZ"  =  "1"  /* Occlusion of Left Pulmonary Vein with Extraluminal Device, Percutaneous Approach */
"02LT3ZZ"  =  "1"  /* Occlusion of Left Pulmonary Vein, Percutaneous Approach */
"02LT4CZ"  =  "1"  /* Occlusion of Left Pulmonary Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"02LT4ZZ"  =  "1"  /* Occlusion of Left Pulmonary Vein, Percutaneous Endoscopic Approach */
"02N40ZZ"  =  "1"  /* Release Coronary Vein, Open Approach */
"02N44ZZ"  =  "1"  /* Release Coronary Vein, Percutaneous Endoscopic Approach */
"02N50ZZ"  =  "1"  /* Release Atrial Septum, Open Approach */
"02N54ZZ"  =  "1"  /* Release Atrial Septum, Percutaneous Endoscopic Approach */
"02N60ZZ"  =  "1"  /* Release Right Atrium, Open Approach */
"02N64ZZ"  =  "1"  /* Release Right Atrium, Percutaneous Endoscopic Approach */
"02N70ZZ"  =  "1"  /* Release Left Atrium, Open Approach */
"02N74ZZ"  =  "1"  /* Release Left Atrium, Percutaneous Endoscopic Approach */
"02N80ZZ"  =  "1"  /* Release Conduction Mechanism, Open Approach */
"02N84ZZ"  =  "1"  /* Release Conduction Mechanism, Percutaneous Endoscopic Approach */
"02N90ZZ"  =  "1"  /* Release Chordae Tendineae, Open Approach */
"02N94ZZ"  =  "1"  /* Release Chordae Tendineae, Percutaneous Endoscopic Approach */
"02ND0ZZ"  =  "1"  /* Release Papillary Muscle, Open Approach */
"02ND4ZZ"  =  "1"  /* Release Papillary Muscle, Percutaneous Endoscopic Approach */
"02NF0ZZ"  =  "1"  /* Release Aortic Valve, Open Approach */
"02NG0ZZ"  =  "1"  /* Release Mitral Valve, Open Approach */
"02NH0ZZ"  =  "1"  /* Release Pulmonary Valve, Open Approach */
"02NJ0ZZ"  =  "1"  /* Release Tricuspid Valve, Open Approach */
"02NK0ZZ"  =  "1"  /* Release Right Ventricle, Open Approach */
"02NK4ZZ"  =  "1"  /* Release Right Ventricle, Percutaneous Endoscopic Approach */
"02NL0ZZ"  =  "1"  /* Release Left Ventricle, Open Approach */
"02NL4ZZ"  =  "1"  /* Release Left Ventricle, Percutaneous Endoscopic Approach */
"02NM0ZZ"  =  "1"  /* Release Ventricular Septum, Open Approach */
"02NM4ZZ"  =  "1"  /* Release Ventricular Septum, Percutaneous Endoscopic Approach */
"02NN0ZZ"  =  "1"  /* Release Pericardium, Open Approach */
"02NN3ZZ"  =  "1"  /* Release Pericardium, Percutaneous Approach */
"02NN4ZZ"  =  "1"  /* Release Pericardium, Percutaneous Endoscopic Approach */
"02PA02Z"  =  "1"  /* Removal of Monitoring Device from Heart, Open Approach */
"02PA03Z"  =  "1"  /* Removal of Infusion Device from Heart, Open Approach */
"02PA07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Heart, Open Approach */
"02PA08Z"  =  "1"  /* Removal of Zooplastic Tissue from Heart, Open Approach */
"02PA0CZ"  =  "1"  /* Removal of Extraluminal Device from Heart, Open Approach */
"02PA0DZ"  =  "1"  /* Removal of Intraluminal Device from Heart, Open Approach */
"02PA0JZ"  =  "1"  /* Removal of Synthetic Substitute from Heart, Open Approach */
"02PA0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Heart, Open Approach */
"02PA0QZ"  =  "1"  /* Removal of Implantable Heart Assist System from Heart, Open Approach */
"02PA0RZ"  =  "1"  /* Removal of External Heart Assist System from Heart, Open Approach */
"02PA32Z"  =  "1"  /* Removal of Monitoring Device from Heart, Percutaneous Approach */
"02PA33Z"  =  "1"  /* Removal of Infusion Device from Heart, Percutaneous Approach */
"02PA37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Heart, Percutaneous Approach */
"02PA38Z"  =  "1"  /* Removal of Zooplastic Tissue from Heart, Percutaneous Approach */
"02PA3CZ"  =  "1"  /* Removal of Extraluminal Device from Heart, Percutaneous Approach */
"02PA3JZ"  =  "1"  /* Removal of Synthetic Substitute from Heart, Percutaneous Approach */
"02PA3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Heart, Percutaneous Approach */
"02PA3QZ"  =  "1"  /* Removal of Implantable Heart Assist System from Heart, Percutaneous Approach */
"02PA3RZ"  =  "1"  /* Removal of External Heart Assist System from Heart, Percutaneous Approach */
"02PA42Z"  =  "1"  /* Removal of Monitoring Device from Heart, Percutaneous Endoscopic Approach */
"02PA43Z"  =  "1"  /* Removal of Infusion Device from Heart, Percutaneous Endoscopic Approach */
"02PA47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Heart, Percutaneous Endoscopic Approach */
"02PA48Z"  =  "1"  /* Removal of Zooplastic Tissue from Heart, Percutaneous Endoscopic Approach */
"02PA4CZ"  =  "1"  /* Removal of Extraluminal Device from Heart, Percutaneous Endoscopic Approach */
"02PA4DZ"  =  "1"  /* Removal of Intraluminal Device from Heart, Percutaneous Endoscopic Approach */
"02PA4JZ"  =  "1"  /* Removal of Synthetic Substitute from Heart, Percutaneous Endoscopic Approach */
"02PA4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Heart, Percutaneous Endoscopic Approach */
"02PA4QZ"  =  "1"  /* Removal of Implantable Heart Assist System from Heart, Percutaneous Endoscopic Approach */
"02PA4RZ"  =  "1"  /* Removal of External Heart Assist System from Heart, Percutaneous Endoscopic Approach */
"02PY02Z"  =  "1"  /* Removal of Monitoring Device from Great Vessel, Open Approach */
"02PY03Z"  =  "1"  /* Removal of Infusion Device from Great Vessel, Open Approach */
"02PY07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Great Vessel, Open Approach */
"02PY08Z"  =  "1"  /* Removal of Zooplastic Tissue from Great Vessel, Open Approach */
"02PY0CZ"  =  "1"  /* Removal of Extraluminal Device from Great Vessel, Open Approach */
"02PY0DZ"  =  "1"  /* Removal of Intraluminal Device from Great Vessel, Open Approach */
"02PY0JZ"  =  "1"  /* Removal of Synthetic Substitute from Great Vessel, Open Approach */
"02PY0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Great Vessel, Open Approach */
"02PY33Z"  =  "1"  /* Removal of Infusion Device from Great Vessel, Percutaneous Approach */
"02PY37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Great Vessel, Percutaneous Approach */
"02PY38Z"  =  "1"  /* Removal of Zooplastic Tissue from Great Vessel, Percutaneous Approach */
"02PY3CZ"  =  "1"  /* Removal of Extraluminal Device from Great Vessel, Percutaneous Approach */
"02PY3JZ"  =  "1"  /* Removal of Synthetic Substitute from Great Vessel, Percutaneous Approach */
"02PY3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Great Vessel, Percutaneous Approach */
"02PY42Z"  =  "1"  /* Removal of Monitoring Device from Great Vessel, Percutaneous Endoscopic Approach */
"02PY43Z"  =  "1"  /* Removal of Infusion Device from Great Vessel, Percutaneous Endoscopic Approach */
"02PY47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Great Vessel, Percutaneous Endoscopic Approach */
"02PY48Z"  =  "1"  /* Removal of Zooplastic Tissue from Great Vessel, Percutaneous Endoscopic Approach */
"02PY4CZ"  =  "1"  /* Removal of Extraluminal Device from Great Vessel, Percutaneous Endoscopic Approach */
"02PY4DZ"  =  "1"  /* Removal of Intraluminal Device from Great Vessel, Percutaneous Endoscopic Approach */
"02PY4JZ"  =  "1"  /* Removal of Synthetic Substitute from Great Vessel, Percutaneous Endoscopic Approach */
"02PY4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Great Vessel, Percutaneous Endoscopic Approach */
"02Q00ZZ"  =  "1"  /* Repair Coronary Artery, One Site, Open Approach */
"02Q04ZZ"  =  "1"  /* Repair Coronary Artery, One Site, Percutaneous Endoscopic Approach */
"02Q10ZZ"  =  "1"  /* Repair Coronary Artery, Two Sites, Open Approach */
"02Q14ZZ"  =  "1"  /* Repair Coronary Artery, Two Sites, Percutaneous Endoscopic Approach */
"02Q20ZZ"  =  "1"  /* Repair Coronary Artery, Three Sites, Open Approach */
"02Q24ZZ"  =  "1"  /* Repair Coronary Artery, Three Sites, Percutaneous Endoscopic Approach */
"02Q30ZZ"  =  "1"  /* Repair Coronary Artery, Four or More Sites, Open Approach */
"02Q34ZZ"  =  "1"  /* Repair Coronary Artery, Four or More Sites, Percutaneous Endoscopic Approach */
"02Q40ZZ"  =  "1"  /* Repair Coronary Vein, Open Approach */
"02Q44ZZ"  =  "1"  /* Repair Coronary Vein, Percutaneous Endoscopic Approach */
"02Q50ZZ"  =  "1"  /* Repair Atrial Septum, Open Approach */
"02Q54ZZ"  =  "1"  /* Repair Atrial Septum, Percutaneous Endoscopic Approach */
"02Q60ZZ"  =  "1"  /* Repair Right Atrium, Open Approach */
"02Q64ZZ"  =  "1"  /* Repair Right Atrium, Percutaneous Endoscopic Approach */
"02Q70ZZ"  =  "1"  /* Repair Left Atrium, Open Approach */
"02Q74ZZ"  =  "1"  /* Repair Left Atrium, Percutaneous Endoscopic Approach */
"02Q80ZZ"  =  "1"  /* Repair Conduction Mechanism, Open Approach */
"02Q84ZZ"  =  "1"  /* Repair Conduction Mechanism, Percutaneous Endoscopic Approach */
"02Q90ZZ"  =  "1"  /* Repair Chordae Tendineae, Open Approach */
"02Q94ZZ"  =  "1"  /* Repair Chordae Tendineae, Percutaneous Endoscopic Approach */
"02QA0ZZ"  =  "1"  /* Repair Heart, Open Approach */
"02QA4ZZ"  =  "1"  /* Repair Heart, Percutaneous Endoscopic Approach */
"02QB0ZZ"  =  "1"  /* Repair Right Heart, Open Approach */
"02QB4ZZ"  =  "1"  /* Repair Right Heart, Percutaneous Endoscopic Approach */
"02QC0ZZ"  =  "1"  /* Repair Left Heart, Open Approach */
"02QC4ZZ"  =  "1"  /* Repair Left Heart, Percutaneous Endoscopic Approach */
"02QD0ZZ"  =  "1"  /* Repair Papillary Muscle, Open Approach */
"02QD4ZZ"  =  "1"  /* Repair Papillary Muscle, Percutaneous Endoscopic Approach */
"02QF0ZZ"  =  "1"  /* Repair Aortic Valve, Open Approach */
"02QF4ZZ"  =  "1"  /* Repair Aortic Valve, Percutaneous Endoscopic Approach */
"02QG0ZZ"  =  "1"  /* Repair Mitral Valve, Open Approach */
"02QG4ZZ"  =  "1"  /* Repair Mitral Valve, Percutaneous Endoscopic Approach */
"02QH0ZZ"  =  "1"  /* Repair Pulmonary Valve, Open Approach */
"02QH4ZZ"  =  "1"  /* Repair Pulmonary Valve, Percutaneous Endoscopic Approach */
"02QJ0ZZ"  =  "1"  /* Repair Tricuspid Valve, Open Approach */
"02QJ4ZZ"  =  "1"  /* Repair Tricuspid Valve, Percutaneous Endoscopic Approach */
"02QK0ZZ"  =  "1"  /* Repair Right Ventricle, Open Approach */
"02QK4ZZ"  =  "1"  /* Repair Right Ventricle, Percutaneous Endoscopic Approach */
"02QL0ZZ"  =  "1"  /* Repair Left Ventricle, Open Approach */
"02QL4ZZ"  =  "1"  /* Repair Left Ventricle, Percutaneous Endoscopic Approach */
"02QM0ZZ"  =  "1"  /* Repair Ventricular Septum, Open Approach */
"02QM4ZZ"  =  "1"  /* Repair Ventricular Septum, Percutaneous Endoscopic Approach */
"02QN0ZZ"  =  "1"  /* Repair Pericardium, Open Approach */
"02QN3ZZ"  =  "1"  /* Repair Pericardium, Percutaneous Approach */
"02QN4ZZ"  =  "1"  /* Repair Pericardium, Percutaneous Endoscopic Approach */
"02R507Z"  =  "1"  /* Replacement of Atrial Septum with Autologous Tissue Substitute, Open Approach */
"02R508Z"  =  "1"  /* Replacement of Atrial Septum with Zooplastic Tissue, Open Approach */
"02R50JZ"  =  "1"  /* Replacement of Atrial Septum with Synthetic Substitute, Open Approach */
"02R50KZ"  =  "1"  /* Replacement of Atrial Septum with Nonautologous Tissue Substitute, Open Approach */
"02R547Z"  =  "1"  /* Replacement of Atrial Septum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02R548Z"  =  "1"  /* Replacement of Atrial Septum with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02R54JZ"  =  "1"  /* Replacement of Atrial Septum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02R54KZ"  =  "1"  /* Replacement of Atrial Septum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02R607Z"  =  "1"  /* Replacement of Right Atrium with Autologous Tissue Substitute, Open Approach */
"02R608Z"  =  "1"  /* Replacement of Right Atrium with Zooplastic Tissue, Open Approach */
"02R60JZ"  =  "1"  /* Replacement of Right Atrium with Synthetic Substitute, Open Approach */
"02R60KZ"  =  "1"  /* Replacement of Right Atrium with Nonautologous Tissue Substitute, Open Approach */
"02R647Z"  =  "1"  /* Replacement of Right Atrium with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02R648Z"  =  "1"  /* Replacement of Right Atrium with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02R64JZ"  =  "1"  /* Replacement of Right Atrium with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02R64KZ"  =  "1"  /* Replacement of Right Atrium with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02R707Z"  =  "1"  /* Replacement of Left Atrium with Autologous Tissue Substitute, Open Approach */
"02R708Z"  =  "1"  /* Replacement of Left Atrium with Zooplastic Tissue, Open Approach */
"02R70JZ"  =  "1"  /* Replacement of Left Atrium with Synthetic Substitute, Open Approach */
"02R70KZ"  =  "1"  /* Replacement of Left Atrium with Nonautologous Tissue Substitute, Open Approach */
"02R747Z"  =  "1"  /* Replacement of Left Atrium with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02R748Z"  =  "1"  /* Replacement of Left Atrium with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02R74JZ"  =  "1"  /* Replacement of Left Atrium with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02R74KZ"  =  "1"  /* Replacement of Left Atrium with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02R907Z"  =  "1"  /* Replacement of Chordae Tendineae with Autologous Tissue Substitute, Open Approach */
"02R908Z"  =  "1"  /* Replacement of Chordae Tendineae with Zooplastic Tissue, Open Approach */
"02R90JZ"  =  "1"  /* Replacement of Chordae Tendineae with Synthetic Substitute, Open Approach */
"02R90KZ"  =  "1"  /* Replacement of Chordae Tendineae with Nonautologous Tissue Substitute, Open Approach */
"02R947Z"  =  "1"  /* Replacement of Chordae Tendineae with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02R948Z"  =  "1"  /* Replacement of Chordae Tendineae with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02R94JZ"  =  "1"  /* Replacement of Chordae Tendineae with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02R94KZ"  =  "1"  /* Replacement of Chordae Tendineae with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RD07Z"  =  "1"  /* Replacement of Papillary Muscle with Autologous Tissue Substitute, Open Approach */
"02RD08Z"  =  "1"  /* Replacement of Papillary Muscle with Zooplastic Tissue, Open Approach */
"02RD0JZ"  =  "1"  /* Replacement of Papillary Muscle with Synthetic Substitute, Open Approach */
"02RD0KZ"  =  "1"  /* Replacement of Papillary Muscle with Nonautologous Tissue Substitute, Open Approach */
"02RD47Z"  =  "1"  /* Replacement of Papillary Muscle with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RD48Z"  =  "1"  /* Replacement of Papillary Muscle with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RD4JZ"  =  "1"  /* Replacement of Papillary Muscle with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RD4KZ"  =  "1"  /* Replacement of Papillary Muscle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RF07Z"  =  "1"  /* Replacement of Aortic Valve with Autologous Tissue Substitute, Open Approach */
"02RF08Z"  =  "1"  /* Replacement of Aortic Valve with Zooplastic Tissue, Open Approach */
"02RF0JZ"  =  "1"  /* Replacement of Aortic Valve with Synthetic Substitute, Open Approach */
"02RF0KZ"  =  "1"  /* Replacement of Aortic Valve with Nonautologous Tissue Substitute, Open Approach */
"02RF47Z"  =  "1"  /* Replacement of Aortic Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RF48Z"  =  "1"  /* Replacement of Aortic Valve with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RF4JZ"  =  "1"  /* Replacement of Aortic Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RF4KZ"  =  "1"  /* Replacement of Aortic Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RG07Z"  =  "1"  /* Replacement of Mitral Valve with Autologous Tissue Substitute, Open Approach */
"02RG08Z"  =  "1"  /* Replacement of Mitral Valve with Zooplastic Tissue, Open Approach */
"02RG0JZ"  =  "1"  /* Replacement of Mitral Valve with Synthetic Substitute, Open Approach */
"02RG0KZ"  =  "1"  /* Replacement of Mitral Valve with Nonautologous Tissue Substitute, Open Approach */
"02RG47Z"  =  "1"  /* Replacement of Mitral Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RG48Z"  =  "1"  /* Replacement of Mitral Valve with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RG4JZ"  =  "1"  /* Replacement of Mitral Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RG4KZ"  =  "1"  /* Replacement of Mitral Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RH07Z"  =  "1"  /* Replacement of Pulmonary Valve with Autologous Tissue Substitute, Open Approach */
"02RH08Z"  =  "1"  /* Replacement of Pulmonary Valve with Zooplastic Tissue, Open Approach */
"02RH0JZ"  =  "1"  /* Replacement of Pulmonary Valve with Synthetic Substitute, Open Approach */
"02RH0KZ"  =  "1"  /* Replacement of Pulmonary Valve with Nonautologous Tissue Substitute, Open Approach */
"02RH47Z"  =  "1"  /* Replacement of Pulmonary Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RH48Z"  =  "1"  /* Replacement of Pulmonary Valve with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RH4JZ"  =  "1"  /* Replacement of Pulmonary Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RH4KZ"  =  "1"  /* Replacement of Pulmonary Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RJ07Z"  =  "1"  /* Replacement of Tricuspid Valve with Autologous Tissue Substitute, Open Approach */
"02RJ08Z"  =  "1"  /* Replacement of Tricuspid Valve with Zooplastic Tissue, Open Approach */
"02RJ0JZ"  =  "1"  /* Replacement of Tricuspid Valve with Synthetic Substitute, Open Approach */
"02RJ0KZ"  =  "1"  /* Replacement of Tricuspid Valve with Nonautologous Tissue Substitute, Open Approach */
"02RJ47Z"  =  "1"  /* Replacement of Tricuspid Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RJ48Z"  =  "1"  /* Replacement of Tricuspid Valve with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RJ4JZ"  =  "1"  /* Replacement of Tricuspid Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RJ4KZ"  =  "1"  /* Replacement of Tricuspid Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RK07Z"  =  "1"  /* Replacement of Right Ventricle with Autologous Tissue Substitute, Open Approach */
"02RK08Z"  =  "1"  /* Replacement of Right Ventricle with Zooplastic Tissue, Open Approach */
"02RK0JZ"  =  "1"  /* Replacement of Right Ventricle with Synthetic Substitute, Open Approach */
"02RK0KZ"  =  "1"  /* Replacement of Right Ventricle with Nonautologous Tissue Substitute, Open Approach */
"02RK47Z"  =  "1"  /* Replacement of Right Ventricle with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RK48Z"  =  "1"  /* Replacement of Right Ventricle with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RK4JZ"  =  "1"  /* Replacement of Right Ventricle with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RK4KZ"  =  "1"  /* Replacement of Right Ventricle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RL07Z"  =  "1"  /* Replacement of Left Ventricle with Autologous Tissue Substitute, Open Approach */
"02RL08Z"  =  "1"  /* Replacement of Left Ventricle with Zooplastic Tissue, Open Approach */
"02RL0JZ"  =  "1"  /* Replacement of Left Ventricle with Synthetic Substitute, Open Approach */
"02RL0KZ"  =  "1"  /* Replacement of Left Ventricle with Nonautologous Tissue Substitute, Open Approach */
"02RL47Z"  =  "1"  /* Replacement of Left Ventricle with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RL48Z"  =  "1"  /* Replacement of Left Ventricle with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RL4JZ"  =  "1"  /* Replacement of Left Ventricle with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RL4KZ"  =  "1"  /* Replacement of Left Ventricle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RM07Z"  =  "1"  /* Replacement of Ventricular Septum with Autologous Tissue Substitute, Open Approach */
"02RM08Z"  =  "1"  /* Replacement of Ventricular Septum with Zooplastic Tissue, Open Approach */
"02RM0JZ"  =  "1"  /* Replacement of Ventricular Septum with Synthetic Substitute, Open Approach */
"02RM0KZ"  =  "1"  /* Replacement of Ventricular Septum with Nonautologous Tissue Substitute, Open Approach */
"02RM47Z"  =  "1"  /* Replacement of Ventricular Septum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RM48Z"  =  "1"  /* Replacement of Ventricular Septum with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RM4JZ"  =  "1"  /* Replacement of Ventricular Septum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RM4KZ"  =  "1"  /* Replacement of Ventricular Septum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RN07Z"  =  "1"  /* Replacement of Pericardium with Autologous Tissue Substitute, Open Approach */
"02RN08Z"  =  "1"  /* Replacement of Pericardium with Zooplastic Tissue, Open Approach */
"02RN0JZ"  =  "1"  /* Replacement of Pericardium with Synthetic Substitute, Open Approach */
"02RN0KZ"  =  "1"  /* Replacement of Pericardium with Nonautologous Tissue Substitute, Open Approach */
"02RN47Z"  =  "1"  /* Replacement of Pericardium with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RN48Z"  =  "1"  /* Replacement of Pericardium with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RN4JZ"  =  "1"  /* Replacement of Pericardium with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RN4KZ"  =  "1"  /* Replacement of Pericardium with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RP07Z"  =  "1"  /* Replacement of Pulmonary Trunk with Autologous Tissue Substitute, Open Approach */
"02RP08Z"  =  "1"  /* Replacement of Pulmonary Trunk with Zooplastic Tissue, Open Approach */
"02RP0JZ"  =  "1"  /* Replacement of Pulmonary Trunk with Synthetic Substitute, Open Approach */
"02RP0KZ"  =  "1"  /* Replacement of Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach */
"02RP47Z"  =  "1"  /* Replacement of Pulmonary Trunk with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RP48Z"  =  "1"  /* Replacement of Pulmonary Trunk with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RP4JZ"  =  "1"  /* Replacement of Pulmonary Trunk with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RP4KZ"  =  "1"  /* Replacement of Pulmonary Trunk with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RQ07Z"  =  "1"  /* Replacement of Right Pulmonary Artery with Autologous Tissue Substitute, Open Approach */
"02RQ08Z"  =  "1"  /* Replacement of Right Pulmonary Artery with Zooplastic Tissue, Open Approach */
"02RQ0JZ"  =  "1"  /* Replacement of Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"02RQ0KZ"  =  "1"  /* Replacement of Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"02RQ47Z"  =  "1"  /* Replacement of Right Pulmonary Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RQ48Z"  =  "1"  /* Replacement of Right Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RQ4JZ"  =  "1"  /* Replacement of Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RQ4KZ"  =  "1"  /* Replacement of Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RR07Z"  =  "1"  /* Replacement of Left Pulmonary Artery with Autologous Tissue Substitute, Open Approach */
"02RR08Z"  =  "1"  /* Replacement of Left Pulmonary Artery with Zooplastic Tissue, Open Approach */
"02RR0JZ"  =  "1"  /* Replacement of Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"02RR0KZ"  =  "1"  /* Replacement of Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"02RR47Z"  =  "1"  /* Replacement of Left Pulmonary Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RR48Z"  =  "1"  /* Replacement of Left Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RR4JZ"  =  "1"  /* Replacement of Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RR4KZ"  =  "1"  /* Replacement of Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RS07Z"  =  "1"  /* Replacement of Right Pulmonary Vein with Autologous Tissue Substitute, Open Approach */
"02RS08Z"  =  "1"  /* Replacement of Right Pulmonary Vein with Zooplastic Tissue, Open Approach */
"02RS0JZ"  =  "1"  /* Replacement of Right Pulmonary Vein with Synthetic Substitute, Open Approach */
"02RS0KZ"  =  "1"  /* Replacement of Right Pulmonary Vein with Nonautologous Tissue Substitute, Open Approach */
"02RS47Z"  =  "1"  /* Replacement of Right Pulmonary Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RS48Z"  =  "1"  /* Replacement of Right Pulmonary Vein with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RS4JZ"  =  "1"  /* Replacement of Right Pulmonary Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RS4KZ"  =  "1"  /* Replacement of Right Pulmonary Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RT07Z"  =  "1"  /* Replacement of Left Pulmonary Vein with Autologous Tissue Substitute, Open Approach */
"02RT08Z"  =  "1"  /* Replacement of Left Pulmonary Vein with Zooplastic Tissue, Open Approach */
"02RT0JZ"  =  "1"  /* Replacement of Left Pulmonary Vein with Synthetic Substitute, Open Approach */
"02RT0KZ"  =  "1"  /* Replacement of Left Pulmonary Vein with Nonautologous Tissue Substitute, Open Approach */
"02RT47Z"  =  "1"  /* Replacement of Left Pulmonary Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RT48Z"  =  "1"  /* Replacement of Left Pulmonary Vein with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RT4JZ"  =  "1"  /* Replacement of Left Pulmonary Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RT4KZ"  =  "1"  /* Replacement of Left Pulmonary Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RV07Z"  =  "1"  /* Replacement of Superior Vena Cava with Autologous Tissue Substitute, Open Approach */
"02RV08Z"  =  "1"  /* Replacement of Superior Vena Cava with Zooplastic Tissue, Open Approach */
"02RV0JZ"  =  "1"  /* Replacement of Superior Vena Cava with Synthetic Substitute, Open Approach */
"02RV0KZ"  =  "1"  /* Replacement of Superior Vena Cava with Nonautologous Tissue Substitute, Open Approach */
"02RV47Z"  =  "1"  /* Replacement of Superior Vena Cava with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RV48Z"  =  "1"  /* Replacement of Superior Vena Cava with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RV4JZ"  =  "1"  /* Replacement of Superior Vena Cava with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RV4KZ"  =  "1"  /* Replacement of Superior Vena Cava with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RW07Z"  =  "1"  /* Replacement of Thoracic Aorta with Autologous Tissue Substitute, Open Approach */
"02RW08Z"  =  "1"  /* Replacement of Thoracic Aorta with Zooplastic Tissue, Open Approach */
"02RW0JZ"  =  "1"  /* Replacement of Thoracic Aorta with Synthetic Substitute, Open Approach */
"02RW0KZ"  =  "1"  /* Replacement of Thoracic Aorta with Nonautologous Tissue Substitute, Open Approach */
"02RW47Z"  =  "1"  /* Replacement of Thoracic Aorta with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RW48Z"  =  "1"  /* Replacement of Thoracic Aorta with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RW4JZ"  =  "1"  /* Replacement of Thoracic Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RW4KZ"  =  "1"  /* Replacement of Thoracic Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02SP0ZZ"  =  "1"  /* Reposition Pulmonary Trunk, Open Approach */
"02SW0ZZ"  =  "1"  /* Reposition Thoracic Aorta, Open Approach */
"02T50ZZ"  =  "1"  /* Resection of Atrial Septum, Open Approach */
"02T54ZZ"  =  "1"  /* Resection of Atrial Septum, Percutaneous Endoscopic Approach */
"02T80ZZ"  =  "1"  /* Resection of Conduction Mechanism, Open Approach */
"02T84ZZ"  =  "1"  /* Resection of Conduction Mechanism, Percutaneous Endoscopic Approach */
"02T90ZZ"  =  "1"  /* Resection of Chordae Tendineae, Open Approach */
"02T94ZZ"  =  "1"  /* Resection of Chordae Tendineae, Percutaneous Endoscopic Approach */
"02TD0ZZ"  =  "1"  /* Resection of Papillary Muscle, Open Approach */
"02TD4ZZ"  =  "1"  /* Resection of Papillary Muscle, Percutaneous Endoscopic Approach */
"02TH0ZZ"  =  "1"  /* Resection of Pulmonary Valve, Open Approach */
"02TH4ZZ"  =  "1"  /* Resection of Pulmonary Valve, Percutaneous Endoscopic Approach */
"02TM0ZZ"  =  "1"  /* Resection of Ventricular Septum, Open Approach */
"02TM4ZZ"  =  "1"  /* Resection of Ventricular Septum, Percutaneous Endoscopic Approach */
"02TN0ZZ"  =  "1"  /* Resection of Pericardium, Open Approach */
"02TN3ZZ"  =  "1"  /* Resection of Pericardium, Percutaneous Approach */
"02TN4ZZ"  =  "1"  /* Resection of Pericardium, Percutaneous Endoscopic Approach */
"02U507Z"  =  "1"  /* Supplement Atrial Septum with Autologous Tissue Substitute, Open Approach */
"02U508Z"  =  "1"  /* Supplement Atrial Septum with Zooplastic Tissue, Open Approach */
"02U50JZ"  =  "1"  /* Supplement Atrial Septum with Synthetic Substitute, Open Approach */
"02U50KZ"  =  "1"  /* Supplement Atrial Septum with Nonautologous Tissue Substitute, Open Approach */
"02U547Z"  =  "1"  /* Supplement Atrial Septum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02U548Z"  =  "1"  /* Supplement Atrial Septum with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02U54JZ"  =  "1"  /* Supplement Atrial Septum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02U54KZ"  =  "1"  /* Supplement Atrial Septum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02U607Z"  =  "1"  /* Supplement Right Atrium with Autologous Tissue Substitute, Open Approach */
"02U608Z"  =  "1"  /* Supplement Right Atrium with Zooplastic Tissue, Open Approach */
"02U60JZ"  =  "1"  /* Supplement Right Atrium with Synthetic Substitute, Open Approach */
"02U60KZ"  =  "1"  /* Supplement Right Atrium with Nonautologous Tissue Substitute, Open Approach */
"02U647Z"  =  "1"  /* Supplement Right Atrium with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02U648Z"  =  "1"  /* Supplement Right Atrium with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02U64JZ"  =  "1"  /* Supplement Right Atrium with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02U64KZ"  =  "1"  /* Supplement Right Atrium with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02U707Z"  =  "1"  /* Supplement Left Atrium with Autologous Tissue Substitute, Open Approach */
"02U708Z"  =  "1"  /* Supplement Left Atrium with Zooplastic Tissue, Open Approach */
"02U70JZ"  =  "1"  /* Supplement Left Atrium with Synthetic Substitute, Open Approach */
"02U70KZ"  =  "1"  /* Supplement Left Atrium with Nonautologous Tissue Substitute, Open Approach */
"02U747Z"  =  "1"  /* Supplement Left Atrium with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02U748Z"  =  "1"  /* Supplement Left Atrium with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02U74KZ"  =  "1"  /* Supplement Left Atrium with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02U907Z"  =  "1"  /* Supplement Chordae Tendineae with Autologous Tissue Substitute, Open Approach */
"02U908Z"  =  "1"  /* Supplement Chordae Tendineae with Zooplastic Tissue, Open Approach */
"02U90JZ"  =  "1"  /* Supplement Chordae Tendineae with Synthetic Substitute, Open Approach */
"02U90KZ"  =  "1"  /* Supplement Chordae Tendineae with Nonautologous Tissue Substitute, Open Approach */
"02U947Z"  =  "1"  /* Supplement Chordae Tendineae with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02U948Z"  =  "1"  /* Supplement Chordae Tendineae with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02U94JZ"  =  "1"  /* Supplement Chordae Tendineae with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02U94KZ"  =  "1"  /* Supplement Chordae Tendineae with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UA07Z"  =  "1"  /* Supplement Heart with Autologous Tissue Substitute, Open Approach */
"02UA08Z"  =  "1"  /* Supplement Heart with Zooplastic Tissue, Open Approach */
"02UA0JZ"  =  "1"  /* Supplement Heart with Synthetic Substitute, Open Approach */
"02UA0KZ"  =  "1"  /* Supplement Heart with Nonautologous Tissue Substitute, Open Approach */
"02UA47Z"  =  "1"  /* Supplement Heart with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UA48Z"  =  "1"  /* Supplement Heart with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UA4JZ"  =  "1"  /* Supplement Heart with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UA4KZ"  =  "1"  /* Supplement Heart with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UD07Z"  =  "1"  /* Supplement Papillary Muscle with Autologous Tissue Substitute, Open Approach */
"02UD08Z"  =  "1"  /* Supplement Papillary Muscle with Zooplastic Tissue, Open Approach */
"02UD0JZ"  =  "1"  /* Supplement Papillary Muscle with Synthetic Substitute, Open Approach */
"02UD0KZ"  =  "1"  /* Supplement Papillary Muscle with Nonautologous Tissue Substitute, Open Approach */
"02UD47Z"  =  "1"  /* Supplement Papillary Muscle with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UD48Z"  =  "1"  /* Supplement Papillary Muscle with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UD4JZ"  =  "1"  /* Supplement Papillary Muscle with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UD4KZ"  =  "1"  /* Supplement Papillary Muscle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UF07Z"  =  "1"  /* Supplement Aortic Valve with Autologous Tissue Substitute, Open Approach */
"02UF08Z"  =  "1"  /* Supplement Aortic Valve with Zooplastic Tissue, Open Approach */
"02UF0JZ"  =  "1"  /* Supplement Aortic Valve with Synthetic Substitute, Open Approach */
"02UF0KZ"  =  "1"  /* Supplement Aortic Valve with Nonautologous Tissue Substitute, Open Approach */
"02UF47Z"  =  "1"  /* Supplement Aortic Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UF48Z"  =  "1"  /* Supplement Aortic Valve with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UF4JZ"  =  "1"  /* Supplement Aortic Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UF4KZ"  =  "1"  /* Supplement Aortic Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UG07Z"  =  "1"  /* Supplement Mitral Valve with Autologous Tissue Substitute, Open Approach */
"02UG08Z"  =  "1"  /* Supplement Mitral Valve with Zooplastic Tissue, Open Approach */
"02UG0JZ"  =  "1"  /* Supplement Mitral Valve with Synthetic Substitute, Open Approach */
"02UG0KZ"  =  "1"  /* Supplement Mitral Valve with Nonautologous Tissue Substitute, Open Approach */
"02UG47Z"  =  "1"  /* Supplement Mitral Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UG48Z"  =  "1"  /* Supplement Mitral Valve with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UG4JZ"  =  "1"  /* Supplement Mitral Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UG4KZ"  =  "1"  /* Supplement Mitral Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UH07Z"  =  "1"  /* Supplement Pulmonary Valve with Autologous Tissue Substitute, Open Approach */
"02UH08Z"  =  "1"  /* Supplement Pulmonary Valve with Zooplastic Tissue, Open Approach */
"02UH0JZ"  =  "1"  /* Supplement Pulmonary Valve with Synthetic Substitute, Open Approach */
"02UH0KZ"  =  "1"  /* Supplement Pulmonary Valve with Nonautologous Tissue Substitute, Open Approach */
"02UH47Z"  =  "1"  /* Supplement Pulmonary Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UH48Z"  =  "1"  /* Supplement Pulmonary Valve with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UH4JZ"  =  "1"  /* Supplement Pulmonary Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UH4KZ"  =  "1"  /* Supplement Pulmonary Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UJ07Z"  =  "1"  /* Supplement Tricuspid Valve with Autologous Tissue Substitute, Open Approach */
"02UJ08Z"  =  "1"  /* Supplement Tricuspid Valve with Zooplastic Tissue, Open Approach */
"02UJ0JZ"  =  "1"  /* Supplement Tricuspid Valve with Synthetic Substitute, Open Approach */
"02UJ0KZ"  =  "1"  /* Supplement Tricuspid Valve with Nonautologous Tissue Substitute, Open Approach */
"02UJ47Z"  =  "1"  /* Supplement Tricuspid Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UJ48Z"  =  "1"  /* Supplement Tricuspid Valve with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UJ4JZ"  =  "1"  /* Supplement Tricuspid Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UJ4KZ"  =  "1"  /* Supplement Tricuspid Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UK07Z"  =  "1"  /* Supplement Right Ventricle with Autologous Tissue Substitute, Open Approach */
"02UK08Z"  =  "1"  /* Supplement Right Ventricle with Zooplastic Tissue, Open Approach */
"02UK0JZ"  =  "1"  /* Supplement Right Ventricle with Synthetic Substitute, Open Approach */
"02UK0KZ"  =  "1"  /* Supplement Right Ventricle with Nonautologous Tissue Substitute, Open Approach */
"02UK47Z"  =  "1"  /* Supplement Right Ventricle with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UK48Z"  =  "1"  /* Supplement Right Ventricle with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UK4JZ"  =  "1"  /* Supplement Right Ventricle with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UK4KZ"  =  "1"  /* Supplement Right Ventricle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UL07Z"  =  "1"  /* Supplement Left Ventricle with Autologous Tissue Substitute, Open Approach */
"02UL08Z"  =  "1"  /* Supplement Left Ventricle with Zooplastic Tissue, Open Approach */
"02UL0JZ"  =  "1"  /* Supplement Left Ventricle with Synthetic Substitute, Open Approach */
"02UL0KZ"  =  "1"  /* Supplement Left Ventricle with Nonautologous Tissue Substitute, Open Approach */
"02UL47Z"  =  "1"  /* Supplement Left Ventricle with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UL48Z"  =  "1"  /* Supplement Left Ventricle with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UL4JZ"  =  "1"  /* Supplement Left Ventricle with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UL4KZ"  =  "1"  /* Supplement Left Ventricle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UM07Z"  =  "1"  /* Supplement Ventricular Septum with Autologous Tissue Substitute, Open Approach */
"02UM08Z"  =  "1"  /* Supplement Ventricular Septum with Zooplastic Tissue, Open Approach */
"02UM0JZ"  =  "1"  /* Supplement Ventricular Septum with Synthetic Substitute, Open Approach */
"02UM0KZ"  =  "1"  /* Supplement Ventricular Septum with Nonautologous Tissue Substitute, Open Approach */
"02UM47Z"  =  "1"  /* Supplement Ventricular Septum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UM48Z"  =  "1"  /* Supplement Ventricular Septum with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UM4JZ"  =  "1"  /* Supplement Ventricular Septum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UM4KZ"  =  "1"  /* Supplement Ventricular Septum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UN07Z"  =  "1"  /* Supplement Pericardium with Autologous Tissue Substitute, Open Approach */
"02UN08Z"  =  "1"  /* Supplement Pericardium with Zooplastic Tissue, Open Approach */
"02UN0JZ"  =  "1"  /* Supplement Pericardium with Synthetic Substitute, Open Approach */
"02UN0KZ"  =  "1"  /* Supplement Pericardium with Nonautologous Tissue Substitute, Open Approach */
"02UN37Z"  =  "1"  /* Supplement Pericardium with Autologous Tissue Substitute, Percutaneous Approach */
"02UN38Z"  =  "1"  /* Supplement Pericardium with Zooplastic Tissue, Percutaneous Approach */
"02UN3JZ"  =  "1"  /* Supplement Pericardium with Synthetic Substitute, Percutaneous Approach */
"02UN3KZ"  =  "1"  /* Supplement Pericardium with Nonautologous Tissue Substitute, Percutaneous Approach */
"02UN47Z"  =  "1"  /* Supplement Pericardium with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UN48Z"  =  "1"  /* Supplement Pericardium with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UN4JZ"  =  "1"  /* Supplement Pericardium with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UN4KZ"  =  "1"  /* Supplement Pericardium with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02VA0CZ"  =  "1"  /* Restriction of Heart with Extraluminal Device, Open Approach */
"02VA0ZZ"  =  "1"  /* Restriction of Heart, Open Approach */
"02VA3CZ"  =  "1"  /* Restriction of Heart with Extraluminal Device, Percutaneous Approach */
"02VA4CZ"  =  "1"  /* Restriction of Heart with Extraluminal Device, Percutaneous Endoscopic Approach */
"02VA4ZZ"  =  "1"  /* Restriction of Heart, Percutaneous Endoscopic Approach */
"02VQ0CZ"  =  "1"  /* Restriction of Right Pulmonary Artery with Extraluminal Device, Open Approach */
"02VQ3CZ"  =  "1"  /* Restriction of Right Pulmonary Artery with Extraluminal Device, Percutaneous Approach */
"02VQ4CZ"  =  "1"  /* Restriction of Right Pulmonary Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"02VR0CZ"  =  "1"  /* Restriction of Left Pulmonary Artery with Extraluminal Device, Open Approach */
"02VR0ZT"  =  "1"  /* Restriction of Ductus Arteriosus, Open Approach */
"02VR3CZ"  =  "1"  /* Restriction of Left Pulmonary Artery with Extraluminal Device, Percutaneous Approach */
"02VR4CZ"  =  "1"  /* Restriction of Left Pulmonary Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"02W50JZ"  =  "1"  /* Revision of Synthetic Substitute in Atrial Septum, Open Approach */
"02W54JZ"  =  "1"  /* Revision of Synthetic Substitute in Atrial Septum, Percutaneous Endoscopic Approach */
"02WA02Z"  =  "1"  /* Revision of Monitoring Device in Heart, Open Approach */
"02WA03Z"  =  "1"  /* Revision of Infusion Device in Heart, Open Approach */
"02WA07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Heart, Open Approach */
"02WA08Z"  =  "1"  /* Revision of Zooplastic Tissue in Heart, Open Approach */
"02WA0CZ"  =  "1"  /* Revision of Extraluminal Device in Heart, Open Approach */
"02WA0DZ"  =  "1"  /* Revision of Intraluminal Device in Heart, Open Approach */
"02WA0JZ"  =  "1"  /* Revision of Synthetic Substitute in Heart, Open Approach */
"02WA0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Heart, Open Approach */
"02WA0QZ"  =  "1"  /* Revision of Implantable Heart Assist System in Heart, Open Approach */
"02WA0RZ"  =  "1"  /* Revision of External Heart Assist System in Heart, Open Approach */
"02WA3CZ"  =  "1"  /* Revision of Extraluminal Device in Heart, Percutaneous Approach */
"02WA3RZ"  =  "1"  /* Revision of External Heart Assist System in Heart, Percutaneous Approach */
"02WA42Z"  =  "1"  /* Revision of Monitoring Device in Heart, Percutaneous Endoscopic Approach */
"02WA43Z"  =  "1"  /* Revision of Infusion Device in Heart, Percutaneous Endoscopic Approach */
"02WA47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Heart, Percutaneous Endoscopic Approach */
"02WA48Z"  =  "1"  /* Revision of Zooplastic Tissue in Heart, Percutaneous Endoscopic Approach */
"02WA4CZ"  =  "1"  /* Revision of Extraluminal Device in Heart, Percutaneous Endoscopic Approach */
"02WA4DZ"  =  "1"  /* Revision of Intraluminal Device in Heart, Percutaneous Endoscopic Approach */
"02WA4JZ"  =  "1"  /* Revision of Synthetic Substitute in Heart, Percutaneous Endoscopic Approach */
"02WA4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Heart, Percutaneous Endoscopic Approach */
"02WA4QZ"  =  "1"  /* Revision of Implantable Heart Assist System in Heart, Percutaneous Endoscopic Approach */
"02WA4RZ"  =  "1"  /* Revision of External Heart Assist System in Heart, Percutaneous Endoscopic Approach */
"02WF07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Aortic Valve, Open Approach */
"02WF08Z"  =  "1"  /* Revision of Zooplastic Tissue in Aortic Valve, Open Approach */
"02WF0JZ"  =  "1"  /* Revision of Synthetic Substitute in Aortic Valve, Open Approach */
"02WF0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Aortic Valve, Open Approach */
"02WF47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Aortic Valve, Percutaneous Endoscopic Approach */
"02WF48Z"  =  "1"  /* Revision of Zooplastic Tissue in Aortic Valve, Percutaneous Endoscopic Approach */
"02WF4JZ"  =  "1"  /* Revision of Synthetic Substitute in Aortic Valve, Percutaneous Endoscopic Approach */
"02WF4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Aortic Valve, Percutaneous Endoscopic Approach */
"02WG07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mitral Valve, Open Approach */
"02WG08Z"  =  "1"  /* Revision of Zooplastic Tissue in Mitral Valve, Open Approach */
"02WG0JZ"  =  "1"  /* Revision of Synthetic Substitute in Mitral Valve, Open Approach */
"02WG0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mitral Valve, Open Approach */
"02WG47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mitral Valve, Percutaneous Endoscopic Approach */
"02WG48Z"  =  "1"  /* Revision of Zooplastic Tissue in Mitral Valve, Percutaneous Endoscopic Approach */
"02WG4JZ"  =  "1"  /* Revision of Synthetic Substitute in Mitral Valve, Percutaneous Endoscopic Approach */
"02WG4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mitral Valve, Percutaneous Endoscopic Approach */
"02WH07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pulmonary Valve, Open Approach */
"02WH08Z"  =  "1"  /* Revision of Zooplastic Tissue in Pulmonary Valve, Open Approach */
"02WH0JZ"  =  "1"  /* Revision of Synthetic Substitute in Pulmonary Valve, Open Approach */
"02WH0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pulmonary Valve, Open Approach */
"02WH47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pulmonary Valve, Percutaneous Endoscopic Approach */
"02WH48Z"  =  "1"  /* Revision of Zooplastic Tissue in Pulmonary Valve, Percutaneous Endoscopic Approach */
"02WH4JZ"  =  "1"  /* Revision of Synthetic Substitute in Pulmonary Valve, Percutaneous Endoscopic Approach */
"02WH4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pulmonary Valve, Percutaneous Endoscopic Approach */
"02WJ07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Tricuspid Valve, Open Approach */
"02WJ08Z"  =  "1"  /* Revision of Zooplastic Tissue in Tricuspid Valve, Open Approach */
"02WJ0JZ"  =  "1"  /* Revision of Synthetic Substitute in Tricuspid Valve, Open Approach */
"02WJ0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Tricuspid Valve, Open Approach */
"02WJ47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Tricuspid Valve, Percutaneous Endoscopic Approach */
"02WJ48Z"  =  "1"  /* Revision of Zooplastic Tissue in Tricuspid Valve, Percutaneous Endoscopic Approach */
"02WJ4JZ"  =  "1"  /* Revision of Synthetic Substitute in Tricuspid Valve, Percutaneous Endoscopic Approach */
"02WJ4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Tricuspid Valve, Percutaneous Endoscopic Approach */
"02WM0JZ"  =  "1"  /* Revision of Synthetic Substitute in Ventricular Septum, Open Approach */
"02WM4JZ"  =  "1"  /* Revision of Synthetic Substitute in Ventricular Septum, Percutaneous Endoscopic Approach */
"02WY02Z"  =  "1"  /* Revision of Monitoring Device in Great Vessel, Open Approach */
"02WY03Z"  =  "1"  /* Revision of Infusion Device in Great Vessel, Open Approach */
"02WY07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Great Vessel, Open Approach */
"02WY08Z"  =  "1"  /* Revision of Zooplastic Tissue in Great Vessel, Open Approach */
"02WY0CZ"  =  "1"  /* Revision of Extraluminal Device in Great Vessel, Open Approach */
"02WY0DZ"  =  "1"  /* Revision of Intraluminal Device in Great Vessel, Open Approach */
"02WY0JZ"  =  "1"  /* Revision of Synthetic Substitute in Great Vessel, Open Approach */
"02WY0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Great Vessel, Open Approach */
"02WY3CZ"  =  "1"  /* Revision of Extraluminal Device in Great Vessel, Percutaneous Approach */
"02WY42Z"  =  "1"  /* Revision of Monitoring Device in Great Vessel, Percutaneous Endoscopic Approach */
"02WY43Z"  =  "1"  /* Revision of Infusion Device in Great Vessel, Percutaneous Endoscopic Approach */
"02WY47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Great Vessel, Percutaneous Endoscopic Approach */
"02WY48Z"  =  "1"  /* Revision of Zooplastic Tissue in Great Vessel, Percutaneous Endoscopic Approach */
"02WY4CZ"  =  "1"  /* Revision of Extraluminal Device in Great Vessel, Percutaneous Endoscopic Approach */
"02WY4DZ"  =  "1"  /* Revision of Intraluminal Device in Great Vessel, Percutaneous Endoscopic Approach */
"02WY4JZ"  =  "1"  /* Revision of Synthetic Substitute in Great Vessel, Percutaneous Endoscopic Approach */
"02WY4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Great Vessel, Percutaneous Endoscopic Approach */
"02YA0Z0"  =  "1"  /* Transplantation of Heart, Allogeneic, Open Approach */
"02YA0Z1"  =  "1"  /* Transplantation of Heart, Syngeneic, Open Approach */
"02YA0Z2"  =  "1"  /* Transplantation of Heart, Zooplastic, Open Approach */
"031309M"  =  "1"  /* Bypass Right Subclavian Artery to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"031309N"  =  "1"  /* Bypass Right Subclavian Artery to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"03130AM"  =  "1"  /* Bypass Right Subclavian Artery to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"03130AN"  =  "1"  /* Bypass Right Subclavian Artery to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"03130JM"  =  "1"  /* Bypass Right Subclavian Artery to Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"03130JN"  =  "1"  /* Bypass Right Subclavian Artery to Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"03130KM"  =  "1"  /* Bypass Right Subclavian Artery to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"03130KN"  =  "1"  /* Bypass Right Subclavian Artery to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"03130ZM"  =  "1"  /* Bypass Right Subclavian Artery to Right Pulmonary Artery, Open Approach */
"03130ZN"  =  "1"  /* Bypass Right Subclavian Artery to Left Pulmonary Artery, Open Approach */
"031409M"  =  "1"  /* Bypass Left Subclavian Artery to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"031409N"  =  "1"  /* Bypass Left Subclavian Artery to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"03140AM"  =  "1"  /* Bypass Left Subclavian Artery to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"03140AN"  =  "1"  /* Bypass Left Subclavian Artery to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"03140JM"  =  "1"  /* Bypass Left Subclavian Artery to Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"03140JN"  =  "1"  /* Bypass Left Subclavian Artery to Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"03140KM"  =  "1"  /* Bypass Left Subclavian Artery to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"03140KN"  =  "1"  /* Bypass Left Subclavian Artery to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"03140ZM"  =  "1"  /* Bypass Left Subclavian Artery to Right Pulmonary Artery, Open Approach */
"03140ZN"  =  "1"  /* Bypass Left Subclavian Artery to Left Pulmonary Artery, Open Approach */
"031H09J"  =  "1"  /* Bypass Right Common Carotid Artery to Right Extracranial Artery with Autologous Venous Tissue, Open Approach */
"031H0AJ"  =  "1"  /* Bypass Right Common Carotid Artery to Right Extracranial Artery with Autologous Arterial Tissue, Open Approach */
"031H0JJ"  =  "1"  /* Bypass Right Common Carotid Artery to Right Extracranial Artery with Synthetic Substitute, Open Approach */
"031H0KJ"  =  "1"  /* Bypass Right Common Carotid Artery to Right Extracranial Artery with Nonautologous Tissue Substitute, Open Approach */
"031H0ZJ"  =  "1"  /* Bypass Right Common Carotid Artery to Right Extracranial Artery, Open Approach */
"031J09K"  =  "1"  /* Bypass Left Common Carotid Artery to Left Extracranial Artery with Autologous Venous Tissue, Open Approach */
"031J0AK"  =  "1"  /* Bypass Left Common Carotid Artery to Left Extracranial Artery with Autologous Arterial Tissue, Open Approach */
"031J0JK"  =  "1"  /* Bypass Left Common Carotid Artery to Left Extracranial Artery with Synthetic Substitute, Open Approach */
"031J0KK"  =  "1"  /* Bypass Left Common Carotid Artery to Left Extracranial Artery with Nonautologous Tissue Substitute, Open Approach */
"031J0ZK"  =  "1"  /* Bypass Left Common Carotid Artery to Left Extracranial Artery, Open Approach */
"03500ZZ"  =  "1"  /* Destruction of Right Internal Mammary Artery, Open Approach */
"03503ZZ"  =  "1"  /* Destruction of Right Internal Mammary Artery, Percutaneous Approach */
"03504ZZ"  =  "1"  /* Destruction of Right Internal Mammary Artery, Percutaneous Endoscopic Approach */
"03510ZZ"  =  "1"  /* Destruction of Left Internal Mammary Artery, Open Approach */
"03513ZZ"  =  "1"  /* Destruction of Left Internal Mammary Artery, Percutaneous Approach */
"03514ZZ"  =  "1"  /* Destruction of Left Internal Mammary Artery, Percutaneous Endoscopic Approach */
"03520ZZ"  =  "1"  /* Destruction of Innominate Artery, Open Approach */
"03524ZZ"  =  "1"  /* Destruction of Innominate Artery, Percutaneous Endoscopic Approach */
"03530ZZ"  =  "1"  /* Destruction of Right Subclavian Artery, Open Approach */
"03534ZZ"  =  "1"  /* Destruction of Right Subclavian Artery, Percutaneous Endoscopic Approach */
"03540ZZ"  =  "1"  /* Destruction of Left Subclavian Artery, Open Approach */
"03544ZZ"  =  "1"  /* Destruction of Left Subclavian Artery, Percutaneous Endoscopic Approach */
"03B00ZZ"  =  "1"  /* Excision of Right Internal Mammary Artery, Open Approach */
"03B03ZZ"  =  "1"  /* Excision of Right Internal Mammary Artery, Percutaneous Approach */
"03B04ZZ"  =  "1"  /* Excision of Right Internal Mammary Artery, Percutaneous Endoscopic Approach */
"03B10ZZ"  =  "1"  /* Excision of Left Internal Mammary Artery, Open Approach */
"03B13ZZ"  =  "1"  /* Excision of Left Internal Mammary Artery, Percutaneous Approach */
"03B14ZZ"  =  "1"  /* Excision of Left Internal Mammary Artery, Percutaneous Endoscopic Approach */
"03B20ZZ"  =  "1"  /* Excision of Innominate Artery, Open Approach */
"03B24ZZ"  =  "1"  /* Excision of Innominate Artery, Percutaneous Endoscopic Approach */
"03B30ZZ"  =  "1"  /* Excision of Right Subclavian Artery, Open Approach */
"03B34ZZ"  =  "1"  /* Excision of Right Subclavian Artery, Percutaneous Endoscopic Approach */
"03B40ZZ"  =  "1"  /* Excision of Left Subclavian Artery, Open Approach */
"03B44ZZ"  =  "1"  /* Excision of Left Subclavian Artery, Percutaneous Endoscopic Approach */
"03C00ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Mammary Artery, Open Approach */
"03C04ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Mammary Artery, Percutaneous Endoscopic Approach */
"03C10ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Mammary Artery, Open Approach */
"03C14ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Mammary Artery, Percutaneous Endoscopic Approach */
"03C20ZZ"  =  "1"  /* Extirpation of Matter from Innominate Artery, Open Approach */
"03C24ZZ"  =  "1"  /* Extirpation of Matter from Innominate Artery, Percutaneous Endoscopic Approach */
"03C30ZZ"  =  "1"  /* Extirpation of Matter from Right Subclavian Artery, Open Approach */
"03C34ZZ"  =  "1"  /* Extirpation of Matter from Right Subclavian Artery, Percutaneous Endoscopic Approach */
"03C40ZZ"  =  "1"  /* Extirpation of Matter from Left Subclavian Artery, Open Approach */
"03C44ZZ"  =  "1"  /* Extirpation of Matter from Left Subclavian Artery, Percutaneous Endoscopic Approach */
"03L00CZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery with Extraluminal Device, Open Approach */
"03L00DZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery with Intraluminal Device, Open Approach */
"03L00ZZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery, Open Approach */
"03L03CZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery with Extraluminal Device, Percutaneous Approach */
"03L03ZZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery, Percutaneous Approach */
"03L04CZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03L04DZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03L04ZZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery, Percutaneous Endoscopic Approach */
"03L10CZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery with Extraluminal Device, Open Approach */
"03L10DZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery with Intraluminal Device, Open Approach */
"03L10ZZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery, Open Approach */
"03L13CZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery with Extraluminal Device, Percutaneous Approach */
"03L13ZZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery, Percutaneous Approach */
"03L14CZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03L14DZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03L14ZZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery, Percutaneous Endoscopic Approach */
"03L20CZ"  =  "1"  /* Occlusion of Innominate Artery with Extraluminal Device, Open Approach */
"03L20DZ"  =  "1"  /* Occlusion of Innominate Artery with Intraluminal Device, Open Approach */
"03L20ZZ"  =  "1"  /* Occlusion of Innominate Artery, Open Approach */
"03L23CZ"  =  "1"  /* Occlusion of Innominate Artery with Extraluminal Device, Percutaneous Approach */
"03L23ZZ"  =  "1"  /* Occlusion of Innominate Artery, Percutaneous Approach */
"03L24CZ"  =  "1"  /* Occlusion of Innominate Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03L24DZ"  =  "1"  /* Occlusion of Innominate Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03L24ZZ"  =  "1"  /* Occlusion of Innominate Artery, Percutaneous Endoscopic Approach */
"03L30CZ"  =  "1"  /* Occlusion of Right Subclavian Artery with Extraluminal Device, Open Approach */
"03L30DZ"  =  "1"  /* Occlusion of Right Subclavian Artery with Intraluminal Device, Open Approach */
"03L30ZZ"  =  "1"  /* Occlusion of Right Subclavian Artery, Open Approach */
"03L33CZ"  =  "1"  /* Occlusion of Right Subclavian Artery with Extraluminal Device, Percutaneous Approach */
"03L33ZZ"  =  "1"  /* Occlusion of Right Subclavian Artery, Percutaneous Approach */
"03L34CZ"  =  "1"  /* Occlusion of Right Subclavian Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03L34DZ"  =  "1"  /* Occlusion of Right Subclavian Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03L34ZZ"  =  "1"  /* Occlusion of Right Subclavian Artery, Percutaneous Endoscopic Approach */
"03L40CZ"  =  "1"  /* Occlusion of Left Subclavian Artery with Extraluminal Device, Open Approach */
"03L40DZ"  =  "1"  /* Occlusion of Left Subclavian Artery with Intraluminal Device, Open Approach */
"03L40ZZ"  =  "1"  /* Occlusion of Left Subclavian Artery, Open Approach */
"03L43CZ"  =  "1"  /* Occlusion of Left Subclavian Artery with Extraluminal Device, Percutaneous Approach */
"03L43ZZ"  =  "1"  /* Occlusion of Left Subclavian Artery, Percutaneous Approach */
"03L44CZ"  =  "1"  /* Occlusion of Left Subclavian Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03L44DZ"  =  "1"  /* Occlusion of Left Subclavian Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03L44ZZ"  =  "1"  /* Occlusion of Left Subclavian Artery, Percutaneous Endoscopic Approach */
"03R007Z"  =  "1"  /* Replacement of Right Internal Mammary Artery with Autologous Tissue Substitute, Open Approach */
"03R00JZ"  =  "1"  /* Replacement of Right Internal Mammary Artery with Synthetic Substitute, Open Approach */
"03R00KZ"  =  "1"  /* Replacement of Right Internal Mammary Artery with Nonautologous Tissue Substitute, Open Approach */
"03R047Z"  =  "1"  /* Replacement of Right Internal Mammary Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"03R04JZ"  =  "1"  /* Replacement of Right Internal Mammary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"03R04KZ"  =  "1"  /* Replacement of Right Internal Mammary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"03R107Z"  =  "1"  /* Replacement of Left Internal Mammary Artery with Autologous Tissue Substitute, Open Approach */
"03R10JZ"  =  "1"  /* Replacement of Left Internal Mammary Artery with Synthetic Substitute, Open Approach */
"03R10KZ"  =  "1"  /* Replacement of Left Internal Mammary Artery with Nonautologous Tissue Substitute, Open Approach */
"03R147Z"  =  "1"  /* Replacement of Left Internal Mammary Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"03R14JZ"  =  "1"  /* Replacement of Left Internal Mammary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"03R14KZ"  =  "1"  /* Replacement of Left Internal Mammary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"03R207Z"  =  "1"  /* Replacement of Innominate Artery with Autologous Tissue Substitute, Open Approach */
"03R20JZ"  =  "1"  /* Replacement of Innominate Artery with Synthetic Substitute, Open Approach */
"03R20KZ"  =  "1"  /* Replacement of Innominate Artery with Nonautologous Tissue Substitute, Open Approach */
"03R247Z"  =  "1"  /* Replacement of Innominate Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"03R24JZ"  =  "1"  /* Replacement of Innominate Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"03R24KZ"  =  "1"  /* Replacement of Innominate Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"03R307Z"  =  "1"  /* Replacement of Right Subclavian Artery with Autologous Tissue Substitute, Open Approach */
"03R30JZ"  =  "1"  /* Replacement of Right Subclavian Artery with Synthetic Substitute, Open Approach */
"03R30KZ"  =  "1"  /* Replacement of Right Subclavian Artery with Nonautologous Tissue Substitute, Open Approach */
"03R347Z"  =  "1"  /* Replacement of Right Subclavian Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"03R34JZ"  =  "1"  /* Replacement of Right Subclavian Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"03R34KZ"  =  "1"  /* Replacement of Right Subclavian Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"03R407Z"  =  "1"  /* Replacement of Left Subclavian Artery with Autologous Tissue Substitute, Open Approach */
"03R40JZ"  =  "1"  /* Replacement of Left Subclavian Artery with Synthetic Substitute, Open Approach */
"03R40KZ"  =  "1"  /* Replacement of Left Subclavian Artery with Nonautologous Tissue Substitute, Open Approach */
"03R447Z"  =  "1"  /* Replacement of Left Subclavian Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"03R44JZ"  =  "1"  /* Replacement of Left Subclavian Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"03R44KZ"  =  "1"  /* Replacement of Left Subclavian Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04500ZZ"  =  "1"  /* Destruction of Abdominal Aorta, Open Approach */
"04504ZZ"  =  "1"  /* Destruction of Abdominal Aorta, Percutaneous Endoscopic Approach */
"04B00ZZ"  =  "1"  /* Excision of Abdominal Aorta, Open Approach */
"04B04ZZ"  =  "1"  /* Excision of Abdominal Aorta, Percutaneous Endoscopic Approach */
"04C00ZZ"  =  "1"  /* Extirpation of Matter from Abdominal Aorta, Open Approach */
"04C04ZZ"  =  "1"  /* Extirpation of Matter from Abdominal Aorta, Percutaneous Endoscopic Approach */
"04L00CZ"  =  "1"  /* Occlusion of Abdominal Aorta with Extraluminal Device, Open Approach */
"04L00DZ"  =  "1"  /* Occlusion of Abdominal Aorta with Intraluminal Device, Open Approach */
"04L00ZZ"  =  "1"  /* Occlusion of Abdominal Aorta, Open Approach */
"04L03CZ"  =  "1"  /* Occlusion of Abdominal Aorta with Extraluminal Device, Percutaneous Approach */
"04L03ZZ"  =  "1"  /* Occlusion of Abdominal Aorta, Percutaneous Approach */
"04L04CZ"  =  "1"  /* Occlusion of Abdominal Aorta with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L04DZ"  =  "1"  /* Occlusion of Abdominal Aorta with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L04ZZ"  =  "1"  /* Occlusion of Abdominal Aorta, Percutaneous Endoscopic Approach */
"04R007Z"  =  "1"  /* Replacement of Abdominal Aorta with Autologous Tissue Substitute, Open Approach */
"04R00JZ"  =  "1"  /* Replacement of Abdominal Aorta with Synthetic Substitute, Open Approach */
"04R00KZ"  =  "1"  /* Replacement of Abdominal Aorta with Nonautologous Tissue Substitute, Open Approach */
"04R047Z"  =  "1"  /* Replacement of Abdominal Aorta with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R04JZ"  =  "1"  /* Replacement of Abdominal Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04R04KZ"  =  "1"  /* Replacement of Abdominal Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"051007Y"  =  "1"  /* Bypass Azygos Vein to Upper Vein with Autologous Tissue Substitute, Open Approach */
"051009Y"  =  "1"  /* Bypass Azygos Vein to Upper Vein with Autologous Venous Tissue, Open Approach */
"05100AY"  =  "1"  /* Bypass Azygos Vein to Upper Vein with Autologous Arterial Tissue, Open Approach */
"05100JY"  =  "1"  /* Bypass Azygos Vein to Upper Vein with Synthetic Substitute, Open Approach */
"05100KY"  =  "1"  /* Bypass Azygos Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach */
"05100ZY"  =  "1"  /* Bypass Azygos Vein to Upper Vein, Open Approach */
"051047Y"  =  "1"  /* Bypass Azygos Vein to Upper Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"051049Y"  =  "1"  /* Bypass Azygos Vein to Upper Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"05104AY"  =  "1"  /* Bypass Azygos Vein to Upper Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"05104JY"  =  "1"  /* Bypass Azygos Vein to Upper Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"05104KY"  =  "1"  /* Bypass Azygos Vein to Upper Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05104ZY"  =  "1"  /* Bypass Azygos Vein to Upper Vein, Percutaneous Endoscopic Approach */
"051107Y"  =  "1"  /* Bypass Hemiazygos Vein to Upper Vein with Autologous Tissue Substitute, Open Approach */
"051109Y"  =  "1"  /* Bypass Hemiazygos Vein to Upper Vein with Autologous Venous Tissue, Open Approach */
"05110AY"  =  "1"  /* Bypass Hemiazygos Vein to Upper Vein with Autologous Arterial Tissue, Open Approach */
"05110JY"  =  "1"  /* Bypass Hemiazygos Vein to Upper Vein with Synthetic Substitute, Open Approach */
"05110KY"  =  "1"  /* Bypass Hemiazygos Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach */
"05110ZY"  =  "1"  /* Bypass Hemiazygos Vein to Upper Vein, Open Approach */
"051147Y"  =  "1"  /* Bypass Hemiazygos Vein to Upper Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"051149Y"  =  "1"  /* Bypass Hemiazygos Vein to Upper Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"05114AY"  =  "1"  /* Bypass Hemiazygos Vein to Upper Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"05114JY"  =  "1"  /* Bypass Hemiazygos Vein to Upper Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"05114KY"  =  "1"  /* Bypass Hemiazygos Vein to Upper Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05114ZY"  =  "1"  /* Bypass Hemiazygos Vein to Upper Vein, Percutaneous Endoscopic Approach */
"051307Y"  =  "1"  /* Bypass Right Innominate Vein to Upper Vein with Autologous Tissue Substitute, Open Approach */
"051309Y"  =  "1"  /* Bypass Right Innominate Vein to Upper Vein with Autologous Venous Tissue, Open Approach */
"05130AY"  =  "1"  /* Bypass Right Innominate Vein to Upper Vein with Autologous Arterial Tissue, Open Approach */
"05130JY"  =  "1"  /* Bypass Right Innominate Vein to Upper Vein with Synthetic Substitute, Open Approach */
"05130KY"  =  "1"  /* Bypass Right Innominate Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach */
"05130ZY"  =  "1"  /* Bypass Right Innominate Vein to Upper Vein, Open Approach */
"051347Y"  =  "1"  /* Bypass Right Innominate Vein to Upper Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"051349Y"  =  "1"  /* Bypass Right Innominate Vein to Upper Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"05134AY"  =  "1"  /* Bypass Right Innominate Vein to Upper Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"05134JY"  =  "1"  /* Bypass Right Innominate Vein to Upper Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"05134KY"  =  "1"  /* Bypass Right Innominate Vein to Upper Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05134ZY"  =  "1"  /* Bypass Right Innominate Vein to Upper Vein, Percutaneous Endoscopic Approach */
"051407Y"  =  "1"  /* Bypass Left Innominate Vein to Upper Vein with Autologous Tissue Substitute, Open Approach */
"051409Y"  =  "1"  /* Bypass Left Innominate Vein to Upper Vein with Autologous Venous Tissue, Open Approach */
"05140AY"  =  "1"  /* Bypass Left Innominate Vein to Upper Vein with Autologous Arterial Tissue, Open Approach */
"05140JY"  =  "1"  /* Bypass Left Innominate Vein to Upper Vein with Synthetic Substitute, Open Approach */
"05140KY"  =  "1"  /* Bypass Left Innominate Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach */
"05140ZY"  =  "1"  /* Bypass Left Innominate Vein to Upper Vein, Open Approach */
"051447Y"  =  "1"  /* Bypass Left Innominate Vein to Upper Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"051449Y"  =  "1"  /* Bypass Left Innominate Vein to Upper Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"05144AY"  =  "1"  /* Bypass Left Innominate Vein to Upper Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"05144JY"  =  "1"  /* Bypass Left Innominate Vein to Upper Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"05144KY"  =  "1"  /* Bypass Left Innominate Vein to Upper Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05144ZY"  =  "1"  /* Bypass Left Innominate Vein to Upper Vein, Percutaneous Endoscopic Approach */
"051507Y"  =  "1"  /* Bypass Right Subclavian Vein to Upper Vein with Autologous Tissue Substitute, Open Approach */
"051509Y"  =  "1"  /* Bypass Right Subclavian Vein to Upper Vein with Autologous Venous Tissue, Open Approach */
"05150AY"  =  "1"  /* Bypass Right Subclavian Vein to Upper Vein with Autologous Arterial Tissue, Open Approach */
"05150JY"  =  "1"  /* Bypass Right Subclavian Vein to Upper Vein with Synthetic Substitute, Open Approach */
"05150KY"  =  "1"  /* Bypass Right Subclavian Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach */
"05150ZY"  =  "1"  /* Bypass Right Subclavian Vein to Upper Vein, Open Approach */
"051547Y"  =  "1"  /* Bypass Right Subclavian Vein to Upper Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"051549Y"  =  "1"  /* Bypass Right Subclavian Vein to Upper Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"05154AY"  =  "1"  /* Bypass Right Subclavian Vein to Upper Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"05154JY"  =  "1"  /* Bypass Right Subclavian Vein to Upper Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"05154KY"  =  "1"  /* Bypass Right Subclavian Vein to Upper Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05154ZY"  =  "1"  /* Bypass Right Subclavian Vein to Upper Vein, Percutaneous Endoscopic Approach */
"051607Y"  =  "1"  /* Bypass Left Subclavian Vein to Upper Vein with Autologous Tissue Substitute, Open Approach */
"051609Y"  =  "1"  /* Bypass Left Subclavian Vein to Upper Vein with Autologous Venous Tissue, Open Approach */
"05160AY"  =  "1"  /* Bypass Left Subclavian Vein to Upper Vein with Autologous Arterial Tissue, Open Approach */
"05160JY"  =  "1"  /* Bypass Left Subclavian Vein to Upper Vein with Synthetic Substitute, Open Approach */
"05160KY"  =  "1"  /* Bypass Left Subclavian Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach */
"05160ZY"  =  "1"  /* Bypass Left Subclavian Vein to Upper Vein, Open Approach */
"051647Y"  =  "1"  /* Bypass Left Subclavian Vein to Upper Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"051649Y"  =  "1"  /* Bypass Left Subclavian Vein to Upper Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"05164AY"  =  "1"  /* Bypass Left Subclavian Vein to Upper Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"05164JY"  =  "1"  /* Bypass Left Subclavian Vein to Upper Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"05164KY"  =  "1"  /* Bypass Left Subclavian Vein to Upper Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05164ZY"  =  "1"  /* Bypass Left Subclavian Vein to Upper Vein, Percutaneous Endoscopic Approach */
"05500ZZ"  =  "1"  /* Destruction of Azygos Vein, Open Approach */
"05504ZZ"  =  "1"  /* Destruction of Azygos Vein, Percutaneous Endoscopic Approach */
"05510ZZ"  =  "1"  /* Destruction of Hemiazygos Vein, Open Approach */
"05514ZZ"  =  "1"  /* Destruction of Hemiazygos Vein, Percutaneous Endoscopic Approach */
"05530ZZ"  =  "1"  /* Destruction of Right Innominate Vein, Open Approach */
"05534ZZ"  =  "1"  /* Destruction of Right Innominate Vein, Percutaneous Endoscopic Approach */
"05540ZZ"  =  "1"  /* Destruction of Left Innominate Vein, Open Approach */
"05544ZZ"  =  "1"  /* Destruction of Left Innominate Vein, Percutaneous Endoscopic Approach */
"05550ZZ"  =  "1"  /* Destruction of Right Subclavian Vein, Open Approach */
"05554ZZ"  =  "1"  /* Destruction of Right Subclavian Vein, Percutaneous Endoscopic Approach */
"05560ZZ"  =  "1"  /* Destruction of Left Subclavian Vein, Open Approach */
"05564ZZ"  =  "1"  /* Destruction of Left Subclavian Vein, Percutaneous Endoscopic Approach */
"05B00ZZ"  =  "1"  /* Excision of Azygos Vein, Open Approach */
"05B04ZZ"  =  "1"  /* Excision of Azygos Vein, Percutaneous Endoscopic Approach */
"05B10ZZ"  =  "1"  /* Excision of Hemiazygos Vein, Open Approach */
"05B14ZZ"  =  "1"  /* Excision of Hemiazygos Vein, Percutaneous Endoscopic Approach */
"05B30ZZ"  =  "1"  /* Excision of Right Innominate Vein, Open Approach */
"05B34ZZ"  =  "1"  /* Excision of Right Innominate Vein, Percutaneous Endoscopic Approach */
"05B40ZZ"  =  "1"  /* Excision of Left Innominate Vein, Open Approach */
"05B44ZZ"  =  "1"  /* Excision of Left Innominate Vein, Percutaneous Endoscopic Approach */
"05B50ZZ"  =  "1"  /* Excision of Right Subclavian Vein, Open Approach */
"05B54ZZ"  =  "1"  /* Excision of Right Subclavian Vein, Percutaneous Endoscopic Approach */
"05B60ZZ"  =  "1"  /* Excision of Left Subclavian Vein, Open Approach */
"05B64ZZ"  =  "1"  /* Excision of Left Subclavian Vein, Percutaneous Endoscopic Approach */
"05C00ZZ"  =  "1"  /* Extirpation of Matter from Azygos Vein, Open Approach */
"05C04ZZ"  =  "1"  /* Extirpation of Matter from Azygos Vein, Percutaneous Endoscopic Approach */
"05C10ZZ"  =  "1"  /* Extirpation of Matter from Hemiazygos Vein, Open Approach */
"05C14ZZ"  =  "1"  /* Extirpation of Matter from Hemiazygos Vein, Percutaneous Endoscopic Approach */
"05C30ZZ"  =  "1"  /* Extirpation of Matter from Right Innominate Vein, Open Approach */
"05C34ZZ"  =  "1"  /* Extirpation of Matter from Right Innominate Vein, Percutaneous Endoscopic Approach */
"05C40ZZ"  =  "1"  /* Extirpation of Matter from Left Innominate Vein, Open Approach */
"05C44ZZ"  =  "1"  /* Extirpation of Matter from Left Innominate Vein, Percutaneous Endoscopic Approach */
"05C50ZZ"  =  "1"  /* Extirpation of Matter from Right Subclavian Vein, Open Approach */
"05C54ZZ"  =  "1"  /* Extirpation of Matter from Right Subclavian Vein, Percutaneous Endoscopic Approach */
"05C60ZZ"  =  "1"  /* Extirpation of Matter from Left Subclavian Vein, Open Approach */
"05C64ZZ"  =  "1"  /* Extirpation of Matter from Left Subclavian Vein, Percutaneous Endoscopic Approach */
"05L00CZ"  =  "1"  /* Occlusion of Azygos Vein with Extraluminal Device, Open Approach */
"05L00DZ"  =  "1"  /* Occlusion of Azygos Vein with Intraluminal Device, Open Approach */
"05L00ZZ"  =  "1"  /* Occlusion of Azygos Vein, Open Approach */
"05L03CZ"  =  "1"  /* Occlusion of Azygos Vein with Extraluminal Device, Percutaneous Approach */
"05L03ZZ"  =  "1"  /* Occlusion of Azygos Vein, Percutaneous Approach */
"05L04CZ"  =  "1"  /* Occlusion of Azygos Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05L04DZ"  =  "1"  /* Occlusion of Azygos Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05L04ZZ"  =  "1"  /* Occlusion of Azygos Vein, Percutaneous Endoscopic Approach */
"05L10CZ"  =  "1"  /* Occlusion of Hemiazygos Vein with Extraluminal Device, Open Approach */
"05L10DZ"  =  "1"  /* Occlusion of Hemiazygos Vein with Intraluminal Device, Open Approach */
"05L10ZZ"  =  "1"  /* Occlusion of Hemiazygos Vein, Open Approach */
"05L13CZ"  =  "1"  /* Occlusion of Hemiazygos Vein with Extraluminal Device, Percutaneous Approach */
"05L13ZZ"  =  "1"  /* Occlusion of Hemiazygos Vein, Percutaneous Approach */
"05L14CZ"  =  "1"  /* Occlusion of Hemiazygos Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05L14DZ"  =  "1"  /* Occlusion of Hemiazygos Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05L14ZZ"  =  "1"  /* Occlusion of Hemiazygos Vein, Percutaneous Endoscopic Approach */
"05L30CZ"  =  "1"  /* Occlusion of Right Innominate Vein with Extraluminal Device, Open Approach */
"05L30DZ"  =  "1"  /* Occlusion of Right Innominate Vein with Intraluminal Device, Open Approach */
"05L30ZZ"  =  "1"  /* Occlusion of Right Innominate Vein, Open Approach */
"05L33CZ"  =  "1"  /* Occlusion of Right Innominate Vein with Extraluminal Device, Percutaneous Approach */
"05L33ZZ"  =  "1"  /* Occlusion of Right Innominate Vein, Percutaneous Approach */
"05L34CZ"  =  "1"  /* Occlusion of Right Innominate Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05L34DZ"  =  "1"  /* Occlusion of Right Innominate Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05L34ZZ"  =  "1"  /* Occlusion of Right Innominate Vein, Percutaneous Endoscopic Approach */
"05L40CZ"  =  "1"  /* Occlusion of Left Innominate Vein with Extraluminal Device, Open Approach */
"05L40DZ"  =  "1"  /* Occlusion of Left Innominate Vein with Intraluminal Device, Open Approach */
"05L40ZZ"  =  "1"  /* Occlusion of Left Innominate Vein, Open Approach */
"05L43CZ"  =  "1"  /* Occlusion of Left Innominate Vein with Extraluminal Device, Percutaneous Approach */
"05L43ZZ"  =  "1"  /* Occlusion of Left Innominate Vein, Percutaneous Approach */
"05L44CZ"  =  "1"  /* Occlusion of Left Innominate Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05L44DZ"  =  "1"  /* Occlusion of Left Innominate Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05L44ZZ"  =  "1"  /* Occlusion of Left Innominate Vein, Percutaneous Endoscopic Approach */
"05L50CZ"  =  "1"  /* Occlusion of Right Subclavian Vein with Extraluminal Device, Open Approach */
"05L50DZ"  =  "1"  /* Occlusion of Right Subclavian Vein with Intraluminal Device, Open Approach */
"05L50ZZ"  =  "1"  /* Occlusion of Right Subclavian Vein, Open Approach */
"05L53CZ"  =  "1"  /* Occlusion of Right Subclavian Vein with Extraluminal Device, Percutaneous Approach */
"05L53ZZ"  =  "1"  /* Occlusion of Right Subclavian Vein, Percutaneous Approach */
"05L54CZ"  =  "1"  /* Occlusion of Right Subclavian Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05L54DZ"  =  "1"  /* Occlusion of Right Subclavian Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05L54ZZ"  =  "1"  /* Occlusion of Right Subclavian Vein, Percutaneous Endoscopic Approach */
"05L60CZ"  =  "1"  /* Occlusion of Left Subclavian Vein with Extraluminal Device, Open Approach */
"05L60DZ"  =  "1"  /* Occlusion of Left Subclavian Vein with Intraluminal Device, Open Approach */
"05L60ZZ"  =  "1"  /* Occlusion of Left Subclavian Vein, Open Approach */
"05L63CZ"  =  "1"  /* Occlusion of Left Subclavian Vein with Extraluminal Device, Percutaneous Approach */
"05L63ZZ"  =  "1"  /* Occlusion of Left Subclavian Vein, Percutaneous Approach */
"05L64CZ"  =  "1"  /* Occlusion of Left Subclavian Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05L64DZ"  =  "1"  /* Occlusion of Left Subclavian Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05L64ZZ"  =  "1"  /* Occlusion of Left Subclavian Vein, Percutaneous Endoscopic Approach */
"05R007Z"  =  "1"  /* Replacement of Azygos Vein with Autologous Tissue Substitute, Open Approach */
"05R00JZ"  =  "1"  /* Replacement of Azygos Vein with Synthetic Substitute, Open Approach */
"05R00KZ"  =  "1"  /* Replacement of Azygos Vein with Nonautologous Tissue Substitute, Open Approach */
"05R047Z"  =  "1"  /* Replacement of Azygos Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05R04JZ"  =  "1"  /* Replacement of Azygos Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"05R04KZ"  =  "1"  /* Replacement of Azygos Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05R107Z"  =  "1"  /* Replacement of Hemiazygos Vein with Autologous Tissue Substitute, Open Approach */
"05R10JZ"  =  "1"  /* Replacement of Hemiazygos Vein with Synthetic Substitute, Open Approach */
"05R10KZ"  =  "1"  /* Replacement of Hemiazygos Vein with Nonautologous Tissue Substitute, Open Approach */
"05R147Z"  =  "1"  /* Replacement of Hemiazygos Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05R14JZ"  =  "1"  /* Replacement of Hemiazygos Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"05R14KZ"  =  "1"  /* Replacement of Hemiazygos Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05R307Z"  =  "1"  /* Replacement of Right Innominate Vein with Autologous Tissue Substitute, Open Approach */
"05R30JZ"  =  "1"  /* Replacement of Right Innominate Vein with Synthetic Substitute, Open Approach */
"05R30KZ"  =  "1"  /* Replacement of Right Innominate Vein with Nonautologous Tissue Substitute, Open Approach */
"05R347Z"  =  "1"  /* Replacement of Right Innominate Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05R34JZ"  =  "1"  /* Replacement of Right Innominate Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"05R34KZ"  =  "1"  /* Replacement of Right Innominate Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05R407Z"  =  "1"  /* Replacement of Left Innominate Vein with Autologous Tissue Substitute, Open Approach */
"05R40JZ"  =  "1"  /* Replacement of Left Innominate Vein with Synthetic Substitute, Open Approach */
"05R40KZ"  =  "1"  /* Replacement of Left Innominate Vein with Nonautologous Tissue Substitute, Open Approach */
"05R447Z"  =  "1"  /* Replacement of Left Innominate Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05R44JZ"  =  "1"  /* Replacement of Left Innominate Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"05R44KZ"  =  "1"  /* Replacement of Left Innominate Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05R507Z"  =  "1"  /* Replacement of Right Subclavian Vein with Autologous Tissue Substitute, Open Approach */
"05R50JZ"  =  "1"  /* Replacement of Right Subclavian Vein with Synthetic Substitute, Open Approach */
"05R50KZ"  =  "1"  /* Replacement of Right Subclavian Vein with Nonautologous Tissue Substitute, Open Approach */
"05R547Z"  =  "1"  /* Replacement of Right Subclavian Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05R54JZ"  =  "1"  /* Replacement of Right Subclavian Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"05R54KZ"  =  "1"  /* Replacement of Right Subclavian Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05R607Z"  =  "1"  /* Replacement of Left Subclavian Vein with Autologous Tissue Substitute, Open Approach */
"05R60JZ"  =  "1"  /* Replacement of Left Subclavian Vein with Synthetic Substitute, Open Approach */
"05R60KZ"  =  "1"  /* Replacement of Left Subclavian Vein with Nonautologous Tissue Substitute, Open Approach */
"05R647Z"  =  "1"  /* Replacement of Left Subclavian Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"05R64JZ"  =  "1"  /* Replacement of Left Subclavian Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"05R64KZ"  =  "1"  /* Replacement of Left Subclavian Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0PT10ZZ"  =  "1"  /* Resection of Right Rib, Open Approach */
"0PT20ZZ"  =  "1"  /* Resection of Left Rib, Open Approach */
"0W9D00Z"  =  "1"  /* Drainage of Pericardial Cavity with Drainage Device, Open Approach */
"0W9D0ZX"  =  "1"  /* Drainage of Pericardial Cavity, Open Approach, Diagnostic */
"0W9D0ZZ"  =  "1"  /* Drainage of Pericardial Cavity, Open Approach */
"0W9D30Z"  =  "1"  /* Drainage of Pericardial Cavity with Drainage Device, Percutaneous Approach */
"0W9D3ZX"  =  "1"  /* Drainage of Pericardial Cavity, Percutaneous Approach, Diagnostic */
"0W9D3ZZ"  =  "1"  /* Drainage of Pericardial Cavity, Percutaneous Approach */
"0W9D40Z"  =  "1"  /* Drainage of Pericardial Cavity with Drainage Device, Percutaneous Endoscopic Approach */
"0W9D4ZX"  =  "1"  /* Drainage of Pericardial Cavity, Percutaneous Endoscopic Approach, Diagnostic */
"0W9D4ZZ"  =  "1"  /* Drainage of Pericardial Cavity, Percutaneous Endoscopic Approach */
"0WCD0ZZ"  =  "1"  /* Extirpation of Matter from Pericardial Cavity, Open Approach */
"0WCD3ZZ"  =  "1"  /* Extirpation of Matter from Pericardial Cavity, Percutaneous Approach */
"0WCD4ZZ"  =  "1"  /* Extirpation of Matter from Pericardial Cavity, Percutaneous Endoscopic Approach */
"0WFD0ZZ"  =  "1"  /* Fragmentation in Pericardial Cavity, Open Approach */
"0WFD3ZZ"  =  "1"  /* Fragmentation in Pericardial Cavity, Percutaneous Approach */
"0WFD4ZZ"  =  "1"  /* Fragmentation in Pericardial Cavity, Percutaneous Endoscopic Approach */
"0WFDXZZ"  =  "1"  /* Fragmentation in Pericardial Cavity, External Approach */
"0WHD03Z"  =  "1"  /* Insertion of Infusion Device into Pericardial Cavity, Open Approach */
"0WHD0YZ"  =  "1"  /* Insertion of Other Device into Pericardial Cavity, Open Approach */
"0WHD33Z"  =  "1"  /* Insertion of Infusion Device into Pericardial Cavity, Percutaneous Approach */
"0WHD3YZ"  =  "1"  /* Insertion of Other Device into Pericardial Cavity, Percutaneous Approach */
"0WHD43Z"  =  "1"  /* Insertion of Infusion Device into Pericardial Cavity, Percutaneous Endoscopic Approach */
"0WHD4YZ"  =  "1"  /* Insertion of Other Device into Pericardial Cavity, Percutaneous Endoscopic Approach */
"0WPD00Z"  =  "1"  /* Removal of Drainage Device from Pericardial Cavity, Open Approach */
"0WPD01Z"  =  "1"  /* Removal of Radioactive Element from Pericardial Cavity, Open Approach */
"0WPD03Z"  =  "1"  /* Removal of Infusion Device from Pericardial Cavity, Open Approach */
"0WPD0YZ"  =  "1"  /* Removal of Other Device from Pericardial Cavity, Open Approach */
"0WPD30Z"  =  "1"  /* Removal of Drainage Device from Pericardial Cavity, Percutaneous Approach */
"0WPD31Z"  =  "1"  /* Removal of Radioactive Element from Pericardial Cavity, Percutaneous Approach */
"0WPD33Z"  =  "1"  /* Removal of Infusion Device from Pericardial Cavity, Percutaneous Approach */
"0WPD3YZ"  =  "1"  /* Removal of Other Device from Pericardial Cavity, Percutaneous Approach */
"0WPD40Z"  =  "1"  /* Removal of Drainage Device from Pericardial Cavity, Percutaneous Endoscopic Approach */
"0WPD41Z"  =  "1"  /* Removal of Radioactive Element from Pericardial Cavity, Percutaneous Endoscopic Approach */
"0WPD43Z"  =  "1"  /* Removal of Infusion Device from Pericardial Cavity, Percutaneous Endoscopic Approach */
"0WPD4YZ"  =  "1"  /* Removal of Other Device from Pericardial Cavity, Percutaneous Endoscopic Approach */
"0WWD00Z"  =  "1"  /* Revision of Drainage Device in Pericardial Cavity, Open Approach */
"0WWD01Z"  =  "1"  /* Revision of Radioactive Element in Pericardial Cavity, Open Approach */
"0WWD03Z"  =  "1"  /* Revision of Infusion Device in Pericardial Cavity, Open Approach */
"0WWD0YZ"  =  "1"  /* Revision of Other Device in Pericardial Cavity, Open Approach */
"0WWD30Z"  =  "1"  /* Revision of Drainage Device in Pericardial Cavity, Percutaneous Approach */
"0WWD31Z"  =  "1"  /* Revision of Radioactive Element in Pericardial Cavity, Percutaneous Approach */
"0WWD33Z"  =  "1"  /* Revision of Infusion Device in Pericardial Cavity, Percutaneous Approach */
"0WWD3YZ"  =  "1"  /* Revision of Other Device in Pericardial Cavity, Percutaneous Approach */
"0WWD40Z"  =  "1"  /* Revision of Drainage Device in Pericardial Cavity, Percutaneous Endoscopic Approach */
"0WWD41Z"  =  "1"  /* Revision of Radioactive Element in Pericardial Cavity, Percutaneous Endoscopic Approach */
"0WWD43Z"  =  "1"  /* Revision of Infusion Device in Pericardial Cavity, Percutaneous Endoscopic Approach */
"0WWD4YZ"  =  "1"  /* Revision of Other Device in Pericardial Cavity, Percutaneous Endoscopic Approach */
OTHER  = "0" ;

 /* EXCLUDE: CARDIAC SURGERY */
 /* DIAGNOSTIC RELATED GROUPS (DRGS): */

VALUE $CARDSDR
"103" = "1" /* HEART TRANSPLANT */
"104" = "1" /* CARDIAC VALVE AND OTHER MAJOR CARDIOTHORACIC PROCEDURES WITH CARDIAC CATHETERIZATION */
"105" = "1" /* CARDIAC VALVE AND OTHER MAJOR CARDIOTHORACIC PROCEDURES WITHOUT CARDIAC CATHETERIZATION */
"106" = "1" /* CORONARY BYPASS WITH PTCA */
"107" = "1" /* CORONARY BYPASS WITH CARDIAC CATHETERIZATION */
"108" = "1" /* OTHER CARDIOTHORACIC PROCEDURES */
"109" = "1" /* CORONARY BYPASS WITHOUT CARDIAC CATHETERIZATION */
"110" = "1" /* MAJOR CARDIOVASCULAR PROCEDURES WITH CC */
"111" = "1" /* MAJOR CARDIOVASCULAR PROCEDURES WITHOUT CC */
"525" = "1" /* HEART ASSIST SYSTEM IMPLANT */
"547" = "1" /* CORONARY BYPASS W CARDIAC CATH W MAJOR CV DX */
"548" = "1" /* CORONARY BYPASS W CARDIAC CATH W/O MAJOR CV DX */
"549" = "1" /* CORONARY BYPASS W/O CARDIAC CATH W MAJOR CV DX */
"550" = "1" /* CORONARY BYPASS W/O CARDIAC CATH W/O MAJOR CV DX */
OTHER = "0" ;

 /* EXCLUDE: DIAPHRAGMATIC SURGERY REPAIR */
 /* ICD-10-CM PROCEDURE CODES: */

VALUE $DIAPHRP
"0BQR0ZZ"  =  "1"  /* Repair Right Diaphragm, Open Approach */
"0BQR3ZZ"  =  "1"  /* Repair Right Diaphragm, Percutaneous Approach */
"0BQR4ZZ"  =  "1"  /* Repair Right Diaphragm, Percutaneous Endoscopic Approach */
"0BQS0ZZ"  =  "1"  /* Repair Left Diaphragm, Open Approach */
"0BQS3ZZ"  =  "1"  /* Repair Left Diaphragm, Percutaneous Approach */
"0BQS4ZZ"  =  "1"  /* Repair Left Diaphragm, Percutaneous Endoscopic Approach */
"0BUR07Z"  =  "1"  /* Supplement Right Diaphragm with Autologous Tissue Substitute, Open Approach */
"0BUR0JZ"  =  "1"  /* Supplement Right Diaphragm with Synthetic Substitute, Open Approach */
"0BUR0KZ"  =  "1"  /* Supplement Right Diaphragm with Nonautologous Tissue Substitute, Open Approach */
"0BUR47Z"  =  "1"  /* Supplement Right Diaphragm with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BUR4JZ"  =  "1"  /* Supplement Right Diaphragm with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BUR4KZ"  =  "1"  /* Supplement Right Diaphragm with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BUS07Z"  =  "1"  /* Supplement Left Diaphragm with Autologous Tissue Substitute, Open Approach */
"0BUS0JZ"  =  "1"  /* Supplement Left Diaphragm with Synthetic Substitute, Open Approach */
"0BUS0KZ"  =  "1"  /* Supplement Left Diaphragm with Nonautologous Tissue Substitute, Open Approach */
"0BUS47Z"  =  "1"  /* Supplement Left Diaphragm with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BUS4JZ"  =  "1"  /* Supplement Left Diaphragm with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BUS4KZ"  =  "1"  /* Supplement Left Diaphragm with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
OTHER  = "0" ;


 /* ----------------------------------------- */
 /* Pediatric Heart Surgery (PRPEDH)          */
 /*   Mortality (post-op) Indicator / PDI #6  */
 /*   Volume Indicator / PDI #7               */
 /* ----------------------------------------- */

 /* Pediatric Heart Surgery (#1) */
 /* Procedures to repair congenital heart defect */
 /* ICD-10-CM Procedure Codes: */

VALUE $PRPED1P
"021609P"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk with Autologous Venous Tissue, Open Approach */
"021609Q"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021609R"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"02160AP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach */
"02160AQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"02160AR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"02160JP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk with Synthetic Substitute, Open Approach */
"02160JQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"02160JR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"02160KP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach */
"02160KQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"02160KR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"02160ZP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk, Open Approach */
"02160ZQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery, Open Approach */
"02160ZR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery, Open Approach */
"021649P"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021649Q"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021649R"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"02164AP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02164AQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02164AR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02164JP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02164JQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02164JR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02164KP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02164KQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02164KR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02164ZP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk, Percutaneous Endoscopic Approach */
"02164ZQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"02164ZR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"021709P"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk with Autologous Venous Tissue, Open Approach */
"021709Q"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021709R"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"02170AP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach */
"02170AQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"02170AR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"02170JP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk with Synthetic Substitute, Open Approach */
"02170JQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"02170JR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"02170KP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach */
"02170KQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"02170KR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"02170ZP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk, Open Approach */
"02170ZQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery, Open Approach */
"02170ZR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery, Open Approach */
"021749P"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021749Q"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021749R"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"02174AP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02174AQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02174AR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"02174JP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02174JQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02174JR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02174KP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02174KQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02174KR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02174ZP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk, Percutaneous Endoscopic Approach */
"02174ZQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"02174ZR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"021K09P"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk with Autologous Venous Tissue, Open Approach */
"021K09Q"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021K09R"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021K0AP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach */
"021K0AQ"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"021K0AR"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"021K0JP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk with Synthetic Substitute, Open Approach */
"021K0JQ"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"021K0JR"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"021K0KP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach */
"021K0KQ"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"021K0KR"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"021K0ZP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk, Open Approach */
"021K0ZQ"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery, Open Approach */
"021K0ZR"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery, Open Approach */
"021K49P"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021K49Q"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021K49R"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021K4AP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021K4AQ"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021K4AR"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021K4JP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021K4JQ"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021K4JR"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021K4KP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021K4KQ"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021K4KR"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021K4ZP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk, Percutaneous Endoscopic Approach */
"021K4ZQ"  =  "1"  /* Bypass Right Ventricle to Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"021K4ZR"  =  "1"  /* Bypass Right Ventricle to Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"021L0ZW"  =  "1"  /* Bypass Left Ventricle to Aorta, Open Approach */
"021L4ZW"  =  "1"  /* Bypass Left Ventricle to Aorta, Percutaneous Endoscopic Approach */
"021V09P"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk with Autologous Venous Tissue, Open Approach */
"021V09Q"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021V09R"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021V0AP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach */
"021V0AQ"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"021V0AR"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"021V0JP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk with Synthetic Substitute, Open Approach */
"021V0JQ"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"021V0JR"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"021V0KP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach */
"021V0KQ"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"021V0KR"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"021V0ZP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk, Open Approach */
"021V0ZQ"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery, Open Approach */
"021V0ZR"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery, Open Approach */
"021V49P"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021V49Q"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021V49R"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021V4AP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021V4AQ"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021V4AR"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021V4JP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021V4JQ"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021V4JR"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021V4KP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021V4KQ"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021V4KR"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021V4ZP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk, Percutaneous Endoscopic Approach */
"021V4ZQ"  =  "1"  /* Bypass Superior Vena Cava to Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"021V4ZR"  =  "1"  /* Bypass Superior Vena Cava to Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"021W09P"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk with Autologous Venous Tissue, Open Approach */
"021W09Q"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021W09R"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"021W0AP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach */
"021W0AQ"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"021W0AR"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"021W0JP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk with Synthetic Substitute, Open Approach */
"021W0JQ"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"021W0JR"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"021W0KP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach */
"021W0KQ"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"021W0KR"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"021W0ZP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk, Open Approach */
"021W0ZQ"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery, Open Approach */
"021W0ZR"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery, Open Approach */
"021W49P"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021W49Q"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021W49R"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"021W4AP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021W4AQ"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021W4AR"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"021W4JP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021W4JQ"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021W4JR"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"021W4KP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021W4KQ"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021W4KR"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"021W4ZP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk, Percutaneous Endoscopic Approach */
"021W4ZQ"  =  "1"  /* Bypass Thoracic Aorta to Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"021W4ZR"  =  "1"  /* Bypass Thoracic Aorta to Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"02540ZZ"  =  "1"  /* Destruction of Coronary Vein, Open Approach */
"02544ZZ"  =  "1"  /* Destruction of Coronary Vein, Percutaneous Endoscopic Approach */
"02550ZZ"  =  "1"  /* Destruction of Atrial Septum, Open Approach */
"02560ZZ"  =  "1"  /* Destruction of Right Atrium, Open Approach */
"02570ZK"  =  "1"  /* Destruction of Left Atrial Appendage, Open Approach */
"02570ZZ"  =  "1"  /* Destruction of Left Atrium, Open Approach */
"02574ZK"  =  "1"  /* Destruction of Left Atrial Appendage, Percutaneous Endoscopic Approach */
"02580ZZ"  =  "1"  /* Destruction of Conduction Mechanism, Open Approach */
"02590ZZ"  =  "1"  /* Destruction of Chordae Tendineae, Open Approach */
"025D0ZZ"  =  "1"  /* Destruction of Papillary Muscle, Open Approach */
"025D4ZZ"  =  "1"  /* Destruction of Papillary Muscle, Percutaneous Endoscopic Approach */
"025F0ZZ"  =  "1"  /* Destruction of Aortic Valve, Open Approach */
"025G0ZZ"  =  "1"  /* Destruction of Mitral Valve, Open Approach */
"025H0ZZ"  =  "1"  /* Destruction of Pulmonary Valve, Open Approach */
"025J0ZZ"  =  "1"  /* Destruction of Tricuspid Valve, Open Approach */
"025K0ZZ"  =  "1"  /* Destruction of Right Ventricle, Open Approach */
"025L0ZZ"  =  "1"  /* Destruction of Left Ventricle, Open Approach */
"025M0ZZ"  =  "1"  /* Destruction of Ventricular Septum, Open Approach */
"027F04Z"  =  "1"  /* Dilation of Aortic Valve with Drug-eluting Intraluminal Device, Open Approach */
"027F0DZ"  =  "1"  /* Dilation of Aortic Valve with Intraluminal Device, Open Approach */
"027F0ZZ"  =  "1"  /* Dilation of Aortic Valve, Open Approach */
"027G04Z"  =  "1"  /* Dilation of Mitral Valve with Drug-eluting Intraluminal Device, Open Approach */
"027G0DZ"  =  "1"  /* Dilation of Mitral Valve with Intraluminal Device, Open Approach */
"027G0ZZ"  =  "1"  /* Dilation of Mitral Valve, Open Approach */
"027H04Z"  =  "1"  /* Dilation of Pulmonary Valve with Drug-eluting Intraluminal Device, Open Approach */
"027H0DZ"  =  "1"  /* Dilation of Pulmonary Valve with Intraluminal Device, Open Approach */
"027H0ZZ"  =  "1"  /* Dilation of Pulmonary Valve, Open Approach */
"027J04Z"  =  "1"  /* Dilation of Tricuspid Valve with Drug-eluting Intraluminal Device, Open Approach */
"027J0DZ"  =  "1"  /* Dilation of Tricuspid Valve with Intraluminal Device, Open Approach */
"027J0ZZ"  =  "1"  /* Dilation of Tricuspid Valve, Open Approach */
"02890ZZ"  =  "1"  /* Division of Chordae Tendineae, Open Approach */
"02894ZZ"  =  "1"  /* Division of Chordae Tendineae, Percutaneous Endoscopic Approach */
"028D0ZZ"  =  "1"  /* Division of Papillary Muscle, Open Approach */
"028D4ZZ"  =  "1"  /* Division of Papillary Muscle, Percutaneous Endoscopic Approach */
"02B40ZZ"  =  "1"  /* Excision of Coronary Vein, Open Approach */
"02B44ZZ"  =  "1"  /* Excision of Coronary Vein, Percutaneous Endoscopic Approach */
"02B50ZZ"  =  "1"  /* Excision of Atrial Septum, Open Approach */
"02B54ZZ"  =  "1"  /* Excision of Atrial Septum, Percutaneous Endoscopic Approach */
"02B60ZZ"  =  "1"  /* Excision of Right Atrium, Open Approach */
"02B70ZK"  =  "1"  /* Excision of Left Atrial Appendage, Open Approach */
"02B70ZZ"  =  "1"  /* Excision of Left Atrium, Open Approach */
"02B74ZK"  =  "1"  /* Excision of Left Atrial Appendage, Percutaneous Endoscopic Approach */
"02B80ZZ"  =  "1"  /* Excision of Conduction Mechanism, Open Approach */
"02B90ZZ"  =  "1"  /* Excision of Chordae Tendineae, Open Approach */
"02BD0ZZ"  =  "1"  /* Excision of Papillary Muscle, Open Approach */
"02BD4ZZ"  =  "1"  /* Excision of Papillary Muscle, Percutaneous Endoscopic Approach */
"02BF0ZZ"  =  "1"  /* Excision of Aortic Valve, Open Approach */
"02BG0ZZ"  =  "1"  /* Excision of Mitral Valve, Open Approach */
"02BH0ZZ"  =  "1"  /* Excision of Pulmonary Valve, Open Approach */
"02BJ0ZZ"  =  "1"  /* Excision of Tricuspid Valve, Open Approach */
"02BK0ZZ"  =  "1"  /* Excision of Right Ventricle, Open Approach */
"02BK4ZZ"  =  "1"  /* Excision of Right Ventricle, Percutaneous Endoscopic Approach */
"02BL0ZZ"  =  "1"  /* Excision of Left Ventricle, Open Approach */
"02BM0ZZ"  =  "1"  /* Excision of Ventricular Septum, Open Approach */
"02C40ZZ"  =  "1"  /* Extirpation of Matter from Coronary Vein, Open Approach */
"02C44ZZ"  =  "1"  /* Extirpation of Matter from Coronary Vein, Percutaneous Endoscopic Approach */
"02C50ZZ"  =  "1"  /* Extirpation of Matter from Atrial Septum, Open Approach */
"02C54ZZ"  =  "1"  /* Extirpation of Matter from Atrial Septum, Percutaneous Endoscopic Approach */
"02CD0ZZ"  =  "1"  /* Extirpation of Matter from Papillary Muscle, Open Approach */
"02CD4ZZ"  =  "1"  /* Extirpation of Matter from Papillary Muscle, Percutaneous Endoscopic Approach */
"02CF0ZZ"  =  "1"  /* Extirpation of Matter from Aortic Valve, Open Approach */
"02CG0ZZ"  =  "1"  /* Extirpation of Matter from Mitral Valve, Open Approach */
"02CH0ZZ"  =  "1"  /* Extirpation of Matter from Pulmonary Valve, Open Approach */
"02CJ0ZZ"  =  "1"  /* Extirpation of Matter from Tricuspid Valve, Open Approach */
"02CM0ZZ"  =  "1"  /* Extirpation of Matter from Ventricular Septum, Open Approach */
"02H402Z"  =  "1"  /* Insertion of Monitoring Device into Coronary Vein, Open Approach */
"02H403Z"  =  "1"  /* Insertion of Infusion Device into Coronary Vein, Open Approach */
"02H40DZ"  =  "1"  /* Insertion of Intraluminal Device into Coronary Vein, Open Approach */
"02H442Z"  =  "1"  /* Insertion of Monitoring Device into Coronary Vein, Percutaneous Endoscopic Approach */
"02H443Z"  =  "1"  /* Insertion of Infusion Device into Coronary Vein, Percutaneous Endoscopic Approach */
"02H44DZ"  =  "1"  /* Insertion of Intraluminal Device into Coronary Vein, Percutaneous Endoscopic Approach */
"02L70CK"  =  "1"  /* Occlusion of Left Atrial Appendage with Extraluminal Device, Open Approach */
"02L70DK"  =  "1"  /* Occlusion of Left Atrial Appendage with Intraluminal Device, Open Approach */
"02L70ZK"  =  "1"  /* Occlusion of Left Atrial Appendage, Open Approach */
"02L74CK"  =  "1"  /* Occlusion of Left Atrial Appendage with Extraluminal Device, Percutaneous Endoscopic Approach */
"02L74DK"  =  "1"  /* Occlusion of Left Atrial Appendage with Intraluminal Device, Percutaneous Endoscopic Approach */
"02L74ZK"  =  "1"  /* Occlusion of Left Atrial Appendage, Percutaneous Endoscopic Approach */
"02LR0ZT"  =  "1"  /* Occlusion of Ductus Arteriosus, Open Approach */
"02LS0ZZ"  =  "1"  /* Occlusion of Right Pulmonary Vein, Open Approach */
"02LT0ZZ"  =  "1"  /* Occlusion of Left Pulmonary Vein, Open Approach */
"02N50ZZ"  =  "1"  /* Release Atrial Septum, Open Approach */
"02N54ZZ"  =  "1"  /* Release Atrial Septum, Percutaneous Endoscopic Approach */
"02N90ZZ"  =  "1"  /* Release Chordae Tendineae, Open Approach */
"02N94ZZ"  =  "1"  /* Release Chordae Tendineae, Percutaneous Endoscopic Approach */
"02ND0ZZ"  =  "1"  /* Release Papillary Muscle, Open Approach */
"02ND4ZZ"  =  "1"  /* Release Papillary Muscle, Percutaneous Endoscopic Approach */
"02NF0ZZ"  =  "1"  /* Release Aortic Valve, Open Approach */
"02NF4ZZ"  =  "1"  /* Release Aortic Valve, Percutaneous Endoscopic Approach */
"02NG0ZZ"  =  "1"  /* Release Mitral Valve, Open Approach */
"02NG4ZZ"  =  "1"  /* Release Mitral Valve, Percutaneous Endoscopic Approach */
"02NH0ZZ"  =  "1"  /* Release Pulmonary Valve, Open Approach */
"02NH4ZZ"  =  "1"  /* Release Pulmonary Valve, Percutaneous Endoscopic Approach */
"02NJ0ZZ"  =  "1"  /* Release Tricuspid Valve, Open Approach */
"02NJ4ZZ"  =  "1"  /* Release Tricuspid Valve, Percutaneous Endoscopic Approach */
"02NK0ZZ"  =  "1"  /* Release Right Ventricle, Open Approach */
"02NK4ZZ"  =  "1"  /* Release Right Ventricle, Percutaneous Endoscopic Approach */
"02NL0ZZ"  =  "1"  /* Release Left Ventricle, Open Approach */
"02NL4ZZ"  =  "1"  /* Release Left Ventricle, Percutaneous Endoscopic Approach */
"02NM0ZZ"  =  "1"  /* Release Ventricular Septum, Open Approach */
"02NM4ZZ"  =  "1"  /* Release Ventricular Septum, Percutaneous Endoscopic Approach */
"02Q00ZZ"  =  "1"  /* Repair Coronary Artery, One Site, Open Approach */
"02Q04ZZ"  =  "1"  /* Repair Coronary Artery, One Site, Percutaneous Endoscopic Approach */
"02Q10ZZ"  =  "1"  /* Repair Coronary Artery, Two Sites, Open Approach */
"02Q14ZZ"  =  "1"  /* Repair Coronary Artery, Two Sites, Percutaneous Endoscopic Approach */
"02Q20ZZ"  =  "1"  /* Repair Coronary Artery, Three Sites, Open Approach */
"02Q24ZZ"  =  "1"  /* Repair Coronary Artery, Three Sites, Percutaneous Endoscopic Approach */
"02Q30ZZ"  =  "1"  /* Repair Coronary Artery, Four or More Sites, Open Approach */
"02Q34ZZ"  =  "1"  /* Repair Coronary Artery, Four or More Sites, Percutaneous Endoscopic Approach */
"02Q40ZZ"  =  "1"  /* Repair Coronary Vein, Open Approach */
"02Q44ZZ"  =  "1"  /* Repair Coronary Vein, Percutaneous Endoscopic Approach */
"02Q50ZZ"  =  "1"  /* Repair Atrial Septum, Open Approach */
"02Q54ZZ"  =  "1"  /* Repair Atrial Septum, Percutaneous Endoscopic Approach */
"02Q70ZZ"  =  "1"  /* Repair Left Atrium, Open Approach */
"02Q74ZZ"  =  "1"  /* Repair Left Atrium, Percutaneous Endoscopic Approach */
"02Q90ZZ"  =  "1"  /* Repair Chordae Tendineae, Open Approach */
"02Q94ZZ"  =  "1"  /* Repair Chordae Tendineae, Percutaneous Endoscopic Approach */
"02QA0ZZ"  =  "1"  /* Repair Heart, Open Approach */
"02QA4ZZ"  =  "1"  /* Repair Heart, Percutaneous Endoscopic Approach */
"02QB0ZZ"  =  "1"  /* Repair Right Heart, Open Approach */
"02QB4ZZ"  =  "1"  /* Repair Right Heart, Percutaneous Endoscopic Approach */
"02QC0ZZ"  =  "1"  /* Repair Left Heart, Open Approach */
"02QC4ZZ"  =  "1"  /* Repair Left Heart, Percutaneous Endoscopic Approach */
"02QD0ZZ"  =  "1"  /* Repair Papillary Muscle, Open Approach */
"02QD4ZZ"  =  "1"  /* Repair Papillary Muscle, Percutaneous Endoscopic Approach */
"02QF0ZZ"  =  "1"  /* Repair Aortic Valve, Open Approach */
"02QF4ZZ"  =  "1"  /* Repair Aortic Valve, Percutaneous Endoscopic Approach */
"02QG0ZZ"  =  "1"  /* Repair Mitral Valve, Open Approach */
"02QG4ZZ"  =  "1"  /* Repair Mitral Valve, Percutaneous Endoscopic Approach */
"02QH0ZZ"  =  "1"  /* Repair Pulmonary Valve, Open Approach */
"02QH4ZZ"  =  "1"  /* Repair Pulmonary Valve, Percutaneous Endoscopic Approach */
"02QJ0ZZ"  =  "1"  /* Repair Tricuspid Valve, Open Approach */
"02QJ4ZZ"  =  "1"  /* Repair Tricuspid Valve, Percutaneous Endoscopic Approach */
"02QM0ZZ"  =  "1"  /* Repair Ventricular Septum, Open Approach */
"02QM4ZZ"  =  "1"  /* Repair Ventricular Septum, Percutaneous Endoscopic Approach */
"02R907Z"  =  "1"  /* Replacement of Chordae Tendineae with Autologous Tissue Substitute, Open Approach */
"02R908Z"  =  "1"  /* Replacement of Chordae Tendineae with Zooplastic Tissue, Open Approach */
"02R90JZ"  =  "1"  /* Replacement of Chordae Tendineae with Synthetic Substitute, Open Approach */
"02R90KZ"  =  "1"  /* Replacement of Chordae Tendineae with Nonautologous Tissue Substitute, Open Approach */
"02R947Z"  =  "1"  /* Replacement of Chordae Tendineae with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02R948Z"  =  "1"  /* Replacement of Chordae Tendineae with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02R94JZ"  =  "1"  /* Replacement of Chordae Tendineae with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02R94KZ"  =  "1"  /* Replacement of Chordae Tendineae with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RD07Z"  =  "1"  /* Replacement of Papillary Muscle with Autologous Tissue Substitute, Open Approach */
"02RD08Z"  =  "1"  /* Replacement of Papillary Muscle with Zooplastic Tissue, Open Approach */
"02RD0JZ"  =  "1"  /* Replacement of Papillary Muscle with Synthetic Substitute, Open Approach */
"02RD0KZ"  =  "1"  /* Replacement of Papillary Muscle with Nonautologous Tissue Substitute, Open Approach */
"02RD47Z"  =  "1"  /* Replacement of Papillary Muscle with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RD48Z"  =  "1"  /* Replacement of Papillary Muscle with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RD4JZ"  =  "1"  /* Replacement of Papillary Muscle with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RD4KZ"  =  "1"  /* Replacement of Papillary Muscle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RF07Z"  =  "1"  /* Replacement of Aortic Valve with Autologous Tissue Substitute, Open Approach */
"02RF08Z"  =  "1"  /* Replacement of Aortic Valve with Zooplastic Tissue, Open Approach */
"02RF0JZ"  =  "1"  /* Replacement of Aortic Valve with Synthetic Substitute, Open Approach */
"02RF0KZ"  =  "1"  /* Replacement of Aortic Valve with Nonautologous Tissue Substitute, Open Approach */
"02RF47Z"  =  "1"  /* Replacement of Aortic Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RF48Z"  =  "1"  /* Replacement of Aortic Valve with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RF4JZ"  =  "1"  /* Replacement of Aortic Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RF4KZ"  =  "1"  /* Replacement of Aortic Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RG07Z"  =  "1"  /* Replacement of Mitral Valve with Autologous Tissue Substitute, Open Approach */
"02RG08Z"  =  "1"  /* Replacement of Mitral Valve with Zooplastic Tissue, Open Approach */
"02RG0JZ"  =  "1"  /* Replacement of Mitral Valve with Synthetic Substitute, Open Approach */
"02RG0KZ"  =  "1"  /* Replacement of Mitral Valve with Nonautologous Tissue Substitute, Open Approach */
"02RG47Z"  =  "1"  /* Replacement of Mitral Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RG48Z"  =  "1"  /* Replacement of Mitral Valve with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RG4JZ"  =  "1"  /* Replacement of Mitral Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RG4KZ"  =  "1"  /* Replacement of Mitral Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RH07Z"  =  "1"  /* Replacement of Pulmonary Valve with Autologous Tissue Substitute, Open Approach */
"02RH08Z"  =  "1"  /* Replacement of Pulmonary Valve with Zooplastic Tissue, Open Approach */
"02RH0JZ"  =  "1"  /* Replacement of Pulmonary Valve with Synthetic Substitute, Open Approach */
"02RH0KZ"  =  "1"  /* Replacement of Pulmonary Valve with Nonautologous Tissue Substitute, Open Approach */
"02RH47Z"  =  "1"  /* Replacement of Pulmonary Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RH48Z"  =  "1"  /* Replacement of Pulmonary Valve with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RH4JZ"  =  "1"  /* Replacement of Pulmonary Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RH4KZ"  =  "1"  /* Replacement of Pulmonary Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RJ07Z"  =  "1"  /* Replacement of Tricuspid Valve with Autologous Tissue Substitute, Open Approach */
"02RJ08Z"  =  "1"  /* Replacement of Tricuspid Valve with Zooplastic Tissue, Open Approach */
"02RJ0JZ"  =  "1"  /* Replacement of Tricuspid Valve with Synthetic Substitute, Open Approach */
"02RJ0KZ"  =  "1"  /* Replacement of Tricuspid Valve with Nonautologous Tissue Substitute, Open Approach */
"02RJ47Z"  =  "1"  /* Replacement of Tricuspid Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RJ48Z"  =  "1"  /* Replacement of Tricuspid Valve with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RJ4JZ"  =  "1"  /* Replacement of Tricuspid Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RJ4KZ"  =  "1"  /* Replacement of Tricuspid Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RK07Z"  =  "1"  /* Replacement of Right Ventricle with Autologous Tissue Substitute, Open Approach */
"02RK0JZ"  =  "1"  /* Replacement of Right Ventricle with Synthetic Substitute, Open Approach */
"02RK0KZ"  =  "1"  /* Replacement of Right Ventricle with Nonautologous Tissue Substitute, Open Approach */
"02RK47Z"  =  "1"  /* Replacement of Right Ventricle with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RK4KZ"  =  "1"  /* Replacement of Right Ventricle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RL07Z"  =  "1"  /* Replacement of Left Ventricle with Autologous Tissue Substitute, Open Approach */
"02RL0JZ"  =  "1"  /* Replacement of Left Ventricle with Synthetic Substitute, Open Approach */
"02RL0KZ"  =  "1"  /* Replacement of Left Ventricle with Nonautologous Tissue Substitute, Open Approach */
"02RL47Z"  =  "1"  /* Replacement of Left Ventricle with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RL4KZ"  =  "1"  /* Replacement of Left Ventricle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RM07Z"  =  "1"  /* Replacement of Ventricular Septum with Autologous Tissue Substitute, Open Approach */
"02RM0JZ"  =  "1"  /* Replacement of Ventricular Septum with Synthetic Substitute, Open Approach */
"02RM0KZ"  =  "1"  /* Replacement of Ventricular Septum with Nonautologous Tissue Substitute, Open Approach */
"02RM47Z"  =  "1"  /* Replacement of Ventricular Septum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RM4JZ"  =  "1"  /* Replacement of Ventricular Septum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RM4KZ"  =  "1"  /* Replacement of Ventricular Septum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RP0JZ"  =  "1"  /* Replacement of Pulmonary Trunk with Synthetic Substitute, Open Approach */
"02RQ07Z"  =  "1"  /* Replacement of Right Pulmonary Artery with Autologous Tissue Substitute, Open Approach */
"02RQ0JZ"  =  "1"  /* Replacement of Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"02RR07Z"  =  "1"  /* Replacement of Left Pulmonary Artery with Autologous Tissue Substitute, Open Approach */
"02RR0JZ"  =  "1"  /* Replacement of Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"02SP0ZZ"  =  "1"  /* Reposition Pulmonary Trunk, Open Approach */
"02SW0ZZ"  =  "1"  /* Reposition Thoracic Aorta, Open Approach */
"02T50ZZ"  =  "1"  /* Resection of Atrial Septum, Open Approach */
"02T54ZZ"  =  "1"  /* Resection of Atrial Septum, Percutaneous Endoscopic Approach */
"02T80ZZ"  =  "1"  /* Resection of Conduction Mechanism, Open Approach */
"02T90ZZ"  =  "1"  /* Resection of Chordae Tendineae, Open Approach */
"02T94ZZ"  =  "1"  /* Resection of Chordae Tendineae, Percutaneous Endoscopic Approach */
"02TD0ZZ"  =  "1"  /* Resection of Papillary Muscle, Open Approach */
"02TD4ZZ"  =  "1"  /* Resection of Papillary Muscle, Percutaneous Endoscopic Approach */
"02TH0ZZ"  =  "1"  /* Resection of Pulmonary Valve, Open Approach */
"02TH4ZZ"  =  "1"  /* Resection of Pulmonary Valve, Percutaneous Endoscopic Approach */
"02TM0ZZ"  =  "1"  /* Resection of Ventricular Septum, Open Approach */
"02TM4ZZ"  =  "1"  /* Resection of Ventricular Septum, Percutaneous Endoscopic Approach */
"02U507Z"  =  "1"  /* Supplement Atrial Septum with Autologous Tissue Substitute, Open Approach */
"02U508Z"  =  "1"  /* Supplement Atrial Septum with Zooplastic Tissue, Open Approach */
"02U50JZ"  =  "1"  /* Supplement Atrial Septum with Synthetic Substitute, Open Approach */
"02U50KZ"  =  "1"  /* Supplement Atrial Septum with Nonautologous Tissue Substitute, Open Approach */
"02U547Z"  =  "1"  /* Supplement Atrial Septum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02U548Z"  =  "1"  /* Supplement Atrial Septum with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02U54JZ"  =  "1"  /* Supplement Atrial Septum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02U54KZ"  =  "1"  /* Supplement Atrial Septum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02U607Z"  =  "1"  /* Supplement Right Atrium with Autologous Tissue Substitute, Open Approach */
"02U608Z"  =  "1"  /* Supplement Right Atrium with Zooplastic Tissue, Open Approach */
"02U60KZ"  =  "1"  /* Supplement Right Atrium with Nonautologous Tissue Substitute, Open Approach */
"02U707Z"  =  "1"  /* Supplement Left Atrium with Autologous Tissue Substitute, Open Approach */
"02U708Z"  =  "1"  /* Supplement Left Atrium with Zooplastic Tissue, Open Approach */
"02U70JZ"  =  "1"  /* Supplement Left Atrium with Synthetic Substitute, Open Approach */
"02U70KZ"  =  "1"  /* Supplement Left Atrium with Nonautologous Tissue Substitute, Open Approach */
"02U747Z"  =  "1"  /* Supplement Left Atrium with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02U748Z"  =  "1"  /* Supplement Left Atrium with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02U74KZ"  =  "1"  /* Supplement Left Atrium with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02U907Z"  =  "1"  /* Supplement Chordae Tendineae with Autologous Tissue Substitute, Open Approach */
"02U908Z"  =  "1"  /* Supplement Chordae Tendineae with Zooplastic Tissue, Open Approach */
"02U90JZ"  =  "1"  /* Supplement Chordae Tendineae with Synthetic Substitute, Open Approach */
"02U90KZ"  =  "1"  /* Supplement Chordae Tendineae with Nonautologous Tissue Substitute, Open Approach */
"02U947Z"  =  "1"  /* Supplement Chordae Tendineae with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02U948Z"  =  "1"  /* Supplement Chordae Tendineae with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02U94JZ"  =  "1"  /* Supplement Chordae Tendineae with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02U94KZ"  =  "1"  /* Supplement Chordae Tendineae with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UD07Z"  =  "1"  /* Supplement Papillary Muscle with Autologous Tissue Substitute, Open Approach */
"02UD08Z"  =  "1"  /* Supplement Papillary Muscle with Zooplastic Tissue, Open Approach */
"02UD0JZ"  =  "1"  /* Supplement Papillary Muscle with Synthetic Substitute, Open Approach */
"02UD0KZ"  =  "1"  /* Supplement Papillary Muscle with Nonautologous Tissue Substitute, Open Approach */
"02UD47Z"  =  "1"  /* Supplement Papillary Muscle with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UD48Z"  =  "1"  /* Supplement Papillary Muscle with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UD4JZ"  =  "1"  /* Supplement Papillary Muscle with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UD4KZ"  =  "1"  /* Supplement Papillary Muscle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UF07Z"  =  "1"  /* Supplement Aortic Valve with Autologous Tissue Substitute, Open Approach */
"02UF08Z"  =  "1"  /* Supplement Aortic Valve with Zooplastic Tissue, Open Approach */
"02UF0JZ"  =  "1"  /* Supplement Aortic Valve with Synthetic Substitute, Open Approach */
"02UF0KZ"  =  "1"  /* Supplement Aortic Valve with Nonautologous Tissue Substitute, Open Approach */
"02UF47Z"  =  "1"  /* Supplement Aortic Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UF48Z"  =  "1"  /* Supplement Aortic Valve with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UF4JZ"  =  "1"  /* Supplement Aortic Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UF4KZ"  =  "1"  /* Supplement Aortic Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UG07Z"  =  "1"  /* Supplement Mitral Valve with Autologous Tissue Substitute, Open Approach */
"02UG08Z"  =  "1"  /* Supplement Mitral Valve with Zooplastic Tissue, Open Approach */
"02UG0JZ"  =  "1"  /* Supplement Mitral Valve with Synthetic Substitute, Open Approach */
"02UG0KZ"  =  "1"  /* Supplement Mitral Valve with Nonautologous Tissue Substitute, Open Approach */
"02UG47Z"  =  "1"  /* Supplement Mitral Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UG48Z"  =  "1"  /* Supplement Mitral Valve with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UG4JZ"  =  "1"  /* Supplement Mitral Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UG4KZ"  =  "1"  /* Supplement Mitral Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UH07Z"  =  "1"  /* Supplement Pulmonary Valve with Autologous Tissue Substitute, Open Approach */
"02UH08Z"  =  "1"  /* Supplement Pulmonary Valve with Zooplastic Tissue, Open Approach */
"02UH0JZ"  =  "1"  /* Supplement Pulmonary Valve with Synthetic Substitute, Open Approach */
"02UH0KZ"  =  "1"  /* Supplement Pulmonary Valve with Nonautologous Tissue Substitute, Open Approach */
"02UH47Z"  =  "1"  /* Supplement Pulmonary Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UH48Z"  =  "1"  /* Supplement Pulmonary Valve with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UH4JZ"  =  "1"  /* Supplement Pulmonary Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UH4KZ"  =  "1"  /* Supplement Pulmonary Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UJ07Z"  =  "1"  /* Supplement Tricuspid Valve with Autologous Tissue Substitute, Open Approach */
"02UJ08Z"  =  "1"  /* Supplement Tricuspid Valve with Zooplastic Tissue, Open Approach */
"02UJ0JZ"  =  "1"  /* Supplement Tricuspid Valve with Synthetic Substitute, Open Approach */
"02UJ0KZ"  =  "1"  /* Supplement Tricuspid Valve with Nonautologous Tissue Substitute, Open Approach */
"02UJ47Z"  =  "1"  /* Supplement Tricuspid Valve with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UJ48Z"  =  "1"  /* Supplement Tricuspid Valve with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UJ4JZ"  =  "1"  /* Supplement Tricuspid Valve with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UJ4KZ"  =  "1"  /* Supplement Tricuspid Valve with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UK0KZ"  =  "1"  /* Supplement Right Ventricle with Nonautologous Tissue Substitute, Open Approach */
"02UK4KZ"  =  "1"  /* Supplement Right Ventricle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UL0KZ"  =  "1"  /* Supplement Left Ventricle with Nonautologous Tissue Substitute, Open Approach */
"02UL4KZ"  =  "1"  /* Supplement Left Ventricle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UM07Z"  =  "1"  /* Supplement Ventricular Septum with Autologous Tissue Substitute, Open Approach */
"02UM0JZ"  =  "1"  /* Supplement Ventricular Septum with Synthetic Substitute, Open Approach */
"02UM0KZ"  =  "1"  /* Supplement Ventricular Septum with Nonautologous Tissue Substitute, Open Approach */
"02UM47Z"  =  "1"  /* Supplement Ventricular Septum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UM48Z"  =  "1"  /* Supplement Ventricular Septum with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UM4JZ"  =  "1"  /* Supplement Ventricular Septum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UM4KZ"  =  "1"  /* Supplement Ventricular Septum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02VR0ZT"  =  "1"  /* Restriction of Ductus Arteriosus, Open Approach */
"02W50JZ"  =  "1"  /* Revision of Synthetic Substitute in Atrial Septum, Open Approach */
"02W54JZ"  =  "1"  /* Revision of Synthetic Substitute in Atrial Septum, Percutaneous Endoscopic Approach */
"02WF07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Aortic Valve, Open Approach */
"02WF08Z"  =  "1"  /* Revision of Zooplastic Tissue in Aortic Valve, Open Approach */
"02WF0JZ"  =  "1"  /* Revision of Synthetic Substitute in Aortic Valve, Open Approach */
"02WF0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Aortic Valve, Open Approach */
"02WF47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Aortic Valve, Percutaneous Endoscopic Approach */
"02WF48Z"  =  "1"  /* Revision of Zooplastic Tissue in Aortic Valve, Percutaneous Endoscopic Approach */
"02WF4JZ"  =  "1"  /* Revision of Synthetic Substitute in Aortic Valve, Percutaneous Endoscopic Approach */
"02WF4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Aortic Valve, Percutaneous Endoscopic Approach */
"02WG07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mitral Valve, Open Approach */
"02WG08Z"  =  "1"  /* Revision of Zooplastic Tissue in Mitral Valve, Open Approach */
"02WG0JZ"  =  "1"  /* Revision of Synthetic Substitute in Mitral Valve, Open Approach */
"02WG0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mitral Valve, Open Approach */
"02WG47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mitral Valve, Percutaneous Endoscopic Approach */
"02WG48Z"  =  "1"  /* Revision of Zooplastic Tissue in Mitral Valve, Percutaneous Endoscopic Approach */
"02WG4JZ"  =  "1"  /* Revision of Synthetic Substitute in Mitral Valve, Percutaneous Endoscopic Approach */
"02WG4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mitral Valve, Percutaneous Endoscopic Approach */
"02WH07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pulmonary Valve, Open Approach */
"02WH08Z"  =  "1"  /* Revision of Zooplastic Tissue in Pulmonary Valve, Open Approach */
"02WH0JZ"  =  "1"  /* Revision of Synthetic Substitute in Pulmonary Valve, Open Approach */
"02WH0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pulmonary Valve, Open Approach */
"02WH47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pulmonary Valve, Percutaneous Endoscopic Approach */
"02WH48Z"  =  "1"  /* Revision of Zooplastic Tissue in Pulmonary Valve, Percutaneous Endoscopic Approach */
"02WH4JZ"  =  "1"  /* Revision of Synthetic Substitute in Pulmonary Valve, Percutaneous Endoscopic Approach */
"02WH4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pulmonary Valve, Percutaneous Endoscopic Approach */
"02WJ07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Tricuspid Valve, Open Approach */
"02WJ08Z"  =  "1"  /* Revision of Zooplastic Tissue in Tricuspid Valve, Open Approach */
"02WJ0JZ"  =  "1"  /* Revision of Synthetic Substitute in Tricuspid Valve, Open Approach */
"02WJ0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Tricuspid Valve, Open Approach */
"02WJ47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Tricuspid Valve, Percutaneous Endoscopic Approach */
"02WJ48Z"  =  "1"  /* Revision of Zooplastic Tissue in Tricuspid Valve, Percutaneous Endoscopic Approach */
"02WJ4JZ"  =  "1"  /* Revision of Synthetic Substitute in Tricuspid Valve, Percutaneous Endoscopic Approach */
"02WJ4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Tricuspid Valve, Percutaneous Endoscopic Approach */
"02WM0JZ"  =  "1"  /* Revision of Synthetic Substitute in Ventricular Septum, Open Approach */
"02WM4JZ"  =  "1"  /* Revision of Synthetic Substitute in Ventricular Septum, Percutaneous Endoscopic Approach */
"02YA0Z0"  =  "1"  /* Transplantation of Heart, Allogeneic, Open Approach */
"02YA0Z1"  =  "1"  /* Transplantation of Heart, Syngeneic, Open Approach */
"02YA0Z2"  =  "1"  /* Transplantation of Heart, Zooplastic, Open Approach */
"031309M"  =  "1"  /* Bypass Right Subclavian Artery to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"031309N"  =  "1"  /* Bypass Right Subclavian Artery to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"03130AM"  =  "1"  /* Bypass Right Subclavian Artery to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"03130AN"  =  "1"  /* Bypass Right Subclavian Artery to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"03130JM"  =  "1"  /* Bypass Right Subclavian Artery to Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"03130JN"  =  "1"  /* Bypass Right Subclavian Artery to Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"03130KM"  =  "1"  /* Bypass Right Subclavian Artery to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"03130KN"  =  "1"  /* Bypass Right Subclavian Artery to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"03130ZM"  =  "1"  /* Bypass Right Subclavian Artery to Right Pulmonary Artery, Open Approach */
"03130ZN"  =  "1"  /* Bypass Right Subclavian Artery to Left Pulmonary Artery, Open Approach */
"031409M"  =  "1"  /* Bypass Left Subclavian Artery to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"031409N"  =  "1"  /* Bypass Left Subclavian Artery to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach */
"03140AM"  =  "1"  /* Bypass Left Subclavian Artery to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"03140AN"  =  "1"  /* Bypass Left Subclavian Artery to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach */
"03140JM"  =  "1"  /* Bypass Left Subclavian Artery to Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"03140JN"  =  "1"  /* Bypass Left Subclavian Artery to Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"03140KM"  =  "1"  /* Bypass Left Subclavian Artery to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"03140KN"  =  "1"  /* Bypass Left Subclavian Artery to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"03140ZM"  =  "1"  /* Bypass Left Subclavian Artery to Right Pulmonary Artery, Open Approach */
"03140ZN"  =  "1"  /* Bypass Left Subclavian Artery to Left Pulmonary Artery, Open Approach */
OTHER  = "0" ;

 /* Pediatric Heart Surgery (#2): */
 /* Other cardiac surgery */

 /* ICD-10-CM Diagnosis Codes: */

VALUE $PRPED2D
"Q200"  =  "1"  /* Common arterial trunk */
"Q201"  =  "1"  /* Double outlet right ventricle */
"Q202"  =  "1"  /* Double outlet left ventricle */
"Q203"  =  "1"  /* Discordant ventriculoarterial connection */
"Q204"  =  "1"  /* Double inlet ventricle */
"Q205"  =  "1"  /* Discordant atrioventricular connection */
"Q206"  =  "1"  /* Isomerism of atrial appendages */
"Q208"  =  "1"  /* Other congenital malformations of cardiac chambers and connections */
"Q209"  =  "1"  /* Congenital malformation of cardiac chambers and connections, unspecified */
"Q210"  =  "1"  /* Ventricular septal defect */
"Q211"  =  "1"  /* Atrial septal defect */
"Q212"  =  "1"  /* Atrioventricular septal defect */
"Q213"  =  "1"  /* Tetralogy of Fallot */
"Q214"  =  "1"  /* Aortopulmonary septal defect */
"Q218"  =  "1"  /* Other congenital malformations of cardiac septa */
"Q219"  =  "1"  /* Congenital malformation of cardiac septum, unspecified */
"Q220"  =  "1"  /* Pulmonary valve atresia */
"Q221"  =  "1"  /* Congenital pulmonary valve stenosis */
"Q222"  =  "1"  /* Congenital pulmonary valve insufficiency */
"Q223"  =  "1"  /* Other congenital malformations of pulmonary valve */
"Q224"  =  "1"  /* Congenital tricuspid stenosis */
"Q225"  =  "1"  /* Ebstein's anomaly */
"Q226"  =  "1"  /* Hypoplastic right heart syndrome */
"Q228"  =  "1"  /* Other congenital malformations of tricuspid valve */
"Q229"  =  "1"  /* Congenital malformation of tricuspid valve, unspecified */
"Q230"  =  "1"  /* Congenital stenosis of aortic valve */
"Q231"  =  "1"  /* Congenital insufficiency of aortic valve */
"Q232"  =  "1"  /* Congenital mitral stenosis */
"Q233"  =  "1"  /* Congenital mitral insufficiency */
"Q234"  =  "1"  /* Hypoplastic left heart syndrome */
"Q238"  =  "1"  /* Other congenital malformations of aortic and mitral valves */
"Q239"  =  "1"  /* Congenital malformation of aortic and mitral valves, unspecified */
"Q240"  =  "1"  /* Dextrocardia */
"Q242"  =  "1"  /* Cor triatriatum */
"Q243"  =  "1"  /* Pulmonary infundibular stenosis */
"Q244"  =  "1"  /* Congenital subaortic stenosis */
"Q245"  =  "1"  /* Malformation of coronary vessels */
"Q248"  =  "1"  /* Other specified congenital malformations of heart */
"Q249"  =  "1"  /* Congenital malformation of heart, unspecified */
"Q250"  =  "1"  /* Patent ductus arteriosus */
"Q251"  =  "1"  /* Coarctation of aorta */
"Q252"  =  "1"  /* Atresia of aorta */
"Q253"  =  "1"  /* Supravalvular aortic stenosis */
"Q254"  =  "1"  /* Other congenital malformations of aorta */
"Q255"  =  "1"  /* Atresia of pulmonary artery */
"Q256"  =  "1"  /* Stenosis of pulmonary artery */
"Q2571"  =  "1"  /* Coarctation of pulmonary artery */
"Q2572"  =  "1"  /* Congenital pulmonary arteriovenous malformation */
"Q2579"  =  "1"  /* Other congenital malformations of pulmonary artery */
"Q258"  =  "1"  /* Other congenital malformations of other great arteries */
"Q259"  =  "1"  /* Congenital malformation of great arteries, unspecified */
"Q260"  =  "1"  /* Congenital stenosis of vena cava */
"Q261"  =  "1"  /* Persistent left superior vena cava */
"Q262"  =  "1"  /* Total anomalous pulmonary venous connection */
"Q263"  =  "1"  /* Partial anomalous pulmonary venous connection */
"Q264"  =  "1"  /* Anomalous pulmonary venous connection, unspecified */
"Q268"  =  "1"  /* Other congenital malformations of great veins */
"Q269"  =  "1"  /* Congenital malformation of great vein, unspecified */
OTHER   = "0" ;

 /* ICD-10-CM Procedure Codes: */

VALUE $PRPED2P
"025P0ZZ"  =  "1"  /* Destruction of Pulmonary Trunk, Open Approach */
"025P4ZZ"  =  "1"  /* Destruction of Pulmonary Trunk, Percutaneous Endoscopic Approach */
"025Q0ZZ"  =  "1"  /* Destruction of Right Pulmonary Artery, Open Approach */
"025Q4ZZ"  =  "1"  /* Destruction of Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"025R0ZZ"  =  "1"  /* Destruction of Left Pulmonary Artery, Open Approach */
"025R4ZZ"  =  "1"  /* Destruction of Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"025S0ZZ"  =  "1"  /* Destruction of Right Pulmonary Vein, Open Approach */
"025S4ZZ"  =  "1"  /* Destruction of Right Pulmonary Vein, Percutaneous Endoscopic Approach */
"025T0ZZ"  =  "1"  /* Destruction of Left Pulmonary Vein, Open Approach */
"025T4ZZ"  =  "1"  /* Destruction of Left Pulmonary Vein, Percutaneous Endoscopic Approach */
"025V0ZZ"  =  "1"  /* Destruction of Superior Vena Cava, Open Approach */
"025V4ZZ"  =  "1"  /* Destruction of Superior Vena Cava, Percutaneous Endoscopic Approach */
"025W0ZZ"  =  "1"  /* Destruction of Thoracic Aorta, Open Approach */
"025W4ZZ"  =  "1"  /* Destruction of Thoracic Aorta, Percutaneous Endoscopic Approach */
"02BP0ZZ"  =  "1"  /* Excision of Pulmonary Trunk, Open Approach */
"02BP4ZZ"  =  "1"  /* Excision of Pulmonary Trunk, Percutaneous Endoscopic Approach */
"02BQ0ZZ"  =  "1"  /* Excision of Right Pulmonary Artery, Open Approach */
"02BQ4ZZ"  =  "1"  /* Excision of Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"02BR0ZZ"  =  "1"  /* Excision of Left Pulmonary Artery, Open Approach */
"02BR4ZZ"  =  "1"  /* Excision of Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"02BS0ZZ"  =  "1"  /* Excision of Right Pulmonary Vein, Open Approach */
"02BS4ZZ"  =  "1"  /* Excision of Right Pulmonary Vein, Percutaneous Endoscopic Approach */
"02BT0ZZ"  =  "1"  /* Excision of Left Pulmonary Vein, Open Approach */
"02BT4ZZ"  =  "1"  /* Excision of Left Pulmonary Vein, Percutaneous Endoscopic Approach */
"02BV0ZZ"  =  "1"  /* Excision of Superior Vena Cava, Open Approach */
"02BV4ZZ"  =  "1"  /* Excision of Superior Vena Cava, Percutaneous Endoscopic Approach */
"02BW0ZZ"  =  "1"  /* Excision of Thoracic Aorta, Open Approach */
"02BW4ZZ"  =  "1"  /* Excision of Thoracic Aorta, Percutaneous Endoscopic Approach */
"02LR0CT"  =  "1"  /* Occlusion of Ductus Arteriosus with Extraluminal Device, Open Approach */
"02LR0DT"  =  "1"  /* Occlusion of Ductus Arteriosus with Intraluminal Device, Open Approach */
"02LR0ZT"  =  "1"  /* Occlusion of Ductus Arteriosus, Open Approach */
"02LR4CT"  =  "1"  /* Occlusion of Ductus Arteriosus with Extraluminal Device, Percutaneous Endoscopic Approach */
"02LR4DT"  =  "1"  /* Occlusion of Ductus Arteriosus with Intraluminal Device, Percutaneous Endoscopic Approach */
"02LR4ZT"  =  "1"  /* Occlusion of Ductus Arteriosus, Percutaneous Endoscopic Approach */
"02LS0CZ"  =  "1"  /* Occlusion of Right Pulmonary Vein with Extraluminal Device, Open Approach */
"02LS0DZ"  =  "1"  /* Occlusion of Right Pulmonary Vein with Intraluminal Device, Open Approach */
"02LS0ZZ"  =  "1"  /* Occlusion of Right Pulmonary Vein, Open Approach */
"02LS4CZ"  =  "1"  /* Occlusion of Right Pulmonary Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"02LS4DZ"  =  "1"  /* Occlusion of Right Pulmonary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"02LS4ZZ"  =  "1"  /* Occlusion of Right Pulmonary Vein, Percutaneous Endoscopic Approach */
"02LT0CZ"  =  "1"  /* Occlusion of Left Pulmonary Vein with Extraluminal Device, Open Approach */
"02LT0DZ"  =  "1"  /* Occlusion of Left Pulmonary Vein with Intraluminal Device, Open Approach */
"02LT0ZZ"  =  "1"  /* Occlusion of Left Pulmonary Vein, Open Approach */
"02LT4CZ"  =  "1"  /* Occlusion of Left Pulmonary Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"02LT4DZ"  =  "1"  /* Occlusion of Left Pulmonary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"02LT4ZZ"  =  "1"  /* Occlusion of Left Pulmonary Vein, Percutaneous Endoscopic Approach */
"02RP07Z"  =  "1"  /* Replacement of Pulmonary Trunk with Autologous Tissue Substitute, Open Approach */
"02RP08Z"  =  "1"  /* Replacement of Pulmonary Trunk with Zooplastic Tissue, Open Approach */
"02RP0JZ"  =  "1"  /* Replacement of Pulmonary Trunk with Synthetic Substitute, Open Approach */
"02RP0KZ"  =  "1"  /* Replacement of Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach */
"02RP47Z"  =  "1"  /* Replacement of Pulmonary Trunk with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RP48Z"  =  "1"  /* Replacement of Pulmonary Trunk with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RP4JZ"  =  "1"  /* Replacement of Pulmonary Trunk with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RP4KZ"  =  "1"  /* Replacement of Pulmonary Trunk with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RQ07Z"  =  "1"  /* Replacement of Right Pulmonary Artery with Autologous Tissue Substitute, Open Approach */
"02RQ08Z"  =  "1"  /* Replacement of Right Pulmonary Artery with Zooplastic Tissue, Open Approach */
"02RQ0JZ"  =  "1"  /* Replacement of Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"02RQ0KZ"  =  "1"  /* Replacement of Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"02RQ47Z"  =  "1"  /* Replacement of Right Pulmonary Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RQ48Z"  =  "1"  /* Replacement of Right Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RQ4JZ"  =  "1"  /* Replacement of Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RQ4KZ"  =  "1"  /* Replacement of Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RR07Z"  =  "1"  /* Replacement of Left Pulmonary Artery with Autologous Tissue Substitute, Open Approach */
"02RR08Z"  =  "1"  /* Replacement of Left Pulmonary Artery with Zooplastic Tissue, Open Approach */
"02RR0JZ"  =  "1"  /* Replacement of Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"02RR0KZ"  =  "1"  /* Replacement of Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"02RR47Z"  =  "1"  /* Replacement of Left Pulmonary Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RR48Z"  =  "1"  /* Replacement of Left Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RR4JZ"  =  "1"  /* Replacement of Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RR4KZ"  =  "1"  /* Replacement of Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RS07Z"  =  "1"  /* Replacement of Right Pulmonary Vein with Autologous Tissue Substitute, Open Approach */
"02RS08Z"  =  "1"  /* Replacement of Right Pulmonary Vein with Zooplastic Tissue, Open Approach */
"02RS0JZ"  =  "1"  /* Replacement of Right Pulmonary Vein with Synthetic Substitute, Open Approach */
"02RS0KZ"  =  "1"  /* Replacement of Right Pulmonary Vein with Nonautologous Tissue Substitute, Open Approach */
"02RS47Z"  =  "1"  /* Replacement of Right Pulmonary Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RS48Z"  =  "1"  /* Replacement of Right Pulmonary Vein with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RS4JZ"  =  "1"  /* Replacement of Right Pulmonary Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RS4KZ"  =  "1"  /* Replacement of Right Pulmonary Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RT07Z"  =  "1"  /* Replacement of Left Pulmonary Vein with Autologous Tissue Substitute, Open Approach */
"02RT08Z"  =  "1"  /* Replacement of Left Pulmonary Vein with Zooplastic Tissue, Open Approach */
"02RT0JZ"  =  "1"  /* Replacement of Left Pulmonary Vein with Synthetic Substitute, Open Approach */
"02RT0KZ"  =  "1"  /* Replacement of Left Pulmonary Vein with Nonautologous Tissue Substitute, Open Approach */
"02RT47Z"  =  "1"  /* Replacement of Left Pulmonary Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RT48Z"  =  "1"  /* Replacement of Left Pulmonary Vein with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RT4JZ"  =  "1"  /* Replacement of Left Pulmonary Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RT4KZ"  =  "1"  /* Replacement of Left Pulmonary Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RV07Z"  =  "1"  /* Replacement of Superior Vena Cava with Autologous Tissue Substitute, Open Approach */
"02RV08Z"  =  "1"  /* Replacement of Superior Vena Cava with Zooplastic Tissue, Open Approach */
"02RV0JZ"  =  "1"  /* Replacement of Superior Vena Cava with Synthetic Substitute, Open Approach */
"02RV0KZ"  =  "1"  /* Replacement of Superior Vena Cava with Nonautologous Tissue Substitute, Open Approach */
"02RV47Z"  =  "1"  /* Replacement of Superior Vena Cava with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RV48Z"  =  "1"  /* Replacement of Superior Vena Cava with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RV4JZ"  =  "1"  /* Replacement of Superior Vena Cava with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RV4KZ"  =  "1"  /* Replacement of Superior Vena Cava with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RW07Z"  =  "1"  /* Replacement of Thoracic Aorta with Autologous Tissue Substitute, Open Approach */
"02RW08Z"  =  "1"  /* Replacement of Thoracic Aorta with Zooplastic Tissue, Open Approach */
"02RW0JZ"  =  "1"  /* Replacement of Thoracic Aorta with Synthetic Substitute, Open Approach */
"02RW0KZ"  =  "1"  /* Replacement of Thoracic Aorta with Nonautologous Tissue Substitute, Open Approach */
"02RW47Z"  =  "1"  /* Replacement of Thoracic Aorta with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02RW48Z"  =  "1"  /* Replacement of Thoracic Aorta with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02RW4JZ"  =  "1"  /* Replacement of Thoracic Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02RW4KZ"  =  "1"  /* Replacement of Thoracic Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02SP0ZZ"  =  "1"  /* Reposition Pulmonary Trunk, Open Approach */
"02SQ0ZZ"  =  "1"  /* Reposition Right Pulmonary Artery, Open Approach */
"02SR0ZZ"  =  "1"  /* Reposition Left Pulmonary Artery, Open Approach */
"02SS0ZZ"  =  "1"  /* Reposition Right Pulmonary Vein, Open Approach */
"02ST0ZZ"  =  "1"  /* Reposition Left Pulmonary Vein, Open Approach */
"02SV0ZZ"  =  "1"  /* Reposition Superior Vena Cava, Open Approach */
"02SW0ZZ"  =  "1"  /* Reposition Thoracic Aorta, Open Approach */
"02UP07Z"  =  "1"  /* Supplement Pulmonary Trunk with Autologous Tissue Substitute, Open Approach */
"02UP08Z"  =  "1"  /* Supplement Pulmonary Trunk with Zooplastic Tissue, Open Approach */
"02UP0JZ"  =  "1"  /* Supplement Pulmonary Trunk with Synthetic Substitute, Open Approach */
"02UP0KZ"  =  "1"  /* Supplement Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach */
"02UP47Z"  =  "1"  /* Supplement Pulmonary Trunk with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UP48Z"  =  "1"  /* Supplement Pulmonary Trunk with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UP4JZ"  =  "1"  /* Supplement Pulmonary Trunk with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UP4KZ"  =  "1"  /* Supplement Pulmonary Trunk with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UQ07Z"  =  "1"  /* Supplement Right Pulmonary Artery with Autologous Tissue Substitute, Open Approach */
"02UQ08Z"  =  "1"  /* Supplement Right Pulmonary Artery with Zooplastic Tissue, Open Approach */
"02UQ0JZ"  =  "1"  /* Supplement Right Pulmonary Artery with Synthetic Substitute, Open Approach */
"02UQ0KZ"  =  "1"  /* Supplement Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"02UQ47Z"  =  "1"  /* Supplement Right Pulmonary Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UQ48Z"  =  "1"  /* Supplement Right Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UQ4JZ"  =  "1"  /* Supplement Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UQ4KZ"  =  "1"  /* Supplement Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UR07Z"  =  "1"  /* Supplement Left Pulmonary Artery with Autologous Tissue Substitute, Open Approach */
"02UR08Z"  =  "1"  /* Supplement Left Pulmonary Artery with Zooplastic Tissue, Open Approach */
"02UR0JZ"  =  "1"  /* Supplement Left Pulmonary Artery with Synthetic Substitute, Open Approach */
"02UR0KZ"  =  "1"  /* Supplement Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach */
"02UR47Z"  =  "1"  /* Supplement Left Pulmonary Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UR48Z"  =  "1"  /* Supplement Left Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UR4JZ"  =  "1"  /* Supplement Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UR4KZ"  =  "1"  /* Supplement Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02US07Z"  =  "1"  /* Supplement Right Pulmonary Vein with Autologous Tissue Substitute, Open Approach */
"02US08Z"  =  "1"  /* Supplement Right Pulmonary Vein with Zooplastic Tissue, Open Approach */
"02US0JZ"  =  "1"  /* Supplement Right Pulmonary Vein with Synthetic Substitute, Open Approach */
"02US0KZ"  =  "1"  /* Supplement Right Pulmonary Vein with Nonautologous Tissue Substitute, Open Approach */
"02US47Z"  =  "1"  /* Supplement Right Pulmonary Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02US48Z"  =  "1"  /* Supplement Right Pulmonary Vein with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02US4JZ"  =  "1"  /* Supplement Right Pulmonary Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02US4KZ"  =  "1"  /* Supplement Right Pulmonary Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UT07Z"  =  "1"  /* Supplement Left Pulmonary Vein with Autologous Tissue Substitute, Open Approach */
"02UT08Z"  =  "1"  /* Supplement Left Pulmonary Vein with Zooplastic Tissue, Open Approach */
"02UT0JZ"  =  "1"  /* Supplement Left Pulmonary Vein with Synthetic Substitute, Open Approach */
"02UT0KZ"  =  "1"  /* Supplement Left Pulmonary Vein with Nonautologous Tissue Substitute, Open Approach */
"02UT47Z"  =  "1"  /* Supplement Left Pulmonary Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UT48Z"  =  "1"  /* Supplement Left Pulmonary Vein with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UT4JZ"  =  "1"  /* Supplement Left Pulmonary Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UT4KZ"  =  "1"  /* Supplement Left Pulmonary Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UV07Z"  =  "1"  /* Supplement Superior Vena Cava with Autologous Tissue Substitute, Open Approach */
"02UV08Z"  =  "1"  /* Supplement Superior Vena Cava with Zooplastic Tissue, Open Approach */
"02UV0JZ"  =  "1"  /* Supplement Superior Vena Cava with Synthetic Substitute, Open Approach */
"02UV0KZ"  =  "1"  /* Supplement Superior Vena Cava with Nonautologous Tissue Substitute, Open Approach */
"02UV47Z"  =  "1"  /* Supplement Superior Vena Cava with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UV48Z"  =  "1"  /* Supplement Superior Vena Cava with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UV4JZ"  =  "1"  /* Supplement Superior Vena Cava with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UV4KZ"  =  "1"  /* Supplement Superior Vena Cava with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UW07Z"  =  "1"  /* Supplement Thoracic Aorta with Autologous Tissue Substitute, Open Approach */
"02UW08Z"  =  "1"  /* Supplement Thoracic Aorta with Zooplastic Tissue, Open Approach */
"02UW0JZ"  =  "1"  /* Supplement Thoracic Aorta with Synthetic Substitute, Open Approach */
"02UW0KZ"  =  "1"  /* Supplement Thoracic Aorta with Nonautologous Tissue Substitute, Open Approach */
"02UW47Z"  =  "1"  /* Supplement Thoracic Aorta with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02UW48Z"  =  "1"  /* Supplement Thoracic Aorta with Zooplastic Tissue, Percutaneous Endoscopic Approach */
"02UW4JZ"  =  "1"  /* Supplement Thoracic Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach */
"02UW4KZ"  =  "1"  /* Supplement Thoracic Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"02VQ0CZ"  =  "1"  /* Restriction of Right Pulmonary Artery with Extraluminal Device, Open Approach */
"02VQ4CZ"  =  "1"  /* Restriction of Right Pulmonary Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"02VR0CZ"  =  "1"  /* Restriction of Left Pulmonary Artery with Extraluminal Device, Open Approach */
"02VR4CZ"  =  "1"  /* Restriction of Left Pulmonary Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04500ZZ"  =  "1"  /* Destruction of Abdominal Aorta, Open Approach */
"04504ZZ"  =  "1"  /* Destruction of Abdominal Aorta, Percutaneous Endoscopic Approach */
"04B00ZZ"  =  "1"  /* Excision of Abdominal Aorta, Open Approach */
"04B04ZZ"  =  "1"  /* Excision of Abdominal Aorta, Percutaneous Endoscopic Approach */
"04L00CZ"  =  "1"  /* Occlusion of Abdominal Aorta with Extraluminal Device, Open Approach */
"04L00DZ"  =  "1"  /* Occlusion of Abdominal Aorta with Intraluminal Device, Open Approach */
"04L00ZZ"  =  "1"  /* Occlusion of Abdominal Aorta, Open Approach */
"04L04CZ"  =  "1"  /* Occlusion of Abdominal Aorta with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L04DZ"  =  "1"  /* Occlusion of Abdominal Aorta with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L04ZZ"  =  "1"  /* Occlusion of Abdominal Aorta, Percutaneous Endoscopic Approach */
"04R007Z"  =  "1"  /* Replacement of Abdominal Aorta with Autologous Tissue Substitute, Open Approach */
"04R00JZ"  =  "1"  /* Replacement of Abdominal Aorta with Synthetic Substitute, Open Approach */
"04R00KZ"  =  "1"  /* Replacement of Abdominal Aorta with Nonautologous Tissue Substitute, Open Approach */
"04R047Z"  =  "1"  /* Replacement of Abdominal Aorta with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R04JZ"  =  "1"  /* Replacement of Abdominal Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04R04KZ"  =  "1"  /* Replacement of Abdominal Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04S00ZZ"  =  "1"  /* Reposition Abdominal Aorta, Open Approach */
"04S04ZZ"  =  "1"  /* Reposition Abdominal Aorta, Percutaneous Endoscopic Approach */
"04U007Z"  =  "1"  /* Supplement Abdominal Aorta with Autologous Tissue Substitute, Open Approach */
"04U00JZ"  =  "1"  /* Supplement Abdominal Aorta with Synthetic Substitute, Open Approach */
"04U00KZ"  =  "1"  /* Supplement Abdominal Aorta with Nonautologous Tissue Substitute, Open Approach */
"04U047Z"  =  "1"  /* Supplement Abdominal Aorta with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04U04JZ"  =  "1"  /* Supplement Abdominal Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04U04KZ"  =  "1"  /* Supplement Abdominal Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06S00ZZ"  =  "1"  /* Reposition Inferior Vena Cava, Open Approach */
"06S04ZZ"  =  "1"  /* Reposition Inferior Vena Cava, Percutaneous Endoscopic Approach */
"06U007Z"  =  "1"  /* Supplement Inferior Vena Cava with Autologous Tissue Substitute, Open Approach */
"06U00JZ"  =  "1"  /* Supplement Inferior Vena Cava with Synthetic Substitute, Open Approach */
"06U00KZ"  =  "1"  /* Supplement Inferior Vena Cava with Nonautologous Tissue Substitute, Open Approach */
"06U047Z"  =  "1"  /* Supplement Inferior Vena Cava with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06U04JZ"  =  "1"  /* Supplement Inferior Vena Cava with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06U04KZ"  =  "1"  /* Supplement Inferior Vena Cava with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
OTHER  = "0" ;

 /* Pediatric Heart Surgery */
 /* Exclusions */

 /* PHS Exclusion #1: */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $PRPED3D
"Q250"  =  "1"  /* Patent ductus arteriosus */
OTHER  = "0" ;

 /* ICD-10-CM Procedure Codes: */

VALUE $PRPD3AP
"02NF4ZZ"  =  "1"  /* Release Aortic Valve, Percutaneous Endoscopic Approach */
"02NG4ZZ"  =  "1"  /* Release Mitral Valve, Percutaneous Endoscopic Approach */
"02NH4ZZ"  =  "1"  /* Release Pulmonary Valve, Percutaneous Endoscopic Approach */
"02NJ4ZZ"  =  "1"  /* Release Tricuspid Valve, Percutaneous Endoscopic Approach */
OTHER  = "0" ;

VALUE $PRPD3BP
"02B50ZZ"  =  "1"  /* Excision of Atrial Septum, Open Approach */
"02B54ZZ"  =  "1"  /* Excision of Atrial Septum, Percutaneous Endoscopic Approach */
OTHER  = "0" ;

VALUE $PRPD3CP
"02Q50ZZ"  =  "1"  /* Repair Atrial Septum, Open Approach */
"02Q54ZZ"  =  "1"  /* Repair Atrial Septum, Percutaneous Endoscopic Approach */
"02U50JZ"  =  "1"  /* Supplement Atrial Septum with Synthetic Substitute, Open Approach */
OTHER  = "0" ;

VALUE $PRPD3DP
"02QM0ZZ"  =  "1"  /* Repair Ventricular Septum, Open Approach */
"02QM4ZZ"  =  "1"  /* Repair Ventricular Septum, Percutaneous Endoscopic Approach */
"02RM0JZ"  =  "1"  /* Replacement of Ventricular Septum with Synthetic Substitute, Open Approach */
"02UM0JZ"  =  "1"  /* Supplement Ventricular Septum with Synthetic Substitute, Open Approach */
"02UM4JZ"  =  "1"  /* Supplement Ventricular Septum with Synthetic Substitute, Percutaneous Endoscopic Approach */
OTHER  = "0" ;

VALUE $PRPD3EP
"02LR0CT"  =  "1"  /* Occlusion of Ductus Arteriosus with Extraluminal Device, Open Approach */
"02LR0DT"  =  "1"  /* Occlusion of Ductus Arteriosus with Intraluminal Device, Open Approach */
"02LR0ZT"  =  "1"  /* Occlusion of Ductus Arteriosus, Open Approach */
"02LR4CT"  =  "1"  /* Occlusion of Ductus Arteriosus with Extraluminal Device, Percutaneous Endoscopic Approach */
"02LR4DT"  =  "1"  /* Occlusion of Ductus Arteriosus with Intraluminal Device, Percutaneous Endoscopic Approach */
"02LR4ZT"  =  "1"  /* Occlusion of Ductus Arteriosus, Percutaneous Endoscopic Approach */
OTHER  = "0" ;

VALUE $PRPD3FP
"02LR0CT"  =  "1"  /* Occlusion of Ductus Arteriosus with Extraluminal Device, Open Approach */
"02LR0DT"  =  "1"  /* Occlusion of Ductus Arteriosus with Intraluminal Device, Open Approach */
"02LR0ZT"  =  "1"  /* Occlusion of Ductus Arteriosus, Open Approach */
"02LR4CT"  =  "1"  /* Occlusion of Ductus Arteriosus with Extraluminal Device, Percutaneous Endoscopic Approach */
"02LR4DT"  =  "1"  /* Occlusion of Ductus Arteriosus with Intraluminal Device, Percutaneous Endoscopic Approach */
"02LR4ZT"  =  "1"  /* Occlusion of Ductus Arteriosus, Percutaneous Endoscopic Approach */
"02SP0ZZ"  =  "1"  /* Reposition Pulmonary Trunk, Open Approach */
"02SQ0ZZ"  =  "1"  /* Reposition Right Pulmonary Artery, Open Approach */
"02SR0ZZ"  =  "1"  /* Reposition Left Pulmonary Artery, Open Approach */
"02SS0ZZ"  =  "1"  /* Reposition Right Pulmonary Vein, Open Approach */
"02ST0ZZ"  =  "1"  /* Reposition Left Pulmonary Vein, Open Approach */
"02SV0ZZ"  =  "1"  /* Reposition Superior Vena Cava, Open Approach */
"02SW0ZZ"  =  "1"  /* Reposition Thoracic Aorta, Open Approach */
"04L00CZ"  =  "1"  /* Occlusion of Abdominal Aorta with Extraluminal Device, Open Approach */
"04L00DZ"  =  "1"  /* Occlusion of Abdominal Aorta with Intraluminal Device, Open Approach */
"04L00ZZ"  =  "1"  /* Occlusion of Abdominal Aorta, Open Approach */
"04L04CZ"  =  "1"  /* Occlusion of Abdominal Aorta with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L04DZ"  =  "1"  /* Occlusion of Abdominal Aorta with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L04ZZ"  =  "1"  /* Occlusion of Abdominal Aorta, Percutaneous Endoscopic Approach */
"04S00ZZ"  =  "1"  /* Reposition Abdominal Aorta, Open Approach */
"04S04ZZ"  =  "1"  /* Reposition Abdominal Aorta, Percutaneous Endoscopic Approach */
"06S00ZZ"  =  "1"  /* Reposition Inferior Vena Cava, Open Approach */
"06S04ZZ"  =  "1"  /* Reposition Inferior Vena Cava, Percutaneous Endoscopic Approach */
OTHER  = "0" ;

VALUE $PRPED5P
"5A1221Z"  =  "1"  /* Performance of Cardiac Output, Continuous */
OTHER  = "0" ;

VALUE $PRPED6P
"4A020N6"  =  "1"  /* Measurement of Cardiac Sampling and Pressure, Right Heart, Open Approach */
"4A020N7"  =  "1"  /* Measurement of Cardiac Sampling and Pressure, Left Heart, Open Approach */
"4A020N8"  =  "1"  /* Measurement of Cardiac Sampling and Pressure, Bilateral, Open Approach */
"4A023N6"  =  "1"  /* Measurement of Cardiac Sampling and Pressure, Right Heart, Percutaneous Approach */
"4A023N7"  =  "1"  /* Measurement of Cardiac Sampling and Pressure, Left Heart, Percutaneous Approach */
"4A023N8"  =  "1"  /* Measurement of Cardiac Sampling and Pressure, Bilateral, Percutaneous Approach */
"B2000ZZ"  =  "1"  /* Plain Radiography of Single Coronary Artery using High Osmolar Contrast */
"B2001ZZ"  =  "1"  /* Plain Radiography of Single Coronary Artery using Low Osmolar Contrast */
"B200YZZ"  =  "1"  /* Plain Radiography of Single Coronary Artery using Other Contrast */
"B2010ZZ"  =  "1"  /* Plain Radiography of Multiple Coronary Arteries using High Osmolar Contrast */
"B2011ZZ"  =  "1"  /* Plain Radiography of Multiple Coronary Arteries using Low Osmolar Contrast */
"B201YZZ"  =  "1"  /* Plain Radiography of Multiple Coronary Arteries using Other Contrast */
"B2020ZZ"  =  "1"  /* Plain Radiography of Single Coronary Artery Bypass Graft using High Osmolar Contrast */
"B2021ZZ"  =  "1"  /* Plain Radiography of Single Coronary Artery Bypass Graft using Low Osmolar Contrast */
"B202YZZ"  =  "1"  /* Plain Radiography of Single Coronary Artery Bypass Graft using Other Contrast */
"B2030ZZ"  =  "1"  /* Plain Radiography of Multiple Coronary Artery Bypass Grafts using High Osmolar Contrast */
"B2031ZZ"  =  "1"  /* Plain Radiography of Multiple Coronary Artery Bypass Grafts using Low Osmolar Contrast */
"B203YZZ"  =  "1"  /* Plain Radiography of Multiple Coronary Artery Bypass Grafts using Other Contrast */
"B2040ZZ"  =  "1"  /* Plain Radiography of Right Heart using High Osmolar Contrast */
"B2041ZZ"  =  "1"  /* Plain Radiography of Right Heart using Low Osmolar Contrast */
"B204YZZ"  =  "1"  /* Plain Radiography of Right Heart using Other Contrast */
"B2050ZZ"  =  "1"  /* Plain Radiography of Left Heart using High Osmolar Contrast */
"B2051ZZ"  =  "1"  /* Plain Radiography of Left Heart using Low Osmolar Contrast */
"B205YZZ"  =  "1"  /* Plain Radiography of Left Heart using Other Contrast */
"B2060ZZ"  =  "1"  /* Plain Radiography of Right and Left Heart using High Osmolar Contrast */
"B2061ZZ"  =  "1"  /* Plain Radiography of Right and Left Heart using Low Osmolar Contrast */
"B206YZZ"  =  "1"  /* Plain Radiography of Right and Left Heart using Other Contrast */
"B2070ZZ"  =  "1"  /* Plain Radiography of Right Internal Mammary Bypass Graft using High Osmolar Contrast */
"B2071ZZ"  =  "1"  /* Plain Radiography of Right Internal Mammary Bypass Graft using Low Osmolar Contrast */
"B207YZZ"  =  "1"  /* Plain Radiography of Right Internal Mammary Bypass Graft using Other Contrast */
"B2080ZZ"  =  "1"  /* Plain Radiography of Left Internal Mammary Bypass Graft using High Osmolar Contrast */
"B2081ZZ"  =  "1"  /* Plain Radiography of Left Internal Mammary Bypass Graft using Low Osmolar Contrast */
"B208YZZ"  =  "1"  /* Plain Radiography of Left Internal Mammary Bypass Graft using Other Contrast */
"B20F0ZZ"  =  "1"  /* Plain Radiography of Other Bypass Graft using High Osmolar Contrast */
"B20F1ZZ"  =  "1"  /* Plain Radiography of Other Bypass Graft using Low Osmolar Contrast */
"B20FYZZ"  =  "1"  /* Plain Radiography of Other Bypass Graft using Other Contrast */
"B2100ZZ"  =  "1"  /* Fluoroscopy of Single Coronary Artery using High Osmolar Contrast */
"B2101ZZ"  =  "1"  /* Fluoroscopy of Single Coronary Artery using Low Osmolar Contrast */
"B210YZZ"  =  "1"  /* Fluoroscopy of Single Coronary Artery using Other Contrast */
"B2110ZZ"  =  "1"  /* Fluoroscopy of Multiple Coronary Arteries using High Osmolar Contrast */
"B2111ZZ"  =  "1"  /* Fluoroscopy of Multiple Coronary Arteries using Low Osmolar Contrast */
"B211YZZ"  =  "1"  /* Fluoroscopy of Multiple Coronary Arteries using Other Contrast */
"B2120ZZ"  =  "1"  /* Fluoroscopy of Single Coronary Artery Bypass Graft using High Osmolar Contrast */
"B2121ZZ"  =  "1"  /* Fluoroscopy of Single Coronary Artery Bypass Graft using Low Osmolar Contrast */
"B212YZZ"  =  "1"  /* Fluoroscopy of Single Coronary Artery Bypass Graft using Other Contrast */
"B2130ZZ"  =  "1"  /* Fluoroscopy of Multiple Coronary Artery Bypass Grafts using High Osmolar Contrast */
"B2131ZZ"  =  "1"  /* Fluoroscopy of Multiple Coronary Artery Bypass Grafts using Low Osmolar Contrast */
"B213YZZ"  =  "1"  /* Fluoroscopy of Multiple Coronary Artery Bypass Grafts using Other Contrast */
"B2140ZZ"  =  "1"  /* Fluoroscopy of Right Heart using High Osmolar Contrast */
"B2141ZZ"  =  "1"  /* Fluoroscopy of Right Heart using Low Osmolar Contrast */
"B214YZZ"  =  "1"  /* Fluoroscopy of Right Heart using Other Contrast */
"B2150ZZ"  =  "1"  /* Fluoroscopy of Left Heart using High Osmolar Contrast */
"B2151ZZ"  =  "1"  /* Fluoroscopy of Left Heart using Low Osmolar Contrast */
"B215YZZ"  =  "1"  /* Fluoroscopy of Left Heart using Other Contrast */
"B2160ZZ"  =  "1"  /* Fluoroscopy of Right and Left Heart using High Osmolar Contrast */
"B2161ZZ"  =  "1"  /* Fluoroscopy of Right and Left Heart using Low Osmolar Contrast */
"B216YZZ"  =  "1"  /* Fluoroscopy of Right and Left Heart using Other Contrast */
"B2170ZZ"  =  "1"  /* Fluoroscopy of Right Internal Mammary Bypass Graft using High Osmolar Contrast */
"B2171ZZ"  =  "1"  /* Fluoroscopy of Right Internal Mammary Bypass Graft using Low Osmolar Contrast */
"B217YZZ"  =  "1"  /* Fluoroscopy of Right Internal Mammary Bypass Graft using Other Contrast */
"B2180ZZ"  =  "1"  /* Fluoroscopy of Left Internal Mammary Bypass Graft using High Osmolar Contrast */
"B2181ZZ"  =  "1"  /* Fluoroscopy of Left Internal Mammary Bypass Graft using Low Osmolar Contrast */
"B218YZZ"  =  "1"  /* Fluoroscopy of Left Internal Mammary Bypass Graft using Other Contrast */
"B21F0ZZ"  =  "1"  /* Fluoroscopy of Other Bypass Graft using High Osmolar Contrast */
"B21F1ZZ"  =  "1"  /* Fluoroscopy of Other Bypass Graft using Low Osmolar Contrast */
"B21FYZZ"  =  "1"  /* Fluoroscopy of Other Bypass Graft using Other Contrast */
"B3000ZZ"  =  "1"  /* Plain Radiography of Thoracic Aorta using High Osmolar Contrast */
"B3001ZZ"  =  "1"  /* Plain Radiography of Thoracic Aorta using Low Osmolar Contrast */
"B300YZZ"  =  "1"  /* Plain Radiography of Thoracic Aorta using Other Contrast */
"B3010ZZ"  =  "1"  /* Plain Radiography of Right Brachiocephalic-Subclavian Artery using High Osmolar Contrast */
"B3011ZZ"  =  "1"  /* Plain Radiography of Right Brachiocephalic-Subclavian Artery using Low Osmolar Contrast */
"B301YZZ"  =  "1"  /* Plain Radiography of Right Brachiocephalic-Subclavian Artery using Other Contrast */
"B3020ZZ"  =  "1"  /* Plain Radiography of Left Subclavian Artery using High Osmolar Contrast */
"B3021ZZ"  =  "1"  /* Plain Radiography of Left Subclavian Artery using Low Osmolar Contrast */
"B302YZZ"  =  "1"  /* Plain Radiography of Left Subclavian Artery using Other Contrast */
"B30L0ZZ"  =  "1"  /* Plain Radiography of Intercostal and Bronchial Arteries using High Osmolar Contrast */
"B30L1ZZ"  =  "1"  /* Plain Radiography of Intercostal and Bronchial Arteries using Low Osmolar Contrast */
"B30LYZZ"  =  "1"  /* Plain Radiography of Intercostal and Bronchial Arteries using Other Contrast */
"B30N0ZZ"  =  "1"  /* Plain Radiography of Other Upper Arteries using High Osmolar Contrast */
"B30N1ZZ"  =  "1"  /* Plain Radiography of Other Upper Arteries using Low Osmolar Contrast */
"B30NYZZ"  =  "1"  /* Plain Radiography of Other Upper Arteries using Other Contrast */
"B30P0ZZ"  =  "1"  /* Plain Radiography of Thoraco-Abdominal Aorta using High Osmolar Contrast */
"B30P1ZZ"  =  "1"  /* Plain Radiography of Thoraco-Abdominal Aorta using Low Osmolar Contrast */
"B30PYZZ"  =  "1"  /* Plain Radiography of Thoraco-Abdominal Aorta using Other Contrast */
"B30S0ZZ"  =  "1"  /* Plain Radiography of Right Pulmonary Artery using High Osmolar Contrast */
"B30S1ZZ"  =  "1"  /* Plain Radiography of Right Pulmonary Artery using Low Osmolar Contrast */
"B30SYZZ"  =  "1"  /* Plain Radiography of Right Pulmonary Artery using Other Contrast */
"B30T0ZZ"  =  "1"  /* Plain Radiography of Left Pulmonary Artery using High Osmolar Contrast */
"B30T1ZZ"  =  "1"  /* Plain Radiography of Left Pulmonary Artery using Low Osmolar Contrast */
"B30TYZZ"  =  "1"  /* Plain Radiography of Left Pulmonary Artery using Other Contrast */
"B3100ZZ"  =  "1"  /* Fluoroscopy of Thoracic Aorta using High Osmolar Contrast */
"B3101ZZ"  =  "1"  /* Fluoroscopy of Thoracic Aorta using Low Osmolar Contrast */
"B310YZZ"  =  "1"  /* Fluoroscopy of Thoracic Aorta using Other Contrast */
"B3110ZZ"  =  "1"  /* Fluoroscopy of Right Brachiocephalic-Subclavian Artery using High Osmolar Contrast */
"B3111ZZ"  =  "1"  /* Fluoroscopy of Right Brachiocephalic-Subclavian Artery using Low Osmolar Contrast */
"B311YZZ"  =  "1"  /* Fluoroscopy of Right Brachiocephalic-Subclavian Artery using Other Contrast */
"B3120ZZ"  =  "1"  /* Fluoroscopy of Left Subclavian Artery using High Osmolar Contrast */
"B3121ZZ"  =  "1"  /* Fluoroscopy of Left Subclavian Artery using Low Osmolar Contrast */
"B312YZZ"  =  "1"  /* Fluoroscopy of Left Subclavian Artery using Other Contrast */
"B31L0ZZ"  =  "1"  /* Fluoroscopy of Intercostal and Bronchial Arteries using High Osmolar Contrast */
"B31L1ZZ"  =  "1"  /* Fluoroscopy of Intercostal and Bronchial Arteries using Low Osmolar Contrast */
"B31LYZZ"  =  "1"  /* Fluoroscopy of Intercostal and Bronchial Arteries using Other Contrast */
"B31N0ZZ"  =  "1"  /* Fluoroscopy of Other Upper Arteries using High Osmolar Contrast */
"B31N1ZZ"  =  "1"  /* Fluoroscopy of Other Upper Arteries using Low Osmolar Contrast */
"B31NYZZ"  =  "1"  /* Fluoroscopy of Other Upper Arteries using Other Contrast */
"B31P0ZZ"  =  "1"  /* Fluoroscopy of Thoraco-Abdominal Aorta using High Osmolar Contrast */
"B31P1ZZ"  =  "1"  /* Fluoroscopy of Thoraco-Abdominal Aorta using Low Osmolar Contrast */
"B31PYZZ"  =  "1"  /* Fluoroscopy of Thoraco-Abdominal Aorta using Other Contrast */
"B31S0ZZ"  =  "1"  /* Fluoroscopy of Right Pulmonary Artery using High Osmolar Contrast */
"B31S1ZZ"  =  "1"  /* Fluoroscopy of Right Pulmonary Artery using Low Osmolar Contrast */
"B31SYZZ"  =  "1"  /* Fluoroscopy of Right Pulmonary Artery using Other Contrast */
"B31T0ZZ"  =  "1"  /* Fluoroscopy of Left Pulmonary Artery using High Osmolar Contrast */
"B31T1ZZ"  =  "1"  /* Fluoroscopy of Left Pulmonary Artery using Low Osmolar Contrast */
"B31TYZZ"  =  "1"  /* Fluoroscopy of Left Pulmonary Artery using Other Contrast */
"B4000ZZ"  =  "1"  /* Plain Radiography of Abdominal Aorta using High Osmolar Contrast */
"B4001ZZ"  =  "1"  /* Plain Radiography of Abdominal Aorta using Low Osmolar Contrast */
"B400YZZ"  =  "1"  /* Plain Radiography of Abdominal Aorta using Other Contrast */
"B40D0ZZ"  =  "1"  /* Plain Radiography of Aorta and Bilateral Lower Extremity Arteries using High Osmolar Contrast */
"B40D1ZZ"  =  "1"  /* Plain Radiography of Aorta and Bilateral Lower Extremity Arteries using Low Osmolar Contrast */
"B40DYZZ"  =  "1"  /* Plain Radiography of Aorta and Bilateral Lower Extremity Arteries using Other Contrast */
"B4100ZZ"  =  "1"  /* Fluoroscopy of Abdominal Aorta using High Osmolar Contrast */
"B4101ZZ"  =  "1"  /* Fluoroscopy of Abdominal Aorta using Low Osmolar Contrast */
"B410YZZ"  =  "1"  /* Fluoroscopy of Abdominal Aorta using Other Contrast */
"B41D0ZZ"  =  "1"  /* Fluoroscopy of Aorta and Bilateral Lower Extremity Arteries using High Osmolar Contrast */
"B41D1ZZ"  =  "1"  /* Fluoroscopy of Aorta and Bilateral Lower Extremity Arteries using Low Osmolar Contrast */
"B41DYZZ"  =  "1"  /* Fluoroscopy of Aorta and Bilateral Lower Extremity Arteries using Other Contrast */
"B5080ZZ"  =  "1"  /* Plain Radiography of Superior Vena Cava using High Osmolar Contrast */
"B5081ZZ"  =  "1"  /* Plain Radiography of Superior Vena Cava using Low Osmolar Contrast */
"B508YZZ"  =  "1"  /* Plain Radiography of Superior Vena Cava using Other Contrast */
"B5090ZZ"  =  "1"  /* Plain Radiography of Inferior Vena Cava using High Osmolar Contrast */
"B5091ZZ"  =  "1"  /* Plain Radiography of Inferior Vena Cava using Low Osmolar Contrast */
"B509YZZ"  =  "1"  /* Plain Radiography of Inferior Vena Cava using Other Contrast */
"B5180ZA"  =  "1"  /* Fluoroscopy of Superior Vena Cava using High Osmolar Contrast, Guidance */
"B5180ZZ"  =  "1"  /* Fluoroscopy of Superior Vena Cava using High Osmolar Contrast */
"B5181ZA"  =  "1"  /* Fluoroscopy of Superior Vena Cava using Low Osmolar Contrast, Guidance */
"B5181ZZ"  =  "1"  /* Fluoroscopy of Superior Vena Cava using Low Osmolar Contrast */
"B518YZA"  =  "1"  /* Fluoroscopy of Superior Vena Cava using Other Contrast, Guidance */
"B518YZZ"  =  "1"  /* Fluoroscopy of Superior Vena Cava using Other Contrast */
"B518ZZA"  =  "1"  /* Fluoroscopy of Superior Vena Cava, Guidance */
"B5190ZA"  =  "1"  /* Fluoroscopy of Inferior Vena Cava using High Osmolar Contrast, Guidance */
"B5190ZZ"  =  "1"  /* Fluoroscopy of Inferior Vena Cava using High Osmolar Contrast */
"B5191ZA"  =  "1"  /* Fluoroscopy of Inferior Vena Cava using Low Osmolar Contrast, Guidance */
"B5191ZZ"  =  "1"  /* Fluoroscopy of Inferior Vena Cava using Low Osmolar Contrast */
"B519YZA"  =  "1"  /* Fluoroscopy of Inferior Vena Cava using Other Contrast, Guidance */
"B519YZZ"  =  "1"  /* Fluoroscopy of Inferior Vena Cava using Other Contrast */
"B519ZZA"  =  "1"  /* Fluoroscopy of Inferior Vena Cava, Guidance */
OTHER  = "0" ;

 /* PHS Exclusion #2: */
 /* ICD-10-CM Procedure Codes: */

VALUE $PRPED4P
"02U54JZ"  =  "1"  /* Supplement Atrial Septum with Synthetic Substitute, Percutaneous Endoscopic Approach */
OTHER  = "0" ;

 /* PHS Exclusion #3: */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $PRPED4D
"P0700"  =  "1"  /* Extremely low birth weight newborn, unspecified weight */
"P0701"  =  "1"  /* Extremely low birth weight newborn, less than 500 grams */
"P0702"  =  "1"  /* Extremely low birth weight newborn, 500-749 grams */
"P0703"  =  "1"  /* Extremely low birth weight newborn, 750-999 grams */
"P0710"  =  "1"  /* Other low birth weight newborn, unspecified weight */
"P0714"  =  "1"  /* Other low birth weight newborn, 1000-1249 grams */
"P0715"  =  "1"  /* Other low birth weight newborn, 1250-1499 grams */
"P0716"  =  "1"  /* Other low birth weight newborn, 1500-1749 grams */
"P0717"  =  "1"  /* Other low birth weight newborn, 1750-1999 grams */
"P0718"  =  "1"  /* Other low birth weight newborn, 2000-2499 grams */
"P0720"  =  "1"  /* Extreme immaturity of newborn, unspecified weeks of gestation */
"P0730"  =  "1"  /* Preterm newborn, unspecified weeks of gestation */
OTHER  = "0" ; 

 /* ICD-10-CM Procedure Codes: */

VALUE $PRPED7P
"02YA0Z0"  =  "1"  /* Transplantation of Heart, Allogeneic, Open Approach */
"02YA0Z1"  =  "1"  /* Transplantation of Heart, Syngeneic, Open Approach */
OTHER  = "0" ;


 /* ASD or VSD */

 /* ICD-10-CM Diagnosis Codes: */

VALUE $PRPED5D
"Q210"  =  "1"  /* Ventricular septal defect */
"Q211"  =  "1"  /* Atrial septal defect */
OTHER   = "0" ;


 /* ------------------------------------- */
 /* PERI-OPERATIVE HEMORRHAGE OR HEMATOMA */
 /*   PROVIDER INDICATOR / PDI # 8        */
 /* ------------------------------------- */

 /* POST-OPERATIVE HEMORRHAGE OR HEMATOMA */
 /* ICD-10-CM DIAGNOSIS CODE: */

/* VALUE $POHMAID Due to the granularity of ICD-10-CM code set, this concept was merged with other concepts to capture the hemorrhage and hematoma POHMRID to POHMRI2D */
VALUE $POHMRI2D
"D7821"  =  "1"  /* Postprocedural hemorrhage and hematoma of the spleen following a procedure on the spleen */
"D7822"  =  "1"  /* Postprocedural hemorrhage and hematoma of the spleen following other procedure */
"E89810"  =  "1"  /* Postprocedural hemorrhage and hematoma of an endocrine system organ or structure following an endocrine system procedure */
"E89811"  =  "1"  /* Postprocedural hemorrhage and hematoma of an endocrine system organ or structure following other procedure */
"G9751"  =  "1"  /* Postprocedural hemorrhage and hematoma of a nervous system organ or structure following a nervous system procedure */
"G9752"  =  "1"  /* Postprocedural hemorrhage and hematoma of a nervous system organ or structure following other procedure */
"H59311"  =  "1"  /* Postprocedural hemorrhage and hematoma of right eye and adnexa following an ophthalmic procedure */
"H59312"  =  "1"  /* Postprocedural hemorrhage and hematoma of left eye and adnexa following an ophthalmic procedure */
"H59313"  =  "1"  /* Postprocedural hemorrhage and hematoma of eye and adnexa following an ophthalmic procedure, bilateral */
"H59319"  =  "1"  /* Postprocedural hemorrhage and hematoma of unspecified eye and adnexa following an ophthalmic procedure */
"H59321"  =  "1"  /* Postprocedural hemorrhage and hematoma of right eye and adnexa following other procedure */
"H59322"  =  "1"  /* Postprocedural hemorrhage and hematoma of left eye and adnexa following other procedure */
"H59323"  =  "1"  /* Postprocedural hemorrhage and hematoma of eye and adnexa following other procedure, bilateral */
"H59329"  =  "1"  /* Postprocedural hemorrhage and hematoma of unspecified eye and adnexa following other procedure */
"H9541"  =  "1"  /* Postprocedural hemorrhage and hematoma of ear and mastoid process following a procedure on the ear and mastoid process */
"H9542"  =  "1"  /* Postprocedural hemorrhage and hematoma of ear and mastoid process following other procedure */
"I97610"  =  "1"  /* Postprocedural hemorrhage and hematoma of a circulatory system organ or structure following a cardiac catheterization */
"I97611"  =  "1"  /* Postprocedural hemorrhage and hematoma of a circulatory system organ or structure following cardiac bypass */
"I97618"  =  "1"  /* Postprocedural hemorrhage and hematoma of a circulatory system organ or structure following other circulatory system procedure */
"I9762"  =  "1"  /* Postprocedural hemorrhage and hematoma of a circulatory system organ or structure following other procedure */
"J95830"  =  "1"  /* Postprocedural hemorrhage and hematoma of a respiratory system organ or structure following a respiratory system procedure */
"J95831"  =  "1"  /* Postprocedural hemorrhage and hematoma of a respiratory system organ or structure following other procedure */
"K91840"  =  "1"  /* Postprocedural hemorrhage and hematoma of a digestive system organ or structure following a digestive system procedure */
"K91841"  =  "1"  /* Postprocedural hemorrhage and hematoma of a digestive system organ or structure following other procedure */
"L7621"  =  "1"  /* Postprocedural hemorrhage and hematoma of skin and subcutaneous tissue following a dermatologic procedure */
"L7622"  =  "1"  /* Postprocedural hemorrhage and hematoma of skin and subcutaneous tissue following other procedure */
"M96830"  =  "1"  /* Postprocedural hemorrhage and hematoma of a musculoskeletal structure following a musculoskeletal system procedure */
"M96831"  =  "1"  /* Postprocedural hemorrhage and hematoma of a musculoskeletal structure following other procedure */
"N99820"  =  "1"  /* Postprocedural hemorrhage and hematoma of a genitourinary system organ or structure following a genitourinary system procedure */
"N99821"  =  "1"  /* Postprocedural hemorrhage and hematoma of a genitourinary system organ or structure following other procedure */
OTHER  = "0" ;

 /* CONTROL OF POST-OPERATIVE HEMATOMA */
 /* CONTROL OF POST-OPERATIVE HEMORRHAGE */
 /* Miscellaneous Hemorrhage or Hematoma-related Procedure codes: */
 /* PDI-08 **/
 /* ICD-10-CM PROCEDURE CODES: */
 
/* $HEMATIP_$HEMORIP_$HEMOTHP combined per UCD SAS name used $HEMOTH2P */
VALUE $HEMOTH2P
"009P3ZZ"  =  "1"  /* Drainage of Glossopharyngeal Nerve, Percutaneous Approach */
"009P40Z"  =  "1"  /* Drainage of Glossopharyngeal Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"009P4ZZ"  =  "1"  /* Drainage of Glossopharyngeal Nerve, Percutaneous Endoscopic Approach */
"009Q00Z"  =  "1"  /* Drainage of Vagus Nerve with Drainage Device, Open Approach */
"009Q0ZZ"  =  "1"  /* Drainage of Vagus Nerve, Open Approach */
"009Q30Z"  =  "1"  /* Drainage of Vagus Nerve with Drainage Device, Percutaneous Approach */
"009Q3ZZ"  =  "1"  /* Drainage of Vagus Nerve, Percutaneous Approach */
"009Q40Z"  =  "1"  /* Drainage of Vagus Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"009Q4ZZ"  =  "1"  /* Drainage of Vagus Nerve, Percutaneous Endoscopic Approach */
"009R00Z"  =  "1"  /* Drainage of Accessory Nerve with Drainage Device, Open Approach */
"009R0ZZ"  =  "1"  /* Drainage of Accessory Nerve, Open Approach */
"009R30Z"  =  "1"  /* Drainage of Accessory Nerve with Drainage Device, Percutaneous Approach */
"009R3ZZ"  =  "1"  /* Drainage of Accessory Nerve, Percutaneous Approach */
"009R40Z"  =  "1"  /* Drainage of Accessory Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"009R4ZZ"  =  "1"  /* Drainage of Accessory Nerve, Percutaneous Endoscopic Approach */
"009S00Z"  =  "1"  /* Drainage of Hypoglossal Nerve with Drainage Device, Open Approach */
"009S0ZZ"  =  "1"  /* Drainage of Hypoglossal Nerve, Open Approach */
"009S30Z"  =  "1"  /* Drainage of Hypoglossal Nerve with Drainage Device, Percutaneous Approach */
"009S3ZZ"  =  "1"  /* Drainage of Hypoglossal Nerve, Percutaneous Approach */
"009S40Z"  =  "1"  /* Drainage of Hypoglossal Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"009S4ZZ"  =  "1"  /* Drainage of Hypoglossal Nerve, Percutaneous Endoscopic Approach */
"009T00Z"  =  "1"  /* Drainage of Spinal Meninges with Drainage Device, Open Approach */
"009T0ZZ"  =  "1"  /* Drainage of Spinal Meninges, Open Approach */
"009T30Z"  =  "1"  /* Drainage of Spinal Meninges with Drainage Device, Percutaneous Approach */
"009T3ZZ"  =  "1"  /* Drainage of Spinal Meninges, Percutaneous Approach */
"009T40Z"  =  "1"  /* Drainage of Spinal Meninges with Drainage Device, Percutaneous Endoscopic Approach */
"009T4ZZ"  =  "1"  /* Drainage of Spinal Meninges, Percutaneous Endoscopic Approach */
"009U00Z"  =  "1"  /* Drainage of Spinal Canal with Drainage Device, Open Approach */
"009U0ZZ"  =  "1"  /* Drainage of Spinal Canal, Open Approach */
"009W00Z"  =  "1"  /* Drainage of Cervical Spinal Cord with Drainage Device, Open Approach */
"009W0ZZ"  =  "1"  /* Drainage of Cervical Spinal Cord, Open Approach */
"009W30Z"  =  "1"  /* Drainage of Cervical Spinal Cord with Drainage Device, Percutaneous Approach */
"009W3ZZ"  =  "1"  /* Drainage of Cervical Spinal Cord, Percutaneous Approach */
"009W40Z"  =  "1"  /* Drainage of Cervical Spinal Cord with Drainage Device, Percutaneous Endoscopic Approach */
"009W4ZZ"  =  "1"  /* Drainage of Cervical Spinal Cord, Percutaneous Endoscopic Approach */
"009X00Z"  =  "1"  /* Drainage of Thoracic Spinal Cord with Drainage Device, Open Approach */
"009X0ZZ"  =  "1"  /* Drainage of Thoracic Spinal Cord, Open Approach */
"009X30Z"  =  "1"  /* Drainage of Thoracic Spinal Cord with Drainage Device, Percutaneous Approach */
"009X3ZZ"  =  "1"  /* Drainage of Thoracic Spinal Cord, Percutaneous Approach */
"009X40Z"  =  "1"  /* Drainage of Thoracic Spinal Cord with Drainage Device, Percutaneous Endoscopic Approach */
"009X4ZZ"  =  "1"  /* Drainage of Thoracic Spinal Cord, Percutaneous Endoscopic Approach */
"009Y00Z"  =  "1"  /* Drainage of Lumbar Spinal Cord with Drainage Device, Open Approach */
"009Y0ZZ"  =  "1"  /* Drainage of Lumbar Spinal Cord, Open Approach */
"009Y30Z"  =  "1"  /* Drainage of Lumbar Spinal Cord with Drainage Device, Percutaneous Approach */
"009Y3ZZ"  =  "1"  /* Drainage of Lumbar Spinal Cord, Percutaneous Approach */
"009Y40Z"  =  "1"  /* Drainage of Lumbar Spinal Cord with Drainage Device, Percutaneous Endoscopic Approach */
"009Y4ZZ"  =  "1"  /* Drainage of Lumbar Spinal Cord, Percutaneous Endoscopic Approach */
"00BN0ZZ"  =  "1"  /* Excision of Acoustic Nerve, Open Approach */
"00C00ZZ"  =  "1"  /* Extirpation of Matter from Brain, Open Approach */
"00C03ZZ"  =  "1"  /* Extirpation of Matter from Brain, Percutaneous Approach */
"00C04ZZ"  =  "1"  /* Extirpation of Matter from Brain, Percutaneous Endoscopic Approach */
"00C10ZZ"  =  "1"  /* Extirpation of Matter from Cerebral Meninges, Open Approach */
"00C13ZZ"  =  "1"  /* Extirpation of Matter from Cerebral Meninges, Percutaneous Approach */
"00C14ZZ"  =  "1"  /* Extirpation of Matter from Cerebral Meninges, Percutaneous Endoscopic Approach */
"00C20ZZ"  =  "1"  /* Extirpation of Matter from Dura Mater, Open Approach */
"00C23ZZ"  =  "1"  /* Extirpation of Matter from Dura Mater, Percutaneous Approach */
"00C24ZZ"  =  "1"  /* Extirpation of Matter from Dura Mater, Percutaneous Endoscopic Approach */
"00C30ZZ"  =  "1"  /* Extirpation of Matter from Epidural Space, Open Approach */
"00C33ZZ"  =  "1"  /* Extirpation of Matter from Epidural Space, Percutaneous Approach */
"00C34ZZ"  =  "1"  /* Extirpation of Matter from Epidural Space, Percutaneous Endoscopic Approach */
"00C40ZZ"  =  "1"  /* Extirpation of Matter from Subdural Space, Open Approach */
"00C43ZZ"  =  "1"  /* Extirpation of Matter from Subdural Space, Percutaneous Approach */
"00C44ZZ"  =  "1"  /* Extirpation of Matter from Subdural Space, Percutaneous Endoscopic Approach */
"00C50ZZ"  =  "1"  /* Extirpation of Matter from Subarachnoid Space, Open Approach */
"00C53ZZ"  =  "1"  /* Extirpation of Matter from Subarachnoid Space, Percutaneous Approach */
"00C54ZZ"  =  "1"  /* Extirpation of Matter from Subarachnoid Space, Percutaneous Endoscopic Approach */
"00C60ZZ"  =  "1"  /* Extirpation of Matter from Cerebral Ventricle, Open Approach */
"00C63ZZ"  =  "1"  /* Extirpation of Matter from Cerebral Ventricle, Percutaneous Approach */
"00C64ZZ"  =  "1"  /* Extirpation of Matter from Cerebral Ventricle, Percutaneous Endoscopic Approach */
"00C70ZZ"  =  "1"  /* Extirpation of Matter from Cerebral Hemisphere, Open Approach */
"00C73ZZ"  =  "1"  /* Extirpation of Matter from Cerebral Hemisphere, Percutaneous Approach */
"00C74ZZ"  =  "1"  /* Extirpation of Matter from Cerebral Hemisphere, Percutaneous Endoscopic Approach */
"00CB0ZZ"  =  "1"  /* Extirpation of Matter from Pons, Open Approach */
"00CB3ZZ"  =  "1"  /* Extirpation of Matter from Pons, Percutaneous Approach */
"00CB4ZZ"  =  "1"  /* Extirpation of Matter from Pons, Percutaneous Endoscopic Approach */
"00CC0ZZ"  =  "1"  /* Extirpation of Matter from Cerebellum, Open Approach */
"00CC3ZZ"  =  "1"  /* Extirpation of Matter from Cerebellum, Percutaneous Approach */
"00CC4ZZ"  =  "1"  /* Extirpation of Matter from Cerebellum, Percutaneous Endoscopic Approach */
"00CD0ZZ"  =  "1"  /* Extirpation of Matter from Medulla Oblongata, Open Approach */
"00CD3ZZ"  =  "1"  /* Extirpation of Matter from Medulla Oblongata, Percutaneous Approach */
"00CD4ZZ"  =  "1"  /* Extirpation of Matter from Medulla Oblongata, Percutaneous Endoscopic Approach */
"00CF0ZZ"  =  "1"  /* Extirpation of Matter from Olfactory Nerve, Open Approach */
"00CF3ZZ"  =  "1"  /* Extirpation of Matter from Olfactory Nerve, Percutaneous Approach */
"00CF4ZZ"  =  "1"  /* Extirpation of Matter from Olfactory Nerve, Percutaneous Endoscopic Approach */
"00CG0ZZ"  =  "1"  /* Extirpation of Matter from Optic Nerve, Open Approach */
"00CG3ZZ"  =  "1"  /* Extirpation of Matter from Optic Nerve, Percutaneous Approach */
"00CG4ZZ"  =  "1"  /* Extirpation of Matter from Optic Nerve, Percutaneous Endoscopic Approach */
"00CH0ZZ"  =  "1"  /* Extirpation of Matter from Oculomotor Nerve, Open Approach */
"00CH3ZZ"  =  "1"  /* Extirpation of Matter from Oculomotor Nerve, Percutaneous Approach */
"00CH4ZZ"  =  "1"  /* Extirpation of Matter from Oculomotor Nerve, Percutaneous Endoscopic Approach */
"00CJ0ZZ"  =  "1"  /* Extirpation of Matter from Trochlear Nerve, Open Approach */
"00CJ3ZZ"  =  "1"  /* Extirpation of Matter from Trochlear Nerve, Percutaneous Approach */
"00CJ4ZZ"  =  "1"  /* Extirpation of Matter from Trochlear Nerve, Percutaneous Endoscopic Approach */
"00CK0ZZ"  =  "1"  /* Extirpation of Matter from Trigeminal Nerve, Open Approach */
"00CK3ZZ"  =  "1"  /* Extirpation of Matter from Trigeminal Nerve, Percutaneous Approach */
"00CK4ZZ"  =  "1"  /* Extirpation of Matter from Trigeminal Nerve, Percutaneous Endoscopic Approach */
"00CL0ZZ"  =  "1"  /* Extirpation of Matter from Abducens Nerve, Open Approach */
"00CL3ZZ"  =  "1"  /* Extirpation of Matter from Abducens Nerve, Percutaneous Approach */
"00CL4ZZ"  =  "1"  /* Extirpation of Matter from Abducens Nerve, Percutaneous Endoscopic Approach */
"00CM0ZZ"  =  "1"  /* Extirpation of Matter from Facial Nerve, Open Approach */
"00CM3ZZ"  =  "1"  /* Extirpation of Matter from Facial Nerve, Percutaneous Approach */
"00CM4ZZ"  =  "1"  /* Extirpation of Matter from Facial Nerve, Percutaneous Endoscopic Approach */
"00CN0ZZ"  =  "1"  /* Extirpation of Matter from Acoustic Nerve, Open Approach */
"00CN3ZZ"  =  "1"  /* Extirpation of Matter from Acoustic Nerve, Percutaneous Approach */
"00CN4ZZ"  =  "1"  /* Extirpation of Matter from Acoustic Nerve, Percutaneous Endoscopic Approach */
"00CP0ZZ"  =  "1"  /* Extirpation of Matter from Glossopharyngeal Nerve, Open Approach */
"00CP3ZZ"  =  "1"  /* Extirpation of Matter from Glossopharyngeal Nerve, Percutaneous Approach */
"00CP4ZZ"  =  "1"  /* Extirpation of Matter from Glossopharyngeal Nerve, Percutaneous Endoscopic Approach */
"00CQ0ZZ"  =  "1"  /* Extirpation of Matter from Vagus Nerve, Open Approach */
"00CQ3ZZ"  =  "1"  /* Extirpation of Matter from Vagus Nerve, Percutaneous Approach */
"00CQ4ZZ"  =  "1"  /* Extirpation of Matter from Vagus Nerve, Percutaneous Endoscopic Approach */
"00CR0ZZ"  =  "1"  /* Extirpation of Matter from Accessory Nerve, Open Approach */
"00CR3ZZ"  =  "1"  /* Extirpation of Matter from Accessory Nerve, Percutaneous Approach */
"00CR4ZZ"  =  "1"  /* Extirpation of Matter from Accessory Nerve, Percutaneous Endoscopic Approach */
"00CS0ZZ"  =  "1"  /* Extirpation of Matter from Hypoglossal Nerve, Open Approach */
"00CS3ZZ"  =  "1"  /* Extirpation of Matter from Hypoglossal Nerve, Percutaneous Approach */
"00CS4ZZ"  =  "1"  /* Extirpation of Matter from Hypoglossal Nerve, Percutaneous Endoscopic Approach */
"00H002Z"  =  "1"  /* Insertion of Monitoring Device into Brain, Open Approach */
"00H003Z"  =  "1"  /* Insertion of Infusion Device into Brain, Open Approach */
"00H032Z"  =  "1"  /* Insertion of Monitoring Device into Brain, Percutaneous Approach */
"00H033Z"  =  "1"  /* Insertion of Infusion Device into Brain, Percutaneous Approach */
"00H042Z"  =  "1"  /* Insertion of Monitoring Device into Brain, Percutaneous Endoscopic Approach */
"00H043Z"  =  "1"  /* Insertion of Infusion Device into Brain, Percutaneous Endoscopic Approach */
"00H602Z"  =  "1"  /* Insertion of Monitoring Device into Cerebral Ventricle, Open Approach */
"00H603Z"  =  "1"  /* Insertion of Infusion Device into Cerebral Ventricle, Open Approach */
"00H632Z"  =  "1"  /* Insertion of Monitoring Device into Cerebral Ventricle, Percutaneous Approach */
"00H633Z"  =  "1"  /* Insertion of Infusion Device into Cerebral Ventricle, Percutaneous Approach */
"00H642Z"  =  "1"  /* Insertion of Monitoring Device into Cerebral Ventricle, Percutaneous Endoscopic Approach */
"00H643Z"  =  "1"  /* Insertion of Infusion Device into Cerebral Ventricle, Percutaneous Endoscopic Approach */
"00HE02Z"  =  "1"  /* Insertion of Monitoring Device into Cranial Nerve, Open Approach */
"00HE03Z"  =  "1"  /* Insertion of Infusion Device into Cranial Nerve, Open Approach */
"00HE32Z"  =  "1"  /* Insertion of Monitoring Device into Cranial Nerve, Percutaneous Approach */
"00HE33Z"  =  "1"  /* Insertion of Infusion Device into Cranial Nerve, Percutaneous Approach */
"00HE42Z"  =  "1"  /* Insertion of Monitoring Device into Cranial Nerve, Percutaneous Endoscopic Approach */
"00HE43Z"  =  "1"  /* Insertion of Infusion Device into Cranial Nerve, Percutaneous Endoscopic Approach */
"00J00ZZ"  =  "1"  /* Inspection of Brain, Open Approach */
"00JU0ZZ"  =  "1"  /* Inspection of Spinal Canal, Open Approach */
"00JV0ZZ"  =  "1"  /* Inspection of Spinal Cord, Open Approach */
"00NW0ZZ"  =  "1"  /* Release Cervical Spinal Cord, Open Approach */
"00NW3ZZ"  =  "1"  /* Release Cervical Spinal Cord, Percutaneous Approach */
"00NW4ZZ"  =  "1"  /* Release Cervical Spinal Cord, Percutaneous Endoscopic Approach */
"00NX0ZZ"  =  "1"  /* Release Thoracic Spinal Cord, Open Approach */
"00NX3ZZ"  =  "1"  /* Release Thoracic Spinal Cord, Percutaneous Approach */
"00NX4ZZ"  =  "1"  /* Release Thoracic Spinal Cord, Percutaneous Endoscopic Approach */
"00NY0ZZ"  =  "1"  /* Release Lumbar Spinal Cord, Open Approach */
"00NY3ZZ"  =  "1"  /* Release Lumbar Spinal Cord, Percutaneous Approach */
"00NY4ZZ"  =  "1"  /* Release Lumbar Spinal Cord, Percutaneous Endoscopic Approach */
"00P000Z"  =  "1"  /* Removal of Drainage Device from Brain, Open Approach */
"00P002Z"  =  "1"  /* Removal of Monitoring Device from Brain, Open Approach */
"00P003Z"  =  "1"  /* Removal of Infusion Device from Brain, Open Approach */
"00P007Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Brain, Open Approach */
"00P00JZ"  =  "1"  /* Removal of Synthetic Substitute from Brain, Open Approach */
"00P00KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Brain, Open Approach */
"00P030Z"  =  "1"  /* Removal of Drainage Device from Brain, Percutaneous Approach */
"00P032Z"  =  "1"  /* Removal of Monitoring Device from Brain, Percutaneous Approach */
"00P033Z"  =  "1"  /* Removal of Infusion Device from Brain, Percutaneous Approach */
"00P037Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Brain, Percutaneous Approach */
"00P03JZ"  =  "1"  /* Removal of Synthetic Substitute from Brain, Percutaneous Approach */
"00P03KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Brain, Percutaneous Approach */
"00P040Z"  =  "1"  /* Removal of Drainage Device from Brain, Percutaneous Endoscopic Approach */
"00P042Z"  =  "1"  /* Removal of Monitoring Device from Brain, Percutaneous Endoscopic Approach */
"00P043Z"  =  "1"  /* Removal of Infusion Device from Brain, Percutaneous Endoscopic Approach */
"00P047Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Brain, Percutaneous Endoscopic Approach */
"00P04JZ"  =  "1"  /* Removal of Synthetic Substitute from Brain, Percutaneous Endoscopic Approach */
"00P04KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Brain, Percutaneous Endoscopic Approach */
"00P600Z"  =  "1"  /* Removal of Drainage Device from Cerebral Ventricle, Open Approach */
"00P602Z"  =  "1"  /* Removal of Monitoring Device from Cerebral Ventricle, Open Approach */
"00P603Z"  =  "1"  /* Removal of Infusion Device from Cerebral Ventricle, Open Approach */
"00P630Z"  =  "1"  /* Removal of Drainage Device from Cerebral Ventricle, Percutaneous Approach */
"00P632Z"  =  "1"  /* Removal of Monitoring Device from Cerebral Ventricle, Percutaneous Approach */
"00P633Z"  =  "1"  /* Removal of Infusion Device from Cerebral Ventricle, Percutaneous Approach */
"00P640Z"  =  "1"  /* Removal of Drainage Device from Cerebral Ventricle, Percutaneous Endoscopic Approach */
"00P642Z"  =  "1"  /* Removal of Monitoring Device from Cerebral Ventricle, Percutaneous Endoscopic Approach */
"00P643Z"  =  "1"  /* Removal of Infusion Device from Cerebral Ventricle, Percutaneous Endoscopic Approach */
"00P6X2Z"  =  "1"  /* Removal of Monitoring Device from Cerebral Ventricle, External Approach */
"00PE00Z"  =  "1"  /* Removal of Drainage Device from Cranial Nerve, Open Approach */
"00PE02Z"  =  "1"  /* Removal of Monitoring Device from Cranial Nerve, Open Approach */
"00PE03Z"  =  "1"  /* Removal of Infusion Device from Cranial Nerve, Open Approach */
"00PE07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Cranial Nerve, Open Approach */
"00PE30Z"  =  "1"  /* Removal of Drainage Device from Cranial Nerve, Percutaneous Approach */
"00PE32Z"  =  "1"  /* Removal of Monitoring Device from Cranial Nerve, Percutaneous Approach */
"00PE33Z"  =  "1"  /* Removal of Infusion Device from Cranial Nerve, Percutaneous Approach */
"00PE37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Cranial Nerve, Percutaneous Approach */
"00PE40Z"  =  "1"  /* Removal of Drainage Device from Cranial Nerve, Percutaneous Endoscopic Approach */
"00PE42Z"  =  "1"  /* Removal of Monitoring Device from Cranial Nerve, Percutaneous Endoscopic Approach */
"00PE43Z"  =  "1"  /* Removal of Infusion Device from Cranial Nerve, Percutaneous Endoscopic Approach */
"00PE47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Cranial Nerve, Percutaneous Endoscopic Approach */
"00W000Z"  =  "1"  /* Revision of Drainage Device in Brain, Open Approach */
"00W002Z"  =  "1"  /* Revision of Monitoring Device in Brain, Open Approach */
"00W003Z"  =  "1"  /* Revision of Infusion Device in Brain, Open Approach */
"00W007Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Brain, Open Approach */
"00W00JZ"  =  "1"  /* Revision of Synthetic Substitute in Brain, Open Approach */
"00W00KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Brain, Open Approach */
"00W00MZ"  =  "1"  /* Revision of Neurostimulator Lead in Brain, Open Approach */
"00W030Z"  =  "1"  /* Revision of Drainage Device in Brain, Percutaneous Approach */
"00W032Z"  =  "1"  /* Revision of Monitoring Device in Brain, Percutaneous Approach */
"00W033Z"  =  "1"  /* Revision of Infusion Device in Brain, Percutaneous Approach */
"00W037Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Brain, Percutaneous Approach */
"00W03JZ"  =  "1"  /* Revision of Synthetic Substitute in Brain, Percutaneous Approach */
"00W03KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Brain, Percutaneous Approach */
"00W03MZ"  =  "1"  /* Revision of Neurostimulator Lead in Brain, Percutaneous Approach */
"00W040Z"  =  "1"  /* Revision of Drainage Device in Brain, Percutaneous Endoscopic Approach */
"00W042Z"  =  "1"  /* Revision of Monitoring Device in Brain, Percutaneous Endoscopic Approach */
"00W043Z"  =  "1"  /* Revision of Infusion Device in Brain, Percutaneous Endoscopic Approach */
"00W047Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Brain, Percutaneous Endoscopic Approach */
"00W04JZ"  =  "1"  /* Revision of Synthetic Substitute in Brain, Percutaneous Endoscopic Approach */
"00W04KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Brain, Percutaneous Endoscopic Approach */
"00W04MZ"  =  "1"  /* Revision of Neurostimulator Lead in Brain, Percutaneous Endoscopic Approach */
"00W600Z"  =  "1"  /* Revision of Drainage Device in Cerebral Ventricle, Open Approach */
"00W602Z"  =  "1"  /* Revision of Monitoring Device in Cerebral Ventricle, Open Approach */
"00W603Z"  =  "1"  /* Revision of Infusion Device in Cerebral Ventricle, Open Approach */
"00W60MZ"  =  "1"  /* Revision of Neurostimulator Lead in Cerebral Ventricle, Open Approach */
"00W630Z"  =  "1"  /* Revision of Drainage Device in Cerebral Ventricle, Percutaneous Approach */
"00W632Z"  =  "1"  /* Revision of Monitoring Device in Cerebral Ventricle, Percutaneous Approach */
"00W633Z"  =  "1"  /* Revision of Infusion Device in Cerebral Ventricle, Percutaneous Approach */
"00W63MZ"  =  "1"  /* Revision of Neurostimulator Lead in Cerebral Ventricle, Percutaneous Approach */
"00W640Z"  =  "1"  /* Revision of Drainage Device in Cerebral Ventricle, Percutaneous Endoscopic Approach */
"00W642Z"  =  "1"  /* Revision of Monitoring Device in Cerebral Ventricle, Percutaneous Endoscopic Approach */
"00W643Z"  =  "1"  /* Revision of Infusion Device in Cerebral Ventricle, Percutaneous Endoscopic Approach */
"00W64MZ"  =  "1"  /* Revision of Neurostimulator Lead in Cerebral Ventricle, Percutaneous Endoscopic Approach */
"00WE00Z"  =  "1"  /* Revision of Drainage Device in Cranial Nerve, Open Approach */
"00WE02Z"  =  "1"  /* Revision of Monitoring Device in Cranial Nerve, Open Approach */
"00WE03Z"  =  "1"  /* Revision of Infusion Device in Cranial Nerve, Open Approach */
"00WE07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Cranial Nerve, Open Approach */
"00WE0MZ"  =  "1"  /* Revision of Neurostimulator Lead in Cranial Nerve, Open Approach */
"00WE30Z"  =  "1"  /* Revision of Drainage Device in Cranial Nerve, Percutaneous Approach */
"00WE32Z"  =  "1"  /* Revision of Monitoring Device in Cranial Nerve, Percutaneous Approach */
"00WE33Z"  =  "1"  /* Revision of Infusion Device in Cranial Nerve, Percutaneous Approach */
"00WE37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Cranial Nerve, Percutaneous Approach */
"00WE3MZ"  =  "1"  /* Revision of Neurostimulator Lead in Cranial Nerve, Percutaneous Approach */
"00WE40Z"  =  "1"  /* Revision of Drainage Device in Cranial Nerve, Percutaneous Endoscopic Approach */
"00WE42Z"  =  "1"  /* Revision of Monitoring Device in Cranial Nerve, Percutaneous Endoscopic Approach */
"00WE43Z"  =  "1"  /* Revision of Infusion Device in Cranial Nerve, Percutaneous Endoscopic Approach */
"00WE47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Cranial Nerve, Percutaneous Endoscopic Approach */
"00WE4MZ"  =  "1"  /* Revision of Neurostimulator Lead in Cranial Nerve, Percutaneous Endoscopic Approach */
"019000Z"  =  "1"  /* Drainage of Cervical Plexus with Drainage Device, Open Approach */
"01900ZZ"  =  "1"  /* Drainage of Cervical Plexus, Open Approach */
"019030Z"  =  "1"  /* Drainage of Cervical Plexus with Drainage Device, Percutaneous Approach */
"01903ZZ"  =  "1"  /* Drainage of Cervical Plexus, Percutaneous Approach */
"019040Z"  =  "1"  /* Drainage of Cervical Plexus with Drainage Device, Percutaneous Endoscopic Approach */
"01904ZZ"  =  "1"  /* Drainage of Cervical Plexus, Percutaneous Endoscopic Approach */
"019100Z"  =  "1"  /* Drainage of Cervical Nerve with Drainage Device, Open Approach */
"01910ZZ"  =  "1"  /* Drainage of Cervical Nerve, Open Approach */
"019130Z"  =  "1"  /* Drainage of Cervical Nerve with Drainage Device, Percutaneous Approach */
"01913ZZ"  =  "1"  /* Drainage of Cervical Nerve, Percutaneous Approach */
"019140Z"  =  "1"  /* Drainage of Cervical Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"01914ZZ"  =  "1"  /* Drainage of Cervical Nerve, Percutaneous Endoscopic Approach */
"019200Z"  =  "1"  /* Drainage of Phrenic Nerve with Drainage Device, Open Approach */
"01920ZZ"  =  "1"  /* Drainage of Phrenic Nerve, Open Approach */
"019230Z"  =  "1"  /* Drainage of Phrenic Nerve with Drainage Device, Percutaneous Approach */
"01923ZZ"  =  "1"  /* Drainage of Phrenic Nerve, Percutaneous Approach */
"019240Z"  =  "1"  /* Drainage of Phrenic Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"01924ZZ"  =  "1"  /* Drainage of Phrenic Nerve, Percutaneous Endoscopic Approach */
"019300Z"  =  "1"  /* Drainage of Brachial Plexus with Drainage Device, Open Approach */
"01930ZZ"  =  "1"  /* Drainage of Brachial Plexus, Open Approach */
"019330Z"  =  "1"  /* Drainage of Brachial Plexus with Drainage Device, Percutaneous Approach */
"01933ZZ"  =  "1"  /* Drainage of Brachial Plexus, Percutaneous Approach */
"019340Z"  =  "1"  /* Drainage of Brachial Plexus with Drainage Device, Percutaneous Endoscopic Approach */
"01934ZZ"  =  "1"  /* Drainage of Brachial Plexus, Percutaneous Endoscopic Approach */
"019400Z"  =  "1"  /* Drainage of Ulnar Nerve with Drainage Device, Open Approach */
"01940ZZ"  =  "1"  /* Drainage of Ulnar Nerve, Open Approach */
"019430Z"  =  "1"  /* Drainage of Ulnar Nerve with Drainage Device, Percutaneous Approach */
"01943ZZ"  =  "1"  /* Drainage of Ulnar Nerve, Percutaneous Approach */
"019440Z"  =  "1"  /* Drainage of Ulnar Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"01944ZZ"  =  "1"  /* Drainage of Ulnar Nerve, Percutaneous Endoscopic Approach */
"019500Z"  =  "1"  /* Drainage of Median Nerve with Drainage Device, Open Approach */
"01950ZZ"  =  "1"  /* Drainage of Median Nerve, Open Approach */
"019530Z"  =  "1"  /* Drainage of Median Nerve with Drainage Device, Percutaneous Approach */
"01953ZZ"  =  "1"  /* Drainage of Median Nerve, Percutaneous Approach */
"019540Z"  =  "1"  /* Drainage of Median Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"01954ZZ"  =  "1"  /* Drainage of Median Nerve, Percutaneous Endoscopic Approach */
"019600Z"  =  "1"  /* Drainage of Radial Nerve with Drainage Device, Open Approach */
"01960ZZ"  =  "1"  /* Drainage of Radial Nerve, Open Approach */
"019630Z"  =  "1"  /* Drainage of Radial Nerve with Drainage Device, Percutaneous Approach */
"01963ZZ"  =  "1"  /* Drainage of Radial Nerve, Percutaneous Approach */
"019640Z"  =  "1"  /* Drainage of Radial Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"01964ZZ"  =  "1"  /* Drainage of Radial Nerve, Percutaneous Endoscopic Approach */
"019800Z"  =  "1"  /* Drainage of Thoracic Nerve with Drainage Device, Open Approach */
"01980ZZ"  =  "1"  /* Drainage of Thoracic Nerve, Open Approach */
"019830Z"  =  "1"  /* Drainage of Thoracic Nerve with Drainage Device, Percutaneous Approach */
"01983ZZ"  =  "1"  /* Drainage of Thoracic Nerve, Percutaneous Approach */
"019840Z"  =  "1"  /* Drainage of Thoracic Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"01984ZZ"  =  "1"  /* Drainage of Thoracic Nerve, Percutaneous Endoscopic Approach */
"019900Z"  =  "1"  /* Drainage of Lumbar Plexus with Drainage Device, Open Approach */
"01990ZZ"  =  "1"  /* Drainage of Lumbar Plexus, Open Approach */
"019930Z"  =  "1"  /* Drainage of Lumbar Plexus with Drainage Device, Percutaneous Approach */
"01993ZZ"  =  "1"  /* Drainage of Lumbar Plexus, Percutaneous Approach */
"019940Z"  =  "1"  /* Drainage of Lumbar Plexus with Drainage Device, Percutaneous Endoscopic Approach */
"01994ZZ"  =  "1"  /* Drainage of Lumbar Plexus, Percutaneous Endoscopic Approach */
"019A00Z"  =  "1"  /* Drainage of Lumbosacral Plexus with Drainage Device, Open Approach */
"019A0ZZ"  =  "1"  /* Drainage of Lumbosacral Plexus, Open Approach */
"019A30Z"  =  "1"  /* Drainage of Lumbosacral Plexus with Drainage Device, Percutaneous Approach */
"019A3ZZ"  =  "1"  /* Drainage of Lumbosacral Plexus, Percutaneous Approach */
"019A40Z"  =  "1"  /* Drainage of Lumbosacral Plexus with Drainage Device, Percutaneous Endoscopic Approach */
"019A4ZZ"  =  "1"  /* Drainage of Lumbosacral Plexus, Percutaneous Endoscopic Approach */
"019B00Z"  =  "1"  /* Drainage of Lumbar Nerve with Drainage Device, Open Approach */
"019B0ZZ"  =  "1"  /* Drainage of Lumbar Nerve, Open Approach */
"019B30Z"  =  "1"  /* Drainage of Lumbar Nerve with Drainage Device, Percutaneous Approach */
"019B3ZZ"  =  "1"  /* Drainage of Lumbar Nerve, Percutaneous Approach */
"019B40Z"  =  "1"  /* Drainage of Lumbar Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"019B4ZZ"  =  "1"  /* Drainage of Lumbar Nerve, Percutaneous Endoscopic Approach */
"019C00Z"  =  "1"  /* Drainage of Pudendal Nerve with Drainage Device, Open Approach */
"019C0ZZ"  =  "1"  /* Drainage of Pudendal Nerve, Open Approach */
"019C30Z"  =  "1"  /* Drainage of Pudendal Nerve with Drainage Device, Percutaneous Approach */
"019C3ZZ"  =  "1"  /* Drainage of Pudendal Nerve, Percutaneous Approach */
"019C40Z"  =  "1"  /* Drainage of Pudendal Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"019C4ZZ"  =  "1"  /* Drainage of Pudendal Nerve, Percutaneous Endoscopic Approach */
"019D00Z"  =  "1"  /* Drainage of Femoral Nerve with Drainage Device, Open Approach */
"019D0ZZ"  =  "1"  /* Drainage of Femoral Nerve, Open Approach */
"019D30Z"  =  "1"  /* Drainage of Femoral Nerve with Drainage Device, Percutaneous Approach */
"019D3ZZ"  =  "1"  /* Drainage of Femoral Nerve, Percutaneous Approach */
"019D40Z"  =  "1"  /* Drainage of Femoral Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"019D4ZZ"  =  "1"  /* Drainage of Femoral Nerve, Percutaneous Endoscopic Approach */
"019F00Z"  =  "1"  /* Drainage of Sciatic Nerve with Drainage Device, Open Approach */
"019F0ZZ"  =  "1"  /* Drainage of Sciatic Nerve, Open Approach */
"019F30Z"  =  "1"  /* Drainage of Sciatic Nerve with Drainage Device, Percutaneous Approach */
"019F3ZZ"  =  "1"  /* Drainage of Sciatic Nerve, Percutaneous Approach */
"019F40Z"  =  "1"  /* Drainage of Sciatic Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"019F4ZZ"  =  "1"  /* Drainage of Sciatic Nerve, Percutaneous Endoscopic Approach */
"019G00Z"  =  "1"  /* Drainage of Tibial Nerve with Drainage Device, Open Approach */
"019G0ZZ"  =  "1"  /* Drainage of Tibial Nerve, Open Approach */
"019G30Z"  =  "1"  /* Drainage of Tibial Nerve with Drainage Device, Percutaneous Approach */
"019G3ZZ"  =  "1"  /* Drainage of Tibial Nerve, Percutaneous Approach */
"019G40Z"  =  "1"  /* Drainage of Tibial Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"019G4ZZ"  =  "1"  /* Drainage of Tibial Nerve, Percutaneous Endoscopic Approach */
"019H00Z"  =  "1"  /* Drainage of Peroneal Nerve with Drainage Device, Open Approach */
"019H0ZZ"  =  "1"  /* Drainage of Peroneal Nerve, Open Approach */
"019H30Z"  =  "1"  /* Drainage of Peroneal Nerve with Drainage Device, Percutaneous Approach */
"019H3ZZ"  =  "1"  /* Drainage of Peroneal Nerve, Percutaneous Approach */
"019H40Z"  =  "1"  /* Drainage of Peroneal Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"019H4ZZ"  =  "1"  /* Drainage of Peroneal Nerve, Percutaneous Endoscopic Approach */
"019Q00Z"  =  "1"  /* Drainage of Sacral Plexus with Drainage Device, Open Approach */
"019Q0ZZ"  =  "1"  /* Drainage of Sacral Plexus, Open Approach */
"019Q30Z"  =  "1"  /* Drainage of Sacral Plexus with Drainage Device, Percutaneous Approach */
"019Q3ZZ"  =  "1"  /* Drainage of Sacral Plexus, Percutaneous Approach */
"019Q40Z"  =  "1"  /* Drainage of Sacral Plexus with Drainage Device, Percutaneous Endoscopic Approach */
"019Q4ZZ"  =  "1"  /* Drainage of Sacral Plexus, Percutaneous Endoscopic Approach */
"019R00Z"  =  "1"  /* Drainage of Sacral Nerve with Drainage Device, Open Approach */
"019R0ZZ"  =  "1"  /* Drainage of Sacral Nerve, Open Approach */
"019R30Z"  =  "1"  /* Drainage of Sacral Nerve with Drainage Device, Percutaneous Approach */
"019R3ZZ"  =  "1"  /* Drainage of Sacral Nerve, Percutaneous Approach */
"019R40Z"  =  "1"  /* Drainage of Sacral Nerve with Drainage Device, Percutaneous Endoscopic Approach */
"019R4ZZ"  =  "1"  /* Drainage of Sacral Nerve, Percutaneous Endoscopic Approach */
"01C00ZZ"  =  "1"  /* Extirpation of Matter from Cervical Plexus, Open Approach */
"01C03ZZ"  =  "1"  /* Extirpation of Matter from Cervical Plexus, Percutaneous Approach */
"01C04ZZ"  =  "1"  /* Extirpation of Matter from Cervical Plexus, Percutaneous Endoscopic Approach */
"01C10ZZ"  =  "1"  /* Extirpation of Matter from Cervical Nerve, Open Approach */
"01C13ZZ"  =  "1"  /* Extirpation of Matter from Cervical Nerve, Percutaneous Approach */
"01C14ZZ"  =  "1"  /* Extirpation of Matter from Cervical Nerve, Percutaneous Endoscopic Approach */
"01C20ZZ"  =  "1"  /* Extirpation of Matter from Phrenic Nerve, Open Approach */
"01C23ZZ"  =  "1"  /* Extirpation of Matter from Phrenic Nerve, Percutaneous Approach */
"01C24ZZ"  =  "1"  /* Extirpation of Matter from Phrenic Nerve, Percutaneous Endoscopic Approach */
"01C30ZZ"  =  "1"  /* Extirpation of Matter from Brachial Plexus, Open Approach */
"01C33ZZ"  =  "1"  /* Extirpation of Matter from Brachial Plexus, Percutaneous Approach */
"01C34ZZ"  =  "1"  /* Extirpation of Matter from Brachial Plexus, Percutaneous Endoscopic Approach */
"01C40ZZ"  =  "1"  /* Extirpation of Matter from Ulnar Nerve, Open Approach */
"01C43ZZ"  =  "1"  /* Extirpation of Matter from Ulnar Nerve, Percutaneous Approach */
"01C44ZZ"  =  "1"  /* Extirpation of Matter from Ulnar Nerve, Percutaneous Endoscopic Approach */
"01C50ZZ"  =  "1"  /* Extirpation of Matter from Median Nerve, Open Approach */
"01C53ZZ"  =  "1"  /* Extirpation of Matter from Median Nerve, Percutaneous Approach */
"01C54ZZ"  =  "1"  /* Extirpation of Matter from Median Nerve, Percutaneous Endoscopic Approach */
"01C60ZZ"  =  "1"  /* Extirpation of Matter from Radial Nerve, Open Approach */
"01C63ZZ"  =  "1"  /* Extirpation of Matter from Radial Nerve, Percutaneous Approach */
"01C64ZZ"  =  "1"  /* Extirpation of Matter from Radial Nerve, Percutaneous Endoscopic Approach */
"01C80ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Nerve, Open Approach */
"01C83ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Nerve, Percutaneous Approach */
"01C84ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Nerve, Percutaneous Endoscopic Approach */
"01C90ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Plexus, Open Approach */
"01C93ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Plexus, Percutaneous Approach */
"01C94ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Plexus, Percutaneous Endoscopic Approach */
"01CA0ZZ"  =  "1"  /* Extirpation of Matter from Lumbosacral Plexus, Open Approach */
"01CA3ZZ"  =  "1"  /* Extirpation of Matter from Lumbosacral Plexus, Percutaneous Approach */
"01CA4ZZ"  =  "1"  /* Extirpation of Matter from Lumbosacral Plexus, Percutaneous Endoscopic Approach */
"01CB0ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Nerve, Open Approach */
"01CB3ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Nerve, Percutaneous Approach */
"01CB4ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Nerve, Percutaneous Endoscopic Approach */
"01CC0ZZ"  =  "1"  /* Extirpation of Matter from Pudendal Nerve, Open Approach */
"01CC3ZZ"  =  "1"  /* Extirpation of Matter from Pudendal Nerve, Percutaneous Approach */
"01CC4ZZ"  =  "1"  /* Extirpation of Matter from Pudendal Nerve, Percutaneous Endoscopic Approach */
"01CD0ZZ"  =  "1"  /* Extirpation of Matter from Femoral Nerve, Open Approach */
"01CD3ZZ"  =  "1"  /* Extirpation of Matter from Femoral Nerve, Percutaneous Approach */
"01CD4ZZ"  =  "1"  /* Extirpation of Matter from Femoral Nerve, Percutaneous Endoscopic Approach */
"01CF0ZZ"  =  "1"  /* Extirpation of Matter from Sciatic Nerve, Open Approach */
"01CF3ZZ"  =  "1"  /* Extirpation of Matter from Sciatic Nerve, Percutaneous Approach */
"01CF4ZZ"  =  "1"  /* Extirpation of Matter from Sciatic Nerve, Percutaneous Endoscopic Approach */
"01CG0ZZ"  =  "1"  /* Extirpation of Matter from Tibial Nerve, Open Approach */
"01CG3ZZ"  =  "1"  /* Extirpation of Matter from Tibial Nerve, Percutaneous Approach */
"01CG4ZZ"  =  "1"  /* Extirpation of Matter from Tibial Nerve, Percutaneous Endoscopic Approach */
"01CH0ZZ"  =  "1"  /* Extirpation of Matter from Peroneal Nerve, Open Approach */
"01CH3ZZ"  =  "1"  /* Extirpation of Matter from Peroneal Nerve, Percutaneous Approach */
"01CH4ZZ"  =  "1"  /* Extirpation of Matter from Peroneal Nerve, Percutaneous Endoscopic Approach */
"01CQ0ZZ"  =  "1"  /* Extirpation of Matter from Sacral Plexus, Open Approach */
"01CQ3ZZ"  =  "1"  /* Extirpation of Matter from Sacral Plexus, Percutaneous Approach */
"01CQ4ZZ"  =  "1"  /* Extirpation of Matter from Sacral Plexus, Percutaneous Endoscopic Approach */
"01CR0ZZ"  =  "1"  /* Extirpation of Matter from Sacral Nerve, Open Approach */
"01CR3ZZ"  =  "1"  /* Extirpation of Matter from Sacral Nerve, Percutaneous Approach */
"01CR4ZZ"  =  "1"  /* Extirpation of Matter from Sacral Nerve, Percutaneous Endoscopic Approach */
"01HY02Z"  =  "1"  /* Insertion of Monitoring Device into Peripheral Nerve, Open Approach */
"01HY32Z"  =  "1"  /* Insertion of Monitoring Device into Peripheral Nerve, Percutaneous Approach */
"01HY42Z"  =  "1"  /* Insertion of Monitoring Device into Peripheral Nerve, Percutaneous Endoscopic Approach */
"01N50ZZ"  =  "1"  /* Release Median Nerve, Open Approach */
"01N53ZZ"  =  "1"  /* Release Median Nerve, Percutaneous Approach */
"01N54ZZ"  =  "1"  /* Release Median Nerve, Percutaneous Endoscopic Approach */
"01NG0ZZ"  =  "1"  /* Release Tibial Nerve, Open Approach */
"01NG3ZZ"  =  "1"  /* Release Tibial Nerve, Percutaneous Approach */
"01NG4ZZ"  =  "1"  /* Release Tibial Nerve, Percutaneous Endoscopic Approach */
"01PY00Z"  =  "1"  /* Removal of Drainage Device from Peripheral Nerve, Open Approach */
"01PY02Z"  =  "1"  /* Removal of Monitoring Device from Peripheral Nerve, Open Approach */
"01PY07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Peripheral Nerve, Open Approach */
"01PY30Z"  =  "1"  /* Removal of Drainage Device from Peripheral Nerve, Percutaneous Approach */
"01PY32Z"  =  "1"  /* Removal of Monitoring Device from Peripheral Nerve, Percutaneous Approach */
"01PY37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Peripheral Nerve, Percutaneous Approach */
"01PY40Z"  =  "1"  /* Removal of Drainage Device from Peripheral Nerve, Percutaneous Endoscopic Approach */
"01PY42Z"  =  "1"  /* Removal of Monitoring Device from Peripheral Nerve, Percutaneous Endoscopic Approach */
"01PY47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Peripheral Nerve, Percutaneous Endoscopic Approach */
"01WY00Z"  =  "1"  /* Revision of Drainage Device in Peripheral Nerve, Open Approach */
"01WY02Z"  =  "1"  /* Revision of Monitoring Device in Peripheral Nerve, Open Approach */
"01WY07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Peripheral Nerve, Open Approach */
"01WY0MZ"  =  "1"  /* Revision of Neurostimulator Lead in Peripheral Nerve, Open Approach */
"01WY30Z"  =  "1"  /* Revision of Drainage Device in Peripheral Nerve, Percutaneous Approach */
"01WY32Z"  =  "1"  /* Revision of Monitoring Device in Peripheral Nerve, Percutaneous Approach */
"01WY37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Peripheral Nerve, Percutaneous Approach */
"01WY3MZ"  =  "1"  /* Revision of Neurostimulator Lead in Peripheral Nerve, Percutaneous Approach */
"01WY40Z"  =  "1"  /* Revision of Drainage Device in Peripheral Nerve, Percutaneous Endoscopic Approach */
"01WY42Z"  =  "1"  /* Revision of Monitoring Device in Peripheral Nerve, Percutaneous Endoscopic Approach */
"01WY47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Peripheral Nerve, Percutaneous Endoscopic Approach */
"01WY4MZ"  =  "1"  /* Revision of Neurostimulator Lead in Peripheral Nerve, Percutaneous Endoscopic Approach */
"0210344"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Approach */
"02103D4"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Vein with Intraluminal Device, Percutaneous Approach */
"0210444"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"02104D4"  =  "1"  /* Bypass Coronary Artery, One Site from Coronary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"0211344"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Approach */
"02113D4"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Vein with Intraluminal Device, Percutaneous Approach */
"0211444"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"02114D4"  =  "1"  /* Bypass Coronary Artery, Two Sites from Coronary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"0212344"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Approach */
"02123D4"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Vein with Intraluminal Device, Percutaneous Approach */
"0212444"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"02124D4"  =  "1"  /* Bypass Coronary Artery, Three Sites from Coronary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"0213344"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Approach */
"02133D4"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Vein with Intraluminal Device, Percutaneous Approach */
"0213444"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach */
"02134D4"  =  "1"  /* Bypass Coronary Artery, Four or More Sites from Coronary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"02170ZP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk, Open Approach */
"02170ZQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery, Open Approach */
"02170ZR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery, Open Approach */
"02540ZZ"  =  "1"  /* Destruction of Coronary Vein, Open Approach */
"02543ZZ"  =  "1"  /* Destruction of Coronary Vein, Percutaneous Approach */
"02544ZZ"  =  "1"  /* Destruction of Coronary Vein, Percutaneous Endoscopic Approach */
"02880ZZ"  =  "1"  /* Division of Conduction Mechanism, Open Approach */
"02883ZZ"  =  "1"  /* Division of Conduction Mechanism, Percutaneous Approach */
"02884ZZ"  =  "1"  /* Division of Conduction Mechanism, Percutaneous Endoscopic Approach */
"02B40ZZ"  =  "1"  /* Excision of Coronary Vein, Open Approach */
"02B43ZZ"  =  "1"  /* Excision of Coronary Vein, Percutaneous Approach */
"02B44ZZ"  =  "1"  /* Excision of Coronary Vein, Percutaneous Endoscopic Approach */
"02C40ZZ"  =  "1"  /* Extirpation of Matter from Coronary Vein, Open Approach */
"02C43ZZ"  =  "1"  /* Extirpation of Matter from Coronary Vein, Percutaneous Approach */
"02C44ZZ"  =  "1"  /* Extirpation of Matter from Coronary Vein, Percutaneous Endoscopic Approach */
"02C60ZZ"  =  "1"  /* Extirpation of Matter from Right Atrium, Open Approach */
"02C63ZZ"  =  "1"  /* Extirpation of Matter from Right Atrium, Percutaneous Approach */
"02C64ZZ"  =  "1"  /* Extirpation of Matter from Right Atrium, Percutaneous Endoscopic Approach */
"02C70ZZ"  =  "1"  /* Extirpation of Matter from Left Atrium, Open Approach */
"02C73ZZ"  =  "1"  /* Extirpation of Matter from Left Atrium, Percutaneous Approach */
"02C74ZZ"  =  "1"  /* Extirpation of Matter from Left Atrium, Percutaneous Endoscopic Approach */
"02C80ZZ"  =  "1"  /* Extirpation of Matter from Conduction Mechanism, Open Approach */
"02C83ZZ"  =  "1"  /* Extirpation of Matter from Conduction Mechanism, Percutaneous Approach */
"02C84ZZ"  =  "1"  /* Extirpation of Matter from Conduction Mechanism, Percutaneous Endoscopic Approach */
"02C90ZZ"  =  "1"  /* Extirpation of Matter from Chordae Tendineae, Open Approach */
"02C93ZZ"  =  "1"  /* Extirpation of Matter from Chordae Tendineae, Percutaneous Approach */
"02C94ZZ"  =  "1"  /* Extirpation of Matter from Chordae Tendineae, Percutaneous Endoscopic Approach */
"02CK0ZZ"  =  "1"  /* Extirpation of Matter from Right Ventricle, Open Approach */
"02CK3ZZ"  =  "1"  /* Extirpation of Matter from Right Ventricle, Percutaneous Approach */
"02CK4ZZ"  =  "1"  /* Extirpation of Matter from Right Ventricle, Percutaneous Endoscopic Approach */
"02CL0ZZ"  =  "1"  /* Extirpation of Matter from Left Ventricle, Open Approach */
"02CL3ZZ"  =  "1"  /* Extirpation of Matter from Left Ventricle, Percutaneous Approach */
"02CL4ZZ"  =  "1"  /* Extirpation of Matter from Left Ventricle, Percutaneous Endoscopic Approach */
"02CP0ZZ"  =  "1"  /* Extirpation of Matter from Pulmonary Trunk, Open Approach */
"02CP3ZZ"  =  "1"  /* Extirpation of Matter from Pulmonary Trunk, Percutaneous Approach */
"02CP4ZZ"  =  "1"  /* Extirpation of Matter from Pulmonary Trunk, Percutaneous Endoscopic Approach */
"02CQ0ZZ"  =  "1"  /* Extirpation of Matter from Right Pulmonary Artery, Open Approach */
"02CQ3ZZ"  =  "1"  /* Extirpation of Matter from Right Pulmonary Artery, Percutaneous Approach */
"02CQ4ZZ"  =  "1"  /* Extirpation of Matter from Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"02CR0ZZ"  =  "1"  /* Extirpation of Matter from Left Pulmonary Artery, Open Approach */
"02CR3ZZ"  =  "1"  /* Extirpation of Matter from Left Pulmonary Artery, Percutaneous Approach */
"02CR4ZZ"  =  "1"  /* Extirpation of Matter from Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"02CS0ZZ"  =  "1"  /* Extirpation of Matter from Right Pulmonary Vein, Open Approach */
"02CS3ZZ"  =  "1"  /* Extirpation of Matter from Right Pulmonary Vein, Percutaneous Approach */
"02CS4ZZ"  =  "1"  /* Extirpation of Matter from Right Pulmonary Vein, Percutaneous Endoscopic Approach */
"02CT0ZZ"  =  "1"  /* Extirpation of Matter from Left Pulmonary Vein, Open Approach */
"02CT3ZZ"  =  "1"  /* Extirpation of Matter from Left Pulmonary Vein, Percutaneous Approach */
"02CT4ZZ"  =  "1"  /* Extirpation of Matter from Left Pulmonary Vein, Percutaneous Endoscopic Approach */
"02CV0ZZ"  =  "1"  /* Extirpation of Matter from Superior Vena Cava, Open Approach */
"02CV3ZZ"  =  "1"  /* Extirpation of Matter from Superior Vena Cava, Percutaneous Approach */
"02CV4ZZ"  =  "1"  /* Extirpation of Matter from Superior Vena Cava, Percutaneous Endoscopic Approach */
"02CW0ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Aorta, Open Approach */
"02CW3ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Aorta, Percutaneous Approach */
"02CW4ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Aorta, Percutaneous Endoscopic Approach */
"02H402Z"  =  "1"  /* Insertion of Monitoring Device into Coronary Vein, Open Approach */
"02H403Z"  =  "1"  /* Insertion of Infusion Device into Coronary Vein, Open Approach */
"02H40DZ"  =  "1"  /* Insertion of Intraluminal Device into Coronary Vein, Open Approach */
"02H432Z"  =  "1"  /* Insertion of Monitoring Device into Coronary Vein, Percutaneous Approach */
"02H433Z"  =  "1"  /* Insertion of Infusion Device into Coronary Vein, Percutaneous Approach */
"02H43DZ"  =  "1"  /* Insertion of Intraluminal Device into Coronary Vein, Percutaneous Approach */
"02H442Z"  =  "1"  /* Insertion of Monitoring Device into Coronary Vein, Percutaneous Endoscopic Approach */
"02H443Z"  =  "1"  /* Insertion of Infusion Device into Coronary Vein, Percutaneous Endoscopic Approach */
"02H44DZ"  =  "1"  /* Insertion of Intraluminal Device into Coronary Vein, Percutaneous Endoscopic Approach */
"02H602Z"  =  "1"  /* Insertion of Monitoring Device into Right Atrium, Open Approach */
"02H603Z"  =  "1"  /* Insertion of Infusion Device into Right Atrium, Open Approach */
"02H60DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Atrium, Open Approach */
"02H632Z"  =  "1"  /* Insertion of Monitoring Device into Right Atrium, Percutaneous Approach */
"02H633Z"  =  "1"  /* Insertion of Infusion Device into Right Atrium, Percutaneous Approach */
"02H63DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Atrium, Percutaneous Approach */
"02H642Z"  =  "1"  /* Insertion of Monitoring Device into Right Atrium, Percutaneous Endoscopic Approach */
"02H643Z"  =  "1"  /* Insertion of Infusion Device into Right Atrium, Percutaneous Endoscopic Approach */
"02H64DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Atrium, Percutaneous Endoscopic Approach */
"02H702Z"  =  "1"  /* Insertion of Monitoring Device into Left Atrium, Open Approach */
"02H703Z"  =  "1"  /* Insertion of Infusion Device into Left Atrium, Open Approach */
"02H70DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Atrium, Open Approach */
"02H732Z"  =  "1"  /* Insertion of Monitoring Device into Left Atrium, Percutaneous Approach */
"02H733Z"  =  "1"  /* Insertion of Infusion Device into Left Atrium, Percutaneous Approach */
"02H73DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Atrium, Percutaneous Approach */
"02H742Z"  =  "1"  /* Insertion of Monitoring Device into Left Atrium, Percutaneous Endoscopic Approach */
"02H743Z"  =  "1"  /* Insertion of Infusion Device into Left Atrium, Percutaneous Endoscopic Approach */
"02H74DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Atrium, Percutaneous Endoscopic Approach */
"02HK03Z"  =  "1"  /* Insertion of Infusion Device into Right Ventricle, Open Approach */
"02HK0DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Ventricle, Open Approach */
"02HK33Z"  =  "1"  /* Insertion of Infusion Device into Right Ventricle, Percutaneous Approach */
"02HK3DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Ventricle, Percutaneous Approach */
"02HK43Z"  =  "1"  /* Insertion of Infusion Device into Right Ventricle, Percutaneous Endoscopic Approach */
"02HK4DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Ventricle, Percutaneous Endoscopic Approach */
"02HL02Z"  =  "1"  /* Insertion of Monitoring Device into Left Ventricle, Open Approach */
"02HL03Z"  =  "1"  /* Insertion of Infusion Device into Left Ventricle, Open Approach */
"02HL0DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Ventricle, Open Approach */
"02HL32Z"  =  "1"  /* Insertion of Monitoring Device into Left Ventricle, Percutaneous Approach */
"02HL33Z"  =  "1"  /* Insertion of Infusion Device into Left Ventricle, Percutaneous Approach */
"02HL3DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Ventricle, Percutaneous Approach */
"02HL42Z"  =  "1"  /* Insertion of Monitoring Device into Left Ventricle, Percutaneous Endoscopic Approach */
"02HL43Z"  =  "1"  /* Insertion of Infusion Device into Left Ventricle, Percutaneous Endoscopic Approach */
"02HL4DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Ventricle, Percutaneous Endoscopic Approach */
"02HP0DZ"  =  "1"  /* Insertion of Intraluminal Device into Pulmonary Trunk, Open Approach */
"02HP3DZ"  =  "1"  /* Insertion of Intraluminal Device into Pulmonary Trunk, Percutaneous Approach */
"02HP4DZ"  =  "1"  /* Insertion of Intraluminal Device into Pulmonary Trunk, Percutaneous Endoscopic Approach */
"02HQ0DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Pulmonary Artery, Open Approach */
"02HQ3DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Pulmonary Artery, Percutaneous Approach */
"02HQ4DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"02HR0DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Pulmonary Artery, Open Approach */
"02HR3DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Pulmonary Artery, Percutaneous Approach */
"02HR4DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"02HS02Z"  =  "1"  /* Insertion of Monitoring Device into Right Pulmonary Vein, Open Approach */
"02HS0DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Pulmonary Vein, Open Approach */
"02HS32Z"  =  "1"  /* Insertion of Monitoring Device into Right Pulmonary Vein, Percutaneous Approach */
"02HS3DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Pulmonary Vein, Percutaneous Approach */
"02HS42Z"  =  "1"  /* Insertion of Monitoring Device into Right Pulmonary Vein, Percutaneous Endoscopic Approach */
"02HS4DZ"  =  "1"  /* Insertion of Intraluminal Device into Right Pulmonary Vein, Percutaneous Endoscopic Approach */
"02HT02Z"  =  "1"  /* Insertion of Monitoring Device into Left Pulmonary Vein, Open Approach */
"02HT0DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Pulmonary Vein, Open Approach */
"02HT32Z"  =  "1"  /* Insertion of Monitoring Device into Left Pulmonary Vein, Percutaneous Approach */
"02HT3DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Pulmonary Vein, Percutaneous Approach */
"02HT42Z"  =  "1"  /* Insertion of Monitoring Device into Left Pulmonary Vein, Percutaneous Endoscopic Approach */
"02HT4DZ"  =  "1"  /* Insertion of Intraluminal Device into Left Pulmonary Vein, Percutaneous Endoscopic Approach */
"02HV02Z"  =  "1"  /* Insertion of Monitoring Device into Superior Vena Cava, Open Approach */
"02HV0DZ"  =  "1"  /* Insertion of Intraluminal Device into Superior Vena Cava, Open Approach */
"02HV32Z"  =  "1"  /* Insertion of Monitoring Device into Superior Vena Cava, Percutaneous Approach */
"02HV3DZ"  =  "1"  /* Insertion of Intraluminal Device into Superior Vena Cava, Percutaneous Approach */
"02HV42Z"  =  "1"  /* Insertion of Monitoring Device into Superior Vena Cava, Percutaneous Endoscopic Approach */
"02HV4DZ"  =  "1"  /* Insertion of Intraluminal Device into Superior Vena Cava, Percutaneous Endoscopic Approach */
"02HW02Z"  =  "1"  /* Insertion of Monitoring Device into Thoracic Aorta, Open Approach */
"02HW0DZ"  =  "1"  /* Insertion of Intraluminal Device into Thoracic Aorta, Open Approach */
"02HW32Z"  =  "1"  /* Insertion of Monitoring Device into Thoracic Aorta, Percutaneous Approach */
"02HW3DZ"  =  "1"  /* Insertion of Intraluminal Device into Thoracic Aorta, Percutaneous Approach */
"02HW42Z"  =  "1"  /* Insertion of Monitoring Device into Thoracic Aorta, Percutaneous Endoscopic Approach */
"02HW4DZ"  =  "1"  /* Insertion of Intraluminal Device into Thoracic Aorta, Percutaneous Endoscopic Approach */
"02JA0ZZ"  =  "1"  /* Inspection of Heart, Open Approach */
"02JA4ZZ"  =  "1"  /* Inspection of Heart, Percutaneous Endoscopic Approach */
"02JY0ZZ"  =  "1"  /* Inspection of Great Vessel, Open Approach */
"02JY4ZZ"  =  "1"  /* Inspection of Great Vessel, Percutaneous Endoscopic Approach */
"02LR0CT"  =  "1"  /* Occlusion of Ductus Arteriosus with Extraluminal Device, Open Approach */
"02LR0DT"  =  "1"  /* Occlusion of Ductus Arteriosus with Intraluminal Device, Open Approach */
"02LR0ZT"  =  "1"  /* Occlusion of Ductus Arteriosus, Open Approach */
"02LR3CT"  =  "1"  /* Occlusion of Ductus Arteriosus with Extraluminal Device, Percutaneous Approach */
"02LR3DT"  =  "1"  /* Occlusion of Ductus Arteriosus with Intraluminal Device, Percutaneous Approach */
"02LR3ZT"  =  "1"  /* Occlusion of Ductus Arteriosus, Percutaneous Approach */
"02LR4CT"  =  "1"  /* Occlusion of Ductus Arteriosus with Extraluminal Device, Percutaneous Endoscopic Approach */
"02LR4DT"  =  "1"  /* Occlusion of Ductus Arteriosus with Intraluminal Device, Percutaneous Endoscopic Approach */
"02LR4ZT"  =  "1"  /* Occlusion of Ductus Arteriosus, Percutaneous Endoscopic Approach */
"02LS0CZ"  =  "1"  /* Occlusion of Right Pulmonary Vein with Extraluminal Device, Open Approach */
"02LS0DZ"  =  "1"  /* Occlusion of Right Pulmonary Vein with Intraluminal Device, Open Approach */
"02LS0ZZ"  =  "1"  /* Occlusion of Right Pulmonary Vein, Open Approach */
"02LS3CZ"  =  "1"  /* Occlusion of Right Pulmonary Vein with Extraluminal Device, Percutaneous Approach */
"02LS3DZ"  =  "1"  /* Occlusion of Right Pulmonary Vein with Intraluminal Device, Percutaneous Approach */
"02LS3ZZ"  =  "1"  /* Occlusion of Right Pulmonary Vein, Percutaneous Approach */
"02LS4CZ"  =  "1"  /* Occlusion of Right Pulmonary Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"02LS4DZ"  =  "1"  /* Occlusion of Right Pulmonary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"02LS4ZZ"  =  "1"  /* Occlusion of Right Pulmonary Vein, Percutaneous Endoscopic Approach */
"02LT0CZ"  =  "1"  /* Occlusion of Left Pulmonary Vein with Extraluminal Device, Open Approach */
"02LT0DZ"  =  "1"  /* Occlusion of Left Pulmonary Vein with Intraluminal Device, Open Approach */
"02LT0ZZ"  =  "1"  /* Occlusion of Left Pulmonary Vein, Open Approach */
"02LT3CZ"  =  "1"  /* Occlusion of Left Pulmonary Vein with Extraluminal Device, Percutaneous Approach */
"02LT3DZ"  =  "1"  /* Occlusion of Left Pulmonary Vein with Intraluminal Device, Percutaneous Approach */
"02LT3ZZ"  =  "1"  /* Occlusion of Left Pulmonary Vein, Percutaneous Approach */
"02LT4CZ"  =  "1"  /* Occlusion of Left Pulmonary Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"02LT4DZ"  =  "1"  /* Occlusion of Left Pulmonary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"02LT4ZZ"  =  "1"  /* Occlusion of Left Pulmonary Vein, Percutaneous Endoscopic Approach */
"02LV0CZ"  =  "1"  /* Occlusion of Superior Vena Cava with Extraluminal Device, Open Approach */
"02LV0DZ"  =  "1"  /* Occlusion of Superior Vena Cava with Intraluminal Device, Open Approach */
"02LV0ZZ"  =  "1"  /* Occlusion of Superior Vena Cava, Open Approach */
"02LV3CZ"  =  "1"  /* Occlusion of Superior Vena Cava with Extraluminal Device, Percutaneous Approach */
"02LV3DZ"  =  "1"  /* Occlusion of Superior Vena Cava with Intraluminal Device, Percutaneous Approach */
"02LV3ZZ"  =  "1"  /* Occlusion of Superior Vena Cava, Percutaneous Approach */
"02LV4CZ"  =  "1"  /* Occlusion of Superior Vena Cava with Extraluminal Device, Percutaneous Endoscopic Approach */
"02LV4DZ"  =  "1"  /* Occlusion of Superior Vena Cava with Intraluminal Device, Percutaneous Endoscopic Approach */
"02LV4ZZ"  =  "1"  /* Occlusion of Superior Vena Cava, Percutaneous Endoscopic Approach */
"02PA02Z"  =  "1"  /* Removal of Monitoring Device from Heart, Open Approach */
"02PA03Z"  =  "1"  /* Removal of Infusion Device from Heart, Open Approach */
"02PA07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Heart, Open Approach */
"02PA08Z"  =  "1"  /* Removal of Zooplastic Tissue from Heart, Open Approach */
"02PA0CZ"  =  "1"  /* Removal of Extraluminal Device from Heart, Open Approach */
"02PA0DZ"  =  "1"  /* Removal of Intraluminal Device from Heart, Open Approach */
"02PA0JZ"  =  "1"  /* Removal of Synthetic Substitute from Heart, Open Approach */
"02PA0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Heart, Open Approach */
"02PA32Z"  =  "1"  /* Removal of Monitoring Device from Heart, Percutaneous Approach */
"02PA33Z"  =  "1"  /* Removal of Infusion Device from Heart, Percutaneous Approach */
"02PA37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Heart, Percutaneous Approach */
"02PA38Z"  =  "1"  /* Removal of Zooplastic Tissue from Heart, Percutaneous Approach */
"02PA3CZ"  =  "1"  /* Removal of Extraluminal Device from Heart, Percutaneous Approach */
"02PA3DZ"  =  "1"  /* Removal of Intraluminal Device from Heart, Percutaneous Approach */
"02PA3JZ"  =  "1"  /* Removal of Synthetic Substitute from Heart, Percutaneous Approach */
"02PA3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Heart, Percutaneous Approach */
"02PA42Z"  =  "1"  /* Removal of Monitoring Device from Heart, Percutaneous Endoscopic Approach */
"02PA43Z"  =  "1"  /* Removal of Infusion Device from Heart, Percutaneous Endoscopic Approach */
"02PA47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Heart, Percutaneous Endoscopic Approach */
"02PA48Z"  =  "1"  /* Removal of Zooplastic Tissue from Heart, Percutaneous Endoscopic Approach */
"02PA4CZ"  =  "1"  /* Removal of Extraluminal Device from Heart, Percutaneous Endoscopic Approach */
"02PA4DZ"  =  "1"  /* Removal of Intraluminal Device from Heart, Percutaneous Endoscopic Approach */
"02PA4JZ"  =  "1"  /* Removal of Synthetic Substitute from Heart, Percutaneous Endoscopic Approach */
"02PA4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Heart, Percutaneous Endoscopic Approach */
"02PY02Z"  =  "1"  /* Removal of Monitoring Device from Great Vessel, Open Approach */
"02PY03Z"  =  "1"  /* Removal of Infusion Device from Great Vessel, Open Approach */
"02PY07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Great Vessel, Open Approach */
"02PY08Z"  =  "1"  /* Removal of Zooplastic Tissue from Great Vessel, Open Approach */
"02PY0CZ"  =  "1"  /* Removal of Extraluminal Device from Great Vessel, Open Approach */
"02PY0DZ"  =  "1"  /* Removal of Intraluminal Device from Great Vessel, Open Approach */
"02PY0JZ"  =  "1"  /* Removal of Synthetic Substitute from Great Vessel, Open Approach */
"02PY0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Great Vessel, Open Approach */
"02PY32Z"  =  "1"  /* Removal of Monitoring Device from Great Vessel, Percutaneous Approach */
"02PY33Z"  =  "1"  /* Removal of Infusion Device from Great Vessel, Percutaneous Approach */
"02PY37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Great Vessel, Percutaneous Approach */
"02PY38Z"  =  "1"  /* Removal of Zooplastic Tissue from Great Vessel, Percutaneous Approach */
"02PY3CZ"  =  "1"  /* Removal of Extraluminal Device from Great Vessel, Percutaneous Approach */
"02PY3DZ"  =  "1"  /* Removal of Intraluminal Device from Great Vessel, Percutaneous Approach */
"02PY3JZ"  =  "1"  /* Removal of Synthetic Substitute from Great Vessel, Percutaneous Approach */
"02PY3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Great Vessel, Percutaneous Approach */
"02PY42Z"  =  "1"  /* Removal of Monitoring Device from Great Vessel, Percutaneous Endoscopic Approach */
"02PY43Z"  =  "1"  /* Removal of Infusion Device from Great Vessel, Percutaneous Endoscopic Approach */
"02PY47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Great Vessel, Percutaneous Endoscopic Approach */
"02PY48Z"  =  "1"  /* Removal of Zooplastic Tissue from Great Vessel, Percutaneous Endoscopic Approach */
"02PY4CZ"  =  "1"  /* Removal of Extraluminal Device from Great Vessel, Percutaneous Endoscopic Approach */
"02PY4DZ"  =  "1"  /* Removal of Intraluminal Device from Great Vessel, Percutaneous Endoscopic Approach */
"02PY4JZ"  =  "1"  /* Removal of Synthetic Substitute from Great Vessel, Percutaneous Endoscopic Approach */
"02PY4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Great Vessel, Percutaneous Endoscopic Approach */
"02Q00ZZ"  =  "1"  /* Repair Coronary Artery, One Site, Open Approach */
"02Q03ZZ"  =  "1"  /* Repair Coronary Artery, One Site, Percutaneous Approach */
"02Q04ZZ"  =  "1"  /* Repair Coronary Artery, One Site, Percutaneous Endoscopic Approach */
"02Q10ZZ"  =  "1"  /* Repair Coronary Artery, Two Sites, Open Approach */
"02Q13ZZ"  =  "1"  /* Repair Coronary Artery, Two Sites, Percutaneous Approach */
"02Q14ZZ"  =  "1"  /* Repair Coronary Artery, Two Sites, Percutaneous Endoscopic Approach */
"02Q20ZZ"  =  "1"  /* Repair Coronary Artery, Three Sites, Open Approach */
"02Q23ZZ"  =  "1"  /* Repair Coronary Artery, Three Sites, Percutaneous Approach */
"02Q24ZZ"  =  "1"  /* Repair Coronary Artery, Three Sites, Percutaneous Endoscopic Approach */
"02Q30ZZ"  =  "1"  /* Repair Coronary Artery, Four or More Sites, Open Approach */
"02Q33ZZ"  =  "1"  /* Repair Coronary Artery, Four or More Sites, Percutaneous Approach */
"02Q34ZZ"  =  "1"  /* Repair Coronary Artery, Four or More Sites, Percutaneous Endoscopic Approach */
"02Q40ZZ"  =  "1"  /* Repair Coronary Vein, Open Approach */
"02Q43ZZ"  =  "1"  /* Repair Coronary Vein, Percutaneous Approach */
"02Q44ZZ"  =  "1"  /* Repair Coronary Vein, Percutaneous Endoscopic Approach */
"02QA0ZZ"  =  "1"  /* Repair Heart, Open Approach */
"02QA3ZZ"  =  "1"  /* Repair Heart, Percutaneous Approach */
"02QA4ZZ"  =  "1"  /* Repair Heart, Percutaneous Endoscopic Approach */
"02QB0ZZ"  =  "1"  /* Repair Right Heart, Open Approach */
"02QC0ZZ"  =  "1"  /* Repair Left Heart, Open Approach */
"02QP0ZZ"  =  "1"  /* Repair Pulmonary Trunk, Open Approach */
"02QP3ZZ"  =  "1"  /* Repair Pulmonary Trunk, Percutaneous Approach */
"02QP4ZZ"  =  "1"  /* Repair Pulmonary Trunk, Percutaneous Endoscopic Approach */
"02QQ0ZZ"  =  "1"  /* Repair Right Pulmonary Artery, Open Approach */
"02QQ3ZZ"  =  "1"  /* Repair Right Pulmonary Artery, Percutaneous Approach */
"02QQ4ZZ"  =  "1"  /* Repair Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"02QR0ZZ"  =  "1"  /* Repair Left Pulmonary Artery, Open Approach */
"02QR3ZZ"  =  "1"  /* Repair Left Pulmonary Artery, Percutaneous Approach */
"02QR4ZZ"  =  "1"  /* Repair Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"02QS0ZZ"  =  "1"  /* Repair Right Pulmonary Vein, Open Approach */
"02QS3ZZ"  =  "1"  /* Repair Right Pulmonary Vein, Percutaneous Approach */
"02QS4ZZ"  =  "1"  /* Repair Right Pulmonary Vein, Percutaneous Endoscopic Approach */
"02QT0ZZ"  =  "1"  /* Repair Left Pulmonary Vein, Open Approach */
"02QT3ZZ"  =  "1"  /* Repair Left Pulmonary Vein, Percutaneous Approach */
"02QT4ZZ"  =  "1"  /* Repair Left Pulmonary Vein, Percutaneous Endoscopic Approach */
"02QV0ZZ"  =  "1"  /* Repair Superior Vena Cava, Open Approach */
"02QV3ZZ"  =  "1"  /* Repair Superior Vena Cava, Percutaneous Approach */
"02QV4ZZ"  =  "1"  /* Repair Superior Vena Cava, Percutaneous Endoscopic Approach */
"02QW0ZZ"  =  "1"  /* Repair Thoracic Aorta, Open Approach */
"02QW3ZZ"  =  "1"  /* Repair Thoracic Aorta, Percutaneous Approach */
"02QW4ZZ"  =  "1"  /* Repair Thoracic Aorta, Percutaneous Endoscopic Approach */
"02RM0JZ"  =  "1"  /* Replacement of Ventricular Septum with Synthetic Substitute, Open Approach */
"02U70JZ"  =  "1"  /* Supplement Left Atrium with Synthetic Substitute, Open Approach */
"02VP0DZ"  =  "1"  /* Restriction of Pulmonary Trunk with Intraluminal Device, Open Approach */
"02VP0ZZ"  =  "1"  /* Restriction of Pulmonary Trunk, Open Approach */
"02VP3DZ"  =  "1"  /* Restriction of Pulmonary Trunk with Intraluminal Device, Percutaneous Approach */
"02VP3ZZ"  =  "1"  /* Restriction of Pulmonary Trunk, Percutaneous Approach */
"02VP4DZ"  =  "1"  /* Restriction of Pulmonary Trunk with Intraluminal Device, Percutaneous Endoscopic Approach */
"02VP4ZZ"  =  "1"  /* Restriction of Pulmonary Trunk, Percutaneous Endoscopic Approach */
"02VQ0CZ"  =  "1"  /* Restriction of Right Pulmonary Artery with Extraluminal Device, Open Approach */
"02VQ0DZ"  =  "1"  /* Restriction of Right Pulmonary Artery with Intraluminal Device, Open Approach */
"02VQ0ZZ"  =  "1"  /* Restriction of Right Pulmonary Artery, Open Approach */
"02VQ3CZ"  =  "1"  /* Restriction of Right Pulmonary Artery with Extraluminal Device, Percutaneous Approach */
"02VQ3DZ"  =  "1"  /* Restriction of Right Pulmonary Artery with Intraluminal Device, Percutaneous Approach */
"02VQ3ZZ"  =  "1"  /* Restriction of Right Pulmonary Artery, Percutaneous Approach */
"02VQ4CZ"  =  "1"  /* Restriction of Right Pulmonary Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"02VQ4DZ"  =  "1"  /* Restriction of Right Pulmonary Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"02VQ4ZZ"  =  "1"  /* Restriction of Right Pulmonary Artery, Percutaneous Endoscopic Approach */
"02VR0CZ"  =  "1"  /* Restriction of Left Pulmonary Artery with Extraluminal Device, Open Approach */
"02VR0DT"  =  "1"  /* Restriction of Ductus Arteriosus with Intraluminal Device, Open Approach */
"02VR0DZ"  =  "1"  /* Restriction of Left Pulmonary Artery with Intraluminal Device, Open Approach */
"02VR0ZT"  =  "1"  /* Restriction of Ductus Arteriosus, Open Approach */
"02VR0ZZ"  =  "1"  /* Restriction of Left Pulmonary Artery, Open Approach */
"02VR3CZ"  =  "1"  /* Restriction of Left Pulmonary Artery with Extraluminal Device, Percutaneous Approach */
"02VR3DT"  =  "1"  /* Restriction of Ductus Arteriosus with Intraluminal Device, Percutaneous Approach */
"02VR3DZ"  =  "1"  /* Restriction of Left Pulmonary Artery with Intraluminal Device, Percutaneous Approach */
"02VR3ZT"  =  "1"  /* Restriction of Ductus Arteriosus, Percutaneous Approach */
"02VR3ZZ"  =  "1"  /* Restriction of Left Pulmonary Artery, Percutaneous Approach */
"02VR4CZ"  =  "1"  /* Restriction of Left Pulmonary Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"02VR4DT"  =  "1"  /* Restriction of Ductus Arteriosus with Intraluminal Device, Percutaneous Endoscopic Approach */
"02VR4DZ"  =  "1"  /* Restriction of Left Pulmonary Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"02VR4ZT"  =  "1"  /* Restriction of Ductus Arteriosus, Percutaneous Endoscopic Approach */
"02VR4ZZ"  =  "1"  /* Restriction of Left Pulmonary Artery, Percutaneous Endoscopic Approach */
"02VS0DZ"  =  "1"  /* Restriction of Right Pulmonary Vein with Intraluminal Device, Open Approach */
"02VS0ZZ"  =  "1"  /* Restriction of Right Pulmonary Vein, Open Approach */
"02VS3DZ"  =  "1"  /* Restriction of Right Pulmonary Vein with Intraluminal Device, Percutaneous Approach */
"02VS3ZZ"  =  "1"  /* Restriction of Right Pulmonary Vein, Percutaneous Approach */
"02VS4DZ"  =  "1"  /* Restriction of Right Pulmonary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"02VS4ZZ"  =  "1"  /* Restriction of Right Pulmonary Vein, Percutaneous Endoscopic Approach */
"02VT0DZ"  =  "1"  /* Restriction of Left Pulmonary Vein with Intraluminal Device, Open Approach */
"02VT0ZZ"  =  "1"  /* Restriction of Left Pulmonary Vein, Open Approach */
"02VT3DZ"  =  "1"  /* Restriction of Left Pulmonary Vein with Intraluminal Device, Percutaneous Approach */
"02VT3ZZ"  =  "1"  /* Restriction of Left Pulmonary Vein, Percutaneous Approach */
"02VT4DZ"  =  "1"  /* Restriction of Left Pulmonary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"02VT4ZZ"  =  "1"  /* Restriction of Left Pulmonary Vein, Percutaneous Endoscopic Approach */
"02VV0CZ"  =  "1"  /* Restriction of Superior Vena Cava with Extraluminal Device, Open Approach */
"02VV0DZ"  =  "1"  /* Restriction of Superior Vena Cava with Intraluminal Device, Open Approach */
"02VV0ZZ"  =  "1"  /* Restriction of Superior Vena Cava, Open Approach */
"02VV3CZ"  =  "1"  /* Restriction of Superior Vena Cava with Extraluminal Device, Percutaneous Approach */
"02VV3DZ"  =  "1"  /* Restriction of Superior Vena Cava with Intraluminal Device, Percutaneous Approach */
"02VV3ZZ"  =  "1"  /* Restriction of Superior Vena Cava, Percutaneous Approach */
"02VV4CZ"  =  "1"  /* Restriction of Superior Vena Cava with Extraluminal Device, Percutaneous Endoscopic Approach */
"02VV4DZ"  =  "1"  /* Restriction of Superior Vena Cava with Intraluminal Device, Percutaneous Endoscopic Approach */
"02VV4ZZ"  =  "1"  /* Restriction of Superior Vena Cava, Percutaneous Endoscopic Approach */
"02VW0ZZ"  =  "1"  /* Restriction of Thoracic Aorta, Open Approach */
"02VW3DZ"  =  "1"  /* Restriction of Thoracic Aorta with Intraluminal Device, Percutaneous Approach */
"02VW3ZZ"  =  "1"  /* Restriction of Thoracic Aorta, Percutaneous Approach */
"02VW4DZ"  =  "1"  /* Restriction of Thoracic Aorta with Intraluminal Device, Percutaneous Endoscopic Approach */
"02VW4ZZ"  =  "1"  /* Restriction of Thoracic Aorta, Percutaneous Endoscopic Approach */
"02WA02Z"  =  "1"  /* Revision of Monitoring Device in Heart, Open Approach */
"02WA03Z"  =  "1"  /* Revision of Infusion Device in Heart, Open Approach */
"02WA07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Heart, Open Approach */
"02WA08Z"  =  "1"  /* Revision of Zooplastic Tissue in Heart, Open Approach */
"02WA0CZ"  =  "1"  /* Revision of Extraluminal Device in Heart, Open Approach */
"02WA0DZ"  =  "1"  /* Revision of Intraluminal Device in Heart, Open Approach */
"02WA0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Heart, Open Approach */
"02WA32Z"  =  "1"  /* Revision of Monitoring Device in Heart, Percutaneous Approach */
"02WA33Z"  =  "1"  /* Revision of Infusion Device in Heart, Percutaneous Approach */
"02WA37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Heart, Percutaneous Approach */
"02WA38Z"  =  "1"  /* Revision of Zooplastic Tissue in Heart, Percutaneous Approach */
"02WA3CZ"  =  "1"  /* Revision of Extraluminal Device in Heart, Percutaneous Approach */
"02WA3DZ"  =  "1"  /* Revision of Intraluminal Device in Heart, Percutaneous Approach */
"02WA3JZ"  =  "1"  /* Revision of Synthetic Substitute in Heart, Percutaneous Approach */
"02WA3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Heart, Percutaneous Approach */
"02WA42Z"  =  "1"  /* Revision of Monitoring Device in Heart, Percutaneous Endoscopic Approach */
"02WA43Z"  =  "1"  /* Revision of Infusion Device in Heart, Percutaneous Endoscopic Approach */
"02WA47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Heart, Percutaneous Endoscopic Approach */
"02WA48Z"  =  "1"  /* Revision of Zooplastic Tissue in Heart, Percutaneous Endoscopic Approach */
"02WA4CZ"  =  "1"  /* Revision of Extraluminal Device in Heart, Percutaneous Endoscopic Approach */
"02WA4DZ"  =  "1"  /* Revision of Intraluminal Device in Heart, Percutaneous Endoscopic Approach */
"02WA4JZ"  =  "1"  /* Revision of Synthetic Substitute in Heart, Percutaneous Endoscopic Approach */
"02WA4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Heart, Percutaneous Endoscopic Approach */
"02WY02Z"  =  "1"  /* Revision of Monitoring Device in Great Vessel, Open Approach */
"02WY03Z"  =  "1"  /* Revision of Infusion Device in Great Vessel, Open Approach */
"02WY07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Great Vessel, Open Approach */
"02WY08Z"  =  "1"  /* Revision of Zooplastic Tissue in Great Vessel, Open Approach */
"02WY0CZ"  =  "1"  /* Revision of Extraluminal Device in Great Vessel, Open Approach */
"02WY0DZ"  =  "1"  /* Revision of Intraluminal Device in Great Vessel, Open Approach */
"02WY0JZ"  =  "1"  /* Revision of Synthetic Substitute in Great Vessel, Open Approach */
"02WY0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Great Vessel, Open Approach */
"02WY32Z"  =  "1"  /* Revision of Monitoring Device in Great Vessel, Percutaneous Approach */
"02WY33Z"  =  "1"  /* Revision of Infusion Device in Great Vessel, Percutaneous Approach */
"02WY37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Great Vessel, Percutaneous Approach */
"02WY38Z"  =  "1"  /* Revision of Zooplastic Tissue in Great Vessel, Percutaneous Approach */
"02WY3CZ"  =  "1"  /* Revision of Extraluminal Device in Great Vessel, Percutaneous Approach */
"02WY3DZ"  =  "1"  /* Revision of Intraluminal Device in Great Vessel, Percutaneous Approach */
"02WY3JZ"  =  "1"  /* Revision of Synthetic Substitute in Great Vessel, Percutaneous Approach */
"02WY3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Great Vessel, Percutaneous Approach */
"02WY42Z"  =  "1"  /* Revision of Monitoring Device in Great Vessel, Percutaneous Endoscopic Approach */
"02WY43Z"  =  "1"  /* Revision of Infusion Device in Great Vessel, Percutaneous Endoscopic Approach */
"02WY47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Great Vessel, Percutaneous Endoscopic Approach */
"02WY48Z"  =  "1"  /* Revision of Zooplastic Tissue in Great Vessel, Percutaneous Endoscopic Approach */
"02WY4CZ"  =  "1"  /* Revision of Extraluminal Device in Great Vessel, Percutaneous Endoscopic Approach */
"02WY4DZ"  =  "1"  /* Revision of Intraluminal Device in Great Vessel, Percutaneous Endoscopic Approach */
"02WY4JZ"  =  "1"  /* Revision of Synthetic Substitute in Great Vessel, Percutaneous Endoscopic Approach */
"02WY4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Great Vessel, Percutaneous Endoscopic Approach */
"039Y0ZZ"  =  "1"  /* Drainage of Upper Artery, Open Approach */
"039Y3ZZ"  =  "1"  /* Drainage of Upper Artery, Percutaneous Approach */
"039Y4ZZ"  =  "1"  /* Drainage of Upper Artery, Percutaneous Endoscopic Approach */
"03C00ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Mammary Artery, Open Approach */
"03C03ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Mammary Artery, Percutaneous Approach */
"03C04ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Mammary Artery, Percutaneous Endoscopic Approach */
"03C10ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Mammary Artery, Open Approach */
"03C13ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Mammary Artery, Percutaneous Approach */
"03C14ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Mammary Artery, Percutaneous Endoscopic Approach */
"03C20ZZ"  =  "1"  /* Extirpation of Matter from Innominate Artery, Open Approach */
"03C23ZZ"  =  "1"  /* Extirpation of Matter from Innominate Artery, Percutaneous Approach */
"03C24ZZ"  =  "1"  /* Extirpation of Matter from Innominate Artery, Percutaneous Endoscopic Approach */
"03C30ZZ"  =  "1"  /* Extirpation of Matter from Right Subclavian Artery, Open Approach */
"03C33ZZ"  =  "1"  /* Extirpation of Matter from Right Subclavian Artery, Percutaneous Approach */
"03C34ZZ"  =  "1"  /* Extirpation of Matter from Right Subclavian Artery, Percutaneous Endoscopic Approach */
"03C40ZZ"  =  "1"  /* Extirpation of Matter from Left Subclavian Artery, Open Approach */
"03C43ZZ"  =  "1"  /* Extirpation of Matter from Left Subclavian Artery, Percutaneous Approach */
"03C44ZZ"  =  "1"  /* Extirpation of Matter from Left Subclavian Artery, Percutaneous Endoscopic Approach */
"03C50ZZ"  =  "1"  /* Extirpation of Matter from Right Axillary Artery, Open Approach */
"03C53ZZ"  =  "1"  /* Extirpation of Matter from Right Axillary Artery, Percutaneous Approach */
"03C54ZZ"  =  "1"  /* Extirpation of Matter from Right Axillary Artery, Percutaneous Endoscopic Approach */
"03C60ZZ"  =  "1"  /* Extirpation of Matter from Left Axillary Artery, Open Approach */
"03C63ZZ"  =  "1"  /* Extirpation of Matter from Left Axillary Artery, Percutaneous Approach */
"03C64ZZ"  =  "1"  /* Extirpation of Matter from Left Axillary Artery, Percutaneous Endoscopic Approach */
"03C70ZZ"  =  "1"  /* Extirpation of Matter from Right Brachial Artery, Open Approach */
"03C73ZZ"  =  "1"  /* Extirpation of Matter from Right Brachial Artery, Percutaneous Approach */
"03C74ZZ"  =  "1"  /* Extirpation of Matter from Right Brachial Artery, Percutaneous Endoscopic Approach */
"03C80ZZ"  =  "1"  /* Extirpation of Matter from Left Brachial Artery, Open Approach */
"03C83ZZ"  =  "1"  /* Extirpation of Matter from Left Brachial Artery, Percutaneous Approach */
"03C84ZZ"  =  "1"  /* Extirpation of Matter from Left Brachial Artery, Percutaneous Endoscopic Approach */
"03C90ZZ"  =  "1"  /* Extirpation of Matter from Right Ulnar Artery, Open Approach */
"03C93ZZ"  =  "1"  /* Extirpation of Matter from Right Ulnar Artery, Percutaneous Approach */
"03C94ZZ"  =  "1"  /* Extirpation of Matter from Right Ulnar Artery, Percutaneous Endoscopic Approach */
"03CA0ZZ"  =  "1"  /* Extirpation of Matter from Left Ulnar Artery, Open Approach */
"03CA3ZZ"  =  "1"  /* Extirpation of Matter from Left Ulnar Artery, Percutaneous Approach */
"03CA4ZZ"  =  "1"  /* Extirpation of Matter from Left Ulnar Artery, Percutaneous Endoscopic Approach */
"03CB0ZZ"  =  "1"  /* Extirpation of Matter from Right Radial Artery, Open Approach */
"03CB3ZZ"  =  "1"  /* Extirpation of Matter from Right Radial Artery, Percutaneous Approach */
"03CB4ZZ"  =  "1"  /* Extirpation of Matter from Right Radial Artery, Percutaneous Endoscopic Approach */
"03CC0ZZ"  =  "1"  /* Extirpation of Matter from Left Radial Artery, Open Approach */
"03CC3ZZ"  =  "1"  /* Extirpation of Matter from Left Radial Artery, Percutaneous Approach */
"03CC4ZZ"  =  "1"  /* Extirpation of Matter from Left Radial Artery, Percutaneous Endoscopic Approach */
"03CD0ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Artery, Open Approach */
"03CD3ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Artery, Percutaneous Approach */
"03CD4ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Artery, Percutaneous Endoscopic Approach */
"03CF0ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Artery, Open Approach */
"03CF3ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Artery, Percutaneous Approach */
"03CF4ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Artery, Percutaneous Endoscopic Approach */
"03CG0ZZ"  =  "1"  /* Extirpation of Matter from Intracranial Artery, Open Approach */
"03CH0ZZ"  =  "1"  /* Extirpation of Matter from Right Common Carotid Artery, Open Approach */
"03CJ0ZZ"  =  "1"  /* Extirpation of Matter from Left Common Carotid Artery, Open Approach */
"03CK0ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Carotid Artery, Open Approach */
"03CL0ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Carotid Artery, Open Approach */
"03CM0ZZ"  =  "1"  /* Extirpation of Matter from Right External Carotid Artery, Open Approach */
"03CN0ZZ"  =  "1"  /* Extirpation of Matter from Left External Carotid Artery, Open Approach */
"03CP0ZZ"  =  "1"  /* Extirpation of Matter from Right Vertebral Artery, Open Approach */
"03CQ0ZZ"  =  "1"  /* Extirpation of Matter from Left Vertebral Artery, Open Approach */
"03CR0ZZ"  =  "1"  /* Extirpation of Matter from Face Artery, Open Approach */
"03CS0ZZ"  =  "1"  /* Extirpation of Matter from Right Temporal Artery, Open Approach */
"03CT0ZZ"  =  "1"  /* Extirpation of Matter from Left Temporal Artery, Open Approach */
"03CU0ZZ"  =  "1"  /* Extirpation of Matter from Right Thyroid Artery, Open Approach */
"03CV0ZZ"  =  "1"  /* Extirpation of Matter from Left Thyroid Artery, Open Approach */
"03CY0ZZ"  =  "1"  /* Extirpation of Matter from Upper Artery, Open Approach */
"03CY3ZZ"  =  "1"  /* Extirpation of Matter from Upper Artery, Percutaneous Approach */
"03CY4ZZ"  =  "1"  /* Extirpation of Matter from Upper Artery, Percutaneous Endoscopic Approach */
"03HY02Z"  =  "1"  /* Insertion of Monitoring Device into Upper Artery, Open Approach */
"03HY32Z"  =  "1"  /* Insertion of Monitoring Device into Upper Artery, Percutaneous Approach */
"03HY42Z"  =  "1"  /* Insertion of Monitoring Device into Upper Artery, Percutaneous Endoscopic Approach */
"03L00CZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery with Extraluminal Device, Open Approach */
"03L00DZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery with Intraluminal Device, Open Approach */
"03L00ZZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery, Open Approach */
"03L03CZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery with Extraluminal Device, Percutaneous Approach */
"03L03DZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery with Intraluminal Device, Percutaneous Approach */
"03L03ZZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery, Percutaneous Approach */
"03L04CZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03L04DZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03L04ZZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery, Percutaneous Endoscopic Approach */
"03L10CZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery with Extraluminal Device, Open Approach */
"03L10DZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery with Intraluminal Device, Open Approach */
"03L10ZZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery, Open Approach */
"03L13CZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery with Extraluminal Device, Percutaneous Approach */
"03L13DZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery with Intraluminal Device, Percutaneous Approach */
"03L13ZZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery, Percutaneous Approach */
"03L14CZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03L14DZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03L14ZZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery, Percutaneous Endoscopic Approach */
"03L20CZ"  =  "1"  /* Occlusion of Innominate Artery with Extraluminal Device, Open Approach */
"03L20DZ"  =  "1"  /* Occlusion of Innominate Artery with Intraluminal Device, Open Approach */
"03L20ZZ"  =  "1"  /* Occlusion of Innominate Artery, Open Approach */
"03L23CZ"  =  "1"  /* Occlusion of Innominate Artery with Extraluminal Device, Percutaneous Approach */
"03L23DZ"  =  "1"  /* Occlusion of Innominate Artery with Intraluminal Device, Percutaneous Approach */
"03L23ZZ"  =  "1"  /* Occlusion of Innominate Artery, Percutaneous Approach */
"03L24CZ"  =  "1"  /* Occlusion of Innominate Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03L24DZ"  =  "1"  /* Occlusion of Innominate Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03L24ZZ"  =  "1"  /* Occlusion of Innominate Artery, Percutaneous Endoscopic Approach */
"03L30CZ"  =  "1"  /* Occlusion of Right Subclavian Artery with Extraluminal Device, Open Approach */
"03L30DZ"  =  "1"  /* Occlusion of Right Subclavian Artery with Intraluminal Device, Open Approach */
"03L30ZZ"  =  "1"  /* Occlusion of Right Subclavian Artery, Open Approach */
"03L33CZ"  =  "1"  /* Occlusion of Right Subclavian Artery with Extraluminal Device, Percutaneous Approach */
"03L33DZ"  =  "1"  /* Occlusion of Right Subclavian Artery with Intraluminal Device, Percutaneous Approach */
"03L33ZZ"  =  "1"  /* Occlusion of Right Subclavian Artery, Percutaneous Approach */
"03L34CZ"  =  "1"  /* Occlusion of Right Subclavian Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03L34DZ"  =  "1"  /* Occlusion of Right Subclavian Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03L34ZZ"  =  "1"  /* Occlusion of Right Subclavian Artery, Percutaneous Endoscopic Approach */
"03L40CZ"  =  "1"  /* Occlusion of Left Subclavian Artery with Extraluminal Device, Open Approach */
"03L40DZ"  =  "1"  /* Occlusion of Left Subclavian Artery with Intraluminal Device, Open Approach */
"03L40ZZ"  =  "1"  /* Occlusion of Left Subclavian Artery, Open Approach */
"03L43CZ"  =  "1"  /* Occlusion of Left Subclavian Artery with Extraluminal Device, Percutaneous Approach */
"03L43DZ"  =  "1"  /* Occlusion of Left Subclavian Artery with Intraluminal Device, Percutaneous Approach */
"03L43ZZ"  =  "1"  /* Occlusion of Left Subclavian Artery, Percutaneous Approach */
"03L44CZ"  =  "1"  /* Occlusion of Left Subclavian Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03L44DZ"  =  "1"  /* Occlusion of Left Subclavian Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03L44ZZ"  =  "1"  /* Occlusion of Left Subclavian Artery, Percutaneous Endoscopic Approach */
"03L50CZ"  =  "1"  /* Occlusion of Right Axillary Artery with Extraluminal Device, Open Approach */
"03L50DZ"  =  "1"  /* Occlusion of Right Axillary Artery with Intraluminal Device, Open Approach */
"03L50ZZ"  =  "1"  /* Occlusion of Right Axillary Artery, Open Approach */
"03L53CZ"  =  "1"  /* Occlusion of Right Axillary Artery with Extraluminal Device, Percutaneous Approach */
"03L53DZ"  =  "1"  /* Occlusion of Right Axillary Artery with Intraluminal Device, Percutaneous Approach */
"03L53ZZ"  =  "1"  /* Occlusion of Right Axillary Artery, Percutaneous Approach */
"03L54CZ"  =  "1"  /* Occlusion of Right Axillary Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03L54DZ"  =  "1"  /* Occlusion of Right Axillary Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03L54ZZ"  =  "1"  /* Occlusion of Right Axillary Artery, Percutaneous Endoscopic Approach */
"03L60CZ"  =  "1"  /* Occlusion of Left Axillary Artery with Extraluminal Device, Open Approach */
"03L60DZ"  =  "1"  /* Occlusion of Left Axillary Artery with Intraluminal Device, Open Approach */
"03L60ZZ"  =  "1"  /* Occlusion of Left Axillary Artery, Open Approach */
"03L63CZ"  =  "1"  /* Occlusion of Left Axillary Artery with Extraluminal Device, Percutaneous Approach */
"03L63DZ"  =  "1"  /* Occlusion of Left Axillary Artery with Intraluminal Device, Percutaneous Approach */
"03L63ZZ"  =  "1"  /* Occlusion of Left Axillary Artery, Percutaneous Approach */
"03L64CZ"  =  "1"  /* Occlusion of Left Axillary Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03L64DZ"  =  "1"  /* Occlusion of Left Axillary Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03L64ZZ"  =  "1"  /* Occlusion of Left Axillary Artery, Percutaneous Endoscopic Approach */
"03L70CZ"  =  "1"  /* Occlusion of Right Brachial Artery with Extraluminal Device, Open Approach */
"03L70DZ"  =  "1"  /* Occlusion of Right Brachial Artery with Intraluminal Device, Open Approach */
"03L70ZZ"  =  "1"  /* Occlusion of Right Brachial Artery, Open Approach */
"03L73CZ"  =  "1"  /* Occlusion of Right Brachial Artery with Extraluminal Device, Percutaneous Approach */
"03L73DZ"  =  "1"  /* Occlusion of Right Brachial Artery with Intraluminal Device, Percutaneous Approach */
"03L73ZZ"  =  "1"  /* Occlusion of Right Brachial Artery, Percutaneous Approach */
"03L74CZ"  =  "1"  /* Occlusion of Right Brachial Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03L74DZ"  =  "1"  /* Occlusion of Right Brachial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03L74ZZ"  =  "1"  /* Occlusion of Right Brachial Artery, Percutaneous Endoscopic Approach */
"03L80CZ"  =  "1"  /* Occlusion of Left Brachial Artery with Extraluminal Device, Open Approach */
"03L80DZ"  =  "1"  /* Occlusion of Left Brachial Artery with Intraluminal Device, Open Approach */
"03L80ZZ"  =  "1"  /* Occlusion of Left Brachial Artery, Open Approach */
"03L83CZ"  =  "1"  /* Occlusion of Left Brachial Artery with Extraluminal Device, Percutaneous Approach */
"03L83DZ"  =  "1"  /* Occlusion of Left Brachial Artery with Intraluminal Device, Percutaneous Approach */
"03L83ZZ"  =  "1"  /* Occlusion of Left Brachial Artery, Percutaneous Approach */
"03L84CZ"  =  "1"  /* Occlusion of Left Brachial Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03L84DZ"  =  "1"  /* Occlusion of Left Brachial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03L84ZZ"  =  "1"  /* Occlusion of Left Brachial Artery, Percutaneous Endoscopic Approach */
"03L90CZ"  =  "1"  /* Occlusion of Right Ulnar Artery with Extraluminal Device, Open Approach */
"03L90DZ"  =  "1"  /* Occlusion of Right Ulnar Artery with Intraluminal Device, Open Approach */
"03L90ZZ"  =  "1"  /* Occlusion of Right Ulnar Artery, Open Approach */
"03L93CZ"  =  "1"  /* Occlusion of Right Ulnar Artery with Extraluminal Device, Percutaneous Approach */
"03L93DZ"  =  "1"  /* Occlusion of Right Ulnar Artery with Intraluminal Device, Percutaneous Approach */
"03L93ZZ"  =  "1"  /* Occlusion of Right Ulnar Artery, Percutaneous Approach */
"03L94CZ"  =  "1"  /* Occlusion of Right Ulnar Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03L94DZ"  =  "1"  /* Occlusion of Right Ulnar Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03L94ZZ"  =  "1"  /* Occlusion of Right Ulnar Artery, Percutaneous Endoscopic Approach */
"03LA0CZ"  =  "1"  /* Occlusion of Left Ulnar Artery with Extraluminal Device, Open Approach */
"03LA0DZ"  =  "1"  /* Occlusion of Left Ulnar Artery with Intraluminal Device, Open Approach */
"03LA0ZZ"  =  "1"  /* Occlusion of Left Ulnar Artery, Open Approach */
"03LA3CZ"  =  "1"  /* Occlusion of Left Ulnar Artery with Extraluminal Device, Percutaneous Approach */
"03LA3DZ"  =  "1"  /* Occlusion of Left Ulnar Artery with Intraluminal Device, Percutaneous Approach */
"03LA3ZZ"  =  "1"  /* Occlusion of Left Ulnar Artery, Percutaneous Approach */
"03LA4CZ"  =  "1"  /* Occlusion of Left Ulnar Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LA4DZ"  =  "1"  /* Occlusion of Left Ulnar Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LA4ZZ"  =  "1"  /* Occlusion of Left Ulnar Artery, Percutaneous Endoscopic Approach */
"03LB0CZ"  =  "1"  /* Occlusion of Right Radial Artery with Extraluminal Device, Open Approach */
"03LB0DZ"  =  "1"  /* Occlusion of Right Radial Artery with Intraluminal Device, Open Approach */
"03LB0ZZ"  =  "1"  /* Occlusion of Right Radial Artery, Open Approach */
"03LB3CZ"  =  "1"  /* Occlusion of Right Radial Artery with Extraluminal Device, Percutaneous Approach */
"03LB3DZ"  =  "1"  /* Occlusion of Right Radial Artery with Intraluminal Device, Percutaneous Approach */
"03LB3ZZ"  =  "1"  /* Occlusion of Right Radial Artery, Percutaneous Approach */
"03LB4CZ"  =  "1"  /* Occlusion of Right Radial Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LB4DZ"  =  "1"  /* Occlusion of Right Radial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LB4ZZ"  =  "1"  /* Occlusion of Right Radial Artery, Percutaneous Endoscopic Approach */
"03LC0CZ"  =  "1"  /* Occlusion of Left Radial Artery with Extraluminal Device, Open Approach */
"03LC0DZ"  =  "1"  /* Occlusion of Left Radial Artery with Intraluminal Device, Open Approach */
"03LC0ZZ"  =  "1"  /* Occlusion of Left Radial Artery, Open Approach */
"03LC3CZ"  =  "1"  /* Occlusion of Left Radial Artery with Extraluminal Device, Percutaneous Approach */
"03LC3DZ"  =  "1"  /* Occlusion of Left Radial Artery with Intraluminal Device, Percutaneous Approach */
"03LC3ZZ"  =  "1"  /* Occlusion of Left Radial Artery, Percutaneous Approach */
"03LC4CZ"  =  "1"  /* Occlusion of Left Radial Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LC4DZ"  =  "1"  /* Occlusion of Left Radial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LC4ZZ"  =  "1"  /* Occlusion of Left Radial Artery, Percutaneous Endoscopic Approach */
"03LD0CZ"  =  "1"  /* Occlusion of Right Hand Artery with Extraluminal Device, Open Approach */
"03LD0DZ"  =  "1"  /* Occlusion of Right Hand Artery with Intraluminal Device, Open Approach */
"03LD0ZZ"  =  "1"  /* Occlusion of Right Hand Artery, Open Approach */
"03LD3CZ"  =  "1"  /* Occlusion of Right Hand Artery with Extraluminal Device, Percutaneous Approach */
"03LD3DZ"  =  "1"  /* Occlusion of Right Hand Artery with Intraluminal Device, Percutaneous Approach */
"03LD3ZZ"  =  "1"  /* Occlusion of Right Hand Artery, Percutaneous Approach */
"03LD4CZ"  =  "1"  /* Occlusion of Right Hand Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LD4DZ"  =  "1"  /* Occlusion of Right Hand Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LD4ZZ"  =  "1"  /* Occlusion of Right Hand Artery, Percutaneous Endoscopic Approach */
"03LF0CZ"  =  "1"  /* Occlusion of Left Hand Artery with Extraluminal Device, Open Approach */
"03LF0DZ"  =  "1"  /* Occlusion of Left Hand Artery with Intraluminal Device, Open Approach */
"03LF0ZZ"  =  "1"  /* Occlusion of Left Hand Artery, Open Approach */
"03LF3CZ"  =  "1"  /* Occlusion of Left Hand Artery with Extraluminal Device, Percutaneous Approach */
"03LF3DZ"  =  "1"  /* Occlusion of Left Hand Artery with Intraluminal Device, Percutaneous Approach */
"03LF3ZZ"  =  "1"  /* Occlusion of Left Hand Artery, Percutaneous Approach */
"03LF4CZ"  =  "1"  /* Occlusion of Left Hand Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LF4DZ"  =  "1"  /* Occlusion of Left Hand Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LF4ZZ"  =  "1"  /* Occlusion of Left Hand Artery, Percutaneous Endoscopic Approach */
"03LG0BZ"  =  "1"  /* Occlusion of Intracranial Artery with Bioactive Intraluminal Device, Open Approach */
"03LG0CZ"  =  "1"  /* Occlusion of Intracranial Artery with Extraluminal Device, Open Approach */
"03LG0DZ"  =  "1"  /* Occlusion of Intracranial Artery with Intraluminal Device, Open Approach */
"03LG0ZZ"  =  "1"  /* Occlusion of Intracranial Artery, Open Approach */
"03LG3BZ"  =  "1"  /* Occlusion of Intracranial Artery with Bioactive Intraluminal Device, Percutaneous Approach */
"03LG3CZ"  =  "1"  /* Occlusion of Intracranial Artery with Extraluminal Device, Percutaneous Approach */
"03LG3DZ"  =  "1"  /* Occlusion of Intracranial Artery with Intraluminal Device, Percutaneous Approach */
"03LG3ZZ"  =  "1"  /* Occlusion of Intracranial Artery, Percutaneous Approach */
"03LG4BZ"  =  "1"  /* Occlusion of Intracranial Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach */
"03LG4CZ"  =  "1"  /* Occlusion of Intracranial Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LG4DZ"  =  "1"  /* Occlusion of Intracranial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LG4ZZ"  =  "1"  /* Occlusion of Intracranial Artery, Percutaneous Endoscopic Approach */
"03LH0BZ"  =  "1"  /* Occlusion of Right Common Carotid Artery with Bioactive Intraluminal Device, Open Approach */
"03LH0CZ"  =  "1"  /* Occlusion of Right Common Carotid Artery with Extraluminal Device, Open Approach */
"03LH0DZ"  =  "1"  /* Occlusion of Right Common Carotid Artery with Intraluminal Device, Open Approach */
"03LH0ZZ"  =  "1"  /* Occlusion of Right Common Carotid Artery, Open Approach */
"03LH3BZ"  =  "1"  /* Occlusion of Right Common Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach */
"03LH3CZ"  =  "1"  /* Occlusion of Right Common Carotid Artery with Extraluminal Device, Percutaneous Approach */
"03LH3DZ"  =  "1"  /* Occlusion of Right Common Carotid Artery with Intraluminal Device, Percutaneous Approach */
"03LH3ZZ"  =  "1"  /* Occlusion of Right Common Carotid Artery, Percutaneous Approach */
"03LH4BZ"  =  "1"  /* Occlusion of Right Common Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach */
"03LH4CZ"  =  "1"  /* Occlusion of Right Common Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LH4DZ"  =  "1"  /* Occlusion of Right Common Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LH4ZZ"  =  "1"  /* Occlusion of Right Common Carotid Artery, Percutaneous Endoscopic Approach */
"03LJ0BZ"  =  "1"  /* Occlusion of Left Common Carotid Artery with Bioactive Intraluminal Device, Open Approach */
"03LJ0CZ"  =  "1"  /* Occlusion of Left Common Carotid Artery with Extraluminal Device, Open Approach */
"03LJ0DZ"  =  "1"  /* Occlusion of Left Common Carotid Artery with Intraluminal Device, Open Approach */
"03LJ0ZZ"  =  "1"  /* Occlusion of Left Common Carotid Artery, Open Approach */
"03LJ3BZ"  =  "1"  /* Occlusion of Left Common Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach */
"03LJ3CZ"  =  "1"  /* Occlusion of Left Common Carotid Artery with Extraluminal Device, Percutaneous Approach */
"03LJ3DZ"  =  "1"  /* Occlusion of Left Common Carotid Artery with Intraluminal Device, Percutaneous Approach */
"03LJ3ZZ"  =  "1"  /* Occlusion of Left Common Carotid Artery, Percutaneous Approach */
"03LJ4BZ"  =  "1"  /* Occlusion of Left Common Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach */
"03LJ4CZ"  =  "1"  /* Occlusion of Left Common Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LJ4DZ"  =  "1"  /* Occlusion of Left Common Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LJ4ZZ"  =  "1"  /* Occlusion of Left Common Carotid Artery, Percutaneous Endoscopic Approach */
"03LK0BZ"  =  "1"  /* Occlusion of Right Internal Carotid Artery with Bioactive Intraluminal Device, Open Approach */
"03LK0CZ"  =  "1"  /* Occlusion of Right Internal Carotid Artery with Extraluminal Device, Open Approach */
"03LK0DZ"  =  "1"  /* Occlusion of Right Internal Carotid Artery with Intraluminal Device, Open Approach */
"03LK0ZZ"  =  "1"  /* Occlusion of Right Internal Carotid Artery, Open Approach */
"03LK3BZ"  =  "1"  /* Occlusion of Right Internal Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach */
"03LK3CZ"  =  "1"  /* Occlusion of Right Internal Carotid Artery with Extraluminal Device, Percutaneous Approach */
"03LK3DZ"  =  "1"  /* Occlusion of Right Internal Carotid Artery with Intraluminal Device, Percutaneous Approach */
"03LK3ZZ"  =  "1"  /* Occlusion of Right Internal Carotid Artery, Percutaneous Approach */
"03LK4BZ"  =  "1"  /* Occlusion of Right Internal Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach */
"03LK4CZ"  =  "1"  /* Occlusion of Right Internal Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LK4DZ"  =  "1"  /* Occlusion of Right Internal Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LK4ZZ"  =  "1"  /* Occlusion of Right Internal Carotid Artery, Percutaneous Endoscopic Approach */
"03LL0BZ"  =  "1"  /* Occlusion of Left Internal Carotid Artery with Bioactive Intraluminal Device, Open Approach */
"03LL0CZ"  =  "1"  /* Occlusion of Left Internal Carotid Artery with Extraluminal Device, Open Approach */
"03LL0DZ"  =  "1"  /* Occlusion of Left Internal Carotid Artery with Intraluminal Device, Open Approach */
"03LL0ZZ"  =  "1"  /* Occlusion of Left Internal Carotid Artery, Open Approach */
"03LL3BZ"  =  "1"  /* Occlusion of Left Internal Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach */
"03LL3CZ"  =  "1"  /* Occlusion of Left Internal Carotid Artery with Extraluminal Device, Percutaneous Approach */
"03LL3DZ"  =  "1"  /* Occlusion of Left Internal Carotid Artery with Intraluminal Device, Percutaneous Approach */
"03LL3ZZ"  =  "1"  /* Occlusion of Left Internal Carotid Artery, Percutaneous Approach */
"03LL4BZ"  =  "1"  /* Occlusion of Left Internal Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach */
"03LL4CZ"  =  "1"  /* Occlusion of Left Internal Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LL4DZ"  =  "1"  /* Occlusion of Left Internal Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LL4ZZ"  =  "1"  /* Occlusion of Left Internal Carotid Artery, Percutaneous Endoscopic Approach */
"03LM0BZ"  =  "1"  /* Occlusion of Right External Carotid Artery with Bioactive Intraluminal Device, Open Approach */
"03LM0CZ"  =  "1"  /* Occlusion of Right External Carotid Artery with Extraluminal Device, Open Approach */
"03LM0DZ"  =  "1"  /* Occlusion of Right External Carotid Artery with Intraluminal Device, Open Approach */
"03LM0ZZ"  =  "1"  /* Occlusion of Right External Carotid Artery, Open Approach */
"03LM3BZ"  =  "1"  /* Occlusion of Right External Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach */
"03LM3CZ"  =  "1"  /* Occlusion of Right External Carotid Artery with Extraluminal Device, Percutaneous Approach */
"03LM3DZ"  =  "1"  /* Occlusion of Right External Carotid Artery with Intraluminal Device, Percutaneous Approach */
"03LM3ZZ"  =  "1"  /* Occlusion of Right External Carotid Artery, Percutaneous Approach */
"03LM4BZ"  =  "1"  /* Occlusion of Right External Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach */
"03LM4CZ"  =  "1"  /* Occlusion of Right External Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LM4DZ"  =  "1"  /* Occlusion of Right External Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LM4ZZ"  =  "1"  /* Occlusion of Right External Carotid Artery, Percutaneous Endoscopic Approach */
"03LN0BZ"  =  "1"  /* Occlusion of Left External Carotid Artery with Bioactive Intraluminal Device, Open Approach */
"03LN0CZ"  =  "1"  /* Occlusion of Left External Carotid Artery with Extraluminal Device, Open Approach */
"03LN0DZ"  =  "1"  /* Occlusion of Left External Carotid Artery with Intraluminal Device, Open Approach */
"03LN0ZZ"  =  "1"  /* Occlusion of Left External Carotid Artery, Open Approach */
"03LN3BZ"  =  "1"  /* Occlusion of Left External Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach */
"03LN3CZ"  =  "1"  /* Occlusion of Left External Carotid Artery with Extraluminal Device, Percutaneous Approach */
"03LN3DZ"  =  "1"  /* Occlusion of Left External Carotid Artery with Intraluminal Device, Percutaneous Approach */
"03LN3ZZ"  =  "1"  /* Occlusion of Left External Carotid Artery, Percutaneous Approach */
"03LN4BZ"  =  "1"  /* Occlusion of Left External Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach */
"03LN4CZ"  =  "1"  /* Occlusion of Left External Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LN4DZ"  =  "1"  /* Occlusion of Left External Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LN4ZZ"  =  "1"  /* Occlusion of Left External Carotid Artery, Percutaneous Endoscopic Approach */
"03LP0BZ"  =  "1"  /* Occlusion of Right Vertebral Artery with Bioactive Intraluminal Device, Open Approach */
"03LP0CZ"  =  "1"  /* Occlusion of Right Vertebral Artery with Extraluminal Device, Open Approach */
"03LP0DZ"  =  "1"  /* Occlusion of Right Vertebral Artery with Intraluminal Device, Open Approach */
"03LP0ZZ"  =  "1"  /* Occlusion of Right Vertebral Artery, Open Approach */
"03LP3BZ"  =  "1"  /* Occlusion of Right Vertebral Artery with Bioactive Intraluminal Device, Percutaneous Approach */
"03LP3CZ"  =  "1"  /* Occlusion of Right Vertebral Artery with Extraluminal Device, Percutaneous Approach */
"03LP3DZ"  =  "1"  /* Occlusion of Right Vertebral Artery with Intraluminal Device, Percutaneous Approach */
"03LP3ZZ"  =  "1"  /* Occlusion of Right Vertebral Artery, Percutaneous Approach */
"03LP4BZ"  =  "1"  /* Occlusion of Right Vertebral Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach */
"03LP4CZ"  =  "1"  /* Occlusion of Right Vertebral Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LP4DZ"  =  "1"  /* Occlusion of Right Vertebral Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LP4ZZ"  =  "1"  /* Occlusion of Right Vertebral Artery, Percutaneous Endoscopic Approach */
"03LQ0BZ"  =  "1"  /* Occlusion of Left Vertebral Artery with Bioactive Intraluminal Device, Open Approach */
"03LQ0CZ"  =  "1"  /* Occlusion of Left Vertebral Artery with Extraluminal Device, Open Approach */
"03LQ0DZ"  =  "1"  /* Occlusion of Left Vertebral Artery with Intraluminal Device, Open Approach */
"03LQ0ZZ"  =  "1"  /* Occlusion of Left Vertebral Artery, Open Approach */
"03LQ3BZ"  =  "1"  /* Occlusion of Left Vertebral Artery with Bioactive Intraluminal Device, Percutaneous Approach */
"03LQ3CZ"  =  "1"  /* Occlusion of Left Vertebral Artery with Extraluminal Device, Percutaneous Approach */
"03LQ3DZ"  =  "1"  /* Occlusion of Left Vertebral Artery with Intraluminal Device, Percutaneous Approach */
"03LQ3ZZ"  =  "1"  /* Occlusion of Left Vertebral Artery, Percutaneous Approach */
"03LQ4BZ"  =  "1"  /* Occlusion of Left Vertebral Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach */
"03LQ4CZ"  =  "1"  /* Occlusion of Left Vertebral Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LQ4DZ"  =  "1"  /* Occlusion of Left Vertebral Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LQ4ZZ"  =  "1"  /* Occlusion of Left Vertebral Artery, Percutaneous Endoscopic Approach */
"03LR0CZ"  =  "1"  /* Occlusion of Face Artery with Extraluminal Device, Open Approach */
"03LR0DZ"  =  "1"  /* Occlusion of Face Artery with Intraluminal Device, Open Approach */
"03LR0ZZ"  =  "1"  /* Occlusion of Face Artery, Open Approach */
"03LR3CZ"  =  "1"  /* Occlusion of Face Artery with Extraluminal Device, Percutaneous Approach */
"03LR3DZ"  =  "1"  /* Occlusion of Face Artery with Intraluminal Device, Percutaneous Approach */
"03LR3ZZ"  =  "1"  /* Occlusion of Face Artery, Percutaneous Approach */
"03LR4CZ"  =  "1"  /* Occlusion of Face Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LR4DZ"  =  "1"  /* Occlusion of Face Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LR4ZZ"  =  "1"  /* Occlusion of Face Artery, Percutaneous Endoscopic Approach */
"03LS0CZ"  =  "1"  /* Occlusion of Right Temporal Artery with Extraluminal Device, Open Approach */
"03LS0DZ"  =  "1"  /* Occlusion of Right Temporal Artery with Intraluminal Device, Open Approach */
"03LS0ZZ"  =  "1"  /* Occlusion of Right Temporal Artery, Open Approach */
"03LS3CZ"  =  "1"  /* Occlusion of Right Temporal Artery with Extraluminal Device, Percutaneous Approach */
"03LS3DZ"  =  "1"  /* Occlusion of Right Temporal Artery with Intraluminal Device, Percutaneous Approach */
"03LS3ZZ"  =  "1"  /* Occlusion of Right Temporal Artery, Percutaneous Approach */
"03LS4CZ"  =  "1"  /* Occlusion of Right Temporal Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LS4DZ"  =  "1"  /* Occlusion of Right Temporal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LS4ZZ"  =  "1"  /* Occlusion of Right Temporal Artery, Percutaneous Endoscopic Approach */
"03LT0CZ"  =  "1"  /* Occlusion of Left Temporal Artery with Extraluminal Device, Open Approach */
"03LT0DZ"  =  "1"  /* Occlusion of Left Temporal Artery with Intraluminal Device, Open Approach */
"03LT0ZZ"  =  "1"  /* Occlusion of Left Temporal Artery, Open Approach */
"03LT3CZ"  =  "1"  /* Occlusion of Left Temporal Artery with Extraluminal Device, Percutaneous Approach */
"03LT3DZ"  =  "1"  /* Occlusion of Left Temporal Artery with Intraluminal Device, Percutaneous Approach */
"03LT3ZZ"  =  "1"  /* Occlusion of Left Temporal Artery, Percutaneous Approach */
"03LT4CZ"  =  "1"  /* Occlusion of Left Temporal Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LT4DZ"  =  "1"  /* Occlusion of Left Temporal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LT4ZZ"  =  "1"  /* Occlusion of Left Temporal Artery, Percutaneous Endoscopic Approach */
"03LU0CZ"  =  "1"  /* Occlusion of Right Thyroid Artery with Extraluminal Device, Open Approach */
"03LU0DZ"  =  "1"  /* Occlusion of Right Thyroid Artery with Intraluminal Device, Open Approach */
"03LU0ZZ"  =  "1"  /* Occlusion of Right Thyroid Artery, Open Approach */
"03LU3CZ"  =  "1"  /* Occlusion of Right Thyroid Artery with Extraluminal Device, Percutaneous Approach */
"03LU3DZ"  =  "1"  /* Occlusion of Right Thyroid Artery with Intraluminal Device, Percutaneous Approach */
"03LU3ZZ"  =  "1"  /* Occlusion of Right Thyroid Artery, Percutaneous Approach */
"03LU4CZ"  =  "1"  /* Occlusion of Right Thyroid Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LU4DZ"  =  "1"  /* Occlusion of Right Thyroid Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LU4ZZ"  =  "1"  /* Occlusion of Right Thyroid Artery, Percutaneous Endoscopic Approach */
"03LV0CZ"  =  "1"  /* Occlusion of Left Thyroid Artery with Extraluminal Device, Open Approach */
"03LV0DZ"  =  "1"  /* Occlusion of Left Thyroid Artery with Intraluminal Device, Open Approach */
"03LV0ZZ"  =  "1"  /* Occlusion of Left Thyroid Artery, Open Approach */
"03LV3CZ"  =  "1"  /* Occlusion of Left Thyroid Artery with Extraluminal Device, Percutaneous Approach */
"03LV3DZ"  =  "1"  /* Occlusion of Left Thyroid Artery with Intraluminal Device, Percutaneous Approach */
"03LV3ZZ"  =  "1"  /* Occlusion of Left Thyroid Artery, Percutaneous Approach */
"03LV4CZ"  =  "1"  /* Occlusion of Left Thyroid Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LV4DZ"  =  "1"  /* Occlusion of Left Thyroid Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LV4ZZ"  =  "1"  /* Occlusion of Left Thyroid Artery, Percutaneous Endoscopic Approach */
"03LY0CZ"  =  "1"  /* Occlusion of Upper Artery with Extraluminal Device, Open Approach */
"03LY0DZ"  =  "1"  /* Occlusion of Upper Artery with Intraluminal Device, Open Approach */
"03LY0ZZ"  =  "1"  /* Occlusion of Upper Artery, Open Approach */
"03LY3CZ"  =  "1"  /* Occlusion of Upper Artery with Extraluminal Device, Percutaneous Approach */
"03LY3DZ"  =  "1"  /* Occlusion of Upper Artery with Intraluminal Device, Percutaneous Approach */
"03LY3ZZ"  =  "1"  /* Occlusion of Upper Artery, Percutaneous Approach */
"03LY4CZ"  =  "1"  /* Occlusion of Upper Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03LY4DZ"  =  "1"  /* Occlusion of Upper Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03LY4ZZ"  =  "1"  /* Occlusion of Upper Artery, Percutaneous Endoscopic Approach */
"03PY00Z"  =  "1"  /* Removal of Drainage Device from Upper Artery, Open Approach */
"03PY02Z"  =  "1"  /* Removal of Monitoring Device from Upper Artery, Open Approach */
"03PY03Z"  =  "1"  /* Removal of Infusion Device from Upper Artery, Open Approach */
"03PY07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Artery, Open Approach */
"03PY0CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Artery, Open Approach */
"03PY0DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Artery, Open Approach */
"03PY0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Artery, Open Approach */
"03PY30Z"  =  "1"  /* Removal of Drainage Device from Upper Artery, Percutaneous Approach */
"03PY32Z"  =  "1"  /* Removal of Monitoring Device from Upper Artery, Percutaneous Approach */
"03PY33Z"  =  "1"  /* Removal of Infusion Device from Upper Artery, Percutaneous Approach */
"03PY37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Artery, Percutaneous Approach */
"03PY3CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Artery, Percutaneous Approach */
"03PY3DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Artery, Percutaneous Approach */
"03PY3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Artery, Percutaneous Approach */
"03PY40Z"  =  "1"  /* Removal of Drainage Device from Upper Artery, Percutaneous Endoscopic Approach */
"03PY42Z"  =  "1"  /* Removal of Monitoring Device from Upper Artery, Percutaneous Endoscopic Approach */
"03PY43Z"  =  "1"  /* Removal of Infusion Device from Upper Artery, Percutaneous Endoscopic Approach */
"03PY47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Artery, Percutaneous Endoscopic Approach */
"03PY4CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Artery, Percutaneous Endoscopic Approach */
"03PY4DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Artery, Percutaneous Endoscopic Approach */
"03PY4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Artery, Percutaneous Endoscopic Approach */
"03Q00ZZ"  =  "1"  /* Repair Right Internal Mammary Artery, Open Approach */
"03Q03ZZ"  =  "1"  /* Repair Right Internal Mammary Artery, Percutaneous Approach */
"03Q04ZZ"  =  "1"  /* Repair Right Internal Mammary Artery, Percutaneous Endoscopic Approach */
"03Q10ZZ"  =  "1"  /* Repair Left Internal Mammary Artery, Open Approach */
"03Q13ZZ"  =  "1"  /* Repair Left Internal Mammary Artery, Percutaneous Approach */
"03Q14ZZ"  =  "1"  /* Repair Left Internal Mammary Artery, Percutaneous Endoscopic Approach */
"03Q20ZZ"  =  "1"  /* Repair Innominate Artery, Open Approach */
"03Q23ZZ"  =  "1"  /* Repair Innominate Artery, Percutaneous Approach */
"03Q24ZZ"  =  "1"  /* Repair Innominate Artery, Percutaneous Endoscopic Approach */
"03Q30ZZ"  =  "1"  /* Repair Right Subclavian Artery, Open Approach */
"03Q33ZZ"  =  "1"  /* Repair Right Subclavian Artery, Percutaneous Approach */
"03Q34ZZ"  =  "1"  /* Repair Right Subclavian Artery, Percutaneous Endoscopic Approach */
"03Q40ZZ"  =  "1"  /* Repair Left Subclavian Artery, Open Approach */
"03Q43ZZ"  =  "1"  /* Repair Left Subclavian Artery, Percutaneous Approach */
"03Q44ZZ"  =  "1"  /* Repair Left Subclavian Artery, Percutaneous Endoscopic Approach */
"03Q50ZZ"  =  "1"  /* Repair Right Axillary Artery, Open Approach */
"03Q53ZZ"  =  "1"  /* Repair Right Axillary Artery, Percutaneous Approach */
"03Q54ZZ"  =  "1"  /* Repair Right Axillary Artery, Percutaneous Endoscopic Approach */
"03Q60ZZ"  =  "1"  /* Repair Left Axillary Artery, Open Approach */
"03Q63ZZ"  =  "1"  /* Repair Left Axillary Artery, Percutaneous Approach */
"03Q64ZZ"  =  "1"  /* Repair Left Axillary Artery, Percutaneous Endoscopic Approach */
"03Q70ZZ"  =  "1"  /* Repair Right Brachial Artery, Open Approach */
"03Q73ZZ"  =  "1"  /* Repair Right Brachial Artery, Percutaneous Approach */
"03Q74ZZ"  =  "1"  /* Repair Right Brachial Artery, Percutaneous Endoscopic Approach */
"03Q80ZZ"  =  "1"  /* Repair Left Brachial Artery, Open Approach */
"03Q83ZZ"  =  "1"  /* Repair Left Brachial Artery, Percutaneous Approach */
"03Q84ZZ"  =  "1"  /* Repair Left Brachial Artery, Percutaneous Endoscopic Approach */
"03Q90ZZ"  =  "1"  /* Repair Right Ulnar Artery, Open Approach */
"03Q93ZZ"  =  "1"  /* Repair Right Ulnar Artery, Percutaneous Approach */
"03Q94ZZ"  =  "1"  /* Repair Right Ulnar Artery, Percutaneous Endoscopic Approach */
"03QA0ZZ"  =  "1"  /* Repair Left Ulnar Artery, Open Approach */
"03QA3ZZ"  =  "1"  /* Repair Left Ulnar Artery, Percutaneous Approach */
"03QA4ZZ"  =  "1"  /* Repair Left Ulnar Artery, Percutaneous Endoscopic Approach */
"03QB0ZZ"  =  "1"  /* Repair Right Radial Artery, Open Approach */
"03QB3ZZ"  =  "1"  /* Repair Right Radial Artery, Percutaneous Approach */
"03QB4ZZ"  =  "1"  /* Repair Right Radial Artery, Percutaneous Endoscopic Approach */
"03QC0ZZ"  =  "1"  /* Repair Left Radial Artery, Open Approach */
"03QC3ZZ"  =  "1"  /* Repair Left Radial Artery, Percutaneous Approach */
"03QC4ZZ"  =  "1"  /* Repair Left Radial Artery, Percutaneous Endoscopic Approach */
"03QD0ZZ"  =  "1"  /* Repair Right Hand Artery, Open Approach */
"03QD3ZZ"  =  "1"  /* Repair Right Hand Artery, Percutaneous Approach */
"03QD4ZZ"  =  "1"  /* Repair Right Hand Artery, Percutaneous Endoscopic Approach */
"03QF0ZZ"  =  "1"  /* Repair Left Hand Artery, Open Approach */
"03QF3ZZ"  =  "1"  /* Repair Left Hand Artery, Percutaneous Approach */
"03QF4ZZ"  =  "1"  /* Repair Left Hand Artery, Percutaneous Endoscopic Approach */
"03QG0ZZ"  =  "1"  /* Repair Intracranial Artery, Open Approach */
"03QG3ZZ"  =  "1"  /* Repair Intracranial Artery, Percutaneous Approach */
"03QG4ZZ"  =  "1"  /* Repair Intracranial Artery, Percutaneous Endoscopic Approach */
"03QH0ZZ"  =  "1"  /* Repair Right Common Carotid Artery, Open Approach */
"03QH3ZZ"  =  "1"  /* Repair Right Common Carotid Artery, Percutaneous Approach */
"03QH4ZZ"  =  "1"  /* Repair Right Common Carotid Artery, Percutaneous Endoscopic Approach */
"03QJ0ZZ"  =  "1"  /* Repair Left Common Carotid Artery, Open Approach */
"03QJ3ZZ"  =  "1"  /* Repair Left Common Carotid Artery, Percutaneous Approach */
"03QJ4ZZ"  =  "1"  /* Repair Left Common Carotid Artery, Percutaneous Endoscopic Approach */
"03QK0ZZ"  =  "1"  /* Repair Right Internal Carotid Artery, Open Approach */
"03QK3ZZ"  =  "1"  /* Repair Right Internal Carotid Artery, Percutaneous Approach */
"03QK4ZZ"  =  "1"  /* Repair Right Internal Carotid Artery, Percutaneous Endoscopic Approach */
"03QL0ZZ"  =  "1"  /* Repair Left Internal Carotid Artery, Open Approach */
"03QL3ZZ"  =  "1"  /* Repair Left Internal Carotid Artery, Percutaneous Approach */
"03QL4ZZ"  =  "1"  /* Repair Left Internal Carotid Artery, Percutaneous Endoscopic Approach */
"03QM0ZZ"  =  "1"  /* Repair Right External Carotid Artery, Open Approach */
"03QM3ZZ"  =  "1"  /* Repair Right External Carotid Artery, Percutaneous Approach */
"03QM4ZZ"  =  "1"  /* Repair Right External Carotid Artery, Percutaneous Endoscopic Approach */
"03QN0ZZ"  =  "1"  /* Repair Left External Carotid Artery, Open Approach */
"03QN3ZZ"  =  "1"  /* Repair Left External Carotid Artery, Percutaneous Approach */
"03QN4ZZ"  =  "1"  /* Repair Left External Carotid Artery, Percutaneous Endoscopic Approach */
"03QP0ZZ"  =  "1"  /* Repair Right Vertebral Artery, Open Approach */
"03QP3ZZ"  =  "1"  /* Repair Right Vertebral Artery, Percutaneous Approach */
"03QP4ZZ"  =  "1"  /* Repair Right Vertebral Artery, Percutaneous Endoscopic Approach */
"03QQ0ZZ"  =  "1"  /* Repair Left Vertebral Artery, Open Approach */
"03QQ3ZZ"  =  "1"  /* Repair Left Vertebral Artery, Percutaneous Approach */
"03QQ4ZZ"  =  "1"  /* Repair Left Vertebral Artery, Percutaneous Endoscopic Approach */
"03QR0ZZ"  =  "1"  /* Repair Face Artery, Open Approach */
"03QR3ZZ"  =  "1"  /* Repair Face Artery, Percutaneous Approach */
"03QR4ZZ"  =  "1"  /* Repair Face Artery, Percutaneous Endoscopic Approach */
"03QS0ZZ"  =  "1"  /* Repair Right Temporal Artery, Open Approach */
"03QS3ZZ"  =  "1"  /* Repair Right Temporal Artery, Percutaneous Approach */
"03QS4ZZ"  =  "1"  /* Repair Right Temporal Artery, Percutaneous Endoscopic Approach */
"03QT0ZZ"  =  "1"  /* Repair Left Temporal Artery, Open Approach */
"03QT3ZZ"  =  "1"  /* Repair Left Temporal Artery, Percutaneous Approach */
"03QT4ZZ"  =  "1"  /* Repair Left Temporal Artery, Percutaneous Endoscopic Approach */
"03QU0ZZ"  =  "1"  /* Repair Right Thyroid Artery, Open Approach */
"03QU3ZZ"  =  "1"  /* Repair Right Thyroid Artery, Percutaneous Approach */
"03QU4ZZ"  =  "1"  /* Repair Right Thyroid Artery, Percutaneous Endoscopic Approach */
"03QV0ZZ"  =  "1"  /* Repair Left Thyroid Artery, Open Approach */
"03QV3ZZ"  =  "1"  /* Repair Left Thyroid Artery, Percutaneous Approach */
"03QV4ZZ"  =  "1"  /* Repair Left Thyroid Artery, Percutaneous Endoscopic Approach */
"03QY0ZZ"  =  "1"  /* Repair Upper Artery, Open Approach */
"03QY3ZZ"  =  "1"  /* Repair Upper Artery, Percutaneous Approach */
"03QY4ZZ"  =  "1"  /* Repair Upper Artery, Percutaneous Endoscopic Approach */
"03V00DZ"  =  "1"  /* Restriction of Right Internal Mammary Artery with Intraluminal Device, Open Approach */
"03V00ZZ"  =  "1"  /* Restriction of Right Internal Mammary Artery, Open Approach */
"03V03DZ"  =  "1"  /* Restriction of Right Internal Mammary Artery with Intraluminal Device, Percutaneous Approach */
"03V03ZZ"  =  "1"  /* Restriction of Right Internal Mammary Artery, Percutaneous Approach */
"03V04DZ"  =  "1"  /* Restriction of Right Internal Mammary Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03V04ZZ"  =  "1"  /* Restriction of Right Internal Mammary Artery, Percutaneous Endoscopic Approach */
"03V10DZ"  =  "1"  /* Restriction of Left Internal Mammary Artery with Intraluminal Device, Open Approach */
"03V10ZZ"  =  "1"  /* Restriction of Left Internal Mammary Artery, Open Approach */
"03V13DZ"  =  "1"  /* Restriction of Left Internal Mammary Artery with Intraluminal Device, Percutaneous Approach */
"03V13ZZ"  =  "1"  /* Restriction of Left Internal Mammary Artery, Percutaneous Approach */
"03V14DZ"  =  "1"  /* Restriction of Left Internal Mammary Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03V14ZZ"  =  "1"  /* Restriction of Left Internal Mammary Artery, Percutaneous Endoscopic Approach */
"03V20DZ"  =  "1"  /* Restriction of Innominate Artery with Intraluminal Device, Open Approach */
"03V20ZZ"  =  "1"  /* Restriction of Innominate Artery, Open Approach */
"03V23DZ"  =  "1"  /* Restriction of Innominate Artery with Intraluminal Device, Percutaneous Approach */
"03V23ZZ"  =  "1"  /* Restriction of Innominate Artery, Percutaneous Approach */
"03V24DZ"  =  "1"  /* Restriction of Innominate Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03V24ZZ"  =  "1"  /* Restriction of Innominate Artery, Percutaneous Endoscopic Approach */
"03V30DZ"  =  "1"  /* Restriction of Right Subclavian Artery with Intraluminal Device, Open Approach */
"03V30ZZ"  =  "1"  /* Restriction of Right Subclavian Artery, Open Approach */
"03V33DZ"  =  "1"  /* Restriction of Right Subclavian Artery with Intraluminal Device, Percutaneous Approach */
"03V33ZZ"  =  "1"  /* Restriction of Right Subclavian Artery, Percutaneous Approach */
"03V34DZ"  =  "1"  /* Restriction of Right Subclavian Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03V34ZZ"  =  "1"  /* Restriction of Right Subclavian Artery, Percutaneous Endoscopic Approach */
"03V40DZ"  =  "1"  /* Restriction of Left Subclavian Artery with Intraluminal Device, Open Approach */
"03V40ZZ"  =  "1"  /* Restriction of Left Subclavian Artery, Open Approach */
"03V43DZ"  =  "1"  /* Restriction of Left Subclavian Artery with Intraluminal Device, Percutaneous Approach */
"03V43ZZ"  =  "1"  /* Restriction of Left Subclavian Artery, Percutaneous Approach */
"03V44DZ"  =  "1"  /* Restriction of Left Subclavian Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03V44ZZ"  =  "1"  /* Restriction of Left Subclavian Artery, Percutaneous Endoscopic Approach */
"03V50DZ"  =  "1"  /* Restriction of Right Axillary Artery with Intraluminal Device, Open Approach */
"03V50ZZ"  =  "1"  /* Restriction of Right Axillary Artery, Open Approach */
"03V53DZ"  =  "1"  /* Restriction of Right Axillary Artery with Intraluminal Device, Percutaneous Approach */
"03V53ZZ"  =  "1"  /* Restriction of Right Axillary Artery, Percutaneous Approach */
"03V54DZ"  =  "1"  /* Restriction of Right Axillary Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03V54ZZ"  =  "1"  /* Restriction of Right Axillary Artery, Percutaneous Endoscopic Approach */
"03V60DZ"  =  "1"  /* Restriction of Left Axillary Artery with Intraluminal Device, Open Approach */
"03V60ZZ"  =  "1"  /* Restriction of Left Axillary Artery, Open Approach */
"03V63DZ"  =  "1"  /* Restriction of Left Axillary Artery with Intraluminal Device, Percutaneous Approach */
"03V63ZZ"  =  "1"  /* Restriction of Left Axillary Artery, Percutaneous Approach */
"03V64DZ"  =  "1"  /* Restriction of Left Axillary Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03V64ZZ"  =  "1"  /* Restriction of Left Axillary Artery, Percutaneous Endoscopic Approach */
"03V70DZ"  =  "1"  /* Restriction of Right Brachial Artery with Intraluminal Device, Open Approach */
"03V70ZZ"  =  "1"  /* Restriction of Right Brachial Artery, Open Approach */
"03V73DZ"  =  "1"  /* Restriction of Right Brachial Artery with Intraluminal Device, Percutaneous Approach */
"03V73ZZ"  =  "1"  /* Restriction of Right Brachial Artery, Percutaneous Approach */
"03V74DZ"  =  "1"  /* Restriction of Right Brachial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03V74ZZ"  =  "1"  /* Restriction of Right Brachial Artery, Percutaneous Endoscopic Approach */
"03V80DZ"  =  "1"  /* Restriction of Left Brachial Artery with Intraluminal Device, Open Approach */
"03V80ZZ"  =  "1"  /* Restriction of Left Brachial Artery, Open Approach */
"03V83DZ"  =  "1"  /* Restriction of Left Brachial Artery with Intraluminal Device, Percutaneous Approach */
"03V83ZZ"  =  "1"  /* Restriction of Left Brachial Artery, Percutaneous Approach */
"03V84DZ"  =  "1"  /* Restriction of Left Brachial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03V84ZZ"  =  "1"  /* Restriction of Left Brachial Artery, Percutaneous Endoscopic Approach */
"03V90DZ"  =  "1"  /* Restriction of Right Ulnar Artery with Intraluminal Device, Open Approach */
"03V90ZZ"  =  "1"  /* Restriction of Right Ulnar Artery, Open Approach */
"03V93DZ"  =  "1"  /* Restriction of Right Ulnar Artery with Intraluminal Device, Percutaneous Approach */
"03V93ZZ"  =  "1"  /* Restriction of Right Ulnar Artery, Percutaneous Approach */
"03V94DZ"  =  "1"  /* Restriction of Right Ulnar Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03V94ZZ"  =  "1"  /* Restriction of Right Ulnar Artery, Percutaneous Endoscopic Approach */
"03VA0DZ"  =  "1"  /* Restriction of Left Ulnar Artery with Intraluminal Device, Open Approach */
"03VA0ZZ"  =  "1"  /* Restriction of Left Ulnar Artery, Open Approach */
"03VA3DZ"  =  "1"  /* Restriction of Left Ulnar Artery with Intraluminal Device, Percutaneous Approach */
"03VA3ZZ"  =  "1"  /* Restriction of Left Ulnar Artery, Percutaneous Approach */
"03VA4DZ"  =  "1"  /* Restriction of Left Ulnar Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VA4ZZ"  =  "1"  /* Restriction of Left Ulnar Artery, Percutaneous Endoscopic Approach */
"03VB0DZ"  =  "1"  /* Restriction of Right Radial Artery with Intraluminal Device, Open Approach */
"03VB0ZZ"  =  "1"  /* Restriction of Right Radial Artery, Open Approach */
"03VB3DZ"  =  "1"  /* Restriction of Right Radial Artery with Intraluminal Device, Percutaneous Approach */
"03VB3ZZ"  =  "1"  /* Restriction of Right Radial Artery, Percutaneous Approach */
"03VB4DZ"  =  "1"  /* Restriction of Right Radial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VB4ZZ"  =  "1"  /* Restriction of Right Radial Artery, Percutaneous Endoscopic Approach */
"03VC0DZ"  =  "1"  /* Restriction of Left Radial Artery with Intraluminal Device, Open Approach */
"03VC0ZZ"  =  "1"  /* Restriction of Left Radial Artery, Open Approach */
"03VC3DZ"  =  "1"  /* Restriction of Left Radial Artery with Intraluminal Device, Percutaneous Approach */
"03VC3ZZ"  =  "1"  /* Restriction of Left Radial Artery, Percutaneous Approach */
"03VC4DZ"  =  "1"  /* Restriction of Left Radial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VC4ZZ"  =  "1"  /* Restriction of Left Radial Artery, Percutaneous Endoscopic Approach */
"03VD0DZ"  =  "1"  /* Restriction of Right Hand Artery with Intraluminal Device, Open Approach */
"03VD0ZZ"  =  "1"  /* Restriction of Right Hand Artery, Open Approach */
"03VD3DZ"  =  "1"  /* Restriction of Right Hand Artery with Intraluminal Device, Percutaneous Approach */
"03VD3ZZ"  =  "1"  /* Restriction of Right Hand Artery, Percutaneous Approach */
"03VD4DZ"  =  "1"  /* Restriction of Right Hand Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VD4ZZ"  =  "1"  /* Restriction of Right Hand Artery, Percutaneous Endoscopic Approach */
"03VF0DZ"  =  "1"  /* Restriction of Left Hand Artery with Intraluminal Device, Open Approach */
"03VF0ZZ"  =  "1"  /* Restriction of Left Hand Artery, Open Approach */
"03VF3DZ"  =  "1"  /* Restriction of Left Hand Artery with Intraluminal Device, Percutaneous Approach */
"03VF3ZZ"  =  "1"  /* Restriction of Left Hand Artery, Percutaneous Approach */
"03VF4DZ"  =  "1"  /* Restriction of Left Hand Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VF4ZZ"  =  "1"  /* Restriction of Left Hand Artery, Percutaneous Endoscopic Approach */
"03VG0DZ"  =  "1"  /* Restriction of Intracranial Artery with Intraluminal Device, Open Approach */
"03VG0ZZ"  =  "1"  /* Restriction of Intracranial Artery, Open Approach */
"03VG3DZ"  =  "1"  /* Restriction of Intracranial Artery with Intraluminal Device, Percutaneous Approach */
"03VG3ZZ"  =  "1"  /* Restriction of Intracranial Artery, Percutaneous Approach */
"03VG4DZ"  =  "1"  /* Restriction of Intracranial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VG4ZZ"  =  "1"  /* Restriction of Intracranial Artery, Percutaneous Endoscopic Approach */
"03VH0DZ"  =  "1"  /* Restriction of Right Common Carotid Artery with Intraluminal Device, Open Approach */
"03VH0ZZ"  =  "1"  /* Restriction of Right Common Carotid Artery, Open Approach */
"03VH3DZ"  =  "1"  /* Restriction of Right Common Carotid Artery with Intraluminal Device, Percutaneous Approach */
"03VH3ZZ"  =  "1"  /* Restriction of Right Common Carotid Artery, Percutaneous Approach */
"03VH4DZ"  =  "1"  /* Restriction of Right Common Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VH4ZZ"  =  "1"  /* Restriction of Right Common Carotid Artery, Percutaneous Endoscopic Approach */
"03VJ0DZ"  =  "1"  /* Restriction of Left Common Carotid Artery with Intraluminal Device, Open Approach */
"03VJ0ZZ"  =  "1"  /* Restriction of Left Common Carotid Artery, Open Approach */
"03VJ3DZ"  =  "1"  /* Restriction of Left Common Carotid Artery with Intraluminal Device, Percutaneous Approach */
"03VJ3ZZ"  =  "1"  /* Restriction of Left Common Carotid Artery, Percutaneous Approach */
"03VJ4DZ"  =  "1"  /* Restriction of Left Common Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VJ4ZZ"  =  "1"  /* Restriction of Left Common Carotid Artery, Percutaneous Endoscopic Approach */
"03VK0CZ"  =  "1"  /* Restriction of Right Internal Carotid Artery with Extraluminal Device, Open Approach */
"03VK0DZ"  =  "1"  /* Restriction of Right Internal Carotid Artery with Intraluminal Device, Open Approach */
"03VK0ZZ"  =  "1"  /* Restriction of Right Internal Carotid Artery, Open Approach */
"03VK3CZ"  =  "1"  /* Restriction of Right Internal Carotid Artery with Extraluminal Device, Percutaneous Approach */
"03VK3DZ"  =  "1"  /* Restriction of Right Internal Carotid Artery with Intraluminal Device, Percutaneous Approach */
"03VK3ZZ"  =  "1"  /* Restriction of Right Internal Carotid Artery, Percutaneous Approach */
"03VK4CZ"  =  "1"  /* Restriction of Right Internal Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03VK4DZ"  =  "1"  /* Restriction of Right Internal Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VK4ZZ"  =  "1"  /* Restriction of Right Internal Carotid Artery, Percutaneous Endoscopic Approach */
"03VL0CZ"  =  "1"  /* Restriction of Left Internal Carotid Artery with Extraluminal Device, Open Approach */
"03VL0DZ"  =  "1"  /* Restriction of Left Internal Carotid Artery with Intraluminal Device, Open Approach */
"03VL0ZZ"  =  "1"  /* Restriction of Left Internal Carotid Artery, Open Approach */
"03VL3CZ"  =  "1"  /* Restriction of Left Internal Carotid Artery with Extraluminal Device, Percutaneous Approach */
"03VL3DZ"  =  "1"  /* Restriction of Left Internal Carotid Artery with Intraluminal Device, Percutaneous Approach */
"03VL3ZZ"  =  "1"  /* Restriction of Left Internal Carotid Artery, Percutaneous Approach */
"03VL4CZ"  =  "1"  /* Restriction of Left Internal Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"03VL4DZ"  =  "1"  /* Restriction of Left Internal Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VL4ZZ"  =  "1"  /* Restriction of Left Internal Carotid Artery, Percutaneous Endoscopic Approach */
"03VM0DZ"  =  "1"  /* Restriction of Right External Carotid Artery with Intraluminal Device, Open Approach */
"03VM0ZZ"  =  "1"  /* Restriction of Right External Carotid Artery, Open Approach */
"03VM3DZ"  =  "1"  /* Restriction of Right External Carotid Artery with Intraluminal Device, Percutaneous Approach */
"03VM3ZZ"  =  "1"  /* Restriction of Right External Carotid Artery, Percutaneous Approach */
"03VM4DZ"  =  "1"  /* Restriction of Right External Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VM4ZZ"  =  "1"  /* Restriction of Right External Carotid Artery, Percutaneous Endoscopic Approach */
"03VN0DZ"  =  "1"  /* Restriction of Left External Carotid Artery with Intraluminal Device, Open Approach */
"03VN0ZZ"  =  "1"  /* Restriction of Left External Carotid Artery, Open Approach */
"03VN3DZ"  =  "1"  /* Restriction of Left External Carotid Artery with Intraluminal Device, Percutaneous Approach */
"03VN3ZZ"  =  "1"  /* Restriction of Left External Carotid Artery, Percutaneous Approach */
"03VN4DZ"  =  "1"  /* Restriction of Left External Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VN4ZZ"  =  "1"  /* Restriction of Left External Carotid Artery, Percutaneous Endoscopic Approach */
"03VP0DZ"  =  "1"  /* Restriction of Right Vertebral Artery with Intraluminal Device, Open Approach */
"03VP0ZZ"  =  "1"  /* Restriction of Right Vertebral Artery, Open Approach */
"03VP3DZ"  =  "1"  /* Restriction of Right Vertebral Artery with Intraluminal Device, Percutaneous Approach */
"03VP3ZZ"  =  "1"  /* Restriction of Right Vertebral Artery, Percutaneous Approach */
"03VP4DZ"  =  "1"  /* Restriction of Right Vertebral Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VP4ZZ"  =  "1"  /* Restriction of Right Vertebral Artery, Percutaneous Endoscopic Approach */
"03VQ0DZ"  =  "1"  /* Restriction of Left Vertebral Artery with Intraluminal Device, Open Approach */
"03VQ0ZZ"  =  "1"  /* Restriction of Left Vertebral Artery, Open Approach */
"03VQ3DZ"  =  "1"  /* Restriction of Left Vertebral Artery with Intraluminal Device, Percutaneous Approach */
"03VQ3ZZ"  =  "1"  /* Restriction of Left Vertebral Artery, Percutaneous Approach */
"03VQ4DZ"  =  "1"  /* Restriction of Left Vertebral Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VQ4ZZ"  =  "1"  /* Restriction of Left Vertebral Artery, Percutaneous Endoscopic Approach */
"03VR0DZ"  =  "1"  /* Restriction of Face Artery with Intraluminal Device, Open Approach */
"03VR0ZZ"  =  "1"  /* Restriction of Face Artery, Open Approach */
"03VR3DZ"  =  "1"  /* Restriction of Face Artery with Intraluminal Device, Percutaneous Approach */
"03VR3ZZ"  =  "1"  /* Restriction of Face Artery, Percutaneous Approach */
"03VR4DZ"  =  "1"  /* Restriction of Face Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VR4ZZ"  =  "1"  /* Restriction of Face Artery, Percutaneous Endoscopic Approach */
"03VS0DZ"  =  "1"  /* Restriction of Right Temporal Artery with Intraluminal Device, Open Approach */
"03VS0ZZ"  =  "1"  /* Restriction of Right Temporal Artery, Open Approach */
"03VS3DZ"  =  "1"  /* Restriction of Right Temporal Artery with Intraluminal Device, Percutaneous Approach */
"03VS3ZZ"  =  "1"  /* Restriction of Right Temporal Artery, Percutaneous Approach */
"03VS4DZ"  =  "1"  /* Restriction of Right Temporal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VS4ZZ"  =  "1"  /* Restriction of Right Temporal Artery, Percutaneous Endoscopic Approach */
"03VT0DZ"  =  "1"  /* Restriction of Left Temporal Artery with Intraluminal Device, Open Approach */
"03VT0ZZ"  =  "1"  /* Restriction of Left Temporal Artery, Open Approach */
"03VT3DZ"  =  "1"  /* Restriction of Left Temporal Artery with Intraluminal Device, Percutaneous Approach */
"03VT3ZZ"  =  "1"  /* Restriction of Left Temporal Artery, Percutaneous Approach */
"03VT4DZ"  =  "1"  /* Restriction of Left Temporal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VT4ZZ"  =  "1"  /* Restriction of Left Temporal Artery, Percutaneous Endoscopic Approach */
"03VU0DZ"  =  "1"  /* Restriction of Right Thyroid Artery with Intraluminal Device, Open Approach */
"03VU0ZZ"  =  "1"  /* Restriction of Right Thyroid Artery, Open Approach */
"03VU3DZ"  =  "1"  /* Restriction of Right Thyroid Artery with Intraluminal Device, Percutaneous Approach */
"03VU3ZZ"  =  "1"  /* Restriction of Right Thyroid Artery, Percutaneous Approach */
"03VU4DZ"  =  "1"  /* Restriction of Right Thyroid Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VU4ZZ"  =  "1"  /* Restriction of Right Thyroid Artery, Percutaneous Endoscopic Approach */
"03VV0DZ"  =  "1"  /* Restriction of Left Thyroid Artery with Intraluminal Device, Open Approach */
"03VV0ZZ"  =  "1"  /* Restriction of Left Thyroid Artery, Open Approach */
"03VV3DZ"  =  "1"  /* Restriction of Left Thyroid Artery with Intraluminal Device, Percutaneous Approach */
"03VV3ZZ"  =  "1"  /* Restriction of Left Thyroid Artery, Percutaneous Approach */
"03VV4DZ"  =  "1"  /* Restriction of Left Thyroid Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VV4ZZ"  =  "1"  /* Restriction of Left Thyroid Artery, Percutaneous Endoscopic Approach */
"03VY0DZ"  =  "1"  /* Restriction of Upper Artery with Intraluminal Device, Open Approach */
"03VY0ZZ"  =  "1"  /* Restriction of Upper Artery, Open Approach */
"03VY3DZ"  =  "1"  /* Restriction of Upper Artery with Intraluminal Device, Percutaneous Approach */
"03VY3ZZ"  =  "1"  /* Restriction of Upper Artery, Percutaneous Approach */
"03VY4DZ"  =  "1"  /* Restriction of Upper Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"03VY4ZZ"  =  "1"  /* Restriction of Upper Artery, Percutaneous Endoscopic Approach */
"03WY00Z"  =  "1"  /* Revision of Drainage Device in Upper Artery, Open Approach */
"03WY02Z"  =  "1"  /* Revision of Monitoring Device in Upper Artery, Open Approach */
"03WY03Z"  =  "1"  /* Revision of Infusion Device in Upper Artery, Open Approach */
"03WY07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Artery, Open Approach */
"03WY0CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Artery, Open Approach */
"03WY0DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Artery, Open Approach */
"03WY0JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Artery, Open Approach */
"03WY0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Artery, Open Approach */
"03WY30Z"  =  "1"  /* Revision of Drainage Device in Upper Artery, Percutaneous Approach */
"03WY32Z"  =  "1"  /* Revision of Monitoring Device in Upper Artery, Percutaneous Approach */
"03WY33Z"  =  "1"  /* Revision of Infusion Device in Upper Artery, Percutaneous Approach */
"03WY37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Artery, Percutaneous Approach */
"03WY3CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Artery, Percutaneous Approach */
"03WY3DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Artery, Percutaneous Approach */
"03WY3JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Artery, Percutaneous Approach */
"03WY3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Artery, Percutaneous Approach */
"03WY40Z"  =  "1"  /* Revision of Drainage Device in Upper Artery, Percutaneous Endoscopic Approach */
"03WY42Z"  =  "1"  /* Revision of Monitoring Device in Upper Artery, Percutaneous Endoscopic Approach */
"03WY43Z"  =  "1"  /* Revision of Infusion Device in Upper Artery, Percutaneous Endoscopic Approach */
"03WY47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Artery, Percutaneous Endoscopic Approach */
"03WY4CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Artery, Percutaneous Endoscopic Approach */
"03WY4DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Artery, Percutaneous Endoscopic Approach */
"03WY4JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Artery, Percutaneous Endoscopic Approach */
"03WY4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Artery, Percutaneous Endoscopic Approach */
"049Y0ZZ"  =  "1"  /* Drainage of Lower Artery, Open Approach */
"049Y3ZZ"  =  "1"  /* Drainage of Lower Artery, Percutaneous Approach */
"049Y4ZZ"  =  "1"  /* Drainage of Lower Artery, Percutaneous Endoscopic Approach */
"04C00ZZ"  =  "1"  /* Extirpation of Matter from Abdominal Aorta, Open Approach */
"04C03ZZ"  =  "1"  /* Extirpation of Matter from Abdominal Aorta, Percutaneous Approach */
"04C04ZZ"  =  "1"  /* Extirpation of Matter from Abdominal Aorta, Percutaneous Endoscopic Approach */
"04C10ZZ"  =  "1"  /* Extirpation of Matter from Celiac Artery, Open Approach */
"04C13ZZ"  =  "1"  /* Extirpation of Matter from Celiac Artery, Percutaneous Approach */
"04C14ZZ"  =  "1"  /* Extirpation of Matter from Celiac Artery, Percutaneous Endoscopic Approach */
"04C20ZZ"  =  "1"  /* Extirpation of Matter from Gastric Artery, Open Approach */
"04C23ZZ"  =  "1"  /* Extirpation of Matter from Gastric Artery, Percutaneous Approach */
"04C24ZZ"  =  "1"  /* Extirpation of Matter from Gastric Artery, Percutaneous Endoscopic Approach */
"04C30ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Artery, Open Approach */
"04C33ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Artery, Percutaneous Approach */
"04C34ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Artery, Percutaneous Endoscopic Approach */
"04C40ZZ"  =  "1"  /* Extirpation of Matter from Splenic Artery, Open Approach */
"04C43ZZ"  =  "1"  /* Extirpation of Matter from Splenic Artery, Percutaneous Approach */
"04C44ZZ"  =  "1"  /* Extirpation of Matter from Splenic Artery, Percutaneous Endoscopic Approach */
"04C50ZZ"  =  "1"  /* Extirpation of Matter from Superior Mesenteric Artery, Open Approach */
"04C53ZZ"  =  "1"  /* Extirpation of Matter from Superior Mesenteric Artery, Percutaneous Approach */
"04C54ZZ"  =  "1"  /* Extirpation of Matter from Superior Mesenteric Artery, Percutaneous Endoscopic Approach */
"04C60ZZ"  =  "1"  /* Extirpation of Matter from Right Colic Artery, Open Approach */
"04C63ZZ"  =  "1"  /* Extirpation of Matter from Right Colic Artery, Percutaneous Approach */
"04C64ZZ"  =  "1"  /* Extirpation of Matter from Right Colic Artery, Percutaneous Endoscopic Approach */
"04C70ZZ"  =  "1"  /* Extirpation of Matter from Left Colic Artery, Open Approach */
"04C73ZZ"  =  "1"  /* Extirpation of Matter from Left Colic Artery, Percutaneous Approach */
"04C74ZZ"  =  "1"  /* Extirpation of Matter from Left Colic Artery, Percutaneous Endoscopic Approach */
"04C80ZZ"  =  "1"  /* Extirpation of Matter from Middle Colic Artery, Open Approach */
"04C83ZZ"  =  "1"  /* Extirpation of Matter from Middle Colic Artery, Percutaneous Approach */
"04C84ZZ"  =  "1"  /* Extirpation of Matter from Middle Colic Artery, Percutaneous Endoscopic Approach */
"04C90ZZ"  =  "1"  /* Extirpation of Matter from Right Renal Artery, Open Approach */
"04C93ZZ"  =  "1"  /* Extirpation of Matter from Right Renal Artery, Percutaneous Approach */
"04C94ZZ"  =  "1"  /* Extirpation of Matter from Right Renal Artery, Percutaneous Endoscopic Approach */
"04CA0ZZ"  =  "1"  /* Extirpation of Matter from Left Renal Artery, Open Approach */
"04CA3ZZ"  =  "1"  /* Extirpation of Matter from Left Renal Artery, Percutaneous Approach */
"04CA4ZZ"  =  "1"  /* Extirpation of Matter from Left Renal Artery, Percutaneous Endoscopic Approach */
"04CB0ZZ"  =  "1"  /* Extirpation of Matter from Inferior Mesenteric Artery, Open Approach */
"04CB3ZZ"  =  "1"  /* Extirpation of Matter from Inferior Mesenteric Artery, Percutaneous Approach */
"04CB4ZZ"  =  "1"  /* Extirpation of Matter from Inferior Mesenteric Artery, Percutaneous Endoscopic Approach */
"04CC0ZZ"  =  "1"  /* Extirpation of Matter from Right Common Iliac Artery, Open Approach */
"04CC3ZZ"  =  "1"  /* Extirpation of Matter from Right Common Iliac Artery, Percutaneous Approach */
"04CC4ZZ"  =  "1"  /* Extirpation of Matter from Right Common Iliac Artery, Percutaneous Endoscopic Approach */
"04CD0ZZ"  =  "1"  /* Extirpation of Matter from Left Common Iliac Artery, Open Approach */
"04CD3ZZ"  =  "1"  /* Extirpation of Matter from Left Common Iliac Artery, Percutaneous Approach */
"04CD4ZZ"  =  "1"  /* Extirpation of Matter from Left Common Iliac Artery, Percutaneous Endoscopic Approach */
"04CE0ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Iliac Artery, Open Approach */
"04CE3ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Iliac Artery, Percutaneous Approach */
"04CE4ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Iliac Artery, Percutaneous Endoscopic Approach */
"04CF0ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Iliac Artery, Open Approach */
"04CF3ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Iliac Artery, Percutaneous Approach */
"04CF4ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Iliac Artery, Percutaneous Endoscopic Approach */
"04CH0ZZ"  =  "1"  /* Extirpation of Matter from Right External Iliac Artery, Open Approach */
"04CH3ZZ"  =  "1"  /* Extirpation of Matter from Right External Iliac Artery, Percutaneous Approach */
"04CH4ZZ"  =  "1"  /* Extirpation of Matter from Right External Iliac Artery, Percutaneous Endoscopic Approach */
"04CJ0ZZ"  =  "1"  /* Extirpation of Matter from Left External Iliac Artery, Open Approach */
"04CJ3ZZ"  =  "1"  /* Extirpation of Matter from Left External Iliac Artery, Percutaneous Approach */
"04CJ4ZZ"  =  "1"  /* Extirpation of Matter from Left External Iliac Artery, Percutaneous Endoscopic Approach */
"04CK0ZZ"  =  "1"  /* Extirpation of Matter from Right Femoral Artery, Open Approach */
"04CK3ZZ"  =  "1"  /* Extirpation of Matter from Right Femoral Artery, Percutaneous Approach */
"04CK4ZZ"  =  "1"  /* Extirpation of Matter from Right Femoral Artery, Percutaneous Endoscopic Approach */
"04CL0ZZ"  =  "1"  /* Extirpation of Matter from Left Femoral Artery, Open Approach */
"04CL3ZZ"  =  "1"  /* Extirpation of Matter from Left Femoral Artery, Percutaneous Approach */
"04CL4ZZ"  =  "1"  /* Extirpation of Matter from Left Femoral Artery, Percutaneous Endoscopic Approach */
"04CM0ZZ"  =  "1"  /* Extirpation of Matter from Right Popliteal Artery, Open Approach */
"04CM3ZZ"  =  "1"  /* Extirpation of Matter from Right Popliteal Artery, Percutaneous Approach */
"04CM4ZZ"  =  "1"  /* Extirpation of Matter from Right Popliteal Artery, Percutaneous Endoscopic Approach */
"04CN0ZZ"  =  "1"  /* Extirpation of Matter from Left Popliteal Artery, Open Approach */
"04CN3ZZ"  =  "1"  /* Extirpation of Matter from Left Popliteal Artery, Percutaneous Approach */
"04CN4ZZ"  =  "1"  /* Extirpation of Matter from Left Popliteal Artery, Percutaneous Endoscopic Approach */
"04CP0ZZ"  =  "1"  /* Extirpation of Matter from Right Anterior Tibial Artery, Open Approach */
"04CP3ZZ"  =  "1"  /* Extirpation of Matter from Right Anterior Tibial Artery, Percutaneous Approach */
"04CP4ZZ"  =  "1"  /* Extirpation of Matter from Right Anterior Tibial Artery, Percutaneous Endoscopic Approach */
"04CQ0ZZ"  =  "1"  /* Extirpation of Matter from Left Anterior Tibial Artery, Open Approach */
"04CQ3ZZ"  =  "1"  /* Extirpation of Matter from Left Anterior Tibial Artery, Percutaneous Approach */
"04CQ4ZZ"  =  "1"  /* Extirpation of Matter from Left Anterior Tibial Artery, Percutaneous Endoscopic Approach */
"04CR0ZZ"  =  "1"  /* Extirpation of Matter from Right Posterior Tibial Artery, Open Approach */
"04CR3ZZ"  =  "1"  /* Extirpation of Matter from Right Posterior Tibial Artery, Percutaneous Approach */
"04CR4ZZ"  =  "1"  /* Extirpation of Matter from Right Posterior Tibial Artery, Percutaneous Endoscopic Approach */
"04CS0ZZ"  =  "1"  /* Extirpation of Matter from Left Posterior Tibial Artery, Open Approach */
"04CS3ZZ"  =  "1"  /* Extirpation of Matter from Left Posterior Tibial Artery, Percutaneous Approach */
"04CS4ZZ"  =  "1"  /* Extirpation of Matter from Left Posterior Tibial Artery, Percutaneous Endoscopic Approach */
"04CT0ZZ"  =  "1"  /* Extirpation of Matter from Right Peroneal Artery, Open Approach */
"04CT3ZZ"  =  "1"  /* Extirpation of Matter from Right Peroneal Artery, Percutaneous Approach */
"04CT4ZZ"  =  "1"  /* Extirpation of Matter from Right Peroneal Artery, Percutaneous Endoscopic Approach */
"04CU0ZZ"  =  "1"  /* Extirpation of Matter from Left Peroneal Artery, Open Approach */
"04CU3ZZ"  =  "1"  /* Extirpation of Matter from Left Peroneal Artery, Percutaneous Approach */
"04CU4ZZ"  =  "1"  /* Extirpation of Matter from Left Peroneal Artery, Percutaneous Endoscopic Approach */
"04CV0ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Artery, Open Approach */
"04CV3ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Artery, Percutaneous Approach */
"04CV4ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Artery, Percutaneous Endoscopic Approach */
"04CW0ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Artery, Open Approach */
"04CW3ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Artery, Percutaneous Approach */
"04CW4ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Artery, Percutaneous Endoscopic Approach */
"04CY0ZZ"  =  "1"  /* Extirpation of Matter from Lower Artery, Open Approach */
"04CY3ZZ"  =  "1"  /* Extirpation of Matter from Lower Artery, Percutaneous Approach */
"04CY4ZZ"  =  "1"  /* Extirpation of Matter from Lower Artery, Percutaneous Endoscopic Approach */
"04HY02Z"  =  "1"  /* Insertion of Monitoring Device into Lower Artery, Open Approach */
"04HY32Z"  =  "1"  /* Insertion of Monitoring Device into Lower Artery, Percutaneous Approach */
"04HY42Z"  =  "1"  /* Insertion of Monitoring Device into Lower Artery, Percutaneous Endoscopic Approach */
"04L00CZ"  =  "1"  /* Occlusion of Abdominal Aorta with Extraluminal Device, Open Approach */
"04L00DZ"  =  "1"  /* Occlusion of Abdominal Aorta with Intraluminal Device, Open Approach */
"04L00ZZ"  =  "1"  /* Occlusion of Abdominal Aorta, Open Approach */
"04L03CZ"  =  "1"  /* Occlusion of Abdominal Aorta with Extraluminal Device, Percutaneous Approach */
"04L03DZ"  =  "1"  /* Occlusion of Abdominal Aorta with Intraluminal Device, Percutaneous Approach */
"04L03ZZ"  =  "1"  /* Occlusion of Abdominal Aorta, Percutaneous Approach */
"04L04CZ"  =  "1"  /* Occlusion of Abdominal Aorta with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L04DZ"  =  "1"  /* Occlusion of Abdominal Aorta with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L04ZZ"  =  "1"  /* Occlusion of Abdominal Aorta, Percutaneous Endoscopic Approach */
"04L10CZ"  =  "1"  /* Occlusion of Celiac Artery with Extraluminal Device, Open Approach */
"04L10DZ"  =  "1"  /* Occlusion of Celiac Artery with Intraluminal Device, Open Approach */
"04L10ZZ"  =  "1"  /* Occlusion of Celiac Artery, Open Approach */
"04L13CZ"  =  "1"  /* Occlusion of Celiac Artery with Extraluminal Device, Percutaneous Approach */
"04L13DZ"  =  "1"  /* Occlusion of Celiac Artery with Intraluminal Device, Percutaneous Approach */
"04L13ZZ"  =  "1"  /* Occlusion of Celiac Artery, Percutaneous Approach */
"04L14CZ"  =  "1"  /* Occlusion of Celiac Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L14DZ"  =  "1"  /* Occlusion of Celiac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L14ZZ"  =  "1"  /* Occlusion of Celiac Artery, Percutaneous Endoscopic Approach */
"04L20CZ"  =  "1"  /* Occlusion of Gastric Artery with Extraluminal Device, Open Approach */
"04L20DZ"  =  "1"  /* Occlusion of Gastric Artery with Intraluminal Device, Open Approach */
"04L20ZZ"  =  "1"  /* Occlusion of Gastric Artery, Open Approach */
"04L23CZ"  =  "1"  /* Occlusion of Gastric Artery with Extraluminal Device, Percutaneous Approach */
"04L23DZ"  =  "1"  /* Occlusion of Gastric Artery with Intraluminal Device, Percutaneous Approach */
"04L23ZZ"  =  "1"  /* Occlusion of Gastric Artery, Percutaneous Approach */
"04L24CZ"  =  "1"  /* Occlusion of Gastric Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L24DZ"  =  "1"  /* Occlusion of Gastric Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L24ZZ"  =  "1"  /* Occlusion of Gastric Artery, Percutaneous Endoscopic Approach */
"04L30CZ"  =  "1"  /* Occlusion of Hepatic Artery with Extraluminal Device, Open Approach */
"04L30DZ"  =  "1"  /* Occlusion of Hepatic Artery with Intraluminal Device, Open Approach */
"04L30ZZ"  =  "1"  /* Occlusion of Hepatic Artery, Open Approach */
"04L33CZ"  =  "1"  /* Occlusion of Hepatic Artery with Extraluminal Device, Percutaneous Approach */
"04L33DZ"  =  "1"  /* Occlusion of Hepatic Artery with Intraluminal Device, Percutaneous Approach */
"04L33ZZ"  =  "1"  /* Occlusion of Hepatic Artery, Percutaneous Approach */
"04L34CZ"  =  "1"  /* Occlusion of Hepatic Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L34DZ"  =  "1"  /* Occlusion of Hepatic Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L34ZZ"  =  "1"  /* Occlusion of Hepatic Artery, Percutaneous Endoscopic Approach */
"04L40CZ"  =  "1"  /* Occlusion of Splenic Artery with Extraluminal Device, Open Approach */
"04L40DZ"  =  "1"  /* Occlusion of Splenic Artery with Intraluminal Device, Open Approach */
"04L40ZZ"  =  "1"  /* Occlusion of Splenic Artery, Open Approach */
"04L43CZ"  =  "1"  /* Occlusion of Splenic Artery with Extraluminal Device, Percutaneous Approach */
"04L43DZ"  =  "1"  /* Occlusion of Splenic Artery with Intraluminal Device, Percutaneous Approach */
"04L43ZZ"  =  "1"  /* Occlusion of Splenic Artery, Percutaneous Approach */
"04L44CZ"  =  "1"  /* Occlusion of Splenic Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L44DZ"  =  "1"  /* Occlusion of Splenic Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L44ZZ"  =  "1"  /* Occlusion of Splenic Artery, Percutaneous Endoscopic Approach */
"04L50CZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery with Extraluminal Device, Open Approach */
"04L50DZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery with Intraluminal Device, Open Approach */
"04L50ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery, Open Approach */
"04L53CZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery with Extraluminal Device, Percutaneous Approach */
"04L53DZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery with Intraluminal Device, Percutaneous Approach */
"04L53ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery, Percutaneous Approach */
"04L54CZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L54DZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L54ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery, Percutaneous Endoscopic Approach */
"04L60CZ"  =  "1"  /* Occlusion of Right Colic Artery with Extraluminal Device, Open Approach */
"04L60DZ"  =  "1"  /* Occlusion of Right Colic Artery with Intraluminal Device, Open Approach */
"04L60ZZ"  =  "1"  /* Occlusion of Right Colic Artery, Open Approach */
"04L63CZ"  =  "1"  /* Occlusion of Right Colic Artery with Extraluminal Device, Percutaneous Approach */
"04L63DZ"  =  "1"  /* Occlusion of Right Colic Artery with Intraluminal Device, Percutaneous Approach */
"04L63ZZ"  =  "1"  /* Occlusion of Right Colic Artery, Percutaneous Approach */
"04L64CZ"  =  "1"  /* Occlusion of Right Colic Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L64DZ"  =  "1"  /* Occlusion of Right Colic Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L64ZZ"  =  "1"  /* Occlusion of Right Colic Artery, Percutaneous Endoscopic Approach */
"04L70CZ"  =  "1"  /* Occlusion of Left Colic Artery with Extraluminal Device, Open Approach */
"04L70DZ"  =  "1"  /* Occlusion of Left Colic Artery with Intraluminal Device, Open Approach */
"04L70ZZ"  =  "1"  /* Occlusion of Left Colic Artery, Open Approach */
"04L73CZ"  =  "1"  /* Occlusion of Left Colic Artery with Extraluminal Device, Percutaneous Approach */
"04L73DZ"  =  "1"  /* Occlusion of Left Colic Artery with Intraluminal Device, Percutaneous Approach */
"04L73ZZ"  =  "1"  /* Occlusion of Left Colic Artery, Percutaneous Approach */
"04L74CZ"  =  "1"  /* Occlusion of Left Colic Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L74DZ"  =  "1"  /* Occlusion of Left Colic Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L74ZZ"  =  "1"  /* Occlusion of Left Colic Artery, Percutaneous Endoscopic Approach */
"04L80CZ"  =  "1"  /* Occlusion of Middle Colic Artery with Extraluminal Device, Open Approach */
"04L80DZ"  =  "1"  /* Occlusion of Middle Colic Artery with Intraluminal Device, Open Approach */
"04L80ZZ"  =  "1"  /* Occlusion of Middle Colic Artery, Open Approach */
"04L83CZ"  =  "1"  /* Occlusion of Middle Colic Artery with Extraluminal Device, Percutaneous Approach */
"04L83DZ"  =  "1"  /* Occlusion of Middle Colic Artery with Intraluminal Device, Percutaneous Approach */
"04L83ZZ"  =  "1"  /* Occlusion of Middle Colic Artery, Percutaneous Approach */
"04L84CZ"  =  "1"  /* Occlusion of Middle Colic Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L84DZ"  =  "1"  /* Occlusion of Middle Colic Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L84ZZ"  =  "1"  /* Occlusion of Middle Colic Artery, Percutaneous Endoscopic Approach */
"04L90CZ"  =  "1"  /* Occlusion of Right Renal Artery with Extraluminal Device, Open Approach */
"04L90DZ"  =  "1"  /* Occlusion of Right Renal Artery with Intraluminal Device, Open Approach */
"04L90ZZ"  =  "1"  /* Occlusion of Right Renal Artery, Open Approach */
"04L93CZ"  =  "1"  /* Occlusion of Right Renal Artery with Extraluminal Device, Percutaneous Approach */
"04L93DZ"  =  "1"  /* Occlusion of Right Renal Artery with Intraluminal Device, Percutaneous Approach */
"04L93ZZ"  =  "1"  /* Occlusion of Right Renal Artery, Percutaneous Approach */
"04L94CZ"  =  "1"  /* Occlusion of Right Renal Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L94DZ"  =  "1"  /* Occlusion of Right Renal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L94ZZ"  =  "1"  /* Occlusion of Right Renal Artery, Percutaneous Endoscopic Approach */
"04LA0CZ"  =  "1"  /* Occlusion of Left Renal Artery with Extraluminal Device, Open Approach */
"04LA0DZ"  =  "1"  /* Occlusion of Left Renal Artery with Intraluminal Device, Open Approach */
"04LA0ZZ"  =  "1"  /* Occlusion of Left Renal Artery, Open Approach */
"04LA3CZ"  =  "1"  /* Occlusion of Left Renal Artery with Extraluminal Device, Percutaneous Approach */
"04LA3DZ"  =  "1"  /* Occlusion of Left Renal Artery with Intraluminal Device, Percutaneous Approach */
"04LA3ZZ"  =  "1"  /* Occlusion of Left Renal Artery, Percutaneous Approach */
"04LA4CZ"  =  "1"  /* Occlusion of Left Renal Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LA4DZ"  =  "1"  /* Occlusion of Left Renal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LA4ZZ"  =  "1"  /* Occlusion of Left Renal Artery, Percutaneous Endoscopic Approach */
"04LB0CZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery with Extraluminal Device, Open Approach */
"04LB0DZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery with Intraluminal Device, Open Approach */
"04LB0ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery, Open Approach */
"04LB3CZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery with Extraluminal Device, Percutaneous Approach */
"04LB3DZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery with Intraluminal Device, Percutaneous Approach */
"04LB3ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery, Percutaneous Approach */
"04LB4CZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LB4DZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LB4ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery, Percutaneous Endoscopic Approach */
"04LC0CZ"  =  "1"  /* Occlusion of Right Common Iliac Artery with Extraluminal Device, Open Approach */
"04LC0DZ"  =  "1"  /* Occlusion of Right Common Iliac Artery with Intraluminal Device, Open Approach */
"04LC0ZZ"  =  "1"  /* Occlusion of Right Common Iliac Artery, Open Approach */
"04LC3CZ"  =  "1"  /* Occlusion of Right Common Iliac Artery with Extraluminal Device, Percutaneous Approach */
"04LC3DZ"  =  "1"  /* Occlusion of Right Common Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04LC3ZZ"  =  "1"  /* Occlusion of Right Common Iliac Artery, Percutaneous Approach */
"04LC4CZ"  =  "1"  /* Occlusion of Right Common Iliac Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LC4DZ"  =  "1"  /* Occlusion of Right Common Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LC4ZZ"  =  "1"  /* Occlusion of Right Common Iliac Artery, Percutaneous Endoscopic Approach */
"04LD0CZ"  =  "1"  /* Occlusion of Left Common Iliac Artery with Extraluminal Device, Open Approach */
"04LD0DZ"  =  "1"  /* Occlusion of Left Common Iliac Artery with Intraluminal Device, Open Approach */
"04LD0ZZ"  =  "1"  /* Occlusion of Left Common Iliac Artery, Open Approach */
"04LD3CZ"  =  "1"  /* Occlusion of Left Common Iliac Artery with Extraluminal Device, Percutaneous Approach */
"04LD3DZ"  =  "1"  /* Occlusion of Left Common Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04LD3ZZ"  =  "1"  /* Occlusion of Left Common Iliac Artery, Percutaneous Approach */
"04LD4CZ"  =  "1"  /* Occlusion of Left Common Iliac Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LD4DZ"  =  "1"  /* Occlusion of Left Common Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LD4ZZ"  =  "1"  /* Occlusion of Left Common Iliac Artery, Percutaneous Endoscopic Approach */
"04LE0CT"  =  "1"  /* Occlusion of Right Uterine Artery with Extraluminal Device, Open Approach */
"04LE0CZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery with Extraluminal Device, Open Approach */
"04LE0DT"  =  "1"  /* Occlusion of Right Uterine Artery with Intraluminal Device, Open Approach */
"04LE0DZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery with Intraluminal Device, Open Approach */
"04LE0ZT"  =  "1"  /* Occlusion of Right Uterine Artery, Open Approach */
"04LE0ZZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery, Open Approach */
"04LE3CT"  =  "1"  /* Occlusion of Right Uterine Artery with Extraluminal Device, Percutaneous Approach */
"04LE3CZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery with Extraluminal Device, Percutaneous Approach */
"04LE3DT"  =  "1"  /* Occlusion of Right Uterine Artery with Intraluminal Device, Percutaneous Approach */
"04LE3DZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04LE3ZT"  =  "1"  /* Occlusion of Right Uterine Artery, Percutaneous Approach */
"04LE3ZZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery, Percutaneous Approach */
"04LE4CT"  =  "1"  /* Occlusion of Right Uterine Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LE4CZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LE4DT"  =  "1"  /* Occlusion of Right Uterine Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LE4DZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LE4ZT"  =  "1"  /* Occlusion of Right Uterine Artery, Percutaneous Endoscopic Approach */
"04LE4ZZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery, Percutaneous Endoscopic Approach */
"04LF0CU"  =  "1"  /* Occlusion of Left Uterine Artery with Extraluminal Device, Open Approach */
"04LF0CZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery with Extraluminal Device, Open Approach */
"04LF0DU"  =  "1"  /* Occlusion of Left Uterine Artery with Intraluminal Device, Open Approach */
"04LF0DZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery with Intraluminal Device, Open Approach */
"04LF0ZU"  =  "1"  /* Occlusion of Left Uterine Artery, Open Approach */
"04LF0ZZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery, Open Approach */
"04LF3CU"  =  "1"  /* Occlusion of Left Uterine Artery with Extraluminal Device, Percutaneous Approach */
"04LF3CZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery with Extraluminal Device, Percutaneous Approach */
"04LF3DU"  =  "1"  /* Occlusion of Left Uterine Artery with Intraluminal Device, Percutaneous Approach */
"04LF3DZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04LF3ZU"  =  "1"  /* Occlusion of Left Uterine Artery, Percutaneous Approach */
"04LF3ZZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery, Percutaneous Approach */
"04LF4CU"  =  "1"  /* Occlusion of Left Uterine Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LF4CZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LF4DU"  =  "1"  /* Occlusion of Left Uterine Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LF4DZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LF4ZU"  =  "1"  /* Occlusion of Left Uterine Artery, Percutaneous Endoscopic Approach */
"04LF4ZZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery, Percutaneous Endoscopic Approach */
"04LH0CZ"  =  "1"  /* Occlusion of Right External Iliac Artery with Extraluminal Device, Open Approach */
"04LH0DZ"  =  "1"  /* Occlusion of Right External Iliac Artery with Intraluminal Device, Open Approach */
"04LH0ZZ"  =  "1"  /* Occlusion of Right External Iliac Artery, Open Approach */
"04LH3CZ"  =  "1"  /* Occlusion of Right External Iliac Artery with Extraluminal Device, Percutaneous Approach */
"04LH3DZ"  =  "1"  /* Occlusion of Right External Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04LH3ZZ"  =  "1"  /* Occlusion of Right External Iliac Artery, Percutaneous Approach */
"04LH4CZ"  =  "1"  /* Occlusion of Right External Iliac Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LH4DZ"  =  "1"  /* Occlusion of Right External Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LH4ZZ"  =  "1"  /* Occlusion of Right External Iliac Artery, Percutaneous Endoscopic Approach */
"04LJ0CZ"  =  "1"  /* Occlusion of Left External Iliac Artery with Extraluminal Device, Open Approach */
"04LJ0DZ"  =  "1"  /* Occlusion of Left External Iliac Artery with Intraluminal Device, Open Approach */
"04LJ0ZZ"  =  "1"  /* Occlusion of Left External Iliac Artery, Open Approach */
"04LJ3CZ"  =  "1"  /* Occlusion of Left External Iliac Artery with Extraluminal Device, Percutaneous Approach */
"04LJ3DZ"  =  "1"  /* Occlusion of Left External Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04LJ3ZZ"  =  "1"  /* Occlusion of Left External Iliac Artery, Percutaneous Approach */
"04LJ4CZ"  =  "1"  /* Occlusion of Left External Iliac Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LJ4DZ"  =  "1"  /* Occlusion of Left External Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LJ4ZZ"  =  "1"  /* Occlusion of Left External Iliac Artery, Percutaneous Endoscopic Approach */
"04LK0CZ"  =  "1"  /* Occlusion of Right Femoral Artery with Extraluminal Device, Open Approach */
"04LK0DZ"  =  "1"  /* Occlusion of Right Femoral Artery with Intraluminal Device, Open Approach */
"04LK0ZZ"  =  "1"  /* Occlusion of Right Femoral Artery, Open Approach */
"04LK3CZ"  =  "1"  /* Occlusion of Right Femoral Artery with Extraluminal Device, Percutaneous Approach */
"04LK3DZ"  =  "1"  /* Occlusion of Right Femoral Artery with Intraluminal Device, Percutaneous Approach */
"04LK3ZZ"  =  "1"  /* Occlusion of Right Femoral Artery, Percutaneous Approach */
"04LK4CZ"  =  "1"  /* Occlusion of Right Femoral Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LK4DZ"  =  "1"  /* Occlusion of Right Femoral Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LK4ZZ"  =  "1"  /* Occlusion of Right Femoral Artery, Percutaneous Endoscopic Approach */
"04LL0CZ"  =  "1"  /* Occlusion of Left Femoral Artery with Extraluminal Device, Open Approach */
"04LL0DZ"  =  "1"  /* Occlusion of Left Femoral Artery with Intraluminal Device, Open Approach */
"04LL0ZZ"  =  "1"  /* Occlusion of Left Femoral Artery, Open Approach */
"04LL3CZ"  =  "1"  /* Occlusion of Left Femoral Artery with Extraluminal Device, Percutaneous Approach */
"04LL3DZ"  =  "1"  /* Occlusion of Left Femoral Artery with Intraluminal Device, Percutaneous Approach */
"04LL3ZZ"  =  "1"  /* Occlusion of Left Femoral Artery, Percutaneous Approach */
"04LL4CZ"  =  "1"  /* Occlusion of Left Femoral Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LL4DZ"  =  "1"  /* Occlusion of Left Femoral Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LL4ZZ"  =  "1"  /* Occlusion of Left Femoral Artery, Percutaneous Endoscopic Approach */
"04LM0CZ"  =  "1"  /* Occlusion of Right Popliteal Artery with Extraluminal Device, Open Approach */
"04LM0DZ"  =  "1"  /* Occlusion of Right Popliteal Artery with Intraluminal Device, Open Approach */
"04LM0ZZ"  =  "1"  /* Occlusion of Right Popliteal Artery, Open Approach */
"04LM3CZ"  =  "1"  /* Occlusion of Right Popliteal Artery with Extraluminal Device, Percutaneous Approach */
"04LM3DZ"  =  "1"  /* Occlusion of Right Popliteal Artery with Intraluminal Device, Percutaneous Approach */
"04LM3ZZ"  =  "1"  /* Occlusion of Right Popliteal Artery, Percutaneous Approach */
"04LM4CZ"  =  "1"  /* Occlusion of Right Popliteal Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LM4DZ"  =  "1"  /* Occlusion of Right Popliteal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LM4ZZ"  =  "1"  /* Occlusion of Right Popliteal Artery, Percutaneous Endoscopic Approach */
"04LN0CZ"  =  "1"  /* Occlusion of Left Popliteal Artery with Extraluminal Device, Open Approach */
"04LN0DZ"  =  "1"  /* Occlusion of Left Popliteal Artery with Intraluminal Device, Open Approach */
"04LN0ZZ"  =  "1"  /* Occlusion of Left Popliteal Artery, Open Approach */
"04LN3CZ"  =  "1"  /* Occlusion of Left Popliteal Artery with Extraluminal Device, Percutaneous Approach */
"04LN3DZ"  =  "1"  /* Occlusion of Left Popliteal Artery with Intraluminal Device, Percutaneous Approach */
"04LN3ZZ"  =  "1"  /* Occlusion of Left Popliteal Artery, Percutaneous Approach */
"04LN4CZ"  =  "1"  /* Occlusion of Left Popliteal Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LN4DZ"  =  "1"  /* Occlusion of Left Popliteal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LN4ZZ"  =  "1"  /* Occlusion of Left Popliteal Artery, Percutaneous Endoscopic Approach */
"04LP0CZ"  =  "1"  /* Occlusion of Right Anterior Tibial Artery with Extraluminal Device, Open Approach */
"04LP0DZ"  =  "1"  /* Occlusion of Right Anterior Tibial Artery with Intraluminal Device, Open Approach */
"04LP0ZZ"  =  "1"  /* Occlusion of Right Anterior Tibial Artery, Open Approach */
"04LP3CZ"  =  "1"  /* Occlusion of Right Anterior Tibial Artery with Extraluminal Device, Percutaneous Approach */
"04LP3DZ"  =  "1"  /* Occlusion of Right Anterior Tibial Artery with Intraluminal Device, Percutaneous Approach */
"04LP3ZZ"  =  "1"  /* Occlusion of Right Anterior Tibial Artery, Percutaneous Approach */
"04LP4CZ"  =  "1"  /* Occlusion of Right Anterior Tibial Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LP4DZ"  =  "1"  /* Occlusion of Right Anterior Tibial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LP4ZZ"  =  "1"  /* Occlusion of Right Anterior Tibial Artery, Percutaneous Endoscopic Approach */
"04LQ0CZ"  =  "1"  /* Occlusion of Left Anterior Tibial Artery with Extraluminal Device, Open Approach */
"04LQ0DZ"  =  "1"  /* Occlusion of Left Anterior Tibial Artery with Intraluminal Device, Open Approach */
"04LQ0ZZ"  =  "1"  /* Occlusion of Left Anterior Tibial Artery, Open Approach */
"04LQ3CZ"  =  "1"  /* Occlusion of Left Anterior Tibial Artery with Extraluminal Device, Percutaneous Approach */
"04LQ3DZ"  =  "1"  /* Occlusion of Left Anterior Tibial Artery with Intraluminal Device, Percutaneous Approach */
"04LQ3ZZ"  =  "1"  /* Occlusion of Left Anterior Tibial Artery, Percutaneous Approach */
"04LQ4CZ"  =  "1"  /* Occlusion of Left Anterior Tibial Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LQ4DZ"  =  "1"  /* Occlusion of Left Anterior Tibial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LQ4ZZ"  =  "1"  /* Occlusion of Left Anterior Tibial Artery, Percutaneous Endoscopic Approach */
"04LR0CZ"  =  "1"  /* Occlusion of Right Posterior Tibial Artery with Extraluminal Device, Open Approach */
"04LR0DZ"  =  "1"  /* Occlusion of Right Posterior Tibial Artery with Intraluminal Device, Open Approach */
"04LR0ZZ"  =  "1"  /* Occlusion of Right Posterior Tibial Artery, Open Approach */
"04LR3CZ"  =  "1"  /* Occlusion of Right Posterior Tibial Artery with Extraluminal Device, Percutaneous Approach */
"04LR3DZ"  =  "1"  /* Occlusion of Right Posterior Tibial Artery with Intraluminal Device, Percutaneous Approach */
"04LR3ZZ"  =  "1"  /* Occlusion of Right Posterior Tibial Artery, Percutaneous Approach */
"04LR4CZ"  =  "1"  /* Occlusion of Right Posterior Tibial Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LR4DZ"  =  "1"  /* Occlusion of Right Posterior Tibial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LR4ZZ"  =  "1"  /* Occlusion of Right Posterior Tibial Artery, Percutaneous Endoscopic Approach */
"04LS0CZ"  =  "1"  /* Occlusion of Left Posterior Tibial Artery with Extraluminal Device, Open Approach */
"04LS0DZ"  =  "1"  /* Occlusion of Left Posterior Tibial Artery with Intraluminal Device, Open Approach */
"04LS0ZZ"  =  "1"  /* Occlusion of Left Posterior Tibial Artery, Open Approach */
"04LS3CZ"  =  "1"  /* Occlusion of Left Posterior Tibial Artery with Extraluminal Device, Percutaneous Approach */
"04LS3DZ"  =  "1"  /* Occlusion of Left Posterior Tibial Artery with Intraluminal Device, Percutaneous Approach */
"04LS3ZZ"  =  "1"  /* Occlusion of Left Posterior Tibial Artery, Percutaneous Approach */
"04LS4CZ"  =  "1"  /* Occlusion of Left Posterior Tibial Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LS4DZ"  =  "1"  /* Occlusion of Left Posterior Tibial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LS4ZZ"  =  "1"  /* Occlusion of Left Posterior Tibial Artery, Percutaneous Endoscopic Approach */
"04LT0CZ"  =  "1"  /* Occlusion of Right Peroneal Artery with Extraluminal Device, Open Approach */
"04LT0DZ"  =  "1"  /* Occlusion of Right Peroneal Artery with Intraluminal Device, Open Approach */
"04LT0ZZ"  =  "1"  /* Occlusion of Right Peroneal Artery, Open Approach */
"04LT3CZ"  =  "1"  /* Occlusion of Right Peroneal Artery with Extraluminal Device, Percutaneous Approach */
"04LT3DZ"  =  "1"  /* Occlusion of Right Peroneal Artery with Intraluminal Device, Percutaneous Approach */
"04LT3ZZ"  =  "1"  /* Occlusion of Right Peroneal Artery, Percutaneous Approach */
"04LT4CZ"  =  "1"  /* Occlusion of Right Peroneal Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LT4DZ"  =  "1"  /* Occlusion of Right Peroneal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LT4ZZ"  =  "1"  /* Occlusion of Right Peroneal Artery, Percutaneous Endoscopic Approach */
"04LU0CZ"  =  "1"  /* Occlusion of Left Peroneal Artery with Extraluminal Device, Open Approach */
"04LU0DZ"  =  "1"  /* Occlusion of Left Peroneal Artery with Intraluminal Device, Open Approach */
"04LU0ZZ"  =  "1"  /* Occlusion of Left Peroneal Artery, Open Approach */
"04LU3CZ"  =  "1"  /* Occlusion of Left Peroneal Artery with Extraluminal Device, Percutaneous Approach */
"04LU3DZ"  =  "1"  /* Occlusion of Left Peroneal Artery with Intraluminal Device, Percutaneous Approach */
"04LU3ZZ"  =  "1"  /* Occlusion of Left Peroneal Artery, Percutaneous Approach */
"04LU4CZ"  =  "1"  /* Occlusion of Left Peroneal Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LU4DZ"  =  "1"  /* Occlusion of Left Peroneal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LU4ZZ"  =  "1"  /* Occlusion of Left Peroneal Artery, Percutaneous Endoscopic Approach */
"04LV0CZ"  =  "1"  /* Occlusion of Right Foot Artery with Extraluminal Device, Open Approach */
"04LV0DZ"  =  "1"  /* Occlusion of Right Foot Artery with Intraluminal Device, Open Approach */
"04LV0ZZ"  =  "1"  /* Occlusion of Right Foot Artery, Open Approach */
"04LV3CZ"  =  "1"  /* Occlusion of Right Foot Artery with Extraluminal Device, Percutaneous Approach */
"04LV3DZ"  =  "1"  /* Occlusion of Right Foot Artery with Intraluminal Device, Percutaneous Approach */
"04LV3ZZ"  =  "1"  /* Occlusion of Right Foot Artery, Percutaneous Approach */
"04LV4CZ"  =  "1"  /* Occlusion of Right Foot Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LV4DZ"  =  "1"  /* Occlusion of Right Foot Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LV4ZZ"  =  "1"  /* Occlusion of Right Foot Artery, Percutaneous Endoscopic Approach */
"04LW0CZ"  =  "1"  /* Occlusion of Left Foot Artery with Extraluminal Device, Open Approach */
"04LW0DZ"  =  "1"  /* Occlusion of Left Foot Artery with Intraluminal Device, Open Approach */
"04LW0ZZ"  =  "1"  /* Occlusion of Left Foot Artery, Open Approach */
"04LW3CZ"  =  "1"  /* Occlusion of Left Foot Artery with Extraluminal Device, Percutaneous Approach */
"04LW3DZ"  =  "1"  /* Occlusion of Left Foot Artery with Intraluminal Device, Percutaneous Approach */
"04LW3ZZ"  =  "1"  /* Occlusion of Left Foot Artery, Percutaneous Approach */
"04LW4CZ"  =  "1"  /* Occlusion of Left Foot Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LW4DZ"  =  "1"  /* Occlusion of Left Foot Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LW4ZZ"  =  "1"  /* Occlusion of Left Foot Artery, Percutaneous Endoscopic Approach */
"04LY0CZ"  =  "1"  /* Occlusion of Lower Artery with Extraluminal Device, Open Approach */
"04LY0DZ"  =  "1"  /* Occlusion of Lower Artery with Intraluminal Device, Open Approach */
"04LY0ZZ"  =  "1"  /* Occlusion of Lower Artery, Open Approach */
"04LY3CZ"  =  "1"  /* Occlusion of Lower Artery with Extraluminal Device, Percutaneous Approach */
"04LY3DZ"  =  "1"  /* Occlusion of Lower Artery with Intraluminal Device, Percutaneous Approach */
"04LY3ZZ"  =  "1"  /* Occlusion of Lower Artery, Percutaneous Approach */
"04LY4CZ"  =  "1"  /* Occlusion of Lower Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LY4DZ"  =  "1"  /* Occlusion of Lower Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LY4ZZ"  =  "1"  /* Occlusion of Lower Artery, Percutaneous Endoscopic Approach */
"04PY00Z"  =  "1"  /* Removal of Drainage Device from Lower Artery, Open Approach */
"04PY02Z"  =  "1"  /* Removal of Monitoring Device from Lower Artery, Open Approach */
"04PY03Z"  =  "1"  /* Removal of Infusion Device from Lower Artery, Open Approach */
"04PY07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Artery, Open Approach */
"04PY0CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Artery, Open Approach */
"04PY0DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Artery, Open Approach */
"04PY0JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Artery, Open Approach */
"04PY0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Artery, Open Approach */
"04PY30Z"  =  "1"  /* Removal of Drainage Device from Lower Artery, Percutaneous Approach */
"04PY32Z"  =  "1"  /* Removal of Monitoring Device from Lower Artery, Percutaneous Approach */
"04PY33Z"  =  "1"  /* Removal of Infusion Device from Lower Artery, Percutaneous Approach */
"04PY37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Artery, Percutaneous Approach */
"04PY3CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Artery, Percutaneous Approach */
"04PY3DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Artery, Percutaneous Approach */
"04PY3JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Artery, Percutaneous Approach */
"04PY3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Artery, Percutaneous Approach */
"04PY40Z"  =  "1"  /* Removal of Drainage Device from Lower Artery, Percutaneous Endoscopic Approach */
"04PY42Z"  =  "1"  /* Removal of Monitoring Device from Lower Artery, Percutaneous Endoscopic Approach */
"04PY43Z"  =  "1"  /* Removal of Infusion Device from Lower Artery, Percutaneous Endoscopic Approach */
"04PY47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Artery, Percutaneous Endoscopic Approach */
"04PY4CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Artery, Percutaneous Endoscopic Approach */
"04PY4DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Artery, Percutaneous Endoscopic Approach */
"04PY4JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Artery, Percutaneous Endoscopic Approach */
"04PY4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Artery, Percutaneous Endoscopic Approach */
"04Q00ZZ"  =  "1"  /* Repair Abdominal Aorta, Open Approach */
"04Q03ZZ"  =  "1"  /* Repair Abdominal Aorta, Percutaneous Approach */
"04Q04ZZ"  =  "1"  /* Repair Abdominal Aorta, Percutaneous Endoscopic Approach */
"04Q10ZZ"  =  "1"  /* Repair Celiac Artery, Open Approach */
"04Q13ZZ"  =  "1"  /* Repair Celiac Artery, Percutaneous Approach */
"04Q14ZZ"  =  "1"  /* Repair Celiac Artery, Percutaneous Endoscopic Approach */
"04Q20ZZ"  =  "1"  /* Repair Gastric Artery, Open Approach */
"04Q23ZZ"  =  "1"  /* Repair Gastric Artery, Percutaneous Approach */
"04Q24ZZ"  =  "1"  /* Repair Gastric Artery, Percutaneous Endoscopic Approach */
"04Q30ZZ"  =  "1"  /* Repair Hepatic Artery, Open Approach */
"04Q33ZZ"  =  "1"  /* Repair Hepatic Artery, Percutaneous Approach */
"04Q34ZZ"  =  "1"  /* Repair Hepatic Artery, Percutaneous Endoscopic Approach */
"04Q40ZZ"  =  "1"  /* Repair Splenic Artery, Open Approach */
"04Q43ZZ"  =  "1"  /* Repair Splenic Artery, Percutaneous Approach */
"04Q44ZZ"  =  "1"  /* Repair Splenic Artery, Percutaneous Endoscopic Approach */
"04Q50ZZ"  =  "1"  /* Repair Superior Mesenteric Artery, Open Approach */
"04Q53ZZ"  =  "1"  /* Repair Superior Mesenteric Artery, Percutaneous Approach */
"04Q54ZZ"  =  "1"  /* Repair Superior Mesenteric Artery, Percutaneous Endoscopic Approach */
"04Q60ZZ"  =  "1"  /* Repair Right Colic Artery, Open Approach */
"04Q63ZZ"  =  "1"  /* Repair Right Colic Artery, Percutaneous Approach */
"04Q64ZZ"  =  "1"  /* Repair Right Colic Artery, Percutaneous Endoscopic Approach */
"04Q70ZZ"  =  "1"  /* Repair Left Colic Artery, Open Approach */
"04Q73ZZ"  =  "1"  /* Repair Left Colic Artery, Percutaneous Approach */
"04Q74ZZ"  =  "1"  /* Repair Left Colic Artery, Percutaneous Endoscopic Approach */
"04Q80ZZ"  =  "1"  /* Repair Middle Colic Artery, Open Approach */
"04Q83ZZ"  =  "1"  /* Repair Middle Colic Artery, Percutaneous Approach */
"04Q84ZZ"  =  "1"  /* Repair Middle Colic Artery, Percutaneous Endoscopic Approach */
"04Q90ZZ"  =  "1"  /* Repair Right Renal Artery, Open Approach */
"04Q93ZZ"  =  "1"  /* Repair Right Renal Artery, Percutaneous Approach */
"04Q94ZZ"  =  "1"  /* Repair Right Renal Artery, Percutaneous Endoscopic Approach */
"04QA0ZZ"  =  "1"  /* Repair Left Renal Artery, Open Approach */
"04QA3ZZ"  =  "1"  /* Repair Left Renal Artery, Percutaneous Approach */
"04QA4ZZ"  =  "1"  /* Repair Left Renal Artery, Percutaneous Endoscopic Approach */
"04QB0ZZ"  =  "1"  /* Repair Inferior Mesenteric Artery, Open Approach */
"04QB3ZZ"  =  "1"  /* Repair Inferior Mesenteric Artery, Percutaneous Approach */
"04QB4ZZ"  =  "1"  /* Repair Inferior Mesenteric Artery, Percutaneous Endoscopic Approach */
"04QC0ZZ"  =  "1"  /* Repair Right Common Iliac Artery, Open Approach */
"04QC3ZZ"  =  "1"  /* Repair Right Common Iliac Artery, Percutaneous Approach */
"04QC4ZZ"  =  "1"  /* Repair Right Common Iliac Artery, Percutaneous Endoscopic Approach */
"04QD0ZZ"  =  "1"  /* Repair Left Common Iliac Artery, Open Approach */
"04QD3ZZ"  =  "1"  /* Repair Left Common Iliac Artery, Percutaneous Approach */
"04QD4ZZ"  =  "1"  /* Repair Left Common Iliac Artery, Percutaneous Endoscopic Approach */
"04QE0ZZ"  =  "1"  /* Repair Right Internal Iliac Artery, Open Approach */
"04QE3ZZ"  =  "1"  /* Repair Right Internal Iliac Artery, Percutaneous Approach */
"04QE4ZZ"  =  "1"  /* Repair Right Internal Iliac Artery, Percutaneous Endoscopic Approach */
"04QF0ZZ"  =  "1"  /* Repair Left Internal Iliac Artery, Open Approach */
"04QF3ZZ"  =  "1"  /* Repair Left Internal Iliac Artery, Percutaneous Approach */
"04QF4ZZ"  =  "1"  /* Repair Left Internal Iliac Artery, Percutaneous Endoscopic Approach */
"04QH0ZZ"  =  "1"  /* Repair Right External Iliac Artery, Open Approach */
"04QH3ZZ"  =  "1"  /* Repair Right External Iliac Artery, Percutaneous Approach */
"04QH4ZZ"  =  "1"  /* Repair Right External Iliac Artery, Percutaneous Endoscopic Approach */
"04QJ0ZZ"  =  "1"  /* Repair Left External Iliac Artery, Open Approach */
"04QJ3ZZ"  =  "1"  /* Repair Left External Iliac Artery, Percutaneous Approach */
"04QJ4ZZ"  =  "1"  /* Repair Left External Iliac Artery, Percutaneous Endoscopic Approach */
"04QK0ZZ"  =  "1"  /* Repair Right Femoral Artery, Open Approach */
"04QK3ZZ"  =  "1"  /* Repair Right Femoral Artery, Percutaneous Approach */
"04QK4ZZ"  =  "1"  /* Repair Right Femoral Artery, Percutaneous Endoscopic Approach */
"04QL0ZZ"  =  "1"  /* Repair Left Femoral Artery, Open Approach */
"04QL3ZZ"  =  "1"  /* Repair Left Femoral Artery, Percutaneous Approach */
"04QL4ZZ"  =  "1"  /* Repair Left Femoral Artery, Percutaneous Endoscopic Approach */
"04QM0ZZ"  =  "1"  /* Repair Right Popliteal Artery, Open Approach */
"04QM3ZZ"  =  "1"  /* Repair Right Popliteal Artery, Percutaneous Approach */
"04QM4ZZ"  =  "1"  /* Repair Right Popliteal Artery, Percutaneous Endoscopic Approach */
"04QN0ZZ"  =  "1"  /* Repair Left Popliteal Artery, Open Approach */
"04QN3ZZ"  =  "1"  /* Repair Left Popliteal Artery, Percutaneous Approach */
"04QN4ZZ"  =  "1"  /* Repair Left Popliteal Artery, Percutaneous Endoscopic Approach */
"04QP0ZZ"  =  "1"  /* Repair Right Anterior Tibial Artery, Open Approach */
"04QP3ZZ"  =  "1"  /* Repair Right Anterior Tibial Artery, Percutaneous Approach */
"04QP4ZZ"  =  "1"  /* Repair Right Anterior Tibial Artery, Percutaneous Endoscopic Approach */
"04QQ0ZZ"  =  "1"  /* Repair Left Anterior Tibial Artery, Open Approach */
"04QQ3ZZ"  =  "1"  /* Repair Left Anterior Tibial Artery, Percutaneous Approach */
"04QQ4ZZ"  =  "1"  /* Repair Left Anterior Tibial Artery, Percutaneous Endoscopic Approach */
"04QR0ZZ"  =  "1"  /* Repair Right Posterior Tibial Artery, Open Approach */
"04QR3ZZ"  =  "1"  /* Repair Right Posterior Tibial Artery, Percutaneous Approach */
"04QR4ZZ"  =  "1"  /* Repair Right Posterior Tibial Artery, Percutaneous Endoscopic Approach */
"04QS0ZZ"  =  "1"  /* Repair Left Posterior Tibial Artery, Open Approach */
"04QS3ZZ"  =  "1"  /* Repair Left Posterior Tibial Artery, Percutaneous Approach */
"04QS4ZZ"  =  "1"  /* Repair Left Posterior Tibial Artery, Percutaneous Endoscopic Approach */
"04QT0ZZ"  =  "1"  /* Repair Right Peroneal Artery, Open Approach */
"04QT3ZZ"  =  "1"  /* Repair Right Peroneal Artery, Percutaneous Approach */
"04QT4ZZ"  =  "1"  /* Repair Right Peroneal Artery, Percutaneous Endoscopic Approach */
"04QU0ZZ"  =  "1"  /* Repair Left Peroneal Artery, Open Approach */
"04QU3ZZ"  =  "1"  /* Repair Left Peroneal Artery, Percutaneous Approach */
"04QU4ZZ"  =  "1"  /* Repair Left Peroneal Artery, Percutaneous Endoscopic Approach */
"04QV0ZZ"  =  "1"  /* Repair Right Foot Artery, Open Approach */
"04QV3ZZ"  =  "1"  /* Repair Right Foot Artery, Percutaneous Approach */
"04QV4ZZ"  =  "1"  /* Repair Right Foot Artery, Percutaneous Endoscopic Approach */
"04QW0ZZ"  =  "1"  /* Repair Left Foot Artery, Open Approach */
"04QW3ZZ"  =  "1"  /* Repair Left Foot Artery, Percutaneous Approach */
"04QW4ZZ"  =  "1"  /* Repair Left Foot Artery, Percutaneous Endoscopic Approach */
"04QY0ZZ"  =  "1"  /* Repair Lower Artery, Open Approach */
"04QY3ZZ"  =  "1"  /* Repair Lower Artery, Percutaneous Approach */
"04QY4ZZ"  =  "1"  /* Repair Lower Artery, Percutaneous Endoscopic Approach */
"04V00DZ"  =  "1"  /* Restriction of Abdominal Aorta with Intraluminal Device, Open Approach */
"04V00ZZ"  =  "1"  /* Restriction of Abdominal Aorta, Open Approach */
"04V03ZZ"  =  "1"  /* Restriction of Abdominal Aorta, Percutaneous Approach */
"04V04ZZ"  =  "1"  /* Restriction of Abdominal Aorta, Percutaneous Endoscopic Approach */
"04V10DZ"  =  "1"  /* Restriction of Celiac Artery with Intraluminal Device, Open Approach */
"04V10ZZ"  =  "1"  /* Restriction of Celiac Artery, Open Approach */
"04V13DZ"  =  "1"  /* Restriction of Celiac Artery with Intraluminal Device, Percutaneous Approach */
"04V13ZZ"  =  "1"  /* Restriction of Celiac Artery, Percutaneous Approach */
"04V14DZ"  =  "1"  /* Restriction of Celiac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04V14ZZ"  =  "1"  /* Restriction of Celiac Artery, Percutaneous Endoscopic Approach */
"04V20DZ"  =  "1"  /* Restriction of Gastric Artery with Intraluminal Device, Open Approach */
"04V20ZZ"  =  "1"  /* Restriction of Gastric Artery, Open Approach */
"04V23DZ"  =  "1"  /* Restriction of Gastric Artery with Intraluminal Device, Percutaneous Approach */
"04V23ZZ"  =  "1"  /* Restriction of Gastric Artery, Percutaneous Approach */
"04V24DZ"  =  "1"  /* Restriction of Gastric Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04V24ZZ"  =  "1"  /* Restriction of Gastric Artery, Percutaneous Endoscopic Approach */
"04V30DZ"  =  "1"  /* Restriction of Hepatic Artery with Intraluminal Device, Open Approach */
"04V30ZZ"  =  "1"  /* Restriction of Hepatic Artery, Open Approach */
"04V33DZ"  =  "1"  /* Restriction of Hepatic Artery with Intraluminal Device, Percutaneous Approach */
"04V33ZZ"  =  "1"  /* Restriction of Hepatic Artery, Percutaneous Approach */
"04V34DZ"  =  "1"  /* Restriction of Hepatic Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04V34ZZ"  =  "1"  /* Restriction of Hepatic Artery, Percutaneous Endoscopic Approach */
"04V40DZ"  =  "1"  /* Restriction of Splenic Artery with Intraluminal Device, Open Approach */
"04V40ZZ"  =  "1"  /* Restriction of Splenic Artery, Open Approach */
"04V43DZ"  =  "1"  /* Restriction of Splenic Artery with Intraluminal Device, Percutaneous Approach */
"04V43ZZ"  =  "1"  /* Restriction of Splenic Artery, Percutaneous Approach */
"04V44DZ"  =  "1"  /* Restriction of Splenic Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04V44ZZ"  =  "1"  /* Restriction of Splenic Artery, Percutaneous Endoscopic Approach */
"04V50DZ"  =  "1"  /* Restriction of Superior Mesenteric Artery with Intraluminal Device, Open Approach */
"04V50ZZ"  =  "1"  /* Restriction of Superior Mesenteric Artery, Open Approach */
"04V53DZ"  =  "1"  /* Restriction of Superior Mesenteric Artery with Intraluminal Device, Percutaneous Approach */
"04V53ZZ"  =  "1"  /* Restriction of Superior Mesenteric Artery, Percutaneous Approach */
"04V54DZ"  =  "1"  /* Restriction of Superior Mesenteric Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04V54ZZ"  =  "1"  /* Restriction of Superior Mesenteric Artery, Percutaneous Endoscopic Approach */
"04V60DZ"  =  "1"  /* Restriction of Right Colic Artery with Intraluminal Device, Open Approach */
"04V60ZZ"  =  "1"  /* Restriction of Right Colic Artery, Open Approach */
"04V63DZ"  =  "1"  /* Restriction of Right Colic Artery with Intraluminal Device, Percutaneous Approach */
"04V63ZZ"  =  "1"  /* Restriction of Right Colic Artery, Percutaneous Approach */
"04V64DZ"  =  "1"  /* Restriction of Right Colic Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04V64ZZ"  =  "1"  /* Restriction of Right Colic Artery, Percutaneous Endoscopic Approach */
"04V70DZ"  =  "1"  /* Restriction of Left Colic Artery with Intraluminal Device, Open Approach */
"04V70ZZ"  =  "1"  /* Restriction of Left Colic Artery, Open Approach */
"04V73DZ"  =  "1"  /* Restriction of Left Colic Artery with Intraluminal Device, Percutaneous Approach */
"04V73ZZ"  =  "1"  /* Restriction of Left Colic Artery, Percutaneous Approach */
"04V74DZ"  =  "1"  /* Restriction of Left Colic Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04V74ZZ"  =  "1"  /* Restriction of Left Colic Artery, Percutaneous Endoscopic Approach */
"04V80DZ"  =  "1"  /* Restriction of Middle Colic Artery with Intraluminal Device, Open Approach */
"04V80ZZ"  =  "1"  /* Restriction of Middle Colic Artery, Open Approach */
"04V83DZ"  =  "1"  /* Restriction of Middle Colic Artery with Intraluminal Device, Percutaneous Approach */
"04V83ZZ"  =  "1"  /* Restriction of Middle Colic Artery, Percutaneous Approach */
"04V84DZ"  =  "1"  /* Restriction of Middle Colic Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04V84ZZ"  =  "1"  /* Restriction of Middle Colic Artery, Percutaneous Endoscopic Approach */
"04V90DZ"  =  "1"  /* Restriction of Right Renal Artery with Intraluminal Device, Open Approach */
"04V90ZZ"  =  "1"  /* Restriction of Right Renal Artery, Open Approach */
"04V93DZ"  =  "1"  /* Restriction of Right Renal Artery with Intraluminal Device, Percutaneous Approach */
"04V93ZZ"  =  "1"  /* Restriction of Right Renal Artery, Percutaneous Approach */
"04V94DZ"  =  "1"  /* Restriction of Right Renal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04V94ZZ"  =  "1"  /* Restriction of Right Renal Artery, Percutaneous Endoscopic Approach */
"04VA0DZ"  =  "1"  /* Restriction of Left Renal Artery with Intraluminal Device, Open Approach */
"04VA0ZZ"  =  "1"  /* Restriction of Left Renal Artery, Open Approach */
"04VA3DZ"  =  "1"  /* Restriction of Left Renal Artery with Intraluminal Device, Percutaneous Approach */
"04VA3ZZ"  =  "1"  /* Restriction of Left Renal Artery, Percutaneous Approach */
"04VA4DZ"  =  "1"  /* Restriction of Left Renal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VA4ZZ"  =  "1"  /* Restriction of Left Renal Artery, Percutaneous Endoscopic Approach */
"04VB0DZ"  =  "1"  /* Restriction of Inferior Mesenteric Artery with Intraluminal Device, Open Approach */
"04VB0ZZ"  =  "1"  /* Restriction of Inferior Mesenteric Artery, Open Approach */
"04VB3DZ"  =  "1"  /* Restriction of Inferior Mesenteric Artery with Intraluminal Device, Percutaneous Approach */
"04VB3ZZ"  =  "1"  /* Restriction of Inferior Mesenteric Artery, Percutaneous Approach */
"04VB4DZ"  =  "1"  /* Restriction of Inferior Mesenteric Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VB4ZZ"  =  "1"  /* Restriction of Inferior Mesenteric Artery, Percutaneous Endoscopic Approach */
"04VC0DZ"  =  "1"  /* Restriction of Right Common Iliac Artery with Intraluminal Device, Open Approach */
"04VC0ZZ"  =  "1"  /* Restriction of Right Common Iliac Artery, Open Approach */
"04VC3DZ"  =  "1"  /* Restriction of Right Common Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04VC3ZZ"  =  "1"  /* Restriction of Right Common Iliac Artery, Percutaneous Approach */
"04VC4DZ"  =  "1"  /* Restriction of Right Common Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VC4ZZ"  =  "1"  /* Restriction of Right Common Iliac Artery, Percutaneous Endoscopic Approach */
"04VD0DZ"  =  "1"  /* Restriction of Left Common Iliac Artery with Intraluminal Device, Open Approach */
"04VD0ZZ"  =  "1"  /* Restriction of Left Common Iliac Artery, Open Approach */
"04VD3DZ"  =  "1"  /* Restriction of Left Common Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04VD3ZZ"  =  "1"  /* Restriction of Left Common Iliac Artery, Percutaneous Approach */
"04VD4DZ"  =  "1"  /* Restriction of Left Common Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VD4ZZ"  =  "1"  /* Restriction of Left Common Iliac Artery, Percutaneous Endoscopic Approach */
"04VE0DZ"  =  "1"  /* Restriction of Right Internal Iliac Artery with Intraluminal Device, Open Approach */
"04VE0ZZ"  =  "1"  /* Restriction of Right Internal Iliac Artery, Open Approach */
"04VE3DZ"  =  "1"  /* Restriction of Right Internal Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04VE3ZZ"  =  "1"  /* Restriction of Right Internal Iliac Artery, Percutaneous Approach */
"04VE4DZ"  =  "1"  /* Restriction of Right Internal Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VE4ZZ"  =  "1"  /* Restriction of Right Internal Iliac Artery, Percutaneous Endoscopic Approach */
"04VF0DZ"  =  "1"  /* Restriction of Left Internal Iliac Artery with Intraluminal Device, Open Approach */
"04VF0ZZ"  =  "1"  /* Restriction of Left Internal Iliac Artery, Open Approach */
"04VF3DZ"  =  "1"  /* Restriction of Left Internal Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04VF3ZZ"  =  "1"  /* Restriction of Left Internal Iliac Artery, Percutaneous Approach */
"04VF4DZ"  =  "1"  /* Restriction of Left Internal Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VF4ZZ"  =  "1"  /* Restriction of Left Internal Iliac Artery, Percutaneous Endoscopic Approach */
"04VH0DZ"  =  "1"  /* Restriction of Right External Iliac Artery with Intraluminal Device, Open Approach */
"04VH0ZZ"  =  "1"  /* Restriction of Right External Iliac Artery, Open Approach */
"04VH3DZ"  =  "1"  /* Restriction of Right External Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04VH3ZZ"  =  "1"  /* Restriction of Right External Iliac Artery, Percutaneous Approach */
"04VH4DZ"  =  "1"  /* Restriction of Right External Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VH4ZZ"  =  "1"  /* Restriction of Right External Iliac Artery, Percutaneous Endoscopic Approach */
"04VJ0DZ"  =  "1"  /* Restriction of Left External Iliac Artery with Intraluminal Device, Open Approach */
"04VJ0ZZ"  =  "1"  /* Restriction of Left External Iliac Artery, Open Approach */
"04VJ3DZ"  =  "1"  /* Restriction of Left External Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04VJ3ZZ"  =  "1"  /* Restriction of Left External Iliac Artery, Percutaneous Approach */
"04VJ4DZ"  =  "1"  /* Restriction of Left External Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VJ4ZZ"  =  "1"  /* Restriction of Left External Iliac Artery, Percutaneous Endoscopic Approach */
"04VK0DZ"  =  "1"  /* Restriction of Right Femoral Artery with Intraluminal Device, Open Approach */
"04VK0ZZ"  =  "1"  /* Restriction of Right Femoral Artery, Open Approach */
"04VK3DZ"  =  "1"  /* Restriction of Right Femoral Artery with Intraluminal Device, Percutaneous Approach */
"04VK3ZZ"  =  "1"  /* Restriction of Right Femoral Artery, Percutaneous Approach */
"04VK4DZ"  =  "1"  /* Restriction of Right Femoral Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VK4ZZ"  =  "1"  /* Restriction of Right Femoral Artery, Percutaneous Endoscopic Approach */
"04VL0DZ"  =  "1"  /* Restriction of Left Femoral Artery with Intraluminal Device, Open Approach */
"04VL0ZZ"  =  "1"  /* Restriction of Left Femoral Artery, Open Approach */
"04VL3DZ"  =  "1"  /* Restriction of Left Femoral Artery with Intraluminal Device, Percutaneous Approach */
"04VL3ZZ"  =  "1"  /* Restriction of Left Femoral Artery, Percutaneous Approach */
"04VL4DZ"  =  "1"  /* Restriction of Left Femoral Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VL4ZZ"  =  "1"  /* Restriction of Left Femoral Artery, Percutaneous Endoscopic Approach */
"04VM0DZ"  =  "1"  /* Restriction of Right Popliteal Artery with Intraluminal Device, Open Approach */
"04VM0ZZ"  =  "1"  /* Restriction of Right Popliteal Artery, Open Approach */
"04VM3DZ"  =  "1"  /* Restriction of Right Popliteal Artery with Intraluminal Device, Percutaneous Approach */
"04VM3ZZ"  =  "1"  /* Restriction of Right Popliteal Artery, Percutaneous Approach */
"04VM4DZ"  =  "1"  /* Restriction of Right Popliteal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VM4ZZ"  =  "1"  /* Restriction of Right Popliteal Artery, Percutaneous Endoscopic Approach */
"04VN0DZ"  =  "1"  /* Restriction of Left Popliteal Artery with Intraluminal Device, Open Approach */
"04VN0ZZ"  =  "1"  /* Restriction of Left Popliteal Artery, Open Approach */
"04VN3DZ"  =  "1"  /* Restriction of Left Popliteal Artery with Intraluminal Device, Percutaneous Approach */
"04VN3ZZ"  =  "1"  /* Restriction of Left Popliteal Artery, Percutaneous Approach */
"04VN4DZ"  =  "1"  /* Restriction of Left Popliteal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VN4ZZ"  =  "1"  /* Restriction of Left Popliteal Artery, Percutaneous Endoscopic Approach */
"04VP0DZ"  =  "1"  /* Restriction of Right Anterior Tibial Artery with Intraluminal Device, Open Approach */
"04VP0ZZ"  =  "1"  /* Restriction of Right Anterior Tibial Artery, Open Approach */
"04VP3DZ"  =  "1"  /* Restriction of Right Anterior Tibial Artery with Intraluminal Device, Percutaneous Approach */
"04VP3ZZ"  =  "1"  /* Restriction of Right Anterior Tibial Artery, Percutaneous Approach */
"04VP4DZ"  =  "1"  /* Restriction of Right Anterior Tibial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VP4ZZ"  =  "1"  /* Restriction of Right Anterior Tibial Artery, Percutaneous Endoscopic Approach */
"04VQ0DZ"  =  "1"  /* Restriction of Left Anterior Tibial Artery with Intraluminal Device, Open Approach */
"04VQ0ZZ"  =  "1"  /* Restriction of Left Anterior Tibial Artery, Open Approach */
"04VQ3DZ"  =  "1"  /* Restriction of Left Anterior Tibial Artery with Intraluminal Device, Percutaneous Approach */
"04VQ3ZZ"  =  "1"  /* Restriction of Left Anterior Tibial Artery, Percutaneous Approach */
"04VQ4DZ"  =  "1"  /* Restriction of Left Anterior Tibial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VQ4ZZ"  =  "1"  /* Restriction of Left Anterior Tibial Artery, Percutaneous Endoscopic Approach */
"04VR0DZ"  =  "1"  /* Restriction of Right Posterior Tibial Artery with Intraluminal Device, Open Approach */
"04VR0ZZ"  =  "1"  /* Restriction of Right Posterior Tibial Artery, Open Approach */
"04VR3DZ"  =  "1"  /* Restriction of Right Posterior Tibial Artery with Intraluminal Device, Percutaneous Approach */
"04VR3ZZ"  =  "1"  /* Restriction of Right Posterior Tibial Artery, Percutaneous Approach */
"04VR4DZ"  =  "1"  /* Restriction of Right Posterior Tibial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VR4ZZ"  =  "1"  /* Restriction of Right Posterior Tibial Artery, Percutaneous Endoscopic Approach */
"04VS0DZ"  =  "1"  /* Restriction of Left Posterior Tibial Artery with Intraluminal Device, Open Approach */
"04VS0ZZ"  =  "1"  /* Restriction of Left Posterior Tibial Artery, Open Approach */
"04VS3DZ"  =  "1"  /* Restriction of Left Posterior Tibial Artery with Intraluminal Device, Percutaneous Approach */
"04VS3ZZ"  =  "1"  /* Restriction of Left Posterior Tibial Artery, Percutaneous Approach */
"04VS4DZ"  =  "1"  /* Restriction of Left Posterior Tibial Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VS4ZZ"  =  "1"  /* Restriction of Left Posterior Tibial Artery, Percutaneous Endoscopic Approach */
"04VT0DZ"  =  "1"  /* Restriction of Right Peroneal Artery with Intraluminal Device, Open Approach */
"04VT0ZZ"  =  "1"  /* Restriction of Right Peroneal Artery, Open Approach */
"04VT3DZ"  =  "1"  /* Restriction of Right Peroneal Artery with Intraluminal Device, Percutaneous Approach */
"04VT3ZZ"  =  "1"  /* Restriction of Right Peroneal Artery, Percutaneous Approach */
"04VT4DZ"  =  "1"  /* Restriction of Right Peroneal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VT4ZZ"  =  "1"  /* Restriction of Right Peroneal Artery, Percutaneous Endoscopic Approach */
"04VU0DZ"  =  "1"  /* Restriction of Left Peroneal Artery with Intraluminal Device, Open Approach */
"04VU0ZZ"  =  "1"  /* Restriction of Left Peroneal Artery, Open Approach */
"04VU3DZ"  =  "1"  /* Restriction of Left Peroneal Artery with Intraluminal Device, Percutaneous Approach */
"04VU3ZZ"  =  "1"  /* Restriction of Left Peroneal Artery, Percutaneous Approach */
"04VU4DZ"  =  "1"  /* Restriction of Left Peroneal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VU4ZZ"  =  "1"  /* Restriction of Left Peroneal Artery, Percutaneous Endoscopic Approach */
"04VV0DZ"  =  "1"  /* Restriction of Right Foot Artery with Intraluminal Device, Open Approach */
"04VV0ZZ"  =  "1"  /* Restriction of Right Foot Artery, Open Approach */
"04VV3DZ"  =  "1"  /* Restriction of Right Foot Artery with Intraluminal Device, Percutaneous Approach */
"04VV3ZZ"  =  "1"  /* Restriction of Right Foot Artery, Percutaneous Approach */
"04VV4DZ"  =  "1"  /* Restriction of Right Foot Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VV4ZZ"  =  "1"  /* Restriction of Right Foot Artery, Percutaneous Endoscopic Approach */
"04VW0DZ"  =  "1"  /* Restriction of Left Foot Artery with Intraluminal Device, Open Approach */
"04VW0ZZ"  =  "1"  /* Restriction of Left Foot Artery, Open Approach */
"04VW3DZ"  =  "1"  /* Restriction of Left Foot Artery with Intraluminal Device, Percutaneous Approach */
"04VW3ZZ"  =  "1"  /* Restriction of Left Foot Artery, Percutaneous Approach */
"04VW4DZ"  =  "1"  /* Restriction of Left Foot Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VW4ZZ"  =  "1"  /* Restriction of Left Foot Artery, Percutaneous Endoscopic Approach */
"04VY0DZ"  =  "1"  /* Restriction of Lower Artery with Intraluminal Device, Open Approach */
"04VY0ZZ"  =  "1"  /* Restriction of Lower Artery, Open Approach */
"04VY3DZ"  =  "1"  /* Restriction of Lower Artery with Intraluminal Device, Percutaneous Approach */
"04VY3ZZ"  =  "1"  /* Restriction of Lower Artery, Percutaneous Approach */
"04VY4DZ"  =  "1"  /* Restriction of Lower Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04VY4ZZ"  =  "1"  /* Restriction of Lower Artery, Percutaneous Endoscopic Approach */
"04WY00Z"  =  "1"  /* Revision of Drainage Device in Lower Artery, Open Approach */
"04WY02Z"  =  "1"  /* Revision of Monitoring Device in Lower Artery, Open Approach */
"04WY03Z"  =  "1"  /* Revision of Infusion Device in Lower Artery, Open Approach */
"04WY07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Artery, Open Approach */
"04WY0CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Artery, Open Approach */
"04WY0DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Artery, Open Approach */
"04WY0JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Artery, Open Approach */
"04WY0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Artery, Open Approach */
"04WY30Z"  =  "1"  /* Revision of Drainage Device in Lower Artery, Percutaneous Approach */
"04WY32Z"  =  "1"  /* Revision of Monitoring Device in Lower Artery, Percutaneous Approach */
"04WY33Z"  =  "1"  /* Revision of Infusion Device in Lower Artery, Percutaneous Approach */
"04WY37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Artery, Percutaneous Approach */
"04WY3CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Artery, Percutaneous Approach */
"04WY3DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Artery, Percutaneous Approach */
"04WY3JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Artery, Percutaneous Approach */
"04WY3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Artery, Percutaneous Approach */
"04WY40Z"  =  "1"  /* Revision of Drainage Device in Lower Artery, Percutaneous Endoscopic Approach */
"04WY42Z"  =  "1"  /* Revision of Monitoring Device in Lower Artery, Percutaneous Endoscopic Approach */
"04WY43Z"  =  "1"  /* Revision of Infusion Device in Lower Artery, Percutaneous Endoscopic Approach */
"04WY47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Artery, Percutaneous Endoscopic Approach */
"04WY4CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Artery, Percutaneous Endoscopic Approach */
"04WY4DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Artery, Percutaneous Endoscopic Approach */
"04WY4JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Artery, Percutaneous Endoscopic Approach */
"04WY4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Artery, Percutaneous Endoscopic Approach */
"059Y00Z"  =  "1"  /* Drainage of Upper Vein with Drainage Device, Open Approach */
"059Y0ZZ"  =  "1"  /* Drainage of Upper Vein, Open Approach */
"059Y30Z"  =  "1"  /* Drainage of Upper Vein with Drainage Device, Percutaneous Approach */
"059Y3ZZ"  =  "1"  /* Drainage of Upper Vein, Percutaneous Approach */
"05C00ZZ"  =  "1"  /* Extirpation of Matter from Azygos Vein, Open Approach */
"05C03ZZ"  =  "1"  /* Extirpation of Matter from Azygos Vein, Percutaneous Approach */
"05C04ZZ"  =  "1"  /* Extirpation of Matter from Azygos Vein, Percutaneous Endoscopic Approach */
"05C10ZZ"  =  "1"  /* Extirpation of Matter from Hemiazygos Vein, Open Approach */
"05C13ZZ"  =  "1"  /* Extirpation of Matter from Hemiazygos Vein, Percutaneous Approach */
"05C14ZZ"  =  "1"  /* Extirpation of Matter from Hemiazygos Vein, Percutaneous Endoscopic Approach */
"05C30ZZ"  =  "1"  /* Extirpation of Matter from Right Innominate Vein, Open Approach */
"05C33ZZ"  =  "1"  /* Extirpation of Matter from Right Innominate Vein, Percutaneous Approach */
"05C34ZZ"  =  "1"  /* Extirpation of Matter from Right Innominate Vein, Percutaneous Endoscopic Approach */
"05C40ZZ"  =  "1"  /* Extirpation of Matter from Left Innominate Vein, Open Approach */
"05C43ZZ"  =  "1"  /* Extirpation of Matter from Left Innominate Vein, Percutaneous Approach */
"05C44ZZ"  =  "1"  /* Extirpation of Matter from Left Innominate Vein, Percutaneous Endoscopic Approach */
"05C50ZZ"  =  "1"  /* Extirpation of Matter from Right Subclavian Vein, Open Approach */
"05C53ZZ"  =  "1"  /* Extirpation of Matter from Right Subclavian Vein, Percutaneous Approach */
"05C54ZZ"  =  "1"  /* Extirpation of Matter from Right Subclavian Vein, Percutaneous Endoscopic Approach */
"05C60ZZ"  =  "1"  /* Extirpation of Matter from Left Subclavian Vein, Open Approach */
"05C63ZZ"  =  "1"  /* Extirpation of Matter from Left Subclavian Vein, Percutaneous Approach */
"05C64ZZ"  =  "1"  /* Extirpation of Matter from Left Subclavian Vein, Percutaneous Endoscopic Approach */
"05C70ZZ"  =  "1"  /* Extirpation of Matter from Right Axillary Vein, Open Approach */
"05C73ZZ"  =  "1"  /* Extirpation of Matter from Right Axillary Vein, Percutaneous Approach */
"05C74ZZ"  =  "1"  /* Extirpation of Matter from Right Axillary Vein, Percutaneous Endoscopic Approach */
"05C80ZZ"  =  "1"  /* Extirpation of Matter from Left Axillary Vein, Open Approach */
"05C83ZZ"  =  "1"  /* Extirpation of Matter from Left Axillary Vein, Percutaneous Approach */
"05C84ZZ"  =  "1"  /* Extirpation of Matter from Left Axillary Vein, Percutaneous Endoscopic Approach */
"05C90ZZ"  =  "1"  /* Extirpation of Matter from Right Brachial Vein, Open Approach */
"05C93ZZ"  =  "1"  /* Extirpation of Matter from Right Brachial Vein, Percutaneous Approach */
"05C94ZZ"  =  "1"  /* Extirpation of Matter from Right Brachial Vein, Percutaneous Endoscopic Approach */
"05CA0ZZ"  =  "1"  /* Extirpation of Matter from Left Brachial Vein, Open Approach */
"05CA3ZZ"  =  "1"  /* Extirpation of Matter from Left Brachial Vein, Percutaneous Approach */
"05CA4ZZ"  =  "1"  /* Extirpation of Matter from Left Brachial Vein, Percutaneous Endoscopic Approach */
"05CB0ZZ"  =  "1"  /* Extirpation of Matter from Right Basilic Vein, Open Approach */
"05CB3ZZ"  =  "1"  /* Extirpation of Matter from Right Basilic Vein, Percutaneous Approach */
"05CB4ZZ"  =  "1"  /* Extirpation of Matter from Right Basilic Vein, Percutaneous Endoscopic Approach */
"05CC0ZZ"  =  "1"  /* Extirpation of Matter from Left Basilic Vein, Open Approach */
"05CC3ZZ"  =  "1"  /* Extirpation of Matter from Left Basilic Vein, Percutaneous Approach */
"05CC4ZZ"  =  "1"  /* Extirpation of Matter from Left Basilic Vein, Percutaneous Endoscopic Approach */
"05CD0ZZ"  =  "1"  /* Extirpation of Matter from Right Cephalic Vein, Open Approach */
"05CD3ZZ"  =  "1"  /* Extirpation of Matter from Right Cephalic Vein, Percutaneous Approach */
"05CD4ZZ"  =  "1"  /* Extirpation of Matter from Right Cephalic Vein, Percutaneous Endoscopic Approach */
"05CF0ZZ"  =  "1"  /* Extirpation of Matter from Left Cephalic Vein, Open Approach */
"05CF3ZZ"  =  "1"  /* Extirpation of Matter from Left Cephalic Vein, Percutaneous Approach */
"05CF4ZZ"  =  "1"  /* Extirpation of Matter from Left Cephalic Vein, Percutaneous Endoscopic Approach */
"05CG0ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Vein, Open Approach */
"05CG3ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Vein, Percutaneous Approach */
"05CG4ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Vein, Percutaneous Endoscopic Approach */
"05CH0ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Vein, Open Approach */
"05CH3ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Vein, Percutaneous Approach */
"05CH4ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Vein, Percutaneous Endoscopic Approach */
"05CL0ZZ"  =  "1"  /* Extirpation of Matter from Intracranial Vein, Open Approach */
"05CL4ZZ"  =  "1"  /* Extirpation of Matter from Intracranial Vein, Percutaneous Endoscopic Approach */
"05CM0ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Jugular Vein, Open Approach */
"05CM3ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Jugular Vein, Percutaneous Approach */
"05CM4ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Jugular Vein, Percutaneous Endoscopic Approach */
"05CN0ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Jugular Vein, Open Approach */
"05CN3ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Jugular Vein, Percutaneous Approach */
"05CN4ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Jugular Vein, Percutaneous Endoscopic Approach */
"05CP0ZZ"  =  "1"  /* Extirpation of Matter from Right External Jugular Vein, Open Approach */
"05CP3ZZ"  =  "1"  /* Extirpation of Matter from Right External Jugular Vein, Percutaneous Approach */
"05CP4ZZ"  =  "1"  /* Extirpation of Matter from Right External Jugular Vein, Percutaneous Endoscopic Approach */
"05CQ0ZZ"  =  "1"  /* Extirpation of Matter from Left External Jugular Vein, Open Approach */
"05CQ3ZZ"  =  "1"  /* Extirpation of Matter from Left External Jugular Vein, Percutaneous Approach */
"05CQ4ZZ"  =  "1"  /* Extirpation of Matter from Left External Jugular Vein, Percutaneous Endoscopic Approach */
"05CR0ZZ"  =  "1"  /* Extirpation of Matter from Right Vertebral Vein, Open Approach */
"05CR3ZZ"  =  "1"  /* Extirpation of Matter from Right Vertebral Vein, Percutaneous Approach */
"05CR4ZZ"  =  "1"  /* Extirpation of Matter from Right Vertebral Vein, Percutaneous Endoscopic Approach */
"05CS0ZZ"  =  "1"  /* Extirpation of Matter from Left Vertebral Vein, Open Approach */
"05CS3ZZ"  =  "1"  /* Extirpation of Matter from Left Vertebral Vein, Percutaneous Approach */
"05CS4ZZ"  =  "1"  /* Extirpation of Matter from Left Vertebral Vein, Percutaneous Endoscopic Approach */
"05CT0ZZ"  =  "1"  /* Extirpation of Matter from Right Face Vein, Open Approach */
"05CT3ZZ"  =  "1"  /* Extirpation of Matter from Right Face Vein, Percutaneous Approach */
"05CT4ZZ"  =  "1"  /* Extirpation of Matter from Right Face Vein, Percutaneous Endoscopic Approach */
"05CV0ZZ"  =  "1"  /* Extirpation of Matter from Left Face Vein, Open Approach */
"05CV3ZZ"  =  "1"  /* Extirpation of Matter from Left Face Vein, Percutaneous Approach */
"05CV4ZZ"  =  "1"  /* Extirpation of Matter from Left Face Vein, Percutaneous Endoscopic Approach */
"05CY0ZZ"  =  "1"  /* Extirpation of Matter from Upper Vein, Open Approach */
"05CY3ZZ"  =  "1"  /* Extirpation of Matter from Upper Vein, Percutaneous Approach */
"05CY4ZZ"  =  "1"  /* Extirpation of Matter from Upper Vein, Percutaneous Endoscopic Approach */
"05D90ZZ"  =  "1"  /* Extraction of Right Brachial Vein, Open Approach */
"05D93ZZ"  =  "1"  /* Extraction of Right Brachial Vein, Percutaneous Approach */
"05DA0ZZ"  =  "1"  /* Extraction of Left Brachial Vein, Open Approach */
"05DA3ZZ"  =  "1"  /* Extraction of Left Brachial Vein, Percutaneous Approach */
"05DB0ZZ"  =  "1"  /* Extraction of Right Basilic Vein, Open Approach */
"05DB3ZZ"  =  "1"  /* Extraction of Right Basilic Vein, Percutaneous Approach */
"05DC0ZZ"  =  "1"  /* Extraction of Left Basilic Vein, Open Approach */
"05DC3ZZ"  =  "1"  /* Extraction of Left Basilic Vein, Percutaneous Approach */
"05DD0ZZ"  =  "1"  /* Extraction of Right Cephalic Vein, Open Approach */
"05DD3ZZ"  =  "1"  /* Extraction of Right Cephalic Vein, Percutaneous Approach */
"05DF0ZZ"  =  "1"  /* Extraction of Left Cephalic Vein, Open Approach */
"05DF3ZZ"  =  "1"  /* Extraction of Left Cephalic Vein, Percutaneous Approach */
"05DG0ZZ"  =  "1"  /* Extraction of Right Hand Vein, Open Approach */
"05DG3ZZ"  =  "1"  /* Extraction of Right Hand Vein, Percutaneous Approach */
"05DH0ZZ"  =  "1"  /* Extraction of Left Hand Vein, Open Approach */
"05DH3ZZ"  =  "1"  /* Extraction of Left Hand Vein, Percutaneous Approach */
"05DY0ZZ"  =  "1"  /* Extraction of Upper Vein, Open Approach */
"05DY3ZZ"  =  "1"  /* Extraction of Upper Vein, Percutaneous Approach */
"05HY02Z"  =  "1"  /* Insertion of Monitoring Device into Upper Vein, Open Approach */
"05HY32Z"  =  "1"  /* Insertion of Monitoring Device into Upper Vein, Percutaneous Approach */
"05HY42Z"  =  "1"  /* Insertion of Monitoring Device into Upper Vein, Percutaneous Endoscopic Approach */
"05L00CZ"  =  "1"  /* Occlusion of Azygos Vein with Extraluminal Device, Open Approach */
"05L00DZ"  =  "1"  /* Occlusion of Azygos Vein with Intraluminal Device, Open Approach */
"05L00ZZ"  =  "1"  /* Occlusion of Azygos Vein, Open Approach */
"05L03CZ"  =  "1"  /* Occlusion of Azygos Vein with Extraluminal Device, Percutaneous Approach */
"05L03DZ"  =  "1"  /* Occlusion of Azygos Vein with Intraluminal Device, Percutaneous Approach */
"05L03ZZ"  =  "1"  /* Occlusion of Azygos Vein, Percutaneous Approach */
"05L04CZ"  =  "1"  /* Occlusion of Azygos Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05L04DZ"  =  "1"  /* Occlusion of Azygos Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05L04ZZ"  =  "1"  /* Occlusion of Azygos Vein, Percutaneous Endoscopic Approach */
"05L10CZ"  =  "1"  /* Occlusion of Hemiazygos Vein with Extraluminal Device, Open Approach */
"05L10DZ"  =  "1"  /* Occlusion of Hemiazygos Vein with Intraluminal Device, Open Approach */
"05L10ZZ"  =  "1"  /* Occlusion of Hemiazygos Vein, Open Approach */
"05L13CZ"  =  "1"  /* Occlusion of Hemiazygos Vein with Extraluminal Device, Percutaneous Approach */
"05L13DZ"  =  "1"  /* Occlusion of Hemiazygos Vein with Intraluminal Device, Percutaneous Approach */
"05L13ZZ"  =  "1"  /* Occlusion of Hemiazygos Vein, Percutaneous Approach */
"05L14CZ"  =  "1"  /* Occlusion of Hemiazygos Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05L14DZ"  =  "1"  /* Occlusion of Hemiazygos Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05L14ZZ"  =  "1"  /* Occlusion of Hemiazygos Vein, Percutaneous Endoscopic Approach */
"05L30CZ"  =  "1"  /* Occlusion of Right Innominate Vein with Extraluminal Device, Open Approach */
"05L30DZ"  =  "1"  /* Occlusion of Right Innominate Vein with Intraluminal Device, Open Approach */
"05L30ZZ"  =  "1"  /* Occlusion of Right Innominate Vein, Open Approach */
"05L33CZ"  =  "1"  /* Occlusion of Right Innominate Vein with Extraluminal Device, Percutaneous Approach */
"05L33DZ"  =  "1"  /* Occlusion of Right Innominate Vein with Intraluminal Device, Percutaneous Approach */
"05L33ZZ"  =  "1"  /* Occlusion of Right Innominate Vein, Percutaneous Approach */
"05L34CZ"  =  "1"  /* Occlusion of Right Innominate Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05L34DZ"  =  "1"  /* Occlusion of Right Innominate Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05L34ZZ"  =  "1"  /* Occlusion of Right Innominate Vein, Percutaneous Endoscopic Approach */
"05L40CZ"  =  "1"  /* Occlusion of Left Innominate Vein with Extraluminal Device, Open Approach */
"05L40DZ"  =  "1"  /* Occlusion of Left Innominate Vein with Intraluminal Device, Open Approach */
"05L40ZZ"  =  "1"  /* Occlusion of Left Innominate Vein, Open Approach */
"05L43CZ"  =  "1"  /* Occlusion of Left Innominate Vein with Extraluminal Device, Percutaneous Approach */
"05L43DZ"  =  "1"  /* Occlusion of Left Innominate Vein with Intraluminal Device, Percutaneous Approach */
"05L43ZZ"  =  "1"  /* Occlusion of Left Innominate Vein, Percutaneous Approach */
"05L44CZ"  =  "1"  /* Occlusion of Left Innominate Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05L44DZ"  =  "1"  /* Occlusion of Left Innominate Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05L44ZZ"  =  "1"  /* Occlusion of Left Innominate Vein, Percutaneous Endoscopic Approach */
"05L50CZ"  =  "1"  /* Occlusion of Right Subclavian Vein with Extraluminal Device, Open Approach */
"05L50DZ"  =  "1"  /* Occlusion of Right Subclavian Vein with Intraluminal Device, Open Approach */
"05L50ZZ"  =  "1"  /* Occlusion of Right Subclavian Vein, Open Approach */
"05L53CZ"  =  "1"  /* Occlusion of Right Subclavian Vein with Extraluminal Device, Percutaneous Approach */
"05L53DZ"  =  "1"  /* Occlusion of Right Subclavian Vein with Intraluminal Device, Percutaneous Approach */
"05L53ZZ"  =  "1"  /* Occlusion of Right Subclavian Vein, Percutaneous Approach */
"05L54CZ"  =  "1"  /* Occlusion of Right Subclavian Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05L54DZ"  =  "1"  /* Occlusion of Right Subclavian Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05L54ZZ"  =  "1"  /* Occlusion of Right Subclavian Vein, Percutaneous Endoscopic Approach */
"05L60CZ"  =  "1"  /* Occlusion of Left Subclavian Vein with Extraluminal Device, Open Approach */
"05L60DZ"  =  "1"  /* Occlusion of Left Subclavian Vein with Intraluminal Device, Open Approach */
"05L60ZZ"  =  "1"  /* Occlusion of Left Subclavian Vein, Open Approach */
"05L63CZ"  =  "1"  /* Occlusion of Left Subclavian Vein with Extraluminal Device, Percutaneous Approach */
"05L63DZ"  =  "1"  /* Occlusion of Left Subclavian Vein with Intraluminal Device, Percutaneous Approach */
"05L63ZZ"  =  "1"  /* Occlusion of Left Subclavian Vein, Percutaneous Approach */
"05L64CZ"  =  "1"  /* Occlusion of Left Subclavian Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05L64DZ"  =  "1"  /* Occlusion of Left Subclavian Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05L64ZZ"  =  "1"  /* Occlusion of Left Subclavian Vein, Percutaneous Endoscopic Approach */
"05L70CZ"  =  "1"  /* Occlusion of Right Axillary Vein with Extraluminal Device, Open Approach */
"05L70DZ"  =  "1"  /* Occlusion of Right Axillary Vein with Intraluminal Device, Open Approach */
"05L70ZZ"  =  "1"  /* Occlusion of Right Axillary Vein, Open Approach */
"05L73CZ"  =  "1"  /* Occlusion of Right Axillary Vein with Extraluminal Device, Percutaneous Approach */
"05L73DZ"  =  "1"  /* Occlusion of Right Axillary Vein with Intraluminal Device, Percutaneous Approach */
"05L73ZZ"  =  "1"  /* Occlusion of Right Axillary Vein, Percutaneous Approach */
"05L74CZ"  =  "1"  /* Occlusion of Right Axillary Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05L74DZ"  =  "1"  /* Occlusion of Right Axillary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05L74ZZ"  =  "1"  /* Occlusion of Right Axillary Vein, Percutaneous Endoscopic Approach */
"05L80CZ"  =  "1"  /* Occlusion of Left Axillary Vein with Extraluminal Device, Open Approach */
"05L80DZ"  =  "1"  /* Occlusion of Left Axillary Vein with Intraluminal Device, Open Approach */
"05L80ZZ"  =  "1"  /* Occlusion of Left Axillary Vein, Open Approach */
"05L83CZ"  =  "1"  /* Occlusion of Left Axillary Vein with Extraluminal Device, Percutaneous Approach */
"05L83DZ"  =  "1"  /* Occlusion of Left Axillary Vein with Intraluminal Device, Percutaneous Approach */
"05L83ZZ"  =  "1"  /* Occlusion of Left Axillary Vein, Percutaneous Approach */
"05L84CZ"  =  "1"  /* Occlusion of Left Axillary Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05L84DZ"  =  "1"  /* Occlusion of Left Axillary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05L84ZZ"  =  "1"  /* Occlusion of Left Axillary Vein, Percutaneous Endoscopic Approach */
"05L90CZ"  =  "1"  /* Occlusion of Right Brachial Vein with Extraluminal Device, Open Approach */
"05L90DZ"  =  "1"  /* Occlusion of Right Brachial Vein with Intraluminal Device, Open Approach */
"05L90ZZ"  =  "1"  /* Occlusion of Right Brachial Vein, Open Approach */
"05L93CZ"  =  "1"  /* Occlusion of Right Brachial Vein with Extraluminal Device, Percutaneous Approach */
"05L93DZ"  =  "1"  /* Occlusion of Right Brachial Vein with Intraluminal Device, Percutaneous Approach */
"05L93ZZ"  =  "1"  /* Occlusion of Right Brachial Vein, Percutaneous Approach */
"05L94CZ"  =  "1"  /* Occlusion of Right Brachial Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05L94DZ"  =  "1"  /* Occlusion of Right Brachial Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05L94ZZ"  =  "1"  /* Occlusion of Right Brachial Vein, Percutaneous Endoscopic Approach */
"05LA0CZ"  =  "1"  /* Occlusion of Left Brachial Vein with Extraluminal Device, Open Approach */
"05LA0DZ"  =  "1"  /* Occlusion of Left Brachial Vein with Intraluminal Device, Open Approach */
"05LA0ZZ"  =  "1"  /* Occlusion of Left Brachial Vein, Open Approach */
"05LA3CZ"  =  "1"  /* Occlusion of Left Brachial Vein with Extraluminal Device, Percutaneous Approach */
"05LA3DZ"  =  "1"  /* Occlusion of Left Brachial Vein with Intraluminal Device, Percutaneous Approach */
"05LA3ZZ"  =  "1"  /* Occlusion of Left Brachial Vein, Percutaneous Approach */
"05LA4CZ"  =  "1"  /* Occlusion of Left Brachial Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LA4DZ"  =  "1"  /* Occlusion of Left Brachial Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LA4ZZ"  =  "1"  /* Occlusion of Left Brachial Vein, Percutaneous Endoscopic Approach */
"05LB0CZ"  =  "1"  /* Occlusion of Right Basilic Vein with Extraluminal Device, Open Approach */
"05LB0DZ"  =  "1"  /* Occlusion of Right Basilic Vein with Intraluminal Device, Open Approach */
"05LB0ZZ"  =  "1"  /* Occlusion of Right Basilic Vein, Open Approach */
"05LB3CZ"  =  "1"  /* Occlusion of Right Basilic Vein with Extraluminal Device, Percutaneous Approach */
"05LB3DZ"  =  "1"  /* Occlusion of Right Basilic Vein with Intraluminal Device, Percutaneous Approach */
"05LB3ZZ"  =  "1"  /* Occlusion of Right Basilic Vein, Percutaneous Approach */
"05LB4CZ"  =  "1"  /* Occlusion of Right Basilic Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LB4DZ"  =  "1"  /* Occlusion of Right Basilic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LB4ZZ"  =  "1"  /* Occlusion of Right Basilic Vein, Percutaneous Endoscopic Approach */
"05LC0CZ"  =  "1"  /* Occlusion of Left Basilic Vein with Extraluminal Device, Open Approach */
"05LC0DZ"  =  "1"  /* Occlusion of Left Basilic Vein with Intraluminal Device, Open Approach */
"05LC0ZZ"  =  "1"  /* Occlusion of Left Basilic Vein, Open Approach */
"05LC3CZ"  =  "1"  /* Occlusion of Left Basilic Vein with Extraluminal Device, Percutaneous Approach */
"05LC3DZ"  =  "1"  /* Occlusion of Left Basilic Vein with Intraluminal Device, Percutaneous Approach */
"05LC3ZZ"  =  "1"  /* Occlusion of Left Basilic Vein, Percutaneous Approach */
"05LC4CZ"  =  "1"  /* Occlusion of Left Basilic Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LC4DZ"  =  "1"  /* Occlusion of Left Basilic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LC4ZZ"  =  "1"  /* Occlusion of Left Basilic Vein, Percutaneous Endoscopic Approach */
"05LD0CZ"  =  "1"  /* Occlusion of Right Cephalic Vein with Extraluminal Device, Open Approach */
"05LD0DZ"  =  "1"  /* Occlusion of Right Cephalic Vein with Intraluminal Device, Open Approach */
"05LD0ZZ"  =  "1"  /* Occlusion of Right Cephalic Vein, Open Approach */
"05LD3CZ"  =  "1"  /* Occlusion of Right Cephalic Vein with Extraluminal Device, Percutaneous Approach */
"05LD3DZ"  =  "1"  /* Occlusion of Right Cephalic Vein with Intraluminal Device, Percutaneous Approach */
"05LD3ZZ"  =  "1"  /* Occlusion of Right Cephalic Vein, Percutaneous Approach */
"05LD4CZ"  =  "1"  /* Occlusion of Right Cephalic Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LD4DZ"  =  "1"  /* Occlusion of Right Cephalic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LD4ZZ"  =  "1"  /* Occlusion of Right Cephalic Vein, Percutaneous Endoscopic Approach */
"05LF0CZ"  =  "1"  /* Occlusion of Left Cephalic Vein with Extraluminal Device, Open Approach */
"05LF0DZ"  =  "1"  /* Occlusion of Left Cephalic Vein with Intraluminal Device, Open Approach */
"05LF0ZZ"  =  "1"  /* Occlusion of Left Cephalic Vein, Open Approach */
"05LF3CZ"  =  "1"  /* Occlusion of Left Cephalic Vein with Extraluminal Device, Percutaneous Approach */
"05LF3DZ"  =  "1"  /* Occlusion of Left Cephalic Vein with Intraluminal Device, Percutaneous Approach */
"05LF3ZZ"  =  "1"  /* Occlusion of Left Cephalic Vein, Percutaneous Approach */
"05LF4CZ"  =  "1"  /* Occlusion of Left Cephalic Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LF4DZ"  =  "1"  /* Occlusion of Left Cephalic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LF4ZZ"  =  "1"  /* Occlusion of Left Cephalic Vein, Percutaneous Endoscopic Approach */
"05LG0CZ"  =  "1"  /* Occlusion of Right Hand Vein with Extraluminal Device, Open Approach */
"05LG0DZ"  =  "1"  /* Occlusion of Right Hand Vein with Intraluminal Device, Open Approach */
"05LG0ZZ"  =  "1"  /* Occlusion of Right Hand Vein, Open Approach */
"05LG3CZ"  =  "1"  /* Occlusion of Right Hand Vein with Extraluminal Device, Percutaneous Approach */
"05LG3DZ"  =  "1"  /* Occlusion of Right Hand Vein with Intraluminal Device, Percutaneous Approach */
"05LG3ZZ"  =  "1"  /* Occlusion of Right Hand Vein, Percutaneous Approach */
"05LG4CZ"  =  "1"  /* Occlusion of Right Hand Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LG4DZ"  =  "1"  /* Occlusion of Right Hand Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LG4ZZ"  =  "1"  /* Occlusion of Right Hand Vein, Percutaneous Endoscopic Approach */
"05LH0CZ"  =  "1"  /* Occlusion of Left Hand Vein with Extraluminal Device, Open Approach */
"05LH0DZ"  =  "1"  /* Occlusion of Left Hand Vein with Intraluminal Device, Open Approach */
"05LH0ZZ"  =  "1"  /* Occlusion of Left Hand Vein, Open Approach */
"05LH3CZ"  =  "1"  /* Occlusion of Left Hand Vein with Extraluminal Device, Percutaneous Approach */
"05LH3DZ"  =  "1"  /* Occlusion of Left Hand Vein with Intraluminal Device, Percutaneous Approach */
"05LH3ZZ"  =  "1"  /* Occlusion of Left Hand Vein, Percutaneous Approach */
"05LH4CZ"  =  "1"  /* Occlusion of Left Hand Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LH4DZ"  =  "1"  /* Occlusion of Left Hand Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LH4ZZ"  =  "1"  /* Occlusion of Left Hand Vein, Percutaneous Endoscopic Approach */
"05LL0CZ"  =  "1"  /* Occlusion of Intracranial Vein with Extraluminal Device, Open Approach */
"05LL0DZ"  =  "1"  /* Occlusion of Intracranial Vein with Intraluminal Device, Open Approach */
"05LL0ZZ"  =  "1"  /* Occlusion of Intracranial Vein, Open Approach */
"05LL3CZ"  =  "1"  /* Occlusion of Intracranial Vein with Extraluminal Device, Percutaneous Approach */
"05LL3DZ"  =  "1"  /* Occlusion of Intracranial Vein with Intraluminal Device, Percutaneous Approach */
"05LL3ZZ"  =  "1"  /* Occlusion of Intracranial Vein, Percutaneous Approach */
"05LL4CZ"  =  "1"  /* Occlusion of Intracranial Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LL4DZ"  =  "1"  /* Occlusion of Intracranial Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LL4ZZ"  =  "1"  /* Occlusion of Intracranial Vein, Percutaneous Endoscopic Approach */
"05LM0CZ"  =  "1"  /* Occlusion of Right Internal Jugular Vein with Extraluminal Device, Open Approach */
"05LM0DZ"  =  "1"  /* Occlusion of Right Internal Jugular Vein with Intraluminal Device, Open Approach */
"05LM0ZZ"  =  "1"  /* Occlusion of Right Internal Jugular Vein, Open Approach */
"05LM3CZ"  =  "1"  /* Occlusion of Right Internal Jugular Vein with Extraluminal Device, Percutaneous Approach */
"05LM3DZ"  =  "1"  /* Occlusion of Right Internal Jugular Vein with Intraluminal Device, Percutaneous Approach */
"05LM3ZZ"  =  "1"  /* Occlusion of Right Internal Jugular Vein, Percutaneous Approach */
"05LM4CZ"  =  "1"  /* Occlusion of Right Internal Jugular Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LM4DZ"  =  "1"  /* Occlusion of Right Internal Jugular Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LM4ZZ"  =  "1"  /* Occlusion of Right Internal Jugular Vein, Percutaneous Endoscopic Approach */
"05LN0CZ"  =  "1"  /* Occlusion of Left Internal Jugular Vein with Extraluminal Device, Open Approach */
"05LN0DZ"  =  "1"  /* Occlusion of Left Internal Jugular Vein with Intraluminal Device, Open Approach */
"05LN0ZZ"  =  "1"  /* Occlusion of Left Internal Jugular Vein, Open Approach */
"05LN3CZ"  =  "1"  /* Occlusion of Left Internal Jugular Vein with Extraluminal Device, Percutaneous Approach */
"05LN3DZ"  =  "1"  /* Occlusion of Left Internal Jugular Vein with Intraluminal Device, Percutaneous Approach */
"05LN3ZZ"  =  "1"  /* Occlusion of Left Internal Jugular Vein, Percutaneous Approach */
"05LN4CZ"  =  "1"  /* Occlusion of Left Internal Jugular Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LN4DZ"  =  "1"  /* Occlusion of Left Internal Jugular Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LN4ZZ"  =  "1"  /* Occlusion of Left Internal Jugular Vein, Percutaneous Endoscopic Approach */
"05LP0CZ"  =  "1"  /* Occlusion of Right External Jugular Vein with Extraluminal Device, Open Approach */
"05LP0DZ"  =  "1"  /* Occlusion of Right External Jugular Vein with Intraluminal Device, Open Approach */
"05LP0ZZ"  =  "1"  /* Occlusion of Right External Jugular Vein, Open Approach */
"05LP3CZ"  =  "1"  /* Occlusion of Right External Jugular Vein with Extraluminal Device, Percutaneous Approach */
"05LP3DZ"  =  "1"  /* Occlusion of Right External Jugular Vein with Intraluminal Device, Percutaneous Approach */
"05LP3ZZ"  =  "1"  /* Occlusion of Right External Jugular Vein, Percutaneous Approach */
"05LP4CZ"  =  "1"  /* Occlusion of Right External Jugular Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LP4DZ"  =  "1"  /* Occlusion of Right External Jugular Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LP4ZZ"  =  "1"  /* Occlusion of Right External Jugular Vein, Percutaneous Endoscopic Approach */
"05LQ0CZ"  =  "1"  /* Occlusion of Left External Jugular Vein with Extraluminal Device, Open Approach */
"05LQ0DZ"  =  "1"  /* Occlusion of Left External Jugular Vein with Intraluminal Device, Open Approach */
"05LQ0ZZ"  =  "1"  /* Occlusion of Left External Jugular Vein, Open Approach */
"05LQ3CZ"  =  "1"  /* Occlusion of Left External Jugular Vein with Extraluminal Device, Percutaneous Approach */
"05LQ3DZ"  =  "1"  /* Occlusion of Left External Jugular Vein with Intraluminal Device, Percutaneous Approach */
"05LQ3ZZ"  =  "1"  /* Occlusion of Left External Jugular Vein, Percutaneous Approach */
"05LQ4CZ"  =  "1"  /* Occlusion of Left External Jugular Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LQ4DZ"  =  "1"  /* Occlusion of Left External Jugular Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LQ4ZZ"  =  "1"  /* Occlusion of Left External Jugular Vein, Percutaneous Endoscopic Approach */
"05LR0CZ"  =  "1"  /* Occlusion of Right Vertebral Vein with Extraluminal Device, Open Approach */
"05LR0DZ"  =  "1"  /* Occlusion of Right Vertebral Vein with Intraluminal Device, Open Approach */
"05LR0ZZ"  =  "1"  /* Occlusion of Right Vertebral Vein, Open Approach */
"05LR3CZ"  =  "1"  /* Occlusion of Right Vertebral Vein with Extraluminal Device, Percutaneous Approach */
"05LR3DZ"  =  "1"  /* Occlusion of Right Vertebral Vein with Intraluminal Device, Percutaneous Approach */
"05LR3ZZ"  =  "1"  /* Occlusion of Right Vertebral Vein, Percutaneous Approach */
"05LR4CZ"  =  "1"  /* Occlusion of Right Vertebral Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LR4DZ"  =  "1"  /* Occlusion of Right Vertebral Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LR4ZZ"  =  "1"  /* Occlusion of Right Vertebral Vein, Percutaneous Endoscopic Approach */
"05LS0CZ"  =  "1"  /* Occlusion of Left Vertebral Vein with Extraluminal Device, Open Approach */
"05LS0DZ"  =  "1"  /* Occlusion of Left Vertebral Vein with Intraluminal Device, Open Approach */
"05LS0ZZ"  =  "1"  /* Occlusion of Left Vertebral Vein, Open Approach */
"05LS3CZ"  =  "1"  /* Occlusion of Left Vertebral Vein with Extraluminal Device, Percutaneous Approach */
"05LS3DZ"  =  "1"  /* Occlusion of Left Vertebral Vein with Intraluminal Device, Percutaneous Approach */
"05LS3ZZ"  =  "1"  /* Occlusion of Left Vertebral Vein, Percutaneous Approach */
"05LS4CZ"  =  "1"  /* Occlusion of Left Vertebral Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LS4DZ"  =  "1"  /* Occlusion of Left Vertebral Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LS4ZZ"  =  "1"  /* Occlusion of Left Vertebral Vein, Percutaneous Endoscopic Approach */
"05LT0CZ"  =  "1"  /* Occlusion of Right Face Vein with Extraluminal Device, Open Approach */
"05LT0DZ"  =  "1"  /* Occlusion of Right Face Vein with Intraluminal Device, Open Approach */
"05LT0ZZ"  =  "1"  /* Occlusion of Right Face Vein, Open Approach */
"05LT3CZ"  =  "1"  /* Occlusion of Right Face Vein with Extraluminal Device, Percutaneous Approach */
"05LT3DZ"  =  "1"  /* Occlusion of Right Face Vein with Intraluminal Device, Percutaneous Approach */
"05LT3ZZ"  =  "1"  /* Occlusion of Right Face Vein, Percutaneous Approach */
"05LT4CZ"  =  "1"  /* Occlusion of Right Face Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LT4DZ"  =  "1"  /* Occlusion of Right Face Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LT4ZZ"  =  "1"  /* Occlusion of Right Face Vein, Percutaneous Endoscopic Approach */
"05LV0CZ"  =  "1"  /* Occlusion of Left Face Vein with Extraluminal Device, Open Approach */
"05LV0DZ"  =  "1"  /* Occlusion of Left Face Vein with Intraluminal Device, Open Approach */
"05LV0ZZ"  =  "1"  /* Occlusion of Left Face Vein, Open Approach */
"05LV3CZ"  =  "1"  /* Occlusion of Left Face Vein with Extraluminal Device, Percutaneous Approach */
"05LV3DZ"  =  "1"  /* Occlusion of Left Face Vein with Intraluminal Device, Percutaneous Approach */
"05LV3ZZ"  =  "1"  /* Occlusion of Left Face Vein, Percutaneous Approach */
"05LV4CZ"  =  "1"  /* Occlusion of Left Face Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LV4DZ"  =  "1"  /* Occlusion of Left Face Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LV4ZZ"  =  "1"  /* Occlusion of Left Face Vein, Percutaneous Endoscopic Approach */
"05LY0CZ"  =  "1"  /* Occlusion of Upper Vein with Extraluminal Device, Open Approach */
"05LY0DZ"  =  "1"  /* Occlusion of Upper Vein with Intraluminal Device, Open Approach */
"05LY0ZZ"  =  "1"  /* Occlusion of Upper Vein, Open Approach */
"05LY3CZ"  =  "1"  /* Occlusion of Upper Vein with Extraluminal Device, Percutaneous Approach */
"05LY3DZ"  =  "1"  /* Occlusion of Upper Vein with Intraluminal Device, Percutaneous Approach */
"05LY3ZZ"  =  "1"  /* Occlusion of Upper Vein, Percutaneous Approach */
"05LY4CZ"  =  "1"  /* Occlusion of Upper Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"05LY4DZ"  =  "1"  /* Occlusion of Upper Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05LY4ZZ"  =  "1"  /* Occlusion of Upper Vein, Percutaneous Endoscopic Approach */
"05PY00Z"  =  "1"  /* Removal of Drainage Device from Upper Vein, Open Approach */
"05PY02Z"  =  "1"  /* Removal of Monitoring Device from Upper Vein, Open Approach */
"05PY03Z"  =  "1"  /* Removal of Infusion Device from Upper Vein, Open Approach */
"05PY07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Vein, Open Approach */
"05PY0CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Vein, Open Approach */
"05PY0DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Vein, Open Approach */
"05PY0JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Vein, Open Approach */
"05PY0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Vein, Open Approach */
"05PY30Z"  =  "1"  /* Removal of Drainage Device from Upper Vein, Percutaneous Approach */
"05PY32Z"  =  "1"  /* Removal of Monitoring Device from Upper Vein, Percutaneous Approach */
"05PY33Z"  =  "1"  /* Removal of Infusion Device from Upper Vein, Percutaneous Approach */
"05PY37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Vein, Percutaneous Approach */
"05PY3CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Vein, Percutaneous Approach */
"05PY3DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Vein, Percutaneous Approach */
"05PY3JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Vein, Percutaneous Approach */
"05PY3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Vein, Percutaneous Approach */
"05PY40Z"  =  "1"  /* Removal of Drainage Device from Upper Vein, Percutaneous Endoscopic Approach */
"05PY42Z"  =  "1"  /* Removal of Monitoring Device from Upper Vein, Percutaneous Endoscopic Approach */
"05PY43Z"  =  "1"  /* Removal of Infusion Device from Upper Vein, Percutaneous Endoscopic Approach */
"05PY47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Vein, Percutaneous Endoscopic Approach */
"05PY4CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Vein, Percutaneous Endoscopic Approach */
"05PY4DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Vein, Percutaneous Endoscopic Approach */
"05PY4JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Vein, Percutaneous Endoscopic Approach */
"05PY4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Vein, Percutaneous Endoscopic Approach */
"05Q00ZZ"  =  "1"  /* Repair Azygos Vein, Open Approach */
"05Q03ZZ"  =  "1"  /* Repair Azygos Vein, Percutaneous Approach */
"05Q04ZZ"  =  "1"  /* Repair Azygos Vein, Percutaneous Endoscopic Approach */
"05Q10ZZ"  =  "1"  /* Repair Hemiazygos Vein, Open Approach */
"05Q13ZZ"  =  "1"  /* Repair Hemiazygos Vein, Percutaneous Approach */
"05Q14ZZ"  =  "1"  /* Repair Hemiazygos Vein, Percutaneous Endoscopic Approach */
"05Q30ZZ"  =  "1"  /* Repair Right Innominate Vein, Open Approach */
"05Q33ZZ"  =  "1"  /* Repair Right Innominate Vein, Percutaneous Approach */
"05Q34ZZ"  =  "1"  /* Repair Right Innominate Vein, Percutaneous Endoscopic Approach */
"05Q40ZZ"  =  "1"  /* Repair Left Innominate Vein, Open Approach */
"05Q43ZZ"  =  "1"  /* Repair Left Innominate Vein, Percutaneous Approach */
"05Q44ZZ"  =  "1"  /* Repair Left Innominate Vein, Percutaneous Endoscopic Approach */
"05Q50ZZ"  =  "1"  /* Repair Right Subclavian Vein, Open Approach */
"05Q53ZZ"  =  "1"  /* Repair Right Subclavian Vein, Percutaneous Approach */
"05Q54ZZ"  =  "1"  /* Repair Right Subclavian Vein, Percutaneous Endoscopic Approach */
"05Q60ZZ"  =  "1"  /* Repair Left Subclavian Vein, Open Approach */
"05Q63ZZ"  =  "1"  /* Repair Left Subclavian Vein, Percutaneous Approach */
"05Q64ZZ"  =  "1"  /* Repair Left Subclavian Vein, Percutaneous Endoscopic Approach */
"05Q70ZZ"  =  "1"  /* Repair Right Axillary Vein, Open Approach */
"05Q73ZZ"  =  "1"  /* Repair Right Axillary Vein, Percutaneous Approach */
"05Q74ZZ"  =  "1"  /* Repair Right Axillary Vein, Percutaneous Endoscopic Approach */
"05Q80ZZ"  =  "1"  /* Repair Left Axillary Vein, Open Approach */
"05Q83ZZ"  =  "1"  /* Repair Left Axillary Vein, Percutaneous Approach */
"05Q84ZZ"  =  "1"  /* Repair Left Axillary Vein, Percutaneous Endoscopic Approach */
"05Q90ZZ"  =  "1"  /* Repair Right Brachial Vein, Open Approach */
"05Q93ZZ"  =  "1"  /* Repair Right Brachial Vein, Percutaneous Approach */
"05Q94ZZ"  =  "1"  /* Repair Right Brachial Vein, Percutaneous Endoscopic Approach */
"05QA0ZZ"  =  "1"  /* Repair Left Brachial Vein, Open Approach */
"05QA3ZZ"  =  "1"  /* Repair Left Brachial Vein, Percutaneous Approach */
"05QA4ZZ"  =  "1"  /* Repair Left Brachial Vein, Percutaneous Endoscopic Approach */
"05QB0ZZ"  =  "1"  /* Repair Right Basilic Vein, Open Approach */
"05QB3ZZ"  =  "1"  /* Repair Right Basilic Vein, Percutaneous Approach */
"05QB4ZZ"  =  "1"  /* Repair Right Basilic Vein, Percutaneous Endoscopic Approach */
"05QC0ZZ"  =  "1"  /* Repair Left Basilic Vein, Open Approach */
"05QC3ZZ"  =  "1"  /* Repair Left Basilic Vein, Percutaneous Approach */
"05QC4ZZ"  =  "1"  /* Repair Left Basilic Vein, Percutaneous Endoscopic Approach */
"05QD0ZZ"  =  "1"  /* Repair Right Cephalic Vein, Open Approach */
"05QD3ZZ"  =  "1"  /* Repair Right Cephalic Vein, Percutaneous Approach */
"05QD4ZZ"  =  "1"  /* Repair Right Cephalic Vein, Percutaneous Endoscopic Approach */
"05QF0ZZ"  =  "1"  /* Repair Left Cephalic Vein, Open Approach */
"05QF3ZZ"  =  "1"  /* Repair Left Cephalic Vein, Percutaneous Approach */
"05QF4ZZ"  =  "1"  /* Repair Left Cephalic Vein, Percutaneous Endoscopic Approach */
"05QG0ZZ"  =  "1"  /* Repair Right Hand Vein, Open Approach */
"05QG3ZZ"  =  "1"  /* Repair Right Hand Vein, Percutaneous Approach */
"05QG4ZZ"  =  "1"  /* Repair Right Hand Vein, Percutaneous Endoscopic Approach */
"05QH0ZZ"  =  "1"  /* Repair Left Hand Vein, Open Approach */
"05QH3ZZ"  =  "1"  /* Repair Left Hand Vein, Percutaneous Approach */
"05QH4ZZ"  =  "1"  /* Repair Left Hand Vein, Percutaneous Endoscopic Approach */
"05QL0ZZ"  =  "1"  /* Repair Intracranial Vein, Open Approach */
"05QL3ZZ"  =  "1"  /* Repair Intracranial Vein, Percutaneous Approach */
"05QL4ZZ"  =  "1"  /* Repair Intracranial Vein, Percutaneous Endoscopic Approach */
"05QM0ZZ"  =  "1"  /* Repair Right Internal Jugular Vein, Open Approach */
"05QM3ZZ"  =  "1"  /* Repair Right Internal Jugular Vein, Percutaneous Approach */
"05QM4ZZ"  =  "1"  /* Repair Right Internal Jugular Vein, Percutaneous Endoscopic Approach */
"05QN0ZZ"  =  "1"  /* Repair Left Internal Jugular Vein, Open Approach */
"05QN3ZZ"  =  "1"  /* Repair Left Internal Jugular Vein, Percutaneous Approach */
"05QN4ZZ"  =  "1"  /* Repair Left Internal Jugular Vein, Percutaneous Endoscopic Approach */
"05QP0ZZ"  =  "1"  /* Repair Right External Jugular Vein, Open Approach */
"05QP3ZZ"  =  "1"  /* Repair Right External Jugular Vein, Percutaneous Approach */
"05QP4ZZ"  =  "1"  /* Repair Right External Jugular Vein, Percutaneous Endoscopic Approach */
"05QQ0ZZ"  =  "1"  /* Repair Left External Jugular Vein, Open Approach */
"05QQ3ZZ"  =  "1"  /* Repair Left External Jugular Vein, Percutaneous Approach */
"05QQ4ZZ"  =  "1"  /* Repair Left External Jugular Vein, Percutaneous Endoscopic Approach */
"05QR0ZZ"  =  "1"  /* Repair Right Vertebral Vein, Open Approach */
"05QR3ZZ"  =  "1"  /* Repair Right Vertebral Vein, Percutaneous Approach */
"05QR4ZZ"  =  "1"  /* Repair Right Vertebral Vein, Percutaneous Endoscopic Approach */
"05QS0ZZ"  =  "1"  /* Repair Left Vertebral Vein, Open Approach */
"05QS3ZZ"  =  "1"  /* Repair Left Vertebral Vein, Percutaneous Approach */
"05QS4ZZ"  =  "1"  /* Repair Left Vertebral Vein, Percutaneous Endoscopic Approach */
"05QT0ZZ"  =  "1"  /* Repair Right Face Vein, Open Approach */
"05QT3ZZ"  =  "1"  /* Repair Right Face Vein, Percutaneous Approach */
"05QT4ZZ"  =  "1"  /* Repair Right Face Vein, Percutaneous Endoscopic Approach */
"05QV0ZZ"  =  "1"  /* Repair Left Face Vein, Open Approach */
"05QV3ZZ"  =  "1"  /* Repair Left Face Vein, Percutaneous Approach */
"05QV4ZZ"  =  "1"  /* Repair Left Face Vein, Percutaneous Endoscopic Approach */
"05QY0ZZ"  =  "1"  /* Repair Upper Vein, Open Approach */
"05QY3ZZ"  =  "1"  /* Repair Upper Vein, Percutaneous Approach */
"05QY4ZZ"  =  "1"  /* Repair Upper Vein, Percutaneous Endoscopic Approach */
"05V00DZ"  =  "1"  /* Restriction of Azygos Vein with Intraluminal Device, Open Approach */
"05V00ZZ"  =  "1"  /* Restriction of Azygos Vein, Open Approach */
"05V03DZ"  =  "1"  /* Restriction of Azygos Vein with Intraluminal Device, Percutaneous Approach */
"05V03ZZ"  =  "1"  /* Restriction of Azygos Vein, Percutaneous Approach */
"05V04DZ"  =  "1"  /* Restriction of Azygos Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05V04ZZ"  =  "1"  /* Restriction of Azygos Vein, Percutaneous Endoscopic Approach */
"05V10DZ"  =  "1"  /* Restriction of Hemiazygos Vein with Intraluminal Device, Open Approach */
"05V10ZZ"  =  "1"  /* Restriction of Hemiazygos Vein, Open Approach */
"05V13DZ"  =  "1"  /* Restriction of Hemiazygos Vein with Intraluminal Device, Percutaneous Approach */
"05V13ZZ"  =  "1"  /* Restriction of Hemiazygos Vein, Percutaneous Approach */
"05V14DZ"  =  "1"  /* Restriction of Hemiazygos Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05V14ZZ"  =  "1"  /* Restriction of Hemiazygos Vein, Percutaneous Endoscopic Approach */
"05V30DZ"  =  "1"  /* Restriction of Right Innominate Vein with Intraluminal Device, Open Approach */
"05V30ZZ"  =  "1"  /* Restriction of Right Innominate Vein, Open Approach */
"05V33DZ"  =  "1"  /* Restriction of Right Innominate Vein with Intraluminal Device, Percutaneous Approach */
"05V33ZZ"  =  "1"  /* Restriction of Right Innominate Vein, Percutaneous Approach */
"05V34DZ"  =  "1"  /* Restriction of Right Innominate Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05V34ZZ"  =  "1"  /* Restriction of Right Innominate Vein, Percutaneous Endoscopic Approach */
"05V40DZ"  =  "1"  /* Restriction of Left Innominate Vein with Intraluminal Device, Open Approach */
"05V40ZZ"  =  "1"  /* Restriction of Left Innominate Vein, Open Approach */
"05V43DZ"  =  "1"  /* Restriction of Left Innominate Vein with Intraluminal Device, Percutaneous Approach */
"05V43ZZ"  =  "1"  /* Restriction of Left Innominate Vein, Percutaneous Approach */
"05V44DZ"  =  "1"  /* Restriction of Left Innominate Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05V44ZZ"  =  "1"  /* Restriction of Left Innominate Vein, Percutaneous Endoscopic Approach */
"05V50DZ"  =  "1"  /* Restriction of Right Subclavian Vein with Intraluminal Device, Open Approach */
"05V50ZZ"  =  "1"  /* Restriction of Right Subclavian Vein, Open Approach */
"05V53DZ"  =  "1"  /* Restriction of Right Subclavian Vein with Intraluminal Device, Percutaneous Approach */
"05V53ZZ"  =  "1"  /* Restriction of Right Subclavian Vein, Percutaneous Approach */
"05V54DZ"  =  "1"  /* Restriction of Right Subclavian Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05V54ZZ"  =  "1"  /* Restriction of Right Subclavian Vein, Percutaneous Endoscopic Approach */
"05V60DZ"  =  "1"  /* Restriction of Left Subclavian Vein with Intraluminal Device, Open Approach */
"05V60ZZ"  =  "1"  /* Restriction of Left Subclavian Vein, Open Approach */
"05V63DZ"  =  "1"  /* Restriction of Left Subclavian Vein with Intraluminal Device, Percutaneous Approach */
"05V63ZZ"  =  "1"  /* Restriction of Left Subclavian Vein, Percutaneous Approach */
"05V64DZ"  =  "1"  /* Restriction of Left Subclavian Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05V64ZZ"  =  "1"  /* Restriction of Left Subclavian Vein, Percutaneous Endoscopic Approach */
"05V70DZ"  =  "1"  /* Restriction of Right Axillary Vein with Intraluminal Device, Open Approach */
"05V70ZZ"  =  "1"  /* Restriction of Right Axillary Vein, Open Approach */
"05V73DZ"  =  "1"  /* Restriction of Right Axillary Vein with Intraluminal Device, Percutaneous Approach */
"05V73ZZ"  =  "1"  /* Restriction of Right Axillary Vein, Percutaneous Approach */
"05V74DZ"  =  "1"  /* Restriction of Right Axillary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05V74ZZ"  =  "1"  /* Restriction of Right Axillary Vein, Percutaneous Endoscopic Approach */
"05V80DZ"  =  "1"  /* Restriction of Left Axillary Vein with Intraluminal Device, Open Approach */
"05V80ZZ"  =  "1"  /* Restriction of Left Axillary Vein, Open Approach */
"05V83DZ"  =  "1"  /* Restriction of Left Axillary Vein with Intraluminal Device, Percutaneous Approach */
"05V83ZZ"  =  "1"  /* Restriction of Left Axillary Vein, Percutaneous Approach */
"05V84DZ"  =  "1"  /* Restriction of Left Axillary Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05V84ZZ"  =  "1"  /* Restriction of Left Axillary Vein, Percutaneous Endoscopic Approach */
"05V90DZ"  =  "1"  /* Restriction of Right Brachial Vein with Intraluminal Device, Open Approach */
"05V90ZZ"  =  "1"  /* Restriction of Right Brachial Vein, Open Approach */
"05V93DZ"  =  "1"  /* Restriction of Right Brachial Vein with Intraluminal Device, Percutaneous Approach */
"05V93ZZ"  =  "1"  /* Restriction of Right Brachial Vein, Percutaneous Approach */
"05V94DZ"  =  "1"  /* Restriction of Right Brachial Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05V94ZZ"  =  "1"  /* Restriction of Right Brachial Vein, Percutaneous Endoscopic Approach */
"05VA0DZ"  =  "1"  /* Restriction of Left Brachial Vein with Intraluminal Device, Open Approach */
"05VA0ZZ"  =  "1"  /* Restriction of Left Brachial Vein, Open Approach */
"05VA3DZ"  =  "1"  /* Restriction of Left Brachial Vein with Intraluminal Device, Percutaneous Approach */
"05VA3ZZ"  =  "1"  /* Restriction of Left Brachial Vein, Percutaneous Approach */
"05VA4DZ"  =  "1"  /* Restriction of Left Brachial Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VA4ZZ"  =  "1"  /* Restriction of Left Brachial Vein, Percutaneous Endoscopic Approach */
"05VB0DZ"  =  "1"  /* Restriction of Right Basilic Vein with Intraluminal Device, Open Approach */
"05VB0ZZ"  =  "1"  /* Restriction of Right Basilic Vein, Open Approach */
"05VB3DZ"  =  "1"  /* Restriction of Right Basilic Vein with Intraluminal Device, Percutaneous Approach */
"05VB3ZZ"  =  "1"  /* Restriction of Right Basilic Vein, Percutaneous Approach */
"05VB4DZ"  =  "1"  /* Restriction of Right Basilic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VB4ZZ"  =  "1"  /* Restriction of Right Basilic Vein, Percutaneous Endoscopic Approach */
"05VC0DZ"  =  "1"  /* Restriction of Left Basilic Vein with Intraluminal Device, Open Approach */
"05VC0ZZ"  =  "1"  /* Restriction of Left Basilic Vein, Open Approach */
"05VC3DZ"  =  "1"  /* Restriction of Left Basilic Vein with Intraluminal Device, Percutaneous Approach */
"05VC3ZZ"  =  "1"  /* Restriction of Left Basilic Vein, Percutaneous Approach */
"05VC4DZ"  =  "1"  /* Restriction of Left Basilic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VC4ZZ"  =  "1"  /* Restriction of Left Basilic Vein, Percutaneous Endoscopic Approach */
"05VD0DZ"  =  "1"  /* Restriction of Right Cephalic Vein with Intraluminal Device, Open Approach */
"05VD0ZZ"  =  "1"  /* Restriction of Right Cephalic Vein, Open Approach */
"05VD3DZ"  =  "1"  /* Restriction of Right Cephalic Vein with Intraluminal Device, Percutaneous Approach */
"05VD3ZZ"  =  "1"  /* Restriction of Right Cephalic Vein, Percutaneous Approach */
"05VD4DZ"  =  "1"  /* Restriction of Right Cephalic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VD4ZZ"  =  "1"  /* Restriction of Right Cephalic Vein, Percutaneous Endoscopic Approach */
"05VF0DZ"  =  "1"  /* Restriction of Left Cephalic Vein with Intraluminal Device, Open Approach */
"05VF0ZZ"  =  "1"  /* Restriction of Left Cephalic Vein, Open Approach */
"05VF3DZ"  =  "1"  /* Restriction of Left Cephalic Vein with Intraluminal Device, Percutaneous Approach */
"05VF3ZZ"  =  "1"  /* Restriction of Left Cephalic Vein, Percutaneous Approach */
"05VF4DZ"  =  "1"  /* Restriction of Left Cephalic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VF4ZZ"  =  "1"  /* Restriction of Left Cephalic Vein, Percutaneous Endoscopic Approach */
"05VG0DZ"  =  "1"  /* Restriction of Right Hand Vein with Intraluminal Device, Open Approach */
"05VG0ZZ"  =  "1"  /* Restriction of Right Hand Vein, Open Approach */
"05VG3DZ"  =  "1"  /* Restriction of Right Hand Vein with Intraluminal Device, Percutaneous Approach */
"05VG3ZZ"  =  "1"  /* Restriction of Right Hand Vein, Percutaneous Approach */
"05VG4DZ"  =  "1"  /* Restriction of Right Hand Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VG4ZZ"  =  "1"  /* Restriction of Right Hand Vein, Percutaneous Endoscopic Approach */
"05VH0DZ"  =  "1"  /* Restriction of Left Hand Vein with Intraluminal Device, Open Approach */
"05VH0ZZ"  =  "1"  /* Restriction of Left Hand Vein, Open Approach */
"05VH3DZ"  =  "1"  /* Restriction of Left Hand Vein with Intraluminal Device, Percutaneous Approach */
"05VH3ZZ"  =  "1"  /* Restriction of Left Hand Vein, Percutaneous Approach */
"05VH4DZ"  =  "1"  /* Restriction of Left Hand Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VH4ZZ"  =  "1"  /* Restriction of Left Hand Vein, Percutaneous Endoscopic Approach */
"05VL0DZ"  =  "1"  /* Restriction of Intracranial Vein with Intraluminal Device, Open Approach */
"05VL0ZZ"  =  "1"  /* Restriction of Intracranial Vein, Open Approach */
"05VL3DZ"  =  "1"  /* Restriction of Intracranial Vein with Intraluminal Device, Percutaneous Approach */
"05VL3ZZ"  =  "1"  /* Restriction of Intracranial Vein, Percutaneous Approach */
"05VL4DZ"  =  "1"  /* Restriction of Intracranial Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VL4ZZ"  =  "1"  /* Restriction of Intracranial Vein, Percutaneous Endoscopic Approach */
"05VM0DZ"  =  "1"  /* Restriction of Right Internal Jugular Vein with Intraluminal Device, Open Approach */
"05VM0ZZ"  =  "1"  /* Restriction of Right Internal Jugular Vein, Open Approach */
"05VM3DZ"  =  "1"  /* Restriction of Right Internal Jugular Vein with Intraluminal Device, Percutaneous Approach */
"05VM3ZZ"  =  "1"  /* Restriction of Right Internal Jugular Vein, Percutaneous Approach */
"05VM4DZ"  =  "1"  /* Restriction of Right Internal Jugular Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VM4ZZ"  =  "1"  /* Restriction of Right Internal Jugular Vein, Percutaneous Endoscopic Approach */
"05VN0DZ"  =  "1"  /* Restriction of Left Internal Jugular Vein with Intraluminal Device, Open Approach */
"05VN0ZZ"  =  "1"  /* Restriction of Left Internal Jugular Vein, Open Approach */
"05VN3DZ"  =  "1"  /* Restriction of Left Internal Jugular Vein with Intraluminal Device, Percutaneous Approach */
"05VN3ZZ"  =  "1"  /* Restriction of Left Internal Jugular Vein, Percutaneous Approach */
"05VN4DZ"  =  "1"  /* Restriction of Left Internal Jugular Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VN4ZZ"  =  "1"  /* Restriction of Left Internal Jugular Vein, Percutaneous Endoscopic Approach */
"05VP0DZ"  =  "1"  /* Restriction of Right External Jugular Vein with Intraluminal Device, Open Approach */
"05VP0ZZ"  =  "1"  /* Restriction of Right External Jugular Vein, Open Approach */
"05VP3DZ"  =  "1"  /* Restriction of Right External Jugular Vein with Intraluminal Device, Percutaneous Approach */
"05VP3ZZ"  =  "1"  /* Restriction of Right External Jugular Vein, Percutaneous Approach */
"05VP4DZ"  =  "1"  /* Restriction of Right External Jugular Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VP4ZZ"  =  "1"  /* Restriction of Right External Jugular Vein, Percutaneous Endoscopic Approach */
"05VQ0DZ"  =  "1"  /* Restriction of Left External Jugular Vein with Intraluminal Device, Open Approach */
"05VQ0ZZ"  =  "1"  /* Restriction of Left External Jugular Vein, Open Approach */
"05VQ3DZ"  =  "1"  /* Restriction of Left External Jugular Vein with Intraluminal Device, Percutaneous Approach */
"05VQ3ZZ"  =  "1"  /* Restriction of Left External Jugular Vein, Percutaneous Approach */
"05VQ4DZ"  =  "1"  /* Restriction of Left External Jugular Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VQ4ZZ"  =  "1"  /* Restriction of Left External Jugular Vein, Percutaneous Endoscopic Approach */
"05VR0DZ"  =  "1"  /* Restriction of Right Vertebral Vein with Intraluminal Device, Open Approach */
"05VR0ZZ"  =  "1"  /* Restriction of Right Vertebral Vein, Open Approach */
"05VR3DZ"  =  "1"  /* Restriction of Right Vertebral Vein with Intraluminal Device, Percutaneous Approach */
"05VR3ZZ"  =  "1"  /* Restriction of Right Vertebral Vein, Percutaneous Approach */
"05VR4DZ"  =  "1"  /* Restriction of Right Vertebral Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VR4ZZ"  =  "1"  /* Restriction of Right Vertebral Vein, Percutaneous Endoscopic Approach */
"05VS0DZ"  =  "1"  /* Restriction of Left Vertebral Vein with Intraluminal Device, Open Approach */
"05VS0ZZ"  =  "1"  /* Restriction of Left Vertebral Vein, Open Approach */
"05VS3DZ"  =  "1"  /* Restriction of Left Vertebral Vein with Intraluminal Device, Percutaneous Approach */
"05VS3ZZ"  =  "1"  /* Restriction of Left Vertebral Vein, Percutaneous Approach */
"05VS4DZ"  =  "1"  /* Restriction of Left Vertebral Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VS4ZZ"  =  "1"  /* Restriction of Left Vertebral Vein, Percutaneous Endoscopic Approach */
"05VT0DZ"  =  "1"  /* Restriction of Right Face Vein with Intraluminal Device, Open Approach */
"05VT0ZZ"  =  "1"  /* Restriction of Right Face Vein, Open Approach */
"05VT3DZ"  =  "1"  /* Restriction of Right Face Vein with Intraluminal Device, Percutaneous Approach */
"05VT3ZZ"  =  "1"  /* Restriction of Right Face Vein, Percutaneous Approach */
"05VT4DZ"  =  "1"  /* Restriction of Right Face Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VT4ZZ"  =  "1"  /* Restriction of Right Face Vein, Percutaneous Endoscopic Approach */
"05VV0DZ"  =  "1"  /* Restriction of Left Face Vein with Intraluminal Device, Open Approach */
"05VV0ZZ"  =  "1"  /* Restriction of Left Face Vein, Open Approach */
"05VV3DZ"  =  "1"  /* Restriction of Left Face Vein with Intraluminal Device, Percutaneous Approach */
"05VV3ZZ"  =  "1"  /* Restriction of Left Face Vein, Percutaneous Approach */
"05VV4DZ"  =  "1"  /* Restriction of Left Face Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VV4ZZ"  =  "1"  /* Restriction of Left Face Vein, Percutaneous Endoscopic Approach */
"05VY0DZ"  =  "1"  /* Restriction of Upper Vein with Intraluminal Device, Open Approach */
"05VY0ZZ"  =  "1"  /* Restriction of Upper Vein, Open Approach */
"05VY3DZ"  =  "1"  /* Restriction of Upper Vein with Intraluminal Device, Percutaneous Approach */
"05VY3ZZ"  =  "1"  /* Restriction of Upper Vein, Percutaneous Approach */
"05VY4DZ"  =  "1"  /* Restriction of Upper Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"05VY4ZZ"  =  "1"  /* Restriction of Upper Vein, Percutaneous Endoscopic Approach */
"05WY00Z"  =  "1"  /* Revision of Drainage Device in Upper Vein, Open Approach */
"05WY02Z"  =  "1"  /* Revision of Monitoring Device in Upper Vein, Open Approach */
"05WY03Z"  =  "1"  /* Revision of Infusion Device in Upper Vein, Open Approach */
"05WY07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Vein, Open Approach */
"05WY0CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Vein, Open Approach */
"05WY0DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Vein, Open Approach */
"05WY0JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Vein, Open Approach */
"05WY0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Vein, Open Approach */
"05WY30Z"  =  "1"  /* Revision of Drainage Device in Upper Vein, Percutaneous Approach */
"05WY32Z"  =  "1"  /* Revision of Monitoring Device in Upper Vein, Percutaneous Approach */
"05WY33Z"  =  "1"  /* Revision of Infusion Device in Upper Vein, Percutaneous Approach */
"05WY37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Vein, Percutaneous Approach */
"05WY3CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Vein, Percutaneous Approach */
"05WY3DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Vein, Percutaneous Approach */
"05WY3JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Vein, Percutaneous Approach */
"05WY3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Vein, Percutaneous Approach */
"05WY40Z"  =  "1"  /* Revision of Drainage Device in Upper Vein, Percutaneous Endoscopic Approach */
"05WY42Z"  =  "1"  /* Revision of Monitoring Device in Upper Vein, Percutaneous Endoscopic Approach */
"05WY43Z"  =  "1"  /* Revision of Infusion Device in Upper Vein, Percutaneous Endoscopic Approach */
"05WY47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Vein, Percutaneous Endoscopic Approach */
"05WY4CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Vein, Percutaneous Endoscopic Approach */
"05WY4DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Vein, Percutaneous Endoscopic Approach */
"05WY4JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Vein, Percutaneous Endoscopic Approach */
"05WY4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Vein, Percutaneous Endoscopic Approach */
"069300Z"  =  "1"  /* Drainage of Esophageal Vein with Drainage Device, Open Approach */
"06930ZZ"  =  "1"  /* Drainage of Esophageal Vein, Open Approach */
"069330Z"  =  "1"  /* Drainage of Esophageal Vein with Drainage Device, Percutaneous Approach */
"06933ZZ"  =  "1"  /* Drainage of Esophageal Vein, Percutaneous Approach */
"069340Z"  =  "1"  /* Drainage of Esophageal Vein with Drainage Device, Percutaneous Endoscopic Approach */
"06934ZZ"  =  "1"  /* Drainage of Esophageal Vein, Percutaneous Endoscopic Approach */
"069Y00Z"  =  "1"  /* Drainage of Lower Vein with Drainage Device, Open Approach */
"069Y0ZZ"  =  "1"  /* Drainage of Lower Vein, Open Approach */
"069Y30Z"  =  "1"  /* Drainage of Lower Vein with Drainage Device, Percutaneous Approach */
"069Y3ZZ"  =  "1"  /* Drainage of Lower Vein, Percutaneous Approach */
"06C00ZZ"  =  "1"  /* Extirpation of Matter from Inferior Vena Cava, Open Approach */
"06C03ZZ"  =  "1"  /* Extirpation of Matter from Inferior Vena Cava, Percutaneous Approach */
"06C04ZZ"  =  "1"  /* Extirpation of Matter from Inferior Vena Cava, Percutaneous Endoscopic Approach */
"06C10ZZ"  =  "1"  /* Extirpation of Matter from Splenic Vein, Open Approach */
"06C13ZZ"  =  "1"  /* Extirpation of Matter from Splenic Vein, Percutaneous Approach */
"06C14ZZ"  =  "1"  /* Extirpation of Matter from Splenic Vein, Percutaneous Endoscopic Approach */
"06C20ZZ"  =  "1"  /* Extirpation of Matter from Gastric Vein, Open Approach */
"06C23ZZ"  =  "1"  /* Extirpation of Matter from Gastric Vein, Percutaneous Approach */
"06C24ZZ"  =  "1"  /* Extirpation of Matter from Gastric Vein, Percutaneous Endoscopic Approach */
"06C30ZZ"  =  "1"  /* Extirpation of Matter from Esophageal Vein, Open Approach */
"06C33ZZ"  =  "1"  /* Extirpation of Matter from Esophageal Vein, Percutaneous Approach */
"06C34ZZ"  =  "1"  /* Extirpation of Matter from Esophageal Vein, Percutaneous Endoscopic Approach */
"06C40ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Vein, Open Approach */
"06C43ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Vein, Percutaneous Approach */
"06C44ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Vein, Percutaneous Endoscopic Approach */
"06C50ZZ"  =  "1"  /* Extirpation of Matter from Superior Mesenteric Vein, Open Approach */
"06C53ZZ"  =  "1"  /* Extirpation of Matter from Superior Mesenteric Vein, Percutaneous Approach */
"06C54ZZ"  =  "1"  /* Extirpation of Matter from Superior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06C60ZZ"  =  "1"  /* Extirpation of Matter from Inferior Mesenteric Vein, Open Approach */
"06C63ZZ"  =  "1"  /* Extirpation of Matter from Inferior Mesenteric Vein, Percutaneous Approach */
"06C64ZZ"  =  "1"  /* Extirpation of Matter from Inferior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06C70ZZ"  =  "1"  /* Extirpation of Matter from Colic Vein, Open Approach */
"06C73ZZ"  =  "1"  /* Extirpation of Matter from Colic Vein, Percutaneous Approach */
"06C74ZZ"  =  "1"  /* Extirpation of Matter from Colic Vein, Percutaneous Endoscopic Approach */
"06C80ZZ"  =  "1"  /* Extirpation of Matter from Portal Vein, Open Approach */
"06C83ZZ"  =  "1"  /* Extirpation of Matter from Portal Vein, Percutaneous Approach */
"06C84ZZ"  =  "1"  /* Extirpation of Matter from Portal Vein, Percutaneous Endoscopic Approach */
"06C90ZZ"  =  "1"  /* Extirpation of Matter from Right Renal Vein, Open Approach */
"06C93ZZ"  =  "1"  /* Extirpation of Matter from Right Renal Vein, Percutaneous Approach */
"06C94ZZ"  =  "1"  /* Extirpation of Matter from Right Renal Vein, Percutaneous Endoscopic Approach */
"06CB0ZZ"  =  "1"  /* Extirpation of Matter from Left Renal Vein, Open Approach */
"06CB3ZZ"  =  "1"  /* Extirpation of Matter from Left Renal Vein, Percutaneous Approach */
"06CB4ZZ"  =  "1"  /* Extirpation of Matter from Left Renal Vein, Percutaneous Endoscopic Approach */
"06CC0ZZ"  =  "1"  /* Extirpation of Matter from Right Common Iliac Vein, Open Approach */
"06CC3ZZ"  =  "1"  /* Extirpation of Matter from Right Common Iliac Vein, Percutaneous Approach */
"06CC4ZZ"  =  "1"  /* Extirpation of Matter from Right Common Iliac Vein, Percutaneous Endoscopic Approach */
"06CD0ZZ"  =  "1"  /* Extirpation of Matter from Left Common Iliac Vein, Open Approach */
"06CD3ZZ"  =  "1"  /* Extirpation of Matter from Left Common Iliac Vein, Percutaneous Approach */
"06CD4ZZ"  =  "1"  /* Extirpation of Matter from Left Common Iliac Vein, Percutaneous Endoscopic Approach */
"06CF0ZZ"  =  "1"  /* Extirpation of Matter from Right External Iliac Vein, Open Approach */
"06CF3ZZ"  =  "1"  /* Extirpation of Matter from Right External Iliac Vein, Percutaneous Approach */
"06CF4ZZ"  =  "1"  /* Extirpation of Matter from Right External Iliac Vein, Percutaneous Endoscopic Approach */
"06CG0ZZ"  =  "1"  /* Extirpation of Matter from Left External Iliac Vein, Open Approach */
"06CG3ZZ"  =  "1"  /* Extirpation of Matter from Left External Iliac Vein, Percutaneous Approach */
"06CG4ZZ"  =  "1"  /* Extirpation of Matter from Left External Iliac Vein, Percutaneous Endoscopic Approach */
"06CH0ZZ"  =  "1"  /* Extirpation of Matter from Right Hypogastric Vein, Open Approach */
"06CH3ZZ"  =  "1"  /* Extirpation of Matter from Right Hypogastric Vein, Percutaneous Approach */
"06CH4ZZ"  =  "1"  /* Extirpation of Matter from Right Hypogastric Vein, Percutaneous Endoscopic Approach */
"06CJ0ZZ"  =  "1"  /* Extirpation of Matter from Left Hypogastric Vein, Open Approach */
"06CJ3ZZ"  =  "1"  /* Extirpation of Matter from Left Hypogastric Vein, Percutaneous Approach */
"06CJ4ZZ"  =  "1"  /* Extirpation of Matter from Left Hypogastric Vein, Percutaneous Endoscopic Approach */
"06CM0ZZ"  =  "1"  /* Extirpation of Matter from Right Femoral Vein, Open Approach */
"06CM3ZZ"  =  "1"  /* Extirpation of Matter from Right Femoral Vein, Percutaneous Approach */
"06CM4ZZ"  =  "1"  /* Extirpation of Matter from Right Femoral Vein, Percutaneous Endoscopic Approach */
"06CN0ZZ"  =  "1"  /* Extirpation of Matter from Left Femoral Vein, Open Approach */
"06CN3ZZ"  =  "1"  /* Extirpation of Matter from Left Femoral Vein, Percutaneous Approach */
"06CN4ZZ"  =  "1"  /* Extirpation of Matter from Left Femoral Vein, Percutaneous Endoscopic Approach */
"06CP0ZZ"  =  "1"  /* Extirpation of Matter from Right Greater Saphenous Vein, Open Approach */
"06CP3ZZ"  =  "1"  /* Extirpation of Matter from Right Greater Saphenous Vein, Percutaneous Approach */
"06CP4ZZ"  =  "1"  /* Extirpation of Matter from Right Greater Saphenous Vein, Percutaneous Endoscopic Approach */
"06CQ0ZZ"  =  "1"  /* Extirpation of Matter from Left Greater Saphenous Vein, Open Approach */
"06CQ3ZZ"  =  "1"  /* Extirpation of Matter from Left Greater Saphenous Vein, Percutaneous Approach */
"06CQ4ZZ"  =  "1"  /* Extirpation of Matter from Left Greater Saphenous Vein, Percutaneous Endoscopic Approach */
"06CR0ZZ"  =  "1"  /* Extirpation of Matter from Right Lesser Saphenous Vein, Open Approach */
"06CR3ZZ"  =  "1"  /* Extirpation of Matter from Right Lesser Saphenous Vein, Percutaneous Approach */
"06CR4ZZ"  =  "1"  /* Extirpation of Matter from Right Lesser Saphenous Vein, Percutaneous Endoscopic Approach */
"06CS0ZZ"  =  "1"  /* Extirpation of Matter from Left Lesser Saphenous Vein, Open Approach */
"06CS3ZZ"  =  "1"  /* Extirpation of Matter from Left Lesser Saphenous Vein, Percutaneous Approach */
"06CS4ZZ"  =  "1"  /* Extirpation of Matter from Left Lesser Saphenous Vein, Percutaneous Endoscopic Approach */
"06CT0ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Vein, Open Approach */
"06CT3ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Vein, Percutaneous Approach */
"06CT4ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Vein, Percutaneous Endoscopic Approach */
"06CV0ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Vein, Open Approach */
"06CV3ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Vein, Percutaneous Approach */
"06CV4ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Vein, Percutaneous Endoscopic Approach */
"06CY0ZZ"  =  "1"  /* Extirpation of Matter from Lower Vein, Open Approach */
"06CY3ZZ"  =  "1"  /* Extirpation of Matter from Lower Vein, Percutaneous Approach */
"06CY4ZZ"  =  "1"  /* Extirpation of Matter from Lower Vein, Percutaneous Endoscopic Approach */
"06DM0ZZ"  =  "1"  /* Extraction of Right Femoral Vein, Open Approach */
"06DM3ZZ"  =  "1"  /* Extraction of Right Femoral Vein, Percutaneous Approach */
"06DM4ZZ"  =  "1"  /* Extraction of Right Femoral Vein, Percutaneous Endoscopic Approach */
"06DN0ZZ"  =  "1"  /* Extraction of Left Femoral Vein, Open Approach */
"06DN3ZZ"  =  "1"  /* Extraction of Left Femoral Vein, Percutaneous Approach */
"06DN4ZZ"  =  "1"  /* Extraction of Left Femoral Vein, Percutaneous Endoscopic Approach */
"06DP0ZZ"  =  "1"  /* Extraction of Right Greater Saphenous Vein, Open Approach */
"06DP3ZZ"  =  "1"  /* Extraction of Right Greater Saphenous Vein, Percutaneous Approach */
"06DP4ZZ"  =  "1"  /* Extraction of Right Greater Saphenous Vein, Percutaneous Endoscopic Approach */
"06DQ0ZZ"  =  "1"  /* Extraction of Left Greater Saphenous Vein, Open Approach */
"06DQ3ZZ"  =  "1"  /* Extraction of Left Greater Saphenous Vein, Percutaneous Approach */
"06DQ4ZZ"  =  "1"  /* Extraction of Left Greater Saphenous Vein, Percutaneous Endoscopic Approach */
"06DR0ZZ"  =  "1"  /* Extraction of Right Lesser Saphenous Vein, Open Approach */
"06DR3ZZ"  =  "1"  /* Extraction of Right Lesser Saphenous Vein, Percutaneous Approach */
"06DR4ZZ"  =  "1"  /* Extraction of Right Lesser Saphenous Vein, Percutaneous Endoscopic Approach */
"06DS0ZZ"  =  "1"  /* Extraction of Left Lesser Saphenous Vein, Open Approach */
"06DS3ZZ"  =  "1"  /* Extraction of Left Lesser Saphenous Vein, Percutaneous Approach */
"06DS4ZZ"  =  "1"  /* Extraction of Left Lesser Saphenous Vein, Percutaneous Endoscopic Approach */
"06DT0ZZ"  =  "1"  /* Extraction of Right Foot Vein, Open Approach */
"06DT3ZZ"  =  "1"  /* Extraction of Right Foot Vein, Percutaneous Approach */
"06DT4ZZ"  =  "1"  /* Extraction of Right Foot Vein, Percutaneous Endoscopic Approach */
"06DV0ZZ"  =  "1"  /* Extraction of Left Foot Vein, Open Approach */
"06DV3ZZ"  =  "1"  /* Extraction of Left Foot Vein, Percutaneous Approach */
"06DV4ZZ"  =  "1"  /* Extraction of Left Foot Vein, Percutaneous Endoscopic Approach */
"06DY0ZZ"  =  "1"  /* Extraction of Lower Vein, Open Approach */
"06DY3ZZ"  =  "1"  /* Extraction of Lower Vein, Percutaneous Approach */
"06DY4ZZ"  =  "1"  /* Extraction of Lower Vein, Percutaneous Endoscopic Approach */
"06H00DZ"  =  "1"  /* Insertion of Intraluminal Device into Inferior Vena Cava, Open Approach */
"06H03DZ"  =  "1"  /* Insertion of Intraluminal Device into Inferior Vena Cava, Percutaneous Approach */
"06H04DZ"  =  "1"  /* Insertion of Intraluminal Device into Inferior Vena Cava, Percutaneous Endoscopic Approach */
"06HY02Z"  =  "1"  /* Insertion of Monitoring Device into Lower Vein, Open Approach */
"06HY32Z"  =  "1"  /* Insertion of Monitoring Device into Lower Vein, Percutaneous Approach */
"06HY42Z"  =  "1"  /* Insertion of Monitoring Device into Lower Vein, Percutaneous Endoscopic Approach */
"06L00CZ"  =  "1"  /* Occlusion of Inferior Vena Cava with Extraluminal Device, Open Approach */
"06L00DZ"  =  "1"  /* Occlusion of Inferior Vena Cava with Intraluminal Device, Open Approach */
"06L00ZZ"  =  "1"  /* Occlusion of Inferior Vena Cava, Open Approach */
"06L03CZ"  =  "1"  /* Occlusion of Inferior Vena Cava with Extraluminal Device, Percutaneous Approach */
"06L03DZ"  =  "1"  /* Occlusion of Inferior Vena Cava with Intraluminal Device, Percutaneous Approach */
"06L03ZZ"  =  "1"  /* Occlusion of Inferior Vena Cava, Percutaneous Approach */
"06L04CZ"  =  "1"  /* Occlusion of Inferior Vena Cava with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L04DZ"  =  "1"  /* Occlusion of Inferior Vena Cava with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L04ZZ"  =  "1"  /* Occlusion of Inferior Vena Cava, Percutaneous Endoscopic Approach */
"06L10CZ"  =  "1"  /* Occlusion of Splenic Vein with Extraluminal Device, Open Approach */
"06L10DZ"  =  "1"  /* Occlusion of Splenic Vein with Intraluminal Device, Open Approach */
"06L10ZZ"  =  "1"  /* Occlusion of Splenic Vein, Open Approach */
"06L13CZ"  =  "1"  /* Occlusion of Splenic Vein with Extraluminal Device, Percutaneous Approach */
"06L13DZ"  =  "1"  /* Occlusion of Splenic Vein with Intraluminal Device, Percutaneous Approach */
"06L13ZZ"  =  "1"  /* Occlusion of Splenic Vein, Percutaneous Approach */
"06L14CZ"  =  "1"  /* Occlusion of Splenic Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L14DZ"  =  "1"  /* Occlusion of Splenic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L14ZZ"  =  "1"  /* Occlusion of Splenic Vein, Percutaneous Endoscopic Approach */
"06L20CZ"  =  "1"  /* Occlusion of Gastric Vein with Extraluminal Device, Open Approach */
"06L20DZ"  =  "1"  /* Occlusion of Gastric Vein with Intraluminal Device, Open Approach */
"06L20ZZ"  =  "1"  /* Occlusion of Gastric Vein, Open Approach */
"06L23CZ"  =  "1"  /* Occlusion of Gastric Vein with Extraluminal Device, Percutaneous Approach */
"06L23DZ"  =  "1"  /* Occlusion of Gastric Vein with Intraluminal Device, Percutaneous Approach */
"06L23ZZ"  =  "1"  /* Occlusion of Gastric Vein, Percutaneous Approach */
"06L24CZ"  =  "1"  /* Occlusion of Gastric Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L24DZ"  =  "1"  /* Occlusion of Gastric Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L24ZZ"  =  "1"  /* Occlusion of Gastric Vein, Percutaneous Endoscopic Approach */
"06L30CZ"  =  "1"  /* Occlusion of Esophageal Vein with Extraluminal Device, Open Approach */
"06L30DZ"  =  "1"  /* Occlusion of Esophageal Vein with Intraluminal Device, Open Approach */
"06L30ZZ"  =  "1"  /* Occlusion of Esophageal Vein, Open Approach */
"06L33CZ"  =  "1"  /* Occlusion of Esophageal Vein with Extraluminal Device, Percutaneous Approach */
"06L33DZ"  =  "1"  /* Occlusion of Esophageal Vein with Intraluminal Device, Percutaneous Approach */
"06L33ZZ"  =  "1"  /* Occlusion of Esophageal Vein, Percutaneous Approach */
"06L34CZ"  =  "1"  /* Occlusion of Esophageal Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L34DZ"  =  "1"  /* Occlusion of Esophageal Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L34ZZ"  =  "1"  /* Occlusion of Esophageal Vein, Percutaneous Endoscopic Approach */
"06L40CZ"  =  "1"  /* Occlusion of Hepatic Vein with Extraluminal Device, Open Approach */
"06L40DZ"  =  "1"  /* Occlusion of Hepatic Vein with Intraluminal Device, Open Approach */
"06L40ZZ"  =  "1"  /* Occlusion of Hepatic Vein, Open Approach */
"06L43CZ"  =  "1"  /* Occlusion of Hepatic Vein with Extraluminal Device, Percutaneous Approach */
"06L43DZ"  =  "1"  /* Occlusion of Hepatic Vein with Intraluminal Device, Percutaneous Approach */
"06L43ZZ"  =  "1"  /* Occlusion of Hepatic Vein, Percutaneous Approach */
"06L44CZ"  =  "1"  /* Occlusion of Hepatic Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L44DZ"  =  "1"  /* Occlusion of Hepatic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L44ZZ"  =  "1"  /* Occlusion of Hepatic Vein, Percutaneous Endoscopic Approach */
"06L50CZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein with Extraluminal Device, Open Approach */
"06L50DZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein with Intraluminal Device, Open Approach */
"06L50ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein, Open Approach */
"06L53CZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein with Extraluminal Device, Percutaneous Approach */
"06L53DZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein with Intraluminal Device, Percutaneous Approach */
"06L53ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein, Percutaneous Approach */
"06L54CZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L54DZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L54ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06L60CZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein with Extraluminal Device, Open Approach */
"06L60DZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein with Intraluminal Device, Open Approach */
"06L60ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein, Open Approach */
"06L63CZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein with Extraluminal Device, Percutaneous Approach */
"06L63DZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein with Intraluminal Device, Percutaneous Approach */
"06L63ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein, Percutaneous Approach */
"06L64CZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L64DZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L64ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06L70CZ"  =  "1"  /* Occlusion of Colic Vein with Extraluminal Device, Open Approach */
"06L70DZ"  =  "1"  /* Occlusion of Colic Vein with Intraluminal Device, Open Approach */
"06L70ZZ"  =  "1"  /* Occlusion of Colic Vein, Open Approach */
"06L73CZ"  =  "1"  /* Occlusion of Colic Vein with Extraluminal Device, Percutaneous Approach */
"06L73DZ"  =  "1"  /* Occlusion of Colic Vein with Intraluminal Device, Percutaneous Approach */
"06L73ZZ"  =  "1"  /* Occlusion of Colic Vein, Percutaneous Approach */
"06L74CZ"  =  "1"  /* Occlusion of Colic Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L74DZ"  =  "1"  /* Occlusion of Colic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L74ZZ"  =  "1"  /* Occlusion of Colic Vein, Percutaneous Endoscopic Approach */
"06L80CZ"  =  "1"  /* Occlusion of Portal Vein with Extraluminal Device, Open Approach */
"06L80DZ"  =  "1"  /* Occlusion of Portal Vein with Intraluminal Device, Open Approach */
"06L80ZZ"  =  "1"  /* Occlusion of Portal Vein, Open Approach */
"06L83CZ"  =  "1"  /* Occlusion of Portal Vein with Extraluminal Device, Percutaneous Approach */
"06L83DZ"  =  "1"  /* Occlusion of Portal Vein with Intraluminal Device, Percutaneous Approach */
"06L83ZZ"  =  "1"  /* Occlusion of Portal Vein, Percutaneous Approach */
"06L84CZ"  =  "1"  /* Occlusion of Portal Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L84DZ"  =  "1"  /* Occlusion of Portal Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L84ZZ"  =  "1"  /* Occlusion of Portal Vein, Percutaneous Endoscopic Approach */
"06L90CZ"  =  "1"  /* Occlusion of Right Renal Vein with Extraluminal Device, Open Approach */
"06L90DZ"  =  "1"  /* Occlusion of Right Renal Vein with Intraluminal Device, Open Approach */
"06L90ZZ"  =  "1"  /* Occlusion of Right Renal Vein, Open Approach */
"06L93CZ"  =  "1"  /* Occlusion of Right Renal Vein with Extraluminal Device, Percutaneous Approach */
"06L93DZ"  =  "1"  /* Occlusion of Right Renal Vein with Intraluminal Device, Percutaneous Approach */
"06L93ZZ"  =  "1"  /* Occlusion of Right Renal Vein, Percutaneous Approach */
"06L94CZ"  =  "1"  /* Occlusion of Right Renal Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L94DZ"  =  "1"  /* Occlusion of Right Renal Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L94ZZ"  =  "1"  /* Occlusion of Right Renal Vein, Percutaneous Endoscopic Approach */
"06LB0CZ"  =  "1"  /* Occlusion of Left Renal Vein with Extraluminal Device, Open Approach */
"06LB0DZ"  =  "1"  /* Occlusion of Left Renal Vein with Intraluminal Device, Open Approach */
"06LB0ZZ"  =  "1"  /* Occlusion of Left Renal Vein, Open Approach */
"06LB3CZ"  =  "1"  /* Occlusion of Left Renal Vein with Extraluminal Device, Percutaneous Approach */
"06LB3DZ"  =  "1"  /* Occlusion of Left Renal Vein with Intraluminal Device, Percutaneous Approach */
"06LB3ZZ"  =  "1"  /* Occlusion of Left Renal Vein, Percutaneous Approach */
"06LB4CZ"  =  "1"  /* Occlusion of Left Renal Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LB4DZ"  =  "1"  /* Occlusion of Left Renal Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LB4ZZ"  =  "1"  /* Occlusion of Left Renal Vein, Percutaneous Endoscopic Approach */
"06LC0CZ"  =  "1"  /* Occlusion of Right Common Iliac Vein with Extraluminal Device, Open Approach */
"06LC0DZ"  =  "1"  /* Occlusion of Right Common Iliac Vein with Intraluminal Device, Open Approach */
"06LC0ZZ"  =  "1"  /* Occlusion of Right Common Iliac Vein, Open Approach */
"06LC3CZ"  =  "1"  /* Occlusion of Right Common Iliac Vein with Extraluminal Device, Percutaneous Approach */
"06LC3DZ"  =  "1"  /* Occlusion of Right Common Iliac Vein with Intraluminal Device, Percutaneous Approach */
"06LC3ZZ"  =  "1"  /* Occlusion of Right Common Iliac Vein, Percutaneous Approach */
"06LC4CZ"  =  "1"  /* Occlusion of Right Common Iliac Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LC4DZ"  =  "1"  /* Occlusion of Right Common Iliac Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LC4ZZ"  =  "1"  /* Occlusion of Right Common Iliac Vein, Percutaneous Endoscopic Approach */
"06LD0CZ"  =  "1"  /* Occlusion of Left Common Iliac Vein with Extraluminal Device, Open Approach */
"06LD0DZ"  =  "1"  /* Occlusion of Left Common Iliac Vein with Intraluminal Device, Open Approach */
"06LD0ZZ"  =  "1"  /* Occlusion of Left Common Iliac Vein, Open Approach */
"06LD3CZ"  =  "1"  /* Occlusion of Left Common Iliac Vein with Extraluminal Device, Percutaneous Approach */
"06LD3DZ"  =  "1"  /* Occlusion of Left Common Iliac Vein with Intraluminal Device, Percutaneous Approach */
"06LD3ZZ"  =  "1"  /* Occlusion of Left Common Iliac Vein, Percutaneous Approach */
"06LD4CZ"  =  "1"  /* Occlusion of Left Common Iliac Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LD4DZ"  =  "1"  /* Occlusion of Left Common Iliac Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LD4ZZ"  =  "1"  /* Occlusion of Left Common Iliac Vein, Percutaneous Endoscopic Approach */
"06LF0CZ"  =  "1"  /* Occlusion of Right External Iliac Vein with Extraluminal Device, Open Approach */
"06LF0DZ"  =  "1"  /* Occlusion of Right External Iliac Vein with Intraluminal Device, Open Approach */
"06LF0ZZ"  =  "1"  /* Occlusion of Right External Iliac Vein, Open Approach */
"06LF3CZ"  =  "1"  /* Occlusion of Right External Iliac Vein with Extraluminal Device, Percutaneous Approach */
"06LF3DZ"  =  "1"  /* Occlusion of Right External Iliac Vein with Intraluminal Device, Percutaneous Approach */
"06LF3ZZ"  =  "1"  /* Occlusion of Right External Iliac Vein, Percutaneous Approach */
"06LF4CZ"  =  "1"  /* Occlusion of Right External Iliac Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LF4DZ"  =  "1"  /* Occlusion of Right External Iliac Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LF4ZZ"  =  "1"  /* Occlusion of Right External Iliac Vein, Percutaneous Endoscopic Approach */
"06LG0CZ"  =  "1"  /* Occlusion of Left External Iliac Vein with Extraluminal Device, Open Approach */
"06LG0DZ"  =  "1"  /* Occlusion of Left External Iliac Vein with Intraluminal Device, Open Approach */
"06LG0ZZ"  =  "1"  /* Occlusion of Left External Iliac Vein, Open Approach */
"06LG3CZ"  =  "1"  /* Occlusion of Left External Iliac Vein with Extraluminal Device, Percutaneous Approach */
"06LG3DZ"  =  "1"  /* Occlusion of Left External Iliac Vein with Intraluminal Device, Percutaneous Approach */
"06LG3ZZ"  =  "1"  /* Occlusion of Left External Iliac Vein, Percutaneous Approach */
"06LG4CZ"  =  "1"  /* Occlusion of Left External Iliac Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LG4DZ"  =  "1"  /* Occlusion of Left External Iliac Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LG4ZZ"  =  "1"  /* Occlusion of Left External Iliac Vein, Percutaneous Endoscopic Approach */
"06LH0CZ"  =  "1"  /* Occlusion of Right Hypogastric Vein with Extraluminal Device, Open Approach */
"06LH0DZ"  =  "1"  /* Occlusion of Right Hypogastric Vein with Intraluminal Device, Open Approach */
"06LH0ZZ"  =  "1"  /* Occlusion of Right Hypogastric Vein, Open Approach */
"06LH3CZ"  =  "1"  /* Occlusion of Right Hypogastric Vein with Extraluminal Device, Percutaneous Approach */
"06LH3DZ"  =  "1"  /* Occlusion of Right Hypogastric Vein with Intraluminal Device, Percutaneous Approach */
"06LH3ZZ"  =  "1"  /* Occlusion of Right Hypogastric Vein, Percutaneous Approach */
"06LH4CZ"  =  "1"  /* Occlusion of Right Hypogastric Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LH4DZ"  =  "1"  /* Occlusion of Right Hypogastric Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LH4ZZ"  =  "1"  /* Occlusion of Right Hypogastric Vein, Percutaneous Endoscopic Approach */
"06LJ0CZ"  =  "1"  /* Occlusion of Left Hypogastric Vein with Extraluminal Device, Open Approach */
"06LJ0DZ"  =  "1"  /* Occlusion of Left Hypogastric Vein with Intraluminal Device, Open Approach */
"06LJ0ZZ"  =  "1"  /* Occlusion of Left Hypogastric Vein, Open Approach */
"06LJ3CZ"  =  "1"  /* Occlusion of Left Hypogastric Vein with Extraluminal Device, Percutaneous Approach */
"06LJ3DZ"  =  "1"  /* Occlusion of Left Hypogastric Vein with Intraluminal Device, Percutaneous Approach */
"06LJ3ZZ"  =  "1"  /* Occlusion of Left Hypogastric Vein, Percutaneous Approach */
"06LJ4CZ"  =  "1"  /* Occlusion of Left Hypogastric Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LJ4DZ"  =  "1"  /* Occlusion of Left Hypogastric Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LJ4ZZ"  =  "1"  /* Occlusion of Left Hypogastric Vein, Percutaneous Endoscopic Approach */
"06LM0CZ"  =  "1"  /* Occlusion of Right Femoral Vein with Extraluminal Device, Open Approach */
"06LM0DZ"  =  "1"  /* Occlusion of Right Femoral Vein with Intraluminal Device, Open Approach */
"06LM0ZZ"  =  "1"  /* Occlusion of Right Femoral Vein, Open Approach */
"06LM3CZ"  =  "1"  /* Occlusion of Right Femoral Vein with Extraluminal Device, Percutaneous Approach */
"06LM3DZ"  =  "1"  /* Occlusion of Right Femoral Vein with Intraluminal Device, Percutaneous Approach */
"06LM3ZZ"  =  "1"  /* Occlusion of Right Femoral Vein, Percutaneous Approach */
"06LM4CZ"  =  "1"  /* Occlusion of Right Femoral Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LM4DZ"  =  "1"  /* Occlusion of Right Femoral Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LM4ZZ"  =  "1"  /* Occlusion of Right Femoral Vein, Percutaneous Endoscopic Approach */
"06LN0CZ"  =  "1"  /* Occlusion of Left Femoral Vein with Extraluminal Device, Open Approach */
"06LN0DZ"  =  "1"  /* Occlusion of Left Femoral Vein with Intraluminal Device, Open Approach */
"06LN0ZZ"  =  "1"  /* Occlusion of Left Femoral Vein, Open Approach */
"06LN3CZ"  =  "1"  /* Occlusion of Left Femoral Vein with Extraluminal Device, Percutaneous Approach */
"06LN3DZ"  =  "1"  /* Occlusion of Left Femoral Vein with Intraluminal Device, Percutaneous Approach */
"06LN3ZZ"  =  "1"  /* Occlusion of Left Femoral Vein, Percutaneous Approach */
"06LN4CZ"  =  "1"  /* Occlusion of Left Femoral Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LN4DZ"  =  "1"  /* Occlusion of Left Femoral Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LN4ZZ"  =  "1"  /* Occlusion of Left Femoral Vein, Percutaneous Endoscopic Approach */
"06LP0CZ"  =  "1"  /* Occlusion of Right Greater Saphenous Vein with Extraluminal Device, Open Approach */
"06LP0DZ"  =  "1"  /* Occlusion of Right Greater Saphenous Vein with Intraluminal Device, Open Approach */
"06LP0ZZ"  =  "1"  /* Occlusion of Right Greater Saphenous Vein, Open Approach */
"06LP3CZ"  =  "1"  /* Occlusion of Right Greater Saphenous Vein with Extraluminal Device, Percutaneous Approach */
"06LP3DZ"  =  "1"  /* Occlusion of Right Greater Saphenous Vein with Intraluminal Device, Percutaneous Approach */
"06LP3ZZ"  =  "1"  /* Occlusion of Right Greater Saphenous Vein, Percutaneous Approach */
"06LP4CZ"  =  "1"  /* Occlusion of Right Greater Saphenous Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LP4DZ"  =  "1"  /* Occlusion of Right Greater Saphenous Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LP4ZZ"  =  "1"  /* Occlusion of Right Greater Saphenous Vein, Percutaneous Endoscopic Approach */
"06LQ0CZ"  =  "1"  /* Occlusion of Left Greater Saphenous Vein with Extraluminal Device, Open Approach */
"06LQ0DZ"  =  "1"  /* Occlusion of Left Greater Saphenous Vein with Intraluminal Device, Open Approach */
"06LQ0ZZ"  =  "1"  /* Occlusion of Left Greater Saphenous Vein, Open Approach */
"06LQ3CZ"  =  "1"  /* Occlusion of Left Greater Saphenous Vein with Extraluminal Device, Percutaneous Approach */
"06LQ3DZ"  =  "1"  /* Occlusion of Left Greater Saphenous Vein with Intraluminal Device, Percutaneous Approach */
"06LQ3ZZ"  =  "1"  /* Occlusion of Left Greater Saphenous Vein, Percutaneous Approach */
"06LQ4CZ"  =  "1"  /* Occlusion of Left Greater Saphenous Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LQ4DZ"  =  "1"  /* Occlusion of Left Greater Saphenous Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LQ4ZZ"  =  "1"  /* Occlusion of Left Greater Saphenous Vein, Percutaneous Endoscopic Approach */
"06LR0CZ"  =  "1"  /* Occlusion of Right Lesser Saphenous Vein with Extraluminal Device, Open Approach */
"06LR0DZ"  =  "1"  /* Occlusion of Right Lesser Saphenous Vein with Intraluminal Device, Open Approach */
"06LR0ZZ"  =  "1"  /* Occlusion of Right Lesser Saphenous Vein, Open Approach */
"06LR3CZ"  =  "1"  /* Occlusion of Right Lesser Saphenous Vein with Extraluminal Device, Percutaneous Approach */
"06LR3DZ"  =  "1"  /* Occlusion of Right Lesser Saphenous Vein with Intraluminal Device, Percutaneous Approach */
"06LR3ZZ"  =  "1"  /* Occlusion of Right Lesser Saphenous Vein, Percutaneous Approach */
"06LR4CZ"  =  "1"  /* Occlusion of Right Lesser Saphenous Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LR4DZ"  =  "1"  /* Occlusion of Right Lesser Saphenous Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LR4ZZ"  =  "1"  /* Occlusion of Right Lesser Saphenous Vein, Percutaneous Endoscopic Approach */
"06LS0CZ"  =  "1"  /* Occlusion of Left Lesser Saphenous Vein with Extraluminal Device, Open Approach */
"06LS0DZ"  =  "1"  /* Occlusion of Left Lesser Saphenous Vein with Intraluminal Device, Open Approach */
"06LS0ZZ"  =  "1"  /* Occlusion of Left Lesser Saphenous Vein, Open Approach */
"06LS3CZ"  =  "1"  /* Occlusion of Left Lesser Saphenous Vein with Extraluminal Device, Percutaneous Approach */
"06LS3DZ"  =  "1"  /* Occlusion of Left Lesser Saphenous Vein with Intraluminal Device, Percutaneous Approach */
"06LS3ZZ"  =  "1"  /* Occlusion of Left Lesser Saphenous Vein, Percutaneous Approach */
"06LS4CZ"  =  "1"  /* Occlusion of Left Lesser Saphenous Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LS4DZ"  =  "1"  /* Occlusion of Left Lesser Saphenous Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LS4ZZ"  =  "1"  /* Occlusion of Left Lesser Saphenous Vein, Percutaneous Endoscopic Approach */
"06LT0CZ"  =  "1"  /* Occlusion of Right Foot Vein with Extraluminal Device, Open Approach */
"06LT0DZ"  =  "1"  /* Occlusion of Right Foot Vein with Intraluminal Device, Open Approach */
"06LT0ZZ"  =  "1"  /* Occlusion of Right Foot Vein, Open Approach */
"06LT3CZ"  =  "1"  /* Occlusion of Right Foot Vein with Extraluminal Device, Percutaneous Approach */
"06LT3DZ"  =  "1"  /* Occlusion of Right Foot Vein with Intraluminal Device, Percutaneous Approach */
"06LT3ZZ"  =  "1"  /* Occlusion of Right Foot Vein, Percutaneous Approach */
"06LT4CZ"  =  "1"  /* Occlusion of Right Foot Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LT4DZ"  =  "1"  /* Occlusion of Right Foot Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LT4ZZ"  =  "1"  /* Occlusion of Right Foot Vein, Percutaneous Endoscopic Approach */
"06LV0CZ"  =  "1"  /* Occlusion of Left Foot Vein with Extraluminal Device, Open Approach */
"06LV0DZ"  =  "1"  /* Occlusion of Left Foot Vein with Intraluminal Device, Open Approach */
"06LV0ZZ"  =  "1"  /* Occlusion of Left Foot Vein, Open Approach */
"06LV3CZ"  =  "1"  /* Occlusion of Left Foot Vein with Extraluminal Device, Percutaneous Approach */
"06LV3DZ"  =  "1"  /* Occlusion of Left Foot Vein with Intraluminal Device, Percutaneous Approach */
"06LV3ZZ"  =  "1"  /* Occlusion of Left Foot Vein, Percutaneous Approach */
"06LV4CZ"  =  "1"  /* Occlusion of Left Foot Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LV4DZ"  =  "1"  /* Occlusion of Left Foot Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LV4ZZ"  =  "1"  /* Occlusion of Left Foot Vein, Percutaneous Endoscopic Approach */
"06LY0CC"  =  "1"  /* Occlusion of Hemorrhoidal Plexus with Extraluminal Device, Open Approach */
"06LY0CZ"  =  "1"  /* Occlusion of Lower Vein with Extraluminal Device, Open Approach */
"06LY0DC"  =  "1"  /* Occlusion of Hemorrhoidal Plexus with Intraluminal Device, Open Approach */
"06LY0DZ"  =  "1"  /* Occlusion of Lower Vein with Intraluminal Device, Open Approach */
"06LY0ZC"  =  "1"  /* Occlusion of Hemorrhoidal Plexus, Open Approach */
"06LY0ZZ"  =  "1"  /* Occlusion of Lower Vein, Open Approach */
"06LY3CC"  =  "1"  /* Occlusion of Hemorrhoidal Plexus with Extraluminal Device, Percutaneous Approach */
"06LY3CZ"  =  "1"  /* Occlusion of Lower Vein with Extraluminal Device, Percutaneous Approach */
"06LY3DC"  =  "1"  /* Occlusion of Hemorrhoidal Plexus with Intraluminal Device, Percutaneous Approach */
"06LY3DZ"  =  "1"  /* Occlusion of Lower Vein with Intraluminal Device, Percutaneous Approach */
"06LY3ZC"  =  "1"  /* Occlusion of Hemorrhoidal Plexus, Percutaneous Approach */
"06LY3ZZ"  =  "1"  /* Occlusion of Lower Vein, Percutaneous Approach */
"06LY4CC"  =  "1"  /* Occlusion of Hemorrhoidal Plexus with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LY4CZ"  =  "1"  /* Occlusion of Lower Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LY4DC"  =  "1"  /* Occlusion of Hemorrhoidal Plexus with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LY4DZ"  =  "1"  /* Occlusion of Lower Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LY4ZC"  =  "1"  /* Occlusion of Hemorrhoidal Plexus, Percutaneous Endoscopic Approach */
"06LY4ZZ"  =  "1"  /* Occlusion of Lower Vein, Percutaneous Endoscopic Approach */
"06PY00Z"  =  "1"  /* Removal of Drainage Device from Lower Vein, Open Approach */
"06PY02Z"  =  "1"  /* Removal of Monitoring Device from Lower Vein, Open Approach */
"06PY03Z"  =  "1"  /* Removal of Infusion Device from Lower Vein, Open Approach */
"06PY07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Vein, Open Approach */
"06PY0CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Vein, Open Approach */
"06PY0DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Vein, Open Approach */
"06PY0JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Vein, Open Approach */
"06PY0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Vein, Open Approach */
"06PY30Z"  =  "1"  /* Removal of Drainage Device from Lower Vein, Percutaneous Approach */
"06PY32Z"  =  "1"  /* Removal of Monitoring Device from Lower Vein, Percutaneous Approach */
"06PY33Z"  =  "1"  /* Removal of Infusion Device from Lower Vein, Percutaneous Approach */
"06PY37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Vein, Percutaneous Approach */
"06PY3CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Vein, Percutaneous Approach */
"06PY3DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Vein, Percutaneous Approach */
"06PY3JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Vein, Percutaneous Approach */
"06PY3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Vein, Percutaneous Approach */
"06PY40Z"  =  "1"  /* Removal of Drainage Device from Lower Vein, Percutaneous Endoscopic Approach */
"06PY42Z"  =  "1"  /* Removal of Monitoring Device from Lower Vein, Percutaneous Endoscopic Approach */
"06PY43Z"  =  "1"  /* Removal of Infusion Device from Lower Vein, Percutaneous Endoscopic Approach */
"06PY47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Vein, Percutaneous Endoscopic Approach */
"06PY4CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Vein, Percutaneous Endoscopic Approach */
"06PY4DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Vein, Percutaneous Endoscopic Approach */
"06PY4JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Vein, Percutaneous Endoscopic Approach */
"06PY4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Vein, Percutaneous Endoscopic Approach */
"06Q00ZZ"  =  "1"  /* Repair Inferior Vena Cava, Open Approach */
"06Q03ZZ"  =  "1"  /* Repair Inferior Vena Cava, Percutaneous Approach */
"06Q04ZZ"  =  "1"  /* Repair Inferior Vena Cava, Percutaneous Endoscopic Approach */
"06Q10ZZ"  =  "1"  /* Repair Splenic Vein, Open Approach */
"06Q13ZZ"  =  "1"  /* Repair Splenic Vein, Percutaneous Approach */
"06Q14ZZ"  =  "1"  /* Repair Splenic Vein, Percutaneous Endoscopic Approach */
"06Q20ZZ"  =  "1"  /* Repair Gastric Vein, Open Approach */
"06Q23ZZ"  =  "1"  /* Repair Gastric Vein, Percutaneous Approach */
"06Q24ZZ"  =  "1"  /* Repair Gastric Vein, Percutaneous Endoscopic Approach */
"06Q30ZZ"  =  "1"  /* Repair Esophageal Vein, Open Approach */
"06Q33ZZ"  =  "1"  /* Repair Esophageal Vein, Percutaneous Approach */
"06Q34ZZ"  =  "1"  /* Repair Esophageal Vein, Percutaneous Endoscopic Approach */
"06Q40ZZ"  =  "1"  /* Repair Hepatic Vein, Open Approach */
"06Q43ZZ"  =  "1"  /* Repair Hepatic Vein, Percutaneous Approach */
"06Q44ZZ"  =  "1"  /* Repair Hepatic Vein, Percutaneous Endoscopic Approach */
"06Q50ZZ"  =  "1"  /* Repair Superior Mesenteric Vein, Open Approach */
"06Q53ZZ"  =  "1"  /* Repair Superior Mesenteric Vein, Percutaneous Approach */
"06Q54ZZ"  =  "1"  /* Repair Superior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06Q60ZZ"  =  "1"  /* Repair Inferior Mesenteric Vein, Open Approach */
"06Q63ZZ"  =  "1"  /* Repair Inferior Mesenteric Vein, Percutaneous Approach */
"06Q64ZZ"  =  "1"  /* Repair Inferior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06Q70ZZ"  =  "1"  /* Repair Colic Vein, Open Approach */
"06Q73ZZ"  =  "1"  /* Repair Colic Vein, Percutaneous Approach */
"06Q74ZZ"  =  "1"  /* Repair Colic Vein, Percutaneous Endoscopic Approach */
"06Q80ZZ"  =  "1"  /* Repair Portal Vein, Open Approach */
"06Q83ZZ"  =  "1"  /* Repair Portal Vein, Percutaneous Approach */
"06Q84ZZ"  =  "1"  /* Repair Portal Vein, Percutaneous Endoscopic Approach */
"06Q90ZZ"  =  "1"  /* Repair Right Renal Vein, Open Approach */
"06Q93ZZ"  =  "1"  /* Repair Right Renal Vein, Percutaneous Approach */
"06Q94ZZ"  =  "1"  /* Repair Right Renal Vein, Percutaneous Endoscopic Approach */
"06QB0ZZ"  =  "1"  /* Repair Left Renal Vein, Open Approach */
"06QB3ZZ"  =  "1"  /* Repair Left Renal Vein, Percutaneous Approach */
"06QB4ZZ"  =  "1"  /* Repair Left Renal Vein, Percutaneous Endoscopic Approach */
"06QC0ZZ"  =  "1"  /* Repair Right Common Iliac Vein, Open Approach */
"06QC3ZZ"  =  "1"  /* Repair Right Common Iliac Vein, Percutaneous Approach */
"06QC4ZZ"  =  "1"  /* Repair Right Common Iliac Vein, Percutaneous Endoscopic Approach */
"06QD0ZZ"  =  "1"  /* Repair Left Common Iliac Vein, Open Approach */
"06QD3ZZ"  =  "1"  /* Repair Left Common Iliac Vein, Percutaneous Approach */
"06QD4ZZ"  =  "1"  /* Repair Left Common Iliac Vein, Percutaneous Endoscopic Approach */
"06QF0ZZ"  =  "1"  /* Repair Right External Iliac Vein, Open Approach */
"06QF3ZZ"  =  "1"  /* Repair Right External Iliac Vein, Percutaneous Approach */
"06QF4ZZ"  =  "1"  /* Repair Right External Iliac Vein, Percutaneous Endoscopic Approach */
"06QG0ZZ"  =  "1"  /* Repair Left External Iliac Vein, Open Approach */
"06QG3ZZ"  =  "1"  /* Repair Left External Iliac Vein, Percutaneous Approach */
"06QG4ZZ"  =  "1"  /* Repair Left External Iliac Vein, Percutaneous Endoscopic Approach */
"06QH0ZZ"  =  "1"  /* Repair Right Hypogastric Vein, Open Approach */
"06QH3ZZ"  =  "1"  /* Repair Right Hypogastric Vein, Percutaneous Approach */
"06QH4ZZ"  =  "1"  /* Repair Right Hypogastric Vein, Percutaneous Endoscopic Approach */
"06QJ0ZZ"  =  "1"  /* Repair Left Hypogastric Vein, Open Approach */
"06QJ3ZZ"  =  "1"  /* Repair Left Hypogastric Vein, Percutaneous Approach */
"06QJ4ZZ"  =  "1"  /* Repair Left Hypogastric Vein, Percutaneous Endoscopic Approach */
"06QM0ZZ"  =  "1"  /* Repair Right Femoral Vein, Open Approach */
"06QM3ZZ"  =  "1"  /* Repair Right Femoral Vein, Percutaneous Approach */
"06QM4ZZ"  =  "1"  /* Repair Right Femoral Vein, Percutaneous Endoscopic Approach */
"06QN0ZZ"  =  "1"  /* Repair Left Femoral Vein, Open Approach */
"06QN3ZZ"  =  "1"  /* Repair Left Femoral Vein, Percutaneous Approach */
"06QN4ZZ"  =  "1"  /* Repair Left Femoral Vein, Percutaneous Endoscopic Approach */
"06QP0ZZ"  =  "1"  /* Repair Right Greater Saphenous Vein, Open Approach */
"06QP3ZZ"  =  "1"  /* Repair Right Greater Saphenous Vein, Percutaneous Approach */
"06QP4ZZ"  =  "1"  /* Repair Right Greater Saphenous Vein, Percutaneous Endoscopic Approach */
"06QQ0ZZ"  =  "1"  /* Repair Left Greater Saphenous Vein, Open Approach */
"06QQ3ZZ"  =  "1"  /* Repair Left Greater Saphenous Vein, Percutaneous Approach */
"06QQ4ZZ"  =  "1"  /* Repair Left Greater Saphenous Vein, Percutaneous Endoscopic Approach */
"06QR0ZZ"  =  "1"  /* Repair Right Lesser Saphenous Vein, Open Approach */
"06QR3ZZ"  =  "1"  /* Repair Right Lesser Saphenous Vein, Percutaneous Approach */
"06QR4ZZ"  =  "1"  /* Repair Right Lesser Saphenous Vein, Percutaneous Endoscopic Approach */
"06QS0ZZ"  =  "1"  /* Repair Left Lesser Saphenous Vein, Open Approach */
"06QS3ZZ"  =  "1"  /* Repair Left Lesser Saphenous Vein, Percutaneous Approach */
"06QS4ZZ"  =  "1"  /* Repair Left Lesser Saphenous Vein, Percutaneous Endoscopic Approach */
"06QT0ZZ"  =  "1"  /* Repair Right Foot Vein, Open Approach */
"06QT3ZZ"  =  "1"  /* Repair Right Foot Vein, Percutaneous Approach */
"06QT4ZZ"  =  "1"  /* Repair Right Foot Vein, Percutaneous Endoscopic Approach */
"06QV0ZZ"  =  "1"  /* Repair Left Foot Vein, Open Approach */
"06QV3ZZ"  =  "1"  /* Repair Left Foot Vein, Percutaneous Approach */
"06QV4ZZ"  =  "1"  /* Repair Left Foot Vein, Percutaneous Endoscopic Approach */
"06QY0ZZ"  =  "1"  /* Repair Lower Vein, Open Approach */
"06QY3ZZ"  =  "1"  /* Repair Lower Vein, Percutaneous Approach */
"06QY4ZZ"  =  "1"  /* Repair Lower Vein, Percutaneous Endoscopic Approach */
"06V00CZ"  =  "1"  /* Restriction of Inferior Vena Cava with Extraluminal Device, Open Approach */
"06V00DZ"  =  "1"  /* Restriction of Inferior Vena Cava with Intraluminal Device, Open Approach */
"06V00ZZ"  =  "1"  /* Restriction of Inferior Vena Cava, Open Approach */
"06V03CZ"  =  "1"  /* Restriction of Inferior Vena Cava with Extraluminal Device, Percutaneous Approach */
"06V03DZ"  =  "1"  /* Restriction of Inferior Vena Cava with Intraluminal Device, Percutaneous Approach */
"06V03ZZ"  =  "1"  /* Restriction of Inferior Vena Cava, Percutaneous Approach */
"06V04CZ"  =  "1"  /* Restriction of Inferior Vena Cava with Extraluminal Device, Percutaneous Endoscopic Approach */
"06V04DZ"  =  "1"  /* Restriction of Inferior Vena Cava with Intraluminal Device, Percutaneous Endoscopic Approach */
"06V04ZZ"  =  "1"  /* Restriction of Inferior Vena Cava, Percutaneous Endoscopic Approach */
"06V10DZ"  =  "1"  /* Restriction of Splenic Vein with Intraluminal Device, Open Approach */
"06V10ZZ"  =  "1"  /* Restriction of Splenic Vein, Open Approach */
"06V13DZ"  =  "1"  /* Restriction of Splenic Vein with Intraluminal Device, Percutaneous Approach */
"06V13ZZ"  =  "1"  /* Restriction of Splenic Vein, Percutaneous Approach */
"06V14DZ"  =  "1"  /* Restriction of Splenic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06V14ZZ"  =  "1"  /* Restriction of Splenic Vein, Percutaneous Endoscopic Approach */
"06V20DZ"  =  "1"  /* Restriction of Gastric Vein with Intraluminal Device, Open Approach */
"06V20ZZ"  =  "1"  /* Restriction of Gastric Vein, Open Approach */
"06V23DZ"  =  "1"  /* Restriction of Gastric Vein with Intraluminal Device, Percutaneous Approach */
"06V23ZZ"  =  "1"  /* Restriction of Gastric Vein, Percutaneous Approach */
"06V24DZ"  =  "1"  /* Restriction of Gastric Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06V24ZZ"  =  "1"  /* Restriction of Gastric Vein, Percutaneous Endoscopic Approach */
"06V30DZ"  =  "1"  /* Restriction of Esophageal Vein with Intraluminal Device, Open Approach */
"06V30ZZ"  =  "1"  /* Restriction of Esophageal Vein, Open Approach */
"06V33DZ"  =  "1"  /* Restriction of Esophageal Vein with Intraluminal Device, Percutaneous Approach */
"06V33ZZ"  =  "1"  /* Restriction of Esophageal Vein, Percutaneous Approach */
"06V34DZ"  =  "1"  /* Restriction of Esophageal Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06V34ZZ"  =  "1"  /* Restriction of Esophageal Vein, Percutaneous Endoscopic Approach */
"06V40DZ"  =  "1"  /* Restriction of Hepatic Vein with Intraluminal Device, Open Approach */
"06V40ZZ"  =  "1"  /* Restriction of Hepatic Vein, Open Approach */
"06V43DZ"  =  "1"  /* Restriction of Hepatic Vein with Intraluminal Device, Percutaneous Approach */
"06V43ZZ"  =  "1"  /* Restriction of Hepatic Vein, Percutaneous Approach */
"06V44DZ"  =  "1"  /* Restriction of Hepatic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06V44ZZ"  =  "1"  /* Restriction of Hepatic Vein, Percutaneous Endoscopic Approach */
"06V50DZ"  =  "1"  /* Restriction of Superior Mesenteric Vein with Intraluminal Device, Open Approach */
"06V50ZZ"  =  "1"  /* Restriction of Superior Mesenteric Vein, Open Approach */
"06V53DZ"  =  "1"  /* Restriction of Superior Mesenteric Vein with Intraluminal Device, Percutaneous Approach */
"06V53ZZ"  =  "1"  /* Restriction of Superior Mesenteric Vein, Percutaneous Approach */
"06V54DZ"  =  "1"  /* Restriction of Superior Mesenteric Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06V54ZZ"  =  "1"  /* Restriction of Superior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06V60DZ"  =  "1"  /* Restriction of Inferior Mesenteric Vein with Intraluminal Device, Open Approach */
"06V60ZZ"  =  "1"  /* Restriction of Inferior Mesenteric Vein, Open Approach */
"06V63DZ"  =  "1"  /* Restriction of Inferior Mesenteric Vein with Intraluminal Device, Percutaneous Approach */
"06V63ZZ"  =  "1"  /* Restriction of Inferior Mesenteric Vein, Percutaneous Approach */
"06V64DZ"  =  "1"  /* Restriction of Inferior Mesenteric Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06V64ZZ"  =  "1"  /* Restriction of Inferior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06V70DZ"  =  "1"  /* Restriction of Colic Vein with Intraluminal Device, Open Approach */
"06V70ZZ"  =  "1"  /* Restriction of Colic Vein, Open Approach */
"06V73DZ"  =  "1"  /* Restriction of Colic Vein with Intraluminal Device, Percutaneous Approach */
"06V73ZZ"  =  "1"  /* Restriction of Colic Vein, Percutaneous Approach */
"06V74DZ"  =  "1"  /* Restriction of Colic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06V74ZZ"  =  "1"  /* Restriction of Colic Vein, Percutaneous Endoscopic Approach */
"06V80DZ"  =  "1"  /* Restriction of Portal Vein with Intraluminal Device, Open Approach */
"06V80ZZ"  =  "1"  /* Restriction of Portal Vein, Open Approach */
"06V83DZ"  =  "1"  /* Restriction of Portal Vein with Intraluminal Device, Percutaneous Approach */
"06V83ZZ"  =  "1"  /* Restriction of Portal Vein, Percutaneous Approach */
"06V84DZ"  =  "1"  /* Restriction of Portal Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06V84ZZ"  =  "1"  /* Restriction of Portal Vein, Percutaneous Endoscopic Approach */
"06V90DZ"  =  "1"  /* Restriction of Right Renal Vein with Intraluminal Device, Open Approach */
"06V90ZZ"  =  "1"  /* Restriction of Right Renal Vein, Open Approach */
"06V93DZ"  =  "1"  /* Restriction of Right Renal Vein with Intraluminal Device, Percutaneous Approach */
"06V93ZZ"  =  "1"  /* Restriction of Right Renal Vein, Percutaneous Approach */
"06V94DZ"  =  "1"  /* Restriction of Right Renal Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06V94ZZ"  =  "1"  /* Restriction of Right Renal Vein, Percutaneous Endoscopic Approach */
"06VB0DZ"  =  "1"  /* Restriction of Left Renal Vein with Intraluminal Device, Open Approach */
"06VB0ZZ"  =  "1"  /* Restriction of Left Renal Vein, Open Approach */
"06VB3DZ"  =  "1"  /* Restriction of Left Renal Vein with Intraluminal Device, Percutaneous Approach */
"06VB3ZZ"  =  "1"  /* Restriction of Left Renal Vein, Percutaneous Approach */
"06VB4DZ"  =  "1"  /* Restriction of Left Renal Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06VB4ZZ"  =  "1"  /* Restriction of Left Renal Vein, Percutaneous Endoscopic Approach */
"06VC0DZ"  =  "1"  /* Restriction of Right Common Iliac Vein with Intraluminal Device, Open Approach */
"06VC0ZZ"  =  "1"  /* Restriction of Right Common Iliac Vein, Open Approach */
"06VC3DZ"  =  "1"  /* Restriction of Right Common Iliac Vein with Intraluminal Device, Percutaneous Approach */
"06VC3ZZ"  =  "1"  /* Restriction of Right Common Iliac Vein, Percutaneous Approach */
"06VC4DZ"  =  "1"  /* Restriction of Right Common Iliac Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06VC4ZZ"  =  "1"  /* Restriction of Right Common Iliac Vein, Percutaneous Endoscopic Approach */
"06VD0DZ"  =  "1"  /* Restriction of Left Common Iliac Vein with Intraluminal Device, Open Approach */
"06VD0ZZ"  =  "1"  /* Restriction of Left Common Iliac Vein, Open Approach */
"06VD3DZ"  =  "1"  /* Restriction of Left Common Iliac Vein with Intraluminal Device, Percutaneous Approach */
"06VD3ZZ"  =  "1"  /* Restriction of Left Common Iliac Vein, Percutaneous Approach */
"06VD4DZ"  =  "1"  /* Restriction of Left Common Iliac Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06VD4ZZ"  =  "1"  /* Restriction of Left Common Iliac Vein, Percutaneous Endoscopic Approach */
"06VF0DZ"  =  "1"  /* Restriction of Right External Iliac Vein with Intraluminal Device, Open Approach */
"06VF0ZZ"  =  "1"  /* Restriction of Right External Iliac Vein, Open Approach */
"06VF3DZ"  =  "1"  /* Restriction of Right External Iliac Vein with Intraluminal Device, Percutaneous Approach */
"06VF3ZZ"  =  "1"  /* Restriction of Right External Iliac Vein, Percutaneous Approach */
"06VF4DZ"  =  "1"  /* Restriction of Right External Iliac Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06VF4ZZ"  =  "1"  /* Restriction of Right External Iliac Vein, Percutaneous Endoscopic Approach */
"06VG0DZ"  =  "1"  /* Restriction of Left External Iliac Vein with Intraluminal Device, Open Approach */
"06VG0ZZ"  =  "1"  /* Restriction of Left External Iliac Vein, Open Approach */
"06VG3DZ"  =  "1"  /* Restriction of Left External Iliac Vein with Intraluminal Device, Percutaneous Approach */
"06VG3ZZ"  =  "1"  /* Restriction of Left External Iliac Vein, Percutaneous Approach */
"06VG4DZ"  =  "1"  /* Restriction of Left External Iliac Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06VG4ZZ"  =  "1"  /* Restriction of Left External Iliac Vein, Percutaneous Endoscopic Approach */
"06VH0DZ"  =  "1"  /* Restriction of Right Hypogastric Vein with Intraluminal Device, Open Approach */
"06VH0ZZ"  =  "1"  /* Restriction of Right Hypogastric Vein, Open Approach */
"06VH3DZ"  =  "1"  /* Restriction of Right Hypogastric Vein with Intraluminal Device, Percutaneous Approach */
"06VH3ZZ"  =  "1"  /* Restriction of Right Hypogastric Vein, Percutaneous Approach */
"06VH4DZ"  =  "1"  /* Restriction of Right Hypogastric Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06VH4ZZ"  =  "1"  /* Restriction of Right Hypogastric Vein, Percutaneous Endoscopic Approach */
"06VJ0DZ"  =  "1"  /* Restriction of Left Hypogastric Vein with Intraluminal Device, Open Approach */
"06VJ0ZZ"  =  "1"  /* Restriction of Left Hypogastric Vein, Open Approach */
"06VJ3DZ"  =  "1"  /* Restriction of Left Hypogastric Vein with Intraluminal Device, Percutaneous Approach */
"06VJ3ZZ"  =  "1"  /* Restriction of Left Hypogastric Vein, Percutaneous Approach */
"06VJ4DZ"  =  "1"  /* Restriction of Left Hypogastric Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06VJ4ZZ"  =  "1"  /* Restriction of Left Hypogastric Vein, Percutaneous Endoscopic Approach */
"06VM0DZ"  =  "1"  /* Restriction of Right Femoral Vein with Intraluminal Device, Open Approach */
"06VM0ZZ"  =  "1"  /* Restriction of Right Femoral Vein, Open Approach */
"06VM3DZ"  =  "1"  /* Restriction of Right Femoral Vein with Intraluminal Device, Percutaneous Approach */
"06VM3ZZ"  =  "1"  /* Restriction of Right Femoral Vein, Percutaneous Approach */
"06VM4DZ"  =  "1"  /* Restriction of Right Femoral Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06VM4ZZ"  =  "1"  /* Restriction of Right Femoral Vein, Percutaneous Endoscopic Approach */
"06VN0DZ"  =  "1"  /* Restriction of Left Femoral Vein with Intraluminal Device, Open Approach */
"06VN0ZZ"  =  "1"  /* Restriction of Left Femoral Vein, Open Approach */
"06VN3DZ"  =  "1"  /* Restriction of Left Femoral Vein with Intraluminal Device, Percutaneous Approach */
"06VN3ZZ"  =  "1"  /* Restriction of Left Femoral Vein, Percutaneous Approach */
"06VN4DZ"  =  "1"  /* Restriction of Left Femoral Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06VN4ZZ"  =  "1"  /* Restriction of Left Femoral Vein, Percutaneous Endoscopic Approach */
"06VP0DZ"  =  "1"  /* Restriction of Right Greater Saphenous Vein with Intraluminal Device, Open Approach */
"06VP0ZZ"  =  "1"  /* Restriction of Right Greater Saphenous Vein, Open Approach */
"06VP3DZ"  =  "1"  /* Restriction of Right Greater Saphenous Vein with Intraluminal Device, Percutaneous Approach */
"06VP3ZZ"  =  "1"  /* Restriction of Right Greater Saphenous Vein, Percutaneous Approach */
"06VP4DZ"  =  "1"  /* Restriction of Right Greater Saphenous Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06VP4ZZ"  =  "1"  /* Restriction of Right Greater Saphenous Vein, Percutaneous Endoscopic Approach */
"06VQ0DZ"  =  "1"  /* Restriction of Left Greater Saphenous Vein with Intraluminal Device, Open Approach */
"06VQ0ZZ"  =  "1"  /* Restriction of Left Greater Saphenous Vein, Open Approach */
"06VQ3DZ"  =  "1"  /* Restriction of Left Greater Saphenous Vein with Intraluminal Device, Percutaneous Approach */
"06VQ3ZZ"  =  "1"  /* Restriction of Left Greater Saphenous Vein, Percutaneous Approach */
"06VQ4DZ"  =  "1"  /* Restriction of Left Greater Saphenous Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06VQ4ZZ"  =  "1"  /* Restriction of Left Greater Saphenous Vein, Percutaneous Endoscopic Approach */
"06VR0DZ"  =  "1"  /* Restriction of Right Lesser Saphenous Vein with Intraluminal Device, Open Approach */
"06VR0ZZ"  =  "1"  /* Restriction of Right Lesser Saphenous Vein, Open Approach */
"06VR3DZ"  =  "1"  /* Restriction of Right Lesser Saphenous Vein with Intraluminal Device, Percutaneous Approach */
"06VR3ZZ"  =  "1"  /* Restriction of Right Lesser Saphenous Vein, Percutaneous Approach */
"06VR4DZ"  =  "1"  /* Restriction of Right Lesser Saphenous Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06VR4ZZ"  =  "1"  /* Restriction of Right Lesser Saphenous Vein, Percutaneous Endoscopic Approach */
"06VS0DZ"  =  "1"  /* Restriction of Left Lesser Saphenous Vein with Intraluminal Device, Open Approach */
"06VS0ZZ"  =  "1"  /* Restriction of Left Lesser Saphenous Vein, Open Approach */
"06VS3DZ"  =  "1"  /* Restriction of Left Lesser Saphenous Vein with Intraluminal Device, Percutaneous Approach */
"06VS3ZZ"  =  "1"  /* Restriction of Left Lesser Saphenous Vein, Percutaneous Approach */
"06VS4DZ"  =  "1"  /* Restriction of Left Lesser Saphenous Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06VS4ZZ"  =  "1"  /* Restriction of Left Lesser Saphenous Vein, Percutaneous Endoscopic Approach */
"06VT0DZ"  =  "1"  /* Restriction of Right Foot Vein with Intraluminal Device, Open Approach */
"06VT0ZZ"  =  "1"  /* Restriction of Right Foot Vein, Open Approach */
"06VT3DZ"  =  "1"  /* Restriction of Right Foot Vein with Intraluminal Device, Percutaneous Approach */
"06VT3ZZ"  =  "1"  /* Restriction of Right Foot Vein, Percutaneous Approach */
"06VT4DZ"  =  "1"  /* Restriction of Right Foot Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06VT4ZZ"  =  "1"  /* Restriction of Right Foot Vein, Percutaneous Endoscopic Approach */
"06VV0DZ"  =  "1"  /* Restriction of Left Foot Vein with Intraluminal Device, Open Approach */
"06VV0ZZ"  =  "1"  /* Restriction of Left Foot Vein, Open Approach */
"06VV3DZ"  =  "1"  /* Restriction of Left Foot Vein with Intraluminal Device, Percutaneous Approach */
"06VV3ZZ"  =  "1"  /* Restriction of Left Foot Vein, Percutaneous Approach */
"06VV4DZ"  =  "1"  /* Restriction of Left Foot Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06VV4ZZ"  =  "1"  /* Restriction of Left Foot Vein, Percutaneous Endoscopic Approach */
"06VY0DZ"  =  "1"  /* Restriction of Lower Vein with Intraluminal Device, Open Approach */
"06VY0ZZ"  =  "1"  /* Restriction of Lower Vein, Open Approach */
"06VY3DZ"  =  "1"  /* Restriction of Lower Vein with Intraluminal Device, Percutaneous Approach */
"06VY3ZZ"  =  "1"  /* Restriction of Lower Vein, Percutaneous Approach */
"06VY4DZ"  =  "1"  /* Restriction of Lower Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06VY4ZZ"  =  "1"  /* Restriction of Lower Vein, Percutaneous Endoscopic Approach */
"06WY00Z"  =  "1"  /* Revision of Drainage Device in Lower Vein, Open Approach */
"06WY02Z"  =  "1"  /* Revision of Monitoring Device in Lower Vein, Open Approach */
"06WY03Z"  =  "1"  /* Revision of Infusion Device in Lower Vein, Open Approach */
"06WY07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Vein, Open Approach */
"06WY0CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Vein, Open Approach */
"06WY0DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Vein, Open Approach */
"06WY0JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Vein, Open Approach */
"06WY0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Vein, Open Approach */
"06WY30Z"  =  "1"  /* Revision of Drainage Device in Lower Vein, Percutaneous Approach */
"06WY32Z"  =  "1"  /* Revision of Monitoring Device in Lower Vein, Percutaneous Approach */
"06WY33Z"  =  "1"  /* Revision of Infusion Device in Lower Vein, Percutaneous Approach */
"06WY37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Vein, Percutaneous Approach */
"06WY3CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Vein, Percutaneous Approach */
"06WY3DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Vein, Percutaneous Approach */
"06WY3JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Vein, Percutaneous Approach */
"06WY3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Vein, Percutaneous Approach */
"06WY40Z"  =  "1"  /* Revision of Drainage Device in Lower Vein, Percutaneous Endoscopic Approach */
"06WY42Z"  =  "1"  /* Revision of Monitoring Device in Lower Vein, Percutaneous Endoscopic Approach */
"06WY43Z"  =  "1"  /* Revision of Infusion Device in Lower Vein, Percutaneous Endoscopic Approach */
"06WY47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Vein, Percutaneous Endoscopic Approach */
"06WY4CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Vein, Percutaneous Endoscopic Approach */
"06WY4DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Vein, Percutaneous Endoscopic Approach */
"06WY4JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Vein, Percutaneous Endoscopic Approach */
"06WY4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Vein, Percutaneous Endoscopic Approach */
"079000Z"  =  "1"  /* Drainage of Head Lymphatic with Drainage Device, Open Approach */
"07900ZZ"  =  "1"  /* Drainage of Head Lymphatic, Open Approach */
"079030Z"  =  "1"  /* Drainage of Head Lymphatic with Drainage Device, Percutaneous Approach */
"07903ZZ"  =  "1"  /* Drainage of Head Lymphatic, Percutaneous Approach */
"079040Z"  =  "1"  /* Drainage of Head Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"07904ZZ"  =  "1"  /* Drainage of Head Lymphatic, Percutaneous Endoscopic Approach */
"079100Z"  =  "1"  /* Drainage of Right Neck Lymphatic with Drainage Device, Open Approach */
"07910ZZ"  =  "1"  /* Drainage of Right Neck Lymphatic, Open Approach */
"079130Z"  =  "1"  /* Drainage of Right Neck Lymphatic with Drainage Device, Percutaneous Approach */
"07913ZZ"  =  "1"  /* Drainage of Right Neck Lymphatic, Percutaneous Approach */
"079140Z"  =  "1"  /* Drainage of Right Neck Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"07914ZZ"  =  "1"  /* Drainage of Right Neck Lymphatic, Percutaneous Endoscopic Approach */
"079200Z"  =  "1"  /* Drainage of Left Neck Lymphatic with Drainage Device, Open Approach */
"07920ZZ"  =  "1"  /* Drainage of Left Neck Lymphatic, Open Approach */
"079230Z"  =  "1"  /* Drainage of Left Neck Lymphatic with Drainage Device, Percutaneous Approach */
"07923ZZ"  =  "1"  /* Drainage of Left Neck Lymphatic, Percutaneous Approach */
"079240Z"  =  "1"  /* Drainage of Left Neck Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"07924ZZ"  =  "1"  /* Drainage of Left Neck Lymphatic, Percutaneous Endoscopic Approach */
"079300Z"  =  "1"  /* Drainage of Right Upper Extremity Lymphatic with Drainage Device, Open Approach */
"07930ZZ"  =  "1"  /* Drainage of Right Upper Extremity Lymphatic, Open Approach */
"079330Z"  =  "1"  /* Drainage of Right Upper Extremity Lymphatic with Drainage Device, Percutaneous Approach */
"07933ZZ"  =  "1"  /* Drainage of Right Upper Extremity Lymphatic, Percutaneous Approach */
"079340Z"  =  "1"  /* Drainage of Right Upper Extremity Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"07934ZZ"  =  "1"  /* Drainage of Right Upper Extremity Lymphatic, Percutaneous Endoscopic Approach */
"079400Z"  =  "1"  /* Drainage of Left Upper Extremity Lymphatic with Drainage Device, Open Approach */
"07940ZZ"  =  "1"  /* Drainage of Left Upper Extremity Lymphatic, Open Approach */
"079430Z"  =  "1"  /* Drainage of Left Upper Extremity Lymphatic with Drainage Device, Percutaneous Approach */
"07943ZZ"  =  "1"  /* Drainage of Left Upper Extremity Lymphatic, Percutaneous Approach */
"079440Z"  =  "1"  /* Drainage of Left Upper Extremity Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"07944ZZ"  =  "1"  /* Drainage of Left Upper Extremity Lymphatic, Percutaneous Endoscopic Approach */
"079500Z"  =  "1"  /* Drainage of Right Axillary Lymphatic with Drainage Device, Open Approach */
"07950ZZ"  =  "1"  /* Drainage of Right Axillary Lymphatic, Open Approach */
"079530Z"  =  "1"  /* Drainage of Right Axillary Lymphatic with Drainage Device, Percutaneous Approach */
"07953ZZ"  =  "1"  /* Drainage of Right Axillary Lymphatic, Percutaneous Approach */
"079540Z"  =  "1"  /* Drainage of Right Axillary Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"07954ZZ"  =  "1"  /* Drainage of Right Axillary Lymphatic, Percutaneous Endoscopic Approach */
"079600Z"  =  "1"  /* Drainage of Left Axillary Lymphatic with Drainage Device, Open Approach */
"07960ZZ"  =  "1"  /* Drainage of Left Axillary Lymphatic, Open Approach */
"079630Z"  =  "1"  /* Drainage of Left Axillary Lymphatic with Drainage Device, Percutaneous Approach */
"07963ZZ"  =  "1"  /* Drainage of Left Axillary Lymphatic, Percutaneous Approach */
"079640Z"  =  "1"  /* Drainage of Left Axillary Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"07964ZZ"  =  "1"  /* Drainage of Left Axillary Lymphatic, Percutaneous Endoscopic Approach */
"079700Z"  =  "1"  /* Drainage of Thorax Lymphatic with Drainage Device, Open Approach */
"07970ZZ"  =  "1"  /* Drainage of Thorax Lymphatic, Open Approach */
"079730Z"  =  "1"  /* Drainage of Thorax Lymphatic with Drainage Device, Percutaneous Approach */
"07973ZZ"  =  "1"  /* Drainage of Thorax Lymphatic, Percutaneous Approach */
"079740Z"  =  "1"  /* Drainage of Thorax Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"07974ZZ"  =  "1"  /* Drainage of Thorax Lymphatic, Percutaneous Endoscopic Approach */
"079800Z"  =  "1"  /* Drainage of Right Internal Mammary Lymphatic with Drainage Device, Open Approach */
"07980ZZ"  =  "1"  /* Drainage of Right Internal Mammary Lymphatic, Open Approach */
"079830Z"  =  "1"  /* Drainage of Right Internal Mammary Lymphatic with Drainage Device, Percutaneous Approach */
"07983ZZ"  =  "1"  /* Drainage of Right Internal Mammary Lymphatic, Percutaneous Approach */
"079840Z"  =  "1"  /* Drainage of Right Internal Mammary Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"07984ZZ"  =  "1"  /* Drainage of Right Internal Mammary Lymphatic, Percutaneous Endoscopic Approach */
"079900Z"  =  "1"  /* Drainage of Left Internal Mammary Lymphatic with Drainage Device, Open Approach */
"07990ZZ"  =  "1"  /* Drainage of Left Internal Mammary Lymphatic, Open Approach */
"079930Z"  =  "1"  /* Drainage of Left Internal Mammary Lymphatic with Drainage Device, Percutaneous Approach */
"07993ZZ"  =  "1"  /* Drainage of Left Internal Mammary Lymphatic, Percutaneous Approach */
"079940Z"  =  "1"  /* Drainage of Left Internal Mammary Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"07994ZZ"  =  "1"  /* Drainage of Left Internal Mammary Lymphatic, Percutaneous Endoscopic Approach */
"079B00Z"  =  "1"  /* Drainage of Mesenteric Lymphatic with Drainage Device, Open Approach */
"079B0ZZ"  =  "1"  /* Drainage of Mesenteric Lymphatic, Open Approach */
"079B30Z"  =  "1"  /* Drainage of Mesenteric Lymphatic with Drainage Device, Percutaneous Approach */
"079B3ZZ"  =  "1"  /* Drainage of Mesenteric Lymphatic, Percutaneous Approach */
"079B40Z"  =  "1"  /* Drainage of Mesenteric Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"079B4ZZ"  =  "1"  /* Drainage of Mesenteric Lymphatic, Percutaneous Endoscopic Approach */
"079C00Z"  =  "1"  /* Drainage of Pelvis Lymphatic with Drainage Device, Open Approach */
"079C0ZZ"  =  "1"  /* Drainage of Pelvis Lymphatic, Open Approach */
"079C30Z"  =  "1"  /* Drainage of Pelvis Lymphatic with Drainage Device, Percutaneous Approach */
"079C3ZZ"  =  "1"  /* Drainage of Pelvis Lymphatic, Percutaneous Approach */
"079C40Z"  =  "1"  /* Drainage of Pelvis Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"079C4ZZ"  =  "1"  /* Drainage of Pelvis Lymphatic, Percutaneous Endoscopic Approach */
"079D00Z"  =  "1"  /* Drainage of Aortic Lymphatic with Drainage Device, Open Approach */
"079D0ZZ"  =  "1"  /* Drainage of Aortic Lymphatic, Open Approach */
"079D30Z"  =  "1"  /* Drainage of Aortic Lymphatic with Drainage Device, Percutaneous Approach */
"079D3ZZ"  =  "1"  /* Drainage of Aortic Lymphatic, Percutaneous Approach */
"079D40Z"  =  "1"  /* Drainage of Aortic Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"079D4ZZ"  =  "1"  /* Drainage of Aortic Lymphatic, Percutaneous Endoscopic Approach */
"079F00Z"  =  "1"  /* Drainage of Right Lower Extremity Lymphatic with Drainage Device, Open Approach */
"079F0ZZ"  =  "1"  /* Drainage of Right Lower Extremity Lymphatic, Open Approach */
"079F30Z"  =  "1"  /* Drainage of Right Lower Extremity Lymphatic with Drainage Device, Percutaneous Approach */
"079F3ZZ"  =  "1"  /* Drainage of Right Lower Extremity Lymphatic, Percutaneous Approach */
"079F40Z"  =  "1"  /* Drainage of Right Lower Extremity Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"079F4ZZ"  =  "1"  /* Drainage of Right Lower Extremity Lymphatic, Percutaneous Endoscopic Approach */
"079G00Z"  =  "1"  /* Drainage of Left Lower Extremity Lymphatic with Drainage Device, Open Approach */
"079G0ZZ"  =  "1"  /* Drainage of Left Lower Extremity Lymphatic, Open Approach */
"079G30Z"  =  "1"  /* Drainage of Left Lower Extremity Lymphatic with Drainage Device, Percutaneous Approach */
"079G3ZZ"  =  "1"  /* Drainage of Left Lower Extremity Lymphatic, Percutaneous Approach */
"079G40Z"  =  "1"  /* Drainage of Left Lower Extremity Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"079G4ZZ"  =  "1"  /* Drainage of Left Lower Extremity Lymphatic, Percutaneous Endoscopic Approach */
"079H00Z"  =  "1"  /* Drainage of Right Inguinal Lymphatic with Drainage Device, Open Approach */
"079H0ZZ"  =  "1"  /* Drainage of Right Inguinal Lymphatic, Open Approach */
"079H30Z"  =  "1"  /* Drainage of Right Inguinal Lymphatic with Drainage Device, Percutaneous Approach */
"079H3ZZ"  =  "1"  /* Drainage of Right Inguinal Lymphatic, Percutaneous Approach */
"079H40Z"  =  "1"  /* Drainage of Right Inguinal Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"079H4ZZ"  =  "1"  /* Drainage of Right Inguinal Lymphatic, Percutaneous Endoscopic Approach */
"079J00Z"  =  "1"  /* Drainage of Left Inguinal Lymphatic with Drainage Device, Open Approach */
"079J0ZZ"  =  "1"  /* Drainage of Left Inguinal Lymphatic, Open Approach */
"079J30Z"  =  "1"  /* Drainage of Left Inguinal Lymphatic with Drainage Device, Percutaneous Approach */
"079J3ZZ"  =  "1"  /* Drainage of Left Inguinal Lymphatic, Percutaneous Approach */
"079J40Z"  =  "1"  /* Drainage of Left Inguinal Lymphatic with Drainage Device, Percutaneous Endoscopic Approach */
"079J4ZZ"  =  "1"  /* Drainage of Left Inguinal Lymphatic, Percutaneous Endoscopic Approach */
"079M00Z"  =  "1"  /* Drainage of Thymus with Drainage Device, Open Approach */
"079M0ZZ"  =  "1"  /* Drainage of Thymus, Open Approach */
"079M30Z"  =  "1"  /* Drainage of Thymus with Drainage Device, Percutaneous Approach */
"079M3ZZ"  =  "1"  /* Drainage of Thymus, Percutaneous Approach */
"079M40Z"  =  "1"  /* Drainage of Thymus with Drainage Device, Percutaneous Endoscopic Approach */
"079M4ZZ"  =  "1"  /* Drainage of Thymus, Percutaneous Endoscopic Approach */
"079P00Z"  =  "1"  /* Drainage of Spleen with Drainage Device, Open Approach */
"079P0ZZ"  =  "1"  /* Drainage of Spleen, Open Approach */
"07C00ZZ"  =  "1"  /* Extirpation of Matter from Head Lymphatic, Open Approach */
"07C03ZZ"  =  "1"  /* Extirpation of Matter from Head Lymphatic, Percutaneous Approach */
"07C04ZZ"  =  "1"  /* Extirpation of Matter from Head Lymphatic, Percutaneous Endoscopic Approach */
"07C10ZZ"  =  "1"  /* Extirpation of Matter from Right Neck Lymphatic, Open Approach */
"07C13ZZ"  =  "1"  /* Extirpation of Matter from Right Neck Lymphatic, Percutaneous Approach */
"07C14ZZ"  =  "1"  /* Extirpation of Matter from Right Neck Lymphatic, Percutaneous Endoscopic Approach */
"07C20ZZ"  =  "1"  /* Extirpation of Matter from Left Neck Lymphatic, Open Approach */
"07C23ZZ"  =  "1"  /* Extirpation of Matter from Left Neck Lymphatic, Percutaneous Approach */
"07C24ZZ"  =  "1"  /* Extirpation of Matter from Left Neck Lymphatic, Percutaneous Endoscopic Approach */
"07C30ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Extremity Lymphatic, Open Approach */
"07C33ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Extremity Lymphatic, Percutaneous Approach */
"07C34ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Extremity Lymphatic, Percutaneous Endoscopic Approach */
"07C40ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Extremity Lymphatic, Open Approach */
"07C43ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Extremity Lymphatic, Percutaneous Approach */
"07C44ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Extremity Lymphatic, Percutaneous Endoscopic Approach */
"07C50ZZ"  =  "1"  /* Extirpation of Matter from Right Axillary Lymphatic, Open Approach */
"07C53ZZ"  =  "1"  /* Extirpation of Matter from Right Axillary Lymphatic, Percutaneous Approach */
"07C54ZZ"  =  "1"  /* Extirpation of Matter from Right Axillary Lymphatic, Percutaneous Endoscopic Approach */
"07C60ZZ"  =  "1"  /* Extirpation of Matter from Left Axillary Lymphatic, Open Approach */
"07C63ZZ"  =  "1"  /* Extirpation of Matter from Left Axillary Lymphatic, Percutaneous Approach */
"07C64ZZ"  =  "1"  /* Extirpation of Matter from Left Axillary Lymphatic, Percutaneous Endoscopic Approach */
"07C70ZZ"  =  "1"  /* Extirpation of Matter from Thorax Lymphatic, Open Approach */
"07C73ZZ"  =  "1"  /* Extirpation of Matter from Thorax Lymphatic, Percutaneous Approach */
"07C74ZZ"  =  "1"  /* Extirpation of Matter from Thorax Lymphatic, Percutaneous Endoscopic Approach */
"07C80ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Mammary Lymphatic, Open Approach */
"07C83ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Mammary Lymphatic, Percutaneous Approach */
"07C84ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Mammary Lymphatic, Percutaneous Endoscopic Approach */
"07C90ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Mammary Lymphatic, Open Approach */
"07C93ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Mammary Lymphatic, Percutaneous Approach */
"07C94ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Mammary Lymphatic, Percutaneous Endoscopic Approach */
"07CB0ZZ"  =  "1"  /* Extirpation of Matter from Mesenteric Lymphatic, Open Approach */
"07CB3ZZ"  =  "1"  /* Extirpation of Matter from Mesenteric Lymphatic, Percutaneous Approach */
"07CB4ZZ"  =  "1"  /* Extirpation of Matter from Mesenteric Lymphatic, Percutaneous Endoscopic Approach */
"07CC0ZZ"  =  "1"  /* Extirpation of Matter from Pelvis Lymphatic, Open Approach */
"07CC3ZZ"  =  "1"  /* Extirpation of Matter from Pelvis Lymphatic, Percutaneous Approach */
"07CC4ZZ"  =  "1"  /* Extirpation of Matter from Pelvis Lymphatic, Percutaneous Endoscopic Approach */
"07CD0ZZ"  =  "1"  /* Extirpation of Matter from Aortic Lymphatic, Open Approach */
"07CD3ZZ"  =  "1"  /* Extirpation of Matter from Aortic Lymphatic, Percutaneous Approach */
"07CD4ZZ"  =  "1"  /* Extirpation of Matter from Aortic Lymphatic, Percutaneous Endoscopic Approach */
"07CF0ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Extremity Lymphatic, Open Approach */
"07CF3ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Extremity Lymphatic, Percutaneous Approach */
"07CF4ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Extremity Lymphatic, Percutaneous Endoscopic Approach */
"07CG0ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Extremity Lymphatic, Open Approach */
"07CG3ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Extremity Lymphatic, Percutaneous Approach */
"07CG4ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Extremity Lymphatic, Percutaneous Endoscopic Approach */
"07CH0ZZ"  =  "1"  /* Extirpation of Matter from Right Inguinal Lymphatic, Open Approach */
"07CH3ZZ"  =  "1"  /* Extirpation of Matter from Right Inguinal Lymphatic, Percutaneous Approach */
"07CH4ZZ"  =  "1"  /* Extirpation of Matter from Right Inguinal Lymphatic, Percutaneous Endoscopic Approach */
"07CJ0ZZ"  =  "1"  /* Extirpation of Matter from Left Inguinal Lymphatic, Open Approach */
"07CJ3ZZ"  =  "1"  /* Extirpation of Matter from Left Inguinal Lymphatic, Percutaneous Approach */
"07CJ4ZZ"  =  "1"  /* Extirpation of Matter from Left Inguinal Lymphatic, Percutaneous Endoscopic Approach */
"07CK0ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Duct, Open Approach */
"07CK3ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Duct, Percutaneous Approach */
"07CK4ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Duct, Percutaneous Endoscopic Approach */
"07CL0ZZ"  =  "1"  /* Extirpation of Matter from Cisterna Chyli, Open Approach */
"07CL3ZZ"  =  "1"  /* Extirpation of Matter from Cisterna Chyli, Percutaneous Approach */
"07CL4ZZ"  =  "1"  /* Extirpation of Matter from Cisterna Chyli, Percutaneous Endoscopic Approach */
"07CM0ZZ"  =  "1"  /* Extirpation of Matter from Thymus, Open Approach */
"07CM3ZZ"  =  "1"  /* Extirpation of Matter from Thymus, Percutaneous Approach */
"07CP0ZZ"  =  "1"  /* Extirpation of Matter from Spleen, Open Approach */
"07JM0ZZ"  =  "1"  /* Inspection of Thymus, Open Approach */
"07JP0ZZ"  =  "1"  /* Inspection of Spleen, Open Approach */
"07PM00Z"  =  "1"  /* Removal of Drainage Device from Thymus, Open Approach */
"07PM03Z"  =  "1"  /* Removal of Infusion Device from Thymus, Open Approach */
"07PM30Z"  =  "1"  /* Removal of Drainage Device from Thymus, Percutaneous Approach */
"07PM33Z"  =  "1"  /* Removal of Infusion Device from Thymus, Percutaneous Approach */
"07PM40Z"  =  "1"  /* Removal of Drainage Device from Thymus, Percutaneous Endoscopic Approach */
"07PM43Z"  =  "1"  /* Removal of Infusion Device from Thymus, Percutaneous Endoscopic Approach */
"07PN00Z"  =  "1"  /* Removal of Drainage Device from Lymphatic, Open Approach */
"07PN03Z"  =  "1"  /* Removal of Infusion Device from Lymphatic, Open Approach */
"07PN0CZ"  =  "1"  /* Removal of Extraluminal Device from Lymphatic, Open Approach */
"07PN0DZ"  =  "1"  /* Removal of Intraluminal Device from Lymphatic, Open Approach */
"07PN30Z"  =  "1"  /* Removal of Drainage Device from Lymphatic, Percutaneous Approach */
"07PN33Z"  =  "1"  /* Removal of Infusion Device from Lymphatic, Percutaneous Approach */
"07PN3CZ"  =  "1"  /* Removal of Extraluminal Device from Lymphatic, Percutaneous Approach */
"07PN3DZ"  =  "1"  /* Removal of Intraluminal Device from Lymphatic, Percutaneous Approach */
"07PN40Z"  =  "1"  /* Removal of Drainage Device from Lymphatic, Percutaneous Endoscopic Approach */
"07PN43Z"  =  "1"  /* Removal of Infusion Device from Lymphatic, Percutaneous Endoscopic Approach */
"07PN4CZ"  =  "1"  /* Removal of Extraluminal Device from Lymphatic, Percutaneous Endoscopic Approach */
"07PN4DZ"  =  "1"  /* Removal of Intraluminal Device from Lymphatic, Percutaneous Endoscopic Approach */
"07PP00Z"  =  "1"  /* Removal of Drainage Device from Spleen, Open Approach */
"07PP03Z"  =  "1"  /* Removal of Infusion Device from Spleen, Open Approach */
"07PP30Z"  =  "1"  /* Removal of Drainage Device from Spleen, Percutaneous Approach */
"07PP33Z"  =  "1"  /* Removal of Infusion Device from Spleen, Percutaneous Approach */
"07PP40Z"  =  "1"  /* Removal of Drainage Device from Spleen, Percutaneous Endoscopic Approach */
"07PP43Z"  =  "1"  /* Removal of Infusion Device from Spleen, Percutaneous Endoscopic Approach */
"07WM00Z"  =  "1"  /* Revision of Drainage Device in Thymus, Open Approach */
"07WM03Z"  =  "1"  /* Revision of Infusion Device in Thymus, Open Approach */
"07WM30Z"  =  "1"  /* Revision of Drainage Device in Thymus, Percutaneous Approach */
"07WM33Z"  =  "1"  /* Revision of Infusion Device in Thymus, Percutaneous Approach */
"07WM40Z"  =  "1"  /* Revision of Drainage Device in Thymus, Percutaneous Endoscopic Approach */
"07WM43Z"  =  "1"  /* Revision of Infusion Device in Thymus, Percutaneous Endoscopic Approach */
"07WN00Z"  =  "1"  /* Revision of Drainage Device in Lymphatic, Open Approach */
"07WN03Z"  =  "1"  /* Revision of Infusion Device in Lymphatic, Open Approach */
"07WN0CZ"  =  "1"  /* Revision of Extraluminal Device in Lymphatic, Open Approach */
"07WN0DZ"  =  "1"  /* Revision of Intraluminal Device in Lymphatic, Open Approach */
"07WN30Z"  =  "1"  /* Revision of Drainage Device in Lymphatic, Percutaneous Approach */
"07WN33Z"  =  "1"  /* Revision of Infusion Device in Lymphatic, Percutaneous Approach */
"07WN3CZ"  =  "1"  /* Revision of Extraluminal Device in Lymphatic, Percutaneous Approach */
"07WN3DZ"  =  "1"  /* Revision of Intraluminal Device in Lymphatic, Percutaneous Approach */
"07WN40Z"  =  "1"  /* Revision of Drainage Device in Lymphatic, Percutaneous Endoscopic Approach */
"07WN43Z"  =  "1"  /* Revision of Infusion Device in Lymphatic, Percutaneous Endoscopic Approach */
"07WN4CZ"  =  "1"  /* Revision of Extraluminal Device in Lymphatic, Percutaneous Endoscopic Approach */
"07WN4DZ"  =  "1"  /* Revision of Intraluminal Device in Lymphatic, Percutaneous Endoscopic Approach */
"07WP00Z"  =  "1"  /* Revision of Drainage Device in Spleen, Open Approach */
"07WP03Z"  =  "1"  /* Revision of Infusion Device in Spleen, Open Approach */
"07WP30Z"  =  "1"  /* Revision of Drainage Device in Spleen, Percutaneous Approach */
"07WP33Z"  =  "1"  /* Revision of Infusion Device in Spleen, Percutaneous Approach */
"07WP40Z"  =  "1"  /* Revision of Drainage Device in Spleen, Percutaneous Endoscopic Approach */
"07WP43Z"  =  "1"  /* Revision of Infusion Device in Spleen, Percutaneous Endoscopic Approach */
"085G3ZZ"  =  "1"  /* Destruction of Right Retinal Vessel, Percutaneous Approach */
"085H3ZZ"  =  "1"  /* Destruction of Left Retinal Vessel, Percutaneous Approach */
"0896X0Z"  =  "1"  /* Drainage of Right Sclera with Drainage Device, External Approach */
"0896XZZ"  =  "1"  /* Drainage of Right Sclera, External Approach */
"0897X0Z"  =  "1"  /* Drainage of Left Sclera with Drainage Device, External Approach */
"0897XZZ"  =  "1"  /* Drainage of Left Sclera, External Approach */
"089A00Z"  =  "1"  /* Drainage of Right Choroid with Drainage Device, Open Approach */
"089A0ZZ"  =  "1"  /* Drainage of Right Choroid, Open Approach */
"089A30Z"  =  "1"  /* Drainage of Right Choroid with Drainage Device, Percutaneous Approach */
"089A3ZZ"  =  "1"  /* Drainage of Right Choroid, Percutaneous Approach */
"089B00Z"  =  "1"  /* Drainage of Left Choroid with Drainage Device, Open Approach */
"089B0ZZ"  =  "1"  /* Drainage of Left Choroid, Open Approach */
"089B30Z"  =  "1"  /* Drainage of Left Choroid with Drainage Device, Percutaneous Approach */
"089B3ZZ"  =  "1"  /* Drainage of Left Choroid, Percutaneous Approach */
"089E30Z"  =  "1"  /* Drainage of Right Retina with Drainage Device, Percutaneous Approach */
"089E3ZZ"  =  "1"  /* Drainage of Right Retina, Percutaneous Approach */
"089F30Z"  =  "1"  /* Drainage of Left Retina with Drainage Device, Percutaneous Approach */
"089F3ZZ"  =  "1"  /* Drainage of Left Retina, Percutaneous Approach */
"089G30Z"  =  "1"  /* Drainage of Right Retinal Vessel with Drainage Device, Percutaneous Approach */
"089G3ZZ"  =  "1"  /* Drainage of Right Retinal Vessel, Percutaneous Approach */
"089H30Z"  =  "1"  /* Drainage of Left Retinal Vessel with Drainage Device, Percutaneous Approach */
"089H3ZZ"  =  "1"  /* Drainage of Left Retinal Vessel, Percutaneous Approach */
"089N00Z"  =  "1"  /* Drainage of Right Upper Eyelid with Drainage Device, Open Approach */
"089N0ZZ"  =  "1"  /* Drainage of Right Upper Eyelid, Open Approach */
"089N30Z"  =  "1"  /* Drainage of Right Upper Eyelid with Drainage Device, Percutaneous Approach */
"089N3ZZ"  =  "1"  /* Drainage of Right Upper Eyelid, Percutaneous Approach */
"089NX0Z"  =  "1"  /* Drainage of Right Upper Eyelid with Drainage Device, External Approach */
"089NXZZ"  =  "1"  /* Drainage of Right Upper Eyelid, External Approach */
"089P00Z"  =  "1"  /* Drainage of Left Upper Eyelid with Drainage Device, Open Approach */
"089P0ZZ"  =  "1"  /* Drainage of Left Upper Eyelid, Open Approach */
"089P30Z"  =  "1"  /* Drainage of Left Upper Eyelid with Drainage Device, Percutaneous Approach */
"089P3ZZ"  =  "1"  /* Drainage of Left Upper Eyelid, Percutaneous Approach */
"089PX0Z"  =  "1"  /* Drainage of Left Upper Eyelid with Drainage Device, External Approach */
"089PXZZ"  =  "1"  /* Drainage of Left Upper Eyelid, External Approach */
"089Q00Z"  =  "1"  /* Drainage of Right Lower Eyelid with Drainage Device, Open Approach */
"089Q0ZZ"  =  "1"  /* Drainage of Right Lower Eyelid, Open Approach */
"089Q30Z"  =  "1"  /* Drainage of Right Lower Eyelid with Drainage Device, Percutaneous Approach */
"089Q3ZZ"  =  "1"  /* Drainage of Right Lower Eyelid, Percutaneous Approach */
"089QX0Z"  =  "1"  /* Drainage of Right Lower Eyelid with Drainage Device, External Approach */
"089QXZZ"  =  "1"  /* Drainage of Right Lower Eyelid, External Approach */
"089R00Z"  =  "1"  /* Drainage of Left Lower Eyelid with Drainage Device, Open Approach */
"089R0ZZ"  =  "1"  /* Drainage of Left Lower Eyelid, Open Approach */
"089R30Z"  =  "1"  /* Drainage of Left Lower Eyelid with Drainage Device, Percutaneous Approach */
"089R3ZZ"  =  "1"  /* Drainage of Left Lower Eyelid, Percutaneous Approach */
"089RX0Z"  =  "1"  /* Drainage of Left Lower Eyelid with Drainage Device, External Approach */
"089RXZZ"  =  "1"  /* Drainage of Left Lower Eyelid, External Approach */
"089V00Z"  =  "1"  /* Drainage of Right Lacrimal Gland with Drainage Device, Open Approach */
"089V0ZZ"  =  "1"  /* Drainage of Right Lacrimal Gland, Open Approach */
"089V30Z"  =  "1"  /* Drainage of Right Lacrimal Gland with Drainage Device, Percutaneous Approach */
"089V3ZZ"  =  "1"  /* Drainage of Right Lacrimal Gland, Percutaneous Approach */
"089W00Z"  =  "1"  /* Drainage of Left Lacrimal Gland with Drainage Device, Open Approach */
"089W0ZZ"  =  "1"  /* Drainage of Left Lacrimal Gland, Open Approach */
"089W30Z"  =  "1"  /* Drainage of Left Lacrimal Gland with Drainage Device, Percutaneous Approach */
"089W3ZZ"  =  "1"  /* Drainage of Left Lacrimal Gland, Percutaneous Approach */
"089X00Z"  =  "1"  /* Drainage of Right Lacrimal Duct with Drainage Device, Open Approach */
"089X0ZZ"  =  "1"  /* Drainage of Right Lacrimal Duct, Open Approach */
"089X30Z"  =  "1"  /* Drainage of Right Lacrimal Duct with Drainage Device, Percutaneous Approach */
"089X3ZZ"  =  "1"  /* Drainage of Right Lacrimal Duct, Percutaneous Approach */
"089X70Z"  =  "1"  /* Drainage of Right Lacrimal Duct with Drainage Device, Via Natural or Artificial Opening */
"089X7ZZ"  =  "1"  /* Drainage of Right Lacrimal Duct, Via Natural or Artificial Opening */
"089X80Z"  =  "1"  /* Drainage of Right Lacrimal Duct with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"089X8ZZ"  =  "1"  /* Drainage of Right Lacrimal Duct, Via Natural or Artificial Opening Endoscopic */
"089Y00Z"  =  "1"  /* Drainage of Left Lacrimal Duct with Drainage Device, Open Approach */
"089Y0ZZ"  =  "1"  /* Drainage of Left Lacrimal Duct, Open Approach */
"089Y30Z"  =  "1"  /* Drainage of Left Lacrimal Duct with Drainage Device, Percutaneous Approach */
"089Y3ZZ"  =  "1"  /* Drainage of Left Lacrimal Duct, Percutaneous Approach */
"089Y70Z"  =  "1"  /* Drainage of Left Lacrimal Duct with Drainage Device, Via Natural or Artificial Opening */
"089Y7ZZ"  =  "1"  /* Drainage of Left Lacrimal Duct, Via Natural or Artificial Opening */
"089Y80Z"  =  "1"  /* Drainage of Left Lacrimal Duct with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"089Y8ZZ"  =  "1"  /* Drainage of Left Lacrimal Duct, Via Natural or Artificial Opening Endoscopic */
"08BA0ZZ"  =  "1"  /* Excision of Right Choroid, Open Approach */
"08BA3ZZ"  =  "1"  /* Excision of Right Choroid, Percutaneous Approach */
"08BB0ZZ"  =  "1"  /* Excision of Left Choroid, Open Approach */
"08BB3ZZ"  =  "1"  /* Excision of Left Choroid, Percutaneous Approach */
"08CG3ZZ"  =  "1"  /* Extirpation of Matter from Right Retinal Vessel, Percutaneous Approach */
"08CH3ZZ"  =  "1"  /* Extirpation of Matter from Left Retinal Vessel, Percutaneous Approach */
"08CN0ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Eyelid, Open Approach */
"08CN3ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Eyelid, Percutaneous Approach */
"08CP0ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Eyelid, Open Approach */
"08CP3ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Eyelid, Percutaneous Approach */
"08CQ0ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Eyelid, Open Approach */
"08CQ3ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Eyelid, Percutaneous Approach */
"08CR0ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Eyelid, Open Approach */
"08CR3ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Eyelid, Percutaneous Approach */
"08CV0ZZ"  =  "1"  /* Extirpation of Matter from Right Lacrimal Gland, Open Approach */
"08CW0ZZ"  =  "1"  /* Extirpation of Matter from Left Lacrimal Gland, Open Approach */
"08CX0ZZ"  =  "1"  /* Extirpation of Matter from Right Lacrimal Duct, Open Approach */
"08CX3ZZ"  =  "1"  /* Extirpation of Matter from Right Lacrimal Duct, Percutaneous Approach */
"08CY0ZZ"  =  "1"  /* Extirpation of Matter from Left Lacrimal Duct, Open Approach */
"08CY3ZZ"  =  "1"  /* Extirpation of Matter from Left Lacrimal Duct, Percutaneous Approach */
"08N6XZZ"  =  "1"  /* Release Right Sclera, External Approach */
"08N7XZZ"  =  "1"  /* Release Left Sclera, External Approach */
"08NA0ZZ"  =  "1"  /* Release Right Choroid, Open Approach */
"08NA3ZZ"  =  "1"  /* Release Right Choroid, Percutaneous Approach */
"08NB0ZZ"  =  "1"  /* Release Left Choroid, Open Approach */
"08NB3ZZ"  =  "1"  /* Release Left Choroid, Percutaneous Approach */
"08NE3ZZ"  =  "1"  /* Release Right Retina, Percutaneous Approach */
"08NF3ZZ"  =  "1"  /* Release Left Retina, Percutaneous Approach */
"08NG3ZZ"  =  "1"  /* Release Right Retinal Vessel, Percutaneous Approach */
"08NH3ZZ"  =  "1"  /* Release Left Retinal Vessel, Percutaneous Approach */
"08Q6XZZ"  =  "1"  /* Repair Right Sclera, External Approach */
"08Q7XZZ"  =  "1"  /* Repair Left Sclera, External Approach */
"08QA0ZZ"  =  "1"  /* Repair Right Choroid, Open Approach */
"08QA3ZZ"  =  "1"  /* Repair Right Choroid, Percutaneous Approach */
"08QB0ZZ"  =  "1"  /* Repair Left Choroid, Open Approach */
"08QB3ZZ"  =  "1"  /* Repair Left Choroid, Percutaneous Approach */
"08QE3ZZ"  =  "1"  /* Repair Right Retina, Percutaneous Approach */
"08QF3ZZ"  =  "1"  /* Repair Left Retina, Percutaneous Approach */
"08QG3ZZ"  =  "1"  /* Repair Right Retinal Vessel, Percutaneous Approach */
"08QH3ZZ"  =  "1"  /* Repair Left Retinal Vessel, Percutaneous Approach */
"08RG37Z"  =  "1"  /* Replacement of Right Retinal Vessel with Autologous Tissue Substitute, Percutaneous Approach */
"08RG3JZ"  =  "1"  /* Replacement of Right Retinal Vessel with Synthetic Substitute, Percutaneous Approach */
"08RG3KZ"  =  "1"  /* Replacement of Right Retinal Vessel with Nonautologous Tissue Substitute, Percutaneous Approach */
"08RH37Z"  =  "1"  /* Replacement of Left Retinal Vessel with Autologous Tissue Substitute, Percutaneous Approach */
"08RH3JZ"  =  "1"  /* Replacement of Left Retinal Vessel with Synthetic Substitute, Percutaneous Approach */
"08RH3KZ"  =  "1"  /* Replacement of Left Retinal Vessel with Nonautologous Tissue Substitute, Percutaneous Approach */
"08SG3ZZ"  =  "1"  /* Reposition Right Retinal Vessel, Percutaneous Approach */
"08SH3ZZ"  =  "1"  /* Reposition Left Retinal Vessel, Percutaneous Approach */
"08UG07Z"  =  "1"  /* Supplement Right Retinal Vessel with Autologous Tissue Substitute, Open Approach */
"08UG0JZ"  =  "1"  /* Supplement Right Retinal Vessel with Synthetic Substitute, Open Approach */
"08UG0KZ"  =  "1"  /* Supplement Right Retinal Vessel with Nonautologous Tissue Substitute, Open Approach */
"08UG37Z"  =  "1"  /* Supplement Right Retinal Vessel with Autologous Tissue Substitute, Percutaneous Approach */
"08UG3JZ"  =  "1"  /* Supplement Right Retinal Vessel with Synthetic Substitute, Percutaneous Approach */
"08UG3KZ"  =  "1"  /* Supplement Right Retinal Vessel with Nonautologous Tissue Substitute, Percutaneous Approach */
"08UH07Z"  =  "1"  /* Supplement Left Retinal Vessel with Autologous Tissue Substitute, Open Approach */
"08UH0JZ"  =  "1"  /* Supplement Left Retinal Vessel with Synthetic Substitute, Open Approach */
"08UH0KZ"  =  "1"  /* Supplement Left Retinal Vessel with Nonautologous Tissue Substitute, Open Approach */
"08UH37Z"  =  "1"  /* Supplement Left Retinal Vessel with Autologous Tissue Substitute, Percutaneous Approach */
"08UH3JZ"  =  "1"  /* Supplement Left Retinal Vessel with Synthetic Substitute, Percutaneous Approach */
"08UH3KZ"  =  "1"  /* Supplement Left Retinal Vessel with Nonautologous Tissue Substitute, Percutaneous Approach */
"095D0ZZ"  =  "1"  /* Destruction of Right Inner Ear, Open Approach */
"095E0ZZ"  =  "1"  /* Destruction of Left Inner Ear, Open Approach */
"095K0ZZ"  =  "1"  /* Destruction of Nose, Open Approach */
"095K3ZZ"  =  "1"  /* Destruction of Nose, Percutaneous Approach */
"095K4ZZ"  =  "1"  /* Destruction of Nose, Percutaneous Endoscopic Approach */
"099000Z"  =  "1"  /* Drainage of Right External Ear with Drainage Device, Open Approach */
"09900ZZ"  =  "1"  /* Drainage of Right External Ear, Open Approach */
"099030Z"  =  "1"  /* Drainage of Right External Ear with Drainage Device, Percutaneous Approach */
"09903ZZ"  =  "1"  /* Drainage of Right External Ear, Percutaneous Approach */
"099040Z"  =  "1"  /* Drainage of Right External Ear with Drainage Device, Percutaneous Endoscopic Approach */
"09904ZZ"  =  "1"  /* Drainage of Right External Ear, Percutaneous Endoscopic Approach */
"0990X0Z"  =  "1"  /* Drainage of Right External Ear with Drainage Device, External Approach */
"0990XZZ"  =  "1"  /* Drainage of Right External Ear, External Approach */
"099100Z"  =  "1"  /* Drainage of Left External Ear with Drainage Device, Open Approach */
"09910ZZ"  =  "1"  /* Drainage of Left External Ear, Open Approach */
"099130Z"  =  "1"  /* Drainage of Left External Ear with Drainage Device, Percutaneous Approach */
"09913ZZ"  =  "1"  /* Drainage of Left External Ear, Percutaneous Approach */
"099140Z"  =  "1"  /* Drainage of Left External Ear with Drainage Device, Percutaneous Endoscopic Approach */
"09914ZZ"  =  "1"  /* Drainage of Left External Ear, Percutaneous Endoscopic Approach */
"0991X0Z"  =  "1"  /* Drainage of Left External Ear with Drainage Device, External Approach */
"0991XZZ"  =  "1"  /* Drainage of Left External Ear, External Approach */
"099300Z"  =  "1"  /* Drainage of Right External Auditory Canal with Drainage Device, Open Approach */
"09930ZZ"  =  "1"  /* Drainage of Right External Auditory Canal, Open Approach */
"099330Z"  =  "1"  /* Drainage of Right External Auditory Canal with Drainage Device, Percutaneous Approach */
"09933ZZ"  =  "1"  /* Drainage of Right External Auditory Canal, Percutaneous Approach */
"099340Z"  =  "1"  /* Drainage of Right External Auditory Canal with Drainage Device, Percutaneous Endoscopic Approach */
"09934ZZ"  =  "1"  /* Drainage of Right External Auditory Canal, Percutaneous Endoscopic Approach */
"099370Z"  =  "1"  /* Drainage of Right External Auditory Canal with Drainage Device, Via Natural or Artificial Opening */
"09937ZZ"  =  "1"  /* Drainage of Right External Auditory Canal, Via Natural or Artificial Opening */
"099380Z"  =  "1"  /* Drainage of Right External Auditory Canal with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"09938ZZ"  =  "1"  /* Drainage of Right External Auditory Canal, Via Natural or Artificial Opening Endoscopic */
"0993X0Z"  =  "1"  /* Drainage of Right External Auditory Canal with Drainage Device, External Approach */
"0993XZZ"  =  "1"  /* Drainage of Right External Auditory Canal, External Approach */
"099400Z"  =  "1"  /* Drainage of Left External Auditory Canal with Drainage Device, Open Approach */
"09940ZZ"  =  "1"  /* Drainage of Left External Auditory Canal, Open Approach */
"099430Z"  =  "1"  /* Drainage of Left External Auditory Canal with Drainage Device, Percutaneous Approach */
"09943ZZ"  =  "1"  /* Drainage of Left External Auditory Canal, Percutaneous Approach */
"099440Z"  =  "1"  /* Drainage of Left External Auditory Canal with Drainage Device, Percutaneous Endoscopic Approach */
"09944ZZ"  =  "1"  /* Drainage of Left External Auditory Canal, Percutaneous Endoscopic Approach */
"099470Z"  =  "1"  /* Drainage of Left External Auditory Canal with Drainage Device, Via Natural or Artificial Opening */
"09947ZZ"  =  "1"  /* Drainage of Left External Auditory Canal, Via Natural or Artificial Opening */
"099480Z"  =  "1"  /* Drainage of Left External Auditory Canal with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"09948ZZ"  =  "1"  /* Drainage of Left External Auditory Canal, Via Natural or Artificial Opening Endoscopic */
"0994X0Z"  =  "1"  /* Drainage of Left External Auditory Canal with Drainage Device, External Approach */
"0994XZZ"  =  "1"  /* Drainage of Left External Auditory Canal, External Approach */
"099500Z"  =  "1"  /* Drainage of Right Middle Ear with Drainage Device, Open Approach */
"09950ZZ"  =  "1"  /* Drainage of Right Middle Ear, Open Approach */
"099600Z"  =  "1"  /* Drainage of Left Middle Ear with Drainage Device, Open Approach */
"09960ZZ"  =  "1"  /* Drainage of Left Middle Ear, Open Approach */
"099700Z"  =  "1"  /* Drainage of Right Tympanic Membrane with Drainage Device, Open Approach */
"09970ZZ"  =  "1"  /* Drainage of Right Tympanic Membrane, Open Approach */
"099730Z"  =  "1"  /* Drainage of Right Tympanic Membrane with Drainage Device, Percutaneous Approach */
"09973ZZ"  =  "1"  /* Drainage of Right Tympanic Membrane, Percutaneous Approach */
"099740Z"  =  "1"  /* Drainage of Right Tympanic Membrane with Drainage Device, Percutaneous Endoscopic Approach */
"09974ZZ"  =  "1"  /* Drainage of Right Tympanic Membrane, Percutaneous Endoscopic Approach */
"099770Z"  =  "1"  /* Drainage of Right Tympanic Membrane with Drainage Device, Via Natural or Artificial Opening */
"09977ZZ"  =  "1"  /* Drainage of Right Tympanic Membrane, Via Natural or Artificial Opening */
"099780Z"  =  "1"  /* Drainage of Right Tympanic Membrane with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"09978ZZ"  =  "1"  /* Drainage of Right Tympanic Membrane, Via Natural or Artificial Opening Endoscopic */
"099800Z"  =  "1"  /* Drainage of Left Tympanic Membrane with Drainage Device, Open Approach */
"09980ZZ"  =  "1"  /* Drainage of Left Tympanic Membrane, Open Approach */
"099830Z"  =  "1"  /* Drainage of Left Tympanic Membrane with Drainage Device, Percutaneous Approach */
"09983ZZ"  =  "1"  /* Drainage of Left Tympanic Membrane, Percutaneous Approach */
"099840Z"  =  "1"  /* Drainage of Left Tympanic Membrane with Drainage Device, Percutaneous Endoscopic Approach */
"09984ZZ"  =  "1"  /* Drainage of Left Tympanic Membrane, Percutaneous Endoscopic Approach */
"099870Z"  =  "1"  /* Drainage of Left Tympanic Membrane with Drainage Device, Via Natural or Artificial Opening */
"09987ZZ"  =  "1"  /* Drainage of Left Tympanic Membrane, Via Natural or Artificial Opening */
"099880Z"  =  "1"  /* Drainage of Left Tympanic Membrane with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"09988ZZ"  =  "1"  /* Drainage of Left Tympanic Membrane, Via Natural or Artificial Opening Endoscopic */
"099B00Z"  =  "1"  /* Drainage of Right Mastoid Sinus with Drainage Device, Open Approach */
"099B0ZZ"  =  "1"  /* Drainage of Right Mastoid Sinus, Open Approach */
"099B30Z"  =  "1"  /* Drainage of Right Mastoid Sinus with Drainage Device, Percutaneous Approach */
"099B3ZZ"  =  "1"  /* Drainage of Right Mastoid Sinus, Percutaneous Approach */
"099B40Z"  =  "1"  /* Drainage of Right Mastoid Sinus with Drainage Device, Percutaneous Endoscopic Approach */
"099B4ZZ"  =  "1"  /* Drainage of Right Mastoid Sinus, Percutaneous Endoscopic Approach */
"099C00Z"  =  "1"  /* Drainage of Left Mastoid Sinus with Drainage Device, Open Approach */
"099C0ZZ"  =  "1"  /* Drainage of Left Mastoid Sinus, Open Approach */
"099C30Z"  =  "1"  /* Drainage of Left Mastoid Sinus with Drainage Device, Percutaneous Approach */
"099C3ZZ"  =  "1"  /* Drainage of Left Mastoid Sinus, Percutaneous Approach */
"099C40Z"  =  "1"  /* Drainage of Left Mastoid Sinus with Drainage Device, Percutaneous Endoscopic Approach */
"099C4ZZ"  =  "1"  /* Drainage of Left Mastoid Sinus, Percutaneous Endoscopic Approach */
"099D00Z"  =  "1"  /* Drainage of Right Inner Ear with Drainage Device, Open Approach */
"099D0ZZ"  =  "1"  /* Drainage of Right Inner Ear, Open Approach */
"099E00Z"  =  "1"  /* Drainage of Left Inner Ear with Drainage Device, Open Approach */
"099E0ZZ"  =  "1"  /* Drainage of Left Inner Ear, Open Approach */
"099K00Z"  =  "1"  /* Drainage of Nose with Drainage Device, Open Approach */
"099K0ZZ"  =  "1"  /* Drainage of Nose, Open Approach */
"099K30Z"  =  "1"  /* Drainage of Nose with Drainage Device, Percutaneous Approach */
"099K3ZZ"  =  "1"  /* Drainage of Nose, Percutaneous Approach */
"099K40Z"  =  "1"  /* Drainage of Nose with Drainage Device, Percutaneous Endoscopic Approach */
"099K4ZZ"  =  "1"  /* Drainage of Nose, Percutaneous Endoscopic Approach */
"099KX0Z"  =  "1"  /* Drainage of Nose with Drainage Device, External Approach */
"099KXZZ"  =  "1"  /* Drainage of Nose, External Approach */
"099L00Z"  =  "1"  /* Drainage of Nasal Turbinate with Drainage Device, Open Approach */
"099L0ZZ"  =  "1"  /* Drainage of Nasal Turbinate, Open Approach */
"099L30Z"  =  "1"  /* Drainage of Nasal Turbinate with Drainage Device, Percutaneous Approach */
"099L3ZZ"  =  "1"  /* Drainage of Nasal Turbinate, Percutaneous Approach */
"099L40Z"  =  "1"  /* Drainage of Nasal Turbinate with Drainage Device, Percutaneous Endoscopic Approach */
"099L4ZZ"  =  "1"  /* Drainage of Nasal Turbinate, Percutaneous Endoscopic Approach */
"099L70Z"  =  "1"  /* Drainage of Nasal Turbinate with Drainage Device, Via Natural or Artificial Opening */
"099L7ZZ"  =  "1"  /* Drainage of Nasal Turbinate, Via Natural or Artificial Opening */
"099L80Z"  =  "1"  /* Drainage of Nasal Turbinate with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"099L8ZZ"  =  "1"  /* Drainage of Nasal Turbinate, Via Natural or Artificial Opening Endoscopic */
"099M00Z"  =  "1"  /* Drainage of Nasal Septum with Drainage Device, Open Approach */
"099M0ZZ"  =  "1"  /* Drainage of Nasal Septum, Open Approach */
"099M30Z"  =  "1"  /* Drainage of Nasal Septum with Drainage Device, Percutaneous Approach */
"099M3ZZ"  =  "1"  /* Drainage of Nasal Septum, Percutaneous Approach */
"099M40Z"  =  "1"  /* Drainage of Nasal Septum with Drainage Device, Percutaneous Endoscopic Approach */
"099M4ZZ"  =  "1"  /* Drainage of Nasal Septum, Percutaneous Endoscopic Approach */
"099Q00Z"  =  "1"  /* Drainage of Right Maxillary Sinus with Drainage Device, Open Approach */
"099Q0ZZ"  =  "1"  /* Drainage of Right Maxillary Sinus, Open Approach */
"099R00Z"  =  "1"  /* Drainage of Left Maxillary Sinus with Drainage Device, Open Approach */
"099R0ZZ"  =  "1"  /* Drainage of Left Maxillary Sinus, Open Approach */
"099S00Z"  =  "1"  /* Drainage of Right Frontal Sinus with Drainage Device, Open Approach */
"099S0ZZ"  =  "1"  /* Drainage of Right Frontal Sinus, Open Approach */
"099T00Z"  =  "1"  /* Drainage of Left Frontal Sinus with Drainage Device, Open Approach */
"099T0ZZ"  =  "1"  /* Drainage of Left Frontal Sinus, Open Approach */
"099U00Z"  =  "1"  /* Drainage of Right Ethmoid Sinus with Drainage Device, Open Approach */
"099U0ZZ"  =  "1"  /* Drainage of Right Ethmoid Sinus, Open Approach */
"099V00Z"  =  "1"  /* Drainage of Left Ethmoid Sinus with Drainage Device, Open Approach */
"099V0ZZ"  =  "1"  /* Drainage of Left Ethmoid Sinus, Open Approach */
"099W00Z"  =  "1"  /* Drainage of Right Sphenoid Sinus with Drainage Device, Open Approach */
"099W0ZZ"  =  "1"  /* Drainage of Right Sphenoid Sinus, Open Approach */
"099X00Z"  =  "1"  /* Drainage of Left Sphenoid Sinus with Drainage Device, Open Approach */
"099X0ZZ"  =  "1"  /* Drainage of Left Sphenoid Sinus, Open Approach */
"09BD0ZZ"  =  "1"  /* Excision of Right Inner Ear, Open Approach */
"09BE0ZZ"  =  "1"  /* Excision of Left Inner Ear, Open Approach */
"09C00ZZ"  =  "1"  /* Extirpation of Matter from Right External Ear, Open Approach */
"09C03ZZ"  =  "1"  /* Extirpation of Matter from Right External Ear, Percutaneous Approach */
"09C04ZZ"  =  "1"  /* Extirpation of Matter from Right External Ear, Percutaneous Endoscopic Approach */
"09C10ZZ"  =  "1"  /* Extirpation of Matter from Left External Ear, Open Approach */
"09C13ZZ"  =  "1"  /* Extirpation of Matter from Left External Ear, Percutaneous Approach */
"09C14ZZ"  =  "1"  /* Extirpation of Matter from Left External Ear, Percutaneous Endoscopic Approach */
"09C30ZZ"  =  "1"  /* Extirpation of Matter from Right External Auditory Canal, Open Approach */
"09C33ZZ"  =  "1"  /* Extirpation of Matter from Right External Auditory Canal, Percutaneous Approach */
"09C34ZZ"  =  "1"  /* Extirpation of Matter from Right External Auditory Canal, Percutaneous Endoscopic Approach */
"09C40ZZ"  =  "1"  /* Extirpation of Matter from Left External Auditory Canal, Open Approach */
"09C43ZZ"  =  "1"  /* Extirpation of Matter from Left External Auditory Canal, Percutaneous Approach */
"09C44ZZ"  =  "1"  /* Extirpation of Matter from Left External Auditory Canal, Percutaneous Endoscopic Approach */
"09C50ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Ear, Open Approach */
"09C60ZZ"  =  "1"  /* Extirpation of Matter from Left Middle Ear, Open Approach */
"09C70ZZ"  =  "1"  /* Extirpation of Matter from Right Tympanic Membrane, Open Approach */
"09C73ZZ"  =  "1"  /* Extirpation of Matter from Right Tympanic Membrane, Percutaneous Approach */
"09C74ZZ"  =  "1"  /* Extirpation of Matter from Right Tympanic Membrane, Percutaneous Endoscopic Approach */
"09C77ZZ"  =  "1"  /* Extirpation of Matter from Right Tympanic Membrane, Via Natural or Artificial Opening */
"09C78ZZ"  =  "1"  /* Extirpation of Matter from Right Tympanic Membrane, Via Natural or Artificial Opening Endoscopic */
"09C80ZZ"  =  "1"  /* Extirpation of Matter from Left Tympanic Membrane, Open Approach */
"09C83ZZ"  =  "1"  /* Extirpation of Matter from Left Tympanic Membrane, Percutaneous Approach */
"09C84ZZ"  =  "1"  /* Extirpation of Matter from Left Tympanic Membrane, Percutaneous Endoscopic Approach */
"09C87ZZ"  =  "1"  /* Extirpation of Matter from Left Tympanic Membrane, Via Natural or Artificial Opening */
"09C88ZZ"  =  "1"  /* Extirpation of Matter from Left Tympanic Membrane, Via Natural or Artificial Opening Endoscopic */
"09CB0ZZ"  =  "1"  /* Extirpation of Matter from Right Mastoid Sinus, Open Approach */
"09CB3ZZ"  =  "1"  /* Extirpation of Matter from Right Mastoid Sinus, Percutaneous Approach */
"09CB4ZZ"  =  "1"  /* Extirpation of Matter from Right Mastoid Sinus, Percutaneous Endoscopic Approach */
"09CC0ZZ"  =  "1"  /* Extirpation of Matter from Left Mastoid Sinus, Open Approach */
"09CC3ZZ"  =  "1"  /* Extirpation of Matter from Left Mastoid Sinus, Percutaneous Approach */
"09CC4ZZ"  =  "1"  /* Extirpation of Matter from Left Mastoid Sinus, Percutaneous Endoscopic Approach */
"09CD0ZZ"  =  "1"  /* Extirpation of Matter from Right Inner Ear, Open Approach */
"09CE0ZZ"  =  "1"  /* Extirpation of Matter from Left Inner Ear, Open Approach */
"09CK0ZZ"  =  "1"  /* Extirpation of Matter from Nose, Open Approach */
"09CK3ZZ"  =  "1"  /* Extirpation of Matter from Nose, Percutaneous Approach */
"09CK4ZZ"  =  "1"  /* Extirpation of Matter from Nose, Percutaneous Endoscopic Approach */
"09CL0ZZ"  =  "1"  /* Extirpation of Matter from Nasal Turbinate, Open Approach */
"09CL3ZZ"  =  "1"  /* Extirpation of Matter from Nasal Turbinate, Percutaneous Approach */
"09CL4ZZ"  =  "1"  /* Extirpation of Matter from Nasal Turbinate, Percutaneous Endoscopic Approach */
"09CL7ZZ"  =  "1"  /* Extirpation of Matter from Nasal Turbinate, Via Natural or Artificial Opening */
"09CL8ZZ"  =  "1"  /* Extirpation of Matter from Nasal Turbinate, Via Natural or Artificial Opening Endoscopic */
"09CM0ZZ"  =  "1"  /* Extirpation of Matter from Nasal Septum, Open Approach */
"09CM3ZZ"  =  "1"  /* Extirpation of Matter from Nasal Septum, Percutaneous Approach */
"09CM4ZZ"  =  "1"  /* Extirpation of Matter from Nasal Septum, Percutaneous Endoscopic Approach */
"09CQ0ZZ"  =  "1"  /* Extirpation of Matter from Right Maxillary Sinus, Open Approach */
"09CQ3ZZ"  =  "1"  /* Extirpation of Matter from Right Maxillary Sinus, Percutaneous Approach */
"09CQ4ZZ"  =  "1"  /* Extirpation of Matter from Right Maxillary Sinus, Percutaneous Endoscopic Approach */
"09CR0ZZ"  =  "1"  /* Extirpation of Matter from Left Maxillary Sinus, Open Approach */
"09CR3ZZ"  =  "1"  /* Extirpation of Matter from Left Maxillary Sinus, Percutaneous Approach */
"09CR4ZZ"  =  "1"  /* Extirpation of Matter from Left Maxillary Sinus, Percutaneous Endoscopic Approach */
"09CS0ZZ"  =  "1"  /* Extirpation of Matter from Right Frontal Sinus, Open Approach */
"09CS3ZZ"  =  "1"  /* Extirpation of Matter from Right Frontal Sinus, Percutaneous Approach */
"09CS4ZZ"  =  "1"  /* Extirpation of Matter from Right Frontal Sinus, Percutaneous Endoscopic Approach */
"09CT0ZZ"  =  "1"  /* Extirpation of Matter from Left Frontal Sinus, Open Approach */
"09CT3ZZ"  =  "1"  /* Extirpation of Matter from Left Frontal Sinus, Percutaneous Approach */
"09CT4ZZ"  =  "1"  /* Extirpation of Matter from Left Frontal Sinus, Percutaneous Endoscopic Approach */
"09CU0ZZ"  =  "1"  /* Extirpation of Matter from Right Ethmoid Sinus, Open Approach */
"09CU3ZZ"  =  "1"  /* Extirpation of Matter from Right Ethmoid Sinus, Percutaneous Approach */
"09CU4ZZ"  =  "1"  /* Extirpation of Matter from Right Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09CV0ZZ"  =  "1"  /* Extirpation of Matter from Left Ethmoid Sinus, Open Approach */
"09CV3ZZ"  =  "1"  /* Extirpation of Matter from Left Ethmoid Sinus, Percutaneous Approach */
"09CV4ZZ"  =  "1"  /* Extirpation of Matter from Left Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09CW0ZZ"  =  "1"  /* Extirpation of Matter from Right Sphenoid Sinus, Open Approach */
"09CW3ZZ"  =  "1"  /* Extirpation of Matter from Right Sphenoid Sinus, Percutaneous Approach */
"09CW4ZZ"  =  "1"  /* Extirpation of Matter from Right Sphenoid Sinus, Percutaneous Endoscopic Approach */
"09CX0ZZ"  =  "1"  /* Extirpation of Matter from Left Sphenoid Sinus, Open Approach */
"09CX3ZZ"  =  "1"  /* Extirpation of Matter from Left Sphenoid Sinus, Percutaneous Approach */
"09CX4ZZ"  =  "1"  /* Extirpation of Matter from Left Sphenoid Sinus, Percutaneous Endoscopic Approach */
"09J78ZZ"  =  "1"  /* Inspection of Right Tympanic Membrane, Via Natural or Artificial Opening Endoscopic */
"09J88ZZ"  =  "1"  /* Inspection of Left Tympanic Membrane, Via Natural or Artificial Opening Endoscopic */
"09JH8ZZ"  =  "1"  /* Inspection of Right Ear, Via Natural or Artificial Opening Endoscopic */
"09JJ8ZZ"  =  "1"  /* Inspection of Left Ear, Via Natural or Artificial Opening Endoscopic */
"09JKXZZ"  =  "1"  /* Inspection of Nose, External Approach */
"09JY0ZZ"  =  "1"  /* Inspection of Sinus, Open Approach */
"09JY3ZZ"  =  "1"  /* Inspection of Sinus, Percutaneous Approach */
"09JY4ZZ"  =  "1"  /* Inspection of Sinus, Percutaneous Endoscopic Approach */
"09N50ZZ"  =  "1"  /* Release Right Middle Ear, Open Approach */
"09N60ZZ"  =  "1"  /* Release Left Middle Ear, Open Approach */
"09PH3JZ"  =  "1"  /* Removal of Synthetic Substitute from Right Ear, Percutaneous Approach */
"09PH3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Right Ear, Percutaneous Approach */
"09PH4JZ"  =  "1"  /* Removal of Synthetic Substitute from Right Ear, Percutaneous Endoscopic Approach */
"09PH4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Right Ear, Percutaneous Endoscopic Approach */
"09PHX7Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Right Ear, External Approach */
"09PJ3JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Ear, Percutaneous Approach */
"09PJ3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Left Ear, Percutaneous Approach */
"09PJ4JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Ear, Percutaneous Endoscopic Approach */
"09PJ4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Left Ear, Percutaneous Endoscopic Approach */
"09PJX7Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Left Ear, External Approach */
"09PK00Z"  =  "1"  /* Removal of Drainage Device from Nose, Open Approach */
"09PK07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Nose, Open Approach */
"09PK0DZ"  =  "1"  /* Removal of Intraluminal Device from Nose, Open Approach */
"09PK0JZ"  =  "1"  /* Removal of Synthetic Substitute from Nose, Open Approach */
"09PK0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Nose, Open Approach */
"09PK30Z"  =  "1"  /* Removal of Drainage Device from Nose, Percutaneous Approach */
"09PK37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Nose, Percutaneous Approach */
"09PK3DZ"  =  "1"  /* Removal of Intraluminal Device from Nose, Percutaneous Approach */
"09PK3JZ"  =  "1"  /* Removal of Synthetic Substitute from Nose, Percutaneous Approach */
"09PK3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Nose, Percutaneous Approach */
"09PK40Z"  =  "1"  /* Removal of Drainage Device from Nose, Percutaneous Endoscopic Approach */
"09PK47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Nose, Percutaneous Endoscopic Approach */
"09PK4DZ"  =  "1"  /* Removal of Intraluminal Device from Nose, Percutaneous Endoscopic Approach */
"09PK4JZ"  =  "1"  /* Removal of Synthetic Substitute from Nose, Percutaneous Endoscopic Approach */
"09PK4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Nose, Percutaneous Endoscopic Approach */
"09PK70Z"  =  "1"  /* Removal of Drainage Device from Nose, Via Natural or Artificial Opening */
"09PK77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Nose, Via Natural or Artificial Opening */
"09PK7JZ"  =  "1"  /* Removal of Synthetic Substitute from Nose, Via Natural or Artificial Opening */
"09PK7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Nose, Via Natural or Artificial Opening */
"09PK80Z"  =  "1"  /* Removal of Drainage Device from Nose, Via Natural or Artificial Opening Endoscopic */
"09PK87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Nose, Via Natural or Artificial Opening Endoscopic */
"09PK8JZ"  =  "1"  /* Removal of Synthetic Substitute from Nose, Via Natural or Artificial Opening Endoscopic */
"09PK8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Nose, Via Natural or Artificial Opening Endoscopic */
"09PKX7Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Nose, External Approach */
"09QKXZZ"  =  "1"  /* Repair Nose, External Approach */
"09RK07Z"  =  "1"  /* Replacement of Nose with Autologous Tissue Substitute, Open Approach */
"09TD0ZZ"  =  "1"  /* Resection of Right Inner Ear, Open Approach */
"09TE0ZZ"  =  "1"  /* Resection of Left Inner Ear, Open Approach */
"09WH3JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Ear, Percutaneous Approach */
"09WH3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Right Ear, Percutaneous Approach */
"09WH4JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Ear, Percutaneous Endoscopic Approach */
"09WH4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Right Ear, Percutaneous Endoscopic Approach */
"09WJ3JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Ear, Percutaneous Approach */
"09WJ3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Left Ear, Percutaneous Approach */
"09WJ4JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Ear, Percutaneous Endoscopic Approach */
"09WJ4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Left Ear, Percutaneous Endoscopic Approach */
"09WK00Z"  =  "1"  /* Revision of Drainage Device in Nose, Open Approach */
"09WK07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Nose, Open Approach */
"09WK0DZ"  =  "1"  /* Revision of Intraluminal Device in Nose, Open Approach */
"09WK0JZ"  =  "1"  /* Revision of Synthetic Substitute in Nose, Open Approach */
"09WK0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Nose, Open Approach */
"09WK30Z"  =  "1"  /* Revision of Drainage Device in Nose, Percutaneous Approach */
"09WK37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Nose, Percutaneous Approach */
"09WK3DZ"  =  "1"  /* Revision of Intraluminal Device in Nose, Percutaneous Approach */
"09WK3JZ"  =  "1"  /* Revision of Synthetic Substitute in Nose, Percutaneous Approach */
"09WK3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Nose, Percutaneous Approach */
"09WK40Z"  =  "1"  /* Revision of Drainage Device in Nose, Percutaneous Endoscopic Approach */
"09WK47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Nose, Percutaneous Endoscopic Approach */
"09WK4DZ"  =  "1"  /* Revision of Intraluminal Device in Nose, Percutaneous Endoscopic Approach */
"09WK4JZ"  =  "1"  /* Revision of Synthetic Substitute in Nose, Percutaneous Endoscopic Approach */
"09WK4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Nose, Percutaneous Endoscopic Approach */
"09WK70Z"  =  "1"  /* Revision of Drainage Device in Nose, Via Natural or Artificial Opening */
"09WK77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Nose, Via Natural or Artificial Opening */
"09WK7DZ"  =  "1"  /* Revision of Intraluminal Device in Nose, Via Natural or Artificial Opening */
"09WK7JZ"  =  "1"  /* Revision of Synthetic Substitute in Nose, Via Natural or Artificial Opening */
"09WK7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Nose, Via Natural or Artificial Opening */
"09WK80Z"  =  "1"  /* Revision of Drainage Device in Nose, Via Natural or Artificial Opening Endoscopic */
"09WK87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Nose, Via Natural or Artificial Opening Endoscopic */
"09WK8DZ"  =  "1"  /* Revision of Intraluminal Device in Nose, Via Natural or Artificial Opening Endoscopic */
"09WK8JZ"  =  "1"  /* Revision of Synthetic Substitute in Nose, Via Natural or Artificial Opening Endoscopic */
"09WK8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Nose, Via Natural or Artificial Opening Endoscopic */
"0B9100Z"  =  "1"  /* Drainage of Trachea with Drainage Device, Open Approach */
"0B910ZZ"  =  "1"  /* Drainage of Trachea, Open Approach */
"0B9130Z"  =  "1"  /* Drainage of Trachea with Drainage Device, Percutaneous Approach */
"0B913ZZ"  =  "1"  /* Drainage of Trachea, Percutaneous Approach */
"0B9140Z"  =  "1"  /* Drainage of Trachea with Drainage Device, Percutaneous Endoscopic Approach */
"0B914ZZ"  =  "1"  /* Drainage of Trachea, Percutaneous Endoscopic Approach */
"0B9170Z"  =  "1"  /* Drainage of Trachea with Drainage Device, Via Natural or Artificial Opening */
"0B917ZZ"  =  "1"  /* Drainage of Trachea, Via Natural or Artificial Opening */
"0B9180Z"  =  "1"  /* Drainage of Trachea with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B918ZZ"  =  "1"  /* Drainage of Trachea, Via Natural or Artificial Opening Endoscopic */
"0B9200Z"  =  "1"  /* Drainage of Carina with Drainage Device, Open Approach */
"0B920ZZ"  =  "1"  /* Drainage of Carina, Open Approach */
"0B9230Z"  =  "1"  /* Drainage of Carina with Drainage Device, Percutaneous Approach */
"0B923ZZ"  =  "1"  /* Drainage of Carina, Percutaneous Approach */
"0B9240Z"  =  "1"  /* Drainage of Carina with Drainage Device, Percutaneous Endoscopic Approach */
"0B924ZZ"  =  "1"  /* Drainage of Carina, Percutaneous Endoscopic Approach */
"0B9270Z"  =  "1"  /* Drainage of Carina with Drainage Device, Via Natural or Artificial Opening */
"0B927ZZ"  =  "1"  /* Drainage of Carina, Via Natural or Artificial Opening */
"0B9280Z"  =  "1"  /* Drainage of Carina with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B928ZZ"  =  "1"  /* Drainage of Carina, Via Natural or Artificial Opening Endoscopic */
"0B9300Z"  =  "1"  /* Drainage of Right Main Bronchus with Drainage Device, Open Approach */
"0B930ZZ"  =  "1"  /* Drainage of Right Main Bronchus, Open Approach */
"0B9330Z"  =  "1"  /* Drainage of Right Main Bronchus with Drainage Device, Percutaneous Approach */
"0B933ZX"  =  "1"  /* Drainage of Right Main Bronchus, Percutaneous Approach, Diagnostic */
"0B933ZZ"  =  "1"  /* Drainage of Right Main Bronchus, Percutaneous Approach */
"0B9340Z"  =  "1"  /* Drainage of Right Main Bronchus with Drainage Device, Percutaneous Endoscopic Approach */
"0B934ZX"  =  "1"  /* Drainage of Right Main Bronchus, Percutaneous Endoscopic Approach, Diagnostic */
"0B934ZZ"  =  "1"  /* Drainage of Right Main Bronchus, Percutaneous Endoscopic Approach */
"0B9370Z"  =  "1"  /* Drainage of Right Main Bronchus with Drainage Device, Via Natural or Artificial Opening */
"0B937ZX"  =  "1"  /* Drainage of Right Main Bronchus, Via Natural or Artificial Opening, Diagnostic */
"0B937ZZ"  =  "1"  /* Drainage of Right Main Bronchus, Via Natural or Artificial Opening */
"0B9380Z"  =  "1"  /* Drainage of Right Main Bronchus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B938ZX"  =  "1"  /* Drainage of Right Main Bronchus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B938ZZ"  =  "1"  /* Drainage of Right Main Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B9400Z"  =  "1"  /* Drainage of Right Upper Lobe Bronchus with Drainage Device, Open Approach */
"0B940ZZ"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Open Approach */
"0B9430Z"  =  "1"  /* Drainage of Right Upper Lobe Bronchus with Drainage Device, Percutaneous Approach */
"0B943ZX"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Percutaneous Approach, Diagnostic */
"0B943ZZ"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Percutaneous Approach */
"0B9440Z"  =  "1"  /* Drainage of Right Upper Lobe Bronchus with Drainage Device, Percutaneous Endoscopic Approach */
"0B944ZX"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Percutaneous Endoscopic Approach, Diagnostic */
"0B944ZZ"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0B9470Z"  =  "1"  /* Drainage of Right Upper Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening */
"0B947ZX"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Via Natural or Artificial Opening, Diagnostic */
"0B947ZZ"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0B9480Z"  =  "1"  /* Drainage of Right Upper Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B948ZX"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B948ZZ"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B9500Z"  =  "1"  /* Drainage of Right Middle Lobe Bronchus with Drainage Device, Open Approach */
"0B950ZZ"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Open Approach */
"0B9530Z"  =  "1"  /* Drainage of Right Middle Lobe Bronchus with Drainage Device, Percutaneous Approach */
"0B953ZX"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Percutaneous Approach, Diagnostic */
"0B953ZZ"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Percutaneous Approach */
"0B9540Z"  =  "1"  /* Drainage of Right Middle Lobe Bronchus with Drainage Device, Percutaneous Endoscopic Approach */
"0B954ZX"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Percutaneous Endoscopic Approach, Diagnostic */
"0B954ZZ"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Percutaneous Endoscopic Approach */
"0B9570Z"  =  "1"  /* Drainage of Right Middle Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening */
"0B957ZX"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Via Natural or Artificial Opening, Diagnostic */
"0B957ZZ"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Via Natural or Artificial Opening */
"0B9580Z"  =  "1"  /* Drainage of Right Middle Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B958ZX"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B958ZZ"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B9600Z"  =  "1"  /* Drainage of Right Lower Lobe Bronchus with Drainage Device, Open Approach */
"0B960ZZ"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Open Approach */
"0B9630Z"  =  "1"  /* Drainage of Right Lower Lobe Bronchus with Drainage Device, Percutaneous Approach */
"0B963ZX"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Percutaneous Approach, Diagnostic */
"0B963ZZ"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Percutaneous Approach */
"0B9640Z"  =  "1"  /* Drainage of Right Lower Lobe Bronchus with Drainage Device, Percutaneous Endoscopic Approach */
"0B964ZX"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Percutaneous Endoscopic Approach, Diagnostic */
"0B964ZZ"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0B9670Z"  =  "1"  /* Drainage of Right Lower Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening */
"0B967ZX"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Via Natural or Artificial Opening, Diagnostic */
"0B967ZZ"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0B9680Z"  =  "1"  /* Drainage of Right Lower Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B968ZX"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B968ZZ"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B9700Z"  =  "1"  /* Drainage of Left Main Bronchus with Drainage Device, Open Approach */
"0B970ZZ"  =  "1"  /* Drainage of Left Main Bronchus, Open Approach */
"0B9730Z"  =  "1"  /* Drainage of Left Main Bronchus with Drainage Device, Percutaneous Approach */
"0B973ZX"  =  "1"  /* Drainage of Left Main Bronchus, Percutaneous Approach, Diagnostic */
"0B973ZZ"  =  "1"  /* Drainage of Left Main Bronchus, Percutaneous Approach */
"0B9740Z"  =  "1"  /* Drainage of Left Main Bronchus with Drainage Device, Percutaneous Endoscopic Approach */
"0B974ZX"  =  "1"  /* Drainage of Left Main Bronchus, Percutaneous Endoscopic Approach, Diagnostic */
"0B974ZZ"  =  "1"  /* Drainage of Left Main Bronchus, Percutaneous Endoscopic Approach */
"0B9770Z"  =  "1"  /* Drainage of Left Main Bronchus with Drainage Device, Via Natural or Artificial Opening */
"0B977ZX"  =  "1"  /* Drainage of Left Main Bronchus, Via Natural or Artificial Opening, Diagnostic */
"0B977ZZ"  =  "1"  /* Drainage of Left Main Bronchus, Via Natural or Artificial Opening */
"0B9780Z"  =  "1"  /* Drainage of Left Main Bronchus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B978ZX"  =  "1"  /* Drainage of Left Main Bronchus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B978ZZ"  =  "1"  /* Drainage of Left Main Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B9800Z"  =  "1"  /* Drainage of Left Upper Lobe Bronchus with Drainage Device, Open Approach */
"0B980ZZ"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Open Approach */
"0B9830Z"  =  "1"  /* Drainage of Left Upper Lobe Bronchus with Drainage Device, Percutaneous Approach */
"0B983ZX"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Percutaneous Approach, Diagnostic */
"0B983ZZ"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Percutaneous Approach */
"0B9840Z"  =  "1"  /* Drainage of Left Upper Lobe Bronchus with Drainage Device, Percutaneous Endoscopic Approach */
"0B984ZX"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Percutaneous Endoscopic Approach, Diagnostic */
"0B984ZZ"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0B9870Z"  =  "1"  /* Drainage of Left Upper Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening */
"0B987ZX"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Via Natural or Artificial Opening, Diagnostic */
"0B987ZZ"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0B9880Z"  =  "1"  /* Drainage of Left Upper Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B988ZX"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B988ZZ"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B9900Z"  =  "1"  /* Drainage of Lingula Bronchus with Drainage Device, Open Approach */
"0B990ZZ"  =  "1"  /* Drainage of Lingula Bronchus, Open Approach */
"0B9930Z"  =  "1"  /* Drainage of Lingula Bronchus with Drainage Device, Percutaneous Approach */
"0B993ZX"  =  "1"  /* Drainage of Lingula Bronchus, Percutaneous Approach, Diagnostic */
"0B993ZZ"  =  "1"  /* Drainage of Lingula Bronchus, Percutaneous Approach */
"0B9940Z"  =  "1"  /* Drainage of Lingula Bronchus with Drainage Device, Percutaneous Endoscopic Approach */
"0B994ZX"  =  "1"  /* Drainage of Lingula Bronchus, Percutaneous Endoscopic Approach, Diagnostic */
"0B994ZZ"  =  "1"  /* Drainage of Lingula Bronchus, Percutaneous Endoscopic Approach */
"0B9970Z"  =  "1"  /* Drainage of Lingula Bronchus with Drainage Device, Via Natural or Artificial Opening */
"0B997ZX"  =  "1"  /* Drainage of Lingula Bronchus, Via Natural or Artificial Opening, Diagnostic */
"0B997ZZ"  =  "1"  /* Drainage of Lingula Bronchus, Via Natural or Artificial Opening */
"0B9980Z"  =  "1"  /* Drainage of Lingula Bronchus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B998ZX"  =  "1"  /* Drainage of Lingula Bronchus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B998ZZ"  =  "1"  /* Drainage of Lingula Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B9B00Z"  =  "1"  /* Drainage of Left Lower Lobe Bronchus with Drainage Device, Open Approach */
"0B9B0ZZ"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Open Approach */
"0B9B30Z"  =  "1"  /* Drainage of Left Lower Lobe Bronchus with Drainage Device, Percutaneous Approach */
"0B9B3ZX"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Percutaneous Approach, Diagnostic */
"0B9B3ZZ"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Percutaneous Approach */
"0B9B40Z"  =  "1"  /* Drainage of Left Lower Lobe Bronchus with Drainage Device, Percutaneous Endoscopic Approach */
"0B9B4ZX"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Percutaneous Endoscopic Approach, Diagnostic */
"0B9B4ZZ"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0B9B70Z"  =  "1"  /* Drainage of Left Lower Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening */
"0B9B7ZX"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Via Natural or Artificial Opening, Diagnostic */
"0B9B7ZZ"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0B9B80Z"  =  "1"  /* Drainage of Left Lower Lobe Bronchus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B9B8ZX"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0B9B8ZZ"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic */
"0B9C00Z"  =  "1"  /* Drainage of Right Upper Lung Lobe with Drainage Device, Open Approach */
"0B9C0ZZ"  =  "1"  /* Drainage of Right Upper Lung Lobe, Open Approach */
"0B9D00Z"  =  "1"  /* Drainage of Right Middle Lung Lobe with Drainage Device, Open Approach */
"0B9D0ZZ"  =  "1"  /* Drainage of Right Middle Lung Lobe, Open Approach */
"0B9F00Z"  =  "1"  /* Drainage of Right Lower Lung Lobe with Drainage Device, Open Approach */
"0B9F0ZZ"  =  "1"  /* Drainage of Right Lower Lung Lobe, Open Approach */
"0B9G00Z"  =  "1"  /* Drainage of Left Upper Lung Lobe with Drainage Device, Open Approach */
"0B9G0ZZ"  =  "1"  /* Drainage of Left Upper Lung Lobe, Open Approach */
"0B9H00Z"  =  "1"  /* Drainage of Lung Lingula with Drainage Device, Open Approach */
"0B9H0ZZ"  =  "1"  /* Drainage of Lung Lingula, Open Approach */
"0B9J00Z"  =  "1"  /* Drainage of Left Lower Lung Lobe with Drainage Device, Open Approach */
"0B9J0ZZ"  =  "1"  /* Drainage of Left Lower Lung Lobe, Open Approach */
"0B9K00Z"  =  "1"  /* Drainage of Right Lung with Drainage Device, Open Approach */
"0B9K0ZZ"  =  "1"  /* Drainage of Right Lung, Open Approach */
"0B9L00Z"  =  "1"  /* Drainage of Left Lung with Drainage Device, Open Approach */
"0B9L0ZZ"  =  "1"  /* Drainage of Left Lung, Open Approach */
"0B9M00Z"  =  "1"  /* Drainage of Bilateral Lungs with Drainage Device, Open Approach */
"0B9M0ZZ"  =  "1"  /* Drainage of Bilateral Lungs, Open Approach */
"0B9N00Z"  =  "1"  /* Drainage of Right Pleura with Drainage Device, Open Approach */
"0B9N0ZZ"  =  "1"  /* Drainage of Right Pleura, Open Approach */
"0B9N30Z"  =  "1"  /* Drainage of Right Pleura with Drainage Device, Percutaneous Approach */
"0B9P00Z"  =  "1"  /* Drainage of Left Pleura with Drainage Device, Open Approach */
"0B9P0ZZ"  =  "1"  /* Drainage of Left Pleura, Open Approach */
"0B9P30Z"  =  "1"  /* Drainage of Left Pleura with Drainage Device, Percutaneous Approach */
"0BB33ZX"  =  "1"  /* Excision of Right Main Bronchus, Percutaneous Approach, Diagnostic */
"0BB34ZX"  =  "1"  /* Excision of Right Main Bronchus, Percutaneous Endoscopic Approach, Diagnostic */
"0BB37ZX"  =  "1"  /* Excision of Right Main Bronchus, Via Natural or Artificial Opening, Diagnostic */
"0BB38ZX"  =  "1"  /* Excision of Right Main Bronchus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BB43ZX"  =  "1"  /* Excision of Right Upper Lobe Bronchus, Percutaneous Approach, Diagnostic */
"0BB44ZX"  =  "1"  /* Excision of Right Upper Lobe Bronchus, Percutaneous Endoscopic Approach, Diagnostic */
"0BB47ZX"  =  "1"  /* Excision of Right Upper Lobe Bronchus, Via Natural or Artificial Opening, Diagnostic */
"0BB48ZX"  =  "1"  /* Excision of Right Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BB53ZX"  =  "1"  /* Excision of Right Middle Lobe Bronchus, Percutaneous Approach, Diagnostic */
"0BB54ZX"  =  "1"  /* Excision of Right Middle Lobe Bronchus, Percutaneous Endoscopic Approach, Diagnostic */
"0BB57ZX"  =  "1"  /* Excision of Right Middle Lobe Bronchus, Via Natural or Artificial Opening, Diagnostic */
"0BB58ZX"  =  "1"  /* Excision of Right Middle Lobe Bronchus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BB63ZX"  =  "1"  /* Excision of Right Lower Lobe Bronchus, Percutaneous Approach, Diagnostic */
"0BB64ZX"  =  "1"  /* Excision of Right Lower Lobe Bronchus, Percutaneous Endoscopic Approach, Diagnostic */
"0BB67ZX"  =  "1"  /* Excision of Right Lower Lobe Bronchus, Via Natural or Artificial Opening, Diagnostic */
"0BB68ZX"  =  "1"  /* Excision of Right Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BB73ZX"  =  "1"  /* Excision of Left Main Bronchus, Percutaneous Approach, Diagnostic */
"0BB74ZX"  =  "1"  /* Excision of Left Main Bronchus, Percutaneous Endoscopic Approach, Diagnostic */
"0BB77ZX"  =  "1"  /* Excision of Left Main Bronchus, Via Natural or Artificial Opening, Diagnostic */
"0BB78ZX"  =  "1"  /* Excision of Left Main Bronchus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BB83ZX"  =  "1"  /* Excision of Left Upper Lobe Bronchus, Percutaneous Approach, Diagnostic */
"0BB84ZX"  =  "1"  /* Excision of Left Upper Lobe Bronchus, Percutaneous Endoscopic Approach, Diagnostic */
"0BB87ZX"  =  "1"  /* Excision of Left Upper Lobe Bronchus, Via Natural or Artificial Opening, Diagnostic */
"0BB88ZX"  =  "1"  /* Excision of Left Upper Lobe Bronchus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BB93ZX"  =  "1"  /* Excision of Lingula Bronchus, Percutaneous Approach, Diagnostic */
"0BB94ZX"  =  "1"  /* Excision of Lingula Bronchus, Percutaneous Endoscopic Approach, Diagnostic */
"0BB97ZX"  =  "1"  /* Excision of Lingula Bronchus, Via Natural or Artificial Opening, Diagnostic */
"0BB98ZX"  =  "1"  /* Excision of Lingula Bronchus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BBB3ZX"  =  "1"  /* Excision of Left Lower Lobe Bronchus, Percutaneous Approach, Diagnostic */
"0BBB4ZX"  =  "1"  /* Excision of Left Lower Lobe Bronchus, Percutaneous Endoscopic Approach, Diagnostic */
"0BBB7ZX"  =  "1"  /* Excision of Left Lower Lobe Bronchus, Via Natural or Artificial Opening, Diagnostic */
"0BBB8ZX"  =  "1"  /* Excision of Left Lower Lobe Bronchus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0BC10ZZ"  =  "1"  /* Extirpation of Matter from Trachea, Open Approach */
"0BC13ZZ"  =  "1"  /* Extirpation of Matter from Trachea, Percutaneous Approach */
"0BC14ZZ"  =  "1"  /* Extirpation of Matter from Trachea, Percutaneous Endoscopic Approach */
"0BC20ZZ"  =  "1"  /* Extirpation of Matter from Carina, Open Approach */
"0BC23ZZ"  =  "1"  /* Extirpation of Matter from Carina, Percutaneous Approach */
"0BC24ZZ"  =  "1"  /* Extirpation of Matter from Carina, Percutaneous Endoscopic Approach */
"0BC30ZZ"  =  "1"  /* Extirpation of Matter from Right Main Bronchus, Open Approach */
"0BC33ZZ"  =  "1"  /* Extirpation of Matter from Right Main Bronchus, Percutaneous Approach */
"0BC34ZZ"  =  "1"  /* Extirpation of Matter from Right Main Bronchus, Percutaneous Endoscopic Approach */
"0BC40ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lobe Bronchus, Open Approach */
"0BC43ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lobe Bronchus, Percutaneous Approach */
"0BC44ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BC50ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lobe Bronchus, Open Approach */
"0BC53ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lobe Bronchus, Percutaneous Approach */
"0BC54ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BC60ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lobe Bronchus, Open Approach */
"0BC63ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lobe Bronchus, Percutaneous Approach */
"0BC64ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BC70ZZ"  =  "1"  /* Extirpation of Matter from Left Main Bronchus, Open Approach */
"0BC73ZZ"  =  "1"  /* Extirpation of Matter from Left Main Bronchus, Percutaneous Approach */
"0BC74ZZ"  =  "1"  /* Extirpation of Matter from Left Main Bronchus, Percutaneous Endoscopic Approach */
"0BC80ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lobe Bronchus, Open Approach */
"0BC83ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lobe Bronchus, Percutaneous Approach */
"0BC84ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BC90ZZ"  =  "1"  /* Extirpation of Matter from Lingula Bronchus, Open Approach */
"0BC93ZZ"  =  "1"  /* Extirpation of Matter from Lingula Bronchus, Percutaneous Approach */
"0BC94ZZ"  =  "1"  /* Extirpation of Matter from Lingula Bronchus, Percutaneous Endoscopic Approach */
"0BCB0ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lobe Bronchus, Open Approach */
"0BCB3ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lobe Bronchus, Percutaneous Approach */
"0BCB4ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BCC0ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lung Lobe, Open Approach */
"0BCC3ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lung Lobe, Percutaneous Approach */
"0BCC4ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0BCC7ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lung Lobe, Via Natural or Artificial Opening */
"0BCC8ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BCD0ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lung Lobe, Open Approach */
"0BCD3ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lung Lobe, Percutaneous Approach */
"0BCD4ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lung Lobe, Percutaneous Endoscopic Approach */
"0BCD7ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lung Lobe, Via Natural or Artificial Opening */
"0BCD8ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BCF0ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lung Lobe, Open Approach */
"0BCF3ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lung Lobe, Percutaneous Approach */
"0BCF4ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0BCF7ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lung Lobe, Via Natural or Artificial Opening */
"0BCF8ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BCG0ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lung Lobe, Open Approach */
"0BCG3ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lung Lobe, Percutaneous Approach */
"0BCG4ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lung Lobe, Percutaneous Endoscopic Approach */
"0BCG7ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lung Lobe, Via Natural or Artificial Opening */
"0BCG8ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BCH0ZZ"  =  "1"  /* Extirpation of Matter from Lung Lingula, Open Approach */
"0BCH3ZZ"  =  "1"  /* Extirpation of Matter from Lung Lingula, Percutaneous Approach */
"0BCH4ZZ"  =  "1"  /* Extirpation of Matter from Lung Lingula, Percutaneous Endoscopic Approach */
"0BCH7ZZ"  =  "1"  /* Extirpation of Matter from Lung Lingula, Via Natural or Artificial Opening */
"0BCH8ZZ"  =  "1"  /* Extirpation of Matter from Lung Lingula, Via Natural or Artificial Opening Endoscopic */
"0BCJ0ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lung Lobe, Open Approach */
"0BCJ3ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lung Lobe, Percutaneous Approach */
"0BCJ4ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lung Lobe, Percutaneous Endoscopic Approach */
"0BCJ7ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lung Lobe, Via Natural or Artificial Opening */
"0BCJ8ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lung Lobe, Via Natural or Artificial Opening Endoscopic */
"0BCK0ZZ"  =  "1"  /* Extirpation of Matter from Right Lung, Open Approach */
"0BCK3ZZ"  =  "1"  /* Extirpation of Matter from Right Lung, Percutaneous Approach */
"0BCK4ZZ"  =  "1"  /* Extirpation of Matter from Right Lung, Percutaneous Endoscopic Approach */
"0BCK7ZZ"  =  "1"  /* Extirpation of Matter from Right Lung, Via Natural or Artificial Opening */
"0BCK8ZZ"  =  "1"  /* Extirpation of Matter from Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BCL0ZZ"  =  "1"  /* Extirpation of Matter from Left Lung, Open Approach */
"0BCL3ZZ"  =  "1"  /* Extirpation of Matter from Left Lung, Percutaneous Approach */
"0BCL4ZZ"  =  "1"  /* Extirpation of Matter from Left Lung, Percutaneous Endoscopic Approach */
"0BCL7ZZ"  =  "1"  /* Extirpation of Matter from Left Lung, Via Natural or Artificial Opening */
"0BCL8ZZ"  =  "1"  /* Extirpation of Matter from Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BCM0ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Lungs, Open Approach */
"0BCM3ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Lungs, Percutaneous Approach */
"0BCM4ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Lungs, Percutaneous Endoscopic Approach */
"0BCM7ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Lungs, Via Natural or Artificial Opening */
"0BCM8ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Lungs, Via Natural or Artificial Opening Endoscopic */
"0BCN0ZZ"  =  "1"  /* Extirpation of Matter from Right Pleura, Open Approach */
"0BCN3ZZ"  =  "1"  /* Extirpation of Matter from Right Pleura, Percutaneous Approach */
"0BCN4ZZ"  =  "1"  /* Extirpation of Matter from Right Pleura, Percutaneous Endoscopic Approach */
"0BCP0ZZ"  =  "1"  /* Extirpation of Matter from Left Pleura, Open Approach */
"0BCP3ZZ"  =  "1"  /* Extirpation of Matter from Left Pleura, Percutaneous Approach */
"0BCP4ZZ"  =  "1"  /* Extirpation of Matter from Left Pleura, Percutaneous Endoscopic Approach */
"0BH002Z"  =  "1"  /* Insertion of Monitoring Device into Tracheobronchial Tree, Open Approach */
"0BH003Z"  =  "1"  /* Insertion of Infusion Device into Tracheobronchial Tree, Open Approach */
"0BH00DZ"  =  "1"  /* Insertion of Intraluminal Device into Tracheobronchial Tree, Open Approach */
"0BH032Z"  =  "1"  /* Insertion of Monitoring Device into Tracheobronchial Tree, Percutaneous Approach */
"0BH033Z"  =  "1"  /* Insertion of Infusion Device into Tracheobronchial Tree, Percutaneous Approach */
"0BH03DZ"  =  "1"  /* Insertion of Intraluminal Device into Tracheobronchial Tree, Percutaneous Approach */
"0BH042Z"  =  "1"  /* Insertion of Monitoring Device into Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BH043Z"  =  "1"  /* Insertion of Infusion Device into Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BH04DZ"  =  "1"  /* Insertion of Intraluminal Device into Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BH102Z"  =  "1"  /* Insertion of Monitoring Device into Trachea, Open Approach */
"0BH10DZ"  =  "1"  /* Insertion of Intraluminal Device into Trachea, Open Approach */
"0BH13DZ"  =  "1"  /* Insertion of Intraluminal Device into Trachea, Percutaneous Approach */
"0BH14DZ"  =  "1"  /* Insertion of Intraluminal Device into Trachea, Percutaneous Endoscopic Approach */
"0BH172Z"  =  "1"  /* Insertion of Monitoring Device into Trachea, Via Natural or Artificial Opening */
"0BH17DZ"  =  "1"  /* Insertion of Intraluminal Device into Trachea, Via Natural or Artificial Opening */
"0BH182Z"  =  "1"  /* Insertion of Monitoring Device into Trachea, Via Natural or Artificial Opening Endoscopic */
"0BH18DZ"  =  "1"  /* Insertion of Intraluminal Device into Trachea, Via Natural or Artificial Opening Endoscopic */
"0BH30GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Main Bronchus, Open Approach */
"0BH33GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Main Bronchus, Percutaneous Approach */
"0BH34GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Main Bronchus, Percutaneous Endoscopic Approach */
"0BH37GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Main Bronchus, Via Natural or Artificial Opening */
"0BH40GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Upper Lobe Bronchus, Open Approach */
"0BH43GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Upper Lobe Bronchus, Percutaneous Approach */
"0BH44GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BH47GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0BH50GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Middle Lobe Bronchus, Open Approach */
"0BH53GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Middle Lobe Bronchus, Percutaneous Approach */
"0BH54GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Middle Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BH57GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Middle Lobe Bronchus, Via Natural or Artificial Opening */
"0BH60GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Lower Lobe Bronchus, Open Approach */
"0BH63GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Lower Lobe Bronchus, Percutaneous Approach */
"0BH64GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BH67GZ"  =  "1"  /* Insertion of Endobronchial Valve into Right Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0BH70GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Main Bronchus, Open Approach */
"0BH73GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Main Bronchus, Percutaneous Approach */
"0BH74GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Main Bronchus, Percutaneous Endoscopic Approach */
"0BH77GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Main Bronchus, Via Natural or Artificial Opening */
"0BH80GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Upper Lobe Bronchus, Open Approach */
"0BH83GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Upper Lobe Bronchus, Percutaneous Approach */
"0BH84GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Upper Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BH87GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Upper Lobe Bronchus, Via Natural or Artificial Opening */
"0BH90GZ"  =  "1"  /* Insertion of Endobronchial Valve into Lingula Bronchus, Open Approach */
"0BH93GZ"  =  "1"  /* Insertion of Endobronchial Valve into Lingula Bronchus, Percutaneous Approach */
"0BH94GZ"  =  "1"  /* Insertion of Endobronchial Valve into Lingula Bronchus, Percutaneous Endoscopic Approach */
"0BH97GZ"  =  "1"  /* Insertion of Endobronchial Valve into Lingula Bronchus, Via Natural or Artificial Opening */
"0BHB0GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Lower Lobe Bronchus, Open Approach */
"0BHB3GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Lower Lobe Bronchus, Percutaneous Approach */
"0BHB4GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Lower Lobe Bronchus, Percutaneous Endoscopic Approach */
"0BHB7GZ"  =  "1"  /* Insertion of Endobronchial Valve into Left Lower Lobe Bronchus, Via Natural or Artificial Opening */
"0BHK02Z"  =  "1"  /* Insertion of Monitoring Device into Right Lung, Open Approach */
"0BHK03Z"  =  "1"  /* Insertion of Infusion Device into Right Lung, Open Approach */
"0BHK32Z"  =  "1"  /* Insertion of Monitoring Device into Right Lung, Percutaneous Approach */
"0BHK33Z"  =  "1"  /* Insertion of Infusion Device into Right Lung, Percutaneous Approach */
"0BHK42Z"  =  "1"  /* Insertion of Monitoring Device into Right Lung, Percutaneous Endoscopic Approach */
"0BHK43Z"  =  "1"  /* Insertion of Infusion Device into Right Lung, Percutaneous Endoscopic Approach */
"0BHK72Z"  =  "1"  /* Insertion of Monitoring Device into Right Lung, Via Natural or Artificial Opening */
"0BHK73Z"  =  "1"  /* Insertion of Infusion Device into Right Lung, Via Natural or Artificial Opening */
"0BHK82Z"  =  "1"  /* Insertion of Monitoring Device into Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BHK83Z"  =  "1"  /* Insertion of Infusion Device into Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BHL02Z"  =  "1"  /* Insertion of Monitoring Device into Left Lung, Open Approach */
"0BHL03Z"  =  "1"  /* Insertion of Infusion Device into Left Lung, Open Approach */
"0BHL32Z"  =  "1"  /* Insertion of Monitoring Device into Left Lung, Percutaneous Approach */
"0BHL33Z"  =  "1"  /* Insertion of Infusion Device into Left Lung, Percutaneous Approach */
"0BHL42Z"  =  "1"  /* Insertion of Monitoring Device into Left Lung, Percutaneous Endoscopic Approach */
"0BHL43Z"  =  "1"  /* Insertion of Infusion Device into Left Lung, Percutaneous Endoscopic Approach */
"0BHL72Z"  =  "1"  /* Insertion of Monitoring Device into Left Lung, Via Natural or Artificial Opening */
"0BHL73Z"  =  "1"  /* Insertion of Infusion Device into Left Lung, Via Natural or Artificial Opening */
"0BHL82Z"  =  "1"  /* Insertion of Monitoring Device into Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BHL83Z"  =  "1"  /* Insertion of Infusion Device into Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BJ04ZZ"  =  "1"  /* Inspection of Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BJ08ZZ"  =  "1"  /* Inspection of Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BJ14ZZ"  =  "1"  /* Inspection of Trachea, Percutaneous Endoscopic Approach */
"0BJ18ZZ"  =  "1"  /* Inspection of Trachea, Via Natural or Artificial Opening Endoscopic */
"0BJK4ZZ"  =  "1"  /* Inspection of Right Lung, Percutaneous Endoscopic Approach */
"0BJK8ZZ"  =  "1"  /* Inspection of Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BJL4ZZ"  =  "1"  /* Inspection of Left Lung, Percutaneous Endoscopic Approach */
"0BJL8ZZ"  =  "1"  /* Inspection of Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BP000Z"  =  "1"  /* Removal of Drainage Device from Tracheobronchial Tree, Open Approach */
"0BP001Z"  =  "1"  /* Removal of Radioactive Element from Tracheobronchial Tree, Open Approach */
"0BP002Z"  =  "1"  /* Removal of Monitoring Device from Tracheobronchial Tree, Open Approach */
"0BP003Z"  =  "1"  /* Removal of Infusion Device from Tracheobronchial Tree, Open Approach */
"0BP007Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Tracheobronchial Tree, Open Approach */
"0BP00CZ"  =  "1"  /* Removal of Extraluminal Device from Tracheobronchial Tree, Open Approach */
"0BP00DZ"  =  "1"  /* Removal of Intraluminal Device from Tracheobronchial Tree, Open Approach */
"0BP00JZ"  =  "1"  /* Removal of Synthetic Substitute from Tracheobronchial Tree, Open Approach */
"0BP00KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Tracheobronchial Tree, Open Approach */
"0BP030Z"  =  "1"  /* Removal of Drainage Device from Tracheobronchial Tree, Percutaneous Approach */
"0BP031Z"  =  "1"  /* Removal of Radioactive Element from Tracheobronchial Tree, Percutaneous Approach */
"0BP032Z"  =  "1"  /* Removal of Monitoring Device from Tracheobronchial Tree, Percutaneous Approach */
"0BP033Z"  =  "1"  /* Removal of Infusion Device from Tracheobronchial Tree, Percutaneous Approach */
"0BP037Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Tracheobronchial Tree, Percutaneous Approach */
"0BP03CZ"  =  "1"  /* Removal of Extraluminal Device from Tracheobronchial Tree, Percutaneous Approach */
"0BP03DZ"  =  "1"  /* Removal of Intraluminal Device from Tracheobronchial Tree, Percutaneous Approach */
"0BP03JZ"  =  "1"  /* Removal of Synthetic Substitute from Tracheobronchial Tree, Percutaneous Approach */
"0BP03KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Tracheobronchial Tree, Percutaneous Approach */
"0BP040Z"  =  "1"  /* Removal of Drainage Device from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP041Z"  =  "1"  /* Removal of Radioactive Element from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP042Z"  =  "1"  /* Removal of Monitoring Device from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP043Z"  =  "1"  /* Removal of Infusion Device from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP047Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP04CZ"  =  "1"  /* Removal of Extraluminal Device from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP04DZ"  =  "1"  /* Removal of Intraluminal Device from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP04JZ"  =  "1"  /* Removal of Synthetic Substitute from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP04KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP070Z"  =  "1"  /* Removal of Drainage Device from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP071Z"  =  "1"  /* Removal of Radioactive Element from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP072Z"  =  "1"  /* Removal of Monitoring Device from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP073Z"  =  "1"  /* Removal of Infusion Device from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP077Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP07CZ"  =  "1"  /* Removal of Extraluminal Device from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP07DZ"  =  "1"  /* Removal of Intraluminal Device from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP07JZ"  =  "1"  /* Removal of Synthetic Substitute from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP07KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP080Z"  =  "1"  /* Removal of Drainage Device from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP081Z"  =  "1"  /* Removal of Radioactive Element from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP082Z"  =  "1"  /* Removal of Monitoring Device from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP083Z"  =  "1"  /* Removal of Infusion Device from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP087Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP08CZ"  =  "1"  /* Removal of Extraluminal Device from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP08DZ"  =  "1"  /* Removal of Intraluminal Device from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP08JZ"  =  "1"  /* Removal of Synthetic Substitute from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP08KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP100Z"  =  "1"  /* Removal of Drainage Device from Trachea, Open Approach */
"0BP102Z"  =  "1"  /* Removal of Monitoring Device from Trachea, Open Approach */
"0BP107Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Trachea, Open Approach */
"0BP10CZ"  =  "1"  /* Removal of Extraluminal Device from Trachea, Open Approach */
"0BP10DZ"  =  "1"  /* Removal of Intraluminal Device from Trachea, Open Approach */
"0BP10FZ"  =  "1"  /* Removal of Tracheostomy Device from Trachea, Open Approach */
"0BP10JZ"  =  "1"  /* Removal of Synthetic Substitute from Trachea, Open Approach */
"0BP10KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Trachea, Open Approach */
"0BP130Z"  =  "1"  /* Removal of Drainage Device from Trachea, Percutaneous Approach */
"0BP132Z"  =  "1"  /* Removal of Monitoring Device from Trachea, Percutaneous Approach */
"0BP137Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Trachea, Percutaneous Approach */
"0BP13CZ"  =  "1"  /* Removal of Extraluminal Device from Trachea, Percutaneous Approach */
"0BP13DZ"  =  "1"  /* Removal of Intraluminal Device from Trachea, Percutaneous Approach */
"0BP13FZ"  =  "1"  /* Removal of Tracheostomy Device from Trachea, Percutaneous Approach */
"0BP13JZ"  =  "1"  /* Removal of Synthetic Substitute from Trachea, Percutaneous Approach */
"0BP13KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Trachea, Percutaneous Approach */
"0BP140Z"  =  "1"  /* Removal of Drainage Device from Trachea, Percutaneous Endoscopic Approach */
"0BP142Z"  =  "1"  /* Removal of Monitoring Device from Trachea, Percutaneous Endoscopic Approach */
"0BP147Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Trachea, Percutaneous Endoscopic Approach */
"0BP14CZ"  =  "1"  /* Removal of Extraluminal Device from Trachea, Percutaneous Endoscopic Approach */
"0BP14DZ"  =  "1"  /* Removal of Intraluminal Device from Trachea, Percutaneous Endoscopic Approach */
"0BP14FZ"  =  "1"  /* Removal of Tracheostomy Device from Trachea, Percutaneous Endoscopic Approach */
"0BP14JZ"  =  "1"  /* Removal of Synthetic Substitute from Trachea, Percutaneous Endoscopic Approach */
"0BP14KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Trachea, Percutaneous Endoscopic Approach */
"0BP170Z"  =  "1"  /* Removal of Drainage Device from Trachea, Via Natural or Artificial Opening */
"0BP172Z"  =  "1"  /* Removal of Monitoring Device from Trachea, Via Natural or Artificial Opening */
"0BP177Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Trachea, Via Natural or Artificial Opening */
"0BP17CZ"  =  "1"  /* Removal of Extraluminal Device from Trachea, Via Natural or Artificial Opening */
"0BP17DZ"  =  "1"  /* Removal of Intraluminal Device from Trachea, Via Natural or Artificial Opening */
"0BP17FZ"  =  "1"  /* Removal of Tracheostomy Device from Trachea, Via Natural or Artificial Opening */
"0BP17JZ"  =  "1"  /* Removal of Synthetic Substitute from Trachea, Via Natural or Artificial Opening */
"0BP17KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Trachea, Via Natural or Artificial Opening */
"0BP180Z"  =  "1"  /* Removal of Drainage Device from Trachea, Via Natural or Artificial Opening Endoscopic */
"0BP182Z"  =  "1"  /* Removal of Monitoring Device from Trachea, Via Natural or Artificial Opening Endoscopic */
"0BP187Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Trachea, Via Natural or Artificial Opening Endoscopic */
"0BP18CZ"  =  "1"  /* Removal of Extraluminal Device from Trachea, Via Natural or Artificial Opening Endoscopic */
"0BP18DZ"  =  "1"  /* Removal of Intraluminal Device from Trachea, Via Natural or Artificial Opening Endoscopic */
"0BP18FZ"  =  "1"  /* Removal of Tracheostomy Device from Trachea, Via Natural or Artificial Opening Endoscopic */
"0BP18JZ"  =  "1"  /* Removal of Synthetic Substitute from Trachea, Via Natural or Artificial Opening Endoscopic */
"0BP18KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Trachea, Via Natural or Artificial Opening Endoscopic */
"0BPK00Z"  =  "1"  /* Removal of Drainage Device from Right Lung, Open Approach */
"0BPK01Z"  =  "1"  /* Removal of Radioactive Element from Right Lung, Open Approach */
"0BPK02Z"  =  "1"  /* Removal of Monitoring Device from Right Lung, Open Approach */
"0BPK03Z"  =  "1"  /* Removal of Infusion Device from Right Lung, Open Approach */
"0BPK30Z"  =  "1"  /* Removal of Drainage Device from Right Lung, Percutaneous Approach */
"0BPK31Z"  =  "1"  /* Removal of Radioactive Element from Right Lung, Percutaneous Approach */
"0BPK32Z"  =  "1"  /* Removal of Monitoring Device from Right Lung, Percutaneous Approach */
"0BPK33Z"  =  "1"  /* Removal of Infusion Device from Right Lung, Percutaneous Approach */
"0BPK40Z"  =  "1"  /* Removal of Drainage Device from Right Lung, Percutaneous Endoscopic Approach */
"0BPK41Z"  =  "1"  /* Removal of Radioactive Element from Right Lung, Percutaneous Endoscopic Approach */
"0BPK42Z"  =  "1"  /* Removal of Monitoring Device from Right Lung, Percutaneous Endoscopic Approach */
"0BPK43Z"  =  "1"  /* Removal of Infusion Device from Right Lung, Percutaneous Endoscopic Approach */
"0BPK70Z"  =  "1"  /* Removal of Drainage Device from Right Lung, Via Natural or Artificial Opening */
"0BPK71Z"  =  "1"  /* Removal of Radioactive Element from Right Lung, Via Natural or Artificial Opening */
"0BPK72Z"  =  "1"  /* Removal of Monitoring Device from Right Lung, Via Natural or Artificial Opening */
"0BPK73Z"  =  "1"  /* Removal of Infusion Device from Right Lung, Via Natural or Artificial Opening */
"0BPK80Z"  =  "1"  /* Removal of Drainage Device from Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BPK81Z"  =  "1"  /* Removal of Radioactive Element from Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BPK82Z"  =  "1"  /* Removal of Monitoring Device from Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BPK83Z"  =  "1"  /* Removal of Infusion Device from Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BPL00Z"  =  "1"  /* Removal of Drainage Device from Left Lung, Open Approach */
"0BPL01Z"  =  "1"  /* Removal of Radioactive Element from Left Lung, Open Approach */
"0BPL02Z"  =  "1"  /* Removal of Monitoring Device from Left Lung, Open Approach */
"0BPL03Z"  =  "1"  /* Removal of Infusion Device from Left Lung, Open Approach */
"0BPL30Z"  =  "1"  /* Removal of Drainage Device from Left Lung, Percutaneous Approach */
"0BPL31Z"  =  "1"  /* Removal of Radioactive Element from Left Lung, Percutaneous Approach */
"0BPL32Z"  =  "1"  /* Removal of Monitoring Device from Left Lung, Percutaneous Approach */
"0BPL33Z"  =  "1"  /* Removal of Infusion Device from Left Lung, Percutaneous Approach */
"0BPL40Z"  =  "1"  /* Removal of Drainage Device from Left Lung, Percutaneous Endoscopic Approach */
"0BPL41Z"  =  "1"  /* Removal of Radioactive Element from Left Lung, Percutaneous Endoscopic Approach */
"0BPL42Z"  =  "1"  /* Removal of Monitoring Device from Left Lung, Percutaneous Endoscopic Approach */
"0BPL43Z"  =  "1"  /* Removal of Infusion Device from Left Lung, Percutaneous Endoscopic Approach */
"0BPL70Z"  =  "1"  /* Removal of Drainage Device from Left Lung, Via Natural or Artificial Opening */
"0BPL71Z"  =  "1"  /* Removal of Radioactive Element from Left Lung, Via Natural or Artificial Opening */
"0BPL72Z"  =  "1"  /* Removal of Monitoring Device from Left Lung, Via Natural or Artificial Opening */
"0BPL73Z"  =  "1"  /* Removal of Infusion Device from Left Lung, Via Natural or Artificial Opening */
"0BPL80Z"  =  "1"  /* Removal of Drainage Device from Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BPL81Z"  =  "1"  /* Removal of Radioactive Element from Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BPL82Z"  =  "1"  /* Removal of Monitoring Device from Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BPL83Z"  =  "1"  /* Removal of Infusion Device from Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BPQ00Z"  =  "1"  /* Removal of Drainage Device from Pleura, Open Approach */
"0BPQ01Z"  =  "1"  /* Removal of Radioactive Element from Pleura, Open Approach */
"0BPQ02Z"  =  "1"  /* Removal of Monitoring Device from Pleura, Open Approach */
"0BPQ30Z"  =  "1"  /* Removal of Drainage Device from Pleura, Percutaneous Approach */
"0BPQ31Z"  =  "1"  /* Removal of Radioactive Element from Pleura, Percutaneous Approach */
"0BPQ32Z"  =  "1"  /* Removal of Monitoring Device from Pleura, Percutaneous Approach */
"0BPQ40Z"  =  "1"  /* Removal of Drainage Device from Pleura, Percutaneous Endoscopic Approach */
"0BPQ41Z"  =  "1"  /* Removal of Radioactive Element from Pleura, Percutaneous Endoscopic Approach */
"0BPQ42Z"  =  "1"  /* Removal of Monitoring Device from Pleura, Percutaneous Endoscopic Approach */
"0BPQ70Z"  =  "1"  /* Removal of Drainage Device from Pleura, Via Natural or Artificial Opening */
"0BPQ71Z"  =  "1"  /* Removal of Radioactive Element from Pleura, Via Natural or Artificial Opening */
"0BPQ72Z"  =  "1"  /* Removal of Monitoring Device from Pleura, Via Natural or Artificial Opening */
"0BPQ80Z"  =  "1"  /* Removal of Drainage Device from Pleura, Via Natural or Artificial Opening Endoscopic */
"0BPQ81Z"  =  "1"  /* Removal of Radioactive Element from Pleura, Via Natural or Artificial Opening Endoscopic */
"0BPQ82Z"  =  "1"  /* Removal of Monitoring Device from Pleura, Via Natural or Artificial Opening Endoscopic */
"0BW000Z"  =  "1"  /* Revision of Drainage Device in Tracheobronchial Tree, Open Approach */
"0BW002Z"  =  "1"  /* Revision of Monitoring Device in Tracheobronchial Tree, Open Approach */
"0BW003Z"  =  "1"  /* Revision of Infusion Device in Tracheobronchial Tree, Open Approach */
"0BW007Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Tracheobronchial Tree, Open Approach */
"0BW00CZ"  =  "1"  /* Revision of Extraluminal Device in Tracheobronchial Tree, Open Approach */
"0BW00DZ"  =  "1"  /* Revision of Intraluminal Device in Tracheobronchial Tree, Open Approach */
"0BW00JZ"  =  "1"  /* Revision of Synthetic Substitute in Tracheobronchial Tree, Open Approach */
"0BW00KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Tracheobronchial Tree, Open Approach */
"0BW030Z"  =  "1"  /* Revision of Drainage Device in Tracheobronchial Tree, Percutaneous Approach */
"0BW032Z"  =  "1"  /* Revision of Monitoring Device in Tracheobronchial Tree, Percutaneous Approach */
"0BW033Z"  =  "1"  /* Revision of Infusion Device in Tracheobronchial Tree, Percutaneous Approach */
"0BW037Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Tracheobronchial Tree, Percutaneous Approach */
"0BW03CZ"  =  "1"  /* Revision of Extraluminal Device in Tracheobronchial Tree, Percutaneous Approach */
"0BW03DZ"  =  "1"  /* Revision of Intraluminal Device in Tracheobronchial Tree, Percutaneous Approach */
"0BW03JZ"  =  "1"  /* Revision of Synthetic Substitute in Tracheobronchial Tree, Percutaneous Approach */
"0BW03KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Tracheobronchial Tree, Percutaneous Approach */
"0BW040Z"  =  "1"  /* Revision of Drainage Device in Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BW042Z"  =  "1"  /* Revision of Monitoring Device in Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BW043Z"  =  "1"  /* Revision of Infusion Device in Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BW047Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BW04CZ"  =  "1"  /* Revision of Extraluminal Device in Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BW04DZ"  =  "1"  /* Revision of Intraluminal Device in Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BW04JZ"  =  "1"  /* Revision of Synthetic Substitute in Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BW04KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BW070Z"  =  "1"  /* Revision of Drainage Device in Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BW072Z"  =  "1"  /* Revision of Monitoring Device in Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BW073Z"  =  "1"  /* Revision of Infusion Device in Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BW077Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BW07CZ"  =  "1"  /* Revision of Extraluminal Device in Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BW07DZ"  =  "1"  /* Revision of Intraluminal Device in Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BW07JZ"  =  "1"  /* Revision of Synthetic Substitute in Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BW07KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BW080Z"  =  "1"  /* Revision of Drainage Device in Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BW082Z"  =  "1"  /* Revision of Monitoring Device in Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BW083Z"  =  "1"  /* Revision of Infusion Device in Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BW087Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BW08CZ"  =  "1"  /* Revision of Extraluminal Device in Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BW08DZ"  =  "1"  /* Revision of Intraluminal Device in Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BW08JZ"  =  "1"  /* Revision of Synthetic Substitute in Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BW08KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BW100Z"  =  "1"  /* Revision of Drainage Device in Trachea, Open Approach */
"0BW102Z"  =  "1"  /* Revision of Monitoring Device in Trachea, Open Approach */
"0BW107Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Trachea, Open Approach */
"0BW10CZ"  =  "1"  /* Revision of Extraluminal Device in Trachea, Open Approach */
"0BW10DZ"  =  "1"  /* Revision of Intraluminal Device in Trachea, Open Approach */
"0BW10JZ"  =  "1"  /* Revision of Synthetic Substitute in Trachea, Open Approach */
"0BW10KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Trachea, Open Approach */
"0BW130Z"  =  "1"  /* Revision of Drainage Device in Trachea, Percutaneous Approach */
"0BW132Z"  =  "1"  /* Revision of Monitoring Device in Trachea, Percutaneous Approach */
"0BW137Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Trachea, Percutaneous Approach */
"0BW13CZ"  =  "1"  /* Revision of Extraluminal Device in Trachea, Percutaneous Approach */
"0BW13DZ"  =  "1"  /* Revision of Intraluminal Device in Trachea, Percutaneous Approach */
"0BW13JZ"  =  "1"  /* Revision of Synthetic Substitute in Trachea, Percutaneous Approach */
"0BW13KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Trachea, Percutaneous Approach */
"0BW140Z"  =  "1"  /* Revision of Drainage Device in Trachea, Percutaneous Endoscopic Approach */
"0BW142Z"  =  "1"  /* Revision of Monitoring Device in Trachea, Percutaneous Endoscopic Approach */
"0BW147Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Trachea, Percutaneous Endoscopic Approach */
"0BW14CZ"  =  "1"  /* Revision of Extraluminal Device in Trachea, Percutaneous Endoscopic Approach */
"0BW14DZ"  =  "1"  /* Revision of Intraluminal Device in Trachea, Percutaneous Endoscopic Approach */
"0BW14JZ"  =  "1"  /* Revision of Synthetic Substitute in Trachea, Percutaneous Endoscopic Approach */
"0BW14KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Trachea, Percutaneous Endoscopic Approach */
"0BW170Z"  =  "1"  /* Revision of Drainage Device in Trachea, Via Natural or Artificial Opening */
"0BW172Z"  =  "1"  /* Revision of Monitoring Device in Trachea, Via Natural or Artificial Opening */
"0BW177Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Trachea, Via Natural or Artificial Opening */
"0BW17CZ"  =  "1"  /* Revision of Extraluminal Device in Trachea, Via Natural or Artificial Opening */
"0BW17DZ"  =  "1"  /* Revision of Intraluminal Device in Trachea, Via Natural or Artificial Opening */
"0BW17FZ"  =  "1"  /* Revision of Tracheostomy Device in Trachea, Via Natural or Artificial Opening */
"0BW17JZ"  =  "1"  /* Revision of Synthetic Substitute in Trachea, Via Natural or Artificial Opening */
"0BW17KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Trachea, Via Natural or Artificial Opening */
"0BW180Z"  =  "1"  /* Revision of Drainage Device in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BW182Z"  =  "1"  /* Revision of Monitoring Device in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BW187Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BW18CZ"  =  "1"  /* Revision of Extraluminal Device in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BW18DZ"  =  "1"  /* Revision of Intraluminal Device in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BW18FZ"  =  "1"  /* Revision of Tracheostomy Device in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BW18JZ"  =  "1"  /* Revision of Synthetic Substitute in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BW18KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BWK00Z"  =  "1"  /* Revision of Drainage Device in Right Lung, Open Approach */
"0BWK02Z"  =  "1"  /* Revision of Monitoring Device in Right Lung, Open Approach */
"0BWK03Z"  =  "1"  /* Revision of Infusion Device in Right Lung, Open Approach */
"0BWK30Z"  =  "1"  /* Revision of Drainage Device in Right Lung, Percutaneous Approach */
"0BWK32Z"  =  "1"  /* Revision of Monitoring Device in Right Lung, Percutaneous Approach */
"0BWK33Z"  =  "1"  /* Revision of Infusion Device in Right Lung, Percutaneous Approach */
"0BWK40Z"  =  "1"  /* Revision of Drainage Device in Right Lung, Percutaneous Endoscopic Approach */
"0BWK42Z"  =  "1"  /* Revision of Monitoring Device in Right Lung, Percutaneous Endoscopic Approach */
"0BWK43Z"  =  "1"  /* Revision of Infusion Device in Right Lung, Percutaneous Endoscopic Approach */
"0BWK70Z"  =  "1"  /* Revision of Drainage Device in Right Lung, Via Natural or Artificial Opening */
"0BWK72Z"  =  "1"  /* Revision of Monitoring Device in Right Lung, Via Natural or Artificial Opening */
"0BWK73Z"  =  "1"  /* Revision of Infusion Device in Right Lung, Via Natural or Artificial Opening */
"0BWK80Z"  =  "1"  /* Revision of Drainage Device in Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BWK82Z"  =  "1"  /* Revision of Monitoring Device in Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BWK83Z"  =  "1"  /* Revision of Infusion Device in Right Lung, Via Natural or Artificial Opening Endoscopic */
"0BWL00Z"  =  "1"  /* Revision of Drainage Device in Left Lung, Open Approach */
"0BWL02Z"  =  "1"  /* Revision of Monitoring Device in Left Lung, Open Approach */
"0BWL03Z"  =  "1"  /* Revision of Infusion Device in Left Lung, Open Approach */
"0BWL30Z"  =  "1"  /* Revision of Drainage Device in Left Lung, Percutaneous Approach */
"0BWL32Z"  =  "1"  /* Revision of Monitoring Device in Left Lung, Percutaneous Approach */
"0BWL33Z"  =  "1"  /* Revision of Infusion Device in Left Lung, Percutaneous Approach */
"0BWL40Z"  =  "1"  /* Revision of Drainage Device in Left Lung, Percutaneous Endoscopic Approach */
"0BWL42Z"  =  "1"  /* Revision of Monitoring Device in Left Lung, Percutaneous Endoscopic Approach */
"0BWL43Z"  =  "1"  /* Revision of Infusion Device in Left Lung, Percutaneous Endoscopic Approach */
"0BWL70Z"  =  "1"  /* Revision of Drainage Device in Left Lung, Via Natural or Artificial Opening */
"0BWL72Z"  =  "1"  /* Revision of Monitoring Device in Left Lung, Via Natural or Artificial Opening */
"0BWL73Z"  =  "1"  /* Revision of Infusion Device in Left Lung, Via Natural or Artificial Opening */
"0BWL80Z"  =  "1"  /* Revision of Drainage Device in Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BWL82Z"  =  "1"  /* Revision of Monitoring Device in Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BWL83Z"  =  "1"  /* Revision of Infusion Device in Left Lung, Via Natural or Artificial Opening Endoscopic */
"0BWQ00Z"  =  "1"  /* Revision of Drainage Device in Pleura, Open Approach */
"0BWQ02Z"  =  "1"  /* Revision of Monitoring Device in Pleura, Open Approach */
"0BWQ30Z"  =  "1"  /* Revision of Drainage Device in Pleura, Percutaneous Approach */
"0BWQ32Z"  =  "1"  /* Revision of Monitoring Device in Pleura, Percutaneous Approach */
"0BWQ40Z"  =  "1"  /* Revision of Drainage Device in Pleura, Percutaneous Endoscopic Approach */
"0BWQ42Z"  =  "1"  /* Revision of Monitoring Device in Pleura, Percutaneous Endoscopic Approach */
"0BWQ70Z"  =  "1"  /* Revision of Drainage Device in Pleura, Via Natural or Artificial Opening */
"0BWQ72Z"  =  "1"  /* Revision of Monitoring Device in Pleura, Via Natural or Artificial Opening */
"0BWQ80Z"  =  "1"  /* Revision of Drainage Device in Pleura, Via Natural or Artificial Opening Endoscopic */
"0BWQ82Z"  =  "1"  /* Revision of Monitoring Device in Pleura, Via Natural or Artificial Opening Endoscopic */
"0C7M7ZZ"  =  "1"  /* Dilation of Pharynx, Via Natural or Artificial Opening */
"0C7M8ZZ"  =  "1"  /* Dilation of Pharynx, Via Natural or Artificial Opening Endoscopic */
"0C9000Z"  =  "1"  /* Drainage of Upper Lip with Drainage Device, Open Approach */
"0C900ZZ"  =  "1"  /* Drainage of Upper Lip, Open Approach */
"0C9030Z"  =  "1"  /* Drainage of Upper Lip with Drainage Device, Percutaneous Approach */
"0C903ZZ"  =  "1"  /* Drainage of Upper Lip, Percutaneous Approach */
"0C90X0Z"  =  "1"  /* Drainage of Upper Lip with Drainage Device, External Approach */
"0C90XZZ"  =  "1"  /* Drainage of Upper Lip, External Approach */
"0C9100Z"  =  "1"  /* Drainage of Lower Lip with Drainage Device, Open Approach */
"0C910ZZ"  =  "1"  /* Drainage of Lower Lip, Open Approach */
"0C9130Z"  =  "1"  /* Drainage of Lower Lip with Drainage Device, Percutaneous Approach */
"0C913ZZ"  =  "1"  /* Drainage of Lower Lip, Percutaneous Approach */
"0C91X0Z"  =  "1"  /* Drainage of Lower Lip with Drainage Device, External Approach */
"0C91XZZ"  =  "1"  /* Drainage of Lower Lip, External Approach */
"0C9400Z"  =  "1"  /* Drainage of Buccal Mucosa with Drainage Device, Open Approach */
"0C940ZZ"  =  "1"  /* Drainage of Buccal Mucosa, Open Approach */
"0C9430Z"  =  "1"  /* Drainage of Buccal Mucosa with Drainage Device, Percutaneous Approach */
"0C943ZZ"  =  "1"  /* Drainage of Buccal Mucosa, Percutaneous Approach */
"0C94X0Z"  =  "1"  /* Drainage of Buccal Mucosa with Drainage Device, External Approach */
"0C94XZZ"  =  "1"  /* Drainage of Buccal Mucosa, External Approach */
"0C9800Z"  =  "1"  /* Drainage of Right Parotid Gland with Drainage Device, Open Approach */
"0C980ZZ"  =  "1"  /* Drainage of Right Parotid Gland, Open Approach */
"0C9830Z"  =  "1"  /* Drainage of Right Parotid Gland with Drainage Device, Percutaneous Approach */
"0C983ZZ"  =  "1"  /* Drainage of Right Parotid Gland, Percutaneous Approach */
"0C9900Z"  =  "1"  /* Drainage of Left Parotid Gland with Drainage Device, Open Approach */
"0C990ZZ"  =  "1"  /* Drainage of Left Parotid Gland, Open Approach */
"0C9930Z"  =  "1"  /* Drainage of Left Parotid Gland with Drainage Device, Percutaneous Approach */
"0C993ZZ"  =  "1"  /* Drainage of Left Parotid Gland, Percutaneous Approach */
"0C9B00Z"  =  "1"  /* Drainage of Right Parotid Duct with Drainage Device, Open Approach */
"0C9B0ZZ"  =  "1"  /* Drainage of Right Parotid Duct, Open Approach */
"0C9B30Z"  =  "1"  /* Drainage of Right Parotid Duct with Drainage Device, Percutaneous Approach */
"0C9B3ZZ"  =  "1"  /* Drainage of Right Parotid Duct, Percutaneous Approach */
"0C9C00Z"  =  "1"  /* Drainage of Left Parotid Duct with Drainage Device, Open Approach */
"0C9C0ZZ"  =  "1"  /* Drainage of Left Parotid Duct, Open Approach */
"0C9C30Z"  =  "1"  /* Drainage of Left Parotid Duct with Drainage Device, Percutaneous Approach */
"0C9C3ZZ"  =  "1"  /* Drainage of Left Parotid Duct, Percutaneous Approach */
"0C9D00Z"  =  "1"  /* Drainage of Right Sublingual Gland with Drainage Device, Open Approach */
"0C9D0ZZ"  =  "1"  /* Drainage of Right Sublingual Gland, Open Approach */
"0C9D30Z"  =  "1"  /* Drainage of Right Sublingual Gland with Drainage Device, Percutaneous Approach */
"0C9D3ZZ"  =  "1"  /* Drainage of Right Sublingual Gland, Percutaneous Approach */
"0C9F00Z"  =  "1"  /* Drainage of Left Sublingual Gland with Drainage Device, Open Approach */
"0C9F0ZZ"  =  "1"  /* Drainage of Left Sublingual Gland, Open Approach */
"0C9F30Z"  =  "1"  /* Drainage of Left Sublingual Gland with Drainage Device, Percutaneous Approach */
"0C9F3ZZ"  =  "1"  /* Drainage of Left Sublingual Gland, Percutaneous Approach */
"0C9G00Z"  =  "1"  /* Drainage of Right Submaxillary Gland with Drainage Device, Open Approach */
"0C9G0ZZ"  =  "1"  /* Drainage of Right Submaxillary Gland, Open Approach */
"0C9G30Z"  =  "1"  /* Drainage of Right Submaxillary Gland with Drainage Device, Percutaneous Approach */
"0C9G3ZZ"  =  "1"  /* Drainage of Right Submaxillary Gland, Percutaneous Approach */
"0C9H00Z"  =  "1"  /* Drainage of Left Submaxillary Gland with Drainage Device, Open Approach */
"0C9H0ZZ"  =  "1"  /* Drainage of Left Submaxillary Gland, Open Approach */
"0C9H30Z"  =  "1"  /* Drainage of Left Submaxillary Gland with Drainage Device, Percutaneous Approach */
"0C9H3ZZ"  =  "1"  /* Drainage of Left Submaxillary Gland, Percutaneous Approach */
"0C9J00Z"  =  "1"  /* Drainage of Minor Salivary Gland with Drainage Device, Open Approach */
"0C9J0ZZ"  =  "1"  /* Drainage of Minor Salivary Gland, Open Approach */
"0C9J30Z"  =  "1"  /* Drainage of Minor Salivary Gland with Drainage Device, Percutaneous Approach */
"0C9J3ZZ"  =  "1"  /* Drainage of Minor Salivary Gland, Percutaneous Approach */
"0C9P00Z"  =  "1"  /* Drainage of Tonsils with Drainage Device, Open Approach */
"0C9P0ZZ"  =  "1"  /* Drainage of Tonsils, Open Approach */
"0C9P30Z"  =  "1"  /* Drainage of Tonsils with Drainage Device, Percutaneous Approach */
"0C9P3ZZ"  =  "1"  /* Drainage of Tonsils, Percutaneous Approach */
"0C9PX0Z"  =  "1"  /* Drainage of Tonsils with Drainage Device, External Approach */
"0C9PXZZ"  =  "1"  /* Drainage of Tonsils, External Approach */
"0C9R00Z"  =  "1"  /* Drainage of Epiglottis with Drainage Device, Open Approach */
"0C9R0ZZ"  =  "1"  /* Drainage of Epiglottis, Open Approach */
"0C9R30Z"  =  "1"  /* Drainage of Epiglottis with Drainage Device, Percutaneous Approach */
"0C9R3ZZ"  =  "1"  /* Drainage of Epiglottis, Percutaneous Approach */
"0C9R40Z"  =  "1"  /* Drainage of Epiglottis with Drainage Device, Percutaneous Endoscopic Approach */
"0C9R4ZZ"  =  "1"  /* Drainage of Epiglottis, Percutaneous Endoscopic Approach */
"0C9R70Z"  =  "1"  /* Drainage of Epiglottis with Drainage Device, Via Natural or Artificial Opening */
"0C9R7ZZ"  =  "1"  /* Drainage of Epiglottis, Via Natural or Artificial Opening */
"0C9R80Z"  =  "1"  /* Drainage of Epiglottis with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0C9R8ZZ"  =  "1"  /* Drainage of Epiglottis, Via Natural or Artificial Opening Endoscopic */
"0C9S00Z"  =  "1"  /* Drainage of Larynx with Drainage Device, Open Approach */
"0C9S0ZZ"  =  "1"  /* Drainage of Larynx, Open Approach */
"0C9S30Z"  =  "1"  /* Drainage of Larynx with Drainage Device, Percutaneous Approach */
"0C9S3ZZ"  =  "1"  /* Drainage of Larynx, Percutaneous Approach */
"0C9S40Z"  =  "1"  /* Drainage of Larynx with Drainage Device, Percutaneous Endoscopic Approach */
"0C9S4ZZ"  =  "1"  /* Drainage of Larynx, Percutaneous Endoscopic Approach */
"0C9S70Z"  =  "1"  /* Drainage of Larynx with Drainage Device, Via Natural or Artificial Opening */
"0C9S7ZZ"  =  "1"  /* Drainage of Larynx, Via Natural or Artificial Opening */
"0C9S80Z"  =  "1"  /* Drainage of Larynx with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0C9S8ZZ"  =  "1"  /* Drainage of Larynx, Via Natural or Artificial Opening Endoscopic */
"0C9T00Z"  =  "1"  /* Drainage of Right Vocal Cord with Drainage Device, Open Approach */
"0C9T0ZZ"  =  "1"  /* Drainage of Right Vocal Cord, Open Approach */
"0C9T30Z"  =  "1"  /* Drainage of Right Vocal Cord with Drainage Device, Percutaneous Approach */
"0C9T3ZZ"  =  "1"  /* Drainage of Right Vocal Cord, Percutaneous Approach */
"0C9T40Z"  =  "1"  /* Drainage of Right Vocal Cord with Drainage Device, Percutaneous Endoscopic Approach */
"0C9T4ZZ"  =  "1"  /* Drainage of Right Vocal Cord, Percutaneous Endoscopic Approach */
"0C9T70Z"  =  "1"  /* Drainage of Right Vocal Cord with Drainage Device, Via Natural or Artificial Opening */
"0C9T7ZZ"  =  "1"  /* Drainage of Right Vocal Cord, Via Natural or Artificial Opening */
"0C9T80Z"  =  "1"  /* Drainage of Right Vocal Cord with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0C9T8ZZ"  =  "1"  /* Drainage of Right Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0C9V00Z"  =  "1"  /* Drainage of Left Vocal Cord with Drainage Device, Open Approach */
"0C9V0ZZ"  =  "1"  /* Drainage of Left Vocal Cord, Open Approach */
"0C9V30Z"  =  "1"  /* Drainage of Left Vocal Cord with Drainage Device, Percutaneous Approach */
"0C9V3ZZ"  =  "1"  /* Drainage of Left Vocal Cord, Percutaneous Approach */
"0C9V40Z"  =  "1"  /* Drainage of Left Vocal Cord with Drainage Device, Percutaneous Endoscopic Approach */
"0C9V4ZZ"  =  "1"  /* Drainage of Left Vocal Cord, Percutaneous Endoscopic Approach */
"0C9V70Z"  =  "1"  /* Drainage of Left Vocal Cord with Drainage Device, Via Natural or Artificial Opening */
"0C9V7ZZ"  =  "1"  /* Drainage of Left Vocal Cord, Via Natural or Artificial Opening */
"0C9V80Z"  =  "1"  /* Drainage of Left Vocal Cord with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0C9V8ZZ"  =  "1"  /* Drainage of Left Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CC80ZZ"  =  "1"  /* Extirpation of Matter from Right Parotid Gland, Open Approach */
"0CC83ZZ"  =  "1"  /* Extirpation of Matter from Right Parotid Gland, Percutaneous Approach */
"0CC90ZZ"  =  "1"  /* Extirpation of Matter from Left Parotid Gland, Open Approach */
"0CC93ZZ"  =  "1"  /* Extirpation of Matter from Left Parotid Gland, Percutaneous Approach */
"0CCB0ZZ"  =  "1"  /* Extirpation of Matter from Right Parotid Duct, Open Approach */
"0CCB3ZZ"  =  "1"  /* Extirpation of Matter from Right Parotid Duct, Percutaneous Approach */
"0CCC0ZZ"  =  "1"  /* Extirpation of Matter from Left Parotid Duct, Open Approach */
"0CCC3ZZ"  =  "1"  /* Extirpation of Matter from Left Parotid Duct, Percutaneous Approach */
"0CCD0ZZ"  =  "1"  /* Extirpation of Matter from Right Sublingual Gland, Open Approach */
"0CCD3ZZ"  =  "1"  /* Extirpation of Matter from Right Sublingual Gland, Percutaneous Approach */
"0CCF0ZZ"  =  "1"  /* Extirpation of Matter from Left Sublingual Gland, Open Approach */
"0CCF3ZZ"  =  "1"  /* Extirpation of Matter from Left Sublingual Gland, Percutaneous Approach */
"0CCG0ZZ"  =  "1"  /* Extirpation of Matter from Right Submaxillary Gland, Open Approach */
"0CCG3ZZ"  =  "1"  /* Extirpation of Matter from Right Submaxillary Gland, Percutaneous Approach */
"0CCH0ZZ"  =  "1"  /* Extirpation of Matter from Left Submaxillary Gland, Open Approach */
"0CCH3ZZ"  =  "1"  /* Extirpation of Matter from Left Submaxillary Gland, Percutaneous Approach */
"0CCJ0ZZ"  =  "1"  /* Extirpation of Matter from Minor Salivary Gland, Open Approach */
"0CCJ3ZZ"  =  "1"  /* Extirpation of Matter from Minor Salivary Gland, Percutaneous Approach */
"0CCR0ZZ"  =  "1"  /* Extirpation of Matter from Epiglottis, Open Approach */
"0CCR3ZZ"  =  "1"  /* Extirpation of Matter from Epiglottis, Percutaneous Approach */
"0CCR4ZZ"  =  "1"  /* Extirpation of Matter from Epiglottis, Percutaneous Endoscopic Approach */
"0CCR7ZZ"  =  "1"  /* Extirpation of Matter from Epiglottis, Via Natural or Artificial Opening */
"0CCR8ZZ"  =  "1"  /* Extirpation of Matter from Epiglottis, Via Natural or Artificial Opening Endoscopic */
"0CCS0ZZ"  =  "1"  /* Extirpation of Matter from Larynx, Open Approach */
"0CCS3ZZ"  =  "1"  /* Extirpation of Matter from Larynx, Percutaneous Approach */
"0CCS4ZZ"  =  "1"  /* Extirpation of Matter from Larynx, Percutaneous Endoscopic Approach */
"0CCT0ZZ"  =  "1"  /* Extirpation of Matter from Right Vocal Cord, Open Approach */
"0CCT3ZZ"  =  "1"  /* Extirpation of Matter from Right Vocal Cord, Percutaneous Approach */
"0CCT4ZZ"  =  "1"  /* Extirpation of Matter from Right Vocal Cord, Percutaneous Endoscopic Approach */
"0CCT7ZZ"  =  "1"  /* Extirpation of Matter from Right Vocal Cord, Via Natural or Artificial Opening */
"0CCT8ZZ"  =  "1"  /* Extirpation of Matter from Right Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CCV0ZZ"  =  "1"  /* Extirpation of Matter from Left Vocal Cord, Open Approach */
"0CCV3ZZ"  =  "1"  /* Extirpation of Matter from Left Vocal Cord, Percutaneous Approach */
"0CCV4ZZ"  =  "1"  /* Extirpation of Matter from Left Vocal Cord, Percutaneous Endoscopic Approach */
"0CCV7ZZ"  =  "1"  /* Extirpation of Matter from Left Vocal Cord, Via Natural or Artificial Opening */
"0CCV8ZZ"  =  "1"  /* Extirpation of Matter from Left Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CFB0ZZ"  =  "1"  /* Fragmentation in Right Parotid Duct, Open Approach */
"0CFB3ZZ"  =  "1"  /* Fragmentation in Right Parotid Duct, Percutaneous Approach */
"0CFB7ZZ"  =  "1"  /* Fragmentation in Right Parotid Duct, Via Natural or Artificial Opening */
"0CFC0ZZ"  =  "1"  /* Fragmentation in Left Parotid Duct, Open Approach */
"0CFC3ZZ"  =  "1"  /* Fragmentation in Left Parotid Duct, Percutaneous Approach */
"0CFC7ZZ"  =  "1"  /* Fragmentation in Left Parotid Duct, Via Natural or Artificial Opening */
"0CJS4ZZ"  =  "1"  /* Inspection of Larynx, Percutaneous Endoscopic Approach */
"0CJS8ZZ"  =  "1"  /* Inspection of Larynx, Via Natural or Artificial Opening Endoscopic */
"0CJY8ZZ"  =  "1"  /* Inspection of Mouth and Throat, Via Natural or Artificial Opening Endoscopic */
"0CPA00Z"  =  "1"  /* Removal of Drainage Device from Salivary Gland, Open Approach */
"0CPA0CZ"  =  "1"  /* Removal of Extraluminal Device from Salivary Gland, Open Approach */
"0CPA30Z"  =  "1"  /* Removal of Drainage Device from Salivary Gland, Percutaneous Approach */
"0CPA3CZ"  =  "1"  /* Removal of Extraluminal Device from Salivary Gland, Percutaneous Approach */
"0CPS00Z"  =  "1"  /* Removal of Drainage Device from Larynx, Open Approach */
"0CPS07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Larynx, Open Approach */
"0CPS0DZ"  =  "1"  /* Removal of Intraluminal Device from Larynx, Open Approach */
"0CPS0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Larynx, Open Approach */
"0CPS30Z"  =  "1"  /* Removal of Drainage Device from Larynx, Percutaneous Approach */
"0CPS37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Larynx, Percutaneous Approach */
"0CPS3DZ"  =  "1"  /* Removal of Intraluminal Device from Larynx, Percutaneous Approach */
"0CPS3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Larynx, Percutaneous Approach */
"0CPS70Z"  =  "1"  /* Removal of Drainage Device from Larynx, Via Natural or Artificial Opening */
"0CPS77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Larynx, Via Natural or Artificial Opening */
"0CPS7DZ"  =  "1"  /* Removal of Intraluminal Device from Larynx, Via Natural or Artificial Opening */
"0CPS7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Larynx, Via Natural or Artificial Opening */
"0CPS80Z"  =  "1"  /* Removal of Drainage Device from Larynx, Via Natural or Artificial Opening Endoscopic */
"0CPS87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Larynx, Via Natural or Artificial Opening Endoscopic */
"0CPS8DZ"  =  "1"  /* Removal of Intraluminal Device from Larynx, Via Natural or Artificial Opening Endoscopic */
"0CPS8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Larynx, Via Natural or Artificial Opening Endoscopic */
"0CWA00Z"  =  "1"  /* Revision of Drainage Device in Salivary Gland, Open Approach */
"0CWA0CZ"  =  "1"  /* Revision of Extraluminal Device in Salivary Gland, Open Approach */
"0CWA30Z"  =  "1"  /* Revision of Drainage Device in Salivary Gland, Percutaneous Approach */
"0CWA3CZ"  =  "1"  /* Revision of Extraluminal Device in Salivary Gland, Percutaneous Approach */
"0CWS00Z"  =  "1"  /* Revision of Drainage Device in Larynx, Open Approach */
"0CWS07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Larynx, Open Approach */
"0CWS0DZ"  =  "1"  /* Revision of Intraluminal Device in Larynx, Open Approach */
"0CWS0JZ"  =  "1"  /* Revision of Synthetic Substitute in Larynx, Open Approach */
"0CWS0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Larynx, Open Approach */
"0CWS30Z"  =  "1"  /* Revision of Drainage Device in Larynx, Percutaneous Approach */
"0CWS37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Larynx, Percutaneous Approach */
"0CWS3DZ"  =  "1"  /* Revision of Intraluminal Device in Larynx, Percutaneous Approach */
"0CWS3JZ"  =  "1"  /* Revision of Synthetic Substitute in Larynx, Percutaneous Approach */
"0CWS3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Larynx, Percutaneous Approach */
"0CWS70Z"  =  "1"  /* Revision of Drainage Device in Larynx, Via Natural or Artificial Opening */
"0CWS77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Larynx, Via Natural or Artificial Opening */
"0CWS7DZ"  =  "1"  /* Revision of Intraluminal Device in Larynx, Via Natural or Artificial Opening */
"0CWS7JZ"  =  "1"  /* Revision of Synthetic Substitute in Larynx, Via Natural or Artificial Opening */
"0CWS7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Larynx, Via Natural or Artificial Opening */
"0CWS80Z"  =  "1"  /* Revision of Drainage Device in Larynx, Via Natural or Artificial Opening Endoscopic */
"0CWS87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Larynx, Via Natural or Artificial Opening Endoscopic */
"0CWS8DZ"  =  "1"  /* Revision of Intraluminal Device in Larynx, Via Natural or Artificial Opening Endoscopic */
"0CWS8JZ"  =  "1"  /* Revision of Synthetic Substitute in Larynx, Via Natural or Artificial Opening Endoscopic */
"0CWS8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Larynx, Via Natural or Artificial Opening Endoscopic */
"0D510ZZ"  =  "1"  /* Destruction of Upper Esophagus, Open Approach */
"0D513ZZ"  =  "1"  /* Destruction of Upper Esophagus, Percutaneous Approach */
"0D514ZZ"  =  "1"  /* Destruction of Upper Esophagus, Percutaneous Endoscopic Approach */
"0D517ZZ"  =  "1"  /* Destruction of Upper Esophagus, Via Natural or Artificial Opening */
"0D518ZZ"  =  "1"  /* Destruction of Upper Esophagus, Via Natural or Artificial Opening Endoscopic */
"0D520ZZ"  =  "1"  /* Destruction of Middle Esophagus, Open Approach */
"0D523ZZ"  =  "1"  /* Destruction of Middle Esophagus, Percutaneous Approach */
"0D524ZZ"  =  "1"  /* Destruction of Middle Esophagus, Percutaneous Endoscopic Approach */
"0D527ZZ"  =  "1"  /* Destruction of Middle Esophagus, Via Natural or Artificial Opening */
"0D528ZZ"  =  "1"  /* Destruction of Middle Esophagus, Via Natural or Artificial Opening Endoscopic */
"0D530ZZ"  =  "1"  /* Destruction of Lower Esophagus, Open Approach */
"0D533ZZ"  =  "1"  /* Destruction of Lower Esophagus, Percutaneous Approach */
"0D534ZZ"  =  "1"  /* Destruction of Lower Esophagus, Percutaneous Endoscopic Approach */
"0D537ZZ"  =  "1"  /* Destruction of Lower Esophagus, Via Natural or Artificial Opening */
"0D538ZZ"  =  "1"  /* Destruction of Lower Esophagus, Via Natural or Artificial Opening Endoscopic */
"0D540ZZ"  =  "1"  /* Destruction of Esophagogastric Junction, Open Approach */
"0D543ZZ"  =  "1"  /* Destruction of Esophagogastric Junction, Percutaneous Approach */
"0D544ZZ"  =  "1"  /* Destruction of Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0D547ZZ"  =  "1"  /* Destruction of Esophagogastric Junction, Via Natural or Artificial Opening */
"0D548ZZ"  =  "1"  /* Destruction of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic */
"0D550ZZ"  =  "1"  /* Destruction of Esophagus, Open Approach */
"0D553ZZ"  =  "1"  /* Destruction of Esophagus, Percutaneous Approach */
"0D554ZZ"  =  "1"  /* Destruction of Esophagus, Percutaneous Endoscopic Approach */
"0D557ZZ"  =  "1"  /* Destruction of Esophagus, Via Natural or Artificial Opening */
"0D558ZZ"  =  "1"  /* Destruction of Esophagus, Via Natural or Artificial Opening Endoscopic */
"0D564ZZ"  =  "1"  /* Destruction of Stomach, Percutaneous Endoscopic Approach */
"0D568ZZ"  =  "1"  /* Destruction of Stomach, Via Natural or Artificial Opening Endoscopic */
"0D574ZZ"  =  "1"  /* Destruction of Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0D578ZZ"  =  "1"  /* Destruction of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0D5E0ZZ"  =  "1"  /* Destruction of Large Intestine, Open Approach */
"0D5E3ZZ"  =  "1"  /* Destruction of Large Intestine, Percutaneous Approach */
"0D5E4ZZ"  =  "1"  /* Destruction of Large Intestine, Percutaneous Endoscopic Approach */
"0D5E7ZZ"  =  "1"  /* Destruction of Large Intestine, Via Natural or Artificial Opening */
"0D5E8ZZ"  =  "1"  /* Destruction of Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0D5F0ZZ"  =  "1"  /* Destruction of Right Large Intestine, Open Approach */
"0D5F3ZZ"  =  "1"  /* Destruction of Right Large Intestine, Percutaneous Approach */
"0D5F4ZZ"  =  "1"  /* Destruction of Right Large Intestine, Percutaneous Endoscopic Approach */
"0D5F7ZZ"  =  "1"  /* Destruction of Right Large Intestine, Via Natural or Artificial Opening */
"0D5F8ZZ"  =  "1"  /* Destruction of Right Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0D5G0ZZ"  =  "1"  /* Destruction of Left Large Intestine, Open Approach */
"0D5G3ZZ"  =  "1"  /* Destruction of Left Large Intestine, Percutaneous Approach */
"0D5G4ZZ"  =  "1"  /* Destruction of Left Large Intestine, Percutaneous Endoscopic Approach */
"0D5G7ZZ"  =  "1"  /* Destruction of Left Large Intestine, Via Natural or Artificial Opening */
"0D5G8ZZ"  =  "1"  /* Destruction of Left Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0D5H0ZZ"  =  "1"  /* Destruction of Cecum, Open Approach */
"0D5H3ZZ"  =  "1"  /* Destruction of Cecum, Percutaneous Approach */
"0D5H4ZZ"  =  "1"  /* Destruction of Cecum, Percutaneous Endoscopic Approach */
"0D5H7ZZ"  =  "1"  /* Destruction of Cecum, Via Natural or Artificial Opening */
"0D5H8ZZ"  =  "1"  /* Destruction of Cecum, Via Natural or Artificial Opening Endoscopic */
"0D5K0ZZ"  =  "1"  /* Destruction of Ascending Colon, Open Approach */
"0D5K3ZZ"  =  "1"  /* Destruction of Ascending Colon, Percutaneous Approach */
"0D5K4ZZ"  =  "1"  /* Destruction of Ascending Colon, Percutaneous Endoscopic Approach */
"0D5K7ZZ"  =  "1"  /* Destruction of Ascending Colon, Via Natural or Artificial Opening */
"0D5K8ZZ"  =  "1"  /* Destruction of Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0D5L0ZZ"  =  "1"  /* Destruction of Transverse Colon, Open Approach */
"0D5L3ZZ"  =  "1"  /* Destruction of Transverse Colon, Percutaneous Approach */
"0D5L4ZZ"  =  "1"  /* Destruction of Transverse Colon, Percutaneous Endoscopic Approach */
"0D5L7ZZ"  =  "1"  /* Destruction of Transverse Colon, Via Natural or Artificial Opening */
"0D5L8ZZ"  =  "1"  /* Destruction of Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D5M0ZZ"  =  "1"  /* Destruction of Descending Colon, Open Approach */
"0D5M3ZZ"  =  "1"  /* Destruction of Descending Colon, Percutaneous Approach */
"0D5M4ZZ"  =  "1"  /* Destruction of Descending Colon, Percutaneous Endoscopic Approach */
"0D5M7ZZ"  =  "1"  /* Destruction of Descending Colon, Via Natural or Artificial Opening */
"0D5M8ZZ"  =  "1"  /* Destruction of Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0D5N0ZZ"  =  "1"  /* Destruction of Sigmoid Colon, Open Approach */
"0D5N3ZZ"  =  "1"  /* Destruction of Sigmoid Colon, Percutaneous Approach */
"0D5N4ZZ"  =  "1"  /* Destruction of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D5N7ZZ"  =  "1"  /* Destruction of Sigmoid Colon, Via Natural or Artificial Opening */
"0D5N8ZZ"  =  "1"  /* Destruction of Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D840ZZ"  =  "1"  /* Division of Esophagogastric Junction, Open Approach */
"0D843ZZ"  =  "1"  /* Division of Esophagogastric Junction, Percutaneous Approach */
"0D844ZZ"  =  "1"  /* Division of Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0D847ZZ"  =  "1"  /* Division of Esophagogastric Junction, Via Natural or Artificial Opening */
"0D848ZZ"  =  "1"  /* Division of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic */
"0D9100Z"  =  "1"  /* Drainage of Upper Esophagus with Drainage Device, Open Approach */
"0D910ZZ"  =  "1"  /* Drainage of Upper Esophagus, Open Approach */
"0D9130Z"  =  "1"  /* Drainage of Upper Esophagus with Drainage Device, Percutaneous Approach */
"0D913ZZ"  =  "1"  /* Drainage of Upper Esophagus, Percutaneous Approach */
"0D9140Z"  =  "1"  /* Drainage of Upper Esophagus with Drainage Device, Percutaneous Endoscopic Approach */
"0D914ZZ"  =  "1"  /* Drainage of Upper Esophagus, Percutaneous Endoscopic Approach */
"0D9170Z"  =  "1"  /* Drainage of Upper Esophagus with Drainage Device, Via Natural or Artificial Opening */
"0D917ZZ"  =  "1"  /* Drainage of Upper Esophagus, Via Natural or Artificial Opening */
"0D9180Z"  =  "1"  /* Drainage of Upper Esophagus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D918ZX"  =  "1"  /* Drainage of Upper Esophagus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0D918ZZ"  =  "1"  /* Drainage of Upper Esophagus, Via Natural or Artificial Opening Endoscopic */
"0D9200Z"  =  "1"  /* Drainage of Middle Esophagus with Drainage Device, Open Approach */
"0D920ZZ"  =  "1"  /* Drainage of Middle Esophagus, Open Approach */
"0D9230Z"  =  "1"  /* Drainage of Middle Esophagus with Drainage Device, Percutaneous Approach */
"0D923ZZ"  =  "1"  /* Drainage of Middle Esophagus, Percutaneous Approach */
"0D9240Z"  =  "1"  /* Drainage of Middle Esophagus with Drainage Device, Percutaneous Endoscopic Approach */
"0D924ZZ"  =  "1"  /* Drainage of Middle Esophagus, Percutaneous Endoscopic Approach */
"0D9270Z"  =  "1"  /* Drainage of Middle Esophagus with Drainage Device, Via Natural or Artificial Opening */
"0D927ZZ"  =  "1"  /* Drainage of Middle Esophagus, Via Natural or Artificial Opening */
"0D9280Z"  =  "1"  /* Drainage of Middle Esophagus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D928ZX"  =  "1"  /* Drainage of Middle Esophagus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0D928ZZ"  =  "1"  /* Drainage of Middle Esophagus, Via Natural or Artificial Opening Endoscopic */
"0D9300Z"  =  "1"  /* Drainage of Lower Esophagus with Drainage Device, Open Approach */
"0D930ZZ"  =  "1"  /* Drainage of Lower Esophagus, Open Approach */
"0D9330Z"  =  "1"  /* Drainage of Lower Esophagus with Drainage Device, Percutaneous Approach */
"0D933ZZ"  =  "1"  /* Drainage of Lower Esophagus, Percutaneous Approach */
"0D9340Z"  =  "1"  /* Drainage of Lower Esophagus with Drainage Device, Percutaneous Endoscopic Approach */
"0D934ZZ"  =  "1"  /* Drainage of Lower Esophagus, Percutaneous Endoscopic Approach */
"0D9370Z"  =  "1"  /* Drainage of Lower Esophagus with Drainage Device, Via Natural or Artificial Opening */
"0D937ZZ"  =  "1"  /* Drainage of Lower Esophagus, Via Natural or Artificial Opening */
"0D9380Z"  =  "1"  /* Drainage of Lower Esophagus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D938ZX"  =  "1"  /* Drainage of Lower Esophagus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0D938ZZ"  =  "1"  /* Drainage of Lower Esophagus, Via Natural or Artificial Opening Endoscopic */
"0D9400Z"  =  "1"  /* Drainage of Esophagogastric Junction with Drainage Device, Open Approach */
"0D940ZZ"  =  "1"  /* Drainage of Esophagogastric Junction, Open Approach */
"0D9430Z"  =  "1"  /* Drainage of Esophagogastric Junction with Drainage Device, Percutaneous Approach */
"0D943ZZ"  =  "1"  /* Drainage of Esophagogastric Junction, Percutaneous Approach */
"0D9440Z"  =  "1"  /* Drainage of Esophagogastric Junction with Drainage Device, Percutaneous Endoscopic Approach */
"0D944ZZ"  =  "1"  /* Drainage of Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0D9470Z"  =  "1"  /* Drainage of Esophagogastric Junction with Drainage Device, Via Natural or Artificial Opening */
"0D947ZZ"  =  "1"  /* Drainage of Esophagogastric Junction, Via Natural or Artificial Opening */
"0D9480Z"  =  "1"  /* Drainage of Esophagogastric Junction with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D948ZX"  =  "1"  /* Drainage of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0D948ZZ"  =  "1"  /* Drainage of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic */
"0D9500Z"  =  "1"  /* Drainage of Esophagus with Drainage Device, Open Approach */
"0D950ZZ"  =  "1"  /* Drainage of Esophagus, Open Approach */
"0D9530Z"  =  "1"  /* Drainage of Esophagus with Drainage Device, Percutaneous Approach */
"0D953ZZ"  =  "1"  /* Drainage of Esophagus, Percutaneous Approach */
"0D9540Z"  =  "1"  /* Drainage of Esophagus with Drainage Device, Percutaneous Endoscopic Approach */
"0D954ZZ"  =  "1"  /* Drainage of Esophagus, Percutaneous Endoscopic Approach */
"0D9570Z"  =  "1"  /* Drainage of Esophagus with Drainage Device, Via Natural or Artificial Opening */
"0D957ZZ"  =  "1"  /* Drainage of Esophagus, Via Natural or Artificial Opening */
"0D9580Z"  =  "1"  /* Drainage of Esophagus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D958ZX"  =  "1"  /* Drainage of Esophagus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0D958ZZ"  =  "1"  /* Drainage of Esophagus, Via Natural or Artificial Opening Endoscopic */
"0D9600Z"  =  "1"  /* Drainage of Stomach with Drainage Device, Open Approach */
"0D960ZZ"  =  "1"  /* Drainage of Stomach, Open Approach */
"0D9630Z"  =  "1"  /* Drainage of Stomach with Drainage Device, Percutaneous Approach */
"0D963ZZ"  =  "1"  /* Drainage of Stomach, Percutaneous Approach */
"0D9640Z"  =  "1"  /* Drainage of Stomach with Drainage Device, Percutaneous Endoscopic Approach */
"0D964ZZ"  =  "1"  /* Drainage of Stomach, Percutaneous Endoscopic Approach */
"0D967ZZ"  =  "1"  /* Drainage of Stomach, Via Natural or Artificial Opening */
"0D968ZX"  =  "1"  /* Drainage of Stomach, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0D968ZZ"  =  "1"  /* Drainage of Stomach, Via Natural or Artificial Opening Endoscopic */
"0D978ZX"  =  "1"  /* Drainage of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0D9800Z"  =  "1"  /* Drainage of Small Intestine with Drainage Device, Open Approach */
"0D980ZZ"  =  "1"  /* Drainage of Small Intestine, Open Approach */
"0D9830Z"  =  "1"  /* Drainage of Small Intestine with Drainage Device, Percutaneous Approach */
"0D983ZZ"  =  "1"  /* Drainage of Small Intestine, Percutaneous Approach */
"0D9840Z"  =  "1"  /* Drainage of Small Intestine with Drainage Device, Percutaneous Endoscopic Approach */
"0D984ZZ"  =  "1"  /* Drainage of Small Intestine, Percutaneous Endoscopic Approach */
"0D9870Z"  =  "1"  /* Drainage of Small Intestine with Drainage Device, Via Natural or Artificial Opening */
"0D987ZZ"  =  "1"  /* Drainage of Small Intestine, Via Natural or Artificial Opening */
"0D9880Z"  =  "1"  /* Drainage of Small Intestine with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D988ZX"  =  "1"  /* Drainage of Small Intestine, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0D988ZZ"  =  "1"  /* Drainage of Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0D9900Z"  =  "1"  /* Drainage of Duodenum with Drainage Device, Open Approach */
"0D990ZZ"  =  "1"  /* Drainage of Duodenum, Open Approach */
"0D9930Z"  =  "1"  /* Drainage of Duodenum with Drainage Device, Percutaneous Approach */
"0D993ZZ"  =  "1"  /* Drainage of Duodenum, Percutaneous Approach */
"0D9940Z"  =  "1"  /* Drainage of Duodenum with Drainage Device, Percutaneous Endoscopic Approach */
"0D994ZZ"  =  "1"  /* Drainage of Duodenum, Percutaneous Endoscopic Approach */
"0D997ZZ"  =  "1"  /* Drainage of Duodenum, Via Natural or Artificial Opening */
"0D998ZX"  =  "1"  /* Drainage of Duodenum, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0D998ZZ"  =  "1"  /* Drainage of Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D9A00Z"  =  "1"  /* Drainage of Jejunum with Drainage Device, Open Approach */
"0D9A0ZZ"  =  "1"  /* Drainage of Jejunum, Open Approach */
"0D9A30Z"  =  "1"  /* Drainage of Jejunum with Drainage Device, Percutaneous Approach */
"0D9A3ZZ"  =  "1"  /* Drainage of Jejunum, Percutaneous Approach */
"0D9A40Z"  =  "1"  /* Drainage of Jejunum with Drainage Device, Percutaneous Endoscopic Approach */
"0D9A4ZZ"  =  "1"  /* Drainage of Jejunum, Percutaneous Endoscopic Approach */
"0D9A7ZZ"  =  "1"  /* Drainage of Jejunum, Via Natural or Artificial Opening */
"0D9A8ZX"  =  "1"  /* Drainage of Jejunum, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0D9A8ZZ"  =  "1"  /* Drainage of Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D9B00Z"  =  "1"  /* Drainage of Ileum with Drainage Device, Open Approach */
"0D9B0ZZ"  =  "1"  /* Drainage of Ileum, Open Approach */
"0D9B30Z"  =  "1"  /* Drainage of Ileum with Drainage Device, Percutaneous Approach */
"0D9B3ZZ"  =  "1"  /* Drainage of Ileum, Percutaneous Approach */
"0D9B40Z"  =  "1"  /* Drainage of Ileum with Drainage Device, Percutaneous Endoscopic Approach */
"0D9B4ZZ"  =  "1"  /* Drainage of Ileum, Percutaneous Endoscopic Approach */
"0D9B7ZZ"  =  "1"  /* Drainage of Ileum, Via Natural or Artificial Opening */
"0D9B8ZZ"  =  "1"  /* Drainage of Ileum, Via Natural or Artificial Opening Endoscopic */
"0D9C00Z"  =  "1"  /* Drainage of Ileocecal Valve with Drainage Device, Open Approach */
"0D9C0ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Open Approach */
"0D9C30Z"  =  "1"  /* Drainage of Ileocecal Valve with Drainage Device, Percutaneous Approach */
"0D9C3ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Percutaneous Approach */
"0D9C40Z"  =  "1"  /* Drainage of Ileocecal Valve with Drainage Device, Percutaneous Endoscopic Approach */
"0D9C4ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Percutaneous Endoscopic Approach */
"0D9C70Z"  =  "1"  /* Drainage of Ileocecal Valve with Drainage Device, Via Natural or Artificial Opening */
"0D9C7ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Via Natural or Artificial Opening */
"0D9C80Z"  =  "1"  /* Drainage of Ileocecal Valve with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D9C8ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Via Natural or Artificial Opening Endoscopic */
"0D9E00Z"  =  "1"  /* Drainage of Large Intestine with Drainage Device, Open Approach */
"0D9E0ZZ"  =  "1"  /* Drainage of Large Intestine, Open Approach */
"0D9E30Z"  =  "1"  /* Drainage of Large Intestine with Drainage Device, Percutaneous Approach */
"0D9E3ZZ"  =  "1"  /* Drainage of Large Intestine, Percutaneous Approach */
"0D9E40Z"  =  "1"  /* Drainage of Large Intestine with Drainage Device, Percutaneous Endoscopic Approach */
"0D9E4ZZ"  =  "1"  /* Drainage of Large Intestine, Percutaneous Endoscopic Approach */
"0D9E70Z"  =  "1"  /* Drainage of Large Intestine with Drainage Device, Via Natural or Artificial Opening */
"0D9E7ZZ"  =  "1"  /* Drainage of Large Intestine, Via Natural or Artificial Opening */
"0D9E80Z"  =  "1"  /* Drainage of Large Intestine with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D9E8ZZ"  =  "1"  /* Drainage of Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0D9F00Z"  =  "1"  /* Drainage of Right Large Intestine with Drainage Device, Open Approach */
"0D9F0ZZ"  =  "1"  /* Drainage of Right Large Intestine, Open Approach */
"0D9F30Z"  =  "1"  /* Drainage of Right Large Intestine with Drainage Device, Percutaneous Approach */
"0D9F3ZZ"  =  "1"  /* Drainage of Right Large Intestine, Percutaneous Approach */
"0D9F40Z"  =  "1"  /* Drainage of Right Large Intestine with Drainage Device, Percutaneous Endoscopic Approach */
"0D9F4ZZ"  =  "1"  /* Drainage of Right Large Intestine, Percutaneous Endoscopic Approach */
"0D9F7ZZ"  =  "1"  /* Drainage of Right Large Intestine, Via Natural or Artificial Opening */
"0D9F8ZZ"  =  "1"  /* Drainage of Right Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0D9G00Z"  =  "1"  /* Drainage of Left Large Intestine with Drainage Device, Open Approach */
"0D9G0ZZ"  =  "1"  /* Drainage of Left Large Intestine, Open Approach */
"0D9G30Z"  =  "1"  /* Drainage of Left Large Intestine with Drainage Device, Percutaneous Approach */
"0D9G3ZZ"  =  "1"  /* Drainage of Left Large Intestine, Percutaneous Approach */
"0D9G40Z"  =  "1"  /* Drainage of Left Large Intestine with Drainage Device, Percutaneous Endoscopic Approach */
"0D9G4ZZ"  =  "1"  /* Drainage of Left Large Intestine, Percutaneous Endoscopic Approach */
"0D9G7ZZ"  =  "1"  /* Drainage of Left Large Intestine, Via Natural or Artificial Opening */
"0D9G8ZZ"  =  "1"  /* Drainage of Left Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0D9H00Z"  =  "1"  /* Drainage of Cecum with Drainage Device, Open Approach */
"0D9H0ZZ"  =  "1"  /* Drainage of Cecum, Open Approach */
"0D9H30Z"  =  "1"  /* Drainage of Cecum with Drainage Device, Percutaneous Approach */
"0D9H3ZZ"  =  "1"  /* Drainage of Cecum, Percutaneous Approach */
"0D9H40Z"  =  "1"  /* Drainage of Cecum with Drainage Device, Percutaneous Endoscopic Approach */
"0D9H4ZZ"  =  "1"  /* Drainage of Cecum, Percutaneous Endoscopic Approach */
"0D9H7ZZ"  =  "1"  /* Drainage of Cecum, Via Natural or Artificial Opening */
"0D9H8ZZ"  =  "1"  /* Drainage of Cecum, Via Natural or Artificial Opening Endoscopic */
"0D9K00Z"  =  "1"  /* Drainage of Ascending Colon with Drainage Device, Open Approach */
"0D9K0ZZ"  =  "1"  /* Drainage of Ascending Colon, Open Approach */
"0D9K30Z"  =  "1"  /* Drainage of Ascending Colon with Drainage Device, Percutaneous Approach */
"0D9K3ZZ"  =  "1"  /* Drainage of Ascending Colon, Percutaneous Approach */
"0D9K40Z"  =  "1"  /* Drainage of Ascending Colon with Drainage Device, Percutaneous Endoscopic Approach */
"0D9K4ZZ"  =  "1"  /* Drainage of Ascending Colon, Percutaneous Endoscopic Approach */
"0D9K7ZZ"  =  "1"  /* Drainage of Ascending Colon, Via Natural or Artificial Opening */
"0D9K8ZZ"  =  "1"  /* Drainage of Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0D9L00Z"  =  "1"  /* Drainage of Transverse Colon with Drainage Device, Open Approach */
"0D9L0ZZ"  =  "1"  /* Drainage of Transverse Colon, Open Approach */
"0D9L30Z"  =  "1"  /* Drainage of Transverse Colon with Drainage Device, Percutaneous Approach */
"0D9L3ZZ"  =  "1"  /* Drainage of Transverse Colon, Percutaneous Approach */
"0D9L40Z"  =  "1"  /* Drainage of Transverse Colon with Drainage Device, Percutaneous Endoscopic Approach */
"0D9L4ZZ"  =  "1"  /* Drainage of Transverse Colon, Percutaneous Endoscopic Approach */
"0D9L7ZZ"  =  "1"  /* Drainage of Transverse Colon, Via Natural or Artificial Opening */
"0D9L8ZZ"  =  "1"  /* Drainage of Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D9M00Z"  =  "1"  /* Drainage of Descending Colon with Drainage Device, Open Approach */
"0D9M0ZZ"  =  "1"  /* Drainage of Descending Colon, Open Approach */
"0D9M30Z"  =  "1"  /* Drainage of Descending Colon with Drainage Device, Percutaneous Approach */
"0D9M3ZZ"  =  "1"  /* Drainage of Descending Colon, Percutaneous Approach */
"0D9M40Z"  =  "1"  /* Drainage of Descending Colon with Drainage Device, Percutaneous Endoscopic Approach */
"0D9M4ZZ"  =  "1"  /* Drainage of Descending Colon, Percutaneous Endoscopic Approach */
"0D9M7ZZ"  =  "1"  /* Drainage of Descending Colon, Via Natural or Artificial Opening */
"0D9M8ZZ"  =  "1"  /* Drainage of Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0D9N00Z"  =  "1"  /* Drainage of Sigmoid Colon with Drainage Device, Open Approach */
"0D9N0ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Open Approach */
"0D9N30Z"  =  "1"  /* Drainage of Sigmoid Colon with Drainage Device, Percutaneous Approach */
"0D9N3ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Percutaneous Approach */
"0D9N40Z"  =  "1"  /* Drainage of Sigmoid Colon with Drainage Device, Percutaneous Endoscopic Approach */
"0D9N4ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D9N7ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Via Natural or Artificial Opening */
"0D9N8ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D9P0ZZ"  =  "1"  /* Drainage of Rectum, Open Approach */
"0D9P3ZZ"  =  "1"  /* Drainage of Rectum, Percutaneous Approach */
"0D9P4ZZ"  =  "1"  /* Drainage of Rectum, Percutaneous Endoscopic Approach */
"0D9P7ZZ"  =  "1"  /* Drainage of Rectum, Via Natural or Artificial Opening */
"0D9P8ZZ"  =  "1"  /* Drainage of Rectum, Via Natural or Artificial Opening Endoscopic */
"0D9S00Z"  =  "1"  /* Drainage of Greater Omentum with Drainage Device, Open Approach */
"0D9S0ZZ"  =  "1"  /* Drainage of Greater Omentum, Open Approach */
"0D9T00Z"  =  "1"  /* Drainage of Lesser Omentum with Drainage Device, Open Approach */
"0D9T0ZZ"  =  "1"  /* Drainage of Lesser Omentum, Open Approach */
"0D9V00Z"  =  "1"  /* Drainage of Mesentery with Drainage Device, Open Approach */
"0D9V0ZZ"  =  "1"  /* Drainage of Mesentery, Open Approach */
"0D9W00Z"  =  "1"  /* Drainage of Peritoneum with Drainage Device, Open Approach */
"0D9W0ZZ"  =  "1"  /* Drainage of Peritoneum, Open Approach */
"0DB10ZZ"  =  "1"  /* Excision of Upper Esophagus, Open Approach */
"0DB13ZZ"  =  "1"  /* Excision of Upper Esophagus, Percutaneous Approach */
"0DB14ZZ"  =  "1"  /* Excision of Upper Esophagus, Percutaneous Endoscopic Approach */
"0DB17ZZ"  =  "1"  /* Excision of Upper Esophagus, Via Natural or Artificial Opening */
"0DB18ZX"  =  "1"  /* Excision of Upper Esophagus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0DB18ZZ"  =  "1"  /* Excision of Upper Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DB20ZZ"  =  "1"  /* Excision of Middle Esophagus, Open Approach */
"0DB23ZZ"  =  "1"  /* Excision of Middle Esophagus, Percutaneous Approach */
"0DB24ZZ"  =  "1"  /* Excision of Middle Esophagus, Percutaneous Endoscopic Approach */
"0DB27ZZ"  =  "1"  /* Excision of Middle Esophagus, Via Natural or Artificial Opening */
"0DB28ZX"  =  "1"  /* Excision of Middle Esophagus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0DB28ZZ"  =  "1"  /* Excision of Middle Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DB30ZZ"  =  "1"  /* Excision of Lower Esophagus, Open Approach */
"0DB33ZZ"  =  "1"  /* Excision of Lower Esophagus, Percutaneous Approach */
"0DB34ZZ"  =  "1"  /* Excision of Lower Esophagus, Percutaneous Endoscopic Approach */
"0DB37ZZ"  =  "1"  /* Excision of Lower Esophagus, Via Natural or Artificial Opening */
"0DB38ZX"  =  "1"  /* Excision of Lower Esophagus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0DB38ZZ"  =  "1"  /* Excision of Lower Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DB40ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Open Approach */
"0DB43ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Percutaneous Approach */
"0DB47ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Via Natural or Artificial Opening */
"0DB48ZX"  =  "1"  /* Excision of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0DB48ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic */
"0DB50ZZ"  =  "1"  /* Excision of Esophagus, Open Approach */
"0DB53ZZ"  =  "1"  /* Excision of Esophagus, Percutaneous Approach */
"0DB54ZZ"  =  "1"  /* Excision of Esophagus, Percutaneous Endoscopic Approach */
"0DB57ZZ"  =  "1"  /* Excision of Esophagus, Via Natural or Artificial Opening */
"0DB58ZX"  =  "1"  /* Excision of Esophagus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0DB58ZZ"  =  "1"  /* Excision of Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DB64ZZ"  =  "1"  /* Excision of Stomach, Percutaneous Endoscopic Approach */
"0DB68ZX"  =  "1"  /* Excision of Stomach, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0DB68ZZ"  =  "1"  /* Excision of Stomach, Via Natural or Artificial Opening Endoscopic */
"0DB74ZZ"  =  "1"  /* Excision of Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0DB78ZX"  =  "1"  /* Excision of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0DB78ZZ"  =  "1"  /* Excision of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0DB88ZX"  =  "1"  /* Excision of Small Intestine, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0DB98ZX"  =  "1"  /* Excision of Duodenum, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0DBA8ZX"  =  "1"  /* Excision of Jejunum, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0DC10ZZ"  =  "1"  /* Extirpation of Matter from Upper Esophagus, Open Approach */
"0DC13ZZ"  =  "1"  /* Extirpation of Matter from Upper Esophagus, Percutaneous Approach */
"0DC14ZZ"  =  "1"  /* Extirpation of Matter from Upper Esophagus, Percutaneous Endoscopic Approach */
"0DC20ZZ"  =  "1"  /* Extirpation of Matter from Middle Esophagus, Open Approach */
"0DC23ZZ"  =  "1"  /* Extirpation of Matter from Middle Esophagus, Percutaneous Approach */
"0DC24ZZ"  =  "1"  /* Extirpation of Matter from Middle Esophagus, Percutaneous Endoscopic Approach */
"0DC30ZZ"  =  "1"  /* Extirpation of Matter from Lower Esophagus, Open Approach */
"0DC33ZZ"  =  "1"  /* Extirpation of Matter from Lower Esophagus, Percutaneous Approach */
"0DC34ZZ"  =  "1"  /* Extirpation of Matter from Lower Esophagus, Percutaneous Endoscopic Approach */
"0DC40ZZ"  =  "1"  /* Extirpation of Matter from Esophagogastric Junction, Open Approach */
"0DC43ZZ"  =  "1"  /* Extirpation of Matter from Esophagogastric Junction, Percutaneous Approach */
"0DC44ZZ"  =  "1"  /* Extirpation of Matter from Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0DC50ZZ"  =  "1"  /* Extirpation of Matter from Esophagus, Open Approach */
"0DC53ZZ"  =  "1"  /* Extirpation of Matter from Esophagus, Percutaneous Approach */
"0DC54ZZ"  =  "1"  /* Extirpation of Matter from Esophagus, Percutaneous Endoscopic Approach */
"0DC60ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Open Approach */
"0DC63ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Percutaneous Approach */
"0DC64ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Percutaneous Endoscopic Approach */
"0DC80ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Open Approach */
"0DC83ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Percutaneous Approach */
"0DC84ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Percutaneous Endoscopic Approach */
"0DC87ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Via Natural or Artificial Opening */
"0DC88ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DC90ZZ"  =  "1"  /* Extirpation of Matter from Duodenum, Open Approach */
"0DC93ZZ"  =  "1"  /* Extirpation of Matter from Duodenum, Percutaneous Approach */
"0DC94ZZ"  =  "1"  /* Extirpation of Matter from Duodenum, Percutaneous Endoscopic Approach */
"0DCA0ZZ"  =  "1"  /* Extirpation of Matter from Jejunum, Open Approach */
"0DCA3ZZ"  =  "1"  /* Extirpation of Matter from Jejunum, Percutaneous Approach */
"0DCA4ZZ"  =  "1"  /* Extirpation of Matter from Jejunum, Percutaneous Endoscopic Approach */
"0DCB0ZZ"  =  "1"  /* Extirpation of Matter from Ileum, Open Approach */
"0DCB3ZZ"  =  "1"  /* Extirpation of Matter from Ileum, Percutaneous Approach */
"0DCB4ZZ"  =  "1"  /* Extirpation of Matter from Ileum, Percutaneous Endoscopic Approach */
"0DCC0ZZ"  =  "1"  /* Extirpation of Matter from Ileocecal Valve, Open Approach */
"0DCC3ZZ"  =  "1"  /* Extirpation of Matter from Ileocecal Valve, Percutaneous Approach */
"0DCC4ZZ"  =  "1"  /* Extirpation of Matter from Ileocecal Valve, Percutaneous Endoscopic Approach */
"0DCE0ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Open Approach */
"0DCE3ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Percutaneous Approach */
"0DCE4ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Percutaneous Endoscopic Approach */
"0DCE7ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Via Natural or Artificial Opening */
"0DCE8ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DCF0ZZ"  =  "1"  /* Extirpation of Matter from Right Large Intestine, Open Approach */
"0DCF3ZZ"  =  "1"  /* Extirpation of Matter from Right Large Intestine, Percutaneous Approach */
"0DCF4ZZ"  =  "1"  /* Extirpation of Matter from Right Large Intestine, Percutaneous Endoscopic Approach */
"0DCG0ZZ"  =  "1"  /* Extirpation of Matter from Left Large Intestine, Open Approach */
"0DCG3ZZ"  =  "1"  /* Extirpation of Matter from Left Large Intestine, Percutaneous Approach */
"0DCG4ZZ"  =  "1"  /* Extirpation of Matter from Left Large Intestine, Percutaneous Endoscopic Approach */
"0DCH0ZZ"  =  "1"  /* Extirpation of Matter from Cecum, Open Approach */
"0DCH3ZZ"  =  "1"  /* Extirpation of Matter from Cecum, Percutaneous Approach */
"0DCH4ZZ"  =  "1"  /* Extirpation of Matter from Cecum, Percutaneous Endoscopic Approach */
"0DCK0ZZ"  =  "1"  /* Extirpation of Matter from Ascending Colon, Open Approach */
"0DCK3ZZ"  =  "1"  /* Extirpation of Matter from Ascending Colon, Percutaneous Approach */
"0DCK4ZZ"  =  "1"  /* Extirpation of Matter from Ascending Colon, Percutaneous Endoscopic Approach */
"0DCL0ZZ"  =  "1"  /* Extirpation of Matter from Transverse Colon, Open Approach */
"0DCL3ZZ"  =  "1"  /* Extirpation of Matter from Transverse Colon, Percutaneous Approach */
"0DCL4ZZ"  =  "1"  /* Extirpation of Matter from Transverse Colon, Percutaneous Endoscopic Approach */
"0DCM0ZZ"  =  "1"  /* Extirpation of Matter from Descending Colon, Open Approach */
"0DCM3ZZ"  =  "1"  /* Extirpation of Matter from Descending Colon, Percutaneous Approach */
"0DCM4ZZ"  =  "1"  /* Extirpation of Matter from Descending Colon, Percutaneous Endoscopic Approach */
"0DCN0ZZ"  =  "1"  /* Extirpation of Matter from Sigmoid Colon, Open Approach */
"0DCN3ZZ"  =  "1"  /* Extirpation of Matter from Sigmoid Colon, Percutaneous Approach */
"0DCN4ZZ"  =  "1"  /* Extirpation of Matter from Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DCP0ZZ"  =  "1"  /* Extirpation of Matter from Rectum, Open Approach */
"0DCP3ZZ"  =  "1"  /* Extirpation of Matter from Rectum, Percutaneous Approach */
"0DCP4ZZ"  =  "1"  /* Extirpation of Matter from Rectum, Percutaneous Endoscopic Approach */
"0DH502Z"  =  "1"  /* Insertion of Monitoring Device into Esophagus, Open Approach */
"0DH503Z"  =  "1"  /* Insertion of Infusion Device into Esophagus, Open Approach */
"0DH532Z"  =  "1"  /* Insertion of Monitoring Device into Esophagus, Percutaneous Approach */
"0DH533Z"  =  "1"  /* Insertion of Infusion Device into Esophagus, Percutaneous Approach */
"0DH542Z"  =  "1"  /* Insertion of Monitoring Device into Esophagus, Percutaneous Endoscopic Approach */
"0DH543Z"  =  "1"  /* Insertion of Infusion Device into Esophagus, Percutaneous Endoscopic Approach */
"0DH572Z"  =  "1"  /* Insertion of Monitoring Device into Esophagus, Via Natural or Artificial Opening */
"0DH573Z"  =  "1"  /* Insertion of Infusion Device into Esophagus, Via Natural or Artificial Opening */
"0DH582Z"  =  "1"  /* Insertion of Monitoring Device into Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DH583Z"  =  "1"  /* Insertion of Infusion Device into Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DH602Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Open Approach */
"0DH603Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Open Approach */
"0DH60UZ"  =  "1"  /* Insertion of Feeding Device into Stomach, Open Approach */
"0DH632Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Percutaneous Approach */
"0DH633Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Percutaneous Approach */
"0DH642Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Percutaneous Endoscopic Approach */
"0DH643Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Percutaneous Endoscopic Approach */
"0DH672Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Via Natural or Artificial Opening */
"0DH673Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Via Natural or Artificial Opening */
"0DH67UZ"  =  "1"  /* Insertion of Feeding Device into Stomach, Via Natural or Artificial Opening */
"0DH682Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Via Natural or Artificial Opening Endoscopic */
"0DH683Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Via Natural or Artificial Opening Endoscopic */
"0DH68UZ"  =  "1"  /* Insertion of Feeding Device into Stomach, Via Natural or Artificial Opening Endoscopic */
"0DH802Z"  =  "1"  /* Insertion of Monitoring Device into Small Intestine, Open Approach */
"0DH803Z"  =  "1"  /* Insertion of Infusion Device into Small Intestine, Open Approach */
"0DH832Z"  =  "1"  /* Insertion of Monitoring Device into Small Intestine, Percutaneous Approach */
"0DH833Z"  =  "1"  /* Insertion of Infusion Device into Small Intestine, Percutaneous Approach */
"0DH842Z"  =  "1"  /* Insertion of Monitoring Device into Small Intestine, Percutaneous Endoscopic Approach */
"0DH843Z"  =  "1"  /* Insertion of Infusion Device into Small Intestine, Percutaneous Endoscopic Approach */
"0DH872Z"  =  "1"  /* Insertion of Monitoring Device into Small Intestine, Via Natural or Artificial Opening */
"0DH873Z"  =  "1"  /* Insertion of Infusion Device into Small Intestine, Via Natural or Artificial Opening */
"0DH882Z"  =  "1"  /* Insertion of Monitoring Device into Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DH883Z"  =  "1"  /* Insertion of Infusion Device into Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DH902Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Open Approach */
"0DH903Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Open Approach */
"0DH932Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Percutaneous Approach */
"0DH933Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Percutaneous Approach */
"0DH942Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Percutaneous Endoscopic Approach */
"0DH943Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Percutaneous Endoscopic Approach */
"0DH972Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Via Natural or Artificial Opening */
"0DH973Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Via Natural or Artificial Opening */
"0DH982Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DH983Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DHA02Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Open Approach */
"0DHA03Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Open Approach */
"0DHA32Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Percutaneous Approach */
"0DHA33Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Percutaneous Approach */
"0DHA42Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Percutaneous Endoscopic Approach */
"0DHA43Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Percutaneous Endoscopic Approach */
"0DHA72Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Via Natural or Artificial Opening */
"0DHA73Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Via Natural or Artificial Opening */
"0DHA82Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DHA83Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DHB02Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Open Approach */
"0DHB03Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Open Approach */
"0DHB32Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Percutaneous Approach */
"0DHB33Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Percutaneous Approach */
"0DHB42Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Percutaneous Endoscopic Approach */
"0DHB43Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Percutaneous Endoscopic Approach */
"0DHB72Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Via Natural or Artificial Opening */
"0DHB73Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Via Natural or Artificial Opening */
"0DHB82Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Via Natural or Artificial Opening Endoscopic */
"0DHB83Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Via Natural or Artificial Opening Endoscopic */
"0DJ00ZZ"  =  "1"  /* Inspection of Upper Intestinal Tract, Open Approach */
"0DJ04ZZ"  =  "1"  /* Inspection of Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DJ07ZZ"  =  "1"  /* Inspection of Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DJ08ZZ"  =  "1"  /* Inspection of Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DJ60ZZ"  =  "1"  /* Inspection of Stomach, Open Approach */
"0DJ64ZZ"  =  "1"  /* Inspection of Stomach, Percutaneous Endoscopic Approach */
"0DJ68ZZ"  =  "1"  /* Inspection of Stomach, Via Natural or Artificial Opening Endoscopic */
"0DJD0ZZ"  =  "1"  /* Inspection of Lower Intestinal Tract, Open Approach */
"0DJD4ZZ"  =  "1"  /* Inspection of Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DJD8ZZ"  =  "1"  /* Inspection of Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DJU0ZZ"  =  "1"  /* Inspection of Omentum, Open Approach */
"0DJU4ZZ"  =  "1"  /* Inspection of Omentum, Percutaneous Endoscopic Approach */
"0DJV0ZZ"  =  "1"  /* Inspection of Mesentery, Open Approach */
"0DJV4ZZ"  =  "1"  /* Inspection of Mesentery, Percutaneous Endoscopic Approach */
"0DJW0ZZ"  =  "1"  /* Inspection of Peritoneum, Open Approach */
"0DJW4ZZ"  =  "1"  /* Inspection of Peritoneum, Percutaneous Endoscopic Approach */
"0DP000Z"  =  "1"  /* Removal of Drainage Device from Upper Intestinal Tract, Open Approach */
"0DP002Z"  =  "1"  /* Removal of Monitoring Device from Upper Intestinal Tract, Open Approach */
"0DP003Z"  =  "1"  /* Removal of Infusion Device from Upper Intestinal Tract, Open Approach */
"0DP007Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Open Approach */
"0DP00CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Intestinal Tract, Open Approach */
"0DP00DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Intestinal Tract, Open Approach */
"0DP00JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Intestinal Tract, Open Approach */
"0DP00KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Open Approach */
"0DP00UZ"  =  "1"  /* Removal of Feeding Device from Upper Intestinal Tract, Open Approach */
"0DP030Z"  =  "1"  /* Removal of Drainage Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP032Z"  =  "1"  /* Removal of Monitoring Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP033Z"  =  "1"  /* Removal of Infusion Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP037Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Percutaneous Approach */
"0DP03CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP03DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP03JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Intestinal Tract, Percutaneous Approach */
"0DP03KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Percutaneous Approach */
"0DP03UZ"  =  "1"  /* Removal of Feeding Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP040Z"  =  "1"  /* Removal of Drainage Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP042Z"  =  "1"  /* Removal of Monitoring Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP043Z"  =  "1"  /* Removal of Infusion Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP047Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP04CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP04DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP04JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP04KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP04UZ"  =  "1"  /* Removal of Feeding Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP070Z"  =  "1"  /* Removal of Drainage Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP072Z"  =  "1"  /* Removal of Monitoring Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP073Z"  =  "1"  /* Removal of Infusion Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP077Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP07CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP07DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP07JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP07KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP07UZ"  =  "1"  /* Removal of Feeding Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP080Z"  =  "1"  /* Removal of Drainage Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP082Z"  =  "1"  /* Removal of Monitoring Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP083Z"  =  "1"  /* Removal of Infusion Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP087Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP08CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP08DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP08JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP08KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP08UZ"  =  "1"  /* Removal of Feeding Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP501Z"  =  "1"  /* Removal of Radioactive Element from Esophagus, Open Approach */
"0DP502Z"  =  "1"  /* Removal of Monitoring Device from Esophagus, Open Approach */
"0DP503Z"  =  "1"  /* Removal of Infusion Device from Esophagus, Open Approach */
"0DP50UZ"  =  "1"  /* Removal of Feeding Device from Esophagus, Open Approach */
"0DP531Z"  =  "1"  /* Removal of Radioactive Element from Esophagus, Percutaneous Approach */
"0DP532Z"  =  "1"  /* Removal of Monitoring Device from Esophagus, Percutaneous Approach */
"0DP533Z"  =  "1"  /* Removal of Infusion Device from Esophagus, Percutaneous Approach */
"0DP53UZ"  =  "1"  /* Removal of Feeding Device from Esophagus, Percutaneous Approach */
"0DP541Z"  =  "1"  /* Removal of Radioactive Element from Esophagus, Percutaneous Endoscopic Approach */
"0DP542Z"  =  "1"  /* Removal of Monitoring Device from Esophagus, Percutaneous Endoscopic Approach */
"0DP543Z"  =  "1"  /* Removal of Infusion Device from Esophagus, Percutaneous Endoscopic Approach */
"0DP54UZ"  =  "1"  /* Removal of Feeding Device from Esophagus, Percutaneous Endoscopic Approach */
"0DP57DZ"  =  "1"  /* Removal of Intraluminal Device from Esophagus, Via Natural or Artificial Opening */
"0DP58DZ"  =  "1"  /* Removal of Intraluminal Device from Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DP600Z"  =  "1"  /* Removal of Drainage Device from Stomach, Open Approach */
"0DP602Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Open Approach */
"0DP603Z"  =  "1"  /* Removal of Infusion Device from Stomach, Open Approach */
"0DP607Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Stomach, Open Approach */
"0DP60CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Open Approach */
"0DP60DZ"  =  "1"  /* Removal of Intraluminal Device from Stomach, Open Approach */
"0DP60JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Open Approach */
"0DP60KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Stomach, Open Approach */
"0DP60UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Open Approach */
"0DP630Z"  =  "1"  /* Removal of Drainage Device from Stomach, Percutaneous Approach */
"0DP632Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Percutaneous Approach */
"0DP633Z"  =  "1"  /* Removal of Infusion Device from Stomach, Percutaneous Approach */
"0DP637Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Stomach, Percutaneous Approach */
"0DP63CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Percutaneous Approach */
"0DP63DZ"  =  "1"  /* Removal of Intraluminal Device from Stomach, Percutaneous Approach */
"0DP63JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Percutaneous Approach */
"0DP63KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Stomach, Percutaneous Approach */
"0DP63UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Percutaneous Approach */
"0DP640Z"  =  "1"  /* Removal of Drainage Device from Stomach, Percutaneous Endoscopic Approach */
"0DP642Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Percutaneous Endoscopic Approach */
"0DP647Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Stomach, Percutaneous Endoscopic Approach */
"0DP64DZ"  =  "1"  /* Removal of Intraluminal Device from Stomach, Percutaneous Endoscopic Approach */
"0DP64JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Percutaneous Endoscopic Approach */
"0DP64KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Stomach, Percutaneous Endoscopic Approach */
"0DP64UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Percutaneous Endoscopic Approach */
"0DP670Z"  =  "1"  /* Removal of Drainage Device from Stomach, Via Natural or Artificial Opening */
"0DP672Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Via Natural or Artificial Opening */
"0DP673Z"  =  "1"  /* Removal of Infusion Device from Stomach, Via Natural or Artificial Opening */
"0DP677Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Stomach, Via Natural or Artificial Opening */
"0DP67CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Via Natural or Artificial Opening */
"0DP67JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Via Natural or Artificial Opening */
"0DP67KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Stomach, Via Natural or Artificial Opening */
"0DP67UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Via Natural or Artificial Opening */
"0DP680Z"  =  "1"  /* Removal of Drainage Device from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP682Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP683Z"  =  "1"  /* Removal of Infusion Device from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP687Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP68CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP68JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP68KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP68UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DPD00Z"  =  "1"  /* Removal of Drainage Device from Lower Intestinal Tract, Open Approach */
"0DPD02Z"  =  "1"  /* Removal of Monitoring Device from Lower Intestinal Tract, Open Approach */
"0DPD03Z"  =  "1"  /* Removal of Infusion Device from Lower Intestinal Tract, Open Approach */
"0DPD07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Open Approach */
"0DPD0CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Intestinal Tract, Open Approach */
"0DPD0DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Intestinal Tract, Open Approach */
"0DPD0JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Intestinal Tract, Open Approach */
"0DPD0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Open Approach */
"0DPD0UZ"  =  "1"  /* Removal of Feeding Device from Lower Intestinal Tract, Open Approach */
"0DPD30Z"  =  "1"  /* Removal of Drainage Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD32Z"  =  "1"  /* Removal of Monitoring Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD33Z"  =  "1"  /* Removal of Infusion Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Percutaneous Approach */
"0DPD3CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD3DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD3JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Intestinal Tract, Percutaneous Approach */
"0DPD3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Percutaneous Approach */
"0DPD3UZ"  =  "1"  /* Removal of Feeding Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD40Z"  =  "1"  /* Removal of Drainage Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD42Z"  =  "1"  /* Removal of Monitoring Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD43Z"  =  "1"  /* Removal of Infusion Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD4CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD4DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD4JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD4UZ"  =  "1"  /* Removal of Feeding Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD70Z"  =  "1"  /* Removal of Drainage Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD72Z"  =  "1"  /* Removal of Monitoring Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD73Z"  =  "1"  /* Removal of Infusion Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD7CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD7DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD7JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD7UZ"  =  "1"  /* Removal of Feeding Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD80Z"  =  "1"  /* Removal of Drainage Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD82Z"  =  "1"  /* Removal of Monitoring Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD83Z"  =  "1"  /* Removal of Infusion Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD8CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD8DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD8JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD8UZ"  =  "1"  /* Removal of Feeding Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPP01Z"  =  "1"  /* Removal of Radioactive Element from Rectum, Open Approach */
"0DPP31Z"  =  "1"  /* Removal of Radioactive Element from Rectum, Percutaneous Approach */
"0DPP41Z"  =  "1"  /* Removal of Radioactive Element from Rectum, Percutaneous Endoscopic Approach */
"0DPU00Z"  =  "1"  /* Removal of Drainage Device from Omentum, Open Approach */
"0DPU01Z"  =  "1"  /* Removal of Radioactive Element from Omentum, Open Approach */
"0DPU07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Omentum, Open Approach */
"0DPU0JZ"  =  "1"  /* Removal of Synthetic Substitute from Omentum, Open Approach */
"0DPU0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Omentum, Open Approach */
"0DPU30Z"  =  "1"  /* Removal of Drainage Device from Omentum, Percutaneous Approach */
"0DPU31Z"  =  "1"  /* Removal of Radioactive Element from Omentum, Percutaneous Approach */
"0DPU37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Omentum, Percutaneous Approach */
"0DPU3JZ"  =  "1"  /* Removal of Synthetic Substitute from Omentum, Percutaneous Approach */
"0DPU3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Omentum, Percutaneous Approach */
"0DPU40Z"  =  "1"  /* Removal of Drainage Device from Omentum, Percutaneous Endoscopic Approach */
"0DPU41Z"  =  "1"  /* Removal of Radioactive Element from Omentum, Percutaneous Endoscopic Approach */
"0DPU47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Omentum, Percutaneous Endoscopic Approach */
"0DPU4JZ"  =  "1"  /* Removal of Synthetic Substitute from Omentum, Percutaneous Endoscopic Approach */
"0DPU4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Omentum, Percutaneous Endoscopic Approach */
"0DPV00Z"  =  "1"  /* Removal of Drainage Device from Mesentery, Open Approach */
"0DPV01Z"  =  "1"  /* Removal of Radioactive Element from Mesentery, Open Approach */
"0DPV07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mesentery, Open Approach */
"0DPV0JZ"  =  "1"  /* Removal of Synthetic Substitute from Mesentery, Open Approach */
"0DPV0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mesentery, Open Approach */
"0DPV30Z"  =  "1"  /* Removal of Drainage Device from Mesentery, Percutaneous Approach */
"0DPV31Z"  =  "1"  /* Removal of Radioactive Element from Mesentery, Percutaneous Approach */
"0DPV37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mesentery, Percutaneous Approach */
"0DPV3JZ"  =  "1"  /* Removal of Synthetic Substitute from Mesentery, Percutaneous Approach */
"0DPV3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mesentery, Percutaneous Approach */
"0DPV40Z"  =  "1"  /* Removal of Drainage Device from Mesentery, Percutaneous Endoscopic Approach */
"0DPV41Z"  =  "1"  /* Removal of Radioactive Element from Mesentery, Percutaneous Endoscopic Approach */
"0DPV47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mesentery, Percutaneous Endoscopic Approach */
"0DPV4JZ"  =  "1"  /* Removal of Synthetic Substitute from Mesentery, Percutaneous Endoscopic Approach */
"0DPV4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mesentery, Percutaneous Endoscopic Approach */
"0DPW00Z"  =  "1"  /* Removal of Drainage Device from Peritoneum, Open Approach */
"0DPW01Z"  =  "1"  /* Removal of Radioactive Element from Peritoneum, Open Approach */
"0DPW07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Peritoneum, Open Approach */
"0DPW0JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneum, Open Approach */
"0DPW0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Peritoneum, Open Approach */
"0DPW30Z"  =  "1"  /* Removal of Drainage Device from Peritoneum, Percutaneous Approach */
"0DPW31Z"  =  "1"  /* Removal of Radioactive Element from Peritoneum, Percutaneous Approach */
"0DPW37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Peritoneum, Percutaneous Approach */
"0DPW3JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneum, Percutaneous Approach */
"0DPW3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Peritoneum, Percutaneous Approach */
"0DPW40Z"  =  "1"  /* Removal of Drainage Device from Peritoneum, Percutaneous Endoscopic Approach */
"0DPW41Z"  =  "1"  /* Removal of Radioactive Element from Peritoneum, Percutaneous Endoscopic Approach */
"0DPW47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Peritoneum, Percutaneous Endoscopic Approach */
"0DPW4JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneum, Percutaneous Endoscopic Approach */
"0DPW4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Peritoneum, Percutaneous Endoscopic Approach */
"0DQ60ZZ"  =  "1"  /* Repair Stomach, Open Approach */
"0DQ63ZZ"  =  "1"  /* Repair Stomach, Percutaneous Approach */
"0DQ64ZZ"  =  "1"  /* Repair Stomach, Percutaneous Endoscopic Approach */
"0DQ67ZZ"  =  "1"  /* Repair Stomach, Via Natural or Artificial Opening */
"0DQ68ZZ"  =  "1"  /* Repair Stomach, Via Natural or Artificial Opening Endoscopic */
"0DQ90ZZ"  =  "1"  /* Repair Duodenum, Open Approach */
"0DQ93ZZ"  =  "1"  /* Repair Duodenum, Percutaneous Approach */
"0DQ94ZZ"  =  "1"  /* Repair Duodenum, Percutaneous Endoscopic Approach */
"0DQ97ZZ"  =  "1"  /* Repair Duodenum, Via Natural or Artificial Opening */
"0DQ98ZZ"  =  "1"  /* Repair Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DQQ0ZZ"  =  "1"  /* Repair Anus, Open Approach */
"0DQQ3ZZ"  =  "1"  /* Repair Anus, Percutaneous Approach */
"0DQQ4ZZ"  =  "1"  /* Repair Anus, Percutaneous Endoscopic Approach */
"0DQQ7ZZ"  =  "1"  /* Repair Anus, Via Natural or Artificial Opening */
"0DQQ8ZZ"  =  "1"  /* Repair Anus, Via Natural or Artificial Opening Endoscopic */
"0DQQXZZ"  =  "1"  /* Repair Anus, External Approach */
"0DW000Z"  =  "1"  /* Revision of Drainage Device in Upper Intestinal Tract, Open Approach */
"0DW002Z"  =  "1"  /* Revision of Monitoring Device in Upper Intestinal Tract, Open Approach */
"0DW003Z"  =  "1"  /* Revision of Infusion Device in Upper Intestinal Tract, Open Approach */
"0DW007Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Open Approach */
"0DW00CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Intestinal Tract, Open Approach */
"0DW00DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Intestinal Tract, Open Approach */
"0DW00JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Intestinal Tract, Open Approach */
"0DW00KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Open Approach */
"0DW00UZ"  =  "1"  /* Revision of Feeding Device in Upper Intestinal Tract, Open Approach */
"0DW030Z"  =  "1"  /* Revision of Drainage Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW032Z"  =  "1"  /* Revision of Monitoring Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW033Z"  =  "1"  /* Revision of Infusion Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW037Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Percutaneous Approach */
"0DW03CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW03DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW03JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Intestinal Tract, Percutaneous Approach */
"0DW03KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Percutaneous Approach */
"0DW03UZ"  =  "1"  /* Revision of Feeding Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW040Z"  =  "1"  /* Revision of Drainage Device in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW042Z"  =  "1"  /* Revision of Monitoring Device in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW043Z"  =  "1"  /* Revision of Infusion Device in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW047Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW04CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW04DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW04JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW04KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW070Z"  =  "1"  /* Revision of Drainage Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW072Z"  =  "1"  /* Revision of Monitoring Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW073Z"  =  "1"  /* Revision of Infusion Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW077Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW07CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW07DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW07JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW07KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW07UZ"  =  "1"  /* Revision of Feeding Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW080Z"  =  "1"  /* Revision of Drainage Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW082Z"  =  "1"  /* Revision of Monitoring Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW083Z"  =  "1"  /* Revision of Infusion Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW087Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW08CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW08DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW08JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW08KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW08UZ"  =  "1"  /* Revision of Feeding Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW57DZ"  =  "1"  /* Revision of Intraluminal Device in Esophagus, Via Natural or Artificial Opening */
"0DW58DZ"  =  "1"  /* Revision of Intraluminal Device in Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DW600Z"  =  "1"  /* Revision of Drainage Device in Stomach, Open Approach */
"0DW602Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Open Approach */
"0DW603Z"  =  "1"  /* Revision of Infusion Device in Stomach, Open Approach */
"0DW607Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Stomach, Open Approach */
"0DW60CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Open Approach */
"0DW60DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Open Approach */
"0DW60JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Open Approach */
"0DW60KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Stomach, Open Approach */
"0DW60MZ"  =  "1"  /* Revision of Stimulator Lead in Stomach, Open Approach */
"0DW60UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Open Approach */
"0DW630Z"  =  "1"  /* Revision of Drainage Device in Stomach, Percutaneous Approach */
"0DW632Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Percutaneous Approach */
"0DW633Z"  =  "1"  /* Revision of Infusion Device in Stomach, Percutaneous Approach */
"0DW637Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Stomach, Percutaneous Approach */
"0DW63CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Percutaneous Approach */
"0DW63DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Percutaneous Approach */
"0DW63JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Percutaneous Approach */
"0DW63KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Stomach, Percutaneous Approach */
"0DW63MZ"  =  "1"  /* Revision of Stimulator Lead in Stomach, Percutaneous Approach */
"0DW63UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Percutaneous Approach */
"0DW640Z"  =  "1"  /* Revision of Drainage Device in Stomach, Percutaneous Endoscopic Approach */
"0DW642Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Percutaneous Endoscopic Approach */
"0DW647Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Stomach, Percutaneous Endoscopic Approach */
"0DW64DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Percutaneous Endoscopic Approach */
"0DW64JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Percutaneous Endoscopic Approach */
"0DW64KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Stomach, Percutaneous Endoscopic Approach */
"0DW64MZ"  =  "1"  /* Revision of Stimulator Lead in Stomach, Percutaneous Endoscopic Approach */
"0DW64UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Percutaneous Endoscopic Approach */
"0DW670Z"  =  "1"  /* Revision of Drainage Device in Stomach, Via Natural or Artificial Opening */
"0DW672Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Via Natural or Artificial Opening */
"0DW673Z"  =  "1"  /* Revision of Infusion Device in Stomach, Via Natural or Artificial Opening */
"0DW677Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Stomach, Via Natural or Artificial Opening */
"0DW67CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Via Natural or Artificial Opening */
"0DW67DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Via Natural or Artificial Opening */
"0DW67JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Via Natural or Artificial Opening */
"0DW67KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Stomach, Via Natural or Artificial Opening */
"0DW67UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Via Natural or Artificial Opening */
"0DW680Z"  =  "1"  /* Revision of Drainage Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW682Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW683Z"  =  "1"  /* Revision of Infusion Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW687Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW68CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW68DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW68JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW68KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW68UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DWD00Z"  =  "1"  /* Revision of Drainage Device in Lower Intestinal Tract, Open Approach */
"0DWD02Z"  =  "1"  /* Revision of Monitoring Device in Lower Intestinal Tract, Open Approach */
"0DWD03Z"  =  "1"  /* Revision of Infusion Device in Lower Intestinal Tract, Open Approach */
"0DWD07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Open Approach */
"0DWD0CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Intestinal Tract, Open Approach */
"0DWD0DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Intestinal Tract, Open Approach */
"0DWD0JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Intestinal Tract, Open Approach */
"0DWD0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Open Approach */
"0DWD0UZ"  =  "1"  /* Revision of Feeding Device in Lower Intestinal Tract, Open Approach */
"0DWD30Z"  =  "1"  /* Revision of Drainage Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD32Z"  =  "1"  /* Revision of Monitoring Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD33Z"  =  "1"  /* Revision of Infusion Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Percutaneous Approach */
"0DWD3CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD3DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD3JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Intestinal Tract, Percutaneous Approach */
"0DWD3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Percutaneous Approach */
"0DWD3UZ"  =  "1"  /* Revision of Feeding Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD40Z"  =  "1"  /* Revision of Drainage Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD42Z"  =  "1"  /* Revision of Monitoring Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD43Z"  =  "1"  /* Revision of Infusion Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD4CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD4DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD4JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD4UZ"  =  "1"  /* Revision of Feeding Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD70Z"  =  "1"  /* Revision of Drainage Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD72Z"  =  "1"  /* Revision of Monitoring Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD73Z"  =  "1"  /* Revision of Infusion Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD7CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD7DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD7JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD7UZ"  =  "1"  /* Revision of Feeding Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD80Z"  =  "1"  /* Revision of Drainage Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD82Z"  =  "1"  /* Revision of Monitoring Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD83Z"  =  "1"  /* Revision of Infusion Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD8CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD8DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD8JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD8UZ"  =  "1"  /* Revision of Feeding Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWU07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Omentum, Open Approach */
"0DWU0JZ"  =  "1"  /* Revision of Synthetic Substitute in Omentum, Open Approach */
"0DWU0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Omentum, Open Approach */
"0DWU37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Omentum, Percutaneous Approach */
"0DWU3JZ"  =  "1"  /* Revision of Synthetic Substitute in Omentum, Percutaneous Approach */
"0DWU3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Omentum, Percutaneous Approach */
"0DWU47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Omentum, Percutaneous Endoscopic Approach */
"0DWU4JZ"  =  "1"  /* Revision of Synthetic Substitute in Omentum, Percutaneous Endoscopic Approach */
"0DWU4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Omentum, Percutaneous Endoscopic Approach */
"0DWV07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mesentery, Open Approach */
"0DWV0JZ"  =  "1"  /* Revision of Synthetic Substitute in Mesentery, Open Approach */
"0DWV0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mesentery, Open Approach */
"0DWV37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mesentery, Percutaneous Approach */
"0DWV3JZ"  =  "1"  /* Revision of Synthetic Substitute in Mesentery, Percutaneous Approach */
"0DWV3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mesentery, Percutaneous Approach */
"0DWV47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mesentery, Percutaneous Endoscopic Approach */
"0DWV4JZ"  =  "1"  /* Revision of Synthetic Substitute in Mesentery, Percutaneous Endoscopic Approach */
"0DWV4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mesentery, Percutaneous Endoscopic Approach */
"0DWW07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Peritoneum, Open Approach */
"0DWW0JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneum, Open Approach */
"0DWW0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Peritoneum, Open Approach */
"0DWW37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Peritoneum, Percutaneous Approach */
"0DWW3JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneum, Percutaneous Approach */
"0DWW3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Peritoneum, Percutaneous Approach */
"0DWW47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Peritoneum, Percutaneous Endoscopic Approach */
"0DWW4JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneum, Percutaneous Endoscopic Approach */
"0DWW4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Peritoneum, Percutaneous Endoscopic Approach */
"0F753DZ"  =  "1"  /* Dilation of Right Hepatic Duct with Intraluminal Device, Percutaneous Approach */
"0F753ZZ"  =  "1"  /* Dilation of Right Hepatic Duct, Percutaneous Approach */
"0F754ZZ"  =  "1"  /* Dilation of Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0F758ZZ"  =  "1"  /* Dilation of Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0F763DZ"  =  "1"  /* Dilation of Left Hepatic Duct with Intraluminal Device, Percutaneous Approach */
"0F763ZZ"  =  "1"  /* Dilation of Left Hepatic Duct, Percutaneous Approach */
"0F764ZZ"  =  "1"  /* Dilation of Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0F768ZZ"  =  "1"  /* Dilation of Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0F783DZ"  =  "1"  /* Dilation of Cystic Duct with Intraluminal Device, Percutaneous Approach */
"0F783ZZ"  =  "1"  /* Dilation of Cystic Duct, Percutaneous Approach */
"0F784ZZ"  =  "1"  /* Dilation of Cystic Duct, Percutaneous Endoscopic Approach */
"0F788ZZ"  =  "1"  /* Dilation of Cystic Duct, Via Natural or Artificial Opening Endoscopic */
"0F793DZ"  =  "1"  /* Dilation of Common Bile Duct with Intraluminal Device, Percutaneous Approach */
"0F793ZZ"  =  "1"  /* Dilation of Common Bile Duct, Percutaneous Approach */
"0F794ZZ"  =  "1"  /* Dilation of Common Bile Duct, Percutaneous Endoscopic Approach */
"0F798ZZ"  =  "1"  /* Dilation of Common Bile Duct, Via Natural or Artificial Opening Endoscopic */
"0F7C8DZ"  =  "1"  /* Dilation of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0F7C8ZZ"  =  "1"  /* Dilation of Ampulla of Vater, Via Natural or Artificial Opening Endoscopic */
"0F8G4ZZ"  =  "1"  /* Division of Pancreas, Percutaneous Endoscopic Approach */
"0F9000Z"  =  "1"  /* Drainage of Liver with Drainage Device, Open Approach */
"0F900ZZ"  =  "1"  /* Drainage of Liver, Open Approach */
"0F9100Z"  =  "1"  /* Drainage of Right Lobe Liver with Drainage Device, Open Approach */
"0F910ZZ"  =  "1"  /* Drainage of Right Lobe Liver, Open Approach */
"0F9200Z"  =  "1"  /* Drainage of Left Lobe Liver with Drainage Device, Open Approach */
"0F920ZZ"  =  "1"  /* Drainage of Left Lobe Liver, Open Approach */
"0F9500Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drainage Device, Open Approach */
"0F950ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Open Approach */
"0F9530Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drainage Device, Percutaneous Approach */
"0F953ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Percutaneous Approach */
"0F9540Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drainage Device, Percutaneous Endoscopic Approach */
"0F954ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0F9570Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drainage Device, Via Natural or Artificial Opening */
"0F957ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Via Natural or Artificial Opening */
"0F9580Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0F958ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0F9600Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drainage Device, Open Approach */
"0F960ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Open Approach */
"0F9630Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drainage Device, Percutaneous Approach */
"0F963ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Percutaneous Approach */
"0F9640Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drainage Device, Percutaneous Endoscopic Approach */
"0F964ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0F9670Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drainage Device, Via Natural or Artificial Opening */
"0F967ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Via Natural or Artificial Opening */
"0F9680Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0F968ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0F9800Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Open Approach */
"0F980ZZ"  =  "1"  /* Drainage of Cystic Duct, Open Approach */
"0F9830Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Percutaneous Approach */
"0F983ZZ"  =  "1"  /* Drainage of Cystic Duct, Percutaneous Approach */
"0F9840Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Percutaneous Endoscopic Approach */
"0F984ZZ"  =  "1"  /* Drainage of Cystic Duct, Percutaneous Endoscopic Approach */
"0F9870Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Via Natural or Artificial Opening */
"0F987ZZ"  =  "1"  /* Drainage of Cystic Duct, Via Natural or Artificial Opening */
"0F9880Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0F988ZZ"  =  "1"  /* Drainage of Cystic Duct, Via Natural or Artificial Opening Endoscopic */
"0F9900Z"  =  "1"  /* Drainage of Common Bile Duct with Drainage Device, Open Approach */
"0F990ZZ"  =  "1"  /* Drainage of Common Bile Duct, Open Approach */
"0F9930Z"  =  "1"  /* Drainage of Common Bile Duct with Drainage Device, Percutaneous Approach */
"0F993ZZ"  =  "1"  /* Drainage of Common Bile Duct, Percutaneous Approach */
"0F9940Z"  =  "1"  /* Drainage of Common Bile Duct with Drainage Device, Percutaneous Endoscopic Approach */
"0F994ZZ"  =  "1"  /* Drainage of Common Bile Duct, Percutaneous Endoscopic Approach */
"0F997ZZ"  =  "1"  /* Drainage of Common Bile Duct, Via Natural or Artificial Opening */
"0F998ZZ"  =  "1"  /* Drainage of Common Bile Duct, Via Natural or Artificial Opening Endoscopic */
"0F9D0ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Open Approach */
"0F9D3ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Percutaneous Approach */
"0F9D4ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Percutaneous Endoscopic Approach */
"0F9D7ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Via Natural or Artificial Opening */
"0F9D8ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0F9F0ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Open Approach */
"0F9F3ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Percutaneous Approach */
"0F9F4ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0F9F7ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0F9F8ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0F9G0ZZ"  =  "1"  /* Drainage of Pancreas, Open Approach */
"0F9G3ZZ"  =  "1"  /* Drainage of Pancreas, Percutaneous Approach */
"0F9G4ZZ"  =  "1"  /* Drainage of Pancreas, Percutaneous Endoscopic Approach */
"0FC00ZZ"  =  "1"  /* Extirpation of Matter from Liver, Open Approach */
"0FC03ZZ"  =  "1"  /* Extirpation of Matter from Liver, Percutaneous Approach */
"0FC04ZZ"  =  "1"  /* Extirpation of Matter from Liver, Percutaneous Endoscopic Approach */
"0FC10ZZ"  =  "1"  /* Extirpation of Matter from Right Lobe Liver, Open Approach */
"0FC13ZZ"  =  "1"  /* Extirpation of Matter from Right Lobe Liver, Percutaneous Approach */
"0FC14ZZ"  =  "1"  /* Extirpation of Matter from Right Lobe Liver, Percutaneous Endoscopic Approach */
"0FC20ZZ"  =  "1"  /* Extirpation of Matter from Left Lobe Liver, Open Approach */
"0FC23ZZ"  =  "1"  /* Extirpation of Matter from Left Lobe Liver, Percutaneous Approach */
"0FC24ZZ"  =  "1"  /* Extirpation of Matter from Left Lobe Liver, Percutaneous Endoscopic Approach */
"0FC50ZZ"  =  "1"  /* Extirpation of Matter from Right Hepatic Duct, Open Approach */
"0FC53ZZ"  =  "1"  /* Extirpation of Matter from Right Hepatic Duct, Percutaneous Approach */
"0FC54ZZ"  =  "1"  /* Extirpation of Matter from Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0FC57ZZ"  =  "1"  /* Extirpation of Matter from Right Hepatic Duct, Via Natural or Artificial Opening */
"0FC58ZZ"  =  "1"  /* Extirpation of Matter from Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FC60ZZ"  =  "1"  /* Extirpation of Matter from Left Hepatic Duct, Open Approach */
"0FC63ZZ"  =  "1"  /* Extirpation of Matter from Left Hepatic Duct, Percutaneous Approach */
"0FC64ZZ"  =  "1"  /* Extirpation of Matter from Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0FC67ZZ"  =  "1"  /* Extirpation of Matter from Left Hepatic Duct, Via Natural or Artificial Opening */
"0FC68ZZ"  =  "1"  /* Extirpation of Matter from Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FC80ZZ"  =  "1"  /* Extirpation of Matter from Cystic Duct, Open Approach */
"0FC83ZZ"  =  "1"  /* Extirpation of Matter from Cystic Duct, Percutaneous Approach */
"0FC84ZZ"  =  "1"  /* Extirpation of Matter from Cystic Duct, Percutaneous Endoscopic Approach */
"0FC87ZZ"  =  "1"  /* Extirpation of Matter from Cystic Duct, Via Natural or Artificial Opening */
"0FC88ZZ"  =  "1"  /* Extirpation of Matter from Cystic Duct, Via Natural or Artificial Opening Endoscopic */
"0FC90ZZ"  =  "1"  /* Extirpation of Matter from Common Bile Duct, Open Approach */
"0FC93ZZ"  =  "1"  /* Extirpation of Matter from Common Bile Duct, Percutaneous Approach */
"0FC94ZZ"  =  "1"  /* Extirpation of Matter from Common Bile Duct, Percutaneous Endoscopic Approach */
"0FC97ZZ"  =  "1"  /* Extirpation of Matter from Common Bile Duct, Via Natural or Artificial Opening */
"0FC98ZZ"  =  "1"  /* Extirpation of Matter from Common Bile Duct, Via Natural or Artificial Opening Endoscopic */
"0FCC4ZZ"  =  "1"  /* Extirpation of Matter from Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FCC8ZZ"  =  "1"  /* Extirpation of Matter from Ampulla of Vater, Via Natural or Artificial Opening Endoscopic */
"0FCD0ZZ"  =  "1"  /* Extirpation of Matter from Pancreatic Duct, Open Approach */
"0FCD7ZZ"  =  "1"  /* Extirpation of Matter from Pancreatic Duct, Via Natural or Artificial Opening */
"0FCF0ZZ"  =  "1"  /* Extirpation of Matter from Accessory Pancreatic Duct, Open Approach */
"0FCF7ZZ"  =  "1"  /* Extirpation of Matter from Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FCG0ZZ"  =  "1"  /* Extirpation of Matter from Pancreas, Open Approach */
"0FCG3ZZ"  =  "1"  /* Extirpation of Matter from Pancreas, Percutaneous Approach */
"0FCG4ZZ"  =  "1"  /* Extirpation of Matter from Pancreas, Percutaneous Endoscopic Approach */
"0FF48ZZ"  =  "1"  /* Fragmentation in Gallbladder, Via Natural or Artificial Opening Endoscopic */
"0FF50ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Open Approach */
"0FF53ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Percutaneous Approach */
"0FF54ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0FF57ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Via Natural or Artificial Opening */
"0FF58ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FF60ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Open Approach */
"0FF63ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Percutaneous Approach */
"0FF64ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0FF67ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Via Natural or Artificial Opening */
"0FF68ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FF80ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Open Approach */
"0FF83ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Percutaneous Approach */
"0FF84ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Percutaneous Endoscopic Approach */
"0FF87ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Via Natural or Artificial Opening */
"0FF88ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Via Natural or Artificial Opening Endoscopic */
"0FF90ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Open Approach */
"0FF93ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Percutaneous Approach */
"0FF94ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Percutaneous Endoscopic Approach */
"0FF97ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Via Natural or Artificial Opening */
"0FF98ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Via Natural or Artificial Opening Endoscopic */
"0FFC0ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Open Approach */
"0FFC3ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Percutaneous Approach */
"0FFC4ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FFC7ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Via Natural or Artificial Opening */
"0FFC8ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Via Natural or Artificial Opening Endoscopic */
"0FFD0ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Open Approach */
"0FFD3ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Percutaneous Approach */
"0FFD4ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FFD7ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Via Natural or Artificial Opening */
"0FFD8ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FFF0ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Open Approach */
"0FFF3ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Percutaneous Approach */
"0FFF4ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FFF7ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FFF8ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FH002Z"  =  "1"  /* Insertion of Monitoring Device into Liver, Open Approach */
"0FH032Z"  =  "1"  /* Insertion of Monitoring Device into Liver, Percutaneous Approach */
"0FH042Z"  =  "1"  /* Insertion of Monitoring Device into Liver, Percutaneous Endoscopic Approach */
"0FH102Z"  =  "1"  /* Insertion of Monitoring Device into Right Lobe Liver, Open Approach */
"0FH132Z"  =  "1"  /* Insertion of Monitoring Device into Right Lobe Liver, Percutaneous Approach */
"0FH142Z"  =  "1"  /* Insertion of Monitoring Device into Right Lobe Liver, Percutaneous Endoscopic Approach */
"0FH202Z"  =  "1"  /* Insertion of Monitoring Device into Left Lobe Liver, Open Approach */
"0FH232Z"  =  "1"  /* Insertion of Monitoring Device into Left Lobe Liver, Percutaneous Approach */
"0FH242Z"  =  "1"  /* Insertion of Monitoring Device into Left Lobe Liver, Percutaneous Endoscopic Approach */
"0FHB02Z"  =  "1"  /* Insertion of Monitoring Device into Hepatobiliary Duct, Open Approach */
"0FHB0DZ"  =  "1"  /* Insertion of Intraluminal Device into Hepatobiliary Duct, Open Approach */
"0FHB32Z"  =  "1"  /* Insertion of Monitoring Device into Hepatobiliary Duct, Percutaneous Approach */
"0FHB3DZ"  =  "1"  /* Insertion of Intraluminal Device into Hepatobiliary Duct, Percutaneous Approach */
"0FHB42Z"  =  "1"  /* Insertion of Monitoring Device into Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FHB72Z"  =  "1"  /* Insertion of Monitoring Device into Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FHB7DZ"  =  "1"  /* Insertion of Intraluminal Device into Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FHB82Z"  =  "1"  /* Insertion of Monitoring Device into Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FHD02Z"  =  "1"  /* Insertion of Monitoring Device into Pancreatic Duct, Open Approach */
"0FHD32Z"  =  "1"  /* Insertion of Monitoring Device into Pancreatic Duct, Percutaneous Approach */
"0FHD42Z"  =  "1"  /* Insertion of Monitoring Device into Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FHD72Z"  =  "1"  /* Insertion of Monitoring Device into Pancreatic Duct, Via Natural or Artificial Opening */
"0FHD82Z"  =  "1"  /* Insertion of Monitoring Device into Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FHG02Z"  =  "1"  /* Insertion of Monitoring Device into Pancreas, Open Approach */
"0FHG32Z"  =  "1"  /* Insertion of Monitoring Device into Pancreas, Percutaneous Approach */
"0FHG42Z"  =  "1"  /* Insertion of Monitoring Device into Pancreas, Percutaneous Endoscopic Approach */
"0FJ00ZZ"  =  "1"  /* Inspection of Liver, Open Approach */
"0FJ04ZZ"  =  "1"  /* Inspection of Liver, Percutaneous Endoscopic Approach */
"0FJ44ZZ"  =  "1"  /* Inspection of Gallbladder, Percutaneous Endoscopic Approach */
"0FJB0ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Open Approach */
"0FJB3ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Percutaneous Approach */
"0FJB4ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FJB7ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FJB8ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FJD4ZZ"  =  "1"  /* Inspection of Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FJG4ZZ"  =  "1"  /* Inspection of Pancreas, Percutaneous Endoscopic Approach */
"0FL53CZ"  =  "1"  /* Occlusion of Right Hepatic Duct with Extraluminal Device, Percutaneous Approach */
"0FL53DZ"  =  "1"  /* Occlusion of Right Hepatic Duct with Intraluminal Device, Percutaneous Approach */
"0FL53ZZ"  =  "1"  /* Occlusion of Right Hepatic Duct, Percutaneous Approach */
"0FL54CZ"  =  "1"  /* Occlusion of Right Hepatic Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FL54DZ"  =  "1"  /* Occlusion of Right Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FL54ZZ"  =  "1"  /* Occlusion of Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0FL57DZ"  =  "1"  /* Occlusion of Right Hepatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0FL57ZZ"  =  "1"  /* Occlusion of Right Hepatic Duct, Via Natural or Artificial Opening */
"0FL58DZ"  =  "1"  /* Occlusion of Right Hepatic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FL58ZZ"  =  "1"  /* Occlusion of Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FL63CZ"  =  "1"  /* Occlusion of Left Hepatic Duct with Extraluminal Device, Percutaneous Approach */
"0FL63DZ"  =  "1"  /* Occlusion of Left Hepatic Duct with Intraluminal Device, Percutaneous Approach */
"0FL63ZZ"  =  "1"  /* Occlusion of Left Hepatic Duct, Percutaneous Approach */
"0FL64CZ"  =  "1"  /* Occlusion of Left Hepatic Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FL64DZ"  =  "1"  /* Occlusion of Left Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FL64ZZ"  =  "1"  /* Occlusion of Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0FL67DZ"  =  "1"  /* Occlusion of Left Hepatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0FL67ZZ"  =  "1"  /* Occlusion of Left Hepatic Duct, Via Natural or Artificial Opening */
"0FL68DZ"  =  "1"  /* Occlusion of Left Hepatic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FL68ZZ"  =  "1"  /* Occlusion of Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FL83CZ"  =  "1"  /* Occlusion of Cystic Duct with Extraluminal Device, Percutaneous Approach */
"0FL83DZ"  =  "1"  /* Occlusion of Cystic Duct with Intraluminal Device, Percutaneous Approach */
"0FL83ZZ"  =  "1"  /* Occlusion of Cystic Duct, Percutaneous Approach */
"0FL84CZ"  =  "1"  /* Occlusion of Cystic Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FL84DZ"  =  "1"  /* Occlusion of Cystic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FL84ZZ"  =  "1"  /* Occlusion of Cystic Duct, Percutaneous Endoscopic Approach */
"0FL87DZ"  =  "1"  /* Occlusion of Cystic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0FL87ZZ"  =  "1"  /* Occlusion of Cystic Duct, Via Natural or Artificial Opening */
"0FL88DZ"  =  "1"  /* Occlusion of Cystic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FL88ZZ"  =  "1"  /* Occlusion of Cystic Duct, Via Natural or Artificial Opening Endoscopic */
"0FL93CZ"  =  "1"  /* Occlusion of Common Bile Duct with Extraluminal Device, Percutaneous Approach */
"0FL93DZ"  =  "1"  /* Occlusion of Common Bile Duct with Intraluminal Device, Percutaneous Approach */
"0FL93ZZ"  =  "1"  /* Occlusion of Common Bile Duct, Percutaneous Approach */
"0FL94CZ"  =  "1"  /* Occlusion of Common Bile Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FL94DZ"  =  "1"  /* Occlusion of Common Bile Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FL94ZZ"  =  "1"  /* Occlusion of Common Bile Duct, Percutaneous Endoscopic Approach */
"0FL97DZ"  =  "1"  /* Occlusion of Common Bile Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0FL97ZZ"  =  "1"  /* Occlusion of Common Bile Duct, Via Natural or Artificial Opening */
"0FL98DZ"  =  "1"  /* Occlusion of Common Bile Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FL98ZZ"  =  "1"  /* Occlusion of Common Bile Duct, Via Natural or Artificial Opening Endoscopic */
"0FM44ZZ"  =  "1"  /* Reattachment of Gallbladder, Percutaneous Endoscopic Approach */
"0FM54ZZ"  =  "1"  /* Reattachment of Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0FM64ZZ"  =  "1"  /* Reattachment of Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0FM84ZZ"  =  "1"  /* Reattachment of Cystic Duct, Percutaneous Endoscopic Approach */
"0FM94ZZ"  =  "1"  /* Reattachment of Common Bile Duct, Percutaneous Endoscopic Approach */
"0FP000Z"  =  "1"  /* Removal of Drainage Device from Liver, Open Approach */
"0FP002Z"  =  "1"  /* Removal of Monitoring Device from Liver, Open Approach */
"0FP003Z"  =  "1"  /* Removal of Infusion Device from Liver, Open Approach */
"0FP030Z"  =  "1"  /* Removal of Drainage Device from Liver, Percutaneous Approach */
"0FP032Z"  =  "1"  /* Removal of Monitoring Device from Liver, Percutaneous Approach */
"0FP033Z"  =  "1"  /* Removal of Infusion Device from Liver, Percutaneous Approach */
"0FP040Z"  =  "1"  /* Removal of Drainage Device from Liver, Percutaneous Endoscopic Approach */
"0FP042Z"  =  "1"  /* Removal of Monitoring Device from Liver, Percutaneous Endoscopic Approach */
"0FP043Z"  =  "1"  /* Removal of Infusion Device from Liver, Percutaneous Endoscopic Approach */
"0FPB00Z"  =  "1"  /* Removal of Drainage Device from Hepatobiliary Duct, Open Approach */
"0FPB01Z"  =  "1"  /* Removal of Radioactive Element from Hepatobiliary Duct, Open Approach */
"0FPB02Z"  =  "1"  /* Removal of Monitoring Device from Hepatobiliary Duct, Open Approach */
"0FPB03Z"  =  "1"  /* Removal of Infusion Device from Hepatobiliary Duct, Open Approach */
"0FPB07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Open Approach */
"0FPB0CZ"  =  "1"  /* Removal of Extraluminal Device from Hepatobiliary Duct, Open Approach */
"0FPB0DZ"  =  "1"  /* Removal of Intraluminal Device from Hepatobiliary Duct, Open Approach */
"0FPB0JZ"  =  "1"  /* Removal of Synthetic Substitute from Hepatobiliary Duct, Open Approach */
"0FPB0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Open Approach */
"0FPB30Z"  =  "1"  /* Removal of Drainage Device from Hepatobiliary Duct, Percutaneous Approach */
"0FPB31Z"  =  "1"  /* Removal of Radioactive Element from Hepatobiliary Duct, Percutaneous Approach */
"0FPB32Z"  =  "1"  /* Removal of Monitoring Device from Hepatobiliary Duct, Percutaneous Approach */
"0FPB33Z"  =  "1"  /* Removal of Infusion Device from Hepatobiliary Duct, Percutaneous Approach */
"0FPB37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Percutaneous Approach */
"0FPB3CZ"  =  "1"  /* Removal of Extraluminal Device from Hepatobiliary Duct, Percutaneous Approach */
"0FPB3DZ"  =  "1"  /* Removal of Intraluminal Device from Hepatobiliary Duct, Percutaneous Approach */
"0FPB3JZ"  =  "1"  /* Removal of Synthetic Substitute from Hepatobiliary Duct, Percutaneous Approach */
"0FPB3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Percutaneous Approach */
"0FPB40Z"  =  "1"  /* Removal of Drainage Device from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB41Z"  =  "1"  /* Removal of Radioactive Element from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB42Z"  =  "1"  /* Removal of Monitoring Device from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB43Z"  =  "1"  /* Removal of Infusion Device from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB4CZ"  =  "1"  /* Removal of Extraluminal Device from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB4DZ"  =  "1"  /* Removal of Intraluminal Device from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB4JZ"  =  "1"  /* Removal of Synthetic Substitute from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB70Z"  =  "1"  /* Removal of Drainage Device from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB71Z"  =  "1"  /* Removal of Radioactive Element from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB72Z"  =  "1"  /* Removal of Monitoring Device from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB73Z"  =  "1"  /* Removal of Infusion Device from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB7CZ"  =  "1"  /* Removal of Extraluminal Device from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB7JZ"  =  "1"  /* Removal of Synthetic Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB80Z"  =  "1"  /* Removal of Drainage Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB81Z"  =  "1"  /* Removal of Radioactive Element from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB82Z"  =  "1"  /* Removal of Monitoring Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB83Z"  =  "1"  /* Removal of Infusion Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB8CZ"  =  "1"  /* Removal of Extraluminal Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB8JZ"  =  "1"  /* Removal of Synthetic Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD00Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Open Approach */
"0FPD01Z"  =  "1"  /* Removal of Radioactive Element from Pancreatic Duct, Open Approach */
"0FPD02Z"  =  "1"  /* Removal of Monitoring Device from Pancreatic Duct, Open Approach */
"0FPD03Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Open Approach */
"0FPD07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Pancreatic Duct, Open Approach */
"0FPD0CZ"  =  "1"  /* Removal of Extraluminal Device from Pancreatic Duct, Open Approach */
"0FPD0DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreatic Duct, Open Approach */
"0FPD0JZ"  =  "1"  /* Removal of Synthetic Substitute from Pancreatic Duct, Open Approach */
"0FPD0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Open Approach */
"0FPD30Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Percutaneous Approach */
"0FPD31Z"  =  "1"  /* Removal of Radioactive Element from Pancreatic Duct, Percutaneous Approach */
"0FPD32Z"  =  "1"  /* Removal of Monitoring Device from Pancreatic Duct, Percutaneous Approach */
"0FPD33Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Percutaneous Approach */
"0FPD37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Pancreatic Duct, Percutaneous Approach */
"0FPD3CZ"  =  "1"  /* Removal of Extraluminal Device from Pancreatic Duct, Percutaneous Approach */
"0FPD3DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreatic Duct, Percutaneous Approach */
"0FPD3JZ"  =  "1"  /* Removal of Synthetic Substitute from Pancreatic Duct, Percutaneous Approach */
"0FPD3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Percutaneous Approach */
"0FPD40Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD41Z"  =  "1"  /* Removal of Radioactive Element from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD42Z"  =  "1"  /* Removal of Monitoring Device from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD43Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD4CZ"  =  "1"  /* Removal of Extraluminal Device from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD4DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD4JZ"  =  "1"  /* Removal of Synthetic Substitute from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD70Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD71Z"  =  "1"  /* Removal of Radioactive Element from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD72Z"  =  "1"  /* Removal of Monitoring Device from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD73Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD7CZ"  =  "1"  /* Removal of Extraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD7DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD7JZ"  =  "1"  /* Removal of Synthetic Substitute from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD80Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD81Z"  =  "1"  /* Removal of Radioactive Element from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD82Z"  =  "1"  /* Removal of Monitoring Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD83Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD8CZ"  =  "1"  /* Removal of Extraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD8DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD8JZ"  =  "1"  /* Removal of Synthetic Substitute from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPG00Z"  =  "1"  /* Removal of Drainage Device from Pancreas, Open Approach */
"0FPG02Z"  =  "1"  /* Removal of Monitoring Device from Pancreas, Open Approach */
"0FPG03Z"  =  "1"  /* Removal of Infusion Device from Pancreas, Open Approach */
"0FPG0DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreas, Open Approach */
"0FPG30Z"  =  "1"  /* Removal of Drainage Device from Pancreas, Percutaneous Approach */
"0FPG32Z"  =  "1"  /* Removal of Monitoring Device from Pancreas, Percutaneous Approach */
"0FPG33Z"  =  "1"  /* Removal of Infusion Device from Pancreas, Percutaneous Approach */
"0FPG3DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreas, Percutaneous Approach */
"0FPG40Z"  =  "1"  /* Removal of Drainage Device from Pancreas, Percutaneous Endoscopic Approach */
"0FPG42Z"  =  "1"  /* Removal of Monitoring Device from Pancreas, Percutaneous Endoscopic Approach */
"0FPG43Z"  =  "1"  /* Removal of Infusion Device from Pancreas, Percutaneous Endoscopic Approach */
"0FPG4DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreas, Percutaneous Endoscopic Approach */
"0FPGXDZ"  =  "1"  /* Removal of Intraluminal Device from Pancreas, External Approach */
"0FV53CZ"  =  "1"  /* Restriction of Right Hepatic Duct with Extraluminal Device, Percutaneous Approach */
"0FV53DZ"  =  "1"  /* Restriction of Right Hepatic Duct with Intraluminal Device, Percutaneous Approach */
"0FV53ZZ"  =  "1"  /* Restriction of Right Hepatic Duct, Percutaneous Approach */
"0FV54CZ"  =  "1"  /* Restriction of Right Hepatic Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FV54DZ"  =  "1"  /* Restriction of Right Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FV54ZZ"  =  "1"  /* Restriction of Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0FV57DZ"  =  "1"  /* Restriction of Right Hepatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0FV57ZZ"  =  "1"  /* Restriction of Right Hepatic Duct, Via Natural or Artificial Opening */
"0FV58DZ"  =  "1"  /* Restriction of Right Hepatic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FV58ZZ"  =  "1"  /* Restriction of Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FV63CZ"  =  "1"  /* Restriction of Left Hepatic Duct with Extraluminal Device, Percutaneous Approach */
"0FV63DZ"  =  "1"  /* Restriction of Left Hepatic Duct with Intraluminal Device, Percutaneous Approach */
"0FV63ZZ"  =  "1"  /* Restriction of Left Hepatic Duct, Percutaneous Approach */
"0FV64CZ"  =  "1"  /* Restriction of Left Hepatic Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FV64DZ"  =  "1"  /* Restriction of Left Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FV64ZZ"  =  "1"  /* Restriction of Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0FV67DZ"  =  "1"  /* Restriction of Left Hepatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0FV67ZZ"  =  "1"  /* Restriction of Left Hepatic Duct, Via Natural or Artificial Opening */
"0FV68DZ"  =  "1"  /* Restriction of Left Hepatic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FV68ZZ"  =  "1"  /* Restriction of Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FV83CZ"  =  "1"  /* Restriction of Cystic Duct with Extraluminal Device, Percutaneous Approach */
"0FV83DZ"  =  "1"  /* Restriction of Cystic Duct with Intraluminal Device, Percutaneous Approach */
"0FV83ZZ"  =  "1"  /* Restriction of Cystic Duct, Percutaneous Approach */
"0FV84CZ"  =  "1"  /* Restriction of Cystic Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FV84DZ"  =  "1"  /* Restriction of Cystic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FV84ZZ"  =  "1"  /* Restriction of Cystic Duct, Percutaneous Endoscopic Approach */
"0FV87DZ"  =  "1"  /* Restriction of Cystic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0FV87ZZ"  =  "1"  /* Restriction of Cystic Duct, Via Natural or Artificial Opening */
"0FV88DZ"  =  "1"  /* Restriction of Cystic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FV88ZZ"  =  "1"  /* Restriction of Cystic Duct, Via Natural or Artificial Opening Endoscopic */
"0FV93CZ"  =  "1"  /* Restriction of Common Bile Duct with Extraluminal Device, Percutaneous Approach */
"0FV93DZ"  =  "1"  /* Restriction of Common Bile Duct with Intraluminal Device, Percutaneous Approach */
"0FV93ZZ"  =  "1"  /* Restriction of Common Bile Duct, Percutaneous Approach */
"0FV94CZ"  =  "1"  /* Restriction of Common Bile Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FV94DZ"  =  "1"  /* Restriction of Common Bile Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FV94ZZ"  =  "1"  /* Restriction of Common Bile Duct, Percutaneous Endoscopic Approach */
"0FV97DZ"  =  "1"  /* Restriction of Common Bile Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0FV97ZZ"  =  "1"  /* Restriction of Common Bile Duct, Via Natural or Artificial Opening */
"0FV98DZ"  =  "1"  /* Restriction of Common Bile Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FV98ZZ"  =  "1"  /* Restriction of Common Bile Duct, Via Natural or Artificial Opening Endoscopic */
"0FW000Z"  =  "1"  /* Revision of Drainage Device in Liver, Open Approach */
"0FW002Z"  =  "1"  /* Revision of Monitoring Device in Liver, Open Approach */
"0FW003Z"  =  "1"  /* Revision of Infusion Device in Liver, Open Approach */
"0FW030Z"  =  "1"  /* Revision of Drainage Device in Liver, Percutaneous Approach */
"0FW032Z"  =  "1"  /* Revision of Monitoring Device in Liver, Percutaneous Approach */
"0FW033Z"  =  "1"  /* Revision of Infusion Device in Liver, Percutaneous Approach */
"0FW040Z"  =  "1"  /* Revision of Drainage Device in Liver, Percutaneous Endoscopic Approach */
"0FW042Z"  =  "1"  /* Revision of Monitoring Device in Liver, Percutaneous Endoscopic Approach */
"0FW043Z"  =  "1"  /* Revision of Infusion Device in Liver, Percutaneous Endoscopic Approach */
"0FWB00Z"  =  "1"  /* Revision of Drainage Device in Hepatobiliary Duct, Open Approach */
"0FWB02Z"  =  "1"  /* Revision of Monitoring Device in Hepatobiliary Duct, Open Approach */
"0FWB03Z"  =  "1"  /* Revision of Infusion Device in Hepatobiliary Duct, Open Approach */
"0FWB07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Open Approach */
"0FWB0CZ"  =  "1"  /* Revision of Extraluminal Device in Hepatobiliary Duct, Open Approach */
"0FWB0DZ"  =  "1"  /* Revision of Intraluminal Device in Hepatobiliary Duct, Open Approach */
"0FWB0JZ"  =  "1"  /* Revision of Synthetic Substitute in Hepatobiliary Duct, Open Approach */
"0FWB0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Open Approach */
"0FWB30Z"  =  "1"  /* Revision of Drainage Device in Hepatobiliary Duct, Percutaneous Approach */
"0FWB32Z"  =  "1"  /* Revision of Monitoring Device in Hepatobiliary Duct, Percutaneous Approach */
"0FWB33Z"  =  "1"  /* Revision of Infusion Device in Hepatobiliary Duct, Percutaneous Approach */
"0FWB37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Percutaneous Approach */
"0FWB3CZ"  =  "1"  /* Revision of Extraluminal Device in Hepatobiliary Duct, Percutaneous Approach */
"0FWB3DZ"  =  "1"  /* Revision of Intraluminal Device in Hepatobiliary Duct, Percutaneous Approach */
"0FWB3JZ"  =  "1"  /* Revision of Synthetic Substitute in Hepatobiliary Duct, Percutaneous Approach */
"0FWB3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Percutaneous Approach */
"0FWB40Z"  =  "1"  /* Revision of Drainage Device in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB42Z"  =  "1"  /* Revision of Monitoring Device in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB43Z"  =  "1"  /* Revision of Infusion Device in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB4CZ"  =  "1"  /* Revision of Extraluminal Device in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB4DZ"  =  "1"  /* Revision of Intraluminal Device in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB4JZ"  =  "1"  /* Revision of Synthetic Substitute in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB70Z"  =  "1"  /* Revision of Drainage Device in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB72Z"  =  "1"  /* Revision of Monitoring Device in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB73Z"  =  "1"  /* Revision of Infusion Device in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB7CZ"  =  "1"  /* Revision of Extraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB7DZ"  =  "1"  /* Revision of Intraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB7JZ"  =  "1"  /* Revision of Synthetic Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB80Z"  =  "1"  /* Revision of Drainage Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB82Z"  =  "1"  /* Revision of Monitoring Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB83Z"  =  "1"  /* Revision of Infusion Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB8CZ"  =  "1"  /* Revision of Extraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB8DZ"  =  "1"  /* Revision of Intraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB8JZ"  =  "1"  /* Revision of Synthetic Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD00Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Open Approach */
"0FWD02Z"  =  "1"  /* Revision of Monitoring Device in Pancreatic Duct, Open Approach */
"0FWD03Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Open Approach */
"0FWD07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pancreatic Duct, Open Approach */
"0FWD0CZ"  =  "1"  /* Revision of Extraluminal Device in Pancreatic Duct, Open Approach */
"0FWD0DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreatic Duct, Open Approach */
"0FWD0JZ"  =  "1"  /* Revision of Synthetic Substitute in Pancreatic Duct, Open Approach */
"0FWD0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Open Approach */
"0FWD30Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Percutaneous Approach */
"0FWD32Z"  =  "1"  /* Revision of Monitoring Device in Pancreatic Duct, Percutaneous Approach */
"0FWD33Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Percutaneous Approach */
"0FWD37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pancreatic Duct, Percutaneous Approach */
"0FWD3CZ"  =  "1"  /* Revision of Extraluminal Device in Pancreatic Duct, Percutaneous Approach */
"0FWD3DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreatic Duct, Percutaneous Approach */
"0FWD3JZ"  =  "1"  /* Revision of Synthetic Substitute in Pancreatic Duct, Percutaneous Approach */
"0FWD3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Percutaneous Approach */
"0FWD40Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD42Z"  =  "1"  /* Revision of Monitoring Device in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD43Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD4CZ"  =  "1"  /* Revision of Extraluminal Device in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD4DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD4JZ"  =  "1"  /* Revision of Synthetic Substitute in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD70Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD72Z"  =  "1"  /* Revision of Monitoring Device in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD73Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD7CZ"  =  "1"  /* Revision of Extraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD7DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD7JZ"  =  "1"  /* Revision of Synthetic Substitute in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD80Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD82Z"  =  "1"  /* Revision of Monitoring Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD83Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD8CZ"  =  "1"  /* Revision of Extraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD8DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD8JZ"  =  "1"  /* Revision of Synthetic Substitute in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWG00Z"  =  "1"  /* Revision of Drainage Device in Pancreas, Open Approach */
"0FWG02Z"  =  "1"  /* Revision of Monitoring Device in Pancreas, Open Approach */
"0FWG03Z"  =  "1"  /* Revision of Infusion Device in Pancreas, Open Approach */
"0FWG0DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreas, Open Approach */
"0FWG30Z"  =  "1"  /* Revision of Drainage Device in Pancreas, Percutaneous Approach */
"0FWG32Z"  =  "1"  /* Revision of Monitoring Device in Pancreas, Percutaneous Approach */
"0FWG33Z"  =  "1"  /* Revision of Infusion Device in Pancreas, Percutaneous Approach */
"0FWG3DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreas, Percutaneous Approach */
"0FWG40Z"  =  "1"  /* Revision of Drainage Device in Pancreas, Percutaneous Endoscopic Approach */
"0FWG42Z"  =  "1"  /* Revision of Monitoring Device in Pancreas, Percutaneous Endoscopic Approach */
"0FWG43Z"  =  "1"  /* Revision of Infusion Device in Pancreas, Percutaneous Endoscopic Approach */
"0FWG4DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreas, Percutaneous Endoscopic Approach */
"0G9000Z"  =  "1"  /* Drainage of Pituitary Gland with Drainage Device, Open Approach */
"0G900ZZ"  =  "1"  /* Drainage of Pituitary Gland, Open Approach */
"0G9030Z"  =  "1"  /* Drainage of Pituitary Gland with Drainage Device, Percutaneous Approach */
"0G903ZZ"  =  "1"  /* Drainage of Pituitary Gland, Percutaneous Approach */
"0G9040Z"  =  "1"  /* Drainage of Pituitary Gland with Drainage Device, Percutaneous Endoscopic Approach */
"0G904ZZ"  =  "1"  /* Drainage of Pituitary Gland, Percutaneous Endoscopic Approach */
"0G9100Z"  =  "1"  /* Drainage of Pineal Body with Drainage Device, Open Approach */
"0G910ZZ"  =  "1"  /* Drainage of Pineal Body, Open Approach */
"0G9130Z"  =  "1"  /* Drainage of Pineal Body with Drainage Device, Percutaneous Approach */
"0G913ZZ"  =  "1"  /* Drainage of Pineal Body, Percutaneous Approach */
"0G9140Z"  =  "1"  /* Drainage of Pineal Body with Drainage Device, Percutaneous Endoscopic Approach */
"0G914ZZ"  =  "1"  /* Drainage of Pineal Body, Percutaneous Endoscopic Approach */
"0G9200Z"  =  "1"  /* Drainage of Left Adrenal Gland with Drainage Device, Open Approach */
"0G920ZZ"  =  "1"  /* Drainage of Left Adrenal Gland, Open Approach */
"0G9230Z"  =  "1"  /* Drainage of Left Adrenal Gland with Drainage Device, Percutaneous Approach */
"0G923ZZ"  =  "1"  /* Drainage of Left Adrenal Gland, Percutaneous Approach */
"0G9240Z"  =  "1"  /* Drainage of Left Adrenal Gland with Drainage Device, Percutaneous Endoscopic Approach */
"0G924ZZ"  =  "1"  /* Drainage of Left Adrenal Gland, Percutaneous Endoscopic Approach */
"0G9300Z"  =  "1"  /* Drainage of Right Adrenal Gland with Drainage Device, Open Approach */
"0G930ZZ"  =  "1"  /* Drainage of Right Adrenal Gland, Open Approach */
"0G9330Z"  =  "1"  /* Drainage of Right Adrenal Gland with Drainage Device, Percutaneous Approach */
"0G933ZZ"  =  "1"  /* Drainage of Right Adrenal Gland, Percutaneous Approach */
"0G9340Z"  =  "1"  /* Drainage of Right Adrenal Gland with Drainage Device, Percutaneous Endoscopic Approach */
"0G934ZZ"  =  "1"  /* Drainage of Right Adrenal Gland, Percutaneous Endoscopic Approach */
"0G9400Z"  =  "1"  /* Drainage of Bilateral Adrenal Glands with Drainage Device, Open Approach */
"0G940ZZ"  =  "1"  /* Drainage of Bilateral Adrenal Glands, Open Approach */
"0G9430Z"  =  "1"  /* Drainage of Bilateral Adrenal Glands with Drainage Device, Percutaneous Approach */
"0G943ZZ"  =  "1"  /* Drainage of Bilateral Adrenal Glands, Percutaneous Approach */
"0G9440Z"  =  "1"  /* Drainage of Bilateral Adrenal Glands with Drainage Device, Percutaneous Endoscopic Approach */
"0G944ZZ"  =  "1"  /* Drainage of Bilateral Adrenal Glands, Percutaneous Endoscopic Approach */
"0G9G00Z"  =  "1"  /* Drainage of Left Thyroid Gland Lobe with Drainage Device, Open Approach */
"0G9G0ZZ"  =  "1"  /* Drainage of Left Thyroid Gland Lobe, Open Approach */
"0G9H00Z"  =  "1"  /* Drainage of Right Thyroid Gland Lobe with Drainage Device, Open Approach */
"0G9H0ZZ"  =  "1"  /* Drainage of Right Thyroid Gland Lobe, Open Approach */
"0G9K00Z"  =  "1"  /* Drainage of Thyroid Gland with Drainage Device, Open Approach */
"0G9K0ZZ"  =  "1"  /* Drainage of Thyroid Gland, Open Approach */
"0G9L00Z"  =  "1"  /* Drainage of Right Superior Parathyroid Gland with Drainage Device, Open Approach */
"0G9L0ZZ"  =  "1"  /* Drainage of Right Superior Parathyroid Gland, Open Approach */
"0G9M00Z"  =  "1"  /* Drainage of Left Superior Parathyroid Gland with Drainage Device, Open Approach */
"0G9M0ZZ"  =  "1"  /* Drainage of Left Superior Parathyroid Gland, Open Approach */
"0G9N00Z"  =  "1"  /* Drainage of Right Inferior Parathyroid Gland with Drainage Device, Open Approach */
"0G9N0ZZ"  =  "1"  /* Drainage of Right Inferior Parathyroid Gland, Open Approach */
"0G9P00Z"  =  "1"  /* Drainage of Left Inferior Parathyroid Gland with Drainage Device, Open Approach */
"0G9P0ZZ"  =  "1"  /* Drainage of Left Inferior Parathyroid Gland, Open Approach */
"0G9Q00Z"  =  "1"  /* Drainage of Multiple Parathyroid Glands with Drainage Device, Open Approach */
"0G9Q0ZZ"  =  "1"  /* Drainage of Multiple Parathyroid Glands, Open Approach */
"0G9R00Z"  =  "1"  /* Drainage of Parathyroid Gland with Drainage Device, Open Approach */
"0G9R0ZZ"  =  "1"  /* Drainage of Parathyroid Gland, Open Approach */
"0GC00ZZ"  =  "1"  /* Extirpation of Matter from Pituitary Gland, Open Approach */
"0GC03ZZ"  =  "1"  /* Extirpation of Matter from Pituitary Gland, Percutaneous Approach */
"0GC04ZZ"  =  "1"  /* Extirpation of Matter from Pituitary Gland, Percutaneous Endoscopic Approach */
"0GC10ZZ"  =  "1"  /* Extirpation of Matter from Pineal Body, Open Approach */
"0GC13ZZ"  =  "1"  /* Extirpation of Matter from Pineal Body, Percutaneous Approach */
"0GC14ZZ"  =  "1"  /* Extirpation of Matter from Pineal Body, Percutaneous Endoscopic Approach */
"0GC20ZZ"  =  "1"  /* Extirpation of Matter from Left Adrenal Gland, Open Approach */
"0GC23ZZ"  =  "1"  /* Extirpation of Matter from Left Adrenal Gland, Percutaneous Approach */
"0GC24ZZ"  =  "1"  /* Extirpation of Matter from Left Adrenal Gland, Percutaneous Endoscopic Approach */
"0GC30ZZ"  =  "1"  /* Extirpation of Matter from Right Adrenal Gland, Open Approach */
"0GC33ZZ"  =  "1"  /* Extirpation of Matter from Right Adrenal Gland, Percutaneous Approach */
"0GC34ZZ"  =  "1"  /* Extirpation of Matter from Right Adrenal Gland, Percutaneous Endoscopic Approach */
"0GC40ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Adrenal Glands, Open Approach */
"0GC43ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Adrenal Glands, Percutaneous Approach */
"0GC44ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Adrenal Glands, Percutaneous Endoscopic Approach */
"0GCG0ZZ"  =  "1"  /* Extirpation of Matter from Left Thyroid Gland Lobe, Open Approach */
"0GCG3ZZ"  =  "1"  /* Extirpation of Matter from Left Thyroid Gland Lobe, Percutaneous Approach */
"0GCG4ZZ"  =  "1"  /* Extirpation of Matter from Left Thyroid Gland Lobe, Percutaneous Endoscopic Approach */
"0GCH0ZZ"  =  "1"  /* Extirpation of Matter from Right Thyroid Gland Lobe, Open Approach */
"0GCH3ZZ"  =  "1"  /* Extirpation of Matter from Right Thyroid Gland Lobe, Percutaneous Approach */
"0GCH4ZZ"  =  "1"  /* Extirpation of Matter from Right Thyroid Gland Lobe, Percutaneous Endoscopic Approach */
"0GCK0ZZ"  =  "1"  /* Extirpation of Matter from Thyroid Gland, Open Approach */
"0GCK3ZZ"  =  "1"  /* Extirpation of Matter from Thyroid Gland, Percutaneous Approach */
"0GCK4ZZ"  =  "1"  /* Extirpation of Matter from Thyroid Gland, Percutaneous Endoscopic Approach */
"0GCL0ZZ"  =  "1"  /* Extirpation of Matter from Right Superior Parathyroid Gland, Open Approach */
"0GCL3ZZ"  =  "1"  /* Extirpation of Matter from Right Superior Parathyroid Gland, Percutaneous Approach */
"0GCL4ZZ"  =  "1"  /* Extirpation of Matter from Right Superior Parathyroid Gland, Percutaneous Endoscopic Approach */
"0GCM0ZZ"  =  "1"  /* Extirpation of Matter from Left Superior Parathyroid Gland, Open Approach */
"0GCM3ZZ"  =  "1"  /* Extirpation of Matter from Left Superior Parathyroid Gland, Percutaneous Approach */
"0GCM4ZZ"  =  "1"  /* Extirpation of Matter from Left Superior Parathyroid Gland, Percutaneous Endoscopic Approach */
"0GCN0ZZ"  =  "1"  /* Extirpation of Matter from Right Inferior Parathyroid Gland, Open Approach */
"0GCN3ZZ"  =  "1"  /* Extirpation of Matter from Right Inferior Parathyroid Gland, Percutaneous Approach */
"0GCN4ZZ"  =  "1"  /* Extirpation of Matter from Right Inferior Parathyroid Gland, Percutaneous Endoscopic Approach */
"0GCP0ZZ"  =  "1"  /* Extirpation of Matter from Left Inferior Parathyroid Gland, Open Approach */
"0GCP3ZZ"  =  "1"  /* Extirpation of Matter from Left Inferior Parathyroid Gland, Percutaneous Approach */
"0GCP4ZZ"  =  "1"  /* Extirpation of Matter from Left Inferior Parathyroid Gland, Percutaneous Endoscopic Approach */
"0GCQ0ZZ"  =  "1"  /* Extirpation of Matter from Multiple Parathyroid Glands, Open Approach */
"0GCQ3ZZ"  =  "1"  /* Extirpation of Matter from Multiple Parathyroid Glands, Percutaneous Approach */
"0GCQ4ZZ"  =  "1"  /* Extirpation of Matter from Multiple Parathyroid Glands, Percutaneous Endoscopic Approach */
"0GCR0ZZ"  =  "1"  /* Extirpation of Matter from Parathyroid Gland, Open Approach */
"0GCR3ZZ"  =  "1"  /* Extirpation of Matter from Parathyroid Gland, Percutaneous Approach */
"0GCR4ZZ"  =  "1"  /* Extirpation of Matter from Parathyroid Gland, Percutaneous Endoscopic Approach */
"0GHS02Z"  =  "1"  /* Insertion of Monitoring Device into Endocrine Gland, Open Approach */
"0GHS03Z"  =  "1"  /* Insertion of Infusion Device into Endocrine Gland, Open Approach */
"0GHS32Z"  =  "1"  /* Insertion of Monitoring Device into Endocrine Gland, Percutaneous Approach */
"0GHS33Z"  =  "1"  /* Insertion of Infusion Device into Endocrine Gland, Percutaneous Approach */
"0GHS42Z"  =  "1"  /* Insertion of Monitoring Device into Endocrine Gland, Percutaneous Endoscopic Approach */
"0GHS43Z"  =  "1"  /* Insertion of Infusion Device into Endocrine Gland, Percutaneous Endoscopic Approach */
"0GJ00ZZ"  =  "1"  /* Inspection of Pituitary Gland, Open Approach */
"0GJ03ZZ"  =  "1"  /* Inspection of Pituitary Gland, Percutaneous Approach */
"0GJ04ZZ"  =  "1"  /* Inspection of Pituitary Gland, Percutaneous Endoscopic Approach */
"0GJ10ZZ"  =  "1"  /* Inspection of Pineal Body, Open Approach */
"0GJ13ZZ"  =  "1"  /* Inspection of Pineal Body, Percutaneous Approach */
"0GJ14ZZ"  =  "1"  /* Inspection of Pineal Body, Percutaneous Endoscopic Approach */
"0GJ50ZZ"  =  "1"  /* Inspection of Adrenal Gland, Open Approach */
"0GJ53ZZ"  =  "1"  /* Inspection of Adrenal Gland, Percutaneous Approach */
"0GJ54ZZ"  =  "1"  /* Inspection of Adrenal Gland, Percutaneous Endoscopic Approach */
"0GJK0ZZ"  =  "1"  /* Inspection of Thyroid Gland, Open Approach */
"0GJR0ZZ"  =  "1"  /* Inspection of Parathyroid Gland, Open Approach */
"0GJS0ZZ"  =  "1"  /* Inspection of Endocrine Gland, Open Approach */
"0GP000Z"  =  "1"  /* Removal of Drainage Device from Pituitary Gland, Open Approach */
"0GP030Z"  =  "1"  /* Removal of Drainage Device from Pituitary Gland, Percutaneous Approach */
"0GP040Z"  =  "1"  /* Removal of Drainage Device from Pituitary Gland, Percutaneous Endoscopic Approach */
"0GP100Z"  =  "1"  /* Removal of Drainage Device from Pineal Body, Open Approach */
"0GP130Z"  =  "1"  /* Removal of Drainage Device from Pineal Body, Percutaneous Approach */
"0GP140Z"  =  "1"  /* Removal of Drainage Device from Pineal Body, Percutaneous Endoscopic Approach */
"0GP500Z"  =  "1"  /* Removal of Drainage Device from Adrenal Gland, Open Approach */
"0GP530Z"  =  "1"  /* Removal of Drainage Device from Adrenal Gland, Percutaneous Approach */
"0GP540Z"  =  "1"  /* Removal of Drainage Device from Adrenal Gland, Percutaneous Endoscopic Approach */
"0GPK00Z"  =  "1"  /* Removal of Drainage Device from Thyroid Gland, Open Approach */
"0GPK30Z"  =  "1"  /* Removal of Drainage Device from Thyroid Gland, Percutaneous Approach */
"0GPK40Z"  =  "1"  /* Removal of Drainage Device from Thyroid Gland, Percutaneous Endoscopic Approach */
"0GPR00Z"  =  "1"  /* Removal of Drainage Device from Parathyroid Gland, Open Approach */
"0GPR30Z"  =  "1"  /* Removal of Drainage Device from Parathyroid Gland, Percutaneous Approach */
"0GPR40Z"  =  "1"  /* Removal of Drainage Device from Parathyroid Gland, Percutaneous Endoscopic Approach */
"0GW000Z"  =  "1"  /* Revision of Drainage Device in Pituitary Gland, Open Approach */
"0GW030Z"  =  "1"  /* Revision of Drainage Device in Pituitary Gland, Percutaneous Approach */
"0GW040Z"  =  "1"  /* Revision of Drainage Device in Pituitary Gland, Percutaneous Endoscopic Approach */
"0GW100Z"  =  "1"  /* Revision of Drainage Device in Pineal Body, Open Approach */
"0GW130Z"  =  "1"  /* Revision of Drainage Device in Pineal Body, Percutaneous Approach */
"0GW140Z"  =  "1"  /* Revision of Drainage Device in Pineal Body, Percutaneous Endoscopic Approach */
"0GW500Z"  =  "1"  /* Revision of Drainage Device in Adrenal Gland, Open Approach */
"0GW530Z"  =  "1"  /* Revision of Drainage Device in Adrenal Gland, Percutaneous Approach */
"0GW540Z"  =  "1"  /* Revision of Drainage Device in Adrenal Gland, Percutaneous Endoscopic Approach */
"0GWK00Z"  =  "1"  /* Revision of Drainage Device in Thyroid Gland, Open Approach */
"0GWK30Z"  =  "1"  /* Revision of Drainage Device in Thyroid Gland, Percutaneous Approach */
"0GWK40Z"  =  "1"  /* Revision of Drainage Device in Thyroid Gland, Percutaneous Endoscopic Approach */
"0GWR00Z"  =  "1"  /* Revision of Drainage Device in Parathyroid Gland, Open Approach */
"0GWR30Z"  =  "1"  /* Revision of Drainage Device in Parathyroid Gland, Percutaneous Approach */
"0GWR40Z"  =  "1"  /* Revision of Drainage Device in Parathyroid Gland, Percutaneous Endoscopic Approach */
"0H80XZZ"  =  "1"  /* Division of Scalp Skin, External Approach */
"0H81XZZ"  =  "1"  /* Division of Face Skin, External Approach */
"0H82XZZ"  =  "1"  /* Division of Right Ear Skin, External Approach */
"0H83XZZ"  =  "1"  /* Division of Left Ear Skin, External Approach */
"0H84XZZ"  =  "1"  /* Division of Neck Skin, External Approach */
"0H85XZZ"  =  "1"  /* Division of Chest Skin, External Approach */
"0H86XZZ"  =  "1"  /* Division of Back Skin, External Approach */
"0H87XZZ"  =  "1"  /* Division of Abdomen Skin, External Approach */
"0H88XZZ"  =  "1"  /* Division of Buttock Skin, External Approach */
"0H89XZZ"  =  "1"  /* Division of Perineum Skin, External Approach */
"0H8AXZZ"  =  "1"  /* Division of Genitalia Skin, External Approach */
"0H8BXZZ"  =  "1"  /* Division of Right Upper Arm Skin, External Approach */
"0H8CXZZ"  =  "1"  /* Division of Left Upper Arm Skin, External Approach */
"0H8DXZZ"  =  "1"  /* Division of Right Lower Arm Skin, External Approach */
"0H8EXZZ"  =  "1"  /* Division of Left Lower Arm Skin, External Approach */
"0H8FXZZ"  =  "1"  /* Division of Right Hand Skin, External Approach */
"0H8GXZZ"  =  "1"  /* Division of Left Hand Skin, External Approach */
"0H8HXZZ"  =  "1"  /* Division of Right Upper Leg Skin, External Approach */
"0H8JXZZ"  =  "1"  /* Division of Left Upper Leg Skin, External Approach */
"0H8KXZZ"  =  "1"  /* Division of Right Lower Leg Skin, External Approach */
"0H8LXZZ"  =  "1"  /* Division of Left Lower Leg Skin, External Approach */
"0H8MXZZ"  =  "1"  /* Division of Right Foot Skin, External Approach */
"0H8NXZZ"  =  "1"  /* Division of Left Foot Skin, External Approach */
"0H90X0Z"  =  "1"  /* Drainage of Scalp Skin with Drainage Device, External Approach */
"0H90XZZ"  =  "1"  /* Drainage of Scalp Skin, External Approach */
"0H91X0Z"  =  "1"  /* Drainage of Face Skin with Drainage Device, External Approach */
"0H91XZZ"  =  "1"  /* Drainage of Face Skin, External Approach */
"0H92X0Z"  =  "1"  /* Drainage of Right Ear Skin with Drainage Device, External Approach */
"0H92XZZ"  =  "1"  /* Drainage of Right Ear Skin, External Approach */
"0H93X0Z"  =  "1"  /* Drainage of Left Ear Skin with Drainage Device, External Approach */
"0H93XZZ"  =  "1"  /* Drainage of Left Ear Skin, External Approach */
"0H94X0Z"  =  "1"  /* Drainage of Neck Skin with Drainage Device, External Approach */
"0H94XZZ"  =  "1"  /* Drainage of Neck Skin, External Approach */
"0H95X0Z"  =  "1"  /* Drainage of Chest Skin with Drainage Device, External Approach */
"0H95XZZ"  =  "1"  /* Drainage of Chest Skin, External Approach */
"0H96X0Z"  =  "1"  /* Drainage of Back Skin with Drainage Device, External Approach */
"0H96XZZ"  =  "1"  /* Drainage of Back Skin, External Approach */
"0H97X0Z"  =  "1"  /* Drainage of Abdomen Skin with Drainage Device, External Approach */
"0H97XZZ"  =  "1"  /* Drainage of Abdomen Skin, External Approach */
"0H98X0Z"  =  "1"  /* Drainage of Buttock Skin with Drainage Device, External Approach */
"0H98XZZ"  =  "1"  /* Drainage of Buttock Skin, External Approach */
"0H99X0Z"  =  "1"  /* Drainage of Perineum Skin with Drainage Device, External Approach */
"0H99XZZ"  =  "1"  /* Drainage of Perineum Skin, External Approach */
"0H9AX0Z"  =  "1"  /* Drainage of Genitalia Skin with Drainage Device, External Approach */
"0H9AXZZ"  =  "1"  /* Drainage of Genitalia Skin, External Approach */
"0H9BX0Z"  =  "1"  /* Drainage of Right Upper Arm Skin with Drainage Device, External Approach */
"0H9BXZZ"  =  "1"  /* Drainage of Right Upper Arm Skin, External Approach */
"0H9CX0Z"  =  "1"  /* Drainage of Left Upper Arm Skin with Drainage Device, External Approach */
"0H9CXZZ"  =  "1"  /* Drainage of Left Upper Arm Skin, External Approach */
"0H9DX0Z"  =  "1"  /* Drainage of Right Lower Arm Skin with Drainage Device, External Approach */
"0H9DXZZ"  =  "1"  /* Drainage of Right Lower Arm Skin, External Approach */
"0H9EX0Z"  =  "1"  /* Drainage of Left Lower Arm Skin with Drainage Device, External Approach */
"0H9EXZZ"  =  "1"  /* Drainage of Left Lower Arm Skin, External Approach */
"0H9FX0Z"  =  "1"  /* Drainage of Right Hand Skin with Drainage Device, External Approach */
"0H9FXZZ"  =  "1"  /* Drainage of Right Hand Skin, External Approach */
"0H9GX0Z"  =  "1"  /* Drainage of Left Hand Skin with Drainage Device, External Approach */
"0H9GXZZ"  =  "1"  /* Drainage of Left Hand Skin, External Approach */
"0H9HX0Z"  =  "1"  /* Drainage of Right Upper Leg Skin with Drainage Device, External Approach */
"0H9HXZZ"  =  "1"  /* Drainage of Right Upper Leg Skin, External Approach */
"0H9JX0Z"  =  "1"  /* Drainage of Left Upper Leg Skin with Drainage Device, External Approach */
"0H9JXZZ"  =  "1"  /* Drainage of Left Upper Leg Skin, External Approach */
"0H9KX0Z"  =  "1"  /* Drainage of Right Lower Leg Skin with Drainage Device, External Approach */
"0H9KXZZ"  =  "1"  /* Drainage of Right Lower Leg Skin, External Approach */
"0H9LX0Z"  =  "1"  /* Drainage of Left Lower Leg Skin with Drainage Device, External Approach */
"0H9LXZZ"  =  "1"  /* Drainage of Left Lower Leg Skin, External Approach */
"0H9MX0Z"  =  "1"  /* Drainage of Right Foot Skin with Drainage Device, External Approach */
"0H9MXZZ"  =  "1"  /* Drainage of Right Foot Skin, External Approach */
"0H9NX0Z"  =  "1"  /* Drainage of Left Foot Skin with Drainage Device, External Approach */
"0H9NXZZ"  =  "1"  /* Drainage of Left Foot Skin, External Approach */
"0H9T00Z"  =  "1"  /* Drainage of Right Breast with Drainage Device, Open Approach */
"0H9T0ZZ"  =  "1"  /* Drainage of Right Breast, Open Approach */
"0H9TX0Z"  =  "1"  /* Drainage of Right Breast with Drainage Device, External Approach */
"0H9TXZZ"  =  "1"  /* Drainage of Right Breast, External Approach */
"0H9U00Z"  =  "1"  /* Drainage of Left Breast with Drainage Device, Open Approach */
"0H9U0ZZ"  =  "1"  /* Drainage of Left Breast, Open Approach */
"0H9UX0Z"  =  "1"  /* Drainage of Left Breast with Drainage Device, External Approach */
"0H9UXZZ"  =  "1"  /* Drainage of Left Breast, External Approach */
"0H9V00Z"  =  "1"  /* Drainage of Bilateral Breast with Drainage Device, Open Approach */
"0H9V0ZZ"  =  "1"  /* Drainage of Bilateral Breast, Open Approach */
"0H9VX0Z"  =  "1"  /* Drainage of Bilateral Breast with Drainage Device, External Approach */
"0H9VXZZ"  =  "1"  /* Drainage of Bilateral Breast, External Approach */
"0H9W00Z"  =  "1"  /* Drainage of Right Nipple with Drainage Device, Open Approach */
"0H9W0ZZ"  =  "1"  /* Drainage of Right Nipple, Open Approach */
"0H9WX0Z"  =  "1"  /* Drainage of Right Nipple with Drainage Device, External Approach */
"0H9WXZZ"  =  "1"  /* Drainage of Right Nipple, External Approach */
"0H9X00Z"  =  "1"  /* Drainage of Left Nipple with Drainage Device, Open Approach */
"0H9X0ZZ"  =  "1"  /* Drainage of Left Nipple, Open Approach */
"0H9XX0Z"  =  "1"  /* Drainage of Left Nipple with Drainage Device, External Approach */
"0H9XXZZ"  =  "1"  /* Drainage of Left Nipple, External Approach */
"0HCT0ZZ"  =  "1"  /* Extirpation of Matter from Right Breast, Open Approach */
"0HCT3ZZ"  =  "1"  /* Extirpation of Matter from Right Breast, Percutaneous Approach */
"0HCT7ZZ"  =  "1"  /* Extirpation of Matter from Right Breast, Via Natural or Artificial Opening */
"0HCT8ZZ"  =  "1"  /* Extirpation of Matter from Right Breast, Via Natural or Artificial Opening Endoscopic */
"0HCTXZZ"  =  "1"  /* Extirpation of Matter from Right Breast, External Approach */
"0HCU0ZZ"  =  "1"  /* Extirpation of Matter from Left Breast, Open Approach */
"0HCU3ZZ"  =  "1"  /* Extirpation of Matter from Left Breast, Percutaneous Approach */
"0HCU7ZZ"  =  "1"  /* Extirpation of Matter from Left Breast, Via Natural or Artificial Opening */
"0HCU8ZZ"  =  "1"  /* Extirpation of Matter from Left Breast, Via Natural or Artificial Opening Endoscopic */
"0HCUXZZ"  =  "1"  /* Extirpation of Matter from Left Breast, External Approach */
"0HCV0ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Breast, Open Approach */
"0HCV3ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Breast, Percutaneous Approach */
"0HCV7ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Breast, Via Natural or Artificial Opening */
"0HCV8ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Breast, Via Natural or Artificial Opening Endoscopic */
"0HCVXZZ"  =  "1"  /* Extirpation of Matter from Bilateral Breast, External Approach */
"0HCW0ZZ"  =  "1"  /* Extirpation of Matter from Right Nipple, Open Approach */
"0HCW3ZZ"  =  "1"  /* Extirpation of Matter from Right Nipple, Percutaneous Approach */
"0HCW7ZZ"  =  "1"  /* Extirpation of Matter from Right Nipple, Via Natural or Artificial Opening */
"0HCW8ZZ"  =  "1"  /* Extirpation of Matter from Right Nipple, Via Natural or Artificial Opening Endoscopic */
"0HCWXZZ"  =  "1"  /* Extirpation of Matter from Right Nipple, External Approach */
"0HCX0ZZ"  =  "1"  /* Extirpation of Matter from Left Nipple, Open Approach */
"0HCX3ZZ"  =  "1"  /* Extirpation of Matter from Left Nipple, Percutaneous Approach */
"0HCX7ZZ"  =  "1"  /* Extirpation of Matter from Left Nipple, Via Natural or Artificial Opening */
"0HCX8ZZ"  =  "1"  /* Extirpation of Matter from Left Nipple, Via Natural or Artificial Opening Endoscopic */
"0HCXXZZ"  =  "1"  /* Extirpation of Matter from Left Nipple, External Approach */
"0HJT0ZZ"  =  "1"  /* Inspection of Right Breast, Open Approach */
"0HJU0ZZ"  =  "1"  /* Inspection of Left Breast, Open Approach */
"0HPT00Z"  =  "1"  /* Removal of Drainage Device from Right Breast, Open Approach */
"0HPT01Z"  =  "1"  /* Removal of Radioactive Element from Right Breast, Open Approach */
"0HPT07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Right Breast, Open Approach */
"0HPT0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Right Breast, Open Approach */
"0HPT30Z"  =  "1"  /* Removal of Drainage Device from Right Breast, Percutaneous Approach */
"0HPT31Z"  =  "1"  /* Removal of Radioactive Element from Right Breast, Percutaneous Approach */
"0HPT37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Right Breast, Percutaneous Approach */
"0HPT3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Right Breast, Percutaneous Approach */
"0HPT70Z"  =  "1"  /* Removal of Drainage Device from Right Breast, Via Natural or Artificial Opening */
"0HPT71Z"  =  "1"  /* Removal of Radioactive Element from Right Breast, Via Natural or Artificial Opening */
"0HPT77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Right Breast, Via Natural or Artificial Opening */
"0HPT7JZ"  =  "1"  /* Removal of Synthetic Substitute from Right Breast, Via Natural or Artificial Opening */
"0HPT7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Right Breast, Via Natural or Artificial Opening */
"0HPT7NZ"  =  "1"  /* Removal of Tissue Expander from Right Breast, Via Natural or Artificial Opening */
"0HPT80Z"  =  "1"  /* Removal of Drainage Device from Right Breast, Via Natural or Artificial Opening Endoscopic */
"0HPT81Z"  =  "1"  /* Removal of Radioactive Element from Right Breast, Via Natural or Artificial Opening Endoscopic */
"0HPT87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Right Breast, Via Natural or Artificial Opening Endoscopic */
"0HPT8JZ"  =  "1"  /* Removal of Synthetic Substitute from Right Breast, Via Natural or Artificial Opening Endoscopic */
"0HPT8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Right Breast, Via Natural or Artificial Opening Endoscopic */
"0HPT8NZ"  =  "1"  /* Removal of Tissue Expander from Right Breast, Via Natural or Artificial Opening Endoscopic */
"0HPU00Z"  =  "1"  /* Removal of Drainage Device from Left Breast, Open Approach */
"0HPU01Z"  =  "1"  /* Removal of Radioactive Element from Left Breast, Open Approach */
"0HPU07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Left Breast, Open Approach */
"0HPU0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Left Breast, Open Approach */
"0HPU30Z"  =  "1"  /* Removal of Drainage Device from Left Breast, Percutaneous Approach */
"0HPU31Z"  =  "1"  /* Removal of Radioactive Element from Left Breast, Percutaneous Approach */
"0HPU37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Left Breast, Percutaneous Approach */
"0HPU3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Left Breast, Percutaneous Approach */
"0HPU70Z"  =  "1"  /* Removal of Drainage Device from Left Breast, Via Natural or Artificial Opening */
"0HPU71Z"  =  "1"  /* Removal of Radioactive Element from Left Breast, Via Natural or Artificial Opening */
"0HPU77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Left Breast, Via Natural or Artificial Opening */
"0HPU7JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Breast, Via Natural or Artificial Opening */
"0HPU7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Left Breast, Via Natural or Artificial Opening */
"0HPU7NZ"  =  "1"  /* Removal of Tissue Expander from Left Breast, Via Natural or Artificial Opening */
"0HPU80Z"  =  "1"  /* Removal of Drainage Device from Left Breast, Via Natural or Artificial Opening Endoscopic */
"0HPU81Z"  =  "1"  /* Removal of Radioactive Element from Left Breast, Via Natural or Artificial Opening Endoscopic */
"0HPU87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Left Breast, Via Natural or Artificial Opening Endoscopic */
"0HPU8JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Breast, Via Natural or Artificial Opening Endoscopic */
"0HPU8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Left Breast, Via Natural or Artificial Opening Endoscopic */
"0HPU8NZ"  =  "1"  /* Removal of Tissue Expander from Left Breast, Via Natural or Artificial Opening Endoscopic */
"0HWT00Z"  =  "1"  /* Revision of Drainage Device in Right Breast, Open Approach */
"0HWT07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Right Breast, Open Approach */
"0HWT0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Right Breast, Open Approach */
"0HWT0NZ"  =  "1"  /* Revision of Tissue Expander in Right Breast, Open Approach */
"0HWT30Z"  =  "1"  /* Revision of Drainage Device in Right Breast, Percutaneous Approach */
"0HWT37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Right Breast, Percutaneous Approach */
"0HWT3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Right Breast, Percutaneous Approach */
"0HWT3NZ"  =  "1"  /* Revision of Tissue Expander in Right Breast, Percutaneous Approach */
"0HWT70Z"  =  "1"  /* Revision of Drainage Device in Right Breast, Via Natural or Artificial Opening */
"0HWT77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Right Breast, Via Natural or Artificial Opening */
"0HWT7JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Breast, Via Natural or Artificial Opening */
"0HWT7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Right Breast, Via Natural or Artificial Opening */
"0HWT7NZ"  =  "1"  /* Revision of Tissue Expander in Right Breast, Via Natural or Artificial Opening */
"0HWT80Z"  =  "1"  /* Revision of Drainage Device in Right Breast, Via Natural or Artificial Opening Endoscopic */
"0HWT87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Right Breast, Via Natural or Artificial Opening Endoscopic */
"0HWT8JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Breast, Via Natural or Artificial Opening Endoscopic */
"0HWT8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Right Breast, Via Natural or Artificial Opening Endoscopic */
"0HWT8NZ"  =  "1"  /* Revision of Tissue Expander in Right Breast, Via Natural or Artificial Opening Endoscopic */
"0HWU00Z"  =  "1"  /* Revision of Drainage Device in Left Breast, Open Approach */
"0HWU07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Left Breast, Open Approach */
"0HWU0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Left Breast, Open Approach */
"0HWU0NZ"  =  "1"  /* Revision of Tissue Expander in Left Breast, Open Approach */
"0HWU30Z"  =  "1"  /* Revision of Drainage Device in Left Breast, Percutaneous Approach */
"0HWU37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Left Breast, Percutaneous Approach */
"0HWU3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Left Breast, Percutaneous Approach */
"0HWU3NZ"  =  "1"  /* Revision of Tissue Expander in Left Breast, Percutaneous Approach */
"0HWU70Z"  =  "1"  /* Revision of Drainage Device in Left Breast, Via Natural or Artificial Opening */
"0HWU77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Left Breast, Via Natural or Artificial Opening */
"0HWU7JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Breast, Via Natural or Artificial Opening */
"0HWU7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Left Breast, Via Natural or Artificial Opening */
"0HWU7NZ"  =  "1"  /* Revision of Tissue Expander in Left Breast, Via Natural or Artificial Opening */
"0HWU80Z"  =  "1"  /* Revision of Drainage Device in Left Breast, Via Natural or Artificial Opening Endoscopic */
"0HWU87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Left Breast, Via Natural or Artificial Opening Endoscopic */
"0HWU8JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Breast, Via Natural or Artificial Opening Endoscopic */
"0HWU8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Left Breast, Via Natural or Artificial Opening Endoscopic */
"0HWU8NZ"  =  "1"  /* Revision of Tissue Expander in Left Breast, Via Natural or Artificial Opening Endoscopic */
"0J810ZZ"  =  "1"  /* Division of Face Subcutaneous Tissue and Fascia, Open Approach */
"0J813ZZ"  =  "1"  /* Division of Face Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0J8J0ZZ"  =  "1"  /* Division of Right Hand Subcutaneous Tissue and Fascia, Open Approach */
"0J8J3ZZ"  =  "1"  /* Division of Right Hand Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0J8K0ZZ"  =  "1"  /* Division of Left Hand Subcutaneous Tissue and Fascia, Open Approach */
"0J8K3ZZ"  =  "1"  /* Division of Left Hand Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0J9000Z"  =  "1"  /* Drainage of Scalp Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J900ZZ"  =  "1"  /* Drainage of Scalp Subcutaneous Tissue and Fascia, Open Approach */
"0J9100Z"  =  "1"  /* Drainage of Face Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J910ZZ"  =  "1"  /* Drainage of Face Subcutaneous Tissue and Fascia, Open Approach */
"0J9130Z"  =  "1"  /* Drainage of Face Subcutaneous Tissue and Fascia with Drainage Device, Percutaneous Approach */
"0J9400Z"  =  "1"  /* Drainage of Anterior Neck Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J940ZZ"  =  "1"  /* Drainage of Anterior Neck Subcutaneous Tissue and Fascia, Open Approach */
"0J9500Z"  =  "1"  /* Drainage of Posterior Neck Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J950ZZ"  =  "1"  /* Drainage of Posterior Neck Subcutaneous Tissue and Fascia, Open Approach */
"0J9600Z"  =  "1"  /* Drainage of Chest Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J960ZZ"  =  "1"  /* Drainage of Chest Subcutaneous Tissue and Fascia, Open Approach */
"0J9700Z"  =  "1"  /* Drainage of Back Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J970ZZ"  =  "1"  /* Drainage of Back Subcutaneous Tissue and Fascia, Open Approach */
"0J9800Z"  =  "1"  /* Drainage of Abdomen Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J980ZZ"  =  "1"  /* Drainage of Abdomen Subcutaneous Tissue and Fascia, Open Approach */
"0J9900Z"  =  "1"  /* Drainage of Buttock Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J990ZZ"  =  "1"  /* Drainage of Buttock Subcutaneous Tissue and Fascia, Open Approach */
"0J9B00Z"  =  "1"  /* Drainage of Perineum Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J9B0ZZ"  =  "1"  /* Drainage of Perineum Subcutaneous Tissue and Fascia, Open Approach */
"0J9C00Z"  =  "1"  /* Drainage of Pelvic Region Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J9C0ZZ"  =  "1"  /* Drainage of Pelvic Region Subcutaneous Tissue and Fascia, Open Approach */
"0J9D00Z"  =  "1"  /* Drainage of Right Upper Arm Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J9D0ZZ"  =  "1"  /* Drainage of Right Upper Arm Subcutaneous Tissue and Fascia, Open Approach */
"0J9F00Z"  =  "1"  /* Drainage of Left Upper Arm Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J9F0ZZ"  =  "1"  /* Drainage of Left Upper Arm Subcutaneous Tissue and Fascia, Open Approach */
"0J9G00Z"  =  "1"  /* Drainage of Right Lower Arm Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J9G0ZZ"  =  "1"  /* Drainage of Right Lower Arm Subcutaneous Tissue and Fascia, Open Approach */
"0J9H00Z"  =  "1"  /* Drainage of Left Lower Arm Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J9H0ZZ"  =  "1"  /* Drainage of Left Lower Arm Subcutaneous Tissue and Fascia, Open Approach */
"0J9J00Z"  =  "1"  /* Drainage of Right Hand Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J9J0ZZ"  =  "1"  /* Drainage of Right Hand Subcutaneous Tissue and Fascia, Open Approach */
"0J9J30Z"  =  "1"  /* Drainage of Right Hand Subcutaneous Tissue and Fascia with Drainage Device, Percutaneous Approach */
"0J9K00Z"  =  "1"  /* Drainage of Left Hand Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J9K0ZZ"  =  "1"  /* Drainage of Left Hand Subcutaneous Tissue and Fascia, Open Approach */
"0J9K30Z"  =  "1"  /* Drainage of Left Hand Subcutaneous Tissue and Fascia with Drainage Device, Percutaneous Approach */
"0J9K3ZZ"  =  "1"  /* Drainage of Left Hand Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0J9L00Z"  =  "1"  /* Drainage of Right Upper Leg Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J9L0ZZ"  =  "1"  /* Drainage of Right Upper Leg Subcutaneous Tissue and Fascia, Open Approach */
"0J9M00Z"  =  "1"  /* Drainage of Left Upper Leg Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J9M0ZZ"  =  "1"  /* Drainage of Left Upper Leg Subcutaneous Tissue and Fascia, Open Approach */
"0J9N00Z"  =  "1"  /* Drainage of Right Lower Leg Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J9N0ZZ"  =  "1"  /* Drainage of Right Lower Leg Subcutaneous Tissue and Fascia, Open Approach */
"0J9P00Z"  =  "1"  /* Drainage of Left Lower Leg Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J9P0ZZ"  =  "1"  /* Drainage of Left Lower Leg Subcutaneous Tissue and Fascia, Open Approach */
"0J9Q00Z"  =  "1"  /* Drainage of Right Foot Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J9Q0ZZ"  =  "1"  /* Drainage of Right Foot Subcutaneous Tissue and Fascia, Open Approach */
"0J9R00Z"  =  "1"  /* Drainage of Left Foot Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J9R0ZZ"  =  "1"  /* Drainage of Left Foot Subcutaneous Tissue and Fascia, Open Approach */
"0JC00ZZ"  =  "1"  /* Extirpation of Matter from Scalp Subcutaneous Tissue and Fascia, Open Approach */
"0JC03ZZ"  =  "1"  /* Extirpation of Matter from Scalp Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JC10ZZ"  =  "1"  /* Extirpation of Matter from Face Subcutaneous Tissue and Fascia, Open Approach */
"0JC13ZZ"  =  "1"  /* Extirpation of Matter from Face Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JC40ZZ"  =  "1"  /* Extirpation of Matter from Anterior Neck Subcutaneous Tissue and Fascia, Open Approach */
"0JC43ZZ"  =  "1"  /* Extirpation of Matter from Anterior Neck Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JC50ZZ"  =  "1"  /* Extirpation of Matter from Posterior Neck Subcutaneous Tissue and Fascia, Open Approach */
"0JC53ZZ"  =  "1"  /* Extirpation of Matter from Posterior Neck Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JC60ZZ"  =  "1"  /* Extirpation of Matter from Chest Subcutaneous Tissue and Fascia, Open Approach */
"0JC63ZZ"  =  "1"  /* Extirpation of Matter from Chest Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JC70ZZ"  =  "1"  /* Extirpation of Matter from Back Subcutaneous Tissue and Fascia, Open Approach */
"0JC73ZZ"  =  "1"  /* Extirpation of Matter from Back Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JC80ZZ"  =  "1"  /* Extirpation of Matter from Abdomen Subcutaneous Tissue and Fascia, Open Approach */
"0JC83ZZ"  =  "1"  /* Extirpation of Matter from Abdomen Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JC90ZZ"  =  "1"  /* Extirpation of Matter from Buttock Subcutaneous Tissue and Fascia, Open Approach */
"0JC93ZZ"  =  "1"  /* Extirpation of Matter from Buttock Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JCB0ZZ"  =  "1"  /* Extirpation of Matter from Perineum Subcutaneous Tissue and Fascia, Open Approach */
"0JCB3ZZ"  =  "1"  /* Extirpation of Matter from Perineum Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JCC0ZZ"  =  "1"  /* Extirpation of Matter from Pelvic Region Subcutaneous Tissue and Fascia, Open Approach */
"0JCC3ZZ"  =  "1"  /* Extirpation of Matter from Pelvic Region Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JCD0ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Arm Subcutaneous Tissue and Fascia, Open Approach */
"0JCD3ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Arm Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JCF0ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Arm Subcutaneous Tissue and Fascia, Open Approach */
"0JCF3ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Arm Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JCG0ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Arm Subcutaneous Tissue and Fascia, Open Approach */
"0JCG3ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Arm Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JCH0ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Arm Subcutaneous Tissue and Fascia, Open Approach */
"0JCH3ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Arm Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JCJ0ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Subcutaneous Tissue and Fascia, Open Approach */
"0JCJ3ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JCK0ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Subcutaneous Tissue and Fascia, Open Approach */
"0JCK3ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JCL0ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Leg Subcutaneous Tissue and Fascia, Open Approach */
"0JCL3ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Leg Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JCM0ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Leg Subcutaneous Tissue and Fascia, Open Approach */
"0JCM3ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Leg Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JCN0ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Leg Subcutaneous Tissue and Fascia, Open Approach */
"0JCN3ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Leg Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JCP0ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Leg Subcutaneous Tissue and Fascia, Open Approach */
"0JCP3ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Leg Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JCQ0ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Subcutaneous Tissue and Fascia, Open Approach */
"0JCQ3ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JCR0ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Subcutaneous Tissue and Fascia, Open Approach */
"0JCR3ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JH602Z"  =  "1"  /* Insertion of Monitoring Device into Chest Subcutaneous Tissue and Fascia, Open Approach */
"0JH60HZ"  =  "1"  /* Insertion of Contraceptive Device into Chest Subcutaneous Tissue and Fascia, Open Approach */
"0JH632Z"  =  "1"  /* Insertion of Monitoring Device into Chest Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JH63HZ"  =  "1"  /* Insertion of Contraceptive Device into Chest Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JH802Z"  =  "1"  /* Insertion of Monitoring Device into Abdomen Subcutaneous Tissue and Fascia, Open Approach */
"0JH80HZ"  =  "1"  /* Insertion of Contraceptive Device into Abdomen Subcutaneous Tissue and Fascia, Open Approach */
"0JH832Z"  =  "1"  /* Insertion of Monitoring Device into Abdomen Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JH83HZ"  =  "1"  /* Insertion of Contraceptive Device into Abdomen Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JHN3HZ"  =  "1"  /* Insertion of Contraceptive Device into Right Lower Leg Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JHP0HZ"  =  "1"  /* Insertion of Contraceptive Device into Left Lower Leg Subcutaneous Tissue and Fascia, Open Approach */
"0JHP3HZ"  =  "1"  /* Insertion of Contraceptive Device into Left Lower Leg Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JJS0ZZ"  =  "1"  /* Inspection of Head and Neck Subcutaneous Tissue and Fascia, Open Approach */
"0JJT0ZZ"  =  "1"  /* Inspection of Trunk Subcutaneous Tissue and Fascia, Open Approach */
"0JJV0ZZ"  =  "1"  /* Inspection of Upper Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JJW0ZZ"  =  "1"  /* Inspection of Lower Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JPT0PZ"  =  "1"  /* Removal of Cardiac Rhythm Related Device from Trunk Subcutaneous Tissue and Fascia, Open Approach */
"0JPT3PZ"  =  "1"  /* Removal of Cardiac Rhythm Related Device from Trunk Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWS00Z"  =  "1"  /* Revision of Drainage Device in Head and Neck Subcutaneous Tissue and Fascia, Open Approach */
"0JWS03Z"  =  "1"  /* Revision of Infusion Device in Head and Neck Subcutaneous Tissue and Fascia, Open Approach */
"0JWS07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Head and Neck Subcutaneous Tissue and Fascia, Open Approach */
"0JWS0JZ"  =  "1"  /* Revision of Synthetic Substitute in Head and Neck Subcutaneous Tissue and Fascia, Open Approach */
"0JWS0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Head and Neck Subcutaneous Tissue and Fascia, Open Approach */
"0JWS0NZ"  =  "1"  /* Revision of Tissue Expander in Head and Neck Subcutaneous Tissue and Fascia, Open Approach */
"0JWS30Z"  =  "1"  /* Revision of Drainage Device in Head and Neck Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWS33Z"  =  "1"  /* Revision of Infusion Device in Head and Neck Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWS37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Head and Neck Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWS3JZ"  =  "1"  /* Revision of Synthetic Substitute in Head and Neck Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWS3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Head and Neck Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWS3NZ"  =  "1"  /* Revision of Tissue Expander in Head and Neck Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWT00Z"  =  "1"  /* Revision of Drainage Device in Trunk Subcutaneous Tissue and Fascia, Open Approach */
"0JWT02Z"  =  "1"  /* Revision of Monitoring Device in Trunk Subcutaneous Tissue and Fascia, Open Approach */
"0JWT03Z"  =  "1"  /* Revision of Infusion Device in Trunk Subcutaneous Tissue and Fascia, Open Approach */
"0JWT07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Trunk Subcutaneous Tissue and Fascia, Open Approach */
"0JWT0HZ"  =  "1"  /* Revision of Contraceptive Device in Trunk Subcutaneous Tissue and Fascia, Open Approach */
"0JWT0JZ"  =  "1"  /* Revision of Synthetic Substitute in Trunk Subcutaneous Tissue and Fascia, Open Approach */
"0JWT0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Trunk Subcutaneous Tissue and Fascia, Open Approach */
"0JWT0MZ"  =  "1"  /* Revision of Stimulator Generator in Trunk Subcutaneous Tissue and Fascia, Open Approach */
"0JWT0NZ"  =  "1"  /* Revision of Tissue Expander in Trunk Subcutaneous Tissue and Fascia, Open Approach */
"0JWT0PZ"  =  "1"  /* Revision of Cardiac Rhythm Related Device in Trunk Subcutaneous Tissue and Fascia, Open Approach */
"0JWT0VZ"  =  "1"  /* Revision of Infusion Pump in Trunk Subcutaneous Tissue and Fascia, Open Approach */
"0JWT0WZ"  =  "1"  /* Revision of Reservoir in Trunk Subcutaneous Tissue and Fascia, Open Approach */
"0JWT0XZ"  =  "1"  /* Revision of Vascular Access Device in Trunk Subcutaneous Tissue and Fascia, Open Approach */
"0JWT30Z"  =  "1"  /* Revision of Drainage Device in Trunk Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWT32Z"  =  "1"  /* Revision of Monitoring Device in Trunk Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWT33Z"  =  "1"  /* Revision of Infusion Device in Trunk Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWT37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Trunk Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWT3HZ"  =  "1"  /* Revision of Contraceptive Device in Trunk Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWT3JZ"  =  "1"  /* Revision of Synthetic Substitute in Trunk Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWT3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Trunk Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWT3MZ"  =  "1"  /* Revision of Stimulator Generator in Trunk Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWT3NZ"  =  "1"  /* Revision of Tissue Expander in Trunk Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWT3PZ"  =  "1"  /* Revision of Cardiac Rhythm Related Device in Trunk Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWT3VZ"  =  "1"  /* Revision of Infusion Pump in Trunk Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWT3WZ"  =  "1"  /* Revision of Reservoir in Trunk Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWT3XZ"  =  "1"  /* Revision of Vascular Access Device in Trunk Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWV00Z"  =  "1"  /* Revision of Drainage Device in Upper Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWV03Z"  =  "1"  /* Revision of Infusion Device in Upper Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWV07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWV0HZ"  =  "1"  /* Revision of Contraceptive Device in Upper Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWV0JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWV0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWV0NZ"  =  "1"  /* Revision of Tissue Expander in Upper Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWV0VZ"  =  "1"  /* Revision of Infusion Pump in Upper Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWV0WZ"  =  "1"  /* Revision of Reservoir in Upper Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWV0XZ"  =  "1"  /* Revision of Vascular Access Device in Upper Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWV30Z"  =  "1"  /* Revision of Drainage Device in Upper Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWV33Z"  =  "1"  /* Revision of Infusion Device in Upper Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWV37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWV3HZ"  =  "1"  /* Revision of Contraceptive Device in Upper Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWV3JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWV3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWV3NZ"  =  "1"  /* Revision of Tissue Expander in Upper Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWV3VZ"  =  "1"  /* Revision of Infusion Pump in Upper Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWV3WZ"  =  "1"  /* Revision of Reservoir in Upper Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWV3XZ"  =  "1"  /* Revision of Vascular Access Device in Upper Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWW00Z"  =  "1"  /* Revision of Drainage Device in Lower Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWW03Z"  =  "1"  /* Revision of Infusion Device in Lower Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWW07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWW0HZ"  =  "1"  /* Revision of Contraceptive Device in Lower Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWW0JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWW0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWW0NZ"  =  "1"  /* Revision of Tissue Expander in Lower Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWW0VZ"  =  "1"  /* Revision of Infusion Pump in Lower Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWW0WZ"  =  "1"  /* Revision of Reservoir in Lower Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWW0XZ"  =  "1"  /* Revision of Vascular Access Device in Lower Extremity Subcutaneous Tissue and Fascia, Open Approach */
"0JWW30Z"  =  "1"  /* Revision of Drainage Device in Lower Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWW33Z"  =  "1"  /* Revision of Infusion Device in Lower Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWW37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWW3HZ"  =  "1"  /* Revision of Contraceptive Device in Lower Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWW3JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWW3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWW3NZ"  =  "1"  /* Revision of Tissue Expander in Lower Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWW3VZ"  =  "1"  /* Revision of Infusion Pump in Lower Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWW3WZ"  =  "1"  /* Revision of Reservoir in Lower Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0JWW3XZ"  =  "1"  /* Revision of Vascular Access Device in Lower Extremity Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0K9000Z"  =  "1"  /* Drainage of Head Muscle with Drainage Device, Open Approach */
"0K900ZZ"  =  "1"  /* Drainage of Head Muscle, Open Approach */
"0K9030Z"  =  "1"  /* Drainage of Head Muscle with Drainage Device, Percutaneous Approach */
"0K9040Z"  =  "1"  /* Drainage of Head Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K904ZZ"  =  "1"  /* Drainage of Head Muscle, Percutaneous Endoscopic Approach */
"0K9100Z"  =  "1"  /* Drainage of Facial Muscle with Drainage Device, Open Approach */
"0K910ZZ"  =  "1"  /* Drainage of Facial Muscle, Open Approach */
"0K9130Z"  =  "1"  /* Drainage of Facial Muscle with Drainage Device, Percutaneous Approach */
"0K9140Z"  =  "1"  /* Drainage of Facial Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K914ZZ"  =  "1"  /* Drainage of Facial Muscle, Percutaneous Endoscopic Approach */
"0K9200Z"  =  "1"  /* Drainage of Right Neck Muscle with Drainage Device, Open Approach */
"0K920ZZ"  =  "1"  /* Drainage of Right Neck Muscle, Open Approach */
"0K9230Z"  =  "1"  /* Drainage of Right Neck Muscle with Drainage Device, Percutaneous Approach */
"0K9240Z"  =  "1"  /* Drainage of Right Neck Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K924ZZ"  =  "1"  /* Drainage of Right Neck Muscle, Percutaneous Endoscopic Approach */
"0K9300Z"  =  "1"  /* Drainage of Left Neck Muscle with Drainage Device, Open Approach */
"0K930ZZ"  =  "1"  /* Drainage of Left Neck Muscle, Open Approach */
"0K9330Z"  =  "1"  /* Drainage of Left Neck Muscle with Drainage Device, Percutaneous Approach */
"0K9340Z"  =  "1"  /* Drainage of Left Neck Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K934ZZ"  =  "1"  /* Drainage of Left Neck Muscle, Percutaneous Endoscopic Approach */
"0K9400Z"  =  "1"  /* Drainage of Tongue, Palate, Pharynx Muscle with Drainage Device, Open Approach */
"0K940ZZ"  =  "1"  /* Drainage of Tongue, Palate, Pharynx Muscle, Open Approach */
"0K9430Z"  =  "1"  /* Drainage of Tongue, Palate, Pharynx Muscle with Drainage Device, Percutaneous Approach */
"0K9440Z"  =  "1"  /* Drainage of Tongue, Palate, Pharynx Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K944ZZ"  =  "1"  /* Drainage of Tongue, Palate, Pharynx Muscle, Percutaneous Endoscopic Approach */
"0K9500Z"  =  "1"  /* Drainage of Right Shoulder Muscle with Drainage Device, Open Approach */
"0K950ZZ"  =  "1"  /* Drainage of Right Shoulder Muscle, Open Approach */
"0K9530Z"  =  "1"  /* Drainage of Right Shoulder Muscle with Drainage Device, Percutaneous Approach */
"0K9540Z"  =  "1"  /* Drainage of Right Shoulder Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K954ZZ"  =  "1"  /* Drainage of Right Shoulder Muscle, Percutaneous Endoscopic Approach */
"0K9600Z"  =  "1"  /* Drainage of Left Shoulder Muscle with Drainage Device, Open Approach */
"0K960ZZ"  =  "1"  /* Drainage of Left Shoulder Muscle, Open Approach */
"0K9630Z"  =  "1"  /* Drainage of Left Shoulder Muscle with Drainage Device, Percutaneous Approach */
"0K9640Z"  =  "1"  /* Drainage of Left Shoulder Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K964ZZ"  =  "1"  /* Drainage of Left Shoulder Muscle, Percutaneous Endoscopic Approach */
"0K9700Z"  =  "1"  /* Drainage of Right Upper Arm Muscle with Drainage Device, Open Approach */
"0K970ZZ"  =  "1"  /* Drainage of Right Upper Arm Muscle, Open Approach */
"0K9730Z"  =  "1"  /* Drainage of Right Upper Arm Muscle with Drainage Device, Percutaneous Approach */
"0K9740Z"  =  "1"  /* Drainage of Right Upper Arm Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K974ZZ"  =  "1"  /* Drainage of Right Upper Arm Muscle, Percutaneous Endoscopic Approach */
"0K9800Z"  =  "1"  /* Drainage of Left Upper Arm Muscle with Drainage Device, Open Approach */
"0K980ZZ"  =  "1"  /* Drainage of Left Upper Arm Muscle, Open Approach */
"0K9830Z"  =  "1"  /* Drainage of Left Upper Arm Muscle with Drainage Device, Percutaneous Approach */
"0K9840Z"  =  "1"  /* Drainage of Left Upper Arm Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K984ZZ"  =  "1"  /* Drainage of Left Upper Arm Muscle, Percutaneous Endoscopic Approach */
"0K9900Z"  =  "1"  /* Drainage of Right Lower Arm and Wrist Muscle with Drainage Device, Open Approach */
"0K990ZZ"  =  "1"  /* Drainage of Right Lower Arm and Wrist Muscle, Open Approach */
"0K9930Z"  =  "1"  /* Drainage of Right Lower Arm and Wrist Muscle with Drainage Device, Percutaneous Approach */
"0K9940Z"  =  "1"  /* Drainage of Right Lower Arm and Wrist Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K994ZZ"  =  "1"  /* Drainage of Right Lower Arm and Wrist Muscle, Percutaneous Endoscopic Approach */
"0K9B00Z"  =  "1"  /* Drainage of Left Lower Arm and Wrist Muscle with Drainage Device, Open Approach */
"0K9B0ZZ"  =  "1"  /* Drainage of Left Lower Arm and Wrist Muscle, Open Approach */
"0K9B30Z"  =  "1"  /* Drainage of Left Lower Arm and Wrist Muscle with Drainage Device, Percutaneous Approach */
"0K9B40Z"  =  "1"  /* Drainage of Left Lower Arm and Wrist Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9B4ZZ"  =  "1"  /* Drainage of Left Lower Arm and Wrist Muscle, Percutaneous Endoscopic Approach */
"0K9C00Z"  =  "1"  /* Drainage of Right Hand Muscle with Drainage Device, Open Approach */
"0K9C0ZZ"  =  "1"  /* Drainage of Right Hand Muscle, Open Approach */
"0K9C30Z"  =  "1"  /* Drainage of Right Hand Muscle with Drainage Device, Percutaneous Approach */
"0K9C40Z"  =  "1"  /* Drainage of Right Hand Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9D00Z"  =  "1"  /* Drainage of Left Hand Muscle with Drainage Device, Open Approach */
"0K9D0ZZ"  =  "1"  /* Drainage of Left Hand Muscle, Open Approach */
"0K9D30Z"  =  "1"  /* Drainage of Left Hand Muscle with Drainage Device, Percutaneous Approach */
"0K9D40Z"  =  "1"  /* Drainage of Left Hand Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9F00Z"  =  "1"  /* Drainage of Right Trunk Muscle with Drainage Device, Open Approach */
"0K9F0ZZ"  =  "1"  /* Drainage of Right Trunk Muscle, Open Approach */
"0K9F30Z"  =  "1"  /* Drainage of Right Trunk Muscle with Drainage Device, Percutaneous Approach */
"0K9F40Z"  =  "1"  /* Drainage of Right Trunk Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9F4ZZ"  =  "1"  /* Drainage of Right Trunk Muscle, Percutaneous Endoscopic Approach */
"0K9G00Z"  =  "1"  /* Drainage of Left Trunk Muscle with Drainage Device, Open Approach */
"0K9G0ZZ"  =  "1"  /* Drainage of Left Trunk Muscle, Open Approach */
"0K9G30Z"  =  "1"  /* Drainage of Left Trunk Muscle with Drainage Device, Percutaneous Approach */
"0K9G40Z"  =  "1"  /* Drainage of Left Trunk Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9G4ZZ"  =  "1"  /* Drainage of Left Trunk Muscle, Percutaneous Endoscopic Approach */
"0K9H00Z"  =  "1"  /* Drainage of Right Thorax Muscle with Drainage Device, Open Approach */
"0K9H0ZZ"  =  "1"  /* Drainage of Right Thorax Muscle, Open Approach */
"0K9H30Z"  =  "1"  /* Drainage of Right Thorax Muscle with Drainage Device, Percutaneous Approach */
"0K9H40Z"  =  "1"  /* Drainage of Right Thorax Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9H4ZZ"  =  "1"  /* Drainage of Right Thorax Muscle, Percutaneous Endoscopic Approach */
"0K9J00Z"  =  "1"  /* Drainage of Left Thorax Muscle with Drainage Device, Open Approach */
"0K9J0ZZ"  =  "1"  /* Drainage of Left Thorax Muscle, Open Approach */
"0K9J30Z"  =  "1"  /* Drainage of Left Thorax Muscle with Drainage Device, Percutaneous Approach */
"0K9J40Z"  =  "1"  /* Drainage of Left Thorax Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9J4ZZ"  =  "1"  /* Drainage of Left Thorax Muscle, Percutaneous Endoscopic Approach */
"0K9K00Z"  =  "1"  /* Drainage of Right Abdomen Muscle with Drainage Device, Open Approach */
"0K9K0ZZ"  =  "1"  /* Drainage of Right Abdomen Muscle, Open Approach */
"0K9K30Z"  =  "1"  /* Drainage of Right Abdomen Muscle with Drainage Device, Percutaneous Approach */
"0K9K40Z"  =  "1"  /* Drainage of Right Abdomen Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9K4ZZ"  =  "1"  /* Drainage of Right Abdomen Muscle, Percutaneous Endoscopic Approach */
"0K9L00Z"  =  "1"  /* Drainage of Left Abdomen Muscle with Drainage Device, Open Approach */
"0K9L0ZZ"  =  "1"  /* Drainage of Left Abdomen Muscle, Open Approach */
"0K9L30Z"  =  "1"  /* Drainage of Left Abdomen Muscle with Drainage Device, Percutaneous Approach */
"0K9L40Z"  =  "1"  /* Drainage of Left Abdomen Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9L4ZZ"  =  "1"  /* Drainage of Left Abdomen Muscle, Percutaneous Endoscopic Approach */
"0K9M00Z"  =  "1"  /* Drainage of Perineum Muscle with Drainage Device, Open Approach */
"0K9M0ZZ"  =  "1"  /* Drainage of Perineum Muscle, Open Approach */
"0K9M30Z"  =  "1"  /* Drainage of Perineum Muscle with Drainage Device, Percutaneous Approach */
"0K9M40Z"  =  "1"  /* Drainage of Perineum Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9M4ZZ"  =  "1"  /* Drainage of Perineum Muscle, Percutaneous Endoscopic Approach */
"0K9N00Z"  =  "1"  /* Drainage of Right Hip Muscle with Drainage Device, Open Approach */
"0K9N0ZZ"  =  "1"  /* Drainage of Right Hip Muscle, Open Approach */
"0K9N30Z"  =  "1"  /* Drainage of Right Hip Muscle with Drainage Device, Percutaneous Approach */
"0K9N40Z"  =  "1"  /* Drainage of Right Hip Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9N4ZZ"  =  "1"  /* Drainage of Right Hip Muscle, Percutaneous Endoscopic Approach */
"0K9P00Z"  =  "1"  /* Drainage of Left Hip Muscle with Drainage Device, Open Approach */
"0K9P0ZZ"  =  "1"  /* Drainage of Left Hip Muscle, Open Approach */
"0K9P30Z"  =  "1"  /* Drainage of Left Hip Muscle with Drainage Device, Percutaneous Approach */
"0K9P40Z"  =  "1"  /* Drainage of Left Hip Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9P4ZZ"  =  "1"  /* Drainage of Left Hip Muscle, Percutaneous Endoscopic Approach */
"0K9Q00Z"  =  "1"  /* Drainage of Right Upper Leg Muscle with Drainage Device, Open Approach */
"0K9Q0ZZ"  =  "1"  /* Drainage of Right Upper Leg Muscle, Open Approach */
"0K9Q30Z"  =  "1"  /* Drainage of Right Upper Leg Muscle with Drainage Device, Percutaneous Approach */
"0K9Q40Z"  =  "1"  /* Drainage of Right Upper Leg Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9Q4ZZ"  =  "1"  /* Drainage of Right Upper Leg Muscle, Percutaneous Endoscopic Approach */
"0K9R00Z"  =  "1"  /* Drainage of Left Upper Leg Muscle with Drainage Device, Open Approach */
"0K9R0ZZ"  =  "1"  /* Drainage of Left Upper Leg Muscle, Open Approach */
"0K9R30Z"  =  "1"  /* Drainage of Left Upper Leg Muscle with Drainage Device, Percutaneous Approach */
"0K9R40Z"  =  "1"  /* Drainage of Left Upper Leg Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9R4ZZ"  =  "1"  /* Drainage of Left Upper Leg Muscle, Percutaneous Endoscopic Approach */
"0K9S00Z"  =  "1"  /* Drainage of Right Lower Leg Muscle with Drainage Device, Open Approach */
"0K9S0ZZ"  =  "1"  /* Drainage of Right Lower Leg Muscle, Open Approach */
"0K9S30Z"  =  "1"  /* Drainage of Right Lower Leg Muscle with Drainage Device, Percutaneous Approach */
"0K9S40Z"  =  "1"  /* Drainage of Right Lower Leg Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9S4ZZ"  =  "1"  /* Drainage of Right Lower Leg Muscle, Percutaneous Endoscopic Approach */
"0K9T00Z"  =  "1"  /* Drainage of Left Lower Leg Muscle with Drainage Device, Open Approach */
"0K9T0ZZ"  =  "1"  /* Drainage of Left Lower Leg Muscle, Open Approach */
"0K9T30Z"  =  "1"  /* Drainage of Left Lower Leg Muscle with Drainage Device, Percutaneous Approach */
"0K9T40Z"  =  "1"  /* Drainage of Left Lower Leg Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9T4ZZ"  =  "1"  /* Drainage of Left Lower Leg Muscle, Percutaneous Endoscopic Approach */
"0K9V00Z"  =  "1"  /* Drainage of Right Foot Muscle with Drainage Device, Open Approach */
"0K9V0ZZ"  =  "1"  /* Drainage of Right Foot Muscle, Open Approach */
"0K9V30Z"  =  "1"  /* Drainage of Right Foot Muscle with Drainage Device, Percutaneous Approach */
"0K9V40Z"  =  "1"  /* Drainage of Right Foot Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9V4ZZ"  =  "1"  /* Drainage of Right Foot Muscle, Percutaneous Endoscopic Approach */
"0K9W00Z"  =  "1"  /* Drainage of Left Foot Muscle with Drainage Device, Open Approach */
"0K9W0ZZ"  =  "1"  /* Drainage of Left Foot Muscle, Open Approach */
"0K9W30Z"  =  "1"  /* Drainage of Left Foot Muscle with Drainage Device, Percutaneous Approach */
"0K9W40Z"  =  "1"  /* Drainage of Left Foot Muscle with Drainage Device, Percutaneous Endoscopic Approach */
"0K9W4ZZ"  =  "1"  /* Drainage of Left Foot Muscle, Percutaneous Endoscopic Approach */
"0KC00ZZ"  =  "1"  /* Extirpation of Matter from Head Muscle, Open Approach */
"0KC03ZZ"  =  "1"  /* Extirpation of Matter from Head Muscle, Percutaneous Approach */
"0KC04ZZ"  =  "1"  /* Extirpation of Matter from Head Muscle, Percutaneous Endoscopic Approach */
"0KC10ZZ"  =  "1"  /* Extirpation of Matter from Facial Muscle, Open Approach */
"0KC13ZZ"  =  "1"  /* Extirpation of Matter from Facial Muscle, Percutaneous Approach */
"0KC14ZZ"  =  "1"  /* Extirpation of Matter from Facial Muscle, Percutaneous Endoscopic Approach */
"0KC20ZZ"  =  "1"  /* Extirpation of Matter from Right Neck Muscle, Open Approach */
"0KC23ZZ"  =  "1"  /* Extirpation of Matter from Right Neck Muscle, Percutaneous Approach */
"0KC24ZZ"  =  "1"  /* Extirpation of Matter from Right Neck Muscle, Percutaneous Endoscopic Approach */
"0KC30ZZ"  =  "1"  /* Extirpation of Matter from Left Neck Muscle, Open Approach */
"0KC33ZZ"  =  "1"  /* Extirpation of Matter from Left Neck Muscle, Percutaneous Approach */
"0KC34ZZ"  =  "1"  /* Extirpation of Matter from Left Neck Muscle, Percutaneous Endoscopic Approach */
"0KC40ZZ"  =  "1"  /* Extirpation of Matter from Tongue, Palate, Pharynx Muscle, Open Approach */
"0KC43ZZ"  =  "1"  /* Extirpation of Matter from Tongue, Palate, Pharynx Muscle, Percutaneous Approach */
"0KC44ZZ"  =  "1"  /* Extirpation of Matter from Tongue, Palate, Pharynx Muscle, Percutaneous Endoscopic Approach */
"0KC50ZZ"  =  "1"  /* Extirpation of Matter from Right Shoulder Muscle, Open Approach */
"0KC53ZZ"  =  "1"  /* Extirpation of Matter from Right Shoulder Muscle, Percutaneous Approach */
"0KC54ZZ"  =  "1"  /* Extirpation of Matter from Right Shoulder Muscle, Percutaneous Endoscopic Approach */
"0KC60ZZ"  =  "1"  /* Extirpation of Matter from Left Shoulder Muscle, Open Approach */
"0KC63ZZ"  =  "1"  /* Extirpation of Matter from Left Shoulder Muscle, Percutaneous Approach */
"0KC64ZZ"  =  "1"  /* Extirpation of Matter from Left Shoulder Muscle, Percutaneous Endoscopic Approach */
"0KC70ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Arm Muscle, Open Approach */
"0KC73ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Arm Muscle, Percutaneous Approach */
"0KC74ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Arm Muscle, Percutaneous Endoscopic Approach */
"0KC80ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Arm Muscle, Open Approach */
"0KC83ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Arm Muscle, Percutaneous Approach */
"0KC84ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Arm Muscle, Percutaneous Endoscopic Approach */
"0KC90ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Arm and Wrist Muscle, Open Approach */
"0KC93ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Arm and Wrist Muscle, Percutaneous Approach */
"0KC94ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Arm and Wrist Muscle, Percutaneous Endoscopic Approach */
"0KCB0ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Arm and Wrist Muscle, Open Approach */
"0KCB3ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Arm and Wrist Muscle, Percutaneous Approach */
"0KCB4ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Arm and Wrist Muscle, Percutaneous Endoscopic Approach */
"0KCC0ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Muscle, Open Approach */
"0KCC3ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Muscle, Percutaneous Approach */
"0KCC4ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Muscle, Percutaneous Endoscopic Approach */
"0KCD0ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Muscle, Open Approach */
"0KCD3ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Muscle, Percutaneous Approach */
"0KCD4ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Muscle, Percutaneous Endoscopic Approach */
"0KCF0ZZ"  =  "1"  /* Extirpation of Matter from Right Trunk Muscle, Open Approach */
"0KCF3ZZ"  =  "1"  /* Extirpation of Matter from Right Trunk Muscle, Percutaneous Approach */
"0KCF4ZZ"  =  "1"  /* Extirpation of Matter from Right Trunk Muscle, Percutaneous Endoscopic Approach */
"0KCG0ZZ"  =  "1"  /* Extirpation of Matter from Left Trunk Muscle, Open Approach */
"0KCG3ZZ"  =  "1"  /* Extirpation of Matter from Left Trunk Muscle, Percutaneous Approach */
"0KCG4ZZ"  =  "1"  /* Extirpation of Matter from Left Trunk Muscle, Percutaneous Endoscopic Approach */
"0KCH0ZZ"  =  "1"  /* Extirpation of Matter from Right Thorax Muscle, Open Approach */
"0KCH3ZZ"  =  "1"  /* Extirpation of Matter from Right Thorax Muscle, Percutaneous Approach */
"0KCH4ZZ"  =  "1"  /* Extirpation of Matter from Right Thorax Muscle, Percutaneous Endoscopic Approach */
"0KCJ0ZZ"  =  "1"  /* Extirpation of Matter from Left Thorax Muscle, Open Approach */
"0KCJ3ZZ"  =  "1"  /* Extirpation of Matter from Left Thorax Muscle, Percutaneous Approach */
"0KCJ4ZZ"  =  "1"  /* Extirpation of Matter from Left Thorax Muscle, Percutaneous Endoscopic Approach */
"0KCK0ZZ"  =  "1"  /* Extirpation of Matter from Right Abdomen Muscle, Open Approach */
"0KCK3ZZ"  =  "1"  /* Extirpation of Matter from Right Abdomen Muscle, Percutaneous Approach */
"0KCK4ZZ"  =  "1"  /* Extirpation of Matter from Right Abdomen Muscle, Percutaneous Endoscopic Approach */
"0KCL0ZZ"  =  "1"  /* Extirpation of Matter from Left Abdomen Muscle, Open Approach */
"0KCL3ZZ"  =  "1"  /* Extirpation of Matter from Left Abdomen Muscle, Percutaneous Approach */
"0KCL4ZZ"  =  "1"  /* Extirpation of Matter from Left Abdomen Muscle, Percutaneous Endoscopic Approach */
"0KCM0ZZ"  =  "1"  /* Extirpation of Matter from Perineum Muscle, Open Approach */
"0KCM3ZZ"  =  "1"  /* Extirpation of Matter from Perineum Muscle, Percutaneous Approach */
"0KCM4ZZ"  =  "1"  /* Extirpation of Matter from Perineum Muscle, Percutaneous Endoscopic Approach */
"0KCN0ZZ"  =  "1"  /* Extirpation of Matter from Right Hip Muscle, Open Approach */
"0KCN3ZZ"  =  "1"  /* Extirpation of Matter from Right Hip Muscle, Percutaneous Approach */
"0KCN4ZZ"  =  "1"  /* Extirpation of Matter from Right Hip Muscle, Percutaneous Endoscopic Approach */
"0KCP0ZZ"  =  "1"  /* Extirpation of Matter from Left Hip Muscle, Open Approach */
"0KCP3ZZ"  =  "1"  /* Extirpation of Matter from Left Hip Muscle, Percutaneous Approach */
"0KCP4ZZ"  =  "1"  /* Extirpation of Matter from Left Hip Muscle, Percutaneous Endoscopic Approach */
"0KCQ0ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Leg Muscle, Open Approach */
"0KCQ3ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Leg Muscle, Percutaneous Approach */
"0KCQ4ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Leg Muscle, Percutaneous Endoscopic Approach */
"0KCR0ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Leg Muscle, Open Approach */
"0KCR3ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Leg Muscle, Percutaneous Approach */
"0KCR4ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Leg Muscle, Percutaneous Endoscopic Approach */
"0KCS0ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Leg Muscle, Open Approach */
"0KCS3ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Leg Muscle, Percutaneous Approach */
"0KCS4ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Leg Muscle, Percutaneous Endoscopic Approach */
"0KCT0ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Leg Muscle, Open Approach */
"0KCT3ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Leg Muscle, Percutaneous Approach */
"0KCT4ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Leg Muscle, Percutaneous Endoscopic Approach */
"0KCV0ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Muscle, Open Approach */
"0KCV3ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Muscle, Percutaneous Approach */
"0KCV4ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Muscle, Percutaneous Endoscopic Approach */
"0KCW0ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Muscle, Open Approach */
"0KCW3ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Muscle, Percutaneous Approach */
"0KCW4ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Muscle, Percutaneous Endoscopic Approach */
"0KJX0ZZ"  =  "1"  /* Inspection of Upper Muscle, Open Approach */
"0KJY0ZZ"  =  "1"  /* Inspection of Lower Muscle, Open Approach */
"0KPX00Z"  =  "1"  /* Removal of Drainage Device from Upper Muscle, Open Approach */
"0KPX07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Muscle, Open Approach */
"0KPX0JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Muscle, Open Approach */
"0KPX0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Muscle, Open Approach */
"0KPX30Z"  =  "1"  /* Removal of Drainage Device from Upper Muscle, Percutaneous Approach */
"0KPX37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Muscle, Percutaneous Approach */
"0KPX3JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Muscle, Percutaneous Approach */
"0KPX3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Muscle, Percutaneous Approach */
"0KPX40Z"  =  "1"  /* Removal of Drainage Device from Upper Muscle, Percutaneous Endoscopic Approach */
"0KPX47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Muscle, Percutaneous Endoscopic Approach */
"0KPX4JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Muscle, Percutaneous Endoscopic Approach */
"0KPX4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Muscle, Percutaneous Endoscopic Approach */
"0KPY00Z"  =  "1"  /* Removal of Drainage Device from Lower Muscle, Open Approach */
"0KPY07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Muscle, Open Approach */
"0KPY0JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Muscle, Open Approach */
"0KPY0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Muscle, Open Approach */
"0KPY30Z"  =  "1"  /* Removal of Drainage Device from Lower Muscle, Percutaneous Approach */
"0KPY37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Muscle, Percutaneous Approach */
"0KPY3JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Muscle, Percutaneous Approach */
"0KPY3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Muscle, Percutaneous Approach */
"0KPY40Z"  =  "1"  /* Removal of Drainage Device from Lower Muscle, Percutaneous Endoscopic Approach */
"0KPY47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Muscle, Percutaneous Endoscopic Approach */
"0KPY4JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Muscle, Percutaneous Endoscopic Approach */
"0KPY4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Muscle, Percutaneous Endoscopic Approach */
"0KWX00Z"  =  "1"  /* Revision of Drainage Device in Upper Muscle, Open Approach */
"0KWX07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Muscle, Open Approach */
"0KWX0JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Muscle, Open Approach */
"0KWX0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Muscle, Open Approach */
"0KWX0MZ"  =  "1"  /* Revision of Stimulator Lead in Upper Muscle, Open Approach */
"0KWX30Z"  =  "1"  /* Revision of Drainage Device in Upper Muscle, Percutaneous Approach */
"0KWX37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Muscle, Percutaneous Approach */
"0KWX3JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Muscle, Percutaneous Approach */
"0KWX3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Muscle, Percutaneous Approach */
"0KWX3MZ"  =  "1"  /* Revision of Stimulator Lead in Upper Muscle, Percutaneous Approach */
"0KWX40Z"  =  "1"  /* Revision of Drainage Device in Upper Muscle, Percutaneous Endoscopic Approach */
"0KWX47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Muscle, Percutaneous Endoscopic Approach */
"0KWX4JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Muscle, Percutaneous Endoscopic Approach */
"0KWX4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Muscle, Percutaneous Endoscopic Approach */
"0KWX4MZ"  =  "1"  /* Revision of Stimulator Lead in Upper Muscle, Percutaneous Endoscopic Approach */
"0KWY00Z"  =  "1"  /* Revision of Drainage Device in Lower Muscle, Open Approach */
"0KWY07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Muscle, Open Approach */
"0KWY0JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Muscle, Open Approach */
"0KWY0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Muscle, Open Approach */
"0KWY0MZ"  =  "1"  /* Revision of Stimulator Lead in Lower Muscle, Open Approach */
"0KWY30Z"  =  "1"  /* Revision of Drainage Device in Lower Muscle, Percutaneous Approach */
"0KWY37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Muscle, Percutaneous Approach */
"0KWY3JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Muscle, Percutaneous Approach */
"0KWY3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Muscle, Percutaneous Approach */
"0KWY3MZ"  =  "1"  /* Revision of Stimulator Lead in Lower Muscle, Percutaneous Approach */
"0KWY40Z"  =  "1"  /* Revision of Drainage Device in Lower Muscle, Percutaneous Endoscopic Approach */
"0KWY47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Muscle, Percutaneous Endoscopic Approach */
"0KWY4JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Muscle, Percutaneous Endoscopic Approach */
"0KWY4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Muscle, Percutaneous Endoscopic Approach */
"0KWY4MZ"  =  "1"  /* Revision of Stimulator Lead in Lower Muscle, Percutaneous Endoscopic Approach */
"0L870ZZ"  =  "1"  /* Division of Right Hand Tendon, Open Approach */
"0L873ZZ"  =  "1"  /* Division of Right Hand Tendon, Percutaneous Approach */
"0L874ZZ"  =  "1"  /* Division of Right Hand Tendon, Percutaneous Endoscopic Approach */
"0L880ZZ"  =  "1"  /* Division of Left Hand Tendon, Open Approach */
"0L883ZZ"  =  "1"  /* Division of Left Hand Tendon, Percutaneous Approach */
"0L884ZZ"  =  "1"  /* Division of Left Hand Tendon, Percutaneous Endoscopic Approach */
"0L9000Z"  =  "1"  /* Drainage of Head and Neck Tendon with Drainage Device, Open Approach */
"0L900ZZ"  =  "1"  /* Drainage of Head and Neck Tendon, Open Approach */
"0L9030Z"  =  "1"  /* Drainage of Head and Neck Tendon with Drainage Device, Percutaneous Approach */
"0L903ZZ"  =  "1"  /* Drainage of Head and Neck Tendon, Percutaneous Approach */
"0L9040Z"  =  "1"  /* Drainage of Head and Neck Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L904ZZ"  =  "1"  /* Drainage of Head and Neck Tendon, Percutaneous Endoscopic Approach */
"0L9100Z"  =  "1"  /* Drainage of Right Shoulder Tendon with Drainage Device, Open Approach */
"0L910ZZ"  =  "1"  /* Drainage of Right Shoulder Tendon, Open Approach */
"0L9130Z"  =  "1"  /* Drainage of Right Shoulder Tendon with Drainage Device, Percutaneous Approach */
"0L913ZZ"  =  "1"  /* Drainage of Right Shoulder Tendon, Percutaneous Approach */
"0L9140Z"  =  "1"  /* Drainage of Right Shoulder Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L914ZZ"  =  "1"  /* Drainage of Right Shoulder Tendon, Percutaneous Endoscopic Approach */
"0L9200Z"  =  "1"  /* Drainage of Left Shoulder Tendon with Drainage Device, Open Approach */
"0L920ZZ"  =  "1"  /* Drainage of Left Shoulder Tendon, Open Approach */
"0L9230Z"  =  "1"  /* Drainage of Left Shoulder Tendon with Drainage Device, Percutaneous Approach */
"0L923ZZ"  =  "1"  /* Drainage of Left Shoulder Tendon, Percutaneous Approach */
"0L9240Z"  =  "1"  /* Drainage of Left Shoulder Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L924ZZ"  =  "1"  /* Drainage of Left Shoulder Tendon, Percutaneous Endoscopic Approach */
"0L9300Z"  =  "1"  /* Drainage of Right Upper Arm Tendon with Drainage Device, Open Approach */
"0L930ZZ"  =  "1"  /* Drainage of Right Upper Arm Tendon, Open Approach */
"0L9330Z"  =  "1"  /* Drainage of Right Upper Arm Tendon with Drainage Device, Percutaneous Approach */
"0L933ZZ"  =  "1"  /* Drainage of Right Upper Arm Tendon, Percutaneous Approach */
"0L9340Z"  =  "1"  /* Drainage of Right Upper Arm Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L934ZZ"  =  "1"  /* Drainage of Right Upper Arm Tendon, Percutaneous Endoscopic Approach */
"0L9400Z"  =  "1"  /* Drainage of Left Upper Arm Tendon with Drainage Device, Open Approach */
"0L940ZZ"  =  "1"  /* Drainage of Left Upper Arm Tendon, Open Approach */
"0L9430Z"  =  "1"  /* Drainage of Left Upper Arm Tendon with Drainage Device, Percutaneous Approach */
"0L943ZZ"  =  "1"  /* Drainage of Left Upper Arm Tendon, Percutaneous Approach */
"0L9440Z"  =  "1"  /* Drainage of Left Upper Arm Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L944ZZ"  =  "1"  /* Drainage of Left Upper Arm Tendon, Percutaneous Endoscopic Approach */
"0L9500Z"  =  "1"  /* Drainage of Right Lower Arm and Wrist Tendon with Drainage Device, Open Approach */
"0L950ZZ"  =  "1"  /* Drainage of Right Lower Arm and Wrist Tendon, Open Approach */
"0L9530Z"  =  "1"  /* Drainage of Right Lower Arm and Wrist Tendon with Drainage Device, Percutaneous Approach */
"0L953ZZ"  =  "1"  /* Drainage of Right Lower Arm and Wrist Tendon, Percutaneous Approach */
"0L9540Z"  =  "1"  /* Drainage of Right Lower Arm and Wrist Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L954ZZ"  =  "1"  /* Drainage of Right Lower Arm and Wrist Tendon, Percutaneous Endoscopic Approach */
"0L9600Z"  =  "1"  /* Drainage of Left Lower Arm and Wrist Tendon with Drainage Device, Open Approach */
"0L960ZZ"  =  "1"  /* Drainage of Left Lower Arm and Wrist Tendon, Open Approach */
"0L9630Z"  =  "1"  /* Drainage of Left Lower Arm and Wrist Tendon with Drainage Device, Percutaneous Approach */
"0L963ZZ"  =  "1"  /* Drainage of Left Lower Arm and Wrist Tendon, Percutaneous Approach */
"0L9640Z"  =  "1"  /* Drainage of Left Lower Arm and Wrist Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L964ZZ"  =  "1"  /* Drainage of Left Lower Arm and Wrist Tendon, Percutaneous Endoscopic Approach */
"0L9700Z"  =  "1"  /* Drainage of Right Hand Tendon with Drainage Device, Open Approach */
"0L970ZZ"  =  "1"  /* Drainage of Right Hand Tendon, Open Approach */
"0L9730Z"  =  "1"  /* Drainage of Right Hand Tendon with Drainage Device, Percutaneous Approach */
"0L9740Z"  =  "1"  /* Drainage of Right Hand Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9800Z"  =  "1"  /* Drainage of Left Hand Tendon with Drainage Device, Open Approach */
"0L980ZZ"  =  "1"  /* Drainage of Left Hand Tendon, Open Approach */
"0L9830Z"  =  "1"  /* Drainage of Left Hand Tendon with Drainage Device, Percutaneous Approach */
"0L9840Z"  =  "1"  /* Drainage of Left Hand Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9900Z"  =  "1"  /* Drainage of Right Trunk Tendon with Drainage Device, Open Approach */
"0L990ZZ"  =  "1"  /* Drainage of Right Trunk Tendon, Open Approach */
"0L9930Z"  =  "1"  /* Drainage of Right Trunk Tendon with Drainage Device, Percutaneous Approach */
"0L993ZZ"  =  "1"  /* Drainage of Right Trunk Tendon, Percutaneous Approach */
"0L9940Z"  =  "1"  /* Drainage of Right Trunk Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L994ZZ"  =  "1"  /* Drainage of Right Trunk Tendon, Percutaneous Endoscopic Approach */
"0L9B00Z"  =  "1"  /* Drainage of Left Trunk Tendon with Drainage Device, Open Approach */
"0L9B0ZZ"  =  "1"  /* Drainage of Left Trunk Tendon, Open Approach */
"0L9B30Z"  =  "1"  /* Drainage of Left Trunk Tendon with Drainage Device, Percutaneous Approach */
"0L9B3ZZ"  =  "1"  /* Drainage of Left Trunk Tendon, Percutaneous Approach */
"0L9B40Z"  =  "1"  /* Drainage of Left Trunk Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9B4ZZ"  =  "1"  /* Drainage of Left Trunk Tendon, Percutaneous Endoscopic Approach */
"0L9C00Z"  =  "1"  /* Drainage of Right Thorax Tendon with Drainage Device, Open Approach */
"0L9C0ZZ"  =  "1"  /* Drainage of Right Thorax Tendon, Open Approach */
"0L9C30Z"  =  "1"  /* Drainage of Right Thorax Tendon with Drainage Device, Percutaneous Approach */
"0L9C3ZZ"  =  "1"  /* Drainage of Right Thorax Tendon, Percutaneous Approach */
"0L9C40Z"  =  "1"  /* Drainage of Right Thorax Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9C4ZZ"  =  "1"  /* Drainage of Right Thorax Tendon, Percutaneous Endoscopic Approach */
"0L9D00Z"  =  "1"  /* Drainage of Left Thorax Tendon with Drainage Device, Open Approach */
"0L9D0ZZ"  =  "1"  /* Drainage of Left Thorax Tendon, Open Approach */
"0L9D30Z"  =  "1"  /* Drainage of Left Thorax Tendon with Drainage Device, Percutaneous Approach */
"0L9D3ZZ"  =  "1"  /* Drainage of Left Thorax Tendon, Percutaneous Approach */
"0L9D40Z"  =  "1"  /* Drainage of Left Thorax Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9D4ZZ"  =  "1"  /* Drainage of Left Thorax Tendon, Percutaneous Endoscopic Approach */
"0L9F00Z"  =  "1"  /* Drainage of Right Abdomen Tendon with Drainage Device, Open Approach */
"0L9F0ZZ"  =  "1"  /* Drainage of Right Abdomen Tendon, Open Approach */
"0L9F30Z"  =  "1"  /* Drainage of Right Abdomen Tendon with Drainage Device, Percutaneous Approach */
"0L9F3ZZ"  =  "1"  /* Drainage of Right Abdomen Tendon, Percutaneous Approach */
"0L9F40Z"  =  "1"  /* Drainage of Right Abdomen Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9F4ZZ"  =  "1"  /* Drainage of Right Abdomen Tendon, Percutaneous Endoscopic Approach */
"0L9G00Z"  =  "1"  /* Drainage of Left Abdomen Tendon with Drainage Device, Open Approach */
"0L9G0ZZ"  =  "1"  /* Drainage of Left Abdomen Tendon, Open Approach */
"0L9G30Z"  =  "1"  /* Drainage of Left Abdomen Tendon with Drainage Device, Percutaneous Approach */
"0L9G3ZZ"  =  "1"  /* Drainage of Left Abdomen Tendon, Percutaneous Approach */
"0L9G40Z"  =  "1"  /* Drainage of Left Abdomen Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9G4ZZ"  =  "1"  /* Drainage of Left Abdomen Tendon, Percutaneous Endoscopic Approach */
"0L9H00Z"  =  "1"  /* Drainage of Perineum Tendon with Drainage Device, Open Approach */
"0L9H0ZZ"  =  "1"  /* Drainage of Perineum Tendon, Open Approach */
"0L9H30Z"  =  "1"  /* Drainage of Perineum Tendon with Drainage Device, Percutaneous Approach */
"0L9H3ZZ"  =  "1"  /* Drainage of Perineum Tendon, Percutaneous Approach */
"0L9H40Z"  =  "1"  /* Drainage of Perineum Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9H4ZZ"  =  "1"  /* Drainage of Perineum Tendon, Percutaneous Endoscopic Approach */
"0L9J00Z"  =  "1"  /* Drainage of Right Hip Tendon with Drainage Device, Open Approach */
"0L9J0ZZ"  =  "1"  /* Drainage of Right Hip Tendon, Open Approach */
"0L9J30Z"  =  "1"  /* Drainage of Right Hip Tendon with Drainage Device, Percutaneous Approach */
"0L9J3ZZ"  =  "1"  /* Drainage of Right Hip Tendon, Percutaneous Approach */
"0L9J40Z"  =  "1"  /* Drainage of Right Hip Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9J4ZZ"  =  "1"  /* Drainage of Right Hip Tendon, Percutaneous Endoscopic Approach */
"0L9K00Z"  =  "1"  /* Drainage of Left Hip Tendon with Drainage Device, Open Approach */
"0L9K0ZZ"  =  "1"  /* Drainage of Left Hip Tendon, Open Approach */
"0L9K30Z"  =  "1"  /* Drainage of Left Hip Tendon with Drainage Device, Percutaneous Approach */
"0L9K3ZZ"  =  "1"  /* Drainage of Left Hip Tendon, Percutaneous Approach */
"0L9K40Z"  =  "1"  /* Drainage of Left Hip Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9K4ZZ"  =  "1"  /* Drainage of Left Hip Tendon, Percutaneous Endoscopic Approach */
"0L9L00Z"  =  "1"  /* Drainage of Right Upper Leg Tendon with Drainage Device, Open Approach */
"0L9L0ZZ"  =  "1"  /* Drainage of Right Upper Leg Tendon, Open Approach */
"0L9L30Z"  =  "1"  /* Drainage of Right Upper Leg Tendon with Drainage Device, Percutaneous Approach */
"0L9L3ZZ"  =  "1"  /* Drainage of Right Upper Leg Tendon, Percutaneous Approach */
"0L9L40Z"  =  "1"  /* Drainage of Right Upper Leg Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9L4ZZ"  =  "1"  /* Drainage of Right Upper Leg Tendon, Percutaneous Endoscopic Approach */
"0L9M00Z"  =  "1"  /* Drainage of Left Upper Leg Tendon with Drainage Device, Open Approach */
"0L9M0ZZ"  =  "1"  /* Drainage of Left Upper Leg Tendon, Open Approach */
"0L9M30Z"  =  "1"  /* Drainage of Left Upper Leg Tendon with Drainage Device, Percutaneous Approach */
"0L9M3ZZ"  =  "1"  /* Drainage of Left Upper Leg Tendon, Percutaneous Approach */
"0L9M40Z"  =  "1"  /* Drainage of Left Upper Leg Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9M4ZZ"  =  "1"  /* Drainage of Left Upper Leg Tendon, Percutaneous Endoscopic Approach */
"0L9N00Z"  =  "1"  /* Drainage of Right Lower Leg Tendon with Drainage Device, Open Approach */
"0L9N0ZZ"  =  "1"  /* Drainage of Right Lower Leg Tendon, Open Approach */
"0L9N30Z"  =  "1"  /* Drainage of Right Lower Leg Tendon with Drainage Device, Percutaneous Approach */
"0L9N3ZZ"  =  "1"  /* Drainage of Right Lower Leg Tendon, Percutaneous Approach */
"0L9N40Z"  =  "1"  /* Drainage of Right Lower Leg Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9N4ZZ"  =  "1"  /* Drainage of Right Lower Leg Tendon, Percutaneous Endoscopic Approach */
"0L9P00Z"  =  "1"  /* Drainage of Left Lower Leg Tendon with Drainage Device, Open Approach */
"0L9P0ZZ"  =  "1"  /* Drainage of Left Lower Leg Tendon, Open Approach */
"0L9P30Z"  =  "1"  /* Drainage of Left Lower Leg Tendon with Drainage Device, Percutaneous Approach */
"0L9P3ZZ"  =  "1"  /* Drainage of Left Lower Leg Tendon, Percutaneous Approach */
"0L9P40Z"  =  "1"  /* Drainage of Left Lower Leg Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9P4ZZ"  =  "1"  /* Drainage of Left Lower Leg Tendon, Percutaneous Endoscopic Approach */
"0L9Q00Z"  =  "1"  /* Drainage of Right Knee Tendon with Drainage Device, Open Approach */
"0L9Q0ZZ"  =  "1"  /* Drainage of Right Knee Tendon, Open Approach */
"0L9Q30Z"  =  "1"  /* Drainage of Right Knee Tendon with Drainage Device, Percutaneous Approach */
"0L9Q3ZZ"  =  "1"  /* Drainage of Right Knee Tendon, Percutaneous Approach */
"0L9Q40Z"  =  "1"  /* Drainage of Right Knee Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9Q4ZZ"  =  "1"  /* Drainage of Right Knee Tendon, Percutaneous Endoscopic Approach */
"0L9R00Z"  =  "1"  /* Drainage of Left Knee Tendon with Drainage Device, Open Approach */
"0L9R0ZZ"  =  "1"  /* Drainage of Left Knee Tendon, Open Approach */
"0L9R30Z"  =  "1"  /* Drainage of Left Knee Tendon with Drainage Device, Percutaneous Approach */
"0L9R3ZZ"  =  "1"  /* Drainage of Left Knee Tendon, Percutaneous Approach */
"0L9R40Z"  =  "1"  /* Drainage of Left Knee Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9R4ZZ"  =  "1"  /* Drainage of Left Knee Tendon, Percutaneous Endoscopic Approach */
"0L9S00Z"  =  "1"  /* Drainage of Right Ankle Tendon with Drainage Device, Open Approach */
"0L9S0ZZ"  =  "1"  /* Drainage of Right Ankle Tendon, Open Approach */
"0L9S30Z"  =  "1"  /* Drainage of Right Ankle Tendon with Drainage Device, Percutaneous Approach */
"0L9S3ZZ"  =  "1"  /* Drainage of Right Ankle Tendon, Percutaneous Approach */
"0L9S40Z"  =  "1"  /* Drainage of Right Ankle Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9S4ZZ"  =  "1"  /* Drainage of Right Ankle Tendon, Percutaneous Endoscopic Approach */
"0L9T00Z"  =  "1"  /* Drainage of Left Ankle Tendon with Drainage Device, Open Approach */
"0L9T0ZZ"  =  "1"  /* Drainage of Left Ankle Tendon, Open Approach */
"0L9T30Z"  =  "1"  /* Drainage of Left Ankle Tendon with Drainage Device, Percutaneous Approach */
"0L9T3ZZ"  =  "1"  /* Drainage of Left Ankle Tendon, Percutaneous Approach */
"0L9T40Z"  =  "1"  /* Drainage of Left Ankle Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9T4ZZ"  =  "1"  /* Drainage of Left Ankle Tendon, Percutaneous Endoscopic Approach */
"0L9V00Z"  =  "1"  /* Drainage of Right Foot Tendon with Drainage Device, Open Approach */
"0L9V0ZZ"  =  "1"  /* Drainage of Right Foot Tendon, Open Approach */
"0L9V30Z"  =  "1"  /* Drainage of Right Foot Tendon with Drainage Device, Percutaneous Approach */
"0L9V3ZZ"  =  "1"  /* Drainage of Right Foot Tendon, Percutaneous Approach */
"0L9V40Z"  =  "1"  /* Drainage of Right Foot Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9V4ZZ"  =  "1"  /* Drainage of Right Foot Tendon, Percutaneous Endoscopic Approach */
"0L9W00Z"  =  "1"  /* Drainage of Left Foot Tendon with Drainage Device, Open Approach */
"0L9W0ZZ"  =  "1"  /* Drainage of Left Foot Tendon, Open Approach */
"0L9W30Z"  =  "1"  /* Drainage of Left Foot Tendon with Drainage Device, Percutaneous Approach */
"0L9W3ZZ"  =  "1"  /* Drainage of Left Foot Tendon, Percutaneous Approach */
"0L9W40Z"  =  "1"  /* Drainage of Left Foot Tendon with Drainage Device, Percutaneous Endoscopic Approach */
"0L9W4ZZ"  =  "1"  /* Drainage of Left Foot Tendon, Percutaneous Endoscopic Approach */
"0LC00ZZ"  =  "1"  /* Extirpation of Matter from Head and Neck Tendon, Open Approach */
"0LC03ZZ"  =  "1"  /* Extirpation of Matter from Head and Neck Tendon, Percutaneous Approach */
"0LC04ZZ"  =  "1"  /* Extirpation of Matter from Head and Neck Tendon, Percutaneous Endoscopic Approach */
"0LC10ZZ"  =  "1"  /* Extirpation of Matter from Right Shoulder Tendon, Open Approach */
"0LC13ZZ"  =  "1"  /* Extirpation of Matter from Right Shoulder Tendon, Percutaneous Approach */
"0LC14ZZ"  =  "1"  /* Extirpation of Matter from Right Shoulder Tendon, Percutaneous Endoscopic Approach */
"0LC20ZZ"  =  "1"  /* Extirpation of Matter from Left Shoulder Tendon, Open Approach */
"0LC23ZZ"  =  "1"  /* Extirpation of Matter from Left Shoulder Tendon, Percutaneous Approach */
"0LC24ZZ"  =  "1"  /* Extirpation of Matter from Left Shoulder Tendon, Percutaneous Endoscopic Approach */
"0LC30ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Arm Tendon, Open Approach */
"0LC33ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Arm Tendon, Percutaneous Approach */
"0LC34ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Arm Tendon, Percutaneous Endoscopic Approach */
"0LC40ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Arm Tendon, Open Approach */
"0LC43ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Arm Tendon, Percutaneous Approach */
"0LC44ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Arm Tendon, Percutaneous Endoscopic Approach */
"0LC50ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Arm and Wrist Tendon, Open Approach */
"0LC53ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Arm and Wrist Tendon, Percutaneous Approach */
"0LC54ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Arm and Wrist Tendon, Percutaneous Endoscopic Approach */
"0LC60ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Arm and Wrist Tendon, Open Approach */
"0LC63ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Arm and Wrist Tendon, Percutaneous Approach */
"0LC64ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Arm and Wrist Tendon, Percutaneous Endoscopic Approach */
"0LC70ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Tendon, Open Approach */
"0LC73ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Tendon, Percutaneous Approach */
"0LC74ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Tendon, Percutaneous Endoscopic Approach */
"0LC80ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Tendon, Open Approach */
"0LC83ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Tendon, Percutaneous Approach */
"0LC84ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Tendon, Percutaneous Endoscopic Approach */
"0LC90ZZ"  =  "1"  /* Extirpation of Matter from Right Trunk Tendon, Open Approach */
"0LC93ZZ"  =  "1"  /* Extirpation of Matter from Right Trunk Tendon, Percutaneous Approach */
"0LC94ZZ"  =  "1"  /* Extirpation of Matter from Right Trunk Tendon, Percutaneous Endoscopic Approach */
"0LCB0ZZ"  =  "1"  /* Extirpation of Matter from Left Trunk Tendon, Open Approach */
"0LCB3ZZ"  =  "1"  /* Extirpation of Matter from Left Trunk Tendon, Percutaneous Approach */
"0LCB4ZZ"  =  "1"  /* Extirpation of Matter from Left Trunk Tendon, Percutaneous Endoscopic Approach */
"0LCC0ZZ"  =  "1"  /* Extirpation of Matter from Right Thorax Tendon, Open Approach */
"0LCC3ZZ"  =  "1"  /* Extirpation of Matter from Right Thorax Tendon, Percutaneous Approach */
"0LCC4ZZ"  =  "1"  /* Extirpation of Matter from Right Thorax Tendon, Percutaneous Endoscopic Approach */
"0LCD0ZZ"  =  "1"  /* Extirpation of Matter from Left Thorax Tendon, Open Approach */
"0LCD3ZZ"  =  "1"  /* Extirpation of Matter from Left Thorax Tendon, Percutaneous Approach */
"0LCD4ZZ"  =  "1"  /* Extirpation of Matter from Left Thorax Tendon, Percutaneous Endoscopic Approach */
"0LCF0ZZ"  =  "1"  /* Extirpation of Matter from Right Abdomen Tendon, Open Approach */
"0LCF3ZZ"  =  "1"  /* Extirpation of Matter from Right Abdomen Tendon, Percutaneous Approach */
"0LCF4ZZ"  =  "1"  /* Extirpation of Matter from Right Abdomen Tendon, Percutaneous Endoscopic Approach */
"0LCG0ZZ"  =  "1"  /* Extirpation of Matter from Left Abdomen Tendon, Open Approach */
"0LCG3ZZ"  =  "1"  /* Extirpation of Matter from Left Abdomen Tendon, Percutaneous Approach */
"0LCG4ZZ"  =  "1"  /* Extirpation of Matter from Left Abdomen Tendon, Percutaneous Endoscopic Approach */
"0LCH0ZZ"  =  "1"  /* Extirpation of Matter from Perineum Tendon, Open Approach */
"0LCH3ZZ"  =  "1"  /* Extirpation of Matter from Perineum Tendon, Percutaneous Approach */
"0LCH4ZZ"  =  "1"  /* Extirpation of Matter from Perineum Tendon, Percutaneous Endoscopic Approach */
"0LCJ0ZZ"  =  "1"  /* Extirpation of Matter from Right Hip Tendon, Open Approach */
"0LCJ3ZZ"  =  "1"  /* Extirpation of Matter from Right Hip Tendon, Percutaneous Approach */
"0LCJ4ZZ"  =  "1"  /* Extirpation of Matter from Right Hip Tendon, Percutaneous Endoscopic Approach */
"0LCK0ZZ"  =  "1"  /* Extirpation of Matter from Left Hip Tendon, Open Approach */
"0LCK3ZZ"  =  "1"  /* Extirpation of Matter from Left Hip Tendon, Percutaneous Approach */
"0LCK4ZZ"  =  "1"  /* Extirpation of Matter from Left Hip Tendon, Percutaneous Endoscopic Approach */
"0LCL0ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Leg Tendon, Open Approach */
"0LCL3ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Leg Tendon, Percutaneous Approach */
"0LCL4ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Leg Tendon, Percutaneous Endoscopic Approach */
"0LCM0ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Leg Tendon, Open Approach */
"0LCM3ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Leg Tendon, Percutaneous Approach */
"0LCM4ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Leg Tendon, Percutaneous Endoscopic Approach */
"0LCN0ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Leg Tendon, Open Approach */
"0LCN3ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Leg Tendon, Percutaneous Approach */
"0LCN4ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Leg Tendon, Percutaneous Endoscopic Approach */
"0LCP0ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Leg Tendon, Open Approach */
"0LCP3ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Leg Tendon, Percutaneous Approach */
"0LCP4ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Leg Tendon, Percutaneous Endoscopic Approach */
"0LCQ0ZZ"  =  "1"  /* Extirpation of Matter from Right Knee Tendon, Open Approach */
"0LCQ3ZZ"  =  "1"  /* Extirpation of Matter from Right Knee Tendon, Percutaneous Approach */
"0LCQ4ZZ"  =  "1"  /* Extirpation of Matter from Right Knee Tendon, Percutaneous Endoscopic Approach */
"0LCR0ZZ"  =  "1"  /* Extirpation of Matter from Left Knee Tendon, Open Approach */
"0LCR3ZZ"  =  "1"  /* Extirpation of Matter from Left Knee Tendon, Percutaneous Approach */
"0LCR4ZZ"  =  "1"  /* Extirpation of Matter from Left Knee Tendon, Percutaneous Endoscopic Approach */
"0LCS0ZZ"  =  "1"  /* Extirpation of Matter from Right Ankle Tendon, Open Approach */
"0LCS3ZZ"  =  "1"  /* Extirpation of Matter from Right Ankle Tendon, Percutaneous Approach */
"0LCS4ZZ"  =  "1"  /* Extirpation of Matter from Right Ankle Tendon, Percutaneous Endoscopic Approach */
"0LCT0ZZ"  =  "1"  /* Extirpation of Matter from Left Ankle Tendon, Open Approach */
"0LCT3ZZ"  =  "1"  /* Extirpation of Matter from Left Ankle Tendon, Percutaneous Approach */
"0LCT4ZZ"  =  "1"  /* Extirpation of Matter from Left Ankle Tendon, Percutaneous Endoscopic Approach */
"0LCV0ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Tendon, Open Approach */
"0LCV3ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Tendon, Percutaneous Approach */
"0LCV4ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Tendon, Percutaneous Endoscopic Approach */
"0LCW0ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Tendon, Open Approach */
"0LCW3ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Tendon, Percutaneous Approach */
"0LCW4ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Tendon, Percutaneous Endoscopic Approach */
"0LJX0ZZ"  =  "1"  /* Inspection of Upper Tendon, Open Approach */
"0LJX3ZZ"  =  "1"  /* Inspection of Upper Tendon, Percutaneous Approach */
"0LJX4ZZ"  =  "1"  /* Inspection of Upper Tendon, Percutaneous Endoscopic Approach */
"0LJY0ZZ"  =  "1"  /* Inspection of Lower Tendon, Open Approach */
"0M800ZZ"  =  "1"  /* Division of Head and Neck Bursa and Ligament, Open Approach */
"0M803ZZ"  =  "1"  /* Division of Head and Neck Bursa and Ligament, Percutaneous Approach */
"0M804ZZ"  =  "1"  /* Division of Head and Neck Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M810ZZ"  =  "1"  /* Division of Right Shoulder Bursa and Ligament, Open Approach */
"0M813ZZ"  =  "1"  /* Division of Right Shoulder Bursa and Ligament, Percutaneous Approach */
"0M814ZZ"  =  "1"  /* Division of Right Shoulder Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M820ZZ"  =  "1"  /* Division of Left Shoulder Bursa and Ligament, Open Approach */
"0M823ZZ"  =  "1"  /* Division of Left Shoulder Bursa and Ligament, Percutaneous Approach */
"0M824ZZ"  =  "1"  /* Division of Left Shoulder Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M830ZZ"  =  "1"  /* Division of Right Elbow Bursa and Ligament, Open Approach */
"0M833ZZ"  =  "1"  /* Division of Right Elbow Bursa and Ligament, Percutaneous Approach */
"0M834ZZ"  =  "1"  /* Division of Right Elbow Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M840ZZ"  =  "1"  /* Division of Left Elbow Bursa and Ligament, Open Approach */
"0M843ZZ"  =  "1"  /* Division of Left Elbow Bursa and Ligament, Percutaneous Approach */
"0M844ZZ"  =  "1"  /* Division of Left Elbow Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M850ZZ"  =  "1"  /* Division of Right Wrist Bursa and Ligament, Open Approach */
"0M853ZZ"  =  "1"  /* Division of Right Wrist Bursa and Ligament, Percutaneous Approach */
"0M854ZZ"  =  "1"  /* Division of Right Wrist Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M860ZZ"  =  "1"  /* Division of Left Wrist Bursa and Ligament, Open Approach */
"0M863ZZ"  =  "1"  /* Division of Left Wrist Bursa and Ligament, Percutaneous Approach */
"0M864ZZ"  =  "1"  /* Division of Left Wrist Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M870ZZ"  =  "1"  /* Division of Right Hand Bursa and Ligament, Open Approach */
"0M873ZZ"  =  "1"  /* Division of Right Hand Bursa and Ligament, Percutaneous Approach */
"0M874ZZ"  =  "1"  /* Division of Right Hand Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M880ZZ"  =  "1"  /* Division of Left Hand Bursa and Ligament, Open Approach */
"0M883ZZ"  =  "1"  /* Division of Left Hand Bursa and Ligament, Percutaneous Approach */
"0M884ZZ"  =  "1"  /* Division of Left Hand Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M890ZZ"  =  "1"  /* Division of Right Upper Extremity Bursa and Ligament, Open Approach */
"0M893ZZ"  =  "1"  /* Division of Right Upper Extremity Bursa and Ligament, Percutaneous Approach */
"0M894ZZ"  =  "1"  /* Division of Right Upper Extremity Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8B0ZZ"  =  "1"  /* Division of Left Upper Extremity Bursa and Ligament, Open Approach */
"0M8B3ZZ"  =  "1"  /* Division of Left Upper Extremity Bursa and Ligament, Percutaneous Approach */
"0M8B4ZZ"  =  "1"  /* Division of Left Upper Extremity Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8C0ZZ"  =  "1"  /* Division of Right Trunk Bursa and Ligament, Open Approach */
"0M8C3ZZ"  =  "1"  /* Division of Right Trunk Bursa and Ligament, Percutaneous Approach */
"0M8C4ZZ"  =  "1"  /* Division of Right Trunk Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8D0ZZ"  =  "1"  /* Division of Left Trunk Bursa and Ligament, Open Approach */
"0M8D3ZZ"  =  "1"  /* Division of Left Trunk Bursa and Ligament, Percutaneous Approach */
"0M8D4ZZ"  =  "1"  /* Division of Left Trunk Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8F0ZZ"  =  "1"  /* Division of Right Thorax Bursa and Ligament, Open Approach */
"0M8F3ZZ"  =  "1"  /* Division of Right Thorax Bursa and Ligament, Percutaneous Approach */
"0M8F4ZZ"  =  "1"  /* Division of Right Thorax Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8G0ZZ"  =  "1"  /* Division of Left Thorax Bursa and Ligament, Open Approach */
"0M8G3ZZ"  =  "1"  /* Division of Left Thorax Bursa and Ligament, Percutaneous Approach */
"0M8G4ZZ"  =  "1"  /* Division of Left Thorax Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8H0ZZ"  =  "1"  /* Division of Right Abdomen Bursa and Ligament, Open Approach */
"0M8H3ZZ"  =  "1"  /* Division of Right Abdomen Bursa and Ligament, Percutaneous Approach */
"0M8H4ZZ"  =  "1"  /* Division of Right Abdomen Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8J0ZZ"  =  "1"  /* Division of Left Abdomen Bursa and Ligament, Open Approach */
"0M8J3ZZ"  =  "1"  /* Division of Left Abdomen Bursa and Ligament, Percutaneous Approach */
"0M8J4ZZ"  =  "1"  /* Division of Left Abdomen Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8K0ZZ"  =  "1"  /* Division of Perineum Bursa and Ligament, Open Approach */
"0M8K3ZZ"  =  "1"  /* Division of Perineum Bursa and Ligament, Percutaneous Approach */
"0M8K4ZZ"  =  "1"  /* Division of Perineum Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8L0ZZ"  =  "1"  /* Division of Right Hip Bursa and Ligament, Open Approach */
"0M8L3ZZ"  =  "1"  /* Division of Right Hip Bursa and Ligament, Percutaneous Approach */
"0M8L4ZZ"  =  "1"  /* Division of Right Hip Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8M0ZZ"  =  "1"  /* Division of Left Hip Bursa and Ligament, Open Approach */
"0M8M3ZZ"  =  "1"  /* Division of Left Hip Bursa and Ligament, Percutaneous Approach */
"0M8M4ZZ"  =  "1"  /* Division of Left Hip Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8N0ZZ"  =  "1"  /* Division of Right Knee Bursa and Ligament, Open Approach */
"0M8N3ZZ"  =  "1"  /* Division of Right Knee Bursa and Ligament, Percutaneous Approach */
"0M8N4ZZ"  =  "1"  /* Division of Right Knee Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8P0ZZ"  =  "1"  /* Division of Left Knee Bursa and Ligament, Open Approach */
"0M8P3ZZ"  =  "1"  /* Division of Left Knee Bursa and Ligament, Percutaneous Approach */
"0M8P4ZZ"  =  "1"  /* Division of Left Knee Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8Q0ZZ"  =  "1"  /* Division of Right Ankle Bursa and Ligament, Open Approach */
"0M8Q3ZZ"  =  "1"  /* Division of Right Ankle Bursa and Ligament, Percutaneous Approach */
"0M8Q4ZZ"  =  "1"  /* Division of Right Ankle Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8R0ZZ"  =  "1"  /* Division of Left Ankle Bursa and Ligament, Open Approach */
"0M8R3ZZ"  =  "1"  /* Division of Left Ankle Bursa and Ligament, Percutaneous Approach */
"0M8R4ZZ"  =  "1"  /* Division of Left Ankle Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8S0ZZ"  =  "1"  /* Division of Right Foot Bursa and Ligament, Open Approach */
"0M8S3ZZ"  =  "1"  /* Division of Right Foot Bursa and Ligament, Percutaneous Approach */
"0M8S4ZZ"  =  "1"  /* Division of Right Foot Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8T0ZZ"  =  "1"  /* Division of Left Foot Bursa and Ligament, Open Approach */
"0M8T3ZZ"  =  "1"  /* Division of Left Foot Bursa and Ligament, Percutaneous Approach */
"0M8T4ZZ"  =  "1"  /* Division of Left Foot Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8V0ZZ"  =  "1"  /* Division of Right Lower Extremity Bursa and Ligament, Open Approach */
"0M8V3ZZ"  =  "1"  /* Division of Right Lower Extremity Bursa and Ligament, Percutaneous Approach */
"0M8V4ZZ"  =  "1"  /* Division of Right Lower Extremity Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M8W0ZZ"  =  "1"  /* Division of Left Lower Extremity Bursa and Ligament, Open Approach */
"0M8W3ZZ"  =  "1"  /* Division of Left Lower Extremity Bursa and Ligament, Percutaneous Approach */
"0M8W4ZZ"  =  "1"  /* Division of Left Lower Extremity Bursa and Ligament, Percutaneous Endoscopic Approach */
"0M9000Z"  =  "1"  /* Drainage of Head and Neck Bursa and Ligament with Drainage Device, Open Approach */
"0M900ZZ"  =  "1"  /* Drainage of Head and Neck Bursa and Ligament, Open Approach */
"0M9100Z"  =  "1"  /* Drainage of Right Shoulder Bursa and Ligament with Drainage Device, Open Approach */
"0M910ZZ"  =  "1"  /* Drainage of Right Shoulder Bursa and Ligament, Open Approach */
"0M9200Z"  =  "1"  /* Drainage of Left Shoulder Bursa and Ligament with Drainage Device, Open Approach */
"0M920ZZ"  =  "1"  /* Drainage of Left Shoulder Bursa and Ligament, Open Approach */
"0M9300Z"  =  "1"  /* Drainage of Right Elbow Bursa and Ligament with Drainage Device, Open Approach */
"0M930ZZ"  =  "1"  /* Drainage of Right Elbow Bursa and Ligament, Open Approach */
"0M9400Z"  =  "1"  /* Drainage of Left Elbow Bursa and Ligament with Drainage Device, Open Approach */
"0M940ZZ"  =  "1"  /* Drainage of Left Elbow Bursa and Ligament, Open Approach */
"0M9500Z"  =  "1"  /* Drainage of Right Wrist Bursa and Ligament with Drainage Device, Open Approach */
"0M950ZZ"  =  "1"  /* Drainage of Right Wrist Bursa and Ligament, Open Approach */
"0M9600Z"  =  "1"  /* Drainage of Left Wrist Bursa and Ligament with Drainage Device, Open Approach */
"0M960ZZ"  =  "1"  /* Drainage of Left Wrist Bursa and Ligament, Open Approach */
"0M9700Z"  =  "1"  /* Drainage of Right Hand Bursa and Ligament with Drainage Device, Open Approach */
"0M970ZZ"  =  "1"  /* Drainage of Right Hand Bursa and Ligament, Open Approach */
"0M9800Z"  =  "1"  /* Drainage of Left Hand Bursa and Ligament with Drainage Device, Open Approach */
"0M980ZZ"  =  "1"  /* Drainage of Left Hand Bursa and Ligament, Open Approach */
"0M9900Z"  =  "1"  /* Drainage of Right Upper Extremity Bursa and Ligament with Drainage Device, Open Approach */
"0M990ZZ"  =  "1"  /* Drainage of Right Upper Extremity Bursa and Ligament, Open Approach */
"0M9B00Z"  =  "1"  /* Drainage of Left Upper Extremity Bursa and Ligament with Drainage Device, Open Approach */
"0M9B0ZZ"  =  "1"  /* Drainage of Left Upper Extremity Bursa and Ligament, Open Approach */
"0M9C00Z"  =  "1"  /* Drainage of Right Trunk Bursa and Ligament with Drainage Device, Open Approach */
"0M9C0ZZ"  =  "1"  /* Drainage of Right Trunk Bursa and Ligament, Open Approach */
"0M9D00Z"  =  "1"  /* Drainage of Left Trunk Bursa and Ligament with Drainage Device, Open Approach */
"0M9D0ZZ"  =  "1"  /* Drainage of Left Trunk Bursa and Ligament, Open Approach */
"0M9F00Z"  =  "1"  /* Drainage of Right Thorax Bursa and Ligament with Drainage Device, Open Approach */
"0M9F0ZZ"  =  "1"  /* Drainage of Right Thorax Bursa and Ligament, Open Approach */
"0M9G00Z"  =  "1"  /* Drainage of Left Thorax Bursa and Ligament with Drainage Device, Open Approach */
"0M9G0ZZ"  =  "1"  /* Drainage of Left Thorax Bursa and Ligament, Open Approach */
"0M9H00Z"  =  "1"  /* Drainage of Right Abdomen Bursa and Ligament with Drainage Device, Open Approach */
"0M9H0ZZ"  =  "1"  /* Drainage of Right Abdomen Bursa and Ligament, Open Approach */
"0M9J00Z"  =  "1"  /* Drainage of Left Abdomen Bursa and Ligament with Drainage Device, Open Approach */
"0M9J0ZZ"  =  "1"  /* Drainage of Left Abdomen Bursa and Ligament, Open Approach */
"0M9K00Z"  =  "1"  /* Drainage of Perineum Bursa and Ligament with Drainage Device, Open Approach */
"0M9K0ZZ"  =  "1"  /* Drainage of Perineum Bursa and Ligament, Open Approach */
"0M9L00Z"  =  "1"  /* Drainage of Right Hip Bursa and Ligament with Drainage Device, Open Approach */
"0M9L0ZZ"  =  "1"  /* Drainage of Right Hip Bursa and Ligament, Open Approach */
"0M9M00Z"  =  "1"  /* Drainage of Left Hip Bursa and Ligament with Drainage Device, Open Approach */
"0M9M0ZZ"  =  "1"  /* Drainage of Left Hip Bursa and Ligament, Open Approach */
"0M9N00Z"  =  "1"  /* Drainage of Right Knee Bursa and Ligament with Drainage Device, Open Approach */
"0M9N0ZZ"  =  "1"  /* Drainage of Right Knee Bursa and Ligament, Open Approach */
"0M9P00Z"  =  "1"  /* Drainage of Left Knee Bursa and Ligament with Drainage Device, Open Approach */
"0M9P0ZZ"  =  "1"  /* Drainage of Left Knee Bursa and Ligament, Open Approach */
"0M9Q00Z"  =  "1"  /* Drainage of Right Ankle Bursa and Ligament with Drainage Device, Open Approach */
"0M9Q0ZZ"  =  "1"  /* Drainage of Right Ankle Bursa and Ligament, Open Approach */
"0M9R00Z"  =  "1"  /* Drainage of Left Ankle Bursa and Ligament with Drainage Device, Open Approach */
"0M9R0ZZ"  =  "1"  /* Drainage of Left Ankle Bursa and Ligament, Open Approach */
"0M9S00Z"  =  "1"  /* Drainage of Right Foot Bursa and Ligament with Drainage Device, Open Approach */
"0M9S0ZZ"  =  "1"  /* Drainage of Right Foot Bursa and Ligament, Open Approach */
"0M9T00Z"  =  "1"  /* Drainage of Left Foot Bursa and Ligament with Drainage Device, Open Approach */
"0M9T0ZZ"  =  "1"  /* Drainage of Left Foot Bursa and Ligament, Open Approach */
"0M9V00Z"  =  "1"  /* Drainage of Right Lower Extremity Bursa and Ligament with Drainage Device, Open Approach */
"0M9V0ZZ"  =  "1"  /* Drainage of Right Lower Extremity Bursa and Ligament, Open Approach */
"0M9W00Z"  =  "1"  /* Drainage of Left Lower Extremity Bursa and Ligament with Drainage Device, Open Approach */
"0M9W0ZZ"  =  "1"  /* Drainage of Left Lower Extremity Bursa and Ligament, Open Approach */
"0MC00ZZ"  =  "1"  /* Extirpation of Matter from Head and Neck Bursa and Ligament, Open Approach */
"0MC03ZZ"  =  "1"  /* Extirpation of Matter from Head and Neck Bursa and Ligament, Percutaneous Approach */
"0MC04ZZ"  =  "1"  /* Extirpation of Matter from Head and Neck Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MC10ZZ"  =  "1"  /* Extirpation of Matter from Right Shoulder Bursa and Ligament, Open Approach */
"0MC13ZZ"  =  "1"  /* Extirpation of Matter from Right Shoulder Bursa and Ligament, Percutaneous Approach */
"0MC14ZZ"  =  "1"  /* Extirpation of Matter from Right Shoulder Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MC20ZZ"  =  "1"  /* Extirpation of Matter from Left Shoulder Bursa and Ligament, Open Approach */
"0MC23ZZ"  =  "1"  /* Extirpation of Matter from Left Shoulder Bursa and Ligament, Percutaneous Approach */
"0MC24ZZ"  =  "1"  /* Extirpation of Matter from Left Shoulder Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MC30ZZ"  =  "1"  /* Extirpation of Matter from Right Elbow Bursa and Ligament, Open Approach */
"0MC33ZZ"  =  "1"  /* Extirpation of Matter from Right Elbow Bursa and Ligament, Percutaneous Approach */
"0MC34ZZ"  =  "1"  /* Extirpation of Matter from Right Elbow Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MC40ZZ"  =  "1"  /* Extirpation of Matter from Left Elbow Bursa and Ligament, Open Approach */
"0MC43ZZ"  =  "1"  /* Extirpation of Matter from Left Elbow Bursa and Ligament, Percutaneous Approach */
"0MC44ZZ"  =  "1"  /* Extirpation of Matter from Left Elbow Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MC50ZZ"  =  "1"  /* Extirpation of Matter from Right Wrist Bursa and Ligament, Open Approach */
"0MC53ZZ"  =  "1"  /* Extirpation of Matter from Right Wrist Bursa and Ligament, Percutaneous Approach */
"0MC54ZZ"  =  "1"  /* Extirpation of Matter from Right Wrist Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MC60ZZ"  =  "1"  /* Extirpation of Matter from Left Wrist Bursa and Ligament, Open Approach */
"0MC63ZZ"  =  "1"  /* Extirpation of Matter from Left Wrist Bursa and Ligament, Percutaneous Approach */
"0MC64ZZ"  =  "1"  /* Extirpation of Matter from Left Wrist Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MC70ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Bursa and Ligament, Open Approach */
"0MC73ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Bursa and Ligament, Percutaneous Approach */
"0MC74ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MC80ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Bursa and Ligament, Open Approach */
"0MC83ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Bursa and Ligament, Percutaneous Approach */
"0MC84ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MC90ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Extremity Bursa and Ligament, Open Approach */
"0MC93ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Extremity Bursa and Ligament, Percutaneous Approach */
"0MC94ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Extremity Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCB0ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Extremity Bursa and Ligament, Open Approach */
"0MCB3ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Extremity Bursa and Ligament, Percutaneous Approach */
"0MCB4ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Extremity Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCC0ZZ"  =  "1"  /* Extirpation of Matter from Right Trunk Bursa and Ligament, Open Approach */
"0MCC3ZZ"  =  "1"  /* Extirpation of Matter from Right Trunk Bursa and Ligament, Percutaneous Approach */
"0MCC4ZZ"  =  "1"  /* Extirpation of Matter from Right Trunk Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCD0ZZ"  =  "1"  /* Extirpation of Matter from Left Trunk Bursa and Ligament, Open Approach */
"0MCD3ZZ"  =  "1"  /* Extirpation of Matter from Left Trunk Bursa and Ligament, Percutaneous Approach */
"0MCD4ZZ"  =  "1"  /* Extirpation of Matter from Left Trunk Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCF0ZZ"  =  "1"  /* Extirpation of Matter from Right Thorax Bursa and Ligament, Open Approach */
"0MCF3ZZ"  =  "1"  /* Extirpation of Matter from Right Thorax Bursa and Ligament, Percutaneous Approach */
"0MCF4ZZ"  =  "1"  /* Extirpation of Matter from Right Thorax Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCG0ZZ"  =  "1"  /* Extirpation of Matter from Left Thorax Bursa and Ligament, Open Approach */
"0MCG3ZZ"  =  "1"  /* Extirpation of Matter from Left Thorax Bursa and Ligament, Percutaneous Approach */
"0MCG4ZZ"  =  "1"  /* Extirpation of Matter from Left Thorax Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCH0ZZ"  =  "1"  /* Extirpation of Matter from Right Abdomen Bursa and Ligament, Open Approach */
"0MCH3ZZ"  =  "1"  /* Extirpation of Matter from Right Abdomen Bursa and Ligament, Percutaneous Approach */
"0MCH4ZZ"  =  "1"  /* Extirpation of Matter from Right Abdomen Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCJ0ZZ"  =  "1"  /* Extirpation of Matter from Left Abdomen Bursa and Ligament, Open Approach */
"0MCJ3ZZ"  =  "1"  /* Extirpation of Matter from Left Abdomen Bursa and Ligament, Percutaneous Approach */
"0MCJ4ZZ"  =  "1"  /* Extirpation of Matter from Left Abdomen Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCK0ZZ"  =  "1"  /* Extirpation of Matter from Perineum Bursa and Ligament, Open Approach */
"0MCK3ZZ"  =  "1"  /* Extirpation of Matter from Perineum Bursa and Ligament, Percutaneous Approach */
"0MCK4ZZ"  =  "1"  /* Extirpation of Matter from Perineum Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCL0ZZ"  =  "1"  /* Extirpation of Matter from Right Hip Bursa and Ligament, Open Approach */
"0MCL3ZZ"  =  "1"  /* Extirpation of Matter from Right Hip Bursa and Ligament, Percutaneous Approach */
"0MCL4ZZ"  =  "1"  /* Extirpation of Matter from Right Hip Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCM0ZZ"  =  "1"  /* Extirpation of Matter from Left Hip Bursa and Ligament, Open Approach */
"0MCM3ZZ"  =  "1"  /* Extirpation of Matter from Left Hip Bursa and Ligament, Percutaneous Approach */
"0MCM4ZZ"  =  "1"  /* Extirpation of Matter from Left Hip Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCN0ZZ"  =  "1"  /* Extirpation of Matter from Right Knee Bursa and Ligament, Open Approach */
"0MCN3ZZ"  =  "1"  /* Extirpation of Matter from Right Knee Bursa and Ligament, Percutaneous Approach */
"0MCN4ZZ"  =  "1"  /* Extirpation of Matter from Right Knee Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCP0ZZ"  =  "1"  /* Extirpation of Matter from Left Knee Bursa and Ligament, Open Approach */
"0MCP3ZZ"  =  "1"  /* Extirpation of Matter from Left Knee Bursa and Ligament, Percutaneous Approach */
"0MCP4ZZ"  =  "1"  /* Extirpation of Matter from Left Knee Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCQ0ZZ"  =  "1"  /* Extirpation of Matter from Right Ankle Bursa and Ligament, Open Approach */
"0MCQ3ZZ"  =  "1"  /* Extirpation of Matter from Right Ankle Bursa and Ligament, Percutaneous Approach */
"0MCQ4ZZ"  =  "1"  /* Extirpation of Matter from Right Ankle Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCR0ZZ"  =  "1"  /* Extirpation of Matter from Left Ankle Bursa and Ligament, Open Approach */
"0MCR3ZZ"  =  "1"  /* Extirpation of Matter from Left Ankle Bursa and Ligament, Percutaneous Approach */
"0MCR4ZZ"  =  "1"  /* Extirpation of Matter from Left Ankle Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCS0ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Bursa and Ligament, Open Approach */
"0MCS3ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Bursa and Ligament, Percutaneous Approach */
"0MCS4ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCT0ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Bursa and Ligament, Open Approach */
"0MCT3ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Bursa and Ligament, Percutaneous Approach */
"0MCT4ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCV0ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Extremity Bursa and Ligament, Open Approach */
"0MCV3ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Extremity Bursa and Ligament, Percutaneous Approach */
"0MCV4ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Extremity Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MCW0ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Extremity Bursa and Ligament, Open Approach */
"0MCW3ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Extremity Bursa and Ligament, Percutaneous Approach */
"0MCW4ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Extremity Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MJX0ZZ"  =  "1"  /* Inspection of Upper Bursa and Ligament, Open Approach */
"0MJY0ZZ"  =  "1"  /* Inspection of Lower Bursa and Ligament, Open Approach */
"0MPX07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Bursa and Ligament, Open Approach */
"0MPX0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Bursa and Ligament, Open Approach */
"0MPX37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Bursa and Ligament, Percutaneous Approach */
"0MPX3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Bursa and Ligament, Percutaneous Approach */
"0MPX47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MPX4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MPY07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Bursa and Ligament, Open Approach */
"0MPY0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Bursa and Ligament, Open Approach */
"0MPY37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Bursa and Ligament, Percutaneous Approach */
"0MPY3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Bursa and Ligament, Percutaneous Approach */
"0MPY47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MPY4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MWX00Z"  =  "1"  /* Revision of Drainage Device in Upper Bursa and Ligament, Open Approach */
"0MWX07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Bursa and Ligament, Open Approach */
"0MWX0JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Bursa and Ligament, Open Approach */
"0MWX0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Bursa and Ligament, Open Approach */
"0MWX30Z"  =  "1"  /* Revision of Drainage Device in Upper Bursa and Ligament, Percutaneous Approach */
"0MWX37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Bursa and Ligament, Percutaneous Approach */
"0MWX3JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Bursa and Ligament, Percutaneous Approach */
"0MWX3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Bursa and Ligament, Percutaneous Approach */
"0MWX40Z"  =  "1"  /* Revision of Drainage Device in Upper Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MWX47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MWX4JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MWX4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MWY00Z"  =  "1"  /* Revision of Drainage Device in Lower Bursa and Ligament, Open Approach */
"0MWY07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Bursa and Ligament, Open Approach */
"0MWY0JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Bursa and Ligament, Open Approach */
"0MWY0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Bursa and Ligament, Open Approach */
"0MWY30Z"  =  "1"  /* Revision of Drainage Device in Lower Bursa and Ligament, Percutaneous Approach */
"0MWY37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Bursa and Ligament, Percutaneous Approach */
"0MWY3JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Bursa and Ligament, Percutaneous Approach */
"0MWY3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Bursa and Ligament, Percutaneous Approach */
"0MWY40Z"  =  "1"  /* Revision of Drainage Device in Lower Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MWY47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MWY4JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Bursa and Ligament, Percutaneous Endoscopic Approach */
"0MWY4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Bursa and Ligament, Percutaneous Endoscopic Approach */
"0N800ZZ"  =  "1"  /* Division of Skull, Open Approach */
"0N803ZZ"  =  "1"  /* Division of Skull, Percutaneous Approach */
"0N804ZZ"  =  "1"  /* Division of Skull, Percutaneous Endoscopic Approach */
"0N810ZZ"  =  "1"  /* Division of Right Frontal Bone, Open Approach */
"0N813ZZ"  =  "1"  /* Division of Right Frontal Bone, Percutaneous Approach */
"0N814ZZ"  =  "1"  /* Division of Right Frontal Bone, Percutaneous Endoscopic Approach */
"0N820ZZ"  =  "1"  /* Division of Left Frontal Bone, Open Approach */
"0N823ZZ"  =  "1"  /* Division of Left Frontal Bone, Percutaneous Approach */
"0N824ZZ"  =  "1"  /* Division of Left Frontal Bone, Percutaneous Endoscopic Approach */
"0N830ZZ"  =  "1"  /* Division of Right Parietal Bone, Open Approach */
"0N833ZZ"  =  "1"  /* Division of Right Parietal Bone, Percutaneous Approach */
"0N834ZZ"  =  "1"  /* Division of Right Parietal Bone, Percutaneous Endoscopic Approach */
"0N840ZZ"  =  "1"  /* Division of Left Parietal Bone, Open Approach */
"0N843ZZ"  =  "1"  /* Division of Left Parietal Bone, Percutaneous Approach */
"0N844ZZ"  =  "1"  /* Division of Left Parietal Bone, Percutaneous Endoscopic Approach */
"0N850ZZ"  =  "1"  /* Division of Right Temporal Bone, Open Approach */
"0N853ZZ"  =  "1"  /* Division of Right Temporal Bone, Percutaneous Approach */
"0N854ZZ"  =  "1"  /* Division of Right Temporal Bone, Percutaneous Endoscopic Approach */
"0N860ZZ"  =  "1"  /* Division of Left Temporal Bone, Open Approach */
"0N863ZZ"  =  "1"  /* Division of Left Temporal Bone, Percutaneous Approach */
"0N864ZZ"  =  "1"  /* Division of Left Temporal Bone, Percutaneous Endoscopic Approach */
"0N870ZZ"  =  "1"  /* Division of Right Occipital Bone, Open Approach */
"0N873ZZ"  =  "1"  /* Division of Right Occipital Bone, Percutaneous Approach */
"0N874ZZ"  =  "1"  /* Division of Right Occipital Bone, Percutaneous Endoscopic Approach */
"0N880ZZ"  =  "1"  /* Division of Left Occipital Bone, Open Approach */
"0N883ZZ"  =  "1"  /* Division of Left Occipital Bone, Percutaneous Approach */
"0N884ZZ"  =  "1"  /* Division of Left Occipital Bone, Percutaneous Endoscopic Approach */
"0N8B0ZZ"  =  "1"  /* Division of Nasal Bone, Open Approach */
"0N8B3ZZ"  =  "1"  /* Division of Nasal Bone, Percutaneous Approach */
"0N8B4ZZ"  =  "1"  /* Division of Nasal Bone, Percutaneous Endoscopic Approach */
"0N8P0ZZ"  =  "1"  /* Division of Right Orbit, Open Approach */
"0N8P3ZZ"  =  "1"  /* Division of Right Orbit, Percutaneous Approach */
"0N8P4ZZ"  =  "1"  /* Division of Right Orbit, Percutaneous Endoscopic Approach */
"0N8Q0ZZ"  =  "1"  /* Division of Left Orbit, Open Approach */
"0N8Q3ZZ"  =  "1"  /* Division of Left Orbit, Percutaneous Approach */
"0N8Q4ZZ"  =  "1"  /* Division of Left Orbit, Percutaneous Endoscopic Approach */
"0N9000Z"  =  "1"  /* Drainage of Skull with Drainage Device, Open Approach */
"0N900ZZ"  =  "1"  /* Drainage of Skull, Open Approach */
"0N9030Z"  =  "1"  /* Drainage of Skull with Drainage Device, Percutaneous Approach */
"0N903ZZ"  =  "1"  /* Drainage of Skull, Percutaneous Approach */
"0N9040Z"  =  "1"  /* Drainage of Skull with Drainage Device, Percutaneous Endoscopic Approach */
"0N904ZZ"  =  "1"  /* Drainage of Skull, Percutaneous Endoscopic Approach */
"0N9100Z"  =  "1"  /* Drainage of Right Frontal Bone with Drainage Device, Open Approach */
"0N910ZZ"  =  "1"  /* Drainage of Right Frontal Bone, Open Approach */
"0N9130Z"  =  "1"  /* Drainage of Right Frontal Bone with Drainage Device, Percutaneous Approach */
"0N913ZZ"  =  "1"  /* Drainage of Right Frontal Bone, Percutaneous Approach */
"0N9140Z"  =  "1"  /* Drainage of Right Frontal Bone with Drainage Device, Percutaneous Endoscopic Approach */
"0N914ZZ"  =  "1"  /* Drainage of Right Frontal Bone, Percutaneous Endoscopic Approach */
"0N9200Z"  =  "1"  /* Drainage of Left Frontal Bone with Drainage Device, Open Approach */
"0N920ZZ"  =  "1"  /* Drainage of Left Frontal Bone, Open Approach */
"0N9230Z"  =  "1"  /* Drainage of Left Frontal Bone with Drainage Device, Percutaneous Approach */
"0N923ZZ"  =  "1"  /* Drainage of Left Frontal Bone, Percutaneous Approach */
"0N9240Z"  =  "1"  /* Drainage of Left Frontal Bone with Drainage Device, Percutaneous Endoscopic Approach */
"0N924ZZ"  =  "1"  /* Drainage of Left Frontal Bone, Percutaneous Endoscopic Approach */
"0N9300Z"  =  "1"  /* Drainage of Right Parietal Bone with Drainage Device, Open Approach */
"0N930ZZ"  =  "1"  /* Drainage of Right Parietal Bone, Open Approach */
"0N9330Z"  =  "1"  /* Drainage of Right Parietal Bone with Drainage Device, Percutaneous Approach */
"0N933ZZ"  =  "1"  /* Drainage of Right Parietal Bone, Percutaneous Approach */
"0N9340Z"  =  "1"  /* Drainage of Right Parietal Bone with Drainage Device, Percutaneous Endoscopic Approach */
"0N934ZZ"  =  "1"  /* Drainage of Right Parietal Bone, Percutaneous Endoscopic Approach */
"0N9400Z"  =  "1"  /* Drainage of Left Parietal Bone with Drainage Device, Open Approach */
"0N940ZZ"  =  "1"  /* Drainage of Left Parietal Bone, Open Approach */
"0N9430Z"  =  "1"  /* Drainage of Left Parietal Bone with Drainage Device, Percutaneous Approach */
"0N943ZZ"  =  "1"  /* Drainage of Left Parietal Bone, Percutaneous Approach */
"0N9440Z"  =  "1"  /* Drainage of Left Parietal Bone with Drainage Device, Percutaneous Endoscopic Approach */
"0N944ZZ"  =  "1"  /* Drainage of Left Parietal Bone, Percutaneous Endoscopic Approach */
"0N9500Z"  =  "1"  /* Drainage of Right Temporal Bone with Drainage Device, Open Approach */
"0N950ZZ"  =  "1"  /* Drainage of Right Temporal Bone, Open Approach */
"0N9530Z"  =  "1"  /* Drainage of Right Temporal Bone with Drainage Device, Percutaneous Approach */
"0N953ZZ"  =  "1"  /* Drainage of Right Temporal Bone, Percutaneous Approach */
"0N9540Z"  =  "1"  /* Drainage of Right Temporal Bone with Drainage Device, Percutaneous Endoscopic Approach */
"0N954ZZ"  =  "1"  /* Drainage of Right Temporal Bone, Percutaneous Endoscopic Approach */
"0N9600Z"  =  "1"  /* Drainage of Left Temporal Bone with Drainage Device, Open Approach */
"0N960ZZ"  =  "1"  /* Drainage of Left Temporal Bone, Open Approach */
"0N9630Z"  =  "1"  /* Drainage of Left Temporal Bone with Drainage Device, Percutaneous Approach */
"0N963ZZ"  =  "1"  /* Drainage of Left Temporal Bone, Percutaneous Approach */
"0N9640Z"  =  "1"  /* Drainage of Left Temporal Bone with Drainage Device, Percutaneous Endoscopic Approach */
"0N964ZZ"  =  "1"  /* Drainage of Left Temporal Bone, Percutaneous Endoscopic Approach */
"0N9700Z"  =  "1"  /* Drainage of Right Occipital Bone with Drainage Device, Open Approach */
"0N970ZZ"  =  "1"  /* Drainage of Right Occipital Bone, Open Approach */
"0N9730Z"  =  "1"  /* Drainage of Right Occipital Bone with Drainage Device, Percutaneous Approach */
"0N973ZZ"  =  "1"  /* Drainage of Right Occipital Bone, Percutaneous Approach */
"0N9740Z"  =  "1"  /* Drainage of Right Occipital Bone with Drainage Device, Percutaneous Endoscopic Approach */
"0N974ZZ"  =  "1"  /* Drainage of Right Occipital Bone, Percutaneous Endoscopic Approach */
"0N9800Z"  =  "1"  /* Drainage of Left Occipital Bone with Drainage Device, Open Approach */
"0N980ZZ"  =  "1"  /* Drainage of Left Occipital Bone, Open Approach */
"0N9830Z"  =  "1"  /* Drainage of Left Occipital Bone with Drainage Device, Percutaneous Approach */
"0N983ZZ"  =  "1"  /* Drainage of Left Occipital Bone, Percutaneous Approach */
"0N9840Z"  =  "1"  /* Drainage of Left Occipital Bone with Drainage Device, Percutaneous Endoscopic Approach */
"0N984ZZ"  =  "1"  /* Drainage of Left Occipital Bone, Percutaneous Endoscopic Approach */
"0N9B00Z"  =  "1"  /* Drainage of Nasal Bone with Drainage Device, Open Approach */
"0N9B0ZZ"  =  "1"  /* Drainage of Nasal Bone, Open Approach */
"0N9B30Z"  =  "1"  /* Drainage of Nasal Bone with Drainage Device, Percutaneous Approach */
"0N9B3ZZ"  =  "1"  /* Drainage of Nasal Bone, Percutaneous Approach */
"0N9B40Z"  =  "1"  /* Drainage of Nasal Bone with Drainage Device, Percutaneous Endoscopic Approach */
"0N9B4ZZ"  =  "1"  /* Drainage of Nasal Bone, Percutaneous Endoscopic Approach */
"0N9P00Z"  =  "1"  /* Drainage of Right Orbit with Drainage Device, Open Approach */
"0N9P0ZZ"  =  "1"  /* Drainage of Right Orbit, Open Approach */
"0N9P30Z"  =  "1"  /* Drainage of Right Orbit with Drainage Device, Percutaneous Approach */
"0N9P40Z"  =  "1"  /* Drainage of Right Orbit with Drainage Device, Percutaneous Endoscopic Approach */
"0N9P4ZZ"  =  "1"  /* Drainage of Right Orbit, Percutaneous Endoscopic Approach */
"0N9Q00Z"  =  "1"  /* Drainage of Left Orbit with Drainage Device, Open Approach */
"0N9Q0ZZ"  =  "1"  /* Drainage of Left Orbit, Open Approach */
"0N9Q30Z"  =  "1"  /* Drainage of Left Orbit with Drainage Device, Percutaneous Approach */
"0N9Q40Z"  =  "1"  /* Drainage of Left Orbit with Drainage Device, Percutaneous Endoscopic Approach */
"0N9Q4ZZ"  =  "1"  /* Drainage of Left Orbit, Percutaneous Endoscopic Approach */
"0NC10ZZ"  =  "1"  /* Extirpation of Matter from Right Frontal Bone, Open Approach */
"0NC13ZZ"  =  "1"  /* Extirpation of Matter from Right Frontal Bone, Percutaneous Approach */
"0NC14ZZ"  =  "1"  /* Extirpation of Matter from Right Frontal Bone, Percutaneous Endoscopic Approach */
"0NC20ZZ"  =  "1"  /* Extirpation of Matter from Left Frontal Bone, Open Approach */
"0NC23ZZ"  =  "1"  /* Extirpation of Matter from Left Frontal Bone, Percutaneous Approach */
"0NC24ZZ"  =  "1"  /* Extirpation of Matter from Left Frontal Bone, Percutaneous Endoscopic Approach */
"0NC30ZZ"  =  "1"  /* Extirpation of Matter from Right Parietal Bone, Open Approach */
"0NC33ZZ"  =  "1"  /* Extirpation of Matter from Right Parietal Bone, Percutaneous Approach */
"0NC34ZZ"  =  "1"  /* Extirpation of Matter from Right Parietal Bone, Percutaneous Endoscopic Approach */
"0NC40ZZ"  =  "1"  /* Extirpation of Matter from Left Parietal Bone, Open Approach */
"0NC43ZZ"  =  "1"  /* Extirpation of Matter from Left Parietal Bone, Percutaneous Approach */
"0NC44ZZ"  =  "1"  /* Extirpation of Matter from Left Parietal Bone, Percutaneous Endoscopic Approach */
"0NC50ZZ"  =  "1"  /* Extirpation of Matter from Right Temporal Bone, Open Approach */
"0NC53ZZ"  =  "1"  /* Extirpation of Matter from Right Temporal Bone, Percutaneous Approach */
"0NC54ZZ"  =  "1"  /* Extirpation of Matter from Right Temporal Bone, Percutaneous Endoscopic Approach */
"0NC60ZZ"  =  "1"  /* Extirpation of Matter from Left Temporal Bone, Open Approach */
"0NC63ZZ"  =  "1"  /* Extirpation of Matter from Left Temporal Bone, Percutaneous Approach */
"0NC64ZZ"  =  "1"  /* Extirpation of Matter from Left Temporal Bone, Percutaneous Endoscopic Approach */
"0NC70ZZ"  =  "1"  /* Extirpation of Matter from Right Occipital Bone, Open Approach */
"0NC73ZZ"  =  "1"  /* Extirpation of Matter from Right Occipital Bone, Percutaneous Approach */
"0NC74ZZ"  =  "1"  /* Extirpation of Matter from Right Occipital Bone, Percutaneous Endoscopic Approach */
"0NC80ZZ"  =  "1"  /* Extirpation of Matter from Left Occipital Bone, Open Approach */
"0NC83ZZ"  =  "1"  /* Extirpation of Matter from Left Occipital Bone, Percutaneous Approach */
"0NC84ZZ"  =  "1"  /* Extirpation of Matter from Left Occipital Bone, Percutaneous Endoscopic Approach */
"0NCB0ZZ"  =  "1"  /* Extirpation of Matter from Nasal Bone, Open Approach */
"0NCB3ZZ"  =  "1"  /* Extirpation of Matter from Nasal Bone, Percutaneous Approach */
"0NCB4ZZ"  =  "1"  /* Extirpation of Matter from Nasal Bone, Percutaneous Endoscopic Approach */
"0NH00MZ"  =  "1"  /* Insertion of Bone Growth Stimulator into Skull, Open Approach */
"0NH03MZ"  =  "1"  /* Insertion of Bone Growth Stimulator into Skull, Percutaneous Approach */
"0NH04MZ"  =  "1"  /* Insertion of Bone Growth Stimulator into Skull, Percutaneous Endoscopic Approach */
"0NHB04Z"  =  "1"  /* Insertion of Internal Fixation Device into Nasal Bone, Open Approach */
"0NHB0MZ"  =  "1"  /* Insertion of Bone Growth Stimulator into Nasal Bone, Open Approach */
"0NHB34Z"  =  "1"  /* Insertion of Internal Fixation Device into Nasal Bone, Percutaneous Approach */
"0NHB3MZ"  =  "1"  /* Insertion of Bone Growth Stimulator into Nasal Bone, Percutaneous Approach */
"0NHB44Z"  =  "1"  /* Insertion of Internal Fixation Device into Nasal Bone, Percutaneous Endoscopic Approach */
"0NHB4MZ"  =  "1"  /* Insertion of Bone Growth Stimulator into Nasal Bone, Percutaneous Endoscopic Approach */
"0NP000Z"  =  "1"  /* Removal of Drainage Device from Skull, Open Approach */
"0NP004Z"  =  "1"  /* Removal of Internal Fixation Device from Skull, Open Approach */
"0NP005Z"  =  "1"  /* Removal of External Fixation Device from Skull, Open Approach */
"0NP007Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Skull, Open Approach */
"0NP00KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Skull, Open Approach */
"0NP00SZ"  =  "1"  /* Removal of Hearing Device from Skull, Open Approach */
"0NP030Z"  =  "1"  /* Removal of Drainage Device from Skull, Percutaneous Approach */
"0NP034Z"  =  "1"  /* Removal of Internal Fixation Device from Skull, Percutaneous Approach */
"0NP037Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Skull, Percutaneous Approach */
"0NP03KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Skull, Percutaneous Approach */
"0NP03SZ"  =  "1"  /* Removal of Hearing Device from Skull, Percutaneous Approach */
"0NP040Z"  =  "1"  /* Removal of Drainage Device from Skull, Percutaneous Endoscopic Approach */
"0NP044Z"  =  "1"  /* Removal of Internal Fixation Device from Skull, Percutaneous Endoscopic Approach */
"0NP047Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Skull, Percutaneous Endoscopic Approach */
"0NP04KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Skull, Percutaneous Endoscopic Approach */
"0NP04SZ"  =  "1"  /* Removal of Hearing Device from Skull, Percutaneous Endoscopic Approach */
"0NP0X4Z"  =  "1"  /* Removal of Internal Fixation Device from Skull, External Approach */
"0NP0XSZ"  =  "1"  /* Removal of Hearing Device from Skull, External Approach */
"0NPB00Z"  =  "1"  /* Removal of Drainage Device from Nasal Bone, Open Approach */
"0NPB04Z"  =  "1"  /* Removal of Internal Fixation Device from Nasal Bone, Open Approach */
"0NPB07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Nasal Bone, Open Approach */
"0NPB0JZ"  =  "1"  /* Removal of Synthetic Substitute from Nasal Bone, Open Approach */
"0NPB0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Nasal Bone, Open Approach */
"0NPB0MZ"  =  "1"  /* Removal of Bone Growth Stimulator from Nasal Bone, Open Approach */
"0NPB30Z"  =  "1"  /* Removal of Drainage Device from Nasal Bone, Percutaneous Approach */
"0NPB34Z"  =  "1"  /* Removal of Internal Fixation Device from Nasal Bone, Percutaneous Approach */
"0NPB37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Nasal Bone, Percutaneous Approach */
"0NPB3JZ"  =  "1"  /* Removal of Synthetic Substitute from Nasal Bone, Percutaneous Approach */
"0NPB3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Nasal Bone, Percutaneous Approach */
"0NPB3MZ"  =  "1"  /* Removal of Bone Growth Stimulator from Nasal Bone, Percutaneous Approach */
"0NPB40Z"  =  "1"  /* Removal of Drainage Device from Nasal Bone, Percutaneous Endoscopic Approach */
"0NPB44Z"  =  "1"  /* Removal of Internal Fixation Device from Nasal Bone, Percutaneous Endoscopic Approach */
"0NPB47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Nasal Bone, Percutaneous Endoscopic Approach */
"0NPB4JZ"  =  "1"  /* Removal of Synthetic Substitute from Nasal Bone, Percutaneous Endoscopic Approach */
"0NPB4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Nasal Bone, Percutaneous Endoscopic Approach */
"0NPB4MZ"  =  "1"  /* Removal of Bone Growth Stimulator from Nasal Bone, Percutaneous Endoscopic Approach */
"0NW000Z"  =  "1"  /* Revision of Drainage Device in Skull, Open Approach */
"0NW004Z"  =  "1"  /* Revision of Internal Fixation Device in Skull, Open Approach */
"0NW005Z"  =  "1"  /* Revision of External Fixation Device in Skull, Open Approach */
"0NW007Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Skull, Open Approach */
"0NW00JZ"  =  "1"  /* Revision of Synthetic Substitute in Skull, Open Approach */
"0NW00KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Skull, Open Approach */
"0NW00MZ"  =  "1"  /* Revision of Bone Growth Stimulator in Skull, Open Approach */
"0NW00NZ"  =  "1"  /* Revision of Neurostimulator Generator in Skull, Open Approach */
"0NW00SZ"  =  "1"  /* Revision of Hearing Device in Skull, Open Approach */
"0NW030Z"  =  "1"  /* Revision of Drainage Device in Skull, Percutaneous Approach */
"0NW034Z"  =  "1"  /* Revision of Internal Fixation Device in Skull, Percutaneous Approach */
"0NW035Z"  =  "1"  /* Revision of External Fixation Device in Skull, Percutaneous Approach */
"0NW037Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Skull, Percutaneous Approach */
"0NW03JZ"  =  "1"  /* Revision of Synthetic Substitute in Skull, Percutaneous Approach */
"0NW03KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Skull, Percutaneous Approach */
"0NW03MZ"  =  "1"  /* Revision of Bone Growth Stimulator in Skull, Percutaneous Approach */
"0NW03SZ"  =  "1"  /* Revision of Hearing Device in Skull, Percutaneous Approach */
"0NW040Z"  =  "1"  /* Revision of Drainage Device in Skull, Percutaneous Endoscopic Approach */
"0NW044Z"  =  "1"  /* Revision of Internal Fixation Device in Skull, Percutaneous Endoscopic Approach */
"0NW045Z"  =  "1"  /* Revision of External Fixation Device in Skull, Percutaneous Endoscopic Approach */
"0NW047Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Skull, Percutaneous Endoscopic Approach */
"0NW04JZ"  =  "1"  /* Revision of Synthetic Substitute in Skull, Percutaneous Endoscopic Approach */
"0NW04KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Skull, Percutaneous Endoscopic Approach */
"0NW04MZ"  =  "1"  /* Revision of Bone Growth Stimulator in Skull, Percutaneous Endoscopic Approach */
"0NW04SZ"  =  "1"  /* Revision of Hearing Device in Skull, Percutaneous Endoscopic Approach */
"0NWB00Z"  =  "1"  /* Revision of Drainage Device in Nasal Bone, Open Approach */
"0NWB04Z"  =  "1"  /* Revision of Internal Fixation Device in Nasal Bone, Open Approach */
"0NWB07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Nasal Bone, Open Approach */
"0NWB0JZ"  =  "1"  /* Revision of Synthetic Substitute in Nasal Bone, Open Approach */
"0NWB0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Nasal Bone, Open Approach */
"0NWB0MZ"  =  "1"  /* Revision of Bone Growth Stimulator in Nasal Bone, Open Approach */
"0NWB30Z"  =  "1"  /* Revision of Drainage Device in Nasal Bone, Percutaneous Approach */
"0NWB34Z"  =  "1"  /* Revision of Internal Fixation Device in Nasal Bone, Percutaneous Approach */
"0NWB37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Nasal Bone, Percutaneous Approach */
"0NWB3JZ"  =  "1"  /* Revision of Synthetic Substitute in Nasal Bone, Percutaneous Approach */
"0NWB3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Nasal Bone, Percutaneous Approach */
"0NWB3MZ"  =  "1"  /* Revision of Bone Growth Stimulator in Nasal Bone, Percutaneous Approach */
"0NWB40Z"  =  "1"  /* Revision of Drainage Device in Nasal Bone, Percutaneous Endoscopic Approach */
"0NWB44Z"  =  "1"  /* Revision of Internal Fixation Device in Nasal Bone, Percutaneous Endoscopic Approach */
"0NWB47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Nasal Bone, Percutaneous Endoscopic Approach */
"0NWB4JZ"  =  "1"  /* Revision of Synthetic Substitute in Nasal Bone, Percutaneous Endoscopic Approach */
"0NWB4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Nasal Bone, Percutaneous Endoscopic Approach */
"0NWB4MZ"  =  "1"  /* Revision of Bone Growth Stimulator in Nasal Bone, Percutaneous Endoscopic Approach */
"0P9000Z"  =  "1"  /* Drainage of Sternum with Drainage Device, Open Approach */
"0P900ZZ"  =  "1"  /* Drainage of Sternum, Open Approach */
"0P9030Z"  =  "1"  /* Drainage of Sternum with Drainage Device, Percutaneous Approach */
"0P903ZZ"  =  "1"  /* Drainage of Sternum, Percutaneous Approach */
"0P9040Z"  =  "1"  /* Drainage of Sternum with Drainage Device, Percutaneous Endoscopic Approach */
"0P904ZZ"  =  "1"  /* Drainage of Sternum, Percutaneous Endoscopic Approach */
"0P9100Z"  =  "1"  /* Drainage of Right Rib with Drainage Device, Open Approach */
"0P910ZZ"  =  "1"  /* Drainage of Right Rib, Open Approach */
"0P9130Z"  =  "1"  /* Drainage of Right Rib with Drainage Device, Percutaneous Approach */
"0P913ZZ"  =  "1"  /* Drainage of Right Rib, Percutaneous Approach */
"0P9140Z"  =  "1"  /* Drainage of Right Rib with Drainage Device, Percutaneous Endoscopic Approach */
"0P914ZZ"  =  "1"  /* Drainage of Right Rib, Percutaneous Endoscopic Approach */
"0P9200Z"  =  "1"  /* Drainage of Left Rib with Drainage Device, Open Approach */
"0P920ZZ"  =  "1"  /* Drainage of Left Rib, Open Approach */
"0P9230Z"  =  "1"  /* Drainage of Left Rib with Drainage Device, Percutaneous Approach */
"0P923ZZ"  =  "1"  /* Drainage of Left Rib, Percutaneous Approach */
"0P9240Z"  =  "1"  /* Drainage of Left Rib with Drainage Device, Percutaneous Endoscopic Approach */
"0P924ZZ"  =  "1"  /* Drainage of Left Rib, Percutaneous Endoscopic Approach */
"0P9300Z"  =  "1"  /* Drainage of Cervical Vertebra with Drainage Device, Open Approach */
"0P930ZZ"  =  "1"  /* Drainage of Cervical Vertebra, Open Approach */
"0P9330Z"  =  "1"  /* Drainage of Cervical Vertebra with Drainage Device, Percutaneous Approach */
"0P933ZZ"  =  "1"  /* Drainage of Cervical Vertebra, Percutaneous Approach */
"0P9340Z"  =  "1"  /* Drainage of Cervical Vertebra with Drainage Device, Percutaneous Endoscopic Approach */
"0P934ZZ"  =  "1"  /* Drainage of Cervical Vertebra, Percutaneous Endoscopic Approach */
"0P9400Z"  =  "1"  /* Drainage of Thoracic Vertebra with Drainage Device, Open Approach */
"0P940ZZ"  =  "1"  /* Drainage of Thoracic Vertebra, Open Approach */
"0P9430Z"  =  "1"  /* Drainage of Thoracic Vertebra with Drainage Device, Percutaneous Approach */
"0P943ZZ"  =  "1"  /* Drainage of Thoracic Vertebra, Percutaneous Approach */
"0P9440Z"  =  "1"  /* Drainage of Thoracic Vertebra with Drainage Device, Percutaneous Endoscopic Approach */
"0P944ZZ"  =  "1"  /* Drainage of Thoracic Vertebra, Percutaneous Endoscopic Approach */
"0P9500Z"  =  "1"  /* Drainage of Right Scapula with Drainage Device, Open Approach */
"0P950ZZ"  =  "1"  /* Drainage of Right Scapula, Open Approach */
"0P9530Z"  =  "1"  /* Drainage of Right Scapula with Drainage Device, Percutaneous Approach */
"0P953ZZ"  =  "1"  /* Drainage of Right Scapula, Percutaneous Approach */
"0P9540Z"  =  "1"  /* Drainage of Right Scapula with Drainage Device, Percutaneous Endoscopic Approach */
"0P954ZZ"  =  "1"  /* Drainage of Right Scapula, Percutaneous Endoscopic Approach */
"0P9600Z"  =  "1"  /* Drainage of Left Scapula with Drainage Device, Open Approach */
"0P960ZZ"  =  "1"  /* Drainage of Left Scapula, Open Approach */
"0P9630Z"  =  "1"  /* Drainage of Left Scapula with Drainage Device, Percutaneous Approach */
"0P963ZZ"  =  "1"  /* Drainage of Left Scapula, Percutaneous Approach */
"0P9640Z"  =  "1"  /* Drainage of Left Scapula with Drainage Device, Percutaneous Endoscopic Approach */
"0P964ZZ"  =  "1"  /* Drainage of Left Scapula, Percutaneous Endoscopic Approach */
"0P9900Z"  =  "1"  /* Drainage of Right Clavicle with Drainage Device, Open Approach */
"0P990ZZ"  =  "1"  /* Drainage of Right Clavicle, Open Approach */
"0P9930Z"  =  "1"  /* Drainage of Right Clavicle with Drainage Device, Percutaneous Approach */
"0P993ZZ"  =  "1"  /* Drainage of Right Clavicle, Percutaneous Approach */
"0P9940Z"  =  "1"  /* Drainage of Right Clavicle with Drainage Device, Percutaneous Endoscopic Approach */
"0P994ZZ"  =  "1"  /* Drainage of Right Clavicle, Percutaneous Endoscopic Approach */
"0P9B00Z"  =  "1"  /* Drainage of Left Clavicle with Drainage Device, Open Approach */
"0P9B0ZZ"  =  "1"  /* Drainage of Left Clavicle, Open Approach */
"0P9B30Z"  =  "1"  /* Drainage of Left Clavicle with Drainage Device, Percutaneous Approach */
"0P9B3ZZ"  =  "1"  /* Drainage of Left Clavicle, Percutaneous Approach */
"0P9B40Z"  =  "1"  /* Drainage of Left Clavicle with Drainage Device, Percutaneous Endoscopic Approach */
"0P9B4ZZ"  =  "1"  /* Drainage of Left Clavicle, Percutaneous Endoscopic Approach */
"0P9C00Z"  =  "1"  /* Drainage of Right Humeral Head with Drainage Device, Open Approach */
"0P9C0ZZ"  =  "1"  /* Drainage of Right Humeral Head, Open Approach */
"0P9C30Z"  =  "1"  /* Drainage of Right Humeral Head with Drainage Device, Percutaneous Approach */
"0P9C3ZZ"  =  "1"  /* Drainage of Right Humeral Head, Percutaneous Approach */
"0P9C40Z"  =  "1"  /* Drainage of Right Humeral Head with Drainage Device, Percutaneous Endoscopic Approach */
"0P9C4ZZ"  =  "1"  /* Drainage of Right Humeral Head, Percutaneous Endoscopic Approach */
"0P9D00Z"  =  "1"  /* Drainage of Left Humeral Head with Drainage Device, Open Approach */
"0P9D0ZZ"  =  "1"  /* Drainage of Left Humeral Head, Open Approach */
"0P9D30Z"  =  "1"  /* Drainage of Left Humeral Head with Drainage Device, Percutaneous Approach */
"0P9D3ZZ"  =  "1"  /* Drainage of Left Humeral Head, Percutaneous Approach */
"0P9D40Z"  =  "1"  /* Drainage of Left Humeral Head with Drainage Device, Percutaneous Endoscopic Approach */
"0P9D4ZZ"  =  "1"  /* Drainage of Left Humeral Head, Percutaneous Endoscopic Approach */
"0P9F00Z"  =  "1"  /* Drainage of Right Humeral Shaft with Drainage Device, Open Approach */
"0P9F0ZZ"  =  "1"  /* Drainage of Right Humeral Shaft, Open Approach */
"0P9F30Z"  =  "1"  /* Drainage of Right Humeral Shaft with Drainage Device, Percutaneous Approach */
"0P9F3ZZ"  =  "1"  /* Drainage of Right Humeral Shaft, Percutaneous Approach */
"0P9F40Z"  =  "1"  /* Drainage of Right Humeral Shaft with Drainage Device, Percutaneous Endoscopic Approach */
"0P9F4ZZ"  =  "1"  /* Drainage of Right Humeral Shaft, Percutaneous Endoscopic Approach */
"0P9G00Z"  =  "1"  /* Drainage of Left Humeral Shaft with Drainage Device, Open Approach */
"0P9G0ZZ"  =  "1"  /* Drainage of Left Humeral Shaft, Open Approach */
"0P9G30Z"  =  "1"  /* Drainage of Left Humeral Shaft with Drainage Device, Percutaneous Approach */
"0P9G3ZZ"  =  "1"  /* Drainage of Left Humeral Shaft, Percutaneous Approach */
"0P9G40Z"  =  "1"  /* Drainage of Left Humeral Shaft with Drainage Device, Percutaneous Endoscopic Approach */
"0P9G4ZZ"  =  "1"  /* Drainage of Left Humeral Shaft, Percutaneous Endoscopic Approach */
"0P9H00Z"  =  "1"  /* Drainage of Right Radius with Drainage Device, Open Approach */
"0P9H0ZZ"  =  "1"  /* Drainage of Right Radius, Open Approach */
"0P9H30Z"  =  "1"  /* Drainage of Right Radius with Drainage Device, Percutaneous Approach */
"0P9H3ZZ"  =  "1"  /* Drainage of Right Radius, Percutaneous Approach */
"0P9H40Z"  =  "1"  /* Drainage of Right Radius with Drainage Device, Percutaneous Endoscopic Approach */
"0P9H4ZZ"  =  "1"  /* Drainage of Right Radius, Percutaneous Endoscopic Approach */
"0P9J00Z"  =  "1"  /* Drainage of Left Radius with Drainage Device, Open Approach */
"0P9J0ZZ"  =  "1"  /* Drainage of Left Radius, Open Approach */
"0P9J30Z"  =  "1"  /* Drainage of Left Radius with Drainage Device, Percutaneous Approach */
"0P9J3ZZ"  =  "1"  /* Drainage of Left Radius, Percutaneous Approach */
"0P9J40Z"  =  "1"  /* Drainage of Left Radius with Drainage Device, Percutaneous Endoscopic Approach */
"0P9J4ZZ"  =  "1"  /* Drainage of Left Radius, Percutaneous Endoscopic Approach */
"0P9K00Z"  =  "1"  /* Drainage of Right Ulna with Drainage Device, Open Approach */
"0P9K0ZZ"  =  "1"  /* Drainage of Right Ulna, Open Approach */
"0P9K30Z"  =  "1"  /* Drainage of Right Ulna with Drainage Device, Percutaneous Approach */
"0P9K3ZZ"  =  "1"  /* Drainage of Right Ulna, Percutaneous Approach */
"0P9K40Z"  =  "1"  /* Drainage of Right Ulna with Drainage Device, Percutaneous Endoscopic Approach */
"0P9K4ZZ"  =  "1"  /* Drainage of Right Ulna, Percutaneous Endoscopic Approach */
"0P9L00Z"  =  "1"  /* Drainage of Left Ulna with Drainage Device, Open Approach */
"0P9L0ZZ"  =  "1"  /* Drainage of Left Ulna, Open Approach */
"0P9L30Z"  =  "1"  /* Drainage of Left Ulna with Drainage Device, Percutaneous Approach */
"0P9L3ZZ"  =  "1"  /* Drainage of Left Ulna, Percutaneous Approach */
"0P9L40Z"  =  "1"  /* Drainage of Left Ulna with Drainage Device, Percutaneous Endoscopic Approach */
"0P9L4ZZ"  =  "1"  /* Drainage of Left Ulna, Percutaneous Endoscopic Approach */
"0P9M00Z"  =  "1"  /* Drainage of Right Carpal with Drainage Device, Open Approach */
"0P9M0ZZ"  =  "1"  /* Drainage of Right Carpal, Open Approach */
"0P9M30Z"  =  "1"  /* Drainage of Right Carpal with Drainage Device, Percutaneous Approach */
"0P9M3ZZ"  =  "1"  /* Drainage of Right Carpal, Percutaneous Approach */
"0P9M40Z"  =  "1"  /* Drainage of Right Carpal with Drainage Device, Percutaneous Endoscopic Approach */
"0P9M4ZZ"  =  "1"  /* Drainage of Right Carpal, Percutaneous Endoscopic Approach */
"0P9N00Z"  =  "1"  /* Drainage of Left Carpal with Drainage Device, Open Approach */
"0P9N0ZZ"  =  "1"  /* Drainage of Left Carpal, Open Approach */
"0P9N30Z"  =  "1"  /* Drainage of Left Carpal with Drainage Device, Percutaneous Approach */
"0P9N3ZZ"  =  "1"  /* Drainage of Left Carpal, Percutaneous Approach */
"0P9N40Z"  =  "1"  /* Drainage of Left Carpal with Drainage Device, Percutaneous Endoscopic Approach */
"0P9N4ZZ"  =  "1"  /* Drainage of Left Carpal, Percutaneous Endoscopic Approach */
"0P9P00Z"  =  "1"  /* Drainage of Right Metacarpal with Drainage Device, Open Approach */
"0P9P0ZZ"  =  "1"  /* Drainage of Right Metacarpal, Open Approach */
"0P9P30Z"  =  "1"  /* Drainage of Right Metacarpal with Drainage Device, Percutaneous Approach */
"0P9P3ZZ"  =  "1"  /* Drainage of Right Metacarpal, Percutaneous Approach */
"0P9P40Z"  =  "1"  /* Drainage of Right Metacarpal with Drainage Device, Percutaneous Endoscopic Approach */
"0P9P4ZZ"  =  "1"  /* Drainage of Right Metacarpal, Percutaneous Endoscopic Approach */
"0P9Q00Z"  =  "1"  /* Drainage of Left Metacarpal with Drainage Device, Open Approach */
"0P9Q0ZZ"  =  "1"  /* Drainage of Left Metacarpal, Open Approach */
"0P9Q30Z"  =  "1"  /* Drainage of Left Metacarpal with Drainage Device, Percutaneous Approach */
"0P9Q3ZZ"  =  "1"  /* Drainage of Left Metacarpal, Percutaneous Approach */
"0P9Q40Z"  =  "1"  /* Drainage of Left Metacarpal with Drainage Device, Percutaneous Endoscopic Approach */
"0P9Q4ZZ"  =  "1"  /* Drainage of Left Metacarpal, Percutaneous Endoscopic Approach */
"0P9T00Z"  =  "1"  /* Drainage of Right Finger Phalanx with Drainage Device, Open Approach */
"0P9T0ZZ"  =  "1"  /* Drainage of Right Finger Phalanx, Open Approach */
"0P9T30Z"  =  "1"  /* Drainage of Right Finger Phalanx with Drainage Device, Percutaneous Approach */
"0P9T3ZZ"  =  "1"  /* Drainage of Right Finger Phalanx, Percutaneous Approach */
"0P9T40Z"  =  "1"  /* Drainage of Right Finger Phalanx with Drainage Device, Percutaneous Endoscopic Approach */
"0P9T4ZZ"  =  "1"  /* Drainage of Right Finger Phalanx, Percutaneous Endoscopic Approach */
"0P9V00Z"  =  "1"  /* Drainage of Left Finger Phalanx with Drainage Device, Open Approach */
"0P9V0ZZ"  =  "1"  /* Drainage of Left Finger Phalanx, Open Approach */
"0P9V30Z"  =  "1"  /* Drainage of Left Finger Phalanx with Drainage Device, Percutaneous Approach */
"0P9V3ZZ"  =  "1"  /* Drainage of Left Finger Phalanx, Percutaneous Approach */
"0P9V40Z"  =  "1"  /* Drainage of Left Finger Phalanx with Drainage Device, Percutaneous Endoscopic Approach */
"0P9V4ZZ"  =  "1"  /* Drainage of Left Finger Phalanx, Percutaneous Endoscopic Approach */
"0Q9000Z"  =  "1"  /* Drainage of Lumbar Vertebra with Drainage Device, Open Approach */
"0Q900ZZ"  =  "1"  /* Drainage of Lumbar Vertebra, Open Approach */
"0Q9030Z"  =  "1"  /* Drainage of Lumbar Vertebra with Drainage Device, Percutaneous Approach */
"0Q903ZZ"  =  "1"  /* Drainage of Lumbar Vertebra, Percutaneous Approach */
"0Q9040Z"  =  "1"  /* Drainage of Lumbar Vertebra with Drainage Device, Percutaneous Endoscopic Approach */
"0Q904ZZ"  =  "1"  /* Drainage of Lumbar Vertebra, Percutaneous Endoscopic Approach */
"0Q9200Z"  =  "1"  /* Drainage of Right Pelvic Bone with Drainage Device, Open Approach */
"0Q920ZZ"  =  "1"  /* Drainage of Right Pelvic Bone, Open Approach */
"0Q9230Z"  =  "1"  /* Drainage of Right Pelvic Bone with Drainage Device, Percutaneous Approach */
"0Q923ZZ"  =  "1"  /* Drainage of Right Pelvic Bone, Percutaneous Approach */
"0Q9240Z"  =  "1"  /* Drainage of Right Pelvic Bone with Drainage Device, Percutaneous Endoscopic Approach */
"0Q924ZZ"  =  "1"  /* Drainage of Right Pelvic Bone, Percutaneous Endoscopic Approach */
"0Q9300Z"  =  "1"  /* Drainage of Left Pelvic Bone with Drainage Device, Open Approach */
"0Q930ZZ"  =  "1"  /* Drainage of Left Pelvic Bone, Open Approach */
"0Q9330Z"  =  "1"  /* Drainage of Left Pelvic Bone with Drainage Device, Percutaneous Approach */
"0Q933ZZ"  =  "1"  /* Drainage of Left Pelvic Bone, Percutaneous Approach */
"0Q9340Z"  =  "1"  /* Drainage of Left Pelvic Bone with Drainage Device, Percutaneous Endoscopic Approach */
"0Q934ZZ"  =  "1"  /* Drainage of Left Pelvic Bone, Percutaneous Endoscopic Approach */
"0Q9600Z"  =  "1"  /* Drainage of Right Upper Femur with Drainage Device, Open Approach */
"0Q960ZZ"  =  "1"  /* Drainage of Right Upper Femur, Open Approach */
"0Q9630Z"  =  "1"  /* Drainage of Right Upper Femur with Drainage Device, Percutaneous Approach */
"0Q963ZZ"  =  "1"  /* Drainage of Right Upper Femur, Percutaneous Approach */
"0Q9640Z"  =  "1"  /* Drainage of Right Upper Femur with Drainage Device, Percutaneous Endoscopic Approach */
"0Q964ZZ"  =  "1"  /* Drainage of Right Upper Femur, Percutaneous Endoscopic Approach */
"0Q9700Z"  =  "1"  /* Drainage of Left Upper Femur with Drainage Device, Open Approach */
"0Q970ZZ"  =  "1"  /* Drainage of Left Upper Femur, Open Approach */
"0Q9730Z"  =  "1"  /* Drainage of Left Upper Femur with Drainage Device, Percutaneous Approach */
"0Q973ZZ"  =  "1"  /* Drainage of Left Upper Femur, Percutaneous Approach */
"0Q9740Z"  =  "1"  /* Drainage of Left Upper Femur with Drainage Device, Percutaneous Endoscopic Approach */
"0Q974ZZ"  =  "1"  /* Drainage of Left Upper Femur, Percutaneous Endoscopic Approach */
"0Q9800Z"  =  "1"  /* Drainage of Right Femoral Shaft with Drainage Device, Open Approach */
"0Q980ZZ"  =  "1"  /* Drainage of Right Femoral Shaft, Open Approach */
"0Q9830Z"  =  "1"  /* Drainage of Right Femoral Shaft with Drainage Device, Percutaneous Approach */
"0Q983ZZ"  =  "1"  /* Drainage of Right Femoral Shaft, Percutaneous Approach */
"0Q9840Z"  =  "1"  /* Drainage of Right Femoral Shaft with Drainage Device, Percutaneous Endoscopic Approach */
"0Q984ZZ"  =  "1"  /* Drainage of Right Femoral Shaft, Percutaneous Endoscopic Approach */
"0Q9900Z"  =  "1"  /* Drainage of Left Femoral Shaft with Drainage Device, Open Approach */
"0Q990ZZ"  =  "1"  /* Drainage of Left Femoral Shaft, Open Approach */
"0Q9930Z"  =  "1"  /* Drainage of Left Femoral Shaft with Drainage Device, Percutaneous Approach */
"0Q993ZZ"  =  "1"  /* Drainage of Left Femoral Shaft, Percutaneous Approach */
"0Q9940Z"  =  "1"  /* Drainage of Left Femoral Shaft with Drainage Device, Percutaneous Endoscopic Approach */
"0Q994ZZ"  =  "1"  /* Drainage of Left Femoral Shaft, Percutaneous Endoscopic Approach */
"0Q9B00Z"  =  "1"  /* Drainage of Right Lower Femur with Drainage Device, Open Approach */
"0Q9B0ZZ"  =  "1"  /* Drainage of Right Lower Femur, Open Approach */
"0Q9B30Z"  =  "1"  /* Drainage of Right Lower Femur with Drainage Device, Percutaneous Approach */
"0Q9B3ZZ"  =  "1"  /* Drainage of Right Lower Femur, Percutaneous Approach */
"0Q9B40Z"  =  "1"  /* Drainage of Right Lower Femur with Drainage Device, Percutaneous Endoscopic Approach */
"0Q9B4ZZ"  =  "1"  /* Drainage of Right Lower Femur, Percutaneous Endoscopic Approach */
"0Q9C00Z"  =  "1"  /* Drainage of Left Lower Femur with Drainage Device, Open Approach */
"0Q9C0ZZ"  =  "1"  /* Drainage of Left Lower Femur, Open Approach */
"0Q9C30Z"  =  "1"  /* Drainage of Left Lower Femur with Drainage Device, Percutaneous Approach */
"0Q9C3ZZ"  =  "1"  /* Drainage of Left Lower Femur, Percutaneous Approach */
"0Q9C40Z"  =  "1"  /* Drainage of Left Lower Femur with Drainage Device, Percutaneous Endoscopic Approach */
"0Q9C4ZZ"  =  "1"  /* Drainage of Left Lower Femur, Percutaneous Endoscopic Approach */
"0Q9D00Z"  =  "1"  /* Drainage of Right Patella with Drainage Device, Open Approach */
"0Q9D0ZZ"  =  "1"  /* Drainage of Right Patella, Open Approach */
"0Q9D30Z"  =  "1"  /* Drainage of Right Patella with Drainage Device, Percutaneous Approach */
"0Q9D3ZZ"  =  "1"  /* Drainage of Right Patella, Percutaneous Approach */
"0Q9D40Z"  =  "1"  /* Drainage of Right Patella with Drainage Device, Percutaneous Endoscopic Approach */
"0Q9D4ZZ"  =  "1"  /* Drainage of Right Patella, Percutaneous Endoscopic Approach */
"0Q9F00Z"  =  "1"  /* Drainage of Left Patella with Drainage Device, Open Approach */
"0Q9F0ZZ"  =  "1"  /* Drainage of Left Patella, Open Approach */
"0Q9F30Z"  =  "1"  /* Drainage of Left Patella with Drainage Device, Percutaneous Approach */
"0Q9F3ZZ"  =  "1"  /* Drainage of Left Patella, Percutaneous Approach */
"0Q9F40Z"  =  "1"  /* Drainage of Left Patella with Drainage Device, Percutaneous Endoscopic Approach */
"0Q9F4ZZ"  =  "1"  /* Drainage of Left Patella, Percutaneous Endoscopic Approach */
"0Q9G00Z"  =  "1"  /* Drainage of Right Tibia with Drainage Device, Open Approach */
"0Q9G0ZZ"  =  "1"  /* Drainage of Right Tibia, Open Approach */
"0Q9G30Z"  =  "1"  /* Drainage of Right Tibia with Drainage Device, Percutaneous Approach */
"0Q9G3ZZ"  =  "1"  /* Drainage of Right Tibia, Percutaneous Approach */
"0Q9G40Z"  =  "1"  /* Drainage of Right Tibia with Drainage Device, Percutaneous Endoscopic Approach */
"0Q9G4ZZ"  =  "1"  /* Drainage of Right Tibia, Percutaneous Endoscopic Approach */
"0Q9H00Z"  =  "1"  /* Drainage of Left Tibia with Drainage Device, Open Approach */
"0Q9H0ZZ"  =  "1"  /* Drainage of Left Tibia, Open Approach */
"0Q9H30Z"  =  "1"  /* Drainage of Left Tibia with Drainage Device, Percutaneous Approach */
"0Q9H3ZZ"  =  "1"  /* Drainage of Left Tibia, Percutaneous Approach */
"0Q9H40Z"  =  "1"  /* Drainage of Left Tibia with Drainage Device, Percutaneous Endoscopic Approach */
"0Q9H4ZZ"  =  "1"  /* Drainage of Left Tibia, Percutaneous Endoscopic Approach */
"0Q9J00Z"  =  "1"  /* Drainage of Right Fibula with Drainage Device, Open Approach */
"0Q9J0ZZ"  =  "1"  /* Drainage of Right Fibula, Open Approach */
"0Q9J30Z"  =  "1"  /* Drainage of Right Fibula with Drainage Device, Percutaneous Approach */
"0Q9J3ZZ"  =  "1"  /* Drainage of Right Fibula, Percutaneous Approach */
"0Q9J40Z"  =  "1"  /* Drainage of Right Fibula with Drainage Device, Percutaneous Endoscopic Approach */
"0Q9J4ZZ"  =  "1"  /* Drainage of Right Fibula, Percutaneous Endoscopic Approach */
"0Q9K00Z"  =  "1"  /* Drainage of Left Fibula with Drainage Device, Open Approach */
"0Q9K0ZZ"  =  "1"  /* Drainage of Left Fibula, Open Approach */
"0Q9K30Z"  =  "1"  /* Drainage of Left Fibula with Drainage Device, Percutaneous Approach */
"0Q9K3ZZ"  =  "1"  /* Drainage of Left Fibula, Percutaneous Approach */
"0Q9K40Z"  =  "1"  /* Drainage of Left Fibula with Drainage Device, Percutaneous Endoscopic Approach */
"0Q9K4ZZ"  =  "1"  /* Drainage of Left Fibula, Percutaneous Endoscopic Approach */
"0Q9L00Z"  =  "1"  /* Drainage of Right Tarsal with Drainage Device, Open Approach */
"0Q9L0ZZ"  =  "1"  /* Drainage of Right Tarsal, Open Approach */
"0Q9L30Z"  =  "1"  /* Drainage of Right Tarsal with Drainage Device, Percutaneous Approach */
"0Q9L3ZZ"  =  "1"  /* Drainage of Right Tarsal, Percutaneous Approach */
"0Q9L40Z"  =  "1"  /* Drainage of Right Tarsal with Drainage Device, Percutaneous Endoscopic Approach */
"0Q9L4ZZ"  =  "1"  /* Drainage of Right Tarsal, Percutaneous Endoscopic Approach */
"0Q9M00Z"  =  "1"  /* Drainage of Left Tarsal with Drainage Device, Open Approach */
"0Q9M0ZZ"  =  "1"  /* Drainage of Left Tarsal, Open Approach */
"0Q9M30Z"  =  "1"  /* Drainage of Left Tarsal with Drainage Device, Percutaneous Approach */
"0Q9M3ZZ"  =  "1"  /* Drainage of Left Tarsal, Percutaneous Approach */
"0Q9M40Z"  =  "1"  /* Drainage of Left Tarsal with Drainage Device, Percutaneous Endoscopic Approach */
"0Q9M4ZZ"  =  "1"  /* Drainage of Left Tarsal, Percutaneous Endoscopic Approach */
"0Q9N00Z"  =  "1"  /* Drainage of Right Metatarsal with Drainage Device, Open Approach */
"0Q9N0ZZ"  =  "1"  /* Drainage of Right Metatarsal, Open Approach */
"0Q9N30Z"  =  "1"  /* Drainage of Right Metatarsal with Drainage Device, Percutaneous Approach */
"0Q9N3ZZ"  =  "1"  /* Drainage of Right Metatarsal, Percutaneous Approach */
"0Q9N40Z"  =  "1"  /* Drainage of Right Metatarsal with Drainage Device, Percutaneous Endoscopic Approach */
"0Q9N4ZZ"  =  "1"  /* Drainage of Right Metatarsal, Percutaneous Endoscopic Approach */
"0Q9P00Z"  =  "1"  /* Drainage of Left Metatarsal with Drainage Device, Open Approach */
"0Q9P0ZZ"  =  "1"  /* Drainage of Left Metatarsal, Open Approach */
"0Q9P30Z"  =  "1"  /* Drainage of Left Metatarsal with Drainage Device, Percutaneous Approach */
"0Q9P3ZZ"  =  "1"  /* Drainage of Left Metatarsal, Percutaneous Approach */
"0Q9P40Z"  =  "1"  /* Drainage of Left Metatarsal with Drainage Device, Percutaneous Endoscopic Approach */
"0Q9P4ZZ"  =  "1"  /* Drainage of Left Metatarsal, Percutaneous Endoscopic Approach */
"0Q9Q00Z"  =  "1"  /* Drainage of Right Toe Phalanx with Drainage Device, Open Approach */
"0Q9Q0ZZ"  =  "1"  /* Drainage of Right Toe Phalanx, Open Approach */
"0Q9Q30Z"  =  "1"  /* Drainage of Right Toe Phalanx with Drainage Device, Percutaneous Approach */
"0Q9Q3ZZ"  =  "1"  /* Drainage of Right Toe Phalanx, Percutaneous Approach */
"0Q9Q40Z"  =  "1"  /* Drainage of Right Toe Phalanx with Drainage Device, Percutaneous Endoscopic Approach */
"0Q9Q4ZZ"  =  "1"  /* Drainage of Right Toe Phalanx, Percutaneous Endoscopic Approach */
"0Q9R00Z"  =  "1"  /* Drainage of Left Toe Phalanx with Drainage Device, Open Approach */
"0Q9R0ZZ"  =  "1"  /* Drainage of Left Toe Phalanx, Open Approach */
"0Q9R30Z"  =  "1"  /* Drainage of Left Toe Phalanx with Drainage Device, Percutaneous Approach */
"0Q9R3ZZ"  =  "1"  /* Drainage of Left Toe Phalanx, Percutaneous Approach */
"0Q9R40Z"  =  "1"  /* Drainage of Left Toe Phalanx with Drainage Device, Percutaneous Endoscopic Approach */
"0Q9R4ZZ"  =  "1"  /* Drainage of Left Toe Phalanx, Percutaneous Endoscopic Approach */
"0QWY00Z"  =  "1"  /* Revision of Drainage Device in Lower Bone, Open Approach */
"0QWY30Z"  =  "1"  /* Revision of Drainage Device in Lower Bone, Percutaneous Approach */
"0QWY40Z"  =  "1"  /* Revision of Drainage Device in Lower Bone, Percutaneous Endoscopic Approach */
"0R9000Z"  =  "1"  /* Drainage of Occipital-cervical Joint with Drainage Device, Open Approach */
"0R900ZZ"  =  "1"  /* Drainage of Occipital-cervical Joint, Open Approach */
"0R9100Z"  =  "1"  /* Drainage of Cervical Vertebral Joint with Drainage Device, Open Approach */
"0R910ZZ"  =  "1"  /* Drainage of Cervical Vertebral Joint, Open Approach */
"0R9300Z"  =  "1"  /* Drainage of Cervical Vertebral Disc with Drainage Device, Open Approach */
"0R930ZZ"  =  "1"  /* Drainage of Cervical Vertebral Disc, Open Approach */
"0R9400Z"  =  "1"  /* Drainage of Cervicothoracic Vertebral Joint with Drainage Device, Open Approach */
"0R940ZZ"  =  "1"  /* Drainage of Cervicothoracic Vertebral Joint, Open Approach */
"0R9500Z"  =  "1"  /* Drainage of Cervicothoracic Vertebral Disc with Drainage Device, Open Approach */
"0R950ZZ"  =  "1"  /* Drainage of Cervicothoracic Vertebral Disc, Open Approach */
"0R9600Z"  =  "1"  /* Drainage of Thoracic Vertebral Joint with Drainage Device, Open Approach */
"0R960ZZ"  =  "1"  /* Drainage of Thoracic Vertebral Joint, Open Approach */
"0R9900Z"  =  "1"  /* Drainage of Thoracic Vertebral Disc with Drainage Device, Open Approach */
"0R990ZZ"  =  "1"  /* Drainage of Thoracic Vertebral Disc, Open Approach */
"0R9A00Z"  =  "1"  /* Drainage of Thoracolumbar Vertebral Joint with Drainage Device, Open Approach */
"0R9A0ZZ"  =  "1"  /* Drainage of Thoracolumbar Vertebral Joint, Open Approach */
"0R9B00Z"  =  "1"  /* Drainage of Thoracolumbar Vertebral Disc with Drainage Device, Open Approach */
"0R9B0ZZ"  =  "1"  /* Drainage of Thoracolumbar Vertebral Disc, Open Approach */
"0R9C00Z"  =  "1"  /* Drainage of Right Temporomandibular Joint with Drainage Device, Open Approach */
"0R9C0ZZ"  =  "1"  /* Drainage of Right Temporomandibular Joint, Open Approach */
"0R9D00Z"  =  "1"  /* Drainage of Left Temporomandibular Joint with Drainage Device, Open Approach */
"0R9D0ZZ"  =  "1"  /* Drainage of Left Temporomandibular Joint, Open Approach */
"0R9J00Z"  =  "1"  /* Drainage of Right Shoulder Joint with Drainage Device, Open Approach */
"0R9J0ZZ"  =  "1"  /* Drainage of Right Shoulder Joint, Open Approach */
"0R9K00Z"  =  "1"  /* Drainage of Left Shoulder Joint with Drainage Device, Open Approach */
"0R9K0ZZ"  =  "1"  /* Drainage of Left Shoulder Joint, Open Approach */
"0R9L00Z"  =  "1"  /* Drainage of Right Elbow Joint with Drainage Device, Open Approach */
"0R9L0ZZ"  =  "1"  /* Drainage of Right Elbow Joint, Open Approach */
"0R9M00Z"  =  "1"  /* Drainage of Left Elbow Joint with Drainage Device, Open Approach */
"0R9M0ZZ"  =  "1"  /* Drainage of Left Elbow Joint, Open Approach */
"0R9N00Z"  =  "1"  /* Drainage of Right Wrist Joint with Drainage Device, Open Approach */
"0R9N0ZZ"  =  "1"  /* Drainage of Right Wrist Joint, Open Approach */
"0R9P00Z"  =  "1"  /* Drainage of Left Wrist Joint with Drainage Device, Open Approach */
"0R9P0ZZ"  =  "1"  /* Drainage of Left Wrist Joint, Open Approach */
"0R9Q00Z"  =  "1"  /* Drainage of Right Carpal Joint with Drainage Device, Open Approach */
"0R9Q0ZZ"  =  "1"  /* Drainage of Right Carpal Joint, Open Approach */
"0R9R00Z"  =  "1"  /* Drainage of Left Carpal Joint with Drainage Device, Open Approach */
"0R9R0ZZ"  =  "1"  /* Drainage of Left Carpal Joint, Open Approach */
"0R9S00Z"  =  "1"  /* Drainage of Right Metacarpocarpal Joint with Drainage Device, Open Approach */
"0R9S0ZZ"  =  "1"  /* Drainage of Right Metacarpocarpal Joint, Open Approach */
"0R9T00Z"  =  "1"  /* Drainage of Left Metacarpocarpal Joint with Drainage Device, Open Approach */
"0R9T0ZZ"  =  "1"  /* Drainage of Left Metacarpocarpal Joint, Open Approach */
"0R9U00Z"  =  "1"  /* Drainage of Right Metacarpophalangeal Joint with Drainage Device, Open Approach */
"0R9U0ZZ"  =  "1"  /* Drainage of Right Metacarpophalangeal Joint, Open Approach */
"0R9V00Z"  =  "1"  /* Drainage of Left Metacarpophalangeal Joint with Drainage Device, Open Approach */
"0R9V0ZZ"  =  "1"  /* Drainage of Left Metacarpophalangeal Joint, Open Approach */
"0R9W00Z"  =  "1"  /* Drainage of Right Finger Phalangeal Joint with Drainage Device, Open Approach */
"0R9W0ZZ"  =  "1"  /* Drainage of Right Finger Phalangeal Joint, Open Approach */
"0R9X00Z"  =  "1"  /* Drainage of Left Finger Phalangeal Joint with Drainage Device, Open Approach */
"0R9X0ZZ"  =  "1"  /* Drainage of Left Finger Phalangeal Joint, Open Approach */
"0RB00ZZ"  =  "1"  /* Excision of Occipital-cervical Joint, Open Approach */
"0RB03ZZ"  =  "1"  /* Excision of Occipital-cervical Joint, Percutaneous Approach */
"0RB04ZZ"  =  "1"  /* Excision of Occipital-cervical Joint, Percutaneous Endoscopic Approach */
"0RB10ZZ"  =  "1"  /* Excision of Cervical Vertebral Joint, Open Approach */
"0RB13ZZ"  =  "1"  /* Excision of Cervical Vertebral Joint, Percutaneous Approach */
"0RB14ZZ"  =  "1"  /* Excision of Cervical Vertebral Joint, Percutaneous Endoscopic Approach */
"0RB40ZZ"  =  "1"  /* Excision of Cervicothoracic Vertebral Joint, Open Approach */
"0RB43ZZ"  =  "1"  /* Excision of Cervicothoracic Vertebral Joint, Percutaneous Approach */
"0RB44ZZ"  =  "1"  /* Excision of Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RB60ZZ"  =  "1"  /* Excision of Thoracic Vertebral Joint, Open Approach */
"0RB63ZZ"  =  "1"  /* Excision of Thoracic Vertebral Joint, Percutaneous Approach */
"0RB64ZZ"  =  "1"  /* Excision of Thoracic Vertebral Joint, Percutaneous Endoscopic Approach */
"0RBA0ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Joint, Open Approach */
"0RBA3ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Joint, Percutaneous Approach */
"0RBA4ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0S9000Z"  =  "1"  /* Drainage of Lumbar Vertebral Joint with Drainage Device, Open Approach */
"0S900ZZ"  =  "1"  /* Drainage of Lumbar Vertebral Joint, Open Approach */
"0S9200Z"  =  "1"  /* Drainage of Lumbar Vertebral Disc with Drainage Device, Open Approach */
"0S920ZZ"  =  "1"  /* Drainage of Lumbar Vertebral Disc, Open Approach */
"0S9300Z"  =  "1"  /* Drainage of Lumbosacral Joint with Drainage Device, Open Approach */
"0S930ZZ"  =  "1"  /* Drainage of Lumbosacral Joint, Open Approach */
"0S9400Z"  =  "1"  /* Drainage of Lumbosacral Disc with Drainage Device, Open Approach */
"0S940ZZ"  =  "1"  /* Drainage of Lumbosacral Disc, Open Approach */
"0S9500Z"  =  "1"  /* Drainage of Sacrococcygeal Joint with Drainage Device, Open Approach */
"0S950ZZ"  =  "1"  /* Drainage of Sacrococcygeal Joint, Open Approach */
"0S9600Z"  =  "1"  /* Drainage of Coccygeal Joint with Drainage Device, Open Approach */
"0S960ZZ"  =  "1"  /* Drainage of Coccygeal Joint, Open Approach */
"0S9700Z"  =  "1"  /* Drainage of Right Sacroiliac Joint with Drainage Device, Open Approach */
"0S970ZZ"  =  "1"  /* Drainage of Right Sacroiliac Joint, Open Approach */
"0S9800Z"  =  "1"  /* Drainage of Left Sacroiliac Joint with Drainage Device, Open Approach */
"0S980ZZ"  =  "1"  /* Drainage of Left Sacroiliac Joint, Open Approach */
"0S9900Z"  =  "1"  /* Drainage of Right Hip Joint with Drainage Device, Open Approach */
"0S990ZZ"  =  "1"  /* Drainage of Right Hip Joint, Open Approach */
"0S9B00Z"  =  "1"  /* Drainage of Left Hip Joint with Drainage Device, Open Approach */
"0S9B0ZZ"  =  "1"  /* Drainage of Left Hip Joint, Open Approach */
"0S9C00Z"  =  "1"  /* Drainage of Right Knee Joint with Drainage Device, Open Approach */
"0S9C0ZZ"  =  "1"  /* Drainage of Right Knee Joint, Open Approach */
"0S9D00Z"  =  "1"  /* Drainage of Left Knee Joint with Drainage Device, Open Approach */
"0S9D0ZZ"  =  "1"  /* Drainage of Left Knee Joint, Open Approach */
"0S9F00Z"  =  "1"  /* Drainage of Right Ankle Joint with Drainage Device, Open Approach */
"0S9F0ZZ"  =  "1"  /* Drainage of Right Ankle Joint, Open Approach */
"0S9G00Z"  =  "1"  /* Drainage of Left Ankle Joint with Drainage Device, Open Approach */
"0S9G0ZZ"  =  "1"  /* Drainage of Left Ankle Joint, Open Approach */
"0S9K00Z"  =  "1"  /* Drainage of Right Metatarsal-Tarsal Joint with Drainage Device, Open Approach */
"0S9K0ZZ"  =  "1"  /* Drainage of Right Metatarsal-Tarsal Joint, Open Approach */
"0S9L00Z"  =  "1"  /* Drainage of Left Metatarsal-Tarsal Joint with Drainage Device, Open Approach */
"0S9L0ZZ"  =  "1"  /* Drainage of Left Metatarsal-Tarsal Joint, Open Approach */
"0S9M00Z"  =  "1"  /* Drainage of Right Metatarsal-Phalangeal Joint with Drainage Device, Open Approach */
"0S9M0ZZ"  =  "1"  /* Drainage of Right Metatarsal-Phalangeal Joint, Open Approach */
"0S9N00Z"  =  "1"  /* Drainage of Left Metatarsal-Phalangeal Joint with Drainage Device, Open Approach */
"0S9N0ZZ"  =  "1"  /* Drainage of Left Metatarsal-Phalangeal Joint, Open Approach */
"0S9P00Z"  =  "1"  /* Drainage of Right Toe Phalangeal Joint with Drainage Device, Open Approach */
"0S9P0ZZ"  =  "1"  /* Drainage of Right Toe Phalangeal Joint, Open Approach */
"0S9Q00Z"  =  "1"  /* Drainage of Left Toe Phalangeal Joint with Drainage Device, Open Approach */
"0S9Q0ZZ"  =  "1"  /* Drainage of Left Toe Phalangeal Joint, Open Approach */
"0SB00ZZ"  =  "1"  /* Excision of Lumbar Vertebral Joint, Open Approach */
"0SB03ZZ"  =  "1"  /* Excision of Lumbar Vertebral Joint, Percutaneous Approach */
"0SB04ZZ"  =  "1"  /* Excision of Lumbar Vertebral Joint, Percutaneous Endoscopic Approach */
"0SB30ZZ"  =  "1"  /* Excision of Lumbosacral Joint, Open Approach */
"0SB33ZZ"  =  "1"  /* Excision of Lumbosacral Joint, Percutaneous Approach */
"0SB34ZZ"  =  "1"  /* Excision of Lumbosacral Joint, Percutaneous Endoscopic Approach */
"0SB50ZZ"  =  "1"  /* Excision of Sacrococcygeal Joint, Open Approach */
"0SB53ZZ"  =  "1"  /* Excision of Sacrococcygeal Joint, Percutaneous Approach */
"0SB54ZZ"  =  "1"  /* Excision of Sacrococcygeal Joint, Percutaneous Endoscopic Approach */
"0SB60ZZ"  =  "1"  /* Excision of Coccygeal Joint, Open Approach */
"0SB63ZZ"  =  "1"  /* Excision of Coccygeal Joint, Percutaneous Approach */
"0SB64ZZ"  =  "1"  /* Excision of Coccygeal Joint, Percutaneous Endoscopic Approach */
"0SB70ZZ"  =  "1"  /* Excision of Right Sacroiliac Joint, Open Approach */
"0SB73ZZ"  =  "1"  /* Excision of Right Sacroiliac Joint, Percutaneous Approach */
"0SB74ZZ"  =  "1"  /* Excision of Right Sacroiliac Joint, Percutaneous Endoscopic Approach */
"0SB80ZZ"  =  "1"  /* Excision of Left Sacroiliac Joint, Open Approach */
"0SB83ZZ"  =  "1"  /* Excision of Left Sacroiliac Joint, Percutaneous Approach */
"0SB84ZZ"  =  "1"  /* Excision of Left Sacroiliac Joint, Percutaneous Endoscopic Approach */
"0T9000Z"  =  "1"  /* Drainage of Right Kidney with Drainage Device, Open Approach */
"0T900ZZ"  =  "1"  /* Drainage of Right Kidney, Open Approach */
"0T9030Z"  =  "1"  /* Drainage of Right Kidney with Drainage Device, Percutaneous Approach */
"0T903ZZ"  =  "1"  /* Drainage of Right Kidney, Percutaneous Approach */
"0T9040Z"  =  "1"  /* Drainage of Right Kidney with Drainage Device, Percutaneous Endoscopic Approach */
"0T904ZZ"  =  "1"  /* Drainage of Right Kidney, Percutaneous Endoscopic Approach */
"0T907ZZ"  =  "1"  /* Drainage of Right Kidney, Via Natural or Artificial Opening */
"0T908ZZ"  =  "1"  /* Drainage of Right Kidney, Via Natural or Artificial Opening Endoscopic */
"0T9100Z"  =  "1"  /* Drainage of Left Kidney with Drainage Device, Open Approach */
"0T910ZZ"  =  "1"  /* Drainage of Left Kidney, Open Approach */
"0T9130Z"  =  "1"  /* Drainage of Left Kidney with Drainage Device, Percutaneous Approach */
"0T913ZZ"  =  "1"  /* Drainage of Left Kidney, Percutaneous Approach */
"0T9140Z"  =  "1"  /* Drainage of Left Kidney with Drainage Device, Percutaneous Endoscopic Approach */
"0T914ZZ"  =  "1"  /* Drainage of Left Kidney, Percutaneous Endoscopic Approach */
"0T917ZZ"  =  "1"  /* Drainage of Left Kidney, Via Natural or Artificial Opening */
"0T918ZZ"  =  "1"  /* Drainage of Left Kidney, Via Natural or Artificial Opening Endoscopic */
"0T9300Z"  =  "1"  /* Drainage of Right Kidney Pelvis with Drainage Device, Open Approach */
"0T930ZZ"  =  "1"  /* Drainage of Right Kidney Pelvis, Open Approach */
"0T9330Z"  =  "1"  /* Drainage of Right Kidney Pelvis with Drainage Device, Percutaneous Approach */
"0T9340Z"  =  "1"  /* Drainage of Right Kidney Pelvis with Drainage Device, Percutaneous Endoscopic Approach */
"0T937ZZ"  =  "1"  /* Drainage of Right Kidney Pelvis, Via Natural or Artificial Opening */
"0T938ZZ"  =  "1"  /* Drainage of Right Kidney Pelvis, Via Natural or Artificial Opening Endoscopic */
"0T9400Z"  =  "1"  /* Drainage of Left Kidney Pelvis with Drainage Device, Open Approach */
"0T940ZZ"  =  "1"  /* Drainage of Left Kidney Pelvis, Open Approach */
"0T9430Z"  =  "1"  /* Drainage of Left Kidney Pelvis with Drainage Device, Percutaneous Approach */
"0T9440Z"  =  "1"  /* Drainage of Left Kidney Pelvis with Drainage Device, Percutaneous Endoscopic Approach */
"0T947ZZ"  =  "1"  /* Drainage of Left Kidney Pelvis, Via Natural or Artificial Opening */
"0T948ZZ"  =  "1"  /* Drainage of Left Kidney Pelvis, Via Natural or Artificial Opening Endoscopic */
"0T960ZZ"  =  "1"  /* Drainage of Right Ureter, Open Approach */
"0T963ZZ"  =  "1"  /* Drainage of Right Ureter, Percutaneous Approach */
"0T964ZZ"  =  "1"  /* Drainage of Right Ureter, Percutaneous Endoscopic Approach */
"0T967ZZ"  =  "1"  /* Drainage of Right Ureter, Via Natural or Artificial Opening */
"0T968ZZ"  =  "1"  /* Drainage of Right Ureter, Via Natural or Artificial Opening Endoscopic */
"0T970ZZ"  =  "1"  /* Drainage of Left Ureter, Open Approach */
"0T973ZZ"  =  "1"  /* Drainage of Left Ureter, Percutaneous Approach */
"0T974ZZ"  =  "1"  /* Drainage of Left Ureter, Percutaneous Endoscopic Approach */
"0T977ZZ"  =  "1"  /* Drainage of Left Ureter, Via Natural or Artificial Opening */
"0T978ZZ"  =  "1"  /* Drainage of Left Ureter, Via Natural or Artificial Opening Endoscopic */
"0T980ZZ"  =  "1"  /* Drainage of Bilateral Ureters, Open Approach */
"0T983ZZ"  =  "1"  /* Drainage of Bilateral Ureters, Percutaneous Approach */
"0T984ZZ"  =  "1"  /* Drainage of Bilateral Ureters, Percutaneous Endoscopic Approach */
"0T987ZZ"  =  "1"  /* Drainage of Bilateral Ureters, Via Natural or Artificial Opening */
"0T988ZZ"  =  "1"  /* Drainage of Bilateral Ureters, Via Natural or Artificial Opening Endoscopic */
"0T9B00Z"  =  "1"  /* Drainage of Bladder with Drainage Device, Open Approach */
"0T9B0ZZ"  =  "1"  /* Drainage of Bladder, Open Approach */
"0T9C00Z"  =  "1"  /* Drainage of Bladder Neck with Drainage Device, Open Approach */
"0T9C0ZZ"  =  "1"  /* Drainage of Bladder Neck, Open Approach */
"0T9D00Z"  =  "1"  /* Drainage of Urethra with Drainage Device, Open Approach */
"0T9D30Z"  =  "1"  /* Drainage of Urethra with Drainage Device, Percutaneous Approach */
"0T9D40Z"  =  "1"  /* Drainage of Urethra with Drainage Device, Percutaneous Endoscopic Approach */
"0TC00ZZ"  =  "1"  /* Extirpation of Matter from Right Kidney, Open Approach */
"0TC07ZZ"  =  "1"  /* Extirpation of Matter from Right Kidney, Via Natural or Artificial Opening */
"0TC08ZZ"  =  "1"  /* Extirpation of Matter from Right Kidney, Via Natural or Artificial Opening Endoscopic */
"0TC10ZZ"  =  "1"  /* Extirpation of Matter from Left Kidney, Open Approach */
"0TC17ZZ"  =  "1"  /* Extirpation of Matter from Left Kidney, Via Natural or Artificial Opening */
"0TC18ZZ"  =  "1"  /* Extirpation of Matter from Left Kidney, Via Natural or Artificial Opening Endoscopic */
"0TC30ZZ"  =  "1"  /* Extirpation of Matter from Right Kidney Pelvis, Open Approach */
"0TC40ZZ"  =  "1"  /* Extirpation of Matter from Left Kidney Pelvis, Open Approach */
"0TC60ZZ"  =  "1"  /* Extirpation of Matter from Right Ureter, Open Approach */
"0TC63ZZ"  =  "1"  /* Extirpation of Matter from Right Ureter, Percutaneous Approach */
"0TC64ZZ"  =  "1"  /* Extirpation of Matter from Right Ureter, Percutaneous Endoscopic Approach */
"0TC70ZZ"  =  "1"  /* Extirpation of Matter from Left Ureter, Open Approach */
"0TC73ZZ"  =  "1"  /* Extirpation of Matter from Left Ureter, Percutaneous Approach */
"0TC74ZZ"  =  "1"  /* Extirpation of Matter from Left Ureter, Percutaneous Endoscopic Approach */
"0TCB0ZZ"  =  "1"  /* Extirpation of Matter from Bladder, Open Approach */
"0TCB3ZZ"  =  "1"  /* Extirpation of Matter from Bladder, Percutaneous Approach */
"0TCB4ZZ"  =  "1"  /* Extirpation of Matter from Bladder, Percutaneous Endoscopic Approach */
"0TCC0ZZ"  =  "1"  /* Extirpation of Matter from Bladder Neck, Open Approach */
"0TCC3ZZ"  =  "1"  /* Extirpation of Matter from Bladder Neck, Percutaneous Approach */
"0TCC4ZZ"  =  "1"  /* Extirpation of Matter from Bladder Neck, Percutaneous Endoscopic Approach */
"0TCD0ZZ"  =  "1"  /* Extirpation of Matter from Urethra, Open Approach */
"0TCD3ZZ"  =  "1"  /* Extirpation of Matter from Urethra, Percutaneous Approach */
"0TCD4ZZ"  =  "1"  /* Extirpation of Matter from Urethra, Percutaneous Endoscopic Approach */
"0TH502Z"  =  "1"  /* Insertion of Monitoring Device into Kidney, Open Approach */
"0TH532Z"  =  "1"  /* Insertion of Monitoring Device into Kidney, Percutaneous Approach */
"0TH542Z"  =  "1"  /* Insertion of Monitoring Device into Kidney, Percutaneous Endoscopic Approach */
"0TH572Z"  =  "1"  /* Insertion of Monitoring Device into Kidney, Via Natural or Artificial Opening */
"0TH582Z"  =  "1"  /* Insertion of Monitoring Device into Kidney, Via Natural or Artificial Opening Endoscopic */
"0TH902Z"  =  "1"  /* Insertion of Monitoring Device into Ureter, Open Approach */
"0TH932Z"  =  "1"  /* Insertion of Monitoring Device into Ureter, Percutaneous Approach */
"0TH942Z"  =  "1"  /* Insertion of Monitoring Device into Ureter, Percutaneous Endoscopic Approach */
"0TH972Z"  =  "1"  /* Insertion of Monitoring Device into Ureter, Via Natural or Artificial Opening */
"0TH982Z"  =  "1"  /* Insertion of Monitoring Device into Ureter, Via Natural or Artificial Opening Endoscopic */
"0THB02Z"  =  "1"  /* Insertion of Monitoring Device into Bladder, Open Approach */
"0THB32Z"  =  "1"  /* Insertion of Monitoring Device into Bladder, Percutaneous Approach */
"0THB42Z"  =  "1"  /* Insertion of Monitoring Device into Bladder, Percutaneous Endoscopic Approach */
"0THB72Z"  =  "1"  /* Insertion of Monitoring Device into Bladder, Via Natural or Artificial Opening */
"0THB82Z"  =  "1"  /* Insertion of Monitoring Device into Bladder, Via Natural or Artificial Opening Endoscopic */
"0THD02Z"  =  "1"  /* Insertion of Monitoring Device into Urethra, Open Approach */
"0THD32Z"  =  "1"  /* Insertion of Monitoring Device into Urethra, Percutaneous Approach */
"0THD42Z"  =  "1"  /* Insertion of Monitoring Device into Urethra, Percutaneous Endoscopic Approach */
"0THD72Z"  =  "1"  /* Insertion of Monitoring Device into Urethra, Via Natural or Artificial Opening */
"0THD82Z"  =  "1"  /* Insertion of Monitoring Device into Urethra, Via Natural or Artificial Opening Endoscopic */
"0THDX2Z"  =  "1"  /* Insertion of Monitoring Device into Urethra, External Approach */
"0TJ50ZZ"  =  "1"  /* Inspection of Kidney, Open Approach */
"0TJ54ZZ"  =  "1"  /* Inspection of Kidney, Percutaneous Endoscopic Approach */
"0TJ58ZZ"  =  "1"  /* Inspection of Kidney, Via Natural or Artificial Opening Endoscopic */
"0TJ90ZZ"  =  "1"  /* Inspection of Ureter, Open Approach */
"0TJ94ZZ"  =  "1"  /* Inspection of Ureter, Percutaneous Endoscopic Approach */
"0TJ98ZZ"  =  "1"  /* Inspection of Ureter, Via Natural or Artificial Opening Endoscopic */
"0TJB0ZZ"  =  "1"  /* Inspection of Bladder, Open Approach */
"0TJB4ZZ"  =  "1"  /* Inspection of Bladder, Percutaneous Endoscopic Approach */
"0TJB8ZZ"  =  "1"  /* Inspection of Bladder, Via Natural or Artificial Opening Endoscopic */
"0TJD0ZZ"  =  "1"  /* Inspection of Urethra, Open Approach */
"0TJD7ZZ"  =  "1"  /* Inspection of Urethra, Via Natural or Artificial Opening */
"0TJD8ZZ"  =  "1"  /* Inspection of Urethra, Via Natural or Artificial Opening Endoscopic */
"0TP500Z"  =  "1"  /* Removal of Drainage Device from Kidney, Open Approach */
"0TP502Z"  =  "1"  /* Removal of Monitoring Device from Kidney, Open Approach */
"0TP503Z"  =  "1"  /* Removal of Infusion Device from Kidney, Open Approach */
"0TP507Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Kidney, Open Approach */
"0TP50CZ"  =  "1"  /* Removal of Extraluminal Device from Kidney, Open Approach */
"0TP50DZ"  =  "1"  /* Removal of Intraluminal Device from Kidney, Open Approach */
"0TP50JZ"  =  "1"  /* Removal of Synthetic Substitute from Kidney, Open Approach */
"0TP50KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Kidney, Open Approach */
"0TP530Z"  =  "1"  /* Removal of Drainage Device from Kidney, Percutaneous Approach */
"0TP532Z"  =  "1"  /* Removal of Monitoring Device from Kidney, Percutaneous Approach */
"0TP533Z"  =  "1"  /* Removal of Infusion Device from Kidney, Percutaneous Approach */
"0TP537Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Kidney, Percutaneous Approach */
"0TP53CZ"  =  "1"  /* Removal of Extraluminal Device from Kidney, Percutaneous Approach */
"0TP53DZ"  =  "1"  /* Removal of Intraluminal Device from Kidney, Percutaneous Approach */
"0TP53JZ"  =  "1"  /* Removal of Synthetic Substitute from Kidney, Percutaneous Approach */
"0TP53KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Kidney, Percutaneous Approach */
"0TP540Z"  =  "1"  /* Removal of Drainage Device from Kidney, Percutaneous Endoscopic Approach */
"0TP542Z"  =  "1"  /* Removal of Monitoring Device from Kidney, Percutaneous Endoscopic Approach */
"0TP543Z"  =  "1"  /* Removal of Infusion Device from Kidney, Percutaneous Endoscopic Approach */
"0TP547Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Kidney, Percutaneous Endoscopic Approach */
"0TP54CZ"  =  "1"  /* Removal of Extraluminal Device from Kidney, Percutaneous Endoscopic Approach */
"0TP54DZ"  =  "1"  /* Removal of Intraluminal Device from Kidney, Percutaneous Endoscopic Approach */
"0TP54JZ"  =  "1"  /* Removal of Synthetic Substitute from Kidney, Percutaneous Endoscopic Approach */
"0TP54KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Kidney, Percutaneous Endoscopic Approach */
"0TP570Z"  =  "1"  /* Removal of Drainage Device from Kidney, Via Natural or Artificial Opening */
"0TP572Z"  =  "1"  /* Removal of Monitoring Device from Kidney, Via Natural or Artificial Opening */
"0TP573Z"  =  "1"  /* Removal of Infusion Device from Kidney, Via Natural or Artificial Opening */
"0TP577Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Kidney, Via Natural or Artificial Opening */
"0TP57CZ"  =  "1"  /* Removal of Extraluminal Device from Kidney, Via Natural or Artificial Opening */
"0TP57DZ"  =  "1"  /* Removal of Intraluminal Device from Kidney, Via Natural or Artificial Opening */
"0TP57JZ"  =  "1"  /* Removal of Synthetic Substitute from Kidney, Via Natural or Artificial Opening */
"0TP57KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Kidney, Via Natural or Artificial Opening */
"0TP580Z"  =  "1"  /* Removal of Drainage Device from Kidney, Via Natural or Artificial Opening Endoscopic */
"0TP582Z"  =  "1"  /* Removal of Monitoring Device from Kidney, Via Natural or Artificial Opening Endoscopic */
"0TP583Z"  =  "1"  /* Removal of Infusion Device from Kidney, Via Natural or Artificial Opening Endoscopic */
"0TP587Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Kidney, Via Natural or Artificial Opening Endoscopic */
"0TP58CZ"  =  "1"  /* Removal of Extraluminal Device from Kidney, Via Natural or Artificial Opening Endoscopic */
"0TP58DZ"  =  "1"  /* Removal of Intraluminal Device from Kidney, Via Natural or Artificial Opening Endoscopic */
"0TP58JZ"  =  "1"  /* Removal of Synthetic Substitute from Kidney, Via Natural or Artificial Opening Endoscopic */
"0TP58KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Kidney, Via Natural or Artificial Opening Endoscopic */
"0TP900Z"  =  "1"  /* Removal of Drainage Device from Ureter, Open Approach */
"0TP902Z"  =  "1"  /* Removal of Monitoring Device from Ureter, Open Approach */
"0TP903Z"  =  "1"  /* Removal of Infusion Device from Ureter, Open Approach */
"0TP907Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Ureter, Open Approach */
"0TP90CZ"  =  "1"  /* Removal of Extraluminal Device from Ureter, Open Approach */
"0TP90DZ"  =  "1"  /* Removal of Intraluminal Device from Ureter, Open Approach */
"0TP90JZ"  =  "1"  /* Removal of Synthetic Substitute from Ureter, Open Approach */
"0TP90KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Ureter, Open Approach */
"0TP930Z"  =  "1"  /* Removal of Drainage Device from Ureter, Percutaneous Approach */
"0TP932Z"  =  "1"  /* Removal of Monitoring Device from Ureter, Percutaneous Approach */
"0TP933Z"  =  "1"  /* Removal of Infusion Device from Ureter, Percutaneous Approach */
"0TP937Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Ureter, Percutaneous Approach */
"0TP93CZ"  =  "1"  /* Removal of Extraluminal Device from Ureter, Percutaneous Approach */
"0TP93DZ"  =  "1"  /* Removal of Intraluminal Device from Ureter, Percutaneous Approach */
"0TP93JZ"  =  "1"  /* Removal of Synthetic Substitute from Ureter, Percutaneous Approach */
"0TP93KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Ureter, Percutaneous Approach */
"0TP940Z"  =  "1"  /* Removal of Drainage Device from Ureter, Percutaneous Endoscopic Approach */
"0TP942Z"  =  "1"  /* Removal of Monitoring Device from Ureter, Percutaneous Endoscopic Approach */
"0TP943Z"  =  "1"  /* Removal of Infusion Device from Ureter, Percutaneous Endoscopic Approach */
"0TP947Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Ureter, Percutaneous Endoscopic Approach */
"0TP94CZ"  =  "1"  /* Removal of Extraluminal Device from Ureter, Percutaneous Endoscopic Approach */
"0TP94DZ"  =  "1"  /* Removal of Intraluminal Device from Ureter, Percutaneous Endoscopic Approach */
"0TP94JZ"  =  "1"  /* Removal of Synthetic Substitute from Ureter, Percutaneous Endoscopic Approach */
"0TP94KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Ureter, Percutaneous Endoscopic Approach */
"0TP970Z"  =  "1"  /* Removal of Drainage Device from Ureter, Via Natural or Artificial Opening */
"0TP972Z"  =  "1"  /* Removal of Monitoring Device from Ureter, Via Natural or Artificial Opening */
"0TP973Z"  =  "1"  /* Removal of Infusion Device from Ureter, Via Natural or Artificial Opening */
"0TP977Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Ureter, Via Natural or Artificial Opening */
"0TP97CZ"  =  "1"  /* Removal of Extraluminal Device from Ureter, Via Natural or Artificial Opening */
"0TP97DZ"  =  "1"  /* Removal of Intraluminal Device from Ureter, Via Natural or Artificial Opening */
"0TP97JZ"  =  "1"  /* Removal of Synthetic Substitute from Ureter, Via Natural or Artificial Opening */
"0TP97KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Ureter, Via Natural or Artificial Opening */
"0TP980Z"  =  "1"  /* Removal of Drainage Device from Ureter, Via Natural or Artificial Opening Endoscopic */
"0TP982Z"  =  "1"  /* Removal of Monitoring Device from Ureter, Via Natural or Artificial Opening Endoscopic */
"0TP983Z"  =  "1"  /* Removal of Infusion Device from Ureter, Via Natural or Artificial Opening Endoscopic */
"0TP987Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Ureter, Via Natural or Artificial Opening Endoscopic */
"0TP98CZ"  =  "1"  /* Removal of Extraluminal Device from Ureter, Via Natural or Artificial Opening Endoscopic */
"0TP98DZ"  =  "1"  /* Removal of Intraluminal Device from Ureter, Via Natural or Artificial Opening Endoscopic */
"0TP98JZ"  =  "1"  /* Removal of Synthetic Substitute from Ureter, Via Natural or Artificial Opening Endoscopic */
"0TP98KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Ureter, Via Natural or Artificial Opening Endoscopic */
"0TPB00Z"  =  "1"  /* Removal of Drainage Device from Bladder, Open Approach */
"0TPB02Z"  =  "1"  /* Removal of Monitoring Device from Bladder, Open Approach */
"0TPB03Z"  =  "1"  /* Removal of Infusion Device from Bladder, Open Approach */
"0TPB07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Bladder, Open Approach */
"0TPB0CZ"  =  "1"  /* Removal of Extraluminal Device from Bladder, Open Approach */
"0TPB0DZ"  =  "1"  /* Removal of Intraluminal Device from Bladder, Open Approach */
"0TPB0JZ"  =  "1"  /* Removal of Synthetic Substitute from Bladder, Open Approach */
"0TPB0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Bladder, Open Approach */
"0TPB0LZ"  =  "1"  /* Removal of Artificial Sphincter from Bladder, Open Approach */
"0TPB30Z"  =  "1"  /* Removal of Drainage Device from Bladder, Percutaneous Approach */
"0TPB32Z"  =  "1"  /* Removal of Monitoring Device from Bladder, Percutaneous Approach */
"0TPB33Z"  =  "1"  /* Removal of Infusion Device from Bladder, Percutaneous Approach */
"0TPB37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Bladder, Percutaneous Approach */
"0TPB3CZ"  =  "1"  /* Removal of Extraluminal Device from Bladder, Percutaneous Approach */
"0TPB3DZ"  =  "1"  /* Removal of Intraluminal Device from Bladder, Percutaneous Approach */
"0TPB3JZ"  =  "1"  /* Removal of Synthetic Substitute from Bladder, Percutaneous Approach */
"0TPB3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Bladder, Percutaneous Approach */
"0TPB3LZ"  =  "1"  /* Removal of Artificial Sphincter from Bladder, Percutaneous Approach */
"0TPB40Z"  =  "1"  /* Removal of Drainage Device from Bladder, Percutaneous Endoscopic Approach */
"0TPB42Z"  =  "1"  /* Removal of Monitoring Device from Bladder, Percutaneous Endoscopic Approach */
"0TPB43Z"  =  "1"  /* Removal of Infusion Device from Bladder, Percutaneous Endoscopic Approach */
"0TPB47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Bladder, Percutaneous Endoscopic Approach */
"0TPB4CZ"  =  "1"  /* Removal of Extraluminal Device from Bladder, Percutaneous Endoscopic Approach */
"0TPB4DZ"  =  "1"  /* Removal of Intraluminal Device from Bladder, Percutaneous Endoscopic Approach */
"0TPB4JZ"  =  "1"  /* Removal of Synthetic Substitute from Bladder, Percutaneous Endoscopic Approach */
"0TPB4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Bladder, Percutaneous Endoscopic Approach */
"0TPB4LZ"  =  "1"  /* Removal of Artificial Sphincter from Bladder, Percutaneous Endoscopic Approach */
"0TPB70Z"  =  "1"  /* Removal of Drainage Device from Bladder, Via Natural or Artificial Opening */
"0TPB72Z"  =  "1"  /* Removal of Monitoring Device from Bladder, Via Natural or Artificial Opening */
"0TPB73Z"  =  "1"  /* Removal of Infusion Device from Bladder, Via Natural or Artificial Opening */
"0TPB77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Bladder, Via Natural or Artificial Opening */
"0TPB7CZ"  =  "1"  /* Removal of Extraluminal Device from Bladder, Via Natural or Artificial Opening */
"0TPB7DZ"  =  "1"  /* Removal of Intraluminal Device from Bladder, Via Natural or Artificial Opening */
"0TPB7JZ"  =  "1"  /* Removal of Synthetic Substitute from Bladder, Via Natural or Artificial Opening */
"0TPB7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Bladder, Via Natural or Artificial Opening */
"0TPB7LZ"  =  "1"  /* Removal of Artificial Sphincter from Bladder, Via Natural or Artificial Opening */
"0TPB80Z"  =  "1"  /* Removal of Drainage Device from Bladder, Via Natural or Artificial Opening Endoscopic */
"0TPB82Z"  =  "1"  /* Removal of Monitoring Device from Bladder, Via Natural or Artificial Opening Endoscopic */
"0TPB83Z"  =  "1"  /* Removal of Infusion Device from Bladder, Via Natural or Artificial Opening Endoscopic */
"0TPB87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Bladder, Via Natural or Artificial Opening Endoscopic */
"0TPB8CZ"  =  "1"  /* Removal of Extraluminal Device from Bladder, Via Natural or Artificial Opening Endoscopic */
"0TPB8DZ"  =  "1"  /* Removal of Intraluminal Device from Bladder, Via Natural or Artificial Opening Endoscopic */
"0TPB8JZ"  =  "1"  /* Removal of Synthetic Substitute from Bladder, Via Natural or Artificial Opening Endoscopic */
"0TPB8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Bladder, Via Natural or Artificial Opening Endoscopic */
"0TPB8LZ"  =  "1"  /* Removal of Artificial Sphincter from Bladder, Via Natural or Artificial Opening Endoscopic */
"0TPD00Z"  =  "1"  /* Removal of Drainage Device from Urethra, Open Approach */
"0TPD02Z"  =  "1"  /* Removal of Monitoring Device from Urethra, Open Approach */
"0TPD03Z"  =  "1"  /* Removal of Infusion Device from Urethra, Open Approach */
"0TPD07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Urethra, Open Approach */
"0TPD0CZ"  =  "1"  /* Removal of Extraluminal Device from Urethra, Open Approach */
"0TPD0DZ"  =  "1"  /* Removal of Intraluminal Device from Urethra, Open Approach */
"0TPD0JZ"  =  "1"  /* Removal of Synthetic Substitute from Urethra, Open Approach */
"0TPD0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Urethra, Open Approach */
"0TPD30Z"  =  "1"  /* Removal of Drainage Device from Urethra, Percutaneous Approach */
"0TPD32Z"  =  "1"  /* Removal of Monitoring Device from Urethra, Percutaneous Approach */
"0TPD33Z"  =  "1"  /* Removal of Infusion Device from Urethra, Percutaneous Approach */
"0TPD37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Urethra, Percutaneous Approach */
"0TPD3CZ"  =  "1"  /* Removal of Extraluminal Device from Urethra, Percutaneous Approach */
"0TPD3DZ"  =  "1"  /* Removal of Intraluminal Device from Urethra, Percutaneous Approach */
"0TPD3JZ"  =  "1"  /* Removal of Synthetic Substitute from Urethra, Percutaneous Approach */
"0TPD3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Urethra, Percutaneous Approach */
"0TPD40Z"  =  "1"  /* Removal of Drainage Device from Urethra, Percutaneous Endoscopic Approach */
"0TPD42Z"  =  "1"  /* Removal of Monitoring Device from Urethra, Percutaneous Endoscopic Approach */
"0TPD43Z"  =  "1"  /* Removal of Infusion Device from Urethra, Percutaneous Endoscopic Approach */
"0TPD47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Urethra, Percutaneous Endoscopic Approach */
"0TPD4CZ"  =  "1"  /* Removal of Extraluminal Device from Urethra, Percutaneous Endoscopic Approach */
"0TPD4DZ"  =  "1"  /* Removal of Intraluminal Device from Urethra, Percutaneous Endoscopic Approach */
"0TPD4JZ"  =  "1"  /* Removal of Synthetic Substitute from Urethra, Percutaneous Endoscopic Approach */
"0TPD4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Urethra, Percutaneous Endoscopic Approach */
"0TPD70Z"  =  "1"  /* Removal of Drainage Device from Urethra, Via Natural or Artificial Opening */
"0TPD72Z"  =  "1"  /* Removal of Monitoring Device from Urethra, Via Natural or Artificial Opening */
"0TPD73Z"  =  "1"  /* Removal of Infusion Device from Urethra, Via Natural or Artificial Opening */
"0TPD77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Urethra, Via Natural or Artificial Opening */
"0TPD7CZ"  =  "1"  /* Removal of Extraluminal Device from Urethra, Via Natural or Artificial Opening */
"0TPD7DZ"  =  "1"  /* Removal of Intraluminal Device from Urethra, Via Natural or Artificial Opening */
"0TPD7JZ"  =  "1"  /* Removal of Synthetic Substitute from Urethra, Via Natural or Artificial Opening */
"0TPD7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Urethra, Via Natural or Artificial Opening */
"0TPD80Z"  =  "1"  /* Removal of Drainage Device from Urethra, Via Natural or Artificial Opening Endoscopic */
"0TPD82Z"  =  "1"  /* Removal of Monitoring Device from Urethra, Via Natural or Artificial Opening Endoscopic */
"0TPD83Z"  =  "1"  /* Removal of Infusion Device from Urethra, Via Natural or Artificial Opening Endoscopic */
"0TPD87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Urethra, Via Natural or Artificial Opening Endoscopic */
"0TPD8CZ"  =  "1"  /* Removal of Extraluminal Device from Urethra, Via Natural or Artificial Opening Endoscopic */
"0TPD8DZ"  =  "1"  /* Removal of Intraluminal Device from Urethra, Via Natural or Artificial Opening Endoscopic */
"0TPD8JZ"  =  "1"  /* Removal of Synthetic Substitute from Urethra, Via Natural or Artificial Opening Endoscopic */
"0TPD8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Urethra, Via Natural or Artificial Opening Endoscopic */
"0TQB0ZZ"  =  "1"  /* Repair Bladder, Open Approach */
"0TQB3ZZ"  =  "1"  /* Repair Bladder, Percutaneous Approach */
"0TQB4ZZ"  =  "1"  /* Repair Bladder, Percutaneous Endoscopic Approach */
"0TQB7ZZ"  =  "1"  /* Repair Bladder, Via Natural or Artificial Opening */
"0TQB8ZZ"  =  "1"  /* Repair Bladder, Via Natural or Artificial Opening Endoscopic */
"0TW500Z"  =  "1"  /* Revision of Drainage Device in Kidney, Open Approach */
"0TW502Z"  =  "1"  /* Revision of Monitoring Device in Kidney, Open Approach */
"0TW503Z"  =  "1"  /* Revision of Infusion Device in Kidney, Open Approach */
"0TW507Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Kidney, Open Approach */
"0TW50CZ"  =  "1"  /* Revision of Extraluminal Device in Kidney, Open Approach */
"0TW50DZ"  =  "1"  /* Revision of Intraluminal Device in Kidney, Open Approach */
"0TW50JZ"  =  "1"  /* Revision of Synthetic Substitute in Kidney, Open Approach */
"0TW50KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Kidney, Open Approach */
"0TW530Z"  =  "1"  /* Revision of Drainage Device in Kidney, Percutaneous Approach */
"0TW532Z"  =  "1"  /* Revision of Monitoring Device in Kidney, Percutaneous Approach */
"0TW533Z"  =  "1"  /* Revision of Infusion Device in Kidney, Percutaneous Approach */
"0TW537Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Kidney, Percutaneous Approach */
"0TW53CZ"  =  "1"  /* Revision of Extraluminal Device in Kidney, Percutaneous Approach */
"0TW53DZ"  =  "1"  /* Revision of Intraluminal Device in Kidney, Percutaneous Approach */
"0TW53JZ"  =  "1"  /* Revision of Synthetic Substitute in Kidney, Percutaneous Approach */
"0TW53KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Kidney, Percutaneous Approach */
"0TW540Z"  =  "1"  /* Revision of Drainage Device in Kidney, Percutaneous Endoscopic Approach */
"0TW542Z"  =  "1"  /* Revision of Monitoring Device in Kidney, Percutaneous Endoscopic Approach */
"0TW543Z"  =  "1"  /* Revision of Infusion Device in Kidney, Percutaneous Endoscopic Approach */
"0TW547Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Kidney, Percutaneous Endoscopic Approach */
"0TW54CZ"  =  "1"  /* Revision of Extraluminal Device in Kidney, Percutaneous Endoscopic Approach */
"0TW54DZ"  =  "1"  /* Revision of Intraluminal Device in Kidney, Percutaneous Endoscopic Approach */
"0TW54JZ"  =  "1"  /* Revision of Synthetic Substitute in Kidney, Percutaneous Endoscopic Approach */
"0TW54KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Kidney, Percutaneous Endoscopic Approach */
"0TW570Z"  =  "1"  /* Revision of Drainage Device in Kidney, Via Natural or Artificial Opening */
"0TW572Z"  =  "1"  /* Revision of Monitoring Device in Kidney, Via Natural or Artificial Opening */
"0TW573Z"  =  "1"  /* Revision of Infusion Device in Kidney, Via Natural or Artificial Opening */
"0TW577Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Kidney, Via Natural or Artificial Opening */
"0TW57CZ"  =  "1"  /* Revision of Extraluminal Device in Kidney, Via Natural or Artificial Opening */
"0TW57DZ"  =  "1"  /* Revision of Intraluminal Device in Kidney, Via Natural or Artificial Opening */
"0TW57JZ"  =  "1"  /* Revision of Synthetic Substitute in Kidney, Via Natural or Artificial Opening */
"0TW57KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Kidney, Via Natural or Artificial Opening */
"0TW580Z"  =  "1"  /* Revision of Drainage Device in Kidney, Via Natural or Artificial Opening Endoscopic */
"0TW582Z"  =  "1"  /* Revision of Monitoring Device in Kidney, Via Natural or Artificial Opening Endoscopic */
"0TW583Z"  =  "1"  /* Revision of Infusion Device in Kidney, Via Natural or Artificial Opening Endoscopic */
"0TW587Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Kidney, Via Natural or Artificial Opening Endoscopic */
"0TW58CZ"  =  "1"  /* Revision of Extraluminal Device in Kidney, Via Natural or Artificial Opening Endoscopic */
"0TW58DZ"  =  "1"  /* Revision of Intraluminal Device in Kidney, Via Natural or Artificial Opening Endoscopic */
"0TW58JZ"  =  "1"  /* Revision of Synthetic Substitute in Kidney, Via Natural or Artificial Opening Endoscopic */
"0TW58KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Kidney, Via Natural or Artificial Opening Endoscopic */
"0TW900Z"  =  "1"  /* Revision of Drainage Device in Ureter, Open Approach */
"0TW902Z"  =  "1"  /* Revision of Monitoring Device in Ureter, Open Approach */
"0TW903Z"  =  "1"  /* Revision of Infusion Device in Ureter, Open Approach */
"0TW907Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Ureter, Open Approach */
"0TW90CZ"  =  "1"  /* Revision of Extraluminal Device in Ureter, Open Approach */
"0TW90DZ"  =  "1"  /* Revision of Intraluminal Device in Ureter, Open Approach */
"0TW90JZ"  =  "1"  /* Revision of Synthetic Substitute in Ureter, Open Approach */
"0TW90KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Ureter, Open Approach */
"0TW90MZ"  =  "1"  /* Revision of Stimulator Lead in Ureter, Open Approach */
"0TW930Z"  =  "1"  /* Revision of Drainage Device in Ureter, Percutaneous Approach */
"0TW932Z"  =  "1"  /* Revision of Monitoring Device in Ureter, Percutaneous Approach */
"0TW933Z"  =  "1"  /* Revision of Infusion Device in Ureter, Percutaneous Approach */
"0TW937Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Ureter, Percutaneous Approach */
"0TW93CZ"  =  "1"  /* Revision of Extraluminal Device in Ureter, Percutaneous Approach */
"0TW93DZ"  =  "1"  /* Revision of Intraluminal Device in Ureter, Percutaneous Approach */
"0TW93JZ"  =  "1"  /* Revision of Synthetic Substitute in Ureter, Percutaneous Approach */
"0TW93KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Ureter, Percutaneous Approach */
"0TW93MZ"  =  "1"  /* Revision of Stimulator Lead in Ureter, Percutaneous Approach */
"0TW940Z"  =  "1"  /* Revision of Drainage Device in Ureter, Percutaneous Endoscopic Approach */
"0TW942Z"  =  "1"  /* Revision of Monitoring Device in Ureter, Percutaneous Endoscopic Approach */
"0TW943Z"  =  "1"  /* Revision of Infusion Device in Ureter, Percutaneous Endoscopic Approach */
"0TW947Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Ureter, Percutaneous Endoscopic Approach */
"0TW94CZ"  =  "1"  /* Revision of Extraluminal Device in Ureter, Percutaneous Endoscopic Approach */
"0TW94DZ"  =  "1"  /* Revision of Intraluminal Device in Ureter, Percutaneous Endoscopic Approach */
"0TW94JZ"  =  "1"  /* Revision of Synthetic Substitute in Ureter, Percutaneous Endoscopic Approach */
"0TW94KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Ureter, Percutaneous Endoscopic Approach */
"0TW94MZ"  =  "1"  /* Revision of Stimulator Lead in Ureter, Percutaneous Endoscopic Approach */
"0TW970Z"  =  "1"  /* Revision of Drainage Device in Ureter, Via Natural or Artificial Opening */
"0TW972Z"  =  "1"  /* Revision of Monitoring Device in Ureter, Via Natural or Artificial Opening */
"0TW973Z"  =  "1"  /* Revision of Infusion Device in Ureter, Via Natural or Artificial Opening */
"0TW977Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Ureter, Via Natural or Artificial Opening */
"0TW97CZ"  =  "1"  /* Revision of Extraluminal Device in Ureter, Via Natural or Artificial Opening */
"0TW97DZ"  =  "1"  /* Revision of Intraluminal Device in Ureter, Via Natural or Artificial Opening */
"0TW97JZ"  =  "1"  /* Revision of Synthetic Substitute in Ureter, Via Natural or Artificial Opening */
"0TW97KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Ureter, Via Natural or Artificial Opening */
"0TW97MZ"  =  "1"  /* Revision of Stimulator Lead in Ureter, Via Natural or Artificial Opening */
"0TW980Z"  =  "1"  /* Revision of Drainage Device in Ureter, Via Natural or Artificial Opening Endoscopic */
"0TW982Z"  =  "1"  /* Revision of Monitoring Device in Ureter, Via Natural or Artificial Opening Endoscopic */
"0TW983Z"  =  "1"  /* Revision of Infusion Device in Ureter, Via Natural or Artificial Opening Endoscopic */
"0TW987Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Ureter, Via Natural or Artificial Opening Endoscopic */
"0TW98CZ"  =  "1"  /* Revision of Extraluminal Device in Ureter, Via Natural or Artificial Opening Endoscopic */
"0TW98DZ"  =  "1"  /* Revision of Intraluminal Device in Ureter, Via Natural or Artificial Opening Endoscopic */
"0TW98JZ"  =  "1"  /* Revision of Synthetic Substitute in Ureter, Via Natural or Artificial Opening Endoscopic */
"0TW98KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Ureter, Via Natural or Artificial Opening Endoscopic */
"0TW98MZ"  =  "1"  /* Revision of Stimulator Lead in Ureter, Via Natural or Artificial Opening Endoscopic */
"0TWB00Z"  =  "1"  /* Revision of Drainage Device in Bladder, Open Approach */
"0TWB02Z"  =  "1"  /* Revision of Monitoring Device in Bladder, Open Approach */
"0TWB03Z"  =  "1"  /* Revision of Infusion Device in Bladder, Open Approach */
"0TWB07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Bladder, Open Approach */
"0TWB0CZ"  =  "1"  /* Revision of Extraluminal Device in Bladder, Open Approach */
"0TWB0DZ"  =  "1"  /* Revision of Intraluminal Device in Bladder, Open Approach */
"0TWB0JZ"  =  "1"  /* Revision of Synthetic Substitute in Bladder, Open Approach */
"0TWB0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Bladder, Open Approach */
"0TWB0LZ"  =  "1"  /* Revision of Artificial Sphincter in Bladder, Open Approach */
"0TWB0MZ"  =  "1"  /* Revision of Stimulator Lead in Bladder, Open Approach */
"0TWB30Z"  =  "1"  /* Revision of Drainage Device in Bladder, Percutaneous Approach */
"0TWB32Z"  =  "1"  /* Revision of Monitoring Device in Bladder, Percutaneous Approach */
"0TWB33Z"  =  "1"  /* Revision of Infusion Device in Bladder, Percutaneous Approach */
"0TWB37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Bladder, Percutaneous Approach */
"0TWB3CZ"  =  "1"  /* Revision of Extraluminal Device in Bladder, Percutaneous Approach */
"0TWB3DZ"  =  "1"  /* Revision of Intraluminal Device in Bladder, Percutaneous Approach */
"0TWB3JZ"  =  "1"  /* Revision of Synthetic Substitute in Bladder, Percutaneous Approach */
"0TWB3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Bladder, Percutaneous Approach */
"0TWB3LZ"  =  "1"  /* Revision of Artificial Sphincter in Bladder, Percutaneous Approach */
"0TWB3MZ"  =  "1"  /* Revision of Stimulator Lead in Bladder, Percutaneous Approach */
"0TWB40Z"  =  "1"  /* Revision of Drainage Device in Bladder, Percutaneous Endoscopic Approach */
"0TWB42Z"  =  "1"  /* Revision of Monitoring Device in Bladder, Percutaneous Endoscopic Approach */
"0TWB43Z"  =  "1"  /* Revision of Infusion Device in Bladder, Percutaneous Endoscopic Approach */
"0TWB47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Bladder, Percutaneous Endoscopic Approach */
"0TWB4CZ"  =  "1"  /* Revision of Extraluminal Device in Bladder, Percutaneous Endoscopic Approach */
"0TWB4DZ"  =  "1"  /* Revision of Intraluminal Device in Bladder, Percutaneous Endoscopic Approach */
"0TWB4JZ"  =  "1"  /* Revision of Synthetic Substitute in Bladder, Percutaneous Endoscopic Approach */
"0TWB4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Bladder, Percutaneous Endoscopic Approach */
"0TWB4LZ"  =  "1"  /* Revision of Artificial Sphincter in Bladder, Percutaneous Endoscopic Approach */
"0TWB4MZ"  =  "1"  /* Revision of Stimulator Lead in Bladder, Percutaneous Endoscopic Approach */
"0TWB70Z"  =  "1"  /* Revision of Drainage Device in Bladder, Via Natural or Artificial Opening */
"0TWB72Z"  =  "1"  /* Revision of Monitoring Device in Bladder, Via Natural or Artificial Opening */
"0TWB73Z"  =  "1"  /* Revision of Infusion Device in Bladder, Via Natural or Artificial Opening */
"0TWB77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Bladder, Via Natural or Artificial Opening */
"0TWB7CZ"  =  "1"  /* Revision of Extraluminal Device in Bladder, Via Natural or Artificial Opening */
"0TWB7DZ"  =  "1"  /* Revision of Intraluminal Device in Bladder, Via Natural or Artificial Opening */
"0TWB7JZ"  =  "1"  /* Revision of Synthetic Substitute in Bladder, Via Natural or Artificial Opening */
"0TWB7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Bladder, Via Natural or Artificial Opening */
"0TWB7LZ"  =  "1"  /* Revision of Artificial Sphincter in Bladder, Via Natural or Artificial Opening */
"0TWB7MZ"  =  "1"  /* Revision of Stimulator Lead in Bladder, Via Natural or Artificial Opening */
"0TWB80Z"  =  "1"  /* Revision of Drainage Device in Bladder, Via Natural or Artificial Opening Endoscopic */
"0TWB82Z"  =  "1"  /* Revision of Monitoring Device in Bladder, Via Natural or Artificial Opening Endoscopic */
"0TWB83Z"  =  "1"  /* Revision of Infusion Device in Bladder, Via Natural or Artificial Opening Endoscopic */
"0TWB87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Bladder, Via Natural or Artificial Opening Endoscopic */
"0TWB8CZ"  =  "1"  /* Revision of Extraluminal Device in Bladder, Via Natural or Artificial Opening Endoscopic */
"0TWB8DZ"  =  "1"  /* Revision of Intraluminal Device in Bladder, Via Natural or Artificial Opening Endoscopic */
"0TWB8JZ"  =  "1"  /* Revision of Synthetic Substitute in Bladder, Via Natural or Artificial Opening Endoscopic */
"0TWB8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Bladder, Via Natural or Artificial Opening Endoscopic */
"0TWB8LZ"  =  "1"  /* Revision of Artificial Sphincter in Bladder, Via Natural or Artificial Opening Endoscopic */
"0TWB8MZ"  =  "1"  /* Revision of Stimulator Lead in Bladder, Via Natural or Artificial Opening Endoscopic */
"0TWD00Z"  =  "1"  /* Revision of Drainage Device in Urethra, Open Approach */
"0TWD02Z"  =  "1"  /* Revision of Monitoring Device in Urethra, Open Approach */
"0TWD03Z"  =  "1"  /* Revision of Infusion Device in Urethra, Open Approach */
"0TWD07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Urethra, Open Approach */
"0TWD0CZ"  =  "1"  /* Revision of Extraluminal Device in Urethra, Open Approach */
"0TWD0DZ"  =  "1"  /* Revision of Intraluminal Device in Urethra, Open Approach */
"0TWD0JZ"  =  "1"  /* Revision of Synthetic Substitute in Urethra, Open Approach */
"0TWD0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Urethra, Open Approach */
"0TWD30Z"  =  "1"  /* Revision of Drainage Device in Urethra, Percutaneous Approach */
"0TWD32Z"  =  "1"  /* Revision of Monitoring Device in Urethra, Percutaneous Approach */
"0TWD33Z"  =  "1"  /* Revision of Infusion Device in Urethra, Percutaneous Approach */
"0TWD37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Urethra, Percutaneous Approach */
"0TWD3CZ"  =  "1"  /* Revision of Extraluminal Device in Urethra, Percutaneous Approach */
"0TWD3DZ"  =  "1"  /* Revision of Intraluminal Device in Urethra, Percutaneous Approach */
"0TWD3JZ"  =  "1"  /* Revision of Synthetic Substitute in Urethra, Percutaneous Approach */
"0TWD3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Urethra, Percutaneous Approach */
"0TWD40Z"  =  "1"  /* Revision of Drainage Device in Urethra, Percutaneous Endoscopic Approach */
"0TWD42Z"  =  "1"  /* Revision of Monitoring Device in Urethra, Percutaneous Endoscopic Approach */
"0TWD43Z"  =  "1"  /* Revision of Infusion Device in Urethra, Percutaneous Endoscopic Approach */
"0TWD47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Urethra, Percutaneous Endoscopic Approach */
"0TWD4CZ"  =  "1"  /* Revision of Extraluminal Device in Urethra, Percutaneous Endoscopic Approach */
"0TWD4DZ"  =  "1"  /* Revision of Intraluminal Device in Urethra, Percutaneous Endoscopic Approach */
"0TWD4JZ"  =  "1"  /* Revision of Synthetic Substitute in Urethra, Percutaneous Endoscopic Approach */
"0TWD4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Urethra, Percutaneous Endoscopic Approach */
"0TWD70Z"  =  "1"  /* Revision of Drainage Device in Urethra, Via Natural or Artificial Opening */
"0TWD72Z"  =  "1"  /* Revision of Monitoring Device in Urethra, Via Natural or Artificial Opening */
"0TWD73Z"  =  "1"  /* Revision of Infusion Device in Urethra, Via Natural or Artificial Opening */
"0TWD77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Urethra, Via Natural or Artificial Opening */
"0TWD7CZ"  =  "1"  /* Revision of Extraluminal Device in Urethra, Via Natural or Artificial Opening */
"0TWD7DZ"  =  "1"  /* Revision of Intraluminal Device in Urethra, Via Natural or Artificial Opening */
"0TWD7JZ"  =  "1"  /* Revision of Synthetic Substitute in Urethra, Via Natural or Artificial Opening */
"0TWD7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Urethra, Via Natural or Artificial Opening */
"0TWD80Z"  =  "1"  /* Revision of Drainage Device in Urethra, Via Natural or Artificial Opening Endoscopic */
"0TWD82Z"  =  "1"  /* Revision of Monitoring Device in Urethra, Via Natural or Artificial Opening Endoscopic */
"0TWD83Z"  =  "1"  /* Revision of Infusion Device in Urethra, Via Natural or Artificial Opening Endoscopic */
"0TWD87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Urethra, Via Natural or Artificial Opening Endoscopic */
"0TWD8CZ"  =  "1"  /* Revision of Extraluminal Device in Urethra, Via Natural or Artificial Opening Endoscopic */
"0TWD8DZ"  =  "1"  /* Revision of Intraluminal Device in Urethra, Via Natural or Artificial Opening Endoscopic */
"0TWD8JZ"  =  "1"  /* Revision of Synthetic Substitute in Urethra, Via Natural or Artificial Opening Endoscopic */
"0TWD8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Urethra, Via Natural or Artificial Opening Endoscopic */
"0U840ZZ"  =  "1"  /* Division of Uterine Supporting Structure, Open Approach */
"0U843ZZ"  =  "1"  /* Division of Uterine Supporting Structure, Percutaneous Approach */
"0U844ZZ"  =  "1"  /* Division of Uterine Supporting Structure, Percutaneous Endoscopic Approach */
"0U9000Z"  =  "1"  /* Drainage of Right Ovary with Drainage Device, Open Approach */
"0U900ZZ"  =  "1"  /* Drainage of Right Ovary, Open Approach */
"0U9040Z"  =  "1"  /* Drainage of Right Ovary with Drainage Device, Percutaneous Endoscopic Approach */
"0U904ZZ"  =  "1"  /* Drainage of Right Ovary, Percutaneous Endoscopic Approach */
"0U9100Z"  =  "1"  /* Drainage of Left Ovary with Drainage Device, Open Approach */
"0U910ZZ"  =  "1"  /* Drainage of Left Ovary, Open Approach */
"0U9140Z"  =  "1"  /* Drainage of Left Ovary with Drainage Device, Percutaneous Endoscopic Approach */
"0U914ZZ"  =  "1"  /* Drainage of Left Ovary, Percutaneous Endoscopic Approach */
"0U9200Z"  =  "1"  /* Drainage of Bilateral Ovaries with Drainage Device, Open Approach */
"0U920ZZ"  =  "1"  /* Drainage of Bilateral Ovaries, Open Approach */
"0U9240Z"  =  "1"  /* Drainage of Bilateral Ovaries with Drainage Device, Percutaneous Endoscopic Approach */
"0U924ZZ"  =  "1"  /* Drainage of Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0U9400Z"  =  "1"  /* Drainage of Uterine Supporting Structure with Drainage Device, Open Approach */
"0U940ZZ"  =  "1"  /* Drainage of Uterine Supporting Structure, Open Approach */
"0U9430Z"  =  "1"  /* Drainage of Uterine Supporting Structure with Drainage Device, Percutaneous Approach */
"0U943ZZ"  =  "1"  /* Drainage of Uterine Supporting Structure, Percutaneous Approach */
"0U9440Z"  =  "1"  /* Drainage of Uterine Supporting Structure with Drainage Device, Percutaneous Endoscopic Approach */
"0U944ZZ"  =  "1"  /* Drainage of Uterine Supporting Structure, Percutaneous Endoscopic Approach */
"0U950ZZ"  =  "1"  /* Drainage of Right Fallopian Tube, Open Approach */
"0U960ZZ"  =  "1"  /* Drainage of Left Fallopian Tube, Open Approach */
"0U970ZZ"  =  "1"  /* Drainage of Bilateral Fallopian Tubes, Open Approach */
"0U9900Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Open Approach */
"0U990ZZ"  =  "1"  /* Drainage of Uterus, Open Approach */
"0U9930Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Percutaneous Approach */
"0U993ZZ"  =  "1"  /* Drainage of Uterus, Percutaneous Approach */
"0U9940Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Percutaneous Endoscopic Approach */
"0U994ZZ"  =  "1"  /* Drainage of Uterus, Percutaneous Endoscopic Approach */
"0U9970Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Via Natural or Artificial Opening */
"0U997ZZ"  =  "1"  /* Drainage of Uterus, Via Natural or Artificial Opening */
"0U9980Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0U998ZZ"  =  "1"  /* Drainage of Uterus, Via Natural or Artificial Opening Endoscopic */
"0U9C00Z"  =  "1"  /* Drainage of Cervix with Drainage Device, Open Approach */
"0U9C0ZZ"  =  "1"  /* Drainage of Cervix, Open Approach */
"0U9C30Z"  =  "1"  /* Drainage of Cervix with Drainage Device, Percutaneous Approach */
"0U9C3ZZ"  =  "1"  /* Drainage of Cervix, Percutaneous Approach */
"0U9C40Z"  =  "1"  /* Drainage of Cervix with Drainage Device, Percutaneous Endoscopic Approach */
"0U9C4ZZ"  =  "1"  /* Drainage of Cervix, Percutaneous Endoscopic Approach */
"0U9C70Z"  =  "1"  /* Drainage of Cervix with Drainage Device, Via Natural or Artificial Opening */
"0U9C7ZZ"  =  "1"  /* Drainage of Cervix, Via Natural or Artificial Opening */
"0U9C80Z"  =  "1"  /* Drainage of Cervix with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0U9C8ZZ"  =  "1"  /* Drainage of Cervix, Via Natural or Artificial Opening Endoscopic */
"0U9F00Z"  =  "1"  /* Drainage of Cul-de-sac with Drainage Device, Open Approach */
"0U9F0ZZ"  =  "1"  /* Drainage of Cul-de-sac, Open Approach */
"0U9F70Z"  =  "1"  /* Drainage of Cul-de-sac with Drainage Device, Via Natural or Artificial Opening */
"0U9F7ZZ"  =  "1"  /* Drainage of Cul-de-sac, Via Natural or Artificial Opening */
"0U9F80Z"  =  "1"  /* Drainage of Cul-de-sac with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0U9F8ZZ"  =  "1"  /* Drainage of Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0U9G00Z"  =  "1"  /* Drainage of Vagina with Drainage Device, Open Approach */
"0U9G0ZZ"  =  "1"  /* Drainage of Vagina, Open Approach */
"0U9G30Z"  =  "1"  /* Drainage of Vagina with Drainage Device, Percutaneous Approach */
"0U9G3ZZ"  =  "1"  /* Drainage of Vagina, Percutaneous Approach */
"0U9G40Z"  =  "1"  /* Drainage of Vagina with Drainage Device, Percutaneous Endoscopic Approach */
"0U9G4ZZ"  =  "1"  /* Drainage of Vagina, Percutaneous Endoscopic Approach */
"0U9G70Z"  =  "1"  /* Drainage of Vagina with Drainage Device, Via Natural or Artificial Opening */
"0U9G7ZZ"  =  "1"  /* Drainage of Vagina, Via Natural or Artificial Opening */
"0U9G80Z"  =  "1"  /* Drainage of Vagina with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0U9G8ZZ"  =  "1"  /* Drainage of Vagina, Via Natural or Artificial Opening Endoscopic */
"0U9GX0Z"  =  "1"  /* Drainage of Vagina with Drainage Device, External Approach */
"0U9GXZZ"  =  "1"  /* Drainage of Vagina, External Approach */
"0U9M00Z"  =  "1"  /* Drainage of Vulva with Drainage Device, Open Approach */
"0U9M0ZZ"  =  "1"  /* Drainage of Vulva, Open Approach */
"0U9MX0Z"  =  "1"  /* Drainage of Vulva with Drainage Device, External Approach */
"0U9MXZZ"  =  "1"  /* Drainage of Vulva, External Approach */
"0UC00ZZ"  =  "1"  /* Extirpation of Matter from Right Ovary, Open Approach */
"0UC03ZZ"  =  "1"  /* Extirpation of Matter from Right Ovary, Percutaneous Approach */
"0UC04ZZ"  =  "1"  /* Extirpation of Matter from Right Ovary, Percutaneous Endoscopic Approach */
"0UC10ZZ"  =  "1"  /* Extirpation of Matter from Left Ovary, Open Approach */
"0UC13ZZ"  =  "1"  /* Extirpation of Matter from Left Ovary, Percutaneous Approach */
"0UC14ZZ"  =  "1"  /* Extirpation of Matter from Left Ovary, Percutaneous Endoscopic Approach */
"0UC20ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Ovaries, Open Approach */
"0UC23ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Ovaries, Percutaneous Approach */
"0UC24ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0UC40ZZ"  =  "1"  /* Extirpation of Matter from Uterine Supporting Structure, Open Approach */
"0UC43ZZ"  =  "1"  /* Extirpation of Matter from Uterine Supporting Structure, Percutaneous Approach */
"0UC44ZZ"  =  "1"  /* Extirpation of Matter from Uterine Supporting Structure, Percutaneous Endoscopic Approach */
"0UC50ZZ"  =  "1"  /* Extirpation of Matter from Right Fallopian Tube, Open Approach */
"0UC53ZZ"  =  "1"  /* Extirpation of Matter from Right Fallopian Tube, Percutaneous Approach */
"0UC54ZZ"  =  "1"  /* Extirpation of Matter from Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0UC57ZZ"  =  "1"  /* Extirpation of Matter from Right Fallopian Tube, Via Natural or Artificial Opening */
"0UC58ZZ"  =  "1"  /* Extirpation of Matter from Right Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UC60ZZ"  =  "1"  /* Extirpation of Matter from Left Fallopian Tube, Open Approach */
"0UC63ZZ"  =  "1"  /* Extirpation of Matter from Left Fallopian Tube, Percutaneous Approach */
"0UC64ZZ"  =  "1"  /* Extirpation of Matter from Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UC67ZZ"  =  "1"  /* Extirpation of Matter from Left Fallopian Tube, Via Natural or Artificial Opening */
"0UC68ZZ"  =  "1"  /* Extirpation of Matter from Left Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UC70ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Fallopian Tubes, Open Approach */
"0UC73ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Fallopian Tubes, Percutaneous Approach */
"0UC74ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0UC77ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Fallopian Tubes, Via Natural or Artificial Opening */
"0UC78ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Fallopian Tubes, Via Natural or Artificial Opening Endoscopic */
"0UC90ZZ"  =  "1"  /* Extirpation of Matter from Uterus, Open Approach */
"0UC93ZZ"  =  "1"  /* Extirpation of Matter from Uterus, Percutaneous Approach */
"0UC94ZZ"  =  "1"  /* Extirpation of Matter from Uterus, Percutaneous Endoscopic Approach */
"0UCF0ZZ"  =  "1"  /* Extirpation of Matter from Cul-de-sac, Open Approach */
"0UCF3ZZ"  =  "1"  /* Extirpation of Matter from Cul-de-sac, Percutaneous Approach */
"0UCF4ZZ"  =  "1"  /* Extirpation of Matter from Cul-de-sac, Percutaneous Endoscopic Approach */
"0UCF7ZZ"  =  "1"  /* Extirpation of Matter from Cul-de-sac, Via Natural or Artificial Opening */
"0UCF8ZZ"  =  "1"  /* Extirpation of Matter from Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UCG0ZZ"  =  "1"  /* Extirpation of Matter from Vagina, Open Approach */
"0UCG3ZZ"  =  "1"  /* Extirpation of Matter from Vagina, Percutaneous Approach */
"0UCG4ZZ"  =  "1"  /* Extirpation of Matter from Vagina, Percutaneous Endoscopic Approach */
"0UCG7ZZ"  =  "1"  /* Extirpation of Matter from Vagina, Via Natural or Artificial Opening */
"0UCG8ZZ"  =  "1"  /* Extirpation of Matter from Vagina, Via Natural or Artificial Opening Endoscopic */
"0UCM0ZZ"  =  "1"  /* Extirpation of Matter from Vulva, Open Approach */
"0UCMXZZ"  =  "1"  /* Extirpation of Matter from Vulva, External Approach */
"0UHH03Z"  =  "1"  /* Insertion of Infusion Device into Vagina and Cul-de-sac, Open Approach */
"0UHH33Z"  =  "1"  /* Insertion of Infusion Device into Vagina and Cul-de-sac, Percutaneous Approach */
"0UHH43Z"  =  "1"  /* Insertion of Infusion Device into Vagina and Cul-de-sac, Percutaneous Endoscopic Approach */
"0UHH73Z"  =  "1"  /* Insertion of Infusion Device into Vagina and Cul-de-sac, Via Natural or Artificial Opening */
"0UHH83Z"  =  "1"  /* Insertion of Infusion Device into Vagina and Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UJD0ZZ"  =  "1"  /* Inspection of Uterus and Cervix, Open Approach */
"0UJD3ZZ"  =  "1"  /* Inspection of Uterus and Cervix, Percutaneous Approach */
"0UJD4ZZ"  =  "1"  /* Inspection of Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UJD7ZZ"  =  "1"  /* Inspection of Uterus and Cervix, Via Natural or Artificial Opening */
"0UJD8ZZ"  =  "1"  /* Inspection of Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UJH0ZZ"  =  "1"  /* Inspection of Vagina and Cul-de-sac, Open Approach */
"0UJH8ZZ"  =  "1"  /* Inspection of Vagina and Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UJM0ZZ"  =  "1"  /* Inspection of Vulva, Open Approach */
"0UP300Z"  =  "1"  /* Removal of Drainage Device from Ovary, Open Approach */
"0UP303Z"  =  "1"  /* Removal of Infusion Device from Ovary, Open Approach */
"0UP330Z"  =  "1"  /* Removal of Drainage Device from Ovary, Percutaneous Approach */
"0UP333Z"  =  "1"  /* Removal of Infusion Device from Ovary, Percutaneous Approach */
"0UP340Z"  =  "1"  /* Removal of Drainage Device from Ovary, Percutaneous Endoscopic Approach */
"0UP343Z"  =  "1"  /* Removal of Infusion Device from Ovary, Percutaneous Endoscopic Approach */
"0UP800Z"  =  "1"  /* Removal of Drainage Device from Fallopian Tube, Open Approach */
"0UP803Z"  =  "1"  /* Removal of Infusion Device from Fallopian Tube, Open Approach */
"0UP807Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Fallopian Tube, Open Approach */
"0UP80CZ"  =  "1"  /* Removal of Extraluminal Device from Fallopian Tube, Open Approach */
"0UP80DZ"  =  "1"  /* Removal of Intraluminal Device from Fallopian Tube, Open Approach */
"0UP80KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Fallopian Tube, Open Approach */
"0UP830Z"  =  "1"  /* Removal of Drainage Device from Fallopian Tube, Percutaneous Approach */
"0UP833Z"  =  "1"  /* Removal of Infusion Device from Fallopian Tube, Percutaneous Approach */
"0UP837Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Fallopian Tube, Percutaneous Approach */
"0UP83CZ"  =  "1"  /* Removal of Extraluminal Device from Fallopian Tube, Percutaneous Approach */
"0UP83DZ"  =  "1"  /* Removal of Intraluminal Device from Fallopian Tube, Percutaneous Approach */
"0UP83KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Fallopian Tube, Percutaneous Approach */
"0UP840Z"  =  "1"  /* Removal of Drainage Device from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP843Z"  =  "1"  /* Removal of Infusion Device from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP847Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP84CZ"  =  "1"  /* Removal of Extraluminal Device from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP84DZ"  =  "1"  /* Removal of Intraluminal Device from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP84KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP870Z"  =  "1"  /* Removal of Drainage Device from Fallopian Tube, Via Natural or Artificial Opening */
"0UP873Z"  =  "1"  /* Removal of Infusion Device from Fallopian Tube, Via Natural or Artificial Opening */
"0UP877Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Fallopian Tube, Via Natural or Artificial Opening */
"0UP87CZ"  =  "1"  /* Removal of Extraluminal Device from Fallopian Tube, Via Natural or Artificial Opening */
"0UP87DZ"  =  "1"  /* Removal of Intraluminal Device from Fallopian Tube, Via Natural or Artificial Opening */
"0UP87KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Fallopian Tube, Via Natural or Artificial Opening */
"0UP880Z"  =  "1"  /* Removal of Drainage Device from Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UP883Z"  =  "1"  /* Removal of Infusion Device from Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UP887Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UP88CZ"  =  "1"  /* Removal of Extraluminal Device from Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UP88DZ"  =  "1"  /* Removal of Intraluminal Device from Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UP88KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UPD00Z"  =  "1"  /* Removal of Drainage Device from Uterus and Cervix, Open Approach */
"0UPD01Z"  =  "1"  /* Removal of Radioactive Element from Uterus and Cervix, Open Approach */
"0UPD03Z"  =  "1"  /* Removal of Infusion Device from Uterus and Cervix, Open Approach */
"0UPD07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Uterus and Cervix, Open Approach */
"0UPD0CZ"  =  "1"  /* Removal of Extraluminal Device from Uterus and Cervix, Open Approach */
"0UPD0DZ"  =  "1"  /* Removal of Intraluminal Device from Uterus and Cervix, Open Approach */
"0UPD0HZ"  =  "1"  /* Removal of Contraceptive Device from Uterus and Cervix, Open Approach */
"0UPD0JZ"  =  "1"  /* Removal of Synthetic Substitute from Uterus and Cervix, Open Approach */
"0UPD0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Uterus and Cervix, Open Approach */
"0UPD30Z"  =  "1"  /* Removal of Drainage Device from Uterus and Cervix, Percutaneous Approach */
"0UPD31Z"  =  "1"  /* Removal of Radioactive Element from Uterus and Cervix, Percutaneous Approach */
"0UPD33Z"  =  "1"  /* Removal of Infusion Device from Uterus and Cervix, Percutaneous Approach */
"0UPD37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Uterus and Cervix, Percutaneous Approach */
"0UPD3DZ"  =  "1"  /* Removal of Intraluminal Device from Uterus and Cervix, Percutaneous Approach */
"0UPD3HZ"  =  "1"  /* Removal of Contraceptive Device from Uterus and Cervix, Percutaneous Approach */
"0UPD3JZ"  =  "1"  /* Removal of Synthetic Substitute from Uterus and Cervix, Percutaneous Approach */
"0UPD3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Uterus and Cervix, Percutaneous Approach */
"0UPD40Z"  =  "1"  /* Removal of Drainage Device from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD41Z"  =  "1"  /* Removal of Radioactive Element from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD43Z"  =  "1"  /* Removal of Infusion Device from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD4DZ"  =  "1"  /* Removal of Intraluminal Device from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD4HZ"  =  "1"  /* Removal of Contraceptive Device from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD4JZ"  =  "1"  /* Removal of Synthetic Substitute from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD70Z"  =  "1"  /* Removal of Drainage Device from Uterus and Cervix, Via Natural or Artificial Opening */
"0UPD71Z"  =  "1"  /* Removal of Radioactive Element from Uterus and Cervix, Via Natural or Artificial Opening */
"0UPD73Z"  =  "1"  /* Removal of Infusion Device from Uterus and Cervix, Via Natural or Artificial Opening */
"0UPD77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Uterus and Cervix, Via Natural or Artificial Opening */
"0UPD7DZ"  =  "1"  /* Removal of Intraluminal Device from Uterus and Cervix, Via Natural or Artificial Opening */
"0UPD7JZ"  =  "1"  /* Removal of Synthetic Substitute from Uterus and Cervix, Via Natural or Artificial Opening */
"0UPD7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Uterus and Cervix, Via Natural or Artificial Opening */
"0UPD80Z"  =  "1"  /* Removal of Drainage Device from Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UPD81Z"  =  "1"  /* Removal of Radioactive Element from Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UPD83Z"  =  "1"  /* Removal of Infusion Device from Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UPD87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UPD8DZ"  =  "1"  /* Removal of Intraluminal Device from Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UPD8JZ"  =  "1"  /* Removal of Synthetic Substitute from Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UPD8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UPH00Z"  =  "1"  /* Removal of Drainage Device from Vagina and Cul-de-sac, Open Approach */
"0UPH01Z"  =  "1"  /* Removal of Radioactive Element from Vagina and Cul-de-sac, Open Approach */
"0UPH03Z"  =  "1"  /* Removal of Infusion Device from Vagina and Cul-de-sac, Open Approach */
"0UPH07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Vagina and Cul-de-sac, Open Approach */
"0UPH0DZ"  =  "1"  /* Removal of Intraluminal Device from Vagina and Cul-de-sac, Open Approach */
"0UPH0JZ"  =  "1"  /* Removal of Synthetic Substitute from Vagina and Cul-de-sac, Open Approach */
"0UPH0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Vagina and Cul-de-sac, Open Approach */
"0UPH30Z"  =  "1"  /* Removal of Drainage Device from Vagina and Cul-de-sac, Percutaneous Approach */
"0UPH31Z"  =  "1"  /* Removal of Radioactive Element from Vagina and Cul-de-sac, Percutaneous Approach */
"0UPH33Z"  =  "1"  /* Removal of Infusion Device from Vagina and Cul-de-sac, Percutaneous Approach */
"0UPH37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Vagina and Cul-de-sac, Percutaneous Approach */
"0UPH3DZ"  =  "1"  /* Removal of Intraluminal Device from Vagina and Cul-de-sac, Percutaneous Approach */
"0UPH3JZ"  =  "1"  /* Removal of Synthetic Substitute from Vagina and Cul-de-sac, Percutaneous Approach */
"0UPH3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Vagina and Cul-de-sac, Percutaneous Approach */
"0UPH40Z"  =  "1"  /* Removal of Drainage Device from Vagina and Cul-de-sac, Percutaneous Endoscopic Approach */
"0UPH41Z"  =  "1"  /* Removal of Radioactive Element from Vagina and Cul-de-sac, Percutaneous Endoscopic Approach */
"0UPH43Z"  =  "1"  /* Removal of Infusion Device from Vagina and Cul-de-sac, Percutaneous Endoscopic Approach */
"0UPH47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Vagina and Cul-de-sac, Percutaneous Endoscopic Approach */
"0UPH4DZ"  =  "1"  /* Removal of Intraluminal Device from Vagina and Cul-de-sac, Percutaneous Endoscopic Approach */
"0UPH4JZ"  =  "1"  /* Removal of Synthetic Substitute from Vagina and Cul-de-sac, Percutaneous Endoscopic Approach */
"0UPH4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Vagina and Cul-de-sac, Percutaneous Endoscopic Approach */
"0UPH70Z"  =  "1"  /* Removal of Drainage Device from Vagina and Cul-de-sac, Via Natural or Artificial Opening */
"0UPH71Z"  =  "1"  /* Removal of Radioactive Element from Vagina and Cul-de-sac, Via Natural or Artificial Opening */
"0UPH73Z"  =  "1"  /* Removal of Infusion Device from Vagina and Cul-de-sac, Via Natural or Artificial Opening */
"0UPH77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Vagina and Cul-de-sac, Via Natural or Artificial Opening */
"0UPH7DZ"  =  "1"  /* Removal of Intraluminal Device from Vagina and Cul-de-sac, Via Natural or Artificial Opening */
"0UPH7JZ"  =  "1"  /* Removal of Synthetic Substitute from Vagina and Cul-de-sac, Via Natural or Artificial Opening */
"0UPH7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Vagina and Cul-de-sac, Via Natural or Artificial Opening */
"0UPH80Z"  =  "1"  /* Removal of Drainage Device from Vagina and Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UPH81Z"  =  "1"  /* Removal of Radioactive Element from Vagina and Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UPH83Z"  =  "1"  /* Removal of Infusion Device from Vagina and Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UPH87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Vagina and Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UPH8DZ"  =  "1"  /* Removal of Intraluminal Device from Vagina and Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UPH8JZ"  =  "1"  /* Removal of Synthetic Substitute from Vagina and Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UPH8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Vagina and Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UPM00Z"  =  "1"  /* Removal of Drainage Device from Vulva, Open Approach */
"0UPM07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Vulva, Open Approach */
"0UPM0JZ"  =  "1"  /* Removal of Synthetic Substitute from Vulva, Open Approach */
"0UPM0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Vulva, Open Approach */
"0UQ40ZZ"  =  "1"  /* Repair Uterine Supporting Structure, Open Approach */
"0UQ43ZZ"  =  "1"  /* Repair Uterine Supporting Structure, Percutaneous Approach */
"0UQ44ZZ"  =  "1"  /* Repair Uterine Supporting Structure, Percutaneous Endoscopic Approach */
"0UT40ZZ"  =  "1"  /* Resection of Uterine Supporting Structure, Open Approach */
"0UT44ZZ"  =  "1"  /* Resection of Uterine Supporting Structure, Percutaneous Endoscopic Approach */
"0UT47ZZ"  =  "1"  /* Resection of Uterine Supporting Structure, Via Natural or Artificial Opening */
"0UT48ZZ"  =  "1"  /* Resection of Uterine Supporting Structure, Via Natural or Artificial Opening Endoscopic */
"0UW300Z"  =  "1"  /* Revision of Drainage Device in Ovary, Open Approach */
"0UW303Z"  =  "1"  /* Revision of Infusion Device in Ovary, Open Approach */
"0UW330Z"  =  "1"  /* Revision of Drainage Device in Ovary, Percutaneous Approach */
"0UW333Z"  =  "1"  /* Revision of Infusion Device in Ovary, Percutaneous Approach */
"0UW340Z"  =  "1"  /* Revision of Drainage Device in Ovary, Percutaneous Endoscopic Approach */
"0UW343Z"  =  "1"  /* Revision of Infusion Device in Ovary, Percutaneous Endoscopic Approach */
"0UW800Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Open Approach */
"0UW803Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Open Approach */
"0UW807Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Fallopian Tube, Open Approach */
"0UW80CZ"  =  "1"  /* Revision of Extraluminal Device in Fallopian Tube, Open Approach */
"0UW80DZ"  =  "1"  /* Revision of Intraluminal Device in Fallopian Tube, Open Approach */
"0UW80JZ"  =  "1"  /* Revision of Synthetic Substitute in Fallopian Tube, Open Approach */
"0UW80KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Fallopian Tube, Open Approach */
"0UW830Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Percutaneous Approach */
"0UW833Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Percutaneous Approach */
"0UW837Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Fallopian Tube, Percutaneous Approach */
"0UW83CZ"  =  "1"  /* Revision of Extraluminal Device in Fallopian Tube, Percutaneous Approach */
"0UW83DZ"  =  "1"  /* Revision of Intraluminal Device in Fallopian Tube, Percutaneous Approach */
"0UW83JZ"  =  "1"  /* Revision of Synthetic Substitute in Fallopian Tube, Percutaneous Approach */
"0UW83KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Fallopian Tube, Percutaneous Approach */
"0UW840Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW843Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW847Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW84CZ"  =  "1"  /* Revision of Extraluminal Device in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW84DZ"  =  "1"  /* Revision of Intraluminal Device in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW84JZ"  =  "1"  /* Revision of Synthetic Substitute in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW84KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW870Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Via Natural or Artificial Opening */
"0UW873Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Via Natural or Artificial Opening */
"0UW877Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Fallopian Tube, Via Natural or Artificial Opening */
"0UW87CZ"  =  "1"  /* Revision of Extraluminal Device in Fallopian Tube, Via Natural or Artificial Opening */
"0UW87DZ"  =  "1"  /* Revision of Intraluminal Device in Fallopian Tube, Via Natural or Artificial Opening */
"0UW87JZ"  =  "1"  /* Revision of Synthetic Substitute in Fallopian Tube, Via Natural or Artificial Opening */
"0UW87KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Fallopian Tube, Via Natural or Artificial Opening */
"0UW880Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UW883Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UW887Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UW88CZ"  =  "1"  /* Revision of Extraluminal Device in Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UW88DZ"  =  "1"  /* Revision of Intraluminal Device in Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UW88JZ"  =  "1"  /* Revision of Synthetic Substitute in Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UW88KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UWD00Z"  =  "1"  /* Revision of Drainage Device in Uterus and Cervix, Open Approach */
"0UWD01Z"  =  "1"  /* Revision of Radioactive Element in Uterus and Cervix, Open Approach */
"0UWD03Z"  =  "1"  /* Revision of Infusion Device in Uterus and Cervix, Open Approach */
"0UWD07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Uterus and Cervix, Open Approach */
"0UWD0CZ"  =  "1"  /* Revision of Extraluminal Device in Uterus and Cervix, Open Approach */
"0UWD0DZ"  =  "1"  /* Revision of Intraluminal Device in Uterus and Cervix, Open Approach */
"0UWD0HZ"  =  "1"  /* Revision of Contraceptive Device in Uterus and Cervix, Open Approach */
"0UWD0JZ"  =  "1"  /* Revision of Synthetic Substitute in Uterus and Cervix, Open Approach */
"0UWD0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Uterus and Cervix, Open Approach */
"0UWD30Z"  =  "1"  /* Revision of Drainage Device in Uterus and Cervix, Percutaneous Approach */
"0UWD31Z"  =  "1"  /* Revision of Radioactive Element in Uterus and Cervix, Percutaneous Approach */
"0UWD33Z"  =  "1"  /* Revision of Infusion Device in Uterus and Cervix, Percutaneous Approach */
"0UWD37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Uterus and Cervix, Percutaneous Approach */
"0UWD3CZ"  =  "1"  /* Revision of Extraluminal Device in Uterus and Cervix, Percutaneous Approach */
"0UWD3DZ"  =  "1"  /* Revision of Intraluminal Device in Uterus and Cervix, Percutaneous Approach */
"0UWD3HZ"  =  "1"  /* Revision of Contraceptive Device in Uterus and Cervix, Percutaneous Approach */
"0UWD3JZ"  =  "1"  /* Revision of Synthetic Substitute in Uterus and Cervix, Percutaneous Approach */
"0UWD3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Uterus and Cervix, Percutaneous Approach */
"0UWD40Z"  =  "1"  /* Revision of Drainage Device in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD41Z"  =  "1"  /* Revision of Radioactive Element in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD43Z"  =  "1"  /* Revision of Infusion Device in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD4CZ"  =  "1"  /* Revision of Extraluminal Device in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD4DZ"  =  "1"  /* Revision of Intraluminal Device in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD4HZ"  =  "1"  /* Revision of Contraceptive Device in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD4JZ"  =  "1"  /* Revision of Synthetic Substitute in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD70Z"  =  "1"  /* Revision of Drainage Device in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD71Z"  =  "1"  /* Revision of Radioactive Element in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD73Z"  =  "1"  /* Revision of Infusion Device in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD7CZ"  =  "1"  /* Revision of Extraluminal Device in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD7DZ"  =  "1"  /* Revision of Intraluminal Device in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD7HZ"  =  "1"  /* Revision of Contraceptive Device in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD7JZ"  =  "1"  /* Revision of Synthetic Substitute in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD80Z"  =  "1"  /* Revision of Drainage Device in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UWD81Z"  =  "1"  /* Revision of Radioactive Element in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UWD83Z"  =  "1"  /* Revision of Infusion Device in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UWD87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UWD8CZ"  =  "1"  /* Revision of Extraluminal Device in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UWD8DZ"  =  "1"  /* Revision of Intraluminal Device in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UWD8HZ"  =  "1"  /* Revision of Contraceptive Device in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UWD8JZ"  =  "1"  /* Revision of Synthetic Substitute in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UWD8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UWH00Z"  =  "1"  /* Revision of Drainage Device in Vagina and Cul-de-sac, Open Approach */
"0UWH01Z"  =  "1"  /* Revision of Radioactive Element in Vagina and Cul-de-sac, Open Approach */
"0UWH03Z"  =  "1"  /* Revision of Infusion Device in Vagina and Cul-de-sac, Open Approach */
"0UWH07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Vagina and Cul-de-sac, Open Approach */
"0UWH0DZ"  =  "1"  /* Revision of Intraluminal Device in Vagina and Cul-de-sac, Open Approach */
"0UWH0JZ"  =  "1"  /* Revision of Synthetic Substitute in Vagina and Cul-de-sac, Open Approach */
"0UWH0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Vagina and Cul-de-sac, Open Approach */
"0UWH30Z"  =  "1"  /* Revision of Drainage Device in Vagina and Cul-de-sac, Percutaneous Approach */
"0UWH31Z"  =  "1"  /* Revision of Radioactive Element in Vagina and Cul-de-sac, Percutaneous Approach */
"0UWH33Z"  =  "1"  /* Revision of Infusion Device in Vagina and Cul-de-sac, Percutaneous Approach */
"0UWH37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Vagina and Cul-de-sac, Percutaneous Approach */
"0UWH3DZ"  =  "1"  /* Revision of Intraluminal Device in Vagina and Cul-de-sac, Percutaneous Approach */
"0UWH3JZ"  =  "1"  /* Revision of Synthetic Substitute in Vagina and Cul-de-sac, Percutaneous Approach */
"0UWH3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Vagina and Cul-de-sac, Percutaneous Approach */
"0UWH40Z"  =  "1"  /* Revision of Drainage Device in Vagina and Cul-de-sac, Percutaneous Endoscopic Approach */
"0UWH41Z"  =  "1"  /* Revision of Radioactive Element in Vagina and Cul-de-sac, Percutaneous Endoscopic Approach */
"0UWH43Z"  =  "1"  /* Revision of Infusion Device in Vagina and Cul-de-sac, Percutaneous Endoscopic Approach */
"0UWH47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Vagina and Cul-de-sac, Percutaneous Endoscopic Approach */
"0UWH4DZ"  =  "1"  /* Revision of Intraluminal Device in Vagina and Cul-de-sac, Percutaneous Endoscopic Approach */
"0UWH4JZ"  =  "1"  /* Revision of Synthetic Substitute in Vagina and Cul-de-sac, Percutaneous Endoscopic Approach */
"0UWH4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Vagina and Cul-de-sac, Percutaneous Endoscopic Approach */
"0UWH70Z"  =  "1"  /* Revision of Drainage Device in Vagina and Cul-de-sac, Via Natural or Artificial Opening */
"0UWH71Z"  =  "1"  /* Revision of Radioactive Element in Vagina and Cul-de-sac, Via Natural or Artificial Opening */
"0UWH73Z"  =  "1"  /* Revision of Infusion Device in Vagina and Cul-de-sac, Via Natural or Artificial Opening */
"0UWH77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Vagina and Cul-de-sac, Via Natural or Artificial Opening */
"0UWH7DZ"  =  "1"  /* Revision of Intraluminal Device in Vagina and Cul-de-sac, Via Natural or Artificial Opening */
"0UWH7JZ"  =  "1"  /* Revision of Synthetic Substitute in Vagina and Cul-de-sac, Via Natural or Artificial Opening */
"0UWH7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Vagina and Cul-de-sac, Via Natural or Artificial Opening */
"0UWH80Z"  =  "1"  /* Revision of Drainage Device in Vagina and Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UWH81Z"  =  "1"  /* Revision of Radioactive Element in Vagina and Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UWH83Z"  =  "1"  /* Revision of Infusion Device in Vagina and Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UWH87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Vagina and Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UWH8DZ"  =  "1"  /* Revision of Intraluminal Device in Vagina and Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UWH8JZ"  =  "1"  /* Revision of Synthetic Substitute in Vagina and Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UWH8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Vagina and Cul-de-sac, Via Natural or Artificial Opening Endoscopic */
"0UWM00Z"  =  "1"  /* Revision of Drainage Device in Vulva, Open Approach */
"0UWM07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Vulva, Open Approach */
"0UWM0JZ"  =  "1"  /* Revision of Synthetic Substitute in Vulva, Open Approach */
"0UWM0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Vulva, Open Approach */
"0V9000Z"  =  "1"  /* Drainage of Prostate with Drainage Device, Open Approach */
"0V900ZZ"  =  "1"  /* Drainage of Prostate, Open Approach */
"0V9070Z"  =  "1"  /* Drainage of Prostate with Drainage Device, Via Natural or Artificial Opening */
"0V907ZZ"  =  "1"  /* Drainage of Prostate, Via Natural or Artificial Opening */
"0V9080Z"  =  "1"  /* Drainage of Prostate with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0V908ZZ"  =  "1"  /* Drainage of Prostate, Via Natural or Artificial Opening Endoscopic */
"0V9500Z"  =  "1"  /* Drainage of Scrotum with Drainage Device, Open Approach */
"0V950ZZ"  =  "1"  /* Drainage of Scrotum, Open Approach */
"0V9530Z"  =  "1"  /* Drainage of Scrotum with Drainage Device, Percutaneous Approach */
"0V953ZZ"  =  "1"  /* Drainage of Scrotum, Percutaneous Approach */
"0V9540Z"  =  "1"  /* Drainage of Scrotum with Drainage Device, Percutaneous Endoscopic Approach */
"0V954ZZ"  =  "1"  /* Drainage of Scrotum, Percutaneous Endoscopic Approach */
"0V95X0Z"  =  "1"  /* Drainage of Scrotum with Drainage Device, External Approach */
"0V95XZZ"  =  "1"  /* Drainage of Scrotum, External Approach */
"0V9600Z"  =  "1"  /* Drainage of Right Tunica Vaginalis with Drainage Device, Open Approach */
"0V960ZZ"  =  "1"  /* Drainage of Right Tunica Vaginalis, Open Approach */
"0V9700Z"  =  "1"  /* Drainage of Left Tunica Vaginalis with Drainage Device, Open Approach */
"0V970ZZ"  =  "1"  /* Drainage of Left Tunica Vaginalis, Open Approach */
"0V9900Z"  =  "1"  /* Drainage of Right Testis with Drainage Device, Open Approach */
"0V990ZZ"  =  "1"  /* Drainage of Right Testis, Open Approach */
"0V9B00Z"  =  "1"  /* Drainage of Left Testis with Drainage Device, Open Approach */
"0V9B0ZZ"  =  "1"  /* Drainage of Left Testis, Open Approach */
"0V9C00Z"  =  "1"  /* Drainage of Bilateral Testes with Drainage Device, Open Approach */
"0V9C0ZZ"  =  "1"  /* Drainage of Bilateral Testes, Open Approach */
"0V9F00Z"  =  "1"  /* Drainage of Right Spermatic Cord with Drainage Device, Open Approach */
"0V9F0ZZ"  =  "1"  /* Drainage of Right Spermatic Cord, Open Approach */
"0V9F30Z"  =  "1"  /* Drainage of Right Spermatic Cord with Drainage Device, Percutaneous Approach */
"0V9F3ZZ"  =  "1"  /* Drainage of Right Spermatic Cord, Percutaneous Approach */
"0V9F40Z"  =  "1"  /* Drainage of Right Spermatic Cord with Drainage Device, Percutaneous Endoscopic Approach */
"0V9F4ZZ"  =  "1"  /* Drainage of Right Spermatic Cord, Percutaneous Endoscopic Approach */
"0V9G00Z"  =  "1"  /* Drainage of Left Spermatic Cord with Drainage Device, Open Approach */
"0V9G0ZZ"  =  "1"  /* Drainage of Left Spermatic Cord, Open Approach */
"0V9G30Z"  =  "1"  /* Drainage of Left Spermatic Cord with Drainage Device, Percutaneous Approach */
"0V9G3ZZ"  =  "1"  /* Drainage of Left Spermatic Cord, Percutaneous Approach */
"0V9G40Z"  =  "1"  /* Drainage of Left Spermatic Cord with Drainage Device, Percutaneous Endoscopic Approach */
"0V9G4ZZ"  =  "1"  /* Drainage of Left Spermatic Cord, Percutaneous Endoscopic Approach */
"0V9H00Z"  =  "1"  /* Drainage of Bilateral Spermatic Cords with Drainage Device, Open Approach */
"0V9H0ZZ"  =  "1"  /* Drainage of Bilateral Spermatic Cords, Open Approach */
"0V9H30Z"  =  "1"  /* Drainage of Bilateral Spermatic Cords with Drainage Device, Percutaneous Approach */
"0V9H3ZZ"  =  "1"  /* Drainage of Bilateral Spermatic Cords, Percutaneous Approach */
"0V9H40Z"  =  "1"  /* Drainage of Bilateral Spermatic Cords with Drainage Device, Percutaneous Endoscopic Approach */
"0V9H4ZZ"  =  "1"  /* Drainage of Bilateral Spermatic Cords, Percutaneous Endoscopic Approach */
"0V9J00Z"  =  "1"  /* Drainage of Right Epididymis with Drainage Device, Open Approach */
"0V9J0ZZ"  =  "1"  /* Drainage of Right Epididymis, Open Approach */
"0V9J30Z"  =  "1"  /* Drainage of Right Epididymis with Drainage Device, Percutaneous Approach */
"0V9J3ZZ"  =  "1"  /* Drainage of Right Epididymis, Percutaneous Approach */
"0V9J40Z"  =  "1"  /* Drainage of Right Epididymis with Drainage Device, Percutaneous Endoscopic Approach */
"0V9J4ZZ"  =  "1"  /* Drainage of Right Epididymis, Percutaneous Endoscopic Approach */
"0V9K00Z"  =  "1"  /* Drainage of Left Epididymis with Drainage Device, Open Approach */
"0V9K0ZZ"  =  "1"  /* Drainage of Left Epididymis, Open Approach */
"0V9K30Z"  =  "1"  /* Drainage of Left Epididymis with Drainage Device, Percutaneous Approach */
"0V9K3ZZ"  =  "1"  /* Drainage of Left Epididymis, Percutaneous Approach */
"0V9K40Z"  =  "1"  /* Drainage of Left Epididymis with Drainage Device, Percutaneous Endoscopic Approach */
"0V9K4ZZ"  =  "1"  /* Drainage of Left Epididymis, Percutaneous Endoscopic Approach */
"0V9L00Z"  =  "1"  /* Drainage of Bilateral Epididymis with Drainage Device, Open Approach */
"0V9L0ZZ"  =  "1"  /* Drainage of Bilateral Epididymis, Open Approach */
"0V9L30Z"  =  "1"  /* Drainage of Bilateral Epididymis with Drainage Device, Percutaneous Approach */
"0V9L3ZZ"  =  "1"  /* Drainage of Bilateral Epididymis, Percutaneous Approach */
"0V9L40Z"  =  "1"  /* Drainage of Bilateral Epididymis with Drainage Device, Percutaneous Endoscopic Approach */
"0V9L4ZZ"  =  "1"  /* Drainage of Bilateral Epididymis, Percutaneous Endoscopic Approach */
"0V9N00Z"  =  "1"  /* Drainage of Right Vas Deferens with Drainage Device, Open Approach */
"0V9N0ZZ"  =  "1"  /* Drainage of Right Vas Deferens, Open Approach */
"0V9N30Z"  =  "1"  /* Drainage of Right Vas Deferens with Drainage Device, Percutaneous Approach */
"0V9N3ZZ"  =  "1"  /* Drainage of Right Vas Deferens, Percutaneous Approach */
"0V9N40Z"  =  "1"  /* Drainage of Right Vas Deferens with Drainage Device, Percutaneous Endoscopic Approach */
"0V9N4ZZ"  =  "1"  /* Drainage of Right Vas Deferens, Percutaneous Endoscopic Approach */
"0V9P00Z"  =  "1"  /* Drainage of Left Vas Deferens with Drainage Device, Open Approach */
"0V9P0ZZ"  =  "1"  /* Drainage of Left Vas Deferens, Open Approach */
"0V9P30Z"  =  "1"  /* Drainage of Left Vas Deferens with Drainage Device, Percutaneous Approach */
"0V9P3ZZ"  =  "1"  /* Drainage of Left Vas Deferens, Percutaneous Approach */
"0V9P40Z"  =  "1"  /* Drainage of Left Vas Deferens with Drainage Device, Percutaneous Endoscopic Approach */
"0V9P4ZZ"  =  "1"  /* Drainage of Left Vas Deferens, Percutaneous Endoscopic Approach */
"0V9Q00Z"  =  "1"  /* Drainage of Bilateral Vas Deferens with Drainage Device, Open Approach */
"0V9Q0ZZ"  =  "1"  /* Drainage of Bilateral Vas Deferens, Open Approach */
"0V9Q30Z"  =  "1"  /* Drainage of Bilateral Vas Deferens with Drainage Device, Percutaneous Approach */
"0V9Q3ZZ"  =  "1"  /* Drainage of Bilateral Vas Deferens, Percutaneous Approach */
"0V9Q40Z"  =  "1"  /* Drainage of Bilateral Vas Deferens with Drainage Device, Percutaneous Endoscopic Approach */
"0V9Q4ZZ"  =  "1"  /* Drainage of Bilateral Vas Deferens, Percutaneous Endoscopic Approach */
"0V9S00Z"  =  "1"  /* Drainage of Penis with Drainage Device, Open Approach */
"0V9S0ZZ"  =  "1"  /* Drainage of Penis, Open Approach */
"0V9S30Z"  =  "1"  /* Drainage of Penis with Drainage Device, Percutaneous Approach */
"0V9S3ZZ"  =  "1"  /* Drainage of Penis, Percutaneous Approach */
"0V9S40Z"  =  "1"  /* Drainage of Penis with Drainage Device, Percutaneous Endoscopic Approach */
"0V9S4ZZ"  =  "1"  /* Drainage of Penis, Percutaneous Endoscopic Approach */
"0V9SX0Z"  =  "1"  /* Drainage of Penis with Drainage Device, External Approach */
"0V9SXZZ"  =  "1"  /* Drainage of Penis, External Approach */
"0V9T00Z"  =  "1"  /* Drainage of Prepuce with Drainage Device, Open Approach */
"0V9T0ZZ"  =  "1"  /* Drainage of Prepuce, Open Approach */
"0V9T30Z"  =  "1"  /* Drainage of Prepuce with Drainage Device, Percutaneous Approach */
"0V9T3ZZ"  =  "1"  /* Drainage of Prepuce, Percutaneous Approach */
"0V9T40Z"  =  "1"  /* Drainage of Prepuce with Drainage Device, Percutaneous Endoscopic Approach */
"0V9T4ZZ"  =  "1"  /* Drainage of Prepuce, Percutaneous Endoscopic Approach */
"0V9TX0Z"  =  "1"  /* Drainage of Prepuce with Drainage Device, External Approach */
"0V9TXZZ"  =  "1"  /* Drainage of Prepuce, External Approach */
"0VBF0ZZ"  =  "1"  /* Excision of Right Spermatic Cord, Open Approach */
"0VBF3ZZ"  =  "1"  /* Excision of Right Spermatic Cord, Percutaneous Approach */
"0VBF4ZZ"  =  "1"  /* Excision of Right Spermatic Cord, Percutaneous Endoscopic Approach */
"0VBG0ZZ"  =  "1"  /* Excision of Left Spermatic Cord, Open Approach */
"0VBG3ZZ"  =  "1"  /* Excision of Left Spermatic Cord, Percutaneous Approach */
"0VBG4ZZ"  =  "1"  /* Excision of Left Spermatic Cord, Percutaneous Endoscopic Approach */
"0VBH0ZZ"  =  "1"  /* Excision of Bilateral Spermatic Cords, Open Approach */
"0VBH3ZZ"  =  "1"  /* Excision of Bilateral Spermatic Cords, Percutaneous Approach */
"0VBH4ZZ"  =  "1"  /* Excision of Bilateral Spermatic Cords, Percutaneous Endoscopic Approach */
"0VC00ZZ"  =  "1"  /* Extirpation of Matter from Prostate, Open Approach */
"0VC03ZZ"  =  "1"  /* Extirpation of Matter from Prostate, Percutaneous Approach */
"0VC04ZZ"  =  "1"  /* Extirpation of Matter from Prostate, Percutaneous Endoscopic Approach */
"0VC07ZZ"  =  "1"  /* Extirpation of Matter from Prostate, Via Natural or Artificial Opening */
"0VC08ZZ"  =  "1"  /* Extirpation of Matter from Prostate, Via Natural or Artificial Opening Endoscopic */
"0VC50ZZ"  =  "1"  /* Extirpation of Matter from Scrotum, Open Approach */
"0VC53ZZ"  =  "1"  /* Extirpation of Matter from Scrotum, Percutaneous Approach */
"0VC54ZZ"  =  "1"  /* Extirpation of Matter from Scrotum, Percutaneous Endoscopic Approach */
"0VC60ZZ"  =  "1"  /* Extirpation of Matter from Right Tunica Vaginalis, Open Approach */
"0VC63ZZ"  =  "1"  /* Extirpation of Matter from Right Tunica Vaginalis, Percutaneous Approach */
"0VC64ZZ"  =  "1"  /* Extirpation of Matter from Right Tunica Vaginalis, Percutaneous Endoscopic Approach */
"0VC70ZZ"  =  "1"  /* Extirpation of Matter from Left Tunica Vaginalis, Open Approach */
"0VC73ZZ"  =  "1"  /* Extirpation of Matter from Left Tunica Vaginalis, Percutaneous Approach */
"0VC74ZZ"  =  "1"  /* Extirpation of Matter from Left Tunica Vaginalis, Percutaneous Endoscopic Approach */
"0VC90ZZ"  =  "1"  /* Extirpation of Matter from Right Testis, Open Approach */
"0VC93ZZ"  =  "1"  /* Extirpation of Matter from Right Testis, Percutaneous Approach */
"0VC94ZZ"  =  "1"  /* Extirpation of Matter from Right Testis, Percutaneous Endoscopic Approach */
"0VCB0ZZ"  =  "1"  /* Extirpation of Matter from Left Testis, Open Approach */
"0VCB3ZZ"  =  "1"  /* Extirpation of Matter from Left Testis, Percutaneous Approach */
"0VCB4ZZ"  =  "1"  /* Extirpation of Matter from Left Testis, Percutaneous Endoscopic Approach */
"0VCC0ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Testes, Open Approach */
"0VCC3ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Testes, Percutaneous Approach */
"0VCC4ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Testes, Percutaneous Endoscopic Approach */
"0VCF0ZZ"  =  "1"  /* Extirpation of Matter from Right Spermatic Cord, Open Approach */
"0VCF3ZZ"  =  "1"  /* Extirpation of Matter from Right Spermatic Cord, Percutaneous Approach */
"0VCF4ZZ"  =  "1"  /* Extirpation of Matter from Right Spermatic Cord, Percutaneous Endoscopic Approach */
"0VCG0ZZ"  =  "1"  /* Extirpation of Matter from Left Spermatic Cord, Open Approach */
"0VCG3ZZ"  =  "1"  /* Extirpation of Matter from Left Spermatic Cord, Percutaneous Approach */
"0VCG4ZZ"  =  "1"  /* Extirpation of Matter from Left Spermatic Cord, Percutaneous Endoscopic Approach */
"0VCH0ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Spermatic Cords, Open Approach */
"0VCH3ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Spermatic Cords, Percutaneous Approach */
"0VCH4ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Spermatic Cords, Percutaneous Endoscopic Approach */
"0VCJ0ZZ"  =  "1"  /* Extirpation of Matter from Right Epididymis, Open Approach */
"0VCJ3ZZ"  =  "1"  /* Extirpation of Matter from Right Epididymis, Percutaneous Approach */
"0VCJ4ZZ"  =  "1"  /* Extirpation of Matter from Right Epididymis, Percutaneous Endoscopic Approach */
"0VCK0ZZ"  =  "1"  /* Extirpation of Matter from Left Epididymis, Open Approach */
"0VCK3ZZ"  =  "1"  /* Extirpation of Matter from Left Epididymis, Percutaneous Approach */
"0VCK4ZZ"  =  "1"  /* Extirpation of Matter from Left Epididymis, Percutaneous Endoscopic Approach */
"0VCL0ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Epididymis, Open Approach */
"0VCL3ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Epididymis, Percutaneous Approach */
"0VCL4ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Epididymis, Percutaneous Endoscopic Approach */
"0VCN0ZZ"  =  "1"  /* Extirpation of Matter from Right Vas Deferens, Open Approach */
"0VCN3ZZ"  =  "1"  /* Extirpation of Matter from Right Vas Deferens, Percutaneous Approach */
"0VCN4ZZ"  =  "1"  /* Extirpation of Matter from Right Vas Deferens, Percutaneous Endoscopic Approach */
"0VCP0ZZ"  =  "1"  /* Extirpation of Matter from Left Vas Deferens, Open Approach */
"0VCP3ZZ"  =  "1"  /* Extirpation of Matter from Left Vas Deferens, Percutaneous Approach */
"0VCP4ZZ"  =  "1"  /* Extirpation of Matter from Left Vas Deferens, Percutaneous Endoscopic Approach */
"0VCQ0ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Vas Deferens, Open Approach */
"0VCQ3ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Vas Deferens, Percutaneous Approach */
"0VCQ4ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Vas Deferens, Percutaneous Endoscopic Approach */
"0VCS0ZZ"  =  "1"  /* Extirpation of Matter from Penis, Open Approach */
"0VCS3ZZ"  =  "1"  /* Extirpation of Matter from Penis, Percutaneous Approach */
"0VCS4ZZ"  =  "1"  /* Extirpation of Matter from Penis, Percutaneous Endoscopic Approach */
"0VCT0ZZ"  =  "1"  /* Extirpation of Matter from Prepuce, Open Approach */
"0VCT3ZZ"  =  "1"  /* Extirpation of Matter from Prepuce, Percutaneous Approach */
"0VCT4ZZ"  =  "1"  /* Extirpation of Matter from Prepuce, Percutaneous Endoscopic Approach */
"0VCTXZZ"  =  "1"  /* Extirpation of Matter from Prepuce, External Approach */
"0VJ40ZZ"  =  "1"  /* Inspection of Prostate and Seminal Vesicles, Open Approach */
"0VJ43ZZ"  =  "1"  /* Inspection of Prostate and Seminal Vesicles, Percutaneous Approach */
"0VJ44ZZ"  =  "1"  /* Inspection of Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VLF0CZ"  =  "1"  /* Occlusion of Right Spermatic Cord with Extraluminal Device, Open Approach */
"0VLF0DZ"  =  "1"  /* Occlusion of Right Spermatic Cord with Intraluminal Device, Open Approach */
"0VLF0ZZ"  =  "1"  /* Occlusion of Right Spermatic Cord, Open Approach */
"0VLF3CZ"  =  "1"  /* Occlusion of Right Spermatic Cord with Extraluminal Device, Percutaneous Approach */
"0VLF3DZ"  =  "1"  /* Occlusion of Right Spermatic Cord with Intraluminal Device, Percutaneous Approach */
"0VLF3ZZ"  =  "1"  /* Occlusion of Right Spermatic Cord, Percutaneous Approach */
"0VLF4CZ"  =  "1"  /* Occlusion of Right Spermatic Cord with Extraluminal Device, Percutaneous Endoscopic Approach */
"0VLF4DZ"  =  "1"  /* Occlusion of Right Spermatic Cord with Intraluminal Device, Percutaneous Endoscopic Approach */
"0VLF4ZZ"  =  "1"  /* Occlusion of Right Spermatic Cord, Percutaneous Endoscopic Approach */
"0VLG0CZ"  =  "1"  /* Occlusion of Left Spermatic Cord with Extraluminal Device, Open Approach */
"0VLG0DZ"  =  "1"  /* Occlusion of Left Spermatic Cord with Intraluminal Device, Open Approach */
"0VLG0ZZ"  =  "1"  /* Occlusion of Left Spermatic Cord, Open Approach */
"0VLG3CZ"  =  "1"  /* Occlusion of Left Spermatic Cord with Extraluminal Device, Percutaneous Approach */
"0VLG3DZ"  =  "1"  /* Occlusion of Left Spermatic Cord with Intraluminal Device, Percutaneous Approach */
"0VLG3ZZ"  =  "1"  /* Occlusion of Left Spermatic Cord, Percutaneous Approach */
"0VLG4CZ"  =  "1"  /* Occlusion of Left Spermatic Cord with Extraluminal Device, Percutaneous Endoscopic Approach */
"0VLG4DZ"  =  "1"  /* Occlusion of Left Spermatic Cord with Intraluminal Device, Percutaneous Endoscopic Approach */
"0VLG4ZZ"  =  "1"  /* Occlusion of Left Spermatic Cord, Percutaneous Endoscopic Approach */
"0VLH0CZ"  =  "1"  /* Occlusion of Bilateral Spermatic Cords with Extraluminal Device, Open Approach */
"0VLH0DZ"  =  "1"  /* Occlusion of Bilateral Spermatic Cords with Intraluminal Device, Open Approach */
"0VLH0ZZ"  =  "1"  /* Occlusion of Bilateral Spermatic Cords, Open Approach */
"0VLH3CZ"  =  "1"  /* Occlusion of Bilateral Spermatic Cords with Extraluminal Device, Percutaneous Approach */
"0VLH3DZ"  =  "1"  /* Occlusion of Bilateral Spermatic Cords with Intraluminal Device, Percutaneous Approach */
"0VLH3ZZ"  =  "1"  /* Occlusion of Bilateral Spermatic Cords, Percutaneous Approach */
"0VLH4CZ"  =  "1"  /* Occlusion of Bilateral Spermatic Cords with Extraluminal Device, Percutaneous Endoscopic Approach */
"0VLH4DZ"  =  "1"  /* Occlusion of Bilateral Spermatic Cords with Intraluminal Device, Percutaneous Endoscopic Approach */
"0VLH4ZZ"  =  "1"  /* Occlusion of Bilateral Spermatic Cords, Percutaneous Endoscopic Approach */
"0VP400Z"  =  "1"  /* Removal of Drainage Device from Prostate and Seminal Vesicles, Open Approach */
"0VP401Z"  =  "1"  /* Removal of Radioactive Element from Prostate and Seminal Vesicles, Open Approach */
"0VP403Z"  =  "1"  /* Removal of Infusion Device from Prostate and Seminal Vesicles, Open Approach */
"0VP407Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Prostate and Seminal Vesicles, Open Approach */
"0VP40JZ"  =  "1"  /* Removal of Synthetic Substitute from Prostate and Seminal Vesicles, Open Approach */
"0VP40KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Prostate and Seminal Vesicles, Open Approach */
"0VP430Z"  =  "1"  /* Removal of Drainage Device from Prostate and Seminal Vesicles, Percutaneous Approach */
"0VP431Z"  =  "1"  /* Removal of Radioactive Element from Prostate and Seminal Vesicles, Percutaneous Approach */
"0VP433Z"  =  "1"  /* Removal of Infusion Device from Prostate and Seminal Vesicles, Percutaneous Approach */
"0VP437Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Prostate and Seminal Vesicles, Percutaneous Approach */
"0VP43JZ"  =  "1"  /* Removal of Synthetic Substitute from Prostate and Seminal Vesicles, Percutaneous Approach */
"0VP43KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Prostate and Seminal Vesicles, Percutaneous Approach */
"0VP440Z"  =  "1"  /* Removal of Drainage Device from Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VP441Z"  =  "1"  /* Removal of Radioactive Element from Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VP443Z"  =  "1"  /* Removal of Infusion Device from Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VP447Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VP44JZ"  =  "1"  /* Removal of Synthetic Substitute from Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VP44KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VP470Z"  =  "1"  /* Removal of Drainage Device from Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VP471Z"  =  "1"  /* Removal of Radioactive Element from Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VP473Z"  =  "1"  /* Removal of Infusion Device from Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VP477Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VP47JZ"  =  "1"  /* Removal of Synthetic Substitute from Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VP47KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VP480Z"  =  "1"  /* Removal of Drainage Device from Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0VP481Z"  =  "1"  /* Removal of Radioactive Element from Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0VP483Z"  =  "1"  /* Removal of Infusion Device from Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0VP487Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0VP48JZ"  =  "1"  /* Removal of Synthetic Substitute from Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0VP48KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0VP800Z"  =  "1"  /* Removal of Drainage Device from Scrotum and Tunica Vaginalis, Open Approach */
"0VP803Z"  =  "1"  /* Removal of Infusion Device from Scrotum and Tunica Vaginalis, Open Approach */
"0VP807Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Scrotum and Tunica Vaginalis, Open Approach */
"0VP80JZ"  =  "1"  /* Removal of Synthetic Substitute from Scrotum and Tunica Vaginalis, Open Approach */
"0VP80KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Scrotum and Tunica Vaginalis, Open Approach */
"0VP830Z"  =  "1"  /* Removal of Drainage Device from Scrotum and Tunica Vaginalis, Percutaneous Approach */
"0VP833Z"  =  "1"  /* Removal of Infusion Device from Scrotum and Tunica Vaginalis, Percutaneous Approach */
"0VP837Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Scrotum and Tunica Vaginalis, Percutaneous Approach */
"0VP83JZ"  =  "1"  /* Removal of Synthetic Substitute from Scrotum and Tunica Vaginalis, Percutaneous Approach */
"0VP83KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Scrotum and Tunica Vaginalis, Percutaneous Approach */
"0VP840Z"  =  "1"  /* Removal of Drainage Device from Scrotum and Tunica Vaginalis, Percutaneous Endoscopic Approach */
"0VP843Z"  =  "1"  /* Removal of Infusion Device from Scrotum and Tunica Vaginalis, Percutaneous Endoscopic Approach */
"0VP847Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Scrotum and Tunica Vaginalis, Percutaneous Endoscopic Approach */
"0VP84JZ"  =  "1"  /* Removal of Synthetic Substitute from Scrotum and Tunica Vaginalis, Percutaneous Endoscopic Approach */
"0VP84KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Scrotum and Tunica Vaginalis, Percutaneous Endoscopic Approach */
"0VP870Z"  =  "1"  /* Removal of Drainage Device from Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening */
"0VP873Z"  =  "1"  /* Removal of Infusion Device from Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening */
"0VP877Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening */
"0VP87JZ"  =  "1"  /* Removal of Synthetic Substitute from Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening */
"0VP87KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening */
"0VP880Z"  =  "1"  /* Removal of Drainage Device from Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening Endoscopic */
"0VP883Z"  =  "1"  /* Removal of Infusion Device from Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening Endoscopic */
"0VP887Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening Endoscopic */
"0VP88JZ"  =  "1"  /* Removal of Synthetic Substitute from Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening Endoscopic */
"0VP88KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening Endoscopic */
"0VPD00Z"  =  "1"  /* Removal of Drainage Device from Testis, Open Approach */
"0VPD03Z"  =  "1"  /* Removal of Infusion Device from Testis, Open Approach */
"0VPD07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Testis, Open Approach */
"0VPD0JZ"  =  "1"  /* Removal of Synthetic Substitute from Testis, Open Approach */
"0VPD0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Testis, Open Approach */
"0VPD30Z"  =  "1"  /* Removal of Drainage Device from Testis, Percutaneous Approach */
"0VPD33Z"  =  "1"  /* Removal of Infusion Device from Testis, Percutaneous Approach */
"0VPD37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Testis, Percutaneous Approach */
"0VPD3JZ"  =  "1"  /* Removal of Synthetic Substitute from Testis, Percutaneous Approach */
"0VPD3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Testis, Percutaneous Approach */
"0VPD40Z"  =  "1"  /* Removal of Drainage Device from Testis, Percutaneous Endoscopic Approach */
"0VPD43Z"  =  "1"  /* Removal of Infusion Device from Testis, Percutaneous Endoscopic Approach */
"0VPD47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Testis, Percutaneous Endoscopic Approach */
"0VPD4JZ"  =  "1"  /* Removal of Synthetic Substitute from Testis, Percutaneous Endoscopic Approach */
"0VPD4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Testis, Percutaneous Endoscopic Approach */
"0VPD70Z"  =  "1"  /* Removal of Drainage Device from Testis, Via Natural or Artificial Opening */
"0VPD73Z"  =  "1"  /* Removal of Infusion Device from Testis, Via Natural or Artificial Opening */
"0VPD77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Testis, Via Natural or Artificial Opening */
"0VPD7JZ"  =  "1"  /* Removal of Synthetic Substitute from Testis, Via Natural or Artificial Opening */
"0VPD7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Testis, Via Natural or Artificial Opening */
"0VPD80Z"  =  "1"  /* Removal of Drainage Device from Testis, Via Natural or Artificial Opening Endoscopic */
"0VPD83Z"  =  "1"  /* Removal of Infusion Device from Testis, Via Natural or Artificial Opening Endoscopic */
"0VPD87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Testis, Via Natural or Artificial Opening Endoscopic */
"0VPD8JZ"  =  "1"  /* Removal of Synthetic Substitute from Testis, Via Natural or Artificial Opening Endoscopic */
"0VPD8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Testis, Via Natural or Artificial Opening Endoscopic */
"0VPM00Z"  =  "1"  /* Removal of Drainage Device from Epididymis and Spermatic Cord, Open Approach */
"0VPM03Z"  =  "1"  /* Removal of Infusion Device from Epididymis and Spermatic Cord, Open Approach */
"0VPM07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Epididymis and Spermatic Cord, Open Approach */
"0VPM0CZ"  =  "1"  /* Removal of Extraluminal Device from Epididymis and Spermatic Cord, Open Approach */
"0VPM0JZ"  =  "1"  /* Removal of Synthetic Substitute from Epididymis and Spermatic Cord, Open Approach */
"0VPM0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Epididymis and Spermatic Cord, Open Approach */
"0VPM30Z"  =  "1"  /* Removal of Drainage Device from Epididymis and Spermatic Cord, Percutaneous Approach */
"0VPM33Z"  =  "1"  /* Removal of Infusion Device from Epididymis and Spermatic Cord, Percutaneous Approach */
"0VPM37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Epididymis and Spermatic Cord, Percutaneous Approach */
"0VPM3CZ"  =  "1"  /* Removal of Extraluminal Device from Epididymis and Spermatic Cord, Percutaneous Approach */
"0VPM3JZ"  =  "1"  /* Removal of Synthetic Substitute from Epididymis and Spermatic Cord, Percutaneous Approach */
"0VPM3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Epididymis and Spermatic Cord, Percutaneous Approach */
"0VPM40Z"  =  "1"  /* Removal of Drainage Device from Epididymis and Spermatic Cord, Percutaneous Endoscopic Approach */
"0VPM43Z"  =  "1"  /* Removal of Infusion Device from Epididymis and Spermatic Cord, Percutaneous Endoscopic Approach */
"0VPM47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Epididymis and Spermatic Cord, Percutaneous Endoscopic Approach */
"0VPM4CZ"  =  "1"  /* Removal of Extraluminal Device from Epididymis and Spermatic Cord, Percutaneous Endoscopic Approach */
"0VPM4JZ"  =  "1"  /* Removal of Synthetic Substitute from Epididymis and Spermatic Cord, Percutaneous Endoscopic Approach */
"0VPM4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Epididymis and Spermatic Cord, Percutaneous Endoscopic Approach */
"0VPM70Z"  =  "1"  /* Removal of Drainage Device from Epididymis and Spermatic Cord, Via Natural or Artificial Opening */
"0VPM73Z"  =  "1"  /* Removal of Infusion Device from Epididymis and Spermatic Cord, Via Natural or Artificial Opening */
"0VPM77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Epididymis and Spermatic Cord, Via Natural or Artificial Opening */
"0VPM7CZ"  =  "1"  /* Removal of Extraluminal Device from Epididymis and Spermatic Cord, Via Natural or Artificial Opening */
"0VPM7JZ"  =  "1"  /* Removal of Synthetic Substitute from Epididymis and Spermatic Cord, Via Natural or Artificial Opening */
"0VPM7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Epididymis and Spermatic Cord, Via Natural or Artificial Opening */
"0VPM80Z"  =  "1"  /* Removal of Drainage Device from Epididymis and Spermatic Cord, Via Natural or Artificial Opening Endoscopic */
"0VPM83Z"  =  "1"  /* Removal of Infusion Device from Epididymis and Spermatic Cord, Via Natural or Artificial Opening Endoscopic */
"0VPM87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Epididymis and Spermatic Cord, Via Natural or Artificial Opening Endoscopic */
"0VPM8CZ"  =  "1"  /* Removal of Extraluminal Device from Epididymis and Spermatic Cord, Via Natural or Artificial Opening Endoscopic */
"0VPM8JZ"  =  "1"  /* Removal of Synthetic Substitute from Epididymis and Spermatic Cord, Via Natural or Artificial Opening Endoscopic */
"0VPM8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Epididymis and Spermatic Cord, Via Natural or Artificial Opening Endoscopic */
"0VPR00Z"  =  "1"  /* Removal of Drainage Device from Vas Deferens, Open Approach */
"0VPR03Z"  =  "1"  /* Removal of Infusion Device from Vas Deferens, Open Approach */
"0VPR07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Vas Deferens, Open Approach */
"0VPR0CZ"  =  "1"  /* Removal of Extraluminal Device from Vas Deferens, Open Approach */
"0VPR0JZ"  =  "1"  /* Removal of Synthetic Substitute from Vas Deferens, Open Approach */
"0VPR0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Vas Deferens, Open Approach */
"0VPR30Z"  =  "1"  /* Removal of Drainage Device from Vas Deferens, Percutaneous Approach */
"0VPR33Z"  =  "1"  /* Removal of Infusion Device from Vas Deferens, Percutaneous Approach */
"0VPR37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Vas Deferens, Percutaneous Approach */
"0VPR3CZ"  =  "1"  /* Removal of Extraluminal Device from Vas Deferens, Percutaneous Approach */
"0VPR3JZ"  =  "1"  /* Removal of Synthetic Substitute from Vas Deferens, Percutaneous Approach */
"0VPR3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Vas Deferens, Percutaneous Approach */
"0VPR40Z"  =  "1"  /* Removal of Drainage Device from Vas Deferens, Percutaneous Endoscopic Approach */
"0VPR43Z"  =  "1"  /* Removal of Infusion Device from Vas Deferens, Percutaneous Endoscopic Approach */
"0VPR47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Vas Deferens, Percutaneous Endoscopic Approach */
"0VPR4CZ"  =  "1"  /* Removal of Extraluminal Device from Vas Deferens, Percutaneous Endoscopic Approach */
"0VPR4JZ"  =  "1"  /* Removal of Synthetic Substitute from Vas Deferens, Percutaneous Endoscopic Approach */
"0VPR4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Vas Deferens, Percutaneous Endoscopic Approach */
"0VPR70Z"  =  "1"  /* Removal of Drainage Device from Vas Deferens, Via Natural or Artificial Opening */
"0VPR73Z"  =  "1"  /* Removal of Infusion Device from Vas Deferens, Via Natural or Artificial Opening */
"0VPR77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Vas Deferens, Via Natural or Artificial Opening */
"0VPR7CZ"  =  "1"  /* Removal of Extraluminal Device from Vas Deferens, Via Natural or Artificial Opening */
"0VPR7JZ"  =  "1"  /* Removal of Synthetic Substitute from Vas Deferens, Via Natural or Artificial Opening */
"0VPR7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Vas Deferens, Via Natural or Artificial Opening */
"0VPR80Z"  =  "1"  /* Removal of Drainage Device from Vas Deferens, Via Natural or Artificial Opening Endoscopic */
"0VPR83Z"  =  "1"  /* Removal of Infusion Device from Vas Deferens, Via Natural or Artificial Opening Endoscopic */
"0VPR87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Vas Deferens, Via Natural or Artificial Opening Endoscopic */
"0VPR8CZ"  =  "1"  /* Removal of Extraluminal Device from Vas Deferens, Via Natural or Artificial Opening Endoscopic */
"0VPR8JZ"  =  "1"  /* Removal of Synthetic Substitute from Vas Deferens, Via Natural or Artificial Opening Endoscopic */
"0VPR8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Vas Deferens, Via Natural or Artificial Opening Endoscopic */
"0VPS00Z"  =  "1"  /* Removal of Drainage Device from Penis, Open Approach */
"0VPS03Z"  =  "1"  /* Removal of Infusion Device from Penis, Open Approach */
"0VPS07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Penis, Open Approach */
"0VPS0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Penis, Open Approach */
"0VPS30Z"  =  "1"  /* Removal of Drainage Device from Penis, Percutaneous Approach */
"0VPS33Z"  =  "1"  /* Removal of Infusion Device from Penis, Percutaneous Approach */
"0VPS37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Penis, Percutaneous Approach */
"0VPS3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Penis, Percutaneous Approach */
"0VPS40Z"  =  "1"  /* Removal of Drainage Device from Penis, Percutaneous Endoscopic Approach */
"0VPS43Z"  =  "1"  /* Removal of Infusion Device from Penis, Percutaneous Endoscopic Approach */
"0VPS47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Penis, Percutaneous Endoscopic Approach */
"0VPS4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Penis, Percutaneous Endoscopic Approach */
"0VPS70Z"  =  "1"  /* Removal of Drainage Device from Penis, Via Natural or Artificial Opening */
"0VPS73Z"  =  "1"  /* Removal of Infusion Device from Penis, Via Natural or Artificial Opening */
"0VPS77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Penis, Via Natural or Artificial Opening */
"0VPS7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Penis, Via Natural or Artificial Opening */
"0VPS80Z"  =  "1"  /* Removal of Drainage Device from Penis, Via Natural or Artificial Opening Endoscopic */
"0VPS83Z"  =  "1"  /* Removal of Infusion Device from Penis, Via Natural or Artificial Opening Endoscopic */
"0VPS87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Penis, Via Natural or Artificial Opening Endoscopic */
"0VPS8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Penis, Via Natural or Artificial Opening Endoscopic */
"0VW400Z"  =  "1"  /* Revision of Drainage Device in Prostate and Seminal Vesicles, Open Approach */
"0VW403Z"  =  "1"  /* Revision of Infusion Device in Prostate and Seminal Vesicles, Open Approach */
"0VW407Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Prostate and Seminal Vesicles, Open Approach */
"0VW40JZ"  =  "1"  /* Revision of Synthetic Substitute in Prostate and Seminal Vesicles, Open Approach */
"0VW40KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Prostate and Seminal Vesicles, Open Approach */
"0VW430Z"  =  "1"  /* Revision of Drainage Device in Prostate and Seminal Vesicles, Percutaneous Approach */
"0VW433Z"  =  "1"  /* Revision of Infusion Device in Prostate and Seminal Vesicles, Percutaneous Approach */
"0VW437Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Prostate and Seminal Vesicles, Percutaneous Approach */
"0VW43JZ"  =  "1"  /* Revision of Synthetic Substitute in Prostate and Seminal Vesicles, Percutaneous Approach */
"0VW43KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Prostate and Seminal Vesicles, Percutaneous Approach */
"0VW440Z"  =  "1"  /* Revision of Drainage Device in Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VW443Z"  =  "1"  /* Revision of Infusion Device in Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VW447Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VW44JZ"  =  "1"  /* Revision of Synthetic Substitute in Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VW44KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VW470Z"  =  "1"  /* Revision of Drainage Device in Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VW473Z"  =  "1"  /* Revision of Infusion Device in Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VW477Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VW47JZ"  =  "1"  /* Revision of Synthetic Substitute in Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VW47KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VW480Z"  =  "1"  /* Revision of Drainage Device in Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0VW483Z"  =  "1"  /* Revision of Infusion Device in Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0VW487Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0VW48JZ"  =  "1"  /* Revision of Synthetic Substitute in Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0VW48KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0VW800Z"  =  "1"  /* Revision of Drainage Device in Scrotum and Tunica Vaginalis, Open Approach */
"0VW803Z"  =  "1"  /* Revision of Infusion Device in Scrotum and Tunica Vaginalis, Open Approach */
"0VW807Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Scrotum and Tunica Vaginalis, Open Approach */
"0VW80JZ"  =  "1"  /* Revision of Synthetic Substitute in Scrotum and Tunica Vaginalis, Open Approach */
"0VW80KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Scrotum and Tunica Vaginalis, Open Approach */
"0VW830Z"  =  "1"  /* Revision of Drainage Device in Scrotum and Tunica Vaginalis, Percutaneous Approach */
"0VW833Z"  =  "1"  /* Revision of Infusion Device in Scrotum and Tunica Vaginalis, Percutaneous Approach */
"0VW837Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Scrotum and Tunica Vaginalis, Percutaneous Approach */
"0VW83JZ"  =  "1"  /* Revision of Synthetic Substitute in Scrotum and Tunica Vaginalis, Percutaneous Approach */
"0VW83KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Scrotum and Tunica Vaginalis, Percutaneous Approach */
"0VW840Z"  =  "1"  /* Revision of Drainage Device in Scrotum and Tunica Vaginalis, Percutaneous Endoscopic Approach */
"0VW843Z"  =  "1"  /* Revision of Infusion Device in Scrotum and Tunica Vaginalis, Percutaneous Endoscopic Approach */
"0VW847Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Scrotum and Tunica Vaginalis, Percutaneous Endoscopic Approach */
"0VW84JZ"  =  "1"  /* Revision of Synthetic Substitute in Scrotum and Tunica Vaginalis, Percutaneous Endoscopic Approach */
"0VW84KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Scrotum and Tunica Vaginalis, Percutaneous Endoscopic Approach */
"0VW870Z"  =  "1"  /* Revision of Drainage Device in Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening */
"0VW873Z"  =  "1"  /* Revision of Infusion Device in Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening */
"0VW877Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening */
"0VW87JZ"  =  "1"  /* Revision of Synthetic Substitute in Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening */
"0VW87KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening */
"0VW880Z"  =  "1"  /* Revision of Drainage Device in Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening Endoscopic */
"0VW883Z"  =  "1"  /* Revision of Infusion Device in Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening Endoscopic */
"0VW887Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening Endoscopic */
"0VW88JZ"  =  "1"  /* Revision of Synthetic Substitute in Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening Endoscopic */
"0VW88KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Scrotum and Tunica Vaginalis, Via Natural or Artificial Opening Endoscopic */
"0VWD00Z"  =  "1"  /* Revision of Drainage Device in Testis, Open Approach */
"0VWD03Z"  =  "1"  /* Revision of Infusion Device in Testis, Open Approach */
"0VWD07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Testis, Open Approach */
"0VWD0JZ"  =  "1"  /* Revision of Synthetic Substitute in Testis, Open Approach */
"0VWD0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Testis, Open Approach */
"0VWD30Z"  =  "1"  /* Revision of Drainage Device in Testis, Percutaneous Approach */
"0VWD33Z"  =  "1"  /* Revision of Infusion Device in Testis, Percutaneous Approach */
"0VWD37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Testis, Percutaneous Approach */
"0VWD3JZ"  =  "1"  /* Revision of Synthetic Substitute in Testis, Percutaneous Approach */
"0VWD3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Testis, Percutaneous Approach */
"0VWD40Z"  =  "1"  /* Revision of Drainage Device in Testis, Percutaneous Endoscopic Approach */
"0VWD43Z"  =  "1"  /* Revision of Infusion Device in Testis, Percutaneous Endoscopic Approach */
"0VWD47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Testis, Percutaneous Endoscopic Approach */
"0VWD4JZ"  =  "1"  /* Revision of Synthetic Substitute in Testis, Percutaneous Endoscopic Approach */
"0VWD4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Testis, Percutaneous Endoscopic Approach */
"0VWD70Z"  =  "1"  /* Revision of Drainage Device in Testis, Via Natural or Artificial Opening */
"0VWD73Z"  =  "1"  /* Revision of Infusion Device in Testis, Via Natural or Artificial Opening */
"0VWD77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Testis, Via Natural or Artificial Opening */
"0VWD7JZ"  =  "1"  /* Revision of Synthetic Substitute in Testis, Via Natural or Artificial Opening */
"0VWD7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Testis, Via Natural or Artificial Opening */
"0VWD80Z"  =  "1"  /* Revision of Drainage Device in Testis, Via Natural or Artificial Opening Endoscopic */
"0VWD83Z"  =  "1"  /* Revision of Infusion Device in Testis, Via Natural or Artificial Opening Endoscopic */
"0VWD87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Testis, Via Natural or Artificial Opening Endoscopic */
"0VWD8JZ"  =  "1"  /* Revision of Synthetic Substitute in Testis, Via Natural or Artificial Opening Endoscopic */
"0VWD8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Testis, Via Natural or Artificial Opening Endoscopic */
"0VWM00Z"  =  "1"  /* Revision of Drainage Device in Epididymis and Spermatic Cord, Open Approach */
"0VWM03Z"  =  "1"  /* Revision of Infusion Device in Epididymis and Spermatic Cord, Open Approach */
"0VWM07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Epididymis and Spermatic Cord, Open Approach */
"0VWM0CZ"  =  "1"  /* Revision of Extraluminal Device in Epididymis and Spermatic Cord, Open Approach */
"0VWM0JZ"  =  "1"  /* Revision of Synthetic Substitute in Epididymis and Spermatic Cord, Open Approach */
"0VWM0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Epididymis and Spermatic Cord, Open Approach */
"0VWM30Z"  =  "1"  /* Revision of Drainage Device in Epididymis and Spermatic Cord, Percutaneous Approach */
"0VWM33Z"  =  "1"  /* Revision of Infusion Device in Epididymis and Spermatic Cord, Percutaneous Approach */
"0VWM37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Epididymis and Spermatic Cord, Percutaneous Approach */
"0VWM3CZ"  =  "1"  /* Revision of Extraluminal Device in Epididymis and Spermatic Cord, Percutaneous Approach */
"0VWM3JZ"  =  "1"  /* Revision of Synthetic Substitute in Epididymis and Spermatic Cord, Percutaneous Approach */
"0VWM3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Epididymis and Spermatic Cord, Percutaneous Approach */
"0VWM40Z"  =  "1"  /* Revision of Drainage Device in Epididymis and Spermatic Cord, Percutaneous Endoscopic Approach */
"0VWM43Z"  =  "1"  /* Revision of Infusion Device in Epididymis and Spermatic Cord, Percutaneous Endoscopic Approach */
"0VWM47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Epididymis and Spermatic Cord, Percutaneous Endoscopic Approach */
"0VWM4CZ"  =  "1"  /* Revision of Extraluminal Device in Epididymis and Spermatic Cord, Percutaneous Endoscopic Approach */
"0VWM4JZ"  =  "1"  /* Revision of Synthetic Substitute in Epididymis and Spermatic Cord, Percutaneous Endoscopic Approach */
"0VWM4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Epididymis and Spermatic Cord, Percutaneous Endoscopic Approach */
"0VWM70Z"  =  "1"  /* Revision of Drainage Device in Epididymis and Spermatic Cord, Via Natural or Artificial Opening */
"0VWM73Z"  =  "1"  /* Revision of Infusion Device in Epididymis and Spermatic Cord, Via Natural or Artificial Opening */
"0VWM77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Epididymis and Spermatic Cord, Via Natural or Artificial Opening */
"0VWM7CZ"  =  "1"  /* Revision of Extraluminal Device in Epididymis and Spermatic Cord, Via Natural or Artificial Opening */
"0VWM7JZ"  =  "1"  /* Revision of Synthetic Substitute in Epididymis and Spermatic Cord, Via Natural or Artificial Opening */
"0VWM7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Epididymis and Spermatic Cord, Via Natural or Artificial Opening */
"0VWM80Z"  =  "1"  /* Revision of Drainage Device in Epididymis and Spermatic Cord, Via Natural or Artificial Opening Endoscopic */
"0VWM83Z"  =  "1"  /* Revision of Infusion Device in Epididymis and Spermatic Cord, Via Natural or Artificial Opening Endoscopic */
"0VWM87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Epididymis and Spermatic Cord, Via Natural or Artificial Opening Endoscopic */
"0VWM8CZ"  =  "1"  /* Revision of Extraluminal Device in Epididymis and Spermatic Cord, Via Natural or Artificial Opening Endoscopic */
"0VWM8JZ"  =  "1"  /* Revision of Synthetic Substitute in Epididymis and Spermatic Cord, Via Natural or Artificial Opening Endoscopic */
"0VWM8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Epididymis and Spermatic Cord, Via Natural or Artificial Opening Endoscopic */
"0VWR00Z"  =  "1"  /* Revision of Drainage Device in Vas Deferens, Open Approach */
"0VWR03Z"  =  "1"  /* Revision of Infusion Device in Vas Deferens, Open Approach */
"0VWR07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Vas Deferens, Open Approach */
"0VWR0CZ"  =  "1"  /* Revision of Extraluminal Device in Vas Deferens, Open Approach */
"0VWR0DZ"  =  "1"  /* Revision of Intraluminal Device in Vas Deferens, Open Approach */
"0VWR0JZ"  =  "1"  /* Revision of Synthetic Substitute in Vas Deferens, Open Approach */
"0VWR0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Vas Deferens, Open Approach */
"0VWR30Z"  =  "1"  /* Revision of Drainage Device in Vas Deferens, Percutaneous Approach */
"0VWR33Z"  =  "1"  /* Revision of Infusion Device in Vas Deferens, Percutaneous Approach */
"0VWR37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Vas Deferens, Percutaneous Approach */
"0VWR3CZ"  =  "1"  /* Revision of Extraluminal Device in Vas Deferens, Percutaneous Approach */
"0VWR3DZ"  =  "1"  /* Revision of Intraluminal Device in Vas Deferens, Percutaneous Approach */
"0VWR3JZ"  =  "1"  /* Revision of Synthetic Substitute in Vas Deferens, Percutaneous Approach */
"0VWR3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Vas Deferens, Percutaneous Approach */
"0VWR40Z"  =  "1"  /* Revision of Drainage Device in Vas Deferens, Percutaneous Endoscopic Approach */
"0VWR43Z"  =  "1"  /* Revision of Infusion Device in Vas Deferens, Percutaneous Endoscopic Approach */
"0VWR47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Vas Deferens, Percutaneous Endoscopic Approach */
"0VWR4CZ"  =  "1"  /* Revision of Extraluminal Device in Vas Deferens, Percutaneous Endoscopic Approach */
"0VWR4DZ"  =  "1"  /* Revision of Intraluminal Device in Vas Deferens, Percutaneous Endoscopic Approach */
"0VWR4JZ"  =  "1"  /* Revision of Synthetic Substitute in Vas Deferens, Percutaneous Endoscopic Approach */
"0VWR4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Vas Deferens, Percutaneous Endoscopic Approach */
"0VWR70Z"  =  "1"  /* Revision of Drainage Device in Vas Deferens, Via Natural or Artificial Opening */
"0VWR73Z"  =  "1"  /* Revision of Infusion Device in Vas Deferens, Via Natural or Artificial Opening */
"0VWR77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Vas Deferens, Via Natural or Artificial Opening */
"0VWR7CZ"  =  "1"  /* Revision of Extraluminal Device in Vas Deferens, Via Natural or Artificial Opening */
"0VWR7DZ"  =  "1"  /* Revision of Intraluminal Device in Vas Deferens, Via Natural or Artificial Opening */
"0VWR7JZ"  =  "1"  /* Revision of Synthetic Substitute in Vas Deferens, Via Natural or Artificial Opening */
"0VWR7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Vas Deferens, Via Natural or Artificial Opening */
"0VWR80Z"  =  "1"  /* Revision of Drainage Device in Vas Deferens, Via Natural or Artificial Opening Endoscopic */
"0VWR83Z"  =  "1"  /* Revision of Infusion Device in Vas Deferens, Via Natural or Artificial Opening Endoscopic */
"0VWR87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Vas Deferens, Via Natural or Artificial Opening Endoscopic */
"0VWR8CZ"  =  "1"  /* Revision of Extraluminal Device in Vas Deferens, Via Natural or Artificial Opening Endoscopic */
"0VWR8DZ"  =  "1"  /* Revision of Intraluminal Device in Vas Deferens, Via Natural or Artificial Opening Endoscopic */
"0VWR8JZ"  =  "1"  /* Revision of Synthetic Substitute in Vas Deferens, Via Natural or Artificial Opening Endoscopic */
"0VWR8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Vas Deferens, Via Natural or Artificial Opening Endoscopic */
"0VWS00Z"  =  "1"  /* Revision of Drainage Device in Penis, Open Approach */
"0VWS03Z"  =  "1"  /* Revision of Infusion Device in Penis, Open Approach */
"0VWS07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Penis, Open Approach */
"0VWS0JZ"  =  "1"  /* Revision of Synthetic Substitute in Penis, Open Approach */
"0VWS0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Penis, Open Approach */
"0VWS30Z"  =  "1"  /* Revision of Drainage Device in Penis, Percutaneous Approach */
"0VWS33Z"  =  "1"  /* Revision of Infusion Device in Penis, Percutaneous Approach */
"0VWS37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Penis, Percutaneous Approach */
"0VWS3JZ"  =  "1"  /* Revision of Synthetic Substitute in Penis, Percutaneous Approach */
"0VWS3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Penis, Percutaneous Approach */
"0VWS40Z"  =  "1"  /* Revision of Drainage Device in Penis, Percutaneous Endoscopic Approach */
"0VWS43Z"  =  "1"  /* Revision of Infusion Device in Penis, Percutaneous Endoscopic Approach */
"0VWS47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Penis, Percutaneous Endoscopic Approach */
"0VWS4JZ"  =  "1"  /* Revision of Synthetic Substitute in Penis, Percutaneous Endoscopic Approach */
"0VWS4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Penis, Percutaneous Endoscopic Approach */
"0VWS70Z"  =  "1"  /* Revision of Drainage Device in Penis, Via Natural or Artificial Opening */
"0VWS73Z"  =  "1"  /* Revision of Infusion Device in Penis, Via Natural or Artificial Opening */
"0VWS77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Penis, Via Natural or Artificial Opening */
"0VWS7JZ"  =  "1"  /* Revision of Synthetic Substitute in Penis, Via Natural or Artificial Opening */
"0VWS7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Penis, Via Natural or Artificial Opening */
"0VWS80Z"  =  "1"  /* Revision of Drainage Device in Penis, Via Natural or Artificial Opening Endoscopic */
"0VWS83Z"  =  "1"  /* Revision of Infusion Device in Penis, Via Natural or Artificial Opening Endoscopic */
"0VWS87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Penis, Via Natural or Artificial Opening Endoscopic */
"0VWS8JZ"  =  "1"  /* Revision of Synthetic Substitute in Penis, Via Natural or Artificial Opening Endoscopic */
"0VWS8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Penis, Via Natural or Artificial Opening Endoscopic */
"0W190J4"  =  "1"  /* Bypass Right Pleural Cavity to Cutaneous with Synthetic Substitute, Open Approach */
"0W190JY"  =  "1"  /* Bypass Right Pleural Cavity to Lower Vein with Synthetic Substitute, Open Approach */
"0W193J4"  =  "1"  /* Bypass Right Pleural Cavity to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0W194J4"  =  "1"  /* Bypass Right Pleural Cavity to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W194JY"  =  "1"  /* Bypass Right Pleural Cavity to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W1B0J4"  =  "1"  /* Bypass Left Pleural Cavity to Cutaneous with Synthetic Substitute, Open Approach */
"0W1B0JY"  =  "1"  /* Bypass Left Pleural Cavity to Lower Vein with Synthetic Substitute, Open Approach */
"0W1B3J4"  =  "1"  /* Bypass Left Pleural Cavity to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0W1B4J4"  =  "1"  /* Bypass Left Pleural Cavity to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W1B4JY"  =  "1"  /* Bypass Left Pleural Cavity to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W300ZZ"  =  "1"  /* Control Bleeding in Head, Open Approach */
"0W303ZZ"  =  "1"  /* Control Bleeding in Head, Percutaneous Approach */
"0W304ZZ"  =  "1"  /* Control Bleeding in Head, Percutaneous Endoscopic Approach */
"0W310ZZ"  =  "1"  /* Control Bleeding in Cranial Cavity, Open Approach */
"0W313ZZ"  =  "1"  /* Control Bleeding in Cranial Cavity, Percutaneous Approach */
"0W314ZZ"  =  "1"  /* Control Bleeding in Cranial Cavity, Percutaneous Endoscopic Approach */
"0W320ZZ"  =  "1"  /* Control Bleeding in Face, Open Approach */
"0W323ZZ"  =  "1"  /* Control Bleeding in Face, Percutaneous Approach */
"0W324ZZ"  =  "1"  /* Control Bleeding in Face, Percutaneous Endoscopic Approach */
"0W330ZZ"  =  "1"  /* Control Bleeding in Oral Cavity and Throat, Open Approach */
"0W333ZZ"  =  "1"  /* Control Bleeding in Oral Cavity and Throat, Percutaneous Approach */
"0W334ZZ"  =  "1"  /* Control Bleeding in Oral Cavity and Throat, Percutaneous Endoscopic Approach */
"0W337ZZ"  =  "1"  /* Control Bleeding in Oral Cavity and Throat, Via Natural or Artificial Opening */
"0W338ZZ"  =  "1"  /* Control Bleeding in Oral Cavity and Throat, Via Natural or Artificial Opening Endoscopic */
"0W33XZZ"  =  "1"  /* Control Bleeding in Oral Cavity and Throat, External Approach */
"0W340ZZ"  =  "1"  /* Control Bleeding in Upper Jaw, Open Approach */
"0W343ZZ"  =  "1"  /* Control Bleeding in Upper Jaw, Percutaneous Approach */
"0W344ZZ"  =  "1"  /* Control Bleeding in Upper Jaw, Percutaneous Endoscopic Approach */
"0W350ZZ"  =  "1"  /* Control Bleeding in Lower Jaw, Open Approach */
"0W353ZZ"  =  "1"  /* Control Bleeding in Lower Jaw, Percutaneous Approach */
"0W354ZZ"  =  "1"  /* Control Bleeding in Lower Jaw, Percutaneous Endoscopic Approach */
"0W360ZZ"  =  "1"  /* Control Bleeding in Neck, Open Approach */
"0W363ZZ"  =  "1"  /* Control Bleeding in Neck, Percutaneous Approach */
"0W364ZZ"  =  "1"  /* Control Bleeding in Neck, Percutaneous Endoscopic Approach */
"0W380ZZ"  =  "1"  /* Control Bleeding in Chest Wall, Open Approach */
"0W383ZZ"  =  "1"  /* Control Bleeding in Chest Wall, Percutaneous Approach */
"0W384ZZ"  =  "1"  /* Control Bleeding in Chest Wall, Percutaneous Endoscopic Approach */
"0W390ZZ"  =  "1"  /* Control Bleeding in Right Pleural Cavity, Open Approach */
"0W393ZZ"  =  "1"  /* Control Bleeding in Right Pleural Cavity, Percutaneous Approach */
"0W394ZZ"  =  "1"  /* Control Bleeding in Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0W3B0ZZ"  =  "1"  /* Control Bleeding in Left Pleural Cavity, Open Approach */
"0W3B3ZZ"  =  "1"  /* Control Bleeding in Left Pleural Cavity, Percutaneous Approach */
"0W3B4ZZ"  =  "1"  /* Control Bleeding in Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0W3C0ZZ"  =  "1"  /* Control Bleeding in Mediastinum, Open Approach */
"0W3C3ZZ"  =  "1"  /* Control Bleeding in Mediastinum, Percutaneous Approach */
"0W3C4ZZ"  =  "1"  /* Control Bleeding in Mediastinum, Percutaneous Endoscopic Approach */
"0W3D0ZZ"  =  "1"  /* Control Bleeding in Pericardial Cavity, Open Approach */
"0W3D3ZZ"  =  "1"  /* Control Bleeding in Pericardial Cavity, Percutaneous Approach */
"0W3D4ZZ"  =  "1"  /* Control Bleeding in Pericardial Cavity, Percutaneous Endoscopic Approach */
"0W3F0ZZ"  =  "1"  /* Control Bleeding in Abdominal Wall, Open Approach */
"0W3F3ZZ"  =  "1"  /* Control Bleeding in Abdominal Wall, Percutaneous Approach */
"0W3F4ZZ"  =  "1"  /* Control Bleeding in Abdominal Wall, Percutaneous Endoscopic Approach */
"0W3G0ZZ"  =  "1"  /* Control Bleeding in Peritoneal Cavity, Open Approach */
"0W3G3ZZ"  =  "1"  /* Control Bleeding in Peritoneal Cavity, Percutaneous Approach */
"0W3G4ZZ"  =  "1"  /* Control Bleeding in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0W3H0ZZ"  =  "1"  /* Control Bleeding in Retroperitoneum, Open Approach */
"0W3H3ZZ"  =  "1"  /* Control Bleeding in Retroperitoneum, Percutaneous Approach */
"0W3H4ZZ"  =  "1"  /* Control Bleeding in Retroperitoneum, Percutaneous Endoscopic Approach */
"0W3J0ZZ"  =  "1"  /* Control Bleeding in Pelvic Cavity, Open Approach */
"0W3J3ZZ"  =  "1"  /* Control Bleeding in Pelvic Cavity, Percutaneous Approach */
"0W3J4ZZ"  =  "1"  /* Control Bleeding in Pelvic Cavity, Percutaneous Endoscopic Approach */
"0W3K0ZZ"  =  "1"  /* Control Bleeding in Upper Back, Open Approach */
"0W3K3ZZ"  =  "1"  /* Control Bleeding in Upper Back, Percutaneous Approach */
"0W3K4ZZ"  =  "1"  /* Control Bleeding in Upper Back, Percutaneous Endoscopic Approach */
"0W3L0ZZ"  =  "1"  /* Control Bleeding in Lower Back, Open Approach */
"0W3L3ZZ"  =  "1"  /* Control Bleeding in Lower Back, Percutaneous Approach */
"0W3L4ZZ"  =  "1"  /* Control Bleeding in Lower Back, Percutaneous Endoscopic Approach */
"0W3M0ZZ"  =  "1"  /* Control Bleeding in Male Perineum, Open Approach */
"0W3M3ZZ"  =  "1"  /* Control Bleeding in Male Perineum, Percutaneous Approach */
"0W3M4ZZ"  =  "1"  /* Control Bleeding in Male Perineum, Percutaneous Endoscopic Approach */
"0W3N0ZZ"  =  "1"  /* Control Bleeding in Female Perineum, Open Approach */
"0W3N3ZZ"  =  "1"  /* Control Bleeding in Female Perineum, Percutaneous Approach */
"0W3N4ZZ"  =  "1"  /* Control Bleeding in Female Perineum, Percutaneous Endoscopic Approach */
"0W3P0ZZ"  =  "1"  /* Control Bleeding in Gastrointestinal Tract, Open Approach */
"0W3P3ZZ"  =  "1"  /* Control Bleeding in Gastrointestinal Tract, Percutaneous Approach */
"0W3P4ZZ"  =  "1"  /* Control Bleeding in Gastrointestinal Tract, Percutaneous Endoscopic Approach */
"0W3P7ZZ"  =  "1"  /* Control Bleeding in Gastrointestinal Tract, Via Natural or Artificial Opening */
"0W3P8ZZ"  =  "1"  /* Control Bleeding in Gastrointestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0W3Q0ZZ"  =  "1"  /* Control Bleeding in Respiratory Tract, Open Approach */
"0W3Q3ZZ"  =  "1"  /* Control Bleeding in Respiratory Tract, Percutaneous Approach */
"0W3Q4ZZ"  =  "1"  /* Control Bleeding in Respiratory Tract, Percutaneous Endoscopic Approach */
"0W3Q7ZZ"  =  "1"  /* Control Bleeding in Respiratory Tract, Via Natural or Artificial Opening */
"0W3Q8ZZ"  =  "1"  /* Control Bleeding in Respiratory Tract, Via Natural or Artificial Opening Endoscopic */
"0W3R0ZZ"  =  "1"  /* Control Bleeding in Genitourinary Tract, Open Approach */
"0W3R3ZZ"  =  "1"  /* Control Bleeding in Genitourinary Tract, Percutaneous Approach */
"0W3R4ZZ"  =  "1"  /* Control Bleeding in Genitourinary Tract, Percutaneous Endoscopic Approach */
"0W3R7ZZ"  =  "1"  /* Control Bleeding in Genitourinary Tract, Via Natural or Artificial Opening */
"0W3R8ZZ"  =  "1"  /* Control Bleeding in Genitourinary Tract, Via Natural or Artificial Opening Endoscopic */
"0W9000Z"  =  "1"  /* Drainage of Head with Drainage Device, Open Approach */
"0W900ZZ"  =  "1"  /* Drainage of Head, Open Approach */
"0W9030Z"  =  "1"  /* Drainage of Head with Drainage Device, Percutaneous Approach */
"0W9040Z"  =  "1"  /* Drainage of Head with Drainage Device, Percutaneous Endoscopic Approach */
"0W904ZZ"  =  "1"  /* Drainage of Head, Percutaneous Endoscopic Approach */
"0W9100Z"  =  "1"  /* Drainage of Cranial Cavity with Drainage Device, Open Approach */
"0W910ZZ"  =  "1"  /* Drainage of Cranial Cavity, Open Approach */
"0W9200Z"  =  "1"  /* Drainage of Face with Drainage Device, Open Approach */
"0W920ZZ"  =  "1"  /* Drainage of Face, Open Approach */
"0W9230Z"  =  "1"  /* Drainage of Face with Drainage Device, Percutaneous Approach */
"0W923ZZ"  =  "1"  /* Drainage of Face, Percutaneous Approach */
"0W9240Z"  =  "1"  /* Drainage of Face with Drainage Device, Percutaneous Endoscopic Approach */
"0W924ZZ"  =  "1"  /* Drainage of Face, Percutaneous Endoscopic Approach */
"0W9300Z"  =  "1"  /* Drainage of Oral Cavity and Throat with Drainage Device, Open Approach */
"0W930ZZ"  =  "1"  /* Drainage of Oral Cavity and Throat, Open Approach */
"0W9330Z"  =  "1"  /* Drainage of Oral Cavity and Throat with Drainage Device, Percutaneous Approach */
"0W933ZZ"  =  "1"  /* Drainage of Oral Cavity and Throat, Percutaneous Approach */
"0W9340Z"  =  "1"  /* Drainage of Oral Cavity and Throat with Drainage Device, Percutaneous Endoscopic Approach */
"0W934ZZ"  =  "1"  /* Drainage of Oral Cavity and Throat, Percutaneous Endoscopic Approach */
"0W9400Z"  =  "1"  /* Drainage of Upper Jaw with Drainage Device, Open Approach */
"0W940ZZ"  =  "1"  /* Drainage of Upper Jaw, Open Approach */
"0W9430Z"  =  "1"  /* Drainage of Upper Jaw with Drainage Device, Percutaneous Approach */
"0W943ZZ"  =  "1"  /* Drainage of Upper Jaw, Percutaneous Approach */
"0W9440Z"  =  "1"  /* Drainage of Upper Jaw with Drainage Device, Percutaneous Endoscopic Approach */
"0W944ZZ"  =  "1"  /* Drainage of Upper Jaw, Percutaneous Endoscopic Approach */
"0W9500Z"  =  "1"  /* Drainage of Lower Jaw with Drainage Device, Open Approach */
"0W950ZZ"  =  "1"  /* Drainage of Lower Jaw, Open Approach */
"0W9530Z"  =  "1"  /* Drainage of Lower Jaw with Drainage Device, Percutaneous Approach */
"0W953ZZ"  =  "1"  /* Drainage of Lower Jaw, Percutaneous Approach */
"0W9540Z"  =  "1"  /* Drainage of Lower Jaw with Drainage Device, Percutaneous Endoscopic Approach */
"0W954ZZ"  =  "1"  /* Drainage of Lower Jaw, Percutaneous Endoscopic Approach */
"0W9600Z"  =  "1"  /* Drainage of Neck with Drainage Device, Open Approach */
"0W960ZZ"  =  "1"  /* Drainage of Neck, Open Approach */
"0W9630Z"  =  "1"  /* Drainage of Neck with Drainage Device, Percutaneous Approach */
"0W963ZZ"  =  "1"  /* Drainage of Neck, Percutaneous Approach */
"0W9640Z"  =  "1"  /* Drainage of Neck with Drainage Device, Percutaneous Endoscopic Approach */
"0W964ZZ"  =  "1"  /* Drainage of Neck, Percutaneous Endoscopic Approach */
"0W9900Z"  =  "1"  /* Drainage of Right Pleural Cavity with Drainage Device, Open Approach */
"0W990ZZ"  =  "1"  /* Drainage of Right Pleural Cavity, Open Approach */
"0W994ZZ"  =  "1"  /* Drainage of Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0W9B00Z"  =  "1"  /* Drainage of Left Pleural Cavity with Drainage Device, Open Approach */
"0W9B0ZZ"  =  "1"  /* Drainage of Left Pleural Cavity, Open Approach */
"0W9B4ZZ"  =  "1"  /* Drainage of Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0W9C00Z"  =  "1"  /* Drainage of Mediastinum with Drainage Device, Open Approach */
"0W9C0ZZ"  =  "1"  /* Drainage of Mediastinum, Open Approach */
"0W9C30Z"  =  "1"  /* Drainage of Mediastinum with Drainage Device, Percutaneous Approach */
"0W9C3ZZ"  =  "1"  /* Drainage of Mediastinum, Percutaneous Approach */
"0W9C40Z"  =  "1"  /* Drainage of Mediastinum with Drainage Device, Percutaneous Endoscopic Approach */
"0W9C4ZZ"  =  "1"  /* Drainage of Mediastinum, Percutaneous Endoscopic Approach */
"0W9D30Z"  =  "1"  /* Drainage of Pericardial Cavity with Drainage Device, Percutaneous Approach */
"0W9D3ZX"  =  "1"  /* Drainage of Pericardial Cavity, Percutaneous Approach, Diagnostic */
"0W9D3ZZ"  =  "1"  /* Drainage of Pericardial Cavity, Percutaneous Approach */
"0W9D40Z"  =  "1"  /* Drainage of Pericardial Cavity with Drainage Device, Percutaneous Endoscopic Approach */
"0W9D4ZX"  =  "1"  /* Drainage of Pericardial Cavity, Percutaneous Endoscopic Approach, Diagnostic */
"0W9D4ZZ"  =  "1"  /* Drainage of Pericardial Cavity, Percutaneous Endoscopic Approach */
"0W9F00Z"  =  "1"  /* Drainage of Abdominal Wall with Drainage Device, Open Approach */
"0W9F0ZZ"  =  "1"  /* Drainage of Abdominal Wall, Open Approach */
"0W9G00Z"  =  "1"  /* Drainage of Peritoneal Cavity with Drainage Device, Open Approach */
"0W9G0ZZ"  =  "1"  /* Drainage of Peritoneal Cavity, Open Approach */
"0W9H00Z"  =  "1"  /* Drainage of Retroperitoneum with Drainage Device, Open Approach */
"0W9H0ZZ"  =  "1"  /* Drainage of Retroperitoneum, Open Approach */
"0W9H30Z"  =  "1"  /* Drainage of Retroperitoneum with Drainage Device, Percutaneous Approach */
"0W9H3ZZ"  =  "1"  /* Drainage of Retroperitoneum, Percutaneous Approach */
"0W9H40Z"  =  "1"  /* Drainage of Retroperitoneum with Drainage Device, Percutaneous Endoscopic Approach */
"0W9H4ZZ"  =  "1"  /* Drainage of Retroperitoneum, Percutaneous Endoscopic Approach */
"0W9J00Z"  =  "1"  /* Drainage of Pelvic Cavity with Drainage Device, Open Approach */
"0W9J0ZZ"  =  "1"  /* Drainage of Pelvic Cavity, Open Approach */
"0W9J40Z"  =  "1"  /* Drainage of Pelvic Cavity with Drainage Device, Percutaneous Endoscopic Approach */
"0W9J4ZZ"  =  "1"  /* Drainage of Pelvic Cavity, Percutaneous Endoscopic Approach */
"0W9K00Z"  =  "1"  /* Drainage of Upper Back with Drainage Device, Open Approach */
"0W9K0ZZ"  =  "1"  /* Drainage of Upper Back, Open Approach */
"0W9K30Z"  =  "1"  /* Drainage of Upper Back with Drainage Device, Percutaneous Approach */
"0W9K40Z"  =  "1"  /* Drainage of Upper Back with Drainage Device, Percutaneous Endoscopic Approach */
"0W9K4ZZ"  =  "1"  /* Drainage of Upper Back, Percutaneous Endoscopic Approach */
"0W9L00Z"  =  "1"  /* Drainage of Lower Back with Drainage Device, Open Approach */
"0W9L0ZZ"  =  "1"  /* Drainage of Lower Back, Open Approach */
"0W9L30Z"  =  "1"  /* Drainage of Lower Back with Drainage Device, Percutaneous Approach */
"0W9L40Z"  =  "1"  /* Drainage of Lower Back with Drainage Device, Percutaneous Endoscopic Approach */
"0W9L4ZZ"  =  "1"  /* Drainage of Lower Back, Percutaneous Endoscopic Approach */
"0W9M00Z"  =  "1"  /* Drainage of Male Perineum with Drainage Device, Open Approach */
"0W9M0ZZ"  =  "1"  /* Drainage of Male Perineum, Open Approach */
"0W9M30Z"  =  "1"  /* Drainage of Male Perineum with Drainage Device, Percutaneous Approach */
"0W9M40Z"  =  "1"  /* Drainage of Male Perineum with Drainage Device, Percutaneous Endoscopic Approach */
"0W9M4ZZ"  =  "1"  /* Drainage of Male Perineum, Percutaneous Endoscopic Approach */
"0W9N00Z"  =  "1"  /* Drainage of Female Perineum with Drainage Device, Open Approach */
"0W9N0ZZ"  =  "1"  /* Drainage of Female Perineum, Open Approach */
"0W9N30Z"  =  "1"  /* Drainage of Female Perineum with Drainage Device, Percutaneous Approach */
"0W9N3ZZ"  =  "1"  /* Drainage of Female Perineum, Percutaneous Approach */
"0W9N40Z"  =  "1"  /* Drainage of Female Perineum with Drainage Device, Percutaneous Endoscopic Approach */
"0W9N4ZZ"  =  "1"  /* Drainage of Female Perineum, Percutaneous Endoscopic Approach */
"0WC10ZZ"  =  "1"  /* Extirpation of Matter from Cranial Cavity, Open Approach */
"0WC13ZZ"  =  "1"  /* Extirpation of Matter from Cranial Cavity, Percutaneous Approach */
"0WC14ZZ"  =  "1"  /* Extirpation of Matter from Cranial Cavity, Percutaneous Endoscopic Approach */
"0WC90ZZ"  =  "1"  /* Extirpation of Matter from Right Pleural Cavity, Open Approach */
"0WC93ZZ"  =  "1"  /* Extirpation of Matter from Right Pleural Cavity, Percutaneous Approach */
"0WC94ZZ"  =  "1"  /* Extirpation of Matter from Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WCB0ZZ"  =  "1"  /* Extirpation of Matter from Left Pleural Cavity, Open Approach */
"0WCB3ZZ"  =  "1"  /* Extirpation of Matter from Left Pleural Cavity, Percutaneous Approach */
"0WCB4ZZ"  =  "1"  /* Extirpation of Matter from Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WCC0ZZ"  =  "1"  /* Extirpation of Matter from Mediastinum, Open Approach */
"0WCC3ZZ"  =  "1"  /* Extirpation of Matter from Mediastinum, Percutaneous Approach */
"0WCC4ZZ"  =  "1"  /* Extirpation of Matter from Mediastinum, Percutaneous Endoscopic Approach */
"0WCJ0ZZ"  =  "1"  /* Extirpation of Matter from Pelvic Cavity, Open Approach */
"0WCJ3ZZ"  =  "1"  /* Extirpation of Matter from Pelvic Cavity, Percutaneous Approach */
"0WCJ4ZZ"  =  "1"  /* Extirpation of Matter from Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WCP0ZZ"  =  "1"  /* Extirpation of Matter from Gastrointestinal Tract, Open Approach */
"0WCP3ZZ"  =  "1"  /* Extirpation of Matter from Gastrointestinal Tract, Percutaneous Approach */
"0WCP4ZZ"  =  "1"  /* Extirpation of Matter from Gastrointestinal Tract, Percutaneous Endoscopic Approach */
"0WCQ0ZZ"  =  "1"  /* Extirpation of Matter from Respiratory Tract, Open Approach */
"0WCQ3ZZ"  =  "1"  /* Extirpation of Matter from Respiratory Tract, Percutaneous Approach */
"0WCQ4ZZ"  =  "1"  /* Extirpation of Matter from Respiratory Tract, Percutaneous Endoscopic Approach */
"0WCR0ZZ"  =  "1"  /* Extirpation of Matter from Genitourinary Tract, Open Approach */
"0WCR3ZZ"  =  "1"  /* Extirpation of Matter from Genitourinary Tract, Percutaneous Approach */
"0WCR4ZZ"  =  "1"  /* Extirpation of Matter from Genitourinary Tract, Percutaneous Endoscopic Approach */
"0WH003Z"  =  "1"  /* Insertion of Infusion Device into Head, Open Approach */
"0WH00YZ"  =  "1"  /* Insertion of Other Device into Head, Open Approach */
"0WH033Z"  =  "1"  /* Insertion of Infusion Device into Head, Percutaneous Approach */
"0WH03YZ"  =  "1"  /* Insertion of Other Device into Head, Percutaneous Approach */
"0WH043Z"  =  "1"  /* Insertion of Infusion Device into Head, Percutaneous Endoscopic Approach */
"0WH04YZ"  =  "1"  /* Insertion of Other Device into Head, Percutaneous Endoscopic Approach */
"0WH10YZ"  =  "1"  /* Insertion of Other Device into Cranial Cavity, Open Approach */
"0WH13YZ"  =  "1"  /* Insertion of Other Device into Cranial Cavity, Percutaneous Approach */
"0WH14YZ"  =  "1"  /* Insertion of Other Device into Cranial Cavity, Percutaneous Endoscopic Approach */
"0WH203Z"  =  "1"  /* Insertion of Infusion Device into Face, Open Approach */
"0WH20YZ"  =  "1"  /* Insertion of Other Device into Face, Open Approach */
"0WH233Z"  =  "1"  /* Insertion of Infusion Device into Face, Percutaneous Approach */
"0WH23YZ"  =  "1"  /* Insertion of Other Device into Face, Percutaneous Approach */
"0WH243Z"  =  "1"  /* Insertion of Infusion Device into Face, Percutaneous Endoscopic Approach */
"0WH24YZ"  =  "1"  /* Insertion of Other Device into Face, Percutaneous Endoscopic Approach */
"0WH403Z"  =  "1"  /* Insertion of Infusion Device into Upper Jaw, Open Approach */
"0WH40YZ"  =  "1"  /* Insertion of Other Device into Upper Jaw, Open Approach */
"0WH433Z"  =  "1"  /* Insertion of Infusion Device into Upper Jaw, Percutaneous Approach */
"0WH43YZ"  =  "1"  /* Insertion of Other Device into Upper Jaw, Percutaneous Approach */
"0WH443Z"  =  "1"  /* Insertion of Infusion Device into Upper Jaw, Percutaneous Endoscopic Approach */
"0WH44YZ"  =  "1"  /* Insertion of Other Device into Upper Jaw, Percutaneous Endoscopic Approach */
"0WH503Z"  =  "1"  /* Insertion of Infusion Device into Lower Jaw, Open Approach */
"0WH50YZ"  =  "1"  /* Insertion of Other Device into Lower Jaw, Open Approach */
"0WH533Z"  =  "1"  /* Insertion of Infusion Device into Lower Jaw, Percutaneous Approach */
"0WH53YZ"  =  "1"  /* Insertion of Other Device into Lower Jaw, Percutaneous Approach */
"0WH543Z"  =  "1"  /* Insertion of Infusion Device into Lower Jaw, Percutaneous Endoscopic Approach */
"0WH54YZ"  =  "1"  /* Insertion of Other Device into Lower Jaw, Percutaneous Endoscopic Approach */
"0WH603Z"  =  "1"  /* Insertion of Infusion Device into Neck, Open Approach */
"0WH60YZ"  =  "1"  /* Insertion of Other Device into Neck, Open Approach */
"0WH633Z"  =  "1"  /* Insertion of Infusion Device into Neck, Percutaneous Approach */
"0WH63YZ"  =  "1"  /* Insertion of Other Device into Neck, Percutaneous Approach */
"0WH643Z"  =  "1"  /* Insertion of Infusion Device into Neck, Percutaneous Endoscopic Approach */
"0WH64YZ"  =  "1"  /* Insertion of Other Device into Neck, Percutaneous Endoscopic Approach */
"0WH903Z"  =  "1"  /* Insertion of Infusion Device into Right Pleural Cavity, Open Approach */
"0WH90YZ"  =  "1"  /* Insertion of Other Device into Right Pleural Cavity, Open Approach */
"0WH933Z"  =  "1"  /* Insertion of Infusion Device into Right Pleural Cavity, Percutaneous Approach */
"0WH93YZ"  =  "1"  /* Insertion of Other Device into Right Pleural Cavity, Percutaneous Approach */
"0WH943Z"  =  "1"  /* Insertion of Infusion Device into Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WH94YZ"  =  "1"  /* Insertion of Other Device into Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WHB03Z"  =  "1"  /* Insertion of Infusion Device into Left Pleural Cavity, Open Approach */
"0WHB0YZ"  =  "1"  /* Insertion of Other Device into Left Pleural Cavity, Open Approach */
"0WHB33Z"  =  "1"  /* Insertion of Infusion Device into Left Pleural Cavity, Percutaneous Approach */
"0WHB3YZ"  =  "1"  /* Insertion of Other Device into Left Pleural Cavity, Percutaneous Approach */
"0WHB43Z"  =  "1"  /* Insertion of Infusion Device into Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WHB4YZ"  =  "1"  /* Insertion of Other Device into Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WHC03Z"  =  "1"  /* Insertion of Infusion Device into Mediastinum, Open Approach */
"0WHC0YZ"  =  "1"  /* Insertion of Other Device into Mediastinum, Open Approach */
"0WHC33Z"  =  "1"  /* Insertion of Infusion Device into Mediastinum, Percutaneous Approach */
"0WHC3YZ"  =  "1"  /* Insertion of Other Device into Mediastinum, Percutaneous Approach */
"0WHC43Z"  =  "1"  /* Insertion of Infusion Device into Mediastinum, Percutaneous Endoscopic Approach */
"0WHC4YZ"  =  "1"  /* Insertion of Other Device into Mediastinum, Percutaneous Endoscopic Approach */
"0WHF03Z"  =  "1"  /* Insertion of Infusion Device into Abdominal Wall, Open Approach */
"0WHF0YZ"  =  "1"  /* Insertion of Other Device into Abdominal Wall, Open Approach */
"0WHF33Z"  =  "1"  /* Insertion of Infusion Device into Abdominal Wall, Percutaneous Approach */
"0WHF3YZ"  =  "1"  /* Insertion of Other Device into Abdominal Wall, Percutaneous Approach */
"0WHF43Z"  =  "1"  /* Insertion of Infusion Device into Abdominal Wall, Percutaneous Endoscopic Approach */
"0WHF4YZ"  =  "1"  /* Insertion of Other Device into Abdominal Wall, Percutaneous Endoscopic Approach */
"0WHG03Z"  =  "1"  /* Insertion of Infusion Device into Peritoneal Cavity, Open Approach */
"0WHG0YZ"  =  "1"  /* Insertion of Other Device into Peritoneal Cavity, Open Approach */
"0WHG33Z"  =  "1"  /* Insertion of Infusion Device into Peritoneal Cavity, Percutaneous Approach */
"0WHG3YZ"  =  "1"  /* Insertion of Other Device into Peritoneal Cavity, Percutaneous Approach */
"0WHG43Z"  =  "1"  /* Insertion of Infusion Device into Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WHG4YZ"  =  "1"  /* Insertion of Other Device into Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WHH03Z"  =  "1"  /* Insertion of Infusion Device into Retroperitoneum, Open Approach */
"0WHH0YZ"  =  "1"  /* Insertion of Other Device into Retroperitoneum, Open Approach */
"0WHH33Z"  =  "1"  /* Insertion of Infusion Device into Retroperitoneum, Percutaneous Approach */
"0WHH3YZ"  =  "1"  /* Insertion of Other Device into Retroperitoneum, Percutaneous Approach */
"0WHH43Z"  =  "1"  /* Insertion of Infusion Device into Retroperitoneum, Percutaneous Endoscopic Approach */
"0WHH4YZ"  =  "1"  /* Insertion of Other Device into Retroperitoneum, Percutaneous Endoscopic Approach */
"0WHJ03Z"  =  "1"  /* Insertion of Infusion Device into Pelvic Cavity, Open Approach */
"0WHJ0YZ"  =  "1"  /* Insertion of Other Device into Pelvic Cavity, Open Approach */
"0WHJ33Z"  =  "1"  /* Insertion of Infusion Device into Pelvic Cavity, Percutaneous Approach */
"0WHJ3YZ"  =  "1"  /* Insertion of Other Device into Pelvic Cavity, Percutaneous Approach */
"0WHJ43Z"  =  "1"  /* Insertion of Infusion Device into Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WHJ4YZ"  =  "1"  /* Insertion of Other Device into Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WHK03Z"  =  "1"  /* Insertion of Infusion Device into Upper Back, Open Approach */
"0WHK0YZ"  =  "1"  /* Insertion of Other Device into Upper Back, Open Approach */
"0WHK33Z"  =  "1"  /* Insertion of Infusion Device into Upper Back, Percutaneous Approach */
"0WHK3YZ"  =  "1"  /* Insertion of Other Device into Upper Back, Percutaneous Approach */
"0WHK43Z"  =  "1"  /* Insertion of Infusion Device into Upper Back, Percutaneous Endoscopic Approach */
"0WHK4YZ"  =  "1"  /* Insertion of Other Device into Upper Back, Percutaneous Endoscopic Approach */
"0WHL03Z"  =  "1"  /* Insertion of Infusion Device into Lower Back, Open Approach */
"0WHL0YZ"  =  "1"  /* Insertion of Other Device into Lower Back, Open Approach */
"0WHL33Z"  =  "1"  /* Insertion of Infusion Device into Lower Back, Percutaneous Approach */
"0WHL3YZ"  =  "1"  /* Insertion of Other Device into Lower Back, Percutaneous Approach */
"0WHL43Z"  =  "1"  /* Insertion of Infusion Device into Lower Back, Percutaneous Endoscopic Approach */
"0WHL4YZ"  =  "1"  /* Insertion of Other Device into Lower Back, Percutaneous Endoscopic Approach */
"0WHM03Z"  =  "1"  /* Insertion of Infusion Device into Male Perineum, Open Approach */
"0WHM0YZ"  =  "1"  /* Insertion of Other Device into Male Perineum, Open Approach */
"0WHM33Z"  =  "1"  /* Insertion of Infusion Device into Male Perineum, Percutaneous Approach */
"0WHM3YZ"  =  "1"  /* Insertion of Other Device into Male Perineum, Percutaneous Approach */
"0WHM43Z"  =  "1"  /* Insertion of Infusion Device into Male Perineum, Percutaneous Endoscopic Approach */
"0WHM4YZ"  =  "1"  /* Insertion of Other Device into Male Perineum, Percutaneous Endoscopic Approach */
"0WHN03Z"  =  "1"  /* Insertion of Infusion Device into Female Perineum, Open Approach */
"0WHN0YZ"  =  "1"  /* Insertion of Other Device into Female Perineum, Open Approach */
"0WHN33Z"  =  "1"  /* Insertion of Infusion Device into Female Perineum, Percutaneous Approach */
"0WHN3YZ"  =  "1"  /* Insertion of Other Device into Female Perineum, Percutaneous Approach */
"0WHN43Z"  =  "1"  /* Insertion of Infusion Device into Female Perineum, Percutaneous Endoscopic Approach */
"0WHN4YZ"  =  "1"  /* Insertion of Other Device into Female Perineum, Percutaneous Endoscopic Approach */
"0WHP03Z"  =  "1"  /* Insertion of Infusion Device into Gastrointestinal Tract, Open Approach */
"0WHQ03Z"  =  "1"  /* Insertion of Infusion Device into Respiratory Tract, Open Approach */
"0WHQ0YZ"  =  "1"  /* Insertion of Other Device into Respiratory Tract, Open Approach */
"0WJ00ZZ"  =  "1"  /* Inspection of Head, Open Approach */
"0WJ10ZZ"  =  "1"  /* Inspection of Cranial Cavity, Open Approach */
"0WJ20ZZ"  =  "1"  /* Inspection of Face, Open Approach */
"0WJ40ZZ"  =  "1"  /* Inspection of Upper Jaw, Open Approach */
"0WJ50ZZ"  =  "1"  /* Inspection of Lower Jaw, Open Approach */
"0WJ60ZZ"  =  "1"  /* Inspection of Neck, Open Approach */
"0WJ63ZZ"  =  "1"  /* Inspection of Neck, Percutaneous Approach */
"0WJ64ZZ"  =  "1"  /* Inspection of Neck, Percutaneous Endoscopic Approach */
"0WJ90ZZ"  =  "1"  /* Inspection of Right Pleural Cavity, Open Approach */
"0WJ94ZZ"  =  "1"  /* Inspection of Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WJB0ZZ"  =  "1"  /* Inspection of Left Pleural Cavity, Open Approach */
"0WJB4ZZ"  =  "1"  /* Inspection of Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WJC0ZZ"  =  "1"  /* Inspection of Mediastinum, Open Approach */
"0WJC4ZZ"  =  "1"  /* Inspection of Mediastinum, Percutaneous Endoscopic Approach */
"0WJD4ZZ"  =  "1"  /* Inspection of Pericardial Cavity, Percutaneous Endoscopic Approach */
"0WJF0ZZ"  =  "1"  /* Inspection of Abdominal Wall, Open Approach */
"0WJF4ZZ"  =  "1"  /* Inspection of Abdominal Wall, Percutaneous Endoscopic Approach */
"0WJG0ZZ"  =  "1"  /* Inspection of Peritoneal Cavity, Open Approach */
"0WJG4ZZ"  =  "1"  /* Inspection of Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WJH0ZZ"  =  "1"  /* Inspection of Retroperitoneum, Open Approach */
"0WJJ0ZZ"  =  "1"  /* Inspection of Pelvic Cavity, Open Approach */
"0WJJ3ZZ"  =  "1"  /* Inspection of Pelvic Cavity, Percutaneous Approach */
"0WJJ4ZZ"  =  "1"  /* Inspection of Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WJK0ZZ"  =  "1"  /* Inspection of Upper Back, Open Approach */
"0WJL0ZZ"  =  "1"  /* Inspection of Lower Back, Open Approach */
"0WJM0ZZ"  =  "1"  /* Inspection of Male Perineum, Open Approach */
"0WJM4ZZ"  =  "1"  /* Inspection of Male Perineum, Percutaneous Endoscopic Approach */
"0WJP0ZZ"  =  "1"  /* Inspection of Gastrointestinal Tract, Open Approach */
"0WJP4ZZ"  =  "1"  /* Inspection of Gastrointestinal Tract, Percutaneous Endoscopic Approach */
"0WJQ0ZZ"  =  "1"  /* Inspection of Respiratory Tract, Open Approach */
"0WJQ4ZZ"  =  "1"  /* Inspection of Respiratory Tract, Percutaneous Endoscopic Approach */
"0WJR0ZZ"  =  "1"  /* Inspection of Genitourinary Tract, Open Approach */
"0WJR4ZZ"  =  "1"  /* Inspection of Genitourinary Tract, Percutaneous Endoscopic Approach */
"0WP100Z"  =  "1"  /* Removal of Drainage Device from Cranial Cavity, Open Approach */
"0WP101Z"  =  "1"  /* Removal of Radioactive Element from Cranial Cavity, Open Approach */
"0WP10JZ"  =  "1"  /* Removal of Synthetic Substitute from Cranial Cavity, Open Approach */
"0WP10YZ"  =  "1"  /* Removal of Other Device from Cranial Cavity, Open Approach */
"0WP130Z"  =  "1"  /* Removal of Drainage Device from Cranial Cavity, Percutaneous Approach */
"0WP131Z"  =  "1"  /* Removal of Radioactive Element from Cranial Cavity, Percutaneous Approach */
"0WP13JZ"  =  "1"  /* Removal of Synthetic Substitute from Cranial Cavity, Percutaneous Approach */
"0WP13YZ"  =  "1"  /* Removal of Other Device from Cranial Cavity, Percutaneous Approach */
"0WP140Z"  =  "1"  /* Removal of Drainage Device from Cranial Cavity, Percutaneous Endoscopic Approach */
"0WP141Z"  =  "1"  /* Removal of Radioactive Element from Cranial Cavity, Percutaneous Endoscopic Approach */
"0WP14JZ"  =  "1"  /* Removal of Synthetic Substitute from Cranial Cavity, Percutaneous Endoscopic Approach */
"0WP14YZ"  =  "1"  /* Removal of Other Device from Cranial Cavity, Percutaneous Endoscopic Approach */
"0WP900Z"  =  "1"  /* Removal of Drainage Device from Right Pleural Cavity, Open Approach */
"0WP901Z"  =  "1"  /* Removal of Radioactive Element from Right Pleural Cavity, Open Approach */
"0WP903Z"  =  "1"  /* Removal of Infusion Device from Right Pleural Cavity, Open Approach */
"0WP90JZ"  =  "1"  /* Removal of Synthetic Substitute from Right Pleural Cavity, Open Approach */
"0WP90YZ"  =  "1"  /* Removal of Other Device from Right Pleural Cavity, Open Approach */
"0WP930Z"  =  "1"  /* Removal of Drainage Device from Right Pleural Cavity, Percutaneous Approach */
"0WP931Z"  =  "1"  /* Removal of Radioactive Element from Right Pleural Cavity, Percutaneous Approach */
"0WP933Z"  =  "1"  /* Removal of Infusion Device from Right Pleural Cavity, Percutaneous Approach */
"0WP93JZ"  =  "1"  /* Removal of Synthetic Substitute from Right Pleural Cavity, Percutaneous Approach */
"0WP93YZ"  =  "1"  /* Removal of Other Device from Right Pleural Cavity, Percutaneous Approach */
"0WP940Z"  =  "1"  /* Removal of Drainage Device from Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WP941Z"  =  "1"  /* Removal of Radioactive Element from Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WP943Z"  =  "1"  /* Removal of Infusion Device from Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WP94JZ"  =  "1"  /* Removal of Synthetic Substitute from Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WP94YZ"  =  "1"  /* Removal of Other Device from Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WPB00Z"  =  "1"  /* Removal of Drainage Device from Left Pleural Cavity, Open Approach */
"0WPB01Z"  =  "1"  /* Removal of Radioactive Element from Left Pleural Cavity, Open Approach */
"0WPB03Z"  =  "1"  /* Removal of Infusion Device from Left Pleural Cavity, Open Approach */
"0WPB0JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Pleural Cavity, Open Approach */
"0WPB0YZ"  =  "1"  /* Removal of Other Device from Left Pleural Cavity, Open Approach */
"0WPB30Z"  =  "1"  /* Removal of Drainage Device from Left Pleural Cavity, Percutaneous Approach */
"0WPB31Z"  =  "1"  /* Removal of Radioactive Element from Left Pleural Cavity, Percutaneous Approach */
"0WPB33Z"  =  "1"  /* Removal of Infusion Device from Left Pleural Cavity, Percutaneous Approach */
"0WPB3JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Pleural Cavity, Percutaneous Approach */
"0WPB3YZ"  =  "1"  /* Removal of Other Device from Left Pleural Cavity, Percutaneous Approach */
"0WPB40Z"  =  "1"  /* Removal of Drainage Device from Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WPB41Z"  =  "1"  /* Removal of Radioactive Element from Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WPB43Z"  =  "1"  /* Removal of Infusion Device from Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WPB4JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WPB4YZ"  =  "1"  /* Removal of Other Device from Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WPC00Z"  =  "1"  /* Removal of Drainage Device from Mediastinum, Open Approach */
"0WPC01Z"  =  "1"  /* Removal of Radioactive Element from Mediastinum, Open Approach */
"0WPC03Z"  =  "1"  /* Removal of Infusion Device from Mediastinum, Open Approach */
"0WPC07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mediastinum, Open Approach */
"0WPC0JZ"  =  "1"  /* Removal of Synthetic Substitute from Mediastinum, Open Approach */
"0WPC0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mediastinum, Open Approach */
"0WPC0YZ"  =  "1"  /* Removal of Other Device from Mediastinum, Open Approach */
"0WPC30Z"  =  "1"  /* Removal of Drainage Device from Mediastinum, Percutaneous Approach */
"0WPC31Z"  =  "1"  /* Removal of Radioactive Element from Mediastinum, Percutaneous Approach */
"0WPC33Z"  =  "1"  /* Removal of Infusion Device from Mediastinum, Percutaneous Approach */
"0WPC37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mediastinum, Percutaneous Approach */
"0WPC3JZ"  =  "1"  /* Removal of Synthetic Substitute from Mediastinum, Percutaneous Approach */
"0WPC3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mediastinum, Percutaneous Approach */
"0WPC3YZ"  =  "1"  /* Removal of Other Device from Mediastinum, Percutaneous Approach */
"0WPC40Z"  =  "1"  /* Removal of Drainage Device from Mediastinum, Percutaneous Endoscopic Approach */
"0WPC41Z"  =  "1"  /* Removal of Radioactive Element from Mediastinum, Percutaneous Endoscopic Approach */
"0WPC43Z"  =  "1"  /* Removal of Infusion Device from Mediastinum, Percutaneous Endoscopic Approach */
"0WPC47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mediastinum, Percutaneous Endoscopic Approach */
"0WPC4JZ"  =  "1"  /* Removal of Synthetic Substitute from Mediastinum, Percutaneous Endoscopic Approach */
"0WPC4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mediastinum, Percutaneous Endoscopic Approach */
"0WPC4YZ"  =  "1"  /* Removal of Other Device from Mediastinum, Percutaneous Endoscopic Approach */
"0WPF00Z"  =  "1"  /* Removal of Drainage Device from Abdominal Wall, Open Approach */
"0WPF01Z"  =  "1"  /* Removal of Radioactive Element from Abdominal Wall, Open Approach */
"0WPF03Z"  =  "1"  /* Removal of Infusion Device from Abdominal Wall, Open Approach */
"0WPF07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Abdominal Wall, Open Approach */
"0WPF0JZ"  =  "1"  /* Removal of Synthetic Substitute from Abdominal Wall, Open Approach */
"0WPF0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Abdominal Wall, Open Approach */
"0WPF0YZ"  =  "1"  /* Removal of Other Device from Abdominal Wall, Open Approach */
"0WPF30Z"  =  "1"  /* Removal of Drainage Device from Abdominal Wall, Percutaneous Approach */
"0WPF31Z"  =  "1"  /* Removal of Radioactive Element from Abdominal Wall, Percutaneous Approach */
"0WPF33Z"  =  "1"  /* Removal of Infusion Device from Abdominal Wall, Percutaneous Approach */
"0WPF37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Abdominal Wall, Percutaneous Approach */
"0WPF3JZ"  =  "1"  /* Removal of Synthetic Substitute from Abdominal Wall, Percutaneous Approach */
"0WPF3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Abdominal Wall, Percutaneous Approach */
"0WPF3YZ"  =  "1"  /* Removal of Other Device from Abdominal Wall, Percutaneous Approach */
"0WPF40Z"  =  "1"  /* Removal of Drainage Device from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF41Z"  =  "1"  /* Removal of Radioactive Element from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF43Z"  =  "1"  /* Removal of Infusion Device from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF4JZ"  =  "1"  /* Removal of Synthetic Substitute from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF4YZ"  =  "1"  /* Removal of Other Device from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPG00Z"  =  "1"  /* Removal of Drainage Device from Peritoneal Cavity, Open Approach */
"0WPG01Z"  =  "1"  /* Removal of Radioactive Element from Peritoneal Cavity, Open Approach */
"0WPG03Z"  =  "1"  /* Removal of Infusion Device from Peritoneal Cavity, Open Approach */
"0WPG0JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneal Cavity, Open Approach */
"0WPG0YZ"  =  "1"  /* Removal of Other Device from Peritoneal Cavity, Open Approach */
"0WPG30Z"  =  "1"  /* Removal of Drainage Device from Peritoneal Cavity, Percutaneous Approach */
"0WPG31Z"  =  "1"  /* Removal of Radioactive Element from Peritoneal Cavity, Percutaneous Approach */
"0WPG33Z"  =  "1"  /* Removal of Infusion Device from Peritoneal Cavity, Percutaneous Approach */
"0WPG3JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneal Cavity, Percutaneous Approach */
"0WPG3YZ"  =  "1"  /* Removal of Other Device from Peritoneal Cavity, Percutaneous Approach */
"0WPG40Z"  =  "1"  /* Removal of Drainage Device from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WPG41Z"  =  "1"  /* Removal of Radioactive Element from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WPG43Z"  =  "1"  /* Removal of Infusion Device from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WPG4JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WPG4YZ"  =  "1"  /* Removal of Other Device from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WPH00Z"  =  "1"  /* Removal of Drainage Device from Retroperitoneum, Open Approach */
"0WPH01Z"  =  "1"  /* Removal of Radioactive Element from Retroperitoneum, Open Approach */
"0WPH03Z"  =  "1"  /* Removal of Infusion Device from Retroperitoneum, Open Approach */
"0WPH0YZ"  =  "1"  /* Removal of Other Device from Retroperitoneum, Open Approach */
"0WPH30Z"  =  "1"  /* Removal of Drainage Device from Retroperitoneum, Percutaneous Approach */
"0WPH31Z"  =  "1"  /* Removal of Radioactive Element from Retroperitoneum, Percutaneous Approach */
"0WPH33Z"  =  "1"  /* Removal of Infusion Device from Retroperitoneum, Percutaneous Approach */
"0WPH3YZ"  =  "1"  /* Removal of Other Device from Retroperitoneum, Percutaneous Approach */
"0WPH40Z"  =  "1"  /* Removal of Drainage Device from Retroperitoneum, Percutaneous Endoscopic Approach */
"0WPH41Z"  =  "1"  /* Removal of Radioactive Element from Retroperitoneum, Percutaneous Endoscopic Approach */
"0WPH43Z"  =  "1"  /* Removal of Infusion Device from Retroperitoneum, Percutaneous Endoscopic Approach */
"0WPH4YZ"  =  "1"  /* Removal of Other Device from Retroperitoneum, Percutaneous Endoscopic Approach */
"0WPN00Z"  =  "1"  /* Removal of Drainage Device from Female Perineum, Open Approach */
"0WPN01Z"  =  "1"  /* Removal of Radioactive Element from Female Perineum, Open Approach */
"0WPN03Z"  =  "1"  /* Removal of Infusion Device from Female Perineum, Open Approach */
"0WPN07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Female Perineum, Open Approach */
"0WPN0JZ"  =  "1"  /* Removal of Synthetic Substitute from Female Perineum, Open Approach */
"0WPN0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Female Perineum, Open Approach */
"0WPN0YZ"  =  "1"  /* Removal of Other Device from Female Perineum, Open Approach */
"0WPN30Z"  =  "1"  /* Removal of Drainage Device from Female Perineum, Percutaneous Approach */
"0WPN31Z"  =  "1"  /* Removal of Radioactive Element from Female Perineum, Percutaneous Approach */
"0WPN33Z"  =  "1"  /* Removal of Infusion Device from Female Perineum, Percutaneous Approach */
"0WPN37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Female Perineum, Percutaneous Approach */
"0WPN3JZ"  =  "1"  /* Removal of Synthetic Substitute from Female Perineum, Percutaneous Approach */
"0WPN3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Female Perineum, Percutaneous Approach */
"0WPN3YZ"  =  "1"  /* Removal of Other Device from Female Perineum, Percutaneous Approach */
"0WPN40Z"  =  "1"  /* Removal of Drainage Device from Female Perineum, Percutaneous Endoscopic Approach */
"0WPN41Z"  =  "1"  /* Removal of Radioactive Element from Female Perineum, Percutaneous Endoscopic Approach */
"0WPN43Z"  =  "1"  /* Removal of Infusion Device from Female Perineum, Percutaneous Endoscopic Approach */
"0WPN47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Female Perineum, Percutaneous Endoscopic Approach */
"0WPN4JZ"  =  "1"  /* Removal of Synthetic Substitute from Female Perineum, Percutaneous Endoscopic Approach */
"0WPN4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Female Perineum, Percutaneous Endoscopic Approach */
"0WPN4YZ"  =  "1"  /* Removal of Other Device from Female Perineum, Percutaneous Endoscopic Approach */
"0WPP01Z"  =  "1"  /* Removal of Radioactive Element from Gastrointestinal Tract, Open Approach */
"0WPP03Z"  =  "1"  /* Removal of Infusion Device from Gastrointestinal Tract, Open Approach */
"0WPP0YZ"  =  "1"  /* Removal of Other Device from Gastrointestinal Tract, Open Approach */
"0WPQ01Z"  =  "1"  /* Removal of Radioactive Element from Respiratory Tract, Open Approach */
"0WPQ03Z"  =  "1"  /* Removal of Infusion Device from Respiratory Tract, Open Approach */
"0WPQ0YZ"  =  "1"  /* Removal of Other Device from Respiratory Tract, Open Approach */
"0WW000Z"  =  "1"  /* Revision of Drainage Device in Head, Open Approach */
"0WW001Z"  =  "1"  /* Revision of Radioactive Element in Head, Open Approach */
"0WW003Z"  =  "1"  /* Revision of Infusion Device in Head, Open Approach */
"0WW007Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Head, Open Approach */
"0WW00JZ"  =  "1"  /* Revision of Synthetic Substitute in Head, Open Approach */
"0WW00KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Head, Open Approach */
"0WW00YZ"  =  "1"  /* Revision of Other Device in Head, Open Approach */
"0WW030Z"  =  "1"  /* Revision of Drainage Device in Head, Percutaneous Approach */
"0WW031Z"  =  "1"  /* Revision of Radioactive Element in Head, Percutaneous Approach */
"0WW033Z"  =  "1"  /* Revision of Infusion Device in Head, Percutaneous Approach */
"0WW037Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Head, Percutaneous Approach */
"0WW03JZ"  =  "1"  /* Revision of Synthetic Substitute in Head, Percutaneous Approach */
"0WW03KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Head, Percutaneous Approach */
"0WW03YZ"  =  "1"  /* Revision of Other Device in Head, Percutaneous Approach */
"0WW040Z"  =  "1"  /* Revision of Drainage Device in Head, Percutaneous Endoscopic Approach */
"0WW041Z"  =  "1"  /* Revision of Radioactive Element in Head, Percutaneous Endoscopic Approach */
"0WW043Z"  =  "1"  /* Revision of Infusion Device in Head, Percutaneous Endoscopic Approach */
"0WW047Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Head, Percutaneous Endoscopic Approach */
"0WW04JZ"  =  "1"  /* Revision of Synthetic Substitute in Head, Percutaneous Endoscopic Approach */
"0WW04KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Head, Percutaneous Endoscopic Approach */
"0WW04YZ"  =  "1"  /* Revision of Other Device in Head, Percutaneous Endoscopic Approach */
"0WW100Z"  =  "1"  /* Revision of Drainage Device in Cranial Cavity, Open Approach */
"0WW101Z"  =  "1"  /* Revision of Radioactive Element in Cranial Cavity, Open Approach */
"0WW103Z"  =  "1"  /* Revision of Infusion Device in Cranial Cavity, Open Approach */
"0WW10JZ"  =  "1"  /* Revision of Synthetic Substitute in Cranial Cavity, Open Approach */
"0WW10YZ"  =  "1"  /* Revision of Other Device in Cranial Cavity, Open Approach */
"0WW130Z"  =  "1"  /* Revision of Drainage Device in Cranial Cavity, Percutaneous Approach */
"0WW131Z"  =  "1"  /* Revision of Radioactive Element in Cranial Cavity, Percutaneous Approach */
"0WW133Z"  =  "1"  /* Revision of Infusion Device in Cranial Cavity, Percutaneous Approach */
"0WW13JZ"  =  "1"  /* Revision of Synthetic Substitute in Cranial Cavity, Percutaneous Approach */
"0WW13YZ"  =  "1"  /* Revision of Other Device in Cranial Cavity, Percutaneous Approach */
"0WW140Z"  =  "1"  /* Revision of Drainage Device in Cranial Cavity, Percutaneous Endoscopic Approach */
"0WW141Z"  =  "1"  /* Revision of Radioactive Element in Cranial Cavity, Percutaneous Endoscopic Approach */
"0WW143Z"  =  "1"  /* Revision of Infusion Device in Cranial Cavity, Percutaneous Endoscopic Approach */
"0WW14JZ"  =  "1"  /* Revision of Synthetic Substitute in Cranial Cavity, Percutaneous Endoscopic Approach */
"0WW14YZ"  =  "1"  /* Revision of Other Device in Cranial Cavity, Percutaneous Endoscopic Approach */
"0WW200Z"  =  "1"  /* Revision of Drainage Device in Face, Open Approach */
"0WW201Z"  =  "1"  /* Revision of Radioactive Element in Face, Open Approach */
"0WW203Z"  =  "1"  /* Revision of Infusion Device in Face, Open Approach */
"0WW207Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Face, Open Approach */
"0WW20JZ"  =  "1"  /* Revision of Synthetic Substitute in Face, Open Approach */
"0WW20KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Face, Open Approach */
"0WW20YZ"  =  "1"  /* Revision of Other Device in Face, Open Approach */
"0WW230Z"  =  "1"  /* Revision of Drainage Device in Face, Percutaneous Approach */
"0WW231Z"  =  "1"  /* Revision of Radioactive Element in Face, Percutaneous Approach */
"0WW233Z"  =  "1"  /* Revision of Infusion Device in Face, Percutaneous Approach */
"0WW237Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Face, Percutaneous Approach */
"0WW23JZ"  =  "1"  /* Revision of Synthetic Substitute in Face, Percutaneous Approach */
"0WW23KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Face, Percutaneous Approach */
"0WW23YZ"  =  "1"  /* Revision of Other Device in Face, Percutaneous Approach */
"0WW240Z"  =  "1"  /* Revision of Drainage Device in Face, Percutaneous Endoscopic Approach */
"0WW241Z"  =  "1"  /* Revision of Radioactive Element in Face, Percutaneous Endoscopic Approach */
"0WW243Z"  =  "1"  /* Revision of Infusion Device in Face, Percutaneous Endoscopic Approach */
"0WW247Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Face, Percutaneous Endoscopic Approach */
"0WW24JZ"  =  "1"  /* Revision of Synthetic Substitute in Face, Percutaneous Endoscopic Approach */
"0WW24KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Face, Percutaneous Endoscopic Approach */
"0WW24YZ"  =  "1"  /* Revision of Other Device in Face, Percutaneous Endoscopic Approach */
"0WW400Z"  =  "1"  /* Revision of Drainage Device in Upper Jaw, Open Approach */
"0WW401Z"  =  "1"  /* Revision of Radioactive Element in Upper Jaw, Open Approach */
"0WW403Z"  =  "1"  /* Revision of Infusion Device in Upper Jaw, Open Approach */
"0WW407Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Jaw, Open Approach */
"0WW40JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Jaw, Open Approach */
"0WW40KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Jaw, Open Approach */
"0WW40YZ"  =  "1"  /* Revision of Other Device in Upper Jaw, Open Approach */
"0WW430Z"  =  "1"  /* Revision of Drainage Device in Upper Jaw, Percutaneous Approach */
"0WW431Z"  =  "1"  /* Revision of Radioactive Element in Upper Jaw, Percutaneous Approach */
"0WW433Z"  =  "1"  /* Revision of Infusion Device in Upper Jaw, Percutaneous Approach */
"0WW437Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Jaw, Percutaneous Approach */
"0WW43JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Jaw, Percutaneous Approach */
"0WW43KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Jaw, Percutaneous Approach */
"0WW43YZ"  =  "1"  /* Revision of Other Device in Upper Jaw, Percutaneous Approach */
"0WW440Z"  =  "1"  /* Revision of Drainage Device in Upper Jaw, Percutaneous Endoscopic Approach */
"0WW441Z"  =  "1"  /* Revision of Radioactive Element in Upper Jaw, Percutaneous Endoscopic Approach */
"0WW443Z"  =  "1"  /* Revision of Infusion Device in Upper Jaw, Percutaneous Endoscopic Approach */
"0WW447Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Jaw, Percutaneous Endoscopic Approach */
"0WW44JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Jaw, Percutaneous Endoscopic Approach */
"0WW44KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Jaw, Percutaneous Endoscopic Approach */
"0WW44YZ"  =  "1"  /* Revision of Other Device in Upper Jaw, Percutaneous Endoscopic Approach */
"0WW500Z"  =  "1"  /* Revision of Drainage Device in Lower Jaw, Open Approach */
"0WW501Z"  =  "1"  /* Revision of Radioactive Element in Lower Jaw, Open Approach */
"0WW503Z"  =  "1"  /* Revision of Infusion Device in Lower Jaw, Open Approach */
"0WW507Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Jaw, Open Approach */
"0WW50JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Jaw, Open Approach */
"0WW50KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Jaw, Open Approach */
"0WW50YZ"  =  "1"  /* Revision of Other Device in Lower Jaw, Open Approach */
"0WW530Z"  =  "1"  /* Revision of Drainage Device in Lower Jaw, Percutaneous Approach */
"0WW531Z"  =  "1"  /* Revision of Radioactive Element in Lower Jaw, Percutaneous Approach */
"0WW533Z"  =  "1"  /* Revision of Infusion Device in Lower Jaw, Percutaneous Approach */
"0WW537Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Jaw, Percutaneous Approach */
"0WW53JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Jaw, Percutaneous Approach */
"0WW53KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Jaw, Percutaneous Approach */
"0WW53YZ"  =  "1"  /* Revision of Other Device in Lower Jaw, Percutaneous Approach */
"0WW540Z"  =  "1"  /* Revision of Drainage Device in Lower Jaw, Percutaneous Endoscopic Approach */
"0WW541Z"  =  "1"  /* Revision of Radioactive Element in Lower Jaw, Percutaneous Endoscopic Approach */
"0WW543Z"  =  "1"  /* Revision of Infusion Device in Lower Jaw, Percutaneous Endoscopic Approach */
"0WW547Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Jaw, Percutaneous Endoscopic Approach */
"0WW54JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Jaw, Percutaneous Endoscopic Approach */
"0WW54KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Jaw, Percutaneous Endoscopic Approach */
"0WW54YZ"  =  "1"  /* Revision of Other Device in Lower Jaw, Percutaneous Endoscopic Approach */
"0WW600Z"  =  "1"  /* Revision of Drainage Device in Neck, Open Approach */
"0WW601Z"  =  "1"  /* Revision of Radioactive Element in Neck, Open Approach */
"0WW603Z"  =  "1"  /* Revision of Infusion Device in Neck, Open Approach */
"0WW607Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Neck, Open Approach */
"0WW60JZ"  =  "1"  /* Revision of Synthetic Substitute in Neck, Open Approach */
"0WW60KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Neck, Open Approach */
"0WW60YZ"  =  "1"  /* Revision of Other Device in Neck, Open Approach */
"0WW630Z"  =  "1"  /* Revision of Drainage Device in Neck, Percutaneous Approach */
"0WW631Z"  =  "1"  /* Revision of Radioactive Element in Neck, Percutaneous Approach */
"0WW633Z"  =  "1"  /* Revision of Infusion Device in Neck, Percutaneous Approach */
"0WW637Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Neck, Percutaneous Approach */
"0WW63JZ"  =  "1"  /* Revision of Synthetic Substitute in Neck, Percutaneous Approach */
"0WW63KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Neck, Percutaneous Approach */
"0WW63YZ"  =  "1"  /* Revision of Other Device in Neck, Percutaneous Approach */
"0WW640Z"  =  "1"  /* Revision of Drainage Device in Neck, Percutaneous Endoscopic Approach */
"0WW641Z"  =  "1"  /* Revision of Radioactive Element in Neck, Percutaneous Endoscopic Approach */
"0WW643Z"  =  "1"  /* Revision of Infusion Device in Neck, Percutaneous Endoscopic Approach */
"0WW647Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Neck, Percutaneous Endoscopic Approach */
"0WW64JZ"  =  "1"  /* Revision of Synthetic Substitute in Neck, Percutaneous Endoscopic Approach */
"0WW64KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Neck, Percutaneous Endoscopic Approach */
"0WW64YZ"  =  "1"  /* Revision of Other Device in Neck, Percutaneous Endoscopic Approach */
"0WW900Z"  =  "1"  /* Revision of Drainage Device in Right Pleural Cavity, Open Approach */
"0WW901Z"  =  "1"  /* Revision of Radioactive Element in Right Pleural Cavity, Open Approach */
"0WW903Z"  =  "1"  /* Revision of Infusion Device in Right Pleural Cavity, Open Approach */
"0WW90JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Pleural Cavity, Open Approach */
"0WW90YZ"  =  "1"  /* Revision of Other Device in Right Pleural Cavity, Open Approach */
"0WW930Z"  =  "1"  /* Revision of Drainage Device in Right Pleural Cavity, Percutaneous Approach */
"0WW931Z"  =  "1"  /* Revision of Radioactive Element in Right Pleural Cavity, Percutaneous Approach */
"0WW933Z"  =  "1"  /* Revision of Infusion Device in Right Pleural Cavity, Percutaneous Approach */
"0WW93JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Pleural Cavity, Percutaneous Approach */
"0WW93YZ"  =  "1"  /* Revision of Other Device in Right Pleural Cavity, Percutaneous Approach */
"0WW940Z"  =  "1"  /* Revision of Drainage Device in Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WW941Z"  =  "1"  /* Revision of Radioactive Element in Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WW943Z"  =  "1"  /* Revision of Infusion Device in Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WW94JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WW94YZ"  =  "1"  /* Revision of Other Device in Right Pleural Cavity, Percutaneous Endoscopic Approach */
"0WWB00Z"  =  "1"  /* Revision of Drainage Device in Left Pleural Cavity, Open Approach */
"0WWB01Z"  =  "1"  /* Revision of Radioactive Element in Left Pleural Cavity, Open Approach */
"0WWB03Z"  =  "1"  /* Revision of Infusion Device in Left Pleural Cavity, Open Approach */
"0WWB0JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Pleural Cavity, Open Approach */
"0WWB0YZ"  =  "1"  /* Revision of Other Device in Left Pleural Cavity, Open Approach */
"0WWB30Z"  =  "1"  /* Revision of Drainage Device in Left Pleural Cavity, Percutaneous Approach */
"0WWB31Z"  =  "1"  /* Revision of Radioactive Element in Left Pleural Cavity, Percutaneous Approach */
"0WWB33Z"  =  "1"  /* Revision of Infusion Device in Left Pleural Cavity, Percutaneous Approach */
"0WWB3JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Pleural Cavity, Percutaneous Approach */
"0WWB3YZ"  =  "1"  /* Revision of Other Device in Left Pleural Cavity, Percutaneous Approach */
"0WWB40Z"  =  "1"  /* Revision of Drainage Device in Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WWB41Z"  =  "1"  /* Revision of Radioactive Element in Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WWB43Z"  =  "1"  /* Revision of Infusion Device in Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WWB4JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WWB4YZ"  =  "1"  /* Revision of Other Device in Left Pleural Cavity, Percutaneous Endoscopic Approach */
"0WWC00Z"  =  "1"  /* Revision of Drainage Device in Mediastinum, Open Approach */
"0WWC01Z"  =  "1"  /* Revision of Radioactive Element in Mediastinum, Open Approach */
"0WWC03Z"  =  "1"  /* Revision of Infusion Device in Mediastinum, Open Approach */
"0WWC07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mediastinum, Open Approach */
"0WWC0JZ"  =  "1"  /* Revision of Synthetic Substitute in Mediastinum, Open Approach */
"0WWC0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mediastinum, Open Approach */
"0WWC0YZ"  =  "1"  /* Revision of Other Device in Mediastinum, Open Approach */
"0WWC30Z"  =  "1"  /* Revision of Drainage Device in Mediastinum, Percutaneous Approach */
"0WWC31Z"  =  "1"  /* Revision of Radioactive Element in Mediastinum, Percutaneous Approach */
"0WWC33Z"  =  "1"  /* Revision of Infusion Device in Mediastinum, Percutaneous Approach */
"0WWC37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mediastinum, Percutaneous Approach */
"0WWC3JZ"  =  "1"  /* Revision of Synthetic Substitute in Mediastinum, Percutaneous Approach */
"0WWC3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mediastinum, Percutaneous Approach */
"0WWC3YZ"  =  "1"  /* Revision of Other Device in Mediastinum, Percutaneous Approach */
"0WWC40Z"  =  "1"  /* Revision of Drainage Device in Mediastinum, Percutaneous Endoscopic Approach */
"0WWC41Z"  =  "1"  /* Revision of Radioactive Element in Mediastinum, Percutaneous Endoscopic Approach */
"0WWC43Z"  =  "1"  /* Revision of Infusion Device in Mediastinum, Percutaneous Endoscopic Approach */
"0WWC47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mediastinum, Percutaneous Endoscopic Approach */
"0WWC4JZ"  =  "1"  /* Revision of Synthetic Substitute in Mediastinum, Percutaneous Endoscopic Approach */
"0WWC4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mediastinum, Percutaneous Endoscopic Approach */
"0WWC4YZ"  =  "1"  /* Revision of Other Device in Mediastinum, Percutaneous Endoscopic Approach */
"0WWF00Z"  =  "1"  /* Revision of Drainage Device in Abdominal Wall, Open Approach */
"0WWF01Z"  =  "1"  /* Revision of Radioactive Element in Abdominal Wall, Open Approach */
"0WWF03Z"  =  "1"  /* Revision of Infusion Device in Abdominal Wall, Open Approach */
"0WWF07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Abdominal Wall, Open Approach */
"0WWF0JZ"  =  "1"  /* Revision of Synthetic Substitute in Abdominal Wall, Open Approach */
"0WWF0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Abdominal Wall, Open Approach */
"0WWF0YZ"  =  "1"  /* Revision of Other Device in Abdominal Wall, Open Approach */
"0WWF30Z"  =  "1"  /* Revision of Drainage Device in Abdominal Wall, Percutaneous Approach */
"0WWF31Z"  =  "1"  /* Revision of Radioactive Element in Abdominal Wall, Percutaneous Approach */
"0WWF33Z"  =  "1"  /* Revision of Infusion Device in Abdominal Wall, Percutaneous Approach */
"0WWF37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Abdominal Wall, Percutaneous Approach */
"0WWF3JZ"  =  "1"  /* Revision of Synthetic Substitute in Abdominal Wall, Percutaneous Approach */
"0WWF3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Abdominal Wall, Percutaneous Approach */
"0WWF3YZ"  =  "1"  /* Revision of Other Device in Abdominal Wall, Percutaneous Approach */
"0WWF40Z"  =  "1"  /* Revision of Drainage Device in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF41Z"  =  "1"  /* Revision of Radioactive Element in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF43Z"  =  "1"  /* Revision of Infusion Device in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF4JZ"  =  "1"  /* Revision of Synthetic Substitute in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF4YZ"  =  "1"  /* Revision of Other Device in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWG00Z"  =  "1"  /* Revision of Drainage Device in Peritoneal Cavity, Open Approach */
"0WWG01Z"  =  "1"  /* Revision of Radioactive Element in Peritoneal Cavity, Open Approach */
"0WWG03Z"  =  "1"  /* Revision of Infusion Device in Peritoneal Cavity, Open Approach */
"0WWG0JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneal Cavity, Open Approach */
"0WWG0YZ"  =  "1"  /* Revision of Other Device in Peritoneal Cavity, Open Approach */
"0WWG30Z"  =  "1"  /* Revision of Drainage Device in Peritoneal Cavity, Percutaneous Approach */
"0WWG31Z"  =  "1"  /* Revision of Radioactive Element in Peritoneal Cavity, Percutaneous Approach */
"0WWG33Z"  =  "1"  /* Revision of Infusion Device in Peritoneal Cavity, Percutaneous Approach */
"0WWG3JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneal Cavity, Percutaneous Approach */
"0WWG3YZ"  =  "1"  /* Revision of Other Device in Peritoneal Cavity, Percutaneous Approach */
"0WWG40Z"  =  "1"  /* Revision of Drainage Device in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WWG41Z"  =  "1"  /* Revision of Radioactive Element in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WWG43Z"  =  "1"  /* Revision of Infusion Device in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WWG4JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WWG4YZ"  =  "1"  /* Revision of Other Device in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WWH00Z"  =  "1"  /* Revision of Drainage Device in Retroperitoneum, Open Approach */
"0WWH01Z"  =  "1"  /* Revision of Radioactive Element in Retroperitoneum, Open Approach */
"0WWH03Z"  =  "1"  /* Revision of Infusion Device in Retroperitoneum, Open Approach */
"0WWH0YZ"  =  "1"  /* Revision of Other Device in Retroperitoneum, Open Approach */
"0WWH30Z"  =  "1"  /* Revision of Drainage Device in Retroperitoneum, Percutaneous Approach */
"0WWH31Z"  =  "1"  /* Revision of Radioactive Element in Retroperitoneum, Percutaneous Approach */
"0WWH33Z"  =  "1"  /* Revision of Infusion Device in Retroperitoneum, Percutaneous Approach */
"0WWH3YZ"  =  "1"  /* Revision of Other Device in Retroperitoneum, Percutaneous Approach */
"0WWH40Z"  =  "1"  /* Revision of Drainage Device in Retroperitoneum, Percutaneous Endoscopic Approach */
"0WWH41Z"  =  "1"  /* Revision of Radioactive Element in Retroperitoneum, Percutaneous Endoscopic Approach */
"0WWH43Z"  =  "1"  /* Revision of Infusion Device in Retroperitoneum, Percutaneous Endoscopic Approach */
"0WWH4YZ"  =  "1"  /* Revision of Other Device in Retroperitoneum, Percutaneous Endoscopic Approach */
"0WWJ00Z"  =  "1"  /* Revision of Drainage Device in Pelvic Cavity, Open Approach */
"0WWJ01Z"  =  "1"  /* Revision of Radioactive Element in Pelvic Cavity, Open Approach */
"0WWJ03Z"  =  "1"  /* Revision of Infusion Device in Pelvic Cavity, Open Approach */
"0WWJ0JZ"  =  "1"  /* Revision of Synthetic Substitute in Pelvic Cavity, Open Approach */
"0WWJ0YZ"  =  "1"  /* Revision of Other Device in Pelvic Cavity, Open Approach */
"0WWJ30Z"  =  "1"  /* Revision of Drainage Device in Pelvic Cavity, Percutaneous Approach */
"0WWJ31Z"  =  "1"  /* Revision of Radioactive Element in Pelvic Cavity, Percutaneous Approach */
"0WWJ33Z"  =  "1"  /* Revision of Infusion Device in Pelvic Cavity, Percutaneous Approach */
"0WWJ3JZ"  =  "1"  /* Revision of Synthetic Substitute in Pelvic Cavity, Percutaneous Approach */
"0WWJ3YZ"  =  "1"  /* Revision of Other Device in Pelvic Cavity, Percutaneous Approach */
"0WWJ40Z"  =  "1"  /* Revision of Drainage Device in Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WWJ41Z"  =  "1"  /* Revision of Radioactive Element in Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WWJ43Z"  =  "1"  /* Revision of Infusion Device in Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WWJ4JZ"  =  "1"  /* Revision of Synthetic Substitute in Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WWJ4YZ"  =  "1"  /* Revision of Other Device in Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WWK00Z"  =  "1"  /* Revision of Drainage Device in Upper Back, Open Approach */
"0WWK01Z"  =  "1"  /* Revision of Radioactive Element in Upper Back, Open Approach */
"0WWK03Z"  =  "1"  /* Revision of Infusion Device in Upper Back, Open Approach */
"0WWK07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Back, Open Approach */
"0WWK0JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Back, Open Approach */
"0WWK0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Back, Open Approach */
"0WWK0YZ"  =  "1"  /* Revision of Other Device in Upper Back, Open Approach */
"0WWK30Z"  =  "1"  /* Revision of Drainage Device in Upper Back, Percutaneous Approach */
"0WWK31Z"  =  "1"  /* Revision of Radioactive Element in Upper Back, Percutaneous Approach */
"0WWK33Z"  =  "1"  /* Revision of Infusion Device in Upper Back, Percutaneous Approach */
"0WWK37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Back, Percutaneous Approach */
"0WWK3JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Back, Percutaneous Approach */
"0WWK3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Back, Percutaneous Approach */
"0WWK3YZ"  =  "1"  /* Revision of Other Device in Upper Back, Percutaneous Approach */
"0WWK40Z"  =  "1"  /* Revision of Drainage Device in Upper Back, Percutaneous Endoscopic Approach */
"0WWK41Z"  =  "1"  /* Revision of Radioactive Element in Upper Back, Percutaneous Endoscopic Approach */
"0WWK43Z"  =  "1"  /* Revision of Infusion Device in Upper Back, Percutaneous Endoscopic Approach */
"0WWK47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Back, Percutaneous Endoscopic Approach */
"0WWK4JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Back, Percutaneous Endoscopic Approach */
"0WWK4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Back, Percutaneous Endoscopic Approach */
"0WWK4YZ"  =  "1"  /* Revision of Other Device in Upper Back, Percutaneous Endoscopic Approach */
"0WWL00Z"  =  "1"  /* Revision of Drainage Device in Lower Back, Open Approach */
"0WWL01Z"  =  "1"  /* Revision of Radioactive Element in Lower Back, Open Approach */
"0WWL03Z"  =  "1"  /* Revision of Infusion Device in Lower Back, Open Approach */
"0WWL07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Back, Open Approach */
"0WWL0JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Back, Open Approach */
"0WWL0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Back, Open Approach */
"0WWL0YZ"  =  "1"  /* Revision of Other Device in Lower Back, Open Approach */
"0WWL30Z"  =  "1"  /* Revision of Drainage Device in Lower Back, Percutaneous Approach */
"0WWL31Z"  =  "1"  /* Revision of Radioactive Element in Lower Back, Percutaneous Approach */
"0WWL33Z"  =  "1"  /* Revision of Infusion Device in Lower Back, Percutaneous Approach */
"0WWL37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Back, Percutaneous Approach */
"0WWL3JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Back, Percutaneous Approach */
"0WWL3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Back, Percutaneous Approach */
"0WWL3YZ"  =  "1"  /* Revision of Other Device in Lower Back, Percutaneous Approach */
"0WWL40Z"  =  "1"  /* Revision of Drainage Device in Lower Back, Percutaneous Endoscopic Approach */
"0WWL41Z"  =  "1"  /* Revision of Radioactive Element in Lower Back, Percutaneous Endoscopic Approach */
"0WWL43Z"  =  "1"  /* Revision of Infusion Device in Lower Back, Percutaneous Endoscopic Approach */
"0WWL47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Back, Percutaneous Endoscopic Approach */
"0WWL4JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Back, Percutaneous Endoscopic Approach */
"0WWL4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Back, Percutaneous Endoscopic Approach */
"0WWL4YZ"  =  "1"  /* Revision of Other Device in Lower Back, Percutaneous Endoscopic Approach */
"0WWM00Z"  =  "1"  /* Revision of Drainage Device in Male Perineum, Open Approach */
"0WWM01Z"  =  "1"  /* Revision of Radioactive Element in Male Perineum, Open Approach */
"0WWM03Z"  =  "1"  /* Revision of Infusion Device in Male Perineum, Open Approach */
"0WWM0JZ"  =  "1"  /* Revision of Synthetic Substitute in Male Perineum, Open Approach */
"0WWM0YZ"  =  "1"  /* Revision of Other Device in Male Perineum, Open Approach */
"0WWM30Z"  =  "1"  /* Revision of Drainage Device in Male Perineum, Percutaneous Approach */
"0WWM31Z"  =  "1"  /* Revision of Radioactive Element in Male Perineum, Percutaneous Approach */
"0WWM33Z"  =  "1"  /* Revision of Infusion Device in Male Perineum, Percutaneous Approach */
"0WWM3JZ"  =  "1"  /* Revision of Synthetic Substitute in Male Perineum, Percutaneous Approach */
"0WWM3YZ"  =  "1"  /* Revision of Other Device in Male Perineum, Percutaneous Approach */
"0WWM40Z"  =  "1"  /* Revision of Drainage Device in Male Perineum, Percutaneous Endoscopic Approach */
"0WWM41Z"  =  "1"  /* Revision of Radioactive Element in Male Perineum, Percutaneous Endoscopic Approach */
"0WWM43Z"  =  "1"  /* Revision of Infusion Device in Male Perineum, Percutaneous Endoscopic Approach */
"0WWM4JZ"  =  "1"  /* Revision of Synthetic Substitute in Male Perineum, Percutaneous Endoscopic Approach */
"0WWM4YZ"  =  "1"  /* Revision of Other Device in Male Perineum, Percutaneous Endoscopic Approach */
"0WWN00Z"  =  "1"  /* Revision of Drainage Device in Female Perineum, Open Approach */
"0WWN01Z"  =  "1"  /* Revision of Radioactive Element in Female Perineum, Open Approach */
"0WWN03Z"  =  "1"  /* Revision of Infusion Device in Female Perineum, Open Approach */
"0WWN07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Female Perineum, Open Approach */
"0WWN0JZ"  =  "1"  /* Revision of Synthetic Substitute in Female Perineum, Open Approach */
"0WWN0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Female Perineum, Open Approach */
"0WWN0YZ"  =  "1"  /* Revision of Other Device in Female Perineum, Open Approach */
"0WWN30Z"  =  "1"  /* Revision of Drainage Device in Female Perineum, Percutaneous Approach */
"0WWN31Z"  =  "1"  /* Revision of Radioactive Element in Female Perineum, Percutaneous Approach */
"0WWN33Z"  =  "1"  /* Revision of Infusion Device in Female Perineum, Percutaneous Approach */
"0WWN37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Female Perineum, Percutaneous Approach */
"0WWN3JZ"  =  "1"  /* Revision of Synthetic Substitute in Female Perineum, Percutaneous Approach */
"0WWN3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Female Perineum, Percutaneous Approach */
"0WWN3YZ"  =  "1"  /* Revision of Other Device in Female Perineum, Percutaneous Approach */
"0WWN40Z"  =  "1"  /* Revision of Drainage Device in Female Perineum, Percutaneous Endoscopic Approach */
"0WWN41Z"  =  "1"  /* Revision of Radioactive Element in Female Perineum, Percutaneous Endoscopic Approach */
"0WWN43Z"  =  "1"  /* Revision of Infusion Device in Female Perineum, Percutaneous Endoscopic Approach */
"0WWN47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Female Perineum, Percutaneous Endoscopic Approach */
"0WWN4JZ"  =  "1"  /* Revision of Synthetic Substitute in Female Perineum, Percutaneous Endoscopic Approach */
"0WWN4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Female Perineum, Percutaneous Endoscopic Approach */
"0WWN4YZ"  =  "1"  /* Revision of Other Device in Female Perineum, Percutaneous Endoscopic Approach */
"0WWP01Z"  =  "1"  /* Revision of Radioactive Element in Gastrointestinal Tract, Open Approach */
"0WWP03Z"  =  "1"  /* Revision of Infusion Device in Gastrointestinal Tract, Open Approach */
"0WWP0YZ"  =  "1"  /* Revision of Other Device in Gastrointestinal Tract, Open Approach */
"0WWQ01Z"  =  "1"  /* Revision of Radioactive Element in Respiratory Tract, Open Approach */
"0WWQ03Z"  =  "1"  /* Revision of Infusion Device in Respiratory Tract, Open Approach */
"0WWQ0YZ"  =  "1"  /* Revision of Other Device in Respiratory Tract, Open Approach */
"0X320ZZ"  =  "1"  /* Control Bleeding in Right Shoulder Region, Open Approach */
"0X323ZZ"  =  "1"  /* Control Bleeding in Right Shoulder Region, Percutaneous Approach */
"0X324ZZ"  =  "1"  /* Control Bleeding in Right Shoulder Region, Percutaneous Endoscopic Approach */
"0X330ZZ"  =  "1"  /* Control Bleeding in Left Shoulder Region, Open Approach */
"0X333ZZ"  =  "1"  /* Control Bleeding in Left Shoulder Region, Percutaneous Approach */
"0X334ZZ"  =  "1"  /* Control Bleeding in Left Shoulder Region, Percutaneous Endoscopic Approach */
"0X340ZZ"  =  "1"  /* Control Bleeding in Right Axilla, Open Approach */
"0X343ZZ"  =  "1"  /* Control Bleeding in Right Axilla, Percutaneous Approach */
"0X344ZZ"  =  "1"  /* Control Bleeding in Right Axilla, Percutaneous Endoscopic Approach */
"0X350ZZ"  =  "1"  /* Control Bleeding in Left Axilla, Open Approach */
"0X353ZZ"  =  "1"  /* Control Bleeding in Left Axilla, Percutaneous Approach */
"0X354ZZ"  =  "1"  /* Control Bleeding in Left Axilla, Percutaneous Endoscopic Approach */
"0X360ZZ"  =  "1"  /* Control Bleeding in Right Upper Extremity, Open Approach */
"0X363ZZ"  =  "1"  /* Control Bleeding in Right Upper Extremity, Percutaneous Approach */
"0X364ZZ"  =  "1"  /* Control Bleeding in Right Upper Extremity, Percutaneous Endoscopic Approach */
"0X370ZZ"  =  "1"  /* Control Bleeding in Left Upper Extremity, Open Approach */
"0X373ZZ"  =  "1"  /* Control Bleeding in Left Upper Extremity, Percutaneous Approach */
"0X374ZZ"  =  "1"  /* Control Bleeding in Left Upper Extremity, Percutaneous Endoscopic Approach */
"0X380ZZ"  =  "1"  /* Control Bleeding in Right Upper Arm, Open Approach */
"0X383ZZ"  =  "1"  /* Control Bleeding in Right Upper Arm, Percutaneous Approach */
"0X384ZZ"  =  "1"  /* Control Bleeding in Right Upper Arm, Percutaneous Endoscopic Approach */
"0X390ZZ"  =  "1"  /* Control Bleeding in Left Upper Arm, Open Approach */
"0X393ZZ"  =  "1"  /* Control Bleeding in Left Upper Arm, Percutaneous Approach */
"0X394ZZ"  =  "1"  /* Control Bleeding in Left Upper Arm, Percutaneous Endoscopic Approach */
"0X3B0ZZ"  =  "1"  /* Control Bleeding in Right Elbow Region, Open Approach */
"0X3B3ZZ"  =  "1"  /* Control Bleeding in Right Elbow Region, Percutaneous Approach */
"0X3B4ZZ"  =  "1"  /* Control Bleeding in Right Elbow Region, Percutaneous Endoscopic Approach */
"0X3C0ZZ"  =  "1"  /* Control Bleeding in Left Elbow Region, Open Approach */
"0X3C3ZZ"  =  "1"  /* Control Bleeding in Left Elbow Region, Percutaneous Approach */
"0X3C4ZZ"  =  "1"  /* Control Bleeding in Left Elbow Region, Percutaneous Endoscopic Approach */
"0X3D0ZZ"  =  "1"  /* Control Bleeding in Right Lower Arm, Open Approach */
"0X3D3ZZ"  =  "1"  /* Control Bleeding in Right Lower Arm, Percutaneous Approach */
"0X3D4ZZ"  =  "1"  /* Control Bleeding in Right Lower Arm, Percutaneous Endoscopic Approach */
"0X3F0ZZ"  =  "1"  /* Control Bleeding in Left Lower Arm, Open Approach */
"0X3F3ZZ"  =  "1"  /* Control Bleeding in Left Lower Arm, Percutaneous Approach */
"0X3F4ZZ"  =  "1"  /* Control Bleeding in Left Lower Arm, Percutaneous Endoscopic Approach */
"0X3G0ZZ"  =  "1"  /* Control Bleeding in Right Wrist Region, Open Approach */
"0X3G3ZZ"  =  "1"  /* Control Bleeding in Right Wrist Region, Percutaneous Approach */
"0X3G4ZZ"  =  "1"  /* Control Bleeding in Right Wrist Region, Percutaneous Endoscopic Approach */
"0X3H0ZZ"  =  "1"  /* Control Bleeding in Left Wrist Region, Open Approach */
"0X3H3ZZ"  =  "1"  /* Control Bleeding in Left Wrist Region, Percutaneous Approach */
"0X3H4ZZ"  =  "1"  /* Control Bleeding in Left Wrist Region, Percutaneous Endoscopic Approach */
"0X3J0ZZ"  =  "1"  /* Control Bleeding in Right Hand, Open Approach */
"0X3J3ZZ"  =  "1"  /* Control Bleeding in Right Hand, Percutaneous Approach */
"0X3J4ZZ"  =  "1"  /* Control Bleeding in Right Hand, Percutaneous Endoscopic Approach */
"0X3K0ZZ"  =  "1"  /* Control Bleeding in Left Hand, Open Approach */
"0X3K3ZZ"  =  "1"  /* Control Bleeding in Left Hand, Percutaneous Approach */
"0X3K4ZZ"  =  "1"  /* Control Bleeding in Left Hand, Percutaneous Endoscopic Approach */
"0X9200Z"  =  "1"  /* Drainage of Right Shoulder Region with Drainage Device, Open Approach */
"0X920ZZ"  =  "1"  /* Drainage of Right Shoulder Region, Open Approach */
"0X9230Z"  =  "1"  /* Drainage of Right Shoulder Region with Drainage Device, Percutaneous Approach */
"0X923ZZ"  =  "1"  /* Drainage of Right Shoulder Region, Percutaneous Approach */
"0X9240Z"  =  "1"  /* Drainage of Right Shoulder Region with Drainage Device, Percutaneous Endoscopic Approach */
"0X924ZZ"  =  "1"  /* Drainage of Right Shoulder Region, Percutaneous Endoscopic Approach */
"0X9300Z"  =  "1"  /* Drainage of Left Shoulder Region with Drainage Device, Open Approach */
"0X930ZZ"  =  "1"  /* Drainage of Left Shoulder Region, Open Approach */
"0X9330Z"  =  "1"  /* Drainage of Left Shoulder Region with Drainage Device, Percutaneous Approach */
"0X933ZZ"  =  "1"  /* Drainage of Left Shoulder Region, Percutaneous Approach */
"0X9340Z"  =  "1"  /* Drainage of Left Shoulder Region with Drainage Device, Percutaneous Endoscopic Approach */
"0X934ZZ"  =  "1"  /* Drainage of Left Shoulder Region, Percutaneous Endoscopic Approach */
"0X9400Z"  =  "1"  /* Drainage of Right Axilla with Drainage Device, Open Approach */
"0X940ZZ"  =  "1"  /* Drainage of Right Axilla, Open Approach */
"0X9430Z"  =  "1"  /* Drainage of Right Axilla with Drainage Device, Percutaneous Approach */
"0X943ZZ"  =  "1"  /* Drainage of Right Axilla, Percutaneous Approach */
"0X9440Z"  =  "1"  /* Drainage of Right Axilla with Drainage Device, Percutaneous Endoscopic Approach */
"0X944ZZ"  =  "1"  /* Drainage of Right Axilla, Percutaneous Endoscopic Approach */
"0X9500Z"  =  "1"  /* Drainage of Left Axilla with Drainage Device, Open Approach */
"0X950ZZ"  =  "1"  /* Drainage of Left Axilla, Open Approach */
"0X9530Z"  =  "1"  /* Drainage of Left Axilla with Drainage Device, Percutaneous Approach */
"0X953ZZ"  =  "1"  /* Drainage of Left Axilla, Percutaneous Approach */
"0X9540Z"  =  "1"  /* Drainage of Left Axilla with Drainage Device, Percutaneous Endoscopic Approach */
"0X954ZZ"  =  "1"  /* Drainage of Left Axilla, Percutaneous Endoscopic Approach */
"0X9600Z"  =  "1"  /* Drainage of Right Upper Extremity with Drainage Device, Open Approach */
"0X960ZZ"  =  "1"  /* Drainage of Right Upper Extremity, Open Approach */
"0X9630Z"  =  "1"  /* Drainage of Right Upper Extremity with Drainage Device, Percutaneous Approach */
"0X963ZZ"  =  "1"  /* Drainage of Right Upper Extremity, Percutaneous Approach */
"0X9640Z"  =  "1"  /* Drainage of Right Upper Extremity with Drainage Device, Percutaneous Endoscopic Approach */
"0X964ZZ"  =  "1"  /* Drainage of Right Upper Extremity, Percutaneous Endoscopic Approach */
"0X9700Z"  =  "1"  /* Drainage of Left Upper Extremity with Drainage Device, Open Approach */
"0X970ZZ"  =  "1"  /* Drainage of Left Upper Extremity, Open Approach */
"0X9730Z"  =  "1"  /* Drainage of Left Upper Extremity with Drainage Device, Percutaneous Approach */
"0X973ZZ"  =  "1"  /* Drainage of Left Upper Extremity, Percutaneous Approach */
"0X9740Z"  =  "1"  /* Drainage of Left Upper Extremity with Drainage Device, Percutaneous Endoscopic Approach */
"0X974ZZ"  =  "1"  /* Drainage of Left Upper Extremity, Percutaneous Endoscopic Approach */
"0X9800Z"  =  "1"  /* Drainage of Right Upper Arm with Drainage Device, Open Approach */
"0X980ZZ"  =  "1"  /* Drainage of Right Upper Arm, Open Approach */
"0X9830Z"  =  "1"  /* Drainage of Right Upper Arm with Drainage Device, Percutaneous Approach */
"0X983ZZ"  =  "1"  /* Drainage of Right Upper Arm, Percutaneous Approach */
"0X9840Z"  =  "1"  /* Drainage of Right Upper Arm with Drainage Device, Percutaneous Endoscopic Approach */
"0X984ZZ"  =  "1"  /* Drainage of Right Upper Arm, Percutaneous Endoscopic Approach */
"0X9900Z"  =  "1"  /* Drainage of Left Upper Arm with Drainage Device, Open Approach */
"0X990ZZ"  =  "1"  /* Drainage of Left Upper Arm, Open Approach */
"0X9930Z"  =  "1"  /* Drainage of Left Upper Arm with Drainage Device, Percutaneous Approach */
"0X993ZZ"  =  "1"  /* Drainage of Left Upper Arm, Percutaneous Approach */
"0X9940Z"  =  "1"  /* Drainage of Left Upper Arm with Drainage Device, Percutaneous Endoscopic Approach */
"0X994ZZ"  =  "1"  /* Drainage of Left Upper Arm, Percutaneous Endoscopic Approach */
"0X9B00Z"  =  "1"  /* Drainage of Right Elbow Region with Drainage Device, Open Approach */
"0X9B0ZZ"  =  "1"  /* Drainage of Right Elbow Region, Open Approach */
"0X9B30Z"  =  "1"  /* Drainage of Right Elbow Region with Drainage Device, Percutaneous Approach */
"0X9B3ZZ"  =  "1"  /* Drainage of Right Elbow Region, Percutaneous Approach */
"0X9B40Z"  =  "1"  /* Drainage of Right Elbow Region with Drainage Device, Percutaneous Endoscopic Approach */
"0X9B4ZZ"  =  "1"  /* Drainage of Right Elbow Region, Percutaneous Endoscopic Approach */
"0X9C00Z"  =  "1"  /* Drainage of Left Elbow Region with Drainage Device, Open Approach */
"0X9C0ZZ"  =  "1"  /* Drainage of Left Elbow Region, Open Approach */
"0X9C30Z"  =  "1"  /* Drainage of Left Elbow Region with Drainage Device, Percutaneous Approach */
"0X9C3ZZ"  =  "1"  /* Drainage of Left Elbow Region, Percutaneous Approach */
"0X9C40Z"  =  "1"  /* Drainage of Left Elbow Region with Drainage Device, Percutaneous Endoscopic Approach */
"0X9C4ZZ"  =  "1"  /* Drainage of Left Elbow Region, Percutaneous Endoscopic Approach */
"0X9D00Z"  =  "1"  /* Drainage of Right Lower Arm with Drainage Device, Open Approach */
"0X9D0ZZ"  =  "1"  /* Drainage of Right Lower Arm, Open Approach */
"0X9D30Z"  =  "1"  /* Drainage of Right Lower Arm with Drainage Device, Percutaneous Approach */
"0X9D3ZZ"  =  "1"  /* Drainage of Right Lower Arm, Percutaneous Approach */
"0X9D40Z"  =  "1"  /* Drainage of Right Lower Arm with Drainage Device, Percutaneous Endoscopic Approach */
"0X9D4ZZ"  =  "1"  /* Drainage of Right Lower Arm, Percutaneous Endoscopic Approach */
"0X9F00Z"  =  "1"  /* Drainage of Left Lower Arm with Drainage Device, Open Approach */
"0X9F0ZZ"  =  "1"  /* Drainage of Left Lower Arm, Open Approach */
"0X9F30Z"  =  "1"  /* Drainage of Left Lower Arm with Drainage Device, Percutaneous Approach */
"0X9F3ZZ"  =  "1"  /* Drainage of Left Lower Arm, Percutaneous Approach */
"0X9F40Z"  =  "1"  /* Drainage of Left Lower Arm with Drainage Device, Percutaneous Endoscopic Approach */
"0X9F4ZZ"  =  "1"  /* Drainage of Left Lower Arm, Percutaneous Endoscopic Approach */
"0X9G00Z"  =  "1"  /* Drainage of Right Wrist Region with Drainage Device, Open Approach */
"0X9G0ZZ"  =  "1"  /* Drainage of Right Wrist Region, Open Approach */
"0X9G30Z"  =  "1"  /* Drainage of Right Wrist Region with Drainage Device, Percutaneous Approach */
"0X9G3ZZ"  =  "1"  /* Drainage of Right Wrist Region, Percutaneous Approach */
"0X9G40Z"  =  "1"  /* Drainage of Right Wrist Region with Drainage Device, Percutaneous Endoscopic Approach */
"0X9G4ZZ"  =  "1"  /* Drainage of Right Wrist Region, Percutaneous Endoscopic Approach */
"0X9H00Z"  =  "1"  /* Drainage of Left Wrist Region with Drainage Device, Open Approach */
"0X9H0ZZ"  =  "1"  /* Drainage of Left Wrist Region, Open Approach */
"0X9H30Z"  =  "1"  /* Drainage of Left Wrist Region with Drainage Device, Percutaneous Approach */
"0X9H3ZZ"  =  "1"  /* Drainage of Left Wrist Region, Percutaneous Approach */
"0X9H40Z"  =  "1"  /* Drainage of Left Wrist Region with Drainage Device, Percutaneous Endoscopic Approach */
"0X9H4ZZ"  =  "1"  /* Drainage of Left Wrist Region, Percutaneous Endoscopic Approach */
"0X9J00Z"  =  "1"  /* Drainage of Right Hand with Drainage Device, Open Approach */
"0X9J0ZZ"  =  "1"  /* Drainage of Right Hand, Open Approach */
"0X9J30Z"  =  "1"  /* Drainage of Right Hand with Drainage Device, Percutaneous Approach */
"0X9J3ZZ"  =  "1"  /* Drainage of Right Hand, Percutaneous Approach */
"0X9J40Z"  =  "1"  /* Drainage of Right Hand with Drainage Device, Percutaneous Endoscopic Approach */
"0X9J4ZZ"  =  "1"  /* Drainage of Right Hand, Percutaneous Endoscopic Approach */
"0X9K00Z"  =  "1"  /* Drainage of Left Hand with Drainage Device, Open Approach */
"0X9K0ZZ"  =  "1"  /* Drainage of Left Hand, Open Approach */
"0X9K30Z"  =  "1"  /* Drainage of Left Hand with Drainage Device, Percutaneous Approach */
"0X9K3ZZ"  =  "1"  /* Drainage of Left Hand, Percutaneous Approach */
"0X9K40Z"  =  "1"  /* Drainage of Left Hand with Drainage Device, Percutaneous Endoscopic Approach */
"0X9K4ZZ"  =  "1"  /* Drainage of Left Hand, Percutaneous Endoscopic Approach */
"0XH203Z"  =  "1"  /* Insertion of Infusion Device into Right Shoulder Region, Open Approach */
"0XH20YZ"  =  "1"  /* Insertion of Other Device into Right Shoulder Region, Open Approach */
"0XH233Z"  =  "1"  /* Insertion of Infusion Device into Right Shoulder Region, Percutaneous Approach */
"0XH23YZ"  =  "1"  /* Insertion of Other Device into Right Shoulder Region, Percutaneous Approach */
"0XH243Z"  =  "1"  /* Insertion of Infusion Device into Right Shoulder Region, Percutaneous Endoscopic Approach */
"0XH24YZ"  =  "1"  /* Insertion of Other Device into Right Shoulder Region, Percutaneous Endoscopic Approach */
"0XH303Z"  =  "1"  /* Insertion of Infusion Device into Left Shoulder Region, Open Approach */
"0XH30YZ"  =  "1"  /* Insertion of Other Device into Left Shoulder Region, Open Approach */
"0XH333Z"  =  "1"  /* Insertion of Infusion Device into Left Shoulder Region, Percutaneous Approach */
"0XH33YZ"  =  "1"  /* Insertion of Other Device into Left Shoulder Region, Percutaneous Approach */
"0XH343Z"  =  "1"  /* Insertion of Infusion Device into Left Shoulder Region, Percutaneous Endoscopic Approach */
"0XH34YZ"  =  "1"  /* Insertion of Other Device into Left Shoulder Region, Percutaneous Endoscopic Approach */
"0XH403Z"  =  "1"  /* Insertion of Infusion Device into Right Axilla, Open Approach */
"0XH40YZ"  =  "1"  /* Insertion of Other Device into Right Axilla, Open Approach */
"0XH433Z"  =  "1"  /* Insertion of Infusion Device into Right Axilla, Percutaneous Approach */
"0XH43YZ"  =  "1"  /* Insertion of Other Device into Right Axilla, Percutaneous Approach */
"0XH443Z"  =  "1"  /* Insertion of Infusion Device into Right Axilla, Percutaneous Endoscopic Approach */
"0XH44YZ"  =  "1"  /* Insertion of Other Device into Right Axilla, Percutaneous Endoscopic Approach */
"0XH503Z"  =  "1"  /* Insertion of Infusion Device into Left Axilla, Open Approach */
"0XH50YZ"  =  "1"  /* Insertion of Other Device into Left Axilla, Open Approach */
"0XH533Z"  =  "1"  /* Insertion of Infusion Device into Left Axilla, Percutaneous Approach */
"0XH53YZ"  =  "1"  /* Insertion of Other Device into Left Axilla, Percutaneous Approach */
"0XH543Z"  =  "1"  /* Insertion of Infusion Device into Left Axilla, Percutaneous Endoscopic Approach */
"0XH54YZ"  =  "1"  /* Insertion of Other Device into Left Axilla, Percutaneous Endoscopic Approach */
"0XH603Z"  =  "1"  /* Insertion of Infusion Device into Right Upper Extremity, Open Approach */
"0XH60YZ"  =  "1"  /* Insertion of Other Device into Right Upper Extremity, Open Approach */
"0XH633Z"  =  "1"  /* Insertion of Infusion Device into Right Upper Extremity, Percutaneous Approach */
"0XH63YZ"  =  "1"  /* Insertion of Other Device into Right Upper Extremity, Percutaneous Approach */
"0XH643Z"  =  "1"  /* Insertion of Infusion Device into Right Upper Extremity, Percutaneous Endoscopic Approach */
"0XH64YZ"  =  "1"  /* Insertion of Other Device into Right Upper Extremity, Percutaneous Endoscopic Approach */
"0XH703Z"  =  "1"  /* Insertion of Infusion Device into Left Upper Extremity, Open Approach */
"0XH70YZ"  =  "1"  /* Insertion of Other Device into Left Upper Extremity, Open Approach */
"0XH733Z"  =  "1"  /* Insertion of Infusion Device into Left Upper Extremity, Percutaneous Approach */
"0XH73YZ"  =  "1"  /* Insertion of Other Device into Left Upper Extremity, Percutaneous Approach */
"0XH743Z"  =  "1"  /* Insertion of Infusion Device into Left Upper Extremity, Percutaneous Endoscopic Approach */
"0XH74YZ"  =  "1"  /* Insertion of Other Device into Left Upper Extremity, Percutaneous Endoscopic Approach */
"0XH803Z"  =  "1"  /* Insertion of Infusion Device into Right Upper Arm, Open Approach */
"0XH80YZ"  =  "1"  /* Insertion of Other Device into Right Upper Arm, Open Approach */
"0XH833Z"  =  "1"  /* Insertion of Infusion Device into Right Upper Arm, Percutaneous Approach */
"0XH83YZ"  =  "1"  /* Insertion of Other Device into Right Upper Arm, Percutaneous Approach */
"0XH843Z"  =  "1"  /* Insertion of Infusion Device into Right Upper Arm, Percutaneous Endoscopic Approach */
"0XH84YZ"  =  "1"  /* Insertion of Other Device into Right Upper Arm, Percutaneous Endoscopic Approach */
"0XH903Z"  =  "1"  /* Insertion of Infusion Device into Left Upper Arm, Open Approach */
"0XH90YZ"  =  "1"  /* Insertion of Other Device into Left Upper Arm, Open Approach */
"0XH933Z"  =  "1"  /* Insertion of Infusion Device into Left Upper Arm, Percutaneous Approach */
"0XH93YZ"  =  "1"  /* Insertion of Other Device into Left Upper Arm, Percutaneous Approach */
"0XH943Z"  =  "1"  /* Insertion of Infusion Device into Left Upper Arm, Percutaneous Endoscopic Approach */
"0XH94YZ"  =  "1"  /* Insertion of Other Device into Left Upper Arm, Percutaneous Endoscopic Approach */
"0XHB03Z"  =  "1"  /* Insertion of Infusion Device into Right Elbow Region, Open Approach */
"0XHB0YZ"  =  "1"  /* Insertion of Other Device into Right Elbow Region, Open Approach */
"0XHB33Z"  =  "1"  /* Insertion of Infusion Device into Right Elbow Region, Percutaneous Approach */
"0XHB3YZ"  =  "1"  /* Insertion of Other Device into Right Elbow Region, Percutaneous Approach */
"0XHB43Z"  =  "1"  /* Insertion of Infusion Device into Right Elbow Region, Percutaneous Endoscopic Approach */
"0XHB4YZ"  =  "1"  /* Insertion of Other Device into Right Elbow Region, Percutaneous Endoscopic Approach */
"0XHC03Z"  =  "1"  /* Insertion of Infusion Device into Left Elbow Region, Open Approach */
"0XHC0YZ"  =  "1"  /* Insertion of Other Device into Left Elbow Region, Open Approach */
"0XHC33Z"  =  "1"  /* Insertion of Infusion Device into Left Elbow Region, Percutaneous Approach */
"0XHC3YZ"  =  "1"  /* Insertion of Other Device into Left Elbow Region, Percutaneous Approach */
"0XHC43Z"  =  "1"  /* Insertion of Infusion Device into Left Elbow Region, Percutaneous Endoscopic Approach */
"0XHC4YZ"  =  "1"  /* Insertion of Other Device into Left Elbow Region, Percutaneous Endoscopic Approach */
"0XHD03Z"  =  "1"  /* Insertion of Infusion Device into Right Lower Arm, Open Approach */
"0XHD0YZ"  =  "1"  /* Insertion of Other Device into Right Lower Arm, Open Approach */
"0XHD33Z"  =  "1"  /* Insertion of Infusion Device into Right Lower Arm, Percutaneous Approach */
"0XHD3YZ"  =  "1"  /* Insertion of Other Device into Right Lower Arm, Percutaneous Approach */
"0XHD43Z"  =  "1"  /* Insertion of Infusion Device into Right Lower Arm, Percutaneous Endoscopic Approach */
"0XHD4YZ"  =  "1"  /* Insertion of Other Device into Right Lower Arm, Percutaneous Endoscopic Approach */
"0XHF03Z"  =  "1"  /* Insertion of Infusion Device into Left Lower Arm, Open Approach */
"0XHF0YZ"  =  "1"  /* Insertion of Other Device into Left Lower Arm, Open Approach */
"0XHF33Z"  =  "1"  /* Insertion of Infusion Device into Left Lower Arm, Percutaneous Approach */
"0XHF3YZ"  =  "1"  /* Insertion of Other Device into Left Lower Arm, Percutaneous Approach */
"0XHF43Z"  =  "1"  /* Insertion of Infusion Device into Left Lower Arm, Percutaneous Endoscopic Approach */
"0XHF4YZ"  =  "1"  /* Insertion of Other Device into Left Lower Arm, Percutaneous Endoscopic Approach */
"0XHG03Z"  =  "1"  /* Insertion of Infusion Device into Right Wrist Region, Open Approach */
"0XHG0YZ"  =  "1"  /* Insertion of Other Device into Right Wrist Region, Open Approach */
"0XHG33Z"  =  "1"  /* Insertion of Infusion Device into Right Wrist Region, Percutaneous Approach */
"0XHG3YZ"  =  "1"  /* Insertion of Other Device into Right Wrist Region, Percutaneous Approach */
"0XHG43Z"  =  "1"  /* Insertion of Infusion Device into Right Wrist Region, Percutaneous Endoscopic Approach */
"0XHG4YZ"  =  "1"  /* Insertion of Other Device into Right Wrist Region, Percutaneous Endoscopic Approach */
"0XHH03Z"  =  "1"  /* Insertion of Infusion Device into Left Wrist Region, Open Approach */
"0XHH0YZ"  =  "1"  /* Insertion of Other Device into Left Wrist Region, Open Approach */
"0XHH33Z"  =  "1"  /* Insertion of Infusion Device into Left Wrist Region, Percutaneous Approach */
"0XHH3YZ"  =  "1"  /* Insertion of Other Device into Left Wrist Region, Percutaneous Approach */
"0XHH43Z"  =  "1"  /* Insertion of Infusion Device into Left Wrist Region, Percutaneous Endoscopic Approach */
"0XHH4YZ"  =  "1"  /* Insertion of Other Device into Left Wrist Region, Percutaneous Endoscopic Approach */
"0XHJ03Z"  =  "1"  /* Insertion of Infusion Device into Right Hand, Open Approach */
"0XHJ0YZ"  =  "1"  /* Insertion of Other Device into Right Hand, Open Approach */
"0XHJ33Z"  =  "1"  /* Insertion of Infusion Device into Right Hand, Percutaneous Approach */
"0XHJ3YZ"  =  "1"  /* Insertion of Other Device into Right Hand, Percutaneous Approach */
"0XHJ43Z"  =  "1"  /* Insertion of Infusion Device into Right Hand, Percutaneous Endoscopic Approach */
"0XHJ4YZ"  =  "1"  /* Insertion of Other Device into Right Hand, Percutaneous Endoscopic Approach */
"0XHK03Z"  =  "1"  /* Insertion of Infusion Device into Left Hand, Open Approach */
"0XHK0YZ"  =  "1"  /* Insertion of Other Device into Left Hand, Open Approach */
"0XHK33Z"  =  "1"  /* Insertion of Infusion Device into Left Hand, Percutaneous Approach */
"0XHK3YZ"  =  "1"  /* Insertion of Other Device into Left Hand, Percutaneous Approach */
"0XHK43Z"  =  "1"  /* Insertion of Infusion Device into Left Hand, Percutaneous Endoscopic Approach */
"0XHK4YZ"  =  "1"  /* Insertion of Other Device into Left Hand, Percutaneous Endoscopic Approach */
"0XJ20ZZ"  =  "1"  /* Inspection of Right Shoulder Region, Open Approach */
"0XJ30ZZ"  =  "1"  /* Inspection of Left Shoulder Region, Open Approach */
"0XJ40ZZ"  =  "1"  /* Inspection of Right Axilla, Open Approach */
"0XJ50ZZ"  =  "1"  /* Inspection of Left Axilla, Open Approach */
"0XJ60ZZ"  =  "1"  /* Inspection of Right Upper Extremity, Open Approach */
"0XJ70ZZ"  =  "1"  /* Inspection of Left Upper Extremity, Open Approach */
"0XJ80ZZ"  =  "1"  /* Inspection of Right Upper Arm, Open Approach */
"0XJ90ZZ"  =  "1"  /* Inspection of Left Upper Arm, Open Approach */
"0XJB0ZZ"  =  "1"  /* Inspection of Right Elbow Region, Open Approach */
"0XJC0ZZ"  =  "1"  /* Inspection of Left Elbow Region, Open Approach */
"0XJD0ZZ"  =  "1"  /* Inspection of Right Lower Arm, Open Approach */
"0XJF0ZZ"  =  "1"  /* Inspection of Left Lower Arm, Open Approach */
"0XJG0ZZ"  =  "1"  /* Inspection of Right Wrist Region, Open Approach */
"0XJH0ZZ"  =  "1"  /* Inspection of Left Wrist Region, Open Approach */
"0XJJ0ZZ"  =  "1"  /* Inspection of Right Hand, Open Approach */
"0XJK0ZZ"  =  "1"  /* Inspection of Left Hand, Open Approach */
"0XW600Z"  =  "1"  /* Revision of Drainage Device in Right Upper Extremity, Open Approach */
"0XW603Z"  =  "1"  /* Revision of Infusion Device in Right Upper Extremity, Open Approach */
"0XW607Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Right Upper Extremity, Open Approach */
"0XW60JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Upper Extremity, Open Approach */
"0XW60KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Right Upper Extremity, Open Approach */
"0XW60YZ"  =  "1"  /* Revision of Other Device in Right Upper Extremity, Open Approach */
"0XW630Z"  =  "1"  /* Revision of Drainage Device in Right Upper Extremity, Percutaneous Approach */
"0XW633Z"  =  "1"  /* Revision of Infusion Device in Right Upper Extremity, Percutaneous Approach */
"0XW637Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Right Upper Extremity, Percutaneous Approach */
"0XW63JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Upper Extremity, Percutaneous Approach */
"0XW63KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Right Upper Extremity, Percutaneous Approach */
"0XW63YZ"  =  "1"  /* Revision of Other Device in Right Upper Extremity, Percutaneous Approach */
"0XW640Z"  =  "1"  /* Revision of Drainage Device in Right Upper Extremity, Percutaneous Endoscopic Approach */
"0XW643Z"  =  "1"  /* Revision of Infusion Device in Right Upper Extremity, Percutaneous Endoscopic Approach */
"0XW647Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Right Upper Extremity, Percutaneous Endoscopic Approach */
"0XW64JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Upper Extremity, Percutaneous Endoscopic Approach */
"0XW64KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Right Upper Extremity, Percutaneous Endoscopic Approach */
"0XW64YZ"  =  "1"  /* Revision of Other Device in Right Upper Extremity, Percutaneous Endoscopic Approach */
"0XW700Z"  =  "1"  /* Revision of Drainage Device in Left Upper Extremity, Open Approach */
"0XW703Z"  =  "1"  /* Revision of Infusion Device in Left Upper Extremity, Open Approach */
"0XW707Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Left Upper Extremity, Open Approach */
"0XW70JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Upper Extremity, Open Approach */
"0XW70KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Left Upper Extremity, Open Approach */
"0XW70YZ"  =  "1"  /* Revision of Other Device in Left Upper Extremity, Open Approach */
"0XW730Z"  =  "1"  /* Revision of Drainage Device in Left Upper Extremity, Percutaneous Approach */
"0XW733Z"  =  "1"  /* Revision of Infusion Device in Left Upper Extremity, Percutaneous Approach */
"0XW737Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Left Upper Extremity, Percutaneous Approach */
"0XW73JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Upper Extremity, Percutaneous Approach */
"0XW73KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Left Upper Extremity, Percutaneous Approach */
"0XW73YZ"  =  "1"  /* Revision of Other Device in Left Upper Extremity, Percutaneous Approach */
"0XW740Z"  =  "1"  /* Revision of Drainage Device in Left Upper Extremity, Percutaneous Endoscopic Approach */
"0XW743Z"  =  "1"  /* Revision of Infusion Device in Left Upper Extremity, Percutaneous Endoscopic Approach */
"0XW747Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Left Upper Extremity, Percutaneous Endoscopic Approach */
"0XW74JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Upper Extremity, Percutaneous Endoscopic Approach */
"0XW74KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Left Upper Extremity, Percutaneous Endoscopic Approach */
"0XW74YZ"  =  "1"  /* Revision of Other Device in Left Upper Extremity, Percutaneous Endoscopic Approach */
"0Y300ZZ"  =  "1"  /* Control Bleeding in Right Buttock, Open Approach */
"0Y303ZZ"  =  "1"  /* Control Bleeding in Right Buttock, Percutaneous Approach */
"0Y304ZZ"  =  "1"  /* Control Bleeding in Right Buttock, Percutaneous Endoscopic Approach */
"0Y310ZZ"  =  "1"  /* Control Bleeding in Left Buttock, Open Approach */
"0Y313ZZ"  =  "1"  /* Control Bleeding in Left Buttock, Percutaneous Approach */
"0Y314ZZ"  =  "1"  /* Control Bleeding in Left Buttock, Percutaneous Endoscopic Approach */
"0Y350ZZ"  =  "1"  /* Control Bleeding in Right Inguinal Region, Open Approach */
"0Y353ZZ"  =  "1"  /* Control Bleeding in Right Inguinal Region, Percutaneous Approach */
"0Y354ZZ"  =  "1"  /* Control Bleeding in Right Inguinal Region, Percutaneous Endoscopic Approach */
"0Y360ZZ"  =  "1"  /* Control Bleeding in Left Inguinal Region, Open Approach */
"0Y363ZZ"  =  "1"  /* Control Bleeding in Left Inguinal Region, Percutaneous Approach */
"0Y364ZZ"  =  "1"  /* Control Bleeding in Left Inguinal Region, Percutaneous Endoscopic Approach */
"0Y370ZZ"  =  "1"  /* Control Bleeding in Right Femoral Region, Open Approach */
"0Y373ZZ"  =  "1"  /* Control Bleeding in Right Femoral Region, Percutaneous Approach */
"0Y374ZZ"  =  "1"  /* Control Bleeding in Right Femoral Region, Percutaneous Endoscopic Approach */
"0Y380ZZ"  =  "1"  /* Control Bleeding in Left Femoral Region, Open Approach */
"0Y383ZZ"  =  "1"  /* Control Bleeding in Left Femoral Region, Percutaneous Approach */
"0Y384ZZ"  =  "1"  /* Control Bleeding in Left Femoral Region, Percutaneous Endoscopic Approach */
"0Y390ZZ"  =  "1"  /* Control Bleeding in Right Lower Extremity, Open Approach */
"0Y393ZZ"  =  "1"  /* Control Bleeding in Right Lower Extremity, Percutaneous Approach */
"0Y394ZZ"  =  "1"  /* Control Bleeding in Right Lower Extremity, Percutaneous Endoscopic Approach */
"0Y3B0ZZ"  =  "1"  /* Control Bleeding in Left Lower Extremity, Open Approach */
"0Y3B3ZZ"  =  "1"  /* Control Bleeding in Left Lower Extremity, Percutaneous Approach */
"0Y3B4ZZ"  =  "1"  /* Control Bleeding in Left Lower Extremity, Percutaneous Endoscopic Approach */
"0Y3C0ZZ"  =  "1"  /* Control Bleeding in Right Upper Leg, Open Approach */
"0Y3C3ZZ"  =  "1"  /* Control Bleeding in Right Upper Leg, Percutaneous Approach */
"0Y3C4ZZ"  =  "1"  /* Control Bleeding in Right Upper Leg, Percutaneous Endoscopic Approach */
"0Y3D0ZZ"  =  "1"  /* Control Bleeding in Left Upper Leg, Open Approach */
"0Y3D3ZZ"  =  "1"  /* Control Bleeding in Left Upper Leg, Percutaneous Approach */
"0Y3D4ZZ"  =  "1"  /* Control Bleeding in Left Upper Leg, Percutaneous Endoscopic Approach */
"0Y3F0ZZ"  =  "1"  /* Control Bleeding in Right Knee Region, Open Approach */
"0Y3F3ZZ"  =  "1"  /* Control Bleeding in Right Knee Region, Percutaneous Approach */
"0Y3F4ZZ"  =  "1"  /* Control Bleeding in Right Knee Region, Percutaneous Endoscopic Approach */
"0Y3G0ZZ"  =  "1"  /* Control Bleeding in Left Knee Region, Open Approach */
"0Y3G3ZZ"  =  "1"  /* Control Bleeding in Left Knee Region, Percutaneous Approach */
"0Y3G4ZZ"  =  "1"  /* Control Bleeding in Left Knee Region, Percutaneous Endoscopic Approach */
"0Y3H0ZZ"  =  "1"  /* Control Bleeding in Right Lower Leg, Open Approach */
"0Y3H3ZZ"  =  "1"  /* Control Bleeding in Right Lower Leg, Percutaneous Approach */
"0Y3H4ZZ"  =  "1"  /* Control Bleeding in Right Lower Leg, Percutaneous Endoscopic Approach */
"0Y3J0ZZ"  =  "1"  /* Control Bleeding in Left Lower Leg, Open Approach */
"0Y3J3ZZ"  =  "1"  /* Control Bleeding in Left Lower Leg, Percutaneous Approach */
"0Y3J4ZZ"  =  "1"  /* Control Bleeding in Left Lower Leg, Percutaneous Endoscopic Approach */
"0Y3K0ZZ"  =  "1"  /* Control Bleeding in Right Ankle Region, Open Approach */
"0Y3K3ZZ"  =  "1"  /* Control Bleeding in Right Ankle Region, Percutaneous Approach */
"0Y3K4ZZ"  =  "1"  /* Control Bleeding in Right Ankle Region, Percutaneous Endoscopic Approach */
"0Y3L0ZZ"  =  "1"  /* Control Bleeding in Left Ankle Region, Open Approach */
"0Y3L3ZZ"  =  "1"  /* Control Bleeding in Left Ankle Region, Percutaneous Approach */
"0Y3L4ZZ"  =  "1"  /* Control Bleeding in Left Ankle Region, Percutaneous Endoscopic Approach */
"0Y3M0ZZ"  =  "1"  /* Control Bleeding in Right Foot, Open Approach */
"0Y3M3ZZ"  =  "1"  /* Control Bleeding in Right Foot, Percutaneous Approach */
"0Y3M4ZZ"  =  "1"  /* Control Bleeding in Right Foot, Percutaneous Endoscopic Approach */
"0Y3N0ZZ"  =  "1"  /* Control Bleeding in Left Foot, Open Approach */
"0Y3N3ZZ"  =  "1"  /* Control Bleeding in Left Foot, Percutaneous Approach */
"0Y3N4ZZ"  =  "1"  /* Control Bleeding in Left Foot, Percutaneous Endoscopic Approach */
"0Y9000Z"  =  "1"  /* Drainage of Right Buttock with Drainage Device, Open Approach */
"0Y900ZZ"  =  "1"  /* Drainage of Right Buttock, Open Approach */
"0Y9030Z"  =  "1"  /* Drainage of Right Buttock with Drainage Device, Percutaneous Approach */
"0Y903ZZ"  =  "1"  /* Drainage of Right Buttock, Percutaneous Approach */
"0Y9040Z"  =  "1"  /* Drainage of Right Buttock with Drainage Device, Percutaneous Endoscopic Approach */
"0Y904ZZ"  =  "1"  /* Drainage of Right Buttock, Percutaneous Endoscopic Approach */
"0Y9100Z"  =  "1"  /* Drainage of Left Buttock with Drainage Device, Open Approach */
"0Y910ZZ"  =  "1"  /* Drainage of Left Buttock, Open Approach */
"0Y9130Z"  =  "1"  /* Drainage of Left Buttock with Drainage Device, Percutaneous Approach */
"0Y913ZZ"  =  "1"  /* Drainage of Left Buttock, Percutaneous Approach */
"0Y9140Z"  =  "1"  /* Drainage of Left Buttock with Drainage Device, Percutaneous Endoscopic Approach */
"0Y914ZZ"  =  "1"  /* Drainage of Left Buttock, Percutaneous Endoscopic Approach */
"0Y9500Z"  =  "1"  /* Drainage of Right Inguinal Region with Drainage Device, Open Approach */
"0Y950ZZ"  =  "1"  /* Drainage of Right Inguinal Region, Open Approach */
"0Y9530Z"  =  "1"  /* Drainage of Right Inguinal Region with Drainage Device, Percutaneous Approach */
"0Y953ZZ"  =  "1"  /* Drainage of Right Inguinal Region, Percutaneous Approach */
"0Y9540Z"  =  "1"  /* Drainage of Right Inguinal Region with Drainage Device, Percutaneous Endoscopic Approach */
"0Y954ZZ"  =  "1"  /* Drainage of Right Inguinal Region, Percutaneous Endoscopic Approach */
"0Y9600Z"  =  "1"  /* Drainage of Left Inguinal Region with Drainage Device, Open Approach */
"0Y960ZZ"  =  "1"  /* Drainage of Left Inguinal Region, Open Approach */
"0Y9630Z"  =  "1"  /* Drainage of Left Inguinal Region with Drainage Device, Percutaneous Approach */
"0Y963ZZ"  =  "1"  /* Drainage of Left Inguinal Region, Percutaneous Approach */
"0Y9640Z"  =  "1"  /* Drainage of Left Inguinal Region with Drainage Device, Percutaneous Endoscopic Approach */
"0Y964ZZ"  =  "1"  /* Drainage of Left Inguinal Region, Percutaneous Endoscopic Approach */
"0Y9700Z"  =  "1"  /* Drainage of Right Femoral Region with Drainage Device, Open Approach */
"0Y970ZZ"  =  "1"  /* Drainage of Right Femoral Region, Open Approach */
"0Y9730Z"  =  "1"  /* Drainage of Right Femoral Region with Drainage Device, Percutaneous Approach */
"0Y973ZZ"  =  "1"  /* Drainage of Right Femoral Region, Percutaneous Approach */
"0Y9740Z"  =  "1"  /* Drainage of Right Femoral Region with Drainage Device, Percutaneous Endoscopic Approach */
"0Y974ZZ"  =  "1"  /* Drainage of Right Femoral Region, Percutaneous Endoscopic Approach */
"0Y9800Z"  =  "1"  /* Drainage of Left Femoral Region with Drainage Device, Open Approach */
"0Y980ZZ"  =  "1"  /* Drainage of Left Femoral Region, Open Approach */
"0Y9830Z"  =  "1"  /* Drainage of Left Femoral Region with Drainage Device, Percutaneous Approach */
"0Y983ZZ"  =  "1"  /* Drainage of Left Femoral Region, Percutaneous Approach */
"0Y9840Z"  =  "1"  /* Drainage of Left Femoral Region with Drainage Device, Percutaneous Endoscopic Approach */
"0Y984ZZ"  =  "1"  /* Drainage of Left Femoral Region, Percutaneous Endoscopic Approach */
"0Y9900Z"  =  "1"  /* Drainage of Right Lower Extremity with Drainage Device, Open Approach */
"0Y990ZZ"  =  "1"  /* Drainage of Right Lower Extremity, Open Approach */
"0Y9930Z"  =  "1"  /* Drainage of Right Lower Extremity with Drainage Device, Percutaneous Approach */
"0Y993ZZ"  =  "1"  /* Drainage of Right Lower Extremity, Percutaneous Approach */
"0Y9940Z"  =  "1"  /* Drainage of Right Lower Extremity with Drainage Device, Percutaneous Endoscopic Approach */
"0Y994ZZ"  =  "1"  /* Drainage of Right Lower Extremity, Percutaneous Endoscopic Approach */
"0Y9B00Z"  =  "1"  /* Drainage of Left Lower Extremity with Drainage Device, Open Approach */
"0Y9B0ZZ"  =  "1"  /* Drainage of Left Lower Extremity, Open Approach */
"0Y9B30Z"  =  "1"  /* Drainage of Left Lower Extremity with Drainage Device, Percutaneous Approach */
"0Y9B3ZZ"  =  "1"  /* Drainage of Left Lower Extremity, Percutaneous Approach */
"0Y9B40Z"  =  "1"  /* Drainage of Left Lower Extremity with Drainage Device, Percutaneous Endoscopic Approach */
"0Y9B4ZZ"  =  "1"  /* Drainage of Left Lower Extremity, Percutaneous Endoscopic Approach */
"0Y9C00Z"  =  "1"  /* Drainage of Right Upper Leg with Drainage Device, Open Approach */
"0Y9C0ZZ"  =  "1"  /* Drainage of Right Upper Leg, Open Approach */
"0Y9C30Z"  =  "1"  /* Drainage of Right Upper Leg with Drainage Device, Percutaneous Approach */
"0Y9C3ZZ"  =  "1"  /* Drainage of Right Upper Leg, Percutaneous Approach */
"0Y9C40Z"  =  "1"  /* Drainage of Right Upper Leg with Drainage Device, Percutaneous Endoscopic Approach */
"0Y9C4ZZ"  =  "1"  /* Drainage of Right Upper Leg, Percutaneous Endoscopic Approach */
"0Y9D00Z"  =  "1"  /* Drainage of Left Upper Leg with Drainage Device, Open Approach */
"0Y9D0ZZ"  =  "1"  /* Drainage of Left Upper Leg, Open Approach */
"0Y9D30Z"  =  "1"  /* Drainage of Left Upper Leg with Drainage Device, Percutaneous Approach */
"0Y9D3ZZ"  =  "1"  /* Drainage of Left Upper Leg, Percutaneous Approach */
"0Y9D40Z"  =  "1"  /* Drainage of Left Upper Leg with Drainage Device, Percutaneous Endoscopic Approach */
"0Y9D4ZZ"  =  "1"  /* Drainage of Left Upper Leg, Percutaneous Endoscopic Approach */
"0Y9F00Z"  =  "1"  /* Drainage of Right Knee Region with Drainage Device, Open Approach */
"0Y9F0ZZ"  =  "1"  /* Drainage of Right Knee Region, Open Approach */
"0Y9F30Z"  =  "1"  /* Drainage of Right Knee Region with Drainage Device, Percutaneous Approach */
"0Y9F3ZZ"  =  "1"  /* Drainage of Right Knee Region, Percutaneous Approach */
"0Y9F40Z"  =  "1"  /* Drainage of Right Knee Region with Drainage Device, Percutaneous Endoscopic Approach */
"0Y9F4ZZ"  =  "1"  /* Drainage of Right Knee Region, Percutaneous Endoscopic Approach */
"0Y9G00Z"  =  "1"  /* Drainage of Left Knee Region with Drainage Device, Open Approach */
"0Y9G0ZZ"  =  "1"  /* Drainage of Left Knee Region, Open Approach */
"0Y9G30Z"  =  "1"  /* Drainage of Left Knee Region with Drainage Device, Percutaneous Approach */
"0Y9G3ZZ"  =  "1"  /* Drainage of Left Knee Region, Percutaneous Approach */
"0Y9G40Z"  =  "1"  /* Drainage of Left Knee Region with Drainage Device, Percutaneous Endoscopic Approach */
"0Y9G4ZZ"  =  "1"  /* Drainage of Left Knee Region, Percutaneous Endoscopic Approach */
"0Y9H00Z"  =  "1"  /* Drainage of Right Lower Leg with Drainage Device, Open Approach */
"0Y9H0ZZ"  =  "1"  /* Drainage of Right Lower Leg, Open Approach */
"0Y9H30Z"  =  "1"  /* Drainage of Right Lower Leg with Drainage Device, Percutaneous Approach */
"0Y9H3ZZ"  =  "1"  /* Drainage of Right Lower Leg, Percutaneous Approach */
"0Y9H40Z"  =  "1"  /* Drainage of Right Lower Leg with Drainage Device, Percutaneous Endoscopic Approach */
"0Y9H4ZZ"  =  "1"  /* Drainage of Right Lower Leg, Percutaneous Endoscopic Approach */
"0Y9J00Z"  =  "1"  /* Drainage of Left Lower Leg with Drainage Device, Open Approach */
"0Y9J0ZZ"  =  "1"  /* Drainage of Left Lower Leg, Open Approach */
"0Y9J30Z"  =  "1"  /* Drainage of Left Lower Leg with Drainage Device, Percutaneous Approach */
"0Y9J3ZZ"  =  "1"  /* Drainage of Left Lower Leg, Percutaneous Approach */
"0Y9J40Z"  =  "1"  /* Drainage of Left Lower Leg with Drainage Device, Percutaneous Endoscopic Approach */
"0Y9J4ZZ"  =  "1"  /* Drainage of Left Lower Leg, Percutaneous Endoscopic Approach */
"0Y9K00Z"  =  "1"  /* Drainage of Right Ankle Region with Drainage Device, Open Approach */
"0Y9K0ZZ"  =  "1"  /* Drainage of Right Ankle Region, Open Approach */
"0Y9K30Z"  =  "1"  /* Drainage of Right Ankle Region with Drainage Device, Percutaneous Approach */
"0Y9K3ZZ"  =  "1"  /* Drainage of Right Ankle Region, Percutaneous Approach */
"0Y9K40Z"  =  "1"  /* Drainage of Right Ankle Region with Drainage Device, Percutaneous Endoscopic Approach */
"0Y9K4ZZ"  =  "1"  /* Drainage of Right Ankle Region, Percutaneous Endoscopic Approach */
"0Y9L00Z"  =  "1"  /* Drainage of Left Ankle Region with Drainage Device, Open Approach */
"0Y9L0ZZ"  =  "1"  /* Drainage of Left Ankle Region, Open Approach */
"0Y9L30Z"  =  "1"  /* Drainage of Left Ankle Region with Drainage Device, Percutaneous Approach */
"0Y9L3ZZ"  =  "1"  /* Drainage of Left Ankle Region, Percutaneous Approach */
"0Y9L40Z"  =  "1"  /* Drainage of Left Ankle Region with Drainage Device, Percutaneous Endoscopic Approach */
"0Y9L4ZZ"  =  "1"  /* Drainage of Left Ankle Region, Percutaneous Endoscopic Approach */
"0Y9M00Z"  =  "1"  /* Drainage of Right Foot with Drainage Device, Open Approach */
"0Y9M0ZZ"  =  "1"  /* Drainage of Right Foot, Open Approach */
"0Y9M30Z"  =  "1"  /* Drainage of Right Foot with Drainage Device, Percutaneous Approach */
"0Y9M3ZZ"  =  "1"  /* Drainage of Right Foot, Percutaneous Approach */
"0Y9M40Z"  =  "1"  /* Drainage of Right Foot with Drainage Device, Percutaneous Endoscopic Approach */
"0Y9M4ZZ"  =  "1"  /* Drainage of Right Foot, Percutaneous Endoscopic Approach */
"0Y9N00Z"  =  "1"  /* Drainage of Left Foot with Drainage Device, Open Approach */
"0Y9N0ZZ"  =  "1"  /* Drainage of Left Foot, Open Approach */
"0Y9N30Z"  =  "1"  /* Drainage of Left Foot with Drainage Device, Percutaneous Approach */
"0Y9N3ZZ"  =  "1"  /* Drainage of Left Foot, Percutaneous Approach */
"0Y9N40Z"  =  "1"  /* Drainage of Left Foot with Drainage Device, Percutaneous Endoscopic Approach */
"0Y9N4ZZ"  =  "1"  /* Drainage of Left Foot, Percutaneous Endoscopic Approach */
"0YH003Z"  =  "1"  /* Insertion of Infusion Device into Right Buttock, Open Approach */
"0YH00YZ"  =  "1"  /* Insertion of Other Device into Right Buttock, Open Approach */
"0YH033Z"  =  "1"  /* Insertion of Infusion Device into Right Buttock, Percutaneous Approach */
"0YH03YZ"  =  "1"  /* Insertion of Other Device into Right Buttock, Percutaneous Approach */
"0YH043Z"  =  "1"  /* Insertion of Infusion Device into Right Buttock, Percutaneous Endoscopic Approach */
"0YH04YZ"  =  "1"  /* Insertion of Other Device into Right Buttock, Percutaneous Endoscopic Approach */
"0YH103Z"  =  "1"  /* Insertion of Infusion Device into Left Buttock, Open Approach */
"0YH10YZ"  =  "1"  /* Insertion of Other Device into Left Buttock, Open Approach */
"0YH133Z"  =  "1"  /* Insertion of Infusion Device into Left Buttock, Percutaneous Approach */
"0YH13YZ"  =  "1"  /* Insertion of Other Device into Left Buttock, Percutaneous Approach */
"0YH143Z"  =  "1"  /* Insertion of Infusion Device into Left Buttock, Percutaneous Endoscopic Approach */
"0YH14YZ"  =  "1"  /* Insertion of Other Device into Left Buttock, Percutaneous Endoscopic Approach */
"0YH503Z"  =  "1"  /* Insertion of Infusion Device into Right Inguinal Region, Open Approach */
"0YH50YZ"  =  "1"  /* Insertion of Other Device into Right Inguinal Region, Open Approach */
"0YH533Z"  =  "1"  /* Insertion of Infusion Device into Right Inguinal Region, Percutaneous Approach */
"0YH53YZ"  =  "1"  /* Insertion of Other Device into Right Inguinal Region, Percutaneous Approach */
"0YH543Z"  =  "1"  /* Insertion of Infusion Device into Right Inguinal Region, Percutaneous Endoscopic Approach */
"0YH54YZ"  =  "1"  /* Insertion of Other Device into Right Inguinal Region, Percutaneous Endoscopic Approach */
"0YH603Z"  =  "1"  /* Insertion of Infusion Device into Left Inguinal Region, Open Approach */
"0YH60YZ"  =  "1"  /* Insertion of Other Device into Left Inguinal Region, Open Approach */
"0YH633Z"  =  "1"  /* Insertion of Infusion Device into Left Inguinal Region, Percutaneous Approach */
"0YH63YZ"  =  "1"  /* Insertion of Other Device into Left Inguinal Region, Percutaneous Approach */
"0YH643Z"  =  "1"  /* Insertion of Infusion Device into Left Inguinal Region, Percutaneous Endoscopic Approach */
"0YH64YZ"  =  "1"  /* Insertion of Other Device into Left Inguinal Region, Percutaneous Endoscopic Approach */
"0YH703Z"  =  "1"  /* Insertion of Infusion Device into Right Femoral Region, Open Approach */
"0YH70YZ"  =  "1"  /* Insertion of Other Device into Right Femoral Region, Open Approach */
"0YH733Z"  =  "1"  /* Insertion of Infusion Device into Right Femoral Region, Percutaneous Approach */
"0YH73YZ"  =  "1"  /* Insertion of Other Device into Right Femoral Region, Percutaneous Approach */
"0YH743Z"  =  "1"  /* Insertion of Infusion Device into Right Femoral Region, Percutaneous Endoscopic Approach */
"0YH74YZ"  =  "1"  /* Insertion of Other Device into Right Femoral Region, Percutaneous Endoscopic Approach */
"0YH803Z"  =  "1"  /* Insertion of Infusion Device into Left Femoral Region, Open Approach */
"0YH80YZ"  =  "1"  /* Insertion of Other Device into Left Femoral Region, Open Approach */
"0YH833Z"  =  "1"  /* Insertion of Infusion Device into Left Femoral Region, Percutaneous Approach */
"0YH83YZ"  =  "1"  /* Insertion of Other Device into Left Femoral Region, Percutaneous Approach */
"0YH843Z"  =  "1"  /* Insertion of Infusion Device into Left Femoral Region, Percutaneous Endoscopic Approach */
"0YH84YZ"  =  "1"  /* Insertion of Other Device into Left Femoral Region, Percutaneous Endoscopic Approach */
"0YH903Z"  =  "1"  /* Insertion of Infusion Device into Right Lower Extremity, Open Approach */
"0YH90YZ"  =  "1"  /* Insertion of Other Device into Right Lower Extremity, Open Approach */
"0YH933Z"  =  "1"  /* Insertion of Infusion Device into Right Lower Extremity, Percutaneous Approach */
"0YH93YZ"  =  "1"  /* Insertion of Other Device into Right Lower Extremity, Percutaneous Approach */
"0YH943Z"  =  "1"  /* Insertion of Infusion Device into Right Lower Extremity, Percutaneous Endoscopic Approach */
"0YH94YZ"  =  "1"  /* Insertion of Other Device into Right Lower Extremity, Percutaneous Endoscopic Approach */
"0YHB03Z"  =  "1"  /* Insertion of Infusion Device into Left Lower Extremity, Open Approach */
"0YHB0YZ"  =  "1"  /* Insertion of Other Device into Left Lower Extremity, Open Approach */
"0YHB33Z"  =  "1"  /* Insertion of Infusion Device into Left Lower Extremity, Percutaneous Approach */
"0YHB3YZ"  =  "1"  /* Insertion of Other Device into Left Lower Extremity, Percutaneous Approach */
"0YHB43Z"  =  "1"  /* Insertion of Infusion Device into Left Lower Extremity, Percutaneous Endoscopic Approach */
"0YHB4YZ"  =  "1"  /* Insertion of Other Device into Left Lower Extremity, Percutaneous Endoscopic Approach */
"0YHC03Z"  =  "1"  /* Insertion of Infusion Device into Right Upper Leg, Open Approach */
"0YHC0YZ"  =  "1"  /* Insertion of Other Device into Right Upper Leg, Open Approach */
"0YHC33Z"  =  "1"  /* Insertion of Infusion Device into Right Upper Leg, Percutaneous Approach */
"0YHC3YZ"  =  "1"  /* Insertion of Other Device into Right Upper Leg, Percutaneous Approach */
"0YHC43Z"  =  "1"  /* Insertion of Infusion Device into Right Upper Leg, Percutaneous Endoscopic Approach */
"0YHC4YZ"  =  "1"  /* Insertion of Other Device into Right Upper Leg, Percutaneous Endoscopic Approach */
"0YHD03Z"  =  "1"  /* Insertion of Infusion Device into Left Upper Leg, Open Approach */
"0YHD0YZ"  =  "1"  /* Insertion of Other Device into Left Upper Leg, Open Approach */
"0YHD33Z"  =  "1"  /* Insertion of Infusion Device into Left Upper Leg, Percutaneous Approach */
"0YHD3YZ"  =  "1"  /* Insertion of Other Device into Left Upper Leg, Percutaneous Approach */
"0YHD43Z"  =  "1"  /* Insertion of Infusion Device into Left Upper Leg, Percutaneous Endoscopic Approach */
"0YHD4YZ"  =  "1"  /* Insertion of Other Device into Left Upper Leg, Percutaneous Endoscopic Approach */
"0YHF03Z"  =  "1"  /* Insertion of Infusion Device into Right Knee Region, Open Approach */
"0YHF0YZ"  =  "1"  /* Insertion of Other Device into Right Knee Region, Open Approach */
"0YHF33Z"  =  "1"  /* Insertion of Infusion Device into Right Knee Region, Percutaneous Approach */
"0YHF3YZ"  =  "1"  /* Insertion of Other Device into Right Knee Region, Percutaneous Approach */
"0YHF43Z"  =  "1"  /* Insertion of Infusion Device into Right Knee Region, Percutaneous Endoscopic Approach */
"0YHF4YZ"  =  "1"  /* Insertion of Other Device into Right Knee Region, Percutaneous Endoscopic Approach */
"0YHG03Z"  =  "1"  /* Insertion of Infusion Device into Left Knee Region, Open Approach */
"0YHG0YZ"  =  "1"  /* Insertion of Other Device into Left Knee Region, Open Approach */
"0YHG33Z"  =  "1"  /* Insertion of Infusion Device into Left Knee Region, Percutaneous Approach */
"0YHG3YZ"  =  "1"  /* Insertion of Other Device into Left Knee Region, Percutaneous Approach */
"0YHG43Z"  =  "1"  /* Insertion of Infusion Device into Left Knee Region, Percutaneous Endoscopic Approach */
"0YHG4YZ"  =  "1"  /* Insertion of Other Device into Left Knee Region, Percutaneous Endoscopic Approach */
"0YHH03Z"  =  "1"  /* Insertion of Infusion Device into Right Lower Leg, Open Approach */
"0YHH0YZ"  =  "1"  /* Insertion of Other Device into Right Lower Leg, Open Approach */
"0YHH33Z"  =  "1"  /* Insertion of Infusion Device into Right Lower Leg, Percutaneous Approach */
"0YHH3YZ"  =  "1"  /* Insertion of Other Device into Right Lower Leg, Percutaneous Approach */
"0YHH43Z"  =  "1"  /* Insertion of Infusion Device into Right Lower Leg, Percutaneous Endoscopic Approach */
"0YHH4YZ"  =  "1"  /* Insertion of Other Device into Right Lower Leg, Percutaneous Endoscopic Approach */
"0YHJ03Z"  =  "1"  /* Insertion of Infusion Device into Left Lower Leg, Open Approach */
"0YHJ0YZ"  =  "1"  /* Insertion of Other Device into Left Lower Leg, Open Approach */
"0YHJ33Z"  =  "1"  /* Insertion of Infusion Device into Left Lower Leg, Percutaneous Approach */
"0YHJ3YZ"  =  "1"  /* Insertion of Other Device into Left Lower Leg, Percutaneous Approach */
"0YHJ43Z"  =  "1"  /* Insertion of Infusion Device into Left Lower Leg, Percutaneous Endoscopic Approach */
"0YHJ4YZ"  =  "1"  /* Insertion of Other Device into Left Lower Leg, Percutaneous Endoscopic Approach */
"0YHK03Z"  =  "1"  /* Insertion of Infusion Device into Right Ankle Region, Open Approach */
"0YHK0YZ"  =  "1"  /* Insertion of Other Device into Right Ankle Region, Open Approach */
"0YHK33Z"  =  "1"  /* Insertion of Infusion Device into Right Ankle Region, Percutaneous Approach */
"0YHK3YZ"  =  "1"  /* Insertion of Other Device into Right Ankle Region, Percutaneous Approach */
"0YHK43Z"  =  "1"  /* Insertion of Infusion Device into Right Ankle Region, Percutaneous Endoscopic Approach */
"0YHK4YZ"  =  "1"  /* Insertion of Other Device into Right Ankle Region, Percutaneous Endoscopic Approach */
"0YHL03Z"  =  "1"  /* Insertion of Infusion Device into Left Ankle Region, Open Approach */
"0YHL0YZ"  =  "1"  /* Insertion of Other Device into Left Ankle Region, Open Approach */
"0YHL33Z"  =  "1"  /* Insertion of Infusion Device into Left Ankle Region, Percutaneous Approach */
"0YHL3YZ"  =  "1"  /* Insertion of Other Device into Left Ankle Region, Percutaneous Approach */
"0YHL43Z"  =  "1"  /* Insertion of Infusion Device into Left Ankle Region, Percutaneous Endoscopic Approach */
"0YHL4YZ"  =  "1"  /* Insertion of Other Device into Left Ankle Region, Percutaneous Endoscopic Approach */
"0YHM03Z"  =  "1"  /* Insertion of Infusion Device into Right Foot, Open Approach */
"0YHM0YZ"  =  "1"  /* Insertion of Other Device into Right Foot, Open Approach */
"0YHM33Z"  =  "1"  /* Insertion of Infusion Device into Right Foot, Percutaneous Approach */
"0YHM3YZ"  =  "1"  /* Insertion of Other Device into Right Foot, Percutaneous Approach */
"0YHM43Z"  =  "1"  /* Insertion of Infusion Device into Right Foot, Percutaneous Endoscopic Approach */
"0YHM4YZ"  =  "1"  /* Insertion of Other Device into Right Foot, Percutaneous Endoscopic Approach */
"0YHN03Z"  =  "1"  /* Insertion of Infusion Device into Left Foot, Open Approach */
"0YHN0YZ"  =  "1"  /* Insertion of Other Device into Left Foot, Open Approach */
"0YHN33Z"  =  "1"  /* Insertion of Infusion Device into Left Foot, Percutaneous Approach */
"0YHN3YZ"  =  "1"  /* Insertion of Other Device into Left Foot, Percutaneous Approach */
"0YHN43Z"  =  "1"  /* Insertion of Infusion Device into Left Foot, Percutaneous Endoscopic Approach */
"0YHN4YZ"  =  "1"  /* Insertion of Other Device into Left Foot, Percutaneous Endoscopic Approach */
"0YJ00ZZ"  =  "1"  /* Inspection of Right Buttock, Open Approach */
"0YJ10ZZ"  =  "1"  /* Inspection of Left Buttock, Open Approach */
"0YJ50ZZ"  =  "1"  /* Inspection of Right Inguinal Region, Open Approach */
"0YJ60ZZ"  =  "1"  /* Inspection of Left Inguinal Region, Open Approach */
"0YJ70ZZ"  =  "1"  /* Inspection of Right Femoral Region, Open Approach */
"0YJ80ZZ"  =  "1"  /* Inspection of Left Femoral Region, Open Approach */
"0YJ90ZZ"  =  "1"  /* Inspection of Right Lower Extremity, Open Approach */
"0YJA0ZZ"  =  "1"  /* Inspection of Bilateral Inguinal Region, Open Approach */
"0YJB0ZZ"  =  "1"  /* Inspection of Left Lower Extremity, Open Approach */
"0YJC0ZZ"  =  "1"  /* Inspection of Right Upper Leg, Open Approach */
"0YJD0ZZ"  =  "1"  /* Inspection of Left Upper Leg, Open Approach */
"0YJE0ZZ"  =  "1"  /* Inspection of Bilateral Femoral Region, Open Approach */
"0YJF0ZZ"  =  "1"  /* Inspection of Right Knee Region, Open Approach */
"0YJG0ZZ"  =  "1"  /* Inspection of Left Knee Region, Open Approach */
"0YJH0ZZ"  =  "1"  /* Inspection of Right Lower Leg, Open Approach */
"0YJJ0ZZ"  =  "1"  /* Inspection of Left Lower Leg, Open Approach */
"0YJK0ZZ"  =  "1"  /* Inspection of Right Ankle Region, Open Approach */
"0YJL0ZZ"  =  "1"  /* Inspection of Left Ankle Region, Open Approach */
"0YJM0ZZ"  =  "1"  /* Inspection of Right Foot, Open Approach */
"0YJN0ZZ"  =  "1"  /* Inspection of Left Foot, Open Approach */
"0YW900Z"  =  "1"  /* Revision of Drainage Device in Right Lower Extremity, Open Approach */
"0YW903Z"  =  "1"  /* Revision of Infusion Device in Right Lower Extremity, Open Approach */
"0YW907Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Right Lower Extremity, Open Approach */
"0YW90JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Lower Extremity, Open Approach */
"0YW90KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Right Lower Extremity, Open Approach */
"0YW90YZ"  =  "1"  /* Revision of Other Device in Right Lower Extremity, Open Approach */
"0YW930Z"  =  "1"  /* Revision of Drainage Device in Right Lower Extremity, Percutaneous Approach */
"0YW933Z"  =  "1"  /* Revision of Infusion Device in Right Lower Extremity, Percutaneous Approach */
"0YW937Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Right Lower Extremity, Percutaneous Approach */
"0YW93JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Lower Extremity, Percutaneous Approach */
"0YW93KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Right Lower Extremity, Percutaneous Approach */
"0YW93YZ"  =  "1"  /* Revision of Other Device in Right Lower Extremity, Percutaneous Approach */
"0YW940Z"  =  "1"  /* Revision of Drainage Device in Right Lower Extremity, Percutaneous Endoscopic Approach */
"0YW943Z"  =  "1"  /* Revision of Infusion Device in Right Lower Extremity, Percutaneous Endoscopic Approach */
"0YW947Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Right Lower Extremity, Percutaneous Endoscopic Approach */
"0YW94JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Lower Extremity, Percutaneous Endoscopic Approach */
"0YW94KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Right Lower Extremity, Percutaneous Endoscopic Approach */
"0YW94YZ"  =  "1"  /* Revision of Other Device in Right Lower Extremity, Percutaneous Endoscopic Approach */
"0YWB00Z"  =  "1"  /* Revision of Drainage Device in Left Lower Extremity, Open Approach */
"0YWB03Z"  =  "1"  /* Revision of Infusion Device in Left Lower Extremity, Open Approach */
"0YWB07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Left Lower Extremity, Open Approach */
"0YWB0JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Lower Extremity, Open Approach */
"0YWB0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Left Lower Extremity, Open Approach */
"0YWB0YZ"  =  "1"  /* Revision of Other Device in Left Lower Extremity, Open Approach */
"0YWB30Z"  =  "1"  /* Revision of Drainage Device in Left Lower Extremity, Percutaneous Approach */
"0YWB33Z"  =  "1"  /* Revision of Infusion Device in Left Lower Extremity, Percutaneous Approach */
"0YWB37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Left Lower Extremity, Percutaneous Approach */
"0YWB3JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Lower Extremity, Percutaneous Approach */
"0YWB3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Left Lower Extremity, Percutaneous Approach */
"0YWB3YZ"  =  "1"  /* Revision of Other Device in Left Lower Extremity, Percutaneous Approach */
"0YWB40Z"  =  "1"  /* Revision of Drainage Device in Left Lower Extremity, Percutaneous Endoscopic Approach */
"0YWB43Z"  =  "1"  /* Revision of Infusion Device in Left Lower Extremity, Percutaneous Endoscopic Approach */
"0YWB47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Left Lower Extremity, Percutaneous Endoscopic Approach */
"0YWB4JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Lower Extremity, Percutaneous Endoscopic Approach */
"0YWB4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Left Lower Extremity, Percutaneous Endoscopic Approach */
"0YWB4YZ"  =  "1"  /* Revision of Other Device in Left Lower Extremity, Percutaneous Endoscopic Approach */
"2Y41X5Z"  =  "1"  /* Packing of Nasal Region using Packing Material */
"3C1ZX8Z"  =  "1"  /* Irrigation of Indwelling Device using Irrigating Substance, External Approach */
"3E03005"  =  "1"  /* Introduction of Other Antineoplastic into Peripheral Vein, Open Approach */
"3E0303Z"  =  "1"  /* Introduction of Anti-inflammatory into Peripheral Vein, Open Approach */
"3E030FZ"  =  "1"  /* Introduction of Intracirculatory Anesthetic into Peripheral Vein, Open Approach */
"3E030HZ"  =  "1"  /* Introduction of Radioactive Substance into Peripheral Vein, Open Approach */
"3E030KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Peripheral Vein, Open Approach */
"3E030RZ"  =  "1"  /* Introduction of Antiarrhythmic into Peripheral Vein, Open Approach */
"3E030WK"  =  "1"  /* Introduction of Immunostimulator into Peripheral Vein, Open */
"3E03305"  =  "1"  /* Introduction of Other Antineoplastic into Peripheral Vein, Percutaneous Approach */
"3E0333Z"  =  "1"  /* Introduction of Anti-inflammatory into Peripheral Vein, Percutaneous Approach */
"3E033FZ"  =  "1"  /* Introduction of Intracirculatory Anesthetic into Peripheral Vein, Percutaneous Approach */
"3E033HZ"  =  "1"  /* Introduction of Radioactive Substance into Peripheral Vein, Percutaneous Approach */
"3E033KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Peripheral Vein, Percutaneous Approach */
"3E033RZ"  =  "1"  /* Introduction of Antiarrhythmic into Peripheral Vein, Percutaneous Approach */
"3E033WK"  =  "1"  /* Introduction of Immunostimulator into Peripheral Vein, Percutaneous */
"3E04005"  =  "1"  /* Introduction of Other Antineoplastic into Central Vein, Open Approach */
"3E0403Z"  =  "1"  /* Introduction of Anti-inflammatory into Central Vein, Open Approach */
"3E040FZ"  =  "1"  /* Introduction of Intracirculatory Anesthetic into Central Vein, Open Approach */
"3E040HZ"  =  "1"  /* Introduction of Radioactive Substance into Central Vein, Open Approach */
"3E040KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Central Vein, Open Approach */
"3E040RZ"  =  "1"  /* Introduction of Antiarrhythmic into Central Vein, Open Approach */
"3E040WK"  =  "1"  /* Introduction of Immunostimulator into Central Vein, Open */
"3E04305"  =  "1"  /* Introduction of Other Antineoplastic into Central Vein, Percutaneous Approach */
"3E0433Z"  =  "1"  /* Introduction of Anti-inflammatory into Central Vein, Percutaneous Approach */
"3E043FZ"  =  "1"  /* Introduction of Intracirculatory Anesthetic into Central Vein, Percutaneous Approach */
"3E043HZ"  =  "1"  /* Introduction of Radioactive Substance into Central Vein, Percutaneous Approach */
"3E043KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Central Vein, Percutaneous Approach */
"3E043RZ"  =  "1"  /* Introduction of Antiarrhythmic into Central Vein, Percutaneous Approach */
"3E043WK"  =  "1"  /* Introduction of Immunostimulator into Central Vein, Percutaneous */
"3E05005"  =  "1"  /* Introduction of Other Antineoplastic into Peripheral Artery, Open Approach */
"3E0503Z"  =  "1"  /* Introduction of Anti-inflammatory into Peripheral Artery, Open Approach */
"3E050FZ"  =  "1"  /* Introduction of Intracirculatory Anesthetic into Peripheral Artery, Open Approach */
"3E050HZ"  =  "1"  /* Introduction of Radioactive Substance into Peripheral Artery, Open Approach */
"3E050KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Peripheral Artery, Open Approach */
"3E050RZ"  =  "1"  /* Introduction of Antiarrhythmic into Peripheral Artery, Open Approach */
"3E050TZ"  =  "1"  /* Introduction of Destructive Agent into Peripheral Artery, Open Approach */
"3E050WK"  =  "1"  /* Introduction of Immunostimulator into Peripheral Artery, Open */
"3E05305"  =  "1"  /* Introduction of Other Antineoplastic into Peripheral Artery, Percutaneous Approach */
"3E0533Z"  =  "1"  /* Introduction of Anti-inflammatory into Peripheral Artery, Percutaneous Approach */
"3E053FZ"  =  "1"  /* Introduction of Intracirculatory Anesthetic into Peripheral Artery, Percutaneous Approach */
"3E053HZ"  =  "1"  /* Introduction of Radioactive Substance into Peripheral Artery, Percutaneous Approach */
"3E053KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Peripheral Artery, Percutaneous Approach */
"3E053RZ"  =  "1"  /* Introduction of Antiarrhythmic into Peripheral Artery, Percutaneous Approach */
"3E053TZ"  =  "1"  /* Introduction of Destructive Agent into Peripheral Artery, Percutaneous Approach */
"3E053WK"  =  "1"  /* Introduction of Immunostimulator into Peripheral Artery, Percutaneous */
"3E06005"  =  "1"  /* Introduction of Other Antineoplastic into Central Artery, Open Approach */
"3E0603Z"  =  "1"  /* Introduction of Anti-inflammatory into Central Artery, Open Approach */
"3E060FZ"  =  "1"  /* Introduction of Intracirculatory Anesthetic into Central Artery, Open Approach */
"3E060HZ"  =  "1"  /* Introduction of Radioactive Substance into Central Artery, Open Approach */
"3E060RZ"  =  "1"  /* Introduction of Antiarrhythmic into Central Artery, Open Approach */
"3E060TZ"  =  "1"  /* Introduction of Destructive Agent into Central Artery, Open Approach */
"3E060WK"  =  "1"  /* Introduction of Immunostimulator into Central Artery, Open */
"3E06305"  =  "1"  /* Introduction of Other Antineoplastic into Central Artery, Percutaneous Approach */
"3E0633Z"  =  "1"  /* Introduction of Anti-inflammatory into Central Artery, Percutaneous Approach */
"3E063FZ"  =  "1"  /* Introduction of Intracirculatory Anesthetic into Central Artery, Percutaneous Approach */
"3E063HZ"  =  "1"  /* Introduction of Radioactive Substance into Central Artery, Percutaneous Approach */
"3E063RZ"  =  "1"  /* Introduction of Antiarrhythmic into Central Artery, Percutaneous Approach */
"3E063TZ"  =  "1"  /* Introduction of Destructive Agent into Central Artery, Percutaneous Approach */
"3E063WK"  =  "1"  /* Introduction of Immunostimulator into Central Artery, Percutaneous */
"3E09305"  =  "1"  /* Introduction of Other Antineoplastic into Nose, Percutaneous Approach */
"3E09329"  =  "1"  /* Introduction of Other Anti-infective into Nose, Percutaneous Approach */
"3E0933Z"  =  "1"  /* Introduction of Anti-inflammatory into Nose, Percutaneous Approach */
"3E0934Z"  =  "1"  /* Introduction of Serum, Toxoid and Vaccine into Nose, Percutaneous Approach */
"3E093BZ"  =  "1"  /* Introduction of Local Anesthetic into Nose, Percutaneous Approach */
"3E093GC"  =  "1"  /* Introduction of Other Therapeutic Substance into Nose, Percutaneous Approach */
"3E093HZ"  =  "1"  /* Introduction of Radioactive Substance into Nose, Percutaneous Approach */
"3E093KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Nose, Percutaneous Approach */
"3E093TZ"  =  "1"  /* Introduction of Destructive Agent into Nose, Percutaneous Approach */
"3E09705"  =  "1"  /* Introduction of Other Antineoplastic into Nose, Via Natural or Artificial Opening */
"3E09729"  =  "1"  /* Introduction of Other Anti-infective into Nose, Via Natural or Artificial Opening */
"3E0973Z"  =  "1"  /* Introduction of Anti-inflammatory into Nose, Via Natural or Artificial Opening */
"3E0974Z"  =  "1"  /* Introduction of Serum, Toxoid and Vaccine into Nose, Via Natural or Artificial Opening */
"3E097BZ"  =  "1"  /* Introduction of Local Anesthetic into Nose, Via Natural or Artificial Opening */
"3E097GC"  =  "1"  /* Introduction of Other Therapeutic Substance into Nose, Via Natural or Artificial Opening */
"3E097HZ"  =  "1"  /* Introduction of Radioactive Substance into Nose, Via Natural or Artificial Opening */
"3E097KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Nose, Via Natural or Artificial Opening */
"3E097TZ"  =  "1"  /* Introduction of Destructive Agent into Nose, Via Natural or Artificial Opening */
"3E09X05"  =  "1"  /* Introduction of Other Antineoplastic into Nose, External Approach */
"3E09X29"  =  "1"  /* Introduction of Other Anti-infective into Nose, External Approach */
"3E09X3Z"  =  "1"  /* Introduction of Anti-inflammatory into Nose, External Approach */
"3E09X4Z"  =  "1"  /* Introduction of Serum, Toxoid and Vaccine into Nose, External Approach */
"3E09XBZ"  =  "1"  /* Introduction of Local Anesthetic into Nose, External Approach */
"3E09XGC"  =  "1"  /* Introduction of Other Therapeutic Substance into Nose, External Approach */
"3E09XHZ"  =  "1"  /* Introduction of Radioactive Substance into Nose, External Approach */
"3E09XKZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Nose, External Approach */
"3E09XTZ"  =  "1"  /* Introduction of Destructive Agent into Nose, External Approach */
"3E0B305"  =  "1"  /* Introduction of Other Antineoplastic into Ear, Percutaneous Approach */
"3E0B705"  =  "1"  /* Introduction of Other Antineoplastic into Ear, Via Natural or Artificial Opening */
"3E0BX05"  =  "1"  /* Introduction of Other Antineoplastic into Ear, External Approach */
"3E0C305"  =  "1"  /* Introduction of Other Antineoplastic into Eye, Percutaneous Approach */
"3E0C705"  =  "1"  /* Introduction of Other Antineoplastic into Eye, Via Natural or Artificial Opening */
"3E0CX05"  =  "1"  /* Introduction of Other Antineoplastic into Eye, External Approach */
"3E0D305"  =  "1"  /* Introduction of Other Antineoplastic into Mouth and Pharynx, Percutaneous Approach */
"3E0D329"  =  "1"  /* Introduction of Other Anti-infective into Mouth and Pharynx, Percutaneous Approach */
"3E0D33Z"  =  "1"  /* Introduction of Anti-inflammatory into Mouth and Pharynx, Percutaneous Approach */
"3E0D34Z"  =  "1"  /* Introduction of Serum, Toxoid and Vaccine into Mouth and Pharynx, Percutaneous Approach */
"3E0D36Z"  =  "1"  /* Introduction of Nutritional Substance into Mouth and Pharynx, Percutaneous Approach */
"3E0D37Z"  =  "1"  /* Introduction of Electrolytic and Water Balance Substance into Mouth and Pharynx, Percutaneous Approach */
"3E0D3BZ"  =  "1"  /* Introduction of Local Anesthetic into Mouth and Pharynx, Percutaneous Approach */
"3E0D3GC"  =  "1"  /* Introduction of Other Therapeutic Substance into Mouth and Pharynx, Percutaneous Approach */
"3E0D3HZ"  =  "1"  /* Introduction of Radioactive Substance into Mouth and Pharynx, Percutaneous Approach */
"3E0D3KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Mouth and Pharynx, Percutaneous Approach */
"3E0D3RZ"  =  "1"  /* Introduction of Antiarrhythmic into Mouth and Pharynx, Percutaneous Approach */
"3E0D3TZ"  =  "1"  /* Introduction of Destructive Agent into Mouth and Pharynx, Percutaneous Approach */
"3E0D705"  =  "1"  /* Introduction of Other Antineoplastic into Mouth and Pharynx, Via Natural or Artificial Opening */
"3E0D729"  =  "1"  /* Introduction of Other Anti-infective into Mouth and Pharynx, Via Natural or Artificial Opening */
"3E0D73Z"  =  "1"  /* Introduction of Anti-inflammatory into Mouth and Pharynx, Via Natural or Artificial Opening */
"3E0D74Z"  =  "1"  /* Introduction of Serum, Toxoid and Vaccine into Mouth and Pharynx, Via Natural or Artificial Opening */
"3E0D76Z"  =  "1"  /* Introduction of Nutritional Substance into Mouth and Pharynx, Via Natural or Artificial Opening */
"3E0D77Z"  =  "1"  /* Introduction of Electrolytic and Water Balance Substance into Mouth and Pharynx, Via Natural or Artificial Opening */
"3E0D7BZ"  =  "1"  /* Introduction of Local Anesthetic into Mouth and Pharynx, Via Natural or Artificial Opening */
"3E0D7GC"  =  "1"  /* Introduction of Other Therapeutic Substance into Mouth and Pharynx, Via Natural or Artificial Opening */
"3E0D7HZ"  =  "1"  /* Introduction of Radioactive Substance into Mouth and Pharynx, Via Natural or Artificial Opening */
"3E0D7KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Mouth and Pharynx, Via Natural or Artificial Opening */
"3E0D7RZ"  =  "1"  /* Introduction of Antiarrhythmic into Mouth and Pharynx, Via Natural or Artificial Opening */
"3E0D7TZ"  =  "1"  /* Introduction of Destructive Agent into Mouth and Pharynx, Via Natural or Artificial Opening */
"3E0DX05"  =  "1"  /* Introduction of Other Antineoplastic into Mouth and Pharynx, External Approach */
"3E0DX29"  =  "1"  /* Introduction of Other Anti-infective into Mouth and Pharynx, External Approach */
"3E0DX3Z"  =  "1"  /* Introduction of Anti-inflammatory into Mouth and Pharynx, External Approach */
"3E0DX4Z"  =  "1"  /* Introduction of Serum, Toxoid and Vaccine into Mouth and Pharynx, External Approach */
"3E0DX6Z"  =  "1"  /* Introduction of Nutritional Substance into Mouth and Pharynx, External Approach */
"3E0DX7Z"  =  "1"  /* Introduction of Electrolytic and Water Balance Substance into Mouth and Pharynx, External Approach */
"3E0DXBZ"  =  "1"  /* Introduction of Local Anesthetic into Mouth and Pharynx, External Approach */
"3E0DXHZ"  =  "1"  /* Introduction of Radioactive Substance into Mouth and Pharynx, External Approach */
"3E0DXKZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Mouth and Pharynx, External Approach */
"3E0DXRZ"  =  "1"  /* Introduction of Antiarrhythmic into Mouth and Pharynx, External Approach */
"3E0DXTZ"  =  "1"  /* Introduction of Destructive Agent into Mouth and Pharynx, External Approach */
"3E0F3SF"  =  "1"  /* Introduction of Other Gas into Respiratory Tract, Percutaneous Approach */
"3E0F7SF"  =  "1"  /* Introduction of Other Gas into Respiratory Tract, Via Natural or Artificial Opening */
"3E0F8SF"  =  "1"  /* Introduction of Other Gas into Respiratory Tract, Via Natural or Artificial Opening Endoscopic */
"3E0G3BZ"  =  "1"  /* Introduction of Local Anesthetic into Upper GI, Percutaneous Approach */
"3E0G3HZ"  =  "1"  /* Introduction of Radioactive Substance into Upper GI, Percutaneous Approach */
"3E0G3KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Upper GI, Percutaneous Approach */
"3E0G3SF"  =  "1"  /* Introduction of Other Gas into Upper GI, Percutaneous Approach */
"3E0G3TZ"  =  "1"  /* Introduction of Destructive Agent into Upper GI, Percutaneous Approach */
"3E0G7BZ"  =  "1"  /* Introduction of Local Anesthetic into Upper GI, Via Natural or Artificial Opening */
"3E0G7HZ"  =  "1"  /* Introduction of Radioactive Substance into Upper GI, Via Natural or Artificial Opening */
"3E0G7KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Upper GI, Via Natural or Artificial Opening */
"3E0G7SF"  =  "1"  /* Introduction of Other Gas into Upper GI, Via Natural or Artificial Opening */
"3E0G7TZ"  =  "1"  /* Introduction of Destructive Agent into Upper GI, Via Natural or Artificial Opening */
"3E0G8BZ"  =  "1"  /* Introduction of Local Anesthetic into Upper GI, Via Natural or Artificial Opening Endoscopic */
"3E0G8HZ"  =  "1"  /* Introduction of Radioactive Substance into Upper GI, Via Natural or Artificial Opening Endoscopic */
"3E0G8KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Upper GI, Via Natural or Artificial Opening Endoscopic */
"3E0G8SF"  =  "1"  /* Introduction of Other Gas into Upper GI, Via Natural or Artificial Opening Endoscopic */
"3E0G8TZ"  =  "1"  /* Introduction of Destructive Agent into Upper GI, Via Natural or Artificial Opening Endoscopic */
"3E0H3BZ"  =  "1"  /* Introduction of Local Anesthetic into Lower GI, Percutaneous Approach */
"3E0H3GC"  =  "1"  /* Introduction of Other Therapeutic Substance into Lower GI, Percutaneous Approach */
"3E0H3HZ"  =  "1"  /* Introduction of Radioactive Substance into Lower GI, Percutaneous Approach */
"3E0H3KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Lower GI, Percutaneous Approach */
"3E0H3SF"  =  "1"  /* Introduction of Other Gas into Lower GI, Percutaneous Approach */
"3E0H3TZ"  =  "1"  /* Introduction of Destructive Agent into Lower GI, Percutaneous Approach */
"3E0H7BZ"  =  "1"  /* Introduction of Local Anesthetic into Lower GI, Via Natural or Artificial Opening */
"3E0H7HZ"  =  "1"  /* Introduction of Radioactive Substance into Lower GI, Via Natural or Artificial Opening */
"3E0H7KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Lower GI, Via Natural or Artificial Opening */
"3E0H7SF"  =  "1"  /* Introduction of Other Gas into Lower GI, Via Natural or Artificial Opening */
"3E0H7TZ"  =  "1"  /* Introduction of Destructive Agent into Lower GI, Via Natural or Artificial Opening */
"3E0H8BZ"  =  "1"  /* Introduction of Local Anesthetic into Lower GI, Via Natural or Artificial Opening Endoscopic */
"3E0H8HZ"  =  "1"  /* Introduction of Radioactive Substance into Lower GI, Via Natural or Artificial Opening Endoscopic */
"3E0H8KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Lower GI, Via Natural or Artificial Opening Endoscopic */
"3E0H8SF"  =  "1"  /* Introduction of Other Gas into Lower GI, Via Natural or Artificial Opening Endoscopic */
"3E0H8TZ"  =  "1"  /* Introduction of Destructive Agent into Lower GI, Via Natural or Artificial Opening Endoscopic */
"3E0J3SF"  =  "1"  /* Introduction of Other Gas into Biliary and Pancreatic Tract, Percutaneous Approach */
"3E0J7SF"  =  "1"  /* Introduction of Other Gas into Biliary and Pancreatic Tract, Via Natural or Artificial Opening */
"3E0J8SF"  =  "1"  /* Introduction of Other Gas into Biliary and Pancreatic Tract, Via Natural or Artificial Opening Endoscopic */
"3E0K3BZ"  =  "1"  /* Introduction of Local Anesthetic into Genitourinary Tract, Percutaneous Approach */
"3E0K3HZ"  =  "1"  /* Introduction of Radioactive Substance into Genitourinary Tract, Percutaneous Approach */
"3E0K3KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Genitourinary Tract, Percutaneous Approach */
"3E0K3SF"  =  "1"  /* Introduction of Other Gas into Genitourinary Tract, Percutaneous Approach */
"3E0K3TZ"  =  "1"  /* Introduction of Destructive Agent into Genitourinary Tract, Percutaneous Approach */
"3E0K7BZ"  =  "1"  /* Introduction of Local Anesthetic into Genitourinary Tract, Via Natural or Artificial Opening */
"3E0K7GC"  =  "1"  /* Introduction of Other Therapeutic Substance into Genitourinary Tract, Via Natural or Artificial Opening */
"3E0K7HZ"  =  "1"  /* Introduction of Radioactive Substance into Genitourinary Tract, Via Natural or Artificial Opening */
"3E0K7KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Genitourinary Tract, Via Natural or Artificial Opening */
"3E0K7SF"  =  "1"  /* Introduction of Other Gas into Genitourinary Tract, Via Natural or Artificial Opening */
"3E0K7TZ"  =  "1"  /* Introduction of Destructive Agent into Genitourinary Tract, Via Natural or Artificial Opening */
"3E0K8BZ"  =  "1"  /* Introduction of Local Anesthetic into Genitourinary Tract, Via Natural or Artificial Opening Endoscopic */
"3E0K8GC"  =  "1"  /* Introduction of Other Therapeutic Substance into Genitourinary Tract, Via Natural or Artificial Opening Endoscopic */
"3E0K8HZ"  =  "1"  /* Introduction of Radioactive Substance into Genitourinary Tract, Via Natural or Artificial Opening Endoscopic */
"3E0K8KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Genitourinary Tract, Via Natural or Artificial Opening Endoscopic */
"3E0K8SF"  =  "1"  /* Introduction of Other Gas into Genitourinary Tract, Via Natural or Artificial Opening Endoscopic */
"3E0K8TZ"  =  "1"  /* Introduction of Destructive Agent into Genitourinary Tract, Via Natural or Artificial Opening Endoscopic */
"3E0N73Z"  =  "1"  /* Introduction of Anti-inflammatory into Male Reproductive, Via Natural or Artificial Opening */
"3E0N76Z"  =  "1"  /* Introduction of Nutritional Substance into Male Reproductive, Via Natural or Artificial Opening */
"3E0N77Z"  =  "1"  /* Introduction of Electrolytic and Water Balance Substance into Male Reproductive, Via Natural or Artificial Opening */
"3E0N7BZ"  =  "1"  /* Introduction of Local Anesthetic into Male Reproductive, Via Natural or Artificial Opening */
"3E0N7GC"  =  "1"  /* Introduction of Other Therapeutic Substance into Male Reproductive, Via Natural or Artificial Opening */
"3E0N7HZ"  =  "1"  /* Introduction of Radioactive Substance into Male Reproductive, Via Natural or Artificial Opening */
"3E0N7KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Male Reproductive, Via Natural or Artificial Opening */
"3E0N7SF"  =  "1"  /* Introduction of Other Gas into Male Reproductive, Via Natural or Artificial Opening */
"3E0N7TZ"  =  "1"  /* Introduction of Destructive Agent into Male Reproductive, Via Natural or Artificial Opening */
"3E0N83Z"  =  "1"  /* Introduction of Anti-inflammatory into Male Reproductive, Via Natural or Artificial Opening Endoscopic */
"3E0N86Z"  =  "1"  /* Introduction of Nutritional Substance into Male Reproductive, Via Natural or Artificial Opening Endoscopic */
"3E0N87Z"  =  "1"  /* Introduction of Electrolytic and Water Balance Substance into Male Reproductive, Via Natural or Artificial Opening Endoscopic */
"3E0N8BZ"  =  "1"  /* Introduction of Local Anesthetic into Male Reproductive, Via Natural or Artificial Opening Endoscopic */
"3E0N8GC"  =  "1"  /* Introduction of Other Therapeutic Substance into Male Reproductive, Via Natural or Artificial Opening Endoscopic */
"3E0N8HZ"  =  "1"  /* Introduction of Radioactive Substance into Male Reproductive, Via Natural or Artificial Opening Endoscopic */
"3E0N8KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Male Reproductive, Via Natural or Artificial Opening Endoscopic */
"3E0N8SF"  =  "1"  /* Introduction of Other Gas into Male Reproductive, Via Natural or Artificial Opening Endoscopic */
"3E0N8TZ"  =  "1"  /* Introduction of Destructive Agent into Male Reproductive, Via Natural or Artificial Opening Endoscopic */
"3E0P3GC"  =  "1"  /* Introduction of Other Therapeutic Substance into Female Reproductive, Percutaneous Approach */
"3E0P3SF"  =  "1"  /* Introduction of Other Gas into Female Reproductive, Percutaneous Approach */
"3E0P7SF"  =  "1"  /* Introduction of Other Gas into Female Reproductive, Via Natural or Artificial Opening */
"3E0P8SF"  =  "1"  /* Introduction of Other Gas into Female Reproductive, Via Natural or Artificial Opening Endoscopic */
"3E0R305"  =  "1"  /* Introduction of Other Antineoplastic into Spinal Canal, Percutaneous Approach */
"3E0S305"  =  "1"  /* Introduction of Other Antineoplastic into Epidural Space, Percutaneous Approach */
"3E0U305"  =  "1"  /* Introduction of Other Antineoplastic into Joints, Percutaneous Approach */
"3E0W33Z"  =  "1"  /* Introduction of Anti-inflammatory into Lymphatics, Percutaneous Approach */
"3E0W36Z"  =  "1"  /* Introduction of Nutritional Substance into Lymphatics, Percutaneous Approach */
"3E0W37Z"  =  "1"  /* Introduction of Electrolytic and Water Balance Substance into Lymphatics, Percutaneous Approach */
"3E0W3BZ"  =  "1"  /* Introduction of Local Anesthetic into Lymphatics, Percutaneous Approach */
"3E0W3GC"  =  "1"  /* Introduction of Other Therapeutic Substance into Lymphatics, Percutaneous Approach */
"3E0W3HZ"  =  "1"  /* Introduction of Radioactive Substance into Lymphatics, Percutaneous Approach */
"3E0W3KZ"  =  "1"  /* Introduction of Other Diagnostic Substance into Lymphatics, Percutaneous Approach */
"3E0W3TZ"  =  "1"  /* Introduction of Destructive Agent into Lymphatics, Percutaneous Approach */
"3E1J38X"  =  "1"  /* Irrigation of Biliary and Pancreatic Tract using Irrigating Substance, Percutaneous Approach, Diagnostic */
"3E1J38Z"  =  "1"  /* Irrigation of Biliary and Pancreatic Tract using Irrigating Substance, Percutaneous Approach */
"3E1Q38X"  =  "1"  /* Irrigation of Cranial Cavity and Brain using Irrigating Substance, Percutaneous Approach, Diagnostic */
"3E1Q38Z"  =  "1"  /* Irrigation of Cranial Cavity and Brain using Irrigating Substance, Percutaneous Approach */
"3E1R38X"  =  "1"  /* Irrigation of Spinal Canal using Irrigating Substance, Percutaneous Approach, Diagnostic */
"3E1R38Z"  =  "1"  /* Irrigation of Spinal Canal using Irrigating Substance, Percutaneous Approach */
"3E1S38X"  =  "1"  /* Irrigation of Epidural Space using Irrigating Substance, Percutaneous Approach, Diagnostic */
"3E1S38Z"  =  "1"  /* Irrigation of Epidural Space using Irrigating Substance, Percutaneous Approach */
"BF100ZZ"  =  "1"  /* Fluoroscopy of Bile Ducts using High Osmolar Contrast */
"BF101ZZ"  =  "1"  /* Fluoroscopy of Bile Ducts using Low Osmolar Contrast */
"BF10YZZ"  =  "1"  /* Fluoroscopy of Bile Ducts using Other Contrast */
"BF110ZZ"  =  "1"  /* Fluoroscopy of Biliary and Pancreatic Ducts using High Osmolar Contrast */
"BF111ZZ"  =  "1"  /* Fluoroscopy of Biliary and Pancreatic Ducts using Low Osmolar Contrast */
"BF11YZZ"  =  "1"  /* Fluoroscopy of Biliary and Pancreatic Ducts using Other Contrast */
"BF180ZZ"  =  "1"  /* Fluoroscopy of Pancreatic Ducts using High Osmolar Contrast */
"BF181ZZ"  =  "1"  /* Fluoroscopy of Pancreatic Ducts using Low Osmolar Contrast */
"BF18YZZ"  =  "1"  /* Fluoroscopy of Pancreatic Ducts using Other Contrast */
OTHER  = "0" ;


 /* COMBINED */
 /* VALUE $HEMIP - no ICD-10-CM PROCEDURE CODES from UCD */

 /* COMBINED */
 /* VALUE $HEMIPB - no ICD-10-CM PROCEDURE CODES from UCD */


 /* COAGULATION DISORDERS */
/* VALUE $COAGDID - Removed for 5.0? */
 /* ICD-10-CM DIAGNOSIS CODES: */

VALUE $COAGDID
"D473"  =  "1"  /* Essential (hemorrhagic) thrombocythemia */
"D65"  =  "1"  /* Disseminated intravascular coagulation [defibrination syndrome] */
"D66"  =  "1"  /* Hereditary factor VIII deficiency */
"D67"  =  "1"  /* Hereditary factor IX deficiency */
"D680"  =  "1"  /* Von Willebrand's disease */
"D681"  =  "1"  /* Hereditary factor XI deficiency */
"D682"  =  "1"  /* Hereditary deficiency of other clotting factors */
"D68311"  =  "1"  /* Acquired hemophilia */
"D68312"  =  "1"  /* Antiphospholipid antibody with hemorrhagic disorder */
"D68318"  =  "1"  /* Other hemorrhagic disorder due to intrinsic circulating anticoagulants, antibodies, or inhibitors */
"D6832"  =  "1"  /* Hemorrhagic disorder due to extrinsic circulating anticoagulants */
"D684"  =  "1"  /* Acquired coagulation factor deficiency */
"D688"  =  "1"  /* Other specified coagulation defects */
"D689"  =  "1"  /* Coagulation defect, unspecified */
"D691"  =  "1"  /* Qualitative platelet defects */
"D693"  =  "1"  /* Immune thrombocytopenic purpura */
"D6941"  =  "1"  /* Evans syndrome */
"D6942"  =  "1"  /* Congenital and hereditary thrombocytopenia purpura */
"D6949"  =  "1"  /* Other primary thrombocytopenia */
"D6951"  =  "1"  /* Posttransfusion purpura */
"D696"  =  "1"  /* Thrombocytopenia, unspecified */
"D698"  =  "1"  /* Other specified hemorrhagic conditions */
"D699"  =  "1"  /* Hemorrhagic condition, unspecified */
OTHER  = "0" ;

/* STRATIFIED BY: COAGULOPATHIES */
/* ICD-10-CM DIAGNOSIS CODE: */

VALUE $ACGOFD
"D473"  =  "1"  /* Essential (hemorrhagic) thrombocythemia */
"D65"  =  "1"  /* Disseminated intravascular coagulation [defibrination syndrome] */
"D66"  =  "1"  /* Hereditary factor VIII deficiency */
"D67"  =  "1"  /* Hereditary factor IX deficiency */
"D680"  =  "1"  /* Von Willebrand's disease */
"D681"  =  "1"  /* Hereditary factor XI deficiency */
"D682"  =  "1"  /* Hereditary deficiency of other clotting factors */
"D68311"  =  "1"  /* Acquired hemophilia */
"D68312"  =  "1"  /* Antiphospholipid antibody with hemorrhagic disorder */
"D68318"  =  "1"  /* Other hemorrhagic disorder due to intrinsic circulating anticoagulants, antibodies, or inhibitors */
"D6832"  =  "1"  /* Hemorrhagic disorder due to extrinsic circulating anticoagulants */
"D684"  =  "1"  /* Acquired coagulation factor deficiency */
"D688"  =  "1"  /* Other specified coagulation defects */
"D689"  =  "1"  /* Coagulation defect, unspecified */
"D691"  =  "1"  /* Qualitative platelet defects */
"D693"  =  "1"  /* Immune thrombocytopenic purpura */
"D6941"  =  "1"  /* Evans syndrome */
"D6942"  =  "1"  /* Congenital and hereditary thrombocytopenia purpura */
"D6949"  =  "1"  /* Other primary thrombocytopenia */
"D6951"  =  "1"  /* Posttransfusion purpura */
"D696"  =  "1"  /* Thrombocytopenia, unspecified */
"D698"  =  "1"  /* Other specified hemorrhagic conditions */
"D699"  =  "1"  /* Hemorrhagic condition, unspecified */
OTHER   = "0" ;

/*STRATIFIED: BY PR CODE FOR ECMO */
/* ICD-10-CM CODE PROCEDURE CODE: */

VALUE $ECMOP
"5A15223"  =  "1"  /* Extracorporeal Membrane Oxygenation, Continuous */
OTHER  = "0" ;


 /* ---------------------------------- */
 /* POST-OPERATIVE RESPIRATORY FAILURE */
 /*   PROVIDER INDICATOR / PDI # 9     */
 /* ---------------------------------- */


 /* ACUTE RESPIRATORY FAILURE DIAGNOSIS CODES (BEFORE oCT 1, 2011) */
 /* VALUE $ACURFID per UCD there's no ICD-10 codes due to: This concept is no longer relevant since ICD-9-CM code 518.5x "Pulmonary insufficieny following trauma and/or surgery" was created October 1, 2011 */

 /* ICD-10-CM ACUTE RESPIRATORY FAILURE DIAGNOSIS CODES (ON OR AFTER oCT 1, 2011)*/

VALUE $ACURF2D
"J95821"  =  "1"  /* Acute postprocedural respiratory failure */
"J95822"  =  "1"  /* Acute and chronic postprocedural respiratory failure */
OTHER   = "0" ;


 /* ICD-10-CM PROCEDURE CODES: */

VALUE $BLANK
OTHER  = "0" ;

VALUE $PR9604P
"0BH13EZ"  =  "1"  /* Insertion of Endotracheal Airway into Trachea, Percutaneous Approach */
"0BH17EZ"  =  "1"  /* Insertion of Endotracheal Airway into Trachea, Via Natural or Artificial Opening */
"0BH18EZ"  =  "1"  /* Insertion of Endotracheal Airway into Trachea, Via Natural or Artificial Opening Endoscopic */
OTHER  = "0" ;

/* VALUE $PR9670P per UCD None of the ICD-10-CM codes apply to this concept */

VALUE $PR9671P
"5A1945Z"  =  "1"  /* Respiratory Ventilation, 24-96 Consecutive Hours */
OTHER  = "0" ;

VALUE $PR9672P
"5A1955Z"  =  "1"  /* Respiratory Ventilation, Greater than 96 Consecutive Hours */
OTHER  = "0" ;

 /* EXCLUDE: TRACHEOSTOMY */;
 /* ICD-10-CM PROCEDURE CODE: */;

VALUE $TRACHIP
"0B110F4"  =  "1"  /* Bypass Trachea to Cutaneous with Tracheostomy Device, Open Approach */
"0B110Z4"  =  "1"  /* Bypass Trachea to Cutaneous, Open Approach */
"0B113F4"  =  "1"  /* Bypass Trachea to Cutaneous with Tracheostomy Device, Percutaneous Approach */
"0B113Z4"  =  "1"  /* Bypass Trachea to Cutaneous, Percutaneous Approach */
"0B114F4"  =  "1"  /* Bypass Trachea to Cutaneous with Tracheostomy Device, Percutaneous Endoscopic Approach */
"0B114Z4"  =  "1"  /* Bypass Trachea to Cutaneous, Percutaneous Endoscopic Approach */
"0BW10FZ"  =  "1"  /* Revision of Tracheostomy Device in Trachea, Open Approach */
"0BW13FZ"  =  "1"  /* Revision of Tracheostomy Device in Trachea, Percutaneous Approach */
"0BW14FZ"  =  "1"  /* Revision of Tracheostomy Device in Trachea, Percutaneous Endoscopic Approach */
"0WB6XZ2"  =  "1"  /* Excision of Neck, Stoma, External Approach */
OTHER  = "0" ;

 /* EXCLUDE: NEUROMUSCULAR DISORDERS */;
 /* ICD-10-CM DIAGNOSIS CODES: */

VALUE $NEUROMD
"G610"  =  "1"  /* Guillain-Barre syndrome */
"G7000"  =  "1"  /* Myasthenia gravis without (acute) exacerbation */
"G7001"  =  "1"  /* Myasthenia gravis with (acute) exacerbation */
"G701"  =  "1"  /* Toxic myoneural disorders */
"G702"  =  "1"  /* Congenital and developmental myasthenia */
"G7080"  =  "1"  /* Lambert-Eaton syndrome, unspecified */
"G7081"  =  "1"  /* Lambert-Eaton syndrome in disease classified elsewhere */
"G7089"  =  "1"  /* Other specified myoneural disorders */
"G709"  =  "1"  /* Myoneural disorder, unspecified */
"G710"  =  "1"  /* Muscular dystrophy */
"G7111"  =  "1"  /* Myotonic muscular dystrophy */
"G7112"  =  "1"  /* Myotonia congenita */
"G7113"  =  "1"  /* Myotonic chondrodystrophy */
"G7119"  =  "1"  /* Other specified myotonic disorders */
"G712"  =  "1"  /* Congenital myopathies */
"G713"  =  "1"  /* Mitochondrial myopathy, not elsewhere classified */
"G718"  =  "1"  /* Other primary disorders of muscles */
"G719"  =  "1"  /* Primary disorder of muscle, unspecified */
"G720"  =  "1"  /* Drug-induced myopathy */
"G721"  =  "1"  /* Alcoholic myopathy */
"G722"  =  "1"  /* Myopathy due to other toxic agents */
"G723"  =  "1"  /* Periodic paralysis */
"G7241"  =  "1"  /* Inclusion body myositis [IBM] */
"G7249"  =  "1"  /* Other inflammatory and immune myopathies, not elsewhere classified */
"G7281"  =  "1"  /* Critical illness myopathy */
"G7289"  =  "1"  /* Other specified myopathies */
"G729"  =  "1"  /* Myopathy, unspecified */
"G731"  =  "1"  /* Lambert-Eaton syndrome in neoplastic disease */
"G733"  =  "1"  /* Myasthenic syndromes in other diseases classified elsewhere */
"G737"  =  "1"  /* Myopathy in diseases classified elsewhere */
"M0540"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of unspecified site */
"M05411"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of right shoulder */
"M05412"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of left shoulder */
"M05419"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of unspecified shoulder */
"M05421"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of right elbow */
"M05422"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of left elbow */
"M05429"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of unspecified elbow */
"M05431"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of right wrist */
"M05432"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of left wrist */
"M05439"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of unspecified wrist */
"M05441"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of right hand */
"M05442"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of left hand */
"M05449"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of unspecified hand */
"M05451"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of right hip */
"M05452"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of left hip */
"M05459"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of unspecified hip */
"M05461"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of right knee */
"M05462"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of left knee */
"M05469"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of unspecified knee */
"M05471"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of right ankle and foot */
"M05472"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of left ankle and foot */
"M05479"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of unspecified ankle and foot */
"M0549"  =  "1"  /* Rheumatoid myopathy with rheumatoid arthritis of multiple sites */
"M3302"  =  "1"  /* Juvenile dermatopolymyositis with myopathy */
"M3312"  =  "1"  /* Other dermatopolymyositis with myopathy */
"M3322"  =  "1"  /* Polymyositis with myopathy */
"M3392"  =  "1"  /* Dermatopolymyositis, unspecified with myopathy */
"M3482"  =  "1"  /* Systemic sclerosis with myopathy */
"M3503"  =  "1"  /* Sicca syndrome with myopathy */
OTHER   = "0" ;


/* merged $CRANI2P $CRANI1P $NECKIP per UCD */
Value $NUCRANP
"008P0ZZ"  =  "1"  /* Division of Glossopharyngeal Nerve, Open Approach */
"008P3ZZ"  =  "1"  /* Division of Glossopharyngeal Nerve, Percutaneous Approach */
"008P4ZZ"  =  "1"  /* Division of Glossopharyngeal Nerve, Percutaneous Endoscopic Approach */
"008Q0ZZ"  =  "1"  /* Division of Vagus Nerve, Open Approach */
"008Q3ZZ"  =  "1"  /* Division of Vagus Nerve, Percutaneous Approach */
"008Q4ZZ"  =  "1"  /* Division of Vagus Nerve, Percutaneous Endoscopic Approach */
"07T10ZZ"  =  "1"  /* Resection of Right Neck Lymphatic, Open Approach */
"07T14ZZ"  =  "1"  /* Resection of Right Neck Lymphatic, Percutaneous Endoscopic Approach */
"07T20ZZ"  =  "1"  /* Resection of Left Neck Lymphatic, Open Approach */
"07T24ZZ"  =  "1"  /* Resection of Left Neck Lymphatic, Percutaneous Endoscopic Approach */
"095L0ZZ"  =  "1"  /* Destruction of Nasal Turbinate, Open Approach */
"095L3ZZ"  =  "1"  /* Destruction of Nasal Turbinate, Percutaneous Approach */
"095L4ZZ"  =  "1"  /* Destruction of Nasal Turbinate, Percutaneous Endoscopic Approach */
"095L7ZZ"  =  "1"  /* Destruction of Nasal Turbinate, Via Natural or Artificial Opening */
"095L8ZZ"  =  "1"  /* Destruction of Nasal Turbinate, Via Natural or Artificial Opening Endoscopic */
"095N0ZZ"  =  "1"  /* Destruction of Nasopharynx, Open Approach */
"095N3ZZ"  =  "1"  /* Destruction of Nasopharynx, Percutaneous Approach */
"095N4ZZ"  =  "1"  /* Destruction of Nasopharynx, Percutaneous Endoscopic Approach */
"095N7ZZ"  =  "1"  /* Destruction of Nasopharynx, Via Natural or Artificial Opening */
"095N8ZZ"  =  "1"  /* Destruction of Nasopharynx, Via Natural or Artificial Opening Endoscopic */
"095P0ZZ"  =  "1"  /* Destruction of Accessory Sinus, Open Approach */
"095P3ZZ"  =  "1"  /* Destruction of Accessory Sinus, Percutaneous Approach */
"095P4ZZ"  =  "1"  /* Destruction of Accessory Sinus, Percutaneous Endoscopic Approach */
"095Q0ZZ"  =  "1"  /* Destruction of Right Maxillary Sinus, Open Approach */
"095Q3ZZ"  =  "1"  /* Destruction of Right Maxillary Sinus, Percutaneous Approach */
"095Q4ZZ"  =  "1"  /* Destruction of Right Maxillary Sinus, Percutaneous Endoscopic Approach */
"095R0ZZ"  =  "1"  /* Destruction of Left Maxillary Sinus, Open Approach */
"095R3ZZ"  =  "1"  /* Destruction of Left Maxillary Sinus, Percutaneous Approach */
"095R4ZZ"  =  "1"  /* Destruction of Left Maxillary Sinus, Percutaneous Endoscopic Approach */
"095S0ZZ"  =  "1"  /* Destruction of Right Frontal Sinus, Open Approach */
"095S3ZZ"  =  "1"  /* Destruction of Right Frontal Sinus, Percutaneous Approach */
"095S4ZZ"  =  "1"  /* Destruction of Right Frontal Sinus, Percutaneous Endoscopic Approach */
"095T0ZZ"  =  "1"  /* Destruction of Left Frontal Sinus, Open Approach */
"095T3ZZ"  =  "1"  /* Destruction of Left Frontal Sinus, Percutaneous Approach */
"095T4ZZ"  =  "1"  /* Destruction of Left Frontal Sinus, Percutaneous Endoscopic Approach */
"095U0ZZ"  =  "1"  /* Destruction of Right Ethmoid Sinus, Open Approach */
"095U3ZZ"  =  "1"  /* Destruction of Right Ethmoid Sinus, Percutaneous Approach */
"095U4ZZ"  =  "1"  /* Destruction of Right Ethmoid Sinus, Percutaneous Endoscopic Approach */
"095V0ZZ"  =  "1"  /* Destruction of Left Ethmoid Sinus, Open Approach */
"095V3ZZ"  =  "1"  /* Destruction of Left Ethmoid Sinus, Percutaneous Approach */
"095V4ZZ"  =  "1"  /* Destruction of Left Ethmoid Sinus, Percutaneous Endoscopic Approach */
"095W0ZZ"  =  "1"  /* Destruction of Right Sphenoid Sinus, Open Approach */
"095W3ZZ"  =  "1"  /* Destruction of Right Sphenoid Sinus, Percutaneous Approach */
"095W4ZZ"  =  "1"  /* Destruction of Right Sphenoid Sinus, Percutaneous Endoscopic Approach */
"095X0ZZ"  =  "1"  /* Destruction of Left Sphenoid Sinus, Open Approach */
"095X3ZZ"  =  "1"  /* Destruction of Left Sphenoid Sinus, Percutaneous Approach */
"095X4ZZ"  =  "1"  /* Destruction of Left Sphenoid Sinus, Percutaneous Endoscopic Approach */
"098L0ZZ"  =  "1"  /* Division of Nasal Turbinate, Open Approach */
"098L3ZZ"  =  "1"  /* Division of Nasal Turbinate, Percutaneous Approach */
"098L4ZZ"  =  "1"  /* Division of Nasal Turbinate, Percutaneous Endoscopic Approach */
"098L7ZZ"  =  "1"  /* Division of Nasal Turbinate, Via Natural or Artificial Opening */
"098L8ZZ"  =  "1"  /* Division of Nasal Turbinate, Via Natural or Artificial Opening Endoscopic */
"099N00Z"  =  "1"  /* Drainage of Nasopharynx with Drainage Device, Open Approach */
"099N0ZZ"  =  "1"  /* Drainage of Nasopharynx, Open Approach */
"099N30Z"  =  "1"  /* Drainage of Nasopharynx with Drainage Device, Percutaneous Approach */
"099N3ZZ"  =  "1"  /* Drainage of Nasopharynx, Percutaneous Approach */
"099N40Z"  =  "1"  /* Drainage of Nasopharynx with Drainage Device, Percutaneous Endoscopic Approach */
"099N4ZZ"  =  "1"  /* Drainage of Nasopharynx, Percutaneous Endoscopic Approach */
"099N70Z"  =  "1"  /* Drainage of Nasopharynx with Drainage Device, Via Natural or Artificial Opening */
"099N7ZZ"  =  "1"  /* Drainage of Nasopharynx, Via Natural or Artificial Opening */
"099N80Z"  =  "1"  /* Drainage of Nasopharynx with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"099N8ZZ"  =  "1"  /* Drainage of Nasopharynx, Via Natural or Artificial Opening Endoscopic */
"099P00Z"  =  "1"  /* Drainage of Accessory Sinus with Drainage Device, Open Approach */
"099P0ZZ"  =  "1"  /* Drainage of Accessory Sinus, Open Approach */
"099Q00Z"  =  "1"  /* Drainage of Right Maxillary Sinus with Drainage Device, Open Approach */
"099Q0ZZ"  =  "1"  /* Drainage of Right Maxillary Sinus, Open Approach */
"099R00Z"  =  "1"  /* Drainage of Left Maxillary Sinus with Drainage Device, Open Approach */
"099R0ZZ"  =  "1"  /* Drainage of Left Maxillary Sinus, Open Approach */
"099S00Z"  =  "1"  /* Drainage of Right Frontal Sinus with Drainage Device, Open Approach */
"099S0ZZ"  =  "1"  /* Drainage of Right Frontal Sinus, Open Approach */
"099T00Z"  =  "1"  /* Drainage of Left Frontal Sinus with Drainage Device, Open Approach */
"099T0ZZ"  =  "1"  /* Drainage of Left Frontal Sinus, Open Approach */
"099U00Z"  =  "1"  /* Drainage of Right Ethmoid Sinus with Drainage Device, Open Approach */
"099U0ZZ"  =  "1"  /* Drainage of Right Ethmoid Sinus, Open Approach */
"099V00Z"  =  "1"  /* Drainage of Left Ethmoid Sinus with Drainage Device, Open Approach */
"099V0ZZ"  =  "1"  /* Drainage of Left Ethmoid Sinus, Open Approach */
"099W00Z"  =  "1"  /* Drainage of Right Sphenoid Sinus with Drainage Device, Open Approach */
"099W0ZZ"  =  "1"  /* Drainage of Right Sphenoid Sinus, Open Approach */
"099X00Z"  =  "1"  /* Drainage of Left Sphenoid Sinus with Drainage Device, Open Approach */
"099X0ZZ"  =  "1"  /* Drainage of Left Sphenoid Sinus, Open Approach */
"09BL0ZZ"  =  "1"  /* Excision of Nasal Turbinate, Open Approach */
"09BL3ZZ"  =  "1"  /* Excision of Nasal Turbinate, Percutaneous Approach */
"09BL4ZZ"  =  "1"  /* Excision of Nasal Turbinate, Percutaneous Endoscopic Approach */
"09BL7ZZ"  =  "1"  /* Excision of Nasal Turbinate, Via Natural or Artificial Opening */
"09BL8ZZ"  =  "1"  /* Excision of Nasal Turbinate, Via Natural or Artificial Opening Endoscopic */
"09BN0ZZ"  =  "1"  /* Excision of Nasopharynx, Open Approach */
"09BN3ZZ"  =  "1"  /* Excision of Nasopharynx, Percutaneous Approach */
"09BN4ZZ"  =  "1"  /* Excision of Nasopharynx, Percutaneous Endoscopic Approach */
"09BN7ZZ"  =  "1"  /* Excision of Nasopharynx, Via Natural or Artificial Opening */
"09BN8ZZ"  =  "1"  /* Excision of Nasopharynx, Via Natural or Artificial Opening Endoscopic */
"09BP0ZZ"  =  "1"  /* Excision of Accessory Sinus, Open Approach */
"09BP3ZZ"  =  "1"  /* Excision of Accessory Sinus, Percutaneous Approach */
"09BP4ZZ"  =  "1"  /* Excision of Accessory Sinus, Percutaneous Endoscopic Approach */
"09BQ0ZZ"  =  "1"  /* Excision of Right Maxillary Sinus, Open Approach */
"09BQ3ZZ"  =  "1"  /* Excision of Right Maxillary Sinus, Percutaneous Approach */
"09BQ4ZZ"  =  "1"  /* Excision of Right Maxillary Sinus, Percutaneous Endoscopic Approach */
"09BR0ZZ"  =  "1"  /* Excision of Left Maxillary Sinus, Open Approach */
"09BR3ZZ"  =  "1"  /* Excision of Left Maxillary Sinus, Percutaneous Approach */
"09BR4ZZ"  =  "1"  /* Excision of Left Maxillary Sinus, Percutaneous Endoscopic Approach */
"09BS0ZZ"  =  "1"  /* Excision of Right Frontal Sinus, Open Approach */
"09BS3ZZ"  =  "1"  /* Excision of Right Frontal Sinus, Percutaneous Approach */
"09BS4ZZ"  =  "1"  /* Excision of Right Frontal Sinus, Percutaneous Endoscopic Approach */
"09BT0ZZ"  =  "1"  /* Excision of Left Frontal Sinus, Open Approach */
"09BT3ZZ"  =  "1"  /* Excision of Left Frontal Sinus, Percutaneous Approach */
"09BT4ZZ"  =  "1"  /* Excision of Left Frontal Sinus, Percutaneous Endoscopic Approach */
"09BU0ZZ"  =  "1"  /* Excision of Right Ethmoid Sinus, Open Approach */
"09BU3ZZ"  =  "1"  /* Excision of Right Ethmoid Sinus, Percutaneous Approach */
"09BU4ZZ"  =  "1"  /* Excision of Right Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09BV0ZZ"  =  "1"  /* Excision of Left Ethmoid Sinus, Open Approach */
"09BV3ZZ"  =  "1"  /* Excision of Left Ethmoid Sinus, Percutaneous Approach */
"09BV4ZZ"  =  "1"  /* Excision of Left Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09BW0ZZ"  =  "1"  /* Excision of Right Sphenoid Sinus, Open Approach */
"09BW3ZZ"  =  "1"  /* Excision of Right Sphenoid Sinus, Percutaneous Approach */
"09BW4ZZ"  =  "1"  /* Excision of Right Sphenoid Sinus, Percutaneous Endoscopic Approach */
"09BX0ZZ"  =  "1"  /* Excision of Left Sphenoid Sinus, Open Approach */
"09BX3ZZ"  =  "1"  /* Excision of Left Sphenoid Sinus, Percutaneous Approach */
"09BX4ZZ"  =  "1"  /* Excision of Left Sphenoid Sinus, Percutaneous Endoscopic Approach */
"09CN0ZZ"  =  "1"  /* Extirpation of Matter from Nasopharynx, Open Approach */
"09CN3ZZ"  =  "1"  /* Extirpation of Matter from Nasopharynx, Percutaneous Approach */
"09CN4ZZ"  =  "1"  /* Extirpation of Matter from Nasopharynx, Percutaneous Endoscopic Approach */
"09CN7ZZ"  =  "1"  /* Extirpation of Matter from Nasopharynx, Via Natural or Artificial Opening */
"09CN8ZZ"  =  "1"  /* Extirpation of Matter from Nasopharynx, Via Natural or Artificial Opening Endoscopic */
"09CP0ZZ"  =  "1"  /* Extirpation of Matter from Accessory Sinus, Open Approach */
"09CP3ZZ"  =  "1"  /* Extirpation of Matter from Accessory Sinus, Percutaneous Approach */
"09CP4ZZ"  =  "1"  /* Extirpation of Matter from Accessory Sinus, Percutaneous Endoscopic Approach */
"09CQ0ZZ"  =  "1"  /* Extirpation of Matter from Right Maxillary Sinus, Open Approach */
"09CQ3ZZ"  =  "1"  /* Extirpation of Matter from Right Maxillary Sinus, Percutaneous Approach */
"09CQ4ZZ"  =  "1"  /* Extirpation of Matter from Right Maxillary Sinus, Percutaneous Endoscopic Approach */
"09CR0ZZ"  =  "1"  /* Extirpation of Matter from Left Maxillary Sinus, Open Approach */
"09CR3ZZ"  =  "1"  /* Extirpation of Matter from Left Maxillary Sinus, Percutaneous Approach */
"09CR4ZZ"  =  "1"  /* Extirpation of Matter from Left Maxillary Sinus, Percutaneous Endoscopic Approach */
"09CS0ZZ"  =  "1"  /* Extirpation of Matter from Right Frontal Sinus, Open Approach */
"09CS3ZZ"  =  "1"  /* Extirpation of Matter from Right Frontal Sinus, Percutaneous Approach */
"09CS4ZZ"  =  "1"  /* Extirpation of Matter from Right Frontal Sinus, Percutaneous Endoscopic Approach */
"09CT0ZZ"  =  "1"  /* Extirpation of Matter from Left Frontal Sinus, Open Approach */
"09CT3ZZ"  =  "1"  /* Extirpation of Matter from Left Frontal Sinus, Percutaneous Approach */
"09CT4ZZ"  =  "1"  /* Extirpation of Matter from Left Frontal Sinus, Percutaneous Endoscopic Approach */
"09CU0ZZ"  =  "1"  /* Extirpation of Matter from Right Ethmoid Sinus, Open Approach */
"09CU3ZZ"  =  "1"  /* Extirpation of Matter from Right Ethmoid Sinus, Percutaneous Approach */
"09CU4ZZ"  =  "1"  /* Extirpation of Matter from Right Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09CV0ZZ"  =  "1"  /* Extirpation of Matter from Left Ethmoid Sinus, Open Approach */
"09CV3ZZ"  =  "1"  /* Extirpation of Matter from Left Ethmoid Sinus, Percutaneous Approach */
"09CV4ZZ"  =  "1"  /* Extirpation of Matter from Left Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09CW0ZZ"  =  "1"  /* Extirpation of Matter from Right Sphenoid Sinus, Open Approach */
"09CW3ZZ"  =  "1"  /* Extirpation of Matter from Right Sphenoid Sinus, Percutaneous Approach */
"09CW4ZZ"  =  "1"  /* Extirpation of Matter from Right Sphenoid Sinus, Percutaneous Endoscopic Approach */
"09CX0ZZ"  =  "1"  /* Extirpation of Matter from Left Sphenoid Sinus, Open Approach */
"09CX3ZZ"  =  "1"  /* Extirpation of Matter from Left Sphenoid Sinus, Percutaneous Approach */
"09CX4ZZ"  =  "1"  /* Extirpation of Matter from Left Sphenoid Sinus, Percutaneous Endoscopic Approach */
"09DL0ZZ"  =  "1"  /* Extraction of Nasal Turbinate, Open Approach */
"09DL3ZZ"  =  "1"  /* Extraction of Nasal Turbinate, Percutaneous Approach */
"09DL4ZZ"  =  "1"  /* Extraction of Nasal Turbinate, Percutaneous Endoscopic Approach */
"09DL7ZZ"  =  "1"  /* Extraction of Nasal Turbinate, Via Natural or Artificial Opening */
"09DL8ZZ"  =  "1"  /* Extraction of Nasal Turbinate, Via Natural or Artificial Opening Endoscopic */
"09DM0ZZ"  =  "1"  /* Extraction of Nasal Septum, Open Approach */
"09DM3ZZ"  =  "1"  /* Extraction of Nasal Septum, Percutaneous Approach */
"09DM4ZZ"  =  "1"  /* Extraction of Nasal Septum, Percutaneous Endoscopic Approach */
"09DP0ZZ"  =  "1"  /* Extraction of Accessory Sinus, Open Approach */
"09DP3ZZ"  =  "1"  /* Extraction of Accessory Sinus, Percutaneous Approach */
"09DP4ZZ"  =  "1"  /* Extraction of Accessory Sinus, Percutaneous Endoscopic Approach */
"09DQ0ZZ"  =  "1"  /* Extraction of Right Maxillary Sinus, Open Approach */
"09DQ3ZZ"  =  "1"  /* Extraction of Right Maxillary Sinus, Percutaneous Approach */
"09DQ4ZZ"  =  "1"  /* Extraction of Right Maxillary Sinus, Percutaneous Endoscopic Approach */
"09DR0ZZ"  =  "1"  /* Extraction of Left Maxillary Sinus, Open Approach */
"09DR3ZZ"  =  "1"  /* Extraction of Left Maxillary Sinus, Percutaneous Approach */
"09DR4ZZ"  =  "1"  /* Extraction of Left Maxillary Sinus, Percutaneous Endoscopic Approach */
"09DS0ZZ"  =  "1"  /* Extraction of Right Frontal Sinus, Open Approach */
"09DS3ZZ"  =  "1"  /* Extraction of Right Frontal Sinus, Percutaneous Approach */
"09DS4ZZ"  =  "1"  /* Extraction of Right Frontal Sinus, Percutaneous Endoscopic Approach */
"09DT0ZZ"  =  "1"  /* Extraction of Left Frontal Sinus, Open Approach */
"09DT3ZZ"  =  "1"  /* Extraction of Left Frontal Sinus, Percutaneous Approach */
"09DT4ZZ"  =  "1"  /* Extraction of Left Frontal Sinus, Percutaneous Endoscopic Approach */
"09DU0ZZ"  =  "1"  /* Extraction of Right Ethmoid Sinus, Open Approach */
"09DU3ZZ"  =  "1"  /* Extraction of Right Ethmoid Sinus, Percutaneous Approach */
"09DU4ZZ"  =  "1"  /* Extraction of Right Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09DV0ZZ"  =  "1"  /* Extraction of Left Ethmoid Sinus, Open Approach */
"09DV3ZZ"  =  "1"  /* Extraction of Left Ethmoid Sinus, Percutaneous Approach */
"09DV4ZZ"  =  "1"  /* Extraction of Left Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09DW0ZZ"  =  "1"  /* Extraction of Right Sphenoid Sinus, Open Approach */
"09DW3ZZ"  =  "1"  /* Extraction of Right Sphenoid Sinus, Percutaneous Approach */
"09DW4ZZ"  =  "1"  /* Extraction of Right Sphenoid Sinus, Percutaneous Endoscopic Approach */
"09DX0ZZ"  =  "1"  /* Extraction of Left Sphenoid Sinus, Open Approach */
"09DX3ZZ"  =  "1"  /* Extraction of Left Sphenoid Sinus, Percutaneous Approach */
"09DX4ZZ"  =  "1"  /* Extraction of Left Sphenoid Sinus, Percutaneous Endoscopic Approach */
"09NN0ZZ"  =  "1"  /* Release Nasopharynx, Open Approach */
"09NN3ZZ"  =  "1"  /* Release Nasopharynx, Percutaneous Approach */
"09NN4ZZ"  =  "1"  /* Release Nasopharynx, Percutaneous Endoscopic Approach */
"09NN7ZZ"  =  "1"  /* Release Nasopharynx, Via Natural or Artificial Opening */
"09NN8ZZ"  =  "1"  /* Release Nasopharynx, Via Natural or Artificial Opening Endoscopic */
"09NP0ZZ"  =  "1"  /* Release Accessory Sinus, Open Approach */
"09NP3ZZ"  =  "1"  /* Release Accessory Sinus, Percutaneous Approach */
"09NP4ZZ"  =  "1"  /* Release Accessory Sinus, Percutaneous Endoscopic Approach */
"09NQ0ZZ"  =  "1"  /* Release Right Maxillary Sinus, Open Approach */
"09NQ3ZZ"  =  "1"  /* Release Right Maxillary Sinus, Percutaneous Approach */
"09NQ4ZZ"  =  "1"  /* Release Right Maxillary Sinus, Percutaneous Endoscopic Approach */
"09NR0ZZ"  =  "1"  /* Release Left Maxillary Sinus, Open Approach */
"09NR3ZZ"  =  "1"  /* Release Left Maxillary Sinus, Percutaneous Approach */
"09NR4ZZ"  =  "1"  /* Release Left Maxillary Sinus, Percutaneous Endoscopic Approach */
"09NS0ZZ"  =  "1"  /* Release Right Frontal Sinus, Open Approach */
"09NS3ZZ"  =  "1"  /* Release Right Frontal Sinus, Percutaneous Approach */
"09NS4ZZ"  =  "1"  /* Release Right Frontal Sinus, Percutaneous Endoscopic Approach */
"09NT0ZZ"  =  "1"  /* Release Left Frontal Sinus, Open Approach */
"09NT3ZZ"  =  "1"  /* Release Left Frontal Sinus, Percutaneous Approach */
"09NT4ZZ"  =  "1"  /* Release Left Frontal Sinus, Percutaneous Endoscopic Approach */
"09NU0ZZ"  =  "1"  /* Release Right Ethmoid Sinus, Open Approach */
"09NU3ZZ"  =  "1"  /* Release Right Ethmoid Sinus, Percutaneous Approach */
"09NU4ZZ"  =  "1"  /* Release Right Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09NV0ZZ"  =  "1"  /* Release Left Ethmoid Sinus, Open Approach */
"09NV3ZZ"  =  "1"  /* Release Left Ethmoid Sinus, Percutaneous Approach */
"09NV4ZZ"  =  "1"  /* Release Left Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09NW0ZZ"  =  "1"  /* Release Right Sphenoid Sinus, Open Approach */
"09NW3ZZ"  =  "1"  /* Release Right Sphenoid Sinus, Percutaneous Approach */
"09NW4ZZ"  =  "1"  /* Release Right Sphenoid Sinus, Percutaneous Endoscopic Approach */
"09NX0ZZ"  =  "1"  /* Release Left Sphenoid Sinus, Open Approach */
"09NX3ZZ"  =  "1"  /* Release Left Sphenoid Sinus, Percutaneous Approach */
"09NX4ZZ"  =  "1"  /* Release Left Sphenoid Sinus, Percutaneous Endoscopic Approach */
"09PY00Z"  =  "1"  /* Removal of Drainage Device from Sinus, Open Approach */
"09PY30Z"  =  "1"  /* Removal of Drainage Device from Sinus, Percutaneous Approach */
"09PY40Z"  =  "1"  /* Removal of Drainage Device from Sinus, Percutaneous Endoscopic Approach */
"09Q30ZZ"  =  "1"  /* Repair Right External Auditory Canal, Open Approach */
"09Q33ZZ"  =  "1"  /* Repair Right External Auditory Canal, Percutaneous Approach */
"09Q34ZZ"  =  "1"  /* Repair Right External Auditory Canal, Percutaneous Endoscopic Approach */
"09Q37ZZ"  =  "1"  /* Repair Right External Auditory Canal, Via Natural or Artificial Opening */
"09Q38ZZ"  =  "1"  /* Repair Right External Auditory Canal, Via Natural or Artificial Opening Endoscopic */
"09Q40ZZ"  =  "1"  /* Repair Left External Auditory Canal, Open Approach */
"09Q43ZZ"  =  "1"  /* Repair Left External Auditory Canal, Percutaneous Approach */
"09Q44ZZ"  =  "1"  /* Repair Left External Auditory Canal, Percutaneous Endoscopic Approach */
"09Q47ZZ"  =  "1"  /* Repair Left External Auditory Canal, Via Natural or Artificial Opening */
"09Q48ZZ"  =  "1"  /* Repair Left External Auditory Canal, Via Natural or Artificial Opening Endoscopic */
"09QK0ZZ"  =  "1"  /* Repair Nose, Open Approach */
"09QK3ZZ"  =  "1"  /* Repair Nose, Percutaneous Approach */
"09QK4ZZ"  =  "1"  /* Repair Nose, Percutaneous Endoscopic Approach */
"09QKXZZ"  =  "1"  /* Repair Nose, External Approach */
"09QL0ZZ"  =  "1"  /* Repair Nasal Turbinate, Open Approach */
"09QL3ZZ"  =  "1"  /* Repair Nasal Turbinate, Percutaneous Approach */
"09QL4ZZ"  =  "1"  /* Repair Nasal Turbinate, Percutaneous Endoscopic Approach */
"09QL7ZZ"  =  "1"  /* Repair Nasal Turbinate, Via Natural or Artificial Opening */
"09QL8ZZ"  =  "1"  /* Repair Nasal Turbinate, Via Natural or Artificial Opening Endoscopic */
"09QN0ZZ"  =  "1"  /* Repair Nasopharynx, Open Approach */
"09QN3ZZ"  =  "1"  /* Repair Nasopharynx, Percutaneous Approach */
"09QN4ZZ"  =  "1"  /* Repair Nasopharynx, Percutaneous Endoscopic Approach */
"09QN7ZZ"  =  "1"  /* Repair Nasopharynx, Via Natural or Artificial Opening */
"09QN8ZZ"  =  "1"  /* Repair Nasopharynx, Via Natural or Artificial Opening Endoscopic */
"09QP0ZZ"  =  "1"  /* Repair Accessory Sinus, Open Approach */
"09QP3ZZ"  =  "1"  /* Repair Accessory Sinus, Percutaneous Approach */
"09QP4ZZ"  =  "1"  /* Repair Accessory Sinus, Percutaneous Endoscopic Approach */
"09QQ0ZZ"  =  "1"  /* Repair Right Maxillary Sinus, Open Approach */
"09QQ3ZZ"  =  "1"  /* Repair Right Maxillary Sinus, Percutaneous Approach */
"09QQ4ZZ"  =  "1"  /* Repair Right Maxillary Sinus, Percutaneous Endoscopic Approach */
"09QR0ZZ"  =  "1"  /* Repair Left Maxillary Sinus, Open Approach */
"09QR3ZZ"  =  "1"  /* Repair Left Maxillary Sinus, Percutaneous Approach */
"09QR4ZZ"  =  "1"  /* Repair Left Maxillary Sinus, Percutaneous Endoscopic Approach */
"09QS0ZZ"  =  "1"  /* Repair Right Frontal Sinus, Open Approach */
"09QS3ZZ"  =  "1"  /* Repair Right Frontal Sinus, Percutaneous Approach */
"09QS4ZZ"  =  "1"  /* Repair Right Frontal Sinus, Percutaneous Endoscopic Approach */
"09QT0ZZ"  =  "1"  /* Repair Left Frontal Sinus, Open Approach */
"09QT3ZZ"  =  "1"  /* Repair Left Frontal Sinus, Percutaneous Approach */
"09QT4ZZ"  =  "1"  /* Repair Left Frontal Sinus, Percutaneous Endoscopic Approach */
"09QU0ZZ"  =  "1"  /* Repair Right Ethmoid Sinus, Open Approach */
"09QU3ZZ"  =  "1"  /* Repair Right Ethmoid Sinus, Percutaneous Approach */
"09QU4ZZ"  =  "1"  /* Repair Right Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09QV0ZZ"  =  "1"  /* Repair Left Ethmoid Sinus, Open Approach */
"09QV3ZZ"  =  "1"  /* Repair Left Ethmoid Sinus, Percutaneous Approach */
"09QV4ZZ"  =  "1"  /* Repair Left Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09QW0ZZ"  =  "1"  /* Repair Right Sphenoid Sinus, Open Approach */
"09QW3ZZ"  =  "1"  /* Repair Right Sphenoid Sinus, Percutaneous Approach */
"09QW4ZZ"  =  "1"  /* Repair Right Sphenoid Sinus, Percutaneous Endoscopic Approach */
"09QX0ZZ"  =  "1"  /* Repair Left Sphenoid Sinus, Open Approach */
"09QX3ZZ"  =  "1"  /* Repair Left Sphenoid Sinus, Percutaneous Approach */
"09QX4ZZ"  =  "1"  /* Repair Left Sphenoid Sinus, Percutaneous Endoscopic Approach */
"09RL07Z"  =  "1"  /* Replacement of Nasal Turbinate with Autologous Tissue Substitute, Open Approach */
"09RL0JZ"  =  "1"  /* Replacement of Nasal Turbinate with Synthetic Substitute, Open Approach */
"09RL0KZ"  =  "1"  /* Replacement of Nasal Turbinate with Nonautologous Tissue Substitute, Open Approach */
"09RL37Z"  =  "1"  /* Replacement of Nasal Turbinate with Autologous Tissue Substitute, Percutaneous Approach */
"09RL3JZ"  =  "1"  /* Replacement of Nasal Turbinate with Synthetic Substitute, Percutaneous Approach */
"09RL3KZ"  =  "1"  /* Replacement of Nasal Turbinate with Nonautologous Tissue Substitute, Percutaneous Approach */
"09RL47Z"  =  "1"  /* Replacement of Nasal Turbinate with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"09RL4JZ"  =  "1"  /* Replacement of Nasal Turbinate with Synthetic Substitute, Percutaneous Endoscopic Approach */
"09RL4KZ"  =  "1"  /* Replacement of Nasal Turbinate with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"09RL77Z"  =  "1"  /* Replacement of Nasal Turbinate with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"09RL7JZ"  =  "1"  /* Replacement of Nasal Turbinate with Synthetic Substitute, Via Natural or Artificial Opening */
"09RL7KZ"  =  "1"  /* Replacement of Nasal Turbinate with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"09RL87Z"  =  "1"  /* Replacement of Nasal Turbinate with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"09RL8JZ"  =  "1"  /* Replacement of Nasal Turbinate with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"09RL8KZ"  =  "1"  /* Replacement of Nasal Turbinate with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"09RN07Z"  =  "1"  /* Replacement of Nasopharynx with Autologous Tissue Substitute, Open Approach */
"09RN0JZ"  =  "1"  /* Replacement of Nasopharynx with Synthetic Substitute, Open Approach */
"09RN0KZ"  =  "1"  /* Replacement of Nasopharynx with Nonautologous Tissue Substitute, Open Approach */
"09RN77Z"  =  "1"  /* Replacement of Nasopharynx with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"09RN7JZ"  =  "1"  /* Replacement of Nasopharynx with Synthetic Substitute, Via Natural or Artificial Opening */
"09RN7KZ"  =  "1"  /* Replacement of Nasopharynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"09RN87Z"  =  "1"  /* Replacement of Nasopharynx with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"09RN8JZ"  =  "1"  /* Replacement of Nasopharynx with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"09RN8KZ"  =  "1"  /* Replacement of Nasopharynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"09SL0ZZ"  =  "1"  /* Reposition Nasal Turbinate, Open Approach */
"09SL4ZZ"  =  "1"  /* Reposition Nasal Turbinate, Percutaneous Endoscopic Approach */
"09SL7ZZ"  =  "1"  /* Reposition Nasal Turbinate, Via Natural or Artificial Opening */
"09SL8ZZ"  =  "1"  /* Reposition Nasal Turbinate, Via Natural or Artificial Opening Endoscopic */
"09TK0ZZ"  =  "1"  /* Resection of Nose, Open Approach */
"09TK4ZZ"  =  "1"  /* Resection of Nose, Percutaneous Endoscopic Approach */
"09TKXZZ"  =  "1"  /* Resection of Nose, External Approach */
"09TL0ZZ"  =  "1"  /* Resection of Nasal Turbinate, Open Approach */
"09TL4ZZ"  =  "1"  /* Resection of Nasal Turbinate, Percutaneous Endoscopic Approach */
"09TL7ZZ"  =  "1"  /* Resection of Nasal Turbinate, Via Natural or Artificial Opening */
"09TL8ZZ"  =  "1"  /* Resection of Nasal Turbinate, Via Natural or Artificial Opening Endoscopic */
"09TN0ZZ"  =  "1"  /* Resection of Nasopharynx, Open Approach */
"09TN4ZZ"  =  "1"  /* Resection of Nasopharynx, Percutaneous Endoscopic Approach */
"09TN7ZZ"  =  "1"  /* Resection of Nasopharynx, Via Natural or Artificial Opening */
"09TN8ZZ"  =  "1"  /* Resection of Nasopharynx, Via Natural or Artificial Opening Endoscopic */
"09TP0ZZ"  =  "1"  /* Resection of Accessory Sinus, Open Approach */
"09TP4ZZ"  =  "1"  /* Resection of Accessory Sinus, Percutaneous Endoscopic Approach */
"09TQ0ZZ"  =  "1"  /* Resection of Right Maxillary Sinus, Open Approach */
"09TQ4ZZ"  =  "1"  /* Resection of Right Maxillary Sinus, Percutaneous Endoscopic Approach */
"09TR0ZZ"  =  "1"  /* Resection of Left Maxillary Sinus, Open Approach */
"09TR4ZZ"  =  "1"  /* Resection of Left Maxillary Sinus, Percutaneous Endoscopic Approach */
"09TS0ZZ"  =  "1"  /* Resection of Right Frontal Sinus, Open Approach */
"09TS4ZZ"  =  "1"  /* Resection of Right Frontal Sinus, Percutaneous Endoscopic Approach */
"09TT0ZZ"  =  "1"  /* Resection of Left Frontal Sinus, Open Approach */
"09TT4ZZ"  =  "1"  /* Resection of Left Frontal Sinus, Percutaneous Endoscopic Approach */
"09TU0ZZ"  =  "1"  /* Resection of Right Ethmoid Sinus, Open Approach */
"09TU4ZZ"  =  "1"  /* Resection of Right Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09TV0ZZ"  =  "1"  /* Resection of Left Ethmoid Sinus, Open Approach */
"09TV4ZZ"  =  "1"  /* Resection of Left Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09TW0ZZ"  =  "1"  /* Resection of Right Sphenoid Sinus, Open Approach */
"09TW4ZZ"  =  "1"  /* Resection of Right Sphenoid Sinus, Percutaneous Endoscopic Approach */
"09TX0ZZ"  =  "1"  /* Resection of Left Sphenoid Sinus, Open Approach */
"09TX4ZZ"  =  "1"  /* Resection of Left Sphenoid Sinus, Percutaneous Endoscopic Approach */
"09UL07Z"  =  "1"  /* Supplement Nasal Turbinate with Autologous Tissue Substitute, Open Approach */
"09UL0JZ"  =  "1"  /* Supplement Nasal Turbinate with Synthetic Substitute, Open Approach */
"09UL0KZ"  =  "1"  /* Supplement Nasal Turbinate with Nonautologous Tissue Substitute, Open Approach */
"09UL37Z"  =  "1"  /* Supplement Nasal Turbinate with Autologous Tissue Substitute, Percutaneous Approach */
"09UL3JZ"  =  "1"  /* Supplement Nasal Turbinate with Synthetic Substitute, Percutaneous Approach */
"09UL3KZ"  =  "1"  /* Supplement Nasal Turbinate with Nonautologous Tissue Substitute, Percutaneous Approach */
"09UL47Z"  =  "1"  /* Supplement Nasal Turbinate with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"09UL4JZ"  =  "1"  /* Supplement Nasal Turbinate with Synthetic Substitute, Percutaneous Endoscopic Approach */
"09UL4KZ"  =  "1"  /* Supplement Nasal Turbinate with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"09UL77Z"  =  "1"  /* Supplement Nasal Turbinate with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"09UL7JZ"  =  "1"  /* Supplement Nasal Turbinate with Synthetic Substitute, Via Natural or Artificial Opening */
"09UL7KZ"  =  "1"  /* Supplement Nasal Turbinate with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"09UL87Z"  =  "1"  /* Supplement Nasal Turbinate with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"09UL8JZ"  =  "1"  /* Supplement Nasal Turbinate with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"09UL8KZ"  =  "1"  /* Supplement Nasal Turbinate with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"09UN07Z"  =  "1"  /* Supplement Nasopharynx with Autologous Tissue Substitute, Open Approach */
"09UN0JZ"  =  "1"  /* Supplement Nasopharynx with Synthetic Substitute, Open Approach */
"09UN0KZ"  =  "1"  /* Supplement Nasopharynx with Nonautologous Tissue Substitute, Open Approach */
"09UN77Z"  =  "1"  /* Supplement Nasopharynx with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"09UN7JZ"  =  "1"  /* Supplement Nasopharynx with Synthetic Substitute, Via Natural or Artificial Opening */
"09UN7KZ"  =  "1"  /* Supplement Nasopharynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"09UN87Z"  =  "1"  /* Supplement Nasopharynx with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"09UN8JZ"  =  "1"  /* Supplement Nasopharynx with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"09UN8KZ"  =  "1"  /* Supplement Nasopharynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"09WY00Z"  =  "1"  /* Revision of Drainage Device in Sinus, Open Approach */
"09WY30Z"  =  "1"  /* Revision of Drainage Device in Sinus, Percutaneous Approach */
"09WY40Z"  =  "1"  /* Revision of Drainage Device in Sinus, Percutaneous Endoscopic Approach */
"0B110D6"  =  "1"  /* Bypass Trachea to Esophagus with Intraluminal Device, Open Approach */
"0B110F4"  =  "1"  /* Bypass Trachea to Cutaneous with Tracheostomy Device, Open Approach */
"0B110Z4"  =  "1"  /* Bypass Trachea to Cutaneous, Open Approach */
"0B113F4"  =  "1"  /* Bypass Trachea to Cutaneous with Tracheostomy Device, Percutaneous Approach */
"0B113Z4"  =  "1"  /* Bypass Trachea to Cutaneous, Percutaneous Approach */
"0B114F4"  =  "1"  /* Bypass Trachea to Cutaneous with Tracheostomy Device, Percutaneous Endoscopic Approach */
"0B114Z4"  =  "1"  /* Bypass Trachea to Cutaneous, Percutaneous Endoscopic Approach */
"0B510ZZ"  =  "1"  /* Destruction of Trachea, Open Approach */
"0B513ZZ"  =  "1"  /* Destruction of Trachea, Percutaneous Approach */
"0B514ZZ"  =  "1"  /* Destruction of Trachea, Percutaneous Endoscopic Approach */
"0B517ZZ"  =  "1"  /* Destruction of Trachea, Via Natural or Artificial Opening */
"0B518ZZ"  =  "1"  /* Destruction of Trachea, Via Natural or Artificial Opening Endoscopic */
"0B520ZZ"  =  "1"  /* Destruction of Carina, Open Approach */
"0B523ZZ"  =  "1"  /* Destruction of Carina, Percutaneous Approach */
"0B524ZZ"  =  "1"  /* Destruction of Carina, Percutaneous Endoscopic Approach */
"0B527ZZ"  =  "1"  /* Destruction of Carina, Via Natural or Artificial Opening */
"0B528ZZ"  =  "1"  /* Destruction of Carina, Via Natural or Artificial Opening Endoscopic */
"0B710DZ"  =  "1"  /* Dilation of Trachea with Intraluminal Device, Open Approach */
"0B710ZZ"  =  "1"  /* Dilation of Trachea, Open Approach */
"0B713DZ"  =  "1"  /* Dilation of Trachea with Intraluminal Device, Percutaneous Approach */
"0B713ZZ"  =  "1"  /* Dilation of Trachea, Percutaneous Approach */
"0B714DZ"  =  "1"  /* Dilation of Trachea with Intraluminal Device, Percutaneous Endoscopic Approach */
"0B714ZZ"  =  "1"  /* Dilation of Trachea, Percutaneous Endoscopic Approach */
"0B717DZ"  =  "1"  /* Dilation of Trachea with Intraluminal Device, Via Natural or Artificial Opening */
"0B717ZZ"  =  "1"  /* Dilation of Trachea, Via Natural or Artificial Opening */
"0B718DZ"  =  "1"  /* Dilation of Trachea with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0B718ZZ"  =  "1"  /* Dilation of Trachea, Via Natural or Artificial Opening Endoscopic */
"0B720DZ"  =  "1"  /* Dilation of Carina with Intraluminal Device, Open Approach */
"0B720ZZ"  =  "1"  /* Dilation of Carina, Open Approach */
"0B723DZ"  =  "1"  /* Dilation of Carina with Intraluminal Device, Percutaneous Approach */
"0B723ZZ"  =  "1"  /* Dilation of Carina, Percutaneous Approach */
"0B724DZ"  =  "1"  /* Dilation of Carina with Intraluminal Device, Percutaneous Endoscopic Approach */
"0B724ZZ"  =  "1"  /* Dilation of Carina, Percutaneous Endoscopic Approach */
"0B727DZ"  =  "1"  /* Dilation of Carina with Intraluminal Device, Via Natural or Artificial Opening */
"0B727ZZ"  =  "1"  /* Dilation of Carina, Via Natural or Artificial Opening */
"0B728DZ"  =  "1"  /* Dilation of Carina with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0B728ZZ"  =  "1"  /* Dilation of Carina, Via Natural or Artificial Opening Endoscopic */
"0B9100Z"  =  "1"  /* Drainage of Trachea with Drainage Device, Open Approach */
"0B910ZZ"  =  "1"  /* Drainage of Trachea, Open Approach */
"0B9130Z"  =  "1"  /* Drainage of Trachea with Drainage Device, Percutaneous Approach */
"0B913ZZ"  =  "1"  /* Drainage of Trachea, Percutaneous Approach */
"0B9140Z"  =  "1"  /* Drainage of Trachea with Drainage Device, Percutaneous Endoscopic Approach */
"0B914ZZ"  =  "1"  /* Drainage of Trachea, Percutaneous Endoscopic Approach */
"0B9170Z"  =  "1"  /* Drainage of Trachea with Drainage Device, Via Natural or Artificial Opening */
"0B917ZZ"  =  "1"  /* Drainage of Trachea, Via Natural or Artificial Opening */
"0B9180Z"  =  "1"  /* Drainage of Trachea with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B918ZZ"  =  "1"  /* Drainage of Trachea, Via Natural or Artificial Opening Endoscopic */
"0B9200Z"  =  "1"  /* Drainage of Carina with Drainage Device, Open Approach */
"0B920ZZ"  =  "1"  /* Drainage of Carina, Open Approach */
"0B9230Z"  =  "1"  /* Drainage of Carina with Drainage Device, Percutaneous Approach */
"0B923ZZ"  =  "1"  /* Drainage of Carina, Percutaneous Approach */
"0B9240Z"  =  "1"  /* Drainage of Carina with Drainage Device, Percutaneous Endoscopic Approach */
"0B924ZZ"  =  "1"  /* Drainage of Carina, Percutaneous Endoscopic Approach */
"0B9270Z"  =  "1"  /* Drainage of Carina with Drainage Device, Via Natural or Artificial Opening */
"0B927ZZ"  =  "1"  /* Drainage of Carina, Via Natural or Artificial Opening */
"0B9280Z"  =  "1"  /* Drainage of Carina with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0B928ZZ"  =  "1"  /* Drainage of Carina, Via Natural or Artificial Opening Endoscopic */
"0BB10ZZ"  =  "1"  /* Excision of Trachea, Open Approach */
"0BB13ZZ"  =  "1"  /* Excision of Trachea, Percutaneous Approach */
"0BB14ZZ"  =  "1"  /* Excision of Trachea, Percutaneous Endoscopic Approach */
"0BB17ZZ"  =  "1"  /* Excision of Trachea, Via Natural or Artificial Opening */
"0BB18ZZ"  =  "1"  /* Excision of Trachea, Via Natural or Artificial Opening Endoscopic */
"0BB20ZZ"  =  "1"  /* Excision of Carina, Open Approach */
"0BB23ZZ"  =  "1"  /* Excision of Carina, Percutaneous Approach */
"0BB24ZZ"  =  "1"  /* Excision of Carina, Percutaneous Endoscopic Approach */
"0BB27ZZ"  =  "1"  /* Excision of Carina, Via Natural or Artificial Opening */
"0BB28ZZ"  =  "1"  /* Excision of Carina, Via Natural or Artificial Opening Endoscopic */
"0BC10ZZ"  =  "1"  /* Extirpation of Matter from Trachea, Open Approach */
"0BC13ZZ"  =  "1"  /* Extirpation of Matter from Trachea, Percutaneous Approach */
"0BC14ZZ"  =  "1"  /* Extirpation of Matter from Trachea, Percutaneous Endoscopic Approach */
"0BC20ZZ"  =  "1"  /* Extirpation of Matter from Carina, Open Approach */
"0BC23ZZ"  =  "1"  /* Extirpation of Matter from Carina, Percutaneous Approach */
"0BC24ZZ"  =  "1"  /* Extirpation of Matter from Carina, Percutaneous Endoscopic Approach */
"0BF10ZZ"  =  "1"  /* Fragmentation in Trachea, Open Approach */
"0BF13ZZ"  =  "1"  /* Fragmentation in Trachea, Percutaneous Approach */
"0BF14ZZ"  =  "1"  /* Fragmentation in Trachea, Percutaneous Endoscopic Approach */
"0BF17ZZ"  =  "1"  /* Fragmentation in Trachea, Via Natural or Artificial Opening */
"0BF18ZZ"  =  "1"  /* Fragmentation in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BF20ZZ"  =  "1"  /* Fragmentation in Carina, Open Approach */
"0BF23ZZ"  =  "1"  /* Fragmentation in Carina, Percutaneous Approach */
"0BF24ZZ"  =  "1"  /* Fragmentation in Carina, Percutaneous Endoscopic Approach */
"0BF27ZZ"  =  "1"  /* Fragmentation in Carina, Via Natural or Artificial Opening */
"0BF28ZZ"  =  "1"  /* Fragmentation in Carina, Via Natural or Artificial Opening Endoscopic */
"0BH102Z"  =  "1"  /* Insertion of Monitoring Device into Trachea, Open Approach */
"0BH10DZ"  =  "1"  /* Insertion of Intraluminal Device into Trachea, Open Approach */
"0BH13DZ"  =  "1"  /* Insertion of Intraluminal Device into Trachea, Percutaneous Approach */
"0BH14DZ"  =  "1"  /* Insertion of Intraluminal Device into Trachea, Percutaneous Endoscopic Approach */
"0BH172Z"  =  "1"  /* Insertion of Monitoring Device into Trachea, Via Natural or Artificial Opening */
"0BH17DZ"  =  "1"  /* Insertion of Intraluminal Device into Trachea, Via Natural or Artificial Opening */
"0BH182Z"  =  "1"  /* Insertion of Monitoring Device into Trachea, Via Natural or Artificial Opening Endoscopic */
"0BH18DZ"  =  "1"  /* Insertion of Intraluminal Device into Trachea, Via Natural or Artificial Opening Endoscopic */
"0BL10CZ"  =  "1"  /* Occlusion of Trachea with Extraluminal Device, Open Approach */
"0BL10DZ"  =  "1"  /* Occlusion of Trachea with Intraluminal Device, Open Approach */
"0BL10ZZ"  =  "1"  /* Occlusion of Trachea, Open Approach */
"0BL13CZ"  =  "1"  /* Occlusion of Trachea with Extraluminal Device, Percutaneous Approach */
"0BL13DZ"  =  "1"  /* Occlusion of Trachea with Intraluminal Device, Percutaneous Approach */
"0BL13ZZ"  =  "1"  /* Occlusion of Trachea, Percutaneous Approach */
"0BL14CZ"  =  "1"  /* Occlusion of Trachea with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BL14DZ"  =  "1"  /* Occlusion of Trachea with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BL14ZZ"  =  "1"  /* Occlusion of Trachea, Percutaneous Endoscopic Approach */
"0BL17DZ"  =  "1"  /* Occlusion of Trachea with Intraluminal Device, Via Natural or Artificial Opening */
"0BL17ZZ"  =  "1"  /* Occlusion of Trachea, Via Natural or Artificial Opening */
"0BL18DZ"  =  "1"  /* Occlusion of Trachea with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BL18ZZ"  =  "1"  /* Occlusion of Trachea, Via Natural or Artificial Opening Endoscopic */
"0BL20CZ"  =  "1"  /* Occlusion of Carina with Extraluminal Device, Open Approach */
"0BL20DZ"  =  "1"  /* Occlusion of Carina with Intraluminal Device, Open Approach */
"0BL20ZZ"  =  "1"  /* Occlusion of Carina, Open Approach */
"0BL23CZ"  =  "1"  /* Occlusion of Carina with Extraluminal Device, Percutaneous Approach */
"0BL23DZ"  =  "1"  /* Occlusion of Carina with Intraluminal Device, Percutaneous Approach */
"0BL23ZZ"  =  "1"  /* Occlusion of Carina, Percutaneous Approach */
"0BL24CZ"  =  "1"  /* Occlusion of Carina with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BL24DZ"  =  "1"  /* Occlusion of Carina with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BL24ZZ"  =  "1"  /* Occlusion of Carina, Percutaneous Endoscopic Approach */
"0BL27DZ"  =  "1"  /* Occlusion of Carina with Intraluminal Device, Via Natural or Artificial Opening */
"0BL27ZZ"  =  "1"  /* Occlusion of Carina, Via Natural or Artificial Opening */
"0BL28DZ"  =  "1"  /* Occlusion of Carina with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BL28ZZ"  =  "1"  /* Occlusion of Carina, Via Natural or Artificial Opening Endoscopic */
"0BM10ZZ"  =  "1"  /* Reattachment of Trachea, Open Approach */
"0BM20ZZ"  =  "1"  /* Reattachment of Carina, Open Approach */
"0BN10ZZ"  =  "1"  /* Release Trachea, Open Approach */
"0BN13ZZ"  =  "1"  /* Release Trachea, Percutaneous Approach */
"0BN14ZZ"  =  "1"  /* Release Trachea, Percutaneous Endoscopic Approach */
"0BN17ZZ"  =  "1"  /* Release Trachea, Via Natural or Artificial Opening */
"0BN18ZZ"  =  "1"  /* Release Trachea, Via Natural or Artificial Opening Endoscopic */
"0BN20ZZ"  =  "1"  /* Release Carina, Open Approach */
"0BN23ZZ"  =  "1"  /* Release Carina, Percutaneous Approach */
"0BN24ZZ"  =  "1"  /* Release Carina, Percutaneous Endoscopic Approach */
"0BN27ZZ"  =  "1"  /* Release Carina, Via Natural or Artificial Opening */
"0BN28ZZ"  =  "1"  /* Release Carina, Via Natural or Artificial Opening Endoscopic */
"0BP003Z"  =  "1"  /* Removal of Infusion Device from Tracheobronchial Tree, Open Approach */
"0BP007Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Tracheobronchial Tree, Open Approach */
"0BP033Z"  =  "1"  /* Removal of Infusion Device from Tracheobronchial Tree, Percutaneous Approach */
"0BP037Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Tracheobronchial Tree, Percutaneous Approach */
"0BP043Z"  =  "1"  /* Removal of Infusion Device from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP047Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BP073Z"  =  "1"  /* Removal of Infusion Device from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP077Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP07CZ"  =  "1"  /* Removal of Extraluminal Device from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP07JZ"  =  "1"  /* Removal of Synthetic Substitute from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP07KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BP083Z"  =  "1"  /* Removal of Infusion Device from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP087Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP08CZ"  =  "1"  /* Removal of Extraluminal Device from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP08JZ"  =  "1"  /* Removal of Synthetic Substitute from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP08KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BP100Z"  =  "1"  /* Removal of Drainage Device from Trachea, Open Approach */
"0BP102Z"  =  "1"  /* Removal of Monitoring Device from Trachea, Open Approach */
"0BP107Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Trachea, Open Approach */
"0BP10CZ"  =  "1"  /* Removal of Extraluminal Device from Trachea, Open Approach */
"0BP10DZ"  =  "1"  /* Removal of Intraluminal Device from Trachea, Open Approach */
"0BP10FZ"  =  "1"  /* Removal of Tracheostomy Device from Trachea, Open Approach */
"0BP10JZ"  =  "1"  /* Removal of Synthetic Substitute from Trachea, Open Approach */
"0BP10KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Trachea, Open Approach */
"0BP130Z"  =  "1"  /* Removal of Drainage Device from Trachea, Percutaneous Approach */
"0BP132Z"  =  "1"  /* Removal of Monitoring Device from Trachea, Percutaneous Approach */
"0BP137Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Trachea, Percutaneous Approach */
"0BP13CZ"  =  "1"  /* Removal of Extraluminal Device from Trachea, Percutaneous Approach */
"0BP13DZ"  =  "1"  /* Removal of Intraluminal Device from Trachea, Percutaneous Approach */
"0BP13FZ"  =  "1"  /* Removal of Tracheostomy Device from Trachea, Percutaneous Approach */
"0BP13JZ"  =  "1"  /* Removal of Synthetic Substitute from Trachea, Percutaneous Approach */
"0BP13KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Trachea, Percutaneous Approach */
"0BP140Z"  =  "1"  /* Removal of Drainage Device from Trachea, Percutaneous Endoscopic Approach */
"0BP142Z"  =  "1"  /* Removal of Monitoring Device from Trachea, Percutaneous Endoscopic Approach */
"0BP147Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Trachea, Percutaneous Endoscopic Approach */
"0BP14CZ"  =  "1"  /* Removal of Extraluminal Device from Trachea, Percutaneous Endoscopic Approach */
"0BP14DZ"  =  "1"  /* Removal of Intraluminal Device from Trachea, Percutaneous Endoscopic Approach */
"0BP14FZ"  =  "1"  /* Removal of Tracheostomy Device from Trachea, Percutaneous Endoscopic Approach */
"0BP14JZ"  =  "1"  /* Removal of Synthetic Substitute from Trachea, Percutaneous Endoscopic Approach */
"0BP14KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Trachea, Percutaneous Endoscopic Approach */
"0BP170Z"  =  "1"  /* Removal of Drainage Device from Trachea, Via Natural or Artificial Opening */
"0BP172Z"  =  "1"  /* Removal of Monitoring Device from Trachea, Via Natural or Artificial Opening */
"0BP177Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Trachea, Via Natural or Artificial Opening */
"0BP17CZ"  =  "1"  /* Removal of Extraluminal Device from Trachea, Via Natural or Artificial Opening */
"0BP17DZ"  =  "1"  /* Removal of Intraluminal Device from Trachea, Via Natural or Artificial Opening */
"0BP17FZ"  =  "1"  /* Removal of Tracheostomy Device from Trachea, Via Natural or Artificial Opening */
"0BP17JZ"  =  "1"  /* Removal of Synthetic Substitute from Trachea, Via Natural or Artificial Opening */
"0BP17KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Trachea, Via Natural or Artificial Opening */
"0BP180Z"  =  "1"  /* Removal of Drainage Device from Trachea, Via Natural or Artificial Opening Endoscopic */
"0BP182Z"  =  "1"  /* Removal of Monitoring Device from Trachea, Via Natural or Artificial Opening Endoscopic */
"0BP187Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Trachea, Via Natural or Artificial Opening Endoscopic */
"0BP18CZ"  =  "1"  /* Removal of Extraluminal Device from Trachea, Via Natural or Artificial Opening Endoscopic */
"0BP18DZ"  =  "1"  /* Removal of Intraluminal Device from Trachea, Via Natural or Artificial Opening Endoscopic */
"0BP18FZ"  =  "1"  /* Removal of Tracheostomy Device from Trachea, Via Natural or Artificial Opening Endoscopic */
"0BP18JZ"  =  "1"  /* Removal of Synthetic Substitute from Trachea, Via Natural or Artificial Opening Endoscopic */
"0BP18KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Trachea, Via Natural or Artificial Opening Endoscopic */
"0BQ10ZZ"  =  "1"  /* Repair Trachea, Open Approach */
"0BQ13ZZ"  =  "1"  /* Repair Trachea, Percutaneous Approach */
"0BQ14ZZ"  =  "1"  /* Repair Trachea, Percutaneous Endoscopic Approach */
"0BQ17ZZ"  =  "1"  /* Repair Trachea, Via Natural or Artificial Opening */
"0BQ18ZZ"  =  "1"  /* Repair Trachea, Via Natural or Artificial Opening Endoscopic */
"0BQ20ZZ"  =  "1"  /* Repair Carina, Open Approach */
"0BQ23ZZ"  =  "1"  /* Repair Carina, Percutaneous Approach */
"0BQ24ZZ"  =  "1"  /* Repair Carina, Percutaneous Endoscopic Approach */
"0BQ27ZZ"  =  "1"  /* Repair Carina, Via Natural or Artificial Opening */
"0BQ28ZZ"  =  "1"  /* Repair Carina, Via Natural or Artificial Opening Endoscopic */
"0BS10ZZ"  =  "1"  /* Reposition Trachea, Open Approach */
"0BS20ZZ"  =  "1"  /* Reposition Carina, Open Approach */
"0BT10ZZ"  =  "1"  /* Resection of Trachea, Open Approach */
"0BT14ZZ"  =  "1"  /* Resection of Trachea, Percutaneous Endoscopic Approach */
"0BT20ZZ"  =  "1"  /* Resection of Carina, Open Approach */
"0BT24ZZ"  =  "1"  /* Resection of Carina, Percutaneous Endoscopic Approach */
"0BU107Z"  =  "1"  /* Supplement Trachea with Autologous Tissue Substitute, Open Approach */
"0BU10JZ"  =  "1"  /* Supplement Trachea with Synthetic Substitute, Open Approach */
"0BU10KZ"  =  "1"  /* Supplement Trachea with Nonautologous Tissue Substitute, Open Approach */
"0BU147Z"  =  "1"  /* Supplement Trachea with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU14JZ"  =  "1"  /* Supplement Trachea with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BU14KZ"  =  "1"  /* Supplement Trachea with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU207Z"  =  "1"  /* Supplement Carina with Autologous Tissue Substitute, Open Approach */
"0BU20JZ"  =  "1"  /* Supplement Carina with Synthetic Substitute, Open Approach */
"0BU20KZ"  =  "1"  /* Supplement Carina with Nonautologous Tissue Substitute, Open Approach */
"0BU247Z"  =  "1"  /* Supplement Carina with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BU24JZ"  =  "1"  /* Supplement Carina with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BU24KZ"  =  "1"  /* Supplement Carina with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BV10CZ"  =  "1"  /* Restriction of Trachea with Extraluminal Device, Open Approach */
"0BV10DZ"  =  "1"  /* Restriction of Trachea with Intraluminal Device, Open Approach */
"0BV10ZZ"  =  "1"  /* Restriction of Trachea, Open Approach */
"0BV13CZ"  =  "1"  /* Restriction of Trachea with Extraluminal Device, Percutaneous Approach */
"0BV13DZ"  =  "1"  /* Restriction of Trachea with Intraluminal Device, Percutaneous Approach */
"0BV13ZZ"  =  "1"  /* Restriction of Trachea, Percutaneous Approach */
"0BV14CZ"  =  "1"  /* Restriction of Trachea with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BV14DZ"  =  "1"  /* Restriction of Trachea with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BV14ZZ"  =  "1"  /* Restriction of Trachea, Percutaneous Endoscopic Approach */
"0BV17DZ"  =  "1"  /* Restriction of Trachea with Intraluminal Device, Via Natural or Artificial Opening */
"0BV17ZZ"  =  "1"  /* Restriction of Trachea, Via Natural or Artificial Opening */
"0BV18DZ"  =  "1"  /* Restriction of Trachea with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BV18ZZ"  =  "1"  /* Restriction of Trachea, Via Natural or Artificial Opening Endoscopic */
"0BV20CZ"  =  "1"  /* Restriction of Carina with Extraluminal Device, Open Approach */
"0BV20DZ"  =  "1"  /* Restriction of Carina with Intraluminal Device, Open Approach */
"0BV20ZZ"  =  "1"  /* Restriction of Carina, Open Approach */
"0BV23CZ"  =  "1"  /* Restriction of Carina with Extraluminal Device, Percutaneous Approach */
"0BV23DZ"  =  "1"  /* Restriction of Carina with Intraluminal Device, Percutaneous Approach */
"0BV23ZZ"  =  "1"  /* Restriction of Carina, Percutaneous Approach */
"0BV24CZ"  =  "1"  /* Restriction of Carina with Extraluminal Device, Percutaneous Endoscopic Approach */
"0BV24DZ"  =  "1"  /* Restriction of Carina with Intraluminal Device, Percutaneous Endoscopic Approach */
"0BV24ZZ"  =  "1"  /* Restriction of Carina, Percutaneous Endoscopic Approach */
"0BV27DZ"  =  "1"  /* Restriction of Carina with Intraluminal Device, Via Natural or Artificial Opening */
"0BV27ZZ"  =  "1"  /* Restriction of Carina, Via Natural or Artificial Opening */
"0BV28DZ"  =  "1"  /* Restriction of Carina with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0BV28ZZ"  =  "1"  /* Restriction of Carina, Via Natural or Artificial Opening Endoscopic */
"0BW003Z"  =  "1"  /* Revision of Infusion Device in Tracheobronchial Tree, Open Approach */
"0BW007Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Tracheobronchial Tree, Open Approach */
"0BW033Z"  =  "1"  /* Revision of Infusion Device in Tracheobronchial Tree, Percutaneous Approach */
"0BW037Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Tracheobronchial Tree, Percutaneous Approach */
"0BW043Z"  =  "1"  /* Revision of Infusion Device in Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BW047Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Tracheobronchial Tree, Percutaneous Endoscopic Approach */
"0BW073Z"  =  "1"  /* Revision of Infusion Device in Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BW077Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BW07CZ"  =  "1"  /* Revision of Extraluminal Device in Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BW07JZ"  =  "1"  /* Revision of Synthetic Substitute in Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BW07KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Tracheobronchial Tree, Via Natural or Artificial Opening */
"0BW083Z"  =  "1"  /* Revision of Infusion Device in Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BW087Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BW08CZ"  =  "1"  /* Revision of Extraluminal Device in Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BW08JZ"  =  "1"  /* Revision of Synthetic Substitute in Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BW08KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Tracheobronchial Tree, Via Natural or Artificial Opening Endoscopic */
"0BW100Z"  =  "1"  /* Revision of Drainage Device in Trachea, Open Approach */
"0BW102Z"  =  "1"  /* Revision of Monitoring Device in Trachea, Open Approach */
"0BW107Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Trachea, Open Approach */
"0BW10CZ"  =  "1"  /* Revision of Extraluminal Device in Trachea, Open Approach */
"0BW10DZ"  =  "1"  /* Revision of Intraluminal Device in Trachea, Open Approach */
"0BW10JZ"  =  "1"  /* Revision of Synthetic Substitute in Trachea, Open Approach */
"0BW10KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Trachea, Open Approach */
"0BW130Z"  =  "1"  /* Revision of Drainage Device in Trachea, Percutaneous Approach */
"0BW132Z"  =  "1"  /* Revision of Monitoring Device in Trachea, Percutaneous Approach */
"0BW137Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Trachea, Percutaneous Approach */
"0BW13CZ"  =  "1"  /* Revision of Extraluminal Device in Trachea, Percutaneous Approach */
"0BW13DZ"  =  "1"  /* Revision of Intraluminal Device in Trachea, Percutaneous Approach */
"0BW13JZ"  =  "1"  /* Revision of Synthetic Substitute in Trachea, Percutaneous Approach */
"0BW13KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Trachea, Percutaneous Approach */
"0BW140Z"  =  "1"  /* Revision of Drainage Device in Trachea, Percutaneous Endoscopic Approach */
"0BW142Z"  =  "1"  /* Revision of Monitoring Device in Trachea, Percutaneous Endoscopic Approach */
"0BW147Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Trachea, Percutaneous Endoscopic Approach */
"0BW14CZ"  =  "1"  /* Revision of Extraluminal Device in Trachea, Percutaneous Endoscopic Approach */
"0BW14DZ"  =  "1"  /* Revision of Intraluminal Device in Trachea, Percutaneous Endoscopic Approach */
"0BW14JZ"  =  "1"  /* Revision of Synthetic Substitute in Trachea, Percutaneous Endoscopic Approach */
"0BW14KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Trachea, Percutaneous Endoscopic Approach */
"0BW170Z"  =  "1"  /* Revision of Drainage Device in Trachea, Via Natural or Artificial Opening */
"0BW172Z"  =  "1"  /* Revision of Monitoring Device in Trachea, Via Natural or Artificial Opening */
"0BW177Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Trachea, Via Natural or Artificial Opening */
"0BW17CZ"  =  "1"  /* Revision of Extraluminal Device in Trachea, Via Natural or Artificial Opening */
"0BW17DZ"  =  "1"  /* Revision of Intraluminal Device in Trachea, Via Natural or Artificial Opening */
"0BW17FZ"  =  "1"  /* Revision of Tracheostomy Device in Trachea, Via Natural or Artificial Opening */
"0BW17JZ"  =  "1"  /* Revision of Synthetic Substitute in Trachea, Via Natural or Artificial Opening */
"0BW17KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Trachea, Via Natural or Artificial Opening */
"0BW180Z"  =  "1"  /* Revision of Drainage Device in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BW182Z"  =  "1"  /* Revision of Monitoring Device in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BW187Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BW18CZ"  =  "1"  /* Revision of Extraluminal Device in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BW18DZ"  =  "1"  /* Revision of Intraluminal Device in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BW18FZ"  =  "1"  /* Revision of Tracheostomy Device in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BW18JZ"  =  "1"  /* Revision of Synthetic Substitute in Trachea, Via Natural or Artificial Opening Endoscopic */
"0BW18KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Trachea, Via Natural or Artificial Opening Endoscopic */
"0C00X7Z"  =  "1"  /* Alteration of Upper Lip with Autologous Tissue Substitute, External Approach */
"0C00XJZ"  =  "1"  /* Alteration of Upper Lip with Synthetic Substitute, External Approach */
"0C00XKZ"  =  "1"  /* Alteration of Upper Lip with Nonautologous Tissue Substitute, External Approach */
"0C00XZZ"  =  "1"  /* Alteration of Upper Lip, External Approach */
"0C01X7Z"  =  "1"  /* Alteration of Lower Lip with Autologous Tissue Substitute, External Approach */
"0C01XJZ"  =  "1"  /* Alteration of Lower Lip with Synthetic Substitute, External Approach */
"0C01XKZ"  =  "1"  /* Alteration of Lower Lip with Nonautologous Tissue Substitute, External Approach */
"0C01XZZ"  =  "1"  /* Alteration of Lower Lip, External Approach */
"0C500ZZ"  =  "1"  /* Destruction of Upper Lip, Open Approach */
"0C503ZZ"  =  "1"  /* Destruction of Upper Lip, Percutaneous Approach */
"0C50XZZ"  =  "1"  /* Destruction of Upper Lip, External Approach */
"0C510ZZ"  =  "1"  /* Destruction of Lower Lip, Open Approach */
"0C513ZZ"  =  "1"  /* Destruction of Lower Lip, Percutaneous Approach */
"0C51XZZ"  =  "1"  /* Destruction of Lower Lip, External Approach */
"0C520ZZ"  =  "1"  /* Destruction of Hard Palate, Open Approach */
"0C523ZZ"  =  "1"  /* Destruction of Hard Palate, Percutaneous Approach */
"0C52XZZ"  =  "1"  /* Destruction of Hard Palate, External Approach */
"0C530ZZ"  =  "1"  /* Destruction of Soft Palate, Open Approach */
"0C533ZZ"  =  "1"  /* Destruction of Soft Palate, Percutaneous Approach */
"0C53XZZ"  =  "1"  /* Destruction of Soft Palate, External Approach */
"0C540ZZ"  =  "1"  /* Destruction of Buccal Mucosa, Open Approach */
"0C543ZZ"  =  "1"  /* Destruction of Buccal Mucosa, Percutaneous Approach */
"0C54XZZ"  =  "1"  /* Destruction of Buccal Mucosa, External Approach */
"0C570ZZ"  =  "1"  /* Destruction of Tongue, Open Approach */
"0C573ZZ"  =  "1"  /* Destruction of Tongue, Percutaneous Approach */
"0C57XZZ"  =  "1"  /* Destruction of Tongue, External Approach */
"0C580ZZ"  =  "1"  /* Destruction of Right Parotid Gland, Open Approach */
"0C583ZZ"  =  "1"  /* Destruction of Right Parotid Gland, Percutaneous Approach */
"0C590ZZ"  =  "1"  /* Destruction of Left Parotid Gland, Open Approach */
"0C593ZZ"  =  "1"  /* Destruction of Left Parotid Gland, Percutaneous Approach */
"0C5B0ZZ"  =  "1"  /* Destruction of Right Parotid Duct, Open Approach */
"0C5B3ZZ"  =  "1"  /* Destruction of Right Parotid Duct, Percutaneous Approach */
"0C5C0ZZ"  =  "1"  /* Destruction of Left Parotid Duct, Open Approach */
"0C5C3ZZ"  =  "1"  /* Destruction of Left Parotid Duct, Percutaneous Approach */
"0C5D0ZZ"  =  "1"  /* Destruction of Right Sublingual Gland, Open Approach */
"0C5D3ZZ"  =  "1"  /* Destruction of Right Sublingual Gland, Percutaneous Approach */
"0C5F0ZZ"  =  "1"  /* Destruction of Left Sublingual Gland, Open Approach */
"0C5F3ZZ"  =  "1"  /* Destruction of Left Sublingual Gland, Percutaneous Approach */
"0C5G0ZZ"  =  "1"  /* Destruction of Right Submaxillary Gland, Open Approach */
"0C5G3ZZ"  =  "1"  /* Destruction of Right Submaxillary Gland, Percutaneous Approach */
"0C5H0ZZ"  =  "1"  /* Destruction of Left Submaxillary Gland, Open Approach */
"0C5H3ZZ"  =  "1"  /* Destruction of Left Submaxillary Gland, Percutaneous Approach */
"0C5J0ZZ"  =  "1"  /* Destruction of Minor Salivary Gland, Open Approach */
"0C5J3ZZ"  =  "1"  /* Destruction of Minor Salivary Gland, Percutaneous Approach */
"0C5M0ZZ"  =  "1"  /* Destruction of Pharynx, Open Approach */
"0C5M3ZZ"  =  "1"  /* Destruction of Pharynx, Percutaneous Approach */
"0C5M4ZZ"  =  "1"  /* Destruction of Pharynx, Percutaneous Endoscopic Approach */
"0C5M7ZZ"  =  "1"  /* Destruction of Pharynx, Via Natural or Artificial Opening */
"0C5M8ZZ"  =  "1"  /* Destruction of Pharynx, Via Natural or Artificial Opening Endoscopic */
"0C5N0ZZ"  =  "1"  /* Destruction of Uvula, Open Approach */
"0C5N3ZZ"  =  "1"  /* Destruction of Uvula, Percutaneous Approach */
"0C5NXZZ"  =  "1"  /* Destruction of Uvula, External Approach */
"0C5P0ZZ"  =  "1"  /* Destruction of Tonsils, Open Approach */
"0C5P3ZZ"  =  "1"  /* Destruction of Tonsils, Percutaneous Approach */
"0C5PXZZ"  =  "1"  /* Destruction of Tonsils, External Approach */
"0C5Q0ZZ"  =  "1"  /* Destruction of Adenoids, Open Approach */
"0C5Q3ZZ"  =  "1"  /* Destruction of Adenoids, Percutaneous Approach */
"0C5QXZZ"  =  "1"  /* Destruction of Adenoids, External Approach */
"0C5R0ZZ"  =  "1"  /* Destruction of Epiglottis, Open Approach */
"0C5R3ZZ"  =  "1"  /* Destruction of Epiglottis, Percutaneous Approach */
"0C5R4ZZ"  =  "1"  /* Destruction of Epiglottis, Percutaneous Endoscopic Approach */
"0C5R7ZZ"  =  "1"  /* Destruction of Epiglottis, Via Natural or Artificial Opening */
"0C5R8ZZ"  =  "1"  /* Destruction of Epiglottis, Via Natural or Artificial Opening Endoscopic */
"0C5S0ZZ"  =  "1"  /* Destruction of Larynx, Open Approach */
"0C5S3ZZ"  =  "1"  /* Destruction of Larynx, Percutaneous Approach */
"0C5S4ZZ"  =  "1"  /* Destruction of Larynx, Percutaneous Endoscopic Approach */
"0C5S7ZZ"  =  "1"  /* Destruction of Larynx, Via Natural or Artificial Opening */
"0C5S8ZZ"  =  "1"  /* Destruction of Larynx, Via Natural or Artificial Opening Endoscopic */
"0C5T0ZZ"  =  "1"  /* Destruction of Right Vocal Cord, Open Approach */
"0C5T3ZZ"  =  "1"  /* Destruction of Right Vocal Cord, Percutaneous Approach */
"0C5T4ZZ"  =  "1"  /* Destruction of Right Vocal Cord, Percutaneous Endoscopic Approach */
"0C5T7ZZ"  =  "1"  /* Destruction of Right Vocal Cord, Via Natural or Artificial Opening */
"0C5T8ZZ"  =  "1"  /* Destruction of Right Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0C5V0ZZ"  =  "1"  /* Destruction of Left Vocal Cord, Open Approach */
"0C5V3ZZ"  =  "1"  /* Destruction of Left Vocal Cord, Percutaneous Approach */
"0C5V4ZZ"  =  "1"  /* Destruction of Left Vocal Cord, Percutaneous Endoscopic Approach */
"0C5V7ZZ"  =  "1"  /* Destruction of Left Vocal Cord, Via Natural or Artificial Opening */
"0C5V8ZZ"  =  "1"  /* Destruction of Left Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0C7M7DZ"  =  "1"  /* Dilation of Pharynx with Intraluminal Device, Via Natural or Artificial Opening */
"0C7M7ZZ"  =  "1"  /* Dilation of Pharynx, Via Natural or Artificial Opening */
"0C7M8DZ"  =  "1"  /* Dilation of Pharynx with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0C7M8ZZ"  =  "1"  /* Dilation of Pharynx, Via Natural or Artificial Opening Endoscopic */
"0C7S0DZ"  =  "1"  /* Dilation of Larynx with Intraluminal Device, Open Approach */
"0C7S0ZZ"  =  "1"  /* Dilation of Larynx, Open Approach */
"0C7S3DZ"  =  "1"  /* Dilation of Larynx with Intraluminal Device, Percutaneous Approach */
"0C7S3ZZ"  =  "1"  /* Dilation of Larynx, Percutaneous Approach */
"0C7S4DZ"  =  "1"  /* Dilation of Larynx with Intraluminal Device, Percutaneous Endoscopic Approach */
"0C7S4ZZ"  =  "1"  /* Dilation of Larynx, Percutaneous Endoscopic Approach */
"0C7S7DZ"  =  "1"  /* Dilation of Larynx with Intraluminal Device, Via Natural or Artificial Opening */
"0C7S7ZZ"  =  "1"  /* Dilation of Larynx, Via Natural or Artificial Opening */
"0C7S8DZ"  =  "1"  /* Dilation of Larynx with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0C7S8ZZ"  =  "1"  /* Dilation of Larynx, Via Natural or Artificial Opening Endoscopic */
"0C9000Z"  =  "1"  /* Drainage of Upper Lip with Drainage Device, Open Approach */
"0C900ZZ"  =  "1"  /* Drainage of Upper Lip, Open Approach */
"0C9030Z"  =  "1"  /* Drainage of Upper Lip with Drainage Device, Percutaneous Approach */
"0C903ZZ"  =  "1"  /* Drainage of Upper Lip, Percutaneous Approach */
"0C90X0Z"  =  "1"  /* Drainage of Upper Lip with Drainage Device, External Approach */
"0C90XZZ"  =  "1"  /* Drainage of Upper Lip, External Approach */
"0C9100Z"  =  "1"  /* Drainage of Lower Lip with Drainage Device, Open Approach */
"0C910ZZ"  =  "1"  /* Drainage of Lower Lip, Open Approach */
"0C9130Z"  =  "1"  /* Drainage of Lower Lip with Drainage Device, Percutaneous Approach */
"0C913ZZ"  =  "1"  /* Drainage of Lower Lip, Percutaneous Approach */
"0C91X0Z"  =  "1"  /* Drainage of Lower Lip with Drainage Device, External Approach */
"0C91XZZ"  =  "1"  /* Drainage of Lower Lip, External Approach */
"0C9200Z"  =  "1"  /* Drainage of Hard Palate with Drainage Device, Open Approach */
"0C920ZZ"  =  "1"  /* Drainage of Hard Palate, Open Approach */
"0C9230Z"  =  "1"  /* Drainage of Hard Palate with Drainage Device, Percutaneous Approach */
"0C923ZZ"  =  "1"  /* Drainage of Hard Palate, Percutaneous Approach */
"0C92X0Z"  =  "1"  /* Drainage of Hard Palate with Drainage Device, External Approach */
"0C92XZZ"  =  "1"  /* Drainage of Hard Palate, External Approach */
"0C9300Z"  =  "1"  /* Drainage of Soft Palate with Drainage Device, Open Approach */
"0C930ZZ"  =  "1"  /* Drainage of Soft Palate, Open Approach */
"0C9330Z"  =  "1"  /* Drainage of Soft Palate with Drainage Device, Percutaneous Approach */
"0C933ZZ"  =  "1"  /* Drainage of Soft Palate, Percutaneous Approach */
"0C93X0Z"  =  "1"  /* Drainage of Soft Palate with Drainage Device, External Approach */
"0C93XZZ"  =  "1"  /* Drainage of Soft Palate, External Approach */
"0C9400Z"  =  "1"  /* Drainage of Buccal Mucosa with Drainage Device, Open Approach */
"0C940ZZ"  =  "1"  /* Drainage of Buccal Mucosa, Open Approach */
"0C9430Z"  =  "1"  /* Drainage of Buccal Mucosa with Drainage Device, Percutaneous Approach */
"0C943ZZ"  =  "1"  /* Drainage of Buccal Mucosa, Percutaneous Approach */
"0C94X0Z"  =  "1"  /* Drainage of Buccal Mucosa with Drainage Device, External Approach */
"0C94XZZ"  =  "1"  /* Drainage of Buccal Mucosa, External Approach */
"0C9700Z"  =  "1"  /* Drainage of Tongue with Drainage Device, Open Approach */
"0C970ZZ"  =  "1"  /* Drainage of Tongue, Open Approach */
"0C9730Z"  =  "1"  /* Drainage of Tongue with Drainage Device, Percutaneous Approach */
"0C973ZZ"  =  "1"  /* Drainage of Tongue, Percutaneous Approach */
"0C97X0Z"  =  "1"  /* Drainage of Tongue with Drainage Device, External Approach */
"0C97XZZ"  =  "1"  /* Drainage of Tongue, External Approach */
"0C980ZZ"  =  "1"  /* Drainage of Right Parotid Gland, Open Approach */
"0C983ZZ"  =  "1"  /* Drainage of Right Parotid Gland, Percutaneous Approach */
"0C990ZZ"  =  "1"  /* Drainage of Left Parotid Gland, Open Approach */
"0C993ZZ"  =  "1"  /* Drainage of Left Parotid Gland, Percutaneous Approach */
"0C9B0ZZ"  =  "1"  /* Drainage of Right Parotid Duct, Open Approach */
"0C9B3ZZ"  =  "1"  /* Drainage of Right Parotid Duct, Percutaneous Approach */
"0C9C0ZZ"  =  "1"  /* Drainage of Left Parotid Duct, Open Approach */
"0C9C3ZZ"  =  "1"  /* Drainage of Left Parotid Duct, Percutaneous Approach */
"0C9D0ZZ"  =  "1"  /* Drainage of Right Sublingual Gland, Open Approach */
"0C9D3ZZ"  =  "1"  /* Drainage of Right Sublingual Gland, Percutaneous Approach */
"0C9F0ZZ"  =  "1"  /* Drainage of Left Sublingual Gland, Open Approach */
"0C9F3ZZ"  =  "1"  /* Drainage of Left Sublingual Gland, Percutaneous Approach */
"0C9G0ZZ"  =  "1"  /* Drainage of Right Submaxillary Gland, Open Approach */
"0C9G3ZZ"  =  "1"  /* Drainage of Right Submaxillary Gland, Percutaneous Approach */
"0C9H0ZZ"  =  "1"  /* Drainage of Left Submaxillary Gland, Open Approach */
"0C9H3ZZ"  =  "1"  /* Drainage of Left Submaxillary Gland, Percutaneous Approach */
"0C9J0ZZ"  =  "1"  /* Drainage of Minor Salivary Gland, Open Approach */
"0C9J3ZZ"  =  "1"  /* Drainage of Minor Salivary Gland, Percutaneous Approach */
"0C9M00Z"  =  "1"  /* Drainage of Pharynx with Drainage Device, Open Approach */
"0C9M0ZZ"  =  "1"  /* Drainage of Pharynx, Open Approach */
"0C9M30Z"  =  "1"  /* Drainage of Pharynx with Drainage Device, Percutaneous Approach */
"0C9M3ZZ"  =  "1"  /* Drainage of Pharynx, Percutaneous Approach */
"0C9M40Z"  =  "1"  /* Drainage of Pharynx with Drainage Device, Percutaneous Endoscopic Approach */
"0C9M4ZZ"  =  "1"  /* Drainage of Pharynx, Percutaneous Endoscopic Approach */
"0C9M70Z"  =  "1"  /* Drainage of Pharynx with Drainage Device, Via Natural or Artificial Opening */
"0C9M7ZZ"  =  "1"  /* Drainage of Pharynx, Via Natural or Artificial Opening */
"0C9M80Z"  =  "1"  /* Drainage of Pharynx with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0C9M8ZZ"  =  "1"  /* Drainage of Pharynx, Via Natural or Artificial Opening Endoscopic */
"0C9N00Z"  =  "1"  /* Drainage of Uvula with Drainage Device, Open Approach */
"0C9N0ZZ"  =  "1"  /* Drainage of Uvula, Open Approach */
"0C9N30Z"  =  "1"  /* Drainage of Uvula with Drainage Device, Percutaneous Approach */
"0C9N3ZZ"  =  "1"  /* Drainage of Uvula, Percutaneous Approach */
"0C9NX0Z"  =  "1"  /* Drainage of Uvula with Drainage Device, External Approach */
"0C9NXZZ"  =  "1"  /* Drainage of Uvula, External Approach */
"0C9P00Z"  =  "1"  /* Drainage of Tonsils with Drainage Device, Open Approach */
"0C9P0ZZ"  =  "1"  /* Drainage of Tonsils, Open Approach */
"0C9P30Z"  =  "1"  /* Drainage of Tonsils with Drainage Device, Percutaneous Approach */
"0C9P3ZZ"  =  "1"  /* Drainage of Tonsils, Percutaneous Approach */
"0C9PX0Z"  =  "1"  /* Drainage of Tonsils with Drainage Device, External Approach */
"0C9PXZZ"  =  "1"  /* Drainage of Tonsils, External Approach */
"0C9Q00Z"  =  "1"  /* Drainage of Adenoids with Drainage Device, Open Approach */
"0C9Q0ZZ"  =  "1"  /* Drainage of Adenoids, Open Approach */
"0C9Q30Z"  =  "1"  /* Drainage of Adenoids with Drainage Device, Percutaneous Approach */
"0C9Q3ZZ"  =  "1"  /* Drainage of Adenoids, Percutaneous Approach */
"0C9QX0Z"  =  "1"  /* Drainage of Adenoids with Drainage Device, External Approach */
"0C9QXZZ"  =  "1"  /* Drainage of Adenoids, External Approach */
"0C9R00Z"  =  "1"  /* Drainage of Epiglottis with Drainage Device, Open Approach */
"0C9R0ZZ"  =  "1"  /* Drainage of Epiglottis, Open Approach */
"0C9R30Z"  =  "1"  /* Drainage of Epiglottis with Drainage Device, Percutaneous Approach */
"0C9R3ZZ"  =  "1"  /* Drainage of Epiglottis, Percutaneous Approach */
"0C9R40Z"  =  "1"  /* Drainage of Epiglottis with Drainage Device, Percutaneous Endoscopic Approach */
"0C9R4ZZ"  =  "1"  /* Drainage of Epiglottis, Percutaneous Endoscopic Approach */
"0C9R70Z"  =  "1"  /* Drainage of Epiglottis with Drainage Device, Via Natural or Artificial Opening */
"0C9R7ZZ"  =  "1"  /* Drainage of Epiglottis, Via Natural or Artificial Opening */
"0C9R80Z"  =  "1"  /* Drainage of Epiglottis with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0C9R8ZZ"  =  "1"  /* Drainage of Epiglottis, Via Natural or Artificial Opening Endoscopic */
"0C9S00Z"  =  "1"  /* Drainage of Larynx with Drainage Device, Open Approach */
"0C9S0ZZ"  =  "1"  /* Drainage of Larynx, Open Approach */
"0C9S30Z"  =  "1"  /* Drainage of Larynx with Drainage Device, Percutaneous Approach */
"0C9S3ZZ"  =  "1"  /* Drainage of Larynx, Percutaneous Approach */
"0C9S40Z"  =  "1"  /* Drainage of Larynx with Drainage Device, Percutaneous Endoscopic Approach */
"0C9S4ZZ"  =  "1"  /* Drainage of Larynx, Percutaneous Endoscopic Approach */
"0C9S70Z"  =  "1"  /* Drainage of Larynx with Drainage Device, Via Natural or Artificial Opening */
"0C9S7ZZ"  =  "1"  /* Drainage of Larynx, Via Natural or Artificial Opening */
"0C9S80Z"  =  "1"  /* Drainage of Larynx with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0C9S8ZZ"  =  "1"  /* Drainage of Larynx, Via Natural or Artificial Opening Endoscopic */
"0C9T00Z"  =  "1"  /* Drainage of Right Vocal Cord with Drainage Device, Open Approach */
"0C9T0ZZ"  =  "1"  /* Drainage of Right Vocal Cord, Open Approach */
"0C9T30Z"  =  "1"  /* Drainage of Right Vocal Cord with Drainage Device, Percutaneous Approach */
"0C9T3ZZ"  =  "1"  /* Drainage of Right Vocal Cord, Percutaneous Approach */
"0C9T40Z"  =  "1"  /* Drainage of Right Vocal Cord with Drainage Device, Percutaneous Endoscopic Approach */
"0C9T4ZZ"  =  "1"  /* Drainage of Right Vocal Cord, Percutaneous Endoscopic Approach */
"0C9T70Z"  =  "1"  /* Drainage of Right Vocal Cord with Drainage Device, Via Natural or Artificial Opening */
"0C9T7ZZ"  =  "1"  /* Drainage of Right Vocal Cord, Via Natural or Artificial Opening */
"0C9T80Z"  =  "1"  /* Drainage of Right Vocal Cord with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0C9T8ZZ"  =  "1"  /* Drainage of Right Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0C9V00Z"  =  "1"  /* Drainage of Left Vocal Cord with Drainage Device, Open Approach */
"0C9V0ZZ"  =  "1"  /* Drainage of Left Vocal Cord, Open Approach */
"0C9V30Z"  =  "1"  /* Drainage of Left Vocal Cord with Drainage Device, Percutaneous Approach */
"0C9V3ZZ"  =  "1"  /* Drainage of Left Vocal Cord, Percutaneous Approach */
"0C9V40Z"  =  "1"  /* Drainage of Left Vocal Cord with Drainage Device, Percutaneous Endoscopic Approach */
"0C9V4ZZ"  =  "1"  /* Drainage of Left Vocal Cord, Percutaneous Endoscopic Approach */
"0C9V70Z"  =  "1"  /* Drainage of Left Vocal Cord with Drainage Device, Via Natural or Artificial Opening */
"0C9V7ZZ"  =  "1"  /* Drainage of Left Vocal Cord, Via Natural or Artificial Opening */
"0C9V80Z"  =  "1"  /* Drainage of Left Vocal Cord with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0C9V8ZZ"  =  "1"  /* Drainage of Left Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CB00ZZ"  =  "1"  /* Excision of Upper Lip, Open Approach */
"0CB03ZZ"  =  "1"  /* Excision of Upper Lip, Percutaneous Approach */
"0CB0XZZ"  =  "1"  /* Excision of Upper Lip, External Approach */
"0CB10ZZ"  =  "1"  /* Excision of Lower Lip, Open Approach */
"0CB13ZZ"  =  "1"  /* Excision of Lower Lip, Percutaneous Approach */
"0CB1XZZ"  =  "1"  /* Excision of Lower Lip, External Approach */
"0CB20ZZ"  =  "1"  /* Excision of Hard Palate, Open Approach */
"0CB23ZZ"  =  "1"  /* Excision of Hard Palate, Percutaneous Approach */
"0CB2XZZ"  =  "1"  /* Excision of Hard Palate, External Approach */
"0CB30ZZ"  =  "1"  /* Excision of Soft Palate, Open Approach */
"0CB33ZZ"  =  "1"  /* Excision of Soft Palate, Percutaneous Approach */
"0CB3XZZ"  =  "1"  /* Excision of Soft Palate, External Approach */
"0CB40ZZ"  =  "1"  /* Excision of Buccal Mucosa, Open Approach */
"0CB43ZZ"  =  "1"  /* Excision of Buccal Mucosa, Percutaneous Approach */
"0CB4XZZ"  =  "1"  /* Excision of Buccal Mucosa, External Approach */
"0CB70ZZ"  =  "1"  /* Excision of Tongue, Open Approach */
"0CB73ZZ"  =  "1"  /* Excision of Tongue, Percutaneous Approach */
"0CB7XZZ"  =  "1"  /* Excision of Tongue, External Approach */
"0CB80ZZ"  =  "1"  /* Excision of Right Parotid Gland, Open Approach */
"0CB83ZZ"  =  "1"  /* Excision of Right Parotid Gland, Percutaneous Approach */
"0CB90ZZ"  =  "1"  /* Excision of Left Parotid Gland, Open Approach */
"0CB93ZZ"  =  "1"  /* Excision of Left Parotid Gland, Percutaneous Approach */
"0CBB0ZZ"  =  "1"  /* Excision of Right Parotid Duct, Open Approach */
"0CBB3ZZ"  =  "1"  /* Excision of Right Parotid Duct, Percutaneous Approach */
"0CBC0ZZ"  =  "1"  /* Excision of Left Parotid Duct, Open Approach */
"0CBC3ZZ"  =  "1"  /* Excision of Left Parotid Duct, Percutaneous Approach */
"0CBD0ZZ"  =  "1"  /* Excision of Right Sublingual Gland, Open Approach */
"0CBD3ZZ"  =  "1"  /* Excision of Right Sublingual Gland, Percutaneous Approach */
"0CBF0ZZ"  =  "1"  /* Excision of Left Sublingual Gland, Open Approach */
"0CBF3ZZ"  =  "1"  /* Excision of Left Sublingual Gland, Percutaneous Approach */
"0CBG0ZZ"  =  "1"  /* Excision of Right Submaxillary Gland, Open Approach */
"0CBG3ZZ"  =  "1"  /* Excision of Right Submaxillary Gland, Percutaneous Approach */
"0CBH0ZZ"  =  "1"  /* Excision of Left Submaxillary Gland, Open Approach */
"0CBH3ZZ"  =  "1"  /* Excision of Left Submaxillary Gland, Percutaneous Approach */
"0CBJ0ZZ"  =  "1"  /* Excision of Minor Salivary Gland, Open Approach */
"0CBJ3ZZ"  =  "1"  /* Excision of Minor Salivary Gland, Percutaneous Approach */
"0CBM0ZZ"  =  "1"  /* Excision of Pharynx, Open Approach */
"0CBM3ZZ"  =  "1"  /* Excision of Pharynx, Percutaneous Approach */
"0CBM4ZZ"  =  "1"  /* Excision of Pharynx, Percutaneous Endoscopic Approach */
"0CBM7ZZ"  =  "1"  /* Excision of Pharynx, Via Natural or Artificial Opening */
"0CBM8ZZ"  =  "1"  /* Excision of Pharynx, Via Natural or Artificial Opening Endoscopic */
"0CBN0ZZ"  =  "1"  /* Excision of Uvula, Open Approach */
"0CBN3ZZ"  =  "1"  /* Excision of Uvula, Percutaneous Approach */
"0CBNXZZ"  =  "1"  /* Excision of Uvula, External Approach */
"0CBP0ZZ"  =  "1"  /* Excision of Tonsils, Open Approach */
"0CBP3ZZ"  =  "1"  /* Excision of Tonsils, Percutaneous Approach */
"0CBPXZZ"  =  "1"  /* Excision of Tonsils, External Approach */
"0CBQ0ZZ"  =  "1"  /* Excision of Adenoids, Open Approach */
"0CBQ3ZZ"  =  "1"  /* Excision of Adenoids, Percutaneous Approach */
"0CBQXZZ"  =  "1"  /* Excision of Adenoids, External Approach */
"0CBR0ZZ"  =  "1"  /* Excision of Epiglottis, Open Approach */
"0CBR3ZZ"  =  "1"  /* Excision of Epiglottis, Percutaneous Approach */
"0CBR4ZZ"  =  "1"  /* Excision of Epiglottis, Percutaneous Endoscopic Approach */
"0CBR7ZZ"  =  "1"  /* Excision of Epiglottis, Via Natural or Artificial Opening */
"0CBR8ZZ"  =  "1"  /* Excision of Epiglottis, Via Natural or Artificial Opening Endoscopic */
"0CBS0ZZ"  =  "1"  /* Excision of Larynx, Open Approach */
"0CBS3ZZ"  =  "1"  /* Excision of Larynx, Percutaneous Approach */
"0CBS4ZZ"  =  "1"  /* Excision of Larynx, Percutaneous Endoscopic Approach */
"0CBS7ZZ"  =  "1"  /* Excision of Larynx, Via Natural or Artificial Opening */
"0CBS8ZZ"  =  "1"  /* Excision of Larynx, Via Natural or Artificial Opening Endoscopic */
"0CBT0ZZ"  =  "1"  /* Excision of Right Vocal Cord, Open Approach */
"0CBT3ZZ"  =  "1"  /* Excision of Right Vocal Cord, Percutaneous Approach */
"0CBT4ZZ"  =  "1"  /* Excision of Right Vocal Cord, Percutaneous Endoscopic Approach */
"0CBT7ZZ"  =  "1"  /* Excision of Right Vocal Cord, Via Natural or Artificial Opening */
"0CBT8ZZ"  =  "1"  /* Excision of Right Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CBV0ZZ"  =  "1"  /* Excision of Left Vocal Cord, Open Approach */
"0CBV3ZZ"  =  "1"  /* Excision of Left Vocal Cord, Percutaneous Approach */
"0CBV4ZZ"  =  "1"  /* Excision of Left Vocal Cord, Percutaneous Endoscopic Approach */
"0CBV7ZZ"  =  "1"  /* Excision of Left Vocal Cord, Via Natural or Artificial Opening */
"0CBV8ZZ"  =  "1"  /* Excision of Left Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CC00ZZ"  =  "1"  /* Extirpation of Matter from Upper Lip, Open Approach */
"0CC03ZZ"  =  "1"  /* Extirpation of Matter from Upper Lip, Percutaneous Approach */
"0CC10ZZ"  =  "1"  /* Extirpation of Matter from Lower Lip, Open Approach */
"0CC13ZZ"  =  "1"  /* Extirpation of Matter from Lower Lip, Percutaneous Approach */
"0CC20ZZ"  =  "1"  /* Extirpation of Matter from Hard Palate, Open Approach */
"0CC23ZZ"  =  "1"  /* Extirpation of Matter from Hard Palate, Percutaneous Approach */
"0CC30ZZ"  =  "1"  /* Extirpation of Matter from Soft Palate, Open Approach */
"0CC33ZZ"  =  "1"  /* Extirpation of Matter from Soft Palate, Percutaneous Approach */
"0CC40ZZ"  =  "1"  /* Extirpation of Matter from Buccal Mucosa, Open Approach */
"0CC43ZZ"  =  "1"  /* Extirpation of Matter from Buccal Mucosa, Percutaneous Approach */
"0CC70ZZ"  =  "1"  /* Extirpation of Matter from Tongue, Open Approach */
"0CC73ZZ"  =  "1"  /* Extirpation of Matter from Tongue, Percutaneous Approach */
"0CCM0ZZ"  =  "1"  /* Extirpation of Matter from Pharynx, Open Approach */
"0CCM3ZZ"  =  "1"  /* Extirpation of Matter from Pharynx, Percutaneous Approach */
"0CCM4ZZ"  =  "1"  /* Extirpation of Matter from Pharynx, Percutaneous Endoscopic Approach */
"0CCN0ZZ"  =  "1"  /* Extirpation of Matter from Uvula, Open Approach */
"0CCN3ZZ"  =  "1"  /* Extirpation of Matter from Uvula, Percutaneous Approach */
"0CCP0ZZ"  =  "1"  /* Extirpation of Matter from Tonsils, Open Approach */
"0CCP3ZZ"  =  "1"  /* Extirpation of Matter from Tonsils, Percutaneous Approach */
"0CCQ0ZZ"  =  "1"  /* Extirpation of Matter from Adenoids, Open Approach */
"0CCQ3ZZ"  =  "1"  /* Extirpation of Matter from Adenoids, Percutaneous Approach */
"0CCR0ZZ"  =  "1"  /* Extirpation of Matter from Epiglottis, Open Approach */
"0CCR3ZZ"  =  "1"  /* Extirpation of Matter from Epiglottis, Percutaneous Approach */
"0CCR4ZZ"  =  "1"  /* Extirpation of Matter from Epiglottis, Percutaneous Endoscopic Approach */
"0CCR7ZZ"  =  "1"  /* Extirpation of Matter from Epiglottis, Via Natural or Artificial Opening */
"0CCR8ZZ"  =  "1"  /* Extirpation of Matter from Epiglottis, Via Natural or Artificial Opening Endoscopic */
"0CCS0ZZ"  =  "1"  /* Extirpation of Matter from Larynx, Open Approach */
"0CCS3ZZ"  =  "1"  /* Extirpation of Matter from Larynx, Percutaneous Approach */
"0CCS4ZZ"  =  "1"  /* Extirpation of Matter from Larynx, Percutaneous Endoscopic Approach */
"0CCT0ZZ"  =  "1"  /* Extirpation of Matter from Right Vocal Cord, Open Approach */
"0CCT3ZZ"  =  "1"  /* Extirpation of Matter from Right Vocal Cord, Percutaneous Approach */
"0CCT4ZZ"  =  "1"  /* Extirpation of Matter from Right Vocal Cord, Percutaneous Endoscopic Approach */
"0CCT7ZZ"  =  "1"  /* Extirpation of Matter from Right Vocal Cord, Via Natural or Artificial Opening */
"0CCT8ZZ"  =  "1"  /* Extirpation of Matter from Right Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CCV0ZZ"  =  "1"  /* Extirpation of Matter from Left Vocal Cord, Open Approach */
"0CCV3ZZ"  =  "1"  /* Extirpation of Matter from Left Vocal Cord, Percutaneous Approach */
"0CCV4ZZ"  =  "1"  /* Extirpation of Matter from Left Vocal Cord, Percutaneous Endoscopic Approach */
"0CCV7ZZ"  =  "1"  /* Extirpation of Matter from Left Vocal Cord, Via Natural or Artificial Opening */
"0CCV8ZZ"  =  "1"  /* Extirpation of Matter from Left Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CDT0ZZ"  =  "1"  /* Extraction of Right Vocal Cord, Open Approach */
"0CDT3ZZ"  =  "1"  /* Extraction of Right Vocal Cord, Percutaneous Approach */
"0CDT4ZZ"  =  "1"  /* Extraction of Right Vocal Cord, Percutaneous Endoscopic Approach */
"0CDT7ZZ"  =  "1"  /* Extraction of Right Vocal Cord, Via Natural or Artificial Opening */
"0CDT8ZZ"  =  "1"  /* Extraction of Right Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CDV0ZZ"  =  "1"  /* Extraction of Left Vocal Cord, Open Approach */
"0CDV3ZZ"  =  "1"  /* Extraction of Left Vocal Cord, Percutaneous Approach */
"0CDV4ZZ"  =  "1"  /* Extraction of Left Vocal Cord, Percutaneous Endoscopic Approach */
"0CDV7ZZ"  =  "1"  /* Extraction of Left Vocal Cord, Via Natural or Artificial Opening */
"0CDV8ZZ"  =  "1"  /* Extraction of Left Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CHY7BZ"  =  "1"  /* Insertion of Airway into Mouth and Throat, Via Natural or Artificial Opening */
"0CHY8BZ"  =  "1"  /* Insertion of Airway into Mouth and Throat, Via Natural or Artificial Opening Endoscopic */
"0CLB0CZ"  =  "1"  /* Occlusion of Right Parotid Duct with Extraluminal Device, Open Approach */
"0CLB0DZ"  =  "1"  /* Occlusion of Right Parotid Duct with Intraluminal Device, Open Approach */
"0CLB0ZZ"  =  "1"  /* Occlusion of Right Parotid Duct, Open Approach */
"0CLB3CZ"  =  "1"  /* Occlusion of Right Parotid Duct with Extraluminal Device, Percutaneous Approach */
"0CLB3DZ"  =  "1"  /* Occlusion of Right Parotid Duct with Intraluminal Device, Percutaneous Approach */
"0CLB3ZZ"  =  "1"  /* Occlusion of Right Parotid Duct, Percutaneous Approach */
"0CLB4CZ"  =  "1"  /* Occlusion of Right Parotid Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0CLB4DZ"  =  "1"  /* Occlusion of Right Parotid Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0CLB4ZZ"  =  "1"  /* Occlusion of Right Parotid Duct, Percutaneous Endoscopic Approach */
"0CLB7DZ"  =  "1"  /* Occlusion of Right Parotid Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0CLB7ZZ"  =  "1"  /* Occlusion of Right Parotid Duct, Via Natural or Artificial Opening */
"0CLB8DZ"  =  "1"  /* Occlusion of Right Parotid Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0CLB8ZZ"  =  "1"  /* Occlusion of Right Parotid Duct, Via Natural or Artificial Opening Endoscopic */
"0CLC0CZ"  =  "1"  /* Occlusion of Left Parotid Duct with Extraluminal Device, Open Approach */
"0CLC0DZ"  =  "1"  /* Occlusion of Left Parotid Duct with Intraluminal Device, Open Approach */
"0CLC0ZZ"  =  "1"  /* Occlusion of Left Parotid Duct, Open Approach */
"0CLC3CZ"  =  "1"  /* Occlusion of Left Parotid Duct with Extraluminal Device, Percutaneous Approach */
"0CLC3DZ"  =  "1"  /* Occlusion of Left Parotid Duct with Intraluminal Device, Percutaneous Approach */
"0CLC3ZZ"  =  "1"  /* Occlusion of Left Parotid Duct, Percutaneous Approach */
"0CLC4CZ"  =  "1"  /* Occlusion of Left Parotid Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0CLC4DZ"  =  "1"  /* Occlusion of Left Parotid Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0CLC4ZZ"  =  "1"  /* Occlusion of Left Parotid Duct, Percutaneous Endoscopic Approach */
"0CLC7DZ"  =  "1"  /* Occlusion of Left Parotid Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0CLC7ZZ"  =  "1"  /* Occlusion of Left Parotid Duct, Via Natural or Artificial Opening */
"0CLC8DZ"  =  "1"  /* Occlusion of Left Parotid Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0CLC8ZZ"  =  "1"  /* Occlusion of Left Parotid Duct, Via Natural or Artificial Opening Endoscopic */
"0CM00ZZ"  =  "1"  /* Reattachment of Upper Lip, Open Approach */
"0CM10ZZ"  =  "1"  /* Reattachment of Lower Lip, Open Approach */
"0CM30ZZ"  =  "1"  /* Reattachment of Soft Palate, Open Approach */
"0CM70ZZ"  =  "1"  /* Reattachment of Tongue, Open Approach */
"0CMN0ZZ"  =  "1"  /* Reattachment of Uvula, Open Approach */
"0CN20ZZ"  =  "1"  /* Release Hard Palate, Open Approach */
"0CN23ZZ"  =  "1"  /* Release Hard Palate, Percutaneous Approach */
"0CN2XZZ"  =  "1"  /* Release Hard Palate, External Approach */
"0CN30ZZ"  =  "1"  /* Release Soft Palate, Open Approach */
"0CN33ZZ"  =  "1"  /* Release Soft Palate, Percutaneous Approach */
"0CN3XZZ"  =  "1"  /* Release Soft Palate, External Approach */
"0CN40ZZ"  =  "1"  /* Release Buccal Mucosa, Open Approach */
"0CN43ZZ"  =  "1"  /* Release Buccal Mucosa, Percutaneous Approach */
"0CN4XZZ"  =  "1"  /* Release Buccal Mucosa, External Approach */
"0CN80ZZ"  =  "1"  /* Release Right Parotid Gland, Open Approach */
"0CN83ZZ"  =  "1"  /* Release Right Parotid Gland, Percutaneous Approach */
"0CN90ZZ"  =  "1"  /* Release Left Parotid Gland, Open Approach */
"0CN93ZZ"  =  "1"  /* Release Left Parotid Gland, Percutaneous Approach */
"0CNB0ZZ"  =  "1"  /* Release Right Parotid Duct, Open Approach */
"0CNB3ZZ"  =  "1"  /* Release Right Parotid Duct, Percutaneous Approach */
"0CNC0ZZ"  =  "1"  /* Release Left Parotid Duct, Open Approach */
"0CNC3ZZ"  =  "1"  /* Release Left Parotid Duct, Percutaneous Approach */
"0CND0ZZ"  =  "1"  /* Release Right Sublingual Gland, Open Approach */
"0CND3ZZ"  =  "1"  /* Release Right Sublingual Gland, Percutaneous Approach */
"0CNF0ZZ"  =  "1"  /* Release Left Sublingual Gland, Open Approach */
"0CNF3ZZ"  =  "1"  /* Release Left Sublingual Gland, Percutaneous Approach */
"0CNG0ZZ"  =  "1"  /* Release Right Submaxillary Gland, Open Approach */
"0CNG3ZZ"  =  "1"  /* Release Right Submaxillary Gland, Percutaneous Approach */
"0CNH0ZZ"  =  "1"  /* Release Left Submaxillary Gland, Open Approach */
"0CNH3ZZ"  =  "1"  /* Release Left Submaxillary Gland, Percutaneous Approach */
"0CNJ0ZZ"  =  "1"  /* Release Minor Salivary Gland, Open Approach */
"0CNJ3ZZ"  =  "1"  /* Release Minor Salivary Gland, Percutaneous Approach */
"0CNM0ZZ"  =  "1"  /* Release Pharynx, Open Approach */
"0CNM3ZZ"  =  "1"  /* Release Pharynx, Percutaneous Approach */
"0CNM4ZZ"  =  "1"  /* Release Pharynx, Percutaneous Endoscopic Approach */
"0CNM7ZZ"  =  "1"  /* Release Pharynx, Via Natural or Artificial Opening */
"0CNM8ZZ"  =  "1"  /* Release Pharynx, Via Natural or Artificial Opening Endoscopic */
"0CNN0ZZ"  =  "1"  /* Release Uvula, Open Approach */
"0CNN3ZZ"  =  "1"  /* Release Uvula, Percutaneous Approach */
"0CNNXZZ"  =  "1"  /* Release Uvula, External Approach */
"0CNP0ZZ"  =  "1"  /* Release Tonsils, Open Approach */
"0CNP3ZZ"  =  "1"  /* Release Tonsils, Percutaneous Approach */
"0CNPXZZ"  =  "1"  /* Release Tonsils, External Approach */
"0CNQ0ZZ"  =  "1"  /* Release Adenoids, Open Approach */
"0CNQ3ZZ"  =  "1"  /* Release Adenoids, Percutaneous Approach */
"0CNQXZZ"  =  "1"  /* Release Adenoids, External Approach */
"0CNR0ZZ"  =  "1"  /* Release Epiglottis, Open Approach */
"0CNR3ZZ"  =  "1"  /* Release Epiglottis, Percutaneous Approach */
"0CNR4ZZ"  =  "1"  /* Release Epiglottis, Percutaneous Endoscopic Approach */
"0CNR7ZZ"  =  "1"  /* Release Epiglottis, Via Natural or Artificial Opening */
"0CNR8ZZ"  =  "1"  /* Release Epiglottis, Via Natural or Artificial Opening Endoscopic */
"0CNS0ZZ"  =  "1"  /* Release Larynx, Open Approach */
"0CNS3ZZ"  =  "1"  /* Release Larynx, Percutaneous Approach */
"0CNS4ZZ"  =  "1"  /* Release Larynx, Percutaneous Endoscopic Approach */
"0CNS7ZZ"  =  "1"  /* Release Larynx, Via Natural or Artificial Opening */
"0CNS8ZZ"  =  "1"  /* Release Larynx, Via Natural or Artificial Opening Endoscopic */
"0CNT0ZZ"  =  "1"  /* Release Right Vocal Cord, Open Approach */
"0CNT3ZZ"  =  "1"  /* Release Right Vocal Cord, Percutaneous Approach */
"0CNT4ZZ"  =  "1"  /* Release Right Vocal Cord, Percutaneous Endoscopic Approach */
"0CNT7ZZ"  =  "1"  /* Release Right Vocal Cord, Via Natural or Artificial Opening */
"0CNT8ZZ"  =  "1"  /* Release Right Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CNV0ZZ"  =  "1"  /* Release Left Vocal Cord, Open Approach */
"0CNV3ZZ"  =  "1"  /* Release Left Vocal Cord, Percutaneous Approach */
"0CNV4ZZ"  =  "1"  /* Release Left Vocal Cord, Percutaneous Endoscopic Approach */
"0CNV7ZZ"  =  "1"  /* Release Left Vocal Cord, Via Natural or Artificial Opening */
"0CNV8ZZ"  =  "1"  /* Release Left Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CPS00Z"  =  "1"  /* Removal of Drainage Device from Larynx, Open Approach */
"0CPS07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Larynx, Open Approach */
"0CPS0DZ"  =  "1"  /* Removal of Intraluminal Device from Larynx, Open Approach */
"0CPS0JZ"  =  "1"  /* Removal of Synthetic Substitute from Larynx, Open Approach */
"0CPS0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Larynx, Open Approach */
"0CPS30Z"  =  "1"  /* Removal of Drainage Device from Larynx, Percutaneous Approach */
"0CPS37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Larynx, Percutaneous Approach */
"0CPS3DZ"  =  "1"  /* Removal of Intraluminal Device from Larynx, Percutaneous Approach */
"0CPS3JZ"  =  "1"  /* Removal of Synthetic Substitute from Larynx, Percutaneous Approach */
"0CPS3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Larynx, Percutaneous Approach */
"0CPS70Z"  =  "1"  /* Removal of Drainage Device from Larynx, Via Natural or Artificial Opening */
"0CPS77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Larynx, Via Natural or Artificial Opening */
"0CPS7DZ"  =  "1"  /* Removal of Intraluminal Device from Larynx, Via Natural or Artificial Opening */
"0CPS7JZ"  =  "1"  /* Removal of Synthetic Substitute from Larynx, Via Natural or Artificial Opening */
"0CPS7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Larynx, Via Natural or Artificial Opening */
"0CPS80Z"  =  "1"  /* Removal of Drainage Device from Larynx, Via Natural or Artificial Opening Endoscopic */
"0CPS87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Larynx, Via Natural or Artificial Opening Endoscopic */
"0CPS8DZ"  =  "1"  /* Removal of Intraluminal Device from Larynx, Via Natural or Artificial Opening Endoscopic */
"0CPS8JZ"  =  "1"  /* Removal of Synthetic Substitute from Larynx, Via Natural or Artificial Opening Endoscopic */
"0CPS8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Larynx, Via Natural or Artificial Opening Endoscopic */
"0CPY00Z"  =  "1"  /* Removal of Drainage Device from Mouth and Throat, Open Approach */
"0CPY01Z"  =  "1"  /* Removal of Radioactive Element from Mouth and Throat, Open Approach */
"0CPY07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mouth and Throat, Open Approach */
"0CPY0DZ"  =  "1"  /* Removal of Intraluminal Device from Mouth and Throat, Open Approach */
"0CPY0JZ"  =  "1"  /* Removal of Synthetic Substitute from Mouth and Throat, Open Approach */
"0CPY0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mouth and Throat, Open Approach */
"0CPY30Z"  =  "1"  /* Removal of Drainage Device from Mouth and Throat, Percutaneous Approach */
"0CPY31Z"  =  "1"  /* Removal of Radioactive Element from Mouth and Throat, Percutaneous Approach */
"0CPY37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mouth and Throat, Percutaneous Approach */
"0CPY3DZ"  =  "1"  /* Removal of Intraluminal Device from Mouth and Throat, Percutaneous Approach */
"0CPY3JZ"  =  "1"  /* Removal of Synthetic Substitute from Mouth and Throat, Percutaneous Approach */
"0CPY3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mouth and Throat, Percutaneous Approach */
"0CPY71Z"  =  "1"  /* Removal of Radioactive Element from Mouth and Throat, Via Natural or Artificial Opening */
"0CPY77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mouth and Throat, Via Natural or Artificial Opening */
"0CPY7JZ"  =  "1"  /* Removal of Synthetic Substitute from Mouth and Throat, Via Natural or Artificial Opening */
"0CPY7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mouth and Throat, Via Natural or Artificial Opening */
"0CPY81Z"  =  "1"  /* Removal of Radioactive Element from Mouth and Throat, Via Natural or Artificial Opening Endoscopic */
"0CPY87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mouth and Throat, Via Natural or Artificial Opening Endoscopic */
"0CPY8JZ"  =  "1"  /* Removal of Synthetic Substitute from Mouth and Throat, Via Natural or Artificial Opening Endoscopic */
"0CPY8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mouth and Throat, Via Natural or Artificial Opening Endoscopic */
"0CQ00ZZ"  =  "1"  /* Repair Upper Lip, Open Approach */
"0CQ03ZZ"  =  "1"  /* Repair Upper Lip, Percutaneous Approach */
"0CQ0XZZ"  =  "1"  /* Repair Upper Lip, External Approach */
"0CQ10ZZ"  =  "1"  /* Repair Lower Lip, Open Approach */
"0CQ13ZZ"  =  "1"  /* Repair Lower Lip, Percutaneous Approach */
"0CQ1XZZ"  =  "1"  /* Repair Lower Lip, External Approach */
"0CQ20ZZ"  =  "1"  /* Repair Hard Palate, Open Approach */
"0CQ23ZZ"  =  "1"  /* Repair Hard Palate, Percutaneous Approach */
"0CQ2XZZ"  =  "1"  /* Repair Hard Palate, External Approach */
"0CQ30ZZ"  =  "1"  /* Repair Soft Palate, Open Approach */
"0CQ33ZZ"  =  "1"  /* Repair Soft Palate, Percutaneous Approach */
"0CQ3XZZ"  =  "1"  /* Repair Soft Palate, External Approach */
"0CQ40ZZ"  =  "1"  /* Repair Buccal Mucosa, Open Approach */
"0CQ43ZZ"  =  "1"  /* Repair Buccal Mucosa, Percutaneous Approach */
"0CQ4XZZ"  =  "1"  /* Repair Buccal Mucosa, External Approach */
"0CQ50ZZ"  =  "1"  /* Repair Upper Gingiva, Open Approach */
"0CQ53ZZ"  =  "1"  /* Repair Upper Gingiva, Percutaneous Approach */
"0CQ5XZZ"  =  "1"  /* Repair Upper Gingiva, External Approach */
"0CQ70ZZ"  =  "1"  /* Repair Tongue, Open Approach */
"0CQ73ZZ"  =  "1"  /* Repair Tongue, Percutaneous Approach */
"0CQ7XZZ"  =  "1"  /* Repair Tongue, External Approach */
"0CQ80ZZ"  =  "1"  /* Repair Right Parotid Gland, Open Approach */
"0CQ83ZZ"  =  "1"  /* Repair Right Parotid Gland, Percutaneous Approach */
"0CQ90ZZ"  =  "1"  /* Repair Left Parotid Gland, Open Approach */
"0CQ93ZZ"  =  "1"  /* Repair Left Parotid Gland, Percutaneous Approach */
"0CQB0ZZ"  =  "1"  /* Repair Right Parotid Duct, Open Approach */
"0CQB3ZZ"  =  "1"  /* Repair Right Parotid Duct, Percutaneous Approach */
"0CQC0ZZ"  =  "1"  /* Repair Left Parotid Duct, Open Approach */
"0CQC3ZZ"  =  "1"  /* Repair Left Parotid Duct, Percutaneous Approach */
"0CQD0ZZ"  =  "1"  /* Repair Right Sublingual Gland, Open Approach */
"0CQD3ZZ"  =  "1"  /* Repair Right Sublingual Gland, Percutaneous Approach */
"0CQF0ZZ"  =  "1"  /* Repair Left Sublingual Gland, Open Approach */
"0CQF3ZZ"  =  "1"  /* Repair Left Sublingual Gland, Percutaneous Approach */
"0CQG0ZZ"  =  "1"  /* Repair Right Submaxillary Gland, Open Approach */
"0CQG3ZZ"  =  "1"  /* Repair Right Submaxillary Gland, Percutaneous Approach */
"0CQH0ZZ"  =  "1"  /* Repair Left Submaxillary Gland, Open Approach */
"0CQH3ZZ"  =  "1"  /* Repair Left Submaxillary Gland, Percutaneous Approach */
"0CQJ0ZZ"  =  "1"  /* Repair Minor Salivary Gland, Open Approach */
"0CQJ3ZZ"  =  "1"  /* Repair Minor Salivary Gland, Percutaneous Approach */
"0CQM0ZZ"  =  "1"  /* Repair Pharynx, Open Approach */
"0CQM3ZZ"  =  "1"  /* Repair Pharynx, Percutaneous Approach */
"0CQM4ZZ"  =  "1"  /* Repair Pharynx, Percutaneous Endoscopic Approach */
"0CQM7ZZ"  =  "1"  /* Repair Pharynx, Via Natural or Artificial Opening */
"0CQM8ZZ"  =  "1"  /* Repair Pharynx, Via Natural or Artificial Opening Endoscopic */
"0CQN0ZZ"  =  "1"  /* Repair Uvula, Open Approach */
"0CQN3ZZ"  =  "1"  /* Repair Uvula, Percutaneous Approach */
"0CQNXZZ"  =  "1"  /* Repair Uvula, External Approach */
"0CQP0ZZ"  =  "1"  /* Repair Tonsils, Open Approach */
"0CQP3ZZ"  =  "1"  /* Repair Tonsils, Percutaneous Approach */
"0CQPXZZ"  =  "1"  /* Repair Tonsils, External Approach */
"0CQQ0ZZ"  =  "1"  /* Repair Adenoids, Open Approach */
"0CQQ3ZZ"  =  "1"  /* Repair Adenoids, Percutaneous Approach */
"0CQQXZZ"  =  "1"  /* Repair Adenoids, External Approach */
"0CQR0ZZ"  =  "1"  /* Repair Epiglottis, Open Approach */
"0CQR3ZZ"  =  "1"  /* Repair Epiglottis, Percutaneous Approach */
"0CQR4ZZ"  =  "1"  /* Repair Epiglottis, Percutaneous Endoscopic Approach */
"0CQR7ZZ"  =  "1"  /* Repair Epiglottis, Via Natural or Artificial Opening */
"0CQR8ZZ"  =  "1"  /* Repair Epiglottis, Via Natural or Artificial Opening Endoscopic */
"0CQS0ZZ"  =  "1"  /* Repair Larynx, Open Approach */
"0CQS3ZZ"  =  "1"  /* Repair Larynx, Percutaneous Approach */
"0CQS4ZZ"  =  "1"  /* Repair Larynx, Percutaneous Endoscopic Approach */
"0CQS7ZZ"  =  "1"  /* Repair Larynx, Via Natural or Artificial Opening */
"0CQS8ZZ"  =  "1"  /* Repair Larynx, Via Natural or Artificial Opening Endoscopic */
"0CQT0ZZ"  =  "1"  /* Repair Right Vocal Cord, Open Approach */
"0CQT3ZZ"  =  "1"  /* Repair Right Vocal Cord, Percutaneous Approach */
"0CQT4ZZ"  =  "1"  /* Repair Right Vocal Cord, Percutaneous Endoscopic Approach */
"0CQT7ZZ"  =  "1"  /* Repair Right Vocal Cord, Via Natural or Artificial Opening */
"0CQT8ZZ"  =  "1"  /* Repair Right Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CQV0ZZ"  =  "1"  /* Repair Left Vocal Cord, Open Approach */
"0CQV3ZZ"  =  "1"  /* Repair Left Vocal Cord, Percutaneous Approach */
"0CQV4ZZ"  =  "1"  /* Repair Left Vocal Cord, Percutaneous Endoscopic Approach */
"0CQV7ZZ"  =  "1"  /* Repair Left Vocal Cord, Via Natural or Artificial Opening */
"0CQV8ZZ"  =  "1"  /* Repair Left Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CR007Z"  =  "1"  /* Replacement of Upper Lip with Autologous Tissue Substitute, Open Approach */
"0CR00JZ"  =  "1"  /* Replacement of Upper Lip with Synthetic Substitute, Open Approach */
"0CR00KZ"  =  "1"  /* Replacement of Upper Lip with Nonautologous Tissue Substitute, Open Approach */
"0CR037Z"  =  "1"  /* Replacement of Upper Lip with Autologous Tissue Substitute, Percutaneous Approach */
"0CR03JZ"  =  "1"  /* Replacement of Upper Lip with Synthetic Substitute, Percutaneous Approach */
"0CR03KZ"  =  "1"  /* Replacement of Upper Lip with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CR0X7Z"  =  "1"  /* Replacement of Upper Lip with Autologous Tissue Substitute, External Approach */
"0CR0XJZ"  =  "1"  /* Replacement of Upper Lip with Synthetic Substitute, External Approach */
"0CR0XKZ"  =  "1"  /* Replacement of Upper Lip with Nonautologous Tissue Substitute, External Approach */
"0CR107Z"  =  "1"  /* Replacement of Lower Lip with Autologous Tissue Substitute, Open Approach */
"0CR10JZ"  =  "1"  /* Replacement of Lower Lip with Synthetic Substitute, Open Approach */
"0CR10KZ"  =  "1"  /* Replacement of Lower Lip with Nonautologous Tissue Substitute, Open Approach */
"0CR137Z"  =  "1"  /* Replacement of Lower Lip with Autologous Tissue Substitute, Percutaneous Approach */
"0CR13JZ"  =  "1"  /* Replacement of Lower Lip with Synthetic Substitute, Percutaneous Approach */
"0CR13KZ"  =  "1"  /* Replacement of Lower Lip with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CR1X7Z"  =  "1"  /* Replacement of Lower Lip with Autologous Tissue Substitute, External Approach */
"0CR1XJZ"  =  "1"  /* Replacement of Lower Lip with Synthetic Substitute, External Approach */
"0CR1XKZ"  =  "1"  /* Replacement of Lower Lip with Nonautologous Tissue Substitute, External Approach */
"0CR207Z"  =  "1"  /* Replacement of Hard Palate with Autologous Tissue Substitute, Open Approach */
"0CR20JZ"  =  "1"  /* Replacement of Hard Palate with Synthetic Substitute, Open Approach */
"0CR20KZ"  =  "1"  /* Replacement of Hard Palate with Nonautologous Tissue Substitute, Open Approach */
"0CR237Z"  =  "1"  /* Replacement of Hard Palate with Autologous Tissue Substitute, Percutaneous Approach */
"0CR23JZ"  =  "1"  /* Replacement of Hard Palate with Synthetic Substitute, Percutaneous Approach */
"0CR23KZ"  =  "1"  /* Replacement of Hard Palate with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CR2X7Z"  =  "1"  /* Replacement of Hard Palate with Autologous Tissue Substitute, External Approach */
"0CR2XJZ"  =  "1"  /* Replacement of Hard Palate with Synthetic Substitute, External Approach */
"0CR2XKZ"  =  "1"  /* Replacement of Hard Palate with Nonautologous Tissue Substitute, External Approach */
"0CR307Z"  =  "1"  /* Replacement of Soft Palate with Autologous Tissue Substitute, Open Approach */
"0CR30JZ"  =  "1"  /* Replacement of Soft Palate with Synthetic Substitute, Open Approach */
"0CR30KZ"  =  "1"  /* Replacement of Soft Palate with Nonautologous Tissue Substitute, Open Approach */
"0CR337Z"  =  "1"  /* Replacement of Soft Palate with Autologous Tissue Substitute, Percutaneous Approach */
"0CR33JZ"  =  "1"  /* Replacement of Soft Palate with Synthetic Substitute, Percutaneous Approach */
"0CR33KZ"  =  "1"  /* Replacement of Soft Palate with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CR3X7Z"  =  "1"  /* Replacement of Soft Palate with Autologous Tissue Substitute, External Approach */
"0CR3XJZ"  =  "1"  /* Replacement of Soft Palate with Synthetic Substitute, External Approach */
"0CR3XKZ"  =  "1"  /* Replacement of Soft Palate with Nonautologous Tissue Substitute, External Approach */
"0CR407Z"  =  "1"  /* Replacement of Buccal Mucosa with Autologous Tissue Substitute, Open Approach */
"0CR40JZ"  =  "1"  /* Replacement of Buccal Mucosa with Synthetic Substitute, Open Approach */
"0CR40KZ"  =  "1"  /* Replacement of Buccal Mucosa with Nonautologous Tissue Substitute, Open Approach */
"0CR437Z"  =  "1"  /* Replacement of Buccal Mucosa with Autologous Tissue Substitute, Percutaneous Approach */
"0CR43JZ"  =  "1"  /* Replacement of Buccal Mucosa with Synthetic Substitute, Percutaneous Approach */
"0CR43KZ"  =  "1"  /* Replacement of Buccal Mucosa with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CR4X7Z"  =  "1"  /* Replacement of Buccal Mucosa with Autologous Tissue Substitute, External Approach */
"0CR4XJZ"  =  "1"  /* Replacement of Buccal Mucosa with Synthetic Substitute, External Approach */
"0CR4XKZ"  =  "1"  /* Replacement of Buccal Mucosa with Nonautologous Tissue Substitute, External Approach */
"0CR507Z"  =  "1"  /* Replacement of Upper Gingiva with Autologous Tissue Substitute, Open Approach */
"0CR50JZ"  =  "1"  /* Replacement of Upper Gingiva with Synthetic Substitute, Open Approach */
"0CR50KZ"  =  "1"  /* Replacement of Upper Gingiva with Nonautologous Tissue Substitute, Open Approach */
"0CR537Z"  =  "1"  /* Replacement of Upper Gingiva with Autologous Tissue Substitute, Percutaneous Approach */
"0CR53JZ"  =  "1"  /* Replacement of Upper Gingiva with Synthetic Substitute, Percutaneous Approach */
"0CR53KZ"  =  "1"  /* Replacement of Upper Gingiva with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CR5X7Z"  =  "1"  /* Replacement of Upper Gingiva with Autologous Tissue Substitute, External Approach */
"0CR5XJZ"  =  "1"  /* Replacement of Upper Gingiva with Synthetic Substitute, External Approach */
"0CR5XKZ"  =  "1"  /* Replacement of Upper Gingiva with Nonautologous Tissue Substitute, External Approach */
"0CR607Z"  =  "1"  /* Replacement of Lower Gingiva with Autologous Tissue Substitute, Open Approach */
"0CR60JZ"  =  "1"  /* Replacement of Lower Gingiva with Synthetic Substitute, Open Approach */
"0CR60KZ"  =  "1"  /* Replacement of Lower Gingiva with Nonautologous Tissue Substitute, Open Approach */
"0CR637Z"  =  "1"  /* Replacement of Lower Gingiva with Autologous Tissue Substitute, Percutaneous Approach */
"0CR63JZ"  =  "1"  /* Replacement of Lower Gingiva with Synthetic Substitute, Percutaneous Approach */
"0CR63KZ"  =  "1"  /* Replacement of Lower Gingiva with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CR6X7Z"  =  "1"  /* Replacement of Lower Gingiva with Autologous Tissue Substitute, External Approach */
"0CR6XJZ"  =  "1"  /* Replacement of Lower Gingiva with Synthetic Substitute, External Approach */
"0CR6XKZ"  =  "1"  /* Replacement of Lower Gingiva with Nonautologous Tissue Substitute, External Approach */
"0CR707Z"  =  "1"  /* Replacement of Tongue with Autologous Tissue Substitute, Open Approach */
"0CR70JZ"  =  "1"  /* Replacement of Tongue with Synthetic Substitute, Open Approach */
"0CR70KZ"  =  "1"  /* Replacement of Tongue with Nonautologous Tissue Substitute, Open Approach */
"0CR737Z"  =  "1"  /* Replacement of Tongue with Autologous Tissue Substitute, Percutaneous Approach */
"0CR73JZ"  =  "1"  /* Replacement of Tongue with Synthetic Substitute, Percutaneous Approach */
"0CR73KZ"  =  "1"  /* Replacement of Tongue with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CR7X7Z"  =  "1"  /* Replacement of Tongue with Autologous Tissue Substitute, External Approach */
"0CR7XJZ"  =  "1"  /* Replacement of Tongue with Synthetic Substitute, External Approach */
"0CR7XKZ"  =  "1"  /* Replacement of Tongue with Nonautologous Tissue Substitute, External Approach */
"0CRB07Z"  =  "1"  /* Replacement of Right Parotid Duct with Autologous Tissue Substitute, Open Approach */
"0CRB0JZ"  =  "1"  /* Replacement of Right Parotid Duct with Synthetic Substitute, Open Approach */
"0CRB0KZ"  =  "1"  /* Replacement of Right Parotid Duct with Nonautologous Tissue Substitute, Open Approach */
"0CRB37Z"  =  "1"  /* Replacement of Right Parotid Duct with Autologous Tissue Substitute, Percutaneous Approach */
"0CRB3JZ"  =  "1"  /* Replacement of Right Parotid Duct with Synthetic Substitute, Percutaneous Approach */
"0CRB3KZ"  =  "1"  /* Replacement of Right Parotid Duct with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CRC07Z"  =  "1"  /* Replacement of Left Parotid Duct with Autologous Tissue Substitute, Open Approach */
"0CRC0JZ"  =  "1"  /* Replacement of Left Parotid Duct with Synthetic Substitute, Open Approach */
"0CRC0KZ"  =  "1"  /* Replacement of Left Parotid Duct with Nonautologous Tissue Substitute, Open Approach */
"0CRC37Z"  =  "1"  /* Replacement of Left Parotid Duct with Autologous Tissue Substitute, Percutaneous Approach */
"0CRC3JZ"  =  "1"  /* Replacement of Left Parotid Duct with Synthetic Substitute, Percutaneous Approach */
"0CRC3KZ"  =  "1"  /* Replacement of Left Parotid Duct with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CRM07Z"  =  "1"  /* Replacement of Pharynx with Autologous Tissue Substitute, Open Approach */
"0CRM0JZ"  =  "1"  /* Replacement of Pharynx with Synthetic Substitute, Open Approach */
"0CRM0KZ"  =  "1"  /* Replacement of Pharynx with Nonautologous Tissue Substitute, Open Approach */
"0CRM77Z"  =  "1"  /* Replacement of Pharynx with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0CRM7JZ"  =  "1"  /* Replacement of Pharynx with Synthetic Substitute, Via Natural or Artificial Opening */
"0CRM7KZ"  =  "1"  /* Replacement of Pharynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0CRM87Z"  =  "1"  /* Replacement of Pharynx with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CRM8JZ"  =  "1"  /* Replacement of Pharynx with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0CRM8KZ"  =  "1"  /* Replacement of Pharynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CRN07Z"  =  "1"  /* Replacement of Uvula with Autologous Tissue Substitute, Open Approach */
"0CRN0JZ"  =  "1"  /* Replacement of Uvula with Synthetic Substitute, Open Approach */
"0CRN0KZ"  =  "1"  /* Replacement of Uvula with Nonautologous Tissue Substitute, Open Approach */
"0CRN37Z"  =  "1"  /* Replacement of Uvula with Autologous Tissue Substitute, Percutaneous Approach */
"0CRN3JZ"  =  "1"  /* Replacement of Uvula with Synthetic Substitute, Percutaneous Approach */
"0CRN3KZ"  =  "1"  /* Replacement of Uvula with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CRNX7Z"  =  "1"  /* Replacement of Uvula with Autologous Tissue Substitute, External Approach */
"0CRNXJZ"  =  "1"  /* Replacement of Uvula with Synthetic Substitute, External Approach */
"0CRNXKZ"  =  "1"  /* Replacement of Uvula with Nonautologous Tissue Substitute, External Approach */
"0CRR07Z"  =  "1"  /* Replacement of Epiglottis with Autologous Tissue Substitute, Open Approach */
"0CRR0JZ"  =  "1"  /* Replacement of Epiglottis with Synthetic Substitute, Open Approach */
"0CRR0KZ"  =  "1"  /* Replacement of Epiglottis with Nonautologous Tissue Substitute, Open Approach */
"0CRR77Z"  =  "1"  /* Replacement of Epiglottis with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0CRR7JZ"  =  "1"  /* Replacement of Epiglottis with Synthetic Substitute, Via Natural or Artificial Opening */
"0CRR7KZ"  =  "1"  /* Replacement of Epiglottis with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0CRR87Z"  =  "1"  /* Replacement of Epiglottis with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CRR8JZ"  =  "1"  /* Replacement of Epiglottis with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0CRR8KZ"  =  "1"  /* Replacement of Epiglottis with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CRS07Z"  =  "1"  /* Replacement of Larynx with Autologous Tissue Substitute, Open Approach */
"0CRS0JZ"  =  "1"  /* Replacement of Larynx with Synthetic Substitute, Open Approach */
"0CRS0KZ"  =  "1"  /* Replacement of Larynx with Nonautologous Tissue Substitute, Open Approach */
"0CRS77Z"  =  "1"  /* Replacement of Larynx with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0CRS7JZ"  =  "1"  /* Replacement of Larynx with Synthetic Substitute, Via Natural or Artificial Opening */
"0CRS7KZ"  =  "1"  /* Replacement of Larynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0CRS87Z"  =  "1"  /* Replacement of Larynx with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CRS8JZ"  =  "1"  /* Replacement of Larynx with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0CRS8KZ"  =  "1"  /* Replacement of Larynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CRT07Z"  =  "1"  /* Replacement of Right Vocal Cord with Autologous Tissue Substitute, Open Approach */
"0CRT0JZ"  =  "1"  /* Replacement of Right Vocal Cord with Synthetic Substitute, Open Approach */
"0CRT0KZ"  =  "1"  /* Replacement of Right Vocal Cord with Nonautologous Tissue Substitute, Open Approach */
"0CRT77Z"  =  "1"  /* Replacement of Right Vocal Cord with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0CRT7JZ"  =  "1"  /* Replacement of Right Vocal Cord with Synthetic Substitute, Via Natural or Artificial Opening */
"0CRT7KZ"  =  "1"  /* Replacement of Right Vocal Cord with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0CRT87Z"  =  "1"  /* Replacement of Right Vocal Cord with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CRT8JZ"  =  "1"  /* Replacement of Right Vocal Cord with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0CRT8KZ"  =  "1"  /* Replacement of Right Vocal Cord with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CRV07Z"  =  "1"  /* Replacement of Left Vocal Cord with Autologous Tissue Substitute, Open Approach */
"0CRV0JZ"  =  "1"  /* Replacement of Left Vocal Cord with Synthetic Substitute, Open Approach */
"0CRV0KZ"  =  "1"  /* Replacement of Left Vocal Cord with Nonautologous Tissue Substitute, Open Approach */
"0CRV77Z"  =  "1"  /* Replacement of Left Vocal Cord with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0CRV7JZ"  =  "1"  /* Replacement of Left Vocal Cord with Synthetic Substitute, Via Natural or Artificial Opening */
"0CRV7KZ"  =  "1"  /* Replacement of Left Vocal Cord with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0CRV87Z"  =  "1"  /* Replacement of Left Vocal Cord with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CRV8JZ"  =  "1"  /* Replacement of Left Vocal Cord with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0CRV8KZ"  =  "1"  /* Replacement of Left Vocal Cord with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CS00ZZ"  =  "1"  /* Reposition Upper Lip, Open Approach */
"0CS0XZZ"  =  "1"  /* Reposition Upper Lip, External Approach */
"0CS10ZZ"  =  "1"  /* Reposition Lower Lip, Open Approach */
"0CS1XZZ"  =  "1"  /* Reposition Lower Lip, External Approach */
"0CS20ZZ"  =  "1"  /* Reposition Hard Palate, Open Approach */
"0CS2XZZ"  =  "1"  /* Reposition Hard Palate, External Approach */
"0CS30ZZ"  =  "1"  /* Reposition Soft Palate, Open Approach */
"0CS3XZZ"  =  "1"  /* Reposition Soft Palate, External Approach */
"0CS70ZZ"  =  "1"  /* Reposition Tongue, Open Approach */
"0CS7XZZ"  =  "1"  /* Reposition Tongue, External Approach */
"0CSB0ZZ"  =  "1"  /* Reposition Right Parotid Duct, Open Approach */
"0CSB3ZZ"  =  "1"  /* Reposition Right Parotid Duct, Percutaneous Approach */
"0CSC0ZZ"  =  "1"  /* Reposition Left Parotid Duct, Open Approach */
"0CSC3ZZ"  =  "1"  /* Reposition Left Parotid Duct, Percutaneous Approach */
"0CSN0ZZ"  =  "1"  /* Reposition Uvula, Open Approach */
"0CSNXZZ"  =  "1"  /* Reposition Uvula, External Approach */
"0CSR0ZZ"  =  "1"  /* Reposition Epiglottis, Open Approach */
"0CSR7ZZ"  =  "1"  /* Reposition Epiglottis, Via Natural or Artificial Opening */
"0CSR8ZZ"  =  "1"  /* Reposition Epiglottis, Via Natural or Artificial Opening Endoscopic */
"0CST0ZZ"  =  "1"  /* Reposition Right Vocal Cord, Open Approach */
"0CST7ZZ"  =  "1"  /* Reposition Right Vocal Cord, Via Natural or Artificial Opening */
"0CST8ZZ"  =  "1"  /* Reposition Right Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CSV0ZZ"  =  "1"  /* Reposition Left Vocal Cord, Open Approach */
"0CSV7ZZ"  =  "1"  /* Reposition Left Vocal Cord, Via Natural or Artificial Opening */
"0CSV8ZZ"  =  "1"  /* Reposition Left Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CT00ZZ"  =  "1"  /* Resection of Upper Lip, Open Approach */
"0CT0XZZ"  =  "1"  /* Resection of Upper Lip, External Approach */
"0CT10ZZ"  =  "1"  /* Resection of Lower Lip, Open Approach */
"0CT1XZZ"  =  "1"  /* Resection of Lower Lip, External Approach */
"0CT20ZZ"  =  "1"  /* Resection of Hard Palate, Open Approach */
"0CT2XZZ"  =  "1"  /* Resection of Hard Palate, External Approach */
"0CT30ZZ"  =  "1"  /* Resection of Soft Palate, Open Approach */
"0CT3XZZ"  =  "1"  /* Resection of Soft Palate, External Approach */
"0CT70ZZ"  =  "1"  /* Resection of Tongue, Open Approach */
"0CT7XZZ"  =  "1"  /* Resection of Tongue, External Approach */
"0CT80ZZ"  =  "1"  /* Resection of Right Parotid Gland, Open Approach */
"0CT90ZZ"  =  "1"  /* Resection of Left Parotid Gland, Open Approach */
"0CTB0ZZ"  =  "1"  /* Resection of Right Parotid Duct, Open Approach */
"0CTC0ZZ"  =  "1"  /* Resection of Left Parotid Duct, Open Approach */
"0CTD0ZZ"  =  "1"  /* Resection of Right Sublingual Gland, Open Approach */
"0CTF0ZZ"  =  "1"  /* Resection of Left Sublingual Gland, Open Approach */
"0CTG0ZZ"  =  "1"  /* Resection of Right Submaxillary Gland, Open Approach */
"0CTH0ZZ"  =  "1"  /* Resection of Left Submaxillary Gland, Open Approach */
"0CTJ0ZZ"  =  "1"  /* Resection of Minor Salivary Gland, Open Approach */
"0CTM0ZZ"  =  "1"  /* Resection of Pharynx, Open Approach */
"0CTM4ZZ"  =  "1"  /* Resection of Pharynx, Percutaneous Endoscopic Approach */
"0CTM7ZZ"  =  "1"  /* Resection of Pharynx, Via Natural or Artificial Opening */
"0CTM8ZZ"  =  "1"  /* Resection of Pharynx, Via Natural or Artificial Opening Endoscopic */
"0CTN0ZZ"  =  "1"  /* Resection of Uvula, Open Approach */
"0CTNXZZ"  =  "1"  /* Resection of Uvula, External Approach */
"0CTR0ZZ"  =  "1"  /* Resection of Epiglottis, Open Approach */
"0CTR4ZZ"  =  "1"  /* Resection of Epiglottis, Percutaneous Endoscopic Approach */
"0CTR7ZZ"  =  "1"  /* Resection of Epiglottis, Via Natural or Artificial Opening */
"0CTR8ZZ"  =  "1"  /* Resection of Epiglottis, Via Natural or Artificial Opening Endoscopic */
"0CTS0ZZ"  =  "1"  /* Resection of Larynx, Open Approach */
"0CTS4ZZ"  =  "1"  /* Resection of Larynx, Percutaneous Endoscopic Approach */
"0CTS7ZZ"  =  "1"  /* Resection of Larynx, Via Natural or Artificial Opening */
"0CTS8ZZ"  =  "1"  /* Resection of Larynx, Via Natural or Artificial Opening Endoscopic */
"0CTT0ZZ"  =  "1"  /* Resection of Right Vocal Cord, Open Approach */
"0CTT4ZZ"  =  "1"  /* Resection of Right Vocal Cord, Percutaneous Endoscopic Approach */
"0CTT7ZZ"  =  "1"  /* Resection of Right Vocal Cord, Via Natural or Artificial Opening */
"0CTT8ZZ"  =  "1"  /* Resection of Right Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CTV0ZZ"  =  "1"  /* Resection of Left Vocal Cord, Open Approach */
"0CTV4ZZ"  =  "1"  /* Resection of Left Vocal Cord, Percutaneous Endoscopic Approach */
"0CTV7ZZ"  =  "1"  /* Resection of Left Vocal Cord, Via Natural or Artificial Opening */
"0CTV8ZZ"  =  "1"  /* Resection of Left Vocal Cord, Via Natural or Artificial Opening Endoscopic */
"0CU007Z"  =  "1"  /* Supplement Upper Lip with Autologous Tissue Substitute, Open Approach */
"0CU00JZ"  =  "1"  /* Supplement Upper Lip with Synthetic Substitute, Open Approach */
"0CU00KZ"  =  "1"  /* Supplement Upper Lip with Nonautologous Tissue Substitute, Open Approach */
"0CU037Z"  =  "1"  /* Supplement Upper Lip with Autologous Tissue Substitute, Percutaneous Approach */
"0CU03JZ"  =  "1"  /* Supplement Upper Lip with Synthetic Substitute, Percutaneous Approach */
"0CU03KZ"  =  "1"  /* Supplement Upper Lip with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CU0X7Z"  =  "1"  /* Supplement Upper Lip with Autologous Tissue Substitute, External Approach */
"0CU0XJZ"  =  "1"  /* Supplement Upper Lip with Synthetic Substitute, External Approach */
"0CU0XKZ"  =  "1"  /* Supplement Upper Lip with Nonautologous Tissue Substitute, External Approach */
"0CU107Z"  =  "1"  /* Supplement Lower Lip with Autologous Tissue Substitute, Open Approach */
"0CU10JZ"  =  "1"  /* Supplement Lower Lip with Synthetic Substitute, Open Approach */
"0CU10KZ"  =  "1"  /* Supplement Lower Lip with Nonautologous Tissue Substitute, Open Approach */
"0CU137Z"  =  "1"  /* Supplement Lower Lip with Autologous Tissue Substitute, Percutaneous Approach */
"0CU13JZ"  =  "1"  /* Supplement Lower Lip with Synthetic Substitute, Percutaneous Approach */
"0CU13KZ"  =  "1"  /* Supplement Lower Lip with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CU1X7Z"  =  "1"  /* Supplement Lower Lip with Autologous Tissue Substitute, External Approach */
"0CU1XJZ"  =  "1"  /* Supplement Lower Lip with Synthetic Substitute, External Approach */
"0CU1XKZ"  =  "1"  /* Supplement Lower Lip with Nonautologous Tissue Substitute, External Approach */
"0CU207Z"  =  "1"  /* Supplement Hard Palate with Autologous Tissue Substitute, Open Approach */
"0CU20JZ"  =  "1"  /* Supplement Hard Palate with Synthetic Substitute, Open Approach */
"0CU20KZ"  =  "1"  /* Supplement Hard Palate with Nonautologous Tissue Substitute, Open Approach */
"0CU237Z"  =  "1"  /* Supplement Hard Palate with Autologous Tissue Substitute, Percutaneous Approach */
"0CU23JZ"  =  "1"  /* Supplement Hard Palate with Synthetic Substitute, Percutaneous Approach */
"0CU23KZ"  =  "1"  /* Supplement Hard Palate with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CU2X7Z"  =  "1"  /* Supplement Hard Palate with Autologous Tissue Substitute, External Approach */
"0CU2XJZ"  =  "1"  /* Supplement Hard Palate with Synthetic Substitute, External Approach */
"0CU2XKZ"  =  "1"  /* Supplement Hard Palate with Nonautologous Tissue Substitute, External Approach */
"0CU307Z"  =  "1"  /* Supplement Soft Palate with Autologous Tissue Substitute, Open Approach */
"0CU30JZ"  =  "1"  /* Supplement Soft Palate with Synthetic Substitute, Open Approach */
"0CU30KZ"  =  "1"  /* Supplement Soft Palate with Nonautologous Tissue Substitute, Open Approach */
"0CU337Z"  =  "1"  /* Supplement Soft Palate with Autologous Tissue Substitute, Percutaneous Approach */
"0CU33JZ"  =  "1"  /* Supplement Soft Palate with Synthetic Substitute, Percutaneous Approach */
"0CU33KZ"  =  "1"  /* Supplement Soft Palate with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CU3X7Z"  =  "1"  /* Supplement Soft Palate with Autologous Tissue Substitute, External Approach */
"0CU3XJZ"  =  "1"  /* Supplement Soft Palate with Synthetic Substitute, External Approach */
"0CU3XKZ"  =  "1"  /* Supplement Soft Palate with Nonautologous Tissue Substitute, External Approach */
"0CU407Z"  =  "1"  /* Supplement Buccal Mucosa with Autologous Tissue Substitute, Open Approach */
"0CU40JZ"  =  "1"  /* Supplement Buccal Mucosa with Synthetic Substitute, Open Approach */
"0CU40KZ"  =  "1"  /* Supplement Buccal Mucosa with Nonautologous Tissue Substitute, Open Approach */
"0CU437Z"  =  "1"  /* Supplement Buccal Mucosa with Autologous Tissue Substitute, Percutaneous Approach */
"0CU43JZ"  =  "1"  /* Supplement Buccal Mucosa with Synthetic Substitute, Percutaneous Approach */
"0CU43KZ"  =  "1"  /* Supplement Buccal Mucosa with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CU4X7Z"  =  "1"  /* Supplement Buccal Mucosa with Autologous Tissue Substitute, External Approach */
"0CU4XJZ"  =  "1"  /* Supplement Buccal Mucosa with Synthetic Substitute, External Approach */
"0CU4XKZ"  =  "1"  /* Supplement Buccal Mucosa with Nonautologous Tissue Substitute, External Approach */
"0CU507Z"  =  "1"  /* Supplement Upper Gingiva with Autologous Tissue Substitute, Open Approach */
"0CU50JZ"  =  "1"  /* Supplement Upper Gingiva with Synthetic Substitute, Open Approach */
"0CU50KZ"  =  "1"  /* Supplement Upper Gingiva with Nonautologous Tissue Substitute, Open Approach */
"0CU537Z"  =  "1"  /* Supplement Upper Gingiva with Autologous Tissue Substitute, Percutaneous Approach */
"0CU53JZ"  =  "1"  /* Supplement Upper Gingiva with Synthetic Substitute, Percutaneous Approach */
"0CU53KZ"  =  "1"  /* Supplement Upper Gingiva with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CU5X7Z"  =  "1"  /* Supplement Upper Gingiva with Autologous Tissue Substitute, External Approach */
"0CU5XJZ"  =  "1"  /* Supplement Upper Gingiva with Synthetic Substitute, External Approach */
"0CU5XKZ"  =  "1"  /* Supplement Upper Gingiva with Nonautologous Tissue Substitute, External Approach */
"0CU607Z"  =  "1"  /* Supplement Lower Gingiva with Autologous Tissue Substitute, Open Approach */
"0CU60JZ"  =  "1"  /* Supplement Lower Gingiva with Synthetic Substitute, Open Approach */
"0CU60KZ"  =  "1"  /* Supplement Lower Gingiva with Nonautologous Tissue Substitute, Open Approach */
"0CU637Z"  =  "1"  /* Supplement Lower Gingiva with Autologous Tissue Substitute, Percutaneous Approach */
"0CU63JZ"  =  "1"  /* Supplement Lower Gingiva with Synthetic Substitute, Percutaneous Approach */
"0CU63KZ"  =  "1"  /* Supplement Lower Gingiva with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CU6X7Z"  =  "1"  /* Supplement Lower Gingiva with Autologous Tissue Substitute, External Approach */
"0CU6XJZ"  =  "1"  /* Supplement Lower Gingiva with Synthetic Substitute, External Approach */
"0CU6XKZ"  =  "1"  /* Supplement Lower Gingiva with Nonautologous Tissue Substitute, External Approach */
"0CU707Z"  =  "1"  /* Supplement Tongue with Autologous Tissue Substitute, Open Approach */
"0CU70JZ"  =  "1"  /* Supplement Tongue with Synthetic Substitute, Open Approach */
"0CU70KZ"  =  "1"  /* Supplement Tongue with Nonautologous Tissue Substitute, Open Approach */
"0CU737Z"  =  "1"  /* Supplement Tongue with Autologous Tissue Substitute, Percutaneous Approach */
"0CU73JZ"  =  "1"  /* Supplement Tongue with Synthetic Substitute, Percutaneous Approach */
"0CU73KZ"  =  "1"  /* Supplement Tongue with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CU7X7Z"  =  "1"  /* Supplement Tongue with Autologous Tissue Substitute, External Approach */
"0CU7XJZ"  =  "1"  /* Supplement Tongue with Synthetic Substitute, External Approach */
"0CU7XKZ"  =  "1"  /* Supplement Tongue with Nonautologous Tissue Substitute, External Approach */
"0CUM07Z"  =  "1"  /* Supplement Pharynx with Autologous Tissue Substitute, Open Approach */
"0CUM0JZ"  =  "1"  /* Supplement Pharynx with Synthetic Substitute, Open Approach */
"0CUM0KZ"  =  "1"  /* Supplement Pharynx with Nonautologous Tissue Substitute, Open Approach */
"0CUM77Z"  =  "1"  /* Supplement Pharynx with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0CUM7JZ"  =  "1"  /* Supplement Pharynx with Synthetic Substitute, Via Natural or Artificial Opening */
"0CUM7KZ"  =  "1"  /* Supplement Pharynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0CUM87Z"  =  "1"  /* Supplement Pharynx with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CUM8JZ"  =  "1"  /* Supplement Pharynx with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0CUM8KZ"  =  "1"  /* Supplement Pharynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CUN07Z"  =  "1"  /* Supplement Uvula with Autologous Tissue Substitute, Open Approach */
"0CUN0JZ"  =  "1"  /* Supplement Uvula with Synthetic Substitute, Open Approach */
"0CUN0KZ"  =  "1"  /* Supplement Uvula with Nonautologous Tissue Substitute, Open Approach */
"0CUN37Z"  =  "1"  /* Supplement Uvula with Autologous Tissue Substitute, Percutaneous Approach */
"0CUN3JZ"  =  "1"  /* Supplement Uvula with Synthetic Substitute, Percutaneous Approach */
"0CUN3KZ"  =  "1"  /* Supplement Uvula with Nonautologous Tissue Substitute, Percutaneous Approach */
"0CUNX7Z"  =  "1"  /* Supplement Uvula with Autologous Tissue Substitute, External Approach */
"0CUNXJZ"  =  "1"  /* Supplement Uvula with Synthetic Substitute, External Approach */
"0CUNXKZ"  =  "1"  /* Supplement Uvula with Nonautologous Tissue Substitute, External Approach */
"0CUR07Z"  =  "1"  /* Supplement Epiglottis with Autologous Tissue Substitute, Open Approach */
"0CUR0JZ"  =  "1"  /* Supplement Epiglottis with Synthetic Substitute, Open Approach */
"0CUR0KZ"  =  "1"  /* Supplement Epiglottis with Nonautologous Tissue Substitute, Open Approach */
"0CUR77Z"  =  "1"  /* Supplement Epiglottis with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0CUR7JZ"  =  "1"  /* Supplement Epiglottis with Synthetic Substitute, Via Natural or Artificial Opening */
"0CUR7KZ"  =  "1"  /* Supplement Epiglottis with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0CUR87Z"  =  "1"  /* Supplement Epiglottis with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CUR8JZ"  =  "1"  /* Supplement Epiglottis with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0CUR8KZ"  =  "1"  /* Supplement Epiglottis with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CUS07Z"  =  "1"  /* Supplement Larynx with Autologous Tissue Substitute, Open Approach */
"0CUS0JZ"  =  "1"  /* Supplement Larynx with Synthetic Substitute, Open Approach */
"0CUS0KZ"  =  "1"  /* Supplement Larynx with Nonautologous Tissue Substitute, Open Approach */
"0CUS77Z"  =  "1"  /* Supplement Larynx with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0CUS7JZ"  =  "1"  /* Supplement Larynx with Synthetic Substitute, Via Natural or Artificial Opening */
"0CUS7KZ"  =  "1"  /* Supplement Larynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0CUS87Z"  =  "1"  /* Supplement Larynx with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CUS8JZ"  =  "1"  /* Supplement Larynx with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0CUS8KZ"  =  "1"  /* Supplement Larynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CUT07Z"  =  "1"  /* Supplement Right Vocal Cord with Autologous Tissue Substitute, Open Approach */
"0CUT0JZ"  =  "1"  /* Supplement Right Vocal Cord with Synthetic Substitute, Open Approach */
"0CUT0KZ"  =  "1"  /* Supplement Right Vocal Cord with Nonautologous Tissue Substitute, Open Approach */
"0CUT77Z"  =  "1"  /* Supplement Right Vocal Cord with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0CUT7JZ"  =  "1"  /* Supplement Right Vocal Cord with Synthetic Substitute, Via Natural or Artificial Opening */
"0CUT7KZ"  =  "1"  /* Supplement Right Vocal Cord with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0CUT87Z"  =  "1"  /* Supplement Right Vocal Cord with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CUT8JZ"  =  "1"  /* Supplement Right Vocal Cord with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0CUT8KZ"  =  "1"  /* Supplement Right Vocal Cord with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CUV07Z"  =  "1"  /* Supplement Left Vocal Cord with Autologous Tissue Substitute, Open Approach */
"0CUV0JZ"  =  "1"  /* Supplement Left Vocal Cord with Synthetic Substitute, Open Approach */
"0CUV0KZ"  =  "1"  /* Supplement Left Vocal Cord with Nonautologous Tissue Substitute, Open Approach */
"0CUV77Z"  =  "1"  /* Supplement Left Vocal Cord with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0CUV7JZ"  =  "1"  /* Supplement Left Vocal Cord with Synthetic Substitute, Via Natural or Artificial Opening */
"0CUV7KZ"  =  "1"  /* Supplement Left Vocal Cord with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0CUV87Z"  =  "1"  /* Supplement Left Vocal Cord with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CUV8JZ"  =  "1"  /* Supplement Left Vocal Cord with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0CUV8KZ"  =  "1"  /* Supplement Left Vocal Cord with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0CVB0CZ"  =  "1"  /* Restriction of Right Parotid Duct with Extraluminal Device, Open Approach */
"0CVB0DZ"  =  "1"  /* Restriction of Right Parotid Duct with Intraluminal Device, Open Approach */
"0CVB0ZZ"  =  "1"  /* Restriction of Right Parotid Duct, Open Approach */
"0CVB3CZ"  =  "1"  /* Restriction of Right Parotid Duct with Extraluminal Device, Percutaneous Approach */
"0CVB3DZ"  =  "1"  /* Restriction of Right Parotid Duct with Intraluminal Device, Percutaneous Approach */
"0CVB3ZZ"  =  "1"  /* Restriction of Right Parotid Duct, Percutaneous Approach */
"0CVB7DZ"  =  "1"  /* Restriction of Right Parotid Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0CVB7ZZ"  =  "1"  /* Restriction of Right Parotid Duct, Via Natural or Artificial Opening */
"0CVB8DZ"  =  "1"  /* Restriction of Right Parotid Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0CVB8ZZ"  =  "1"  /* Restriction of Right Parotid Duct, Via Natural or Artificial Opening Endoscopic */
"0CVC0CZ"  =  "1"  /* Restriction of Left Parotid Duct with Extraluminal Device, Open Approach */
"0CVC0DZ"  =  "1"  /* Restriction of Left Parotid Duct with Intraluminal Device, Open Approach */
"0CVC0ZZ"  =  "1"  /* Restriction of Left Parotid Duct, Open Approach */
"0CVC3CZ"  =  "1"  /* Restriction of Left Parotid Duct with Extraluminal Device, Percutaneous Approach */
"0CVC3DZ"  =  "1"  /* Restriction of Left Parotid Duct with Intraluminal Device, Percutaneous Approach */
"0CVC3ZZ"  =  "1"  /* Restriction of Left Parotid Duct, Percutaneous Approach */
"0CVC7DZ"  =  "1"  /* Restriction of Left Parotid Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0CVC7ZZ"  =  "1"  /* Restriction of Left Parotid Duct, Via Natural or Artificial Opening */
"0CVC8DZ"  =  "1"  /* Restriction of Left Parotid Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0CVC8ZZ"  =  "1"  /* Restriction of Left Parotid Duct, Via Natural or Artificial Opening Endoscopic */
"0CWS00Z"  =  "1"  /* Revision of Drainage Device in Larynx, Open Approach */
"0CWS07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Larynx, Open Approach */
"0CWS0DZ"  =  "1"  /* Revision of Intraluminal Device in Larynx, Open Approach */
"0CWS0JZ"  =  "1"  /* Revision of Synthetic Substitute in Larynx, Open Approach */
"0CWS0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Larynx, Open Approach */
"0CWS30Z"  =  "1"  /* Revision of Drainage Device in Larynx, Percutaneous Approach */
"0CWS37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Larynx, Percutaneous Approach */
"0CWS3DZ"  =  "1"  /* Revision of Intraluminal Device in Larynx, Percutaneous Approach */
"0CWS3JZ"  =  "1"  /* Revision of Synthetic Substitute in Larynx, Percutaneous Approach */
"0CWS3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Larynx, Percutaneous Approach */
"0CWS70Z"  =  "1"  /* Revision of Drainage Device in Larynx, Via Natural or Artificial Opening */
"0CWS77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Larynx, Via Natural or Artificial Opening */
"0CWS7DZ"  =  "1"  /* Revision of Intraluminal Device in Larynx, Via Natural or Artificial Opening */
"0CWS7JZ"  =  "1"  /* Revision of Synthetic Substitute in Larynx, Via Natural or Artificial Opening */
"0CWS7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Larynx, Via Natural or Artificial Opening */
"0CWS80Z"  =  "1"  /* Revision of Drainage Device in Larynx, Via Natural or Artificial Opening Endoscopic */
"0CWS87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Larynx, Via Natural or Artificial Opening Endoscopic */
"0CWS8DZ"  =  "1"  /* Revision of Intraluminal Device in Larynx, Via Natural or Artificial Opening Endoscopic */
"0CWS8JZ"  =  "1"  /* Revision of Synthetic Substitute in Larynx, Via Natural or Artificial Opening Endoscopic */
"0CWS8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Larynx, Via Natural or Artificial Opening Endoscopic */
"0CWY00Z"  =  "1"  /* Revision of Drainage Device in Mouth and Throat, Open Approach */
"0CWY01Z"  =  "1"  /* Revision of Radioactive Element in Mouth and Throat, Open Approach */
"0CWY0DZ"  =  "1"  /* Revision of Intraluminal Device in Mouth and Throat, Open Approach */
"0CWY0JZ"  =  "1"  /* Revision of Synthetic Substitute in Mouth and Throat, Open Approach */
"0CWY0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mouth and Throat, Open Approach */
"0CWY30Z"  =  "1"  /* Revision of Drainage Device in Mouth and Throat, Percutaneous Approach */
"0CWY31Z"  =  "1"  /* Revision of Radioactive Element in Mouth and Throat, Percutaneous Approach */
"0CWY37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mouth and Throat, Percutaneous Approach */
"0CWY3DZ"  =  "1"  /* Revision of Intraluminal Device in Mouth and Throat, Percutaneous Approach */
"0CWY3JZ"  =  "1"  /* Revision of Synthetic Substitute in Mouth and Throat, Percutaneous Approach */
"0CWY3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mouth and Throat, Percutaneous Approach */
"0CWY70Z"  =  "1"  /* Revision of Drainage Device in Mouth and Throat, Via Natural or Artificial Opening */
"0CWY71Z"  =  "1"  /* Revision of Radioactive Element in Mouth and Throat, Via Natural or Artificial Opening */
"0CWY77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mouth and Throat, Via Natural or Artificial Opening */
"0CWY7DZ"  =  "1"  /* Revision of Intraluminal Device in Mouth and Throat, Via Natural or Artificial Opening */
"0CWY7JZ"  =  "1"  /* Revision of Synthetic Substitute in Mouth and Throat, Via Natural or Artificial Opening */
"0CWY7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mouth and Throat, Via Natural or Artificial Opening */
"0CWY80Z"  =  "1"  /* Revision of Drainage Device in Mouth and Throat, Via Natural or Artificial Opening Endoscopic */
"0CWY81Z"  =  "1"  /* Revision of Radioactive Element in Mouth and Throat, Via Natural or Artificial Opening Endoscopic */
"0CWY87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mouth and Throat, Via Natural or Artificial Opening Endoscopic */
"0CWY8DZ"  =  "1"  /* Revision of Intraluminal Device in Mouth and Throat, Via Natural or Artificial Opening Endoscopic */
"0CWY8JZ"  =  "1"  /* Revision of Synthetic Substitute in Mouth and Throat, Via Natural or Artificial Opening Endoscopic */
"0CWY8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mouth and Throat, Via Natural or Artificial Opening Endoscopic */
"0CX00ZZ"  =  "1"  /* Transfer Upper Lip, Open Approach */
"0CX0XZZ"  =  "1"  /* Transfer Upper Lip, External Approach */
"0CX10ZZ"  =  "1"  /* Transfer Lower Lip, Open Approach */
"0CX1XZZ"  =  "1"  /* Transfer Lower Lip, External Approach */
"0CX30ZZ"  =  "1"  /* Transfer Soft Palate, Open Approach */
"0CX3XZZ"  =  "1"  /* Transfer Soft Palate, External Approach */
"0CX40ZZ"  =  "1"  /* Transfer Buccal Mucosa, Open Approach */
"0CX4XZZ"  =  "1"  /* Transfer Buccal Mucosa, External Approach */
"0CX50ZZ"  =  "1"  /* Transfer Upper Gingiva, Open Approach */
"0CX5XZZ"  =  "1"  /* Transfer Upper Gingiva, External Approach */
"0CX60ZZ"  =  "1"  /* Transfer Lower Gingiva, Open Approach */
"0CX6XZZ"  =  "1"  /* Transfer Lower Gingiva, External Approach */
"0CX70ZZ"  =  "1"  /* Transfer Tongue, Open Approach */
"0CX7XZZ"  =  "1"  /* Transfer Tongue, External Approach */
"0DQ50ZZ"  =  "1"  /* Repair Esophagus, Open Approach */
"0DQ53ZZ"  =  "1"  /* Repair Esophagus, Percutaneous Approach */
"0DQ54ZZ"  =  "1"  /* Repair Esophagus, Percutaneous Endoscopic Approach */
"0DQ57ZZ"  =  "1"  /* Repair Esophagus, Via Natural or Artificial Opening */
"0DQ58ZZ"  =  "1"  /* Repair Esophagus, Via Natural or Artificial Opening Endoscopic */
"0GTG0ZZ"  =  "1"  /* Resection of Left Thyroid Gland Lobe, Open Approach */
"0GTG4ZZ"  =  "1"  /* Resection of Left Thyroid Gland Lobe, Percutaneous Endoscopic Approach */
"0GTH0ZZ"  =  "1"  /* Resection of Right Thyroid Gland Lobe, Open Approach */
"0GTH4ZZ"  =  "1"  /* Resection of Right Thyroid Gland Lobe, Percutaneous Endoscopic Approach */
"0GTK0ZZ"  =  "1"  /* Resection of Thyroid Gland, Open Approach */
"0GTK4ZZ"  =  "1"  /* Resection of Thyroid Gland, Percutaneous Endoscopic Approach */
"0HQ4XZZ"  =  "1"  /* Repair Neck Skin, External Approach */
"0J9100Z"  =  "1"  /* Drainage of Face Subcutaneous Tissue and Fascia with Drainage Device, Open Approach */
"0J910ZZ"  =  "1"  /* Drainage of Face Subcutaneous Tissue and Fascia, Open Approach */
"0J9130Z"  =  "1"  /* Drainage of Face Subcutaneous Tissue and Fascia with Drainage Device, Percutaneous Approach */
"0JB40ZZ"  =  "1"  /* Excision of Anterior Neck Subcutaneous Tissue and Fascia, Open Approach */
"0JB43ZZ"  =  "1"  /* Excision of Anterior Neck Subcutaneous Tissue and Fascia, Percutaneous Approach */
"0K840ZZ"  =  "1"  /* Division of Tongue, Palate, Pharynx Muscle, Open Approach */
"0K843ZZ"  =  "1"  /* Division of Tongue, Palate, Pharynx Muscle, Percutaneous Approach */
"0K844ZZ"  =  "1"  /* Division of Tongue, Palate, Pharynx Muscle, Percutaneous Endoscopic Approach */
"0NBC0ZZ"  =  "1"  /* Excision of Right Sphenoid Bone, Open Approach */
"0NBC3ZZ"  =  "1"  /* Excision of Right Sphenoid Bone, Percutaneous Approach */
"0NBC4ZZ"  =  "1"  /* Excision of Right Sphenoid Bone, Percutaneous Endoscopic Approach */
"0NBD0ZZ"  =  "1"  /* Excision of Left Sphenoid Bone, Open Approach */
"0NBD3ZZ"  =  "1"  /* Excision of Left Sphenoid Bone, Percutaneous Approach */
"0NBD4ZZ"  =  "1"  /* Excision of Left Sphenoid Bone, Percutaneous Endoscopic Approach */
"0NBF0ZZ"  =  "1"  /* Excision of Right Ethmoid Bone, Open Approach */
"0NBF3ZZ"  =  "1"  /* Excision of Right Ethmoid Bone, Percutaneous Approach */
"0NBF4ZZ"  =  "1"  /* Excision of Right Ethmoid Bone, Percutaneous Endoscopic Approach */
"0NBG0ZZ"  =  "1"  /* Excision of Left Ethmoid Bone, Open Approach */
"0NBG3ZZ"  =  "1"  /* Excision of Left Ethmoid Bone, Percutaneous Approach */
"0NBG4ZZ"  =  "1"  /* Excision of Left Ethmoid Bone, Percutaneous Endoscopic Approach */
"0NBH0ZZ"  =  "1"  /* Excision of Right Lacrimal Bone, Open Approach */
"0NBH3ZZ"  =  "1"  /* Excision of Right Lacrimal Bone, Percutaneous Approach */
"0NBH4ZZ"  =  "1"  /* Excision of Right Lacrimal Bone, Percutaneous Endoscopic Approach */
"0NBJ0ZZ"  =  "1"  /* Excision of Left Lacrimal Bone, Open Approach */
"0NBJ3ZZ"  =  "1"  /* Excision of Left Lacrimal Bone, Percutaneous Approach */
"0NBJ4ZZ"  =  "1"  /* Excision of Left Lacrimal Bone, Percutaneous Endoscopic Approach */
"0NBK0ZZ"  =  "1"  /* Excision of Right Palatine Bone, Open Approach */
"0NBK3ZZ"  =  "1"  /* Excision of Right Palatine Bone, Percutaneous Approach */
"0NBK4ZZ"  =  "1"  /* Excision of Right Palatine Bone, Percutaneous Endoscopic Approach */
"0NBL0ZZ"  =  "1"  /* Excision of Left Palatine Bone, Open Approach */
"0NBL3ZZ"  =  "1"  /* Excision of Left Palatine Bone, Percutaneous Approach */
"0NBL4ZZ"  =  "1"  /* Excision of Left Palatine Bone, Percutaneous Endoscopic Approach */
"0NBM0ZZ"  =  "1"  /* Excision of Right Zygomatic Bone, Open Approach */
"0NBM3ZZ"  =  "1"  /* Excision of Right Zygomatic Bone, Percutaneous Approach */
"0NBM4ZZ"  =  "1"  /* Excision of Right Zygomatic Bone, Percutaneous Endoscopic Approach */
"0NBN0ZZ"  =  "1"  /* Excision of Left Zygomatic Bone, Open Approach */
"0NBN3ZZ"  =  "1"  /* Excision of Left Zygomatic Bone, Percutaneous Approach */
"0NBN4ZZ"  =  "1"  /* Excision of Left Zygomatic Bone, Percutaneous Endoscopic Approach */
"0NBR0ZZ"  =  "1"  /* Excision of Right Maxilla, Open Approach */
"0NBR3ZZ"  =  "1"  /* Excision of Right Maxilla, Percutaneous Approach */
"0NBR4ZZ"  =  "1"  /* Excision of Right Maxilla, Percutaneous Endoscopic Approach */
"0NBS0ZZ"  =  "1"  /* Excision of Left Maxilla, Open Approach */
"0NBS3ZZ"  =  "1"  /* Excision of Left Maxilla, Percutaneous Approach */
"0NBS4ZZ"  =  "1"  /* Excision of Left Maxilla, Percutaneous Endoscopic Approach */
"0NBT0ZZ"  =  "1"  /* Excision of Right Mandible, Open Approach */
"0NBT3ZZ"  =  "1"  /* Excision of Right Mandible, Percutaneous Approach */
"0NBT4ZZ"  =  "1"  /* Excision of Right Mandible, Percutaneous Endoscopic Approach */
"0NBV0ZZ"  =  "1"  /* Excision of Left Mandible, Open Approach */
"0NBV3ZZ"  =  "1"  /* Excision of Left Mandible, Percutaneous Approach */
"0NBV4ZZ"  =  "1"  /* Excision of Left Mandible, Percutaneous Endoscopic Approach */
"0NBX0ZZ"  =  "1"  /* Excision of Hyoid Bone, Open Approach */
"0NBX3ZZ"  =  "1"  /* Excision of Hyoid Bone, Percutaneous Approach */
"0NBX4ZZ"  =  "1"  /* Excision of Hyoid Bone, Percutaneous Endoscopic Approach */
"0NNC0ZZ"  =  "1"  /* Release Right Sphenoid Bone, Open Approach */
"0NNC3ZZ"  =  "1"  /* Release Right Sphenoid Bone, Percutaneous Approach */
"0NNC4ZZ"  =  "1"  /* Release Right Sphenoid Bone, Percutaneous Endoscopic Approach */
"0NND0ZZ"  =  "1"  /* Release Left Sphenoid Bone, Open Approach */
"0NND3ZZ"  =  "1"  /* Release Left Sphenoid Bone, Percutaneous Approach */
"0NND4ZZ"  =  "1"  /* Release Left Sphenoid Bone, Percutaneous Endoscopic Approach */
"0NNF0ZZ"  =  "1"  /* Release Right Ethmoid Bone, Open Approach */
"0NNF3ZZ"  =  "1"  /* Release Right Ethmoid Bone, Percutaneous Approach */
"0NNF4ZZ"  =  "1"  /* Release Right Ethmoid Bone, Percutaneous Endoscopic Approach */
"0NNG0ZZ"  =  "1"  /* Release Left Ethmoid Bone, Open Approach */
"0NNG3ZZ"  =  "1"  /* Release Left Ethmoid Bone, Percutaneous Approach */
"0NNG4ZZ"  =  "1"  /* Release Left Ethmoid Bone, Percutaneous Endoscopic Approach */
"0NNH0ZZ"  =  "1"  /* Release Right Lacrimal Bone, Open Approach */
"0NNH3ZZ"  =  "1"  /* Release Right Lacrimal Bone, Percutaneous Approach */
"0NNH4ZZ"  =  "1"  /* Release Right Lacrimal Bone, Percutaneous Endoscopic Approach */
"0NNJ0ZZ"  =  "1"  /* Release Left Lacrimal Bone, Open Approach */
"0NNJ3ZZ"  =  "1"  /* Release Left Lacrimal Bone, Percutaneous Approach */
"0NNJ4ZZ"  =  "1"  /* Release Left Lacrimal Bone, Percutaneous Endoscopic Approach */
"0NNK0ZZ"  =  "1"  /* Release Right Palatine Bone, Open Approach */
"0NNK3ZZ"  =  "1"  /* Release Right Palatine Bone, Percutaneous Approach */
"0NNK4ZZ"  =  "1"  /* Release Right Palatine Bone, Percutaneous Endoscopic Approach */
"0NNL0ZZ"  =  "1"  /* Release Left Palatine Bone, Open Approach */
"0NNL3ZZ"  =  "1"  /* Release Left Palatine Bone, Percutaneous Approach */
"0NNL4ZZ"  =  "1"  /* Release Left Palatine Bone, Percutaneous Endoscopic Approach */
"0NNM0ZZ"  =  "1"  /* Release Right Zygomatic Bone, Open Approach */
"0NNM3ZZ"  =  "1"  /* Release Right Zygomatic Bone, Percutaneous Approach */
"0NNM4ZZ"  =  "1"  /* Release Right Zygomatic Bone, Percutaneous Endoscopic Approach */
"0NNN0ZZ"  =  "1"  /* Release Left Zygomatic Bone, Open Approach */
"0NNN3ZZ"  =  "1"  /* Release Left Zygomatic Bone, Percutaneous Approach */
"0NNN4ZZ"  =  "1"  /* Release Left Zygomatic Bone, Percutaneous Endoscopic Approach */
"0NNP0ZZ"  =  "1"  /* Release Right Orbit, Open Approach */
"0NNP3ZZ"  =  "1"  /* Release Right Orbit, Percutaneous Approach */
"0NNP4ZZ"  =  "1"  /* Release Right Orbit, Percutaneous Endoscopic Approach */
"0NNQ0ZZ"  =  "1"  /* Release Left Orbit, Open Approach */
"0NNQ3ZZ"  =  "1"  /* Release Left Orbit, Percutaneous Approach */
"0NNQ4ZZ"  =  "1"  /* Release Left Orbit, Percutaneous Endoscopic Approach */
"0NNR0ZZ"  =  "1"  /* Release Right Maxilla, Open Approach */
"0NNR3ZZ"  =  "1"  /* Release Right Maxilla, Percutaneous Approach */
"0NNR4ZZ"  =  "1"  /* Release Right Maxilla, Percutaneous Endoscopic Approach */
"0NNS0ZZ"  =  "1"  /* Release Left Maxilla, Open Approach */
"0NNS3ZZ"  =  "1"  /* Release Left Maxilla, Percutaneous Approach */
"0NNS4ZZ"  =  "1"  /* Release Left Maxilla, Percutaneous Endoscopic Approach */
"0NNT0ZZ"  =  "1"  /* Release Right Mandible, Open Approach */
"0NNT3ZZ"  =  "1"  /* Release Right Mandible, Percutaneous Approach */
"0NNT4ZZ"  =  "1"  /* Release Right Mandible, Percutaneous Endoscopic Approach */
"0NNV0ZZ"  =  "1"  /* Release Left Mandible, Open Approach */
"0NNV3ZZ"  =  "1"  /* Release Left Mandible, Percutaneous Approach */
"0NNV4ZZ"  =  "1"  /* Release Left Mandible, Percutaneous Endoscopic Approach */
"0NNX0ZZ"  =  "1"  /* Release Hyoid Bone, Open Approach */
"0NNX3ZZ"  =  "1"  /* Release Hyoid Bone, Percutaneous Approach */
"0NNX4ZZ"  =  "1"  /* Release Hyoid Bone, Percutaneous Endoscopic Approach */
"0NPW04Z"  =  "1"  /* Removal of Internal Fixation Device from Facial Bone, Open Approach */
"0NPW34Z"  =  "1"  /* Removal of Internal Fixation Device from Facial Bone, Percutaneous Approach */
"0NPW44Z"  =  "1"  /* Removal of Internal Fixation Device from Facial Bone, Percutaneous Endoscopic Approach */
"0NPWX4Z"  =  "1"  /* Removal of Internal Fixation Device from Facial Bone, External Approach */
"0NQC0ZZ"  =  "1"  /* Repair Right Sphenoid Bone, Open Approach */
"0NQC3ZZ"  =  "1"  /* Repair Right Sphenoid Bone, Percutaneous Approach */
"0NQC4ZZ"  =  "1"  /* Repair Right Sphenoid Bone, Percutaneous Endoscopic Approach */
"0NQCXZZ"  =  "1"  /* Repair Right Sphenoid Bone, External Approach */
"0NQD0ZZ"  =  "1"  /* Repair Left Sphenoid Bone, Open Approach */
"0NQD3ZZ"  =  "1"  /* Repair Left Sphenoid Bone, Percutaneous Approach */
"0NQD4ZZ"  =  "1"  /* Repair Left Sphenoid Bone, Percutaneous Endoscopic Approach */
"0NQDXZZ"  =  "1"  /* Repair Left Sphenoid Bone, External Approach */
"0NQF0ZZ"  =  "1"  /* Repair Right Ethmoid Bone, Open Approach */
"0NQF3ZZ"  =  "1"  /* Repair Right Ethmoid Bone, Percutaneous Approach */
"0NQF4ZZ"  =  "1"  /* Repair Right Ethmoid Bone, Percutaneous Endoscopic Approach */
"0NQFXZZ"  =  "1"  /* Repair Right Ethmoid Bone, External Approach */
"0NQG0ZZ"  =  "1"  /* Repair Left Ethmoid Bone, Open Approach */
"0NQG3ZZ"  =  "1"  /* Repair Left Ethmoid Bone, Percutaneous Approach */
"0NQG4ZZ"  =  "1"  /* Repair Left Ethmoid Bone, Percutaneous Endoscopic Approach */
"0NQGXZZ"  =  "1"  /* Repair Left Ethmoid Bone, External Approach */
"0NQH0ZZ"  =  "1"  /* Repair Right Lacrimal Bone, Open Approach */
"0NQH3ZZ"  =  "1"  /* Repair Right Lacrimal Bone, Percutaneous Approach */
"0NQH4ZZ"  =  "1"  /* Repair Right Lacrimal Bone, Percutaneous Endoscopic Approach */
"0NQHXZZ"  =  "1"  /* Repair Right Lacrimal Bone, External Approach */
"0NQJ0ZZ"  =  "1"  /* Repair Left Lacrimal Bone, Open Approach */
"0NQJ3ZZ"  =  "1"  /* Repair Left Lacrimal Bone, Percutaneous Approach */
"0NQJ4ZZ"  =  "1"  /* Repair Left Lacrimal Bone, Percutaneous Endoscopic Approach */
"0NQJXZZ"  =  "1"  /* Repair Left Lacrimal Bone, External Approach */
"0NQK0ZZ"  =  "1"  /* Repair Right Palatine Bone, Open Approach */
"0NQK3ZZ"  =  "1"  /* Repair Right Palatine Bone, Percutaneous Approach */
"0NQK4ZZ"  =  "1"  /* Repair Right Palatine Bone, Percutaneous Endoscopic Approach */
"0NQKXZZ"  =  "1"  /* Repair Right Palatine Bone, External Approach */
"0NQL0ZZ"  =  "1"  /* Repair Left Palatine Bone, Open Approach */
"0NQL3ZZ"  =  "1"  /* Repair Left Palatine Bone, Percutaneous Approach */
"0NQL4ZZ"  =  "1"  /* Repair Left Palatine Bone, Percutaneous Endoscopic Approach */
"0NQLXZZ"  =  "1"  /* Repair Left Palatine Bone, External Approach */
"0NQM0ZZ"  =  "1"  /* Repair Right Zygomatic Bone, Open Approach */
"0NQM3ZZ"  =  "1"  /* Repair Right Zygomatic Bone, Percutaneous Approach */
"0NQM4ZZ"  =  "1"  /* Repair Right Zygomatic Bone, Percutaneous Endoscopic Approach */
"0NQMXZZ"  =  "1"  /* Repair Right Zygomatic Bone, External Approach */
"0NQN0ZZ"  =  "1"  /* Repair Left Zygomatic Bone, Open Approach */
"0NQN3ZZ"  =  "1"  /* Repair Left Zygomatic Bone, Percutaneous Approach */
"0NQN4ZZ"  =  "1"  /* Repair Left Zygomatic Bone, Percutaneous Endoscopic Approach */
"0NQNXZZ"  =  "1"  /* Repair Left Zygomatic Bone, External Approach */
"0NQR0ZZ"  =  "1"  /* Repair Right Maxilla, Open Approach */
"0NQR3ZZ"  =  "1"  /* Repair Right Maxilla, Percutaneous Approach */
"0NQR4ZZ"  =  "1"  /* Repair Right Maxilla, Percutaneous Endoscopic Approach */
"0NQRXZZ"  =  "1"  /* Repair Right Maxilla, External Approach */
"0NQS0ZZ"  =  "1"  /* Repair Left Maxilla, Open Approach */
"0NQS3ZZ"  =  "1"  /* Repair Left Maxilla, Percutaneous Approach */
"0NQS4ZZ"  =  "1"  /* Repair Left Maxilla, Percutaneous Endoscopic Approach */
"0NQSXZZ"  =  "1"  /* Repair Left Maxilla, External Approach */
"0NQT0ZZ"  =  "1"  /* Repair Right Mandible, Open Approach */
"0NQT3ZZ"  =  "1"  /* Repair Right Mandible, Percutaneous Approach */
"0NQT4ZZ"  =  "1"  /* Repair Right Mandible, Percutaneous Endoscopic Approach */
"0NQTXZZ"  =  "1"  /* Repair Right Mandible, External Approach */
"0NQV0ZZ"  =  "1"  /* Repair Left Mandible, Open Approach */
"0NQV3ZZ"  =  "1"  /* Repair Left Mandible, Percutaneous Approach */
"0NQV4ZZ"  =  "1"  /* Repair Left Mandible, Percutaneous Endoscopic Approach */
"0NQVXZZ"  =  "1"  /* Repair Left Mandible, External Approach */
"0NQX0ZZ"  =  "1"  /* Repair Hyoid Bone, Open Approach */
"0NQX3ZZ"  =  "1"  /* Repair Hyoid Bone, Percutaneous Approach */
"0NQX4ZZ"  =  "1"  /* Repair Hyoid Bone, Percutaneous Endoscopic Approach */
"0NQXXZZ"  =  "1"  /* Repair Hyoid Bone, External Approach */
"0NRC07Z"  =  "1"  /* Replacement of Right Sphenoid Bone with Autologous Tissue Substitute, Open Approach */
"0NRC0JZ"  =  "1"  /* Replacement of Right Sphenoid Bone with Synthetic Substitute, Open Approach */
"0NRC0KZ"  =  "1"  /* Replacement of Right Sphenoid Bone with Nonautologous Tissue Substitute, Open Approach */
"0NRC37Z"  =  "1"  /* Replacement of Right Sphenoid Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NRC3JZ"  =  "1"  /* Replacement of Right Sphenoid Bone with Synthetic Substitute, Percutaneous Approach */
"0NRC3KZ"  =  "1"  /* Replacement of Right Sphenoid Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRC47Z"  =  "1"  /* Replacement of Right Sphenoid Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRC4JZ"  =  "1"  /* Replacement of Right Sphenoid Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NRC4KZ"  =  "1"  /* Replacement of Right Sphenoid Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRD07Z"  =  "1"  /* Replacement of Left Sphenoid Bone with Autologous Tissue Substitute, Open Approach */
"0NRD0JZ"  =  "1"  /* Replacement of Left Sphenoid Bone with Synthetic Substitute, Open Approach */
"0NRD0KZ"  =  "1"  /* Replacement of Left Sphenoid Bone with Nonautologous Tissue Substitute, Open Approach */
"0NRD37Z"  =  "1"  /* Replacement of Left Sphenoid Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NRD3JZ"  =  "1"  /* Replacement of Left Sphenoid Bone with Synthetic Substitute, Percutaneous Approach */
"0NRD3KZ"  =  "1"  /* Replacement of Left Sphenoid Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRD47Z"  =  "1"  /* Replacement of Left Sphenoid Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRD4JZ"  =  "1"  /* Replacement of Left Sphenoid Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NRD4KZ"  =  "1"  /* Replacement of Left Sphenoid Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRF07Z"  =  "1"  /* Replacement of Right Ethmoid Bone with Autologous Tissue Substitute, Open Approach */
"0NRF0JZ"  =  "1"  /* Replacement of Right Ethmoid Bone with Synthetic Substitute, Open Approach */
"0NRF0KZ"  =  "1"  /* Replacement of Right Ethmoid Bone with Nonautologous Tissue Substitute, Open Approach */
"0NRF37Z"  =  "1"  /* Replacement of Right Ethmoid Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NRF3JZ"  =  "1"  /* Replacement of Right Ethmoid Bone with Synthetic Substitute, Percutaneous Approach */
"0NRF3KZ"  =  "1"  /* Replacement of Right Ethmoid Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRF47Z"  =  "1"  /* Replacement of Right Ethmoid Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRF4JZ"  =  "1"  /* Replacement of Right Ethmoid Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NRF4KZ"  =  "1"  /* Replacement of Right Ethmoid Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRG07Z"  =  "1"  /* Replacement of Left Ethmoid Bone with Autologous Tissue Substitute, Open Approach */
"0NRG0JZ"  =  "1"  /* Replacement of Left Ethmoid Bone with Synthetic Substitute, Open Approach */
"0NRG0KZ"  =  "1"  /* Replacement of Left Ethmoid Bone with Nonautologous Tissue Substitute, Open Approach */
"0NRG37Z"  =  "1"  /* Replacement of Left Ethmoid Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NRG3JZ"  =  "1"  /* Replacement of Left Ethmoid Bone with Synthetic Substitute, Percutaneous Approach */
"0NRG3KZ"  =  "1"  /* Replacement of Left Ethmoid Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRG47Z"  =  "1"  /* Replacement of Left Ethmoid Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRG4JZ"  =  "1"  /* Replacement of Left Ethmoid Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NRG4KZ"  =  "1"  /* Replacement of Left Ethmoid Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRH07Z"  =  "1"  /* Replacement of Right Lacrimal Bone with Autologous Tissue Substitute, Open Approach */
"0NRH0JZ"  =  "1"  /* Replacement of Right Lacrimal Bone with Synthetic Substitute, Open Approach */
"0NRH0KZ"  =  "1"  /* Replacement of Right Lacrimal Bone with Nonautologous Tissue Substitute, Open Approach */
"0NRH37Z"  =  "1"  /* Replacement of Right Lacrimal Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NRH3JZ"  =  "1"  /* Replacement of Right Lacrimal Bone with Synthetic Substitute, Percutaneous Approach */
"0NRH3KZ"  =  "1"  /* Replacement of Right Lacrimal Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRH47Z"  =  "1"  /* Replacement of Right Lacrimal Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRH4JZ"  =  "1"  /* Replacement of Right Lacrimal Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NRH4KZ"  =  "1"  /* Replacement of Right Lacrimal Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRJ07Z"  =  "1"  /* Replacement of Left Lacrimal Bone with Autologous Tissue Substitute, Open Approach */
"0NRJ0JZ"  =  "1"  /* Replacement of Left Lacrimal Bone with Synthetic Substitute, Open Approach */
"0NRJ0KZ"  =  "1"  /* Replacement of Left Lacrimal Bone with Nonautologous Tissue Substitute, Open Approach */
"0NRJ37Z"  =  "1"  /* Replacement of Left Lacrimal Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NRJ3JZ"  =  "1"  /* Replacement of Left Lacrimal Bone with Synthetic Substitute, Percutaneous Approach */
"0NRJ3KZ"  =  "1"  /* Replacement of Left Lacrimal Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRJ47Z"  =  "1"  /* Replacement of Left Lacrimal Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRJ4JZ"  =  "1"  /* Replacement of Left Lacrimal Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NRJ4KZ"  =  "1"  /* Replacement of Left Lacrimal Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRK07Z"  =  "1"  /* Replacement of Right Palatine Bone with Autologous Tissue Substitute, Open Approach */
"0NRK0JZ"  =  "1"  /* Replacement of Right Palatine Bone with Synthetic Substitute, Open Approach */
"0NRK0KZ"  =  "1"  /* Replacement of Right Palatine Bone with Nonautologous Tissue Substitute, Open Approach */
"0NRK37Z"  =  "1"  /* Replacement of Right Palatine Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NRK3JZ"  =  "1"  /* Replacement of Right Palatine Bone with Synthetic Substitute, Percutaneous Approach */
"0NRK3KZ"  =  "1"  /* Replacement of Right Palatine Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRK47Z"  =  "1"  /* Replacement of Right Palatine Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRK4JZ"  =  "1"  /* Replacement of Right Palatine Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NRK4KZ"  =  "1"  /* Replacement of Right Palatine Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRL07Z"  =  "1"  /* Replacement of Left Palatine Bone with Autologous Tissue Substitute, Open Approach */
"0NRL0JZ"  =  "1"  /* Replacement of Left Palatine Bone with Synthetic Substitute, Open Approach */
"0NRL0KZ"  =  "1"  /* Replacement of Left Palatine Bone with Nonautologous Tissue Substitute, Open Approach */
"0NRL37Z"  =  "1"  /* Replacement of Left Palatine Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NRL3JZ"  =  "1"  /* Replacement of Left Palatine Bone with Synthetic Substitute, Percutaneous Approach */
"0NRL3KZ"  =  "1"  /* Replacement of Left Palatine Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRL47Z"  =  "1"  /* Replacement of Left Palatine Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRL4JZ"  =  "1"  /* Replacement of Left Palatine Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NRL4KZ"  =  "1"  /* Replacement of Left Palatine Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRM07Z"  =  "1"  /* Replacement of Right Zygomatic Bone with Autologous Tissue Substitute, Open Approach */
"0NRM0JZ"  =  "1"  /* Replacement of Right Zygomatic Bone with Synthetic Substitute, Open Approach */
"0NRM0KZ"  =  "1"  /* Replacement of Right Zygomatic Bone with Nonautologous Tissue Substitute, Open Approach */
"0NRM37Z"  =  "1"  /* Replacement of Right Zygomatic Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NRM3JZ"  =  "1"  /* Replacement of Right Zygomatic Bone with Synthetic Substitute, Percutaneous Approach */
"0NRM3KZ"  =  "1"  /* Replacement of Right Zygomatic Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRM47Z"  =  "1"  /* Replacement of Right Zygomatic Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRM4JZ"  =  "1"  /* Replacement of Right Zygomatic Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NRM4KZ"  =  "1"  /* Replacement of Right Zygomatic Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRN07Z"  =  "1"  /* Replacement of Left Zygomatic Bone with Autologous Tissue Substitute, Open Approach */
"0NRN0JZ"  =  "1"  /* Replacement of Left Zygomatic Bone with Synthetic Substitute, Open Approach */
"0NRN0KZ"  =  "1"  /* Replacement of Left Zygomatic Bone with Nonautologous Tissue Substitute, Open Approach */
"0NRN37Z"  =  "1"  /* Replacement of Left Zygomatic Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NRN3JZ"  =  "1"  /* Replacement of Left Zygomatic Bone with Synthetic Substitute, Percutaneous Approach */
"0NRN3KZ"  =  "1"  /* Replacement of Left Zygomatic Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRN47Z"  =  "1"  /* Replacement of Left Zygomatic Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRN4JZ"  =  "1"  /* Replacement of Left Zygomatic Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NRN4KZ"  =  "1"  /* Replacement of Left Zygomatic Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRP0JZ"  =  "1"  /* Replacement of Right Orbit with Synthetic Substitute, Open Approach */
"0NRP0KZ"  =  "1"  /* Replacement of Right Orbit with Nonautologous Tissue Substitute, Open Approach */
"0NRP3KZ"  =  "1"  /* Replacement of Right Orbit with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRP4KZ"  =  "1"  /* Replacement of Right Orbit with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRQ0JZ"  =  "1"  /* Replacement of Left Orbit with Synthetic Substitute, Open Approach */
"0NRQ0KZ"  =  "1"  /* Replacement of Left Orbit with Nonautologous Tissue Substitute, Open Approach */
"0NRQ3KZ"  =  "1"  /* Replacement of Left Orbit with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRQ4KZ"  =  "1"  /* Replacement of Left Orbit with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRR07Z"  =  "1"  /* Replacement of Right Maxilla with Autologous Tissue Substitute, Open Approach */
"0NRR0JZ"  =  "1"  /* Replacement of Right Maxilla with Synthetic Substitute, Open Approach */
"0NRR0KZ"  =  "1"  /* Replacement of Right Maxilla with Nonautologous Tissue Substitute, Open Approach */
"0NRR37Z"  =  "1"  /* Replacement of Right Maxilla with Autologous Tissue Substitute, Percutaneous Approach */
"0NRR3JZ"  =  "1"  /* Replacement of Right Maxilla with Synthetic Substitute, Percutaneous Approach */
"0NRR3KZ"  =  "1"  /* Replacement of Right Maxilla with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRR47Z"  =  "1"  /* Replacement of Right Maxilla with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRR4JZ"  =  "1"  /* Replacement of Right Maxilla with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NRR4KZ"  =  "1"  /* Replacement of Right Maxilla with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRS07Z"  =  "1"  /* Replacement of Left Maxilla with Autologous Tissue Substitute, Open Approach */
"0NRS0JZ"  =  "1"  /* Replacement of Left Maxilla with Synthetic Substitute, Open Approach */
"0NRS0KZ"  =  "1"  /* Replacement of Left Maxilla with Nonautologous Tissue Substitute, Open Approach */
"0NRS37Z"  =  "1"  /* Replacement of Left Maxilla with Autologous Tissue Substitute, Percutaneous Approach */
"0NRS3JZ"  =  "1"  /* Replacement of Left Maxilla with Synthetic Substitute, Percutaneous Approach */
"0NRS3KZ"  =  "1"  /* Replacement of Left Maxilla with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRS47Z"  =  "1"  /* Replacement of Left Maxilla with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRS4JZ"  =  "1"  /* Replacement of Left Maxilla with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NRS4KZ"  =  "1"  /* Replacement of Left Maxilla with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRT07Z"  =  "1"  /* Replacement of Right Mandible with Autologous Tissue Substitute, Open Approach */
"0NRT0JZ"  =  "1"  /* Replacement of Right Mandible with Synthetic Substitute, Open Approach */
"0NRT0KZ"  =  "1"  /* Replacement of Right Mandible with Nonautologous Tissue Substitute, Open Approach */
"0NRT37Z"  =  "1"  /* Replacement of Right Mandible with Autologous Tissue Substitute, Percutaneous Approach */
"0NRT3JZ"  =  "1"  /* Replacement of Right Mandible with Synthetic Substitute, Percutaneous Approach */
"0NRT3KZ"  =  "1"  /* Replacement of Right Mandible with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRT47Z"  =  "1"  /* Replacement of Right Mandible with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRT4JZ"  =  "1"  /* Replacement of Right Mandible with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NRT4KZ"  =  "1"  /* Replacement of Right Mandible with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRV07Z"  =  "1"  /* Replacement of Left Mandible with Autologous Tissue Substitute, Open Approach */
"0NRV0JZ"  =  "1"  /* Replacement of Left Mandible with Synthetic Substitute, Open Approach */
"0NRV0KZ"  =  "1"  /* Replacement of Left Mandible with Nonautologous Tissue Substitute, Open Approach */
"0NRV37Z"  =  "1"  /* Replacement of Left Mandible with Autologous Tissue Substitute, Percutaneous Approach */
"0NRV3JZ"  =  "1"  /* Replacement of Left Mandible with Synthetic Substitute, Percutaneous Approach */
"0NRV3KZ"  =  "1"  /* Replacement of Left Mandible with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRV47Z"  =  "1"  /* Replacement of Left Mandible with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRV4JZ"  =  "1"  /* Replacement of Left Mandible with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NRV4KZ"  =  "1"  /* Replacement of Left Mandible with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRX07Z"  =  "1"  /* Replacement of Hyoid Bone with Autologous Tissue Substitute, Open Approach */
"0NRX0JZ"  =  "1"  /* Replacement of Hyoid Bone with Synthetic Substitute, Open Approach */
"0NRX0KZ"  =  "1"  /* Replacement of Hyoid Bone with Nonautologous Tissue Substitute, Open Approach */
"0NRX37Z"  =  "1"  /* Replacement of Hyoid Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NRX3JZ"  =  "1"  /* Replacement of Hyoid Bone with Synthetic Substitute, Percutaneous Approach */
"0NRX3KZ"  =  "1"  /* Replacement of Hyoid Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NRX47Z"  =  "1"  /* Replacement of Hyoid Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NRX4JZ"  =  "1"  /* Replacement of Hyoid Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NRX4KZ"  =  "1"  /* Replacement of Hyoid Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NSB04Z"  =  "1"  /* Reposition Nasal Bone with Internal Fixation Device, Open Approach */
"0NSB0ZZ"  =  "1"  /* Reposition Nasal Bone, Open Approach */
"0NSC04Z"  =  "1"  /* Reposition Right Sphenoid Bone with Internal Fixation Device, Open Approach */
"0NSC0ZZ"  =  "1"  /* Reposition Right Sphenoid Bone, Open Approach */
"0NSC34Z"  =  "1"  /* Reposition Right Sphenoid Bone with Internal Fixation Device, Percutaneous Approach */
"0NSC3ZZ"  =  "1"  /* Reposition Right Sphenoid Bone, Percutaneous Approach */
"0NSC44Z"  =  "1"  /* Reposition Right Sphenoid Bone with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NSC4ZZ"  =  "1"  /* Reposition Right Sphenoid Bone, Percutaneous Endoscopic Approach */
"0NSCXZZ"  =  "1"  /* Reposition Right Sphenoid Bone, External Approach */
"0NSD04Z"  =  "1"  /* Reposition Left Sphenoid Bone with Internal Fixation Device, Open Approach */
"0NSD0ZZ"  =  "1"  /* Reposition Left Sphenoid Bone, Open Approach */
"0NSD34Z"  =  "1"  /* Reposition Left Sphenoid Bone with Internal Fixation Device, Percutaneous Approach */
"0NSD3ZZ"  =  "1"  /* Reposition Left Sphenoid Bone, Percutaneous Approach */
"0NSD44Z"  =  "1"  /* Reposition Left Sphenoid Bone with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NSD4ZZ"  =  "1"  /* Reposition Left Sphenoid Bone, Percutaneous Endoscopic Approach */
"0NSDXZZ"  =  "1"  /* Reposition Left Sphenoid Bone, External Approach */
"0NSF04Z"  =  "1"  /* Reposition Right Ethmoid Bone with Internal Fixation Device, Open Approach */
"0NSF0ZZ"  =  "1"  /* Reposition Right Ethmoid Bone, Open Approach */
"0NSF34Z"  =  "1"  /* Reposition Right Ethmoid Bone with Internal Fixation Device, Percutaneous Approach */
"0NSF3ZZ"  =  "1"  /* Reposition Right Ethmoid Bone, Percutaneous Approach */
"0NSF44Z"  =  "1"  /* Reposition Right Ethmoid Bone with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NSF4ZZ"  =  "1"  /* Reposition Right Ethmoid Bone, Percutaneous Endoscopic Approach */
"0NSFXZZ"  =  "1"  /* Reposition Right Ethmoid Bone, External Approach */
"0NSG04Z"  =  "1"  /* Reposition Left Ethmoid Bone with Internal Fixation Device, Open Approach */
"0NSG0ZZ"  =  "1"  /* Reposition Left Ethmoid Bone, Open Approach */
"0NSG34Z"  =  "1"  /* Reposition Left Ethmoid Bone with Internal Fixation Device, Percutaneous Approach */
"0NSG3ZZ"  =  "1"  /* Reposition Left Ethmoid Bone, Percutaneous Approach */
"0NSG44Z"  =  "1"  /* Reposition Left Ethmoid Bone with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NSG4ZZ"  =  "1"  /* Reposition Left Ethmoid Bone, Percutaneous Endoscopic Approach */
"0NSGXZZ"  =  "1"  /* Reposition Left Ethmoid Bone, External Approach */
"0NSH04Z"  =  "1"  /* Reposition Right Lacrimal Bone with Internal Fixation Device, Open Approach */
"0NSH0ZZ"  =  "1"  /* Reposition Right Lacrimal Bone, Open Approach */
"0NSH34Z"  =  "1"  /* Reposition Right Lacrimal Bone with Internal Fixation Device, Percutaneous Approach */
"0NSH3ZZ"  =  "1"  /* Reposition Right Lacrimal Bone, Percutaneous Approach */
"0NSH44Z"  =  "1"  /* Reposition Right Lacrimal Bone with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NSH4ZZ"  =  "1"  /* Reposition Right Lacrimal Bone, Percutaneous Endoscopic Approach */
"0NSHXZZ"  =  "1"  /* Reposition Right Lacrimal Bone, External Approach */
"0NSJ04Z"  =  "1"  /* Reposition Left Lacrimal Bone with Internal Fixation Device, Open Approach */
"0NSJ0ZZ"  =  "1"  /* Reposition Left Lacrimal Bone, Open Approach */
"0NSJ34Z"  =  "1"  /* Reposition Left Lacrimal Bone with Internal Fixation Device, Percutaneous Approach */
"0NSJ3ZZ"  =  "1"  /* Reposition Left Lacrimal Bone, Percutaneous Approach */
"0NSJ44Z"  =  "1"  /* Reposition Left Lacrimal Bone with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NSJ4ZZ"  =  "1"  /* Reposition Left Lacrimal Bone, Percutaneous Endoscopic Approach */
"0NSJXZZ"  =  "1"  /* Reposition Left Lacrimal Bone, External Approach */
"0NSK04Z"  =  "1"  /* Reposition Right Palatine Bone with Internal Fixation Device, Open Approach */
"0NSK0ZZ"  =  "1"  /* Reposition Right Palatine Bone, Open Approach */
"0NSK34Z"  =  "1"  /* Reposition Right Palatine Bone with Internal Fixation Device, Percutaneous Approach */
"0NSK3ZZ"  =  "1"  /* Reposition Right Palatine Bone, Percutaneous Approach */
"0NSK44Z"  =  "1"  /* Reposition Right Palatine Bone with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NSK4ZZ"  =  "1"  /* Reposition Right Palatine Bone, Percutaneous Endoscopic Approach */
"0NSKXZZ"  =  "1"  /* Reposition Right Palatine Bone, External Approach */
"0NSL04Z"  =  "1"  /* Reposition Left Palatine Bone with Internal Fixation Device, Open Approach */
"0NSL0ZZ"  =  "1"  /* Reposition Left Palatine Bone, Open Approach */
"0NSL34Z"  =  "1"  /* Reposition Left Palatine Bone with Internal Fixation Device, Percutaneous Approach */
"0NSL3ZZ"  =  "1"  /* Reposition Left Palatine Bone, Percutaneous Approach */
"0NSL44Z"  =  "1"  /* Reposition Left Palatine Bone with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NSL4ZZ"  =  "1"  /* Reposition Left Palatine Bone, Percutaneous Endoscopic Approach */
"0NSLXZZ"  =  "1"  /* Reposition Left Palatine Bone, External Approach */
"0NSM04Z"  =  "1"  /* Reposition Right Zygomatic Bone with Internal Fixation Device, Open Approach */
"0NSM0ZZ"  =  "1"  /* Reposition Right Zygomatic Bone, Open Approach */
"0NSM34Z"  =  "1"  /* Reposition Right Zygomatic Bone with Internal Fixation Device, Percutaneous Approach */
"0NSM3ZZ"  =  "1"  /* Reposition Right Zygomatic Bone, Percutaneous Approach */
"0NSM44Z"  =  "1"  /* Reposition Right Zygomatic Bone with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NSM4ZZ"  =  "1"  /* Reposition Right Zygomatic Bone, Percutaneous Endoscopic Approach */
"0NSMXZZ"  =  "1"  /* Reposition Right Zygomatic Bone, External Approach */
"0NSN04Z"  =  "1"  /* Reposition Left Zygomatic Bone with Internal Fixation Device, Open Approach */
"0NSN0ZZ"  =  "1"  /* Reposition Left Zygomatic Bone, Open Approach */
"0NSN34Z"  =  "1"  /* Reposition Left Zygomatic Bone with Internal Fixation Device, Percutaneous Approach */
"0NSN3ZZ"  =  "1"  /* Reposition Left Zygomatic Bone, Percutaneous Approach */
"0NSN44Z"  =  "1"  /* Reposition Left Zygomatic Bone with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NSN4ZZ"  =  "1"  /* Reposition Left Zygomatic Bone, Percutaneous Endoscopic Approach */
"0NSNXZZ"  =  "1"  /* Reposition Left Zygomatic Bone, External Approach */
"0NSP04Z"  =  "1"  /* Reposition Right Orbit with Internal Fixation Device, Open Approach */
"0NSP0ZZ"  =  "1"  /* Reposition Right Orbit, Open Approach */
"0NSP34Z"  =  "1"  /* Reposition Right Orbit with Internal Fixation Device, Percutaneous Approach */
"0NSP3ZZ"  =  "1"  /* Reposition Right Orbit, Percutaneous Approach */
"0NSP44Z"  =  "1"  /* Reposition Right Orbit with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NSP4ZZ"  =  "1"  /* Reposition Right Orbit, Percutaneous Endoscopic Approach */
"0NSPXZZ"  =  "1"  /* Reposition Right Orbit, External Approach */
"0NSQ04Z"  =  "1"  /* Reposition Left Orbit with Internal Fixation Device, Open Approach */
"0NSQ0ZZ"  =  "1"  /* Reposition Left Orbit, Open Approach */
"0NSQ34Z"  =  "1"  /* Reposition Left Orbit with Internal Fixation Device, Percutaneous Approach */
"0NSQ3ZZ"  =  "1"  /* Reposition Left Orbit, Percutaneous Approach */
"0NSQ44Z"  =  "1"  /* Reposition Left Orbit with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NSQ4ZZ"  =  "1"  /* Reposition Left Orbit, Percutaneous Endoscopic Approach */
"0NSQXZZ"  =  "1"  /* Reposition Left Orbit, External Approach */
"0NSR04Z"  =  "1"  /* Reposition Right Maxilla with Internal Fixation Device, Open Approach */
"0NSR05Z"  =  "1"  /* Reposition Right Maxilla with External Fixation Device, Open Approach */
"0NSR0ZZ"  =  "1"  /* Reposition Right Maxilla, Open Approach */
"0NSR34Z"  =  "1"  /* Reposition Right Maxilla with Internal Fixation Device, Percutaneous Approach */
"0NSR35Z"  =  "1"  /* Reposition Right Maxilla with External Fixation Device, Percutaneous Approach */
"0NSR3ZZ"  =  "1"  /* Reposition Right Maxilla, Percutaneous Approach */
"0NSR44Z"  =  "1"  /* Reposition Right Maxilla with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NSR45Z"  =  "1"  /* Reposition Right Maxilla with External Fixation Device, Percutaneous Endoscopic Approach */
"0NSR4ZZ"  =  "1"  /* Reposition Right Maxilla, Percutaneous Endoscopic Approach */
"0NSRXZZ"  =  "1"  /* Reposition Right Maxilla, External Approach */
"0NSS04Z"  =  "1"  /* Reposition Left Maxilla with Internal Fixation Device, Open Approach */
"0NSS05Z"  =  "1"  /* Reposition Left Maxilla with External Fixation Device, Open Approach */
"0NSS0ZZ"  =  "1"  /* Reposition Left Maxilla, Open Approach */
"0NSS34Z"  =  "1"  /* Reposition Left Maxilla with Internal Fixation Device, Percutaneous Approach */
"0NSS35Z"  =  "1"  /* Reposition Left Maxilla with External Fixation Device, Percutaneous Approach */
"0NSS3ZZ"  =  "1"  /* Reposition Left Maxilla, Percutaneous Approach */
"0NSS44Z"  =  "1"  /* Reposition Left Maxilla with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NSS45Z"  =  "1"  /* Reposition Left Maxilla with External Fixation Device, Percutaneous Endoscopic Approach */
"0NSS4ZZ"  =  "1"  /* Reposition Left Maxilla, Percutaneous Endoscopic Approach */
"0NSSXZZ"  =  "1"  /* Reposition Left Maxilla, External Approach */
"0NST04Z"  =  "1"  /* Reposition Right Mandible with Internal Fixation Device, Open Approach */
"0NST05Z"  =  "1"  /* Reposition Right Mandible with External Fixation Device, Open Approach */
"0NST0ZZ"  =  "1"  /* Reposition Right Mandible, Open Approach */
"0NST34Z"  =  "1"  /* Reposition Right Mandible with Internal Fixation Device, Percutaneous Approach */
"0NST35Z"  =  "1"  /* Reposition Right Mandible with External Fixation Device, Percutaneous Approach */
"0NST3ZZ"  =  "1"  /* Reposition Right Mandible, Percutaneous Approach */
"0NST44Z"  =  "1"  /* Reposition Right Mandible with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NST45Z"  =  "1"  /* Reposition Right Mandible with External Fixation Device, Percutaneous Endoscopic Approach */
"0NST4ZZ"  =  "1"  /* Reposition Right Mandible, Percutaneous Endoscopic Approach */
"0NSTXZZ"  =  "1"  /* Reposition Right Mandible, External Approach */
"0NSV04Z"  =  "1"  /* Reposition Left Mandible with Internal Fixation Device, Open Approach */
"0NSV05Z"  =  "1"  /* Reposition Left Mandible with External Fixation Device, Open Approach */
"0NSV0ZZ"  =  "1"  /* Reposition Left Mandible, Open Approach */
"0NSV34Z"  =  "1"  /* Reposition Left Mandible with Internal Fixation Device, Percutaneous Approach */
"0NSV35Z"  =  "1"  /* Reposition Left Mandible with External Fixation Device, Percutaneous Approach */
"0NSV3ZZ"  =  "1"  /* Reposition Left Mandible, Percutaneous Approach */
"0NSV44Z"  =  "1"  /* Reposition Left Mandible with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NSV45Z"  =  "1"  /* Reposition Left Mandible with External Fixation Device, Percutaneous Endoscopic Approach */
"0NSV4ZZ"  =  "1"  /* Reposition Left Mandible, Percutaneous Endoscopic Approach */
"0NSVXZZ"  =  "1"  /* Reposition Left Mandible, External Approach */
"0NSX04Z"  =  "1"  /* Reposition Hyoid Bone with Internal Fixation Device, Open Approach */
"0NSX0ZZ"  =  "1"  /* Reposition Hyoid Bone, Open Approach */
"0NSX34Z"  =  "1"  /* Reposition Hyoid Bone with Internal Fixation Device, Percutaneous Approach */
"0NSX3ZZ"  =  "1"  /* Reposition Hyoid Bone, Percutaneous Approach */
"0NSX44Z"  =  "1"  /* Reposition Hyoid Bone with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0NSX4ZZ"  =  "1"  /* Reposition Hyoid Bone, Percutaneous Endoscopic Approach */
"0NSXXZZ"  =  "1"  /* Reposition Hyoid Bone, External Approach */
"0NTB0ZZ"  =  "1"  /* Resection of Nasal Bone, Open Approach */
"0NTC0ZZ"  =  "1"  /* Resection of Right Sphenoid Bone, Open Approach */
"0NTD0ZZ"  =  "1"  /* Resection of Left Sphenoid Bone, Open Approach */
"0NTF0ZZ"  =  "1"  /* Resection of Right Ethmoid Bone, Open Approach */
"0NTG0ZZ"  =  "1"  /* Resection of Left Ethmoid Bone, Open Approach */
"0NTH0ZZ"  =  "1"  /* Resection of Right Lacrimal Bone, Open Approach */
"0NTJ0ZZ"  =  "1"  /* Resection of Left Lacrimal Bone, Open Approach */
"0NTK0ZZ"  =  "1"  /* Resection of Right Palatine Bone, Open Approach */
"0NTL0ZZ"  =  "1"  /* Resection of Left Palatine Bone, Open Approach */
"0NTM0ZZ"  =  "1"  /* Resection of Right Zygomatic Bone, Open Approach */
"0NTN0ZZ"  =  "1"  /* Resection of Left Zygomatic Bone, Open Approach */
"0NTP0ZZ"  =  "1"  /* Resection of Right Orbit, Open Approach */
"0NTQ0ZZ"  =  "1"  /* Resection of Left Orbit, Open Approach */
"0NTR0ZZ"  =  "1"  /* Resection of Right Maxilla, Open Approach */
"0NTS0ZZ"  =  "1"  /* Resection of Left Maxilla, Open Approach */
"0NTT0ZZ"  =  "1"  /* Resection of Right Mandible, Open Approach */
"0NTV0ZZ"  =  "1"  /* Resection of Left Mandible, Open Approach */
"0NTX0ZZ"  =  "1"  /* Resection of Hyoid Bone, Open Approach */
"0NUC07Z"  =  "1"  /* Supplement Right Sphenoid Bone with Autologous Tissue Substitute, Open Approach */
"0NUC0JZ"  =  "1"  /* Supplement Right Sphenoid Bone with Synthetic Substitute, Open Approach */
"0NUC0KZ"  =  "1"  /* Supplement Right Sphenoid Bone with Nonautologous Tissue Substitute, Open Approach */
"0NUC37Z"  =  "1"  /* Supplement Right Sphenoid Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NUC3JZ"  =  "1"  /* Supplement Right Sphenoid Bone with Synthetic Substitute, Percutaneous Approach */
"0NUC3KZ"  =  "1"  /* Supplement Right Sphenoid Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUC47Z"  =  "1"  /* Supplement Right Sphenoid Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUC4JZ"  =  "1"  /* Supplement Right Sphenoid Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NUC4KZ"  =  "1"  /* Supplement Right Sphenoid Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUD07Z"  =  "1"  /* Supplement Left Sphenoid Bone with Autologous Tissue Substitute, Open Approach */
"0NUD0JZ"  =  "1"  /* Supplement Left Sphenoid Bone with Synthetic Substitute, Open Approach */
"0NUD0KZ"  =  "1"  /* Supplement Left Sphenoid Bone with Nonautologous Tissue Substitute, Open Approach */
"0NUD37Z"  =  "1"  /* Supplement Left Sphenoid Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NUD3JZ"  =  "1"  /* Supplement Left Sphenoid Bone with Synthetic Substitute, Percutaneous Approach */
"0NUD3KZ"  =  "1"  /* Supplement Left Sphenoid Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUD47Z"  =  "1"  /* Supplement Left Sphenoid Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUD4JZ"  =  "1"  /* Supplement Left Sphenoid Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NUD4KZ"  =  "1"  /* Supplement Left Sphenoid Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUF07Z"  =  "1"  /* Supplement Right Ethmoid Bone with Autologous Tissue Substitute, Open Approach */
"0NUF0JZ"  =  "1"  /* Supplement Right Ethmoid Bone with Synthetic Substitute, Open Approach */
"0NUF0KZ"  =  "1"  /* Supplement Right Ethmoid Bone with Nonautologous Tissue Substitute, Open Approach */
"0NUF37Z"  =  "1"  /* Supplement Right Ethmoid Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NUF3JZ"  =  "1"  /* Supplement Right Ethmoid Bone with Synthetic Substitute, Percutaneous Approach */
"0NUF3KZ"  =  "1"  /* Supplement Right Ethmoid Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUF47Z"  =  "1"  /* Supplement Right Ethmoid Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUF4JZ"  =  "1"  /* Supplement Right Ethmoid Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NUF4KZ"  =  "1"  /* Supplement Right Ethmoid Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUG07Z"  =  "1"  /* Supplement Left Ethmoid Bone with Autologous Tissue Substitute, Open Approach */
"0NUG0JZ"  =  "1"  /* Supplement Left Ethmoid Bone with Synthetic Substitute, Open Approach */
"0NUG0KZ"  =  "1"  /* Supplement Left Ethmoid Bone with Nonautologous Tissue Substitute, Open Approach */
"0NUG37Z"  =  "1"  /* Supplement Left Ethmoid Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NUG3JZ"  =  "1"  /* Supplement Left Ethmoid Bone with Synthetic Substitute, Percutaneous Approach */
"0NUG3KZ"  =  "1"  /* Supplement Left Ethmoid Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUG47Z"  =  "1"  /* Supplement Left Ethmoid Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUG4JZ"  =  "1"  /* Supplement Left Ethmoid Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NUG4KZ"  =  "1"  /* Supplement Left Ethmoid Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUH07Z"  =  "1"  /* Supplement Right Lacrimal Bone with Autologous Tissue Substitute, Open Approach */
"0NUH0JZ"  =  "1"  /* Supplement Right Lacrimal Bone with Synthetic Substitute, Open Approach */
"0NUH0KZ"  =  "1"  /* Supplement Right Lacrimal Bone with Nonautologous Tissue Substitute, Open Approach */
"0NUH37Z"  =  "1"  /* Supplement Right Lacrimal Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NUH3JZ"  =  "1"  /* Supplement Right Lacrimal Bone with Synthetic Substitute, Percutaneous Approach */
"0NUH3KZ"  =  "1"  /* Supplement Right Lacrimal Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUH47Z"  =  "1"  /* Supplement Right Lacrimal Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUH4JZ"  =  "1"  /* Supplement Right Lacrimal Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NUH4KZ"  =  "1"  /* Supplement Right Lacrimal Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUJ07Z"  =  "1"  /* Supplement Left Lacrimal Bone with Autologous Tissue Substitute, Open Approach */
"0NUJ0JZ"  =  "1"  /* Supplement Left Lacrimal Bone with Synthetic Substitute, Open Approach */
"0NUJ0KZ"  =  "1"  /* Supplement Left Lacrimal Bone with Nonautologous Tissue Substitute, Open Approach */
"0NUJ37Z"  =  "1"  /* Supplement Left Lacrimal Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NUJ3JZ"  =  "1"  /* Supplement Left Lacrimal Bone with Synthetic Substitute, Percutaneous Approach */
"0NUJ3KZ"  =  "1"  /* Supplement Left Lacrimal Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUJ47Z"  =  "1"  /* Supplement Left Lacrimal Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUJ4JZ"  =  "1"  /* Supplement Left Lacrimal Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NUJ4KZ"  =  "1"  /* Supplement Left Lacrimal Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUK07Z"  =  "1"  /* Supplement Right Palatine Bone with Autologous Tissue Substitute, Open Approach */
"0NUK0JZ"  =  "1"  /* Supplement Right Palatine Bone with Synthetic Substitute, Open Approach */
"0NUK0KZ"  =  "1"  /* Supplement Right Palatine Bone with Nonautologous Tissue Substitute, Open Approach */
"0NUK37Z"  =  "1"  /* Supplement Right Palatine Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NUK3JZ"  =  "1"  /* Supplement Right Palatine Bone with Synthetic Substitute, Percutaneous Approach */
"0NUK3KZ"  =  "1"  /* Supplement Right Palatine Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUK47Z"  =  "1"  /* Supplement Right Palatine Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUK4JZ"  =  "1"  /* Supplement Right Palatine Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NUK4KZ"  =  "1"  /* Supplement Right Palatine Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUL07Z"  =  "1"  /* Supplement Left Palatine Bone with Autologous Tissue Substitute, Open Approach */
"0NUL0JZ"  =  "1"  /* Supplement Left Palatine Bone with Synthetic Substitute, Open Approach */
"0NUL0KZ"  =  "1"  /* Supplement Left Palatine Bone with Nonautologous Tissue Substitute, Open Approach */
"0NUL37Z"  =  "1"  /* Supplement Left Palatine Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NUL3JZ"  =  "1"  /* Supplement Left Palatine Bone with Synthetic Substitute, Percutaneous Approach */
"0NUL3KZ"  =  "1"  /* Supplement Left Palatine Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUL47Z"  =  "1"  /* Supplement Left Palatine Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUL4JZ"  =  "1"  /* Supplement Left Palatine Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NUL4KZ"  =  "1"  /* Supplement Left Palatine Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUM07Z"  =  "1"  /* Supplement Right Zygomatic Bone with Autologous Tissue Substitute, Open Approach */
"0NUM0JZ"  =  "1"  /* Supplement Right Zygomatic Bone with Synthetic Substitute, Open Approach */
"0NUM0KZ"  =  "1"  /* Supplement Right Zygomatic Bone with Nonautologous Tissue Substitute, Open Approach */
"0NUM37Z"  =  "1"  /* Supplement Right Zygomatic Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NUM3JZ"  =  "1"  /* Supplement Right Zygomatic Bone with Synthetic Substitute, Percutaneous Approach */
"0NUM3KZ"  =  "1"  /* Supplement Right Zygomatic Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUM47Z"  =  "1"  /* Supplement Right Zygomatic Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUM4JZ"  =  "1"  /* Supplement Right Zygomatic Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NUM4KZ"  =  "1"  /* Supplement Right Zygomatic Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUN07Z"  =  "1"  /* Supplement Left Zygomatic Bone with Autologous Tissue Substitute, Open Approach */
"0NUN0JZ"  =  "1"  /* Supplement Left Zygomatic Bone with Synthetic Substitute, Open Approach */
"0NUN0KZ"  =  "1"  /* Supplement Left Zygomatic Bone with Nonautologous Tissue Substitute, Open Approach */
"0NUN37Z"  =  "1"  /* Supplement Left Zygomatic Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NUN3JZ"  =  "1"  /* Supplement Left Zygomatic Bone with Synthetic Substitute, Percutaneous Approach */
"0NUN3KZ"  =  "1"  /* Supplement Left Zygomatic Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUN47Z"  =  "1"  /* Supplement Left Zygomatic Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUN4JZ"  =  "1"  /* Supplement Left Zygomatic Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NUN4KZ"  =  "1"  /* Supplement Left Zygomatic Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUP07Z"  =  "1"  /* Supplement Right Orbit with Autologous Tissue Substitute, Open Approach */
"0NUP0KZ"  =  "1"  /* Supplement Right Orbit with Nonautologous Tissue Substitute, Open Approach */
"0NUP37Z"  =  "1"  /* Supplement Right Orbit with Autologous Tissue Substitute, Percutaneous Approach */
"0NUP3KZ"  =  "1"  /* Supplement Right Orbit with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUP47Z"  =  "1"  /* Supplement Right Orbit with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUP4KZ"  =  "1"  /* Supplement Right Orbit with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUQ07Z"  =  "1"  /* Supplement Left Orbit with Autologous Tissue Substitute, Open Approach */
"0NUQ0KZ"  =  "1"  /* Supplement Left Orbit with Nonautologous Tissue Substitute, Open Approach */
"0NUQ37Z"  =  "1"  /* Supplement Left Orbit with Autologous Tissue Substitute, Percutaneous Approach */
"0NUQ3KZ"  =  "1"  /* Supplement Left Orbit with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUQ47Z"  =  "1"  /* Supplement Left Orbit with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUQ4KZ"  =  "1"  /* Supplement Left Orbit with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUR07Z"  =  "1"  /* Supplement Right Maxilla with Autologous Tissue Substitute, Open Approach */
"0NUR0JZ"  =  "1"  /* Supplement Right Maxilla with Synthetic Substitute, Open Approach */
"0NUR0KZ"  =  "1"  /* Supplement Right Maxilla with Nonautologous Tissue Substitute, Open Approach */
"0NUR37Z"  =  "1"  /* Supplement Right Maxilla with Autologous Tissue Substitute, Percutaneous Approach */
"0NUR3JZ"  =  "1"  /* Supplement Right Maxilla with Synthetic Substitute, Percutaneous Approach */
"0NUR3KZ"  =  "1"  /* Supplement Right Maxilla with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUR47Z"  =  "1"  /* Supplement Right Maxilla with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUR4JZ"  =  "1"  /* Supplement Right Maxilla with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NUR4KZ"  =  "1"  /* Supplement Right Maxilla with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUS07Z"  =  "1"  /* Supplement Left Maxilla with Autologous Tissue Substitute, Open Approach */
"0NUS0JZ"  =  "1"  /* Supplement Left Maxilla with Synthetic Substitute, Open Approach */
"0NUS0KZ"  =  "1"  /* Supplement Left Maxilla with Nonautologous Tissue Substitute, Open Approach */
"0NUS37Z"  =  "1"  /* Supplement Left Maxilla with Autologous Tissue Substitute, Percutaneous Approach */
"0NUS3JZ"  =  "1"  /* Supplement Left Maxilla with Synthetic Substitute, Percutaneous Approach */
"0NUS3KZ"  =  "1"  /* Supplement Left Maxilla with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUS47Z"  =  "1"  /* Supplement Left Maxilla with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUS4JZ"  =  "1"  /* Supplement Left Maxilla with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NUS4KZ"  =  "1"  /* Supplement Left Maxilla with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUT07Z"  =  "1"  /* Supplement Right Mandible with Autologous Tissue Substitute, Open Approach */
"0NUT0JZ"  =  "1"  /* Supplement Right Mandible with Synthetic Substitute, Open Approach */
"0NUT0KZ"  =  "1"  /* Supplement Right Mandible with Nonautologous Tissue Substitute, Open Approach */
"0NUT37Z"  =  "1"  /* Supplement Right Mandible with Autologous Tissue Substitute, Percutaneous Approach */
"0NUT3JZ"  =  "1"  /* Supplement Right Mandible with Synthetic Substitute, Percutaneous Approach */
"0NUT3KZ"  =  "1"  /* Supplement Right Mandible with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUT47Z"  =  "1"  /* Supplement Right Mandible with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUT4JZ"  =  "1"  /* Supplement Right Mandible with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NUT4KZ"  =  "1"  /* Supplement Right Mandible with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUV07Z"  =  "1"  /* Supplement Left Mandible with Autologous Tissue Substitute, Open Approach */
"0NUV0JZ"  =  "1"  /* Supplement Left Mandible with Synthetic Substitute, Open Approach */
"0NUV0KZ"  =  "1"  /* Supplement Left Mandible with Nonautologous Tissue Substitute, Open Approach */
"0NUV37Z"  =  "1"  /* Supplement Left Mandible with Autologous Tissue Substitute, Percutaneous Approach */
"0NUV3JZ"  =  "1"  /* Supplement Left Mandible with Synthetic Substitute, Percutaneous Approach */
"0NUV3KZ"  =  "1"  /* Supplement Left Mandible with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUV47Z"  =  "1"  /* Supplement Left Mandible with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUV4JZ"  =  "1"  /* Supplement Left Mandible with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NUV4KZ"  =  "1"  /* Supplement Left Mandible with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUX07Z"  =  "1"  /* Supplement Hyoid Bone with Autologous Tissue Substitute, Open Approach */
"0NUX0JZ"  =  "1"  /* Supplement Hyoid Bone with Synthetic Substitute, Open Approach */
"0NUX0KZ"  =  "1"  /* Supplement Hyoid Bone with Nonautologous Tissue Substitute, Open Approach */
"0NUX37Z"  =  "1"  /* Supplement Hyoid Bone with Autologous Tissue Substitute, Percutaneous Approach */
"0NUX3JZ"  =  "1"  /* Supplement Hyoid Bone with Synthetic Substitute, Percutaneous Approach */
"0NUX3KZ"  =  "1"  /* Supplement Hyoid Bone with Nonautologous Tissue Substitute, Percutaneous Approach */
"0NUX47Z"  =  "1"  /* Supplement Hyoid Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0NUX4JZ"  =  "1"  /* Supplement Hyoid Bone with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0NUX4KZ"  =  "1"  /* Supplement Hyoid Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0RGC04Z"  =  "1"  /* Fusion of Right Temporomandibular Joint with Internal Fixation Device, Open Approach */
"0RGC07Z"  =  "1"  /* Fusion of Right Temporomandibular Joint with Autologous Tissue Substitute, Open Approach */
"0RGC0JZ"  =  "1"  /* Fusion of Right Temporomandibular Joint with Synthetic Substitute, Open Approach */
"0RGC0KZ"  =  "1"  /* Fusion of Right Temporomandibular Joint with Nonautologous Tissue Substitute, Open Approach */
"0RGC0ZZ"  =  "1"  /* Fusion of Right Temporomandibular Joint, Open Approach */
"0RGC34Z"  =  "1"  /* Fusion of Right Temporomandibular Joint with Internal Fixation Device, Percutaneous Approach */
"0RGC37Z"  =  "1"  /* Fusion of Right Temporomandibular Joint with Autologous Tissue Substitute, Percutaneous Approach */
"0RGC3JZ"  =  "1"  /* Fusion of Right Temporomandibular Joint with Synthetic Substitute, Percutaneous Approach */
"0RGC3KZ"  =  "1"  /* Fusion of Right Temporomandibular Joint with Nonautologous Tissue Substitute, Percutaneous Approach */
"0RGC3ZZ"  =  "1"  /* Fusion of Right Temporomandibular Joint, Percutaneous Approach */
"0RGC44Z"  =  "1"  /* Fusion of Right Temporomandibular Joint with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0RGC47Z"  =  "1"  /* Fusion of Right Temporomandibular Joint with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0RGC4JZ"  =  "1"  /* Fusion of Right Temporomandibular Joint with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0RGC4KZ"  =  "1"  /* Fusion of Right Temporomandibular Joint with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0RGC4ZZ"  =  "1"  /* Fusion of Right Temporomandibular Joint, Percutaneous Endoscopic Approach */
"0RGD04Z"  =  "1"  /* Fusion of Left Temporomandibular Joint with Internal Fixation Device, Open Approach */
"0RGD07Z"  =  "1"  /* Fusion of Left Temporomandibular Joint with Autologous Tissue Substitute, Open Approach */
"0RGD0JZ"  =  "1"  /* Fusion of Left Temporomandibular Joint with Synthetic Substitute, Open Approach */
"0RGD0KZ"  =  "1"  /* Fusion of Left Temporomandibular Joint with Nonautologous Tissue Substitute, Open Approach */
"0RGD0ZZ"  =  "1"  /* Fusion of Left Temporomandibular Joint, Open Approach */
"0RGD34Z"  =  "1"  /* Fusion of Left Temporomandibular Joint with Internal Fixation Device, Percutaneous Approach */
"0RGD37Z"  =  "1"  /* Fusion of Left Temporomandibular Joint with Autologous Tissue Substitute, Percutaneous Approach */
"0RGD3JZ"  =  "1"  /* Fusion of Left Temporomandibular Joint with Synthetic Substitute, Percutaneous Approach */
"0RGD3KZ"  =  "1"  /* Fusion of Left Temporomandibular Joint with Nonautologous Tissue Substitute, Percutaneous Approach */
"0RGD3ZZ"  =  "1"  /* Fusion of Left Temporomandibular Joint, Percutaneous Approach */
"0RGD44Z"  =  "1"  /* Fusion of Left Temporomandibular Joint with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0RGD47Z"  =  "1"  /* Fusion of Left Temporomandibular Joint with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0RGD4JZ"  =  "1"  /* Fusion of Left Temporomandibular Joint with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0RGD4KZ"  =  "1"  /* Fusion of Left Temporomandibular Joint with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0RGD4ZZ"  =  "1"  /* Fusion of Left Temporomandibular Joint, Percutaneous Endoscopic Approach */
"0RNC0ZZ"  =  "1"  /* Release Right Temporomandibular Joint, Open Approach */
"0RNC3ZZ"  =  "1"  /* Release Right Temporomandibular Joint, Percutaneous Approach */
"0RNC4ZZ"  =  "1"  /* Release Right Temporomandibular Joint, Percutaneous Endoscopic Approach */
"0RNCXZZ"  =  "1"  /* Release Right Temporomandibular Joint, External Approach */
"0RND0ZZ"  =  "1"  /* Release Left Temporomandibular Joint, Open Approach */
"0RND3ZZ"  =  "1"  /* Release Left Temporomandibular Joint, Percutaneous Approach */
"0RND4ZZ"  =  "1"  /* Release Left Temporomandibular Joint, Percutaneous Endoscopic Approach */
"0RNDXZZ"  =  "1"  /* Release Left Temporomandibular Joint, External Approach */
"0RPC04Z"  =  "1"  /* Removal of Internal Fixation Device from Right Temporomandibular Joint, Open Approach */
"0RPC34Z"  =  "1"  /* Removal of Internal Fixation Device from Right Temporomandibular Joint, Percutaneous Approach */
"0RPC44Z"  =  "1"  /* Removal of Internal Fixation Device from Right Temporomandibular Joint, Percutaneous Endoscopic Approach */
"0RPCX4Z"  =  "1"  /* Removal of Internal Fixation Device from Right Temporomandibular Joint, External Approach */
"0RPD04Z"  =  "1"  /* Removal of Internal Fixation Device from Left Temporomandibular Joint, Open Approach */
"0RPD34Z"  =  "1"  /* Removal of Internal Fixation Device from Left Temporomandibular Joint, Percutaneous Approach */
"0RPD44Z"  =  "1"  /* Removal of Internal Fixation Device from Left Temporomandibular Joint, Percutaneous Endoscopic Approach */
"0RPDX4Z"  =  "1"  /* Removal of Internal Fixation Device from Left Temporomandibular Joint, External Approach */
"0RQCXZZ"  =  "1"  /* Repair Right Temporomandibular Joint, External Approach */
"0RQDXZZ"  =  "1"  /* Repair Left Temporomandibular Joint, External Approach */
"0RSC04Z"  =  "1"  /* Reposition Right Temporomandibular Joint with Internal Fixation Device, Open Approach */
"0RSC0ZZ"  =  "1"  /* Reposition Right Temporomandibular Joint, Open Approach */
"0RSC34Z"  =  "1"  /* Reposition Right Temporomandibular Joint with Internal Fixation Device, Percutaneous Approach */
"0RSC3ZZ"  =  "1"  /* Reposition Right Temporomandibular Joint, Percutaneous Approach */
"0RSC44Z"  =  "1"  /* Reposition Right Temporomandibular Joint with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0RSC4ZZ"  =  "1"  /* Reposition Right Temporomandibular Joint, Percutaneous Endoscopic Approach */
"0RSCX4Z"  =  "1"  /* Reposition Right Temporomandibular Joint with Internal Fixation Device, External Approach */
"0RSCXZZ"  =  "1"  /* Reposition Right Temporomandibular Joint, External Approach */
"0RSD04Z"  =  "1"  /* Reposition Left Temporomandibular Joint with Internal Fixation Device, Open Approach */
"0RSD0ZZ"  =  "1"  /* Reposition Left Temporomandibular Joint, Open Approach */
"0RSD34Z"  =  "1"  /* Reposition Left Temporomandibular Joint with Internal Fixation Device, Percutaneous Approach */
"0RSD3ZZ"  =  "1"  /* Reposition Left Temporomandibular Joint, Percutaneous Approach */
"0RSD44Z"  =  "1"  /* Reposition Left Temporomandibular Joint with Internal Fixation Device, Percutaneous Endoscopic Approach */
"0RSD4ZZ"  =  "1"  /* Reposition Left Temporomandibular Joint, Percutaneous Endoscopic Approach */
"0RSDX4Z"  =  "1"  /* Reposition Left Temporomandibular Joint with Internal Fixation Device, External Approach */
"0RSDXZZ"  =  "1"  /* Reposition Left Temporomandibular Joint, External Approach */
"0RTC0ZZ"  =  "1"  /* Resection of Right Temporomandibular Joint, Open Approach */
"0RTD0ZZ"  =  "1"  /* Resection of Left Temporomandibular Joint, Open Approach */
"0W9200Z"  =  "1"  /* Drainage of Face with Drainage Device, Open Approach */
"0W920ZZ"  =  "1"  /* Drainage of Face, Open Approach */
"0W9230Z"  =  "1"  /* Drainage of Face with Drainage Device, Percutaneous Approach */
"0W923ZZ"  =  "1"  /* Drainage of Face, Percutaneous Approach */
"0W9240Z"  =  "1"  /* Drainage of Face with Drainage Device, Percutaneous Endoscopic Approach */
"0W924ZZ"  =  "1"  /* Drainage of Face, Percutaneous Endoscopic Approach */
"0W9300Z"  =  "1"  /* Drainage of Oral Cavity and Throat with Drainage Device, Open Approach */
"0W930ZZ"  =  "1"  /* Drainage of Oral Cavity and Throat, Open Approach */
"0W9330Z"  =  "1"  /* Drainage of Oral Cavity and Throat with Drainage Device, Percutaneous Approach */
"0W933ZZ"  =  "1"  /* Drainage of Oral Cavity and Throat, Percutaneous Approach */
"0W9340Z"  =  "1"  /* Drainage of Oral Cavity and Throat with Drainage Device, Percutaneous Endoscopic Approach */
"0W934ZZ"  =  "1"  /* Drainage of Oral Cavity and Throat, Percutaneous Endoscopic Approach */
"0W9400Z"  =  "1"  /* Drainage of Upper Jaw with Drainage Device, Open Approach */
"0W940ZZ"  =  "1"  /* Drainage of Upper Jaw, Open Approach */
"0W9430Z"  =  "1"  /* Drainage of Upper Jaw with Drainage Device, Percutaneous Approach */
"0W943ZZ"  =  "1"  /* Drainage of Upper Jaw, Percutaneous Approach */
"0W9440Z"  =  "1"  /* Drainage of Upper Jaw with Drainage Device, Percutaneous Endoscopic Approach */
"0W944ZZ"  =  "1"  /* Drainage of Upper Jaw, Percutaneous Endoscopic Approach */
"0W9500Z"  =  "1"  /* Drainage of Lower Jaw with Drainage Device, Open Approach */
"0W950ZZ"  =  "1"  /* Drainage of Lower Jaw, Open Approach */
"0W9530Z"  =  "1"  /* Drainage of Lower Jaw with Drainage Device, Percutaneous Approach */
"0W953ZZ"  =  "1"  /* Drainage of Lower Jaw, Percutaneous Approach */
"0W9540Z"  =  "1"  /* Drainage of Lower Jaw with Drainage Device, Percutaneous Endoscopic Approach */
"0W954ZZ"  =  "1"  /* Drainage of Lower Jaw, Percutaneous Endoscopic Approach */
"0WC30ZZ"  =  "1"  /* Extirpation of Matter from Oral Cavity and Throat, Open Approach */
"0WC33ZZ"  =  "1"  /* Extirpation of Matter from Oral Cavity and Throat, Percutaneous Approach */
"0WC34ZZ"  =  "1"  /* Extirpation of Matter from Oral Cavity and Throat, Percutaneous Endoscopic Approach */
"0WF30ZZ"  =  "1"  /* Fragmentation in Oral Cavity and Throat, Open Approach */
"0WF33ZZ"  =  "1"  /* Fragmentation in Oral Cavity and Throat, Percutaneous Approach */
"0WF34ZZ"  =  "1"  /* Fragmentation in Oral Cavity and Throat, Percutaneous Endoscopic Approach */
"0WH303Z"  =  "1"  /* Insertion of Infusion Device into Oral Cavity and Throat, Open Approach */
"0WH30YZ"  =  "1"  /* Insertion of Other Device into Oral Cavity and Throat, Open Approach */
"0WH333Z"  =  "1"  /* Insertion of Infusion Device into Oral Cavity and Throat, Percutaneous Approach */
"0WH33YZ"  =  "1"  /* Insertion of Other Device into Oral Cavity and Throat, Percutaneous Approach */
"0WH343Z"  =  "1"  /* Insertion of Infusion Device into Oral Cavity and Throat, Percutaneous Endoscopic Approach */
"0WH34YZ"  =  "1"  /* Insertion of Other Device into Oral Cavity and Throat, Percutaneous Endoscopic Approach */
OTHER  = '0' ;

 /* EXCLUDE: CRANIOFACIAL ANOMALIES */;
 /* ICD-10 CODE: */;

VALUE $CRANIID
"Q184"  =  "1"  /* Macrostomia */
"Q185"  =  "1"  /* Microstomia */
"Q189"  =  "1"  /* Congenital malformation of face and neck, unspecified */
"Q311"  =  "1"  /* Congenital subglottic stenosis */
"Q312"  =  "1"  /* Laryngeal hypoplasia */
"Q313"  =  "1"  /* Laryngocele */
"Q315"  =  "1"  /* Congenital laryngomalacia */
"Q318"  =  "1"  /* Other congenital malformations of larynx */
"Q319"  =  "1"  /* Congenital malformation of larynx, unspecified */
"Q320"  =  "1"  /* Congenital tracheomalacia */
"Q321"  =  "1"  /* Other congenital malformations of trachea */
"Q322"  =  "1"  /* Congenital bronchomalacia */
"Q323"  =  "1"  /* Congenital stenosis of bronchus */
"Q324"  =  "1"  /* Other congenital malformations of bronchus */
"Q750"  =  "1"  /* Craniosynostosis */
"Q751"  =  "1"  /* Craniofacial dysostosis */
"Q752"  =  "1"  /* Hypertelorism */
"Q753"  =  "1"  /* Macrocephaly */
"Q754"  =  "1"  /* Mandibulofacial dysostosis */
"Q755"  =  "1"  /* Oculomandibular dysostosis */
"Q758"  =  "1"  /* Other specified congenital malformations of skull and face bones */
"Q759"  =  "1"  /* Congenital malformation of skull and face bones, unspecified */
"Q870"  =  "1"  /* Congenital malformation syndromes predominantly affecting facial appearance */
OTHER  = "0" ;


 /* ------------------------------- */
 /* POST-OPERATIVE SEPSIS           */
 /*   PROVIDER INDICATOR / PSI # 10 */
 /* ------------------------------- */

 /* SEPTICEMIA */
 /* ICD-10-CM DIAGNOSIS CODES: */

/* VALUE $SEPTIID per UCD - */
/* This concept is no longer relevant with the use */
/* of ICD-9-CM code 785.59 because another */
/* concept was revised to exclude the ICD-9-CM */
/* code 785.59.  This was tested when the ICD-9-CM */
/* code 785.52 septic shock was created 10/1/2003 */


 /* ICD-10-CM DIAGNOSIS CODES: */

VALUE $SEPTI2D
"A021"  =  "1"  /* Salmonella sepsis */
"A227"  =  "1"  /* Anthrax sepsis */
"A267"  =  "1"  /* Erysipelothrix sepsis */
"A327"  =  "1"  /* Listerial sepsis */
"A400"  =  "1"  /* Sepsis due to streptococcus, group A */
"A401"  =  "1"  /* Sepsis due to streptococcus, group B */
"A403"  =  "1"  /* Sepsis due to Streptococcus pneumoniae */
"A408"  =  "1"  /* Other streptococcal sepsis */
"A409"  =  "1"  /* Streptococcal sepsis, unspecified */
"A4101"  =  "1"  /* Sepsis due to Methicillin susceptible Staphylococcus aureus */
"A4102"  =  "1"  /* Sepsis due to Methicillin resistant Staphylococcus aureus */
"A411"  =  "1"  /* Sepsis due to other specified staphylococcus */
"A412"  =  "1"  /* Sepsis due to unspecified staphylococcus */
"A413"  =  "1"  /* Sepsis due to Hemophilus influenzae */
"A414"  =  "1"  /* Sepsis due to anaerobes */
"A4150"  =  "1"  /* Gram-negative sepsis, unspecified */
"A4151"  =  "1"  /* Sepsis due to Escherichia coli [E. coli] */
"A4152"  =  "1"  /* Sepsis due to Pseudomonas */
"A4153"  =  "1"  /* Sepsis due to Serratia */
"A4159"  =  "1"  /* Other Gram-negative sepsis */
"A4181"  =  "1"  /* Sepsis due to Enterococcus */
"A4189"  =  "1"  /* Other specified sepsis */
"A419"  =  "1"  /* Sepsis, unspecified organism */
"A427"  =  "1"  /* Actinomycotic sepsis */
"A5486"  =  "1"  /* Gonococcal sepsis */
"B377"  =  "1"  /* Candidal sepsis */
"R6520"  =  "1"  /* Severe sepsis without septic shock */
"R6521"  =  "1"  /* Severe sepsis with septic shock */
"T8110XA"  =  "1"  /* Postprocedural shock unspecified, initial encounter */
"T8112XA"  =  "1"  /* Postprocedural septic shock, initial encounter */
OTHER   = "0" ;


 /* DRGS SURGICAL CLASS =1, TYPE OF DRGS = CLEAN */

VALUE DRG1C
0101 = '1'
0102 = '1'
0103 = '1'
0104 = '1'
0105 = '1'
0106 = '1'
0107 = '1'
0108 = '1'
0201 = '1'
0202 = '1'
0203 = '1'
0301 = '1'
0302 = '1'
0303 = '1'
0304 = '1'
0305 = '1'
0501 = '1'
0502 = '1'
0503 = '1'
0504 = '1'
0505 = '1'
0506 = '1'
0507 = '1'
0508 = '1'
0509 = '1'
0510 = '1'
0511 = '1'
0513 = '1'
0514 = '1'
0515 = '1'
0516 = '1'
0517 = '1'
0518 = '1'
0519 = '1'
0540 = '1'
0599 = '1'
0609 = '1'
0801 = '1'
0802 = '1'
0803 = '1'
0804 = '1'
0805 = '1'
0806 = '1'
0807 = '1'
0808 = '1'
0809 = '1'
0810 = '1'
0811 = '1'
0812 = '1'
0813 = '1'
0814 = '1'
0815 = '1'
0816 = '1'
0817 = '1'
0818 = '1'
0819 = '1'
0820 = '1'
0821 = '1'
0822 = '1'
0823 = '1'
0824 = '1'
0825 = '1'
0826 = '1'
0899 = '1'
0904 = '1'
0905 = '1'
1001 = '1'
1002 = '1'
1004 = '1'
1005 = '1'
1006 = '1'
1203 = '1'
1601 = '1'
1602 = '1'
7701 = '1'
7705 = '1'
OTHER = '0' ;

/* DRGS SURGICAL CLASS =2, TYPE OF DRGS = CLEAN CONTAMINATED */

VALUE DRG2C
0113 = '1'
0401 = '1'
0402 = '1'
0601 = '1'
0602 = '1'
0603 = '1'
0604 = '1'
0606 = '1'
0607 = '1'
0608 = '1'
0611 = '1'
0701 = '1'
0702 = '1'
0703 = '1'
0704 = '1'
0705 = '1'
0706 = '1'
0707 = '1'
0902 = '1'
0903 = '1'
1003 = '1'
1101 = '1'
1102 = '1'
1103 = '1'
1104 = '1'
1105 = '1'
1106 = '1'
1107 = '1'
1108 = '1'
1109 = '1'
1201 = '1'
1202 = '1'
1204 = '1'
1205 = '1'
1206 = '1'
1301 = '1'
1302 = '1'
1303 = '1'
1304 = '1'
1305 = '1'
1306 = '1'
1307 = '1'
1308 = '1'
1401 = '1'
1402 = '1'
1403 = '1'
1404 = '1'
1405 = '1'
7702 = '1'
7703 = '1'
7704 = '1'
7706 = '1'
7799 = '1'
8899 = '1'
OTHER = '0' ;

/* DRGS SURGICAL CLASS =3, TYPE OF DRGS = POTENTIALLY CONTAMINATED */

VALUE DRG3C
0901 = '1'
2101 = '1'
2102 = '1'
2103 = '1'
2104 = '1'
2201 = '1'
2210 = '1'
2406 = '1'
2407 = '1'
2408 = '1'
OTHER = '0' ;

/* DRGS CLASS =4, TYPE OF DRGS = LIKELY INFECTED                    */

VALUE DRG4C
0605 = '1'
1801 = '1'
1802 = '1'
1899 = '1'
OTHER = '0' ;

/* DRGS CLASS =9, TYPE OF DRGS = SURGICAL PROCEDURES NOT SPECIFIED  */

VALUE DRG9C
1707 = '1'
1708 = '1'
1709 = '1'
1710 = '1'
1799 = '1'
1909 = '1'
2301 = '1'
2501 = '1'
OTHER = '0' ;


 /* -------------------------------- */
 /* POST-OPERATIVE WOUND DEHISCENCE  */
 /*   PROVIDER INDICATOR / PDI # 11  */
 /*   AREA     INDICATOR / PDI # TBD */
 /* -------------------------------- */

 /* ABDOMINOPELVIC */;
 /* ICD-10-CM PROCEDURE CODES: */;

VALUE $ABDOMIP
"008Q0ZZ"  =  "1"  /* Division of Vagus Nerve, Open Approach */
"008Q3ZZ"  =  "1"  /* Division of Vagus Nerve, Percutaneous Approach */
"008Q4ZZ"  =  "1"  /* Division of Vagus Nerve, Percutaneous Endoscopic Approach */
"015P0ZZ"  =  "1"  /* Destruction of Sacral Sympathetic Nerve, Open Approach */
"015P3ZZ"  =  "1"  /* Destruction of Sacral Sympathetic Nerve, Percutaneous Approach */
"015P4ZZ"  =  "1"  /* Destruction of Sacral Sympathetic Nerve, Percutaneous Endoscopic Approach */
"0410090"  =  "1"  /* Bypass Abdominal Aorta to Abdominal Aorta with Autologous Venous Tissue, Open Approach */
"0410091"  =  "1"  /* Bypass Abdominal Aorta to Celiac Artery with Autologous Venous Tissue, Open Approach */
"0410092"  =  "1"  /* Bypass Abdominal Aorta to Mesenteric Artery with Autologous Venous Tissue, Open Approach */
"0410093"  =  "1"  /* Bypass Abdominal Aorta to Right Renal Artery with Autologous Venous Tissue, Open Approach */
"0410094"  =  "1"  /* Bypass Abdominal Aorta to Left Renal Artery with Autologous Venous Tissue, Open Approach */
"0410095"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Renal Artery with Autologous Venous Tissue, Open Approach */
"0410096"  =  "1"  /* Bypass Abdominal Aorta to Right Common Iliac Artery with Autologous Venous Tissue, Open Approach */
"0410097"  =  "1"  /* Bypass Abdominal Aorta to Left Common Iliac Artery with Autologous Venous Tissue, Open Approach */
"0410098"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Common Iliac Arteries with Autologous Venous Tissue, Open Approach */
"0410099"  =  "1"  /* Bypass Abdominal Aorta to Right Internal Iliac Artery with Autologous Venous Tissue, Open Approach */
"041009B"  =  "1"  /* Bypass Abdominal Aorta to Left Internal Iliac Artery with Autologous Venous Tissue, Open Approach */
"041009C"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Open Approach */
"041009D"  =  "1"  /* Bypass Abdominal Aorta to Right External Iliac Artery with Autologous Venous Tissue, Open Approach */
"041009F"  =  "1"  /* Bypass Abdominal Aorta to Left External Iliac Artery with Autologous Venous Tissue, Open Approach */
"041009G"  =  "1"  /* Bypass Abdominal Aorta to Bilateral External Iliac Arteries with Autologous Venous Tissue, Open Approach */
"041009H"  =  "1"  /* Bypass Abdominal Aorta to Right Femoral Artery with Autologous Venous Tissue, Open Approach */
"041009J"  =  "1"  /* Bypass Abdominal Aorta to Left Femoral Artery with Autologous Venous Tissue, Open Approach */
"041009K"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Femoral Arteries with Autologous Venous Tissue, Open Approach */
"041009Q"  =  "1"  /* Bypass Abdominal Aorta to Lower Extremity Artery with Autologous Venous Tissue, Open Approach */
"041009R"  =  "1"  /* Bypass Abdominal Aorta to Lower Artery with Autologous Venous Tissue, Open Approach */
"04100A0"  =  "1"  /* Bypass Abdominal Aorta to Abdominal Aorta with Autologous Arterial Tissue, Open Approach */
"04100A1"  =  "1"  /* Bypass Abdominal Aorta to Celiac Artery with Autologous Arterial Tissue, Open Approach */
"04100A2"  =  "1"  /* Bypass Abdominal Aorta to Mesenteric Artery with Autologous Arterial Tissue, Open Approach */
"04100A3"  =  "1"  /* Bypass Abdominal Aorta to Right Renal Artery with Autologous Arterial Tissue, Open Approach */
"04100A4"  =  "1"  /* Bypass Abdominal Aorta to Left Renal Artery with Autologous Arterial Tissue, Open Approach */
"04100A5"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Renal Artery with Autologous Arterial Tissue, Open Approach */
"04100A6"  =  "1"  /* Bypass Abdominal Aorta to Right Common Iliac Artery with Autologous Arterial Tissue, Open Approach */
"04100A7"  =  "1"  /* Bypass Abdominal Aorta to Left Common Iliac Artery with Autologous Arterial Tissue, Open Approach */
"04100A8"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Common Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"04100A9"  =  "1"  /* Bypass Abdominal Aorta to Right Internal Iliac Artery with Autologous Arterial Tissue, Open Approach */
"04100AB"  =  "1"  /* Bypass Abdominal Aorta to Left Internal Iliac Artery with Autologous Arterial Tissue, Open Approach */
"04100AC"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"04100AD"  =  "1"  /* Bypass Abdominal Aorta to Right External Iliac Artery with Autologous Arterial Tissue, Open Approach */
"04100AF"  =  "1"  /* Bypass Abdominal Aorta to Left External Iliac Artery with Autologous Arterial Tissue, Open Approach */
"04100AG"  =  "1"  /* Bypass Abdominal Aorta to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"04100AH"  =  "1"  /* Bypass Abdominal Aorta to Right Femoral Artery with Autologous Arterial Tissue, Open Approach */
"04100AJ"  =  "1"  /* Bypass Abdominal Aorta to Left Femoral Artery with Autologous Arterial Tissue, Open Approach */
"04100AK"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Femoral Arteries with Autologous Arterial Tissue, Open Approach */
"04100AQ"  =  "1"  /* Bypass Abdominal Aorta to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach */
"04100AR"  =  "1"  /* Bypass Abdominal Aorta to Lower Artery with Autologous Arterial Tissue, Open Approach */
"04100J0"  =  "1"  /* Bypass Abdominal Aorta to Abdominal Aorta with Synthetic Substitute, Open Approach */
"04100J1"  =  "1"  /* Bypass Abdominal Aorta to Celiac Artery with Synthetic Substitute, Open Approach */
"04100J2"  =  "1"  /* Bypass Abdominal Aorta to Mesenteric Artery with Synthetic Substitute, Open Approach */
"04100J3"  =  "1"  /* Bypass Abdominal Aorta to Right Renal Artery with Synthetic Substitute, Open Approach */
"04100J4"  =  "1"  /* Bypass Abdominal Aorta to Left Renal Artery with Synthetic Substitute, Open Approach */
"04100J5"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Renal Artery with Synthetic Substitute, Open Approach */
"04100J6"  =  "1"  /* Bypass Abdominal Aorta to Right Common Iliac Artery with Synthetic Substitute, Open Approach */
"04100J7"  =  "1"  /* Bypass Abdominal Aorta to Left Common Iliac Artery with Synthetic Substitute, Open Approach */
"04100J8"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Common Iliac Arteries with Synthetic Substitute, Open Approach */
"04100J9"  =  "1"  /* Bypass Abdominal Aorta to Right Internal Iliac Artery with Synthetic Substitute, Open Approach */
"04100JB"  =  "1"  /* Bypass Abdominal Aorta to Left Internal Iliac Artery with Synthetic Substitute, Open Approach */
"04100JC"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Internal Iliac Arteries with Synthetic Substitute, Open Approach */
"04100JD"  =  "1"  /* Bypass Abdominal Aorta to Right External Iliac Artery with Synthetic Substitute, Open Approach */
"04100JF"  =  "1"  /* Bypass Abdominal Aorta to Left External Iliac Artery with Synthetic Substitute, Open Approach */
"04100JG"  =  "1"  /* Bypass Abdominal Aorta to Bilateral External Iliac Arteries with Synthetic Substitute, Open Approach */
"04100JH"  =  "1"  /* Bypass Abdominal Aorta to Right Femoral Artery with Synthetic Substitute, Open Approach */
"04100JJ"  =  "1"  /* Bypass Abdominal Aorta to Left Femoral Artery with Synthetic Substitute, Open Approach */
"04100JK"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Femoral Arteries with Synthetic Substitute, Open Approach */
"04100JQ"  =  "1"  /* Bypass Abdominal Aorta to Lower Extremity Artery with Synthetic Substitute, Open Approach */
"04100JR"  =  "1"  /* Bypass Abdominal Aorta to Lower Artery with Synthetic Substitute, Open Approach */
"04100K0"  =  "1"  /* Bypass Abdominal Aorta to Abdominal Aorta with Nonautologous Tissue Substitute, Open Approach */
"04100K1"  =  "1"  /* Bypass Abdominal Aorta to Celiac Artery with Nonautologous Tissue Substitute, Open Approach */
"04100K2"  =  "1"  /* Bypass Abdominal Aorta to Mesenteric Artery with Nonautologous Tissue Substitute, Open Approach */
"04100K3"  =  "1"  /* Bypass Abdominal Aorta to Right Renal Artery with Nonautologous Tissue Substitute, Open Approach */
"04100K4"  =  "1"  /* Bypass Abdominal Aorta to Left Renal Artery with Nonautologous Tissue Substitute, Open Approach */
"04100K5"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Renal Artery with Nonautologous Tissue Substitute, Open Approach */
"04100K6"  =  "1"  /* Bypass Abdominal Aorta to Right Common Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"04100K7"  =  "1"  /* Bypass Abdominal Aorta to Left Common Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"04100K8"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Common Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"04100K9"  =  "1"  /* Bypass Abdominal Aorta to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"04100KB"  =  "1"  /* Bypass Abdominal Aorta to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"04100KC"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"04100KD"  =  "1"  /* Bypass Abdominal Aorta to Right External Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"04100KF"  =  "1"  /* Bypass Abdominal Aorta to Left External Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"04100KG"  =  "1"  /* Bypass Abdominal Aorta to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"04100KH"  =  "1"  /* Bypass Abdominal Aorta to Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach */
"04100KJ"  =  "1"  /* Bypass Abdominal Aorta to Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach */
"04100KK"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Open Approach */
"04100KQ"  =  "1"  /* Bypass Abdominal Aorta to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach */
"04100KR"  =  "1"  /* Bypass Abdominal Aorta to Lower Artery with Nonautologous Tissue Substitute, Open Approach */
"04100Z0"  =  "1"  /* Bypass Abdominal Aorta to Abdominal Aorta, Open Approach */
"04100Z1"  =  "1"  /* Bypass Abdominal Aorta to Celiac Artery, Open Approach */
"04100Z2"  =  "1"  /* Bypass Abdominal Aorta to Mesenteric Artery, Open Approach */
"04100Z3"  =  "1"  /* Bypass Abdominal Aorta to Right Renal Artery, Open Approach */
"04100Z4"  =  "1"  /* Bypass Abdominal Aorta to Left Renal Artery, Open Approach */
"04100Z5"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Renal Artery, Open Approach */
"04100Z6"  =  "1"  /* Bypass Abdominal Aorta to Right Common Iliac Artery, Open Approach */
"04100Z7"  =  "1"  /* Bypass Abdominal Aorta to Left Common Iliac Artery, Open Approach */
"04100Z8"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Common Iliac Arteries, Open Approach */
"04100Z9"  =  "1"  /* Bypass Abdominal Aorta to Right Internal Iliac Artery, Open Approach */
"04100ZB"  =  "1"  /* Bypass Abdominal Aorta to Left Internal Iliac Artery, Open Approach */
"04100ZC"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Internal Iliac Arteries, Open Approach */
"04100ZD"  =  "1"  /* Bypass Abdominal Aorta to Right External Iliac Artery, Open Approach */
"04100ZF"  =  "1"  /* Bypass Abdominal Aorta to Left External Iliac Artery, Open Approach */
"04100ZG"  =  "1"  /* Bypass Abdominal Aorta to Bilateral External Iliac Arteries, Open Approach */
"04100ZH"  =  "1"  /* Bypass Abdominal Aorta to Right Femoral Artery, Open Approach */
"04100ZJ"  =  "1"  /* Bypass Abdominal Aorta to Left Femoral Artery, Open Approach */
"04100ZK"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Femoral Arteries, Open Approach */
"04100ZQ"  =  "1"  /* Bypass Abdominal Aorta to Lower Extremity Artery, Open Approach */
"04100ZR"  =  "1"  /* Bypass Abdominal Aorta to Lower Artery, Open Approach */
"0410490"  =  "1"  /* Bypass Abdominal Aorta to Abdominal Aorta with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0410491"  =  "1"  /* Bypass Abdominal Aorta to Celiac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0410492"  =  "1"  /* Bypass Abdominal Aorta to Mesenteric Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0410493"  =  "1"  /* Bypass Abdominal Aorta to Right Renal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0410494"  =  "1"  /* Bypass Abdominal Aorta to Left Renal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0410495"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Renal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0410496"  =  "1"  /* Bypass Abdominal Aorta to Right Common Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0410497"  =  "1"  /* Bypass Abdominal Aorta to Left Common Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0410498"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Common Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0410499"  =  "1"  /* Bypass Abdominal Aorta to Right Internal Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041049B"  =  "1"  /* Bypass Abdominal Aorta to Left Internal Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041049C"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041049D"  =  "1"  /* Bypass Abdominal Aorta to Right External Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041049F"  =  "1"  /* Bypass Abdominal Aorta to Left External Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041049G"  =  "1"  /* Bypass Abdominal Aorta to Bilateral External Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041049H"  =  "1"  /* Bypass Abdominal Aorta to Right Femoral Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041049J"  =  "1"  /* Bypass Abdominal Aorta to Left Femoral Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041049K"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Femoral Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041049Q"  =  "1"  /* Bypass Abdominal Aorta to Lower Extremity Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041049R"  =  "1"  /* Bypass Abdominal Aorta to Lower Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"04104A0"  =  "1"  /* Bypass Abdominal Aorta to Abdominal Aorta with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104A1"  =  "1"  /* Bypass Abdominal Aorta to Celiac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104A2"  =  "1"  /* Bypass Abdominal Aorta to Mesenteric Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104A3"  =  "1"  /* Bypass Abdominal Aorta to Right Renal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104A4"  =  "1"  /* Bypass Abdominal Aorta to Left Renal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104A5"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Renal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104A6"  =  "1"  /* Bypass Abdominal Aorta to Right Common Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104A7"  =  "1"  /* Bypass Abdominal Aorta to Left Common Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104A8"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Common Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104A9"  =  "1"  /* Bypass Abdominal Aorta to Right Internal Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104AB"  =  "1"  /* Bypass Abdominal Aorta to Left Internal Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104AC"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104AD"  =  "1"  /* Bypass Abdominal Aorta to Right External Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104AF"  =  "1"  /* Bypass Abdominal Aorta to Left External Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104AG"  =  "1"  /* Bypass Abdominal Aorta to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104AH"  =  "1"  /* Bypass Abdominal Aorta to Right Femoral Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104AJ"  =  "1"  /* Bypass Abdominal Aorta to Left Femoral Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104AK"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Femoral Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104AQ"  =  "1"  /* Bypass Abdominal Aorta to Lower Extremity Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104AR"  =  "1"  /* Bypass Abdominal Aorta to Lower Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04104J0"  =  "1"  /* Bypass Abdominal Aorta to Abdominal Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104J1"  =  "1"  /* Bypass Abdominal Aorta to Celiac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104J2"  =  "1"  /* Bypass Abdominal Aorta to Mesenteric Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104J3"  =  "1"  /* Bypass Abdominal Aorta to Right Renal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104J4"  =  "1"  /* Bypass Abdominal Aorta to Left Renal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104J5"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Renal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104J6"  =  "1"  /* Bypass Abdominal Aorta to Right Common Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104J7"  =  "1"  /* Bypass Abdominal Aorta to Left Common Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104J8"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Common Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104J9"  =  "1"  /* Bypass Abdominal Aorta to Right Internal Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104JB"  =  "1"  /* Bypass Abdominal Aorta to Left Internal Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104JC"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Internal Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104JD"  =  "1"  /* Bypass Abdominal Aorta to Right External Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104JF"  =  "1"  /* Bypass Abdominal Aorta to Left External Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104JG"  =  "1"  /* Bypass Abdominal Aorta to Bilateral External Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104JH"  =  "1"  /* Bypass Abdominal Aorta to Right Femoral Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104JJ"  =  "1"  /* Bypass Abdominal Aorta to Left Femoral Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104JK"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Femoral Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104JQ"  =  "1"  /* Bypass Abdominal Aorta to Lower Extremity Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104JR"  =  "1"  /* Bypass Abdominal Aorta to Lower Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04104K0"  =  "1"  /* Bypass Abdominal Aorta to Abdominal Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104K1"  =  "1"  /* Bypass Abdominal Aorta to Celiac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104K2"  =  "1"  /* Bypass Abdominal Aorta to Mesenteric Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104K3"  =  "1"  /* Bypass Abdominal Aorta to Right Renal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104K4"  =  "1"  /* Bypass Abdominal Aorta to Left Renal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104K5"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Renal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104K6"  =  "1"  /* Bypass Abdominal Aorta to Right Common Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104K7"  =  "1"  /* Bypass Abdominal Aorta to Left Common Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104K8"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Common Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104K9"  =  "1"  /* Bypass Abdominal Aorta to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104KB"  =  "1"  /* Bypass Abdominal Aorta to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104KC"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104KD"  =  "1"  /* Bypass Abdominal Aorta to Right External Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104KF"  =  "1"  /* Bypass Abdominal Aorta to Left External Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104KG"  =  "1"  /* Bypass Abdominal Aorta to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104KH"  =  "1"  /* Bypass Abdominal Aorta to Right Femoral Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104KJ"  =  "1"  /* Bypass Abdominal Aorta to Left Femoral Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104KK"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104KQ"  =  "1"  /* Bypass Abdominal Aorta to Lower Extremity Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104KR"  =  "1"  /* Bypass Abdominal Aorta to Lower Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04104Z0"  =  "1"  /* Bypass Abdominal Aorta to Abdominal Aorta, Percutaneous Endoscopic Approach */
"04104Z1"  =  "1"  /* Bypass Abdominal Aorta to Celiac Artery, Percutaneous Endoscopic Approach */
"04104Z2"  =  "1"  /* Bypass Abdominal Aorta to Mesenteric Artery, Percutaneous Endoscopic Approach */
"04104Z3"  =  "1"  /* Bypass Abdominal Aorta to Right Renal Artery, Percutaneous Endoscopic Approach */
"04104Z4"  =  "1"  /* Bypass Abdominal Aorta to Left Renal Artery, Percutaneous Endoscopic Approach */
"04104Z5"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Renal Artery, Percutaneous Endoscopic Approach */
"04104Z6"  =  "1"  /* Bypass Abdominal Aorta to Right Common Iliac Artery, Percutaneous Endoscopic Approach */
"04104Z7"  =  "1"  /* Bypass Abdominal Aorta to Left Common Iliac Artery, Percutaneous Endoscopic Approach */
"04104Z8"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Common Iliac Arteries, Percutaneous Endoscopic Approach */
"04104Z9"  =  "1"  /* Bypass Abdominal Aorta to Right Internal Iliac Artery, Percutaneous Endoscopic Approach */
"04104ZB"  =  "1"  /* Bypass Abdominal Aorta to Left Internal Iliac Artery, Percutaneous Endoscopic Approach */
"04104ZC"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Internal Iliac Arteries, Percutaneous Endoscopic Approach */
"04104ZD"  =  "1"  /* Bypass Abdominal Aorta to Right External Iliac Artery, Percutaneous Endoscopic Approach */
"04104ZF"  =  "1"  /* Bypass Abdominal Aorta to Left External Iliac Artery, Percutaneous Endoscopic Approach */
"04104ZG"  =  "1"  /* Bypass Abdominal Aorta to Bilateral External Iliac Arteries, Percutaneous Endoscopic Approach */
"04104ZH"  =  "1"  /* Bypass Abdominal Aorta to Right Femoral Artery, Percutaneous Endoscopic Approach */
"04104ZJ"  =  "1"  /* Bypass Abdominal Aorta to Left Femoral Artery, Percutaneous Endoscopic Approach */
"04104ZK"  =  "1"  /* Bypass Abdominal Aorta to Bilateral Femoral Arteries, Percutaneous Endoscopic Approach */
"04104ZQ"  =  "1"  /* Bypass Abdominal Aorta to Lower Extremity Artery, Percutaneous Endoscopic Approach */
"04104ZR"  =  "1"  /* Bypass Abdominal Aorta to Lower Artery, Percutaneous Endoscopic Approach */
"0414093"  =  "1"  /* Bypass Splenic Artery to Right Renal Artery with Autologous Venous Tissue, Open Approach */
"0414094"  =  "1"  /* Bypass Splenic Artery to Left Renal Artery with Autologous Venous Tissue, Open Approach */
"0414095"  =  "1"  /* Bypass Splenic Artery to Bilateral Renal Artery with Autologous Venous Tissue, Open Approach */
"04140A3"  =  "1"  /* Bypass Splenic Artery to Right Renal Artery with Autologous Arterial Tissue, Open Approach */
"04140A4"  =  "1"  /* Bypass Splenic Artery to Left Renal Artery with Autologous Arterial Tissue, Open Approach */
"04140A5"  =  "1"  /* Bypass Splenic Artery to Bilateral Renal Artery with Autologous Arterial Tissue, Open Approach */
"04140J3"  =  "1"  /* Bypass Splenic Artery to Right Renal Artery with Synthetic Substitute, Open Approach */
"04140J4"  =  "1"  /* Bypass Splenic Artery to Left Renal Artery with Synthetic Substitute, Open Approach */
"04140J5"  =  "1"  /* Bypass Splenic Artery to Bilateral Renal Artery with Synthetic Substitute, Open Approach */
"04140K3"  =  "1"  /* Bypass Splenic Artery to Right Renal Artery with Nonautologous Tissue Substitute, Open Approach */
"04140K4"  =  "1"  /* Bypass Splenic Artery to Left Renal Artery with Nonautologous Tissue Substitute, Open Approach */
"04140K5"  =  "1"  /* Bypass Splenic Artery to Bilateral Renal Artery with Nonautologous Tissue Substitute, Open Approach */
"04140Z3"  =  "1"  /* Bypass Splenic Artery to Right Renal Artery, Open Approach */
"04140Z4"  =  "1"  /* Bypass Splenic Artery to Left Renal Artery, Open Approach */
"04140Z5"  =  "1"  /* Bypass Splenic Artery to Bilateral Renal Artery, Open Approach */
"0414493"  =  "1"  /* Bypass Splenic Artery to Right Renal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0414494"  =  "1"  /* Bypass Splenic Artery to Left Renal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0414495"  =  "1"  /* Bypass Splenic Artery to Bilateral Renal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"04144A3"  =  "1"  /* Bypass Splenic Artery to Right Renal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04144A4"  =  "1"  /* Bypass Splenic Artery to Left Renal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04144A5"  =  "1"  /* Bypass Splenic Artery to Bilateral Renal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"04144J3"  =  "1"  /* Bypass Splenic Artery to Right Renal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04144J4"  =  "1"  /* Bypass Splenic Artery to Left Renal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04144J5"  =  "1"  /* Bypass Splenic Artery to Bilateral Renal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04144K3"  =  "1"  /* Bypass Splenic Artery to Right Renal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04144K4"  =  "1"  /* Bypass Splenic Artery to Left Renal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04144K5"  =  "1"  /* Bypass Splenic Artery to Bilateral Renal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04144Z3"  =  "1"  /* Bypass Splenic Artery to Right Renal Artery, Percutaneous Endoscopic Approach */
"04144Z4"  =  "1"  /* Bypass Splenic Artery to Left Renal Artery, Percutaneous Endoscopic Approach */
"04144Z5"  =  "1"  /* Bypass Splenic Artery to Bilateral Renal Artery, Percutaneous Endoscopic Approach */
"041C090"  =  "1"  /* Bypass Right Common Iliac Artery to Abdominal Aorta with Autologous Venous Tissue, Open Approach */
"041C091"  =  "1"  /* Bypass Right Common Iliac Artery to Celiac Artery with Autologous Venous Tissue, Open Approach */
"041C092"  =  "1"  /* Bypass Right Common Iliac Artery to Mesenteric Artery with Autologous Venous Tissue, Open Approach */
"041C093"  =  "1"  /* Bypass Right Common Iliac Artery to Right Renal Artery with Autologous Venous Tissue, Open Approach */
"041C094"  =  "1"  /* Bypass Right Common Iliac Artery to Left Renal Artery with Autologous Venous Tissue, Open Approach */
"041C095"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Renal Artery with Autologous Venous Tissue, Open Approach */
"041C096"  =  "1"  /* Bypass Right Common Iliac Artery to Right Common Iliac Artery with Autologous Venous Tissue, Open Approach */
"041C097"  =  "1"  /* Bypass Right Common Iliac Artery to Left Common Iliac Artery with Autologous Venous Tissue, Open Approach */
"041C098"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Common Iliac Arteries with Autologous Venous Tissue, Open Approach */
"041C099"  =  "1"  /* Bypass Right Common Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Open Approach */
"041C09B"  =  "1"  /* Bypass Right Common Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Open Approach */
"041C09C"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Open Approach */
"041C09D"  =  "1"  /* Bypass Right Common Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Open Approach */
"041C09F"  =  "1"  /* Bypass Right Common Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Open Approach */
"041C09G"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Open Approach */
"041C09H"  =  "1"  /* Bypass Right Common Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Open Approach */
"041C09J"  =  "1"  /* Bypass Right Common Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Open Approach */
"041C09K"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Open Approach */
"041C09Q"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Open Approach */
"041C09R"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Artery with Autologous Venous Tissue, Open Approach */
"041C0A0"  =  "1"  /* Bypass Right Common Iliac Artery to Abdominal Aorta with Autologous Arterial Tissue, Open Approach */
"041C0A1"  =  "1"  /* Bypass Right Common Iliac Artery to Celiac Artery with Autologous Arterial Tissue, Open Approach */
"041C0A2"  =  "1"  /* Bypass Right Common Iliac Artery to Mesenteric Artery with Autologous Arterial Tissue, Open Approach */
"041C0A3"  =  "1"  /* Bypass Right Common Iliac Artery to Right Renal Artery with Autologous Arterial Tissue, Open Approach */
"041C0A4"  =  "1"  /* Bypass Right Common Iliac Artery to Left Renal Artery with Autologous Arterial Tissue, Open Approach */
"041C0A5"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Renal Artery with Autologous Arterial Tissue, Open Approach */
"041C0A6"  =  "1"  /* Bypass Right Common Iliac Artery to Right Common Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041C0A7"  =  "1"  /* Bypass Right Common Iliac Artery to Left Common Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041C0A8"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Common Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"041C0A9"  =  "1"  /* Bypass Right Common Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041C0AB"  =  "1"  /* Bypass Right Common Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041C0AC"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"041C0AD"  =  "1"  /* Bypass Right Common Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041C0AF"  =  "1"  /* Bypass Right Common Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041C0AG"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"041C0AH"  =  "1"  /* Bypass Right Common Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Open Approach */
"041C0AJ"  =  "1"  /* Bypass Right Common Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Open Approach */
"041C0AK"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Open Approach */
"041C0AQ"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach */
"041C0AR"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Artery with Autologous Arterial Tissue, Open Approach */
"041C0J0"  =  "1"  /* Bypass Right Common Iliac Artery to Abdominal Aorta with Synthetic Substitute, Open Approach */
"041C0J1"  =  "1"  /* Bypass Right Common Iliac Artery to Celiac Artery with Synthetic Substitute, Open Approach */
"041C0J2"  =  "1"  /* Bypass Right Common Iliac Artery to Mesenteric Artery with Synthetic Substitute, Open Approach */
"041C0J3"  =  "1"  /* Bypass Right Common Iliac Artery to Right Renal Artery with Synthetic Substitute, Open Approach */
"041C0J4"  =  "1"  /* Bypass Right Common Iliac Artery to Left Renal Artery with Synthetic Substitute, Open Approach */
"041C0J5"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Renal Artery with Synthetic Substitute, Open Approach */
"041C0J6"  =  "1"  /* Bypass Right Common Iliac Artery to Right Common Iliac Artery with Synthetic Substitute, Open Approach */
"041C0J7"  =  "1"  /* Bypass Right Common Iliac Artery to Left Common Iliac Artery with Synthetic Substitute, Open Approach */
"041C0J8"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Common Iliac Arteries with Synthetic Substitute, Open Approach */
"041C0J9"  =  "1"  /* Bypass Right Common Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Open Approach */
"041C0JB"  =  "1"  /* Bypass Right Common Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Open Approach */
"041C0JC"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Open Approach */
"041C0JD"  =  "1"  /* Bypass Right Common Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Open Approach */
"041C0JF"  =  "1"  /* Bypass Right Common Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Open Approach */
"041C0JG"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Open Approach */
"041C0JH"  =  "1"  /* Bypass Right Common Iliac Artery to Right Femoral Artery with Synthetic Substitute, Open Approach */
"041C0JJ"  =  "1"  /* Bypass Right Common Iliac Artery to Left Femoral Artery with Synthetic Substitute, Open Approach */
"041C0JK"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Open Approach */
"041C0JQ"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Open Approach */
"041C0JR"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Artery with Synthetic Substitute, Open Approach */
"041C0K0"  =  "1"  /* Bypass Right Common Iliac Artery to Abdominal Aorta with Nonautologous Tissue Substitute, Open Approach */
"041C0K1"  =  "1"  /* Bypass Right Common Iliac Artery to Celiac Artery with Nonautologous Tissue Substitute, Open Approach */
"041C0K2"  =  "1"  /* Bypass Right Common Iliac Artery to Mesenteric Artery with Nonautologous Tissue Substitute, Open Approach */
"041C0K3"  =  "1"  /* Bypass Right Common Iliac Artery to Right Renal Artery with Nonautologous Tissue Substitute, Open Approach */
"041C0K4"  =  "1"  /* Bypass Right Common Iliac Artery to Left Renal Artery with Nonautologous Tissue Substitute, Open Approach */
"041C0K5"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Renal Artery with Nonautologous Tissue Substitute, Open Approach */
"041C0K6"  =  "1"  /* Bypass Right Common Iliac Artery to Right Common Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041C0K7"  =  "1"  /* Bypass Right Common Iliac Artery to Left Common Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041C0K8"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Common Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"041C0K9"  =  "1"  /* Bypass Right Common Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041C0KB"  =  "1"  /* Bypass Right Common Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041C0KC"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"041C0KD"  =  "1"  /* Bypass Right Common Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041C0KF"  =  "1"  /* Bypass Right Common Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041C0KG"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"041C0KH"  =  "1"  /* Bypass Right Common Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach */
"041C0KJ"  =  "1"  /* Bypass Right Common Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach */
"041C0KK"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Open Approach */
"041C0KQ"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach */
"041C0KR"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Artery with Nonautologous Tissue Substitute, Open Approach */
"041C0Z0"  =  "1"  /* Bypass Right Common Iliac Artery to Abdominal Aorta, Open Approach */
"041C0Z1"  =  "1"  /* Bypass Right Common Iliac Artery to Celiac Artery, Open Approach */
"041C0Z2"  =  "1"  /* Bypass Right Common Iliac Artery to Mesenteric Artery, Open Approach */
"041C0Z3"  =  "1"  /* Bypass Right Common Iliac Artery to Right Renal Artery, Open Approach */
"041C0Z4"  =  "1"  /* Bypass Right Common Iliac Artery to Left Renal Artery, Open Approach */
"041C0Z5"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Renal Artery, Open Approach */
"041C0Z6"  =  "1"  /* Bypass Right Common Iliac Artery to Right Common Iliac Artery, Open Approach */
"041C0Z7"  =  "1"  /* Bypass Right Common Iliac Artery to Left Common Iliac Artery, Open Approach */
"041C0Z8"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Common Iliac Arteries, Open Approach */
"041C0Z9"  =  "1"  /* Bypass Right Common Iliac Artery to Right Internal Iliac Artery, Open Approach */
"041C0ZB"  =  "1"  /* Bypass Right Common Iliac Artery to Left Internal Iliac Artery, Open Approach */
"041C0ZC"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Internal Iliac Arteries, Open Approach */
"041C0ZD"  =  "1"  /* Bypass Right Common Iliac Artery to Right External Iliac Artery, Open Approach */
"041C0ZF"  =  "1"  /* Bypass Right Common Iliac Artery to Left External Iliac Artery, Open Approach */
"041C0ZG"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral External Iliac Arteries, Open Approach */
"041C0ZH"  =  "1"  /* Bypass Right Common Iliac Artery to Right Femoral Artery, Open Approach */
"041C0ZJ"  =  "1"  /* Bypass Right Common Iliac Artery to Left Femoral Artery, Open Approach */
"041C0ZK"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Femoral Arteries, Open Approach */
"041C0ZQ"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Extremity Artery, Open Approach */
"041C0ZR"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Artery, Open Approach */
"041C490"  =  "1"  /* Bypass Right Common Iliac Artery to Abdominal Aorta with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C491"  =  "1"  /* Bypass Right Common Iliac Artery to Celiac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C492"  =  "1"  /* Bypass Right Common Iliac Artery to Mesenteric Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C493"  =  "1"  /* Bypass Right Common Iliac Artery to Right Renal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C494"  =  "1"  /* Bypass Right Common Iliac Artery to Left Renal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C495"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Renal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C496"  =  "1"  /* Bypass Right Common Iliac Artery to Right Common Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C497"  =  "1"  /* Bypass Right Common Iliac Artery to Left Common Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C498"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Common Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C499"  =  "1"  /* Bypass Right Common Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C49B"  =  "1"  /* Bypass Right Common Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C49C"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C49D"  =  "1"  /* Bypass Right Common Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C49F"  =  "1"  /* Bypass Right Common Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C49G"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C49H"  =  "1"  /* Bypass Right Common Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C49J"  =  "1"  /* Bypass Right Common Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C49K"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C49Q"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C49R"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041C4A0"  =  "1"  /* Bypass Right Common Iliac Artery to Abdominal Aorta with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4A1"  =  "1"  /* Bypass Right Common Iliac Artery to Celiac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4A2"  =  "1"  /* Bypass Right Common Iliac Artery to Mesenteric Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4A3"  =  "1"  /* Bypass Right Common Iliac Artery to Right Renal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4A4"  =  "1"  /* Bypass Right Common Iliac Artery to Left Renal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4A5"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Renal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4A6"  =  "1"  /* Bypass Right Common Iliac Artery to Right Common Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4A7"  =  "1"  /* Bypass Right Common Iliac Artery to Left Common Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4A8"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Common Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4A9"  =  "1"  /* Bypass Right Common Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4AB"  =  "1"  /* Bypass Right Common Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4AC"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4AD"  =  "1"  /* Bypass Right Common Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4AF"  =  "1"  /* Bypass Right Common Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4AG"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4AH"  =  "1"  /* Bypass Right Common Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4AJ"  =  "1"  /* Bypass Right Common Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4AK"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4AQ"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4AR"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041C4J0"  =  "1"  /* Bypass Right Common Iliac Artery to Abdominal Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4J1"  =  "1"  /* Bypass Right Common Iliac Artery to Celiac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4J2"  =  "1"  /* Bypass Right Common Iliac Artery to Mesenteric Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4J3"  =  "1"  /* Bypass Right Common Iliac Artery to Right Renal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4J4"  =  "1"  /* Bypass Right Common Iliac Artery to Left Renal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4J5"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Renal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4J6"  =  "1"  /* Bypass Right Common Iliac Artery to Right Common Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4J7"  =  "1"  /* Bypass Right Common Iliac Artery to Left Common Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4J8"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Common Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4J9"  =  "1"  /* Bypass Right Common Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4JB"  =  "1"  /* Bypass Right Common Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4JC"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4JD"  =  "1"  /* Bypass Right Common Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4JF"  =  "1"  /* Bypass Right Common Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4JG"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4JH"  =  "1"  /* Bypass Right Common Iliac Artery to Right Femoral Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4JJ"  =  "1"  /* Bypass Right Common Iliac Artery to Left Femoral Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4JK"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4JQ"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4JR"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041C4K0"  =  "1"  /* Bypass Right Common Iliac Artery to Abdominal Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4K1"  =  "1"  /* Bypass Right Common Iliac Artery to Celiac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4K2"  =  "1"  /* Bypass Right Common Iliac Artery to Mesenteric Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4K3"  =  "1"  /* Bypass Right Common Iliac Artery to Right Renal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4K4"  =  "1"  /* Bypass Right Common Iliac Artery to Left Renal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4K5"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Renal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4K6"  =  "1"  /* Bypass Right Common Iliac Artery to Right Common Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4K7"  =  "1"  /* Bypass Right Common Iliac Artery to Left Common Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4K8"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Common Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4K9"  =  "1"  /* Bypass Right Common Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4KB"  =  "1"  /* Bypass Right Common Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4KC"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4KD"  =  "1"  /* Bypass Right Common Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4KF"  =  "1"  /* Bypass Right Common Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4KG"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4KH"  =  "1"  /* Bypass Right Common Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4KJ"  =  "1"  /* Bypass Right Common Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4KK"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4KQ"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4KR"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041C4Z0"  =  "1"  /* Bypass Right Common Iliac Artery to Abdominal Aorta, Percutaneous Endoscopic Approach */
"041C4Z1"  =  "1"  /* Bypass Right Common Iliac Artery to Celiac Artery, Percutaneous Endoscopic Approach */
"041C4Z2"  =  "1"  /* Bypass Right Common Iliac Artery to Mesenteric Artery, Percutaneous Endoscopic Approach */
"041C4Z3"  =  "1"  /* Bypass Right Common Iliac Artery to Right Renal Artery, Percutaneous Endoscopic Approach */
"041C4Z4"  =  "1"  /* Bypass Right Common Iliac Artery to Left Renal Artery, Percutaneous Endoscopic Approach */
"041C4Z5"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Renal Artery, Percutaneous Endoscopic Approach */
"041C4Z6"  =  "1"  /* Bypass Right Common Iliac Artery to Right Common Iliac Artery, Percutaneous Endoscopic Approach */
"041C4Z7"  =  "1"  /* Bypass Right Common Iliac Artery to Left Common Iliac Artery, Percutaneous Endoscopic Approach */
"041C4Z8"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Common Iliac Arteries, Percutaneous Endoscopic Approach */
"041C4Z9"  =  "1"  /* Bypass Right Common Iliac Artery to Right Internal Iliac Artery, Percutaneous Endoscopic Approach */
"041C4ZB"  =  "1"  /* Bypass Right Common Iliac Artery to Left Internal Iliac Artery, Percutaneous Endoscopic Approach */
"041C4ZC"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Internal Iliac Arteries, Percutaneous Endoscopic Approach */
"041C4ZD"  =  "1"  /* Bypass Right Common Iliac Artery to Right External Iliac Artery, Percutaneous Endoscopic Approach */
"041C4ZF"  =  "1"  /* Bypass Right Common Iliac Artery to Left External Iliac Artery, Percutaneous Endoscopic Approach */
"041C4ZG"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral External Iliac Arteries, Percutaneous Endoscopic Approach */
"041C4ZH"  =  "1"  /* Bypass Right Common Iliac Artery to Right Femoral Artery, Percutaneous Endoscopic Approach */
"041C4ZJ"  =  "1"  /* Bypass Right Common Iliac Artery to Left Femoral Artery, Percutaneous Endoscopic Approach */
"041C4ZK"  =  "1"  /* Bypass Right Common Iliac Artery to Bilateral Femoral Arteries, Percutaneous Endoscopic Approach */
"041C4ZQ"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Extremity Artery, Percutaneous Endoscopic Approach */
"041C4ZR"  =  "1"  /* Bypass Right Common Iliac Artery to Lower Artery, Percutaneous Endoscopic Approach */
"041D090"  =  "1"  /* Bypass Left Common Iliac Artery to Abdominal Aorta with Autologous Venous Tissue, Open Approach */
"041D091"  =  "1"  /* Bypass Left Common Iliac Artery to Celiac Artery with Autologous Venous Tissue, Open Approach */
"041D092"  =  "1"  /* Bypass Left Common Iliac Artery to Mesenteric Artery with Autologous Venous Tissue, Open Approach */
"041D093"  =  "1"  /* Bypass Left Common Iliac Artery to Right Renal Artery with Autologous Venous Tissue, Open Approach */
"041D094"  =  "1"  /* Bypass Left Common Iliac Artery to Left Renal Artery with Autologous Venous Tissue, Open Approach */
"041D095"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Renal Artery with Autologous Venous Tissue, Open Approach */
"041D096"  =  "1"  /* Bypass Left Common Iliac Artery to Right Common Iliac Artery with Autologous Venous Tissue, Open Approach */
"041D097"  =  "1"  /* Bypass Left Common Iliac Artery to Left Common Iliac Artery with Autologous Venous Tissue, Open Approach */
"041D098"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Common Iliac Arteries with Autologous Venous Tissue, Open Approach */
"041D099"  =  "1"  /* Bypass Left Common Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Open Approach */
"041D09B"  =  "1"  /* Bypass Left Common Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Open Approach */
"041D09C"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Open Approach */
"041D09D"  =  "1"  /* Bypass Left Common Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Open Approach */
"041D09F"  =  "1"  /* Bypass Left Common Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Open Approach */
"041D09G"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Open Approach */
"041D09H"  =  "1"  /* Bypass Left Common Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Open Approach */
"041D09J"  =  "1"  /* Bypass Left Common Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Open Approach */
"041D09K"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Open Approach */
"041D09Q"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Open Approach */
"041D09R"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Artery with Autologous Venous Tissue, Open Approach */
"041D0A0"  =  "1"  /* Bypass Left Common Iliac Artery to Abdominal Aorta with Autologous Arterial Tissue, Open Approach */
"041D0A1"  =  "1"  /* Bypass Left Common Iliac Artery to Celiac Artery with Autologous Arterial Tissue, Open Approach */
"041D0A2"  =  "1"  /* Bypass Left Common Iliac Artery to Mesenteric Artery with Autologous Arterial Tissue, Open Approach */
"041D0A3"  =  "1"  /* Bypass Left Common Iliac Artery to Right Renal Artery with Autologous Arterial Tissue, Open Approach */
"041D0A4"  =  "1"  /* Bypass Left Common Iliac Artery to Left Renal Artery with Autologous Arterial Tissue, Open Approach */
"041D0A5"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Renal Artery with Autologous Arterial Tissue, Open Approach */
"041D0A6"  =  "1"  /* Bypass Left Common Iliac Artery to Right Common Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041D0A7"  =  "1"  /* Bypass Left Common Iliac Artery to Left Common Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041D0A8"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Common Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"041D0A9"  =  "1"  /* Bypass Left Common Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041D0AB"  =  "1"  /* Bypass Left Common Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041D0AC"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"041D0AD"  =  "1"  /* Bypass Left Common Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041D0AF"  =  "1"  /* Bypass Left Common Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041D0AG"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"041D0AH"  =  "1"  /* Bypass Left Common Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Open Approach */
"041D0AJ"  =  "1"  /* Bypass Left Common Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Open Approach */
"041D0AK"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Open Approach */
"041D0AQ"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach */
"041D0AR"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Artery with Autologous Arterial Tissue, Open Approach */
"041D0J0"  =  "1"  /* Bypass Left Common Iliac Artery to Abdominal Aorta with Synthetic Substitute, Open Approach */
"041D0J1"  =  "1"  /* Bypass Left Common Iliac Artery to Celiac Artery with Synthetic Substitute, Open Approach */
"041D0J2"  =  "1"  /* Bypass Left Common Iliac Artery to Mesenteric Artery with Synthetic Substitute, Open Approach */
"041D0J3"  =  "1"  /* Bypass Left Common Iliac Artery to Right Renal Artery with Synthetic Substitute, Open Approach */
"041D0J4"  =  "1"  /* Bypass Left Common Iliac Artery to Left Renal Artery with Synthetic Substitute, Open Approach */
"041D0J5"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Renal Artery with Synthetic Substitute, Open Approach */
"041D0J6"  =  "1"  /* Bypass Left Common Iliac Artery to Right Common Iliac Artery with Synthetic Substitute, Open Approach */
"041D0J7"  =  "1"  /* Bypass Left Common Iliac Artery to Left Common Iliac Artery with Synthetic Substitute, Open Approach */
"041D0J8"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Common Iliac Arteries with Synthetic Substitute, Open Approach */
"041D0J9"  =  "1"  /* Bypass Left Common Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Open Approach */
"041D0JB"  =  "1"  /* Bypass Left Common Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Open Approach */
"041D0JC"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Open Approach */
"041D0JD"  =  "1"  /* Bypass Left Common Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Open Approach */
"041D0JF"  =  "1"  /* Bypass Left Common Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Open Approach */
"041D0JG"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Open Approach */
"041D0JH"  =  "1"  /* Bypass Left Common Iliac Artery to Right Femoral Artery with Synthetic Substitute, Open Approach */
"041D0JJ"  =  "1"  /* Bypass Left Common Iliac Artery to Left Femoral Artery with Synthetic Substitute, Open Approach */
"041D0JK"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Open Approach */
"041D0JQ"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Open Approach */
"041D0JR"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Artery with Synthetic Substitute, Open Approach */
"041D0K0"  =  "1"  /* Bypass Left Common Iliac Artery to Abdominal Aorta with Nonautologous Tissue Substitute, Open Approach */
"041D0K1"  =  "1"  /* Bypass Left Common Iliac Artery to Celiac Artery with Nonautologous Tissue Substitute, Open Approach */
"041D0K2"  =  "1"  /* Bypass Left Common Iliac Artery to Mesenteric Artery with Nonautologous Tissue Substitute, Open Approach */
"041D0K3"  =  "1"  /* Bypass Left Common Iliac Artery to Right Renal Artery with Nonautologous Tissue Substitute, Open Approach */
"041D0K4"  =  "1"  /* Bypass Left Common Iliac Artery to Left Renal Artery with Nonautologous Tissue Substitute, Open Approach */
"041D0K5"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Renal Artery with Nonautologous Tissue Substitute, Open Approach */
"041D0K6"  =  "1"  /* Bypass Left Common Iliac Artery to Right Common Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041D0K7"  =  "1"  /* Bypass Left Common Iliac Artery to Left Common Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041D0K8"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Common Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"041D0K9"  =  "1"  /* Bypass Left Common Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041D0KB"  =  "1"  /* Bypass Left Common Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041D0KC"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"041D0KD"  =  "1"  /* Bypass Left Common Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041D0KF"  =  "1"  /* Bypass Left Common Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041D0KG"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"041D0KH"  =  "1"  /* Bypass Left Common Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach */
"041D0KJ"  =  "1"  /* Bypass Left Common Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach */
"041D0KK"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Open Approach */
"041D0KQ"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach */
"041D0KR"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Artery with Nonautologous Tissue Substitute, Open Approach */
"041D0Z0"  =  "1"  /* Bypass Left Common Iliac Artery to Abdominal Aorta, Open Approach */
"041D0Z1"  =  "1"  /* Bypass Left Common Iliac Artery to Celiac Artery, Open Approach */
"041D0Z2"  =  "1"  /* Bypass Left Common Iliac Artery to Mesenteric Artery, Open Approach */
"041D0Z3"  =  "1"  /* Bypass Left Common Iliac Artery to Right Renal Artery, Open Approach */
"041D0Z4"  =  "1"  /* Bypass Left Common Iliac Artery to Left Renal Artery, Open Approach */
"041D0Z5"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Renal Artery, Open Approach */
"041D0Z6"  =  "1"  /* Bypass Left Common Iliac Artery to Right Common Iliac Artery, Open Approach */
"041D0Z7"  =  "1"  /* Bypass Left Common Iliac Artery to Left Common Iliac Artery, Open Approach */
"041D0Z8"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Common Iliac Arteries, Open Approach */
"041D0Z9"  =  "1"  /* Bypass Left Common Iliac Artery to Right Internal Iliac Artery, Open Approach */
"041D0ZB"  =  "1"  /* Bypass Left Common Iliac Artery to Left Internal Iliac Artery, Open Approach */
"041D0ZC"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Internal Iliac Arteries, Open Approach */
"041D0ZD"  =  "1"  /* Bypass Left Common Iliac Artery to Right External Iliac Artery, Open Approach */
"041D0ZF"  =  "1"  /* Bypass Left Common Iliac Artery to Left External Iliac Artery, Open Approach */
"041D0ZG"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral External Iliac Arteries, Open Approach */
"041D0ZH"  =  "1"  /* Bypass Left Common Iliac Artery to Right Femoral Artery, Open Approach */
"041D0ZJ"  =  "1"  /* Bypass Left Common Iliac Artery to Left Femoral Artery, Open Approach */
"041D0ZK"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Femoral Arteries, Open Approach */
"041D0ZQ"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Extremity Artery, Open Approach */
"041D0ZR"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Artery, Open Approach */
"041D490"  =  "1"  /* Bypass Left Common Iliac Artery to Abdominal Aorta with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D491"  =  "1"  /* Bypass Left Common Iliac Artery to Celiac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D492"  =  "1"  /* Bypass Left Common Iliac Artery to Mesenteric Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D493"  =  "1"  /* Bypass Left Common Iliac Artery to Right Renal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D494"  =  "1"  /* Bypass Left Common Iliac Artery to Left Renal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D495"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Renal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D496"  =  "1"  /* Bypass Left Common Iliac Artery to Right Common Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D497"  =  "1"  /* Bypass Left Common Iliac Artery to Left Common Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D498"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Common Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D499"  =  "1"  /* Bypass Left Common Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D49B"  =  "1"  /* Bypass Left Common Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D49C"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D49D"  =  "1"  /* Bypass Left Common Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D49F"  =  "1"  /* Bypass Left Common Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D49G"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D49H"  =  "1"  /* Bypass Left Common Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D49J"  =  "1"  /* Bypass Left Common Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D49K"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D49Q"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D49R"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041D4A0"  =  "1"  /* Bypass Left Common Iliac Artery to Abdominal Aorta with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4A1"  =  "1"  /* Bypass Left Common Iliac Artery to Celiac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4A2"  =  "1"  /* Bypass Left Common Iliac Artery to Mesenteric Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4A3"  =  "1"  /* Bypass Left Common Iliac Artery to Right Renal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4A4"  =  "1"  /* Bypass Left Common Iliac Artery to Left Renal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4A5"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Renal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4A6"  =  "1"  /* Bypass Left Common Iliac Artery to Right Common Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4A7"  =  "1"  /* Bypass Left Common Iliac Artery to Left Common Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4A8"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Common Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4A9"  =  "1"  /* Bypass Left Common Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4AB"  =  "1"  /* Bypass Left Common Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4AC"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4AD"  =  "1"  /* Bypass Left Common Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4AF"  =  "1"  /* Bypass Left Common Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4AG"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4AH"  =  "1"  /* Bypass Left Common Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4AJ"  =  "1"  /* Bypass Left Common Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4AK"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4AQ"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4AR"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041D4J0"  =  "1"  /* Bypass Left Common Iliac Artery to Abdominal Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4J1"  =  "1"  /* Bypass Left Common Iliac Artery to Celiac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4J2"  =  "1"  /* Bypass Left Common Iliac Artery to Mesenteric Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4J3"  =  "1"  /* Bypass Left Common Iliac Artery to Right Renal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4J4"  =  "1"  /* Bypass Left Common Iliac Artery to Left Renal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4J5"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Renal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4J6"  =  "1"  /* Bypass Left Common Iliac Artery to Right Common Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4J7"  =  "1"  /* Bypass Left Common Iliac Artery to Left Common Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4J8"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Common Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4J9"  =  "1"  /* Bypass Left Common Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4JB"  =  "1"  /* Bypass Left Common Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4JC"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4JD"  =  "1"  /* Bypass Left Common Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4JF"  =  "1"  /* Bypass Left Common Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4JG"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4JH"  =  "1"  /* Bypass Left Common Iliac Artery to Right Femoral Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4JJ"  =  "1"  /* Bypass Left Common Iliac Artery to Left Femoral Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4JK"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4JQ"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4JR"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041D4K0"  =  "1"  /* Bypass Left Common Iliac Artery to Abdominal Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4K1"  =  "1"  /* Bypass Left Common Iliac Artery to Celiac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4K2"  =  "1"  /* Bypass Left Common Iliac Artery to Mesenteric Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4K3"  =  "1"  /* Bypass Left Common Iliac Artery to Right Renal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4K4"  =  "1"  /* Bypass Left Common Iliac Artery to Left Renal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4K5"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Renal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4K6"  =  "1"  /* Bypass Left Common Iliac Artery to Right Common Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4K7"  =  "1"  /* Bypass Left Common Iliac Artery to Left Common Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4K8"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Common Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4K9"  =  "1"  /* Bypass Left Common Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4KB"  =  "1"  /* Bypass Left Common Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4KC"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4KD"  =  "1"  /* Bypass Left Common Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4KF"  =  "1"  /* Bypass Left Common Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4KG"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4KH"  =  "1"  /* Bypass Left Common Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4KJ"  =  "1"  /* Bypass Left Common Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4KK"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4KQ"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4KR"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041D4Z0"  =  "1"  /* Bypass Left Common Iliac Artery to Abdominal Aorta, Percutaneous Endoscopic Approach */
"041D4Z1"  =  "1"  /* Bypass Left Common Iliac Artery to Celiac Artery, Percutaneous Endoscopic Approach */
"041D4Z2"  =  "1"  /* Bypass Left Common Iliac Artery to Mesenteric Artery, Percutaneous Endoscopic Approach */
"041D4Z3"  =  "1"  /* Bypass Left Common Iliac Artery to Right Renal Artery, Percutaneous Endoscopic Approach */
"041D4Z4"  =  "1"  /* Bypass Left Common Iliac Artery to Left Renal Artery, Percutaneous Endoscopic Approach */
"041D4Z5"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Renal Artery, Percutaneous Endoscopic Approach */
"041D4Z6"  =  "1"  /* Bypass Left Common Iliac Artery to Right Common Iliac Artery, Percutaneous Endoscopic Approach */
"041D4Z7"  =  "1"  /* Bypass Left Common Iliac Artery to Left Common Iliac Artery, Percutaneous Endoscopic Approach */
"041D4Z8"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Common Iliac Arteries, Percutaneous Endoscopic Approach */
"041D4Z9"  =  "1"  /* Bypass Left Common Iliac Artery to Right Internal Iliac Artery, Percutaneous Endoscopic Approach */
"041D4ZB"  =  "1"  /* Bypass Left Common Iliac Artery to Left Internal Iliac Artery, Percutaneous Endoscopic Approach */
"041D4ZC"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Internal Iliac Arteries, Percutaneous Endoscopic Approach */
"041D4ZD"  =  "1"  /* Bypass Left Common Iliac Artery to Right External Iliac Artery, Percutaneous Endoscopic Approach */
"041D4ZF"  =  "1"  /* Bypass Left Common Iliac Artery to Left External Iliac Artery, Percutaneous Endoscopic Approach */
"041D4ZG"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral External Iliac Arteries, Percutaneous Endoscopic Approach */
"041D4ZH"  =  "1"  /* Bypass Left Common Iliac Artery to Right Femoral Artery, Percutaneous Endoscopic Approach */
"041D4ZJ"  =  "1"  /* Bypass Left Common Iliac Artery to Left Femoral Artery, Percutaneous Endoscopic Approach */
"041D4ZK"  =  "1"  /* Bypass Left Common Iliac Artery to Bilateral Femoral Arteries, Percutaneous Endoscopic Approach */
"041D4ZQ"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Extremity Artery, Percutaneous Endoscopic Approach */
"041D4ZR"  =  "1"  /* Bypass Left Common Iliac Artery to Lower Artery, Percutaneous Endoscopic Approach */
"041E099"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Open Approach */
"041E09B"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Open Approach */
"041E09C"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Open Approach */
"041E09D"  =  "1"  /* Bypass Right Internal Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Open Approach */
"041E09F"  =  "1"  /* Bypass Right Internal Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Open Approach */
"041E09G"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Open Approach */
"041E09H"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Open Approach */
"041E09J"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Open Approach */
"041E09K"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Open Approach */
"041E09P"  =  "1"  /* Bypass Right Internal Iliac Artery to Foot Artery with Autologous Venous Tissue, Open Approach */
"041E09Q"  =  "1"  /* Bypass Right Internal Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Open Approach */
"041E0A9"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041E0AB"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041E0AC"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"041E0AD"  =  "1"  /* Bypass Right Internal Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041E0AF"  =  "1"  /* Bypass Right Internal Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041E0AG"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"041E0AH"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Open Approach */
"041E0AJ"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Open Approach */
"041E0AK"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Open Approach */
"041E0AP"  =  "1"  /* Bypass Right Internal Iliac Artery to Foot Artery with Autologous Arterial Tissue, Open Approach */
"041E0AQ"  =  "1"  /* Bypass Right Internal Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach */
"041E0J9"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Open Approach */
"041E0JB"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Open Approach */
"041E0JC"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Open Approach */
"041E0JD"  =  "1"  /* Bypass Right Internal Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Open Approach */
"041E0JF"  =  "1"  /* Bypass Right Internal Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Open Approach */
"041E0JG"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Open Approach */
"041E0JH"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Femoral Artery with Synthetic Substitute, Open Approach */
"041E0JJ"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Femoral Artery with Synthetic Substitute, Open Approach */
"041E0JK"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Open Approach */
"041E0JP"  =  "1"  /* Bypass Right Internal Iliac Artery to Foot Artery with Synthetic Substitute, Open Approach */
"041E0JQ"  =  "1"  /* Bypass Right Internal Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Open Approach */
"041E0K9"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041E0KB"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041E0KC"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"041E0KD"  =  "1"  /* Bypass Right Internal Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041E0KF"  =  "1"  /* Bypass Right Internal Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041E0KG"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"041E0KH"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach */
"041E0KJ"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach */
"041E0KK"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Open Approach */
"041E0KP"  =  "1"  /* Bypass Right Internal Iliac Artery to Foot Artery with Nonautologous Tissue Substitute, Open Approach */
"041E0KQ"  =  "1"  /* Bypass Right Internal Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach */
"041E0Z9"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Internal Iliac Artery, Open Approach */
"041E0ZB"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Internal Iliac Artery, Open Approach */
"041E0ZC"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Internal Iliac Arteries, Open Approach */
"041E0ZD"  =  "1"  /* Bypass Right Internal Iliac Artery to Right External Iliac Artery, Open Approach */
"041E0ZF"  =  "1"  /* Bypass Right Internal Iliac Artery to Left External Iliac Artery, Open Approach */
"041E0ZG"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral External Iliac Arteries, Open Approach */
"041E0ZH"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Femoral Artery, Open Approach */
"041E0ZJ"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Femoral Artery, Open Approach */
"041E0ZK"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Femoral Arteries, Open Approach */
"041E0ZP"  =  "1"  /* Bypass Right Internal Iliac Artery to Foot Artery, Open Approach */
"041E0ZQ"  =  "1"  /* Bypass Right Internal Iliac Artery to Lower Extremity Artery, Open Approach */
"041E499"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041E49B"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041E49C"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041E49D"  =  "1"  /* Bypass Right Internal Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041E49F"  =  "1"  /* Bypass Right Internal Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041E49G"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041E49H"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041E49J"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041E49K"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041E49P"  =  "1"  /* Bypass Right Internal Iliac Artery to Foot Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041E49Q"  =  "1"  /* Bypass Right Internal Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041E4A9"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041E4AB"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041E4AC"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041E4AD"  =  "1"  /* Bypass Right Internal Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041E4AF"  =  "1"  /* Bypass Right Internal Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041E4AG"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041E4AH"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041E4AJ"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041E4AK"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041E4AP"  =  "1"  /* Bypass Right Internal Iliac Artery to Foot Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041E4AQ"  =  "1"  /* Bypass Right Internal Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041E4J9"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041E4JB"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041E4JC"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041E4JD"  =  "1"  /* Bypass Right Internal Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041E4JF"  =  "1"  /* Bypass Right Internal Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041E4JG"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041E4JH"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Femoral Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041E4JJ"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Femoral Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041E4JK"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041E4JP"  =  "1"  /* Bypass Right Internal Iliac Artery to Foot Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041E4JQ"  =  "1"  /* Bypass Right Internal Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041E4K9"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041E4KB"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041E4KC"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041E4KD"  =  "1"  /* Bypass Right Internal Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041E4KF"  =  "1"  /* Bypass Right Internal Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041E4KG"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041E4KH"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041E4KJ"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041E4KK"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041E4KP"  =  "1"  /* Bypass Right Internal Iliac Artery to Foot Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041E4KQ"  =  "1"  /* Bypass Right Internal Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041E4Z9"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Internal Iliac Artery, Percutaneous Endoscopic Approach */
"041E4ZB"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Internal Iliac Artery, Percutaneous Endoscopic Approach */
"041E4ZC"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Internal Iliac Arteries, Percutaneous Endoscopic Approach */
"041E4ZD"  =  "1"  /* Bypass Right Internal Iliac Artery to Right External Iliac Artery, Percutaneous Endoscopic Approach */
"041E4ZF"  =  "1"  /* Bypass Right Internal Iliac Artery to Left External Iliac Artery, Percutaneous Endoscopic Approach */
"041E4ZG"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral External Iliac Arteries, Percutaneous Endoscopic Approach */
"041E4ZH"  =  "1"  /* Bypass Right Internal Iliac Artery to Right Femoral Artery, Percutaneous Endoscopic Approach */
"041E4ZJ"  =  "1"  /* Bypass Right Internal Iliac Artery to Left Femoral Artery, Percutaneous Endoscopic Approach */
"041E4ZK"  =  "1"  /* Bypass Right Internal Iliac Artery to Bilateral Femoral Arteries, Percutaneous Endoscopic Approach */
"041E4ZP"  =  "1"  /* Bypass Right Internal Iliac Artery to Foot Artery, Percutaneous Endoscopic Approach */
"041E4ZQ"  =  "1"  /* Bypass Right Internal Iliac Artery to Lower Extremity Artery, Percutaneous Endoscopic Approach */
"041F099"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Open Approach */
"041F09B"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Open Approach */
"041F09C"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Open Approach */
"041F09D"  =  "1"  /* Bypass Left Internal Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Open Approach */
"041F09F"  =  "1"  /* Bypass Left Internal Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Open Approach */
"041F09G"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Open Approach */
"041F09H"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Open Approach */
"041F09J"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Open Approach */
"041F09K"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Open Approach */
"041F09P"  =  "1"  /* Bypass Left Internal Iliac Artery to Foot Artery with Autologous Venous Tissue, Open Approach */
"041F09Q"  =  "1"  /* Bypass Left Internal Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Open Approach */
"041F0A9"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041F0AB"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041F0AC"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"041F0AD"  =  "1"  /* Bypass Left Internal Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041F0AF"  =  "1"  /* Bypass Left Internal Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041F0AG"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"041F0AH"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Open Approach */
"041F0AJ"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Open Approach */
"041F0AK"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Open Approach */
"041F0AP"  =  "1"  /* Bypass Left Internal Iliac Artery to Foot Artery with Autologous Arterial Tissue, Open Approach */
"041F0AQ"  =  "1"  /* Bypass Left Internal Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach */
"041F0J9"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Open Approach */
"041F0JB"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Open Approach */
"041F0JC"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Open Approach */
"041F0JD"  =  "1"  /* Bypass Left Internal Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Open Approach */
"041F0JF"  =  "1"  /* Bypass Left Internal Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Open Approach */
"041F0JG"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Open Approach */
"041F0JH"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Femoral Artery with Synthetic Substitute, Open Approach */
"041F0JJ"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Femoral Artery with Synthetic Substitute, Open Approach */
"041F0JK"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Open Approach */
"041F0JP"  =  "1"  /* Bypass Left Internal Iliac Artery to Foot Artery with Synthetic Substitute, Open Approach */
"041F0JQ"  =  "1"  /* Bypass Left Internal Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Open Approach */
"041F0K9"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041F0KB"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041F0KC"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"041F0KD"  =  "1"  /* Bypass Left Internal Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041F0KF"  =  "1"  /* Bypass Left Internal Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041F0KG"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"041F0KH"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach */
"041F0KJ"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach */
"041F0KK"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Open Approach */
"041F0KP"  =  "1"  /* Bypass Left Internal Iliac Artery to Foot Artery with Nonautologous Tissue Substitute, Open Approach */
"041F0KQ"  =  "1"  /* Bypass Left Internal Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach */
"041F0Z9"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Internal Iliac Artery, Open Approach */
"041F0ZB"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Internal Iliac Artery, Open Approach */
"041F0ZC"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Internal Iliac Arteries, Open Approach */
"041F0ZD"  =  "1"  /* Bypass Left Internal Iliac Artery to Right External Iliac Artery, Open Approach */
"041F0ZF"  =  "1"  /* Bypass Left Internal Iliac Artery to Left External Iliac Artery, Open Approach */
"041F0ZG"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral External Iliac Arteries, Open Approach */
"041F0ZH"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Femoral Artery, Open Approach */
"041F0ZJ"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Femoral Artery, Open Approach */
"041F0ZK"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Femoral Arteries, Open Approach */
"041F0ZP"  =  "1"  /* Bypass Left Internal Iliac Artery to Foot Artery, Open Approach */
"041F0ZQ"  =  "1"  /* Bypass Left Internal Iliac Artery to Lower Extremity Artery, Open Approach */
"041F499"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041F49B"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041F49C"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041F49D"  =  "1"  /* Bypass Left Internal Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041F49F"  =  "1"  /* Bypass Left Internal Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041F49G"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041F49H"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041F49J"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041F49K"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041F49P"  =  "1"  /* Bypass Left Internal Iliac Artery to Foot Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041F49Q"  =  "1"  /* Bypass Left Internal Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041F4A9"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041F4AB"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041F4AC"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041F4AD"  =  "1"  /* Bypass Left Internal Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041F4AF"  =  "1"  /* Bypass Left Internal Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041F4AG"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041F4AH"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041F4AJ"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041F4AK"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041F4AP"  =  "1"  /* Bypass Left Internal Iliac Artery to Foot Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041F4AQ"  =  "1"  /* Bypass Left Internal Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041F4J9"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041F4JB"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041F4JC"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041F4JD"  =  "1"  /* Bypass Left Internal Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041F4JF"  =  "1"  /* Bypass Left Internal Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041F4JG"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041F4JH"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Femoral Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041F4JJ"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Femoral Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041F4JK"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041F4JP"  =  "1"  /* Bypass Left Internal Iliac Artery to Foot Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041F4JQ"  =  "1"  /* Bypass Left Internal Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041F4K9"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041F4KB"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041F4KC"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041F4KD"  =  "1"  /* Bypass Left Internal Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041F4KF"  =  "1"  /* Bypass Left Internal Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041F4KG"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041F4KH"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041F4KJ"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041F4KK"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041F4KP"  =  "1"  /* Bypass Left Internal Iliac Artery to Foot Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041F4KQ"  =  "1"  /* Bypass Left Internal Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041F4Z9"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Internal Iliac Artery, Percutaneous Endoscopic Approach */
"041F4ZB"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Internal Iliac Artery, Percutaneous Endoscopic Approach */
"041F4ZC"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Internal Iliac Arteries, Percutaneous Endoscopic Approach */
"041F4ZD"  =  "1"  /* Bypass Left Internal Iliac Artery to Right External Iliac Artery, Percutaneous Endoscopic Approach */
"041F4ZF"  =  "1"  /* Bypass Left Internal Iliac Artery to Left External Iliac Artery, Percutaneous Endoscopic Approach */
"041F4ZG"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral External Iliac Arteries, Percutaneous Endoscopic Approach */
"041F4ZH"  =  "1"  /* Bypass Left Internal Iliac Artery to Right Femoral Artery, Percutaneous Endoscopic Approach */
"041F4ZJ"  =  "1"  /* Bypass Left Internal Iliac Artery to Left Femoral Artery, Percutaneous Endoscopic Approach */
"041F4ZK"  =  "1"  /* Bypass Left Internal Iliac Artery to Bilateral Femoral Arteries, Percutaneous Endoscopic Approach */
"041F4ZP"  =  "1"  /* Bypass Left Internal Iliac Artery to Foot Artery, Percutaneous Endoscopic Approach */
"041F4ZQ"  =  "1"  /* Bypass Left Internal Iliac Artery to Lower Extremity Artery, Percutaneous Endoscopic Approach */
"041H099"  =  "1"  /* Bypass Right External Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Open Approach */
"041H09B"  =  "1"  /* Bypass Right External Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Open Approach */
"041H09C"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Open Approach */
"041H09D"  =  "1"  /* Bypass Right External Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Open Approach */
"041H09F"  =  "1"  /* Bypass Right External Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Open Approach */
"041H09G"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Open Approach */
"041H09H"  =  "1"  /* Bypass Right External Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Open Approach */
"041H09J"  =  "1"  /* Bypass Right External Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Open Approach */
"041H09K"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Open Approach */
"041H09P"  =  "1"  /* Bypass Right External Iliac Artery to Foot Artery with Autologous Venous Tissue, Open Approach */
"041H09Q"  =  "1"  /* Bypass Right External Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Open Approach */
"041H0A9"  =  "1"  /* Bypass Right External Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041H0AB"  =  "1"  /* Bypass Right External Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041H0AC"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"041H0AD"  =  "1"  /* Bypass Right External Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041H0AF"  =  "1"  /* Bypass Right External Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041H0AG"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"041H0AH"  =  "1"  /* Bypass Right External Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Open Approach */
"041H0AJ"  =  "1"  /* Bypass Right External Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Open Approach */
"041H0AK"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Open Approach */
"041H0AP"  =  "1"  /* Bypass Right External Iliac Artery to Foot Artery with Autologous Arterial Tissue, Open Approach */
"041H0AQ"  =  "1"  /* Bypass Right External Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach */
"041H0J9"  =  "1"  /* Bypass Right External Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Open Approach */
"041H0JB"  =  "1"  /* Bypass Right External Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Open Approach */
"041H0JC"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Open Approach */
"041H0JD"  =  "1"  /* Bypass Right External Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Open Approach */
"041H0JF"  =  "1"  /* Bypass Right External Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Open Approach */
"041H0JG"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Open Approach */
"041H0JH"  =  "1"  /* Bypass Right External Iliac Artery to Right Femoral Artery with Synthetic Substitute, Open Approach */
"041H0JJ"  =  "1"  /* Bypass Right External Iliac Artery to Left Femoral Artery with Synthetic Substitute, Open Approach */
"041H0JK"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Open Approach */
"041H0JP"  =  "1"  /* Bypass Right External Iliac Artery to Foot Artery with Synthetic Substitute, Open Approach */
"041H0JQ"  =  "1"  /* Bypass Right External Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Open Approach */
"041H0K9"  =  "1"  /* Bypass Right External Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041H0KB"  =  "1"  /* Bypass Right External Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041H0KC"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"041H0KD"  =  "1"  /* Bypass Right External Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041H0KF"  =  "1"  /* Bypass Right External Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041H0KG"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"041H0KH"  =  "1"  /* Bypass Right External Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach */
"041H0KJ"  =  "1"  /* Bypass Right External Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach */
"041H0KK"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Open Approach */
"041H0KP"  =  "1"  /* Bypass Right External Iliac Artery to Foot Artery with Nonautologous Tissue Substitute, Open Approach */
"041H0KQ"  =  "1"  /* Bypass Right External Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach */
"041H0Z9"  =  "1"  /* Bypass Right External Iliac Artery to Right Internal Iliac Artery, Open Approach */
"041H0ZB"  =  "1"  /* Bypass Right External Iliac Artery to Left Internal Iliac Artery, Open Approach */
"041H0ZC"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Internal Iliac Arteries, Open Approach */
"041H0ZD"  =  "1"  /* Bypass Right External Iliac Artery to Right External Iliac Artery, Open Approach */
"041H0ZF"  =  "1"  /* Bypass Right External Iliac Artery to Left External Iliac Artery, Open Approach */
"041H0ZG"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral External Iliac Arteries, Open Approach */
"041H0ZH"  =  "1"  /* Bypass Right External Iliac Artery to Right Femoral Artery, Open Approach */
"041H0ZJ"  =  "1"  /* Bypass Right External Iliac Artery to Left Femoral Artery, Open Approach */
"041H0ZK"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Femoral Arteries, Open Approach */
"041H0ZP"  =  "1"  /* Bypass Right External Iliac Artery to Foot Artery, Open Approach */
"041H0ZQ"  =  "1"  /* Bypass Right External Iliac Artery to Lower Extremity Artery, Open Approach */
"041H499"  =  "1"  /* Bypass Right External Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041H49B"  =  "1"  /* Bypass Right External Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041H49C"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041H49D"  =  "1"  /* Bypass Right External Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041H49F"  =  "1"  /* Bypass Right External Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041H49G"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041H49H"  =  "1"  /* Bypass Right External Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041H49J"  =  "1"  /* Bypass Right External Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041H49K"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041H49P"  =  "1"  /* Bypass Right External Iliac Artery to Foot Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041H49Q"  =  "1"  /* Bypass Right External Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041H4A9"  =  "1"  /* Bypass Right External Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041H4AB"  =  "1"  /* Bypass Right External Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041H4AC"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041H4AD"  =  "1"  /* Bypass Right External Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041H4AF"  =  "1"  /* Bypass Right External Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041H4AG"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041H4AH"  =  "1"  /* Bypass Right External Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041H4AJ"  =  "1"  /* Bypass Right External Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041H4AK"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041H4AP"  =  "1"  /* Bypass Right External Iliac Artery to Foot Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041H4AQ"  =  "1"  /* Bypass Right External Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041H4J9"  =  "1"  /* Bypass Right External Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041H4JB"  =  "1"  /* Bypass Right External Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041H4JC"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041H4JD"  =  "1"  /* Bypass Right External Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041H4JF"  =  "1"  /* Bypass Right External Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041H4JG"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041H4JH"  =  "1"  /* Bypass Right External Iliac Artery to Right Femoral Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041H4JJ"  =  "1"  /* Bypass Right External Iliac Artery to Left Femoral Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041H4JK"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041H4JP"  =  "1"  /* Bypass Right External Iliac Artery to Foot Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041H4JQ"  =  "1"  /* Bypass Right External Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041H4K9"  =  "1"  /* Bypass Right External Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041H4KB"  =  "1"  /* Bypass Right External Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041H4KC"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041H4KD"  =  "1"  /* Bypass Right External Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041H4KF"  =  "1"  /* Bypass Right External Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041H4KG"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041H4KH"  =  "1"  /* Bypass Right External Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041H4KJ"  =  "1"  /* Bypass Right External Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041H4KK"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041H4KP"  =  "1"  /* Bypass Right External Iliac Artery to Foot Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041H4KQ"  =  "1"  /* Bypass Right External Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041H4Z9"  =  "1"  /* Bypass Right External Iliac Artery to Right Internal Iliac Artery, Percutaneous Endoscopic Approach */
"041H4ZB"  =  "1"  /* Bypass Right External Iliac Artery to Left Internal Iliac Artery, Percutaneous Endoscopic Approach */
"041H4ZC"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Internal Iliac Arteries, Percutaneous Endoscopic Approach */
"041H4ZD"  =  "1"  /* Bypass Right External Iliac Artery to Right External Iliac Artery, Percutaneous Endoscopic Approach */
"041H4ZF"  =  "1"  /* Bypass Right External Iliac Artery to Left External Iliac Artery, Percutaneous Endoscopic Approach */
"041H4ZG"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral External Iliac Arteries, Percutaneous Endoscopic Approach */
"041H4ZH"  =  "1"  /* Bypass Right External Iliac Artery to Right Femoral Artery, Percutaneous Endoscopic Approach */
"041H4ZJ"  =  "1"  /* Bypass Right External Iliac Artery to Left Femoral Artery, Percutaneous Endoscopic Approach */
"041H4ZK"  =  "1"  /* Bypass Right External Iliac Artery to Bilateral Femoral Arteries, Percutaneous Endoscopic Approach */
"041H4ZP"  =  "1"  /* Bypass Right External Iliac Artery to Foot Artery, Percutaneous Endoscopic Approach */
"041H4ZQ"  =  "1"  /* Bypass Right External Iliac Artery to Lower Extremity Artery, Percutaneous Endoscopic Approach */
"041J099"  =  "1"  /* Bypass Left External Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Open Approach */
"041J09B"  =  "1"  /* Bypass Left External Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Open Approach */
"041J09C"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Open Approach */
"041J09D"  =  "1"  /* Bypass Left External Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Open Approach */
"041J09F"  =  "1"  /* Bypass Left External Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Open Approach */
"041J09G"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Open Approach */
"041J09H"  =  "1"  /* Bypass Left External Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Open Approach */
"041J09J"  =  "1"  /* Bypass Left External Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Open Approach */
"041J09K"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Open Approach */
"041J09P"  =  "1"  /* Bypass Left External Iliac Artery to Foot Artery with Autologous Venous Tissue, Open Approach */
"041J09Q"  =  "1"  /* Bypass Left External Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Open Approach */
"041J0A9"  =  "1"  /* Bypass Left External Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041J0AB"  =  "1"  /* Bypass Left External Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041J0AC"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"041J0AD"  =  "1"  /* Bypass Left External Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041J0AF"  =  "1"  /* Bypass Left External Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Open Approach */
"041J0AG"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Open Approach */
"041J0AH"  =  "1"  /* Bypass Left External Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Open Approach */
"041J0AJ"  =  "1"  /* Bypass Left External Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Open Approach */
"041J0AK"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Open Approach */
"041J0AP"  =  "1"  /* Bypass Left External Iliac Artery to Foot Artery with Autologous Arterial Tissue, Open Approach */
"041J0AQ"  =  "1"  /* Bypass Left External Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach */
"041J0J9"  =  "1"  /* Bypass Left External Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Open Approach */
"041J0JB"  =  "1"  /* Bypass Left External Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Open Approach */
"041J0JC"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Open Approach */
"041J0JD"  =  "1"  /* Bypass Left External Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Open Approach */
"041J0JF"  =  "1"  /* Bypass Left External Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Open Approach */
"041J0JG"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Open Approach */
"041J0JH"  =  "1"  /* Bypass Left External Iliac Artery to Right Femoral Artery with Synthetic Substitute, Open Approach */
"041J0JJ"  =  "1"  /* Bypass Left External Iliac Artery to Left Femoral Artery with Synthetic Substitute, Open Approach */
"041J0JK"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Open Approach */
"041J0JP"  =  "1"  /* Bypass Left External Iliac Artery to Foot Artery with Synthetic Substitute, Open Approach */
"041J0JQ"  =  "1"  /* Bypass Left External Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Open Approach */
"041J0K9"  =  "1"  /* Bypass Left External Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041J0KB"  =  "1"  /* Bypass Left External Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041J0KC"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"041J0KD"  =  "1"  /* Bypass Left External Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041J0KF"  =  "1"  /* Bypass Left External Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"041J0KG"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Open Approach */
"041J0KH"  =  "1"  /* Bypass Left External Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach */
"041J0KJ"  =  "1"  /* Bypass Left External Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach */
"041J0KK"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Open Approach */
"041J0KP"  =  "1"  /* Bypass Left External Iliac Artery to Foot Artery with Nonautologous Tissue Substitute, Open Approach */
"041J0KQ"  =  "1"  /* Bypass Left External Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach */
"041J0Z9"  =  "1"  /* Bypass Left External Iliac Artery to Right Internal Iliac Artery, Open Approach */
"041J0ZB"  =  "1"  /* Bypass Left External Iliac Artery to Left Internal Iliac Artery, Open Approach */
"041J0ZC"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Internal Iliac Arteries, Open Approach */
"041J0ZD"  =  "1"  /* Bypass Left External Iliac Artery to Right External Iliac Artery, Open Approach */
"041J0ZF"  =  "1"  /* Bypass Left External Iliac Artery to Left External Iliac Artery, Open Approach */
"041J0ZG"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral External Iliac Arteries, Open Approach */
"041J0ZH"  =  "1"  /* Bypass Left External Iliac Artery to Right Femoral Artery, Open Approach */
"041J0ZJ"  =  "1"  /* Bypass Left External Iliac Artery to Left Femoral Artery, Open Approach */
"041J0ZK"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Femoral Arteries, Open Approach */
"041J0ZP"  =  "1"  /* Bypass Left External Iliac Artery to Foot Artery, Open Approach */
"041J0ZQ"  =  "1"  /* Bypass Left External Iliac Artery to Lower Extremity Artery, Open Approach */
"041J499"  =  "1"  /* Bypass Left External Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041J49B"  =  "1"  /* Bypass Left External Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041J49C"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041J49D"  =  "1"  /* Bypass Left External Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041J49F"  =  "1"  /* Bypass Left External Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041J49G"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041J49H"  =  "1"  /* Bypass Left External Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041J49J"  =  "1"  /* Bypass Left External Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041J49K"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041J49P"  =  "1"  /* Bypass Left External Iliac Artery to Foot Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041J49Q"  =  "1"  /* Bypass Left External Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"041J4A9"  =  "1"  /* Bypass Left External Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041J4AB"  =  "1"  /* Bypass Left External Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041J4AC"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041J4AD"  =  "1"  /* Bypass Left External Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041J4AF"  =  "1"  /* Bypass Left External Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041J4AG"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041J4AH"  =  "1"  /* Bypass Left External Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041J4AJ"  =  "1"  /* Bypass Left External Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041J4AK"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041J4AP"  =  "1"  /* Bypass Left External Iliac Artery to Foot Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041J4AQ"  =  "1"  /* Bypass Left External Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"041J4J9"  =  "1"  /* Bypass Left External Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041J4JB"  =  "1"  /* Bypass Left External Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041J4JC"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041J4JD"  =  "1"  /* Bypass Left External Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041J4JF"  =  "1"  /* Bypass Left External Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041J4JG"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041J4JH"  =  "1"  /* Bypass Left External Iliac Artery to Right Femoral Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041J4JJ"  =  "1"  /* Bypass Left External Iliac Artery to Left Femoral Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041J4JK"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041J4JP"  =  "1"  /* Bypass Left External Iliac Artery to Foot Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041J4JQ"  =  "1"  /* Bypass Left External Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"041J4K9"  =  "1"  /* Bypass Left External Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041J4KB"  =  "1"  /* Bypass Left External Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041J4KC"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041J4KD"  =  "1"  /* Bypass Left External Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041J4KF"  =  "1"  /* Bypass Left External Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041J4KG"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041J4KH"  =  "1"  /* Bypass Left External Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041J4KJ"  =  "1"  /* Bypass Left External Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041J4KK"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041J4KP"  =  "1"  /* Bypass Left External Iliac Artery to Foot Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041J4KQ"  =  "1"  /* Bypass Left External Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"041J4Z9"  =  "1"  /* Bypass Left External Iliac Artery to Right Internal Iliac Artery, Percutaneous Endoscopic Approach */
"041J4ZB"  =  "1"  /* Bypass Left External Iliac Artery to Left Internal Iliac Artery, Percutaneous Endoscopic Approach */
"041J4ZC"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Internal Iliac Arteries, Percutaneous Endoscopic Approach */
"041J4ZD"  =  "1"  /* Bypass Left External Iliac Artery to Right External Iliac Artery, Percutaneous Endoscopic Approach */
"041J4ZF"  =  "1"  /* Bypass Left External Iliac Artery to Left External Iliac Artery, Percutaneous Endoscopic Approach */
"041J4ZG"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral External Iliac Arteries, Percutaneous Endoscopic Approach */
"041J4ZH"  =  "1"  /* Bypass Left External Iliac Artery to Right Femoral Artery, Percutaneous Endoscopic Approach */
"041J4ZJ"  =  "1"  /* Bypass Left External Iliac Artery to Left Femoral Artery, Percutaneous Endoscopic Approach */
"041J4ZK"  =  "1"  /* Bypass Left External Iliac Artery to Bilateral Femoral Arteries, Percutaneous Endoscopic Approach */
"041J4ZP"  =  "1"  /* Bypass Left External Iliac Artery to Foot Artery, Percutaneous Endoscopic Approach */
"041J4ZQ"  =  "1"  /* Bypass Left External Iliac Artery to Lower Extremity Artery, Percutaneous Endoscopic Approach */
"04500ZZ"  =  "1"  /* Destruction of Abdominal Aorta, Open Approach */
"04503ZZ"  =  "1"  /* Destruction of Abdominal Aorta, Percutaneous Approach */
"04504ZZ"  =  "1"  /* Destruction of Abdominal Aorta, Percutaneous Endoscopic Approach */
"04510ZZ"  =  "1"  /* Destruction of Celiac Artery, Open Approach */
"04513ZZ"  =  "1"  /* Destruction of Celiac Artery, Percutaneous Approach */
"04514ZZ"  =  "1"  /* Destruction of Celiac Artery, Percutaneous Endoscopic Approach */
"04520ZZ"  =  "1"  /* Destruction of Gastric Artery, Open Approach */
"04523ZZ"  =  "1"  /* Destruction of Gastric Artery, Percutaneous Approach */
"04524ZZ"  =  "1"  /* Destruction of Gastric Artery, Percutaneous Endoscopic Approach */
"04530ZZ"  =  "1"  /* Destruction of Hepatic Artery, Open Approach */
"04533ZZ"  =  "1"  /* Destruction of Hepatic Artery, Percutaneous Approach */
"04534ZZ"  =  "1"  /* Destruction of Hepatic Artery, Percutaneous Endoscopic Approach */
"04540ZZ"  =  "1"  /* Destruction of Splenic Artery, Open Approach */
"04543ZZ"  =  "1"  /* Destruction of Splenic Artery, Percutaneous Approach */
"04544ZZ"  =  "1"  /* Destruction of Splenic Artery, Percutaneous Endoscopic Approach */
"04550ZZ"  =  "1"  /* Destruction of Superior Mesenteric Artery, Open Approach */
"04553ZZ"  =  "1"  /* Destruction of Superior Mesenteric Artery, Percutaneous Approach */
"04554ZZ"  =  "1"  /* Destruction of Superior Mesenteric Artery, Percutaneous Endoscopic Approach */
"04560ZZ"  =  "1"  /* Destruction of Right Colic Artery, Open Approach */
"04563ZZ"  =  "1"  /* Destruction of Right Colic Artery, Percutaneous Approach */
"04564ZZ"  =  "1"  /* Destruction of Right Colic Artery, Percutaneous Endoscopic Approach */
"04570ZZ"  =  "1"  /* Destruction of Left Colic Artery, Open Approach */
"04573ZZ"  =  "1"  /* Destruction of Left Colic Artery, Percutaneous Approach */
"04574ZZ"  =  "1"  /* Destruction of Left Colic Artery, Percutaneous Endoscopic Approach */
"04580ZZ"  =  "1"  /* Destruction of Middle Colic Artery, Open Approach */
"04583ZZ"  =  "1"  /* Destruction of Middle Colic Artery, Percutaneous Approach */
"04584ZZ"  =  "1"  /* Destruction of Middle Colic Artery, Percutaneous Endoscopic Approach */
"04590ZZ"  =  "1"  /* Destruction of Right Renal Artery, Open Approach */
"04593ZZ"  =  "1"  /* Destruction of Right Renal Artery, Percutaneous Approach */
"04594ZZ"  =  "1"  /* Destruction of Right Renal Artery, Percutaneous Endoscopic Approach */
"045A0ZZ"  =  "1"  /* Destruction of Left Renal Artery, Open Approach */
"045A3ZZ"  =  "1"  /* Destruction of Left Renal Artery, Percutaneous Approach */
"045A4ZZ"  =  "1"  /* Destruction of Left Renal Artery, Percutaneous Endoscopic Approach */
"045B0ZZ"  =  "1"  /* Destruction of Inferior Mesenteric Artery, Open Approach */
"045B3ZZ"  =  "1"  /* Destruction of Inferior Mesenteric Artery, Percutaneous Approach */
"045B4ZZ"  =  "1"  /* Destruction of Inferior Mesenteric Artery, Percutaneous Endoscopic Approach */
"045C0ZZ"  =  "1"  /* Destruction of Right Common Iliac Artery, Open Approach */
"045C3ZZ"  =  "1"  /* Destruction of Right Common Iliac Artery, Percutaneous Approach */
"045C4ZZ"  =  "1"  /* Destruction of Right Common Iliac Artery, Percutaneous Endoscopic Approach */
"045D0ZZ"  =  "1"  /* Destruction of Left Common Iliac Artery, Open Approach */
"045D3ZZ"  =  "1"  /* Destruction of Left Common Iliac Artery, Percutaneous Approach */
"045D4ZZ"  =  "1"  /* Destruction of Left Common Iliac Artery, Percutaneous Endoscopic Approach */
"045E0ZZ"  =  "1"  /* Destruction of Right Internal Iliac Artery, Open Approach */
"045E3ZZ"  =  "1"  /* Destruction of Right Internal Iliac Artery, Percutaneous Approach */
"045E4ZZ"  =  "1"  /* Destruction of Right Internal Iliac Artery, Percutaneous Endoscopic Approach */
"045F0ZZ"  =  "1"  /* Destruction of Left Internal Iliac Artery, Open Approach */
"045F3ZZ"  =  "1"  /* Destruction of Left Internal Iliac Artery, Percutaneous Approach */
"045F4ZZ"  =  "1"  /* Destruction of Left Internal Iliac Artery, Percutaneous Endoscopic Approach */
"045H0ZZ"  =  "1"  /* Destruction of Right External Iliac Artery, Open Approach */
"045H3ZZ"  =  "1"  /* Destruction of Right External Iliac Artery, Percutaneous Approach */
"045H4ZZ"  =  "1"  /* Destruction of Right External Iliac Artery, Percutaneous Endoscopic Approach */
"045J0ZZ"  =  "1"  /* Destruction of Left External Iliac Artery, Open Approach */
"045J3ZZ"  =  "1"  /* Destruction of Left External Iliac Artery, Percutaneous Approach */
"045J4ZZ"  =  "1"  /* Destruction of Left External Iliac Artery, Percutaneous Endoscopic Approach */
"04B00ZZ"  =  "1"  /* Excision of Abdominal Aorta, Open Approach */
"04B03ZZ"  =  "1"  /* Excision of Abdominal Aorta, Percutaneous Approach */
"04B04ZZ"  =  "1"  /* Excision of Abdominal Aorta, Percutaneous Endoscopic Approach */
"04B10ZZ"  =  "1"  /* Excision of Celiac Artery, Open Approach */
"04B13ZZ"  =  "1"  /* Excision of Celiac Artery, Percutaneous Approach */
"04B14ZZ"  =  "1"  /* Excision of Celiac Artery, Percutaneous Endoscopic Approach */
"04B20ZZ"  =  "1"  /* Excision of Gastric Artery, Open Approach */
"04B23ZZ"  =  "1"  /* Excision of Gastric Artery, Percutaneous Approach */
"04B24ZZ"  =  "1"  /* Excision of Gastric Artery, Percutaneous Endoscopic Approach */
"04B30ZZ"  =  "1"  /* Excision of Hepatic Artery, Open Approach */
"04B33ZZ"  =  "1"  /* Excision of Hepatic Artery, Percutaneous Approach */
"04B34ZZ"  =  "1"  /* Excision of Hepatic Artery, Percutaneous Endoscopic Approach */
"04B40ZZ"  =  "1"  /* Excision of Splenic Artery, Open Approach */
"04B43ZZ"  =  "1"  /* Excision of Splenic Artery, Percutaneous Approach */
"04B44ZZ"  =  "1"  /* Excision of Splenic Artery, Percutaneous Endoscopic Approach */
"04B50ZZ"  =  "1"  /* Excision of Superior Mesenteric Artery, Open Approach */
"04B53ZZ"  =  "1"  /* Excision of Superior Mesenteric Artery, Percutaneous Approach */
"04B54ZZ"  =  "1"  /* Excision of Superior Mesenteric Artery, Percutaneous Endoscopic Approach */
"04B60ZZ"  =  "1"  /* Excision of Right Colic Artery, Open Approach */
"04B63ZZ"  =  "1"  /* Excision of Right Colic Artery, Percutaneous Approach */
"04B64ZZ"  =  "1"  /* Excision of Right Colic Artery, Percutaneous Endoscopic Approach */
"04B70ZZ"  =  "1"  /* Excision of Left Colic Artery, Open Approach */
"04B73ZZ"  =  "1"  /* Excision of Left Colic Artery, Percutaneous Approach */
"04B74ZZ"  =  "1"  /* Excision of Left Colic Artery, Percutaneous Endoscopic Approach */
"04B80ZZ"  =  "1"  /* Excision of Middle Colic Artery, Open Approach */
"04B83ZZ"  =  "1"  /* Excision of Middle Colic Artery, Percutaneous Approach */
"04B84ZZ"  =  "1"  /* Excision of Middle Colic Artery, Percutaneous Endoscopic Approach */
"04B90ZZ"  =  "1"  /* Excision of Right Renal Artery, Open Approach */
"04B93ZZ"  =  "1"  /* Excision of Right Renal Artery, Percutaneous Approach */
"04B94ZZ"  =  "1"  /* Excision of Right Renal Artery, Percutaneous Endoscopic Approach */
"04BA0ZZ"  =  "1"  /* Excision of Left Renal Artery, Open Approach */
"04BA3ZZ"  =  "1"  /* Excision of Left Renal Artery, Percutaneous Approach */
"04BA4ZZ"  =  "1"  /* Excision of Left Renal Artery, Percutaneous Endoscopic Approach */
"04BB0ZZ"  =  "1"  /* Excision of Inferior Mesenteric Artery, Open Approach */
"04BB3ZZ"  =  "1"  /* Excision of Inferior Mesenteric Artery, Percutaneous Approach */
"04BB4ZZ"  =  "1"  /* Excision of Inferior Mesenteric Artery, Percutaneous Endoscopic Approach */
"04BC0ZZ"  =  "1"  /* Excision of Right Common Iliac Artery, Open Approach */
"04BC3ZZ"  =  "1"  /* Excision of Right Common Iliac Artery, Percutaneous Approach */
"04BC4ZZ"  =  "1"  /* Excision of Right Common Iliac Artery, Percutaneous Endoscopic Approach */
"04BD0ZZ"  =  "1"  /* Excision of Left Common Iliac Artery, Open Approach */
"04BD3ZZ"  =  "1"  /* Excision of Left Common Iliac Artery, Percutaneous Approach */
"04BD4ZZ"  =  "1"  /* Excision of Left Common Iliac Artery, Percutaneous Endoscopic Approach */
"04BE0ZZ"  =  "1"  /* Excision of Right Internal Iliac Artery, Open Approach */
"04BE3ZZ"  =  "1"  /* Excision of Right Internal Iliac Artery, Percutaneous Approach */
"04BE4ZZ"  =  "1"  /* Excision of Right Internal Iliac Artery, Percutaneous Endoscopic Approach */
"04BF0ZZ"  =  "1"  /* Excision of Left Internal Iliac Artery, Open Approach */
"04BF3ZZ"  =  "1"  /* Excision of Left Internal Iliac Artery, Percutaneous Approach */
"04BF4ZZ"  =  "1"  /* Excision of Left Internal Iliac Artery, Percutaneous Endoscopic Approach */
"04BH0ZZ"  =  "1"  /* Excision of Right External Iliac Artery, Open Approach */
"04BH3ZZ"  =  "1"  /* Excision of Right External Iliac Artery, Percutaneous Approach */
"04BH4ZZ"  =  "1"  /* Excision of Right External Iliac Artery, Percutaneous Endoscopic Approach */
"04BJ0ZZ"  =  "1"  /* Excision of Left External Iliac Artery, Open Approach */
"04BJ3ZZ"  =  "1"  /* Excision of Left External Iliac Artery, Percutaneous Approach */
"04BJ4ZZ"  =  "1"  /* Excision of Left External Iliac Artery, Percutaneous Endoscopic Approach */
"04C00ZZ"  =  "1"  /* Extirpation of Matter from Abdominal Aorta, Open Approach */
"04C03ZZ"  =  "1"  /* Extirpation of Matter from Abdominal Aorta, Percutaneous Approach */
"04C04ZZ"  =  "1"  /* Extirpation of Matter from Abdominal Aorta, Percutaneous Endoscopic Approach */
"04C10ZZ"  =  "1"  /* Extirpation of Matter from Celiac Artery, Open Approach */
"04C13ZZ"  =  "1"  /* Extirpation of Matter from Celiac Artery, Percutaneous Approach */
"04C14ZZ"  =  "1"  /* Extirpation of Matter from Celiac Artery, Percutaneous Endoscopic Approach */
"04C20ZZ"  =  "1"  /* Extirpation of Matter from Gastric Artery, Open Approach */
"04C23ZZ"  =  "1"  /* Extirpation of Matter from Gastric Artery, Percutaneous Approach */
"04C24ZZ"  =  "1"  /* Extirpation of Matter from Gastric Artery, Percutaneous Endoscopic Approach */
"04C30ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Artery, Open Approach */
"04C33ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Artery, Percutaneous Approach */
"04C34ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Artery, Percutaneous Endoscopic Approach */
"04C40ZZ"  =  "1"  /* Extirpation of Matter from Splenic Artery, Open Approach */
"04C43ZZ"  =  "1"  /* Extirpation of Matter from Splenic Artery, Percutaneous Approach */
"04C44ZZ"  =  "1"  /* Extirpation of Matter from Splenic Artery, Percutaneous Endoscopic Approach */
"04C50ZZ"  =  "1"  /* Extirpation of Matter from Superior Mesenteric Artery, Open Approach */
"04C53ZZ"  =  "1"  /* Extirpation of Matter from Superior Mesenteric Artery, Percutaneous Approach */
"04C54ZZ"  =  "1"  /* Extirpation of Matter from Superior Mesenteric Artery, Percutaneous Endoscopic Approach */
"04C60ZZ"  =  "1"  /* Extirpation of Matter from Right Colic Artery, Open Approach */
"04C63ZZ"  =  "1"  /* Extirpation of Matter from Right Colic Artery, Percutaneous Approach */
"04C64ZZ"  =  "1"  /* Extirpation of Matter from Right Colic Artery, Percutaneous Endoscopic Approach */
"04C70ZZ"  =  "1"  /* Extirpation of Matter from Left Colic Artery, Open Approach */
"04C73ZZ"  =  "1"  /* Extirpation of Matter from Left Colic Artery, Percutaneous Approach */
"04C74ZZ"  =  "1"  /* Extirpation of Matter from Left Colic Artery, Percutaneous Endoscopic Approach */
"04C80ZZ"  =  "1"  /* Extirpation of Matter from Middle Colic Artery, Open Approach */
"04C83ZZ"  =  "1"  /* Extirpation of Matter from Middle Colic Artery, Percutaneous Approach */
"04C84ZZ"  =  "1"  /* Extirpation of Matter from Middle Colic Artery, Percutaneous Endoscopic Approach */
"04C90ZZ"  =  "1"  /* Extirpation of Matter from Right Renal Artery, Open Approach */
"04C93ZZ"  =  "1"  /* Extirpation of Matter from Right Renal Artery, Percutaneous Approach */
"04C94ZZ"  =  "1"  /* Extirpation of Matter from Right Renal Artery, Percutaneous Endoscopic Approach */
"04CA0ZZ"  =  "1"  /* Extirpation of Matter from Left Renal Artery, Open Approach */
"04CA3ZZ"  =  "1"  /* Extirpation of Matter from Left Renal Artery, Percutaneous Approach */
"04CA4ZZ"  =  "1"  /* Extirpation of Matter from Left Renal Artery, Percutaneous Endoscopic Approach */
"04CB0ZZ"  =  "1"  /* Extirpation of Matter from Inferior Mesenteric Artery, Open Approach */
"04CB3ZZ"  =  "1"  /* Extirpation of Matter from Inferior Mesenteric Artery, Percutaneous Approach */
"04CB4ZZ"  =  "1"  /* Extirpation of Matter from Inferior Mesenteric Artery, Percutaneous Endoscopic Approach */
"04CC0ZZ"  =  "1"  /* Extirpation of Matter from Right Common Iliac Artery, Open Approach */
"04CC3ZZ"  =  "1"  /* Extirpation of Matter from Right Common Iliac Artery, Percutaneous Approach */
"04CC4ZZ"  =  "1"  /* Extirpation of Matter from Right Common Iliac Artery, Percutaneous Endoscopic Approach */
"04CD0ZZ"  =  "1"  /* Extirpation of Matter from Left Common Iliac Artery, Open Approach */
"04CD3ZZ"  =  "1"  /* Extirpation of Matter from Left Common Iliac Artery, Percutaneous Approach */
"04CD4ZZ"  =  "1"  /* Extirpation of Matter from Left Common Iliac Artery, Percutaneous Endoscopic Approach */
"04CE0ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Iliac Artery, Open Approach */
"04CE3ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Iliac Artery, Percutaneous Approach */
"04CE4ZZ"  =  "1"  /* Extirpation of Matter from Right Internal Iliac Artery, Percutaneous Endoscopic Approach */
"04CF0ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Iliac Artery, Open Approach */
"04CF3ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Iliac Artery, Percutaneous Approach */
"04CF4ZZ"  =  "1"  /* Extirpation of Matter from Left Internal Iliac Artery, Percutaneous Endoscopic Approach */
"04CH0ZZ"  =  "1"  /* Extirpation of Matter from Right External Iliac Artery, Open Approach */
"04CH3ZZ"  =  "1"  /* Extirpation of Matter from Right External Iliac Artery, Percutaneous Approach */
"04CH4ZZ"  =  "1"  /* Extirpation of Matter from Right External Iliac Artery, Percutaneous Endoscopic Approach */
"04CJ0ZZ"  =  "1"  /* Extirpation of Matter from Left External Iliac Artery, Open Approach */
"04CJ3ZZ"  =  "1"  /* Extirpation of Matter from Left External Iliac Artery, Percutaneous Approach */
"04CJ4ZZ"  =  "1"  /* Extirpation of Matter from Left External Iliac Artery, Percutaneous Endoscopic Approach */
"04L00CZ"  =  "1"  /* Occlusion of Abdominal Aorta with Extraluminal Device, Open Approach */
"04L00DZ"  =  "1"  /* Occlusion of Abdominal Aorta with Intraluminal Device, Open Approach */
"04L00ZZ"  =  "1"  /* Occlusion of Abdominal Aorta, Open Approach */
"04L03CZ"  =  "1"  /* Occlusion of Abdominal Aorta with Extraluminal Device, Percutaneous Approach */
"04L03DZ"  =  "1"  /* Occlusion of Abdominal Aorta with Intraluminal Device, Percutaneous Approach */
"04L03ZZ"  =  "1"  /* Occlusion of Abdominal Aorta, Percutaneous Approach */
"04L04CZ"  =  "1"  /* Occlusion of Abdominal Aorta with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L04DZ"  =  "1"  /* Occlusion of Abdominal Aorta with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L04ZZ"  =  "1"  /* Occlusion of Abdominal Aorta, Percutaneous Endoscopic Approach */
"04L10CZ"  =  "1"  /* Occlusion of Celiac Artery with Extraluminal Device, Open Approach */
"04L10DZ"  =  "1"  /* Occlusion of Celiac Artery with Intraluminal Device, Open Approach */
"04L10ZZ"  =  "1"  /* Occlusion of Celiac Artery, Open Approach */
"04L13CZ"  =  "1"  /* Occlusion of Celiac Artery with Extraluminal Device, Percutaneous Approach */
"04L13DZ"  =  "1"  /* Occlusion of Celiac Artery with Intraluminal Device, Percutaneous Approach */
"04L13ZZ"  =  "1"  /* Occlusion of Celiac Artery, Percutaneous Approach */
"04L14CZ"  =  "1"  /* Occlusion of Celiac Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L14DZ"  =  "1"  /* Occlusion of Celiac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L14ZZ"  =  "1"  /* Occlusion of Celiac Artery, Percutaneous Endoscopic Approach */
"04L20CZ"  =  "1"  /* Occlusion of Gastric Artery with Extraluminal Device, Open Approach */
"04L20DZ"  =  "1"  /* Occlusion of Gastric Artery with Intraluminal Device, Open Approach */
"04L20ZZ"  =  "1"  /* Occlusion of Gastric Artery, Open Approach */
"04L23CZ"  =  "1"  /* Occlusion of Gastric Artery with Extraluminal Device, Percutaneous Approach */
"04L23ZZ"  =  "1"  /* Occlusion of Gastric Artery, Percutaneous Approach */
"04L24CZ"  =  "1"  /* Occlusion of Gastric Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L24DZ"  =  "1"  /* Occlusion of Gastric Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L24ZZ"  =  "1"  /* Occlusion of Gastric Artery, Percutaneous Endoscopic Approach */
"04L30CZ"  =  "1"  /* Occlusion of Hepatic Artery with Extraluminal Device, Open Approach */
"04L30DZ"  =  "1"  /* Occlusion of Hepatic Artery with Intraluminal Device, Open Approach */
"04L30ZZ"  =  "1"  /* Occlusion of Hepatic Artery, Open Approach */
"04L33CZ"  =  "1"  /* Occlusion of Hepatic Artery with Extraluminal Device, Percutaneous Approach */
"04L33DZ"  =  "1"  /* Occlusion of Hepatic Artery with Intraluminal Device, Percutaneous Approach */
"04L33ZZ"  =  "1"  /* Occlusion of Hepatic Artery, Percutaneous Approach */
"04L34CZ"  =  "1"  /* Occlusion of Hepatic Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L34DZ"  =  "1"  /* Occlusion of Hepatic Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L34ZZ"  =  "1"  /* Occlusion of Hepatic Artery, Percutaneous Endoscopic Approach */
"04L40CZ"  =  "1"  /* Occlusion of Splenic Artery with Extraluminal Device, Open Approach */
"04L40DZ"  =  "1"  /* Occlusion of Splenic Artery with Intraluminal Device, Open Approach */
"04L40ZZ"  =  "1"  /* Occlusion of Splenic Artery, Open Approach */
"04L43CZ"  =  "1"  /* Occlusion of Splenic Artery with Extraluminal Device, Percutaneous Approach */
"04L43DZ"  =  "1"  /* Occlusion of Splenic Artery with Intraluminal Device, Percutaneous Approach */
"04L43ZZ"  =  "1"  /* Occlusion of Splenic Artery, Percutaneous Approach */
"04L44CZ"  =  "1"  /* Occlusion of Splenic Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L44DZ"  =  "1"  /* Occlusion of Splenic Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L44ZZ"  =  "1"  /* Occlusion of Splenic Artery, Percutaneous Endoscopic Approach */
"04L50CZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery with Extraluminal Device, Open Approach */
"04L50DZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery with Intraluminal Device, Open Approach */
"04L50ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery, Open Approach */
"04L53CZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery with Extraluminal Device, Percutaneous Approach */
"04L53DZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery with Intraluminal Device, Percutaneous Approach */
"04L53ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery, Percutaneous Approach */
"04L54CZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L54DZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L54ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery, Percutaneous Endoscopic Approach */
"04L60CZ"  =  "1"  /* Occlusion of Right Colic Artery with Extraluminal Device, Open Approach */
"04L60DZ"  =  "1"  /* Occlusion of Right Colic Artery with Intraluminal Device, Open Approach */
"04L60ZZ"  =  "1"  /* Occlusion of Right Colic Artery, Open Approach */
"04L63CZ"  =  "1"  /* Occlusion of Right Colic Artery with Extraluminal Device, Percutaneous Approach */
"04L63DZ"  =  "1"  /* Occlusion of Right Colic Artery with Intraluminal Device, Percutaneous Approach */
"04L63ZZ"  =  "1"  /* Occlusion of Right Colic Artery, Percutaneous Approach */
"04L64CZ"  =  "1"  /* Occlusion of Right Colic Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L64DZ"  =  "1"  /* Occlusion of Right Colic Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L64ZZ"  =  "1"  /* Occlusion of Right Colic Artery, Percutaneous Endoscopic Approach */
"04L70CZ"  =  "1"  /* Occlusion of Left Colic Artery with Extraluminal Device, Open Approach */
"04L70DZ"  =  "1"  /* Occlusion of Left Colic Artery with Intraluminal Device, Open Approach */
"04L70ZZ"  =  "1"  /* Occlusion of Left Colic Artery, Open Approach */
"04L73CZ"  =  "1"  /* Occlusion of Left Colic Artery with Extraluminal Device, Percutaneous Approach */
"04L73DZ"  =  "1"  /* Occlusion of Left Colic Artery with Intraluminal Device, Percutaneous Approach */
"04L73ZZ"  =  "1"  /* Occlusion of Left Colic Artery, Percutaneous Approach */
"04L74CZ"  =  "1"  /* Occlusion of Left Colic Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L74DZ"  =  "1"  /* Occlusion of Left Colic Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L74ZZ"  =  "1"  /* Occlusion of Left Colic Artery, Percutaneous Endoscopic Approach */
"04L80CZ"  =  "1"  /* Occlusion of Middle Colic Artery with Extraluminal Device, Open Approach */
"04L80DZ"  =  "1"  /* Occlusion of Middle Colic Artery with Intraluminal Device, Open Approach */
"04L80ZZ"  =  "1"  /* Occlusion of Middle Colic Artery, Open Approach */
"04L83CZ"  =  "1"  /* Occlusion of Middle Colic Artery with Extraluminal Device, Percutaneous Approach */
"04L83DZ"  =  "1"  /* Occlusion of Middle Colic Artery with Intraluminal Device, Percutaneous Approach */
"04L83ZZ"  =  "1"  /* Occlusion of Middle Colic Artery, Percutaneous Approach */
"04L84CZ"  =  "1"  /* Occlusion of Middle Colic Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L84DZ"  =  "1"  /* Occlusion of Middle Colic Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L84ZZ"  =  "1"  /* Occlusion of Middle Colic Artery, Percutaneous Endoscopic Approach */
"04L90CZ"  =  "1"  /* Occlusion of Right Renal Artery with Extraluminal Device, Open Approach */
"04L90DZ"  =  "1"  /* Occlusion of Right Renal Artery with Intraluminal Device, Open Approach */
"04L90ZZ"  =  "1"  /* Occlusion of Right Renal Artery, Open Approach */
"04L93CZ"  =  "1"  /* Occlusion of Right Renal Artery with Extraluminal Device, Percutaneous Approach */
"04L93DZ"  =  "1"  /* Occlusion of Right Renal Artery with Intraluminal Device, Percutaneous Approach */
"04L93ZZ"  =  "1"  /* Occlusion of Right Renal Artery, Percutaneous Approach */
"04L94CZ"  =  "1"  /* Occlusion of Right Renal Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04L94DZ"  =  "1"  /* Occlusion of Right Renal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04L94ZZ"  =  "1"  /* Occlusion of Right Renal Artery, Percutaneous Endoscopic Approach */
"04LA0CZ"  =  "1"  /* Occlusion of Left Renal Artery with Extraluminal Device, Open Approach */
"04LA0DZ"  =  "1"  /* Occlusion of Left Renal Artery with Intraluminal Device, Open Approach */
"04LA0ZZ"  =  "1"  /* Occlusion of Left Renal Artery, Open Approach */
"04LA3CZ"  =  "1"  /* Occlusion of Left Renal Artery with Extraluminal Device, Percutaneous Approach */
"04LA3DZ"  =  "1"  /* Occlusion of Left Renal Artery with Intraluminal Device, Percutaneous Approach */
"04LA3ZZ"  =  "1"  /* Occlusion of Left Renal Artery, Percutaneous Approach */
"04LA4CZ"  =  "1"  /* Occlusion of Left Renal Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LA4DZ"  =  "1"  /* Occlusion of Left Renal Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LA4ZZ"  =  "1"  /* Occlusion of Left Renal Artery, Percutaneous Endoscopic Approach */
"04LB0CZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery with Extraluminal Device, Open Approach */
"04LB0DZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery with Intraluminal Device, Open Approach */
"04LB0ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery, Open Approach */
"04LB3CZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery with Extraluminal Device, Percutaneous Approach */
"04LB3DZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery with Intraluminal Device, Percutaneous Approach */
"04LB3ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery, Percutaneous Approach */
"04LB4CZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LB4DZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LB4ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery, Percutaneous Endoscopic Approach */
"04LC0CZ"  =  "1"  /* Occlusion of Right Common Iliac Artery with Extraluminal Device, Open Approach */
"04LC0DZ"  =  "1"  /* Occlusion of Right Common Iliac Artery with Intraluminal Device, Open Approach */
"04LC0ZZ"  =  "1"  /* Occlusion of Right Common Iliac Artery, Open Approach */
"04LC3CZ"  =  "1"  /* Occlusion of Right Common Iliac Artery with Extraluminal Device, Percutaneous Approach */
"04LC3DZ"  =  "1"  /* Occlusion of Right Common Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04LC3ZZ"  =  "1"  /* Occlusion of Right Common Iliac Artery, Percutaneous Approach */
"04LC4CZ"  =  "1"  /* Occlusion of Right Common Iliac Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LC4DZ"  =  "1"  /* Occlusion of Right Common Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LC4ZZ"  =  "1"  /* Occlusion of Right Common Iliac Artery, Percutaneous Endoscopic Approach */
"04LD0CZ"  =  "1"  /* Occlusion of Left Common Iliac Artery with Extraluminal Device, Open Approach */
"04LD0DZ"  =  "1"  /* Occlusion of Left Common Iliac Artery with Intraluminal Device, Open Approach */
"04LD0ZZ"  =  "1"  /* Occlusion of Left Common Iliac Artery, Open Approach */
"04LD3CZ"  =  "1"  /* Occlusion of Left Common Iliac Artery with Extraluminal Device, Percutaneous Approach */
"04LD3DZ"  =  "1"  /* Occlusion of Left Common Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04LD3ZZ"  =  "1"  /* Occlusion of Left Common Iliac Artery, Percutaneous Approach */
"04LD4CZ"  =  "1"  /* Occlusion of Left Common Iliac Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LD4DZ"  =  "1"  /* Occlusion of Left Common Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LD4ZZ"  =  "1"  /* Occlusion of Left Common Iliac Artery, Percutaneous Endoscopic Approach */
"04LE0CZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery with Extraluminal Device, Open Approach */
"04LE0DZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery with Intraluminal Device, Open Approach */
"04LE0ZZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery, Open Approach */
"04LE3CZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery with Extraluminal Device, Percutaneous Approach */
"04LE3DZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04LE3ZZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery, Percutaneous Approach */
"04LE4CZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LE4DZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LE4ZZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery, Percutaneous Endoscopic Approach */
"04LF0CZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery with Extraluminal Device, Open Approach */
"04LF0DZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery with Intraluminal Device, Open Approach */
"04LF0ZZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery, Open Approach */
"04LF3CZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery with Extraluminal Device, Percutaneous Approach */
"04LF3DZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04LF3ZZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery, Percutaneous Approach */
"04LF4CZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LF4DZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LF4ZZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery, Percutaneous Endoscopic Approach */
"04LH0CZ"  =  "1"  /* Occlusion of Right External Iliac Artery with Extraluminal Device, Open Approach */
"04LH0DZ"  =  "1"  /* Occlusion of Right External Iliac Artery with Intraluminal Device, Open Approach */
"04LH0ZZ"  =  "1"  /* Occlusion of Right External Iliac Artery, Open Approach */
"04LH3CZ"  =  "1"  /* Occlusion of Right External Iliac Artery with Extraluminal Device, Percutaneous Approach */
"04LH3DZ"  =  "1"  /* Occlusion of Right External Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04LH3ZZ"  =  "1"  /* Occlusion of Right External Iliac Artery, Percutaneous Approach */
"04LH4CZ"  =  "1"  /* Occlusion of Right External Iliac Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LH4DZ"  =  "1"  /* Occlusion of Right External Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LH4ZZ"  =  "1"  /* Occlusion of Right External Iliac Artery, Percutaneous Endoscopic Approach */
"04LJ0CZ"  =  "1"  /* Occlusion of Left External Iliac Artery with Extraluminal Device, Open Approach */
"04LJ0DZ"  =  "1"  /* Occlusion of Left External Iliac Artery with Intraluminal Device, Open Approach */
"04LJ0ZZ"  =  "1"  /* Occlusion of Left External Iliac Artery, Open Approach */
"04LJ3CZ"  =  "1"  /* Occlusion of Left External Iliac Artery with Extraluminal Device, Percutaneous Approach */
"04LJ3DZ"  =  "1"  /* Occlusion of Left External Iliac Artery with Intraluminal Device, Percutaneous Approach */
"04LJ3ZZ"  =  "1"  /* Occlusion of Left External Iliac Artery, Percutaneous Approach */
"04LJ4CZ"  =  "1"  /* Occlusion of Left External Iliac Artery with Extraluminal Device, Percutaneous Endoscopic Approach */
"04LJ4DZ"  =  "1"  /* Occlusion of Left External Iliac Artery with Intraluminal Device, Percutaneous Endoscopic Approach */
"04LJ4ZZ"  =  "1"  /* Occlusion of Left External Iliac Artery, Percutaneous Endoscopic Approach */
"04R007Z"  =  "1"  /* Replacement of Abdominal Aorta with Autologous Tissue Substitute, Open Approach */
"04R00JZ"  =  "1"  /* Replacement of Abdominal Aorta with Synthetic Substitute, Open Approach */
"04R00KZ"  =  "1"  /* Replacement of Abdominal Aorta with Nonautologous Tissue Substitute, Open Approach */
"04R047Z"  =  "1"  /* Replacement of Abdominal Aorta with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R04JZ"  =  "1"  /* Replacement of Abdominal Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04R04KZ"  =  "1"  /* Replacement of Abdominal Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R107Z"  =  "1"  /* Replacement of Celiac Artery with Autologous Tissue Substitute, Open Approach */
"04R10JZ"  =  "1"  /* Replacement of Celiac Artery with Synthetic Substitute, Open Approach */
"04R10KZ"  =  "1"  /* Replacement of Celiac Artery with Nonautologous Tissue Substitute, Open Approach */
"04R147Z"  =  "1"  /* Replacement of Celiac Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R14JZ"  =  "1"  /* Replacement of Celiac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04R14KZ"  =  "1"  /* Replacement of Celiac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R207Z"  =  "1"  /* Replacement of Gastric Artery with Autologous Tissue Substitute, Open Approach */
"04R20JZ"  =  "1"  /* Replacement of Gastric Artery with Synthetic Substitute, Open Approach */
"04R20KZ"  =  "1"  /* Replacement of Gastric Artery with Nonautologous Tissue Substitute, Open Approach */
"04R247Z"  =  "1"  /* Replacement of Gastric Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R24JZ"  =  "1"  /* Replacement of Gastric Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04R24KZ"  =  "1"  /* Replacement of Gastric Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R307Z"  =  "1"  /* Replacement of Hepatic Artery with Autologous Tissue Substitute, Open Approach */
"04R30JZ"  =  "1"  /* Replacement of Hepatic Artery with Synthetic Substitute, Open Approach */
"04R30KZ"  =  "1"  /* Replacement of Hepatic Artery with Nonautologous Tissue Substitute, Open Approach */
"04R347Z"  =  "1"  /* Replacement of Hepatic Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R34JZ"  =  "1"  /* Replacement of Hepatic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04R34KZ"  =  "1"  /* Replacement of Hepatic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R407Z"  =  "1"  /* Replacement of Splenic Artery with Autologous Tissue Substitute, Open Approach */
"04R40JZ"  =  "1"  /* Replacement of Splenic Artery with Synthetic Substitute, Open Approach */
"04R40KZ"  =  "1"  /* Replacement of Splenic Artery with Nonautologous Tissue Substitute, Open Approach */
"04R447Z"  =  "1"  /* Replacement of Splenic Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R44JZ"  =  "1"  /* Replacement of Splenic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04R44KZ"  =  "1"  /* Replacement of Splenic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R507Z"  =  "1"  /* Replacement of Superior Mesenteric Artery with Autologous Tissue Substitute, Open Approach */
"04R50JZ"  =  "1"  /* Replacement of Superior Mesenteric Artery with Synthetic Substitute, Open Approach */
"04R50KZ"  =  "1"  /* Replacement of Superior Mesenteric Artery with Nonautologous Tissue Substitute, Open Approach */
"04R547Z"  =  "1"  /* Replacement of Superior Mesenteric Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R54JZ"  =  "1"  /* Replacement of Superior Mesenteric Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04R54KZ"  =  "1"  /* Replacement of Superior Mesenteric Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R607Z"  =  "1"  /* Replacement of Right Colic Artery with Autologous Tissue Substitute, Open Approach */
"04R60JZ"  =  "1"  /* Replacement of Right Colic Artery with Synthetic Substitute, Open Approach */
"04R60KZ"  =  "1"  /* Replacement of Right Colic Artery with Nonautologous Tissue Substitute, Open Approach */
"04R647Z"  =  "1"  /* Replacement of Right Colic Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R64JZ"  =  "1"  /* Replacement of Right Colic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04R64KZ"  =  "1"  /* Replacement of Right Colic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R707Z"  =  "1"  /* Replacement of Left Colic Artery with Autologous Tissue Substitute, Open Approach */
"04R70JZ"  =  "1"  /* Replacement of Left Colic Artery with Synthetic Substitute, Open Approach */
"04R70KZ"  =  "1"  /* Replacement of Left Colic Artery with Nonautologous Tissue Substitute, Open Approach */
"04R747Z"  =  "1"  /* Replacement of Left Colic Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R74JZ"  =  "1"  /* Replacement of Left Colic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04R74KZ"  =  "1"  /* Replacement of Left Colic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R807Z"  =  "1"  /* Replacement of Middle Colic Artery with Autologous Tissue Substitute, Open Approach */
"04R80JZ"  =  "1"  /* Replacement of Middle Colic Artery with Synthetic Substitute, Open Approach */
"04R80KZ"  =  "1"  /* Replacement of Middle Colic Artery with Nonautologous Tissue Substitute, Open Approach */
"04R847Z"  =  "1"  /* Replacement of Middle Colic Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R84JZ"  =  "1"  /* Replacement of Middle Colic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04R84KZ"  =  "1"  /* Replacement of Middle Colic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R907Z"  =  "1"  /* Replacement of Right Renal Artery with Autologous Tissue Substitute, Open Approach */
"04R90JZ"  =  "1"  /* Replacement of Right Renal Artery with Synthetic Substitute, Open Approach */
"04R90KZ"  =  "1"  /* Replacement of Right Renal Artery with Nonautologous Tissue Substitute, Open Approach */
"04R947Z"  =  "1"  /* Replacement of Right Renal Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04R94JZ"  =  "1"  /* Replacement of Right Renal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04R94KZ"  =  "1"  /* Replacement of Right Renal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04RA07Z"  =  "1"  /* Replacement of Left Renal Artery with Autologous Tissue Substitute, Open Approach */
"04RA0JZ"  =  "1"  /* Replacement of Left Renal Artery with Synthetic Substitute, Open Approach */
"04RA0KZ"  =  "1"  /* Replacement of Left Renal Artery with Nonautologous Tissue Substitute, Open Approach */
"04RA47Z"  =  "1"  /* Replacement of Left Renal Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04RA4JZ"  =  "1"  /* Replacement of Left Renal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04RA4KZ"  =  "1"  /* Replacement of Left Renal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04RB07Z"  =  "1"  /* Replacement of Inferior Mesenteric Artery with Autologous Tissue Substitute, Open Approach */
"04RB0JZ"  =  "1"  /* Replacement of Inferior Mesenteric Artery with Synthetic Substitute, Open Approach */
"04RB0KZ"  =  "1"  /* Replacement of Inferior Mesenteric Artery with Nonautologous Tissue Substitute, Open Approach */
"04RB47Z"  =  "1"  /* Replacement of Inferior Mesenteric Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04RB4JZ"  =  "1"  /* Replacement of Inferior Mesenteric Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04RB4KZ"  =  "1"  /* Replacement of Inferior Mesenteric Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04RC07Z"  =  "1"  /* Replacement of Right Common Iliac Artery with Autologous Tissue Substitute, Open Approach */
"04RC0JZ"  =  "1"  /* Replacement of Right Common Iliac Artery with Synthetic Substitute, Open Approach */
"04RC0KZ"  =  "1"  /* Replacement of Right Common Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"04RC47Z"  =  "1"  /* Replacement of Right Common Iliac Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04RC4JZ"  =  "1"  /* Replacement of Right Common Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04RC4KZ"  =  "1"  /* Replacement of Right Common Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04RD07Z"  =  "1"  /* Replacement of Left Common Iliac Artery with Autologous Tissue Substitute, Open Approach */
"04RD0JZ"  =  "1"  /* Replacement of Left Common Iliac Artery with Synthetic Substitute, Open Approach */
"04RD0KZ"  =  "1"  /* Replacement of Left Common Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"04RD47Z"  =  "1"  /* Replacement of Left Common Iliac Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04RD4JZ"  =  "1"  /* Replacement of Left Common Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04RD4KZ"  =  "1"  /* Replacement of Left Common Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04RE07Z"  =  "1"  /* Replacement of Right Internal Iliac Artery with Autologous Tissue Substitute, Open Approach */
"04RE0JZ"  =  "1"  /* Replacement of Right Internal Iliac Artery with Synthetic Substitute, Open Approach */
"04RE0KZ"  =  "1"  /* Replacement of Right Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"04RE47Z"  =  "1"  /* Replacement of Right Internal Iliac Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04RE4JZ"  =  "1"  /* Replacement of Right Internal Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04RE4KZ"  =  "1"  /* Replacement of Right Internal Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04RF07Z"  =  "1"  /* Replacement of Left Internal Iliac Artery with Autologous Tissue Substitute, Open Approach */
"04RF0JZ"  =  "1"  /* Replacement of Left Internal Iliac Artery with Synthetic Substitute, Open Approach */
"04RF0KZ"  =  "1"  /* Replacement of Left Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"04RF47Z"  =  "1"  /* Replacement of Left Internal Iliac Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04RF4JZ"  =  "1"  /* Replacement of Left Internal Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04RF4KZ"  =  "1"  /* Replacement of Left Internal Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04RH07Z"  =  "1"  /* Replacement of Right External Iliac Artery with Autologous Tissue Substitute, Open Approach */
"04RH0JZ"  =  "1"  /* Replacement of Right External Iliac Artery with Synthetic Substitute, Open Approach */
"04RH0KZ"  =  "1"  /* Replacement of Right External Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"04RH47Z"  =  "1"  /* Replacement of Right External Iliac Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04RH4JZ"  =  "1"  /* Replacement of Right External Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04RH4KZ"  =  "1"  /* Replacement of Right External Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04RJ07Z"  =  "1"  /* Replacement of Left External Iliac Artery with Autologous Tissue Substitute, Open Approach */
"04RJ0JZ"  =  "1"  /* Replacement of Left External Iliac Artery with Synthetic Substitute, Open Approach */
"04RJ0KZ"  =  "1"  /* Replacement of Left External Iliac Artery with Nonautologous Tissue Substitute, Open Approach */
"04RJ47Z"  =  "1"  /* Replacement of Left External Iliac Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"04RJ4JZ"  =  "1"  /* Replacement of Left External Iliac Artery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"04RJ4KZ"  =  "1"  /* Replacement of Left External Iliac Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0610075"  =  "1"  /* Bypass Inferior Vena Cava to Superior Mesenteric Vein with Autologous Tissue Substitute, Open Approach */
"0610076"  =  "1"  /* Bypass Inferior Vena Cava to Inferior Mesenteric Vein with Autologous Tissue Substitute, Open Approach */
"061007Y"  =  "1"  /* Bypass Inferior Vena Cava to Lower Vein with Autologous Tissue Substitute, Open Approach */
"0610095"  =  "1"  /* Bypass Inferior Vena Cava to Superior Mesenteric Vein with Autologous Venous Tissue, Open Approach */
"0610096"  =  "1"  /* Bypass Inferior Vena Cava to Inferior Mesenteric Vein with Autologous Venous Tissue, Open Approach */
"061009Y"  =  "1"  /* Bypass Inferior Vena Cava to Lower Vein with Autologous Venous Tissue, Open Approach */
"06100A5"  =  "1"  /* Bypass Inferior Vena Cava to Superior Mesenteric Vein with Autologous Arterial Tissue, Open Approach */
"06100A6"  =  "1"  /* Bypass Inferior Vena Cava to Inferior Mesenteric Vein with Autologous Arterial Tissue, Open Approach */
"06100AY"  =  "1"  /* Bypass Inferior Vena Cava to Lower Vein with Autologous Arterial Tissue, Open Approach */
"06100J5"  =  "1"  /* Bypass Inferior Vena Cava to Superior Mesenteric Vein with Synthetic Substitute, Open Approach */
"06100J6"  =  "1"  /* Bypass Inferior Vena Cava to Inferior Mesenteric Vein with Synthetic Substitute, Open Approach */
"06100JY"  =  "1"  /* Bypass Inferior Vena Cava to Lower Vein with Synthetic Substitute, Open Approach */
"06100K5"  =  "1"  /* Bypass Inferior Vena Cava to Superior Mesenteric Vein with Nonautologous Tissue Substitute, Open Approach */
"06100K6"  =  "1"  /* Bypass Inferior Vena Cava to Inferior Mesenteric Vein with Nonautologous Tissue Substitute, Open Approach */
"06100KY"  =  "1"  /* Bypass Inferior Vena Cava to Lower Vein with Nonautologous Tissue Substitute, Open Approach */
"06100Z5"  =  "1"  /* Bypass Inferior Vena Cava to Superior Mesenteric Vein, Open Approach */
"06100Z6"  =  "1"  /* Bypass Inferior Vena Cava to Inferior Mesenteric Vein, Open Approach */
"06100ZY"  =  "1"  /* Bypass Inferior Vena Cava to Lower Vein, Open Approach */
"0610475"  =  "1"  /* Bypass Inferior Vena Cava to Superior Mesenteric Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0610476"  =  "1"  /* Bypass Inferior Vena Cava to Inferior Mesenteric Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"061047Y"  =  "1"  /* Bypass Inferior Vena Cava to Lower Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0610495"  =  "1"  /* Bypass Inferior Vena Cava to Superior Mesenteric Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"0610496"  =  "1"  /* Bypass Inferior Vena Cava to Inferior Mesenteric Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"061049Y"  =  "1"  /* Bypass Inferior Vena Cava to Lower Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"06104A5"  =  "1"  /* Bypass Inferior Vena Cava to Superior Mesenteric Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"06104A6"  =  "1"  /* Bypass Inferior Vena Cava to Inferior Mesenteric Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"06104AY"  =  "1"  /* Bypass Inferior Vena Cava to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"06104J5"  =  "1"  /* Bypass Inferior Vena Cava to Superior Mesenteric Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06104J6"  =  "1"  /* Bypass Inferior Vena Cava to Inferior Mesenteric Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06104JY"  =  "1"  /* Bypass Inferior Vena Cava to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06104K5"  =  "1"  /* Bypass Inferior Vena Cava to Superior Mesenteric Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06104K6"  =  "1"  /* Bypass Inferior Vena Cava to Inferior Mesenteric Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06104KY"  =  "1"  /* Bypass Inferior Vena Cava to Lower Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06104Z5"  =  "1"  /* Bypass Inferior Vena Cava to Superior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06104Z6"  =  "1"  /* Bypass Inferior Vena Cava to Inferior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06104ZY"  =  "1"  /* Bypass Inferior Vena Cava to Lower Vein, Percutaneous Endoscopic Approach */
"0611079"  =  "1"  /* Bypass Splenic Vein to Right Renal Vein with Autologous Tissue Substitute, Open Approach */
"061107B"  =  "1"  /* Bypass Splenic Vein to Left Renal Vein with Autologous Tissue Substitute, Open Approach */
"061107Y"  =  "1"  /* Bypass Splenic Vein to Lower Vein with Autologous Tissue Substitute, Open Approach */
"0611099"  =  "1"  /* Bypass Splenic Vein to Right Renal Vein with Autologous Venous Tissue, Open Approach */
"061109B"  =  "1"  /* Bypass Splenic Vein to Left Renal Vein with Autologous Venous Tissue, Open Approach */
"061109Y"  =  "1"  /* Bypass Splenic Vein to Lower Vein with Autologous Venous Tissue, Open Approach */
"06110A9"  =  "1"  /* Bypass Splenic Vein to Right Renal Vein with Autologous Arterial Tissue, Open Approach */
"06110AB"  =  "1"  /* Bypass Splenic Vein to Left Renal Vein with Autologous Arterial Tissue, Open Approach */
"06110AY"  =  "1"  /* Bypass Splenic Vein to Lower Vein with Autologous Arterial Tissue, Open Approach */
"06110J9"  =  "1"  /* Bypass Splenic Vein to Right Renal Vein with Synthetic Substitute, Open Approach */
"06110JB"  =  "1"  /* Bypass Splenic Vein to Left Renal Vein with Synthetic Substitute, Open Approach */
"06110JY"  =  "1"  /* Bypass Splenic Vein to Lower Vein with Synthetic Substitute, Open Approach */
"06110K9"  =  "1"  /* Bypass Splenic Vein to Right Renal Vein with Nonautologous Tissue Substitute, Open Approach */
"06110KB"  =  "1"  /* Bypass Splenic Vein to Left Renal Vein with Nonautologous Tissue Substitute, Open Approach */
"06110KY"  =  "1"  /* Bypass Splenic Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach */
"06110Z9"  =  "1"  /* Bypass Splenic Vein to Right Renal Vein, Open Approach */
"06110ZB"  =  "1"  /* Bypass Splenic Vein to Left Renal Vein, Open Approach */
"06110ZY"  =  "1"  /* Bypass Splenic Vein to Lower Vein, Open Approach */
"0611479"  =  "1"  /* Bypass Splenic Vein to Right Renal Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"061147B"  =  "1"  /* Bypass Splenic Vein to Left Renal Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"061147Y"  =  "1"  /* Bypass Splenic Vein to Lower Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0611499"  =  "1"  /* Bypass Splenic Vein to Right Renal Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"061149B"  =  "1"  /* Bypass Splenic Vein to Left Renal Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"061149Y"  =  "1"  /* Bypass Splenic Vein to Lower Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"06114A9"  =  "1"  /* Bypass Splenic Vein to Right Renal Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"06114AB"  =  "1"  /* Bypass Splenic Vein to Left Renal Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"06114AY"  =  "1"  /* Bypass Splenic Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"06114J9"  =  "1"  /* Bypass Splenic Vein to Right Renal Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06114JB"  =  "1"  /* Bypass Splenic Vein to Left Renal Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06114JY"  =  "1"  /* Bypass Splenic Vein to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06114K9"  =  "1"  /* Bypass Splenic Vein to Right Renal Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06114KB"  =  "1"  /* Bypass Splenic Vein to Left Renal Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06114KY"  =  "1"  /* Bypass Splenic Vein to Lower Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06114Z9"  =  "1"  /* Bypass Splenic Vein to Right Renal Vein, Percutaneous Endoscopic Approach */
"06114ZB"  =  "1"  /* Bypass Splenic Vein to Left Renal Vein, Percutaneous Endoscopic Approach */
"06114ZY"  =  "1"  /* Bypass Splenic Vein to Lower Vein, Percutaneous Endoscopic Approach */
"061207Y"  =  "1"  /* Bypass Gastric Vein to Lower Vein with Autologous Tissue Substitute, Open Approach */
"061209Y"  =  "1"  /* Bypass Gastric Vein to Lower Vein with Autologous Venous Tissue, Open Approach */
"06120AY"  =  "1"  /* Bypass Gastric Vein to Lower Vein with Autologous Arterial Tissue, Open Approach */
"06120JY"  =  "1"  /* Bypass Gastric Vein to Lower Vein with Synthetic Substitute, Open Approach */
"06120KY"  =  "1"  /* Bypass Gastric Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach */
"06120ZY"  =  "1"  /* Bypass Gastric Vein to Lower Vein, Open Approach */
"061247Y"  =  "1"  /* Bypass Gastric Vein to Lower Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"061249Y"  =  "1"  /* Bypass Gastric Vein to Lower Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"06124AY"  =  "1"  /* Bypass Gastric Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"06124JY"  =  "1"  /* Bypass Gastric Vein to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06124KY"  =  "1"  /* Bypass Gastric Vein to Lower Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06124ZY"  =  "1"  /* Bypass Gastric Vein to Lower Vein, Percutaneous Endoscopic Approach */
"061407Y"  =  "1"  /* Bypass Hepatic Vein to Lower Vein with Autologous Tissue Substitute, Open Approach */
"061409Y"  =  "1"  /* Bypass Hepatic Vein to Lower Vein with Autologous Venous Tissue, Open Approach */
"06140AY"  =  "1"  /* Bypass Hepatic Vein to Lower Vein with Autologous Arterial Tissue, Open Approach */
"06140JY"  =  "1"  /* Bypass Hepatic Vein to Lower Vein with Synthetic Substitute, Open Approach */
"06140KY"  =  "1"  /* Bypass Hepatic Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach */
"06140ZY"  =  "1"  /* Bypass Hepatic Vein to Lower Vein, Open Approach */
"061447Y"  =  "1"  /* Bypass Hepatic Vein to Lower Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"061449Y"  =  "1"  /* Bypass Hepatic Vein to Lower Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"06144AY"  =  "1"  /* Bypass Hepatic Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"06144JY"  =  "1"  /* Bypass Hepatic Vein to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06144KY"  =  "1"  /* Bypass Hepatic Vein to Lower Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06144ZY"  =  "1"  /* Bypass Hepatic Vein to Lower Vein, Percutaneous Endoscopic Approach */
"061507Y"  =  "1"  /* Bypass Superior Mesenteric Vein to Lower Vein with Autologous Tissue Substitute, Open Approach */
"061509Y"  =  "1"  /* Bypass Superior Mesenteric Vein to Lower Vein with Autologous Venous Tissue, Open Approach */
"06150AY"  =  "1"  /* Bypass Superior Mesenteric Vein to Lower Vein with Autologous Arterial Tissue, Open Approach */
"06150JY"  =  "1"  /* Bypass Superior Mesenteric Vein to Lower Vein with Synthetic Substitute, Open Approach */
"06150KY"  =  "1"  /* Bypass Superior Mesenteric Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach */
"06150ZY"  =  "1"  /* Bypass Superior Mesenteric Vein to Lower Vein, Open Approach */
"061547Y"  =  "1"  /* Bypass Superior Mesenteric Vein to Lower Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"061549Y"  =  "1"  /* Bypass Superior Mesenteric Vein to Lower Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"06154AY"  =  "1"  /* Bypass Superior Mesenteric Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"06154JY"  =  "1"  /* Bypass Superior Mesenteric Vein to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06154KY"  =  "1"  /* Bypass Superior Mesenteric Vein to Lower Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06154ZY"  =  "1"  /* Bypass Superior Mesenteric Vein to Lower Vein, Percutaneous Endoscopic Approach */
"061607Y"  =  "1"  /* Bypass Inferior Mesenteric Vein to Lower Vein with Autologous Tissue Substitute, Open Approach */
"061609Y"  =  "1"  /* Bypass Inferior Mesenteric Vein to Lower Vein with Autologous Venous Tissue, Open Approach */
"06160AY"  =  "1"  /* Bypass Inferior Mesenteric Vein to Lower Vein with Autologous Arterial Tissue, Open Approach */
"06160JY"  =  "1"  /* Bypass Inferior Mesenteric Vein to Lower Vein with Synthetic Substitute, Open Approach */
"06160KY"  =  "1"  /* Bypass Inferior Mesenteric Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach */
"06160ZY"  =  "1"  /* Bypass Inferior Mesenteric Vein to Lower Vein, Open Approach */
"061647Y"  =  "1"  /* Bypass Inferior Mesenteric Vein to Lower Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"061649Y"  =  "1"  /* Bypass Inferior Mesenteric Vein to Lower Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"06164AY"  =  "1"  /* Bypass Inferior Mesenteric Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"06164JY"  =  "1"  /* Bypass Inferior Mesenteric Vein to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06164KY"  =  "1"  /* Bypass Inferior Mesenteric Vein to Lower Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06164ZY"  =  "1"  /* Bypass Inferior Mesenteric Vein to Lower Vein, Percutaneous Endoscopic Approach */
"061707Y"  =  "1"  /* Bypass Colic Vein to Lower Vein with Autologous Tissue Substitute, Open Approach */
"061709Y"  =  "1"  /* Bypass Colic Vein to Lower Vein with Autologous Venous Tissue, Open Approach */
"06170AY"  =  "1"  /* Bypass Colic Vein to Lower Vein with Autologous Arterial Tissue, Open Approach */
"06170JY"  =  "1"  /* Bypass Colic Vein to Lower Vein with Synthetic Substitute, Open Approach */
"06170KY"  =  "1"  /* Bypass Colic Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach */
"06170ZY"  =  "1"  /* Bypass Colic Vein to Lower Vein, Open Approach */
"061747Y"  =  "1"  /* Bypass Colic Vein to Lower Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"061749Y"  =  "1"  /* Bypass Colic Vein to Lower Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"06174AY"  =  "1"  /* Bypass Colic Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"06174JY"  =  "1"  /* Bypass Colic Vein to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06174KY"  =  "1"  /* Bypass Colic Vein to Lower Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06174ZY"  =  "1"  /* Bypass Colic Vein to Lower Vein, Percutaneous Endoscopic Approach */
"0618079"  =  "1"  /* Bypass Portal Vein to Right Renal Vein with Autologous Tissue Substitute, Open Approach */
"061807B"  =  "1"  /* Bypass Portal Vein to Left Renal Vein with Autologous Tissue Substitute, Open Approach */
"061807Y"  =  "1"  /* Bypass Portal Vein to Lower Vein with Autologous Tissue Substitute, Open Approach */
"0618099"  =  "1"  /* Bypass Portal Vein to Right Renal Vein with Autologous Venous Tissue, Open Approach */
"061809B"  =  "1"  /* Bypass Portal Vein to Left Renal Vein with Autologous Venous Tissue, Open Approach */
"061809Y"  =  "1"  /* Bypass Portal Vein to Lower Vein with Autologous Venous Tissue, Open Approach */
"06180A9"  =  "1"  /* Bypass Portal Vein to Right Renal Vein with Autologous Arterial Tissue, Open Approach */
"06180AB"  =  "1"  /* Bypass Portal Vein to Left Renal Vein with Autologous Arterial Tissue, Open Approach */
"06180AY"  =  "1"  /* Bypass Portal Vein to Lower Vein with Autologous Arterial Tissue, Open Approach */
"06180J9"  =  "1"  /* Bypass Portal Vein to Right Renal Vein with Synthetic Substitute, Open Approach */
"06180JB"  =  "1"  /* Bypass Portal Vein to Left Renal Vein with Synthetic Substitute, Open Approach */
"06180JY"  =  "1"  /* Bypass Portal Vein to Lower Vein with Synthetic Substitute, Open Approach */
"06180K9"  =  "1"  /* Bypass Portal Vein to Right Renal Vein with Nonautologous Tissue Substitute, Open Approach */
"06180KB"  =  "1"  /* Bypass Portal Vein to Left Renal Vein with Nonautologous Tissue Substitute, Open Approach */
"06180KY"  =  "1"  /* Bypass Portal Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach */
"06180Z9"  =  "1"  /* Bypass Portal Vein to Right Renal Vein, Open Approach */
"06180ZB"  =  "1"  /* Bypass Portal Vein to Left Renal Vein, Open Approach */
"06180ZY"  =  "1"  /* Bypass Portal Vein to Lower Vein, Open Approach */
"06183DY"  =  "1"  /* Bypass Portal Vein to Lower Vein with Intraluminal Device, Percutaneous Approach */
"0618479"  =  "1"  /* Bypass Portal Vein to Right Renal Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"061847B"  =  "1"  /* Bypass Portal Vein to Left Renal Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"061847Y"  =  "1"  /* Bypass Portal Vein to Lower Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0618499"  =  "1"  /* Bypass Portal Vein to Right Renal Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"061849B"  =  "1"  /* Bypass Portal Vein to Left Renal Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"061849Y"  =  "1"  /* Bypass Portal Vein to Lower Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"06184A9"  =  "1"  /* Bypass Portal Vein to Right Renal Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"06184AB"  =  "1"  /* Bypass Portal Vein to Left Renal Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"06184AY"  =  "1"  /* Bypass Portal Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"06184DY"  =  "1"  /* Bypass Portal Vein to Lower Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06184J9"  =  "1"  /* Bypass Portal Vein to Right Renal Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06184JB"  =  "1"  /* Bypass Portal Vein to Left Renal Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06184JY"  =  "1"  /* Bypass Portal Vein to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06184K9"  =  "1"  /* Bypass Portal Vein to Right Renal Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06184KB"  =  "1"  /* Bypass Portal Vein to Left Renal Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06184KY"  =  "1"  /* Bypass Portal Vein to Lower Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06184Z9"  =  "1"  /* Bypass Portal Vein to Right Renal Vein, Percutaneous Endoscopic Approach */
"06184ZB"  =  "1"  /* Bypass Portal Vein to Left Renal Vein, Percutaneous Endoscopic Approach */
"06184ZY"  =  "1"  /* Bypass Portal Vein to Lower Vein, Percutaneous Endoscopic Approach */
"061907Y"  =  "1"  /* Bypass Right Renal Vein to Lower Vein with Autologous Tissue Substitute, Open Approach */
"061909Y"  =  "1"  /* Bypass Right Renal Vein to Lower Vein with Autologous Venous Tissue, Open Approach */
"06190AY"  =  "1"  /* Bypass Right Renal Vein to Lower Vein with Autologous Arterial Tissue, Open Approach */
"06190JY"  =  "1"  /* Bypass Right Renal Vein to Lower Vein with Synthetic Substitute, Open Approach */
"06190KY"  =  "1"  /* Bypass Right Renal Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach */
"06190ZY"  =  "1"  /* Bypass Right Renal Vein to Lower Vein, Open Approach */
"061947Y"  =  "1"  /* Bypass Right Renal Vein to Lower Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"061949Y"  =  "1"  /* Bypass Right Renal Vein to Lower Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"06194AY"  =  "1"  /* Bypass Right Renal Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"06194JY"  =  "1"  /* Bypass Right Renal Vein to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06194KY"  =  "1"  /* Bypass Right Renal Vein to Lower Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06194ZY"  =  "1"  /* Bypass Right Renal Vein to Lower Vein, Percutaneous Endoscopic Approach */
"061B07Y"  =  "1"  /* Bypass Left Renal Vein to Lower Vein with Autologous Tissue Substitute, Open Approach */
"061B09Y"  =  "1"  /* Bypass Left Renal Vein to Lower Vein with Autologous Venous Tissue, Open Approach */
"061B0AY"  =  "1"  /* Bypass Left Renal Vein to Lower Vein with Autologous Arterial Tissue, Open Approach */
"061B0JY"  =  "1"  /* Bypass Left Renal Vein to Lower Vein with Synthetic Substitute, Open Approach */
"061B0KY"  =  "1"  /* Bypass Left Renal Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach */
"061B0ZY"  =  "1"  /* Bypass Left Renal Vein to Lower Vein, Open Approach */
"061B47Y"  =  "1"  /* Bypass Left Renal Vein to Lower Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"061B49Y"  =  "1"  /* Bypass Left Renal Vein to Lower Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"061B4AY"  =  "1"  /* Bypass Left Renal Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"061B4JY"  =  "1"  /* Bypass Left Renal Vein to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"061B4KY"  =  "1"  /* Bypass Left Renal Vein to Lower Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"061B4ZY"  =  "1"  /* Bypass Left Renal Vein to Lower Vein, Percutaneous Endoscopic Approach */
"061J07Y"  =  "1"  /* Bypass Left Hypogastric Vein to Lower Vein with Autologous Tissue Substitute, Open Approach */
"061J09Y"  =  "1"  /* Bypass Left Hypogastric Vein to Lower Vein with Autologous Venous Tissue, Open Approach */
"061J0AY"  =  "1"  /* Bypass Left Hypogastric Vein to Lower Vein with Autologous Arterial Tissue, Open Approach */
"061J0JY"  =  "1"  /* Bypass Left Hypogastric Vein to Lower Vein with Synthetic Substitute, Open Approach */
"061J0KY"  =  "1"  /* Bypass Left Hypogastric Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach */
"061J0ZY"  =  "1"  /* Bypass Left Hypogastric Vein to Lower Vein, Open Approach */
"061J47Y"  =  "1"  /* Bypass Left Hypogastric Vein to Lower Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"061J49Y"  =  "1"  /* Bypass Left Hypogastric Vein to Lower Vein with Autologous Venous Tissue, Percutaneous Endoscopic Approach */
"061J4AY"  =  "1"  /* Bypass Left Hypogastric Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach */
"061J4JY"  =  "1"  /* Bypass Left Hypogastric Vein to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"061J4KY"  =  "1"  /* Bypass Left Hypogastric Vein to Lower Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"061J4ZY"  =  "1"  /* Bypass Left Hypogastric Vein to Lower Vein, Percutaneous Endoscopic Approach */
"06500ZZ"  =  "1"  /* Destruction of Inferior Vena Cava, Open Approach */
"06503ZZ"  =  "1"  /* Destruction of Inferior Vena Cava, Percutaneous Approach */
"06504ZZ"  =  "1"  /* Destruction of Inferior Vena Cava, Percutaneous Endoscopic Approach */
"06510ZZ"  =  "1"  /* Destruction of Splenic Vein, Open Approach */
"06513ZZ"  =  "1"  /* Destruction of Splenic Vein, Percutaneous Approach */
"06514ZZ"  =  "1"  /* Destruction of Splenic Vein, Percutaneous Endoscopic Approach */
"06520ZZ"  =  "1"  /* Destruction of Gastric Vein, Open Approach */
"06523ZZ"  =  "1"  /* Destruction of Gastric Vein, Percutaneous Approach */
"06524ZZ"  =  "1"  /* Destruction of Gastric Vein, Percutaneous Endoscopic Approach */
"06540ZZ"  =  "1"  /* Destruction of Hepatic Vein, Open Approach */
"06543ZZ"  =  "1"  /* Destruction of Hepatic Vein, Percutaneous Approach */
"06544ZZ"  =  "1"  /* Destruction of Hepatic Vein, Percutaneous Endoscopic Approach */
"06550ZZ"  =  "1"  /* Destruction of Superior Mesenteric Vein, Open Approach */
"06553ZZ"  =  "1"  /* Destruction of Superior Mesenteric Vein, Percutaneous Approach */
"06554ZZ"  =  "1"  /* Destruction of Superior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06560ZZ"  =  "1"  /* Destruction of Inferior Mesenteric Vein, Open Approach */
"06563ZZ"  =  "1"  /* Destruction of Inferior Mesenteric Vein, Percutaneous Approach */
"06564ZZ"  =  "1"  /* Destruction of Inferior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06570ZZ"  =  "1"  /* Destruction of Colic Vein, Open Approach */
"06573ZZ"  =  "1"  /* Destruction of Colic Vein, Percutaneous Approach */
"06574ZZ"  =  "1"  /* Destruction of Colic Vein, Percutaneous Endoscopic Approach */
"06580ZZ"  =  "1"  /* Destruction of Portal Vein, Open Approach */
"06583ZZ"  =  "1"  /* Destruction of Portal Vein, Percutaneous Approach */
"06584ZZ"  =  "1"  /* Destruction of Portal Vein, Percutaneous Endoscopic Approach */
"06590ZZ"  =  "1"  /* Destruction of Right Renal Vein, Open Approach */
"06593ZZ"  =  "1"  /* Destruction of Right Renal Vein, Percutaneous Approach */
"06594ZZ"  =  "1"  /* Destruction of Right Renal Vein, Percutaneous Endoscopic Approach */
"065B0ZZ"  =  "1"  /* Destruction of Left Renal Vein, Open Approach */
"065B3ZZ"  =  "1"  /* Destruction of Left Renal Vein, Percutaneous Approach */
"065B4ZZ"  =  "1"  /* Destruction of Left Renal Vein, Percutaneous Endoscopic Approach */
"065C0ZZ"  =  "1"  /* Destruction of Right Common Iliac Vein, Open Approach */
"065C3ZZ"  =  "1"  /* Destruction of Right Common Iliac Vein, Percutaneous Approach */
"065C4ZZ"  =  "1"  /* Destruction of Right Common Iliac Vein, Percutaneous Endoscopic Approach */
"065D0ZZ"  =  "1"  /* Destruction of Left Common Iliac Vein, Open Approach */
"065D3ZZ"  =  "1"  /* Destruction of Left Common Iliac Vein, Percutaneous Approach */
"065D4ZZ"  =  "1"  /* Destruction of Left Common Iliac Vein, Percutaneous Endoscopic Approach */
"065F0ZZ"  =  "1"  /* Destruction of Right External Iliac Vein, Open Approach */
"065F3ZZ"  =  "1"  /* Destruction of Right External Iliac Vein, Percutaneous Approach */
"065F4ZZ"  =  "1"  /* Destruction of Right External Iliac Vein, Percutaneous Endoscopic Approach */
"065G0ZZ"  =  "1"  /* Destruction of Left External Iliac Vein, Open Approach */
"065G3ZZ"  =  "1"  /* Destruction of Left External Iliac Vein, Percutaneous Approach */
"065G4ZZ"  =  "1"  /* Destruction of Left External Iliac Vein, Percutaneous Endoscopic Approach */
"065H0ZZ"  =  "1"  /* Destruction of Right Hypogastric Vein, Open Approach */
"065H3ZZ"  =  "1"  /* Destruction of Right Hypogastric Vein, Percutaneous Approach */
"065H4ZZ"  =  "1"  /* Destruction of Right Hypogastric Vein, Percutaneous Endoscopic Approach */
"065J0ZZ"  =  "1"  /* Destruction of Left Hypogastric Vein, Open Approach */
"065J3ZZ"  =  "1"  /* Destruction of Left Hypogastric Vein, Percutaneous Approach */
"065J4ZZ"  =  "1"  /* Destruction of Left Hypogastric Vein, Percutaneous Endoscopic Approach */
"06B00ZZ"  =  "1"  /* Excision of Inferior Vena Cava, Open Approach */
"06B03ZZ"  =  "1"  /* Excision of Inferior Vena Cava, Percutaneous Approach */
"06B04ZZ"  =  "1"  /* Excision of Inferior Vena Cava, Percutaneous Endoscopic Approach */
"06B10ZZ"  =  "1"  /* Excision of Splenic Vein, Open Approach */
"06B13ZZ"  =  "1"  /* Excision of Splenic Vein, Percutaneous Approach */
"06B14ZZ"  =  "1"  /* Excision of Splenic Vein, Percutaneous Endoscopic Approach */
"06B20ZZ"  =  "1"  /* Excision of Gastric Vein, Open Approach */
"06B23ZZ"  =  "1"  /* Excision of Gastric Vein, Percutaneous Approach */
"06B24ZZ"  =  "1"  /* Excision of Gastric Vein, Percutaneous Endoscopic Approach */
"06B40ZZ"  =  "1"  /* Excision of Hepatic Vein, Open Approach */
"06B43ZZ"  =  "1"  /* Excision of Hepatic Vein, Percutaneous Approach */
"06B44ZZ"  =  "1"  /* Excision of Hepatic Vein, Percutaneous Endoscopic Approach */
"06B50ZZ"  =  "1"  /* Excision of Superior Mesenteric Vein, Open Approach */
"06B53ZZ"  =  "1"  /* Excision of Superior Mesenteric Vein, Percutaneous Approach */
"06B54ZZ"  =  "1"  /* Excision of Superior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06B60ZZ"  =  "1"  /* Excision of Inferior Mesenteric Vein, Open Approach */
"06B63ZZ"  =  "1"  /* Excision of Inferior Mesenteric Vein, Percutaneous Approach */
"06B64ZZ"  =  "1"  /* Excision of Inferior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06B70ZZ"  =  "1"  /* Excision of Colic Vein, Open Approach */
"06B73ZZ"  =  "1"  /* Excision of Colic Vein, Percutaneous Approach */
"06B74ZZ"  =  "1"  /* Excision of Colic Vein, Percutaneous Endoscopic Approach */
"06B80ZZ"  =  "1"  /* Excision of Portal Vein, Open Approach */
"06B83ZZ"  =  "1"  /* Excision of Portal Vein, Percutaneous Approach */
"06B84ZZ"  =  "1"  /* Excision of Portal Vein, Percutaneous Endoscopic Approach */
"06B90ZZ"  =  "1"  /* Excision of Right Renal Vein, Open Approach */
"06B93ZZ"  =  "1"  /* Excision of Right Renal Vein, Percutaneous Approach */
"06B94ZZ"  =  "1"  /* Excision of Right Renal Vein, Percutaneous Endoscopic Approach */
"06BB0ZZ"  =  "1"  /* Excision of Left Renal Vein, Open Approach */
"06BB3ZZ"  =  "1"  /* Excision of Left Renal Vein, Percutaneous Approach */
"06BB4ZZ"  =  "1"  /* Excision of Left Renal Vein, Percutaneous Endoscopic Approach */
"06BC0ZZ"  =  "1"  /* Excision of Right Common Iliac Vein, Open Approach */
"06BC3ZZ"  =  "1"  /* Excision of Right Common Iliac Vein, Percutaneous Approach */
"06BC4ZZ"  =  "1"  /* Excision of Right Common Iliac Vein, Percutaneous Endoscopic Approach */
"06BD0ZZ"  =  "1"  /* Excision of Left Common Iliac Vein, Open Approach */
"06BD3ZZ"  =  "1"  /* Excision of Left Common Iliac Vein, Percutaneous Approach */
"06BD4ZZ"  =  "1"  /* Excision of Left Common Iliac Vein, Percutaneous Endoscopic Approach */
"06BF0ZZ"  =  "1"  /* Excision of Right External Iliac Vein, Open Approach */
"06BF3ZZ"  =  "1"  /* Excision of Right External Iliac Vein, Percutaneous Approach */
"06BF4ZZ"  =  "1"  /* Excision of Right External Iliac Vein, Percutaneous Endoscopic Approach */
"06BG0ZZ"  =  "1"  /* Excision of Left External Iliac Vein, Open Approach */
"06BG3ZZ"  =  "1"  /* Excision of Left External Iliac Vein, Percutaneous Approach */
"06BG4ZZ"  =  "1"  /* Excision of Left External Iliac Vein, Percutaneous Endoscopic Approach */
"06BH0ZZ"  =  "1"  /* Excision of Right Hypogastric Vein, Open Approach */
"06BH3ZZ"  =  "1"  /* Excision of Right Hypogastric Vein, Percutaneous Approach */
"06BH4ZZ"  =  "1"  /* Excision of Right Hypogastric Vein, Percutaneous Endoscopic Approach */
"06BJ0ZZ"  =  "1"  /* Excision of Left Hypogastric Vein, Open Approach */
"06BJ3ZZ"  =  "1"  /* Excision of Left Hypogastric Vein, Percutaneous Approach */
"06BJ4ZZ"  =  "1"  /* Excision of Left Hypogastric Vein, Percutaneous Endoscopic Approach */
"06C00ZZ"  =  "1"  /* Extirpation of Matter from Inferior Vena Cava, Open Approach */
"06C03ZZ"  =  "1"  /* Extirpation of Matter from Inferior Vena Cava, Percutaneous Approach */
"06C04ZZ"  =  "1"  /* Extirpation of Matter from Inferior Vena Cava, Percutaneous Endoscopic Approach */
"06C10ZZ"  =  "1"  /* Extirpation of Matter from Splenic Vein, Open Approach */
"06C13ZZ"  =  "1"  /* Extirpation of Matter from Splenic Vein, Percutaneous Approach */
"06C14ZZ"  =  "1"  /* Extirpation of Matter from Splenic Vein, Percutaneous Endoscopic Approach */
"06C20ZZ"  =  "1"  /* Extirpation of Matter from Gastric Vein, Open Approach */
"06C23ZZ"  =  "1"  /* Extirpation of Matter from Gastric Vein, Percutaneous Approach */
"06C24ZZ"  =  "1"  /* Extirpation of Matter from Gastric Vein, Percutaneous Endoscopic Approach */
"06C40ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Vein, Open Approach */
"06C43ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Vein, Percutaneous Approach */
"06C44ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Vein, Percutaneous Endoscopic Approach */
"06C50ZZ"  =  "1"  /* Extirpation of Matter from Superior Mesenteric Vein, Open Approach */
"06C53ZZ"  =  "1"  /* Extirpation of Matter from Superior Mesenteric Vein, Percutaneous Approach */
"06C54ZZ"  =  "1"  /* Extirpation of Matter from Superior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06C60ZZ"  =  "1"  /* Extirpation of Matter from Inferior Mesenteric Vein, Open Approach */
"06C63ZZ"  =  "1"  /* Extirpation of Matter from Inferior Mesenteric Vein, Percutaneous Approach */
"06C64ZZ"  =  "1"  /* Extirpation of Matter from Inferior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06C70ZZ"  =  "1"  /* Extirpation of Matter from Colic Vein, Open Approach */
"06C73ZZ"  =  "1"  /* Extirpation of Matter from Colic Vein, Percutaneous Approach */
"06C74ZZ"  =  "1"  /* Extirpation of Matter from Colic Vein, Percutaneous Endoscopic Approach */
"06C80ZZ"  =  "1"  /* Extirpation of Matter from Portal Vein, Open Approach */
"06C83ZZ"  =  "1"  /* Extirpation of Matter from Portal Vein, Percutaneous Approach */
"06C84ZZ"  =  "1"  /* Extirpation of Matter from Portal Vein, Percutaneous Endoscopic Approach */
"06C90ZZ"  =  "1"  /* Extirpation of Matter from Right Renal Vein, Open Approach */
"06C93ZZ"  =  "1"  /* Extirpation of Matter from Right Renal Vein, Percutaneous Approach */
"06C94ZZ"  =  "1"  /* Extirpation of Matter from Right Renal Vein, Percutaneous Endoscopic Approach */
"06CB0ZZ"  =  "1"  /* Extirpation of Matter from Left Renal Vein, Open Approach */
"06CB3ZZ"  =  "1"  /* Extirpation of Matter from Left Renal Vein, Percutaneous Approach */
"06CB4ZZ"  =  "1"  /* Extirpation of Matter from Left Renal Vein, Percutaneous Endoscopic Approach */
"06CC0ZZ"  =  "1"  /* Extirpation of Matter from Right Common Iliac Vein, Open Approach */
"06CC3ZZ"  =  "1"  /* Extirpation of Matter from Right Common Iliac Vein, Percutaneous Approach */
"06CC4ZZ"  =  "1"  /* Extirpation of Matter from Right Common Iliac Vein, Percutaneous Endoscopic Approach */
"06CD0ZZ"  =  "1"  /* Extirpation of Matter from Left Common Iliac Vein, Open Approach */
"06CD3ZZ"  =  "1"  /* Extirpation of Matter from Left Common Iliac Vein, Percutaneous Approach */
"06CD4ZZ"  =  "1"  /* Extirpation of Matter from Left Common Iliac Vein, Percutaneous Endoscopic Approach */
"06CF0ZZ"  =  "1"  /* Extirpation of Matter from Right External Iliac Vein, Open Approach */
"06CF3ZZ"  =  "1"  /* Extirpation of Matter from Right External Iliac Vein, Percutaneous Approach */
"06CF4ZZ"  =  "1"  /* Extirpation of Matter from Right External Iliac Vein, Percutaneous Endoscopic Approach */
"06CG0ZZ"  =  "1"  /* Extirpation of Matter from Left External Iliac Vein, Open Approach */
"06CG3ZZ"  =  "1"  /* Extirpation of Matter from Left External Iliac Vein, Percutaneous Approach */
"06CG4ZZ"  =  "1"  /* Extirpation of Matter from Left External Iliac Vein, Percutaneous Endoscopic Approach */
"06CH0ZZ"  =  "1"  /* Extirpation of Matter from Right Hypogastric Vein, Open Approach */
"06CH3ZZ"  =  "1"  /* Extirpation of Matter from Right Hypogastric Vein, Percutaneous Approach */
"06CH4ZZ"  =  "1"  /* Extirpation of Matter from Right Hypogastric Vein, Percutaneous Endoscopic Approach */
"06CJ0ZZ"  =  "1"  /* Extirpation of Matter from Left Hypogastric Vein, Open Approach */
"06CJ3ZZ"  =  "1"  /* Extirpation of Matter from Left Hypogastric Vein, Percutaneous Approach */
"06CJ4ZZ"  =  "1"  /* Extirpation of Matter from Left Hypogastric Vein, Percutaneous Endoscopic Approach */
"06L10CZ"  =  "1"  /* Occlusion of Splenic Vein with Extraluminal Device, Open Approach */
"06L10DZ"  =  "1"  /* Occlusion of Splenic Vein with Intraluminal Device, Open Approach */
"06L10ZZ"  =  "1"  /* Occlusion of Splenic Vein, Open Approach */
"06L13CZ"  =  "1"  /* Occlusion of Splenic Vein with Extraluminal Device, Percutaneous Approach */
"06L13DZ"  =  "1"  /* Occlusion of Splenic Vein with Intraluminal Device, Percutaneous Approach */
"06L13ZZ"  =  "1"  /* Occlusion of Splenic Vein, Percutaneous Approach */
"06L14CZ"  =  "1"  /* Occlusion of Splenic Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L14DZ"  =  "1"  /* Occlusion of Splenic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L14ZZ"  =  "1"  /* Occlusion of Splenic Vein, Percutaneous Endoscopic Approach */
"06L20CZ"  =  "1"  /* Occlusion of Gastric Vein with Extraluminal Device, Open Approach */
"06L20DZ"  =  "1"  /* Occlusion of Gastric Vein with Intraluminal Device, Open Approach */
"06L20ZZ"  =  "1"  /* Occlusion of Gastric Vein, Open Approach */
"06L23CZ"  =  "1"  /* Occlusion of Gastric Vein with Extraluminal Device, Percutaneous Approach */
"06L23DZ"  =  "1"  /* Occlusion of Gastric Vein with Intraluminal Device, Percutaneous Approach */
"06L23ZZ"  =  "1"  /* Occlusion of Gastric Vein, Percutaneous Approach */
"06L24CZ"  =  "1"  /* Occlusion of Gastric Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L24DZ"  =  "1"  /* Occlusion of Gastric Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L24ZZ"  =  "1"  /* Occlusion of Gastric Vein, Percutaneous Endoscopic Approach */
"06L30CZ"  =  "1"  /* Occlusion of Esophageal Vein with Extraluminal Device, Open Approach */
"06L30DZ"  =  "1"  /* Occlusion of Esophageal Vein with Intraluminal Device, Open Approach */
"06L30ZZ"  =  "1"  /* Occlusion of Esophageal Vein, Open Approach */
"06L33CZ"  =  "1"  /* Occlusion of Esophageal Vein with Extraluminal Device, Percutaneous Approach */
"06L33DZ"  =  "1"  /* Occlusion of Esophageal Vein with Intraluminal Device, Percutaneous Approach */
"06L33ZZ"  =  "1"  /* Occlusion of Esophageal Vein, Percutaneous Approach */
"06L34CZ"  =  "1"  /* Occlusion of Esophageal Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L34DZ"  =  "1"  /* Occlusion of Esophageal Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L34ZZ"  =  "1"  /* Occlusion of Esophageal Vein, Percutaneous Endoscopic Approach */
"06L40CZ"  =  "1"  /* Occlusion of Hepatic Vein with Extraluminal Device, Open Approach */
"06L40DZ"  =  "1"  /* Occlusion of Hepatic Vein with Intraluminal Device, Open Approach */
"06L40ZZ"  =  "1"  /* Occlusion of Hepatic Vein, Open Approach */
"06L43CZ"  =  "1"  /* Occlusion of Hepatic Vein with Extraluminal Device, Percutaneous Approach */
"06L43DZ"  =  "1"  /* Occlusion of Hepatic Vein with Intraluminal Device, Percutaneous Approach */
"06L43ZZ"  =  "1"  /* Occlusion of Hepatic Vein, Percutaneous Approach */
"06L44CZ"  =  "1"  /* Occlusion of Hepatic Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L44DZ"  =  "1"  /* Occlusion of Hepatic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L44ZZ"  =  "1"  /* Occlusion of Hepatic Vein, Percutaneous Endoscopic Approach */
"06L50CZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein with Extraluminal Device, Open Approach */
"06L50DZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein with Intraluminal Device, Open Approach */
"06L50ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein, Open Approach */
"06L53CZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein with Extraluminal Device, Percutaneous Approach */
"06L53DZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein with Intraluminal Device, Percutaneous Approach */
"06L53ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein, Percutaneous Approach */
"06L54CZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L54DZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L54ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06L60CZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein with Extraluminal Device, Open Approach */
"06L60DZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein with Intraluminal Device, Open Approach */
"06L60ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein, Open Approach */
"06L63CZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein with Extraluminal Device, Percutaneous Approach */
"06L63DZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein with Intraluminal Device, Percutaneous Approach */
"06L63ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein, Percutaneous Approach */
"06L64CZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L64DZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L64ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein, Percutaneous Endoscopic Approach */
"06L70CZ"  =  "1"  /* Occlusion of Colic Vein with Extraluminal Device, Open Approach */
"06L70DZ"  =  "1"  /* Occlusion of Colic Vein with Intraluminal Device, Open Approach */
"06L70ZZ"  =  "1"  /* Occlusion of Colic Vein, Open Approach */
"06L73CZ"  =  "1"  /* Occlusion of Colic Vein with Extraluminal Device, Percutaneous Approach */
"06L73DZ"  =  "1"  /* Occlusion of Colic Vein with Intraluminal Device, Percutaneous Approach */
"06L73ZZ"  =  "1"  /* Occlusion of Colic Vein, Percutaneous Approach */
"06L74CZ"  =  "1"  /* Occlusion of Colic Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L74DZ"  =  "1"  /* Occlusion of Colic Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L74ZZ"  =  "1"  /* Occlusion of Colic Vein, Percutaneous Endoscopic Approach */
"06L80CZ"  =  "1"  /* Occlusion of Portal Vein with Extraluminal Device, Open Approach */
"06L80DZ"  =  "1"  /* Occlusion of Portal Vein with Intraluminal Device, Open Approach */
"06L80ZZ"  =  "1"  /* Occlusion of Portal Vein, Open Approach */
"06L83CZ"  =  "1"  /* Occlusion of Portal Vein with Extraluminal Device, Percutaneous Approach */
"06L83DZ"  =  "1"  /* Occlusion of Portal Vein with Intraluminal Device, Percutaneous Approach */
"06L83ZZ"  =  "1"  /* Occlusion of Portal Vein, Percutaneous Approach */
"06L84CZ"  =  "1"  /* Occlusion of Portal Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L84DZ"  =  "1"  /* Occlusion of Portal Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L84ZZ"  =  "1"  /* Occlusion of Portal Vein, Percutaneous Endoscopic Approach */
"06L90CZ"  =  "1"  /* Occlusion of Right Renal Vein with Extraluminal Device, Open Approach */
"06L90DZ"  =  "1"  /* Occlusion of Right Renal Vein with Intraluminal Device, Open Approach */
"06L90ZZ"  =  "1"  /* Occlusion of Right Renal Vein, Open Approach */
"06L93CZ"  =  "1"  /* Occlusion of Right Renal Vein with Extraluminal Device, Percutaneous Approach */
"06L93DZ"  =  "1"  /* Occlusion of Right Renal Vein with Intraluminal Device, Percutaneous Approach */
"06L93ZZ"  =  "1"  /* Occlusion of Right Renal Vein, Percutaneous Approach */
"06L94CZ"  =  "1"  /* Occlusion of Right Renal Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06L94DZ"  =  "1"  /* Occlusion of Right Renal Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06L94ZZ"  =  "1"  /* Occlusion of Right Renal Vein, Percutaneous Endoscopic Approach */
"06LB0CZ"  =  "1"  /* Occlusion of Left Renal Vein with Extraluminal Device, Open Approach */
"06LB0DZ"  =  "1"  /* Occlusion of Left Renal Vein with Intraluminal Device, Open Approach */
"06LB0ZZ"  =  "1"  /* Occlusion of Left Renal Vein, Open Approach */
"06LB3CZ"  =  "1"  /* Occlusion of Left Renal Vein with Extraluminal Device, Percutaneous Approach */
"06LB3DZ"  =  "1"  /* Occlusion of Left Renal Vein with Intraluminal Device, Percutaneous Approach */
"06LB3ZZ"  =  "1"  /* Occlusion of Left Renal Vein, Percutaneous Approach */
"06LB4CZ"  =  "1"  /* Occlusion of Left Renal Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LB4DZ"  =  "1"  /* Occlusion of Left Renal Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LB4ZZ"  =  "1"  /* Occlusion of Left Renal Vein, Percutaneous Endoscopic Approach */
"06LC0CZ"  =  "1"  /* Occlusion of Right Common Iliac Vein with Extraluminal Device, Open Approach */
"06LC0DZ"  =  "1"  /* Occlusion of Right Common Iliac Vein with Intraluminal Device, Open Approach */
"06LC0ZZ"  =  "1"  /* Occlusion of Right Common Iliac Vein, Open Approach */
"06LC3CZ"  =  "1"  /* Occlusion of Right Common Iliac Vein with Extraluminal Device, Percutaneous Approach */
"06LC3DZ"  =  "1"  /* Occlusion of Right Common Iliac Vein with Intraluminal Device, Percutaneous Approach */
"06LC3ZZ"  =  "1"  /* Occlusion of Right Common Iliac Vein, Percutaneous Approach */
"06LC4CZ"  =  "1"  /* Occlusion of Right Common Iliac Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LC4DZ"  =  "1"  /* Occlusion of Right Common Iliac Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LC4ZZ"  =  "1"  /* Occlusion of Right Common Iliac Vein, Percutaneous Endoscopic Approach */
"06LD0CZ"  =  "1"  /* Occlusion of Left Common Iliac Vein with Extraluminal Device, Open Approach */
"06LD0DZ"  =  "1"  /* Occlusion of Left Common Iliac Vein with Intraluminal Device, Open Approach */
"06LD0ZZ"  =  "1"  /* Occlusion of Left Common Iliac Vein, Open Approach */
"06LD3CZ"  =  "1"  /* Occlusion of Left Common Iliac Vein with Extraluminal Device, Percutaneous Approach */
"06LD3DZ"  =  "1"  /* Occlusion of Left Common Iliac Vein with Intraluminal Device, Percutaneous Approach */
"06LD3ZZ"  =  "1"  /* Occlusion of Left Common Iliac Vein, Percutaneous Approach */
"06LD4CZ"  =  "1"  /* Occlusion of Left Common Iliac Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LD4DZ"  =  "1"  /* Occlusion of Left Common Iliac Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LD4ZZ"  =  "1"  /* Occlusion of Left Common Iliac Vein, Percutaneous Endoscopic Approach */
"06LF0CZ"  =  "1"  /* Occlusion of Right External Iliac Vein with Extraluminal Device, Open Approach */
"06LF0DZ"  =  "1"  /* Occlusion of Right External Iliac Vein with Intraluminal Device, Open Approach */
"06LF0ZZ"  =  "1"  /* Occlusion of Right External Iliac Vein, Open Approach */
"06LF3CZ"  =  "1"  /* Occlusion of Right External Iliac Vein with Extraluminal Device, Percutaneous Approach */
"06LF3DZ"  =  "1"  /* Occlusion of Right External Iliac Vein with Intraluminal Device, Percutaneous Approach */
"06LF3ZZ"  =  "1"  /* Occlusion of Right External Iliac Vein, Percutaneous Approach */
"06LF4CZ"  =  "1"  /* Occlusion of Right External Iliac Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LF4DZ"  =  "1"  /* Occlusion of Right External Iliac Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LF4ZZ"  =  "1"  /* Occlusion of Right External Iliac Vein, Percutaneous Endoscopic Approach */
"06LG0CZ"  =  "1"  /* Occlusion of Left External Iliac Vein with Extraluminal Device, Open Approach */
"06LG0DZ"  =  "1"  /* Occlusion of Left External Iliac Vein with Intraluminal Device, Open Approach */
"06LG0ZZ"  =  "1"  /* Occlusion of Left External Iliac Vein, Open Approach */
"06LG3CZ"  =  "1"  /* Occlusion of Left External Iliac Vein with Extraluminal Device, Percutaneous Approach */
"06LG3DZ"  =  "1"  /* Occlusion of Left External Iliac Vein with Intraluminal Device, Percutaneous Approach */
"06LG3ZZ"  =  "1"  /* Occlusion of Left External Iliac Vein, Percutaneous Approach */
"06LG4CZ"  =  "1"  /* Occlusion of Left External Iliac Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LG4DZ"  =  "1"  /* Occlusion of Left External Iliac Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LG4ZZ"  =  "1"  /* Occlusion of Left External Iliac Vein, Percutaneous Endoscopic Approach */
"06LH0CZ"  =  "1"  /* Occlusion of Right Hypogastric Vein with Extraluminal Device, Open Approach */
"06LH0DZ"  =  "1"  /* Occlusion of Right Hypogastric Vein with Intraluminal Device, Open Approach */
"06LH0ZZ"  =  "1"  /* Occlusion of Right Hypogastric Vein, Open Approach */
"06LH3CZ"  =  "1"  /* Occlusion of Right Hypogastric Vein with Extraluminal Device, Percutaneous Approach */
"06LH3DZ"  =  "1"  /* Occlusion of Right Hypogastric Vein with Intraluminal Device, Percutaneous Approach */
"06LH3ZZ"  =  "1"  /* Occlusion of Right Hypogastric Vein, Percutaneous Approach */
"06LH4CZ"  =  "1"  /* Occlusion of Right Hypogastric Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LH4DZ"  =  "1"  /* Occlusion of Right Hypogastric Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LH4ZZ"  =  "1"  /* Occlusion of Right Hypogastric Vein, Percutaneous Endoscopic Approach */
"06LJ0CZ"  =  "1"  /* Occlusion of Left Hypogastric Vein with Extraluminal Device, Open Approach */
"06LJ0DZ"  =  "1"  /* Occlusion of Left Hypogastric Vein with Intraluminal Device, Open Approach */
"06LJ0ZZ"  =  "1"  /* Occlusion of Left Hypogastric Vein, Open Approach */
"06LJ3CZ"  =  "1"  /* Occlusion of Left Hypogastric Vein with Extraluminal Device, Percutaneous Approach */
"06LJ3DZ"  =  "1"  /* Occlusion of Left Hypogastric Vein with Intraluminal Device, Percutaneous Approach */
"06LJ3ZZ"  =  "1"  /* Occlusion of Left Hypogastric Vein, Percutaneous Approach */
"06LJ4CZ"  =  "1"  /* Occlusion of Left Hypogastric Vein with Extraluminal Device, Percutaneous Endoscopic Approach */
"06LJ4DZ"  =  "1"  /* Occlusion of Left Hypogastric Vein with Intraluminal Device, Percutaneous Endoscopic Approach */
"06LJ4ZZ"  =  "1"  /* Occlusion of Left Hypogastric Vein, Percutaneous Endoscopic Approach */
"06R007Z"  =  "1"  /* Replacement of Inferior Vena Cava with Autologous Tissue Substitute, Open Approach */
"06R00JZ"  =  "1"  /* Replacement of Inferior Vena Cava with Synthetic Substitute, Open Approach */
"06R00KZ"  =  "1"  /* Replacement of Inferior Vena Cava with Nonautologous Tissue Substitute, Open Approach */
"06R047Z"  =  "1"  /* Replacement of Inferior Vena Cava with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R04JZ"  =  "1"  /* Replacement of Inferior Vena Cava with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06R04KZ"  =  "1"  /* Replacement of Inferior Vena Cava with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R107Z"  =  "1"  /* Replacement of Splenic Vein with Autologous Tissue Substitute, Open Approach */
"06R10JZ"  =  "1"  /* Replacement of Splenic Vein with Synthetic Substitute, Open Approach */
"06R10KZ"  =  "1"  /* Replacement of Splenic Vein with Nonautologous Tissue Substitute, Open Approach */
"06R147Z"  =  "1"  /* Replacement of Splenic Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R14JZ"  =  "1"  /* Replacement of Splenic Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06R14KZ"  =  "1"  /* Replacement of Splenic Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R207Z"  =  "1"  /* Replacement of Gastric Vein with Autologous Tissue Substitute, Open Approach */
"06R20JZ"  =  "1"  /* Replacement of Gastric Vein with Synthetic Substitute, Open Approach */
"06R20KZ"  =  "1"  /* Replacement of Gastric Vein with Nonautologous Tissue Substitute, Open Approach */
"06R247Z"  =  "1"  /* Replacement of Gastric Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R24JZ"  =  "1"  /* Replacement of Gastric Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06R24KZ"  =  "1"  /* Replacement of Gastric Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R407Z"  =  "1"  /* Replacement of Hepatic Vein with Autologous Tissue Substitute, Open Approach */
"06R40JZ"  =  "1"  /* Replacement of Hepatic Vein with Synthetic Substitute, Open Approach */
"06R40KZ"  =  "1"  /* Replacement of Hepatic Vein with Nonautologous Tissue Substitute, Open Approach */
"06R447Z"  =  "1"  /* Replacement of Hepatic Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R44JZ"  =  "1"  /* Replacement of Hepatic Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06R44KZ"  =  "1"  /* Replacement of Hepatic Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R507Z"  =  "1"  /* Replacement of Superior Mesenteric Vein with Autologous Tissue Substitute, Open Approach */
"06R50JZ"  =  "1"  /* Replacement of Superior Mesenteric Vein with Synthetic Substitute, Open Approach */
"06R50KZ"  =  "1"  /* Replacement of Superior Mesenteric Vein with Nonautologous Tissue Substitute, Open Approach */
"06R547Z"  =  "1"  /* Replacement of Superior Mesenteric Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R54JZ"  =  "1"  /* Replacement of Superior Mesenteric Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06R54KZ"  =  "1"  /* Replacement of Superior Mesenteric Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R607Z"  =  "1"  /* Replacement of Inferior Mesenteric Vein with Autologous Tissue Substitute, Open Approach */
"06R60JZ"  =  "1"  /* Replacement of Inferior Mesenteric Vein with Synthetic Substitute, Open Approach */
"06R60KZ"  =  "1"  /* Replacement of Inferior Mesenteric Vein with Nonautologous Tissue Substitute, Open Approach */
"06R647Z"  =  "1"  /* Replacement of Inferior Mesenteric Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R64JZ"  =  "1"  /* Replacement of Inferior Mesenteric Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06R64KZ"  =  "1"  /* Replacement of Inferior Mesenteric Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R707Z"  =  "1"  /* Replacement of Colic Vein with Autologous Tissue Substitute, Open Approach */
"06R70JZ"  =  "1"  /* Replacement of Colic Vein with Synthetic Substitute, Open Approach */
"06R70KZ"  =  "1"  /* Replacement of Colic Vein with Nonautologous Tissue Substitute, Open Approach */
"06R747Z"  =  "1"  /* Replacement of Colic Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R74JZ"  =  "1"  /* Replacement of Colic Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06R74KZ"  =  "1"  /* Replacement of Colic Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R807Z"  =  "1"  /* Replacement of Portal Vein with Autologous Tissue Substitute, Open Approach */
"06R80JZ"  =  "1"  /* Replacement of Portal Vein with Synthetic Substitute, Open Approach */
"06R80KZ"  =  "1"  /* Replacement of Portal Vein with Nonautologous Tissue Substitute, Open Approach */
"06R847Z"  =  "1"  /* Replacement of Portal Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R84JZ"  =  "1"  /* Replacement of Portal Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06R84KZ"  =  "1"  /* Replacement of Portal Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R907Z"  =  "1"  /* Replacement of Right Renal Vein with Autologous Tissue Substitute, Open Approach */
"06R90JZ"  =  "1"  /* Replacement of Right Renal Vein with Synthetic Substitute, Open Approach */
"06R90KZ"  =  "1"  /* Replacement of Right Renal Vein with Nonautologous Tissue Substitute, Open Approach */
"06R947Z"  =  "1"  /* Replacement of Right Renal Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06R94JZ"  =  "1"  /* Replacement of Right Renal Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06R94KZ"  =  "1"  /* Replacement of Right Renal Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06RB07Z"  =  "1"  /* Replacement of Left Renal Vein with Autologous Tissue Substitute, Open Approach */
"06RB0JZ"  =  "1"  /* Replacement of Left Renal Vein with Synthetic Substitute, Open Approach */
"06RB0KZ"  =  "1"  /* Replacement of Left Renal Vein with Nonautologous Tissue Substitute, Open Approach */
"06RB47Z"  =  "1"  /* Replacement of Left Renal Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06RB4JZ"  =  "1"  /* Replacement of Left Renal Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06RB4KZ"  =  "1"  /* Replacement of Left Renal Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06RC07Z"  =  "1"  /* Replacement of Right Common Iliac Vein with Autologous Tissue Substitute, Open Approach */
"06RC0JZ"  =  "1"  /* Replacement of Right Common Iliac Vein with Synthetic Substitute, Open Approach */
"06RC0KZ"  =  "1"  /* Replacement of Right Common Iliac Vein with Nonautologous Tissue Substitute, Open Approach */
"06RC47Z"  =  "1"  /* Replacement of Right Common Iliac Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06RC4JZ"  =  "1"  /* Replacement of Right Common Iliac Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06RC4KZ"  =  "1"  /* Replacement of Right Common Iliac Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06RD07Z"  =  "1"  /* Replacement of Left Common Iliac Vein with Autologous Tissue Substitute, Open Approach */
"06RD0JZ"  =  "1"  /* Replacement of Left Common Iliac Vein with Synthetic Substitute, Open Approach */
"06RD0KZ"  =  "1"  /* Replacement of Left Common Iliac Vein with Nonautologous Tissue Substitute, Open Approach */
"06RD47Z"  =  "1"  /* Replacement of Left Common Iliac Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06RD4JZ"  =  "1"  /* Replacement of Left Common Iliac Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06RD4KZ"  =  "1"  /* Replacement of Left Common Iliac Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06RF07Z"  =  "1"  /* Replacement of Right External Iliac Vein with Autologous Tissue Substitute, Open Approach */
"06RF0JZ"  =  "1"  /* Replacement of Right External Iliac Vein with Synthetic Substitute, Open Approach */
"06RF0KZ"  =  "1"  /* Replacement of Right External Iliac Vein with Nonautologous Tissue Substitute, Open Approach */
"06RF47Z"  =  "1"  /* Replacement of Right External Iliac Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06RF4JZ"  =  "1"  /* Replacement of Right External Iliac Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06RF4KZ"  =  "1"  /* Replacement of Right External Iliac Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06RG07Z"  =  "1"  /* Replacement of Left External Iliac Vein with Autologous Tissue Substitute, Open Approach */
"06RG0JZ"  =  "1"  /* Replacement of Left External Iliac Vein with Synthetic Substitute, Open Approach */
"06RG0KZ"  =  "1"  /* Replacement of Left External Iliac Vein with Nonautologous Tissue Substitute, Open Approach */
"06RG47Z"  =  "1"  /* Replacement of Left External Iliac Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06RG4JZ"  =  "1"  /* Replacement of Left External Iliac Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06RG4KZ"  =  "1"  /* Replacement of Left External Iliac Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06RH07Z"  =  "1"  /* Replacement of Right Hypogastric Vein with Autologous Tissue Substitute, Open Approach */
"06RH0JZ"  =  "1"  /* Replacement of Right Hypogastric Vein with Synthetic Substitute, Open Approach */
"06RH0KZ"  =  "1"  /* Replacement of Right Hypogastric Vein with Nonautologous Tissue Substitute, Open Approach */
"06RH47Z"  =  "1"  /* Replacement of Right Hypogastric Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06RH4JZ"  =  "1"  /* Replacement of Right Hypogastric Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06RH4KZ"  =  "1"  /* Replacement of Right Hypogastric Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06RJ07Z"  =  "1"  /* Replacement of Left Hypogastric Vein with Autologous Tissue Substitute, Open Approach */
"06RJ0JZ"  =  "1"  /* Replacement of Left Hypogastric Vein with Synthetic Substitute, Open Approach */
"06RJ0KZ"  =  "1"  /* Replacement of Left Hypogastric Vein with Nonautologous Tissue Substitute, Open Approach */
"06RJ47Z"  =  "1"  /* Replacement of Left Hypogastric Vein with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"06RJ4JZ"  =  "1"  /* Replacement of Left Hypogastric Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"06RJ4KZ"  =  "1"  /* Replacement of Left Hypogastric Vein with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"075P0ZZ"  =  "1"  /* Destruction of Spleen, Open Approach */
"075P3ZZ"  =  "1"  /* Destruction of Spleen, Percutaneous Approach */
"075P4ZZ"  =  "1"  /* Destruction of Spleen, Percutaneous Endoscopic Approach */
"079P00Z"  =  "1"  /* Drainage of Spleen with Drainage Device, Open Approach */
"079P0ZX"  =  "1"  /* Drainage of Spleen, Open Approach, Diagnostic */
"079P0ZZ"  =  "1"  /* Drainage of Spleen, Open Approach */
"079P3ZZ"  =  "1"  /* Drainage of Spleen, Percutaneous Approach */
"079P4ZZ"  =  "1"  /* Drainage of Spleen, Percutaneous Endoscopic Approach */
"07BP0ZX"  =  "1"  /* Excision of Spleen, Open Approach, Diagnostic */
"07BP0ZZ"  =  "1"  /* Excision of Spleen, Open Approach */
"07BP3ZZ"  =  "1"  /* Excision of Spleen, Percutaneous Approach */
"07BP4ZZ"  =  "1"  /* Excision of Spleen, Percutaneous Endoscopic Approach */
"07CP0ZZ"  =  "1"  /* Extirpation of Matter from Spleen, Open Approach */
"07JP0ZZ"  =  "1"  /* Inspection of Spleen, Open Approach */
"07NP0ZZ"  =  "1"  /* Release Spleen, Open Approach */
"07NP3ZZ"  =  "1"  /* Release Spleen, Percutaneous Approach */
"07NP4ZZ"  =  "1"  /* Release Spleen, Percutaneous Endoscopic Approach */
"07PP00Z"  =  "1"  /* Removal of Drainage Device from Spleen, Open Approach */
"07PP03Z"  =  "1"  /* Removal of Infusion Device from Spleen, Open Approach */
"07PP30Z"  =  "1"  /* Removal of Drainage Device from Spleen, Percutaneous Approach */
"07PP33Z"  =  "1"  /* Removal of Infusion Device from Spleen, Percutaneous Approach */
"07PP40Z"  =  "1"  /* Removal of Drainage Device from Spleen, Percutaneous Endoscopic Approach */
"07PP43Z"  =  "1"  /* Removal of Infusion Device from Spleen, Percutaneous Endoscopic Approach */
"07QP0ZZ"  =  "1"  /* Repair Spleen, Open Approach */
"07QP3ZZ"  =  "1"  /* Repair Spleen, Percutaneous Approach */
"07QP4ZZ"  =  "1"  /* Repair Spleen, Percutaneous Endoscopic Approach */
"07SP0ZZ"  =  "1"  /* Reposition Spleen, Open Approach */
"07TC0ZZ"  =  "1"  /* Resection of Pelvis Lymphatic, Open Approach */
"07TC4ZZ"  =  "1"  /* Resection of Pelvis Lymphatic, Percutaneous Endoscopic Approach */
"07TD0ZZ"  =  "1"  /* Resection of Aortic Lymphatic, Open Approach */
"07TD4ZZ"  =  "1"  /* Resection of Aortic Lymphatic, Percutaneous Endoscopic Approach */
"07TP0ZZ"  =  "1"  /* Resection of Spleen, Open Approach */
"07TP4ZZ"  =  "1"  /* Resection of Spleen, Percutaneous Endoscopic Approach */
"07WP00Z"  =  "1"  /* Revision of Drainage Device in Spleen, Open Approach */
"07WP03Z"  =  "1"  /* Revision of Infusion Device in Spleen, Open Approach */
"07WP30Z"  =  "1"  /* Revision of Drainage Device in Spleen, Percutaneous Approach */
"07WP33Z"  =  "1"  /* Revision of Infusion Device in Spleen, Percutaneous Approach */
"07WP40Z"  =  "1"  /* Revision of Drainage Device in Spleen, Percutaneous Endoscopic Approach */
"07WP43Z"  =  "1"  /* Revision of Infusion Device in Spleen, Percutaneous Endoscopic Approach */
"07YP0Z0"  =  "1"  /* Transplantation of Spleen, Allogeneic, Open Approach */
"07YP0Z1"  =  "1"  /* Transplantation of Spleen, Syngeneic, Open Approach */
"07YP0Z2"  =  "1"  /* Transplantation of Spleen, Zooplastic, Open Approach */
"0BQR0ZZ"  =  "1"  /* Repair Right Diaphragm, Open Approach */
"0BQR3ZZ"  =  "1"  /* Repair Right Diaphragm, Percutaneous Approach */
"0BQR4ZZ"  =  "1"  /* Repair Right Diaphragm, Percutaneous Endoscopic Approach */
"0BQS0ZZ"  =  "1"  /* Repair Left Diaphragm, Open Approach */
"0BQS3ZZ"  =  "1"  /* Repair Left Diaphragm, Percutaneous Approach */
"0BQS4ZZ"  =  "1"  /* Repair Left Diaphragm, Percutaneous Endoscopic Approach */
"0BUR07Z"  =  "1"  /* Supplement Right Diaphragm with Autologous Tissue Substitute, Open Approach */
"0BUR0JZ"  =  "1"  /* Supplement Right Diaphragm with Synthetic Substitute, Open Approach */
"0BUR0KZ"  =  "1"  /* Supplement Right Diaphragm with Nonautologous Tissue Substitute, Open Approach */
"0BUR47Z"  =  "1"  /* Supplement Right Diaphragm with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BUR4JZ"  =  "1"  /* Supplement Right Diaphragm with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BUR4KZ"  =  "1"  /* Supplement Right Diaphragm with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BUS07Z"  =  "1"  /* Supplement Left Diaphragm with Autologous Tissue Substitute, Open Approach */
"0BUS0JZ"  =  "1"  /* Supplement Left Diaphragm with Synthetic Substitute, Open Approach */
"0BUS0KZ"  =  "1"  /* Supplement Left Diaphragm with Nonautologous Tissue Substitute, Open Approach */
"0BUS47Z"  =  "1"  /* Supplement Left Diaphragm with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0BUS4JZ"  =  "1"  /* Supplement Left Diaphragm with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0BUS4KZ"  =  "1"  /* Supplement Left Diaphragm with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D11079"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1107A"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1107B"  =  "1"  /* Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D110J9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D110JA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D110JB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D110K9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D110KA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D110KB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D110Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Open Approach */
"0D110ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Open Approach */
"0D110ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Open Approach */
"0D11479"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1147A"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1147B"  =  "1"  /* Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114J9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114JA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114JB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114K9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114KA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114KB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D114ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D114ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D11879"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1187A"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1187B"  =  "1"  /* Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118J9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118JA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118JB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118K9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118KA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118KB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D118ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D118ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D12079"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1207A"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1207B"  =  "1"  /* Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D120J9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D120JA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D120JB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D120K9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D120KA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D120KB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D120Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Open Approach */
"0D120ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Open Approach */
"0D120ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Open Approach */
"0D12479"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1247A"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1247B"  =  "1"  /* Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124J9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124JA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124JB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124K9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124KA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124KB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D124ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D124ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D12879"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1287A"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1287B"  =  "1"  /* Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128J9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128JA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128JB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128K9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128KA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128KB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D128ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D128ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D13079"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1307A"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1307B"  =  "1"  /* Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D130J9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D130JA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D130JB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D130K9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D130KA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D130KB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D130Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Open Approach */
"0D130ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Open Approach */
"0D130ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Open Approach */
"0D13479"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1347A"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1347B"  =  "1"  /* Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134J9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134JA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134JB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134K9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134KA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134KB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D134ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D134ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D13879"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1387A"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1387B"  =  "1"  /* Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138J9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138JA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138JB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138K9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138KA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138KB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D138ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D138ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D15079"  =  "1"  /* Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1507A"  =  "1"  /* Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1507B"  =  "1"  /* Bypass Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D150J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D150JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D150JB"  =  "1"  /* Bypass Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D150K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D150KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D150KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D150Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Open Approach */
"0D150ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Open Approach */
"0D150ZB"  =  "1"  /* Bypass Esophagus to Ileum, Open Approach */
"0D15479"  =  "1"  /* Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1547A"  =  "1"  /* Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1547B"  =  "1"  /* Bypass Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154JB"  =  "1"  /* Bypass Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D154ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D154ZB"  =  "1"  /* Bypass Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D15879"  =  "1"  /* Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1587A"  =  "1"  /* Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1587B"  =  "1"  /* Bypass Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158JB"  =  "1"  /* Bypass Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D158ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D158ZB"  =  "1"  /* Bypass Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D16079"  =  "1"  /* Bypass Stomach to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1607A"  =  "1"  /* Bypass Stomach to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1607B"  =  "1"  /* Bypass Stomach to Ileum with Autologous Tissue Substitute, Open Approach */
"0D1607L"  =  "1"  /* Bypass Stomach to Transverse Colon with Autologous Tissue Substitute, Open Approach */
"0D160J9"  =  "1"  /* Bypass Stomach to Duodenum with Synthetic Substitute, Open Approach */
"0D160JA"  =  "1"  /* Bypass Stomach to Jejunum with Synthetic Substitute, Open Approach */
"0D160JB"  =  "1"  /* Bypass Stomach to Ileum with Synthetic Substitute, Open Approach */
"0D160JL"  =  "1"  /* Bypass Stomach to Transverse Colon with Synthetic Substitute, Open Approach */
"0D160K9"  =  "1"  /* Bypass Stomach to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D160KA"  =  "1"  /* Bypass Stomach to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D160KB"  =  "1"  /* Bypass Stomach to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D160KL"  =  "1"  /* Bypass Stomach to Transverse Colon with Nonautologous Tissue Substitute, Open Approach */
"0D160Z9"  =  "1"  /* Bypass Stomach to Duodenum, Open Approach */
"0D160ZA"  =  "1"  /* Bypass Stomach to Jejunum, Open Approach */
"0D160ZB"  =  "1"  /* Bypass Stomach to Ileum, Open Approach */
"0D160ZL"  =  "1"  /* Bypass Stomach to Transverse Colon, Open Approach */
"0D164ZA"  =  "1"  /* Bypass Stomach to Jejunum, Percutaneous Endoscopic Approach */
"0D16879"  =  "1"  /* Bypass Stomach to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1687A"  =  "1"  /* Bypass Stomach to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1687B"  =  "1"  /* Bypass Stomach to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1687L"  =  "1"  /* Bypass Stomach to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168J9"  =  "1"  /* Bypass Stomach to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168JA"  =  "1"  /* Bypass Stomach to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168JB"  =  "1"  /* Bypass Stomach to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168JL"  =  "1"  /* Bypass Stomach to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168K9"  =  "1"  /* Bypass Stomach to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168KA"  =  "1"  /* Bypass Stomach to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168KB"  =  "1"  /* Bypass Stomach to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168KL"  =  "1"  /* Bypass Stomach to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D168Z9"  =  "1"  /* Bypass Stomach to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D168ZA"  =  "1"  /* Bypass Stomach to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D168ZB"  =  "1"  /* Bypass Stomach to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D168ZL"  =  "1"  /* Bypass Stomach to Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D19074"  =  "1"  /* Bypass Duodenum to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D19079"  =  "1"  /* Bypass Duodenum to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1907A"  =  "1"  /* Bypass Duodenum to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1907B"  =  "1"  /* Bypass Duodenum to Ileum with Autologous Tissue Substitute, Open Approach */
"0D1907L"  =  "1"  /* Bypass Duodenum to Transverse Colon with Autologous Tissue Substitute, Open Approach */
"0D190J4"  =  "1"  /* Bypass Duodenum to Cutaneous with Synthetic Substitute, Open Approach */
"0D190J9"  =  "1"  /* Bypass Duodenum to Duodenum with Synthetic Substitute, Open Approach */
"0D190JA"  =  "1"  /* Bypass Duodenum to Jejunum with Synthetic Substitute, Open Approach */
"0D190JB"  =  "1"  /* Bypass Duodenum to Ileum with Synthetic Substitute, Open Approach */
"0D190JL"  =  "1"  /* Bypass Duodenum to Transverse Colon with Synthetic Substitute, Open Approach */
"0D190K4"  =  "1"  /* Bypass Duodenum to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D190K9"  =  "1"  /* Bypass Duodenum to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D190KA"  =  "1"  /* Bypass Duodenum to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D190KB"  =  "1"  /* Bypass Duodenum to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D190KL"  =  "1"  /* Bypass Duodenum to Transverse Colon with Nonautologous Tissue Substitute, Open Approach */
"0D190Z4"  =  "1"  /* Bypass Duodenum to Cutaneous, Open Approach */
"0D190Z9"  =  "1"  /* Bypass Duodenum to Duodenum, Open Approach */
"0D190ZA"  =  "1"  /* Bypass Duodenum to Jejunum, Open Approach */
"0D190ZB"  =  "1"  /* Bypass Duodenum to Ileum, Open Approach */
"0D190ZL"  =  "1"  /* Bypass Duodenum to Transverse Colon, Open Approach */
"0D193J4"  =  "1"  /* Bypass Duodenum to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D19474"  =  "1"  /* Bypass Duodenum to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D19479"  =  "1"  /* Bypass Duodenum to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1947A"  =  "1"  /* Bypass Duodenum to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1947B"  =  "1"  /* Bypass Duodenum to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1947L"  =  "1"  /* Bypass Duodenum to Transverse Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D194J4"  =  "1"  /* Bypass Duodenum to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D194J9"  =  "1"  /* Bypass Duodenum to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D194JA"  =  "1"  /* Bypass Duodenum to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D194JB"  =  "1"  /* Bypass Duodenum to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D194JL"  =  "1"  /* Bypass Duodenum to Transverse Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D194K4"  =  "1"  /* Bypass Duodenum to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D194K9"  =  "1"  /* Bypass Duodenum to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D194KA"  =  "1"  /* Bypass Duodenum to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D194KB"  =  "1"  /* Bypass Duodenum to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D194KL"  =  "1"  /* Bypass Duodenum to Transverse Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D194Z4"  =  "1"  /* Bypass Duodenum to Cutaneous, Percutaneous Endoscopic Approach */
"0D194Z9"  =  "1"  /* Bypass Duodenum to Duodenum, Percutaneous Endoscopic Approach */
"0D194ZA"  =  "1"  /* Bypass Duodenum to Jejunum, Percutaneous Endoscopic Approach */
"0D194ZB"  =  "1"  /* Bypass Duodenum to Ileum, Percutaneous Endoscopic Approach */
"0D194ZL"  =  "1"  /* Bypass Duodenum to Transverse Colon, Percutaneous Endoscopic Approach */
"0D19874"  =  "1"  /* Bypass Duodenum to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D19879"  =  "1"  /* Bypass Duodenum to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1987A"  =  "1"  /* Bypass Duodenum to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1987B"  =  "1"  /* Bypass Duodenum to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1987L"  =  "1"  /* Bypass Duodenum to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198J4"  =  "1"  /* Bypass Duodenum to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198J9"  =  "1"  /* Bypass Duodenum to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198JA"  =  "1"  /* Bypass Duodenum to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198JB"  =  "1"  /* Bypass Duodenum to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198JL"  =  "1"  /* Bypass Duodenum to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198K4"  =  "1"  /* Bypass Duodenum to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198K9"  =  "1"  /* Bypass Duodenum to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198KA"  =  "1"  /* Bypass Duodenum to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198KB"  =  "1"  /* Bypass Duodenum to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198KL"  =  "1"  /* Bypass Duodenum to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D198Z4"  =  "1"  /* Bypass Duodenum to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D198Z9"  =  "1"  /* Bypass Duodenum to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D198ZA"  =  "1"  /* Bypass Duodenum to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D198ZB"  =  "1"  /* Bypass Duodenum to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D198ZL"  =  "1"  /* Bypass Duodenum to Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D1A074"  =  "1"  /* Bypass Jejunum to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D1A07A"  =  "1"  /* Bypass Jejunum to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1A07B"  =  "1"  /* Bypass Jejunum to Ileum with Autologous Tissue Substitute, Open Approach */
"0D1A07H"  =  "1"  /* Bypass Jejunum to Cecum with Autologous Tissue Substitute, Open Approach */
"0D1A07K"  =  "1"  /* Bypass Jejunum to Ascending Colon with Autologous Tissue Substitute, Open Approach */
"0D1A07L"  =  "1"  /* Bypass Jejunum to Transverse Colon with Autologous Tissue Substitute, Open Approach */
"0D1A07M"  =  "1"  /* Bypass Jejunum to Descending Colon with Autologous Tissue Substitute, Open Approach */
"0D1A07N"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Autologous Tissue Substitute, Open Approach */
"0D1A07P"  =  "1"  /* Bypass Jejunum to Rectum with Autologous Tissue Substitute, Open Approach */
"0D1A07Q"  =  "1"  /* Bypass Jejunum to Anus with Autologous Tissue Substitute, Open Approach */
"0D1A0J4"  =  "1"  /* Bypass Jejunum to Cutaneous with Synthetic Substitute, Open Approach */
"0D1A0JA"  =  "1"  /* Bypass Jejunum to Jejunum with Synthetic Substitute, Open Approach */
"0D1A0JB"  =  "1"  /* Bypass Jejunum to Ileum with Synthetic Substitute, Open Approach */
"0D1A0JH"  =  "1"  /* Bypass Jejunum to Cecum with Synthetic Substitute, Open Approach */
"0D1A0JK"  =  "1"  /* Bypass Jejunum to Ascending Colon with Synthetic Substitute, Open Approach */
"0D1A0JL"  =  "1"  /* Bypass Jejunum to Transverse Colon with Synthetic Substitute, Open Approach */
"0D1A0JM"  =  "1"  /* Bypass Jejunum to Descending Colon with Synthetic Substitute, Open Approach */
"0D1A0JN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Synthetic Substitute, Open Approach */
"0D1A0JP"  =  "1"  /* Bypass Jejunum to Rectum with Synthetic Substitute, Open Approach */
"0D1A0JQ"  =  "1"  /* Bypass Jejunum to Anus with Synthetic Substitute, Open Approach */
"0D1A0K4"  =  "1"  /* Bypass Jejunum to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KA"  =  "1"  /* Bypass Jejunum to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KB"  =  "1"  /* Bypass Jejunum to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KH"  =  "1"  /* Bypass Jejunum to Cecum with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KK"  =  "1"  /* Bypass Jejunum to Ascending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KL"  =  "1"  /* Bypass Jejunum to Transverse Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KM"  =  "1"  /* Bypass Jejunum to Descending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KP"  =  "1"  /* Bypass Jejunum to Rectum with Nonautologous Tissue Substitute, Open Approach */
"0D1A0KQ"  =  "1"  /* Bypass Jejunum to Anus with Nonautologous Tissue Substitute, Open Approach */
"0D1A0Z4"  =  "1"  /* Bypass Jejunum to Cutaneous, Open Approach */
"0D1A0ZA"  =  "1"  /* Bypass Jejunum to Jejunum, Open Approach */
"0D1A0ZB"  =  "1"  /* Bypass Jejunum to Ileum, Open Approach */
"0D1A0ZH"  =  "1"  /* Bypass Jejunum to Cecum, Open Approach */
"0D1A0ZK"  =  "1"  /* Bypass Jejunum to Ascending Colon, Open Approach */
"0D1A0ZL"  =  "1"  /* Bypass Jejunum to Transverse Colon, Open Approach */
"0D1A0ZM"  =  "1"  /* Bypass Jejunum to Descending Colon, Open Approach */
"0D1A0ZN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon, Open Approach */
"0D1A0ZP"  =  "1"  /* Bypass Jejunum to Rectum, Open Approach */
"0D1A0ZQ"  =  "1"  /* Bypass Jejunum to Anus, Open Approach */
"0D1A3J4"  =  "1"  /* Bypass Jejunum to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D1A474"  =  "1"  /* Bypass Jejunum to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47A"  =  "1"  /* Bypass Jejunum to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47B"  =  "1"  /* Bypass Jejunum to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47H"  =  "1"  /* Bypass Jejunum to Cecum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47K"  =  "1"  /* Bypass Jejunum to Ascending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47L"  =  "1"  /* Bypass Jejunum to Transverse Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47M"  =  "1"  /* Bypass Jejunum to Descending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47N"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47P"  =  "1"  /* Bypass Jejunum to Rectum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A47Q"  =  "1"  /* Bypass Jejunum to Anus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4J4"  =  "1"  /* Bypass Jejunum to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JA"  =  "1"  /* Bypass Jejunum to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JB"  =  "1"  /* Bypass Jejunum to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JH"  =  "1"  /* Bypass Jejunum to Cecum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JK"  =  "1"  /* Bypass Jejunum to Ascending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JL"  =  "1"  /* Bypass Jejunum to Transverse Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JM"  =  "1"  /* Bypass Jejunum to Descending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JP"  =  "1"  /* Bypass Jejunum to Rectum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4JQ"  =  "1"  /* Bypass Jejunum to Anus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1A4K4"  =  "1"  /* Bypass Jejunum to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KA"  =  "1"  /* Bypass Jejunum to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KB"  =  "1"  /* Bypass Jejunum to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KH"  =  "1"  /* Bypass Jejunum to Cecum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KK"  =  "1"  /* Bypass Jejunum to Ascending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KL"  =  "1"  /* Bypass Jejunum to Transverse Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KM"  =  "1"  /* Bypass Jejunum to Descending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KP"  =  "1"  /* Bypass Jejunum to Rectum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4KQ"  =  "1"  /* Bypass Jejunum to Anus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1A4Z4"  =  "1"  /* Bypass Jejunum to Cutaneous, Percutaneous Endoscopic Approach */
"0D1A4ZA"  =  "1"  /* Bypass Jejunum to Jejunum, Percutaneous Endoscopic Approach */
"0D1A4ZB"  =  "1"  /* Bypass Jejunum to Ileum, Percutaneous Endoscopic Approach */
"0D1A4ZH"  =  "1"  /* Bypass Jejunum to Cecum, Percutaneous Endoscopic Approach */
"0D1A4ZK"  =  "1"  /* Bypass Jejunum to Ascending Colon, Percutaneous Endoscopic Approach */
"0D1A4ZL"  =  "1"  /* Bypass Jejunum to Transverse Colon, Percutaneous Endoscopic Approach */
"0D1A4ZM"  =  "1"  /* Bypass Jejunum to Descending Colon, Percutaneous Endoscopic Approach */
"0D1A4ZN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D1A4ZP"  =  "1"  /* Bypass Jejunum to Rectum, Percutaneous Endoscopic Approach */
"0D1A4ZQ"  =  "1"  /* Bypass Jejunum to Anus, Percutaneous Endoscopic Approach */
"0D1A874"  =  "1"  /* Bypass Jejunum to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87A"  =  "1"  /* Bypass Jejunum to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87B"  =  "1"  /* Bypass Jejunum to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87H"  =  "1"  /* Bypass Jejunum to Cecum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87K"  =  "1"  /* Bypass Jejunum to Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87L"  =  "1"  /* Bypass Jejunum to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87M"  =  "1"  /* Bypass Jejunum to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87N"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87P"  =  "1"  /* Bypass Jejunum to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A87Q"  =  "1"  /* Bypass Jejunum to Anus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8J4"  =  "1"  /* Bypass Jejunum to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JA"  =  "1"  /* Bypass Jejunum to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JB"  =  "1"  /* Bypass Jejunum to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JH"  =  "1"  /* Bypass Jejunum to Cecum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JK"  =  "1"  /* Bypass Jejunum to Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JL"  =  "1"  /* Bypass Jejunum to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JM"  =  "1"  /* Bypass Jejunum to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JP"  =  "1"  /* Bypass Jejunum to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8JQ"  =  "1"  /* Bypass Jejunum to Anus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8K4"  =  "1"  /* Bypass Jejunum to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KA"  =  "1"  /* Bypass Jejunum to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KB"  =  "1"  /* Bypass Jejunum to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KH"  =  "1"  /* Bypass Jejunum to Cecum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KK"  =  "1"  /* Bypass Jejunum to Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KL"  =  "1"  /* Bypass Jejunum to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KM"  =  "1"  /* Bypass Jejunum to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KP"  =  "1"  /* Bypass Jejunum to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8KQ"  =  "1"  /* Bypass Jejunum to Anus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1A8Z4"  =  "1"  /* Bypass Jejunum to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZA"  =  "1"  /* Bypass Jejunum to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZB"  =  "1"  /* Bypass Jejunum to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZH"  =  "1"  /* Bypass Jejunum to Cecum, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZK"  =  "1"  /* Bypass Jejunum to Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZL"  =  "1"  /* Bypass Jejunum to Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZM"  =  "1"  /* Bypass Jejunum to Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZP"  =  "1"  /* Bypass Jejunum to Rectum, Via Natural or Artificial Opening Endoscopic */
"0D1A8ZQ"  =  "1"  /* Bypass Jejunum to Anus, Via Natural or Artificial Opening Endoscopic */
"0D1B074"  =  "1"  /* Bypass Ileum to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D1B07B"  =  "1"  /* Bypass Ileum to Ileum with Autologous Tissue Substitute, Open Approach */
"0D1B07H"  =  "1"  /* Bypass Ileum to Cecum with Autologous Tissue Substitute, Open Approach */
"0D1B07K"  =  "1"  /* Bypass Ileum to Ascending Colon with Autologous Tissue Substitute, Open Approach */
"0D1B07L"  =  "1"  /* Bypass Ileum to Transverse Colon with Autologous Tissue Substitute, Open Approach */
"0D1B07M"  =  "1"  /* Bypass Ileum to Descending Colon with Autologous Tissue Substitute, Open Approach */
"0D1B07N"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Autologous Tissue Substitute, Open Approach */
"0D1B07P"  =  "1"  /* Bypass Ileum to Rectum with Autologous Tissue Substitute, Open Approach */
"0D1B07Q"  =  "1"  /* Bypass Ileum to Anus with Autologous Tissue Substitute, Open Approach */
"0D1B0J4"  =  "1"  /* Bypass Ileum to Cutaneous with Synthetic Substitute, Open Approach */
"0D1B0JB"  =  "1"  /* Bypass Ileum to Ileum with Synthetic Substitute, Open Approach */
"0D1B0JH"  =  "1"  /* Bypass Ileum to Cecum with Synthetic Substitute, Open Approach */
"0D1B0JK"  =  "1"  /* Bypass Ileum to Ascending Colon with Synthetic Substitute, Open Approach */
"0D1B0JL"  =  "1"  /* Bypass Ileum to Transverse Colon with Synthetic Substitute, Open Approach */
"0D1B0JM"  =  "1"  /* Bypass Ileum to Descending Colon with Synthetic Substitute, Open Approach */
"0D1B0JN"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Synthetic Substitute, Open Approach */
"0D1B0JP"  =  "1"  /* Bypass Ileum to Rectum with Synthetic Substitute, Open Approach */
"0D1B0JQ"  =  "1"  /* Bypass Ileum to Anus with Synthetic Substitute, Open Approach */
"0D1B0K4"  =  "1"  /* Bypass Ileum to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D1B0KB"  =  "1"  /* Bypass Ileum to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D1B0KH"  =  "1"  /* Bypass Ileum to Cecum with Nonautologous Tissue Substitute, Open Approach */
"0D1B0KK"  =  "1"  /* Bypass Ileum to Ascending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1B0KL"  =  "1"  /* Bypass Ileum to Transverse Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1B0KM"  =  "1"  /* Bypass Ileum to Descending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1B0KN"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1B0KP"  =  "1"  /* Bypass Ileum to Rectum with Nonautologous Tissue Substitute, Open Approach */
"0D1B0KQ"  =  "1"  /* Bypass Ileum to Anus with Nonautologous Tissue Substitute, Open Approach */
"0D1B0Z4"  =  "1"  /* Bypass Ileum to Cutaneous, Open Approach */
"0D1B0ZB"  =  "1"  /* Bypass Ileum to Ileum, Open Approach */
"0D1B0ZH"  =  "1"  /* Bypass Ileum to Cecum, Open Approach */
"0D1B0ZK"  =  "1"  /* Bypass Ileum to Ascending Colon, Open Approach */
"0D1B0ZL"  =  "1"  /* Bypass Ileum to Transverse Colon, Open Approach */
"0D1B0ZM"  =  "1"  /* Bypass Ileum to Descending Colon, Open Approach */
"0D1B0ZN"  =  "1"  /* Bypass Ileum to Sigmoid Colon, Open Approach */
"0D1B0ZP"  =  "1"  /* Bypass Ileum to Rectum, Open Approach */
"0D1B0ZQ"  =  "1"  /* Bypass Ileum to Anus, Open Approach */
"0D1B3J4"  =  "1"  /* Bypass Ileum to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D1B474"  =  "1"  /* Bypass Ileum to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B47B"  =  "1"  /* Bypass Ileum to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B47H"  =  "1"  /* Bypass Ileum to Cecum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B47K"  =  "1"  /* Bypass Ileum to Ascending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B47L"  =  "1"  /* Bypass Ileum to Transverse Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B47M"  =  "1"  /* Bypass Ileum to Descending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B47N"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B47P"  =  "1"  /* Bypass Ileum to Rectum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B47Q"  =  "1"  /* Bypass Ileum to Anus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4J4"  =  "1"  /* Bypass Ileum to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4JB"  =  "1"  /* Bypass Ileum to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4JH"  =  "1"  /* Bypass Ileum to Cecum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4JK"  =  "1"  /* Bypass Ileum to Ascending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4JL"  =  "1"  /* Bypass Ileum to Transverse Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4JM"  =  "1"  /* Bypass Ileum to Descending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4JN"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4JP"  =  "1"  /* Bypass Ileum to Rectum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4JQ"  =  "1"  /* Bypass Ileum to Anus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1B4K4"  =  "1"  /* Bypass Ileum to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4KB"  =  "1"  /* Bypass Ileum to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4KH"  =  "1"  /* Bypass Ileum to Cecum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4KK"  =  "1"  /* Bypass Ileum to Ascending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4KL"  =  "1"  /* Bypass Ileum to Transverse Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4KM"  =  "1"  /* Bypass Ileum to Descending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4KN"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4KP"  =  "1"  /* Bypass Ileum to Rectum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4KQ"  =  "1"  /* Bypass Ileum to Anus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1B4Z4"  =  "1"  /* Bypass Ileum to Cutaneous, Percutaneous Endoscopic Approach */
"0D1B4ZB"  =  "1"  /* Bypass Ileum to Ileum, Percutaneous Endoscopic Approach */
"0D1B4ZH"  =  "1"  /* Bypass Ileum to Cecum, Percutaneous Endoscopic Approach */
"0D1B4ZK"  =  "1"  /* Bypass Ileum to Ascending Colon, Percutaneous Endoscopic Approach */
"0D1B4ZL"  =  "1"  /* Bypass Ileum to Transverse Colon, Percutaneous Endoscopic Approach */
"0D1B4ZM"  =  "1"  /* Bypass Ileum to Descending Colon, Percutaneous Endoscopic Approach */
"0D1B4ZN"  =  "1"  /* Bypass Ileum to Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D1B4ZP"  =  "1"  /* Bypass Ileum to Rectum, Percutaneous Endoscopic Approach */
"0D1B4ZQ"  =  "1"  /* Bypass Ileum to Anus, Percutaneous Endoscopic Approach */
"0D1B874"  =  "1"  /* Bypass Ileum to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B87B"  =  "1"  /* Bypass Ileum to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B87H"  =  "1"  /* Bypass Ileum to Cecum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B87K"  =  "1"  /* Bypass Ileum to Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B87L"  =  "1"  /* Bypass Ileum to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B87M"  =  "1"  /* Bypass Ileum to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B87N"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B87P"  =  "1"  /* Bypass Ileum to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B87Q"  =  "1"  /* Bypass Ileum to Anus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8J4"  =  "1"  /* Bypass Ileum to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8JB"  =  "1"  /* Bypass Ileum to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8JH"  =  "1"  /* Bypass Ileum to Cecum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8JK"  =  "1"  /* Bypass Ileum to Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8JL"  =  "1"  /* Bypass Ileum to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8JM"  =  "1"  /* Bypass Ileum to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8JN"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8JP"  =  "1"  /* Bypass Ileum to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8JQ"  =  "1"  /* Bypass Ileum to Anus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8K4"  =  "1"  /* Bypass Ileum to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8KB"  =  "1"  /* Bypass Ileum to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8KH"  =  "1"  /* Bypass Ileum to Cecum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8KK"  =  "1"  /* Bypass Ileum to Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8KL"  =  "1"  /* Bypass Ileum to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8KM"  =  "1"  /* Bypass Ileum to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8KN"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8KP"  =  "1"  /* Bypass Ileum to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8KQ"  =  "1"  /* Bypass Ileum to Anus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1B8Z4"  =  "1"  /* Bypass Ileum to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D1B8ZB"  =  "1"  /* Bypass Ileum to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D1B8ZH"  =  "1"  /* Bypass Ileum to Cecum, Via Natural or Artificial Opening Endoscopic */
"0D1B8ZK"  =  "1"  /* Bypass Ileum to Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1B8ZL"  =  "1"  /* Bypass Ileum to Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D1B8ZM"  =  "1"  /* Bypass Ileum to Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1B8ZN"  =  "1"  /* Bypass Ileum to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D1B8ZP"  =  "1"  /* Bypass Ileum to Rectum, Via Natural or Artificial Opening Endoscopic */
"0D1B8ZQ"  =  "1"  /* Bypass Ileum to Anus, Via Natural or Artificial Opening Endoscopic */
"0D1H074"  =  "1"  /* Bypass Cecum to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D1H07H"  =  "1"  /* Bypass Cecum to Cecum with Autologous Tissue Substitute, Open Approach */
"0D1H07K"  =  "1"  /* Bypass Cecum to Ascending Colon with Autologous Tissue Substitute, Open Approach */
"0D1H07L"  =  "1"  /* Bypass Cecum to Transverse Colon with Autologous Tissue Substitute, Open Approach */
"0D1H07M"  =  "1"  /* Bypass Cecum to Descending Colon with Autologous Tissue Substitute, Open Approach */
"0D1H07N"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Autologous Tissue Substitute, Open Approach */
"0D1H07P"  =  "1"  /* Bypass Cecum to Rectum with Autologous Tissue Substitute, Open Approach */
"0D1H0J4"  =  "1"  /* Bypass Cecum to Cutaneous with Synthetic Substitute, Open Approach */
"0D1H0JH"  =  "1"  /* Bypass Cecum to Cecum with Synthetic Substitute, Open Approach */
"0D1H0JK"  =  "1"  /* Bypass Cecum to Ascending Colon with Synthetic Substitute, Open Approach */
"0D1H0JL"  =  "1"  /* Bypass Cecum to Transverse Colon with Synthetic Substitute, Open Approach */
"0D1H0JM"  =  "1"  /* Bypass Cecum to Descending Colon with Synthetic Substitute, Open Approach */
"0D1H0JN"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Synthetic Substitute, Open Approach */
"0D1H0JP"  =  "1"  /* Bypass Cecum to Rectum with Synthetic Substitute, Open Approach */
"0D1H0K4"  =  "1"  /* Bypass Cecum to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D1H0KH"  =  "1"  /* Bypass Cecum to Cecum with Nonautologous Tissue Substitute, Open Approach */
"0D1H0KK"  =  "1"  /* Bypass Cecum to Ascending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1H0KL"  =  "1"  /* Bypass Cecum to Transverse Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1H0KM"  =  "1"  /* Bypass Cecum to Descending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1H0KN"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1H0KP"  =  "1"  /* Bypass Cecum to Rectum with Nonautologous Tissue Substitute, Open Approach */
"0D1H0Z4"  =  "1"  /* Bypass Cecum to Cutaneous, Open Approach */
"0D1H0ZH"  =  "1"  /* Bypass Cecum to Cecum, Open Approach */
"0D1H0ZK"  =  "1"  /* Bypass Cecum to Ascending Colon, Open Approach */
"0D1H0ZL"  =  "1"  /* Bypass Cecum to Transverse Colon, Open Approach */
"0D1H0ZM"  =  "1"  /* Bypass Cecum to Descending Colon, Open Approach */
"0D1H0ZN"  =  "1"  /* Bypass Cecum to Sigmoid Colon, Open Approach */
"0D1H0ZP"  =  "1"  /* Bypass Cecum to Rectum, Open Approach */
"0D1H3J4"  =  "1"  /* Bypass Cecum to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D1H474"  =  "1"  /* Bypass Cecum to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H47H"  =  "1"  /* Bypass Cecum to Cecum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H47K"  =  "1"  /* Bypass Cecum to Ascending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H47L"  =  "1"  /* Bypass Cecum to Transverse Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H47M"  =  "1"  /* Bypass Cecum to Descending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H47N"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H47P"  =  "1"  /* Bypass Cecum to Rectum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H4J4"  =  "1"  /* Bypass Cecum to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1H4JH"  =  "1"  /* Bypass Cecum to Cecum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1H4JK"  =  "1"  /* Bypass Cecum to Ascending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1H4JL"  =  "1"  /* Bypass Cecum to Transverse Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1H4JM"  =  "1"  /* Bypass Cecum to Descending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1H4JN"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1H4JP"  =  "1"  /* Bypass Cecum to Rectum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1H4K4"  =  "1"  /* Bypass Cecum to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H4KH"  =  "1"  /* Bypass Cecum to Cecum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H4KK"  =  "1"  /* Bypass Cecum to Ascending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H4KL"  =  "1"  /* Bypass Cecum to Transverse Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H4KM"  =  "1"  /* Bypass Cecum to Descending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H4KN"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H4KP"  =  "1"  /* Bypass Cecum to Rectum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1H4Z4"  =  "1"  /* Bypass Cecum to Cutaneous, Percutaneous Endoscopic Approach */
"0D1H4ZH"  =  "1"  /* Bypass Cecum to Cecum, Percutaneous Endoscopic Approach */
"0D1H4ZK"  =  "1"  /* Bypass Cecum to Ascending Colon, Percutaneous Endoscopic Approach */
"0D1H4ZL"  =  "1"  /* Bypass Cecum to Transverse Colon, Percutaneous Endoscopic Approach */
"0D1H4ZM"  =  "1"  /* Bypass Cecum to Descending Colon, Percutaneous Endoscopic Approach */
"0D1H4ZN"  =  "1"  /* Bypass Cecum to Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D1H4ZP"  =  "1"  /* Bypass Cecum to Rectum, Percutaneous Endoscopic Approach */
"0D1H874"  =  "1"  /* Bypass Cecum to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H87H"  =  "1"  /* Bypass Cecum to Cecum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H87K"  =  "1"  /* Bypass Cecum to Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H87L"  =  "1"  /* Bypass Cecum to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H87M"  =  "1"  /* Bypass Cecum to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H87N"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H87P"  =  "1"  /* Bypass Cecum to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8J4"  =  "1"  /* Bypass Cecum to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8JH"  =  "1"  /* Bypass Cecum to Cecum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8JK"  =  "1"  /* Bypass Cecum to Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8JL"  =  "1"  /* Bypass Cecum to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8JM"  =  "1"  /* Bypass Cecum to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8JN"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8JP"  =  "1"  /* Bypass Cecum to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8K4"  =  "1"  /* Bypass Cecum to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8KH"  =  "1"  /* Bypass Cecum to Cecum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8KK"  =  "1"  /* Bypass Cecum to Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8KL"  =  "1"  /* Bypass Cecum to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8KM"  =  "1"  /* Bypass Cecum to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8KN"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8KP"  =  "1"  /* Bypass Cecum to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1H8Z4"  =  "1"  /* Bypass Cecum to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D1H8ZH"  =  "1"  /* Bypass Cecum to Cecum, Via Natural or Artificial Opening Endoscopic */
"0D1H8ZK"  =  "1"  /* Bypass Cecum to Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1H8ZL"  =  "1"  /* Bypass Cecum to Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D1H8ZM"  =  "1"  /* Bypass Cecum to Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1H8ZN"  =  "1"  /* Bypass Cecum to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D1H8ZP"  =  "1"  /* Bypass Cecum to Rectum, Via Natural or Artificial Opening Endoscopic */
"0D1K074"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D1K07K"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Autologous Tissue Substitute, Open Approach */
"0D1K07L"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Autologous Tissue Substitute, Open Approach */
"0D1K07M"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Autologous Tissue Substitute, Open Approach */
"0D1K07N"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Autologous Tissue Substitute, Open Approach */
"0D1K07P"  =  "1"  /* Bypass Ascending Colon to Rectum with Autologous Tissue Substitute, Open Approach */
"0D1K0J4"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Synthetic Substitute, Open Approach */
"0D1K0JK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Synthetic Substitute, Open Approach */
"0D1K0JL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Synthetic Substitute, Open Approach */
"0D1K0JM"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Synthetic Substitute, Open Approach */
"0D1K0JN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Synthetic Substitute, Open Approach */
"0D1K0JP"  =  "1"  /* Bypass Ascending Colon to Rectum with Synthetic Substitute, Open Approach */
"0D1K0K4"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D1K0KK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1K0KL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1K0KM"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1K0KN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1K0KP"  =  "1"  /* Bypass Ascending Colon to Rectum with Nonautologous Tissue Substitute, Open Approach */
"0D1K0Z4"  =  "1"  /* Bypass Ascending Colon to Cutaneous, Open Approach */
"0D1K0ZK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon, Open Approach */
"0D1K0ZL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon, Open Approach */
"0D1K0ZM"  =  "1"  /* Bypass Ascending Colon to Descending Colon, Open Approach */
"0D1K0ZN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon, Open Approach */
"0D1K0ZP"  =  "1"  /* Bypass Ascending Colon to Rectum, Open Approach */
"0D1K3J4"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D1K474"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K47K"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K47L"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K47M"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K47N"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K47P"  =  "1"  /* Bypass Ascending Colon to Rectum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K4J4"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1K4JK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1K4JL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1K4JM"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1K4JN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1K4JP"  =  "1"  /* Bypass Ascending Colon to Rectum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1K4K4"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K4KK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K4KL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K4KM"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K4KN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K4KP"  =  "1"  /* Bypass Ascending Colon to Rectum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1K4Z4"  =  "1"  /* Bypass Ascending Colon to Cutaneous, Percutaneous Endoscopic Approach */
"0D1K4ZK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon, Percutaneous Endoscopic Approach */
"0D1K4ZL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon, Percutaneous Endoscopic Approach */
"0D1K4ZM"  =  "1"  /* Bypass Ascending Colon to Descending Colon, Percutaneous Endoscopic Approach */
"0D1K4ZN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D1K4ZP"  =  "1"  /* Bypass Ascending Colon to Rectum, Percutaneous Endoscopic Approach */
"0D1K874"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K87K"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K87L"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K87M"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K87N"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K87P"  =  "1"  /* Bypass Ascending Colon to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8J4"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8JK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8JL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8JM"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8JN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8JP"  =  "1"  /* Bypass Ascending Colon to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8K4"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8KK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8KL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8KM"  =  "1"  /* Bypass Ascending Colon to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8KN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8KP"  =  "1"  /* Bypass Ascending Colon to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1K8Z4"  =  "1"  /* Bypass Ascending Colon to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D1K8ZK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1K8ZL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D1K8ZM"  =  "1"  /* Bypass Ascending Colon to Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1K8ZN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D1K8ZP"  =  "1"  /* Bypass Ascending Colon to Rectum, Via Natural or Artificial Opening Endoscopic */
"0D1L074"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D1L07L"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Autologous Tissue Substitute, Open Approach */
"0D1L07M"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Autologous Tissue Substitute, Open Approach */
"0D1L07N"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Autologous Tissue Substitute, Open Approach */
"0D1L07P"  =  "1"  /* Bypass Transverse Colon to Rectum with Autologous Tissue Substitute, Open Approach */
"0D1L0J4"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Synthetic Substitute, Open Approach */
"0D1L0JL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Synthetic Substitute, Open Approach */
"0D1L0JM"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Synthetic Substitute, Open Approach */
"0D1L0JN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Synthetic Substitute, Open Approach */
"0D1L0JP"  =  "1"  /* Bypass Transverse Colon to Rectum with Synthetic Substitute, Open Approach */
"0D1L0K4"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D1L0KL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1L0KM"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1L0KN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1L0KP"  =  "1"  /* Bypass Transverse Colon to Rectum with Nonautologous Tissue Substitute, Open Approach */
"0D1L0Z4"  =  "1"  /* Bypass Transverse Colon to Cutaneous, Open Approach */
"0D1L0ZL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon, Open Approach */
"0D1L0ZM"  =  "1"  /* Bypass Transverse Colon to Descending Colon, Open Approach */
"0D1L0ZN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon, Open Approach */
"0D1L0ZP"  =  "1"  /* Bypass Transverse Colon to Rectum, Open Approach */
"0D1L3J4"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D1L474"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L47L"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L47M"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L47N"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L47P"  =  "1"  /* Bypass Transverse Colon to Rectum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L4J4"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1L4JL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1L4JM"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1L4JN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1L4JP"  =  "1"  /* Bypass Transverse Colon to Rectum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1L4K4"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L4KL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L4KM"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L4KN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L4KP"  =  "1"  /* Bypass Transverse Colon to Rectum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1L4Z4"  =  "1"  /* Bypass Transverse Colon to Cutaneous, Percutaneous Endoscopic Approach */
"0D1L4ZL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon, Percutaneous Endoscopic Approach */
"0D1L4ZM"  =  "1"  /* Bypass Transverse Colon to Descending Colon, Percutaneous Endoscopic Approach */
"0D1L4ZN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D1L4ZP"  =  "1"  /* Bypass Transverse Colon to Rectum, Percutaneous Endoscopic Approach */
"0D1L874"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L87L"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L87M"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L87N"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L87P"  =  "1"  /* Bypass Transverse Colon to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8J4"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8JL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8JM"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8JN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8JP"  =  "1"  /* Bypass Transverse Colon to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8K4"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8KL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8KM"  =  "1"  /* Bypass Transverse Colon to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8KN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8KP"  =  "1"  /* Bypass Transverse Colon to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1L8Z4"  =  "1"  /* Bypass Transverse Colon to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D1L8ZL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D1L8ZM"  =  "1"  /* Bypass Transverse Colon to Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1L8ZN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D1L8ZP"  =  "1"  /* Bypass Transverse Colon to Rectum, Via Natural or Artificial Opening Endoscopic */
"0D1M074"  =  "1"  /* Bypass Descending Colon to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D1M07M"  =  "1"  /* Bypass Descending Colon to Descending Colon with Autologous Tissue Substitute, Open Approach */
"0D1M07N"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Autologous Tissue Substitute, Open Approach */
"0D1M07P"  =  "1"  /* Bypass Descending Colon to Rectum with Autologous Tissue Substitute, Open Approach */
"0D1M0J4"  =  "1"  /* Bypass Descending Colon to Cutaneous with Synthetic Substitute, Open Approach */
"0D1M0JM"  =  "1"  /* Bypass Descending Colon to Descending Colon with Synthetic Substitute, Open Approach */
"0D1M0JN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Synthetic Substitute, Open Approach */
"0D1M0JP"  =  "1"  /* Bypass Descending Colon to Rectum with Synthetic Substitute, Open Approach */
"0D1M0K4"  =  "1"  /* Bypass Descending Colon to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D1M0KM"  =  "1"  /* Bypass Descending Colon to Descending Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1M0KN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1M0KP"  =  "1"  /* Bypass Descending Colon to Rectum with Nonautologous Tissue Substitute, Open Approach */
"0D1M0Z4"  =  "1"  /* Bypass Descending Colon to Cutaneous, Open Approach */
"0D1M0ZM"  =  "1"  /* Bypass Descending Colon to Descending Colon, Open Approach */
"0D1M0ZN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon, Open Approach */
"0D1M0ZP"  =  "1"  /* Bypass Descending Colon to Rectum, Open Approach */
"0D1M3J4"  =  "1"  /* Bypass Descending Colon to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D1M474"  =  "1"  /* Bypass Descending Colon to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1M47M"  =  "1"  /* Bypass Descending Colon to Descending Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1M47N"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1M47P"  =  "1"  /* Bypass Descending Colon to Rectum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1M4J4"  =  "1"  /* Bypass Descending Colon to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1M4JM"  =  "1"  /* Bypass Descending Colon to Descending Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1M4JN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1M4JP"  =  "1"  /* Bypass Descending Colon to Rectum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1M4K4"  =  "1"  /* Bypass Descending Colon to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1M4KM"  =  "1"  /* Bypass Descending Colon to Descending Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1M4KN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1M4KP"  =  "1"  /* Bypass Descending Colon to Rectum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1M4Z4"  =  "1"  /* Bypass Descending Colon to Cutaneous, Percutaneous Endoscopic Approach */
"0D1M4ZM"  =  "1"  /* Bypass Descending Colon to Descending Colon, Percutaneous Endoscopic Approach */
"0D1M4ZN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D1M4ZP"  =  "1"  /* Bypass Descending Colon to Rectum, Percutaneous Endoscopic Approach */
"0D1M874"  =  "1"  /* Bypass Descending Colon to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M87M"  =  "1"  /* Bypass Descending Colon to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M87N"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M87P"  =  "1"  /* Bypass Descending Colon to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8J4"  =  "1"  /* Bypass Descending Colon to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8JM"  =  "1"  /* Bypass Descending Colon to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8JN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8JP"  =  "1"  /* Bypass Descending Colon to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8K4"  =  "1"  /* Bypass Descending Colon to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8KM"  =  "1"  /* Bypass Descending Colon to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8KN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8KP"  =  "1"  /* Bypass Descending Colon to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1M8Z4"  =  "1"  /* Bypass Descending Colon to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D1M8ZM"  =  "1"  /* Bypass Descending Colon to Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0D1M8ZN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D1M8ZP"  =  "1"  /* Bypass Descending Colon to Rectum, Via Natural or Artificial Opening Endoscopic */
"0D1N074"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D1N07N"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Autologous Tissue Substitute, Open Approach */
"0D1N07P"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Autologous Tissue Substitute, Open Approach */
"0D1N0J4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Synthetic Substitute, Open Approach */
"0D1N0JN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Synthetic Substitute, Open Approach */
"0D1N0JP"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Synthetic Substitute, Open Approach */
"0D1N0K4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D1N0KN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach */
"0D1N0KP"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Nonautologous Tissue Substitute, Open Approach */
"0D1N0Z4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous, Open Approach */
"0D1N0ZN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon, Open Approach */
"0D1N0ZP"  =  "1"  /* Bypass Sigmoid Colon to Rectum, Open Approach */
"0D1N3J4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D1N474"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1N47N"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1N47P"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1N4J4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1N4JN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1N4JP"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D1N4K4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1N4KN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1N4KP"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1N4Z4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous, Percutaneous Endoscopic Approach */
"0D1N4ZN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D1N4ZP"  =  "1"  /* Bypass Sigmoid Colon to Rectum, Percutaneous Endoscopic Approach */
"0D1N874"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N87N"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N87P"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N8J4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N8JN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N8JP"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N8K4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N8KN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N8KP"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1N8Z4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D1N8ZN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D1N8ZP"  =  "1"  /* Bypass Sigmoid Colon to Rectum, Via Natural or Artificial Opening Endoscopic */
"0D560ZZ"  =  "1"  /* Destruction of Stomach, Open Approach */
"0D563ZZ"  =  "1"  /* Destruction of Stomach, Percutaneous Approach */
"0D567ZZ"  =  "1"  /* Destruction of Stomach, Via Natural or Artificial Opening */
"0D570ZZ"  =  "1"  /* Destruction of Stomach, Pylorus, Open Approach */
"0D573ZZ"  =  "1"  /* Destruction of Stomach, Pylorus, Percutaneous Approach */
"0D577ZZ"  =  "1"  /* Destruction of Stomach, Pylorus, Via Natural or Artificial Opening */
"0D580ZZ"  =  "1"  /* Destruction of Small Intestine, Open Approach */
"0D583ZZ"  =  "1"  /* Destruction of Small Intestine, Percutaneous Approach */
"0D584ZZ"  =  "1"  /* Destruction of Small Intestine, Percutaneous Endoscopic Approach */
"0D587ZZ"  =  "1"  /* Destruction of Small Intestine, Via Natural or Artificial Opening */
"0D588ZZ"  =  "1"  /* Destruction of Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0D590ZZ"  =  "1"  /* Destruction of Duodenum, Open Approach */
"0D593ZZ"  =  "1"  /* Destruction of Duodenum, Percutaneous Approach */
"0D597ZZ"  =  "1"  /* Destruction of Duodenum, Via Natural or Artificial Opening */
"0D5A0ZZ"  =  "1"  /* Destruction of Jejunum, Open Approach */
"0D5A3ZZ"  =  "1"  /* Destruction of Jejunum, Percutaneous Approach */
"0D5A4ZZ"  =  "1"  /* Destruction of Jejunum, Percutaneous Endoscopic Approach */
"0D5A7ZZ"  =  "1"  /* Destruction of Jejunum, Via Natural or Artificial Opening */
"0D5A8ZZ"  =  "1"  /* Destruction of Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D5B0ZZ"  =  "1"  /* Destruction of Ileum, Open Approach */
"0D5B3ZZ"  =  "1"  /* Destruction of Ileum, Percutaneous Approach */
"0D5B4ZZ"  =  "1"  /* Destruction of Ileum, Percutaneous Endoscopic Approach */
"0D5B7ZZ"  =  "1"  /* Destruction of Ileum, Via Natural or Artificial Opening */
"0D5B8ZZ"  =  "1"  /* Destruction of Ileum, Via Natural or Artificial Opening Endoscopic */
"0D5C0ZZ"  =  "1"  /* Destruction of Ileocecal Valve, Open Approach */
"0D5C3ZZ"  =  "1"  /* Destruction of Ileocecal Valve, Percutaneous Approach */
"0D5C4ZZ"  =  "1"  /* Destruction of Ileocecal Valve, Percutaneous Endoscopic Approach */
"0D5C7ZZ"  =  "1"  /* Destruction of Ileocecal Valve, Via Natural or Artificial Opening */
"0D5C8ZZ"  =  "1"  /* Destruction of Ileocecal Valve, Via Natural or Artificial Opening Endoscopic */
"0D5E0ZZ"  =  "1"  /* Destruction of Large Intestine, Open Approach */
"0D5E3ZZ"  =  "1"  /* Destruction of Large Intestine, Percutaneous Approach */
"0D5E7ZZ"  =  "1"  /* Destruction of Large Intestine, Via Natural or Artificial Opening */
"0D5F0ZZ"  =  "1"  /* Destruction of Right Large Intestine, Open Approach */
"0D5F3ZZ"  =  "1"  /* Destruction of Right Large Intestine, Percutaneous Approach */
"0D5F7ZZ"  =  "1"  /* Destruction of Right Large Intestine, Via Natural or Artificial Opening */
"0D5G0ZZ"  =  "1"  /* Destruction of Left Large Intestine, Open Approach */
"0D5G3ZZ"  =  "1"  /* Destruction of Left Large Intestine, Percutaneous Approach */
"0D5G7ZZ"  =  "1"  /* Destruction of Left Large Intestine, Via Natural or Artificial Opening */
"0D5H0ZZ"  =  "1"  /* Destruction of Cecum, Open Approach */
"0D5H3ZZ"  =  "1"  /* Destruction of Cecum, Percutaneous Approach */
"0D5H7ZZ"  =  "1"  /* Destruction of Cecum, Via Natural or Artificial Opening */
"0D5J0ZZ"  =  "1"  /* Destruction of Appendix, Open Approach */
"0D5J3ZZ"  =  "1"  /* Destruction of Appendix, Percutaneous Approach */
"0D5J4ZZ"  =  "1"  /* Destruction of Appendix, Percutaneous Endoscopic Approach */
"0D5J7ZZ"  =  "1"  /* Destruction of Appendix, Via Natural or Artificial Opening */
"0D5J8ZZ"  =  "1"  /* Destruction of Appendix, Via Natural or Artificial Opening Endoscopic */
"0D5K0ZZ"  =  "1"  /* Destruction of Ascending Colon, Open Approach */
"0D5K3ZZ"  =  "1"  /* Destruction of Ascending Colon, Percutaneous Approach */
"0D5K7ZZ"  =  "1"  /* Destruction of Ascending Colon, Via Natural or Artificial Opening */
"0D5L0ZZ"  =  "1"  /* Destruction of Transverse Colon, Open Approach */
"0D5L3ZZ"  =  "1"  /* Destruction of Transverse Colon, Percutaneous Approach */
"0D5L7ZZ"  =  "1"  /* Destruction of Transverse Colon, Via Natural or Artificial Opening */
"0D5M0ZZ"  =  "1"  /* Destruction of Descending Colon, Open Approach */
"0D5M3ZZ"  =  "1"  /* Destruction of Descending Colon, Percutaneous Approach */
"0D5M7ZZ"  =  "1"  /* Destruction of Descending Colon, Via Natural or Artificial Opening */
"0D5N0ZZ"  =  "1"  /* Destruction of Sigmoid Colon, Open Approach */
"0D5N3ZZ"  =  "1"  /* Destruction of Sigmoid Colon, Percutaneous Approach */
"0D5N7ZZ"  =  "1"  /* Destruction of Sigmoid Colon, Via Natural or Artificial Opening */
"0D5S0ZZ"  =  "1"  /* Destruction of Greater Omentum, Open Approach */
"0D5S3ZZ"  =  "1"  /* Destruction of Greater Omentum, Percutaneous Approach */
"0D5S4ZZ"  =  "1"  /* Destruction of Greater Omentum, Percutaneous Endoscopic Approach */
"0D5T0ZZ"  =  "1"  /* Destruction of Lesser Omentum, Open Approach */
"0D5T3ZZ"  =  "1"  /* Destruction of Lesser Omentum, Percutaneous Approach */
"0D5T4ZZ"  =  "1"  /* Destruction of Lesser Omentum, Percutaneous Endoscopic Approach */
"0D5V0ZZ"  =  "1"  /* Destruction of Mesentery, Open Approach */
"0D5V3ZZ"  =  "1"  /* Destruction of Mesentery, Percutaneous Approach */
"0D5V4ZZ"  =  "1"  /* Destruction of Mesentery, Percutaneous Endoscopic Approach */
"0D5W0ZZ"  =  "1"  /* Destruction of Peritoneum, Open Approach */
"0D5W3ZZ"  =  "1"  /* Destruction of Peritoneum, Percutaneous Approach */
"0D5W4ZZ"  =  "1"  /* Destruction of Peritoneum, Percutaneous Endoscopic Approach */
"0D760DZ"  =  "1"  /* Dilation of Stomach with Intraluminal Device, Open Approach */
"0D760ZZ"  =  "1"  /* Dilation of Stomach, Open Approach */
"0D763DZ"  =  "1"  /* Dilation of Stomach with Intraluminal Device, Percutaneous Approach */
"0D763ZZ"  =  "1"  /* Dilation of Stomach, Percutaneous Approach */
"0D764DZ"  =  "1"  /* Dilation of Stomach with Intraluminal Device, Percutaneous Endoscopic Approach */
"0D764ZZ"  =  "1"  /* Dilation of Stomach, Percutaneous Endoscopic Approach */
"0D767DZ"  =  "1"  /* Dilation of Stomach with Intraluminal Device, Via Natural or Artificial Opening */
"0D767ZZ"  =  "1"  /* Dilation of Stomach, Via Natural or Artificial Opening */
"0D768DZ"  =  "1"  /* Dilation of Stomach with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0D768ZZ"  =  "1"  /* Dilation of Stomach, Via Natural or Artificial Opening Endoscopic */
"0D770DZ"  =  "1"  /* Dilation of Stomach, Pylorus with Intraluminal Device, Open Approach */
"0D770ZZ"  =  "1"  /* Dilation of Stomach, Pylorus, Open Approach */
"0D773DZ"  =  "1"  /* Dilation of Stomach, Pylorus with Intraluminal Device, Percutaneous Approach */
"0D773ZZ"  =  "1"  /* Dilation of Stomach, Pylorus, Percutaneous Approach */
"0D774ZZ"  =  "1"  /* Dilation of Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0D777DZ"  =  "1"  /* Dilation of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening */
"0D777ZZ"  =  "1"  /* Dilation of Stomach, Pylorus, Via Natural or Artificial Opening */
"0D778ZZ"  =  "1"  /* Dilation of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0D780ZZ"  =  "1"  /* Dilation of Small Intestine, Open Approach */
"0D783ZZ"  =  "1"  /* Dilation of Small Intestine, Percutaneous Approach */
"0D784ZZ"  =  "1"  /* Dilation of Small Intestine, Percutaneous Endoscopic Approach */
"0D790ZZ"  =  "1"  /* Dilation of Duodenum, Open Approach */
"0D793ZZ"  =  "1"  /* Dilation of Duodenum, Percutaneous Approach */
"0D794ZZ"  =  "1"  /* Dilation of Duodenum, Percutaneous Endoscopic Approach */
"0D7A0ZZ"  =  "1"  /* Dilation of Jejunum, Open Approach */
"0D7A3ZZ"  =  "1"  /* Dilation of Jejunum, Percutaneous Approach */
"0D7A4ZZ"  =  "1"  /* Dilation of Jejunum, Percutaneous Endoscopic Approach */
"0D7B0ZZ"  =  "1"  /* Dilation of Ileum, Open Approach */
"0D7B3ZZ"  =  "1"  /* Dilation of Ileum, Percutaneous Approach */
"0D7B4ZZ"  =  "1"  /* Dilation of Ileum, Percutaneous Endoscopic Approach */
"0D7C0ZZ"  =  "1"  /* Dilation of Ileocecal Valve, Open Approach */
"0D7C3ZZ"  =  "1"  /* Dilation of Ileocecal Valve, Percutaneous Approach */
"0D7C4ZZ"  =  "1"  /* Dilation of Ileocecal Valve, Percutaneous Endoscopic Approach */
"0D7E0ZZ"  =  "1"  /* Dilation of Large Intestine, Open Approach */
"0D7E3ZZ"  =  "1"  /* Dilation of Large Intestine, Percutaneous Approach */
"0D7E4ZZ"  =  "1"  /* Dilation of Large Intestine, Percutaneous Endoscopic Approach */
"0D7F0ZZ"  =  "1"  /* Dilation of Right Large Intestine, Open Approach */
"0D7F3ZZ"  =  "1"  /* Dilation of Right Large Intestine, Percutaneous Approach */
"0D7F4ZZ"  =  "1"  /* Dilation of Right Large Intestine, Percutaneous Endoscopic Approach */
"0D7G0ZZ"  =  "1"  /* Dilation of Left Large Intestine, Open Approach */
"0D7G3ZZ"  =  "1"  /* Dilation of Left Large Intestine, Percutaneous Approach */
"0D7G4ZZ"  =  "1"  /* Dilation of Left Large Intestine, Percutaneous Endoscopic Approach */
"0D7H0ZZ"  =  "1"  /* Dilation of Cecum, Open Approach */
"0D7H3ZZ"  =  "1"  /* Dilation of Cecum, Percutaneous Approach */
"0D7H4ZZ"  =  "1"  /* Dilation of Cecum, Percutaneous Endoscopic Approach */
"0D7K0ZZ"  =  "1"  /* Dilation of Ascending Colon, Open Approach */
"0D7K3ZZ"  =  "1"  /* Dilation of Ascending Colon, Percutaneous Approach */
"0D7K4ZZ"  =  "1"  /* Dilation of Ascending Colon, Percutaneous Endoscopic Approach */
"0D7L0ZZ"  =  "1"  /* Dilation of Transverse Colon, Open Approach */
"0D7L3ZZ"  =  "1"  /* Dilation of Transverse Colon, Percutaneous Approach */
"0D7L4ZZ"  =  "1"  /* Dilation of Transverse Colon, Percutaneous Endoscopic Approach */
"0D7M0ZZ"  =  "1"  /* Dilation of Descending Colon, Open Approach */
"0D7M3ZZ"  =  "1"  /* Dilation of Descending Colon, Percutaneous Approach */
"0D7M4ZZ"  =  "1"  /* Dilation of Descending Colon, Percutaneous Endoscopic Approach */
"0D7N0ZZ"  =  "1"  /* Dilation of Sigmoid Colon, Open Approach */
"0D7N3ZZ"  =  "1"  /* Dilation of Sigmoid Colon, Percutaneous Approach */
"0D7N4ZZ"  =  "1"  /* Dilation of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D7P0DZ"  =  "1"  /* Dilation of Rectum with Intraluminal Device, Open Approach */
"0D7P0ZZ"  =  "1"  /* Dilation of Rectum, Open Approach */
"0D7P3DZ"  =  "1"  /* Dilation of Rectum with Intraluminal Device, Percutaneous Approach */
"0D7P3ZZ"  =  "1"  /* Dilation of Rectum, Percutaneous Approach */
"0D7P4DZ"  =  "1"  /* Dilation of Rectum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0D7P4ZZ"  =  "1"  /* Dilation of Rectum, Percutaneous Endoscopic Approach */
"0D870ZZ"  =  "1"  /* Division of Stomach, Pylorus, Open Approach */
"0D873ZZ"  =  "1"  /* Division of Stomach, Pylorus, Percutaneous Approach */
"0D874ZZ"  =  "1"  /* Division of Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0D877ZZ"  =  "1"  /* Division of Stomach, Pylorus, Via Natural or Artificial Opening */
"0D878ZZ"  =  "1"  /* Division of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0D9600Z"  =  "1"  /* Drainage of Stomach with Drainage Device, Open Approach */
"0D960ZX"  =  "1"  /* Drainage of Stomach, Open Approach, Diagnostic */
"0D960ZZ"  =  "1"  /* Drainage of Stomach, Open Approach */
"0D9630Z"  =  "1"  /* Drainage of Stomach with Drainage Device, Percutaneous Approach */
"0D963ZZ"  =  "1"  /* Drainage of Stomach, Percutaneous Approach */
"0D9640Z"  =  "1"  /* Drainage of Stomach with Drainage Device, Percutaneous Endoscopic Approach */
"0D964ZZ"  =  "1"  /* Drainage of Stomach, Percutaneous Endoscopic Approach */
"0D967ZZ"  =  "1"  /* Drainage of Stomach, Via Natural or Artificial Opening */
"0D968ZZ"  =  "1"  /* Drainage of Stomach, Via Natural or Artificial Opening Endoscopic */
"0D9700Z"  =  "1"  /* Drainage of Stomach, Pylorus with Drainage Device, Open Approach */
"0D970ZX"  =  "1"  /* Drainage of Stomach, Pylorus, Open Approach, Diagnostic */
"0D970ZZ"  =  "1"  /* Drainage of Stomach, Pylorus, Open Approach */
"0D9730Z"  =  "1"  /* Drainage of Stomach, Pylorus with Drainage Device, Percutaneous Approach */
"0D973ZZ"  =  "1"  /* Drainage of Stomach, Pylorus, Percutaneous Approach */
"0D9740Z"  =  "1"  /* Drainage of Stomach, Pylorus with Drainage Device, Percutaneous Endoscopic Approach */
"0D974ZZ"  =  "1"  /* Drainage of Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0D977ZZ"  =  "1"  /* Drainage of Stomach, Pylorus, Via Natural or Artificial Opening */
"0D978ZZ"  =  "1"  /* Drainage of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0D9800Z"  =  "1"  /* Drainage of Small Intestine with Drainage Device, Open Approach */
"0D980ZZ"  =  "1"  /* Drainage of Small Intestine, Open Approach */
"0D9830Z"  =  "1"  /* Drainage of Small Intestine with Drainage Device, Percutaneous Approach */
"0D983ZZ"  =  "1"  /* Drainage of Small Intestine, Percutaneous Approach */
"0D9840Z"  =  "1"  /* Drainage of Small Intestine with Drainage Device, Percutaneous Endoscopic Approach */
"0D984ZZ"  =  "1"  /* Drainage of Small Intestine, Percutaneous Endoscopic Approach */
"0D9870Z"  =  "1"  /* Drainage of Small Intestine with Drainage Device, Via Natural or Artificial Opening */
"0D987ZZ"  =  "1"  /* Drainage of Small Intestine, Via Natural or Artificial Opening */
"0D9880Z"  =  "1"  /* Drainage of Small Intestine with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D988ZZ"  =  "1"  /* Drainage of Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0D9900Z"  =  "1"  /* Drainage of Duodenum with Drainage Device, Open Approach */
"0D990ZZ"  =  "1"  /* Drainage of Duodenum, Open Approach */
"0D9930Z"  =  "1"  /* Drainage of Duodenum with Drainage Device, Percutaneous Approach */
"0D993ZZ"  =  "1"  /* Drainage of Duodenum, Percutaneous Approach */
"0D9940Z"  =  "1"  /* Drainage of Duodenum with Drainage Device, Percutaneous Endoscopic Approach */
"0D994ZZ"  =  "1"  /* Drainage of Duodenum, Percutaneous Endoscopic Approach */
"0D997ZZ"  =  "1"  /* Drainage of Duodenum, Via Natural or Artificial Opening */
"0D998ZZ"  =  "1"  /* Drainage of Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D9A00Z"  =  "1"  /* Drainage of Jejunum with Drainage Device, Open Approach */
"0D9A0ZZ"  =  "1"  /* Drainage of Jejunum, Open Approach */
"0D9A30Z"  =  "1"  /* Drainage of Jejunum with Drainage Device, Percutaneous Approach */
"0D9A3ZZ"  =  "1"  /* Drainage of Jejunum, Percutaneous Approach */
"0D9A40Z"  =  "1"  /* Drainage of Jejunum with Drainage Device, Percutaneous Endoscopic Approach */
"0D9A4ZZ"  =  "1"  /* Drainage of Jejunum, Percutaneous Endoscopic Approach */
"0D9A7ZZ"  =  "1"  /* Drainage of Jejunum, Via Natural or Artificial Opening */
"0D9A8ZZ"  =  "1"  /* Drainage of Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D9B00Z"  =  "1"  /* Drainage of Ileum with Drainage Device, Open Approach */
"0D9B0ZZ"  =  "1"  /* Drainage of Ileum, Open Approach */
"0D9B30Z"  =  "1"  /* Drainage of Ileum with Drainage Device, Percutaneous Approach */
"0D9B3ZZ"  =  "1"  /* Drainage of Ileum, Percutaneous Approach */
"0D9B40Z"  =  "1"  /* Drainage of Ileum with Drainage Device, Percutaneous Endoscopic Approach */
"0D9B4ZZ"  =  "1"  /* Drainage of Ileum, Percutaneous Endoscopic Approach */
"0D9B7ZZ"  =  "1"  /* Drainage of Ileum, Via Natural or Artificial Opening */
"0D9B8ZZ"  =  "1"  /* Drainage of Ileum, Via Natural or Artificial Opening Endoscopic */
"0D9C00Z"  =  "1"  /* Drainage of Ileocecal Valve with Drainage Device, Open Approach */
"0D9C0ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Open Approach */
"0D9C30Z"  =  "1"  /* Drainage of Ileocecal Valve with Drainage Device, Percutaneous Approach */
"0D9C3ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Percutaneous Approach */
"0D9C40Z"  =  "1"  /* Drainage of Ileocecal Valve with Drainage Device, Percutaneous Endoscopic Approach */
"0D9C4ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Percutaneous Endoscopic Approach */
"0D9C70Z"  =  "1"  /* Drainage of Ileocecal Valve with Drainage Device, Via Natural or Artificial Opening */
"0D9C7ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Via Natural or Artificial Opening */
"0D9C80Z"  =  "1"  /* Drainage of Ileocecal Valve with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D9C8ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Via Natural or Artificial Opening Endoscopic */
"0D9E00Z"  =  "1"  /* Drainage of Large Intestine with Drainage Device, Open Approach */
"0D9E0ZZ"  =  "1"  /* Drainage of Large Intestine, Open Approach */
"0D9E30Z"  =  "1"  /* Drainage of Large Intestine with Drainage Device, Percutaneous Approach */
"0D9E3ZZ"  =  "1"  /* Drainage of Large Intestine, Percutaneous Approach */
"0D9E40Z"  =  "1"  /* Drainage of Large Intestine with Drainage Device, Percutaneous Endoscopic Approach */
"0D9E4ZZ"  =  "1"  /* Drainage of Large Intestine, Percutaneous Endoscopic Approach */
"0D9E70Z"  =  "1"  /* Drainage of Large Intestine with Drainage Device, Via Natural or Artificial Opening */
"0D9E7ZZ"  =  "1"  /* Drainage of Large Intestine, Via Natural or Artificial Opening */
"0D9E80Z"  =  "1"  /* Drainage of Large Intestine with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D9E8ZZ"  =  "1"  /* Drainage of Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0D9F00Z"  =  "1"  /* Drainage of Right Large Intestine with Drainage Device, Open Approach */
"0D9F0ZZ"  =  "1"  /* Drainage of Right Large Intestine, Open Approach */
"0D9F30Z"  =  "1"  /* Drainage of Right Large Intestine with Drainage Device, Percutaneous Approach */
"0D9F3ZZ"  =  "1"  /* Drainage of Right Large Intestine, Percutaneous Approach */
"0D9F40Z"  =  "1"  /* Drainage of Right Large Intestine with Drainage Device, Percutaneous Endoscopic Approach */
"0D9F4ZZ"  =  "1"  /* Drainage of Right Large Intestine, Percutaneous Endoscopic Approach */
"0D9F7ZZ"  =  "1"  /* Drainage of Right Large Intestine, Via Natural or Artificial Opening */
"0D9F8ZZ"  =  "1"  /* Drainage of Right Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0D9G00Z"  =  "1"  /* Drainage of Left Large Intestine with Drainage Device, Open Approach */
"0D9G0ZZ"  =  "1"  /* Drainage of Left Large Intestine, Open Approach */
"0D9G30Z"  =  "1"  /* Drainage of Left Large Intestine with Drainage Device, Percutaneous Approach */
"0D9G3ZZ"  =  "1"  /* Drainage of Left Large Intestine, Percutaneous Approach */
"0D9G40Z"  =  "1"  /* Drainage of Left Large Intestine with Drainage Device, Percutaneous Endoscopic Approach */
"0D9G4ZZ"  =  "1"  /* Drainage of Left Large Intestine, Percutaneous Endoscopic Approach */
"0D9G7ZZ"  =  "1"  /* Drainage of Left Large Intestine, Via Natural or Artificial Opening */
"0D9G8ZZ"  =  "1"  /* Drainage of Left Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0D9H00Z"  =  "1"  /* Drainage of Cecum with Drainage Device, Open Approach */
"0D9H0ZZ"  =  "1"  /* Drainage of Cecum, Open Approach */
"0D9H30Z"  =  "1"  /* Drainage of Cecum with Drainage Device, Percutaneous Approach */
"0D9H3ZZ"  =  "1"  /* Drainage of Cecum, Percutaneous Approach */
"0D9H40Z"  =  "1"  /* Drainage of Cecum with Drainage Device, Percutaneous Endoscopic Approach */
"0D9H4ZZ"  =  "1"  /* Drainage of Cecum, Percutaneous Endoscopic Approach */
"0D9H7ZZ"  =  "1"  /* Drainage of Cecum, Via Natural or Artificial Opening */
"0D9H8ZZ"  =  "1"  /* Drainage of Cecum, Via Natural or Artificial Opening Endoscopic */
"0D9J00Z"  =  "1"  /* Drainage of Appendix with Drainage Device, Open Approach */
"0D9J0ZX"  =  "1"  /* Drainage of Appendix, Open Approach, Diagnostic */
"0D9J0ZZ"  =  "1"  /* Drainage of Appendix, Open Approach */
"0D9J30Z"  =  "1"  /* Drainage of Appendix with Drainage Device, Percutaneous Approach */
"0D9J3ZX"  =  "1"  /* Drainage of Appendix, Percutaneous Approach, Diagnostic */
"0D9J3ZZ"  =  "1"  /* Drainage of Appendix, Percutaneous Approach */
"0D9J40Z"  =  "1"  /* Drainage of Appendix with Drainage Device, Percutaneous Endoscopic Approach */
"0D9J4ZX"  =  "1"  /* Drainage of Appendix, Percutaneous Endoscopic Approach, Diagnostic */
"0D9J4ZZ"  =  "1"  /* Drainage of Appendix, Percutaneous Endoscopic Approach */
"0D9J70Z"  =  "1"  /* Drainage of Appendix with Drainage Device, Via Natural or Artificial Opening */
"0D9J7ZX"  =  "1"  /* Drainage of Appendix, Via Natural or Artificial Opening, Diagnostic */
"0D9J7ZZ"  =  "1"  /* Drainage of Appendix, Via Natural or Artificial Opening */
"0D9J80Z"  =  "1"  /* Drainage of Appendix with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0D9J8ZX"  =  "1"  /* Drainage of Appendix, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0D9J8ZZ"  =  "1"  /* Drainage of Appendix, Via Natural or Artificial Opening Endoscopic */
"0D9K00Z"  =  "1"  /* Drainage of Ascending Colon with Drainage Device, Open Approach */
"0D9K0ZZ"  =  "1"  /* Drainage of Ascending Colon, Open Approach */
"0D9K30Z"  =  "1"  /* Drainage of Ascending Colon with Drainage Device, Percutaneous Approach */
"0D9K3ZZ"  =  "1"  /* Drainage of Ascending Colon, Percutaneous Approach */
"0D9K40Z"  =  "1"  /* Drainage of Ascending Colon with Drainage Device, Percutaneous Endoscopic Approach */
"0D9K4ZZ"  =  "1"  /* Drainage of Ascending Colon, Percutaneous Endoscopic Approach */
"0D9K7ZZ"  =  "1"  /* Drainage of Ascending Colon, Via Natural or Artificial Opening */
"0D9K8ZZ"  =  "1"  /* Drainage of Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0D9L00Z"  =  "1"  /* Drainage of Transverse Colon with Drainage Device, Open Approach */
"0D9L0ZZ"  =  "1"  /* Drainage of Transverse Colon, Open Approach */
"0D9L30Z"  =  "1"  /* Drainage of Transverse Colon with Drainage Device, Percutaneous Approach */
"0D9L3ZZ"  =  "1"  /* Drainage of Transverse Colon, Percutaneous Approach */
"0D9L40Z"  =  "1"  /* Drainage of Transverse Colon with Drainage Device, Percutaneous Endoscopic Approach */
"0D9L4ZZ"  =  "1"  /* Drainage of Transverse Colon, Percutaneous Endoscopic Approach */
"0D9L7ZZ"  =  "1"  /* Drainage of Transverse Colon, Via Natural or Artificial Opening */
"0D9L8ZZ"  =  "1"  /* Drainage of Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0D9M00Z"  =  "1"  /* Drainage of Descending Colon with Drainage Device, Open Approach */
"0D9M0ZZ"  =  "1"  /* Drainage of Descending Colon, Open Approach */
"0D9M30Z"  =  "1"  /* Drainage of Descending Colon with Drainage Device, Percutaneous Approach */
"0D9M3ZZ"  =  "1"  /* Drainage of Descending Colon, Percutaneous Approach */
"0D9M40Z"  =  "1"  /* Drainage of Descending Colon with Drainage Device, Percutaneous Endoscopic Approach */
"0D9M4ZZ"  =  "1"  /* Drainage of Descending Colon, Percutaneous Endoscopic Approach */
"0D9M7ZZ"  =  "1"  /* Drainage of Descending Colon, Via Natural or Artificial Opening */
"0D9M8ZZ"  =  "1"  /* Drainage of Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0D9N00Z"  =  "1"  /* Drainage of Sigmoid Colon with Drainage Device, Open Approach */
"0D9N0ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Open Approach */
"0D9N30Z"  =  "1"  /* Drainage of Sigmoid Colon with Drainage Device, Percutaneous Approach */
"0D9N3ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Percutaneous Approach */
"0D9N40Z"  =  "1"  /* Drainage of Sigmoid Colon with Drainage Device, Percutaneous Endoscopic Approach */
"0D9N4ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D9N7ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Via Natural or Artificial Opening */
"0D9N8ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0D9S00Z"  =  "1"  /* Drainage of Greater Omentum with Drainage Device, Open Approach */
"0D9S0ZX"  =  "1"  /* Drainage of Greater Omentum, Open Approach, Diagnostic */
"0D9S0ZZ"  =  "1"  /* Drainage of Greater Omentum, Open Approach */
"0D9S3ZX"  =  "1"  /* Drainage of Greater Omentum, Percutaneous Approach, Diagnostic */
"0D9S4ZX"  =  "1"  /* Drainage of Greater Omentum, Percutaneous Endoscopic Approach, Diagnostic */
"0D9T00Z"  =  "1"  /* Drainage of Lesser Omentum with Drainage Device, Open Approach */
"0D9T0ZX"  =  "1"  /* Drainage of Lesser Omentum, Open Approach, Diagnostic */
"0D9T0ZZ"  =  "1"  /* Drainage of Lesser Omentum, Open Approach */
"0D9T3ZX"  =  "1"  /* Drainage of Lesser Omentum, Percutaneous Approach, Diagnostic */
"0D9T4ZX"  =  "1"  /* Drainage of Lesser Omentum, Percutaneous Endoscopic Approach, Diagnostic */
"0D9V00Z"  =  "1"  /* Drainage of Mesentery with Drainage Device, Open Approach */
"0D9V0ZX"  =  "1"  /* Drainage of Mesentery, Open Approach, Diagnostic */
"0D9V0ZZ"  =  "1"  /* Drainage of Mesentery, Open Approach */
"0D9V3ZX"  =  "1"  /* Drainage of Mesentery, Percutaneous Approach, Diagnostic */
"0D9V4ZX"  =  "1"  /* Drainage of Mesentery, Percutaneous Endoscopic Approach, Diagnostic */
"0D9W00Z"  =  "1"  /* Drainage of Peritoneum with Drainage Device, Open Approach */
"0D9W0ZX"  =  "1"  /* Drainage of Peritoneum, Open Approach, Diagnostic */
"0D9W0ZZ"  =  "1"  /* Drainage of Peritoneum, Open Approach */
"0D9W3ZX"  =  "1"  /* Drainage of Peritoneum, Percutaneous Approach, Diagnostic */
"0D9W4ZX"  =  "1"  /* Drainage of Peritoneum, Percutaneous Endoscopic Approach, Diagnostic */
"0DB10ZZ"  =  "1"  /* Excision of Upper Esophagus, Open Approach */
"0DB13ZZ"  =  "1"  /* Excision of Upper Esophagus, Percutaneous Approach */
"0DB17ZZ"  =  "1"  /* Excision of Upper Esophagus, Via Natural or Artificial Opening */
"0DB20ZZ"  =  "1"  /* Excision of Middle Esophagus, Open Approach */
"0DB23ZZ"  =  "1"  /* Excision of Middle Esophagus, Percutaneous Approach */
"0DB27ZZ"  =  "1"  /* Excision of Middle Esophagus, Via Natural or Artificial Opening */
"0DB30ZZ"  =  "1"  /* Excision of Lower Esophagus, Open Approach */
"0DB33ZZ"  =  "1"  /* Excision of Lower Esophagus, Percutaneous Approach */
"0DB37ZZ"  =  "1"  /* Excision of Lower Esophagus, Via Natural or Artificial Opening */
"0DB40ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Open Approach */
"0DB43ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Percutaneous Approach */
"0DB44ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0DB47ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Via Natural or Artificial Opening */
"0DB50ZZ"  =  "1"  /* Excision of Esophagus, Open Approach */
"0DB53ZZ"  =  "1"  /* Excision of Esophagus, Percutaneous Approach */
"0DB57ZZ"  =  "1"  /* Excision of Esophagus, Via Natural or Artificial Opening */
"0DB60Z3"  =  "1"  /* Excision of Stomach, Open Approach, Vertical */
"0DB60ZX"  =  "1"  /* Excision of Stomach, Open Approach, Diagnostic */
"0DB60ZZ"  =  "1"  /* Excision of Stomach, Open Approach */
"0DB63Z3"  =  "1"  /* Excision of Stomach, Percutaneous Approach, Vertical */
"0DB63ZZ"  =  "1"  /* Excision of Stomach, Percutaneous Approach */
"0DB64Z3"  =  "1"  /* Excision of Stomach, Percutaneous Endoscopic Approach, Vertical */
"0DB64ZZ"  =  "1"  /* Excision of Stomach, Percutaneous Endoscopic Approach */
"0DB67Z3"  =  "1"  /* Excision of Stomach, Via Natural or Artificial Opening, Vertical */
"0DB67ZZ"  =  "1"  /* Excision of Stomach, Via Natural or Artificial Opening */
"0DB68Z3"  =  "1"  /* Excision of Stomach, Via Natural or Artificial Opening Endoscopic, Vertical */
"0DB68ZZ"  =  "1"  /* Excision of Stomach, Via Natural or Artificial Opening Endoscopic */
"0DB70ZX"  =  "1"  /* Excision of Stomach, Pylorus, Open Approach, Diagnostic */
"0DB70ZZ"  =  "1"  /* Excision of Stomach, Pylorus, Open Approach */
"0DB73ZZ"  =  "1"  /* Excision of Stomach, Pylorus, Percutaneous Approach */
"0DB77ZZ"  =  "1"  /* Excision of Stomach, Pylorus, Via Natural or Artificial Opening */
"0DB80ZZ"  =  "1"  /* Excision of Small Intestine, Open Approach */
"0DB83ZZ"  =  "1"  /* Excision of Small Intestine, Percutaneous Approach */
"0DB84ZZ"  =  "1"  /* Excision of Small Intestine, Percutaneous Endoscopic Approach */
"0DB87ZZ"  =  "1"  /* Excision of Small Intestine, Via Natural or Artificial Opening */
"0DB88ZZ"  =  "1"  /* Excision of Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DB90ZZ"  =  "1"  /* Excision of Duodenum, Open Approach */
"0DB93ZZ"  =  "1"  /* Excision of Duodenum, Percutaneous Approach */
"0DB94ZZ"  =  "1"  /* Excision of Duodenum, Percutaneous Endoscopic Approach */
"0DB97ZZ"  =  "1"  /* Excision of Duodenum, Via Natural or Artificial Opening */
"0DB98ZZ"  =  "1"  /* Excision of Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DBA0ZZ"  =  "1"  /* Excision of Jejunum, Open Approach */
"0DBA3ZZ"  =  "1"  /* Excision of Jejunum, Percutaneous Approach */
"0DBA4ZZ"  =  "1"  /* Excision of Jejunum, Percutaneous Endoscopic Approach */
"0DBA7ZZ"  =  "1"  /* Excision of Jejunum, Via Natural or Artificial Opening */
"0DBA8ZZ"  =  "1"  /* Excision of Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DBB0ZZ"  =  "1"  /* Excision of Ileum, Open Approach */
"0DBB3ZZ"  =  "1"  /* Excision of Ileum, Percutaneous Approach */
"0DBB4ZZ"  =  "1"  /* Excision of Ileum, Percutaneous Endoscopic Approach */
"0DBB7ZZ"  =  "1"  /* Excision of Ileum, Via Natural or Artificial Opening */
"0DBB8ZZ"  =  "1"  /* Excision of Ileum, Via Natural or Artificial Opening Endoscopic */
"0DBC0ZZ"  =  "1"  /* Excision of Ileocecal Valve, Open Approach */
"0DBC3ZZ"  =  "1"  /* Excision of Ileocecal Valve, Percutaneous Approach */
"0DBC4ZZ"  =  "1"  /* Excision of Ileocecal Valve, Percutaneous Endoscopic Approach */
"0DBC7ZZ"  =  "1"  /* Excision of Ileocecal Valve, Via Natural or Artificial Opening */
"0DBC8ZZ"  =  "1"  /* Excision of Ileocecal Valve, Via Natural or Artificial Opening Endoscopic */
"0DBE0ZZ"  =  "1"  /* Excision of Large Intestine, Open Approach */
"0DBE3ZZ"  =  "1"  /* Excision of Large Intestine, Percutaneous Approach */
"0DBE4ZZ"  =  "1"  /* Excision of Large Intestine, Percutaneous Endoscopic Approach */
"0DBE7ZZ"  =  "1"  /* Excision of Large Intestine, Via Natural or Artificial Opening */
"0DBE8ZZ"  =  "1"  /* Excision of Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DBF0ZZ"  =  "1"  /* Excision of Right Large Intestine, Open Approach */
"0DBF3ZZ"  =  "1"  /* Excision of Right Large Intestine, Percutaneous Approach */
"0DBF4ZZ"  =  "1"  /* Excision of Right Large Intestine, Percutaneous Endoscopic Approach */
"0DBF7ZZ"  =  "1"  /* Excision of Right Large Intestine, Via Natural or Artificial Opening */
"0DBF8ZZ"  =  "1"  /* Excision of Right Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DBG0ZZ"  =  "1"  /* Excision of Left Large Intestine, Open Approach */
"0DBG3ZZ"  =  "1"  /* Excision of Left Large Intestine, Percutaneous Approach */
"0DBG4ZZ"  =  "1"  /* Excision of Left Large Intestine, Percutaneous Endoscopic Approach */
"0DBG7ZZ"  =  "1"  /* Excision of Left Large Intestine, Via Natural or Artificial Opening */
"0DBG8ZZ"  =  "1"  /* Excision of Left Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DBH0ZZ"  =  "1"  /* Excision of Cecum, Open Approach */
"0DBH3ZZ"  =  "1"  /* Excision of Cecum, Percutaneous Approach */
"0DBH4ZZ"  =  "1"  /* Excision of Cecum, Percutaneous Endoscopic Approach */
"0DBH7ZZ"  =  "1"  /* Excision of Cecum, Via Natural or Artificial Opening */
"0DBH8ZZ"  =  "1"  /* Excision of Cecum, Via Natural or Artificial Opening Endoscopic */
"0DBJ0ZX"  =  "1"  /* Excision of Appendix, Open Approach, Diagnostic */
"0DBJ0ZZ"  =  "1"  /* Excision of Appendix, Open Approach */
"0DBJ3ZX"  =  "1"  /* Excision of Appendix, Percutaneous Approach, Diagnostic */
"0DBJ3ZZ"  =  "1"  /* Excision of Appendix, Percutaneous Approach */
"0DBJ4ZX"  =  "1"  /* Excision of Appendix, Percutaneous Endoscopic Approach, Diagnostic */
"0DBJ4ZZ"  =  "1"  /* Excision of Appendix, Percutaneous Endoscopic Approach */
"0DBJ7ZX"  =  "1"  /* Excision of Appendix, Via Natural or Artificial Opening, Diagnostic */
"0DBJ7ZZ"  =  "1"  /* Excision of Appendix, Via Natural or Artificial Opening */
"0DBJ8ZX"  =  "1"  /* Excision of Appendix, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0DBJ8ZZ"  =  "1"  /* Excision of Appendix, Via Natural or Artificial Opening Endoscopic */
"0DBK0ZZ"  =  "1"  /* Excision of Ascending Colon, Open Approach */
"0DBK3ZZ"  =  "1"  /* Excision of Ascending Colon, Percutaneous Approach */
"0DBK4ZZ"  =  "1"  /* Excision of Ascending Colon, Percutaneous Endoscopic Approach */
"0DBK7ZZ"  =  "1"  /* Excision of Ascending Colon, Via Natural or Artificial Opening */
"0DBK8ZZ"  =  "1"  /* Excision of Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0DBL0ZZ"  =  "1"  /* Excision of Transverse Colon, Open Approach */
"0DBL3ZZ"  =  "1"  /* Excision of Transverse Colon, Percutaneous Approach */
"0DBL4ZZ"  =  "1"  /* Excision of Transverse Colon, Percutaneous Endoscopic Approach */
"0DBL7ZZ"  =  "1"  /* Excision of Transverse Colon, Via Natural or Artificial Opening */
"0DBL8ZZ"  =  "1"  /* Excision of Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0DBM0ZZ"  =  "1"  /* Excision of Descending Colon, Open Approach */
"0DBM3ZZ"  =  "1"  /* Excision of Descending Colon, Percutaneous Approach */
"0DBM4ZZ"  =  "1"  /* Excision of Descending Colon, Percutaneous Endoscopic Approach */
"0DBM7ZZ"  =  "1"  /* Excision of Descending Colon, Via Natural or Artificial Opening */
"0DBM8ZZ"  =  "1"  /* Excision of Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0DBN0ZZ"  =  "1"  /* Excision of Sigmoid Colon, Open Approach */
"0DBN3ZZ"  =  "1"  /* Excision of Sigmoid Colon, Percutaneous Approach */
"0DBN4ZZ"  =  "1"  /* Excision of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DBN7ZZ"  =  "1"  /* Excision of Sigmoid Colon, Via Natural or Artificial Opening */
"0DBN8ZZ"  =  "1"  /* Excision of Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0DBS0ZX"  =  "1"  /* Excision of Greater Omentum, Open Approach, Diagnostic */
"0DBS0ZZ"  =  "1"  /* Excision of Greater Omentum, Open Approach */
"0DBS3ZZ"  =  "1"  /* Excision of Greater Omentum, Percutaneous Approach */
"0DBS4ZZ"  =  "1"  /* Excision of Greater Omentum, Percutaneous Endoscopic Approach */
"0DBT0ZX"  =  "1"  /* Excision of Lesser Omentum, Open Approach, Diagnostic */
"0DBT0ZZ"  =  "1"  /* Excision of Lesser Omentum, Open Approach */
"0DBT3ZZ"  =  "1"  /* Excision of Lesser Omentum, Percutaneous Approach */
"0DBT4ZZ"  =  "1"  /* Excision of Lesser Omentum, Percutaneous Endoscopic Approach */
"0DBV0ZX"  =  "1"  /* Excision of Mesentery, Open Approach, Diagnostic */
"0DBV0ZZ"  =  "1"  /* Excision of Mesentery, Open Approach */
"0DBV3ZZ"  =  "1"  /* Excision of Mesentery, Percutaneous Approach */
"0DBV4ZZ"  =  "1"  /* Excision of Mesentery, Percutaneous Endoscopic Approach */
"0DBW0ZX"  =  "1"  /* Excision of Peritoneum, Open Approach, Diagnostic */
"0DBW0ZZ"  =  "1"  /* Excision of Peritoneum, Open Approach */
"0DBW3ZZ"  =  "1"  /* Excision of Peritoneum, Percutaneous Approach */
"0DBW4ZZ"  =  "1"  /* Excision of Peritoneum, Percutaneous Endoscopic Approach */
"0DC60ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Open Approach */
"0DC63ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Percutaneous Approach */
"0DC64ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Percutaneous Endoscopic Approach */
"0DC70ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Pylorus, Open Approach */
"0DC73ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Pylorus, Percutaneous Approach */
"0DC74ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0DC80ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Open Approach */
"0DC83ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Percutaneous Approach */
"0DC84ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Percutaneous Endoscopic Approach */
"0DC87ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Via Natural or Artificial Opening */
"0DC88ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DC90ZZ"  =  "1"  /* Extirpation of Matter from Duodenum, Open Approach */
"0DC93ZZ"  =  "1"  /* Extirpation of Matter from Duodenum, Percutaneous Approach */
"0DC94ZZ"  =  "1"  /* Extirpation of Matter from Duodenum, Percutaneous Endoscopic Approach */
"0DCA0ZZ"  =  "1"  /* Extirpation of Matter from Jejunum, Open Approach */
"0DCA3ZZ"  =  "1"  /* Extirpation of Matter from Jejunum, Percutaneous Approach */
"0DCA4ZZ"  =  "1"  /* Extirpation of Matter from Jejunum, Percutaneous Endoscopic Approach */
"0DCB0ZZ"  =  "1"  /* Extirpation of Matter from Ileum, Open Approach */
"0DCB3ZZ"  =  "1"  /* Extirpation of Matter from Ileum, Percutaneous Approach */
"0DCB4ZZ"  =  "1"  /* Extirpation of Matter from Ileum, Percutaneous Endoscopic Approach */
"0DCC0ZZ"  =  "1"  /* Extirpation of Matter from Ileocecal Valve, Open Approach */
"0DCC3ZZ"  =  "1"  /* Extirpation of Matter from Ileocecal Valve, Percutaneous Approach */
"0DCC4ZZ"  =  "1"  /* Extirpation of Matter from Ileocecal Valve, Percutaneous Endoscopic Approach */
"0DCE0ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Open Approach */
"0DCE3ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Percutaneous Approach */
"0DCE4ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Percutaneous Endoscopic Approach */
"0DCE7ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Via Natural or Artificial Opening */
"0DCE8ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DCF0ZZ"  =  "1"  /* Extirpation of Matter from Right Large Intestine, Open Approach */
"0DCF3ZZ"  =  "1"  /* Extirpation of Matter from Right Large Intestine, Percutaneous Approach */
"0DCF4ZZ"  =  "1"  /* Extirpation of Matter from Right Large Intestine, Percutaneous Endoscopic Approach */
"0DCG0ZZ"  =  "1"  /* Extirpation of Matter from Left Large Intestine, Open Approach */
"0DCG3ZZ"  =  "1"  /* Extirpation of Matter from Left Large Intestine, Percutaneous Approach */
"0DCG4ZZ"  =  "1"  /* Extirpation of Matter from Left Large Intestine, Percutaneous Endoscopic Approach */
"0DCH0ZZ"  =  "1"  /* Extirpation of Matter from Cecum, Open Approach */
"0DCH3ZZ"  =  "1"  /* Extirpation of Matter from Cecum, Percutaneous Approach */
"0DCH4ZZ"  =  "1"  /* Extirpation of Matter from Cecum, Percutaneous Endoscopic Approach */
"0DCJ0ZZ"  =  "1"  /* Extirpation of Matter from Appendix, Open Approach */
"0DCJ3ZZ"  =  "1"  /* Extirpation of Matter from Appendix, Percutaneous Approach */
"0DCJ4ZZ"  =  "1"  /* Extirpation of Matter from Appendix, Percutaneous Endoscopic Approach */
"0DCJ7ZZ"  =  "1"  /* Extirpation of Matter from Appendix, Via Natural or Artificial Opening */
"0DCJ8ZZ"  =  "1"  /* Extirpation of Matter from Appendix, Via Natural or Artificial Opening Endoscopic */
"0DCK0ZZ"  =  "1"  /* Extirpation of Matter from Ascending Colon, Open Approach */
"0DCK3ZZ"  =  "1"  /* Extirpation of Matter from Ascending Colon, Percutaneous Approach */
"0DCK4ZZ"  =  "1"  /* Extirpation of Matter from Ascending Colon, Percutaneous Endoscopic Approach */
"0DCL0ZZ"  =  "1"  /* Extirpation of Matter from Transverse Colon, Open Approach */
"0DCL3ZZ"  =  "1"  /* Extirpation of Matter from Transverse Colon, Percutaneous Approach */
"0DCL4ZZ"  =  "1"  /* Extirpation of Matter from Transverse Colon, Percutaneous Endoscopic Approach */
"0DCM0ZZ"  =  "1"  /* Extirpation of Matter from Descending Colon, Open Approach */
"0DCM3ZZ"  =  "1"  /* Extirpation of Matter from Descending Colon, Percutaneous Approach */
"0DCM4ZZ"  =  "1"  /* Extirpation of Matter from Descending Colon, Percutaneous Endoscopic Approach */
"0DCN0ZZ"  =  "1"  /* Extirpation of Matter from Sigmoid Colon, Open Approach */
"0DCN3ZZ"  =  "1"  /* Extirpation of Matter from Sigmoid Colon, Percutaneous Approach */
"0DCN4ZZ"  =  "1"  /* Extirpation of Matter from Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DCS0ZZ"  =  "1"  /* Extirpation of Matter from Greater Omentum, Open Approach */
"0DCS3ZZ"  =  "1"  /* Extirpation of Matter from Greater Omentum, Percutaneous Approach */
"0DCS4ZZ"  =  "1"  /* Extirpation of Matter from Greater Omentum, Percutaneous Endoscopic Approach */
"0DCT0ZZ"  =  "1"  /* Extirpation of Matter from Lesser Omentum, Open Approach */
"0DCT3ZZ"  =  "1"  /* Extirpation of Matter from Lesser Omentum, Percutaneous Approach */
"0DCT4ZZ"  =  "1"  /* Extirpation of Matter from Lesser Omentum, Percutaneous Endoscopic Approach */
"0DCV0ZZ"  =  "1"  /* Extirpation of Matter from Mesentery, Open Approach */
"0DCV3ZZ"  =  "1"  /* Extirpation of Matter from Mesentery, Percutaneous Approach */
"0DCV4ZZ"  =  "1"  /* Extirpation of Matter from Mesentery, Percutaneous Endoscopic Approach */
"0DCW0ZZ"  =  "1"  /* Extirpation of Matter from Peritoneum, Open Approach */
"0DCW3ZZ"  =  "1"  /* Extirpation of Matter from Peritoneum, Percutaneous Approach */
"0DCW4ZZ"  =  "1"  /* Extirpation of Matter from Peritoneum, Percutaneous Endoscopic Approach */
"0DF60ZZ"  =  "1"  /* Fragmentation in Stomach, Open Approach */
"0DF63ZZ"  =  "1"  /* Fragmentation in Stomach, Percutaneous Approach */
"0DF64ZZ"  =  "1"  /* Fragmentation in Stomach, Percutaneous Endoscopic Approach */
"0DF67ZZ"  =  "1"  /* Fragmentation in Stomach, Via Natural or Artificial Opening */
"0DF68ZZ"  =  "1"  /* Fragmentation in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DFJ0ZZ"  =  "1"  /* Fragmentation in Appendix, Open Approach */
"0DFJ3ZZ"  =  "1"  /* Fragmentation in Appendix, Percutaneous Approach */
"0DFJ4ZZ"  =  "1"  /* Fragmentation in Appendix, Percutaneous Endoscopic Approach */
"0DFJ7ZZ"  =  "1"  /* Fragmentation in Appendix, Via Natural or Artificial Opening */
"0DFJ8ZZ"  =  "1"  /* Fragmentation in Appendix, Via Natural or Artificial Opening Endoscopic */
"0DH602Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Open Approach */
"0DH603Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Open Approach */
"0DH60DZ"  =  "1"  /* Insertion of Intraluminal Device into Stomach, Open Approach */
"0DH60UZ"  =  "1"  /* Insertion of Feeding Device into Stomach, Open Approach */
"0DH632Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Percutaneous Approach */
"0DH633Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Percutaneous Approach */
"0DH63DZ"  =  "1"  /* Insertion of Intraluminal Device into Stomach, Percutaneous Approach */
"0DH642Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Percutaneous Endoscopic Approach */
"0DH643Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Percutaneous Endoscopic Approach */
"0DH64DZ"  =  "1"  /* Insertion of Intraluminal Device into Stomach, Percutaneous Endoscopic Approach */
"0DH672Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Via Natural or Artificial Opening */
"0DH673Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Via Natural or Artificial Opening */
"0DH67DZ"  =  "1"  /* Insertion of Intraluminal Device into Stomach, Via Natural or Artificial Opening */
"0DH67UZ"  =  "1"  /* Insertion of Feeding Device into Stomach, Via Natural or Artificial Opening */
"0DH682Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Via Natural or Artificial Opening Endoscopic */
"0DH683Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Via Natural or Artificial Opening Endoscopic */
"0DH68DZ"  =  "1"  /* Insertion of Intraluminal Device into Stomach, Via Natural or Artificial Opening Endoscopic */
"0DH68UZ"  =  "1"  /* Insertion of Feeding Device into Stomach, Via Natural or Artificial Opening Endoscopic */
"0DH802Z"  =  "1"  /* Insertion of Monitoring Device into Small Intestine, Open Approach */
"0DH803Z"  =  "1"  /* Insertion of Infusion Device into Small Intestine, Open Approach */
"0DH832Z"  =  "1"  /* Insertion of Monitoring Device into Small Intestine, Percutaneous Approach */
"0DH833Z"  =  "1"  /* Insertion of Infusion Device into Small Intestine, Percutaneous Approach */
"0DH842Z"  =  "1"  /* Insertion of Monitoring Device into Small Intestine, Percutaneous Endoscopic Approach */
"0DH843Z"  =  "1"  /* Insertion of Infusion Device into Small Intestine, Percutaneous Endoscopic Approach */
"0DH872Z"  =  "1"  /* Insertion of Monitoring Device into Small Intestine, Via Natural or Artificial Opening */
"0DH873Z"  =  "1"  /* Insertion of Infusion Device into Small Intestine, Via Natural or Artificial Opening */
"0DH882Z"  =  "1"  /* Insertion of Monitoring Device into Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DH883Z"  =  "1"  /* Insertion of Infusion Device into Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DH902Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Open Approach */
"0DH903Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Open Approach */
"0DH932Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Percutaneous Approach */
"0DH933Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Percutaneous Approach */
"0DH942Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Percutaneous Endoscopic Approach */
"0DH943Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Percutaneous Endoscopic Approach */
"0DH972Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Via Natural or Artificial Opening */
"0DH973Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Via Natural or Artificial Opening */
"0DH982Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DH983Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DHA02Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Open Approach */
"0DHA03Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Open Approach */
"0DHA32Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Percutaneous Approach */
"0DHA33Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Percutaneous Approach */
"0DHA42Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Percutaneous Endoscopic Approach */
"0DHA43Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Percutaneous Endoscopic Approach */
"0DHA72Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Via Natural or Artificial Opening */
"0DHA73Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Via Natural or Artificial Opening */
"0DHA82Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DHA83Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DHB02Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Open Approach */
"0DHB03Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Open Approach */
"0DHB32Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Percutaneous Approach */
"0DHB33Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Percutaneous Approach */
"0DHB42Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Percutaneous Endoscopic Approach */
"0DHB43Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Percutaneous Endoscopic Approach */
"0DHB72Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Via Natural or Artificial Opening */
"0DHB73Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Via Natural or Artificial Opening */
"0DHB82Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Via Natural or Artificial Opening Endoscopic */
"0DHB83Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Via Natural or Artificial Opening Endoscopic */
"0DJ00ZZ"  =  "1"  /* Inspection of Upper Intestinal Tract, Open Approach */
"0DJ60ZZ"  =  "1"  /* Inspection of Stomach, Open Approach */
"0DJ64ZZ"  =  "1"  /* Inspection of Stomach, Percutaneous Endoscopic Approach */
"0DJD0ZZ"  =  "1"  /* Inspection of Lower Intestinal Tract, Open Approach */
"0DJU0ZZ"  =  "1"  /* Inspection of Omentum, Open Approach */
"0DJV0ZZ"  =  "1"  /* Inspection of Mesentery, Open Approach */
"0DJW0ZZ"  =  "1"  /* Inspection of Peritoneum, Open Approach */
"0DL60CZ"  =  "1"  /* Occlusion of Stomach with Extraluminal Device, Open Approach */
"0DL60DZ"  =  "1"  /* Occlusion of Stomach with Intraluminal Device, Open Approach */
"0DL60ZZ"  =  "1"  /* Occlusion of Stomach, Open Approach */
"0DL63CZ"  =  "1"  /* Occlusion of Stomach with Extraluminal Device, Percutaneous Approach */
"0DL63DZ"  =  "1"  /* Occlusion of Stomach with Intraluminal Device, Percutaneous Approach */
"0DL63ZZ"  =  "1"  /* Occlusion of Stomach, Percutaneous Approach */
"0DL64CZ"  =  "1"  /* Occlusion of Stomach with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DL64DZ"  =  "1"  /* Occlusion of Stomach with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DL64ZZ"  =  "1"  /* Occlusion of Stomach, Percutaneous Endoscopic Approach */
"0DL67DZ"  =  "1"  /* Occlusion of Stomach with Intraluminal Device, Via Natural or Artificial Opening */
"0DL67ZZ"  =  "1"  /* Occlusion of Stomach, Via Natural or Artificial Opening */
"0DL68DZ"  =  "1"  /* Occlusion of Stomach with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DL68ZZ"  =  "1"  /* Occlusion of Stomach, Via Natural or Artificial Opening Endoscopic */
"0DL70CZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Extraluminal Device, Open Approach */
"0DL70DZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Intraluminal Device, Open Approach */
"0DL70ZZ"  =  "1"  /* Occlusion of Stomach, Pylorus, Open Approach */
"0DL73CZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Extraluminal Device, Percutaneous Approach */
"0DL73DZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Intraluminal Device, Percutaneous Approach */
"0DL73ZZ"  =  "1"  /* Occlusion of Stomach, Pylorus, Percutaneous Approach */
"0DL74CZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DL74DZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DL74ZZ"  =  "1"  /* Occlusion of Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0DL77DZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening */
"0DL77ZZ"  =  "1"  /* Occlusion of Stomach, Pylorus, Via Natural or Artificial Opening */
"0DL78DZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DL78ZZ"  =  "1"  /* Occlusion of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0DL80CZ"  =  "1"  /* Occlusion of Small Intestine with Extraluminal Device, Open Approach */
"0DL80DZ"  =  "1"  /* Occlusion of Small Intestine with Intraluminal Device, Open Approach */
"0DL80ZZ"  =  "1"  /* Occlusion of Small Intestine, Open Approach */
"0DL83CZ"  =  "1"  /* Occlusion of Small Intestine with Extraluminal Device, Percutaneous Approach */
"0DL83DZ"  =  "1"  /* Occlusion of Small Intestine with Intraluminal Device, Percutaneous Approach */
"0DL83ZZ"  =  "1"  /* Occlusion of Small Intestine, Percutaneous Approach */
"0DL84CZ"  =  "1"  /* Occlusion of Small Intestine with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DL84DZ"  =  "1"  /* Occlusion of Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DL84ZZ"  =  "1"  /* Occlusion of Small Intestine, Percutaneous Endoscopic Approach */
"0DL87DZ"  =  "1"  /* Occlusion of Small Intestine with Intraluminal Device, Via Natural or Artificial Opening */
"0DL87ZZ"  =  "1"  /* Occlusion of Small Intestine, Via Natural or Artificial Opening */
"0DL88DZ"  =  "1"  /* Occlusion of Small Intestine with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DL88ZZ"  =  "1"  /* Occlusion of Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DL90CZ"  =  "1"  /* Occlusion of Duodenum with Extraluminal Device, Open Approach */
"0DL90DZ"  =  "1"  /* Occlusion of Duodenum with Intraluminal Device, Open Approach */
"0DL90ZZ"  =  "1"  /* Occlusion of Duodenum, Open Approach */
"0DL93CZ"  =  "1"  /* Occlusion of Duodenum with Extraluminal Device, Percutaneous Approach */
"0DL93DZ"  =  "1"  /* Occlusion of Duodenum with Intraluminal Device, Percutaneous Approach */
"0DL93ZZ"  =  "1"  /* Occlusion of Duodenum, Percutaneous Approach */
"0DL94CZ"  =  "1"  /* Occlusion of Duodenum with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DL94DZ"  =  "1"  /* Occlusion of Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DL94ZZ"  =  "1"  /* Occlusion of Duodenum, Percutaneous Endoscopic Approach */
"0DL97DZ"  =  "1"  /* Occlusion of Duodenum with Intraluminal Device, Via Natural or Artificial Opening */
"0DL97ZZ"  =  "1"  /* Occlusion of Duodenum, Via Natural or Artificial Opening */
"0DL98DZ"  =  "1"  /* Occlusion of Duodenum with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DL98ZZ"  =  "1"  /* Occlusion of Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DLA0CZ"  =  "1"  /* Occlusion of Jejunum with Extraluminal Device, Open Approach */
"0DLA0DZ"  =  "1"  /* Occlusion of Jejunum with Intraluminal Device, Open Approach */
"0DLA0ZZ"  =  "1"  /* Occlusion of Jejunum, Open Approach */
"0DLA3CZ"  =  "1"  /* Occlusion of Jejunum with Extraluminal Device, Percutaneous Approach */
"0DLA3DZ"  =  "1"  /* Occlusion of Jejunum with Intraluminal Device, Percutaneous Approach */
"0DLA3ZZ"  =  "1"  /* Occlusion of Jejunum, Percutaneous Approach */
"0DLA4CZ"  =  "1"  /* Occlusion of Jejunum with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLA4DZ"  =  "1"  /* Occlusion of Jejunum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLA4ZZ"  =  "1"  /* Occlusion of Jejunum, Percutaneous Endoscopic Approach */
"0DLA7DZ"  =  "1"  /* Occlusion of Jejunum with Intraluminal Device, Via Natural or Artificial Opening */
"0DLA7ZZ"  =  "1"  /* Occlusion of Jejunum, Via Natural or Artificial Opening */
"0DLA8DZ"  =  "1"  /* Occlusion of Jejunum with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLA8ZZ"  =  "1"  /* Occlusion of Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DLB0CZ"  =  "1"  /* Occlusion of Ileum with Extraluminal Device, Open Approach */
"0DLB0DZ"  =  "1"  /* Occlusion of Ileum with Intraluminal Device, Open Approach */
"0DLB0ZZ"  =  "1"  /* Occlusion of Ileum, Open Approach */
"0DLB3CZ"  =  "1"  /* Occlusion of Ileum with Extraluminal Device, Percutaneous Approach */
"0DLB3DZ"  =  "1"  /* Occlusion of Ileum with Intraluminal Device, Percutaneous Approach */
"0DLB3ZZ"  =  "1"  /* Occlusion of Ileum, Percutaneous Approach */
"0DLB4CZ"  =  "1"  /* Occlusion of Ileum with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLB4DZ"  =  "1"  /* Occlusion of Ileum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLB4ZZ"  =  "1"  /* Occlusion of Ileum, Percutaneous Endoscopic Approach */
"0DLB7DZ"  =  "1"  /* Occlusion of Ileum with Intraluminal Device, Via Natural or Artificial Opening */
"0DLB7ZZ"  =  "1"  /* Occlusion of Ileum, Via Natural or Artificial Opening */
"0DLB8DZ"  =  "1"  /* Occlusion of Ileum with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLB8ZZ"  =  "1"  /* Occlusion of Ileum, Via Natural or Artificial Opening Endoscopic */
"0DLC0CZ"  =  "1"  /* Occlusion of Ileocecal Valve with Extraluminal Device, Open Approach */
"0DLC0DZ"  =  "1"  /* Occlusion of Ileocecal Valve with Intraluminal Device, Open Approach */
"0DLC0ZZ"  =  "1"  /* Occlusion of Ileocecal Valve, Open Approach */
"0DLC3CZ"  =  "1"  /* Occlusion of Ileocecal Valve with Extraluminal Device, Percutaneous Approach */
"0DLC3DZ"  =  "1"  /* Occlusion of Ileocecal Valve with Intraluminal Device, Percutaneous Approach */
"0DLC3ZZ"  =  "1"  /* Occlusion of Ileocecal Valve, Percutaneous Approach */
"0DLC4CZ"  =  "1"  /* Occlusion of Ileocecal Valve with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLC4DZ"  =  "1"  /* Occlusion of Ileocecal Valve with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLC4ZZ"  =  "1"  /* Occlusion of Ileocecal Valve, Percutaneous Endoscopic Approach */
"0DLC7DZ"  =  "1"  /* Occlusion of Ileocecal Valve with Intraluminal Device, Via Natural or Artificial Opening */
"0DLC7ZZ"  =  "1"  /* Occlusion of Ileocecal Valve, Via Natural or Artificial Opening */
"0DLC8DZ"  =  "1"  /* Occlusion of Ileocecal Valve with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLC8ZZ"  =  "1"  /* Occlusion of Ileocecal Valve, Via Natural or Artificial Opening Endoscopic */
"0DLE0CZ"  =  "1"  /* Occlusion of Large Intestine with Extraluminal Device, Open Approach */
"0DLE0DZ"  =  "1"  /* Occlusion of Large Intestine with Intraluminal Device, Open Approach */
"0DLE0ZZ"  =  "1"  /* Occlusion of Large Intestine, Open Approach */
"0DLE3CZ"  =  "1"  /* Occlusion of Large Intestine with Extraluminal Device, Percutaneous Approach */
"0DLE3DZ"  =  "1"  /* Occlusion of Large Intestine with Intraluminal Device, Percutaneous Approach */
"0DLE3ZZ"  =  "1"  /* Occlusion of Large Intestine, Percutaneous Approach */
"0DLE4CZ"  =  "1"  /* Occlusion of Large Intestine with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLE4DZ"  =  "1"  /* Occlusion of Large Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLE4ZZ"  =  "1"  /* Occlusion of Large Intestine, Percutaneous Endoscopic Approach */
"0DLE7DZ"  =  "1"  /* Occlusion of Large Intestine with Intraluminal Device, Via Natural or Artificial Opening */
"0DLE7ZZ"  =  "1"  /* Occlusion of Large Intestine, Via Natural or Artificial Opening */
"0DLE8DZ"  =  "1"  /* Occlusion of Large Intestine with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLE8ZZ"  =  "1"  /* Occlusion of Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DLF0CZ"  =  "1"  /* Occlusion of Right Large Intestine with Extraluminal Device, Open Approach */
"0DLF0DZ"  =  "1"  /* Occlusion of Right Large Intestine with Intraluminal Device, Open Approach */
"0DLF0ZZ"  =  "1"  /* Occlusion of Right Large Intestine, Open Approach */
"0DLF3CZ"  =  "1"  /* Occlusion of Right Large Intestine with Extraluminal Device, Percutaneous Approach */
"0DLF3DZ"  =  "1"  /* Occlusion of Right Large Intestine with Intraluminal Device, Percutaneous Approach */
"0DLF3ZZ"  =  "1"  /* Occlusion of Right Large Intestine, Percutaneous Approach */
"0DLF4CZ"  =  "1"  /* Occlusion of Right Large Intestine with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLF4DZ"  =  "1"  /* Occlusion of Right Large Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLF4ZZ"  =  "1"  /* Occlusion of Right Large Intestine, Percutaneous Endoscopic Approach */
"0DLF7DZ"  =  "1"  /* Occlusion of Right Large Intestine with Intraluminal Device, Via Natural or Artificial Opening */
"0DLF7ZZ"  =  "1"  /* Occlusion of Right Large Intestine, Via Natural or Artificial Opening */
"0DLF8DZ"  =  "1"  /* Occlusion of Right Large Intestine with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLF8ZZ"  =  "1"  /* Occlusion of Right Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DLG0CZ"  =  "1"  /* Occlusion of Left Large Intestine with Extraluminal Device, Open Approach */
"0DLG0DZ"  =  "1"  /* Occlusion of Left Large Intestine with Intraluminal Device, Open Approach */
"0DLG0ZZ"  =  "1"  /* Occlusion of Left Large Intestine, Open Approach */
"0DLG3CZ"  =  "1"  /* Occlusion of Left Large Intestine with Extraluminal Device, Percutaneous Approach */
"0DLG3DZ"  =  "1"  /* Occlusion of Left Large Intestine with Intraluminal Device, Percutaneous Approach */
"0DLG3ZZ"  =  "1"  /* Occlusion of Left Large Intestine, Percutaneous Approach */
"0DLG4CZ"  =  "1"  /* Occlusion of Left Large Intestine with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLG4DZ"  =  "1"  /* Occlusion of Left Large Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLG4ZZ"  =  "1"  /* Occlusion of Left Large Intestine, Percutaneous Endoscopic Approach */
"0DLG7DZ"  =  "1"  /* Occlusion of Left Large Intestine with Intraluminal Device, Via Natural or Artificial Opening */
"0DLG7ZZ"  =  "1"  /* Occlusion of Left Large Intestine, Via Natural or Artificial Opening */
"0DLG8DZ"  =  "1"  /* Occlusion of Left Large Intestine with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLG8ZZ"  =  "1"  /* Occlusion of Left Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DLH0CZ"  =  "1"  /* Occlusion of Cecum with Extraluminal Device, Open Approach */
"0DLH0DZ"  =  "1"  /* Occlusion of Cecum with Intraluminal Device, Open Approach */
"0DLH0ZZ"  =  "1"  /* Occlusion of Cecum, Open Approach */
"0DLH3CZ"  =  "1"  /* Occlusion of Cecum with Extraluminal Device, Percutaneous Approach */
"0DLH3DZ"  =  "1"  /* Occlusion of Cecum with Intraluminal Device, Percutaneous Approach */
"0DLH3ZZ"  =  "1"  /* Occlusion of Cecum, Percutaneous Approach */
"0DLH4CZ"  =  "1"  /* Occlusion of Cecum with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLH4DZ"  =  "1"  /* Occlusion of Cecum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLH4ZZ"  =  "1"  /* Occlusion of Cecum, Percutaneous Endoscopic Approach */
"0DLH7DZ"  =  "1"  /* Occlusion of Cecum with Intraluminal Device, Via Natural or Artificial Opening */
"0DLH7ZZ"  =  "1"  /* Occlusion of Cecum, Via Natural or Artificial Opening */
"0DLH8DZ"  =  "1"  /* Occlusion of Cecum with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLH8ZZ"  =  "1"  /* Occlusion of Cecum, Via Natural or Artificial Opening Endoscopic */
"0DLK0CZ"  =  "1"  /* Occlusion of Ascending Colon with Extraluminal Device, Open Approach */
"0DLK0DZ"  =  "1"  /* Occlusion of Ascending Colon with Intraluminal Device, Open Approach */
"0DLK0ZZ"  =  "1"  /* Occlusion of Ascending Colon, Open Approach */
"0DLK3CZ"  =  "1"  /* Occlusion of Ascending Colon with Extraluminal Device, Percutaneous Approach */
"0DLK3DZ"  =  "1"  /* Occlusion of Ascending Colon with Intraluminal Device, Percutaneous Approach */
"0DLK3ZZ"  =  "1"  /* Occlusion of Ascending Colon, Percutaneous Approach */
"0DLK4CZ"  =  "1"  /* Occlusion of Ascending Colon with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLK4DZ"  =  "1"  /* Occlusion of Ascending Colon with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLK4ZZ"  =  "1"  /* Occlusion of Ascending Colon, Percutaneous Endoscopic Approach */
"0DLK7DZ"  =  "1"  /* Occlusion of Ascending Colon with Intraluminal Device, Via Natural or Artificial Opening */
"0DLK7ZZ"  =  "1"  /* Occlusion of Ascending Colon, Via Natural or Artificial Opening */
"0DLK8DZ"  =  "1"  /* Occlusion of Ascending Colon with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLK8ZZ"  =  "1"  /* Occlusion of Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0DLL0CZ"  =  "1"  /* Occlusion of Transverse Colon with Extraluminal Device, Open Approach */
"0DLL0DZ"  =  "1"  /* Occlusion of Transverse Colon with Intraluminal Device, Open Approach */
"0DLL0ZZ"  =  "1"  /* Occlusion of Transverse Colon, Open Approach */
"0DLL3CZ"  =  "1"  /* Occlusion of Transverse Colon with Extraluminal Device, Percutaneous Approach */
"0DLL3DZ"  =  "1"  /* Occlusion of Transverse Colon with Intraluminal Device, Percutaneous Approach */
"0DLL3ZZ"  =  "1"  /* Occlusion of Transverse Colon, Percutaneous Approach */
"0DLL4CZ"  =  "1"  /* Occlusion of Transverse Colon with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLL4DZ"  =  "1"  /* Occlusion of Transverse Colon with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLL4ZZ"  =  "1"  /* Occlusion of Transverse Colon, Percutaneous Endoscopic Approach */
"0DLL7DZ"  =  "1"  /* Occlusion of Transverse Colon with Intraluminal Device, Via Natural or Artificial Opening */
"0DLL7ZZ"  =  "1"  /* Occlusion of Transverse Colon, Via Natural or Artificial Opening */
"0DLL8DZ"  =  "1"  /* Occlusion of Transverse Colon with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLL8ZZ"  =  "1"  /* Occlusion of Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0DLM0CZ"  =  "1"  /* Occlusion of Descending Colon with Extraluminal Device, Open Approach */
"0DLM0DZ"  =  "1"  /* Occlusion of Descending Colon with Intraluminal Device, Open Approach */
"0DLM0ZZ"  =  "1"  /* Occlusion of Descending Colon, Open Approach */
"0DLM3CZ"  =  "1"  /* Occlusion of Descending Colon with Extraluminal Device, Percutaneous Approach */
"0DLM3DZ"  =  "1"  /* Occlusion of Descending Colon with Intraluminal Device, Percutaneous Approach */
"0DLM3ZZ"  =  "1"  /* Occlusion of Descending Colon, Percutaneous Approach */
"0DLM4CZ"  =  "1"  /* Occlusion of Descending Colon with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLM4DZ"  =  "1"  /* Occlusion of Descending Colon with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLM4ZZ"  =  "1"  /* Occlusion of Descending Colon, Percutaneous Endoscopic Approach */
"0DLM7DZ"  =  "1"  /* Occlusion of Descending Colon with Intraluminal Device, Via Natural or Artificial Opening */
"0DLM7ZZ"  =  "1"  /* Occlusion of Descending Colon, Via Natural or Artificial Opening */
"0DLM8DZ"  =  "1"  /* Occlusion of Descending Colon with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLM8ZZ"  =  "1"  /* Occlusion of Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0DLN0CZ"  =  "1"  /* Occlusion of Sigmoid Colon with Extraluminal Device, Open Approach */
"0DLN0DZ"  =  "1"  /* Occlusion of Sigmoid Colon with Intraluminal Device, Open Approach */
"0DLN0ZZ"  =  "1"  /* Occlusion of Sigmoid Colon, Open Approach */
"0DLN3CZ"  =  "1"  /* Occlusion of Sigmoid Colon with Extraluminal Device, Percutaneous Approach */
"0DLN3DZ"  =  "1"  /* Occlusion of Sigmoid Colon with Intraluminal Device, Percutaneous Approach */
"0DLN3ZZ"  =  "1"  /* Occlusion of Sigmoid Colon, Percutaneous Approach */
"0DLN4CZ"  =  "1"  /* Occlusion of Sigmoid Colon with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DLN4DZ"  =  "1"  /* Occlusion of Sigmoid Colon with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DLN4ZZ"  =  "1"  /* Occlusion of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DLN7DZ"  =  "1"  /* Occlusion of Sigmoid Colon with Intraluminal Device, Via Natural or Artificial Opening */
"0DLN7ZZ"  =  "1"  /* Occlusion of Sigmoid Colon, Via Natural or Artificial Opening */
"0DLN8DZ"  =  "1"  /* Occlusion of Sigmoid Colon with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DLN8ZZ"  =  "1"  /* Occlusion of Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0DM60ZZ"  =  "1"  /* Reattachment of Stomach, Open Approach */
"0DM64ZZ"  =  "1"  /* Reattachment of Stomach, Percutaneous Endoscopic Approach */
"0DN60ZZ"  =  "1"  /* Release Stomach, Open Approach */
"0DN63ZZ"  =  "1"  /* Release Stomach, Percutaneous Approach */
"0DN64ZZ"  =  "1"  /* Release Stomach, Percutaneous Endoscopic Approach */
"0DN67ZZ"  =  "1"  /* Release Stomach, Via Natural or Artificial Opening */
"0DN68ZZ"  =  "1"  /* Release Stomach, Via Natural or Artificial Opening Endoscopic */
"0DN70ZZ"  =  "1"  /* Release Stomach, Pylorus, Open Approach */
"0DN73ZZ"  =  "1"  /* Release Stomach, Pylorus, Percutaneous Approach */
"0DN74ZZ"  =  "1"  /* Release Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0DN77ZZ"  =  "1"  /* Release Stomach, Pylorus, Via Natural or Artificial Opening */
"0DN78ZZ"  =  "1"  /* Release Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0DN80ZZ"  =  "1"  /* Release Small Intestine, Open Approach */
"0DN83ZZ"  =  "1"  /* Release Small Intestine, Percutaneous Approach */
"0DN90ZZ"  =  "1"  /* Release Duodenum, Open Approach */
"0DN93ZZ"  =  "1"  /* Release Duodenum, Percutaneous Approach */
"0DNA0ZZ"  =  "1"  /* Release Jejunum, Open Approach */
"0DNA3ZZ"  =  "1"  /* Release Jejunum, Percutaneous Approach */
"0DNB0ZZ"  =  "1"  /* Release Ileum, Open Approach */
"0DNB3ZZ"  =  "1"  /* Release Ileum, Percutaneous Approach */
"0DNC0ZZ"  =  "1"  /* Release Ileocecal Valve, Open Approach */
"0DNC3ZZ"  =  "1"  /* Release Ileocecal Valve, Percutaneous Approach */
"0DNE0ZZ"  =  "1"  /* Release Large Intestine, Open Approach */
"0DNE3ZZ"  =  "1"  /* Release Large Intestine, Percutaneous Approach */
"0DNF0ZZ"  =  "1"  /* Release Right Large Intestine, Open Approach */
"0DNF3ZZ"  =  "1"  /* Release Right Large Intestine, Percutaneous Approach */
"0DNG0ZZ"  =  "1"  /* Release Left Large Intestine, Open Approach */
"0DNG3ZZ"  =  "1"  /* Release Left Large Intestine, Percutaneous Approach */
"0DNH0ZZ"  =  "1"  /* Release Cecum, Open Approach */
"0DNH3ZZ"  =  "1"  /* Release Cecum, Percutaneous Approach */
"0DNJ0ZZ"  =  "1"  /* Release Appendix, Open Approach */
"0DNJ3ZZ"  =  "1"  /* Release Appendix, Percutaneous Approach */
"0DNJ7ZZ"  =  "1"  /* Release Appendix, Via Natural or Artificial Opening */
"0DNJ8ZZ"  =  "1"  /* Release Appendix, Via Natural or Artificial Opening Endoscopic */
"0DNK0ZZ"  =  "1"  /* Release Ascending Colon, Open Approach */
"0DNK3ZZ"  =  "1"  /* Release Ascending Colon, Percutaneous Approach */
"0DNL0ZZ"  =  "1"  /* Release Transverse Colon, Open Approach */
"0DNL3ZZ"  =  "1"  /* Release Transverse Colon, Percutaneous Approach */
"0DNM0ZZ"  =  "1"  /* Release Descending Colon, Open Approach */
"0DNM3ZZ"  =  "1"  /* Release Descending Colon, Percutaneous Approach */
"0DNN0ZZ"  =  "1"  /* Release Sigmoid Colon, Open Approach */
"0DNN3ZZ"  =  "1"  /* Release Sigmoid Colon, Percutaneous Approach */
"0DNS0ZZ"  =  "1"  /* Release Greater Omentum, Open Approach */
"0DNS3ZZ"  =  "1"  /* Release Greater Omentum, Percutaneous Approach */
"0DNT0ZZ"  =  "1"  /* Release Lesser Omentum, Open Approach */
"0DNT3ZZ"  =  "1"  /* Release Lesser Omentum, Percutaneous Approach */
"0DNV0ZZ"  =  "1"  /* Release Mesentery, Open Approach */
"0DNV3ZZ"  =  "1"  /* Release Mesentery, Percutaneous Approach */
"0DNW0ZZ"  =  "1"  /* Release Peritoneum, Open Approach */
"0DNW3ZZ"  =  "1"  /* Release Peritoneum, Percutaneous Approach */
"0DP000Z"  =  "1"  /* Removal of Drainage Device from Upper Intestinal Tract, Open Approach */
"0DP002Z"  =  "1"  /* Removal of Monitoring Device from Upper Intestinal Tract, Open Approach */
"0DP003Z"  =  "1"  /* Removal of Infusion Device from Upper Intestinal Tract, Open Approach */
"0DP007Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Open Approach */
"0DP00CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Intestinal Tract, Open Approach */
"0DP00DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Intestinal Tract, Open Approach */
"0DP00JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Intestinal Tract, Open Approach */
"0DP00KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Open Approach */
"0DP00UZ"  =  "1"  /* Removal of Feeding Device from Upper Intestinal Tract, Open Approach */
"0DP030Z"  =  "1"  /* Removal of Drainage Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP032Z"  =  "1"  /* Removal of Monitoring Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP033Z"  =  "1"  /* Removal of Infusion Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP037Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Percutaneous Approach */
"0DP03CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP03DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP03JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Intestinal Tract, Percutaneous Approach */
"0DP03KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Percutaneous Approach */
"0DP03UZ"  =  "1"  /* Removal of Feeding Device from Upper Intestinal Tract, Percutaneous Approach */
"0DP040Z"  =  "1"  /* Removal of Drainage Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP042Z"  =  "1"  /* Removal of Monitoring Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP043Z"  =  "1"  /* Removal of Infusion Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP047Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP04CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP04DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP04JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP04KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP04UZ"  =  "1"  /* Removal of Feeding Device from Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DP070Z"  =  "1"  /* Removal of Drainage Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP072Z"  =  "1"  /* Removal of Monitoring Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP073Z"  =  "1"  /* Removal of Infusion Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP077Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP07CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP07DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP07JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP07KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP07UZ"  =  "1"  /* Removal of Feeding Device from Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DP080Z"  =  "1"  /* Removal of Drainage Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP082Z"  =  "1"  /* Removal of Monitoring Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP083Z"  =  "1"  /* Removal of Infusion Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP087Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP08CZ"  =  "1"  /* Removal of Extraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP08DZ"  =  "1"  /* Removal of Intraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP08JZ"  =  "1"  /* Removal of Synthetic Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP08KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP08UZ"  =  "1"  /* Removal of Feeding Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DP600Z"  =  "1"  /* Removal of Drainage Device from Stomach, Open Approach */
"0DP602Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Open Approach */
"0DP603Z"  =  "1"  /* Removal of Infusion Device from Stomach, Open Approach */
"0DP607Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Stomach, Open Approach */
"0DP60CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Open Approach */
"0DP60DZ"  =  "1"  /* Removal of Intraluminal Device from Stomach, Open Approach */
"0DP60JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Open Approach */
"0DP60KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Stomach, Open Approach */
"0DP60UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Open Approach */
"0DP630Z"  =  "1"  /* Removal of Drainage Device from Stomach, Percutaneous Approach */
"0DP632Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Percutaneous Approach */
"0DP633Z"  =  "1"  /* Removal of Infusion Device from Stomach, Percutaneous Approach */
"0DP637Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Stomach, Percutaneous Approach */
"0DP63CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Percutaneous Approach */
"0DP63DZ"  =  "1"  /* Removal of Intraluminal Device from Stomach, Percutaneous Approach */
"0DP63JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Percutaneous Approach */
"0DP63KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Stomach, Percutaneous Approach */
"0DP63UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Percutaneous Approach */
"0DP640Z"  =  "1"  /* Removal of Drainage Device from Stomach, Percutaneous Endoscopic Approach */
"0DP642Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Percutaneous Endoscopic Approach */
"0DP647Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Stomach, Percutaneous Endoscopic Approach */
"0DP64DZ"  =  "1"  /* Removal of Intraluminal Device from Stomach, Percutaneous Endoscopic Approach */
"0DP64JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Percutaneous Endoscopic Approach */
"0DP64KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Stomach, Percutaneous Endoscopic Approach */
"0DP64UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Percutaneous Endoscopic Approach */
"0DP670Z"  =  "1"  /* Removal of Drainage Device from Stomach, Via Natural or Artificial Opening */
"0DP672Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Via Natural or Artificial Opening */
"0DP673Z"  =  "1"  /* Removal of Infusion Device from Stomach, Via Natural or Artificial Opening */
"0DP677Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Stomach, Via Natural or Artificial Opening */
"0DP67CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Via Natural or Artificial Opening */
"0DP67JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Via Natural or Artificial Opening */
"0DP67KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Stomach, Via Natural or Artificial Opening */
"0DP67UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Via Natural or Artificial Opening */
"0DP680Z"  =  "1"  /* Removal of Drainage Device from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP682Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP683Z"  =  "1"  /* Removal of Infusion Device from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP687Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP68CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP68JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP68KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DP68UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Via Natural or Artificial Opening Endoscopic */
"0DPD00Z"  =  "1"  /* Removal of Drainage Device from Lower Intestinal Tract, Open Approach */
"0DPD02Z"  =  "1"  /* Removal of Monitoring Device from Lower Intestinal Tract, Open Approach */
"0DPD03Z"  =  "1"  /* Removal of Infusion Device from Lower Intestinal Tract, Open Approach */
"0DPD07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Open Approach */
"0DPD0CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Intestinal Tract, Open Approach */
"0DPD0DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Intestinal Tract, Open Approach */
"0DPD0JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Intestinal Tract, Open Approach */
"0DPD0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Open Approach */
"0DPD0UZ"  =  "1"  /* Removal of Feeding Device from Lower Intestinal Tract, Open Approach */
"0DPD30Z"  =  "1"  /* Removal of Drainage Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD32Z"  =  "1"  /* Removal of Monitoring Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD33Z"  =  "1"  /* Removal of Infusion Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Percutaneous Approach */
"0DPD3CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD3DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD3JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Intestinal Tract, Percutaneous Approach */
"0DPD3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Percutaneous Approach */
"0DPD3UZ"  =  "1"  /* Removal of Feeding Device from Lower Intestinal Tract, Percutaneous Approach */
"0DPD40Z"  =  "1"  /* Removal of Drainage Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD42Z"  =  "1"  /* Removal of Monitoring Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD43Z"  =  "1"  /* Removal of Infusion Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD4CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD4DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD4JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD4UZ"  =  "1"  /* Removal of Feeding Device from Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DPD70Z"  =  "1"  /* Removal of Drainage Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD72Z"  =  "1"  /* Removal of Monitoring Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD73Z"  =  "1"  /* Removal of Infusion Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD7CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD7DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD7JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD7UZ"  =  "1"  /* Removal of Feeding Device from Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DPD80Z"  =  "1"  /* Removal of Drainage Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD82Z"  =  "1"  /* Removal of Monitoring Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD83Z"  =  "1"  /* Removal of Infusion Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD8CZ"  =  "1"  /* Removal of Extraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD8DZ"  =  "1"  /* Removal of Intraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD8JZ"  =  "1"  /* Removal of Synthetic Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPD8UZ"  =  "1"  /* Removal of Feeding Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DPU00Z"  =  "1"  /* Removal of Drainage Device from Omentum, Open Approach */
"0DPU01Z"  =  "1"  /* Removal of Radioactive Element from Omentum, Open Approach */
"0DPU07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Omentum, Open Approach */
"0DPU0JZ"  =  "1"  /* Removal of Synthetic Substitute from Omentum, Open Approach */
"0DPU0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Omentum, Open Approach */
"0DPU30Z"  =  "1"  /* Removal of Drainage Device from Omentum, Percutaneous Approach */
"0DPU31Z"  =  "1"  /* Removal of Radioactive Element from Omentum, Percutaneous Approach */
"0DPU37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Omentum, Percutaneous Approach */
"0DPU3JZ"  =  "1"  /* Removal of Synthetic Substitute from Omentum, Percutaneous Approach */
"0DPU3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Omentum, Percutaneous Approach */
"0DPU40Z"  =  "1"  /* Removal of Drainage Device from Omentum, Percutaneous Endoscopic Approach */
"0DPU41Z"  =  "1"  /* Removal of Radioactive Element from Omentum, Percutaneous Endoscopic Approach */
"0DPU47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Omentum, Percutaneous Endoscopic Approach */
"0DPU4JZ"  =  "1"  /* Removal of Synthetic Substitute from Omentum, Percutaneous Endoscopic Approach */
"0DPU4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Omentum, Percutaneous Endoscopic Approach */
"0DPV00Z"  =  "1"  /* Removal of Drainage Device from Mesentery, Open Approach */
"0DPV01Z"  =  "1"  /* Removal of Radioactive Element from Mesentery, Open Approach */
"0DPV07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mesentery, Open Approach */
"0DPV0JZ"  =  "1"  /* Removal of Synthetic Substitute from Mesentery, Open Approach */
"0DPV0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mesentery, Open Approach */
"0DPV30Z"  =  "1"  /* Removal of Drainage Device from Mesentery, Percutaneous Approach */
"0DPV31Z"  =  "1"  /* Removal of Radioactive Element from Mesentery, Percutaneous Approach */
"0DPV37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mesentery, Percutaneous Approach */
"0DPV3JZ"  =  "1"  /* Removal of Synthetic Substitute from Mesentery, Percutaneous Approach */
"0DPV3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mesentery, Percutaneous Approach */
"0DPV40Z"  =  "1"  /* Removal of Drainage Device from Mesentery, Percutaneous Endoscopic Approach */
"0DPV41Z"  =  "1"  /* Removal of Radioactive Element from Mesentery, Percutaneous Endoscopic Approach */
"0DPV47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Mesentery, Percutaneous Endoscopic Approach */
"0DPV4JZ"  =  "1"  /* Removal of Synthetic Substitute from Mesentery, Percutaneous Endoscopic Approach */
"0DPV4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Mesentery, Percutaneous Endoscopic Approach */
"0DPW00Z"  =  "1"  /* Removal of Drainage Device from Peritoneum, Open Approach */
"0DPW01Z"  =  "1"  /* Removal of Radioactive Element from Peritoneum, Open Approach */
"0DPW07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Peritoneum, Open Approach */
"0DPW0JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneum, Open Approach */
"0DPW0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Peritoneum, Open Approach */
"0DPW30Z"  =  "1"  /* Removal of Drainage Device from Peritoneum, Percutaneous Approach */
"0DPW31Z"  =  "1"  /* Removal of Radioactive Element from Peritoneum, Percutaneous Approach */
"0DPW37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Peritoneum, Percutaneous Approach */
"0DPW3JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneum, Percutaneous Approach */
"0DPW3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Peritoneum, Percutaneous Approach */
"0DPW40Z"  =  "1"  /* Removal of Drainage Device from Peritoneum, Percutaneous Endoscopic Approach */
"0DPW41Z"  =  "1"  /* Removal of Radioactive Element from Peritoneum, Percutaneous Endoscopic Approach */
"0DPW47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Peritoneum, Percutaneous Endoscopic Approach */
"0DPW4JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneum, Percutaneous Endoscopic Approach */
"0DPW4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Peritoneum, Percutaneous Endoscopic Approach */
"0DQ40ZZ"  =  "1"  /* Repair Esophagogastric Junction, Open Approach */
"0DQ43ZZ"  =  "1"  /* Repair Esophagogastric Junction, Percutaneous Approach */
"0DQ44ZZ"  =  "1"  /* Repair Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0DQ47ZZ"  =  "1"  /* Repair Esophagogastric Junction, Via Natural or Artificial Opening */
"0DQ48ZZ"  =  "1"  /* Repair Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic */
"0DQ60ZZ"  =  "1"  /* Repair Stomach, Open Approach */
"0DQ63ZZ"  =  "1"  /* Repair Stomach, Percutaneous Approach */
"0DQ64ZZ"  =  "1"  /* Repair Stomach, Percutaneous Endoscopic Approach */
"0DQ67ZZ"  =  "1"  /* Repair Stomach, Via Natural or Artificial Opening */
"0DQ68ZZ"  =  "1"  /* Repair Stomach, Via Natural or Artificial Opening Endoscopic */
"0DQ70ZZ"  =  "1"  /* Repair Stomach, Pylorus, Open Approach */
"0DQ73ZZ"  =  "1"  /* Repair Stomach, Pylorus, Percutaneous Approach */
"0DQ74ZZ"  =  "1"  /* Repair Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0DQ77ZZ"  =  "1"  /* Repair Stomach, Pylorus, Via Natural or Artificial Opening */
"0DQ78ZZ"  =  "1"  /* Repair Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0DQ80ZZ"  =  "1"  /* Repair Small Intestine, Open Approach */
"0DQ83ZZ"  =  "1"  /* Repair Small Intestine, Percutaneous Approach */
"0DQ84ZZ"  =  "1"  /* Repair Small Intestine, Percutaneous Endoscopic Approach */
"0DQ87ZZ"  =  "1"  /* Repair Small Intestine, Via Natural or Artificial Opening */
"0DQ88ZZ"  =  "1"  /* Repair Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DQ90ZZ"  =  "1"  /* Repair Duodenum, Open Approach */
"0DQ93ZZ"  =  "1"  /* Repair Duodenum, Percutaneous Approach */
"0DQ94ZZ"  =  "1"  /* Repair Duodenum, Percutaneous Endoscopic Approach */
"0DQ97ZZ"  =  "1"  /* Repair Duodenum, Via Natural or Artificial Opening */
"0DQ98ZZ"  =  "1"  /* Repair Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DQA0ZZ"  =  "1"  /* Repair Jejunum, Open Approach */
"0DQA3ZZ"  =  "1"  /* Repair Jejunum, Percutaneous Approach */
"0DQA4ZZ"  =  "1"  /* Repair Jejunum, Percutaneous Endoscopic Approach */
"0DQA7ZZ"  =  "1"  /* Repair Jejunum, Via Natural or Artificial Opening */
"0DQA8ZZ"  =  "1"  /* Repair Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DQB0ZZ"  =  "1"  /* Repair Ileum, Open Approach */
"0DQB3ZZ"  =  "1"  /* Repair Ileum, Percutaneous Approach */
"0DQB4ZZ"  =  "1"  /* Repair Ileum, Percutaneous Endoscopic Approach */
"0DQB7ZZ"  =  "1"  /* Repair Ileum, Via Natural or Artificial Opening */
"0DQB8ZZ"  =  "1"  /* Repair Ileum, Via Natural or Artificial Opening Endoscopic */
"0DQE0ZZ"  =  "1"  /* Repair Large Intestine, Open Approach */
"0DQE3ZZ"  =  "1"  /* Repair Large Intestine, Percutaneous Approach */
"0DQE4ZZ"  =  "1"  /* Repair Large Intestine, Percutaneous Endoscopic Approach */
"0DQE7ZZ"  =  "1"  /* Repair Large Intestine, Via Natural or Artificial Opening */
"0DQE8ZZ"  =  "1"  /* Repair Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DQF0ZZ"  =  "1"  /* Repair Right Large Intestine, Open Approach */
"0DQG0ZZ"  =  "1"  /* Repair Left Large Intestine, Open Approach */
"0DQH0ZZ"  =  "1"  /* Repair Cecum, Open Approach */
"0DQH3ZZ"  =  "1"  /* Repair Cecum, Percutaneous Approach */
"0DQH4ZZ"  =  "1"  /* Repair Cecum, Percutaneous Endoscopic Approach */
"0DQH7ZZ"  =  "1"  /* Repair Cecum, Via Natural or Artificial Opening */
"0DQH8ZZ"  =  "1"  /* Repair Cecum, Via Natural or Artificial Opening Endoscopic */
"0DQJ0ZZ"  =  "1"  /* Repair Appendix, Open Approach */
"0DQJ3ZZ"  =  "1"  /* Repair Appendix, Percutaneous Approach */
"0DQJ4ZZ"  =  "1"  /* Repair Appendix, Percutaneous Endoscopic Approach */
"0DQJ7ZZ"  =  "1"  /* Repair Appendix, Via Natural or Artificial Opening */
"0DQJ8ZZ"  =  "1"  /* Repair Appendix, Via Natural or Artificial Opening Endoscopic */
"0DQK0ZZ"  =  "1"  /* Repair Ascending Colon, Open Approach */
"0DQL0ZZ"  =  "1"  /* Repair Transverse Colon, Open Approach */
"0DQM0ZZ"  =  "1"  /* Repair Descending Colon, Open Approach */
"0DQN0ZZ"  =  "1"  /* Repair Sigmoid Colon, Open Approach */
"0DQN3ZZ"  =  "1"  /* Repair Sigmoid Colon, Percutaneous Approach */
"0DQN4ZZ"  =  "1"  /* Repair Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DQN7ZZ"  =  "1"  /* Repair Sigmoid Colon, Via Natural or Artificial Opening */
"0DQN8ZZ"  =  "1"  /* Repair Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0DQP0ZZ"  =  "1"  /* Repair Rectum, Open Approach */
"0DQP3ZZ"  =  "1"  /* Repair Rectum, Percutaneous Approach */
"0DQP4ZZ"  =  "1"  /* Repair Rectum, Percutaneous Endoscopic Approach */
"0DQP7ZZ"  =  "1"  /* Repair Rectum, Via Natural or Artificial Opening */
"0DQP8ZZ"  =  "1"  /* Repair Rectum, Via Natural or Artificial Opening Endoscopic */
"0DQS0ZZ"  =  "1"  /* Repair Greater Omentum, Open Approach */
"0DQS3ZZ"  =  "1"  /* Repair Greater Omentum, Percutaneous Approach */
"0DQS4ZZ"  =  "1"  /* Repair Greater Omentum, Percutaneous Endoscopic Approach */
"0DQT0ZZ"  =  "1"  /* Repair Lesser Omentum, Open Approach */
"0DQT3ZZ"  =  "1"  /* Repair Lesser Omentum, Percutaneous Approach */
"0DQT4ZZ"  =  "1"  /* Repair Lesser Omentum, Percutaneous Endoscopic Approach */
"0DQV0ZZ"  =  "1"  /* Repair Mesentery, Open Approach */
"0DQV3ZZ"  =  "1"  /* Repair Mesentery, Percutaneous Approach */
"0DQV4ZZ"  =  "1"  /* Repair Mesentery, Percutaneous Endoscopic Approach */
"0DQW0ZZ"  =  "1"  /* Repair Peritoneum, Open Approach */
"0DQW3ZZ"  =  "1"  /* Repair Peritoneum, Percutaneous Approach */
"0DQW4ZZ"  =  "1"  /* Repair Peritoneum, Percutaneous Endoscopic Approach */
"0DRS07Z"  =  "1"  /* Replacement of Greater Omentum with Autologous Tissue Substitute, Open Approach */
"0DRS0JZ"  =  "1"  /* Replacement of Greater Omentum with Synthetic Substitute, Open Approach */
"0DRS0KZ"  =  "1"  /* Replacement of Greater Omentum with Nonautologous Tissue Substitute, Open Approach */
"0DRS47Z"  =  "1"  /* Replacement of Greater Omentum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DRS4JZ"  =  "1"  /* Replacement of Greater Omentum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DRS4KZ"  =  "1"  /* Replacement of Greater Omentum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DRT07Z"  =  "1"  /* Replacement of Lesser Omentum with Autologous Tissue Substitute, Open Approach */
"0DRT0JZ"  =  "1"  /* Replacement of Lesser Omentum with Synthetic Substitute, Open Approach */
"0DRT0KZ"  =  "1"  /* Replacement of Lesser Omentum with Nonautologous Tissue Substitute, Open Approach */
"0DRT47Z"  =  "1"  /* Replacement of Lesser Omentum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DRT4JZ"  =  "1"  /* Replacement of Lesser Omentum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DRT4KZ"  =  "1"  /* Replacement of Lesser Omentum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DRV07Z"  =  "1"  /* Replacement of Mesentery with Autologous Tissue Substitute, Open Approach */
"0DRV0JZ"  =  "1"  /* Replacement of Mesentery with Synthetic Substitute, Open Approach */
"0DRV0KZ"  =  "1"  /* Replacement of Mesentery with Nonautologous Tissue Substitute, Open Approach */
"0DRV47Z"  =  "1"  /* Replacement of Mesentery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DRV4JZ"  =  "1"  /* Replacement of Mesentery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DRV4KZ"  =  "1"  /* Replacement of Mesentery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DRW07Z"  =  "1"  /* Replacement of Peritoneum with Autologous Tissue Substitute, Open Approach */
"0DRW0JZ"  =  "1"  /* Replacement of Peritoneum with Synthetic Substitute, Open Approach */
"0DRW0KZ"  =  "1"  /* Replacement of Peritoneum with Nonautologous Tissue Substitute, Open Approach */
"0DRW47Z"  =  "1"  /* Replacement of Peritoneum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DRW4JZ"  =  "1"  /* Replacement of Peritoneum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DRW4KZ"  =  "1"  /* Replacement of Peritoneum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DS60ZZ"  =  "1"  /* Reposition Stomach, Open Approach */
"0DS64ZZ"  =  "1"  /* Reposition Stomach, Percutaneous Endoscopic Approach */
"0DS67ZZ"  =  "1"  /* Reposition Stomach, Via Natural or Artificial Opening */
"0DS68ZZ"  =  "1"  /* Reposition Stomach, Via Natural or Artificial Opening Endoscopic */
"0DS6XZZ"  =  "1"  /* Reposition Stomach, External Approach */
"0DS90ZZ"  =  "1"  /* Reposition Duodenum, Open Approach */
"0DS94ZZ"  =  "1"  /* Reposition Duodenum, Percutaneous Endoscopic Approach */
"0DS97ZZ"  =  "1"  /* Reposition Duodenum, Via Natural or Artificial Opening */
"0DS98ZZ"  =  "1"  /* Reposition Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DSA0ZZ"  =  "1"  /* Reposition Jejunum, Open Approach */
"0DSA4ZZ"  =  "1"  /* Reposition Jejunum, Percutaneous Endoscopic Approach */
"0DSA7ZZ"  =  "1"  /* Reposition Jejunum, Via Natural or Artificial Opening */
"0DSA8ZZ"  =  "1"  /* Reposition Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DSB0ZZ"  =  "1"  /* Reposition Ileum, Open Approach */
"0DSB4ZZ"  =  "1"  /* Reposition Ileum, Percutaneous Endoscopic Approach */
"0DSB7ZZ"  =  "1"  /* Reposition Ileum, Via Natural or Artificial Opening */
"0DSB8ZZ"  =  "1"  /* Reposition Ileum, Via Natural or Artificial Opening Endoscopic */
"0DSH0ZZ"  =  "1"  /* Reposition Cecum, Open Approach */
"0DSH4ZZ"  =  "1"  /* Reposition Cecum, Percutaneous Endoscopic Approach */
"0DSH7ZZ"  =  "1"  /* Reposition Cecum, Via Natural or Artificial Opening */
"0DSH8ZZ"  =  "1"  /* Reposition Cecum, Via Natural or Artificial Opening Endoscopic */
"0DSK0ZZ"  =  "1"  /* Reposition Ascending Colon, Open Approach */
"0DSK4ZZ"  =  "1"  /* Reposition Ascending Colon, Percutaneous Endoscopic Approach */
"0DSK7ZZ"  =  "1"  /* Reposition Ascending Colon, Via Natural or Artificial Opening */
"0DSK8ZZ"  =  "1"  /* Reposition Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0DSL0ZZ"  =  "1"  /* Reposition Transverse Colon, Open Approach */
"0DSL4ZZ"  =  "1"  /* Reposition Transverse Colon, Percutaneous Endoscopic Approach */
"0DSL7ZZ"  =  "1"  /* Reposition Transverse Colon, Via Natural or Artificial Opening */
"0DSL8ZZ"  =  "1"  /* Reposition Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0DSM0ZZ"  =  "1"  /* Reposition Descending Colon, Open Approach */
"0DSM4ZZ"  =  "1"  /* Reposition Descending Colon, Percutaneous Endoscopic Approach */
"0DSM7ZZ"  =  "1"  /* Reposition Descending Colon, Via Natural or Artificial Opening */
"0DSM8ZZ"  =  "1"  /* Reposition Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0DSN0ZZ"  =  "1"  /* Reposition Sigmoid Colon, Open Approach */
"0DSN4ZZ"  =  "1"  /* Reposition Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DSN7ZZ"  =  "1"  /* Reposition Sigmoid Colon, Via Natural or Artificial Opening */
"0DSN8ZZ"  =  "1"  /* Reposition Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0DSP0ZZ"  =  "1"  /* Reposition Rectum, Open Approach */
"0DSP4ZZ"  =  "1"  /* Reposition Rectum, Percutaneous Endoscopic Approach */
"0DT10ZZ"  =  "1"  /* Resection of Upper Esophagus, Open Approach */
"0DT14ZZ"  =  "1"  /* Resection of Upper Esophagus, Percutaneous Endoscopic Approach */
"0DT17ZZ"  =  "1"  /* Resection of Upper Esophagus, Via Natural or Artificial Opening */
"0DT18ZZ"  =  "1"  /* Resection of Upper Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT20ZZ"  =  "1"  /* Resection of Middle Esophagus, Open Approach */
"0DT24ZZ"  =  "1"  /* Resection of Middle Esophagus, Percutaneous Endoscopic Approach */
"0DT27ZZ"  =  "1"  /* Resection of Middle Esophagus, Via Natural or Artificial Opening */
"0DT28ZZ"  =  "1"  /* Resection of Middle Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT30ZZ"  =  "1"  /* Resection of Lower Esophagus, Open Approach */
"0DT34ZZ"  =  "1"  /* Resection of Lower Esophagus, Percutaneous Endoscopic Approach */
"0DT37ZZ"  =  "1"  /* Resection of Lower Esophagus, Via Natural or Artificial Opening */
"0DT38ZZ"  =  "1"  /* Resection of Lower Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT40ZZ"  =  "1"  /* Resection of Esophagogastric Junction, Open Approach */
"0DT44ZZ"  =  "1"  /* Resection of Esophagogastric Junction, Percutaneous Endoscopic Approach */
"0DT47ZZ"  =  "1"  /* Resection of Esophagogastric Junction, Via Natural or Artificial Opening */
"0DT48ZZ"  =  "1"  /* Resection of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic */
"0DT50ZZ"  =  "1"  /* Resection of Esophagus, Open Approach */
"0DT54ZZ"  =  "1"  /* Resection of Esophagus, Percutaneous Endoscopic Approach */
"0DT57ZZ"  =  "1"  /* Resection of Esophagus, Via Natural or Artificial Opening */
"0DT58ZZ"  =  "1"  /* Resection of Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT60ZZ"  =  "1"  /* Resection of Stomach, Open Approach */
"0DT64ZZ"  =  "1"  /* Resection of Stomach, Percutaneous Endoscopic Approach */
"0DT67ZZ"  =  "1"  /* Resection of Stomach, Via Natural or Artificial Opening */
"0DT68ZZ"  =  "1"  /* Resection of Stomach, Via Natural or Artificial Opening Endoscopic */
"0DT70ZZ"  =  "1"  /* Resection of Stomach, Pylorus, Open Approach */
"0DT74ZZ"  =  "1"  /* Resection of Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0DT77ZZ"  =  "1"  /* Resection of Stomach, Pylorus, Via Natural or Artificial Opening */
"0DT78ZZ"  =  "1"  /* Resection of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0DT80ZZ"  =  "1"  /* Resection of Small Intestine, Open Approach */
"0DT84ZZ"  =  "1"  /* Resection of Small Intestine, Percutaneous Endoscopic Approach */
"0DT87ZZ"  =  "1"  /* Resection of Small Intestine, Via Natural or Artificial Opening */
"0DT88ZZ"  =  "1"  /* Resection of Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DT90ZZ"  =  "1"  /* Resection of Duodenum, Open Approach */
"0DT94ZZ"  =  "1"  /* Resection of Duodenum, Percutaneous Endoscopic Approach */
"0DT97ZZ"  =  "1"  /* Resection of Duodenum, Via Natural or Artificial Opening */
"0DT98ZZ"  =  "1"  /* Resection of Duodenum, Via Natural or Artificial Opening Endoscopic */
"0DTA0ZZ"  =  "1"  /* Resection of Jejunum, Open Approach */
"0DTA4ZZ"  =  "1"  /* Resection of Jejunum, Percutaneous Endoscopic Approach */
"0DTA7ZZ"  =  "1"  /* Resection of Jejunum, Via Natural or Artificial Opening */
"0DTA8ZZ"  =  "1"  /* Resection of Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DTB0ZZ"  =  "1"  /* Resection of Ileum, Open Approach */
"0DTB4ZZ"  =  "1"  /* Resection of Ileum, Percutaneous Endoscopic Approach */
"0DTB7ZZ"  =  "1"  /* Resection of Ileum, Via Natural or Artificial Opening */
"0DTB8ZZ"  =  "1"  /* Resection of Ileum, Via Natural or Artificial Opening Endoscopic */
"0DTC0ZZ"  =  "1"  /* Resection of Ileocecal Valve, Open Approach */
"0DTC4ZZ"  =  "1"  /* Resection of Ileocecal Valve, Percutaneous Endoscopic Approach */
"0DTC7ZZ"  =  "1"  /* Resection of Ileocecal Valve, Via Natural or Artificial Opening */
"0DTC8ZZ"  =  "1"  /* Resection of Ileocecal Valve, Via Natural or Artificial Opening Endoscopic */
"0DTE0ZZ"  =  "1"  /* Resection of Large Intestine, Open Approach */
"0DTE4ZZ"  =  "1"  /* Resection of Large Intestine, Percutaneous Endoscopic Approach */
"0DTE7ZZ"  =  "1"  /* Resection of Large Intestine, Via Natural or Artificial Opening */
"0DTE8ZZ"  =  "1"  /* Resection of Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DTF0ZZ"  =  "1"  /* Resection of Right Large Intestine, Open Approach */
"0DTF4ZZ"  =  "1"  /* Resection of Right Large Intestine, Percutaneous Endoscopic Approach */
"0DTF7ZZ"  =  "1"  /* Resection of Right Large Intestine, Via Natural or Artificial Opening */
"0DTF8ZZ"  =  "1"  /* Resection of Right Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DTG0ZZ"  =  "1"  /* Resection of Left Large Intestine, Open Approach */
"0DTG4ZZ"  =  "1"  /* Resection of Left Large Intestine, Percutaneous Endoscopic Approach */
"0DTG7ZZ"  =  "1"  /* Resection of Left Large Intestine, Via Natural or Artificial Opening */
"0DTG8ZZ"  =  "1"  /* Resection of Left Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DTH0ZZ"  =  "1"  /* Resection of Cecum, Open Approach */
"0DTH4ZZ"  =  "1"  /* Resection of Cecum, Percutaneous Endoscopic Approach */
"0DTH7ZZ"  =  "1"  /* Resection of Cecum, Via Natural or Artificial Opening */
"0DTH8ZZ"  =  "1"  /* Resection of Cecum, Via Natural or Artificial Opening Endoscopic */
"0DTJ0ZZ"  =  "1"  /* Resection of Appendix, Open Approach */
"0DTJ7ZZ"  =  "1"  /* Resection of Appendix, Via Natural or Artificial Opening */
"0DTJ8ZZ"  =  "1"  /* Resection of Appendix, Via Natural or Artificial Opening Endoscopic */
"0DTK0ZZ"  =  "1"  /* Resection of Ascending Colon, Open Approach */
"0DTK4ZZ"  =  "1"  /* Resection of Ascending Colon, Percutaneous Endoscopic Approach */
"0DTK7ZZ"  =  "1"  /* Resection of Ascending Colon, Via Natural or Artificial Opening */
"0DTK8ZZ"  =  "1"  /* Resection of Ascending Colon, Via Natural or Artificial Opening Endoscopic */
"0DTL0ZZ"  =  "1"  /* Resection of Transverse Colon, Open Approach */
"0DTL4ZZ"  =  "1"  /* Resection of Transverse Colon, Percutaneous Endoscopic Approach */
"0DTL7ZZ"  =  "1"  /* Resection of Transverse Colon, Via Natural or Artificial Opening */
"0DTL8ZZ"  =  "1"  /* Resection of Transverse Colon, Via Natural or Artificial Opening Endoscopic */
"0DTM0ZZ"  =  "1"  /* Resection of Descending Colon, Open Approach */
"0DTM4ZZ"  =  "1"  /* Resection of Descending Colon, Percutaneous Endoscopic Approach */
"0DTM7ZZ"  =  "1"  /* Resection of Descending Colon, Via Natural or Artificial Opening */
"0DTM8ZZ"  =  "1"  /* Resection of Descending Colon, Via Natural or Artificial Opening Endoscopic */
"0DTN0ZZ"  =  "1"  /* Resection of Sigmoid Colon, Open Approach */
"0DTN4ZZ"  =  "1"  /* Resection of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DTN7ZZ"  =  "1"  /* Resection of Sigmoid Colon, Via Natural or Artificial Opening */
"0DTN8ZZ"  =  "1"  /* Resection of Sigmoid Colon, Via Natural or Artificial Opening Endoscopic */
"0DTP0ZZ"  =  "1"  /* Resection of Rectum, Open Approach */
"0DTP4ZZ"  =  "1"  /* Resection of Rectum, Percutaneous Endoscopic Approach */
"0DTP7ZZ"  =  "1"  /* Resection of Rectum, Via Natural or Artificial Opening */
"0DTP8ZZ"  =  "1"  /* Resection of Rectum, Via Natural or Artificial Opening Endoscopic */
"0DTS0ZZ"  =  "1"  /* Resection of Greater Omentum, Open Approach */
"0DTS4ZZ"  =  "1"  /* Resection of Greater Omentum, Percutaneous Endoscopic Approach */
"0DTT0ZZ"  =  "1"  /* Resection of Lesser Omentum, Open Approach */
"0DTT4ZZ"  =  "1"  /* Resection of Lesser Omentum, Percutaneous Endoscopic Approach */
"0DU407Z"  =  "1"  /* Supplement Esophagogastric Junction with Autologous Tissue Substitute, Open Approach */
"0DU40JZ"  =  "1"  /* Supplement Esophagogastric Junction with Synthetic Substitute, Open Approach */
"0DU40KZ"  =  "1"  /* Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Open Approach */
"0DU447Z"  =  "1"  /* Supplement Esophagogastric Junction with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU44JZ"  =  "1"  /* Supplement Esophagogastric Junction with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DU44KZ"  =  "1"  /* Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU477Z"  =  "1"  /* Supplement Esophagogastric Junction with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU47JZ"  =  "1"  /* Supplement Esophagogastric Junction with Synthetic Substitute, Via Natural or Artificial Opening */
"0DU47KZ"  =  "1"  /* Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU487Z"  =  "1"  /* Supplement Esophagogastric Junction with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU48JZ"  =  "1"  /* Supplement Esophagogastric Junction with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU48KZ"  =  "1"  /* Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU607Z"  =  "1"  /* Supplement Stomach with Autologous Tissue Substitute, Open Approach */
"0DU60JZ"  =  "1"  /* Supplement Stomach with Synthetic Substitute, Open Approach */
"0DU60KZ"  =  "1"  /* Supplement Stomach with Nonautologous Tissue Substitute, Open Approach */
"0DU647Z"  =  "1"  /* Supplement Stomach with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU64JZ"  =  "1"  /* Supplement Stomach with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DU64KZ"  =  "1"  /* Supplement Stomach with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU677Z"  =  "1"  /* Supplement Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU67JZ"  =  "1"  /* Supplement Stomach with Synthetic Substitute, Via Natural or Artificial Opening */
"0DU67KZ"  =  "1"  /* Supplement Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU687Z"  =  "1"  /* Supplement Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU68JZ"  =  "1"  /* Supplement Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU68KZ"  =  "1"  /* Supplement Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU707Z"  =  "1"  /* Supplement Stomach, Pylorus with Autologous Tissue Substitute, Open Approach */
"0DU70JZ"  =  "1"  /* Supplement Stomach, Pylorus with Synthetic Substitute, Open Approach */
"0DU70KZ"  =  "1"  /* Supplement Stomach, Pylorus with Nonautologous Tissue Substitute, Open Approach */
"0DU747Z"  =  "1"  /* Supplement Stomach, Pylorus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU74JZ"  =  "1"  /* Supplement Stomach, Pylorus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DU74KZ"  =  "1"  /* Supplement Stomach, Pylorus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DU777Z"  =  "1"  /* Supplement Stomach, Pylorus with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU77JZ"  =  "1"  /* Supplement Stomach, Pylorus with Synthetic Substitute, Via Natural or Artificial Opening */
"0DU77KZ"  =  "1"  /* Supplement Stomach, Pylorus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0DU787Z"  =  "1"  /* Supplement Stomach, Pylorus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU78JZ"  =  "1"  /* Supplement Stomach, Pylorus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0DU78KZ"  =  "1"  /* Supplement Stomach, Pylorus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0DUS07Z"  =  "1"  /* Supplement Greater Omentum with Autologous Tissue Substitute, Open Approach */
"0DUS0JZ"  =  "1"  /* Supplement Greater Omentum with Synthetic Substitute, Open Approach */
"0DUS0KZ"  =  "1"  /* Supplement Greater Omentum with Nonautologous Tissue Substitute, Open Approach */
"0DUS47Z"  =  "1"  /* Supplement Greater Omentum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUS4JZ"  =  "1"  /* Supplement Greater Omentum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUS4KZ"  =  "1"  /* Supplement Greater Omentum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUT07Z"  =  "1"  /* Supplement Lesser Omentum with Autologous Tissue Substitute, Open Approach */
"0DUT0JZ"  =  "1"  /* Supplement Lesser Omentum with Synthetic Substitute, Open Approach */
"0DUT0KZ"  =  "1"  /* Supplement Lesser Omentum with Nonautologous Tissue Substitute, Open Approach */
"0DUT47Z"  =  "1"  /* Supplement Lesser Omentum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUT4JZ"  =  "1"  /* Supplement Lesser Omentum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUT4KZ"  =  "1"  /* Supplement Lesser Omentum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUV07Z"  =  "1"  /* Supplement Mesentery with Autologous Tissue Substitute, Open Approach */
"0DUV0JZ"  =  "1"  /* Supplement Mesentery with Synthetic Substitute, Open Approach */
"0DUV0KZ"  =  "1"  /* Supplement Mesentery with Nonautologous Tissue Substitute, Open Approach */
"0DUV47Z"  =  "1"  /* Supplement Mesentery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUV4JZ"  =  "1"  /* Supplement Mesentery with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUV4KZ"  =  "1"  /* Supplement Mesentery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUW07Z"  =  "1"  /* Supplement Peritoneum with Autologous Tissue Substitute, Open Approach */
"0DUW0JZ"  =  "1"  /* Supplement Peritoneum with Synthetic Substitute, Open Approach */
"0DUW0KZ"  =  "1"  /* Supplement Peritoneum with Nonautologous Tissue Substitute, Open Approach */
"0DUW47Z"  =  "1"  /* Supplement Peritoneum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DUW4JZ"  =  "1"  /* Supplement Peritoneum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0DUW4KZ"  =  "1"  /* Supplement Peritoneum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0DV40CZ"  =  "1"  /* Restriction of Esophagogastric Junction with Extraluminal Device, Open Approach */
"0DV40DZ"  =  "1"  /* Restriction of Esophagogastric Junction with Intraluminal Device, Open Approach */
"0DV40ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Open Approach */
"0DV43CZ"  =  "1"  /* Restriction of Esophagogastric Junction with Extraluminal Device, Percutaneous Approach */
"0DV43DZ"  =  "1"  /* Restriction of Esophagogastric Junction with Intraluminal Device, Percutaneous Approach */
"0DV43ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Percutaneous Approach */
"0DV47DZ"  =  "1"  /* Restriction of Esophagogastric Junction with Intraluminal Device, Via Natural or Artificial Opening */
"0DV47ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Via Natural or Artificial Opening */
"0DV48DZ"  =  "1"  /* Restriction of Esophagogastric Junction with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DV48ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic */
"0DV60CZ"  =  "1"  /* Restriction of Stomach with Extraluminal Device, Open Approach */
"0DV60DZ"  =  "1"  /* Restriction of Stomach with Intraluminal Device, Open Approach */
"0DV60ZZ"  =  "1"  /* Restriction of Stomach, Open Approach */
"0DV63CZ"  =  "1"  /* Restriction of Stomach with Extraluminal Device, Percutaneous Approach */
"0DV63DZ"  =  "1"  /* Restriction of Stomach with Intraluminal Device, Percutaneous Approach */
"0DV63ZZ"  =  "1"  /* Restriction of Stomach, Percutaneous Approach */
"0DV64DZ"  =  "1"  /* Restriction of Stomach with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DV64ZZ"  =  "1"  /* Restriction of Stomach, Percutaneous Endoscopic Approach */
"0DV67ZZ"  =  "1"  /* Restriction of Stomach, Via Natural or Artificial Opening */
"0DV68ZZ"  =  "1"  /* Restriction of Stomach, Via Natural or Artificial Opening Endoscopic */
"0DV70CZ"  =  "1"  /* Restriction of Stomach, Pylorus with Extraluminal Device, Open Approach */
"0DV70DZ"  =  "1"  /* Restriction of Stomach, Pylorus with Intraluminal Device, Open Approach */
"0DV70ZZ"  =  "1"  /* Restriction of Stomach, Pylorus, Open Approach */
"0DV73CZ"  =  "1"  /* Restriction of Stomach, Pylorus with Extraluminal Device, Percutaneous Approach */
"0DV73DZ"  =  "1"  /* Restriction of Stomach, Pylorus with Intraluminal Device, Percutaneous Approach */
"0DV73ZZ"  =  "1"  /* Restriction of Stomach, Pylorus, Percutaneous Approach */
"0DV74CZ"  =  "1"  /* Restriction of Stomach, Pylorus with Extraluminal Device, Percutaneous Endoscopic Approach */
"0DV74DZ"  =  "1"  /* Restriction of Stomach, Pylorus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0DV74ZZ"  =  "1"  /* Restriction of Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0DV77DZ"  =  "1"  /* Restriction of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening */
"0DV77ZZ"  =  "1"  /* Restriction of Stomach, Pylorus, Via Natural or Artificial Opening */
"0DV78DZ"  =  "1"  /* Restriction of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0DV78ZZ"  =  "1"  /* Restriction of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic */
"0DW000Z"  =  "1"  /* Revision of Drainage Device in Upper Intestinal Tract, Open Approach */
"0DW002Z"  =  "1"  /* Revision of Monitoring Device in Upper Intestinal Tract, Open Approach */
"0DW003Z"  =  "1"  /* Revision of Infusion Device in Upper Intestinal Tract, Open Approach */
"0DW007Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Open Approach */
"0DW00CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Intestinal Tract, Open Approach */
"0DW00DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Intestinal Tract, Open Approach */
"0DW00JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Intestinal Tract, Open Approach */
"0DW00KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Open Approach */
"0DW00UZ"  =  "1"  /* Revision of Feeding Device in Upper Intestinal Tract, Open Approach */
"0DW030Z"  =  "1"  /* Revision of Drainage Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW032Z"  =  "1"  /* Revision of Monitoring Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW033Z"  =  "1"  /* Revision of Infusion Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW037Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Percutaneous Approach */
"0DW03CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW03DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW03JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Intestinal Tract, Percutaneous Approach */
"0DW03KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Percutaneous Approach */
"0DW03UZ"  =  "1"  /* Revision of Feeding Device in Upper Intestinal Tract, Percutaneous Approach */
"0DW040Z"  =  "1"  /* Revision of Drainage Device in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW042Z"  =  "1"  /* Revision of Monitoring Device in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW043Z"  =  "1"  /* Revision of Infusion Device in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW047Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW04CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW04DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW04JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW04KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Percutaneous Endoscopic Approach */
"0DW070Z"  =  "1"  /* Revision of Drainage Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW072Z"  =  "1"  /* Revision of Monitoring Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW073Z"  =  "1"  /* Revision of Infusion Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW077Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW07CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW07DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW07JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW07KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW07UZ"  =  "1"  /* Revision of Feeding Device in Upper Intestinal Tract, Via Natural or Artificial Opening */
"0DW080Z"  =  "1"  /* Revision of Drainage Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW082Z"  =  "1"  /* Revision of Monitoring Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW083Z"  =  "1"  /* Revision of Infusion Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW087Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW08CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW08DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW08JZ"  =  "1"  /* Revision of Synthetic Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW08KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW08UZ"  =  "1"  /* Revision of Feeding Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DW600Z"  =  "1"  /* Revision of Drainage Device in Stomach, Open Approach */
"0DW602Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Open Approach */
"0DW603Z"  =  "1"  /* Revision of Infusion Device in Stomach, Open Approach */
"0DW607Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Stomach, Open Approach */
"0DW60CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Open Approach */
"0DW60DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Open Approach */
"0DW60JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Open Approach */
"0DW60KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Stomach, Open Approach */
"0DW60MZ"  =  "1"  /* Revision of Stimulator Lead in Stomach, Open Approach */
"0DW60UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Open Approach */
"0DW630Z"  =  "1"  /* Revision of Drainage Device in Stomach, Percutaneous Approach */
"0DW632Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Percutaneous Approach */
"0DW633Z"  =  "1"  /* Revision of Infusion Device in Stomach, Percutaneous Approach */
"0DW637Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Stomach, Percutaneous Approach */
"0DW63CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Percutaneous Approach */
"0DW63DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Percutaneous Approach */
"0DW63JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Percutaneous Approach */
"0DW63KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Stomach, Percutaneous Approach */
"0DW63MZ"  =  "1"  /* Revision of Stimulator Lead in Stomach, Percutaneous Approach */
"0DW63UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Percutaneous Approach */
"0DW640Z"  =  "1"  /* Revision of Drainage Device in Stomach, Percutaneous Endoscopic Approach */
"0DW642Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Percutaneous Endoscopic Approach */
"0DW647Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Stomach, Percutaneous Endoscopic Approach */
"0DW64DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Percutaneous Endoscopic Approach */
"0DW64JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Percutaneous Endoscopic Approach */
"0DW64KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Stomach, Percutaneous Endoscopic Approach */
"0DW64MZ"  =  "1"  /* Revision of Stimulator Lead in Stomach, Percutaneous Endoscopic Approach */
"0DW64UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Percutaneous Endoscopic Approach */
"0DW670Z"  =  "1"  /* Revision of Drainage Device in Stomach, Via Natural or Artificial Opening */
"0DW672Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Via Natural or Artificial Opening */
"0DW673Z"  =  "1"  /* Revision of Infusion Device in Stomach, Via Natural or Artificial Opening */
"0DW677Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Stomach, Via Natural or Artificial Opening */
"0DW67CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Via Natural or Artificial Opening */
"0DW67DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Via Natural or Artificial Opening */
"0DW67JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Via Natural or Artificial Opening */
"0DW67KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Stomach, Via Natural or Artificial Opening */
"0DW67UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Via Natural or Artificial Opening */
"0DW680Z"  =  "1"  /* Revision of Drainage Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW682Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW683Z"  =  "1"  /* Revision of Infusion Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW687Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW68CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW68DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW68JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW68KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW68UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Via Natural or Artificial Opening Endoscopic */
"0DW807Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Small Intestine, Open Approach */
"0DW80JZ"  =  "1"  /* Revision of Synthetic Substitute in Small Intestine, Open Approach */
"0DW80KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Small Intestine, Open Approach */
"0DW847Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Small Intestine, Percutaneous Endoscopic Approach */
"0DW84JZ"  =  "1"  /* Revision of Synthetic Substitute in Small Intestine, Percutaneous Endoscopic Approach */
"0DW84KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Small Intestine, Percutaneous Endoscopic Approach */
"0DW877Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Small Intestine, Via Natural or Artificial Opening */
"0DW87JZ"  =  "1"  /* Revision of Synthetic Substitute in Small Intestine, Via Natural or Artificial Opening */
"0DW87KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Small Intestine, Via Natural or Artificial Opening */
"0DW887Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DW88JZ"  =  "1"  /* Revision of Synthetic Substitute in Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DW88KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Small Intestine, Via Natural or Artificial Opening Endoscopic */
"0DWD00Z"  =  "1"  /* Revision of Drainage Device in Lower Intestinal Tract, Open Approach */
"0DWD02Z"  =  "1"  /* Revision of Monitoring Device in Lower Intestinal Tract, Open Approach */
"0DWD03Z"  =  "1"  /* Revision of Infusion Device in Lower Intestinal Tract, Open Approach */
"0DWD07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Open Approach */
"0DWD0CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Intestinal Tract, Open Approach */
"0DWD0DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Intestinal Tract, Open Approach */
"0DWD0JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Intestinal Tract, Open Approach */
"0DWD0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Open Approach */
"0DWD0UZ"  =  "1"  /* Revision of Feeding Device in Lower Intestinal Tract, Open Approach */
"0DWD30Z"  =  "1"  /* Revision of Drainage Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD32Z"  =  "1"  /* Revision of Monitoring Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD33Z"  =  "1"  /* Revision of Infusion Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Percutaneous Approach */
"0DWD3CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD3DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD3JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Intestinal Tract, Percutaneous Approach */
"0DWD3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Percutaneous Approach */
"0DWD3UZ"  =  "1"  /* Revision of Feeding Device in Lower Intestinal Tract, Percutaneous Approach */
"0DWD40Z"  =  "1"  /* Revision of Drainage Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD42Z"  =  "1"  /* Revision of Monitoring Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD43Z"  =  "1"  /* Revision of Infusion Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD4CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD4DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD4JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD4UZ"  =  "1"  /* Revision of Feeding Device in Lower Intestinal Tract, Percutaneous Endoscopic Approach */
"0DWD70Z"  =  "1"  /* Revision of Drainage Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD72Z"  =  "1"  /* Revision of Monitoring Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD73Z"  =  "1"  /* Revision of Infusion Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD7CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD7DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD7JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD7UZ"  =  "1"  /* Revision of Feeding Device in Lower Intestinal Tract, Via Natural or Artificial Opening */
"0DWD80Z"  =  "1"  /* Revision of Drainage Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD82Z"  =  "1"  /* Revision of Monitoring Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD83Z"  =  "1"  /* Revision of Infusion Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD8CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD8DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD8JZ"  =  "1"  /* Revision of Synthetic Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWD8UZ"  =  "1"  /* Revision of Feeding Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic */
"0DWE07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Large Intestine, Open Approach */
"0DWE0JZ"  =  "1"  /* Revision of Synthetic Substitute in Large Intestine, Open Approach */
"0DWE0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Large Intestine, Open Approach */
"0DWE47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Large Intestine, Percutaneous Endoscopic Approach */
"0DWE4JZ"  =  "1"  /* Revision of Synthetic Substitute in Large Intestine, Percutaneous Endoscopic Approach */
"0DWE4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Large Intestine, Percutaneous Endoscopic Approach */
"0DWE77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Large Intestine, Via Natural or Artificial Opening */
"0DWE7JZ"  =  "1"  /* Revision of Synthetic Substitute in Large Intestine, Via Natural or Artificial Opening */
"0DWE7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Large Intestine, Via Natural or Artificial Opening */
"0DWE87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DWE8JZ"  =  "1"  /* Revision of Synthetic Substitute in Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DWE8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Large Intestine, Via Natural or Artificial Opening Endoscopic */
"0DWU07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Omentum, Open Approach */
"0DWU0JZ"  =  "1"  /* Revision of Synthetic Substitute in Omentum, Open Approach */
"0DWU0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Omentum, Open Approach */
"0DWU37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Omentum, Percutaneous Approach */
"0DWU3JZ"  =  "1"  /* Revision of Synthetic Substitute in Omentum, Percutaneous Approach */
"0DWU3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Omentum, Percutaneous Approach */
"0DWU47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Omentum, Percutaneous Endoscopic Approach */
"0DWU4JZ"  =  "1"  /* Revision of Synthetic Substitute in Omentum, Percutaneous Endoscopic Approach */
"0DWU4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Omentum, Percutaneous Endoscopic Approach */
"0DWV07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mesentery, Open Approach */
"0DWV0JZ"  =  "1"  /* Revision of Synthetic Substitute in Mesentery, Open Approach */
"0DWV0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mesentery, Open Approach */
"0DWV37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mesentery, Percutaneous Approach */
"0DWV3JZ"  =  "1"  /* Revision of Synthetic Substitute in Mesentery, Percutaneous Approach */
"0DWV3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mesentery, Percutaneous Approach */
"0DWV47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Mesentery, Percutaneous Endoscopic Approach */
"0DWV4JZ"  =  "1"  /* Revision of Synthetic Substitute in Mesentery, Percutaneous Endoscopic Approach */
"0DWV4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Mesentery, Percutaneous Endoscopic Approach */
"0DWW07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Peritoneum, Open Approach */
"0DWW0JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneum, Open Approach */
"0DWW0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Peritoneum, Open Approach */
"0DWW37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Peritoneum, Percutaneous Approach */
"0DWW3JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneum, Percutaneous Approach */
"0DWW3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Peritoneum, Percutaneous Approach */
"0DWW47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Peritoneum, Percutaneous Endoscopic Approach */
"0DWW4JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneum, Percutaneous Endoscopic Approach */
"0DWW4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Peritoneum, Percutaneous Endoscopic Approach */
"0DX80Z5"  =  "1"  /* Transfer Small Intestine to Esophagus, Open Approach */
"0DX84Z5"  =  "1"  /* Transfer Small Intestine to Esophagus, Percutaneous Endoscopic Approach */
"0DXE0Z5"  =  "1"  /* Transfer Large Intestine to Esophagus, Open Approach */
"0DXE4Z5"  =  "1"  /* Transfer Large Intestine to Esophagus, Percutaneous Endoscopic Approach */
"0DY60Z0"  =  "1"  /* Transplantation of Stomach, Allogeneic, Open Approach */
"0DY60Z1"  =  "1"  /* Transplantation of Stomach, Syngeneic, Open Approach */
"0DY60Z2"  =  "1"  /* Transplantation of Stomach, Zooplastic, Open Approach */
"0F140D3"  =  "1"  /* Bypass Gallbladder to Duodenum with Intraluminal Device, Open Approach */
"0F140D4"  =  "1"  /* Bypass Gallbladder to Stomach with Intraluminal Device, Open Approach */
"0F140D5"  =  "1"  /* Bypass Gallbladder to Right Hepatic Duct with Intraluminal Device, Open Approach */
"0F140D6"  =  "1"  /* Bypass Gallbladder to Left Hepatic Duct with Intraluminal Device, Open Approach */
"0F140D7"  =  "1"  /* Bypass Gallbladder to Caudate Hepatic Duct with Intraluminal Device, Open Approach */
"0F140D8"  =  "1"  /* Bypass Gallbladder to Cystic Duct with Intraluminal Device, Open Approach */
"0F140D9"  =  "1"  /* Bypass Gallbladder to Common Bile Duct with Intraluminal Device, Open Approach */
"0F140DB"  =  "1"  /* Bypass Gallbladder to Small Intestine with Intraluminal Device, Open Approach */
"0F140Z3"  =  "1"  /* Bypass Gallbladder to Duodenum, Open Approach */
"0F140Z4"  =  "1"  /* Bypass Gallbladder to Stomach, Open Approach */
"0F140Z5"  =  "1"  /* Bypass Gallbladder to Right Hepatic Duct, Open Approach */
"0F140Z6"  =  "1"  /* Bypass Gallbladder to Left Hepatic Duct, Open Approach */
"0F140Z7"  =  "1"  /* Bypass Gallbladder to Caudate Hepatic Duct, Open Approach */
"0F140Z8"  =  "1"  /* Bypass Gallbladder to Cystic Duct, Open Approach */
"0F140Z9"  =  "1"  /* Bypass Gallbladder to Common Bile Duct, Open Approach */
"0F140ZB"  =  "1"  /* Bypass Gallbladder to Small Intestine, Open Approach */
"0F144D3"  =  "1"  /* Bypass Gallbladder to Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F144D4"  =  "1"  /* Bypass Gallbladder to Stomach with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F144D5"  =  "1"  /* Bypass Gallbladder to Right Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F144D6"  =  "1"  /* Bypass Gallbladder to Left Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F144D7"  =  "1"  /* Bypass Gallbladder to Caudate Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F144D8"  =  "1"  /* Bypass Gallbladder to Cystic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F144D9"  =  "1"  /* Bypass Gallbladder to Common Bile Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F144DB"  =  "1"  /* Bypass Gallbladder to Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F144Z3"  =  "1"  /* Bypass Gallbladder to Duodenum, Percutaneous Endoscopic Approach */
"0F144Z4"  =  "1"  /* Bypass Gallbladder to Stomach, Percutaneous Endoscopic Approach */
"0F144Z5"  =  "1"  /* Bypass Gallbladder to Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0F144Z6"  =  "1"  /* Bypass Gallbladder to Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0F144Z7"  =  "1"  /* Bypass Gallbladder to Caudate Hepatic Duct, Percutaneous Endoscopic Approach */
"0F144Z8"  =  "1"  /* Bypass Gallbladder to Cystic Duct, Percutaneous Endoscopic Approach */
"0F144Z9"  =  "1"  /* Bypass Gallbladder to Common Bile Duct, Percutaneous Endoscopic Approach */
"0F144ZB"  =  "1"  /* Bypass Gallbladder to Small Intestine, Percutaneous Endoscopic Approach */
"0F150D3"  =  "1"  /* Bypass Right Hepatic Duct to Duodenum with Intraluminal Device, Open Approach */
"0F150D4"  =  "1"  /* Bypass Right Hepatic Duct to Stomach with Intraluminal Device, Open Approach */
"0F150D5"  =  "1"  /* Bypass Right Hepatic Duct to Right Hepatic Duct with Intraluminal Device, Open Approach */
"0F150D6"  =  "1"  /* Bypass Right Hepatic Duct to Left Hepatic Duct with Intraluminal Device, Open Approach */
"0F150D7"  =  "1"  /* Bypass Right Hepatic Duct to Caudate Hepatic Duct with Intraluminal Device, Open Approach */
"0F150D8"  =  "1"  /* Bypass Right Hepatic Duct to Cystic Duct with Intraluminal Device, Open Approach */
"0F150D9"  =  "1"  /* Bypass Right Hepatic Duct to Common Bile Duct with Intraluminal Device, Open Approach */
"0F150DB"  =  "1"  /* Bypass Right Hepatic Duct to Small Intestine with Intraluminal Device, Open Approach */
"0F150Z3"  =  "1"  /* Bypass Right Hepatic Duct to Duodenum, Open Approach */
"0F150Z4"  =  "1"  /* Bypass Right Hepatic Duct to Stomach, Open Approach */
"0F150Z5"  =  "1"  /* Bypass Right Hepatic Duct to Right Hepatic Duct, Open Approach */
"0F150Z6"  =  "1"  /* Bypass Right Hepatic Duct to Left Hepatic Duct, Open Approach */
"0F150Z7"  =  "1"  /* Bypass Right Hepatic Duct to Caudate Hepatic Duct, Open Approach */
"0F150Z8"  =  "1"  /* Bypass Right Hepatic Duct to Cystic Duct, Open Approach */
"0F150Z9"  =  "1"  /* Bypass Right Hepatic Duct to Common Bile Duct, Open Approach */
"0F150ZB"  =  "1"  /* Bypass Right Hepatic Duct to Small Intestine, Open Approach */
"0F154D3"  =  "1"  /* Bypass Right Hepatic Duct to Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F154D4"  =  "1"  /* Bypass Right Hepatic Duct to Stomach with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F154D5"  =  "1"  /* Bypass Right Hepatic Duct to Right Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F154D6"  =  "1"  /* Bypass Right Hepatic Duct to Left Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F154D7"  =  "1"  /* Bypass Right Hepatic Duct to Caudate Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F154D8"  =  "1"  /* Bypass Right Hepatic Duct to Cystic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F154D9"  =  "1"  /* Bypass Right Hepatic Duct to Common Bile Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F154DB"  =  "1"  /* Bypass Right Hepatic Duct to Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F154Z3"  =  "1"  /* Bypass Right Hepatic Duct to Duodenum, Percutaneous Endoscopic Approach */
"0F154Z4"  =  "1"  /* Bypass Right Hepatic Duct to Stomach, Percutaneous Endoscopic Approach */
"0F154Z5"  =  "1"  /* Bypass Right Hepatic Duct to Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0F154Z6"  =  "1"  /* Bypass Right Hepatic Duct to Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0F154Z7"  =  "1"  /* Bypass Right Hepatic Duct to Caudate Hepatic Duct, Percutaneous Endoscopic Approach */
"0F154Z8"  =  "1"  /* Bypass Right Hepatic Duct to Cystic Duct, Percutaneous Endoscopic Approach */
"0F154Z9"  =  "1"  /* Bypass Right Hepatic Duct to Common Bile Duct, Percutaneous Endoscopic Approach */
"0F154ZB"  =  "1"  /* Bypass Right Hepatic Duct to Small Intestine, Percutaneous Endoscopic Approach */
"0F160D3"  =  "1"  /* Bypass Left Hepatic Duct to Duodenum with Intraluminal Device, Open Approach */
"0F160D4"  =  "1"  /* Bypass Left Hepatic Duct to Stomach with Intraluminal Device, Open Approach */
"0F160D5"  =  "1"  /* Bypass Left Hepatic Duct to Right Hepatic Duct with Intraluminal Device, Open Approach */
"0F160D6"  =  "1"  /* Bypass Left Hepatic Duct to Left Hepatic Duct with Intraluminal Device, Open Approach */
"0F160D7"  =  "1"  /* Bypass Left Hepatic Duct to Caudate Hepatic Duct with Intraluminal Device, Open Approach */
"0F160D8"  =  "1"  /* Bypass Left Hepatic Duct to Cystic Duct with Intraluminal Device, Open Approach */
"0F160D9"  =  "1"  /* Bypass Left Hepatic Duct to Common Bile Duct with Intraluminal Device, Open Approach */
"0F160DB"  =  "1"  /* Bypass Left Hepatic Duct to Small Intestine with Intraluminal Device, Open Approach */
"0F160Z3"  =  "1"  /* Bypass Left Hepatic Duct to Duodenum, Open Approach */
"0F160Z4"  =  "1"  /* Bypass Left Hepatic Duct to Stomach, Open Approach */
"0F160Z5"  =  "1"  /* Bypass Left Hepatic Duct to Right Hepatic Duct, Open Approach */
"0F160Z6"  =  "1"  /* Bypass Left Hepatic Duct to Left Hepatic Duct, Open Approach */
"0F160Z7"  =  "1"  /* Bypass Left Hepatic Duct to Caudate Hepatic Duct, Open Approach */
"0F160Z8"  =  "1"  /* Bypass Left Hepatic Duct to Cystic Duct, Open Approach */
"0F160Z9"  =  "1"  /* Bypass Left Hepatic Duct to Common Bile Duct, Open Approach */
"0F160ZB"  =  "1"  /* Bypass Left Hepatic Duct to Small Intestine, Open Approach */
"0F164D3"  =  "1"  /* Bypass Left Hepatic Duct to Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F164D4"  =  "1"  /* Bypass Left Hepatic Duct to Stomach with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F164D5"  =  "1"  /* Bypass Left Hepatic Duct to Right Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F164D6"  =  "1"  /* Bypass Left Hepatic Duct to Left Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F164D7"  =  "1"  /* Bypass Left Hepatic Duct to Caudate Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F164D8"  =  "1"  /* Bypass Left Hepatic Duct to Cystic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F164D9"  =  "1"  /* Bypass Left Hepatic Duct to Common Bile Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F164DB"  =  "1"  /* Bypass Left Hepatic Duct to Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F164Z3"  =  "1"  /* Bypass Left Hepatic Duct to Duodenum, Percutaneous Endoscopic Approach */
"0F164Z4"  =  "1"  /* Bypass Left Hepatic Duct to Stomach, Percutaneous Endoscopic Approach */
"0F164Z5"  =  "1"  /* Bypass Left Hepatic Duct to Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0F164Z6"  =  "1"  /* Bypass Left Hepatic Duct to Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0F164Z7"  =  "1"  /* Bypass Left Hepatic Duct to Caudate Hepatic Duct, Percutaneous Endoscopic Approach */
"0F164Z8"  =  "1"  /* Bypass Left Hepatic Duct to Cystic Duct, Percutaneous Endoscopic Approach */
"0F164Z9"  =  "1"  /* Bypass Left Hepatic Duct to Common Bile Duct, Percutaneous Endoscopic Approach */
"0F164ZB"  =  "1"  /* Bypass Left Hepatic Duct to Small Intestine, Percutaneous Endoscopic Approach */
"0F180D3"  =  "1"  /* Bypass Cystic Duct to Duodenum with Intraluminal Device, Open Approach */
"0F180D4"  =  "1"  /* Bypass Cystic Duct to Stomach with Intraluminal Device, Open Approach */
"0F180D5"  =  "1"  /* Bypass Cystic Duct to Right Hepatic Duct with Intraluminal Device, Open Approach */
"0F180D6"  =  "1"  /* Bypass Cystic Duct to Left Hepatic Duct with Intraluminal Device, Open Approach */
"0F180D7"  =  "1"  /* Bypass Cystic Duct to Caudate Hepatic Duct with Intraluminal Device, Open Approach */
"0F180D8"  =  "1"  /* Bypass Cystic Duct to Cystic Duct with Intraluminal Device, Open Approach */
"0F180D9"  =  "1"  /* Bypass Cystic Duct to Common Bile Duct with Intraluminal Device, Open Approach */
"0F180DB"  =  "1"  /* Bypass Cystic Duct to Small Intestine with Intraluminal Device, Open Approach */
"0F180Z3"  =  "1"  /* Bypass Cystic Duct to Duodenum, Open Approach */
"0F180Z4"  =  "1"  /* Bypass Cystic Duct to Stomach, Open Approach */
"0F180Z5"  =  "1"  /* Bypass Cystic Duct to Right Hepatic Duct, Open Approach */
"0F180Z6"  =  "1"  /* Bypass Cystic Duct to Left Hepatic Duct, Open Approach */
"0F180Z7"  =  "1"  /* Bypass Cystic Duct to Caudate Hepatic Duct, Open Approach */
"0F180Z8"  =  "1"  /* Bypass Cystic Duct to Cystic Duct, Open Approach */
"0F180Z9"  =  "1"  /* Bypass Cystic Duct to Common Bile Duct, Open Approach */
"0F180ZB"  =  "1"  /* Bypass Cystic Duct to Small Intestine, Open Approach */
"0F184D3"  =  "1"  /* Bypass Cystic Duct to Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F184D4"  =  "1"  /* Bypass Cystic Duct to Stomach with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F184D5"  =  "1"  /* Bypass Cystic Duct to Right Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F184D6"  =  "1"  /* Bypass Cystic Duct to Left Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F184D7"  =  "1"  /* Bypass Cystic Duct to Caudate Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F184D8"  =  "1"  /* Bypass Cystic Duct to Cystic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F184D9"  =  "1"  /* Bypass Cystic Duct to Common Bile Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F184DB"  =  "1"  /* Bypass Cystic Duct to Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F184Z3"  =  "1"  /* Bypass Cystic Duct to Duodenum, Percutaneous Endoscopic Approach */
"0F184Z4"  =  "1"  /* Bypass Cystic Duct to Stomach, Percutaneous Endoscopic Approach */
"0F184Z5"  =  "1"  /* Bypass Cystic Duct to Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0F184Z6"  =  "1"  /* Bypass Cystic Duct to Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0F184Z7"  =  "1"  /* Bypass Cystic Duct to Caudate Hepatic Duct, Percutaneous Endoscopic Approach */
"0F184Z8"  =  "1"  /* Bypass Cystic Duct to Cystic Duct, Percutaneous Endoscopic Approach */
"0F184Z9"  =  "1"  /* Bypass Cystic Duct to Common Bile Duct, Percutaneous Endoscopic Approach */
"0F184ZB"  =  "1"  /* Bypass Cystic Duct to Small Intestine, Percutaneous Endoscopic Approach */
"0F190D3"  =  "1"  /* Bypass Common Bile Duct to Duodenum with Intraluminal Device, Open Approach */
"0F190D4"  =  "1"  /* Bypass Common Bile Duct to Stomach with Intraluminal Device, Open Approach */
"0F190D5"  =  "1"  /* Bypass Common Bile Duct to Right Hepatic Duct with Intraluminal Device, Open Approach */
"0F190D6"  =  "1"  /* Bypass Common Bile Duct to Left Hepatic Duct with Intraluminal Device, Open Approach */
"0F190D7"  =  "1"  /* Bypass Common Bile Duct to Caudate Hepatic Duct with Intraluminal Device, Open Approach */
"0F190D8"  =  "1"  /* Bypass Common Bile Duct to Cystic Duct with Intraluminal Device, Open Approach */
"0F190D9"  =  "1"  /* Bypass Common Bile Duct to Common Bile Duct with Intraluminal Device, Open Approach */
"0F190DB"  =  "1"  /* Bypass Common Bile Duct to Small Intestine with Intraluminal Device, Open Approach */
"0F190Z3"  =  "1"  /* Bypass Common Bile Duct to Duodenum, Open Approach */
"0F190Z4"  =  "1"  /* Bypass Common Bile Duct to Stomach, Open Approach */
"0F190Z5"  =  "1"  /* Bypass Common Bile Duct to Right Hepatic Duct, Open Approach */
"0F190Z6"  =  "1"  /* Bypass Common Bile Duct to Left Hepatic Duct, Open Approach */
"0F190Z7"  =  "1"  /* Bypass Common Bile Duct to Caudate Hepatic Duct, Open Approach */
"0F190Z8"  =  "1"  /* Bypass Common Bile Duct to Cystic Duct, Open Approach */
"0F190Z9"  =  "1"  /* Bypass Common Bile Duct to Common Bile Duct, Open Approach */
"0F190ZB"  =  "1"  /* Bypass Common Bile Duct to Small Intestine, Open Approach */
"0F194D3"  =  "1"  /* Bypass Common Bile Duct to Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F194D4"  =  "1"  /* Bypass Common Bile Duct to Stomach with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F194D5"  =  "1"  /* Bypass Common Bile Duct to Right Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F194D6"  =  "1"  /* Bypass Common Bile Duct to Left Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F194D7"  =  "1"  /* Bypass Common Bile Duct to Caudate Hepatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F194D8"  =  "1"  /* Bypass Common Bile Duct to Cystic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F194D9"  =  "1"  /* Bypass Common Bile Duct to Common Bile Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F194DB"  =  "1"  /* Bypass Common Bile Duct to Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F194Z3"  =  "1"  /* Bypass Common Bile Duct to Duodenum, Percutaneous Endoscopic Approach */
"0F194Z4"  =  "1"  /* Bypass Common Bile Duct to Stomach, Percutaneous Endoscopic Approach */
"0F194Z5"  =  "1"  /* Bypass Common Bile Duct to Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0F194Z6"  =  "1"  /* Bypass Common Bile Duct to Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0F194Z7"  =  "1"  /* Bypass Common Bile Duct to Caudate Hepatic Duct, Percutaneous Endoscopic Approach */
"0F194Z8"  =  "1"  /* Bypass Common Bile Duct to Cystic Duct, Percutaneous Endoscopic Approach */
"0F194Z9"  =  "1"  /* Bypass Common Bile Duct to Common Bile Duct, Percutaneous Endoscopic Approach */
"0F194ZB"  =  "1"  /* Bypass Common Bile Duct to Small Intestine, Percutaneous Endoscopic Approach */
"0F1D0D3"  =  "1"  /* Bypass Pancreatic Duct to Duodenum with Intraluminal Device, Open Approach */
"0F1D0DB"  =  "1"  /* Bypass Pancreatic Duct to Small Intestine with Intraluminal Device, Open Approach */
"0F1D0DC"  =  "1"  /* Bypass Pancreatic Duct to Large Intestine with Intraluminal Device, Open Approach */
"0F1D0Z3"  =  "1"  /* Bypass Pancreatic Duct to Duodenum, Open Approach */
"0F1D0ZB"  =  "1"  /* Bypass Pancreatic Duct to Small Intestine, Open Approach */
"0F1D0ZC"  =  "1"  /* Bypass Pancreatic Duct to Large Intestine, Open Approach */
"0F1D4D3"  =  "1"  /* Bypass Pancreatic Duct to Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1D4DB"  =  "1"  /* Bypass Pancreatic Duct to Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1D4DC"  =  "1"  /* Bypass Pancreatic Duct to Large Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1D4Z3"  =  "1"  /* Bypass Pancreatic Duct to Duodenum, Percutaneous Endoscopic Approach */
"0F1D4ZB"  =  "1"  /* Bypass Pancreatic Duct to Small Intestine, Percutaneous Endoscopic Approach */
"0F1D4ZC"  =  "1"  /* Bypass Pancreatic Duct to Large Intestine, Percutaneous Endoscopic Approach */
"0F1F0D3"  =  "1"  /* Bypass Accessory Pancreatic Duct to Duodenum with Intraluminal Device, Open Approach */
"0F1F0DB"  =  "1"  /* Bypass Accessory Pancreatic Duct to Small Intestine with Intraluminal Device, Open Approach */
"0F1F0DC"  =  "1"  /* Bypass Accessory Pancreatic Duct to Large Intestine with Intraluminal Device, Open Approach */
"0F1F0Z3"  =  "1"  /* Bypass Accessory Pancreatic Duct to Duodenum, Open Approach */
"0F1F0ZB"  =  "1"  /* Bypass Accessory Pancreatic Duct to Small Intestine, Open Approach */
"0F1F0ZC"  =  "1"  /* Bypass Accessory Pancreatic Duct to Large Intestine, Open Approach */
"0F1F4D3"  =  "1"  /* Bypass Accessory Pancreatic Duct to Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1F4DB"  =  "1"  /* Bypass Accessory Pancreatic Duct to Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1F4DC"  =  "1"  /* Bypass Accessory Pancreatic Duct to Large Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1F4Z3"  =  "1"  /* Bypass Accessory Pancreatic Duct to Duodenum, Percutaneous Endoscopic Approach */
"0F1F4ZB"  =  "1"  /* Bypass Accessory Pancreatic Duct to Small Intestine, Percutaneous Endoscopic Approach */
"0F1F4ZC"  =  "1"  /* Bypass Accessory Pancreatic Duct to Large Intestine, Percutaneous Endoscopic Approach */
"0F1G0D3"  =  "1"  /* Bypass Pancreas to Duodenum with Intraluminal Device, Open Approach */
"0F1G0DB"  =  "1"  /* Bypass Pancreas to Small Intestine with Intraluminal Device, Open Approach */
"0F1G0DC"  =  "1"  /* Bypass Pancreas to Large Intestine with Intraluminal Device, Open Approach */
"0F1G0Z3"  =  "1"  /* Bypass Pancreas to Duodenum, Open Approach */
"0F1G0ZB"  =  "1"  /* Bypass Pancreas to Small Intestine, Open Approach */
"0F1G0ZC"  =  "1"  /* Bypass Pancreas to Large Intestine, Open Approach */
"0F1G4D3"  =  "1"  /* Bypass Pancreas to Duodenum with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1G4DB"  =  "1"  /* Bypass Pancreas to Small Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1G4DC"  =  "1"  /* Bypass Pancreas to Large Intestine with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F1G4Z3"  =  "1"  /* Bypass Pancreas to Duodenum, Percutaneous Endoscopic Approach */
"0F1G4ZB"  =  "1"  /* Bypass Pancreas to Small Intestine, Percutaneous Endoscopic Approach */
"0F1G4ZC"  =  "1"  /* Bypass Pancreas to Large Intestine, Percutaneous Endoscopic Approach */
"0F500ZZ"  =  "1"  /* Destruction of Liver, Open Approach */
"0F503ZZ"  =  "1"  /* Destruction of Liver, Percutaneous Approach */
"0F504ZZ"  =  "1"  /* Destruction of Liver, Percutaneous Endoscopic Approach */
"0F510ZZ"  =  "1"  /* Destruction of Right Lobe Liver, Open Approach */
"0F513ZZ"  =  "1"  /* Destruction of Right Lobe Liver, Percutaneous Approach */
"0F514ZZ"  =  "1"  /* Destruction of Right Lobe Liver, Percutaneous Endoscopic Approach */
"0F520ZZ"  =  "1"  /* Destruction of Left Lobe Liver, Open Approach */
"0F523ZZ"  =  "1"  /* Destruction of Left Lobe Liver, Percutaneous Approach */
"0F524ZZ"  =  "1"  /* Destruction of Left Lobe Liver, Percutaneous Endoscopic Approach */
"0F540ZZ"  =  "1"  /* Destruction of Gallbladder, Open Approach */
"0F543ZZ"  =  "1"  /* Destruction of Gallbladder, Percutaneous Approach */
"0F550ZZ"  =  "1"  /* Destruction of Right Hepatic Duct, Open Approach */
"0F553ZZ"  =  "1"  /* Destruction of Right Hepatic Duct, Percutaneous Approach */
"0F557ZZ"  =  "1"  /* Destruction of Right Hepatic Duct, Via Natural or Artificial Opening */
"0F560ZZ"  =  "1"  /* Destruction of Left Hepatic Duct, Open Approach */
"0F563ZZ"  =  "1"  /* Destruction of Left Hepatic Duct, Percutaneous Approach */
"0F567ZZ"  =  "1"  /* Destruction of Left Hepatic Duct, Via Natural or Artificial Opening */
"0F580ZZ"  =  "1"  /* Destruction of Cystic Duct, Open Approach */
"0F583ZZ"  =  "1"  /* Destruction of Cystic Duct, Percutaneous Approach */
"0F587ZZ"  =  "1"  /* Destruction of Cystic Duct, Via Natural or Artificial Opening */
"0F590ZZ"  =  "1"  /* Destruction of Common Bile Duct, Open Approach */
"0F593ZZ"  =  "1"  /* Destruction of Common Bile Duct, Percutaneous Approach */
"0F597ZZ"  =  "1"  /* Destruction of Common Bile Duct, Via Natural or Artificial Opening */
"0F5C0ZZ"  =  "1"  /* Destruction of Ampulla of Vater, Open Approach */
"0F5C3ZZ"  =  "1"  /* Destruction of Ampulla of Vater, Percutaneous Approach */
"0F5C7ZZ"  =  "1"  /* Destruction of Ampulla of Vater, Via Natural or Artificial Opening */
"0F5D0ZZ"  =  "1"  /* Destruction of Pancreatic Duct, Open Approach */
"0F5D3ZZ"  =  "1"  /* Destruction of Pancreatic Duct, Percutaneous Approach */
"0F5D7ZZ"  =  "1"  /* Destruction of Pancreatic Duct, Via Natural or Artificial Opening */
"0F5F0ZZ"  =  "1"  /* Destruction of Accessory Pancreatic Duct, Open Approach */
"0F5F3ZZ"  =  "1"  /* Destruction of Accessory Pancreatic Duct, Percutaneous Approach */
"0F5F7ZZ"  =  "1"  /* Destruction of Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0F5G0ZZ"  =  "1"  /* Destruction of Pancreas, Open Approach */
"0F5G3ZZ"  =  "1"  /* Destruction of Pancreas, Percutaneous Approach */
"0F750DZ"  =  "1"  /* Dilation of Right Hepatic Duct with Intraluminal Device, Open Approach */
"0F750ZZ"  =  "1"  /* Dilation of Right Hepatic Duct, Open Approach */
"0F757DZ"  =  "1"  /* Dilation of Right Hepatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0F757ZZ"  =  "1"  /* Dilation of Right Hepatic Duct, Via Natural or Artificial Opening */
"0F760DZ"  =  "1"  /* Dilation of Left Hepatic Duct with Intraluminal Device, Open Approach */
"0F760ZZ"  =  "1"  /* Dilation of Left Hepatic Duct, Open Approach */
"0F767DZ"  =  "1"  /* Dilation of Left Hepatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0F767ZZ"  =  "1"  /* Dilation of Left Hepatic Duct, Via Natural or Artificial Opening */
"0F780DZ"  =  "1"  /* Dilation of Cystic Duct with Intraluminal Device, Open Approach */
"0F780ZZ"  =  "1"  /* Dilation of Cystic Duct, Open Approach */
"0F787DZ"  =  "1"  /* Dilation of Cystic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0F787ZZ"  =  "1"  /* Dilation of Cystic Duct, Via Natural or Artificial Opening */
"0F790DZ"  =  "1"  /* Dilation of Common Bile Duct with Intraluminal Device, Open Approach */
"0F790ZZ"  =  "1"  /* Dilation of Common Bile Duct, Open Approach */
"0F797DZ"  =  "1"  /* Dilation of Common Bile Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0F797ZZ"  =  "1"  /* Dilation of Common Bile Duct, Via Natural or Artificial Opening */
"0F7C0DZ"  =  "1"  /* Dilation of Ampulla of Vater with Intraluminal Device, Open Approach */
"0F7C0ZZ"  =  "1"  /* Dilation of Ampulla of Vater, Open Approach */
"0F7C3DZ"  =  "1"  /* Dilation of Ampulla of Vater with Intraluminal Device, Percutaneous Approach */
"0F7C3ZZ"  =  "1"  /* Dilation of Ampulla of Vater, Percutaneous Approach */
"0F7C4DZ"  =  "1"  /* Dilation of Ampulla of Vater with Intraluminal Device, Percutaneous Endoscopic Approach */
"0F7C4ZZ"  =  "1"  /* Dilation of Ampulla of Vater, Percutaneous Endoscopic Approach */
"0F7C7DZ"  =  "1"  /* Dilation of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening */
"0F7C7ZZ"  =  "1"  /* Dilation of Ampulla of Vater, Via Natural or Artificial Opening */
"0F7D0DZ"  =  "1"  /* Dilation of Pancreatic Duct with Intraluminal Device, Open Approach */
"0F7D0ZZ"  =  "1"  /* Dilation of Pancreatic Duct, Open Approach */
"0F7D3DZ"  =  "1"  /* Dilation of Pancreatic Duct with Intraluminal Device, Percutaneous Approach */
"0F7D3ZZ"  =  "1"  /* Dilation of Pancreatic Duct, Percutaneous Approach */
"0F7D7DZ"  =  "1"  /* Dilation of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0F7D7ZZ"  =  "1"  /* Dilation of Pancreatic Duct, Via Natural or Artificial Opening */
"0F7F0DZ"  =  "1"  /* Dilation of Accessory Pancreatic Duct with Intraluminal Device, Open Approach */
"0F7F0ZZ"  =  "1"  /* Dilation of Accessory Pancreatic Duct, Open Approach */
"0F7F3DZ"  =  "1"  /* Dilation of Accessory Pancreatic Duct with Intraluminal Device, Percutaneous Approach */
"0F7F3ZZ"  =  "1"  /* Dilation of Accessory Pancreatic Duct, Percutaneous Approach */
"0F7F7DZ"  =  "1"  /* Dilation of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0F7F7ZZ"  =  "1"  /* Dilation of Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0F8G0ZZ"  =  "1"  /* Division of Pancreas, Open Approach */
"0F8G3ZZ"  =  "1"  /* Division of Pancreas, Percutaneous Approach */
"0F8G4ZZ"  =  "1"  /* Division of Pancreas, Percutaneous Endoscopic Approach */
"0F9000Z"  =  "1"  /* Drainage of Liver with Drainage Device, Open Approach */
"0F900ZX"  =  "1"  /* Drainage of Liver, Open Approach, Diagnostic */
"0F900ZZ"  =  "1"  /* Drainage of Liver, Open Approach */
"0F903ZZ"  =  "1"  /* Drainage of Liver, Percutaneous Approach */
"0F904ZZ"  =  "1"  /* Drainage of Liver, Percutaneous Endoscopic Approach */
"0F9100Z"  =  "1"  /* Drainage of Right Lobe Liver with Drainage Device, Open Approach */
"0F910ZX"  =  "1"  /* Drainage of Right Lobe Liver, Open Approach, Diagnostic */
"0F910ZZ"  =  "1"  /* Drainage of Right Lobe Liver, Open Approach */
"0F9200Z"  =  "1"  /* Drainage of Left Lobe Liver with Drainage Device, Open Approach */
"0F920ZX"  =  "1"  /* Drainage of Left Lobe Liver, Open Approach, Diagnostic */
"0F920ZZ"  =  "1"  /* Drainage of Left Lobe Liver, Open Approach */
"0F9400Z"  =  "1"  /* Drainage of Gallbladder with Drainage Device, Open Approach */
"0F940ZX"  =  "1"  /* Drainage of Gallbladder, Open Approach, Diagnostic */
"0F940ZZ"  =  "1"  /* Drainage of Gallbladder, Open Approach */
"0F9500Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drainage Device, Open Approach */
"0F950ZX"  =  "1"  /* Drainage of Right Hepatic Duct, Open Approach, Diagnostic */
"0F950ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Open Approach */
"0F9530Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drainage Device, Percutaneous Approach */
"0F953ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Percutaneous Approach */
"0F9540Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drainage Device, Percutaneous Endoscopic Approach */
"0F954ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0F9570Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drainage Device, Via Natural or Artificial Opening */
"0F957ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Via Natural or Artificial Opening */
"0F9580Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0F958ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0F9600Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drainage Device, Open Approach */
"0F960ZX"  =  "1"  /* Drainage of Left Hepatic Duct, Open Approach, Diagnostic */
"0F960ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Open Approach */
"0F9630Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drainage Device, Percutaneous Approach */
"0F963ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Percutaneous Approach */
"0F9640Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drainage Device, Percutaneous Endoscopic Approach */
"0F964ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0F9670Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drainage Device, Via Natural or Artificial Opening */
"0F967ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Via Natural or Artificial Opening */
"0F9680Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0F968ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0F9800Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Open Approach */
"0F980ZX"  =  "1"  /* Drainage of Cystic Duct, Open Approach, Diagnostic */
"0F980ZZ"  =  "1"  /* Drainage of Cystic Duct, Open Approach */
"0F9830Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Percutaneous Approach */
"0F983ZZ"  =  "1"  /* Drainage of Cystic Duct, Percutaneous Approach */
"0F9840Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Percutaneous Endoscopic Approach */
"0F984ZZ"  =  "1"  /* Drainage of Cystic Duct, Percutaneous Endoscopic Approach */
"0F9870Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Via Natural or Artificial Opening */
"0F987ZZ"  =  "1"  /* Drainage of Cystic Duct, Via Natural or Artificial Opening */
"0F9880Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0F988ZZ"  =  "1"  /* Drainage of Cystic Duct, Via Natural or Artificial Opening Endoscopic */
"0F9900Z"  =  "1"  /* Drainage of Common Bile Duct with Drainage Device, Open Approach */
"0F990ZX"  =  "1"  /* Drainage of Common Bile Duct, Open Approach, Diagnostic */
"0F990ZZ"  =  "1"  /* Drainage of Common Bile Duct, Open Approach */
"0F9930Z"  =  "1"  /* Drainage of Common Bile Duct with Drainage Device, Percutaneous Approach */
"0F9940Z"  =  "1"  /* Drainage of Common Bile Duct with Drainage Device, Percutaneous Endoscopic Approach */
"0F9970Z"  =  "1"  /* Drainage of Common Bile Duct with Drainage Device, Via Natural or Artificial Opening */
"0F9C00Z"  =  "1"  /* Drainage of Ampulla of Vater with Drainage Device, Open Approach */
"0F9C0ZX"  =  "1"  /* Drainage of Ampulla of Vater, Open Approach, Diagnostic */
"0F9C0ZZ"  =  "1"  /* Drainage of Ampulla of Vater, Open Approach */
"0F9C30Z"  =  "1"  /* Drainage of Ampulla of Vater with Drainage Device, Percutaneous Approach */
"0F9C3ZZ"  =  "1"  /* Drainage of Ampulla of Vater, Percutaneous Approach */
"0F9C70Z"  =  "1"  /* Drainage of Ampulla of Vater with Drainage Device, Via Natural or Artificial Opening */
"0F9C7ZZ"  =  "1"  /* Drainage of Ampulla of Vater, Via Natural or Artificial Opening */
"0F9D00Z"  =  "1"  /* Drainage of Pancreatic Duct with Drainage Device, Open Approach */
"0F9D0ZX"  =  "1"  /* Drainage of Pancreatic Duct, Open Approach, Diagnostic */
"0F9D0ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Open Approach */
"0F9D30Z"  =  "1"  /* Drainage of Pancreatic Duct with Drainage Device, Percutaneous Approach */
"0F9D3ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Percutaneous Approach */
"0F9D40Z"  =  "1"  /* Drainage of Pancreatic Duct with Drainage Device, Percutaneous Endoscopic Approach */
"0F9D4ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Percutaneous Endoscopic Approach */
"0F9D70Z"  =  "1"  /* Drainage of Pancreatic Duct with Drainage Device, Via Natural or Artificial Opening */
"0F9D7ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Via Natural or Artificial Opening */
"0F9D8ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0F9F00Z"  =  "1"  /* Drainage of Accessory Pancreatic Duct with Drainage Device, Open Approach */
"0F9F0ZX"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Open Approach, Diagnostic */
"0F9F0ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Open Approach */
"0F9F30Z"  =  "1"  /* Drainage of Accessory Pancreatic Duct with Drainage Device, Percutaneous Approach */
"0F9F3ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Percutaneous Approach */
"0F9F40Z"  =  "1"  /* Drainage of Accessory Pancreatic Duct with Drainage Device, Percutaneous Endoscopic Approach */
"0F9F4ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0F9F70Z"  =  "1"  /* Drainage of Accessory Pancreatic Duct with Drainage Device, Via Natural or Artificial Opening */
"0F9F7ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0F9F8ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0F9G00Z"  =  "1"  /* Drainage of Pancreas with Drainage Device, Open Approach */
"0F9G0ZX"  =  "1"  /* Drainage of Pancreas, Open Approach, Diagnostic */
"0F9G0ZZ"  =  "1"  /* Drainage of Pancreas, Open Approach */
"0F9G30Z"  =  "1"  /* Drainage of Pancreas with Drainage Device, Percutaneous Approach */
"0F9G3ZZ"  =  "1"  /* Drainage of Pancreas, Percutaneous Approach */
"0F9G40Z"  =  "1"  /* Drainage of Pancreas with Drainage Device, Percutaneous Endoscopic Approach */
"0F9G4ZZ"  =  "1"  /* Drainage of Pancreas, Percutaneous Endoscopic Approach */
"0FB00ZX"  =  "1"  /* Excision of Liver, Open Approach, Diagnostic */
"0FB00ZZ"  =  "1"  /* Excision of Liver, Open Approach */
"0FB03ZZ"  =  "1"  /* Excision of Liver, Percutaneous Approach */
"0FB04ZZ"  =  "1"  /* Excision of Liver, Percutaneous Endoscopic Approach */
"0FB10ZX"  =  "1"  /* Excision of Right Lobe Liver, Open Approach, Diagnostic */
"0FB10ZZ"  =  "1"  /* Excision of Right Lobe Liver, Open Approach */
"0FB13ZZ"  =  "1"  /* Excision of Right Lobe Liver, Percutaneous Approach */
"0FB14ZZ"  =  "1"  /* Excision of Right Lobe Liver, Percutaneous Endoscopic Approach */
"0FB20ZX"  =  "1"  /* Excision of Left Lobe Liver, Open Approach, Diagnostic */
"0FB20ZZ"  =  "1"  /* Excision of Left Lobe Liver, Open Approach */
"0FB23ZZ"  =  "1"  /* Excision of Left Lobe Liver, Percutaneous Approach */
"0FB24ZZ"  =  "1"  /* Excision of Left Lobe Liver, Percutaneous Endoscopic Approach */
"0FB40ZX"  =  "1"  /* Excision of Gallbladder, Open Approach, Diagnostic */
"0FB40ZZ"  =  "1"  /* Excision of Gallbladder, Open Approach */
"0FB43ZZ"  =  "1"  /* Excision of Gallbladder, Percutaneous Approach */
"0FB50ZX"  =  "1"  /* Excision of Right Hepatic Duct, Open Approach, Diagnostic */
"0FB50ZZ"  =  "1"  /* Excision of Right Hepatic Duct, Open Approach */
"0FB53ZZ"  =  "1"  /* Excision of Right Hepatic Duct, Percutaneous Approach */
"0FB57ZZ"  =  "1"  /* Excision of Right Hepatic Duct, Via Natural or Artificial Opening */
"0FB60ZX"  =  "1"  /* Excision of Left Hepatic Duct, Open Approach, Diagnostic */
"0FB60ZZ"  =  "1"  /* Excision of Left Hepatic Duct, Open Approach */
"0FB63ZZ"  =  "1"  /* Excision of Left Hepatic Duct, Percutaneous Approach */
"0FB67ZZ"  =  "1"  /* Excision of Left Hepatic Duct, Via Natural or Artificial Opening */
"0FB80ZX"  =  "1"  /* Excision of Cystic Duct, Open Approach, Diagnostic */
"0FB80ZZ"  =  "1"  /* Excision of Cystic Duct, Open Approach */
"0FB83ZZ"  =  "1"  /* Excision of Cystic Duct, Percutaneous Approach */
"0FB87ZZ"  =  "1"  /* Excision of Cystic Duct, Via Natural or Artificial Opening */
"0FB90ZX"  =  "1"  /* Excision of Common Bile Duct, Open Approach, Diagnostic */
"0FB90ZZ"  =  "1"  /* Excision of Common Bile Duct, Open Approach */
"0FB93ZZ"  =  "1"  /* Excision of Common Bile Duct, Percutaneous Approach */
"0FB97ZZ"  =  "1"  /* Excision of Common Bile Duct, Via Natural or Artificial Opening */
"0FBC0ZX"  =  "1"  /* Excision of Ampulla of Vater, Open Approach, Diagnostic */
"0FBC0ZZ"  =  "1"  /* Excision of Ampulla of Vater, Open Approach */
"0FBC3ZZ"  =  "1"  /* Excision of Ampulla of Vater, Percutaneous Approach */
"0FBC7ZZ"  =  "1"  /* Excision of Ampulla of Vater, Via Natural or Artificial Opening */
"0FBD0ZX"  =  "1"  /* Excision of Pancreatic Duct, Open Approach, Diagnostic */
"0FBD0ZZ"  =  "1"  /* Excision of Pancreatic Duct, Open Approach */
"0FBD3ZZ"  =  "1"  /* Excision of Pancreatic Duct, Percutaneous Approach */
"0FBD7ZZ"  =  "1"  /* Excision of Pancreatic Duct, Via Natural or Artificial Opening */
"0FBF0ZX"  =  "1"  /* Excision of Accessory Pancreatic Duct, Open Approach, Diagnostic */
"0FBF0ZZ"  =  "1"  /* Excision of Accessory Pancreatic Duct, Open Approach */
"0FBF3ZZ"  =  "1"  /* Excision of Accessory Pancreatic Duct, Percutaneous Approach */
"0FBF7ZZ"  =  "1"  /* Excision of Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FBG0ZX"  =  "1"  /* Excision of Pancreas, Open Approach, Diagnostic */
"0FBG0ZZ"  =  "1"  /* Excision of Pancreas, Open Approach */
"0FBG3ZZ"  =  "1"  /* Excision of Pancreas, Percutaneous Approach */
"0FBG4ZZ"  =  "1"  /* Excision of Pancreas, Percutaneous Endoscopic Approach */
"0FC00ZZ"  =  "1"  /* Extirpation of Matter from Liver, Open Approach */
"0FC03ZZ"  =  "1"  /* Extirpation of Matter from Liver, Percutaneous Approach */
"0FC04ZZ"  =  "1"  /* Extirpation of Matter from Liver, Percutaneous Endoscopic Approach */
"0FC10ZZ"  =  "1"  /* Extirpation of Matter from Right Lobe Liver, Open Approach */
"0FC13ZZ"  =  "1"  /* Extirpation of Matter from Right Lobe Liver, Percutaneous Approach */
"0FC14ZZ"  =  "1"  /* Extirpation of Matter from Right Lobe Liver, Percutaneous Endoscopic Approach */
"0FC20ZZ"  =  "1"  /* Extirpation of Matter from Left Lobe Liver, Open Approach */
"0FC23ZZ"  =  "1"  /* Extirpation of Matter from Left Lobe Liver, Percutaneous Approach */
"0FC24ZZ"  =  "1"  /* Extirpation of Matter from Left Lobe Liver, Percutaneous Endoscopic Approach */
"0FC40ZZ"  =  "1"  /* Extirpation of Matter from Gallbladder, Open Approach */
"0FC43ZZ"  =  "1"  /* Extirpation of Matter from Gallbladder, Percutaneous Approach */
"0FC44ZZ"  =  "1"  /* Extirpation of Matter from Gallbladder, Percutaneous Endoscopic Approach */
"0FC50ZZ"  =  "1"  /* Extirpation of Matter from Right Hepatic Duct, Open Approach */
"0FC60ZZ"  =  "1"  /* Extirpation of Matter from Left Hepatic Duct, Open Approach */
"0FC80ZZ"  =  "1"  /* Extirpation of Matter from Cystic Duct, Open Approach */
"0FC90ZZ"  =  "1"  /* Extirpation of Matter from Common Bile Duct, Open Approach */
"0FCC0ZZ"  =  "1"  /* Extirpation of Matter from Ampulla of Vater, Open Approach */
"0FCC3ZZ"  =  "1"  /* Extirpation of Matter from Ampulla of Vater, Percutaneous Approach */
"0FCC7ZZ"  =  "1"  /* Extirpation of Matter from Ampulla of Vater, Via Natural or Artificial Opening */
"0FCD0ZZ"  =  "1"  /* Extirpation of Matter from Pancreatic Duct, Open Approach */
"0FCD7ZZ"  =  "1"  /* Extirpation of Matter from Pancreatic Duct, Via Natural or Artificial Opening */
"0FCF0ZZ"  =  "1"  /* Extirpation of Matter from Accessory Pancreatic Duct, Open Approach */
"0FCF7ZZ"  =  "1"  /* Extirpation of Matter from Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FCG0ZZ"  =  "1"  /* Extirpation of Matter from Pancreas, Open Approach */
"0FCG3ZZ"  =  "1"  /* Extirpation of Matter from Pancreas, Percutaneous Approach */
"0FCG4ZZ"  =  "1"  /* Extirpation of Matter from Pancreas, Percutaneous Endoscopic Approach */
"0FF40ZZ"  =  "1"  /* Fragmentation in Gallbladder, Open Approach */
"0FF43ZZ"  =  "1"  /* Fragmentation in Gallbladder, Percutaneous Approach */
"0FF44ZZ"  =  "1"  /* Fragmentation in Gallbladder, Percutaneous Endoscopic Approach */
"0FF47ZZ"  =  "1"  /* Fragmentation in Gallbladder, Via Natural or Artificial Opening */
"0FF50ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Open Approach */
"0FF53ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Percutaneous Approach */
"0FF54ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0FF57ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Via Natural or Artificial Opening */
"0FF60ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Open Approach */
"0FF63ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Percutaneous Approach */
"0FF64ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0FF67ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Via Natural or Artificial Opening */
"0FF80ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Open Approach */
"0FF83ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Percutaneous Approach */
"0FF84ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Percutaneous Endoscopic Approach */
"0FF87ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Via Natural or Artificial Opening */
"0FF90ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Open Approach */
"0FF93ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Percutaneous Approach */
"0FF94ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Percutaneous Endoscopic Approach */
"0FF97ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Via Natural or Artificial Opening */
"0FFC0ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Open Approach */
"0FFC3ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Percutaneous Approach */
"0FFC4ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FFC7ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Via Natural or Artificial Opening */
"0FFD0ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Open Approach */
"0FFD3ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Percutaneous Approach */
"0FFD4ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FFD7ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Via Natural or Artificial Opening */
"0FFD8ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FFF0ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Open Approach */
"0FFF3ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Percutaneous Approach */
"0FFF4ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FFF7ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FFF8ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FH002Z"  =  "1"  /* Insertion of Monitoring Device into Liver, Open Approach */
"0FH032Z"  =  "1"  /* Insertion of Monitoring Device into Liver, Percutaneous Approach */
"0FH042Z"  =  "1"  /* Insertion of Monitoring Device into Liver, Percutaneous Endoscopic Approach */
"0FH102Z"  =  "1"  /* Insertion of Monitoring Device into Right Lobe Liver, Open Approach */
"0FH132Z"  =  "1"  /* Insertion of Monitoring Device into Right Lobe Liver, Percutaneous Approach */
"0FH142Z"  =  "1"  /* Insertion of Monitoring Device into Right Lobe Liver, Percutaneous Endoscopic Approach */
"0FH202Z"  =  "1"  /* Insertion of Monitoring Device into Left Lobe Liver, Open Approach */
"0FH232Z"  =  "1"  /* Insertion of Monitoring Device into Left Lobe Liver, Percutaneous Approach */
"0FH242Z"  =  "1"  /* Insertion of Monitoring Device into Left Lobe Liver, Percutaneous Endoscopic Approach */
"0FH402Z"  =  "1"  /* Insertion of Monitoring Device into Gallbladder, Open Approach */
"0FH432Z"  =  "1"  /* Insertion of Monitoring Device into Gallbladder, Percutaneous Approach */
"0FH442Z"  =  "1"  /* Insertion of Monitoring Device into Gallbladder, Percutaneous Endoscopic Approach */
"0FHB02Z"  =  "1"  /* Insertion of Monitoring Device into Hepatobiliary Duct, Open Approach */
"0FHB0DZ"  =  "1"  /* Insertion of Intraluminal Device into Hepatobiliary Duct, Open Approach */
"0FHB32Z"  =  "1"  /* Insertion of Monitoring Device into Hepatobiliary Duct, Percutaneous Approach */
"0FHB3DZ"  =  "1"  /* Insertion of Intraluminal Device into Hepatobiliary Duct, Percutaneous Approach */
"0FHB42Z"  =  "1"  /* Insertion of Monitoring Device into Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FHB72Z"  =  "1"  /* Insertion of Monitoring Device into Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FHB7DZ"  =  "1"  /* Insertion of Intraluminal Device into Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FHB82Z"  =  "1"  /* Insertion of Monitoring Device into Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FHD02Z"  =  "1"  /* Insertion of Monitoring Device into Pancreatic Duct, Open Approach */
"0FHD0DZ"  =  "1"  /* Insertion of Intraluminal Device into Pancreatic Duct, Open Approach */
"0FHD32Z"  =  "1"  /* Insertion of Monitoring Device into Pancreatic Duct, Percutaneous Approach */
"0FHD3DZ"  =  "1"  /* Insertion of Intraluminal Device into Pancreatic Duct, Percutaneous Approach */
"0FHD42Z"  =  "1"  /* Insertion of Monitoring Device into Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FHD72Z"  =  "1"  /* Insertion of Monitoring Device into Pancreatic Duct, Via Natural or Artificial Opening */
"0FHD7DZ"  =  "1"  /* Insertion of Intraluminal Device into Pancreatic Duct, Via Natural or Artificial Opening */
"0FHD82Z"  =  "1"  /* Insertion of Monitoring Device into Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FHG02Z"  =  "1"  /* Insertion of Monitoring Device into Pancreas, Open Approach */
"0FHG32Z"  =  "1"  /* Insertion of Monitoring Device into Pancreas, Percutaneous Approach */
"0FHG42Z"  =  "1"  /* Insertion of Monitoring Device into Pancreas, Percutaneous Endoscopic Approach */
"0FJ00ZZ"  =  "1"  /* Inspection of Liver, Open Approach */
"0FJB0ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Open Approach */
"0FJB3ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Percutaneous Approach */
"0FJB4ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FJB7ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FJB8ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FL50CZ"  =  "1"  /* Occlusion of Right Hepatic Duct with Extraluminal Device, Open Approach */
"0FL50DZ"  =  "1"  /* Occlusion of Right Hepatic Duct with Intraluminal Device, Open Approach */
"0FL50ZZ"  =  "1"  /* Occlusion of Right Hepatic Duct, Open Approach */
"0FL60CZ"  =  "1"  /* Occlusion of Left Hepatic Duct with Extraluminal Device, Open Approach */
"0FL60DZ"  =  "1"  /* Occlusion of Left Hepatic Duct with Intraluminal Device, Open Approach */
"0FL60ZZ"  =  "1"  /* Occlusion of Left Hepatic Duct, Open Approach */
"0FL80CZ"  =  "1"  /* Occlusion of Cystic Duct with Extraluminal Device, Open Approach */
"0FL80DZ"  =  "1"  /* Occlusion of Cystic Duct with Intraluminal Device, Open Approach */
"0FL80ZZ"  =  "1"  /* Occlusion of Cystic Duct, Open Approach */
"0FL90CZ"  =  "1"  /* Occlusion of Common Bile Duct with Extraluminal Device, Open Approach */
"0FL90DZ"  =  "1"  /* Occlusion of Common Bile Duct with Intraluminal Device, Open Approach */
"0FL90ZZ"  =  "1"  /* Occlusion of Common Bile Duct, Open Approach */
"0FLC0CZ"  =  "1"  /* Occlusion of Ampulla of Vater with Extraluminal Device, Open Approach */
"0FLC0DZ"  =  "1"  /* Occlusion of Ampulla of Vater with Intraluminal Device, Open Approach */
"0FLC0ZZ"  =  "1"  /* Occlusion of Ampulla of Vater, Open Approach */
"0FLC3CZ"  =  "1"  /* Occlusion of Ampulla of Vater with Extraluminal Device, Percutaneous Approach */
"0FLC3DZ"  =  "1"  /* Occlusion of Ampulla of Vater with Intraluminal Device, Percutaneous Approach */
"0FLC3ZZ"  =  "1"  /* Occlusion of Ampulla of Vater, Percutaneous Approach */
"0FLC4CZ"  =  "1"  /* Occlusion of Ampulla of Vater with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FLC4DZ"  =  "1"  /* Occlusion of Ampulla of Vater with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FLC4ZZ"  =  "1"  /* Occlusion of Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FLC7DZ"  =  "1"  /* Occlusion of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening */
"0FLC7ZZ"  =  "1"  /* Occlusion of Ampulla of Vater, Via Natural or Artificial Opening */
"0FLC8DZ"  =  "1"  /* Occlusion of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FLC8ZZ"  =  "1"  /* Occlusion of Ampulla of Vater, Via Natural or Artificial Opening Endoscopic */
"0FLD0CZ"  =  "1"  /* Occlusion of Pancreatic Duct with Extraluminal Device, Open Approach */
"0FLD0DZ"  =  "1"  /* Occlusion of Pancreatic Duct with Intraluminal Device, Open Approach */
"0FLD0ZZ"  =  "1"  /* Occlusion of Pancreatic Duct, Open Approach */
"0FLD3CZ"  =  "1"  /* Occlusion of Pancreatic Duct with Extraluminal Device, Percutaneous Approach */
"0FLD3DZ"  =  "1"  /* Occlusion of Pancreatic Duct with Intraluminal Device, Percutaneous Approach */
"0FLD3ZZ"  =  "1"  /* Occlusion of Pancreatic Duct, Percutaneous Approach */
"0FLD4CZ"  =  "1"  /* Occlusion of Pancreatic Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FLD4DZ"  =  "1"  /* Occlusion of Pancreatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FLD4ZZ"  =  "1"  /* Occlusion of Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FLD7DZ"  =  "1"  /* Occlusion of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0FLD7ZZ"  =  "1"  /* Occlusion of Pancreatic Duct, Via Natural or Artificial Opening */
"0FLD8DZ"  =  "1"  /* Occlusion of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FLD8ZZ"  =  "1"  /* Occlusion of Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FLF0CZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct with Extraluminal Device, Open Approach */
"0FLF0DZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct with Intraluminal Device, Open Approach */
"0FLF0ZZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct, Open Approach */
"0FLF3CZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct with Extraluminal Device, Percutaneous Approach */
"0FLF3DZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct with Intraluminal Device, Percutaneous Approach */
"0FLF3ZZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct, Percutaneous Approach */
"0FLF4CZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FLF4DZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FLF4ZZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FLF7DZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0FLF7ZZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FLF8DZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FLF8ZZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FM00ZZ"  =  "1"  /* Reattachment of Liver, Open Approach */
"0FM04ZZ"  =  "1"  /* Reattachment of Liver, Percutaneous Endoscopic Approach */
"0FM10ZZ"  =  "1"  /* Reattachment of Right Lobe Liver, Open Approach */
"0FM14ZZ"  =  "1"  /* Reattachment of Right Lobe Liver, Percutaneous Endoscopic Approach */
"0FM20ZZ"  =  "1"  /* Reattachment of Left Lobe Liver, Open Approach */
"0FM24ZZ"  =  "1"  /* Reattachment of Left Lobe Liver, Percutaneous Endoscopic Approach */
"0FM40ZZ"  =  "1"  /* Reattachment of Gallbladder, Open Approach */
"0FM50ZZ"  =  "1"  /* Reattachment of Right Hepatic Duct, Open Approach */
"0FM60ZZ"  =  "1"  /* Reattachment of Left Hepatic Duct, Open Approach */
"0FM80ZZ"  =  "1"  /* Reattachment of Cystic Duct, Open Approach */
"0FM90ZZ"  =  "1"  /* Reattachment of Common Bile Duct, Open Approach */
"0FMC0ZZ"  =  "1"  /* Reattachment of Ampulla of Vater, Open Approach */
"0FMC4ZZ"  =  "1"  /* Reattachment of Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FMD0ZZ"  =  "1"  /* Reattachment of Pancreatic Duct, Open Approach */
"0FMD4ZZ"  =  "1"  /* Reattachment of Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FMF0ZZ"  =  "1"  /* Reattachment of Accessory Pancreatic Duct, Open Approach */
"0FMF4ZZ"  =  "1"  /* Reattachment of Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FMG0ZZ"  =  "1"  /* Reattachment of Pancreas, Open Approach */
"0FMG4ZZ"  =  "1"  /* Reattachment of Pancreas, Percutaneous Endoscopic Approach */
"0FN00ZZ"  =  "1"  /* Release Liver, Open Approach */
"0FN03ZZ"  =  "1"  /* Release Liver, Percutaneous Approach */
"0FN10ZZ"  =  "1"  /* Release Right Lobe Liver, Open Approach */
"0FN13ZZ"  =  "1"  /* Release Right Lobe Liver, Percutaneous Approach */
"0FN20ZZ"  =  "1"  /* Release Left Lobe Liver, Open Approach */
"0FN23ZZ"  =  "1"  /* Release Left Lobe Liver, Percutaneous Approach */
"0FN40ZZ"  =  "1"  /* Release Gallbladder, Open Approach */
"0FN43ZZ"  =  "1"  /* Release Gallbladder, Percutaneous Approach */
"0FN50ZZ"  =  "1"  /* Release Right Hepatic Duct, Open Approach */
"0FN53ZZ"  =  "1"  /* Release Right Hepatic Duct, Percutaneous Approach */
"0FN57ZZ"  =  "1"  /* Release Right Hepatic Duct, Via Natural or Artificial Opening */
"0FN58ZZ"  =  "1"  /* Release Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FN60ZZ"  =  "1"  /* Release Left Hepatic Duct, Open Approach */
"0FN63ZZ"  =  "1"  /* Release Left Hepatic Duct, Percutaneous Approach */
"0FN67ZZ"  =  "1"  /* Release Left Hepatic Duct, Via Natural or Artificial Opening */
"0FN68ZZ"  =  "1"  /* Release Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FN80ZZ"  =  "1"  /* Release Cystic Duct, Open Approach */
"0FN83ZZ"  =  "1"  /* Release Cystic Duct, Percutaneous Approach */
"0FN87ZZ"  =  "1"  /* Release Cystic Duct, Via Natural or Artificial Opening */
"0FN88ZZ"  =  "1"  /* Release Cystic Duct, Via Natural or Artificial Opening Endoscopic */
"0FN90ZZ"  =  "1"  /* Release Common Bile Duct, Open Approach */
"0FN93ZZ"  =  "1"  /* Release Common Bile Duct, Percutaneous Approach */
"0FN97ZZ"  =  "1"  /* Release Common Bile Duct, Via Natural or Artificial Opening */
"0FN98ZZ"  =  "1"  /* Release Common Bile Duct, Via Natural or Artificial Opening Endoscopic */
"0FNC0ZZ"  =  "1"  /* Release Ampulla of Vater, Open Approach */
"0FNC3ZZ"  =  "1"  /* Release Ampulla of Vater, Percutaneous Approach */
"0FNC7ZZ"  =  "1"  /* Release Ampulla of Vater, Via Natural or Artificial Opening */
"0FNC8ZZ"  =  "1"  /* Release Ampulla of Vater, Via Natural or Artificial Opening Endoscopic */
"0FND0ZZ"  =  "1"  /* Release Pancreatic Duct, Open Approach */
"0FND3ZZ"  =  "1"  /* Release Pancreatic Duct, Percutaneous Approach */
"0FND7ZZ"  =  "1"  /* Release Pancreatic Duct, Via Natural or Artificial Opening */
"0FND8ZZ"  =  "1"  /* Release Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FNF0ZZ"  =  "1"  /* Release Accessory Pancreatic Duct, Open Approach */
"0FNF3ZZ"  =  "1"  /* Release Accessory Pancreatic Duct, Percutaneous Approach */
"0FNF7ZZ"  =  "1"  /* Release Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FNF8ZZ"  =  "1"  /* Release Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FNG0ZZ"  =  "1"  /* Release Pancreas, Open Approach */
"0FNG3ZZ"  =  "1"  /* Release Pancreas, Percutaneous Approach */
"0FP000Z"  =  "1"  /* Removal of Drainage Device from Liver, Open Approach */
"0FP002Z"  =  "1"  /* Removal of Monitoring Device from Liver, Open Approach */
"0FP003Z"  =  "1"  /* Removal of Infusion Device from Liver, Open Approach */
"0FP030Z"  =  "1"  /* Removal of Drainage Device from Liver, Percutaneous Approach */
"0FP032Z"  =  "1"  /* Removal of Monitoring Device from Liver, Percutaneous Approach */
"0FP033Z"  =  "1"  /* Removal of Infusion Device from Liver, Percutaneous Approach */
"0FP040Z"  =  "1"  /* Removal of Drainage Device from Liver, Percutaneous Endoscopic Approach */
"0FP042Z"  =  "1"  /* Removal of Monitoring Device from Liver, Percutaneous Endoscopic Approach */
"0FP043Z"  =  "1"  /* Removal of Infusion Device from Liver, Percutaneous Endoscopic Approach */
"0FP400Z"  =  "1"  /* Removal of Drainage Device from Gallbladder, Open Approach */
"0FP402Z"  =  "1"  /* Removal of Monitoring Device from Gallbladder, Open Approach */
"0FP403Z"  =  "1"  /* Removal of Infusion Device from Gallbladder, Open Approach */
"0FP40DZ"  =  "1"  /* Removal of Intraluminal Device from Gallbladder, Open Approach */
"0FP430Z"  =  "1"  /* Removal of Drainage Device from Gallbladder, Percutaneous Approach */
"0FP432Z"  =  "1"  /* Removal of Monitoring Device from Gallbladder, Percutaneous Approach */
"0FP433Z"  =  "1"  /* Removal of Infusion Device from Gallbladder, Percutaneous Approach */
"0FP43DZ"  =  "1"  /* Removal of Intraluminal Device from Gallbladder, Percutaneous Approach */
"0FP440Z"  =  "1"  /* Removal of Drainage Device from Gallbladder, Percutaneous Endoscopic Approach */
"0FP442Z"  =  "1"  /* Removal of Monitoring Device from Gallbladder, Percutaneous Endoscopic Approach */
"0FP443Z"  =  "1"  /* Removal of Infusion Device from Gallbladder, Percutaneous Endoscopic Approach */
"0FP44DZ"  =  "1"  /* Removal of Intraluminal Device from Gallbladder, Percutaneous Endoscopic Approach */
"0FPB00Z"  =  "1"  /* Removal of Drainage Device from Hepatobiliary Duct, Open Approach */
"0FPB01Z"  =  "1"  /* Removal of Radioactive Element from Hepatobiliary Duct, Open Approach */
"0FPB02Z"  =  "1"  /* Removal of Monitoring Device from Hepatobiliary Duct, Open Approach */
"0FPB03Z"  =  "1"  /* Removal of Infusion Device from Hepatobiliary Duct, Open Approach */
"0FPB07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Open Approach */
"0FPB0CZ"  =  "1"  /* Removal of Extraluminal Device from Hepatobiliary Duct, Open Approach */
"0FPB0DZ"  =  "1"  /* Removal of Intraluminal Device from Hepatobiliary Duct, Open Approach */
"0FPB0JZ"  =  "1"  /* Removal of Synthetic Substitute from Hepatobiliary Duct, Open Approach */
"0FPB0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Open Approach */
"0FPB30Z"  =  "1"  /* Removal of Drainage Device from Hepatobiliary Duct, Percutaneous Approach */
"0FPB31Z"  =  "1"  /* Removal of Radioactive Element from Hepatobiliary Duct, Percutaneous Approach */
"0FPB32Z"  =  "1"  /* Removal of Monitoring Device from Hepatobiliary Duct, Percutaneous Approach */
"0FPB33Z"  =  "1"  /* Removal of Infusion Device from Hepatobiliary Duct, Percutaneous Approach */
"0FPB37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Percutaneous Approach */
"0FPB3CZ"  =  "1"  /* Removal of Extraluminal Device from Hepatobiliary Duct, Percutaneous Approach */
"0FPB3DZ"  =  "1"  /* Removal of Intraluminal Device from Hepatobiliary Duct, Percutaneous Approach */
"0FPB3JZ"  =  "1"  /* Removal of Synthetic Substitute from Hepatobiliary Duct, Percutaneous Approach */
"0FPB3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Percutaneous Approach */
"0FPB40Z"  =  "1"  /* Removal of Drainage Device from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB41Z"  =  "1"  /* Removal of Radioactive Element from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB42Z"  =  "1"  /* Removal of Monitoring Device from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB43Z"  =  "1"  /* Removal of Infusion Device from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB4CZ"  =  "1"  /* Removal of Extraluminal Device from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB4DZ"  =  "1"  /* Removal of Intraluminal Device from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB4JZ"  =  "1"  /* Removal of Synthetic Substitute from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FPB70Z"  =  "1"  /* Removal of Drainage Device from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB71Z"  =  "1"  /* Removal of Radioactive Element from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB72Z"  =  "1"  /* Removal of Monitoring Device from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB73Z"  =  "1"  /* Removal of Infusion Device from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB7CZ"  =  "1"  /* Removal of Extraluminal Device from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB7DZ"  =  "1"  /* Removal of Intraluminal Device from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB7JZ"  =  "1"  /* Removal of Synthetic Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FPB80Z"  =  "1"  /* Removal of Drainage Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB81Z"  =  "1"  /* Removal of Radioactive Element from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB82Z"  =  "1"  /* Removal of Monitoring Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB83Z"  =  "1"  /* Removal of Infusion Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB8CZ"  =  "1"  /* Removal of Extraluminal Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB8DZ"  =  "1"  /* Removal of Intraluminal Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB8JZ"  =  "1"  /* Removal of Synthetic Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPB8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD00Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Open Approach */
"0FPD01Z"  =  "1"  /* Removal of Radioactive Element from Pancreatic Duct, Open Approach */
"0FPD02Z"  =  "1"  /* Removal of Monitoring Device from Pancreatic Duct, Open Approach */
"0FPD03Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Open Approach */
"0FPD07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Pancreatic Duct, Open Approach */
"0FPD0CZ"  =  "1"  /* Removal of Extraluminal Device from Pancreatic Duct, Open Approach */
"0FPD0DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreatic Duct, Open Approach */
"0FPD0JZ"  =  "1"  /* Removal of Synthetic Substitute from Pancreatic Duct, Open Approach */
"0FPD0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Open Approach */
"0FPD30Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Percutaneous Approach */
"0FPD31Z"  =  "1"  /* Removal of Radioactive Element from Pancreatic Duct, Percutaneous Approach */
"0FPD32Z"  =  "1"  /* Removal of Monitoring Device from Pancreatic Duct, Percutaneous Approach */
"0FPD33Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Percutaneous Approach */
"0FPD37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Pancreatic Duct, Percutaneous Approach */
"0FPD3CZ"  =  "1"  /* Removal of Extraluminal Device from Pancreatic Duct, Percutaneous Approach */
"0FPD3DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreatic Duct, Percutaneous Approach */
"0FPD3JZ"  =  "1"  /* Removal of Synthetic Substitute from Pancreatic Duct, Percutaneous Approach */
"0FPD3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Percutaneous Approach */
"0FPD40Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD41Z"  =  "1"  /* Removal of Radioactive Element from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD42Z"  =  "1"  /* Removal of Monitoring Device from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD43Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD4CZ"  =  "1"  /* Removal of Extraluminal Device from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD4DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD4JZ"  =  "1"  /* Removal of Synthetic Substitute from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FPD70Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD71Z"  =  "1"  /* Removal of Radioactive Element from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD72Z"  =  "1"  /* Removal of Monitoring Device from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD73Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD7CZ"  =  "1"  /* Removal of Extraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD7DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD7JZ"  =  "1"  /* Removal of Synthetic Substitute from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening */
"0FPD80Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD81Z"  =  "1"  /* Removal of Radioactive Element from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD82Z"  =  "1"  /* Removal of Monitoring Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD83Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD8CZ"  =  "1"  /* Removal of Extraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD8DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD8JZ"  =  "1"  /* Removal of Synthetic Substitute from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPD8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FPG00Z"  =  "1"  /* Removal of Drainage Device from Pancreas, Open Approach */
"0FPG02Z"  =  "1"  /* Removal of Monitoring Device from Pancreas, Open Approach */
"0FPG03Z"  =  "1"  /* Removal of Infusion Device from Pancreas, Open Approach */
"0FPG0DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreas, Open Approach */
"0FPG30Z"  =  "1"  /* Removal of Drainage Device from Pancreas, Percutaneous Approach */
"0FPG32Z"  =  "1"  /* Removal of Monitoring Device from Pancreas, Percutaneous Approach */
"0FPG33Z"  =  "1"  /* Removal of Infusion Device from Pancreas, Percutaneous Approach */
"0FPG3DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreas, Percutaneous Approach */
"0FPG40Z"  =  "1"  /* Removal of Drainage Device from Pancreas, Percutaneous Endoscopic Approach */
"0FPG42Z"  =  "1"  /* Removal of Monitoring Device from Pancreas, Percutaneous Endoscopic Approach */
"0FPG43Z"  =  "1"  /* Removal of Infusion Device from Pancreas, Percutaneous Endoscopic Approach */
"0FPG4DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreas, Percutaneous Endoscopic Approach */
"0FPGXDZ"  =  "1"  /* Removal of Intraluminal Device from Pancreas, External Approach */
"0FQ00ZZ"  =  "1"  /* Repair Liver, Open Approach */
"0FQ03ZZ"  =  "1"  /* Repair Liver, Percutaneous Approach */
"0FQ04ZZ"  =  "1"  /* Repair Liver, Percutaneous Endoscopic Approach */
"0FQ40ZZ"  =  "1"  /* Repair Gallbladder, Open Approach */
"0FQ43ZZ"  =  "1"  /* Repair Gallbladder, Percutaneous Approach */
"0FQ44ZZ"  =  "1"  /* Repair Gallbladder, Percutaneous Endoscopic Approach */
"0FQ50ZZ"  =  "1"  /* Repair Right Hepatic Duct, Open Approach */
"0FQ53ZZ"  =  "1"  /* Repair Right Hepatic Duct, Percutaneous Approach */
"0FQ54ZZ"  =  "1"  /* Repair Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0FQ57ZZ"  =  "1"  /* Repair Right Hepatic Duct, Via Natural or Artificial Opening */
"0FQ58ZZ"  =  "1"  /* Repair Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FQ60ZZ"  =  "1"  /* Repair Left Hepatic Duct, Open Approach */
"0FQ63ZZ"  =  "1"  /* Repair Left Hepatic Duct, Percutaneous Approach */
"0FQ64ZZ"  =  "1"  /* Repair Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0FQ67ZZ"  =  "1"  /* Repair Left Hepatic Duct, Via Natural or Artificial Opening */
"0FQ68ZZ"  =  "1"  /* Repair Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FQ80ZZ"  =  "1"  /* Repair Cystic Duct, Open Approach */
"0FQ83ZZ"  =  "1"  /* Repair Cystic Duct, Percutaneous Approach */
"0FQ84ZZ"  =  "1"  /* Repair Cystic Duct, Percutaneous Endoscopic Approach */
"0FQ87ZZ"  =  "1"  /* Repair Cystic Duct, Via Natural or Artificial Opening */
"0FQ88ZZ"  =  "1"  /* Repair Cystic Duct, Via Natural or Artificial Opening Endoscopic */
"0FQ90ZZ"  =  "1"  /* Repair Common Bile Duct, Open Approach */
"0FQ93ZZ"  =  "1"  /* Repair Common Bile Duct, Percutaneous Approach */
"0FQ94ZZ"  =  "1"  /* Repair Common Bile Duct, Percutaneous Endoscopic Approach */
"0FQ97ZZ"  =  "1"  /* Repair Common Bile Duct, Via Natural or Artificial Opening */
"0FQ98ZZ"  =  "1"  /* Repair Common Bile Duct, Via Natural or Artificial Opening Endoscopic */
"0FQC0ZZ"  =  "1"  /* Repair Ampulla of Vater, Open Approach */
"0FQC3ZZ"  =  "1"  /* Repair Ampulla of Vater, Percutaneous Approach */
"0FQC4ZZ"  =  "1"  /* Repair Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FQC7ZZ"  =  "1"  /* Repair Ampulla of Vater, Via Natural or Artificial Opening */
"0FQC8ZZ"  =  "1"  /* Repair Ampulla of Vater, Via Natural or Artificial Opening Endoscopic */
"0FQD0ZZ"  =  "1"  /* Repair Pancreatic Duct, Open Approach */
"0FQD3ZZ"  =  "1"  /* Repair Pancreatic Duct, Percutaneous Approach */
"0FQD4ZZ"  =  "1"  /* Repair Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FQD7ZZ"  =  "1"  /* Repair Pancreatic Duct, Via Natural or Artificial Opening */
"0FQD8ZZ"  =  "1"  /* Repair Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FQF0ZZ"  =  "1"  /* Repair Accessory Pancreatic Duct, Open Approach */
"0FQF3ZZ"  =  "1"  /* Repair Accessory Pancreatic Duct, Percutaneous Approach */
"0FQF4ZZ"  =  "1"  /* Repair Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FQF7ZZ"  =  "1"  /* Repair Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FQF8ZZ"  =  "1"  /* Repair Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FQG0ZZ"  =  "1"  /* Repair Pancreas, Open Approach */
"0FQG3ZZ"  =  "1"  /* Repair Pancreas, Percutaneous Approach */
"0FQG4ZZ"  =  "1"  /* Repair Pancreas, Percutaneous Endoscopic Approach */
"0FR507Z"  =  "1"  /* Replacement of Right Hepatic Duct with Autologous Tissue Substitute, Open Approach */
"0FR50JZ"  =  "1"  /* Replacement of Right Hepatic Duct with Synthetic Substitute, Open Approach */
"0FR50KZ"  =  "1"  /* Replacement of Right Hepatic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FR547Z"  =  "1"  /* Replacement of Right Hepatic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FR54JZ"  =  "1"  /* Replacement of Right Hepatic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FR54KZ"  =  "1"  /* Replacement of Right Hepatic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FR607Z"  =  "1"  /* Replacement of Left Hepatic Duct with Autologous Tissue Substitute, Open Approach */
"0FR60JZ"  =  "1"  /* Replacement of Left Hepatic Duct with Synthetic Substitute, Open Approach */
"0FR60KZ"  =  "1"  /* Replacement of Left Hepatic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FR647Z"  =  "1"  /* Replacement of Left Hepatic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FR64JZ"  =  "1"  /* Replacement of Left Hepatic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FR64KZ"  =  "1"  /* Replacement of Left Hepatic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FR807Z"  =  "1"  /* Replacement of Cystic Duct with Autologous Tissue Substitute, Open Approach */
"0FR80JZ"  =  "1"  /* Replacement of Cystic Duct with Synthetic Substitute, Open Approach */
"0FR80KZ"  =  "1"  /* Replacement of Cystic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FR847Z"  =  "1"  /* Replacement of Cystic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FR84JZ"  =  "1"  /* Replacement of Cystic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FR84KZ"  =  "1"  /* Replacement of Cystic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FR907Z"  =  "1"  /* Replacement of Common Bile Duct with Autologous Tissue Substitute, Open Approach */
"0FR90JZ"  =  "1"  /* Replacement of Common Bile Duct with Synthetic Substitute, Open Approach */
"0FR90KZ"  =  "1"  /* Replacement of Common Bile Duct with Nonautologous Tissue Substitute, Open Approach */
"0FR947Z"  =  "1"  /* Replacement of Common Bile Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FR94JZ"  =  "1"  /* Replacement of Common Bile Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FR94KZ"  =  "1"  /* Replacement of Common Bile Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FRC07Z"  =  "1"  /* Replacement of Ampulla of Vater with Autologous Tissue Substitute, Open Approach */
"0FRC0JZ"  =  "1"  /* Replacement of Ampulla of Vater with Synthetic Substitute, Open Approach */
"0FRC0KZ"  =  "1"  /* Replacement of Ampulla of Vater with Nonautologous Tissue Substitute, Open Approach */
"0FRC47Z"  =  "1"  /* Replacement of Ampulla of Vater with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FRC4JZ"  =  "1"  /* Replacement of Ampulla of Vater with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FRC4KZ"  =  "1"  /* Replacement of Ampulla of Vater with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FRD07Z"  =  "1"  /* Replacement of Pancreatic Duct with Autologous Tissue Substitute, Open Approach */
"0FRD0JZ"  =  "1"  /* Replacement of Pancreatic Duct with Synthetic Substitute, Open Approach */
"0FRD0KZ"  =  "1"  /* Replacement of Pancreatic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FRD47Z"  =  "1"  /* Replacement of Pancreatic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FRD4JZ"  =  "1"  /* Replacement of Pancreatic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FRD4KZ"  =  "1"  /* Replacement of Pancreatic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FRF07Z"  =  "1"  /* Replacement of Accessory Pancreatic Duct with Autologous Tissue Substitute, Open Approach */
"0FRF0JZ"  =  "1"  /* Replacement of Accessory Pancreatic Duct with Synthetic Substitute, Open Approach */
"0FRF0KZ"  =  "1"  /* Replacement of Accessory Pancreatic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FRF47Z"  =  "1"  /* Replacement of Accessory Pancreatic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FRF4JZ"  =  "1"  /* Replacement of Accessory Pancreatic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FRF4KZ"  =  "1"  /* Replacement of Accessory Pancreatic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FS00ZZ"  =  "1"  /* Reposition Liver, Open Approach */
"0FS04ZZ"  =  "1"  /* Reposition Liver, Percutaneous Endoscopic Approach */
"0FS40ZZ"  =  "1"  /* Reposition Gallbladder, Open Approach */
"0FS44ZZ"  =  "1"  /* Reposition Gallbladder, Percutaneous Endoscopic Approach */
"0FS50ZZ"  =  "1"  /* Reposition Right Hepatic Duct, Open Approach */
"0FS54ZZ"  =  "1"  /* Reposition Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0FS60ZZ"  =  "1"  /* Reposition Left Hepatic Duct, Open Approach */
"0FS64ZZ"  =  "1"  /* Reposition Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0FS80ZZ"  =  "1"  /* Reposition Cystic Duct, Open Approach */
"0FS84ZZ"  =  "1"  /* Reposition Cystic Duct, Percutaneous Endoscopic Approach */
"0FS90ZZ"  =  "1"  /* Reposition Common Bile Duct, Open Approach */
"0FS94ZZ"  =  "1"  /* Reposition Common Bile Duct, Percutaneous Endoscopic Approach */
"0FSC0ZZ"  =  "1"  /* Reposition Ampulla of Vater, Open Approach */
"0FSC4ZZ"  =  "1"  /* Reposition Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FSD0ZZ"  =  "1"  /* Reposition Pancreatic Duct, Open Approach */
"0FSD4ZZ"  =  "1"  /* Reposition Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FSF0ZZ"  =  "1"  /* Reposition Accessory Pancreatic Duct, Open Approach */
"0FSF4ZZ"  =  "1"  /* Reposition Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FSG0ZZ"  =  "1"  /* Reposition Pancreas, Open Approach */
"0FSG4ZZ"  =  "1"  /* Reposition Pancreas, Percutaneous Endoscopic Approach */
"0FT00ZZ"  =  "1"  /* Resection of Liver, Open Approach */
"0FT04ZZ"  =  "1"  /* Resection of Liver, Percutaneous Endoscopic Approach */
"0FT10ZZ"  =  "1"  /* Resection of Right Lobe Liver, Open Approach */
"0FT14ZZ"  =  "1"  /* Resection of Right Lobe Liver, Percutaneous Endoscopic Approach */
"0FT20ZZ"  =  "1"  /* Resection of Left Lobe Liver, Open Approach */
"0FT24ZZ"  =  "1"  /* Resection of Left Lobe Liver, Percutaneous Endoscopic Approach */
"0FT40ZZ"  =  "1"  /* Resection of Gallbladder, Open Approach */
"0FT50ZZ"  =  "1"  /* Resection of Right Hepatic Duct, Open Approach */
"0FT54ZZ"  =  "1"  /* Resection of Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0FT57ZZ"  =  "1"  /* Resection of Right Hepatic Duct, Via Natural or Artificial Opening */
"0FT58ZZ"  =  "1"  /* Resection of Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FT60ZZ"  =  "1"  /* Resection of Left Hepatic Duct, Open Approach */
"0FT64ZZ"  =  "1"  /* Resection of Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0FT67ZZ"  =  "1"  /* Resection of Left Hepatic Duct, Via Natural or Artificial Opening */
"0FT68ZZ"  =  "1"  /* Resection of Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FT80ZZ"  =  "1"  /* Resection of Cystic Duct, Open Approach */
"0FT84ZZ"  =  "1"  /* Resection of Cystic Duct, Percutaneous Endoscopic Approach */
"0FT87ZZ"  =  "1"  /* Resection of Cystic Duct, Via Natural or Artificial Opening */
"0FT88ZZ"  =  "1"  /* Resection of Cystic Duct, Via Natural or Artificial Opening Endoscopic */
"0FT90ZZ"  =  "1"  /* Resection of Common Bile Duct, Open Approach */
"0FT94ZZ"  =  "1"  /* Resection of Common Bile Duct, Percutaneous Endoscopic Approach */
"0FT97ZZ"  =  "1"  /* Resection of Common Bile Duct, Via Natural or Artificial Opening */
"0FT98ZZ"  =  "1"  /* Resection of Common Bile Duct, Via Natural or Artificial Opening Endoscopic */
"0FTC0ZZ"  =  "1"  /* Resection of Ampulla of Vater, Open Approach */
"0FTC4ZZ"  =  "1"  /* Resection of Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FTC7ZZ"  =  "1"  /* Resection of Ampulla of Vater, Via Natural or Artificial Opening */
"0FTC8ZZ"  =  "1"  /* Resection of Ampulla of Vater, Via Natural or Artificial Opening Endoscopic */
"0FTD0ZZ"  =  "1"  /* Resection of Pancreatic Duct, Open Approach */
"0FTD7ZZ"  =  "1"  /* Resection of Pancreatic Duct, Via Natural or Artificial Opening */
"0FTF0ZZ"  =  "1"  /* Resection of Accessory Pancreatic Duct, Open Approach */
"0FTF7ZZ"  =  "1"  /* Resection of Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FTG0ZZ"  =  "1"  /* Resection of Pancreas, Open Approach */
"0FTG4ZZ"  =  "1"  /* Resection of Pancreas, Percutaneous Endoscopic Approach */
"0FU507Z"  =  "1"  /* Supplement Right Hepatic Duct with Autologous Tissue Substitute, Open Approach */
"0FU50JZ"  =  "1"  /* Supplement Right Hepatic Duct with Synthetic Substitute, Open Approach */
"0FU50KZ"  =  "1"  /* Supplement Right Hepatic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FU537Z"  =  "1"  /* Supplement Right Hepatic Duct with Autologous Tissue Substitute, Percutaneous Approach */
"0FU53JZ"  =  "1"  /* Supplement Right Hepatic Duct with Synthetic Substitute, Percutaneous Approach */
"0FU53KZ"  =  "1"  /* Supplement Right Hepatic Duct with Nonautologous Tissue Substitute, Percutaneous Approach */
"0FU547Z"  =  "1"  /* Supplement Right Hepatic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FU54JZ"  =  "1"  /* Supplement Right Hepatic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FU54KZ"  =  "1"  /* Supplement Right Hepatic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FU607Z"  =  "1"  /* Supplement Left Hepatic Duct with Autologous Tissue Substitute, Open Approach */
"0FU60JZ"  =  "1"  /* Supplement Left Hepatic Duct with Synthetic Substitute, Open Approach */
"0FU60KZ"  =  "1"  /* Supplement Left Hepatic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FU637Z"  =  "1"  /* Supplement Left Hepatic Duct with Autologous Tissue Substitute, Percutaneous Approach */
"0FU63JZ"  =  "1"  /* Supplement Left Hepatic Duct with Synthetic Substitute, Percutaneous Approach */
"0FU63KZ"  =  "1"  /* Supplement Left Hepatic Duct with Nonautologous Tissue Substitute, Percutaneous Approach */
"0FU647Z"  =  "1"  /* Supplement Left Hepatic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FU64JZ"  =  "1"  /* Supplement Left Hepatic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FU64KZ"  =  "1"  /* Supplement Left Hepatic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FU807Z"  =  "1"  /* Supplement Cystic Duct with Autologous Tissue Substitute, Open Approach */
"0FU80JZ"  =  "1"  /* Supplement Cystic Duct with Synthetic Substitute, Open Approach */
"0FU80KZ"  =  "1"  /* Supplement Cystic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FU837Z"  =  "1"  /* Supplement Cystic Duct with Autologous Tissue Substitute, Percutaneous Approach */
"0FU83JZ"  =  "1"  /* Supplement Cystic Duct with Synthetic Substitute, Percutaneous Approach */
"0FU83KZ"  =  "1"  /* Supplement Cystic Duct with Nonautologous Tissue Substitute, Percutaneous Approach */
"0FU847Z"  =  "1"  /* Supplement Cystic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FU84JZ"  =  "1"  /* Supplement Cystic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FU84KZ"  =  "1"  /* Supplement Cystic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FU907Z"  =  "1"  /* Supplement Common Bile Duct with Autologous Tissue Substitute, Open Approach */
"0FU90JZ"  =  "1"  /* Supplement Common Bile Duct with Synthetic Substitute, Open Approach */
"0FU90KZ"  =  "1"  /* Supplement Common Bile Duct with Nonautologous Tissue Substitute, Open Approach */
"0FU937Z"  =  "1"  /* Supplement Common Bile Duct with Autologous Tissue Substitute, Percutaneous Approach */
"0FU93JZ"  =  "1"  /* Supplement Common Bile Duct with Synthetic Substitute, Percutaneous Approach */
"0FU93KZ"  =  "1"  /* Supplement Common Bile Duct with Nonautologous Tissue Substitute, Percutaneous Approach */
"0FU947Z"  =  "1"  /* Supplement Common Bile Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FU94JZ"  =  "1"  /* Supplement Common Bile Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FU94KZ"  =  "1"  /* Supplement Common Bile Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FUC07Z"  =  "1"  /* Supplement Ampulla of Vater with Autologous Tissue Substitute, Open Approach */
"0FUC0JZ"  =  "1"  /* Supplement Ampulla of Vater with Synthetic Substitute, Open Approach */
"0FUC0KZ"  =  "1"  /* Supplement Ampulla of Vater with Nonautologous Tissue Substitute, Open Approach */
"0FUC37Z"  =  "1"  /* Supplement Ampulla of Vater with Autologous Tissue Substitute, Percutaneous Approach */
"0FUC3JZ"  =  "1"  /* Supplement Ampulla of Vater with Synthetic Substitute, Percutaneous Approach */
"0FUC3KZ"  =  "1"  /* Supplement Ampulla of Vater with Nonautologous Tissue Substitute, Percutaneous Approach */
"0FUC47Z"  =  "1"  /* Supplement Ampulla of Vater with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FUC4JZ"  =  "1"  /* Supplement Ampulla of Vater with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FUC4KZ"  =  "1"  /* Supplement Ampulla of Vater with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FUD07Z"  =  "1"  /* Supplement Pancreatic Duct with Autologous Tissue Substitute, Open Approach */
"0FUD0JZ"  =  "1"  /* Supplement Pancreatic Duct with Synthetic Substitute, Open Approach */
"0FUD0KZ"  =  "1"  /* Supplement Pancreatic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FUD37Z"  =  "1"  /* Supplement Pancreatic Duct with Autologous Tissue Substitute, Percutaneous Approach */
"0FUD3JZ"  =  "1"  /* Supplement Pancreatic Duct with Synthetic Substitute, Percutaneous Approach */
"0FUD3KZ"  =  "1"  /* Supplement Pancreatic Duct with Nonautologous Tissue Substitute, Percutaneous Approach */
"0FUD47Z"  =  "1"  /* Supplement Pancreatic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FUD4JZ"  =  "1"  /* Supplement Pancreatic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FUD4KZ"  =  "1"  /* Supplement Pancreatic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FUF07Z"  =  "1"  /* Supplement Accessory Pancreatic Duct with Autologous Tissue Substitute, Open Approach */
"0FUF0JZ"  =  "1"  /* Supplement Accessory Pancreatic Duct with Synthetic Substitute, Open Approach */
"0FUF0KZ"  =  "1"  /* Supplement Accessory Pancreatic Duct with Nonautologous Tissue Substitute, Open Approach */
"0FUF37Z"  =  "1"  /* Supplement Accessory Pancreatic Duct with Autologous Tissue Substitute, Percutaneous Approach */
"0FUF3JZ"  =  "1"  /* Supplement Accessory Pancreatic Duct with Synthetic Substitute, Percutaneous Approach */
"0FUF3KZ"  =  "1"  /* Supplement Accessory Pancreatic Duct with Nonautologous Tissue Substitute, Percutaneous Approach */
"0FUF47Z"  =  "1"  /* Supplement Accessory Pancreatic Duct with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FUF4JZ"  =  "1"  /* Supplement Accessory Pancreatic Duct with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0FUF4KZ"  =  "1"  /* Supplement Accessory Pancreatic Duct with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0FV50CZ"  =  "1"  /* Restriction of Right Hepatic Duct with Extraluminal Device, Open Approach */
"0FV50DZ"  =  "1"  /* Restriction of Right Hepatic Duct with Intraluminal Device, Open Approach */
"0FV50ZZ"  =  "1"  /* Restriction of Right Hepatic Duct, Open Approach */
"0FV60CZ"  =  "1"  /* Restriction of Left Hepatic Duct with Extraluminal Device, Open Approach */
"0FV60DZ"  =  "1"  /* Restriction of Left Hepatic Duct with Intraluminal Device, Open Approach */
"0FV60ZZ"  =  "1"  /* Restriction of Left Hepatic Duct, Open Approach */
"0FV80CZ"  =  "1"  /* Restriction of Cystic Duct with Extraluminal Device, Open Approach */
"0FV80DZ"  =  "1"  /* Restriction of Cystic Duct with Intraluminal Device, Open Approach */
"0FV80ZZ"  =  "1"  /* Restriction of Cystic Duct, Open Approach */
"0FV90CZ"  =  "1"  /* Restriction of Common Bile Duct with Extraluminal Device, Open Approach */
"0FV90DZ"  =  "1"  /* Restriction of Common Bile Duct with Intraluminal Device, Open Approach */
"0FV90ZZ"  =  "1"  /* Restriction of Common Bile Duct, Open Approach */
"0FVC0CZ"  =  "1"  /* Restriction of Ampulla of Vater with Extraluminal Device, Open Approach */
"0FVC0DZ"  =  "1"  /* Restriction of Ampulla of Vater with Intraluminal Device, Open Approach */
"0FVC0ZZ"  =  "1"  /* Restriction of Ampulla of Vater, Open Approach */
"0FVC3CZ"  =  "1"  /* Restriction of Ampulla of Vater with Extraluminal Device, Percutaneous Approach */
"0FVC3DZ"  =  "1"  /* Restriction of Ampulla of Vater with Intraluminal Device, Percutaneous Approach */
"0FVC3ZZ"  =  "1"  /* Restriction of Ampulla of Vater, Percutaneous Approach */
"0FVC4CZ"  =  "1"  /* Restriction of Ampulla of Vater with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FVC4DZ"  =  "1"  /* Restriction of Ampulla of Vater with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FVC4ZZ"  =  "1"  /* Restriction of Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FVC7DZ"  =  "1"  /* Restriction of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening */
"0FVC7ZZ"  =  "1"  /* Restriction of Ampulla of Vater, Via Natural or Artificial Opening */
"0FVC8DZ"  =  "1"  /* Restriction of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FVC8ZZ"  =  "1"  /* Restriction of Ampulla of Vater, Via Natural or Artificial Opening Endoscopic */
"0FVD0CZ"  =  "1"  /* Restriction of Pancreatic Duct with Extraluminal Device, Open Approach */
"0FVD0DZ"  =  "1"  /* Restriction of Pancreatic Duct with Intraluminal Device, Open Approach */
"0FVD0ZZ"  =  "1"  /* Restriction of Pancreatic Duct, Open Approach */
"0FVD3CZ"  =  "1"  /* Restriction of Pancreatic Duct with Extraluminal Device, Percutaneous Approach */
"0FVD3DZ"  =  "1"  /* Restriction of Pancreatic Duct with Intraluminal Device, Percutaneous Approach */
"0FVD3ZZ"  =  "1"  /* Restriction of Pancreatic Duct, Percutaneous Approach */
"0FVD4CZ"  =  "1"  /* Restriction of Pancreatic Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FVD4DZ"  =  "1"  /* Restriction of Pancreatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FVD4ZZ"  =  "1"  /* Restriction of Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FVD7DZ"  =  "1"  /* Restriction of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0FVD7ZZ"  =  "1"  /* Restriction of Pancreatic Duct, Via Natural or Artificial Opening */
"0FVD8DZ"  =  "1"  /* Restriction of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FVD8ZZ"  =  "1"  /* Restriction of Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FVF0CZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct with Extraluminal Device, Open Approach */
"0FVF0DZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct with Intraluminal Device, Open Approach */
"0FVF0ZZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct, Open Approach */
"0FVF3CZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct with Extraluminal Device, Percutaneous Approach */
"0FVF3DZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct with Intraluminal Device, Percutaneous Approach */
"0FVF3ZZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct, Percutaneous Approach */
"0FVF4CZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct with Extraluminal Device, Percutaneous Endoscopic Approach */
"0FVF4DZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct with Intraluminal Device, Percutaneous Endoscopic Approach */
"0FVF4ZZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FVF7DZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening */
"0FVF7ZZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct, Via Natural or Artificial Opening */
"0FVF8DZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0FVF8ZZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FW000Z"  =  "1"  /* Revision of Drainage Device in Liver, Open Approach */
"0FW002Z"  =  "1"  /* Revision of Monitoring Device in Liver, Open Approach */
"0FW003Z"  =  "1"  /* Revision of Infusion Device in Liver, Open Approach */
"0FW030Z"  =  "1"  /* Revision of Drainage Device in Liver, Percutaneous Approach */
"0FW032Z"  =  "1"  /* Revision of Monitoring Device in Liver, Percutaneous Approach */
"0FW033Z"  =  "1"  /* Revision of Infusion Device in Liver, Percutaneous Approach */
"0FW040Z"  =  "1"  /* Revision of Drainage Device in Liver, Percutaneous Endoscopic Approach */
"0FW042Z"  =  "1"  /* Revision of Monitoring Device in Liver, Percutaneous Endoscopic Approach */
"0FW043Z"  =  "1"  /* Revision of Infusion Device in Liver, Percutaneous Endoscopic Approach */
"0FW400Z"  =  "1"  /* Revision of Drainage Device in Gallbladder, Open Approach */
"0FW402Z"  =  "1"  /* Revision of Monitoring Device in Gallbladder, Open Approach */
"0FW403Z"  =  "1"  /* Revision of Infusion Device in Gallbladder, Open Approach */
"0FW40DZ"  =  "1"  /* Revision of Intraluminal Device in Gallbladder, Open Approach */
"0FW430Z"  =  "1"  /* Revision of Drainage Device in Gallbladder, Percutaneous Approach */
"0FW432Z"  =  "1"  /* Revision of Monitoring Device in Gallbladder, Percutaneous Approach */
"0FW433Z"  =  "1"  /* Revision of Infusion Device in Gallbladder, Percutaneous Approach */
"0FW43DZ"  =  "1"  /* Revision of Intraluminal Device in Gallbladder, Percutaneous Approach */
"0FW440Z"  =  "1"  /* Revision of Drainage Device in Gallbladder, Percutaneous Endoscopic Approach */
"0FW442Z"  =  "1"  /* Revision of Monitoring Device in Gallbladder, Percutaneous Endoscopic Approach */
"0FW443Z"  =  "1"  /* Revision of Infusion Device in Gallbladder, Percutaneous Endoscopic Approach */
"0FW44DZ"  =  "1"  /* Revision of Intraluminal Device in Gallbladder, Percutaneous Endoscopic Approach */
"0FWB00Z"  =  "1"  /* Revision of Drainage Device in Hepatobiliary Duct, Open Approach */
"0FWB02Z"  =  "1"  /* Revision of Monitoring Device in Hepatobiliary Duct, Open Approach */
"0FWB03Z"  =  "1"  /* Revision of Infusion Device in Hepatobiliary Duct, Open Approach */
"0FWB07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Open Approach */
"0FWB0CZ"  =  "1"  /* Revision of Extraluminal Device in Hepatobiliary Duct, Open Approach */
"0FWB0DZ"  =  "1"  /* Revision of Intraluminal Device in Hepatobiliary Duct, Open Approach */
"0FWB0JZ"  =  "1"  /* Revision of Synthetic Substitute in Hepatobiliary Duct, Open Approach */
"0FWB0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Open Approach */
"0FWB30Z"  =  "1"  /* Revision of Drainage Device in Hepatobiliary Duct, Percutaneous Approach */
"0FWB32Z"  =  "1"  /* Revision of Monitoring Device in Hepatobiliary Duct, Percutaneous Approach */
"0FWB33Z"  =  "1"  /* Revision of Infusion Device in Hepatobiliary Duct, Percutaneous Approach */
"0FWB37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Percutaneous Approach */
"0FWB3CZ"  =  "1"  /* Revision of Extraluminal Device in Hepatobiliary Duct, Percutaneous Approach */
"0FWB3DZ"  =  "1"  /* Revision of Intraluminal Device in Hepatobiliary Duct, Percutaneous Approach */
"0FWB3JZ"  =  "1"  /* Revision of Synthetic Substitute in Hepatobiliary Duct, Percutaneous Approach */
"0FWB3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Percutaneous Approach */
"0FWB40Z"  =  "1"  /* Revision of Drainage Device in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB42Z"  =  "1"  /* Revision of Monitoring Device in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB43Z"  =  "1"  /* Revision of Infusion Device in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB4CZ"  =  "1"  /* Revision of Extraluminal Device in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB4DZ"  =  "1"  /* Revision of Intraluminal Device in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB4JZ"  =  "1"  /* Revision of Synthetic Substitute in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Percutaneous Endoscopic Approach */
"0FWB70Z"  =  "1"  /* Revision of Drainage Device in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB72Z"  =  "1"  /* Revision of Monitoring Device in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB73Z"  =  "1"  /* Revision of Infusion Device in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB7CZ"  =  "1"  /* Revision of Extraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB7DZ"  =  "1"  /* Revision of Intraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB7JZ"  =  "1"  /* Revision of Synthetic Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening */
"0FWB80Z"  =  "1"  /* Revision of Drainage Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB82Z"  =  "1"  /* Revision of Monitoring Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB83Z"  =  "1"  /* Revision of Infusion Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB8CZ"  =  "1"  /* Revision of Extraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB8DZ"  =  "1"  /* Revision of Intraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB8JZ"  =  "1"  /* Revision of Synthetic Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWB8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD00Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Open Approach */
"0FWD02Z"  =  "1"  /* Revision of Monitoring Device in Pancreatic Duct, Open Approach */
"0FWD03Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Open Approach */
"0FWD07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pancreatic Duct, Open Approach */
"0FWD0CZ"  =  "1"  /* Revision of Extraluminal Device in Pancreatic Duct, Open Approach */
"0FWD0DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreatic Duct, Open Approach */
"0FWD0JZ"  =  "1"  /* Revision of Synthetic Substitute in Pancreatic Duct, Open Approach */
"0FWD0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Open Approach */
"0FWD30Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Percutaneous Approach */
"0FWD32Z"  =  "1"  /* Revision of Monitoring Device in Pancreatic Duct, Percutaneous Approach */
"0FWD33Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Percutaneous Approach */
"0FWD37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pancreatic Duct, Percutaneous Approach */
"0FWD3CZ"  =  "1"  /* Revision of Extraluminal Device in Pancreatic Duct, Percutaneous Approach */
"0FWD3DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreatic Duct, Percutaneous Approach */
"0FWD3JZ"  =  "1"  /* Revision of Synthetic Substitute in Pancreatic Duct, Percutaneous Approach */
"0FWD3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Percutaneous Approach */
"0FWD40Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD42Z"  =  "1"  /* Revision of Monitoring Device in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD43Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD4CZ"  =  "1"  /* Revision of Extraluminal Device in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD4DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD4JZ"  =  "1"  /* Revision of Synthetic Substitute in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FWD70Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD72Z"  =  "1"  /* Revision of Monitoring Device in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD73Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD7CZ"  =  "1"  /* Revision of Extraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD7DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD7JZ"  =  "1"  /* Revision of Synthetic Substitute in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening */
"0FWD80Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD82Z"  =  "1"  /* Revision of Monitoring Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD83Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD8CZ"  =  "1"  /* Revision of Extraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD8DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD8JZ"  =  "1"  /* Revision of Synthetic Substitute in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWD8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic */
"0FWG00Z"  =  "1"  /* Revision of Drainage Device in Pancreas, Open Approach */
"0FWG02Z"  =  "1"  /* Revision of Monitoring Device in Pancreas, Open Approach */
"0FWG03Z"  =  "1"  /* Revision of Infusion Device in Pancreas, Open Approach */
"0FWG0DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreas, Open Approach */
"0FWG30Z"  =  "1"  /* Revision of Drainage Device in Pancreas, Percutaneous Approach */
"0FWG32Z"  =  "1"  /* Revision of Monitoring Device in Pancreas, Percutaneous Approach */
"0FWG33Z"  =  "1"  /* Revision of Infusion Device in Pancreas, Percutaneous Approach */
"0FWG3DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreas, Percutaneous Approach */
"0FWG40Z"  =  "1"  /* Revision of Drainage Device in Pancreas, Percutaneous Endoscopic Approach */
"0FWG42Z"  =  "1"  /* Revision of Monitoring Device in Pancreas, Percutaneous Endoscopic Approach */
"0FWG43Z"  =  "1"  /* Revision of Infusion Device in Pancreas, Percutaneous Endoscopic Approach */
"0FWG4DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreas, Percutaneous Endoscopic Approach */
"0FY00Z0"  =  "1"  /* Transplantation of Liver, Allogeneic, Open Approach */
"0FY00Z1"  =  "1"  /* Transplantation of Liver, Syngeneic, Open Approach */
"0FY00Z2"  =  "1"  /* Transplantation of Liver, Zooplastic, Open Approach */
"0FYG0Z0"  =  "1"  /* Transplantation of Pancreas, Allogeneic, Open Approach */
"0FYG0Z1"  =  "1"  /* Transplantation of Pancreas, Syngeneic, Open Approach */
"0FYG0Z2"  =  "1"  /* Transplantation of Pancreas, Zooplastic, Open Approach */
"0HQ6XZZ"  =  "1"  /* Repair Back Skin, External Approach */
"0HQ7XZZ"  =  "1"  /* Repair Abdomen Skin, External Approach */
"0HQ9XZZ"  =  "1"  /* Repair Perineum Skin, External Approach */
"0T13073"  =  "1"  /* Bypass Right Kidney Pelvis to Right Kidney Pelvis with Autologous Tissue Substitute, Open Approach */
"0T13074"  =  "1"  /* Bypass Right Kidney Pelvis to Left Kidney Pelvis with Autologous Tissue Substitute, Open Approach */
"0T13076"  =  "1"  /* Bypass Right Kidney Pelvis to Right Ureter with Autologous Tissue Substitute, Open Approach */
"0T13077"  =  "1"  /* Bypass Right Kidney Pelvis to Left Ureter with Autologous Tissue Substitute, Open Approach */
"0T13078"  =  "1"  /* Bypass Right Kidney Pelvis to Colon with Autologous Tissue Substitute, Open Approach */
"0T13079"  =  "1"  /* Bypass Right Kidney Pelvis to Colocutaneous with Autologous Tissue Substitute, Open Approach */
"0T1307A"  =  "1"  /* Bypass Right Kidney Pelvis to Ileum with Autologous Tissue Substitute, Open Approach */
"0T1307B"  =  "1"  /* Bypass Right Kidney Pelvis to Bladder with Autologous Tissue Substitute, Open Approach */
"0T1307C"  =  "1"  /* Bypass Right Kidney Pelvis to Ileocutaneous with Autologous Tissue Substitute, Open Approach */
"0T1307D"  =  "1"  /* Bypass Right Kidney Pelvis to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0T130J3"  =  "1"  /* Bypass Right Kidney Pelvis to Right Kidney Pelvis with Synthetic Substitute, Open Approach */
"0T130J4"  =  "1"  /* Bypass Right Kidney Pelvis to Left Kidney Pelvis with Synthetic Substitute, Open Approach */
"0T130J6"  =  "1"  /* Bypass Right Kidney Pelvis to Right Ureter with Synthetic Substitute, Open Approach */
"0T130J7"  =  "1"  /* Bypass Right Kidney Pelvis to Left Ureter with Synthetic Substitute, Open Approach */
"0T130J8"  =  "1"  /* Bypass Right Kidney Pelvis to Colon with Synthetic Substitute, Open Approach */
"0T130J9"  =  "1"  /* Bypass Right Kidney Pelvis to Colocutaneous with Synthetic Substitute, Open Approach */
"0T130JA"  =  "1"  /* Bypass Right Kidney Pelvis to Ileum with Synthetic Substitute, Open Approach */
"0T130JB"  =  "1"  /* Bypass Right Kidney Pelvis to Bladder with Synthetic Substitute, Open Approach */
"0T130JC"  =  "1"  /* Bypass Right Kidney Pelvis to Ileocutaneous with Synthetic Substitute, Open Approach */
"0T130JD"  =  "1"  /* Bypass Right Kidney Pelvis to Cutaneous with Synthetic Substitute, Open Approach */
"0T130K3"  =  "1"  /* Bypass Right Kidney Pelvis to Right Kidney Pelvis with Nonautologous Tissue Substitute, Open Approach */
"0T130K4"  =  "1"  /* Bypass Right Kidney Pelvis to Left Kidney Pelvis with Nonautologous Tissue Substitute, Open Approach */
"0T130K6"  =  "1"  /* Bypass Right Kidney Pelvis to Right Ureter with Nonautologous Tissue Substitute, Open Approach */
"0T130K7"  =  "1"  /* Bypass Right Kidney Pelvis to Left Ureter with Nonautologous Tissue Substitute, Open Approach */
"0T130K8"  =  "1"  /* Bypass Right Kidney Pelvis to Colon with Nonautologous Tissue Substitute, Open Approach */
"0T130K9"  =  "1"  /* Bypass Right Kidney Pelvis to Colocutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T130KA"  =  "1"  /* Bypass Right Kidney Pelvis to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0T130KB"  =  "1"  /* Bypass Right Kidney Pelvis to Bladder with Nonautologous Tissue Substitute, Open Approach */
"0T130KC"  =  "1"  /* Bypass Right Kidney Pelvis to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T130KD"  =  "1"  /* Bypass Right Kidney Pelvis to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T130Z3"  =  "1"  /* Bypass Right Kidney Pelvis to Right Kidney Pelvis, Open Approach */
"0T130Z4"  =  "1"  /* Bypass Right Kidney Pelvis to Left Kidney Pelvis, Open Approach */
"0T130Z6"  =  "1"  /* Bypass Right Kidney Pelvis to Right Ureter, Open Approach */
"0T130Z7"  =  "1"  /* Bypass Right Kidney Pelvis to Left Ureter, Open Approach */
"0T130Z8"  =  "1"  /* Bypass Right Kidney Pelvis to Colon, Open Approach */
"0T130Z9"  =  "1"  /* Bypass Right Kidney Pelvis to Colocutaneous, Open Approach */
"0T130ZA"  =  "1"  /* Bypass Right Kidney Pelvis to Ileum, Open Approach */
"0T130ZB"  =  "1"  /* Bypass Right Kidney Pelvis to Bladder, Open Approach */
"0T130ZC"  =  "1"  /* Bypass Right Kidney Pelvis to Ileocutaneous, Open Approach */
"0T130ZD"  =  "1"  /* Bypass Right Kidney Pelvis to Cutaneous, Open Approach */
"0T133JD"  =  "1"  /* Bypass Right Kidney Pelvis to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0T13473"  =  "1"  /* Bypass Right Kidney Pelvis to Right Kidney Pelvis with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T13474"  =  "1"  /* Bypass Right Kidney Pelvis to Left Kidney Pelvis with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T13476"  =  "1"  /* Bypass Right Kidney Pelvis to Right Ureter with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T13477"  =  "1"  /* Bypass Right Kidney Pelvis to Left Ureter with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T13478"  =  "1"  /* Bypass Right Kidney Pelvis to Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T13479"  =  "1"  /* Bypass Right Kidney Pelvis to Colocutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1347A"  =  "1"  /* Bypass Right Kidney Pelvis to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1347B"  =  "1"  /* Bypass Right Kidney Pelvis to Bladder with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1347C"  =  "1"  /* Bypass Right Kidney Pelvis to Ileocutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1347D"  =  "1"  /* Bypass Right Kidney Pelvis to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T134J3"  =  "1"  /* Bypass Right Kidney Pelvis to Right Kidney Pelvis with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T134J4"  =  "1"  /* Bypass Right Kidney Pelvis to Left Kidney Pelvis with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T134J6"  =  "1"  /* Bypass Right Kidney Pelvis to Right Ureter with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T134J7"  =  "1"  /* Bypass Right Kidney Pelvis to Left Ureter with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T134J8"  =  "1"  /* Bypass Right Kidney Pelvis to Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T134J9"  =  "1"  /* Bypass Right Kidney Pelvis to Colocutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T134JA"  =  "1"  /* Bypass Right Kidney Pelvis to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T134JB"  =  "1"  /* Bypass Right Kidney Pelvis to Bladder with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T134JC"  =  "1"  /* Bypass Right Kidney Pelvis to Ileocutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T134JD"  =  "1"  /* Bypass Right Kidney Pelvis to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T134K3"  =  "1"  /* Bypass Right Kidney Pelvis to Right Kidney Pelvis with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T134K4"  =  "1"  /* Bypass Right Kidney Pelvis to Left Kidney Pelvis with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T134K6"  =  "1"  /* Bypass Right Kidney Pelvis to Right Ureter with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T134K7"  =  "1"  /* Bypass Right Kidney Pelvis to Left Ureter with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T134K8"  =  "1"  /* Bypass Right Kidney Pelvis to Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T134K9"  =  "1"  /* Bypass Right Kidney Pelvis to Colocutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T134KA"  =  "1"  /* Bypass Right Kidney Pelvis to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T134KB"  =  "1"  /* Bypass Right Kidney Pelvis to Bladder with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T134KC"  =  "1"  /* Bypass Right Kidney Pelvis to Ileocutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T134KD"  =  "1"  /* Bypass Right Kidney Pelvis to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T134Z3"  =  "1"  /* Bypass Right Kidney Pelvis to Right Kidney Pelvis, Percutaneous Endoscopic Approach */
"0T134Z4"  =  "1"  /* Bypass Right Kidney Pelvis to Left Kidney Pelvis, Percutaneous Endoscopic Approach */
"0T134Z6"  =  "1"  /* Bypass Right Kidney Pelvis to Right Ureter, Percutaneous Endoscopic Approach */
"0T134Z7"  =  "1"  /* Bypass Right Kidney Pelvis to Left Ureter, Percutaneous Endoscopic Approach */
"0T134Z8"  =  "1"  /* Bypass Right Kidney Pelvis to Colon, Percutaneous Endoscopic Approach */
"0T134Z9"  =  "1"  /* Bypass Right Kidney Pelvis to Colocutaneous, Percutaneous Endoscopic Approach */
"0T134ZA"  =  "1"  /* Bypass Right Kidney Pelvis to Ileum, Percutaneous Endoscopic Approach */
"0T134ZB"  =  "1"  /* Bypass Right Kidney Pelvis to Bladder, Percutaneous Endoscopic Approach */
"0T134ZC"  =  "1"  /* Bypass Right Kidney Pelvis to Ileocutaneous, Percutaneous Endoscopic Approach */
"0T134ZD"  =  "1"  /* Bypass Right Kidney Pelvis to Cutaneous, Percutaneous Endoscopic Approach */
"0T14073"  =  "1"  /* Bypass Left Kidney Pelvis to Right Kidney Pelvis with Autologous Tissue Substitute, Open Approach */
"0T14074"  =  "1"  /* Bypass Left Kidney Pelvis to Left Kidney Pelvis with Autologous Tissue Substitute, Open Approach */
"0T14076"  =  "1"  /* Bypass Left Kidney Pelvis to Right Ureter with Autologous Tissue Substitute, Open Approach */
"0T14077"  =  "1"  /* Bypass Left Kidney Pelvis to Left Ureter with Autologous Tissue Substitute, Open Approach */
"0T14078"  =  "1"  /* Bypass Left Kidney Pelvis to Colon with Autologous Tissue Substitute, Open Approach */
"0T14079"  =  "1"  /* Bypass Left Kidney Pelvis to Colocutaneous with Autologous Tissue Substitute, Open Approach */
"0T1407A"  =  "1"  /* Bypass Left Kidney Pelvis to Ileum with Autologous Tissue Substitute, Open Approach */
"0T1407B"  =  "1"  /* Bypass Left Kidney Pelvis to Bladder with Autologous Tissue Substitute, Open Approach */
"0T1407C"  =  "1"  /* Bypass Left Kidney Pelvis to Ileocutaneous with Autologous Tissue Substitute, Open Approach */
"0T1407D"  =  "1"  /* Bypass Left Kidney Pelvis to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0T140J3"  =  "1"  /* Bypass Left Kidney Pelvis to Right Kidney Pelvis with Synthetic Substitute, Open Approach */
"0T140J4"  =  "1"  /* Bypass Left Kidney Pelvis to Left Kidney Pelvis with Synthetic Substitute, Open Approach */
"0T140J6"  =  "1"  /* Bypass Left Kidney Pelvis to Right Ureter with Synthetic Substitute, Open Approach */
"0T140J7"  =  "1"  /* Bypass Left Kidney Pelvis to Left Ureter with Synthetic Substitute, Open Approach */
"0T140J8"  =  "1"  /* Bypass Left Kidney Pelvis to Colon with Synthetic Substitute, Open Approach */
"0T140J9"  =  "1"  /* Bypass Left Kidney Pelvis to Colocutaneous with Synthetic Substitute, Open Approach */
"0T140JA"  =  "1"  /* Bypass Left Kidney Pelvis to Ileum with Synthetic Substitute, Open Approach */
"0T140JB"  =  "1"  /* Bypass Left Kidney Pelvis to Bladder with Synthetic Substitute, Open Approach */
"0T140JC"  =  "1"  /* Bypass Left Kidney Pelvis to Ileocutaneous with Synthetic Substitute, Open Approach */
"0T140JD"  =  "1"  /* Bypass Left Kidney Pelvis to Cutaneous with Synthetic Substitute, Open Approach */
"0T140K3"  =  "1"  /* Bypass Left Kidney Pelvis to Right Kidney Pelvis with Nonautologous Tissue Substitute, Open Approach */
"0T140K4"  =  "1"  /* Bypass Left Kidney Pelvis to Left Kidney Pelvis with Nonautologous Tissue Substitute, Open Approach */
"0T140K6"  =  "1"  /* Bypass Left Kidney Pelvis to Right Ureter with Nonautologous Tissue Substitute, Open Approach */
"0T140K7"  =  "1"  /* Bypass Left Kidney Pelvis to Left Ureter with Nonautologous Tissue Substitute, Open Approach */
"0T140K8"  =  "1"  /* Bypass Left Kidney Pelvis to Colon with Nonautologous Tissue Substitute, Open Approach */
"0T140K9"  =  "1"  /* Bypass Left Kidney Pelvis to Colocutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T140KA"  =  "1"  /* Bypass Left Kidney Pelvis to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0T140KB"  =  "1"  /* Bypass Left Kidney Pelvis to Bladder with Nonautologous Tissue Substitute, Open Approach */
"0T140KC"  =  "1"  /* Bypass Left Kidney Pelvis to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T140KD"  =  "1"  /* Bypass Left Kidney Pelvis to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T140Z3"  =  "1"  /* Bypass Left Kidney Pelvis to Right Kidney Pelvis, Open Approach */
"0T140Z4"  =  "1"  /* Bypass Left Kidney Pelvis to Left Kidney Pelvis, Open Approach */
"0T140Z6"  =  "1"  /* Bypass Left Kidney Pelvis to Right Ureter, Open Approach */
"0T140Z7"  =  "1"  /* Bypass Left Kidney Pelvis to Left Ureter, Open Approach */
"0T140Z8"  =  "1"  /* Bypass Left Kidney Pelvis to Colon, Open Approach */
"0T140Z9"  =  "1"  /* Bypass Left Kidney Pelvis to Colocutaneous, Open Approach */
"0T140ZA"  =  "1"  /* Bypass Left Kidney Pelvis to Ileum, Open Approach */
"0T140ZB"  =  "1"  /* Bypass Left Kidney Pelvis to Bladder, Open Approach */
"0T140ZC"  =  "1"  /* Bypass Left Kidney Pelvis to Ileocutaneous, Open Approach */
"0T140ZD"  =  "1"  /* Bypass Left Kidney Pelvis to Cutaneous, Open Approach */
"0T143JD"  =  "1"  /* Bypass Left Kidney Pelvis to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0T14473"  =  "1"  /* Bypass Left Kidney Pelvis to Right Kidney Pelvis with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T14474"  =  "1"  /* Bypass Left Kidney Pelvis to Left Kidney Pelvis with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T14476"  =  "1"  /* Bypass Left Kidney Pelvis to Right Ureter with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T14477"  =  "1"  /* Bypass Left Kidney Pelvis to Left Ureter with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T14478"  =  "1"  /* Bypass Left Kidney Pelvis to Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T14479"  =  "1"  /* Bypass Left Kidney Pelvis to Colocutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1447A"  =  "1"  /* Bypass Left Kidney Pelvis to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1447B"  =  "1"  /* Bypass Left Kidney Pelvis to Bladder with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1447C"  =  "1"  /* Bypass Left Kidney Pelvis to Ileocutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1447D"  =  "1"  /* Bypass Left Kidney Pelvis to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T144J3"  =  "1"  /* Bypass Left Kidney Pelvis to Right Kidney Pelvis with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T144J4"  =  "1"  /* Bypass Left Kidney Pelvis to Left Kidney Pelvis with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T144J6"  =  "1"  /* Bypass Left Kidney Pelvis to Right Ureter with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T144J7"  =  "1"  /* Bypass Left Kidney Pelvis to Left Ureter with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T144J8"  =  "1"  /* Bypass Left Kidney Pelvis to Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T144J9"  =  "1"  /* Bypass Left Kidney Pelvis to Colocutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T144JA"  =  "1"  /* Bypass Left Kidney Pelvis to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T144JB"  =  "1"  /* Bypass Left Kidney Pelvis to Bladder with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T144JC"  =  "1"  /* Bypass Left Kidney Pelvis to Ileocutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T144JD"  =  "1"  /* Bypass Left Kidney Pelvis to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T144K3"  =  "1"  /* Bypass Left Kidney Pelvis to Right Kidney Pelvis with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T144K4"  =  "1"  /* Bypass Left Kidney Pelvis to Left Kidney Pelvis with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T144K6"  =  "1"  /* Bypass Left Kidney Pelvis to Right Ureter with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T144K7"  =  "1"  /* Bypass Left Kidney Pelvis to Left Ureter with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T144K8"  =  "1"  /* Bypass Left Kidney Pelvis to Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T144K9"  =  "1"  /* Bypass Left Kidney Pelvis to Colocutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T144KA"  =  "1"  /* Bypass Left Kidney Pelvis to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T144KB"  =  "1"  /* Bypass Left Kidney Pelvis to Bladder with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T144KC"  =  "1"  /* Bypass Left Kidney Pelvis to Ileocutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T144KD"  =  "1"  /* Bypass Left Kidney Pelvis to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T144Z3"  =  "1"  /* Bypass Left Kidney Pelvis to Right Kidney Pelvis, Percutaneous Endoscopic Approach */
"0T144Z4"  =  "1"  /* Bypass Left Kidney Pelvis to Left Kidney Pelvis, Percutaneous Endoscopic Approach */
"0T144Z6"  =  "1"  /* Bypass Left Kidney Pelvis to Right Ureter, Percutaneous Endoscopic Approach */
"0T144Z7"  =  "1"  /* Bypass Left Kidney Pelvis to Left Ureter, Percutaneous Endoscopic Approach */
"0T144Z8"  =  "1"  /* Bypass Left Kidney Pelvis to Colon, Percutaneous Endoscopic Approach */
"0T144Z9"  =  "1"  /* Bypass Left Kidney Pelvis to Colocutaneous, Percutaneous Endoscopic Approach */
"0T144ZA"  =  "1"  /* Bypass Left Kidney Pelvis to Ileum, Percutaneous Endoscopic Approach */
"0T144ZB"  =  "1"  /* Bypass Left Kidney Pelvis to Bladder, Percutaneous Endoscopic Approach */
"0T144ZC"  =  "1"  /* Bypass Left Kidney Pelvis to Ileocutaneous, Percutaneous Endoscopic Approach */
"0T144ZD"  =  "1"  /* Bypass Left Kidney Pelvis to Cutaneous, Percutaneous Endoscopic Approach */
"0T16076"  =  "1"  /* Bypass Right Ureter to Right Ureter with Autologous Tissue Substitute, Open Approach */
"0T16077"  =  "1"  /* Bypass Right Ureter to Left Ureter with Autologous Tissue Substitute, Open Approach */
"0T16078"  =  "1"  /* Bypass Right Ureter to Colon with Autologous Tissue Substitute, Open Approach */
"0T16079"  =  "1"  /* Bypass Right Ureter to Colocutaneous with Autologous Tissue Substitute, Open Approach */
"0T1607A"  =  "1"  /* Bypass Right Ureter to Ileum with Autologous Tissue Substitute, Open Approach */
"0T1607B"  =  "1"  /* Bypass Right Ureter to Bladder with Autologous Tissue Substitute, Open Approach */
"0T1607C"  =  "1"  /* Bypass Right Ureter to Ileocutaneous with Autologous Tissue Substitute, Open Approach */
"0T1607D"  =  "1"  /* Bypass Right Ureter to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0T160J6"  =  "1"  /* Bypass Right Ureter to Right Ureter with Synthetic Substitute, Open Approach */
"0T160J7"  =  "1"  /* Bypass Right Ureter to Left Ureter with Synthetic Substitute, Open Approach */
"0T160J8"  =  "1"  /* Bypass Right Ureter to Colon with Synthetic Substitute, Open Approach */
"0T160J9"  =  "1"  /* Bypass Right Ureter to Colocutaneous with Synthetic Substitute, Open Approach */
"0T160JA"  =  "1"  /* Bypass Right Ureter to Ileum with Synthetic Substitute, Open Approach */
"0T160JB"  =  "1"  /* Bypass Right Ureter to Bladder with Synthetic Substitute, Open Approach */
"0T160JC"  =  "1"  /* Bypass Right Ureter to Ileocutaneous with Synthetic Substitute, Open Approach */
"0T160JD"  =  "1"  /* Bypass Right Ureter to Cutaneous with Synthetic Substitute, Open Approach */
"0T160K6"  =  "1"  /* Bypass Right Ureter to Right Ureter with Nonautologous Tissue Substitute, Open Approach */
"0T160K7"  =  "1"  /* Bypass Right Ureter to Left Ureter with Nonautologous Tissue Substitute, Open Approach */
"0T160K8"  =  "1"  /* Bypass Right Ureter to Colon with Nonautologous Tissue Substitute, Open Approach */
"0T160K9"  =  "1"  /* Bypass Right Ureter to Colocutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T160KA"  =  "1"  /* Bypass Right Ureter to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0T160KB"  =  "1"  /* Bypass Right Ureter to Bladder with Nonautologous Tissue Substitute, Open Approach */
"0T160KC"  =  "1"  /* Bypass Right Ureter to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T160KD"  =  "1"  /* Bypass Right Ureter to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T160Z6"  =  "1"  /* Bypass Right Ureter to Right Ureter, Open Approach */
"0T160Z7"  =  "1"  /* Bypass Right Ureter to Left Ureter, Open Approach */
"0T160Z8"  =  "1"  /* Bypass Right Ureter to Colon, Open Approach */
"0T160Z9"  =  "1"  /* Bypass Right Ureter to Colocutaneous, Open Approach */
"0T160ZA"  =  "1"  /* Bypass Right Ureter to Ileum, Open Approach */
"0T160ZB"  =  "1"  /* Bypass Right Ureter to Bladder, Open Approach */
"0T160ZC"  =  "1"  /* Bypass Right Ureter to Ileocutaneous, Open Approach */
"0T160ZD"  =  "1"  /* Bypass Right Ureter to Cutaneous, Open Approach */
"0T163JD"  =  "1"  /* Bypass Right Ureter to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0T16476"  =  "1"  /* Bypass Right Ureter to Right Ureter with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T16477"  =  "1"  /* Bypass Right Ureter to Left Ureter with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T16478"  =  "1"  /* Bypass Right Ureter to Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T16479"  =  "1"  /* Bypass Right Ureter to Colocutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1647A"  =  "1"  /* Bypass Right Ureter to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1647B"  =  "1"  /* Bypass Right Ureter to Bladder with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1647C"  =  "1"  /* Bypass Right Ureter to Ileocutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1647D"  =  "1"  /* Bypass Right Ureter to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T164J6"  =  "1"  /* Bypass Right Ureter to Right Ureter with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T164J7"  =  "1"  /* Bypass Right Ureter to Left Ureter with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T164J8"  =  "1"  /* Bypass Right Ureter to Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T164J9"  =  "1"  /* Bypass Right Ureter to Colocutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T164JA"  =  "1"  /* Bypass Right Ureter to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T164JB"  =  "1"  /* Bypass Right Ureter to Bladder with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T164JC"  =  "1"  /* Bypass Right Ureter to Ileocutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T164JD"  =  "1"  /* Bypass Right Ureter to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T164K6"  =  "1"  /* Bypass Right Ureter to Right Ureter with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T164K7"  =  "1"  /* Bypass Right Ureter to Left Ureter with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T164K8"  =  "1"  /* Bypass Right Ureter to Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T164K9"  =  "1"  /* Bypass Right Ureter to Colocutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T164KA"  =  "1"  /* Bypass Right Ureter to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T164KB"  =  "1"  /* Bypass Right Ureter to Bladder with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T164KC"  =  "1"  /* Bypass Right Ureter to Ileocutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T164KD"  =  "1"  /* Bypass Right Ureter to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T164Z6"  =  "1"  /* Bypass Right Ureter to Right Ureter, Percutaneous Endoscopic Approach */
"0T164Z7"  =  "1"  /* Bypass Right Ureter to Left Ureter, Percutaneous Endoscopic Approach */
"0T164Z8"  =  "1"  /* Bypass Right Ureter to Colon, Percutaneous Endoscopic Approach */
"0T164Z9"  =  "1"  /* Bypass Right Ureter to Colocutaneous, Percutaneous Endoscopic Approach */
"0T164ZA"  =  "1"  /* Bypass Right Ureter to Ileum, Percutaneous Endoscopic Approach */
"0T164ZB"  =  "1"  /* Bypass Right Ureter to Bladder, Percutaneous Endoscopic Approach */
"0T164ZC"  =  "1"  /* Bypass Right Ureter to Ileocutaneous, Percutaneous Endoscopic Approach */
"0T164ZD"  =  "1"  /* Bypass Right Ureter to Cutaneous, Percutaneous Endoscopic Approach */
"0T17076"  =  "1"  /* Bypass Left Ureter to Right Ureter with Autologous Tissue Substitute, Open Approach */
"0T17077"  =  "1"  /* Bypass Left Ureter to Left Ureter with Autologous Tissue Substitute, Open Approach */
"0T17078"  =  "1"  /* Bypass Left Ureter to Colon with Autologous Tissue Substitute, Open Approach */
"0T17079"  =  "1"  /* Bypass Left Ureter to Colocutaneous with Autologous Tissue Substitute, Open Approach */
"0T1707A"  =  "1"  /* Bypass Left Ureter to Ileum with Autologous Tissue Substitute, Open Approach */
"0T1707B"  =  "1"  /* Bypass Left Ureter to Bladder with Autologous Tissue Substitute, Open Approach */
"0T1707C"  =  "1"  /* Bypass Left Ureter to Ileocutaneous with Autologous Tissue Substitute, Open Approach */
"0T1707D"  =  "1"  /* Bypass Left Ureter to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0T170J6"  =  "1"  /* Bypass Left Ureter to Right Ureter with Synthetic Substitute, Open Approach */
"0T170J7"  =  "1"  /* Bypass Left Ureter to Left Ureter with Synthetic Substitute, Open Approach */
"0T170J8"  =  "1"  /* Bypass Left Ureter to Colon with Synthetic Substitute, Open Approach */
"0T170J9"  =  "1"  /* Bypass Left Ureter to Colocutaneous with Synthetic Substitute, Open Approach */
"0T170JA"  =  "1"  /* Bypass Left Ureter to Ileum with Synthetic Substitute, Open Approach */
"0T170JB"  =  "1"  /* Bypass Left Ureter to Bladder with Synthetic Substitute, Open Approach */
"0T170JC"  =  "1"  /* Bypass Left Ureter to Ileocutaneous with Synthetic Substitute, Open Approach */
"0T170JD"  =  "1"  /* Bypass Left Ureter to Cutaneous with Synthetic Substitute, Open Approach */
"0T170K6"  =  "1"  /* Bypass Left Ureter to Right Ureter with Nonautologous Tissue Substitute, Open Approach */
"0T170K7"  =  "1"  /* Bypass Left Ureter to Left Ureter with Nonautologous Tissue Substitute, Open Approach */
"0T170K8"  =  "1"  /* Bypass Left Ureter to Colon with Nonautologous Tissue Substitute, Open Approach */
"0T170K9"  =  "1"  /* Bypass Left Ureter to Colocutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T170KA"  =  "1"  /* Bypass Left Ureter to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0T170KB"  =  "1"  /* Bypass Left Ureter to Bladder with Nonautologous Tissue Substitute, Open Approach */
"0T170KC"  =  "1"  /* Bypass Left Ureter to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T170KD"  =  "1"  /* Bypass Left Ureter to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T170Z6"  =  "1"  /* Bypass Left Ureter to Right Ureter, Open Approach */
"0T170Z7"  =  "1"  /* Bypass Left Ureter to Left Ureter, Open Approach */
"0T170Z8"  =  "1"  /* Bypass Left Ureter to Colon, Open Approach */
"0T170Z9"  =  "1"  /* Bypass Left Ureter to Colocutaneous, Open Approach */
"0T170ZA"  =  "1"  /* Bypass Left Ureter to Ileum, Open Approach */
"0T170ZB"  =  "1"  /* Bypass Left Ureter to Bladder, Open Approach */
"0T170ZC"  =  "1"  /* Bypass Left Ureter to Ileocutaneous, Open Approach */
"0T170ZD"  =  "1"  /* Bypass Left Ureter to Cutaneous, Open Approach */
"0T173JD"  =  "1"  /* Bypass Left Ureter to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0T17476"  =  "1"  /* Bypass Left Ureter to Right Ureter with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T17477"  =  "1"  /* Bypass Left Ureter to Left Ureter with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T17478"  =  "1"  /* Bypass Left Ureter to Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T17479"  =  "1"  /* Bypass Left Ureter to Colocutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1747A"  =  "1"  /* Bypass Left Ureter to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1747B"  =  "1"  /* Bypass Left Ureter to Bladder with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1747C"  =  "1"  /* Bypass Left Ureter to Ileocutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1747D"  =  "1"  /* Bypass Left Ureter to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T174J6"  =  "1"  /* Bypass Left Ureter to Right Ureter with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T174J7"  =  "1"  /* Bypass Left Ureter to Left Ureter with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T174J8"  =  "1"  /* Bypass Left Ureter to Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T174J9"  =  "1"  /* Bypass Left Ureter to Colocutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T174JA"  =  "1"  /* Bypass Left Ureter to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T174JB"  =  "1"  /* Bypass Left Ureter to Bladder with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T174JC"  =  "1"  /* Bypass Left Ureter to Ileocutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T174JD"  =  "1"  /* Bypass Left Ureter to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T174K6"  =  "1"  /* Bypass Left Ureter to Right Ureter with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T174K7"  =  "1"  /* Bypass Left Ureter to Left Ureter with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T174K8"  =  "1"  /* Bypass Left Ureter to Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T174K9"  =  "1"  /* Bypass Left Ureter to Colocutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T174KA"  =  "1"  /* Bypass Left Ureter to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T174KB"  =  "1"  /* Bypass Left Ureter to Bladder with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T174KC"  =  "1"  /* Bypass Left Ureter to Ileocutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T174KD"  =  "1"  /* Bypass Left Ureter to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T174Z6"  =  "1"  /* Bypass Left Ureter to Right Ureter, Percutaneous Endoscopic Approach */
"0T174Z7"  =  "1"  /* Bypass Left Ureter to Left Ureter, Percutaneous Endoscopic Approach */
"0T174Z8"  =  "1"  /* Bypass Left Ureter to Colon, Percutaneous Endoscopic Approach */
"0T174Z9"  =  "1"  /* Bypass Left Ureter to Colocutaneous, Percutaneous Endoscopic Approach */
"0T174ZA"  =  "1"  /* Bypass Left Ureter to Ileum, Percutaneous Endoscopic Approach */
"0T174ZB"  =  "1"  /* Bypass Left Ureter to Bladder, Percutaneous Endoscopic Approach */
"0T174ZC"  =  "1"  /* Bypass Left Ureter to Ileocutaneous, Percutaneous Endoscopic Approach */
"0T174ZD"  =  "1"  /* Bypass Left Ureter to Cutaneous, Percutaneous Endoscopic Approach */
"0T18076"  =  "1"  /* Bypass Bilateral Ureters to Right Ureter with Autologous Tissue Substitute, Open Approach */
"0T18077"  =  "1"  /* Bypass Bilateral Ureters to Left Ureter with Autologous Tissue Substitute, Open Approach */
"0T18078"  =  "1"  /* Bypass Bilateral Ureters to Colon with Autologous Tissue Substitute, Open Approach */
"0T18079"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous with Autologous Tissue Substitute, Open Approach */
"0T1807A"  =  "1"  /* Bypass Bilateral Ureters to Ileum with Autologous Tissue Substitute, Open Approach */
"0T1807B"  =  "1"  /* Bypass Bilateral Ureters to Bladder with Autologous Tissue Substitute, Open Approach */
"0T1807C"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous with Autologous Tissue Substitute, Open Approach */
"0T1807D"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0T180J6"  =  "1"  /* Bypass Bilateral Ureters to Right Ureter with Synthetic Substitute, Open Approach */
"0T180J7"  =  "1"  /* Bypass Bilateral Ureters to Left Ureter with Synthetic Substitute, Open Approach */
"0T180J8"  =  "1"  /* Bypass Bilateral Ureters to Colon with Synthetic Substitute, Open Approach */
"0T180J9"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous with Synthetic Substitute, Open Approach */
"0T180JA"  =  "1"  /* Bypass Bilateral Ureters to Ileum with Synthetic Substitute, Open Approach */
"0T180JB"  =  "1"  /* Bypass Bilateral Ureters to Bladder with Synthetic Substitute, Open Approach */
"0T180JC"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous with Synthetic Substitute, Open Approach */
"0T180JD"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous with Synthetic Substitute, Open Approach */
"0T180K6"  =  "1"  /* Bypass Bilateral Ureters to Right Ureter with Nonautologous Tissue Substitute, Open Approach */
"0T180K7"  =  "1"  /* Bypass Bilateral Ureters to Left Ureter with Nonautologous Tissue Substitute, Open Approach */
"0T180K8"  =  "1"  /* Bypass Bilateral Ureters to Colon with Nonautologous Tissue Substitute, Open Approach */
"0T180K9"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T180KA"  =  "1"  /* Bypass Bilateral Ureters to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0T180KB"  =  "1"  /* Bypass Bilateral Ureters to Bladder with Nonautologous Tissue Substitute, Open Approach */
"0T180KC"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T180KD"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0T180Z6"  =  "1"  /* Bypass Bilateral Ureters to Right Ureter, Open Approach */
"0T180Z7"  =  "1"  /* Bypass Bilateral Ureters to Left Ureter, Open Approach */
"0T180Z8"  =  "1"  /* Bypass Bilateral Ureters to Colon, Open Approach */
"0T180Z9"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous, Open Approach */
"0T180ZA"  =  "1"  /* Bypass Bilateral Ureters to Ileum, Open Approach */
"0T180ZB"  =  "1"  /* Bypass Bilateral Ureters to Bladder, Open Approach */
"0T180ZC"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous, Open Approach */
"0T180ZD"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous, Open Approach */
"0T183JD"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0T18476"  =  "1"  /* Bypass Bilateral Ureters to Right Ureter with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T18477"  =  "1"  /* Bypass Bilateral Ureters to Left Ureter with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T18478"  =  "1"  /* Bypass Bilateral Ureters to Colon with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T18479"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1847A"  =  "1"  /* Bypass Bilateral Ureters to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1847B"  =  "1"  /* Bypass Bilateral Ureters to Bladder with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1847C"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T1847D"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T184J6"  =  "1"  /* Bypass Bilateral Ureters to Right Ureter with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T184J7"  =  "1"  /* Bypass Bilateral Ureters to Left Ureter with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T184J8"  =  "1"  /* Bypass Bilateral Ureters to Colon with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T184J9"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T184JA"  =  "1"  /* Bypass Bilateral Ureters to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T184JB"  =  "1"  /* Bypass Bilateral Ureters to Bladder with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T184JC"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T184JD"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0T184K6"  =  "1"  /* Bypass Bilateral Ureters to Right Ureter with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T184K7"  =  "1"  /* Bypass Bilateral Ureters to Left Ureter with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T184K8"  =  "1"  /* Bypass Bilateral Ureters to Colon with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T184K9"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T184KA"  =  "1"  /* Bypass Bilateral Ureters to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T184KB"  =  "1"  /* Bypass Bilateral Ureters to Bladder with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T184KC"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T184KD"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0T184Z6"  =  "1"  /* Bypass Bilateral Ureters to Right Ureter, Percutaneous Endoscopic Approach */
"0T184Z7"  =  "1"  /* Bypass Bilateral Ureters to Left Ureter, Percutaneous Endoscopic Approach */
"0T184Z8"  =  "1"  /* Bypass Bilateral Ureters to Colon, Percutaneous Endoscopic Approach */
"0T184Z9"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous, Percutaneous Endoscopic Approach */
"0T184ZA"  =  "1"  /* Bypass Bilateral Ureters to Ileum, Percutaneous Endoscopic Approach */
"0T184ZB"  =  "1"  /* Bypass Bilateral Ureters to Bladder, Percutaneous Endoscopic Approach */
"0T184ZC"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous, Percutaneous Endoscopic Approach */
"0T184ZD"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous, Percutaneous Endoscopic Approach */
"0T500ZZ"  =  "1"  /* Destruction of Right Kidney, Open Approach */
"0T507ZZ"  =  "1"  /* Destruction of Right Kidney, Via Natural or Artificial Opening */
"0T508ZZ"  =  "1"  /* Destruction of Right Kidney, Via Natural or Artificial Opening Endoscopic */
"0T510ZZ"  =  "1"  /* Destruction of Left Kidney, Open Approach */
"0T517ZZ"  =  "1"  /* Destruction of Left Kidney, Via Natural or Artificial Opening */
"0T518ZZ"  =  "1"  /* Destruction of Left Kidney, Via Natural or Artificial Opening Endoscopic */
"0T530ZZ"  =  "1"  /* Destruction of Right Kidney Pelvis, Open Approach */
"0T537ZZ"  =  "1"  /* Destruction of Right Kidney Pelvis, Via Natural or Artificial Opening */
"0T538ZZ"  =  "1"  /* Destruction of Right Kidney Pelvis, Via Natural or Artificial Opening Endoscopic */
"0T540ZZ"  =  "1"  /* Destruction of Left Kidney Pelvis, Open Approach */
"0T547ZZ"  =  "1"  /* Destruction of Left Kidney Pelvis, Via Natural or Artificial Opening */
"0T548ZZ"  =  "1"  /* Destruction of Left Kidney Pelvis, Via Natural or Artificial Opening Endoscopic */
"0T770ZZ"  =  "1"  /* Dilation of Left Ureter, Open Approach */
"0T773ZZ"  =  "1"  /* Dilation of Left Ureter, Percutaneous Approach */
"0T774ZZ"  =  "1"  /* Dilation of Left Ureter, Percutaneous Endoscopic Approach */
"0T780ZZ"  =  "1"  /* Dilation of Bilateral Ureters, Open Approach */
"0T783ZZ"  =  "1"  /* Dilation of Bilateral Ureters, Percutaneous Approach */
"0T784ZZ"  =  "1"  /* Dilation of Bilateral Ureters, Percutaneous Endoscopic Approach */
"0T820ZZ"  =  "1"  /* Division of Bilateral Kidneys, Open Approach */
"0T823ZZ"  =  "1"  /* Division of Bilateral Kidneys, Percutaneous Approach */
"0T824ZZ"  =  "1"  /* Division of Bilateral Kidneys, Percutaneous Endoscopic Approach */
"0T9000Z"  =  "1"  /* Drainage of Right Kidney with Drainage Device, Open Approach */
"0T900ZZ"  =  "1"  /* Drainage of Right Kidney, Open Approach */
"0T9030Z"  =  "1"  /* Drainage of Right Kidney with Drainage Device, Percutaneous Approach */
"0T903ZZ"  =  "1"  /* Drainage of Right Kidney, Percutaneous Approach */
"0T9040Z"  =  "1"  /* Drainage of Right Kidney with Drainage Device, Percutaneous Endoscopic Approach */
"0T904ZZ"  =  "1"  /* Drainage of Right Kidney, Percutaneous Endoscopic Approach */
"0T9100Z"  =  "1"  /* Drainage of Left Kidney with Drainage Device, Open Approach */
"0T910ZZ"  =  "1"  /* Drainage of Left Kidney, Open Approach */
"0T9130Z"  =  "1"  /* Drainage of Left Kidney with Drainage Device, Percutaneous Approach */
"0T913ZZ"  =  "1"  /* Drainage of Left Kidney, Percutaneous Approach */
"0T9140Z"  =  "1"  /* Drainage of Left Kidney with Drainage Device, Percutaneous Endoscopic Approach */
"0T914ZZ"  =  "1"  /* Drainage of Left Kidney, Percutaneous Endoscopic Approach */
"0T9300Z"  =  "1"  /* Drainage of Right Kidney Pelvis with Drainage Device, Open Approach */
"0T930ZZ"  =  "1"  /* Drainage of Right Kidney Pelvis, Open Approach */
"0T9330Z"  =  "1"  /* Drainage of Right Kidney Pelvis with Drainage Device, Percutaneous Approach */
"0T9340Z"  =  "1"  /* Drainage of Right Kidney Pelvis with Drainage Device, Percutaneous Endoscopic Approach */
"0T9400Z"  =  "1"  /* Drainage of Left Kidney Pelvis with Drainage Device, Open Approach */
"0T940ZZ"  =  "1"  /* Drainage of Left Kidney Pelvis, Open Approach */
"0T9430Z"  =  "1"  /* Drainage of Left Kidney Pelvis with Drainage Device, Percutaneous Approach */
"0T9440Z"  =  "1"  /* Drainage of Left Kidney Pelvis with Drainage Device, Percutaneous Endoscopic Approach */
"0TD00ZZ"  =  "1"  /* Extraction of Right Kidney, Open Approach */
"0TD03ZZ"  =  "1"  /* Extraction of Right Kidney, Percutaneous Approach */
"0TD04ZZ"  =  "1"  /* Extraction of Right Kidney, Percutaneous Endoscopic Approach */
"0TD10ZZ"  =  "1"  /* Extraction of Left Kidney, Open Approach */
"0TD13ZZ"  =  "1"  /* Extraction of Left Kidney, Percutaneous Approach */
"0TD14ZZ"  =  "1"  /* Extraction of Left Kidney, Percutaneous Endoscopic Approach */
"0TJ50ZZ"  =  "1"  /* Inspection of Kidney, Open Approach */
"0TL60CZ"  =  "1"  /* Occlusion of Right Ureter with Extraluminal Device, Open Approach */
"0TL60ZZ"  =  "1"  /* Occlusion of Right Ureter, Open Approach */
"0TL63CZ"  =  "1"  /* Occlusion of Right Ureter with Extraluminal Device, Percutaneous Approach */
"0TL63ZZ"  =  "1"  /* Occlusion of Right Ureter, Percutaneous Approach */
"0TL64CZ"  =  "1"  /* Occlusion of Right Ureter with Extraluminal Device, Percutaneous Endoscopic Approach */
"0TL64ZZ"  =  "1"  /* Occlusion of Right Ureter, Percutaneous Endoscopic Approach */
"0TL67ZZ"  =  "1"  /* Occlusion of Right Ureter, Via Natural or Artificial Opening */
"0TL68ZZ"  =  "1"  /* Occlusion of Right Ureter, Via Natural or Artificial Opening Endoscopic */
"0TL70CZ"  =  "1"  /* Occlusion of Left Ureter with Extraluminal Device, Open Approach */
"0TL70ZZ"  =  "1"  /* Occlusion of Left Ureter, Open Approach */
"0TL73CZ"  =  "1"  /* Occlusion of Left Ureter with Extraluminal Device, Percutaneous Approach */
"0TL73ZZ"  =  "1"  /* Occlusion of Left Ureter, Percutaneous Approach */
"0TL74CZ"  =  "1"  /* Occlusion of Left Ureter with Extraluminal Device, Percutaneous Endoscopic Approach */
"0TL74ZZ"  =  "1"  /* Occlusion of Left Ureter, Percutaneous Endoscopic Approach */
"0TL77ZZ"  =  "1"  /* Occlusion of Left Ureter, Via Natural or Artificial Opening */
"0TL78ZZ"  =  "1"  /* Occlusion of Left Ureter, Via Natural or Artificial Opening Endoscopic */
"0TM60ZZ"  =  "1"  /* Reattachment of Right Ureter, Open Approach */
"0TM64ZZ"  =  "1"  /* Reattachment of Right Ureter, Percutaneous Endoscopic Approach */
"0TM70ZZ"  =  "1"  /* Reattachment of Left Ureter, Open Approach */
"0TM74ZZ"  =  "1"  /* Reattachment of Left Ureter, Percutaneous Endoscopic Approach */
"0TM80ZZ"  =  "1"  /* Reattachment of Bilateral Ureters, Open Approach */
"0TM84ZZ"  =  "1"  /* Reattachment of Bilateral Ureters, Percutaneous Endoscopic Approach */
"0TN00ZZ"  =  "1"  /* Release Right Kidney, Open Approach */
"0TN03ZZ"  =  "1"  /* Release Right Kidney, Percutaneous Approach */
"0TN07ZZ"  =  "1"  /* Release Right Kidney, Via Natural or Artificial Opening */
"0TN08ZZ"  =  "1"  /* Release Right Kidney, Via Natural or Artificial Opening Endoscopic */
"0TN10ZZ"  =  "1"  /* Release Left Kidney, Open Approach */
"0TN13ZZ"  =  "1"  /* Release Left Kidney, Percutaneous Approach */
"0TN17ZZ"  =  "1"  /* Release Left Kidney, Via Natural or Artificial Opening */
"0TN18ZZ"  =  "1"  /* Release Left Kidney, Via Natural or Artificial Opening Endoscopic */
"0TN30ZZ"  =  "1"  /* Release Right Kidney Pelvis, Open Approach */
"0TN33ZZ"  =  "1"  /* Release Right Kidney Pelvis, Percutaneous Approach */
"0TN37ZZ"  =  "1"  /* Release Right Kidney Pelvis, Via Natural or Artificial Opening */
"0TN38ZZ"  =  "1"  /* Release Right Kidney Pelvis, Via Natural or Artificial Opening Endoscopic */
"0TN40ZZ"  =  "1"  /* Release Left Kidney Pelvis, Open Approach */
"0TN43ZZ"  =  "1"  /* Release Left Kidney Pelvis, Percutaneous Approach */
"0TN47ZZ"  =  "1"  /* Release Left Kidney Pelvis, Via Natural or Artificial Opening */
"0TN48ZZ"  =  "1"  /* Release Left Kidney Pelvis, Via Natural or Artificial Opening Endoscopic */
"0TN60ZZ"  =  "1"  /* Release Right Ureter, Open Approach */
"0TN63ZZ"  =  "1"  /* Release Right Ureter, Percutaneous Approach */
"0TN70ZZ"  =  "1"  /* Release Left Ureter, Open Approach */
"0TN73ZZ"  =  "1"  /* Release Left Ureter, Percutaneous Approach */
"0TQ00ZZ"  =  "1"  /* Repair Right Kidney, Open Approach */
"0TQ03ZZ"  =  "1"  /* Repair Right Kidney, Percutaneous Approach */
"0TQ04ZZ"  =  "1"  /* Repair Right Kidney, Percutaneous Endoscopic Approach */
"0TQ10ZZ"  =  "1"  /* Repair Left Kidney, Open Approach */
"0TQ13ZZ"  =  "1"  /* Repair Left Kidney, Percutaneous Approach */
"0TQ14ZZ"  =  "1"  /* Repair Left Kidney, Percutaneous Endoscopic Approach */
"0TQ30ZZ"  =  "1"  /* Repair Right Kidney Pelvis, Open Approach */
"0TQ33ZZ"  =  "1"  /* Repair Right Kidney Pelvis, Percutaneous Approach */
"0TQ34ZZ"  =  "1"  /* Repair Right Kidney Pelvis, Percutaneous Endoscopic Approach */
"0TQ37ZZ"  =  "1"  /* Repair Right Kidney Pelvis, Via Natural or Artificial Opening */
"0TQ38ZZ"  =  "1"  /* Repair Right Kidney Pelvis, Via Natural or Artificial Opening Endoscopic */
"0TQ40ZZ"  =  "1"  /* Repair Left Kidney Pelvis, Open Approach */
"0TQ43ZZ"  =  "1"  /* Repair Left Kidney Pelvis, Percutaneous Approach */
"0TQ44ZZ"  =  "1"  /* Repair Left Kidney Pelvis, Percutaneous Endoscopic Approach */
"0TQ47ZZ"  =  "1"  /* Repair Left Kidney Pelvis, Via Natural or Artificial Opening */
"0TQ48ZZ"  =  "1"  /* Repair Left Kidney Pelvis, Via Natural or Artificial Opening Endoscopic */
"0TQ60ZZ"  =  "1"  /* Repair Right Ureter, Open Approach */
"0TQ63ZZ"  =  "1"  /* Repair Right Ureter, Percutaneous Approach */
"0TQ64ZZ"  =  "1"  /* Repair Right Ureter, Percutaneous Endoscopic Approach */
"0TQ67ZZ"  =  "1"  /* Repair Right Ureter, Via Natural or Artificial Opening */
"0TQ68ZZ"  =  "1"  /* Repair Right Ureter, Via Natural or Artificial Opening Endoscopic */
"0TQ70ZZ"  =  "1"  /* Repair Left Ureter, Open Approach */
"0TQ73ZZ"  =  "1"  /* Repair Left Ureter, Percutaneous Approach */
"0TQ74ZZ"  =  "1"  /* Repair Left Ureter, Percutaneous Endoscopic Approach */
"0TQ77ZZ"  =  "1"  /* Repair Left Ureter, Via Natural or Artificial Opening */
"0TQ78ZZ"  =  "1"  /* Repair Left Ureter, Via Natural or Artificial Opening Endoscopic */
"0TQB0ZZ"  =  "1"  /* Repair Bladder, Open Approach */
"0TQB3ZZ"  =  "1"  /* Repair Bladder, Percutaneous Approach */
"0TQB4ZZ"  =  "1"  /* Repair Bladder, Percutaneous Endoscopic Approach */
"0TQB7ZZ"  =  "1"  /* Repair Bladder, Via Natural or Artificial Opening */
"0TQB8ZZ"  =  "1"  /* Repair Bladder, Via Natural or Artificial Opening Endoscopic */
"0TR607Z"  =  "1"  /* Replacement of Right Ureter with Autologous Tissue Substitute, Open Approach */
"0TR60JZ"  =  "1"  /* Replacement of Right Ureter with Synthetic Substitute, Open Approach */
"0TR60KZ"  =  "1"  /* Replacement of Right Ureter with Nonautologous Tissue Substitute, Open Approach */
"0TR647Z"  =  "1"  /* Replacement of Right Ureter with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0TR64JZ"  =  "1"  /* Replacement of Right Ureter with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0TR64KZ"  =  "1"  /* Replacement of Right Ureter with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0TR677Z"  =  "1"  /* Replacement of Right Ureter with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0TR67JZ"  =  "1"  /* Replacement of Right Ureter with Synthetic Substitute, Via Natural or Artificial Opening */
"0TR67KZ"  =  "1"  /* Replacement of Right Ureter with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0TR687Z"  =  "1"  /* Replacement of Right Ureter with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0TR68JZ"  =  "1"  /* Replacement of Right Ureter with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0TR68KZ"  =  "1"  /* Replacement of Right Ureter with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0TR707Z"  =  "1"  /* Replacement of Left Ureter with Autologous Tissue Substitute, Open Approach */
"0TR70JZ"  =  "1"  /* Replacement of Left Ureter with Synthetic Substitute, Open Approach */
"0TR70KZ"  =  "1"  /* Replacement of Left Ureter with Nonautologous Tissue Substitute, Open Approach */
"0TR747Z"  =  "1"  /* Replacement of Left Ureter with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0TR74JZ"  =  "1"  /* Replacement of Left Ureter with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0TR74KZ"  =  "1"  /* Replacement of Left Ureter with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0TR777Z"  =  "1"  /* Replacement of Left Ureter with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0TR77JZ"  =  "1"  /* Replacement of Left Ureter with Synthetic Substitute, Via Natural or Artificial Opening */
"0TR77KZ"  =  "1"  /* Replacement of Left Ureter with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0TR787Z"  =  "1"  /* Replacement of Left Ureter with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0TR78JZ"  =  "1"  /* Replacement of Left Ureter with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0TR78KZ"  =  "1"  /* Replacement of Left Ureter with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0TRB07Z"  =  "1"  /* Replacement of Bladder with Autologous Tissue Substitute, Open Approach */
"0TRB0JZ"  =  "1"  /* Replacement of Bladder with Synthetic Substitute, Open Approach */
"0TRB0KZ"  =  "1"  /* Replacement of Bladder with Nonautologous Tissue Substitute, Open Approach */
"0TRB47Z"  =  "1"  /* Replacement of Bladder with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0TRB4JZ"  =  "1"  /* Replacement of Bladder with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0TRB4KZ"  =  "1"  /* Replacement of Bladder with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0TRB77Z"  =  "1"  /* Replacement of Bladder with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0TRB7JZ"  =  "1"  /* Replacement of Bladder with Synthetic Substitute, Via Natural or Artificial Opening */
"0TRB7KZ"  =  "1"  /* Replacement of Bladder with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0TRB87Z"  =  "1"  /* Replacement of Bladder with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0TRB8JZ"  =  "1"  /* Replacement of Bladder with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0TRB8KZ"  =  "1"  /* Replacement of Bladder with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0TRC07Z"  =  "1"  /* Replacement of Bladder Neck with Autologous Tissue Substitute, Open Approach */
"0TRC0JZ"  =  "1"  /* Replacement of Bladder Neck with Synthetic Substitute, Open Approach */
"0TRC0KZ"  =  "1"  /* Replacement of Bladder Neck with Nonautologous Tissue Substitute, Open Approach */
"0TRC47Z"  =  "1"  /* Replacement of Bladder Neck with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0TRC4JZ"  =  "1"  /* Replacement of Bladder Neck with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0TRC4KZ"  =  "1"  /* Replacement of Bladder Neck with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0TRC77Z"  =  "1"  /* Replacement of Bladder Neck with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0TRC7JZ"  =  "1"  /* Replacement of Bladder Neck with Synthetic Substitute, Via Natural or Artificial Opening */
"0TRC7KZ"  =  "1"  /* Replacement of Bladder Neck with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0TRC87Z"  =  "1"  /* Replacement of Bladder Neck with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0TRC8JZ"  =  "1"  /* Replacement of Bladder Neck with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0TRC8KZ"  =  "1"  /* Replacement of Bladder Neck with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0TS00ZZ"  =  "1"  /* Reposition Right Kidney, Open Approach */
"0TS04ZZ"  =  "1"  /* Reposition Right Kidney, Percutaneous Endoscopic Approach */
"0TS10ZZ"  =  "1"  /* Reposition Left Kidney, Open Approach */
"0TS14ZZ"  =  "1"  /* Reposition Left Kidney, Percutaneous Endoscopic Approach */
"0TS20ZZ"  =  "1"  /* Reposition Bilateral Kidneys, Open Approach */
"0TS24ZZ"  =  "1"  /* Reposition Bilateral Kidneys, Percutaneous Endoscopic Approach */
"0TS60ZZ"  =  "1"  /* Reposition Right Ureter, Open Approach */
"0TS64ZZ"  =  "1"  /* Reposition Right Ureter, Percutaneous Endoscopic Approach */
"0TS70ZZ"  =  "1"  /* Reposition Left Ureter, Open Approach */
"0TS74ZZ"  =  "1"  /* Reposition Left Ureter, Percutaneous Endoscopic Approach */
"0TS80ZZ"  =  "1"  /* Reposition Bilateral Ureters, Open Approach */
"0TS84ZZ"  =  "1"  /* Reposition Bilateral Ureters, Percutaneous Endoscopic Approach */
"0TT00ZZ"  =  "1"  /* Resection of Right Kidney, Open Approach */
"0TT04ZZ"  =  "1"  /* Resection of Right Kidney, Percutaneous Endoscopic Approach */
"0TT10ZZ"  =  "1"  /* Resection of Left Kidney, Open Approach */
"0TT14ZZ"  =  "1"  /* Resection of Left Kidney, Percutaneous Endoscopic Approach */
"0TT20ZZ"  =  "1"  /* Resection of Bilateral Kidneys, Open Approach */
"0TT24ZZ"  =  "1"  /* Resection of Bilateral Kidneys, Percutaneous Endoscopic Approach */
"0TT60ZZ"  =  "1"  /* Resection of Right Ureter, Open Approach */
"0TT64ZZ"  =  "1"  /* Resection of Right Ureter, Percutaneous Endoscopic Approach */
"0TT67ZZ"  =  "1"  /* Resection of Right Ureter, Via Natural or Artificial Opening */
"0TT68ZZ"  =  "1"  /* Resection of Right Ureter, Via Natural or Artificial Opening Endoscopic */
"0TT70ZZ"  =  "1"  /* Resection of Left Ureter, Open Approach */
"0TT74ZZ"  =  "1"  /* Resection of Left Ureter, Percutaneous Endoscopic Approach */
"0TT77ZZ"  =  "1"  /* Resection of Left Ureter, Via Natural or Artificial Opening */
"0TT78ZZ"  =  "1"  /* Resection of Left Ureter, Via Natural or Artificial Opening Endoscopic */
"0TTB0ZZ"  =  "1"  /* Resection of Bladder, Open Approach */
"0TTB4ZZ"  =  "1"  /* Resection of Bladder, Percutaneous Endoscopic Approach */
"0TTB7ZZ"  =  "1"  /* Resection of Bladder, Via Natural or Artificial Opening */
"0TTB8ZZ"  =  "1"  /* Resection of Bladder, Via Natural or Artificial Opening Endoscopic */
"0TTD0ZZ"  =  "1"  /* Resection of Urethra, Open Approach */
"0TTD4ZZ"  =  "1"  /* Resection of Urethra, Percutaneous Endoscopic Approach */
"0TTD7ZZ"  =  "1"  /* Resection of Urethra, Via Natural or Artificial Opening */
"0TTD8ZZ"  =  "1"  /* Resection of Urethra, Via Natural or Artificial Opening Endoscopic */
"0TU607Z"  =  "1"  /* Supplement Right Ureter with Autologous Tissue Substitute, Open Approach */
"0TU60JZ"  =  "1"  /* Supplement Right Ureter with Synthetic Substitute, Open Approach */
"0TU60KZ"  =  "1"  /* Supplement Right Ureter with Nonautologous Tissue Substitute, Open Approach */
"0TU647Z"  =  "1"  /* Supplement Right Ureter with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0TU64JZ"  =  "1"  /* Supplement Right Ureter with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0TU64KZ"  =  "1"  /* Supplement Right Ureter with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0TU677Z"  =  "1"  /* Supplement Right Ureter with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0TU67JZ"  =  "1"  /* Supplement Right Ureter with Synthetic Substitute, Via Natural or Artificial Opening */
"0TU67KZ"  =  "1"  /* Supplement Right Ureter with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0TU687Z"  =  "1"  /* Supplement Right Ureter with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0TU68JZ"  =  "1"  /* Supplement Right Ureter with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0TU68KZ"  =  "1"  /* Supplement Right Ureter with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0TU707Z"  =  "1"  /* Supplement Left Ureter with Autologous Tissue Substitute, Open Approach */
"0TU70JZ"  =  "1"  /* Supplement Left Ureter with Synthetic Substitute, Open Approach */
"0TU70KZ"  =  "1"  /* Supplement Left Ureter with Nonautologous Tissue Substitute, Open Approach */
"0TU747Z"  =  "1"  /* Supplement Left Ureter with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0TU74JZ"  =  "1"  /* Supplement Left Ureter with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0TU74KZ"  =  "1"  /* Supplement Left Ureter with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0TU777Z"  =  "1"  /* Supplement Left Ureter with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0TU77JZ"  =  "1"  /* Supplement Left Ureter with Synthetic Substitute, Via Natural or Artificial Opening */
"0TU77KZ"  =  "1"  /* Supplement Left Ureter with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0TU787Z"  =  "1"  /* Supplement Left Ureter with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0TU78JZ"  =  "1"  /* Supplement Left Ureter with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0TU78KZ"  =  "1"  /* Supplement Left Ureter with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0TUB07Z"  =  "1"  /* Supplement Bladder with Autologous Tissue Substitute, Open Approach */
"0TUB0JZ"  =  "1"  /* Supplement Bladder with Synthetic Substitute, Open Approach */
"0TUB0KZ"  =  "1"  /* Supplement Bladder with Nonautologous Tissue Substitute, Open Approach */
"0TUB47Z"  =  "1"  /* Supplement Bladder with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0TUB4JZ"  =  "1"  /* Supplement Bladder with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0TUB4KZ"  =  "1"  /* Supplement Bladder with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0TUB77Z"  =  "1"  /* Supplement Bladder with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0TUB7JZ"  =  "1"  /* Supplement Bladder with Synthetic Substitute, Via Natural or Artificial Opening */
"0TUB7KZ"  =  "1"  /* Supplement Bladder with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0TUB87Z"  =  "1"  /* Supplement Bladder with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0TUB8JZ"  =  "1"  /* Supplement Bladder with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0TUB8KZ"  =  "1"  /* Supplement Bladder with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0TV60CZ"  =  "1"  /* Restriction of Right Ureter with Extraluminal Device, Open Approach */
"0TV60DZ"  =  "1"  /* Restriction of Right Ureter with Intraluminal Device, Open Approach */
"0TV60ZZ"  =  "1"  /* Restriction of Right Ureter, Open Approach */
"0TV63CZ"  =  "1"  /* Restriction of Right Ureter with Extraluminal Device, Percutaneous Approach */
"0TV63DZ"  =  "1"  /* Restriction of Right Ureter with Intraluminal Device, Percutaneous Approach */
"0TV63ZZ"  =  "1"  /* Restriction of Right Ureter, Percutaneous Approach */
"0TV64CZ"  =  "1"  /* Restriction of Right Ureter with Extraluminal Device, Percutaneous Endoscopic Approach */
"0TV64DZ"  =  "1"  /* Restriction of Right Ureter with Intraluminal Device, Percutaneous Endoscopic Approach */
"0TV64ZZ"  =  "1"  /* Restriction of Right Ureter, Percutaneous Endoscopic Approach */
"0TV67DZ"  =  "1"  /* Restriction of Right Ureter with Intraluminal Device, Via Natural or Artificial Opening */
"0TV67ZZ"  =  "1"  /* Restriction of Right Ureter, Via Natural or Artificial Opening */
"0TV68DZ"  =  "1"  /* Restriction of Right Ureter with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0TV68ZZ"  =  "1"  /* Restriction of Right Ureter, Via Natural or Artificial Opening Endoscopic */
"0TV70CZ"  =  "1"  /* Restriction of Left Ureter with Extraluminal Device, Open Approach */
"0TV70DZ"  =  "1"  /* Restriction of Left Ureter with Intraluminal Device, Open Approach */
"0TV70ZZ"  =  "1"  /* Restriction of Left Ureter, Open Approach */
"0TV73CZ"  =  "1"  /* Restriction of Left Ureter with Extraluminal Device, Percutaneous Approach */
"0TV73DZ"  =  "1"  /* Restriction of Left Ureter with Intraluminal Device, Percutaneous Approach */
"0TV73ZZ"  =  "1"  /* Restriction of Left Ureter, Percutaneous Approach */
"0TV74CZ"  =  "1"  /* Restriction of Left Ureter with Extraluminal Device, Percutaneous Endoscopic Approach */
"0TV74DZ"  =  "1"  /* Restriction of Left Ureter with Intraluminal Device, Percutaneous Endoscopic Approach */
"0TV74ZZ"  =  "1"  /* Restriction of Left Ureter, Percutaneous Endoscopic Approach */
"0TV77DZ"  =  "1"  /* Restriction of Left Ureter with Intraluminal Device, Via Natural or Artificial Opening */
"0TV77ZZ"  =  "1"  /* Restriction of Left Ureter, Via Natural or Artificial Opening */
"0TV78DZ"  =  "1"  /* Restriction of Left Ureter with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0TV78ZZ"  =  "1"  /* Restriction of Left Ureter, Via Natural or Artificial Opening Endoscopic */
"0TY00Z0"  =  "1"  /* Transplantation of Right Kidney, Allogeneic, Open Approach */
"0TY00Z1"  =  "1"  /* Transplantation of Right Kidney, Syngeneic, Open Approach */
"0TY00Z2"  =  "1"  /* Transplantation of Right Kidney, Zooplastic, Open Approach */
"0TY10Z0"  =  "1"  /* Transplantation of Left Kidney, Allogeneic, Open Approach */
"0TY10Z1"  =  "1"  /* Transplantation of Left Kidney, Syngeneic, Open Approach */
"0TY10Z2"  =  "1"  /* Transplantation of Left Kidney, Zooplastic, Open Approach */
"0U15075"  =  "1"  /* Bypass Right Fallopian Tube to Right Fallopian Tube with Autologous Tissue Substitute, Open Approach */
"0U15076"  =  "1"  /* Bypass Right Fallopian Tube to Left Fallopian Tube with Autologous Tissue Substitute, Open Approach */
"0U15079"  =  "1"  /* Bypass Right Fallopian Tube to Uterus with Autologous Tissue Substitute, Open Approach */
"0U150J5"  =  "1"  /* Bypass Right Fallopian Tube to Right Fallopian Tube with Synthetic Substitute, Open Approach */
"0U150J6"  =  "1"  /* Bypass Right Fallopian Tube to Left Fallopian Tube with Synthetic Substitute, Open Approach */
"0U150J9"  =  "1"  /* Bypass Right Fallopian Tube to Uterus with Synthetic Substitute, Open Approach */
"0U150K5"  =  "1"  /* Bypass Right Fallopian Tube to Right Fallopian Tube with Nonautologous Tissue Substitute, Open Approach */
"0U150K6"  =  "1"  /* Bypass Right Fallopian Tube to Left Fallopian Tube with Nonautologous Tissue Substitute, Open Approach */
"0U150K9"  =  "1"  /* Bypass Right Fallopian Tube to Uterus with Nonautologous Tissue Substitute, Open Approach */
"0U150Z5"  =  "1"  /* Bypass Right Fallopian Tube to Right Fallopian Tube, Open Approach */
"0U150Z6"  =  "1"  /* Bypass Right Fallopian Tube to Left Fallopian Tube, Open Approach */
"0U150Z9"  =  "1"  /* Bypass Right Fallopian Tube to Uterus, Open Approach */
"0U15475"  =  "1"  /* Bypass Right Fallopian Tube to Right Fallopian Tube with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U15476"  =  "1"  /* Bypass Right Fallopian Tube to Left Fallopian Tube with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U15479"  =  "1"  /* Bypass Right Fallopian Tube to Uterus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U154J5"  =  "1"  /* Bypass Right Fallopian Tube to Right Fallopian Tube with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0U154J6"  =  "1"  /* Bypass Right Fallopian Tube to Left Fallopian Tube with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0U154J9"  =  "1"  /* Bypass Right Fallopian Tube to Uterus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0U154K5"  =  "1"  /* Bypass Right Fallopian Tube to Right Fallopian Tube with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U154K6"  =  "1"  /* Bypass Right Fallopian Tube to Left Fallopian Tube with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U154K9"  =  "1"  /* Bypass Right Fallopian Tube to Uterus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U154Z5"  =  "1"  /* Bypass Right Fallopian Tube to Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0U154Z6"  =  "1"  /* Bypass Right Fallopian Tube to Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0U154Z9"  =  "1"  /* Bypass Right Fallopian Tube to Uterus, Percutaneous Endoscopic Approach */
"0U16075"  =  "1"  /* Bypass Left Fallopian Tube to Right Fallopian Tube with Autologous Tissue Substitute, Open Approach */
"0U16076"  =  "1"  /* Bypass Left Fallopian Tube to Left Fallopian Tube with Autologous Tissue Substitute, Open Approach */
"0U16079"  =  "1"  /* Bypass Left Fallopian Tube to Uterus with Autologous Tissue Substitute, Open Approach */
"0U160J5"  =  "1"  /* Bypass Left Fallopian Tube to Right Fallopian Tube with Synthetic Substitute, Open Approach */
"0U160J6"  =  "1"  /* Bypass Left Fallopian Tube to Left Fallopian Tube with Synthetic Substitute, Open Approach */
"0U160J9"  =  "1"  /* Bypass Left Fallopian Tube to Uterus with Synthetic Substitute, Open Approach */
"0U160K5"  =  "1"  /* Bypass Left Fallopian Tube to Right Fallopian Tube with Nonautologous Tissue Substitute, Open Approach */
"0U160K6"  =  "1"  /* Bypass Left Fallopian Tube to Left Fallopian Tube with Nonautologous Tissue Substitute, Open Approach */
"0U160K9"  =  "1"  /* Bypass Left Fallopian Tube to Uterus with Nonautologous Tissue Substitute, Open Approach */
"0U160Z5"  =  "1"  /* Bypass Left Fallopian Tube to Right Fallopian Tube, Open Approach */
"0U160Z6"  =  "1"  /* Bypass Left Fallopian Tube to Left Fallopian Tube, Open Approach */
"0U160Z9"  =  "1"  /* Bypass Left Fallopian Tube to Uterus, Open Approach */
"0U16475"  =  "1"  /* Bypass Left Fallopian Tube to Right Fallopian Tube with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U16476"  =  "1"  /* Bypass Left Fallopian Tube to Left Fallopian Tube with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U16479"  =  "1"  /* Bypass Left Fallopian Tube to Uterus with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U164J5"  =  "1"  /* Bypass Left Fallopian Tube to Right Fallopian Tube with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0U164J6"  =  "1"  /* Bypass Left Fallopian Tube to Left Fallopian Tube with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0U164J9"  =  "1"  /* Bypass Left Fallopian Tube to Uterus with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0U164K5"  =  "1"  /* Bypass Left Fallopian Tube to Right Fallopian Tube with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U164K6"  =  "1"  /* Bypass Left Fallopian Tube to Left Fallopian Tube with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U164K9"  =  "1"  /* Bypass Left Fallopian Tube to Uterus with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0U164Z5"  =  "1"  /* Bypass Left Fallopian Tube to Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0U164Z6"  =  "1"  /* Bypass Left Fallopian Tube to Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0U164Z9"  =  "1"  /* Bypass Left Fallopian Tube to Uterus, Percutaneous Endoscopic Approach */
"0U500ZZ"  =  "1"  /* Destruction of Right Ovary, Open Approach */
"0U503ZZ"  =  "1"  /* Destruction of Right Ovary, Percutaneous Approach */
"0U510ZZ"  =  "1"  /* Destruction of Left Ovary, Open Approach */
"0U513ZZ"  =  "1"  /* Destruction of Left Ovary, Percutaneous Approach */
"0U520ZZ"  =  "1"  /* Destruction of Bilateral Ovaries, Open Approach */
"0U523ZZ"  =  "1"  /* Destruction of Bilateral Ovaries, Percutaneous Approach */
"0U550ZZ"  =  "1"  /* Destruction of Right Fallopian Tube, Open Approach */
"0U553ZZ"  =  "1"  /* Destruction of Right Fallopian Tube, Percutaneous Approach */
"0U554ZZ"  =  "1"  /* Destruction of Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0U557ZZ"  =  "1"  /* Destruction of Right Fallopian Tube, Via Natural or Artificial Opening */
"0U558ZZ"  =  "1"  /* Destruction of Right Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0U560ZZ"  =  "1"  /* Destruction of Left Fallopian Tube, Open Approach */
"0U563ZZ"  =  "1"  /* Destruction of Left Fallopian Tube, Percutaneous Approach */
"0U564ZZ"  =  "1"  /* Destruction of Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0U567ZZ"  =  "1"  /* Destruction of Left Fallopian Tube, Via Natural or Artificial Opening */
"0U568ZZ"  =  "1"  /* Destruction of Left Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0U570ZZ"  =  "1"  /* Destruction of Bilateral Fallopian Tubes, Open Approach */
"0U573ZZ"  =  "1"  /* Destruction of Bilateral Fallopian Tubes, Percutaneous Approach */
"0U574ZZ"  =  "1"  /* Destruction of Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0U577ZZ"  =  "1"  /* Destruction of Bilateral Fallopian Tubes, Via Natural or Artificial Opening */
"0U578ZZ"  =  "1"  /* Destruction of Bilateral Fallopian Tubes, Via Natural or Artificial Opening Endoscopic */
"0U790DZ"  =  "1"  /* Dilation of Uterus with Intraluminal Device, Open Approach */
"0U790ZZ"  =  "1"  /* Dilation of Uterus, Open Approach */
"0U793DZ"  =  "1"  /* Dilation of Uterus with Intraluminal Device, Percutaneous Approach */
"0U793ZZ"  =  "1"  /* Dilation of Uterus, Percutaneous Approach */
"0U794DZ"  =  "1"  /* Dilation of Uterus with Intraluminal Device, Percutaneous Endoscopic Approach */
"0U794ZZ"  =  "1"  /* Dilation of Uterus, Percutaneous Endoscopic Approach */
"0U797DZ"  =  "1"  /* Dilation of Uterus with Intraluminal Device, Via Natural or Artificial Opening */
"0U797ZZ"  =  "1"  /* Dilation of Uterus, Via Natural or Artificial Opening */
"0U798DZ"  =  "1"  /* Dilation of Uterus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0U798ZZ"  =  "1"  /* Dilation of Uterus, Via Natural or Artificial Opening Endoscopic */
"0U800ZZ"  =  "1"  /* Division of Right Ovary, Open Approach */
"0U803ZZ"  =  "1"  /* Division of Right Ovary, Percutaneous Approach */
"0U804ZZ"  =  "1"  /* Division of Right Ovary, Percutaneous Endoscopic Approach */
"0U810ZZ"  =  "1"  /* Division of Left Ovary, Open Approach */
"0U813ZZ"  =  "1"  /* Division of Left Ovary, Percutaneous Approach */
"0U814ZZ"  =  "1"  /* Division of Left Ovary, Percutaneous Endoscopic Approach */
"0U820ZZ"  =  "1"  /* Division of Bilateral Ovaries, Open Approach */
"0U823ZZ"  =  "1"  /* Division of Bilateral Ovaries, Percutaneous Approach */
"0U824ZZ"  =  "1"  /* Division of Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0U9000Z"  =  "1"  /* Drainage of Right Ovary with Drainage Device, Open Approach */
"0U900ZX"  =  "1"  /* Drainage of Right Ovary, Open Approach, Diagnostic */
"0U900ZZ"  =  "1"  /* Drainage of Right Ovary, Open Approach */
"0U903ZZ"  =  "1"  /* Drainage of Right Ovary, Percutaneous Approach */
"0U90XZZ"  =  "1"  /* Drainage of Right Ovary, External Approach */
"0U9100Z"  =  "1"  /* Drainage of Left Ovary with Drainage Device, Open Approach */
"0U910ZX"  =  "1"  /* Drainage of Left Ovary, Open Approach, Diagnostic */
"0U910ZZ"  =  "1"  /* Drainage of Left Ovary, Open Approach */
"0U913ZZ"  =  "1"  /* Drainage of Left Ovary, Percutaneous Approach */
"0U91XZZ"  =  "1"  /* Drainage of Left Ovary, External Approach */
"0U9200Z"  =  "1"  /* Drainage of Bilateral Ovaries with Drainage Device, Open Approach */
"0U920ZX"  =  "1"  /* Drainage of Bilateral Ovaries, Open Approach, Diagnostic */
"0U920ZZ"  =  "1"  /* Drainage of Bilateral Ovaries, Open Approach */
"0U923ZZ"  =  "1"  /* Drainage of Bilateral Ovaries, Percutaneous Approach */
"0U92XZZ"  =  "1"  /* Drainage of Bilateral Ovaries, External Approach */
"0U940ZX"  =  "1"  /* Drainage of Uterine Supporting Structure, Open Approach, Diagnostic */
"0U9500Z"  =  "1"  /* Drainage of Right Fallopian Tube with Drainage Device, Open Approach */
"0U950ZZ"  =  "1"  /* Drainage of Right Fallopian Tube, Open Approach */
"0U9530Z"  =  "1"  /* Drainage of Right Fallopian Tube with Drainage Device, Percutaneous Approach */
"0U9540Z"  =  "1"  /* Drainage of Right Fallopian Tube with Drainage Device, Percutaneous Endoscopic Approach */
"0U9570Z"  =  "1"  /* Drainage of Right Fallopian Tube with Drainage Device, Via Natural or Artificial Opening */
"0U9580Z"  =  "1"  /* Drainage of Right Fallopian Tube with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0U9600Z"  =  "1"  /* Drainage of Left Fallopian Tube with Drainage Device, Open Approach */
"0U960ZZ"  =  "1"  /* Drainage of Left Fallopian Tube, Open Approach */
"0U9630Z"  =  "1"  /* Drainage of Left Fallopian Tube with Drainage Device, Percutaneous Approach */
"0U9640Z"  =  "1"  /* Drainage of Left Fallopian Tube with Drainage Device, Percutaneous Endoscopic Approach */
"0U9670Z"  =  "1"  /* Drainage of Left Fallopian Tube with Drainage Device, Via Natural or Artificial Opening */
"0U9680Z"  =  "1"  /* Drainage of Left Fallopian Tube with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0U9700Z"  =  "1"  /* Drainage of Bilateral Fallopian Tubes with Drainage Device, Open Approach */
"0U970ZZ"  =  "1"  /* Drainage of Bilateral Fallopian Tubes, Open Approach */
"0U9730Z"  =  "1"  /* Drainage of Bilateral Fallopian Tubes with Drainage Device, Percutaneous Approach */
"0U9740Z"  =  "1"  /* Drainage of Bilateral Fallopian Tubes with Drainage Device, Percutaneous Endoscopic Approach */
"0U9770Z"  =  "1"  /* Drainage of Bilateral Fallopian Tubes with Drainage Device, Via Natural or Artificial Opening */
"0U9780Z"  =  "1"  /* Drainage of Bilateral Fallopian Tubes with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0U9900Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Open Approach */
"0U990ZX"  =  "1"  /* Drainage of Uterus, Open Approach, Diagnostic */
"0U990ZZ"  =  "1"  /* Drainage of Uterus, Open Approach */
"0U9930Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Percutaneous Approach */
"0U993ZZ"  =  "1"  /* Drainage of Uterus, Percutaneous Approach */
"0U9940Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Percutaneous Endoscopic Approach */
"0U994ZZ"  =  "1"  /* Drainage of Uterus, Percutaneous Endoscopic Approach */
"0U9970Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Via Natural or Artificial Opening */
"0U997ZZ"  =  "1"  /* Drainage of Uterus, Via Natural or Artificial Opening */
"0U9980Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Via Natural or Artificial Opening Endoscopic */
"0U998ZZ"  =  "1"  /* Drainage of Uterus, Via Natural or Artificial Opening Endoscopic */
"0UB00ZX"  =  "1"  /* Excision of Right Ovary, Open Approach, Diagnostic */
"0UB00ZZ"  =  "1"  /* Excision of Right Ovary, Open Approach */
"0UB03ZZ"  =  "1"  /* Excision of Right Ovary, Percutaneous Approach */
"0UB07ZX"  =  "1"  /* Excision of Right Ovary, Via Natural or Artificial Opening, Diagnostic */
"0UB07ZZ"  =  "1"  /* Excision of Right Ovary, Via Natural or Artificial Opening */
"0UB08ZX"  =  "1"  /* Excision of Right Ovary, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0UB08ZZ"  =  "1"  /* Excision of Right Ovary, Via Natural or Artificial Opening Endoscopic */
"0UB10ZX"  =  "1"  /* Excision of Left Ovary, Open Approach, Diagnostic */
"0UB10ZZ"  =  "1"  /* Excision of Left Ovary, Open Approach */
"0UB13ZZ"  =  "1"  /* Excision of Left Ovary, Percutaneous Approach */
"0UB17ZX"  =  "1"  /* Excision of Left Ovary, Via Natural or Artificial Opening, Diagnostic */
"0UB17ZZ"  =  "1"  /* Excision of Left Ovary, Via Natural or Artificial Opening */
"0UB18ZX"  =  "1"  /* Excision of Left Ovary, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0UB18ZZ"  =  "1"  /* Excision of Left Ovary, Via Natural or Artificial Opening Endoscopic */
"0UB20ZX"  =  "1"  /* Excision of Bilateral Ovaries, Open Approach, Diagnostic */
"0UB20ZZ"  =  "1"  /* Excision of Bilateral Ovaries, Open Approach */
"0UB23ZZ"  =  "1"  /* Excision of Bilateral Ovaries, Percutaneous Approach */
"0UB27ZX"  =  "1"  /* Excision of Bilateral Ovaries, Via Natural or Artificial Opening, Diagnostic */
"0UB27ZZ"  =  "1"  /* Excision of Bilateral Ovaries, Via Natural or Artificial Opening */
"0UB28ZX"  =  "1"  /* Excision of Bilateral Ovaries, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0UB28ZZ"  =  "1"  /* Excision of Bilateral Ovaries, Via Natural or Artificial Opening Endoscopic */
"0UB40ZX"  =  "1"  /* Excision of Uterine Supporting Structure, Open Approach, Diagnostic */
"0UB50ZZ"  =  "1"  /* Excision of Right Fallopian Tube, Open Approach */
"0UB53ZZ"  =  "1"  /* Excision of Right Fallopian Tube, Percutaneous Approach */
"0UB54ZZ"  =  "1"  /* Excision of Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0UB57ZZ"  =  "1"  /* Excision of Right Fallopian Tube, Via Natural or Artificial Opening */
"0UB58ZZ"  =  "1"  /* Excision of Right Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UB60ZZ"  =  "1"  /* Excision of Left Fallopian Tube, Open Approach */
"0UB63ZZ"  =  "1"  /* Excision of Left Fallopian Tube, Percutaneous Approach */
"0UB64ZZ"  =  "1"  /* Excision of Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UB67ZZ"  =  "1"  /* Excision of Left Fallopian Tube, Via Natural or Artificial Opening */
"0UB68ZZ"  =  "1"  /* Excision of Left Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UB70ZZ"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Open Approach */
"0UB73ZZ"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Percutaneous Approach */
"0UB74ZZ"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0UB77ZZ"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Via Natural or Artificial Opening */
"0UB78ZZ"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Via Natural or Artificial Opening Endoscopic */
"0UB90ZX"  =  "1"  /* Excision of Uterus, Open Approach, Diagnostic */
"0UC00ZZ"  =  "1"  /* Extirpation of Matter from Right Ovary, Open Approach */
"0UC03ZZ"  =  "1"  /* Extirpation of Matter from Right Ovary, Percutaneous Approach */
"0UC10ZZ"  =  "1"  /* Extirpation of Matter from Left Ovary, Open Approach */
"0UC13ZZ"  =  "1"  /* Extirpation of Matter from Left Ovary, Percutaneous Approach */
"0UC20ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Ovaries, Open Approach */
"0UC23ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Ovaries, Percutaneous Approach */
"0UC50ZZ"  =  "1"  /* Extirpation of Matter from Right Fallopian Tube, Open Approach */
"0UC53ZZ"  =  "1"  /* Extirpation of Matter from Right Fallopian Tube, Percutaneous Approach */
"0UC54ZZ"  =  "1"  /* Extirpation of Matter from Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0UC57ZZ"  =  "1"  /* Extirpation of Matter from Right Fallopian Tube, Via Natural or Artificial Opening */
"0UC58ZZ"  =  "1"  /* Extirpation of Matter from Right Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UC60ZZ"  =  "1"  /* Extirpation of Matter from Left Fallopian Tube, Open Approach */
"0UC63ZZ"  =  "1"  /* Extirpation of Matter from Left Fallopian Tube, Percutaneous Approach */
"0UC64ZZ"  =  "1"  /* Extirpation of Matter from Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UC67ZZ"  =  "1"  /* Extirpation of Matter from Left Fallopian Tube, Via Natural or Artificial Opening */
"0UC68ZZ"  =  "1"  /* Extirpation of Matter from Left Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UC70ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Fallopian Tubes, Open Approach */
"0UC73ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Fallopian Tubes, Percutaneous Approach */
"0UC74ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0UC77ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Fallopian Tubes, Via Natural or Artificial Opening */
"0UC78ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Fallopian Tubes, Via Natural or Artificial Opening Endoscopic */
"0UC90ZZ"  =  "1"  /* Extirpation of Matter from Uterus, Open Approach */
"0UC93ZZ"  =  "1"  /* Extirpation of Matter from Uterus, Percutaneous Approach */
"0UC94ZZ"  =  "1"  /* Extirpation of Matter from Uterus, Percutaneous Endoscopic Approach */
"0UF50ZZ"  =  "1"  /* Fragmentation in Right Fallopian Tube, Open Approach */
"0UF53ZZ"  =  "1"  /* Fragmentation in Right Fallopian Tube, Percutaneous Approach */
"0UF54ZZ"  =  "1"  /* Fragmentation in Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0UF57ZZ"  =  "1"  /* Fragmentation in Right Fallopian Tube, Via Natural or Artificial Opening */
"0UF58ZZ"  =  "1"  /* Fragmentation in Right Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UF60ZZ"  =  "1"  /* Fragmentation in Left Fallopian Tube, Open Approach */
"0UF63ZZ"  =  "1"  /* Fragmentation in Left Fallopian Tube, Percutaneous Approach */
"0UF64ZZ"  =  "1"  /* Fragmentation in Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UF67ZZ"  =  "1"  /* Fragmentation in Left Fallopian Tube, Via Natural or Artificial Opening */
"0UF68ZZ"  =  "1"  /* Fragmentation in Left Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UF70ZZ"  =  "1"  /* Fragmentation in Bilateral Fallopian Tubes, Open Approach */
"0UF73ZZ"  =  "1"  /* Fragmentation in Bilateral Fallopian Tubes, Percutaneous Approach */
"0UF74ZZ"  =  "1"  /* Fragmentation in Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0UF77ZZ"  =  "1"  /* Fragmentation in Bilateral Fallopian Tubes, Via Natural or Artificial Opening */
"0UF78ZZ"  =  "1"  /* Fragmentation in Bilateral Fallopian Tubes, Via Natural or Artificial Opening Endoscopic */
"0UJD0ZZ"  =  "1"  /* Inspection of Uterus and Cervix, Open Approach */
"0UJD3ZZ"  =  "1"  /* Inspection of Uterus and Cervix, Percutaneous Approach */
"0UJD4ZZ"  =  "1"  /* Inspection of Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UL50CZ"  =  "1"  /* Occlusion of Right Fallopian Tube with Extraluminal Device, Open Approach */
"0UL50DZ"  =  "1"  /* Occlusion of Right Fallopian Tube with Intraluminal Device, Open Approach */
"0UL50ZZ"  =  "1"  /* Occlusion of Right Fallopian Tube, Open Approach */
"0UL53CZ"  =  "1"  /* Occlusion of Right Fallopian Tube with Extraluminal Device, Percutaneous Approach */
"0UL53DZ"  =  "1"  /* Occlusion of Right Fallopian Tube with Intraluminal Device, Percutaneous Approach */
"0UL53ZZ"  =  "1"  /* Occlusion of Right Fallopian Tube, Percutaneous Approach */
"0UL54CZ"  =  "1"  /* Occlusion of Right Fallopian Tube with Extraluminal Device, Percutaneous Endoscopic Approach */
"0UL54DZ"  =  "1"  /* Occlusion of Right Fallopian Tube with Intraluminal Device, Percutaneous Endoscopic Approach */
"0UL54ZZ"  =  "1"  /* Occlusion of Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0UL57DZ"  =  "1"  /* Occlusion of Right Fallopian Tube with Intraluminal Device, Via Natural or Artificial Opening */
"0UL57ZZ"  =  "1"  /* Occlusion of Right Fallopian Tube, Via Natural or Artificial Opening */
"0UL58DZ"  =  "1"  /* Occlusion of Right Fallopian Tube with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0UL58ZZ"  =  "1"  /* Occlusion of Right Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UL60CZ"  =  "1"  /* Occlusion of Left Fallopian Tube with Extraluminal Device, Open Approach */
"0UL60DZ"  =  "1"  /* Occlusion of Left Fallopian Tube with Intraluminal Device, Open Approach */
"0UL60ZZ"  =  "1"  /* Occlusion of Left Fallopian Tube, Open Approach */
"0UL63CZ"  =  "1"  /* Occlusion of Left Fallopian Tube with Extraluminal Device, Percutaneous Approach */
"0UL63DZ"  =  "1"  /* Occlusion of Left Fallopian Tube with Intraluminal Device, Percutaneous Approach */
"0UL63ZZ"  =  "1"  /* Occlusion of Left Fallopian Tube, Percutaneous Approach */
"0UL64CZ"  =  "1"  /* Occlusion of Left Fallopian Tube with Extraluminal Device, Percutaneous Endoscopic Approach */
"0UL64DZ"  =  "1"  /* Occlusion of Left Fallopian Tube with Intraluminal Device, Percutaneous Endoscopic Approach */
"0UL64ZZ"  =  "1"  /* Occlusion of Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UL67DZ"  =  "1"  /* Occlusion of Left Fallopian Tube with Intraluminal Device, Via Natural or Artificial Opening */
"0UL67ZZ"  =  "1"  /* Occlusion of Left Fallopian Tube, Via Natural or Artificial Opening */
"0UL68DZ"  =  "1"  /* Occlusion of Left Fallopian Tube with Intraluminal Device, Via Natural or Artificial Opening Endoscopic */
"0UL68ZZ"  =  "1"  /* Occlusion of Left Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UL70CZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes with Extraluminal Device, Open Approach */
"0UL70DZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes with Intraluminal Device, Open Approach */
"0UL70ZZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes, Open Approach */
"0UL73CZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes with Extraluminal Device, Percutaneous Approach */
"0UL73DZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes with Intraluminal Device, Percutaneous Approach */
"0UL73ZZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes, Percutaneous Approach */
"0UL74ZZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0UL77DZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes with Intraluminal Device, Via Natural or Artificial Opening */
"0UL77ZZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes, Via Natural or Artificial Opening */
"0UM00ZZ"  =  "1"  /* Reattachment of Right Ovary, Open Approach */
"0UM10ZZ"  =  "1"  /* Reattachment of Left Ovary, Open Approach */
"0UM20ZZ"  =  "1"  /* Reattachment of Bilateral Ovaries, Open Approach */
"0UM50ZZ"  =  "1"  /* Reattachment of Right Fallopian Tube, Open Approach */
"0UM54ZZ"  =  "1"  /* Reattachment of Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0UM60ZZ"  =  "1"  /* Reattachment of Left Fallopian Tube, Open Approach */
"0UM64ZZ"  =  "1"  /* Reattachment of Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UM70ZZ"  =  "1"  /* Reattachment of Bilateral Fallopian Tubes, Open Approach */
"0UM74ZZ"  =  "1"  /* Reattachment of Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0UN00ZZ"  =  "1"  /* Release Right Ovary, Open Approach */
"0UN03ZZ"  =  "1"  /* Release Right Ovary, Percutaneous Approach */
"0UN04ZZ"  =  "1"  /* Release Right Ovary, Percutaneous Endoscopic Approach */
"0UN10ZZ"  =  "1"  /* Release Left Ovary, Open Approach */
"0UN13ZZ"  =  "1"  /* Release Left Ovary, Percutaneous Approach */
"0UN14ZZ"  =  "1"  /* Release Left Ovary, Percutaneous Endoscopic Approach */
"0UN20ZZ"  =  "1"  /* Release Bilateral Ovaries, Open Approach */
"0UN23ZZ"  =  "1"  /* Release Bilateral Ovaries, Percutaneous Approach */
"0UN24ZZ"  =  "1"  /* Release Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0UN50ZZ"  =  "1"  /* Release Right Fallopian Tube, Open Approach */
"0UN53ZZ"  =  "1"  /* Release Right Fallopian Tube, Percutaneous Approach */
"0UN57ZZ"  =  "1"  /* Release Right Fallopian Tube, Via Natural or Artificial Opening */
"0UN58ZZ"  =  "1"  /* Release Right Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UN60ZZ"  =  "1"  /* Release Left Fallopian Tube, Open Approach */
"0UN63ZZ"  =  "1"  /* Release Left Fallopian Tube, Percutaneous Approach */
"0UN67ZZ"  =  "1"  /* Release Left Fallopian Tube, Via Natural or Artificial Opening */
"0UN68ZZ"  =  "1"  /* Release Left Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UN70ZZ"  =  "1"  /* Release Bilateral Fallopian Tubes, Open Approach */
"0UN73ZZ"  =  "1"  /* Release Bilateral Fallopian Tubes, Percutaneous Approach */
"0UN77ZZ"  =  "1"  /* Release Bilateral Fallopian Tubes, Via Natural or Artificial Opening */
"0UN78ZZ"  =  "1"  /* Release Bilateral Fallopian Tubes, Via Natural or Artificial Opening Endoscopic */
"0UP300Z"  =  "1"  /* Removal of Drainage Device from Ovary, Open Approach */
"0UP303Z"  =  "1"  /* Removal of Infusion Device from Ovary, Open Approach */
"0UP330Z"  =  "1"  /* Removal of Drainage Device from Ovary, Percutaneous Approach */
"0UP333Z"  =  "1"  /* Removal of Infusion Device from Ovary, Percutaneous Approach */
"0UP800Z"  =  "1"  /* Removal of Drainage Device from Fallopian Tube, Open Approach */
"0UP803Z"  =  "1"  /* Removal of Infusion Device from Fallopian Tube, Open Approach */
"0UP807Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Fallopian Tube, Open Approach */
"0UP80CZ"  =  "1"  /* Removal of Extraluminal Device from Fallopian Tube, Open Approach */
"0UP80DZ"  =  "1"  /* Removal of Intraluminal Device from Fallopian Tube, Open Approach */
"0UP80KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Fallopian Tube, Open Approach */
"0UP830Z"  =  "1"  /* Removal of Drainage Device from Fallopian Tube, Percutaneous Approach */
"0UP833Z"  =  "1"  /* Removal of Infusion Device from Fallopian Tube, Percutaneous Approach */
"0UP837Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Fallopian Tube, Percutaneous Approach */
"0UP83CZ"  =  "1"  /* Removal of Extraluminal Device from Fallopian Tube, Percutaneous Approach */
"0UP83DZ"  =  "1"  /* Removal of Intraluminal Device from Fallopian Tube, Percutaneous Approach */
"0UP83KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Fallopian Tube, Percutaneous Approach */
"0UP840Z"  =  "1"  /* Removal of Drainage Device from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP843Z"  =  "1"  /* Removal of Infusion Device from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP847Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP84CZ"  =  "1"  /* Removal of Extraluminal Device from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP84DZ"  =  "1"  /* Removal of Intraluminal Device from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP84KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Fallopian Tube, Percutaneous Endoscopic Approach */
"0UP870Z"  =  "1"  /* Removal of Drainage Device from Fallopian Tube, Via Natural or Artificial Opening */
"0UP873Z"  =  "1"  /* Removal of Infusion Device from Fallopian Tube, Via Natural or Artificial Opening */
"0UP877Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Fallopian Tube, Via Natural or Artificial Opening */
"0UP87CZ"  =  "1"  /* Removal of Extraluminal Device from Fallopian Tube, Via Natural or Artificial Opening */
"0UP87DZ"  =  "1"  /* Removal of Intraluminal Device from Fallopian Tube, Via Natural or Artificial Opening */
"0UP87KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Fallopian Tube, Via Natural or Artificial Opening */
"0UP880Z"  =  "1"  /* Removal of Drainage Device from Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UP883Z"  =  "1"  /* Removal of Infusion Device from Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UP887Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UP88CZ"  =  "1"  /* Removal of Extraluminal Device from Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UP88DZ"  =  "1"  /* Removal of Intraluminal Device from Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UP88KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UPD00Z"  =  "1"  /* Removal of Drainage Device from Uterus and Cervix, Open Approach */
"0UPD01Z"  =  "1"  /* Removal of Radioactive Element from Uterus and Cervix, Open Approach */
"0UPD03Z"  =  "1"  /* Removal of Infusion Device from Uterus and Cervix, Open Approach */
"0UPD07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Uterus and Cervix, Open Approach */
"0UPD0DZ"  =  "1"  /* Removal of Intraluminal Device from Uterus and Cervix, Open Approach */
"0UPD0HZ"  =  "1"  /* Removal of Contraceptive Device from Uterus and Cervix, Open Approach */
"0UPD0JZ"  =  "1"  /* Removal of Synthetic Substitute from Uterus and Cervix, Open Approach */
"0UPD0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Uterus and Cervix, Open Approach */
"0UPD30Z"  =  "1"  /* Removal of Drainage Device from Uterus and Cervix, Percutaneous Approach */
"0UPD31Z"  =  "1"  /* Removal of Radioactive Element from Uterus and Cervix, Percutaneous Approach */
"0UPD33Z"  =  "1"  /* Removal of Infusion Device from Uterus and Cervix, Percutaneous Approach */
"0UPD37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Uterus and Cervix, Percutaneous Approach */
"0UPD3DZ"  =  "1"  /* Removal of Intraluminal Device from Uterus and Cervix, Percutaneous Approach */
"0UPD3HZ"  =  "1"  /* Removal of Contraceptive Device from Uterus and Cervix, Percutaneous Approach */
"0UPD3JZ"  =  "1"  /* Removal of Synthetic Substitute from Uterus and Cervix, Percutaneous Approach */
"0UPD3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Uterus and Cervix, Percutaneous Approach */
"0UPD40Z"  =  "1"  /* Removal of Drainage Device from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD41Z"  =  "1"  /* Removal of Radioactive Element from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD43Z"  =  "1"  /* Removal of Infusion Device from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD4DZ"  =  "1"  /* Removal of Intraluminal Device from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD4HZ"  =  "1"  /* Removal of Contraceptive Device from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD4JZ"  =  "1"  /* Removal of Synthetic Substitute from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UPD70Z"  =  "1"  /* Removal of Drainage Device from Uterus and Cervix, Via Natural or Artificial Opening */
"0UPD71Z"  =  "1"  /* Removal of Radioactive Element from Uterus and Cervix, Via Natural or Artificial Opening */
"0UPD73Z"  =  "1"  /* Removal of Infusion Device from Uterus and Cervix, Via Natural or Artificial Opening */
"0UPD77Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Uterus and Cervix, Via Natural or Artificial Opening */
"0UPD7DZ"  =  "1"  /* Removal of Intraluminal Device from Uterus and Cervix, Via Natural or Artificial Opening */
"0UPD7JZ"  =  "1"  /* Removal of Synthetic Substitute from Uterus and Cervix, Via Natural or Artificial Opening */
"0UPD7KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Uterus and Cervix, Via Natural or Artificial Opening */
"0UPD80Z"  =  "1"  /* Removal of Drainage Device from Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UPD81Z"  =  "1"  /* Removal of Radioactive Element from Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UPD83Z"  =  "1"  /* Removal of Infusion Device from Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UPD87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UPD8DZ"  =  "1"  /* Removal of Intraluminal Device from Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UPD8JZ"  =  "1"  /* Removal of Synthetic Substitute from Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UPD8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UQ00ZZ"  =  "1"  /* Repair Right Ovary, Open Approach */
"0UQ03ZZ"  =  "1"  /* Repair Right Ovary, Percutaneous Approach */
"0UQ04ZZ"  =  "1"  /* Repair Right Ovary, Percutaneous Endoscopic Approach */
"0UQ10ZZ"  =  "1"  /* Repair Left Ovary, Open Approach */
"0UQ13ZZ"  =  "1"  /* Repair Left Ovary, Percutaneous Approach */
"0UQ14ZZ"  =  "1"  /* Repair Left Ovary, Percutaneous Endoscopic Approach */
"0UQ20ZZ"  =  "1"  /* Repair Bilateral Ovaries, Open Approach */
"0UQ23ZZ"  =  "1"  /* Repair Bilateral Ovaries, Percutaneous Approach */
"0UQ24ZZ"  =  "1"  /* Repair Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0UQ50ZZ"  =  "1"  /* Repair Right Fallopian Tube, Open Approach */
"0UQ53ZZ"  =  "1"  /* Repair Right Fallopian Tube, Percutaneous Approach */
"0UQ54ZZ"  =  "1"  /* Repair Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0UQ57ZZ"  =  "1"  /* Repair Right Fallopian Tube, Via Natural or Artificial Opening */
"0UQ58ZZ"  =  "1"  /* Repair Right Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UQ60ZZ"  =  "1"  /* Repair Left Fallopian Tube, Open Approach */
"0UQ63ZZ"  =  "1"  /* Repair Left Fallopian Tube, Percutaneous Approach */
"0UQ64ZZ"  =  "1"  /* Repair Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UQ67ZZ"  =  "1"  /* Repair Left Fallopian Tube, Via Natural or Artificial Opening */
"0UQ68ZZ"  =  "1"  /* Repair Left Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UQ70ZZ"  =  "1"  /* Repair Bilateral Fallopian Tubes, Open Approach */
"0UQ73ZZ"  =  "1"  /* Repair Bilateral Fallopian Tubes, Percutaneous Approach */
"0UQ74ZZ"  =  "1"  /* Repair Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0UQ77ZZ"  =  "1"  /* Repair Bilateral Fallopian Tubes, Via Natural or Artificial Opening */
"0UQ78ZZ"  =  "1"  /* Repair Bilateral Fallopian Tubes, Via Natural or Artificial Opening Endoscopic */
"0UQ90ZZ"  =  "1"  /* Repair Uterus, Open Approach */
"0UQ93ZZ"  =  "1"  /* Repair Uterus, Percutaneous Approach */
"0UQ94ZZ"  =  "1"  /* Repair Uterus, Percutaneous Endoscopic Approach */
"0UQ97ZZ"  =  "1"  /* Repair Uterus, Via Natural or Artificial Opening */
"0UQ98ZZ"  =  "1"  /* Repair Uterus, Via Natural or Artificial Opening Endoscopic */
"0UQC0ZZ"  =  "1"  /* Repair Cervix, Open Approach */
"0UQC3ZZ"  =  "1"  /* Repair Cervix, Percutaneous Approach */
"0UQC4ZZ"  =  "1"  /* Repair Cervix, Percutaneous Endoscopic Approach */
"0UQC7ZZ"  =  "1"  /* Repair Cervix, Via Natural or Artificial Opening */
"0UQC8ZZ"  =  "1"  /* Repair Cervix, Via Natural or Artificial Opening Endoscopic */
"0UQG0ZZ"  =  "1"  /* Repair Vagina, Open Approach */
"0UQG3ZZ"  =  "1"  /* Repair Vagina, Percutaneous Approach */
"0UQG4ZZ"  =  "1"  /* Repair Vagina, Percutaneous Endoscopic Approach */
"0UQG7ZZ"  =  "1"  /* Repair Vagina, Via Natural or Artificial Opening */
"0UQG8ZZ"  =  "1"  /* Repair Vagina, Via Natural or Artificial Opening Endoscopic */
"0US00ZZ"  =  "1"  /* Reposition Right Ovary, Open Approach */
"0US10ZZ"  =  "1"  /* Reposition Left Ovary, Open Approach */
"0US20ZZ"  =  "1"  /* Reposition Bilateral Ovaries, Open Approach */
"0US50ZZ"  =  "1"  /* Reposition Right Fallopian Tube, Open Approach */
"0US54ZZ"  =  "1"  /* Reposition Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0US60ZZ"  =  "1"  /* Reposition Left Fallopian Tube, Open Approach */
"0US64ZZ"  =  "1"  /* Reposition Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0US70ZZ"  =  "1"  /* Reposition Bilateral Fallopian Tubes, Open Approach */
"0US74ZZ"  =  "1"  /* Reposition Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0US90ZZ"  =  "1"  /* Reposition Uterus, Open Approach */
"0US94ZZ"  =  "1"  /* Reposition Uterus, Percutaneous Endoscopic Approach */
"0USC0ZZ"  =  "1"  /* Reposition Cervix, Open Approach */
"0USC4ZZ"  =  "1"  /* Reposition Cervix, Percutaneous Endoscopic Approach */
"0UT00ZZ"  =  "1"  /* Resection of Right Ovary, Open Approach */
"0UT07ZZ"  =  "1"  /* Resection of Right Ovary, Via Natural or Artificial Opening */
"0UT08ZZ"  =  "1"  /* Resection of Right Ovary, Via Natural or Artificial Opening Endoscopic */
"0UT0FZZ"  =  "1"  /* Resection of Right Ovary, Via Natural or Artificial Opening With Percutaneous Endoscopic Assistance */
"0UT10ZZ"  =  "1"  /* Resection of Left Ovary, Open Approach */
"0UT17ZZ"  =  "1"  /* Resection of Left Ovary, Via Natural or Artificial Opening */
"0UT18ZZ"  =  "1"  /* Resection of Left Ovary, Via Natural or Artificial Opening Endoscopic */
"0UT1FZZ"  =  "1"  /* Resection of Left Ovary, Via Natural or Artificial Opening With Percutaneous Endoscopic Assistance */
"0UT20ZZ"  =  "1"  /* Resection of Bilateral Ovaries, Open Approach */
"0UT27ZZ"  =  "1"  /* Resection of Bilateral Ovaries, Via Natural or Artificial Opening */
"0UT28ZZ"  =  "1"  /* Resection of Bilateral Ovaries, Via Natural or Artificial Opening Endoscopic */
"0UT2FZZ"  =  "1"  /* Resection of Bilateral Ovaries, Via Natural or Artificial Opening With Percutaneous Endoscopic Assistance */
"0UT40ZZ"  =  "1"  /* Resection of Uterine Supporting Structure, Open Approach */
"0UT44ZZ"  =  "1"  /* Resection of Uterine Supporting Structure, Percutaneous Endoscopic Approach */
"0UT50ZZ"  =  "1"  /* Resection of Right Fallopian Tube, Open Approach */
"0UT54ZZ"  =  "1"  /* Resection of Right Fallopian Tube, Percutaneous Endoscopic Approach */
"0UT57ZZ"  =  "1"  /* Resection of Right Fallopian Tube, Via Natural or Artificial Opening */
"0UT58ZZ"  =  "1"  /* Resection of Right Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UT5FZZ"  =  "1"  /* Resection of Right Fallopian Tube, Via Natural or Artificial Opening With Percutaneous Endoscopic Assistance */
"0UT60ZZ"  =  "1"  /* Resection of Left Fallopian Tube, Open Approach */
"0UT64ZZ"  =  "1"  /* Resection of Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UT67ZZ"  =  "1"  /* Resection of Left Fallopian Tube, Via Natural or Artificial Opening */
"0UT68ZZ"  =  "1"  /* Resection of Left Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UT6FZZ"  =  "1"  /* Resection of Left Fallopian Tube, Via Natural or Artificial Opening With Percutaneous Endoscopic Assistance */
"0UT70ZZ"  =  "1"  /* Resection of Bilateral Fallopian Tubes, Open Approach */
"0UT74ZZ"  =  "1"  /* Resection of Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach */
"0UT77ZZ"  =  "1"  /* Resection of Bilateral Fallopian Tubes, Via Natural or Artificial Opening */
"0UT78ZZ"  =  "1"  /* Resection of Bilateral Fallopian Tubes, Via Natural or Artificial Opening Endoscopic */
"0UT7FZZ"  =  "1"  /* Resection of Bilateral Fallopian Tubes, Via Natural or Artificial Opening With Percutaneous Endoscopic Assistance */
"0UT90ZZ"  =  "1"  /* Resection of Uterus, Open Approach */
"0UT94ZZ"  =  "1"  /* Resection of Uterus, Percutaneous Endoscopic Approach */
"0UTC0ZZ"  =  "1"  /* Resection of Cervix, Open Approach */
"0UTC4ZZ"  =  "1"  /* Resection of Cervix, Percutaneous Endoscopic Approach */
"0UTG0ZZ"  =  "1"  /* Resection of Vagina, Open Approach */
"0UU507Z"  =  "1"  /* Supplement Right Fallopian Tube with Autologous Tissue Substitute, Open Approach */
"0UU50KZ"  =  "1"  /* Supplement Right Fallopian Tube with Nonautologous Tissue Substitute, Open Approach */
"0UU547Z"  =  "1"  /* Supplement Right Fallopian Tube with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0UU54KZ"  =  "1"  /* Supplement Right Fallopian Tube with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0UU577Z"  =  "1"  /* Supplement Right Fallopian Tube with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0UU57KZ"  =  "1"  /* Supplement Right Fallopian Tube with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0UU587Z"  =  "1"  /* Supplement Right Fallopian Tube with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0UU58KZ"  =  "1"  /* Supplement Right Fallopian Tube with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0UU607Z"  =  "1"  /* Supplement Left Fallopian Tube with Autologous Tissue Substitute, Open Approach */
"0UU60KZ"  =  "1"  /* Supplement Left Fallopian Tube with Nonautologous Tissue Substitute, Open Approach */
"0UU647Z"  =  "1"  /* Supplement Left Fallopian Tube with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0UU64KZ"  =  "1"  /* Supplement Left Fallopian Tube with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0UU677Z"  =  "1"  /* Supplement Left Fallopian Tube with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0UU67KZ"  =  "1"  /* Supplement Left Fallopian Tube with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0UU687Z"  =  "1"  /* Supplement Left Fallopian Tube with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0UU68KZ"  =  "1"  /* Supplement Left Fallopian Tube with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0UU707Z"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Autologous Tissue Substitute, Open Approach */
"0UU70KZ"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Nonautologous Tissue Substitute, Open Approach */
"0UU747Z"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0UU74KZ"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0UU777Z"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Autologous Tissue Substitute, Via Natural or Artificial Opening */
"0UU77KZ"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Nonautologous Tissue Substitute, Via Natural or Artificial Opening */
"0UU787Z"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0UU78KZ"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0UW300Z"  =  "1"  /* Revision of Drainage Device in Ovary, Open Approach */
"0UW303Z"  =  "1"  /* Revision of Infusion Device in Ovary, Open Approach */
"0UW330Z"  =  "1"  /* Revision of Drainage Device in Ovary, Percutaneous Approach */
"0UW333Z"  =  "1"  /* Revision of Infusion Device in Ovary, Percutaneous Approach */
"0UW800Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Open Approach */
"0UW803Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Open Approach */
"0UW807Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Fallopian Tube, Open Approach */
"0UW80CZ"  =  "1"  /* Revision of Extraluminal Device in Fallopian Tube, Open Approach */
"0UW80DZ"  =  "1"  /* Revision of Intraluminal Device in Fallopian Tube, Open Approach */
"0UW80JZ"  =  "1"  /* Revision of Synthetic Substitute in Fallopian Tube, Open Approach */
"0UW80KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Fallopian Tube, Open Approach */
"0UW830Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Percutaneous Approach */
"0UW833Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Percutaneous Approach */
"0UW837Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Fallopian Tube, Percutaneous Approach */
"0UW83CZ"  =  "1"  /* Revision of Extraluminal Device in Fallopian Tube, Percutaneous Approach */
"0UW83DZ"  =  "1"  /* Revision of Intraluminal Device in Fallopian Tube, Percutaneous Approach */
"0UW83JZ"  =  "1"  /* Revision of Synthetic Substitute in Fallopian Tube, Percutaneous Approach */
"0UW83KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Fallopian Tube, Percutaneous Approach */
"0UW840Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW843Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW847Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW84CZ"  =  "1"  /* Revision of Extraluminal Device in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW84DZ"  =  "1"  /* Revision of Intraluminal Device in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW84JZ"  =  "1"  /* Revision of Synthetic Substitute in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW84KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Fallopian Tube, Percutaneous Endoscopic Approach */
"0UW870Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Via Natural or Artificial Opening */
"0UW873Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Via Natural or Artificial Opening */
"0UW877Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Fallopian Tube, Via Natural or Artificial Opening */
"0UW87CZ"  =  "1"  /* Revision of Extraluminal Device in Fallopian Tube, Via Natural or Artificial Opening */
"0UW87DZ"  =  "1"  /* Revision of Intraluminal Device in Fallopian Tube, Via Natural or Artificial Opening */
"0UW87JZ"  =  "1"  /* Revision of Synthetic Substitute in Fallopian Tube, Via Natural or Artificial Opening */
"0UW87KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Fallopian Tube, Via Natural or Artificial Opening */
"0UW880Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UW883Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UW887Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UW88CZ"  =  "1"  /* Revision of Extraluminal Device in Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UW88DZ"  =  "1"  /* Revision of Intraluminal Device in Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UW88JZ"  =  "1"  /* Revision of Synthetic Substitute in Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UW88KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Fallopian Tube, Via Natural or Artificial Opening Endoscopic */
"0UWD00Z"  =  "1"  /* Revision of Drainage Device in Uterus and Cervix, Open Approach */
"0UWD01Z"  =  "1"  /* Revision of Radioactive Element in Uterus and Cervix, Open Approach */
"0UWD03Z"  =  "1"  /* Revision of Infusion Device in Uterus and Cervix, Open Approach */
"0UWD07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Uterus and Cervix, Open Approach */
"0UWD0DZ"  =  "1"  /* Revision of Intraluminal Device in Uterus and Cervix, Open Approach */
"0UWD0HZ"  =  "1"  /* Revision of Contraceptive Device in Uterus and Cervix, Open Approach */
"0UWD0JZ"  =  "1"  /* Revision of Synthetic Substitute in Uterus and Cervix, Open Approach */
"0UWD0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Uterus and Cervix, Open Approach */
"0UWD30Z"  =  "1"  /* Revision of Drainage Device in Uterus and Cervix, Percutaneous Approach */
"0UWD31Z"  =  "1"  /* Revision of Radioactive Element in Uterus and Cervix, Percutaneous Approach */
"0UWD33Z"  =  "1"  /* Revision of Infusion Device in Uterus and Cervix, Percutaneous Approach */
"0UWD37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Uterus and Cervix, Percutaneous Approach */
"0UWD3DZ"  =  "1"  /* Revision of Intraluminal Device in Uterus and Cervix, Percutaneous Approach */
"0UWD3HZ"  =  "1"  /* Revision of Contraceptive Device in Uterus and Cervix, Percutaneous Approach */
"0UWD3JZ"  =  "1"  /* Revision of Synthetic Substitute in Uterus and Cervix, Percutaneous Approach */
"0UWD3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Uterus and Cervix, Percutaneous Approach */
"0UWD40Z"  =  "1"  /* Revision of Drainage Device in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD41Z"  =  "1"  /* Revision of Radioactive Element in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD43Z"  =  "1"  /* Revision of Infusion Device in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD4DZ"  =  "1"  /* Revision of Intraluminal Device in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD4HZ"  =  "1"  /* Revision of Contraceptive Device in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD4JZ"  =  "1"  /* Revision of Synthetic Substitute in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Uterus and Cervix, Percutaneous Endoscopic Approach */
"0UWD70Z"  =  "1"  /* Revision of Drainage Device in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD71Z"  =  "1"  /* Revision of Radioactive Element in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD73Z"  =  "1"  /* Revision of Infusion Device in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD77Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD7DZ"  =  "1"  /* Revision of Intraluminal Device in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD7HZ"  =  "1"  /* Revision of Contraceptive Device in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD7JZ"  =  "1"  /* Revision of Synthetic Substitute in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD7KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Uterus and Cervix, Via Natural or Artificial Opening */
"0UWD80Z"  =  "1"  /* Revision of Drainage Device in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UWD81Z"  =  "1"  /* Revision of Radioactive Element in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UWD83Z"  =  "1"  /* Revision of Infusion Device in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UWD87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UWD8DZ"  =  "1"  /* Revision of Intraluminal Device in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UWD8HZ"  =  "1"  /* Revision of Contraceptive Device in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UWD8JZ"  =  "1"  /* Revision of Synthetic Substitute in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UWD8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Uterus and Cervix, Via Natural or Artificial Opening Endoscopic */
"0UY00Z0"  =  "1"  /* Transplantation of Right Ovary, Allogeneic, Open Approach */
"0UY00Z1"  =  "1"  /* Transplantation of Right Ovary, Syngeneic, Open Approach */
"0UY00Z2"  =  "1"  /* Transplantation of Right Ovary, Zooplastic, Open Approach */
"0UY10Z0"  =  "1"  /* Transplantation of Left Ovary, Allogeneic, Open Approach */
"0UY10Z1"  =  "1"  /* Transplantation of Left Ovary, Syngeneic, Open Approach */
"0UY10Z2"  =  "1"  /* Transplantation of Left Ovary, Zooplastic, Open Approach */
"0V510ZZ"  =  "1"  /* Destruction of Right Seminal Vesicle, Open Approach */
"0V513ZZ"  =  "1"  /* Destruction of Right Seminal Vesicle, Percutaneous Approach */
"0V514ZZ"  =  "1"  /* Destruction of Right Seminal Vesicle, Percutaneous Endoscopic Approach */
"0V520ZZ"  =  "1"  /* Destruction of Left Seminal Vesicle, Open Approach */
"0V523ZZ"  =  "1"  /* Destruction of Left Seminal Vesicle, Percutaneous Approach */
"0V524ZZ"  =  "1"  /* Destruction of Left Seminal Vesicle, Percutaneous Endoscopic Approach */
"0V530ZZ"  =  "1"  /* Destruction of Bilateral Seminal Vesicles, Open Approach */
"0V533ZZ"  =  "1"  /* Destruction of Bilateral Seminal Vesicles, Percutaneous Approach */
"0V534ZZ"  =  "1"  /* Destruction of Bilateral Seminal Vesicles, Percutaneous Endoscopic Approach */
"0V900ZX"  =  "1"  /* Drainage of Prostate, Open Approach, Diagnostic */
"0V9100Z"  =  "1"  /* Drainage of Right Seminal Vesicle with Drainage Device, Open Approach */
"0V910ZX"  =  "1"  /* Drainage of Right Seminal Vesicle, Open Approach, Diagnostic */
"0V910ZZ"  =  "1"  /* Drainage of Right Seminal Vesicle, Open Approach */
"0V9200Z"  =  "1"  /* Drainage of Left Seminal Vesicle with Drainage Device, Open Approach */
"0V920ZX"  =  "1"  /* Drainage of Left Seminal Vesicle, Open Approach, Diagnostic */
"0V920ZZ"  =  "1"  /* Drainage of Left Seminal Vesicle, Open Approach */
"0V9300Z"  =  "1"  /* Drainage of Bilateral Seminal Vesicles with Drainage Device, Open Approach */
"0V930ZX"  =  "1"  /* Drainage of Bilateral Seminal Vesicles, Open Approach, Diagnostic */
"0V930ZZ"  =  "1"  /* Drainage of Bilateral Seminal Vesicles, Open Approach */
"0VB00ZX"  =  "1"  /* Excision of Prostate, Open Approach, Diagnostic */
"0VB00ZZ"  =  "1"  /* Excision of Prostate, Open Approach */
"0VB03ZX"  =  "1"  /* Excision of Prostate, Percutaneous Approach, Diagnostic */
"0VB03ZZ"  =  "1"  /* Excision of Prostate, Percutaneous Approach */
"0VB04ZX"  =  "1"  /* Excision of Prostate, Percutaneous Endoscopic Approach, Diagnostic */
"0VB04ZZ"  =  "1"  /* Excision of Prostate, Percutaneous Endoscopic Approach */
"0VB07ZX"  =  "1"  /* Excision of Prostate, Via Natural or Artificial Opening, Diagnostic */
"0VB08ZX"  =  "1"  /* Excision of Prostate, Via Natural or Artificial Opening Endoscopic, Diagnostic */
"0VB10ZX"  =  "1"  /* Excision of Right Seminal Vesicle, Open Approach, Diagnostic */
"0VB10ZZ"  =  "1"  /* Excision of Right Seminal Vesicle, Open Approach */
"0VB13ZZ"  =  "1"  /* Excision of Right Seminal Vesicle, Percutaneous Approach */
"0VB14ZZ"  =  "1"  /* Excision of Right Seminal Vesicle, Percutaneous Endoscopic Approach */
"0VB20ZX"  =  "1"  /* Excision of Left Seminal Vesicle, Open Approach, Diagnostic */
"0VB20ZZ"  =  "1"  /* Excision of Left Seminal Vesicle, Open Approach */
"0VB23ZZ"  =  "1"  /* Excision of Left Seminal Vesicle, Percutaneous Approach */
"0VB24ZZ"  =  "1"  /* Excision of Left Seminal Vesicle, Percutaneous Endoscopic Approach */
"0VB30ZX"  =  "1"  /* Excision of Bilateral Seminal Vesicles, Open Approach, Diagnostic */
"0VB30ZZ"  =  "1"  /* Excision of Bilateral Seminal Vesicles, Open Approach */
"0VB33ZZ"  =  "1"  /* Excision of Bilateral Seminal Vesicles, Percutaneous Approach */
"0VB34ZZ"  =  "1"  /* Excision of Bilateral Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VC10ZZ"  =  "1"  /* Extirpation of Matter from Right Seminal Vesicle, Open Approach */
"0VC13ZZ"  =  "1"  /* Extirpation of Matter from Right Seminal Vesicle, Percutaneous Approach */
"0VC14ZZ"  =  "1"  /* Extirpation of Matter from Right Seminal Vesicle, Percutaneous Endoscopic Approach */
"0VC20ZZ"  =  "1"  /* Extirpation of Matter from Left Seminal Vesicle, Open Approach */
"0VC23ZZ"  =  "1"  /* Extirpation of Matter from Left Seminal Vesicle, Percutaneous Approach */
"0VC24ZZ"  =  "1"  /* Extirpation of Matter from Left Seminal Vesicle, Percutaneous Endoscopic Approach */
"0VC30ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Seminal Vesicles, Open Approach */
"0VC33ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Seminal Vesicles, Percutaneous Approach */
"0VC34ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VJ40ZZ"  =  "1"  /* Inspection of Prostate and Seminal Vesicles, Open Approach */
"0VJ43ZZ"  =  "1"  /* Inspection of Prostate and Seminal Vesicles, Percutaneous Approach */
"0VJ44ZZ"  =  "1"  /* Inspection of Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VN00ZZ"  =  "1"  /* Release Prostate, Open Approach */
"0VN03ZZ"  =  "1"  /* Release Prostate, Percutaneous Approach */
"0VN04ZZ"  =  "1"  /* Release Prostate, Percutaneous Endoscopic Approach */
"0VN07ZZ"  =  "1"  /* Release Prostate, Via Natural or Artificial Opening */
"0VN08ZZ"  =  "1"  /* Release Prostate, Via Natural or Artificial Opening Endoscopic */
"0VN10ZZ"  =  "1"  /* Release Right Seminal Vesicle, Open Approach */
"0VN13ZZ"  =  "1"  /* Release Right Seminal Vesicle, Percutaneous Approach */
"0VN14ZZ"  =  "1"  /* Release Right Seminal Vesicle, Percutaneous Endoscopic Approach */
"0VN20ZZ"  =  "1"  /* Release Left Seminal Vesicle, Open Approach */
"0VN23ZZ"  =  "1"  /* Release Left Seminal Vesicle, Percutaneous Approach */
"0VN24ZZ"  =  "1"  /* Release Left Seminal Vesicle, Percutaneous Endoscopic Approach */
"0VN30ZZ"  =  "1"  /* Release Bilateral Seminal Vesicles, Open Approach */
"0VN33ZZ"  =  "1"  /* Release Bilateral Seminal Vesicles, Percutaneous Approach */
"0VN34ZZ"  =  "1"  /* Release Bilateral Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VP400Z"  =  "1"  /* Removal of Drainage Device from Prostate and Seminal Vesicles, Open Approach */
"0VP401Z"  =  "1"  /* Removal of Radioactive Element from Prostate and Seminal Vesicles, Open Approach */
"0VP403Z"  =  "1"  /* Removal of Infusion Device from Prostate and Seminal Vesicles, Open Approach */
"0VP407Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Prostate and Seminal Vesicles, Open Approach */
"0VP40JZ"  =  "1"  /* Removal of Synthetic Substitute from Prostate and Seminal Vesicles, Open Approach */
"0VP40KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Prostate and Seminal Vesicles, Open Approach */
"0VP430Z"  =  "1"  /* Removal of Drainage Device from Prostate and Seminal Vesicles, Percutaneous Approach */
"0VP431Z"  =  "1"  /* Removal of Radioactive Element from Prostate and Seminal Vesicles, Percutaneous Approach */
"0VP433Z"  =  "1"  /* Removal of Infusion Device from Prostate and Seminal Vesicles, Percutaneous Approach */
"0VP437Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Prostate and Seminal Vesicles, Percutaneous Approach */
"0VP43JZ"  =  "1"  /* Removal of Synthetic Substitute from Prostate and Seminal Vesicles, Percutaneous Approach */
"0VP43KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Prostate and Seminal Vesicles, Percutaneous Approach */
"0VP440Z"  =  "1"  /* Removal of Drainage Device from Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VP441Z"  =  "1"  /* Removal of Radioactive Element from Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VP443Z"  =  "1"  /* Removal of Infusion Device from Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VP447Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VP44JZ"  =  "1"  /* Removal of Synthetic Substitute from Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VP44KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VP477Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VP47JZ"  =  "1"  /* Removal of Synthetic Substitute from Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VP47KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VP487Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0VP48JZ"  =  "1"  /* Removal of Synthetic Substitute from Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0VP48KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0VQ00ZZ"  =  "1"  /* Repair Prostate, Open Approach */
"0VQ03ZZ"  =  "1"  /* Repair Prostate, Percutaneous Approach */
"0VQ04ZZ"  =  "1"  /* Repair Prostate, Percutaneous Endoscopic Approach */
"0VQ07ZZ"  =  "1"  /* Repair Prostate, Via Natural or Artificial Opening */
"0VQ08ZZ"  =  "1"  /* Repair Prostate, Via Natural or Artificial Opening Endoscopic */
"0VQ10ZZ"  =  "1"  /* Repair Right Seminal Vesicle, Open Approach */
"0VQ13ZZ"  =  "1"  /* Repair Right Seminal Vesicle, Percutaneous Approach */
"0VQ14ZZ"  =  "1"  /* Repair Right Seminal Vesicle, Percutaneous Endoscopic Approach */
"0VQ20ZZ"  =  "1"  /* Repair Left Seminal Vesicle, Open Approach */
"0VQ23ZZ"  =  "1"  /* Repair Left Seminal Vesicle, Percutaneous Approach */
"0VQ24ZZ"  =  "1"  /* Repair Left Seminal Vesicle, Percutaneous Endoscopic Approach */
"0VQ30ZZ"  =  "1"  /* Repair Bilateral Seminal Vesicles, Open Approach */
"0VQ33ZZ"  =  "1"  /* Repair Bilateral Seminal Vesicles, Percutaneous Approach */
"0VQ34ZZ"  =  "1"  /* Repair Bilateral Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VT00ZZ"  =  "1"  /* Resection of Prostate, Open Approach */
"0VT04ZZ"  =  "1"  /* Resection of Prostate, Percutaneous Endoscopic Approach */
"0VT07ZZ"  =  "1"  /* Resection of Prostate, Via Natural or Artificial Opening */
"0VT08ZZ"  =  "1"  /* Resection of Prostate, Via Natural or Artificial Opening Endoscopic */
"0VT10ZZ"  =  "1"  /* Resection of Right Seminal Vesicle, Open Approach */
"0VT14ZZ"  =  "1"  /* Resection of Right Seminal Vesicle, Percutaneous Endoscopic Approach */
"0VT20ZZ"  =  "1"  /* Resection of Left Seminal Vesicle, Open Approach */
"0VT24ZZ"  =  "1"  /* Resection of Left Seminal Vesicle, Percutaneous Endoscopic Approach */
"0VT30ZZ"  =  "1"  /* Resection of Bilateral Seminal Vesicles, Open Approach */
"0VT34ZZ"  =  "1"  /* Resection of Bilateral Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VU107Z"  =  "1"  /* Supplement Right Seminal Vesicle with Autologous Tissue Substitute, Open Approach */
"0VU10JZ"  =  "1"  /* Supplement Right Seminal Vesicle with Synthetic Substitute, Open Approach */
"0VU10KZ"  =  "1"  /* Supplement Right Seminal Vesicle with Nonautologous Tissue Substitute, Open Approach */
"0VU147Z"  =  "1"  /* Supplement Right Seminal Vesicle with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0VU14JZ"  =  "1"  /* Supplement Right Seminal Vesicle with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0VU14KZ"  =  "1"  /* Supplement Right Seminal Vesicle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0VU207Z"  =  "1"  /* Supplement Left Seminal Vesicle with Autologous Tissue Substitute, Open Approach */
"0VU20JZ"  =  "1"  /* Supplement Left Seminal Vesicle with Synthetic Substitute, Open Approach */
"0VU20KZ"  =  "1"  /* Supplement Left Seminal Vesicle with Nonautologous Tissue Substitute, Open Approach */
"0VU247Z"  =  "1"  /* Supplement Left Seminal Vesicle with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0VU24JZ"  =  "1"  /* Supplement Left Seminal Vesicle with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0VU24KZ"  =  "1"  /* Supplement Left Seminal Vesicle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0VU307Z"  =  "1"  /* Supplement Bilateral Seminal Vesicles with Autologous Tissue Substitute, Open Approach */
"0VU30JZ"  =  "1"  /* Supplement Bilateral Seminal Vesicles with Synthetic Substitute, Open Approach */
"0VU30KZ"  =  "1"  /* Supplement Bilateral Seminal Vesicles with Nonautologous Tissue Substitute, Open Approach */
"0VU347Z"  =  "1"  /* Supplement Bilateral Seminal Vesicles with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0VU34JZ"  =  "1"  /* Supplement Bilateral Seminal Vesicles with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0VU34KZ"  =  "1"  /* Supplement Bilateral Seminal Vesicles with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0VW400Z"  =  "1"  /* Revision of Drainage Device in Prostate and Seminal Vesicles, Open Approach */
"0VW403Z"  =  "1"  /* Revision of Infusion Device in Prostate and Seminal Vesicles, Open Approach */
"0VW407Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Prostate and Seminal Vesicles, Open Approach */
"0VW40JZ"  =  "1"  /* Revision of Synthetic Substitute in Prostate and Seminal Vesicles, Open Approach */
"0VW40KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Prostate and Seminal Vesicles, Open Approach */
"0VW430Z"  =  "1"  /* Revision of Drainage Device in Prostate and Seminal Vesicles, Percutaneous Approach */
"0VW433Z"  =  "1"  /* Revision of Infusion Device in Prostate and Seminal Vesicles, Percutaneous Approach */
"0VW437Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Prostate and Seminal Vesicles, Percutaneous Approach */
"0VW43JZ"  =  "1"  /* Revision of Synthetic Substitute in Prostate and Seminal Vesicles, Percutaneous Approach */
"0VW43KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Prostate and Seminal Vesicles, Percutaneous Approach */
"0VW440Z"  =  "1"  /* Revision of Drainage Device in Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VW443Z"  =  "1"  /* Revision of Infusion Device in Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VW447Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VW44JZ"  =  "1"  /* Revision of Synthetic Substitute in Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VW44KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Prostate and Seminal Vesicles, Percutaneous Endoscopic Approach */
"0VW477Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VW47JZ"  =  "1"  /* Revision of Synthetic Substitute in Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VW47KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Prostate and Seminal Vesicles, Via Natural or Artificial Opening */
"0VW487Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0VW48JZ"  =  "1"  /* Revision of Synthetic Substitute in Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0VW48KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Prostate and Seminal Vesicles, Via Natural or Artificial Opening Endoscopic */
"0W1G0J4"  =  "1"  /* Bypass Peritoneal Cavity to Cutaneous with Synthetic Substitute, Open Approach */
"0W1G0JY"  =  "1"  /* Bypass Peritoneal Cavity to Lower Vein with Synthetic Substitute, Open Approach */
"0W1G3J4"  =  "1"  /* Bypass Peritoneal Cavity to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0W1G4J4"  =  "1"  /* Bypass Peritoneal Cavity to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W1G4JY"  =  "1"  /* Bypass Peritoneal Cavity to Lower Vein with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0W9F00Z"  =  "1"  /* Drainage of Abdominal Wall with Drainage Device, Open Approach */
"0W9F0ZX"  =  "1"  /* Drainage of Abdominal Wall, Open Approach, Diagnostic */
"0W9F0ZZ"  =  "1"  /* Drainage of Abdominal Wall, Open Approach */
"0W9F3ZX"  =  "1"  /* Drainage of Abdominal Wall, Percutaneous Approach, Diagnostic */
"0W9F4ZX"  =  "1"  /* Drainage of Abdominal Wall, Percutaneous Endoscopic Approach, Diagnostic */
"0W9G00Z"  =  "1"  /* Drainage of Peritoneal Cavity with Drainage Device, Open Approach */
"0W9G0ZZ"  =  "1"  /* Drainage of Peritoneal Cavity, Open Approach */
"0W9H00Z"  =  "1"  /* Drainage of Retroperitoneum with Drainage Device, Open Approach */
"0W9H0ZZ"  =  "1"  /* Drainage of Retroperitoneum, Open Approach */
"0W9H30Z"  =  "1"  /* Drainage of Retroperitoneum with Drainage Device, Percutaneous Approach */
"0W9H3ZZ"  =  "1"  /* Drainage of Retroperitoneum, Percutaneous Approach */
"0W9H40Z"  =  "1"  /* Drainage of Retroperitoneum with Drainage Device, Percutaneous Endoscopic Approach */
"0W9H4ZZ"  =  "1"  /* Drainage of Retroperitoneum, Percutaneous Endoscopic Approach */
"0W9J00Z"  =  "1"  /* Drainage of Pelvic Cavity with Drainage Device, Open Approach */
"0W9J0ZZ"  =  "1"  /* Drainage of Pelvic Cavity, Open Approach */
"0W9J40Z"  =  "1"  /* Drainage of Pelvic Cavity with Drainage Device, Percutaneous Endoscopic Approach */
"0W9J4ZZ"  =  "1"  /* Drainage of Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WBF0ZX"  =  "1"  /* Excision of Abdominal Wall, Open Approach, Diagnostic */
"0WBF0ZZ"  =  "1"  /* Excision of Abdominal Wall, Open Approach */
"0WBF3ZX"  =  "1"  /* Excision of Abdominal Wall, Percutaneous Approach, Diagnostic */
"0WBF3ZZ"  =  "1"  /* Excision of Abdominal Wall, Percutaneous Approach */
"0WBF4ZX"  =  "1"  /* Excision of Abdominal Wall, Percutaneous Endoscopic Approach, Diagnostic */
"0WBF4ZZ"  =  "1"  /* Excision of Abdominal Wall, Percutaneous Endoscopic Approach */
"0WBFXZ2"  =  "1"  /* Excision of Abdominal Wall, Stoma, External Approach */
"0WBFXZX"  =  "1"  /* Excision of Abdominal Wall, External Approach, Diagnostic */
"0WBFXZZ"  =  "1"  /* Excision of Abdominal Wall, External Approach */
"0WBH0ZX"  =  "1"  /* Excision of Retroperitoneum, Open Approach, Diagnostic */
"0WBH0ZZ"  =  "1"  /* Excision of Retroperitoneum, Open Approach */
"0WBH3ZZ"  =  "1"  /* Excision of Retroperitoneum, Percutaneous Approach */
"0WBH4ZZ"  =  "1"  /* Excision of Retroperitoneum, Percutaneous Endoscopic Approach */
"0WCG0ZZ"  =  "1"  /* Extirpation of Matter from Peritoneal Cavity, Open Approach */
"0WCG3ZZ"  =  "1"  /* Extirpation of Matter from Peritoneal Cavity, Percutaneous Approach */
"0WCG4ZZ"  =  "1"  /* Extirpation of Matter from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WCJ0ZZ"  =  "1"  /* Extirpation of Matter from Pelvic Cavity, Open Approach */
"0WCJ3ZZ"  =  "1"  /* Extirpation of Matter from Pelvic Cavity, Percutaneous Approach */
"0WCJ4ZZ"  =  "1"  /* Extirpation of Matter from Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WCP0ZZ"  =  "1"  /* Extirpation of Matter from Gastrointestinal Tract, Open Approach */
"0WCP3ZZ"  =  "1"  /* Extirpation of Matter from Gastrointestinal Tract, Percutaneous Approach */
"0WCP4ZZ"  =  "1"  /* Extirpation of Matter from Gastrointestinal Tract, Percutaneous Endoscopic Approach */
"0WCR0ZZ"  =  "1"  /* Extirpation of Matter from Genitourinary Tract, Open Approach */
"0WCR3ZZ"  =  "1"  /* Extirpation of Matter from Genitourinary Tract, Percutaneous Approach */
"0WCR4ZZ"  =  "1"  /* Extirpation of Matter from Genitourinary Tract, Percutaneous Endoscopic Approach */
"0WFG0ZZ"  =  "1"  /* Fragmentation in Peritoneal Cavity, Open Approach */
"0WFG3ZZ"  =  "1"  /* Fragmentation in Peritoneal Cavity, Percutaneous Approach */
"0WFG4ZZ"  =  "1"  /* Fragmentation in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WHF03Z"  =  "1"  /* Insertion of Infusion Device into Abdominal Wall, Open Approach */
"0WHF0YZ"  =  "1"  /* Insertion of Other Device into Abdominal Wall, Open Approach */
"0WHF33Z"  =  "1"  /* Insertion of Infusion Device into Abdominal Wall, Percutaneous Approach */
"0WHF3YZ"  =  "1"  /* Insertion of Other Device into Abdominal Wall, Percutaneous Approach */
"0WHF43Z"  =  "1"  /* Insertion of Infusion Device into Abdominal Wall, Percutaneous Endoscopic Approach */
"0WHF4YZ"  =  "1"  /* Insertion of Other Device into Abdominal Wall, Percutaneous Endoscopic Approach */
"0WHG03Z"  =  "1"  /* Insertion of Infusion Device into Peritoneal Cavity, Open Approach */
"0WHG0YZ"  =  "1"  /* Insertion of Other Device into Peritoneal Cavity, Open Approach */
"0WHG33Z"  =  "1"  /* Insertion of Infusion Device into Peritoneal Cavity, Percutaneous Approach */
"0WHG3YZ"  =  "1"  /* Insertion of Other Device into Peritoneal Cavity, Percutaneous Approach */
"0WHG43Z"  =  "1"  /* Insertion of Infusion Device into Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WHG4YZ"  =  "1"  /* Insertion of Other Device into Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WHH03Z"  =  "1"  /* Insertion of Infusion Device into Retroperitoneum, Open Approach */
"0WHH0YZ"  =  "1"  /* Insertion of Other Device into Retroperitoneum, Open Approach */
"0WHH33Z"  =  "1"  /* Insertion of Infusion Device into Retroperitoneum, Percutaneous Approach */
"0WHH3YZ"  =  "1"  /* Insertion of Other Device into Retroperitoneum, Percutaneous Approach */
"0WHH43Z"  =  "1"  /* Insertion of Infusion Device into Retroperitoneum, Percutaneous Endoscopic Approach */
"0WHH4YZ"  =  "1"  /* Insertion of Other Device into Retroperitoneum, Percutaneous Endoscopic Approach */
"0WHJ03Z"  =  "1"  /* Insertion of Infusion Device into Pelvic Cavity, Open Approach */
"0WHJ0YZ"  =  "1"  /* Insertion of Other Device into Pelvic Cavity, Open Approach */
"0WHJ33Z"  =  "1"  /* Insertion of Infusion Device into Pelvic Cavity, Percutaneous Approach */
"0WHJ3YZ"  =  "1"  /* Insertion of Other Device into Pelvic Cavity, Percutaneous Approach */
"0WHJ43Z"  =  "1"  /* Insertion of Infusion Device into Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WHJ4YZ"  =  "1"  /* Insertion of Other Device into Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WHP03Z"  =  "1"  /* Insertion of Infusion Device into Gastrointestinal Tract, Open Approach */
"0WJF0ZZ"  =  "1"  /* Inspection of Abdominal Wall, Open Approach */
"0WJG0ZZ"  =  "1"  /* Inspection of Peritoneal Cavity, Open Approach */
"0WJH0ZZ"  =  "1"  /* Inspection of Retroperitoneum, Open Approach */
"0WJJ0ZZ"  =  "1"  /* Inspection of Pelvic Cavity, Open Approach */
"0WJP0ZZ"  =  "1"  /* Inspection of Gastrointestinal Tract, Open Approach */
"0WJR0ZZ"  =  "1"  /* Inspection of Genitourinary Tract, Open Approach */
"0WMF0ZZ"  =  "1"  /* Reattachment of Abdominal Wall, Open Approach */
"0WPF00Z"  =  "1"  /* Removal of Drainage Device from Abdominal Wall, Open Approach */
"0WPF01Z"  =  "1"  /* Removal of Radioactive Element from Abdominal Wall, Open Approach */
"0WPF03Z"  =  "1"  /* Removal of Infusion Device from Abdominal Wall, Open Approach */
"0WPF07Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Abdominal Wall, Open Approach */
"0WPF0JZ"  =  "1"  /* Removal of Synthetic Substitute from Abdominal Wall, Open Approach */
"0WPF0KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Abdominal Wall, Open Approach */
"0WPF0YZ"  =  "1"  /* Removal of Other Device from Abdominal Wall, Open Approach */
"0WPF30Z"  =  "1"  /* Removal of Drainage Device from Abdominal Wall, Percutaneous Approach */
"0WPF31Z"  =  "1"  /* Removal of Radioactive Element from Abdominal Wall, Percutaneous Approach */
"0WPF33Z"  =  "1"  /* Removal of Infusion Device from Abdominal Wall, Percutaneous Approach */
"0WPF37Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Abdominal Wall, Percutaneous Approach */
"0WPF3JZ"  =  "1"  /* Removal of Synthetic Substitute from Abdominal Wall, Percutaneous Approach */
"0WPF3KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Abdominal Wall, Percutaneous Approach */
"0WPF3YZ"  =  "1"  /* Removal of Other Device from Abdominal Wall, Percutaneous Approach */
"0WPF40Z"  =  "1"  /* Removal of Drainage Device from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF41Z"  =  "1"  /* Removal of Radioactive Element from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF43Z"  =  "1"  /* Removal of Infusion Device from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF47Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF4JZ"  =  "1"  /* Removal of Synthetic Substitute from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF4KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPF4YZ"  =  "1"  /* Removal of Other Device from Abdominal Wall, Percutaneous Endoscopic Approach */
"0WPG00Z"  =  "1"  /* Removal of Drainage Device from Peritoneal Cavity, Open Approach */
"0WPG01Z"  =  "1"  /* Removal of Radioactive Element from Peritoneal Cavity, Open Approach */
"0WPG03Z"  =  "1"  /* Removal of Infusion Device from Peritoneal Cavity, Open Approach */
"0WPG0JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneal Cavity, Open Approach */
"0WPG0YZ"  =  "1"  /* Removal of Other Device from Peritoneal Cavity, Open Approach */
"0WPG30Z"  =  "1"  /* Removal of Drainage Device from Peritoneal Cavity, Percutaneous Approach */
"0WPG31Z"  =  "1"  /* Removal of Radioactive Element from Peritoneal Cavity, Percutaneous Approach */
"0WPG33Z"  =  "1"  /* Removal of Infusion Device from Peritoneal Cavity, Percutaneous Approach */
"0WPG3JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneal Cavity, Percutaneous Approach */
"0WPG3YZ"  =  "1"  /* Removal of Other Device from Peritoneal Cavity, Percutaneous Approach */
"0WPG40Z"  =  "1"  /* Removal of Drainage Device from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WPG41Z"  =  "1"  /* Removal of Radioactive Element from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WPG43Z"  =  "1"  /* Removal of Infusion Device from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WPG4JZ"  =  "1"  /* Removal of Synthetic Substitute from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WPG4YZ"  =  "1"  /* Removal of Other Device from Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WPH00Z"  =  "1"  /* Removal of Drainage Device from Retroperitoneum, Open Approach */
"0WPH01Z"  =  "1"  /* Removal of Radioactive Element from Retroperitoneum, Open Approach */
"0WPH03Z"  =  "1"  /* Removal of Infusion Device from Retroperitoneum, Open Approach */
"0WPH0YZ"  =  "1"  /* Removal of Other Device from Retroperitoneum, Open Approach */
"0WPH30Z"  =  "1"  /* Removal of Drainage Device from Retroperitoneum, Percutaneous Approach */
"0WPH31Z"  =  "1"  /* Removal of Radioactive Element from Retroperitoneum, Percutaneous Approach */
"0WPH33Z"  =  "1"  /* Removal of Infusion Device from Retroperitoneum, Percutaneous Approach */
"0WPH3YZ"  =  "1"  /* Removal of Other Device from Retroperitoneum, Percutaneous Approach */
"0WPH40Z"  =  "1"  /* Removal of Drainage Device from Retroperitoneum, Percutaneous Endoscopic Approach */
"0WPH41Z"  =  "1"  /* Removal of Radioactive Element from Retroperitoneum, Percutaneous Endoscopic Approach */
"0WPH43Z"  =  "1"  /* Removal of Infusion Device from Retroperitoneum, Percutaneous Endoscopic Approach */
"0WPH4YZ"  =  "1"  /* Removal of Other Device from Retroperitoneum, Percutaneous Endoscopic Approach */
"0WPP01Z"  =  "1"  /* Removal of Radioactive Element from Gastrointestinal Tract, Open Approach */
"0WPP03Z"  =  "1"  /* Removal of Infusion Device from Gastrointestinal Tract, Open Approach */
"0WPP0YZ"  =  "1"  /* Removal of Other Device from Gastrointestinal Tract, Open Approach */
"0WQF0ZZ"  =  "1"  /* Repair Abdominal Wall, Open Approach */
"0WQF3ZZ"  =  "1"  /* Repair Abdominal Wall, Percutaneous Approach */
"0WQF4ZZ"  =  "1"  /* Repair Abdominal Wall, Percutaneous Endoscopic Approach */
"0WQFXZ2"  =  "1"  /* Repair Abdominal Wall, Stoma, External Approach */
"0WQFXZZ"  =  "1"  /* Repair Abdominal Wall, External Approach */
"0WUF07Z"  =  "1"  /* Supplement Abdominal Wall with Autologous Tissue Substitute, Open Approach */
"0WUF0JZ"  =  "1"  /* Supplement Abdominal Wall with Synthetic Substitute, Open Approach */
"0WUF0KZ"  =  "1"  /* Supplement Abdominal Wall with Nonautologous Tissue Substitute, Open Approach */
"0WWF00Z"  =  "1"  /* Revision of Drainage Device in Abdominal Wall, Open Approach */
"0WWF01Z"  =  "1"  /* Revision of Radioactive Element in Abdominal Wall, Open Approach */
"0WWF03Z"  =  "1"  /* Revision of Infusion Device in Abdominal Wall, Open Approach */
"0WWF07Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Abdominal Wall, Open Approach */
"0WWF0JZ"  =  "1"  /* Revision of Synthetic Substitute in Abdominal Wall, Open Approach */
"0WWF0KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Abdominal Wall, Open Approach */
"0WWF0YZ"  =  "1"  /* Revision of Other Device in Abdominal Wall, Open Approach */
"0WWF30Z"  =  "1"  /* Revision of Drainage Device in Abdominal Wall, Percutaneous Approach */
"0WWF31Z"  =  "1"  /* Revision of Radioactive Element in Abdominal Wall, Percutaneous Approach */
"0WWF33Z"  =  "1"  /* Revision of Infusion Device in Abdominal Wall, Percutaneous Approach */
"0WWF37Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Abdominal Wall, Percutaneous Approach */
"0WWF3JZ"  =  "1"  /* Revision of Synthetic Substitute in Abdominal Wall, Percutaneous Approach */
"0WWF3KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Abdominal Wall, Percutaneous Approach */
"0WWF3YZ"  =  "1"  /* Revision of Other Device in Abdominal Wall, Percutaneous Approach */
"0WWF40Z"  =  "1"  /* Revision of Drainage Device in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF41Z"  =  "1"  /* Revision of Radioactive Element in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF43Z"  =  "1"  /* Revision of Infusion Device in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF47Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF4JZ"  =  "1"  /* Revision of Synthetic Substitute in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF4KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWF4YZ"  =  "1"  /* Revision of Other Device in Abdominal Wall, Percutaneous Endoscopic Approach */
"0WWG00Z"  =  "1"  /* Revision of Drainage Device in Peritoneal Cavity, Open Approach */
"0WWG01Z"  =  "1"  /* Revision of Radioactive Element in Peritoneal Cavity, Open Approach */
"0WWG03Z"  =  "1"  /* Revision of Infusion Device in Peritoneal Cavity, Open Approach */
"0WWG0JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneal Cavity, Open Approach */
"0WWG0YZ"  =  "1"  /* Revision of Other Device in Peritoneal Cavity, Open Approach */
"0WWG30Z"  =  "1"  /* Revision of Drainage Device in Peritoneal Cavity, Percutaneous Approach */
"0WWG31Z"  =  "1"  /* Revision of Radioactive Element in Peritoneal Cavity, Percutaneous Approach */
"0WWG33Z"  =  "1"  /* Revision of Infusion Device in Peritoneal Cavity, Percutaneous Approach */
"0WWG3JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneal Cavity, Percutaneous Approach */
"0WWG3YZ"  =  "1"  /* Revision of Other Device in Peritoneal Cavity, Percutaneous Approach */
"0WWG40Z"  =  "1"  /* Revision of Drainage Device in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WWG41Z"  =  "1"  /* Revision of Radioactive Element in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WWG43Z"  =  "1"  /* Revision of Infusion Device in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WWG4JZ"  =  "1"  /* Revision of Synthetic Substitute in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WWG4YZ"  =  "1"  /* Revision of Other Device in Peritoneal Cavity, Percutaneous Endoscopic Approach */
"0WWH00Z"  =  "1"  /* Revision of Drainage Device in Retroperitoneum, Open Approach */
"0WWH01Z"  =  "1"  /* Revision of Radioactive Element in Retroperitoneum, Open Approach */
"0WWH03Z"  =  "1"  /* Revision of Infusion Device in Retroperitoneum, Open Approach */
"0WWH0YZ"  =  "1"  /* Revision of Other Device in Retroperitoneum, Open Approach */
"0WWH30Z"  =  "1"  /* Revision of Drainage Device in Retroperitoneum, Percutaneous Approach */
"0WWH31Z"  =  "1"  /* Revision of Radioactive Element in Retroperitoneum, Percutaneous Approach */
"0WWH33Z"  =  "1"  /* Revision of Infusion Device in Retroperitoneum, Percutaneous Approach */
"0WWH3YZ"  =  "1"  /* Revision of Other Device in Retroperitoneum, Percutaneous Approach */
"0WWH40Z"  =  "1"  /* Revision of Drainage Device in Retroperitoneum, Percutaneous Endoscopic Approach */
"0WWH41Z"  =  "1"  /* Revision of Radioactive Element in Retroperitoneum, Percutaneous Endoscopic Approach */
"0WWH43Z"  =  "1"  /* Revision of Infusion Device in Retroperitoneum, Percutaneous Endoscopic Approach */
"0WWH4YZ"  =  "1"  /* Revision of Other Device in Retroperitoneum, Percutaneous Endoscopic Approach */
"0WWJ00Z"  =  "1"  /* Revision of Drainage Device in Pelvic Cavity, Open Approach */
"0WWJ01Z"  =  "1"  /* Revision of Radioactive Element in Pelvic Cavity, Open Approach */
"0WWJ03Z"  =  "1"  /* Revision of Infusion Device in Pelvic Cavity, Open Approach */
"0WWJ0JZ"  =  "1"  /* Revision of Synthetic Substitute in Pelvic Cavity, Open Approach */
"0WWJ0YZ"  =  "1"  /* Revision of Other Device in Pelvic Cavity, Open Approach */
"0WWJ30Z"  =  "1"  /* Revision of Drainage Device in Pelvic Cavity, Percutaneous Approach */
"0WWJ31Z"  =  "1"  /* Revision of Radioactive Element in Pelvic Cavity, Percutaneous Approach */
"0WWJ33Z"  =  "1"  /* Revision of Infusion Device in Pelvic Cavity, Percutaneous Approach */
"0WWJ3JZ"  =  "1"  /* Revision of Synthetic Substitute in Pelvic Cavity, Percutaneous Approach */
"0WWJ3YZ"  =  "1"  /* Revision of Other Device in Pelvic Cavity, Percutaneous Approach */
"0WWJ40Z"  =  "1"  /* Revision of Drainage Device in Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WWJ41Z"  =  "1"  /* Revision of Radioactive Element in Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WWJ43Z"  =  "1"  /* Revision of Infusion Device in Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WWJ4JZ"  =  "1"  /* Revision of Synthetic Substitute in Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WWJ4YZ"  =  "1"  /* Revision of Other Device in Pelvic Cavity, Percutaneous Endoscopic Approach */
"0WWP01Z"  =  "1"  /* Revision of Radioactive Element in Gastrointestinal Tract, Open Approach */
"0WWP03Z"  =  "1"  /* Revision of Infusion Device in Gastrointestinal Tract, Open Approach */
"0WWP0YZ"  =  "1"  /* Revision of Other Device in Gastrointestinal Tract, Open Approach */
"0Y9500Z"  =  "1"  /* Drainage of Right Inguinal Region with Drainage Device, Open Approach */
"0Y950ZZ"  =  "1"  /* Drainage of Right Inguinal Region, Open Approach */
"0Y9530Z"  =  "1"  /* Drainage of Right Inguinal Region with Drainage Device, Percutaneous Approach */
"0Y953ZZ"  =  "1"  /* Drainage of Right Inguinal Region, Percutaneous Approach */
"0Y9540Z"  =  "1"  /* Drainage of Right Inguinal Region with Drainage Device, Percutaneous Endoscopic Approach */
"0Y954ZZ"  =  "1"  /* Drainage of Right Inguinal Region, Percutaneous Endoscopic Approach */
"0Y9600Z"  =  "1"  /* Drainage of Left Inguinal Region with Drainage Device, Open Approach */
"0Y960ZZ"  =  "1"  /* Drainage of Left Inguinal Region, Open Approach */
"0Y9630Z"  =  "1"  /* Drainage of Left Inguinal Region with Drainage Device, Percutaneous Approach */
"0Y963ZZ"  =  "1"  /* Drainage of Left Inguinal Region, Percutaneous Approach */
"0Y9640Z"  =  "1"  /* Drainage of Left Inguinal Region with Drainage Device, Percutaneous Endoscopic Approach */
"0Y964ZZ"  =  "1"  /* Drainage of Left Inguinal Region, Percutaneous Endoscopic Approach */
"0YB50ZZ"  =  "1"  /* Excision of Right Inguinal Region, Open Approach */
"0YB53ZZ"  =  "1"  /* Excision of Right Inguinal Region, Percutaneous Approach */
"0YB54ZZ"  =  "1"  /* Excision of Right Inguinal Region, Percutaneous Endoscopic Approach */
"0YB60ZZ"  =  "1"  /* Excision of Left Inguinal Region, Open Approach */
"0YB63ZZ"  =  "1"  /* Excision of Left Inguinal Region, Percutaneous Approach */
"0YB64ZZ"  =  "1"  /* Excision of Left Inguinal Region, Percutaneous Endoscopic Approach */
"0YB70ZZ"  =  "1"  /* Excision of Right Femoral Region, Open Approach */
"0YB73ZZ"  =  "1"  /* Excision of Right Femoral Region, Percutaneous Approach */
"0YB74ZZ"  =  "1"  /* Excision of Right Femoral Region, Percutaneous Endoscopic Approach */
"0YB80ZZ"  =  "1"  /* Excision of Left Femoral Region, Open Approach */
"0YB83ZZ"  =  "1"  /* Excision of Left Femoral Region, Percutaneous Approach */
"0YB84ZZ"  =  "1"  /* Excision of Left Femoral Region, Percutaneous Endoscopic Approach */
"0YJ50ZZ"  =  "1"  /* Inspection of Right Inguinal Region, Open Approach */
"0YJ60ZZ"  =  "1"  /* Inspection of Left Inguinal Region, Open Approach */
"0YJ70ZZ"  =  "1"  /* Inspection of Right Femoral Region, Open Approach */
"0YJA0ZZ"  =  "1"  /* Inspection of Bilateral Inguinal Region, Open Approach */
"0YQ50ZZ"  =  "1"  /* Repair Right Inguinal Region, Open Approach */
"0YQ53ZZ"  =  "1"  /* Repair Right Inguinal Region, Percutaneous Approach */
"0YQ54ZZ"  =  "1"  /* Repair Right Inguinal Region, Percutaneous Endoscopic Approach */
"0YQ60ZZ"  =  "1"  /* Repair Left Inguinal Region, Open Approach */
"0YQ63ZZ"  =  "1"  /* Repair Left Inguinal Region, Percutaneous Approach */
"0YQ64ZZ"  =  "1"  /* Repair Left Inguinal Region, Percutaneous Endoscopic Approach */
"0YQ70ZZ"  =  "1"  /* Repair Right Femoral Region, Open Approach */
"0YQ73ZZ"  =  "1"  /* Repair Right Femoral Region, Percutaneous Approach */
"0YQ74ZZ"  =  "1"  /* Repair Right Femoral Region, Percutaneous Endoscopic Approach */
"0YQ80ZZ"  =  "1"  /* Repair Left Femoral Region, Open Approach */
"0YQ83ZZ"  =  "1"  /* Repair Left Femoral Region, Percutaneous Approach */
"0YQ84ZZ"  =  "1"  /* Repair Left Femoral Region, Percutaneous Endoscopic Approach */
"0YQA0ZZ"  =  "1"  /* Repair Bilateral Inguinal Region, Open Approach */
"0YQA3ZZ"  =  "1"  /* Repair Bilateral Inguinal Region, Percutaneous Approach */
"0YQA4ZZ"  =  "1"  /* Repair Bilateral Inguinal Region, Percutaneous Endoscopic Approach */
"0YQE0ZZ"  =  "1"  /* Repair Bilateral Femoral Region, Open Approach */
"0YQE3ZZ"  =  "1"  /* Repair Bilateral Femoral Region, Percutaneous Approach */
"0YQE4ZZ"  =  "1"  /* Repair Bilateral Femoral Region, Percutaneous Endoscopic Approach */
"0YU507Z"  =  "1"  /* Supplement Right Inguinal Region with Autologous Tissue Substitute, Open Approach */
"0YU50JZ"  =  "1"  /* Supplement Right Inguinal Region with Synthetic Substitute, Open Approach */
"0YU50KZ"  =  "1"  /* Supplement Right Inguinal Region with Nonautologous Tissue Substitute, Open Approach */
"0YU607Z"  =  "1"  /* Supplement Left Inguinal Region with Autologous Tissue Substitute, Open Approach */
"0YU60JZ"  =  "1"  /* Supplement Left Inguinal Region with Synthetic Substitute, Open Approach */
"0YU60KZ"  =  "1"  /* Supplement Left Inguinal Region with Nonautologous Tissue Substitute, Open Approach */
"0YU707Z"  =  "1"  /* Supplement Right Femoral Region with Autologous Tissue Substitute, Open Approach */
"0YU70JZ"  =  "1"  /* Supplement Right Femoral Region with Synthetic Substitute, Open Approach */
"0YU70KZ"  =  "1"  /* Supplement Right Femoral Region with Nonautologous Tissue Substitute, Open Approach */
"0YU747Z"  =  "1"  /* Supplement Right Femoral Region with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0YU74JZ"  =  "1"  /* Supplement Right Femoral Region with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0YU74KZ"  =  "1"  /* Supplement Right Femoral Region with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0YU807Z"  =  "1"  /* Supplement Left Femoral Region with Autologous Tissue Substitute, Open Approach */
"0YU80JZ"  =  "1"  /* Supplement Left Femoral Region with Synthetic Substitute, Open Approach */
"0YU80KZ"  =  "1"  /* Supplement Left Femoral Region with Nonautologous Tissue Substitute, Open Approach */
"0YU847Z"  =  "1"  /* Supplement Left Femoral Region with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0YU84JZ"  =  "1"  /* Supplement Left Femoral Region with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0YU84KZ"  =  "1"  /* Supplement Left Femoral Region with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0YUA07Z"  =  "1"  /* Supplement Bilateral Inguinal Region with Autologous Tissue Substitute, Open Approach */
"0YUA0JZ"  =  "1"  /* Supplement Bilateral Inguinal Region with Synthetic Substitute, Open Approach */
"0YUA0KZ"  =  "1"  /* Supplement Bilateral Inguinal Region with Nonautologous Tissue Substitute, Open Approach */
"0YUE07Z"  =  "1"  /* Supplement Bilateral Femoral Region with Autologous Tissue Substitute, Open Approach */
"0YUE0JZ"  =  "1"  /* Supplement Bilateral Femoral Region with Synthetic Substitute, Open Approach */
"0YUE0KZ"  =  "1"  /* Supplement Bilateral Femoral Region with Nonautologous Tissue Substitute, Open Approach */
"0YUE47Z"  =  "1"  /* Supplement Bilateral Femoral Region with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0YUE4JZ"  =  "1"  /* Supplement Bilateral Femoral Region with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0YUE4KZ"  =  "1"  /* Supplement Bilateral Femoral Region with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
OTHER  = "0" ;

/* VALUE $ABDOM2P - no mapping found */

 /* RECLOSURE OF POSTOPERATIVE DISRUPTION OF ABDOMINAL WALL */;
 /* ICD-10-CM PROCEDURE CODE: */

VALUE $RECLOIP
"0WQFXZZ"  =  "1"  /* Repair Abdominal Wall, External Approach */
OTHER   = "0" ;

 /* REPAIR OF GASTROSCHISIS OR UMBILICAL HERNIA */;
 /* ICD-10-CM PROCEDURE CODE: */

VALUE $REPGAST
"0WQF0ZZ"  =  "1"  /* Repair Abdominal Wall, Open Approach */
"0WQF3ZZ"  =  "1"  /* Repair Abdominal Wall, Percutaneous Approach */
"0WQF4ZZ"  =  "1"  /* Repair Abdominal Wall, Percutaneous Endoscopic Approach */
"0WUF07Z"  =  "1"  /* Supplement Abdominal Wall with Autologous Tissue Substitute, Open Approach */
"0WUF0JZ"  =  "1"  /* Supplement Abdominal Wall with Synthetic Substitute, Open Approach */
"0WUF0KZ"  =  "1"  /* Supplement Abdominal Wall with Nonautologous Tissue Substitute, Open Approach */
OTHER   = "0" ;

/* EXCLUDE: */
/* HIGH RISK GROUP */
/* ICD-10-CM DIAGNOSIS CODES: */

VALUE $IMMUNHD
"B20"  =  "1"  /* Human immunodeficiency virus [HIV] disease */
"B59"  =  "1"  /* Pneumocystosis */
"C802"  =  "1"  /* Malignant neoplasm associated with transplanted organ */
"C8300"  =  "1"  /* Small cell B-cell lymphoma, unspecified site */
"C8301"  =  "1"  /* Small cell B-cell lymphoma, lymph nodes of head, face, and neck */
"C8302"  =  "1"  /* Small cell B-cell lymphoma, intrathoracic lymph nodes */
"C8303"  =  "1"  /* Small cell B-cell lymphoma, intra-abdominal lymph nodes */
"C8304"  =  "1"  /* Small cell B-cell lymphoma, lymph nodes of axilla and upper limb */
"C8305"  =  "1"  /* Small cell B-cell lymphoma, lymph nodes of inguinal region and lower limb */
"C8306"  =  "1"  /* Small cell B-cell lymphoma, intrapelvic lymph nodes */
"C8307"  =  "1"  /* Small cell B-cell lymphoma, spleen */
"C8308"  =  "1"  /* Small cell B-cell lymphoma, lymph nodes of multiple sites */
"C8309"  =  "1"  /* Small cell B-cell lymphoma, extranodal and solid organ sites */
"C8330"  =  "1"  /* Diffuse large B-cell lymphoma, unspecified site */
"C8331"  =  "1"  /* Diffuse large B-cell lymphoma, lymph nodes of head, face, and neck */
"C8332"  =  "1"  /* Diffuse large B-cell lymphoma, intrathoracic lymph nodes */
"C8333"  =  "1"  /* Diffuse large B-cell lymphoma, intra-abdominal lymph nodes */
"C8334"  =  "1"  /* Diffuse large B-cell lymphoma, lymph nodes of axilla and upper limb */
"C8335"  =  "1"  /* Diffuse large B-cell lymphoma, lymph nodes of inguinal region and lower limb */
"C8336"  =  "1"  /* Diffuse large B-cell lymphoma, intrapelvic lymph nodes */
"C8337"  =  "1"  /* Diffuse large B-cell lymphoma, spleen */
"C8338"  =  "1"  /* Diffuse large B-cell lymphoma, lymph nodes of multiple sites */
"C8339"  =  "1"  /* Diffuse large B-cell lymphoma, extranodal and solid organ sites */
"C8350"  =  "1"  /* Lymphoblastic (diffuse) lymphoma, unspecified site */
"C8351"  =  "1"  /* Lymphoblastic (diffuse) lymphoma, lymph nodes of head, face, and neck */
"C8352"  =  "1"  /* Lymphoblastic (diffuse) lymphoma, intrathoracic lymph nodes */
"C8353"  =  "1"  /* Lymphoblastic (diffuse) lymphoma, intra-abdominal lymph nodes */
"C8354"  =  "1"  /* Lymphoblastic (diffuse) lymphoma, lymph nodes of axilla and upper limb */
"C8355"  =  "1"  /* Lymphoblastic (diffuse) lymphoma, lymph nodes of inguinal region and lower limb */
"C8356"  =  "1"  /* Lymphoblastic (diffuse) lymphoma, intrapelvic lymph nodes */
"C8357"  =  "1"  /* Lymphoblastic (diffuse) lymphoma, spleen */
"C8358"  =  "1"  /* Lymphoblastic (diffuse) lymphoma, lymph nodes of multiple sites */
"C8359"  =  "1"  /* Lymphoblastic (diffuse) lymphoma, extranodal and solid organ sites */
"C8370"  =  "1"  /* Burkitt lymphoma, unspecified site */
"C8371"  =  "1"  /* Burkitt lymphoma, lymph nodes of head, face, and neck */
"C8372"  =  "1"  /* Burkitt lymphoma, intrathoracic lymph nodes */
"C8373"  =  "1"  /* Burkitt lymphoma, intra-abdominal lymph nodes */
"C8374"  =  "1"  /* Burkitt lymphoma, lymph nodes of axilla and upper limb */
"C8375"  =  "1"  /* Burkitt lymphoma, lymph nodes of inguinal region and lower limb */
"C8376"  =  "1"  /* Burkitt lymphoma, intrapelvic lymph nodes */
"C8377"  =  "1"  /* Burkitt lymphoma, spleen */
"C8378"  =  "1"  /* Burkitt lymphoma, lymph nodes of multiple sites */
"C8379"  =  "1"  /* Burkitt lymphoma, extranodal and solid organ sites */
"C8380"  =  "1"  /* Other non-follicular lymphoma, unspecified site */
"C8381"  =  "1"  /* Other non-follicular lymphoma, lymph nodes of head, face, and neck */
"C8382"  =  "1"  /* Other non-follicular lymphoma, intrathoracic lymph nodes */
"C8383"  =  "1"  /* Other non-follicular lymphoma, intra-abdominal lymph nodes */
"C8384"  =  "1"  /* Other non-follicular lymphoma, lymph nodes of axilla and upper limb */
"C8385"  =  "1"  /* Other non-follicular lymphoma, lymph nodes of inguinal region and lower limb */
"C8386"  =  "1"  /* Other non-follicular lymphoma, intrapelvic lymph nodes */
"C8387"  =  "1"  /* Other non-follicular lymphoma, spleen */
"C8388"  =  "1"  /* Other non-follicular lymphoma, lymph nodes of multiple sites */
"C8389"  =  "1"  /* Other non-follicular lymphoma, extranodal and solid organ sites */
"C8390"  =  "1"  /* Non-follicular (diffuse) lymphoma, unspecified, unspecified site */
"C8391"  =  "1"  /* Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of head, face, and neck */
"C8392"  =  "1"  /* Non-follicular (diffuse) lymphoma, unspecified, intrathoracic lymph nodes */
"C8393"  =  "1"  /* Non-follicular (diffuse) lymphoma, unspecified, intra-abdominal lymph nodes */
"C8394"  =  "1"  /* Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of axilla and upper limb */
"C8395"  =  "1"  /* Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of inguinal region and lower limb */
"C8396"  =  "1"  /* Non-follicular (diffuse) lymphoma, unspecified, intrapelvic lymph nodes */
"C8397"  =  "1"  /* Non-follicular (diffuse) lymphoma, unspecified, spleen */
"C8398"  =  "1"  /* Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of multiple sites */
"C8399"  =  "1"  /* Non-follicular (diffuse) lymphoma, unspecified, extranodal and solid organ sites */
"C865"  =  "1"  /* Angioimmunoblastic T-cell lymphoma */
"C866"  =  "1"  /* Primary cutaneous CD30-positive T-cell proliferations */
"C888"  =  "1"  /* Other malignant immunoproliferative diseases */
"C9002"  =  "1"  /* Multiple myeloma in relapse */
"C9012"  =  "1"  /* Plasma cell leukemia in relapse */
"C9022"  =  "1"  /* Extramedullary plasmacytoma in relapse */
"C9032"  =  "1"  /* Solitary plasmacytoma in relapse */
"C9102"  =  "1"  /* Acute lymphoblastic leukemia, in relapse */
"C9112"  =  "1"  /* Chronic lymphocytic leukemia of B-cell type in relapse */
"C9132"  =  "1"  /* Prolymphocytic leukemia of B-cell type, in relapse */
"C9152"  =  "1"  /* Adult T-cell lymphoma/leukemia (HTLV-1-associated), in relapse */
"C9162"  =  "1"  /* Prolymphocytic leukemia of T-cell type, in relapse */
"C9192"  =  "1"  /* Lymphoid leukemia, unspecified, in relapse */
"C91A2"  =  "1"  /* Mature B-cell leukemia Burkitt-type, in relapse */
"C91Z2"  =  "1"  /* Other lymphoid leukemia, in relapse */
"C9200"  =  "1"  /* Acute myeloblastic leukemia, not having achieved remission */
"C9201"  =  "1"  /* Acute myeloblastic leukemia, in remission */
"C9202"  =  "1"  /* Acute myeloblastic leukemia, in relapse */
"C9210"  =  "1"  /* Chronic myeloid leukemia, BCR/ABL-positive, not having achieved remission */
"C9211"  =  "1"  /* Chronic myeloid leukemia, BCR/ABL-positive, in remission */
"C9212"  =  "1"  /* Chronic myeloid leukemia, BCR/ABL-positive, in relapse */
"C9220"  =  "1"  /* Atypical chronic myeloid leukemia, BCR/ABL-negative, not having achieved remission */
"C9221"  =  "1"  /* Atypical chronic myeloid leukemia, BCR/ABL-negative, in remission */
"C9222"  =  "1"  /* Atypical chronic myeloid leukemia, BCR/ABL-negative, in relapse */
"C9230"  =  "1"  /* Myeloid sarcoma, not having achieved remission */
"C9231"  =  "1"  /* Myeloid sarcoma, in remission */
"C9232"  =  "1"  /* Myeloid sarcoma, in relapse */
"C9240"  =  "1"  /* Acute promyelocytic leukemia, not having achieved remission */
"C9241"  =  "1"  /* Acute promyelocytic leukemia, in remission */
"C9242"  =  "1"  /* Acute promyelocytic leukemia, in relapse */
"C9250"  =  "1"  /* Acute myelomonocytic leukemia, not having achieved remission */
"C9251"  =  "1"  /* Acute myelomonocytic leukemia, in remission */
"C9252"  =  "1"  /* Acute myelomonocytic leukemia, in relapse */
"C9260"  =  "1"  /* Acute myeloid leukemia with 11q23-abnormality not having achieved remission */
"C9261"  =  "1"  /* Acute myeloid leukemia with 11q23-abnormality in remission */
"C9262"  =  "1"  /* Acute myeloid leukemia with 11q23-abnormality in relapse */
"C9290"  =  "1"  /* Myeloid leukemia, unspecified, not having achieved remission */
"C9291"  =  "1"  /* Myeloid leukemia, unspecified in remission */
"C9292"  =  "1"  /* Myeloid leukemia, unspecified in relapse */
"C92A0"  =  "1"  /* Acute myeloid leukemia with multilineage dysplasia, not having achieved remission */
"C92A1"  =  "1"  /* Acute myeloid leukemia with multilineage dysplasia, in remission */
"C92A2"  =  "1"  /* Acute myeloid leukemia with multilineage dysplasia, in relapse */
"C92Z0"  =  "1"  /* Other myeloid leukemia not having achieved remission */
"C92Z1"  =  "1"  /* Other myeloid leukemia, in remission */
"C92Z2"  =  "1"  /* Other myeloid leukemia, in relapse */
"C9300"  =  "1"  /* Acute monoblastic/monocytic leukemia, not having achieved remission */
"C9301"  =  "1"  /* Acute monoblastic/monocytic leukemia, in remission */
"C9302"  =  "1"  /* Acute monoblastic/monocytic leukemia, in relapse */
"C9310"  =  "1"  /* Chronic myelomonocytic leukemia not having achieved remission */
"C9311"  =  "1"  /* Chronic myelomonocytic leukemia, in remission */
"C9312"  =  "1"  /* Chronic myelomonocytic leukemia, in relapse */
"C9330"  =  "1"  /* Juvenile myelomonocytic leukemia, not having achieved remission */
"C9331"  =  "1"  /* Juvenile myelomonocytic leukemia, in remission */
"C9332"  =  "1"  /* Juvenile myelomonocytic leukemia, in relapse */
"C9390"  =  "1"  /* Monocytic leukemia, unspecified, not having achieved remission */
"C9391"  =  "1"  /* Monocytic leukemia, unspecified in remission */
"C9392"  =  "1"  /* Monocytic leukemia, unspecified in relapse */
"C93Z0"  =  "1"  /* Other monocytic leukemia, not having achieved remission */
"C93Z1"  =  "1"  /* Other monocytic leukemia, in remission */
"C93Z2"  =  "1"  /* Other monocytic leukemia, in relapse */
"C9400"  =  "1"  /* Acute erythroid leukemia, not having achieved remission */
"C9401"  =  "1"  /* Acute erythroid leukemia, in remission */
"C9402"  =  "1"  /* Acute erythroid leukemia, in relapse */
"C9420"  =  "1"  /* Acute megakaryoblastic leukemia not having achieved remission */
"C9421"  =  "1"  /* Acute megakaryoblastic leukemia, in remission */
"C9422"  =  "1"  /* Acute megakaryoblastic leukemia, in relapse */
"C9430"  =  "1"  /* Mast cell leukemia not having achieved remission */
"C9431"  =  "1"  /* Mast cell leukemia, in remission */
"C9432"  =  "1"  /* Mast cell leukemia, in relapse */
"C9440"  =  "1"  /* Acute panmyelosis with myelofibrosis not having achieved remission */
"C9441"  =  "1"  /* Acute panmyelosis with myelofibrosis, in remission */
"C9442"  =  "1"  /* Acute panmyelosis with myelofibrosis, in relapse */
"C946"  =  "1"  /* Myelodysplastic disease, not classified */
"C9480"  =  "1"  /* Other specified leukemias not having achieved remission */
"C9481"  =  "1"  /* Other specified leukemias, in remission */
"C9482"  =  "1"  /* Other specified leukemias, in relapse */
"C9500"  =  "1"  /* Acute leukemia of unspecified cell type not having achieved remission */
"C9501"  =  "1"  /* Acute leukemia of unspecified cell type, in remission */
"C9502"  =  "1"  /* Acute leukemia of unspecified cell type, in relapse */
"C9510"  =  "1"  /* Chronic leukemia of unspecified cell type not having achieved remission */
"C9511"  =  "1"  /* Chronic leukemia of unspecified cell type, in remission */
"C9512"  =  "1"  /* Chronic leukemia of unspecified cell type, in relapse */
"C9590"  =  "1"  /* Leukemia, unspecified not having achieved remission */
"C9591"  =  "1"  /* Leukemia, unspecified, in remission */
"C9592"  =  "1"  /* Leukemia, unspecified, in relapse */
"D45"  =  "1"  /* Polycythemia vera */
"D4622"  =  "1"  /* Refractory anemia with excess of blasts 2 */
"D471"  =  "1"  /* Chronic myeloproliferative disease */
"D479"  =  "1"  /* Neoplasm of uncertain behavior of lymphoid, hematopoietic and related tissue, unspecified */
"D47Z1"  =  "1"  /* Post-transplant lymphoproliferative disorder (PTLD) */
"D47Z9"  =  "1"  /* Other specified neoplasms of uncertain behavior of lymphoid, hematopoietic and related tissue */
"D6109"  =  "1"  /* Other constitutional aplastic anemia */
"D61810"  =  "1"  /* Antineoplastic chemotherapy induced pancytopenia */
"D61811"  =  "1"  /* Other drug-induced pancytopenia */
"D61818"  =  "1"  /* Other pancytopenia */
"D700"  =  "1"  /* Congenital agranulocytosis */
"D701"  =  "1"  /* Agranulocytosis secondary to cancer chemotherapy */
"D702"  =  "1"  /* Other drug-induced agranulocytosis */
"D704"  =  "1"  /* Cyclic neutropenia */
"D708"  =  "1"  /* Other neutropenia */
"D709"  =  "1"  /* Neutropenia, unspecified */
"D71"  =  "1"  /* Functional disorders of polymorphonuclear neutrophils */
"D720"  =  "1"  /* Genetic anomalies of leukocytes */
"D72810"  =  "1"  /* Lymphocytopenia */
"D72818"  =  "1"  /* Other decreased white blood cell count */
"D72819"  =  "1"  /* Decreased white blood cell count, unspecified */
"D7381"  =  "1"  /* Neutropenic splenomegaly */
"D7581"  =  "1"  /* Myelofibrosis */
"D761"  =  "1"  /* Hemophagocytic lymphohistiocytosis */
"D762"  =  "1"  /* Hemophagocytic syndrome, infection-associated */
"D763"  =  "1"  /* Other histiocytosis syndromes */
"D800"  =  "1"  /* Hereditary hypogammaglobulinemia */
"D801"  =  "1"  /* Nonfamilial hypogammaglobulinemia */
"D802"  =  "1"  /* Selective deficiency of immunoglobulin A [IgA] */
"D803"  =  "1"  /* Selective deficiency of immunoglobulin G [IgG] subclasses */
"D804"  =  "1"  /* Selective deficiency of immunoglobulin M [IgM] */
"D805"  =  "1"  /* Immunodeficiency with increased immunoglobulin M [IgM] */
"D806"  =  "1"  /* Antibody deficiency with near-normal immunoglobulins or with hyperimmunoglobulinemia */
"D807"  =  "1"  /* Transient hypogammaglobulinemia of infancy */
"D808"  =  "1"  /* Other immunodeficiencies with predominantly antibody defects */
"D809"  =  "1"  /* Immunodeficiency with predominantly antibody defects, unspecified */
"D810"  =  "1"  /* Severe combined immunodeficiency [SCID] with reticular dysgenesis */
"D811"  =  "1"  /* Severe combined immunodeficiency [SCID] with low T- and B-cell numbers */
"D812"  =  "1"  /* Severe combined immunodeficiency [SCID] with low or normal B-cell numbers */
"D814"  =  "1"  /* Nezelof's syndrome */
"D816"  =  "1"  /* Major histocompatibility complex class I deficiency */
"D817"  =  "1"  /* Major histocompatibility complex class II deficiency */
"D8189"  =  "1"  /* Other combined immunodeficiencies */
"D819"  =  "1"  /* Combined immunodeficiency, unspecified */
"D820"  =  "1"  /* Wiskott-Aldrich syndrome */
"D821"  =  "1"  /* Di George's syndrome */
"D822"  =  "1"  /* Immunodeficiency with short-limbed stature */
"D823"  =  "1"  /* Immunodeficiency following hereditary defective response to Epstein-Barr virus */
"D824"  =  "1"  /* Hyperimmunoglobulin E [IgE] syndrome */
"D828"  =  "1"  /* Immunodeficiency associated with other specified major defects */
"D829"  =  "1"  /* Immunodeficiency associated with major defect, unspecified */
"D830"  =  "1"  /* Common variable immunodeficiency with predominant abnormalities of B-cell numbers and function */
"D831"  =  "1"  /* Common variable immunodeficiency with predominant immunoregulatory T-cell disorders */
"D832"  =  "1"  /* Common variable immunodeficiency with autoantibodies to B- or T-cells */
"D838"  =  "1"  /* Other common variable immunodeficiencies */
"D839"  =  "1"  /* Common variable immunodeficiency, unspecified */
"D840"  =  "1"  /* Lymphocyte function antigen-1 [LFA-1] defect */
"D841"  =  "1"  /* Defects in the complement system */
"D848"  =  "1"  /* Other specified immunodeficiencies */
"D849"  =  "1"  /* Immunodeficiency, unspecified */
"D893"  =  "1"  /* Immune reconstitution syndrome */
"D89810"  =  "1"  /* Acute graft-versus-host disease */
"D89811"  =  "1"  /* Chronic graft-versus-host disease */
"D89812"  =  "1"  /* Acute on chronic graft-versus-host disease */
"D89813"  =  "1"  /* Graft-versus-host disease, unspecified */
"D8982"  =  "1"  /* Autoimmune lymphoproliferative syndrome [ALPS] */
"D8989"  =  "1"  /* Other specified disorders involving the immune mechanism, not elsewhere classified */
"D899"  =  "1"  /* Disorder involving the immune mechanism, unspecified */
"E40"  =  "1"  /* Kwashiorkor */
"E41"  =  "1"  /* Nutritional marasmus */
"E42"  =  "1"  /* Marasmic kwashiorkor */
"E43"  =  "1"  /* Unspecified severe protein-calorie malnutrition */
"I120"  =  "1"  /* Hypertensive chronic kidney disease with stage 5 chronic kidney disease or end stage renal disease */
"I1311"  =  "1"  /* Hypertensive heart and chronic kidney disease without heart failure, with stage 5 chronic kidney disease, or end stage renal disease */
"I132"  =  "1"  /* Hypertensive heart and chronic kidney disease with heart failure and with stage 5 chronic kidney disease, or end stage renal disease */
"K912"  =  "1"  /* Postsurgical malabsorption, not elsewhere classified */
"N185"  =  "1"  /* Chronic kidney disease, stage 5 */
"N186"  =  "1"  /* End stage renal disease */
"T8600"  =  "1"  /* Unspecified complication of bone marrow transplant */
"T8601"  =  "1"  /* Bone marrow transplant rejection */
"T8602"  =  "1"  /* Bone marrow transplant failure */
"T8603"  =  "1"  /* Bone marrow transplant infection */
"T8609"  =  "1"  /* Other complications of bone marrow transplant */
"T8610"  =  "1"  /* Unspecified complication of kidney transplant */
"T8611"  =  "1"  /* Kidney transplant rejection */
"T8612"  =  "1"  /* Kidney transplant failure */
"T8613"  =  "1"  /* Kidney transplant infection */
"T8619"  =  "1"  /* Other complication of kidney transplant */
"T8620"  =  "1"  /* Unspecified complication of heart transplant */
"T8621"  =  "1"  /* Heart transplant rejection */
"T8622"  =  "1"  /* Heart transplant failure */
"T8623"  =  "1"  /* Heart transplant infection */
"T86290"  =  "1"  /* Cardiac allograft vasculopathy */
"T86298"  =  "1"  /* Other complications of heart transplant */
"T8630"  =  "1"  /* Unspecified complication of heart-lung transplant */
"T8631"  =  "1"  /* Heart-lung transplant rejection */
"T8632"  =  "1"  /* Heart-lung transplant failure */
"T8633"  =  "1"  /* Heart-lung transplant infection */
"T8639"  =  "1"  /* Other complications of heart-lung transplant */
"T8640"  =  "1"  /* Unspecified complication of liver transplant */
"T8641"  =  "1"  /* Liver transplant rejection */
"T8642"  =  "1"  /* Liver transplant failure */
"T8643"  =  "1"  /* Liver transplant infection */
"T8649"  =  "1"  /* Other complications of liver transplant */
"T865"  =  "1"  /* Complications of stem cell transplant */
"T86810"  =  "1"  /* Lung transplant rejection */
"T86811"  =  "1"  /* Lung transplant failure */
"T86812"  =  "1"  /* Lung transplant infection */
"T86818"  =  "1"  /* Other complications of lung transplant */
"T86819"  =  "1"  /* Unspecified complication of lung transplant */
"T86830"  =  "1"  /* Bone graft rejection */
"T86831"  =  "1"  /* Bone graft failure */
"T86832"  =  "1"  /* Bone graft infection */
"T86838"  =  "1"  /* Other complications of bone graft */
"T86839"  =  "1"  /* Unspecified complication of bone graft */
"T86850"  =  "1"  /* Intestine transplant rejection */
"T86851"  =  "1"  /* Intestine transplant failure */
"T86852"  =  "1"  /* Intestine transplant infection */
"T86858"  =  "1"  /* Other complications of intestine transplant */
"T86859"  =  "1"  /* Unspecified complication of intestine transplant */
"T86890"  =  "1"  /* Other transplanted tissue rejection */
"T86891"  =  "1"  /* Other transplanted tissue failure */
"T86892"  =  "1"  /* Other transplanted tissue infection */
"T86898"  =  "1"  /* Other complications of other transplanted tissue */
"T86899"  =  "1"  /* Unspecified complication of other transplanted tissue */
"T8690"  =  "1"  /* Unspecified complication of unspecified transplanted organ and tissue */
"T8691"  =  "1"  /* Unspecified transplanted organ and tissue rejection */
"T8692"  =  "1"  /* Unspecified transplanted organ and tissue failure */
"T8693"  =  "1"  /* Unspecified transplanted organ and tissue infection */
"T8699"  =  "1"  /* Other complications of unspecified transplanted organ and tissue */
"Z4821"  =  "1"  /* Encounter for aftercare following heart transplant */
"Z4822"  =  "1"  /* Encounter for aftercare following kidney transplant */
"Z4823"  =  "1"  /* Encounter for aftercare following liver transplant */
"Z4824"  =  "1"  /* Encounter for aftercare following lung transplant */
"Z48280"  =  "1"  /* Encounter for aftercare following heart-lung transplant */
"Z48290"  =  "1"  /* Encounter for aftercare following bone marrow transplant */
"Z48298"  =  "1"  /* Encounter for aftercare following other organ transplant */
"Z4901"  =  "1"  /* Encounter for fitting and adjustment of extracorporeal dialysis catheter */
"Z4902"  =  "1"  /* Encounter for fitting and adjustment of peritoneal dialysis catheter */
"Z4931"  =  "1"  /* Encounter for adequacy testing for hemodialysis */
"Z4932"  =  "1"  /* Encounter for adequacy testing for peritoneal dialysis */
"Z940"  =  "1"  /* Kidney transplant status */
"Z941"  =  "1"  /* Heart transplant status */
"Z942"  =  "1"  /* Lung transplant status */
"Z943"  =  "1"  /* Heart and lungs transplant status */
"Z944"  =  "1"  /* Liver transplant status */
"Z9481"  =  "1"  /* Bone marrow transplant status */
"Z9482"  =  "1"  /* Intestine transplant status */
"Z9483"  =  "1"  /* Pancreas transplant status */
"Z9484"  =  "1"  /* Stem cells transplant status */
"Z9489"  =  "1"  /* Other transplanted organ and tissue status */
"Z992"  =  "1"  /* Dependence on renal dialysis */
OTHER   = '0' ;

/* ICD-10-CM PROCEDURE CODES: */

VALUE $TRANSPP
"02YA0Z0"  =  "1"  /* Transplantation of Heart, Allogeneic, Open Approach */
"02YA0Z2"  =  "1"  /* Transplantation of Heart, Zooplastic, Open Approach */
"0BYC0Z0"  =  "1"  /* Transplantation of Right Upper Lung Lobe, Allogeneic, Open Approach */
"0BYC0Z2"  =  "1"  /* Transplantation of Right Upper Lung Lobe, Zooplastic, Open Approach */
"0BYD0Z0"  =  "1"  /* Transplantation of Right Middle Lung Lobe, Allogeneic, Open Approach */
"0BYD0Z2"  =  "1"  /* Transplantation of Right Middle Lung Lobe, Zooplastic, Open Approach */
"0BYF0Z0"  =  "1"  /* Transplantation of Right Lower Lung Lobe, Allogeneic, Open Approach */
"0BYF0Z2"  =  "1"  /* Transplantation of Right Lower Lung Lobe, Zooplastic, Open Approach */
"0BYG0Z0"  =  "1"  /* Transplantation of Left Upper Lung Lobe, Allogeneic, Open Approach */
"0BYG0Z2"  =  "1"  /* Transplantation of Left Upper Lung Lobe, Zooplastic, Open Approach */
"0BYH0Z0"  =  "1"  /* Transplantation of Lung Lingula, Allogeneic, Open Approach */
"0BYH0Z2"  =  "1"  /* Transplantation of Lung Lingula, Zooplastic, Open Approach */
"0BYJ0Z0"  =  "1"  /* Transplantation of Left Lower Lung Lobe, Allogeneic, Open Approach */
"0BYJ0Z2"  =  "1"  /* Transplantation of Left Lower Lung Lobe, Zooplastic, Open Approach */
"0BYK0Z0"  =  "1"  /* Transplantation of Right Lung, Allogeneic, Open Approach */
"0BYK0Z2"  =  "1"  /* Transplantation of Right Lung, Zooplastic, Open Approach */
"0BYL0Z0"  =  "1"  /* Transplantation of Left Lung, Allogeneic, Open Approach */
"0BYL0Z2"  =  "1"  /* Transplantation of Left Lung, Zooplastic, Open Approach */
"0BYM0Z0"  =  "1"  /* Transplantation of Bilateral Lungs, Allogeneic, Open Approach */
"0BYM0Z2"  =  "1"  /* Transplantation of Bilateral Lungs, Zooplastic, Open Approach */
"0DY50Z0"  =  "1"  /* Transplantation of Esophagus, Allogeneic, Open Approach */
"0DY50Z2"  =  "1"  /* Transplantation of Esophagus, Zooplastic, Open Approach */
"0DY60Z0"  =  "1"  /* Transplantation of Stomach, Allogeneic, Open Approach */
"0DY60Z2"  =  "1"  /* Transplantation of Stomach, Zooplastic, Open Approach */
"0DY80Z0"  =  "1"  /* Transplantation of Small Intestine, Allogeneic, Open Approach */
"0DY80Z2"  =  "1"  /* Transplantation of Small Intestine, Zooplastic, Open Approach */
"0DYE0Z0"  =  "1"  /* Transplantation of Large Intestine, Allogeneic, Open Approach */
"0DYE0Z2"  =  "1"  /* Transplantation of Large Intestine, Zooplastic, Open Approach */
"0FSG0ZZ"  =  "1"  /* Reposition Pancreas, Open Approach */
"0FSG4ZZ"  =  "1"  /* Reposition Pancreas, Percutaneous Endoscopic Approach */
"0FY00Z0"  =  "1"  /* Transplantation of Liver, Allogeneic, Open Approach */
"0FY00Z2"  =  "1"  /* Transplantation of Liver, Zooplastic, Open Approach */
"0FYG0Z0"  =  "1"  /* Transplantation of Pancreas, Allogeneic, Open Approach */
"0FYG0Z2"  =  "1"  /* Transplantation of Pancreas, Zooplastic, Open Approach */
"0TY00Z0"  =  "1"  /* Transplantation of Right Kidney, Allogeneic, Open Approach */
"0TY00Z2"  =  "1"  /* Transplantation of Right Kidney, Zooplastic, Open Approach */
"0TY10Z0"  =  "1"  /* Transplantation of Left Kidney, Allogeneic, Open Approach */
"0TY10Z2"  =  "1"  /* Transplantation of Left Kidney, Zooplastic, Open Approach */
"30230AZ"  =  "1"  /* Transfusion of Embryonic Stem Cells into Peripheral Vein, Open Approach */
"30230G0"  =  "1"  /* Transfusion of Autologous Bone Marrow into Peripheral Vein, Open Approach */
"30230G1"  =  "1"  /* Transfusion of Nonautologous Bone Marrow into Peripheral Vein, Open Approach */
"30230X0"  =  "1"  /* Transfusion of Autologous Cord Blood Stem Cells into Peripheral Vein, Open Approach */
"30230X1"  =  "1"  /* Transfusion of Nonautologous Cord Blood Stem Cells into Peripheral Vein, Open Approach */
"30230Y0"  =  "1"  /* Transfusion of Autologous Hematopoietic Stem Cells into Peripheral Vein, Open Approach */
"30230Y1"  =  "1"  /* Transfusion of Nonautologous Hematopoietic Stem Cells into Peripheral Vein, Open Approach */
"30233AZ"  =  "1"  /* Transfusion of Embryonic Stem Cells into Peripheral Vein, Percutaneous Approach */
"30233G0"  =  "1"  /* Transfusion of Autologous Bone Marrow into Peripheral Vein, Percutaneous Approach */
"30233G1"  =  "1"  /* Transfusion of Nonautologous Bone Marrow into Peripheral Vein, Percutaneous Approach */
"30233X0"  =  "1"  /* Transfusion of Autologous Cord Blood Stem Cells into Peripheral Vein, Percutaneous Approach */
"30233X1"  =  "1"  /* Transfusion of Nonautologous Cord Blood Stem Cells into Peripheral Vein, Percutaneous Approach */
"30233Y0"  =  "1"  /* Transfusion of Autologous Hematopoietic Stem Cells into Peripheral Vein, Percutaneous Approach */
"30233Y1"  =  "1"  /* Transfusion of Nonautologous Hematopoietic Stem Cells into Peripheral Vein, Percutaneous Approach */
"30240AZ"  =  "1"  /* Transfusion of Embryonic Stem Cells into Central Vein, Open Approach */
"30240G0"  =  "1"  /* Transfusion of Autologous Bone Marrow into Central Vein, Open Approach */
"30240G1"  =  "1"  /* Transfusion of Nonautologous Bone Marrow into Central Vein, Open Approach */
"30240X0"  =  "1"  /* Transfusion of Autologous Cord Blood Stem Cells into Central Vein, Open Approach */
"30240X1"  =  "1"  /* Transfusion of Nonautologous Cord Blood Stem Cells into Central Vein, Open Approach */
"30240Y0"  =  "1"  /* Transfusion of Autologous Hematopoietic Stem Cells into Central Vein, Open Approach */
"30240Y1"  =  "1"  /* Transfusion of Nonautologous Hematopoietic Stem Cells into Central Vein, Open Approach */
"30243AZ"  =  "1"  /* Transfusion of Embryonic Stem Cells into Central Vein, Percutaneous Approach */
"30243G0"  =  "1"  /* Transfusion of Autologous Bone Marrow into Central Vein, Percutaneous Approach */
"30243G1"  =  "1"  /* Transfusion of Nonautologous Bone Marrow into Central Vein, Percutaneous Approach */
"30243X0"  =  "1"  /* Transfusion of Autologous Cord Blood Stem Cells into Central Vein, Percutaneous Approach */
"30243X1"  =  "1"  /* Transfusion of Nonautologous Cord Blood Stem Cells into Central Vein, Percutaneous Approach */
"30243Y0"  =  "1"  /* Transfusion of Autologous Hematopoietic Stem Cells into Central Vein, Percutaneous Approach */
"30243Y1"  =  "1"  /* Transfusion of Nonautologous Hematopoietic Stem Cells into Central Vein, Percutaneous Approach */
"30250G0"  =  "1"  /* Transfusion of Autologous Bone Marrow into Peripheral Artery, Open Approach */
"30250G1"  =  "1"  /* Transfusion of Nonautologous Bone Marrow into Peripheral Artery, Open Approach */
"30250X0"  =  "1"  /* Transfusion of Autologous Cord Blood Stem Cells into Peripheral Artery, Open Approach */
"30250X1"  =  "1"  /* Transfusion of Nonautologous Cord Blood Stem Cells into Peripheral Artery, Open Approach */
"30250Y0"  =  "1"  /* Transfusion of Autologous Hematopoietic Stem Cells into Peripheral Artery, Open Approach */
"30250Y1"  =  "1"  /* Transfusion of Nonautologous Hematopoietic Stem Cells into Peripheral Artery, Open Approach */
"30253G0"  =  "1"  /* Transfusion of Autologous Bone Marrow into Peripheral Artery, Percutaneous Approach */
"30253G1"  =  "1"  /* Transfusion of Nonautologous Bone Marrow into Peripheral Artery, Percutaneous Approach */
"30253X0"  =  "1"  /* Transfusion of Autologous Cord Blood Stem Cells into Peripheral Artery, Percutaneous Approach */
"30253X1"  =  "1"  /* Transfusion of Nonautologous Cord Blood Stem Cells into Peripheral Artery, Percutaneous Approach */
"30253Y0"  =  "1"  /* Transfusion of Autologous Hematopoietic Stem Cells into Peripheral Artery, Percutaneous Approach */
"30253Y1"  =  "1"  /* Transfusion of Nonautologous Hematopoietic Stem Cells into Peripheral Artery, Percutaneous Approach */
"30260G0"  =  "1"  /* Transfusion of Autologous Bone Marrow into Central Artery, Open Approach */
"30260G1"  =  "1"  /* Transfusion of Nonautologous Bone Marrow into Central Artery, Open Approach */
"30260X0"  =  "1"  /* Transfusion of Autologous Cord Blood Stem Cells into Central Artery, Open Approach */
"30260X1"  =  "1"  /* Transfusion of Nonautologous Cord Blood Stem Cells into Central Artery, Open Approach */
"30260Y0"  =  "1"  /* Transfusion of Autologous Hematopoietic Stem Cells into Central Artery, Open Approach */
"30260Y1"  =  "1"  /* Transfusion of Nonautologous Hematopoietic Stem Cells into Central Artery, Open Approach */
"30263G0"  =  "1"  /* Transfusion of Autologous Bone Marrow into Central Artery, Percutaneous Approach */
"30263G1"  =  "1"  /* Transfusion of Nonautologous Bone Marrow into Central Artery, Percutaneous Approach */
"30263X0"  =  "1"  /* Transfusion of Autologous Cord Blood Stem Cells into Central Artery, Percutaneous Approach */
"30263X1"  =  "1"  /* Transfusion of Nonautologous Cord Blood Stem Cells into Central Artery, Percutaneous Approach */
"30263Y0"  =  "1"  /* Transfusion of Autologous Hematopoietic Stem Cells into Central Artery, Percutaneous Approach */
"30263Y1"  =  "1"  /* Transfusion of Nonautologous Hematopoietic Stem Cells into Central Artery, Percutaneous Approach */
"3E030U1"  =  "1"  /* Introduction of Nonautologous Pancreatic Islet Cells into Peripheral Vein, Open Approach */
"3E033U1"  =  "1"  /* Introduction of Nonautologous Pancreatic Islet Cells into Peripheral Vein, Percutaneous Approach */
"3E0J3U1"  =  "1"  /* Introduction of Nonautologous Pancreatic Islet Cells into Biliary and Pancreatic Tract, Percutaneous Approach */
"3E0J7U1"  =  "1"  /* Introduction of Nonautologous Pancreatic Islet Cells into Biliary and Pancreatic Tract, Via Natural or Artificial Opening */
"3E0J8U1"  =  "1"  /* Introduction of Nonautologous Pancreatic Islet Cells into Biliary and Pancreatic Tract, Via Natural or Artificial Opening Endoscopic */
OTHER  = "0" ;

/* ABDOMINAL WALL CLOSURE */
VALUE $ABWALLC
"T8132XA" = "1" 
OTHER = "0" ;

VALUE $UMBHERN
"Q792" = "1"
"Q793" = "1"
OTHER = "0" ;


/* EXCLUDE: */
/* INTERMEDIATE RISK GROUP */
/* ICD-10-CM DIAGNOSIS CODES: */

VALUE $IMMUITD
"B180"  =  "1"  /* Chronic viral hepatitis B with delta-agent */
"B181"  =  "1"  /* Chronic viral hepatitis B without delta-agent */
"B182"  =  "1"  /* Chronic viral hepatitis C */
"B520"  =  "1"  /* Plasmodium malariae malaria with nephropathy */
"D5702"  =  "1"  /* Hb-SS disease with splenic sequestration */
"D57212"  =  "1"  /* Sickle-cell/Hb-C disease with splenic sequestration */
"D57412"  =  "1"  /* Sickle-cell thalassemia with splenic sequestration */
"D57812"  =  "1"  /* Other sickle-cell disorders with splenic sequestration */
"D730"  =  "1"  /* Hyposplenism */
"D731"  =  "1"  /* Hypersplenism */
"D732"  =  "1"  /* Chronic congestive splenomegaly */
"D733"  =  "1"  /* Abscess of spleen */
"D735"  =  "1"  /* Infarction of spleen */
"D7389"  =  "1"  /* Other diseases of spleen */
"D739"  =  "1"  /* Disease of spleen, unspecified */
"I8500"  =  "1"  /* Esophageal varices without bleeding */
"I8501"  =  "1"  /* Esophageal varices with bleeding */
"I8510"  =  "1"  /* Secondary esophageal varices without bleeding */
"I8511"  =  "1"  /* Secondary esophageal varices with bleeding */
"K7210"  =  "1"  /* Chronic hepatic failure without coma */
"K7211"  =  "1"  /* Chronic hepatic failure with coma */
"K7290"  =  "1"  /* Hepatic failure, unspecified without coma */
"K7291"  =  "1"  /* Hepatic failure, unspecified with coma */
"K766"  =  "1"  /* Portal hypertension */
"K7681"  =  "1"  /* Hepatopulmonary syndrome */
"M320"  =  "1"  /* Drug-induced systemic lupus erythematosus */
"M3210"  =  "1"  /* Systemic lupus erythematosus, organ or system involvement unspecified */
"M3211"  =  "1"  /* Endocarditis in systemic lupus erythematosus */
"M3212"  =  "1"  /* Pericarditis in systemic lupus erythematosus */
"M3213"  =  "1"  /* Lung involvement in systemic lupus erythematosus */
"M3214"  =  "1"  /* Glomerular disease in systemic lupus erythematosus */
"M3215"  =  "1"  /* Tubulo-interstitial nephropathy in systemic lupus erythematosus */
"M3219"  =  "1"  /* Other organ or system involvement in systemic lupus erythematosus */
"M328"  =  "1"  /* Other forms of systemic lupus erythematosus */
"M329"  =  "1"  /* Systemic lupus erythematosus, unspecified */
"M3300"  =  "1"  /* Juvenile dermatopolymyositis, organ involvement unspecified */
"M3301"  =  "1"  /* Juvenile dermatopolymyositis with respiratory involvement */
"M3302"  =  "1"  /* Juvenile dermatopolymyositis with myopathy */
"M3309"  =  "1"  /* Juvenile dermatopolymyositis with other organ involvement */
"M3310"  =  "1"  /* Other dermatopolymyositis, organ involvement unspecified */
"M3311"  =  "1"  /* Other dermatopolymyositis with respiratory involvement */
"M3312"  =  "1"  /* Other dermatopolymyositis with myopathy */
"M3319"  =  "1"  /* Other dermatopolymyositis with other organ involvement */
"M3320"  =  "1"  /* Polymyositis, organ involvement unspecified */
"M3321"  =  "1"  /* Polymyositis with respiratory involvement */
"M3322"  =  "1"  /* Polymyositis with myopathy */
"M3329"  =  "1"  /* Polymyositis with other organ involvement */
"M3390"  =  "1"  /* Dermatopolymyositis, unspecified, organ involvement unspecified */
"M3391"  =  "1"  /* Dermatopolymyositis, unspecified with respiratory involvement */
"M3392"  =  "1"  /* Dermatopolymyositis, unspecified with myopathy */
"M3399"  =  "1"  /* Dermatopolymyositis, unspecified with other organ involvement */
"M340"  =  "1"  /* Progressive systemic sclerosis */
"M341"  =  "1"  /* CR(E)ST syndrome */
"M342"  =  "1"  /* Systemic sclerosis induced by drug and chemical */
"M3481"  =  "1"  /* Systemic sclerosis with lung involvement */
"M3482"  =  "1"  /* Systemic sclerosis with myopathy */
"M3483"  =  "1"  /* Systemic sclerosis with polyneuropathy */
"M3489"  =  "1"  /* Other systemic sclerosis */
"M349"  =  "1"  /* Systemic sclerosis, unspecified */
"M3500"  =  "1"  /* Sicca syndrome, unspecified */
"M3501"  =  "1"  /* Sicca syndrome with keratoconjunctivitis */
"M3502"  =  "1"  /* Sicca syndrome with lung involvement */
"M3503"  =  "1"  /* Sicca syndrome with myopathy */
"M3504"  =  "1"  /* Sicca syndrome with tubulo-interstitial nephropathy */
"M3509"  =  "1"  /* Sicca syndrome with other organ involvement */
"M351"  =  "1"  /* Other overlap syndromes */
"M355"  =  "1"  /* Multifocal fibrosclerosis */
"M358"  =  "1"  /* Other specified systemic involvement of connective tissue */
"M359"  =  "1"  /* Systemic involvement of connective tissue, unspecified */
"M360"  =  "1"  /* Dermato(poly)myositis in neoplastic disease */
"M368"  =  "1"  /* Systemic disorders of connective tissue in other diseases classified elsewhere */
"N000"  =  "1"  /* Acute nephritic syndrome with minor glomerular abnormality */
"N001"  =  "1"  /* Acute nephritic syndrome with focal and segmental glomerular lesions */
"N002"  =  "1"  /* Acute nephritic syndrome with diffuse membranous glomerulonephritis */
"N003"  =  "1"  /* Acute nephritic syndrome with diffuse mesangial proliferative glomerulonephritis */
"N004"  =  "1"  /* Acute nephritic syndrome with diffuse endocapillary proliferative glomerulonephritis */
"N005"  =  "1"  /* Acute nephritic syndrome with diffuse mesangiocapillary glomerulonephritis */
"N006"  =  "1"  /* Acute nephritic syndrome with dense deposit disease */
"N007"  =  "1"  /* Acute nephritic syndrome with diffuse crescentic glomerulonephritis */
"N008"  =  "1"  /* Acute nephritic syndrome with other morphologic changes */
"N009"  =  "1"  /* Acute nephritic syndrome with unspecified morphologic changes */
"N010"  =  "1"  /* Rapidly progressive nephritic syndrome with minor glomerular abnormality */
"N011"  =  "1"  /* Rapidly progressive nephritic syndrome with focal and segmental glomerular lesions */
"N012"  =  "1"  /* Rapidly progressive nephritic syndrome with diffuse membranous glomerulonephritis */
"N013"  =  "1"  /* Rapidly progressive nephritic syndrome with diffuse mesangial proliferative glomerulonephritis */
"N014"  =  "1"  /* Rapidly progressive nephritic syndrome with diffuse endocapillary proliferative glomerulonephritis */
"N015"  =  "1"  /* Rapidly progressive nephritic syndrome with diffuse mesangiocapillary glomerulonephritis */
"N016"  =  "1"  /* Rapidly progressive nephritic syndrome with dense deposit disease */
"N017"  =  "1"  /* Rapidly progressive nephritic syndrome with diffuse crescentic glomerulonephritis */
"N018"  =  "1"  /* Rapidly progressive nephritic syndrome with other morphologic changes */
"N019"  =  "1"  /* Rapidly progressive nephritic syndrome with unspecified morphologic changes */
"N020"  =  "1"  /* Recurrent and persistent hematuria with minor glomerular abnormality */
"N021"  =  "1"  /* Recurrent and persistent hematuria with focal and segmental glomerular lesions */
"N022"  =  "1"  /* Recurrent and persistent hematuria with diffuse membranous glomerulonephritis */
"N023"  =  "1"  /* Recurrent and persistent hematuria with diffuse mesangial proliferative glomerulonephritis */
"N024"  =  "1"  /* Recurrent and persistent hematuria with diffuse endocapillary proliferative glomerulonephritis */
"N025"  =  "1"  /* Recurrent and persistent hematuria with diffuse mesangiocapillary glomerulonephritis */
"N026"  =  "1"  /* Recurrent and persistent hematuria with dense deposit disease */
"N027"  =  "1"  /* Recurrent and persistent hematuria with diffuse crescentic glomerulonephritis */
"N028"  =  "1"  /* Recurrent and persistent hematuria with other morphologic changes */
"N029"  =  "1"  /* Recurrent and persistent hematuria with unspecified morphologic changes */
"N030"  =  "1"  /* Chronic nephritic syndrome with minor glomerular abnormality */
"N031"  =  "1"  /* Chronic nephritic syndrome with focal and segmental glomerular lesions */
"N032"  =  "1"  /* Chronic nephritic syndrome with diffuse membranous glomerulonephritis */
"N033"  =  "1"  /* Chronic nephritic syndrome with diffuse mesangial proliferative glomerulonephritis */
"N034"  =  "1"  /* Chronic nephritic syndrome with diffuse endocapillary proliferative glomerulonephritis */
"N035"  =  "1"  /* Chronic nephritic syndrome with diffuse mesangiocapillary glomerulonephritis */
"N036"  =  "1"  /* Chronic nephritic syndrome with dense deposit disease */
"N037"  =  "1"  /* Chronic nephritic syndrome with diffuse crescentic glomerulonephritis */
"N038"  =  "1"  /* Chronic nephritic syndrome with other morphologic changes */
"N039"  =  "1"  /* Chronic nephritic syndrome with unspecified morphologic changes */
"N040"  =  "1"  /* Nephrotic syndrome with minor glomerular abnormality */
"N041"  =  "1"  /* Nephrotic syndrome with focal and segmental glomerular lesions */
"N042"  =  "1"  /* Nephrotic syndrome with diffuse membranous glomerulonephritis */
"N043"  =  "1"  /* Nephrotic syndrome with diffuse mesangial proliferative glomerulonephritis */
"N044"  =  "1"  /* Nephrotic syndrome with diffuse endocapillary proliferative glomerulonephritis */
"N045"  =  "1"  /* Nephrotic syndrome with diffuse mesangiocapillary glomerulonephritis */
"N046"  =  "1"  /* Nephrotic syndrome with dense deposit disease */
"N047"  =  "1"  /* Nephrotic syndrome with diffuse crescentic glomerulonephritis */
"N048"  =  "1"  /* Nephrotic syndrome with other morphologic changes */
"N049"  =  "1"  /* Nephrotic syndrome with unspecified morphologic changes */
"N050"  =  "1"  /* Unspecified nephritic syndrome with minor glomerular abnormality */
"N051"  =  "1"  /* Unspecified nephritic syndrome with focal and segmental glomerular lesions */
"N052"  =  "1"  /* Unspecified nephritic syndrome with diffuse membranous glomerulonephritis */
"N053"  =  "1"  /* Unspecified nephritic syndrome with diffuse mesangial proliferative glomerulonephritis */
"N054"  =  "1"  /* Unspecified nephritic syndrome with diffuse endocapillary proliferative glomerulonephritis */
"N055"  =  "1"  /* Unspecified nephritic syndrome with diffuse mesangiocapillary glomerulonephritis */
"N056"  =  "1"  /* Unspecified nephritic syndrome with dense deposit disease */
"N057"  =  "1"  /* Unspecified nephritic syndrome with diffuse crescentic glomerulonephritis */
"N058"  =  "1"  /* Unspecified nephritic syndrome with other morphologic changes */
"N059"  =  "1"  /* Unspecified nephritic syndrome with unspecified morphologic changes */
"N060"  =  "1"  /* Isolated proteinuria with minor glomerular abnormality */
"N061"  =  "1"  /* Isolated proteinuria with focal and segmental glomerular lesions */
"N062"  =  "1"  /* Isolated proteinuria with diffuse membranous glomerulonephritis */
"N063"  =  "1"  /* Isolated proteinuria with diffuse mesangial proliferative glomerulonephritis */
"N064"  =  "1"  /* Isolated proteinuria with diffuse endocapillary proliferative glomerulonephritis */
"N065"  =  "1"  /* Isolated proteinuria with diffuse mesangiocapillary glomerulonephritis */
"N066"  =  "1"  /* Isolated proteinuria with dense deposit disease */
"N067"  =  "1"  /* Isolated proteinuria with diffuse crescentic glomerulonephritis */
"N068"  =  "1"  /* Isolated proteinuria with other morphologic lesion */
"N069"  =  "1"  /* Isolated proteinuria with unspecified morphologic lesion */
"N070"  =  "1"  /* Hereditary nephropathy, not elsewhere classified with minor glomerular abnormality */
"N071"  =  "1"  /* Hereditary nephropathy, not elsewhere classified with focal and segmental glomerular lesions */
"N072"  =  "1"  /* Hereditary nephropathy, not elsewhere classified with diffuse membranous glomerulonephritis */
"N073"  =  "1"  /* Hereditary nephropathy, not elsewhere classified with diffuse mesangial proliferative glomerulonephritis */
"N074"  =  "1"  /* Hereditary nephropathy, not elsewhere classified with diffuse endocapillary proliferative glomerulonephritis */
"N075"  =  "1"  /* Hereditary nephropathy, not elsewhere classified with diffuse mesangiocapillary glomerulonephritis */
"N076"  =  "1"  /* Hereditary nephropathy, not elsewhere classified with dense deposit disease */
"N077"  =  "1"  /* Hereditary nephropathy, not elsewhere classified with diffuse crescentic glomerulonephritis */
"N078"  =  "1"  /* Hereditary nephropathy, not elsewhere classified with other morphologic lesions */
"N079"  =  "1"  /* Hereditary nephropathy, not elsewhere classified with unspecified morphologic lesions */
"N140"  =  "1"  /* Analgesic nephropathy */
"N141"  =  "1"  /* Nephropathy induced by other drugs, medicaments and biological substances */
"N142"  =  "1"  /* Nephropathy induced by unspecified drug, medicament or biological substance */
"N143"  =  "1"  /* Nephropathy induced by heavy metals */
"N144"  =  "1"  /* Toxic nephropathy, not elsewhere classified */
"N150"  =  "1"  /* Balkan nephropathy */
"N158"  =  "1"  /* Other specified renal tubulo-interstitial diseases */
"N159"  =  "1"  /* Renal tubulo-interstitial disease, unspecified */
"N16"  =  "1"  /* Renal tubulo-interstitial disorders in diseases classified elsewhere */
"Q8901"  =  "1"  /* Asplenia (congenital) */
"Q8909"  =  "1"  /* Congenital malformations of spleen */
"R64"  =  "1"  /* Cachexia */
"S3600XA"  =  "1"  /* Unspecified injury of spleen, initial encounter */
"S36020A"  =  "1"  /* Minor contusion of spleen, initial encounter */
"S36021A"  =  "1"  /* Major contusion of spleen, initial encounter */
"S36029A"  =  "1"  /* Unspecified contusion of spleen, initial encounter */
"S36030A"  =  "1"  /* Superficial (capsular) laceration of spleen, initial encounter */
"S36031A"  =  "1"  /* Moderate laceration of spleen, initial encounter */
"S36032A"  =  "1"  /* Major laceration of spleen, initial encounter */
"S36039A"  =  "1"  /* Unspecified laceration of spleen, initial encounter */
"S3609XA"  =  "1"  /* Other injury of spleen, initial encounter */
"Z4823"  =  "1"  /* Encounter for aftercare following liver transplant */
"Z944"  =  "1"  /* Liver transplant status */
OTHER   = "0" ;

/* HEPTIC FAILURE CONDITION PART II */
VALUE $HEPFA2D
"K702"  =  "1"  /* Alcoholic fibrosis and sclerosis of liver */
"K7030"  =  "1"  /* Alcoholic cirrhosis of liver without ascites */
"K7031"  =  "1"  /* Alcoholic cirrhosis of liver with ascites */
"K740"  =  "1"  /* Hepatic fibrosis */
"K743"  =  "1"  /* Primary biliary cirrhosis */
"K744"  =  "1"  /* Secondary biliary cirrhosis */
"K745"  =  "1"  /* Biliary cirrhosis, unspecified */
"K7460"  =  "1"  /* Unspecified cirrhosis of liver */
"K7469"  =  "1"  /* Other cirrhosis of liver */
OTHER   = "0" ;

VALUE $HEPFA3D
"K7040"  =  "1"  /* Alcoholic hepatic failure without coma */
"K7041"  =  "1"  /* Alcoholic hepatic failure with coma */
"K7110"  =  "1"  /* Toxic liver disease with hepatic necrosis, without coma */
"K7111"  =  "1"  /* Toxic liver disease with hepatic necrosis, with coma */
"K7200"  =  "1"  /* Acute and subacute hepatic failure without coma */
"K7201"  =  "1"  /* Acute and subacute hepatic failure with coma */
"K7210"  =  "1"  /* Chronic hepatic failure without coma */
"K7211"  =  "1"  /* Chronic hepatic failure with coma */
"K7290"  =  "1"  /* Hepatic failure, unspecified without coma */
"K7291"  =  "1"  /* Hepatic failure, unspecified with coma */
"K767"  =  "1"  /* Hepatorenal syndrome */
OTHER   = "0" ;


 /*Central Venous Catheter-Related Blood Stream Infections (PSI 7, PSI 23 and PDI 12) */
 /* ---------------------------------------------------------*/
 /* CENTRAL VENOUS CATHETER-RELATED BLOOD STREAM INFECTIONS  */
 /*   PROVIDER INDICATOR / PDI#  12                          */
 /* ---------------------------------------------------------*/

 /* ICD-9-CM DIAGNOSIS CODES: HOSPITAL ASSOCIATED INFECTION DIAGNOSIS CODES FOR DISCHARGES BEFORE OCTOBER 1, 2007  */;
 /* VALUE $IDTMCID per UCD - Due to the granularity of ICD-10-CM code set, this concept was merged with other concepts to capture this infection $IDTMC3D */

 /* ICD-9-CM DIAGNOSIS CODES: BLOODSTREAM INFECTION INFECTION DIAGNOSIS CODES FOR DISCHARGES ON OR AFTER OCTOBER 1, 2007 BUT BEFORE OCTOBER 1, 2011 */;
 /* VALUE $IDTMC2D per UCD - Due to the granularity of ICD-10-CM code set, this concept was merged with other concepts to capture this infection $IDTMC3D */

 /* ICD-10-CM DIAGNOSIS CODES: CENTRAL LINE-ASSOCIATED BLOODSTREAM INFECTION DIAGNOSIS CODES FOR DISCHARGES ON OR AFTER OCTOBER 1, 2011  */;

VALUE $IDTMC3D            
"T80211A"  =  "1"  /* Bloodstream infection due to central venous catheter, initial encounter */
OTHER   = "0" ;

/* STRATIFY BY: HIGH RISK GROUP */
/* ICD-10-CM DIAGNOSIS CODES: */

VALUE $CANITD
"C000"  =  "1"  /* Malignant neoplasm of external upper lip */
"C001"  =  "1"  /* Malignant neoplasm of external lower lip */
"C002"  =  "1"  /* Malignant neoplasm of external lip, unspecified */
"C003"  =  "1"  /* Malignant neoplasm of upper lip, inner aspect */
"C004"  =  "1"  /* Malignant neoplasm of lower lip, inner aspect */
"C005"  =  "1"  /* Malignant neoplasm of lip, unspecified, inner aspect */
"C006"  =  "1"  /* Malignant neoplasm of commissure of lip, unspecified */
"C008"  =  "1"  /* Malignant neoplasm of overlapping sites of lip */
"C009"  =  "1"  /* Malignant neoplasm of lip, unspecified */
"C01"  =  "1"  /* Malignant neoplasm of base of tongue */
"C020"  =  "1"  /* Malignant neoplasm of dorsal surface of tongue */
"C021"  =  "1"  /* Malignant neoplasm of border of tongue */
"C022"  =  "1"  /* Malignant neoplasm of ventral surface of tongue */
"C023"  =  "1"  /* Malignant neoplasm of anterior two-thirds of tongue, part unspecified */
"C024"  =  "1"  /* Malignant neoplasm of lingual tonsil */
"C028"  =  "1"  /* Malignant neoplasm of overlapping sites of tongue */
"C029"  =  "1"  /* Malignant neoplasm of tongue, unspecified */
"C030"  =  "1"  /* Malignant neoplasm of upper gum */
"C031"  =  "1"  /* Malignant neoplasm of lower gum */
"C039"  =  "1"  /* Malignant neoplasm of gum, unspecified */
"C040"  =  "1"  /* Malignant neoplasm of anterior floor of mouth */
"C041"  =  "1"  /* Malignant neoplasm of lateral floor of mouth */
"C048"  =  "1"  /* Malignant neoplasm of overlapping sites of floor of mouth */
"C049"  =  "1"  /* Malignant neoplasm of floor of mouth, unspecified */
"C050"  =  "1"  /* Malignant neoplasm of hard palate */
"C051"  =  "1"  /* Malignant neoplasm of soft palate */
"C052"  =  "1"  /* Malignant neoplasm of uvula */
"C058"  =  "1"  /* Malignant neoplasm of overlapping sites of palate */
"C059"  =  "1"  /* Malignant neoplasm of palate, unspecified */
"C060"  =  "1"  /* Malignant neoplasm of cheek mucosa */
"C061"  =  "1"  /* Malignant neoplasm of vestibule of mouth */
"C062"  =  "1"  /* Malignant neoplasm of retromolar area */
"C0680"  =  "1"  /* Malignant neoplasm of overlapping sites of unspecified parts of mouth */
"C0689"  =  "1"  /* Malignant neoplasm of overlapping sites of other parts of mouth */
"C069"  =  "1"  /* Malignant neoplasm of mouth, unspecified */
"C07"  =  "1"  /* Malignant neoplasm of parotid gland */
"C080"  =  "1"  /* Malignant neoplasm of submandibular gland */
"C081"  =  "1"  /* Malignant neoplasm of sublingual gland */
"C089"  =  "1"  /* Malignant neoplasm of major salivary gland, unspecified */
"C090"  =  "1"  /* Malignant neoplasm of tonsillar fossa */
"C091"  =  "1"  /* Malignant neoplasm of tonsillar pillar (anterior) (posterior) */
"C098"  =  "1"  /* Malignant neoplasm of overlapping sites of tonsil */
"C099"  =  "1"  /* Malignant neoplasm of tonsil, unspecified */
"C100"  =  "1"  /* Malignant neoplasm of vallecula */
"C101"  =  "1"  /* Malignant neoplasm of anterior surface of epiglottis */
"C102"  =  "1"  /* Malignant neoplasm of lateral wall of oropharynx */
"C103"  =  "1"  /* Malignant neoplasm of posterior wall of oropharynx */
"C104"  =  "1"  /* Malignant neoplasm of branchial cleft */
"C108"  =  "1"  /* Malignant neoplasm of overlapping sites of oropharynx */
"C109"  =  "1"  /* Malignant neoplasm of oropharynx, unspecified */
"C110"  =  "1"  /* Malignant neoplasm of superior wall of nasopharynx */
"C111"  =  "1"  /* Malignant neoplasm of posterior wall of nasopharynx */
"C112"  =  "1"  /* Malignant neoplasm of lateral wall of nasopharynx */
"C113"  =  "1"  /* Malignant neoplasm of anterior wall of nasopharynx */
"C118"  =  "1"  /* Malignant neoplasm of overlapping sites of nasopharynx */
"C119"  =  "1"  /* Malignant neoplasm of nasopharynx, unspecified */
"C12"  =  "1"  /* Malignant neoplasm of pyriform sinus */
"C130"  =  "1"  /* Malignant neoplasm of postcricoid region */
"C131"  =  "1"  /* Malignant neoplasm of aryepiglottic fold, hypopharyngeal aspect */
"C132"  =  "1"  /* Malignant neoplasm of posterior wall of hypopharynx */
"C138"  =  "1"  /* Malignant neoplasm of overlapping sites of hypopharynx */
"C139"  =  "1"  /* Malignant neoplasm of hypopharynx, unspecified */
"C140"  =  "1"  /* Malignant neoplasm of pharynx, unspecified */
"C142"  =  "1"  /* Malignant neoplasm of Waldeyer's ring */
"C148"  =  "1"  /* Malignant neoplasm of overlapping sites of lip, oral cavity and pharynx */
"C153"  =  "1"  /* Malignant neoplasm of upper third of esophagus */
"C154"  =  "1"  /* Malignant neoplasm of middle third of esophagus */
"C155"  =  "1"  /* Malignant neoplasm of lower third of esophagus */
"C158"  =  "1"  /* Malignant neoplasm of overlapping sites of esophagus */
"C159"  =  "1"  /* Malignant neoplasm of esophagus, unspecified */
"C160"  =  "1"  /* Malignant neoplasm of cardia */
"C161"  =  "1"  /* Malignant neoplasm of fundus of stomach */
"C162"  =  "1"  /* Malignant neoplasm of body of stomach */
"C163"  =  "1"  /* Malignant neoplasm of pyloric antrum */
"C164"  =  "1"  /* Malignant neoplasm of pylorus */
"C165"  =  "1"  /* Malignant neoplasm of lesser curvature of stomach, unspecified */
"C166"  =  "1"  /* Malignant neoplasm of greater curvature of stomach, unspecified */
"C168"  =  "1"  /* Malignant neoplasm of overlapping sites of stomach */
"C169"  =  "1"  /* Malignant neoplasm of stomach, unspecified */
"C170"  =  "1"  /* Malignant neoplasm of duodenum */
"C171"  =  "1"  /* Malignant neoplasm of jejunum */
"C172"  =  "1"  /* Malignant neoplasm of ileum */
"C173"  =  "1"  /* Meckel's diverticulum, malignant */
"C178"  =  "1"  /* Malignant neoplasm of overlapping sites of small intestine */
"C179"  =  "1"  /* Malignant neoplasm of small intestine, unspecified */
"C180"  =  "1"  /* Malignant neoplasm of cecum */
"C181"  =  "1"  /* Malignant neoplasm of appendix */
"C182"  =  "1"  /* Malignant neoplasm of ascending colon */
"C183"  =  "1"  /* Malignant neoplasm of hepatic flexure */
"C184"  =  "1"  /* Malignant neoplasm of transverse colon */
"C185"  =  "1"  /* Malignant neoplasm of splenic flexure */
"C186"  =  "1"  /* Malignant neoplasm of descending colon */
"C187"  =  "1"  /* Malignant neoplasm of sigmoid colon */
"C188"  =  "1"  /* Malignant neoplasm of overlapping sites of colon */
"C189"  =  "1"  /* Malignant neoplasm of colon, unspecified */
"C19"  =  "1"  /* Malignant neoplasm of rectosigmoid junction */
"C20"  =  "1"  /* Malignant neoplasm of rectum */
"C210"  =  "1"  /* Malignant neoplasm of anus, unspecified */
"C211"  =  "1"  /* Malignant neoplasm of anal canal */
"C212"  =  "1"  /* Malignant neoplasm of cloacogenic zone */
"C218"  =  "1"  /* Malignant neoplasm of overlapping sites of rectum, anus and anal canal */
"C220"  =  "1"  /* Liver cell carcinoma */
"C221"  =  "1"  /* Intrahepatic bile duct carcinoma */
"C222"  =  "1"  /* Hepatoblastoma */
"C223"  =  "1"  /* Angiosarcoma of liver */
"C224"  =  "1"  /* Other sarcomas of liver */
"C227"  =  "1"  /* Other specified carcinomas of liver */
"C228"  =  "1"  /* Malignant neoplasm of liver, primary, unspecified as to type */
"C229"  =  "1"  /* Malignant neoplasm of liver, not specified as primary or secondary */
"C23"  =  "1"  /* Malignant neoplasm of gallbladder */
"C240"  =  "1"  /* Malignant neoplasm of extrahepatic bile duct */
"C241"  =  "1"  /* Malignant neoplasm of ampulla of Vater */
"C248"  =  "1"  /* Malignant neoplasm of overlapping sites of biliary tract */
"C249"  =  "1"  /* Malignant neoplasm of biliary tract, unspecified */
"C250"  =  "1"  /* Malignant neoplasm of head of pancreas */
"C251"  =  "1"  /* Malignant neoplasm of body of pancreas */
"C252"  =  "1"  /* Malignant neoplasm of tail of pancreas */
"C253"  =  "1"  /* Malignant neoplasm of pancreatic duct */
"C254"  =  "1"  /* Malignant neoplasm of endocrine pancreas */
"C257"  =  "1"  /* Malignant neoplasm of other parts of pancreas */
"C258"  =  "1"  /* Malignant neoplasm of overlapping sites of pancreas */
"C259"  =  "1"  /* Malignant neoplasm of pancreas, unspecified */
"C260"  =  "1"  /* Malignant neoplasm of intestinal tract, part unspecified */
"C261"  =  "1"  /* Malignant neoplasm of spleen */
"C269"  =  "1"  /* Malignant neoplasm of ill-defined sites within the digestive system */
"C300"  =  "1"  /* Malignant neoplasm of nasal cavity */
"C301"  =  "1"  /* Malignant neoplasm of middle ear */
"C310"  =  "1"  /* Malignant neoplasm of maxillary sinus */
"C311"  =  "1"  /* Malignant neoplasm of ethmoidal sinus */
"C312"  =  "1"  /* Malignant neoplasm of frontal sinus */
"C313"  =  "1"  /* Malignant neoplasm of sphenoid sinus */
"C318"  =  "1"  /* Malignant neoplasm of overlapping sites of accessory sinuses */
"C319"  =  "1"  /* Malignant neoplasm of accessory sinus, unspecified */
"C320"  =  "1"  /* Malignant neoplasm of glottis */
"C321"  =  "1"  /* Malignant neoplasm of supraglottis */
"C322"  =  "1"  /* Malignant neoplasm of subglottis */
"C323"  =  "1"  /* Malignant neoplasm of laryngeal cartilage */
"C328"  =  "1"  /* Malignant neoplasm of overlapping sites of larynx */
"C329"  =  "1"  /* Malignant neoplasm of larynx, unspecified */
"C33"  =  "1"  /* Malignant neoplasm of trachea */
"C3400"  =  "1"  /* Malignant neoplasm of unspecified main bronchus */
"C3401"  =  "1"  /* Malignant neoplasm of right main bronchus */
"C3402"  =  "1"  /* Malignant neoplasm of left main bronchus */
"C3410"  =  "1"  /* Malignant neoplasm of upper lobe, unspecified bronchus or lung */
"C3411"  =  "1"  /* Malignant neoplasm of upper lobe, right bronchus or lung */
"C3412"  =  "1"  /* Malignant neoplasm of upper lobe, left bronchus or lung */
"C342"  =  "1"  /* Malignant neoplasm of middle lobe, bronchus or lung */
"C3430"  =  "1"  /* Malignant neoplasm of lower lobe, unspecified bronchus or lung */
"C3431"  =  "1"  /* Malignant neoplasm of lower lobe, right bronchus or lung */
"C3432"  =  "1"  /* Malignant neoplasm of lower lobe, left bronchus or lung */
"C3480"  =  "1"  /* Malignant neoplasm of overlapping sites of unspecified bronchus and lung */
"C3481"  =  "1"  /* Malignant neoplasm of overlapping sites of right bronchus and lung */
"C3482"  =  "1"  /* Malignant neoplasm of overlapping sites of left bronchus and lung */
"C3490"  =  "1"  /* Malignant neoplasm of unspecified part of unspecified bronchus or lung */
"C3491"  =  "1"  /* Malignant neoplasm of unspecified part of right bronchus or lung */
"C3492"  =  "1"  /* Malignant neoplasm of unspecified part of left bronchus or lung */
"C37"  =  "1"  /* Malignant neoplasm of thymus */
"C380"  =  "1"  /* Malignant neoplasm of heart */
"C381"  =  "1"  /* Malignant neoplasm of anterior mediastinum */
"C382"  =  "1"  /* Malignant neoplasm of posterior mediastinum */
"C383"  =  "1"  /* Malignant neoplasm of mediastinum, part unspecified */
"C384"  =  "1"  /* Malignant neoplasm of pleura */
"C388"  =  "1"  /* Malignant neoplasm of overlapping sites of heart, mediastinum and pleura */
"C390"  =  "1"  /* Malignant neoplasm of upper respiratory tract, part unspecified */
"C399"  =  "1"  /* Malignant neoplasm of lower respiratory tract, part unspecified */
"C4000"  =  "1"  /* Malignant neoplasm of scapula and long bones of unspecified upper limb */
"C4001"  =  "1"  /* Malignant neoplasm of scapula and long bones of right upper limb */
"C4002"  =  "1"  /* Malignant neoplasm of scapula and long bones of left upper limb */
"C4010"  =  "1"  /* Malignant neoplasm of short bones of unspecified upper limb */
"C4011"  =  "1"  /* Malignant neoplasm of short bones of right upper limb */
"C4012"  =  "1"  /* Malignant neoplasm of short bones of left upper limb */
"C4020"  =  "1"  /* Malignant neoplasm of long bones of unspecified lower limb */
"C4021"  =  "1"  /* Malignant neoplasm of long bones of right lower limb */
"C4022"  =  "1"  /* Malignant neoplasm of long bones of left lower limb */
"C4030"  =  "1"  /* Malignant neoplasm of short bones of unspecified lower limb */
"C4031"  =  "1"  /* Malignant neoplasm of short bones of right lower limb */
"C4032"  =  "1"  /* Malignant neoplasm of short bones of left lower limb */
"C4080"  =  "1"  /* Malignant neoplasm of overlapping sites of bone and articular cartilage of unspecified limb */
"C4081"  =  "1"  /* Malignant neoplasm of overlapping sites of bone and articular cartilage of right limb */
"C4082"  =  "1"  /* Malignant neoplasm of overlapping sites of bone and articular cartilage of left limb */
"C4090"  =  "1"  /* Malignant neoplasm of unspecified bones and articular cartilage of unspecified limb */
"C4091"  =  "1"  /* Malignant neoplasm of unspecified bones and articular cartilage of right limb */
"C4092"  =  "1"  /* Malignant neoplasm of unspecified bones and articular cartilage of left limb */
"C410"  =  "1"  /* Malignant neoplasm of bones of skull and face */
"C411"  =  "1"  /* Malignant neoplasm of mandible */
"C412"  =  "1"  /* Malignant neoplasm of vertebral column */
"C413"  =  "1"  /* Malignant neoplasm of ribs, sternum and clavicle */
"C414"  =  "1"  /* Malignant neoplasm of pelvic bones, sacrum and coccyx */
"C419"  =  "1"  /* Malignant neoplasm of bone and articular cartilage, unspecified */
"C430"  =  "1"  /* Malignant melanoma of lip */
"C4310"  =  "1"  /* Malignant melanoma of unspecified eyelid, including canthus */
"C4311"  =  "1"  /* Malignant melanoma of right eyelid, including canthus */
"C4312"  =  "1"  /* Malignant melanoma of left eyelid, including canthus */
"C4320"  =  "1"  /* Malignant melanoma of unspecified ear and external auricular canal */
"C4321"  =  "1"  /* Malignant melanoma of right ear and external auricular canal */
"C4322"  =  "1"  /* Malignant melanoma of left ear and external auricular canal */
"C4330"  =  "1"  /* Malignant melanoma of unspecified part of face */
"C4331"  =  "1"  /* Malignant melanoma of nose */
"C4339"  =  "1"  /* Malignant melanoma of other parts of face */
"C434"  =  "1"  /* Malignant melanoma of scalp and neck */
"C4351"  =  "1"  /* Malignant melanoma of anal skin */
"C4352"  =  "1"  /* Malignant melanoma of skin of breast */
"C4359"  =  "1"  /* Malignant melanoma of other part of trunk */
"C4360"  =  "1"  /* Malignant melanoma of unspecified upper limb, including shoulder */
"C4361"  =  "1"  /* Malignant melanoma of right upper limb, including shoulder */
"C4362"  =  "1"  /* Malignant melanoma of left upper limb, including shoulder */
"C4370"  =  "1"  /* Malignant melanoma of unspecified lower limb, including hip */
"C4371"  =  "1"  /* Malignant melanoma of right lower limb, including hip */
"C4372"  =  "1"  /* Malignant melanoma of left lower limb, including hip */
"C438"  =  "1"  /* Malignant melanoma of overlapping sites of skin */
"C439"  =  "1"  /* Malignant melanoma of skin, unspecified */
"C450"  =  "1"  /* Mesothelioma of pleura */
"C451"  =  "1"  /* Mesothelioma of peritoneum */
"C452"  =  "1"  /* Mesothelioma of pericardium */
"C457"  =  "1"  /* Mesothelioma of other sites */
"C459"  =  "1"  /* Mesothelioma, unspecified */
"C460"  =  "1"  /* Kaposi's sarcoma of skin */
"C461"  =  "1"  /* Kaposi's sarcoma of soft tissue */
"C462"  =  "1"  /* Kaposi's sarcoma of palate */
"C463"  =  "1"  /* Kaposi's sarcoma of lymph nodes */
"C464"  =  "1"  /* Kaposi's sarcoma of gastrointestinal sites */
"C4650"  =  "1"  /* Kaposi's sarcoma of unspecified lung */
"C4651"  =  "1"  /* Kaposi's sarcoma of right lung */
"C4652"  =  "1"  /* Kaposi's sarcoma of left lung */
"C467"  =  "1"  /* Kaposi's sarcoma of other sites */
"C469"  =  "1"  /* Kaposi's sarcoma, unspecified */
"C470"  =  "1"  /* Malignant neoplasm of peripheral nerves of head, face and neck */
"C4710"  =  "1"  /* Malignant neoplasm of peripheral nerves of unspecified upper limb, including shoulder */
"C4711"  =  "1"  /* Malignant neoplasm of peripheral nerves of right upper limb, including shoulder */
"C4712"  =  "1"  /* Malignant neoplasm of peripheral nerves of left upper limb, including shoulder */
"C4720"  =  "1"  /* Malignant neoplasm of peripheral nerves of unspecified lower limb, including hip */
"C4721"  =  "1"  /* Malignant neoplasm of peripheral nerves of right lower limb, including hip */
"C4722"  =  "1"  /* Malignant neoplasm of peripheral nerves of left lower limb, including hip */
"C473"  =  "1"  /* Malignant neoplasm of peripheral nerves of thorax */
"C474"  =  "1"  /* Malignant neoplasm of peripheral nerves of abdomen */
"C475"  =  "1"  /* Malignant neoplasm of peripheral nerves of pelvis */
"C476"  =  "1"  /* Malignant neoplasm of peripheral nerves of trunk, unspecified */
"C478"  =  "1"  /* Malignant neoplasm of overlapping sites of peripheral nerves and autonomic nervous system */
"C479"  =  "1"  /* Malignant neoplasm of peripheral nerves and autonomic nervous system, unspecified */
"C480"  =  "1"  /* Malignant neoplasm of retroperitoneum */
"C481"  =  "1"  /* Malignant neoplasm of specified parts of peritoneum */
"C482"  =  "1"  /* Malignant neoplasm of peritoneum, unspecified */
"C488"  =  "1"  /* Malignant neoplasm of overlapping sites of retroperitoneum and peritoneum */
"C490"  =  "1"  /* Malignant neoplasm of connective and soft tissue of head, face and neck */
"C4910"  =  "1"  /* Malignant neoplasm of connective and soft tissue of unspecified upper limb, including shoulder */
"C4911"  =  "1"  /* Malignant neoplasm of connective and soft tissue of right upper limb, including shoulder */
"C4912"  =  "1"  /* Malignant neoplasm of connective and soft tissue of left upper limb, including shoulder */
"C4920"  =  "1"  /* Malignant neoplasm of connective and soft tissue of unspecified lower limb, including hip */
"C4921"  =  "1"  /* Malignant neoplasm of connective and soft tissue of right lower limb, including hip */
"C4922"  =  "1"  /* Malignant neoplasm of connective and soft tissue of left lower limb, including hip */
"C493"  =  "1"  /* Malignant neoplasm of connective and soft tissue of thorax */
"C494"  =  "1"  /* Malignant neoplasm of connective and soft tissue of abdomen */
"C495"  =  "1"  /* Malignant neoplasm of connective and soft tissue of pelvis */
"C496"  =  "1"  /* Malignant neoplasm of connective and soft tissue of trunk, unspecified */
"C498"  =  "1"  /* Malignant neoplasm of overlapping sites of connective and soft tissue */
"C499"  =  "1"  /* Malignant neoplasm of connective and soft tissue, unspecified */
"C50011"  =  "1"  /* Malignant neoplasm of nipple and areola, right female breast */
"C50012"  =  "1"  /* Malignant neoplasm of nipple and areola, left female breast */
"C50019"  =  "1"  /* Malignant neoplasm of nipple and areola, unspecified female breast */
"C50021"  =  "1"  /* Malignant neoplasm of nipple and areola, right male breast */
"C50022"  =  "1"  /* Malignant neoplasm of nipple and areola, left male breast */
"C50029"  =  "1"  /* Malignant neoplasm of nipple and areola, unspecified male breast */
"C50111"  =  "1"  /* Malignant neoplasm of central portion of right female breast */
"C50112"  =  "1"  /* Malignant neoplasm of central portion of left female breast */
"C50119"  =  "1"  /* Malignant neoplasm of central portion of unspecified female breast */
"C50121"  =  "1"  /* Malignant neoplasm of central portion of right male breast */
"C50122"  =  "1"  /* Malignant neoplasm of central portion of left male breast */
"C50129"  =  "1"  /* Malignant neoplasm of central portion of unspecified male breast */
"C50211"  =  "1"  /* Malignant neoplasm of upper-inner quadrant of right female breast */
"C50212"  =  "1"  /* Malignant neoplasm of upper-inner quadrant of left female breast */
"C50219"  =  "1"  /* Malignant neoplasm of upper-inner quadrant of unspecified female breast */
"C50221"  =  "1"  /* Malignant neoplasm of upper-inner quadrant of right male breast */
"C50222"  =  "1"  /* Malignant neoplasm of upper-inner quadrant of left male breast */
"C50229"  =  "1"  /* Malignant neoplasm of upper-inner quadrant of unspecified male breast */
"C50311"  =  "1"  /* Malignant neoplasm of lower-inner quadrant of right female breast */
"C50312"  =  "1"  /* Malignant neoplasm of lower-inner quadrant of left female breast */
"C50319"  =  "1"  /* Malignant neoplasm of lower-inner quadrant of unspecified female breast */
"C50321"  =  "1"  /* Malignant neoplasm of lower-inner quadrant of right male breast */
"C50322"  =  "1"  /* Malignant neoplasm of lower-inner quadrant of left male breast */
"C50329"  =  "1"  /* Malignant neoplasm of lower-inner quadrant of unspecified male breast */
"C50411"  =  "1"  /* Malignant neoplasm of upper-outer quadrant of right female breast */
"C50412"  =  "1"  /* Malignant neoplasm of upper-outer quadrant of left female breast */
"C50419"  =  "1"  /* Malignant neoplasm of upper-outer quadrant of unspecified female breast */
"C50421"  =  "1"  /* Malignant neoplasm of upper-outer quadrant of right male breast */
"C50422"  =  "1"  /* Malignant neoplasm of upper-outer quadrant of left male breast */
"C50429"  =  "1"  /* Malignant neoplasm of upper-outer quadrant of unspecified male breast */
"C50511"  =  "1"  /* Malignant neoplasm of lower-outer quadrant of right female breast */
"C50512"  =  "1"  /* Malignant neoplasm of lower-outer quadrant of left female breast */
"C50519"  =  "1"  /* Malignant neoplasm of lower-outer quadrant of unspecified female breast */
"C50521"  =  "1"  /* Malignant neoplasm of lower-outer quadrant of right male breast */
"C50522"  =  "1"  /* Malignant neoplasm of lower-outer quadrant of left male breast */
"C50529"  =  "1"  /* Malignant neoplasm of lower-outer quadrant of unspecified male breast */
"C50611"  =  "1"  /* Malignant neoplasm of axillary tail of right female breast */
"C50612"  =  "1"  /* Malignant neoplasm of axillary tail of left female breast */
"C50619"  =  "1"  /* Malignant neoplasm of axillary tail of unspecified female breast */
"C50621"  =  "1"  /* Malignant neoplasm of axillary tail of right male breast */
"C50622"  =  "1"  /* Malignant neoplasm of axillary tail of left male breast */
"C50629"  =  "1"  /* Malignant neoplasm of axillary tail of unspecified male breast */
"C50811"  =  "1"  /* Malignant neoplasm of overlapping sites of right female breast */
"C50812"  =  "1"  /* Malignant neoplasm of overlapping sites of left female breast */
"C50819"  =  "1"  /* Malignant neoplasm of overlapping sites of unspecified female breast */
"C50821"  =  "1"  /* Malignant neoplasm of overlapping sites of right male breast */
"C50822"  =  "1"  /* Malignant neoplasm of overlapping sites of left male breast */
"C50829"  =  "1"  /* Malignant neoplasm of overlapping sites of unspecified male breast */
"C50911"  =  "1"  /* Malignant neoplasm of unspecified site of right female breast */
"C50912"  =  "1"  /* Malignant neoplasm of unspecified site of left female breast */
"C50919"  =  "1"  /* Malignant neoplasm of unspecified site of unspecified female breast */
"C50921"  =  "1"  /* Malignant neoplasm of unspecified site of right male breast */
"C50922"  =  "1"  /* Malignant neoplasm of unspecified site of left male breast */
"C50929"  =  "1"  /* Malignant neoplasm of unspecified site of unspecified male breast */
"C510"  =  "1"  /* Malignant neoplasm of labium majus */
"C511"  =  "1"  /* Malignant neoplasm of labium minus */
"C512"  =  "1"  /* Malignant neoplasm of clitoris */
"C518"  =  "1"  /* Malignant neoplasm of overlapping sites of vulva */
"C519"  =  "1"  /* Malignant neoplasm of vulva, unspecified */
"C52"  =  "1"  /* Malignant neoplasm of vagina */
"C530"  =  "1"  /* Malignant neoplasm of endocervix */
"C531"  =  "1"  /* Malignant neoplasm of exocervix */
"C538"  =  "1"  /* Malignant neoplasm of overlapping sites of cervix uteri */
"C539"  =  "1"  /* Malignant neoplasm of cervix uteri, unspecified */
"C540"  =  "1"  /* Malignant neoplasm of isthmus uteri */
"C541"  =  "1"  /* Malignant neoplasm of endometrium */
"C542"  =  "1"  /* Malignant neoplasm of myometrium */
"C543"  =  "1"  /* Malignant neoplasm of fundus uteri */
"C548"  =  "1"  /* Malignant neoplasm of overlapping sites of corpus uteri */
"C549"  =  "1"  /* Malignant neoplasm of corpus uteri, unspecified */
"C55"  =  "1"  /* Malignant neoplasm of uterus, part unspecified */
"C561"  =  "1"  /* Malignant neoplasm of right ovary */
"C562"  =  "1"  /* Malignant neoplasm of left ovary */
"C569"  =  "1"  /* Malignant neoplasm of unspecified ovary */
"C5700"  =  "1"  /* Malignant neoplasm of unspecified fallopian tube */
"C5701"  =  "1"  /* Malignant neoplasm of right fallopian tube */
"C5702"  =  "1"  /* Malignant neoplasm of left fallopian tube */
"C5710"  =  "1"  /* Malignant neoplasm of unspecified broad ligament */
"C5711"  =  "1"  /* Malignant neoplasm of right broad ligament */
"C5712"  =  "1"  /* Malignant neoplasm of left broad ligament */
"C5720"  =  "1"  /* Malignant neoplasm of unspecified round ligament */
"C5721"  =  "1"  /* Malignant neoplasm of right round ligament */
"C5722"  =  "1"  /* Malignant neoplasm of left round ligament */
"C573"  =  "1"  /* Malignant neoplasm of parametrium */
"C574"  =  "1"  /* Malignant neoplasm of uterine adnexa, unspecified */
"C577"  =  "1"  /* Malignant neoplasm of other specified female genital organs */
"C578"  =  "1"  /* Malignant neoplasm of overlapping sites of female genital organs */
"C579"  =  "1"  /* Malignant neoplasm of female genital organ, unspecified */
"C58"  =  "1"  /* Malignant neoplasm of placenta */
"C600"  =  "1"  /* Malignant neoplasm of prepuce */
"C601"  =  "1"  /* Malignant neoplasm of glans penis */
"C602"  =  "1"  /* Malignant neoplasm of body of penis */
"C608"  =  "1"  /* Malignant neoplasm of overlapping sites of penis */
"C609"  =  "1"  /* Malignant neoplasm of penis, unspecified */
"C61"  =  "1"  /* Malignant neoplasm of prostate */
"C6200"  =  "1"  /* Malignant neoplasm of unspecified undescended testis */
"C6201"  =  "1"  /* Malignant neoplasm of undescended right testis */
"C6202"  =  "1"  /* Malignant neoplasm of undescended left testis */
"C6210"  =  "1"  /* Malignant neoplasm of unspecified descended testis */
"C6211"  =  "1"  /* Malignant neoplasm of descended right testis */
"C6212"  =  "1"  /* Malignant neoplasm of descended left testis */
"C6290"  =  "1"  /* Malignant neoplasm of unspecified testis, unspecified whether descended or undescended */
"C6291"  =  "1"  /* Malignant neoplasm of right testis, unspecified whether descended or undescended */
"C6292"  =  "1"  /* Malignant neoplasm of left testis, unspecified whether descended or undescended */
"C6300"  =  "1"  /* Malignant neoplasm of unspecified epididymis */
"C6301"  =  "1"  /* Malignant neoplasm of right epididymis */
"C6302"  =  "1"  /* Malignant neoplasm of left epididymis */
"C6310"  =  "1"  /* Malignant neoplasm of unspecified spermatic cord */
"C6311"  =  "1"  /* Malignant neoplasm of right spermatic cord */
"C6312"  =  "1"  /* Malignant neoplasm of left spermatic cord */
"C632"  =  "1"  /* Malignant neoplasm of scrotum */
"C637"  =  "1"  /* Malignant neoplasm of other specified male genital organs */
"C638"  =  "1"  /* Malignant neoplasm of overlapping sites of male genital organs */
"C639"  =  "1"  /* Malignant neoplasm of male genital organ, unspecified */
"C641"  =  "1"  /* Malignant neoplasm of right kidney, except renal pelvis */
"C642"  =  "1"  /* Malignant neoplasm of left kidney, except renal pelvis */
"C649"  =  "1"  /* Malignant neoplasm of unspecified kidney, except renal pelvis */
"C651"  =  "1"  /* Malignant neoplasm of right renal pelvis */
"C652"  =  "1"  /* Malignant neoplasm of left renal pelvis */
"C659"  =  "1"  /* Malignant neoplasm of unspecified renal pelvis */
"C661"  =  "1"  /* Malignant neoplasm of right ureter */
"C662"  =  "1"  /* Malignant neoplasm of left ureter */
"C669"  =  "1"  /* Malignant neoplasm of unspecified ureter */
"C670"  =  "1"  /* Malignant neoplasm of trigone of bladder */
"C671"  =  "1"  /* Malignant neoplasm of dome of bladder */
"C672"  =  "1"  /* Malignant neoplasm of lateral wall of bladder */
"C673"  =  "1"  /* Malignant neoplasm of anterior wall of bladder */
"C674"  =  "1"  /* Malignant neoplasm of posterior wall of bladder */
"C675"  =  "1"  /* Malignant neoplasm of bladder neck */
"C676"  =  "1"  /* Malignant neoplasm of ureteric orifice */
"C677"  =  "1"  /* Malignant neoplasm of urachus */
"C678"  =  "1"  /* Malignant neoplasm of overlapping sites of bladder */
"C679"  =  "1"  /* Malignant neoplasm of bladder, unspecified */
"C680"  =  "1"  /* Malignant neoplasm of urethra */
"C681"  =  "1"  /* Malignant neoplasm of paraurethral glands */
"C688"  =  "1"  /* Malignant neoplasm of overlapping sites of urinary organs */
"C689"  =  "1"  /* Malignant neoplasm of urinary organ, unspecified */
"C6900"  =  "1"  /* Malignant neoplasm of unspecified conjunctiva */
"C6901"  =  "1"  /* Malignant neoplasm of right conjunctiva */
"C6902"  =  "1"  /* Malignant neoplasm of left conjunctiva */
"C6910"  =  "1"  /* Malignant neoplasm of unspecified cornea */
"C6911"  =  "1"  /* Malignant neoplasm of right cornea */
"C6912"  =  "1"  /* Malignant neoplasm of left cornea */
"C6920"  =  "1"  /* Malignant neoplasm of unspecified retina */
"C6921"  =  "1"  /* Malignant neoplasm of right retina */
"C6922"  =  "1"  /* Malignant neoplasm of left retina */
"C6930"  =  "1"  /* Malignant neoplasm of unspecified choroid */
"C6931"  =  "1"  /* Malignant neoplasm of right choroid */
"C6932"  =  "1"  /* Malignant neoplasm of left choroid */
"C6940"  =  "1"  /* Malignant neoplasm of unspecified ciliary body */
"C6941"  =  "1"  /* Malignant neoplasm of right ciliary body */
"C6942"  =  "1"  /* Malignant neoplasm of left ciliary body */
"C6950"  =  "1"  /* Malignant neoplasm of unspecified lacrimal gland and duct */
"C6951"  =  "1"  /* Malignant neoplasm of right lacrimal gland and duct */
"C6952"  =  "1"  /* Malignant neoplasm of left lacrimal gland and duct */
"C6960"  =  "1"  /* Malignant neoplasm of unspecified orbit */
"C6961"  =  "1"  /* Malignant neoplasm of right orbit */
"C6962"  =  "1"  /* Malignant neoplasm of left orbit */
"C6980"  =  "1"  /* Malignant neoplasm of overlapping sites of unspecified eye and adnexa */
"C6981"  =  "1"  /* Malignant neoplasm of overlapping sites of right eye and adnexa */
"C6982"  =  "1"  /* Malignant neoplasm of overlapping sites of left eye and adnexa */
"C6990"  =  "1"  /* Malignant neoplasm of unspecified site of unspecified eye */
"C6991"  =  "1"  /* Malignant neoplasm of unspecified site of right eye */
"C6992"  =  "1"  /* Malignant neoplasm of unspecified site of left eye */
"C700"  =  "1"  /* Malignant neoplasm of cerebral meninges */
"C701"  =  "1"  /* Malignant neoplasm of spinal meninges */
"C709"  =  "1"  /* Malignant neoplasm of meninges, unspecified */
"C710"  =  "1"  /* Malignant neoplasm of cerebrum, except lobes and ventricles */
"C711"  =  "1"  /* Malignant neoplasm of frontal lobe */
"C712"  =  "1"  /* Malignant neoplasm of temporal lobe */
"C713"  =  "1"  /* Malignant neoplasm of parietal lobe */
"C714"  =  "1"  /* Malignant neoplasm of occipital lobe */
"C715"  =  "1"  /* Malignant neoplasm of cerebral ventricle */
"C716"  =  "1"  /* Malignant neoplasm of cerebellum */
"C717"  =  "1"  /* Malignant neoplasm of brain stem */
"C718"  =  "1"  /* Malignant neoplasm of overlapping sites of brain */
"C719"  =  "1"  /* Malignant neoplasm of brain, unspecified */
"C720"  =  "1"  /* Malignant neoplasm of spinal cord */
"C721"  =  "1"  /* Malignant neoplasm of cauda equina */
"C7220"  =  "1"  /* Malignant neoplasm of unspecified olfactory nerve */
"C7221"  =  "1"  /* Malignant neoplasm of right olfactory nerve */
"C7222"  =  "1"  /* Malignant neoplasm of left olfactory nerve */
"C7230"  =  "1"  /* Malignant neoplasm of unspecified optic nerve */
"C7231"  =  "1"  /* Malignant neoplasm of right optic nerve */
"C7232"  =  "1"  /* Malignant neoplasm of left optic nerve */
"C7240"  =  "1"  /* Malignant neoplasm of unspecified acoustic nerve */
"C7241"  =  "1"  /* Malignant neoplasm of right acoustic nerve */
"C7242"  =  "1"  /* Malignant neoplasm of left acoustic nerve */
"C7250"  =  "1"  /* Malignant neoplasm of unspecified cranial nerve */
"C7259"  =  "1"  /* Malignant neoplasm of other cranial nerves */
"C729"  =  "1"  /* Malignant neoplasm of central nervous system, unspecified */
"C73"  =  "1"  /* Malignant neoplasm of thyroid gland */
"C7400"  =  "1"  /* Malignant neoplasm of cortex of unspecified adrenal gland */
"C7401"  =  "1"  /* Malignant neoplasm of cortex of right adrenal gland */
"C7402"  =  "1"  /* Malignant neoplasm of cortex of left adrenal gland */
"C7410"  =  "1"  /* Malignant neoplasm of medulla of unspecified adrenal gland */
"C7411"  =  "1"  /* Malignant neoplasm of medulla of right adrenal gland */
"C7412"  =  "1"  /* Malignant neoplasm of medulla of left adrenal gland */
"C7490"  =  "1"  /* Malignant neoplasm of unspecified part of unspecified adrenal gland */
"C7491"  =  "1"  /* Malignant neoplasm of unspecified part of right adrenal gland */
"C7492"  =  "1"  /* Malignant neoplasm of unspecified part of left adrenal gland */
"C750"  =  "1"  /* Malignant neoplasm of parathyroid gland */
"C751"  =  "1"  /* Malignant neoplasm of pituitary gland */
"C752"  =  "1"  /* Malignant neoplasm of craniopharyngeal duct */
"C753"  =  "1"  /* Malignant neoplasm of pineal gland */
"C754"  =  "1"  /* Malignant neoplasm of carotid body */
"C755"  =  "1"  /* Malignant neoplasm of aortic body and other paraganglia */
"C758"  =  "1"  /* Malignant neoplasm with pluriglandular involvement, unspecified */
"C759"  =  "1"  /* Malignant neoplasm of endocrine gland, unspecified */
"C760"  =  "1"  /* Malignant neoplasm of head, face and neck */
"C761"  =  "1"  /* Malignant neoplasm of thorax */
"C762"  =  "1"  /* Malignant neoplasm of abdomen */
"C763"  =  "1"  /* Malignant neoplasm of pelvis */
"C7640"  =  "1"  /* Malignant neoplasm of unspecified upper limb */
"C7641"  =  "1"  /* Malignant neoplasm of right upper limb */
"C7642"  =  "1"  /* Malignant neoplasm of left upper limb */
"C7650"  =  "1"  /* Malignant neoplasm of unspecified lower limb */
"C7651"  =  "1"  /* Malignant neoplasm of right lower limb */
"C7652"  =  "1"  /* Malignant neoplasm of left lower limb */
"C768"  =  "1"  /* Malignant neoplasm of other specified ill-defined sites */
"C770"  =  "1"  /* Secondary and unspecified malignant neoplasm of lymph nodes of head, face and neck */
"C771"  =  "1"  /* Secondary and unspecified malignant neoplasm of intrathoracic lymph nodes */
"C772"  =  "1"  /* Secondary and unspecified malignant neoplasm of intra-abdominal lymph nodes */
"C773"  =  "1"  /* Secondary and unspecified malignant neoplasm of axilla and upper limb lymph nodes */
"C774"  =  "1"  /* Secondary and unspecified malignant neoplasm of inguinal and lower limb lymph nodes */
"C775"  =  "1"  /* Secondary and unspecified malignant neoplasm of intrapelvic lymph nodes */
"C778"  =  "1"  /* Secondary and unspecified malignant neoplasm of lymph nodes of multiple regions */
"C779"  =  "1"  /* Secondary and unspecified malignant neoplasm of lymph node, unspecified */
"C7800"  =  "1"  /* Secondary malignant neoplasm of unspecified lung */
"C7801"  =  "1"  /* Secondary malignant neoplasm of right lung */
"C7802"  =  "1"  /* Secondary malignant neoplasm of left lung */
"C781"  =  "1"  /* Secondary malignant neoplasm of mediastinum */
"C782"  =  "1"  /* Secondary malignant neoplasm of pleura */
"C7830"  =  "1"  /* Secondary malignant neoplasm of unspecified respiratory organ */
"C7839"  =  "1"  /* Secondary malignant neoplasm of other respiratory organs */
"C784"  =  "1"  /* Secondary malignant neoplasm of small intestine */
"C785"  =  "1"  /* Secondary malignant neoplasm of large intestine and rectum */
"C786"  =  "1"  /* Secondary malignant neoplasm of retroperitoneum and peritoneum */
"C787"  =  "1"  /* Secondary malignant neoplasm of liver and intrahepatic bile duct */
"C7880"  =  "1"  /* Secondary malignant neoplasm of unspecified digestive organ */
"C7889"  =  "1"  /* Secondary malignant neoplasm of other digestive organs */
"C7900"  =  "1"  /* Secondary malignant neoplasm of unspecified kidney and renal pelvis */
"C7901"  =  "1"  /* Secondary malignant neoplasm of right kidney and renal pelvis */
"C7902"  =  "1"  /* Secondary malignant neoplasm of left kidney and renal pelvis */
"C7910"  =  "1"  /* Secondary malignant neoplasm of unspecified urinary organs */
"C7911"  =  "1"  /* Secondary malignant neoplasm of bladder */
"C7919"  =  "1"  /* Secondary malignant neoplasm of other urinary organs */
"C792"  =  "1"  /* Secondary malignant neoplasm of skin */
"C7931"  =  "1"  /* Secondary malignant neoplasm of brain */
"C7932"  =  "1"  /* Secondary malignant neoplasm of cerebral meninges */
"C7940"  =  "1"  /* Secondary malignant neoplasm of unspecified part of nervous system */
"C7949"  =  "1"  /* Secondary malignant neoplasm of other parts of nervous system */
"C7951"  =  "1"  /* Secondary malignant neoplasm of bone */
"C7952"  =  "1"  /* Secondary malignant neoplasm of bone marrow */
"C7960"  =  "1"  /* Secondary malignant neoplasm of unspecified ovary */
"C7961"  =  "1"  /* Secondary malignant neoplasm of right ovary */
"C7962"  =  "1"  /* Secondary malignant neoplasm of left ovary */
"C7970"  =  "1"  /* Secondary malignant neoplasm of unspecified adrenal gland */
"C7971"  =  "1"  /* Secondary malignant neoplasm of right adrenal gland */
"C7972"  =  "1"  /* Secondary malignant neoplasm of left adrenal gland */
"C7981"  =  "1"  /* Secondary malignant neoplasm of breast */
"C7982"  =  "1"  /* Secondary malignant neoplasm of genital organs */
"C7989"  =  "1"  /* Secondary malignant neoplasm of other specified sites */
"C799"  =  "1"  /* Secondary malignant neoplasm of unspecified site */
"C800"  =  "1"  /* Disseminated malignant neoplasm, unspecified */
"C801"  =  "1"  /* Malignant (primary) neoplasm, unspecified */
"C8100"  =  "1"  /* Nodular lymphocyte predominant Hodgkin lymphoma, unspecified site */
"C8101"  =  "1"  /* Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of head, face, and neck */
"C8102"  =  "1"  /* Nodular lymphocyte predominant Hodgkin lymphoma, intrathoracic lymph nodes */
"C8103"  =  "1"  /* Nodular lymphocyte predominant Hodgkin lymphoma, intra-abdominal lymph nodes */
"C8104"  =  "1"  /* Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of axilla and upper limb */
"C8105"  =  "1"  /* Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of inguinal region and lower limb */
"C8106"  =  "1"  /* Nodular lymphocyte predominant Hodgkin lymphoma, intrapelvic lymph nodes */
"C8107"  =  "1"  /* Nodular lymphocyte predominant Hodgkin lymphoma, spleen */
"C8108"  =  "1"  /* Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of multiple sites */
"C8109"  =  "1"  /* Nodular lymphocyte predominant Hodgkin lymphoma, extranodal and solid organ sites */
"C8110"  =  "1"  /* Nodular sclerosis classical Hodgkin lymphoma, unspecified site */
"C8111"  =  "1"  /* Nodular sclerosis classical Hodgkin lymphoma, lymph nodes of head, face, and neck */
"C8112"  =  "1"  /* Nodular sclerosis classical Hodgkin lymphoma, intrathoracic lymph nodes */
"C8113"  =  "1"  /* Nodular sclerosis classical Hodgkin lymphoma, intra-abdominal lymph nodes */
"C8114"  =  "1"  /* Nodular sclerosis classical Hodgkin lymphoma, lymph nodes of axilla and upper limb */
"C8115"  =  "1"  /* Nodular sclerosis classical Hodgkin lymphoma, lymph nodes of inguinal region and lower limb */
"C8116"  =  "1"  /* Nodular sclerosis classical Hodgkin lymphoma, intrapelvic lymph nodes */
"C8117"  =  "1"  /* Nodular sclerosis classical Hodgkin lymphoma, spleen */
"C8118"  =  "1"  /* Nodular sclerosis classical Hodgkin lymphoma, lymph nodes of multiple sites */
"C8119"  =  "1"  /* Nodular sclerosis classical Hodgkin lymphoma, extranodal and solid organ sites */
"C8120"  =  "1"  /* Mixed cellularity classical Hodgkin lymphoma, unspecified site */
"C8121"  =  "1"  /* Mixed cellularity classical Hodgkin lymphoma, lymph nodes of head, face, and neck */
"C8122"  =  "1"  /* Mixed cellularity classical Hodgkin lymphoma, intrathoracic lymph nodes */
"C8123"  =  "1"  /* Mixed cellularity classical Hodgkin lymphoma, intra-abdominal lymph nodes */
"C8124"  =  "1"  /* Mixed cellularity classical Hodgkin lymphoma, lymph nodes of axilla and upper limb */
"C8125"  =  "1"  /* Mixed cellularity classical Hodgkin lymphoma, lymph nodes of inguinal region and lower limb */
"C8126"  =  "1"  /* Mixed cellularity classical Hodgkin lymphoma, intrapelvic lymph nodes */
"C8127"  =  "1"  /* Mixed cellularity classical Hodgkin lymphoma, spleen */
"C8128"  =  "1"  /* Mixed cellularity classical Hodgkin lymphoma, lymph nodes of multiple sites */
"C8129"  =  "1"  /* Mixed cellularity classical Hodgkin lymphoma, extranodal and solid organ sites */
"C8130"  =  "1"  /* Lymphocyte depleted classical Hodgkin lymphoma, unspecified site */
"C8131"  =  "1"  /* Lymphocyte depleted classical Hodgkin lymphoma, lymph nodes of head, face, and neck */
"C8132"  =  "1"  /* Lymphocyte depleted classical Hodgkin lymphoma, intrathoracic lymph nodes */
"C8133"  =  "1"  /* Lymphocyte depleted classical Hodgkin lymphoma, intra-abdominal lymph nodes */
"C8134"  =  "1"  /* Lymphocyte depleted classical Hodgkin lymphoma, lymph nodes of axilla and upper limb */
"C8135"  =  "1"  /* Lymphocyte depleted classical Hodgkin lymphoma, lymph nodes of inguinal region and lower limb */
"C8136"  =  "1"  /* Lymphocyte depleted classical Hodgkin lymphoma, intrapelvic lymph nodes */
"C8137"  =  "1"  /* Lymphocyte depleted classical Hodgkin lymphoma, spleen */
"C8138"  =  "1"  /* Lymphocyte depleted classical Hodgkin lymphoma, lymph nodes of multiple sites */
"C8139"  =  "1"  /* Lymphocyte depleted classical Hodgkin lymphoma, extranodal and solid organ sites */
"C8140"  =  "1"  /* Lymphocyte-rich classical Hodgkin lymphoma, unspecified site */
"C8141"  =  "1"  /* Lymphocyte-rich classical Hodgkin lymphoma, lymph nodes of head, face, and neck */
"C8142"  =  "1"  /* Lymphocyte-rich classical Hodgkin lymphoma, intrathoracic lymph nodes */
"C8143"  =  "1"  /* Lymphocyte-rich classical Hodgkin lymphoma, intra-abdominal lymph nodes */
"C8144"  =  "1"  /* Lymphocyte-rich classical Hodgkin lymphoma, lymph nodes of axilla and upper limb */
"C8145"  =  "1"  /* Lymphocyte-rich classical Hodgkin lymphoma, lymph nodes of inguinal region and lower limb */
"C8146"  =  "1"  /* Lymphocyte-rich classical Hodgkin lymphoma, intrapelvic lymph nodes */
"C8147"  =  "1"  /* Lymphocyte-rich classical Hodgkin lymphoma, spleen */
"C8148"  =  "1"  /* Lymphocyte-rich classical Hodgkin lymphoma, lymph nodes of multiple sites */
"C8149"  =  "1"  /* Lymphocyte-rich classical Hodgkin lymphoma, extranodal and solid organ sites */
"C8170"  =  "1"  /* Other classical Hodgkin lymphoma, unspecified site */
"C8171"  =  "1"  /* Other classical Hodgkin lymphoma, lymph nodes of head, face, and neck */
"C8172"  =  "1"  /* Other classical Hodgkin lymphoma, intrathoracic lymph nodes */
"C8173"  =  "1"  /* Other classical Hodgkin lymphoma, intra-abdominal lymph nodes */
"C8174"  =  "1"  /* Other classical Hodgkin lymphoma, lymph nodes of axilla and upper limb */
"C8175"  =  "1"  /* Other classical Hodgkin lymphoma, lymph nodes of inguinal region and lower limb */
"C8176"  =  "1"  /* Other classical Hodgkin lymphoma, intrapelvic lymph nodes */
"C8177"  =  "1"  /* Other classical Hodgkin lymphoma, spleen */
"C8178"  =  "1"  /* Other classical Hodgkin lymphoma, lymph nodes of multiple sites */
"C8179"  =  "1"  /* Other classical Hodgkin lymphoma, extranodal and solid organ sites */
"C8190"  =  "1"  /* Hodgkin lymphoma, unspecified, unspecified site */
"C8191"  =  "1"  /* Hodgkin lymphoma, unspecified, lymph nodes of head, face, and neck */
"C8192"  =  "1"  /* Hodgkin lymphoma, unspecified, intrathoracic lymph nodes */
"C8193"  =  "1"  /* Hodgkin lymphoma, unspecified, intra-abdominal lymph nodes */
"C8194"  =  "1"  /* Hodgkin lymphoma, unspecified, lymph nodes of axilla and upper limb */
"C8195"  =  "1"  /* Hodgkin lymphoma, unspecified, lymph nodes of inguinal region and lower limb */
"C8196"  =  "1"  /* Hodgkin lymphoma, unspecified, intrapelvic lymph nodes */
"C8197"  =  "1"  /* Hodgkin lymphoma, unspecified, spleen */
"C8198"  =  "1"  /* Hodgkin lymphoma, unspecified, lymph nodes of multiple sites */
"C8199"  =  "1"  /* Hodgkin lymphoma, unspecified, extranodal and solid organ sites */
"C8200"  =  "1"  /* Follicular lymphoma grade I, unspecified site */
"C8201"  =  "1"  /* Follicular lymphoma grade I, lymph nodes of head, face, and neck */
"C8202"  =  "1"  /* Follicular lymphoma grade I, intrathoracic lymph nodes */
"C8203"  =  "1"  /* Follicular lymphoma grade I, intra-abdominal lymph nodes */
"C8204"  =  "1"  /* Follicular lymphoma grade I, lymph nodes of axilla and upper limb */
"C8205"  =  "1"  /* Follicular lymphoma grade I, lymph nodes of inguinal region and lower limb */
"C8206"  =  "1"  /* Follicular lymphoma grade I, intrapelvic lymph nodes */
"C8207"  =  "1"  /* Follicular lymphoma grade I, spleen */
"C8208"  =  "1"  /* Follicular lymphoma grade I, lymph nodes of multiple sites */
"C8209"  =  "1"  /* Follicular lymphoma grade I, extranodal and solid organ sites */
"C8210"  =  "1"  /* Follicular lymphoma grade II, unspecified site */
"C8211"  =  "1"  /* Follicular lymphoma grade II, lymph nodes of head, face, and neck */
"C8212"  =  "1"  /* Follicular lymphoma grade II, intrathoracic lymph nodes */
"C8213"  =  "1"  /* Follicular lymphoma grade II, intra-abdominal lymph nodes */
"C8214"  =  "1"  /* Follicular lymphoma grade II, lymph nodes of axilla and upper limb */
"C8215"  =  "1"  /* Follicular lymphoma grade II, lymph nodes of inguinal region and lower limb */
"C8216"  =  "1"  /* Follicular lymphoma grade II, intrapelvic lymph nodes */
"C8217"  =  "1"  /* Follicular lymphoma grade II, spleen */
"C8218"  =  "1"  /* Follicular lymphoma grade II, lymph nodes of multiple sites */
"C8219"  =  "1"  /* Follicular lymphoma grade II, extranodal and solid organ sites */
"C8220"  =  "1"  /* Follicular lymphoma grade III, unspecified, unspecified site */
"C8221"  =  "1"  /* Follicular lymphoma grade III, unspecified, lymph nodes of head, face, and neck */
"C8222"  =  "1"  /* Follicular lymphoma grade III, unspecified, intrathoracic lymph nodes */
"C8223"  =  "1"  /* Follicular lymphoma grade III, unspecified, intra-abdominal lymph nodes */
"C8224"  =  "1"  /* Follicular lymphoma grade III, unspecified, lymph nodes of axilla and upper limb */
"C8225"  =  "1"  /* Follicular lymphoma grade III, unspecified, lymph nodes of inguinal region and lower limb */
"C8226"  =  "1"  /* Follicular lymphoma grade III, unspecified, intrapelvic lymph nodes */
"C8227"  =  "1"  /* Follicular lymphoma grade III, unspecified, spleen */
"C8228"  =  "1"  /* Follicular lymphoma grade III, unspecified, lymph nodes of multiple sites */
"C8229"  =  "1"  /* Follicular lymphoma grade III, unspecified, extranodal and solid organ sites */
"C8230"  =  "1"  /* Follicular lymphoma grade IIIa, unspecified site */
"C8231"  =  "1"  /* Follicular lymphoma grade IIIa, lymph nodes of head, face, and neck */
"C8232"  =  "1"  /* Follicular lymphoma grade IIIa, intrathoracic lymph nodes */
"C8233"  =  "1"  /* Follicular lymphoma grade IIIa, intra-abdominal lymph nodes */
"C8234"  =  "1"  /* Follicular lymphoma grade IIIa, lymph nodes of axilla and upper limb */
"C8235"  =  "1"  /* Follicular lymphoma grade IIIa, lymph nodes of inguinal region and lower limb */
"C8236"  =  "1"  /* Follicular lymphoma grade IIIa, intrapelvic lymph nodes */
"C8237"  =  "1"  /* Follicular lymphoma grade IIIa, spleen */
"C8238"  =  "1"  /* Follicular lymphoma grade IIIa, lymph nodes of multiple sites */
"C8239"  =  "1"  /* Follicular lymphoma grade IIIa, extranodal and solid organ sites */
"C8240"  =  "1"  /* Follicular lymphoma grade IIIb, unspecified site */
"C8241"  =  "1"  /* Follicular lymphoma grade IIIb, lymph nodes of head, face, and neck */
"C8242"  =  "1"  /* Follicular lymphoma grade IIIb, intrathoracic lymph nodes */
"C8243"  =  "1"  /* Follicular lymphoma grade IIIb, intra-abdominal lymph nodes */
"C8244"  =  "1"  /* Follicular lymphoma grade IIIb, lymph nodes of axilla and upper limb */
"C8245"  =  "1"  /* Follicular lymphoma grade IIIb, lymph nodes of inguinal region and lower limb */
"C8246"  =  "1"  /* Follicular lymphoma grade IIIb, intrapelvic lymph nodes */
"C8247"  =  "1"  /* Follicular lymphoma grade IIIb, spleen */
"C8248"  =  "1"  /* Follicular lymphoma grade IIIb, lymph nodes of multiple sites */
"C8249"  =  "1"  /* Follicular lymphoma grade IIIb, extranodal and solid organ sites */
"C8250"  =  "1"  /* Diffuse follicle center lymphoma, unspecified site */
"C8251"  =  "1"  /* Diffuse follicle center lymphoma, lymph nodes of head, face, and neck */
"C8252"  =  "1"  /* Diffuse follicle center lymphoma, intrathoracic lymph nodes */
"C8253"  =  "1"  /* Diffuse follicle center lymphoma, intra-abdominal lymph nodes */
"C8254"  =  "1"  /* Diffuse follicle center lymphoma, lymph nodes of axilla and upper limb */
"C8255"  =  "1"  /* Diffuse follicle center lymphoma, lymph nodes of inguinal region and lower limb */
"C8256"  =  "1"  /* Diffuse follicle center lymphoma, intrapelvic lymph nodes */
"C8257"  =  "1"  /* Diffuse follicle center lymphoma, spleen */
"C8258"  =  "1"  /* Diffuse follicle center lymphoma, lymph nodes of multiple sites */
"C8259"  =  "1"  /* Diffuse follicle center lymphoma, extranodal and solid organ sites */
"C8260"  =  "1"  /* Cutaneous follicle center lymphoma, unspecified site */
"C8261"  =  "1"  /* Cutaneous follicle center lymphoma, lymph nodes of head, face, and neck */
"C8262"  =  "1"  /* Cutaneous follicle center lymphoma, intrathoracic lymph nodes */
"C8263"  =  "1"  /* Cutaneous follicle center lymphoma, intra-abdominal lymph nodes */
"C8264"  =  "1"  /* Cutaneous follicle center lymphoma, lymph nodes of axilla and upper limb */
"C8265"  =  "1"  /* Cutaneous follicle center lymphoma, lymph nodes of inguinal region and lower limb */
"C8266"  =  "1"  /* Cutaneous follicle center lymphoma, intrapelvic lymph nodes */
"C8267"  =  "1"  /* Cutaneous follicle center lymphoma, spleen */
"C8268"  =  "1"  /* Cutaneous follicle center lymphoma, lymph nodes of multiple sites */
"C8269"  =  "1"  /* Cutaneous follicle center lymphoma, extranodal and solid organ sites */
"C8280"  =  "1"  /* Other types of follicular lymphoma, unspecified site */
"C8281"  =  "1"  /* Other types of follicular lymphoma, lymph nodes of head, face, and neck */
"C8282"  =  "1"  /* Other types of follicular lymphoma, intrathoracic lymph nodes */
"C8283"  =  "1"  /* Other types of follicular lymphoma, intra-abdominal lymph nodes */
"C8284"  =  "1"  /* Other types of follicular lymphoma, lymph nodes of axilla and upper limb */
"C8285"  =  "1"  /* Other types of follicular lymphoma, lymph nodes of inguinal region and lower limb */
"C8286"  =  "1"  /* Other types of follicular lymphoma, intrapelvic lymph nodes */
"C8287"  =  "1"  /* Other types of follicular lymphoma, spleen */
"C8288"  =  "1"  /* Other types of follicular lymphoma, lymph nodes of multiple sites */
"C8289"  =  "1"  /* Other types of follicular lymphoma, extranodal and solid organ sites */
"C8290"  =  "1"  /* Follicular lymphoma, unspecified, unspecified site */
"C8291"  =  "1"  /* Follicular lymphoma, unspecified, lymph nodes of head, face, and neck */
"C8292"  =  "1"  /* Follicular lymphoma, unspecified, intrathoracic lymph nodes */
"C8293"  =  "1"  /* Follicular lymphoma, unspecified, intra-abdominal lymph nodes */
"C8294"  =  "1"  /* Follicular lymphoma, unspecified, lymph nodes of axilla and upper limb */
"C8295"  =  "1"  /* Follicular lymphoma, unspecified, lymph nodes of inguinal region and lower limb */
"C8296"  =  "1"  /* Follicular lymphoma, unspecified, intrapelvic lymph nodes */
"C8297"  =  "1"  /* Follicular lymphoma, unspecified, spleen */
"C8298"  =  "1"  /* Follicular lymphoma, unspecified, lymph nodes of multiple sites */
"C8299"  =  "1"  /* Follicular lymphoma, unspecified, extranodal and solid organ sites */
"C8400"  =  "1"  /* Mycosis fungoides, unspecified site */
"C8401"  =  "1"  /* Mycosis fungoides, lymph nodes of head, face, and neck */
"C8402"  =  "1"  /* Mycosis fungoides, intrathoracic lymph nodes */
"C8403"  =  "1"  /* Mycosis fungoides, intra-abdominal lymph nodes */
"C8404"  =  "1"  /* Mycosis fungoides, lymph nodes of axilla and upper limb */
"C8405"  =  "1"  /* Mycosis fungoides, lymph nodes of inguinal region and lower limb */
"C8406"  =  "1"  /* Mycosis fungoides, intrapelvic lymph nodes */
"C8407"  =  "1"  /* Mycosis fungoides, spleen */
"C8408"  =  "1"  /* Mycosis fungoides, lymph nodes of multiple sites */
"C8409"  =  "1"  /* Mycosis fungoides, extranodal and solid organ sites */
"C8410"  =  "1"  /* Sezary disease, unspecified site */
"C8411"  =  "1"  /* Sezary disease, lymph nodes of head, face, and neck */
"C8412"  =  "1"  /* Sezary disease, intrathoracic lymph nodes */
"C8413"  =  "1"  /* Sezary disease, intra-abdominal lymph nodes */
"C8414"  =  "1"  /* Sezary disease, lymph nodes of axilla and upper limb */
"C8415"  =  "1"  /* Sezary disease, lymph nodes of inguinal region and lower limb */
"C8416"  =  "1"  /* Sezary disease, intrapelvic lymph nodes */
"C8417"  =  "1"  /* Sezary disease, spleen */
"C8418"  =  "1"  /* Sezary disease, lymph nodes of multiple sites */
"C8419"  =  "1"  /* Sezary disease, extranodal and solid organ sites */
"C8490"  =  "1"  /* Mature T/NK-cell lymphomas, unspecified, unspecified site */
"C8491"  =  "1"  /* Mature T/NK-cell lymphomas, unspecified, lymph nodes of head, face, and neck */
"C8492"  =  "1"  /* Mature T/NK-cell lymphomas, unspecified, intrathoracic lymph nodes */
"C8493"  =  "1"  /* Mature T/NK-cell lymphomas, unspecified, intra-abdominal lymph nodes */
"C8494"  =  "1"  /* Mature T/NK-cell lymphomas, unspecified, lymph nodes of axilla and upper limb */
"C8495"  =  "1"  /* Mature T/NK-cell lymphomas, unspecified, lymph nodes of inguinal region and lower limb */
"C8496"  =  "1"  /* Mature T/NK-cell lymphomas, unspecified, intrapelvic lymph nodes */
"C8497"  =  "1"  /* Mature T/NK-cell lymphomas, unspecified, spleen */
"C8498"  =  "1"  /* Mature T/NK-cell lymphomas, unspecified, lymph nodes of multiple sites */
"C8499"  =  "1"  /* Mature T/NK-cell lymphomas, unspecified, extranodal and solid organ sites */
"C84A0"  =  "1"  /* Cutaneous T-cell lymphoma, unspecified, unspecified site */
"C84A1"  =  "1"  /* Cutaneous T-cell lymphoma, unspecified lymph nodes of head, face, and neck */
"C84A2"  =  "1"  /* Cutaneous T-cell lymphoma, unspecified, intrathoracic lymph nodes */
"C84A3"  =  "1"  /* Cutaneous T-cell lymphoma, unspecified, intra-abdominal lymph nodes */
"C84A4"  =  "1"  /* Cutaneous T-cell lymphoma, unspecified, lymph nodes of axilla and upper limb */
"C84A5"  =  "1"  /* Cutaneous T-cell lymphoma, unspecified, lymph nodes of inguinal region and lower limb */
"C84A6"  =  "1"  /* Cutaneous T-cell lymphoma, unspecified, intrapelvic lymph nodes */
"C84A7"  =  "1"  /* Cutaneous T-cell lymphoma, unspecified, spleen */
"C84A8"  =  "1"  /* Cutaneous T-cell lymphoma, unspecified, lymph nodes of multiple sites */
"C84A9"  =  "1"  /* Cutaneous T-cell lymphoma, unspecified, extranodal and solid organ sites */
"C84Z0"  =  "1"  /* Other mature T/NK-cell lymphomas, unspecified site */
"C84Z1"  =  "1"  /* Other mature T/NK-cell lymphomas, lymph nodes of head, face, and neck */
"C84Z2"  =  "1"  /* Other mature T/NK-cell lymphomas, intrathoracic lymph nodes */
"C84Z3"  =  "1"  /* Other mature T/NK-cell lymphomas, intra-abdominal lymph nodes */
"C84Z4"  =  "1"  /* Other mature T/NK-cell lymphomas, lymph nodes of axilla and upper limb */
"C84Z5"  =  "1"  /* Other mature T/NK-cell lymphomas, lymph nodes of inguinal region and lower limb */
"C84Z6"  =  "1"  /* Other mature T/NK-cell lymphomas, intrapelvic lymph nodes */
"C84Z7"  =  "1"  /* Other mature T/NK-cell lymphomas, spleen */
"C84Z8"  =  "1"  /* Other mature T/NK-cell lymphomas, lymph nodes of multiple sites */
"C84Z9"  =  "1"  /* Other mature T/NK-cell lymphomas, extranodal and solid organ sites */
"C8510"  =  "1"  /* Unspecified B-cell lymphoma, unspecified site */
"C8511"  =  "1"  /* Unspecified B-cell lymphoma, lymph nodes of head, face, and neck */
"C8512"  =  "1"  /* Unspecified B-cell lymphoma, intrathoracic lymph nodes */
"C8513"  =  "1"  /* Unspecified B-cell lymphoma, intra-abdominal lymph nodes */
"C8514"  =  "1"  /* Unspecified B-cell lymphoma, lymph nodes of axilla and upper limb */
"C8515"  =  "1"  /* Unspecified B-cell lymphoma, lymph nodes of inguinal region and lower limb */
"C8516"  =  "1"  /* Unspecified B-cell lymphoma, intrapelvic lymph nodes */
"C8517"  =  "1"  /* Unspecified B-cell lymphoma, spleen */
"C8518"  =  "1"  /* Unspecified B-cell lymphoma, lymph nodes of multiple sites */
"C8519"  =  "1"  /* Unspecified B-cell lymphoma, extranodal and solid organ sites */
"C8520"  =  "1"  /* Mediastinal (thymic) large B-cell lymphoma, unspecified site */
"C8521"  =  "1"  /* Mediastinal (thymic) large B-cell lymphoma, lymph nodes of head, face, and neck */
"C8522"  =  "1"  /* Mediastinal (thymic) large B-cell lymphoma, intrathoracic lymph nodes */
"C8523"  =  "1"  /* Mediastinal (thymic) large B-cell lymphoma, intra-abdominal lymph nodes */
"C8524"  =  "1"  /* Mediastinal (thymic) large B-cell lymphoma, lymph nodes of axilla and upper limb */
"C8525"  =  "1"  /* Mediastinal (thymic) large B-cell lymphoma, lymph nodes of inguinal region and lower limb */
"C8526"  =  "1"  /* Mediastinal (thymic) large B-cell lymphoma, intrapelvic lymph nodes */
"C8527"  =  "1"  /* Mediastinal (thymic) large B-cell lymphoma, spleen */
"C8528"  =  "1"  /* Mediastinal (thymic) large B-cell lymphoma, lymph nodes of multiple sites */
"C8529"  =  "1"  /* Mediastinal (thymic) large B-cell lymphoma, extranodal and solid organ sites */
"C8580"  =  "1"  /* Other specified types of non-Hodgkin lymphoma, unspecified site */
"C8581"  =  "1"  /* Other specified types of non-Hodgkin lymphoma, lymph nodes of head, face, and neck */
"C8582"  =  "1"  /* Other specified types of non-Hodgkin lymphoma, intrathoracic lymph nodes */
"C8583"  =  "1"  /* Other specified types of non-Hodgkin lymphoma, intra-abdominal lymph nodes */
"C8584"  =  "1"  /* Other specified types of non-Hodgkin lymphoma, lymph nodes of axilla and upper limb */
"C8585"  =  "1"  /* Other specified types of non-Hodgkin lymphoma, lymph nodes of inguinal region and lower limb */
"C8586"  =  "1"  /* Other specified types of non-Hodgkin lymphoma, intrapelvic lymph nodes */
"C8587"  =  "1"  /* Other specified types of non-Hodgkin lymphoma, spleen */
"C8588"  =  "1"  /* Other specified types of non-Hodgkin lymphoma, lymph nodes of multiple sites */
"C8589"  =  "1"  /* Other specified types of non-Hodgkin lymphoma, extranodal and solid organ sites */
"C8590"  =  "1"  /* Non-Hodgkin lymphoma, unspecified, unspecified site */
"C8591"  =  "1"  /* Non-Hodgkin lymphoma, unspecified, lymph nodes of head, face, and neck */
"C8592"  =  "1"  /* Non-Hodgkin lymphoma, unspecified, intrathoracic lymph nodes */
"C8593"  =  "1"  /* Non-Hodgkin lymphoma, unspecified, intra-abdominal lymph nodes */
"C8594"  =  "1"  /* Non-Hodgkin lymphoma, unspecified, lymph nodes of axilla and upper limb */
"C8595"  =  "1"  /* Non-Hodgkin lymphoma, unspecified, lymph nodes of inguinal region and lower limb */
"C8596"  =  "1"  /* Non-Hodgkin lymphoma, unspecified, intrapelvic lymph nodes */
"C8597"  =  "1"  /* Non-Hodgkin lymphoma, unspecified, spleen */
"C8598"  =  "1"  /* Non-Hodgkin lymphoma, unspecified, lymph nodes of multiple sites */
"C8599"  =  "1"  /* Non-Hodgkin lymphoma, unspecified, extranodal and solid organ sites */
"C860"  =  "1"  /* Extranodal NK/T-cell lymphoma, nasal type */
"C861"  =  "1"  /* Hepatosplenic T-cell lymphoma */
"C862"  =  "1"  /* Enteropathy-type (intestinal) T-cell lymphoma */
"C863"  =  "1"  /* Subcutaneous panniculitis-like T-cell lymphoma */
"C864"  =  "1"  /* Blastic NK-cell lymphoma */
"C880"  =  "1"  /* Waldenstrom macroglobulinemia */
"C882"  =  "1"  /* Heavy chain disease */
"C883"  =  "1"  /* Immunoproliferative small intestinal disease */
"C888"  =  "1"  /* Other malignant immunoproliferative diseases */
"C889"  =  "1"  /* Malignant immunoproliferative disease, unspecified */
"C9000"  =  "1"  /* Multiple myeloma not having achieved remission */
"C9001"  =  "1"  /* Multiple myeloma in remission */
"C9010"  =  "1"  /* Plasma cell leukemia not having achieved remission */
"C9011"  =  "1"  /* Plasma cell leukemia in remission */
"C9020"  =  "1"  /* Extramedullary plasmacytoma not having achieved remission */
"C9021"  =  "1"  /* Extramedullary plasmacytoma in remission */
"C9030"  =  "1"  /* Solitary plasmacytoma not having achieved remission */
"C9031"  =  "1"  /* Solitary plasmacytoma in remission */
"C9100"  =  "1"  /* Acute lymphoblastic leukemia not having achieved remission */
"C9101"  =  "1"  /* Acute lymphoblastic leukemia, in remission */
"C9110"  =  "1"  /* Chronic lymphocytic leukemia of B-cell type not having achieved remission */
"C9111"  =  "1"  /* Chronic lymphocytic leukemia of B-cell type in remission */
"C9130"  =  "1"  /* Prolymphocytic leukemia of B-cell type not having achieved remission */
"C9131"  =  "1"  /* Prolymphocytic leukemia of B-cell type, in remission */
"C9140"  =  "1"  /* Hairy cell leukemia not having achieved remission */
"C9141"  =  "1"  /* Hairy cell leukemia, in remission */
"C9142"  =  "1"  /* Hairy cell leukemia, in relapse */
"C9150"  =  "1"  /* Adult T-cell lymphoma/leukemia (HTLV-1-associated) not having achieved remission */
"C9151"  =  "1"  /* Adult T-cell lymphoma/leukemia (HTLV-1-associated), in remission */
"C9160"  =  "1"  /* Prolymphocytic leukemia of T-cell type not having achieved remission */
"C9161"  =  "1"  /* Prolymphocytic leukemia of T-cell type, in remission */
"C9190"  =  "1"  /* Lymphoid leukemia, unspecified not having achieved remission */
"C9191"  =  "1"  /* Lymphoid leukemia, unspecified, in remission */
"C91A0"  =  "1"  /* Mature B-cell leukemia Burkitt-type not having achieved remission */
"C91A1"  =  "1"  /* Mature B-cell leukemia Burkitt-type, in remission */
"C91Z0"  =  "1"  /* Other lymphoid leukemia not having achieved remission */
"C91Z1"  =  "1"  /* Other lymphoid leukemia, in remission */
"C960"  =  "1"  /* Multifocal and multisystemic (disseminated) Langerhans-cell histiocytosis */
"C962"  =  "1"  /* Malignant mast cell tumor */
"C964"  =  "1"  /* Sarcoma of dendritic cells (accessory cells) */
"C969"  =  "1"  /* Malignant neoplasm of lymphoid, hematopoietic and related tissue, unspecified */
"C96A"  =  "1"  /* Histiocytic sarcoma */
"C96Z"  =  "1"  /* Other specified malignant neoplasms of lymphoid, hematopoietic and related tissue */
"D47Z9"  =  "1"  /* Other specified neoplasms of uncertain behavior of lymphoid, hematopoietic and related tissue */
OTHER  = "0" ;

/* STRATIFY BY: INTERMEDIATE RISK GROUP */
/* ICD-10-CM DIAGNOSIS CODES: */

/* HEMOPHILIA */

VALUE $HEMOPHD
"D66"  =  "1"  /* Hereditary factor VIII deficiency */
"D67"  =  "1"  /* Hereditary factor IX deficiency */
"D680"  =  "1"  /* Von Willebrand's disease */
"D681"  =  "1"  /* Hereditary factor XI deficiency */
"D682"  =  "1"  /* Hereditary deficiency of other clotting factors */
OTHER  = "0" ;


 /* -------------------------------- */
 /* TRANSFUSION REACTION             */
 /*   PROVIDER INDICATOR / PDI # 13  */
 /*   AREA     INDICATOR / PSI # TBD */
 /* -------------------------------- */

 /* ICD-10-CM DIAGNOSIS CODES: */

VALUE $TRANFID
"T8030XA"  =  "1"  /* ABO incompatibility reaction due to transfusion of blood or blood products, unspecified, initial encounter */
"T80310A"  =  "1"  /* ABO incompatibility with acute hemolytic transfusion reaction, initial encounter */
"T80311A"  =  "1"  /* ABO incompatibility with delayed hemolytic transfusion reaction, initial encounter */
"T80319A"  =  "1"  /* ABO incompatibility with hemolytic transfusion reaction, unspecified, initial encounter */
"T8039XA"  =  "1"  /* Other ABO incompatibility reaction due to transfusion of blood or blood products, initial encounter */
"T8040XA"  =  "1"  /* Rh incompatibility reaction due to transfusion of blood or blood products, unspecified, initial encounter */
"T80410A"  =  "1"  /* Rh incompatibility with acute hemolytic transfusion reaction, initial encounter */
"T80411A"  =  "1"  /* Rh incompatibility with delayed hemolytic transfusion reaction, initial encounter */
"T80419A"  =  "1"  /* Rh incompatibility with hemolytic transfusion reaction, unspecified, initial encounter */
"T8049XA"  =  "1"  /* Other Rh incompatibility reaction due to transfusion of blood or blood products, initial encounter */
"Y650"  =  "1"  /* Mismatched blood in transfusion */
OTHER   = "0" ;

 /* ------------------------------- */
 /* BIRTH TRAUMA                    */
 /*   PROVIDER INDICATOR / PSI # 17 */
 /* ------------------------------- */

 
 /* BIRTH TRAUMA */
 /* ICD-10-CM DIAGNOSIS CODES: */
 /* VALUE $BIRTIDA $BIRTIDB $BIRTIDC - no mapping found from UCD */
 
VALUE $BIRTHID
"P100"  =  "1"  /* Subdural hemorrhage due to birth injury */
"P101"  =  "1"  /* Cerebral hemorrhage due to birth injury */
"P102"  =  "1"  /* Intraventricular hemorrhage due to birth injury */
"P103"  =  "1"  /* Subarachnoid hemorrhage due to birth injury */
"P104"  =  "1"  /* Tentorial tear due to birth injury */
"P108"  =  "1"  /* Other intracranial lacerations and hemorrhages due to birth injury */
"P109"  =  "1"  /* Unspecified intracranial laceration and hemorrhage due to birth injury */
"P110"  =  "1"  /* Cerebral edema due to birth injury */
"P111"  =  "1"  /* Other specified brain damage due to birth injury */
"P112"  =  "1"  /* Unspecified brain damage due to birth injury */
"P113"  =  "1"  /* Birth injury to facial nerve */
"P114"  =  "1"  /* Birth injury to other cranial nerves */
"P115"  =  "1"  /* Birth injury to spine and spinal cord */
"P119"  =  "1"  /* Birth injury to central nervous system, unspecified */
"P122"  =  "1"  /* Epicranial subaponeurotic hemorrhage due to birth injury */
"P130"  =  "1"  /* Fracture of skull due to birth injury */
"P131"  =  "1"  /* Other birth injuries to skull */
"P132"  =  "1"  /* Birth injury to femur */
"P133"  =  "1"  /* Birth injury to other long bones */
"P138"  =  "1"  /* Birth injuries to other parts of skeleton */
"P139"  =  "1"  /* Birth injury to skeleton, unspecified */
"P142"  =  "1"  /* Phrenic nerve paralysis due to birth injury */
"P148"  =  "1"  /* Birth injuries to other parts of peripheral nervous system */
"P149"  =  "1"  /* Birth injury to peripheral nervous system, unspecified */
"P150"  =  "1"  /* Birth injury to liver */
"P151"  =  "1"  /* Birth injury to spleen */
"P152"  =  "1"  /* Sternomastoid injury due to birth injury */
"P153"  =  "1"  /* Birth injury to eye */
"P154"  =  "1"  /* Birth injury to face */
"P155"  =  "1"  /* Birth injury to external genitalia */
"P156"  =  "1"  /* Subcutaneous fat necrosis due to birth injury */
"P158"  =  "1"  /* Other specified birth injuries */
OTHER   = 0 ;

 /* BIRTH TRAUMA */
 /* ICD-10-CM DIAGNOSIS CODES */

VALUE $LIVEBND
"Z3800"  =  "1"  /* Single liveborn infant, delivered vaginally */
"Z3801"  =  "1"  /* Single liveborn infant, delivered by cesarean */
"Z3830"  =  "1"  /* Twin liveborn infant, delivered vaginally */
"Z3831"  =  "1"  /* Twin liveborn infant, delivered by cesarean */
"Z3861"  =  "1"  /* Triplet liveborn infant, delivered vaginally */
"Z3862"  =  "1"  /* Triplet liveborn infant, delivered by cesarean */
"Z3863"  =  "1"  /* Quadruplet liveborn infant, delivered vaginally */
"Z3864"  =  "1"  /* Quadruplet liveborn infant, delivered by cesarean */
"Z3865"  =  "1"  /* Quintuplet liveborn infant, delivered vaginally */
"Z3866"  =  "1"  /* Quintuplet liveborn infant, delivered by cesarean */
"Z3868"  =  "1"  /* Other multiple liveborn infant, delivered vaginally */
"Z3869"  =  "1"  /* Other multiple liveborn infant, delivered by cesarean */
OTHER   = 0 ;

/* VALUE $V29D per UCD - There are no acceptable ICD-10-CM codes */

VALUE $LIVEB2D
"Z381"  =  "1"  /* Single liveborn infant, born outside hospital */
"Z384"  =  "1"  /* Twin liveborn infant, born outside hospital */
"Z387"  =  "1"  /* Other multiple liveborn infant, born outside hospital */
OTHER   = 0 ;

 
 /* EXCLUDE: PRETERM INFANT */
 /* ICD-10-CM DIAGNOSIS CODES: */

VALUE $PRETEID
"P0700"  =  "1"  /* Extremely low birth weight newborn, unspecified weight */
"P0701"  =  "1"  /* Extremely low birth weight newborn, less than 500 grams */
"P0702"  =  "1"  /* Extremely low birth weight newborn, 500-749 grams */
"P0703"  =  "1"  /* Extremely low birth weight newborn, 750-999 grams */
"P0710"  =  "1"  /* Other low birth weight newborn, unspecified weight */
"P0714"  =  "1"  /* Other low birth weight newborn, 1000-1249 grams */
"P0715"  =  "1"  /* Other low birth weight newborn, 1250-1499 grams */
"P0716"  =  "1"  /* Other low birth weight newborn, 1500-1749 grams */
"P0717"  =  "1"  /* Other low birth weight newborn, 1750-1999 grams */
OTHER   = "0" ;

 /* EXCLUDE: INJURY BRACHIAL PLEXUS */;
 /* ICD-10-CM DIAGNOSIS CODES: */

VALUE $BRACHID
"P140"  =  "1"  /* Erb's paralysis due to birth injury */
"P141"  =  "1"  /* Klumpke's paralysis due to birth injury */
"P143"  =  "1"  /* Other brachial plexus birth injuries */
OTHER   = "0" ;

 /* EXCLUDE: OSTEOGENESIS IMPERFECTA */;
 /* ICD-10-CM DIAGNOSIS CODES: */

VALUE $OSTEOID
"Q780"  =  "1"  /* Osteogenesis imperfecta */
OTHER   = "0" ;


 /* --------------------------------------------------------------- */
 /* ---   NEONATAL MORTALITY                                      - */
 /*          PROVIDER INDICATOR / NQI # 2                         - */
 /* --------------------------------------------------------------- */

VALUE $NEOMTDX
"Q000"  =  "1"  /* Anencephaly */
"Q001"  =  "1"  /* Craniorachischisis */
"Q002"  =  "1"  /* Iniencephaly */
"Q6111"  =  "1"  /* Cystic dilatation of collecting ducts */
"Q6119"  =  "1"  /* Other polycystic kidney, infantile type */
"Q910"  =  "1"  /* Trisomy 18, nonmosaicism (meiotic nondisjunction) */
"Q911"  =  "1"  /* Trisomy 18, mosaicism (mitotic nondisjunction) */
"Q912"  =  "1"  /* Trisomy 18, translocation */
"Q913"  =  "1"  /* Trisomy 18, unspecified */
"Q914"  =  "1"  /* Trisomy 13, nonmosaicism (meiotic nondisjunction) */
"Q915"  =  "1"  /* Trisomy 13, mosaicism (mitotic nondisjunction) */
"Q916"  =  "1"  /* Trisomy 13, translocation */
"Q917"  =  "1"  /* Trisomy 13, unspecified */
OTHER   = "0";

 /* ---------------------------------------------------------------- */
 /* --- BLOOD STREAM INFECTION NEONATES                            - */
 /*        PROVIDER INDICATOR / NQI # 3                            - */
 /* ---------------------------------------------------------------- */

VALUE $GESTCD
"P0720"  =  "1"  /* Extreme immaturity of newborn, unspecified weeks of gestation */
"P0723"  =  "1"  /* Extreme immaturity of newborn, gestational age 24 completed weeks */
"P0724"  =  "1"  /* Extreme immaturity of newborn, gestational age 25 completed weeks */
"P0725"  =  "1"  /* Extreme immaturity of newborn, gestational age 26 completed weeks */
"P0726"  =  "1"  /* Extreme immaturity of newborn, gestational age 27 completed weeks */
"P0731"  =  "1"  /* Preterm newborn, gestational age 28 completed weeks */
"P0732"  =  "1"  /* Preterm newborn, gestational age 29 completed weeks */
"P0733"  =  "1"  /* Preterm newborn, gestational age 30 completed weeks */
OTHER   = "0";

VALUE $GESTC1D
"P0721"  =  "1"  /* Extreme immaturity of newborn, gestational age less than 23 completed weeks */
"P0722"  =  "1"  /* Extreme immaturity of newborn, gestational age 23 completed weeks */
OTHER   = "0";
     
VALUE $GESTC2D
"P0723"  =  "1"  /* Extreme immaturity of newborn, gestational age 24 completed weeks */
OTHER   = "0";

VALUE $GESTC3D
"P0724"  =  "1"  /* Extreme immaturity of newborn, gestational age 25 completed weeks */
"P0725"  =  "1"  /* Extreme immaturity of newborn, gestational age 26 completed weeks */
OTHER   = "0";

VALUE $GESTC4D
"P0726"  =  "1"  /* Extreme immaturity of newborn, gestational age 27 completed weeks */
"P0731"  =  "1"  /* Preterm newborn, gestational age 28 completed weeks */
OTHER   = "0";

VALUE $GESTC5D
"P0732"  =  "1"  /* Preterm newborn, gestational age 29 completed weeks */
"P0733"  =  "1"  /* Preterm newborn, gestational age 30 completed weeks */
OTHER   = "0";

VALUE $GESTC6D
"P0734"  =  "1"  /* Preterm newborn, gestational age 31 completed weeks */
"P0735"  =  "1"  /* Preterm newborn, gestational age 32 completed weeks */
OTHER   = "0";

VALUE $GESTC7D
"P0736"  =  "1"  /* Preterm newborn, gestational age 33 completed weeks */
"P0737"  =  "1"  /* Preterm newborn, gestational age 34 completed weeks */
OTHER   = "0";

VALUE $GESTC8D
"P0738"  =  "1"  /* Preterm newborn, gestational age 35 completed weeks */
"P0739"  =  "1"  /* Preterm newborn, gestational age 36 completed weeks */
OTHER   = "0";

/* VALUE $GESTC9D per UCD - ICD-10-CM does not have information on gestational age from 37 through 39 weeks */

VALUE $MECHVCD
"5A1935Z"  =  "1"  /* Respiratory Ventilation, Less than 24 Consecutive Hours */
"5A1945Z"  =  "1"  /* Respiratory Ventilation, 24-96 Consecutive Hours */
"5A1955Z"  =  "1"  /* Respiratory Ventilation, Greater than 96 Consecutive Hours */
OTHER   = "0";

/* VALUE $BSI1DX per UCD - The mapped ICD-10-CM A41 codes are specific to the non-neonatal age group.  The A41 codes specifically excluded neonatal sepsis. */

VALUE $BSI2DX
"P3619"  =  "1"  /* Sepsis of newborn due to other streptococci */
"P362"  =  "1"  /* Sepsis of newborn due to Staphylococcus aureus */
"P3630"  =  "1"  /* Sepsis of newborn due to unspecified staphylococci */
"P3639"  =  "1"  /* Sepsis of newborn due to other staphylococci */
"P364"  =  "1"  /* Sepsis of newborn due to Escherichia coli */
"P368"  =  "1"  /* Other bacterial sepsis of newborn */
"R7881"  =  "1"  /* Bacteremia */
OTHER   = "0" ;

/* BACTERIAL AGENT                         */
/* ICD-10-CM Diagnosis Codes: */

VALUE $BSI3DX
"B961"  =  "1"  /* Klebsiella pneumoniae [K. pneumoniae] as the cause of diseases classified elsewhere */
"B9620"  =  "1"  /* Unspecified Escherichia coli [E. coli] as the cause of diseases classified elsewhere */
"B9621"  =  "1"  /* Shiga toxin-producing Escherichia coli [E. coli] (STEC) O157 as the cause of diseases classified elsewhere */
"B9622"  =  "1"  /* Other specified Shiga toxin-producing Escherichia coli [E. coli] (STEC) as the cause of diseases classified elsewhere */
"B9623"  =  "1"  /* Unspecified Shiga toxin-producing Escherichia coli [E. coli] (STEC) as the cause of diseases classified elsewhere */
"B9629"  =  "1"  /* Other Escherichia coli [E. coli] as the cause of diseases classified elsewhere */
"B965"  =  "1"  /* Pseudomonas (aeruginosa) (mallei) (pseudomallei) as the cause of diseases classified elsewhere */
"B9689"  =  "1"  /* Other specified bacterial agents as the cause of diseases classified elsewhere */
OTHER   = "0" ;

VALUE $BSI4DX
"A021"  =  "1"  /* Salmonella sepsis */
"A227"  =  "1"  /* Anthrax sepsis */
"A267"  =  "1"  /* Erysipelothrix sepsis */
"A327"  =  "1"  /* Listerial sepsis */
"A400"  =  "1"  /* Sepsis due to streptococcus, group A */
"A401"  =  "1"  /* Sepsis due to streptococcus, group B */
"A403"  =  "1"  /* Sepsis due to Streptococcus pneumoniae */
"A408"  =  "1"  /* Other streptococcal sepsis */
"A409"  =  "1"  /* Streptococcal sepsis, unspecified */
"A4101"  =  "1"  /* Sepsis due to Methicillin susceptible Staphylococcus aureus */
"A4102"  =  "1"  /* Sepsis due to Methicillin resistant Staphylococcus aureus */
"A411"  =  "1"  /* Sepsis due to other specified staphylococcus */
"A412"  =  "1"  /* Sepsis due to unspecified staphylococcus */
"A413"  =  "1"  /* Sepsis due to Hemophilus influenzae */
"A414"  =  "1"  /* Sepsis due to anaerobes */
"A4150"  =  "1"  /* Gram-negative sepsis, unspecified */
"A4151"  =  "1"  /* Sepsis due to Escherichia coli [E. coli] */
"A4152"  =  "1"  /* Sepsis due to Pseudomonas */
"A4153"  =  "1"  /* Sepsis due to Serratia */
"A4159"  =  "1"  /* Other Gram-negative sepsis */
"A4181"  =  "1"  /* Sepsis due to Enterococcus */
"A4189"  =  "1"  /* Other specified sepsis */
"A419"  =  "1"  /* Sepsis, unspecified organism */
"A427"  =  "1"  /* Actinomycotic sepsis */
"A5486"  =  "1"  /* Gonococcal sepsis */
"B377"  =  "1"  /* Candidal sepsis */
"P360"  =  "1"  /* Sepsis of newborn due to streptococcus, group B */
"P3610"  =  "1"  /* Sepsis of newborn due to unspecified streptococci */
"P3619"  =  "1"  /* Sepsis of newborn due to other streptococci */
"P362"  =  "1"  /* Sepsis of newborn due to Staphylococcus aureus */
"P3630"  =  "1"  /* Sepsis of newborn due to unspecified staphylococci */
"P3639"  =  "1"  /* Sepsis of newborn due to other staphylococci */
"P364"  =  "1"  /* Sepsis of newborn due to Escherichia coli */
"P365"  =  "1"  /* Sepsis of newborn due to anaerobes */
"P368"  =  "1"  /* Other bacterial sepsis of newborn */
"P369"  =  "1"  /* Bacterial sepsis of newborn, unspecified */
"R6520"  =  "1"  /* Severe sepsis without septic shock */
"R6521"  =  "1"  /* Severe sepsis with septic shock */
"R7881"  =  "1"  /* Bacteremia */
"T8112XA"  =  "1"  /* Postprocedural septic shock, initial encounter */
OTHER   = "0" ;

 /* VALUE $BSI5DX no ICD-10 mapping */


/*  Multiple Birth                    */
/* ICD-10-CM Diagnosis Codes: */

VALUE $MULTIPD
"O30001"  =  "1"  /* Twin pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, first trimester */
"O30002"  =  "1"  /* Twin pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, second trimester */
"O30003"  =  "1"  /* Twin pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, third trimester */
"O30009"  =  "1"  /* Twin pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, unspecified trimester */
"O30011"  =  "1"  /* Twin pregnancy, monochorionic/monoamniotic, first trimester */
"O30012"  =  "1"  /* Twin pregnancy, monochorionic/monoamniotic, second trimester */
"O30013"  =  "1"  /* Twin pregnancy, monochorionic/monoamniotic, third trimester */
"O30019"  =  "1"  /* Twin pregnancy, monochorionic/monoamniotic, unspecified trimester */
"O30031"  =  "1"  /* Twin pregnancy, monochorionic/diamniotic, first trimester */
"O30032"  =  "1"  /* Twin pregnancy, monochorionic/diamniotic, second trimester */
"O30033"  =  "1"  /* Twin pregnancy, monochorionic/diamniotic, third trimester */
"O30039"  =  "1"  /* Twin pregnancy, monochorionic/diamniotic, unspecified trimester */
"O30041"  =  "1"  /* Twin pregnancy, dichorionic/diamniotic, first trimester */
"O30042"  =  "1"  /* Twin pregnancy, dichorionic/diamniotic, second trimester */
"O30043"  =  "1"  /* Twin pregnancy, dichorionic/diamniotic, third trimester */
"O30049"  =  "1"  /* Twin pregnancy, dichorionic/diamniotic, unspecified trimester */
"O30091"  =  "1"  /* Twin pregnancy, unable to determine number of placenta and number of amniotic sacs, first trimester */
"O30092"  =  "1"  /* Twin pregnancy, unable to determine number of placenta and number of amniotic sacs, second trimester */
"O30093"  =  "1"  /* Twin pregnancy, unable to determine number of placenta and number of amniotic sacs, third trimester */
"O30099"  =  "1"  /* Twin pregnancy, unable to determine number of placenta and number of amniotic sacs, unspecified trimester */
"O30101"  =  "1"  /* Triplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, first trimester */
"O30102"  =  "1"  /* Triplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, second trimester */
"O30103"  =  "1"  /* Triplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, third trimester */
"O30109"  =  "1"  /* Triplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, unspecified trimester */
"O30111"  =  "1"  /* Triplet pregnancy with two or more monochorionic fetuses, first trimester */
"O30112"  =  "1"  /* Triplet pregnancy with two or more monochorionic fetuses, second trimester */
"O30113"  =  "1"  /* Triplet pregnancy with two or more monochorionic fetuses, third trimester */
"O30119"  =  "1"  /* Triplet pregnancy with two or more monochorionic fetuses, unspecified trimester */
"O30121"  =  "1"  /* Triplet pregnancy with two or more monoamniotic fetuses, first trimester */
"O30122"  =  "1"  /* Triplet pregnancy with two or more monoamniotic fetuses, second trimester */
"O30123"  =  "1"  /* Triplet pregnancy with two or more monoamniotic fetuses, third trimester */
"O30129"  =  "1"  /* Triplet pregnancy with two or more monoamniotic fetuses, unspecified trimester */
"O30191"  =  "1"  /* Triplet pregnancy, unable to determine number of placenta and number of amniotic sacs, first trimester */
"O30192"  =  "1"  /* Triplet pregnancy, unable to determine number of placenta and number of amniotic sacs, second trimester */
"O30193"  =  "1"  /* Triplet pregnancy, unable to determine number of placenta and number of amniotic sacs, third trimester */
"O30199"  =  "1"  /* Triplet pregnancy, unable to determine number of placenta and number of amniotic sacs, unspecified trimester */
"O30201"  =  "1"  /* Quadruplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, first trimester */
"O30202"  =  "1"  /* Quadruplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, second trimester */
"O30203"  =  "1"  /* Quadruplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, third trimester */
"O30209"  =  "1"  /* Quadruplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, unspecified trimester */
"O30211"  =  "1"  /* Quadruplet pregnancy with two or more monochorionic fetuses, first trimester */
"O30212"  =  "1"  /* Quadruplet pregnancy with two or more monochorionic fetuses, second trimester */
"O30213"  =  "1"  /* Quadruplet pregnancy with two or more monochorionic fetuses, third trimester */
"O30219"  =  "1"  /* Quadruplet pregnancy with two or more monochorionic fetuses, unspecified trimester */
"O30221"  =  "1"  /* Quadruplet pregnancy with two or more monoamniotic fetuses, first trimester */
"O30222"  =  "1"  /* Quadruplet pregnancy with two or more monoamniotic fetuses, second trimester */
"O30223"  =  "1"  /* Quadruplet pregnancy with two or more monoamniotic fetuses, third trimester */
"O30229"  =  "1"  /* Quadruplet pregnancy with two or more monoamniotic fetuses, unspecified trimester */
"O30291"  =  "1"  /* Quadruplet pregnancy, unable to determine number of placenta and number of amniotic sacs, first trimester */
"O30292"  =  "1"  /* Quadruplet pregnancy, unable to determine number of placenta and number of amniotic sacs, second trimester */
"O30293"  =  "1"  /* Quadruplet pregnancy, unable to determine number of placenta and number of amniotic sacs, third trimester */
"O30299"  =  "1"  /* Quadruplet pregnancy, unable to determine number of placenta and number of amniotic sacs, unspecified trimester */
"O30801"  =  "1"  /* Other specified multiple gestation, unspecified number of placenta and unspecified number of amniotic sacs, first trimester */
"O30802"  =  "1"  /* Other specified multiple gestation, unspecified number of placenta and unspecified number of amniotic sacs, second trimester */
"O30803"  =  "1"  /* Other specified multiple gestation, unspecified number of placenta and unspecified number of amniotic sacs, third trimester */
"O30809"  =  "1"  /* Other specified multiple gestation, unspecified number of placenta and unspecified number of amniotic sacs, unspecified trimester */
"O30811"  =  "1"  /* Other specified multiple gestation with two or more monochorionic fetuses, first trimester */
"O30812"  =  "1"  /* Other specified multiple gestation with two or more monochorionic fetuses, second trimester */
"O30813"  =  "1"  /* Other specified multiple gestation with two or more monochorionic fetuses, third trimester */
"O30819"  =  "1"  /* Other specified multiple gestation with two or more monochorionic fetuses, unspecified trimester */
"O30821"  =  "1"  /* Other specified multiple gestation with two or more monoamniotic fetuses, first trimester */
"O30822"  =  "1"  /* Other specified multiple gestation with two or more monoamniotic fetuses, second trimester */
"O30823"  =  "1"  /* Other specified multiple gestation with two or more monoamniotic fetuses, third trimester */
"O30829"  =  "1"  /* Other specified multiple gestation with two or more monoamniotic fetuses, unspecified trimester */
"O30891"  =  "1"  /* Other specified multiple gestation, unable to determine number of placenta and number of amniotic sacs, first trimester */
"O30892"  =  "1"  /* Other specified multiple gestation, unable to determine number of placenta and number of amniotic sacs, second trimester */
"O30893"  =  "1"  /* Other specified multiple gestation, unable to determine number of placenta and number of amniotic sacs, third trimester */
"O30899"  =  "1"  /* Other specified multiple gestation, unable to determine number of placenta and number of amniotic sacs, unspecified trimester */
"O3090"  =  "1"  /* Multiple gestation, unspecified, unspecified trimester */
"O3091"  =  "1"  /* Multiple gestation, unspecified, first trimester */
"O3092"  =  "1"  /* Multiple gestation, unspecified, second trimester */
"O3093"  =  "1"  /* Multiple gestation, unspecified, third trimester */
"O3110X0"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, not applicable or unspecified */
"O3110X1"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 1 */
"O3110X2"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 2 */
"O3110X3"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 3 */
"O3110X4"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 4 */
"O3110X5"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 5 */
"O3110X9"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, other fetus */
"O3111X0"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, not applicable or unspecified */
"O3111X1"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 1 */
"O3111X2"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 2 */
"O3111X3"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 3 */
"O3111X4"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 4 */
"O3111X5"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 5 */
"O3111X9"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, other fetus */
"O3112X0"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, not applicable or unspecified */
"O3112X1"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 1 */
"O3112X2"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 2 */
"O3112X3"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 3 */
"O3112X4"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 4 */
"O3112X5"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 5 */
"O3112X9"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, other fetus */
"O3113X0"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, not applicable or unspecified */
"O3113X1"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 1 */
"O3113X2"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 2 */
"O3113X3"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 3 */
"O3113X4"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 4 */
"O3113X5"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 5 */
"O3113X9"  =  "1"  /* Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, other fetus */
"O3120X0"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, not applicable or unspecified */
"O3120X1"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 1 */
"O3120X2"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 2 */
"O3120X3"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 3 */
"O3120X4"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 4 */
"O3120X5"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 5 */
"O3120X9"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, other fetus */
"O3121X0"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, first trimester, not applicable or unspecified */
"O3121X1"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 1 */
"O3121X2"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 2 */
"O3121X3"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 3 */
"O3121X4"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 4 */
"O3121X5"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 5 */
"O3121X9"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, first trimester, other fetus */
"O3122X0"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, second trimester, not applicable or unspecified */
"O3122X1"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 1 */
"O3122X2"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 2 */
"O3122X3"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 3 */
"O3122X4"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 4 */
"O3122X5"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 5 */
"O3122X9"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, second trimester, other fetus */
"O3123X0"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, third trimester, not applicable or unspecified */
"O3123X1"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 1 */
"O3123X2"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 2 */
"O3123X3"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 3 */
"O3123X4"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 4 */
"O3123X5"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 5 */
"O3123X9"  =  "1"  /* Continuing pregnancy after intrauterine death of one fetus or more, third trimester, other fetus */
"O318X10"  =  "1"  /* Other complications specific to multiple gestation, first trimester, not applicable or unspecified */
"O318X11"  =  "1"  /* Other complications specific to multiple gestation, first trimester, fetus 1 */
"O318X12"  =  "1"  /* Other complications specific to multiple gestation, first trimester, fetus 2 */
"O318X13"  =  "1"  /* Other complications specific to multiple gestation, first trimester, fetus 3 */
"O318X14"  =  "1"  /* Other complications specific to multiple gestation, first trimester, fetus 4 */
"O318X15"  =  "1"  /* Other complications specific to multiple gestation, first trimester, fetus 5 */
"O318X19"  =  "1"  /* Other complications specific to multiple gestation, first trimester, other fetus */
"O318X20"  =  "1"  /* Other complications specific to multiple gestation, second trimester, not applicable or unspecified */
"O318X21"  =  "1"  /* Other complications specific to multiple gestation, second trimester, fetus 1 */
"O318X22"  =  "1"  /* Other complications specific to multiple gestation, second trimester, fetus 2 */
"O318X23"  =  "1"  /* Other complications specific to multiple gestation, second trimester, fetus 3 */
"O318X24"  =  "1"  /* Other complications specific to multiple gestation, second trimester, fetus 4 */
"O318X25"  =  "1"  /* Other complications specific to multiple gestation, second trimester, fetus 5 */
"O318X29"  =  "1"  /* Other complications specific to multiple gestation, second trimester, other fetus */
"O318X30"  =  "1"  /* Other complications specific to multiple gestation, third trimester, not applicable or unspecified */
"O318X31"  =  "1"  /* Other complications specific to multiple gestation, third trimester, fetus 1 */
"O318X32"  =  "1"  /* Other complications specific to multiple gestation, third trimester, fetus 2 */
"O318X33"  =  "1"  /* Other complications specific to multiple gestation, third trimester, fetus 3 */
"O318X34"  =  "1"  /* Other complications specific to multiple gestation, third trimester, fetus 4 */
"O318X35"  =  "1"  /* Other complications specific to multiple gestation, third trimester, fetus 5 */
"O318X39"  =  "1"  /* Other complications specific to multiple gestation, third trimester, other fetus */
"O318X90"  =  "1"  /* Other complications specific to multiple gestation, unspecified trimester, not applicable or unspecified */
"O318X91"  =  "1"  /* Other complications specific to multiple gestation, unspecified trimester, fetus 1 */
"O318X92"  =  "1"  /* Other complications specific to multiple gestation, unspecified trimester, fetus 2 */
"O318X93"  =  "1"  /* Other complications specific to multiple gestation, unspecified trimester, fetus 3 */
"O318X94"  =  "1"  /* Other complications specific to multiple gestation, unspecified trimester, fetus 4 */
"O318X95"  =  "1"  /* Other complications specific to multiple gestation, unspecified trimester, fetus 5 */
"O318X99"  =  "1"  /* Other complications specific to multiple gestation, unspecified trimester, other fetus */
"O329XX0"  =  "1"  /* Maternal care for malpresentation of fetus, unspecified, not applicable or unspecified */
"O329XX1"  =  "1"  /* Maternal care for malpresentation of fetus, unspecified, fetus 1 */
"O329XX2"  =  "1"  /* Maternal care for malpresentation of fetus, unspecified, fetus 2 */
"O329XX3"  =  "1"  /* Maternal care for malpresentation of fetus, unspecified, fetus 3 */
"O329XX4"  =  "1"  /* Maternal care for malpresentation of fetus, unspecified, fetus 4 */
"O329XX5"  =  "1"  /* Maternal care for malpresentation of fetus, unspecified, fetus 5 */
"O329XX9"  =  "1"  /* Maternal care for malpresentation of fetus, unspecified, other fetus */
"O632"  =  "1"  /* Delayed delivery of second twin, triplet, etc. */
"O661"  =  "1"  /* Obstructed labor due to locked twins */
"O666"  =  "1"  /* Obstructed labor due to other multiple fetuses */
"Z371"  =  "1"  /* Single stillbirth */
"Z372"  =  "1"  /* Twins, both liveborn */
"Z373"  =  "1"  /* Twins, one liveborn and one stillborn */
"Z374"  =  "1"  /* Twins, both stillborn */
"Z3750"  =  "1"  /* Multiple births, unspecified, all liveborn */
"Z3751"  =  "1"  /* Triplets, all liveborn */
"Z3752"  =  "1"  /* Quadruplets, all liveborn */
"Z3753"  =  "1"  /* Quintuplets, all liveborn */
"Z3754"  =  "1"  /* Sextuplets, all liveborn */
"Z3759"  =  "1"  /* Other multiple births, all liveborn */
"Z3760"  =  "1"  /* Multiple births, unspecified, some liveborn */
"Z3761"  =  "1"  /* Triplets, some liveborn */
"Z3762"  =  "1"  /* Quadruplets, some liveborn */
"Z3763"  =  "1"  /* Quintuplets, some liveborn */
"Z3764"  =  "1"  /* Sextuplets, some liveborn */
"Z3769"  =  "1"  /* Other multiple births, some liveborn */
"Z377"  =  "1"  /* Other multiple births, all stillborn */
OTHER  = '0' ;

/*  Congenital Anamolies */
/* ICD-10-CM Diagnosis Codes: */

VALUE $CONGN1D
"K562"  =  "1"  /* Volvulus */
"P351"  =  "1"  /* Congenital cytomegalovirus infection */
"Q390"  =  "1"  /* Atresia of esophagus without fistula */
"Q391"  =  "1"  /* Atresia of esophagus with tracheo-esophageal fistula */
"Q392"  =  "1"  /* Congenital tracheo-esophageal fistula without atresia */
"Q393"  =  "1"  /* Congenital stenosis and stricture of esophagus */
"Q394"  =  "1"  /* Esophageal web */
"Q395"  =  "1"  /* Congenital dilatation of esophagus */
"Q396"  =  "1"  /* Congenital diverticulum of esophagus */
"Q398"  =  "1"  /* Other congenital malformations of esophagus */
"Q399"  =  "1"  /* Congenital malformation of esophagus, unspecified */
"Q400"  =  "1"  /* Congenital hypertrophic pyloric stenosis */
"Q402"  =  "1"  /* Other specified congenital malformations of stomach */
"Q403"  =  "1"  /* Congenital malformation of stomach, unspecified */
"Q408"  =  "1"  /* Other specified congenital malformations of upper alimentary tract */
"Q409"  =  "1"  /* Congenital malformation of upper alimentary tract, unspecified */
"Q410"  =  "1"  /* Congenital absence, atresia and stenosis of duodenum */
"Q411"  =  "1"  /* Congenital absence, atresia and stenosis of jejunum */
"Q412"  =  "1"  /* Congenital absence, atresia and stenosis of ileum */
"Q418"  =  "1"  /* Congenital absence, atresia and stenosis of other specified parts of small intestine */
"Q419"  =  "1"  /* Congenital absence, atresia and stenosis of small intestine, part unspecified */
"Q420"  =  "1"  /* Congenital absence, atresia and stenosis of rectum with fistula */
"Q421"  =  "1"  /* Congenital absence, atresia and stenosis of rectum without fistula */
"Q422"  =  "1"  /* Congenital absence, atresia and stenosis of anus with fistula */
"Q423"  =  "1"  /* Congenital absence, atresia and stenosis of anus without fistula */
"Q428"  =  "1"  /* Congenital absence, atresia and stenosis of other parts of large intestine */
"Q429"  =  "1"  /* Congenital absence, atresia and stenosis of large intestine, part unspecified */
"Q430"  =  "1"  /* Meckel's diverticulum (displaced) (hypertrophic) */
"Q431"  =  "1"  /* Hirschsprung's disease */
"Q432"  =  "1"  /* Other congenital functional disorders of colon */
"Q433"  =  "1"  /* Congenital malformations of intestinal fixation */
"Q434"  =  "1"  /* Duplication of intestine */
"Q435"  =  "1"  /* Ectopic anus */
"Q436"  =  "1"  /* Congenital fistula of rectum and anus */
"Q437"  =  "1"  /* Persistent cloaca */
"Q438"  =  "1"  /* Other specified congenital malformations of intestine */
"Q439"  =  "1"  /* Congenital malformation of intestine, unspecified */
"Q440"  =  "1"  /* Agenesis, aplasia and hypoplasia of gallbladder */
"Q441"  =  "1"  /* Other congenital malformations of gallbladder */
"Q442"  =  "1"  /* Atresia of bile ducts */
"Q443"  =  "1"  /* Congenital stenosis and stricture of bile ducts */
"Q444"  =  "1"  /* Choledochal cyst */
"Q445"  =  "1"  /* Other congenital malformations of bile ducts */
"Q447"  =  "1"  /* Other congenital malformations of liver */
"Q450"  =  "1"  /* Agenesis, aplasia and hypoplasia of pancreas */
"Q451"  =  "1"  /* Annular pancreas */
"Q452"  =  "1"  /* Congenital pancreatic cyst */
"Q453"  =  "1"  /* Other congenital malformations of pancreas and pancreatic duct */
"Q458"  =  "1"  /* Other specified congenital malformations of digestive system */
"Q459"  =  "1"  /* Congenital malformation of digestive system, unspecified */
"Q7959"  =  "1"  /* Other congenital malformations of abdominal wall */
OTHER   = '0' ;

VALUE $CONGN2D
"Q600"  =  "1"  /* Renal agenesis, unilateral */
"Q601"  =  "1"  /* Renal agenesis, bilateral */
"Q602"  =  "1"  /* Renal agenesis, unspecified */
"Q603"  =  "1"  /* Renal hypoplasia, unilateral */
"Q604"  =  "1"  /* Renal hypoplasia, bilateral */
"Q605"  =  "1"  /* Renal hypoplasia, unspecified */
"Q606"  =  "1"  /* Potter's syndrome */
"Q6111"  =  "1"  /* Cystic dilatation of collecting ducts */
"Q6119"  =  "1"  /* Other polycystic kidney, infantile type */
"Q613"  =  "1"  /* Polycystic kidney, unspecified */
"Q614"  =  "1"  /* Renal dysplasia */
"Q618"  =  "1"  /* Other cystic kidney diseases */
"Q619"  =  "1"  /* Cystic kidney disease, unspecified */
"Q6211"  =  "1"  /* Congenital occlusion of ureteropelvic junction */
"Q6212"  =  "1"  /* Congenital occlusion of ureterovesical orifice */
"Q622"  =  "1"  /* Congenital megaureter */
"Q6231"  =  "1"  /* Congenital ureterocele, orthotopic */
"Q6232"  =  "1"  /* Cecoureterocele */
"Q6239"  =  "1"  /* Other obstructive defects of renal pelvis and ureter */
"Q624"  =  "1"  /* Agenesis of ureter */
"Q625"  =  "1"  /* Duplication of ureter */
"Q6260"  =  "1"  /* Malposition of ureter, unspecified */
"Q6261"  =  "1"  /* Deviation of ureter */
"Q6262"  =  "1"  /* Displacement of ureter */
"Q6263"  =  "1"  /* Anomalous implantation of ureter */
"Q6269"  =  "1"  /* Other malposition of ureter */
"Q627"  =  "1"  /* Congenital vesico-uretero-renal reflux */
"Q628"  =  "1"  /* Other congenital malformations of ureter */
"Q630"  =  "1"  /* Accessory kidney */
"Q631"  =  "1"  /* Lobulated, fused and horseshoe kidney */
"Q632"  =  "1"  /* Ectopic kidney */
"Q633"  =  "1"  /* Hyperplastic and giant kidney */
"Q638"  =  "1"  /* Other specified congenital malformations of kidney */
"Q639"  =  "1"  /* Congenital malformation of kidney, unspecified */
"Q6411"  =  "1"  /* Supravesical fissure of urinary bladder */
"Q642"  =  "1"  /* Congenital posterior urethral valves */
"Q6431"  =  "1"  /* Congenital bladder neck obstruction */
"Q6432"  =  "1"  /* Congenital stricture of urethra */
"Q6433"  =  "1"  /* Congenital stricture of urinary meatus */
"Q6439"  =  "1"  /* Other atresia and stenosis of urethra and bladder neck */
"Q644"  =  "1"  /* Malformation of urachus */
"Q645"  =  "1"  /* Congenital absence of bladder and urethra */
"Q646"  =  "1"  /* Congenital diverticulum of bladder */
"Q6470"  =  "1"  /* Unspecified congenital malformation of bladder and urethra */
"Q6471"  =  "1"  /* Congenital prolapse of urethra */
"Q6472"  =  "1"  /* Congenital prolapse of urinary meatus */
"Q6473"  =  "1"  /* Congenital urethrorectal fistula */
"Q6474"  =  "1"  /* Double urethra */
"Q6475"  =  "1"  /* Double urinary meatus */
"Q6479"  =  "1"  /* Other congenital malformations of bladder and urethra */
"Q648"  =  "1"  /* Other specified congenital malformations of urinary system */
"Q649"  =  "1"  /* Congenital malformation of urinary system, unspecified */
"Q794"  =  "1"  /* Prune belly syndrome */
"Q7951"  =  "1"  /* Congenital hernia of bladder */
OTHER   = '0' ;

VALUE $CONGN3D
"Q010"  =  "1"  /* Frontal encephalocele */
"Q011"  =  "1"  /* Nasofrontal encephalocele */
"Q012"  =  "1"  /* Occipital encephalocele */
"Q018"  =  "1"  /* Encephalocele of other sites */
"Q019"  =  "1"  /* Encephalocele, unspecified */
"Q02"  =  "1"  /* Microcephaly */
"Q030"  =  "1"  /* Malformations of aqueduct of Sylvius */
"Q031"  =  "1"  /* Atresia of foramina of Magendie and Luschka */
"Q038"  =  "1"  /* Other congenital hydrocephalus */
"Q039"  =  "1"  /* Congenital hydrocephalus, unspecified */
"Q040"  =  "1"  /* Congenital malformations of corpus callosum */
"Q041"  =  "1"  /* Arhinencephaly */
"Q042"  =  "1"  /* Holoprosencephaly */
"Q043"  =  "1"  /* Other reduction deformities of brain */
"Q044"  =  "1"  /* Septo-optic dysplasia of brain */
"Q045"  =  "1"  /* Megalencephaly */
"Q046"  =  "1"  /* Congenital cerebral cysts */
"Q048"  =  "1"  /* Other specified congenital malformations of brain */
"Q049"  =  "1"  /* Congenital malformation of brain, unspecified */
"Q050"  =  "1"  /* Cervical spina bifida with hydrocephalus */
"Q051"  =  "1"  /* Thoracic spina bifida with hydrocephalus */
"Q052"  =  "1"  /* Lumbar spina bifida with hydrocephalus */
"Q053"  =  "1"  /* Sacral spina bifida with hydrocephalus */
"Q054"  =  "1"  /* Unspecified spina bifida with hydrocephalus */
"Q055"  =  "1"  /* Cervical spina bifida without hydrocephalus */
"Q056"  =  "1"  /* Thoracic spina bifida without hydrocephalus */
"Q057"  =  "1"  /* Lumbar spina bifida without hydrocephalus */
"Q058"  =  "1"  /* Sacral spina bifida without hydrocephalus */
"Q059"  =  "1"  /* Spina bifida, unspecified */
"Q060"  =  "1"  /* Amyelia */
"Q061"  =  "1"  /* Hypoplasia and dysplasia of spinal cord */
"Q063"  =  "1"  /* Other congenital cauda equina malformations */
"Q068"  =  "1"  /* Other specified congenital malformations of spinal cord */
"Q069"  =  "1"  /* Congenital malformation of spinal cord, unspecified */
"Q0700"  =  "1"  /* Arnold-Chiari syndrome without spina bifida or hydrocephalus */
"Q0701"  =  "1"  /* Arnold-Chiari syndrome with spina bifida */
"Q0702"  =  "1"  /* Arnold-Chiari syndrome with hydrocephalus */
"Q0703"  =  "1"  /* Arnold-Chiari syndrome with spina bifida and hydrocephalus */
"Q078"  =  "1"  /* Other specified congenital malformations of nervous system */
"Q079"  =  "1"  /* Congenital malformation of nervous system, unspecified */
OTHER   = '0' ;

VALUE $CONGN4D
"J986"  =  "1"  /* Disorders of diaphragm */
"K449"  =  "1"  /* Diaphragmatic hernia without obstruction or gangrene */
"Q311"  =  "1"  /* Congenital subglottic stenosis */
"Q312"  =  "1"  /* Laryngeal hypoplasia */
"Q313"  =  "1"  /* Laryngocele */
"Q315"  =  "1"  /* Congenital laryngomalacia */
"Q318"  =  "1"  /* Other congenital malformations of larynx */
"Q319"  =  "1"  /* Congenital malformation of larynx, unspecified */
"Q320"  =  "1"  /* Congenital tracheomalacia */
"Q321"  =  "1"  /* Other congenital malformations of trachea */
"Q322"  =  "1"  /* Congenital bronchomalacia */
"Q323"  =  "1"  /* Congenital stenosis of bronchus */
"Q324"  =  "1"  /* Other congenital malformations of bronchus */
"Q330"  =  "1"  /* Congenital cystic lung */
"Q331"  =  "1"  /* Accessory lobe of lung */
"Q335"  =  "1"  /* Ectopic tissue in lung */
"Q338"  =  "1"  /* Other congenital malformations of lung */
"Q339"  =  "1"  /* Congenital malformation of lung, unspecified */
"Q340"  =  "1"  /* Anomaly of pleura */
"Q341"  =  "1"  /* Congenital cyst of mediastinum */
"Q348"  =  "1"  /* Other specified congenital malformations of respiratory system */
"Q349"  =  "1"  /* Congenital malformation of respiratory system, unspecified */
"Q401"  =  "1"  /* Congenital hiatus hernia */
"Q790"  =  "1"  /* Congenital diaphragmatic hernia */
"Q791"  =  "1"  /* Other congenital malformations of diaphragm */
OTHER   = '0' ;

VALUE $CONGN5D
"I340"  =  "1"  /* Nonrheumatic mitral (valve) insufficiency */
"I341"  =  "1"  /* Nonrheumatic mitral (valve) prolapse */
"I342"  =  "1"  /* Nonrheumatic mitral (valve) stenosis */
"I348"  =  "1"  /* Other nonrheumatic mitral valve disorders */
"I349"  =  "1"  /* Nonrheumatic mitral valve disorder, unspecified */
"I350"  =  "1"  /* Nonrheumatic aortic (valve) stenosis */
"I351"  =  "1"  /* Nonrheumatic aortic (valve) insufficiency */
"I352"  =  "1"  /* Nonrheumatic aortic (valve) stenosis with insufficiency */
"I358"  =  "1"  /* Other nonrheumatic aortic valve disorders */
"I359"  =  "1"  /* Nonrheumatic aortic valve disorder, unspecified */
"I424"  =  "1"  /* Endocardial fibroelastosis */
"Q200"  =  "1"  /* Common arterial trunk */
"Q201"  =  "1"  /* Double outlet right ventricle */
"Q202"  =  "1"  /* Double outlet left ventricle */
"Q203"  =  "1"  /* Discordant ventriculoarterial connection */
"Q204"  =  "1"  /* Double inlet ventricle */
"Q205"  =  "1"  /* Discordant atrioventricular connection */
"Q208"  =  "1"  /* Other congenital malformations of cardiac chambers and connections */
"Q212"  =  "1"  /* Atrioventricular septal defect */
"Q213"  =  "1"  /* Tetralogy of Fallot */
"Q220"  =  "1"  /* Pulmonary valve atresia */
"Q222"  =  "1"  /* Congenital pulmonary valve insufficiency */
"Q224"  =  "1"  /* Congenital tricuspid stenosis */
"Q225"  =  "1"  /* Ebstein's anomaly */
"Q226"  =  "1"  /* Hypoplastic right heart syndrome */
"Q228"  =  "1"  /* Other congenital malformations of tricuspid valve */
"Q229"  =  "1"  /* Congenital malformation of tricuspid valve, unspecified */
"Q230"  =  "1"  /* Congenital stenosis of aortic valve */
"Q231"  =  "1"  /* Congenital insufficiency of aortic valve */
"Q232"  =  "1"  /* Congenital mitral stenosis */
"Q233"  =  "1"  /* Congenital mitral insufficiency */
"Q234"  =  "1"  /* Hypoplastic left heart syndrome */
"Q242"  =  "1"  /* Cor triatriatum */
"Q243"  =  "1"  /* Pulmonary infundibular stenosis */
"Q244"  =  "1"  /* Congenital subaortic stenosis */
"Q245"  =  "1"  /* Malformation of coronary vessels */
"Q248"  =  "1"  /* Other specified congenital malformations of heart */
"Q251"  =  "1"  /* Coarctation of aorta */
"Q252"  =  "1"  /* Atresia of aorta */
"Q253"  =  "1"  /* Supravalvular aortic stenosis */
"Q254"  =  "1"  /* Other congenital malformations of aorta */
"Q258"  =  "1"  /* Other congenital malformations of other great arteries */
"Q259"  =  "1"  /* Congenital malformation of great arteries, unspecified */
"Q260"  =  "1"  /* Congenital stenosis of vena cava */
"Q261"  =  "1"  /* Persistent left superior vena cava */
"Q262"  =  "1"  /* Total anomalous pulmonary venous connection */
"Q263"  =  "1"  /* Partial anomalous pulmonary venous connection */
"Q264"  =  "1"  /* Anomalous pulmonary venous connection, unspecified */
"Q268"  =  "1"  /* Other congenital malformations of great veins */
"Q269"  =  "1"  /* Congenital malformation of great vein, unspecified */
OTHER   = '0' ;

VALUE $CONGN6D
"Q773"  =  "1"  /* Chondrodysplasia punctata */
"Q776"  =  "1"  /* Chondroectodermal dysplasia */
"Q780"  =  "1"  /* Osteogenesis imperfecta */
"Q785"  =  "1"  /* Metaphyseal dysplasia */
"Q786"  =  "1"  /* Multiple congenital exostoses */
"Q788"  =  "1"  /* Other specified osteochondrodysplasias */
"Q789"  =  "1"  /* Osteochondrodysplasia, unspecified */
OTHER   = '0' ;

VALUE $CONGN7D
"Q872"  =  "1"  /* Congenital malformation syndromes predominantly involving limbs */
"Q873"  =  "1"  /* Congenital malformation syndromes involving early overgrowth */
"Q875"  =  "1"  /* Other congenital malformation syndromes with other skeletal changes */
"Q8781"  =  "1"  /* Alport syndrome */
"Q8789"  =  "1"  /* Other specified congenital malformation syndromes, not elsewhere classified */
"Q894"  =  "1"  /* Conjoined twins */
"Q897"  =  "1"  /* Multiple congenital malformations, not elsewhere classified */
"Q898"  =  "1"  /* Other specified congenital malformations */
"Q899"  =  "1"  /* Congenital malformation, unspecified */
"Q920"  =  "1"  /* Whole chromosome trisomy, nonmosaicism (meiotic nondisjunction) */
"Q921"  =  "1"  /* Whole chromosome trisomy, mosaicism (mitotic nondisjunction) */
"Q922"  =  "1"  /* Partial trisomy */
"Q925"  =  "1"  /* Duplications with other complex rearrangements */
"Q9262"  =  "1"  /* Marker chromosomes in abnormal individual */
"Q927"  =  "1"  /* Triploidy and polyploidy */
"Q928"  =  "1"  /* Other specified trisomies and partial trisomies of autosomes */
"Q929"  =  "1"  /* Trisomy and partial trisomy of autosomes, unspecified */
"Q930"  =  "1"  /* Whole chromosome monosomy, nonmosaicism (meiotic nondisjunction) */
"Q931"  =  "1"  /* Whole chromosome monosomy, mosaicism (mitotic nondisjunction) */
"Q932"  =  "1"  /* Chromosome replaced with ring, dicentric or isochromosome */
"Q952"  =  "1"  /* Balanced autosomal rearrangement in abnormal individual */
"Q953"  =  "1"  /* Balanced sex/autosomal rearrangement in abnormal individual */
"Q998"  =  "1"  /* Other specified chromosome abnormalities */
"Q999"  =  "1"  /* Chromosomal abnormality, unspecified */
OTHER   = '0' ;
 
VALUE $CONGN8D
"P613"  =  "1"  /* Congenital anemia from fetal blood loss */
"P614"  =  "1"  /* Other congenital anemias, not elsewhere classified */
"P832"  =  "1"  /* Hydrops fetalis not due to hemolytic disease */
"Q858"  =  "1"  /* Other phakomatoses, not elsewhere classified */
"Q859"  =  "1"  /* Phakomatosis, unspecified */
OTHER   = '0' ;

 /* ---------------------------------------------- */
 /* Asthma rate (ACSAST)                           */
 /*   PDI #14                                      */
 /* ---------------------------------------------- */

 /* Asthma                    */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $ACSASTD
"J4521"  =  "1"  /* Mild intermittent asthma with (acute) exacerbation */
"J4522"  =  "1"  /* Mild intermittent asthma with status asthmaticus */
"J4531"  =  "1"  /* Mild persistent asthma with (acute) exacerbation */
"J4532"  =  "1"  /* Mild persistent asthma with status asthmaticus */
"J4541"  =  "1"  /* Moderate persistent asthma with (acute) exacerbation */
"J4542"  =  "1"  /* Moderate persistent asthma with status asthmaticus */
"J4551"  =  "1"  /* Severe persistent asthma with (acute) exacerbation */
"J4552"  =  "1"  /* Severe persistent asthma with status asthmaticus */
"J45901"  =  "1"  /* Unspecified asthma with (acute) exacerbation */
"J45902"  =  "1"  /* Unspecified asthma with status asthmaticus */
"J45990"  =  "1"  /* Exercise induced bronchospasm */
"J45991"  =  "1"  /* Cough variant asthma */
"J45998"  =  "1"  /* Other asthma */
OTHER   = "0" ;

 /* Exclude: CYSTIC FIBROSIS      */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $ACSCYFD
"E840"  =  "1"  /* Cystic fibrosis with pulmonary manifestations */
"E8411"  =  "1"  /* Meconium ileus in cystic fibrosis */
"E8419"  =  "1"  /* Cystic fibrosis with other intestinal manifestations */
"E848"  =  "1"  /* Cystic fibrosis with other manifestations */
"E849"  =  "1"  /* Cystic fibrosis, unspecified */
OTHER   = "0" ;

 /* EXCLUDE: CYSTIC FIBROSIS AND ANOMALIES OF RESPIRATORY SYSTEM */
 /* ICD-10-CM DIAGNOSIS CODES */

VALUE $RESPAN
"E840"  =  "1"  /* Cystic fibrosis with pulmonary manifestations */
"E8411"  =  "1"  /* Meconium ileus in cystic fibrosis */
"E8419"  =  "1"  /* Cystic fibrosis with other intestinal manifestations */
"E848"  =  "1"  /* Cystic fibrosis with other manifestations */
"E849"  =  "1"  /* Cystic fibrosis, unspecified */
"J8483"  =  "1"  /* Surfactant mutations of the lung */
"J84841"  =  "1"  /* Neuroendocrine cell hyperplasia of infancy */
"J84842"  =  "1"  /* Pulmonary interstitial glycogenosis */
"J84843"  =  "1"  /* Alveolar capillary dysplasia with vein misalignment */
"J84848"  =  "1"  /* Other interstitial  lung diseases of childhood */
"P270"  =  "1"  /* Wilson-Mikity syndrome */
"P271"  =  "1"  /* Bronchopulmonary dysplasia originating in the perinatal period */
"P278"  =  "1"  /* Other chronic respiratory diseases originating in the perinatal period */
"P279"  =  "1"  /* Unspecified chronic respiratory disease originating in the perinatal period */
"Q254"  =  "1"  /* Other congenital malformations of aorta */
"Q311"  =  "1"  /* Congenital subglottic stenosis */
"Q312"  =  "1"  /* Laryngeal hypoplasia */
"Q313"  =  "1"  /* Laryngocele */
"Q315"  =  "1"  /* Congenital laryngomalacia */
"Q318"  =  "1"  /* Other congenital malformations of larynx */
"Q319"  =  "1"  /* Congenital malformation of larynx, unspecified */
"Q320"  =  "1"  /* Congenital tracheomalacia */
"Q321"  =  "1"  /* Other congenital malformations of trachea */
"Q322"  =  "1"  /* Congenital bronchomalacia */
"Q323"  =  "1"  /* Congenital stenosis of bronchus */
"Q324"  =  "1"  /* Other congenital malformations of bronchus */
"Q330"  =  "1"  /* Congenital cystic lung */
"Q331"  =  "1"  /* Accessory lobe of lung */
"Q332"  =  "1"  /* Sequestration of lung */
"Q333"  =  "1"  /* Agenesis of lung */
"Q334"  =  "1"  /* Congenital bronchiectasis */
"Q335"  =  "1"  /* Ectopic tissue in lung */
"Q336"  =  "1"  /* Congenital hypoplasia and dysplasia of lung */
"Q338"  =  "1"  /* Other congenital malformations of lung */
"Q339"  =  "1"  /* Congenital malformation of lung, unspecified */
"Q340"  =  "1"  /* Anomaly of pleura */
"Q341"  =  "1"  /* Congenital cyst of mediastinum */
"Q348"  =  "1"  /* Other specified congenital malformations of respiratory system */
"Q349"  =  "1"  /* Congenital malformation of respiratory system, unspecified */
"Q390"  =  "1"  /* Atresia of esophagus without fistula */
"Q391"  =  "1"  /* Atresia of esophagus with tracheo-esophageal fistula */
"Q392"  =  "1"  /* Congenital tracheo-esophageal fistula without atresia */
"Q393"  =  "1"  /* Congenital stenosis and stricture of esophagus */
"Q394"  =  "1"  /* Esophageal web */
"Q893"  =  "1"  /* Situs inversus */
OTHER   = 0;


 /* ---------------------------------------------- */
 /* Diabetes short term complication rate (ACDIAS) */
 /*   PDI #15                                      */
 /* ---------------------------------------------- */

 /* Diabetes with Short Term Complications */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $ACDIASD
"E1010"  =  "1"  /* Type 1 diabetes mellitus with ketoacidosis without coma */
"E1011"  =  "1"  /* Type 1 diabetes mellitus with ketoacidosis with coma */
"E10641"  =  "1"  /* Type 1 diabetes mellitus with hypoglycemia with coma */
"E1065"  =  "1"  /* Type 1 diabetes mellitus with hyperglycemia */
"E1100"  =  "1"  /* Type 2 diabetes mellitus with hyperosmolarity without nonketotic hyperglycemic-hyperosmolar coma (NKHHC) */
"E1101"  =  "1"  /* Type 2 diabetes mellitus with hyperosmolarity with coma */
"E11641"  =  "1"  /* Type 2 diabetes mellitus with hypoglycemia with coma */
"E1165"  =  "1"  /* Type 2 diabetes mellitus with hyperglycemia */
OTHER   = "0" ;


 /* ---------------------------------------------- */
 /* Pediatric Gastroenteritis rate (ACPGAS)        */
 /*   PDI #16                                      */
 /* ---------------------------------------------- */

 /* Gastroenteritis */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $ACPGASD
"A080"  =  "1"  /* Rotaviral enteritis */
"A0811"  =  "1"  /* Acute gastroenteropathy due to Norwalk agent */
"A0819"  =  "1"  /* Acute gastroenteropathy due to other small round viruses */
"A082"  =  "1"  /* Adenoviral enteritis */
"A0831"  =  "1"  /* Calicivirus enteritis */
"A0832"  =  "1"  /* Astrovirus enteritis */
"A0839"  =  "1"  /* Other viral enteritis */
"A084"  =  "1"  /* Viral intestinal infection, unspecified */
"A088"  =  "1"  /* Other specified intestinal infections */
"A09"  =  "1"  /* Infectious gastroenteritis and colitis, unspecified */
"K5289"  =  "1"  /* Other specified noninfective gastroenteritis and colitis */
"K529"  =  "1"  /* Noninfective gastroenteritis and colitis, unspecified */
OTHER   = "0" ;

 /* Dehydration */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $ACSDEHD
"E860"  =  "1"  /* Dehydration */
"E861"  =  "1"  /* Hypovolemia */
"E869"  =  "1"  /* Volume depletion, unspecified */
OTHER   = "0" ;

/* EXCLUDE: */
/* Gastrintestinal Abnormalities */
/* ICD-10-CM Diagnosis Codes: */

VALUE $ACGDISD
"K5000"  =  "1"  /* Crohn's disease of small intestine without complications */
"K50011"  =  "1"  /* Crohn's disease of small intestine with rectal bleeding */
"K50012"  =  "1"  /* Crohn's disease of small intestine with intestinal obstruction */
"K50013"  =  "1"  /* Crohn's disease of small intestine with fistula */
"K50014"  =  "1"  /* Crohn's disease of small intestine with abscess */
"K50018"  =  "1"  /* Crohn's disease of small intestine with other complication */
"K50019"  =  "1"  /* Crohn's disease of small intestine with unspecified complications */
"K5010"  =  "1"  /* Crohn's disease of large intestine without complications */
"K50111"  =  "1"  /* Crohn's disease of large intestine with rectal bleeding */
"K50112"  =  "1"  /* Crohn's disease of large intestine with intestinal obstruction */
"K50113"  =  "1"  /* Crohn's disease of large intestine with fistula */
"K50114"  =  "1"  /* Crohn's disease of large intestine with abscess */
"K50118"  =  "1"  /* Crohn's disease of large intestine with other complication */
"K50119"  =  "1"  /* Crohn's disease of large intestine with unspecified complications */
"K5080"  =  "1"  /* Crohn's disease of both small and large intestine without complications */
"K50811"  =  "1"  /* Crohn's disease of both small and large intestine with rectal bleeding */
"K50812"  =  "1"  /* Crohn's disease of both small and large intestine with intestinal obstruction */
"K50813"  =  "1"  /* Crohn's disease of both small and large intestine with fistula */
"K50814"  =  "1"  /* Crohn's disease of both small and large intestine with abscess */
"K50818"  =  "1"  /* Crohn's disease of both small and large intestine with other complication */
"K50819"  =  "1"  /* Crohn's disease of both small and large intestine with unspecified complications */
"K5090"  =  "1"  /* Crohn's disease, unspecified, without complications */
"K50911"  =  "1"  /* Crohn's disease, unspecified, with rectal bleeding */
"K50912"  =  "1"  /* Crohn's disease, unspecified, with intestinal obstruction */
"K50913"  =  "1"  /* Crohn's disease, unspecified, with fistula */
"K50914"  =  "1"  /* Crohn's disease, unspecified, with abscess */
"K50918"  =  "1"  /* Crohn's disease, unspecified, with other complication */
"K50919"  =  "1"  /* Crohn's disease, unspecified, with unspecified complications */
"K5100"  =  "1"  /* Ulcerative (chronic) pancolitis without complications */
"K51011"  =  "1"  /* Ulcerative (chronic) pancolitis with rectal bleeding */
"K51012"  =  "1"  /* Ulcerative (chronic) pancolitis with intestinal obstruction */
"K51013"  =  "1"  /* Ulcerative (chronic) pancolitis with fistula */
"K51014"  =  "1"  /* Ulcerative (chronic) pancolitis with abscess */
"K51018"  =  "1"  /* Ulcerative (chronic) pancolitis with other complication */
"K51019"  =  "1"  /* Ulcerative (chronic) pancolitis with unspecified complications */
"K5120"  =  "1"  /* Ulcerative (chronic) proctitis without complications */
"K51211"  =  "1"  /* Ulcerative (chronic) proctitis with rectal bleeding */
"K51212"  =  "1"  /* Ulcerative (chronic) proctitis with intestinal obstruction */
"K51213"  =  "1"  /* Ulcerative (chronic) proctitis with fistula */
"K51214"  =  "1"  /* Ulcerative (chronic) proctitis with abscess */
"K51218"  =  "1"  /* Ulcerative (chronic) proctitis with other complication */
"K51219"  =  "1"  /* Ulcerative (chronic) proctitis with unspecified complications */
"K5130"  =  "1"  /* Ulcerative (chronic) rectosigmoiditis without complications */
"K51311"  =  "1"  /* Ulcerative (chronic) rectosigmoiditis with rectal bleeding */
"K51312"  =  "1"  /* Ulcerative (chronic) rectosigmoiditis with intestinal obstruction */
"K51313"  =  "1"  /* Ulcerative (chronic) rectosigmoiditis with fistula */
"K51314"  =  "1"  /* Ulcerative (chronic) rectosigmoiditis with abscess */
"K51318"  =  "1"  /* Ulcerative (chronic) rectosigmoiditis with other complication */
"K51319"  =  "1"  /* Ulcerative (chronic) rectosigmoiditis with unspecified complications */
"K5140"  =  "1"  /* Inflammatory polyps of colon without complications */
"K51411"  =  "1"  /* Inflammatory polyps of colon with rectal bleeding */
"K51412"  =  "1"  /* Inflammatory polyps of colon with intestinal obstruction */
"K51413"  =  "1"  /* Inflammatory polyps of colon with fistula */
"K51414"  =  "1"  /* Inflammatory polyps of colon with abscess */
"K51418"  =  "1"  /* Inflammatory polyps of colon with other complication */
"K51419"  =  "1"  /* Inflammatory polyps of colon with unspecified complications */
"K5150"  =  "1"  /* Left sided colitis without complications */
"K51511"  =  "1"  /* Left sided colitis with rectal bleeding */
"K51512"  =  "1"  /* Left sided colitis with intestinal obstruction */
"K51513"  =  "1"  /* Left sided colitis with fistula */
"K51514"  =  "1"  /* Left sided colitis with abscess */
"K51518"  =  "1"  /* Left sided colitis with other complication */
"K51519"  =  "1"  /* Left sided colitis with unspecified complications */
"K5180"  =  "1"  /* Other ulcerative colitis without complications */
"K51811"  =  "1"  /* Other ulcerative colitis with rectal bleeding */
"K51812"  =  "1"  /* Other ulcerative colitis with intestinal obstruction */
"K51813"  =  "1"  /* Other ulcerative colitis with fistula */
"K51814"  =  "1"  /* Other ulcerative colitis with abscess */
"K51818"  =  "1"  /* Other ulcerative colitis with other complication */
"K51819"  =  "1"  /* Other ulcerative colitis with unspecified complications */
"K5190"  =  "1"  /* Ulcerative colitis, unspecified, without complications */
"K51911"  =  "1"  /* Ulcerative colitis, unspecified with rectal bleeding */
"K51912"  =  "1"  /* Ulcerative colitis, unspecified with intestinal obstruction */
"K51913"  =  "1"  /* Ulcerative colitis, unspecified with fistula */
"K51914"  =  "1"  /* Ulcerative colitis, unspecified with abscess */
"K51918"  =  "1"  /* Ulcerative colitis, unspecified with other complication */
"K51919"  =  "1"  /* Ulcerative colitis, unspecified with unspecified complications */
"K520"  =  "1"  /* Gastroenteritis and colitis due to radiation */
"K521"  =  "1"  /* Toxic gastroenteritis and colitis */
"K522"  =  "1"  /* Allergic and dietetic gastroenteritis and colitis */
"K5281"  =  "1"  /* Eosinophilic gastritis or gastroenteritis */
"K5282"  =  "1"  /* Eosinophilic colitis */
"K900"  =  "1"  /* Celiac disease */
"K901"  =  "1"  /* Tropical sprue */
"K902"  =  "1"  /* Blind loop syndrome, not elsewhere classified */
"K903"  =  "1"  /* Pancreatic steatorrhea */
"K904"  =  "1"  /* Malabsorption due to intolerance, not elsewhere classified */
"K9089"  =  "1"  /* Other intestinal malabsorption */
"K909"  =  "1"  /* Intestinal malabsorption, unspecified */
"K912"  =  "1"  /* Postsurgical malabsorption, not elsewhere classified */
"K9281"  =  "1"  /* Gastrointestinal mucositis (ulcerative) */
OTHER   = "0" ;

/* Bacterial Gastoenteritis  */
/* ICD-10-CM Diagnosis Codes: */

VALUE $ACBACGD
"A020"  =  "1"  /* Salmonella enteritis */
"A030"  =  "1"  /* Shigellosis due to Shigella dysenteriae */
"A031"  =  "1"  /* Shigellosis due to Shigella flexneri */
"A032"  =  "1"  /* Shigellosis due to Shigella boydii */
"A033"  =  "1"  /* Shigellosis due to Shigella sonnei */
"A038"  =  "1"  /* Other shigellosis */
"A039"  =  "1"  /* Shigellosis, unspecified */
"A040"  =  "1"  /* Enteropathogenic Escherichia coli infection */
"A041"  =  "1"  /* Enterotoxigenic Escherichia coli infection */
"A042"  =  "1"  /* Enteroinvasive Escherichia coli infection */
"A043"  =  "1"  /* Enterohemorrhagic Escherichia coli infection */
"A044"  =  "1"  /* Other intestinal Escherichia coli infections */
"A045"  =  "1"  /* Campylobacter enteritis */
"A046"  =  "1"  /* Enteritis due to Yersinia enterocolitica */
"A047"  =  "1"  /* Enterocolitis due to Clostridium difficile */
"A048"  =  "1"  /* Other specified bacterial intestinal infections */
"A049"  =  "1"  /* Bacterial intestinal infection, unspecified */
"A050"  =  "1"  /* Foodborne staphylococcal intoxication */
"A051"  =  "1"  /* Botulism food poisoning */
"A052"  =  "1"  /* Foodborne Clostridium perfringens [Clostridium welchii] intoxication */
"A053"  =  "1"  /* Foodborne Vibrio parahaemolyticus intoxication */
"A054"  =  "1"  /* Foodborne Bacillus cereus intoxication */
"A055"  =  "1"  /* Foodborne Vibrio vulnificus intoxication */
"A058"  =  "1"  /* Other specified bacterial foodborne intoxications */
"A059"  =  "1"  /* Bacterial foodborne intoxication, unspecified */
"A060"  =  "1"  /* Acute amebic dysentery */
"A061"  =  "1"  /* Chronic intestinal amebiasis */
"A070"  =  "1"  /* Balantidiasis */
"A071"  =  "1"  /* Giardiasis [lambliasis] */
"A072"  =  "1"  /* Cryptosporidiosis */
"A073"  =  "1"  /* Isosporiasis */
"A074"  =  "1"  /* Cyclosporiasis */
"A078"  =  "1"  /* Other specified protozoal intestinal diseases */
"A079"  =  "1"  /* Protozoal intestinal disease, unspecified */
"B3782"  =  "1"  /* Candidal enteritis */
OTHER   = "0" ;


 /* ---------------------------------------------- */
 /* Perforated Appendix rate (ACSAPP)              */
 /*   PDI #17                                      */
 /* ---------------------------------------------- */

 /* Perforated Appendix (Outcome of Interest) */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $ACSAPPD
"K352"  =  "1"  /* Acute appendicitis with generalized peritonitis */
"K353"  =  "1"  /* Acute appendicitis with localized peritonitis */
OTHER   = "0" ;

 /* Appendicitis (Population at Risk) */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $ACSAP2D
"K352"  =  "1"  /* Acute appendicitis with generalized peritonitis */
"K353"  =  "1"  /* Acute appendicitis with localized peritonitis */
"K3580"  =  "1"  /* Unspecified acute appendicitis */
"K3589"  =  "1"  /* Other acute appendicitis */
"K37"  =  "1"  /* Unspecified appendicitis */
OTHER   = "0" ;


 /* ---------------------------------------------- */
 /* Urinary Infection rate (ACSUTI)                */
 /*   PDI #18                                      */
 /* ---------------------------------------------- */

 /* Urinary Infection */
 /* ICD-10-CM Diagnosis Codes: */

VALUE $ACSUTID
"N10"  =  "1"  /* Acute tubulo-interstitial nephritis */
"N119"  =  "1"  /* Chronic tubulo-interstitial nephritis, unspecified */
"N12"  =  "1"  /* Tubulo-interstitial nephritis, not specified as acute or chronic */
"N151"  =  "1"  /* Renal and perinephric abscess */
"N159"  =  "1"  /* Renal tubulo-interstitial disease, unspecified */
"N16"  =  "1"  /* Renal tubulo-interstitial disorders in diseases classified elsewhere */
"N2884"  =  "1"  /* Pyelitis cystica */
"N2885"  =  "1"  /* Pyeloureteritis cystica */
"N2886"  =  "1"  /* Ureteritis cystica */
"N3000"  =  "1"  /* Acute cystitis without hematuria */
"N3001"  =  "1"  /* Acute cystitis with hematuria */
"N3090"  =  "1"  /* Cystitis, unspecified without hematuria */
"N3091"  =  "1"  /* Cystitis, unspecified with hematuria */
"N390"  =  "1"  /* Urinary tract infection, site not specified */
OTHER   = "0" ;

 /* EXCLUDE: KIDNEY OR URINARY TRACT DISORDER */
 /* ICD-10-CM DIAGNOSIS CODES */

VALUE $KIDNEY
"N110"  =  "1"  /* Nonobstructive reflux-associated chronic pyelonephritis */
"N111"  =  "1"  /* Chronic obstructive pyelonephritis */
"N118"  =  "1"  /* Other chronic tubulo-interstitial nephritis */
"N119"  =  "1"  /* Chronic tubulo-interstitial nephritis, unspecified */
"N1370"  =  "1"  /* Vesicoureteral-reflux, unspecified */
"N1371"  =  "1"  /* Vesicoureteral-reflux without reflux nephropathy */
"N13721"  =  "1"  /* Vesicoureteral-reflux with reflux nephropathy without hydroureter, unilateral */
"N13722"  =  "1"  /* Vesicoureteral-reflux with reflux nephropathy without hydroureter, bilateral */
"N13729"  =  "1"  /* Vesicoureteral-reflux with reflux nephropathy without hydroureter, unspecified */
"N13731"  =  "1"  /* Vesicoureteral-reflux with reflux nephropathy with hydroureter, unilateral */
"N13732"  =  "1"  /* Vesicoureteral-reflux with reflux nephropathy with hydroureter, bilateral */
"N13739"  =  "1"  /* Vesicoureteral-reflux with reflux nephropathy with hydroureter, unspecified */
"N139"  =  "1"  /* Obstructive and reflux uropathy, unspecified */
"Q600"  =  "1"  /* Renal agenesis, unilateral */
"Q601"  =  "1"  /* Renal agenesis, bilateral */
"Q602"  =  "1"  /* Renal agenesis, unspecified */
"Q603"  =  "1"  /* Renal hypoplasia, unilateral */
"Q604"  =  "1"  /* Renal hypoplasia, bilateral */
"Q605"  =  "1"  /* Renal hypoplasia, unspecified */
"Q606"  =  "1"  /* Potter's syndrome */
"Q6100"  =  "1"  /* Congenital renal cyst, unspecified */
"Q6101"  =  "1"  /* Congenital single renal cyst */
"Q6102"  =  "1"  /* Congenital multiple renal cysts */
"Q6111"  =  "1"  /* Cystic dilatation of collecting ducts */
"Q6119"  =  "1"  /* Other polycystic kidney, infantile type */
"Q612"  =  "1"  /* Polycystic kidney, adult type */
"Q613"  =  "1"  /* Polycystic kidney, unspecified */
"Q614"  =  "1"  /* Renal dysplasia */
"Q615"  =  "1"  /* Medullary cystic kidney */
"Q618"  =  "1"  /* Other cystic kidney diseases */
"Q619"  =  "1"  /* Cystic kidney disease, unspecified */
"Q620"  =  "1"  /* Congenital hydronephrosis */
"Q6210"  =  "1"  /* Congenital occlusion of ureter, unspecified */
"Q6211"  =  "1"  /* Congenital occlusion of ureteropelvic junction */
"Q6212"  =  "1"  /* Congenital occlusion of ureterovesical orifice */
"Q622"  =  "1"  /* Congenital megaureter */
"Q6231"  =  "1"  /* Congenital ureterocele, orthotopic */
"Q6232"  =  "1"  /* Cecoureterocele */
"Q6239"  =  "1"  /* Other obstructive defects of renal pelvis and ureter */
"Q624"  =  "1"  /* Agenesis of ureter */
"Q625"  =  "1"  /* Duplication of ureter */
"Q6260"  =  "1"  /* Malposition of ureter, unspecified */
"Q6261"  =  "1"  /* Deviation of ureter */
"Q6262"  =  "1"  /* Displacement of ureter */
"Q6263"  =  "1"  /* Anomalous implantation of ureter */
"Q6269"  =  "1"  /* Other malposition of ureter */
"Q627"  =  "1"  /* Congenital vesico-uretero-renal reflux */
"Q628"  =  "1"  /* Other congenital malformations of ureter */
"Q630"  =  "1"  /* Accessory kidney */
"Q631"  =  "1"  /* Lobulated, fused and horseshoe kidney */
"Q632"  =  "1"  /* Ectopic kidney */
"Q633"  =  "1"  /* Hyperplastic and giant kidney */
"Q638"  =  "1"  /* Other specified congenital malformations of kidney */
"Q639"  =  "1"  /* Congenital malformation of kidney, unspecified */
"Q6410"  =  "1"  /* Exstrophy of urinary bladder, unspecified */
"Q6411"  =  "1"  /* Supravesical fissure of urinary bladder */
"Q6412"  =  "1"  /* Cloacal extrophy of urinary bladder */
"Q6419"  =  "1"  /* Other exstrophy of urinary bladder */
"Q642"  =  "1"  /* Congenital posterior urethral valves */
"Q6431"  =  "1"  /* Congenital bladder neck obstruction */
"Q6432"  =  "1"  /* Congenital stricture of urethra */
"Q6433"  =  "1"  /* Congenital stricture of urinary meatus */
"Q6439"  =  "1"  /* Other atresia and stenosis of urethra and bladder neck */
"Q645"  =  "1"  /* Congenital absence of bladder and urethra */
"Q646"  =  "1"  /* Congenital diverticulum of bladder */
"Q6470"  =  "1"  /* Unspecified congenital malformation of bladder and urethra */
"Q6471"  =  "1"  /* Congenital prolapse of urethra */
"Q6472"  =  "1"  /* Congenital prolapse of urinary meatus */
"Q6473"  =  "1"  /* Congenital urethrorectal fistula */
"Q6474"  =  "1"  /* Double urethra */
"Q6475"  =  "1"  /* Double urinary meatus */
"Q6479"  =  "1"  /* Other congenital malformations of bladder and urethra */
"Q648"  =  "1"  /* Other specified congenital malformations of urinary system */
"Q649"  =  "1"  /* Congenital malformation of urinary system, unspecified */
OTHER   = 0;


 /* Low Birth Weight (Outcome of Interest) */
 /* ICD-10-CM Diagnosis Codes:              */

VALUE $ACSLBWD
"P0501"  =  "1"  /* Newborn light for gestational age, less than 500 grams */
"P0502"  =  "1"  /* Newborn light for gestational age, 500-749 grams */
"P0503"  =  "1"  /* Newborn light for gestational age, 750-999 grams */
"P0504"  =  "1"  /* Newborn light for gestational age, 1000-1249 grams */
"P0505"  =  "1"  /* Newborn light for gestational age, 1250-1499 grams */
"P0506"  =  "1"  /* Newborn light for gestational age, 1500-1749 grams */
"P0507"  =  "1"  /* Newborn light for gestational age, 1750-1999 grams */
"P0508"  =  "1"  /* Newborn light for gestational age, 2000-2499 grams */
"P0511"  =  "1"  /* Newborn small for gestational age, less than 500 grams */
"P0512"  =  "1"  /* Newborn small for gestational age, 500-749 grams */
"P0513"  =  "1"  /* Newborn small for gestational age, 750-999 grams */
"P0514"  =  "1"  /* Newborn small for gestational age, 1000-1249 grams */
"P0515"  =  "1"  /* Newborn small for gestational age, 1250-1499 grams */
"P0516"  =  "1"  /* Newborn small for gestational age, 1500-1749 grams */
"P0517"  =  "1"  /* Newborn small for gestational age, 1750-1999 grams */
"P0518"  =  "1"  /* Newborn small for gestational age, 2000-2499 grams */
"P0700"  =  "1"  /* Extremely low birth weight newborn, unspecified weight */
"P0701"  =  "1"  /* Extremely low birth weight newborn, less than 500 grams */
"P0702"  =  "1"  /* Extremely low birth weight newborn, 500-749 grams */
"P0703"  =  "1"  /* Extremely low birth weight newborn, 750-999 grams */
"P0710"  =  "1"  /* Other low birth weight newborn, unspecified weight */
"P0714"  =  "1"  /* Other low birth weight newborn, 1000-1249 grams */
"P0715"  =  "1"  /* Other low birth weight newborn, 1250-1499 grams */
"P0716"  =  "1"  /* Other low birth weight newborn, 1500-1749 grams */
"P0717"  =  "1"  /* Other low birth weight newborn, 1750-1999 grams */
"P0718"  =  "1"  /* Other low birth weight newborn, 2000-2499 grams */
OTHER   = 0 ;

 * ----------------------------------------------------- ;
 * ---   AREA FORMATS     ------------------------------ ;
 * ---   TRANSLATES FIPS STATE AND COUNTY CODE (SSCCC)   ;
 * ---   TO MSA (METROPOLITAN STATISTICAL AREA)          ;
 * ----------------------------------------------------- ;

/* MODIFIED FIPS */

VALUE $M1AREA
'02013' = '02001'
'02016' = '02001'
'02020' = '02001'
'02050' = '02001'
'02060' = '02001'
'02068' = '02001'
'02070' = '02001'
'02090' = '02001'
'02100' = '02001'
'02110' = '02001'
'02122' = '02001'
'02130' = '02001'
'02150' = '02001'
'02164' = '02001'
'02170' = '02001'
'02180' = '02001'
'02185' = '02001'
'02188' = '02001'
'02201' = '02001'
'02220' = '02001'
'02232' = '02001'
'02240' = '02001'
'02261' = '02001'
'02270' = '02001'
'02280' = '02001'
'02282' = '02001'
'02290' = '02001'
'15005' = '15009'
'24510' = '24007'
'29510' = '29191'
'32510' = '32025'
'51515' = '51019'
'51520' = '51119'
'51530' = '51163'
'51540' = '51003'
'51560' = '51005'
'51570' = '51041'
'51580' = '51005'
'51590' = '51143'
'51595' = '51081'
'51600' = '51059'
'51610' = '51059'
'51620' = '51175'
'51630' = '51177'
'51640' = '51077'
'51660' = '51165'
'51670' = '51149'
'51678' = '51163'
'51680' = '51031'
'51683' = '51153'
'51685' = '51153'
'51690' = '51089'
'51710' = '51129'
'51720' = '51195'
'51730' = '51053'
'51735' = '51199'
'51740' = '51129'
'51750' = '51121'
'51760' = '51087'
'51770' = '51161'
'51775' = '51161'
'51780' = '51083'
'51790' = '51015'
'51800' = '51123'
'51820' = '51015'
'51830' = '51095'
'51840' = '51069'
;

/* OMB 1999 METRO AREA */

VALUE $M2AREA
'01001' = '5240 '
'01003' = '5160 '
'01009' = '1000 '
'01015' = '0450 '
'01033' = '2650 '
'01045' = '2180 '
'01051' = '5240 '
'01055' = '2880 '
'01069' = '2180 '
'01073' = '1000 '
'01077' = '2650 '
'01079' = '2030 '
'01081' = '0580 '
'01083' = '3440 '
'01089' = '3440 '
'01097' = '5160 '
'01101' = '5240 '
'01103' = '2030 '
'01113' = '1800 '
'01115' = '1000 '
'01117' = '1000 '
'01125' = '8600 '
'02020' = '0380 '
'04005' = '2620 '
'04013' = '6200 '
'04015' = '4120 '
'04019' = '8520 '
'04021' = '6200 '
'04027' = '9360 '
'05007' = '2580 '
'05031' = '3700 '
'05033' = '2720 '
'05035' = '4920 '
'05045' = '4400 '
'05069' = '6240 '
'05085' = '4400 '
'05091' = '8360 '
'05119' = '4400 '
'05125' = '4400 '
'05131' = '2720 '
'05143' = '2580 '
'06001' = '5775 '
'06007' = '1620 '
'06013' = '5775 '
'06017' = '6920 '
'06019' = '2840 '
'06029' = '0680 '
'06037' = '4480 '
'06039' = '2840 '
'06041' = '7360 '
'06047' = '4940 '
'06053' = '7120 '
'06055' = '8720 '
'06059' = '5945 '
'06061' = '6920 '
'06065' = '6780 '
'06067' = '6920 '
'06071' = '6780 '
'06073' = '7320 '
'06075' = '7360 '
'06077' = '8120 '
'06079' = '7460 '
'06081' = '7360 '
'06083' = '7480 '
'06085' = '7400 '
'06087' = '7485 '
'06089' = '6690 '
'06095' = '8720 '
'06097' = '7500 '
'06099' = '5170 '
'06101' = '9340 '
'06107' = '8780 '
'06111' = '8735 '
'06113' = '9270 '
'06115' = '9340 '
'08001' = '2080 '
'08005' = '2080 '
'08013' = '1125 '
'08031' = '2080 '
'08035' = '2080 '
'08041' = '1720 '
'08059' = '2080 '
'08069' = '2670 '
'08077' = '2995 '
'08101' = '6560 '
'08123' = '3060 '
'09001' = '5483 '
'09003' = '3283 '
'09007' = '3283 '
'09009' = '5483 '
'09011' = '5523 '
'09013' = '3283 '
'10001' = '2190 '
'10003' = '9160 '
'11001' = '8840 '
'12001' = '2900 '
'12005' = '6015 '
'12009' = '4900 '
'12011' = '2680 '
'12015' = '6580 '
'12019' = '3600 '
'12021' = '5345 '
'12031' = '3600 '
'12033' = '6080 '
'12035' = '2020 '
'12039' = '8240 '
'12053' = '8280 '
'12057' = '8280 '
'12069' = '5960 '
'12071' = '2700 '
'12073' = '8240 '
'12081' = '7510 '
'12083' = '5790 '
'12085' = '2710 '
'12086' = '5000 '
'12089' = '3600 '
'12091' = '2750 '
'12095' = '5960 '
'12097' = '5960 '
'12099' = '8960 '
'12101' = '8280 '
'12103' = '8280 '
'12105' = '3980 '
'12109' = '3600 '
'12111' = '2710 '
'12113' = '6080 '
'12115' = '7510 '
'12117' = '5960 '
'12127' = '2020 '
'13013' = '0520 '
'13015' = '0520 '
'13021' = '4680 '
'13029' = '7520 '
'13045' = '0520 '
'13047' = '1560 '
'13051' = '7520 '
'13053' = '1800 '
'13057' = '0520 '
'13059' = '0500 '
'13063' = '0520 '
'13067' = '0520 '
'13073' = '0600 '
'13077' = '0520 '
'13083' = '1560 '
'13089' = '0520 '
'13095' = '0120 '
'13097' = '0520 '
'13103' = '7520 '
'13113' = '0520 '
'13117' = '0520 '
'13121' = '0520 '
'13135' = '0520 '
'13145' = '1800 '
'13151' = '0520 '
'13153' = '4680 '
'13169' = '4680 '
'13177' = '0120 '
'13189' = '0600 '
'13195' = '0500 '
'13215' = '1800 '
'13217' = '0520 '
'13219' = '0500 '
'13223' = '0520 '
'13225' = '4680 '
'13227' = '0520 '
'13245' = '0600 '
'13247' = '0520 '
'13255' = '0520 '
'13289' = '4680 '
'13295' = '1560 '
'13297' = '0520 '
'15003' = '3320 '
'16001' = '1080 '
'16005' = '6340 '
'16027' = '1080 '
'17007' = '6880 '
'17019' = '1400 '
'17027' = '7040 '
'17031' = '1600 '
'17037' = '1600 '
'17043' = '1600 '
'17063' = '1600 '
'17073' = '1960 '
'17083' = '7040 '
'17089' = '1600 '
'17091' = '3740 '
'17093' = '1600 '
'17097' = '1600 '
'17111' = '1600 '
'17113' = '1040 '
'17115' = '2040 '
'17119' = '7040 '
'17129' = '7880 '
'17133' = '7040 '
'17141' = '6880 '
'17143' = '6120 '
'17161' = '1960 '
'17163' = '7040 '
'17167' = '7880 '
'17179' = '6120 '
'17197' = '1600 '
'17201' = '6880 '
'17203' = '6120 '
'18001' = '2760 '
'18003' = '2760 '
'18011' = '3480 '
'18019' = '4520 '
'18021' = '8320 '
'18023' = '3920 '
'18029' = '1640 '
'18033' = '2760 '
'18035' = '5280 '
'18039' = '2330 '
'18043' = '4520 '
'18057' = '3480 '
'18059' = '3480 '
'18061' = '4520 '
'18063' = '3480 '
'18067' = '3850 '
'18069' = '2760 '
'18081' = '3480 '
'18089' = '2960 '
'18095' = '3480 '
'18097' = '3480 '
'18105' = '1020 '
'18109' = '3480 '
'18115' = '1640 '
'18127' = '2960 '
'18129' = '2440 '
'18141' = '7800 '
'18143' = '4520 '
'18145' = '3480 '
'18157' = '3920 '
'18159' = '3850 '
'18163' = '2440 '
'18165' = '8320 '
'18167' = '8320 '
'18173' = '2440 '
'18179' = '2760 '
'18183' = '2760 '
'19013' = '8920 '
'19049' = '2120 '
'19061' = '2200 '
'19103' = '3500 '
'19113' = '1360 '
'19153' = '2120 '
'19155' = '5920 '
'19163' = '1960 '
'19181' = '2120 '
'19193' = '7720 '
'20015' = '9040 '
'20045' = '4150 '
'20079' = '9040 '
'20091' = '3760 '
'20103' = '3760 '
'20121' = '3760 '
'20173' = '9040 '
'20177' = '8440 '
'20209' = '3760 '
'21015' = '1640 '
'21017' = '4280 '
'21019' = '3400 '
'21029' = '4520 '
'21037' = '1640 '
'21043' = '3400 '
'21047' = '1660 '
'21049' = '4280 '
'21059' = '5990 '
'21067' = '4280 '
'21077' = '1640 '
'21081' = '1640 '
'21089' = '3400 '
'21101' = '2440 '
'21111' = '4520 '
'21113' = '4280 '
'21117' = '1640 '
'21151' = '4280 '
'21185' = '4520 '
'21191' = '1640 '
'21209' = '4280 '
'21239' = '4280 '
'22001' = '3880 '
'22005' = '0760 '
'22015' = '7680 '
'22017' = '7680 '
'22019' = '3960 '
'22033' = '0760 '
'22051' = '5560 '
'22055' = '3880 '
'22057' = '3350 '
'22063' = '0760 '
'22071' = '5560 '
'22073' = '5200 '
'22075' = '5560 '
'22079' = '0220 '
'22087' = '5560 '
'22089' = '5560 '
'22093' = '5560 '
'22095' = '5560 '
'22097' = '3880 '
'22099' = '3880 '
'22103' = '5560 '
'22109' = '3350 '
'22119' = '7680 '
'22121' = '0760 '
'23001' = '4243 '
'23005' = '6403 '
'23019' = '0733 '
'24001' = '1900 '
'24003' = '0720 '
'24005' = '0720 '
'24009' = '8840 '
'24013' = '0720 '
'24015' = '9160 '
'24017' = '8840 '
'24021' = '8840 '
'24025' = '0720 '
'24027' = '0720 '
'24031' = '8840 '
'24033' = '8840 '
'24035' = '0720 '
'24043' = '3180 '
'24510' = '0720 '
'25001' = '0743 '
'25003' = '6323 '
'25005' = '1123 '
'25009' = '1123 '
'25013' = '8003 '
'25015' = '8003 '
'25017' = '1123 '
'25021' = '1123 '
'25023' = '1123 '
'25025' = '1123 '
'25027' = '1123 '
'26005' = '3000 '
'26017' = '6960 '
'26021' = '0870 '
'26025' = '3720 '
'26037' = '4040 '
'26045' = '4040 '
'26049' = '2640 '
'26065' = '4040 '
'26075' = '3520 '
'26077' = '3720 '
'26081' = '3000 '
'26087' = '2160 '
'26091' = '0440 '
'26093' = '0440 '
'26099' = '2160 '
'26111' = '6960 '
'26115' = '2160 '
'26121' = '3000 '
'26125' = '2160 '
'26139' = '3000 '
'26145' = '6960 '
'26147' = '2160 '
'26159' = '3720 '
'26161' = '0440 '
'26163' = '2160 '
'27003' = '5120 '
'27009' = '6980 '
'27019' = '5120 '
'27025' = '5120 '
'27027' = '2520 '
'27037' = '5120 '
'27053' = '5120 '
'27055' = '3870 '
'27059' = '5120 '
'27109' = '6820 '
'27119' = '2985 '
'27123' = '5120 '
'27137' = '2240 '
'27139' = '5120 '
'27141' = '5120 '
'27145' = '6980 '
'27163' = '5120 '
'27171' = '5120 '
'28033' = '4920 '
'28035' = '3285 '
'28045' = '0920 '
'28047' = '0920 '
'28049' = '3560 '
'28059' = '0920 '
'28073' = '3285 '
'28089' = '3560 '
'28121' = '3560 '
'29003' = '7000 '
'29019' = '1740 '
'29021' = '7000 '
'29037' = '3760 '
'29043' = '7920 '
'29047' = '3760 '
'29049' = '3760 '
'29071' = '7040 '
'29077' = '7920 '
'29095' = '3760 '
'29097' = '3710 '
'29099' = '7040 '
'29107' = '3760 '
'29113' = '7040 '
'29145' = '3710 '
'29165' = '3760 '
'29177' = '3760 '
'29183' = '7040 '
'29189' = '7040 '
'29219' = '7040 '
'29225' = '7920 '
'29510' = '7040 '
'30013' = '3040 '
'30063' = '5140 '
'30111' = '0880 '
'31025' = '5920 '
'31043' = '7720 '
'31055' = '5920 '
'31109' = '4360 '
'31153' = '5920 '
'31177' = '5920 '
'32003' = '4120 '
'32023' = '4120 '
'32031' = '6720 '
'33011' = '1123 '
'33015' = '1123 '
'33017' = '1123 '
'34001' = '0560 '
'34003' = '0875 '
'34005' = '6160 '
'34007' = '6160 '
'34009' = '0560 '
'34011' = '8760 '
'34013' = '5640 '
'34015' = '6160 '
'34017' = '3640 '
'34019' = '5015 '
'34021' = '8480 '
'34023' = '5015 '
'34025' = '5190 '
'34027' = '5640 '
'34029' = '5190 '
'34031' = '0875 '
'34033' = '6160 '
'34035' = '5015 '
'34037' = '5640 '
'34039' = '5640 '
'34041' = '5640 '
'35001' = '0200 '
'35013' = '4100 '
'35028' = '7490 '
'35043' = '0200 '
'35049' = '7490 '
'35061' = '0200 '
'36001' = '0160 '
'36005' = '5600 '
'36007' = '0960 '
'36011' = '8160 '
'36013' = '3610 '
'36015' = '2335 '
'36027' = '2281 '
'36029' = '1280 '
'36037' = '6840 '
'36043' = '8680 '
'36047' = '5600 '
'36051' = '6840 '
'36053' = '8160 '
'36055' = '6840 '
'36057' = '0160 '
'36059' = '5380 '
'36061' = '5600 '
'36063' = '1280 '
'36065' = '8680 '
'36067' = '8160 '
'36069' = '6840 '
'36071' = '5660 '
'36073' = '6840 '
'36075' = '8160 '
'36079' = '5600 '
'36081' = '5600 '
'36083' = '0160 '
'36085' = '5600 '
'36087' = '5600 '
'36091' = '0160 '
'36093' = '0160 '
'36095' = '0160 '
'36103' = '5380 '
'36107' = '0960 '
'36113' = '2975 '
'36115' = '2975 '
'36117' = '6840 '
'36119' = '5600 '
'37001' = '3120 '
'37003' = '3290 '
'37019' = '9200 '
'37021' = '0480 '
'37023' = '3290 '
'37025' = '1520 '
'37027' = '3290 '
'37035' = '3290 '
'37037' = '6640 '
'37051' = '2560 '
'37053' = '5720 '
'37057' = '3120 '
'37059' = '3120 '
'37063' = '6640 '
'37065' = '6895 '
'37067' = '3120 '
'37069' = '6640 '
'37071' = '1520 '
'37081' = '3120 '
'37101' = '6640 '
'37109' = '1520 '
'37115' = '0480 '
'37119' = '1520 '
'37127' = '6895 '
'37129' = '9200 '
'37133' = '3605 '
'37135' = '6640 '
'37147' = '3150 '
'37151' = '3120 '
'37159' = '1520 '
'37169' = '3120 '
'37179' = '1520 '
'37183' = '6640 '
'37191' = '2980 '
'37197' = '3120 '
'38015' = '1010 '
'38017' = '2520 '
'38035' = '2985 '
'38059' = '1010 '
'39003' = '4320 '
'39007' = '1680 '
'39011' = '4320 '
'39013' = '9000 '
'39015' = '1640 '
'39017' = '3200 '
'39019' = '1320 '
'39023' = '2000 '
'39025' = '1640 '
'39029' = '9320 '
'39033' = '4800 '
'39035' = '1680 '
'39041' = '1840 '
'39045' = '1840 '
'39049' = '1840 '
'39051' = '8400 '
'39055' = '1680 '
'39057' = '2000 '
'39061' = '1640 '
'39081' = '8080 '
'39085' = '1680 '
'39087' = '3400 '
'39089' = '1840 '
'39093' = '1680 '
'39095' = '8400 '
'39097' = '1840 '
'39099' = '9320 '
'39103' = '1680 '
'39109' = '2000 '
'39113' = '2000 '
'39129' = '1840 '
'39133' = '0080 '
'39139' = '4800 '
'39151' = '1320 '
'39153' = '0080 '
'39155' = '9320 '
'39165' = '1640 '
'39167' = '6020 '
'39173' = '8400 '
'40017' = '5880 '
'40027' = '5880 '
'40031' = '4200 '
'40037' = '8560 '
'40047' = '2340 '
'40083' = '5880 '
'40087' = '5880 '
'40109' = '5880 '
'40113' = '8560 '
'40125' = '5880 '
'40131' = '8560 '
'40135' = '2720 '
'40143' = '8560 '
'40145' = '8560 '
'41003' = '1890 '
'41005' = '6440 '
'41009' = '6440 '
'41029' = '4890 '
'41039' = '2400 '
'41047' = '7080 '
'41051' = '6440 '
'41053' = '7080 '
'41067' = '6440 '
'41071' = '6440 '
'42003' = '6280 '
'42007' = '6280 '
'42011' = '6680 '
'42013' = '0280 '
'42017' = '6160 '
'42019' = '6280 '
'42021' = '3680 '
'42025' = '0240 '
'42027' = '8050 '
'42029' = '6160 '
'42037' = '7560 '
'42041' = '3240 '
'42043' = '3240 '
'42045' = '6160 '
'42049' = '2360 '
'42051' = '6280 '
'42069' = '7560 '
'42071' = '4000 '
'42075' = '3240 '
'42077' = '0240 '
'42079' = '7560 '
'42081' = '9140 '
'42085' = '7610 '
'42091' = '6160 '
'42095' = '0240 '
'42099' = '3240 '
'42101' = '6160 '
'42103' = '5660 '
'42111' = '3680 '
'42125' = '6280 '
'42129' = '6280 '
'42131' = '7560 '
'42133' = '9280 '
'44001' = '6483 '
'44003' = '6483 '
'44007' = '6483 '
'44009' = '6483 '
'45003' = '0600 '
'45007' = '3160 '
'45015' = '1440 '
'45019' = '1440 '
'45021' = '3160 '
'45035' = '1440 '
'45037' = '0600 '
'45041' = '2655 '
'45045' = '3160 '
'45051' = '5330 '
'45063' = '1760 '
'45077' = '3160 '
'45079' = '1760 '
'45083' = '3160 '
'45085' = '8140 '
'45091' = '1520 '
'46083' = '7760 '
'46099' = '7760 '
'46103' = '6660 '
'47001' = '3840 '
'47009' = '3840 '
'47019' = '3660 '
'47021' = '5360 '
'47023' = '3580 '
'47037' = '5360 '
'47043' = '5360 '
'47047' = '4920 '
'47065' = '1560 '
'47073' = '3660 '
'47093' = '3840 '
'47105' = '3840 '
'47113' = '3580 '
'47115' = '1560 '
'47125' = '1660 '
'47147' = '5360 '
'47149' = '5360 '
'47155' = '3840 '
'47157' = '4920 '
'47163' = '3660 '
'47165' = '5360 '
'47167' = '4920 '
'47171' = '3660 '
'47173' = '3840 '
'47179' = '3660 '
'47187' = '5360 '
'47189' = '5360 '
'48009' = '9080 '
'48021' = '0640 '
'48027' = '3810 '
'48029' = '7240 '
'48037' = '8360 '
'48039' = '1145 '
'48041' = '1260 '
'48055' = '0640 '
'48061' = '1240 '
'48071' = '3360 '
'48085' = '1920 '
'48091' = '7240 '
'48099' = '3810 '
'48113' = '1920 '
'48121' = '1920 '
'48135' = '5800 '
'48139' = '1920 '
'48141' = '2320 '
'48157' = '3360 '
'48167' = '2920 '
'48181' = '7640 '
'48183' = '4420 '
'48187' = '7240 '
'48199' = '0840 '
'48201' = '3360 '
'48203' = '4420 '
'48209' = '0640 '
'48213' = '1920 '
'48215' = '4880 '
'48221' = '2800 '
'48231' = '1920 '
'48245' = '0840 '
'48251' = '2800 '
'48257' = '1920 '
'48291' = '3360 '
'48303' = '4600 '
'48309' = '8800 '
'48329' = '5800 '
'48339' = '3360 '
'48355' = '1880 '
'48361' = '0840 '
'48367' = '2800 '
'48375' = '0320 '
'48381' = '0320 '
'48397' = '1920 '
'48409' = '1880 '
'48423' = '8640 '
'48439' = '2800 '
'48441' = '0040 '
'48451' = '7200 '
'48453' = '0640 '
'48459' = '4420 '
'48469' = '8750 '
'48473' = '3360 '
'48479' = '4080 '
'48485' = '9080 '
'48491' = '0640 '
'48493' = '7240 '
'49011' = '7160 '
'49025' = '2620 '
'49035' = '7160 '
'49049' = '6520 '
'49057' = '7160 '
'50007' = '1303 '
'50011' = '1303 '
'50013' = '1303 '
'51003' = '1540 '
'51009' = '4640 '
'51013' = '8840 '
'51019' = '4640 '
'51023' = '6800 '
'51031' = '4640 '
'51036' = '6760 '
'51041' = '6760 '
'51043' = '8840 '
'51047' = '8840 '
'51053' = '6760 '
'51059' = '8840 '
'51061' = '8840 '
'51065' = '1540 '
'51073' = '5720 '
'51075' = '6760 '
'51079' = '1540 '
'51085' = '6760 '
'51087' = '6760 '
'51093' = '5720 '
'51095' = '5720 '
'51099' = '8840 '
'51107' = '8840 '
'51115' = '5720 '
'51127' = '6760 '
'51143' = '1950 '
'51145' = '6760 '
'51149' = '6760 '
'51153' = '8840 '
'51161' = '6800 '
'51169' = '3660 '
'51177' = '8840 '
'51179' = '8840 '
'51187' = '8840 '
'51191' = '3660 '
'51199' = '5720 '
'51510' = '8840 '
'51515' = '4640 '
'51520' = '3660 '
'51540' = '1540 '
'51550' = '5720 '
'51570' = '6760 '
'51590' = '1950 '
'51600' = '8840 '
'51610' = '8840 '
'51630' = '8840 '
'51650' = '5720 '
'51670' = '6760 '
'51680' = '4640 '
'51683' = '8840 '
'51685' = '8840 '
'51700' = '5720 '
'51710' = '5720 '
'51730' = '6760 '
'51735' = '5720 '
'51740' = '5720 '
'51760' = '6760 '
'51770' = '6800 '
'51775' = '6800 '
'51800' = '5720 '
'51810' = '5720 '
'51830' = '5720 '
'53005' = '6740 '
'53011' = '6440 '
'53021' = '6740 '
'53029' = '7600 '
'53033' = '7600 '
'53035' = '1150 '
'53053' = '8200 '
'53061' = '7600 '
'53063' = '7840 '
'53067' = '5910 '
'53073' = '0860 '
'53077' = '9260 '
'54003' = '8840 '
'54009' = '8080 '
'54011' = '3400 '
'54029' = '8080 '
'54037' = '8840 '
'54039' = '1480 '
'54051' = '9000 '
'54057' = '1900 '
'54069' = '9000 '
'54079' = '1480 '
'54099' = '3400 '
'54107' = '6020 '
'55009' = '3080 '
'55015' = '0460 '
'55017' = '2290 '
'55025' = '4720 '
'55031' = '2240 '
'55035' = '2290 '
'55059' = '3800 '
'55063' = '3870 '
'55073' = '8940 '
'55079' = '5080 '
'55087' = '0460 '
'55089' = '5080 '
'55093' = '5120 '
'55101' = '6600 '
'55105' = '3620 '
'55109' = '5120 '
'55117' = '7620 '
'55131' = '5080 '
'55133' = '5080 '
'55139' = '0460 '
'56021' = '1580 '
'56025' = '1350 '
;

/* OMB 2003 METRO AREA */

VALUE $M3AREA
'01001' = '33860'
'01003' = '19300'
'01005' = '21640'
'01007' = '13820'
'01009' = '13820'
'01015' = '11500'
'01017' = '46740'
'01021' = '13820'
'01031' = '21460'
'01033' = '22520'
'01037' = '10760'
'01043' = '18980'
'01045' = '21460'
'01047' = '42820'
'01049' = '22840'
'01051' = '33860'
'01055' = '23460'
'01061' = '20020'
'01063' = '46220'
'01065' = '46220'
'01067' = '20020'
'01069' = '20020'
'01071' = '42460'
'01073' = '13820'
'01077' = '22520'
'01079' = '19460'
'01081' = '12220'
'01083' = '26620'
'01085' = '33860'
'01087' = '46260'
'01089' = '26620'
'01095' = '10700'
'01097' = '33660'
'01101' = '33860'
'01103' = '19460'
'01109' = '45980'
'01113' = '17980'
'01115' = '13820'
'01117' = '13820'
'01121' = '45180'
'01123' = '10760'
'01125' = '46220'
'01127' = '13820'
'02020' = '11260'
'02090' = '21820'
'02110' = '27940'
'02130' = '28540'
'02150' = '28980'
'02170' = '11260'
'04003' = '43420'
'04005' = '22380'
'04007' = '37740'
'04009' = '40940'
'04011' = '40940'
'04013' = '38060'
'04015' = '29420'
'04019' = '46060'
'04021' = '38060'
'04023' = '35700'
'04025' = '39140'
'04027' = '49740'
'05005' = '34260'
'05007' = '22220'
'05009' = '25460'
'05013' = '15780'
'05019' = '11660'
'05025' = '38220'
'05027' = '31620'
'05031' = '27860'
'05033' = '22900'
'05035' = '32820'
'05045' = '30780'
'05047' = '22900'
'05051' = '26300'
'05053' = '30780'
'05055' = '37500'
'05057' = '26260'
'05063' = '12900'
'05069' = '38220'
'05079' = '38220'
'05085' = '30780'
'05087' = '22220'
'05091' = '45500'
'05093' = '14180'
'05099' = '26260'
'05101' = '25460'
'05103' = '15780'
'05105' = '30780'
'05107' = '48340'
'05111' = '27860'
'05115' = '40780'
'05119' = '30780'
'05123' = '22620'
'05125' = '30780'
'05131' = '22900'
'05139' = '20980'
'05143' = '22220'
'05145' = '42620'
'05149' = '40780'
'06001' = '41860'
'06007' = '17020'
'06013' = '41860'
'06015' = '18860'
'06017' = '40900'
'06019' = '23420'
'06023' = '21700'
'06025' = '20940'
'06027' = '13860'
'06029' = '12540'
'06031' = '25260'
'06033' = '17340'
'06035' = '45000'
'06037' = '31100'
'06039' = '31460'
'06041' = '41860'
'06045' = '46380'
'06047' = '32900'
'06053' = '41500'
'06055' = '34900'
'06057' = '46020'
'06059' = '31100'
'06061' = '40900'
'06065' = '40140'
'06067' = '40900'
'06069' = '41940'
'06071' = '40140'
'06073' = '41740'
'06075' = '41860'
'06077' = '44700'
'06079' = '42020'
'06081' = '41860'
'06083' = '42060'
'06085' = '41940'
'06087' = '42100'
'06089' = '39820'
'06095' = '46700'
'06097' = '42220'
'06099' = '33700'
'06101' = '49700'
'06103' = '39780'
'06107' = '47300'
'06109' = '38020'
'06111' = '37100'
'06113' = '40900'
'06115' = '49700'
'08001' = '19740'
'08005' = '19740'
'08013' = '14500'
'08014' = '19740'
'08019' = '19740'
'08031' = '19740'
'08035' = '19740'
'08037' = '20780'
'08039' = '19740'
'08041' = '17820'
'08043' = '15860'
'08047' = '19740'
'08059' = '19740'
'08065' = '20780'
'08067' = '20420'
'08069' = '22660'
'08075' = '44540'
'08077' = '24300'
'08085' = '33940'
'08087' = '22820'
'08093' = '19740'
'08101' = '39380'
'08117' = '43540'
'08119' = '17820'
'08123' = '24540'
'09001' = '14860'
'09003' = '25540'
'09005' = '45860'
'09007' = '25540'
'09009' = '35300'
'09011' = '35980'
'09013' = '25540'
'09015' = '48740'
'10001' = '20100'
'10003' = '37980'
'10005' = '42580'
'11001' = '47900'
'12001' = '23540'
'12003' = '27260'
'12005' = '37460'
'12009' = '37340'
'12011' = '33100'
'12015' = '39460'
'12017' = '26140'
'12019' = '27260'
'12021' = '34940'
'12023' = '29380'
'12027' = '11580'
'12031' = '27260'
'12033' = '37860'
'12035' = '37380'
'12039' = '45220'
'12041' = '23540'
'12049' = '48100'
'12051' = '17500'
'12053' = '45300'
'12055' = '42700'
'12057' = '45300'
'12061' = '46940'
'12065' = '45220'
'12069' = '36740'
'12071' = '15980'
'12073' = '45220'
'12081' = '42260'
'12083' = '36100'
'12085' = '38940'
'12086' = '33100'
'12087' = '28580'
'12089' = '27260'
'12091' = '23020'
'12093' = '36380'
'12095' = '36740'
'12097' = '36740'
'12099' = '33100'
'12101' = '45300'
'12103' = '45300'
'12105' = '29460'
'12107' = '37260'
'12109' = '27260'
'12111' = '38940'
'12113' = '37860'
'12115' = '42260'
'12117' = '36740'
'12119' = '45540'
'12127' = '19660'
'12129' = '45220'
'13003' = '20060'
'13007' = '10500'
'13009' = '33300'
'13013' = '12060'
'13015' = '12060'
'13017' = '22340'
'13021' = '31420'
'13025' = '15260'
'13027' = '46660'
'13029' = '42340'
'13031' = '44340'
'13033' = '12260'
'13035' = '12060'
'13039' = '41220'
'13045' = '12060'
'13047' = '16860'
'13051' = '42340'
'13053' = '17980'
'13055' = '44900'
'13057' = '12060'
'13059' = '12020'
'13063' = '12060'
'13067' = '12060'
'13069' = '20060'
'13071' = '34220'
'13073' = '12260'
'13077' = '12060'
'13079' = '31420'
'13081' = '18380'
'13083' = '16860'
'13085' = '12060'
'13087' = '12460'
'13089' = '12060'
'13095' = '10500'
'13097' = '12060'
'13101' = '46660'
'13103' = '42340'
'13113' = '12060'
'13115' = '40660'
'13117' = '12060'
'13121' = '12060'
'13127' = '15260'
'13129' = '15660'
'13135' = '12060'
'13137' = '18460'
'13139' = '23580'
'13141' = '33300'
'13143' = '12060'
'13145' = '17980'
'13149' = '12060'
'13151' = '12060'
'13153' = '47580'
'13155' = '22340'
'13159' = '12060'
'13167' = '20140'
'13169' = '31420'
'13171' = '12060'
'13173' = '46660'
'13175' = '20140'
'13177' = '10500'
'13179' = '25980'
'13183' = '25980'
'13185' = '46660'
'13189' = '12260'
'13191' = '15260'
'13195' = '12020'
'13197' = '17980'
'13199' = '12060'
'13207' = '31420'
'13209' = '47080'
'13213' = '19140'
'13215' = '17980'
'13217' = '12060'
'13219' = '12020'
'13221' = '12020'
'13223' = '12060'
'13225' = '22980'
'13227' = '12060'
'13229' = '48180'
'13231' = '12060'
'13233' = '16340'
'13239' = '21640'
'13245' = '12260'
'13247' = '12060'
'13249' = '11140'
'13255' = '12060'
'13257' = '45740'
'13261' = '11140'
'13273' = '10500'
'13275' = '45620'
'13277' = '45700'
'13279' = '47080'
'13285' = '29300'
'13289' = '31420'
'13293' = '45580'
'13295' = '16860'
'13297' = '12060'
'13299' = '48180'
'13305' = '27700'
'13313' = '19140'
'13321' = '10500'
'15001' = '25900'
'15003' = '26180'
'15007' = '28180'
'15009' = '27980'
'16001' = '14260'
'16005' = '38540'
'16011' = '13940'
'16015' = '14260'
'16019' = '26820'
'16027' = '14260'
'16031' = '15420'
'16039' = '34300'
'16041' = '30860'
'16043' = '39940'
'16045' = '14260'
'16051' = '26820'
'16053' = '46300'
'16055' = '17660'
'16057' = '34140'
'16065' = '39940'
'16067' = '15420'
'16069' = '30300'
'16073' = '14260'
'16075' = '36620'
'16077' = '38540'
'16081' = '27220'
'16083' = '46300'
'17001' = '39500'
'17003' = '16020'
'17005' = '41180'
'17007' = '40420'
'17011' = '36860'
'17013' = '41180'
'17019' = '16580'
'17021' = '45380'
'17027' = '41180'
'17029' = '16660'
'17031' = '16980'
'17035' = '16660'
'17037' = '16980'
'17043' = '16980'
'17049' = '20820'
'17053' = '16580'
'17057' = '15900'
'17063' = '16980'
'17065' = '34500'
'17071' = '15460'
'17073' = '19340'
'17077' = '16060'
'17081' = '34500'
'17083' = '41180'
'17089' = '16980'
'17091' = '28100'
'17093' = '16980'
'17095' = '23660'
'17097' = '16980'
'17099' = '36860'
'17103' = '19940'
'17105' = '38700'
'17107' = '30660'
'17109' = '31380'
'17111' = '16980'
'17113' = '14060'
'17115' = '19500'
'17117' = '41180'
'17119' = '41180'
'17121' = '16460'
'17123' = '37900'
'17127' = '37140'
'17129' = '44100'
'17131' = '19340'
'17133' = '41180'
'17137' = '27300'
'17141' = '40300'
'17143' = '37900'
'17147' = '16580'
'17155' = '36860'
'17161' = '19340'
'17163' = '41180'
'17165' = '25380'
'17167' = '44100'
'17171' = '27300'
'17175' = '37900'
'17177' = '23300'
'17179' = '37900'
'17183' = '19180'
'17187' = '23660'
'17195' = '44580'
'17197' = '16980'
'17199' = '32060'
'17201' = '40420'
'17203' = '37900'
'18001' = '19540'
'18003' = '23060'
'18005' = '18020'
'18007' = '29140'
'18011' = '26900'
'18013' = '26900'
'18015' = '29140'
'18017' = '30900'
'18019' = '31140'
'18021' = '45460'
'18023' = '23140'
'18027' = '47780'
'18029' = '17140'
'18031' = '24700'
'18033' = '12140'
'18035' = '34620'
'18037' = '27540'
'18039' = '21140'
'18041' = '18220'
'18043' = '31140'
'18047' = '17140'
'18051' = '21780'
'18053' = '31980'
'18055' = '14020'
'18057' = '26900'
'18059' = '26900'
'18061' = '31140'
'18063' = '26900'
'18065' = '35220'
'18067' = '29020'
'18069' = '26540'
'18071' = '42980'
'18073' = '16980'
'18077' = '31500'
'18079' = '35860'
'18081' = '26900'
'18083' = '47180'
'18085' = '47700'
'18089' = '16980'
'18091' = '33140'
'18093' = '13260'
'18095' = '11300'
'18097' = '26900'
'18099' = '38500'
'18103' = '37940'
'18105' = '14020'
'18107' = '18820'
'18109' = '26900'
'18111' = '16980'
'18113' = '28340'
'18115' = '17140'
'18119' = '14020'
'18125' = '27540'
'18127' = '16980'
'18129' = '21780'
'18133' = '26900'
'18141' = '43780'
'18143' = '42500'
'18145' = '26900'
'18151' = '11420'
'18153' = '45460'
'18157' = '29140'
'18159' = '29020'
'18163' = '21780'
'18165' = '45460'
'18167' = '45460'
'18169' = '47340'
'18173' = '21780'
'18175' = '31140'
'18177' = '39980'
'18179' = '23060'
'18183' = '23060'
'19011' = '16300'
'19013' = '47940'
'19015' = '14340'
'19017' = '47940'
'19021' = '44740'
'19033' = '32380'
'19041' = '43980'
'19045' = '17540'
'19049' = '19780'
'19057' = '15460'
'19059' = '44020'
'19061' = '20220'
'19075' = '47940'
'19077' = '19780'
'19085' = '36540'
'19099' = '35500'
'19103' = '26980'
'19105' = '16300'
'19111' = '28460'
'19113' = '16300'
'19115' = '34700'
'19121' = '19780'
'19123' = '36820'
'19125' = '37800'
'19127' = '32260'
'19129' = '36540'
'19139' = '34700'
'19153' = '19780'
'19155' = '36540'
'19163' = '19340'
'19169' = '11180'
'19179' = '36900'
'19181' = '19780'
'19183' = '26980'
'19187' = '22700'
'19193' = '43580'
'19195' = '32380'
'20005' = '11860'
'20009' = '24460'
'20015' = '48620'
'20017' = '21380'
'20035' = '49060'
'20037' = '38260'
'20043' = '41140'
'20045' = '29940'
'20051' = '25700'
'20055' = '23780'
'20057' = '19980'
'20059' = '28140'
'20061' = '31740'
'20079' = '48620'
'20085' = '45820'
'20087' = '45820'
'20091' = '28140'
'20099' = '37660'
'20103' = '28140'
'20107' = '28140'
'20111' = '21380'
'20113' = '32700'
'20121' = '28140'
'20125' = '17700'
'20139' = '45820'
'20143' = '41460'
'20149' = '31740'
'20155' = '26740'
'20161' = '31740'
'20169' = '41460'
'20173' = '48620'
'20175' = '30580'
'20177' = '45820'
'20191' = '48620'
'20197' = '45820'
'20209' = '28140'
'21005' = '23180'
'21007' = '37140'
'21009' = '23980'
'21011' = '34460'
'21013' = '33180'
'21015' = '17140'
'21017' = '30460'
'21019' = '26580'
'21021' = '19220'
'21023' = '17140'
'21029' = '31140'
'21035' = '34660'
'21037' = '17140'
'21047' = '17300'
'21049' = '30460'
'21059' = '36980'
'21061' = '14540'
'21067' = '30460'
'21073' = '23180'
'21075' = '46460'
'21077' = '17140'
'21081' = '17140'
'21083' = '32460'
'21089' = '26580'
'21091' = '36980'
'21093' = '21060'
'21101' = '21780'
'21103' = '31140'
'21107' = '31580'
'21111' = '31140'
'21113' = '30460'
'21117' = '17140'
'21123' = '21060'
'21125' = '30940'
'21135' = '32500'
'21137' = '19220'
'21139' = '37140'
'21145' = '37140'
'21149' = '36980'
'21151' = '40080'
'21161' = '32500'
'21163' = '31140'
'21165' = '34460'
'21169' = '23980'
'21173' = '34460'
'21177' = '16420'
'21179' = '31140'
'21185' = '31140'
'21191' = '17140'
'21199' = '43700'
'21203' = '40080'
'21209' = '30460'
'21211' = '31140'
'21215' = '31140'
'21217' = '15820'
'21221' = '17300'
'21223' = '31140'
'21227' = '14540'
'21233' = '21780'
'21235' = '18340'
'21239' = '30460'
'22001' = '18940'
'22005' = '12940'
'22007' = '38200'
'22011' = '19760'
'22015' = '43340'
'22017' = '43340'
'22019' = '29340'
'22023' = '29340'
'22029' = '35020'
'22031' = '43340'
'22033' = '12940'
'22037' = '12940'
'22043' = '10780'
'22045' = '35340'
'22047' = '12940'
'22049' = '40820'
'22051' = '35380'
'22053' = '27660'
'22055' = '29180'
'22057' = '26380'
'22061' = '40820'
'22063' = '12940'
'22065' = '45260'
'22067' = '12820'
'22069' = '35060'
'22071' = '35380'
'22073' = '33740'
'22075' = '35380'
'22077' = '12940'
'22079' = '10780'
'22087' = '35380'
'22089' = '35380'
'22091' = '12940'
'22095' = '35380'
'22097' = '36660'
'22099' = '29180'
'22101' = '34020'
'22103' = '35380'
'22105' = '25220'
'22109' = '26380'
'22111' = '33740'
'22113' = '10020'
'22115' = '22860'
'22117' = '14220'
'22119' = '33380'
'22121' = '12940'
'22125' = '12940'
'23001' = '30340'
'23005' = '38860'
'23011' = '12300'
'23013' = '40500'
'23019' = '12620'
'23023' = '38860'
'23031' = '38860'
'24001' = '19060'
'24003' = '12580'
'24005' = '12580'
'24009' = '47900'
'24013' = '12580'
'24015' = '37980'
'24017' = '47900'
'24019' = '15700'
'24021' = '47900'
'24025' = '12580'
'24027' = '12580'
'24031' = '47900'
'24033' = '47900'
'24035' = '12580'
'24037' = '30500'
'24039' = '41540'
'24041' = '20660'
'24043' = '25180'
'24045' = '41540'
'24047' = '36180'
'24510' = '12580'
'25001' = '12700'
'25003' = '38340'
'25005' = '39300'
'25009' = '14460'
'25011' = '44140'
'25013' = '44140'
'25015' = '44140'
'25017' = '14460'
'25021' = '14460'
'25023' = '14460'
'25025' = '14460'
'25027' = '49340'
'26005' = '10880'
'26007' = '10980'
'26015' = '24340'
'26017' = '13020'
'26019' = '45900'
'26021' = '35660'
'26023' = '17740'
'26025' = '12980'
'26027' = '43780'
'26033' = '42300'
'26037' = '29620'
'26041' = '21540'
'26043' = '27020'
'26045' = '29620'
'26049' = '22420'
'26055' = '45900'
'26057' = '10940'
'26061' = '26340'
'26065' = '29620'
'26067' = '24340'
'26073' = '34380'
'26075' = '27100'
'26077' = '28020'
'26079' = '45900'
'26081' = '24340'
'26083' = '26340'
'26087' = '19820'
'26089' = '45900'
'26091' = '10300'
'26093' = '19820'
'26099' = '19820'
'26103' = '32100'
'26107' = '13660'
'26109' = '31940'
'26111' = '33220'
'26113' = '15620'
'26115' = '33780'
'26121' = '34740'
'26123' = '24340'
'26125' = '19820'
'26139' = '26100'
'26145' = '40980'
'26147' = '19820'
'26149' = '44780'
'26155' = '37020'
'26159' = '28020'
'26161' = '11460'
'26163' = '19820'
'26165' = '15620'
'27003' = '33460'
'27007' = '13420'
'27009' = '41060'
'27013' = '31860'
'27015' = '35580'
'27017' = '20260'
'27019' = '33460'
'27021' = '14660'
'27025' = '33460'
'27027' = '22020'
'27035' = '14660'
'27037' = '33460'
'27039' = '40340'
'27041' = '10820'
'27047' = '10660'
'27049' = '39860'
'27053' = '33460'
'27055' = '29100'
'27059' = '33460'
'27067' = '48820'
'27083' = '32140'
'27085' = '26780'
'27091' = '21860'
'27099' = '12380'
'27103' = '31860'
'27105' = '49380'
'27109' = '40340'
'27111' = '22260'
'27119' = '24220'
'27123' = '33460'
'27131' = '22060'
'27137' = '20260'
'27139' = '33460'
'27141' = '33460'
'27145' = '41060'
'27147' = '36940'
'27157' = '40340'
'27163' = '33460'
'27167' = '47420'
'27169' = '49100'
'27171' = '33460'
'28001' = '35020'
'28003' = '18420'
'28005' = '32620'
'28011' = '17380'
'28015' = '24900'
'28023' = '32940'
'28025' = '48500'
'28027' = '17260'
'28029' = '27140'
'28033' = '32820'
'28035' = '25620'
'28039' = '37700'
'28043' = '24980'
'28045' = '25060'
'28047' = '25060'
'28049' = '27140'
'28057' = '46180'
'28059' = '37700'
'28061' = '29860'
'28067' = '29860'
'28069' = '32940'
'28071' = '37060'
'28073' = '25620'
'28075' = '32940'
'28081' = '46180'
'28083' = '24900'
'28085' = '15020'
'28087' = '18060'
'28089' = '27140'
'28093' = '32820'
'28105' = '44260'
'28109' = '38100'
'28111' = '25620'
'28113' = '32620'
'28115' = '46180'
'28121' = '27140'
'28127' = '27140'
'28131' = '25060'
'28133' = '26940'
'28137' = '32820'
'28143' = '32820'
'28149' = '46980'
'28151' = '24740'
'28163' = '49540'
'29001' = '28860'
'29003' = '41140'
'29007' = '33020'
'29013' = '28140'
'29017' = '16020'
'29019' = '17860'
'29021' = '41140'
'29023' = '38740'
'29025' = '28140'
'29027' = '27620'
'29031' = '16020'
'29037' = '28140'
'29043' = '44180'
'29045' = '28460'
'29047' = '28140'
'29049' = '28140'
'29051' = '27620'
'29059' = '44180'
'29063' = '41140'
'29069' = '28380'
'29071' = '41180'
'29077' = '44180'
'29089' = '17860'
'29091' = '48460'
'29095' = '28140'
'29097' = '27900'
'29099' = '41180'
'29101' = '47660'
'29105' = '30060'
'29107' = '28140'
'29111' = '39500'
'29113' = '41180'
'29119' = '22220'
'29127' = '25300'
'29135' = '27620'
'29145' = '27900'
'29147' = '32340'
'29151' = '27620'
'29159' = '42740'
'29161' = '40620'
'29165' = '28140'
'29167' = '44180'
'29169' = '22780'
'29173' = '25300'
'29175' = '33620'
'29177' = '28140'
'29183' = '41180'
'29187' = '22100'
'29189' = '41180'
'29195' = '32180'
'29197' = '28860'
'29201' = '43460'
'29209' = '14700'
'29213' = '14700'
'29219' = '41180'
'29221' = '41180'
'29225' = '44180'
'29510' = '41180'
'30009' = '13740'
'30013' = '24500'
'30029' = '28060'
'30031' = '14580'
'30041' = '25660'
'30043' = '25740'
'30049' = '25740'
'30063' = '33540'
'30093' = '15580'
'30111' = '13740'
'31001' = '25580'
'31007' = '42420'
'31019' = '28260'
'31025' = '36540'
'31035' = '25580'
'31043' = '43580'
'31047' = '30420'
'31051' = '43580'
'31053' = '23340'
'31055' = '36540'
'31067' = '13100'
'31073' = '30420'
'31079' = '24260'
'31093' = '24260'
'31099' = '28260'
'31109' = '30700'
'31111' = '35820'
'31113' = '35820'
'31117' = '35820'
'31119' = '35740'
'31121' = '24260'
'31139' = '35740'
'31141' = '18100'
'31153' = '36540'
'31155' = '36540'
'31157' = '42420'
'31159' = '30700'
'31167' = '35740'
'31177' = '36540'
'32001' = '21980'
'32003' = '29820'
'32005' = '23820'
'32007' = '21220'
'32011' = '21220'
'32023' = '37220'
'32029' = '39900'
'32031' = '39900'
'32510' = '16180'
'33001' = '29060'
'33005' = '28300'
'33007' = '13620'
'33009' = '30100'
'33011' = '31700'
'33013' = '18180'
'33015' = '14460'
'33017' = '14460'
'33019' = '17200'
'34001' = '12100'
'34003' = '35620'
'34005' = '37980'
'34007' = '37980'
'34009' = '36140'
'34011' = '47220'
'34013' = '35620'
'34015' = '37980'
'34017' = '35620'
'34019' = '35620'
'34021' = '45940'
'34023' = '35620'
'34025' = '35620'
'34027' = '35620'
'34029' = '35620'
'34031' = '35620'
'34033' = '37980'
'34035' = '35620'
'34037' = '35620'
'34039' = '35620'
'34041' = '10900'
'35001' = '10740'
'35005' = '40740'
'35006' = '24380'
'35009' = '17580'
'35013' = '29740'
'35015' = '16100'
'35017' = '43500'
'35025' = '26020'
'35028' = '31060'
'35029' = '19700'
'35031' = '23700'
'35035' = '10460'
'35039' = '21580'
'35041' = '38780'
'35043' = '10740'
'35045' = '22140'
'35047' = '29780'
'35049' = '42140'
'35055' = '45340'
'35057' = '10740'
'35061' = '10740'
'36001' = '10580'
'36005' = '35620'
'36007' = '13780'
'36009' = '36460'
'36011' = '12180'
'36013' = '27460'
'36015' = '21300'
'36019' = '38460'
'36021' = '26460'
'36023' = '18660'
'36027' = '39100'
'36029' = '15380'
'36033' = '31660'
'36035' = '24100'
'36037' = '12860'
'36043' = '46540'
'36045' = '48060'
'36047' = '35620'
'36051' = '40380'
'36053' = '45060'
'36055' = '40380'
'36057' = '11220'
'36059' = '35620'
'36061' = '35620'
'36063' = '15380'
'36065' = '46540'
'36067' = '45060'
'36069' = '40380'
'36071' = '39100'
'36073' = '40380'
'36075' = '45060'
'36077' = '36580'
'36079' = '35620'
'36081' = '35620'
'36083' = '10580'
'36085' = '35620'
'36087' = '35620'
'36089' = '36300'
'36091' = '10580'
'36093' = '10580'
'36095' = '10580'
'36099' = '42900'
'36101' = '18500'
'36103' = '35620'
'36107' = '13780'
'36109' = '27060'
'36111' = '28740'
'36113' = '24020'
'36115' = '24020'
'36117' = '40380'
'36119' = '35620'
'37001' = '15500'
'37003' = '25860'
'37007' = '16740'
'37013' = '47820'
'37019' = '48900'
'37021' = '11700'
'37023' = '25860'
'37025' = '16740'
'37027' = '25860'
'37029' = '21020'
'37031' = '33980'
'37035' = '25860'
'37037' = '20500'
'37045' = '43140'
'37049' = '35100'
'37051' = '22180'
'37053' = '47260'
'37055' = '28620'
'37057' = '45640'
'37059' = '49180'
'37063' = '20500'
'37065' = '40580'
'37067' = '49180'
'37069' = '39580'
'37071' = '16740'
'37079' = '24780'
'37081' = '24660'
'37083' = '40260'
'37085' = '20380'
'37087' = '11700'
'37089' = '11700'
'37093' = '22180'
'37097' = '44380'
'37101' = '39580'
'37103' = '35100'
'37105' = '41820'
'37107' = '28820'
'37109' = '30740'
'37115' = '11700'
'37119' = '16740'
'37125' = '43860'
'37127' = '40580'
'37129' = '48900'
'37131' = '40260'
'37133' = '27340'
'37135' = '20500'
'37137' = '35100'
'37139' = '21020'
'37141' = '48900'
'37143' = '21020'
'37145' = '20500'
'37147' = '24780'
'37151' = '24660'
'37153' = '40460'
'37155' = '31300'
'37157' = '24660'
'37159' = '41580'
'37161' = '22580'
'37165' = '29900'
'37167' = '10620'
'37169' = '49180'
'37171' = '34340'
'37175' = '14820'
'37179' = '16740'
'37181' = '25780'
'37183' = '39580'
'37189' = '14380'
'37191' = '24140'
'37193' = '35900'
'37195' = '48980'
'37197' = '49180'
'38007' = '19860'
'38015' = '13900'
'38017' = '22020'
'38035' = '24220'
'38049' = '33500'
'38059' = '13900'
'38075' = '33500'
'38077' = '47420'
'38089' = '19860'
'38093' = '27420'
'38101' = '33500'
'38105' = '48780'
'39003' = '30620'
'39005' = '11740'
'39007' = '11780'
'39009' = '11900'
'39011' = '47540'
'39013' = '48540'
'39015' = '17140'
'39017' = '17140'
'39019' = '15940'
'39021' = '46500'
'39023' = '44220'
'39025' = '17140'
'39027' = '48940'
'39029' = '20620'
'39031' = '18740'
'39033' = '15340'
'39035' = '17460'
'39037' = '24820'
'39039' = '19580'
'39041' = '18140'
'39043' = '41780'
'39045' = '18140'
'39047' = '47860'
'39049' = '18140'
'39051' = '45780'
'39053' = '38580'
'39055' = '17460'
'39057' = '19380'
'39059' = '15740'
'39061' = '17140'
'39063' = '22300'
'39077' = '35940'
'39081' = '48260'
'39083' = '34540'
'39085' = '17460'
'39087' = '26580'
'39089' = '18140'
'39091' = '13340'
'39093' = '17460'
'39095' = '45780'
'39097' = '18140'
'39099' = '49660'
'39101' = '32020'
'39103' = '17460'
'39107' = '16380'
'39109' = '19380'
'39113' = '19380'
'39117' = '18140'
'39119' = '49780'
'39123' = '45780'
'39129' = '18140'
'39133' = '10420'
'39135' = '19380'
'39139' = '31900'
'39141' = '17060'
'39143' = '23380'
'39145' = '39020'
'39147' = '45660'
'39149' = '43380'
'39151' = '15940'
'39153' = '10420'
'39155' = '49660'
'39157' = '35420'
'39159' = '18140'
'39161' = '46780'
'39165' = '17140'
'39167' = '37620'
'39169' = '49300'
'39173' = '45780'
'40009' = '21120'
'40013' = '20460'
'40017' = '36420'
'40019' = '11620'
'40021' = '45140'
'40027' = '36420'
'40031' = '30020'
'40037' = '46140'
'40047' = '21420'
'40051' = '36420'
'40065' = '11060'
'40071' = '38620'
'40079' = '22900'
'40081' = '36420'
'40083' = '36420'
'40085' = '11620'
'40087' = '36420'
'40101' = '34780'
'40109' = '36420'
'40111' = '46140'
'40113' = '46140'
'40115' = '33060'
'40117' = '46140'
'40119' = '44660'
'40121' = '32540'
'40123' = '10220'
'40125' = '43060'
'40131' = '46140'
'40135' = '22900'
'40137' = '20340'
'40139' = '25100'
'40143' = '46140'
'40145' = '46140'
'40147' = '12780'
'40153' = '49260'
'41003' = '18700'
'41005' = '38900'
'41007' = '11820'
'41009' = '38900'
'41011' = '18300'
'41013' = '39260'
'41015' = '15060'
'41017' = '13460'
'41019' = '40700'
'41027' = '26220'
'41029' = '32780'
'41033' = '24420'
'41035' = '28900'
'41039' = '21660'
'41043' = '10540'
'41045' = '36620'
'41047' = '41420'
'41049' = '37820'
'41051' = '38900'
'41053' = '41420'
'41059' = '37820'
'41061' = '29260'
'41065' = '17180'
'41067' = '38900'
'41071' = '38900'
'42001' = '23900'
'42003' = '38300'
'42005' = '38300'
'42007' = '38300'
'42011' = '39740'
'42013' = '11020'
'42015' = '42380'
'42017' = '37980'
'42019' = '38300'
'42021' = '27780'
'42025' = '10900'
'42027' = '44300'
'42029' = '37980'
'42033' = '20180'
'42035' = '30820'
'42037' = '14100'
'42039' = '32740'
'42041' = '25420'
'42043' = '25420'
'42045' = '37980'
'42047' = '41260'
'42049' = '21500'
'42051' = '38300'
'42055' = '16540'
'42061' = '26500'
'42063' = '26860'
'42069' = '42540'
'42071' = '29540'
'42073' = '35260'
'42075' = '30140'
'42077' = '10900'
'42079' = '42540'
'42081' = '48700'
'42083' = '14620'
'42085' = '49660'
'42087' = '30380'
'42089' = '20700'
'42091' = '37980'
'42093' = '14100'
'42095' = '10900'
'42097' = '44980'
'42099' = '25420'
'42101' = '37980'
'42103' = '35620'
'42107' = '39060'
'42109' = '42780'
'42111' = '43740'
'42119' = '30260'
'42121' = '36340'
'42123' = '47620'
'42125' = '38300'
'42129' = '38300'
'42131' = '42540'
'42133' = '49620'
'44001' = '39300'
'44003' = '39300'
'44005' = '39300'
'44007' = '39300'
'44009' = '39300'
'45003' = '12260'
'45007' = '11340'
'45013' = '25940'
'45015' = '16700'
'45017' = '17900'
'45019' = '16700'
'45021' = '23500'
'45023' = '16900'
'45029' = '47500'
'45031' = '22500'
'45033' = '19900'
'45035' = '16700'
'45037' = '12260'
'45039' = '17900'
'45041' = '22500'
'45043' = '23860'
'45045' = '24860'
'45047' = '24940'
'45051' = '34820'
'45053' = '25940'
'45055' = '17900'
'45057' = '29580'
'45059' = '24860'
'45063' = '17900'
'45069' = '13500'
'45071' = '35140'
'45073' = '42860'
'45075' = '36700'
'45077' = '24860'
'45079' = '17900'
'45081' = '17900'
'45083' = '43900'
'45085' = '44940'
'45087' = '46420'
'45091' = '16740'
'46005' = '26700'
'46011' = '15100'
'46013' = '10100'
'46027' = '46820'
'46029' = '47980'
'46035' = '33580'
'46045' = '10100'
'46057' = '47980'
'46061' = '33580'
'46065' = '38180'
'46081' = '43940'
'46083' = '43620'
'46087' = '43620'
'46093' = '39660'
'46099' = '43620'
'46103' = '39660'
'46117' = '38180'
'46125' = '43620'
'46127' = '43580'
'46135' = '49460'
'47001' = '28940'
'47003' = '43180'
'47009' = '28940'
'47011' = '17420'
'47013' = '29220'
'47015' = '34980'
'47019' = '27740'
'47021' = '34980'
'47023' = '27180'
'47029' = '35460'
'47031' = '46100'
'47035' = '18900'
'47037' = '34980'
'47043' = '34980'
'47045' = '20540'
'47047' = '32820'
'47051' = '46100'
'47053' = '26480'
'47057' = '34100'
'47059' = '24620'
'47063' = '34100'
'47065' = '16860'
'47073' = '28700'
'47075' = '15140'
'47079' = '37540'
'47081' = '34980'
'47087' = '18260'
'47089' = '34100'
'47093' = '28940'
'47099' = '29980'
'47105' = '28940'
'47107' = '11940'
'47111' = '34980'
'47113' = '27180'
'47115' = '16860'
'47117' = '30280'
'47119' = '17940'
'47125' = '17300'
'47127' = '46100'
'47131' = '46460'
'47133' = '18260'
'47139' = '17420'
'47141' = '18260'
'47145' = '25340'
'47147' = '34980'
'47149' = '34980'
'47153' = '16860'
'47155' = '42940'
'47157' = '32820'
'47159' = '34980'
'47161' = '17300'
'47163' = '28700'
'47165' = '34980'
'47167' = '32820'
'47169' = '34980'
'47171' = '27740'
'47173' = '28940'
'47177' = '32660'
'47179' = '27740'
'47183' = '32280'
'47187' = '34980'
'47189' = '34980'
'48001' = '37300'
'48003' = '11380'
'48005' = '31260'
'48007' = '18580'
'48009' = '48660'
'48011' = '11100'
'48013' = '41700'
'48015' = '26420'
'48019' = '41700'
'48021' = '12420'
'48025' = '13300'
'48027' = '28660'
'48029' = '41700'
'48037' = '45500'
'48039' = '26420'
'48041' = '17780'
'48049' = '15220'
'48051' = '17780'
'48055' = '12420'
'48057' = '47020'
'48059' = '10180'
'48061' = '15180'
'48065' = '11100'
'48071' = '26420'
'48073' = '27380'
'48077' = '48660'
'48085' = '19100'
'48091' = '41700'
'48097' = '23620'
'48099' = '28660'
'48107' = '31180'
'48113' = '19100'
'48115' = '29500'
'48117' = '25820'
'48119' = '19100'
'48121' = '19100'
'48135' = '36220'
'48139' = '19100'
'48141' = '21340'
'48143' = '44500'
'48147' = '14300'
'48157' = '26420'
'48167' = '26420'
'48175' = '47020'
'48179' = '37420'
'48181' = '43300'
'48183' = '30980'
'48187' = '41700'
'48189' = '38380'
'48199' = '13140'
'48201' = '26420'
'48203' = '32220'
'48209' = '12420'
'48213' = '11980'
'48215' = '32580'
'48219' = '30220'
'48221' = '24180'
'48223' = '44860'
'48227' = '13700'
'48231' = '19100'
'48233' = '14420'
'48235' = '41660'
'48245' = '13140'
'48249' = '10860'
'48251' = '19100'
'48253' = '10180'
'48257' = '19100'
'48259' = '41700'
'48261' = '28780'
'48265' = '28500'
'48273' = '28780'
'48277' = '37580'
'48281' = '28660'
'48291' = '26420'
'48303' = '31180'
'48309' = '47380'
'48321' = '13060'
'48323' = '20580'
'48325' = '41700'
'48329' = '33260'
'48339' = '26420'
'48341' = '20300'
'48347' = '34860'
'48349' = '18620'
'48353' = '45020'
'48355' = '18580'
'48361' = '13140'
'48363' = '33420'
'48367' = '19100'
'48375' = '11100'
'48381' = '11100'
'48389' = '37780'
'48393' = '37420'
'48395' = '17780'
'48397' = '19100'
'48401' = '30980'
'48407' = '26420'
'48409' = '18580'
'48415' = '43660'
'48423' = '46340'
'48425' = '24180'
'48427' = '40100'
'48439' = '19100'
'48441' = '10180'
'48449' = '34420'
'48451' = '41660'
'48453' = '12420'
'48459' = '30980'
'48463' = '46620'
'48465' = '19620'
'48469' = '47020'
'48471' = '26660'
'48473' = '26420'
'48477' = '14780'
'48479' = '29700'
'48481' = '20900'
'48485' = '48660'
'48487' = '46900'
'48489' = '39700'
'48491' = '12420'
'48493' = '41700'
'48497' = '19100'
'49003' = '14940'
'49005' = '30860'
'49007' = '39220'
'49011' = '36260'
'49021' = '16260'
'49023' = '39340'
'49029' = '36260'
'49035' = '41620'
'49043' = '41620'
'49045' = '41620'
'49047' = '46860'
'49049' = '39340'
'49051' = '25720'
'49053' = '41100'
'49057' = '36260'
'50003' = '13540'
'50007' = '15540'
'50009' = '13620'
'50011' = '15540'
'50013' = '15540'
'50017' = '30100'
'50021' = '40860'
'50023' = '12740'
'50027' = '30100'
'51003' = '16820'
'51007' = '40060'
'51009' = '31340'
'51011' = '31340'
'51013' = '47900'
'51015' = '44420'
'51019' = '31340'
'51023' = '40220'
'51031' = '31340'
'51033' = '40060'
'51036' = '40060'
'51041' = '40060'
'51043' = '47900'
'51045' = '40220'
'51047' = '19020'
'51049' = '40060'
'51053' = '40060'
'51059' = '47900'
'51061' = '47900'
'51065' = '16820'
'51067' = '40220'
'51069' = '49020'
'51071' = '13980'
'51073' = '47260'
'51075' = '40060'
'51079' = '16820'
'51085' = '40060'
'51087' = '40060'
'51089' = '32300'
'51093' = '47260'
'51095' = '47260'
'51097' = '40060'
'51101' = '40060'
'51107' = '47900'
'51109' = '40060'
'51115' = '47260'
'51121' = '13980'
'51125' = '16820'
'51127' = '40060'
'51143' = '19260'
'51145' = '40060'
'51149' = '40060'
'51153' = '47900'
'51155' = '13980'
'51161' = '40220'
'51165' = '25500'
'51169' = '28700'
'51177' = '47900'
'51179' = '47900'
'51181' = '47260'
'51183' = '40060'
'51185' = '14140'
'51187' = '47900'
'51191' = '28700'
'51199' = '47260'
'51510' = '47900'
'51515' = '31340'
'51520' = '28700'
'51540' = '16820'
'51550' = '47260'
'51570' = '40060'
'51590' = '19260'
'51600' = '47900'
'51610' = '47900'
'51630' = '47900'
'51650' = '47260'
'51660' = '25500'
'51670' = '40060'
'51680' = '31340'
'51683' = '47900'
'51685' = '47900'
'51690' = '32300'
'51700' = '47260'
'51710' = '47260'
'51730' = '40060'
'51735' = '47260'
'51740' = '47260'
'51750' = '13980'
'51760' = '40060'
'51770' = '40220'
'51775' = '40220'
'51790' = '44420'
'51800' = '47260'
'51810' = '47260'
'51820' = '44420'
'51830' = '47260'
'51840' = '49020'
'53003' = '30300'
'53005' = '28420'
'53007' = '48300'
'53009' = '38820'
'53011' = '38900'
'53015' = '31020'
'53017' = '48300'
'53021' = '28420'
'53025' = '34180'
'53027' = '10140'
'53029' = '36020'
'53033' = '42660'
'53035' = '14740'
'53037' = '21260'
'53041' = '16500'
'53045' = '43220'
'53053' = '42660'
'53057' = '34580'
'53059' = '38900'
'53061' = '42660'
'53063' = '44060'
'53067' = '36500'
'53071' = '47460'
'53073' = '13380'
'53075' = '39420'
'53077' = '49420'
'54003' = '25180'
'54005' = '16620'
'54009' = '48260'
'54011' = '26580'
'54015' = '16620'
'54017' = '17220'
'54019' = '36060'
'54027' = '49020'
'54029' = '48260'
'54033' = '17220'
'54037' = '47900'
'54039' = '16620'
'54043' = '16620'
'54049' = '21900'
'54051' = '48540'
'54053' = '38580'
'54055' = '14140'
'54057' = '19060'
'54061' = '34060'
'54065' = '25180'
'54069' = '48540'
'54073' = '37620'
'54077' = '34060'
'54079' = '16620'
'54081' = '13220'
'54091' = '17220'
'54099' = '26580'
'54105' = '37620'
'54107' = '37620'
'55009' = '24580'
'55015' = '11540'
'55017' = '20740'
'55021' = '31540'
'55025' = '31540'
'55027' = '13180'
'55031' = '20260'
'55033' = '32860'
'55035' = '20740'
'55037' = '27020'
'55039' = '22540'
'55043' = '38420'
'55045' = '33820'
'55049' = '31540'
'55055' = '48020'
'55059' = '16980'
'55061' = '24580'
'55063' = '29100'
'55069' = '32980'
'55071' = '31820'
'55073' = '48140'
'55075' = '31940'
'55079' = '33340'
'55083' = '24580'
'55087' = '11540'
'55089' = '33340'
'55093' = '33460'
'55097' = '44620'
'55101' = '39540'
'55105' = '27500'
'55109' = '33460'
'55111' = '12660'
'55117' = '43100'
'55127' = '48580'
'55131' = '33340'
'55133' = '33340'
'55139' = '36780'
'55141' = '49220'
'56001' = '29660'
'56005' = '23940'
'56013' = '40180'
'56021' = '16940'
'56025' = '16220'
'56033' = '43260'
'56037' = '40540'
'56039' = '27220'
'56041' = '21740'
;

 /* --------------------------------------------------------------- */
 /* ---   OPERATING ROOM PROCEDURE CODES                           - */
 /* --------------------------------------------------------------- */

VALUE $ORPROC
"0016071"  =  "1"  /* Bypass Cereb Vent to Mastoid Sinus w Autol Sub, Open */
"0016072"  =  "1"  /* Bypass Cereb Vent to Atrium with Autol Sub, Open Approach */
"0016073"  =  "1"  /* Bypass Cereb Vent to Blood Vess w Autol Sub, Open */
"0016074"  =  "1"  /* Bypass Cereb Vent to Pleural Cav w Autol Sub, Open */
"0016075"  =  "1"  /* Bypass Cereb Vent to Intestine with Autol Sub, Open Approach */
"0016076"  =  "1"  /* Bypass Cereb Vent to Periton Cav w Autol Sub, Open */
"0016077"  =  "1"  /* Bypass Cereb Vent to Urinary Tract w Autol Sub, Open */
"0016078"  =  "1"  /* Bypass Cereb Vent to Bone Mar with Autol Sub, Open Approach */
"001607B"  =  "1"  /* Bypass Cereb Vent to Cereb Cistern w Autol Sub, Open */
"00160J0"  =  "1"  /* Bypass Cereb Vent to Nasophar with Synth Sub, Open Approach */
"00160J1"  =  "1"  /* Bypass Cereb Vent to Mastoid Sinus w Synth Sub, Open */
"00160J2"  =  "1"  /* Bypass Cereb Vent to Atrium with Synth Sub, Open Approach */
"00160J3"  =  "1"  /* Bypass Cereb Vent to Blood Vess w Synth Sub, Open */
"00160J4"  =  "1"  /* Bypass Cereb Vent to Pleural Cav w Synth Sub, Open */
"00160J5"  =  "1"  /* Bypass Cereb Vent to Intestine with Synth Sub, Open Approach */
"00160J6"  =  "1"  /* Bypass Cereb Vent to Periton Cav w Synth Sub, Open */
"00160J7"  =  "1"  /* Bypass Cereb Vent to Urinary Tract w Synth Sub, Open */
"00160J8"  =  "1"  /* Bypass Cereb Vent to Bone Mar with Synth Sub, Open Approach */
"00160JB"  =  "1"  /* Bypass Cereb Vent to Cereb Cistern w Synth Sub, Open */
"00160K0"  =  "1"  /* Bypass Cereb Vent to Nasophar with Nonaut Sub, Open Approach */
"00160K1"  =  "1"  /* Bypass Cereb Vent to Mastoid Sinus w Nonaut Sub, Open */
"00160K2"  =  "1"  /* Bypass Cereb Vent to Atrium with Nonaut Sub, Open Approach */
"00160K3"  =  "1"  /* Bypass Cereb Vent to Blood Vess w Nonaut Sub, Open */
"00160K4"  =  "1"  /* Bypass Cereb Vent to Pleural Cav w Nonaut Sub, Open */
"00160K5"  =  "1"  /* Bypass Cereb Vent to Intestine w Nonaut Sub, Open */
"00160K6"  =  "1"  /* Bypass Cereb Vent to Periton Cav w Nonaut Sub, Open */
"00160K7"  =  "1"  /* Bypass Cereb Vent to Urinary Tract w Nonaut Sub, Open */
"00160K8"  =  "1"  /* Bypass Cereb Vent to Bone Mar with Nonaut Sub, Open Approach */
"00160KB"  =  "1"  /* Bypass Cereb Vent to Cereb Cistern w Nonaut Sub, Open */
"0016370"  =  "1"  /* Bypass Cereb Vent to Nasophar with Autol Sub, Perc Approach */
"0016371"  =  "1"  /* Bypass Cereb Vent to Mastoid Sinus w Autol Sub, Perc */
"0016372"  =  "1"  /* Bypass Cereb Vent to Atrium with Autol Sub, Perc Approach */
"0016373"  =  "1"  /* Bypass Cereb Vent to Blood Vess w Autol Sub, Perc */
"0016374"  =  "1"  /* Bypass Cereb Vent to Pleural Cav w Autol Sub, Perc */
"0016375"  =  "1"  /* Bypass Cereb Vent to Intestine with Autol Sub, Perc Approach */
"0016376"  =  "1"  /* Bypass Cereb Vent to Periton Cav w Autol Sub, Perc */
"0016377"  =  "1"  /* Bypass Cereb Vent to Urinary Tract w Autol Sub, Perc */
"0016378"  =  "1"  /* Bypass Cereb Vent to Bone Mar with Autol Sub, Perc Approach */
"001637B"  =  "1"  /* Bypass Cereb Vent to Cereb Cistern w Autol Sub, Perc */
"00163J0"  =  "1"  /* Bypass Cereb Vent to Nasophar with Synth Sub, Perc Approach */
"00163J1"  =  "1"  /* Bypass Cereb Vent to Mastoid Sinus w Synth Sub, Perc */
"00163J2"  =  "1"  /* Bypass Cereb Vent to Atrium with Synth Sub, Perc Approach */
"00163J3"  =  "1"  /* Bypass Cereb Vent to Blood Vess w Synth Sub, Perc */
"00163J4"  =  "1"  /* Bypass Cereb Vent to Pleural Cav w Synth Sub, Perc */
"00163J5"  =  "1"  /* Bypass Cereb Vent to Intestine with Synth Sub, Perc Approach */
"00163J6"  =  "1"  /* Bypass Cereb Vent to Periton Cav w Synth Sub, Perc */
"00163J7"  =  "1"  /* Bypass Cereb Vent to Urinary Tract w Synth Sub, Perc */
"00163J8"  =  "1"  /* Bypass Cereb Vent to Bone Mar with Synth Sub, Perc Approach */
"00163JB"  =  "1"  /* Bypass Cereb Vent to Cereb Cistern w Synth Sub, Perc */
"00163K0"  =  "1"  /* Bypass Cereb Vent to Nasophar with Nonaut Sub, Perc Approach */
"00163K1"  =  "1"  /* Bypass Cereb Vent to Mastoid Sinus w Nonaut Sub, Perc */
"00163K2"  =  "1"  /* Bypass Cereb Vent to Atrium with Nonaut Sub, Perc Approach */
"00163K3"  =  "1"  /* Bypass Cereb Vent to Blood Vess w Nonaut Sub, Perc */
"00163K4"  =  "1"  /* Bypass Cereb Vent to Pleural Cav w Nonaut Sub, Perc */
"00163K5"  =  "1"  /* Bypass Cereb Vent to Intestine w Nonaut Sub, Perc */
"00163K6"  =  "1"  /* Bypass Cereb Vent to Periton Cav w Nonaut Sub, Perc */
"00163K7"  =  "1"  /* Bypass Cereb Vent to Urinary Tract w Nonaut Sub, Perc */
"00163K8"  =  "1"  /* Bypass Cereb Vent to Bone Mar with Nonaut Sub, Perc Approach */
"00163KB"  =  "1"  /* Bypass Cereb Vent to Cereb Cistern w Nonaut Sub, Perc */
"001U074"  =  "1"  /* Bypass Spinal Canal to Pleural Cav w Autol Sub, Open */
"001U076"  =  "1"  /* Bypass Spinal Canal to Periton Cav w Autol Sub, Open */
"001U077"  =  "1"  /* Bypass Spinal Canal to Urinary Tract w Autol Sub, Open */
"001U079"  =  "1"  /* Bypass Spinal Canal to Fallopian w Autol Sub, Open */
"001U0J4"  =  "1"  /* Bypass Spinal Canal to Pleural Cav w Synth Sub, Open */
"001U0J6"  =  "1"  /* Bypass Spinal Canal to Periton Cav w Synth Sub, Open */
"001U0J7"  =  "1"  /* Bypass Spinal Canal to Urinary Tract w Synth Sub, Open */
"001U0J9"  =  "1"  /* Bypass Spinal Canal to Fallopian w Synth Sub, Open */
"001U0K4"  =  "1"  /* Bypass Spinal Canal to Pleural Cav w Nonaut Sub, Open */
"001U0K6"  =  "1"  /* Bypass Spinal Canal to Periton Cav w Nonaut Sub, Open */
"001U0K7"  =  "1"  /* Bypass Spinal Canal to Urinary Tract w Nonaut Sub, Open */
"001U0K9"  =  "1"  /* Bypass Spinal Canal to Fallopian w Nonaut Sub, Open */
"001U374"  =  "1"  /* Bypass Spinal Canal to Pleural Cav w Autol Sub, Perc */
"001U376"  =  "1"  /* Bypass Spinal Canal to Periton Cav w Autol Sub, Perc */
"001U377"  =  "1"  /* Bypass Spinal Canal to Urinary Tract w Autol Sub, Perc */
"001U379"  =  "1"  /* Bypass Spinal Canal to Fallopian w Autol Sub, Perc */
"001U3J4"  =  "1"  /* Bypass Spinal Canal to Pleural Cav w Synth Sub, Perc */
"001U3J6"  =  "1"  /* Bypass Spinal Canal to Periton Cav w Synth Sub, Perc */
"001U3J7"  =  "1"  /* Bypass Spinal Canal to Urinary Tract w Synth Sub, Perc */
"001U3J9"  =  "1"  /* Bypass Spinal Canal to Fallopian w Synth Sub, Perc */
"001U3K4"  =  "1"  /* Bypass Spinal Canal to Pleural Cav w Nonaut Sub, Perc */
"001U3K6"  =  "1"  /* Bypass Spinal Canal to Periton Cav w Nonaut Sub, Perc */
"001U3K7"  =  "1"  /* Bypass Spinal Canal to Urinary Tract w Nonaut Sub, Perc */
"001U3K9"  =  "1"  /* Bypass Spinal Canal to Fallopian w Nonaut Sub, Perc */
"00500ZZ"  =  "1"  /* Destruction of Brain, Open Approach */
"00503ZZ"  =  "1"  /* Destruction of Brain, Percutaneous Approach */
"00504ZZ"  =  "1"  /* Destruction of Brain, Percutaneous Endoscopic Approach */
"00510ZZ"  =  "1"  /* Destruction of Cerebral Meninges, Open Approach */
"00513ZZ"  =  "1"  /* Destruction of Cerebral Meninges, Percutaneous Approach */
"00514ZZ"  =  "1"  /* Destruction of Cerebral Meninges, Perc Endo Approach */
"00520ZZ"  =  "1"  /* Destruction of Dura Mater, Open Approach */
"00523ZZ"  =  "1"  /* Destruction of Dura Mater, Percutaneous Approach */
"00524ZZ"  =  "1"  /* Destruction of Dura Mater, Percutaneous Endoscopic Approach */
"00560ZZ"  =  "1"  /* Destruction of Cerebral Ventricle, Open Approach */
"00563ZZ"  =  "1"  /* Destruction of Cerebral Ventricle, Percutaneous Approach */
"00564ZZ"  =  "1"  /* Destruction of Cerebral Ventricle, Perc Endo Approach */
"00570ZZ"  =  "1"  /* Destruction of Cerebral Hemisphere, Open Approach */
"00573ZZ"  =  "1"  /* Destruction of Cerebral Hemisphere, Percutaneous Approach */
"00574ZZ"  =  "1"  /* Destruction of Cerebral Hemisphere, Perc Endo Approach */
"00580ZZ"  =  "1"  /* Destruction of Basal Ganglia, Open Approach */
"00583ZZ"  =  "1"  /* Destruction of Basal Ganglia, Percutaneous Approach */
"00584ZZ"  =  "1"  /* Destruction of Basal Ganglia, Perc Endo Approach */
"00590ZZ"  =  "1"  /* Destruction of Thalamus, Open Approach */
"00593ZZ"  =  "1"  /* Destruction of Thalamus, Percutaneous Approach */
"00594ZZ"  =  "1"  /* Destruction of Thalamus, Percutaneous Endoscopic Approach */
"005A0ZZ"  =  "1"  /* Destruction of Hypothalamus, Open Approach */
"005A3ZZ"  =  "1"  /* Destruction of Hypothalamus, Percutaneous Approach */
"005A4ZZ"  =  "1"  /* Destruction of Hypothalamus, Perc Endo Approach */
"005B0ZZ"  =  "1"  /* Destruction of Pons, Open Approach */
"005B3ZZ"  =  "1"  /* Destruction of Pons, Percutaneous Approach */
"005B4ZZ"  =  "1"  /* Destruction of Pons, Percutaneous Endoscopic Approach */
"005C0ZZ"  =  "1"  /* Destruction of Cerebellum, Open Approach */
"005C3ZZ"  =  "1"  /* Destruction of Cerebellum, Percutaneous Approach */
"005C4ZZ"  =  "1"  /* Destruction of Cerebellum, Percutaneous Endoscopic Approach */
"005D0ZZ"  =  "1"  /* Destruction of Medulla Oblongata, Open Approach */
"005D3ZZ"  =  "1"  /* Destruction of Medulla Oblongata, Percutaneous Approach */
"005D4ZZ"  =  "1"  /* Destruction of Medulla Oblongata, Perc Endo Approach */
"005T0ZZ"  =  "1"  /* Destruction of Spinal Meninges, Open Approach */
"005T3ZZ"  =  "1"  /* Destruction of Spinal Meninges, Percutaneous Approach */
"005T4ZZ"  =  "1"  /* Destruction of Spinal Meninges, Perc Endo Approach */
"005W0ZZ"  =  "1"  /* Destruction of Cervical Spinal Cord, Open Approach */
"005W3ZZ"  =  "1"  /* Destruction of Cervical Spinal Cord, Percutaneous Approach */
"005W4ZZ"  =  "1"  /* Destruction of Cervical Spinal Cord, Perc Endo Approach */
"005X0ZZ"  =  "1"  /* Destruction of Thoracic Spinal Cord, Open Approach */
"005X3ZZ"  =  "1"  /* Destruction of Thoracic Spinal Cord, Percutaneous Approach */
"005X4ZZ"  =  "1"  /* Destruction of Thoracic Spinal Cord, Perc Endo Approach */
"005Y0ZZ"  =  "1"  /* Destruction of Lumbar Spinal Cord, Open Approach */
"005Y3ZZ"  =  "1"  /* Destruction of Lumbar Spinal Cord, Percutaneous Approach */
"005Y4ZZ"  =  "1"  /* Destruction of Lumbar Spinal Cord, Perc Endo Approach */
"00800ZZ"  =  "1"  /* Division of Brain, Open Approach */
"00803ZZ"  =  "1"  /* Division of Brain, Percutaneous Approach */
"00804ZZ"  =  "1"  /* Division of Brain, Percutaneous Endoscopic Approach */
"00870ZZ"  =  "1"  /* Division of Cerebral Hemisphere, Open Approach */
"00873ZZ"  =  "1"  /* Division of Cerebral Hemisphere, Percutaneous Approach */
"00874ZZ"  =  "1"  /* Division of Cerebral Hemisphere, Perc Endo Approach */
"00880ZZ"  =  "1"  /* Division of Basal Ganglia, Open Approach */
"00883ZZ"  =  "1"  /* Division of Basal Ganglia, Percutaneous Approach */
"00884ZZ"  =  "1"  /* Division of Basal Ganglia, Percutaneous Endoscopic Approach */
"008F0ZZ"  =  "1"  /* Division of Olfactory Nerve, Open Approach */
"008F3ZZ"  =  "1"  /* Division of Olfactory Nerve, Percutaneous Approach */
"008F4ZZ"  =  "1"  /* Division of Olfactory Nerve, Perc Endo Approach */
"008G0ZZ"  =  "1"  /* Division of Optic Nerve, Open Approach */
"008G3ZZ"  =  "1"  /* Division of Optic Nerve, Percutaneous Approach */
"008G4ZZ"  =  "1"  /* Division of Optic Nerve, Percutaneous Endoscopic Approach */
"008H0ZZ"  =  "1"  /* Division of Oculomotor Nerve, Open Approach */
"008H3ZZ"  =  "1"  /* Division of Oculomotor Nerve, Percutaneous Approach */
"008H4ZZ"  =  "1"  /* Division of Oculomotor Nerve, Perc Endo Approach */
"008J0ZZ"  =  "1"  /* Division of Trochlear Nerve, Open Approach */
"008J3ZZ"  =  "1"  /* Division of Trochlear Nerve, Percutaneous Approach */
"008J4ZZ"  =  "1"  /* Division of Trochlear Nerve, Perc Endo Approach */
"008K0ZZ"  =  "1"  /* Division of Trigeminal Nerve, Open Approach */
"008K3ZZ"  =  "1"  /* Division of Trigeminal Nerve, Percutaneous Approach */
"008K4ZZ"  =  "1"  /* Division of Trigeminal Nerve, Perc Endo Approach */
"008L0ZZ"  =  "1"  /* Division of Abducens Nerve, Open Approach */
"008L3ZZ"  =  "1"  /* Division of Abducens Nerve, Percutaneous Approach */
"008L4ZZ"  =  "1"  /* Division of Abducens Nerve, Percutaneous Endoscopic Approach */
"008M0ZZ"  =  "1"  /* Division of Facial Nerve, Open Approach */
"008M3ZZ"  =  "1"  /* Division of Facial Nerve, Percutaneous Approach */
"008M4ZZ"  =  "1"  /* Division of Facial Nerve, Percutaneous Endoscopic Approach */
"008N0ZZ"  =  "1"  /* Division of Acoustic Nerve, Open Approach */
"008N3ZZ"  =  "1"  /* Division of Acoustic Nerve, Percutaneous Approach */
"008N4ZZ"  =  "1"  /* Division of Acoustic Nerve, Percutaneous Endoscopic Approach */
"008P0ZZ"  =  "1"  /* Division of Glossopharyngeal Nerve, Open Approach */
"008P3ZZ"  =  "1"  /* Division of Glossopharyngeal Nerve, Percutaneous Approach */
"008P4ZZ"  =  "1"  /* Division of Glossopharyngeal Nerve, Perc Endo Approach */
"008Q0ZZ"  =  "1"  /* Division of Vagus Nerve, Open Approach */
"008Q3ZZ"  =  "1"  /* Division of Vagus Nerve, Percutaneous Approach */
"008Q4ZZ"  =  "1"  /* Division of Vagus Nerve, Percutaneous Endoscopic Approach */
"008R0ZZ"  =  "1"  /* Division of Accessory Nerve, Open Approach */
"008R3ZZ"  =  "1"  /* Division of Accessory Nerve, Percutaneous Approach */
"008R4ZZ"  =  "1"  /* Division of Accessory Nerve, Perc Endo Approach */
"008S0ZZ"  =  "1"  /* Division of Hypoglossal Nerve, Open Approach */
"008S3ZZ"  =  "1"  /* Division of Hypoglossal Nerve, Percutaneous Approach */
"008S4ZZ"  =  "1"  /* Division of Hypoglossal Nerve, Perc Endo Approach */
"008W0ZZ"  =  "1"  /* Division of Cervical Spinal Cord, Open Approach */
"008W3ZZ"  =  "1"  /* Division of Cervical Spinal Cord, Percutaneous Approach */
"008W4ZZ"  =  "1"  /* Division of Cervical Spinal Cord, Perc Endo Approach */
"008X0ZZ"  =  "1"  /* Division of Thoracic Spinal Cord, Open Approach */
"008X3ZZ"  =  "1"  /* Division of Thoracic Spinal Cord, Percutaneous Approach */
"008X4ZZ"  =  "1"  /* Division of Thoracic Spinal Cord, Perc Endo Approach */
"008Y0ZZ"  =  "1"  /* Division of Lumbar Spinal Cord, Open Approach */
"008Y3ZZ"  =  "1"  /* Division of Lumbar Spinal Cord, Percutaneous Approach */
"008Y4ZZ"  =  "1"  /* Division of Lumbar Spinal Cord, Perc Endo Approach */
"009000Z"  =  "1"  /* Drainage of Brain with Drainage Device, Open Approach */
"00900ZX"  =  "1"  /* Drainage of Brain, Open Approach, Diagnostic */
"00900ZZ"  =  "1"  /* Drainage of Brain, Open Approach */
"009030Z"  =  "1"  /* Drainage of Brain with Drainage Device, Perc Approach */
"00903ZZ"  =  "1"  /* Drainage of Brain, Percutaneous Approach */
"009040Z"  =  "1"  /* Drainage of Brain with Drainage Device, Perc Endo Approach */
"00904ZZ"  =  "1"  /* Drainage of Brain, Percutaneous Endoscopic Approach */
"009100Z"  =  "1"  /* Drainage of Cerebral Meninges with Drain Dev, Open Approach */
"00910ZX"  =  "1"  /* Drainage of Cerebral Meninges, Open Approach, Diagnostic */
"00910ZZ"  =  "1"  /* Drainage of Cerebral Meninges, Open Approach */
"009200Z"  =  "1"  /* Drainage of Dura Mater with Drainage Device, Open Approach */
"00920ZX"  =  "1"  /* Drainage of Dura Mater, Open Approach, Diagnostic */
"00920ZZ"  =  "1"  /* Drainage of Dura Mater, Open Approach */
"009300Z"  =  "1"  /* Drainage of Epidural Space with Drain Dev, Open Approach */
"00930ZX"  =  "1"  /* Drainage of Epidural Space, Open Approach, Diagnostic */
"00930ZZ"  =  "1"  /* Drainage of Epidural Space, Open Approach */
"009330Z"  =  "1"  /* Drainage of Epidural Space with Drain Dev, Perc Approach */
"00933ZZ"  =  "1"  /* Drainage of Epidural Space, Percutaneous Approach */
"009340Z"  =  "1"  /* Drain of Epidural Space with Drain Dev, Perc Endo Approach */
"00934ZZ"  =  "1"  /* Drainage of Epidural Space, Percutaneous Endoscopic Approach */
"009400Z"  =  "1"  /* Drainage of Subdural Space with Drain Dev, Open Approach */
"00940ZX"  =  "1"  /* Drainage of Subdural Space, Open Approach, Diagnostic */
"00940ZZ"  =  "1"  /* Drainage of Subdural Space, Open Approach */
"009500Z"  =  "1"  /* Drainage of Subarachnoid Space with Drain Dev, Open Approach */
"00950ZX"  =  "1"  /* Drainage of Subarachnoid Space, Open Approach, Diagnostic */
"00950ZZ"  =  "1"  /* Drainage of Subarachnoid Space, Open Approach */
"009600Z"  =  "1"  /* Drainage of Cerebral Ventricle with Drain Dev, Open Approach */
"00960ZX"  =  "1"  /* Drainage of Cerebral Ventricle, Open Approach, Diagnostic */
"00960ZZ"  =  "1"  /* Drainage of Cerebral Ventricle, Open Approach */
"009630Z"  =  "1"  /* Drainage of Cerebral Ventricle with Drain Dev, Perc Approach */
"009640Z"  =  "1"  /* Drainage of Cereb Vent with Drain Dev, Perc Endo Approach */
"009700Z"  =  "1"  /* Drainage of Cereb Hemis with Drain Dev, Open Approach */
"00970ZX"  =  "1"  /* Drainage of Cerebral Hemisphere, Open Approach, Diagnostic */
"00970ZZ"  =  "1"  /* Drainage of Cerebral Hemisphere, Open Approach */
"009730Z"  =  "1"  /* Drainage of Cereb Hemis with Drain Dev, Perc Approach */
"00973ZZ"  =  "1"  /* Drainage of Cerebral Hemisphere, Percutaneous Approach */
"009740Z"  =  "1"  /* Drainage of Cereb Hemis with Drain Dev, Perc Endo Approach */
"00974ZZ"  =  "1"  /* Drainage of Cerebral Hemisphere, Perc Endo Approach */
"009800Z"  =  "1"  /* Drainage of Basal Ganglia with Drain Dev, Open Approach */
"00980ZX"  =  "1"  /* Drainage of Basal Ganglia, Open Approach, Diagnostic */
"00980ZZ"  =  "1"  /* Drainage of Basal Ganglia, Open Approach */
"009830Z"  =  "1"  /* Drainage of Basal Ganglia with Drain Dev, Perc Approach */
"00983ZZ"  =  "1"  /* Drainage of Basal Ganglia, Percutaneous Approach */
"009840Z"  =  "1"  /* Drainage of Basal Ganglia with Drain Dev, Perc Endo Approach */
"00984ZZ"  =  "1"  /* Drainage of Basal Ganglia, Percutaneous Endoscopic Approach */
"009900Z"  =  "1"  /* Drainage of Thalamus with Drainage Device, Open Approach */
"00990ZX"  =  "1"  /* Drainage of Thalamus, Open Approach, Diagnostic */
"00990ZZ"  =  "1"  /* Drainage of Thalamus, Open Approach */
"009930Z"  =  "1"  /* Drainage of Thalamus with Drainage Device, Perc Approach */
"00993ZZ"  =  "1"  /* Drainage of Thalamus, Percutaneous Approach */
"009940Z"  =  "1"  /* Drainage of Thalamus with Drain Dev, Perc Endo Approach */
"00994ZZ"  =  "1"  /* Drainage of Thalamus, Percutaneous Endoscopic Approach */
"009A00Z"  =  "1"  /* Drainage of Hypothalamus with Drainage Device, Open Approach */
"009A0ZX"  =  "1"  /* Drainage of Hypothalamus, Open Approach, Diagnostic */
"009A0ZZ"  =  "1"  /* Drainage of Hypothalamus, Open Approach */
"009A30Z"  =  "1"  /* Drainage of Hypothalamus with Drainage Device, Perc Approach */
"009A3ZZ"  =  "1"  /* Drainage of Hypothalamus, Percutaneous Approach */
"009A40Z"  =  "1"  /* Drainage of Hypothalamus with Drain Dev, Perc Endo Approach */
"009A4ZZ"  =  "1"  /* Drainage of Hypothalamus, Percutaneous Endoscopic Approach */
"009B00Z"  =  "1"  /* Drainage of Pons with Drainage Device, Open Approach */
"009B0ZX"  =  "1"  /* Drainage of Pons, Open Approach, Diagnostic */
"009B0ZZ"  =  "1"  /* Drainage of Pons, Open Approach */
"009B30Z"  =  "1"  /* Drainage of Pons with Drainage Device, Percutaneous Approach */
"009B3ZZ"  =  "1"  /* Drainage of Pons, Percutaneous Approach */
"009B40Z"  =  "1"  /* Drainage of Pons with Drainage Device, Perc Endo Approach */
"009B4ZZ"  =  "1"  /* Drainage of Pons, Percutaneous Endoscopic Approach */
"009C00Z"  =  "1"  /* Drainage of Cerebellum with Drainage Device, Open Approach */
"009C0ZX"  =  "1"  /* Drainage of Cerebellum, Open Approach, Diagnostic */
"009C0ZZ"  =  "1"  /* Drainage of Cerebellum, Open Approach */
"009C30Z"  =  "1"  /* Drainage of Cerebellum with Drainage Device, Perc Approach */
"009C3ZZ"  =  "1"  /* Drainage of Cerebellum, Percutaneous Approach */
"009C40Z"  =  "1"  /* Drainage of Cerebellum with Drain Dev, Perc Endo Approach */
"009C4ZZ"  =  "1"  /* Drainage of Cerebellum, Percutaneous Endoscopic Approach */
"009D00Z"  =  "1"  /* Drainage of Medulla Oblongata with Drain Dev, Open Approach */
"009D0ZX"  =  "1"  /* Drainage of Medulla Oblongata, Open Approach, Diagnostic */
"009D0ZZ"  =  "1"  /* Drainage of Medulla Oblongata, Open Approach */
"009D30Z"  =  "1"  /* Drainage of Medulla Oblongata with Drain Dev, Perc Approach */
"009D3ZZ"  =  "1"  /* Drainage of Medulla Oblongata, Percutaneous Approach */
"009D40Z"  =  "1"  /* Drain of Medulla Oblong with Drain Dev, Perc Endo Approach */
"009D4ZZ"  =  "1"  /* Drainage of Medulla Oblongata, Perc Endo Approach */
"009F00Z"  =  "1"  /* Drainage of Olfactory Nerve with Drain Dev, Open Approach */
"009F0ZX"  =  "1"  /* Drainage of Olfactory Nerve, Open Approach, Diagnostic */
"009F0ZZ"  =  "1"  /* Drainage of Olfactory Nerve, Open Approach */
"009F30Z"  =  "1"  /* Drainage of Olfactory Nerve with Drain Dev, Perc Approach */
"009F3ZZ"  =  "1"  /* Drainage of Olfactory Nerve, Percutaneous Approach */
"009F40Z"  =  "1"  /* Drainage of Olfact Nrv with Drain Dev, Perc Endo Approach */
"009F4ZZ"  =  "1"  /* Drainage of Olfactory Nerve, Perc Endo Approach */
"009G00Z"  =  "1"  /* Drainage of Optic Nerve with Drainage Device, Open Approach */
"009G0ZX"  =  "1"  /* Drainage of Optic Nerve, Open Approach, Diagnostic */
"009G0ZZ"  =  "1"  /* Drainage of Optic Nerve, Open Approach */
"009G30Z"  =  "1"  /* Drainage of Optic Nerve with Drainage Device, Perc Approach */
"009G3ZZ"  =  "1"  /* Drainage of Optic Nerve, Percutaneous Approach */
"009G40Z"  =  "1"  /* Drainage of Optic Nerve with Drain Dev, Perc Endo Approach */
"009G4ZZ"  =  "1"  /* Drainage of Optic Nerve, Percutaneous Endoscopic Approach */
"009H00Z"  =  "1"  /* Drainage of Oculomotor Nerve with Drain Dev, Open Approach */
"009H0ZX"  =  "1"  /* Drainage of Oculomotor Nerve, Open Approach, Diagnostic */
"009H0ZZ"  =  "1"  /* Drainage of Oculomotor Nerve, Open Approach */
"009H30Z"  =  "1"  /* Drainage of Oculomotor Nerve with Drain Dev, Perc Approach */
"009H3ZZ"  =  "1"  /* Drainage of Oculomotor Nerve, Percutaneous Approach */
"009H40Z"  =  "1"  /* Drain of Oculomotor Nrv with Drain Dev, Perc Endo Approach */
"009H4ZZ"  =  "1"  /* Drainage of Oculomotor Nerve, Perc Endo Approach */
"009J00Z"  =  "1"  /* Drainage of Trochlear Nerve with Drain Dev, Open Approach */
"009J0ZX"  =  "1"  /* Drainage of Trochlear Nerve, Open Approach, Diagnostic */
"009J0ZZ"  =  "1"  /* Drainage of Trochlear Nerve, Open Approach */
"009J30Z"  =  "1"  /* Drainage of Trochlear Nerve with Drain Dev, Perc Approach */
"009J3ZZ"  =  "1"  /* Drainage of Trochlear Nerve, Percutaneous Approach */
"009J40Z"  =  "1"  /* Drainage of Trochlear Nrv with Drain Dev, Perc Endo Approach */
"009J4ZZ"  =  "1"  /* Drainage of Trochlear Nerve, Perc Endo Approach */
"009K00Z"  =  "1"  /* Drainage of Trigeminal Nerve with Drain Dev, Open Approach */
"009K0ZX"  =  "1"  /* Drainage of Trigeminal Nerve, Open Approach, Diagnostic */
"009K0ZZ"  =  "1"  /* Drainage of Trigeminal Nerve, Open Approach */
"009K30Z"  =  "1"  /* Drainage of Trigeminal Nerve with Drain Dev, Perc Approach */
"009K3ZZ"  =  "1"  /* Drainage of Trigeminal Nerve, Percutaneous Approach */
"009K40Z"  =  "1"  /* Drainage of Trigem Nrv with Drain Dev, Perc Endo Approach */
"009K4ZZ"  =  "1"  /* Drainage of Trigeminal Nerve, Perc Endo Approach */
"009L00Z"  =  "1"  /* Drainage of Abducens Nerve with Drain Dev, Open Approach */
"009L0ZX"  =  "1"  /* Drainage of Abducens Nerve, Open Approach, Diagnostic */
"009L0ZZ"  =  "1"  /* Drainage of Abducens Nerve, Open Approach */
"009L30Z"  =  "1"  /* Drainage of Abducens Nerve with Drain Dev, Perc Approach */
"009L3ZZ"  =  "1"  /* Drainage of Abducens Nerve, Percutaneous Approach */
"009L40Z"  =  "1"  /* Drainage of Abducen Nrv with Drain Dev, Perc Endo Approach */
"009L4ZZ"  =  "1"  /* Drainage of Abducens Nerve, Percutaneous Endoscopic Approach */
"009M00Z"  =  "1"  /* Drainage of Facial Nerve with Drainage Device, Open Approach */
"009M0ZX"  =  "1"  /* Drainage of Facial Nerve, Open Approach, Diagnostic */
"009M0ZZ"  =  "1"  /* Drainage of Facial Nerve, Open Approach */
"009M30Z"  =  "1"  /* Drainage of Facial Nerve with Drainage Device, Perc Approach */
"009M3ZZ"  =  "1"  /* Drainage of Facial Nerve, Percutaneous Approach */
"009M40Z"  =  "1"  /* Drainage of Facial Nerve with Drain Dev, Perc Endo Approach */
"009M4ZZ"  =  "1"  /* Drainage of Facial Nerve, Percutaneous Endoscopic Approach */
"009N00Z"  =  "1"  /* Drainage of Acoustic Nerve with Drain Dev, Open Approach */
"009N0ZX"  =  "1"  /* Drainage of Acoustic Nerve, Open Approach, Diagnostic */
"009N0ZZ"  =  "1"  /* Drainage of Acoustic Nerve, Open Approach */
"009N30Z"  =  "1"  /* Drainage of Acoustic Nerve with Drain Dev, Perc Approach */
"009N3ZZ"  =  "1"  /* Drainage of Acoustic Nerve, Percutaneous Approach */
"009N40Z"  =  "1"  /* Drainage of Acoust Nrv with Drain Dev, Perc Endo Approach */
"009N4ZZ"  =  "1"  /* Drainage of Acoustic Nerve, Percutaneous Endoscopic Approach */
"009P00Z"  =  "1"  /* Drainage of Glossophar Nrv with Drain Dev, Open Approach */
"009P0ZX"  =  "1"  /* Drainage of Glossopharyngeal Nerve, Open Approach, Diagn */
"009P0ZZ"  =  "1"  /* Drainage of Glossopharyngeal Nerve, Open Approach */
"009P30Z"  =  "1"  /* Drainage of Glossophar Nrv with Drain Dev, Perc Approach */
"009P3ZZ"  =  "1"  /* Drainage of Glossopharyngeal Nerve, Percutaneous Approach */
"009P40Z"  =  "1"  /* Drain of Glossophar Nrv with Drain Dev, Perc Endo Approach */
"009P4ZZ"  =  "1"  /* Drainage of Glossopharyngeal Nerve, Perc Endo Approach */
"009Q00Z"  =  "1"  /* Drainage of Vagus Nerve with Drainage Device, Open Approach */
"009Q0ZX"  =  "1"  /* Drainage of Vagus Nerve, Open Approach, Diagnostic */
"009Q0ZZ"  =  "1"  /* Drainage of Vagus Nerve, Open Approach */
"009Q30Z"  =  "1"  /* Drainage of Vagus Nerve with Drainage Device, Perc Approach */
"009Q3ZZ"  =  "1"  /* Drainage of Vagus Nerve, Percutaneous Approach */
"009Q40Z"  =  "1"  /* Drainage of Vagus Nerve with Drain Dev, Perc Endo Approach */
"009Q4ZZ"  =  "1"  /* Drainage of Vagus Nerve, Percutaneous Endoscopic Approach */
"009R00Z"  =  "1"  /* Drainage of Accessory Nerve with Drain Dev, Open Approach */
"009R0ZX"  =  "1"  /* Drainage of Accessory Nerve, Open Approach, Diagnostic */
"009R0ZZ"  =  "1"  /* Drainage of Accessory Nerve, Open Approach */
"009R30Z"  =  "1"  /* Drainage of Accessory Nerve with Drain Dev, Perc Approach */
"009R3ZZ"  =  "1"  /* Drainage of Accessory Nerve, Percutaneous Approach */
"009R40Z"  =  "1"  /* Drainage of Accessory Nrv with Drain Dev, Perc Endo Approach */
"009R4ZZ"  =  "1"  /* Drainage of Accessory Nerve, Perc Endo Approach */
"009S00Z"  =  "1"  /* Drainage of Hypoglossal Nerve with Drain Dev, Open Approach */
"009S0ZX"  =  "1"  /* Drainage of Hypoglossal Nerve, Open Approach, Diagnostic */
"009S0ZZ"  =  "1"  /* Drainage of Hypoglossal Nerve, Open Approach */
"009S30Z"  =  "1"  /* Drainage of Hypoglossal Nerve with Drain Dev, Perc Approach */
"009S3ZZ"  =  "1"  /* Drainage of Hypoglossal Nerve, Percutaneous Approach */
"009S40Z"  =  "1"  /* Drainage of Hypogloss Nrv with Drain Dev, Perc Endo Approach */
"009S4ZZ"  =  "1"  /* Drainage of Hypoglossal Nerve, Perc Endo Approach */
"009T00Z"  =  "1"  /* Drainage of Spinal Meninges with Drain Dev, Open Approach */
"009T0ZX"  =  "1"  /* Drainage of Spinal Meninges, Open Approach, Diagnostic */
"009T0ZZ"  =  "1"  /* Drainage of Spinal Meninges, Open Approach */
"009T30Z"  =  "1"  /* Drainage of Spinal Meninges with Drain Dev, Perc Approach */
"009T3ZX"  =  "1"  /* Drainage of Spinal Meninges, Percutaneous Approach, Diagn */
"009T3ZZ"  =  "1"  /* Drainage of Spinal Meninges, Percutaneous Approach */
"009T40Z"  =  "1"  /* Drain of Spinal Meninges with Drain Dev, Perc Endo Approach */
"009T4ZX"  =  "1"  /* Drainage of Spinal Meninges, Perc Endo Approach, Diagn */
"009T4ZZ"  =  "1"  /* Drainage of Spinal Meninges, Perc Endo Approach */
"009U00Z"  =  "1"  /* Drainage of Spinal Canal with Drainage Device, Open Approach */
"009U0ZX"  =  "1"  /* Drainage of Spinal Canal, Open Approach, Diagnostic */
"009U0ZZ"  =  "1"  /* Drainage of Spinal Canal, Open Approach */
"009W00Z"  =  "1"  /* Drainage of Cerv Spinal Cord with Drain Dev, Open Approach */
"009W0ZX"  =  "1"  /* Drainage of Cervical Spinal Cord, Open Approach, Diagnostic */
"009W0ZZ"  =  "1"  /* Drainage of Cervical Spinal Cord, Open Approach */
"009W30Z"  =  "1"  /* Drainage of Cerv Spinal Cord with Drain Dev, Perc Approach */
"009W3ZX"  =  "1"  /* Drainage of Cervical Spinal Cord, Perc Approach, Diagn */
"009W3ZZ"  =  "1"  /* Drainage of Cervical Spinal Cord, Percutaneous Approach */
"009W40Z"  =  "1"  /* Drain of Cerv Spinal Cord with Drain Dev, Perc Endo Approach */
"009W4ZX"  =  "1"  /* Drainage of Cervical Spinal Cord, Perc Endo Approach, Diagn */
"009W4ZZ"  =  "1"  /* Drainage of Cervical Spinal Cord, Perc Endo Approach */
"009X00Z"  =  "1"  /* Drainage of Thor Spinal Cord with Drain Dev, Open Approach */
"009X0ZX"  =  "1"  /* Drainage of Thoracic Spinal Cord, Open Approach, Diagnostic */
"009X0ZZ"  =  "1"  /* Drainage of Thoracic Spinal Cord, Open Approach */
"009X30Z"  =  "1"  /* Drainage of Thor Spinal Cord with Drain Dev, Perc Approach */
"009X3ZX"  =  "1"  /* Drainage of Thoracic Spinal Cord, Perc Approach, Diagn */
"009X3ZZ"  =  "1"  /* Drainage of Thoracic Spinal Cord, Percutaneous Approach */
"009X40Z"  =  "1"  /* Drain of Thor Spinal Cord with Drain Dev, Perc Endo Approach */
"009X4ZX"  =  "1"  /* Drainage of Thoracic Spinal Cord, Perc Endo Approach, Diagn */
"009X4ZZ"  =  "1"  /* Drainage of Thoracic Spinal Cord, Perc Endo Approach */
"009Y00Z"  =  "1"  /* Drainage of Lumbar Spinal Cord with Drain Dev, Open Approach */
"009Y0ZX"  =  "1"  /* Drainage of Lumbar Spinal Cord, Open Approach, Diagnostic */
"009Y0ZZ"  =  "1"  /* Drainage of Lumbar Spinal Cord, Open Approach */
"009Y30Z"  =  "1"  /* Drainage of Lumbar Spinal Cord with Drain Dev, Perc Approach */
"009Y3ZX"  =  "1"  /* Drainage of Lumbar Spinal Cord, Percutaneous Approach, Diagn */
"009Y3ZZ"  =  "1"  /* Drainage of Lumbar Spinal Cord, Percutaneous Approach */
"009Y40Z"  =  "1"  /* Drain of Lum Spinal Cord with Drain Dev, Perc Endo Approach */
"009Y4ZX"  =  "1"  /* Drainage of Lumbar Spinal Cord, Perc Endo Approach, Diagn */
"009Y4ZZ"  =  "1"  /* Drainage of Lumbar Spinal Cord, Perc Endo Approach */
"00B00ZX"  =  "1"  /* Excision of Brain, Open Approach, Diagnostic */
"00B00ZZ"  =  "1"  /* Excision of Brain, Open Approach */
"00B03ZZ"  =  "1"  /* Excision of Brain, Percutaneous Approach */
"00B04ZZ"  =  "1"  /* Excision of Brain, Percutaneous Endoscopic Approach */
"00B10ZX"  =  "1"  /* Excision of Cerebral Meninges, Open Approach, Diagnostic */
"00B10ZZ"  =  "1"  /* Excision of Cerebral Meninges, Open Approach */
"00B13ZZ"  =  "1"  /* Excision of Cerebral Meninges, Percutaneous Approach */
"00B14ZZ"  =  "1"  /* Excision of Cerebral Meninges, Perc Endo Approach */
"00B20ZX"  =  "1"  /* Excision of Dura Mater, Open Approach, Diagnostic */
"00B20ZZ"  =  "1"  /* Excision of Dura Mater, Open Approach */
"00B23ZZ"  =  "1"  /* Excision of Dura Mater, Percutaneous Approach */
"00B24ZZ"  =  "1"  /* Excision of Dura Mater, Percutaneous Endoscopic Approach */
"00B60ZX"  =  "1"  /* Excision of Cerebral Ventricle, Open Approach, Diagnostic */
"00B60ZZ"  =  "1"  /* Excision of Cerebral Ventricle, Open Approach */
"00B63ZZ"  =  "1"  /* Excision of Cerebral Ventricle, Percutaneous Approach */
"00B64ZZ"  =  "1"  /* Excision of Cerebral Ventricle, Perc Endo Approach */
"00B70ZX"  =  "1"  /* Excision of Cerebral Hemisphere, Open Approach, Diagnostic */
"00B70ZZ"  =  "1"  /* Excision of Cerebral Hemisphere, Open Approach */
"00B73ZZ"  =  "1"  /* Excision of Cerebral Hemisphere, Percutaneous Approach */
"00B74ZZ"  =  "1"  /* Excision of Cerebral Hemisphere, Perc Endo Approach */
"00B80ZX"  =  "1"  /* Excision of Basal Ganglia, Open Approach, Diagnostic */
"00B80ZZ"  =  "1"  /* Excision of Basal Ganglia, Open Approach */
"00B83ZZ"  =  "1"  /* Excision of Basal Ganglia, Percutaneous Approach */
"00B84ZZ"  =  "1"  /* Excision of Basal Ganglia, Percutaneous Endoscopic Approach */
"00B90ZX"  =  "1"  /* Excision of Thalamus, Open Approach, Diagnostic */
"00B90ZZ"  =  "1"  /* Excision of Thalamus, Open Approach */
"00B93ZZ"  =  "1"  /* Excision of Thalamus, Percutaneous Approach */
"00B94ZZ"  =  "1"  /* Excision of Thalamus, Percutaneous Endoscopic Approach */
"00BA0ZX"  =  "1"  /* Excision of Hypothalamus, Open Approach, Diagnostic */
"00BA0ZZ"  =  "1"  /* Excision of Hypothalamus, Open Approach */
"00BA3ZZ"  =  "1"  /* Excision of Hypothalamus, Percutaneous Approach */
"00BA4ZZ"  =  "1"  /* Excision of Hypothalamus, Percutaneous Endoscopic Approach */
"00BB0ZX"  =  "1"  /* Excision of Pons, Open Approach, Diagnostic */
"00BB0ZZ"  =  "1"  /* Excision of Pons, Open Approach */
"00BB3ZZ"  =  "1"  /* Excision of Pons, Percutaneous Approach */
"00BB4ZZ"  =  "1"  /* Excision of Pons, Percutaneous Endoscopic Approach */
"00BC0ZX"  =  "1"  /* Excision of Cerebellum, Open Approach, Diagnostic */
"00BC0ZZ"  =  "1"  /* Excision of Cerebellum, Open Approach */
"00BC3ZZ"  =  "1"  /* Excision of Cerebellum, Percutaneous Approach */
"00BC4ZZ"  =  "1"  /* Excision of Cerebellum, Percutaneous Endoscopic Approach */
"00BD0ZX"  =  "1"  /* Excision of Medulla Oblongata, Open Approach, Diagnostic */
"00BD0ZZ"  =  "1"  /* Excision of Medulla Oblongata, Open Approach */
"00BD3ZZ"  =  "1"  /* Excision of Medulla Oblongata, Percutaneous Approach */
"00BD4ZZ"  =  "1"  /* Excision of Medulla Oblongata, Perc Endo Approach */
"00BF0ZX"  =  "1"  /* Excision of Olfactory Nerve, Open Approach, Diagnostic */
"00BF0ZZ"  =  "1"  /* Excision of Olfactory Nerve, Open Approach */
"00BF3ZZ"  =  "1"  /* Excision of Olfactory Nerve, Percutaneous Approach */
"00BF4ZZ"  =  "1"  /* Excision of Olfactory Nerve, Perc Endo Approach */
"00BG0ZX"  =  "1"  /* Excision of Optic Nerve, Open Approach, Diagnostic */
"00BG0ZZ"  =  "1"  /* Excision of Optic Nerve, Open Approach */
"00BG3ZZ"  =  "1"  /* Excision of Optic Nerve, Percutaneous Approach */
"00BG4ZZ"  =  "1"  /* Excision of Optic Nerve, Percutaneous Endoscopic Approach */
"00BH0ZX"  =  "1"  /* Excision of Oculomotor Nerve, Open Approach, Diagnostic */
"00BH0ZZ"  =  "1"  /* Excision of Oculomotor Nerve, Open Approach */
"00BH3ZZ"  =  "1"  /* Excision of Oculomotor Nerve, Percutaneous Approach */
"00BH4ZZ"  =  "1"  /* Excision of Oculomotor Nerve, Perc Endo Approach */
"00BJ0ZX"  =  "1"  /* Excision of Trochlear Nerve, Open Approach, Diagnostic */
"00BJ0ZZ"  =  "1"  /* Excision of Trochlear Nerve, Open Approach */
"00BJ3ZZ"  =  "1"  /* Excision of Trochlear Nerve, Percutaneous Approach */
"00BJ4ZZ"  =  "1"  /* Excision of Trochlear Nerve, Perc Endo Approach */
"00BK0ZX"  =  "1"  /* Excision of Trigeminal Nerve, Open Approach, Diagnostic */
"00BK0ZZ"  =  "1"  /* Excision of Trigeminal Nerve, Open Approach */
"00BK3ZZ"  =  "1"  /* Excision of Trigeminal Nerve, Percutaneous Approach */
"00BK4ZZ"  =  "1"  /* Excision of Trigeminal Nerve, Perc Endo Approach */
"00BL0ZX"  =  "1"  /* Excision of Abducens Nerve, Open Approach, Diagnostic */
"00BL0ZZ"  =  "1"  /* Excision of Abducens Nerve, Open Approach */
"00BL3ZZ"  =  "1"  /* Excision of Abducens Nerve, Percutaneous Approach */
"00BL4ZZ"  =  "1"  /* Excision of Abducens Nerve, Percutaneous Endoscopic Approach */
"00BM0ZX"  =  "1"  /* Excision of Facial Nerve, Open Approach, Diagnostic */
"00BM0ZZ"  =  "1"  /* Excision of Facial Nerve, Open Approach */
"00BM3ZZ"  =  "1"  /* Excision of Facial Nerve, Percutaneous Approach */
"00BM4ZZ"  =  "1"  /* Excision of Facial Nerve, Percutaneous Endoscopic Approach */
"00BN0ZX"  =  "1"  /* Excision of Acoustic Nerve, Open Approach, Diagnostic */
"00BN0ZZ"  =  "1"  /* Excision of Acoustic Nerve, Open Approach */
"00BN3ZZ"  =  "1"  /* Excision of Acoustic Nerve, Percutaneous Approach */
"00BN4ZZ"  =  "1"  /* Excision of Acoustic Nerve, Percutaneous Endoscopic Approach */
"00BP0ZX"  =  "1"  /* Excision of Glossopharyngeal Nerve, Open Approach, Diagn */
"00BP0ZZ"  =  "1"  /* Excision of Glossopharyngeal Nerve, Open Approach */
"00BP3ZZ"  =  "1"  /* Excision of Glossopharyngeal Nerve, Percutaneous Approach */
"00BP4ZZ"  =  "1"  /* Excision of Glossopharyngeal Nerve, Perc Endo Approach */
"00BQ0ZX"  =  "1"  /* Excision of Vagus Nerve, Open Approach, Diagnostic */
"00BQ0ZZ"  =  "1"  /* Excision of Vagus Nerve, Open Approach */
"00BQ3ZZ"  =  "1"  /* Excision of Vagus Nerve, Percutaneous Approach */
"00BQ4ZZ"  =  "1"  /* Excision of Vagus Nerve, Percutaneous Endoscopic Approach */
"00BR0ZX"  =  "1"  /* Excision of Accessory Nerve, Open Approach, Diagnostic */
"00BR0ZZ"  =  "1"  /* Excision of Accessory Nerve, Open Approach */
"00BR3ZZ"  =  "1"  /* Excision of Accessory Nerve, Percutaneous Approach */
"00BR4ZZ"  =  "1"  /* Excision of Accessory Nerve, Perc Endo Approach */
"00BS0ZX"  =  "1"  /* Excision of Hypoglossal Nerve, Open Approach, Diagnostic */
"00BS0ZZ"  =  "1"  /* Excision of Hypoglossal Nerve, Open Approach */
"00BS3ZZ"  =  "1"  /* Excision of Hypoglossal Nerve, Percutaneous Approach */
"00BS4ZZ"  =  "1"  /* Excision of Hypoglossal Nerve, Perc Endo Approach */
"00BT0ZX"  =  "1"  /* Excision of Spinal Meninges, Open Approach, Diagnostic */
"00BT0ZZ"  =  "1"  /* Excision of Spinal Meninges, Open Approach */
"00BT3ZX"  =  "1"  /* Excision of Spinal Meninges, Percutaneous Approach, Diagn */
"00BT3ZZ"  =  "1"  /* Excision of Spinal Meninges, Percutaneous Approach */
"00BT4ZX"  =  "1"  /* Excision of Spinal Meninges, Perc Endo Approach, Diagn */
"00BT4ZZ"  =  "1"  /* Excision of Spinal Meninges, Perc Endo Approach */
"00BW0ZX"  =  "1"  /* Excision of Cervical Spinal Cord, Open Approach, Diagnostic */
"00BW0ZZ"  =  "1"  /* Excision of Cervical Spinal Cord, Open Approach */
"00BW3ZX"  =  "1"  /* Excision of Cervical Spinal Cord, Perc Approach, Diagn */
"00BW3ZZ"  =  "1"  /* Excision of Cervical Spinal Cord, Percutaneous Approach */
"00BW4ZX"  =  "1"  /* Excision of Cervical Spinal Cord, Perc Endo Approach, Diagn */
"00BW4ZZ"  =  "1"  /* Excision of Cervical Spinal Cord, Perc Endo Approach */
"00BX0ZX"  =  "1"  /* Excision of Thoracic Spinal Cord, Open Approach, Diagnostic */
"00BX0ZZ"  =  "1"  /* Excision of Thoracic Spinal Cord, Open Approach */
"00BX3ZX"  =  "1"  /* Excision of Thoracic Spinal Cord, Perc Approach, Diagn */
"00BX3ZZ"  =  "1"  /* Excision of Thoracic Spinal Cord, Percutaneous Approach */
"00BX4ZX"  =  "1"  /* Excision of Thoracic Spinal Cord, Perc Endo Approach, Diagn */
"00BX4ZZ"  =  "1"  /* Excision of Thoracic Spinal Cord, Perc Endo Approach */
"00BY0ZX"  =  "1"  /* Excision of Lumbar Spinal Cord, Open Approach, Diagnostic */
"00BY0ZZ"  =  "1"  /* Excision of Lumbar Spinal Cord, Open Approach */
"00BY3ZX"  =  "1"  /* Excision of Lumbar Spinal Cord, Percutaneous Approach, Diagn */
"00BY3ZZ"  =  "1"  /* Excision of Lumbar Spinal Cord, Percutaneous Approach */
"00BY4ZX"  =  "1"  /* Excision of Lumbar Spinal Cord, Perc Endo Approach, Diagn */
"00BY4ZZ"  =  "1"  /* Excision of Lumbar Spinal Cord, Perc Endo Approach */
"00C00ZZ"  =  "1"  /* Extirpation of Matter from Brain, Open Approach */
"00C03ZZ"  =  "1"  /* Extirpation of Matter from Brain, Percutaneous Approach */
"00C04ZZ"  =  "1"  /* Extirpation of Matter from Brain, Perc Endo Approach */
"00C10ZZ"  =  "1"  /* Extirpation of Matter from Cerebral Meninges, Open Approach */
"00C13ZZ"  =  "1"  /* Extirpation of Matter from Cerebral Meninges, Perc Approach */
"00C14ZZ"  =  "1"  /* Extirpate of Matter from Cereb Meninges, Perc Endo Approach */
"00C20ZZ"  =  "1"  /* Extirpation of Matter from Dura Mater, Open Approach */
"00C23ZZ"  =  "1"  /* Extirpation of Matter from Dura Mater, Percutaneous Approach */
"00C24ZZ"  =  "1"  /* Extirpation of Matter from Dura Mater, Perc Endo Approach */
"00C30ZZ"  =  "1"  /* Extirpation of Matter from Epidural Space, Open Approach */
"00C33ZZ"  =  "1"  /* Extirpation of Matter from Epidural Space, Perc Approach */
"00C34ZZ"  =  "1"  /* Extirpate of Matter from Epidural Space, Perc Endo Approach */
"00C40ZZ"  =  "1"  /* Extirpation of Matter from Subdural Space, Open Approach */
"00C43ZZ"  =  "1"  /* Extirpation of Matter from Subdural Space, Perc Approach */
"00C44ZZ"  =  "1"  /* Extirpate of Matter from Subdural Space, Perc Endo Approach */
"00C50ZZ"  =  "1"  /* Extirpation of Matter from Subarachnoid Space, Open Approach */
"00C53ZZ"  =  "1"  /* Extirpation of Matter from Subarachnoid Space, Perc Approach */
"00C54ZZ"  =  "1"  /* Extirpate of Matter from Subarach Space, Perc Endo Approach */
"00C60ZZ"  =  "1"  /* Extirpation of Matter from Cerebral Ventricle, Open Approach */
"00C63ZZ"  =  "1"  /* Extirpation of Matter from Cerebral Ventricle, Perc Approach */
"00C64ZZ"  =  "1"  /* Extirpation of Matter from Cereb Vent, Perc Endo Approach */
"00C70ZZ"  =  "1"  /* Extirpation of Matter from Cereb Hemis, Open Approach */
"00C73ZZ"  =  "1"  /* Extirpation of Matter from Cereb Hemis, Perc Approach */
"00C74ZZ"  =  "1"  /* Extirpation of Matter from Cereb Hemis, Perc Endo Approach */
"00C80ZZ"  =  "1"  /* Extirpation of Matter from Basal Ganglia, Open Approach */
"00C83ZZ"  =  "1"  /* Extirpation of Matter from Basal Ganglia, Perc Approach */
"00C84ZZ"  =  "1"  /* Extirpation of Matter from Basal Ganglia, Perc Endo Approach */
"00C90ZZ"  =  "1"  /* Extirpation of Matter from Thalamus, Open Approach */
"00C93ZZ"  =  "1"  /* Extirpation of Matter from Thalamus, Percutaneous Approach */
"00C94ZZ"  =  "1"  /* Extirpation of Matter from Thalamus, Perc Endo Approach */
"00CA0ZZ"  =  "1"  /* Extirpation of Matter from Hypothalamus, Open Approach */
"00CA3ZZ"  =  "1"  /* Extirpation of Matter from Hypothalamus, Perc Approach */
"00CA4ZZ"  =  "1"  /* Extirpation of Matter from Hypothalamus, Perc Endo Approach */
"00CB0ZZ"  =  "1"  /* Extirpation of Matter from Pons, Open Approach */
"00CB3ZZ"  =  "1"  /* Extirpation of Matter from Pons, Percutaneous Approach */
"00CB4ZZ"  =  "1"  /* Extirpation of Matter from Pons, Perc Endo Approach */
"00CC0ZZ"  =  "1"  /* Extirpation of Matter from Cerebellum, Open Approach */
"00CC3ZZ"  =  "1"  /* Extirpation of Matter from Cerebellum, Percutaneous Approach */
"00CC4ZZ"  =  "1"  /* Extirpation of Matter from Cerebellum, Perc Endo Approach */
"00CD0ZZ"  =  "1"  /* Extirpation of Matter from Medulla Oblongata, Open Approach */
"00CD3ZZ"  =  "1"  /* Extirpation of Matter from Medulla Oblongata, Perc Approach */
"00CD4ZZ"  =  "1"  /* Extirpate of Matter from Medulla Oblong, Perc Endo Approach */
"00CF0ZZ"  =  "1"  /* Extirpation of Matter from Olfactory Nerve, Open Approach */
"00CF3ZZ"  =  "1"  /* Extirpation of Matter from Olfactory Nerve, Perc Approach */
"00CF4ZZ"  =  "1"  /* Extirpation of Matter from Olfact Nrv, Perc Endo Approach */
"00CG0ZZ"  =  "1"  /* Extirpation of Matter from Optic Nerve, Open Approach */
"00CG3ZZ"  =  "1"  /* Extirpation of Matter from Optic Nerve, Perc Approach */
"00CG4ZZ"  =  "1"  /* Extirpation of Matter from Optic Nerve, Perc Endo Approach */
"00CH0ZZ"  =  "1"  /* Extirpation of Matter from Oculomotor Nerve, Open Approach */
"00CH3ZZ"  =  "1"  /* Extirpation of Matter from Oculomotor Nerve, Perc Approach */
"00CH4ZZ"  =  "1"  /* Extirpate of Matter from Oculomotor Nrv, Perc Endo Approach */
"00CJ0ZZ"  =  "1"  /* Extirpation of Matter from Trochlear Nerve, Open Approach */
"00CJ3ZZ"  =  "1"  /* Extirpation of Matter from Trochlear Nerve, Perc Approach */
"00CJ4ZZ"  =  "1"  /* Extirpation of Matter from Trochlear Nrv, Perc Endo Approach */
"00CK0ZZ"  =  "1"  /* Extirpation of Matter from Trigeminal Nerve, Open Approach */
"00CK3ZZ"  =  "1"  /* Extirpation of Matter from Trigeminal Nerve, Perc Approach */
"00CK4ZZ"  =  "1"  /* Extirpation of Matter from Trigem Nrv, Perc Endo Approach */
"00CL0ZZ"  =  "1"  /* Extirpation of Matter from Abducens Nerve, Open Approach */
"00CL3ZZ"  =  "1"  /* Extirpation of Matter from Abducens Nerve, Perc Approach */
"00CL4ZZ"  =  "1"  /* Extirpation of Matter from Abducen Nrv, Perc Endo Approach */
"00CM0ZZ"  =  "1"  /* Extirpation of Matter from Facial Nerve, Open Approach */
"00CM3ZZ"  =  "1"  /* Extirpation of Matter from Facial Nerve, Perc Approach */
"00CM4ZZ"  =  "1"  /* Extirpation of Matter from Facial Nerve, Perc Endo Approach */
"00CN0ZZ"  =  "1"  /* Extirpation of Matter from Acoustic Nerve, Open Approach */
"00CN3ZZ"  =  "1"  /* Extirpation of Matter from Acoustic Nerve, Perc Approach */
"00CN4ZZ"  =  "1"  /* Extirpation of Matter from Acoust Nrv, Perc Endo Approach */
"00CP0ZZ"  =  "1"  /* Extirpation of Matter from Glossophar Nrv, Open Approach */
"00CP3ZZ"  =  "1"  /* Extirpation of Matter from Glossophar Nrv, Perc Approach */
"00CP4ZZ"  =  "1"  /* Extirpate of Matter from Glossophar Nrv, Perc Endo Approach */
"00CQ0ZZ"  =  "1"  /* Extirpation of Matter from Vagus Nerve, Open Approach */
"00CQ3ZZ"  =  "1"  /* Extirpation of Matter from Vagus Nerve, Perc Approach */
"00CQ4ZZ"  =  "1"  /* Extirpation of Matter from Vagus Nerve, Perc Endo Approach */
"00CR0ZZ"  =  "1"  /* Extirpation of Matter from Accessory Nerve, Open Approach */
"00CR3ZZ"  =  "1"  /* Extirpation of Matter from Accessory Nerve, Perc Approach */
"00CR4ZZ"  =  "1"  /* Extirpation of Matter from Accessory Nrv, Perc Endo Approach */
"00CS0ZZ"  =  "1"  /* Extirpation of Matter from Hypoglossal Nerve, Open Approach */
"00CS3ZZ"  =  "1"  /* Extirpation of Matter from Hypoglossal Nerve, Perc Approach */
"00CS4ZZ"  =  "1"  /* Extirpation of Matter from Hypogloss Nrv, Perc Endo Approach */
"00CT0ZZ"  =  "1"  /* Extirpation of Matter from Spinal Meninges, Open Approach */
"00CT3ZZ"  =  "1"  /* Extirpation of Matter from Spinal Meninges, Perc Approach */
"00CT4ZZ"  =  "1"  /* Extirpate of Matter from Spinal Meninges, Perc Endo Approach */
"00CW0ZZ"  =  "1"  /* Extirpation of Matter from Cerv Spinal Cord, Open Approach */
"00CW3ZZ"  =  "1"  /* Extirpation of Matter from Cerv Spinal Cord, Perc Approach */
"00CW4ZZ"  =  "1"  /* Extirpate matter from Cerv Spinal Cord, Perc Endo */
"00CX0ZZ"  =  "1"  /* Extirpation of Matter from Thor Spinal Cord, Open Approach */
"00CX3ZZ"  =  "1"  /* Extirpation of Matter from Thor Spinal Cord, Perc Approach */
"00CX4ZZ"  =  "1"  /* Extirpate matter from Thor Spinal Cord, Perc Endo */
"00CY0ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Spinal Cord, Open Approach */
"00CY3ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Spinal Cord, Perc Approach */
"00CY4ZZ"  =  "1"  /* Extirpate of Matter from Lum Spinal Cord, Perc Endo Approach */
"00D10ZZ"  =  "1"  /* Extraction of Cerebral Meninges, Open Approach */
"00D13ZZ"  =  "1"  /* Extraction of Cerebral Meninges, Percutaneous Approach */
"00D14ZZ"  =  "1"  /* Extraction of Cerebral Meninges, Perc Endo Approach */
"00D20ZZ"  =  "1"  /* Extraction of Dura Mater, Open Approach */
"00D23ZZ"  =  "1"  /* Extraction of Dura Mater, Percutaneous Approach */
"00D24ZZ"  =  "1"  /* Extraction of Dura Mater, Percutaneous Endoscopic Approach */
"00DF0ZZ"  =  "1"  /* Extraction of Olfactory Nerve, Open Approach */
"00DF3ZZ"  =  "1"  /* Extraction of Olfactory Nerve, Percutaneous Approach */
"00DF4ZZ"  =  "1"  /* Extraction of Olfactory Nerve, Perc Endo Approach */
"00DG0ZZ"  =  "1"  /* Extraction of Optic Nerve, Open Approach */
"00DG3ZZ"  =  "1"  /* Extraction of Optic Nerve, Percutaneous Approach */
"00DG4ZZ"  =  "1"  /* Extraction of Optic Nerve, Percutaneous Endoscopic Approach */
"00DH0ZZ"  =  "1"  /* Extraction of Oculomotor Nerve, Open Approach */
"00DH3ZZ"  =  "1"  /* Extraction of Oculomotor Nerve, Percutaneous Approach */
"00DH4ZZ"  =  "1"  /* Extraction of Oculomotor Nerve, Perc Endo Approach */
"00DJ0ZZ"  =  "1"  /* Extraction of Trochlear Nerve, Open Approach */
"00DJ3ZZ"  =  "1"  /* Extraction of Trochlear Nerve, Percutaneous Approach */
"00DJ4ZZ"  =  "1"  /* Extraction of Trochlear Nerve, Perc Endo Approach */
"00DK0ZZ"  =  "1"  /* Extraction of Trigeminal Nerve, Open Approach */
"00DK3ZZ"  =  "1"  /* Extraction of Trigeminal Nerve, Percutaneous Approach */
"00DK4ZZ"  =  "1"  /* Extraction of Trigeminal Nerve, Perc Endo Approach */
"00DL0ZZ"  =  "1"  /* Extraction of Abducens Nerve, Open Approach */
"00DL3ZZ"  =  "1"  /* Extraction of Abducens Nerve, Percutaneous Approach */
"00DL4ZZ"  =  "1"  /* Extraction of Abducens Nerve, Perc Endo Approach */
"00DM0ZZ"  =  "1"  /* Extraction of Facial Nerve, Open Approach */
"00DM3ZZ"  =  "1"  /* Extraction of Facial Nerve, Percutaneous Approach */
"00DM4ZZ"  =  "1"  /* Extraction of Facial Nerve, Percutaneous Endoscopic Approach */
"00DN0ZZ"  =  "1"  /* Extraction of Acoustic Nerve, Open Approach */
"00DN3ZZ"  =  "1"  /* Extraction of Acoustic Nerve, Percutaneous Approach */
"00DN4ZZ"  =  "1"  /* Extraction of Acoustic Nerve, Perc Endo Approach */
"00DP0ZZ"  =  "1"  /* Extraction of Glossopharyngeal Nerve, Open Approach */
"00DP3ZZ"  =  "1"  /* Extraction of Glossopharyngeal Nerve, Percutaneous Approach */
"00DP4ZZ"  =  "1"  /* Extraction of Glossopharyngeal Nerve, Perc Endo Approach */
"00DQ0ZZ"  =  "1"  /* Extraction of Vagus Nerve, Open Approach */
"00DQ3ZZ"  =  "1"  /* Extraction of Vagus Nerve, Percutaneous Approach */
"00DQ4ZZ"  =  "1"  /* Extraction of Vagus Nerve, Percutaneous Endoscopic Approach */
"00DR0ZZ"  =  "1"  /* Extraction of Accessory Nerve, Open Approach */
"00DR3ZZ"  =  "1"  /* Extraction of Accessory Nerve, Percutaneous Approach */
"00DR4ZZ"  =  "1"  /* Extraction of Accessory Nerve, Perc Endo Approach */
"00DS0ZZ"  =  "1"  /* Extraction of Hypoglossal Nerve, Open Approach */
"00DS3ZZ"  =  "1"  /* Extraction of Hypoglossal Nerve, Percutaneous Approach */
"00DS4ZZ"  =  "1"  /* Extraction of Hypoglossal Nerve, Perc Endo Approach */
"00DT0ZZ"  =  "1"  /* Extraction of Spinal Meninges, Open Approach */
"00DT3ZZ"  =  "1"  /* Extraction of Spinal Meninges, Percutaneous Approach */
"00DT4ZZ"  =  "1"  /* Extraction of Spinal Meninges, Perc Endo Approach */
"00F30ZZ"  =  "1"  /* Fragmentation in Epidural Space, Open Approach */
"00F33ZZ"  =  "1"  /* Fragmentation in Epidural Space, Percutaneous Approach */
"00F34ZZ"  =  "1"  /* Fragmentation in Epidural Space, Perc Endo Approach */
"00F40ZZ"  =  "1"  /* Fragmentation in Subdural Space, Open Approach */
"00F43ZZ"  =  "1"  /* Fragmentation in Subdural Space, Percutaneous Approach */
"00F44ZZ"  =  "1"  /* Fragmentation in Subdural Space, Perc Endo Approach */
"00F50ZZ"  =  "1"  /* Fragmentation in Subarachnoid Space, Open Approach */
"00F53ZZ"  =  "1"  /* Fragmentation in Subarachnoid Space, Percutaneous Approach */
"00F54ZZ"  =  "1"  /* Fragmentation in Subarachnoid Space, Perc Endo Approach */
"00F60ZZ"  =  "1"  /* Fragmentation in Cerebral Ventricle, Open Approach */
"00F63ZZ"  =  "1"  /* Fragmentation in Cerebral Ventricle, Percutaneous Approach */
"00F64ZZ"  =  "1"  /* Fragmentation in Cerebral Ventricle, Perc Endo Approach */
"00FU0ZZ"  =  "1"  /* Fragmentation in Spinal Canal, Open Approach */
"00FU3ZZ"  =  "1"  /* Fragmentation in Spinal Canal, Percutaneous Approach */
"00FU4ZZ"  =  "1"  /* Fragmentation in Spinal Canal, Perc Endo Approach */
"00FUXZZ"  =  "1"  /* Fragmentation in Spinal Canal, External Approach */
"00H002Z"  =  "1"  /* Insertion of Monitoring Device into Brain, Open Approach */
"00H003Z"  =  "1"  /* Insertion of Infusion Device into Brain, Open Approach */
"00H00MZ"  =  "1"  /* Insertion of Neurostimulator Lead into Brain, Open Approach */
"00H032Z"  =  "1"  /* Insertion of Monitoring Device into Brain, Perc Approach */
"00H033Z"  =  "1"  /* Insertion of Infusion Device into Brain, Perc Approach */
"00H03MZ"  =  "1"  /* Insertion of Neurostimulator Lead into Brain, Perc Approach */
"00H042Z"  =  "1"  /* Insertion of Monitor Dev into Brain, Perc Endo Approach */
"00H043Z"  =  "1"  /* Insertion of Infusion Device into Brain, Perc Endo Approach */
"00H04MZ"  =  "1"  /* Insertion of Neuro Lead into Brain, Perc Endo Approach */
"00H602Z"  =  "1"  /* Insertion of Monitor Dev into Cereb Vent, Open Approach */
"00H603Z"  =  "1"  /* Insertion of Infusion Device into Cereb Vent, Open Approach */
"00H60MZ"  =  "1"  /* Insertion of Neuro Lead into Cereb Vent, Open Approach */
"00H632Z"  =  "1"  /* Insertion of Monitor Dev into Cereb Vent, Perc Approach */
"00H633Z"  =  "1"  /* Insertion of Infusion Device into Cereb Vent, Perc Approach */
"00H63MZ"  =  "1"  /* Insertion of Neuro Lead into Cereb Vent, Perc Approach */
"00H642Z"  =  "1"  /* Insertion of Monitor Dev into Cereb Vent, Perc Endo Approach */
"00H643Z"  =  "1"  /* Insert of Infusion Dev into Cereb Vent, Perc Endo Approach */
"00H64MZ"  =  "1"  /* Insertion of Neuro Lead into Cereb Vent, Perc Endo Approach */
"00HE02Z"  =  "1"  /* Insertion of Monitor Dev into Cranial Nrv, Open Approach */
"00HE03Z"  =  "1"  /* Insertion of Infusion Device into Cranial Nrv, Open Approach */
"00HE0MZ"  =  "1"  /* Insertion of Neuro Lead into Cranial Nrv, Open Approach */
"00HE32Z"  =  "1"  /* Insertion of Monitor Dev into Cranial Nrv, Perc Approach */
"00HE33Z"  =  "1"  /* Insertion of Infusion Device into Cranial Nrv, Perc Approach */
"00HE3MZ"  =  "1"  /* Insertion of Neuro Lead into Cranial Nrv, Perc Approach */
"00HE42Z"  =  "1"  /* Insert of Monitor Dev into Cranial Nrv, Perc Endo Approach */
"00HE43Z"  =  "1"  /* Insert of Infusion Dev into Cranial Nrv, Perc Endo Approach */
"00HE4MZ"  =  "1"  /* Insertion of Neuro Lead into Cranial Nrv, Perc Endo Approach */
"00HU02Z"  =  "1"  /* Insertion of Monitor Dev into Spinal Canal, Open Approach */
"00HU0MZ"  =  "1"  /* Insertion of Neuro Lead into Spinal Canal, Open Approach */
"00HU32Z"  =  "1"  /* Insertion of Monitor Dev into Spinal Canal, Perc Approach */
"00HU3MZ"  =  "1"  /* Insertion of Neuro Lead into Spinal Canal, Perc Approach */
"00HU42Z"  =  "1"  /* Insert of Monitor Dev into Spinal Canal, Perc Endo Approach */
"00HU4MZ"  =  "1"  /* Insert of Neuro Lead into Spinal Canal, Perc Endo Approach */
"00HV02Z"  =  "1"  /* Insertion of Monitor Dev into Spinal Cord, Open Approach */
"00HV0MZ"  =  "1"  /* Insertion of Neuro Lead into Spinal Cord, Open Approach */
"00HV32Z"  =  "1"  /* Insertion of Monitor Dev into Spinal Cord, Perc Approach */
"00HV3MZ"  =  "1"  /* Insertion of Neuro Lead into Spinal Cord, Perc Approach */
"00HV42Z"  =  "1"  /* Insert of Monitor Dev into Spinal Cord, Perc Endo Approach */
"00HV4MZ"  =  "1"  /* Insertion of Neuro Lead into Spinal Cord, Perc Endo Approach */
"00J00ZZ"  =  "1"  /* Inspection of Brain, Open Approach */
"00J03ZZ"  =  "1"  /* Inspection of Brain, Percutaneous Approach */
"00J04ZZ"  =  "1"  /* Inspection of Brain, Percutaneous Endoscopic Approach */
"00JE0ZZ"  =  "1"  /* Inspection of Cranial Nerve, Open Approach */
"00JE3ZZ"  =  "1"  /* Inspection of Cranial Nerve, Percutaneous Approach */
"00JE4ZZ"  =  "1"  /* Inspection of Cranial Nerve, Perc Endo Approach */
"00JU0ZZ"  =  "1"  /* Inspection of Spinal Canal, Open Approach */
"00JU3ZZ"  =  "1"  /* Inspection of Spinal Canal, Percutaneous Approach */
"00JU4ZZ"  =  "1"  /* Inspection of Spinal Canal, Percutaneous Endoscopic Approach */
"00JV0ZZ"  =  "1"  /* Inspection of Spinal Cord, Open Approach */
"00JV3ZZ"  =  "1"  /* Inspection of Spinal Cord, Percutaneous Approach */
"00JV4ZZ"  =  "1"  /* Inspection of Spinal Cord, Percutaneous Endoscopic Approach */
"00K00ZZ"  =  "1"  /* Map Brain, Open Approach */
"00K03ZZ"  =  "1"  /* Map Brain, Percutaneous Approach */
"00K04ZZ"  =  "1"  /* Map Brain, Percutaneous Endoscopic Approach */
"00K70ZZ"  =  "1"  /* Map Cerebral Hemisphere, Open Approach */
"00K73ZZ"  =  "1"  /* Map Cerebral Hemisphere, Percutaneous Approach */
"00K74ZZ"  =  "1"  /* Map Cerebral Hemisphere, Percutaneous Endoscopic Approach */
"00K80ZZ"  =  "1"  /* Map Basal Ganglia, Open Approach */
"00K83ZZ"  =  "1"  /* Map Basal Ganglia, Percutaneous Approach */
"00K84ZZ"  =  "1"  /* Map Basal Ganglia, Percutaneous Endoscopic Approach */
"00K90ZZ"  =  "1"  /* Map Thalamus, Open Approach */
"00K93ZZ"  =  "1"  /* Map Thalamus, Percutaneous Approach */
"00K94ZZ"  =  "1"  /* Map Thalamus, Percutaneous Endoscopic Approach */
"00KA0ZZ"  =  "1"  /* Map Hypothalamus, Open Approach */
"00KA3ZZ"  =  "1"  /* Map Hypothalamus, Percutaneous Approach */
"00KA4ZZ"  =  "1"  /* Map Hypothalamus, Percutaneous Endoscopic Approach */
"00KB0ZZ"  =  "1"  /* Map Pons, Open Approach */
"00KB3ZZ"  =  "1"  /* Map Pons, Percutaneous Approach */
"00KB4ZZ"  =  "1"  /* Map Pons, Percutaneous Endoscopic Approach */
"00KC0ZZ"  =  "1"  /* Map Cerebellum, Open Approach */
"00KC3ZZ"  =  "1"  /* Map Cerebellum, Percutaneous Approach */
"00KC4ZZ"  =  "1"  /* Map Cerebellum, Percutaneous Endoscopic Approach */
"00KD0ZZ"  =  "1"  /* Map Medulla Oblongata, Open Approach */
"00KD3ZZ"  =  "1"  /* Map Medulla Oblongata, Percutaneous Approach */
"00KD4ZZ"  =  "1"  /* Map Medulla Oblongata, Percutaneous Endoscopic Approach */
"00N00ZZ"  =  "1"  /* Release Brain, Open Approach */
"00N03ZZ"  =  "1"  /* Release Brain, Percutaneous Approach */
"00N04ZZ"  =  "1"  /* Release Brain, Percutaneous Endoscopic Approach */
"00N10ZZ"  =  "1"  /* Release Cerebral Meninges, Open Approach */
"00N13ZZ"  =  "1"  /* Release Cerebral Meninges, Percutaneous Approach */
"00N14ZZ"  =  "1"  /* Release Cerebral Meninges, Percutaneous Endoscopic Approach */
"00N20ZZ"  =  "1"  /* Release Dura Mater, Open Approach */
"00N23ZZ"  =  "1"  /* Release Dura Mater, Percutaneous Approach */
"00N24ZZ"  =  "1"  /* Release Dura Mater, Percutaneous Endoscopic Approach */
"00N60ZZ"  =  "1"  /* Release Cerebral Ventricle, Open Approach */
"00N63ZZ"  =  "1"  /* Release Cerebral Ventricle, Percutaneous Approach */
"00N64ZZ"  =  "1"  /* Release Cerebral Ventricle, Percutaneous Endoscopic Approach */
"00N70ZZ"  =  "1"  /* Release Cerebral Hemisphere, Open Approach */
"00N73ZZ"  =  "1"  /* Release Cerebral Hemisphere, Percutaneous Approach */
"00N74ZZ"  =  "1"  /* Release Cerebral Hemisphere, Perc Endo Approach */
"00N80ZZ"  =  "1"  /* Release Basal Ganglia, Open Approach */
"00N83ZZ"  =  "1"  /* Release Basal Ganglia, Percutaneous Approach */
"00N84ZZ"  =  "1"  /* Release Basal Ganglia, Percutaneous Endoscopic Approach */
"00N90ZZ"  =  "1"  /* Release Thalamus, Open Approach */
"00N93ZZ"  =  "1"  /* Release Thalamus, Percutaneous Approach */
"00N94ZZ"  =  "1"  /* Release Thalamus, Percutaneous Endoscopic Approach */
"00NA0ZZ"  =  "1"  /* Release Hypothalamus, Open Approach */
"00NA3ZZ"  =  "1"  /* Release Hypothalamus, Percutaneous Approach */
"00NA4ZZ"  =  "1"  /* Release Hypothalamus, Percutaneous Endoscopic Approach */
"00NB0ZZ"  =  "1"  /* Release Pons, Open Approach */
"00NB3ZZ"  =  "1"  /* Release Pons, Percutaneous Approach */
"00NB4ZZ"  =  "1"  /* Release Pons, Percutaneous Endoscopic Approach */
"00NC0ZZ"  =  "1"  /* Release Cerebellum, Open Approach */
"00NC3ZZ"  =  "1"  /* Release Cerebellum, Percutaneous Approach */
"00NC4ZZ"  =  "1"  /* Release Cerebellum, Percutaneous Endoscopic Approach */
"00ND0ZZ"  =  "1"  /* Release Medulla Oblongata, Open Approach */
"00ND3ZZ"  =  "1"  /* Release Medulla Oblongata, Percutaneous Approach */
"00ND4ZZ"  =  "1"  /* Release Medulla Oblongata, Percutaneous Endoscopic Approach */
"00NF0ZZ"  =  "1"  /* Release Olfactory Nerve, Open Approach */
"00NF3ZZ"  =  "1"  /* Release Olfactory Nerve, Percutaneous Approach */
"00NF4ZZ"  =  "1"  /* Release Olfactory Nerve, Percutaneous Endoscopic Approach */
"00NG0ZZ"  =  "1"  /* Release Optic Nerve, Open Approach */
"00NG3ZZ"  =  "1"  /* Release Optic Nerve, Percutaneous Approach */
"00NG4ZZ"  =  "1"  /* Release Optic Nerve, Percutaneous Endoscopic Approach */
"00NH0ZZ"  =  "1"  /* Release Oculomotor Nerve, Open Approach */
"00NH3ZZ"  =  "1"  /* Release Oculomotor Nerve, Percutaneous Approach */
"00NH4ZZ"  =  "1"  /* Release Oculomotor Nerve, Percutaneous Endoscopic Approach */
"00NJ0ZZ"  =  "1"  /* Release Trochlear Nerve, Open Approach */
"00NJ3ZZ"  =  "1"  /* Release Trochlear Nerve, Percutaneous Approach */
"00NJ4ZZ"  =  "1"  /* Release Trochlear Nerve, Percutaneous Endoscopic Approach */
"00NK0ZZ"  =  "1"  /* Release Trigeminal Nerve, Open Approach */
"00NK3ZZ"  =  "1"  /* Release Trigeminal Nerve, Percutaneous Approach */
"00NK4ZZ"  =  "1"  /* Release Trigeminal Nerve, Percutaneous Endoscopic Approach */
"00NL0ZZ"  =  "1"  /* Release Abducens Nerve, Open Approach */
"00NL3ZZ"  =  "1"  /* Release Abducens Nerve, Percutaneous Approach */
"00NL4ZZ"  =  "1"  /* Release Abducens Nerve, Percutaneous Endoscopic Approach */
"00NM0ZZ"  =  "1"  /* Release Facial Nerve, Open Approach */
"00NM3ZZ"  =  "1"  /* Release Facial Nerve, Percutaneous Approach */
"00NM4ZZ"  =  "1"  /* Release Facial Nerve, Percutaneous Endoscopic Approach */
"00NN0ZZ"  =  "1"  /* Release Acoustic Nerve, Open Approach */
"00NN3ZZ"  =  "1"  /* Release Acoustic Nerve, Percutaneous Approach */
"00NN4ZZ"  =  "1"  /* Release Acoustic Nerve, Percutaneous Endoscopic Approach */
"00NP0ZZ"  =  "1"  /* Release Glossopharyngeal Nerve, Open Approach */
"00NP3ZZ"  =  "1"  /* Release Glossopharyngeal Nerve, Percutaneous Approach */
"00NP4ZZ"  =  "1"  /* Release Glossopharyngeal Nerve, Perc Endo Approach */
"00NQ0ZZ"  =  "1"  /* Release Vagus Nerve, Open Approach */
"00NQ3ZZ"  =  "1"  /* Release Vagus Nerve, Percutaneous Approach */
"00NQ4ZZ"  =  "1"  /* Release Vagus Nerve, Percutaneous Endoscopic Approach */
"00NR0ZZ"  =  "1"  /* Release Accessory Nerve, Open Approach */
"00NR3ZZ"  =  "1"  /* Release Accessory Nerve, Percutaneous Approach */
"00NR4ZZ"  =  "1"  /* Release Accessory Nerve, Percutaneous Endoscopic Approach */
"00NS0ZZ"  =  "1"  /* Release Hypoglossal Nerve, Open Approach */
"00NS3ZZ"  =  "1"  /* Release Hypoglossal Nerve, Percutaneous Approach */
"00NS4ZZ"  =  "1"  /* Release Hypoglossal Nerve, Percutaneous Endoscopic Approach */
"00NT0ZZ"  =  "1"  /* Release Spinal Meninges, Open Approach */
"00NT3ZZ"  =  "1"  /* Release Spinal Meninges, Percutaneous Approach */
"00NT4ZZ"  =  "1"  /* Release Spinal Meninges, Percutaneous Endoscopic Approach */
"00NW0ZZ"  =  "1"  /* Release Cervical Spinal Cord, Open Approach */
"00NW3ZZ"  =  "1"  /* Release Cervical Spinal Cord, Percutaneous Approach */
"00NW4ZZ"  =  "1"  /* Release Cervical Spinal Cord, Perc Endo Approach */
"00NX0ZZ"  =  "1"  /* Release Thoracic Spinal Cord, Open Approach */
"00NX3ZZ"  =  "1"  /* Release Thoracic Spinal Cord, Percutaneous Approach */
"00NX4ZZ"  =  "1"  /* Release Thoracic Spinal Cord, Perc Endo Approach */
"00NY0ZZ"  =  "1"  /* Release Lumbar Spinal Cord, Open Approach */
"00NY3ZZ"  =  "1"  /* Release Lumbar Spinal Cord, Percutaneous Approach */
"00NY4ZZ"  =  "1"  /* Release Lumbar Spinal Cord, Percutaneous Endoscopic Approach */
"00P000Z"  =  "1"  /* Removal of Drainage Device from Brain, Open Approach */
"00P002Z"  =  "1"  /* Removal of Monitoring Device from Brain, Open Approach */
"00P003Z"  =  "1"  /* Removal of Infusion Device from Brain, Open Approach */
"00P007Z"  =  "1"  /* Removal of Autol Sub from Brain, Open Approach */
"00P00JZ"  =  "1"  /* Removal of Synthetic Substitute from Brain, Open Approach */
"00P00KZ"  =  "1"  /* Removal of Nonaut Sub from Brain, Open Approach */
"00P00MZ"  =  "1"  /* Removal of Neurostimulator Lead from Brain, Open Approach */
"00P030Z"  =  "1"  /* Removal of Drainage Device from Brain, Percutaneous Approach */
"00P032Z"  =  "1"  /* Removal of Monitoring Device from Brain, Perc Approach */
"00P033Z"  =  "1"  /* Removal of Infusion Device from Brain, Percutaneous Approach */
"00P037Z"  =  "1"  /* Removal of Autol Sub from Brain, Perc Approach */
"00P03JZ"  =  "1"  /* Removal of Synthetic Substitute from Brain, Perc Approach */
"00P03KZ"  =  "1"  /* Removal of Nonaut Sub from Brain, Perc Approach */
"00P03MZ"  =  "1"  /* Removal of Neurostimulator Lead from Brain, Perc Approach */
"00P040Z"  =  "1"  /* Removal of Drainage Device from Brain, Perc Endo Approach */
"00P042Z"  =  "1"  /* Removal of Monitoring Device from Brain, Perc Endo Approach */
"00P043Z"  =  "1"  /* Removal of Infusion Device from Brain, Perc Endo Approach */
"00P047Z"  =  "1"  /* Removal of Autol Sub from Brain, Perc Endo Approach */
"00P04JZ"  =  "1"  /* Removal of Synth Sub from Brain, Perc Endo Approach */
"00P04KZ"  =  "1"  /* Removal of Nonaut Sub from Brain, Perc Endo Approach */
"00P04MZ"  =  "1"  /* Removal of Neuro Lead from Brain, Perc Endo Approach */
"00P600Z"  =  "1"  /* Removal of Drainage Device from Cereb Vent, Open Approach */
"00P602Z"  =  "1"  /* Removal of Monitoring Device from Cereb Vent, Open Approach */
"00P603Z"  =  "1"  /* Removal of Infusion Device from Cereb Vent, Open Approach */
"00P60JZ"  =  "1"  /* Removal of Synth Sub from Cereb Vent, Open Approach */
"00P60MZ"  =  "1"  /* Removal of Neuro Lead from Cereb Vent, Open Approach */
"00P630Z"  =  "1"  /* Removal of Drainage Device from Cereb Vent, Perc Approach */
"00P632Z"  =  "1"  /* Removal of Monitoring Device from Cereb Vent, Perc Approach */
"00P633Z"  =  "1"  /* Removal of Infusion Device from Cereb Vent, Perc Approach */
"00P63JZ"  =  "1"  /* Removal of Synth Sub from Cereb Vent, Perc Approach */
"00P63MZ"  =  "1"  /* Removal of Neuro Lead from Cereb Vent, Perc Approach */
"00P640Z"  =  "1"  /* Removal of Drain Dev from Cereb Vent, Perc Endo Approach */
"00P642Z"  =  "1"  /* Removal of Monitor Dev from Cereb Vent, Perc Endo Approach */
"00P643Z"  =  "1"  /* Removal of Infusion Dev from Cereb Vent, Perc Endo Approach */
"00P64JZ"  =  "1"  /* Removal of Synth Sub from Cereb Vent, Perc Endo Approach */
"00P64MZ"  =  "1"  /* Removal of Neuro Lead from Cereb Vent, Perc Endo Approach */
"00P6X2Z"  =  "1"  /* Removal of Monitor Dev from Cereb Vent, Extern Approach */
"00P6XMZ"  =  "1"  /* Removal of Neuro Lead from Cereb Vent, Extern Approach */
"00PE00Z"  =  "1"  /* Removal of Drainage Device from Cranial Nerve, Open Approach */
"00PE02Z"  =  "1"  /* Removal of Monitoring Device from Cranial Nrv, Open Approach */
"00PE03Z"  =  "1"  /* Removal of Infusion Device from Cranial Nerve, Open Approach */
"00PE07Z"  =  "1"  /* Removal of Autol Sub from Cranial Nrv, Open Approach */
"00PE0MZ"  =  "1"  /* Removal of Neuro Lead from Cranial Nrv, Open Approach */
"00PE30Z"  =  "1"  /* Removal of Drainage Device from Cranial Nerve, Perc Approach */
"00PE32Z"  =  "1"  /* Removal of Monitoring Device from Cranial Nrv, Perc Approach */
"00PE33Z"  =  "1"  /* Removal of Infusion Device from Cranial Nerve, Perc Approach */
"00PE37Z"  =  "1"  /* Removal of Autol Sub from Cranial Nrv, Perc Approach */
"00PE3MZ"  =  "1"  /* Removal of Neuro Lead from Cranial Nrv, Perc Approach */
"00PE40Z"  =  "1"  /* Removal of Drain Dev from Cranial Nrv, Perc Endo Approach */
"00PE42Z"  =  "1"  /* Removal of Monitor Dev from Cranial Nrv, Perc Endo Approach */
"00PE43Z"  =  "1"  /* Removal of Infusion Dev from Cranial Nrv, Perc Endo Approach */
"00PE47Z"  =  "1"  /* Removal of Autol Sub from Cranial Nrv, Perc Endo Approach */
"00PE4MZ"  =  "1"  /* Removal of Neuro Lead from Cranial Nrv, Perc Endo Approach */
"00PEXMZ"  =  "1"  /* Removal of Neuro Lead from Cranial Nrv, Extern Approach */
"00PU00Z"  =  "1"  /* Removal of Drainage Device from Spinal Canal, Open Approach */
"00PU02Z"  =  "1"  /* Removal of Monitor Dev from Spinal Canal, Open Approach */
"00PU03Z"  =  "1"  /* Removal of Infusion Device from Spinal Canal, Open Approach */
"00PU0JZ"  =  "1"  /* Removal of Synth Sub from Spinal Canal, Open Approach */
"00PU0MZ"  =  "1"  /* Removal of Neuro Lead from Spinal Canal, Open Approach */
"00PU30Z"  =  "1"  /* Removal of Drainage Device from Spinal Canal, Perc Approach */
"00PU32Z"  =  "1"  /* Removal of Monitor Dev from Spinal Canal, Perc Approach */
"00PU33Z"  =  "1"  /* Removal of Infusion Device from Spinal Canal, Perc Approach */
"00PU3JZ"  =  "1"  /* Removal of Synth Sub from Spinal Canal, Perc Approach */
"00PU3MZ"  =  "1"  /* Removal of Neuro Lead from Spinal Canal, Perc Approach */
"00PU40Z"  =  "1"  /* Removal of Drain Dev from Spinal Canal, Perc Endo Approach */
"00PU42Z"  =  "1"  /* Removal of Monitor Dev from Spinal Canal, Perc Endo Approach */
"00PU43Z"  =  "1"  /* Remove of Infusion Dev from Spinal Canal, Perc Endo Approach */
"00PU4JZ"  =  "1"  /* Removal of Synth Sub from Spinal Canal, Perc Endo Approach */
"00PU4MZ"  =  "1"  /* Removal of Neuro Lead from Spinal Canal, Perc Endo Approach */
"00PV00Z"  =  "1"  /* Removal of Drainage Device from Spinal Cord, Open Approach */
"00PV02Z"  =  "1"  /* Removal of Monitoring Device from Spinal Cord, Open Approach */
"00PV03Z"  =  "1"  /* Removal of Infusion Device from Spinal Cord, Open Approach */
"00PV07Z"  =  "1"  /* Removal of Autol Sub from Spinal Cord, Open Approach */
"00PV0JZ"  =  "1"  /* Removal of Synth Sub from Spinal Cord, Open Approach */
"00PV0KZ"  =  "1"  /* Removal of Nonaut Sub from Spinal Cord, Open Approach */
"00PV0MZ"  =  "1"  /* Removal of Neuro Lead from Spinal Cord, Open Approach */
"00PV30Z"  =  "1"  /* Removal of Drainage Device from Spinal Cord, Perc Approach */
"00PV32Z"  =  "1"  /* Removal of Monitoring Device from Spinal Cord, Perc Approach */
"00PV33Z"  =  "1"  /* Removal of Infusion Device from Spinal Cord, Perc Approach */
"00PV37Z"  =  "1"  /* Removal of Autol Sub from Spinal Cord, Perc Approach */
"00PV3JZ"  =  "1"  /* Removal of Synth Sub from Spinal Cord, Perc Approach */
"00PV3KZ"  =  "1"  /* Removal of Nonaut Sub from Spinal Cord, Perc Approach */
"00PV3MZ"  =  "1"  /* Removal of Neuro Lead from Spinal Cord, Perc Approach */
"00PV40Z"  =  "1"  /* Removal of Drain Dev from Spinal Cord, Perc Endo Approach */
"00PV42Z"  =  "1"  /* Removal of Monitor Dev from Spinal Cord, Perc Endo Approach */
"00PV43Z"  =  "1"  /* Removal of Infusion Dev from Spinal Cord, Perc Endo Approach */
"00PV47Z"  =  "1"  /* Removal of Autol Sub from Spinal Cord, Perc Endo Approach */
"00PV4JZ"  =  "1"  /* Removal of Synth Sub from Spinal Cord, Perc Endo Approach */
"00PV4KZ"  =  "1"  /* Removal of Nonaut Sub from Spinal Cord, Perc Endo Approach */
"00PV4MZ"  =  "1"  /* Removal of Neuro Lead from Spinal Cord, Perc Endo Approach */
"00Q00ZZ"  =  "1"  /* Repair Brain, Open Approach */
"00Q03ZZ"  =  "1"  /* Repair Brain, Percutaneous Approach */
"00Q04ZZ"  =  "1"  /* Repair Brain, Percutaneous Endoscopic Approach */
"00Q10ZZ"  =  "1"  /* Repair Cerebral Meninges, Open Approach */
"00Q13ZZ"  =  "1"  /* Repair Cerebral Meninges, Percutaneous Approach */
"00Q14ZZ"  =  "1"  /* Repair Cerebral Meninges, Percutaneous Endoscopic Approach */
"00Q20ZZ"  =  "1"  /* Repair Dura Mater, Open Approach */
"00Q23ZZ"  =  "1"  /* Repair Dura Mater, Percutaneous Approach */
"00Q24ZZ"  =  "1"  /* Repair Dura Mater, Percutaneous Endoscopic Approach */
"00Q60ZZ"  =  "1"  /* Repair Cerebral Ventricle, Open Approach */
"00Q63ZZ"  =  "1"  /* Repair Cerebral Ventricle, Percutaneous Approach */
"00Q64ZZ"  =  "1"  /* Repair Cerebral Ventricle, Percutaneous Endoscopic Approach */
"00Q70ZZ"  =  "1"  /* Repair Cerebral Hemisphere, Open Approach */
"00Q73ZZ"  =  "1"  /* Repair Cerebral Hemisphere, Percutaneous Approach */
"00Q74ZZ"  =  "1"  /* Repair Cerebral Hemisphere, Percutaneous Endoscopic Approach */
"00Q80ZZ"  =  "1"  /* Repair Basal Ganglia, Open Approach */
"00Q83ZZ"  =  "1"  /* Repair Basal Ganglia, Percutaneous Approach */
"00Q84ZZ"  =  "1"  /* Repair Basal Ganglia, Percutaneous Endoscopic Approach */
"00Q90ZZ"  =  "1"  /* Repair Thalamus, Open Approach */
"00Q93ZZ"  =  "1"  /* Repair Thalamus, Percutaneous Approach */
"00Q94ZZ"  =  "1"  /* Repair Thalamus, Percutaneous Endoscopic Approach */
"00QA0ZZ"  =  "1"  /* Repair Hypothalamus, Open Approach */
"00QA3ZZ"  =  "1"  /* Repair Hypothalamus, Percutaneous Approach */
"00QA4ZZ"  =  "1"  /* Repair Hypothalamus, Percutaneous Endoscopic Approach */
"00QB0ZZ"  =  "1"  /* Repair Pons, Open Approach */
"00QB3ZZ"  =  "1"  /* Repair Pons, Percutaneous Approach */
"00QB4ZZ"  =  "1"  /* Repair Pons, Percutaneous Endoscopic Approach */
"00QC0ZZ"  =  "1"  /* Repair Cerebellum, Open Approach */
"00QC3ZZ"  =  "1"  /* Repair Cerebellum, Percutaneous Approach */
"00QC4ZZ"  =  "1"  /* Repair Cerebellum, Percutaneous Endoscopic Approach */
"00QD0ZZ"  =  "1"  /* Repair Medulla Oblongata, Open Approach */
"00QD3ZZ"  =  "1"  /* Repair Medulla Oblongata, Percutaneous Approach */
"00QD4ZZ"  =  "1"  /* Repair Medulla Oblongata, Percutaneous Endoscopic Approach */
"00QF0ZZ"  =  "1"  /* Repair Olfactory Nerve, Open Approach */
"00QF3ZZ"  =  "1"  /* Repair Olfactory Nerve, Percutaneous Approach */
"00QF4ZZ"  =  "1"  /* Repair Olfactory Nerve, Percutaneous Endoscopic Approach */
"00QG0ZZ"  =  "1"  /* Repair Optic Nerve, Open Approach */
"00QG3ZZ"  =  "1"  /* Repair Optic Nerve, Percutaneous Approach */
"00QG4ZZ"  =  "1"  /* Repair Optic Nerve, Percutaneous Endoscopic Approach */
"00QH0ZZ"  =  "1"  /* Repair Oculomotor Nerve, Open Approach */
"00QH3ZZ"  =  "1"  /* Repair Oculomotor Nerve, Percutaneous Approach */
"00QH4ZZ"  =  "1"  /* Repair Oculomotor Nerve, Percutaneous Endoscopic Approach */
"00QJ0ZZ"  =  "1"  /* Repair Trochlear Nerve, Open Approach */
"00QJ3ZZ"  =  "1"  /* Repair Trochlear Nerve, Percutaneous Approach */
"00QJ4ZZ"  =  "1"  /* Repair Trochlear Nerve, Percutaneous Endoscopic Approach */
"00QK0ZZ"  =  "1"  /* Repair Trigeminal Nerve, Open Approach */
"00QK3ZZ"  =  "1"  /* Repair Trigeminal Nerve, Percutaneous Approach */
"00QK4ZZ"  =  "1"  /* Repair Trigeminal Nerve, Percutaneous Endoscopic Approach */
"00QL0ZZ"  =  "1"  /* Repair Abducens Nerve, Open Approach */
"00QL3ZZ"  =  "1"  /* Repair Abducens Nerve, Percutaneous Approach */
"00QL4ZZ"  =  "1"  /* Repair Abducens Nerve, Percutaneous Endoscopic Approach */
"00QM0ZZ"  =  "1"  /* Repair Facial Nerve, Open Approach */
"00QM3ZZ"  =  "1"  /* Repair Facial Nerve, Percutaneous Approach */
"00QM4ZZ"  =  "1"  /* Repair Facial Nerve, Percutaneous Endoscopic Approach */
"00QN0ZZ"  =  "1"  /* Repair Acoustic Nerve, Open Approach */
"00QN3ZZ"  =  "1"  /* Repair Acoustic Nerve, Percutaneous Approach */
"00QN4ZZ"  =  "1"  /* Repair Acoustic Nerve, Percutaneous Endoscopic Approach */
"00QP0ZZ"  =  "1"  /* Repair Glossopharyngeal Nerve, Open Approach */
"00QP3ZZ"  =  "1"  /* Repair Glossopharyngeal Nerve, Percutaneous Approach */
"00QP4ZZ"  =  "1"  /* Repair Glossopharyngeal Nerve, Perc Endo Approach */
"00QQ0ZZ"  =  "1"  /* Repair Vagus Nerve, Open Approach */
"00QQ3ZZ"  =  "1"  /* Repair Vagus Nerve, Percutaneous Approach */
"00QQ4ZZ"  =  "1"  /* Repair Vagus Nerve, Percutaneous Endoscopic Approach */
"00QR0ZZ"  =  "1"  /* Repair Accessory Nerve, Open Approach */
"00QR3ZZ"  =  "1"  /* Repair Accessory Nerve, Percutaneous Approach */
"00QR4ZZ"  =  "1"  /* Repair Accessory Nerve, Percutaneous Endoscopic Approach */
"00QS0ZZ"  =  "1"  /* Repair Hypoglossal Nerve, Open Approach */
"00QS3ZZ"  =  "1"  /* Repair Hypoglossal Nerve, Percutaneous Approach */
"00QS4ZZ"  =  "1"  /* Repair Hypoglossal Nerve, Percutaneous Endoscopic Approach */
"00QT0ZZ"  =  "1"  /* Repair Spinal Meninges, Open Approach */
"00QT3ZZ"  =  "1"  /* Repair Spinal Meninges, Percutaneous Approach */
"00QT4ZZ"  =  "1"  /* Repair Spinal Meninges, Percutaneous Endoscopic Approach */
"00QW0ZZ"  =  "1"  /* Repair Cervical Spinal Cord, Open Approach */
"00QW3ZZ"  =  "1"  /* Repair Cervical Spinal Cord, Percutaneous Approach */
"00QW4ZZ"  =  "1"  /* Repair Cervical Spinal Cord, Perc Endo Approach */
"00QX0ZZ"  =  "1"  /* Repair Thoracic Spinal Cord, Open Approach */
"00QX3ZZ"  =  "1"  /* Repair Thoracic Spinal Cord, Percutaneous Approach */
"00QX4ZZ"  =  "1"  /* Repair Thoracic Spinal Cord, Perc Endo Approach */
"00QY0ZZ"  =  "1"  /* Repair Lumbar Spinal Cord, Open Approach */
"00QY3ZZ"  =  "1"  /* Repair Lumbar Spinal Cord, Percutaneous Approach */
"00QY4ZZ"  =  "1"  /* Repair Lumbar Spinal Cord, Percutaneous Endoscopic Approach */
"00SF0ZZ"  =  "1"  /* Reposition Olfactory Nerve, Open Approach */
"00SF3ZZ"  =  "1"  /* Reposition Olfactory Nerve, Percutaneous Approach */
"00SF4ZZ"  =  "1"  /* Reposition Olfactory Nerve, Percutaneous Endoscopic Approach */
"00SG0ZZ"  =  "1"  /* Reposition Optic Nerve, Open Approach */
"00SG3ZZ"  =  "1"  /* Reposition Optic Nerve, Percutaneous Approach */
"00SG4ZZ"  =  "1"  /* Reposition Optic Nerve, Percutaneous Endoscopic Approach */
"00SH0ZZ"  =  "1"  /* Reposition Oculomotor Nerve, Open Approach */
"00SH3ZZ"  =  "1"  /* Reposition Oculomotor Nerve, Percutaneous Approach */
"00SH4ZZ"  =  "1"  /* Reposition Oculomotor Nerve, Perc Endo Approach */
"00SJ0ZZ"  =  "1"  /* Reposition Trochlear Nerve, Open Approach */
"00SJ3ZZ"  =  "1"  /* Reposition Trochlear Nerve, Percutaneous Approach */
"00SJ4ZZ"  =  "1"  /* Reposition Trochlear Nerve, Percutaneous Endoscopic Approach */
"00SK0ZZ"  =  "1"  /* Reposition Trigeminal Nerve, Open Approach */
"00SK3ZZ"  =  "1"  /* Reposition Trigeminal Nerve, Percutaneous Approach */
"00SK4ZZ"  =  "1"  /* Reposition Trigeminal Nerve, Perc Endo Approach */
"00SL0ZZ"  =  "1"  /* Reposition Abducens Nerve, Open Approach */
"00SL3ZZ"  =  "1"  /* Reposition Abducens Nerve, Percutaneous Approach */
"00SL4ZZ"  =  "1"  /* Reposition Abducens Nerve, Percutaneous Endoscopic Approach */
"00SM0ZZ"  =  "1"  /* Reposition Facial Nerve, Open Approach */
"00SM3ZZ"  =  "1"  /* Reposition Facial Nerve, Percutaneous Approach */
"00SM4ZZ"  =  "1"  /* Reposition Facial Nerve, Percutaneous Endoscopic Approach */
"00SN0ZZ"  =  "1"  /* Reposition Acoustic Nerve, Open Approach */
"00SN3ZZ"  =  "1"  /* Reposition Acoustic Nerve, Percutaneous Approach */
"00SN4ZZ"  =  "1"  /* Reposition Acoustic Nerve, Percutaneous Endoscopic Approach */
"00SP0ZZ"  =  "1"  /* Reposition Glossopharyngeal Nerve, Open Approach */
"00SP3ZZ"  =  "1"  /* Reposition Glossopharyngeal Nerve, Percutaneous Approach */
"00SP4ZZ"  =  "1"  /* Reposition Glossopharyngeal Nerve, Perc Endo Approach */
"00SQ0ZZ"  =  "1"  /* Reposition Vagus Nerve, Open Approach */
"00SQ3ZZ"  =  "1"  /* Reposition Vagus Nerve, Percutaneous Approach */
"00SQ4ZZ"  =  "1"  /* Reposition Vagus Nerve, Percutaneous Endoscopic Approach */
"00SR0ZZ"  =  "1"  /* Reposition Accessory Nerve, Open Approach */
"00SR3ZZ"  =  "1"  /* Reposition Accessory Nerve, Percutaneous Approach */
"00SR4ZZ"  =  "1"  /* Reposition Accessory Nerve, Percutaneous Endoscopic Approach */
"00SS0ZZ"  =  "1"  /* Reposition Hypoglossal Nerve, Open Approach */
"00SS3ZZ"  =  "1"  /* Reposition Hypoglossal Nerve, Percutaneous Approach */
"00SS4ZZ"  =  "1"  /* Reposition Hypoglossal Nerve, Perc Endo Approach */
"00SW0ZZ"  =  "1"  /* Reposition Cervical Spinal Cord, Open Approach */
"00SW3ZZ"  =  "1"  /* Reposition Cervical Spinal Cord, Percutaneous Approach */
"00SW4ZZ"  =  "1"  /* Reposition Cervical Spinal Cord, Perc Endo Approach */
"00SX0ZZ"  =  "1"  /* Reposition Thoracic Spinal Cord, Open Approach */
"00SX3ZZ"  =  "1"  /* Reposition Thoracic Spinal Cord, Percutaneous Approach */
"00SX4ZZ"  =  "1"  /* Reposition Thoracic Spinal Cord, Perc Endo Approach */
"00SY0ZZ"  =  "1"  /* Reposition Lumbar Spinal Cord, Open Approach */
"00SY3ZZ"  =  "1"  /* Reposition Lumbar Spinal Cord, Percutaneous Approach */
"00SY4ZZ"  =  "1"  /* Reposition Lumbar Spinal Cord, Perc Endo Approach */
"00T70ZZ"  =  "1"  /* Resection of Cerebral Hemisphere, Open Approach */
"00T73ZZ"  =  "1"  /* Resection of Cerebral Hemisphere, Percutaneous Approach */
"00T74ZZ"  =  "1"  /* Resection of Cerebral Hemisphere, Perc Endo Approach */
"00U107Z"  =  "1"  /* Supplement Cerebral Meninges with Autol Sub, Open Approach */
"00U10JZ"  =  "1"  /* Supplement Cerebral Meninges with Synth Sub, Open Approach */
"00U10KZ"  =  "1"  /* Supplement Cerebral Meninges with Nonaut Sub, Open Approach */
"00U137Z"  =  "1"  /* Supplement Cerebral Meninges with Autol Sub, Perc Approach */
"00U13JZ"  =  "1"  /* Supplement Cerebral Meninges with Synth Sub, Perc Approach */
"00U13KZ"  =  "1"  /* Supplement Cerebral Meninges with Nonaut Sub, Perc Approach */
"00U147Z"  =  "1"  /* Supplement Cereb Meninges with Autol Sub, Perc Endo Approach */
"00U14JZ"  =  "1"  /* Supplement Cereb Meninges with Synth Sub, Perc Endo Approach */
"00U14KZ"  =  "1"  /* Supplement Cereb Meninges w Nonaut Sub, Perc Endo */
"00U207Z"  =  "1"  /* Supplement Dura Mater with Autol Sub, Open Approach */
"00U20JZ"  =  "1"  /* Supplement Dura Mater with Synth Sub, Open Approach */
"00U20KZ"  =  "1"  /* Supplement Dura Mater with Nonaut Sub, Open Approach */
"00U237Z"  =  "1"  /* Supplement Dura Mater with Autol Sub, Perc Approach */
"00U23JZ"  =  "1"  /* Supplement Dura Mater with Synth Sub, Perc Approach */
"00U23KZ"  =  "1"  /* Supplement Dura Mater with Nonaut Sub, Perc Approach */
"00U247Z"  =  "1"  /* Supplement Dura Mater with Autol Sub, Perc Endo Approach */
"00U24JZ"  =  "1"  /* Supplement Dura Mater with Synth Sub, Perc Endo Approach */
"00U24KZ"  =  "1"  /* Supplement Dura Mater with Nonaut Sub, Perc Endo Approach */
"00UF07Z"  =  "1"  /* Supplement Olfactory Nerve with Autol Sub, Open Approach */
"00UF37Z"  =  "1"  /* Supplement Olfactory Nerve with Autol Sub, Perc Approach */
"00UF47Z"  =  "1"  /* Supplement Olfact Nrv with Autol Sub, Perc Endo Approach */
"00UG07Z"  =  "1"  /* Supplement Optic Nerve with Autol Sub, Open Approach */
"00UG37Z"  =  "1"  /* Supplement Optic Nerve with Autol Sub, Perc Approach */
"00UG47Z"  =  "1"  /* Supplement Optic Nerve with Autol Sub, Perc Endo Approach */
"00UH07Z"  =  "1"  /* Supplement Oculomotor Nerve with Autol Sub, Open Approach */
"00UH37Z"  =  "1"  /* Supplement Oculomotor Nerve with Autol Sub, Perc Approach */
"00UH47Z"  =  "1"  /* Supplement Oculomotor Nrv with Autol Sub, Perc Endo Approach */
"00UJ07Z"  =  "1"  /* Supplement Trochlear Nerve with Autol Sub, Open Approach */
"00UJ37Z"  =  "1"  /* Supplement Trochlear Nerve with Autol Sub, Perc Approach */
"00UJ47Z"  =  "1"  /* Supplement Trochlear Nrv with Autol Sub, Perc Endo Approach */
"00UK07Z"  =  "1"  /* Supplement Trigeminal Nerve with Autol Sub, Open Approach */
"00UK37Z"  =  "1"  /* Supplement Trigeminal Nerve with Autol Sub, Perc Approach */
"00UK47Z"  =  "1"  /* Supplement Trigem Nrv with Autol Sub, Perc Endo Approach */
"00UL07Z"  =  "1"  /* Supplement Abducens Nerve with Autol Sub, Open Approach */
"00UL37Z"  =  "1"  /* Supplement Abducens Nerve with Autol Sub, Perc Approach */
"00UL47Z"  =  "1"  /* Supplement Abducens Nerve with Autol Sub, Perc Endo Approach */
"00UM07Z"  =  "1"  /* Supplement Facial Nerve with Autol Sub, Open Approach */
"00UM37Z"  =  "1"  /* Supplement Facial Nerve with Autol Sub, Perc Approach */
"00UM47Z"  =  "1"  /* Supplement Facial Nerve with Autol Sub, Perc Endo Approach */
"00UN07Z"  =  "1"  /* Supplement Acoustic Nerve with Autol Sub, Open Approach */
"00UN37Z"  =  "1"  /* Supplement Acoustic Nerve with Autol Sub, Perc Approach */
"00UN47Z"  =  "1"  /* Supplement Acoustic Nerve with Autol Sub, Perc Endo Approach */
"00UP07Z"  =  "1"  /* Supplement Glossophar Nrv with Autol Sub, Open Approach */
"00UP37Z"  =  "1"  /* Supplement Glossophar Nrv with Autol Sub, Perc Approach */
"00UP47Z"  =  "1"  /* Supplement Glossophar Nrv with Autol Sub, Perc Endo Approach */
"00UQ07Z"  =  "1"  /* Supplement Vagus Nerve with Autol Sub, Open Approach */
"00UQ37Z"  =  "1"  /* Supplement Vagus Nerve with Autol Sub, Perc Approach */
"00UQ47Z"  =  "1"  /* Supplement Vagus Nerve with Autol Sub, Perc Endo Approach */
"00UR07Z"  =  "1"  /* Supplement Accessory Nerve with Autol Sub, Open Approach */
"00UR37Z"  =  "1"  /* Supplement Accessory Nerve with Autol Sub, Perc Approach */
"00UR47Z"  =  "1"  /* Supplement Accessory Nrv with Autol Sub, Perc Endo Approach */
"00US07Z"  =  "1"  /* Supplement Hypoglossal Nerve with Autol Sub, Open Approach */
"00US37Z"  =  "1"  /* Supplement Hypoglossal Nerve with Autol Sub, Perc Approach */
"00US47Z"  =  "1"  /* Supplement Hypogloss Nrv with Autol Sub, Perc Endo Approach */
"00UT07Z"  =  "1"  /* Supplement Spinal Meninges with Autol Sub, Open Approach */
"00UT0JZ"  =  "1"  /* Supplement Spinal Meninges with Synth Sub, Open Approach */
"00UT0KZ"  =  "1"  /* Supplement Spinal Meninges with Nonaut Sub, Open Approach */
"00UT37Z"  =  "1"  /* Supplement Spinal Meninges with Autol Sub, Perc Approach */
"00UT3JZ"  =  "1"  /* Supplement Spinal Meninges with Synth Sub, Perc Approach */
"00UT3KZ"  =  "1"  /* Supplement Spinal Meninges with Nonaut Sub, Perc Approach */
"00UT47Z"  =  "1"  /* Supplement Spinal Meninges w Autol Sub, Perc Endo */
"00UT4JZ"  =  "1"  /* Supplement Spinal Meninges w Synth Sub, Perc Endo */
"00UT4KZ"  =  "1"  /* Supplement Spinal Meninges w Nonaut Sub, Perc Endo */
"00W000Z"  =  "1"  /* Revision of Drainage Device in Brain, Open Approach */
"00W002Z"  =  "1"  /* Revision of Monitoring Device in Brain, Open Approach */
"00W003Z"  =  "1"  /* Revision of Infusion Device in Brain, Open Approach */
"00W007Z"  =  "1"  /* Revision of Autol Sub in Brain, Open Approach */
"00W00JZ"  =  "1"  /* Revision of Synthetic Substitute in Brain, Open Approach */
"00W00KZ"  =  "1"  /* Revision of Nonaut Sub in Brain, Open Approach */
"00W00MZ"  =  "1"  /* Revision of Neurostimulator Lead in Brain, Open Approach */
"00W030Z"  =  "1"  /* Revision of Drainage Device in Brain, Percutaneous Approach */
"00W032Z"  =  "1"  /* Revision of Monitoring Device in Brain, Perc Approach */
"00W033Z"  =  "1"  /* Revision of Infusion Device in Brain, Percutaneous Approach */
"00W037Z"  =  "1"  /* Revision of Autol Sub in Brain, Perc Approach */
"00W03JZ"  =  "1"  /* Revision of Synthetic Substitute in Brain, Perc Approach */
"00W03KZ"  =  "1"  /* Revision of Nonaut Sub in Brain, Perc Approach */
"00W03MZ"  =  "1"  /* Revision of Neurostimulator Lead in Brain, Perc Approach */
"00W040Z"  =  "1"  /* Revision of Drainage Device in Brain, Perc Endo Approach */
"00W042Z"  =  "1"  /* Revision of Monitoring Device in Brain, Perc Endo Approach */
"00W043Z"  =  "1"  /* Revision of Infusion Device in Brain, Perc Endo Approach */
"00W047Z"  =  "1"  /* Revision of Autol Sub in Brain, Perc Endo Approach */
"00W04JZ"  =  "1"  /* Revision of Synth Sub in Brain, Perc Endo Approach */
"00W04KZ"  =  "1"  /* Revision of Nonaut Sub in Brain, Perc Endo Approach */
"00W04MZ"  =  "1"  /* Revision of Neuro Lead in Brain, Perc Endo Approach */
"00W600Z"  =  "1"  /* Revision of Drainage Device in Cereb Vent, Open Approach */
"00W602Z"  =  "1"  /* Revision of Monitoring Device in Cereb Vent, Open Approach */
"00W603Z"  =  "1"  /* Revision of Infusion Device in Cereb Vent, Open Approach */
"00W60JZ"  =  "1"  /* Revision of Synth Sub in Cereb Vent, Open Approach */
"00W60MZ"  =  "1"  /* Revision of Neuro Lead in Cereb Vent, Open Approach */
"00W630Z"  =  "1"  /* Revision of Drainage Device in Cereb Vent, Perc Approach */
"00W632Z"  =  "1"  /* Revision of Monitoring Device in Cereb Vent, Perc Approach */
"00W633Z"  =  "1"  /* Revision of Infusion Device in Cereb Vent, Perc Approach */
"00W63JZ"  =  "1"  /* Revision of Synth Sub in Cereb Vent, Perc Approach */
"00W63MZ"  =  "1"  /* Revision of Neuro Lead in Cereb Vent, Perc Approach */
"00W640Z"  =  "1"  /* Revision of Drain Dev in Cereb Vent, Perc Endo Approach */
"00W642Z"  =  "1"  /* Revision of Monitor Dev in Cereb Vent, Perc Endo Approach */
"00W643Z"  =  "1"  /* Revision of Infusion Dev in Cereb Vent, Perc Endo Approach */
"00W64JZ"  =  "1"  /* Revision of Synth Sub in Cereb Vent, Perc Endo Approach */
"00W64MZ"  =  "1"  /* Revision of Neuro Lead in Cereb Vent, Perc Endo Approach */
"00WE00Z"  =  "1"  /* Revision of Drainage Device in Cranial Nerve, Open Approach */
"00WE02Z"  =  "1"  /* Revision of Monitoring Device in Cranial Nrv, Open Approach */
"00WE03Z"  =  "1"  /* Revision of Infusion Device in Cranial Nerve, Open Approach */
"00WE07Z"  =  "1"  /* Revision of Autol Sub in Cranial Nrv, Open Approach */
"00WE0MZ"  =  "1"  /* Revision of Neuro Lead in Cranial Nrv, Open Approach */
"00WE30Z"  =  "1"  /* Revision of Drainage Device in Cranial Nerve, Perc Approach */
"00WE32Z"  =  "1"  /* Revision of Monitoring Device in Cranial Nrv, Perc Approach */
"00WE33Z"  =  "1"  /* Revision of Infusion Device in Cranial Nerve, Perc Approach */
"00WE37Z"  =  "1"  /* Revision of Autol Sub in Cranial Nrv, Perc Approach */
"00WE3MZ"  =  "1"  /* Revision of Neuro Lead in Cranial Nrv, Perc Approach */
"00WE40Z"  =  "1"  /* Revision of Drain Dev in Cranial Nrv, Perc Endo Approach */
"00WE42Z"  =  "1"  /* Revision of Monitor Dev in Cranial Nrv, Perc Endo Approach */
"00WE43Z"  =  "1"  /* Revision of Infusion Dev in Cranial Nrv, Perc Endo Approach */
"00WE47Z"  =  "1"  /* Revision of Autol Sub in Cranial Nrv, Perc Endo Approach */
"00WE4MZ"  =  "1"  /* Revision of Neuro Lead in Cranial Nrv, Perc Endo Approach */
"00WU00Z"  =  "1"  /* Revision of Drainage Device in Spinal Canal, Open Approach */
"00WU02Z"  =  "1"  /* Revision of Monitoring Device in Spinal Canal, Open Approach */
"00WU03Z"  =  "1"  /* Revision of Infusion Device in Spinal Canal, Open Approach */
"00WU0JZ"  =  "1"  /* Revision of Synth Sub in Spinal Canal, Open Approach */
"00WU0MZ"  =  "1"  /* Revision of Neuro Lead in Spinal Canal, Open Approach */
"00WU30Z"  =  "1"  /* Revision of Drainage Device in Spinal Canal, Perc Approach */
"00WU32Z"  =  "1"  /* Revision of Monitoring Device in Spinal Canal, Perc Approach */
"00WU33Z"  =  "1"  /* Revision of Infusion Device in Spinal Canal, Perc Approach */
"00WU3JZ"  =  "1"  /* Revision of Synth Sub in Spinal Canal, Perc Approach */
"00WU3MZ"  =  "1"  /* Revision of Neuro Lead in Spinal Canal, Perc Approach */
"00WU40Z"  =  "1"  /* Revision of Drain Dev in Spinal Canal, Perc Endo Approach */
"00WU42Z"  =  "1"  /* Revision of Monitor Dev in Spinal Canal, Perc Endo Approach */
"00WU43Z"  =  "1"  /* Revision of Infusion Dev in Spinal Canal, Perc Endo Approach */
"00WU4JZ"  =  "1"  /* Revision of Synth Sub in Spinal Canal, Perc Endo Approach */
"00WU4MZ"  =  "1"  /* Revision of Neuro Lead in Spinal Canal, Perc Endo Approach */
"00WV00Z"  =  "1"  /* Revision of Drainage Device in Spinal Cord, Open Approach */
"00WV02Z"  =  "1"  /* Revision of Monitoring Device in Spinal Cord, Open Approach */
"00WV03Z"  =  "1"  /* Revision of Infusion Device in Spinal Cord, Open Approach */
"00WV07Z"  =  "1"  /* Revision of Autol Sub in Spinal Cord, Open Approach */
"00WV0JZ"  =  "1"  /* Revision of Synth Sub in Spinal Cord, Open Approach */
"00WV0KZ"  =  "1"  /* Revision of Nonaut Sub in Spinal Cord, Open Approach */
"00WV0MZ"  =  "1"  /* Revision of Neuro Lead in Spinal Cord, Open Approach */
"00WV30Z"  =  "1"  /* Revision of Drainage Device in Spinal Cord, Perc Approach */
"00WV32Z"  =  "1"  /* Revision of Monitoring Device in Spinal Cord, Perc Approach */
"00WV33Z"  =  "1"  /* Revision of Infusion Device in Spinal Cord, Perc Approach */
"00WV37Z"  =  "1"  /* Revision of Autol Sub in Spinal Cord, Perc Approach */
"00WV3JZ"  =  "1"  /* Revision of Synth Sub in Spinal Cord, Perc Approach */
"00WV3KZ"  =  "1"  /* Revision of Nonaut Sub in Spinal Cord, Perc Approach */
"00WV3MZ"  =  "1"  /* Revision of Neuro Lead in Spinal Cord, Perc Approach */
"00WV40Z"  =  "1"  /* Revision of Drain Dev in Spinal Cord, Perc Endo Approach */
"00WV42Z"  =  "1"  /* Revision of Monitor Dev in Spinal Cord, Perc Endo Approach */
"00WV43Z"  =  "1"  /* Revision of Infusion Dev in Spinal Cord, Perc Endo Approach */
"00WV47Z"  =  "1"  /* Revision of Autol Sub in Spinal Cord, Perc Endo Approach */
"00WV4JZ"  =  "1"  /* Revision of Synth Sub in Spinal Cord, Perc Endo Approach */
"00WV4KZ"  =  "1"  /* Revision of Nonaut Sub in Spinal Cord, Perc Endo Approach */
"00WV4MZ"  =  "1"  /* Revision of Neuro Lead in Spinal Cord, Perc Endo Approach */
"00XF0ZF"  =  "1"  /* Transfer Olfactory Nerve to Olfactory Nerve, Open Approach */
"00XF0ZG"  =  "1"  /* Transfer Olfactory Nerve to Optic Nerve, Open Approach */
"00XF0ZH"  =  "1"  /* Transfer Olfactory Nerve to Oculomotor Nerve, Open Approach */
"00XF0ZJ"  =  "1"  /* Transfer Olfactory Nerve to Trochlear Nerve, Open Approach */
"00XF0ZK"  =  "1"  /* Transfer Olfactory Nerve to Trigeminal Nerve, Open Approach */
"00XF0ZL"  =  "1"  /* Transfer Olfactory Nerve to Abducens Nerve, Open Approach */
"00XF0ZM"  =  "1"  /* Transfer Olfactory Nerve to Facial Nerve, Open Approach */
"00XF0ZN"  =  "1"  /* Transfer Olfactory Nerve to Acoustic Nerve, Open Approach */
"00XF0ZP"  =  "1"  /* Transfer Olfactory Nerve to Glossophar Nrv, Open Approach */
"00XF0ZQ"  =  "1"  /* Transfer Olfactory Nerve to Vagus Nerve, Open Approach */
"00XF0ZR"  =  "1"  /* Transfer Olfactory Nerve to Accessory Nerve, Open Approach */
"00XF0ZS"  =  "1"  /* Transfer Olfactory Nerve to Hypoglossal Nerve, Open Approach */
"00XF4ZF"  =  "1"  /* Transfer Olfactory Nerve to Olfact Nrv, Perc Endo Approach */
"00XF4ZG"  =  "1"  /* Transfer Olfactory Nerve to Optic Nerve, Perc Endo Approach */
"00XF4ZH"  =  "1"  /* Transfer Olfactory Nerve to Oculomotor N, Perc Endo Approach */
"00XF4ZJ"  =  "1"  /* Transfer Olfact Nrv to Trochlear Nrv, Perc Endo Approach */
"00XF4ZK"  =  "1"  /* Transfer Olfactory Nerve to Trigem Nrv, Perc Endo Approach */
"00XF4ZL"  =  "1"  /* Transfer Olfactory Nerve to Abducen Nrv, Perc Endo Approach */
"00XF4ZM"  =  "1"  /* Transfer Olfactory Nerve to Facial Nerve, Perc Endo Approach */
"00XF4ZN"  =  "1"  /* Transfer Olfactory Nerve to Acoust Nrv, Perc Endo Approach */
"00XF4ZP"  =  "1"  /* Transfer Olfact Nrv to Glossophar Nrv, Perc Endo Approach */
"00XF4ZQ"  =  "1"  /* Transfer Olfactory Nerve to Vagus Nerve, Perc Endo Approach */
"00XF4ZR"  =  "1"  /* Transfer Olfact Nrv to Accessory Nrv, Perc Endo Approach */
"00XF4ZS"  =  "1"  /* Transfer Olfact Nrv to Hypogloss Nrv, Perc Endo Approach */
"00XG0ZF"  =  "1"  /* Transfer Optic Nerve to Olfactory Nerve, Open Approach */
"00XG0ZG"  =  "1"  /* Transfer Optic Nerve to Optic Nerve, Open Approach */
"00XG0ZH"  =  "1"  /* Transfer Optic Nerve to Oculomotor Nerve, Open Approach */
"00XG0ZJ"  =  "1"  /* Transfer Optic Nerve to Trochlear Nerve, Open Approach */
"00XG0ZK"  =  "1"  /* Transfer Optic Nerve to Trigeminal Nerve, Open Approach */
"00XG0ZL"  =  "1"  /* Transfer Optic Nerve to Abducens Nerve, Open Approach */
"00XG0ZM"  =  "1"  /* Transfer Optic Nerve to Facial Nerve, Open Approach */
"00XG0ZN"  =  "1"  /* Transfer Optic Nerve to Acoustic Nerve, Open Approach */
"00XG0ZP"  =  "1"  /* Transfer Optic Nerve to Glossophar Nrv, Open Approach */
"00XG0ZQ"  =  "1"  /* Transfer Optic Nerve to Vagus Nerve, Open Approach */
"00XG0ZR"  =  "1"  /* Transfer Optic Nerve to Accessory Nerve, Open Approach */
"00XG0ZS"  =  "1"  /* Transfer Optic Nerve to Hypoglossal Nerve, Open Approach */
"00XG4ZF"  =  "1"  /* Transfer Optic Nerve to Olfactory Nerve, Perc Endo Approach */
"00XG4ZG"  =  "1"  /* Transfer Optic Nerve to Optic Nerve, Perc Endo Approach */
"00XG4ZH"  =  "1"  /* Transfer Optic Nerve to Oculomotor Nerve, Perc Endo Approach */
"00XG4ZJ"  =  "1"  /* Transfer Optic Nerve to Trochlear Nerve, Perc Endo Approach */
"00XG4ZK"  =  "1"  /* Transfer Optic Nerve to Trigeminal Nerve, Perc Endo Approach */
"00XG4ZL"  =  "1"  /* Transfer Optic Nerve to Abducens Nerve, Perc Endo Approach */
"00XG4ZM"  =  "1"  /* Transfer Optic Nerve to Facial Nerve, Perc Endo Approach */
"00XG4ZN"  =  "1"  /* Transfer Optic Nerve to Acoustic Nerve, Perc Endo Approach */
"00XG4ZP"  =  "1"  /* Transfer Optic Nerve to Glossophar Nrv, Perc Endo Approach */
"00XG4ZQ"  =  "1"  /* Transfer Optic Nerve to Vagus Nerve, Perc Endo Approach */
"00XG4ZR"  =  "1"  /* Transfer Optic Nerve to Accessory Nerve, Perc Endo Approach */
"00XG4ZS"  =  "1"  /* Transfer Optic Nerve to Hypogloss Nrv, Perc Endo Approach */
"00XH0ZF"  =  "1"  /* Transfer Oculomotor Nerve to Olfactory Nerve, Open Approach */
"00XH0ZG"  =  "1"  /* Transfer Oculomotor Nerve to Optic Nerve, Open Approach */
"00XH0ZH"  =  "1"  /* Transfer Oculomotor Nerve to Oculomotor Nerve, Open Approach */
"00XH0ZJ"  =  "1"  /* Transfer Oculomotor Nerve to Trochlear Nerve, Open Approach */
"00XH0ZK"  =  "1"  /* Transfer Oculomotor Nerve to Trigeminal Nerve, Open Approach */
"00XH0ZL"  =  "1"  /* Transfer Oculomotor Nerve to Abducens Nerve, Open Approach */
"00XH0ZM"  =  "1"  /* Transfer Oculomotor Nerve to Facial Nerve, Open Approach */
"00XH0ZN"  =  "1"  /* Transfer Oculomotor Nerve to Acoustic Nerve, Open Approach */
"00XH0ZP"  =  "1"  /* Transfer Oculomotor Nerve to Glossophar Nrv, Open Approach */
"00XH0ZQ"  =  "1"  /* Transfer Oculomotor Nerve to Vagus Nerve, Open Approach */
"00XH0ZR"  =  "1"  /* Transfer Oculomotor Nerve to Accessory Nerve, Open Approach */
"00XH0ZS"  =  "1"  /* Transfer Oculomotor Nerve to Hypogloss Nrv, Open Approach */
"00XH4ZF"  =  "1"  /* Transfer Oculomotor Nerve to Olfact Nrv, Perc Endo Approach */
"00XH4ZG"  =  "1"  /* Transfer Oculomotor Nerve to Optic Nerve, Perc Endo Approach */
"00XH4ZH"  =  "1"  /* Transfer Oculomotor Nrv to Oculomotor N, Perc Endo Approach */
"00XH4ZJ"  =  "1"  /* Transfer Oculomotor Nrv to Trochlear Nrv, Perc Endo Approach */
"00XH4ZK"  =  "1"  /* Transfer Oculomotor Nerve to Trigem Nrv, Perc Endo Approach */
"00XH4ZL"  =  "1"  /* Transfer Oculomotor Nerve to Abducen Nrv, Perc Endo Approach */
"00XH4ZM"  =  "1"  /* Transfer Oculomotor Nerve to Facial Nrv, Perc Endo Approach */
"00XH4ZN"  =  "1"  /* Transfer Oculomotor Nerve to Acoust Nrv, Perc Endo Approach */
"00XH4ZP"  =  "1"  /* Transfer Oculomotor Nrv to Glossophar Nrv, Perc Endo */
"00XH4ZQ"  =  "1"  /* Transfer Oculomotor Nerve to Vagus Nerve, Perc Endo Approach */
"00XH4ZR"  =  "1"  /* Transfer Oculomotor Nrv to Accessory Nrv, Perc Endo Approach */
"00XH4ZS"  =  "1"  /* Transfer Oculomotor Nrv to Hypogloss Nrv, Perc Endo Approach */
"00XJ0ZF"  =  "1"  /* Transfer Trochlear Nerve to Olfactory Nerve, Open Approach */
"00XJ0ZG"  =  "1"  /* Transfer Trochlear Nerve to Optic Nerve, Open Approach */
"00XJ0ZH"  =  "1"  /* Transfer Trochlear Nerve to Oculomotor Nerve, Open Approach */
"00XJ0ZJ"  =  "1"  /* Transfer Trochlear Nerve to Trochlear Nerve, Open Approach */
"00XJ0ZK"  =  "1"  /* Transfer Trochlear Nerve to Trigeminal Nerve, Open Approach */
"00XJ0ZL"  =  "1"  /* Transfer Trochlear Nerve to Abducens Nerve, Open Approach */
"00XJ0ZM"  =  "1"  /* Transfer Trochlear Nerve to Facial Nerve, Open Approach */
"00XJ0ZN"  =  "1"  /* Transfer Trochlear Nerve to Acoustic Nerve, Open Approach */
"00XJ0ZP"  =  "1"  /* Transfer Trochlear Nerve to Glossophar Nrv, Open Approach */
"00XJ0ZQ"  =  "1"  /* Transfer Trochlear Nerve to Vagus Nerve, Open Approach */
"00XJ0ZR"  =  "1"  /* Transfer Trochlear Nerve to Accessory Nerve, Open Approach */
"00XJ0ZS"  =  "1"  /* Transfer Trochlear Nerve to Hypoglossal Nerve, Open Approach */
"00XJ4ZF"  =  "1"  /* Transfer Trochlear Nerve to Olfact Nrv, Perc Endo Approach */
"00XJ4ZG"  =  "1"  /* Transfer Trochlear Nerve to Optic Nerve, Perc Endo Approach */
"00XJ4ZH"  =  "1"  /* Transfer Trochlear Nerve to Oculomotor N, Perc Endo Approach */
"00XJ4ZJ"  =  "1"  /* Transfer Trochlear Nrv to Trochlear Nrv, Perc Endo Approach */
"00XJ4ZK"  =  "1"  /* Transfer Trochlear Nerve to Trigem Nrv, Perc Endo Approach */
"00XJ4ZL"  =  "1"  /* Transfer Trochlear Nerve to Abducen Nrv, Perc Endo Approach */
"00XJ4ZM"  =  "1"  /* Transfer Trochlear Nerve to Facial Nerve, Perc Endo Approach */
"00XJ4ZN"  =  "1"  /* Transfer Trochlear Nerve to Acoust Nrv, Perc Endo Approach */
"00XJ4ZP"  =  "1"  /* Transfer Trochlear Nrv to Glossophar Nrv, Perc Endo Approach */
"00XJ4ZQ"  =  "1"  /* Transfer Trochlear Nerve to Vagus Nerve, Perc Endo Approach */
"00XJ4ZR"  =  "1"  /* Transfer Trochlear Nrv to Accessory Nrv, Perc Endo Approach */
"00XJ4ZS"  =  "1"  /* Transfer Trochlear Nrv to Hypogloss Nrv, Perc Endo Approach */
"00XK0ZF"  =  "1"  /* Transfer Trigeminal Nerve to Olfactory Nerve, Open Approach */
"00XK0ZG"  =  "1"  /* Transfer Trigeminal Nerve to Optic Nerve, Open Approach */
"00XK0ZH"  =  "1"  /* Transfer Trigeminal Nerve to Oculomotor Nerve, Open Approach */
"00XK0ZJ"  =  "1"  /* Transfer Trigeminal Nerve to Trochlear Nerve, Open Approach */
"00XK0ZK"  =  "1"  /* Transfer Trigeminal Nerve to Trigeminal Nerve, Open Approach */
"00XK0ZL"  =  "1"  /* Transfer Trigeminal Nerve to Abducens Nerve, Open Approach */
"00XK0ZM"  =  "1"  /* Transfer Trigeminal Nerve to Facial Nerve, Open Approach */
"00XK0ZN"  =  "1"  /* Transfer Trigeminal Nerve to Acoustic Nerve, Open Approach */
"00XK0ZP"  =  "1"  /* Transfer Trigeminal Nerve to Glossophar Nrv, Open Approach */
"00XK0ZQ"  =  "1"  /* Transfer Trigeminal Nerve to Vagus Nerve, Open Approach */
"00XK0ZR"  =  "1"  /* Transfer Trigeminal Nerve to Accessory Nerve, Open Approach */
"00XK0ZS"  =  "1"  /* Transfer Trigeminal Nerve to Hypogloss Nrv, Open Approach */
"00XK4ZF"  =  "1"  /* Transfer Trigeminal Nerve to Olfact Nrv, Perc Endo Approach */
"00XK4ZG"  =  "1"  /* Transfer Trigeminal Nerve to Optic Nerve, Perc Endo Approach */
"00XK4ZH"  =  "1"  /* Transfer Trigem Nrv to Oculomotor N, Perc Endo Approach */
"00XK4ZJ"  =  "1"  /* Transfer Trigem Nrv to Trochlear Nrv, Perc Endo Approach */
"00XK4ZK"  =  "1"  /* Transfer Trigeminal Nerve to Trigem Nrv, Perc Endo Approach */
"00XK4ZL"  =  "1"  /* Transfer Trigeminal Nerve to Abducen Nrv, Perc Endo Approach */
"00XK4ZM"  =  "1"  /* Transfer Trigeminal Nerve to Facial Nrv, Perc Endo Approach */
"00XK4ZN"  =  "1"  /* Transfer Trigeminal Nerve to Acoust Nrv, Perc Endo Approach */
"00XK4ZP"  =  "1"  /* Transfer Trigem Nrv to Glossophar Nrv, Perc Endo Approach */
"00XK4ZQ"  =  "1"  /* Transfer Trigeminal Nerve to Vagus Nerve, Perc Endo Approach */
"00XK4ZR"  =  "1"  /* Transfer Trigem Nrv to Accessory Nrv, Perc Endo Approach */
"00XK4ZS"  =  "1"  /* Transfer Trigem Nrv to Hypogloss Nrv, Perc Endo Approach */
"00XL0ZF"  =  "1"  /* Transfer Abducens Nerve to Olfactory Nerve, Open Approach */
"00XL0ZG"  =  "1"  /* Transfer Abducens Nerve to Optic Nerve, Open Approach */
"00XL0ZH"  =  "1"  /* Transfer Abducens Nerve to Oculomotor Nerve, Open Approach */
"00XL0ZJ"  =  "1"  /* Transfer Abducens Nerve to Trochlear Nerve, Open Approach */
"00XL0ZK"  =  "1"  /* Transfer Abducens Nerve to Trigeminal Nerve, Open Approach */
"00XL0ZL"  =  "1"  /* Transfer Abducens Nerve to Abducens Nerve, Open Approach */
"00XL0ZM"  =  "1"  /* Transfer Abducens Nerve to Facial Nerve, Open Approach */
"00XL0ZN"  =  "1"  /* Transfer Abducens Nerve to Acoustic Nerve, Open Approach */
"00XL0ZP"  =  "1"  /* Transfer Abducens Nerve to Glossophar Nrv, Open Approach */
"00XL0ZQ"  =  "1"  /* Transfer Abducens Nerve to Vagus Nerve, Open Approach */
"00XL0ZR"  =  "1"  /* Transfer Abducens Nerve to Accessory Nerve, Open Approach */
"00XL0ZS"  =  "1"  /* Transfer Abducens Nerve to Hypoglossal Nerve, Open Approach */
"00XL4ZF"  =  "1"  /* Transfer Abducens Nerve to Olfact Nrv, Perc Endo Approach */
"00XL4ZG"  =  "1"  /* Transfer Abducens Nerve to Optic Nerve, Perc Endo Approach */
"00XL4ZH"  =  "1"  /* Transfer Abducens Nerve to Oculomotor N, Perc Endo Approach */
"00XL4ZJ"  =  "1"  /* Transfer Abducens Nerve to Trochlear Nrv, Perc Endo Approach */
"00XL4ZK"  =  "1"  /* Transfer Abducens Nerve to Trigem Nrv, Perc Endo Approach */
"00XL4ZL"  =  "1"  /* Transfer Abducens Nerve to Abducen Nrv, Perc Endo Approach */
"00XL4ZM"  =  "1"  /* Transfer Abducens Nerve to Facial Nerve, Perc Endo Approach */
"00XL4ZN"  =  "1"  /* Transfer Abducens Nerve to Acoust Nrv, Perc Endo Approach */
"00XL4ZP"  =  "1"  /* Transfer Abducen Nrv to Glossophar Nrv, Perc Endo Approach */
"00XL4ZQ"  =  "1"  /* Transfer Abducens Nerve to Vagus Nerve, Perc Endo Approach */
"00XL4ZR"  =  "1"  /* Transfer Abducens Nerve to Accessory Nrv, Perc Endo Approach */
"00XL4ZS"  =  "1"  /* Transfer Abducens Nerve to Hypogloss Nrv, Perc Endo Approach */
"00XM0ZF"  =  "1"  /* Transfer Facial Nerve to Olfactory Nerve, Open Approach */
"00XM0ZG"  =  "1"  /* Transfer Facial Nerve to Optic Nerve, Open Approach */
"00XM0ZH"  =  "1"  /* Transfer Facial Nerve to Oculomotor Nerve, Open Approach */
"00XM0ZJ"  =  "1"  /* Transfer Facial Nerve to Trochlear Nerve, Open Approach */
"00XM0ZK"  =  "1"  /* Transfer Facial Nerve to Trigeminal Nerve, Open Approach */
"00XM0ZL"  =  "1"  /* Transfer Facial Nerve to Abducens Nerve, Open Approach */
"00XM0ZM"  =  "1"  /* Transfer Facial Nerve to Facial Nerve, Open Approach */
"00XM0ZN"  =  "1"  /* Transfer Facial Nerve to Acoustic Nerve, Open Approach */
"00XM0ZP"  =  "1"  /* Transfer Facial Nerve to Glossophar Nrv, Open Approach */
"00XM0ZQ"  =  "1"  /* Transfer Facial Nerve to Vagus Nerve, Open Approach */
"00XM0ZR"  =  "1"  /* Transfer Facial Nerve to Accessory Nerve, Open Approach */
"00XM0ZS"  =  "1"  /* Transfer Facial Nerve to Hypoglossal Nerve, Open Approach */
"00XM4ZF"  =  "1"  /* Transfer Facial Nerve to Olfactory Nerve, Perc Endo Approach */
"00XM4ZG"  =  "1"  /* Transfer Facial Nerve to Optic Nerve, Perc Endo Approach */
"00XM4ZH"  =  "1"  /* Transfer Facial Nerve to Oculomotor N, Perc Endo Approach */
"00XM4ZJ"  =  "1"  /* Transfer Facial Nerve to Trochlear Nerve, Perc Endo Approach */
"00XM4ZK"  =  "1"  /* Transfer Facial Nerve to Trigem Nrv, Perc Endo Approach */
"00XM4ZL"  =  "1"  /* Transfer Facial Nerve to Abducens Nerve, Perc Endo Approach */
"00XM4ZM"  =  "1"  /* Transfer Facial Nerve to Facial Nerve, Perc Endo Approach */
"00XM4ZN"  =  "1"  /* Transfer Facial Nerve to Acoustic Nerve, Perc Endo Approach */
"00XM4ZP"  =  "1"  /* Transfer Facial Nerve to Glossophar Nrv, Perc Endo Approach */
"00XM4ZQ"  =  "1"  /* Transfer Facial Nerve to Vagus Nerve, Perc Endo Approach */
"00XM4ZR"  =  "1"  /* Transfer Facial Nerve to Accessory Nerve, Perc Endo Approach */
"00XM4ZS"  =  "1"  /* Transfer Facial Nerve to Hypogloss Nrv, Perc Endo Approach */
"00XN0ZF"  =  "1"  /* Transfer Acoustic Nerve to Olfactory Nerve, Open Approach */
"00XN0ZG"  =  "1"  /* Transfer Acoustic Nerve to Optic Nerve, Open Approach */
"00XN0ZH"  =  "1"  /* Transfer Acoustic Nerve to Oculomotor Nerve, Open Approach */
"00XN0ZJ"  =  "1"  /* Transfer Acoustic Nerve to Trochlear Nerve, Open Approach */
"00XN0ZK"  =  "1"  /* Transfer Acoustic Nerve to Trigeminal Nerve, Open Approach */
"00XN0ZL"  =  "1"  /* Transfer Acoustic Nerve to Abducens Nerve, Open Approach */
"00XN0ZM"  =  "1"  /* Transfer Acoustic Nerve to Facial Nerve, Open Approach */
"00XN0ZN"  =  "1"  /* Transfer Acoustic Nerve to Acoustic Nerve, Open Approach */
"00XN0ZP"  =  "1"  /* Transfer Acoustic Nerve to Glossophar Nrv, Open Approach */
"00XN0ZQ"  =  "1"  /* Transfer Acoustic Nerve to Vagus Nerve, Open Approach */
"00XN0ZR"  =  "1"  /* Transfer Acoustic Nerve to Accessory Nerve, Open Approach */
"00XN0ZS"  =  "1"  /* Transfer Acoustic Nerve to Hypoglossal Nerve, Open Approach */
"00XN4ZF"  =  "1"  /* Transfer Acoustic Nerve to Olfact Nrv, Perc Endo Approach */
"00XN4ZG"  =  "1"  /* Transfer Acoustic Nerve to Optic Nerve, Perc Endo Approach */
"00XN4ZH"  =  "1"  /* Transfer Acoustic Nerve to Oculomotor N, Perc Endo Approach */
"00XN4ZJ"  =  "1"  /* Transfer Acoustic Nerve to Trochlear Nrv, Perc Endo Approach */
"00XN4ZK"  =  "1"  /* Transfer Acoustic Nerve to Trigem Nrv, Perc Endo Approach */
"00XN4ZL"  =  "1"  /* Transfer Acoustic Nerve to Abducen Nrv, Perc Endo Approach */
"00XN4ZM"  =  "1"  /* Transfer Acoustic Nerve to Facial Nerve, Perc Endo Approach */
"00XN4ZN"  =  "1"  /* Transfer Acoustic Nerve to Acoust Nrv, Perc Endo Approach */
"00XN4ZP"  =  "1"  /* Transfer Acoust Nrv to Glossophar Nrv, Perc Endo Approach */
"00XN4ZQ"  =  "1"  /* Transfer Acoustic Nerve to Vagus Nerve, Perc Endo Approach */
"00XN4ZR"  =  "1"  /* Transfer Acoustic Nerve to Accessory Nrv, Perc Endo Approach */
"00XN4ZS"  =  "1"  /* Transfer Acoustic Nerve to Hypogloss Nrv, Perc Endo Approach */
"00XP0ZF"  =  "1"  /* Transfer Glossopharyngeal Nerve to Olfact Nrv, Open Approach */
"00XP0ZG"  =  "1"  /* Transfer Glossopharyngeal Nerve to Optic Nrv, Open Approach */
"00XP0ZH"  =  "1"  /* Transfer Glossophar Nrv to Oculomotor N, Open Approach */
"00XP0ZJ"  =  "1"  /* Transfer Glossophar Nrv to Trochlear Nrv, Open Approach */
"00XP0ZK"  =  "1"  /* Transfer Glossopharyngeal Nerve to Trigem Nrv, Open Approach */
"00XP0ZL"  =  "1"  /* Transfer Glossophar Nrv to Abducen Nrv, Open Approach */
"00XP0ZM"  =  "1"  /* Transfer Glossopharyngeal Nerve to Facial Nrv, Open Approach */
"00XP0ZN"  =  "1"  /* Transfer Glossopharyngeal Nerve to Acoust Nrv, Open Approach */
"00XP0ZP"  =  "1"  /* Transfer Glossophar Nrv to Glossophar Nrv, Open Approach */
"00XP0ZQ"  =  "1"  /* Transfer Glossopharyngeal Nerve to Vagus Nrv, Open Approach */
"00XP0ZR"  =  "1"  /* Transfer Glossophar Nrv to Accessory Nrv, Open Approach */
"00XP0ZS"  =  "1"  /* Transfer Glossophar Nrv to Hypogloss Nrv, Open Approach */
"00XP4ZF"  =  "1"  /* Transfer Glossophar Nrv to Olfact Nrv, Perc Endo Approach */
"00XP4ZG"  =  "1"  /* Transfer Glossophar Nrv to Optic Nrv, Perc Endo Approach */
"00XP4ZH"  =  "1"  /* Transfer Glossophar Nrv to Oculomotor N, Perc Endo Approach */
"00XP4ZJ"  =  "1"  /* Transfer Glossophar Nrv to Trochlear Nrv, Perc Endo Approach */
"00XP4ZK"  =  "1"  /* Transfer Glossophar Nrv to Trigem Nrv, Perc Endo Approach */
"00XP4ZL"  =  "1"  /* Transfer Glossophar Nrv to Abducen Nrv, Perc Endo Approach */
"00XP4ZM"  =  "1"  /* Transfer Glossophar Nrv to Facial Nrv, Perc Endo Approach */
"00XP4ZN"  =  "1"  /* Transfer Glossophar Nrv to Acoust Nrv, Perc Endo Approach */
"00XP4ZP"  =  "1"  /* Transfer Glossophar Nrv to Glossophar Nrv, Perc Endo */
"00XP4ZQ"  =  "1"  /* Transfer Glossophar Nrv to Vagus Nrv, Perc Endo Approach */
"00XP4ZR"  =  "1"  /* Transfer Glossophar Nrv to Accessory Nrv, Perc Endo Approach */
"00XP4ZS"  =  "1"  /* Transfer Glossophar Nrv to Hypogloss Nrv, Perc Endo Approach */
"00XQ0ZF"  =  "1"  /* Transfer Vagus Nerve to Olfactory Nerve, Open Approach */
"00XQ0ZG"  =  "1"  /* Transfer Vagus Nerve to Optic Nerve, Open Approach */
"00XQ0ZH"  =  "1"  /* Transfer Vagus Nerve to Oculomotor Nerve, Open Approach */
"00XQ0ZJ"  =  "1"  /* Transfer Vagus Nerve to Trochlear Nerve, Open Approach */
"00XQ0ZK"  =  "1"  /* Transfer Vagus Nerve to Trigeminal Nerve, Open Approach */
"00XQ0ZL"  =  "1"  /* Transfer Vagus Nerve to Abducens Nerve, Open Approach */
"00XQ0ZM"  =  "1"  /* Transfer Vagus Nerve to Facial Nerve, Open Approach */
"00XQ0ZN"  =  "1"  /* Transfer Vagus Nerve to Acoustic Nerve, Open Approach */
"00XQ0ZP"  =  "1"  /* Transfer Vagus Nerve to Glossophar Nrv, Open Approach */
"00XQ0ZQ"  =  "1"  /* Transfer Vagus Nerve to Vagus Nerve, Open Approach */
"00XQ0ZR"  =  "1"  /* Transfer Vagus Nerve to Accessory Nerve, Open Approach */
"00XQ0ZS"  =  "1"  /* Transfer Vagus Nerve to Hypoglossal Nerve, Open Approach */
"00XQ4ZF"  =  "1"  /* Transfer Vagus Nerve to Olfactory Nerve, Perc Endo Approach */
"00XQ4ZG"  =  "1"  /* Transfer Vagus Nerve to Optic Nerve, Perc Endo Approach */
"00XQ4ZH"  =  "1"  /* Transfer Vagus Nerve to Oculomotor Nerve, Perc Endo Approach */
"00XQ4ZJ"  =  "1"  /* Transfer Vagus Nerve to Trochlear Nerve, Perc Endo Approach */
"00XQ4ZK"  =  "1"  /* Transfer Vagus Nerve to Trigeminal Nerve, Perc Endo Approach */
"00XQ4ZL"  =  "1"  /* Transfer Vagus Nerve to Abducens Nerve, Perc Endo Approach */
"00XQ4ZM"  =  "1"  /* Transfer Vagus Nerve to Facial Nerve, Perc Endo Approach */
"00XQ4ZN"  =  "1"  /* Transfer Vagus Nerve to Acoustic Nerve, Perc Endo Approach */
"00XQ4ZP"  =  "1"  /* Transfer Vagus Nerve to Glossophar Nrv, Perc Endo Approach */
"00XQ4ZQ"  =  "1"  /* Transfer Vagus Nerve to Vagus Nerve, Perc Endo Approach */
"00XQ4ZR"  =  "1"  /* Transfer Vagus Nerve to Accessory Nerve, Perc Endo Approach */
"00XQ4ZS"  =  "1"  /* Transfer Vagus Nerve to Hypogloss Nrv, Perc Endo Approach */
"00XR0ZF"  =  "1"  /* Transfer Accessory Nerve to Olfactory Nerve, Open Approach */
"00XR0ZG"  =  "1"  /* Transfer Accessory Nerve to Optic Nerve, Open Approach */
"00XR0ZH"  =  "1"  /* Transfer Accessory Nerve to Oculomotor Nerve, Open Approach */
"00XR0ZJ"  =  "1"  /* Transfer Accessory Nerve to Trochlear Nerve, Open Approach */
"00XR0ZK"  =  "1"  /* Transfer Accessory Nerve to Trigeminal Nerve, Open Approach */
"00XR0ZL"  =  "1"  /* Transfer Accessory Nerve to Abducens Nerve, Open Approach */
"00XR0ZM"  =  "1"  /* Transfer Accessory Nerve to Facial Nerve, Open Approach */
"00XR0ZN"  =  "1"  /* Transfer Accessory Nerve to Acoustic Nerve, Open Approach */
"00XR0ZP"  =  "1"  /* Transfer Accessory Nerve to Glossophar Nrv, Open Approach */
"00XR0ZQ"  =  "1"  /* Transfer Accessory Nerve to Vagus Nerve, Open Approach */
"00XR0ZR"  =  "1"  /* Transfer Accessory Nerve to Accessory Nerve, Open Approach */
"00XR0ZS"  =  "1"  /* Transfer Accessory Nerve to Hypoglossal Nerve, Open Approach */
"00XR4ZF"  =  "1"  /* Transfer Accessory Nerve to Olfact Nrv, Perc Endo Approach */
"00XR4ZG"  =  "1"  /* Transfer Accessory Nerve to Optic Nerve, Perc Endo Approach */
"00XR4ZH"  =  "1"  /* Transfer Accessory Nerve to Oculomotor N, Perc Endo Approach */
"00XR4ZJ"  =  "1"  /* Transfer Accessory Nrv to Trochlear Nrv, Perc Endo Approach */
"00XR4ZK"  =  "1"  /* Transfer Accessory Nerve to Trigem Nrv, Perc Endo Approach */
"00XR4ZL"  =  "1"  /* Transfer Accessory Nerve to Abducen Nrv, Perc Endo Approach */
"00XR4ZM"  =  "1"  /* Transfer Accessory Nerve to Facial Nerve, Perc Endo Approach */
"00XR4ZN"  =  "1"  /* Transfer Accessory Nerve to Acoust Nrv, Perc Endo Approach */
"00XR4ZP"  =  "1"  /* Transfer Accessory Nrv to Glossophar Nrv, Perc Endo Approach */
"00XR4ZQ"  =  "1"  /* Transfer Accessory Nerve to Vagus Nerve, Perc Endo Approach */
"00XR4ZR"  =  "1"  /* Transfer Accessory Nrv to Accessory Nrv, Perc Endo Approach */
"00XR4ZS"  =  "1"  /* Transfer Accessory Nrv to Hypogloss Nrv, Perc Endo Approach */
"00XS0ZF"  =  "1"  /* Transfer Hypoglossal Nerve to Olfactory Nerve, Open Approach */
"00XS0ZG"  =  "1"  /* Transfer Hypoglossal Nerve to Optic Nerve, Open Approach */
"00XS0ZH"  =  "1"  /* Transfer Hypoglossal Nerve to Oculomotor N, Open Approach */
"00XS0ZJ"  =  "1"  /* Transfer Hypoglossal Nerve to Trochlear Nerve, Open Approach */
"00XS0ZK"  =  "1"  /* Transfer Hypoglossal Nerve to Trigem Nrv, Open Approach */
"00XS0ZL"  =  "1"  /* Transfer Hypoglossal Nerve to Abducens Nerve, Open Approach */
"00XS0ZM"  =  "1"  /* Transfer Hypoglossal Nerve to Facial Nerve, Open Approach */
"00XS0ZN"  =  "1"  /* Transfer Hypoglossal Nerve to Acoustic Nerve, Open Approach */
"00XS0ZP"  =  "1"  /* Transfer Hypoglossal Nerve to Glossophar Nrv, Open Approach */
"00XS0ZQ"  =  "1"  /* Transfer Hypoglossal Nerve to Vagus Nerve, Open Approach */
"00XS0ZR"  =  "1"  /* Transfer Hypoglossal Nerve to Accessory Nerve, Open Approach */
"00XS0ZS"  =  "1"  /* Transfer Hypoglossal Nerve to Hypogloss Nrv, Open Approach */
"00XS4ZF"  =  "1"  /* Transfer Hypoglossal Nerve to Olfact Nrv, Perc Endo Approach */
"00XS4ZG"  =  "1"  /* Transfer Hypoglossal Nerve to Optic Nrv, Perc Endo Approach */
"00XS4ZH"  =  "1"  /* Transfer Hypogloss Nrv to Oculomotor N, Perc Endo Approach */
"00XS4ZJ"  =  "1"  /* Transfer Hypogloss Nrv to Trochlear Nrv, Perc Endo Approach */
"00XS4ZK"  =  "1"  /* Transfer Hypoglossal Nerve to Trigem Nrv, Perc Endo Approach */
"00XS4ZL"  =  "1"  /* Transfer Hypogloss Nrv to Abducen Nrv, Perc Endo Approach */
"00XS4ZM"  =  "1"  /* Transfer Hypoglossal Nerve to Facial Nrv, Perc Endo Approach */
"00XS4ZN"  =  "1"  /* Transfer Hypoglossal Nerve to Acoust Nrv, Perc Endo Approach */
"00XS4ZP"  =  "1"  /* Transfer Hypogloss Nrv to Glossophar Nrv, Perc Endo Approach */
"00XS4ZQ"  =  "1"  /* Transfer Hypoglossal Nerve to Vagus Nrv, Perc Endo Approach */
"00XS4ZR"  =  "1"  /* Transfer Hypogloss Nrv to Accessory Nrv, Perc Endo Approach */
"00XS4ZS"  =  "1"  /* Transfer Hypogloss Nrv to Hypogloss Nrv, Perc Endo Approach */
"01510ZZ"  =  "1"  /* Destruction of Cervical Nerve, Open Approach */
"01514ZZ"  =  "1"  /* Destruction of Cervical Nerve, Perc Endo Approach */
"01580ZZ"  =  "1"  /* Destruction of Thoracic Nerve, Open Approach */
"01584ZZ"  =  "1"  /* Destruction of Thoracic Nerve, Perc Endo Approach */
"015B0ZZ"  =  "1"  /* Destruction of Lumbar Nerve, Open Approach */
"015B4ZZ"  =  "1"  /* Destruction of Lumbar Nerve, Perc Endo Approach */
"015K0ZZ"  =  "1"  /* Destruction of Head Neck Symp Nrv, Open Approach */
"015K3ZZ"  =  "1"  /* Destruction of Head Neck Symp Nrv, Perc Approach */
"015K4ZZ"  =  "1"  /* Destruction of Head Neck Symp Nrv, Perc Endo Approach */
"015L0ZZ"  =  "1"  /* Destruction of Thoracic Sympathetic Nerve, Open Approach */
"015L3ZZ"  =  "1"  /* Destruction of Thoracic Sympathetic Nerve, Perc Approach */
"015L4ZZ"  =  "1"  /* Destruction of Thor Symp Nrv, Perc Endo Approach */
"015M0ZZ"  =  "1"  /* Destruction of Abdominal Sympathetic Nerve, Open Approach */
"015M3ZZ"  =  "1"  /* Destruction of Abdominal Sympathetic Nerve, Perc Approach */
"015M4ZZ"  =  "1"  /* Destruction of Abd Symp Nrv, Perc Endo Approach */
"015N0ZZ"  =  "1"  /* Destruction of Lumbar Sympathetic Nerve, Open Approach */
"015N3ZZ"  =  "1"  /* Destruction of Lumbar Sympathetic Nerve, Perc Approach */
"015N4ZZ"  =  "1"  /* Destruction of Lumbar Sympathetic Nerve, Perc Endo Approach */
"015P0ZZ"  =  "1"  /* Destruction of Sacral Sympathetic Nerve, Open Approach */
"015P3ZZ"  =  "1"  /* Destruction of Sacral Sympathetic Nerve, Perc Approach */
"015P4ZZ"  =  "1"  /* Destruction of Sacral Sympathetic Nerve, Perc Endo Approach */
"015R0ZZ"  =  "1"  /* Destruction of Sacral Nerve, Open Approach */
"015R4ZZ"  =  "1"  /* Destruction of Sacral Nerve, Perc Endo Approach */
"01800ZZ"  =  "1"  /* Division of Cervical Plexus, Open Approach */
"01803ZZ"  =  "1"  /* Division of Cervical Plexus, Percutaneous Approach */
"01804ZZ"  =  "1"  /* Division of Cervical Plexus, Perc Endo Approach */
"01810ZZ"  =  "1"  /* Division of Cervical Nerve, Open Approach */
"01813ZZ"  =  "1"  /* Division of Cervical Nerve, Percutaneous Approach */
"01814ZZ"  =  "1"  /* Division of Cervical Nerve, Percutaneous Endoscopic Approach */
"01820ZZ"  =  "1"  /* Division of Phrenic Nerve, Open Approach */
"01823ZZ"  =  "1"  /* Division of Phrenic Nerve, Percutaneous Approach */
"01824ZZ"  =  "1"  /* Division of Phrenic Nerve, Percutaneous Endoscopic Approach */
"01830ZZ"  =  "1"  /* Division of Brachial Plexus, Open Approach */
"01833ZZ"  =  "1"  /* Division of Brachial Plexus, Percutaneous Approach */
"01834ZZ"  =  "1"  /* Division of Brachial Plexus, Perc Endo Approach */
"01840ZZ"  =  "1"  /* Division of Ulnar Nerve, Open Approach */
"01843ZZ"  =  "1"  /* Division of Ulnar Nerve, Percutaneous Approach */
"01844ZZ"  =  "1"  /* Division of Ulnar Nerve, Percutaneous Endoscopic Approach */
"01850ZZ"  =  "1"  /* Division of Median Nerve, Open Approach */
"01853ZZ"  =  "1"  /* Division of Median Nerve, Percutaneous Approach */
"01854ZZ"  =  "1"  /* Division of Median Nerve, Percutaneous Endoscopic Approach */
"01860ZZ"  =  "1"  /* Division of Radial Nerve, Open Approach */
"01863ZZ"  =  "1"  /* Division of Radial Nerve, Percutaneous Approach */
"01864ZZ"  =  "1"  /* Division of Radial Nerve, Percutaneous Endoscopic Approach */
"01880ZZ"  =  "1"  /* Division of Thoracic Nerve, Open Approach */
"01883ZZ"  =  "1"  /* Division of Thoracic Nerve, Percutaneous Approach */
"01884ZZ"  =  "1"  /* Division of Thoracic Nerve, Percutaneous Endoscopic Approach */
"01890ZZ"  =  "1"  /* Division of Lumbar Plexus, Open Approach */
"01893ZZ"  =  "1"  /* Division of Lumbar Plexus, Percutaneous Approach */
"01894ZZ"  =  "1"  /* Division of Lumbar Plexus, Percutaneous Endoscopic Approach */
"018A0ZZ"  =  "1"  /* Division of Lumbosacral Plexus, Open Approach */
"018A3ZZ"  =  "1"  /* Division of Lumbosacral Plexus, Percutaneous Approach */
"018A4ZZ"  =  "1"  /* Division of Lumbosacral Plexus, Perc Endo Approach */
"018B0ZZ"  =  "1"  /* Division of Lumbar Nerve, Open Approach */
"018B3ZZ"  =  "1"  /* Division of Lumbar Nerve, Percutaneous Approach */
"018B4ZZ"  =  "1"  /* Division of Lumbar Nerve, Percutaneous Endoscopic Approach */
"018C0ZZ"  =  "1"  /* Division of Pudendal Nerve, Open Approach */
"018C3ZZ"  =  "1"  /* Division of Pudendal Nerve, Percutaneous Approach */
"018C4ZZ"  =  "1"  /* Division of Pudendal Nerve, Percutaneous Endoscopic Approach */
"018D0ZZ"  =  "1"  /* Division of Femoral Nerve, Open Approach */
"018D3ZZ"  =  "1"  /* Division of Femoral Nerve, Percutaneous Approach */
"018D4ZZ"  =  "1"  /* Division of Femoral Nerve, Percutaneous Endoscopic Approach */
"018F0ZZ"  =  "1"  /* Division of Sciatic Nerve, Open Approach */
"018F3ZZ"  =  "1"  /* Division of Sciatic Nerve, Percutaneous Approach */
"018F4ZZ"  =  "1"  /* Division of Sciatic Nerve, Percutaneous Endoscopic Approach */
"018G0ZZ"  =  "1"  /* Division of Tibial Nerve, Open Approach */
"018G3ZZ"  =  "1"  /* Division of Tibial Nerve, Percutaneous Approach */
"018G4ZZ"  =  "1"  /* Division of Tibial Nerve, Percutaneous Endoscopic Approach */
"018H0ZZ"  =  "1"  /* Division of Peroneal Nerve, Open Approach */
"018H3ZZ"  =  "1"  /* Division of Peroneal Nerve, Percutaneous Approach */
"018H4ZZ"  =  "1"  /* Division of Peroneal Nerve, Percutaneous Endoscopic Approach */
"018K0ZZ"  =  "1"  /* Division of Head and Neck Sympathetic Nerve, Open Approach */
"018K3ZZ"  =  "1"  /* Division of Head and Neck Sympathetic Nerve, Perc Approach */
"018K4ZZ"  =  "1"  /* Division of Head Neck Symp Nrv, Perc Endo Approach */
"018L0ZZ"  =  "1"  /* Division of Thoracic Sympathetic Nerve, Open Approach */
"018L3ZZ"  =  "1"  /* Division of Thoracic Sympathetic Nerve, Perc Approach */
"018L4ZZ"  =  "1"  /* Division of Thoracic Sympathetic Nerve, Perc Endo Approach */
"018M0ZZ"  =  "1"  /* Division of Abdominal Sympathetic Nerve, Open Approach */
"018M3ZZ"  =  "1"  /* Division of Abdominal Sympathetic Nerve, Perc Approach */
"018M4ZZ"  =  "1"  /* Division of Abdominal Sympathetic Nerve, Perc Endo Approach */
"018N0ZZ"  =  "1"  /* Division of Lumbar Sympathetic Nerve, Open Approach */
"018N3ZZ"  =  "1"  /* Division of Lumbar Sympathetic Nerve, Percutaneous Approach */
"018N4ZZ"  =  "1"  /* Division of Lumbar Sympathetic Nerve, Perc Endo Approach */
"018P0ZZ"  =  "1"  /* Division of Sacral Sympathetic Nerve, Open Approach */
"018P3ZZ"  =  "1"  /* Division of Sacral Sympathetic Nerve, Percutaneous Approach */
"018P4ZZ"  =  "1"  /* Division of Sacral Sympathetic Nerve, Perc Endo Approach */
"018Q0ZZ"  =  "1"  /* Division of Sacral Plexus, Open Approach */
"018Q3ZZ"  =  "1"  /* Division of Sacral Plexus, Percutaneous Approach */
"018Q4ZZ"  =  "1"  /* Division of Sacral Plexus, Percutaneous Endoscopic Approach */
"018R0ZZ"  =  "1"  /* Division of Sacral Nerve, Open Approach */
"018R3ZZ"  =  "1"  /* Division of Sacral Nerve, Percutaneous Approach */
"018R4ZZ"  =  "1"  /* Division of Sacral Nerve, Percutaneous Endoscopic Approach */
"019000Z"  =  "1"  /* Drainage of Cervical Plexus with Drain Dev, Open Approach */
"01900ZX"  =  "1"  /* Drainage of Cervical Plexus, Open Approach, Diagnostic */
"01900ZZ"  =  "1"  /* Drainage of Cervical Plexus, Open Approach */
"019030Z"  =  "1"  /* Drainage of Cervical Plexus with Drain Dev, Perc Approach */
"01903ZZ"  =  "1"  /* Drainage of Cervical Plexus, Percutaneous Approach */
"019040Z"  =  "1"  /* Drainage of Cerv Plexus with Drain Dev, Perc Endo Approach */
"01904ZZ"  =  "1"  /* Drainage of Cervical Plexus, Perc Endo Approach */
"019100Z"  =  "1"  /* Drainage of Cervical Nerve with Drain Dev, Open Approach */
"01910ZX"  =  "1"  /* Drainage of Cervical Nerve, Open Approach, Diagnostic */
"01910ZZ"  =  "1"  /* Drainage of Cervical Nerve, Open Approach */
"019130Z"  =  "1"  /* Drainage of Cervical Nerve with Drain Dev, Perc Approach */
"01913ZZ"  =  "1"  /* Drainage of Cervical Nerve, Percutaneous Approach */
"019140Z"  =  "1"  /* Drainage of Cerv Nrv with Drain Dev, Perc Endo Approach */
"01914ZZ"  =  "1"  /* Drainage of Cervical Nerve, Percutaneous Endoscopic Approach */
"019200Z"  =  "1"  /* Drainage of Phrenic Nerve with Drain Dev, Open Approach */
"01920ZX"  =  "1"  /* Drainage of Phrenic Nerve, Open Approach, Diagnostic */
"01920ZZ"  =  "1"  /* Drainage of Phrenic Nerve, Open Approach */
"019230Z"  =  "1"  /* Drainage of Phrenic Nerve with Drain Dev, Perc Approach */
"01923ZZ"  =  "1"  /* Drainage of Phrenic Nerve, Percutaneous Approach */
"019240Z"  =  "1"  /* Drainage of Phrenic Nerve with Drain Dev, Perc Endo Approach */
"01924ZZ"  =  "1"  /* Drainage of Phrenic Nerve, Percutaneous Endoscopic Approach */
"019300Z"  =  "1"  /* Drainage of Brachial Plexus with Drain Dev, Open Approach */
"01930ZX"  =  "1"  /* Drainage of Brachial Plexus, Open Approach, Diagnostic */
"01930ZZ"  =  "1"  /* Drainage of Brachial Plexus, Open Approach */
"019330Z"  =  "1"  /* Drainage of Brachial Plexus with Drain Dev, Perc Approach */
"01933ZZ"  =  "1"  /* Drainage of Brachial Plexus, Percutaneous Approach */
"019340Z"  =  "1"  /* Drainage of Brach Plexus with Drain Dev, Perc Endo Approach */
"01934ZZ"  =  "1"  /* Drainage of Brachial Plexus, Perc Endo Approach */
"019400Z"  =  "1"  /* Drainage of Ulnar Nerve with Drainage Device, Open Approach */
"01940ZX"  =  "1"  /* Drainage of Ulnar Nerve, Open Approach, Diagnostic */
"01940ZZ"  =  "1"  /* Drainage of Ulnar Nerve, Open Approach */
"019430Z"  =  "1"  /* Drainage of Ulnar Nerve with Drainage Device, Perc Approach */
"01943ZZ"  =  "1"  /* Drainage of Ulnar Nerve, Percutaneous Approach */
"019440Z"  =  "1"  /* Drainage of Ulnar Nerve with Drain Dev, Perc Endo Approach */
"01944ZZ"  =  "1"  /* Drainage of Ulnar Nerve, Percutaneous Endoscopic Approach */
"019500Z"  =  "1"  /* Drainage of Median Nerve with Drainage Device, Open Approach */
"01950ZX"  =  "1"  /* Drainage of Median Nerve, Open Approach, Diagnostic */
"01950ZZ"  =  "1"  /* Drainage of Median Nerve, Open Approach */
"019530Z"  =  "1"  /* Drainage of Median Nerve with Drainage Device, Perc Approach */
"01953ZZ"  =  "1"  /* Drainage of Median Nerve, Percutaneous Approach */
"019540Z"  =  "1"  /* Drainage of Median Nerve with Drain Dev, Perc Endo Approach */
"01954ZZ"  =  "1"  /* Drainage of Median Nerve, Percutaneous Endoscopic Approach */
"019600Z"  =  "1"  /* Drainage of Radial Nerve with Drainage Device, Open Approach */
"01960ZX"  =  "1"  /* Drainage of Radial Nerve, Open Approach, Diagnostic */
"01960ZZ"  =  "1"  /* Drainage of Radial Nerve, Open Approach */
"019630Z"  =  "1"  /* Drainage of Radial Nerve with Drainage Device, Perc Approach */
"01963ZZ"  =  "1"  /* Drainage of Radial Nerve, Percutaneous Approach */
"019640Z"  =  "1"  /* Drainage of Radial Nerve with Drain Dev, Perc Endo Approach */
"01964ZZ"  =  "1"  /* Drainage of Radial Nerve, Percutaneous Endoscopic Approach */
"019800Z"  =  "1"  /* Drainage of Thoracic Nerve with Drain Dev, Open Approach */
"01980ZX"  =  "1"  /* Drainage of Thoracic Nerve, Open Approach, Diagnostic */
"01980ZZ"  =  "1"  /* Drainage of Thoracic Nerve, Open Approach */
"019830Z"  =  "1"  /* Drainage of Thoracic Nerve with Drain Dev, Perc Approach */
"01983ZZ"  =  "1"  /* Drainage of Thoracic Nerve, Percutaneous Approach */
"019840Z"  =  "1"  /* Drainage of Thoracic Nrv with Drain Dev, Perc Endo Approach */
"01984ZZ"  =  "1"  /* Drainage of Thoracic Nerve, Percutaneous Endoscopic Approach */
"019900Z"  =  "1"  /* Drainage of Lumbar Plexus with Drain Dev, Open Approach */
"01990ZX"  =  "1"  /* Drainage of Lumbar Plexus, Open Approach, Diagnostic */
"01990ZZ"  =  "1"  /* Drainage of Lumbar Plexus, Open Approach */
"019930Z"  =  "1"  /* Drainage of Lumbar Plexus with Drain Dev, Perc Approach */
"01993ZZ"  =  "1"  /* Drainage of Lumbar Plexus, Percutaneous Approach */
"019940Z"  =  "1"  /* Drainage of Lumbar Plexus with Drain Dev, Perc Endo Approach */
"01994ZZ"  =  "1"  /* Drainage of Lumbar Plexus, Percutaneous Endoscopic Approach */
"019A00Z"  =  "1"  /* Drainage of Lumbosacral Plexus with Drain Dev, Open Approach */
"019A0ZX"  =  "1"  /* Drainage of Lumbosacral Plexus, Open Approach, Diagnostic */
"019A0ZZ"  =  "1"  /* Drainage of Lumbosacral Plexus, Open Approach */
"019A30Z"  =  "1"  /* Drainage of Lumbosacral Plexus with Drain Dev, Perc Approach */
"019A3ZZ"  =  "1"  /* Drainage of Lumbosacral Plexus, Percutaneous Approach */
"019A40Z"  =  "1"  /* Drain of Lumbosacr Plexus with Drain Dev, Perc Endo Approach */
"019A4ZZ"  =  "1"  /* Drainage of Lumbosacral Plexus, Perc Endo Approach */
"019B00Z"  =  "1"  /* Drainage of Lumbar Nerve with Drainage Device, Open Approach */
"019B0ZX"  =  "1"  /* Drainage of Lumbar Nerve, Open Approach, Diagnostic */
"019B0ZZ"  =  "1"  /* Drainage of Lumbar Nerve, Open Approach */
"019B30Z"  =  "1"  /* Drainage of Lumbar Nerve with Drainage Device, Perc Approach */
"019B3ZZ"  =  "1"  /* Drainage of Lumbar Nerve, Percutaneous Approach */
"019B40Z"  =  "1"  /* Drainage of Lumbar Nerve with Drain Dev, Perc Endo Approach */
"019B4ZZ"  =  "1"  /* Drainage of Lumbar Nerve, Percutaneous Endoscopic Approach */
"019C00Z"  =  "1"  /* Drainage of Pudendal Nerve with Drain Dev, Open Approach */
"019C0ZX"  =  "1"  /* Drainage of Pudendal Nerve, Open Approach, Diagnostic */
"019C0ZZ"  =  "1"  /* Drainage of Pudendal Nerve, Open Approach */
"019C30Z"  =  "1"  /* Drainage of Pudendal Nerve with Drain Dev, Perc Approach */
"019C3ZZ"  =  "1"  /* Drainage of Pudendal Nerve, Percutaneous Approach */
"019C40Z"  =  "1"  /* Drainage of Pudend Nrv with Drain Dev, Perc Endo Approach */
"019C4ZZ"  =  "1"  /* Drainage of Pudendal Nerve, Percutaneous Endoscopic Approach */
"019D00Z"  =  "1"  /* Drainage of Femoral Nerve with Drain Dev, Open Approach */
"019D0ZX"  =  "1"  /* Drainage of Femoral Nerve, Open Approach, Diagnostic */
"019D0ZZ"  =  "1"  /* Drainage of Femoral Nerve, Open Approach */
"019D30Z"  =  "1"  /* Drainage of Femoral Nerve with Drain Dev, Perc Approach */
"019D3ZZ"  =  "1"  /* Drainage of Femoral Nerve, Percutaneous Approach */
"019D40Z"  =  "1"  /* Drainage of Femoral Nerve with Drain Dev, Perc Endo Approach */
"019D4ZZ"  =  "1"  /* Drainage of Femoral Nerve, Percutaneous Endoscopic Approach */
"019F00Z"  =  "1"  /* Drainage of Sciatic Nerve with Drain Dev, Open Approach */
"019F0ZX"  =  "1"  /* Drainage of Sciatic Nerve, Open Approach, Diagnostic */
"019F0ZZ"  =  "1"  /* Drainage of Sciatic Nerve, Open Approach */
"019F30Z"  =  "1"  /* Drainage of Sciatic Nerve with Drain Dev, Perc Approach */
"019F3ZZ"  =  "1"  /* Drainage of Sciatic Nerve, Percutaneous Approach */
"019F40Z"  =  "1"  /* Drainage of Sciatic Nerve with Drain Dev, Perc Endo Approach */
"019F4ZZ"  =  "1"  /* Drainage of Sciatic Nerve, Percutaneous Endoscopic Approach */
"019G00Z"  =  "1"  /* Drainage of Tibial Nerve with Drainage Device, Open Approach */
"019G0ZX"  =  "1"  /* Drainage of Tibial Nerve, Open Approach, Diagnostic */
"019G0ZZ"  =  "1"  /* Drainage of Tibial Nerve, Open Approach */
"019G30Z"  =  "1"  /* Drainage of Tibial Nerve with Drainage Device, Perc Approach */
"019G3ZZ"  =  "1"  /* Drainage of Tibial Nerve, Percutaneous Approach */
"019G40Z"  =  "1"  /* Drainage of Tibial Nerve with Drain Dev, Perc Endo Approach */
"019G4ZZ"  =  "1"  /* Drainage of Tibial Nerve, Percutaneous Endoscopic Approach */
"019H00Z"  =  "1"  /* Drainage of Peroneal Nerve with Drain Dev, Open Approach */
"019H0ZX"  =  "1"  /* Drainage of Peroneal Nerve, Open Approach, Diagnostic */
"019H0ZZ"  =  "1"  /* Drainage of Peroneal Nerve, Open Approach */
"019H30Z"  =  "1"  /* Drainage of Peroneal Nerve with Drain Dev, Perc Approach */
"019H3ZZ"  =  "1"  /* Drainage of Peroneal Nerve, Percutaneous Approach */
"019H40Z"  =  "1"  /* Drainage of Peroneal Nrv with Drain Dev, Perc Endo Approach */
"019H4ZZ"  =  "1"  /* Drainage of Peroneal Nerve, Percutaneous Endoscopic Approach */
"019K00Z"  =  "1"  /* Drainage of Head Neck Symp Nrv with Drain Dev, Open Approach */
"019K0ZX"  =  "1"  /* Drainage of Head Neck Symp Nrv, Open Approach, Diagn */
"019K0ZZ"  =  "1"  /* Drainage of Head and Neck Sympathetic Nerve, Open Approach */
"019K30Z"  =  "1"  /* Drainage of Head Neck Symp Nrv with Drain Dev, Perc Approach */
"019K3ZX"  =  "1"  /* Drainage of Head Neck Symp Nrv, Perc Approach, Diagn */
"019K3ZZ"  =  "1"  /* Drainage of Head and Neck Sympathetic Nerve, Perc Approach */
"019K40Z"  =  "1"  /* Drain Head Neck Symp Nrv w Drain Dev, Perc Endo */
"019K4ZX"  =  "1"  /* Drainage of Head Neck Symp Nrv, Perc Endo Approach, Diagn */
"019K4ZZ"  =  "1"  /* Drainage of Head Neck Symp Nrv, Perc Endo Approach */
"019L00Z"  =  "1"  /* Drainage of Thor Symp Nrv with Drain Dev, Open Approach */
"019L0ZX"  =  "1"  /* Drainage of Thoracic Sympathetic Nerve, Open Approach, Diagn */
"019L0ZZ"  =  "1"  /* Drainage of Thoracic Sympathetic Nerve, Open Approach */
"019L30Z"  =  "1"  /* Drainage of Thor Symp Nrv with Drain Dev, Perc Approach */
"019L3ZX"  =  "1"  /* Drainage of Thoracic Sympathetic Nerve, Perc Approach, Diagn */
"019L3ZZ"  =  "1"  /* Drainage of Thoracic Sympathetic Nerve, Perc Approach */
"019L40Z"  =  "1"  /* Drainage of Thor Symp Nrv with Drain Dev, Perc Endo Approach */
"019L4ZX"  =  "1"  /* Drainage of Thor Symp Nrv, Perc Endo Approach, Diagn */
"019L4ZZ"  =  "1"  /* Drainage of Thoracic Sympathetic Nerve, Perc Endo Approach */
"019M00Z"  =  "1"  /* Drainage of Abd Symp Nrv with Drain Dev, Open Approach */
"019M0ZX"  =  "1"  /* Drainage of Abd Symp Nrv, Open Approach, Diagn */
"019M0ZZ"  =  "1"  /* Drainage of Abdominal Sympathetic Nerve, Open Approach */
"019M30Z"  =  "1"  /* Drainage of Abd Symp Nrv with Drain Dev, Perc Approach */
"019M3ZX"  =  "1"  /* Drainage of Abd Symp Nrv, Perc Approach, Diagn */
"019M3ZZ"  =  "1"  /* Drainage of Abdominal Sympathetic Nerve, Perc Approach */
"019M40Z"  =  "1"  /* Drainage of Abd Symp Nrv with Drain Dev, Perc Endo Approach */
"019M4ZX"  =  "1"  /* Drainage of Abd Symp Nrv, Perc Endo Approach, Diagn */
"019M4ZZ"  =  "1"  /* Drainage of Abdominal Sympathetic Nerve, Perc Endo Approach */
"019N00Z"  =  "1"  /* Drainage of Lumbar Symp Nrv with Drain Dev, Open Approach */
"019N0ZX"  =  "1"  /* Drainage of Lumbar Sympathetic Nerve, Open Approach, Diagn */
"019N0ZZ"  =  "1"  /* Drainage of Lumbar Sympathetic Nerve, Open Approach */
"019N30Z"  =  "1"  /* Drainage of Lumbar Symp Nrv with Drain Dev, Perc Approach */
"019N3ZX"  =  "1"  /* Drainage of Lumbar Sympathetic Nerve, Perc Approach, Diagn */
"019N3ZZ"  =  "1"  /* Drainage of Lumbar Sympathetic Nerve, Percutaneous Approach */
"019N40Z"  =  "1"  /* Drain of Lumbar Symp Nrv with Drain Dev, Perc Endo Approach */
"019N4ZX"  =  "1"  /* Drainage of Lumbar Symp Nrv, Perc Endo Approach, Diagn */
"019N4ZZ"  =  "1"  /* Drainage of Lumbar Sympathetic Nerve, Perc Endo Approach */
"019P00Z"  =  "1"  /* Drainage of Sacral Symp Nrv with Drain Dev, Open Approach */
"019P0ZX"  =  "1"  /* Drainage of Sacral Sympathetic Nerve, Open Approach, Diagn */
"019P0ZZ"  =  "1"  /* Drainage of Sacral Sympathetic Nerve, Open Approach */
"019P30Z"  =  "1"  /* Drainage of Sacral Symp Nrv with Drain Dev, Perc Approach */
"019P3ZX"  =  "1"  /* Drainage of Sacral Sympathetic Nerve, Perc Approach, Diagn */
"019P3ZZ"  =  "1"  /* Drainage of Sacral Sympathetic Nerve, Percutaneous Approach */
"019P40Z"  =  "1"  /* Drain of Sacral Symp Nrv with Drain Dev, Perc Endo Approach */
"019P4ZX"  =  "1"  /* Drainage of Sacral Symp Nrv, Perc Endo Approach, Diagn */
"019P4ZZ"  =  "1"  /* Drainage of Sacral Sympathetic Nerve, Perc Endo Approach */
"019Q00Z"  =  "1"  /* Drainage of Sacral Plexus with Drain Dev, Open Approach */
"019Q0ZX"  =  "1"  /* Drainage of Sacral Plexus, Open Approach, Diagnostic */
"019Q0ZZ"  =  "1"  /* Drainage of Sacral Plexus, Open Approach */
"019Q30Z"  =  "1"  /* Drainage of Sacral Plexus with Drain Dev, Perc Approach */
"019Q3ZZ"  =  "1"  /* Drainage of Sacral Plexus, Percutaneous Approach */
"019Q40Z"  =  "1"  /* Drainage of Sacral Plexus with Drain Dev, Perc Endo Approach */
"019Q4ZZ"  =  "1"  /* Drainage of Sacral Plexus, Percutaneous Endoscopic Approach */
"019R00Z"  =  "1"  /* Drainage of Sacral Nerve with Drainage Device, Open Approach */
"019R0ZX"  =  "1"  /* Drainage of Sacral Nerve, Open Approach, Diagnostic */
"019R0ZZ"  =  "1"  /* Drainage of Sacral Nerve, Open Approach */
"019R30Z"  =  "1"  /* Drainage of Sacral Nerve with Drainage Device, Perc Approach */
"019R3ZZ"  =  "1"  /* Drainage of Sacral Nerve, Percutaneous Approach */
"019R40Z"  =  "1"  /* Drainage of Sacral Nerve with Drain Dev, Perc Endo Approach */
"019R4ZZ"  =  "1"  /* Drainage of Sacral Nerve, Percutaneous Endoscopic Approach */
"01B00ZX"  =  "1"  /* Excision of Cervical Plexus, Open Approach, Diagnostic */
"01B00ZZ"  =  "1"  /* Excision of Cervical Plexus, Open Approach */
"01B03ZZ"  =  "1"  /* Excision of Cervical Plexus, Percutaneous Approach */
"01B04ZZ"  =  "1"  /* Excision of Cervical Plexus, Perc Endo Approach */
"01B10ZX"  =  "1"  /* Excision of Cervical Nerve, Open Approach, Diagnostic */
"01B10ZZ"  =  "1"  /* Excision of Cervical Nerve, Open Approach */
"01B13ZZ"  =  "1"  /* Excision of Cervical Nerve, Percutaneous Approach */
"01B14ZZ"  =  "1"  /* Excision of Cervical Nerve, Percutaneous Endoscopic Approach */
"01B20ZX"  =  "1"  /* Excision of Phrenic Nerve, Open Approach, Diagnostic */
"01B20ZZ"  =  "1"  /* Excision of Phrenic Nerve, Open Approach */
"01B23ZZ"  =  "1"  /* Excision of Phrenic Nerve, Percutaneous Approach */
"01B24ZZ"  =  "1"  /* Excision of Phrenic Nerve, Percutaneous Endoscopic Approach */
"01B30ZX"  =  "1"  /* Excision of Brachial Plexus, Open Approach, Diagnostic */
"01B30ZZ"  =  "1"  /* Excision of Brachial Plexus, Open Approach */
"01B33ZZ"  =  "1"  /* Excision of Brachial Plexus, Percutaneous Approach */
"01B34ZZ"  =  "1"  /* Excision of Brachial Plexus, Perc Endo Approach */
"01B40ZX"  =  "1"  /* Excision of Ulnar Nerve, Open Approach, Diagnostic */
"01B40ZZ"  =  "1"  /* Excision of Ulnar Nerve, Open Approach */
"01B43ZZ"  =  "1"  /* Excision of Ulnar Nerve, Percutaneous Approach */
"01B44ZZ"  =  "1"  /* Excision of Ulnar Nerve, Percutaneous Endoscopic Approach */
"01B50ZX"  =  "1"  /* Excision of Median Nerve, Open Approach, Diagnostic */
"01B50ZZ"  =  "1"  /* Excision of Median Nerve, Open Approach */
"01B53ZZ"  =  "1"  /* Excision of Median Nerve, Percutaneous Approach */
"01B54ZZ"  =  "1"  /* Excision of Median Nerve, Percutaneous Endoscopic Approach */
"01B60ZX"  =  "1"  /* Excision of Radial Nerve, Open Approach, Diagnostic */
"01B60ZZ"  =  "1"  /* Excision of Radial Nerve, Open Approach */
"01B63ZZ"  =  "1"  /* Excision of Radial Nerve, Percutaneous Approach */
"01B64ZZ"  =  "1"  /* Excision of Radial Nerve, Percutaneous Endoscopic Approach */
"01B80ZX"  =  "1"  /* Excision of Thoracic Nerve, Open Approach, Diagnostic */
"01B80ZZ"  =  "1"  /* Excision of Thoracic Nerve, Open Approach */
"01B83ZZ"  =  "1"  /* Excision of Thoracic Nerve, Percutaneous Approach */
"01B84ZZ"  =  "1"  /* Excision of Thoracic Nerve, Percutaneous Endoscopic Approach */
"01B90ZX"  =  "1"  /* Excision of Lumbar Plexus, Open Approach, Diagnostic */
"01B90ZZ"  =  "1"  /* Excision of Lumbar Plexus, Open Approach */
"01B93ZZ"  =  "1"  /* Excision of Lumbar Plexus, Percutaneous Approach */
"01B94ZZ"  =  "1"  /* Excision of Lumbar Plexus, Percutaneous Endoscopic Approach */
"01BA0ZX"  =  "1"  /* Excision of Lumbosacral Plexus, Open Approach, Diagnostic */
"01BA0ZZ"  =  "1"  /* Excision of Lumbosacral Plexus, Open Approach */
"01BA3ZZ"  =  "1"  /* Excision of Lumbosacral Plexus, Percutaneous Approach */
"01BA4ZZ"  =  "1"  /* Excision of Lumbosacral Plexus, Perc Endo Approach */
"01BB0ZX"  =  "1"  /* Excision of Lumbar Nerve, Open Approach, Diagnostic */
"01BB0ZZ"  =  "1"  /* Excision of Lumbar Nerve, Open Approach */
"01BB3ZZ"  =  "1"  /* Excision of Lumbar Nerve, Percutaneous Approach */
"01BB4ZZ"  =  "1"  /* Excision of Lumbar Nerve, Percutaneous Endoscopic Approach */
"01BC0ZX"  =  "1"  /* Excision of Pudendal Nerve, Open Approach, Diagnostic */
"01BC0ZZ"  =  "1"  /* Excision of Pudendal Nerve, Open Approach */
"01BC3ZZ"  =  "1"  /* Excision of Pudendal Nerve, Percutaneous Approach */
"01BC4ZZ"  =  "1"  /* Excision of Pudendal Nerve, Percutaneous Endoscopic Approach */
"01BD0ZX"  =  "1"  /* Excision of Femoral Nerve, Open Approach, Diagnostic */
"01BD0ZZ"  =  "1"  /* Excision of Femoral Nerve, Open Approach */
"01BD3ZZ"  =  "1"  /* Excision of Femoral Nerve, Percutaneous Approach */
"01BD4ZZ"  =  "1"  /* Excision of Femoral Nerve, Percutaneous Endoscopic Approach */
"01BF0ZX"  =  "1"  /* Excision of Sciatic Nerve, Open Approach, Diagnostic */
"01BF0ZZ"  =  "1"  /* Excision of Sciatic Nerve, Open Approach */
"01BF3ZZ"  =  "1"  /* Excision of Sciatic Nerve, Percutaneous Approach */
"01BF4ZZ"  =  "1"  /* Excision of Sciatic Nerve, Percutaneous Endoscopic Approach */
"01BG0ZX"  =  "1"  /* Excision of Tibial Nerve, Open Approach, Diagnostic */
"01BG0ZZ"  =  "1"  /* Excision of Tibial Nerve, Open Approach */
"01BG3ZZ"  =  "1"  /* Excision of Tibial Nerve, Percutaneous Approach */
"01BG4ZZ"  =  "1"  /* Excision of Tibial Nerve, Percutaneous Endoscopic Approach */
"01BH0ZX"  =  "1"  /* Excision of Peroneal Nerve, Open Approach, Diagnostic */
"01BH0ZZ"  =  "1"  /* Excision of Peroneal Nerve, Open Approach */
"01BH3ZZ"  =  "1"  /* Excision of Peroneal Nerve, Percutaneous Approach */
"01BH4ZZ"  =  "1"  /* Excision of Peroneal Nerve, Percutaneous Endoscopic Approach */
"01BK0ZX"  =  "1"  /* Excision of Head Neck Symp Nrv, Open Approach, Diagn */
"01BK0ZZ"  =  "1"  /* Excision of Head and Neck Sympathetic Nerve, Open Approach */
"01BK3ZX"  =  "1"  /* Excision of Head Neck Symp Nrv, Perc Approach, Diagn */
"01BK3ZZ"  =  "1"  /* Excision of Head and Neck Sympathetic Nerve, Perc Approach */
"01BK4ZX"  =  "1"  /* Excision of Head Neck Symp Nrv, Perc Endo Approach, Diagn */
"01BK4ZZ"  =  "1"  /* Excision of Head Neck Symp Nrv, Perc Endo Approach */
"01BL0ZX"  =  "1"  /* Excision of Thoracic Sympathetic Nerve, Open Approach, Diagn */
"01BL0ZZ"  =  "1"  /* Excision of Thoracic Sympathetic Nerve, Open Approach */
"01BL3ZX"  =  "1"  /* Excision of Thoracic Sympathetic Nerve, Perc Approach, Diagn */
"01BL3ZZ"  =  "1"  /* Excision of Thoracic Sympathetic Nerve, Perc Approach */
"01BL4ZX"  =  "1"  /* Excision of Thor Symp Nrv, Perc Endo Approach, Diagn */
"01BL4ZZ"  =  "1"  /* Excision of Thoracic Sympathetic Nerve, Perc Endo Approach */
"01BM0ZX"  =  "1"  /* Excision of Abd Symp Nrv, Open Approach, Diagn */
"01BM0ZZ"  =  "1"  /* Excision of Abdominal Sympathetic Nerve, Open Approach */
"01BM3ZX"  =  "1"  /* Excision of Abd Symp Nrv, Perc Approach, Diagn */
"01BM3ZZ"  =  "1"  /* Excision of Abdominal Sympathetic Nerve, Perc Approach */
"01BM4ZX"  =  "1"  /* Excision of Abd Symp Nrv, Perc Endo Approach, Diagn */
"01BM4ZZ"  =  "1"  /* Excision of Abdominal Sympathetic Nerve, Perc Endo Approach */
"01BN0ZX"  =  "1"  /* Excision of Lumbar Sympathetic Nerve, Open Approach, Diagn */
"01BN0ZZ"  =  "1"  /* Excision of Lumbar Sympathetic Nerve, Open Approach */
"01BN3ZX"  =  "1"  /* Excision of Lumbar Sympathetic Nerve, Perc Approach, Diagn */
"01BN3ZZ"  =  "1"  /* Excision of Lumbar Sympathetic Nerve, Percutaneous Approach */
"01BN4ZX"  =  "1"  /* Excision of Lumbar Symp Nrv, Perc Endo Approach, Diagn */
"01BN4ZZ"  =  "1"  /* Excision of Lumbar Sympathetic Nerve, Perc Endo Approach */
"01BP0ZX"  =  "1"  /* Excision of Sacral Sympathetic Nerve, Open Approach, Diagn */
"01BP0ZZ"  =  "1"  /* Excision of Sacral Sympathetic Nerve, Open Approach */
"01BP3ZX"  =  "1"  /* Excision of Sacral Sympathetic Nerve, Perc Approach, Diagn */
"01BP3ZZ"  =  "1"  /* Excision of Sacral Sympathetic Nerve, Percutaneous Approach */
"01BP4ZX"  =  "1"  /* Excision of Sacral Symp Nrv, Perc Endo Approach, Diagn */
"01BP4ZZ"  =  "1"  /* Excision of Sacral Sympathetic Nerve, Perc Endo Approach */
"01BQ0ZX"  =  "1"  /* Excision of Sacral Plexus, Open Approach, Diagnostic */
"01BQ0ZZ"  =  "1"  /* Excision of Sacral Plexus, Open Approach */
"01BQ3ZZ"  =  "1"  /* Excision of Sacral Plexus, Percutaneous Approach */
"01BQ4ZZ"  =  "1"  /* Excision of Sacral Plexus, Percutaneous Endoscopic Approach */
"01BR0ZX"  =  "1"  /* Excision of Sacral Nerve, Open Approach, Diagnostic */
"01BR0ZZ"  =  "1"  /* Excision of Sacral Nerve, Open Approach */
"01BR3ZZ"  =  "1"  /* Excision of Sacral Nerve, Percutaneous Approach */
"01BR4ZZ"  =  "1"  /* Excision of Sacral Nerve, Percutaneous Endoscopic Approach */
"01C00ZZ"  =  "1"  /* Extirpation of Matter from Cervical Plexus, Open Approach */
"01C03ZZ"  =  "1"  /* Extirpation of Matter from Cervical Plexus, Perc Approach */
"01C04ZZ"  =  "1"  /* Extirpation of Matter from Cerv Plexus, Perc Endo Approach */
"01C10ZZ"  =  "1"  /* Extirpation of Matter from Cervical Nerve, Open Approach */
"01C13ZZ"  =  "1"  /* Extirpation of Matter from Cervical Nerve, Perc Approach */
"01C14ZZ"  =  "1"  /* Extirpation of Matter from Cerv Nrv, Perc Endo Approach */
"01C20ZZ"  =  "1"  /* Extirpation of Matter from Phrenic Nerve, Open Approach */
"01C23ZZ"  =  "1"  /* Extirpation of Matter from Phrenic Nerve, Perc Approach */
"01C24ZZ"  =  "1"  /* Extirpation of Matter from Phrenic Nerve, Perc Endo Approach */
"01C30ZZ"  =  "1"  /* Extirpation of Matter from Brachial Plexus, Open Approach */
"01C33ZZ"  =  "1"  /* Extirpation of Matter from Brachial Plexus, Perc Approach */
"01C34ZZ"  =  "1"  /* Extirpation of Matter from Brach Plexus, Perc Endo Approach */
"01C40ZZ"  =  "1"  /* Extirpation of Matter from Ulnar Nerve, Open Approach */
"01C43ZZ"  =  "1"  /* Extirpation of Matter from Ulnar Nerve, Perc Approach */
"01C44ZZ"  =  "1"  /* Extirpation of Matter from Ulnar Nerve, Perc Endo Approach */
"01C50ZZ"  =  "1"  /* Extirpation of Matter from Median Nerve, Open Approach */
"01C53ZZ"  =  "1"  /* Extirpation of Matter from Median Nerve, Perc Approach */
"01C54ZZ"  =  "1"  /* Extirpation of Matter from Median Nerve, Perc Endo Approach */
"01C60ZZ"  =  "1"  /* Extirpation of Matter from Radial Nerve, Open Approach */
"01C63ZZ"  =  "1"  /* Extirpation of Matter from Radial Nerve, Perc Approach */
"01C64ZZ"  =  "1"  /* Extirpation of Matter from Radial Nerve, Perc Endo Approach */
"01C80ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Nerve, Open Approach */
"01C83ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Nerve, Perc Approach */
"01C84ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Nrv, Perc Endo Approach */
"01C90ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Plexus, Open Approach */
"01C93ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Plexus, Perc Approach */
"01C94ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Plexus, Perc Endo Approach */
"01CA0ZZ"  =  "1"  /* Extirpation of Matter from Lumbosacral Plexus, Open Approach */
"01CA3ZZ"  =  "1"  /* Extirpation of Matter from Lumbosacral Plexus, Perc Approach */
"01CA4ZZ"  =  "1"  /* Extirpate matter from Lumbosacr Plexus, Perc Endo */
"01CB0ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Nerve, Open Approach */
"01CB3ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Nerve, Perc Approach */
"01CB4ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Nerve, Perc Endo Approach */
"01CC0ZZ"  =  "1"  /* Extirpation of Matter from Pudendal Nerve, Open Approach */
"01CC3ZZ"  =  "1"  /* Extirpation of Matter from Pudendal Nerve, Perc Approach */
"01CC4ZZ"  =  "1"  /* Extirpation of Matter from Pudend Nrv, Perc Endo Approach */
"01CD0ZZ"  =  "1"  /* Extirpation of Matter from Femoral Nerve, Open Approach */
"01CD3ZZ"  =  "1"  /* Extirpation of Matter from Femoral Nerve, Perc Approach */
"01CD4ZZ"  =  "1"  /* Extirpation of Matter from Femoral Nerve, Perc Endo Approach */
"01CF0ZZ"  =  "1"  /* Extirpation of Matter from Sciatic Nerve, Open Approach */
"01CF3ZZ"  =  "1"  /* Extirpation of Matter from Sciatic Nerve, Perc Approach */
"01CF4ZZ"  =  "1"  /* Extirpation of Matter from Sciatic Nerve, Perc Endo Approach */
"01CG0ZZ"  =  "1"  /* Extirpation of Matter from Tibial Nerve, Open Approach */
"01CG3ZZ"  =  "1"  /* Extirpation of Matter from Tibial Nerve, Perc Approach */
"01CG4ZZ"  =  "1"  /* Extirpation of Matter from Tibial Nerve, Perc Endo Approach */
"01CH0ZZ"  =  "1"  /* Extirpation of Matter from Peroneal Nerve, Open Approach */
"01CH3ZZ"  =  "1"  /* Extirpation of Matter from Peroneal Nerve, Perc Approach */
"01CH4ZZ"  =  "1"  /* Extirpation of Matter from Peroneal Nrv, Perc Endo Approach */
"01CK0ZZ"  =  "1"  /* Extirpation of Matter from Head Neck Symp Nrv, Open Approach */
"01CK3ZZ"  =  "1"  /* Extirpation of Matter from Head Neck Symp Nrv, Perc Approach */
"01CK4ZZ"  =  "1"  /* Extirpate matter from Head Neck Symp Nrv, Perc Endo */
"01CL0ZZ"  =  "1"  /* Extirpation of Matter from Thor Symp Nrv, Open Approach */
"01CL3ZZ"  =  "1"  /* Extirpation of Matter from Thor Symp Nrv, Perc Approach */
"01CL4ZZ"  =  "1"  /* Extirpation of Matter from Thor Symp Nrv, Perc Endo Approach */
"01CM0ZZ"  =  "1"  /* Extirpation of Matter from Abd Symp Nrv, Open Approach */
"01CM3ZZ"  =  "1"  /* Extirpation of Matter from Abd Symp Nrv, Perc Approach */
"01CM4ZZ"  =  "1"  /* Extirpation of Matter from Abd Symp Nrv, Perc Endo Approach */
"01CN0ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Symp Nrv, Open Approach */
"01CN3ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Symp Nrv, Perc Approach */
"01CN4ZZ"  =  "1"  /* Extirpate of Matter from Lumbar Symp Nrv, Perc Endo Approach */
"01CP0ZZ"  =  "1"  /* Extirpation of Matter from Sacral Symp Nrv, Open Approach */
"01CP3ZZ"  =  "1"  /* Extirpation of Matter from Sacral Symp Nrv, Perc Approach */
"01CP4ZZ"  =  "1"  /* Extirpate of Matter from Sacral Symp Nrv, Perc Endo Approach */
"01CQ0ZZ"  =  "1"  /* Extirpation of Matter from Sacral Plexus, Open Approach */
"01CQ3ZZ"  =  "1"  /* Extirpation of Matter from Sacral Plexus, Perc Approach */
"01CQ4ZZ"  =  "1"  /* Extirpation of Matter from Sacral Plexus, Perc Endo Approach */
"01CR0ZZ"  =  "1"  /* Extirpation of Matter from Sacral Nerve, Open Approach */
"01CR3ZZ"  =  "1"  /* Extirpation of Matter from Sacral Nerve, Perc Approach */
"01CR4ZZ"  =  "1"  /* Extirpation of Matter from Sacral Nerve, Perc Endo Approach */
"01D00ZZ"  =  "1"  /* Extraction of Cervical Plexus, Open Approach */
"01D03ZZ"  =  "1"  /* Extraction of Cervical Plexus, Percutaneous Approach */
"01D04ZZ"  =  "1"  /* Extraction of Cervical Plexus, Perc Endo Approach */
"01D10ZZ"  =  "1"  /* Extraction of Cervical Nerve, Open Approach */
"01D13ZZ"  =  "1"  /* Extraction of Cervical Nerve, Percutaneous Approach */
"01D14ZZ"  =  "1"  /* Extraction of Cervical Nerve, Perc Endo Approach */
"01D20ZZ"  =  "1"  /* Extraction of Phrenic Nerve, Open Approach */
"01D23ZZ"  =  "1"  /* Extraction of Phrenic Nerve, Percutaneous Approach */
"01D24ZZ"  =  "1"  /* Extraction of Phrenic Nerve, Perc Endo Approach */
"01D30ZZ"  =  "1"  /* Extraction of Brachial Plexus, Open Approach */
"01D33ZZ"  =  "1"  /* Extraction of Brachial Plexus, Percutaneous Approach */
"01D34ZZ"  =  "1"  /* Extraction of Brachial Plexus, Perc Endo Approach */
"01D40ZZ"  =  "1"  /* Extraction of Ulnar Nerve, Open Approach */
"01D43ZZ"  =  "1"  /* Extraction of Ulnar Nerve, Percutaneous Approach */
"01D44ZZ"  =  "1"  /* Extraction of Ulnar Nerve, Percutaneous Endoscopic Approach */
"01D50ZZ"  =  "1"  /* Extraction of Median Nerve, Open Approach */
"01D53ZZ"  =  "1"  /* Extraction of Median Nerve, Percutaneous Approach */
"01D54ZZ"  =  "1"  /* Extraction of Median Nerve, Percutaneous Endoscopic Approach */
"01D60ZZ"  =  "1"  /* Extraction of Radial Nerve, Open Approach */
"01D63ZZ"  =  "1"  /* Extraction of Radial Nerve, Percutaneous Approach */
"01D64ZZ"  =  "1"  /* Extraction of Radial Nerve, Percutaneous Endoscopic Approach */
"01D80ZZ"  =  "1"  /* Extraction of Thoracic Nerve, Open Approach */
"01D83ZZ"  =  "1"  /* Extraction of Thoracic Nerve, Percutaneous Approach */
"01D84ZZ"  =  "1"  /* Extraction of Thoracic Nerve, Perc Endo Approach */
"01D90ZZ"  =  "1"  /* Extraction of Lumbar Plexus, Open Approach */
"01D93ZZ"  =  "1"  /* Extraction of Lumbar Plexus, Percutaneous Approach */
"01D94ZZ"  =  "1"  /* Extraction of Lumbar Plexus, Perc Endo Approach */
"01DA0ZZ"  =  "1"  /* Extraction of Lumbosacral Plexus, Open Approach */
"01DA3ZZ"  =  "1"  /* Extraction of Lumbosacral Plexus, Percutaneous Approach */
"01DA4ZZ"  =  "1"  /* Extraction of Lumbosacral Plexus, Perc Endo Approach */
"01DB0ZZ"  =  "1"  /* Extraction of Lumbar Nerve, Open Approach */
"01DB3ZZ"  =  "1"  /* Extraction of Lumbar Nerve, Percutaneous Approach */
"01DB4ZZ"  =  "1"  /* Extraction of Lumbar Nerve, Percutaneous Endoscopic Approach */
"01DC0ZZ"  =  "1"  /* Extraction of Pudendal Nerve, Open Approach */
"01DC3ZZ"  =  "1"  /* Extraction of Pudendal Nerve, Percutaneous Approach */
"01DC4ZZ"  =  "1"  /* Extraction of Pudendal Nerve, Perc Endo Approach */
"01DD0ZZ"  =  "1"  /* Extraction of Femoral Nerve, Open Approach */
"01DD3ZZ"  =  "1"  /* Extraction of Femoral Nerve, Percutaneous Approach */
"01DD4ZZ"  =  "1"  /* Extraction of Femoral Nerve, Perc Endo Approach */
"01DF0ZZ"  =  "1"  /* Extraction of Sciatic Nerve, Open Approach */
"01DF3ZZ"  =  "1"  /* Extraction of Sciatic Nerve, Percutaneous Approach */
"01DF4ZZ"  =  "1"  /* Extraction of Sciatic Nerve, Perc Endo Approach */
"01DG0ZZ"  =  "1"  /* Extraction of Tibial Nerve, Open Approach */
"01DG3ZZ"  =  "1"  /* Extraction of Tibial Nerve, Percutaneous Approach */
"01DG4ZZ"  =  "1"  /* Extraction of Tibial Nerve, Percutaneous Endoscopic Approach */
"01DH0ZZ"  =  "1"  /* Extraction of Peroneal Nerve, Open Approach */
"01DH3ZZ"  =  "1"  /* Extraction of Peroneal Nerve, Percutaneous Approach */
"01DH4ZZ"  =  "1"  /* Extraction of Peroneal Nerve, Perc Endo Approach */
"01DK0ZZ"  =  "1"  /* Extraction of Head and Neck Sympathetic Nerve, Open Approach */
"01DK3ZZ"  =  "1"  /* Extraction of Head and Neck Sympathetic Nerve, Perc Approach */
"01DK4ZZ"  =  "1"  /* Extraction of Head Neck Symp Nrv, Perc Endo Approach */
"01DL0ZZ"  =  "1"  /* Extraction of Thoracic Sympathetic Nerve, Open Approach */
"01DL3ZZ"  =  "1"  /* Extraction of Thoracic Sympathetic Nerve, Perc Approach */
"01DL4ZZ"  =  "1"  /* Extraction of Thoracic Sympathetic Nerve, Perc Endo Approach */
"01DM0ZZ"  =  "1"  /* Extraction of Abdominal Sympathetic Nerve, Open Approach */
"01DM3ZZ"  =  "1"  /* Extraction of Abdominal Sympathetic Nerve, Perc Approach */
"01DM4ZZ"  =  "1"  /* Extraction of Abd Symp Nrv, Perc Endo Approach */
"01DN0ZZ"  =  "1"  /* Extraction of Lumbar Sympathetic Nerve, Open Approach */
"01DN3ZZ"  =  "1"  /* Extraction of Lumbar Sympathetic Nerve, Perc Approach */
"01DN4ZZ"  =  "1"  /* Extraction of Lumbar Sympathetic Nerve, Perc Endo Approach */
"01DP0ZZ"  =  "1"  /* Extraction of Sacral Sympathetic Nerve, Open Approach */
"01DP3ZZ"  =  "1"  /* Extraction of Sacral Sympathetic Nerve, Perc Approach */
"01DP4ZZ"  =  "1"  /* Extraction of Sacral Sympathetic Nerve, Perc Endo Approach */
"01DQ0ZZ"  =  "1"  /* Extraction of Sacral Plexus, Open Approach */
"01DQ3ZZ"  =  "1"  /* Extraction of Sacral Plexus, Percutaneous Approach */
"01DQ4ZZ"  =  "1"  /* Extraction of Sacral Plexus, Perc Endo Approach */
"01DR0ZZ"  =  "1"  /* Extraction of Sacral Nerve, Open Approach */
"01DR3ZZ"  =  "1"  /* Extraction of Sacral Nerve, Percutaneous Approach */
"01DR4ZZ"  =  "1"  /* Extraction of Sacral Nerve, Percutaneous Endoscopic Approach */
"01HY02Z"  =  "1"  /* Insertion of Monitor Dev into Periph Nrv, Open Approach */
"01HY0MZ"  =  "1"  /* Insertion of Neuro Lead into Periph Nrv, Open Approach */
"01HY32Z"  =  "1"  /* Insertion of Monitor Dev into Periph Nrv, Perc Approach */
"01HY3MZ"  =  "1"  /* Insertion of Neuro Lead into Periph Nrv, Perc Approach */
"01HY42Z"  =  "1"  /* Insertion of Monitor Dev into Periph Nrv, Perc Endo Approach */
"01HY4MZ"  =  "1"  /* Insertion of Neuro Lead into Periph Nrv, Perc Endo Approach */
"01JY0ZZ"  =  "1"  /* Inspection of Peripheral Nerve, Open Approach */
"01JY3ZZ"  =  "1"  /* Inspection of Peripheral Nerve, Percutaneous Approach */
"01JY4ZZ"  =  "1"  /* Inspection of Peripheral Nerve, Perc Endo Approach */
"01N00ZZ"  =  "1"  /* Release Cervical Plexus, Open Approach */
"01N03ZZ"  =  "1"  /* Release Cervical Plexus, Percutaneous Approach */
"01N04ZZ"  =  "1"  /* Release Cervical Plexus, Percutaneous Endoscopic Approach */
"01N10ZZ"  =  "1"  /* Release Cervical Nerve, Open Approach */
"01N13ZZ"  =  "1"  /* Release Cervical Nerve, Percutaneous Approach */
"01N14ZZ"  =  "1"  /* Release Cervical Nerve, Percutaneous Endoscopic Approach */
"01N20ZZ"  =  "1"  /* Release Phrenic Nerve, Open Approach */
"01N23ZZ"  =  "1"  /* Release Phrenic Nerve, Percutaneous Approach */
"01N24ZZ"  =  "1"  /* Release Phrenic Nerve, Percutaneous Endoscopic Approach */
"01N30ZZ"  =  "1"  /* Release Brachial Plexus, Open Approach */
"01N33ZZ"  =  "1"  /* Release Brachial Plexus, Percutaneous Approach */
"01N34ZZ"  =  "1"  /* Release Brachial Plexus, Percutaneous Endoscopic Approach */
"01N40ZZ"  =  "1"  /* Release Ulnar Nerve, Open Approach */
"01N43ZZ"  =  "1"  /* Release Ulnar Nerve, Percutaneous Approach */
"01N44ZZ"  =  "1"  /* Release Ulnar Nerve, Percutaneous Endoscopic Approach */
"01N50ZZ"  =  "1"  /* Release Median Nerve, Open Approach */
"01N53ZZ"  =  "1"  /* Release Median Nerve, Percutaneous Approach */
"01N54ZZ"  =  "1"  /* Release Median Nerve, Percutaneous Endoscopic Approach */
"01N60ZZ"  =  "1"  /* Release Radial Nerve, Open Approach */
"01N63ZZ"  =  "1"  /* Release Radial Nerve, Percutaneous Approach */
"01N64ZZ"  =  "1"  /* Release Radial Nerve, Percutaneous Endoscopic Approach */
"01N80ZZ"  =  "1"  /* Release Thoracic Nerve, Open Approach */
"01N83ZZ"  =  "1"  /* Release Thoracic Nerve, Percutaneous Approach */
"01N84ZZ"  =  "1"  /* Release Thoracic Nerve, Percutaneous Endoscopic Approach */
"01N90ZZ"  =  "1"  /* Release Lumbar Plexus, Open Approach */
"01N93ZZ"  =  "1"  /* Release Lumbar Plexus, Percutaneous Approach */
"01N94ZZ"  =  "1"  /* Release Lumbar Plexus, Percutaneous Endoscopic Approach */
"01NA0ZZ"  =  "1"  /* Release Lumbosacral Plexus, Open Approach */
"01NA3ZZ"  =  "1"  /* Release Lumbosacral Plexus, Percutaneous Approach */
"01NA4ZZ"  =  "1"  /* Release Lumbosacral Plexus, Percutaneous Endoscopic Approach */
"01NB0ZZ"  =  "1"  /* Release Lumbar Nerve, Open Approach */
"01NB3ZZ"  =  "1"  /* Release Lumbar Nerve, Percutaneous Approach */
"01NB4ZZ"  =  "1"  /* Release Lumbar Nerve, Percutaneous Endoscopic Approach */
"01NC0ZZ"  =  "1"  /* Release Pudendal Nerve, Open Approach */
"01NC3ZZ"  =  "1"  /* Release Pudendal Nerve, Percutaneous Approach */
"01NC4ZZ"  =  "1"  /* Release Pudendal Nerve, Percutaneous Endoscopic Approach */
"01ND0ZZ"  =  "1"  /* Release Femoral Nerve, Open Approach */
"01ND3ZZ"  =  "1"  /* Release Femoral Nerve, Percutaneous Approach */
"01ND4ZZ"  =  "1"  /* Release Femoral Nerve, Percutaneous Endoscopic Approach */
"01NF0ZZ"  =  "1"  /* Release Sciatic Nerve, Open Approach */
"01NF3ZZ"  =  "1"  /* Release Sciatic Nerve, Percutaneous Approach */
"01NF4ZZ"  =  "1"  /* Release Sciatic Nerve, Percutaneous Endoscopic Approach */
"01NG0ZZ"  =  "1"  /* Release Tibial Nerve, Open Approach */
"01NG3ZZ"  =  "1"  /* Release Tibial Nerve, Percutaneous Approach */
"01NG4ZZ"  =  "1"  /* Release Tibial Nerve, Percutaneous Endoscopic Approach */
"01NH0ZZ"  =  "1"  /* Release Peroneal Nerve, Open Approach */
"01NH3ZZ"  =  "1"  /* Release Peroneal Nerve, Percutaneous Approach */
"01NH4ZZ"  =  "1"  /* Release Peroneal Nerve, Percutaneous Endoscopic Approach */
"01NK0ZZ"  =  "1"  /* Release Head and Neck Sympathetic Nerve, Open Approach */
"01NK3ZZ"  =  "1"  /* Release Head and Neck Sympathetic Nerve, Perc Approach */
"01NK4ZZ"  =  "1"  /* Release Head and Neck Sympathetic Nerve, Perc Endo Approach */
"01NL0ZZ"  =  "1"  /* Release Thoracic Sympathetic Nerve, Open Approach */
"01NL3ZZ"  =  "1"  /* Release Thoracic Sympathetic Nerve, Percutaneous Approach */
"01NL4ZZ"  =  "1"  /* Release Thoracic Sympathetic Nerve, Perc Endo Approach */
"01NM0ZZ"  =  "1"  /* Release Abdominal Sympathetic Nerve, Open Approach */
"01NM3ZZ"  =  "1"  /* Release Abdominal Sympathetic Nerve, Percutaneous Approach */
"01NM4ZZ"  =  "1"  /* Release Abdominal Sympathetic Nerve, Perc Endo Approach */
"01NN0ZZ"  =  "1"  /* Release Lumbar Sympathetic Nerve, Open Approach */
"01NN3ZZ"  =  "1"  /* Release Lumbar Sympathetic Nerve, Percutaneous Approach */
"01NN4ZZ"  =  "1"  /* Release Lumbar Sympathetic Nerve, Perc Endo Approach */
"01NP0ZZ"  =  "1"  /* Release Sacral Sympathetic Nerve, Open Approach */
"01NP3ZZ"  =  "1"  /* Release Sacral Sympathetic Nerve, Percutaneous Approach */
"01NP4ZZ"  =  "1"  /* Release Sacral Sympathetic Nerve, Perc Endo Approach */
"01NQ0ZZ"  =  "1"  /* Release Sacral Plexus, Open Approach */
"01NQ3ZZ"  =  "1"  /* Release Sacral Plexus, Percutaneous Approach */
"01NQ4ZZ"  =  "1"  /* Release Sacral Plexus, Percutaneous Endoscopic Approach */
"01NR0ZZ"  =  "1"  /* Release Sacral Nerve, Open Approach */
"01NR3ZZ"  =  "1"  /* Release Sacral Nerve, Percutaneous Approach */
"01NR4ZZ"  =  "1"  /* Release Sacral Nerve, Percutaneous Endoscopic Approach */
"01PY00Z"  =  "1"  /* Removal of Drainage Device from Periph Nrv, Open Approach */
"01PY02Z"  =  "1"  /* Removal of Monitoring Device from Periph Nrv, Open Approach */
"01PY07Z"  =  "1"  /* Removal of Autol Sub from Periph Nrv, Open Approach */
"01PY0MZ"  =  "1"  /* Removal of Neuro Lead from Periph Nrv, Open Approach */
"01PY30Z"  =  "1"  /* Removal of Drainage Device from Periph Nrv, Perc Approach */
"01PY32Z"  =  "1"  /* Removal of Monitoring Device from Periph Nrv, Perc Approach */
"01PY37Z"  =  "1"  /* Removal of Autol Sub from Periph Nrv, Perc Approach */
"01PY3MZ"  =  "1"  /* Removal of Neuro Lead from Periph Nrv, Perc Approach */
"01PY40Z"  =  "1"  /* Removal of Drain Dev from Periph Nrv, Perc Endo Approach */
"01PY42Z"  =  "1"  /* Removal of Monitor Dev from Periph Nrv, Perc Endo Approach */
"01PY47Z"  =  "1"  /* Removal of Autol Sub from Periph Nrv, Perc Endo Approach */
"01PY4MZ"  =  "1"  /* Removal of Neuro Lead from Periph Nrv, Perc Endo Approach */
"01PYXMZ"  =  "1"  /* Removal of Neuro Lead from Periph Nrv, Extern Approach */
"01Q00ZZ"  =  "1"  /* Repair Cervical Plexus, Open Approach */
"01Q03ZZ"  =  "1"  /* Repair Cervical Plexus, Percutaneous Approach */
"01Q04ZZ"  =  "1"  /* Repair Cervical Plexus, Percutaneous Endoscopic Approach */
"01Q10ZZ"  =  "1"  /* Repair Cervical Nerve, Open Approach */
"01Q13ZZ"  =  "1"  /* Repair Cervical Nerve, Percutaneous Approach */
"01Q14ZZ"  =  "1"  /* Repair Cervical Nerve, Percutaneous Endoscopic Approach */
"01Q20ZZ"  =  "1"  /* Repair Phrenic Nerve, Open Approach */
"01Q23ZZ"  =  "1"  /* Repair Phrenic Nerve, Percutaneous Approach */
"01Q24ZZ"  =  "1"  /* Repair Phrenic Nerve, Percutaneous Endoscopic Approach */
"01Q30ZZ"  =  "1"  /* Repair Brachial Plexus, Open Approach */
"01Q33ZZ"  =  "1"  /* Repair Brachial Plexus, Percutaneous Approach */
"01Q34ZZ"  =  "1"  /* Repair Brachial Plexus, Percutaneous Endoscopic Approach */
"01Q40ZZ"  =  "1"  /* Repair Ulnar Nerve, Open Approach */
"01Q43ZZ"  =  "1"  /* Repair Ulnar Nerve, Percutaneous Approach */
"01Q44ZZ"  =  "1"  /* Repair Ulnar Nerve, Percutaneous Endoscopic Approach */
"01Q50ZZ"  =  "1"  /* Repair Median Nerve, Open Approach */
"01Q53ZZ"  =  "1"  /* Repair Median Nerve, Percutaneous Approach */
"01Q54ZZ"  =  "1"  /* Repair Median Nerve, Percutaneous Endoscopic Approach */
"01Q60ZZ"  =  "1"  /* Repair Radial Nerve, Open Approach */
"01Q63ZZ"  =  "1"  /* Repair Radial Nerve, Percutaneous Approach */
"01Q64ZZ"  =  "1"  /* Repair Radial Nerve, Percutaneous Endoscopic Approach */
"01Q80ZZ"  =  "1"  /* Repair Thoracic Nerve, Open Approach */
"01Q83ZZ"  =  "1"  /* Repair Thoracic Nerve, Percutaneous Approach */
"01Q84ZZ"  =  "1"  /* Repair Thoracic Nerve, Percutaneous Endoscopic Approach */
"01Q90ZZ"  =  "1"  /* Repair Lumbar Plexus, Open Approach */
"01Q93ZZ"  =  "1"  /* Repair Lumbar Plexus, Percutaneous Approach */
"01Q94ZZ"  =  "1"  /* Repair Lumbar Plexus, Percutaneous Endoscopic Approach */
"01QA0ZZ"  =  "1"  /* Repair Lumbosacral Plexus, Open Approach */
"01QA3ZZ"  =  "1"  /* Repair Lumbosacral Plexus, Percutaneous Approach */
"01QA4ZZ"  =  "1"  /* Repair Lumbosacral Plexus, Percutaneous Endoscopic Approach */
"01QB0ZZ"  =  "1"  /* Repair Lumbar Nerve, Open Approach */
"01QB3ZZ"  =  "1"  /* Repair Lumbar Nerve, Percutaneous Approach */
"01QB4ZZ"  =  "1"  /* Repair Lumbar Nerve, Percutaneous Endoscopic Approach */
"01QC0ZZ"  =  "1"  /* Repair Pudendal Nerve, Open Approach */
"01QC3ZZ"  =  "1"  /* Repair Pudendal Nerve, Percutaneous Approach */
"01QC4ZZ"  =  "1"  /* Repair Pudendal Nerve, Percutaneous Endoscopic Approach */
"01QD0ZZ"  =  "1"  /* Repair Femoral Nerve, Open Approach */
"01QD3ZZ"  =  "1"  /* Repair Femoral Nerve, Percutaneous Approach */
"01QD4ZZ"  =  "1"  /* Repair Femoral Nerve, Percutaneous Endoscopic Approach */
"01QF0ZZ"  =  "1"  /* Repair Sciatic Nerve, Open Approach */
"01QF3ZZ"  =  "1"  /* Repair Sciatic Nerve, Percutaneous Approach */
"01QF4ZZ"  =  "1"  /* Repair Sciatic Nerve, Percutaneous Endoscopic Approach */
"01QG0ZZ"  =  "1"  /* Repair Tibial Nerve, Open Approach */
"01QG3ZZ"  =  "1"  /* Repair Tibial Nerve, Percutaneous Approach */
"01QG4ZZ"  =  "1"  /* Repair Tibial Nerve, Percutaneous Endoscopic Approach */
"01QH0ZZ"  =  "1"  /* Repair Peroneal Nerve, Open Approach */
"01QH3ZZ"  =  "1"  /* Repair Peroneal Nerve, Percutaneous Approach */
"01QH4ZZ"  =  "1"  /* Repair Peroneal Nerve, Percutaneous Endoscopic Approach */
"01QK0ZZ"  =  "1"  /* Repair Head and Neck Sympathetic Nerve, Open Approach */
"01QK3ZZ"  =  "1"  /* Repair Head and Neck Sympathetic Nerve, Perc Approach */
"01QK4ZZ"  =  "1"  /* Repair Head and Neck Sympathetic Nerve, Perc Endo Approach */
"01QL0ZZ"  =  "1"  /* Repair Thoracic Sympathetic Nerve, Open Approach */
"01QL3ZZ"  =  "1"  /* Repair Thoracic Sympathetic Nerve, Percutaneous Approach */
"01QL4ZZ"  =  "1"  /* Repair Thoracic Sympathetic Nerve, Perc Endo Approach */
"01QM0ZZ"  =  "1"  /* Repair Abdominal Sympathetic Nerve, Open Approach */
"01QM3ZZ"  =  "1"  /* Repair Abdominal Sympathetic Nerve, Percutaneous Approach */
"01QM4ZZ"  =  "1"  /* Repair Abdominal Sympathetic Nerve, Perc Endo Approach */
"01QN0ZZ"  =  "1"  /* Repair Lumbar Sympathetic Nerve, Open Approach */
"01QN3ZZ"  =  "1"  /* Repair Lumbar Sympathetic Nerve, Percutaneous Approach */
"01QN4ZZ"  =  "1"  /* Repair Lumbar Sympathetic Nerve, Perc Endo Approach */
"01QP0ZZ"  =  "1"  /* Repair Sacral Sympathetic Nerve, Open Approach */
"01QP3ZZ"  =  "1"  /* Repair Sacral Sympathetic Nerve, Percutaneous Approach */
"01QP4ZZ"  =  "1"  /* Repair Sacral Sympathetic Nerve, Perc Endo Approach */
"01QQ0ZZ"  =  "1"  /* Repair Sacral Plexus, Open Approach */
"01QQ3ZZ"  =  "1"  /* Repair Sacral Plexus, Percutaneous Approach */
"01QQ4ZZ"  =  "1"  /* Repair Sacral Plexus, Percutaneous Endoscopic Approach */
"01QR0ZZ"  =  "1"  /* Repair Sacral Nerve, Open Approach */
"01QR3ZZ"  =  "1"  /* Repair Sacral Nerve, Percutaneous Approach */
"01QR4ZZ"  =  "1"  /* Repair Sacral Nerve, Percutaneous Endoscopic Approach */
"01S00ZZ"  =  "1"  /* Reposition Cervical Plexus, Open Approach */
"01S03ZZ"  =  "1"  /* Reposition Cervical Plexus, Percutaneous Approach */
"01S04ZZ"  =  "1"  /* Reposition Cervical Plexus, Percutaneous Endoscopic Approach */
"01S10ZZ"  =  "1"  /* Reposition Cervical Nerve, Open Approach */
"01S13ZZ"  =  "1"  /* Reposition Cervical Nerve, Percutaneous Approach */
"01S14ZZ"  =  "1"  /* Reposition Cervical Nerve, Percutaneous Endoscopic Approach */
"01S20ZZ"  =  "1"  /* Reposition Phrenic Nerve, Open Approach */
"01S23ZZ"  =  "1"  /* Reposition Phrenic Nerve, Percutaneous Approach */
"01S24ZZ"  =  "1"  /* Reposition Phrenic Nerve, Percutaneous Endoscopic Approach */
"01S30ZZ"  =  "1"  /* Reposition Brachial Plexus, Open Approach */
"01S33ZZ"  =  "1"  /* Reposition Brachial Plexus, Percutaneous Approach */
"01S34ZZ"  =  "1"  /* Reposition Brachial Plexus, Percutaneous Endoscopic Approach */
"01S40ZZ"  =  "1"  /* Reposition Ulnar Nerve, Open Approach */
"01S43ZZ"  =  "1"  /* Reposition Ulnar Nerve, Percutaneous Approach */
"01S44ZZ"  =  "1"  /* Reposition Ulnar Nerve, Percutaneous Endoscopic Approach */
"01S50ZZ"  =  "1"  /* Reposition Median Nerve, Open Approach */
"01S53ZZ"  =  "1"  /* Reposition Median Nerve, Percutaneous Approach */
"01S54ZZ"  =  "1"  /* Reposition Median Nerve, Percutaneous Endoscopic Approach */
"01S60ZZ"  =  "1"  /* Reposition Radial Nerve, Open Approach */
"01S63ZZ"  =  "1"  /* Reposition Radial Nerve, Percutaneous Approach */
"01S64ZZ"  =  "1"  /* Reposition Radial Nerve, Percutaneous Endoscopic Approach */
"01S80ZZ"  =  "1"  /* Reposition Thoracic Nerve, Open Approach */
"01S83ZZ"  =  "1"  /* Reposition Thoracic Nerve, Percutaneous Approach */
"01S84ZZ"  =  "1"  /* Reposition Thoracic Nerve, Percutaneous Endoscopic Approach */
"01S90ZZ"  =  "1"  /* Reposition Lumbar Plexus, Open Approach */
"01S93ZZ"  =  "1"  /* Reposition Lumbar Plexus, Percutaneous Approach */
"01S94ZZ"  =  "1"  /* Reposition Lumbar Plexus, Percutaneous Endoscopic Approach */
"01SA0ZZ"  =  "1"  /* Reposition Lumbosacral Plexus, Open Approach */
"01SA3ZZ"  =  "1"  /* Reposition Lumbosacral Plexus, Percutaneous Approach */
"01SA4ZZ"  =  "1"  /* Reposition Lumbosacral Plexus, Perc Endo Approach */
"01SB0ZZ"  =  "1"  /* Reposition Lumbar Nerve, Open Approach */
"01SB3ZZ"  =  "1"  /* Reposition Lumbar Nerve, Percutaneous Approach */
"01SB4ZZ"  =  "1"  /* Reposition Lumbar Nerve, Percutaneous Endoscopic Approach */
"01SC0ZZ"  =  "1"  /* Reposition Pudendal Nerve, Open Approach */
"01SC3ZZ"  =  "1"  /* Reposition Pudendal Nerve, Percutaneous Approach */
"01SC4ZZ"  =  "1"  /* Reposition Pudendal Nerve, Percutaneous Endoscopic Approach */
"01SD0ZZ"  =  "1"  /* Reposition Femoral Nerve, Open Approach */
"01SD3ZZ"  =  "1"  /* Reposition Femoral Nerve, Percutaneous Approach */
"01SD4ZZ"  =  "1"  /* Reposition Femoral Nerve, Percutaneous Endoscopic Approach */
"01SF0ZZ"  =  "1"  /* Reposition Sciatic Nerve, Open Approach */
"01SF3ZZ"  =  "1"  /* Reposition Sciatic Nerve, Percutaneous Approach */
"01SF4ZZ"  =  "1"  /* Reposition Sciatic Nerve, Percutaneous Endoscopic Approach */
"01SG0ZZ"  =  "1"  /* Reposition Tibial Nerve, Open Approach */
"01SG3ZZ"  =  "1"  /* Reposition Tibial Nerve, Percutaneous Approach */
"01SG4ZZ"  =  "1"  /* Reposition Tibial Nerve, Percutaneous Endoscopic Approach */
"01SH0ZZ"  =  "1"  /* Reposition Peroneal Nerve, Open Approach */
"01SH3ZZ"  =  "1"  /* Reposition Peroneal Nerve, Percutaneous Approach */
"01SH4ZZ"  =  "1"  /* Reposition Peroneal Nerve, Percutaneous Endoscopic Approach */
"01SQ0ZZ"  =  "1"  /* Reposition Sacral Plexus, Open Approach */
"01SQ3ZZ"  =  "1"  /* Reposition Sacral Plexus, Percutaneous Approach */
"01SQ4ZZ"  =  "1"  /* Reposition Sacral Plexus, Percutaneous Endoscopic Approach */
"01SR0ZZ"  =  "1"  /* Reposition Sacral Nerve, Open Approach */
"01SR3ZZ"  =  "1"  /* Reposition Sacral Nerve, Percutaneous Approach */
"01SR4ZZ"  =  "1"  /* Reposition Sacral Nerve, Percutaneous Endoscopic Approach */
"01U107Z"  =  "1"  /* Supplement Cervical Nerve with Autol Sub, Open Approach */
"01U137Z"  =  "1"  /* Supplement Cervical Nerve with Autol Sub, Perc Approach */
"01U147Z"  =  "1"  /* Supplement Cervical Nerve with Autol Sub, Perc Endo Approach */
"01U207Z"  =  "1"  /* Supplement Phrenic Nerve with Autol Sub, Open Approach */
"01U237Z"  =  "1"  /* Supplement Phrenic Nerve with Autol Sub, Perc Approach */
"01U247Z"  =  "1"  /* Supplement Phrenic Nerve with Autol Sub, Perc Endo Approach */
"01U407Z"  =  "1"  /* Supplement Ulnar Nerve with Autol Sub, Open Approach */
"01U437Z"  =  "1"  /* Supplement Ulnar Nerve with Autol Sub, Perc Approach */
"01U447Z"  =  "1"  /* Supplement Ulnar Nerve with Autol Sub, Perc Endo Approach */
"01U507Z"  =  "1"  /* Supplement Median Nerve with Autol Sub, Open Approach */
"01U537Z"  =  "1"  /* Supplement Median Nerve with Autol Sub, Perc Approach */
"01U547Z"  =  "1"  /* Supplement Median Nerve with Autol Sub, Perc Endo Approach */
"01U607Z"  =  "1"  /* Supplement Radial Nerve with Autol Sub, Open Approach */
"01U637Z"  =  "1"  /* Supplement Radial Nerve with Autol Sub, Perc Approach */
"01U647Z"  =  "1"  /* Supplement Radial Nerve with Autol Sub, Perc Endo Approach */
"01U807Z"  =  "1"  /* Supplement Thoracic Nerve with Autol Sub, Open Approach */
"01U837Z"  =  "1"  /* Supplement Thoracic Nerve with Autol Sub, Perc Approach */
"01U847Z"  =  "1"  /* Supplement Thoracic Nerve with Autol Sub, Perc Endo Approach */
"01UB07Z"  =  "1"  /* Supplement Lumbar Nerve with Autol Sub, Open Approach */
"01UB37Z"  =  "1"  /* Supplement Lumbar Nerve with Autol Sub, Perc Approach */
"01UB47Z"  =  "1"  /* Supplement Lumbar Nerve with Autol Sub, Perc Endo Approach */
"01UC07Z"  =  "1"  /* Supplement Pudendal Nerve with Autol Sub, Open Approach */
"01UC37Z"  =  "1"  /* Supplement Pudendal Nerve with Autol Sub, Perc Approach */
"01UC47Z"  =  "1"  /* Supplement Pudendal Nerve with Autol Sub, Perc Endo Approach */
"01UD07Z"  =  "1"  /* Supplement Femoral Nerve with Autol Sub, Open Approach */
"01UD37Z"  =  "1"  /* Supplement Femoral Nerve with Autol Sub, Perc Approach */
"01UD47Z"  =  "1"  /* Supplement Femoral Nerve with Autol Sub, Perc Endo Approach */
"01UF07Z"  =  "1"  /* Supplement Sciatic Nerve with Autol Sub, Open Approach */
"01UF37Z"  =  "1"  /* Supplement Sciatic Nerve with Autol Sub, Perc Approach */
"01UF47Z"  =  "1"  /* Supplement Sciatic Nerve with Autol Sub, Perc Endo Approach */
"01UG07Z"  =  "1"  /* Supplement Tibial Nerve with Autol Sub, Open Approach */
"01UG37Z"  =  "1"  /* Supplement Tibial Nerve with Autol Sub, Perc Approach */
"01UG47Z"  =  "1"  /* Supplement Tibial Nerve with Autol Sub, Perc Endo Approach */
"01UH07Z"  =  "1"  /* Supplement Peroneal Nerve with Autol Sub, Open Approach */
"01UH37Z"  =  "1"  /* Supplement Peroneal Nerve with Autol Sub, Perc Approach */
"01UH47Z"  =  "1"  /* Supplement Peroneal Nerve with Autol Sub, Perc Endo Approach */
"01UR07Z"  =  "1"  /* Supplement Sacral Nerve with Autol Sub, Open Approach */
"01UR37Z"  =  "1"  /* Supplement Sacral Nerve with Autol Sub, Perc Approach */
"01UR47Z"  =  "1"  /* Supplement Sacral Nerve with Autol Sub, Perc Endo Approach */
"01WY00Z"  =  "1"  /* Revision of Drainage Device in Periph Nrv, Open Approach */
"01WY02Z"  =  "1"  /* Revision of Monitoring Device in Periph Nrv, Open Approach */
"01WY07Z"  =  "1"  /* Revision of Autol Sub in Periph Nrv, Open Approach */
"01WY0MZ"  =  "1"  /* Revision of Neuro Lead in Periph Nrv, Open Approach */
"01WY30Z"  =  "1"  /* Revision of Drainage Device in Periph Nrv, Perc Approach */
"01WY32Z"  =  "1"  /* Revision of Monitoring Device in Periph Nrv, Perc Approach */
"01WY37Z"  =  "1"  /* Revision of Autol Sub in Periph Nrv, Perc Approach */
"01WY3MZ"  =  "1"  /* Revision of Neuro Lead in Periph Nrv, Perc Approach */
"01WY40Z"  =  "1"  /* Revision of Drain Dev in Periph Nrv, Perc Endo Approach */
"01WY42Z"  =  "1"  /* Revision of Monitor Dev in Periph Nrv, Perc Endo Approach */
"01WY47Z"  =  "1"  /* Revision of Autol Sub in Periph Nrv, Perc Endo Approach */
"01WY4MZ"  =  "1"  /* Revision of Neuro Lead in Periph Nrv, Perc Endo Approach */
"01X10Z1"  =  "1"  /* Transfer Cervical Nerve to Cervical Nerve, Open Approach */
"01X10Z2"  =  "1"  /* Transfer Cervical Nerve to Phrenic Nerve, Open Approach */
"01X14Z1"  =  "1"  /* Transfer Cervical Nerve to Cervical Nrv, Perc Endo Approach */
"01X14Z2"  =  "1"  /* Transfer Cervical Nerve to Phrenic Nerve, Perc Endo Approach */
"01X20Z1"  =  "1"  /* Transfer Phrenic Nerve to Cervical Nerve, Open Approach */
"01X20Z2"  =  "1"  /* Transfer Phrenic Nerve to Phrenic Nerve, Open Approach */
"01X24Z1"  =  "1"  /* Transfer Phrenic Nerve to Cervical Nerve, Perc Endo Approach */
"01X24Z2"  =  "1"  /* Transfer Phrenic Nerve to Phrenic Nerve, Perc Endo Approach */
"01X40Z4"  =  "1"  /* Transfer Ulnar Nerve to Ulnar Nerve, Open Approach */
"01X40Z5"  =  "1"  /* Transfer Ulnar Nerve to Median Nerve, Open Approach */
"01X40Z6"  =  "1"  /* Transfer Ulnar Nerve to Radial Nerve, Open Approach */
"01X44Z4"  =  "1"  /* Transfer Ulnar Nerve to Ulnar Nerve, Perc Endo Approach */
"01X44Z5"  =  "1"  /* Transfer Ulnar Nerve to Median Nerve, Perc Endo Approach */
"01X44Z6"  =  "1"  /* Transfer Ulnar Nerve to Radial Nerve, Perc Endo Approach */
"01X50Z4"  =  "1"  /* Transfer Median Nerve to Ulnar Nerve, Open Approach */
"01X50Z5"  =  "1"  /* Transfer Median Nerve to Median Nerve, Open Approach */
"01X50Z6"  =  "1"  /* Transfer Median Nerve to Radial Nerve, Open Approach */
"01X54Z4"  =  "1"  /* Transfer Median Nerve to Ulnar Nerve, Perc Endo Approach */
"01X54Z5"  =  "1"  /* Transfer Median Nerve to Median Nerve, Perc Endo Approach */
"01X54Z6"  =  "1"  /* Transfer Median Nerve to Radial Nerve, Perc Endo Approach */
"01X60Z4"  =  "1"  /* Transfer Radial Nerve to Ulnar Nerve, Open Approach */
"01X60Z5"  =  "1"  /* Transfer Radial Nerve to Median Nerve, Open Approach */
"01X60Z6"  =  "1"  /* Transfer Radial Nerve to Radial Nerve, Open Approach */
"01X64Z4"  =  "1"  /* Transfer Radial Nerve to Ulnar Nerve, Perc Endo Approach */
"01X64Z5"  =  "1"  /* Transfer Radial Nerve to Median Nerve, Perc Endo Approach */
"01X64Z6"  =  "1"  /* Transfer Radial Nerve to Radial Nerve, Perc Endo Approach */
"01X80Z8"  =  "1"  /* Transfer Thoracic Nerve to Thoracic Nerve, Open Approach */
"01X84Z8"  =  "1"  /* Transfer Thoracic Nerve to Thor Nrv, Perc Endo Approach */
"01XB0ZB"  =  "1"  /* Transfer Lumbar Nerve to Lumbar Nerve, Open Approach */
"01XB0ZC"  =  "1"  /* Transfer Lumbar Nerve to Perineal Nerve, Open Approach */
"01XB4ZB"  =  "1"  /* Transfer Lumbar Nerve to Lumbar Nerve, Perc Endo Approach */
"01XB4ZC"  =  "1"  /* Transfer Lumbar Nerve to Perineal Nerve, Perc Endo Approach */
"01XC0ZB"  =  "1"  /* Transfer Pudendal Nerve to Lumbar Nerve, Open Approach */
"01XC0ZC"  =  "1"  /* Transfer Pudendal Nerve to Perineal Nerve, Open Approach */
"01XC4ZB"  =  "1"  /* Transfer Pudendal Nerve to Lumbar Nerve, Perc Endo Approach */
"01XC4ZC"  =  "1"  /* Transfer Pudendal Nerve to Perineal Nrv, Perc Endo Approach */
"01XD0ZD"  =  "1"  /* Transfer Femoral Nerve to Femoral Nerve, Open Approach */
"01XD0ZF"  =  "1"  /* Transfer Femoral Nerve to Sciatic Nerve, Open Approach */
"01XD0ZG"  =  "1"  /* Transfer Femoral Nerve to Tibial Nerve, Open Approach */
"01XD0ZH"  =  "1"  /* Transfer Femoral Nerve to Peroneal Nerve, Open Approach */
"01XD4ZD"  =  "1"  /* Transfer Femoral Nerve to Femoral Nerve, Perc Endo Approach */
"01XD4ZF"  =  "1"  /* Transfer Femoral Nerve to Sciatic Nerve, Perc Endo Approach */
"01XD4ZG"  =  "1"  /* Transfer Femoral Nerve to Tibial Nerve, Perc Endo Approach */
"01XD4ZH"  =  "1"  /* Transfer Femoral Nerve to Peroneal Nerve, Perc Endo Approach */
"01XF0ZD"  =  "1"  /* Transfer Sciatic Nerve to Femoral Nerve, Open Approach */
"01XF0ZF"  =  "1"  /* Transfer Sciatic Nerve to Sciatic Nerve, Open Approach */
"01XF0ZG"  =  "1"  /* Transfer Sciatic Nerve to Tibial Nerve, Open Approach */
"01XF0ZH"  =  "1"  /* Transfer Sciatic Nerve to Peroneal Nerve, Open Approach */
"01XF4ZD"  =  "1"  /* Transfer Sciatic Nerve to Femoral Nerve, Perc Endo Approach */
"01XF4ZF"  =  "1"  /* Transfer Sciatic Nerve to Sciatic Nerve, Perc Endo Approach */
"01XF4ZG"  =  "1"  /* Transfer Sciatic Nerve to Tibial Nerve, Perc Endo Approach */
"01XF4ZH"  =  "1"  /* Transfer Sciatic Nerve to Peroneal Nerve, Perc Endo Approach */
"01XG0ZD"  =  "1"  /* Transfer Tibial Nerve to Femoral Nerve, Open Approach */
"01XG0ZF"  =  "1"  /* Transfer Tibial Nerve to Sciatic Nerve, Open Approach */
"01XG0ZG"  =  "1"  /* Transfer Tibial Nerve to Tibial Nerve, Open Approach */
"01XG0ZH"  =  "1"  /* Transfer Tibial Nerve to Peroneal Nerve, Open Approach */
"01XG4ZD"  =  "1"  /* Transfer Tibial Nerve to Femoral Nerve, Perc Endo Approach */
"01XG4ZF"  =  "1"  /* Transfer Tibial Nerve to Sciatic Nerve, Perc Endo Approach */
"01XG4ZG"  =  "1"  /* Transfer Tibial Nerve to Tibial Nerve, Perc Endo Approach */
"01XG4ZH"  =  "1"  /* Transfer Tibial Nerve to Peroneal Nerve, Perc Endo Approach */
"01XH0ZD"  =  "1"  /* Transfer Peroneal Nerve to Femoral Nerve, Open Approach */
"01XH0ZF"  =  "1"  /* Transfer Peroneal Nerve to Sciatic Nerve, Open Approach */
"01XH0ZG"  =  "1"  /* Transfer Peroneal Nerve to Tibial Nerve, Open Approach */
"01XH0ZH"  =  "1"  /* Transfer Peroneal Nerve to Peroneal Nerve, Open Approach */
"01XH4ZD"  =  "1"  /* Transfer Peroneal Nerve to Femoral Nerve, Perc Endo Approach */
"01XH4ZF"  =  "1"  /* Transfer Peroneal Nerve to Sciatic Nerve, Perc Endo Approach */
"01XH4ZG"  =  "1"  /* Transfer Peroneal Nerve to Tibial Nerve, Perc Endo Approach */
"01XH4ZH"  =  "1"  /* Transfer Peroneal Nerve to Peroneal Nrv, Perc Endo Approach */
"0210093"  =  "1"  /* Bypass 1 Cor Art from Cor Art with Autol Vn, Open Approach */
"0210098"  =  "1"  /* Bypass 1 Cor Art from R Int Mammary w Autol Vn, Open */
"0210099"  =  "1"  /* Bypass 1 Cor Art from L Int Mammary w Autol Vn, Open */
"021009C"  =  "1"  /* Bypass 1 Cor Art from Thor Art with Autol Vn, Open Approach */
"021009F"  =  "1"  /* Bypass 1 Cor Art from Abd Art with Autol Vn, Open Approach */
"021009W"  =  "1"  /* Bypass 1 Cor Art from Aorta with Autol Vn, Open Approach */
"02100A3"  =  "1"  /* Bypass 1 Cor Art from Cor Art with Autol Art, Open Approach */
"02100A8"  =  "1"  /* Bypass 1 Cor Art from R Int Mammary w Autol Art, Open */
"02100A9"  =  "1"  /* Bypass 1 Cor Art from L Int Mammary w Autol Art, Open */
"02100AC"  =  "1"  /* Bypass 1 Cor Art from Thor Art with Autol Art, Open Approach */
"02100AF"  =  "1"  /* Bypass 1 Cor Art from Abd Art with Autol Art, Open Approach */
"02100AW"  =  "1"  /* Bypass 1 Cor Art from Aorta with Autol Art, Open Approach */
"02100J3"  =  "1"  /* Bypass 1 Cor Art from Cor Art with Synth Sub, Open Approach */
"02100J8"  =  "1"  /* Bypass 1 Cor Art from R Int Mammary w Synth Sub, Open */
"02100J9"  =  "1"  /* Bypass 1 Cor Art from L Int Mammary w Synth Sub, Open */
"02100JC"  =  "1"  /* Bypass 1 Cor Art from Thor Art with Synth Sub, Open Approach */
"02100JF"  =  "1"  /* Bypass 1 Cor Art from Abd Art with Synth Sub, Open Approach */
"02100JW"  =  "1"  /* Bypass 1 Cor Art from Aorta with Synth Sub, Open Approach */
"02100K3"  =  "1"  /* Bypass 1 Cor Art from Cor Art with Nonaut Sub, Open Approach */
"02100K8"  =  "1"  /* Bypass 1 Cor Art from R Int Mammary w Nonaut Sub, Open */
"02100K9"  =  "1"  /* Bypass 1 Cor Art from L Int Mammary w Nonaut Sub, Open */
"02100KC"  =  "1"  /* Bypass 1 Cor Art from Thor Art w Nonaut Sub, Open */
"02100KF"  =  "1"  /* Bypass 1 Cor Art from Abd Art with Nonaut Sub, Open Approach */
"02100KW"  =  "1"  /* Bypass 1 Cor Art from Aorta with Nonaut Sub, Open Approach */
"02100Z3"  =  "1"  /* Bypass Coronary Artery, One Site from Cor Art, Open Approach */
"02100Z8"  =  "1"  /* Bypass 1 Cor Art from R Int Mammary, Open Approach */
"02100Z9"  =  "1"  /* Bypass 1 Cor Art from L Int Mammary, Open Approach */
"02100ZC"  =  "1"  /* Bypass 1 Cor Art from Thor Art, Open Approach */
"02100ZF"  =  "1"  /* Bypass Coronary Artery, One Site from Abd Art, Open Approach */
"0210493"  =  "1"  /* Bypass 1 Cor Art from Cor Art w Autol Vn, Perc Endo */
"0210498"  =  "1"  /* Bypass 1 Cor Art from R Int Mammary w Autol Vn, Perc Endo */
"0210499"  =  "1"  /* Bypass 1 Cor Art from L Int Mammary w Autol Vn, Perc Endo */
"021049C"  =  "1"  /* Bypass 1 Cor Art from Thor Art w Autol Vn, Perc Endo */
"021049F"  =  "1"  /* Bypass 1 Cor Art from Abd Art w Autol Vn, Perc Endo */
"021049W"  =  "1"  /* Bypass 1 Cor Art from Aorta w Autol Vn, Perc Endo */
"02104A3"  =  "1"  /* Bypass 1 Cor Art from Cor Art w Autol Art, Perc Endo */
"02104A8"  =  "1"  /* Bypass 1 Cor Art from R Int Mammary w Autol Art, Perc Endo */
"02104A9"  =  "1"  /* Bypass 1 Cor Art from L Int Mammary w Autol Art, Perc Endo */
"02104AC"  =  "1"  /* Bypass 1 Cor Art from Thor Art w Autol Art, Perc Endo */
"02104AF"  =  "1"  /* Bypass 1 Cor Art from Abd Art w Autol Art, Perc Endo */
"02104AW"  =  "1"  /* Bypass 1 Cor Art from Aorta w Autol Art, Perc Endo */
"02104J3"  =  "1"  /* Bypass 1 Cor Art from Cor Art w Synth Sub, Perc Endo */
"02104J8"  =  "1"  /* Bypass 1 Cor Art from R Int Mammary w Synth Sub, Perc Endo */
"02104J9"  =  "1"  /* Bypass 1 Cor Art from L Int Mammary w Synth Sub, Perc Endo */
"02104JC"  =  "1"  /* Bypass 1 Cor Art from Thor Art w Synth Sub, Perc Endo */
"02104JF"  =  "1"  /* Bypass 1 Cor Art from Abd Art w Synth Sub, Perc Endo */
"02104JW"  =  "1"  /* Bypass 1 Cor Art from Aorta w Synth Sub, Perc Endo */
"02104K3"  =  "1"  /* Bypass 1 Cor Art from Cor Art w Nonaut Sub, Perc Endo */
"02104K8"  =  "1"  /* Bypass 1 Cor Art from R Int Mammary w Nonaut Sub, Perc Endo */
"02104K9"  =  "1"  /* Bypass 1 Cor Art from L Int Mammary w Nonaut Sub, Perc Endo */
"02104KC"  =  "1"  /* Bypass 1 Cor Art from Thor Art w Nonaut Sub, Perc Endo */
"02104KF"  =  "1"  /* Bypass 1 Cor Art from Abd Art w Nonaut Sub, Perc Endo */
"02104KW"  =  "1"  /* Bypass 1 Cor Art from Aorta w Nonaut Sub, Perc Endo */
"02104Z3"  =  "1"  /* Bypass 1 Cor Art from Cor Art, Perc Endo Approach */
"02104Z8"  =  "1"  /* Bypass 1 Cor Art from R Int Mammary, Perc Endo Approach */
"02104Z9"  =  "1"  /* Bypass 1 Cor Art from L Int Mammary, Perc Endo Approach */
"02104ZC"  =  "1"  /* Bypass 1 Cor Art from Thor Art, Perc Endo Approach */
"02104ZF"  =  "1"  /* Bypass 1 Cor Art from Abd Art, Perc Endo Approach */
"0211093"  =  "1"  /* Bypass 2 Cor Art from Cor Art with Autol Vn, Open Approach */
"0211098"  =  "1"  /* Bypass 2 Cor Art from R Int Mammary w Autol Vn, Open */
"0211099"  =  "1"  /* Bypass 2 Cor Art from L Int Mammary w Autol Vn, Open */
"021109C"  =  "1"  /* Bypass 2 Cor Art from Thor Art with Autol Vn, Open Approach */
"021109F"  =  "1"  /* Bypass 2 Cor Art from Abd Art with Autol Vn, Open Approach */
"021109W"  =  "1"  /* Bypass 2 Cor Art from Aorta with Autol Vn, Open Approach */
"02110A3"  =  "1"  /* Bypass 2 Cor Art from Cor Art with Autol Art, Open Approach */
"02110A8"  =  "1"  /* Bypass 2 Cor Art from R Int Mammary w Autol Art, Open */
"02110A9"  =  "1"  /* Bypass 2 Cor Art from L Int Mammary w Autol Art, Open */
"02110AC"  =  "1"  /* Bypass 2 Cor Art from Thor Art with Autol Art, Open Approach */
"02110AF"  =  "1"  /* Bypass 2 Cor Art from Abd Art with Autol Art, Open Approach */
"02110AW"  =  "1"  /* Bypass 2 Cor Art from Aorta with Autol Art, Open Approach */
"02110J3"  =  "1"  /* Bypass 2 Cor Art from Cor Art with Synth Sub, Open Approach */
"02110J8"  =  "1"  /* Bypass 2 Cor Art from R Int Mammary w Synth Sub, Open */
"02110J9"  =  "1"  /* Bypass 2 Cor Art from L Int Mammary w Synth Sub, Open */
"02110JC"  =  "1"  /* Bypass 2 Cor Art from Thor Art with Synth Sub, Open Approach */
"02110JF"  =  "1"  /* Bypass 2 Cor Art from Abd Art with Synth Sub, Open Approach */
"02110JW"  =  "1"  /* Bypass 2 Cor Art from Aorta with Synth Sub, Open Approach */
"02110K3"  =  "1"  /* Bypass 2 Cor Art from Cor Art with Nonaut Sub, Open Approach */
"02110K8"  =  "1"  /* Bypass 2 Cor Art from R Int Mammary w Nonaut Sub, Open */
"02110K9"  =  "1"  /* Bypass 2 Cor Art from L Int Mammary w Nonaut Sub, Open */
"02110KC"  =  "1"  /* Bypass 2 Cor Art from Thor Art w Nonaut Sub, Open */
"02110KF"  =  "1"  /* Bypass 2 Cor Art from Abd Art with Nonaut Sub, Open Approach */
"02110KW"  =  "1"  /* Bypass 2 Cor Art from Aorta with Nonaut Sub, Open Approach */
"02110Z3"  =  "1"  /* Bypass 2 Cor Art from Cor Art, Open Approach */
"02110Z8"  =  "1"  /* Bypass 2 Cor Art from R Int Mammary, Open Approach */
"02110Z9"  =  "1"  /* Bypass 2 Cor Art from L Int Mammary, Open Approach */
"02110ZC"  =  "1"  /* Bypass 2 Cor Art from Thor Art, Open Approach */
"02110ZF"  =  "1"  /* Bypass 2 Cor Art from Abd Art, Open Approach */
"0211493"  =  "1"  /* Bypass 2 Cor Art from Cor Art w Autol Vn, Perc Endo */
"0211498"  =  "1"  /* Bypass 2 Cor Art from R Int Mammary w Autol Vn, Perc Endo */
"0211499"  =  "1"  /* Bypass 2 Cor Art from L Int Mammary w Autol Vn, Perc Endo */
"021149C"  =  "1"  /* Bypass 2 Cor Art from Thor Art w Autol Vn, Perc Endo */
"021149F"  =  "1"  /* Bypass 2 Cor Art from Abd Art w Autol Vn, Perc Endo */
"021149W"  =  "1"  /* Bypass 2 Cor Art from Aorta w Autol Vn, Perc Endo */
"02114A3"  =  "1"  /* Bypass 2 Cor Art from Cor Art w Autol Art, Perc Endo */
"02114A8"  =  "1"  /* Bypass 2 Cor Art from R Int Mammary w Autol Art, Perc Endo */
"02114A9"  =  "1"  /* Bypass 2 Cor Art from L Int Mammary w Autol Art, Perc Endo */
"02114AC"  =  "1"  /* Bypass 2 Cor Art from Thor Art w Autol Art, Perc Endo */
"02114AF"  =  "1"  /* Bypass 2 Cor Art from Abd Art w Autol Art, Perc Endo */
"02114AW"  =  "1"  /* Bypass 2 Cor Art from Aorta w Autol Art, Perc Endo */
"02114J3"  =  "1"  /* Bypass 2 Cor Art from Cor Art w Synth Sub, Perc Endo */
"02114J8"  =  "1"  /* Bypass 2 Cor Art from R Int Mammary w Synth Sub, Perc Endo */
"02114J9"  =  "1"  /* Bypass 2 Cor Art from L Int Mammary w Synth Sub, Perc Endo */
"02114JC"  =  "1"  /* Bypass 2 Cor Art from Thor Art w Synth Sub, Perc Endo */
"02114JF"  =  "1"  /* Bypass 2 Cor Art from Abd Art w Synth Sub, Perc Endo */
"02114JW"  =  "1"  /* Bypass 2 Cor Art from Aorta w Synth Sub, Perc Endo */
"02114K3"  =  "1"  /* Bypass 2 Cor Art from Cor Art w Nonaut Sub, Perc Endo */
"02114K8"  =  "1"  /* Bypass 2 Cor Art from R Int Mammary w Nonaut Sub, Perc Endo */
"02114K9"  =  "1"  /* Bypass 2 Cor Art from L Int Mammary w Nonaut Sub, Perc Endo */
"02114KC"  =  "1"  /* Bypass 2 Cor Art from Thor Art w Nonaut Sub, Perc Endo */
"02114KF"  =  "1"  /* Bypass 2 Cor Art from Abd Art w Nonaut Sub, Perc Endo */
"02114KW"  =  "1"  /* Bypass 2 Cor Art from Aorta w Nonaut Sub, Perc Endo */
"02114Z3"  =  "1"  /* Bypass 2 Cor Art from Cor Art, Perc Endo Approach */
"02114Z8"  =  "1"  /* Bypass 2 Cor Art from R Int Mammary, Perc Endo Approach */
"02114Z9"  =  "1"  /* Bypass 2 Cor Art from L Int Mammary, Perc Endo Approach */
"02114ZC"  =  "1"  /* Bypass 2 Cor Art from Thor Art, Perc Endo Approach */
"02114ZF"  =  "1"  /* Bypass 2 Cor Art from Abd Art, Perc Endo Approach */
"0212093"  =  "1"  /* Bypass 3 Cor Art from Cor Art with Autol Vn, Open Approach */
"0212098"  =  "1"  /* Bypass 3 Cor Art from R Int Mammary w Autol Vn, Open */
"0212099"  =  "1"  /* Bypass 3 Cor Art from L Int Mammary w Autol Vn, Open */
"021209C"  =  "1"  /* Bypass 3 Cor Art from Thor Art with Autol Vn, Open Approach */
"021209F"  =  "1"  /* Bypass 3 Cor Art from Abd Art with Autol Vn, Open Approach */
"021209W"  =  "1"  /* Bypass 3 Cor Art from Aorta with Autol Vn, Open Approach */
"02120A3"  =  "1"  /* Bypass 3 Cor Art from Cor Art with Autol Art, Open Approach */
"02120A8"  =  "1"  /* Bypass 3 Cor Art from R Int Mammary w Autol Art, Open */
"02120A9"  =  "1"  /* Bypass 3 Cor Art from L Int Mammary w Autol Art, Open */
"02120AC"  =  "1"  /* Bypass 3 Cor Art from Thor Art with Autol Art, Open Approach */
"02120AF"  =  "1"  /* Bypass 3 Cor Art from Abd Art with Autol Art, Open Approach */
"02120AW"  =  "1"  /* Bypass 3 Cor Art from Aorta with Autol Art, Open Approach */
"02120J3"  =  "1"  /* Bypass 3 Cor Art from Cor Art with Synth Sub, Open Approach */
"02120J8"  =  "1"  /* Bypass 3 Cor Art from R Int Mammary w Synth Sub, Open */
"02120J9"  =  "1"  /* Bypass 3 Cor Art from L Int Mammary w Synth Sub, Open */
"02120JC"  =  "1"  /* Bypass 3 Cor Art from Thor Art with Synth Sub, Open Approach */
"02120JF"  =  "1"  /* Bypass 3 Cor Art from Abd Art with Synth Sub, Open Approach */
"02120JW"  =  "1"  /* Bypass 3 Cor Art from Aorta with Synth Sub, Open Approach */
"02120K3"  =  "1"  /* Bypass 3 Cor Art from Cor Art with Nonaut Sub, Open Approach */
"02120K8"  =  "1"  /* Bypass 3 Cor Art from R Int Mammary w Nonaut Sub, Open */
"02120K9"  =  "1"  /* Bypass 3 Cor Art from L Int Mammary w Nonaut Sub, Open */
"02120KC"  =  "1"  /* Bypass 3 Cor Art from Thor Art w Nonaut Sub, Open */
"02120KF"  =  "1"  /* Bypass 3 Cor Art from Abd Art with Nonaut Sub, Open Approach */
"02120KW"  =  "1"  /* Bypass 3 Cor Art from Aorta with Nonaut Sub, Open Approach */
"02120Z3"  =  "1"  /* Bypass 3 Cor Art from Cor Art, Open Approach */
"02120Z8"  =  "1"  /* Bypass 3 Cor Art from R Int Mammary, Open Approach */
"02120Z9"  =  "1"  /* Bypass 3 Cor Art from L Int Mammary, Open Approach */
"02120ZC"  =  "1"  /* Bypass 3 Cor Art from Thor Art, Open Approach */
"02120ZF"  =  "1"  /* Bypass 3 Cor Art from Abd Art, Open Approach */
"0212493"  =  "1"  /* Bypass 3 Cor Art from Cor Art w Autol Vn, Perc Endo */
"0212498"  =  "1"  /* Bypass 3 Cor Art from R Int Mammary w Autol Vn, Perc Endo */
"0212499"  =  "1"  /* Bypass 3 Cor Art from L Int Mammary w Autol Vn, Perc Endo */
"021249C"  =  "1"  /* Bypass 3 Cor Art from Thor Art w Autol Vn, Perc Endo */
"021249F"  =  "1"  /* Bypass 3 Cor Art from Abd Art w Autol Vn, Perc Endo */
"021249W"  =  "1"  /* Bypass 3 Cor Art from Aorta w Autol Vn, Perc Endo */
"02124A3"  =  "1"  /* Bypass 3 Cor Art from Cor Art w Autol Art, Perc Endo */
"02124A8"  =  "1"  /* Bypass 3 Cor Art from R Int Mammary w Autol Art, Perc Endo */
"02124A9"  =  "1"  /* Bypass 3 Cor Art from L Int Mammary w Autol Art, Perc Endo */
"02124AC"  =  "1"  /* Bypass 3 Cor Art from Thor Art w Autol Art, Perc Endo */
"02124AF"  =  "1"  /* Bypass 3 Cor Art from Abd Art w Autol Art, Perc Endo */
"02124AW"  =  "1"  /* Bypass 3 Cor Art from Aorta w Autol Art, Perc Endo */
"02124J3"  =  "1"  /* Bypass 3 Cor Art from Cor Art w Synth Sub, Perc Endo */
"02124J8"  =  "1"  /* Bypass 3 Cor Art from R Int Mammary w Synth Sub, Perc Endo */
"02124J9"  =  "1"  /* Bypass 3 Cor Art from L Int Mammary w Synth Sub, Perc Endo */
"02124JC"  =  "1"  /* Bypass 3 Cor Art from Thor Art w Synth Sub, Perc Endo */
"02124JF"  =  "1"  /* Bypass 3 Cor Art from Abd Art w Synth Sub, Perc Endo */
"02124JW"  =  "1"  /* Bypass 3 Cor Art from Aorta w Synth Sub, Perc Endo */
"02124K3"  =  "1"  /* Bypass 3 Cor Art from Cor Art w Nonaut Sub, Perc Endo */
"02124K8"  =  "1"  /* Bypass 3 Cor Art from R Int Mammary w Nonaut Sub, Perc Endo */
"02124K9"  =  "1"  /* Bypass 3 Cor Art from L Int Mammary w Nonaut Sub, Perc Endo */
"02124KC"  =  "1"  /* Bypass 3 Cor Art from Thor Art w Nonaut Sub, Perc Endo */
"02124KF"  =  "1"  /* Bypass 3 Cor Art from Abd Art w Nonaut Sub, Perc Endo */
"02124KW"  =  "1"  /* Bypass 3 Cor Art from Aorta w Nonaut Sub, Perc Endo */
"02124Z3"  =  "1"  /* Bypass 3 Cor Art from Cor Art, Perc Endo Approach */
"02124Z8"  =  "1"  /* Bypass 3 Cor Art from R Int Mammary, Perc Endo Approach */
"02124Z9"  =  "1"  /* Bypass 3 Cor Art from L Int Mammary, Perc Endo Approach */
"02124ZC"  =  "1"  /* Bypass 3 Cor Art from Thor Art, Perc Endo Approach */
"02124ZF"  =  "1"  /* Bypass 3 Cor Art from Abd Art, Perc Endo Approach */
"0213093"  =  "1"  /* Bypass 4+ Cor Art from Cor Art with Autol Vn, Open Approach */
"0213098"  =  "1"  /* Bypass 4+ Cor Art from R Int Mammary w Autol Vn, Open */
"0213099"  =  "1"  /* Bypass 4+ Cor Art from L Int Mammary w Autol Vn, Open */
"021309C"  =  "1"  /* Bypass 4+ Cor Art from Thor Art with Autol Vn, Open Approach */
"021309F"  =  "1"  /* Bypass 4+ Cor Art from Abd Art with Autol Vn, Open Approach */
"021309W"  =  "1"  /* Bypass 4+ Cor Art from Aorta with Autol Vn, Open Approach */
"02130A3"  =  "1"  /* Bypass 4+ Cor Art from Cor Art with Autol Art, Open Approach */
"02130A8"  =  "1"  /* Bypass 4+ Cor Art from R Int Mammary w Autol Art, Open */
"02130A9"  =  "1"  /* Bypass 4+ Cor Art from L Int Mammary w Autol Art, Open */
"02130AC"  =  "1"  /* Bypass 4+ Cor Art from Thor Art w Autol Art, Open */
"02130AF"  =  "1"  /* Bypass 4+ Cor Art from Abd Art with Autol Art, Open Approach */
"02130AW"  =  "1"  /* Bypass 4+ Cor Art from Aorta with Autol Art, Open Approach */
"02130J3"  =  "1"  /* Bypass 4+ Cor Art from Cor Art with Synth Sub, Open Approach */
"02130J8"  =  "1"  /* Bypass 4+ Cor Art from R Int Mammary w Synth Sub, Open */
"02130J9"  =  "1"  /* Bypass 4+ Cor Art from L Int Mammary w Synth Sub, Open */
"02130JC"  =  "1"  /* Bypass 4+ Cor Art from Thor Art w Synth Sub, Open */
"02130JF"  =  "1"  /* Bypass 4+ Cor Art from Abd Art with Synth Sub, Open Approach */
"02130JW"  =  "1"  /* Bypass 4+ Cor Art from Aorta with Synth Sub, Open Approach */
"02130K3"  =  "1"  /* Bypass 4+ Cor Art from Cor Art w Nonaut Sub, Open */
"02130K8"  =  "1"  /* Bypass 4+ Cor Art from R Int Mammary w Nonaut Sub, Open */
"02130K9"  =  "1"  /* Bypass 4+ Cor Art from L Int Mammary w Nonaut Sub, Open */
"02130KC"  =  "1"  /* Bypass 4+ Cor Art from Thor Art w Nonaut Sub, Open */
"02130KF"  =  "1"  /* Bypass 4+ Cor Art from Abd Art w Nonaut Sub, Open */
"02130KW"  =  "1"  /* Bypass 4+ Cor Art from Aorta with Nonaut Sub, Open Approach */
"02130Z3"  =  "1"  /* Bypass 4+ Cor Art from Cor Art, Open Approach */
"02130Z8"  =  "1"  /* Bypass 4+ Cor Art from R Int Mammary, Open Approach */
"02130Z9"  =  "1"  /* Bypass 4+ Cor Art from L Int Mammary, Open Approach */
"02130ZC"  =  "1"  /* Bypass 4+ Cor Art from Thor Art, Open Approach */
"02130ZF"  =  "1"  /* Bypass 4+ Cor Art from Abd Art, Open Approach */
"0213493"  =  "1"  /* Bypass 4+ Cor Art from Cor Art w Autol Vn, Perc Endo */
"0213498"  =  "1"  /* Bypass 4+ Cor Art from R Int Mammary w Autol Vn, Perc Endo */
"0213499"  =  "1"  /* Bypass 4+ Cor Art from L Int Mammary w Autol Vn, Perc Endo */
"021349C"  =  "1"  /* Bypass 4+ Cor Art from Thor Art w Autol Vn, Perc Endo */
"021349F"  =  "1"  /* Bypass 4+ Cor Art from Abd Art w Autol Vn, Perc Endo */
"021349W"  =  "1"  /* Bypass 4+ Cor Art from Aorta w Autol Vn, Perc Endo */
"02134A3"  =  "1"  /* Bypass 4+ Cor Art from Cor Art w Autol Art, Perc Endo */
"02134A8"  =  "1"  /* Bypass 4+ Cor Art from R Int Mammary w Autol Art, Perc Endo */
"02134A9"  =  "1"  /* Bypass 4+ Cor Art from L Int Mammary w Autol Art, Perc Endo */
"02134AC"  =  "1"  /* Bypass 4+ Cor Art from Thor Art w Autol Art, Perc Endo */
"02134AF"  =  "1"  /* Bypass 4+ Cor Art from Abd Art w Autol Art, Perc Endo */
"02134AW"  =  "1"  /* Bypass 4+ Cor Art from Aorta w Autol Art, Perc Endo */
"02134J3"  =  "1"  /* Bypass 4+ Cor Art from Cor Art w Synth Sub, Perc Endo */
"02134J8"  =  "1"  /* Bypass 4+ Cor Art from R Int Mammary w Synth Sub, Perc Endo */
"02134J9"  =  "1"  /* Bypass 4+ Cor Art from L Int Mammary w Synth Sub, Perc Endo */
"02134JC"  =  "1"  /* Bypass 4+ Cor Art from Thor Art w Synth Sub, Perc Endo */
"02134JF"  =  "1"  /* Bypass 4+ Cor Art from Abd Art w Synth Sub, Perc Endo */
"02134JW"  =  "1"  /* Bypass 4+ Cor Art from Aorta w Synth Sub, Perc Endo */
"02134K3"  =  "1"  /* Bypass 4+ Cor Art from Cor Art w Nonaut Sub, Perc Endo */
"02134K8"  =  "1"  /* Bypass 4+ Cor Art from R Int Mammary w Nonaut Sub, Perc Endo */
"02134K9"  =  "1"  /* Bypass 4+ Cor Art from L Int Mammary w Nonaut Sub, Perc Endo */
"02134KC"  =  "1"  /* Bypass 4+ Cor Art from Thor Art w Nonaut Sub, Perc Endo */
"02134KF"  =  "1"  /* Bypass 4+ Cor Art from Abd Art w Nonaut Sub, Perc Endo */
"02134KW"  =  "1"  /* Bypass 4+ Cor Art from Aorta w Nonaut Sub, Perc Endo */
"02134Z3"  =  "1"  /* Bypass 4+ Cor Art from Cor Art, Perc Endo Approach */
"02134Z8"  =  "1"  /* Bypass 4+ Cor Art from R Int Mammary, Perc Endo Approach */
"02134Z9"  =  "1"  /* Bypass 4+ Cor Art from L Int Mammary, Perc Endo Approach */
"02134ZC"  =  "1"  /* Bypass 4+ Cor Art from Thor Art, Perc Endo Approach */
"02134ZF"  =  "1"  /* Bypass 4+ Cor Art from Abd Art, Perc Endo Approach */
"021609P"  =  "1"  /* Bypass R Atrium to Pulm Trunk with Autol Vn, Open Approach */
"021609Q"  =  "1"  /* Bypass R Atrium to R Pulm Art with Autol Vn, Open Approach */
"021609R"  =  "1"  /* Bypass R Atrium to L Pulm Art with Autol Vn, Open Approach */
"02160AP"  =  "1"  /* Bypass R Atrium to Pulm Trunk with Autol Art, Open Approach */
"02160AQ"  =  "1"  /* Bypass R Atrium to R Pulm Art with Autol Art, Open Approach */
"02160AR"  =  "1"  /* Bypass R Atrium to L Pulm Art with Autol Art, Open Approach */
"02160JP"  =  "1"  /* Bypass R Atrium to Pulm Trunk with Synth Sub, Open Approach */
"02160JQ"  =  "1"  /* Bypass R Atrium to R Pulm Art with Synth Sub, Open Approach */
"02160JR"  =  "1"  /* Bypass R Atrium to L Pulm Art with Synth Sub, Open Approach */
"02160KP"  =  "1"  /* Bypass R Atrium to Pulm Trunk with Nonaut Sub, Open Approach */
"02160KQ"  =  "1"  /* Bypass R Atrium to R Pulm Art with Nonaut Sub, Open Approach */
"02160KR"  =  "1"  /* Bypass R Atrium to L Pulm Art with Nonaut Sub, Open Approach */
"02160Z7"  =  "1"  /* Bypass Right Atrium to Left Atrium, Open Approach */
"02160ZP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk, Open Approach */
"02160ZQ"  =  "1"  /* Bypass Right Atrium to Right Pulmonary Artery, Open Approach */
"02160ZR"  =  "1"  /* Bypass Right Atrium to Left Pulmonary Artery, Open Approach */
"021649P"  =  "1"  /* Bypass R Atrium to Pulm Trunk w Autol Vn, Perc Endo */
"021649Q"  =  "1"  /* Bypass R Atrium to R Pulm Art w Autol Vn, Perc Endo */
"021649R"  =  "1"  /* Bypass R Atrium to L Pulm Art w Autol Vn, Perc Endo */
"02164AP"  =  "1"  /* Bypass R Atrium to Pulm Trunk w Autol Art, Perc Endo */
"02164AQ"  =  "1"  /* Bypass R Atrium to R Pulm Art w Autol Art, Perc Endo */
"02164AR"  =  "1"  /* Bypass R Atrium to L Pulm Art w Autol Art, Perc Endo */
"02164JP"  =  "1"  /* Bypass R Atrium to Pulm Trunk w Synth Sub, Perc Endo */
"02164JQ"  =  "1"  /* Bypass R Atrium to R Pulm Art w Synth Sub, Perc Endo */
"02164JR"  =  "1"  /* Bypass R Atrium to L Pulm Art w Synth Sub, Perc Endo */
"02164KP"  =  "1"  /* Bypass R Atrium to Pulm Trunk w Nonaut Sub, Perc Endo */
"02164KQ"  =  "1"  /* Bypass R Atrium to R Pulm Art w Nonaut Sub, Perc Endo */
"02164KR"  =  "1"  /* Bypass R Atrium to L Pulm Art w Nonaut Sub, Perc Endo */
"02164Z7"  =  "1"  /* Bypass Right Atrium to Left Atrium, Perc Endo Approach */
"02164ZP"  =  "1"  /* Bypass Right Atrium to Pulmonary Trunk, Perc Endo Approach */
"02164ZQ"  =  "1"  /* Bypass Right Atrium to R Pulm Art, Perc Endo Approach */
"02164ZR"  =  "1"  /* Bypass Right Atrium to L Pulm Art, Perc Endo Approach */
"021709P"  =  "1"  /* Bypass L Atrium to Pulm Trunk with Autol Vn, Open Approach */
"021709Q"  =  "1"  /* Bypass L Atrium to R Pulm Art with Autol Vn, Open Approach */
"021709R"  =  "1"  /* Bypass L Atrium to L Pulm Art with Autol Vn, Open Approach */
"02170AP"  =  "1"  /* Bypass L Atrium to Pulm Trunk with Autol Art, Open Approach */
"02170AQ"  =  "1"  /* Bypass L Atrium to R Pulm Art with Autol Art, Open Approach */
"02170AR"  =  "1"  /* Bypass L Atrium to L Pulm Art with Autol Art, Open Approach */
"02170JP"  =  "1"  /* Bypass L Atrium to Pulm Trunk with Synth Sub, Open Approach */
"02170JQ"  =  "1"  /* Bypass L Atrium to R Pulm Art with Synth Sub, Open Approach */
"02170JR"  =  "1"  /* Bypass L Atrium to L Pulm Art with Synth Sub, Open Approach */
"02170KP"  =  "1"  /* Bypass L Atrium to Pulm Trunk with Nonaut Sub, Open Approach */
"02170KQ"  =  "1"  /* Bypass L Atrium to R Pulm Art with Nonaut Sub, Open Approach */
"02170KR"  =  "1"  /* Bypass L Atrium to L Pulm Art with Nonaut Sub, Open Approach */
"02170ZP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk, Open Approach */
"02170ZQ"  =  "1"  /* Bypass Left Atrium to Right Pulmonary Artery, Open Approach */
"02170ZR"  =  "1"  /* Bypass Left Atrium to Left Pulmonary Artery, Open Approach */
"021749P"  =  "1"  /* Bypass L Atrium to Pulm Trunk w Autol Vn, Perc Endo */
"021749Q"  =  "1"  /* Bypass L Atrium to R Pulm Art w Autol Vn, Perc Endo */
"021749R"  =  "1"  /* Bypass L Atrium to L Pulm Art w Autol Vn, Perc Endo */
"02174AP"  =  "1"  /* Bypass L Atrium to Pulm Trunk w Autol Art, Perc Endo */
"02174AQ"  =  "1"  /* Bypass L Atrium to R Pulm Art w Autol Art, Perc Endo */
"02174AR"  =  "1"  /* Bypass L Atrium to L Pulm Art w Autol Art, Perc Endo */
"02174JP"  =  "1"  /* Bypass L Atrium to Pulm Trunk w Synth Sub, Perc Endo */
"02174JQ"  =  "1"  /* Bypass L Atrium to R Pulm Art w Synth Sub, Perc Endo */
"02174JR"  =  "1"  /* Bypass L Atrium to L Pulm Art w Synth Sub, Perc Endo */
"02174KP"  =  "1"  /* Bypass L Atrium to Pulm Trunk w Nonaut Sub, Perc Endo */
"02174KQ"  =  "1"  /* Bypass L Atrium to R Pulm Art w Nonaut Sub, Perc Endo */
"02174KR"  =  "1"  /* Bypass L Atrium to L Pulm Art w Nonaut Sub, Perc Endo */
"02174ZP"  =  "1"  /* Bypass Left Atrium to Pulmonary Trunk, Perc Endo Approach */
"02174ZQ"  =  "1"  /* Bypass Left Atrium to R Pulm Art, Perc Endo Approach */
"02174ZR"  =  "1"  /* Bypass Left Atrium to L Pulm Art, Perc Endo Approach */
"021K09P"  =  "1"  /* Bypass R Ventricle to Pulm Trunk w Autol Vn, Open */
"021K09Q"  =  "1"  /* Bypass R Ventricle to R Pulm Art w Autol Vn, Open */
"021K09R"  =  "1"  /* Bypass R Ventricle to L Pulm Art w Autol Vn, Open */
"021K0AP"  =  "1"  /* Bypass R Ventricle to Pulm Trunk w Autol Art, Open */
"021K0AQ"  =  "1"  /* Bypass R Ventricle to R Pulm Art w Autol Art, Open */
"021K0AR"  =  "1"  /* Bypass R Ventricle to L Pulm Art w Autol Art, Open */
"021K0JP"  =  "1"  /* Bypass R Ventricle to Pulm Trunk w Synth Sub, Open */
"021K0JQ"  =  "1"  /* Bypass R Ventricle to R Pulm Art w Synth Sub, Open */
"021K0JR"  =  "1"  /* Bypass R Ventricle to L Pulm Art w Synth Sub, Open */
"021K0KP"  =  "1"  /* Bypass R Ventricle to Pulm Trunk w Nonaut Sub, Open */
"021K0KQ"  =  "1"  /* Bypass R Ventricle to R Pulm Art w Nonaut Sub, Open */
"021K0KR"  =  "1"  /* Bypass R Ventricle to L Pulm Art w Nonaut Sub, Open */
"021K0Z5"  =  "1"  /* Bypass Right Ventricle to Cor Circ, Open Approach */
"021K0Z8"  =  "1"  /* Bypass Right Ventricle to R Int Mammary, Open Approach */
"021K0Z9"  =  "1"  /* Bypass Right Ventricle to L Int Mammary, Open Approach */
"021K0ZC"  =  "1"  /* Bypass Right Ventricle to Thoracic Artery, Open Approach */
"021K0ZF"  =  "1"  /* Bypass Right Ventricle to Abdominal Artery, Open Approach */
"021K0ZP"  =  "1"  /* Bypass Right Ventricle to Pulmonary Trunk, Open Approach */
"021K0ZQ"  =  "1"  /* Bypass Right Ventricle to R Pulm Art, Open Approach */
"021K0ZR"  =  "1"  /* Bypass Right Ventricle to L Pulm Art, Open Approach */
"021K0ZW"  =  "1"  /* Bypass Right Ventricle to Aorta, Open Approach */
"021K49P"  =  "1"  /* Bypass R Ventricle to Pulm Trunk w Autol Vn, Perc Endo */
"021K49Q"  =  "1"  /* Bypass R Ventricle to R Pulm Art w Autol Vn, Perc Endo */
"021K49R"  =  "1"  /* Bypass R Ventricle to L Pulm Art w Autol Vn, Perc Endo */
"021K4AP"  =  "1"  /* Bypass R Ventricle to Pulm Trunk w Autol Art, Perc Endo */
"021K4AQ"  =  "1"  /* Bypass R Ventricle to R Pulm Art w Autol Art, Perc Endo */
"021K4AR"  =  "1"  /* Bypass R Ventricle to L Pulm Art w Autol Art, Perc Endo */
"021K4JP"  =  "1"  /* Bypass R Ventricle to Pulm Trunk w Synth Sub, Perc Endo */
"021K4JQ"  =  "1"  /* Bypass R Ventricle to R Pulm Art w Synth Sub, Perc Endo */
"021K4JR"  =  "1"  /* Bypass R Ventricle to L Pulm Art w Synth Sub, Perc Endo */
"021K4KP"  =  "1"  /* Bypass R Ventricle to Pulm Trunk w Nonaut Sub, Perc Endo */
"021K4KQ"  =  "1"  /* Bypass R Ventricle to R Pulm Art w Nonaut Sub, Perc Endo */
"021K4KR"  =  "1"  /* Bypass R Ventricle to L Pulm Art w Nonaut Sub, Perc Endo */
"021K4Z5"  =  "1"  /* Bypass Right Ventricle to Cor Circ, Perc Endo Approach */
"021K4Z8"  =  "1"  /* Bypass Right Ventricle to R Int Mammary, Perc Endo Approach */
"021K4Z9"  =  "1"  /* Bypass Right Ventricle to L Int Mammary, Perc Endo Approach */
"021K4ZC"  =  "1"  /* Bypass Right Ventricle to Thor Art, Perc Endo Approach */
"021K4ZF"  =  "1"  /* Bypass Right Ventricle to Abd Art, Perc Endo Approach */
"021K4ZP"  =  "1"  /* Bypass Right Ventricle to Pulm Trunk, Perc Endo Approach */
"021K4ZQ"  =  "1"  /* Bypass Right Ventricle to R Pulm Art, Perc Endo Approach */
"021K4ZR"  =  "1"  /* Bypass Right Ventricle to L Pulm Art, Perc Endo Approach */
"021K4ZW"  =  "1"  /* Bypass Right Ventricle to Aorta, Perc Endo Approach */
"021L09P"  =  "1"  /* Bypass L Ventricle to Pulm Trunk w Autol Vn, Open */
"021L09Q"  =  "1"  /* Bypass L Ventricle to R Pulm Art w Autol Vn, Open */
"021L09R"  =  "1"  /* Bypass L Ventricle to L Pulm Art w Autol Vn, Open */
"021L0AP"  =  "1"  /* Bypass L Ventricle to Pulm Trunk w Autol Art, Open */
"021L0AQ"  =  "1"  /* Bypass L Ventricle to R Pulm Art w Autol Art, Open */
"021L0AR"  =  "1"  /* Bypass L Ventricle to L Pulm Art w Autol Art, Open */
"021L0JP"  =  "1"  /* Bypass L Ventricle to Pulm Trunk w Synth Sub, Open */
"021L0JQ"  =  "1"  /* Bypass L Ventricle to R Pulm Art w Synth Sub, Open */
"021L0JR"  =  "1"  /* Bypass L Ventricle to L Pulm Art w Synth Sub, Open */
"021L0KP"  =  "1"  /* Bypass L Ventricle to Pulm Trunk w Nonaut Sub, Open */
"021L0KQ"  =  "1"  /* Bypass L Ventricle to R Pulm Art w Nonaut Sub, Open */
"021L0KR"  =  "1"  /* Bypass L Ventricle to L Pulm Art w Nonaut Sub, Open */
"021L0Z5"  =  "1"  /* Bypass Left Ventricle to Coronary Circulation, Open Approach */
"021L0Z8"  =  "1"  /* Bypass Left Ventricle to R Int Mammary, Open Approach */
"021L0Z9"  =  "1"  /* Bypass Left Ventricle to L Int Mammary, Open Approach */
"021L0ZC"  =  "1"  /* Bypass Left Ventricle to Thoracic Artery, Open Approach */
"021L0ZF"  =  "1"  /* Bypass Left Ventricle to Abdominal Artery, Open Approach */
"021L0ZP"  =  "1"  /* Bypass Left Ventricle to Pulmonary Trunk, Open Approach */
"021L0ZQ"  =  "1"  /* Bypass Left Ventricle to R Pulm Art, Open Approach */
"021L0ZR"  =  "1"  /* Bypass Left Ventricle to L Pulm Art, Open Approach */
"021L0ZW"  =  "1"  /* Bypass Left Ventricle to Aorta, Open Approach */
"021L49P"  =  "1"  /* Bypass L Ventricle to Pulm Trunk w Autol Vn, Perc Endo */
"021L49Q"  =  "1"  /* Bypass L Ventricle to R Pulm Art w Autol Vn, Perc Endo */
"021L49R"  =  "1"  /* Bypass L Ventricle to L Pulm Art w Autol Vn, Perc Endo */
"021L4AP"  =  "1"  /* Bypass L Ventricle to Pulm Trunk w Autol Art, Perc Endo */
"021L4AQ"  =  "1"  /* Bypass L Ventricle to R Pulm Art w Autol Art, Perc Endo */
"021L4AR"  =  "1"  /* Bypass L Ventricle to L Pulm Art w Autol Art, Perc Endo */
"021L4JP"  =  "1"  /* Bypass L Ventricle to Pulm Trunk w Synth Sub, Perc Endo */
"021L4JQ"  =  "1"  /* Bypass L Ventricle to R Pulm Art w Synth Sub, Perc Endo */
"021L4JR"  =  "1"  /* Bypass L Ventricle to L Pulm Art w Synth Sub, Perc Endo */
"021L4KP"  =  "1"  /* Bypass L Ventricle to Pulm Trunk w Nonaut Sub, Perc Endo */
"021L4KQ"  =  "1"  /* Bypass L Ventricle to R Pulm Art w Nonaut Sub, Perc Endo */
"021L4KR"  =  "1"  /* Bypass L Ventricle to L Pulm Art w Nonaut Sub, Perc Endo */
"021L4Z5"  =  "1"  /* Bypass Left Ventricle to Cor Circ, Perc Endo Approach */
"021L4Z8"  =  "1"  /* Bypass Left Ventricle to R Int Mammary, Perc Endo Approach */
"021L4Z9"  =  "1"  /* Bypass Left Ventricle to L Int Mammary, Perc Endo Approach */
"021L4ZC"  =  "1"  /* Bypass Left Ventricle to Thoracic Artery, Perc Endo Approach */
"021L4ZF"  =  "1"  /* Bypass Left Ventricle to Abd Art, Perc Endo Approach */
"021L4ZP"  =  "1"  /* Bypass Left Ventricle to Pulmonary Trunk, Perc Endo Approach */
"021L4ZQ"  =  "1"  /* Bypass Left Ventricle to R Pulm Art, Perc Endo Approach */
"021L4ZR"  =  "1"  /* Bypass Left Ventricle to L Pulm Art, Perc Endo Approach */
"021L4ZW"  =  "1"  /* Bypass Left Ventricle to Aorta, Perc Endo Approach */
"021V09P"  =  "1"  /* Bypass Sup Vena Cava to Pulm Trunk w Autol Vn, Open */
"021V09Q"  =  "1"  /* Bypass Sup Vena Cava to R Pulm Art w Autol Vn, Open */
"021V09R"  =  "1"  /* Bypass Sup Vena Cava to L Pulm Art w Autol Vn, Open */
"021V0AP"  =  "1"  /* Bypass Sup Vena Cava to Pulm Trunk w Autol Art, Open */
"021V0AQ"  =  "1"  /* Bypass Sup Vena Cava to R Pulm Art w Autol Art, Open */
"021V0AR"  =  "1"  /* Bypass Sup Vena Cava to L Pulm Art w Autol Art, Open */
"021V0JP"  =  "1"  /* Bypass Sup Vena Cava to Pulm Trunk w Synth Sub, Open */
"021V0JQ"  =  "1"  /* Bypass Sup Vena Cava to R Pulm Art w Synth Sub, Open */
"021V0JR"  =  "1"  /* Bypass Sup Vena Cava to L Pulm Art w Synth Sub, Open */
"021V0KP"  =  "1"  /* Bypass Sup Vena Cava to Pulm Trunk w Nonaut Sub, Open */
"021V0KQ"  =  "1"  /* Bypass Sup Vena Cava to R Pulm Art w Nonaut Sub, Open */
"021V0KR"  =  "1"  /* Bypass Sup Vena Cava to L Pulm Art w Nonaut Sub, Open */
"021V0ZP"  =  "1"  /* Bypass Superior Vena Cava to Pulmonary Trunk, Open Approach */
"021V0ZQ"  =  "1"  /* Bypass Superior Vena Cava to R Pulm Art, Open Approach */
"021V0ZR"  =  "1"  /* Bypass Superior Vena Cava to L Pulm Art, Open Approach */
"021V49P"  =  "1"  /* Bypass Sup Vena Cava to Pulm Trunk w Autol Vn, Perc Endo */
"021V49Q"  =  "1"  /* Bypass Sup Vena Cava to R Pulm Art w Autol Vn, Perc Endo */
"021V49R"  =  "1"  /* Bypass Sup Vena Cava to L Pulm Art w Autol Vn, Perc Endo */
"021V4AP"  =  "1"  /* Bypass Sup Vena Cava to Pulm Trunk w Autol Art, Perc Endo */
"021V4AQ"  =  "1"  /* Bypass Sup Vena Cava to R Pulm Art w Autol Art, Perc Endo */
"021V4AR"  =  "1"  /* Bypass Sup Vena Cava to L Pulm Art w Autol Art, Perc Endo */
"021V4JP"  =  "1"  /* Bypass Sup Vena Cava to Pulm Trunk w Synth Sub, Perc Endo */
"021V4JQ"  =  "1"  /* Bypass Sup Vena Cava to R Pulm Art w Synth Sub, Perc Endo */
"021V4JR"  =  "1"  /* Bypass Sup Vena Cava to L Pulm Art w Synth Sub, Perc Endo */
"021V4KP"  =  "1"  /* Bypass Sup Vena Cava to Pulm Trunk w Nonaut Sub, Perc Endo */
"021V4KQ"  =  "1"  /* Bypass Sup Vena Cava to R Pulm Art w Nonaut Sub, Perc Endo */
"021V4KR"  =  "1"  /* Bypass Sup Vena Cava to L Pulm Art w Nonaut Sub, Perc Endo */
"021V4ZP"  =  "1"  /* Bypass Superior Vena Cava to Pulm Trunk, Perc Endo Approach */
"021V4ZQ"  =  "1"  /* Bypass Superior Vena Cava to R Pulm Art, Perc Endo Approach */
"021V4ZR"  =  "1"  /* Bypass Superior Vena Cava to L Pulm Art, Perc Endo Approach */
"021W09B"  =  "1"  /* Bypass Thor Aorta to Subclav with Autol Vn, Open Approach */
"021W09D"  =  "1"  /* Bypass Thor Aorta to Carotid with Autol Vn, Open Approach */
"021W09P"  =  "1"  /* Bypass Thor Aorta to Pulm Trunk with Autol Vn, Open Approach */
"021W09Q"  =  "1"  /* Bypass Thor Aorta to R Pulm Art with Autol Vn, Open Approach */
"021W09R"  =  "1"  /* Bypass Thor Aorta to L Pulm Art with Autol Vn, Open Approach */
"021W0AB"  =  "1"  /* Bypass Thor Aorta to Subclav with Autol Art, Open Approach */
"021W0AD"  =  "1"  /* Bypass Thor Aorta to Carotid with Autol Art, Open Approach */
"021W0AP"  =  "1"  /* Bypass Thor Aorta to Pulm Trunk w Autol Art, Open */
"021W0AQ"  =  "1"  /* Bypass Thor Aorta to R Pulm Art w Autol Art, Open */
"021W0AR"  =  "1"  /* Bypass Thor Aorta to L Pulm Art w Autol Art, Open */
"021W0JB"  =  "1"  /* Bypass Thor Aorta to Subclav with Synth Sub, Open Approach */
"021W0JD"  =  "1"  /* Bypass Thor Aorta to Carotid with Synth Sub, Open Approach */
"021W0JP"  =  "1"  /* Bypass Thor Aorta to Pulm Trunk w Synth Sub, Open */
"021W0JQ"  =  "1"  /* Bypass Thor Aorta to R Pulm Art w Synth Sub, Open */
"021W0JR"  =  "1"  /* Bypass Thor Aorta to L Pulm Art w Synth Sub, Open */
"021W0KB"  =  "1"  /* Bypass Thor Aorta to Subclav with Nonaut Sub, Open Approach */
"021W0KD"  =  "1"  /* Bypass Thor Aorta to Carotid with Nonaut Sub, Open Approach */
"021W0KP"  =  "1"  /* Bypass Thor Aorta to Pulm Trunk w Nonaut Sub, Open */
"021W0KQ"  =  "1"  /* Bypass Thor Aorta to R Pulm Art w Nonaut Sub, Open */
"021W0KR"  =  "1"  /* Bypass Thor Aorta to L Pulm Art w Nonaut Sub, Open */
"021W0ZB"  =  "1"  /* Bypass Thoracic Aorta to Subclavian, Open Approach */
"021W0ZD"  =  "1"  /* Bypass Thoracic Aorta to Carotid, Open Approach */
"021W0ZP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk, Open Approach */
"021W0ZQ"  =  "1"  /* Bypass Thoracic Aorta to R Pulm Art, Open Approach */
"021W0ZR"  =  "1"  /* Bypass Thoracic Aorta to L Pulm Art, Open Approach */
"021W49B"  =  "1"  /* Bypass Thor Aorta to Subclav w Autol Vn, Perc Endo */
"021W49D"  =  "1"  /* Bypass Thor Aorta to Carotid w Autol Vn, Perc Endo */
"021W49P"  =  "1"  /* Bypass Thor Aorta to Pulm Trunk w Autol Vn, Perc Endo */
"021W49Q"  =  "1"  /* Bypass Thor Aorta to R Pulm Art w Autol Vn, Perc Endo */
"021W49R"  =  "1"  /* Bypass Thor Aorta to L Pulm Art w Autol Vn, Perc Endo */
"021W4AB"  =  "1"  /* Bypass Thor Aorta to Subclav w Autol Art, Perc Endo */
"021W4AD"  =  "1"  /* Bypass Thor Aorta to Carotid w Autol Art, Perc Endo */
"021W4AP"  =  "1"  /* Bypass Thor Aorta to Pulm Trunk w Autol Art, Perc Endo */
"021W4AQ"  =  "1"  /* Bypass Thor Aorta to R Pulm Art w Autol Art, Perc Endo */
"021W4AR"  =  "1"  /* Bypass Thor Aorta to L Pulm Art w Autol Art, Perc Endo */
"021W4JB"  =  "1"  /* Bypass Thor Aorta to Subclav w Synth Sub, Perc Endo */
"021W4JD"  =  "1"  /* Bypass Thor Aorta to Carotid w Synth Sub, Perc Endo */
"021W4JP"  =  "1"  /* Bypass Thor Aorta to Pulm Trunk w Synth Sub, Perc Endo */
"021W4JQ"  =  "1"  /* Bypass Thor Aorta to R Pulm Art w Synth Sub, Perc Endo */
"021W4JR"  =  "1"  /* Bypass Thor Aorta to L Pulm Art w Synth Sub, Perc Endo */
"021W4KB"  =  "1"  /* Bypass Thor Aorta to Subclav w Nonaut Sub, Perc Endo */
"021W4KD"  =  "1"  /* Bypass Thor Aorta to Carotid w Nonaut Sub, Perc Endo */
"021W4KP"  =  "1"  /* Bypass Thor Aorta to Pulm Trunk w Nonaut Sub, Perc Endo */
"021W4KQ"  =  "1"  /* Bypass Thor Aorta to R Pulm Art w Nonaut Sub, Perc Endo */
"021W4KR"  =  "1"  /* Bypass Thor Aorta to L Pulm Art w Nonaut Sub, Perc Endo */
"021W4ZB"  =  "1"  /* Bypass Thoracic Aorta to Subclavian, Perc Endo Approach */
"021W4ZD"  =  "1"  /* Bypass Thoracic Aorta to Carotid, Perc Endo Approach */
"021W4ZP"  =  "1"  /* Bypass Thoracic Aorta to Pulmonary Trunk, Perc Endo Approach */
"021W4ZQ"  =  "1"  /* Bypass Thoracic Aorta to R Pulm Art, Perc Endo Approach */
"021W4ZR"  =  "1"  /* Bypass Thoracic Aorta to L Pulm Art, Perc Endo Approach */
"02540ZZ"  =  "1"  /* Destruction of Coronary Vein, Open Approach */
"02543ZZ"  =  "1"  /* Destruction of Coronary Vein, Percutaneous Approach */
"02544ZZ"  =  "1"  /* Destruction of Coronary Vein, Perc Endo Approach */
"02550ZZ"  =  "1"  /* Destruction of Atrial Septum, Open Approach */
"02553ZZ"  =  "1"  /* Destruction of Atrial Septum, Percutaneous Approach */
"02554ZZ"  =  "1"  /* Destruction of Atrial Septum, Perc Endo Approach */
"02560ZZ"  =  "1"  /* Destruction of Right Atrium, Open Approach */
"02563ZZ"  =  "1"  /* Destruction of Right Atrium, Percutaneous Approach */
"02564ZZ"  =  "1"  /* Destruction of Right Atrium, Perc Endo Approach */
"02570ZZ"  =  "1"  /* Destruction of Left Atrium, Open Approach */
"02573ZZ"  =  "1"  /* Destruction of Left Atrium, Percutaneous Approach */
"02574ZZ"  =  "1"  /* Destruction of Left Atrium, Percutaneous Endoscopic Approach */
"02580ZZ"  =  "1"  /* Destruction of Conduction Mechanism, Open Approach */
"02583ZZ"  =  "1"  /* Destruction of Conduction Mechanism, Percutaneous Approach */
"02584ZZ"  =  "1"  /* Destruction of Conduction Mechanism, Perc Endo Approach */
"02590ZZ"  =  "1"  /* Destruction of Chordae Tendineae, Open Approach */
"02593ZZ"  =  "1"  /* Destruction of Chordae Tendineae, Percutaneous Approach */
"02594ZZ"  =  "1"  /* Destruction of Chordae Tendineae, Perc Endo Approach */
"025D0ZZ"  =  "1"  /* Destruction of Papillary Muscle, Open Approach */
"025D3ZZ"  =  "1"  /* Destruction of Papillary Muscle, Percutaneous Approach */
"025D4ZZ"  =  "1"  /* Destruction of Papillary Muscle, Perc Endo Approach */
"025F0ZZ"  =  "1"  /* Destruction of Aortic Valve, Open Approach */
"025F3ZZ"  =  "1"  /* Destruction of Aortic Valve, Percutaneous Approach */
"025F4ZZ"  =  "1"  /* Destruction of Aortic Valve, Perc Endo Approach */
"025G0ZZ"  =  "1"  /* Destruction of Mitral Valve, Open Approach */
"025G3ZZ"  =  "1"  /* Destruction of Mitral Valve, Percutaneous Approach */
"025G4ZZ"  =  "1"  /* Destruction of Mitral Valve, Perc Endo Approach */
"025H0ZZ"  =  "1"  /* Destruction of Pulmonary Valve, Open Approach */
"025H3ZZ"  =  "1"  /* Destruction of Pulmonary Valve, Percutaneous Approach */
"025H4ZZ"  =  "1"  /* Destruction of Pulmonary Valve, Perc Endo Approach */
"025J0ZZ"  =  "1"  /* Destruction of Tricuspid Valve, Open Approach */
"025J3ZZ"  =  "1"  /* Destruction of Tricuspid Valve, Percutaneous Approach */
"025J4ZZ"  =  "1"  /* Destruction of Tricuspid Valve, Perc Endo Approach */
"025K0ZZ"  =  "1"  /* Destruction of Right Ventricle, Open Approach */
"025K3ZZ"  =  "1"  /* Destruction of Right Ventricle, Percutaneous Approach */
"025K4ZZ"  =  "1"  /* Destruction of Right Ventricle, Perc Endo Approach */
"025L0ZZ"  =  "1"  /* Destruction of Left Ventricle, Open Approach */
"025L3ZZ"  =  "1"  /* Destruction of Left Ventricle, Percutaneous Approach */
"025L4ZZ"  =  "1"  /* Destruction of Left Ventricle, Perc Endo Approach */
"025M0ZZ"  =  "1"  /* Destruction of Ventricular Septum, Open Approach */
"025M3ZZ"  =  "1"  /* Destruction of Ventricular Septum, Percutaneous Approach */
"025M4ZZ"  =  "1"  /* Destruction of Ventricular Septum, Perc Endo Approach */
"025N0ZZ"  =  "1"  /* Destruction of Pericardium, Open Approach */
"025N3ZZ"  =  "1"  /* Destruction of Pericardium, Percutaneous Approach */
"025N4ZZ"  =  "1"  /* Destruction of Pericardium, Percutaneous Endoscopic Approach */
"025P0ZZ"  =  "1"  /* Destruction of Pulmonary Trunk, Open Approach */
"025P3ZZ"  =  "1"  /* Destruction of Pulmonary Trunk, Percutaneous Approach */
"025P4ZZ"  =  "1"  /* Destruction of Pulmonary Trunk, Perc Endo Approach */
"025Q0ZZ"  =  "1"  /* Destruction of Right Pulmonary Artery, Open Approach */
"025Q3ZZ"  =  "1"  /* Destruction of Right Pulmonary Artery, Percutaneous Approach */
"025Q4ZZ"  =  "1"  /* Destruction of Right Pulmonary Artery, Perc Endo Approach */
"025R0ZZ"  =  "1"  /* Destruction of Left Pulmonary Artery, Open Approach */
"025R3ZZ"  =  "1"  /* Destruction of Left Pulmonary Artery, Percutaneous Approach */
"025R4ZZ"  =  "1"  /* Destruction of Left Pulmonary Artery, Perc Endo Approach */
"025S0ZZ"  =  "1"  /* Destruction of Right Pulmonary Vein, Open Approach */
"025S3ZZ"  =  "1"  /* Destruction of Right Pulmonary Vein, Percutaneous Approach */
"025S4ZZ"  =  "1"  /* Destruction of Right Pulmonary Vein, Perc Endo Approach */
"025T0ZZ"  =  "1"  /* Destruction of Left Pulmonary Vein, Open Approach */
"025T3ZZ"  =  "1"  /* Destruction of Left Pulmonary Vein, Percutaneous Approach */
"025T4ZZ"  =  "1"  /* Destruction of Left Pulmonary Vein, Perc Endo Approach */
"025V0ZZ"  =  "1"  /* Destruction of Superior Vena Cava, Open Approach */
"025V3ZZ"  =  "1"  /* Destruction of Superior Vena Cava, Percutaneous Approach */
"025V4ZZ"  =  "1"  /* Destruction of Superior Vena Cava, Perc Endo Approach */
"025W0ZZ"  =  "1"  /* Destruction of Thoracic Aorta, Open Approach */
"025W3ZZ"  =  "1"  /* Destruction of Thoracic Aorta, Percutaneous Approach */
"025W4ZZ"  =  "1"  /* Destruction of Thoracic Aorta, Perc Endo Approach */
"0270046"  =  "1"  /* Dilate 1 Cor Art, Bifurc, w Drug-elut Intralum, Open */
"027004Z"  =  "1"  /* Dilation of 1 Cor Art with Drug-elut Intralum, Open Approach */
"02700D6"  =  "1"  /* Dilate 1 Cor Art, Bifurc, w Intralum Dev, Open */
"02700DZ"  =  "1"  /* Dilation of 1 Cor Art with Intralum Dev, Open Approach */
"02700T6"  =  "1"  /* Dilate 1 Cor Art, Bifurc, w Radioact Intralum, Open */
"02700TZ"  =  "1"  /* Dilation of 1 Cor Art with Radioact Intralum, Open Approach */
"02700Z6"  =  "1"  /* Dilation of Coronary Artery, One Site, Bifurc, Open Approach */
"02700ZZ"  =  "1"  /* Dilation of Coronary Artery, One Site, Open Approach */
"0270346"  =  "1"  /* Dilate 1 Cor Art, Bifurc, w Drug-elut Intralum, Perc */
"027034Z"  =  "1"  /* Dilation of 1 Cor Art with Drug-elut Intralum, Perc Approach */
"02703D6"  =  "1"  /* Dilate 1 Cor Art, Bifurc, w Intralum Dev, Perc */
"02703DZ"  =  "1"  /* Dilation of 1 Cor Art with Intralum Dev, Perc Approach */
"02703T6"  =  "1"  /* Dilate 1 Cor Art, Bifurc, w Radioact Intralum, Perc */
"02703TZ"  =  "1"  /* Dilation of 1 Cor Art with Radioact Intralum, Perc Approach */
"02703Z6"  =  "1"  /* Dilation of Coronary Artery, One Site, Bifurc, Perc Approach */
"02703ZZ"  =  "1"  /* Dilation of Coronary Artery, One Site, Percutaneous Approach */
"0270446"  =  "1"  /* Dilate 1 Cor Art, Bifurc, w Drug-elut Intralum, Perc Endo */
"027044Z"  =  "1"  /* Dilate 1 Cor Art w Drug-elut Intralum, Perc Endo */
"02704D6"  =  "1"  /* Dilate 1 Cor Art, Bifurc, w Intralum Dev, Perc Endo */
"02704DZ"  =  "1"  /* Dilation of 1 Cor Art with Intralum Dev, Perc Endo Approach */
"02704T6"  =  "1"  /* Dilate 1 Cor Art, Bifurc, w Radioact Intralum, Perc Endo */
"02704TZ"  =  "1"  /* Dilate 1 Cor Art w Radioact Intralum, Perc Endo */
"02704Z6"  =  "1"  /* Dilation of 1 Cor Art, Bifurc, Perc Endo Approach */
"02704ZZ"  =  "1"  /* Dilation of Coronary Artery, One Site, Perc Endo Approach */
"0271046"  =  "1"  /* Dilate 2 Cor Art, Bifurc, w Drug-elut Intralum, Open */
"027104Z"  =  "1"  /* Dilation of 2 Cor Art with Drug-elut Intralum, Open Approach */
"02710D6"  =  "1"  /* Dilate 2 Cor Art, Bifurc, w Intralum Dev, Open */
"02710DZ"  =  "1"  /* Dilation of 2 Cor Art with Intralum Dev, Open Approach */
"02710T6"  =  "1"  /* Dilate 2 Cor Art, Bifurc, w Radioact Intralum, Open */
"02710TZ"  =  "1"  /* Dilation of 2 Cor Art with Radioact Intralum, Open Approach */
"02710Z6"  =  "1"  /* Dilation of 2 Cor Art, Bifurc, Open Approach */
"02710ZZ"  =  "1"  /* Dilation of Coronary Artery, Two Sites, Open Approach */
"0271346"  =  "1"  /* Dilate 2 Cor Art, Bifurc, w Drug-elut Intralum, Perc */
"027134Z"  =  "1"  /* Dilation of 2 Cor Art with Drug-elut Intralum, Perc Approach */
"02713D6"  =  "1"  /* Dilate 2 Cor Art, Bifurc, w Intralum Dev, Perc */
"02713DZ"  =  "1"  /* Dilation of 2 Cor Art with Intralum Dev, Perc Approach */
"02713T6"  =  "1"  /* Dilate 2 Cor Art, Bifurc, w Radioact Intralum, Perc */
"02713TZ"  =  "1"  /* Dilation of 2 Cor Art with Radioact Intralum, Perc Approach */
"02713Z6"  =  "1"  /* Dilation of 2 Cor Art, Bifurc, Perc Approach */
"02713ZZ"  =  "1"  /* Dilation of Coronary Artery, Two Sites, Perc Approach */
"0271446"  =  "1"  /* Dilate 2 Cor Art, Bifurc, w Drug-elut Intralum, Perc Endo */
"027144Z"  =  "1"  /* Dilate 2 Cor Art w Drug-elut Intralum, Perc Endo */
"02714D6"  =  "1"  /* Dilate 2 Cor Art, Bifurc, w Intralum Dev, Perc Endo */
"02714DZ"  =  "1"  /* Dilation of 2 Cor Art with Intralum Dev, Perc Endo Approach */
"02714T6"  =  "1"  /* Dilate 2 Cor Art, Bifurc, w Radioact Intralum, Perc Endo */
"02714TZ"  =  "1"  /* Dilate 2 Cor Art w Radioact Intralum, Perc Endo */
"02714Z6"  =  "1"  /* Dilation of 2 Cor Art, Bifurc, Perc Endo Approach */
"02714ZZ"  =  "1"  /* Dilation of Coronary Artery, Two Sites, Perc Endo Approach */
"0272046"  =  "1"  /* Dilate 3 Cor Art, Bifurc, w Drug-elut Intralum, Open */
"027204Z"  =  "1"  /* Dilation of 3 Cor Art with Drug-elut Intralum, Open Approach */
"02720D6"  =  "1"  /* Dilate 3 Cor Art, Bifurc, w Intralum Dev, Open */
"02720DZ"  =  "1"  /* Dilation of 3 Cor Art with Intralum Dev, Open Approach */
"02720T6"  =  "1"  /* Dilate 3 Cor Art, Bifurc, w Radioact Intralum, Open */
"02720TZ"  =  "1"  /* Dilation of 3 Cor Art with Radioact Intralum, Open Approach */
"02720Z6"  =  "1"  /* Dilation of 3 Cor Art, Bifurc, Open Approach */
"02720ZZ"  =  "1"  /* Dilation of Coronary Artery, Three Sites, Open Approach */
"0272346"  =  "1"  /* Dilate 3 Cor Art, Bifurc, w Drug-elut Intralum, Perc */
"027234Z"  =  "1"  /* Dilation of 3 Cor Art with Drug-elut Intralum, Perc Approach */
"02723D6"  =  "1"  /* Dilate 3 Cor Art, Bifurc, w Intralum Dev, Perc */
"02723DZ"  =  "1"  /* Dilation of 3 Cor Art with Intralum Dev, Perc Approach */
"02723T6"  =  "1"  /* Dilate 3 Cor Art, Bifurc, w Radioact Intralum, Perc */
"02723TZ"  =  "1"  /* Dilation of 3 Cor Art with Radioact Intralum, Perc Approach */
"02723Z6"  =  "1"  /* Dilation of 3 Cor Art, Bifurc, Perc Approach */
"02723ZZ"  =  "1"  /* Dilation of Coronary Artery, Three Sites, Perc Approach */
"0272446"  =  "1"  /* Dilate 3 Cor Art, Bifurc, w Drug-elut Intralum, Perc Endo */
"027244Z"  =  "1"  /* Dilate 3 Cor Art w Drug-elut Intralum, Perc Endo */
"02724D6"  =  "1"  /* Dilate 3 Cor Art, Bifurc, w Intralum Dev, Perc Endo */
"02724DZ"  =  "1"  /* Dilation of 3 Cor Art with Intralum Dev, Perc Endo Approach */
"02724T6"  =  "1"  /* Dilate 3 Cor Art, Bifurc, w Radioact Intralum, Perc Endo */
"02724TZ"  =  "1"  /* Dilate 3 Cor Art w Radioact Intralum, Perc Endo */
"02724Z6"  =  "1"  /* Dilation of 3 Cor Art, Bifurc, Perc Endo Approach */
"02724ZZ"  =  "1"  /* Dilation of Coronary Artery, Three Sites, Perc Endo Approach */
"0273046"  =  "1"  /* Dilate 4+ Cor Art, Bifurc, w Drug-elut Intralum, Open */
"027304Z"  =  "1"  /* Dilate of 4+ Cor Art with Drug-elut Intralum, Open Approach */
"02730D6"  =  "1"  /* Dilate 4+ Cor Art, Bifurc, w Intralum Dev, Open */
"02730DZ"  =  "1"  /* Dilation of 4+ Cor Art with Intralum Dev, Open Approach */
"02730T6"  =  "1"  /* Dilate 4+ Cor Art, Bifurc, w Radioact Intralum, Open */
"02730TZ"  =  "1"  /* Dilation of 4+ Cor Art with Radioact Intralum, Open Approach */
"02730Z6"  =  "1"  /* Dilation of 4+ Cor Art, Bifurc, Open Approach */
"02730ZZ"  =  "1"  /* Dilation of 4+ Cor Art, Open Approach */
"0273346"  =  "1"  /* Dilate 4+ Cor Art, Bifurc, w Drug-elut Intralum, Perc */
"027334Z"  =  "1"  /* Dilate of 4+ Cor Art with Drug-elut Intralum, Perc Approach */
"02733D6"  =  "1"  /* Dilate 4+ Cor Art, Bifurc, w Intralum Dev, Perc */
"02733DZ"  =  "1"  /* Dilation of 4+ Cor Art with Intralum Dev, Perc Approach */
"02733T6"  =  "1"  /* Dilate 4+ Cor Art, Bifurc, w Radioact Intralum, Perc */
"02733TZ"  =  "1"  /* Dilation of 4+ Cor Art with Radioact Intralum, Perc Approach */
"02733Z6"  =  "1"  /* Dilation of 4+ Cor Art, Bifurc, Perc Approach */
"02733ZZ"  =  "1"  /* Dilation of 4+ Cor Art, Perc Approach */
"0273446"  =  "1"  /* Dilate 4+ Cor Art, Bifurc, w Drug-elut Intralum, Perc Endo */
"027344Z"  =  "1"  /* Dilate 4+ Cor Art w Drug-elut Intralum, Perc Endo */
"02734D6"  =  "1"  /* Dilate 4+ Cor Art, Bifurc, w Intralum Dev, Perc Endo */
"02734DZ"  =  "1"  /* Dilation of 4+ Cor Art with Intralum Dev, Perc Endo Approach */
"02734T6"  =  "1"  /* Dilate 4+ Cor Art, Bifurc, w Radioact Intralum, Perc Endo */
"02734TZ"  =  "1"  /* Dilate 4+ Cor Art w Radioact Intralum, Perc Endo */
"02734Z6"  =  "1"  /* Dilation of 4+ Cor Art, Bifurc, Perc Endo Approach */
"02734ZZ"  =  "1"  /* Dilation of 4+ Cor Art, Perc Endo Approach */
"027F04Z"  =  "1"  /* Dilate Aortic Valve w Drug-elut Intralum, Open */
"027F0DZ"  =  "1"  /* Dilation of Aortic Valve with Intralum Dev, Open Approach */
"027F0ZZ"  =  "1"  /* Dilation of Aortic Valve, Open Approach */
"027F34Z"  =  "1"  /* Dilate Aortic Valve w Drug-elut Intralum, Perc */
"027F3DZ"  =  "1"  /* Dilation of Aortic Valve with Intralum Dev, Perc Approach */
"027F3ZZ"  =  "1"  /* Dilation of Aortic Valve, Percutaneous Approach */
"027F44Z"  =  "1"  /* Dilate Aortic Valve w Drug-elut Intralum, Perc Endo */
"027F4DZ"  =  "1"  /* Dilate of Aortic Valve with Intralum Dev, Perc Endo Approach */
"027F4ZZ"  =  "1"  /* Dilation of Aortic Valve, Percutaneous Endoscopic Approach */
"027G04Z"  =  "1"  /* Dilate Mitral Valve w Drug-elut Intralum, Open */
"027G0DZ"  =  "1"  /* Dilation of Mitral Valve with Intralum Dev, Open Approach */
"027G0ZZ"  =  "1"  /* Dilation of Mitral Valve, Open Approach */
"027G34Z"  =  "1"  /* Dilate Mitral Valve w Drug-elut Intralum, Perc */
"027G3DZ"  =  "1"  /* Dilation of Mitral Valve with Intralum Dev, Perc Approach */
"027G3ZZ"  =  "1"  /* Dilation of Mitral Valve, Percutaneous Approach */
"027G44Z"  =  "1"  /* Dilate Mitral Valve w Drug-elut Intralum, Perc Endo */
"027G4DZ"  =  "1"  /* Dilate of Mitral Valve with Intralum Dev, Perc Endo Approach */
"027G4ZZ"  =  "1"  /* Dilation of Mitral Valve, Percutaneous Endoscopic Approach */
"027H04Z"  =  "1"  /* Dilate of Pulm Valve with Drug-elut Intralum, Open Approach */
"027H0DZ"  =  "1"  /* Dilation of Pulmonary Valve with Intralum Dev, Open Approach */
"027H0ZZ"  =  "1"  /* Dilation of Pulmonary Valve, Open Approach */
"027H34Z"  =  "1"  /* Dilate of Pulm Valve with Drug-elut Intralum, Perc Approach */
"027H3DZ"  =  "1"  /* Dilation of Pulmonary Valve with Intralum Dev, Perc Approach */
"027H3ZZ"  =  "1"  /* Dilation of Pulmonary Valve, Percutaneous Approach */
"027H44Z"  =  "1"  /* Dilate Pulm Valve w Drug-elut Intralum, Perc Endo */
"027H4DZ"  =  "1"  /* Dilation of Pulm Valve with Intralum Dev, Perc Endo Approach */
"027H4ZZ"  =  "1"  /* Dilation of Pulmonary Valve, Perc Endo Approach */
"027J04Z"  =  "1"  /* Dilate Tricusp Valve w Drug-elut Intralum, Open */
"027J0DZ"  =  "1"  /* Dilation of Tricuspid Valve with Intralum Dev, Open Approach */
"027J0ZZ"  =  "1"  /* Dilation of Tricuspid Valve, Open Approach */
"027J34Z"  =  "1"  /* Dilate Tricusp Valve w Drug-elut Intralum, Perc */
"027J3DZ"  =  "1"  /* Dilation of Tricuspid Valve with Intralum Dev, Perc Approach */
"027J3ZZ"  =  "1"  /* Dilation of Tricuspid Valve, Percutaneous Approach */
"027J44Z"  =  "1"  /* Dilate Tricusp Valve w Drug-elut Intralum, Perc Endo */
"027J4DZ"  =  "1"  /* Dilate Tricusp Valve w Intralum Dev, Perc Endo */
"027J4ZZ"  =  "1"  /* Dilation of Tricuspid Valve, Perc Endo Approach */
"027K04Z"  =  "1"  /* Dilate of R Ventricle with Drug-elut Intralum, Open Approach */
"027K0DZ"  =  "1"  /* Dilation of Right Ventricle with Intralum Dev, Open Approach */
"027K0ZZ"  =  "1"  /* Dilation of Right Ventricle, Open Approach */
"027K34Z"  =  "1"  /* Dilate of R Ventricle with Drug-elut Intralum, Perc Approach */
"027K3DZ"  =  "1"  /* Dilation of Right Ventricle with Intralum Dev, Perc Approach */
"027K3ZZ"  =  "1"  /* Dilation of Right Ventricle, Percutaneous Approach */
"027K44Z"  =  "1"  /* Dilate R Ventricle w Drug-elut Intralum, Perc Endo */
"027K4DZ"  =  "1"  /* Dilate of R Ventricle with Intralum Dev, Perc Endo Approach */
"027K4ZZ"  =  "1"  /* Dilation of Right Ventricle, Perc Endo Approach */
"027P04Z"  =  "1"  /* Dilate of Pulm Trunk with Drug-elut Intralum, Open Approach */
"027P0DZ"  =  "1"  /* Dilation of Pulmonary Trunk with Intralum Dev, Open Approach */
"027P0ZZ"  =  "1"  /* Dilation of Pulmonary Trunk, Open Approach */
"027P34Z"  =  "1"  /* Dilate of Pulm Trunk with Drug-elut Intralum, Perc Approach */
"027P3DZ"  =  "1"  /* Dilation of Pulmonary Trunk with Intralum Dev, Perc Approach */
"027P3ZZ"  =  "1"  /* Dilation of Pulmonary Trunk, Percutaneous Approach */
"027P44Z"  =  "1"  /* Dilate Pulm Trunk w Drug-elut Intralum, Perc Endo */
"027P4DZ"  =  "1"  /* Dilation of Pulm Trunk with Intralum Dev, Perc Endo Approach */
"027P4ZZ"  =  "1"  /* Dilation of Pulmonary Trunk, Perc Endo Approach */
"027Q04Z"  =  "1"  /* Dilate of R Pulm Art with Drug-elut Intralum, Open Approach */
"027Q0DZ"  =  "1"  /* Dilation of R Pulm Art with Intralum Dev, Open Approach */
"027Q0ZZ"  =  "1"  /* Dilation of Right Pulmonary Artery, Open Approach */
"027Q34Z"  =  "1"  /* Dilate of R Pulm Art with Drug-elut Intralum, Perc Approach */
"027Q3DZ"  =  "1"  /* Dilation of R Pulm Art with Intralum Dev, Perc Approach */
"027Q3ZZ"  =  "1"  /* Dilation of Right Pulmonary Artery, Percutaneous Approach */
"027Q44Z"  =  "1"  /* Dilate R Pulm Art w Drug-elut Intralum, Perc Endo */
"027Q4DZ"  =  "1"  /* Dilation of R Pulm Art with Intralum Dev, Perc Endo Approach */
"027Q4ZZ"  =  "1"  /* Dilation of Right Pulmonary Artery, Perc Endo Approach */
"027R04T"  =  "1"  /* Dilate Ductus Arterio w Drug-elut Intralum, Open */
"027R04Z"  =  "1"  /* Dilate of L Pulm Art with Drug-elut Intralum, Open Approach */
"027R0DT"  =  "1"  /* Dilation of Ductus Arterio with Intralum Dev, Open Approach */
"027R0DZ"  =  "1"  /* Dilation of L Pulm Art with Intralum Dev, Open Approach */
"027R0ZT"  =  "1"  /* Dilation of Ductus Arteriosus, Open Approach */
"027R0ZZ"  =  "1"  /* Dilation of Left Pulmonary Artery, Open Approach */
"027R34T"  =  "1"  /* Dilate Ductus Arterio w Drug-elut Intralum, Perc */
"027R34Z"  =  "1"  /* Dilate of L Pulm Art with Drug-elut Intralum, Perc Approach */
"027R3DT"  =  "1"  /* Dilation of Ductus Arterio with Intralum Dev, Perc Approach */
"027R3DZ"  =  "1"  /* Dilation of L Pulm Art with Intralum Dev, Perc Approach */
"027R3ZT"  =  "1"  /* Dilation of Ductus Arteriosus, Percutaneous Approach */
"027R3ZZ"  =  "1"  /* Dilation of Left Pulmonary Artery, Percutaneous Approach */
"027R44T"  =  "1"  /* Dilate Ductus Arterio w Drug-elut Intralum, Perc Endo */
"027R44Z"  =  "1"  /* Dilate L Pulm Art w Drug-elut Intralum, Perc Endo */
"027R4DT"  =  "1"  /* Dilate Ductus Arterio w Intralum Dev, Perc Endo */
"027R4DZ"  =  "1"  /* Dilation of L Pulm Art with Intralum Dev, Perc Endo Approach */
"027R4ZT"  =  "1"  /* Dilation of Ductus Arteriosus, Perc Endo Approach */
"027R4ZZ"  =  "1"  /* Dilation of Left Pulmonary Artery, Perc Endo Approach */
"027S04Z"  =  "1"  /* Dilate of R Pulm Vein with Drug-elut Intralum, Open Approach */
"027S0DZ"  =  "1"  /* Dilation of R Pulm Vein with Intralum Dev, Open Approach */
"027S0ZZ"  =  "1"  /* Dilation of Right Pulmonary Vein, Open Approach */
"027S34Z"  =  "1"  /* Dilate of R Pulm Vein with Drug-elut Intralum, Perc Approach */
"027S3DZ"  =  "1"  /* Dilation of R Pulm Vein with Intralum Dev, Perc Approach */
"027S3ZZ"  =  "1"  /* Dilation of Right Pulmonary Vein, Percutaneous Approach */
"027S44Z"  =  "1"  /* Dilate R Pulm Vein w Drug-elut Intralum, Perc Endo */
"027S4DZ"  =  "1"  /* Dilate of R Pulm Vein with Intralum Dev, Perc Endo Approach */
"027S4ZZ"  =  "1"  /* Dilation of Right Pulmonary Vein, Perc Endo Approach */
"027T04Z"  =  "1"  /* Dilate of L Pulm Vein with Drug-elut Intralum, Open Approach */
"027T0DZ"  =  "1"  /* Dilation of L Pulm Vein with Intralum Dev, Open Approach */
"027T0ZZ"  =  "1"  /* Dilation of Left Pulmonary Vein, Open Approach */
"027T34Z"  =  "1"  /* Dilate of L Pulm Vein with Drug-elut Intralum, Perc Approach */
"027T3DZ"  =  "1"  /* Dilation of L Pulm Vein with Intralum Dev, Perc Approach */
"027T3ZZ"  =  "1"  /* Dilation of Left Pulmonary Vein, Percutaneous Approach */
"027T44Z"  =  "1"  /* Dilate L Pulm Vein w Drug-elut Intralum, Perc Endo */
"027T4DZ"  =  "1"  /* Dilate of L Pulm Vein with Intralum Dev, Perc Endo Approach */
"027T4ZZ"  =  "1"  /* Dilation of Left Pulmonary Vein, Perc Endo Approach */
"027V04Z"  =  "1"  /* Dilate Sup Vena Cava w Drug-elut Intralum, Open */
"027V0DZ"  =  "1"  /* Dilation of Sup Vena Cava with Intralum Dev, Open Approach */
"027V0ZZ"  =  "1"  /* Dilation of Superior Vena Cava, Open Approach */
"027V34Z"  =  "1"  /* Dilate Sup Vena Cava w Drug-elut Intralum, Perc */
"027V3DZ"  =  "1"  /* Dilation of Sup Vena Cava with Intralum Dev, Perc Approach */
"027V3ZZ"  =  "1"  /* Dilation of Superior Vena Cava, Percutaneous Approach */
"027V44Z"  =  "1"  /* Dilate Sup Vena Cava w Drug-elut Intralum, Perc Endo */
"027V4DZ"  =  "1"  /* Dilate Sup Vena Cava w Intralum Dev, Perc Endo */
"027V4ZZ"  =  "1"  /* Dilation of Superior Vena Cava, Perc Endo Approach */
"027W04Z"  =  "1"  /* Dilate of Thor Aorta with Drug-elut Intralum, Open Approach */
"027W0DZ"  =  "1"  /* Dilation of Thoracic Aorta with Intralum Dev, Open Approach */
"027W0ZZ"  =  "1"  /* Dilation of Thoracic Aorta, Open Approach */
"027W34Z"  =  "1"  /* Dilate of Thor Aorta with Drug-elut Intralum, Perc Approach */
"027W3DZ"  =  "1"  /* Dilation of Thoracic Aorta with Intralum Dev, Perc Approach */
"027W3ZZ"  =  "1"  /* Dilation of Thoracic Aorta, Percutaneous Approach */
"027W44Z"  =  "1"  /* Dilate Thor Aorta w Drug-elut Intralum, Perc Endo */
"027W4DZ"  =  "1"  /* Dilation of Thor Aorta with Intralum Dev, Perc Endo Approach */
"027W4ZZ"  =  "1"  /* Dilation of Thoracic Aorta, Percutaneous Endoscopic Approach */
"02880ZZ"  =  "1"  /* Division of Conduction Mechanism, Open Approach */
"02883ZZ"  =  "1"  /* Division of Conduction Mechanism, Percutaneous Approach */
"02884ZZ"  =  "1"  /* Division of Conduction Mechanism, Perc Endo Approach */
"02890ZZ"  =  "1"  /* Division of Chordae Tendineae, Open Approach */
"02893ZZ"  =  "1"  /* Division of Chordae Tendineae, Percutaneous Approach */
"02894ZZ"  =  "1"  /* Division of Chordae Tendineae, Perc Endo Approach */
"028D0ZZ"  =  "1"  /* Division of Papillary Muscle, Open Approach */
"028D3ZZ"  =  "1"  /* Division of Papillary Muscle, Percutaneous Approach */
"028D4ZZ"  =  "1"  /* Division of Papillary Muscle, Perc Endo Approach */
"02B40ZZ"  =  "1"  /* Excision of Coronary Vein, Open Approach */
"02B43ZZ"  =  "1"  /* Excision of Coronary Vein, Percutaneous Approach */
"02B44ZZ"  =  "1"  /* Excision of Coronary Vein, Percutaneous Endoscopic Approach */
"02B50ZZ"  =  "1"  /* Excision of Atrial Septum, Open Approach */
"02B53ZZ"  =  "1"  /* Excision of Atrial Septum, Percutaneous Approach */
"02B54ZZ"  =  "1"  /* Excision of Atrial Septum, Percutaneous Endoscopic Approach */
"02B60ZZ"  =  "1"  /* Excision of Right Atrium, Open Approach */
"02B63ZZ"  =  "1"  /* Excision of Right Atrium, Percutaneous Approach */
"02B64ZZ"  =  "1"  /* Excision of Right Atrium, Percutaneous Endoscopic Approach */
"02B70ZZ"  =  "1"  /* Excision of Left Atrium, Open Approach */
"02B73ZZ"  =  "1"  /* Excision of Left Atrium, Percutaneous Approach */
"02B74ZZ"  =  "1"  /* Excision of Left Atrium, Percutaneous Endoscopic Approach */
"02B80ZZ"  =  "1"  /* Excision of Conduction Mechanism, Open Approach */
"02B83ZZ"  =  "1"  /* Excision of Conduction Mechanism, Percutaneous Approach */
"02B84ZZ"  =  "1"  /* Excision of Conduction Mechanism, Perc Endo Approach */
"02B90ZZ"  =  "1"  /* Excision of Chordae Tendineae, Open Approach */
"02B93ZZ"  =  "1"  /* Excision of Chordae Tendineae, Percutaneous Approach */
"02B94ZZ"  =  "1"  /* Excision of Chordae Tendineae, Perc Endo Approach */
"02BD0ZZ"  =  "1"  /* Excision of Papillary Muscle, Open Approach */
"02BD3ZZ"  =  "1"  /* Excision of Papillary Muscle, Percutaneous Approach */
"02BD4ZZ"  =  "1"  /* Excision of Papillary Muscle, Perc Endo Approach */
"02BF0ZZ"  =  "1"  /* Excision of Aortic Valve, Open Approach */
"02BF3ZZ"  =  "1"  /* Excision of Aortic Valve, Percutaneous Approach */
"02BF4ZZ"  =  "1"  /* Excision of Aortic Valve, Percutaneous Endoscopic Approach */
"02BG0ZZ"  =  "1"  /* Excision of Mitral Valve, Open Approach */
"02BG3ZZ"  =  "1"  /* Excision of Mitral Valve, Percutaneous Approach */
"02BG4ZZ"  =  "1"  /* Excision of Mitral Valve, Percutaneous Endoscopic Approach */
"02BH0ZZ"  =  "1"  /* Excision of Pulmonary Valve, Open Approach */
"02BH3ZZ"  =  "1"  /* Excision of Pulmonary Valve, Percutaneous Approach */
"02BH4ZZ"  =  "1"  /* Excision of Pulmonary Valve, Perc Endo Approach */
"02BJ0ZZ"  =  "1"  /* Excision of Tricuspid Valve, Open Approach */
"02BJ3ZZ"  =  "1"  /* Excision of Tricuspid Valve, Percutaneous Approach */
"02BJ4ZZ"  =  "1"  /* Excision of Tricuspid Valve, Perc Endo Approach */
"02BK0ZZ"  =  "1"  /* Excision of Right Ventricle, Open Approach */
"02BK3ZZ"  =  "1"  /* Excision of Right Ventricle, Percutaneous Approach */
"02BK4ZZ"  =  "1"  /* Excision of Right Ventricle, Perc Endo Approach */
"02BL0ZZ"  =  "1"  /* Excision of Left Ventricle, Open Approach */
"02BL3ZZ"  =  "1"  /* Excision of Left Ventricle, Percutaneous Approach */
"02BL4ZZ"  =  "1"  /* Excision of Left Ventricle, Percutaneous Endoscopic Approach */
"02BM0ZZ"  =  "1"  /* Excision of Ventricular Septum, Open Approach */
"02BM3ZZ"  =  "1"  /* Excision of Ventricular Septum, Percutaneous Approach */
"02BM4ZZ"  =  "1"  /* Excision of Ventricular Septum, Perc Endo Approach */
"02BN0ZX"  =  "1"  /* Excision of Pericardium, Open Approach, Diagnostic */
"02BN0ZZ"  =  "1"  /* Excision of Pericardium, Open Approach */
"02BN3ZX"  =  "1"  /* Excision of Pericardium, Percutaneous Approach, Diagnostic */
"02BN3ZZ"  =  "1"  /* Excision of Pericardium, Percutaneous Approach */
"02BN4ZX"  =  "1"  /* Excision of Pericardium, Perc Endo Approach, Diagn */
"02BN4ZZ"  =  "1"  /* Excision of Pericardium, Percutaneous Endoscopic Approach */
"02BP0ZX"  =  "1"  /* Excision of Pulmonary Trunk, Open Approach, Diagnostic */
"02BP0ZZ"  =  "1"  /* Excision of Pulmonary Trunk, Open Approach */
"02BP3ZX"  =  "1"  /* Excision of Pulmonary Trunk, Percutaneous Approach, Diagn */
"02BP3ZZ"  =  "1"  /* Excision of Pulmonary Trunk, Percutaneous Approach */
"02BP4ZX"  =  "1"  /* Excision of Pulmonary Trunk, Perc Endo Approach, Diagn */
"02BP4ZZ"  =  "1"  /* Excision of Pulmonary Trunk, Perc Endo Approach */
"02BQ0ZX"  =  "1"  /* Excision of Right Pulmonary Artery, Open Approach, Diagn */
"02BQ0ZZ"  =  "1"  /* Excision of Right Pulmonary Artery, Open Approach */
"02BQ3ZX"  =  "1"  /* Excision of Right Pulmonary Artery, Perc Approach, Diagn */
"02BQ3ZZ"  =  "1"  /* Excision of Right Pulmonary Artery, Percutaneous Approach */
"02BQ4ZX"  =  "1"  /* Excision of R Pulm Art, Perc Endo Approach, Diagn */
"02BQ4ZZ"  =  "1"  /* Excision of Right Pulmonary Artery, Perc Endo Approach */
"02BR0ZX"  =  "1"  /* Excision of Left Pulmonary Artery, Open Approach, Diagnostic */
"02BR0ZZ"  =  "1"  /* Excision of Left Pulmonary Artery, Open Approach */
"02BR3ZX"  =  "1"  /* Excision of Left Pulmonary Artery, Perc Approach, Diagn */
"02BR3ZZ"  =  "1"  /* Excision of Left Pulmonary Artery, Percutaneous Approach */
"02BR4ZX"  =  "1"  /* Excision of Left Pulmonary Artery, Perc Endo Approach, Diagn */
"02BR4ZZ"  =  "1"  /* Excision of Left Pulmonary Artery, Perc Endo Approach */
"02BS0ZX"  =  "1"  /* Excision of Right Pulmonary Vein, Open Approach, Diagnostic */
"02BS0ZZ"  =  "1"  /* Excision of Right Pulmonary Vein, Open Approach */
"02BS3ZX"  =  "1"  /* Excision of Right Pulmonary Vein, Perc Approach, Diagn */
"02BS3ZZ"  =  "1"  /* Excision of Right Pulmonary Vein, Percutaneous Approach */
"02BS4ZX"  =  "1"  /* Excision of Right Pulmonary Vein, Perc Endo Approach, Diagn */
"02BS4ZZ"  =  "1"  /* Excision of Right Pulmonary Vein, Perc Endo Approach */
"02BT0ZX"  =  "1"  /* Excision of Left Pulmonary Vein, Open Approach, Diagnostic */
"02BT0ZZ"  =  "1"  /* Excision of Left Pulmonary Vein, Open Approach */
"02BT3ZX"  =  "1"  /* Excision of Left Pulmonary Vein, Perc Approach, Diagn */
"02BT3ZZ"  =  "1"  /* Excision of Left Pulmonary Vein, Percutaneous Approach */
"02BT4ZX"  =  "1"  /* Excision of Left Pulmonary Vein, Perc Endo Approach, Diagn */
"02BT4ZZ"  =  "1"  /* Excision of Left Pulmonary Vein, Perc Endo Approach */
"02BV0ZX"  =  "1"  /* Excision of Superior Vena Cava, Open Approach, Diagnostic */
"02BV0ZZ"  =  "1"  /* Excision of Superior Vena Cava, Open Approach */
"02BV3ZX"  =  "1"  /* Excision of Superior Vena Cava, Percutaneous Approach, Diagn */
"02BV3ZZ"  =  "1"  /* Excision of Superior Vena Cava, Percutaneous Approach */
"02BV4ZX"  =  "1"  /* Excision of Superior Vena Cava, Perc Endo Approach, Diagn */
"02BV4ZZ"  =  "1"  /* Excision of Superior Vena Cava, Perc Endo Approach */
"02BW0ZX"  =  "1"  /* Excision of Thoracic Aorta, Open Approach, Diagnostic */
"02BW0ZZ"  =  "1"  /* Excision of Thoracic Aorta, Open Approach */
"02BW3ZX"  =  "1"  /* Excision of Thoracic Aorta, Percutaneous Approach, Diagn */
"02BW3ZZ"  =  "1"  /* Excision of Thoracic Aorta, Percutaneous Approach */
"02BW4ZX"  =  "1"  /* Excision of Thoracic Aorta, Perc Endo Approach, Diagn */
"02BW4ZZ"  =  "1"  /* Excision of Thoracic Aorta, Percutaneous Endoscopic Approach */
"02C00ZZ"  =  "1"  /* Extirpation of Matter from 1 Cor Art, Open Approach */
"02C03ZZ"  =  "1"  /* Extirpation of Matter from 1 Cor Art, Perc Approach */
"02C04ZZ"  =  "1"  /* Extirpation of Matter from 1 Cor Art, Perc Endo Approach */
"02C10ZZ"  =  "1"  /* Extirpation of Matter from 2 Cor Art, Open Approach */
"02C13ZZ"  =  "1"  /* Extirpation of Matter from 2 Cor Art, Perc Approach */
"02C14ZZ"  =  "1"  /* Extirpation of Matter from 2 Cor Art, Perc Endo Approach */
"02C20ZZ"  =  "1"  /* Extirpation of Matter from 3 Cor Art, Open Approach */
"02C23ZZ"  =  "1"  /* Extirpation of Matter from 3 Cor Art, Perc Approach */
"02C24ZZ"  =  "1"  /* Extirpation of Matter from 3 Cor Art, Perc Endo Approach */
"02C30ZZ"  =  "1"  /* Extirpation of Matter from 4+ Cor Art, Open Approach */
"02C33ZZ"  =  "1"  /* Extirpation of Matter from 4+ Cor Art, Perc Approach */
"02C34ZZ"  =  "1"  /* Extirpation of Matter from 4+ Cor Art, Perc Endo Approach */
"02C40ZZ"  =  "1"  /* Extirpation of Matter from Coronary Vein, Open Approach */
"02C43ZZ"  =  "1"  /* Extirpation of Matter from Coronary Vein, Perc Approach */
"02C44ZZ"  =  "1"  /* Extirpation of Matter from Coronary Vein, Perc Endo Approach */
"02C50ZZ"  =  "1"  /* Extirpation of Matter from Atrial Septum, Open Approach */
"02C53ZZ"  =  "1"  /* Extirpation of Matter from Atrial Septum, Perc Approach */
"02C54ZZ"  =  "1"  /* Extirpation of Matter from Atrial Septum, Perc Endo Approach */
"02C60ZZ"  =  "1"  /* Extirpation of Matter from Right Atrium, Open Approach */
"02C63ZZ"  =  "1"  /* Extirpation of Matter from Right Atrium, Perc Approach */
"02C64ZZ"  =  "1"  /* Extirpation of Matter from Right Atrium, Perc Endo Approach */
"02C70ZZ"  =  "1"  /* Extirpation of Matter from Left Atrium, Open Approach */
"02C73ZZ"  =  "1"  /* Extirpation of Matter from Left Atrium, Perc Approach */
"02C74ZZ"  =  "1"  /* Extirpation of Matter from Left Atrium, Perc Endo Approach */
"02C80ZZ"  =  "1"  /* Extirpate of Matter from Conduction Mechanism, Open Approach */
"02C83ZZ"  =  "1"  /* Extirpate of Matter from Conduction Mechanism, Perc Approach */
"02C84ZZ"  =  "1"  /* Extirpate matter from Conduction Mechanism, Perc Endo */
"02C90ZZ"  =  "1"  /* Extirpation of Matter from Chordae Tendineae, Open Approach */
"02C93ZZ"  =  "1"  /* Extirpation of Matter from Chordae Tendineae, Perc Approach */
"02C94ZZ"  =  "1"  /* Extirpate matter from Chordae Tendineae, Perc Endo */
"02CD0ZZ"  =  "1"  /* Extirpation of Matter from Papillary Muscle, Open Approach */
"02CD3ZZ"  =  "1"  /* Extirpation of Matter from Papillary Muscle, Perc Approach */
"02CD4ZZ"  =  "1"  /* Extirpate matter from Papillary Muscle, Perc Endo */
"02CF0ZZ"  =  "1"  /* Extirpation of Matter from Aortic Valve, Open Approach */
"02CF3ZZ"  =  "1"  /* Extirpation of Matter from Aortic Valve, Perc Approach */
"02CF4ZZ"  =  "1"  /* Extirpation of Matter from Aortic Valve, Perc Endo Approach */
"02CG0ZZ"  =  "1"  /* Extirpation of Matter from Mitral Valve, Open Approach */
"02CG3ZZ"  =  "1"  /* Extirpation of Matter from Mitral Valve, Perc Approach */
"02CG4ZZ"  =  "1"  /* Extirpation of Matter from Mitral Valve, Perc Endo Approach */
"02CH0ZZ"  =  "1"  /* Extirpation of Matter from Pulmonary Valve, Open Approach */
"02CH3ZZ"  =  "1"  /* Extirpation of Matter from Pulmonary Valve, Perc Approach */
"02CH4ZZ"  =  "1"  /* Extirpation of Matter from Pulm Valve, Perc Endo Approach */
"02CJ0ZZ"  =  "1"  /* Extirpation of Matter from Tricuspid Valve, Open Approach */
"02CJ3ZZ"  =  "1"  /* Extirpation of Matter from Tricuspid Valve, Perc Approach */
"02CJ4ZZ"  =  "1"  /* Extirpation of Matter from Tricusp Valve, Perc Endo Approach */
"02CK0ZZ"  =  "1"  /* Extirpation of Matter from Right Ventricle, Open Approach */
"02CK3ZZ"  =  "1"  /* Extirpation of Matter from Right Ventricle, Perc Approach */
"02CK4ZZ"  =  "1"  /* Extirpation of Matter from R Ventricle, Perc Endo Approach */
"02CL0ZZ"  =  "1"  /* Extirpation of Matter from Left Ventricle, Open Approach */
"02CL3ZZ"  =  "1"  /* Extirpation of Matter from Left Ventricle, Perc Approach */
"02CL4ZZ"  =  "1"  /* Extirpation of Matter from L Ventricle, Perc Endo Approach */
"02CM0ZZ"  =  "1"  /* Extirpation of Matter from Ventricular Septum, Open Approach */
"02CM3ZZ"  =  "1"  /* Extirpation of Matter from Ventricular Septum, Perc Approach */
"02CM4ZZ"  =  "1"  /* Extirpate matter from Ventricular Sept, Perc Endo */
"02CN0ZZ"  =  "1"  /* Extirpation of Matter from Pericardium, Open Approach */
"02CN3ZZ"  =  "1"  /* Extirpation of Matter from Pericardium, Perc Approach */
"02CN4ZZ"  =  "1"  /* Extirpation of Matter from Pericardium, Perc Endo Approach */
"02CP0ZZ"  =  "1"  /* Extirpation of Matter from Pulmonary Trunk, Open Approach */
"02CP3ZZ"  =  "1"  /* Extirpation of Matter from Pulmonary Trunk, Perc Approach */
"02CP4ZZ"  =  "1"  /* Extirpation of Matter from Pulm Trunk, Perc Endo Approach */
"02CQ0ZZ"  =  "1"  /* Extirpation of Matter from R Pulm Art, Open Approach */
"02CQ3ZZ"  =  "1"  /* Extirpation of Matter from R Pulm Art, Perc Approach */
"02CQ4ZZ"  =  "1"  /* Extirpation of Matter from R Pulm Art, Perc Endo Approach */
"02CR0ZZ"  =  "1"  /* Extirpation of Matter from L Pulm Art, Open Approach */
"02CR3ZZ"  =  "1"  /* Extirpation of Matter from L Pulm Art, Perc Approach */
"02CR4ZZ"  =  "1"  /* Extirpation of Matter from L Pulm Art, Perc Endo Approach */
"02CS0ZZ"  =  "1"  /* Extirpation of Matter from R Pulm Vein, Open Approach */
"02CS3ZZ"  =  "1"  /* Extirpation of Matter from R Pulm Vein, Perc Approach */
"02CS4ZZ"  =  "1"  /* Extirpation of Matter from R Pulm Vein, Perc Endo Approach */
"02CT0ZZ"  =  "1"  /* Extirpation of Matter from L Pulm Vein, Open Approach */
"02CT3ZZ"  =  "1"  /* Extirpation of Matter from L Pulm Vein, Perc Approach */
"02CT4ZZ"  =  "1"  /* Extirpation of Matter from L Pulm Vein, Perc Endo Approach */
"02CV0ZZ"  =  "1"  /* Extirpation of Matter from Superior Vena Cava, Open Approach */
"02CV3ZZ"  =  "1"  /* Extirpation of Matter from Superior Vena Cava, Perc Approach */
"02CV4ZZ"  =  "1"  /* Extirpation of Matter from Sup Vena Cava, Perc Endo Approach */
"02CW0ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Aorta, Open Approach */
"02CW3ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Aorta, Perc Approach */
"02CW4ZZ"  =  "1"  /* Extirpation of Matter from Thor Aorta, Perc Endo Approach */
"02FN0ZZ"  =  "1"  /* Fragmentation in Pericardium, Open Approach */
"02FN3ZZ"  =  "1"  /* Fragmentation in Pericardium, Percutaneous Approach */
"02FN4ZZ"  =  "1"  /* Fragmentation in Pericardium, Perc Endo Approach */
"02H400Z"  =  "1"  /* Insertion of Pressure Sens into Cor Vein, Open Approach */
"02H402Z"  =  "1"  /* Insertion of Monitoring Device into Cor Vein, Open Approach */
"02H403Z"  =  "1"  /* Insertion of Infusion Device into Cor Vein, Open Approach */
"02H40DZ"  =  "1"  /* Insertion of Intralum Dev into Cor Vein, Open Approach */
"02H40KZ"  =  "1"  /* Insertion of Defibrillator Lead into Cor Vein, Open Approach */
"02H430Z"  =  "1"  /* Insertion of Pressure Sens into Cor Vein, Perc Approach */
"02H432Z"  =  "1"  /* Insertion of Monitoring Device into Cor Vein, Perc Approach */
"02H433Z"  =  "1"  /* Insertion of Infusion Device into Cor Vein, Perc Approach */
"02H43DZ"  =  "1"  /* Insertion of Intralum Dev into Cor Vein, Perc Approach */
"02H43JZ"  =  "1"  /* Insertion of Pacemaker Lead into Cor Vein, Perc Approach */
"02H43KZ"  =  "1"  /* Insertion of Defibrillator Lead into Cor Vein, Perc Approach */
"02H43MZ"  =  "1"  /* Insertion of Cardiac Lead into Coronary Vein, Perc Approach */
"02H440Z"  =  "1"  /* Insertion of Pressure Sens into Cor Vein, Perc Endo Approach */
"02H442Z"  =  "1"  /* Insertion of Monitor Dev into Cor Vein, Perc Endo Approach */
"02H443Z"  =  "1"  /* Insertion of Infusion Dev into Cor Vein, Perc Endo Approach */
"02H44DZ"  =  "1"  /* Insertion of Intralum Dev into Cor Vein, Perc Endo Approach */
"02H44KZ"  =  "1"  /* Insertion of Defib Lead into Cor Vein, Perc Endo Approach */
"02H600Z"  =  "1"  /* Insertion of Pressure Sens into R Atrium, Open Approach */
"02H602Z"  =  "1"  /* Insertion of Monitoring Device into R Atrium, Open Approach */
"02H603Z"  =  "1"  /* Insertion of Infusion Device into R Atrium, Open Approach */
"02H60DZ"  =  "1"  /* Insertion of Intralum Dev into R Atrium, Open Approach */
"02H60KZ"  =  "1"  /* Insertion of Defibrillator Lead into R Atrium, Open Approach */
"02H630Z"  =  "1"  /* Insertion of Pressure Sens into R Atrium, Perc Approach */
"02H632Z"  =  "1"  /* Insertion of Monitoring Device into R Atrium, Perc Approach */
"02H633Z"  =  "1"  /* Insertion of Infusion Device into R Atrium, Perc Approach */
"02H63DZ"  =  "1"  /* Insertion of Intralum Dev into R Atrium, Perc Approach */
"02H63KZ"  =  "1"  /* Insertion of Defibrillator Lead into R Atrium, Perc Approach */
"02H63MZ"  =  "1"  /* Insertion of Cardiac Lead into Right Atrium, Perc Approach */
"02H640Z"  =  "1"  /* Insertion of Pressure Sens into R Atrium, Perc Endo Approach */
"02H642Z"  =  "1"  /* Insertion of Monitor Dev into R Atrium, Perc Endo Approach */
"02H643Z"  =  "1"  /* Insertion of Infusion Dev into R Atrium, Perc Endo Approach */
"02H64DZ"  =  "1"  /* Insertion of Intralum Dev into R Atrium, Perc Endo Approach */
"02H64KZ"  =  "1"  /* Insertion of Defib Lead into R Atrium, Perc Endo Approach */
"02H700Z"  =  "1"  /* Insertion of Pressure Sens into L Atrium, Open Approach */
"02H702Z"  =  "1"  /* Insertion of Monitoring Device into L Atrium, Open Approach */
"02H703Z"  =  "1"  /* Insertion of Infusion Device into Left Atrium, Open Approach */
"02H70DZ"  =  "1"  /* Insertion of Intralum Dev into L Atrium, Open Approach */
"02H70KZ"  =  "1"  /* Insertion of Defibrillator Lead into L Atrium, Open Approach */
"02H730Z"  =  "1"  /* Insertion of Pressure Sens into L Atrium, Perc Approach */
"02H732Z"  =  "1"  /* Insertion of Monitoring Device into L Atrium, Perc Approach */
"02H733Z"  =  "1"  /* Insertion of Infusion Device into Left Atrium, Perc Approach */
"02H73DZ"  =  "1"  /* Insertion of Intralum Dev into L Atrium, Perc Approach */
"02H73KZ"  =  "1"  /* Insertion of Defibrillator Lead into L Atrium, Perc Approach */
"02H73MZ"  =  "1"  /* Insertion of Cardiac Lead into Left Atrium, Perc Approach */
"02H740Z"  =  "1"  /* Insertion of Pressure Sens into L Atrium, Perc Endo Approach */
"02H742Z"  =  "1"  /* Insertion of Monitor Dev into L Atrium, Perc Endo Approach */
"02H743Z"  =  "1"  /* Insertion of Infusion Dev into L Atrium, Perc Endo Approach */
"02H74DZ"  =  "1"  /* Insertion of Intralum Dev into L Atrium, Perc Endo Approach */
"02H74KZ"  =  "1"  /* Insertion of Defib Lead into L Atrium, Perc Endo Approach */
"02HA0QZ"  =  "1"  /* Insertion of Implant Heart Assist into Heart, Open Approach */
"02HA0RS"  =  "1"  /* Insert of Bivent Ext Heart Assist into Heart, Open Approach */
"02HA0RZ"  =  "1"  /* Insertion of Ext Heart Assist into Heart, Open Approach */
"02HA3QZ"  =  "1"  /* Insertion of Implant Heart Assist into Heart, Perc Approach */
"02HA3RS"  =  "1"  /* Insert of Bivent Ext Heart Assist into Heart, Perc Approach */
"02HA3RZ"  =  "1"  /* Insertion of Ext Heart Assist into Heart, Perc Approach */
"02HA4QZ"  =  "1"  /* Insert Implant Heart Assist in Heart, Perc Endo */
"02HA4RS"  =  "1"  /* Insert Bivent Ext Heart Assist in Heart, Perc Endo */
"02HA4RZ"  =  "1"  /* Insertion of Ext Heart Assist into Heart, Perc Endo Approach */
"02HK00Z"  =  "1"  /* Insertion of Pressure Sens into R Ventricle, Open Approach */
"02HK02Z"  =  "1"  /* Insertion of Monitor Dev into R Ventricle, Open Approach */
"02HK03Z"  =  "1"  /* Insertion of Infusion Device into R Ventricle, Open Approach */
"02HK0DZ"  =  "1"  /* Insertion of Intralum Dev into R Ventricle, Open Approach */
"02HK0KZ"  =  "1"  /* Insertion of Defib Lead into R Ventricle, Open Approach */
"02HK30Z"  =  "1"  /* Insertion of Pressure Sens into R Ventricle, Perc Approach */
"02HK32Z"  =  "1"  /* Insertion of Monitor Dev into R Ventricle, Perc Approach */
"02HK33Z"  =  "1"  /* Insertion of Infusion Device into R Ventricle, Perc Approach */
"02HK3DZ"  =  "1"  /* Insertion of Intralum Dev into R Ventricle, Perc Approach */
"02HK3KZ"  =  "1"  /* Insertion of Defib Lead into R Ventricle, Perc Approach */
"02HK40Z"  =  "1"  /* Insert of Pressure Sens into R Ventricle, Perc Endo Approach */
"02HK42Z"  =  "1"  /* Insert of Monitor Dev into R Ventricle, Perc Endo Approach */
"02HK43Z"  =  "1"  /* Insert of Infusion Dev into R Ventricle, Perc Endo Approach */
"02HK4DZ"  =  "1"  /* Insert of Intralum Dev into R Ventricle, Perc Endo Approach */
"02HK4KZ"  =  "1"  /* Insertion of Defib Lead into R Ventricle, Perc Endo Approach */
"02HL00Z"  =  "1"  /* Insertion of Pressure Sens into L Ventricle, Open Approach */
"02HL02Z"  =  "1"  /* Insertion of Monitor Dev into L Ventricle, Open Approach */
"02HL03Z"  =  "1"  /* Insertion of Infusion Device into L Ventricle, Open Approach */
"02HL0DZ"  =  "1"  /* Insertion of Intralum Dev into L Ventricle, Open Approach */
"02HL0KZ"  =  "1"  /* Insertion of Defib Lead into L Ventricle, Open Approach */
"02HL30Z"  =  "1"  /* Insertion of Pressure Sens into L Ventricle, Perc Approach */
"02HL32Z"  =  "1"  /* Insertion of Monitor Dev into L Ventricle, Perc Approach */
"02HL33Z"  =  "1"  /* Insertion of Infusion Device into L Ventricle, Perc Approach */
"02HL3DZ"  =  "1"  /* Insertion of Intralum Dev into L Ventricle, Perc Approach */
"02HL3KZ"  =  "1"  /* Insertion of Defib Lead into L Ventricle, Perc Approach */
"02HL40Z"  =  "1"  /* Insert of Pressure Sens into L Ventricle, Perc Endo Approach */
"02HL42Z"  =  "1"  /* Insert of Monitor Dev into L Ventricle, Perc Endo Approach */
"02HL43Z"  =  "1"  /* Insert of Infusion Dev into L Ventricle, Perc Endo Approach */
"02HL4DZ"  =  "1"  /* Insert of Intralum Dev into L Ventricle, Perc Endo Approach */
"02HL4KZ"  =  "1"  /* Insertion of Defib Lead into L Ventricle, Perc Endo Approach */
"02HN00Z"  =  "1"  /* Insertion of Pressure Sens into Pericardium, Open Approach */
"02HN02Z"  =  "1"  /* Insertion of Monitor Dev into Pericardium, Open Approach */
"02HN0JZ"  =  "1"  /* Insertion of Pacemaker Lead into Pericardium, Open Approach */
"02HN0KZ"  =  "1"  /* Insertion of Defib Lead into Pericardium, Open Approach */
"02HN0MZ"  =  "1"  /* Insertion of Cardiac Lead into Pericardium, Open Approach */
"02HN30Z"  =  "1"  /* Insertion of Pressure Sens into Pericardium, Perc Approach */
"02HN32Z"  =  "1"  /* Insertion of Monitor Dev into Pericardium, Perc Approach */
"02HN3JZ"  =  "1"  /* Insertion of Pacemaker Lead into Pericardium, Perc Approach */
"02HN3KZ"  =  "1"  /* Insertion of Defib Lead into Pericardium, Perc Approach */
"02HN3MZ"  =  "1"  /* Insertion of Cardiac Lead into Pericardium, Perc Approach */
"02HN40Z"  =  "1"  /* Insert of Pressure Sens into Pericardium, Perc Endo Approach */
"02HN42Z"  =  "1"  /* Insert of Monitor Dev into Pericardium, Perc Endo Approach */
"02HN4JZ"  =  "1"  /* Insertion of Pace Lead into Pericardium, Perc Endo Approach */
"02HN4KZ"  =  "1"  /* Insertion of Defib Lead into Pericardium, Perc Endo Approach */
"02HN4MZ"  =  "1"  /* Insert of Cardiac Lead into Pericardium, Perc Endo Approach */
"02HP0DZ"  =  "1"  /* Insertion of Intralum Dev into Pulm Trunk, Open Approach */
"02HP3DZ"  =  "1"  /* Insertion of Intralum Dev into Pulm Trunk, Perc Approach */
"02HP4DZ"  =  "1"  /* Insert of Intralum Dev into Pulm Trunk, Perc Endo Approach */
"02HQ0DZ"  =  "1"  /* Insertion of Intralum Dev into R Pulm Art, Open Approach */
"02HQ3DZ"  =  "1"  /* Insertion of Intralum Dev into R Pulm Art, Perc Approach */
"02HQ4DZ"  =  "1"  /* Insert of Intralum Dev into R Pulm Art, Perc Endo Approach */
"02HR0DZ"  =  "1"  /* Insertion of Intralum Dev into L Pulm Art, Open Approach */
"02HR3DZ"  =  "1"  /* Insertion of Intralum Dev into L Pulm Art, Perc Approach */
"02HR4DZ"  =  "1"  /* Insert of Intralum Dev into L Pulm Art, Perc Endo Approach */
"02HS00Z"  =  "1"  /* Insertion of Pressure Sens into R Pulm Vein, Open Approach */
"02HS02Z"  =  "1"  /* Insertion of Monitor Dev into R Pulm Vein, Open Approach */
"02HS0DZ"  =  "1"  /* Insertion of Intralum Dev into R Pulm Vein, Open Approach */
"02HS30Z"  =  "1"  /* Insertion of Pressure Sens into R Pulm Vein, Perc Approach */
"02HS32Z"  =  "1"  /* Insertion of Monitor Dev into R Pulm Vein, Perc Approach */
"02HS3DZ"  =  "1"  /* Insertion of Intralum Dev into R Pulm Vein, Perc Approach */
"02HS40Z"  =  "1"  /* Insert of Pressure Sens into R Pulm Vein, Perc Endo Approach */
"02HS42Z"  =  "1"  /* Insert of Monitor Dev into R Pulm Vein, Perc Endo Approach */
"02HS4DZ"  =  "1"  /* Insert of Intralum Dev into R Pulm Vein, Perc Endo Approach */
"02HT00Z"  =  "1"  /* Insertion of Pressure Sens into L Pulm Vein, Open Approach */
"02HT02Z"  =  "1"  /* Insertion of Monitor Dev into L Pulm Vein, Open Approach */
"02HT0DZ"  =  "1"  /* Insertion of Intralum Dev into L Pulm Vein, Open Approach */
"02HT30Z"  =  "1"  /* Insertion of Pressure Sens into L Pulm Vein, Perc Approach */
"02HT32Z"  =  "1"  /* Insertion of Monitor Dev into L Pulm Vein, Perc Approach */
"02HT3DZ"  =  "1"  /* Insertion of Intralum Dev into L Pulm Vein, Perc Approach */
"02HT40Z"  =  "1"  /* Insert of Pressure Sens into L Pulm Vein, Perc Endo Approach */
"02HT42Z"  =  "1"  /* Insert of Monitor Dev into L Pulm Vein, Perc Endo Approach */
"02HT4DZ"  =  "1"  /* Insert of Intralum Dev into L Pulm Vein, Perc Endo Approach */
"02HV00Z"  =  "1"  /* Insertion of Pressure Sens into Sup Vena Cava, Open Approach */
"02HV02Z"  =  "1"  /* Insertion of Monitor Dev into Sup Vena Cava, Open Approach */
"02HV0DZ"  =  "1"  /* Insertion of Intralum Dev into Sup Vena Cava, Open Approach */
"02HV30Z"  =  "1"  /* Insertion of Pressure Sens into Sup Vena Cava, Perc Approach */
"02HV32Z"  =  "1"  /* Insertion of Monitor Dev into Sup Vena Cava, Perc Approach */
"02HV3DZ"  =  "1"  /* Insertion of Intralum Dev into Sup Vena Cava, Perc Approach */
"02HV40Z"  =  "1"  /* Insert Pressure Sens in Sup Vena Cava, Perc Endo */
"02HV42Z"  =  "1"  /* Insert of Monitor Dev into Sup Vena Cava, Perc Endo Approach */
"02HV4DZ"  =  "1"  /* Insert Intralum Dev in Sup Vena Cava, Perc Endo */
"02HW02Z"  =  "1"  /* Insertion of Monitor Dev into Thor Aorta, Open Approach */
"02HW0DZ"  =  "1"  /* Insertion of Intralum Dev into Thor Aorta, Open Approach */
"02HW32Z"  =  "1"  /* Insertion of Monitor Dev into Thor Aorta, Perc Approach */
"02HW3DZ"  =  "1"  /* Insertion of Intralum Dev into Thor Aorta, Perc Approach */
"02HW42Z"  =  "1"  /* Insertion of Monitor Dev into Thor Aorta, Perc Endo Approach */
"02HW4DZ"  =  "1"  /* Insert of Intralum Dev into Thor Aorta, Perc Endo Approach */
"02JA0ZZ"  =  "1"  /* Inspection of Heart, Open Approach */
"02JA4ZZ"  =  "1"  /* Inspection of Heart, Percutaneous Endoscopic Approach */
"02JY0ZZ"  =  "1"  /* Inspection of Great Vessel, Open Approach */
"02JY4ZZ"  =  "1"  /* Inspection of Great Vessel, Percutaneous Endoscopic Approach */
"02LR0CT"  =  "1"  /* Occlusion of Ductus Arterio with Extralum Dev, Open Approach */
"02LR0DT"  =  "1"  /* Occlusion of Ductus Arterio with Intralum Dev, Open Approach */
"02LR0ZT"  =  "1"  /* Occlusion of Ductus Arteriosus, Open Approach */
"02LR3CT"  =  "1"  /* Occlusion of Ductus Arterio with Extralum Dev, Perc Approach */
"02LR3DT"  =  "1"  /* Occlusion of Ductus Arterio with Intralum Dev, Perc Approach */
"02LR3ZT"  =  "1"  /* Occlusion of Ductus Arteriosus, Percutaneous Approach */
"02LR4CT"  =  "1"  /* Occlusion Ductus Arterio w Extralum Dev, Perc Endo */
"02LR4DT"  =  "1"  /* Occlusion Ductus Arterio w Intralum Dev, Perc Endo */
"02LR4ZT"  =  "1"  /* Occlusion of Ductus Arteriosus, Perc Endo Approach */
"02LS0CZ"  =  "1"  /* Occlusion of R Pulm Vein with Extralum Dev, Open Approach */
"02LS0DZ"  =  "1"  /* Occlusion of R Pulm Vein with Intralum Dev, Open Approach */
"02LS0ZZ"  =  "1"  /* Occlusion of Right Pulmonary Vein, Open Approach */
"02LS3CZ"  =  "1"  /* Occlusion of R Pulm Vein with Extralum Dev, Perc Approach */
"02LS3DZ"  =  "1"  /* Occlusion of R Pulm Vein with Intralum Dev, Perc Approach */
"02LS3ZZ"  =  "1"  /* Occlusion of Right Pulmonary Vein, Percutaneous Approach */
"02LS4CZ"  =  "1"  /* Occlusion R Pulm Vein w Extralum Dev, Perc Endo */
"02LS4DZ"  =  "1"  /* Occlusion R Pulm Vein w Intralum Dev, Perc Endo */
"02LS4ZZ"  =  "1"  /* Occlusion of Right Pulmonary Vein, Perc Endo Approach */
"02LT0CZ"  =  "1"  /* Occlusion of L Pulm Vein with Extralum Dev, Open Approach */
"02LT0DZ"  =  "1"  /* Occlusion of L Pulm Vein with Intralum Dev, Open Approach */
"02LT0ZZ"  =  "1"  /* Occlusion of Left Pulmonary Vein, Open Approach */
"02LT3CZ"  =  "1"  /* Occlusion of L Pulm Vein with Extralum Dev, Perc Approach */
"02LT3DZ"  =  "1"  /* Occlusion of L Pulm Vein with Intralum Dev, Perc Approach */
"02LT3ZZ"  =  "1"  /* Occlusion of Left Pulmonary Vein, Percutaneous Approach */
"02LT4CZ"  =  "1"  /* Occlusion L Pulm Vein w Extralum Dev, Perc Endo */
"02LT4DZ"  =  "1"  /* Occlusion L Pulm Vein w Intralum Dev, Perc Endo */
"02LT4ZZ"  =  "1"  /* Occlusion of Left Pulmonary Vein, Perc Endo Approach */
"02LV0CZ"  =  "1"  /* Occlusion of Sup Vena Cava with Extralum Dev, Open Approach */
"02LV0DZ"  =  "1"  /* Occlusion of Sup Vena Cava with Intralum Dev, Open Approach */
"02LV0ZZ"  =  "1"  /* Occlusion of Superior Vena Cava, Open Approach */
"02LV3CZ"  =  "1"  /* Occlusion of Sup Vena Cava with Extralum Dev, Perc Approach */
"02LV3DZ"  =  "1"  /* Occlusion of Sup Vena Cava with Intralum Dev, Perc Approach */
"02LV3ZZ"  =  "1"  /* Occlusion of Superior Vena Cava, Percutaneous Approach */
"02LV4CZ"  =  "1"  /* Occlusion Sup Vena Cava w Extralum Dev, Perc Endo */
"02LV4DZ"  =  "1"  /* Occlusion Sup Vena Cava w Intralum Dev, Perc Endo */
"02LV4ZZ"  =  "1"  /* Occlusion of Superior Vena Cava, Perc Endo Approach */
"02N40ZZ"  =  "1"  /* Release Coronary Vein, Open Approach */
"02N43ZZ"  =  "1"  /* Release Coronary Vein, Percutaneous Approach */
"02N44ZZ"  =  "1"  /* Release Coronary Vein, Percutaneous Endoscopic Approach */
"02N50ZZ"  =  "1"  /* Release Atrial Septum, Open Approach */
"02N53ZZ"  =  "1"  /* Release Atrial Septum, Percutaneous Approach */
"02N54ZZ"  =  "1"  /* Release Atrial Septum, Percutaneous Endoscopic Approach */
"02N60ZZ"  =  "1"  /* Release Right Atrium, Open Approach */
"02N63ZZ"  =  "1"  /* Release Right Atrium, Percutaneous Approach */
"02N64ZZ"  =  "1"  /* Release Right Atrium, Percutaneous Endoscopic Approach */
"02N70ZZ"  =  "1"  /* Release Left Atrium, Open Approach */
"02N73ZZ"  =  "1"  /* Release Left Atrium, Percutaneous Approach */
"02N74ZZ"  =  "1"  /* Release Left Atrium, Percutaneous Endoscopic Approach */
"02N80ZZ"  =  "1"  /* Release Conduction Mechanism, Open Approach */
"02N83ZZ"  =  "1"  /* Release Conduction Mechanism, Percutaneous Approach */
"02N84ZZ"  =  "1"  /* Release Conduction Mechanism, Perc Endo Approach */
"02N90ZZ"  =  "1"  /* Release Chordae Tendineae, Open Approach */
"02N93ZZ"  =  "1"  /* Release Chordae Tendineae, Percutaneous Approach */
"02N94ZZ"  =  "1"  /* Release Chordae Tendineae, Percutaneous Endoscopic Approach */
"02ND0ZZ"  =  "1"  /* Release Papillary Muscle, Open Approach */
"02ND3ZZ"  =  "1"  /* Release Papillary Muscle, Percutaneous Approach */
"02ND4ZZ"  =  "1"  /* Release Papillary Muscle, Percutaneous Endoscopic Approach */
"02NF0ZZ"  =  "1"  /* Release Aortic Valve, Open Approach */
"02NF3ZZ"  =  "1"  /* Release Aortic Valve, Percutaneous Approach */
"02NF4ZZ"  =  "1"  /* Release Aortic Valve, Percutaneous Endoscopic Approach */
"02NG0ZZ"  =  "1"  /* Release Mitral Valve, Open Approach */
"02NG3ZZ"  =  "1"  /* Release Mitral Valve, Percutaneous Approach */
"02NG4ZZ"  =  "1"  /* Release Mitral Valve, Percutaneous Endoscopic Approach */
"02NH0ZZ"  =  "1"  /* Release Pulmonary Valve, Open Approach */
"02NH3ZZ"  =  "1"  /* Release Pulmonary Valve, Percutaneous Approach */
"02NH4ZZ"  =  "1"  /* Release Pulmonary Valve, Percutaneous Endoscopic Approach */
"02NJ0ZZ"  =  "1"  /* Release Tricuspid Valve, Open Approach */
"02NJ3ZZ"  =  "1"  /* Release Tricuspid Valve, Percutaneous Approach */
"02NJ4ZZ"  =  "1"  /* Release Tricuspid Valve, Percutaneous Endoscopic Approach */
"02NK0ZZ"  =  "1"  /* Release Right Ventricle, Open Approach */
"02NK3ZZ"  =  "1"  /* Release Right Ventricle, Percutaneous Approach */
"02NK4ZZ"  =  "1"  /* Release Right Ventricle, Percutaneous Endoscopic Approach */
"02NL0ZZ"  =  "1"  /* Release Left Ventricle, Open Approach */
"02NL3ZZ"  =  "1"  /* Release Left Ventricle, Percutaneous Approach */
"02NL4ZZ"  =  "1"  /* Release Left Ventricle, Percutaneous Endoscopic Approach */
"02NM0ZZ"  =  "1"  /* Release Ventricular Septum, Open Approach */
"02NM3ZZ"  =  "1"  /* Release Ventricular Septum, Percutaneous Approach */
"02NM4ZZ"  =  "1"  /* Release Ventricular Septum, Percutaneous Endoscopic Approach */
"02NN0ZZ"  =  "1"  /* Release Pericardium, Open Approach */
"02NN3ZZ"  =  "1"  /* Release Pericardium, Percutaneous Approach */
"02NN4ZZ"  =  "1"  /* Release Pericardium, Percutaneous Endoscopic Approach */
"02NP0ZZ"  =  "1"  /* Release Pulmonary Trunk, Open Approach */
"02NP3ZZ"  =  "1"  /* Release Pulmonary Trunk, Percutaneous Approach */
"02NP4ZZ"  =  "1"  /* Release Pulmonary Trunk, Percutaneous Endoscopic Approach */
"02NQ0ZZ"  =  "1"  /* Release Right Pulmonary Artery, Open Approach */
"02NQ3ZZ"  =  "1"  /* Release Right Pulmonary Artery, Percutaneous Approach */
"02NQ4ZZ"  =  "1"  /* Release Right Pulmonary Artery, Perc Endo Approach */
"02NR0ZZ"  =  "1"  /* Release Left Pulmonary Artery, Open Approach */
"02NR3ZZ"  =  "1"  /* Release Left Pulmonary Artery, Percutaneous Approach */
"02NR4ZZ"  =  "1"  /* Release Left Pulmonary Artery, Perc Endo Approach */
"02NS0ZZ"  =  "1"  /* Release Right Pulmonary Vein, Open Approach */
"02NS3ZZ"  =  "1"  /* Release Right Pulmonary Vein, Percutaneous Approach */
"02NS4ZZ"  =  "1"  /* Release Right Pulmonary Vein, Perc Endo Approach */
"02NT0ZZ"  =  "1"  /* Release Left Pulmonary Vein, Open Approach */
"02NT3ZZ"  =  "1"  /* Release Left Pulmonary Vein, Percutaneous Approach */
"02NT4ZZ"  =  "1"  /* Release Left Pulmonary Vein, Perc Endo Approach */
"02NV0ZZ"  =  "1"  /* Release Superior Vena Cava, Open Approach */
"02NV3ZZ"  =  "1"  /* Release Superior Vena Cava, Percutaneous Approach */
"02NV4ZZ"  =  "1"  /* Release Superior Vena Cava, Percutaneous Endoscopic Approach */
"02NW0ZZ"  =  "1"  /* Release Thoracic Aorta, Open Approach */
"02NW3ZZ"  =  "1"  /* Release Thoracic Aorta, Percutaneous Approach */
"02NW4ZZ"  =  "1"  /* Release Thoracic Aorta, Percutaneous Endoscopic Approach */
"02PA02Z"  =  "1"  /* Removal of Monitoring Device from Heart, Open Approach */
"02PA03Z"  =  "1"  /* Removal of Infusion Device from Heart, Open Approach */
"02PA07Z"  =  "1"  /* Removal of Autol Sub from Heart, Open Approach */
"02PA08Z"  =  "1"  /* Removal of Zooplastic Tissue from Heart, Open Approach */
"02PA0CZ"  =  "1"  /* Removal of Extraluminal Device from Heart, Open Approach */
"02PA0DZ"  =  "1"  /* Removal of Intraluminal Device from Heart, Open Approach */
"02PA0JZ"  =  "1"  /* Removal of Synthetic Substitute from Heart, Open Approach */
"02PA0KZ"  =  "1"  /* Removal of Nonaut Sub from Heart, Open Approach */
"02PA0MZ"  =  "1"  /* Removal of Cardiac Lead from Heart, Open Approach */
"02PA0QZ"  =  "1"  /* Removal of Implant Heart Assist from Heart, Open Approach */
"02PA0RZ"  =  "1"  /* Removal of Ext Heart Assist from Heart, Open Approach */
"02PA32Z"  =  "1"  /* Removal of Monitoring Device from Heart, Perc Approach */
"02PA33Z"  =  "1"  /* Removal of Infusion Device from Heart, Percutaneous Approach */
"02PA37Z"  =  "1"  /* Removal of Autol Sub from Heart, Perc Approach */
"02PA38Z"  =  "1"  /* Removal of Zooplastic Tissue from Heart, Perc Approach */
"02PA3CZ"  =  "1"  /* Removal of Extraluminal Device from Heart, Perc Approach */
"02PA3DZ"  =  "1"  /* Removal of Intraluminal Device from Heart, Perc Approach */
"02PA3JZ"  =  "1"  /* Removal of Synthetic Substitute from Heart, Perc Approach */
"02PA3KZ"  =  "1"  /* Removal of Nonaut Sub from Heart, Perc Approach */
"02PA3MZ"  =  "1"  /* Removal of Cardiac Lead from Heart, Percutaneous Approach */
"02PA3QZ"  =  "1"  /* Removal of Implant Heart Assist from Heart, Perc Approach */
"02PA3RZ"  =  "1"  /* Removal of Ext Heart Assist from Heart, Perc Approach */
"02PA42Z"  =  "1"  /* Removal of Monitoring Device from Heart, Perc Endo Approach */
"02PA43Z"  =  "1"  /* Removal of Infusion Device from Heart, Perc Endo Approach */
"02PA47Z"  =  "1"  /* Removal of Autol Sub from Heart, Perc Endo Approach */
"02PA48Z"  =  "1"  /* Removal of Zooplastic Tissue from Heart, Perc Endo Approach */
"02PA4CZ"  =  "1"  /* Removal of Extralum Dev from Heart, Perc Endo Approach */
"02PA4DZ"  =  "1"  /* Removal of Intralum Dev from Heart, Perc Endo Approach */
"02PA4JZ"  =  "1"  /* Removal of Synth Sub from Heart, Perc Endo Approach */
"02PA4KZ"  =  "1"  /* Removal of Nonaut Sub from Heart, Perc Endo Approach */
"02PA4MZ"  =  "1"  /* Removal of Cardiac Lead from Heart, Perc Endo Approach */
"02PA4QZ"  =  "1"  /* Remove Implant Heart Assist from Heart, Perc Endo */
"02PA4RZ"  =  "1"  /* Removal of Ext Heart Assist from Heart, Perc Endo Approach */
"02PAXMZ"  =  "1"  /* Removal of Cardiac Lead from Heart, External Approach */
"02PY02Z"  =  "1"  /* Removal of Monitor Dev from Great Vessel, Open Approach */
"02PY03Z"  =  "1"  /* Removal of Infusion Device from Great Vessel, Open Approach */
"02PY07Z"  =  "1"  /* Removal of Autol Sub from Great Vessel, Open Approach */
"02PY08Z"  =  "1"  /* Removal of Zooplastic from Great Vessel, Open Approach */
"02PY0CZ"  =  "1"  /* Removal of Extralum Dev from Great Vessel, Open Approach */
"02PY0DZ"  =  "1"  /* Removal of Intralum Dev from Great Vessel, Open Approach */
"02PY0JZ"  =  "1"  /* Removal of Synth Sub from Great Vessel, Open Approach */
"02PY0KZ"  =  "1"  /* Removal of Nonaut Sub from Great Vessel, Open Approach */
"02PY32Z"  =  "1"  /* Removal of Monitor Dev from Great Vessel, Perc Approach */
"02PY33Z"  =  "1"  /* Removal of Infusion Device from Great Vessel, Perc Approach */
"02PY37Z"  =  "1"  /* Removal of Autol Sub from Great Vessel, Perc Approach */
"02PY38Z"  =  "1"  /* Removal of Zooplastic from Great Vessel, Perc Approach */
"02PY3CZ"  =  "1"  /* Removal of Extralum Dev from Great Vessel, Perc Approach */
"02PY3DZ"  =  "1"  /* Removal of Intralum Dev from Great Vessel, Perc Approach */
"02PY3JZ"  =  "1"  /* Removal of Synth Sub from Great Vessel, Perc Approach */
"02PY3KZ"  =  "1"  /* Removal of Nonaut Sub from Great Vessel, Perc Approach */
"02PY42Z"  =  "1"  /* Removal of Monitor Dev from Great Vessel, Perc Endo Approach */
"02PY43Z"  =  "1"  /* Remove of Infusion Dev from Great Vessel, Perc Endo Approach */
"02PY47Z"  =  "1"  /* Removal of Autol Sub from Great Vessel, Perc Endo Approach */
"02PY48Z"  =  "1"  /* Removal of Zooplastic from Great Vessel, Perc Endo Approach */
"02PY4CZ"  =  "1"  /* Remove of Extralum Dev from Great Vessel, Perc Endo Approach */
"02PY4DZ"  =  "1"  /* Remove of Intralum Dev from Great Vessel, Perc Endo Approach */
"02PY4JZ"  =  "1"  /* Removal of Synth Sub from Great Vessel, Perc Endo Approach */
"02PY4KZ"  =  "1"  /* Removal of Nonaut Sub from Great Vessel, Perc Endo Approach */
"02Q00ZZ"  =  "1"  /* Repair Coronary Artery, One Site, Open Approach */
"02Q03ZZ"  =  "1"  /* Repair Coronary Artery, One Site, Percutaneous Approach */
"02Q04ZZ"  =  "1"  /* Repair Coronary Artery, One Site, Perc Endo Approach */
"02Q10ZZ"  =  "1"  /* Repair Coronary Artery, Two Sites, Open Approach */
"02Q13ZZ"  =  "1"  /* Repair Coronary Artery, Two Sites, Percutaneous Approach */
"02Q14ZZ"  =  "1"  /* Repair Coronary Artery, Two Sites, Perc Endo Approach */
"02Q20ZZ"  =  "1"  /* Repair Coronary Artery, Three Sites, Open Approach */
"02Q23ZZ"  =  "1"  /* Repair Coronary Artery, Three Sites, Percutaneous Approach */
"02Q24ZZ"  =  "1"  /* Repair Coronary Artery, Three Sites, Perc Endo Approach */
"02Q30ZZ"  =  "1"  /* Repair Coronary Artery, Four or More Sites, Open Approach */
"02Q33ZZ"  =  "1"  /* Repair Coronary Artery, Four or More Sites, Perc Approach */
"02Q34ZZ"  =  "1"  /* Repair 4+ Cor Art, Perc Endo Approach */
"02Q40ZZ"  =  "1"  /* Repair Coronary Vein, Open Approach */
"02Q43ZZ"  =  "1"  /* Repair Coronary Vein, Percutaneous Approach */
"02Q44ZZ"  =  "1"  /* Repair Coronary Vein, Percutaneous Endoscopic Approach */
"02Q50ZZ"  =  "1"  /* Repair Atrial Septum, Open Approach */
"02Q53ZZ"  =  "1"  /* Repair Atrial Septum, Percutaneous Approach */
"02Q54ZZ"  =  "1"  /* Repair Atrial Septum, Percutaneous Endoscopic Approach */
"02Q60ZZ"  =  "1"  /* Repair Right Atrium, Open Approach */
"02Q63ZZ"  =  "1"  /* Repair Right Atrium, Percutaneous Approach */
"02Q64ZZ"  =  "1"  /* Repair Right Atrium, Percutaneous Endoscopic Approach */
"02Q70ZZ"  =  "1"  /* Repair Left Atrium, Open Approach */
"02Q73ZZ"  =  "1"  /* Repair Left Atrium, Percutaneous Approach */
"02Q74ZZ"  =  "1"  /* Repair Left Atrium, Percutaneous Endoscopic Approach */
"02Q80ZZ"  =  "1"  /* Repair Conduction Mechanism, Open Approach */
"02Q83ZZ"  =  "1"  /* Repair Conduction Mechanism, Percutaneous Approach */
"02Q84ZZ"  =  "1"  /* Repair Conduction Mechanism, Perc Endo Approach */
"02Q90ZZ"  =  "1"  /* Repair Chordae Tendineae, Open Approach */
"02Q93ZZ"  =  "1"  /* Repair Chordae Tendineae, Percutaneous Approach */
"02Q94ZZ"  =  "1"  /* Repair Chordae Tendineae, Percutaneous Endoscopic Approach */
"02QA0ZZ"  =  "1"  /* Repair Heart, Open Approach */
"02QA3ZZ"  =  "1"  /* Repair Heart, Percutaneous Approach */
"02QA4ZZ"  =  "1"  /* Repair Heart, Percutaneous Endoscopic Approach */
"02QB0ZZ"  =  "1"  /* Repair Right Heart, Open Approach */
"02QB3ZZ"  =  "1"  /* Repair Right Heart, Percutaneous Approach */
"02QB4ZZ"  =  "1"  /* Repair Right Heart, Percutaneous Endoscopic Approach */
"02QC0ZZ"  =  "1"  /* Repair Left Heart, Open Approach */
"02QC3ZZ"  =  "1"  /* Repair Left Heart, Percutaneous Approach */
"02QC4ZZ"  =  "1"  /* Repair Left Heart, Percutaneous Endoscopic Approach */
"02QD0ZZ"  =  "1"  /* Repair Papillary Muscle, Open Approach */
"02QD3ZZ"  =  "1"  /* Repair Papillary Muscle, Percutaneous Approach */
"02QD4ZZ"  =  "1"  /* Repair Papillary Muscle, Percutaneous Endoscopic Approach */
"02QF0ZZ"  =  "1"  /* Repair Aortic Valve, Open Approach */
"02QF3ZZ"  =  "1"  /* Repair Aortic Valve, Percutaneous Approach */
"02QF4ZZ"  =  "1"  /* Repair Aortic Valve, Percutaneous Endoscopic Approach */
"02QG0ZZ"  =  "1"  /* Repair Mitral Valve, Open Approach */
"02QG3ZZ"  =  "1"  /* Repair Mitral Valve, Percutaneous Approach */
"02QG4ZZ"  =  "1"  /* Repair Mitral Valve, Percutaneous Endoscopic Approach */
"02QH0ZZ"  =  "1"  /* Repair Pulmonary Valve, Open Approach */
"02QH3ZZ"  =  "1"  /* Repair Pulmonary Valve, Percutaneous Approach */
"02QH4ZZ"  =  "1"  /* Repair Pulmonary Valve, Percutaneous Endoscopic Approach */
"02QJ0ZZ"  =  "1"  /* Repair Tricuspid Valve, Open Approach */
"02QJ3ZZ"  =  "1"  /* Repair Tricuspid Valve, Percutaneous Approach */
"02QJ4ZZ"  =  "1"  /* Repair Tricuspid Valve, Percutaneous Endoscopic Approach */
"02QK0ZZ"  =  "1"  /* Repair Right Ventricle, Open Approach */
"02QK3ZZ"  =  "1"  /* Repair Right Ventricle, Percutaneous Approach */
"02QK4ZZ"  =  "1"  /* Repair Right Ventricle, Percutaneous Endoscopic Approach */
"02QL0ZZ"  =  "1"  /* Repair Left Ventricle, Open Approach */
"02QL3ZZ"  =  "1"  /* Repair Left Ventricle, Percutaneous Approach */
"02QL4ZZ"  =  "1"  /* Repair Left Ventricle, Percutaneous Endoscopic Approach */
"02QM0ZZ"  =  "1"  /* Repair Ventricular Septum, Open Approach */
"02QM3ZZ"  =  "1"  /* Repair Ventricular Septum, Percutaneous Approach */
"02QM4ZZ"  =  "1"  /* Repair Ventricular Septum, Percutaneous Endoscopic Approach */
"02QN0ZZ"  =  "1"  /* Repair Pericardium, Open Approach */
"02QN3ZZ"  =  "1"  /* Repair Pericardium, Percutaneous Approach */
"02QN4ZZ"  =  "1"  /* Repair Pericardium, Percutaneous Endoscopic Approach */
"02QP0ZZ"  =  "1"  /* Repair Pulmonary Trunk, Open Approach */
"02QP3ZZ"  =  "1"  /* Repair Pulmonary Trunk, Percutaneous Approach */
"02QP4ZZ"  =  "1"  /* Repair Pulmonary Trunk, Percutaneous Endoscopic Approach */
"02QQ0ZZ"  =  "1"  /* Repair Right Pulmonary Artery, Open Approach */
"02QQ3ZZ"  =  "1"  /* Repair Right Pulmonary Artery, Percutaneous Approach */
"02QQ4ZZ"  =  "1"  /* Repair Right Pulmonary Artery, Perc Endo Approach */
"02QR0ZZ"  =  "1"  /* Repair Left Pulmonary Artery, Open Approach */
"02QR3ZZ"  =  "1"  /* Repair Left Pulmonary Artery, Percutaneous Approach */
"02QR4ZZ"  =  "1"  /* Repair Left Pulmonary Artery, Perc Endo Approach */
"02QS0ZZ"  =  "1"  /* Repair Right Pulmonary Vein, Open Approach */
"02QS3ZZ"  =  "1"  /* Repair Right Pulmonary Vein, Percutaneous Approach */
"02QS4ZZ"  =  "1"  /* Repair Right Pulmonary Vein, Perc Endo Approach */
"02QT0ZZ"  =  "1"  /* Repair Left Pulmonary Vein, Open Approach */
"02QT3ZZ"  =  "1"  /* Repair Left Pulmonary Vein, Percutaneous Approach */
"02QT4ZZ"  =  "1"  /* Repair Left Pulmonary Vein, Percutaneous Endoscopic Approach */
"02QV0ZZ"  =  "1"  /* Repair Superior Vena Cava, Open Approach */
"02QV3ZZ"  =  "1"  /* Repair Superior Vena Cava, Percutaneous Approach */
"02QV4ZZ"  =  "1"  /* Repair Superior Vena Cava, Percutaneous Endoscopic Approach */
"02QW0ZZ"  =  "1"  /* Repair Thoracic Aorta, Open Approach */
"02QW3ZZ"  =  "1"  /* Repair Thoracic Aorta, Percutaneous Approach */
"02QW4ZZ"  =  "1"  /* Repair Thoracic Aorta, Percutaneous Endoscopic Approach */
"02R507Z"  =  "1"  /* Replacement of Atrial Septum with Autol Sub, Open Approach */
"02R508Z"  =  "1"  /* Replacement of Atrial Septum with Zooplastic, Open Approach */
"02R50JZ"  =  "1"  /* Replacement of Atrial Septum with Synth Sub, Open Approach */
"02R50KZ"  =  "1"  /* Replacement of Atrial Septum with Nonaut Sub, Open Approach */
"02R547Z"  =  "1"  /* Replace of Atrial Sept with Autol Sub, Perc Endo Approach */
"02R548Z"  =  "1"  /* Replace of Atrial Sept with Zooplastic, Perc Endo Approach */
"02R54JZ"  =  "1"  /* Replace of Atrial Sept with Synth Sub, Perc Endo Approach */
"02R54KZ"  =  "1"  /* Replace of Atrial Sept with Nonaut Sub, Perc Endo Approach */
"02R607Z"  =  "1"  /* Replacement of Right Atrium with Autol Sub, Open Approach */
"02R608Z"  =  "1"  /* Replacement of Right Atrium with Zooplastic, Open Approach */
"02R60JZ"  =  "1"  /* Replacement of Right Atrium with Synth Sub, Open Approach */
"02R60KZ"  =  "1"  /* Replacement of Right Atrium with Nonaut Sub, Open Approach */
"02R647Z"  =  "1"  /* Replacement of R Atrium with Autol Sub, Perc Endo Approach */
"02R648Z"  =  "1"  /* Replacement of R Atrium with Zooplastic, Perc Endo Approach */
"02R64JZ"  =  "1"  /* Replacement of R Atrium with Synth Sub, Perc Endo Approach */
"02R64KZ"  =  "1"  /* Replacement of R Atrium with Nonaut Sub, Perc Endo Approach */
"02R707Z"  =  "1"  /* Replacement of Left Atrium with Autol Sub, Open Approach */
"02R708Z"  =  "1"  /* Replacement of Left Atrium with Zooplastic, Open Approach */
"02R70JZ"  =  "1"  /* Replacement of Left Atrium with Synth Sub, Open Approach */
"02R70KZ"  =  "1"  /* Replacement of Left Atrium with Nonaut Sub, Open Approach */
"02R747Z"  =  "1"  /* Replacement of L Atrium with Autol Sub, Perc Endo Approach */
"02R748Z"  =  "1"  /* Replacement of L Atrium with Zooplastic, Perc Endo Approach */
"02R74JZ"  =  "1"  /* Replacement of L Atrium with Synth Sub, Perc Endo Approach */
"02R74KZ"  =  "1"  /* Replacement of L Atrium with Nonaut Sub, Perc Endo Approach */
"02R907Z"  =  "1"  /* Replace of Chordae Tendineae with Autol Sub, Open Approach */
"02R908Z"  =  "1"  /* Replace of Chordae Tendineae with Zooplastic, Open Approach */
"02R90JZ"  =  "1"  /* Replace of Chordae Tendineae with Synth Sub, Open Approach */
"02R90KZ"  =  "1"  /* Replace of Chordae Tendineae with Nonaut Sub, Open Approach */
"02R947Z"  =  "1"  /* Replace Chordae Tendineae w Autol Sub, Perc Endo */
"02R948Z"  =  "1"  /* Replace Chordae Tendineae w Zooplastic, Perc Endo */
"02R94JZ"  =  "1"  /* Replace Chordae Tendineae w Synth Sub, Perc Endo */
"02R94KZ"  =  "1"  /* Replace Chordae Tendineae w Nonaut Sub, Perc Endo */
"02RD07Z"  =  "1"  /* Replace of Papillary Muscle with Autol Sub, Open Approach */
"02RD08Z"  =  "1"  /* Replace of Papillary Muscle with Zooplastic, Open Approach */
"02RD0JZ"  =  "1"  /* Replace of Papillary Muscle with Synth Sub, Open Approach */
"02RD0KZ"  =  "1"  /* Replace of Papillary Muscle with Nonaut Sub, Open Approach */
"02RD47Z"  =  "1"  /* Replace Papillary Muscle w Autol Sub, Perc Endo */
"02RD48Z"  =  "1"  /* Replace Papillary Muscle w Zooplastic, Perc Endo */
"02RD4JZ"  =  "1"  /* Replace Papillary Muscle w Synth Sub, Perc Endo */
"02RD4KZ"  =  "1"  /* Replace Papillary Muscle w Nonaut Sub, Perc Endo */
"02RF07Z"  =  "1"  /* Replacement of Aortic Valve with Autol Sub, Open Approach */
"02RF08Z"  =  "1"  /* Replacement of Aortic Valve with Zooplastic, Open Approach */
"02RF0JZ"  =  "1"  /* Replacement of Aortic Valve with Synth Sub, Open Approach */
"02RF0KZ"  =  "1"  /* Replacement of Aortic Valve with Nonaut Sub, Open Approach */
"02RF37H"  =  "1"  /* Replace Aortic Valve w Autol Sub, Transap, Perc */
"02RF37Z"  =  "1"  /* Replacement of Aortic Valve with Autol Sub, Perc Approach */
"02RF38H"  =  "1"  /* Replace Aortic Valve w Zooplastic, Transap, Perc */
"02RF38Z"  =  "1"  /* Replacement of Aortic Valve with Zooplastic, Perc Approach */
"02RF3JH"  =  "1"  /* Replace Aortic Valve w Synth Sub, Transap, Perc */
"02RF3JZ"  =  "1"  /* Replacement of Aortic Valve with Synth Sub, Perc Approach */
"02RF3KH"  =  "1"  /* Replace Aortic Valve w Nonaut Sub, Transap, Perc */
"02RF3KZ"  =  "1"  /* Replacement of Aortic Valve with Nonaut Sub, Perc Approach */
"02RF47Z"  =  "1"  /* Replace of Aortic Valve with Autol Sub, Perc Endo Approach */
"02RF48Z"  =  "1"  /* Replace of Aortic Valve with Zooplastic, Perc Endo Approach */
"02RF4JZ"  =  "1"  /* Replace of Aortic Valve with Synth Sub, Perc Endo Approach */
"02RF4KZ"  =  "1"  /* Replace of Aortic Valve with Nonaut Sub, Perc Endo Approach */
"02RG07Z"  =  "1"  /* Replacement of Mitral Valve with Autol Sub, Open Approach */
"02RG08Z"  =  "1"  /* Replacement of Mitral Valve with Zooplastic, Open Approach */
"02RG0JZ"  =  "1"  /* Replacement of Mitral Valve with Synth Sub, Open Approach */
"02RG0KZ"  =  "1"  /* Replacement of Mitral Valve with Nonaut Sub, Open Approach */
"02RG37H"  =  "1"  /* Replace Mitral Valve w Autol Sub, Transap, Perc */
"02RG37Z"  =  "1"  /* Replacement of Mitral Valve with Autol Sub, Perc Approach */
"02RG38H"  =  "1"  /* Replace Mitral Valve w Zooplastic, Transap, Perc */
"02RG38Z"  =  "1"  /* Replacement of Mitral Valve with Zooplastic, Perc Approach */
"02RG3JH"  =  "1"  /* Replace Mitral Valve w Synth Sub, Transap, Perc */
"02RG3JZ"  =  "1"  /* Replacement of Mitral Valve with Synth Sub, Perc Approach */
"02RG3KH"  =  "1"  /* Replace Mitral Valve w Nonaut Sub, Transap, Perc */
"02RG3KZ"  =  "1"  /* Replacement of Mitral Valve with Nonaut Sub, Perc Approach */
"02RG47Z"  =  "1"  /* Replace of Mitral Valve with Autol Sub, Perc Endo Approach */
"02RG48Z"  =  "1"  /* Replace of Mitral Valve with Zooplastic, Perc Endo Approach */
"02RG4JZ"  =  "1"  /* Replace of Mitral Valve with Synth Sub, Perc Endo Approach */
"02RG4KZ"  =  "1"  /* Replace of Mitral Valve with Nonaut Sub, Perc Endo Approach */
"02RH07Z"  =  "1"  /* Replacement of Pulmonary Valve with Autol Sub, Open Approach */
"02RH08Z"  =  "1"  /* Replacement of Pulm Valve with Zooplastic, Open Approach */
"02RH0JZ"  =  "1"  /* Replacement of Pulmonary Valve with Synth Sub, Open Approach */
"02RH0KZ"  =  "1"  /* Replacement of Pulm Valve with Nonaut Sub, Open Approach */
"02RH37H"  =  "1"  /* Replace of Pulm Valve with Autol Sub, Transap, Perc Approach */
"02RH37Z"  =  "1"  /* Replacement of Pulmonary Valve with Autol Sub, Perc Approach */
"02RH38H"  =  "1"  /* Replace Pulm Valve w Zooplastic, Transap, Perc */
"02RH38Z"  =  "1"  /* Replacement of Pulm Valve with Zooplastic, Perc Approach */
"02RH3JH"  =  "1"  /* Replace of Pulm Valve with Synth Sub, Transap, Perc Approach */
"02RH3JZ"  =  "1"  /* Replacement of Pulmonary Valve with Synth Sub, Perc Approach */
"02RH3KH"  =  "1"  /* Replace Pulm Valve w Nonaut Sub, Transap, Perc */
"02RH3KZ"  =  "1"  /* Replacement of Pulm Valve with Nonaut Sub, Perc Approach */
"02RH47Z"  =  "1"  /* Replacement of Pulm Valve with Autol Sub, Perc Endo Approach */
"02RH48Z"  =  "1"  /* Replace of Pulm Valve with Zooplastic, Perc Endo Approach */
"02RH4JZ"  =  "1"  /* Replacement of Pulm Valve with Synth Sub, Perc Endo Approach */
"02RH4KZ"  =  "1"  /* Replace of Pulm Valve with Nonaut Sub, Perc Endo Approach */
"02RJ07Z"  =  "1"  /* Replacement of Tricuspid Valve with Autol Sub, Open Approach */
"02RJ08Z"  =  "1"  /* Replacement of Tricusp Valve with Zooplastic, Open Approach */
"02RJ0JZ"  =  "1"  /* Replacement of Tricuspid Valve with Synth Sub, Open Approach */
"02RJ0KZ"  =  "1"  /* Replacement of Tricusp Valve with Nonaut Sub, Open Approach */
"02RJ47Z"  =  "1"  /* Replace of Tricusp Valve with Autol Sub, Perc Endo Approach */
"02RJ48Z"  =  "1"  /* Replace of Tricusp Valve with Zooplastic, Perc Endo Approach */
"02RJ4JZ"  =  "1"  /* Replace of Tricusp Valve with Synth Sub, Perc Endo Approach */
"02RJ4KZ"  =  "1"  /* Replace of Tricusp Valve with Nonaut Sub, Perc Endo Approach */
"02RK07Z"  =  "1"  /* Replacement of Right Ventricle with Autol Sub, Open Approach */
"02RK08Z"  =  "1"  /* Replacement of R Ventricle with Zooplastic, Open Approach */
"02RK0JZ"  =  "1"  /* Replacement of Right Ventricle with Synth Sub, Open Approach */
"02RK0KZ"  =  "1"  /* Replacement of R Ventricle with Nonaut Sub, Open Approach */
"02RK47Z"  =  "1"  /* Replace of R Ventricle with Autol Sub, Perc Endo Approach */
"02RK48Z"  =  "1"  /* Replace of R Ventricle with Zooplastic, Perc Endo Approach */
"02RK4JZ"  =  "1"  /* Replace of R Ventricle with Synth Sub, Perc Endo Approach */
"02RK4KZ"  =  "1"  /* Replace of R Ventricle with Nonaut Sub, Perc Endo Approach */
"02RL07Z"  =  "1"  /* Replacement of Left Ventricle with Autol Sub, Open Approach */
"02RL08Z"  =  "1"  /* Replacement of Left Ventricle with Zooplastic, Open Approach */
"02RL0JZ"  =  "1"  /* Replacement of Left Ventricle with Synth Sub, Open Approach */
"02RL0KZ"  =  "1"  /* Replacement of Left Ventricle with Nonaut Sub, Open Approach */
"02RL47Z"  =  "1"  /* Replace of L Ventricle with Autol Sub, Perc Endo Approach */
"02RL48Z"  =  "1"  /* Replace of L Ventricle with Zooplastic, Perc Endo Approach */
"02RL4JZ"  =  "1"  /* Replace of L Ventricle with Synth Sub, Perc Endo Approach */
"02RL4KZ"  =  "1"  /* Replace of L Ventricle with Nonaut Sub, Perc Endo Approach */
"02RM07Z"  =  "1"  /* Replace of Ventricular Sept with Autol Sub, Open Approach */
"02RM08Z"  =  "1"  /* Replace of Ventricular Sept with Zooplastic, Open Approach */
"02RM0JZ"  =  "1"  /* Replace of Ventricular Sept with Synth Sub, Open Approach */
"02RM0KZ"  =  "1"  /* Replace of Ventricular Sept with Nonaut Sub, Open Approach */
"02RM47Z"  =  "1"  /* Replace Ventricular Sept w Autol Sub, Perc Endo */
"02RM48Z"  =  "1"  /* Replace Ventricular Sept w Zooplastic, Perc Endo */
"02RM4JZ"  =  "1"  /* Replace Ventricular Sept w Synth Sub, Perc Endo */
"02RM4KZ"  =  "1"  /* Replace Ventricular Sept w Nonaut Sub, Perc Endo */
"02RN07Z"  =  "1"  /* Replacement of Pericardium with Autol Sub, Open Approach */
"02RN08Z"  =  "1"  /* Replacement of Pericardium with Zooplastic, Open Approach */
"02RN0JZ"  =  "1"  /* Replacement of Pericardium with Synth Sub, Open Approach */
"02RN0KZ"  =  "1"  /* Replacement of Pericardium with Nonaut Sub, Open Approach */
"02RN47Z"  =  "1"  /* Replace of Pericardium with Autol Sub, Perc Endo Approach */
"02RN48Z"  =  "1"  /* Replace of Pericardium with Zooplastic, Perc Endo Approach */
"02RN4JZ"  =  "1"  /* Replace of Pericardium with Synth Sub, Perc Endo Approach */
"02RN4KZ"  =  "1"  /* Replace of Pericardium with Nonaut Sub, Perc Endo Approach */
"02RP07Z"  =  "1"  /* Replacement of Pulmonary Trunk with Autol Sub, Open Approach */
"02RP08Z"  =  "1"  /* Replacement of Pulm Trunk with Zooplastic, Open Approach */
"02RP0JZ"  =  "1"  /* Replacement of Pulmonary Trunk with Synth Sub, Open Approach */
"02RP0KZ"  =  "1"  /* Replacement of Pulm Trunk with Nonaut Sub, Open Approach */
"02RP47Z"  =  "1"  /* Replacement of Pulm Trunk with Autol Sub, Perc Endo Approach */
"02RP48Z"  =  "1"  /* Replace of Pulm Trunk with Zooplastic, Perc Endo Approach */
"02RP4JZ"  =  "1"  /* Replacement of Pulm Trunk with Synth Sub, Perc Endo Approach */
"02RP4KZ"  =  "1"  /* Replace of Pulm Trunk with Nonaut Sub, Perc Endo Approach */
"02RQ07Z"  =  "1"  /* Replacement of R Pulm Art with Autol Sub, Open Approach */
"02RQ08Z"  =  "1"  /* Replacement of R Pulm Art with Zooplastic, Open Approach */
"02RQ0JZ"  =  "1"  /* Replacement of R Pulm Art with Synth Sub, Open Approach */
"02RQ0KZ"  =  "1"  /* Replacement of R Pulm Art with Nonaut Sub, Open Approach */
"02RQ47Z"  =  "1"  /* Replacement of R Pulm Art with Autol Sub, Perc Endo Approach */
"02RQ48Z"  =  "1"  /* Replace of R Pulm Art with Zooplastic, Perc Endo Approach */
"02RQ4JZ"  =  "1"  /* Replacement of R Pulm Art with Synth Sub, Perc Endo Approach */
"02RQ4KZ"  =  "1"  /* Replace of R Pulm Art with Nonaut Sub, Perc Endo Approach */
"02RR07Z"  =  "1"  /* Replacement of L Pulm Art with Autol Sub, Open Approach */
"02RR08Z"  =  "1"  /* Replacement of L Pulm Art with Zooplastic, Open Approach */
"02RR0JZ"  =  "1"  /* Replacement of L Pulm Art with Synth Sub, Open Approach */
"02RR0KZ"  =  "1"  /* Replacement of L Pulm Art with Nonaut Sub, Open Approach */
"02RR47Z"  =  "1"  /* Replacement of L Pulm Art with Autol Sub, Perc Endo Approach */
"02RR48Z"  =  "1"  /* Replace of L Pulm Art with Zooplastic, Perc Endo Approach */
"02RR4JZ"  =  "1"  /* Replacement of L Pulm Art with Synth Sub, Perc Endo Approach */
"02RR4KZ"  =  "1"  /* Replace of L Pulm Art with Nonaut Sub, Perc Endo Approach */
"02RS07Z"  =  "1"  /* Replacement of R Pulm Vein with Autol Sub, Open Approach */
"02RS08Z"  =  "1"  /* Replacement of R Pulm Vein with Zooplastic, Open Approach */
"02RS0JZ"  =  "1"  /* Replacement of R Pulm Vein with Synth Sub, Open Approach */
"02RS0KZ"  =  "1"  /* Replacement of R Pulm Vein with Nonaut Sub, Open Approach */
"02RS47Z"  =  "1"  /* Replace of R Pulm Vein with Autol Sub, Perc Endo Approach */
"02RS48Z"  =  "1"  /* Replace of R Pulm Vein with Zooplastic, Perc Endo Approach */
"02RS4JZ"  =  "1"  /* Replace of R Pulm Vein with Synth Sub, Perc Endo Approach */
"02RS4KZ"  =  "1"  /* Replace of R Pulm Vein with Nonaut Sub, Perc Endo Approach */
"02RT07Z"  =  "1"  /* Replacement of L Pulm Vein with Autol Sub, Open Approach */
"02RT08Z"  =  "1"  /* Replacement of L Pulm Vein with Zooplastic, Open Approach */
"02RT0JZ"  =  "1"  /* Replacement of L Pulm Vein with Synth Sub, Open Approach */
"02RT0KZ"  =  "1"  /* Replacement of L Pulm Vein with Nonaut Sub, Open Approach */
"02RT47Z"  =  "1"  /* Replace of L Pulm Vein with Autol Sub, Perc Endo Approach */
"02RT48Z"  =  "1"  /* Replace of L Pulm Vein with Zooplastic, Perc Endo Approach */
"02RT4JZ"  =  "1"  /* Replace of L Pulm Vein with Synth Sub, Perc Endo Approach */
"02RT4KZ"  =  "1"  /* Replace of L Pulm Vein with Nonaut Sub, Perc Endo Approach */
"02RV07Z"  =  "1"  /* Replacement of Sup Vena Cava with Autol Sub, Open Approach */
"02RV08Z"  =  "1"  /* Replacement of Sup Vena Cava with Zooplastic, Open Approach */
"02RV0JZ"  =  "1"  /* Replacement of Sup Vena Cava with Synth Sub, Open Approach */
"02RV0KZ"  =  "1"  /* Replacement of Sup Vena Cava with Nonaut Sub, Open Approach */
"02RV47Z"  =  "1"  /* Replace of Sup Vena Cava with Autol Sub, Perc Endo Approach */
"02RV48Z"  =  "1"  /* Replace of Sup Vena Cava with Zooplastic, Perc Endo Approach */
"02RV4JZ"  =  "1"  /* Replace of Sup Vena Cava with Synth Sub, Perc Endo Approach */
"02RV4KZ"  =  "1"  /* Replace of Sup Vena Cava with Nonaut Sub, Perc Endo Approach */
"02RW07Z"  =  "1"  /* Replacement of Thoracic Aorta with Autol Sub, Open Approach */
"02RW08Z"  =  "1"  /* Replacement of Thoracic Aorta with Zooplastic, Open Approach */
"02RW0JZ"  =  "1"  /* Replacement of Thoracic Aorta with Synth Sub, Open Approach */
"02RW0KZ"  =  "1"  /* Replacement of Thoracic Aorta with Nonaut Sub, Open Approach */
"02RW47Z"  =  "1"  /* Replacement of Thor Aorta with Autol Sub, Perc Endo Approach */
"02RW48Z"  =  "1"  /* Replace of Thor Aorta with Zooplastic, Perc Endo Approach */
"02RW4JZ"  =  "1"  /* Replacement of Thor Aorta with Synth Sub, Perc Endo Approach */
"02RW4KZ"  =  "1"  /* Replace of Thor Aorta with Nonaut Sub, Perc Endo Approach */
"02SP0ZZ"  =  "1"  /* Reposition Pulmonary Trunk, Open Approach */
"02SQ0ZZ"  =  "1"  /* Reposition Right Pulmonary Artery, Open Approach */
"02SR0ZZ"  =  "1"  /* Reposition Left Pulmonary Artery, Open Approach */
"02SS0ZZ"  =  "1"  /* Reposition Right Pulmonary Vein, Open Approach */
"02ST0ZZ"  =  "1"  /* Reposition Left Pulmonary Vein, Open Approach */
"02SV0ZZ"  =  "1"  /* Reposition Superior Vena Cava, Open Approach */
"02SW0ZZ"  =  "1"  /* Reposition Thoracic Aorta, Open Approach */
"02T50ZZ"  =  "1"  /* Resection of Atrial Septum, Open Approach */
"02T53ZZ"  =  "1"  /* Resection of Atrial Septum, Percutaneous Approach */
"02T54ZZ"  =  "1"  /* Resection of Atrial Septum, Percutaneous Endoscopic Approach */
"02T80ZZ"  =  "1"  /* Resection of Conduction Mechanism, Open Approach */
"02T83ZZ"  =  "1"  /* Resection of Conduction Mechanism, Percutaneous Approach */
"02T84ZZ"  =  "1"  /* Resection of Conduction Mechanism, Perc Endo Approach */
"02T90ZZ"  =  "1"  /* Resection of Chordae Tendineae, Open Approach */
"02T93ZZ"  =  "1"  /* Resection of Chordae Tendineae, Percutaneous Approach */
"02T94ZZ"  =  "1"  /* Resection of Chordae Tendineae, Perc Endo Approach */
"02TD0ZZ"  =  "1"  /* Resection of Papillary Muscle, Open Approach */
"02TD3ZZ"  =  "1"  /* Resection of Papillary Muscle, Percutaneous Approach */
"02TD4ZZ"  =  "1"  /* Resection of Papillary Muscle, Perc Endo Approach */
"02TH0ZZ"  =  "1"  /* Resection of Pulmonary Valve, Open Approach */
"02TH3ZZ"  =  "1"  /* Resection of Pulmonary Valve, Percutaneous Approach */
"02TH4ZZ"  =  "1"  /* Resection of Pulmonary Valve, Perc Endo Approach */
"02TM0ZZ"  =  "1"  /* Resection of Ventricular Septum, Open Approach */
"02TM3ZZ"  =  "1"  /* Resection of Ventricular Septum, Percutaneous Approach */
"02TM4ZZ"  =  "1"  /* Resection of Ventricular Septum, Perc Endo Approach */
"02TN0ZZ"  =  "1"  /* Resection of Pericardium, Open Approach */
"02TN3ZZ"  =  "1"  /* Resection of Pericardium, Percutaneous Approach */
"02TN4ZZ"  =  "1"  /* Resection of Pericardium, Percutaneous Endoscopic Approach */
"02U507Z"  =  "1"  /* Supplement Atrial Septum with Autol Sub, Open Approach */
"02U508Z"  =  "1"  /* Supplement Atrial Septum with Zooplastic, Open Approach */
"02U50JZ"  =  "1"  /* Supplement Atrial Septum with Synth Sub, Open Approach */
"02U50KZ"  =  "1"  /* Supplement Atrial Septum with Nonaut Sub, Open Approach */
"02U537Z"  =  "1"  /* Supplement Atrial Septum with Autol Sub, Perc Approach */
"02U538Z"  =  "1"  /* Supplement Atrial Septum with Zooplastic, Perc Approach */
"02U53JZ"  =  "1"  /* Supplement Atrial Septum with Synth Sub, Perc Approach */
"02U53KZ"  =  "1"  /* Supplement Atrial Septum with Nonaut Sub, Perc Approach */
"02U547Z"  =  "1"  /* Supplement Atrial Septum with Autol Sub, Perc Endo Approach */
"02U548Z"  =  "1"  /* Supplement Atrial Septum with Zooplastic, Perc Endo Approach */
"02U54JZ"  =  "1"  /* Supplement Atrial Septum with Synth Sub, Perc Endo Approach */
"02U54KZ"  =  "1"  /* Supplement Atrial Septum with Nonaut Sub, Perc Endo Approach */
"02U607Z"  =  "1"  /* Supplement Right Atrium with Autol Sub, Open Approach */
"02U608Z"  =  "1"  /* Supplement Right Atrium with Zooplastic, Open Approach */
"02U60JZ"  =  "1"  /* Supplement Right Atrium with Synth Sub, Open Approach */
"02U60KZ"  =  "1"  /* Supplement Right Atrium with Nonaut Sub, Open Approach */
"02U637Z"  =  "1"  /* Supplement Right Atrium with Autol Sub, Perc Approach */
"02U638Z"  =  "1"  /* Supplement Right Atrium with Zooplastic, Perc Approach */
"02U63JZ"  =  "1"  /* Supplement Right Atrium with Synth Sub, Perc Approach */
"02U63KZ"  =  "1"  /* Supplement Right Atrium with Nonaut Sub, Perc Approach */
"02U647Z"  =  "1"  /* Supplement Right Atrium with Autol Sub, Perc Endo Approach */
"02U648Z"  =  "1"  /* Supplement Right Atrium with Zooplastic, Perc Endo Approach */
"02U64JZ"  =  "1"  /* Supplement Right Atrium with Synth Sub, Perc Endo Approach */
"02U64KZ"  =  "1"  /* Supplement Right Atrium with Nonaut Sub, Perc Endo Approach */
"02U707Z"  =  "1"  /* Supplement Left Atrium with Autol Sub, Open Approach */
"02U708Z"  =  "1"  /* Supplement Left Atrium with Zooplastic Tissue, Open Approach */
"02U70JZ"  =  "1"  /* Supplement Left Atrium with Synth Sub, Open Approach */
"02U70KZ"  =  "1"  /* Supplement Left Atrium with Nonaut Sub, Open Approach */
"02U737Z"  =  "1"  /* Supplement Left Atrium with Autol Sub, Perc Approach */
"02U738Z"  =  "1"  /* Supplement Left Atrium with Zooplastic Tissue, Perc Approach */
"02U73KZ"  =  "1"  /* Supplement Left Atrium with Nonaut Sub, Perc Approach */
"02U747Z"  =  "1"  /* Supplement Left Atrium with Autol Sub, Perc Endo Approach */
"02U748Z"  =  "1"  /* Supplement Left Atrium with Zooplastic, Perc Endo Approach */
"02U74KZ"  =  "1"  /* Supplement Left Atrium with Nonaut Sub, Perc Endo Approach */
"02U907Z"  =  "1"  /* Supplement Chordae Tendineae with Autol Sub, Open Approach */
"02U908Z"  =  "1"  /* Supplement Chordae Tendineae with Zooplastic, Open Approach */
"02U90JZ"  =  "1"  /* Supplement Chordae Tendineae with Synth Sub, Open Approach */
"02U90KZ"  =  "1"  /* Supplement Chordae Tendineae with Nonaut Sub, Open Approach */
"02U937Z"  =  "1"  /* Supplement Chordae Tendineae with Autol Sub, Perc Approach */
"02U938Z"  =  "1"  /* Supplement Chordae Tendineae with Zooplastic, Perc Approach */
"02U93JZ"  =  "1"  /* Supplement Chordae Tendineae with Synth Sub, Perc Approach */
"02U93KZ"  =  "1"  /* Supplement Chordae Tendineae with Nonaut Sub, Perc Approach */
"02U947Z"  =  "1"  /* Supplement Chordae Tendineae w Autol Sub, Perc Endo */
"02U948Z"  =  "1"  /* Supplement Chordae Tendineae w Zooplastic, Perc Endo */
"02U94JZ"  =  "1"  /* Supplement Chordae Tendineae w Synth Sub, Perc Endo */
"02U94KZ"  =  "1"  /* Supplement Chordae Tendineae w Nonaut Sub, Perc Endo */
"02UA07Z"  =  "1"  /* Supplement Heart with Autol Sub, Open Approach */
"02UA08Z"  =  "1"  /* Supplement Heart with Zooplastic Tissue, Open Approach */
"02UA0JZ"  =  "1"  /* Supplement Heart with Synthetic Substitute, Open Approach */
"02UA0KZ"  =  "1"  /* Supplement Heart with Nonaut Sub, Open Approach */
"02UA37Z"  =  "1"  /* Supplement Heart with Autol Sub, Perc Approach */
"02UA38Z"  =  "1"  /* Supplement Heart with Zooplastic Tissue, Perc Approach */
"02UA3JZ"  =  "1"  /* Supplement Heart with Synthetic Substitute, Perc Approach */
"02UA3KZ"  =  "1"  /* Supplement Heart with Nonaut Sub, Perc Approach */
"02UA47Z"  =  "1"  /* Supplement Heart with Autol Sub, Perc Endo Approach */
"02UA48Z"  =  "1"  /* Supplement Heart with Zooplastic Tissue, Perc Endo Approach */
"02UA4JZ"  =  "1"  /* Supplement Heart with Synth Sub, Perc Endo Approach */
"02UA4KZ"  =  "1"  /* Supplement Heart with Nonaut Sub, Perc Endo Approach */
"02UD07Z"  =  "1"  /* Supplement Papillary Muscle with Autol Sub, Open Approach */
"02UD08Z"  =  "1"  /* Supplement Papillary Muscle with Zooplastic, Open Approach */
"02UD0JZ"  =  "1"  /* Supplement Papillary Muscle with Synth Sub, Open Approach */
"02UD0KZ"  =  "1"  /* Supplement Papillary Muscle with Nonaut Sub, Open Approach */
"02UD37Z"  =  "1"  /* Supplement Papillary Muscle with Autol Sub, Perc Approach */
"02UD38Z"  =  "1"  /* Supplement Papillary Muscle with Zooplastic, Perc Approach */
"02UD3JZ"  =  "1"  /* Supplement Papillary Muscle with Synth Sub, Perc Approach */
"02UD3KZ"  =  "1"  /* Supplement Papillary Muscle with Nonaut Sub, Perc Approach */
"02UD47Z"  =  "1"  /* Supplement Papillary Muscle w Autol Sub, Perc Endo */
"02UD48Z"  =  "1"  /* Supplement Papillary Muscle w Zooplastic, Perc Endo */
"02UD4JZ"  =  "1"  /* Supplement Papillary Muscle w Synth Sub, Perc Endo */
"02UD4KZ"  =  "1"  /* Supplement Papillary Muscle w Nonaut Sub, Perc Endo */
"02UF07Z"  =  "1"  /* Supplement Aortic Valve with Autol Sub, Open Approach */
"02UF08Z"  =  "1"  /* Supplement Aortic Valve with Zooplastic, Open Approach */
"02UF0JZ"  =  "1"  /* Supplement Aortic Valve with Synth Sub, Open Approach */
"02UF0KZ"  =  "1"  /* Supplement Aortic Valve with Nonaut Sub, Open Approach */
"02UF37Z"  =  "1"  /* Supplement Aortic Valve with Autol Sub, Perc Approach */
"02UF38Z"  =  "1"  /* Supplement Aortic Valve with Zooplastic, Perc Approach */
"02UF3JZ"  =  "1"  /* Supplement Aortic Valve with Synth Sub, Perc Approach */
"02UF3KZ"  =  "1"  /* Supplement Aortic Valve with Nonaut Sub, Perc Approach */
"02UF47Z"  =  "1"  /* Supplement Aortic Valve with Autol Sub, Perc Endo Approach */
"02UF48Z"  =  "1"  /* Supplement Aortic Valve with Zooplastic, Perc Endo Approach */
"02UF4JZ"  =  "1"  /* Supplement Aortic Valve with Synth Sub, Perc Endo Approach */
"02UF4KZ"  =  "1"  /* Supplement Aortic Valve with Nonaut Sub, Perc Endo Approach */
"02UG07Z"  =  "1"  /* Supplement Mitral Valve with Autol Sub, Open Approach */
"02UG08Z"  =  "1"  /* Supplement Mitral Valve with Zooplastic, Open Approach */
"02UG0JZ"  =  "1"  /* Supplement Mitral Valve with Synth Sub, Open Approach */
"02UG0KZ"  =  "1"  /* Supplement Mitral Valve with Nonaut Sub, Open Approach */
"02UG37Z"  =  "1"  /* Supplement Mitral Valve with Autol Sub, Perc Approach */
"02UG38Z"  =  "1"  /* Supplement Mitral Valve with Zooplastic, Perc Approach */
"02UG3JZ"  =  "1"  /* Supplement Mitral Valve with Synth Sub, Perc Approach */
"02UG3KZ"  =  "1"  /* Supplement Mitral Valve with Nonaut Sub, Perc Approach */
"02UG47Z"  =  "1"  /* Supplement Mitral Valve with Autol Sub, Perc Endo Approach */
"02UG48Z"  =  "1"  /* Supplement Mitral Valve with Zooplastic, Perc Endo Approach */
"02UG4JZ"  =  "1"  /* Supplement Mitral Valve with Synth Sub, Perc Endo Approach */
"02UG4KZ"  =  "1"  /* Supplement Mitral Valve with Nonaut Sub, Perc Endo Approach */
"02UH07Z"  =  "1"  /* Supplement Pulmonary Valve with Autol Sub, Open Approach */
"02UH08Z"  =  "1"  /* Supplement Pulmonary Valve with Zooplastic, Open Approach */
"02UH0JZ"  =  "1"  /* Supplement Pulmonary Valve with Synth Sub, Open Approach */
"02UH0KZ"  =  "1"  /* Supplement Pulmonary Valve with Nonaut Sub, Open Approach */
"02UH37Z"  =  "1"  /* Supplement Pulmonary Valve with Autol Sub, Perc Approach */
"02UH38Z"  =  "1"  /* Supplement Pulmonary Valve with Zooplastic, Perc Approach */
"02UH3JZ"  =  "1"  /* Supplement Pulmonary Valve with Synth Sub, Perc Approach */
"02UH3KZ"  =  "1"  /* Supplement Pulmonary Valve with Nonaut Sub, Perc Approach */
"02UH47Z"  =  "1"  /* Supplement Pulm Valve with Autol Sub, Perc Endo Approach */
"02UH48Z"  =  "1"  /* Supplement Pulm Valve with Zooplastic, Perc Endo Approach */
"02UH4JZ"  =  "1"  /* Supplement Pulm Valve with Synth Sub, Perc Endo Approach */
"02UH4KZ"  =  "1"  /* Supplement Pulm Valve with Nonaut Sub, Perc Endo Approach */
"02UJ07Z"  =  "1"  /* Supplement Tricuspid Valve with Autol Sub, Open Approach */
"02UJ08Z"  =  "1"  /* Supplement Tricuspid Valve with Zooplastic, Open Approach */
"02UJ0JZ"  =  "1"  /* Supplement Tricuspid Valve with Synth Sub, Open Approach */
"02UJ0KZ"  =  "1"  /* Supplement Tricuspid Valve with Nonaut Sub, Open Approach */
"02UJ37Z"  =  "1"  /* Supplement Tricuspid Valve with Autol Sub, Perc Approach */
"02UJ38Z"  =  "1"  /* Supplement Tricuspid Valve with Zooplastic, Perc Approach */
"02UJ3JZ"  =  "1"  /* Supplement Tricuspid Valve with Synth Sub, Perc Approach */
"02UJ3KZ"  =  "1"  /* Supplement Tricuspid Valve with Nonaut Sub, Perc Approach */
"02UJ47Z"  =  "1"  /* Supplement Tricusp Valve with Autol Sub, Perc Endo Approach */
"02UJ48Z"  =  "1"  /* Supplement Tricusp Valve with Zooplastic, Perc Endo Approach */
"02UJ4JZ"  =  "1"  /* Supplement Tricusp Valve with Synth Sub, Perc Endo Approach */
"02UJ4KZ"  =  "1"  /* Supplement Tricusp Valve with Nonaut Sub, Perc Endo Approach */
"02UK07Z"  =  "1"  /* Supplement Right Ventricle with Autol Sub, Open Approach */
"02UK08Z"  =  "1"  /* Supplement Right Ventricle with Zooplastic, Open Approach */
"02UK0JZ"  =  "1"  /* Supplement Right Ventricle with Synth Sub, Open Approach */
"02UK0KZ"  =  "1"  /* Supplement Right Ventricle with Nonaut Sub, Open Approach */
"02UK37Z"  =  "1"  /* Supplement Right Ventricle with Autol Sub, Perc Approach */
"02UK38Z"  =  "1"  /* Supplement Right Ventricle with Zooplastic, Perc Approach */
"02UK3JZ"  =  "1"  /* Supplement Right Ventricle with Synth Sub, Perc Approach */
"02UK3KZ"  =  "1"  /* Supplement Right Ventricle with Nonaut Sub, Perc Approach */
"02UK47Z"  =  "1"  /* Supplement R Ventricle with Autol Sub, Perc Endo Approach */
"02UK48Z"  =  "1"  /* Supplement R Ventricle with Zooplastic, Perc Endo Approach */
"02UK4JZ"  =  "1"  /* Supplement R Ventricle with Synth Sub, Perc Endo Approach */
"02UK4KZ"  =  "1"  /* Supplement R Ventricle with Nonaut Sub, Perc Endo Approach */
"02UL07Z"  =  "1"  /* Supplement Left Ventricle with Autol Sub, Open Approach */
"02UL08Z"  =  "1"  /* Supplement Left Ventricle with Zooplastic, Open Approach */
"02UL0JZ"  =  "1"  /* Supplement Left Ventricle with Synth Sub, Open Approach */
"02UL0KZ"  =  "1"  /* Supplement Left Ventricle with Nonaut Sub, Open Approach */
"02UL37Z"  =  "1"  /* Supplement Left Ventricle with Autol Sub, Perc Approach */
"02UL38Z"  =  "1"  /* Supplement Left Ventricle with Zooplastic, Perc Approach */
"02UL3JZ"  =  "1"  /* Supplement Left Ventricle with Synth Sub, Perc Approach */
"02UL3KZ"  =  "1"  /* Supplement Left Ventricle with Nonaut Sub, Perc Approach */
"02UL47Z"  =  "1"  /* Supplement Left Ventricle with Autol Sub, Perc Endo Approach */
"02UL48Z"  =  "1"  /* Supplement L Ventricle with Zooplastic, Perc Endo Approach */
"02UL4JZ"  =  "1"  /* Supplement Left Ventricle with Synth Sub, Perc Endo Approach */
"02UL4KZ"  =  "1"  /* Supplement L Ventricle with Nonaut Sub, Perc Endo Approach */
"02UM07Z"  =  "1"  /* Supplement Ventricular Septum with Autol Sub, Open Approach */
"02UM08Z"  =  "1"  /* Supplement Ventricular Septum with Zooplastic, Open Approach */
"02UM0JZ"  =  "1"  /* Supplement Ventricular Septum with Synth Sub, Open Approach */
"02UM0KZ"  =  "1"  /* Supplement Ventricular Septum with Nonaut Sub, Open Approach */
"02UM37Z"  =  "1"  /* Supplement Ventricular Septum with Autol Sub, Perc Approach */
"02UM38Z"  =  "1"  /* Supplement Ventricular Septum with Zooplastic, Perc Approach */
"02UM3JZ"  =  "1"  /* Supplement Ventricular Septum with Synth Sub, Perc Approach */
"02UM3KZ"  =  "1"  /* Supplement Ventricular Septum with Nonaut Sub, Perc Approach */
"02UM47Z"  =  "1"  /* Supplement Ventricular Sept w Autol Sub, Perc Endo */
"02UM48Z"  =  "1"  /* Supplement Ventricular Sept w Zooplastic, Perc Endo */
"02UM4JZ"  =  "1"  /* Supplement Ventricular Sept w Synth Sub, Perc Endo */
"02UM4KZ"  =  "1"  /* Supplement Ventricular Sept w Nonaut Sub, Perc Endo */
"02UN07Z"  =  "1"  /* Supplement Pericardium with Autol Sub, Open Approach */
"02UN08Z"  =  "1"  /* Supplement Pericardium with Zooplastic Tissue, Open Approach */
"02UN0JZ"  =  "1"  /* Supplement Pericardium with Synth Sub, Open Approach */
"02UN0KZ"  =  "1"  /* Supplement Pericardium with Nonaut Sub, Open Approach */
"02UN37Z"  =  "1"  /* Supplement Pericardium with Autol Sub, Perc Approach */
"02UN38Z"  =  "1"  /* Supplement Pericardium with Zooplastic Tissue, Perc Approach */
"02UN3JZ"  =  "1"  /* Supplement Pericardium with Synth Sub, Perc Approach */
"02UN3KZ"  =  "1"  /* Supplement Pericardium with Nonaut Sub, Perc Approach */
"02UN47Z"  =  "1"  /* Supplement Pericardium with Autol Sub, Perc Endo Approach */
"02UN48Z"  =  "1"  /* Supplement Pericardium with Zooplastic, Perc Endo Approach */
"02UN4JZ"  =  "1"  /* Supplement Pericardium with Synth Sub, Perc Endo Approach */
"02UN4KZ"  =  "1"  /* Supplement Pericardium with Nonaut Sub, Perc Endo Approach */
"02UP07Z"  =  "1"  /* Supplement Pulmonary Trunk with Autol Sub, Open Approach */
"02UP08Z"  =  "1"  /* Supplement Pulmonary Trunk with Zooplastic, Open Approach */
"02UP0JZ"  =  "1"  /* Supplement Pulmonary Trunk with Synth Sub, Open Approach */
"02UP0KZ"  =  "1"  /* Supplement Pulmonary Trunk with Nonaut Sub, Open Approach */
"02UP37Z"  =  "1"  /* Supplement Pulmonary Trunk with Autol Sub, Perc Approach */
"02UP38Z"  =  "1"  /* Supplement Pulmonary Trunk with Zooplastic, Perc Approach */
"02UP3JZ"  =  "1"  /* Supplement Pulmonary Trunk with Synth Sub, Perc Approach */
"02UP3KZ"  =  "1"  /* Supplement Pulmonary Trunk with Nonaut Sub, Perc Approach */
"02UP47Z"  =  "1"  /* Supplement Pulm Trunk with Autol Sub, Perc Endo Approach */
"02UP48Z"  =  "1"  /* Supplement Pulm Trunk with Zooplastic, Perc Endo Approach */
"02UP4JZ"  =  "1"  /* Supplement Pulm Trunk with Synth Sub, Perc Endo Approach */
"02UP4KZ"  =  "1"  /* Supplement Pulm Trunk with Nonaut Sub, Perc Endo Approach */
"02UQ07Z"  =  "1"  /* Supplement R Pulm Art with Autol Sub, Open Approach */
"02UQ08Z"  =  "1"  /* Supplement R Pulm Art with Zooplastic, Open Approach */
"02UQ0JZ"  =  "1"  /* Supplement R Pulm Art with Synth Sub, Open Approach */
"02UQ0KZ"  =  "1"  /* Supplement R Pulm Art with Nonaut Sub, Open Approach */
"02UQ37Z"  =  "1"  /* Supplement R Pulm Art with Autol Sub, Perc Approach */
"02UQ38Z"  =  "1"  /* Supplement R Pulm Art with Zooplastic, Perc Approach */
"02UQ3JZ"  =  "1"  /* Supplement R Pulm Art with Synth Sub, Perc Approach */
"02UQ3KZ"  =  "1"  /* Supplement R Pulm Art with Nonaut Sub, Perc Approach */
"02UQ47Z"  =  "1"  /* Supplement R Pulm Art with Autol Sub, Perc Endo Approach */
"02UQ48Z"  =  "1"  /* Supplement R Pulm Art with Zooplastic, Perc Endo Approach */
"02UQ4JZ"  =  "1"  /* Supplement R Pulm Art with Synth Sub, Perc Endo Approach */
"02UQ4KZ"  =  "1"  /* Supplement R Pulm Art with Nonaut Sub, Perc Endo Approach */
"02UR07Z"  =  "1"  /* Supplement L Pulm Art with Autol Sub, Open Approach */
"02UR08Z"  =  "1"  /* Supplement L Pulm Art with Zooplastic, Open Approach */
"02UR0JZ"  =  "1"  /* Supplement L Pulm Art with Synth Sub, Open Approach */
"02UR0KZ"  =  "1"  /* Supplement L Pulm Art with Nonaut Sub, Open Approach */
"02UR37Z"  =  "1"  /* Supplement L Pulm Art with Autol Sub, Perc Approach */
"02UR38Z"  =  "1"  /* Supplement L Pulm Art with Zooplastic, Perc Approach */
"02UR3JZ"  =  "1"  /* Supplement L Pulm Art with Synth Sub, Perc Approach */
"02UR3KZ"  =  "1"  /* Supplement L Pulm Art with Nonaut Sub, Perc Approach */
"02UR47Z"  =  "1"  /* Supplement L Pulm Art with Autol Sub, Perc Endo Approach */
"02UR48Z"  =  "1"  /* Supplement L Pulm Art with Zooplastic, Perc Endo Approach */
"02UR4JZ"  =  "1"  /* Supplement L Pulm Art with Synth Sub, Perc Endo Approach */
"02UR4KZ"  =  "1"  /* Supplement L Pulm Art with Nonaut Sub, Perc Endo Approach */
"02US07Z"  =  "1"  /* Supplement R Pulm Vein with Autol Sub, Open Approach */
"02US08Z"  =  "1"  /* Supplement R Pulm Vein with Zooplastic, Open Approach */
"02US0JZ"  =  "1"  /* Supplement R Pulm Vein with Synth Sub, Open Approach */
"02US0KZ"  =  "1"  /* Supplement R Pulm Vein with Nonaut Sub, Open Approach */
"02US37Z"  =  "1"  /* Supplement R Pulm Vein with Autol Sub, Perc Approach */
"02US38Z"  =  "1"  /* Supplement R Pulm Vein with Zooplastic, Perc Approach */
"02US3JZ"  =  "1"  /* Supplement R Pulm Vein with Synth Sub, Perc Approach */
"02US3KZ"  =  "1"  /* Supplement R Pulm Vein with Nonaut Sub, Perc Approach */
"02US47Z"  =  "1"  /* Supplement R Pulm Vein with Autol Sub, Perc Endo Approach */
"02US48Z"  =  "1"  /* Supplement R Pulm Vein with Zooplastic, Perc Endo Approach */
"02US4JZ"  =  "1"  /* Supplement R Pulm Vein with Synth Sub, Perc Endo Approach */
"02US4KZ"  =  "1"  /* Supplement R Pulm Vein with Nonaut Sub, Perc Endo Approach */
"02UT07Z"  =  "1"  /* Supplement Left Pulmonary Vein with Autol Sub, Open Approach */
"02UT08Z"  =  "1"  /* Supplement L Pulm Vein with Zooplastic, Open Approach */
"02UT0JZ"  =  "1"  /* Supplement Left Pulmonary Vein with Synth Sub, Open Approach */
"02UT0KZ"  =  "1"  /* Supplement L Pulm Vein with Nonaut Sub, Open Approach */
"02UT37Z"  =  "1"  /* Supplement Left Pulmonary Vein with Autol Sub, Perc Approach */
"02UT38Z"  =  "1"  /* Supplement L Pulm Vein with Zooplastic, Perc Approach */
"02UT3JZ"  =  "1"  /* Supplement Left Pulmonary Vein with Synth Sub, Perc Approach */
"02UT3KZ"  =  "1"  /* Supplement L Pulm Vein with Nonaut Sub, Perc Approach */
"02UT47Z"  =  "1"  /* Supplement L Pulm Vein with Autol Sub, Perc Endo Approach */
"02UT48Z"  =  "1"  /* Supplement L Pulm Vein with Zooplastic, Perc Endo Approach */
"02UT4JZ"  =  "1"  /* Supplement L Pulm Vein with Synth Sub, Perc Endo Approach */
"02UT4KZ"  =  "1"  /* Supplement L Pulm Vein with Nonaut Sub, Perc Endo Approach */
"02UV07Z"  =  "1"  /* Supplement Superior Vena Cava with Autol Sub, Open Approach */
"02UV08Z"  =  "1"  /* Supplement Superior Vena Cava with Zooplastic, Open Approach */
"02UV0JZ"  =  "1"  /* Supplement Superior Vena Cava with Synth Sub, Open Approach */
"02UV0KZ"  =  "1"  /* Supplement Superior Vena Cava with Nonaut Sub, Open Approach */
"02UV37Z"  =  "1"  /* Supplement Superior Vena Cava with Autol Sub, Perc Approach */
"02UV38Z"  =  "1"  /* Supplement Superior Vena Cava with Zooplastic, Perc Approach */
"02UV3JZ"  =  "1"  /* Supplement Superior Vena Cava with Synth Sub, Perc Approach */
"02UV3KZ"  =  "1"  /* Supplement Superior Vena Cava with Nonaut Sub, Perc Approach */
"02UV47Z"  =  "1"  /* Supplement Sup Vena Cava with Autol Sub, Perc Endo Approach */
"02UV48Z"  =  "1"  /* Supplement Sup Vena Cava with Zooplastic, Perc Endo Approach */
"02UV4JZ"  =  "1"  /* Supplement Sup Vena Cava with Synth Sub, Perc Endo Approach */
"02UV4KZ"  =  "1"  /* Supplement Sup Vena Cava with Nonaut Sub, Perc Endo Approach */
"02UW07Z"  =  "1"  /* Supplement Thoracic Aorta with Autol Sub, Open Approach */
"02UW08Z"  =  "1"  /* Supplement Thoracic Aorta with Zooplastic, Open Approach */
"02UW0JZ"  =  "1"  /* Supplement Thoracic Aorta with Synth Sub, Open Approach */
"02UW0KZ"  =  "1"  /* Supplement Thoracic Aorta with Nonaut Sub, Open Approach */
"02UW37Z"  =  "1"  /* Supplement Thoracic Aorta with Autol Sub, Perc Approach */
"02UW38Z"  =  "1"  /* Supplement Thoracic Aorta with Zooplastic, Perc Approach */
"02UW3JZ"  =  "1"  /* Supplement Thoracic Aorta with Synth Sub, Perc Approach */
"02UW3KZ"  =  "1"  /* Supplement Thoracic Aorta with Nonaut Sub, Perc Approach */
"02UW47Z"  =  "1"  /* Supplement Thoracic Aorta with Autol Sub, Perc Endo Approach */
"02UW48Z"  =  "1"  /* Supplement Thor Aorta with Zooplastic, Perc Endo Approach */
"02UW4JZ"  =  "1"  /* Supplement Thoracic Aorta with Synth Sub, Perc Endo Approach */
"02UW4KZ"  =  "1"  /* Supplement Thor Aorta with Nonaut Sub, Perc Endo Approach */
"02VA0CZ"  =  "1"  /* Restriction of Heart with Extraluminal Device, Open Approach */
"02VA0ZZ"  =  "1"  /* Restriction of Heart, Open Approach */
"02VA3CZ"  =  "1"  /* Restriction of Heart with Extraluminal Device, Perc Approach */
"02VA3ZZ"  =  "1"  /* Restriction of Heart, Percutaneous Approach */
"02VA4CZ"  =  "1"  /* Restriction of Heart with Extralum Dev, Perc Endo Approach */
"02VA4ZZ"  =  "1"  /* Restriction of Heart, Percutaneous Endoscopic Approach */
"02VP0CZ"  =  "1"  /* Restriction of Pulm Trunk with Extralum Dev, Open Approach */
"02VP0DZ"  =  "1"  /* Restriction of Pulm Trunk with Intralum Dev, Open Approach */
"02VP0ZZ"  =  "1"  /* Restriction of Pulmonary Trunk, Open Approach */
"02VP3CZ"  =  "1"  /* Restriction of Pulm Trunk with Extralum Dev, Perc Approach */
"02VP3DZ"  =  "1"  /* Restriction of Pulm Trunk with Intralum Dev, Perc Approach */
"02VP3ZZ"  =  "1"  /* Restriction of Pulmonary Trunk, Percutaneous Approach */
"02VP4CZ"  =  "1"  /* Restrict of Pulm Trunk with Extralum Dev, Perc Endo Approach */
"02VP4DZ"  =  "1"  /* Restrict of Pulm Trunk with Intralum Dev, Perc Endo Approach */
"02VP4ZZ"  =  "1"  /* Restriction of Pulmonary Trunk, Perc Endo Approach */
"02VQ0CZ"  =  "1"  /* Restriction of R Pulm Art with Extralum Dev, Open Approach */
"02VQ0DZ"  =  "1"  /* Restriction of R Pulm Art with Intralum Dev, Open Approach */
"02VQ0ZZ"  =  "1"  /* Restriction of Right Pulmonary Artery, Open Approach */
"02VQ3CZ"  =  "1"  /* Restriction of R Pulm Art with Extralum Dev, Perc Approach */
"02VQ3DZ"  =  "1"  /* Restriction of R Pulm Art with Intralum Dev, Perc Approach */
"02VQ3ZZ"  =  "1"  /* Restriction of Right Pulmonary Artery, Percutaneous Approach */
"02VQ4CZ"  =  "1"  /* Restrict of R Pulm Art with Extralum Dev, Perc Endo Approach */
"02VQ4DZ"  =  "1"  /* Restrict of R Pulm Art with Intralum Dev, Perc Endo Approach */
"02VQ4ZZ"  =  "1"  /* Restriction of Right Pulmonary Artery, Perc Endo Approach */
"02VR0CT"  =  "1"  /* Restrict of Ductus Arterio with Extralum Dev, Open Approach */
"02VR0CZ"  =  "1"  /* Restriction of L Pulm Art with Extralum Dev, Open Approach */
"02VR0DT"  =  "1"  /* Restrict of Ductus Arterio with Intralum Dev, Open Approach */
"02VR0DZ"  =  "1"  /* Restriction of L Pulm Art with Intralum Dev, Open Approach */
"02VR0ZT"  =  "1"  /* Restriction of Ductus Arteriosus, Open Approach */
"02VR0ZZ"  =  "1"  /* Restriction of Left Pulmonary Artery, Open Approach */
"02VR3CT"  =  "1"  /* Restrict of Ductus Arterio with Extralum Dev, Perc Approach */
"02VR3CZ"  =  "1"  /* Restriction of L Pulm Art with Extralum Dev, Perc Approach */
"02VR3DT"  =  "1"  /* Restrict of Ductus Arterio with Intralum Dev, Perc Approach */
"02VR3DZ"  =  "1"  /* Restriction of L Pulm Art with Intralum Dev, Perc Approach */
"02VR3ZT"  =  "1"  /* Restriction of Ductus Arteriosus, Percutaneous Approach */
"02VR3ZZ"  =  "1"  /* Restriction of Left Pulmonary Artery, Percutaneous Approach */
"02VR4CT"  =  "1"  /* Restrict Ductus Arterio w Extralum Dev, Perc Endo */
"02VR4CZ"  =  "1"  /* Restrict of L Pulm Art with Extralum Dev, Perc Endo Approach */
"02VR4DT"  =  "1"  /* Restrict Ductus Arterio w Intralum Dev, Perc Endo */
"02VR4DZ"  =  "1"  /* Restrict of L Pulm Art with Intralum Dev, Perc Endo Approach */
"02VR4ZT"  =  "1"  /* Restriction of Ductus Arteriosus, Perc Endo Approach */
"02VR4ZZ"  =  "1"  /* Restriction of Left Pulmonary Artery, Perc Endo Approach */
"02VS0CZ"  =  "1"  /* Restriction of R Pulm Vein with Extralum Dev, Open Approach */
"02VS0DZ"  =  "1"  /* Restriction of R Pulm Vein with Intralum Dev, Open Approach */
"02VS0ZZ"  =  "1"  /* Restriction of Right Pulmonary Vein, Open Approach */
"02VS3CZ"  =  "1"  /* Restriction of R Pulm Vein with Extralum Dev, Perc Approach */
"02VS3DZ"  =  "1"  /* Restriction of R Pulm Vein with Intralum Dev, Perc Approach */
"02VS3ZZ"  =  "1"  /* Restriction of Right Pulmonary Vein, Percutaneous Approach */
"02VS4CZ"  =  "1"  /* Restrict R Pulm Vein w Extralum Dev, Perc Endo */
"02VS4DZ"  =  "1"  /* Restrict R Pulm Vein w Intralum Dev, Perc Endo */
"02VS4ZZ"  =  "1"  /* Restriction of Right Pulmonary Vein, Perc Endo Approach */
"02VT0CZ"  =  "1"  /* Restriction of L Pulm Vein with Extralum Dev, Open Approach */
"02VT0DZ"  =  "1"  /* Restriction of L Pulm Vein with Intralum Dev, Open Approach */
"02VT0ZZ"  =  "1"  /* Restriction of Left Pulmonary Vein, Open Approach */
"02VT3CZ"  =  "1"  /* Restriction of L Pulm Vein with Extralum Dev, Perc Approach */
"02VT3DZ"  =  "1"  /* Restriction of L Pulm Vein with Intralum Dev, Perc Approach */
"02VT3ZZ"  =  "1"  /* Restriction of Left Pulmonary Vein, Percutaneous Approach */
"02VT4CZ"  =  "1"  /* Restrict L Pulm Vein w Extralum Dev, Perc Endo */
"02VT4DZ"  =  "1"  /* Restrict L Pulm Vein w Intralum Dev, Perc Endo */
"02VT4ZZ"  =  "1"  /* Restriction of Left Pulmonary Vein, Perc Endo Approach */
"02VV0CZ"  =  "1"  /* Restrict of Sup Vena Cava with Extralum Dev, Open Approach */
"02VV0DZ"  =  "1"  /* Restrict of Sup Vena Cava with Intralum Dev, Open Approach */
"02VV0ZZ"  =  "1"  /* Restriction of Superior Vena Cava, Open Approach */
"02VV3CZ"  =  "1"  /* Restrict of Sup Vena Cava with Extralum Dev, Perc Approach */
"02VV3DZ"  =  "1"  /* Restrict of Sup Vena Cava with Intralum Dev, Perc Approach */
"02VV3ZZ"  =  "1"  /* Restriction of Superior Vena Cava, Percutaneous Approach */
"02VV4CZ"  =  "1"  /* Restrict Sup Vena Cava w Extralum Dev, Perc Endo */
"02VV4DZ"  =  "1"  /* Restrict Sup Vena Cava w Intralum Dev, Perc Endo */
"02VV4ZZ"  =  "1"  /* Restriction of Superior Vena Cava, Perc Endo Approach */
"02VW0CZ"  =  "1"  /* Restriction of Thor Aorta with Extralum Dev, Open Approach */
"02VW0DZ"  =  "1"  /* Restriction of Thor Aorta with Intralum Dev, Open Approach */
"02VW0ZZ"  =  "1"  /* Restriction of Thoracic Aorta, Open Approach */
"02VW3CZ"  =  "1"  /* Restriction of Thor Aorta with Extralum Dev, Perc Approach */
"02VW3DZ"  =  "1"  /* Restriction of Thor Aorta with Intralum Dev, Perc Approach */
"02VW3ZZ"  =  "1"  /* Restriction of Thoracic Aorta, Percutaneous Approach */
"02VW4CZ"  =  "1"  /* Restrict of Thor Aorta with Extralum Dev, Perc Endo Approach */
"02VW4DZ"  =  "1"  /* Restrict of Thor Aorta with Intralum Dev, Perc Endo Approach */
"02VW4ZZ"  =  "1"  /* Restriction of Thoracic Aorta, Perc Endo Approach */
"02W50JZ"  =  "1"  /* Revision of Synth Sub in Atrial Sept, Open Approach */
"02W54JZ"  =  "1"  /* Revision of Synth Sub in Atrial Sept, Perc Endo Approach */
"02WA02Z"  =  "1"  /* Revision of Monitoring Device in Heart, Open Approach */
"02WA03Z"  =  "1"  /* Revision of Infusion Device in Heart, Open Approach */
"02WA07Z"  =  "1"  /* Revision of Autol Sub in Heart, Open Approach */
"02WA08Z"  =  "1"  /* Revision of Zooplastic Tissue in Heart, Open Approach */
"02WA0CZ"  =  "1"  /* Revision of Extraluminal Device in Heart, Open Approach */
"02WA0DZ"  =  "1"  /* Revision of Intraluminal Device in Heart, Open Approach */
"02WA0JZ"  =  "1"  /* Revision of Synthetic Substitute in Heart, Open Approach */
"02WA0KZ"  =  "1"  /* Revision of Nonaut Sub in Heart, Open Approach */
"02WA0MZ"  =  "1"  /* Revision of Cardiac Lead in Heart, Open Approach */
"02WA0QZ"  =  "1"  /* Revision of Implant Heart Assist in Heart, Open Approach */
"02WA0RZ"  =  "1"  /* Revision of Ext Heart Assist in Heart, Open Approach */
"02WA32Z"  =  "1"  /* Revision of Monitoring Device in Heart, Perc Approach */
"02WA33Z"  =  "1"  /* Revision of Infusion Device in Heart, Percutaneous Approach */
"02WA37Z"  =  "1"  /* Revision of Autol Sub in Heart, Perc Approach */
"02WA38Z"  =  "1"  /* Revision of Zooplastic Tissue in Heart, Perc Approach */
"02WA3CZ"  =  "1"  /* Revision of Extraluminal Device in Heart, Perc Approach */
"02WA3DZ"  =  "1"  /* Revision of Intraluminal Device in Heart, Perc Approach */
"02WA3JZ"  =  "1"  /* Revision of Synthetic Substitute in Heart, Perc Approach */
"02WA3KZ"  =  "1"  /* Revision of Nonaut Sub in Heart, Perc Approach */
"02WA3MZ"  =  "1"  /* Revision of Cardiac Lead in Heart, Percutaneous Approach */
"02WA3QZ"  =  "1"  /* Revision of Implant Heart Assist in Heart, Perc Approach */
"02WA3RZ"  =  "1"  /* Revision of Ext Heart Assist in Heart, Perc Approach */
"02WA42Z"  =  "1"  /* Revision of Monitoring Device in Heart, Perc Endo Approach */
"02WA43Z"  =  "1"  /* Revision of Infusion Device in Heart, Perc Endo Approach */
"02WA47Z"  =  "1"  /* Revision of Autol Sub in Heart, Perc Endo Approach */
"02WA48Z"  =  "1"  /* Revision of Zooplastic Tissue in Heart, Perc Endo Approach */
"02WA4CZ"  =  "1"  /* Revision of Extraluminal Device in Heart, Perc Endo Approach */
"02WA4DZ"  =  "1"  /* Revision of Intraluminal Device in Heart, Perc Endo Approach */
"02WA4JZ"  =  "1"  /* Revision of Synth Sub in Heart, Perc Endo Approach */
"02WA4KZ"  =  "1"  /* Revision of Nonaut Sub in Heart, Perc Endo Approach */
"02WA4MZ"  =  "1"  /* Revision of Cardiac Lead in Heart, Perc Endo Approach */
"02WA4QZ"  =  "1"  /* Revise of Implant Heart Assist in Heart, Perc Endo Approach */
"02WA4RZ"  =  "1"  /* Revision of Ext Heart Assist in Heart, Perc Endo Approach */
"02WF07Z"  =  "1"  /* Revision of Autol Sub in Aortic Valve, Open Approach */
"02WF08Z"  =  "1"  /* Revision of Zooplastic Tissue in Aortic Valve, Open Approach */
"02WF0JZ"  =  "1"  /* Revision of Synth Sub in Aortic Valve, Open Approach */
"02WF0KZ"  =  "1"  /* Revision of Nonaut Sub in Aortic Valve, Open Approach */
"02WF47Z"  =  "1"  /* Revision of Autol Sub in Aortic Valve, Perc Endo Approach */
"02WF48Z"  =  "1"  /* Revision of Zooplastic in Aortic Valve, Perc Endo Approach */
"02WF4JZ"  =  "1"  /* Revision of Synth Sub in Aortic Valve, Perc Endo Approach */
"02WF4KZ"  =  "1"  /* Revision of Nonaut Sub in Aortic Valve, Perc Endo Approach */
"02WG07Z"  =  "1"  /* Revision of Autol Sub in Mitral Valve, Open Approach */
"02WG08Z"  =  "1"  /* Revision of Zooplastic Tissue in Mitral Valve, Open Approach */
"02WG0JZ"  =  "1"  /* Revision of Synth Sub in Mitral Valve, Open Approach */
"02WG0KZ"  =  "1"  /* Revision of Nonaut Sub in Mitral Valve, Open Approach */
"02WG47Z"  =  "1"  /* Revision of Autol Sub in Mitral Valve, Perc Endo Approach */
"02WG48Z"  =  "1"  /* Revision of Zooplastic in Mitral Valve, Perc Endo Approach */
"02WG4JZ"  =  "1"  /* Revision of Synth Sub in Mitral Valve, Perc Endo Approach */
"02WG4KZ"  =  "1"  /* Revision of Nonaut Sub in Mitral Valve, Perc Endo Approach */
"02WH07Z"  =  "1"  /* Revision of Autol Sub in Pulm Valve, Open Approach */
"02WH08Z"  =  "1"  /* Revision of Zooplastic Tissue in Pulm Valve, Open Approach */
"02WH0JZ"  =  "1"  /* Revision of Synth Sub in Pulm Valve, Open Approach */
"02WH0KZ"  =  "1"  /* Revision of Nonaut Sub in Pulm Valve, Open Approach */
"02WH47Z"  =  "1"  /* Revision of Autol Sub in Pulm Valve, Perc Endo Approach */
"02WH48Z"  =  "1"  /* Revision of Zooplastic in Pulm Valve, Perc Endo Approach */
"02WH4JZ"  =  "1"  /* Revision of Synth Sub in Pulm Valve, Perc Endo Approach */
"02WH4KZ"  =  "1"  /* Revision of Nonaut Sub in Pulm Valve, Perc Endo Approach */
"02WJ07Z"  =  "1"  /* Revision of Autol Sub in Tricusp Valve, Open Approach */
"02WJ08Z"  =  "1"  /* Revision of Zooplastic in Tricusp Valve, Open Approach */
"02WJ0JZ"  =  "1"  /* Revision of Synth Sub in Tricusp Valve, Open Approach */
"02WJ0KZ"  =  "1"  /* Revision of Nonaut Sub in Tricusp Valve, Open Approach */
"02WJ47Z"  =  "1"  /* Revision of Autol Sub in Tricusp Valve, Perc Endo Approach */
"02WJ48Z"  =  "1"  /* Revision of Zooplastic in Tricusp Valve, Perc Endo Approach */
"02WJ4JZ"  =  "1"  /* Revision of Synth Sub in Tricusp Valve, Perc Endo Approach */
"02WJ4KZ"  =  "1"  /* Revision of Nonaut Sub in Tricusp Valve, Perc Endo Approach */
"02WM0JZ"  =  "1"  /* Revision of Synth Sub in Ventricular Sept, Open Approach */
"02WM4JZ"  =  "1"  /* Revise of Synth Sub in Ventricular Sept, Perc Endo Approach */
"02WY02Z"  =  "1"  /* Revision of Monitoring Device in Great Vessel, Open Approach */
"02WY03Z"  =  "1"  /* Revision of Infusion Device in Great Vessel, Open Approach */
"02WY07Z"  =  "1"  /* Revision of Autol Sub in Great Vessel, Open Approach */
"02WY08Z"  =  "1"  /* Revision of Zooplastic Tissue in Great Vessel, Open Approach */
"02WY0CZ"  =  "1"  /* Revision of Extralum Dev in Great Vessel, Open Approach */
"02WY0DZ"  =  "1"  /* Revision of Intralum Dev in Great Vessel, Open Approach */
"02WY0JZ"  =  "1"  /* Revision of Synth Sub in Great Vessel, Open Approach */
"02WY0KZ"  =  "1"  /* Revision of Nonaut Sub in Great Vessel, Open Approach */
"02WY32Z"  =  "1"  /* Revision of Monitoring Device in Great Vessel, Perc Approach */
"02WY33Z"  =  "1"  /* Revision of Infusion Device in Great Vessel, Perc Approach */
"02WY37Z"  =  "1"  /* Revision of Autol Sub in Great Vessel, Perc Approach */
"02WY38Z"  =  "1"  /* Revision of Zooplastic Tissue in Great Vessel, Perc Approach */
"02WY3CZ"  =  "1"  /* Revision of Extralum Dev in Great Vessel, Perc Approach */
"02WY3DZ"  =  "1"  /* Revision of Intralum Dev in Great Vessel, Perc Approach */
"02WY3JZ"  =  "1"  /* Revision of Synth Sub in Great Vessel, Perc Approach */
"02WY3KZ"  =  "1"  /* Revision of Nonaut Sub in Great Vessel, Perc Approach */
"02WY42Z"  =  "1"  /* Revision of Monitor Dev in Great Vessel, Perc Endo Approach */
"02WY43Z"  =  "1"  /* Revision of Infusion Dev in Great Vessel, Perc Endo Approach */
"02WY47Z"  =  "1"  /* Revision of Autol Sub in Great Vessel, Perc Endo Approach */
"02WY48Z"  =  "1"  /* Revision of Zooplastic in Great Vessel, Perc Endo Approach */
"02WY4CZ"  =  "1"  /* Revision of Extralum Dev in Great Vessel, Perc Endo Approach */
"02WY4DZ"  =  "1"  /* Revision of Intralum Dev in Great Vessel, Perc Endo Approach */
"02WY4JZ"  =  "1"  /* Revision of Synth Sub in Great Vessel, Perc Endo Approach */
"02WY4KZ"  =  "1"  /* Revision of Nonaut Sub in Great Vessel, Perc Endo Approach */
"02YA0Z0"  =  "1"  /* Transplantation of Heart, Allogeneic, Open Approach */
"02YA0Z1"  =  "1"  /* Transplantation of Heart, Syngeneic, Open Approach */
"02YA0Z2"  =  "1"  /* Transplantation of Heart, Zooplastic, Open Approach */
"0312090"  =  "1"  /* Bypass Innom Art to R Up Arm Art w Autol Vn, Open */
"0312091"  =  "1"  /* Bypass Innom Art to L Up Arm Art w Autol Vn, Open */
"0312092"  =  "1"  /* Bypass Innom Art to Bi Up Arm Art w Autol Vn, Open */
"0312093"  =  "1"  /* Bypass Innom Art to R Low Arm Art w Autol Vn, Open */
"0312094"  =  "1"  /* Bypass Innom Art to L Low Arm Art w Autol Vn, Open */
"0312095"  =  "1"  /* Bypass Innom Art to Bi Low Arm Art w Autol Vn, Open */
"0312096"  =  "1"  /* Bypass Innom Art to R Up Leg Art w Autol Vn, Open */
"0312097"  =  "1"  /* Bypass Innom Art to L Up Leg Art w Autol Vn, Open */
"0312098"  =  "1"  /* Bypass Innom Art to Bi Up Leg Art w Autol Vn, Open */
"0312099"  =  "1"  /* Bypass Innom Art to R Low Leg Art w Autol Vn, Open */
"031209B"  =  "1"  /* Bypass Innom Art to L Low Leg Art w Autol Vn, Open */
"031209C"  =  "1"  /* Bypass Innom Art to Bi Low Leg Art w Autol Vn, Open */
"031209D"  =  "1"  /* Bypass Innom Art to Up Arm Vein with Autol Vn, Open Approach */
"031209F"  =  "1"  /* Bypass Innom Art to Low Arm Vein w Autol Vn, Open */
"031209J"  =  "1"  /* Bypass Innom Art to R Extracran Art w Autol Vn, Open */
"031209K"  =  "1"  /* Bypass Innom Art to L Extracran Art w Autol Vn, Open */
"03120A0"  =  "1"  /* Bypass Innom Art to R Up Arm Art w Autol Art, Open */
"03120A1"  =  "1"  /* Bypass Innom Art to L Up Arm Art w Autol Art, Open */
"03120A2"  =  "1"  /* Bypass Innom Art to Bi Up Arm Art w Autol Art, Open */
"03120A3"  =  "1"  /* Bypass Innom Art to R Low Arm Art w Autol Art, Open */
"03120A4"  =  "1"  /* Bypass Innom Art to L Low Arm Art w Autol Art, Open */
"03120A5"  =  "1"  /* Bypass Innom Art to Bi Low Arm Art w Autol Art, Open */
"03120A6"  =  "1"  /* Bypass Innom Art to R Up Leg Art w Autol Art, Open */
"03120A7"  =  "1"  /* Bypass Innom Art to L Up Leg Art w Autol Art, Open */
"03120A8"  =  "1"  /* Bypass Innom Art to Bi Up Leg Art w Autol Art, Open */
"03120A9"  =  "1"  /* Bypass Innom Art to R Low Leg Art w Autol Art, Open */
"03120AB"  =  "1"  /* Bypass Innom Art to L Low Leg Art w Autol Art, Open */
"03120AC"  =  "1"  /* Bypass Innom Art to Bi Low Leg Art w Autol Art, Open */
"03120AD"  =  "1"  /* Bypass Innom Art to Up Arm Vein w Autol Art, Open */
"03120AF"  =  "1"  /* Bypass Innom Art to Low Arm Vein w Autol Art, Open */
"03120AJ"  =  "1"  /* Bypass Innom Art to R Extracran Art w Autol Art, Open */
"03120AK"  =  "1"  /* Bypass Innom Art to L Extracran Art w Autol Art, Open */
"03120J0"  =  "1"  /* Bypass Innom Art to R Up Arm Art w Synth Sub, Open */
"03120J1"  =  "1"  /* Bypass Innom Art to L Up Arm Art w Synth Sub, Open */
"03120J2"  =  "1"  /* Bypass Innom Art to Bi Up Arm Art w Synth Sub, Open */
"03120J3"  =  "1"  /* Bypass Innom Art to R Low Arm Art w Synth Sub, Open */
"03120J4"  =  "1"  /* Bypass Innom Art to L Low Arm Art w Synth Sub, Open */
"03120J5"  =  "1"  /* Bypass Innom Art to Bi Low Arm Art w Synth Sub, Open */
"03120J6"  =  "1"  /* Bypass Innom Art to R Up Leg Art w Synth Sub, Open */
"03120J7"  =  "1"  /* Bypass Innom Art to L Up Leg Art w Synth Sub, Open */
"03120J8"  =  "1"  /* Bypass Innom Art to Bi Up Leg Art w Synth Sub, Open */
"03120J9"  =  "1"  /* Bypass Innom Art to R Low Leg Art w Synth Sub, Open */
"03120JB"  =  "1"  /* Bypass Innom Art to L Low Leg Art w Synth Sub, Open */
"03120JC"  =  "1"  /* Bypass Innom Art to Bi Low Leg Art w Synth Sub, Open */
"03120JD"  =  "1"  /* Bypass Innom Art to Up Arm Vein w Synth Sub, Open */
"03120JF"  =  "1"  /* Bypass Innom Art to Low Arm Vein w Synth Sub, Open */
"03120JJ"  =  "1"  /* Bypass Innom Art to R Extracran Art w Synth Sub, Open */
"03120JK"  =  "1"  /* Bypass Innom Art to L Extracran Art w Synth Sub, Open */
"03120K0"  =  "1"  /* Bypass Innom Art to R Up Arm Art w Nonaut Sub, Open */
"03120K1"  =  "1"  /* Bypass Innom Art to L Up Arm Art w Nonaut Sub, Open */
"03120K2"  =  "1"  /* Bypass Innom Art to Bi Up Arm Art w Nonaut Sub, Open */
"03120K3"  =  "1"  /* Bypass Innom Art to R Low Arm Art w Nonaut Sub, Open */
"03120K4"  =  "1"  /* Bypass Innom Art to L Low Arm Art w Nonaut Sub, Open */
"03120K5"  =  "1"  /* Bypass Innom Art to Bi Low Arm Art w Nonaut Sub, Open */
"03120K6"  =  "1"  /* Bypass Innom Art to R Up Leg Art w Nonaut Sub, Open */
"03120K7"  =  "1"  /* Bypass Innom Art to L Up Leg Art w Nonaut Sub, Open */
"03120K8"  =  "1"  /* Bypass Innom Art to Bi Up Leg Art w Nonaut Sub, Open */
"03120K9"  =  "1"  /* Bypass Innom Art to R Low Leg Art w Nonaut Sub, Open */
"03120KB"  =  "1"  /* Bypass Innom Art to L Low Leg Art w Nonaut Sub, Open */
"03120KC"  =  "1"  /* Bypass Innom Art to Bi Low Leg Art w Nonaut Sub, Open */
"03120KD"  =  "1"  /* Bypass Innom Art to Up Arm Vein w Nonaut Sub, Open */
"03120KF"  =  "1"  /* Bypass Innom Art to Low Arm Vein w Nonaut Sub, Open */
"03120KJ"  =  "1"  /* Bypass Innom Art to R Extracran Art w Nonaut Sub, Open */
"03120KK"  =  "1"  /* Bypass Innom Art to L Extracran Art w Nonaut Sub, Open */
"03120Z0"  =  "1"  /* Bypass Innominate Artery to R Up Arm Art, Open Approach */
"03120Z1"  =  "1"  /* Bypass Innominate Artery to L Up Arm Art, Open Approach */
"03120Z2"  =  "1"  /* Bypass Innominate Artery to Bi Up Arm Art, Open Approach */
"03120Z3"  =  "1"  /* Bypass Innominate Artery to R Low Arm Art, Open Approach */
"03120Z4"  =  "1"  /* Bypass Innominate Artery to L Low Arm Art, Open Approach */
"03120Z5"  =  "1"  /* Bypass Innominate Artery to Bi Low Arm Art, Open Approach */
"03120Z6"  =  "1"  /* Bypass Innominate Artery to R Up Leg Art, Open Approach */
"03120Z7"  =  "1"  /* Bypass Innominate Artery to L Up Leg Art, Open Approach */
"03120Z8"  =  "1"  /* Bypass Innominate Artery to Bi Up Leg Art, Open Approach */
"03120Z9"  =  "1"  /* Bypass Innominate Artery to R Low Leg Art, Open Approach */
"03120ZB"  =  "1"  /* Bypass Innominate Artery to L Low Leg Art, Open Approach */
"03120ZC"  =  "1"  /* Bypass Innominate Artery to Bi Low Leg Art, Open Approach */
"03120ZD"  =  "1"  /* Bypass Innominate Artery to Upper Arm Vein, Open Approach */
"03120ZF"  =  "1"  /* Bypass Innominate Artery to Lower Arm Vein, Open Approach */
"03120ZJ"  =  "1"  /* Bypass Innominate Artery to R Extracran Art, Open Approach */
"03120ZK"  =  "1"  /* Bypass Innominate Artery to L Extracran Art, Open Approach */
"0313090"  =  "1"  /* Bypass R Subclav Art to R Up Arm Art w Autol Vn, Open */
"0313091"  =  "1"  /* Bypass R Subclav Art to L Up Arm Art w Autol Vn, Open */
"0313092"  =  "1"  /* Bypass R Subclav Art to Bi Up Arm Art w Autol Vn, Open */
"0313093"  =  "1"  /* Bypass R Subclav Art to R Low Arm Art w Autol Vn, Open */
"0313094"  =  "1"  /* Bypass R Subclav Art to L Low Arm Art w Autol Vn, Open */
"0313095"  =  "1"  /* Bypass R Subclav Art to Bi Low Arm Art w Autol Vn, Open */
"0313096"  =  "1"  /* Bypass R Subclav Art to R Up Leg Art w Autol Vn, Open */
"0313097"  =  "1"  /* Bypass R Subclav Art to L Up Leg Art w Autol Vn, Open */
"0313098"  =  "1"  /* Bypass R Subclav Art to Bi Up Leg Art w Autol Vn, Open */
"0313099"  =  "1"  /* Bypass R Subclav Art to R Low Leg Art w Autol Vn, Open */
"031309B"  =  "1"  /* Bypass R Subclav Art to L Low Leg Art w Autol Vn, Open */
"031309C"  =  "1"  /* Bypass R Subclav Art to Bi Low Leg Art w Autol Vn, Open */
"031309D"  =  "1"  /* Bypass R Subclav Art to Up Arm Vein w Autol Vn, Open */
"031309F"  =  "1"  /* Bypass R Subclav Art to Low Arm Vein w Autol Vn, Open */
"031309J"  =  "1"  /* Bypass R Subclav Art to R Extracran Art w Autol Vn, Open */
"031309K"  =  "1"  /* Bypass R Subclav Art to L Extracran Art w Autol Vn, Open */
"031309M"  =  "1"  /* Bypass R Subclav Art to R Pulm Art w Autol Vn, Open */
"031309N"  =  "1"  /* Bypass R Subclav Art to L Pulm Art w Autol Vn, Open */
"03130A0"  =  "1"  /* Bypass R Subclav Art to R Up Arm Art w Autol Art, Open */
"03130A1"  =  "1"  /* Bypass R Subclav Art to L Up Arm Art w Autol Art, Open */
"03130A2"  =  "1"  /* Bypass R Subclav Art to Bi Up Arm Art w Autol Art, Open */
"03130A3"  =  "1"  /* Bypass R Subclav Art to R Low Arm Art w Autol Art, Open */
"03130A4"  =  "1"  /* Bypass R Subclav Art to L Low Arm Art w Autol Art, Open */
"03130A5"  =  "1"  /* Bypass R Subclav Art to Bi Low Arm Art w Autol Art, Open */
"03130A6"  =  "1"  /* Bypass R Subclav Art to R Up Leg Art w Autol Art, Open */
"03130A7"  =  "1"  /* Bypass R Subclav Art to L Up Leg Art w Autol Art, Open */
"03130A8"  =  "1"  /* Bypass R Subclav Art to Bi Up Leg Art w Autol Art, Open */
"03130A9"  =  "1"  /* Bypass R Subclav Art to R Low Leg Art w Autol Art, Open */
"03130AB"  =  "1"  /* Bypass R Subclav Art to L Low Leg Art w Autol Art, Open */
"03130AC"  =  "1"  /* Bypass R Subclav Art to Bi Low Leg Art w Autol Art, Open */
"03130AD"  =  "1"  /* Bypass R Subclav Art to Up Arm Vein w Autol Art, Open */
"03130AF"  =  "1"  /* Bypass R Subclav Art to Low Arm Vein w Autol Art, Open */
"03130AJ"  =  "1"  /* Bypass R Subclav Art to R Extracran Art w Autol Art, Open */
"03130AK"  =  "1"  /* Bypass R Subclav Art to L Extracran Art w Autol Art, Open */
"03130AM"  =  "1"  /* Bypass R Subclav Art to R Pulm Art w Autol Art, Open */
"03130AN"  =  "1"  /* Bypass R Subclav Art to L Pulm Art w Autol Art, Open */
"03130J0"  =  "1"  /* Bypass R Subclav Art to R Up Arm Art w Synth Sub, Open */
"03130J1"  =  "1"  /* Bypass R Subclav Art to L Up Arm Art w Synth Sub, Open */
"03130J2"  =  "1"  /* Bypass R Subclav Art to Bi Up Arm Art w Synth Sub, Open */
"03130J3"  =  "1"  /* Bypass R Subclav Art to R Low Arm Art w Synth Sub, Open */
"03130J4"  =  "1"  /* Bypass R Subclav Art to L Low Arm Art w Synth Sub, Open */
"03130J5"  =  "1"  /* Bypass R Subclav Art to Bi Low Arm Art w Synth Sub, Open */
"03130J6"  =  "1"  /* Bypass R Subclav Art to R Up Leg Art w Synth Sub, Open */
"03130J7"  =  "1"  /* Bypass R Subclav Art to L Up Leg Art w Synth Sub, Open */
"03130J8"  =  "1"  /* Bypass R Subclav Art to Bi Up Leg Art w Synth Sub, Open */
"03130J9"  =  "1"  /* Bypass R Subclav Art to R Low Leg Art w Synth Sub, Open */
"03130JB"  =  "1"  /* Bypass R Subclav Art to L Low Leg Art w Synth Sub, Open */
"03130JC"  =  "1"  /* Bypass R Subclav Art to Bi Low Leg Art w Synth Sub, Open */
"03130JD"  =  "1"  /* Bypass R Subclav Art to Up Arm Vein w Synth Sub, Open */
"03130JF"  =  "1"  /* Bypass R Subclav Art to Low Arm Vein w Synth Sub, Open */
"03130JJ"  =  "1"  /* Bypass R Subclav Art to R Extracran Art w Synth Sub, Open */
"03130JK"  =  "1"  /* Bypass R Subclav Art to L Extracran Art w Synth Sub, Open */
"03130JM"  =  "1"  /* Bypass R Subclav Art to R Pulm Art w Synth Sub, Open */
"03130JN"  =  "1"  /* Bypass R Subclav Art to L Pulm Art w Synth Sub, Open */
"03130K0"  =  "1"  /* Bypass R Subclav Art to R Up Arm Art w Nonaut Sub, Open */
"03130K1"  =  "1"  /* Bypass R Subclav Art to L Up Arm Art w Nonaut Sub, Open */
"03130K2"  =  "1"  /* Bypass R Subclav Art to Bi Up Arm Art w Nonaut Sub, Open */
"03130K3"  =  "1"  /* Bypass R Subclav Art to R Low Arm Art w Nonaut Sub, Open */
"03130K4"  =  "1"  /* Bypass R Subclav Art to L Low Arm Art w Nonaut Sub, Open */
"03130K5"  =  "1"  /* Bypass R Subclav Art to Bi Low Arm Art w Nonaut Sub, Open */
"03130K6"  =  "1"  /* Bypass R Subclav Art to R Up Leg Art w Nonaut Sub, Open */
"03130K7"  =  "1"  /* Bypass R Subclav Art to L Up Leg Art w Nonaut Sub, Open */
"03130K8"  =  "1"  /* Bypass R Subclav Art to Bi Up Leg Art w Nonaut Sub, Open */
"03130K9"  =  "1"  /* Bypass R Subclav Art to R Low Leg Art w Nonaut Sub, Open */
"03130KB"  =  "1"  /* Bypass R Subclav Art to L Low Leg Art w Nonaut Sub, Open */
"03130KC"  =  "1"  /* Bypass R Subclav Art to Bi Low Leg Art w Nonaut Sub, Open */
"03130KD"  =  "1"  /* Bypass R Subclav Art to Up Arm Vein w Nonaut Sub, Open */
"03130KF"  =  "1"  /* Bypass R Subclav Art to Low Arm Vein w Nonaut Sub, Open */
"03130KJ"  =  "1"  /* Bypass R Subclav Art to R Extracran Art w Nonaut Sub, Open */
"03130KK"  =  "1"  /* Bypass R Subclav Art to L Extracran Art w Nonaut Sub, Open */
"03130KM"  =  "1"  /* Bypass R Subclav Art to R Pulm Art w Nonaut Sub, Open */
"03130KN"  =  "1"  /* Bypass R Subclav Art to L Pulm Art w Nonaut Sub, Open */
"03130Z0"  =  "1"  /* Bypass R Subclav Art to R Up Arm Art, Open Approach */
"03130Z1"  =  "1"  /* Bypass R Subclav Art to L Up Arm Art, Open Approach */
"03130Z2"  =  "1"  /* Bypass R Subclav Art to Bi Up Arm Art, Open Approach */
"03130Z3"  =  "1"  /* Bypass R Subclav Art to R Low Arm Art, Open Approach */
"03130Z4"  =  "1"  /* Bypass R Subclav Art to L Low Arm Art, Open Approach */
"03130Z5"  =  "1"  /* Bypass R Subclav Art to Bi Low Arm Art, Open Approach */
"03130Z6"  =  "1"  /* Bypass R Subclav Art to R Up Leg Art, Open Approach */
"03130Z7"  =  "1"  /* Bypass R Subclav Art to L Up Leg Art, Open Approach */
"03130Z8"  =  "1"  /* Bypass R Subclav Art to Bi Up Leg Art, Open Approach */
"03130Z9"  =  "1"  /* Bypass R Subclav Art to R Low Leg Art, Open Approach */
"03130ZB"  =  "1"  /* Bypass R Subclav Art to L Low Leg Art, Open Approach */
"03130ZC"  =  "1"  /* Bypass R Subclav Art to Bi Low Leg Art, Open Approach */
"03130ZD"  =  "1"  /* Bypass Right Subclavian Artery to Up Arm Vein, Open Approach */
"03130ZF"  =  "1"  /* Bypass R Subclav Art to Low Arm Vein, Open Approach */
"03130ZJ"  =  "1"  /* Bypass R Subclav Art to R Extracran Art, Open Approach */
"03130ZK"  =  "1"  /* Bypass R Subclav Art to L Extracran Art, Open Approach */
"03130ZM"  =  "1"  /* Bypass Right Subclavian Artery to R Pulm Art, Open Approach */
"03130ZN"  =  "1"  /* Bypass Right Subclavian Artery to L Pulm Art, Open Approach */
"0314090"  =  "1"  /* Bypass L Subclav Art to R Up Arm Art w Autol Vn, Open */
"0314091"  =  "1"  /* Bypass L Subclav Art to L Up Arm Art w Autol Vn, Open */
"0314092"  =  "1"  /* Bypass L Subclav Art to Bi Up Arm Art w Autol Vn, Open */
"0314093"  =  "1"  /* Bypass L Subclav Art to R Low Arm Art w Autol Vn, Open */
"0314094"  =  "1"  /* Bypass L Subclav Art to L Low Arm Art w Autol Vn, Open */
"0314095"  =  "1"  /* Bypass L Subclav Art to Bi Low Arm Art w Autol Vn, Open */
"0314096"  =  "1"  /* Bypass L Subclav Art to R Up Leg Art w Autol Vn, Open */
"0314097"  =  "1"  /* Bypass L Subclav Art to L Up Leg Art w Autol Vn, Open */
"0314098"  =  "1"  /* Bypass L Subclav Art to Bi Up Leg Art w Autol Vn, Open */
"0314099"  =  "1"  /* Bypass L Subclav Art to R Low Leg Art w Autol Vn, Open */
"031409B"  =  "1"  /* Bypass L Subclav Art to L Low Leg Art w Autol Vn, Open */
"031409C"  =  "1"  /* Bypass L Subclav Art to Bi Low Leg Art w Autol Vn, Open */
"031409D"  =  "1"  /* Bypass L Subclav Art to Up Arm Vein w Autol Vn, Open */
"031409F"  =  "1"  /* Bypass L Subclav Art to Low Arm Vein w Autol Vn, Open */
"031409J"  =  "1"  /* Bypass L Subclav Art to R Extracran Art w Autol Vn, Open */
"031409K"  =  "1"  /* Bypass L Subclav Art to L Extracran Art w Autol Vn, Open */
"031409M"  =  "1"  /* Bypass L Subclav Art to R Pulm Art w Autol Vn, Open */
"031409N"  =  "1"  /* Bypass L Subclav Art to L Pulm Art w Autol Vn, Open */
"03140A0"  =  "1"  /* Bypass L Subclav Art to R Up Arm Art w Autol Art, Open */
"03140A1"  =  "1"  /* Bypass L Subclav Art to L Up Arm Art w Autol Art, Open */
"03140A2"  =  "1"  /* Bypass L Subclav Art to Bi Up Arm Art w Autol Art, Open */
"03140A3"  =  "1"  /* Bypass L Subclav Art to R Low Arm Art w Autol Art, Open */
"03140A4"  =  "1"  /* Bypass L Subclav Art to L Low Arm Art w Autol Art, Open */
"03140A5"  =  "1"  /* Bypass L Subclav Art to Bi Low Arm Art w Autol Art, Open */
"03140A6"  =  "1"  /* Bypass L Subclav Art to R Up Leg Art w Autol Art, Open */
"03140A7"  =  "1"  /* Bypass L Subclav Art to L Up Leg Art w Autol Art, Open */
"03140A8"  =  "1"  /* Bypass L Subclav Art to Bi Up Leg Art w Autol Art, Open */
"03140A9"  =  "1"  /* Bypass L Subclav Art to R Low Leg Art w Autol Art, Open */
"03140AB"  =  "1"  /* Bypass L Subclav Art to L Low Leg Art w Autol Art, Open */
"03140AC"  =  "1"  /* Bypass L Subclav Art to Bi Low Leg Art w Autol Art, Open */
"03140AD"  =  "1"  /* Bypass L Subclav Art to Up Arm Vein w Autol Art, Open */
"03140AF"  =  "1"  /* Bypass L Subclav Art to Low Arm Vein w Autol Art, Open */
"03140AJ"  =  "1"  /* Bypass L Subclav Art to R Extracran Art w Autol Art, Open */
"03140AK"  =  "1"  /* Bypass L Subclav Art to L Extracran Art w Autol Art, Open */
"03140AM"  =  "1"  /* Bypass L Subclav Art to R Pulm Art w Autol Art, Open */
"03140AN"  =  "1"  /* Bypass L Subclav Art to L Pulm Art w Autol Art, Open */
"03140J0"  =  "1"  /* Bypass L Subclav Art to R Up Arm Art w Synth Sub, Open */
"03140J1"  =  "1"  /* Bypass L Subclav Art to L Up Arm Art w Synth Sub, Open */
"03140J2"  =  "1"  /* Bypass L Subclav Art to Bi Up Arm Art w Synth Sub, Open */
"03140J3"  =  "1"  /* Bypass L Subclav Art to R Low Arm Art w Synth Sub, Open */
"03140J4"  =  "1"  /* Bypass L Subclav Art to L Low Arm Art w Synth Sub, Open */
"03140J5"  =  "1"  /* Bypass L Subclav Art to Bi Low Arm Art w Synth Sub, Open */
"03140J6"  =  "1"  /* Bypass L Subclav Art to R Up Leg Art w Synth Sub, Open */
"03140J7"  =  "1"  /* Bypass L Subclav Art to L Up Leg Art w Synth Sub, Open */
"03140J8"  =  "1"  /* Bypass L Subclav Art to Bi Up Leg Art w Synth Sub, Open */
"03140J9"  =  "1"  /* Bypass L Subclav Art to R Low Leg Art w Synth Sub, Open */
"03140JB"  =  "1"  /* Bypass L Subclav Art to L Low Leg Art w Synth Sub, Open */
"03140JC"  =  "1"  /* Bypass L Subclav Art to Bi Low Leg Art w Synth Sub, Open */
"03140JD"  =  "1"  /* Bypass L Subclav Art to Up Arm Vein w Synth Sub, Open */
"03140JF"  =  "1"  /* Bypass L Subclav Art to Low Arm Vein w Synth Sub, Open */
"03140JJ"  =  "1"  /* Bypass L Subclav Art to R Extracran Art w Synth Sub, Open */
"03140JK"  =  "1"  /* Bypass L Subclav Art to L Extracran Art w Synth Sub, Open */
"03140JM"  =  "1"  /* Bypass L Subclav Art to R Pulm Art w Synth Sub, Open */
"03140JN"  =  "1"  /* Bypass L Subclav Art to L Pulm Art w Synth Sub, Open */
"03140K0"  =  "1"  /* Bypass L Subclav Art to R Up Arm Art w Nonaut Sub, Open */
"03140K1"  =  "1"  /* Bypass L Subclav Art to L Up Arm Art w Nonaut Sub, Open */
"03140K2"  =  "1"  /* Bypass L Subclav Art to Bi Up Arm Art w Nonaut Sub, Open */
"03140K3"  =  "1"  /* Bypass L Subclav Art to R Low Arm Art w Nonaut Sub, Open */
"03140K4"  =  "1"  /* Bypass L Subclav Art to L Low Arm Art w Nonaut Sub, Open */
"03140K5"  =  "1"  /* Bypass L Subclav Art to Bi Low Arm Art w Nonaut Sub, Open */
"03140K6"  =  "1"  /* Bypass L Subclav Art to R Up Leg Art w Nonaut Sub, Open */
"03140K7"  =  "1"  /* Bypass L Subclav Art to L Up Leg Art w Nonaut Sub, Open */
"03140K8"  =  "1"  /* Bypass L Subclav Art to Bi Up Leg Art w Nonaut Sub, Open */
"03140K9"  =  "1"  /* Bypass L Subclav Art to R Low Leg Art w Nonaut Sub, Open */
"03140KB"  =  "1"  /* Bypass L Subclav Art to L Low Leg Art w Nonaut Sub, Open */
"03140KC"  =  "1"  /* Bypass L Subclav Art to Bi Low Leg Art w Nonaut Sub, Open */
"03140KD"  =  "1"  /* Bypass L Subclav Art to Up Arm Vein w Nonaut Sub, Open */
"03140KF"  =  "1"  /* Bypass L Subclav Art to Low Arm Vein w Nonaut Sub, Open */
"03140KJ"  =  "1"  /* Bypass L Subclav Art to R Extracran Art w Nonaut Sub, Open */
"03140KK"  =  "1"  /* Bypass L Subclav Art to L Extracran Art w Nonaut Sub, Open */
"03140KM"  =  "1"  /* Bypass L Subclav Art to R Pulm Art w Nonaut Sub, Open */
"03140KN"  =  "1"  /* Bypass L Subclav Art to L Pulm Art w Nonaut Sub, Open */
"03140Z0"  =  "1"  /* Bypass Left Subclavian Artery to R Up Arm Art, Open Approach */
"03140Z1"  =  "1"  /* Bypass Left Subclavian Artery to L Up Arm Art, Open Approach */
"03140Z2"  =  "1"  /* Bypass L Subclav Art to Bi Up Arm Art, Open Approach */
"03140Z3"  =  "1"  /* Bypass L Subclav Art to R Low Arm Art, Open Approach */
"03140Z4"  =  "1"  /* Bypass L Subclav Art to L Low Arm Art, Open Approach */
"03140Z5"  =  "1"  /* Bypass L Subclav Art to Bi Low Arm Art, Open Approach */
"03140Z6"  =  "1"  /* Bypass Left Subclavian Artery to R Up Leg Art, Open Approach */
"03140Z7"  =  "1"  /* Bypass Left Subclavian Artery to L Up Leg Art, Open Approach */
"03140Z8"  =  "1"  /* Bypass L Subclav Art to Bi Up Leg Art, Open Approach */
"03140Z9"  =  "1"  /* Bypass L Subclav Art to R Low Leg Art, Open Approach */
"03140ZB"  =  "1"  /* Bypass L Subclav Art to L Low Leg Art, Open Approach */
"03140ZC"  =  "1"  /* Bypass L Subclav Art to Bi Low Leg Art, Open Approach */
"03140ZD"  =  "1"  /* Bypass Left Subclavian Artery to Up Arm Vein, Open Approach */
"03140ZF"  =  "1"  /* Bypass Left Subclavian Artery to Low Arm Vein, Open Approach */
"03140ZJ"  =  "1"  /* Bypass L Subclav Art to R Extracran Art, Open Approach */
"03140ZK"  =  "1"  /* Bypass L Subclav Art to L Extracran Art, Open Approach */
"03140ZM"  =  "1"  /* Bypass Left Subclavian Artery to R Pulm Art, Open Approach */
"03140ZN"  =  "1"  /* Bypass Left Subclavian Artery to L Pulm Art, Open Approach */
"0315090"  =  "1"  /* Bypass R Axilla Art to R Up Arm Art w Autol Vn, Open */
"0315091"  =  "1"  /* Bypass R Axilla Art to L Up Arm Art w Autol Vn, Open */
"0315092"  =  "1"  /* Bypass R Axilla Art to Bi Up Arm Art w Autol Vn, Open */
"0315093"  =  "1"  /* Bypass R Axilla Art to R Low Arm Art w Autol Vn, Open */
"0315094"  =  "1"  /* Bypass R Axilla Art to L Low Arm Art w Autol Vn, Open */
"0315095"  =  "1"  /* Bypass R Axilla Art to Bi Low Arm Art w Autol Vn, Open */
"0315096"  =  "1"  /* Bypass R Axilla Art to R Up Leg Art w Autol Vn, Open */
"0315097"  =  "1"  /* Bypass R Axilla Art to L Up Leg Art w Autol Vn, Open */
"0315098"  =  "1"  /* Bypass R Axilla Art to Bi Up Leg Art w Autol Vn, Open */
"0315099"  =  "1"  /* Bypass R Axilla Art to R Low Leg Art w Autol Vn, Open */
"031509B"  =  "1"  /* Bypass R Axilla Art to L Low Leg Art w Autol Vn, Open */
"031509C"  =  "1"  /* Bypass R Axilla Art to Bi Low Leg Art w Autol Vn, Open */
"031509D"  =  "1"  /* Bypass R Axilla Art to Up Arm Vein w Autol Vn, Open */
"031509F"  =  "1"  /* Bypass R Axilla Art to Low Arm Vein w Autol Vn, Open */
"031509J"  =  "1"  /* Bypass R Axilla Art to R Extracran Art w Autol Vn, Open */
"031509K"  =  "1"  /* Bypass R Axilla Art to L Extracran Art w Autol Vn, Open */
"03150A0"  =  "1"  /* Bypass R Axilla Art to R Up Arm Art w Autol Art, Open */
"03150A1"  =  "1"  /* Bypass R Axilla Art to L Up Arm Art w Autol Art, Open */
"03150A2"  =  "1"  /* Bypass R Axilla Art to Bi Up Arm Art w Autol Art, Open */
"03150A3"  =  "1"  /* Bypass R Axilla Art to R Low Arm Art w Autol Art, Open */
"03150A4"  =  "1"  /* Bypass R Axilla Art to L Low Arm Art w Autol Art, Open */
"03150A5"  =  "1"  /* Bypass R Axilla Art to Bi Low Arm Art w Autol Art, Open */
"03150A6"  =  "1"  /* Bypass R Axilla Art to R Up Leg Art w Autol Art, Open */
"03150A7"  =  "1"  /* Bypass R Axilla Art to L Up Leg Art w Autol Art, Open */
"03150A8"  =  "1"  /* Bypass R Axilla Art to Bi Up Leg Art w Autol Art, Open */
"03150A9"  =  "1"  /* Bypass R Axilla Art to R Low Leg Art w Autol Art, Open */
"03150AB"  =  "1"  /* Bypass R Axilla Art to L Low Leg Art w Autol Art, Open */
"03150AC"  =  "1"  /* Bypass R Axilla Art to Bi Low Leg Art w Autol Art, Open */
"03150AD"  =  "1"  /* Bypass R Axilla Art to Up Arm Vein w Autol Art, Open */
"03150AF"  =  "1"  /* Bypass R Axilla Art to Low Arm Vein w Autol Art, Open */
"03150AJ"  =  "1"  /* Bypass R Axilla Art to R Extracran Art w Autol Art, Open */
"03150AK"  =  "1"  /* Bypass R Axilla Art to L Extracran Art w Autol Art, Open */
"03150J0"  =  "1"  /* Bypass R Axilla Art to R Up Arm Art w Synth Sub, Open */
"03150J1"  =  "1"  /* Bypass R Axilla Art to L Up Arm Art w Synth Sub, Open */
"03150J2"  =  "1"  /* Bypass R Axilla Art to Bi Up Arm Art w Synth Sub, Open */
"03150J3"  =  "1"  /* Bypass R Axilla Art to R Low Arm Art w Synth Sub, Open */
"03150J4"  =  "1"  /* Bypass R Axilla Art to L Low Arm Art w Synth Sub, Open */
"03150J5"  =  "1"  /* Bypass R Axilla Art to Bi Low Arm Art w Synth Sub, Open */
"03150J6"  =  "1"  /* Bypass R Axilla Art to R Up Leg Art w Synth Sub, Open */
"03150J7"  =  "1"  /* Bypass R Axilla Art to L Up Leg Art w Synth Sub, Open */
"03150J8"  =  "1"  /* Bypass R Axilla Art to Bi Up Leg Art w Synth Sub, Open */
"03150J9"  =  "1"  /* Bypass R Axilla Art to R Low Leg Art w Synth Sub, Open */
"03150JB"  =  "1"  /* Bypass R Axilla Art to L Low Leg Art w Synth Sub, Open */
"03150JC"  =  "1"  /* Bypass R Axilla Art to Bi Low Leg Art w Synth Sub, Open */
"03150JD"  =  "1"  /* Bypass R Axilla Art to Up Arm Vein w Synth Sub, Open */
"03150JF"  =  "1"  /* Bypass R Axilla Art to Low Arm Vein w Synth Sub, Open */
"03150JJ"  =  "1"  /* Bypass R Axilla Art to R Extracran Art w Synth Sub, Open */
"03150JK"  =  "1"  /* Bypass R Axilla Art to L Extracran Art w Synth Sub, Open */
"03150K0"  =  "1"  /* Bypass R Axilla Art to R Up Arm Art w Nonaut Sub, Open */
"03150K1"  =  "1"  /* Bypass R Axilla Art to L Up Arm Art w Nonaut Sub, Open */
"03150K2"  =  "1"  /* Bypass R Axilla Art to Bi Up Arm Art w Nonaut Sub, Open */
"03150K3"  =  "1"  /* Bypass R Axilla Art to R Low Arm Art w Nonaut Sub, Open */
"03150K4"  =  "1"  /* Bypass R Axilla Art to L Low Arm Art w Nonaut Sub, Open */
"03150K5"  =  "1"  /* Bypass R Axilla Art to Bi Low Arm Art w Nonaut Sub, Open */
"03150K6"  =  "1"  /* Bypass R Axilla Art to R Up Leg Art w Nonaut Sub, Open */
"03150K7"  =  "1"  /* Bypass R Axilla Art to L Up Leg Art w Nonaut Sub, Open */
"03150K8"  =  "1"  /* Bypass R Axilla Art to Bi Up Leg Art w Nonaut Sub, Open */
"03150K9"  =  "1"  /* Bypass R Axilla Art to R Low Leg Art w Nonaut Sub, Open */
"03150KB"  =  "1"  /* Bypass R Axilla Art to L Low Leg Art w Nonaut Sub, Open */
"03150KC"  =  "1"  /* Bypass R Axilla Art to Bi Low Leg Art w Nonaut Sub, Open */
"03150KD"  =  "1"  /* Bypass R Axilla Art to Up Arm Vein w Nonaut Sub, Open */
"03150KF"  =  "1"  /* Bypass R Axilla Art to Low Arm Vein w Nonaut Sub, Open */
"03150KJ"  =  "1"  /* Bypass R Axilla Art to R Extracran Art w Nonaut Sub, Open */
"03150KK"  =  "1"  /* Bypass R Axilla Art to L Extracran Art w Nonaut Sub, Open */
"03150Z0"  =  "1"  /* Bypass Right Axillary Artery to R Up Arm Art, Open Approach */
"03150Z1"  =  "1"  /* Bypass Right Axillary Artery to L Up Arm Art, Open Approach */
"03150Z2"  =  "1"  /* Bypass Right Axillary Artery to Bi Up Arm Art, Open Approach */
"03150Z3"  =  "1"  /* Bypass Right Axillary Artery to R Low Arm Art, Open Approach */
"03150Z4"  =  "1"  /* Bypass Right Axillary Artery to L Low Arm Art, Open Approach */
"03150Z5"  =  "1"  /* Bypass R Axilla Art to Bi Low Arm Art, Open Approach */
"03150Z6"  =  "1"  /* Bypass Right Axillary Artery to R Up Leg Art, Open Approach */
"03150Z7"  =  "1"  /* Bypass Right Axillary Artery to L Up Leg Art, Open Approach */
"03150Z8"  =  "1"  /* Bypass Right Axillary Artery to Bi Up Leg Art, Open Approach */
"03150Z9"  =  "1"  /* Bypass Right Axillary Artery to R Low Leg Art, Open Approach */
"03150ZB"  =  "1"  /* Bypass Right Axillary Artery to L Low Leg Art, Open Approach */
"03150ZC"  =  "1"  /* Bypass R Axilla Art to Bi Low Leg Art, Open Approach */
"03150ZD"  =  "1"  /* Bypass Right Axillary Artery to Up Arm Vein, Open Approach */
"03150ZF"  =  "1"  /* Bypass Right Axillary Artery to Low Arm Vein, Open Approach */
"03150ZJ"  =  "1"  /* Bypass R Axilla Art to R Extracran Art, Open Approach */
"03150ZK"  =  "1"  /* Bypass R Axilla Art to L Extracran Art, Open Approach */
"0316090"  =  "1"  /* Bypass L Axilla Art to R Up Arm Art w Autol Vn, Open */
"0316091"  =  "1"  /* Bypass L Axilla Art to L Up Arm Art w Autol Vn, Open */
"0316092"  =  "1"  /* Bypass L Axilla Art to Bi Up Arm Art w Autol Vn, Open */
"0316093"  =  "1"  /* Bypass L Axilla Art to R Low Arm Art w Autol Vn, Open */
"0316094"  =  "1"  /* Bypass L Axilla Art to L Low Arm Art w Autol Vn, Open */
"0316095"  =  "1"  /* Bypass L Axilla Art to Bi Low Arm Art w Autol Vn, Open */
"0316096"  =  "1"  /* Bypass L Axilla Art to R Up Leg Art w Autol Vn, Open */
"0316097"  =  "1"  /* Bypass L Axilla Art to L Up Leg Art w Autol Vn, Open */
"0316098"  =  "1"  /* Bypass L Axilla Art to Bi Up Leg Art w Autol Vn, Open */
"0316099"  =  "1"  /* Bypass L Axilla Art to R Low Leg Art w Autol Vn, Open */
"031609B"  =  "1"  /* Bypass L Axilla Art to L Low Leg Art w Autol Vn, Open */
"031609C"  =  "1"  /* Bypass L Axilla Art to Bi Low Leg Art w Autol Vn, Open */
"031609D"  =  "1"  /* Bypass L Axilla Art to Up Arm Vein w Autol Vn, Open */
"031609F"  =  "1"  /* Bypass L Axilla Art to Low Arm Vein w Autol Vn, Open */
"031609J"  =  "1"  /* Bypass L Axilla Art to R Extracran Art w Autol Vn, Open */
"031609K"  =  "1"  /* Bypass L Axilla Art to L Extracran Art w Autol Vn, Open */
"03160A0"  =  "1"  /* Bypass L Axilla Art to R Up Arm Art w Autol Art, Open */
"03160A1"  =  "1"  /* Bypass L Axilla Art to L Up Arm Art w Autol Art, Open */
"03160A2"  =  "1"  /* Bypass L Axilla Art to Bi Up Arm Art w Autol Art, Open */
"03160A3"  =  "1"  /* Bypass L Axilla Art to R Low Arm Art w Autol Art, Open */
"03160A4"  =  "1"  /* Bypass L Axilla Art to L Low Arm Art w Autol Art, Open */
"03160A5"  =  "1"  /* Bypass L Axilla Art to Bi Low Arm Art w Autol Art, Open */
"03160A6"  =  "1"  /* Bypass L Axilla Art to R Up Leg Art w Autol Art, Open */
"03160A7"  =  "1"  /* Bypass L Axilla Art to L Up Leg Art w Autol Art, Open */
"03160A8"  =  "1"  /* Bypass L Axilla Art to Bi Up Leg Art w Autol Art, Open */
"03160A9"  =  "1"  /* Bypass L Axilla Art to R Low Leg Art w Autol Art, Open */
"03160AB"  =  "1"  /* Bypass L Axilla Art to L Low Leg Art w Autol Art, Open */
"03160AC"  =  "1"  /* Bypass L Axilla Art to Bi Low Leg Art w Autol Art, Open */
"03160AD"  =  "1"  /* Bypass L Axilla Art to Up Arm Vein w Autol Art, Open */
"03160AF"  =  "1"  /* Bypass L Axilla Art to Low Arm Vein w Autol Art, Open */
"03160AJ"  =  "1"  /* Bypass L Axilla Art to R Extracran Art w Autol Art, Open */
"03160AK"  =  "1"  /* Bypass L Axilla Art to L Extracran Art w Autol Art, Open */
"03160J0"  =  "1"  /* Bypass L Axilla Art to R Up Arm Art w Synth Sub, Open */
"03160J1"  =  "1"  /* Bypass L Axilla Art to L Up Arm Art w Synth Sub, Open */
"03160J2"  =  "1"  /* Bypass L Axilla Art to Bi Up Arm Art w Synth Sub, Open */
"03160J3"  =  "1"  /* Bypass L Axilla Art to R Low Arm Art w Synth Sub, Open */
"03160J4"  =  "1"  /* Bypass L Axilla Art to L Low Arm Art w Synth Sub, Open */
"03160J5"  =  "1"  /* Bypass L Axilla Art to Bi Low Arm Art w Synth Sub, Open */
"03160J6"  =  "1"  /* Bypass L Axilla Art to R Up Leg Art w Synth Sub, Open */
"03160J7"  =  "1"  /* Bypass L Axilla Art to L Up Leg Art w Synth Sub, Open */
"03160J8"  =  "1"  /* Bypass L Axilla Art to Bi Up Leg Art w Synth Sub, Open */
"03160J9"  =  "1"  /* Bypass L Axilla Art to R Low Leg Art w Synth Sub, Open */
"03160JB"  =  "1"  /* Bypass L Axilla Art to L Low Leg Art w Synth Sub, Open */
"03160JC"  =  "1"  /* Bypass L Axilla Art to Bi Low Leg Art w Synth Sub, Open */
"03160JD"  =  "1"  /* Bypass L Axilla Art to Up Arm Vein w Synth Sub, Open */
"03160JF"  =  "1"  /* Bypass L Axilla Art to Low Arm Vein w Synth Sub, Open */
"03160JJ"  =  "1"  /* Bypass L Axilla Art to R Extracran Art w Synth Sub, Open */
"03160JK"  =  "1"  /* Bypass L Axilla Art to L Extracran Art w Synth Sub, Open */
"03160K0"  =  "1"  /* Bypass L Axilla Art to R Up Arm Art w Nonaut Sub, Open */
"03160K1"  =  "1"  /* Bypass L Axilla Art to L Up Arm Art w Nonaut Sub, Open */
"03160K2"  =  "1"  /* Bypass L Axilla Art to Bi Up Arm Art w Nonaut Sub, Open */
"03160K3"  =  "1"  /* Bypass L Axilla Art to R Low Arm Art w Nonaut Sub, Open */
"03160K4"  =  "1"  /* Bypass L Axilla Art to L Low Arm Art w Nonaut Sub, Open */
"03160K5"  =  "1"  /* Bypass L Axilla Art to Bi Low Arm Art w Nonaut Sub, Open */
"03160K6"  =  "1"  /* Bypass L Axilla Art to R Up Leg Art w Nonaut Sub, Open */
"03160K7"  =  "1"  /* Bypass L Axilla Art to L Up Leg Art w Nonaut Sub, Open */
"03160K8"  =  "1"  /* Bypass L Axilla Art to Bi Up Leg Art w Nonaut Sub, Open */
"03160K9"  =  "1"  /* Bypass L Axilla Art to R Low Leg Art w Nonaut Sub, Open */
"03160KB"  =  "1"  /* Bypass L Axilla Art to L Low Leg Art w Nonaut Sub, Open */
"03160KC"  =  "1"  /* Bypass L Axilla Art to Bi Low Leg Art w Nonaut Sub, Open */
"03160KD"  =  "1"  /* Bypass L Axilla Art to Up Arm Vein w Nonaut Sub, Open */
"03160KF"  =  "1"  /* Bypass L Axilla Art to Low Arm Vein w Nonaut Sub, Open */
"03160KJ"  =  "1"  /* Bypass L Axilla Art to R Extracran Art w Nonaut Sub, Open */
"03160KK"  =  "1"  /* Bypass L Axilla Art to L Extracran Art w Nonaut Sub, Open */
"03160Z0"  =  "1"  /* Bypass Left Axillary Artery to R Up Arm Art, Open Approach */
"03160Z1"  =  "1"  /* Bypass Left Axillary Artery to L Up Arm Art, Open Approach */
"03160Z2"  =  "1"  /* Bypass Left Axillary Artery to Bi Up Arm Art, Open Approach */
"03160Z3"  =  "1"  /* Bypass Left Axillary Artery to R Low Arm Art, Open Approach */
"03160Z4"  =  "1"  /* Bypass Left Axillary Artery to L Low Arm Art, Open Approach */
"03160Z5"  =  "1"  /* Bypass Left Axillary Artery to Bi Low Arm Art, Open Approach */
"03160Z6"  =  "1"  /* Bypass Left Axillary Artery to R Up Leg Art, Open Approach */
"03160Z7"  =  "1"  /* Bypass Left Axillary Artery to L Up Leg Art, Open Approach */
"03160Z8"  =  "1"  /* Bypass Left Axillary Artery to Bi Up Leg Art, Open Approach */
"03160Z9"  =  "1"  /* Bypass Left Axillary Artery to R Low Leg Art, Open Approach */
"03160ZB"  =  "1"  /* Bypass Left Axillary Artery to L Low Leg Art, Open Approach */
"03160ZC"  =  "1"  /* Bypass Left Axillary Artery to Bi Low Leg Art, Open Approach */
"03160ZD"  =  "1"  /* Bypass Left Axillary Artery to Upper Arm Vein, Open Approach */
"03160ZF"  =  "1"  /* Bypass Left Axillary Artery to Lower Arm Vein, Open Approach */
"03160ZJ"  =  "1"  /* Bypass L Axilla Art to R Extracran Art, Open Approach */
"03160ZK"  =  "1"  /* Bypass L Axilla Art to L Extracran Art, Open Approach */
"0317090"  =  "1"  /* Bypass R Brach Art to R Up Arm Art w Autol Vn, Open */
"0317093"  =  "1"  /* Bypass R Brach Art to R Low Arm Art w Autol Vn, Open */
"031709D"  =  "1"  /* Bypass R Brach Art to Up Arm Vein w Autol Vn, Open */
"031709F"  =  "1"  /* Bypass R Brach Art to Low Arm Vein w Autol Vn, Open */
"03170A0"  =  "1"  /* Bypass R Brach Art to R Up Arm Art w Autol Art, Open */
"03170A3"  =  "1"  /* Bypass R Brach Art to R Low Arm Art w Autol Art, Open */
"03170AD"  =  "1"  /* Bypass R Brach Art to Up Arm Vein w Autol Art, Open */
"03170AF"  =  "1"  /* Bypass R Brach Art to Low Arm Vein w Autol Art, Open */
"03170J0"  =  "1"  /* Bypass R Brach Art to R Up Arm Art w Synth Sub, Open */
"03170J3"  =  "1"  /* Bypass R Brach Art to R Low Arm Art w Synth Sub, Open */
"03170JD"  =  "1"  /* Bypass R Brach Art to Up Arm Vein w Synth Sub, Open */
"03170JF"  =  "1"  /* Bypass R Brach Art to Low Arm Vein w Synth Sub, Open */
"03170K0"  =  "1"  /* Bypass R Brach Art to R Up Arm Art w Nonaut Sub, Open */
"03170K3"  =  "1"  /* Bypass R Brach Art to R Low Arm Art w Nonaut Sub, Open */
"03170KD"  =  "1"  /* Bypass R Brach Art to Up Arm Vein w Nonaut Sub, Open */
"03170KF"  =  "1"  /* Bypass R Brach Art to Low Arm Vein w Nonaut Sub, Open */
"03170Z0"  =  "1"  /* Bypass Right Brachial Artery to R Up Arm Art, Open Approach */
"03170Z3"  =  "1"  /* Bypass Right Brachial Artery to R Low Arm Art, Open Approach */
"03170ZD"  =  "1"  /* Bypass Right Brachial Artery to Up Arm Vein, Open Approach */
"03170ZF"  =  "1"  /* Bypass Right Brachial Artery to Low Arm Vein, Open Approach */
"0318091"  =  "1"  /* Bypass L Brach Art to L Up Arm Art w Autol Vn, Open */
"0318094"  =  "1"  /* Bypass L Brach Art to L Low Arm Art w Autol Vn, Open */
"031809D"  =  "1"  /* Bypass L Brach Art to Up Arm Vein w Autol Vn, Open */
"031809F"  =  "1"  /* Bypass L Brach Art to Low Arm Vein w Autol Vn, Open */
"03180A1"  =  "1"  /* Bypass L Brach Art to L Up Arm Art w Autol Art, Open */
"03180A4"  =  "1"  /* Bypass L Brach Art to L Low Arm Art w Autol Art, Open */
"03180AD"  =  "1"  /* Bypass L Brach Art to Up Arm Vein w Autol Art, Open */
"03180AF"  =  "1"  /* Bypass L Brach Art to Low Arm Vein w Autol Art, Open */
"03180J1"  =  "1"  /* Bypass L Brach Art to L Up Arm Art w Synth Sub, Open */
"03180J4"  =  "1"  /* Bypass L Brach Art to L Low Arm Art w Synth Sub, Open */
"03180JD"  =  "1"  /* Bypass L Brach Art to Up Arm Vein w Synth Sub, Open */
"03180JF"  =  "1"  /* Bypass L Brach Art to Low Arm Vein w Synth Sub, Open */
"03180K1"  =  "1"  /* Bypass L Brach Art to L Up Arm Art w Nonaut Sub, Open */
"03180K4"  =  "1"  /* Bypass L Brach Art to L Low Arm Art w Nonaut Sub, Open */
"03180KD"  =  "1"  /* Bypass L Brach Art to Up Arm Vein w Nonaut Sub, Open */
"03180KF"  =  "1"  /* Bypass L Brach Art to Low Arm Vein w Nonaut Sub, Open */
"03180Z1"  =  "1"  /* Bypass Left Brachial Artery to L Up Arm Art, Open Approach */
"03180Z4"  =  "1"  /* Bypass Left Brachial Artery to L Low Arm Art, Open Approach */
"03180ZD"  =  "1"  /* Bypass Left Brachial Artery to Upper Arm Vein, Open Approach */
"03180ZF"  =  "1"  /* Bypass Left Brachial Artery to Lower Arm Vein, Open Approach */
"0319093"  =  "1"  /* Bypass R Ulnar Art to R Low Arm Art w Autol Vn, Open */
"031909F"  =  "1"  /* Bypass R Ulnar Art to Low Arm Vein w Autol Vn, Open */
"03190A3"  =  "1"  /* Bypass R Ulnar Art to R Low Arm Art w Autol Art, Open */
"03190AF"  =  "1"  /* Bypass R Ulnar Art to Low Arm Vein w Autol Art, Open */
"03190J3"  =  "1"  /* Bypass R Ulnar Art to R Low Arm Art w Synth Sub, Open */
"03190JF"  =  "1"  /* Bypass R Ulnar Art to Low Arm Vein w Synth Sub, Open */
"03190K3"  =  "1"  /* Bypass R Ulnar Art to R Low Arm Art w Nonaut Sub, Open */
"03190KF"  =  "1"  /* Bypass R Ulnar Art to Low Arm Vein w Nonaut Sub, Open */
"03190Z3"  =  "1"  /* Bypass Right Ulnar Artery to R Low Arm Art, Open Approach */
"03190ZF"  =  "1"  /* Bypass Right Ulnar Artery to Lower Arm Vein, Open Approach */
"031A094"  =  "1"  /* Bypass L Ulnar Art to L Low Arm Art w Autol Vn, Open */
"031A09F"  =  "1"  /* Bypass L Ulnar Art to Low Arm Vein w Autol Vn, Open */
"031A0A4"  =  "1"  /* Bypass L Ulnar Art to L Low Arm Art w Autol Art, Open */
"031A0AF"  =  "1"  /* Bypass L Ulnar Art to Low Arm Vein w Autol Art, Open */
"031A0J4"  =  "1"  /* Bypass L Ulnar Art to L Low Arm Art w Synth Sub, Open */
"031A0JF"  =  "1"  /* Bypass L Ulnar Art to Low Arm Vein w Synth Sub, Open */
"031A0K4"  =  "1"  /* Bypass L Ulnar Art to L Low Arm Art w Nonaut Sub, Open */
"031A0KF"  =  "1"  /* Bypass L Ulnar Art to Low Arm Vein w Nonaut Sub, Open */
"031A0Z4"  =  "1"  /* Bypass Left Ulnar Artery to L Low Arm Art, Open Approach */
"031A0ZF"  =  "1"  /* Bypass Left Ulnar Artery to Lower Arm Vein, Open Approach */
"031B093"  =  "1"  /* Bypass R Radial Art to R Low Arm Art w Autol Vn, Open */
"031B09F"  =  "1"  /* Bypass R Radial Art to Low Arm Vein w Autol Vn, Open */
"031B0A3"  =  "1"  /* Bypass R Radial Art to R Low Arm Art w Autol Art, Open */
"031B0AF"  =  "1"  /* Bypass R Radial Art to Low Arm Vein w Autol Art, Open */
"031B0J3"  =  "1"  /* Bypass R Radial Art to R Low Arm Art w Synth Sub, Open */
"031B0JF"  =  "1"  /* Bypass R Radial Art to Low Arm Vein w Synth Sub, Open */
"031B0K3"  =  "1"  /* Bypass R Radial Art to R Low Arm Art w Nonaut Sub, Open */
"031B0KF"  =  "1"  /* Bypass R Radial Art to Low Arm Vein w Nonaut Sub, Open */
"031B0Z3"  =  "1"  /* Bypass Right Radial Artery to R Low Arm Art, Open Approach */
"031B0ZF"  =  "1"  /* Bypass Right Radial Artery to Lower Arm Vein, Open Approach */
"031C094"  =  "1"  /* Bypass L Radial Art to L Low Arm Art w Autol Vn, Open */
"031C09F"  =  "1"  /* Bypass L Radial Art to Low Arm Vein w Autol Vn, Open */
"031C0A4"  =  "1"  /* Bypass L Radial Art to L Low Arm Art w Autol Art, Open */
"031C0AF"  =  "1"  /* Bypass L Radial Art to Low Arm Vein w Autol Art, Open */
"031C0J4"  =  "1"  /* Bypass L Radial Art to L Low Arm Art w Synth Sub, Open */
"031C0JF"  =  "1"  /* Bypass L Radial Art to Low Arm Vein w Synth Sub, Open */
"031C0K4"  =  "1"  /* Bypass L Radial Art to L Low Arm Art w Nonaut Sub, Open */
"031C0KF"  =  "1"  /* Bypass L Radial Art to Low Arm Vein w Nonaut Sub, Open */
"031C0Z4"  =  "1"  /* Bypass Left Radial Artery to L Low Arm Art, Open Approach */
"031C0ZF"  =  "1"  /* Bypass Left Radial Artery to Lower Arm Vein, Open Approach */
"031G09G"  =  "1"  /* Bypass Intracran Art to Intracran Art w Autol Vn, Open */
"031G0AG"  =  "1"  /* Bypass Intracran Art to Intracran Art w Autol Art, Open */
"031G0JG"  =  "1"  /* Bypass Intracran Art to Intracran Art w Synth Sub, Open */
"031G0KG"  =  "1"  /* Bypass Intracran Art to Intracran Art w Nonaut Sub, Open */
"031G0ZG"  =  "1"  /* Bypass Intracranial Artery to Intracran Art, Open Approach */
"031H09G"  =  "1"  /* Bypass R Com Carotid to Intracran Art w Autol Vn, Open */
"031H09J"  =  "1"  /* Bypass R Com Carotid to R Extracran Art w Autol Vn, Open */
"031H0AG"  =  "1"  /* Bypass R Com Carotid to Intracran Art w Autol Art, Open */
"031H0AJ"  =  "1"  /* Bypass R Com Carotid to R Extracran Art w Autol Art, Open */
"031H0JG"  =  "1"  /* Bypass R Com Carotid to Intracran Art w Synth Sub, Open */
"031H0JJ"  =  "1"  /* Bypass R Com Carotid to R Extracran Art w Synth Sub, Open */
"031H0KG"  =  "1"  /* Bypass R Com Carotid to Intracran Art w Nonaut Sub, Open */
"031H0KJ"  =  "1"  /* Bypass R Com Carotid to R Extracran Art w Nonaut Sub, Open */
"031H0ZG"  =  "1"  /* Bypass R Com Carotid to Intracran Art, Open Approach */
"031H0ZJ"  =  "1"  /* Bypass R Com Carotid to R Extracran Art, Open Approach */
"031J09G"  =  "1"  /* Bypass L Com Carotid to Intracran Art w Autol Vn, Open */
"031J09K"  =  "1"  /* Bypass L Com Carotid to L Extracran Art w Autol Vn, Open */
"031J0AG"  =  "1"  /* Bypass L Com Carotid to Intracran Art w Autol Art, Open */
"031J0AK"  =  "1"  /* Bypass L Com Carotid to L Extracran Art w Autol Art, Open */
"031J0JG"  =  "1"  /* Bypass L Com Carotid to Intracran Art w Synth Sub, Open */
"031J0JK"  =  "1"  /* Bypass L Com Carotid to L Extracran Art w Synth Sub, Open */
"031J0KG"  =  "1"  /* Bypass L Com Carotid to Intracran Art w Nonaut Sub, Open */
"031J0KK"  =  "1"  /* Bypass L Com Carotid to L Extracran Art w Nonaut Sub, Open */
"031J0ZG"  =  "1"  /* Bypass L Com Carotid to Intracran Art, Open Approach */
"031J0ZK"  =  "1"  /* Bypass L Com Carotid to L Extracran Art, Open Approach */
"031K09J"  =  "1"  /* Bypass R Int Carotid to R Extracran Art w Autol Vn, Open */
"031K0AJ"  =  "1"  /* Bypass R Int Carotid to R Extracran Art w Autol Art, Open */
"031K0JJ"  =  "1"  /* Bypass R Int Carotid to R Extracran Art w Synth Sub, Open */
"031K0KJ"  =  "1"  /* Bypass R Int Carotid to R Extracran Art w Nonaut Sub, Open */
"031K0ZJ"  =  "1"  /* Bypass R Int Carotid to R Extracran Art, Open Approach */
"031L09K"  =  "1"  /* Bypass L Int Carotid to L Extracran Art w Autol Vn, Open */
"031L0AK"  =  "1"  /* Bypass L Int Carotid to L Extracran Art w Autol Art, Open */
"031L0JK"  =  "1"  /* Bypass L Int Carotid to L Extracran Art w Synth Sub, Open */
"031L0KK"  =  "1"  /* Bypass L Int Carotid to L Extracran Art w Nonaut Sub, Open */
"031L0ZK"  =  "1"  /* Bypass L Int Carotid to L Extracran Art, Open Approach */
"031M09J"  =  "1"  /* Bypass R Ext Carotid to R Extracran Art w Autol Vn, Open */
"031M0AJ"  =  "1"  /* Bypass R Ext Carotid to R Extracran Art w Autol Art, Open */
"031M0JJ"  =  "1"  /* Bypass R Ext Carotid to R Extracran Art w Synth Sub, Open */
"031M0KJ"  =  "1"  /* Bypass R Ext Carotid to R Extracran Art w Nonaut Sub, Open */
"031M0ZJ"  =  "1"  /* Bypass R Ext Carotid to R Extracran Art, Open Approach */
"031N09K"  =  "1"  /* Bypass L Ext Carotid to L Extracran Art w Autol Vn, Open */
"031N0AK"  =  "1"  /* Bypass L Ext Carotid to L Extracran Art w Autol Art, Open */
"031N0JK"  =  "1"  /* Bypass L Ext Carotid to L Extracran Art w Synth Sub, Open */
"031N0KK"  =  "1"  /* Bypass L Ext Carotid to L Extracran Art w Nonaut Sub, Open */
"031N0ZK"  =  "1"  /* Bypass L Ext Carotid to L Extracran Art, Open Approach */
"031S09G"  =  "1"  /* Bypass R Temporal Art to Intracran Art w Autol Vn, Open */
"031S0AG"  =  "1"  /* Bypass R Temporal Art to Intracran Art w Autol Art, Open */
"031S0JG"  =  "1"  /* Bypass R Temporal Art to Intracran Art w Synth Sub, Open */
"031S0KG"  =  "1"  /* Bypass R Temporal Art to Intracran Art w Nonaut Sub, Open */
"031S0ZG"  =  "1"  /* Bypass Right Temporal Artery to Intracran Art, Open Approach */
"031T09G"  =  "1"  /* Bypass L Temporal Art to Intracran Art w Autol Vn, Open */
"031T0AG"  =  "1"  /* Bypass L Temporal Art to Intracran Art w Autol Art, Open */
"031T0JG"  =  "1"  /* Bypass L Temporal Art to Intracran Art w Synth Sub, Open */
"031T0KG"  =  "1"  /* Bypass L Temporal Art to Intracran Art w Nonaut Sub, Open */
"031T0ZG"  =  "1"  /* Bypass Left Temporal Artery to Intracran Art, Open Approach */
"03500ZZ"  =  "1"  /* Destruction of Right Internal Mammary Artery, Open Approach */
"03503ZZ"  =  "1"  /* Destruction of Right Internal Mammary Artery, Perc Approach */
"03504ZZ"  =  "1"  /* Destruction of R Int Mamm Art, Perc Endo Approach */
"03510ZZ"  =  "1"  /* Destruction of Left Internal Mammary Artery, Open Approach */
"03513ZZ"  =  "1"  /* Destruction of Left Internal Mammary Artery, Perc Approach */
"03514ZZ"  =  "1"  /* Destruction of L Int Mamm Art, Perc Endo Approach */
"03520ZZ"  =  "1"  /* Destruction of Innominate Artery, Open Approach */
"03523ZZ"  =  "1"  /* Destruction of Innominate Artery, Percutaneous Approach */
"03524ZZ"  =  "1"  /* Destruction of Innominate Artery, Perc Endo Approach */
"03530ZZ"  =  "1"  /* Destruction of Right Subclavian Artery, Open Approach */
"03533ZZ"  =  "1"  /* Destruction of Right Subclavian Artery, Perc Approach */
"03534ZZ"  =  "1"  /* Destruction of Right Subclavian Artery, Perc Endo Approach */
"03540ZZ"  =  "1"  /* Destruction of Left Subclavian Artery, Open Approach */
"03543ZZ"  =  "1"  /* Destruction of Left Subclavian Artery, Percutaneous Approach */
"03544ZZ"  =  "1"  /* Destruction of Left Subclavian Artery, Perc Endo Approach */
"03550ZZ"  =  "1"  /* Destruction of Right Axillary Artery, Open Approach */
"03553ZZ"  =  "1"  /* Destruction of Right Axillary Artery, Percutaneous Approach */
"03554ZZ"  =  "1"  /* Destruction of Right Axillary Artery, Perc Endo Approach */
"03560ZZ"  =  "1"  /* Destruction of Left Axillary Artery, Open Approach */
"03563ZZ"  =  "1"  /* Destruction of Left Axillary Artery, Percutaneous Approach */
"03564ZZ"  =  "1"  /* Destruction of Left Axillary Artery, Perc Endo Approach */
"03570ZZ"  =  "1"  /* Destruction of Right Brachial Artery, Open Approach */
"03573ZZ"  =  "1"  /* Destruction of Right Brachial Artery, Percutaneous Approach */
"03574ZZ"  =  "1"  /* Destruction of Right Brachial Artery, Perc Endo Approach */
"03580ZZ"  =  "1"  /* Destruction of Left Brachial Artery, Open Approach */
"03583ZZ"  =  "1"  /* Destruction of Left Brachial Artery, Percutaneous Approach */
"03584ZZ"  =  "1"  /* Destruction of Left Brachial Artery, Perc Endo Approach */
"03590ZZ"  =  "1"  /* Destruction of Right Ulnar Artery, Open Approach */
"03593ZZ"  =  "1"  /* Destruction of Right Ulnar Artery, Percutaneous Approach */
"03594ZZ"  =  "1"  /* Destruction of Right Ulnar Artery, Perc Endo Approach */
"035A0ZZ"  =  "1"  /* Destruction of Left Ulnar Artery, Open Approach */
"035A3ZZ"  =  "1"  /* Destruction of Left Ulnar Artery, Percutaneous Approach */
"035A4ZZ"  =  "1"  /* Destruction of Left Ulnar Artery, Perc Endo Approach */
"035B0ZZ"  =  "1"  /* Destruction of Right Radial Artery, Open Approach */
"035B3ZZ"  =  "1"  /* Destruction of Right Radial Artery, Percutaneous Approach */
"035B4ZZ"  =  "1"  /* Destruction of Right Radial Artery, Perc Endo Approach */
"035C0ZZ"  =  "1"  /* Destruction of Left Radial Artery, Open Approach */
"035C3ZZ"  =  "1"  /* Destruction of Left Radial Artery, Percutaneous Approach */
"035C4ZZ"  =  "1"  /* Destruction of Left Radial Artery, Perc Endo Approach */
"035D0ZZ"  =  "1"  /* Destruction of Right Hand Artery, Open Approach */
"035D3ZZ"  =  "1"  /* Destruction of Right Hand Artery, Percutaneous Approach */
"035D4ZZ"  =  "1"  /* Destruction of Right Hand Artery, Perc Endo Approach */
"035F0ZZ"  =  "1"  /* Destruction of Left Hand Artery, Open Approach */
"035F3ZZ"  =  "1"  /* Destruction of Left Hand Artery, Percutaneous Approach */
"035F4ZZ"  =  "1"  /* Destruction of Left Hand Artery, Perc Endo Approach */
"035G0ZZ"  =  "1"  /* Destruction of Intracranial Artery, Open Approach */
"035G3ZZ"  =  "1"  /* Destruction of Intracranial Artery, Percutaneous Approach */
"035G4ZZ"  =  "1"  /* Destruction of Intracranial Artery, Perc Endo Approach */
"035H0ZZ"  =  "1"  /* Destruction of Right Common Carotid Artery, Open Approach */
"035H3ZZ"  =  "1"  /* Destruction of Right Common Carotid Artery, Perc Approach */
"035H4ZZ"  =  "1"  /* Destruction of R Com Carotid, Perc Endo Approach */
"035J0ZZ"  =  "1"  /* Destruction of Left Common Carotid Artery, Open Approach */
"035J3ZZ"  =  "1"  /* Destruction of Left Common Carotid Artery, Perc Approach */
"035J4ZZ"  =  "1"  /* Destruction of L Com Carotid, Perc Endo Approach */
"035K0ZZ"  =  "1"  /* Destruction of Right Internal Carotid Artery, Open Approach */
"035K3ZZ"  =  "1"  /* Destruction of Right Internal Carotid Artery, Perc Approach */
"035K4ZZ"  =  "1"  /* Destruction of R Int Carotid, Perc Endo Approach */
"035L0ZZ"  =  "1"  /* Destruction of Left Internal Carotid Artery, Open Approach */
"035L3ZZ"  =  "1"  /* Destruction of Left Internal Carotid Artery, Perc Approach */
"035L4ZZ"  =  "1"  /* Destruction of L Int Carotid, Perc Endo Approach */
"035M0ZZ"  =  "1"  /* Destruction of Right External Carotid Artery, Open Approach */
"035M3ZZ"  =  "1"  /* Destruction of Right External Carotid Artery, Perc Approach */
"035M4ZZ"  =  "1"  /* Destruction of R Ext Carotid, Perc Endo Approach */
"035N0ZZ"  =  "1"  /* Destruction of Left External Carotid Artery, Open Approach */
"035N3ZZ"  =  "1"  /* Destruction of Left External Carotid Artery, Perc Approach */
"035N4ZZ"  =  "1"  /* Destruction of L Ext Carotid, Perc Endo Approach */
"035P0ZZ"  =  "1"  /* Destruction of Right Vertebral Artery, Open Approach */
"035P3ZZ"  =  "1"  /* Destruction of Right Vertebral Artery, Percutaneous Approach */
"035P4ZZ"  =  "1"  /* Destruction of Right Vertebral Artery, Perc Endo Approach */
"035Q0ZZ"  =  "1"  /* Destruction of Left Vertebral Artery, Open Approach */
"035Q3ZZ"  =  "1"  /* Destruction of Left Vertebral Artery, Percutaneous Approach */
"035Q4ZZ"  =  "1"  /* Destruction of Left Vertebral Artery, Perc Endo Approach */
"035R0ZZ"  =  "1"  /* Destruction of Face Artery, Open Approach */
"035R3ZZ"  =  "1"  /* Destruction of Face Artery, Percutaneous Approach */
"035R4ZZ"  =  "1"  /* Destruction of Face Artery, Percutaneous Endoscopic Approach */
"035S0ZZ"  =  "1"  /* Destruction of Right Temporal Artery, Open Approach */
"035S3ZZ"  =  "1"  /* Destruction of Right Temporal Artery, Percutaneous Approach */
"035S4ZZ"  =  "1"  /* Destruction of Right Temporal Artery, Perc Endo Approach */
"035T0ZZ"  =  "1"  /* Destruction of Left Temporal Artery, Open Approach */
"035T3ZZ"  =  "1"  /* Destruction of Left Temporal Artery, Percutaneous Approach */
"035T4ZZ"  =  "1"  /* Destruction of Left Temporal Artery, Perc Endo Approach */
"035U0ZZ"  =  "1"  /* Destruction of Right Thyroid Artery, Open Approach */
"035U3ZZ"  =  "1"  /* Destruction of Right Thyroid Artery, Percutaneous Approach */
"035U4ZZ"  =  "1"  /* Destruction of Right Thyroid Artery, Perc Endo Approach */
"035V0ZZ"  =  "1"  /* Destruction of Left Thyroid Artery, Open Approach */
"035V3ZZ"  =  "1"  /* Destruction of Left Thyroid Artery, Percutaneous Approach */
"035V4ZZ"  =  "1"  /* Destruction of Left Thyroid Artery, Perc Endo Approach */
"035Y0ZZ"  =  "1"  /* Destruction of Upper Artery, Open Approach */
"035Y3ZZ"  =  "1"  /* Destruction of Upper Artery, Percutaneous Approach */
"035Y4ZZ"  =  "1"  /* Destruction of Upper Artery, Perc Endo Approach */
"037004Z"  =  "1"  /* Dilate R Int Mamm Art w Drug-elut Intralum, Open */
"03700DZ"  =  "1"  /* Dilation of R Int Mamm Art with Intralum Dev, Open Approach */
"03700ZZ"  =  "1"  /* Dilation of Right Internal Mammary Artery, Open Approach */
"037034Z"  =  "1"  /* Dilate R Int Mamm Art w Drug-elut Intralum, Perc */
"03703DZ"  =  "1"  /* Dilation of R Int Mamm Art with Intralum Dev, Perc Approach */
"03703ZZ"  =  "1"  /* Dilation of Right Internal Mammary Artery, Perc Approach */
"037044Z"  =  "1"  /* Dilate R Int Mamm Art w Drug-elut Intralum, Perc Endo */
"03704DZ"  =  "1"  /* Dilate R Int Mamm Art w Intralum Dev, Perc Endo */
"03704ZZ"  =  "1"  /* Dilation of R Int Mamm Art, Perc Endo Approach */
"037104Z"  =  "1"  /* Dilate L Int Mamm Art w Drug-elut Intralum, Open */
"03710DZ"  =  "1"  /* Dilation of L Int Mamm Art with Intralum Dev, Open Approach */
"03710ZZ"  =  "1"  /* Dilation of Left Internal Mammary Artery, Open Approach */
"037134Z"  =  "1"  /* Dilate L Int Mamm Art w Drug-elut Intralum, Perc */
"03713DZ"  =  "1"  /* Dilation of L Int Mamm Art with Intralum Dev, Perc Approach */
"03713ZZ"  =  "1"  /* Dilation of Left Internal Mammary Artery, Perc Approach */
"037144Z"  =  "1"  /* Dilate L Int Mamm Art w Drug-elut Intralum, Perc Endo */
"03714DZ"  =  "1"  /* Dilate L Int Mamm Art w Intralum Dev, Perc Endo */
"03714ZZ"  =  "1"  /* Dilation of Left Internal Mammary Artery, Perc Endo Approach */
"037204Z"  =  "1"  /* Dilation of Innom Art with Drug-elut Intralum, Open Approach */
"03720DZ"  =  "1"  /* Dilation of Innom Art with Intralum Dev, Open Approach */
"03720ZZ"  =  "1"  /* Dilation of Innominate Artery, Open Approach */
"037234Z"  =  "1"  /* Dilation of Innom Art with Drug-elut Intralum, Perc Approach */
"03723DZ"  =  "1"  /* Dilation of Innom Art with Intralum Dev, Perc Approach */
"03723ZZ"  =  "1"  /* Dilation of Innominate Artery, Percutaneous Approach */
"037244Z"  =  "1"  /* Dilate Innom Art w Drug-elut Intralum, Perc Endo */
"03724DZ"  =  "1"  /* Dilation of Innom Art with Intralum Dev, Perc Endo Approach */
"03724ZZ"  =  "1"  /* Dilation of Innominate Artery, Perc Endo Approach */
"037304Z"  =  "1"  /* Dilate R Subclav Art w Drug-elut Intralum, Open */
"03730DZ"  =  "1"  /* Dilation of R Subclav Art with Intralum Dev, Open Approach */
"03730ZZ"  =  "1"  /* Dilation of Right Subclavian Artery, Open Approach */
"037334Z"  =  "1"  /* Dilate R Subclav Art w Drug-elut Intralum, Perc */
"03733DZ"  =  "1"  /* Dilation of R Subclav Art with Intralum Dev, Perc Approach */
"03733ZZ"  =  "1"  /* Dilation of Right Subclavian Artery, Percutaneous Approach */
"037344Z"  =  "1"  /* Dilate R Subclav Art w Drug-elut Intralum, Perc Endo */
"03734DZ"  =  "1"  /* Dilate R Subclav Art w Intralum Dev, Perc Endo */
"03734ZZ"  =  "1"  /* Dilation of Right Subclavian Artery, Perc Endo Approach */
"037404Z"  =  "1"  /* Dilate L Subclav Art w Drug-elut Intralum, Open */
"03740DZ"  =  "1"  /* Dilation of L Subclav Art with Intralum Dev, Open Approach */
"03740ZZ"  =  "1"  /* Dilation of Left Subclavian Artery, Open Approach */
"037434Z"  =  "1"  /* Dilate L Subclav Art w Drug-elut Intralum, Perc */
"03743DZ"  =  "1"  /* Dilation of L Subclav Art with Intralum Dev, Perc Approach */
"03743ZZ"  =  "1"  /* Dilation of Left Subclavian Artery, Percutaneous Approach */
"037444Z"  =  "1"  /* Dilate L Subclav Art w Drug-elut Intralum, Perc Endo */
"03744DZ"  =  "1"  /* Dilate L Subclav Art w Intralum Dev, Perc Endo */
"03744ZZ"  =  "1"  /* Dilation of Left Subclavian Artery, Perc Endo Approach */
"037504Z"  =  "1"  /* Dilate R Axilla Art w Drug-elut Intralum, Open */
"03750DZ"  =  "1"  /* Dilation of R Axilla Art with Intralum Dev, Open Approach */
"03750ZZ"  =  "1"  /* Dilation of Right Axillary Artery, Open Approach */
"037534Z"  =  "1"  /* Dilate R Axilla Art w Drug-elut Intralum, Perc */
"03753DZ"  =  "1"  /* Dilation of R Axilla Art with Intralum Dev, Perc Approach */
"03753ZZ"  =  "1"  /* Dilation of Right Axillary Artery, Percutaneous Approach */
"037544Z"  =  "1"  /* Dilate R Axilla Art w Drug-elut Intralum, Perc Endo */
"03754DZ"  =  "1"  /* Dilate of R Axilla Art with Intralum Dev, Perc Endo Approach */
"03754ZZ"  =  "1"  /* Dilation of Right Axillary Artery, Perc Endo Approach */
"037604Z"  =  "1"  /* Dilate L Axilla Art w Drug-elut Intralum, Open */
"03760DZ"  =  "1"  /* Dilation of L Axilla Art with Intralum Dev, Open Approach */
"03760ZZ"  =  "1"  /* Dilation of Left Axillary Artery, Open Approach */
"037634Z"  =  "1"  /* Dilate L Axilla Art w Drug-elut Intralum, Perc */
"03763DZ"  =  "1"  /* Dilation of L Axilla Art with Intralum Dev, Perc Approach */
"03763ZZ"  =  "1"  /* Dilation of Left Axillary Artery, Percutaneous Approach */
"037644Z"  =  "1"  /* Dilate L Axilla Art w Drug-elut Intralum, Perc Endo */
"03764DZ"  =  "1"  /* Dilate of L Axilla Art with Intralum Dev, Perc Endo Approach */
"03764ZZ"  =  "1"  /* Dilation of Left Axillary Artery, Perc Endo Approach */
"037704Z"  =  "1"  /* Dilate of R Brach Art with Drug-elut Intralum, Open Approach */
"03770DZ"  =  "1"  /* Dilation of R Brach Art with Intralum Dev, Open Approach */
"03770ZZ"  =  "1"  /* Dilation of Right Brachial Artery, Open Approach */
"037734Z"  =  "1"  /* Dilate of R Brach Art with Drug-elut Intralum, Perc Approach */
"03773DZ"  =  "1"  /* Dilation of R Brach Art with Intralum Dev, Perc Approach */
"03773ZZ"  =  "1"  /* Dilation of Right Brachial Artery, Percutaneous Approach */
"037744Z"  =  "1"  /* Dilate R Brach Art w Drug-elut Intralum, Perc Endo */
"03774DZ"  =  "1"  /* Dilate of R Brach Art with Intralum Dev, Perc Endo Approach */
"03774ZZ"  =  "1"  /* Dilation of Right Brachial Artery, Perc Endo Approach */
"037804Z"  =  "1"  /* Dilate of L Brach Art with Drug-elut Intralum, Open Approach */
"03780DZ"  =  "1"  /* Dilation of L Brach Art with Intralum Dev, Open Approach */
"03780ZZ"  =  "1"  /* Dilation of Left Brachial Artery, Open Approach */
"037834Z"  =  "1"  /* Dilate of L Brach Art with Drug-elut Intralum, Perc Approach */
"03783DZ"  =  "1"  /* Dilation of L Brach Art with Intralum Dev, Perc Approach */
"03783ZZ"  =  "1"  /* Dilation of Left Brachial Artery, Percutaneous Approach */
"037844Z"  =  "1"  /* Dilate L Brach Art w Drug-elut Intralum, Perc Endo */
"03784DZ"  =  "1"  /* Dilate of L Brach Art with Intralum Dev, Perc Endo Approach */
"03784ZZ"  =  "1"  /* Dilation of Left Brachial Artery, Perc Endo Approach */
"037904Z"  =  "1"  /* Dilate of R Ulnar Art with Drug-elut Intralum, Open Approach */
"03790DZ"  =  "1"  /* Dilation of R Ulnar Art with Intralum Dev, Open Approach */
"03790ZZ"  =  "1"  /* Dilation of Right Ulnar Artery, Open Approach */
"037934Z"  =  "1"  /* Dilate of R Ulnar Art with Drug-elut Intralum, Perc Approach */
"03793DZ"  =  "1"  /* Dilation of R Ulnar Art with Intralum Dev, Perc Approach */
"03793ZZ"  =  "1"  /* Dilation of Right Ulnar Artery, Percutaneous Approach */
"037944Z"  =  "1"  /* Dilate R Ulnar Art w Drug-elut Intralum, Perc Endo */
"03794DZ"  =  "1"  /* Dilate of R Ulnar Art with Intralum Dev, Perc Endo Approach */
"03794ZZ"  =  "1"  /* Dilation of Right Ulnar Artery, Perc Endo Approach */
"037A04Z"  =  "1"  /* Dilate of L Ulnar Art with Drug-elut Intralum, Open Approach */
"037A0DZ"  =  "1"  /* Dilation of L Ulnar Art with Intralum Dev, Open Approach */
"037A0ZZ"  =  "1"  /* Dilation of Left Ulnar Artery, Open Approach */
"037A34Z"  =  "1"  /* Dilate of L Ulnar Art with Drug-elut Intralum, Perc Approach */
"037A3DZ"  =  "1"  /* Dilation of L Ulnar Art with Intralum Dev, Perc Approach */
"037A3ZZ"  =  "1"  /* Dilation of Left Ulnar Artery, Percutaneous Approach */
"037A44Z"  =  "1"  /* Dilate L Ulnar Art w Drug-elut Intralum, Perc Endo */
"037A4DZ"  =  "1"  /* Dilate of L Ulnar Art with Intralum Dev, Perc Endo Approach */
"037A4ZZ"  =  "1"  /* Dilation of Left Ulnar Artery, Perc Endo Approach */
"037B04Z"  =  "1"  /* Dilate R Radial Art w Drug-elut Intralum, Open */
"037B0DZ"  =  "1"  /* Dilation of R Radial Art with Intralum Dev, Open Approach */
"037B0ZZ"  =  "1"  /* Dilation of Right Radial Artery, Open Approach */
"037B34Z"  =  "1"  /* Dilate R Radial Art w Drug-elut Intralum, Perc */
"037B3DZ"  =  "1"  /* Dilation of R Radial Art with Intralum Dev, Perc Approach */
"037B3ZZ"  =  "1"  /* Dilation of Right Radial Artery, Percutaneous Approach */
"037B44Z"  =  "1"  /* Dilate R Radial Art w Drug-elut Intralum, Perc Endo */
"037B4DZ"  =  "1"  /* Dilate of R Radial Art with Intralum Dev, Perc Endo Approach */
"037B4ZZ"  =  "1"  /* Dilation of Right Radial Artery, Perc Endo Approach */
"037C04Z"  =  "1"  /* Dilate L Radial Art w Drug-elut Intralum, Open */
"037C0DZ"  =  "1"  /* Dilation of L Radial Art with Intralum Dev, Open Approach */
"037C0ZZ"  =  "1"  /* Dilation of Left Radial Artery, Open Approach */
"037C34Z"  =  "1"  /* Dilate L Radial Art w Drug-elut Intralum, Perc */
"037C3DZ"  =  "1"  /* Dilation of L Radial Art with Intralum Dev, Perc Approach */
"037C3ZZ"  =  "1"  /* Dilation of Left Radial Artery, Percutaneous Approach */
"037C44Z"  =  "1"  /* Dilate L Radial Art w Drug-elut Intralum, Perc Endo */
"037C4DZ"  =  "1"  /* Dilate of L Radial Art with Intralum Dev, Perc Endo Approach */
"037C4ZZ"  =  "1"  /* Dilation of Left Radial Artery, Perc Endo Approach */
"037D04Z"  =  "1"  /* Dilate of R Hand Art with Drug-elut Intralum, Open Approach */
"037D0DZ"  =  "1"  /* Dilation of R Hand Art with Intralum Dev, Open Approach */
"037D0ZZ"  =  "1"  /* Dilation of Right Hand Artery, Open Approach */
"037D34Z"  =  "1"  /* Dilate of R Hand Art with Drug-elut Intralum, Perc Approach */
"037D3DZ"  =  "1"  /* Dilation of R Hand Art with Intralum Dev, Perc Approach */
"037D3ZZ"  =  "1"  /* Dilation of Right Hand Artery, Percutaneous Approach */
"037D44Z"  =  "1"  /* Dilate R Hand Art w Drug-elut Intralum, Perc Endo */
"037D4DZ"  =  "1"  /* Dilation of R Hand Art with Intralum Dev, Perc Endo Approach */
"037D4ZZ"  =  "1"  /* Dilation of Right Hand Artery, Perc Endo Approach */
"037F04Z"  =  "1"  /* Dilate of L Hand Art with Drug-elut Intralum, Open Approach */
"037F0DZ"  =  "1"  /* Dilation of L Hand Art with Intralum Dev, Open Approach */
"037F0ZZ"  =  "1"  /* Dilation of Left Hand Artery, Open Approach */
"037F34Z"  =  "1"  /* Dilate of L Hand Art with Drug-elut Intralum, Perc Approach */
"037F3DZ"  =  "1"  /* Dilation of L Hand Art with Intralum Dev, Perc Approach */
"037F3ZZ"  =  "1"  /* Dilation of Left Hand Artery, Percutaneous Approach */
"037F44Z"  =  "1"  /* Dilate L Hand Art w Drug-elut Intralum, Perc Endo */
"037F4DZ"  =  "1"  /* Dilation of L Hand Art with Intralum Dev, Perc Endo Approach */
"037F4ZZ"  =  "1"  /* Dilation of Left Hand Artery, Perc Endo Approach */
"037G04Z"  =  "1"  /* Dilate Intracran Art w Drug-elut Intralum, Open */
"037G0DZ"  =  "1"  /* Dilation of Intracran Art with Intralum Dev, Open Approach */
"037G0ZZ"  =  "1"  /* Dilation of Intracranial Artery, Open Approach */
"037G34Z"  =  "1"  /* Dilate Intracran Art w Drug-elut Intralum, Perc */
"037G3DZ"  =  "1"  /* Dilation of Intracran Art with Intralum Dev, Perc Approach */
"037G3ZZ"  =  "1"  /* Dilation of Intracranial Artery, Percutaneous Approach */
"037G44Z"  =  "1"  /* Dilate Intracran Art w Drug-elut Intralum, Perc Endo */
"037G4DZ"  =  "1"  /* Dilate Intracran Art w Intralum Dev, Perc Endo */
"037G4ZZ"  =  "1"  /* Dilation of Intracranial Artery, Perc Endo Approach */
"037H04Z"  =  "1"  /* Dilate R Com Carotid w Drug-elut Intralum, Open */
"037H0DZ"  =  "1"  /* Dilation of R Com Carotid with Intralum Dev, Open Approach */
"037H0ZZ"  =  "1"  /* Dilation of Right Common Carotid Artery, Open Approach */
"037H34Z"  =  "1"  /* Dilate R Com Carotid w Drug-elut Intralum, Perc */
"037H3DZ"  =  "1"  /* Dilation of R Com Carotid with Intralum Dev, Perc Approach */
"037H3ZZ"  =  "1"  /* Dilation of Right Common Carotid Artery, Perc Approach */
"037H44Z"  =  "1"  /* Dilate R Com Carotid w Drug-elut Intralum, Perc Endo */
"037H4DZ"  =  "1"  /* Dilate R Com Carotid w Intralum Dev, Perc Endo */
"037H4ZZ"  =  "1"  /* Dilation of Right Common Carotid Artery, Perc Endo Approach */
"037J04Z"  =  "1"  /* Dilate L Com Carotid w Drug-elut Intralum, Open */
"037J0DZ"  =  "1"  /* Dilation of L Com Carotid with Intralum Dev, Open Approach */
"037J0ZZ"  =  "1"  /* Dilation of Left Common Carotid Artery, Open Approach */
"037J34Z"  =  "1"  /* Dilate L Com Carotid w Drug-elut Intralum, Perc */
"037J3DZ"  =  "1"  /* Dilation of L Com Carotid with Intralum Dev, Perc Approach */
"037J3ZZ"  =  "1"  /* Dilation of Left Common Carotid Artery, Perc Approach */
"037J44Z"  =  "1"  /* Dilate L Com Carotid w Drug-elut Intralum, Perc Endo */
"037J4DZ"  =  "1"  /* Dilate L Com Carotid w Intralum Dev, Perc Endo */
"037J4ZZ"  =  "1"  /* Dilation of Left Common Carotid Artery, Perc Endo Approach */
"037K04Z"  =  "1"  /* Dilate R Int Carotid w Drug-elut Intralum, Open */
"037K0DZ"  =  "1"  /* Dilation of R Int Carotid with Intralum Dev, Open Approach */
"037K0ZZ"  =  "1"  /* Dilation of Right Internal Carotid Artery, Open Approach */
"037K34Z"  =  "1"  /* Dilate R Int Carotid w Drug-elut Intralum, Perc */
"037K3DZ"  =  "1"  /* Dilation of R Int Carotid with Intralum Dev, Perc Approach */
"037K3ZZ"  =  "1"  /* Dilation of Right Internal Carotid Artery, Perc Approach */
"037K44Z"  =  "1"  /* Dilate R Int Carotid w Drug-elut Intralum, Perc Endo */
"037K4DZ"  =  "1"  /* Dilate R Int Carotid w Intralum Dev, Perc Endo */
"037K4ZZ"  =  "1"  /* Dilation of R Int Carotid, Perc Endo Approach */
"037L04Z"  =  "1"  /* Dilate L Int Carotid w Drug-elut Intralum, Open */
"037L0DZ"  =  "1"  /* Dilation of L Int Carotid with Intralum Dev, Open Approach */
"037L0ZZ"  =  "1"  /* Dilation of Left Internal Carotid Artery, Open Approach */
"037L34Z"  =  "1"  /* Dilate L Int Carotid w Drug-elut Intralum, Perc */
"037L3DZ"  =  "1"  /* Dilation of L Int Carotid with Intralum Dev, Perc Approach */
"037L3ZZ"  =  "1"  /* Dilation of Left Internal Carotid Artery, Perc Approach */
"037L44Z"  =  "1"  /* Dilate L Int Carotid w Drug-elut Intralum, Perc Endo */
"037L4DZ"  =  "1"  /* Dilate L Int Carotid w Intralum Dev, Perc Endo */
"037L4ZZ"  =  "1"  /* Dilation of Left Internal Carotid Artery, Perc Endo Approach */
"037M04Z"  =  "1"  /* Dilate R Ext Carotid w Drug-elut Intralum, Open */
"037M0DZ"  =  "1"  /* Dilation of R Ext Carotid with Intralum Dev, Open Approach */
"037M0ZZ"  =  "1"  /* Dilation of Right External Carotid Artery, Open Approach */
"037M34Z"  =  "1"  /* Dilate R Ext Carotid w Drug-elut Intralum, Perc */
"037M3DZ"  =  "1"  /* Dilation of R Ext Carotid with Intralum Dev, Perc Approach */
"037M3ZZ"  =  "1"  /* Dilation of Right External Carotid Artery, Perc Approach */
"037M44Z"  =  "1"  /* Dilate R Ext Carotid w Drug-elut Intralum, Perc Endo */
"037M4DZ"  =  "1"  /* Dilate R Ext Carotid w Intralum Dev, Perc Endo */
"037M4ZZ"  =  "1"  /* Dilation of R Ext Carotid, Perc Endo Approach */
"037N04Z"  =  "1"  /* Dilate L Ext Carotid w Drug-elut Intralum, Open */
"037N0DZ"  =  "1"  /* Dilation of L Ext Carotid with Intralum Dev, Open Approach */
"037N0ZZ"  =  "1"  /* Dilation of Left External Carotid Artery, Open Approach */
"037N34Z"  =  "1"  /* Dilate L Ext Carotid w Drug-elut Intralum, Perc */
"037N3DZ"  =  "1"  /* Dilation of L Ext Carotid with Intralum Dev, Perc Approach */
"037N3ZZ"  =  "1"  /* Dilation of Left External Carotid Artery, Perc Approach */
"037N44Z"  =  "1"  /* Dilate L Ext Carotid w Drug-elut Intralum, Perc Endo */
"037N4DZ"  =  "1"  /* Dilate L Ext Carotid w Intralum Dev, Perc Endo */
"037N4ZZ"  =  "1"  /* Dilation of Left External Carotid Artery, Perc Endo Approach */
"037P04Z"  =  "1"  /* Dilate R Verteb Art w Drug-elut Intralum, Open */
"037P0DZ"  =  "1"  /* Dilation of R Verteb Art with Intralum Dev, Open Approach */
"037P0ZZ"  =  "1"  /* Dilation of Right Vertebral Artery, Open Approach */
"037P34Z"  =  "1"  /* Dilate R Verteb Art w Drug-elut Intralum, Perc */
"037P3DZ"  =  "1"  /* Dilation of R Verteb Art with Intralum Dev, Perc Approach */
"037P3ZZ"  =  "1"  /* Dilation of Right Vertebral Artery, Percutaneous Approach */
"037P44Z"  =  "1"  /* Dilate R Verteb Art w Drug-elut Intralum, Perc Endo */
"037P4DZ"  =  "1"  /* Dilate of R Verteb Art with Intralum Dev, Perc Endo Approach */
"037P4ZZ"  =  "1"  /* Dilation of Right Vertebral Artery, Perc Endo Approach */
"037Q04Z"  =  "1"  /* Dilate L Verteb Art w Drug-elut Intralum, Open */
"037Q0DZ"  =  "1"  /* Dilation of L Verteb Art with Intralum Dev, Open Approach */
"037Q0ZZ"  =  "1"  /* Dilation of Left Vertebral Artery, Open Approach */
"037Q34Z"  =  "1"  /* Dilate L Verteb Art w Drug-elut Intralum, Perc */
"037Q3DZ"  =  "1"  /* Dilation of L Verteb Art with Intralum Dev, Perc Approach */
"037Q3ZZ"  =  "1"  /* Dilation of Left Vertebral Artery, Percutaneous Approach */
"037Q44Z"  =  "1"  /* Dilate L Verteb Art w Drug-elut Intralum, Perc Endo */
"037Q4DZ"  =  "1"  /* Dilate of L Verteb Art with Intralum Dev, Perc Endo Approach */
"037Q4ZZ"  =  "1"  /* Dilation of Left Vertebral Artery, Perc Endo Approach */
"037R04Z"  =  "1"  /* Dilation of Face Art with Drug-elut Intralum, Open Approach */
"037R0DZ"  =  "1"  /* Dilation of Face Artery with Intralum Dev, Open Approach */
"037R0ZZ"  =  "1"  /* Dilation of Face Artery, Open Approach */
"037R34Z"  =  "1"  /* Dilation of Face Art with Drug-elut Intralum, Perc Approach */
"037R3DZ"  =  "1"  /* Dilation of Face Artery with Intralum Dev, Perc Approach */
"037R3ZZ"  =  "1"  /* Dilation of Face Artery, Percutaneous Approach */
"037R44Z"  =  "1"  /* Dilate Face Art w Drug-elut Intralum, Perc Endo */
"037R4DZ"  =  "1"  /* Dilation of Face Art with Intralum Dev, Perc Endo Approach */
"037R4ZZ"  =  "1"  /* Dilation of Face Artery, Percutaneous Endoscopic Approach */
"037S04Z"  =  "1"  /* Dilate R Temporal Art w Drug-elut Intralum, Open */
"037S0DZ"  =  "1"  /* Dilation of R Temporal Art with Intralum Dev, Open Approach */
"037S0ZZ"  =  "1"  /* Dilation of Right Temporal Artery, Open Approach */
"037S34Z"  =  "1"  /* Dilate R Temporal Art w Drug-elut Intralum, Perc */
"037S3DZ"  =  "1"  /* Dilation of R Temporal Art with Intralum Dev, Perc Approach */
"037S3ZZ"  =  "1"  /* Dilation of Right Temporal Artery, Percutaneous Approach */
"037S44Z"  =  "1"  /* Dilate R Temporal Art w Drug-elut Intralum, Perc Endo */
"037S4DZ"  =  "1"  /* Dilate R Temporal Art w Intralum Dev, Perc Endo */
"037S4ZZ"  =  "1"  /* Dilation of Right Temporal Artery, Perc Endo Approach */
"037T04Z"  =  "1"  /* Dilate L Temporal Art w Drug-elut Intralum, Open */
"037T0DZ"  =  "1"  /* Dilation of L Temporal Art with Intralum Dev, Open Approach */
"037T0ZZ"  =  "1"  /* Dilation of Left Temporal Artery, Open Approach */
"037T34Z"  =  "1"  /* Dilate L Temporal Art w Drug-elut Intralum, Perc */
"037T3DZ"  =  "1"  /* Dilation of L Temporal Art with Intralum Dev, Perc Approach */
"037T3ZZ"  =  "1"  /* Dilation of Left Temporal Artery, Percutaneous Approach */
"037T44Z"  =  "1"  /* Dilate L Temporal Art w Drug-elut Intralum, Perc Endo */
"037T4DZ"  =  "1"  /* Dilate L Temporal Art w Intralum Dev, Perc Endo */
"037T4ZZ"  =  "1"  /* Dilation of Left Temporal Artery, Perc Endo Approach */
"037U04Z"  =  "1"  /* Dilate R Thyroid Art w Drug-elut Intralum, Open */
"037U0DZ"  =  "1"  /* Dilation of R Thyroid Art with Intralum Dev, Open Approach */
"037U0ZZ"  =  "1"  /* Dilation of Right Thyroid Artery, Open Approach */
"037U34Z"  =  "1"  /* Dilate R Thyroid Art w Drug-elut Intralum, Perc */
"037U3DZ"  =  "1"  /* Dilation of R Thyroid Art with Intralum Dev, Perc Approach */
"037U3ZZ"  =  "1"  /* Dilation of Right Thyroid Artery, Percutaneous Approach */
"037U44Z"  =  "1"  /* Dilate R Thyroid Art w Drug-elut Intralum, Perc Endo */
"037U4DZ"  =  "1"  /* Dilate R Thyroid Art w Intralum Dev, Perc Endo */
"037U4ZZ"  =  "1"  /* Dilation of Right Thyroid Artery, Perc Endo Approach */
"037V04Z"  =  "1"  /* Dilate L Thyroid Art w Drug-elut Intralum, Open */
"037V0DZ"  =  "1"  /* Dilation of L Thyroid Art with Intralum Dev, Open Approach */
"037V0ZZ"  =  "1"  /* Dilation of Left Thyroid Artery, Open Approach */
"037V34Z"  =  "1"  /* Dilate L Thyroid Art w Drug-elut Intralum, Perc */
"037V3DZ"  =  "1"  /* Dilation of L Thyroid Art with Intralum Dev, Perc Approach */
"037V3ZZ"  =  "1"  /* Dilation of Left Thyroid Artery, Percutaneous Approach */
"037V44Z"  =  "1"  /* Dilate L Thyroid Art w Drug-elut Intralum, Perc Endo */
"037V4DZ"  =  "1"  /* Dilate L Thyroid Art w Intralum Dev, Perc Endo */
"037V4ZZ"  =  "1"  /* Dilation of Left Thyroid Artery, Perc Endo Approach */
"037Y04Z"  =  "1"  /* Dilation of Up Art with Drug-elut Intralum, Open Approach */
"037Y0DZ"  =  "1"  /* Dilation of Upper Artery with Intralum Dev, Open Approach */
"037Y0ZZ"  =  "1"  /* Dilation of Upper Artery, Open Approach */
"037Y34Z"  =  "1"  /* Dilation of Up Art with Drug-elut Intralum, Perc Approach */
"037Y3DZ"  =  "1"  /* Dilation of Upper Artery with Intralum Dev, Perc Approach */
"037Y3ZZ"  =  "1"  /* Dilation of Upper Artery, Percutaneous Approach */
"037Y44Z"  =  "1"  /* Dilate of Up Art with Drug-elut Intralum, Perc Endo Approach */
"037Y4DZ"  =  "1"  /* Dilation of Up Art with Intralum Dev, Perc Endo Approach */
"037Y4ZZ"  =  "1"  /* Dilation of Upper Artery, Percutaneous Endoscopic Approach */
"03900ZX"  =  "1"  /* Drainage of R Int Mamm Art, Open Approach, Diagn */
"03903ZX"  =  "1"  /* Drainage of R Int Mamm Art, Perc Approach, Diagn */
"03904ZX"  =  "1"  /* Drainage of R Int Mamm Art, Perc Endo Approach, Diagn */
"03910ZX"  =  "1"  /* Drainage of L Int Mamm Art, Open Approach, Diagn */
"03913ZX"  =  "1"  /* Drainage of L Int Mamm Art, Perc Approach, Diagn */
"03914ZX"  =  "1"  /* Drainage of L Int Mamm Art, Perc Endo Approach, Diagn */
"03920ZX"  =  "1"  /* Drainage of Innominate Artery, Open Approach, Diagnostic */
"03923ZX"  =  "1"  /* Drainage of Innominate Artery, Percutaneous Approach, Diagn */
"03924ZX"  =  "1"  /* Drainage of Innominate Artery, Perc Endo Approach, Diagn */
"03930ZX"  =  "1"  /* Drainage of Right Subclavian Artery, Open Approach, Diagn */
"03933ZX"  =  "1"  /* Drainage of Right Subclavian Artery, Perc Approach, Diagn */
"03934ZX"  =  "1"  /* Drainage of R Subclav Art, Perc Endo Approach, Diagn */
"03940ZX"  =  "1"  /* Drainage of Left Subclavian Artery, Open Approach, Diagn */
"03943ZX"  =  "1"  /* Drainage of Left Subclavian Artery, Perc Approach, Diagn */
"03944ZX"  =  "1"  /* Drainage of L Subclav Art, Perc Endo Approach, Diagn */
"03950ZX"  =  "1"  /* Drainage of Right Axillary Artery, Open Approach, Diagnostic */
"03953ZX"  =  "1"  /* Drainage of Right Axillary Artery, Perc Approach, Diagn */
"03954ZX"  =  "1"  /* Drainage of Right Axillary Artery, Perc Endo Approach, Diagn */
"03960ZX"  =  "1"  /* Drainage of Left Axillary Artery, Open Approach, Diagnostic */
"03963ZX"  =  "1"  /* Drainage of Left Axillary Artery, Perc Approach, Diagn */
"03964ZX"  =  "1"  /* Drainage of Left Axillary Artery, Perc Endo Approach, Diagn */
"03970ZX"  =  "1"  /* Drainage of Right Brachial Artery, Open Approach, Diagnostic */
"03973ZX"  =  "1"  /* Drainage of Right Brachial Artery, Perc Approach, Diagn */
"03974ZX"  =  "1"  /* Drainage of Right Brachial Artery, Perc Endo Approach, Diagn */
"03980ZX"  =  "1"  /* Drainage of Left Brachial Artery, Open Approach, Diagnostic */
"03983ZX"  =  "1"  /* Drainage of Left Brachial Artery, Perc Approach, Diagn */
"03984ZX"  =  "1"  /* Drainage of Left Brachial Artery, Perc Endo Approach, Diagn */
"03990ZX"  =  "1"  /* Drainage of Right Ulnar Artery, Open Approach, Diagnostic */
"03993ZX"  =  "1"  /* Drainage of Right Ulnar Artery, Percutaneous Approach, Diagn */
"03994ZX"  =  "1"  /* Drainage of Right Ulnar Artery, Perc Endo Approach, Diagn */
"039A0ZX"  =  "1"  /* Drainage of Left Ulnar Artery, Open Approach, Diagnostic */
"039A3ZX"  =  "1"  /* Drainage of Left Ulnar Artery, Percutaneous Approach, Diagn */
"039A4ZX"  =  "1"  /* Drainage of Left Ulnar Artery, Perc Endo Approach, Diagn */
"039B0ZX"  =  "1"  /* Drainage of Right Radial Artery, Open Approach, Diagnostic */
"039B3ZX"  =  "1"  /* Drainage of Right Radial Artery, Perc Approach, Diagn */
"039B4ZX"  =  "1"  /* Drainage of Right Radial Artery, Perc Endo Approach, Diagn */
"039C0ZX"  =  "1"  /* Drainage of Left Radial Artery, Open Approach, Diagnostic */
"039C3ZX"  =  "1"  /* Drainage of Left Radial Artery, Percutaneous Approach, Diagn */
"039C4ZX"  =  "1"  /* Drainage of Left Radial Artery, Perc Endo Approach, Diagn */
"039D0ZX"  =  "1"  /* Drainage of Right Hand Artery, Open Approach, Diagnostic */
"039D3ZX"  =  "1"  /* Drainage of Right Hand Artery, Percutaneous Approach, Diagn */
"039D4ZX"  =  "1"  /* Drainage of Right Hand Artery, Perc Endo Approach, Diagn */
"039F0ZX"  =  "1"  /* Drainage of Left Hand Artery, Open Approach, Diagnostic */
"039F3ZX"  =  "1"  /* Drainage of Left Hand Artery, Percutaneous Approach, Diagn */
"039F4ZX"  =  "1"  /* Drainage of Left Hand Artery, Perc Endo Approach, Diagn */
"039G0ZX"  =  "1"  /* Drainage of Intracranial Artery, Open Approach, Diagnostic */
"039G3ZX"  =  "1"  /* Drainage of Intracranial Artery, Perc Approach, Diagn */
"039G4ZX"  =  "1"  /* Drainage of Intracranial Artery, Perc Endo Approach, Diagn */
"039H0ZX"  =  "1"  /* Drainage of R Com Carotid, Open Approach, Diagn */
"039H3ZX"  =  "1"  /* Drainage of R Com Carotid, Perc Approach, Diagn */
"039H4ZX"  =  "1"  /* Drainage of R Com Carotid, Perc Endo Approach, Diagn */
"039J0ZX"  =  "1"  /* Drainage of Left Common Carotid Artery, Open Approach, Diagn */
"039J3ZX"  =  "1"  /* Drainage of Left Common Carotid Artery, Perc Approach, Diagn */
"039J4ZX"  =  "1"  /* Drainage of L Com Carotid, Perc Endo Approach, Diagn */
"039K0ZX"  =  "1"  /* Drainage of R Int Carotid, Open Approach, Diagn */
"039K3ZX"  =  "1"  /* Drainage of R Int Carotid, Perc Approach, Diagn */
"039K4ZX"  =  "1"  /* Drainage of R Int Carotid, Perc Endo Approach, Diagn */
"039L0ZX"  =  "1"  /* Drainage of L Int Carotid, Open Approach, Diagn */
"039L3ZX"  =  "1"  /* Drainage of L Int Carotid, Perc Approach, Diagn */
"039L4ZX"  =  "1"  /* Drainage of L Int Carotid, Perc Endo Approach, Diagn */
"039M0ZX"  =  "1"  /* Drainage of R Ext Carotid, Open Approach, Diagn */
"039M3ZX"  =  "1"  /* Drainage of R Ext Carotid, Perc Approach, Diagn */
"039M4ZX"  =  "1"  /* Drainage of R Ext Carotid, Perc Endo Approach, Diagn */
"039N0ZX"  =  "1"  /* Drainage of L Ext Carotid, Open Approach, Diagn */
"039N3ZX"  =  "1"  /* Drainage of L Ext Carotid, Perc Approach, Diagn */
"039N4ZX"  =  "1"  /* Drainage of L Ext Carotid, Perc Endo Approach, Diagn */
"039P0ZX"  =  "1"  /* Drainage of Right Vertebral Artery, Open Approach, Diagn */
"039P3ZX"  =  "1"  /* Drainage of Right Vertebral Artery, Perc Approach, Diagn */
"039P4ZX"  =  "1"  /* Drainage of R Verteb Art, Perc Endo Approach, Diagn */
"039Q0ZX"  =  "1"  /* Drainage of Left Vertebral Artery, Open Approach, Diagnostic */
"039Q3ZX"  =  "1"  /* Drainage of Left Vertebral Artery, Perc Approach, Diagn */
"039Q4ZX"  =  "1"  /* Drainage of Left Vertebral Artery, Perc Endo Approach, Diagn */
"039R0ZX"  =  "1"  /* Drainage of Face Artery, Open Approach, Diagnostic */
"039R3ZX"  =  "1"  /* Drainage of Face Artery, Percutaneous Approach, Diagnostic */
"039R4ZX"  =  "1"  /* Drainage of Face Artery, Perc Endo Approach, Diagn */
"039S0ZX"  =  "1"  /* Drainage of Right Temporal Artery, Open Approach, Diagnostic */
"039S3ZX"  =  "1"  /* Drainage of Right Temporal Artery, Perc Approach, Diagn */
"039S4ZX"  =  "1"  /* Drainage of Right Temporal Artery, Perc Endo Approach, Diagn */
"039T0ZX"  =  "1"  /* Drainage of Left Temporal Artery, Open Approach, Diagnostic */
"039T3ZX"  =  "1"  /* Drainage of Left Temporal Artery, Perc Approach, Diagn */
"039T4ZX"  =  "1"  /* Drainage of Left Temporal Artery, Perc Endo Approach, Diagn */
"039U0ZX"  =  "1"  /* Drainage of Right Thyroid Artery, Open Approach, Diagnostic */
"039U3ZX"  =  "1"  /* Drainage of Right Thyroid Artery, Perc Approach, Diagn */
"039U4ZX"  =  "1"  /* Drainage of Right Thyroid Artery, Perc Endo Approach, Diagn */
"039V0ZX"  =  "1"  /* Drainage of Left Thyroid Artery, Open Approach, Diagnostic */
"039V3ZX"  =  "1"  /* Drainage of Left Thyroid Artery, Perc Approach, Diagn */
"039V4ZX"  =  "1"  /* Drainage of Left Thyroid Artery, Perc Endo Approach, Diagn */
"039Y0ZX"  =  "1"  /* Drainage of Upper Artery, Open Approach, Diagnostic */
"039Y3ZX"  =  "1"  /* Drainage of Upper Artery, Percutaneous Approach, Diagnostic */
"039Y4ZX"  =  "1"  /* Drainage of Upper Artery, Perc Endo Approach, Diagn */
"03B00ZX"  =  "1"  /* Excision of R Int Mamm Art, Open Approach, Diagn */
"03B00ZZ"  =  "1"  /* Excision of Right Internal Mammary Artery, Open Approach */
"03B03ZX"  =  "1"  /* Excision of R Int Mamm Art, Perc Approach, Diagn */
"03B03ZZ"  =  "1"  /* Excision of Right Internal Mammary Artery, Perc Approach */
"03B04ZX"  =  "1"  /* Excision of R Int Mamm Art, Perc Endo Approach, Diagn */
"03B04ZZ"  =  "1"  /* Excision of R Int Mamm Art, Perc Endo Approach */
"03B10ZX"  =  "1"  /* Excision of L Int Mamm Art, Open Approach, Diagn */
"03B10ZZ"  =  "1"  /* Excision of Left Internal Mammary Artery, Open Approach */
"03B13ZX"  =  "1"  /* Excision of L Int Mamm Art, Perc Approach, Diagn */
"03B13ZZ"  =  "1"  /* Excision of Left Internal Mammary Artery, Perc Approach */
"03B14ZX"  =  "1"  /* Excision of L Int Mamm Art, Perc Endo Approach, Diagn */
"03B14ZZ"  =  "1"  /* Excision of Left Internal Mammary Artery, Perc Endo Approach */
"03B20ZX"  =  "1"  /* Excision of Innominate Artery, Open Approach, Diagnostic */
"03B20ZZ"  =  "1"  /* Excision of Innominate Artery, Open Approach */
"03B23ZX"  =  "1"  /* Excision of Innominate Artery, Percutaneous Approach, Diagn */
"03B23ZZ"  =  "1"  /* Excision of Innominate Artery, Percutaneous Approach */
"03B24ZX"  =  "1"  /* Excision of Innominate Artery, Perc Endo Approach, Diagn */
"03B24ZZ"  =  "1"  /* Excision of Innominate Artery, Perc Endo Approach */
"03B30ZX"  =  "1"  /* Excision of Right Subclavian Artery, Open Approach, Diagn */
"03B30ZZ"  =  "1"  /* Excision of Right Subclavian Artery, Open Approach */
"03B33ZX"  =  "1"  /* Excision of Right Subclavian Artery, Perc Approach, Diagn */
"03B33ZZ"  =  "1"  /* Excision of Right Subclavian Artery, Percutaneous Approach */
"03B34ZX"  =  "1"  /* Excision of R Subclav Art, Perc Endo Approach, Diagn */
"03B34ZZ"  =  "1"  /* Excision of Right Subclavian Artery, Perc Endo Approach */
"03B40ZX"  =  "1"  /* Excision of Left Subclavian Artery, Open Approach, Diagn */
"03B40ZZ"  =  "1"  /* Excision of Left Subclavian Artery, Open Approach */
"03B43ZX"  =  "1"  /* Excision of Left Subclavian Artery, Perc Approach, Diagn */
"03B43ZZ"  =  "1"  /* Excision of Left Subclavian Artery, Percutaneous Approach */
"03B44ZX"  =  "1"  /* Excision of L Subclav Art, Perc Endo Approach, Diagn */
"03B44ZZ"  =  "1"  /* Excision of Left Subclavian Artery, Perc Endo Approach */
"03B50ZX"  =  "1"  /* Excision of Right Axillary Artery, Open Approach, Diagnostic */
"03B50ZZ"  =  "1"  /* Excision of Right Axillary Artery, Open Approach */
"03B53ZX"  =  "1"  /* Excision of Right Axillary Artery, Perc Approach, Diagn */
"03B53ZZ"  =  "1"  /* Excision of Right Axillary Artery, Percutaneous Approach */
"03B54ZX"  =  "1"  /* Excision of Right Axillary Artery, Perc Endo Approach, Diagn */
"03B54ZZ"  =  "1"  /* Excision of Right Axillary Artery, Perc Endo Approach */
"03B60ZX"  =  "1"  /* Excision of Left Axillary Artery, Open Approach, Diagnostic */
"03B60ZZ"  =  "1"  /* Excision of Left Axillary Artery, Open Approach */
"03B63ZX"  =  "1"  /* Excision of Left Axillary Artery, Perc Approach, Diagn */
"03B63ZZ"  =  "1"  /* Excision of Left Axillary Artery, Percutaneous Approach */
"03B64ZX"  =  "1"  /* Excision of Left Axillary Artery, Perc Endo Approach, Diagn */
"03B64ZZ"  =  "1"  /* Excision of Left Axillary Artery, Perc Endo Approach */
"03B70ZX"  =  "1"  /* Excision of Right Brachial Artery, Open Approach, Diagnostic */
"03B70ZZ"  =  "1"  /* Excision of Right Brachial Artery, Open Approach */
"03B73ZX"  =  "1"  /* Excision of Right Brachial Artery, Perc Approach, Diagn */
"03B73ZZ"  =  "1"  /* Excision of Right Brachial Artery, Percutaneous Approach */
"03B74ZX"  =  "1"  /* Excision of Right Brachial Artery, Perc Endo Approach, Diagn */
"03B74ZZ"  =  "1"  /* Excision of Right Brachial Artery, Perc Endo Approach */
"03B80ZX"  =  "1"  /* Excision of Left Brachial Artery, Open Approach, Diagnostic */
"03B80ZZ"  =  "1"  /* Excision of Left Brachial Artery, Open Approach */
"03B83ZX"  =  "1"  /* Excision of Left Brachial Artery, Perc Approach, Diagn */
"03B83ZZ"  =  "1"  /* Excision of Left Brachial Artery, Percutaneous Approach */
"03B84ZX"  =  "1"  /* Excision of Left Brachial Artery, Perc Endo Approach, Diagn */
"03B84ZZ"  =  "1"  /* Excision of Left Brachial Artery, Perc Endo Approach */
"03B90ZX"  =  "1"  /* Excision of Right Ulnar Artery, Open Approach, Diagnostic */
"03B90ZZ"  =  "1"  /* Excision of Right Ulnar Artery, Open Approach */
"03B93ZX"  =  "1"  /* Excision of Right Ulnar Artery, Percutaneous Approach, Diagn */
"03B93ZZ"  =  "1"  /* Excision of Right Ulnar Artery, Percutaneous Approach */
"03B94ZX"  =  "1"  /* Excision of Right Ulnar Artery, Perc Endo Approach, Diagn */
"03B94ZZ"  =  "1"  /* Excision of Right Ulnar Artery, Perc Endo Approach */
"03BA0ZX"  =  "1"  /* Excision of Left Ulnar Artery, Open Approach, Diagnostic */
"03BA0ZZ"  =  "1"  /* Excision of Left Ulnar Artery, Open Approach */
"03BA3ZX"  =  "1"  /* Excision of Left Ulnar Artery, Percutaneous Approach, Diagn */
"03BA3ZZ"  =  "1"  /* Excision of Left Ulnar Artery, Percutaneous Approach */
"03BA4ZX"  =  "1"  /* Excision of Left Ulnar Artery, Perc Endo Approach, Diagn */
"03BA4ZZ"  =  "1"  /* Excision of Left Ulnar Artery, Perc Endo Approach */
"03BB0ZX"  =  "1"  /* Excision of Right Radial Artery, Open Approach, Diagnostic */
"03BB0ZZ"  =  "1"  /* Excision of Right Radial Artery, Open Approach */
"03BB3ZX"  =  "1"  /* Excision of Right Radial Artery, Perc Approach, Diagn */
"03BB3ZZ"  =  "1"  /* Excision of Right Radial Artery, Percutaneous Approach */
"03BB4ZX"  =  "1"  /* Excision of Right Radial Artery, Perc Endo Approach, Diagn */
"03BB4ZZ"  =  "1"  /* Excision of Right Radial Artery, Perc Endo Approach */
"03BC0ZX"  =  "1"  /* Excision of Left Radial Artery, Open Approach, Diagnostic */
"03BC0ZZ"  =  "1"  /* Excision of Left Radial Artery, Open Approach */
"03BC3ZX"  =  "1"  /* Excision of Left Radial Artery, Percutaneous Approach, Diagn */
"03BC3ZZ"  =  "1"  /* Excision of Left Radial Artery, Percutaneous Approach */
"03BC4ZX"  =  "1"  /* Excision of Left Radial Artery, Perc Endo Approach, Diagn */
"03BC4ZZ"  =  "1"  /* Excision of Left Radial Artery, Perc Endo Approach */
"03BD0ZX"  =  "1"  /* Excision of Right Hand Artery, Open Approach, Diagnostic */
"03BD0ZZ"  =  "1"  /* Excision of Right Hand Artery, Open Approach */
"03BD3ZX"  =  "1"  /* Excision of Right Hand Artery, Percutaneous Approach, Diagn */
"03BD3ZZ"  =  "1"  /* Excision of Right Hand Artery, Percutaneous Approach */
"03BD4ZX"  =  "1"  /* Excision of Right Hand Artery, Perc Endo Approach, Diagn */
"03BD4ZZ"  =  "1"  /* Excision of Right Hand Artery, Perc Endo Approach */
"03BF0ZX"  =  "1"  /* Excision of Left Hand Artery, Open Approach, Diagnostic */
"03BF0ZZ"  =  "1"  /* Excision of Left Hand Artery, Open Approach */
"03BF3ZX"  =  "1"  /* Excision of Left Hand Artery, Percutaneous Approach, Diagn */
"03BF3ZZ"  =  "1"  /* Excision of Left Hand Artery, Percutaneous Approach */
"03BF4ZX"  =  "1"  /* Excision of Left Hand Artery, Perc Endo Approach, Diagn */
"03BF4ZZ"  =  "1"  /* Excision of Left Hand Artery, Perc Endo Approach */
"03BG0ZX"  =  "1"  /* Excision of Intracranial Artery, Open Approach, Diagnostic */
"03BG0ZZ"  =  "1"  /* Excision of Intracranial Artery, Open Approach */
"03BG3ZX"  =  "1"  /* Excision of Intracranial Artery, Perc Approach, Diagn */
"03BG3ZZ"  =  "1"  /* Excision of Intracranial Artery, Percutaneous Approach */
"03BG4ZX"  =  "1"  /* Excision of Intracranial Artery, Perc Endo Approach, Diagn */
"03BG4ZZ"  =  "1"  /* Excision of Intracranial Artery, Perc Endo Approach */
"03BH0ZX"  =  "1"  /* Excision of R Com Carotid, Open Approach, Diagn */
"03BH0ZZ"  =  "1"  /* Excision of Right Common Carotid Artery, Open Approach */
"03BH3ZX"  =  "1"  /* Excision of R Com Carotid, Perc Approach, Diagn */
"03BH3ZZ"  =  "1"  /* Excision of Right Common Carotid Artery, Perc Approach */
"03BH4ZX"  =  "1"  /* Excision of R Com Carotid, Perc Endo Approach, Diagn */
"03BH4ZZ"  =  "1"  /* Excision of Right Common Carotid Artery, Perc Endo Approach */
"03BJ0ZX"  =  "1"  /* Excision of Left Common Carotid Artery, Open Approach, Diagn */
"03BJ0ZZ"  =  "1"  /* Excision of Left Common Carotid Artery, Open Approach */
"03BJ3ZX"  =  "1"  /* Excision of Left Common Carotid Artery, Perc Approach, Diagn */
"03BJ3ZZ"  =  "1"  /* Excision of Left Common Carotid Artery, Perc Approach */
"03BJ4ZX"  =  "1"  /* Excision of L Com Carotid, Perc Endo Approach, Diagn */
"03BJ4ZZ"  =  "1"  /* Excision of Left Common Carotid Artery, Perc Endo Approach */
"03BK0ZX"  =  "1"  /* Excision of R Int Carotid, Open Approach, Diagn */
"03BK0ZZ"  =  "1"  /* Excision of Right Internal Carotid Artery, Open Approach */
"03BK3ZX"  =  "1"  /* Excision of R Int Carotid, Perc Approach, Diagn */
"03BK3ZZ"  =  "1"  /* Excision of Right Internal Carotid Artery, Perc Approach */
"03BK4ZX"  =  "1"  /* Excision of R Int Carotid, Perc Endo Approach, Diagn */
"03BK4ZZ"  =  "1"  /* Excision of R Int Carotid, Perc Endo Approach */
"03BL0ZX"  =  "1"  /* Excision of L Int Carotid, Open Approach, Diagn */
"03BL0ZZ"  =  "1"  /* Excision of Left Internal Carotid Artery, Open Approach */
"03BL3ZX"  =  "1"  /* Excision of L Int Carotid, Perc Approach, Diagn */
"03BL3ZZ"  =  "1"  /* Excision of Left Internal Carotid Artery, Perc Approach */
"03BL4ZX"  =  "1"  /* Excision of L Int Carotid, Perc Endo Approach, Diagn */
"03BL4ZZ"  =  "1"  /* Excision of Left Internal Carotid Artery, Perc Endo Approach */
"03BM0ZX"  =  "1"  /* Excision of R Ext Carotid, Open Approach, Diagn */
"03BM0ZZ"  =  "1"  /* Excision of Right External Carotid Artery, Open Approach */
"03BM3ZX"  =  "1"  /* Excision of R Ext Carotid, Perc Approach, Diagn */
"03BM3ZZ"  =  "1"  /* Excision of Right External Carotid Artery, Perc Approach */
"03BM4ZX"  =  "1"  /* Excision of R Ext Carotid, Perc Endo Approach, Diagn */
"03BM4ZZ"  =  "1"  /* Excision of R Ext Carotid, Perc Endo Approach */
"03BN0ZX"  =  "1"  /* Excision of L Ext Carotid, Open Approach, Diagn */
"03BN0ZZ"  =  "1"  /* Excision of Left External Carotid Artery, Open Approach */
"03BN3ZX"  =  "1"  /* Excision of L Ext Carotid, Perc Approach, Diagn */
"03BN3ZZ"  =  "1"  /* Excision of Left External Carotid Artery, Perc Approach */
"03BN4ZX"  =  "1"  /* Excision of L Ext Carotid, Perc Endo Approach, Diagn */
"03BN4ZZ"  =  "1"  /* Excision of Left External Carotid Artery, Perc Endo Approach */
"03BP0ZX"  =  "1"  /* Excision of Right Vertebral Artery, Open Approach, Diagn */
"03BP0ZZ"  =  "1"  /* Excision of Right Vertebral Artery, Open Approach */
"03BP3ZX"  =  "1"  /* Excision of Right Vertebral Artery, Perc Approach, Diagn */
"03BP3ZZ"  =  "1"  /* Excision of Right Vertebral Artery, Percutaneous Approach */
"03BP4ZX"  =  "1"  /* Excision of R Verteb Art, Perc Endo Approach, Diagn */
"03BP4ZZ"  =  "1"  /* Excision of Right Vertebral Artery, Perc Endo Approach */
"03BQ0ZX"  =  "1"  /* Excision of Left Vertebral Artery, Open Approach, Diagnostic */
"03BQ0ZZ"  =  "1"  /* Excision of Left Vertebral Artery, Open Approach */
"03BQ3ZX"  =  "1"  /* Excision of Left Vertebral Artery, Perc Approach, Diagn */
"03BQ3ZZ"  =  "1"  /* Excision of Left Vertebral Artery, Percutaneous Approach */
"03BQ4ZX"  =  "1"  /* Excision of Left Vertebral Artery, Perc Endo Approach, Diagn */
"03BQ4ZZ"  =  "1"  /* Excision of Left Vertebral Artery, Perc Endo Approach */
"03BR0ZX"  =  "1"  /* Excision of Face Artery, Open Approach, Diagnostic */
"03BR0ZZ"  =  "1"  /* Excision of Face Artery, Open Approach */
"03BR3ZX"  =  "1"  /* Excision of Face Artery, Percutaneous Approach, Diagnostic */
"03BR3ZZ"  =  "1"  /* Excision of Face Artery, Percutaneous Approach */
"03BR4ZX"  =  "1"  /* Excision of Face Artery, Perc Endo Approach, Diagn */
"03BR4ZZ"  =  "1"  /* Excision of Face Artery, Percutaneous Endoscopic Approach */
"03BS0ZX"  =  "1"  /* Excision of Right Temporal Artery, Open Approach, Diagnostic */
"03BS0ZZ"  =  "1"  /* Excision of Right Temporal Artery, Open Approach */
"03BS3ZX"  =  "1"  /* Excision of Right Temporal Artery, Perc Approach, Diagn */
"03BS3ZZ"  =  "1"  /* Excision of Right Temporal Artery, Percutaneous Approach */
"03BS4ZX"  =  "1"  /* Excision of Right Temporal Artery, Perc Endo Approach, Diagn */
"03BS4ZZ"  =  "1"  /* Excision of Right Temporal Artery, Perc Endo Approach */
"03BT0ZX"  =  "1"  /* Excision of Left Temporal Artery, Open Approach, Diagnostic */
"03BT0ZZ"  =  "1"  /* Excision of Left Temporal Artery, Open Approach */
"03BT3ZX"  =  "1"  /* Excision of Left Temporal Artery, Perc Approach, Diagn */
"03BT3ZZ"  =  "1"  /* Excision of Left Temporal Artery, Percutaneous Approach */
"03BT4ZX"  =  "1"  /* Excision of Left Temporal Artery, Perc Endo Approach, Diagn */
"03BT4ZZ"  =  "1"  /* Excision of Left Temporal Artery, Perc Endo Approach */
"03BU0ZX"  =  "1"  /* Excision of Right Thyroid Artery, Open Approach, Diagnostic */
"03BU0ZZ"  =  "1"  /* Excision of Right Thyroid Artery, Open Approach */
"03BU3ZX"  =  "1"  /* Excision of Right Thyroid Artery, Perc Approach, Diagn */
"03BU3ZZ"  =  "1"  /* Excision of Right Thyroid Artery, Percutaneous Approach */
"03BU4ZX"  =  "1"  /* Excision of Right Thyroid Artery, Perc Endo Approach, Diagn */
"03BU4ZZ"  =  "1"  /* Excision of Right Thyroid Artery, Perc Endo Approach */
"03BV0ZX"  =  "1"  /* Excision of Left Thyroid Artery, Open Approach, Diagnostic */
"03BV0ZZ"  =  "1"  /* Excision of Left Thyroid Artery, Open Approach */
"03BV3ZX"  =  "1"  /* Excision of Left Thyroid Artery, Perc Approach, Diagn */
"03BV3ZZ"  =  "1"  /* Excision of Left Thyroid Artery, Percutaneous Approach */
"03BV4ZX"  =  "1"  /* Excision of Left Thyroid Artery, Perc Endo Approach, Diagn */
"03BV4ZZ"  =  "1"  /* Excision of Left Thyroid Artery, Perc Endo Approach */
"03BY0ZX"  =  "1"  /* Excision of Upper Artery, Open Approach, Diagnostic */
"03BY0ZZ"  =  "1"  /* Excision of Upper Artery, Open Approach */
"03BY3ZX"  =  "1"  /* Excision of Upper Artery, Percutaneous Approach, Diagnostic */
"03BY3ZZ"  =  "1"  /* Excision of Upper Artery, Percutaneous Approach */
"03BY4ZX"  =  "1"  /* Excision of Upper Artery, Perc Endo Approach, Diagn */
"03BY4ZZ"  =  "1"  /* Excision of Upper Artery, Percutaneous Endoscopic Approach */
"03C00ZZ"  =  "1"  /* Extirpation of Matter from R Int Mamm Art, Open Approach */
"03C03ZZ"  =  "1"  /* Extirpation of Matter from R Int Mamm Art, Perc Approach */
"03C04ZZ"  =  "1"  /* Extirpate of Matter from R Int Mamm Art, Perc Endo Approach */
"03C10ZZ"  =  "1"  /* Extirpation of Matter from L Int Mamm Art, Open Approach */
"03C13ZZ"  =  "1"  /* Extirpation of Matter from L Int Mamm Art, Perc Approach */
"03C14ZZ"  =  "1"  /* Extirpate of Matter from L Int Mamm Art, Perc Endo Approach */
"03C20ZZ"  =  "1"  /* Extirpation of Matter from Innominate Artery, Open Approach */
"03C23ZZ"  =  "1"  /* Extirpation of Matter from Innominate Artery, Perc Approach */
"03C24ZZ"  =  "1"  /* Extirpation of Matter from Innom Art, Perc Endo Approach */
"03C30ZZ"  =  "1"  /* Extirpation of Matter from R Subclav Art, Open Approach */
"03C33ZZ"  =  "1"  /* Extirpation of Matter from R Subclav Art, Perc Approach */
"03C34ZZ"  =  "1"  /* Extirpation of Matter from R Subclav Art, Perc Endo Approach */
"03C40ZZ"  =  "1"  /* Extirpation of Matter from L Subclav Art, Open Approach */
"03C43ZZ"  =  "1"  /* Extirpation of Matter from L Subclav Art, Perc Approach */
"03C44ZZ"  =  "1"  /* Extirpation of Matter from L Subclav Art, Perc Endo Approach */
"03C50ZZ"  =  "1"  /* Extirpation of Matter from R Axilla Art, Open Approach */
"03C53ZZ"  =  "1"  /* Extirpation of Matter from R Axilla Art, Perc Approach */
"03C54ZZ"  =  "1"  /* Extirpation of Matter from R Axilla Art, Perc Endo Approach */
"03C60ZZ"  =  "1"  /* Extirpation of Matter from L Axilla Art, Open Approach */
"03C63ZZ"  =  "1"  /* Extirpation of Matter from L Axilla Art, Perc Approach */
"03C64ZZ"  =  "1"  /* Extirpation of Matter from L Axilla Art, Perc Endo Approach */
"03C70ZZ"  =  "1"  /* Extirpation of Matter from R Brach Art, Open Approach */
"03C73ZZ"  =  "1"  /* Extirpation of Matter from R Brach Art, Perc Approach */
"03C74ZZ"  =  "1"  /* Extirpation of Matter from R Brach Art, Perc Endo Approach */
"03C80ZZ"  =  "1"  /* Extirpation of Matter from L Brach Art, Open Approach */
"03C83ZZ"  =  "1"  /* Extirpation of Matter from L Brach Art, Perc Approach */
"03C84ZZ"  =  "1"  /* Extirpation of Matter from L Brach Art, Perc Endo Approach */
"03C90ZZ"  =  "1"  /* Extirpation of Matter from Right Ulnar Artery, Open Approach */
"03C93ZZ"  =  "1"  /* Extirpation of Matter from Right Ulnar Artery, Perc Approach */
"03C94ZZ"  =  "1"  /* Extirpation of Matter from R Ulnar Art, Perc Endo Approach */
"03CA0ZZ"  =  "1"  /* Extirpation of Matter from Left Ulnar Artery, Open Approach */
"03CA3ZZ"  =  "1"  /* Extirpation of Matter from Left Ulnar Artery, Perc Approach */
"03CA4ZZ"  =  "1"  /* Extirpation of Matter from L Ulnar Art, Perc Endo Approach */
"03CB0ZZ"  =  "1"  /* Extirpation of Matter from R Radial Art, Open Approach */
"03CB3ZZ"  =  "1"  /* Extirpation of Matter from R Radial Art, Perc Approach */
"03CB4ZZ"  =  "1"  /* Extirpation of Matter from R Radial Art, Perc Endo Approach */
"03CC0ZZ"  =  "1"  /* Extirpation of Matter from Left Radial Artery, Open Approach */
"03CC3ZZ"  =  "1"  /* Extirpation of Matter from Left Radial Artery, Perc Approach */
"03CC4ZZ"  =  "1"  /* Extirpation of Matter from L Radial Art, Perc Endo Approach */
"03CD0ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Artery, Open Approach */
"03CD3ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Artery, Perc Approach */
"03CD4ZZ"  =  "1"  /* Extirpation of Matter from R Hand Art, Perc Endo Approach */
"03CF0ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Artery, Open Approach */
"03CF3ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Artery, Perc Approach */
"03CF4ZZ"  =  "1"  /* Extirpation of Matter from L Hand Art, Perc Endo Approach */
"03CG0ZZ"  =  "1"  /* Extirpation of Matter from Intracran Art, Open Approach */
"03CG3ZZ"  =  "1"  /* Extirpation of Matter from Intracran Art, Perc Approach */
"03CG4ZZ"  =  "1"  /* Extirpation of Matter from Intracran Art, Perc Endo Approach */
"03CH0ZZ"  =  "1"  /* Extirpation of Matter from R Com Carotid, Open Approach */
"03CH3ZZ"  =  "1"  /* Extirpation of Matter from R Com Carotid, Perc Approach */
"03CH4ZZ"  =  "1"  /* Extirpation of Matter from R Com Carotid, Perc Endo Approach */
"03CJ0ZZ"  =  "1"  /* Extirpation of Matter from L Com Carotid, Open Approach */
"03CJ3ZZ"  =  "1"  /* Extirpation of Matter from L Com Carotid, Perc Approach */
"03CJ4ZZ"  =  "1"  /* Extirpation of Matter from L Com Carotid, Perc Endo Approach */
"03CK0ZZ"  =  "1"  /* Extirpation of Matter from R Int Carotid, Open Approach */
"03CK3ZZ"  =  "1"  /* Extirpation of Matter from R Int Carotid, Perc Approach */
"03CK4ZZ"  =  "1"  /* Extirpation of Matter from R Int Carotid, Perc Endo Approach */
"03CL0ZZ"  =  "1"  /* Extirpation of Matter from L Int Carotid, Open Approach */
"03CL3ZZ"  =  "1"  /* Extirpation of Matter from L Int Carotid, Perc Approach */
"03CL4ZZ"  =  "1"  /* Extirpation of Matter from L Int Carotid, Perc Endo Approach */
"03CM0ZZ"  =  "1"  /* Extirpation of Matter from R Ext Carotid, Open Approach */
"03CM3ZZ"  =  "1"  /* Extirpation of Matter from R Ext Carotid, Perc Approach */
"03CM4ZZ"  =  "1"  /* Extirpation of Matter from R Ext Carotid, Perc Endo Approach */
"03CN0ZZ"  =  "1"  /* Extirpation of Matter from L Ext Carotid, Open Approach */
"03CN3ZZ"  =  "1"  /* Extirpation of Matter from L Ext Carotid, Perc Approach */
"03CN4ZZ"  =  "1"  /* Extirpation of Matter from L Ext Carotid, Perc Endo Approach */
"03CP0ZZ"  =  "1"  /* Extirpation of Matter from R Verteb Art, Open Approach */
"03CP3ZZ"  =  "1"  /* Extirpation of Matter from R Verteb Art, Perc Approach */
"03CP4ZZ"  =  "1"  /* Extirpation of Matter from R Verteb Art, Perc Endo Approach */
"03CQ0ZZ"  =  "1"  /* Extirpation of Matter from L Verteb Art, Open Approach */
"03CQ3ZZ"  =  "1"  /* Extirpation of Matter from L Verteb Art, Perc Approach */
"03CQ4ZZ"  =  "1"  /* Extirpation of Matter from L Verteb Art, Perc Endo Approach */
"03CR0ZZ"  =  "1"  /* Extirpation of Matter from Face Artery, Open Approach */
"03CR3ZZ"  =  "1"  /* Extirpation of Matter from Face Artery, Perc Approach */
"03CR4ZZ"  =  "1"  /* Extirpation of Matter from Face Artery, Perc Endo Approach */
"03CS0ZZ"  =  "1"  /* Extirpation of Matter from R Temporal Art, Open Approach */
"03CS3ZZ"  =  "1"  /* Extirpation of Matter from R Temporal Art, Perc Approach */
"03CS4ZZ"  =  "1"  /* Extirpate of Matter from R Temporal Art, Perc Endo Approach */
"03CT0ZZ"  =  "1"  /* Extirpation of Matter from L Temporal Art, Open Approach */
"03CT3ZZ"  =  "1"  /* Extirpation of Matter from L Temporal Art, Perc Approach */
"03CT4ZZ"  =  "1"  /* Extirpate of Matter from L Temporal Art, Perc Endo Approach */
"03CU0ZZ"  =  "1"  /* Extirpation of Matter from R Thyroid Art, Open Approach */
"03CU3ZZ"  =  "1"  /* Extirpation of Matter from R Thyroid Art, Perc Approach */
"03CU4ZZ"  =  "1"  /* Extirpation of Matter from R Thyroid Art, Perc Endo Approach */
"03CV0ZZ"  =  "1"  /* Extirpation of Matter from L Thyroid Art, Open Approach */
"03CV3ZZ"  =  "1"  /* Extirpation of Matter from L Thyroid Art, Perc Approach */
"03CV4ZZ"  =  "1"  /* Extirpation of Matter from L Thyroid Art, Perc Endo Approach */
"03CY0ZZ"  =  "1"  /* Extirpation of Matter from Upper Artery, Open Approach */
"03CY3ZZ"  =  "1"  /* Extirpation of Matter from Upper Artery, Perc Approach */
"03CY4ZZ"  =  "1"  /* Extirpation of Matter from Upper Artery, Perc Endo Approach */
"03H00DZ"  =  "1"  /* Insertion of Intralum Dev into R Int Mamm Art, Open Approach */
"03H03DZ"  =  "1"  /* Insertion of Intralum Dev into R Int Mamm Art, Perc Approach */
"03H04DZ"  =  "1"  /* Insert Intralum Dev in R Int Mamm Art, Perc Endo */
"03H10DZ"  =  "1"  /* Insertion of Intralum Dev into L Int Mamm Art, Open Approach */
"03H13DZ"  =  "1"  /* Insertion of Intralum Dev into L Int Mamm Art, Perc Approach */
"03H14DZ"  =  "1"  /* Insert Intralum Dev in L Int Mamm Art, Perc Endo */
"03H20DZ"  =  "1"  /* Insertion of Intralum Dev into Innom Art, Open Approach */
"03H23DZ"  =  "1"  /* Insertion of Intralum Dev into Innom Art, Perc Approach */
"03H24DZ"  =  "1"  /* Insertion of Intralum Dev into Innom Art, Perc Endo Approach */
"03H30DZ"  =  "1"  /* Insertion of Intralum Dev into R Subclav Art, Open Approach */
"03H33DZ"  =  "1"  /* Insertion of Intralum Dev into R Subclav Art, Perc Approach */
"03H34DZ"  =  "1"  /* Insert Intralum Dev in R Subclav Art, Perc Endo */
"03H40DZ"  =  "1"  /* Insertion of Intralum Dev into L Subclav Art, Open Approach */
"03H43DZ"  =  "1"  /* Insertion of Intralum Dev into L Subclav Art, Perc Approach */
"03H44DZ"  =  "1"  /* Insert Intralum Dev in L Subclav Art, Perc Endo */
"03H50DZ"  =  "1"  /* Insertion of Intralum Dev into R Axilla Art, Open Approach */
"03H53DZ"  =  "1"  /* Insertion of Intralum Dev into R Axilla Art, Perc Approach */
"03H54DZ"  =  "1"  /* Insert of Intralum Dev into R Axilla Art, Perc Endo Approach */
"03H60DZ"  =  "1"  /* Insertion of Intralum Dev into L Axilla Art, Open Approach */
"03H63DZ"  =  "1"  /* Insertion of Intralum Dev into L Axilla Art, Perc Approach */
"03H64DZ"  =  "1"  /* Insert of Intralum Dev into L Axilla Art, Perc Endo Approach */
"03H70DZ"  =  "1"  /* Insertion of Intralum Dev into R Brach Art, Open Approach */
"03H73DZ"  =  "1"  /* Insertion of Intralum Dev into R Brach Art, Perc Approach */
"03H74DZ"  =  "1"  /* Insert of Intralum Dev into R Brach Art, Perc Endo Approach */
"03H80DZ"  =  "1"  /* Insertion of Intralum Dev into L Brach Art, Open Approach */
"03H83DZ"  =  "1"  /* Insertion of Intralum Dev into L Brach Art, Perc Approach */
"03H84DZ"  =  "1"  /* Insert of Intralum Dev into L Brach Art, Perc Endo Approach */
"03H90DZ"  =  "1"  /* Insertion of Intralum Dev into R Ulnar Art, Open Approach */
"03H93DZ"  =  "1"  /* Insertion of Intralum Dev into R Ulnar Art, Perc Approach */
"03H94DZ"  =  "1"  /* Insert of Intralum Dev into R Ulnar Art, Perc Endo Approach */
"03HA0DZ"  =  "1"  /* Insertion of Intralum Dev into L Ulnar Art, Open Approach */
"03HA3DZ"  =  "1"  /* Insertion of Intralum Dev into L Ulnar Art, Perc Approach */
"03HA4DZ"  =  "1"  /* Insert of Intralum Dev into L Ulnar Art, Perc Endo Approach */
"03HB0DZ"  =  "1"  /* Insertion of Intralum Dev into R Radial Art, Open Approach */
"03HB3DZ"  =  "1"  /* Insertion of Intralum Dev into R Radial Art, Perc Approach */
"03HB4DZ"  =  "1"  /* Insert of Intralum Dev into R Radial Art, Perc Endo Approach */
"03HC0DZ"  =  "1"  /* Insertion of Intralum Dev into L Radial Art, Open Approach */
"03HC3DZ"  =  "1"  /* Insertion of Intralum Dev into L Radial Art, Perc Approach */
"03HC4DZ"  =  "1"  /* Insert of Intralum Dev into L Radial Art, Perc Endo Approach */
"03HD0DZ"  =  "1"  /* Insertion of Intralum Dev into R Hand Art, Open Approach */
"03HD3DZ"  =  "1"  /* Insertion of Intralum Dev into R Hand Art, Perc Approach */
"03HD4DZ"  =  "1"  /* Insert of Intralum Dev into R Hand Art, Perc Endo Approach */
"03HF0DZ"  =  "1"  /* Insertion of Intralum Dev into L Hand Art, Open Approach */
"03HF3DZ"  =  "1"  /* Insertion of Intralum Dev into L Hand Art, Perc Approach */
"03HF4DZ"  =  "1"  /* Insert of Intralum Dev into L Hand Art, Perc Endo Approach */
"03HG0DZ"  =  "1"  /* Insertion of Intralum Dev into Intracran Art, Open Approach */
"03HG3DZ"  =  "1"  /* Insertion of Intralum Dev into Intracran Art, Perc Approach */
"03HG4DZ"  =  "1"  /* Insert Intralum Dev in Intracran Art, Perc Endo */
"03HH0DZ"  =  "1"  /* Insertion of Intralum Dev into R Com Carotid, Open Approach */
"03HH3DZ"  =  "1"  /* Insertion of Intralum Dev into R Com Carotid, Perc Approach */
"03HH4DZ"  =  "1"  /* Insert Intralum Dev in R Com Carotid, Perc Endo */
"03HJ0DZ"  =  "1"  /* Insertion of Intralum Dev into L Com Carotid, Open Approach */
"03HJ3DZ"  =  "1"  /* Insertion of Intralum Dev into L Com Carotid, Perc Approach */
"03HJ4DZ"  =  "1"  /* Insert Intralum Dev in L Com Carotid, Perc Endo */
"03HK0DZ"  =  "1"  /* Insertion of Intralum Dev into R Int Carotid, Open Approach */
"03HK0MZ"  =  "1"  /* Insertion of Stim Lead into R Int Carotid, Open Approach */
"03HK3DZ"  =  "1"  /* Insertion of Intralum Dev into R Int Carotid, Perc Approach */
"03HK3MZ"  =  "1"  /* Insertion of Stim Lead into R Int Carotid, Perc Approach */
"03HK4DZ"  =  "1"  /* Insert Intralum Dev in R Int Carotid, Perc Endo */
"03HK4MZ"  =  "1"  /* Insert of Stim Lead into R Int Carotid, Perc Endo Approach */
"03HL0DZ"  =  "1"  /* Insertion of Intralum Dev into L Int Carotid, Open Approach */
"03HL0MZ"  =  "1"  /* Insertion of Stim Lead into L Int Carotid, Open Approach */
"03HL3DZ"  =  "1"  /* Insertion of Intralum Dev into L Int Carotid, Perc Approach */
"03HL3MZ"  =  "1"  /* Insertion of Stim Lead into L Int Carotid, Perc Approach */
"03HL4DZ"  =  "1"  /* Insert Intralum Dev in L Int Carotid, Perc Endo */
"03HL4MZ"  =  "1"  /* Insert of Stim Lead into L Int Carotid, Perc Endo Approach */
"03HM0DZ"  =  "1"  /* Insertion of Intralum Dev into R Ext Carotid, Open Approach */
"03HM3DZ"  =  "1"  /* Insertion of Intralum Dev into R Ext Carotid, Perc Approach */
"03HM4DZ"  =  "1"  /* Insert Intralum Dev in R Ext Carotid, Perc Endo */
"03HN0DZ"  =  "1"  /* Insertion of Intralum Dev into L Ext Carotid, Open Approach */
"03HN3DZ"  =  "1"  /* Insertion of Intralum Dev into L Ext Carotid, Perc Approach */
"03HN4DZ"  =  "1"  /* Insert Intralum Dev in L Ext Carotid, Perc Endo */
"03HP0DZ"  =  "1"  /* Insertion of Intralum Dev into R Verteb Art, Open Approach */
"03HP3DZ"  =  "1"  /* Insertion of Intralum Dev into R Verteb Art, Perc Approach */
"03HP4DZ"  =  "1"  /* Insert of Intralum Dev into R Verteb Art, Perc Endo Approach */
"03HQ0DZ"  =  "1"  /* Insertion of Intralum Dev into L Verteb Art, Open Approach */
"03HQ3DZ"  =  "1"  /* Insertion of Intralum Dev into L Verteb Art, Perc Approach */
"03HQ4DZ"  =  "1"  /* Insert of Intralum Dev into L Verteb Art, Perc Endo Approach */
"03HR0DZ"  =  "1"  /* Insertion of Intralum Dev into Face Art, Open Approach */
"03HR3DZ"  =  "1"  /* Insertion of Intralum Dev into Face Art, Perc Approach */
"03HR4DZ"  =  "1"  /* Insertion of Intralum Dev into Face Art, Perc Endo Approach */
"03HS0DZ"  =  "1"  /* Insertion of Intralum Dev into R Temporal Art, Open Approach */
"03HS3DZ"  =  "1"  /* Insertion of Intralum Dev into R Temporal Art, Perc Approach */
"03HS4DZ"  =  "1"  /* Insert Intralum Dev in R Temporal Art, Perc Endo */
"03HT0DZ"  =  "1"  /* Insertion of Intralum Dev into L Temporal Art, Open Approach */
"03HT3DZ"  =  "1"  /* Insertion of Intralum Dev into L Temporal Art, Perc Approach */
"03HT4DZ"  =  "1"  /* Insert Intralum Dev in L Temporal Art, Perc Endo */
"03HU0DZ"  =  "1"  /* Insertion of Intralum Dev into R Thyroid Art, Open Approach */
"03HU3DZ"  =  "1"  /* Insertion of Intralum Dev into R Thyroid Art, Perc Approach */
"03HU4DZ"  =  "1"  /* Insert Intralum Dev in R Thyroid Art, Perc Endo */
"03HV0DZ"  =  "1"  /* Insertion of Intralum Dev into L Thyroid Art, Open Approach */
"03HV3DZ"  =  "1"  /* Insertion of Intralum Dev into L Thyroid Art, Perc Approach */
"03HV4DZ"  =  "1"  /* Insert Intralum Dev in L Thyroid Art, Perc Endo */
"03HY02Z"  =  "1"  /* Insertion of Monitoring Device into Up Art, Open Approach */
"03HY0DZ"  =  "1"  /* Insertion of Intraluminal Device into Up Art, Open Approach */
"03HY32Z"  =  "1"  /* Insertion of Monitoring Device into Up Art, Perc Approach */
"03HY3DZ"  =  "1"  /* Insertion of Intraluminal Device into Up Art, Perc Approach */
"03HY42Z"  =  "1"  /* Insertion of Monitor Dev into Up Art, Perc Endo Approach */
"03HY4DZ"  =  "1"  /* Insertion of Intralum Dev into Up Art, Perc Endo Approach */
"03JY0ZZ"  =  "1"  /* Inspection of Upper Artery, Open Approach */
"03JY3ZZ"  =  "1"  /* Inspection of Upper Artery, Percutaneous Approach */
"03L00CZ"  =  "1"  /* Occlusion of R Int Mamm Art with Extralum Dev, Open Approach */
"03L00DZ"  =  "1"  /* Occlusion of R Int Mamm Art with Intralum Dev, Open Approach */
"03L00ZZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery, Open Approach */
"03L03CZ"  =  "1"  /* Occlusion of R Int Mamm Art with Extralum Dev, Perc Approach */
"03L03DZ"  =  "1"  /* Occlusion of R Int Mamm Art with Intralum Dev, Perc Approach */
"03L03ZZ"  =  "1"  /* Occlusion of Right Internal Mammary Artery, Perc Approach */
"03L04CZ"  =  "1"  /* Occlusion R Int Mamm Art w Extralum Dev, Perc Endo */
"03L04DZ"  =  "1"  /* Occlusion R Int Mamm Art w Intralum Dev, Perc Endo */
"03L04ZZ"  =  "1"  /* Occlusion of R Int Mamm Art, Perc Endo Approach */
"03L10CZ"  =  "1"  /* Occlusion of L Int Mamm Art with Extralum Dev, Open Approach */
"03L10DZ"  =  "1"  /* Occlusion of L Int Mamm Art with Intralum Dev, Open Approach */
"03L10ZZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery, Open Approach */
"03L13CZ"  =  "1"  /* Occlusion of L Int Mamm Art with Extralum Dev, Perc Approach */
"03L13DZ"  =  "1"  /* Occlusion of L Int Mamm Art with Intralum Dev, Perc Approach */
"03L13ZZ"  =  "1"  /* Occlusion of Left Internal Mammary Artery, Perc Approach */
"03L14CZ"  =  "1"  /* Occlusion L Int Mamm Art w Extralum Dev, Perc Endo */
"03L14DZ"  =  "1"  /* Occlusion L Int Mamm Art w Intralum Dev, Perc Endo */
"03L14ZZ"  =  "1"  /* Occlusion of L Int Mamm Art, Perc Endo Approach */
"03L20CZ"  =  "1"  /* Occlusion of Innom Art with Extralum Dev, Open Approach */
"03L20DZ"  =  "1"  /* Occlusion of Innom Art with Intralum Dev, Open Approach */
"03L20ZZ"  =  "1"  /* Occlusion of Innominate Artery, Open Approach */
"03L23CZ"  =  "1"  /* Occlusion of Innom Art with Extralum Dev, Perc Approach */
"03L23DZ"  =  "1"  /* Occlusion of Innom Art with Intralum Dev, Perc Approach */
"03L23ZZ"  =  "1"  /* Occlusion of Innominate Artery, Percutaneous Approach */
"03L24CZ"  =  "1"  /* Occlusion of Innom Art with Extralum Dev, Perc Endo Approach */
"03L24DZ"  =  "1"  /* Occlusion of Innom Art with Intralum Dev, Perc Endo Approach */
"03L24ZZ"  =  "1"  /* Occlusion of Innominate Artery, Perc Endo Approach */
"03L30CZ"  =  "1"  /* Occlusion of R Subclav Art with Extralum Dev, Open Approach */
"03L30DZ"  =  "1"  /* Occlusion of R Subclav Art with Intralum Dev, Open Approach */
"03L30ZZ"  =  "1"  /* Occlusion of Right Subclavian Artery, Open Approach */
"03L33CZ"  =  "1"  /* Occlusion of R Subclav Art with Extralum Dev, Perc Approach */
"03L33DZ"  =  "1"  /* Occlusion of R Subclav Art with Intralum Dev, Perc Approach */
"03L33ZZ"  =  "1"  /* Occlusion of Right Subclavian Artery, Percutaneous Approach */
"03L34CZ"  =  "1"  /* Occlusion R Subclav Art w Extralum Dev, Perc Endo */
"03L34DZ"  =  "1"  /* Occlusion R Subclav Art w Intralum Dev, Perc Endo */
"03L34ZZ"  =  "1"  /* Occlusion of Right Subclavian Artery, Perc Endo Approach */
"03L40CZ"  =  "1"  /* Occlusion of L Subclav Art with Extralum Dev, Open Approach */
"03L40DZ"  =  "1"  /* Occlusion of L Subclav Art with Intralum Dev, Open Approach */
"03L40ZZ"  =  "1"  /* Occlusion of Left Subclavian Artery, Open Approach */
"03L43CZ"  =  "1"  /* Occlusion of L Subclav Art with Extralum Dev, Perc Approach */
"03L43DZ"  =  "1"  /* Occlusion of L Subclav Art with Intralum Dev, Perc Approach */
"03L43ZZ"  =  "1"  /* Occlusion of Left Subclavian Artery, Percutaneous Approach */
"03L44CZ"  =  "1"  /* Occlusion L Subclav Art w Extralum Dev, Perc Endo */
"03L44DZ"  =  "1"  /* Occlusion L Subclav Art w Intralum Dev, Perc Endo */
"03L44ZZ"  =  "1"  /* Occlusion of Left Subclavian Artery, Perc Endo Approach */
"03L50CZ"  =  "1"  /* Occlusion of R Axilla Art with Extralum Dev, Open Approach */
"03L50DZ"  =  "1"  /* Occlusion of R Axilla Art with Intralum Dev, Open Approach */
"03L50ZZ"  =  "1"  /* Occlusion of Right Axillary Artery, Open Approach */
"03L53CZ"  =  "1"  /* Occlusion of R Axilla Art with Extralum Dev, Perc Approach */
"03L53DZ"  =  "1"  /* Occlusion of R Axilla Art with Intralum Dev, Perc Approach */
"03L53ZZ"  =  "1"  /* Occlusion of Right Axillary Artery, Percutaneous Approach */
"03L54CZ"  =  "1"  /* Occlusion R Axilla Art w Extralum Dev, Perc Endo */
"03L54DZ"  =  "1"  /* Occlusion R Axilla Art w Intralum Dev, Perc Endo */
"03L54ZZ"  =  "1"  /* Occlusion of Right Axillary Artery, Perc Endo Approach */
"03L60CZ"  =  "1"  /* Occlusion of L Axilla Art with Extralum Dev, Open Approach */
"03L60DZ"  =  "1"  /* Occlusion of L Axilla Art with Intralum Dev, Open Approach */
"03L60ZZ"  =  "1"  /* Occlusion of Left Axillary Artery, Open Approach */
"03L63CZ"  =  "1"  /* Occlusion of L Axilla Art with Extralum Dev, Perc Approach */
"03L63DZ"  =  "1"  /* Occlusion of L Axilla Art with Intralum Dev, Perc Approach */
"03L63ZZ"  =  "1"  /* Occlusion of Left Axillary Artery, Percutaneous Approach */
"03L64CZ"  =  "1"  /* Occlusion L Axilla Art w Extralum Dev, Perc Endo */
"03L64DZ"  =  "1"  /* Occlusion L Axilla Art w Intralum Dev, Perc Endo */
"03L64ZZ"  =  "1"  /* Occlusion of Left Axillary Artery, Perc Endo Approach */
"03L70CZ"  =  "1"  /* Occlusion of R Brach Art with Extralum Dev, Open Approach */
"03L70DZ"  =  "1"  /* Occlusion of R Brach Art with Intralum Dev, Open Approach */
"03L70ZZ"  =  "1"  /* Occlusion of Right Brachial Artery, Open Approach */
"03L73CZ"  =  "1"  /* Occlusion of R Brach Art with Extralum Dev, Perc Approach */
"03L73DZ"  =  "1"  /* Occlusion of R Brach Art with Intralum Dev, Perc Approach */
"03L73ZZ"  =  "1"  /* Occlusion of Right Brachial Artery, Percutaneous Approach */
"03L74CZ"  =  "1"  /* Occlusion R Brach Art w Extralum Dev, Perc Endo */
"03L74DZ"  =  "1"  /* Occlusion R Brach Art w Intralum Dev, Perc Endo */
"03L74ZZ"  =  "1"  /* Occlusion of Right Brachial Artery, Perc Endo Approach */
"03L80CZ"  =  "1"  /* Occlusion of L Brach Art with Extralum Dev, Open Approach */
"03L80DZ"  =  "1"  /* Occlusion of L Brach Art with Intralum Dev, Open Approach */
"03L80ZZ"  =  "1"  /* Occlusion of Left Brachial Artery, Open Approach */
"03L83CZ"  =  "1"  /* Occlusion of L Brach Art with Extralum Dev, Perc Approach */
"03L83DZ"  =  "1"  /* Occlusion of L Brach Art with Intralum Dev, Perc Approach */
"03L83ZZ"  =  "1"  /* Occlusion of Left Brachial Artery, Percutaneous Approach */
"03L84CZ"  =  "1"  /* Occlusion L Brach Art w Extralum Dev, Perc Endo */
"03L84DZ"  =  "1"  /* Occlusion L Brach Art w Intralum Dev, Perc Endo */
"03L84ZZ"  =  "1"  /* Occlusion of Left Brachial Artery, Perc Endo Approach */
"03L90CZ"  =  "1"  /* Occlusion of R Ulnar Art with Extralum Dev, Open Approach */
"03L90DZ"  =  "1"  /* Occlusion of R Ulnar Art with Intralum Dev, Open Approach */
"03L90ZZ"  =  "1"  /* Occlusion of Right Ulnar Artery, Open Approach */
"03L93CZ"  =  "1"  /* Occlusion of R Ulnar Art with Extralum Dev, Perc Approach */
"03L93DZ"  =  "1"  /* Occlusion of R Ulnar Art with Intralum Dev, Perc Approach */
"03L93ZZ"  =  "1"  /* Occlusion of Right Ulnar Artery, Percutaneous Approach */
"03L94CZ"  =  "1"  /* Occlusion R Ulnar Art w Extralum Dev, Perc Endo */
"03L94DZ"  =  "1"  /* Occlusion R Ulnar Art w Intralum Dev, Perc Endo */
"03L94ZZ"  =  "1"  /* Occlusion of Right Ulnar Artery, Perc Endo Approach */
"03LA0CZ"  =  "1"  /* Occlusion of L Ulnar Art with Extralum Dev, Open Approach */
"03LA0DZ"  =  "1"  /* Occlusion of L Ulnar Art with Intralum Dev, Open Approach */
"03LA0ZZ"  =  "1"  /* Occlusion of Left Ulnar Artery, Open Approach */
"03LA3CZ"  =  "1"  /* Occlusion of L Ulnar Art with Extralum Dev, Perc Approach */
"03LA3DZ"  =  "1"  /* Occlusion of L Ulnar Art with Intralum Dev, Perc Approach */
"03LA3ZZ"  =  "1"  /* Occlusion of Left Ulnar Artery, Percutaneous Approach */
"03LA4CZ"  =  "1"  /* Occlusion L Ulnar Art w Extralum Dev, Perc Endo */
"03LA4DZ"  =  "1"  /* Occlusion L Ulnar Art w Intralum Dev, Perc Endo */
"03LA4ZZ"  =  "1"  /* Occlusion of Left Ulnar Artery, Perc Endo Approach */
"03LB0CZ"  =  "1"  /* Occlusion of R Radial Art with Extralum Dev, Open Approach */
"03LB0DZ"  =  "1"  /* Occlusion of R Radial Art with Intralum Dev, Open Approach */
"03LB0ZZ"  =  "1"  /* Occlusion of Right Radial Artery, Open Approach */
"03LB3CZ"  =  "1"  /* Occlusion of R Radial Art with Extralum Dev, Perc Approach */
"03LB3DZ"  =  "1"  /* Occlusion of R Radial Art with Intralum Dev, Perc Approach */
"03LB3ZZ"  =  "1"  /* Occlusion of Right Radial Artery, Percutaneous Approach */
"03LB4CZ"  =  "1"  /* Occlusion R Radial Art w Extralum Dev, Perc Endo */
"03LB4DZ"  =  "1"  /* Occlusion R Radial Art w Intralum Dev, Perc Endo */
"03LB4ZZ"  =  "1"  /* Occlusion of Right Radial Artery, Perc Endo Approach */
"03LC0CZ"  =  "1"  /* Occlusion of L Radial Art with Extralum Dev, Open Approach */
"03LC0DZ"  =  "1"  /* Occlusion of L Radial Art with Intralum Dev, Open Approach */
"03LC0ZZ"  =  "1"  /* Occlusion of Left Radial Artery, Open Approach */
"03LC3CZ"  =  "1"  /* Occlusion of L Radial Art with Extralum Dev, Perc Approach */
"03LC3DZ"  =  "1"  /* Occlusion of L Radial Art with Intralum Dev, Perc Approach */
"03LC3ZZ"  =  "1"  /* Occlusion of Left Radial Artery, Percutaneous Approach */
"03LC4CZ"  =  "1"  /* Occlusion L Radial Art w Extralum Dev, Perc Endo */
"03LC4DZ"  =  "1"  /* Occlusion L Radial Art w Intralum Dev, Perc Endo */
"03LC4ZZ"  =  "1"  /* Occlusion of Left Radial Artery, Perc Endo Approach */
"03LD0CZ"  =  "1"  /* Occlusion of R Hand Art with Extralum Dev, Open Approach */
"03LD0DZ"  =  "1"  /* Occlusion of R Hand Art with Intralum Dev, Open Approach */
"03LD0ZZ"  =  "1"  /* Occlusion of Right Hand Artery, Open Approach */
"03LD3CZ"  =  "1"  /* Occlusion of R Hand Art with Extralum Dev, Perc Approach */
"03LD3DZ"  =  "1"  /* Occlusion of R Hand Art with Intralum Dev, Perc Approach */
"03LD3ZZ"  =  "1"  /* Occlusion of Right Hand Artery, Percutaneous Approach */
"03LD4CZ"  =  "1"  /* Occlusion R Hand Art w Extralum Dev, Perc Endo */
"03LD4DZ"  =  "1"  /* Occlusion R Hand Art w Intralum Dev, Perc Endo */
"03LD4ZZ"  =  "1"  /* Occlusion of Right Hand Artery, Perc Endo Approach */
"03LF0CZ"  =  "1"  /* Occlusion of L Hand Art with Extralum Dev, Open Approach */
"03LF0DZ"  =  "1"  /* Occlusion of L Hand Art with Intralum Dev, Open Approach */
"03LF0ZZ"  =  "1"  /* Occlusion of Left Hand Artery, Open Approach */
"03LF3CZ"  =  "1"  /* Occlusion of L Hand Art with Extralum Dev, Perc Approach */
"03LF3DZ"  =  "1"  /* Occlusion of L Hand Art with Intralum Dev, Perc Approach */
"03LF3ZZ"  =  "1"  /* Occlusion of Left Hand Artery, Percutaneous Approach */
"03LF4CZ"  =  "1"  /* Occlusion L Hand Art w Extralum Dev, Perc Endo */
"03LF4DZ"  =  "1"  /* Occlusion L Hand Art w Intralum Dev, Perc Endo */
"03LF4ZZ"  =  "1"  /* Occlusion of Left Hand Artery, Perc Endo Approach */
"03LG0BZ"  =  "1"  /* Occlusion Intracran Art w Bioact Intralum, Open */
"03LG0CZ"  =  "1"  /* Occlusion of Intracran Art with Extralum Dev, Open Approach */
"03LG0DZ"  =  "1"  /* Occlusion of Intracran Art with Intralum Dev, Open Approach */
"03LG0ZZ"  =  "1"  /* Occlusion of Intracranial Artery, Open Approach */
"03LG3BZ"  =  "1"  /* Occlusion Intracran Art w Bioact Intralum, Perc */
"03LG3CZ"  =  "1"  /* Occlusion of Intracran Art with Extralum Dev, Perc Approach */
"03LG3DZ"  =  "1"  /* Occlusion of Intracran Art with Intralum Dev, Perc Approach */
"03LG3ZZ"  =  "1"  /* Occlusion of Intracranial Artery, Percutaneous Approach */
"03LG4BZ"  =  "1"  /* Occlusion Intracran Art w Bioact Intralum, Perc Endo */
"03LG4CZ"  =  "1"  /* Occlusion Intracran Art w Extralum Dev, Perc Endo */
"03LG4DZ"  =  "1"  /* Occlusion Intracran Art w Intralum Dev, Perc Endo */
"03LG4ZZ"  =  "1"  /* Occlusion of Intracranial Artery, Perc Endo Approach */
"03LH0BZ"  =  "1"  /* Occlusion R Com Carotid w Bioact Intralum, Open */
"03LH0CZ"  =  "1"  /* Occlusion of R Com Carotid with Extralum Dev, Open Approach */
"03LH0DZ"  =  "1"  /* Occlusion of R Com Carotid with Intralum Dev, Open Approach */
"03LH0ZZ"  =  "1"  /* Occlusion of Right Common Carotid Artery, Open Approach */
"03LH3BZ"  =  "1"  /* Occlusion R Com Carotid w Bioact Intralum, Perc */
"03LH3CZ"  =  "1"  /* Occlusion of R Com Carotid with Extralum Dev, Perc Approach */
"03LH3DZ"  =  "1"  /* Occlusion of R Com Carotid with Intralum Dev, Perc Approach */
"03LH3ZZ"  =  "1"  /* Occlusion of Right Common Carotid Artery, Perc Approach */
"03LH4BZ"  =  "1"  /* Occlusion R Com Carotid w Bioact Intralum, Perc Endo */
"03LH4CZ"  =  "1"  /* Occlusion R Com Carotid w Extralum Dev, Perc Endo */
"03LH4DZ"  =  "1"  /* Occlusion R Com Carotid w Intralum Dev, Perc Endo */
"03LH4ZZ"  =  "1"  /* Occlusion of Right Common Carotid Artery, Perc Endo Approach */
"03LJ0BZ"  =  "1"  /* Occlusion L Com Carotid w Bioact Intralum, Open */
"03LJ0CZ"  =  "1"  /* Occlusion of L Com Carotid with Extralum Dev, Open Approach */
"03LJ0DZ"  =  "1"  /* Occlusion of L Com Carotid with Intralum Dev, Open Approach */
"03LJ0ZZ"  =  "1"  /* Occlusion of Left Common Carotid Artery, Open Approach */
"03LJ3BZ"  =  "1"  /* Occlusion L Com Carotid w Bioact Intralum, Perc */
"03LJ3CZ"  =  "1"  /* Occlusion of L Com Carotid with Extralum Dev, Perc Approach */
"03LJ3DZ"  =  "1"  /* Occlusion of L Com Carotid with Intralum Dev, Perc Approach */
"03LJ3ZZ"  =  "1"  /* Occlusion of Left Common Carotid Artery, Perc Approach */
"03LJ4BZ"  =  "1"  /* Occlusion L Com Carotid w Bioact Intralum, Perc Endo */
"03LJ4CZ"  =  "1"  /* Occlusion L Com Carotid w Extralum Dev, Perc Endo */
"03LJ4DZ"  =  "1"  /* Occlusion L Com Carotid w Intralum Dev, Perc Endo */
"03LJ4ZZ"  =  "1"  /* Occlusion of Left Common Carotid Artery, Perc Endo Approach */
"03LK0BZ"  =  "1"  /* Occlusion R Int Carotid w Bioact Intralum, Open */
"03LK0CZ"  =  "1"  /* Occlusion of R Int Carotid with Extralum Dev, Open Approach */
"03LK0DZ"  =  "1"  /* Occlusion of R Int Carotid with Intralum Dev, Open Approach */
"03LK0ZZ"  =  "1"  /* Occlusion of Right Internal Carotid Artery, Open Approach */
"03LK3BZ"  =  "1"  /* Occlusion R Int Carotid w Bioact Intralum, Perc */
"03LK3CZ"  =  "1"  /* Occlusion of R Int Carotid with Extralum Dev, Perc Approach */
"03LK3DZ"  =  "1"  /* Occlusion of R Int Carotid with Intralum Dev, Perc Approach */
"03LK3ZZ"  =  "1"  /* Occlusion of Right Internal Carotid Artery, Perc Approach */
"03LK4BZ"  =  "1"  /* Occlusion R Int Carotid w Bioact Intralum, Perc Endo */
"03LK4CZ"  =  "1"  /* Occlusion R Int Carotid w Extralum Dev, Perc Endo */
"03LK4DZ"  =  "1"  /* Occlusion R Int Carotid w Intralum Dev, Perc Endo */
"03LK4ZZ"  =  "1"  /* Occlusion of R Int Carotid, Perc Endo Approach */
"03LL0BZ"  =  "1"  /* Occlusion L Int Carotid w Bioact Intralum, Open */
"03LL0CZ"  =  "1"  /* Occlusion of L Int Carotid with Extralum Dev, Open Approach */
"03LL0DZ"  =  "1"  /* Occlusion of L Int Carotid with Intralum Dev, Open Approach */
"03LL0ZZ"  =  "1"  /* Occlusion of Left Internal Carotid Artery, Open Approach */
"03LL3BZ"  =  "1"  /* Occlusion L Int Carotid w Bioact Intralum, Perc */
"03LL3CZ"  =  "1"  /* Occlusion of L Int Carotid with Extralum Dev, Perc Approach */
"03LL3DZ"  =  "1"  /* Occlusion of L Int Carotid with Intralum Dev, Perc Approach */
"03LL3ZZ"  =  "1"  /* Occlusion of Left Internal Carotid Artery, Perc Approach */
"03LL4BZ"  =  "1"  /* Occlusion L Int Carotid w Bioact Intralum, Perc Endo */
"03LL4CZ"  =  "1"  /* Occlusion L Int Carotid w Extralum Dev, Perc Endo */
"03LL4DZ"  =  "1"  /* Occlusion L Int Carotid w Intralum Dev, Perc Endo */
"03LL4ZZ"  =  "1"  /* Occlusion of L Int Carotid, Perc Endo Approach */
"03LM0BZ"  =  "1"  /* Occlusion R Ext Carotid w Bioact Intralum, Open */
"03LM0CZ"  =  "1"  /* Occlusion of R Ext Carotid with Extralum Dev, Open Approach */
"03LM0DZ"  =  "1"  /* Occlusion of R Ext Carotid with Intralum Dev, Open Approach */
"03LM0ZZ"  =  "1"  /* Occlusion of Right External Carotid Artery, Open Approach */
"03LM3BZ"  =  "1"  /* Occlusion R Ext Carotid w Bioact Intralum, Perc */
"03LM3CZ"  =  "1"  /* Occlusion of R Ext Carotid with Extralum Dev, Perc Approach */
"03LM3DZ"  =  "1"  /* Occlusion of R Ext Carotid with Intralum Dev, Perc Approach */
"03LM3ZZ"  =  "1"  /* Occlusion of Right External Carotid Artery, Perc Approach */
"03LM4BZ"  =  "1"  /* Occlusion R Ext Carotid w Bioact Intralum, Perc Endo */
"03LM4CZ"  =  "1"  /* Occlusion R Ext Carotid w Extralum Dev, Perc Endo */
"03LM4DZ"  =  "1"  /* Occlusion R Ext Carotid w Intralum Dev, Perc Endo */
"03LM4ZZ"  =  "1"  /* Occlusion of R Ext Carotid, Perc Endo Approach */
"03LN0BZ"  =  "1"  /* Occlusion L Ext Carotid w Bioact Intralum, Open */
"03LN0CZ"  =  "1"  /* Occlusion of L Ext Carotid with Extralum Dev, Open Approach */
"03LN0DZ"  =  "1"  /* Occlusion of L Ext Carotid with Intralum Dev, Open Approach */
"03LN0ZZ"  =  "1"  /* Occlusion of Left External Carotid Artery, Open Approach */
"03LN3BZ"  =  "1"  /* Occlusion L Ext Carotid w Bioact Intralum, Perc */
"03LN3CZ"  =  "1"  /* Occlusion of L Ext Carotid with Extralum Dev, Perc Approach */
"03LN3DZ"  =  "1"  /* Occlusion of L Ext Carotid with Intralum Dev, Perc Approach */
"03LN3ZZ"  =  "1"  /* Occlusion of Left External Carotid Artery, Perc Approach */
"03LN4BZ"  =  "1"  /* Occlusion L Ext Carotid w Bioact Intralum, Perc Endo */
"03LN4CZ"  =  "1"  /* Occlusion L Ext Carotid w Extralum Dev, Perc Endo */
"03LN4DZ"  =  "1"  /* Occlusion L Ext Carotid w Intralum Dev, Perc Endo */
"03LN4ZZ"  =  "1"  /* Occlusion of L Ext Carotid, Perc Endo Approach */
"03LP0BZ"  =  "1"  /* Occlusion R Verteb Art w Bioact Intralum, Open */
"03LP0CZ"  =  "1"  /* Occlusion of R Verteb Art with Extralum Dev, Open Approach */
"03LP0DZ"  =  "1"  /* Occlusion of R Verteb Art with Intralum Dev, Open Approach */
"03LP0ZZ"  =  "1"  /* Occlusion of Right Vertebral Artery, Open Approach */
"03LP3BZ"  =  "1"  /* Occlusion R Verteb Art w Bioact Intralum, Perc */
"03LP3CZ"  =  "1"  /* Occlusion of R Verteb Art with Extralum Dev, Perc Approach */
"03LP3DZ"  =  "1"  /* Occlusion of R Verteb Art with Intralum Dev, Perc Approach */
"03LP3ZZ"  =  "1"  /* Occlusion of Right Vertebral Artery, Percutaneous Approach */
"03LP4BZ"  =  "1"  /* Occlusion R Verteb Art w Bioact Intralum, Perc Endo */
"03LP4CZ"  =  "1"  /* Occlusion R Verteb Art w Extralum Dev, Perc Endo */
"03LP4DZ"  =  "1"  /* Occlusion R Verteb Art w Intralum Dev, Perc Endo */
"03LP4ZZ"  =  "1"  /* Occlusion of Right Vertebral Artery, Perc Endo Approach */
"03LQ0BZ"  =  "1"  /* Occlusion L Verteb Art w Bioact Intralum, Open */
"03LQ0CZ"  =  "1"  /* Occlusion of L Verteb Art with Extralum Dev, Open Approach */
"03LQ0DZ"  =  "1"  /* Occlusion of L Verteb Art with Intralum Dev, Open Approach */
"03LQ0ZZ"  =  "1"  /* Occlusion of Left Vertebral Artery, Open Approach */
"03LQ3BZ"  =  "1"  /* Occlusion L Verteb Art w Bioact Intralum, Perc */
"03LQ3CZ"  =  "1"  /* Occlusion of L Verteb Art with Extralum Dev, Perc Approach */
"03LQ3DZ"  =  "1"  /* Occlusion of L Verteb Art with Intralum Dev, Perc Approach */
"03LQ3ZZ"  =  "1"  /* Occlusion of Left Vertebral Artery, Percutaneous Approach */
"03LQ4BZ"  =  "1"  /* Occlusion L Verteb Art w Bioact Intralum, Perc Endo */
"03LQ4CZ"  =  "1"  /* Occlusion L Verteb Art w Extralum Dev, Perc Endo */
"03LQ4DZ"  =  "1"  /* Occlusion L Verteb Art w Intralum Dev, Perc Endo */
"03LQ4ZZ"  =  "1"  /* Occlusion of Left Vertebral Artery, Perc Endo Approach */
"03LR0CZ"  =  "1"  /* Occlusion of Face Artery with Extralum Dev, Open Approach */
"03LR0DZ"  =  "1"  /* Occlusion of Face Artery with Intralum Dev, Open Approach */
"03LR0ZZ"  =  "1"  /* Occlusion of Face Artery, Open Approach */
"03LR3CZ"  =  "1"  /* Occlusion of Face Artery with Extralum Dev, Perc Approach */
"03LR3DZ"  =  "1"  /* Occlusion of Face Artery with Intralum Dev, Perc Approach */
"03LR3ZZ"  =  "1"  /* Occlusion of Face Artery, Percutaneous Approach */
"03LR4CZ"  =  "1"  /* Occlusion of Face Art with Extralum Dev, Perc Endo Approach */
"03LR4DZ"  =  "1"  /* Occlusion of Face Art with Intralum Dev, Perc Endo Approach */
"03LR4ZZ"  =  "1"  /* Occlusion of Face Artery, Percutaneous Endoscopic Approach */
"03LS0CZ"  =  "1"  /* Occlusion of R Temporal Art with Extralum Dev, Open Approach */
"03LS0DZ"  =  "1"  /* Occlusion of R Temporal Art with Intralum Dev, Open Approach */
"03LS0ZZ"  =  "1"  /* Occlusion of Right Temporal Artery, Open Approach */
"03LS3CZ"  =  "1"  /* Occlusion of R Temporal Art with Extralum Dev, Perc Approach */
"03LS3DZ"  =  "1"  /* Occlusion of R Temporal Art with Intralum Dev, Perc Approach */
"03LS3ZZ"  =  "1"  /* Occlusion of Right Temporal Artery, Percutaneous Approach */
"03LS4CZ"  =  "1"  /* Occlusion R Temporal Art w Extralum Dev, Perc Endo */
"03LS4DZ"  =  "1"  /* Occlusion R Temporal Art w Intralum Dev, Perc Endo */
"03LS4ZZ"  =  "1"  /* Occlusion of Right Temporal Artery, Perc Endo Approach */
"03LT0CZ"  =  "1"  /* Occlusion of L Temporal Art with Extralum Dev, Open Approach */
"03LT0DZ"  =  "1"  /* Occlusion of L Temporal Art with Intralum Dev, Open Approach */
"03LT0ZZ"  =  "1"  /* Occlusion of Left Temporal Artery, Open Approach */
"03LT3CZ"  =  "1"  /* Occlusion of L Temporal Art with Extralum Dev, Perc Approach */
"03LT3DZ"  =  "1"  /* Occlusion of L Temporal Art with Intralum Dev, Perc Approach */
"03LT3ZZ"  =  "1"  /* Occlusion of Left Temporal Artery, Percutaneous Approach */
"03LT4CZ"  =  "1"  /* Occlusion L Temporal Art w Extralum Dev, Perc Endo */
"03LT4DZ"  =  "1"  /* Occlusion L Temporal Art w Intralum Dev, Perc Endo */
"03LT4ZZ"  =  "1"  /* Occlusion of Left Temporal Artery, Perc Endo Approach */
"03LU0CZ"  =  "1"  /* Occlusion of R Thyroid Art with Extralum Dev, Open Approach */
"03LU0DZ"  =  "1"  /* Occlusion of R Thyroid Art with Intralum Dev, Open Approach */
"03LU0ZZ"  =  "1"  /* Occlusion of Right Thyroid Artery, Open Approach */
"03LU3CZ"  =  "1"  /* Occlusion of R Thyroid Art with Extralum Dev, Perc Approach */
"03LU3DZ"  =  "1"  /* Occlusion of R Thyroid Art with Intralum Dev, Perc Approach */
"03LU3ZZ"  =  "1"  /* Occlusion of Right Thyroid Artery, Percutaneous Approach */
"03LU4CZ"  =  "1"  /* Occlusion R Thyroid Art w Extralum Dev, Perc Endo */
"03LU4DZ"  =  "1"  /* Occlusion R Thyroid Art w Intralum Dev, Perc Endo */
"03LU4ZZ"  =  "1"  /* Occlusion of Right Thyroid Artery, Perc Endo Approach */
"03LV0CZ"  =  "1"  /* Occlusion of L Thyroid Art with Extralum Dev, Open Approach */
"03LV0DZ"  =  "1"  /* Occlusion of L Thyroid Art with Intralum Dev, Open Approach */
"03LV0ZZ"  =  "1"  /* Occlusion of Left Thyroid Artery, Open Approach */
"03LV3CZ"  =  "1"  /* Occlusion of L Thyroid Art with Extralum Dev, Perc Approach */
"03LV3DZ"  =  "1"  /* Occlusion of L Thyroid Art with Intralum Dev, Perc Approach */
"03LV3ZZ"  =  "1"  /* Occlusion of Left Thyroid Artery, Percutaneous Approach */
"03LV4CZ"  =  "1"  /* Occlusion L Thyroid Art w Extralum Dev, Perc Endo */
"03LV4DZ"  =  "1"  /* Occlusion L Thyroid Art w Intralum Dev, Perc Endo */
"03LV4ZZ"  =  "1"  /* Occlusion of Left Thyroid Artery, Perc Endo Approach */
"03LY0CZ"  =  "1"  /* Occlusion of Upper Artery with Extralum Dev, Open Approach */
"03LY0DZ"  =  "1"  /* Occlusion of Upper Artery with Intralum Dev, Open Approach */
"03LY0ZZ"  =  "1"  /* Occlusion of Upper Artery, Open Approach */
"03LY3CZ"  =  "1"  /* Occlusion of Upper Artery with Extralum Dev, Perc Approach */
"03LY3DZ"  =  "1"  /* Occlusion of Upper Artery with Intralum Dev, Perc Approach */
"03LY3ZZ"  =  "1"  /* Occlusion of Upper Artery, Percutaneous Approach */
"03LY4CZ"  =  "1"  /* Occlusion of Up Art with Extralum Dev, Perc Endo Approach */
"03LY4DZ"  =  "1"  /* Occlusion of Up Art with Intralum Dev, Perc Endo Approach */
"03LY4ZZ"  =  "1"  /* Occlusion of Upper Artery, Percutaneous Endoscopic Approach */
"03N00ZZ"  =  "1"  /* Release Right Internal Mammary Artery, Open Approach */
"03N03ZZ"  =  "1"  /* Release Right Internal Mammary Artery, Percutaneous Approach */
"03N04ZZ"  =  "1"  /* Release Right Internal Mammary Artery, Perc Endo Approach */
"03N10ZZ"  =  "1"  /* Release Left Internal Mammary Artery, Open Approach */
"03N13ZZ"  =  "1"  /* Release Left Internal Mammary Artery, Percutaneous Approach */
"03N14ZZ"  =  "1"  /* Release Left Internal Mammary Artery, Perc Endo Approach */
"03N20ZZ"  =  "1"  /* Release Innominate Artery, Open Approach */
"03N23ZZ"  =  "1"  /* Release Innominate Artery, Percutaneous Approach */
"03N24ZZ"  =  "1"  /* Release Innominate Artery, Percutaneous Endoscopic Approach */
"03N30ZZ"  =  "1"  /* Release Right Subclavian Artery, Open Approach */
"03N33ZZ"  =  "1"  /* Release Right Subclavian Artery, Percutaneous Approach */
"03N34ZZ"  =  "1"  /* Release Right Subclavian Artery, Perc Endo Approach */
"03N40ZZ"  =  "1"  /* Release Left Subclavian Artery, Open Approach */
"03N43ZZ"  =  "1"  /* Release Left Subclavian Artery, Percutaneous Approach */
"03N44ZZ"  =  "1"  /* Release Left Subclavian Artery, Perc Endo Approach */
"03N50ZZ"  =  "1"  /* Release Right Axillary Artery, Open Approach */
"03N53ZZ"  =  "1"  /* Release Right Axillary Artery, Percutaneous Approach */
"03N54ZZ"  =  "1"  /* Release Right Axillary Artery, Perc Endo Approach */
"03N60ZZ"  =  "1"  /* Release Left Axillary Artery, Open Approach */
"03N63ZZ"  =  "1"  /* Release Left Axillary Artery, Percutaneous Approach */
"03N64ZZ"  =  "1"  /* Release Left Axillary Artery, Perc Endo Approach */
"03N70ZZ"  =  "1"  /* Release Right Brachial Artery, Open Approach */
"03N73ZZ"  =  "1"  /* Release Right Brachial Artery, Percutaneous Approach */
"03N74ZZ"  =  "1"  /* Release Right Brachial Artery, Perc Endo Approach */
"03N80ZZ"  =  "1"  /* Release Left Brachial Artery, Open Approach */
"03N83ZZ"  =  "1"  /* Release Left Brachial Artery, Percutaneous Approach */
"03N84ZZ"  =  "1"  /* Release Left Brachial Artery, Perc Endo Approach */
"03N90ZZ"  =  "1"  /* Release Right Ulnar Artery, Open Approach */
"03N93ZZ"  =  "1"  /* Release Right Ulnar Artery, Percutaneous Approach */
"03N94ZZ"  =  "1"  /* Release Right Ulnar Artery, Percutaneous Endoscopic Approach */
"03NA0ZZ"  =  "1"  /* Release Left Ulnar Artery, Open Approach */
"03NA3ZZ"  =  "1"  /* Release Left Ulnar Artery, Percutaneous Approach */
"03NA4ZZ"  =  "1"  /* Release Left Ulnar Artery, Percutaneous Endoscopic Approach */
"03NB0ZZ"  =  "1"  /* Release Right Radial Artery, Open Approach */
"03NB3ZZ"  =  "1"  /* Release Right Radial Artery, Percutaneous Approach */
"03NB4ZZ"  =  "1"  /* Release Right Radial Artery, Perc Endo Approach */
"03NC0ZZ"  =  "1"  /* Release Left Radial Artery, Open Approach */
"03NC3ZZ"  =  "1"  /* Release Left Radial Artery, Percutaneous Approach */
"03NC4ZZ"  =  "1"  /* Release Left Radial Artery, Percutaneous Endoscopic Approach */
"03ND0ZZ"  =  "1"  /* Release Right Hand Artery, Open Approach */
"03ND3ZZ"  =  "1"  /* Release Right Hand Artery, Percutaneous Approach */
"03ND4ZZ"  =  "1"  /* Release Right Hand Artery, Percutaneous Endoscopic Approach */
"03NF0ZZ"  =  "1"  /* Release Left Hand Artery, Open Approach */
"03NF3ZZ"  =  "1"  /* Release Left Hand Artery, Percutaneous Approach */
"03NF4ZZ"  =  "1"  /* Release Left Hand Artery, Percutaneous Endoscopic Approach */
"03NG0ZZ"  =  "1"  /* Release Intracranial Artery, Open Approach */
"03NG3ZZ"  =  "1"  /* Release Intracranial Artery, Percutaneous Approach */
"03NG4ZZ"  =  "1"  /* Release Intracranial Artery, Perc Endo Approach */
"03NH0ZZ"  =  "1"  /* Release Right Common Carotid Artery, Open Approach */
"03NH3ZZ"  =  "1"  /* Release Right Common Carotid Artery, Percutaneous Approach */
"03NH4ZZ"  =  "1"  /* Release Right Common Carotid Artery, Perc Endo Approach */
"03NJ0ZZ"  =  "1"  /* Release Left Common Carotid Artery, Open Approach */
"03NJ3ZZ"  =  "1"  /* Release Left Common Carotid Artery, Percutaneous Approach */
"03NJ4ZZ"  =  "1"  /* Release Left Common Carotid Artery, Perc Endo Approach */
"03NK0ZZ"  =  "1"  /* Release Right Internal Carotid Artery, Open Approach */
"03NK3ZZ"  =  "1"  /* Release Right Internal Carotid Artery, Percutaneous Approach */
"03NK4ZZ"  =  "1"  /* Release Right Internal Carotid Artery, Perc Endo Approach */
"03NL0ZZ"  =  "1"  /* Release Left Internal Carotid Artery, Open Approach */
"03NL3ZZ"  =  "1"  /* Release Left Internal Carotid Artery, Percutaneous Approach */
"03NL4ZZ"  =  "1"  /* Release Left Internal Carotid Artery, Perc Endo Approach */
"03NM0ZZ"  =  "1"  /* Release Right External Carotid Artery, Open Approach */
"03NM3ZZ"  =  "1"  /* Release Right External Carotid Artery, Percutaneous Approach */
"03NM4ZZ"  =  "1"  /* Release Right External Carotid Artery, Perc Endo Approach */
"03NN0ZZ"  =  "1"  /* Release Left External Carotid Artery, Open Approach */
"03NN3ZZ"  =  "1"  /* Release Left External Carotid Artery, Percutaneous Approach */
"03NN4ZZ"  =  "1"  /* Release Left External Carotid Artery, Perc Endo Approach */
"03NP0ZZ"  =  "1"  /* Release Right Vertebral Artery, Open Approach */
"03NP3ZZ"  =  "1"  /* Release Right Vertebral Artery, Percutaneous Approach */
"03NP4ZZ"  =  "1"  /* Release Right Vertebral Artery, Perc Endo Approach */
"03NQ0ZZ"  =  "1"  /* Release Left Vertebral Artery, Open Approach */
"03NQ3ZZ"  =  "1"  /* Release Left Vertebral Artery, Percutaneous Approach */
"03NQ4ZZ"  =  "1"  /* Release Left Vertebral Artery, Perc Endo Approach */
"03NR0ZZ"  =  "1"  /* Release Face Artery, Open Approach */
"03NR3ZZ"  =  "1"  /* Release Face Artery, Percutaneous Approach */
"03NR4ZZ"  =  "1"  /* Release Face Artery, Percutaneous Endoscopic Approach */
"03NS0ZZ"  =  "1"  /* Release Right Temporal Artery, Open Approach */
"03NS3ZZ"  =  "1"  /* Release Right Temporal Artery, Percutaneous Approach */
"03NS4ZZ"  =  "1"  /* Release Right Temporal Artery, Perc Endo Approach */
"03NT0ZZ"  =  "1"  /* Release Left Temporal Artery, Open Approach */
"03NT3ZZ"  =  "1"  /* Release Left Temporal Artery, Percutaneous Approach */
"03NT4ZZ"  =  "1"  /* Release Left Temporal Artery, Perc Endo Approach */
"03NU0ZZ"  =  "1"  /* Release Right Thyroid Artery, Open Approach */
"03NU3ZZ"  =  "1"  /* Release Right Thyroid Artery, Percutaneous Approach */
"03NU4ZZ"  =  "1"  /* Release Right Thyroid Artery, Perc Endo Approach */
"03NV0ZZ"  =  "1"  /* Release Left Thyroid Artery, Open Approach */
"03NV3ZZ"  =  "1"  /* Release Left Thyroid Artery, Percutaneous Approach */
"03NV4ZZ"  =  "1"  /* Release Left Thyroid Artery, Perc Endo Approach */
"03NY0ZZ"  =  "1"  /* Release Upper Artery, Open Approach */
"03NY3ZZ"  =  "1"  /* Release Upper Artery, Percutaneous Approach */
"03NY4ZZ"  =  "1"  /* Release Upper Artery, Percutaneous Endoscopic Approach */
"03PY00Z"  =  "1"  /* Removal of Drainage Device from Upper Artery, Open Approach */
"03PY02Z"  =  "1"  /* Removal of Monitoring Device from Up Art, Open Approach */
"03PY03Z"  =  "1"  /* Removal of Infusion Device from Upper Artery, Open Approach */
"03PY07Z"  =  "1"  /* Removal of Autol Sub from Up Art, Open Approach */
"03PY0CZ"  =  "1"  /* Removal of Extraluminal Device from Up Art, Open Approach */
"03PY0DZ"  =  "1"  /* Removal of Intraluminal Device from Up Art, Open Approach */
"03PY0JZ"  =  "1"  /* Removal of Synthetic Substitute from Up Art, Open Approach */
"03PY0KZ"  =  "1"  /* Removal of Nonaut Sub from Up Art, Open Approach */
"03PY0MZ"  =  "1"  /* Removal of Stimulator Lead from Upper Artery, Open Approach */
"03PY30Z"  =  "1"  /* Removal of Drainage Device from Upper Artery, Perc Approach */
"03PY32Z"  =  "1"  /* Removal of Monitoring Device from Up Art, Perc Approach */
"03PY33Z"  =  "1"  /* Removal of Infusion Device from Upper Artery, Perc Approach */
"03PY37Z"  =  "1"  /* Removal of Autol Sub from Up Art, Perc Approach */
"03PY3CZ"  =  "1"  /* Removal of Extraluminal Device from Up Art, Perc Approach */
"03PY3DZ"  =  "1"  /* Removal of Intraluminal Device from Up Art, Perc Approach */
"03PY3JZ"  =  "1"  /* Removal of Synthetic Substitute from Up Art, Perc Approach */
"03PY3KZ"  =  "1"  /* Removal of Nonaut Sub from Up Art, Perc Approach */
"03PY3MZ"  =  "1"  /* Removal of Stimulator Lead from Upper Artery, Perc Approach */
"03PY40Z"  =  "1"  /* Removal of Drainage Device from Up Art, Perc Endo Approach */
"03PY42Z"  =  "1"  /* Removal of Monitoring Device from Up Art, Perc Endo Approach */
"03PY43Z"  =  "1"  /* Removal of Infusion Device from Up Art, Perc Endo Approach */
"03PY47Z"  =  "1"  /* Removal of Autol Sub from Up Art, Perc Endo Approach */
"03PY4CZ"  =  "1"  /* Removal of Extralum Dev from Up Art, Perc Endo Approach */
"03PY4DZ"  =  "1"  /* Removal of Intralum Dev from Up Art, Perc Endo Approach */
"03PY4JZ"  =  "1"  /* Removal of Synth Sub from Up Art, Perc Endo Approach */
"03PY4KZ"  =  "1"  /* Removal of Nonaut Sub from Up Art, Perc Endo Approach */
"03PY4MZ"  =  "1"  /* Removal of Stimulator Lead from Up Art, Perc Endo Approach */
"03Q00ZZ"  =  "1"  /* Repair Right Internal Mammary Artery, Open Approach */
"03Q03ZZ"  =  "1"  /* Repair Right Internal Mammary Artery, Percutaneous Approach */
"03Q04ZZ"  =  "1"  /* Repair Right Internal Mammary Artery, Perc Endo Approach */
"03Q10ZZ"  =  "1"  /* Repair Left Internal Mammary Artery, Open Approach */
"03Q13ZZ"  =  "1"  /* Repair Left Internal Mammary Artery, Percutaneous Approach */
"03Q14ZZ"  =  "1"  /* Repair Left Internal Mammary Artery, Perc Endo Approach */
"03Q20ZZ"  =  "1"  /* Repair Innominate Artery, Open Approach */
"03Q23ZZ"  =  "1"  /* Repair Innominate Artery, Percutaneous Approach */
"03Q24ZZ"  =  "1"  /* Repair Innominate Artery, Percutaneous Endoscopic Approach */
"03Q30ZZ"  =  "1"  /* Repair Right Subclavian Artery, Open Approach */
"03Q33ZZ"  =  "1"  /* Repair Right Subclavian Artery, Percutaneous Approach */
"03Q34ZZ"  =  "1"  /* Repair Right Subclavian Artery, Perc Endo Approach */
"03Q40ZZ"  =  "1"  /* Repair Left Subclavian Artery, Open Approach */
"03Q43ZZ"  =  "1"  /* Repair Left Subclavian Artery, Percutaneous Approach */
"03Q44ZZ"  =  "1"  /* Repair Left Subclavian Artery, Perc Endo Approach */
"03Q50ZZ"  =  "1"  /* Repair Right Axillary Artery, Open Approach */
"03Q53ZZ"  =  "1"  /* Repair Right Axillary Artery, Percutaneous Approach */
"03Q54ZZ"  =  "1"  /* Repair Right Axillary Artery, Perc Endo Approach */
"03Q60ZZ"  =  "1"  /* Repair Left Axillary Artery, Open Approach */
"03Q63ZZ"  =  "1"  /* Repair Left Axillary Artery, Percutaneous Approach */
"03Q64ZZ"  =  "1"  /* Repair Left Axillary Artery, Perc Endo Approach */
"03Q70ZZ"  =  "1"  /* Repair Right Brachial Artery, Open Approach */
"03Q73ZZ"  =  "1"  /* Repair Right Brachial Artery, Percutaneous Approach */
"03Q74ZZ"  =  "1"  /* Repair Right Brachial Artery, Perc Endo Approach */
"03Q80ZZ"  =  "1"  /* Repair Left Brachial Artery, Open Approach */
"03Q83ZZ"  =  "1"  /* Repair Left Brachial Artery, Percutaneous Approach */
"03Q84ZZ"  =  "1"  /* Repair Left Brachial Artery, Perc Endo Approach */
"03Q90ZZ"  =  "1"  /* Repair Right Ulnar Artery, Open Approach */
"03Q93ZZ"  =  "1"  /* Repair Right Ulnar Artery, Percutaneous Approach */
"03Q94ZZ"  =  "1"  /* Repair Right Ulnar Artery, Percutaneous Endoscopic Approach */
"03QA0ZZ"  =  "1"  /* Repair Left Ulnar Artery, Open Approach */
"03QA3ZZ"  =  "1"  /* Repair Left Ulnar Artery, Percutaneous Approach */
"03QA4ZZ"  =  "1"  /* Repair Left Ulnar Artery, Percutaneous Endoscopic Approach */
"03QB0ZZ"  =  "1"  /* Repair Right Radial Artery, Open Approach */
"03QB3ZZ"  =  "1"  /* Repair Right Radial Artery, Percutaneous Approach */
"03QB4ZZ"  =  "1"  /* Repair Right Radial Artery, Percutaneous Endoscopic Approach */
"03QC0ZZ"  =  "1"  /* Repair Left Radial Artery, Open Approach */
"03QC3ZZ"  =  "1"  /* Repair Left Radial Artery, Percutaneous Approach */
"03QC4ZZ"  =  "1"  /* Repair Left Radial Artery, Percutaneous Endoscopic Approach */
"03QD0ZZ"  =  "1"  /* Repair Right Hand Artery, Open Approach */
"03QD3ZZ"  =  "1"  /* Repair Right Hand Artery, Percutaneous Approach */
"03QD4ZZ"  =  "1"  /* Repair Right Hand Artery, Percutaneous Endoscopic Approach */
"03QF0ZZ"  =  "1"  /* Repair Left Hand Artery, Open Approach */
"03QF3ZZ"  =  "1"  /* Repair Left Hand Artery, Percutaneous Approach */
"03QF4ZZ"  =  "1"  /* Repair Left Hand Artery, Percutaneous Endoscopic Approach */
"03QG0ZZ"  =  "1"  /* Repair Intracranial Artery, Open Approach */
"03QG3ZZ"  =  "1"  /* Repair Intracranial Artery, Percutaneous Approach */
"03QG4ZZ"  =  "1"  /* Repair Intracranial Artery, Percutaneous Endoscopic Approach */
"03QH0ZZ"  =  "1"  /* Repair Right Common Carotid Artery, Open Approach */
"03QH3ZZ"  =  "1"  /* Repair Right Common Carotid Artery, Percutaneous Approach */
"03QH4ZZ"  =  "1"  /* Repair Right Common Carotid Artery, Perc Endo Approach */
"03QJ0ZZ"  =  "1"  /* Repair Left Common Carotid Artery, Open Approach */
"03QJ3ZZ"  =  "1"  /* Repair Left Common Carotid Artery, Percutaneous Approach */
"03QJ4ZZ"  =  "1"  /* Repair Left Common Carotid Artery, Perc Endo Approach */
"03QK0ZZ"  =  "1"  /* Repair Right Internal Carotid Artery, Open Approach */
"03QK3ZZ"  =  "1"  /* Repair Right Internal Carotid Artery, Percutaneous Approach */
"03QK4ZZ"  =  "1"  /* Repair Right Internal Carotid Artery, Perc Endo Approach */
"03QL0ZZ"  =  "1"  /* Repair Left Internal Carotid Artery, Open Approach */
"03QL3ZZ"  =  "1"  /* Repair Left Internal Carotid Artery, Percutaneous Approach */
"03QL4ZZ"  =  "1"  /* Repair Left Internal Carotid Artery, Perc Endo Approach */
"03QM0ZZ"  =  "1"  /* Repair Right External Carotid Artery, Open Approach */
"03QM3ZZ"  =  "1"  /* Repair Right External Carotid Artery, Percutaneous Approach */
"03QM4ZZ"  =  "1"  /* Repair Right External Carotid Artery, Perc Endo Approach */
"03QN0ZZ"  =  "1"  /* Repair Left External Carotid Artery, Open Approach */
"03QN3ZZ"  =  "1"  /* Repair Left External Carotid Artery, Percutaneous Approach */
"03QN4ZZ"  =  "1"  /* Repair Left External Carotid Artery, Perc Endo Approach */
"03QP0ZZ"  =  "1"  /* Repair Right Vertebral Artery, Open Approach */
"03QP3ZZ"  =  "1"  /* Repair Right Vertebral Artery, Percutaneous Approach */
"03QP4ZZ"  =  "1"  /* Repair Right Vertebral Artery, Perc Endo Approach */
"03QQ0ZZ"  =  "1"  /* Repair Left Vertebral Artery, Open Approach */
"03QQ3ZZ"  =  "1"  /* Repair Left Vertebral Artery, Percutaneous Approach */
"03QQ4ZZ"  =  "1"  /* Repair Left Vertebral Artery, Perc Endo Approach */
"03QR0ZZ"  =  "1"  /* Repair Face Artery, Open Approach */
"03QR3ZZ"  =  "1"  /* Repair Face Artery, Percutaneous Approach */
"03QR4ZZ"  =  "1"  /* Repair Face Artery, Percutaneous Endoscopic Approach */
"03QS0ZZ"  =  "1"  /* Repair Right Temporal Artery, Open Approach */
"03QS3ZZ"  =  "1"  /* Repair Right Temporal Artery, Percutaneous Approach */
"03QS4ZZ"  =  "1"  /* Repair Right Temporal Artery, Perc Endo Approach */
"03QT0ZZ"  =  "1"  /* Repair Left Temporal Artery, Open Approach */
"03QT3ZZ"  =  "1"  /* Repair Left Temporal Artery, Percutaneous Approach */
"03QT4ZZ"  =  "1"  /* Repair Left Temporal Artery, Perc Endo Approach */
"03QU0ZZ"  =  "1"  /* Repair Right Thyroid Artery, Open Approach */
"03QU3ZZ"  =  "1"  /* Repair Right Thyroid Artery, Percutaneous Approach */
"03QU4ZZ"  =  "1"  /* Repair Right Thyroid Artery, Perc Endo Approach */
"03QV0ZZ"  =  "1"  /* Repair Left Thyroid Artery, Open Approach */
"03QV3ZZ"  =  "1"  /* Repair Left Thyroid Artery, Percutaneous Approach */
"03QV4ZZ"  =  "1"  /* Repair Left Thyroid Artery, Percutaneous Endoscopic Approach */
"03QY0ZZ"  =  "1"  /* Repair Upper Artery, Open Approach */
"03QY3ZZ"  =  "1"  /* Repair Upper Artery, Percutaneous Approach */
"03QY4ZZ"  =  "1"  /* Repair Upper Artery, Percutaneous Endoscopic Approach */
"03R007Z"  =  "1"  /* Replacement of R Int Mamm Art with Autol Sub, Open Approach */
"03R00JZ"  =  "1"  /* Replacement of R Int Mamm Art with Synth Sub, Open Approach */
"03R00KZ"  =  "1"  /* Replacement of R Int Mamm Art with Nonaut Sub, Open Approach */
"03R047Z"  =  "1"  /* Replace of R Int Mamm Art with Autol Sub, Perc Endo Approach */
"03R04JZ"  =  "1"  /* Replace of R Int Mamm Art with Synth Sub, Perc Endo Approach */
"03R04KZ"  =  "1"  /* Replace R Int Mamm Art w Nonaut Sub, Perc Endo */
"03R107Z"  =  "1"  /* Replacement of L Int Mamm Art with Autol Sub, Open Approach */
"03R10JZ"  =  "1"  /* Replacement of L Int Mamm Art with Synth Sub, Open Approach */
"03R10KZ"  =  "1"  /* Replacement of L Int Mamm Art with Nonaut Sub, Open Approach */
"03R147Z"  =  "1"  /* Replace of L Int Mamm Art with Autol Sub, Perc Endo Approach */
"03R14JZ"  =  "1"  /* Replace of L Int Mamm Art with Synth Sub, Perc Endo Approach */
"03R14KZ"  =  "1"  /* Replace L Int Mamm Art w Nonaut Sub, Perc Endo */
"03R207Z"  =  "1"  /* Replacement of Innom Art with Autol Sub, Open Approach */
"03R20JZ"  =  "1"  /* Replacement of Innom Art with Synth Sub, Open Approach */
"03R20KZ"  =  "1"  /* Replacement of Innom Art with Nonaut Sub, Open Approach */
"03R247Z"  =  "1"  /* Replacement of Innom Art with Autol Sub, Perc Endo Approach */
"03R24JZ"  =  "1"  /* Replacement of Innom Art with Synth Sub, Perc Endo Approach */
"03R24KZ"  =  "1"  /* Replacement of Innom Art with Nonaut Sub, Perc Endo Approach */
"03R307Z"  =  "1"  /* Replacement of R Subclav Art with Autol Sub, Open Approach */
"03R30JZ"  =  "1"  /* Replacement of R Subclav Art with Synth Sub, Open Approach */
"03R30KZ"  =  "1"  /* Replacement of R Subclav Art with Nonaut Sub, Open Approach */
"03R347Z"  =  "1"  /* Replace of R Subclav Art with Autol Sub, Perc Endo Approach */
"03R34JZ"  =  "1"  /* Replace of R Subclav Art with Synth Sub, Perc Endo Approach */
"03R34KZ"  =  "1"  /* Replace of R Subclav Art with Nonaut Sub, Perc Endo Approach */
"03R407Z"  =  "1"  /* Replacement of L Subclav Art with Autol Sub, Open Approach */
"03R40JZ"  =  "1"  /* Replacement of L Subclav Art with Synth Sub, Open Approach */
"03R40KZ"  =  "1"  /* Replacement of L Subclav Art with Nonaut Sub, Open Approach */
"03R447Z"  =  "1"  /* Replace of L Subclav Art with Autol Sub, Perc Endo Approach */
"03R44JZ"  =  "1"  /* Replace of L Subclav Art with Synth Sub, Perc Endo Approach */
"03R44KZ"  =  "1"  /* Replace of L Subclav Art with Nonaut Sub, Perc Endo Approach */
"03R507Z"  =  "1"  /* Replacement of R Axilla Art with Autol Sub, Open Approach */
"03R50JZ"  =  "1"  /* Replacement of R Axilla Art with Synth Sub, Open Approach */
"03R50KZ"  =  "1"  /* Replacement of R Axilla Art with Nonaut Sub, Open Approach */
"03R547Z"  =  "1"  /* Replace of R Axilla Art with Autol Sub, Perc Endo Approach */
"03R54JZ"  =  "1"  /* Replace of R Axilla Art with Synth Sub, Perc Endo Approach */
"03R54KZ"  =  "1"  /* Replace of R Axilla Art with Nonaut Sub, Perc Endo Approach */
"03R607Z"  =  "1"  /* Replacement of L Axilla Art with Autol Sub, Open Approach */
"03R60JZ"  =  "1"  /* Replacement of L Axilla Art with Synth Sub, Open Approach */
"03R60KZ"  =  "1"  /* Replacement of L Axilla Art with Nonaut Sub, Open Approach */
"03R647Z"  =  "1"  /* Replace of L Axilla Art with Autol Sub, Perc Endo Approach */
"03R64JZ"  =  "1"  /* Replace of L Axilla Art with Synth Sub, Perc Endo Approach */
"03R64KZ"  =  "1"  /* Replace of L Axilla Art with Nonaut Sub, Perc Endo Approach */
"03R707Z"  =  "1"  /* Replacement of R Brach Art with Autol Sub, Open Approach */
"03R70JZ"  =  "1"  /* Replacement of R Brach Art with Synth Sub, Open Approach */
"03R70KZ"  =  "1"  /* Replacement of R Brach Art with Nonaut Sub, Open Approach */
"03R747Z"  =  "1"  /* Replace of R Brach Art with Autol Sub, Perc Endo Approach */
"03R74JZ"  =  "1"  /* Replace of R Brach Art with Synth Sub, Perc Endo Approach */
"03R74KZ"  =  "1"  /* Replace of R Brach Art with Nonaut Sub, Perc Endo Approach */
"03R807Z"  =  "1"  /* Replacement of L Brach Art with Autol Sub, Open Approach */
"03R80JZ"  =  "1"  /* Replacement of L Brach Art with Synth Sub, Open Approach */
"03R80KZ"  =  "1"  /* Replacement of L Brach Art with Nonaut Sub, Open Approach */
"03R847Z"  =  "1"  /* Replace of L Brach Art with Autol Sub, Perc Endo Approach */
"03R84JZ"  =  "1"  /* Replace of L Brach Art with Synth Sub, Perc Endo Approach */
"03R84KZ"  =  "1"  /* Replace of L Brach Art with Nonaut Sub, Perc Endo Approach */
"03R907Z"  =  "1"  /* Replacement of R Ulnar Art with Autol Sub, Open Approach */
"03R90JZ"  =  "1"  /* Replacement of R Ulnar Art with Synth Sub, Open Approach */
"03R90KZ"  =  "1"  /* Replacement of R Ulnar Art with Nonaut Sub, Open Approach */
"03R947Z"  =  "1"  /* Replace of R Ulnar Art with Autol Sub, Perc Endo Approach */
"03R94JZ"  =  "1"  /* Replace of R Ulnar Art with Synth Sub, Perc Endo Approach */
"03R94KZ"  =  "1"  /* Replace of R Ulnar Art with Nonaut Sub, Perc Endo Approach */
"03RA07Z"  =  "1"  /* Replacement of L Ulnar Art with Autol Sub, Open Approach */
"03RA0JZ"  =  "1"  /* Replacement of L Ulnar Art with Synth Sub, Open Approach */
"03RA0KZ"  =  "1"  /* Replacement of L Ulnar Art with Nonaut Sub, Open Approach */
"03RA47Z"  =  "1"  /* Replace of L Ulnar Art with Autol Sub, Perc Endo Approach */
"03RA4JZ"  =  "1"  /* Replace of L Ulnar Art with Synth Sub, Perc Endo Approach */
"03RA4KZ"  =  "1"  /* Replace of L Ulnar Art with Nonaut Sub, Perc Endo Approach */
"03RB07Z"  =  "1"  /* Replacement of R Radial Art with Autol Sub, Open Approach */
"03RB0JZ"  =  "1"  /* Replacement of R Radial Art with Synth Sub, Open Approach */
"03RB0KZ"  =  "1"  /* Replacement of R Radial Art with Nonaut Sub, Open Approach */
"03RB47Z"  =  "1"  /* Replace of R Radial Art with Autol Sub, Perc Endo Approach */
"03RB4JZ"  =  "1"  /* Replace of R Radial Art with Synth Sub, Perc Endo Approach */
"03RB4KZ"  =  "1"  /* Replace of R Radial Art with Nonaut Sub, Perc Endo Approach */
"03RC07Z"  =  "1"  /* Replacement of L Radial Art with Autol Sub, Open Approach */
"03RC0JZ"  =  "1"  /* Replacement of L Radial Art with Synth Sub, Open Approach */
"03RC0KZ"  =  "1"  /* Replacement of L Radial Art with Nonaut Sub, Open Approach */
"03RC47Z"  =  "1"  /* Replace of L Radial Art with Autol Sub, Perc Endo Approach */
"03RC4JZ"  =  "1"  /* Replace of L Radial Art with Synth Sub, Perc Endo Approach */
"03RC4KZ"  =  "1"  /* Replace of L Radial Art with Nonaut Sub, Perc Endo Approach */
"03RD07Z"  =  "1"  /* Replacement of R Hand Art with Autol Sub, Open Approach */
"03RD0JZ"  =  "1"  /* Replacement of R Hand Art with Synth Sub, Open Approach */
"03RD0KZ"  =  "1"  /* Replacement of R Hand Art with Nonaut Sub, Open Approach */
"03RD47Z"  =  "1"  /* Replacement of R Hand Art with Autol Sub, Perc Endo Approach */
"03RD4JZ"  =  "1"  /* Replacement of R Hand Art with Synth Sub, Perc Endo Approach */
"03RD4KZ"  =  "1"  /* Replace of R Hand Art with Nonaut Sub, Perc Endo Approach */
"03RF07Z"  =  "1"  /* Replacement of L Hand Art with Autol Sub, Open Approach */
"03RF0JZ"  =  "1"  /* Replacement of L Hand Art with Synth Sub, Open Approach */
"03RF0KZ"  =  "1"  /* Replacement of L Hand Art with Nonaut Sub, Open Approach */
"03RF47Z"  =  "1"  /* Replacement of L Hand Art with Autol Sub, Perc Endo Approach */
"03RF4JZ"  =  "1"  /* Replacement of L Hand Art with Synth Sub, Perc Endo Approach */
"03RF4KZ"  =  "1"  /* Replace of L Hand Art with Nonaut Sub, Perc Endo Approach */
"03RG07Z"  =  "1"  /* Replacement of Intracran Art with Autol Sub, Open Approach */
"03RG0JZ"  =  "1"  /* Replacement of Intracran Art with Synth Sub, Open Approach */
"03RG0KZ"  =  "1"  /* Replacement of Intracran Art with Nonaut Sub, Open Approach */
"03RG47Z"  =  "1"  /* Replace of Intracran Art with Autol Sub, Perc Endo Approach */
"03RG4JZ"  =  "1"  /* Replace of Intracran Art with Synth Sub, Perc Endo Approach */
"03RG4KZ"  =  "1"  /* Replace of Intracran Art with Nonaut Sub, Perc Endo Approach */
"03RH07Z"  =  "1"  /* Replacement of R Com Carotid with Autol Sub, Open Approach */
"03RH0JZ"  =  "1"  /* Replacement of R Com Carotid with Synth Sub, Open Approach */
"03RH0KZ"  =  "1"  /* Replacement of R Com Carotid with Nonaut Sub, Open Approach */
"03RH47Z"  =  "1"  /* Replace of R Com Carotid with Autol Sub, Perc Endo Approach */
"03RH4JZ"  =  "1"  /* Replace of R Com Carotid with Synth Sub, Perc Endo Approach */
"03RH4KZ"  =  "1"  /* Replace of R Com Carotid with Nonaut Sub, Perc Endo Approach */
"03RJ07Z"  =  "1"  /* Replacement of L Com Carotid with Autol Sub, Open Approach */
"03RJ0JZ"  =  "1"  /* Replacement of L Com Carotid with Synth Sub, Open Approach */
"03RJ0KZ"  =  "1"  /* Replacement of L Com Carotid with Nonaut Sub, Open Approach */
"03RJ47Z"  =  "1"  /* Replace of L Com Carotid with Autol Sub, Perc Endo Approach */
"03RJ4JZ"  =  "1"  /* Replace of L Com Carotid with Synth Sub, Perc Endo Approach */
"03RJ4KZ"  =  "1"  /* Replace of L Com Carotid with Nonaut Sub, Perc Endo Approach */
"03RK07Z"  =  "1"  /* Replacement of R Int Carotid with Autol Sub, Open Approach */
"03RK0JZ"  =  "1"  /* Replacement of R Int Carotid with Synth Sub, Open Approach */
"03RK0KZ"  =  "1"  /* Replacement of R Int Carotid with Nonaut Sub, Open Approach */
"03RK47Z"  =  "1"  /* Replace of R Int Carotid with Autol Sub, Perc Endo Approach */
"03RK4JZ"  =  "1"  /* Replace of R Int Carotid with Synth Sub, Perc Endo Approach */
"03RK4KZ"  =  "1"  /* Replace of R Int Carotid with Nonaut Sub, Perc Endo Approach */
"03RL07Z"  =  "1"  /* Replacement of L Int Carotid with Autol Sub, Open Approach */
"03RL0JZ"  =  "1"  /* Replacement of L Int Carotid with Synth Sub, Open Approach */
"03RL0KZ"  =  "1"  /* Replacement of L Int Carotid with Nonaut Sub, Open Approach */
"03RL47Z"  =  "1"  /* Replace of L Int Carotid with Autol Sub, Perc Endo Approach */
"03RL4JZ"  =  "1"  /* Replace of L Int Carotid with Synth Sub, Perc Endo Approach */
"03RL4KZ"  =  "1"  /* Replace of L Int Carotid with Nonaut Sub, Perc Endo Approach */
"03RM07Z"  =  "1"  /* Replacement of R Ext Carotid with Autol Sub, Open Approach */
"03RM0JZ"  =  "1"  /* Replacement of R Ext Carotid with Synth Sub, Open Approach */
"03RM0KZ"  =  "1"  /* Replacement of R Ext Carotid with Nonaut Sub, Open Approach */
"03RM47Z"  =  "1"  /* Replace of R Ext Carotid with Autol Sub, Perc Endo Approach */
"03RM4JZ"  =  "1"  /* Replace of R Ext Carotid with Synth Sub, Perc Endo Approach */
"03RM4KZ"  =  "1"  /* Replace of R Ext Carotid with Nonaut Sub, Perc Endo Approach */
"03RN07Z"  =  "1"  /* Replacement of L Ext Carotid with Autol Sub, Open Approach */
"03RN0JZ"  =  "1"  /* Replacement of L Ext Carotid with Synth Sub, Open Approach */
"03RN0KZ"  =  "1"  /* Replacement of L Ext Carotid with Nonaut Sub, Open Approach */
"03RN47Z"  =  "1"  /* Replace of L Ext Carotid with Autol Sub, Perc Endo Approach */
"03RN4JZ"  =  "1"  /* Replace of L Ext Carotid with Synth Sub, Perc Endo Approach */
"03RN4KZ"  =  "1"  /* Replace of L Ext Carotid with Nonaut Sub, Perc Endo Approach */
"03RP07Z"  =  "1"  /* Replacement of R Verteb Art with Autol Sub, Open Approach */
"03RP0JZ"  =  "1"  /* Replacement of R Verteb Art with Synth Sub, Open Approach */
"03RP0KZ"  =  "1"  /* Replacement of R Verteb Art with Nonaut Sub, Open Approach */
"03RP47Z"  =  "1"  /* Replace of R Verteb Art with Autol Sub, Perc Endo Approach */
"03RP4JZ"  =  "1"  /* Replace of R Verteb Art with Synth Sub, Perc Endo Approach */
"03RP4KZ"  =  "1"  /* Replace of R Verteb Art with Nonaut Sub, Perc Endo Approach */
"03RQ07Z"  =  "1"  /* Replacement of L Verteb Art with Autol Sub, Open Approach */
"03RQ0JZ"  =  "1"  /* Replacement of L Verteb Art with Synth Sub, Open Approach */
"03RQ0KZ"  =  "1"  /* Replacement of L Verteb Art with Nonaut Sub, Open Approach */
"03RQ47Z"  =  "1"  /* Replace of L Verteb Art with Autol Sub, Perc Endo Approach */
"03RQ4JZ"  =  "1"  /* Replace of L Verteb Art with Synth Sub, Perc Endo Approach */
"03RQ4KZ"  =  "1"  /* Replace of L Verteb Art with Nonaut Sub, Perc Endo Approach */
"03RR07Z"  =  "1"  /* Replacement of Face Artery with Autol Sub, Open Approach */
"03RR0JZ"  =  "1"  /* Replacement of Face Artery with Synth Sub, Open Approach */
"03RR0KZ"  =  "1"  /* Replacement of Face Artery with Nonaut Sub, Open Approach */
"03RR47Z"  =  "1"  /* Replacement of Face Art with Autol Sub, Perc Endo Approach */
"03RR4JZ"  =  "1"  /* Replacement of Face Art with Synth Sub, Perc Endo Approach */
"03RR4KZ"  =  "1"  /* Replacement of Face Art with Nonaut Sub, Perc Endo Approach */
"03RS07Z"  =  "1"  /* Replacement of R Temporal Art with Autol Sub, Open Approach */
"03RS0JZ"  =  "1"  /* Replacement of R Temporal Art with Synth Sub, Open Approach */
"03RS0KZ"  =  "1"  /* Replacement of R Temporal Art with Nonaut Sub, Open Approach */
"03RS47Z"  =  "1"  /* Replace of R Temporal Art with Autol Sub, Perc Endo Approach */
"03RS4JZ"  =  "1"  /* Replace of R Temporal Art with Synth Sub, Perc Endo Approach */
"03RS4KZ"  =  "1"  /* Replace R Temporal Art w Nonaut Sub, Perc Endo */
"03RT07Z"  =  "1"  /* Replacement of L Temporal Art with Autol Sub, Open Approach */
"03RT0JZ"  =  "1"  /* Replacement of L Temporal Art with Synth Sub, Open Approach */
"03RT0KZ"  =  "1"  /* Replacement of L Temporal Art with Nonaut Sub, Open Approach */
"03RT47Z"  =  "1"  /* Replace of L Temporal Art with Autol Sub, Perc Endo Approach */
"03RT4JZ"  =  "1"  /* Replace of L Temporal Art with Synth Sub, Perc Endo Approach */
"03RT4KZ"  =  "1"  /* Replace L Temporal Art w Nonaut Sub, Perc Endo */
"03RU07Z"  =  "1"  /* Replacement of R Thyroid Art with Autol Sub, Open Approach */
"03RU0JZ"  =  "1"  /* Replacement of R Thyroid Art with Synth Sub, Open Approach */
"03RU0KZ"  =  "1"  /* Replacement of R Thyroid Art with Nonaut Sub, Open Approach */
"03RU47Z"  =  "1"  /* Replace of R Thyroid Art with Autol Sub, Perc Endo Approach */
"03RU4JZ"  =  "1"  /* Replace of R Thyroid Art with Synth Sub, Perc Endo Approach */
"03RU4KZ"  =  "1"  /* Replace of R Thyroid Art with Nonaut Sub, Perc Endo Approach */
"03RV07Z"  =  "1"  /* Replacement of L Thyroid Art with Autol Sub, Open Approach */
"03RV0JZ"  =  "1"  /* Replacement of L Thyroid Art with Synth Sub, Open Approach */
"03RV0KZ"  =  "1"  /* Replacement of L Thyroid Art with Nonaut Sub, Open Approach */
"03RV47Z"  =  "1"  /* Replace of L Thyroid Art with Autol Sub, Perc Endo Approach */
"03RV4JZ"  =  "1"  /* Replace of L Thyroid Art with Synth Sub, Perc Endo Approach */
"03RV4KZ"  =  "1"  /* Replace of L Thyroid Art with Nonaut Sub, Perc Endo Approach */
"03RY07Z"  =  "1"  /* Replacement of Upper Artery with Autol Sub, Open Approach */
"03RY0JZ"  =  "1"  /* Replacement of Upper Artery with Synth Sub, Open Approach */
"03RY0KZ"  =  "1"  /* Replacement of Upper Artery with Nonaut Sub, Open Approach */
"03RY47Z"  =  "1"  /* Replacement of Up Art with Autol Sub, Perc Endo Approach */
"03RY4JZ"  =  "1"  /* Replacement of Up Art with Synth Sub, Perc Endo Approach */
"03RY4KZ"  =  "1"  /* Replacement of Up Art with Nonaut Sub, Perc Endo Approach */
"03S00ZZ"  =  "1"  /* Reposition Right Internal Mammary Artery, Open Approach */
"03S03ZZ"  =  "1"  /* Reposition Right Internal Mammary Artery, Perc Approach */
"03S04ZZ"  =  "1"  /* Reposition Right Internal Mammary Artery, Perc Endo Approach */
"03S10ZZ"  =  "1"  /* Reposition Left Internal Mammary Artery, Open Approach */
"03S13ZZ"  =  "1"  /* Reposition Left Internal Mammary Artery, Perc Approach */
"03S14ZZ"  =  "1"  /* Reposition Left Internal Mammary Artery, Perc Endo Approach */
"03S20ZZ"  =  "1"  /* Reposition Innominate Artery, Open Approach */
"03S23ZZ"  =  "1"  /* Reposition Innominate Artery, Percutaneous Approach */
"03S24ZZ"  =  "1"  /* Reposition Innominate Artery, Perc Endo Approach */
"03S30ZZ"  =  "1"  /* Reposition Right Subclavian Artery, Open Approach */
"03S33ZZ"  =  "1"  /* Reposition Right Subclavian Artery, Percutaneous Approach */
"03S34ZZ"  =  "1"  /* Reposition Right Subclavian Artery, Perc Endo Approach */
"03S40ZZ"  =  "1"  /* Reposition Left Subclavian Artery, Open Approach */
"03S43ZZ"  =  "1"  /* Reposition Left Subclavian Artery, Percutaneous Approach */
"03S44ZZ"  =  "1"  /* Reposition Left Subclavian Artery, Perc Endo Approach */
"03S50ZZ"  =  "1"  /* Reposition Right Axillary Artery, Open Approach */
"03S53ZZ"  =  "1"  /* Reposition Right Axillary Artery, Percutaneous Approach */
"03S54ZZ"  =  "1"  /* Reposition Right Axillary Artery, Perc Endo Approach */
"03S60ZZ"  =  "1"  /* Reposition Left Axillary Artery, Open Approach */
"03S63ZZ"  =  "1"  /* Reposition Left Axillary Artery, Percutaneous Approach */
"03S64ZZ"  =  "1"  /* Reposition Left Axillary Artery, Perc Endo Approach */
"03S70ZZ"  =  "1"  /* Reposition Right Brachial Artery, Open Approach */
"03S73ZZ"  =  "1"  /* Reposition Right Brachial Artery, Percutaneous Approach */
"03S74ZZ"  =  "1"  /* Reposition Right Brachial Artery, Perc Endo Approach */
"03S80ZZ"  =  "1"  /* Reposition Left Brachial Artery, Open Approach */
"03S83ZZ"  =  "1"  /* Reposition Left Brachial Artery, Percutaneous Approach */
"03S84ZZ"  =  "1"  /* Reposition Left Brachial Artery, Perc Endo Approach */
"03S90ZZ"  =  "1"  /* Reposition Right Ulnar Artery, Open Approach */
"03S93ZZ"  =  "1"  /* Reposition Right Ulnar Artery, Percutaneous Approach */
"03S94ZZ"  =  "1"  /* Reposition Right Ulnar Artery, Perc Endo Approach */
"03SA0ZZ"  =  "1"  /* Reposition Left Ulnar Artery, Open Approach */
"03SA3ZZ"  =  "1"  /* Reposition Left Ulnar Artery, Percutaneous Approach */
"03SA4ZZ"  =  "1"  /* Reposition Left Ulnar Artery, Perc Endo Approach */
"03SB0ZZ"  =  "1"  /* Reposition Right Radial Artery, Open Approach */
"03SB3ZZ"  =  "1"  /* Reposition Right Radial Artery, Percutaneous Approach */
"03SB4ZZ"  =  "1"  /* Reposition Right Radial Artery, Perc Endo Approach */
"03SC0ZZ"  =  "1"  /* Reposition Left Radial Artery, Open Approach */
"03SC3ZZ"  =  "1"  /* Reposition Left Radial Artery, Percutaneous Approach */
"03SC4ZZ"  =  "1"  /* Reposition Left Radial Artery, Perc Endo Approach */
"03SD0ZZ"  =  "1"  /* Reposition Right Hand Artery, Open Approach */
"03SD3ZZ"  =  "1"  /* Reposition Right Hand Artery, Percutaneous Approach */
"03SD4ZZ"  =  "1"  /* Reposition Right Hand Artery, Perc Endo Approach */
"03SF0ZZ"  =  "1"  /* Reposition Left Hand Artery, Open Approach */
"03SF3ZZ"  =  "1"  /* Reposition Left Hand Artery, Percutaneous Approach */
"03SF4ZZ"  =  "1"  /* Reposition Left Hand Artery, Perc Endo Approach */
"03SG0ZZ"  =  "1"  /* Reposition Intracranial Artery, Open Approach */
"03SG3ZZ"  =  "1"  /* Reposition Intracranial Artery, Percutaneous Approach */
"03SG4ZZ"  =  "1"  /* Reposition Intracranial Artery, Perc Endo Approach */
"03SH0ZZ"  =  "1"  /* Reposition Right Common Carotid Artery, Open Approach */
"03SH3ZZ"  =  "1"  /* Reposition Right Common Carotid Artery, Perc Approach */
"03SH4ZZ"  =  "1"  /* Reposition Right Common Carotid Artery, Perc Endo Approach */
"03SJ0ZZ"  =  "1"  /* Reposition Left Common Carotid Artery, Open Approach */
"03SJ3ZZ"  =  "1"  /* Reposition Left Common Carotid Artery, Percutaneous Approach */
"03SJ4ZZ"  =  "1"  /* Reposition Left Common Carotid Artery, Perc Endo Approach */
"03SK0ZZ"  =  "1"  /* Reposition Right Internal Carotid Artery, Open Approach */
"03SK3ZZ"  =  "1"  /* Reposition Right Internal Carotid Artery, Perc Approach */
"03SK4ZZ"  =  "1"  /* Reposition Right Internal Carotid Artery, Perc Endo Approach */
"03SL0ZZ"  =  "1"  /* Reposition Left Internal Carotid Artery, Open Approach */
"03SL3ZZ"  =  "1"  /* Reposition Left Internal Carotid Artery, Perc Approach */
"03SL4ZZ"  =  "1"  /* Reposition Left Internal Carotid Artery, Perc Endo Approach */
"03SM0ZZ"  =  "1"  /* Reposition Right External Carotid Artery, Open Approach */
"03SM3ZZ"  =  "1"  /* Reposition Right External Carotid Artery, Perc Approach */
"03SM4ZZ"  =  "1"  /* Reposition Right External Carotid Artery, Perc Endo Approach */
"03SN0ZZ"  =  "1"  /* Reposition Left External Carotid Artery, Open Approach */
"03SN3ZZ"  =  "1"  /* Reposition Left External Carotid Artery, Perc Approach */
"03SN4ZZ"  =  "1"  /* Reposition Left External Carotid Artery, Perc Endo Approach */
"03SP0ZZ"  =  "1"  /* Reposition Right Vertebral Artery, Open Approach */
"03SP3ZZ"  =  "1"  /* Reposition Right Vertebral Artery, Percutaneous Approach */
"03SP4ZZ"  =  "1"  /* Reposition Right Vertebral Artery, Perc Endo Approach */
"03SQ0ZZ"  =  "1"  /* Reposition Left Vertebral Artery, Open Approach */
"03SQ3ZZ"  =  "1"  /* Reposition Left Vertebral Artery, Percutaneous Approach */
"03SQ4ZZ"  =  "1"  /* Reposition Left Vertebral Artery, Perc Endo Approach */
"03SR0ZZ"  =  "1"  /* Reposition Face Artery, Open Approach */
"03SR3ZZ"  =  "1"  /* Reposition Face Artery, Percutaneous Approach */
"03SR4ZZ"  =  "1"  /* Reposition Face Artery, Percutaneous Endoscopic Approach */
"03SS0ZZ"  =  "1"  /* Reposition Right Temporal Artery, Open Approach */
"03SS3ZZ"  =  "1"  /* Reposition Right Temporal Artery, Percutaneous Approach */
"03SS4ZZ"  =  "1"  /* Reposition Right Temporal Artery, Perc Endo Approach */
"03ST0ZZ"  =  "1"  /* Reposition Left Temporal Artery, Open Approach */
"03ST3ZZ"  =  "1"  /* Reposition Left Temporal Artery, Percutaneous Approach */
"03ST4ZZ"  =  "1"  /* Reposition Left Temporal Artery, Perc Endo Approach */
"03SU0ZZ"  =  "1"  /* Reposition Right Thyroid Artery, Open Approach */
"03SU3ZZ"  =  "1"  /* Reposition Right Thyroid Artery, Percutaneous Approach */
"03SU4ZZ"  =  "1"  /* Reposition Right Thyroid Artery, Perc Endo Approach */
"03SV0ZZ"  =  "1"  /* Reposition Left Thyroid Artery, Open Approach */
"03SV3ZZ"  =  "1"  /* Reposition Left Thyroid Artery, Percutaneous Approach */
"03SV4ZZ"  =  "1"  /* Reposition Left Thyroid Artery, Perc Endo Approach */
"03SY0ZZ"  =  "1"  /* Reposition Upper Artery, Open Approach */
"03SY3ZZ"  =  "1"  /* Reposition Upper Artery, Percutaneous Approach */
"03SY4ZZ"  =  "1"  /* Reposition Upper Artery, Percutaneous Endoscopic Approach */
"03U007Z"  =  "1"  /* Supplement R Int Mamm Art with Autol Sub, Open Approach */
"03U00JZ"  =  "1"  /* Supplement R Int Mamm Art with Synth Sub, Open Approach */
"03U00KZ"  =  "1"  /* Supplement R Int Mamm Art with Nonaut Sub, Open Approach */
"03U037Z"  =  "1"  /* Supplement R Int Mamm Art with Autol Sub, Perc Approach */
"03U03JZ"  =  "1"  /* Supplement R Int Mamm Art with Synth Sub, Perc Approach */
"03U03KZ"  =  "1"  /* Supplement R Int Mamm Art with Nonaut Sub, Perc Approach */
"03U047Z"  =  "1"  /* Supplement R Int Mamm Art with Autol Sub, Perc Endo Approach */
"03U04JZ"  =  "1"  /* Supplement R Int Mamm Art with Synth Sub, Perc Endo Approach */
"03U04KZ"  =  "1"  /* Supplement R Int Mamm Art w Nonaut Sub, Perc Endo */
"03U107Z"  =  "1"  /* Supplement L Int Mamm Art with Autol Sub, Open Approach */
"03U10JZ"  =  "1"  /* Supplement L Int Mamm Art with Synth Sub, Open Approach */
"03U10KZ"  =  "1"  /* Supplement L Int Mamm Art with Nonaut Sub, Open Approach */
"03U137Z"  =  "1"  /* Supplement L Int Mamm Art with Autol Sub, Perc Approach */
"03U13JZ"  =  "1"  /* Supplement L Int Mamm Art with Synth Sub, Perc Approach */
"03U13KZ"  =  "1"  /* Supplement L Int Mamm Art with Nonaut Sub, Perc Approach */
"03U147Z"  =  "1"  /* Supplement L Int Mamm Art with Autol Sub, Perc Endo Approach */
"03U14JZ"  =  "1"  /* Supplement L Int Mamm Art with Synth Sub, Perc Endo Approach */
"03U14KZ"  =  "1"  /* Supplement L Int Mamm Art w Nonaut Sub, Perc Endo */
"03U207Z"  =  "1"  /* Supplement Innominate Artery with Autol Sub, Open Approach */
"03U20JZ"  =  "1"  /* Supplement Innominate Artery with Synth Sub, Open Approach */
"03U20KZ"  =  "1"  /* Supplement Innominate Artery with Nonaut Sub, Open Approach */
"03U237Z"  =  "1"  /* Supplement Innominate Artery with Autol Sub, Perc Approach */
"03U23JZ"  =  "1"  /* Supplement Innominate Artery with Synth Sub, Perc Approach */
"03U23KZ"  =  "1"  /* Supplement Innominate Artery with Nonaut Sub, Perc Approach */
"03U247Z"  =  "1"  /* Supplement Innom Art with Autol Sub, Perc Endo Approach */
"03U24JZ"  =  "1"  /* Supplement Innom Art with Synth Sub, Perc Endo Approach */
"03U24KZ"  =  "1"  /* Supplement Innom Art with Nonaut Sub, Perc Endo Approach */
"03U307Z"  =  "1"  /* Supplement R Subclav Art with Autol Sub, Open Approach */
"03U30JZ"  =  "1"  /* Supplement R Subclav Art with Synth Sub, Open Approach */
"03U30KZ"  =  "1"  /* Supplement R Subclav Art with Nonaut Sub, Open Approach */
"03U337Z"  =  "1"  /* Supplement R Subclav Art with Autol Sub, Perc Approach */
"03U33JZ"  =  "1"  /* Supplement R Subclav Art with Synth Sub, Perc Approach */
"03U33KZ"  =  "1"  /* Supplement R Subclav Art with Nonaut Sub, Perc Approach */
"03U347Z"  =  "1"  /* Supplement R Subclav Art with Autol Sub, Perc Endo Approach */
"03U34JZ"  =  "1"  /* Supplement R Subclav Art with Synth Sub, Perc Endo Approach */
"03U34KZ"  =  "1"  /* Supplement R Subclav Art with Nonaut Sub, Perc Endo Approach */
"03U407Z"  =  "1"  /* Supplement L Subclav Art with Autol Sub, Open Approach */
"03U40JZ"  =  "1"  /* Supplement L Subclav Art with Synth Sub, Open Approach */
"03U40KZ"  =  "1"  /* Supplement L Subclav Art with Nonaut Sub, Open Approach */
"03U437Z"  =  "1"  /* Supplement L Subclav Art with Autol Sub, Perc Approach */
"03U43JZ"  =  "1"  /* Supplement L Subclav Art with Synth Sub, Perc Approach */
"03U43KZ"  =  "1"  /* Supplement L Subclav Art with Nonaut Sub, Perc Approach */
"03U447Z"  =  "1"  /* Supplement L Subclav Art with Autol Sub, Perc Endo Approach */
"03U44JZ"  =  "1"  /* Supplement L Subclav Art with Synth Sub, Perc Endo Approach */
"03U44KZ"  =  "1"  /* Supplement L Subclav Art with Nonaut Sub, Perc Endo Approach */
"03U507Z"  =  "1"  /* Supplement R Axilla Art with Autol Sub, Open Approach */
"03U50JZ"  =  "1"  /* Supplement R Axilla Art with Synth Sub, Open Approach */
"03U50KZ"  =  "1"  /* Supplement R Axilla Art with Nonaut Sub, Open Approach */
"03U537Z"  =  "1"  /* Supplement R Axilla Art with Autol Sub, Perc Approach */
"03U53JZ"  =  "1"  /* Supplement R Axilla Art with Synth Sub, Perc Approach */
"03U53KZ"  =  "1"  /* Supplement R Axilla Art with Nonaut Sub, Perc Approach */
"03U547Z"  =  "1"  /* Supplement R Axilla Art with Autol Sub, Perc Endo Approach */
"03U54JZ"  =  "1"  /* Supplement R Axilla Art with Synth Sub, Perc Endo Approach */
"03U54KZ"  =  "1"  /* Supplement R Axilla Art with Nonaut Sub, Perc Endo Approach */
"03U607Z"  =  "1"  /* Supplement L Axilla Art with Autol Sub, Open Approach */
"03U60JZ"  =  "1"  /* Supplement L Axilla Art with Synth Sub, Open Approach */
"03U60KZ"  =  "1"  /* Supplement L Axilla Art with Nonaut Sub, Open Approach */
"03U637Z"  =  "1"  /* Supplement L Axilla Art with Autol Sub, Perc Approach */
"03U63JZ"  =  "1"  /* Supplement L Axilla Art with Synth Sub, Perc Approach */
"03U63KZ"  =  "1"  /* Supplement L Axilla Art with Nonaut Sub, Perc Approach */
"03U647Z"  =  "1"  /* Supplement L Axilla Art with Autol Sub, Perc Endo Approach */
"03U64JZ"  =  "1"  /* Supplement L Axilla Art with Synth Sub, Perc Endo Approach */
"03U64KZ"  =  "1"  /* Supplement L Axilla Art with Nonaut Sub, Perc Endo Approach */
"03U707Z"  =  "1"  /* Supplement R Brach Art with Autol Sub, Open Approach */
"03U70JZ"  =  "1"  /* Supplement R Brach Art with Synth Sub, Open Approach */
"03U70KZ"  =  "1"  /* Supplement R Brach Art with Nonaut Sub, Open Approach */
"03U737Z"  =  "1"  /* Supplement R Brach Art with Autol Sub, Perc Approach */
"03U73JZ"  =  "1"  /* Supplement R Brach Art with Synth Sub, Perc Approach */
"03U73KZ"  =  "1"  /* Supplement R Brach Art with Nonaut Sub, Perc Approach */
"03U747Z"  =  "1"  /* Supplement R Brach Art with Autol Sub, Perc Endo Approach */
"03U74JZ"  =  "1"  /* Supplement R Brach Art with Synth Sub, Perc Endo Approach */
"03U74KZ"  =  "1"  /* Supplement R Brach Art with Nonaut Sub, Perc Endo Approach */
"03U807Z"  =  "1"  /* Supplement L Brach Art with Autol Sub, Open Approach */
"03U80JZ"  =  "1"  /* Supplement L Brach Art with Synth Sub, Open Approach */
"03U80KZ"  =  "1"  /* Supplement L Brach Art with Nonaut Sub, Open Approach */
"03U837Z"  =  "1"  /* Supplement L Brach Art with Autol Sub, Perc Approach */
"03U83JZ"  =  "1"  /* Supplement L Brach Art with Synth Sub, Perc Approach */
"03U83KZ"  =  "1"  /* Supplement L Brach Art with Nonaut Sub, Perc Approach */
"03U847Z"  =  "1"  /* Supplement L Brach Art with Autol Sub, Perc Endo Approach */
"03U84JZ"  =  "1"  /* Supplement L Brach Art with Synth Sub, Perc Endo Approach */
"03U84KZ"  =  "1"  /* Supplement L Brach Art with Nonaut Sub, Perc Endo Approach */
"03U907Z"  =  "1"  /* Supplement Right Ulnar Artery with Autol Sub, Open Approach */
"03U90JZ"  =  "1"  /* Supplement Right Ulnar Artery with Synth Sub, Open Approach */
"03U90KZ"  =  "1"  /* Supplement Right Ulnar Artery with Nonaut Sub, Open Approach */
"03U937Z"  =  "1"  /* Supplement Right Ulnar Artery with Autol Sub, Perc Approach */
"03U93JZ"  =  "1"  /* Supplement Right Ulnar Artery with Synth Sub, Perc Approach */
"03U93KZ"  =  "1"  /* Supplement Right Ulnar Artery with Nonaut Sub, Perc Approach */
"03U947Z"  =  "1"  /* Supplement R Ulnar Art with Autol Sub, Perc Endo Approach */
"03U94JZ"  =  "1"  /* Supplement R Ulnar Art with Synth Sub, Perc Endo Approach */
"03U94KZ"  =  "1"  /* Supplement R Ulnar Art with Nonaut Sub, Perc Endo Approach */
"03UA07Z"  =  "1"  /* Supplement Left Ulnar Artery with Autol Sub, Open Approach */
"03UA0JZ"  =  "1"  /* Supplement Left Ulnar Artery with Synth Sub, Open Approach */
"03UA0KZ"  =  "1"  /* Supplement Left Ulnar Artery with Nonaut Sub, Open Approach */
"03UA37Z"  =  "1"  /* Supplement Left Ulnar Artery with Autol Sub, Perc Approach */
"03UA3JZ"  =  "1"  /* Supplement Left Ulnar Artery with Synth Sub, Perc Approach */
"03UA3KZ"  =  "1"  /* Supplement Left Ulnar Artery with Nonaut Sub, Perc Approach */
"03UA47Z"  =  "1"  /* Supplement L Ulnar Art with Autol Sub, Perc Endo Approach */
"03UA4JZ"  =  "1"  /* Supplement L Ulnar Art with Synth Sub, Perc Endo Approach */
"03UA4KZ"  =  "1"  /* Supplement L Ulnar Art with Nonaut Sub, Perc Endo Approach */
"03UB07Z"  =  "1"  /* Supplement Right Radial Artery with Autol Sub, Open Approach */
"03UB0JZ"  =  "1"  /* Supplement Right Radial Artery with Synth Sub, Open Approach */
"03UB0KZ"  =  "1"  /* Supplement R Radial Art with Nonaut Sub, Open Approach */
"03UB37Z"  =  "1"  /* Supplement Right Radial Artery with Autol Sub, Perc Approach */
"03UB3JZ"  =  "1"  /* Supplement Right Radial Artery with Synth Sub, Perc Approach */
"03UB3KZ"  =  "1"  /* Supplement R Radial Art with Nonaut Sub, Perc Approach */
"03UB47Z"  =  "1"  /* Supplement R Radial Art with Autol Sub, Perc Endo Approach */
"03UB4JZ"  =  "1"  /* Supplement R Radial Art with Synth Sub, Perc Endo Approach */
"03UB4KZ"  =  "1"  /* Supplement R Radial Art with Nonaut Sub, Perc Endo Approach */
"03UC07Z"  =  "1"  /* Supplement Left Radial Artery with Autol Sub, Open Approach */
"03UC0JZ"  =  "1"  /* Supplement Left Radial Artery with Synth Sub, Open Approach */
"03UC0KZ"  =  "1"  /* Supplement Left Radial Artery with Nonaut Sub, Open Approach */
"03UC37Z"  =  "1"  /* Supplement Left Radial Artery with Autol Sub, Perc Approach */
"03UC3JZ"  =  "1"  /* Supplement Left Radial Artery with Synth Sub, Perc Approach */
"03UC3KZ"  =  "1"  /* Supplement Left Radial Artery with Nonaut Sub, Perc Approach */
"03UC47Z"  =  "1"  /* Supplement L Radial Art with Autol Sub, Perc Endo Approach */
"03UC4JZ"  =  "1"  /* Supplement L Radial Art with Synth Sub, Perc Endo Approach */
"03UC4KZ"  =  "1"  /* Supplement L Radial Art with Nonaut Sub, Perc Endo Approach */
"03UD07Z"  =  "1"  /* Supplement Right Hand Artery with Autol Sub, Open Approach */
"03UD0JZ"  =  "1"  /* Supplement Right Hand Artery with Synth Sub, Open Approach */
"03UD0KZ"  =  "1"  /* Supplement Right Hand Artery with Nonaut Sub, Open Approach */
"03UD37Z"  =  "1"  /* Supplement Right Hand Artery with Autol Sub, Perc Approach */
"03UD3JZ"  =  "1"  /* Supplement Right Hand Artery with Synth Sub, Perc Approach */
"03UD3KZ"  =  "1"  /* Supplement Right Hand Artery with Nonaut Sub, Perc Approach */
"03UD47Z"  =  "1"  /* Supplement R Hand Art with Autol Sub, Perc Endo Approach */
"03UD4JZ"  =  "1"  /* Supplement R Hand Art with Synth Sub, Perc Endo Approach */
"03UD4KZ"  =  "1"  /* Supplement R Hand Art with Nonaut Sub, Perc Endo Approach */
"03UF07Z"  =  "1"  /* Supplement Left Hand Artery with Autol Sub, Open Approach */
"03UF0JZ"  =  "1"  /* Supplement Left Hand Artery with Synth Sub, Open Approach */
"03UF0KZ"  =  "1"  /* Supplement Left Hand Artery with Nonaut Sub, Open Approach */
"03UF37Z"  =  "1"  /* Supplement Left Hand Artery with Autol Sub, Perc Approach */
"03UF3JZ"  =  "1"  /* Supplement Left Hand Artery with Synth Sub, Perc Approach */
"03UF3KZ"  =  "1"  /* Supplement Left Hand Artery with Nonaut Sub, Perc Approach */
"03UF47Z"  =  "1"  /* Supplement L Hand Art with Autol Sub, Perc Endo Approach */
"03UF4JZ"  =  "1"  /* Supplement L Hand Art with Synth Sub, Perc Endo Approach */
"03UF4KZ"  =  "1"  /* Supplement L Hand Art with Nonaut Sub, Perc Endo Approach */
"03UG07Z"  =  "1"  /* Supplement Intracranial Artery with Autol Sub, Open Approach */
"03UG0JZ"  =  "1"  /* Supplement Intracranial Artery with Synth Sub, Open Approach */
"03UG0KZ"  =  "1"  /* Supplement Intracran Art with Nonaut Sub, Open Approach */
"03UG37Z"  =  "1"  /* Supplement Intracranial Artery with Autol Sub, Perc Approach */
"03UG3JZ"  =  "1"  /* Supplement Intracranial Artery with Synth Sub, Perc Approach */
"03UG3KZ"  =  "1"  /* Supplement Intracran Art with Nonaut Sub, Perc Approach */
"03UG47Z"  =  "1"  /* Supplement Intracran Art with Autol Sub, Perc Endo Approach */
"03UG4JZ"  =  "1"  /* Supplement Intracran Art with Synth Sub, Perc Endo Approach */
"03UG4KZ"  =  "1"  /* Supplement Intracran Art with Nonaut Sub, Perc Endo Approach */
"03UH07Z"  =  "1"  /* Supplement R Com Carotid with Autol Sub, Open Approach */
"03UH0JZ"  =  "1"  /* Supplement R Com Carotid with Synth Sub, Open Approach */
"03UH0KZ"  =  "1"  /* Supplement R Com Carotid with Nonaut Sub, Open Approach */
"03UH37Z"  =  "1"  /* Supplement R Com Carotid with Autol Sub, Perc Approach */
"03UH3JZ"  =  "1"  /* Supplement R Com Carotid with Synth Sub, Perc Approach */
"03UH3KZ"  =  "1"  /* Supplement R Com Carotid with Nonaut Sub, Perc Approach */
"03UH47Z"  =  "1"  /* Supplement R Com Carotid with Autol Sub, Perc Endo Approach */
"03UH4JZ"  =  "1"  /* Supplement R Com Carotid with Synth Sub, Perc Endo Approach */
"03UH4KZ"  =  "1"  /* Supplement R Com Carotid with Nonaut Sub, Perc Endo Approach */
"03UJ07Z"  =  "1"  /* Supplement L Com Carotid with Autol Sub, Open Approach */
"03UJ0JZ"  =  "1"  /* Supplement L Com Carotid with Synth Sub, Open Approach */
"03UJ0KZ"  =  "1"  /* Supplement L Com Carotid with Nonaut Sub, Open Approach */
"03UJ37Z"  =  "1"  /* Supplement L Com Carotid with Autol Sub, Perc Approach */
"03UJ3JZ"  =  "1"  /* Supplement L Com Carotid with Synth Sub, Perc Approach */
"03UJ3KZ"  =  "1"  /* Supplement L Com Carotid with Nonaut Sub, Perc Approach */
"03UJ47Z"  =  "1"  /* Supplement L Com Carotid with Autol Sub, Perc Endo Approach */
"03UJ4JZ"  =  "1"  /* Supplement L Com Carotid with Synth Sub, Perc Endo Approach */
"03UJ4KZ"  =  "1"  /* Supplement L Com Carotid with Nonaut Sub, Perc Endo Approach */
"03UK07Z"  =  "1"  /* Supplement R Int Carotid with Autol Sub, Open Approach */
"03UK0JZ"  =  "1"  /* Supplement R Int Carotid with Synth Sub, Open Approach */
"03UK0KZ"  =  "1"  /* Supplement R Int Carotid with Nonaut Sub, Open Approach */
"03UK37Z"  =  "1"  /* Supplement R Int Carotid with Autol Sub, Perc Approach */
"03UK3JZ"  =  "1"  /* Supplement R Int Carotid with Synth Sub, Perc Approach */
"03UK3KZ"  =  "1"  /* Supplement R Int Carotid with Nonaut Sub, Perc Approach */
"03UK47Z"  =  "1"  /* Supplement R Int Carotid with Autol Sub, Perc Endo Approach */
"03UK4JZ"  =  "1"  /* Supplement R Int Carotid with Synth Sub, Perc Endo Approach */
"03UK4KZ"  =  "1"  /* Supplement R Int Carotid with Nonaut Sub, Perc Endo Approach */
"03UL07Z"  =  "1"  /* Supplement L Int Carotid with Autol Sub, Open Approach */
"03UL0JZ"  =  "1"  /* Supplement L Int Carotid with Synth Sub, Open Approach */
"03UL0KZ"  =  "1"  /* Supplement L Int Carotid with Nonaut Sub, Open Approach */
"03UL37Z"  =  "1"  /* Supplement L Int Carotid with Autol Sub, Perc Approach */
"03UL3JZ"  =  "1"  /* Supplement L Int Carotid with Synth Sub, Perc Approach */
"03UL3KZ"  =  "1"  /* Supplement L Int Carotid with Nonaut Sub, Perc Approach */
"03UL47Z"  =  "1"  /* Supplement L Int Carotid with Autol Sub, Perc Endo Approach */
"03UL4JZ"  =  "1"  /* Supplement L Int Carotid with Synth Sub, Perc Endo Approach */
"03UL4KZ"  =  "1"  /* Supplement L Int Carotid with Nonaut Sub, Perc Endo Approach */
"03UM07Z"  =  "1"  /* Supplement R Ext Carotid with Autol Sub, Open Approach */
"03UM0JZ"  =  "1"  /* Supplement R Ext Carotid with Synth Sub, Open Approach */
"03UM0KZ"  =  "1"  /* Supplement R Ext Carotid with Nonaut Sub, Open Approach */
"03UM37Z"  =  "1"  /* Supplement R Ext Carotid with Autol Sub, Perc Approach */
"03UM3JZ"  =  "1"  /* Supplement R Ext Carotid with Synth Sub, Perc Approach */
"03UM3KZ"  =  "1"  /* Supplement R Ext Carotid with Nonaut Sub, Perc Approach */
"03UM47Z"  =  "1"  /* Supplement R Ext Carotid with Autol Sub, Perc Endo Approach */
"03UM4JZ"  =  "1"  /* Supplement R Ext Carotid with Synth Sub, Perc Endo Approach */
"03UM4KZ"  =  "1"  /* Supplement R Ext Carotid with Nonaut Sub, Perc Endo Approach */
"03UN07Z"  =  "1"  /* Supplement L Ext Carotid with Autol Sub, Open Approach */
"03UN0JZ"  =  "1"  /* Supplement L Ext Carotid with Synth Sub, Open Approach */
"03UN0KZ"  =  "1"  /* Supplement L Ext Carotid with Nonaut Sub, Open Approach */
"03UN37Z"  =  "1"  /* Supplement L Ext Carotid with Autol Sub, Perc Approach */
"03UN3JZ"  =  "1"  /* Supplement L Ext Carotid with Synth Sub, Perc Approach */
"03UN3KZ"  =  "1"  /* Supplement L Ext Carotid with Nonaut Sub, Perc Approach */
"03UN47Z"  =  "1"  /* Supplement L Ext Carotid with Autol Sub, Perc Endo Approach */
"03UN4JZ"  =  "1"  /* Supplement L Ext Carotid with Synth Sub, Perc Endo Approach */
"03UN4KZ"  =  "1"  /* Supplement L Ext Carotid with Nonaut Sub, Perc Endo Approach */
"03UP07Z"  =  "1"  /* Supplement R Verteb Art with Autol Sub, Open Approach */
"03UP0JZ"  =  "1"  /* Supplement R Verteb Art with Synth Sub, Open Approach */
"03UP0KZ"  =  "1"  /* Supplement R Verteb Art with Nonaut Sub, Open Approach */
"03UP37Z"  =  "1"  /* Supplement R Verteb Art with Autol Sub, Perc Approach */
"03UP3JZ"  =  "1"  /* Supplement R Verteb Art with Synth Sub, Perc Approach */
"03UP3KZ"  =  "1"  /* Supplement R Verteb Art with Nonaut Sub, Perc Approach */
"03UP47Z"  =  "1"  /* Supplement R Verteb Art with Autol Sub, Perc Endo Approach */
"03UP4JZ"  =  "1"  /* Supplement R Verteb Art with Synth Sub, Perc Endo Approach */
"03UP4KZ"  =  "1"  /* Supplement R Verteb Art with Nonaut Sub, Perc Endo Approach */
"03UQ07Z"  =  "1"  /* Supplement L Verteb Art with Autol Sub, Open Approach */
"03UQ0JZ"  =  "1"  /* Supplement L Verteb Art with Synth Sub, Open Approach */
"03UQ0KZ"  =  "1"  /* Supplement L Verteb Art with Nonaut Sub, Open Approach */
"03UQ37Z"  =  "1"  /* Supplement L Verteb Art with Autol Sub, Perc Approach */
"03UQ3JZ"  =  "1"  /* Supplement L Verteb Art with Synth Sub, Perc Approach */
"03UQ3KZ"  =  "1"  /* Supplement L Verteb Art with Nonaut Sub, Perc Approach */
"03UQ47Z"  =  "1"  /* Supplement L Verteb Art with Autol Sub, Perc Endo Approach */
"03UQ4JZ"  =  "1"  /* Supplement L Verteb Art with Synth Sub, Perc Endo Approach */
"03UQ4KZ"  =  "1"  /* Supplement L Verteb Art with Nonaut Sub, Perc Endo Approach */
"03UR07Z"  =  "1"  /* Supplement Face Artery with Autol Sub, Open Approach */
"03UR0JZ"  =  "1"  /* Supplement Face Artery with Synth Sub, Open Approach */
"03UR0KZ"  =  "1"  /* Supplement Face Artery with Nonaut Sub, Open Approach */
"03UR37Z"  =  "1"  /* Supplement Face Artery with Autol Sub, Perc Approach */
"03UR3JZ"  =  "1"  /* Supplement Face Artery with Synth Sub, Perc Approach */
"03UR3KZ"  =  "1"  /* Supplement Face Artery with Nonaut Sub, Perc Approach */
"03UR47Z"  =  "1"  /* Supplement Face Artery with Autol Sub, Perc Endo Approach */
"03UR4JZ"  =  "1"  /* Supplement Face Artery with Synth Sub, Perc Endo Approach */
"03UR4KZ"  =  "1"  /* Supplement Face Artery with Nonaut Sub, Perc Endo Approach */
"03US07Z"  =  "1"  /* Supplement R Temporal Art with Autol Sub, Open Approach */
"03US0JZ"  =  "1"  /* Supplement R Temporal Art with Synth Sub, Open Approach */
"03US0KZ"  =  "1"  /* Supplement R Temporal Art with Nonaut Sub, Open Approach */
"03US37Z"  =  "1"  /* Supplement R Temporal Art with Autol Sub, Perc Approach */
"03US3JZ"  =  "1"  /* Supplement R Temporal Art with Synth Sub, Perc Approach */
"03US3KZ"  =  "1"  /* Supplement R Temporal Art with Nonaut Sub, Perc Approach */
"03US47Z"  =  "1"  /* Supplement R Temporal Art with Autol Sub, Perc Endo Approach */
"03US4JZ"  =  "1"  /* Supplement R Temporal Art with Synth Sub, Perc Endo Approach */
"03US4KZ"  =  "1"  /* Supplement R Temporal Art w Nonaut Sub, Perc Endo */
"03UT07Z"  =  "1"  /* Supplement L Temporal Art with Autol Sub, Open Approach */
"03UT0JZ"  =  "1"  /* Supplement L Temporal Art with Synth Sub, Open Approach */
"03UT0KZ"  =  "1"  /* Supplement L Temporal Art with Nonaut Sub, Open Approach */
"03UT37Z"  =  "1"  /* Supplement L Temporal Art with Autol Sub, Perc Approach */
"03UT3JZ"  =  "1"  /* Supplement L Temporal Art with Synth Sub, Perc Approach */
"03UT3KZ"  =  "1"  /* Supplement L Temporal Art with Nonaut Sub, Perc Approach */
"03UT47Z"  =  "1"  /* Supplement L Temporal Art with Autol Sub, Perc Endo Approach */
"03UT4JZ"  =  "1"  /* Supplement L Temporal Art with Synth Sub, Perc Endo Approach */
"03UT4KZ"  =  "1"  /* Supplement L Temporal Art w Nonaut Sub, Perc Endo */
"03UU07Z"  =  "1"  /* Supplement R Thyroid Art with Autol Sub, Open Approach */
"03UU0JZ"  =  "1"  /* Supplement R Thyroid Art with Synth Sub, Open Approach */
"03UU0KZ"  =  "1"  /* Supplement R Thyroid Art with Nonaut Sub, Open Approach */
"03UU37Z"  =  "1"  /* Supplement R Thyroid Art with Autol Sub, Perc Approach */
"03UU3JZ"  =  "1"  /* Supplement R Thyroid Art with Synth Sub, Perc Approach */
"03UU3KZ"  =  "1"  /* Supplement R Thyroid Art with Nonaut Sub, Perc Approach */
"03UU47Z"  =  "1"  /* Supplement R Thyroid Art with Autol Sub, Perc Endo Approach */
"03UU4JZ"  =  "1"  /* Supplement R Thyroid Art with Synth Sub, Perc Endo Approach */
"03UU4KZ"  =  "1"  /* Supplement R Thyroid Art with Nonaut Sub, Perc Endo Approach */
"03UV07Z"  =  "1"  /* Supplement Left Thyroid Artery with Autol Sub, Open Approach */
"03UV0JZ"  =  "1"  /* Supplement Left Thyroid Artery with Synth Sub, Open Approach */
"03UV0KZ"  =  "1"  /* Supplement L Thyroid Art with Nonaut Sub, Open Approach */
"03UV37Z"  =  "1"  /* Supplement Left Thyroid Artery with Autol Sub, Perc Approach */
"03UV3JZ"  =  "1"  /* Supplement Left Thyroid Artery with Synth Sub, Perc Approach */
"03UV3KZ"  =  "1"  /* Supplement L Thyroid Art with Nonaut Sub, Perc Approach */
"03UV47Z"  =  "1"  /* Supplement L Thyroid Art with Autol Sub, Perc Endo Approach */
"03UV4JZ"  =  "1"  /* Supplement L Thyroid Art with Synth Sub, Perc Endo Approach */
"03UV4KZ"  =  "1"  /* Supplement L Thyroid Art with Nonaut Sub, Perc Endo Approach */
"03UY07Z"  =  "1"  /* Supplement Upper Artery with Autol Sub, Open Approach */
"03UY0JZ"  =  "1"  /* Supplement Upper Artery with Synth Sub, Open Approach */
"03UY0KZ"  =  "1"  /* Supplement Upper Artery with Nonaut Sub, Open Approach */
"03UY37Z"  =  "1"  /* Supplement Upper Artery with Autol Sub, Perc Approach */
"03UY3JZ"  =  "1"  /* Supplement Upper Artery with Synth Sub, Perc Approach */
"03UY3KZ"  =  "1"  /* Supplement Upper Artery with Nonaut Sub, Perc Approach */
"03UY47Z"  =  "1"  /* Supplement Upper Artery with Autol Sub, Perc Endo Approach */
"03UY4JZ"  =  "1"  /* Supplement Upper Artery with Synth Sub, Perc Endo Approach */
"03UY4KZ"  =  "1"  /* Supplement Upper Artery with Nonaut Sub, Perc Endo Approach */
"03V00CZ"  =  "1"  /* Restrict of R Int Mamm Art with Extralum Dev, Open Approach */
"03V00DZ"  =  "1"  /* Restrict of R Int Mamm Art with Intralum Dev, Open Approach */
"03V00ZZ"  =  "1"  /* Restriction of Right Internal Mammary Artery, Open Approach */
"03V03CZ"  =  "1"  /* Restrict of R Int Mamm Art with Extralum Dev, Perc Approach */
"03V03DZ"  =  "1"  /* Restrict of R Int Mamm Art with Intralum Dev, Perc Approach */
"03V03ZZ"  =  "1"  /* Restriction of Right Internal Mammary Artery, Perc Approach */
"03V04CZ"  =  "1"  /* Restrict R Int Mamm Art w Extralum Dev, Perc Endo */
"03V04DZ"  =  "1"  /* Restrict R Int Mamm Art w Intralum Dev, Perc Endo */
"03V04ZZ"  =  "1"  /* Restriction of R Int Mamm Art, Perc Endo Approach */
"03V10CZ"  =  "1"  /* Restrict of L Int Mamm Art with Extralum Dev, Open Approach */
"03V10DZ"  =  "1"  /* Restrict of L Int Mamm Art with Intralum Dev, Open Approach */
"03V10ZZ"  =  "1"  /* Restriction of Left Internal Mammary Artery, Open Approach */
"03V13CZ"  =  "1"  /* Restrict of L Int Mamm Art with Extralum Dev, Perc Approach */
"03V13DZ"  =  "1"  /* Restrict of L Int Mamm Art with Intralum Dev, Perc Approach */
"03V13ZZ"  =  "1"  /* Restriction of Left Internal Mammary Artery, Perc Approach */
"03V14CZ"  =  "1"  /* Restrict L Int Mamm Art w Extralum Dev, Perc Endo */
"03V14DZ"  =  "1"  /* Restrict L Int Mamm Art w Intralum Dev, Perc Endo */
"03V14ZZ"  =  "1"  /* Restriction of L Int Mamm Art, Perc Endo Approach */
"03V20CZ"  =  "1"  /* Restriction of Innom Art with Extralum Dev, Open Approach */
"03V20DZ"  =  "1"  /* Restriction of Innom Art with Intralum Dev, Open Approach */
"03V20ZZ"  =  "1"  /* Restriction of Innominate Artery, Open Approach */
"03V23CZ"  =  "1"  /* Restriction of Innom Art with Extralum Dev, Perc Approach */
"03V23DZ"  =  "1"  /* Restriction of Innom Art with Intralum Dev, Perc Approach */
"03V23ZZ"  =  "1"  /* Restriction of Innominate Artery, Percutaneous Approach */
"03V24CZ"  =  "1"  /* Restrict of Innom Art with Extralum Dev, Perc Endo Approach */
"03V24DZ"  =  "1"  /* Restrict of Innom Art with Intralum Dev, Perc Endo Approach */
"03V24ZZ"  =  "1"  /* Restriction of Innominate Artery, Perc Endo Approach */
"03V30CZ"  =  "1"  /* Restrict of R Subclav Art with Extralum Dev, Open Approach */
"03V30DZ"  =  "1"  /* Restrict of R Subclav Art with Intralum Dev, Open Approach */
"03V30ZZ"  =  "1"  /* Restriction of Right Subclavian Artery, Open Approach */
"03V33CZ"  =  "1"  /* Restrict of R Subclav Art with Extralum Dev, Perc Approach */
"03V33DZ"  =  "1"  /* Restrict of R Subclav Art with Intralum Dev, Perc Approach */
"03V33ZZ"  =  "1"  /* Restriction of Right Subclavian Artery, Perc Approach */
"03V34CZ"  =  "1"  /* Restrict R Subclav Art w Extralum Dev, Perc Endo */
"03V34DZ"  =  "1"  /* Restrict R Subclav Art w Intralum Dev, Perc Endo */
"03V34ZZ"  =  "1"  /* Restriction of Right Subclavian Artery, Perc Endo Approach */
"03V40CZ"  =  "1"  /* Restrict of L Subclav Art with Extralum Dev, Open Approach */
"03V40DZ"  =  "1"  /* Restrict of L Subclav Art with Intralum Dev, Open Approach */
"03V40ZZ"  =  "1"  /* Restriction of Left Subclavian Artery, Open Approach */
"03V43CZ"  =  "1"  /* Restrict of L Subclav Art with Extralum Dev, Perc Approach */
"03V43DZ"  =  "1"  /* Restrict of L Subclav Art with Intralum Dev, Perc Approach */
"03V43ZZ"  =  "1"  /* Restriction of Left Subclavian Artery, Percutaneous Approach */
"03V44CZ"  =  "1"  /* Restrict L Subclav Art w Extralum Dev, Perc Endo */
"03V44DZ"  =  "1"  /* Restrict L Subclav Art w Intralum Dev, Perc Endo */
"03V44ZZ"  =  "1"  /* Restriction of Left Subclavian Artery, Perc Endo Approach */
"03V50CZ"  =  "1"  /* Restriction of R Axilla Art with Extralum Dev, Open Approach */
"03V50DZ"  =  "1"  /* Restriction of R Axilla Art with Intralum Dev, Open Approach */
"03V50ZZ"  =  "1"  /* Restriction of Right Axillary Artery, Open Approach */
"03V53CZ"  =  "1"  /* Restriction of R Axilla Art with Extralum Dev, Perc Approach */
"03V53DZ"  =  "1"  /* Restriction of R Axilla Art with Intralum Dev, Perc Approach */
"03V53ZZ"  =  "1"  /* Restriction of Right Axillary Artery, Percutaneous Approach */
"03V54CZ"  =  "1"  /* Restrict R Axilla Art w Extralum Dev, Perc Endo */
"03V54DZ"  =  "1"  /* Restrict R Axilla Art w Intralum Dev, Perc Endo */
"03V54ZZ"  =  "1"  /* Restriction of Right Axillary Artery, Perc Endo Approach */
"03V60CZ"  =  "1"  /* Restriction of L Axilla Art with Extralum Dev, Open Approach */
"03V60DZ"  =  "1"  /* Restriction of L Axilla Art with Intralum Dev, Open Approach */
"03V60ZZ"  =  "1"  /* Restriction of Left Axillary Artery, Open Approach */
"03V63CZ"  =  "1"  /* Restriction of L Axilla Art with Extralum Dev, Perc Approach */
"03V63DZ"  =  "1"  /* Restriction of L Axilla Art with Intralum Dev, Perc Approach */
"03V63ZZ"  =  "1"  /* Restriction of Left Axillary Artery, Percutaneous Approach */
"03V64CZ"  =  "1"  /* Restrict L Axilla Art w Extralum Dev, Perc Endo */
"03V64DZ"  =  "1"  /* Restrict L Axilla Art w Intralum Dev, Perc Endo */
"03V64ZZ"  =  "1"  /* Restriction of Left Axillary Artery, Perc Endo Approach */
"03V70CZ"  =  "1"  /* Restriction of R Brach Art with Extralum Dev, Open Approach */
"03V70DZ"  =  "1"  /* Restriction of R Brach Art with Intralum Dev, Open Approach */
"03V70ZZ"  =  "1"  /* Restriction of Right Brachial Artery, Open Approach */
"03V73CZ"  =  "1"  /* Restriction of R Brach Art with Extralum Dev, Perc Approach */
"03V73DZ"  =  "1"  /* Restriction of R Brach Art with Intralum Dev, Perc Approach */
"03V73ZZ"  =  "1"  /* Restriction of Right Brachial Artery, Percutaneous Approach */
"03V74CZ"  =  "1"  /* Restrict R Brach Art w Extralum Dev, Perc Endo */
"03V74DZ"  =  "1"  /* Restrict R Brach Art w Intralum Dev, Perc Endo */
"03V74ZZ"  =  "1"  /* Restriction of Right Brachial Artery, Perc Endo Approach */
"03V80CZ"  =  "1"  /* Restriction of L Brach Art with Extralum Dev, Open Approach */
"03V80DZ"  =  "1"  /* Restriction of L Brach Art with Intralum Dev, Open Approach */
"03V80ZZ"  =  "1"  /* Restriction of Left Brachial Artery, Open Approach */
"03V83CZ"  =  "1"  /* Restriction of L Brach Art with Extralum Dev, Perc Approach */
"03V83DZ"  =  "1"  /* Restriction of L Brach Art with Intralum Dev, Perc Approach */
"03V83ZZ"  =  "1"  /* Restriction of Left Brachial Artery, Percutaneous Approach */
"03V84CZ"  =  "1"  /* Restrict L Brach Art w Extralum Dev, Perc Endo */
"03V84DZ"  =  "1"  /* Restrict L Brach Art w Intralum Dev, Perc Endo */
"03V84ZZ"  =  "1"  /* Restriction of Left Brachial Artery, Perc Endo Approach */
"03V90CZ"  =  "1"  /* Restriction of R Ulnar Art with Extralum Dev, Open Approach */
"03V90DZ"  =  "1"  /* Restriction of R Ulnar Art with Intralum Dev, Open Approach */
"03V90ZZ"  =  "1"  /* Restriction of Right Ulnar Artery, Open Approach */
"03V93CZ"  =  "1"  /* Restriction of R Ulnar Art with Extralum Dev, Perc Approach */
"03V93DZ"  =  "1"  /* Restriction of R Ulnar Art with Intralum Dev, Perc Approach */
"03V93ZZ"  =  "1"  /* Restriction of Right Ulnar Artery, Percutaneous Approach */
"03V94CZ"  =  "1"  /* Restrict R Ulnar Art w Extralum Dev, Perc Endo */
"03V94DZ"  =  "1"  /* Restrict R Ulnar Art w Intralum Dev, Perc Endo */
"03V94ZZ"  =  "1"  /* Restriction of Right Ulnar Artery, Perc Endo Approach */
"03VA0CZ"  =  "1"  /* Restriction of L Ulnar Art with Extralum Dev, Open Approach */
"03VA0DZ"  =  "1"  /* Restriction of L Ulnar Art with Intralum Dev, Open Approach */
"03VA0ZZ"  =  "1"  /* Restriction of Left Ulnar Artery, Open Approach */
"03VA3CZ"  =  "1"  /* Restriction of L Ulnar Art with Extralum Dev, Perc Approach */
"03VA3DZ"  =  "1"  /* Restriction of L Ulnar Art with Intralum Dev, Perc Approach */
"03VA3ZZ"  =  "1"  /* Restriction of Left Ulnar Artery, Percutaneous Approach */
"03VA4CZ"  =  "1"  /* Restrict L Ulnar Art w Extralum Dev, Perc Endo */
"03VA4DZ"  =  "1"  /* Restrict L Ulnar Art w Intralum Dev, Perc Endo */
"03VA4ZZ"  =  "1"  /* Restriction of Left Ulnar Artery, Perc Endo Approach */
"03VB0CZ"  =  "1"  /* Restriction of R Radial Art with Extralum Dev, Open Approach */
"03VB0DZ"  =  "1"  /* Restriction of R Radial Art with Intralum Dev, Open Approach */
"03VB0ZZ"  =  "1"  /* Restriction of Right Radial Artery, Open Approach */
"03VB3CZ"  =  "1"  /* Restriction of R Radial Art with Extralum Dev, Perc Approach */
"03VB3DZ"  =  "1"  /* Restriction of R Radial Art with Intralum Dev, Perc Approach */
"03VB3ZZ"  =  "1"  /* Restriction of Right Radial Artery, Percutaneous Approach */
"03VB4CZ"  =  "1"  /* Restrict R Radial Art w Extralum Dev, Perc Endo */
"03VB4DZ"  =  "1"  /* Restrict R Radial Art w Intralum Dev, Perc Endo */
"03VB4ZZ"  =  "1"  /* Restriction of Right Radial Artery, Perc Endo Approach */
"03VC0CZ"  =  "1"  /* Restriction of L Radial Art with Extralum Dev, Open Approach */
"03VC0DZ"  =  "1"  /* Restriction of L Radial Art with Intralum Dev, Open Approach */
"03VC0ZZ"  =  "1"  /* Restriction of Left Radial Artery, Open Approach */
"03VC3CZ"  =  "1"  /* Restriction of L Radial Art with Extralum Dev, Perc Approach */
"03VC3DZ"  =  "1"  /* Restriction of L Radial Art with Intralum Dev, Perc Approach */
"03VC3ZZ"  =  "1"  /* Restriction of Left Radial Artery, Percutaneous Approach */
"03VC4CZ"  =  "1"  /* Restrict L Radial Art w Extralum Dev, Perc Endo */
"03VC4DZ"  =  "1"  /* Restrict L Radial Art w Intralum Dev, Perc Endo */
"03VC4ZZ"  =  "1"  /* Restriction of Left Radial Artery, Perc Endo Approach */
"03VD0CZ"  =  "1"  /* Restriction of R Hand Art with Extralum Dev, Open Approach */
"03VD0DZ"  =  "1"  /* Restriction of R Hand Art with Intralum Dev, Open Approach */
"03VD0ZZ"  =  "1"  /* Restriction of Right Hand Artery, Open Approach */
"03VD3CZ"  =  "1"  /* Restriction of R Hand Art with Extralum Dev, Perc Approach */
"03VD3DZ"  =  "1"  /* Restriction of R Hand Art with Intralum Dev, Perc Approach */
"03VD3ZZ"  =  "1"  /* Restriction of Right Hand Artery, Percutaneous Approach */
"03VD4CZ"  =  "1"  /* Restrict of R Hand Art with Extralum Dev, Perc Endo Approach */
"03VD4DZ"  =  "1"  /* Restrict of R Hand Art with Intralum Dev, Perc Endo Approach */
"03VD4ZZ"  =  "1"  /* Restriction of Right Hand Artery, Perc Endo Approach */
"03VF0CZ"  =  "1"  /* Restriction of L Hand Art with Extralum Dev, Open Approach */
"03VF0DZ"  =  "1"  /* Restriction of L Hand Art with Intralum Dev, Open Approach */
"03VF0ZZ"  =  "1"  /* Restriction of Left Hand Artery, Open Approach */
"03VF3CZ"  =  "1"  /* Restriction of L Hand Art with Extralum Dev, Perc Approach */
"03VF3DZ"  =  "1"  /* Restriction of L Hand Art with Intralum Dev, Perc Approach */
"03VF3ZZ"  =  "1"  /* Restriction of Left Hand Artery, Percutaneous Approach */
"03VF4CZ"  =  "1"  /* Restrict of L Hand Art with Extralum Dev, Perc Endo Approach */
"03VF4DZ"  =  "1"  /* Restrict of L Hand Art with Intralum Dev, Perc Endo Approach */
"03VF4ZZ"  =  "1"  /* Restriction of Left Hand Artery, Perc Endo Approach */
"03VG0BZ"  =  "1"  /* Restrict Intracran Art w Bioact Intralum, Open */
"03VG0CZ"  =  "1"  /* Restrict of Intracran Art with Extralum Dev, Open Approach */
"03VG0DZ"  =  "1"  /* Restrict of Intracran Art with Intralum Dev, Open Approach */
"03VG0ZZ"  =  "1"  /* Restriction of Intracranial Artery, Open Approach */
"03VG3BZ"  =  "1"  /* Restrict Intracran Art w Bioact Intralum, Perc */
"03VG3CZ"  =  "1"  /* Restrict of Intracran Art with Extralum Dev, Perc Approach */
"03VG3DZ"  =  "1"  /* Restrict of Intracran Art with Intralum Dev, Perc Approach */
"03VG3ZZ"  =  "1"  /* Restriction of Intracranial Artery, Percutaneous Approach */
"03VG4BZ"  =  "1"  /* Restrict Intracran Art w Bioact Intralum, Perc Endo */
"03VG4CZ"  =  "1"  /* Restrict Intracran Art w Extralum Dev, Perc Endo */
"03VG4DZ"  =  "1"  /* Restrict Intracran Art w Intralum Dev, Perc Endo */
"03VG4ZZ"  =  "1"  /* Restriction of Intracranial Artery, Perc Endo Approach */
"03VH0BZ"  =  "1"  /* Restrict R Com Carotid w Bioact Intralum, Open */
"03VH0CZ"  =  "1"  /* Restrict of R Com Carotid with Extralum Dev, Open Approach */
"03VH0DZ"  =  "1"  /* Restrict of R Com Carotid with Intralum Dev, Open Approach */
"03VH0ZZ"  =  "1"  /* Restriction of Right Common Carotid Artery, Open Approach */
"03VH3BZ"  =  "1"  /* Restrict R Com Carotid w Bioact Intralum, Perc */
"03VH3CZ"  =  "1"  /* Restrict of R Com Carotid with Extralum Dev, Perc Approach */
"03VH3DZ"  =  "1"  /* Restrict of R Com Carotid with Intralum Dev, Perc Approach */
"03VH3ZZ"  =  "1"  /* Restriction of Right Common Carotid Artery, Perc Approach */
"03VH4BZ"  =  "1"  /* Restrict R Com Carotid w Bioact Intralum, Perc Endo */
"03VH4CZ"  =  "1"  /* Restrict R Com Carotid w Extralum Dev, Perc Endo */
"03VH4DZ"  =  "1"  /* Restrict R Com Carotid w Intralum Dev, Perc Endo */
"03VH4ZZ"  =  "1"  /* Restriction of R Com Carotid, Perc Endo Approach */
"03VJ0BZ"  =  "1"  /* Restrict L Com Carotid w Bioact Intralum, Open */
"03VJ0CZ"  =  "1"  /* Restrict of L Com Carotid with Extralum Dev, Open Approach */
"03VJ0DZ"  =  "1"  /* Restrict of L Com Carotid with Intralum Dev, Open Approach */
"03VJ0ZZ"  =  "1"  /* Restriction of Left Common Carotid Artery, Open Approach */
"03VJ3BZ"  =  "1"  /* Restrict L Com Carotid w Bioact Intralum, Perc */
"03VJ3CZ"  =  "1"  /* Restrict of L Com Carotid with Extralum Dev, Perc Approach */
"03VJ3DZ"  =  "1"  /* Restrict of L Com Carotid with Intralum Dev, Perc Approach */
"03VJ3ZZ"  =  "1"  /* Restriction of Left Common Carotid Artery, Perc Approach */
"03VJ4BZ"  =  "1"  /* Restrict L Com Carotid w Bioact Intralum, Perc Endo */
"03VJ4CZ"  =  "1"  /* Restrict L Com Carotid w Extralum Dev, Perc Endo */
"03VJ4DZ"  =  "1"  /* Restrict L Com Carotid w Intralum Dev, Perc Endo */
"03VJ4ZZ"  =  "1"  /* Restriction of L Com Carotid, Perc Endo Approach */
"03VK0BZ"  =  "1"  /* Restrict R Int Carotid w Bioact Intralum, Open */
"03VK0CZ"  =  "1"  /* Restrict of R Int Carotid with Extralum Dev, Open Approach */
"03VK0DZ"  =  "1"  /* Restrict of R Int Carotid with Intralum Dev, Open Approach */
"03VK0ZZ"  =  "1"  /* Restriction of Right Internal Carotid Artery, Open Approach */
"03VK3BZ"  =  "1"  /* Restrict R Int Carotid w Bioact Intralum, Perc */
"03VK3CZ"  =  "1"  /* Restrict of R Int Carotid with Extralum Dev, Perc Approach */
"03VK3DZ"  =  "1"  /* Restrict of R Int Carotid with Intralum Dev, Perc Approach */
"03VK3ZZ"  =  "1"  /* Restriction of Right Internal Carotid Artery, Perc Approach */
"03VK4BZ"  =  "1"  /* Restrict R Int Carotid w Bioact Intralum, Perc Endo */
"03VK4CZ"  =  "1"  /* Restrict R Int Carotid w Extralum Dev, Perc Endo */
"03VK4DZ"  =  "1"  /* Restrict R Int Carotid w Intralum Dev, Perc Endo */
"03VK4ZZ"  =  "1"  /* Restriction of R Int Carotid, Perc Endo Approach */
"03VL0BZ"  =  "1"  /* Restrict L Int Carotid w Bioact Intralum, Open */
"03VL0CZ"  =  "1"  /* Restrict of L Int Carotid with Extralum Dev, Open Approach */
"03VL0DZ"  =  "1"  /* Restrict of L Int Carotid with Intralum Dev, Open Approach */
"03VL0ZZ"  =  "1"  /* Restriction of Left Internal Carotid Artery, Open Approach */
"03VL3BZ"  =  "1"  /* Restrict L Int Carotid w Bioact Intralum, Perc */
"03VL3CZ"  =  "1"  /* Restrict of L Int Carotid with Extralum Dev, Perc Approach */
"03VL3DZ"  =  "1"  /* Restrict of L Int Carotid with Intralum Dev, Perc Approach */
"03VL3ZZ"  =  "1"  /* Restriction of Left Internal Carotid Artery, Perc Approach */
"03VL4BZ"  =  "1"  /* Restrict L Int Carotid w Bioact Intralum, Perc Endo */
"03VL4CZ"  =  "1"  /* Restrict L Int Carotid w Extralum Dev, Perc Endo */
"03VL4DZ"  =  "1"  /* Restrict L Int Carotid w Intralum Dev, Perc Endo */
"03VL4ZZ"  =  "1"  /* Restriction of L Int Carotid, Perc Endo Approach */
"03VM0BZ"  =  "1"  /* Restrict R Ext Carotid w Bioact Intralum, Open */
"03VM0CZ"  =  "1"  /* Restrict of R Ext Carotid with Extralum Dev, Open Approach */
"03VM0DZ"  =  "1"  /* Restrict of R Ext Carotid with Intralum Dev, Open Approach */
"03VM0ZZ"  =  "1"  /* Restriction of Right External Carotid Artery, Open Approach */
"03VM3BZ"  =  "1"  /* Restrict R Ext Carotid w Bioact Intralum, Perc */
"03VM3CZ"  =  "1"  /* Restrict of R Ext Carotid with Extralum Dev, Perc Approach */
"03VM3DZ"  =  "1"  /* Restrict of R Ext Carotid with Intralum Dev, Perc Approach */
"03VM3ZZ"  =  "1"  /* Restriction of Right External Carotid Artery, Perc Approach */
"03VM4BZ"  =  "1"  /* Restrict R Ext Carotid w Bioact Intralum, Perc Endo */
"03VM4CZ"  =  "1"  /* Restrict R Ext Carotid w Extralum Dev, Perc Endo */
"03VM4DZ"  =  "1"  /* Restrict R Ext Carotid w Intralum Dev, Perc Endo */
"03VM4ZZ"  =  "1"  /* Restriction of R Ext Carotid, Perc Endo Approach */
"03VN0BZ"  =  "1"  /* Restrict L Ext Carotid w Bioact Intralum, Open */
"03VN0CZ"  =  "1"  /* Restrict of L Ext Carotid with Extralum Dev, Open Approach */
"03VN0DZ"  =  "1"  /* Restrict of L Ext Carotid with Intralum Dev, Open Approach */
"03VN0ZZ"  =  "1"  /* Restriction of Left External Carotid Artery, Open Approach */
"03VN3BZ"  =  "1"  /* Restrict L Ext Carotid w Bioact Intralum, Perc */
"03VN3CZ"  =  "1"  /* Restrict of L Ext Carotid with Extralum Dev, Perc Approach */
"03VN3DZ"  =  "1"  /* Restrict of L Ext Carotid with Intralum Dev, Perc Approach */
"03VN3ZZ"  =  "1"  /* Restriction of Left External Carotid Artery, Perc Approach */
"03VN4BZ"  =  "1"  /* Restrict L Ext Carotid w Bioact Intralum, Perc Endo */
"03VN4CZ"  =  "1"  /* Restrict L Ext Carotid w Extralum Dev, Perc Endo */
"03VN4DZ"  =  "1"  /* Restrict L Ext Carotid w Intralum Dev, Perc Endo */
"03VN4ZZ"  =  "1"  /* Restriction of L Ext Carotid, Perc Endo Approach */
"03VP0BZ"  =  "1"  /* Restrict of R Verteb Art with Bioact Intralum, Open Approach */
"03VP0CZ"  =  "1"  /* Restriction of R Verteb Art with Extralum Dev, Open Approach */
"03VP0DZ"  =  "1"  /* Restriction of R Verteb Art with Intralum Dev, Open Approach */
"03VP0ZZ"  =  "1"  /* Restriction of Right Vertebral Artery, Open Approach */
"03VP3BZ"  =  "1"  /* Restrict of R Verteb Art with Bioact Intralum, Perc Approach */
"03VP3CZ"  =  "1"  /* Restriction of R Verteb Art with Extralum Dev, Perc Approach */
"03VP3DZ"  =  "1"  /* Restriction of R Verteb Art with Intralum Dev, Perc Approach */
"03VP3ZZ"  =  "1"  /* Restriction of Right Vertebral Artery, Percutaneous Approach */
"03VP4BZ"  =  "1"  /* Restrict R Verteb Art w Bioact Intralum, Perc Endo */
"03VP4CZ"  =  "1"  /* Restrict R Verteb Art w Extralum Dev, Perc Endo */
"03VP4DZ"  =  "1"  /* Restrict R Verteb Art w Intralum Dev, Perc Endo */
"03VP4ZZ"  =  "1"  /* Restriction of Right Vertebral Artery, Perc Endo Approach */
"03VQ0BZ"  =  "1"  /* Restrict of L Verteb Art with Bioact Intralum, Open Approach */
"03VQ0CZ"  =  "1"  /* Restriction of L Verteb Art with Extralum Dev, Open Approach */
"03VQ0DZ"  =  "1"  /* Restriction of L Verteb Art with Intralum Dev, Open Approach */
"03VQ0ZZ"  =  "1"  /* Restriction of Left Vertebral Artery, Open Approach */
"03VQ3BZ"  =  "1"  /* Restrict of L Verteb Art with Bioact Intralum, Perc Approach */
"03VQ3CZ"  =  "1"  /* Restriction of L Verteb Art with Extralum Dev, Perc Approach */
"03VQ3DZ"  =  "1"  /* Restriction of L Verteb Art with Intralum Dev, Perc Approach */
"03VQ3ZZ"  =  "1"  /* Restriction of Left Vertebral Artery, Percutaneous Approach */
"03VQ4BZ"  =  "1"  /* Restrict L Verteb Art w Bioact Intralum, Perc Endo */
"03VQ4CZ"  =  "1"  /* Restrict L Verteb Art w Extralum Dev, Perc Endo */
"03VQ4DZ"  =  "1"  /* Restrict L Verteb Art w Intralum Dev, Perc Endo */
"03VQ4ZZ"  =  "1"  /* Restriction of Left Vertebral Artery, Perc Endo Approach */
"03VR0CZ"  =  "1"  /* Restriction of Face Artery with Extralum Dev, Open Approach */
"03VR0DZ"  =  "1"  /* Restriction of Face Artery with Intralum Dev, Open Approach */
"03VR0ZZ"  =  "1"  /* Restriction of Face Artery, Open Approach */
"03VR3CZ"  =  "1"  /* Restriction of Face Artery with Extralum Dev, Perc Approach */
"03VR3DZ"  =  "1"  /* Restriction of Face Artery with Intralum Dev, Perc Approach */
"03VR3ZZ"  =  "1"  /* Restriction of Face Artery, Percutaneous Approach */
"03VR4CZ"  =  "1"  /* Restrict of Face Art with Extralum Dev, Perc Endo Approach */
"03VR4DZ"  =  "1"  /* Restrict of Face Art with Intralum Dev, Perc Endo Approach */
"03VR4ZZ"  =  "1"  /* Restriction of Face Artery, Percutaneous Endoscopic Approach */
"03VS0CZ"  =  "1"  /* Restrict of R Temporal Art with Extralum Dev, Open Approach */
"03VS0DZ"  =  "1"  /* Restrict of R Temporal Art with Intralum Dev, Open Approach */
"03VS0ZZ"  =  "1"  /* Restriction of Right Temporal Artery, Open Approach */
"03VS3CZ"  =  "1"  /* Restrict of R Temporal Art with Extralum Dev, Perc Approach */
"03VS3DZ"  =  "1"  /* Restrict of R Temporal Art with Intralum Dev, Perc Approach */
"03VS3ZZ"  =  "1"  /* Restriction of Right Temporal Artery, Percutaneous Approach */
"03VS4CZ"  =  "1"  /* Restrict R Temporal Art w Extralum Dev, Perc Endo */
"03VS4DZ"  =  "1"  /* Restrict R Temporal Art w Intralum Dev, Perc Endo */
"03VS4ZZ"  =  "1"  /* Restriction of Right Temporal Artery, Perc Endo Approach */
"03VT0CZ"  =  "1"  /* Restrict of L Temporal Art with Extralum Dev, Open Approach */
"03VT0DZ"  =  "1"  /* Restrict of L Temporal Art with Intralum Dev, Open Approach */
"03VT0ZZ"  =  "1"  /* Restriction of Left Temporal Artery, Open Approach */
"03VT3CZ"  =  "1"  /* Restrict of L Temporal Art with Extralum Dev, Perc Approach */
"03VT3DZ"  =  "1"  /* Restrict of L Temporal Art with Intralum Dev, Perc Approach */
"03VT3ZZ"  =  "1"  /* Restriction of Left Temporal Artery, Percutaneous Approach */
"03VT4CZ"  =  "1"  /* Restrict L Temporal Art w Extralum Dev, Perc Endo */
"03VT4DZ"  =  "1"  /* Restrict L Temporal Art w Intralum Dev, Perc Endo */
"03VT4ZZ"  =  "1"  /* Restriction of Left Temporal Artery, Perc Endo Approach */
"03VU0CZ"  =  "1"  /* Restrict of R Thyroid Art with Extralum Dev, Open Approach */
"03VU0DZ"  =  "1"  /* Restrict of R Thyroid Art with Intralum Dev, Open Approach */
"03VU0ZZ"  =  "1"  /* Restriction of Right Thyroid Artery, Open Approach */
"03VU3CZ"  =  "1"  /* Restrict of R Thyroid Art with Extralum Dev, Perc Approach */
"03VU3DZ"  =  "1"  /* Restrict of R Thyroid Art with Intralum Dev, Perc Approach */
"03VU3ZZ"  =  "1"  /* Restriction of Right Thyroid Artery, Percutaneous Approach */
"03VU4CZ"  =  "1"  /* Restrict R Thyroid Art w Extralum Dev, Perc Endo */
"03VU4DZ"  =  "1"  /* Restrict R Thyroid Art w Intralum Dev, Perc Endo */
"03VU4ZZ"  =  "1"  /* Restriction of Right Thyroid Artery, Perc Endo Approach */
"03VV0CZ"  =  "1"  /* Restrict of L Thyroid Art with Extralum Dev, Open Approach */
"03VV0DZ"  =  "1"  /* Restrict of L Thyroid Art with Intralum Dev, Open Approach */
"03VV0ZZ"  =  "1"  /* Restriction of Left Thyroid Artery, Open Approach */
"03VV3CZ"  =  "1"  /* Restrict of L Thyroid Art with Extralum Dev, Perc Approach */
"03VV3DZ"  =  "1"  /* Restrict of L Thyroid Art with Intralum Dev, Perc Approach */
"03VV3ZZ"  =  "1"  /* Restriction of Left Thyroid Artery, Percutaneous Approach */
"03VV4CZ"  =  "1"  /* Restrict L Thyroid Art w Extralum Dev, Perc Endo */
"03VV4DZ"  =  "1"  /* Restrict L Thyroid Art w Intralum Dev, Perc Endo */
"03VV4ZZ"  =  "1"  /* Restriction of Left Thyroid Artery, Perc Endo Approach */
"03VY0CZ"  =  "1"  /* Restriction of Upper Artery with Extralum Dev, Open Approach */
"03VY0DZ"  =  "1"  /* Restriction of Upper Artery with Intralum Dev, Open Approach */
"03VY0ZZ"  =  "1"  /* Restriction of Upper Artery, Open Approach */
"03VY3CZ"  =  "1"  /* Restriction of Upper Artery with Extralum Dev, Perc Approach */
"03VY3DZ"  =  "1"  /* Restriction of Upper Artery with Intralum Dev, Perc Approach */
"03VY3ZZ"  =  "1"  /* Restriction of Upper Artery, Percutaneous Approach */
"03VY4CZ"  =  "1"  /* Restriction of Up Art with Extralum Dev, Perc Endo Approach */
"03VY4DZ"  =  "1"  /* Restriction of Up Art with Intralum Dev, Perc Endo Approach */
"03VY4ZZ"  =  "1"  /* Restriction of Upper Artery, Perc Endo Approach */
"03WY00Z"  =  "1"  /* Revision of Drainage Device in Upper Artery, Open Approach */
"03WY02Z"  =  "1"  /* Revision of Monitoring Device in Upper Artery, Open Approach */
"03WY03Z"  =  "1"  /* Revision of Infusion Device in Upper Artery, Open Approach */
"03WY07Z"  =  "1"  /* Revision of Autol Sub in Up Art, Open Approach */
"03WY0CZ"  =  "1"  /* Revision of Extraluminal Device in Up Art, Open Approach */
"03WY0DZ"  =  "1"  /* Revision of Intraluminal Device in Up Art, Open Approach */
"03WY0JZ"  =  "1"  /* Revision of Synthetic Substitute in Up Art, Open Approach */
"03WY0KZ"  =  "1"  /* Revision of Nonaut Sub in Up Art, Open Approach */
"03WY0MZ"  =  "1"  /* Revision of Stimulator Lead in Upper Artery, Open Approach */
"03WY30Z"  =  "1"  /* Revision of Drainage Device in Upper Artery, Perc Approach */
"03WY32Z"  =  "1"  /* Revision of Monitoring Device in Upper Artery, Perc Approach */
"03WY33Z"  =  "1"  /* Revision of Infusion Device in Upper Artery, Perc Approach */
"03WY37Z"  =  "1"  /* Revision of Autol Sub in Up Art, Perc Approach */
"03WY3CZ"  =  "1"  /* Revision of Extraluminal Device in Up Art, Perc Approach */
"03WY3DZ"  =  "1"  /* Revision of Intraluminal Device in Up Art, Perc Approach */
"03WY3JZ"  =  "1"  /* Revision of Synthetic Substitute in Up Art, Perc Approach */
"03WY3KZ"  =  "1"  /* Revision of Nonaut Sub in Up Art, Perc Approach */
"03WY3MZ"  =  "1"  /* Revision of Stimulator Lead in Upper Artery, Perc Approach */
"03WY40Z"  =  "1"  /* Revision of Drainage Device in Up Art, Perc Endo Approach */
"03WY42Z"  =  "1"  /* Revision of Monitoring Device in Up Art, Perc Endo Approach */
"03WY43Z"  =  "1"  /* Revision of Infusion Device in Up Art, Perc Endo Approach */
"03WY47Z"  =  "1"  /* Revision of Autol Sub in Up Art, Perc Endo Approach */
"03WY4CZ"  =  "1"  /* Revision of Extralum Dev in Up Art, Perc Endo Approach */
"03WY4DZ"  =  "1"  /* Revision of Intralum Dev in Up Art, Perc Endo Approach */
"03WY4JZ"  =  "1"  /* Revision of Synth Sub in Up Art, Perc Endo Approach */
"03WY4KZ"  =  "1"  /* Revision of Nonaut Sub in Up Art, Perc Endo Approach */
"03WY4MZ"  =  "1"  /* Revision of Stimulator Lead in Up Art, Perc Endo Approach */
"0410090"  =  "1"  /* Bypass Abd Aorta to Abd Aorta with Autol Vn, Open Approach */
"0410091"  =  "1"  /* Bypass Abd Aorta to Celiac Art with Autol Vn, Open Approach */
"0410092"  =  "1"  /* Bypass Abd Aorta to Mesent Art with Autol Vn, Open Approach */
"0410093"  =  "1"  /* Bypass Abd Aorta to R Renal A with Autol Vn, Open Approach */
"0410094"  =  "1"  /* Bypass Abd Aorta to L Renal A with Autol Vn, Open Approach */
"0410095"  =  "1"  /* Bypass Abd Aorta to B Renal A with Autol Vn, Open Approach */
"0410096"  =  "1"  /* Bypass Abd Aorta to R Com Ilia with Autol Vn, Open Approach */
"0410097"  =  "1"  /* Bypass Abd Aorta to L Com Ilia with Autol Vn, Open Approach */
"0410098"  =  "1"  /* Bypass Abd Aorta to B Com Ilia with Autol Vn, Open Approach */
"0410099"  =  "1"  /* Bypass Abd Aorta to R Int Ilia with Autol Vn, Open Approach */
"041009B"  =  "1"  /* Bypass Abd Aorta to L Int Ilia with Autol Vn, Open Approach */
"041009C"  =  "1"  /* Bypass Abd Aorta to B Int Ilia with Autol Vn, Open Approach */
"041009D"  =  "1"  /* Bypass Abd Aorta to R Ext Ilia with Autol Vn, Open Approach */
"041009F"  =  "1"  /* Bypass Abd Aorta to L Ext Ilia with Autol Vn, Open Approach */
"041009G"  =  "1"  /* Bypass Abd Aorta to B Ext Ilia with Autol Vn, Open Approach */
"041009H"  =  "1"  /* Bypass Abd Aorta to R Femor A with Autol Vn, Open Approach */
"041009J"  =  "1"  /* Bypass Abd Aorta to L Femor A with Autol Vn, Open Approach */
"041009K"  =  "1"  /* Bypass Abd Aorta to B Femor A with Autol Vn, Open Approach */
"041009Q"  =  "1"  /* Bypass Abd Aorta to Low Ex Art with Autol Vn, Open Approach */
"041009R"  =  "1"  /* Bypass Abd Aorta to Low Art with Autol Vn, Open Approach */
"04100A0"  =  "1"  /* Bypass Abd Aorta to Abd Aorta with Autol Art, Open Approach */
"04100A1"  =  "1"  /* Bypass Abd Aorta to Celiac Art with Autol Art, Open Approach */
"04100A2"  =  "1"  /* Bypass Abd Aorta to Mesent Art with Autol Art, Open Approach */
"04100A3"  =  "1"  /* Bypass Abd Aorta to R Renal A with Autol Art, Open Approach */
"04100A4"  =  "1"  /* Bypass Abd Aorta to L Renal A with Autol Art, Open Approach */
"04100A5"  =  "1"  /* Bypass Abd Aorta to B Renal A with Autol Art, Open Approach */
"04100A6"  =  "1"  /* Bypass Abd Aorta to R Com Ilia with Autol Art, Open Approach */
"04100A7"  =  "1"  /* Bypass Abd Aorta to L Com Ilia with Autol Art, Open Approach */
"04100A8"  =  "1"  /* Bypass Abd Aorta to B Com Ilia with Autol Art, Open Approach */
"04100A9"  =  "1"  /* Bypass Abd Aorta to R Int Ilia with Autol Art, Open Approach */
"04100AB"  =  "1"  /* Bypass Abd Aorta to L Int Ilia with Autol Art, Open Approach */
"04100AC"  =  "1"  /* Bypass Abd Aorta to B Int Ilia with Autol Art, Open Approach */
"04100AD"  =  "1"  /* Bypass Abd Aorta to R Ext Ilia with Autol Art, Open Approach */
"04100AF"  =  "1"  /* Bypass Abd Aorta to L Ext Ilia with Autol Art, Open Approach */
"04100AG"  =  "1"  /* Bypass Abd Aorta to B Ext Ilia with Autol Art, Open Approach */
"04100AH"  =  "1"  /* Bypass Abd Aorta to R Femor A with Autol Art, Open Approach */
"04100AJ"  =  "1"  /* Bypass Abd Aorta to L Femor A with Autol Art, Open Approach */
"04100AK"  =  "1"  /* Bypass Abd Aorta to B Femor A with Autol Art, Open Approach */
"04100AQ"  =  "1"  /* Bypass Abd Aorta to Low Ex Art with Autol Art, Open Approach */
"04100AR"  =  "1"  /* Bypass Abd Aorta to Low Art with Autol Art, Open Approach */
"04100J0"  =  "1"  /* Bypass Abd Aorta to Abd Aorta with Synth Sub, Open Approach */
"04100J1"  =  "1"  /* Bypass Abd Aorta to Celiac Art with Synth Sub, Open Approach */
"04100J2"  =  "1"  /* Bypass Abd Aorta to Mesent Art with Synth Sub, Open Approach */
"04100J3"  =  "1"  /* Bypass Abd Aorta to R Renal A with Synth Sub, Open Approach */
"04100J4"  =  "1"  /* Bypass Abd Aorta to L Renal A with Synth Sub, Open Approach */
"04100J5"  =  "1"  /* Bypass Abd Aorta to B Renal A with Synth Sub, Open Approach */
"04100J6"  =  "1"  /* Bypass Abd Aorta to R Com Ilia with Synth Sub, Open Approach */
"04100J7"  =  "1"  /* Bypass Abd Aorta to L Com Ilia with Synth Sub, Open Approach */
"04100J8"  =  "1"  /* Bypass Abd Aorta to B Com Ilia with Synth Sub, Open Approach */
"04100J9"  =  "1"  /* Bypass Abd Aorta to R Int Ilia with Synth Sub, Open Approach */
"04100JB"  =  "1"  /* Bypass Abd Aorta to L Int Ilia with Synth Sub, Open Approach */
"04100JC"  =  "1"  /* Bypass Abd Aorta to B Int Ilia with Synth Sub, Open Approach */
"04100JD"  =  "1"  /* Bypass Abd Aorta to R Ext Ilia with Synth Sub, Open Approach */
"04100JF"  =  "1"  /* Bypass Abd Aorta to L Ext Ilia with Synth Sub, Open Approach */
"04100JG"  =  "1"  /* Bypass Abd Aorta to B Ext Ilia with Synth Sub, Open Approach */
"04100JH"  =  "1"  /* Bypass Abd Aorta to R Femor A with Synth Sub, Open Approach */
"04100JJ"  =  "1"  /* Bypass Abd Aorta to L Femor A with Synth Sub, Open Approach */
"04100JK"  =  "1"  /* Bypass Abd Aorta to B Femor A with Synth Sub, Open Approach */
"04100JQ"  =  "1"  /* Bypass Abd Aorta to Low Ex Art with Synth Sub, Open Approach */
"04100JR"  =  "1"  /* Bypass Abd Aorta to Low Art with Synth Sub, Open Approach */
"04100K0"  =  "1"  /* Bypass Abd Aorta to Abd Aorta with Nonaut Sub, Open Approach */
"04100K1"  =  "1"  /* Bypass Abd Aorta to Celiac Art w Nonaut Sub, Open */
"04100K2"  =  "1"  /* Bypass Abd Aorta to Mesent Art w Nonaut Sub, Open */
"04100K3"  =  "1"  /* Bypass Abd Aorta to R Renal A with Nonaut Sub, Open Approach */
"04100K4"  =  "1"  /* Bypass Abd Aorta to L Renal A with Nonaut Sub, Open Approach */
"04100K5"  =  "1"  /* Bypass Abd Aorta to B Renal A with Nonaut Sub, Open Approach */
"04100K6"  =  "1"  /* Bypass Abd Aorta to R Com Ilia w Nonaut Sub, Open */
"04100K7"  =  "1"  /* Bypass Abd Aorta to L Com Ilia w Nonaut Sub, Open */
"04100K8"  =  "1"  /* Bypass Abd Aorta to B Com Ilia w Nonaut Sub, Open */
"04100K9"  =  "1"  /* Bypass Abd Aorta to R Int Ilia w Nonaut Sub, Open */
"04100KB"  =  "1"  /* Bypass Abd Aorta to L Int Ilia w Nonaut Sub, Open */
"04100KC"  =  "1"  /* Bypass Abd Aorta to B Int Ilia w Nonaut Sub, Open */
"04100KD"  =  "1"  /* Bypass Abd Aorta to R Ext Ilia w Nonaut Sub, Open */
"04100KF"  =  "1"  /* Bypass Abd Aorta to L Ext Ilia w Nonaut Sub, Open */
"04100KG"  =  "1"  /* Bypass Abd Aorta to B Ext Ilia w Nonaut Sub, Open */
"04100KH"  =  "1"  /* Bypass Abd Aorta to R Femor A with Nonaut Sub, Open Approach */
"04100KJ"  =  "1"  /* Bypass Abd Aorta to L Femor A with Nonaut Sub, Open Approach */
"04100KK"  =  "1"  /* Bypass Abd Aorta to B Femor A with Nonaut Sub, Open Approach */
"04100KQ"  =  "1"  /* Bypass Abd Aorta to Low Ex Art w Nonaut Sub, Open */
"04100KR"  =  "1"  /* Bypass Abd Aorta to Low Art with Nonaut Sub, Open Approach */
"04100Z0"  =  "1"  /* Bypass Abdominal Aorta to Abdominal Aorta, Open Approach */
"04100Z1"  =  "1"  /* Bypass Abdominal Aorta to Celiac Artery, Open Approach */
"04100Z2"  =  "1"  /* Bypass Abdominal Aorta to Mesenteric Artery, Open Approach */
"04100Z3"  =  "1"  /* Bypass Abdominal Aorta to Right Renal Artery, Open Approach */
"04100Z4"  =  "1"  /* Bypass Abdominal Aorta to Left Renal Artery, Open Approach */
"04100Z5"  =  "1"  /* Bypass Abdominal Aorta to B Renal A, Open Approach */
"04100Z6"  =  "1"  /* Bypass Abdominal Aorta to R Com Ilia, Open Approach */
"04100Z7"  =  "1"  /* Bypass Abdominal Aorta to L Com Ilia, Open Approach */
"04100Z8"  =  "1"  /* Bypass Abdominal Aorta to B Com Ilia, Open Approach */
"04100Z9"  =  "1"  /* Bypass Abdominal Aorta to R Int Ilia, Open Approach */
"04100ZB"  =  "1"  /* Bypass Abdominal Aorta to L Int Ilia, Open Approach */
"04100ZC"  =  "1"  /* Bypass Abdominal Aorta to B Int Ilia, Open Approach */
"04100ZD"  =  "1"  /* Bypass Abdominal Aorta to R Ext Ilia, Open Approach */
"04100ZF"  =  "1"  /* Bypass Abdominal Aorta to L Ext Ilia, Open Approach */
"04100ZG"  =  "1"  /* Bypass Abdominal Aorta to B Ext Ilia, Open Approach */
"04100ZH"  =  "1"  /* Bypass Abdominal Aorta to R Femor A, Open Approach */
"04100ZJ"  =  "1"  /* Bypass Abdominal Aorta to Left Femoral Artery, Open Approach */
"04100ZK"  =  "1"  /* Bypass Abdominal Aorta to B Femor A, Open Approach */
"04100ZQ"  =  "1"  /* Bypass Abdominal Aorta to Low Ex Art, Open Approach */
"04100ZR"  =  "1"  /* Bypass Abdominal Aorta to Lower Artery, Open Approach */
"0410490"  =  "1"  /* Bypass Abd Aorta to Abd Aorta w Autol Vn, Perc Endo */
"0410491"  =  "1"  /* Bypass Abd Aorta to Celiac Art w Autol Vn, Perc Endo */
"0410492"  =  "1"  /* Bypass Abd Aorta to Mesent Art w Autol Vn, Perc Endo */
"0410493"  =  "1"  /* Bypass Abd Aorta to R Renal A w Autol Vn, Perc Endo */
"0410494"  =  "1"  /* Bypass Abd Aorta to L Renal A w Autol Vn, Perc Endo */
"0410495"  =  "1"  /* Bypass Abd Aorta to B Renal A w Autol Vn, Perc Endo */
"0410496"  =  "1"  /* Bypass Abd Aorta to R Com Ilia w Autol Vn, Perc Endo */
"0410497"  =  "1"  /* Bypass Abd Aorta to L Com Ilia w Autol Vn, Perc Endo */
"0410498"  =  "1"  /* Bypass Abd Aorta to B Com Ilia w Autol Vn, Perc Endo */
"0410499"  =  "1"  /* Bypass Abd Aorta to R Int Ilia w Autol Vn, Perc Endo */
"041049B"  =  "1"  /* Bypass Abd Aorta to L Int Ilia w Autol Vn, Perc Endo */
"041049C"  =  "1"  /* Bypass Abd Aorta to B Int Ilia w Autol Vn, Perc Endo */
"041049D"  =  "1"  /* Bypass Abd Aorta to R Ext Ilia w Autol Vn, Perc Endo */
"041049F"  =  "1"  /* Bypass Abd Aorta to L Ext Ilia w Autol Vn, Perc Endo */
"041049G"  =  "1"  /* Bypass Abd Aorta to B Ext Ilia w Autol Vn, Perc Endo */
"041049H"  =  "1"  /* Bypass Abd Aorta to R Femor A w Autol Vn, Perc Endo */
"041049J"  =  "1"  /* Bypass Abd Aorta to L Femor A w Autol Vn, Perc Endo */
"041049K"  =  "1"  /* Bypass Abd Aorta to B Femor A w Autol Vn, Perc Endo */
"041049Q"  =  "1"  /* Bypass Abd Aorta to Low Ex Art w Autol Vn, Perc Endo */
"041049R"  =  "1"  /* Bypass Abd Aorta to Low Art w Autol Vn, Perc Endo */
"04104A0"  =  "1"  /* Bypass Abd Aorta to Abd Aorta w Autol Art, Perc Endo */
"04104A1"  =  "1"  /* Bypass Abd Aorta to Celiac Art w Autol Art, Perc Endo */
"04104A2"  =  "1"  /* Bypass Abd Aorta to Mesent Art w Autol Art, Perc Endo */
"04104A3"  =  "1"  /* Bypass Abd Aorta to R Renal A w Autol Art, Perc Endo */
"04104A4"  =  "1"  /* Bypass Abd Aorta to L Renal A w Autol Art, Perc Endo */
"04104A5"  =  "1"  /* Bypass Abd Aorta to B Renal A w Autol Art, Perc Endo */
"04104A6"  =  "1"  /* Bypass Abd Aorta to R Com Ilia w Autol Art, Perc Endo */
"04104A7"  =  "1"  /* Bypass Abd Aorta to L Com Ilia w Autol Art, Perc Endo */
"04104A8"  =  "1"  /* Bypass Abd Aorta to B Com Ilia w Autol Art, Perc Endo */
"04104A9"  =  "1"  /* Bypass Abd Aorta to R Int Ilia w Autol Art, Perc Endo */
"04104AB"  =  "1"  /* Bypass Abd Aorta to L Int Ilia w Autol Art, Perc Endo */
"04104AC"  =  "1"  /* Bypass Abd Aorta to B Int Ilia w Autol Art, Perc Endo */
"04104AD"  =  "1"  /* Bypass Abd Aorta to R Ext Ilia w Autol Art, Perc Endo */
"04104AF"  =  "1"  /* Bypass Abd Aorta to L Ext Ilia w Autol Art, Perc Endo */
"04104AG"  =  "1"  /* Bypass Abd Aorta to B Ext Ilia w Autol Art, Perc Endo */
"04104AH"  =  "1"  /* Bypass Abd Aorta to R Femor A w Autol Art, Perc Endo */
"04104AJ"  =  "1"  /* Bypass Abd Aorta to L Femor A w Autol Art, Perc Endo */
"04104AK"  =  "1"  /* Bypass Abd Aorta to B Femor A w Autol Art, Perc Endo */
"04104AQ"  =  "1"  /* Bypass Abd Aorta to Low Ex Art w Autol Art, Perc Endo */
"04104AR"  =  "1"  /* Bypass Abd Aorta to Low Art w Autol Art, Perc Endo */
"04104J0"  =  "1"  /* Bypass Abd Aorta to Abd Aorta w Synth Sub, Perc Endo */
"04104J1"  =  "1"  /* Bypass Abd Aorta to Celiac Art w Synth Sub, Perc Endo */
"04104J2"  =  "1"  /* Bypass Abd Aorta to Mesent Art w Synth Sub, Perc Endo */
"04104J3"  =  "1"  /* Bypass Abd Aorta to R Renal A w Synth Sub, Perc Endo */
"04104J4"  =  "1"  /* Bypass Abd Aorta to L Renal A w Synth Sub, Perc Endo */
"04104J5"  =  "1"  /* Bypass Abd Aorta to B Renal A w Synth Sub, Perc Endo */
"04104J6"  =  "1"  /* Bypass Abd Aorta to R Com Ilia w Synth Sub, Perc Endo */
"04104J7"  =  "1"  /* Bypass Abd Aorta to L Com Ilia w Synth Sub, Perc Endo */
"04104J8"  =  "1"  /* Bypass Abd Aorta to B Com Ilia w Synth Sub, Perc Endo */
"04104J9"  =  "1"  /* Bypass Abd Aorta to R Int Ilia w Synth Sub, Perc Endo */
"04104JB"  =  "1"  /* Bypass Abd Aorta to L Int Ilia w Synth Sub, Perc Endo */
"04104JC"  =  "1"  /* Bypass Abd Aorta to B Int Ilia w Synth Sub, Perc Endo */
"04104JD"  =  "1"  /* Bypass Abd Aorta to R Ext Ilia w Synth Sub, Perc Endo */
"04104JF"  =  "1"  /* Bypass Abd Aorta to L Ext Ilia w Synth Sub, Perc Endo */
"04104JG"  =  "1"  /* Bypass Abd Aorta to B Ext Ilia w Synth Sub, Perc Endo */
"04104JH"  =  "1"  /* Bypass Abd Aorta to R Femor A w Synth Sub, Perc Endo */
"04104JJ"  =  "1"  /* Bypass Abd Aorta to L Femor A w Synth Sub, Perc Endo */
"04104JK"  =  "1"  /* Bypass Abd Aorta to B Femor A w Synth Sub, Perc Endo */
"04104JQ"  =  "1"  /* Bypass Abd Aorta to Low Ex Art w Synth Sub, Perc Endo */
"04104JR"  =  "1"  /* Bypass Abd Aorta to Low Art w Synth Sub, Perc Endo */
"04104K0"  =  "1"  /* Bypass Abd Aorta to Abd Aorta w Nonaut Sub, Perc Endo */
"04104K1"  =  "1"  /* Bypass Abd Aorta to Celiac Art w Nonaut Sub, Perc Endo */
"04104K2"  =  "1"  /* Bypass Abd Aorta to Mesent Art w Nonaut Sub, Perc Endo */
"04104K3"  =  "1"  /* Bypass Abd Aorta to R Renal A w Nonaut Sub, Perc Endo */
"04104K4"  =  "1"  /* Bypass Abd Aorta to L Renal A w Nonaut Sub, Perc Endo */
"04104K5"  =  "1"  /* Bypass Abd Aorta to B Renal A w Nonaut Sub, Perc Endo */
"04104K6"  =  "1"  /* Bypass Abd Aorta to R Com Ilia w Nonaut Sub, Perc Endo */
"04104K7"  =  "1"  /* Bypass Abd Aorta to L Com Ilia w Nonaut Sub, Perc Endo */
"04104K8"  =  "1"  /* Bypass Abd Aorta to B Com Ilia w Nonaut Sub, Perc Endo */
"04104K9"  =  "1"  /* Bypass Abd Aorta to R Int Ilia w Nonaut Sub, Perc Endo */
"04104KB"  =  "1"  /* Bypass Abd Aorta to L Int Ilia w Nonaut Sub, Perc Endo */
"04104KC"  =  "1"  /* Bypass Abd Aorta to B Int Ilia w Nonaut Sub, Perc Endo */
"04104KD"  =  "1"  /* Bypass Abd Aorta to R Ext Ilia w Nonaut Sub, Perc Endo */
"04104KF"  =  "1"  /* Bypass Abd Aorta to L Ext Ilia w Nonaut Sub, Perc Endo */
"04104KG"  =  "1"  /* Bypass Abd Aorta to B Ext Ilia w Nonaut Sub, Perc Endo */
"04104KH"  =  "1"  /* Bypass Abd Aorta to R Femor A w Nonaut Sub, Perc Endo */
"04104KJ"  =  "1"  /* Bypass Abd Aorta to L Femor A w Nonaut Sub, Perc Endo */
"04104KK"  =  "1"  /* Bypass Abd Aorta to B Femor A w Nonaut Sub, Perc Endo */
"04104KQ"  =  "1"  /* Bypass Abd Aorta to Low Ex Art w Nonaut Sub, Perc Endo */
"04104KR"  =  "1"  /* Bypass Abd Aorta to Low Art w Nonaut Sub, Perc Endo */
"04104Z0"  =  "1"  /* Bypass Abdominal Aorta to Abd Aorta, Perc Endo Approach */
"04104Z1"  =  "1"  /* Bypass Abdominal Aorta to Celiac Artery, Perc Endo Approach */
"04104Z2"  =  "1"  /* Bypass Abdominal Aorta to Mesent Art, Perc Endo Approach */
"04104Z3"  =  "1"  /* Bypass Abdominal Aorta to R Renal A, Perc Endo Approach */
"04104Z4"  =  "1"  /* Bypass Abdominal Aorta to L Renal A, Perc Endo Approach */
"04104Z5"  =  "1"  /* Bypass Abdominal Aorta to B Renal A, Perc Endo Approach */
"04104Z6"  =  "1"  /* Bypass Abdominal Aorta to R Com Ilia, Perc Endo Approach */
"04104Z7"  =  "1"  /* Bypass Abdominal Aorta to L Com Ilia, Perc Endo Approach */
"04104Z8"  =  "1"  /* Bypass Abdominal Aorta to B Com Ilia, Perc Endo Approach */
"04104Z9"  =  "1"  /* Bypass Abdominal Aorta to R Int Ilia, Perc Endo Approach */
"04104ZB"  =  "1"  /* Bypass Abdominal Aorta to L Int Ilia, Perc Endo Approach */
"04104ZC"  =  "1"  /* Bypass Abdominal Aorta to B Int Ilia, Perc Endo Approach */
"04104ZD"  =  "1"  /* Bypass Abdominal Aorta to R Ext Ilia, Perc Endo Approach */
"04104ZF"  =  "1"  /* Bypass Abdominal Aorta to L Ext Ilia, Perc Endo Approach */
"04104ZG"  =  "1"  /* Bypass Abdominal Aorta to B Ext Ilia, Perc Endo Approach */
"04104ZH"  =  "1"  /* Bypass Abdominal Aorta to R Femor A, Perc Endo Approach */
"04104ZJ"  =  "1"  /* Bypass Abdominal Aorta to L Femor A, Perc Endo Approach */
"04104ZK"  =  "1"  /* Bypass Abdominal Aorta to B Femor A, Perc Endo Approach */
"04104ZQ"  =  "1"  /* Bypass Abdominal Aorta to Low Ex Art, Perc Endo Approach */
"04104ZR"  =  "1"  /* Bypass Abdominal Aorta to Lower Artery, Perc Endo Approach */
"0414093"  =  "1"  /* Bypass Splenic Art to R Renal A with Autol Vn, Open Approach */
"0414094"  =  "1"  /* Bypass Splenic Art to L Renal A with Autol Vn, Open Approach */
"0414095"  =  "1"  /* Bypass Splenic Art to B Renal A with Autol Vn, Open Approach */
"04140A3"  =  "1"  /* Bypass Splenic Art to R Renal A w Autol Art, Open */
"04140A4"  =  "1"  /* Bypass Splenic Art to L Renal A w Autol Art, Open */
"04140A5"  =  "1"  /* Bypass Splenic Art to B Renal A w Autol Art, Open */
"04140J3"  =  "1"  /* Bypass Splenic Art to R Renal A w Synth Sub, Open */
"04140J4"  =  "1"  /* Bypass Splenic Art to L Renal A w Synth Sub, Open */
"04140J5"  =  "1"  /* Bypass Splenic Art to B Renal A w Synth Sub, Open */
"04140K3"  =  "1"  /* Bypass Splenic Art to R Renal A w Nonaut Sub, Open */
"04140K4"  =  "1"  /* Bypass Splenic Art to L Renal A w Nonaut Sub, Open */
"04140K5"  =  "1"  /* Bypass Splenic Art to B Renal A w Nonaut Sub, Open */
"04140Z3"  =  "1"  /* Bypass Splenic Artery to Right Renal Artery, Open Approach */
"04140Z4"  =  "1"  /* Bypass Splenic Artery to Left Renal Artery, Open Approach */
"04140Z5"  =  "1"  /* Bypass Splenic Artery to B Renal A, Open Approach */
"0414493"  =  "1"  /* Bypass Splenic Art to R Renal A w Autol Vn, Perc Endo */
"0414494"  =  "1"  /* Bypass Splenic Art to L Renal A w Autol Vn, Perc Endo */
"0414495"  =  "1"  /* Bypass Splenic Art to B Renal A w Autol Vn, Perc Endo */
"04144A3"  =  "1"  /* Bypass Splenic Art to R Renal A w Autol Art, Perc Endo */
"04144A4"  =  "1"  /* Bypass Splenic Art to L Renal A w Autol Art, Perc Endo */
"04144A5"  =  "1"  /* Bypass Splenic Art to B Renal A w Autol Art, Perc Endo */
"04144J3"  =  "1"  /* Bypass Splenic Art to R Renal A w Synth Sub, Perc Endo */
"04144J4"  =  "1"  /* Bypass Splenic Art to L Renal A w Synth Sub, Perc Endo */
"04144J5"  =  "1"  /* Bypass Splenic Art to B Renal A w Synth Sub, Perc Endo */
"04144K3"  =  "1"  /* Bypass Splenic Art to R Renal A w Nonaut Sub, Perc Endo */
"04144K4"  =  "1"  /* Bypass Splenic Art to L Renal A w Nonaut Sub, Perc Endo */
"04144K5"  =  "1"  /* Bypass Splenic Art to B Renal A w Nonaut Sub, Perc Endo */
"04144Z3"  =  "1"  /* Bypass Splenic Artery to R Renal A, Perc Endo Approach */
"04144Z4"  =  "1"  /* Bypass Splenic Artery to L Renal A, Perc Endo Approach */
"04144Z5"  =  "1"  /* Bypass Splenic Artery to B Renal A, Perc Endo Approach */
"041C090"  =  "1"  /* Bypass R Com Iliac Art to Abd Aorta w Autol Vn, Open */
"041C091"  =  "1"  /* Bypass R Com Iliac Art to Celiac Art w Autol Vn, Open */
"041C092"  =  "1"  /* Bypass R Com Iliac Art to Mesent Art w Autol Vn, Open */
"041C093"  =  "1"  /* Bypass R Com Iliac Art to R Renal A w Autol Vn, Open */
"041C094"  =  "1"  /* Bypass R Com Iliac Art to L Renal A w Autol Vn, Open */
"041C095"  =  "1"  /* Bypass R Com Iliac Art to B Renal A w Autol Vn, Open */
"041C096"  =  "1"  /* Bypass R Com Iliac Art to R Com Ilia w Autol Vn, Open */
"041C097"  =  "1"  /* Bypass R Com Iliac Art to L Com Ilia w Autol Vn, Open */
"041C098"  =  "1"  /* Bypass R Com Iliac Art to B Com Ilia w Autol Vn, Open */
"041C099"  =  "1"  /* Bypass R Com Iliac Art to R Int Ilia w Autol Vn, Open */
"041C09B"  =  "1"  /* Bypass R Com Iliac Art to L Int Ilia w Autol Vn, Open */
"041C09C"  =  "1"  /* Bypass R Com Iliac Art to B Int Ilia w Autol Vn, Open */
"041C09D"  =  "1"  /* Bypass R Com Iliac Art to R Ext Ilia w Autol Vn, Open */
"041C09F"  =  "1"  /* Bypass R Com Iliac Art to L Ext Ilia w Autol Vn, Open */
"041C09G"  =  "1"  /* Bypass R Com Iliac Art to B Ext Ilia w Autol Vn, Open */
"041C09H"  =  "1"  /* Bypass R Com Iliac Art to R Femor A w Autol Vn, Open */
"041C09J"  =  "1"  /* Bypass R Com Iliac Art to L Femor A w Autol Vn, Open */
"041C09K"  =  "1"  /* Bypass R Com Iliac Art to B Femor A w Autol Vn, Open */
"041C09Q"  =  "1"  /* Bypass R Com Iliac Art to Low Ex Art w Autol Vn, Open */
"041C09R"  =  "1"  /* Bypass R Com Iliac Art to Low Art w Autol Vn, Open */
"041C0A0"  =  "1"  /* Bypass R Com Iliac Art to Abd Aorta w Autol Art, Open */
"041C0A1"  =  "1"  /* Bypass R Com Iliac Art to Celiac Art w Autol Art, Open */
"041C0A2"  =  "1"  /* Bypass R Com Iliac Art to Mesent Art w Autol Art, Open */
"041C0A3"  =  "1"  /* Bypass R Com Iliac Art to R Renal A w Autol Art, Open */
"041C0A4"  =  "1"  /* Bypass R Com Iliac Art to L Renal A w Autol Art, Open */
"041C0A5"  =  "1"  /* Bypass R Com Iliac Art to B Renal A w Autol Art, Open */
"041C0A6"  =  "1"  /* Bypass R Com Iliac Art to R Com Ilia w Autol Art, Open */
"041C0A7"  =  "1"  /* Bypass R Com Iliac Art to L Com Ilia w Autol Art, Open */
"041C0A8"  =  "1"  /* Bypass R Com Iliac Art to B Com Ilia w Autol Art, Open */
"041C0A9"  =  "1"  /* Bypass R Com Iliac Art to R Int Ilia w Autol Art, Open */
"041C0AB"  =  "1"  /* Bypass R Com Iliac Art to L Int Ilia w Autol Art, Open */
"041C0AC"  =  "1"  /* Bypass R Com Iliac Art to B Int Ilia w Autol Art, Open */
"041C0AD"  =  "1"  /* Bypass R Com Iliac Art to R Ext Ilia w Autol Art, Open */
"041C0AF"  =  "1"  /* Bypass R Com Iliac Art to L Ext Ilia w Autol Art, Open */
"041C0AG"  =  "1"  /* Bypass R Com Iliac Art to B Ext Ilia w Autol Art, Open */
"041C0AH"  =  "1"  /* Bypass R Com Iliac Art to R Femor A w Autol Art, Open */
"041C0AJ"  =  "1"  /* Bypass R Com Iliac Art to L Femor A w Autol Art, Open */
"041C0AK"  =  "1"  /* Bypass R Com Iliac Art to B Femor A w Autol Art, Open */
"041C0AQ"  =  "1"  /* Bypass R Com Iliac Art to Low Ex Art w Autol Art, Open */
"041C0AR"  =  "1"  /* Bypass R Com Iliac Art to Low Art w Autol Art, Open */
"041C0J0"  =  "1"  /* Bypass R Com Iliac Art to Abd Aorta w Synth Sub, Open */
"041C0J1"  =  "1"  /* Bypass R Com Iliac Art to Celiac Art w Synth Sub, Open */
"041C0J2"  =  "1"  /* Bypass R Com Iliac Art to Mesent Art w Synth Sub, Open */
"041C0J3"  =  "1"  /* Bypass R Com Iliac Art to R Renal A w Synth Sub, Open */
"041C0J4"  =  "1"  /* Bypass R Com Iliac Art to L Renal A w Synth Sub, Open */
"041C0J5"  =  "1"  /* Bypass R Com Iliac Art to B Renal A w Synth Sub, Open */
"041C0J6"  =  "1"  /* Bypass R Com Iliac Art to R Com Ilia w Synth Sub, Open */
"041C0J7"  =  "1"  /* Bypass R Com Iliac Art to L Com Ilia w Synth Sub, Open */
"041C0J8"  =  "1"  /* Bypass R Com Iliac Art to B Com Ilia w Synth Sub, Open */
"041C0J9"  =  "1"  /* Bypass R Com Iliac Art to R Int Ilia w Synth Sub, Open */
"041C0JB"  =  "1"  /* Bypass R Com Iliac Art to L Int Ilia w Synth Sub, Open */
"041C0JC"  =  "1"  /* Bypass R Com Iliac Art to B Int Ilia w Synth Sub, Open */
"041C0JD"  =  "1"  /* Bypass R Com Iliac Art to R Ext Ilia w Synth Sub, Open */
"041C0JF"  =  "1"  /* Bypass R Com Iliac Art to L Ext Ilia w Synth Sub, Open */
"041C0JG"  =  "1"  /* Bypass R Com Iliac Art to B Ext Ilia w Synth Sub, Open */
"041C0JH"  =  "1"  /* Bypass R Com Iliac Art to R Femor A w Synth Sub, Open */
"041C0JJ"  =  "1"  /* Bypass R Com Iliac Art to L Femor A w Synth Sub, Open */
"041C0JK"  =  "1"  /* Bypass R Com Iliac Art to B Femor A w Synth Sub, Open */
"041C0JQ"  =  "1"  /* Bypass R Com Iliac Art to Low Ex Art w Synth Sub, Open */
"041C0JR"  =  "1"  /* Bypass R Com Iliac Art to Low Art w Synth Sub, Open */
"041C0K0"  =  "1"  /* Bypass R Com Iliac Art to Abd Aorta w Nonaut Sub, Open */
"041C0K1"  =  "1"  /* Bypass R Com Iliac Art to Celiac Art w Nonaut Sub, Open */
"041C0K2"  =  "1"  /* Bypass R Com Iliac Art to Mesent Art w Nonaut Sub, Open */
"041C0K3"  =  "1"  /* Bypass R Com Iliac Art to R Renal A w Nonaut Sub, Open */
"041C0K4"  =  "1"  /* Bypass R Com Iliac Art to L Renal A w Nonaut Sub, Open */
"041C0K5"  =  "1"  /* Bypass R Com Iliac Art to B Renal A w Nonaut Sub, Open */
"041C0K6"  =  "1"  /* Bypass R Com Iliac Art to R Com Ilia w Nonaut Sub, Open */
"041C0K7"  =  "1"  /* Bypass R Com Iliac Art to L Com Ilia w Nonaut Sub, Open */
"041C0K8"  =  "1"  /* Bypass R Com Iliac Art to B Com Ilia w Nonaut Sub, Open */
"041C0K9"  =  "1"  /* Bypass R Com Iliac Art to R Int Ilia w Nonaut Sub, Open */
"041C0KB"  =  "1"  /* Bypass R Com Iliac Art to L Int Ilia w Nonaut Sub, Open */
"041C0KC"  =  "1"  /* Bypass R Com Iliac Art to B Int Ilia w Nonaut Sub, Open */
"041C0KD"  =  "1"  /* Bypass R Com Iliac Art to R Ext Ilia w Nonaut Sub, Open */
"041C0KF"  =  "1"  /* Bypass R Com Iliac Art to L Ext Ilia w Nonaut Sub, Open */
"041C0KG"  =  "1"  /* Bypass R Com Iliac Art to B Ext Ilia w Nonaut Sub, Open */
"041C0KH"  =  "1"  /* Bypass R Com Iliac Art to R Femor A w Nonaut Sub, Open */
"041C0KJ"  =  "1"  /* Bypass R Com Iliac Art to L Femor A w Nonaut Sub, Open */
"041C0KK"  =  "1"  /* Bypass R Com Iliac Art to B Femor A w Nonaut Sub, Open */
"041C0KQ"  =  "1"  /* Bypass R Com Iliac Art to Low Ex Art w Nonaut Sub, Open */
"041C0KR"  =  "1"  /* Bypass R Com Iliac Art to Low Art w Nonaut Sub, Open */
"041C0Z0"  =  "1"  /* Bypass Right Common Iliac Artery to Abd Aorta, Open Approach */
"041C0Z1"  =  "1"  /* Bypass R Com Iliac Art to Celiac Art, Open Approach */
"041C0Z2"  =  "1"  /* Bypass R Com Iliac Art to Mesent Art, Open Approach */
"041C0Z3"  =  "1"  /* Bypass Right Common Iliac Artery to R Renal A, Open Approach */
"041C0Z4"  =  "1"  /* Bypass Right Common Iliac Artery to L Renal A, Open Approach */
"041C0Z5"  =  "1"  /* Bypass Right Common Iliac Artery to B Renal A, Open Approach */
"041C0Z6"  =  "1"  /* Bypass R Com Iliac Art to R Com Ilia, Open Approach */
"041C0Z7"  =  "1"  /* Bypass R Com Iliac Art to L Com Ilia, Open Approach */
"041C0Z8"  =  "1"  /* Bypass R Com Iliac Art to B Com Ilia, Open Approach */
"041C0Z9"  =  "1"  /* Bypass R Com Iliac Art to R Int Ilia, Open Approach */
"041C0ZB"  =  "1"  /* Bypass R Com Iliac Art to L Int Ilia, Open Approach */
"041C0ZC"  =  "1"  /* Bypass R Com Iliac Art to B Int Ilia, Open Approach */
"041C0ZD"  =  "1"  /* Bypass R Com Iliac Art to R Ext Ilia, Open Approach */
"041C0ZF"  =  "1"  /* Bypass R Com Iliac Art to L Ext Ilia, Open Approach */
"041C0ZG"  =  "1"  /* Bypass R Com Iliac Art to B Ext Ilia, Open Approach */
"041C0ZH"  =  "1"  /* Bypass Right Common Iliac Artery to R Femor A, Open Approach */
"041C0ZJ"  =  "1"  /* Bypass Right Common Iliac Artery to L Femor A, Open Approach */
"041C0ZK"  =  "1"  /* Bypass Right Common Iliac Artery to B Femor A, Open Approach */
"041C0ZQ"  =  "1"  /* Bypass R Com Iliac Art to Low Ex Art, Open Approach */
"041C0ZR"  =  "1"  /* Bypass Right Common Iliac Artery to Low Art, Open Approach */
"041C490"  =  "1"  /* Bypass R Com Iliac Art to Abd Aorta w Autol Vn, Perc Endo */
"041C491"  =  "1"  /* Bypass R Com Iliac Art to Celiac Art w Autol Vn, Perc Endo */
"041C492"  =  "1"  /* Bypass R Com Iliac Art to Mesent Art w Autol Vn, Perc Endo */
"041C493"  =  "1"  /* Bypass R Com Iliac Art to R Renal A w Autol Vn, Perc Endo */
"041C494"  =  "1"  /* Bypass R Com Iliac Art to L Renal A w Autol Vn, Perc Endo */
"041C495"  =  "1"  /* Bypass R Com Iliac Art to B Renal A w Autol Vn, Perc Endo */
"041C496"  =  "1"  /* Bypass R Com Iliac Art to R Com Ilia w Autol Vn, Perc Endo */
"041C497"  =  "1"  /* Bypass R Com Iliac Art to L Com Ilia w Autol Vn, Perc Endo */
"041C498"  =  "1"  /* Bypass R Com Iliac Art to B Com Ilia w Autol Vn, Perc Endo */
"041C499"  =  "1"  /* Bypass R Com Iliac Art to R Int Ilia w Autol Vn, Perc Endo */
"041C49B"  =  "1"  /* Bypass R Com Iliac Art to L Int Ilia w Autol Vn, Perc Endo */
"041C49C"  =  "1"  /* Bypass R Com Iliac Art to B Int Ilia w Autol Vn, Perc Endo */
"041C49D"  =  "1"  /* Bypass R Com Iliac Art to R Ext Ilia w Autol Vn, Perc Endo */
"041C49F"  =  "1"  /* Bypass R Com Iliac Art to L Ext Ilia w Autol Vn, Perc Endo */
"041C49G"  =  "1"  /* Bypass R Com Iliac Art to B Ext Ilia w Autol Vn, Perc Endo */
"041C49H"  =  "1"  /* Bypass R Com Iliac Art to R Femor A w Autol Vn, Perc Endo */
"041C49J"  =  "1"  /* Bypass R Com Iliac Art to L Femor A w Autol Vn, Perc Endo */
"041C49K"  =  "1"  /* Bypass R Com Iliac Art to B Femor A w Autol Vn, Perc Endo */
"041C49Q"  =  "1"  /* Bypass R Com Iliac Art to Low Ex Art w Autol Vn, Perc Endo */
"041C49R"  =  "1"  /* Bypass R Com Iliac Art to Low Art w Autol Vn, Perc Endo */
"041C4A0"  =  "1"  /* Bypass R Com Iliac Art to Abd Aorta w Autol Art, Perc Endo */
"041C4A1"  =  "1"  /* Bypass R Com Iliac Art to Celiac Art w Autol Art, Perc Endo */
"041C4A2"  =  "1"  /* Bypass R Com Iliac Art to Mesent Art w Autol Art, Perc Endo */
"041C4A3"  =  "1"  /* Bypass R Com Iliac Art to R Renal A w Autol Art, Perc Endo */
"041C4A4"  =  "1"  /* Bypass R Com Iliac Art to L Renal A w Autol Art, Perc Endo */
"041C4A5"  =  "1"  /* Bypass R Com Iliac Art to B Renal A w Autol Art, Perc Endo */
"041C4A6"  =  "1"  /* Bypass R Com Iliac Art to R Com Ilia w Autol Art, Perc Endo */
"041C4A7"  =  "1"  /* Bypass R Com Iliac Art to L Com Ilia w Autol Art, Perc Endo */
"041C4A8"  =  "1"  /* Bypass R Com Iliac Art to B Com Ilia w Autol Art, Perc Endo */
"041C4A9"  =  "1"  /* Bypass R Com Iliac Art to R Int Ilia w Autol Art, Perc Endo */
"041C4AB"  =  "1"  /* Bypass R Com Iliac Art to L Int Ilia w Autol Art, Perc Endo */
"041C4AC"  =  "1"  /* Bypass R Com Iliac Art to B Int Ilia w Autol Art, Perc Endo */
"041C4AD"  =  "1"  /* Bypass R Com Iliac Art to R Ext Ilia w Autol Art, Perc Endo */
"041C4AF"  =  "1"  /* Bypass R Com Iliac Art to L Ext Ilia w Autol Art, Perc Endo */
"041C4AG"  =  "1"  /* Bypass R Com Iliac Art to B Ext Ilia w Autol Art, Perc Endo */
"041C4AH"  =  "1"  /* Bypass R Com Iliac Art to R Femor A w Autol Art, Perc Endo */
"041C4AJ"  =  "1"  /* Bypass R Com Iliac Art to L Femor A w Autol Art, Perc Endo */
"041C4AK"  =  "1"  /* Bypass R Com Iliac Art to B Femor A w Autol Art, Perc Endo */
"041C4AQ"  =  "1"  /* Bypass R Com Iliac Art to Low Ex Art w Autol Art, Perc Endo */
"041C4AR"  =  "1"  /* Bypass R Com Iliac Art to Low Art w Autol Art, Perc Endo */
"041C4J0"  =  "1"  /* Bypass R Com Iliac Art to Abd Aorta w Synth Sub, Perc Endo */
"041C4J1"  =  "1"  /* Bypass R Com Iliac Art to Celiac Art w Synth Sub, Perc Endo */
"041C4J2"  =  "1"  /* Bypass R Com Iliac Art to Mesent Art w Synth Sub, Perc Endo */
"041C4J3"  =  "1"  /* Bypass R Com Iliac Art to R Renal A w Synth Sub, Perc Endo */
"041C4J4"  =  "1"  /* Bypass R Com Iliac Art to L Renal A w Synth Sub, Perc Endo */
"041C4J5"  =  "1"  /* Bypass R Com Iliac Art to B Renal A w Synth Sub, Perc Endo */
"041C4J6"  =  "1"  /* Bypass R Com Iliac Art to R Com Ilia w Synth Sub, Perc Endo */
"041C4J7"  =  "1"  /* Bypass R Com Iliac Art to L Com Ilia w Synth Sub, Perc Endo */
"041C4J8"  =  "1"  /* Bypass R Com Iliac Art to B Com Ilia w Synth Sub, Perc Endo */
"041C4J9"  =  "1"  /* Bypass R Com Iliac Art to R Int Ilia w Synth Sub, Perc Endo */
"041C4JB"  =  "1"  /* Bypass R Com Iliac Art to L Int Ilia w Synth Sub, Perc Endo */
"041C4JC"  =  "1"  /* Bypass R Com Iliac Art to B Int Ilia w Synth Sub, Perc Endo */
"041C4JD"  =  "1"  /* Bypass R Com Iliac Art to R Ext Ilia w Synth Sub, Perc Endo */
"041C4JF"  =  "1"  /* Bypass R Com Iliac Art to L Ext Ilia w Synth Sub, Perc Endo */
"041C4JG"  =  "1"  /* Bypass R Com Iliac Art to B Ext Ilia w Synth Sub, Perc Endo */
"041C4JH"  =  "1"  /* Bypass R Com Iliac Art to R Femor A w Synth Sub, Perc Endo */
"041C4JJ"  =  "1"  /* Bypass R Com Iliac Art to L Femor A w Synth Sub, Perc Endo */
"041C4JK"  =  "1"  /* Bypass R Com Iliac Art to B Femor A w Synth Sub, Perc Endo */
"041C4JQ"  =  "1"  /* Bypass R Com Iliac Art to Low Ex Art w Synth Sub, Perc Endo */
"041C4JR"  =  "1"  /* Bypass R Com Iliac Art to Low Art w Synth Sub, Perc Endo */
"041C4K0"  =  "1"  /* Bypass R Com Iliac Art to Abd Aorta w Nonaut Sub, Perc Endo */
"041C4K1"  =  "1"  /* Bypass R Com Iliac Art to Celiac Art w Nonaut Sub, Perc Endo */
"041C4K2"  =  "1"  /* Bypass R Com Iliac Art to Mesent Art w Nonaut Sub, Perc Endo */
"041C4K3"  =  "1"  /* Bypass R Com Iliac Art to R Renal A w Nonaut Sub, Perc Endo */
"041C4K4"  =  "1"  /* Bypass R Com Iliac Art to L Renal A w Nonaut Sub, Perc Endo */
"041C4K5"  =  "1"  /* Bypass R Com Iliac Art to B Renal A w Nonaut Sub, Perc Endo */
"041C4K6"  =  "1"  /* Bypass R Com Iliac Art to R Com Ilia w Nonaut Sub, Perc Endo */
"041C4K7"  =  "1"  /* Bypass R Com Iliac Art to L Com Ilia w Nonaut Sub, Perc Endo */
"041C4K8"  =  "1"  /* Bypass R Com Iliac Art to B Com Ilia w Nonaut Sub, Perc Endo */
"041C4K9"  =  "1"  /* Bypass R Com Iliac Art to R Int Ilia w Nonaut Sub, Perc Endo */
"041C4KB"  =  "1"  /* Bypass R Com Iliac Art to L Int Ilia w Nonaut Sub, Perc Endo */
"041C4KC"  =  "1"  /* Bypass R Com Iliac Art to B Int Ilia w Nonaut Sub, Perc Endo */
"041C4KD"  =  "1"  /* Bypass R Com Iliac Art to R Ext Ilia w Nonaut Sub, Perc Endo */
"041C4KF"  =  "1"  /* Bypass R Com Iliac Art to L Ext Ilia w Nonaut Sub, Perc Endo */
"041C4KG"  =  "1"  /* Bypass R Com Iliac Art to B Ext Ilia w Nonaut Sub, Perc Endo */
"041C4KH"  =  "1"  /* Bypass R Com Iliac Art to R Femor A w Nonaut Sub, Perc Endo */
"041C4KJ"  =  "1"  /* Bypass R Com Iliac Art to L Femor A w Nonaut Sub, Perc Endo */
"041C4KK"  =  "1"  /* Bypass R Com Iliac Art to B Femor A w Nonaut Sub, Perc Endo */
"041C4KQ"  =  "1"  /* Bypass R Com Iliac Art to Low Ex Art w Nonaut Sub, Perc Endo */
"041C4KR"  =  "1"  /* Bypass R Com Iliac Art to Low Art w Nonaut Sub, Perc Endo */
"041C4Z0"  =  "1"  /* Bypass R Com Iliac Art to Abd Aorta, Perc Endo Approach */
"041C4Z1"  =  "1"  /* Bypass R Com Iliac Art to Celiac Art, Perc Endo Approach */
"041C4Z2"  =  "1"  /* Bypass R Com Iliac Art to Mesent Art, Perc Endo Approach */
"041C4Z3"  =  "1"  /* Bypass R Com Iliac Art to R Renal A, Perc Endo Approach */
"041C4Z4"  =  "1"  /* Bypass R Com Iliac Art to L Renal A, Perc Endo Approach */
"041C4Z5"  =  "1"  /* Bypass R Com Iliac Art to B Renal A, Perc Endo Approach */
"041C4Z6"  =  "1"  /* Bypass R Com Iliac Art to R Com Ilia, Perc Endo Approach */
"041C4Z7"  =  "1"  /* Bypass R Com Iliac Art to L Com Ilia, Perc Endo Approach */
"041C4Z8"  =  "1"  /* Bypass R Com Iliac Art to B Com Ilia, Perc Endo Approach */
"041C4Z9"  =  "1"  /* Bypass R Com Iliac Art to R Int Ilia, Perc Endo Approach */
"041C4ZB"  =  "1"  /* Bypass R Com Iliac Art to L Int Ilia, Perc Endo Approach */
"041C4ZC"  =  "1"  /* Bypass R Com Iliac Art to B Int Ilia, Perc Endo Approach */
"041C4ZD"  =  "1"  /* Bypass R Com Iliac Art to R Ext Ilia, Perc Endo Approach */
"041C4ZF"  =  "1"  /* Bypass R Com Iliac Art to L Ext Ilia, Perc Endo Approach */
"041C4ZG"  =  "1"  /* Bypass R Com Iliac Art to B Ext Ilia, Perc Endo Approach */
"041C4ZH"  =  "1"  /* Bypass R Com Iliac Art to R Femor A, Perc Endo Approach */
"041C4ZJ"  =  "1"  /* Bypass R Com Iliac Art to L Femor A, Perc Endo Approach */
"041C4ZK"  =  "1"  /* Bypass R Com Iliac Art to B Femor A, Perc Endo Approach */
"041C4ZQ"  =  "1"  /* Bypass R Com Iliac Art to Low Ex Art, Perc Endo Approach */
"041C4ZR"  =  "1"  /* Bypass R Com Iliac Art to Low Art, Perc Endo Approach */
"041D090"  =  "1"  /* Bypass L Com Iliac Art to Abd Aorta w Autol Vn, Open */
"041D091"  =  "1"  /* Bypass L Com Iliac Art to Celiac Art w Autol Vn, Open */
"041D092"  =  "1"  /* Bypass L Com Iliac Art to Mesent Art w Autol Vn, Open */
"041D093"  =  "1"  /* Bypass L Com Iliac Art to R Renal A w Autol Vn, Open */
"041D094"  =  "1"  /* Bypass L Com Iliac Art to L Renal A w Autol Vn, Open */
"041D095"  =  "1"  /* Bypass L Com Iliac Art to B Renal A w Autol Vn, Open */
"041D096"  =  "1"  /* Bypass L Com Iliac Art to R Com Ilia w Autol Vn, Open */
"041D097"  =  "1"  /* Bypass L Com Iliac Art to L Com Ilia w Autol Vn, Open */
"041D098"  =  "1"  /* Bypass L Com Iliac Art to B Com Ilia w Autol Vn, Open */
"041D099"  =  "1"  /* Bypass L Com Iliac Art to R Int Ilia w Autol Vn, Open */
"041D09B"  =  "1"  /* Bypass L Com Iliac Art to L Int Ilia w Autol Vn, Open */
"041D09C"  =  "1"  /* Bypass L Com Iliac Art to B Int Ilia w Autol Vn, Open */
"041D09D"  =  "1"  /* Bypass L Com Iliac Art to R Ext Ilia w Autol Vn, Open */
"041D09F"  =  "1"  /* Bypass L Com Iliac Art to L Ext Ilia w Autol Vn, Open */
"041D09G"  =  "1"  /* Bypass L Com Iliac Art to B Ext Ilia w Autol Vn, Open */
"041D09H"  =  "1"  /* Bypass L Com Iliac Art to R Femor A w Autol Vn, Open */
"041D09J"  =  "1"  /* Bypass L Com Iliac Art to L Femor A w Autol Vn, Open */
"041D09K"  =  "1"  /* Bypass L Com Iliac Art to B Femor A w Autol Vn, Open */
"041D09Q"  =  "1"  /* Bypass L Com Iliac Art to Low Ex Art w Autol Vn, Open */
"041D09R"  =  "1"  /* Bypass L Com Iliac Art to Low Art w Autol Vn, Open */
"041D0A0"  =  "1"  /* Bypass L Com Iliac Art to Abd Aorta w Autol Art, Open */
"041D0A1"  =  "1"  /* Bypass L Com Iliac Art to Celiac Art w Autol Art, Open */
"041D0A2"  =  "1"  /* Bypass L Com Iliac Art to Mesent Art w Autol Art, Open */
"041D0A3"  =  "1"  /* Bypass L Com Iliac Art to R Renal A w Autol Art, Open */
"041D0A4"  =  "1"  /* Bypass L Com Iliac Art to L Renal A w Autol Art, Open */
"041D0A5"  =  "1"  /* Bypass L Com Iliac Art to B Renal A w Autol Art, Open */
"041D0A6"  =  "1"  /* Bypass L Com Iliac Art to R Com Ilia w Autol Art, Open */
"041D0A7"  =  "1"  /* Bypass L Com Iliac Art to L Com Ilia w Autol Art, Open */
"041D0A8"  =  "1"  /* Bypass L Com Iliac Art to B Com Ilia w Autol Art, Open */
"041D0A9"  =  "1"  /* Bypass L Com Iliac Art to R Int Ilia w Autol Art, Open */
"041D0AB"  =  "1"  /* Bypass L Com Iliac Art to L Int Ilia w Autol Art, Open */
"041D0AC"  =  "1"  /* Bypass L Com Iliac Art to B Int Ilia w Autol Art, Open */
"041D0AD"  =  "1"  /* Bypass L Com Iliac Art to R Ext Ilia w Autol Art, Open */
"041D0AF"  =  "1"  /* Bypass L Com Iliac Art to L Ext Ilia w Autol Art, Open */
"041D0AG"  =  "1"  /* Bypass L Com Iliac Art to B Ext Ilia w Autol Art, Open */
"041D0AH"  =  "1"  /* Bypass L Com Iliac Art to R Femor A w Autol Art, Open */
"041D0AJ"  =  "1"  /* Bypass L Com Iliac Art to L Femor A w Autol Art, Open */
"041D0AK"  =  "1"  /* Bypass L Com Iliac Art to B Femor A w Autol Art, Open */
"041D0AQ"  =  "1"  /* Bypass L Com Iliac Art to Low Ex Art w Autol Art, Open */
"041D0AR"  =  "1"  /* Bypass L Com Iliac Art to Low Art w Autol Art, Open */
"041D0J0"  =  "1"  /* Bypass L Com Iliac Art to Abd Aorta w Synth Sub, Open */
"041D0J1"  =  "1"  /* Bypass L Com Iliac Art to Celiac Art w Synth Sub, Open */
"041D0J2"  =  "1"  /* Bypass L Com Iliac Art to Mesent Art w Synth Sub, Open */
"041D0J3"  =  "1"  /* Bypass L Com Iliac Art to R Renal A w Synth Sub, Open */
"041D0J4"  =  "1"  /* Bypass L Com Iliac Art to L Renal A w Synth Sub, Open */
"041D0J5"  =  "1"  /* Bypass L Com Iliac Art to B Renal A w Synth Sub, Open */
"041D0J6"  =  "1"  /* Bypass L Com Iliac Art to R Com Ilia w Synth Sub, Open */
"041D0J7"  =  "1"  /* Bypass L Com Iliac Art to L Com Ilia w Synth Sub, Open */
"041D0J8"  =  "1"  /* Bypass L Com Iliac Art to B Com Ilia w Synth Sub, Open */
"041D0J9"  =  "1"  /* Bypass L Com Iliac Art to R Int Ilia w Synth Sub, Open */
"041D0JB"  =  "1"  /* Bypass L Com Iliac Art to L Int Ilia w Synth Sub, Open */
"041D0JC"  =  "1"  /* Bypass L Com Iliac Art to B Int Ilia w Synth Sub, Open */
"041D0JD"  =  "1"  /* Bypass L Com Iliac Art to R Ext Ilia w Synth Sub, Open */
"041D0JF"  =  "1"  /* Bypass L Com Iliac Art to L Ext Ilia w Synth Sub, Open */
"041D0JG"  =  "1"  /* Bypass L Com Iliac Art to B Ext Ilia w Synth Sub, Open */
"041D0JH"  =  "1"  /* Bypass L Com Iliac Art to R Femor A w Synth Sub, Open */
"041D0JJ"  =  "1"  /* Bypass L Com Iliac Art to L Femor A w Synth Sub, Open */
"041D0JK"  =  "1"  /* Bypass L Com Iliac Art to B Femor A w Synth Sub, Open */
"041D0JQ"  =  "1"  /* Bypass L Com Iliac Art to Low Ex Art w Synth Sub, Open */
"041D0JR"  =  "1"  /* Bypass L Com Iliac Art to Low Art w Synth Sub, Open */
"041D0K0"  =  "1"  /* Bypass L Com Iliac Art to Abd Aorta w Nonaut Sub, Open */
"041D0K1"  =  "1"  /* Bypass L Com Iliac Art to Celiac Art w Nonaut Sub, Open */
"041D0K2"  =  "1"  /* Bypass L Com Iliac Art to Mesent Art w Nonaut Sub, Open */
"041D0K3"  =  "1"  /* Bypass L Com Iliac Art to R Renal A w Nonaut Sub, Open */
"041D0K4"  =  "1"  /* Bypass L Com Iliac Art to L Renal A w Nonaut Sub, Open */
"041D0K5"  =  "1"  /* Bypass L Com Iliac Art to B Renal A w Nonaut Sub, Open */
"041D0K6"  =  "1"  /* Bypass L Com Iliac Art to R Com Ilia w Nonaut Sub, Open */
"041D0K7"  =  "1"  /* Bypass L Com Iliac Art to L Com Ilia w Nonaut Sub, Open */
"041D0K8"  =  "1"  /* Bypass L Com Iliac Art to B Com Ilia w Nonaut Sub, Open */
"041D0K9"  =  "1"  /* Bypass L Com Iliac Art to R Int Ilia w Nonaut Sub, Open */
"041D0KB"  =  "1"  /* Bypass L Com Iliac Art to L Int Ilia w Nonaut Sub, Open */
"041D0KC"  =  "1"  /* Bypass L Com Iliac Art to B Int Ilia w Nonaut Sub, Open */
"041D0KD"  =  "1"  /* Bypass L Com Iliac Art to R Ext Ilia w Nonaut Sub, Open */
"041D0KF"  =  "1"  /* Bypass L Com Iliac Art to L Ext Ilia w Nonaut Sub, Open */
"041D0KG"  =  "1"  /* Bypass L Com Iliac Art to B Ext Ilia w Nonaut Sub, Open */
"041D0KH"  =  "1"  /* Bypass L Com Iliac Art to R Femor A w Nonaut Sub, Open */
"041D0KJ"  =  "1"  /* Bypass L Com Iliac Art to L Femor A w Nonaut Sub, Open */
"041D0KK"  =  "1"  /* Bypass L Com Iliac Art to B Femor A w Nonaut Sub, Open */
"041D0KQ"  =  "1"  /* Bypass L Com Iliac Art to Low Ex Art w Nonaut Sub, Open */
"041D0KR"  =  "1"  /* Bypass L Com Iliac Art to Low Art w Nonaut Sub, Open */
"041D0Z0"  =  "1"  /* Bypass Left Common Iliac Artery to Abd Aorta, Open Approach */
"041D0Z1"  =  "1"  /* Bypass Left Common Iliac Artery to Celiac Art, Open Approach */
"041D0Z2"  =  "1"  /* Bypass Left Common Iliac Artery to Mesent Art, Open Approach */
"041D0Z3"  =  "1"  /* Bypass Left Common Iliac Artery to R Renal A, Open Approach */
"041D0Z4"  =  "1"  /* Bypass Left Common Iliac Artery to L Renal A, Open Approach */
"041D0Z5"  =  "1"  /* Bypass Left Common Iliac Artery to B Renal A, Open Approach */
"041D0Z6"  =  "1"  /* Bypass Left Common Iliac Artery to R Com Ilia, Open Approach */
"041D0Z7"  =  "1"  /* Bypass Left Common Iliac Artery to L Com Ilia, Open Approach */
"041D0Z8"  =  "1"  /* Bypass Left Common Iliac Artery to B Com Ilia, Open Approach */
"041D0Z9"  =  "1"  /* Bypass Left Common Iliac Artery to R Int Ilia, Open Approach */
"041D0ZB"  =  "1"  /* Bypass Left Common Iliac Artery to L Int Ilia, Open Approach */
"041D0ZC"  =  "1"  /* Bypass Left Common Iliac Artery to B Int Ilia, Open Approach */
"041D0ZD"  =  "1"  /* Bypass Left Common Iliac Artery to R Ext Ilia, Open Approach */
"041D0ZF"  =  "1"  /* Bypass Left Common Iliac Artery to L Ext Ilia, Open Approach */
"041D0ZG"  =  "1"  /* Bypass Left Common Iliac Artery to B Ext Ilia, Open Approach */
"041D0ZH"  =  "1"  /* Bypass Left Common Iliac Artery to R Femor A, Open Approach */
"041D0ZJ"  =  "1"  /* Bypass Left Common Iliac Artery to L Femor A, Open Approach */
"041D0ZK"  =  "1"  /* Bypass Left Common Iliac Artery to B Femor A, Open Approach */
"041D0ZQ"  =  "1"  /* Bypass Left Common Iliac Artery to Low Ex Art, Open Approach */
"041D0ZR"  =  "1"  /* Bypass Left Common Iliac Artery to Low Art, Open Approach */
"041D490"  =  "1"  /* Bypass L Com Iliac Art to Abd Aorta w Autol Vn, Perc Endo */
"041D491"  =  "1"  /* Bypass L Com Iliac Art to Celiac Art w Autol Vn, Perc Endo */
"041D492"  =  "1"  /* Bypass L Com Iliac Art to Mesent Art w Autol Vn, Perc Endo */
"041D493"  =  "1"  /* Bypass L Com Iliac Art to R Renal A w Autol Vn, Perc Endo */
"041D494"  =  "1"  /* Bypass L Com Iliac Art to L Renal A w Autol Vn, Perc Endo */
"041D495"  =  "1"  /* Bypass L Com Iliac Art to B Renal A w Autol Vn, Perc Endo */
"041D496"  =  "1"  /* Bypass L Com Iliac Art to R Com Ilia w Autol Vn, Perc Endo */
"041D497"  =  "1"  /* Bypass L Com Iliac Art to L Com Ilia w Autol Vn, Perc Endo */
"041D498"  =  "1"  /* Bypass L Com Iliac Art to B Com Ilia w Autol Vn, Perc Endo */
"041D499"  =  "1"  /* Bypass L Com Iliac Art to R Int Ilia w Autol Vn, Perc Endo */
"041D49B"  =  "1"  /* Bypass L Com Iliac Art to L Int Ilia w Autol Vn, Perc Endo */
"041D49C"  =  "1"  /* Bypass L Com Iliac Art to B Int Ilia w Autol Vn, Perc Endo */
"041D49D"  =  "1"  /* Bypass L Com Iliac Art to R Ext Ilia w Autol Vn, Perc Endo */
"041D49F"  =  "1"  /* Bypass L Com Iliac Art to L Ext Ilia w Autol Vn, Perc Endo */
"041D49G"  =  "1"  /* Bypass L Com Iliac Art to B Ext Ilia w Autol Vn, Perc Endo */
"041D49H"  =  "1"  /* Bypass L Com Iliac Art to R Femor A w Autol Vn, Perc Endo */
"041D49J"  =  "1"  /* Bypass L Com Iliac Art to L Femor A w Autol Vn, Perc Endo */
"041D49K"  =  "1"  /* Bypass L Com Iliac Art to B Femor A w Autol Vn, Perc Endo */
"041D49Q"  =  "1"  /* Bypass L Com Iliac Art to Low Ex Art w Autol Vn, Perc Endo */
"041D49R"  =  "1"  /* Bypass L Com Iliac Art to Low Art w Autol Vn, Perc Endo */
"041D4A0"  =  "1"  /* Bypass L Com Iliac Art to Abd Aorta w Autol Art, Perc Endo */
"041D4A1"  =  "1"  /* Bypass L Com Iliac Art to Celiac Art w Autol Art, Perc Endo */
"041D4A2"  =  "1"  /* Bypass L Com Iliac Art to Mesent Art w Autol Art, Perc Endo */
"041D4A3"  =  "1"  /* Bypass L Com Iliac Art to R Renal A w Autol Art, Perc Endo */
"041D4A4"  =  "1"  /* Bypass L Com Iliac Art to L Renal A w Autol Art, Perc Endo */
"041D4A5"  =  "1"  /* Bypass L Com Iliac Art to B Renal A w Autol Art, Perc Endo */
"041D4A6"  =  "1"  /* Bypass L Com Iliac Art to R Com Ilia w Autol Art, Perc Endo */
"041D4A7"  =  "1"  /* Bypass L Com Iliac Art to L Com Ilia w Autol Art, Perc Endo */
"041D4A8"  =  "1"  /* Bypass L Com Iliac Art to B Com Ilia w Autol Art, Perc Endo */
"041D4A9"  =  "1"  /* Bypass L Com Iliac Art to R Int Ilia w Autol Art, Perc Endo */
"041D4AB"  =  "1"  /* Bypass L Com Iliac Art to L Int Ilia w Autol Art, Perc Endo */
"041D4AC"  =  "1"  /* Bypass L Com Iliac Art to B Int Ilia w Autol Art, Perc Endo */
"041D4AD"  =  "1"  /* Bypass L Com Iliac Art to R Ext Ilia w Autol Art, Perc Endo */
"041D4AF"  =  "1"  /* Bypass L Com Iliac Art to L Ext Ilia w Autol Art, Perc Endo */
"041D4AG"  =  "1"  /* Bypass L Com Iliac Art to B Ext Ilia w Autol Art, Perc Endo */
"041D4AH"  =  "1"  /* Bypass L Com Iliac Art to R Femor A w Autol Art, Perc Endo */
"041D4AJ"  =  "1"  /* Bypass L Com Iliac Art to L Femor A w Autol Art, Perc Endo */
"041D4AK"  =  "1"  /* Bypass L Com Iliac Art to B Femor A w Autol Art, Perc Endo */
"041D4AQ"  =  "1"  /* Bypass L Com Iliac Art to Low Ex Art w Autol Art, Perc Endo */
"041D4AR"  =  "1"  /* Bypass L Com Iliac Art to Low Art w Autol Art, Perc Endo */
"041D4J0"  =  "1"  /* Bypass L Com Iliac Art to Abd Aorta w Synth Sub, Perc Endo */
"041D4J1"  =  "1"  /* Bypass L Com Iliac Art to Celiac Art w Synth Sub, Perc Endo */
"041D4J2"  =  "1"  /* Bypass L Com Iliac Art to Mesent Art w Synth Sub, Perc Endo */
"041D4J3"  =  "1"  /* Bypass L Com Iliac Art to R Renal A w Synth Sub, Perc Endo */
"041D4J4"  =  "1"  /* Bypass L Com Iliac Art to L Renal A w Synth Sub, Perc Endo */
"041D4J5"  =  "1"  /* Bypass L Com Iliac Art to B Renal A w Synth Sub, Perc Endo */
"041D4J6"  =  "1"  /* Bypass L Com Iliac Art to R Com Ilia w Synth Sub, Perc Endo */
"041D4J7"  =  "1"  /* Bypass L Com Iliac Art to L Com Ilia w Synth Sub, Perc Endo */
"041D4J8"  =  "1"  /* Bypass L Com Iliac Art to B Com Ilia w Synth Sub, Perc Endo */
"041D4J9"  =  "1"  /* Bypass L Com Iliac Art to R Int Ilia w Synth Sub, Perc Endo */
"041D4JB"  =  "1"  /* Bypass L Com Iliac Art to L Int Ilia w Synth Sub, Perc Endo */
"041D4JC"  =  "1"  /* Bypass L Com Iliac Art to B Int Ilia w Synth Sub, Perc Endo */
"041D4JD"  =  "1"  /* Bypass L Com Iliac Art to R Ext Ilia w Synth Sub, Perc Endo */
"041D4JF"  =  "1"  /* Bypass L Com Iliac Art to L Ext Ilia w Synth Sub, Perc Endo */
"041D4JG"  =  "1"  /* Bypass L Com Iliac Art to B Ext Ilia w Synth Sub, Perc Endo */
"041D4JH"  =  "1"  /* Bypass L Com Iliac Art to R Femor A w Synth Sub, Perc Endo */
"041D4JJ"  =  "1"  /* Bypass L Com Iliac Art to L Femor A w Synth Sub, Perc Endo */
"041D4JK"  =  "1"  /* Bypass L Com Iliac Art to B Femor A w Synth Sub, Perc Endo */
"041D4JQ"  =  "1"  /* Bypass L Com Iliac Art to Low Ex Art w Synth Sub, Perc Endo */
"041D4JR"  =  "1"  /* Bypass L Com Iliac Art to Low Art w Synth Sub, Perc Endo */
"041D4K0"  =  "1"  /* Bypass L Com Iliac Art to Abd Aorta w Nonaut Sub, Perc Endo */
"041D4K1"  =  "1"  /* Bypass L Com Iliac Art to Celiac Art w Nonaut Sub, Perc Endo */
"041D4K2"  =  "1"  /* Bypass L Com Iliac Art to Mesent Art w Nonaut Sub, Perc Endo */
"041D4K3"  =  "1"  /* Bypass L Com Iliac Art to R Renal A w Nonaut Sub, Perc Endo */
"041D4K4"  =  "1"  /* Bypass L Com Iliac Art to L Renal A w Nonaut Sub, Perc Endo */
"041D4K5"  =  "1"  /* Bypass L Com Iliac Art to B Renal A w Nonaut Sub, Perc Endo */
"041D4K6"  =  "1"  /* Bypass L Com Iliac Art to R Com Ilia w Nonaut Sub, Perc Endo */
"041D4K7"  =  "1"  /* Bypass L Com Iliac Art to L Com Ilia w Nonaut Sub, Perc Endo */
"041D4K8"  =  "1"  /* Bypass L Com Iliac Art to B Com Ilia w Nonaut Sub, Perc Endo */
"041D4K9"  =  "1"  /* Bypass L Com Iliac Art to R Int Ilia w Nonaut Sub, Perc Endo */
"041D4KB"  =  "1"  /* Bypass L Com Iliac Art to L Int Ilia w Nonaut Sub, Perc Endo */
"041D4KC"  =  "1"  /* Bypass L Com Iliac Art to B Int Ilia w Nonaut Sub, Perc Endo */
"041D4KD"  =  "1"  /* Bypass L Com Iliac Art to R Ext Ilia w Nonaut Sub, Perc Endo */
"041D4KF"  =  "1"  /* Bypass L Com Iliac Art to L Ext Ilia w Nonaut Sub, Perc Endo */
"041D4KG"  =  "1"  /* Bypass L Com Iliac Art to B Ext Ilia w Nonaut Sub, Perc Endo */
"041D4KH"  =  "1"  /* Bypass L Com Iliac Art to R Femor A w Nonaut Sub, Perc Endo */
"041D4KJ"  =  "1"  /* Bypass L Com Iliac Art to L Femor A w Nonaut Sub, Perc Endo */
"041D4KK"  =  "1"  /* Bypass L Com Iliac Art to B Femor A w Nonaut Sub, Perc Endo */
"041D4KQ"  =  "1"  /* Bypass L Com Iliac Art to Low Ex Art w Nonaut Sub, Perc Endo */
"041D4KR"  =  "1"  /* Bypass L Com Iliac Art to Low Art w Nonaut Sub, Perc Endo */
"041D4Z0"  =  "1"  /* Bypass L Com Iliac Art to Abd Aorta, Perc Endo Approach */
"041D4Z1"  =  "1"  /* Bypass L Com Iliac Art to Celiac Art, Perc Endo Approach */
"041D4Z2"  =  "1"  /* Bypass L Com Iliac Art to Mesent Art, Perc Endo Approach */
"041D4Z3"  =  "1"  /* Bypass L Com Iliac Art to R Renal A, Perc Endo Approach */
"041D4Z4"  =  "1"  /* Bypass L Com Iliac Art to L Renal A, Perc Endo Approach */
"041D4Z5"  =  "1"  /* Bypass L Com Iliac Art to B Renal A, Perc Endo Approach */
"041D4Z6"  =  "1"  /* Bypass L Com Iliac Art to R Com Ilia, Perc Endo Approach */
"041D4Z7"  =  "1"  /* Bypass L Com Iliac Art to L Com Ilia, Perc Endo Approach */
"041D4Z8"  =  "1"  /* Bypass L Com Iliac Art to B Com Ilia, Perc Endo Approach */
"041D4Z9"  =  "1"  /* Bypass L Com Iliac Art to R Int Ilia, Perc Endo Approach */
"041D4ZB"  =  "1"  /* Bypass L Com Iliac Art to L Int Ilia, Perc Endo Approach */
"041D4ZC"  =  "1"  /* Bypass L Com Iliac Art to B Int Ilia, Perc Endo Approach */
"041D4ZD"  =  "1"  /* Bypass L Com Iliac Art to R Ext Ilia, Perc Endo Approach */
"041D4ZF"  =  "1"  /* Bypass L Com Iliac Art to L Ext Ilia, Perc Endo Approach */
"041D4ZG"  =  "1"  /* Bypass L Com Iliac Art to B Ext Ilia, Perc Endo Approach */
"041D4ZH"  =  "1"  /* Bypass L Com Iliac Art to R Femor A, Perc Endo Approach */
"041D4ZJ"  =  "1"  /* Bypass L Com Iliac Art to L Femor A, Perc Endo Approach */
"041D4ZK"  =  "1"  /* Bypass L Com Iliac Art to B Femor A, Perc Endo Approach */
"041D4ZQ"  =  "1"  /* Bypass L Com Iliac Art to Low Ex Art, Perc Endo Approach */
"041D4ZR"  =  "1"  /* Bypass L Com Iliac Art to Low Art, Perc Endo Approach */
"041E099"  =  "1"  /* Bypass R Int Iliac Art to R Int Ilia w Autol Vn, Open */
"041E09B"  =  "1"  /* Bypass R Int Iliac Art to L Int Ilia w Autol Vn, Open */
"041E09C"  =  "1"  /* Bypass R Int Iliac Art to B Int Ilia w Autol Vn, Open */
"041E09D"  =  "1"  /* Bypass R Int Iliac Art to R Ext Ilia w Autol Vn, Open */
"041E09F"  =  "1"  /* Bypass R Int Iliac Art to L Ext Ilia w Autol Vn, Open */
"041E09G"  =  "1"  /* Bypass R Int Iliac Art to B Ext Ilia w Autol Vn, Open */
"041E09H"  =  "1"  /* Bypass R Int Iliac Art to R Femor A w Autol Vn, Open */
"041E09J"  =  "1"  /* Bypass R Int Iliac Art to L Femor A w Autol Vn, Open */
"041E09K"  =  "1"  /* Bypass R Int Iliac Art to B Femor A w Autol Vn, Open */
"041E09P"  =  "1"  /* Bypass R Int Iliac Art to Foot Art w Autol Vn, Open */
"041E09Q"  =  "1"  /* Bypass R Int Iliac Art to Low Ex Art w Autol Vn, Open */
"041E0A9"  =  "1"  /* Bypass R Int Iliac Art to R Int Ilia w Autol Art, Open */
"041E0AB"  =  "1"  /* Bypass R Int Iliac Art to L Int Ilia w Autol Art, Open */
"041E0AC"  =  "1"  /* Bypass R Int Iliac Art to B Int Ilia w Autol Art, Open */
"041E0AD"  =  "1"  /* Bypass R Int Iliac Art to R Ext Ilia w Autol Art, Open */
"041E0AF"  =  "1"  /* Bypass R Int Iliac Art to L Ext Ilia w Autol Art, Open */
"041E0AG"  =  "1"  /* Bypass R Int Iliac Art to B Ext Ilia w Autol Art, Open */
"041E0AH"  =  "1"  /* Bypass R Int Iliac Art to R Femor A w Autol Art, Open */
"041E0AJ"  =  "1"  /* Bypass R Int Iliac Art to L Femor A w Autol Art, Open */
"041E0AK"  =  "1"  /* Bypass R Int Iliac Art to B Femor A w Autol Art, Open */
"041E0AP"  =  "1"  /* Bypass R Int Iliac Art to Foot Art w Autol Art, Open */
"041E0AQ"  =  "1"  /* Bypass R Int Iliac Art to Low Ex Art w Autol Art, Open */
"041E0J9"  =  "1"  /* Bypass R Int Iliac Art to R Int Ilia w Synth Sub, Open */
"041E0JB"  =  "1"  /* Bypass R Int Iliac Art to L Int Ilia w Synth Sub, Open */
"041E0JC"  =  "1"  /* Bypass R Int Iliac Art to B Int Ilia w Synth Sub, Open */
"041E0JD"  =  "1"  /* Bypass R Int Iliac Art to R Ext Ilia w Synth Sub, Open */
"041E0JF"  =  "1"  /* Bypass R Int Iliac Art to L Ext Ilia w Synth Sub, Open */
"041E0JG"  =  "1"  /* Bypass R Int Iliac Art to B Ext Ilia w Synth Sub, Open */
"041E0JH"  =  "1"  /* Bypass R Int Iliac Art to R Femor A w Synth Sub, Open */
"041E0JJ"  =  "1"  /* Bypass R Int Iliac Art to L Femor A w Synth Sub, Open */
"041E0JK"  =  "1"  /* Bypass R Int Iliac Art to B Femor A w Synth Sub, Open */
"041E0JP"  =  "1"  /* Bypass R Int Iliac Art to Foot Art w Synth Sub, Open */
"041E0JQ"  =  "1"  /* Bypass R Int Iliac Art to Low Ex Art w Synth Sub, Open */
"041E0K9"  =  "1"  /* Bypass R Int Iliac Art to R Int Ilia w Nonaut Sub, Open */
"041E0KB"  =  "1"  /* Bypass R Int Iliac Art to L Int Ilia w Nonaut Sub, Open */
"041E0KC"  =  "1"  /* Bypass R Int Iliac Art to B Int Ilia w Nonaut Sub, Open */
"041E0KD"  =  "1"  /* Bypass R Int Iliac Art to R Ext Ilia w Nonaut Sub, Open */
"041E0KF"  =  "1"  /* Bypass R Int Iliac Art to L Ext Ilia w Nonaut Sub, Open */
"041E0KG"  =  "1"  /* Bypass R Int Iliac Art to B Ext Ilia w Nonaut Sub, Open */
"041E0KH"  =  "1"  /* Bypass R Int Iliac Art to R Femor A w Nonaut Sub, Open */
"041E0KJ"  =  "1"  /* Bypass R Int Iliac Art to L Femor A w Nonaut Sub, Open */
"041E0KK"  =  "1"  /* Bypass R Int Iliac Art to B Femor A w Nonaut Sub, Open */
"041E0KP"  =  "1"  /* Bypass R Int Iliac Art to Foot Art w Nonaut Sub, Open */
"041E0KQ"  =  "1"  /* Bypass R Int Iliac Art to Low Ex Art w Nonaut Sub, Open */
"041E0Z9"  =  "1"  /* Bypass R Int Iliac Art to R Int Ilia, Open Approach */
"041E0ZB"  =  "1"  /* Bypass R Int Iliac Art to L Int Ilia, Open Approach */
"041E0ZC"  =  "1"  /* Bypass R Int Iliac Art to B Int Ilia, Open Approach */
"041E0ZD"  =  "1"  /* Bypass R Int Iliac Art to R Ext Ilia, Open Approach */
"041E0ZF"  =  "1"  /* Bypass R Int Iliac Art to L Ext Ilia, Open Approach */
"041E0ZG"  =  "1"  /* Bypass R Int Iliac Art to B Ext Ilia, Open Approach */
"041E0ZH"  =  "1"  /* Bypass R Int Iliac Art to R Femor A, Open Approach */
"041E0ZJ"  =  "1"  /* Bypass R Int Iliac Art to L Femor A, Open Approach */
"041E0ZK"  =  "1"  /* Bypass R Int Iliac Art to B Femor A, Open Approach */
"041E0ZP"  =  "1"  /* Bypass R Int Iliac Art to Foot Art, Open Approach */
"041E0ZQ"  =  "1"  /* Bypass R Int Iliac Art to Low Ex Art, Open Approach */
"041E499"  =  "1"  /* Bypass R Int Iliac Art to R Int Ilia w Autol Vn, Perc Endo */
"041E49B"  =  "1"  /* Bypass R Int Iliac Art to L Int Ilia w Autol Vn, Perc Endo */
"041E49C"  =  "1"  /* Bypass R Int Iliac Art to B Int Ilia w Autol Vn, Perc Endo */
"041E49D"  =  "1"  /* Bypass R Int Iliac Art to R Ext Ilia w Autol Vn, Perc Endo */
"041E49F"  =  "1"  /* Bypass R Int Iliac Art to L Ext Ilia w Autol Vn, Perc Endo */
"041E49G"  =  "1"  /* Bypass R Int Iliac Art to B Ext Ilia w Autol Vn, Perc Endo */
"041E49H"  =  "1"  /* Bypass R Int Iliac Art to R Femor A w Autol Vn, Perc Endo */
"041E49J"  =  "1"  /* Bypass R Int Iliac Art to L Femor A w Autol Vn, Perc Endo */
"041E49K"  =  "1"  /* Bypass R Int Iliac Art to B Femor A w Autol Vn, Perc Endo */
"041E49P"  =  "1"  /* Bypass R Int Iliac Art to Foot Art w Autol Vn, Perc Endo */
"041E49Q"  =  "1"  /* Bypass R Int Iliac Art to Low Ex Art w Autol Vn, Perc Endo */
"041E4A9"  =  "1"  /* Bypass R Int Iliac Art to R Int Ilia w Autol Art, Perc Endo */
"041E4AB"  =  "1"  /* Bypass R Int Iliac Art to L Int Ilia w Autol Art, Perc Endo */
"041E4AC"  =  "1"  /* Bypass R Int Iliac Art to B Int Ilia w Autol Art, Perc Endo */
"041E4AD"  =  "1"  /* Bypass R Int Iliac Art to R Ext Ilia w Autol Art, Perc Endo */
"041E4AF"  =  "1"  /* Bypass R Int Iliac Art to L Ext Ilia w Autol Art, Perc Endo */
"041E4AG"  =  "1"  /* Bypass R Int Iliac Art to B Ext Ilia w Autol Art, Perc Endo */
"041E4AH"  =  "1"  /* Bypass R Int Iliac Art to R Femor A w Autol Art, Perc Endo */
"041E4AJ"  =  "1"  /* Bypass R Int Iliac Art to L Femor A w Autol Art, Perc Endo */
"041E4AK"  =  "1"  /* Bypass R Int Iliac Art to B Femor A w Autol Art, Perc Endo */
"041E4AP"  =  "1"  /* Bypass R Int Iliac Art to Foot Art w Autol Art, Perc Endo */
"041E4AQ"  =  "1"  /* Bypass R Int Iliac Art to Low Ex Art w Autol Art, Perc Endo */
"041E4J9"  =  "1"  /* Bypass R Int Iliac Art to R Int Ilia w Synth Sub, Perc Endo */
"041E4JB"  =  "1"  /* Bypass R Int Iliac Art to L Int Ilia w Synth Sub, Perc Endo */
"041E4JC"  =  "1"  /* Bypass R Int Iliac Art to B Int Ilia w Synth Sub, Perc Endo */
"041E4JD"  =  "1"  /* Bypass R Int Iliac Art to R Ext Ilia w Synth Sub, Perc Endo */
"041E4JF"  =  "1"  /* Bypass R Int Iliac Art to L Ext Ilia w Synth Sub, Perc Endo */
"041E4JG"  =  "1"  /* Bypass R Int Iliac Art to B Ext Ilia w Synth Sub, Perc Endo */
"041E4JH"  =  "1"  /* Bypass R Int Iliac Art to R Femor A w Synth Sub, Perc Endo */
"041E4JJ"  =  "1"  /* Bypass R Int Iliac Art to L Femor A w Synth Sub, Perc Endo */
"041E4JK"  =  "1"  /* Bypass R Int Iliac Art to B Femor A w Synth Sub, Perc Endo */
"041E4JP"  =  "1"  /* Bypass R Int Iliac Art to Foot Art w Synth Sub, Perc Endo */
"041E4JQ"  =  "1"  /* Bypass R Int Iliac Art to Low Ex Art w Synth Sub, Perc Endo */
"041E4K9"  =  "1"  /* Bypass R Int Iliac Art to R Int Ilia w Nonaut Sub, Perc Endo */
"041E4KB"  =  "1"  /* Bypass R Int Iliac Art to L Int Ilia w Nonaut Sub, Perc Endo */
"041E4KC"  =  "1"  /* Bypass R Int Iliac Art to B Int Ilia w Nonaut Sub, Perc Endo */
"041E4KD"  =  "1"  /* Bypass R Int Iliac Art to R Ext Ilia w Nonaut Sub, Perc Endo */
"041E4KF"  =  "1"  /* Bypass R Int Iliac Art to L Ext Ilia w Nonaut Sub, Perc Endo */
"041E4KG"  =  "1"  /* Bypass R Int Iliac Art to B Ext Ilia w Nonaut Sub, Perc Endo */
"041E4KH"  =  "1"  /* Bypass R Int Iliac Art to R Femor A w Nonaut Sub, Perc Endo */
"041E4KJ"  =  "1"  /* Bypass R Int Iliac Art to L Femor A w Nonaut Sub, Perc Endo */
"041E4KK"  =  "1"  /* Bypass R Int Iliac Art to B Femor A w Nonaut Sub, Perc Endo */
"041E4KP"  =  "1"  /* Bypass R Int Iliac Art to Foot Art w Nonaut Sub, Perc Endo */
"041E4KQ"  =  "1"  /* Bypass R Int Iliac Art to Low Ex Art w Nonaut Sub, Perc Endo */
"041E4Z9"  =  "1"  /* Bypass R Int Iliac Art to R Int Ilia, Perc Endo Approach */
"041E4ZB"  =  "1"  /* Bypass R Int Iliac Art to L Int Ilia, Perc Endo Approach */
"041E4ZC"  =  "1"  /* Bypass R Int Iliac Art to B Int Ilia, Perc Endo Approach */
"041E4ZD"  =  "1"  /* Bypass R Int Iliac Art to R Ext Ilia, Perc Endo Approach */
"041E4ZF"  =  "1"  /* Bypass R Int Iliac Art to L Ext Ilia, Perc Endo Approach */
"041E4ZG"  =  "1"  /* Bypass R Int Iliac Art to B Ext Ilia, Perc Endo Approach */
"041E4ZH"  =  "1"  /* Bypass R Int Iliac Art to R Femor A, Perc Endo Approach */
"041E4ZJ"  =  "1"  /* Bypass R Int Iliac Art to L Femor A, Perc Endo Approach */
"041E4ZK"  =  "1"  /* Bypass R Int Iliac Art to B Femor A, Perc Endo Approach */
"041E4ZP"  =  "1"  /* Bypass R Int Iliac Art to Foot Art, Perc Endo Approach */
"041E4ZQ"  =  "1"  /* Bypass R Int Iliac Art to Low Ex Art, Perc Endo Approach */
"041F099"  =  "1"  /* Bypass L Int Iliac Art to R Int Ilia w Autol Vn, Open */
"041F09B"  =  "1"  /* Bypass L Int Iliac Art to L Int Ilia w Autol Vn, Open */
"041F09C"  =  "1"  /* Bypass L Int Iliac Art to B Int Ilia w Autol Vn, Open */
"041F09D"  =  "1"  /* Bypass L Int Iliac Art to R Ext Ilia w Autol Vn, Open */
"041F09F"  =  "1"  /* Bypass L Int Iliac Art to L Ext Ilia w Autol Vn, Open */
"041F09G"  =  "1"  /* Bypass L Int Iliac Art to B Ext Ilia w Autol Vn, Open */
"041F09H"  =  "1"  /* Bypass L Int Iliac Art to R Femor A w Autol Vn, Open */
"041F09J"  =  "1"  /* Bypass L Int Iliac Art to L Femor A w Autol Vn, Open */
"041F09K"  =  "1"  /* Bypass L Int Iliac Art to B Femor A w Autol Vn, Open */
"041F09P"  =  "1"  /* Bypass L Int Iliac Art to Foot Art w Autol Vn, Open */
"041F09Q"  =  "1"  /* Bypass L Int Iliac Art to Low Ex Art w Autol Vn, Open */
"041F0A9"  =  "1"  /* Bypass L Int Iliac Art to R Int Ilia w Autol Art, Open */
"041F0AB"  =  "1"  /* Bypass L Int Iliac Art to L Int Ilia w Autol Art, Open */
"041F0AC"  =  "1"  /* Bypass L Int Iliac Art to B Int Ilia w Autol Art, Open */
"041F0AD"  =  "1"  /* Bypass L Int Iliac Art to R Ext Ilia w Autol Art, Open */
"041F0AF"  =  "1"  /* Bypass L Int Iliac Art to L Ext Ilia w Autol Art, Open */
"041F0AG"  =  "1"  /* Bypass L Int Iliac Art to B Ext Ilia w Autol Art, Open */
"041F0AH"  =  "1"  /* Bypass L Int Iliac Art to R Femor A w Autol Art, Open */
"041F0AJ"  =  "1"  /* Bypass L Int Iliac Art to L Femor A w Autol Art, Open */
"041F0AK"  =  "1"  /* Bypass L Int Iliac Art to B Femor A w Autol Art, Open */
"041F0AP"  =  "1"  /* Bypass L Int Iliac Art to Foot Art w Autol Art, Open */
"041F0AQ"  =  "1"  /* Bypass L Int Iliac Art to Low Ex Art w Autol Art, Open */
"041F0J9"  =  "1"  /* Bypass L Int Iliac Art to R Int Ilia w Synth Sub, Open */
"041F0JB"  =  "1"  /* Bypass L Int Iliac Art to L Int Ilia w Synth Sub, Open */
"041F0JC"  =  "1"  /* Bypass L Int Iliac Art to B Int Ilia w Synth Sub, Open */
"041F0JD"  =  "1"  /* Bypass L Int Iliac Art to R Ext Ilia w Synth Sub, Open */
"041F0JF"  =  "1"  /* Bypass L Int Iliac Art to L Ext Ilia w Synth Sub, Open */
"041F0JG"  =  "1"  /* Bypass L Int Iliac Art to B Ext Ilia w Synth Sub, Open */
"041F0JH"  =  "1"  /* Bypass L Int Iliac Art to R Femor A w Synth Sub, Open */
"041F0JJ"  =  "1"  /* Bypass L Int Iliac Art to L Femor A w Synth Sub, Open */
"041F0JK"  =  "1"  /* Bypass L Int Iliac Art to B Femor A w Synth Sub, Open */
"041F0JP"  =  "1"  /* Bypass L Int Iliac Art to Foot Art w Synth Sub, Open */
"041F0JQ"  =  "1"  /* Bypass L Int Iliac Art to Low Ex Art w Synth Sub, Open */
"041F0K9"  =  "1"  /* Bypass L Int Iliac Art to R Int Ilia w Nonaut Sub, Open */
"041F0KB"  =  "1"  /* Bypass L Int Iliac Art to L Int Ilia w Nonaut Sub, Open */
"041F0KC"  =  "1"  /* Bypass L Int Iliac Art to B Int Ilia w Nonaut Sub, Open */
"041F0KD"  =  "1"  /* Bypass L Int Iliac Art to R Ext Ilia w Nonaut Sub, Open */
"041F0KF"  =  "1"  /* Bypass L Int Iliac Art to L Ext Ilia w Nonaut Sub, Open */
"041F0KG"  =  "1"  /* Bypass L Int Iliac Art to B Ext Ilia w Nonaut Sub, Open */
"041F0KH"  =  "1"  /* Bypass L Int Iliac Art to R Femor A w Nonaut Sub, Open */
"041F0KJ"  =  "1"  /* Bypass L Int Iliac Art to L Femor A w Nonaut Sub, Open */
"041F0KK"  =  "1"  /* Bypass L Int Iliac Art to B Femor A w Nonaut Sub, Open */
"041F0KP"  =  "1"  /* Bypass L Int Iliac Art to Foot Art w Nonaut Sub, Open */
"041F0KQ"  =  "1"  /* Bypass L Int Iliac Art to Low Ex Art w Nonaut Sub, Open */
"041F0Z9"  =  "1"  /* Bypass L Int Iliac Art to R Int Ilia, Open Approach */
"041F0ZB"  =  "1"  /* Bypass L Int Iliac Art to L Int Ilia, Open Approach */
"041F0ZC"  =  "1"  /* Bypass L Int Iliac Art to B Int Ilia, Open Approach */
"041F0ZD"  =  "1"  /* Bypass L Int Iliac Art to R Ext Ilia, Open Approach */
"041F0ZF"  =  "1"  /* Bypass L Int Iliac Art to L Ext Ilia, Open Approach */
"041F0ZG"  =  "1"  /* Bypass L Int Iliac Art to B Ext Ilia, Open Approach */
"041F0ZH"  =  "1"  /* Bypass L Int Iliac Art to R Femor A, Open Approach */
"041F0ZJ"  =  "1"  /* Bypass L Int Iliac Art to L Femor A, Open Approach */
"041F0ZK"  =  "1"  /* Bypass L Int Iliac Art to B Femor A, Open Approach */
"041F0ZP"  =  "1"  /* Bypass Left Internal Iliac Artery to Foot Art, Open Approach */
"041F0ZQ"  =  "1"  /* Bypass L Int Iliac Art to Low Ex Art, Open Approach */
"041F499"  =  "1"  /* Bypass L Int Iliac Art to R Int Ilia w Autol Vn, Perc Endo */
"041F49B"  =  "1"  /* Bypass L Int Iliac Art to L Int Ilia w Autol Vn, Perc Endo */
"041F49C"  =  "1"  /* Bypass L Int Iliac Art to B Int Ilia w Autol Vn, Perc Endo */
"041F49D"  =  "1"  /* Bypass L Int Iliac Art to R Ext Ilia w Autol Vn, Perc Endo */
"041F49F"  =  "1"  /* Bypass L Int Iliac Art to L Ext Ilia w Autol Vn, Perc Endo */
"041F49G"  =  "1"  /* Bypass L Int Iliac Art to B Ext Ilia w Autol Vn, Perc Endo */
"041F49H"  =  "1"  /* Bypass L Int Iliac Art to R Femor A w Autol Vn, Perc Endo */
"041F49J"  =  "1"  /* Bypass L Int Iliac Art to L Femor A w Autol Vn, Perc Endo */
"041F49K"  =  "1"  /* Bypass L Int Iliac Art to B Femor A w Autol Vn, Perc Endo */
"041F49P"  =  "1"  /* Bypass L Int Iliac Art to Foot Art w Autol Vn, Perc Endo */
"041F49Q"  =  "1"  /* Bypass L Int Iliac Art to Low Ex Art w Autol Vn, Perc Endo */
"041F4A9"  =  "1"  /* Bypass L Int Iliac Art to R Int Ilia w Autol Art, Perc Endo */
"041F4AB"  =  "1"  /* Bypass L Int Iliac Art to L Int Ilia w Autol Art, Perc Endo */
"041F4AC"  =  "1"  /* Bypass L Int Iliac Art to B Int Ilia w Autol Art, Perc Endo */
"041F4AD"  =  "1"  /* Bypass L Int Iliac Art to R Ext Ilia w Autol Art, Perc Endo */
"041F4AF"  =  "1"  /* Bypass L Int Iliac Art to L Ext Ilia w Autol Art, Perc Endo */
"041F4AG"  =  "1"  /* Bypass L Int Iliac Art to B Ext Ilia w Autol Art, Perc Endo */
"041F4AH"  =  "1"  /* Bypass L Int Iliac Art to R Femor A w Autol Art, Perc Endo */
"041F4AJ"  =  "1"  /* Bypass L Int Iliac Art to L Femor A w Autol Art, Perc Endo */
"041F4AK"  =  "1"  /* Bypass L Int Iliac Art to B Femor A w Autol Art, Perc Endo */
"041F4AP"  =  "1"  /* Bypass L Int Iliac Art to Foot Art w Autol Art, Perc Endo */
"041F4AQ"  =  "1"  /* Bypass L Int Iliac Art to Low Ex Art w Autol Art, Perc Endo */
"041F4J9"  =  "1"  /* Bypass L Int Iliac Art to R Int Ilia w Synth Sub, Perc Endo */
"041F4JB"  =  "1"  /* Bypass L Int Iliac Art to L Int Ilia w Synth Sub, Perc Endo */
"041F4JC"  =  "1"  /* Bypass L Int Iliac Art to B Int Ilia w Synth Sub, Perc Endo */
"041F4JD"  =  "1"  /* Bypass L Int Iliac Art to R Ext Ilia w Synth Sub, Perc Endo */
"041F4JF"  =  "1"  /* Bypass L Int Iliac Art to L Ext Ilia w Synth Sub, Perc Endo */
"041F4JG"  =  "1"  /* Bypass L Int Iliac Art to B Ext Ilia w Synth Sub, Perc Endo */
"041F4JH"  =  "1"  /* Bypass L Int Iliac Art to R Femor A w Synth Sub, Perc Endo */
"041F4JJ"  =  "1"  /* Bypass L Int Iliac Art to L Femor A w Synth Sub, Perc Endo */
"041F4JK"  =  "1"  /* Bypass L Int Iliac Art to B Femor A w Synth Sub, Perc Endo */
"041F4JP"  =  "1"  /* Bypass L Int Iliac Art to Foot Art w Synth Sub, Perc Endo */
"041F4JQ"  =  "1"  /* Bypass L Int Iliac Art to Low Ex Art w Synth Sub, Perc Endo */
"041F4K9"  =  "1"  /* Bypass L Int Iliac Art to R Int Ilia w Nonaut Sub, Perc Endo */
"041F4KB"  =  "1"  /* Bypass L Int Iliac Art to L Int Ilia w Nonaut Sub, Perc Endo */
"041F4KC"  =  "1"  /* Bypass L Int Iliac Art to B Int Ilia w Nonaut Sub, Perc Endo */
"041F4KD"  =  "1"  /* Bypass L Int Iliac Art to R Ext Ilia w Nonaut Sub, Perc Endo */
"041F4KF"  =  "1"  /* Bypass L Int Iliac Art to L Ext Ilia w Nonaut Sub, Perc Endo */
"041F4KG"  =  "1"  /* Bypass L Int Iliac Art to B Ext Ilia w Nonaut Sub, Perc Endo */
"041F4KH"  =  "1"  /* Bypass L Int Iliac Art to R Femor A w Nonaut Sub, Perc Endo */
"041F4KJ"  =  "1"  /* Bypass L Int Iliac Art to L Femor A w Nonaut Sub, Perc Endo */
"041F4KK"  =  "1"  /* Bypass L Int Iliac Art to B Femor A w Nonaut Sub, Perc Endo */
"041F4KP"  =  "1"  /* Bypass L Int Iliac Art to Foot Art w Nonaut Sub, Perc Endo */
"041F4KQ"  =  "1"  /* Bypass L Int Iliac Art to Low Ex Art w Nonaut Sub, Perc Endo */
"041F4Z9"  =  "1"  /* Bypass L Int Iliac Art to R Int Ilia, Perc Endo Approach */
"041F4ZB"  =  "1"  /* Bypass L Int Iliac Art to L Int Ilia, Perc Endo Approach */
"041F4ZC"  =  "1"  /* Bypass L Int Iliac Art to B Int Ilia, Perc Endo Approach */
"041F4ZD"  =  "1"  /* Bypass L Int Iliac Art to R Ext Ilia, Perc Endo Approach */
"041F4ZF"  =  "1"  /* Bypass L Int Iliac Art to L Ext Ilia, Perc Endo Approach */
"041F4ZG"  =  "1"  /* Bypass L Int Iliac Art to B Ext Ilia, Perc Endo Approach */
"041F4ZH"  =  "1"  /* Bypass L Int Iliac Art to R Femor A, Perc Endo Approach */
"041F4ZJ"  =  "1"  /* Bypass L Int Iliac Art to L Femor A, Perc Endo Approach */
"041F4ZK"  =  "1"  /* Bypass L Int Iliac Art to B Femor A, Perc Endo Approach */
"041F4ZP"  =  "1"  /* Bypass L Int Iliac Art to Foot Art, Perc Endo Approach */
"041F4ZQ"  =  "1"  /* Bypass L Int Iliac Art to Low Ex Art, Perc Endo Approach */
"041H099"  =  "1"  /* Bypass R Ext Iliac Art to R Int Ilia w Autol Vn, Open */
"041H09B"  =  "1"  /* Bypass R Ext Iliac Art to L Int Ilia w Autol Vn, Open */
"041H09C"  =  "1"  /* Bypass R Ext Iliac Art to B Int Ilia w Autol Vn, Open */
"041H09D"  =  "1"  /* Bypass R Ext Iliac Art to R Ext Ilia w Autol Vn, Open */
"041H09F"  =  "1"  /* Bypass R Ext Iliac Art to L Ext Ilia w Autol Vn, Open */
"041H09G"  =  "1"  /* Bypass R Ext Iliac Art to B Ext Ilia w Autol Vn, Open */
"041H09H"  =  "1"  /* Bypass R Ext Iliac Art to R Femor A w Autol Vn, Open */
"041H09J"  =  "1"  /* Bypass R Ext Iliac Art to L Femor A w Autol Vn, Open */
"041H09K"  =  "1"  /* Bypass R Ext Iliac Art to B Femor A w Autol Vn, Open */
"041H09P"  =  "1"  /* Bypass R Ext Iliac Art to Foot Art w Autol Vn, Open */
"041H09Q"  =  "1"  /* Bypass R Ext Iliac Art to Low Ex Art w Autol Vn, Open */
"041H0A9"  =  "1"  /* Bypass R Ext Iliac Art to R Int Ilia w Autol Art, Open */
"041H0AB"  =  "1"  /* Bypass R Ext Iliac Art to L Int Ilia w Autol Art, Open */
"041H0AC"  =  "1"  /* Bypass R Ext Iliac Art to B Int Ilia w Autol Art, Open */
"041H0AD"  =  "1"  /* Bypass R Ext Iliac Art to R Ext Ilia w Autol Art, Open */
"041H0AF"  =  "1"  /* Bypass R Ext Iliac Art to L Ext Ilia w Autol Art, Open */
"041H0AG"  =  "1"  /* Bypass R Ext Iliac Art to B Ext Ilia w Autol Art, Open */
"041H0AH"  =  "1"  /* Bypass R Ext Iliac Art to R Femor A w Autol Art, Open */
"041H0AJ"  =  "1"  /* Bypass R Ext Iliac Art to L Femor A w Autol Art, Open */
"041H0AK"  =  "1"  /* Bypass R Ext Iliac Art to B Femor A w Autol Art, Open */
"041H0AP"  =  "1"  /* Bypass R Ext Iliac Art to Foot Art w Autol Art, Open */
"041H0AQ"  =  "1"  /* Bypass R Ext Iliac Art to Low Ex Art w Autol Art, Open */
"041H0J9"  =  "1"  /* Bypass R Ext Iliac Art to R Int Ilia w Synth Sub, Open */
"041H0JB"  =  "1"  /* Bypass R Ext Iliac Art to L Int Ilia w Synth Sub, Open */
"041H0JC"  =  "1"  /* Bypass R Ext Iliac Art to B Int Ilia w Synth Sub, Open */
"041H0JD"  =  "1"  /* Bypass R Ext Iliac Art to R Ext Ilia w Synth Sub, Open */
"041H0JF"  =  "1"  /* Bypass R Ext Iliac Art to L Ext Ilia w Synth Sub, Open */
"041H0JG"  =  "1"  /* Bypass R Ext Iliac Art to B Ext Ilia w Synth Sub, Open */
"041H0JH"  =  "1"  /* Bypass R Ext Iliac Art to R Femor A w Synth Sub, Open */
"041H0JJ"  =  "1"  /* Bypass R Ext Iliac Art to L Femor A w Synth Sub, Open */
"041H0JK"  =  "1"  /* Bypass R Ext Iliac Art to B Femor A w Synth Sub, Open */
"041H0JP"  =  "1"  /* Bypass R Ext Iliac Art to Foot Art w Synth Sub, Open */
"041H0JQ"  =  "1"  /* Bypass R Ext Iliac Art to Low Ex Art w Synth Sub, Open */
"041H0K9"  =  "1"  /* Bypass R Ext Iliac Art to R Int Ilia w Nonaut Sub, Open */
"041H0KB"  =  "1"  /* Bypass R Ext Iliac Art to L Int Ilia w Nonaut Sub, Open */
"041H0KC"  =  "1"  /* Bypass R Ext Iliac Art to B Int Ilia w Nonaut Sub, Open */
"041H0KD"  =  "1"  /* Bypass R Ext Iliac Art to R Ext Ilia w Nonaut Sub, Open */
"041H0KF"  =  "1"  /* Bypass R Ext Iliac Art to L Ext Ilia w Nonaut Sub, Open */
"041H0KG"  =  "1"  /* Bypass R Ext Iliac Art to B Ext Ilia w Nonaut Sub, Open */
"041H0KH"  =  "1"  /* Bypass R Ext Iliac Art to R Femor A w Nonaut Sub, Open */
"041H0KJ"  =  "1"  /* Bypass R Ext Iliac Art to L Femor A w Nonaut Sub, Open */
"041H0KK"  =  "1"  /* Bypass R Ext Iliac Art to B Femor A w Nonaut Sub, Open */
"041H0KP"  =  "1"  /* Bypass R Ext Iliac Art to Foot Art w Nonaut Sub, Open */
"041H0KQ"  =  "1"  /* Bypass R Ext Iliac Art to Low Ex Art w Nonaut Sub, Open */
"041H0Z9"  =  "1"  /* Bypass R Ext Iliac Art to R Int Ilia, Open Approach */
"041H0ZB"  =  "1"  /* Bypass R Ext Iliac Art to L Int Ilia, Open Approach */
"041H0ZC"  =  "1"  /* Bypass R Ext Iliac Art to B Int Ilia, Open Approach */
"041H0ZD"  =  "1"  /* Bypass R Ext Iliac Art to R Ext Ilia, Open Approach */
"041H0ZF"  =  "1"  /* Bypass R Ext Iliac Art to L Ext Ilia, Open Approach */
"041H0ZG"  =  "1"  /* Bypass R Ext Iliac Art to B Ext Ilia, Open Approach */
"041H0ZH"  =  "1"  /* Bypass R Ext Iliac Art to R Femor A, Open Approach */
"041H0ZJ"  =  "1"  /* Bypass R Ext Iliac Art to L Femor A, Open Approach */
"041H0ZK"  =  "1"  /* Bypass R Ext Iliac Art to B Femor A, Open Approach */
"041H0ZP"  =  "1"  /* Bypass R Ext Iliac Art to Foot Art, Open Approach */
"041H0ZQ"  =  "1"  /* Bypass R Ext Iliac Art to Low Ex Art, Open Approach */
"041H499"  =  "1"  /* Bypass R Ext Iliac Art to R Int Ilia w Autol Vn, Perc Endo */
"041H49B"  =  "1"  /* Bypass R Ext Iliac Art to L Int Ilia w Autol Vn, Perc Endo */
"041H49C"  =  "1"  /* Bypass R Ext Iliac Art to B Int Ilia w Autol Vn, Perc Endo */
"041H49D"  =  "1"  /* Bypass R Ext Iliac Art to R Ext Ilia w Autol Vn, Perc Endo */
"041H49F"  =  "1"  /* Bypass R Ext Iliac Art to L Ext Ilia w Autol Vn, Perc Endo */
"041H49G"  =  "1"  /* Bypass R Ext Iliac Art to B Ext Ilia w Autol Vn, Perc Endo */
"041H49H"  =  "1"  /* Bypass R Ext Iliac Art to R Femor A w Autol Vn, Perc Endo */
"041H49J"  =  "1"  /* Bypass R Ext Iliac Art to L Femor A w Autol Vn, Perc Endo */
"041H49K"  =  "1"  /* Bypass R Ext Iliac Art to B Femor A w Autol Vn, Perc Endo */
"041H49P"  =  "1"  /* Bypass R Ext Iliac Art to Foot Art w Autol Vn, Perc Endo */
"041H49Q"  =  "1"  /* Bypass R Ext Iliac Art to Low Ex Art w Autol Vn, Perc Endo */
"041H4A9"  =  "1"  /* Bypass R Ext Iliac Art to R Int Ilia w Autol Art, Perc Endo */
"041H4AB"  =  "1"  /* Bypass R Ext Iliac Art to L Int Ilia w Autol Art, Perc Endo */
"041H4AC"  =  "1"  /* Bypass R Ext Iliac Art to B Int Ilia w Autol Art, Perc Endo */
"041H4AD"  =  "1"  /* Bypass R Ext Iliac Art to R Ext Ilia w Autol Art, Perc Endo */
"041H4AF"  =  "1"  /* Bypass R Ext Iliac Art to L Ext Ilia w Autol Art, Perc Endo */
"041H4AG"  =  "1"  /* Bypass R Ext Iliac Art to B Ext Ilia w Autol Art, Perc Endo */
"041H4AH"  =  "1"  /* Bypass R Ext Iliac Art to R Femor A w Autol Art, Perc Endo */
"041H4AJ"  =  "1"  /* Bypass R Ext Iliac Art to L Femor A w Autol Art, Perc Endo */
"041H4AK"  =  "1"  /* Bypass R Ext Iliac Art to B Femor A w Autol Art, Perc Endo */
"041H4AP"  =  "1"  /* Bypass R Ext Iliac Art to Foot Art w Autol Art, Perc Endo */
"041H4AQ"  =  "1"  /* Bypass R Ext Iliac Art to Low Ex Art w Autol Art, Perc Endo */
"041H4J9"  =  "1"  /* Bypass R Ext Iliac Art to R Int Ilia w Synth Sub, Perc Endo */
"041H4JB"  =  "1"  /* Bypass R Ext Iliac Art to L Int Ilia w Synth Sub, Perc Endo */
"041H4JC"  =  "1"  /* Bypass R Ext Iliac Art to B Int Ilia w Synth Sub, Perc Endo */
"041H4JD"  =  "1"  /* Bypass R Ext Iliac Art to R Ext Ilia w Synth Sub, Perc Endo */
"041H4JF"  =  "1"  /* Bypass R Ext Iliac Art to L Ext Ilia w Synth Sub, Perc Endo */
"041H4JG"  =  "1"  /* Bypass R Ext Iliac Art to B Ext Ilia w Synth Sub, Perc Endo */
"041H4JH"  =  "1"  /* Bypass R Ext Iliac Art to R Femor A w Synth Sub, Perc Endo */
"041H4JJ"  =  "1"  /* Bypass R Ext Iliac Art to L Femor A w Synth Sub, Perc Endo */
"041H4JK"  =  "1"  /* Bypass R Ext Iliac Art to B Femor A w Synth Sub, Perc Endo */
"041H4JP"  =  "1"  /* Bypass R Ext Iliac Art to Foot Art w Synth Sub, Perc Endo */
"041H4JQ"  =  "1"  /* Bypass R Ext Iliac Art to Low Ex Art w Synth Sub, Perc Endo */
"041H4K9"  =  "1"  /* Bypass R Ext Iliac Art to R Int Ilia w Nonaut Sub, Perc Endo */
"041H4KB"  =  "1"  /* Bypass R Ext Iliac Art to L Int Ilia w Nonaut Sub, Perc Endo */
"041H4KC"  =  "1"  /* Bypass R Ext Iliac Art to B Int Ilia w Nonaut Sub, Perc Endo */
"041H4KD"  =  "1"  /* Bypass R Ext Iliac Art to R Ext Ilia w Nonaut Sub, Perc Endo */
"041H4KF"  =  "1"  /* Bypass R Ext Iliac Art to L Ext Ilia w Nonaut Sub, Perc Endo */
"041H4KG"  =  "1"  /* Bypass R Ext Iliac Art to B Ext Ilia w Nonaut Sub, Perc Endo */
"041H4KH"  =  "1"  /* Bypass R Ext Iliac Art to R Femor A w Nonaut Sub, Perc Endo */
"041H4KJ"  =  "1"  /* Bypass R Ext Iliac Art to L Femor A w Nonaut Sub, Perc Endo */
"041H4KK"  =  "1"  /* Bypass R Ext Iliac Art to B Femor A w Nonaut Sub, Perc Endo */
"041H4KP"  =  "1"  /* Bypass R Ext Iliac Art to Foot Art w Nonaut Sub, Perc Endo */
"041H4KQ"  =  "1"  /* Bypass R Ext Iliac Art to Low Ex Art w Nonaut Sub, Perc Endo */
"041H4Z9"  =  "1"  /* Bypass R Ext Iliac Art to R Int Ilia, Perc Endo Approach */
"041H4ZB"  =  "1"  /* Bypass R Ext Iliac Art to L Int Ilia, Perc Endo Approach */
"041H4ZC"  =  "1"  /* Bypass R Ext Iliac Art to B Int Ilia, Perc Endo Approach */
"041H4ZD"  =  "1"  /* Bypass R Ext Iliac Art to R Ext Ilia, Perc Endo Approach */
"041H4ZF"  =  "1"  /* Bypass R Ext Iliac Art to L Ext Ilia, Perc Endo Approach */
"041H4ZG"  =  "1"  /* Bypass R Ext Iliac Art to B Ext Ilia, Perc Endo Approach */
"041H4ZH"  =  "1"  /* Bypass R Ext Iliac Art to R Femor A, Perc Endo Approach */
"041H4ZJ"  =  "1"  /* Bypass R Ext Iliac Art to L Femor A, Perc Endo Approach */
"041H4ZK"  =  "1"  /* Bypass R Ext Iliac Art to B Femor A, Perc Endo Approach */
"041H4ZP"  =  "1"  /* Bypass R Ext Iliac Art to Foot Art, Perc Endo Approach */
"041H4ZQ"  =  "1"  /* Bypass R Ext Iliac Art to Low Ex Art, Perc Endo Approach */
"041J099"  =  "1"  /* Bypass L Ext Iliac Art to R Int Ilia w Autol Vn, Open */
"041J09B"  =  "1"  /* Bypass L Ext Iliac Art to L Int Ilia w Autol Vn, Open */
"041J09C"  =  "1"  /* Bypass L Ext Iliac Art to B Int Ilia w Autol Vn, Open */
"041J09D"  =  "1"  /* Bypass L Ext Iliac Art to R Ext Ilia w Autol Vn, Open */
"041J09F"  =  "1"  /* Bypass L Ext Iliac Art to L Ext Ilia w Autol Vn, Open */
"041J09G"  =  "1"  /* Bypass L Ext Iliac Art to B Ext Ilia w Autol Vn, Open */
"041J09H"  =  "1"  /* Bypass L Ext Iliac Art to R Femor A w Autol Vn, Open */
"041J09J"  =  "1"  /* Bypass L Ext Iliac Art to L Femor A w Autol Vn, Open */
"041J09K"  =  "1"  /* Bypass L Ext Iliac Art to B Femor A w Autol Vn, Open */
"041J09P"  =  "1"  /* Bypass L Ext Iliac Art to Foot Art w Autol Vn, Open */
"041J09Q"  =  "1"  /* Bypass L Ext Iliac Art to Low Ex Art w Autol Vn, Open */
"041J0A9"  =  "1"  /* Bypass L Ext Iliac Art to R Int Ilia w Autol Art, Open */
"041J0AB"  =  "1"  /* Bypass L Ext Iliac Art to L Int Ilia w Autol Art, Open */
"041J0AC"  =  "1"  /* Bypass L Ext Iliac Art to B Int Ilia w Autol Art, Open */
"041J0AD"  =  "1"  /* Bypass L Ext Iliac Art to R Ext Ilia w Autol Art, Open */
"041J0AF"  =  "1"  /* Bypass L Ext Iliac Art to L Ext Ilia w Autol Art, Open */
"041J0AG"  =  "1"  /* Bypass L Ext Iliac Art to B Ext Ilia w Autol Art, Open */
"041J0AH"  =  "1"  /* Bypass L Ext Iliac Art to R Femor A w Autol Art, Open */
"041J0AJ"  =  "1"  /* Bypass L Ext Iliac Art to L Femor A w Autol Art, Open */
"041J0AK"  =  "1"  /* Bypass L Ext Iliac Art to B Femor A w Autol Art, Open */
"041J0AP"  =  "1"  /* Bypass L Ext Iliac Art to Foot Art w Autol Art, Open */
"041J0AQ"  =  "1"  /* Bypass L Ext Iliac Art to Low Ex Art w Autol Art, Open */
"041J0J9"  =  "1"  /* Bypass L Ext Iliac Art to R Int Ilia w Synth Sub, Open */
"041J0JB"  =  "1"  /* Bypass L Ext Iliac Art to L Int Ilia w Synth Sub, Open */
"041J0JC"  =  "1"  /* Bypass L Ext Iliac Art to B Int Ilia w Synth Sub, Open */
"041J0JD"  =  "1"  /* Bypass L Ext Iliac Art to R Ext Ilia w Synth Sub, Open */
"041J0JF"  =  "1"  /* Bypass L Ext Iliac Art to L Ext Ilia w Synth Sub, Open */
"041J0JG"  =  "1"  /* Bypass L Ext Iliac Art to B Ext Ilia w Synth Sub, Open */
"041J0JH"  =  "1"  /* Bypass L Ext Iliac Art to R Femor A w Synth Sub, Open */
"041J0JJ"  =  "1"  /* Bypass L Ext Iliac Art to L Femor A w Synth Sub, Open */
"041J0JK"  =  "1"  /* Bypass L Ext Iliac Art to B Femor A w Synth Sub, Open */
"041J0JP"  =  "1"  /* Bypass L Ext Iliac Art to Foot Art w Synth Sub, Open */
"041J0JQ"  =  "1"  /* Bypass L Ext Iliac Art to Low Ex Art w Synth Sub, Open */
"041J0K9"  =  "1"  /* Bypass L Ext Iliac Art to R Int Ilia w Nonaut Sub, Open */
"041J0KB"  =  "1"  /* Bypass L Ext Iliac Art to L Int Ilia w Nonaut Sub, Open */
"041J0KC"  =  "1"  /* Bypass L Ext Iliac Art to B Int Ilia w Nonaut Sub, Open */
"041J0KD"  =  "1"  /* Bypass L Ext Iliac Art to R Ext Ilia w Nonaut Sub, Open */
"041J0KF"  =  "1"  /* Bypass L Ext Iliac Art to L Ext Ilia w Nonaut Sub, Open */
"041J0KG"  =  "1"  /* Bypass L Ext Iliac Art to B Ext Ilia w Nonaut Sub, Open */
"041J0KH"  =  "1"  /* Bypass L Ext Iliac Art to R Femor A w Nonaut Sub, Open */
"041J0KJ"  =  "1"  /* Bypass L Ext Iliac Art to L Femor A w Nonaut Sub, Open */
"041J0KK"  =  "1"  /* Bypass L Ext Iliac Art to B Femor A w Nonaut Sub, Open */
"041J0KP"  =  "1"  /* Bypass L Ext Iliac Art to Foot Art w Nonaut Sub, Open */
"041J0KQ"  =  "1"  /* Bypass L Ext Iliac Art to Low Ex Art w Nonaut Sub, Open */
"041J0Z9"  =  "1"  /* Bypass L Ext Iliac Art to R Int Ilia, Open Approach */
"041J0ZB"  =  "1"  /* Bypass L Ext Iliac Art to L Int Ilia, Open Approach */
"041J0ZC"  =  "1"  /* Bypass L Ext Iliac Art to B Int Ilia, Open Approach */
"041J0ZD"  =  "1"  /* Bypass L Ext Iliac Art to R Ext Ilia, Open Approach */
"041J0ZF"  =  "1"  /* Bypass L Ext Iliac Art to L Ext Ilia, Open Approach */
"041J0ZG"  =  "1"  /* Bypass L Ext Iliac Art to B Ext Ilia, Open Approach */
"041J0ZH"  =  "1"  /* Bypass L Ext Iliac Art to R Femor A, Open Approach */
"041J0ZJ"  =  "1"  /* Bypass L Ext Iliac Art to L Femor A, Open Approach */
"041J0ZK"  =  "1"  /* Bypass L Ext Iliac Art to B Femor A, Open Approach */
"041J0ZP"  =  "1"  /* Bypass Left External Iliac Artery to Foot Art, Open Approach */
"041J0ZQ"  =  "1"  /* Bypass L Ext Iliac Art to Low Ex Art, Open Approach */
"041J499"  =  "1"  /* Bypass L Ext Iliac Art to R Int Ilia w Autol Vn, Perc Endo */
"041J49B"  =  "1"  /* Bypass L Ext Iliac Art to L Int Ilia w Autol Vn, Perc Endo */
"041J49C"  =  "1"  /* Bypass L Ext Iliac Art to B Int Ilia w Autol Vn, Perc Endo */
"041J49D"  =  "1"  /* Bypass L Ext Iliac Art to R Ext Ilia w Autol Vn, Perc Endo */
"041J49F"  =  "1"  /* Bypass L Ext Iliac Art to L Ext Ilia w Autol Vn, Perc Endo */
"041J49G"  =  "1"  /* Bypass L Ext Iliac Art to B Ext Ilia w Autol Vn, Perc Endo */
"041J49H"  =  "1"  /* Bypass L Ext Iliac Art to R Femor A w Autol Vn, Perc Endo */
"041J49J"  =  "1"  /* Bypass L Ext Iliac Art to L Femor A w Autol Vn, Perc Endo */
"041J49K"  =  "1"  /* Bypass L Ext Iliac Art to B Femor A w Autol Vn, Perc Endo */
"041J49P"  =  "1"  /* Bypass L Ext Iliac Art to Foot Art w Autol Vn, Perc Endo */
"041J49Q"  =  "1"  /* Bypass L Ext Iliac Art to Low Ex Art w Autol Vn, Perc Endo */
"041J4A9"  =  "1"  /* Bypass L Ext Iliac Art to R Int Ilia w Autol Art, Perc Endo */
"041J4AB"  =  "1"  /* Bypass L Ext Iliac Art to L Int Ilia w Autol Art, Perc Endo */
"041J4AC"  =  "1"  /* Bypass L Ext Iliac Art to B Int Ilia w Autol Art, Perc Endo */
"041J4AD"  =  "1"  /* Bypass L Ext Iliac Art to R Ext Ilia w Autol Art, Perc Endo */
"041J4AF"  =  "1"  /* Bypass L Ext Iliac Art to L Ext Ilia w Autol Art, Perc Endo */
"041J4AG"  =  "1"  /* Bypass L Ext Iliac Art to B Ext Ilia w Autol Art, Perc Endo */
"041J4AH"  =  "1"  /* Bypass L Ext Iliac Art to R Femor A w Autol Art, Perc Endo */
"041J4AJ"  =  "1"  /* Bypass L Ext Iliac Art to L Femor A w Autol Art, Perc Endo */
"041J4AK"  =  "1"  /* Bypass L Ext Iliac Art to B Femor A w Autol Art, Perc Endo */
"041J4AP"  =  "1"  /* Bypass L Ext Iliac Art to Foot Art w Autol Art, Perc Endo */
"041J4AQ"  =  "1"  /* Bypass L Ext Iliac Art to Low Ex Art w Autol Art, Perc Endo */
"041J4J9"  =  "1"  /* Bypass L Ext Iliac Art to R Int Ilia w Synth Sub, Perc Endo */
"041J4JB"  =  "1"  /* Bypass L Ext Iliac Art to L Int Ilia w Synth Sub, Perc Endo */
"041J4JC"  =  "1"  /* Bypass L Ext Iliac Art to B Int Ilia w Synth Sub, Perc Endo */
"041J4JD"  =  "1"  /* Bypass L Ext Iliac Art to R Ext Ilia w Synth Sub, Perc Endo */
"041J4JF"  =  "1"  /* Bypass L Ext Iliac Art to L Ext Ilia w Synth Sub, Perc Endo */
"041J4JG"  =  "1"  /* Bypass L Ext Iliac Art to B Ext Ilia w Synth Sub, Perc Endo */
"041J4JH"  =  "1"  /* Bypass L Ext Iliac Art to R Femor A w Synth Sub, Perc Endo */
"041J4JJ"  =  "1"  /* Bypass L Ext Iliac Art to L Femor A w Synth Sub, Perc Endo */
"041J4JK"  =  "1"  /* Bypass L Ext Iliac Art to B Femor A w Synth Sub, Perc Endo */
"041J4JP"  =  "1"  /* Bypass L Ext Iliac Art to Foot Art w Synth Sub, Perc Endo */
"041J4JQ"  =  "1"  /* Bypass L Ext Iliac Art to Low Ex Art w Synth Sub, Perc Endo */
"041J4K9"  =  "1"  /* Bypass L Ext Iliac Art to R Int Ilia w Nonaut Sub, Perc Endo */
"041J4KB"  =  "1"  /* Bypass L Ext Iliac Art to L Int Ilia w Nonaut Sub, Perc Endo */
"041J4KC"  =  "1"  /* Bypass L Ext Iliac Art to B Int Ilia w Nonaut Sub, Perc Endo */
"041J4KD"  =  "1"  /* Bypass L Ext Iliac Art to R Ext Ilia w Nonaut Sub, Perc Endo */
"041J4KF"  =  "1"  /* Bypass L Ext Iliac Art to L Ext Ilia w Nonaut Sub, Perc Endo */
"041J4KG"  =  "1"  /* Bypass L Ext Iliac Art to B Ext Ilia w Nonaut Sub, Perc Endo */
"041J4KH"  =  "1"  /* Bypass L Ext Iliac Art to R Femor A w Nonaut Sub, Perc Endo */
"041J4KJ"  =  "1"  /* Bypass L Ext Iliac Art to L Femor A w Nonaut Sub, Perc Endo */
"041J4KK"  =  "1"  /* Bypass L Ext Iliac Art to B Femor A w Nonaut Sub, Perc Endo */
"041J4KP"  =  "1"  /* Bypass L Ext Iliac Art to Foot Art w Nonaut Sub, Perc Endo */
"041J4KQ"  =  "1"  /* Bypass L Ext Iliac Art to Low Ex Art w Nonaut Sub, Perc Endo */
"041J4Z9"  =  "1"  /* Bypass L Ext Iliac Art to R Int Ilia, Perc Endo Approach */
"041J4ZB"  =  "1"  /* Bypass L Ext Iliac Art to L Int Ilia, Perc Endo Approach */
"041J4ZC"  =  "1"  /* Bypass L Ext Iliac Art to B Int Ilia, Perc Endo Approach */
"041J4ZD"  =  "1"  /* Bypass L Ext Iliac Art to R Ext Ilia, Perc Endo Approach */
"041J4ZF"  =  "1"  /* Bypass L Ext Iliac Art to L Ext Ilia, Perc Endo Approach */
"041J4ZG"  =  "1"  /* Bypass L Ext Iliac Art to B Ext Ilia, Perc Endo Approach */
"041J4ZH"  =  "1"  /* Bypass L Ext Iliac Art to R Femor A, Perc Endo Approach */
"041J4ZJ"  =  "1"  /* Bypass L Ext Iliac Art to L Femor A, Perc Endo Approach */
"041J4ZK"  =  "1"  /* Bypass L Ext Iliac Art to B Femor A, Perc Endo Approach */
"041J4ZP"  =  "1"  /* Bypass L Ext Iliac Art to Foot Art, Perc Endo Approach */
"041J4ZQ"  =  "1"  /* Bypass L Ext Iliac Art to Low Ex Art, Perc Endo Approach */
"041K09H"  =  "1"  /* Bypass R Femor Art to R Femor A with Autol Vn, Open Approach */
"041K09J"  =  "1"  /* Bypass R Femor Art to L Femor A with Autol Vn, Open Approach */
"041K09K"  =  "1"  /* Bypass R Femor Art to B Femor A with Autol Vn, Open Approach */
"041K09L"  =  "1"  /* Bypass R Femor Art to Poplit Art w Autol Vn, Open */
"041K09M"  =  "1"  /* Bypass R Femor Art to Peron Art with Autol Vn, Open Approach */
"041K09N"  =  "1"  /* Bypass R Femor Art to Post Tib Art w Autol Vn, Open */
"041K09P"  =  "1"  /* Bypass R Femor Art to Foot Art with Autol Vn, Open Approach */
"041K09Q"  =  "1"  /* Bypass R Femor Art to Low Ex Art w Autol Vn, Open */
"041K09S"  =  "1"  /* Bypass R Femor Art to Low Ex Vein w Autol Vn, Open */
"041K0AH"  =  "1"  /* Bypass R Femor Art to R Femor A w Autol Art, Open */
"041K0AJ"  =  "1"  /* Bypass R Femor Art to L Femor A w Autol Art, Open */
"041K0AK"  =  "1"  /* Bypass R Femor Art to B Femor A w Autol Art, Open */
"041K0AL"  =  "1"  /* Bypass R Femor Art to Poplit Art w Autol Art, Open */
"041K0AM"  =  "1"  /* Bypass R Femor Art to Peron Art w Autol Art, Open */
"041K0AN"  =  "1"  /* Bypass R Femor Art to Post Tib Art w Autol Art, Open */
"041K0AP"  =  "1"  /* Bypass R Femor Art to Foot Art with Autol Art, Open Approach */
"041K0AQ"  =  "1"  /* Bypass R Femor Art to Low Ex Art w Autol Art, Open */
"041K0AS"  =  "1"  /* Bypass R Femor Art to Low Ex Vein w Autol Art, Open */
"041K0JH"  =  "1"  /* Bypass R Femor Art to R Femor A w Synth Sub, Open */
"041K0JJ"  =  "1"  /* Bypass R Femor Art to L Femor A w Synth Sub, Open */
"041K0JK"  =  "1"  /* Bypass R Femor Art to B Femor A w Synth Sub, Open */
"041K0JL"  =  "1"  /* Bypass R Femor Art to Poplit Art w Synth Sub, Open */
"041K0JM"  =  "1"  /* Bypass R Femor Art to Peron Art w Synth Sub, Open */
"041K0JN"  =  "1"  /* Bypass R Femor Art to Post Tib Art w Synth Sub, Open */
"041K0JP"  =  "1"  /* Bypass R Femor Art to Foot Art with Synth Sub, Open Approach */
"041K0JQ"  =  "1"  /* Bypass R Femor Art to Low Ex Art w Synth Sub, Open */
"041K0JS"  =  "1"  /* Bypass R Femor Art to Low Ex Vein w Synth Sub, Open */
"041K0KH"  =  "1"  /* Bypass R Femor Art to R Femor A w Nonaut Sub, Open */
"041K0KJ"  =  "1"  /* Bypass R Femor Art to L Femor A w Nonaut Sub, Open */
"041K0KK"  =  "1"  /* Bypass R Femor Art to B Femor A w Nonaut Sub, Open */
"041K0KL"  =  "1"  /* Bypass R Femor Art to Poplit Art w Nonaut Sub, Open */
"041K0KM"  =  "1"  /* Bypass R Femor Art to Peron Art w Nonaut Sub, Open */
"041K0KN"  =  "1"  /* Bypass R Femor Art to Post Tib Art w Nonaut Sub, Open */
"041K0KP"  =  "1"  /* Bypass R Femor Art to Foot Art w Nonaut Sub, Open */
"041K0KQ"  =  "1"  /* Bypass R Femor Art to Low Ex Art w Nonaut Sub, Open */
"041K0KS"  =  "1"  /* Bypass R Femor Art to Low Ex Vein w Nonaut Sub, Open */
"041K0ZH"  =  "1"  /* Bypass Right Femoral Artery to R Femor A, Open Approach */
"041K0ZJ"  =  "1"  /* Bypass Right Femoral Artery to L Femor A, Open Approach */
"041K0ZK"  =  "1"  /* Bypass Right Femoral Artery to B Femor A, Open Approach */
"041K0ZL"  =  "1"  /* Bypass Right Femoral Artery to Poplit Art, Open Approach */
"041K0ZM"  =  "1"  /* Bypass Right Femoral Artery to Peron Art, Open Approach */
"041K0ZN"  =  "1"  /* Bypass Right Femoral Artery to Post Tib Art, Open Approach */
"041K0ZP"  =  "1"  /* Bypass Right Femoral Artery to Foot Artery, Open Approach */
"041K0ZQ"  =  "1"  /* Bypass Right Femoral Artery to Low Ex Art, Open Approach */
"041K0ZS"  =  "1"  /* Bypass Right Femoral Artery to Low Ex Vein, Open Approach */
"041K49H"  =  "1"  /* Bypass R Femor Art to R Femor A w Autol Vn, Perc Endo */
"041K49J"  =  "1"  /* Bypass R Femor Art to L Femor A w Autol Vn, Perc Endo */
"041K49K"  =  "1"  /* Bypass R Femor Art to B Femor A w Autol Vn, Perc Endo */
"041K49L"  =  "1"  /* Bypass R Femor Art to Poplit Art w Autol Vn, Perc Endo */
"041K49M"  =  "1"  /* Bypass R Femor Art to Peron Art w Autol Vn, Perc Endo */
"041K49N"  =  "1"  /* Bypass R Femor Art to Post Tib Art w Autol Vn, Perc Endo */
"041K49P"  =  "1"  /* Bypass R Femor Art to Foot Art w Autol Vn, Perc Endo */
"041K49Q"  =  "1"  /* Bypass R Femor Art to Low Ex Art w Autol Vn, Perc Endo */
"041K49S"  =  "1"  /* Bypass R Femor Art to Low Ex Vein w Autol Vn, Perc Endo */
"041K4AH"  =  "1"  /* Bypass R Femor Art to R Femor A w Autol Art, Perc Endo */
"041K4AJ"  =  "1"  /* Bypass R Femor Art to L Femor A w Autol Art, Perc Endo */
"041K4AK"  =  "1"  /* Bypass R Femor Art to B Femor A w Autol Art, Perc Endo */
"041K4AL"  =  "1"  /* Bypass R Femor Art to Poplit Art w Autol Art, Perc Endo */
"041K4AM"  =  "1"  /* Bypass R Femor Art to Peron Art w Autol Art, Perc Endo */
"041K4AN"  =  "1"  /* Bypass R Femor Art to Post Tib Art w Autol Art, Perc Endo */
"041K4AP"  =  "1"  /* Bypass R Femor Art to Foot Art w Autol Art, Perc Endo */
"041K4AQ"  =  "1"  /* Bypass R Femor Art to Low Ex Art w Autol Art, Perc Endo */
"041K4AS"  =  "1"  /* Bypass R Femor Art to Low Ex Vein w Autol Art, Perc Endo */
"041K4JH"  =  "1"  /* Bypass R Femor Art to R Femor A w Synth Sub, Perc Endo */
"041K4JJ"  =  "1"  /* Bypass R Femor Art to L Femor A w Synth Sub, Perc Endo */
"041K4JK"  =  "1"  /* Bypass R Femor Art to B Femor A w Synth Sub, Perc Endo */
"041K4JL"  =  "1"  /* Bypass R Femor Art to Poplit Art w Synth Sub, Perc Endo */
"041K4JM"  =  "1"  /* Bypass R Femor Art to Peron Art w Synth Sub, Perc Endo */
"041K4JN"  =  "1"  /* Bypass R Femor Art to Post Tib Art w Synth Sub, Perc Endo */
"041K4JP"  =  "1"  /* Bypass R Femor Art to Foot Art w Synth Sub, Perc Endo */
"041K4JQ"  =  "1"  /* Bypass R Femor Art to Low Ex Art w Synth Sub, Perc Endo */
"041K4JS"  =  "1"  /* Bypass R Femor Art to Low Ex Vein w Synth Sub, Perc Endo */
"041K4KH"  =  "1"  /* Bypass R Femor Art to R Femor A w Nonaut Sub, Perc Endo */
"041K4KJ"  =  "1"  /* Bypass R Femor Art to L Femor A w Nonaut Sub, Perc Endo */
"041K4KK"  =  "1"  /* Bypass R Femor Art to B Femor A w Nonaut Sub, Perc Endo */
"041K4KL"  =  "1"  /* Bypass R Femor Art to Poplit Art w Nonaut Sub, Perc Endo */
"041K4KM"  =  "1"  /* Bypass R Femor Art to Peron Art w Nonaut Sub, Perc Endo */
"041K4KN"  =  "1"  /* Bypass R Femor Art to Post Tib Art w Nonaut Sub, Perc Endo */
"041K4KP"  =  "1"  /* Bypass R Femor Art to Foot Art w Nonaut Sub, Perc Endo */
"041K4KQ"  =  "1"  /* Bypass R Femor Art to Low Ex Art w Nonaut Sub, Perc Endo */
"041K4KS"  =  "1"  /* Bypass R Femor Art to Low Ex Vein w Nonaut Sub, Perc Endo */
"041K4ZH"  =  "1"  /* Bypass Right Femoral Artery to R Femor A, Perc Endo Approach */
"041K4ZJ"  =  "1"  /* Bypass Right Femoral Artery to L Femor A, Perc Endo Approach */
"041K4ZK"  =  "1"  /* Bypass Right Femoral Artery to B Femor A, Perc Endo Approach */
"041K4ZL"  =  "1"  /* Bypass R Femor Art to Poplit Art, Perc Endo Approach */
"041K4ZM"  =  "1"  /* Bypass Right Femoral Artery to Peron Art, Perc Endo Approach */
"041K4ZN"  =  "1"  /* Bypass R Femor Art to Post Tib Art, Perc Endo Approach */
"041K4ZP"  =  "1"  /* Bypass Right Femoral Artery to Foot Art, Perc Endo Approach */
"041K4ZQ"  =  "1"  /* Bypass R Femor Art to Low Ex Art, Perc Endo Approach */
"041K4ZS"  =  "1"  /* Bypass R Femor Art to Low Ex Vein, Perc Endo Approach */
"041L09H"  =  "1"  /* Bypass L Femor Art to R Femor A with Autol Vn, Open Approach */
"041L09J"  =  "1"  /* Bypass L Femor Art to L Femor A with Autol Vn, Open Approach */
"041L09K"  =  "1"  /* Bypass L Femor Art to B Femor A with Autol Vn, Open Approach */
"041L09L"  =  "1"  /* Bypass L Femor Art to Poplit Art w Autol Vn, Open */
"041L09M"  =  "1"  /* Bypass L Femor Art to Peron Art with Autol Vn, Open Approach */
"041L09N"  =  "1"  /* Bypass L Femor Art to Post Tib Art w Autol Vn, Open */
"041L09P"  =  "1"  /* Bypass L Femor Art to Foot Art with Autol Vn, Open Approach */
"041L09Q"  =  "1"  /* Bypass L Femor Art to Low Ex Art w Autol Vn, Open */
"041L09S"  =  "1"  /* Bypass L Femor Art to Low Ex Vein w Autol Vn, Open */
"041L0AH"  =  "1"  /* Bypass L Femor Art to R Femor A w Autol Art, Open */
"041L0AJ"  =  "1"  /* Bypass L Femor Art to L Femor A w Autol Art, Open */
"041L0AK"  =  "1"  /* Bypass L Femor Art to B Femor A w Autol Art, Open */
"041L0AL"  =  "1"  /* Bypass L Femor Art to Poplit Art w Autol Art, Open */
"041L0AM"  =  "1"  /* Bypass L Femor Art to Peron Art w Autol Art, Open */
"041L0AN"  =  "1"  /* Bypass L Femor Art to Post Tib Art w Autol Art, Open */
"041L0AP"  =  "1"  /* Bypass L Femor Art to Foot Art with Autol Art, Open Approach */
"041L0AQ"  =  "1"  /* Bypass L Femor Art to Low Ex Art w Autol Art, Open */
"041L0AS"  =  "1"  /* Bypass L Femor Art to Low Ex Vein w Autol Art, Open */
"041L0JH"  =  "1"  /* Bypass L Femor Art to R Femor A w Synth Sub, Open */
"041L0JJ"  =  "1"  /* Bypass L Femor Art to L Femor A w Synth Sub, Open */
"041L0JK"  =  "1"  /* Bypass L Femor Art to B Femor A w Synth Sub, Open */
"041L0JL"  =  "1"  /* Bypass L Femor Art to Poplit Art w Synth Sub, Open */
"041L0JM"  =  "1"  /* Bypass L Femor Art to Peron Art w Synth Sub, Open */
"041L0JN"  =  "1"  /* Bypass L Femor Art to Post Tib Art w Synth Sub, Open */
"041L0JP"  =  "1"  /* Bypass L Femor Art to Foot Art with Synth Sub, Open Approach */
"041L0JQ"  =  "1"  /* Bypass L Femor Art to Low Ex Art w Synth Sub, Open */
"041L0JS"  =  "1"  /* Bypass L Femor Art to Low Ex Vein w Synth Sub, Open */
"041L0KH"  =  "1"  /* Bypass L Femor Art to R Femor A w Nonaut Sub, Open */
"041L0KJ"  =  "1"  /* Bypass L Femor Art to L Femor A w Nonaut Sub, Open */
"041L0KK"  =  "1"  /* Bypass L Femor Art to B Femor A w Nonaut Sub, Open */
"041L0KL"  =  "1"  /* Bypass L Femor Art to Poplit Art w Nonaut Sub, Open */
"041L0KM"  =  "1"  /* Bypass L Femor Art to Peron Art w Nonaut Sub, Open */
"041L0KN"  =  "1"  /* Bypass L Femor Art to Post Tib Art w Nonaut Sub, Open */
"041L0KP"  =  "1"  /* Bypass L Femor Art to Foot Art w Nonaut Sub, Open */
"041L0KQ"  =  "1"  /* Bypass L Femor Art to Low Ex Art w Nonaut Sub, Open */
"041L0KS"  =  "1"  /* Bypass L Femor Art to Low Ex Vein w Nonaut Sub, Open */
"041L0ZH"  =  "1"  /* Bypass Left Femoral Artery to R Femor A, Open Approach */
"041L0ZJ"  =  "1"  /* Bypass Left Femoral Artery to L Femor A, Open Approach */
"041L0ZK"  =  "1"  /* Bypass Left Femoral Artery to B Femor A, Open Approach */
"041L0ZL"  =  "1"  /* Bypass Left Femoral Artery to Poplit Art, Open Approach */
"041L0ZM"  =  "1"  /* Bypass Left Femoral Artery to Peroneal Artery, Open Approach */
"041L0ZN"  =  "1"  /* Bypass Left Femoral Artery to Post Tib Art, Open Approach */
"041L0ZP"  =  "1"  /* Bypass Left Femoral Artery to Foot Artery, Open Approach */
"041L0ZQ"  =  "1"  /* Bypass Left Femoral Artery to Low Ex Art, Open Approach */
"041L0ZS"  =  "1"  /* Bypass Left Femoral Artery to Low Ex Vein, Open Approach */
"041L49H"  =  "1"  /* Bypass L Femor Art to R Femor A w Autol Vn, Perc Endo */
"041L49J"  =  "1"  /* Bypass L Femor Art to L Femor A w Autol Vn, Perc Endo */
"041L49K"  =  "1"  /* Bypass L Femor Art to B Femor A w Autol Vn, Perc Endo */
"041L49L"  =  "1"  /* Bypass L Femor Art to Poplit Art w Autol Vn, Perc Endo */
"041L49M"  =  "1"  /* Bypass L Femor Art to Peron Art w Autol Vn, Perc Endo */
"041L49N"  =  "1"  /* Bypass L Femor Art to Post Tib Art w Autol Vn, Perc Endo */
"041L49P"  =  "1"  /* Bypass L Femor Art to Foot Art w Autol Vn, Perc Endo */
"041L49Q"  =  "1"  /* Bypass L Femor Art to Low Ex Art w Autol Vn, Perc Endo */
"041L49S"  =  "1"  /* Bypass L Femor Art to Low Ex Vein w Autol Vn, Perc Endo */
"041L4AH"  =  "1"  /* Bypass L Femor Art to R Femor A w Autol Art, Perc Endo */
"041L4AJ"  =  "1"  /* Bypass L Femor Art to L Femor A w Autol Art, Perc Endo */
"041L4AK"  =  "1"  /* Bypass L Femor Art to B Femor A w Autol Art, Perc Endo */
"041L4AL"  =  "1"  /* Bypass L Femor Art to Poplit Art w Autol Art, Perc Endo */
"041L4AM"  =  "1"  /* Bypass L Femor Art to Peron Art w Autol Art, Perc Endo */
"041L4AN"  =  "1"  /* Bypass L Femor Art to Post Tib Art w Autol Art, Perc Endo */
"041L4AP"  =  "1"  /* Bypass L Femor Art to Foot Art w Autol Art, Perc Endo */
"041L4AQ"  =  "1"  /* Bypass L Femor Art to Low Ex Art w Autol Art, Perc Endo */
"041L4AS"  =  "1"  /* Bypass L Femor Art to Low Ex Vein w Autol Art, Perc Endo */
"041L4JH"  =  "1"  /* Bypass L Femor Art to R Femor A w Synth Sub, Perc Endo */
"041L4JJ"  =  "1"  /* Bypass L Femor Art to L Femor A w Synth Sub, Perc Endo */
"041L4JK"  =  "1"  /* Bypass L Femor Art to B Femor A w Synth Sub, Perc Endo */
"041L4JL"  =  "1"  /* Bypass L Femor Art to Poplit Art w Synth Sub, Perc Endo */
"041L4JM"  =  "1"  /* Bypass L Femor Art to Peron Art w Synth Sub, Perc Endo */
"041L4JN"  =  "1"  /* Bypass L Femor Art to Post Tib Art w Synth Sub, Perc Endo */
"041L4JP"  =  "1"  /* Bypass L Femor Art to Foot Art w Synth Sub, Perc Endo */
"041L4JQ"  =  "1"  /* Bypass L Femor Art to Low Ex Art w Synth Sub, Perc Endo */
"041L4JS"  =  "1"  /* Bypass L Femor Art to Low Ex Vein w Synth Sub, Perc Endo */
"041L4KH"  =  "1"  /* Bypass L Femor Art to R Femor A w Nonaut Sub, Perc Endo */
"041L4KJ"  =  "1"  /* Bypass L Femor Art to L Femor A w Nonaut Sub, Perc Endo */
"041L4KK"  =  "1"  /* Bypass L Femor Art to B Femor A w Nonaut Sub, Perc Endo */
"041L4KL"  =  "1"  /* Bypass L Femor Art to Poplit Art w Nonaut Sub, Perc Endo */
"041L4KM"  =  "1"  /* Bypass L Femor Art to Peron Art w Nonaut Sub, Perc Endo */
"041L4KN"  =  "1"  /* Bypass L Femor Art to Post Tib Art w Nonaut Sub, Perc Endo */
"041L4KP"  =  "1"  /* Bypass L Femor Art to Foot Art w Nonaut Sub, Perc Endo */
"041L4KQ"  =  "1"  /* Bypass L Femor Art to Low Ex Art w Nonaut Sub, Perc Endo */
"041L4KS"  =  "1"  /* Bypass L Femor Art to Low Ex Vein w Nonaut Sub, Perc Endo */
"041L4ZH"  =  "1"  /* Bypass Left Femoral Artery to R Femor A, Perc Endo Approach */
"041L4ZJ"  =  "1"  /* Bypass Left Femoral Artery to L Femor A, Perc Endo Approach */
"041L4ZK"  =  "1"  /* Bypass Left Femoral Artery to B Femor A, Perc Endo Approach */
"041L4ZL"  =  "1"  /* Bypass Left Femoral Artery to Poplit Art, Perc Endo Approach */
"041L4ZM"  =  "1"  /* Bypass Left Femoral Artery to Peron Art, Perc Endo Approach */
"041L4ZN"  =  "1"  /* Bypass L Femor Art to Post Tib Art, Perc Endo Approach */
"041L4ZP"  =  "1"  /* Bypass Left Femoral Artery to Foot Art, Perc Endo Approach */
"041L4ZQ"  =  "1"  /* Bypass Left Femoral Artery to Low Ex Art, Perc Endo Approach */
"041L4ZS"  =  "1"  /* Bypass L Femor Art to Low Ex Vein, Perc Endo Approach */
"041M09L"  =  "1"  /* Bypass R Poplit Art to Poplit Art w Autol Vn, Open */
"041M09M"  =  "1"  /* Bypass R Poplit Art to Peron Art w Autol Vn, Open */
"041M09P"  =  "1"  /* Bypass R Poplit Art to Foot Art with Autol Vn, Open Approach */
"041M09Q"  =  "1"  /* Bypass R Poplit Art to Low Ex Art w Autol Vn, Open */
"041M09S"  =  "1"  /* Bypass R Poplit Art to Low Ex Vein w Autol Vn, Open */
"041M0AL"  =  "1"  /* Bypass R Poplit Art to Poplit Art w Autol Art, Open */
"041M0AM"  =  "1"  /* Bypass R Poplit Art to Peron Art w Autol Art, Open */
"041M0AP"  =  "1"  /* Bypass R Poplit Art to Foot Art w Autol Art, Open */
"041M0AQ"  =  "1"  /* Bypass R Poplit Art to Low Ex Art w Autol Art, Open */
"041M0AS"  =  "1"  /* Bypass R Poplit Art to Low Ex Vein w Autol Art, Open */
"041M0JL"  =  "1"  /* Bypass R Poplit Art to Poplit Art w Synth Sub, Open */
"041M0JM"  =  "1"  /* Bypass R Poplit Art to Peron Art w Synth Sub, Open */
"041M0JP"  =  "1"  /* Bypass R Poplit Art to Foot Art w Synth Sub, Open */
"041M0JQ"  =  "1"  /* Bypass R Poplit Art to Low Ex Art w Synth Sub, Open */
"041M0JS"  =  "1"  /* Bypass R Poplit Art to Low Ex Vein w Synth Sub, Open */
"041M0KL"  =  "1"  /* Bypass R Poplit Art to Poplit Art w Nonaut Sub, Open */
"041M0KM"  =  "1"  /* Bypass R Poplit Art to Peron Art w Nonaut Sub, Open */
"041M0KP"  =  "1"  /* Bypass R Poplit Art to Foot Art w Nonaut Sub, Open */
"041M0KQ"  =  "1"  /* Bypass R Poplit Art to Low Ex Art w Nonaut Sub, Open */
"041M0KS"  =  "1"  /* Bypass R Poplit Art to Low Ex Vein w Nonaut Sub, Open */
"041M0ZL"  =  "1"  /* Bypass Right Popliteal Artery to Poplit Art, Open Approach */
"041M0ZM"  =  "1"  /* Bypass Right Popliteal Artery to Peron Art, Open Approach */
"041M0ZP"  =  "1"  /* Bypass Right Popliteal Artery to Foot Artery, Open Approach */
"041M0ZQ"  =  "1"  /* Bypass Right Popliteal Artery to Low Ex Art, Open Approach */
"041M0ZS"  =  "1"  /* Bypass Right Popliteal Artery to Low Ex Vein, Open Approach */
"041M49L"  =  "1"  /* Bypass R Poplit Art to Poplit Art w Autol Vn, Perc Endo */
"041M49M"  =  "1"  /* Bypass R Poplit Art to Peron Art w Autol Vn, Perc Endo */
"041M49P"  =  "1"  /* Bypass R Poplit Art to Foot Art w Autol Vn, Perc Endo */
"041M49Q"  =  "1"  /* Bypass R Poplit Art to Low Ex Art w Autol Vn, Perc Endo */
"041M49S"  =  "1"  /* Bypass R Poplit Art to Low Ex Vein w Autol Vn, Perc Endo */
"041M4AL"  =  "1"  /* Bypass R Poplit Art to Poplit Art w Autol Art, Perc Endo */
"041M4AM"  =  "1"  /* Bypass R Poplit Art to Peron Art w Autol Art, Perc Endo */
"041M4AP"  =  "1"  /* Bypass R Poplit Art to Foot Art w Autol Art, Perc Endo */
"041M4AQ"  =  "1"  /* Bypass R Poplit Art to Low Ex Art w Autol Art, Perc Endo */
"041M4AS"  =  "1"  /* Bypass R Poplit Art to Low Ex Vein w Autol Art, Perc Endo */
"041M4JL"  =  "1"  /* Bypass R Poplit Art to Poplit Art w Synth Sub, Perc Endo */
"041M4JM"  =  "1"  /* Bypass R Poplit Art to Peron Art w Synth Sub, Perc Endo */
"041M4JP"  =  "1"  /* Bypass R Poplit Art to Foot Art w Synth Sub, Perc Endo */
"041M4JQ"  =  "1"  /* Bypass R Poplit Art to Low Ex Art w Synth Sub, Perc Endo */
"041M4JS"  =  "1"  /* Bypass R Poplit Art to Low Ex Vein w Synth Sub, Perc Endo */
"041M4KL"  =  "1"  /* Bypass R Poplit Art to Poplit Art w Nonaut Sub, Perc Endo */
"041M4KM"  =  "1"  /* Bypass R Poplit Art to Peron Art w Nonaut Sub, Perc Endo */
"041M4KP"  =  "1"  /* Bypass R Poplit Art to Foot Art w Nonaut Sub, Perc Endo */
"041M4KQ"  =  "1"  /* Bypass R Poplit Art to Low Ex Art w Nonaut Sub, Perc Endo */
"041M4KS"  =  "1"  /* Bypass R Poplit Art to Low Ex Vein w Nonaut Sub, Perc Endo */
"041M4ZL"  =  "1"  /* Bypass R Poplit Art to Poplit Art, Perc Endo Approach */
"041M4ZM"  =  "1"  /* Bypass R Poplit Art to Peron Art, Perc Endo Approach */
"041M4ZP"  =  "1"  /* Bypass R Poplit Art to Foot Art, Perc Endo Approach */
"041M4ZQ"  =  "1"  /* Bypass R Poplit Art to Low Ex Art, Perc Endo Approach */
"041M4ZS"  =  "1"  /* Bypass R Poplit Art to Low Ex Vein, Perc Endo Approach */
"041N09L"  =  "1"  /* Bypass L Poplit Art to Poplit Art w Autol Vn, Open */
"041N09M"  =  "1"  /* Bypass L Poplit Art to Peron Art w Autol Vn, Open */
"041N09P"  =  "1"  /* Bypass L Poplit Art to Foot Art with Autol Vn, Open Approach */
"041N09Q"  =  "1"  /* Bypass L Poplit Art to Low Ex Art w Autol Vn, Open */
"041N09S"  =  "1"  /* Bypass L Poplit Art to Low Ex Vein w Autol Vn, Open */
"041N0AL"  =  "1"  /* Bypass L Poplit Art to Poplit Art w Autol Art, Open */
"041N0AM"  =  "1"  /* Bypass L Poplit Art to Peron Art w Autol Art, Open */
"041N0AP"  =  "1"  /* Bypass L Poplit Art to Foot Art w Autol Art, Open */
"041N0AQ"  =  "1"  /* Bypass L Poplit Art to Low Ex Art w Autol Art, Open */
"041N0AS"  =  "1"  /* Bypass L Poplit Art to Low Ex Vein w Autol Art, Open */
"041N0JL"  =  "1"  /* Bypass L Poplit Art to Poplit Art w Synth Sub, Open */
"041N0JM"  =  "1"  /* Bypass L Poplit Art to Peron Art w Synth Sub, Open */
"041N0JP"  =  "1"  /* Bypass L Poplit Art to Foot Art w Synth Sub, Open */
"041N0JQ"  =  "1"  /* Bypass L Poplit Art to Low Ex Art w Synth Sub, Open */
"041N0JS"  =  "1"  /* Bypass L Poplit Art to Low Ex Vein w Synth Sub, Open */
"041N0KL"  =  "1"  /* Bypass L Poplit Art to Poplit Art w Nonaut Sub, Open */
"041N0KM"  =  "1"  /* Bypass L Poplit Art to Peron Art w Nonaut Sub, Open */
"041N0KP"  =  "1"  /* Bypass L Poplit Art to Foot Art w Nonaut Sub, Open */
"041N0KQ"  =  "1"  /* Bypass L Poplit Art to Low Ex Art w Nonaut Sub, Open */
"041N0KS"  =  "1"  /* Bypass L Poplit Art to Low Ex Vein w Nonaut Sub, Open */
"041N0ZL"  =  "1"  /* Bypass Left Popliteal Artery to Poplit Art, Open Approach */
"041N0ZM"  =  "1"  /* Bypass Left Popliteal Artery to Peron Art, Open Approach */
"041N0ZP"  =  "1"  /* Bypass Left Popliteal Artery to Foot Artery, Open Approach */
"041N0ZQ"  =  "1"  /* Bypass Left Popliteal Artery to Low Ex Art, Open Approach */
"041N0ZS"  =  "1"  /* Bypass Left Popliteal Artery to Low Ex Vein, Open Approach */
"041N49L"  =  "1"  /* Bypass L Poplit Art to Poplit Art w Autol Vn, Perc Endo */
"041N49M"  =  "1"  /* Bypass L Poplit Art to Peron Art w Autol Vn, Perc Endo */
"041N49P"  =  "1"  /* Bypass L Poplit Art to Foot Art w Autol Vn, Perc Endo */
"041N49Q"  =  "1"  /* Bypass L Poplit Art to Low Ex Art w Autol Vn, Perc Endo */
"041N49S"  =  "1"  /* Bypass L Poplit Art to Low Ex Vein w Autol Vn, Perc Endo */
"041N4AL"  =  "1"  /* Bypass L Poplit Art to Poplit Art w Autol Art, Perc Endo */
"041N4AM"  =  "1"  /* Bypass L Poplit Art to Peron Art w Autol Art, Perc Endo */
"041N4AP"  =  "1"  /* Bypass L Poplit Art to Foot Art w Autol Art, Perc Endo */
"041N4AQ"  =  "1"  /* Bypass L Poplit Art to Low Ex Art w Autol Art, Perc Endo */
"041N4AS"  =  "1"  /* Bypass L Poplit Art to Low Ex Vein w Autol Art, Perc Endo */
"041N4JL"  =  "1"  /* Bypass L Poplit Art to Poplit Art w Synth Sub, Perc Endo */
"041N4JM"  =  "1"  /* Bypass L Poplit Art to Peron Art w Synth Sub, Perc Endo */
"041N4JP"  =  "1"  /* Bypass L Poplit Art to Foot Art w Synth Sub, Perc Endo */
"041N4JQ"  =  "1"  /* Bypass L Poplit Art to Low Ex Art w Synth Sub, Perc Endo */
"041N4JS"  =  "1"  /* Bypass L Poplit Art to Low Ex Vein w Synth Sub, Perc Endo */
"041N4KL"  =  "1"  /* Bypass L Poplit Art to Poplit Art w Nonaut Sub, Perc Endo */
"041N4KM"  =  "1"  /* Bypass L Poplit Art to Peron Art w Nonaut Sub, Perc Endo */
"041N4KP"  =  "1"  /* Bypass L Poplit Art to Foot Art w Nonaut Sub, Perc Endo */
"041N4KQ"  =  "1"  /* Bypass L Poplit Art to Low Ex Art w Nonaut Sub, Perc Endo */
"041N4KS"  =  "1"  /* Bypass L Poplit Art to Low Ex Vein w Nonaut Sub, Perc Endo */
"041N4ZL"  =  "1"  /* Bypass L Poplit Art to Poplit Art, Perc Endo Approach */
"041N4ZM"  =  "1"  /* Bypass L Poplit Art to Peron Art, Perc Endo Approach */
"041N4ZP"  =  "1"  /* Bypass Left Popliteal Artery to Foot Art, Perc Endo Approach */
"041N4ZQ"  =  "1"  /* Bypass L Poplit Art to Low Ex Art, Perc Endo Approach */
"041N4ZS"  =  "1"  /* Bypass L Poplit Art to Low Ex Vein, Perc Endo Approach */
"04500ZZ"  =  "1"  /* Destruction of Abdominal Aorta, Open Approach */
"04503ZZ"  =  "1"  /* Destruction of Abdominal Aorta, Percutaneous Approach */
"04504ZZ"  =  "1"  /* Destruction of Abdominal Aorta, Perc Endo Approach */
"04510ZZ"  =  "1"  /* Destruction of Celiac Artery, Open Approach */
"04513ZZ"  =  "1"  /* Destruction of Celiac Artery, Percutaneous Approach */
"04514ZZ"  =  "1"  /* Destruction of Celiac Artery, Perc Endo Approach */
"04520ZZ"  =  "1"  /* Destruction of Gastric Artery, Open Approach */
"04523ZZ"  =  "1"  /* Destruction of Gastric Artery, Percutaneous Approach */
"04524ZZ"  =  "1"  /* Destruction of Gastric Artery, Perc Endo Approach */
"04530ZZ"  =  "1"  /* Destruction of Hepatic Artery, Open Approach */
"04533ZZ"  =  "1"  /* Destruction of Hepatic Artery, Percutaneous Approach */
"04534ZZ"  =  "1"  /* Destruction of Hepatic Artery, Perc Endo Approach */
"04540ZZ"  =  "1"  /* Destruction of Splenic Artery, Open Approach */
"04543ZZ"  =  "1"  /* Destruction of Splenic Artery, Percutaneous Approach */
"04544ZZ"  =  "1"  /* Destruction of Splenic Artery, Perc Endo Approach */
"04550ZZ"  =  "1"  /* Destruction of Superior Mesenteric Artery, Open Approach */
"04553ZZ"  =  "1"  /* Destruction of Superior Mesenteric Artery, Perc Approach */
"04554ZZ"  =  "1"  /* Destruction of Sup Mesent Art, Perc Endo Approach */
"04560ZZ"  =  "1"  /* Destruction of Right Colic Artery, Open Approach */
"04563ZZ"  =  "1"  /* Destruction of Right Colic Artery, Percutaneous Approach */
"04564ZZ"  =  "1"  /* Destruction of Right Colic Artery, Perc Endo Approach */
"04570ZZ"  =  "1"  /* Destruction of Left Colic Artery, Open Approach */
"04573ZZ"  =  "1"  /* Destruction of Left Colic Artery, Percutaneous Approach */
"04574ZZ"  =  "1"  /* Destruction of Left Colic Artery, Perc Endo Approach */
"04580ZZ"  =  "1"  /* Destruction of Middle Colic Artery, Open Approach */
"04583ZZ"  =  "1"  /* Destruction of Middle Colic Artery, Percutaneous Approach */
"04584ZZ"  =  "1"  /* Destruction of Middle Colic Artery, Perc Endo Approach */
"04590ZZ"  =  "1"  /* Destruction of Right Renal Artery, Open Approach */
"04593ZZ"  =  "1"  /* Destruction of Right Renal Artery, Percutaneous Approach */
"04594ZZ"  =  "1"  /* Destruction of Right Renal Artery, Perc Endo Approach */
"045A0ZZ"  =  "1"  /* Destruction of Left Renal Artery, Open Approach */
"045A3ZZ"  =  "1"  /* Destruction of Left Renal Artery, Percutaneous Approach */
"045A4ZZ"  =  "1"  /* Destruction of Left Renal Artery, Perc Endo Approach */
"045B0ZZ"  =  "1"  /* Destruction of Inferior Mesenteric Artery, Open Approach */
"045B3ZZ"  =  "1"  /* Destruction of Inferior Mesenteric Artery, Perc Approach */
"045B4ZZ"  =  "1"  /* Destruction of Inf Mesent Art, Perc Endo Approach */
"045C0ZZ"  =  "1"  /* Destruction of Right Common Iliac Artery, Open Approach */
"045C3ZZ"  =  "1"  /* Destruction of Right Common Iliac Artery, Perc Approach */
"045C4ZZ"  =  "1"  /* Destruction of Right Common Iliac Artery, Perc Endo Approach */
"045D0ZZ"  =  "1"  /* Destruction of Left Common Iliac Artery, Open Approach */
"045D3ZZ"  =  "1"  /* Destruction of Left Common Iliac Artery, Perc Approach */
"045D4ZZ"  =  "1"  /* Destruction of Left Common Iliac Artery, Perc Endo Approach */
"045E0ZZ"  =  "1"  /* Destruction of Right Internal Iliac Artery, Open Approach */
"045E3ZZ"  =  "1"  /* Destruction of Right Internal Iliac Artery, Perc Approach */
"045E4ZZ"  =  "1"  /* Destruction of R Int Iliac Art, Perc Endo Approach */
"045F0ZZ"  =  "1"  /* Destruction of Left Internal Iliac Artery, Open Approach */
"045F3ZZ"  =  "1"  /* Destruction of Left Internal Iliac Artery, Perc Approach */
"045F4ZZ"  =  "1"  /* Destruction of L Int Iliac Art, Perc Endo Approach */
"045H0ZZ"  =  "1"  /* Destruction of Right External Iliac Artery, Open Approach */
"045H3ZZ"  =  "1"  /* Destruction of Right External Iliac Artery, Perc Approach */
"045H4ZZ"  =  "1"  /* Destruction of R Ext Iliac Art, Perc Endo Approach */
"045J0ZZ"  =  "1"  /* Destruction of Left External Iliac Artery, Open Approach */
"045J3ZZ"  =  "1"  /* Destruction of Left External Iliac Artery, Perc Approach */
"045J4ZZ"  =  "1"  /* Destruction of L Ext Iliac Art, Perc Endo Approach */
"045K0ZZ"  =  "1"  /* Destruction of Right Femoral Artery, Open Approach */
"045K3ZZ"  =  "1"  /* Destruction of Right Femoral Artery, Percutaneous Approach */
"045K4ZZ"  =  "1"  /* Destruction of Right Femoral Artery, Perc Endo Approach */
"045L0ZZ"  =  "1"  /* Destruction of Left Femoral Artery, Open Approach */
"045L3ZZ"  =  "1"  /* Destruction of Left Femoral Artery, Percutaneous Approach */
"045L4ZZ"  =  "1"  /* Destruction of Left Femoral Artery, Perc Endo Approach */
"045M0ZZ"  =  "1"  /* Destruction of Right Popliteal Artery, Open Approach */
"045M3ZZ"  =  "1"  /* Destruction of Right Popliteal Artery, Percutaneous Approach */
"045M4ZZ"  =  "1"  /* Destruction of Right Popliteal Artery, Perc Endo Approach */
"045N0ZZ"  =  "1"  /* Destruction of Left Popliteal Artery, Open Approach */
"045N3ZZ"  =  "1"  /* Destruction of Left Popliteal Artery, Percutaneous Approach */
"045N4ZZ"  =  "1"  /* Destruction of Left Popliteal Artery, Perc Endo Approach */
"045P0ZZ"  =  "1"  /* Destruction of Right Anterior Tibial Artery, Open Approach */
"045P3ZZ"  =  "1"  /* Destruction of Right Anterior Tibial Artery, Perc Approach */
"045P4ZZ"  =  "1"  /* Destruction of R Ant Tib Art, Perc Endo Approach */
"045Q0ZZ"  =  "1"  /* Destruction of Left Anterior Tibial Artery, Open Approach */
"045Q3ZZ"  =  "1"  /* Destruction of Left Anterior Tibial Artery, Perc Approach */
"045Q4ZZ"  =  "1"  /* Destruction of L Ant Tib Art, Perc Endo Approach */
"045R0ZZ"  =  "1"  /* Destruction of Right Posterior Tibial Artery, Open Approach */
"045R3ZZ"  =  "1"  /* Destruction of Right Posterior Tibial Artery, Perc Approach */
"045R4ZZ"  =  "1"  /* Destruction of R Post Tib Art, Perc Endo Approach */
"045S0ZZ"  =  "1"  /* Destruction of Left Posterior Tibial Artery, Open Approach */
"045S3ZZ"  =  "1"  /* Destruction of Left Posterior Tibial Artery, Perc Approach */
"045S4ZZ"  =  "1"  /* Destruction of L Post Tib Art, Perc Endo Approach */
"045T0ZZ"  =  "1"  /* Destruction of Right Peroneal Artery, Open Approach */
"045T3ZZ"  =  "1"  /* Destruction of Right Peroneal Artery, Percutaneous Approach */
"045T4ZZ"  =  "1"  /* Destruction of Right Peroneal Artery, Perc Endo Approach */
"045U0ZZ"  =  "1"  /* Destruction of Left Peroneal Artery, Open Approach */
"045U3ZZ"  =  "1"  /* Destruction of Left Peroneal Artery, Percutaneous Approach */
"045U4ZZ"  =  "1"  /* Destruction of Left Peroneal Artery, Perc Endo Approach */
"045V0ZZ"  =  "1"  /* Destruction of Right Foot Artery, Open Approach */
"045V3ZZ"  =  "1"  /* Destruction of Right Foot Artery, Percutaneous Approach */
"045V4ZZ"  =  "1"  /* Destruction of Right Foot Artery, Perc Endo Approach */
"045W0ZZ"  =  "1"  /* Destruction of Left Foot Artery, Open Approach */
"045W3ZZ"  =  "1"  /* Destruction of Left Foot Artery, Percutaneous Approach */
"045W4ZZ"  =  "1"  /* Destruction of Left Foot Artery, Perc Endo Approach */
"045Y0ZZ"  =  "1"  /* Destruction of Lower Artery, Open Approach */
"045Y3ZZ"  =  "1"  /* Destruction of Lower Artery, Percutaneous Approach */
"045Y4ZZ"  =  "1"  /* Destruction of Lower Artery, Perc Endo Approach */
"047004Z"  =  "1"  /* Dilation of Abd Aorta with Drug-elut Intralum, Open Approach */
"04700DZ"  =  "1"  /* Dilation of Abdominal Aorta with Intralum Dev, Open Approach */
"04700ZZ"  =  "1"  /* Dilation of Abdominal Aorta, Open Approach */
"047034Z"  =  "1"  /* Dilation of Abd Aorta with Drug-elut Intralum, Perc Approach */
"04703DZ"  =  "1"  /* Dilation of Abdominal Aorta with Intralum Dev, Perc Approach */
"04703ZZ"  =  "1"  /* Dilation of Abdominal Aorta, Percutaneous Approach */
"047044Z"  =  "1"  /* Dilate Abd Aorta w Drug-elut Intralum, Perc Endo */
"04704DZ"  =  "1"  /* Dilation of Abd Aorta with Intralum Dev, Perc Endo Approach */
"04704ZZ"  =  "1"  /* Dilation of Abdominal Aorta, Perc Endo Approach */
"047104Z"  =  "1"  /* Dilate of Celiac Art with Drug-elut Intralum, Open Approach */
"04710DZ"  =  "1"  /* Dilation of Celiac Artery with Intralum Dev, Open Approach */
"04710ZZ"  =  "1"  /* Dilation of Celiac Artery, Open Approach */
"047134Z"  =  "1"  /* Dilate of Celiac Art with Drug-elut Intralum, Perc Approach */
"04713DZ"  =  "1"  /* Dilation of Celiac Artery with Intralum Dev, Perc Approach */
"04713ZZ"  =  "1"  /* Dilation of Celiac Artery, Percutaneous Approach */
"047144Z"  =  "1"  /* Dilate Celiac Art w Drug-elut Intralum, Perc Endo */
"04714DZ"  =  "1"  /* Dilation of Celiac Art with Intralum Dev, Perc Endo Approach */
"04714ZZ"  =  "1"  /* Dilation of Celiac Artery, Percutaneous Endoscopic Approach */
"047204Z"  =  "1"  /* Dilate of Gastric Art with Drug-elut Intralum, Open Approach */
"04720DZ"  =  "1"  /* Dilation of Gastric Artery with Intralum Dev, Open Approach */
"04720ZZ"  =  "1"  /* Dilation of Gastric Artery, Open Approach */
"047234Z"  =  "1"  /* Dilate of Gastric Art with Drug-elut Intralum, Perc Approach */
"04723DZ"  =  "1"  /* Dilation of Gastric Artery with Intralum Dev, Perc Approach */
"04723ZZ"  =  "1"  /* Dilation of Gastric Artery, Percutaneous Approach */
"047244Z"  =  "1"  /* Dilate Gastric Art w Drug-elut Intralum, Perc Endo */
"04724DZ"  =  "1"  /* Dilate of Gastric Art with Intralum Dev, Perc Endo Approach */
"04724ZZ"  =  "1"  /* Dilation of Gastric Artery, Percutaneous Endoscopic Approach */
"047304Z"  =  "1"  /* Dilate of Hepatic Art with Drug-elut Intralum, Open Approach */
"04730DZ"  =  "1"  /* Dilation of Hepatic Artery with Intralum Dev, Open Approach */
"04730ZZ"  =  "1"  /* Dilation of Hepatic Artery, Open Approach */
"047334Z"  =  "1"  /* Dilate of Hepatic Art with Drug-elut Intralum, Perc Approach */
"04733DZ"  =  "1"  /* Dilation of Hepatic Artery with Intralum Dev, Perc Approach */
"04733ZZ"  =  "1"  /* Dilation of Hepatic Artery, Percutaneous Approach */
"047344Z"  =  "1"  /* Dilate Hepatic Art w Drug-elut Intralum, Perc Endo */
"04734DZ"  =  "1"  /* Dilate of Hepatic Art with Intralum Dev, Perc Endo Approach */
"04734ZZ"  =  "1"  /* Dilation of Hepatic Artery, Percutaneous Endoscopic Approach */
"047404Z"  =  "1"  /* Dilate of Splenic Art with Drug-elut Intralum, Open Approach */
"04740DZ"  =  "1"  /* Dilation of Splenic Artery with Intralum Dev, Open Approach */
"04740ZZ"  =  "1"  /* Dilation of Splenic Artery, Open Approach */
"047434Z"  =  "1"  /* Dilate of Splenic Art with Drug-elut Intralum, Perc Approach */
"04743DZ"  =  "1"  /* Dilation of Splenic Artery with Intralum Dev, Perc Approach */
"04743ZZ"  =  "1"  /* Dilation of Splenic Artery, Percutaneous Approach */
"047444Z"  =  "1"  /* Dilate Splenic Art w Drug-elut Intralum, Perc Endo */
"04744DZ"  =  "1"  /* Dilate of Splenic Art with Intralum Dev, Perc Endo Approach */
"04744ZZ"  =  "1"  /* Dilation of Splenic Artery, Percutaneous Endoscopic Approach */
"047504Z"  =  "1"  /* Dilate Sup Mesent Art w Drug-elut Intralum, Open */
"04750DZ"  =  "1"  /* Dilation of Sup Mesent Art with Intralum Dev, Open Approach */
"04750ZZ"  =  "1"  /* Dilation of Superior Mesenteric Artery, Open Approach */
"047534Z"  =  "1"  /* Dilate Sup Mesent Art w Drug-elut Intralum, Perc */
"04753DZ"  =  "1"  /* Dilation of Sup Mesent Art with Intralum Dev, Perc Approach */
"04753ZZ"  =  "1"  /* Dilation of Superior Mesenteric Artery, Perc Approach */
"047544Z"  =  "1"  /* Dilate Sup Mesent Art w Drug-elut Intralum, Perc Endo */
"04754DZ"  =  "1"  /* Dilate Sup Mesent Art w Intralum Dev, Perc Endo */
"04754ZZ"  =  "1"  /* Dilation of Superior Mesenteric Artery, Perc Endo Approach */
"047604Z"  =  "1"  /* Dilate of R Colic Art with Drug-elut Intralum, Open Approach */
"04760DZ"  =  "1"  /* Dilation of R Colic Art with Intralum Dev, Open Approach */
"04760ZZ"  =  "1"  /* Dilation of Right Colic Artery, Open Approach */
"047634Z"  =  "1"  /* Dilate of R Colic Art with Drug-elut Intralum, Perc Approach */
"04763DZ"  =  "1"  /* Dilation of R Colic Art with Intralum Dev, Perc Approach */
"04763ZZ"  =  "1"  /* Dilation of Right Colic Artery, Percutaneous Approach */
"047644Z"  =  "1"  /* Dilate R Colic Art w Drug-elut Intralum, Perc Endo */
"04764DZ"  =  "1"  /* Dilate of R Colic Art with Intralum Dev, Perc Endo Approach */
"04764ZZ"  =  "1"  /* Dilation of Right Colic Artery, Perc Endo Approach */
"047704Z"  =  "1"  /* Dilate of L Colic Art with Drug-elut Intralum, Open Approach */
"04770DZ"  =  "1"  /* Dilation of L Colic Art with Intralum Dev, Open Approach */
"04770ZZ"  =  "1"  /* Dilation of Left Colic Artery, Open Approach */
"047734Z"  =  "1"  /* Dilate of L Colic Art with Drug-elut Intralum, Perc Approach */
"04773DZ"  =  "1"  /* Dilation of L Colic Art with Intralum Dev, Perc Approach */
"04773ZZ"  =  "1"  /* Dilation of Left Colic Artery, Percutaneous Approach */
"047744Z"  =  "1"  /* Dilate L Colic Art w Drug-elut Intralum, Perc Endo */
"04774DZ"  =  "1"  /* Dilate of L Colic Art with Intralum Dev, Perc Endo Approach */
"04774ZZ"  =  "1"  /* Dilation of Left Colic Artery, Perc Endo Approach */
"047804Z"  =  "1"  /* Dilate Mid Colic Art w Drug-elut Intralum, Open */
"04780DZ"  =  "1"  /* Dilation of Mid Colic Art with Intralum Dev, Open Approach */
"04780ZZ"  =  "1"  /* Dilation of Middle Colic Artery, Open Approach */
"047834Z"  =  "1"  /* Dilate Mid Colic Art w Drug-elut Intralum, Perc */
"04783DZ"  =  "1"  /* Dilation of Mid Colic Art with Intralum Dev, Perc Approach */
"04783ZZ"  =  "1"  /* Dilation of Middle Colic Artery, Percutaneous Approach */
"047844Z"  =  "1"  /* Dilate Mid Colic Art w Drug-elut Intralum, Perc Endo */
"04784DZ"  =  "1"  /* Dilate Mid Colic Art w Intralum Dev, Perc Endo */
"04784ZZ"  =  "1"  /* Dilation of Middle Colic Artery, Perc Endo Approach */
"047904Z"  =  "1"  /* Dilate of R Renal Art with Drug-elut Intralum, Open Approach */
"04790DZ"  =  "1"  /* Dilation of R Renal Art with Intralum Dev, Open Approach */
"04790ZZ"  =  "1"  /* Dilation of Right Renal Artery, Open Approach */
"047934Z"  =  "1"  /* Dilate of R Renal Art with Drug-elut Intralum, Perc Approach */
"04793DZ"  =  "1"  /* Dilation of R Renal Art with Intralum Dev, Perc Approach */
"04793ZZ"  =  "1"  /* Dilation of Right Renal Artery, Percutaneous Approach */
"047944Z"  =  "1"  /* Dilate R Renal Art w Drug-elut Intralum, Perc Endo */
"04794DZ"  =  "1"  /* Dilate of R Renal Art with Intralum Dev, Perc Endo Approach */
"04794ZZ"  =  "1"  /* Dilation of Right Renal Artery, Perc Endo Approach */
"047A04Z"  =  "1"  /* Dilate of L Renal Art with Drug-elut Intralum, Open Approach */
"047A0DZ"  =  "1"  /* Dilation of L Renal Art with Intralum Dev, Open Approach */
"047A0ZZ"  =  "1"  /* Dilation of Left Renal Artery, Open Approach */
"047A34Z"  =  "1"  /* Dilate of L Renal Art with Drug-elut Intralum, Perc Approach */
"047A3DZ"  =  "1"  /* Dilation of L Renal Art with Intralum Dev, Perc Approach */
"047A3ZZ"  =  "1"  /* Dilation of Left Renal Artery, Percutaneous Approach */
"047A44Z"  =  "1"  /* Dilate L Renal Art w Drug-elut Intralum, Perc Endo */
"047A4DZ"  =  "1"  /* Dilate of L Renal Art with Intralum Dev, Perc Endo Approach */
"047A4ZZ"  =  "1"  /* Dilation of Left Renal Artery, Perc Endo Approach */
"047B04Z"  =  "1"  /* Dilate Inf Mesent Art w Drug-elut Intralum, Open */
"047B0DZ"  =  "1"  /* Dilation of Inf Mesent Art with Intralum Dev, Open Approach */
"047B0ZZ"  =  "1"  /* Dilation of Inferior Mesenteric Artery, Open Approach */
"047B34Z"  =  "1"  /* Dilate Inf Mesent Art w Drug-elut Intralum, Perc */
"047B3DZ"  =  "1"  /* Dilation of Inf Mesent Art with Intralum Dev, Perc Approach */
"047B3ZZ"  =  "1"  /* Dilation of Inferior Mesenteric Artery, Perc Approach */
"047B44Z"  =  "1"  /* Dilate Inf Mesent Art w Drug-elut Intralum, Perc Endo */
"047B4DZ"  =  "1"  /* Dilate Inf Mesent Art w Intralum Dev, Perc Endo */
"047B4ZZ"  =  "1"  /* Dilation of Inferior Mesenteric Artery, Perc Endo Approach */
"047C04Z"  =  "1"  /* Dilate R Com Iliac Art w Drug-elut Intralum, Open */
"047C0DZ"  =  "1"  /* Dilation of R Com Iliac Art with Intralum Dev, Open Approach */
"047C0ZZ"  =  "1"  /* Dilation of Right Common Iliac Artery, Open Approach */
"047C34Z"  =  "1"  /* Dilate R Com Iliac Art w Drug-elut Intralum, Perc */
"047C3DZ"  =  "1"  /* Dilation of R Com Iliac Art with Intralum Dev, Perc Approach */
"047C3ZZ"  =  "1"  /* Dilation of Right Common Iliac Artery, Percutaneous Approach */
"047C44Z"  =  "1"  /* Dilate R Com Iliac Art w Drug-elut Intralum, Perc Endo */
"047C4DZ"  =  "1"  /* Dilate R Com Iliac Art w Intralum Dev, Perc Endo */
"047C4ZZ"  =  "1"  /* Dilation of Right Common Iliac Artery, Perc Endo Approach */
"047D04Z"  =  "1"  /* Dilate L Com Iliac Art w Drug-elut Intralum, Open */
"047D0DZ"  =  "1"  /* Dilation of L Com Iliac Art with Intralum Dev, Open Approach */
"047D0ZZ"  =  "1"  /* Dilation of Left Common Iliac Artery, Open Approach */
"047D34Z"  =  "1"  /* Dilate L Com Iliac Art w Drug-elut Intralum, Perc */
"047D3DZ"  =  "1"  /* Dilation of L Com Iliac Art with Intralum Dev, Perc Approach */
"047D3ZZ"  =  "1"  /* Dilation of Left Common Iliac Artery, Percutaneous Approach */
"047D44Z"  =  "1"  /* Dilate L Com Iliac Art w Drug-elut Intralum, Perc Endo */
"047D4DZ"  =  "1"  /* Dilate L Com Iliac Art w Intralum Dev, Perc Endo */
"047D4ZZ"  =  "1"  /* Dilation of Left Common Iliac Artery, Perc Endo Approach */
"047E04Z"  =  "1"  /* Dilate R Int Iliac Art w Drug-elut Intralum, Open */
"047E0DZ"  =  "1"  /* Dilation of R Int Iliac Art with Intralum Dev, Open Approach */
"047E0ZZ"  =  "1"  /* Dilation of Right Internal Iliac Artery, Open Approach */
"047E34Z"  =  "1"  /* Dilate R Int Iliac Art w Drug-elut Intralum, Perc */
"047E3DZ"  =  "1"  /* Dilation of R Int Iliac Art with Intralum Dev, Perc Approach */
"047E3ZZ"  =  "1"  /* Dilation of Right Internal Iliac Artery, Perc Approach */
"047E44Z"  =  "1"  /* Dilate R Int Iliac Art w Drug-elut Intralum, Perc Endo */
"047E4DZ"  =  "1"  /* Dilate R Int Iliac Art w Intralum Dev, Perc Endo */
"047E4ZZ"  =  "1"  /* Dilation of Right Internal Iliac Artery, Perc Endo Approach */
"047F04Z"  =  "1"  /* Dilate L Int Iliac Art w Drug-elut Intralum, Open */
"047F0DZ"  =  "1"  /* Dilation of L Int Iliac Art with Intralum Dev, Open Approach */
"047F0ZZ"  =  "1"  /* Dilation of Left Internal Iliac Artery, Open Approach */
"047F34Z"  =  "1"  /* Dilate L Int Iliac Art w Drug-elut Intralum, Perc */
"047F3DZ"  =  "1"  /* Dilation of L Int Iliac Art with Intralum Dev, Perc Approach */
"047F3ZZ"  =  "1"  /* Dilation of Left Internal Iliac Artery, Perc Approach */
"047F44Z"  =  "1"  /* Dilate L Int Iliac Art w Drug-elut Intralum, Perc Endo */
"047F4DZ"  =  "1"  /* Dilate L Int Iliac Art w Intralum Dev, Perc Endo */
"047F4ZZ"  =  "1"  /* Dilation of Left Internal Iliac Artery, Perc Endo Approach */
"047H04Z"  =  "1"  /* Dilate R Ext Iliac Art w Drug-elut Intralum, Open */
"047H0DZ"  =  "1"  /* Dilation of R Ext Iliac Art with Intralum Dev, Open Approach */
"047H0ZZ"  =  "1"  /* Dilation of Right External Iliac Artery, Open Approach */
"047H34Z"  =  "1"  /* Dilate R Ext Iliac Art w Drug-elut Intralum, Perc */
"047H3DZ"  =  "1"  /* Dilation of R Ext Iliac Art with Intralum Dev, Perc Approach */
"047H3ZZ"  =  "1"  /* Dilation of Right External Iliac Artery, Perc Approach */
"047H44Z"  =  "1"  /* Dilate R Ext Iliac Art w Drug-elut Intralum, Perc Endo */
"047H4DZ"  =  "1"  /* Dilate R Ext Iliac Art w Intralum Dev, Perc Endo */
"047H4ZZ"  =  "1"  /* Dilation of Right External Iliac Artery, Perc Endo Approach */
"047J04Z"  =  "1"  /* Dilate L Ext Iliac Art w Drug-elut Intralum, Open */
"047J0DZ"  =  "1"  /* Dilation of L Ext Iliac Art with Intralum Dev, Open Approach */
"047J0ZZ"  =  "1"  /* Dilation of Left External Iliac Artery, Open Approach */
"047J34Z"  =  "1"  /* Dilate L Ext Iliac Art w Drug-elut Intralum, Perc */
"047J3DZ"  =  "1"  /* Dilation of L Ext Iliac Art with Intralum Dev, Perc Approach */
"047J3ZZ"  =  "1"  /* Dilation of Left External Iliac Artery, Perc Approach */
"047J44Z"  =  "1"  /* Dilate L Ext Iliac Art w Drug-elut Intralum, Perc Endo */
"047J4DZ"  =  "1"  /* Dilate L Ext Iliac Art w Intralum Dev, Perc Endo */
"047J4ZZ"  =  "1"  /* Dilation of Left External Iliac Artery, Perc Endo Approach */
"047K04Z"  =  "1"  /* Dilate of R Femor Art with Drug-elut Intralum, Open Approach */
"047K0DZ"  =  "1"  /* Dilation of R Femor Art with Intralum Dev, Open Approach */
"047K0ZZ"  =  "1"  /* Dilation of Right Femoral Artery, Open Approach */
"047K34Z"  =  "1"  /* Dilate of R Femor Art with Drug-elut Intralum, Perc Approach */
"047K3DZ"  =  "1"  /* Dilation of R Femor Art with Intralum Dev, Perc Approach */
"047K3ZZ"  =  "1"  /* Dilation of Right Femoral Artery, Percutaneous Approach */
"047K44Z"  =  "1"  /* Dilate R Femor Art w Drug-elut Intralum, Perc Endo */
"047K4DZ"  =  "1"  /* Dilate of R Femor Art with Intralum Dev, Perc Endo Approach */
"047K4ZZ"  =  "1"  /* Dilation of Right Femoral Artery, Perc Endo Approach */
"047L04Z"  =  "1"  /* Dilate of L Femor Art with Drug-elut Intralum, Open Approach */
"047L0DZ"  =  "1"  /* Dilation of L Femor Art with Intralum Dev, Open Approach */
"047L0ZZ"  =  "1"  /* Dilation of Left Femoral Artery, Open Approach */
"047L34Z"  =  "1"  /* Dilate of L Femor Art with Drug-elut Intralum, Perc Approach */
"047L3DZ"  =  "1"  /* Dilation of L Femor Art with Intralum Dev, Perc Approach */
"047L3ZZ"  =  "1"  /* Dilation of Left Femoral Artery, Percutaneous Approach */
"047L44Z"  =  "1"  /* Dilate L Femor Art w Drug-elut Intralum, Perc Endo */
"047L4DZ"  =  "1"  /* Dilate of L Femor Art with Intralum Dev, Perc Endo Approach */
"047L4ZZ"  =  "1"  /* Dilation of Left Femoral Artery, Perc Endo Approach */
"047M04Z"  =  "1"  /* Dilate R Poplit Art w Drug-elut Intralum, Open */
"047M0DZ"  =  "1"  /* Dilation of R Poplit Art with Intralum Dev, Open Approach */
"047M0ZZ"  =  "1"  /* Dilation of Right Popliteal Artery, Open Approach */
"047M34Z"  =  "1"  /* Dilate R Poplit Art w Drug-elut Intralum, Perc */
"047M3DZ"  =  "1"  /* Dilation of R Poplit Art with Intralum Dev, Perc Approach */
"047M3ZZ"  =  "1"  /* Dilation of Right Popliteal Artery, Percutaneous Approach */
"047M44Z"  =  "1"  /* Dilate R Poplit Art w Drug-elut Intralum, Perc Endo */
"047M4DZ"  =  "1"  /* Dilate of R Poplit Art with Intralum Dev, Perc Endo Approach */
"047M4ZZ"  =  "1"  /* Dilation of Right Popliteal Artery, Perc Endo Approach */
"047N04Z"  =  "1"  /* Dilate L Poplit Art w Drug-elut Intralum, Open */
"047N0DZ"  =  "1"  /* Dilation of L Poplit Art with Intralum Dev, Open Approach */
"047N0ZZ"  =  "1"  /* Dilation of Left Popliteal Artery, Open Approach */
"047N34Z"  =  "1"  /* Dilate L Poplit Art w Drug-elut Intralum, Perc */
"047N3DZ"  =  "1"  /* Dilation of L Poplit Art with Intralum Dev, Perc Approach */
"047N3ZZ"  =  "1"  /* Dilation of Left Popliteal Artery, Percutaneous Approach */
"047N44Z"  =  "1"  /* Dilate L Poplit Art w Drug-elut Intralum, Perc Endo */
"047N4DZ"  =  "1"  /* Dilate of L Poplit Art with Intralum Dev, Perc Endo Approach */
"047N4ZZ"  =  "1"  /* Dilation of Left Popliteal Artery, Perc Endo Approach */
"047P04Z"  =  "1"  /* Dilate R Ant Tib Art w Drug-elut Intralum, Open */
"047P0DZ"  =  "1"  /* Dilation of R Ant Tib Art with Intralum Dev, Open Approach */
"047P0ZZ"  =  "1"  /* Dilation of Right Anterior Tibial Artery, Open Approach */
"047P34Z"  =  "1"  /* Dilate R Ant Tib Art w Drug-elut Intralum, Perc */
"047P3DZ"  =  "1"  /* Dilation of R Ant Tib Art with Intralum Dev, Perc Approach */
"047P3ZZ"  =  "1"  /* Dilation of Right Anterior Tibial Artery, Perc Approach */
"047P44Z"  =  "1"  /* Dilate R Ant Tib Art w Drug-elut Intralum, Perc Endo */
"047P4DZ"  =  "1"  /* Dilate R Ant Tib Art w Intralum Dev, Perc Endo */
"047P4ZZ"  =  "1"  /* Dilation of Right Anterior Tibial Artery, Perc Endo Approach */
"047Q04Z"  =  "1"  /* Dilate L Ant Tib Art w Drug-elut Intralum, Open */
"047Q0DZ"  =  "1"  /* Dilation of L Ant Tib Art with Intralum Dev, Open Approach */
"047Q0ZZ"  =  "1"  /* Dilation of Left Anterior Tibial Artery, Open Approach */
"047Q34Z"  =  "1"  /* Dilate L Ant Tib Art w Drug-elut Intralum, Perc */
"047Q3DZ"  =  "1"  /* Dilation of L Ant Tib Art with Intralum Dev, Perc Approach */
"047Q3ZZ"  =  "1"  /* Dilation of Left Anterior Tibial Artery, Perc Approach */
"047Q44Z"  =  "1"  /* Dilate L Ant Tib Art w Drug-elut Intralum, Perc Endo */
"047Q4DZ"  =  "1"  /* Dilate L Ant Tib Art w Intralum Dev, Perc Endo */
"047Q4ZZ"  =  "1"  /* Dilation of Left Anterior Tibial Artery, Perc Endo Approach */
"047R04Z"  =  "1"  /* Dilate R Post Tib Art w Drug-elut Intralum, Open */
"047R0DZ"  =  "1"  /* Dilation of R Post Tib Art with Intralum Dev, Open Approach */
"047R0ZZ"  =  "1"  /* Dilation of Right Posterior Tibial Artery, Open Approach */
"047R34Z"  =  "1"  /* Dilate R Post Tib Art w Drug-elut Intralum, Perc */
"047R3DZ"  =  "1"  /* Dilation of R Post Tib Art with Intralum Dev, Perc Approach */
"047R3ZZ"  =  "1"  /* Dilation of Right Posterior Tibial Artery, Perc Approach */
"047R44Z"  =  "1"  /* Dilate R Post Tib Art w Drug-elut Intralum, Perc Endo */
"047R4DZ"  =  "1"  /* Dilate R Post Tib Art w Intralum Dev, Perc Endo */
"047R4ZZ"  =  "1"  /* Dilation of R Post Tib Art, Perc Endo Approach */
"047S04Z"  =  "1"  /* Dilate L Post Tib Art w Drug-elut Intralum, Open */
"047S0DZ"  =  "1"  /* Dilation of L Post Tib Art with Intralum Dev, Open Approach */
"047S0ZZ"  =  "1"  /* Dilation of Left Posterior Tibial Artery, Open Approach */
"047S34Z"  =  "1"  /* Dilate L Post Tib Art w Drug-elut Intralum, Perc */
"047S3DZ"  =  "1"  /* Dilation of L Post Tib Art with Intralum Dev, Perc Approach */
"047S3ZZ"  =  "1"  /* Dilation of Left Posterior Tibial Artery, Perc Approach */
"047S44Z"  =  "1"  /* Dilate L Post Tib Art w Drug-elut Intralum, Perc Endo */
"047S4DZ"  =  "1"  /* Dilate L Post Tib Art w Intralum Dev, Perc Endo */
"047S4ZZ"  =  "1"  /* Dilation of Left Posterior Tibial Artery, Perc Endo Approach */
"047T04Z"  =  "1"  /* Dilate R Peroneal Art w Drug-elut Intralum, Open */
"047T0DZ"  =  "1"  /* Dilation of R Peroneal Art with Intralum Dev, Open Approach */
"047T0ZZ"  =  "1"  /* Dilation of Right Peroneal Artery, Open Approach */
"047T34Z"  =  "1"  /* Dilate R Peroneal Art w Drug-elut Intralum, Perc */
"047T3DZ"  =  "1"  /* Dilation of R Peroneal Art with Intralum Dev, Perc Approach */
"047T3ZZ"  =  "1"  /* Dilation of Right Peroneal Artery, Percutaneous Approach */
"047T44Z"  =  "1"  /* Dilate R Peroneal Art w Drug-elut Intralum, Perc Endo */
"047T4DZ"  =  "1"  /* Dilate R Peroneal Art w Intralum Dev, Perc Endo */
"047T4ZZ"  =  "1"  /* Dilation of Right Peroneal Artery, Perc Endo Approach */
"047U04Z"  =  "1"  /* Dilate L Peroneal Art w Drug-elut Intralum, Open */
"047U0DZ"  =  "1"  /* Dilation of L Peroneal Art with Intralum Dev, Open Approach */
"047U0ZZ"  =  "1"  /* Dilation of Left Peroneal Artery, Open Approach */
"047U34Z"  =  "1"  /* Dilate L Peroneal Art w Drug-elut Intralum, Perc */
"047U3DZ"  =  "1"  /* Dilation of L Peroneal Art with Intralum Dev, Perc Approach */
"047U3ZZ"  =  "1"  /* Dilation of Left Peroneal Artery, Percutaneous Approach */
"047U44Z"  =  "1"  /* Dilate L Peroneal Art w Drug-elut Intralum, Perc Endo */
"047U4DZ"  =  "1"  /* Dilate L Peroneal Art w Intralum Dev, Perc Endo */
"047U4ZZ"  =  "1"  /* Dilation of Left Peroneal Artery, Perc Endo Approach */
"047V04Z"  =  "1"  /* Dilate of R Foot Art with Drug-elut Intralum, Open Approach */
"047V0DZ"  =  "1"  /* Dilation of R Foot Art with Intralum Dev, Open Approach */
"047V0ZZ"  =  "1"  /* Dilation of Right Foot Artery, Open Approach */
"047V34Z"  =  "1"  /* Dilate of R Foot Art with Drug-elut Intralum, Perc Approach */
"047V3DZ"  =  "1"  /* Dilation of R Foot Art with Intralum Dev, Perc Approach */
"047V3ZZ"  =  "1"  /* Dilation of Right Foot Artery, Percutaneous Approach */
"047V44Z"  =  "1"  /* Dilate R Foot Art w Drug-elut Intralum, Perc Endo */
"047V4DZ"  =  "1"  /* Dilation of R Foot Art with Intralum Dev, Perc Endo Approach */
"047V4ZZ"  =  "1"  /* Dilation of Right Foot Artery, Perc Endo Approach */
"047W04Z"  =  "1"  /* Dilate of L Foot Art with Drug-elut Intralum, Open Approach */
"047W0DZ"  =  "1"  /* Dilation of L Foot Art with Intralum Dev, Open Approach */
"047W0ZZ"  =  "1"  /* Dilation of Left Foot Artery, Open Approach */
"047W34Z"  =  "1"  /* Dilate of L Foot Art with Drug-elut Intralum, Perc Approach */
"047W3DZ"  =  "1"  /* Dilation of L Foot Art with Intralum Dev, Perc Approach */
"047W3ZZ"  =  "1"  /* Dilation of Left Foot Artery, Percutaneous Approach */
"047W44Z"  =  "1"  /* Dilate L Foot Art w Drug-elut Intralum, Perc Endo */
"047W4DZ"  =  "1"  /* Dilation of L Foot Art with Intralum Dev, Perc Endo Approach */
"047W4ZZ"  =  "1"  /* Dilation of Left Foot Artery, Perc Endo Approach */
"047Y04Z"  =  "1"  /* Dilation of Low Art with Drug-elut Intralum, Open Approach */
"047Y0DZ"  =  "1"  /* Dilation of Lower Artery with Intralum Dev, Open Approach */
"047Y0ZZ"  =  "1"  /* Dilation of Lower Artery, Open Approach */
"047Y34Z"  =  "1"  /* Dilation of Low Art with Drug-elut Intralum, Perc Approach */
"047Y3DZ"  =  "1"  /* Dilation of Lower Artery with Intralum Dev, Perc Approach */
"047Y3ZZ"  =  "1"  /* Dilation of Lower Artery, Percutaneous Approach */
"047Y44Z"  =  "1"  /* Dilate Low Art w Drug-elut Intralum, Perc Endo */
"047Y4DZ"  =  "1"  /* Dilation of Low Art with Intralum Dev, Perc Endo Approach */
"047Y4ZZ"  =  "1"  /* Dilation of Lower Artery, Percutaneous Endoscopic Approach */
"04900ZX"  =  "1"  /* Drainage of Abdominal Aorta, Open Approach, Diagnostic */
"04903ZX"  =  "1"  /* Drainage of Abdominal Aorta, Percutaneous Approach, Diagn */
"04904ZX"  =  "1"  /* Drainage of Abdominal Aorta, Perc Endo Approach, Diagn */
"04910ZX"  =  "1"  /* Drainage of Celiac Artery, Open Approach, Diagnostic */
"04913ZX"  =  "1"  /* Drainage of Celiac Artery, Percutaneous Approach, Diagnostic */
"04914ZX"  =  "1"  /* Drainage of Celiac Artery, Perc Endo Approach, Diagn */
"04920ZX"  =  "1"  /* Drainage of Gastric Artery, Open Approach, Diagnostic */
"04923ZX"  =  "1"  /* Drainage of Gastric Artery, Percutaneous Approach, Diagn */
"04924ZX"  =  "1"  /* Drainage of Gastric Artery, Perc Endo Approach, Diagn */
"04930ZX"  =  "1"  /* Drainage of Hepatic Artery, Open Approach, Diagnostic */
"04933ZX"  =  "1"  /* Drainage of Hepatic Artery, Percutaneous Approach, Diagn */
"04934ZX"  =  "1"  /* Drainage of Hepatic Artery, Perc Endo Approach, Diagn */
"04940ZX"  =  "1"  /* Drainage of Splenic Artery, Open Approach, Diagnostic */
"04943ZX"  =  "1"  /* Drainage of Splenic Artery, Percutaneous Approach, Diagn */
"04944ZX"  =  "1"  /* Drainage of Splenic Artery, Perc Endo Approach, Diagn */
"04950ZX"  =  "1"  /* Drainage of Superior Mesenteric Artery, Open Approach, Diagn */
"04953ZX"  =  "1"  /* Drainage of Superior Mesenteric Artery, Perc Approach, Diagn */
"04954ZX"  =  "1"  /* Drainage of Sup Mesent Art, Perc Endo Approach, Diagn */
"04960ZX"  =  "1"  /* Drainage of Right Colic Artery, Open Approach, Diagnostic */
"04963ZX"  =  "1"  /* Drainage of Right Colic Artery, Percutaneous Approach, Diagn */
"04964ZX"  =  "1"  /* Drainage of Right Colic Artery, Perc Endo Approach, Diagn */
"04970ZX"  =  "1"  /* Drainage of Left Colic Artery, Open Approach, Diagnostic */
"04973ZX"  =  "1"  /* Drainage of Left Colic Artery, Percutaneous Approach, Diagn */
"04974ZX"  =  "1"  /* Drainage of Left Colic Artery, Perc Endo Approach, Diagn */
"04980ZX"  =  "1"  /* Drainage of Middle Colic Artery, Open Approach, Diagnostic */
"04983ZX"  =  "1"  /* Drainage of Middle Colic Artery, Perc Approach, Diagn */
"04984ZX"  =  "1"  /* Drainage of Middle Colic Artery, Perc Endo Approach, Diagn */
"04990ZX"  =  "1"  /* Drainage of Right Renal Artery, Open Approach, Diagnostic */
"04993ZX"  =  "1"  /* Drainage of Right Renal Artery, Percutaneous Approach, Diagn */
"04994ZX"  =  "1"  /* Drainage of Right Renal Artery, Perc Endo Approach, Diagn */
"049A0ZX"  =  "1"  /* Drainage of Left Renal Artery, Open Approach, Diagnostic */
"049A3ZX"  =  "1"  /* Drainage of Left Renal Artery, Percutaneous Approach, Diagn */
"049A4ZX"  =  "1"  /* Drainage of Left Renal Artery, Perc Endo Approach, Diagn */
"049B0ZX"  =  "1"  /* Drainage of Inferior Mesenteric Artery, Open Approach, Diagn */
"049B3ZX"  =  "1"  /* Drainage of Inferior Mesenteric Artery, Perc Approach, Diagn */
"049B4ZX"  =  "1"  /* Drainage of Inf Mesent Art, Perc Endo Approach, Diagn */
"049C0ZX"  =  "1"  /* Drainage of Right Common Iliac Artery, Open Approach, Diagn */
"049C3ZX"  =  "1"  /* Drainage of Right Common Iliac Artery, Perc Approach, Diagn */
"049C4ZX"  =  "1"  /* Drainage of R Com Iliac Art, Perc Endo Approach, Diagn */
"049D0ZX"  =  "1"  /* Drainage of Left Common Iliac Artery, Open Approach, Diagn */
"049D3ZX"  =  "1"  /* Drainage of Left Common Iliac Artery, Perc Approach, Diagn */
"049D4ZX"  =  "1"  /* Drainage of L Com Iliac Art, Perc Endo Approach, Diagn */
"049E0ZX"  =  "1"  /* Drainage of R Int Iliac Art, Open Approach, Diagn */
"049E3ZX"  =  "1"  /* Drainage of R Int Iliac Art, Perc Approach, Diagn */
"049E4ZX"  =  "1"  /* Drainage of R Int Iliac Art, Perc Endo Approach, Diagn */
"049F0ZX"  =  "1"  /* Drainage of Left Internal Iliac Artery, Open Approach, Diagn */
"049F3ZX"  =  "1"  /* Drainage of Left Internal Iliac Artery, Perc Approach, Diagn */
"049F4ZX"  =  "1"  /* Drainage of L Int Iliac Art, Perc Endo Approach, Diagn */
"049H0ZX"  =  "1"  /* Drainage of R Ext Iliac Art, Open Approach, Diagn */
"049H3ZX"  =  "1"  /* Drainage of R Ext Iliac Art, Perc Approach, Diagn */
"049H4ZX"  =  "1"  /* Drainage of R Ext Iliac Art, Perc Endo Approach, Diagn */
"049J0ZX"  =  "1"  /* Drainage of Left External Iliac Artery, Open Approach, Diagn */
"049J3ZX"  =  "1"  /* Drainage of Left External Iliac Artery, Perc Approach, Diagn */
"049J4ZX"  =  "1"  /* Drainage of L Ext Iliac Art, Perc Endo Approach, Diagn */
"049K0ZX"  =  "1"  /* Drainage of Right Femoral Artery, Open Approach, Diagnostic */
"049K3ZX"  =  "1"  /* Drainage of Right Femoral Artery, Perc Approach, Diagn */
"049K4ZX"  =  "1"  /* Drainage of Right Femoral Artery, Perc Endo Approach, Diagn */
"049L0ZX"  =  "1"  /* Drainage of Left Femoral Artery, Open Approach, Diagnostic */
"049L3ZX"  =  "1"  /* Drainage of Left Femoral Artery, Perc Approach, Diagn */
"049L4ZX"  =  "1"  /* Drainage of Left Femoral Artery, Perc Endo Approach, Diagn */
"049M0ZX"  =  "1"  /* Drainage of Right Popliteal Artery, Open Approach, Diagn */
"049M3ZX"  =  "1"  /* Drainage of Right Popliteal Artery, Perc Approach, Diagn */
"049M4ZX"  =  "1"  /* Drainage of R Poplit Art, Perc Endo Approach, Diagn */
"049N0ZX"  =  "1"  /* Drainage of Left Popliteal Artery, Open Approach, Diagnostic */
"049N3ZX"  =  "1"  /* Drainage of Left Popliteal Artery, Perc Approach, Diagn */
"049N4ZX"  =  "1"  /* Drainage of Left Popliteal Artery, Perc Endo Approach, Diagn */
"049P0ZX"  =  "1"  /* Drainage of R Ant Tib Art, Open Approach, Diagn */
"049P3ZX"  =  "1"  /* Drainage of R Ant Tib Art, Perc Approach, Diagn */
"049P4ZX"  =  "1"  /* Drainage of R Ant Tib Art, Perc Endo Approach, Diagn */
"049Q0ZX"  =  "1"  /* Drainage of L Ant Tib Art, Open Approach, Diagn */
"049Q3ZX"  =  "1"  /* Drainage of L Ant Tib Art, Perc Approach, Diagn */
"049Q4ZX"  =  "1"  /* Drainage of L Ant Tib Art, Perc Endo Approach, Diagn */
"049R0ZX"  =  "1"  /* Drainage of R Post Tib Art, Open Approach, Diagn */
"049R3ZX"  =  "1"  /* Drainage of R Post Tib Art, Perc Approach, Diagn */
"049R4ZX"  =  "1"  /* Drainage of R Post Tib Art, Perc Endo Approach, Diagn */
"049S0ZX"  =  "1"  /* Drainage of L Post Tib Art, Open Approach, Diagn */
"049S3ZX"  =  "1"  /* Drainage of L Post Tib Art, Perc Approach, Diagn */
"049S4ZX"  =  "1"  /* Drainage of L Post Tib Art, Perc Endo Approach, Diagn */
"049T0ZX"  =  "1"  /* Drainage of Right Peroneal Artery, Open Approach, Diagnostic */
"049T3ZX"  =  "1"  /* Drainage of Right Peroneal Artery, Perc Approach, Diagn */
"049T4ZX"  =  "1"  /* Drainage of Right Peroneal Artery, Perc Endo Approach, Diagn */
"049U0ZX"  =  "1"  /* Drainage of Left Peroneal Artery, Open Approach, Diagnostic */
"049U3ZX"  =  "1"  /* Drainage of Left Peroneal Artery, Perc Approach, Diagn */
"049U4ZX"  =  "1"  /* Drainage of Left Peroneal Artery, Perc Endo Approach, Diagn */
"049V0ZX"  =  "1"  /* Drainage of Right Foot Artery, Open Approach, Diagnostic */
"049V3ZX"  =  "1"  /* Drainage of Right Foot Artery, Percutaneous Approach, Diagn */
"049V4ZX"  =  "1"  /* Drainage of Right Foot Artery, Perc Endo Approach, Diagn */
"049W0ZX"  =  "1"  /* Drainage of Left Foot Artery, Open Approach, Diagnostic */
"049W3ZX"  =  "1"  /* Drainage of Left Foot Artery, Percutaneous Approach, Diagn */
"049W4ZX"  =  "1"  /* Drainage of Left Foot Artery, Perc Endo Approach, Diagn */
"049Y0ZX"  =  "1"  /* Drainage of Lower Artery, Open Approach, Diagnostic */
"049Y3ZX"  =  "1"  /* Drainage of Lower Artery, Percutaneous Approach, Diagnostic */
"049Y4ZX"  =  "1"  /* Drainage of Lower Artery, Perc Endo Approach, Diagn */
"04B00ZX"  =  "1"  /* Excision of Abdominal Aorta, Open Approach, Diagnostic */
"04B00ZZ"  =  "1"  /* Excision of Abdominal Aorta, Open Approach */
"04B03ZX"  =  "1"  /* Excision of Abdominal Aorta, Percutaneous Approach, Diagn */
"04B03ZZ"  =  "1"  /* Excision of Abdominal Aorta, Percutaneous Approach */
"04B04ZX"  =  "1"  /* Excision of Abdominal Aorta, Perc Endo Approach, Diagn */
"04B04ZZ"  =  "1"  /* Excision of Abdominal Aorta, Perc Endo Approach */
"04B10ZX"  =  "1"  /* Excision of Celiac Artery, Open Approach, Diagnostic */
"04B10ZZ"  =  "1"  /* Excision of Celiac Artery, Open Approach */
"04B13ZX"  =  "1"  /* Excision of Celiac Artery, Percutaneous Approach, Diagnostic */
"04B13ZZ"  =  "1"  /* Excision of Celiac Artery, Percutaneous Approach */
"04B14ZX"  =  "1"  /* Excision of Celiac Artery, Perc Endo Approach, Diagn */
"04B14ZZ"  =  "1"  /* Excision of Celiac Artery, Percutaneous Endoscopic Approach */
"04B20ZX"  =  "1"  /* Excision of Gastric Artery, Open Approach, Diagnostic */
"04B20ZZ"  =  "1"  /* Excision of Gastric Artery, Open Approach */
"04B23ZX"  =  "1"  /* Excision of Gastric Artery, Percutaneous Approach, Diagn */
"04B23ZZ"  =  "1"  /* Excision of Gastric Artery, Percutaneous Approach */
"04B24ZX"  =  "1"  /* Excision of Gastric Artery, Perc Endo Approach, Diagn */
"04B24ZZ"  =  "1"  /* Excision of Gastric Artery, Percutaneous Endoscopic Approach */
"04B30ZX"  =  "1"  /* Excision of Hepatic Artery, Open Approach, Diagnostic */
"04B30ZZ"  =  "1"  /* Excision of Hepatic Artery, Open Approach */
"04B33ZX"  =  "1"  /* Excision of Hepatic Artery, Percutaneous Approach, Diagn */
"04B33ZZ"  =  "1"  /* Excision of Hepatic Artery, Percutaneous Approach */
"04B34ZX"  =  "1"  /* Excision of Hepatic Artery, Perc Endo Approach, Diagn */
"04B34ZZ"  =  "1"  /* Excision of Hepatic Artery, Percutaneous Endoscopic Approach */
"04B40ZX"  =  "1"  /* Excision of Splenic Artery, Open Approach, Diagnostic */
"04B40ZZ"  =  "1"  /* Excision of Splenic Artery, Open Approach */
"04B43ZX"  =  "1"  /* Excision of Splenic Artery, Percutaneous Approach, Diagn */
"04B43ZZ"  =  "1"  /* Excision of Splenic Artery, Percutaneous Approach */
"04B44ZX"  =  "1"  /* Excision of Splenic Artery, Perc Endo Approach, Diagn */
"04B44ZZ"  =  "1"  /* Excision of Splenic Artery, Percutaneous Endoscopic Approach */
"04B50ZX"  =  "1"  /* Excision of Superior Mesenteric Artery, Open Approach, Diagn */
"04B50ZZ"  =  "1"  /* Excision of Superior Mesenteric Artery, Open Approach */
"04B53ZX"  =  "1"  /* Excision of Superior Mesenteric Artery, Perc Approach, Diagn */
"04B53ZZ"  =  "1"  /* Excision of Superior Mesenteric Artery, Perc Approach */
"04B54ZX"  =  "1"  /* Excision of Sup Mesent Art, Perc Endo Approach, Diagn */
"04B54ZZ"  =  "1"  /* Excision of Superior Mesenteric Artery, Perc Endo Approach */
"04B60ZX"  =  "1"  /* Excision of Right Colic Artery, Open Approach, Diagnostic */
"04B60ZZ"  =  "1"  /* Excision of Right Colic Artery, Open Approach */
"04B63ZX"  =  "1"  /* Excision of Right Colic Artery, Percutaneous Approach, Diagn */
"04B63ZZ"  =  "1"  /* Excision of Right Colic Artery, Percutaneous Approach */
"04B64ZX"  =  "1"  /* Excision of Right Colic Artery, Perc Endo Approach, Diagn */
"04B64ZZ"  =  "1"  /* Excision of Right Colic Artery, Perc Endo Approach */
"04B70ZX"  =  "1"  /* Excision of Left Colic Artery, Open Approach, Diagnostic */
"04B70ZZ"  =  "1"  /* Excision of Left Colic Artery, Open Approach */
"04B73ZX"  =  "1"  /* Excision of Left Colic Artery, Percutaneous Approach, Diagn */
"04B73ZZ"  =  "1"  /* Excision of Left Colic Artery, Percutaneous Approach */
"04B74ZX"  =  "1"  /* Excision of Left Colic Artery, Perc Endo Approach, Diagn */
"04B74ZZ"  =  "1"  /* Excision of Left Colic Artery, Perc Endo Approach */
"04B80ZX"  =  "1"  /* Excision of Middle Colic Artery, Open Approach, Diagnostic */
"04B80ZZ"  =  "1"  /* Excision of Middle Colic Artery, Open Approach */
"04B83ZX"  =  "1"  /* Excision of Middle Colic Artery, Perc Approach, Diagn */
"04B83ZZ"  =  "1"  /* Excision of Middle Colic Artery, Percutaneous Approach */
"04B84ZX"  =  "1"  /* Excision of Middle Colic Artery, Perc Endo Approach, Diagn */
"04B84ZZ"  =  "1"  /* Excision of Middle Colic Artery, Perc Endo Approach */
"04B90ZX"  =  "1"  /* Excision of Right Renal Artery, Open Approach, Diagnostic */
"04B90ZZ"  =  "1"  /* Excision of Right Renal Artery, Open Approach */
"04B93ZX"  =  "1"  /* Excision of Right Renal Artery, Percutaneous Approach, Diagn */
"04B93ZZ"  =  "1"  /* Excision of Right Renal Artery, Percutaneous Approach */
"04B94ZX"  =  "1"  /* Excision of Right Renal Artery, Perc Endo Approach, Diagn */
"04B94ZZ"  =  "1"  /* Excision of Right Renal Artery, Perc Endo Approach */
"04BA0ZX"  =  "1"  /* Excision of Left Renal Artery, Open Approach, Diagnostic */
"04BA0ZZ"  =  "1"  /* Excision of Left Renal Artery, Open Approach */
"04BA3ZX"  =  "1"  /* Excision of Left Renal Artery, Percutaneous Approach, Diagn */
"04BA3ZZ"  =  "1"  /* Excision of Left Renal Artery, Percutaneous Approach */
"04BA4ZX"  =  "1"  /* Excision of Left Renal Artery, Perc Endo Approach, Diagn */
"04BA4ZZ"  =  "1"  /* Excision of Left Renal Artery, Perc Endo Approach */
"04BB0ZX"  =  "1"  /* Excision of Inferior Mesenteric Artery, Open Approach, Diagn */
"04BB0ZZ"  =  "1"  /* Excision of Inferior Mesenteric Artery, Open Approach */
"04BB3ZX"  =  "1"  /* Excision of Inferior Mesenteric Artery, Perc Approach, Diagn */
"04BB3ZZ"  =  "1"  /* Excision of Inferior Mesenteric Artery, Perc Approach */
"04BB4ZX"  =  "1"  /* Excision of Inf Mesent Art, Perc Endo Approach, Diagn */
"04BB4ZZ"  =  "1"  /* Excision of Inferior Mesenteric Artery, Perc Endo Approach */
"04BC0ZX"  =  "1"  /* Excision of Right Common Iliac Artery, Open Approach, Diagn */
"04BC0ZZ"  =  "1"  /* Excision of Right Common Iliac Artery, Open Approach */
"04BC3ZX"  =  "1"  /* Excision of Right Common Iliac Artery, Perc Approach, Diagn */
"04BC3ZZ"  =  "1"  /* Excision of Right Common Iliac Artery, Percutaneous Approach */
"04BC4ZX"  =  "1"  /* Excision of R Com Iliac Art, Perc Endo Approach, Diagn */
"04BC4ZZ"  =  "1"  /* Excision of Right Common Iliac Artery, Perc Endo Approach */
"04BD0ZX"  =  "1"  /* Excision of Left Common Iliac Artery, Open Approach, Diagn */
"04BD0ZZ"  =  "1"  /* Excision of Left Common Iliac Artery, Open Approach */
"04BD3ZX"  =  "1"  /* Excision of Left Common Iliac Artery, Perc Approach, Diagn */
"04BD3ZZ"  =  "1"  /* Excision of Left Common Iliac Artery, Percutaneous Approach */
"04BD4ZX"  =  "1"  /* Excision of L Com Iliac Art, Perc Endo Approach, Diagn */
"04BD4ZZ"  =  "1"  /* Excision of Left Common Iliac Artery, Perc Endo Approach */
"04BE0ZX"  =  "1"  /* Excision of R Int Iliac Art, Open Approach, Diagn */
"04BE0ZZ"  =  "1"  /* Excision of Right Internal Iliac Artery, Open Approach */
"04BE3ZX"  =  "1"  /* Excision of R Int Iliac Art, Perc Approach, Diagn */
"04BE3ZZ"  =  "1"  /* Excision of Right Internal Iliac Artery, Perc Approach */
"04BE4ZX"  =  "1"  /* Excision of R Int Iliac Art, Perc Endo Approach, Diagn */
"04BE4ZZ"  =  "1"  /* Excision of Right Internal Iliac Artery, Perc Endo Approach */
"04BF0ZX"  =  "1"  /* Excision of Left Internal Iliac Artery, Open Approach, Diagn */
"04BF0ZZ"  =  "1"  /* Excision of Left Internal Iliac Artery, Open Approach */
"04BF3ZX"  =  "1"  /* Excision of Left Internal Iliac Artery, Perc Approach, Diagn */
"04BF3ZZ"  =  "1"  /* Excision of Left Internal Iliac Artery, Perc Approach */
"04BF4ZX"  =  "1"  /* Excision of L Int Iliac Art, Perc Endo Approach, Diagn */
"04BF4ZZ"  =  "1"  /* Excision of Left Internal Iliac Artery, Perc Endo Approach */
"04BH0ZX"  =  "1"  /* Excision of R Ext Iliac Art, Open Approach, Diagn */
"04BH0ZZ"  =  "1"  /* Excision of Right External Iliac Artery, Open Approach */
"04BH3ZX"  =  "1"  /* Excision of R Ext Iliac Art, Perc Approach, Diagn */
"04BH3ZZ"  =  "1"  /* Excision of Right External Iliac Artery, Perc Approach */
"04BH4ZX"  =  "1"  /* Excision of R Ext Iliac Art, Perc Endo Approach, Diagn */
"04BH4ZZ"  =  "1"  /* Excision of Right External Iliac Artery, Perc Endo Approach */
"04BJ0ZX"  =  "1"  /* Excision of Left External Iliac Artery, Open Approach, Diagn */
"04BJ0ZZ"  =  "1"  /* Excision of Left External Iliac Artery, Open Approach */
"04BJ3ZX"  =  "1"  /* Excision of Left External Iliac Artery, Perc Approach, Diagn */
"04BJ3ZZ"  =  "1"  /* Excision of Left External Iliac Artery, Perc Approach */
"04BJ4ZX"  =  "1"  /* Excision of L Ext Iliac Art, Perc Endo Approach, Diagn */
"04BJ4ZZ"  =  "1"  /* Excision of Left External Iliac Artery, Perc Endo Approach */
"04BK0ZX"  =  "1"  /* Excision of Right Femoral Artery, Open Approach, Diagnostic */
"04BK0ZZ"  =  "1"  /* Excision of Right Femoral Artery, Open Approach */
"04BK3ZX"  =  "1"  /* Excision of Right Femoral Artery, Perc Approach, Diagn */
"04BK3ZZ"  =  "1"  /* Excision of Right Femoral Artery, Percutaneous Approach */
"04BK4ZX"  =  "1"  /* Excision of Right Femoral Artery, Perc Endo Approach, Diagn */
"04BK4ZZ"  =  "1"  /* Excision of Right Femoral Artery, Perc Endo Approach */
"04BL0ZX"  =  "1"  /* Excision of Left Femoral Artery, Open Approach, Diagnostic */
"04BL0ZZ"  =  "1"  /* Excision of Left Femoral Artery, Open Approach */
"04BL3ZX"  =  "1"  /* Excision of Left Femoral Artery, Perc Approach, Diagn */
"04BL3ZZ"  =  "1"  /* Excision of Left Femoral Artery, Percutaneous Approach */
"04BL4ZX"  =  "1"  /* Excision of Left Femoral Artery, Perc Endo Approach, Diagn */
"04BL4ZZ"  =  "1"  /* Excision of Left Femoral Artery, Perc Endo Approach */
"04BM0ZX"  =  "1"  /* Excision of Right Popliteal Artery, Open Approach, Diagn */
"04BM0ZZ"  =  "1"  /* Excision of Right Popliteal Artery, Open Approach */
"04BM3ZX"  =  "1"  /* Excision of Right Popliteal Artery, Perc Approach, Diagn */
"04BM3ZZ"  =  "1"  /* Excision of Right Popliteal Artery, Percutaneous Approach */
"04BM4ZX"  =  "1"  /* Excision of R Poplit Art, Perc Endo Approach, Diagn */
"04BM4ZZ"  =  "1"  /* Excision of Right Popliteal Artery, Perc Endo Approach */
"04BN0ZX"  =  "1"  /* Excision of Left Popliteal Artery, Open Approach, Diagnostic */
"04BN0ZZ"  =  "1"  /* Excision of Left Popliteal Artery, Open Approach */
"04BN3ZX"  =  "1"  /* Excision of Left Popliteal Artery, Perc Approach, Diagn */
"04BN3ZZ"  =  "1"  /* Excision of Left Popliteal Artery, Percutaneous Approach */
"04BN4ZX"  =  "1"  /* Excision of Left Popliteal Artery, Perc Endo Approach, Diagn */
"04BN4ZZ"  =  "1"  /* Excision of Left Popliteal Artery, Perc Endo Approach */
"04BP0ZX"  =  "1"  /* Excision of R Ant Tib Art, Open Approach, Diagn */
"04BP0ZZ"  =  "1"  /* Excision of Right Anterior Tibial Artery, Open Approach */
"04BP3ZX"  =  "1"  /* Excision of R Ant Tib Art, Perc Approach, Diagn */
"04BP3ZZ"  =  "1"  /* Excision of Right Anterior Tibial Artery, Perc Approach */
"04BP4ZX"  =  "1"  /* Excision of R Ant Tib Art, Perc Endo Approach, Diagn */
"04BP4ZZ"  =  "1"  /* Excision of Right Anterior Tibial Artery, Perc Endo Approach */
"04BQ0ZX"  =  "1"  /* Excision of L Ant Tib Art, Open Approach, Diagn */
"04BQ0ZZ"  =  "1"  /* Excision of Left Anterior Tibial Artery, Open Approach */
"04BQ3ZX"  =  "1"  /* Excision of L Ant Tib Art, Perc Approach, Diagn */
"04BQ3ZZ"  =  "1"  /* Excision of Left Anterior Tibial Artery, Perc Approach */
"04BQ4ZX"  =  "1"  /* Excision of L Ant Tib Art, Perc Endo Approach, Diagn */
"04BQ4ZZ"  =  "1"  /* Excision of Left Anterior Tibial Artery, Perc Endo Approach */
"04BR0ZX"  =  "1"  /* Excision of R Post Tib Art, Open Approach, Diagn */
"04BR0ZZ"  =  "1"  /* Excision of Right Posterior Tibial Artery, Open Approach */
"04BR3ZX"  =  "1"  /* Excision of R Post Tib Art, Perc Approach, Diagn */
"04BR3ZZ"  =  "1"  /* Excision of Right Posterior Tibial Artery, Perc Approach */
"04BR4ZX"  =  "1"  /* Excision of R Post Tib Art, Perc Endo Approach, Diagn */
"04BR4ZZ"  =  "1"  /* Excision of R Post Tib Art, Perc Endo Approach */
"04BS0ZX"  =  "1"  /* Excision of L Post Tib Art, Open Approach, Diagn */
"04BS0ZZ"  =  "1"  /* Excision of Left Posterior Tibial Artery, Open Approach */
"04BS3ZX"  =  "1"  /* Excision of L Post Tib Art, Perc Approach, Diagn */
"04BS3ZZ"  =  "1"  /* Excision of Left Posterior Tibial Artery, Perc Approach */
"04BS4ZX"  =  "1"  /* Excision of L Post Tib Art, Perc Endo Approach, Diagn */
"04BS4ZZ"  =  "1"  /* Excision of Left Posterior Tibial Artery, Perc Endo Approach */
"04BT0ZX"  =  "1"  /* Excision of Right Peroneal Artery, Open Approach, Diagnostic */
"04BT0ZZ"  =  "1"  /* Excision of Right Peroneal Artery, Open Approach */
"04BT3ZX"  =  "1"  /* Excision of Right Peroneal Artery, Perc Approach, Diagn */
"04BT3ZZ"  =  "1"  /* Excision of Right Peroneal Artery, Percutaneous Approach */
"04BT4ZX"  =  "1"  /* Excision of Right Peroneal Artery, Perc Endo Approach, Diagn */
"04BT4ZZ"  =  "1"  /* Excision of Right Peroneal Artery, Perc Endo Approach */
"04BU0ZX"  =  "1"  /* Excision of Left Peroneal Artery, Open Approach, Diagnostic */
"04BU0ZZ"  =  "1"  /* Excision of Left Peroneal Artery, Open Approach */
"04BU3ZX"  =  "1"  /* Excision of Left Peroneal Artery, Perc Approach, Diagn */
"04BU3ZZ"  =  "1"  /* Excision of Left Peroneal Artery, Percutaneous Approach */
"04BU4ZX"  =  "1"  /* Excision of Left Peroneal Artery, Perc Endo Approach, Diagn */
"04BU4ZZ"  =  "1"  /* Excision of Left Peroneal Artery, Perc Endo Approach */
"04BV0ZX"  =  "1"  /* Excision of Right Foot Artery, Open Approach, Diagnostic */
"04BV0ZZ"  =  "1"  /* Excision of Right Foot Artery, Open Approach */
"04BV3ZX"  =  "1"  /* Excision of Right Foot Artery, Percutaneous Approach, Diagn */
"04BV3ZZ"  =  "1"  /* Excision of Right Foot Artery, Percutaneous Approach */
"04BV4ZX"  =  "1"  /* Excision of Right Foot Artery, Perc Endo Approach, Diagn */
"04BV4ZZ"  =  "1"  /* Excision of Right Foot Artery, Perc Endo Approach */
"04BW0ZX"  =  "1"  /* Excision of Left Foot Artery, Open Approach, Diagnostic */
"04BW0ZZ"  =  "1"  /* Excision of Left Foot Artery, Open Approach */
"04BW3ZX"  =  "1"  /* Excision of Left Foot Artery, Percutaneous Approach, Diagn */
"04BW3ZZ"  =  "1"  /* Excision of Left Foot Artery, Percutaneous Approach */
"04BW4ZX"  =  "1"  /* Excision of Left Foot Artery, Perc Endo Approach, Diagn */
"04BW4ZZ"  =  "1"  /* Excision of Left Foot Artery, Perc Endo Approach */
"04BY0ZX"  =  "1"  /* Excision of Lower Artery, Open Approach, Diagnostic */
"04BY0ZZ"  =  "1"  /* Excision of Lower Artery, Open Approach */
"04BY3ZX"  =  "1"  /* Excision of Lower Artery, Percutaneous Approach, Diagnostic */
"04BY3ZZ"  =  "1"  /* Excision of Lower Artery, Percutaneous Approach */
"04BY4ZX"  =  "1"  /* Excision of Lower Artery, Perc Endo Approach, Diagn */
"04BY4ZZ"  =  "1"  /* Excision of Lower Artery, Percutaneous Endoscopic Approach */
"04C00ZZ"  =  "1"  /* Extirpation of Matter from Abdominal Aorta, Open Approach */
"04C03ZZ"  =  "1"  /* Extirpation of Matter from Abdominal Aorta, Perc Approach */
"04C04ZZ"  =  "1"  /* Extirpation of Matter from Abd Aorta, Perc Endo Approach */
"04C10ZZ"  =  "1"  /* Extirpation of Matter from Celiac Artery, Open Approach */
"04C13ZZ"  =  "1"  /* Extirpation of Matter from Celiac Artery, Perc Approach */
"04C14ZZ"  =  "1"  /* Extirpation of Matter from Celiac Artery, Perc Endo Approach */
"04C20ZZ"  =  "1"  /* Extirpation of Matter from Gastric Artery, Open Approach */
"04C23ZZ"  =  "1"  /* Extirpation of Matter from Gastric Artery, Perc Approach */
"04C24ZZ"  =  "1"  /* Extirpation of Matter from Gastric Art, Perc Endo Approach */
"04C30ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Artery, Open Approach */
"04C33ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Artery, Perc Approach */
"04C34ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Art, Perc Endo Approach */
"04C40ZZ"  =  "1"  /* Extirpation of Matter from Splenic Artery, Open Approach */
"04C43ZZ"  =  "1"  /* Extirpation of Matter from Splenic Artery, Perc Approach */
"04C44ZZ"  =  "1"  /* Extirpation of Matter from Splenic Art, Perc Endo Approach */
"04C50ZZ"  =  "1"  /* Extirpation of Matter from Sup Mesent Art, Open Approach */
"04C53ZZ"  =  "1"  /* Extirpation of Matter from Sup Mesent Art, Perc Approach */
"04C54ZZ"  =  "1"  /* Extirpate of Matter from Sup Mesent Art, Perc Endo Approach */
"04C60ZZ"  =  "1"  /* Extirpation of Matter from Right Colic Artery, Open Approach */
"04C63ZZ"  =  "1"  /* Extirpation of Matter from Right Colic Artery, Perc Approach */
"04C64ZZ"  =  "1"  /* Extirpation of Matter from R Colic Art, Perc Endo Approach */
"04C70ZZ"  =  "1"  /* Extirpation of Matter from Left Colic Artery, Open Approach */
"04C73ZZ"  =  "1"  /* Extirpation of Matter from Left Colic Artery, Perc Approach */
"04C74ZZ"  =  "1"  /* Extirpation of Matter from L Colic Art, Perc Endo Approach */
"04C80ZZ"  =  "1"  /* Extirpation of Matter from Mid Colic Art, Open Approach */
"04C83ZZ"  =  "1"  /* Extirpation of Matter from Mid Colic Art, Perc Approach */
"04C84ZZ"  =  "1"  /* Extirpation of Matter from Mid Colic Art, Perc Endo Approach */
"04C90ZZ"  =  "1"  /* Extirpation of Matter from Right Renal Artery, Open Approach */
"04C93ZZ"  =  "1"  /* Extirpation of Matter from Right Renal Artery, Perc Approach */
"04C94ZZ"  =  "1"  /* Extirpation of Matter from R Renal Art, Perc Endo Approach */
"04CA0ZZ"  =  "1"  /* Extirpation of Matter from Left Renal Artery, Open Approach */
"04CA3ZZ"  =  "1"  /* Extirpation of Matter from Left Renal Artery, Perc Approach */
"04CA4ZZ"  =  "1"  /* Extirpation of Matter from L Renal Art, Perc Endo Approach */
"04CB0ZZ"  =  "1"  /* Extirpation of Matter from Inf Mesent Art, Open Approach */
"04CB3ZZ"  =  "1"  /* Extirpation of Matter from Inf Mesent Art, Perc Approach */
"04CB4ZZ"  =  "1"  /* Extirpate of Matter from Inf Mesent Art, Perc Endo Approach */
"04CC0ZZ"  =  "1"  /* Extirpation of Matter from R Com Iliac Art, Open Approach */
"04CC3ZZ"  =  "1"  /* Extirpation of Matter from R Com Iliac Art, Perc Approach */
"04CC4ZZ"  =  "1"  /* Extirpate of Matter from R Com Iliac Art, Perc Endo Approach */
"04CD0ZZ"  =  "1"  /* Extirpation of Matter from L Com Iliac Art, Open Approach */
"04CD3ZZ"  =  "1"  /* Extirpation of Matter from L Com Iliac Art, Perc Approach */
"04CD4ZZ"  =  "1"  /* Extirpate of Matter from L Com Iliac Art, Perc Endo Approach */
"04CE0ZZ"  =  "1"  /* Extirpation of Matter from R Int Iliac Art, Open Approach */
"04CE3ZZ"  =  "1"  /* Extirpation of Matter from R Int Iliac Art, Perc Approach */
"04CE4ZZ"  =  "1"  /* Extirpate of Matter from R Int Iliac Art, Perc Endo Approach */
"04CF0ZZ"  =  "1"  /* Extirpation of Matter from L Int Iliac Art, Open Approach */
"04CF3ZZ"  =  "1"  /* Extirpation of Matter from L Int Iliac Art, Perc Approach */
"04CF4ZZ"  =  "1"  /* Extirpate of Matter from L Int Iliac Art, Perc Endo Approach */
"04CH0ZZ"  =  "1"  /* Extirpation of Matter from R Ext Iliac Art, Open Approach */
"04CH3ZZ"  =  "1"  /* Extirpation of Matter from R Ext Iliac Art, Perc Approach */
"04CH4ZZ"  =  "1"  /* Extirpate of Matter from R Ext Iliac Art, Perc Endo Approach */
"04CJ0ZZ"  =  "1"  /* Extirpation of Matter from L Ext Iliac Art, Open Approach */
"04CJ3ZZ"  =  "1"  /* Extirpation of Matter from L Ext Iliac Art, Perc Approach */
"04CJ4ZZ"  =  "1"  /* Extirpate of Matter from L Ext Iliac Art, Perc Endo Approach */
"04CK0ZZ"  =  "1"  /* Extirpation of Matter from R Femor Art, Open Approach */
"04CK3ZZ"  =  "1"  /* Extirpation of Matter from R Femor Art, Perc Approach */
"04CK4ZZ"  =  "1"  /* Extirpation of Matter from R Femor Art, Perc Endo Approach */
"04CL0ZZ"  =  "1"  /* Extirpation of Matter from L Femor Art, Open Approach */
"04CL3ZZ"  =  "1"  /* Extirpation of Matter from L Femor Art, Perc Approach */
"04CL4ZZ"  =  "1"  /* Extirpation of Matter from L Femor Art, Perc Endo Approach */
"04CM0ZZ"  =  "1"  /* Extirpation of Matter from R Poplit Art, Open Approach */
"04CM3ZZ"  =  "1"  /* Extirpation of Matter from R Poplit Art, Perc Approach */
"04CM4ZZ"  =  "1"  /* Extirpation of Matter from R Poplit Art, Perc Endo Approach */
"04CN0ZZ"  =  "1"  /* Extirpation of Matter from L Poplit Art, Open Approach */
"04CN3ZZ"  =  "1"  /* Extirpation of Matter from L Poplit Art, Perc Approach */
"04CN4ZZ"  =  "1"  /* Extirpation of Matter from L Poplit Art, Perc Endo Approach */
"04CP0ZZ"  =  "1"  /* Extirpation of Matter from R Ant Tib Art, Open Approach */
"04CP3ZZ"  =  "1"  /* Extirpation of Matter from R Ant Tib Art, Perc Approach */
"04CP4ZZ"  =  "1"  /* Extirpation of Matter from R Ant Tib Art, Perc Endo Approach */
"04CQ0ZZ"  =  "1"  /* Extirpation of Matter from L Ant Tib Art, Open Approach */
"04CQ3ZZ"  =  "1"  /* Extirpation of Matter from L Ant Tib Art, Perc Approach */
"04CQ4ZZ"  =  "1"  /* Extirpation of Matter from L Ant Tib Art, Perc Endo Approach */
"04CR0ZZ"  =  "1"  /* Extirpation of Matter from R Post Tib Art, Open Approach */
"04CR3ZZ"  =  "1"  /* Extirpation of Matter from R Post Tib Art, Perc Approach */
"04CR4ZZ"  =  "1"  /* Extirpate of Matter from R Post Tib Art, Perc Endo Approach */
"04CS0ZZ"  =  "1"  /* Extirpation of Matter from L Post Tib Art, Open Approach */
"04CS3ZZ"  =  "1"  /* Extirpation of Matter from L Post Tib Art, Perc Approach */
"04CS4ZZ"  =  "1"  /* Extirpate of Matter from L Post Tib Art, Perc Endo Approach */
"04CT0ZZ"  =  "1"  /* Extirpation of Matter from R Peroneal Art, Open Approach */
"04CT3ZZ"  =  "1"  /* Extirpation of Matter from R Peroneal Art, Perc Approach */
"04CT4ZZ"  =  "1"  /* Extirpate of Matter from R Peroneal Art, Perc Endo Approach */
"04CU0ZZ"  =  "1"  /* Extirpation of Matter from L Peroneal Art, Open Approach */
"04CU3ZZ"  =  "1"  /* Extirpation of Matter from L Peroneal Art, Perc Approach */
"04CU4ZZ"  =  "1"  /* Extirpate of Matter from L Peroneal Art, Perc Endo Approach */
"04CV0ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Artery, Open Approach */
"04CV3ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Artery, Perc Approach */
"04CV4ZZ"  =  "1"  /* Extirpation of Matter from R Foot Art, Perc Endo Approach */
"04CW0ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Artery, Open Approach */
"04CW3ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Artery, Perc Approach */
"04CW4ZZ"  =  "1"  /* Extirpation of Matter from L Foot Art, Perc Endo Approach */
"04CY0ZZ"  =  "1"  /* Extirpation of Matter from Lower Artery, Open Approach */
"04CY3ZZ"  =  "1"  /* Extirpation of Matter from Lower Artery, Perc Approach */
"04CY4ZZ"  =  "1"  /* Extirpation of Matter from Lower Artery, Perc Endo Approach */
"04H00DZ"  =  "1"  /* Insertion of Intralum Dev into Abd Aorta, Open Approach */
"04H03DZ"  =  "1"  /* Insertion of Intralum Dev into Abd Aorta, Perc Approach */
"04H04DZ"  =  "1"  /* Insertion of Intralum Dev into Abd Aorta, Perc Endo Approach */
"04H10DZ"  =  "1"  /* Insertion of Intralum Dev into Celiac Art, Open Approach */
"04H13DZ"  =  "1"  /* Insertion of Intralum Dev into Celiac Art, Perc Approach */
"04H14DZ"  =  "1"  /* Insert of Intralum Dev into Celiac Art, Perc Endo Approach */
"04H20DZ"  =  "1"  /* Insertion of Intralum Dev into Gastric Art, Open Approach */
"04H23DZ"  =  "1"  /* Insertion of Intralum Dev into Gastric Art, Perc Approach */
"04H24DZ"  =  "1"  /* Insert of Intralum Dev into Gastric Art, Perc Endo Approach */
"04H30DZ"  =  "1"  /* Insertion of Intralum Dev into Hepatic Art, Open Approach */
"04H33DZ"  =  "1"  /* Insertion of Intralum Dev into Hepatic Art, Perc Approach */
"04H34DZ"  =  "1"  /* Insert of Intralum Dev into Hepatic Art, Perc Endo Approach */
"04H40DZ"  =  "1"  /* Insertion of Intralum Dev into Splenic Art, Open Approach */
"04H43DZ"  =  "1"  /* Insertion of Intralum Dev into Splenic Art, Perc Approach */
"04H44DZ"  =  "1"  /* Insert of Intralum Dev into Splenic Art, Perc Endo Approach */
"04H50DZ"  =  "1"  /* Insertion of Intralum Dev into Sup Mesent Art, Open Approach */
"04H53DZ"  =  "1"  /* Insertion of Intralum Dev into Sup Mesent Art, Perc Approach */
"04H54DZ"  =  "1"  /* Insert Intralum Dev in Sup Mesent Art, Perc Endo */
"04H60DZ"  =  "1"  /* Insertion of Intralum Dev into R Colic Art, Open Approach */
"04H63DZ"  =  "1"  /* Insertion of Intralum Dev into R Colic Art, Perc Approach */
"04H64DZ"  =  "1"  /* Insert of Intralum Dev into R Colic Art, Perc Endo Approach */
"04H70DZ"  =  "1"  /* Insertion of Intralum Dev into L Colic Art, Open Approach */
"04H73DZ"  =  "1"  /* Insertion of Intralum Dev into L Colic Art, Perc Approach */
"04H74DZ"  =  "1"  /* Insert of Intralum Dev into L Colic Art, Perc Endo Approach */
"04H80DZ"  =  "1"  /* Insertion of Intralum Dev into Mid Colic Art, Open Approach */
"04H83DZ"  =  "1"  /* Insertion of Intralum Dev into Mid Colic Art, Perc Approach */
"04H84DZ"  =  "1"  /* Insert Intralum Dev in Mid Colic Art, Perc Endo */
"04H90DZ"  =  "1"  /* Insertion of Intralum Dev into R Renal Art, Open Approach */
"04H93DZ"  =  "1"  /* Insertion of Intralum Dev into R Renal Art, Perc Approach */
"04H94DZ"  =  "1"  /* Insert of Intralum Dev into R Renal Art, Perc Endo Approach */
"04HA0DZ"  =  "1"  /* Insertion of Intralum Dev into L Renal Art, Open Approach */
"04HA3DZ"  =  "1"  /* Insertion of Intralum Dev into L Renal Art, Perc Approach */
"04HA4DZ"  =  "1"  /* Insert of Intralum Dev into L Renal Art, Perc Endo Approach */
"04HB0DZ"  =  "1"  /* Insertion of Intralum Dev into Inf Mesent Art, Open Approach */
"04HB3DZ"  =  "1"  /* Insertion of Intralum Dev into Inf Mesent Art, Perc Approach */
"04HB4DZ"  =  "1"  /* Insert Intralum Dev in Inf Mesent Art, Perc Endo */
"04HC0DZ"  =  "1"  /* Insert of Intralum Dev into R Com Iliac Art, Open Approach */
"04HC3DZ"  =  "1"  /* Insert of Intralum Dev into R Com Iliac Art, Perc Approach */
"04HC4DZ"  =  "1"  /* Insert Intralum Dev in R Com Iliac Art, Perc Endo */
"04HD0DZ"  =  "1"  /* Insert of Intralum Dev into L Com Iliac Art, Open Approach */
"04HD3DZ"  =  "1"  /* Insert of Intralum Dev into L Com Iliac Art, Perc Approach */
"04HD4DZ"  =  "1"  /* Insert Intralum Dev in L Com Iliac Art, Perc Endo */
"04HE0DZ"  =  "1"  /* Insert of Intralum Dev into R Int Iliac Art, Open Approach */
"04HE3DZ"  =  "1"  /* Insert of Intralum Dev into R Int Iliac Art, Perc Approach */
"04HE4DZ"  =  "1"  /* Insert Intralum Dev in R Int Iliac Art, Perc Endo */
"04HF0DZ"  =  "1"  /* Insert of Intralum Dev into L Int Iliac Art, Open Approach */
"04HF3DZ"  =  "1"  /* Insert of Intralum Dev into L Int Iliac Art, Perc Approach */
"04HF4DZ"  =  "1"  /* Insert Intralum Dev in L Int Iliac Art, Perc Endo */
"04HH0DZ"  =  "1"  /* Insert of Intralum Dev into R Ext Iliac Art, Open Approach */
"04HH3DZ"  =  "1"  /* Insert of Intralum Dev into R Ext Iliac Art, Perc Approach */
"04HH4DZ"  =  "1"  /* Insert Intralum Dev in R Ext Iliac Art, Perc Endo */
"04HJ0DZ"  =  "1"  /* Insert of Intralum Dev into L Ext Iliac Art, Open Approach */
"04HJ3DZ"  =  "1"  /* Insert of Intralum Dev into L Ext Iliac Art, Perc Approach */
"04HJ4DZ"  =  "1"  /* Insert Intralum Dev in L Ext Iliac Art, Perc Endo */
"04HK0DZ"  =  "1"  /* Insertion of Intralum Dev into R Femor Art, Open Approach */
"04HK3DZ"  =  "1"  /* Insertion of Intralum Dev into R Femor Art, Perc Approach */
"04HK4DZ"  =  "1"  /* Insert of Intralum Dev into R Femor Art, Perc Endo Approach */
"04HL0DZ"  =  "1"  /* Insertion of Intralum Dev into L Femor Art, Open Approach */
"04HL3DZ"  =  "1"  /* Insertion of Intralum Dev into L Femor Art, Perc Approach */
"04HL4DZ"  =  "1"  /* Insert of Intralum Dev into L Femor Art, Perc Endo Approach */
"04HM0DZ"  =  "1"  /* Insertion of Intralum Dev into R Poplit Art, Open Approach */
"04HM3DZ"  =  "1"  /* Insertion of Intralum Dev into R Poplit Art, Perc Approach */
"04HM4DZ"  =  "1"  /* Insert of Intralum Dev into R Poplit Art, Perc Endo Approach */
"04HN0DZ"  =  "1"  /* Insertion of Intralum Dev into L Poplit Art, Open Approach */
"04HN3DZ"  =  "1"  /* Insertion of Intralum Dev into L Poplit Art, Perc Approach */
"04HN4DZ"  =  "1"  /* Insert of Intralum Dev into L Poplit Art, Perc Endo Approach */
"04HP0DZ"  =  "1"  /* Insertion of Intralum Dev into R Ant Tib Art, Open Approach */
"04HP3DZ"  =  "1"  /* Insertion of Intralum Dev into R Ant Tib Art, Perc Approach */
"04HP4DZ"  =  "1"  /* Insert Intralum Dev in R Ant Tib Art, Perc Endo */
"04HQ0DZ"  =  "1"  /* Insertion of Intralum Dev into L Ant Tib Art, Open Approach */
"04HQ3DZ"  =  "1"  /* Insertion of Intralum Dev into L Ant Tib Art, Perc Approach */
"04HQ4DZ"  =  "1"  /* Insert Intralum Dev in L Ant Tib Art, Perc Endo */
"04HR0DZ"  =  "1"  /* Insertion of Intralum Dev into R Post Tib Art, Open Approach */
"04HR3DZ"  =  "1"  /* Insertion of Intralum Dev into R Post Tib Art, Perc Approach */
"04HR4DZ"  =  "1"  /* Insert Intralum Dev in R Post Tib Art, Perc Endo */
"04HS0DZ"  =  "1"  /* Insertion of Intralum Dev into L Post Tib Art, Open Approach */
"04HS3DZ"  =  "1"  /* Insertion of Intralum Dev into L Post Tib Art, Perc Approach */
"04HS4DZ"  =  "1"  /* Insert Intralum Dev in L Post Tib Art, Perc Endo */
"04HT0DZ"  =  "1"  /* Insertion of Intralum Dev into R Peroneal Art, Open Approach */
"04HT3DZ"  =  "1"  /* Insertion of Intralum Dev into R Peroneal Art, Perc Approach */
"04HT4DZ"  =  "1"  /* Insert Intralum Dev in R Peroneal Art, Perc Endo */
"04HU0DZ"  =  "1"  /* Insertion of Intralum Dev into L Peroneal Art, Open Approach */
"04HU3DZ"  =  "1"  /* Insertion of Intralum Dev into L Peroneal Art, Perc Approach */
"04HU4DZ"  =  "1"  /* Insert Intralum Dev in L Peroneal Art, Perc Endo */
"04HV0DZ"  =  "1"  /* Insertion of Intralum Dev into R Foot Art, Open Approach */
"04HV3DZ"  =  "1"  /* Insertion of Intralum Dev into R Foot Art, Perc Approach */
"04HV4DZ"  =  "1"  /* Insert of Intralum Dev into R Foot Art, Perc Endo Approach */
"04HW0DZ"  =  "1"  /* Insertion of Intralum Dev into L Foot Art, Open Approach */
"04HW3DZ"  =  "1"  /* Insertion of Intralum Dev into L Foot Art, Perc Approach */
"04HW4DZ"  =  "1"  /* Insert of Intralum Dev into L Foot Art, Perc Endo Approach */
"04HY02Z"  =  "1"  /* Insertion of Monitoring Device into Low Art, Open Approach */
"04HY0DZ"  =  "1"  /* Insertion of Intraluminal Device into Low Art, Open Approach */
"04HY32Z"  =  "1"  /* Insertion of Monitoring Device into Low Art, Perc Approach */
"04HY3DZ"  =  "1"  /* Insertion of Intraluminal Device into Low Art, Perc Approach */
"04HY42Z"  =  "1"  /* Insertion of Monitor Dev into Low Art, Perc Endo Approach */
"04HY4DZ"  =  "1"  /* Insertion of Intralum Dev into Low Art, Perc Endo Approach */
"04JY0ZZ"  =  "1"  /* Inspection of Lower Artery, Open Approach */
"04JY3ZZ"  =  "1"  /* Inspection of Lower Artery, Percutaneous Approach */
"04L00CZ"  =  "1"  /* Occlusion of Abd Aorta with Extralum Dev, Open Approach */
"04L00DZ"  =  "1"  /* Occlusion of Abd Aorta with Intralum Dev, Open Approach */
"04L00ZZ"  =  "1"  /* Occlusion of Abdominal Aorta, Open Approach */
"04L03CZ"  =  "1"  /* Occlusion of Abd Aorta with Extralum Dev, Perc Approach */
"04L03DZ"  =  "1"  /* Occlusion of Abd Aorta with Intralum Dev, Perc Approach */
"04L03ZZ"  =  "1"  /* Occlusion of Abdominal Aorta, Percutaneous Approach */
"04L04CZ"  =  "1"  /* Occlusion of Abd Aorta with Extralum Dev, Perc Endo Approach */
"04L04DZ"  =  "1"  /* Occlusion of Abd Aorta with Intralum Dev, Perc Endo Approach */
"04L04ZZ"  =  "1"  /* Occlusion of Abdominal Aorta, Perc Endo Approach */
"04L10CZ"  =  "1"  /* Occlusion of Celiac Artery with Extralum Dev, Open Approach */
"04L10DZ"  =  "1"  /* Occlusion of Celiac Artery with Intralum Dev, Open Approach */
"04L10ZZ"  =  "1"  /* Occlusion of Celiac Artery, Open Approach */
"04L13CZ"  =  "1"  /* Occlusion of Celiac Artery with Extralum Dev, Perc Approach */
"04L13DZ"  =  "1"  /* Occlusion of Celiac Artery with Intralum Dev, Perc Approach */
"04L13ZZ"  =  "1"  /* Occlusion of Celiac Artery, Percutaneous Approach */
"04L14CZ"  =  "1"  /* Occlusion Celiac Art w Extralum Dev, Perc Endo */
"04L14DZ"  =  "1"  /* Occlusion Celiac Art w Intralum Dev, Perc Endo */
"04L14ZZ"  =  "1"  /* Occlusion of Celiac Artery, Percutaneous Endoscopic Approach */
"04L20CZ"  =  "1"  /* Occlusion of Gastric Artery with Extralum Dev, Open Approach */
"04L20DZ"  =  "1"  /* Occlusion of Gastric Artery with Intralum Dev, Open Approach */
"04L20ZZ"  =  "1"  /* Occlusion of Gastric Artery, Open Approach */
"04L23CZ"  =  "1"  /* Occlusion of Gastric Artery with Extralum Dev, Perc Approach */
"04L23ZZ"  =  "1"  /* Occlusion of Gastric Artery, Percutaneous Approach */
"04L24CZ"  =  "1"  /* Occlusion Gastric Art w Extralum Dev, Perc Endo */
"04L24DZ"  =  "1"  /* Occlusion Gastric Art w Intralum Dev, Perc Endo */
"04L24ZZ"  =  "1"  /* Occlusion of Gastric Artery, Perc Endo Approach */
"04L30CZ"  =  "1"  /* Occlusion of Hepatic Artery with Extralum Dev, Open Approach */
"04L30DZ"  =  "1"  /* Occlusion of Hepatic Artery with Intralum Dev, Open Approach */
"04L30ZZ"  =  "1"  /* Occlusion of Hepatic Artery, Open Approach */
"04L33CZ"  =  "1"  /* Occlusion of Hepatic Artery with Extralum Dev, Perc Approach */
"04L33DZ"  =  "1"  /* Occlusion of Hepatic Artery with Intralum Dev, Perc Approach */
"04L33ZZ"  =  "1"  /* Occlusion of Hepatic Artery, Percutaneous Approach */
"04L34CZ"  =  "1"  /* Occlusion Hepatic Art w Extralum Dev, Perc Endo */
"04L34DZ"  =  "1"  /* Occlusion Hepatic Art w Intralum Dev, Perc Endo */
"04L34ZZ"  =  "1"  /* Occlusion of Hepatic Artery, Perc Endo Approach */
"04L40CZ"  =  "1"  /* Occlusion of Splenic Artery with Extralum Dev, Open Approach */
"04L40DZ"  =  "1"  /* Occlusion of Splenic Artery with Intralum Dev, Open Approach */
"04L40ZZ"  =  "1"  /* Occlusion of Splenic Artery, Open Approach */
"04L43CZ"  =  "1"  /* Occlusion of Splenic Artery with Extralum Dev, Perc Approach */
"04L43DZ"  =  "1"  /* Occlusion of Splenic Artery with Intralum Dev, Perc Approach */
"04L43ZZ"  =  "1"  /* Occlusion of Splenic Artery, Percutaneous Approach */
"04L44CZ"  =  "1"  /* Occlusion Splenic Art w Extralum Dev, Perc Endo */
"04L44DZ"  =  "1"  /* Occlusion Splenic Art w Intralum Dev, Perc Endo */
"04L44ZZ"  =  "1"  /* Occlusion of Splenic Artery, Perc Endo Approach */
"04L50CZ"  =  "1"  /* Occlusion of Sup Mesent Art with Extralum Dev, Open Approach */
"04L50DZ"  =  "1"  /* Occlusion of Sup Mesent Art with Intralum Dev, Open Approach */
"04L50ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery, Open Approach */
"04L53CZ"  =  "1"  /* Occlusion of Sup Mesent Art with Extralum Dev, Perc Approach */
"04L53DZ"  =  "1"  /* Occlusion of Sup Mesent Art with Intralum Dev, Perc Approach */
"04L53ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery, Perc Approach */
"04L54CZ"  =  "1"  /* Occlusion Sup Mesent Art w Extralum Dev, Perc Endo */
"04L54DZ"  =  "1"  /* Occlusion Sup Mesent Art w Intralum Dev, Perc Endo */
"04L54ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Artery, Perc Endo Approach */
"04L60CZ"  =  "1"  /* Occlusion of R Colic Art with Extralum Dev, Open Approach */
"04L60DZ"  =  "1"  /* Occlusion of R Colic Art with Intralum Dev, Open Approach */
"04L60ZZ"  =  "1"  /* Occlusion of Right Colic Artery, Open Approach */
"04L63CZ"  =  "1"  /* Occlusion of R Colic Art with Extralum Dev, Perc Approach */
"04L63DZ"  =  "1"  /* Occlusion of R Colic Art with Intralum Dev, Perc Approach */
"04L63ZZ"  =  "1"  /* Occlusion of Right Colic Artery, Percutaneous Approach */
"04L64CZ"  =  "1"  /* Occlusion R Colic Art w Extralum Dev, Perc Endo */
"04L64DZ"  =  "1"  /* Occlusion R Colic Art w Intralum Dev, Perc Endo */
"04L64ZZ"  =  "1"  /* Occlusion of Right Colic Artery, Perc Endo Approach */
"04L70CZ"  =  "1"  /* Occlusion of L Colic Art with Extralum Dev, Open Approach */
"04L70DZ"  =  "1"  /* Occlusion of L Colic Art with Intralum Dev, Open Approach */
"04L70ZZ"  =  "1"  /* Occlusion of Left Colic Artery, Open Approach */
"04L73CZ"  =  "1"  /* Occlusion of L Colic Art with Extralum Dev, Perc Approach */
"04L73DZ"  =  "1"  /* Occlusion of L Colic Art with Intralum Dev, Perc Approach */
"04L73ZZ"  =  "1"  /* Occlusion of Left Colic Artery, Percutaneous Approach */
"04L74CZ"  =  "1"  /* Occlusion L Colic Art w Extralum Dev, Perc Endo */
"04L74DZ"  =  "1"  /* Occlusion L Colic Art w Intralum Dev, Perc Endo */
"04L74ZZ"  =  "1"  /* Occlusion of Left Colic Artery, Perc Endo Approach */
"04L80CZ"  =  "1"  /* Occlusion of Mid Colic Art with Extralum Dev, Open Approach */
"04L80DZ"  =  "1"  /* Occlusion of Mid Colic Art with Intralum Dev, Open Approach */
"04L80ZZ"  =  "1"  /* Occlusion of Middle Colic Artery, Open Approach */
"04L83CZ"  =  "1"  /* Occlusion of Mid Colic Art with Extralum Dev, Perc Approach */
"04L83DZ"  =  "1"  /* Occlusion of Mid Colic Art with Intralum Dev, Perc Approach */
"04L83ZZ"  =  "1"  /* Occlusion of Middle Colic Artery, Percutaneous Approach */
"04L84CZ"  =  "1"  /* Occlusion Mid Colic Art w Extralum Dev, Perc Endo */
"04L84DZ"  =  "1"  /* Occlusion Mid Colic Art w Intralum Dev, Perc Endo */
"04L84ZZ"  =  "1"  /* Occlusion of Middle Colic Artery, Perc Endo Approach */
"04L90CZ"  =  "1"  /* Occlusion of R Renal Art with Extralum Dev, Open Approach */
"04L90DZ"  =  "1"  /* Occlusion of R Renal Art with Intralum Dev, Open Approach */
"04L90ZZ"  =  "1"  /* Occlusion of Right Renal Artery, Open Approach */
"04L93CZ"  =  "1"  /* Occlusion of R Renal Art with Extralum Dev, Perc Approach */
"04L93DZ"  =  "1"  /* Occlusion of R Renal Art with Intralum Dev, Perc Approach */
"04L93ZZ"  =  "1"  /* Occlusion of Right Renal Artery, Percutaneous Approach */
"04L94CZ"  =  "1"  /* Occlusion R Renal Art w Extralum Dev, Perc Endo */
"04L94DZ"  =  "1"  /* Occlusion R Renal Art w Intralum Dev, Perc Endo */
"04L94ZZ"  =  "1"  /* Occlusion of Right Renal Artery, Perc Endo Approach */
"04LA0CZ"  =  "1"  /* Occlusion of L Renal Art with Extralum Dev, Open Approach */
"04LA0DZ"  =  "1"  /* Occlusion of L Renal Art with Intralum Dev, Open Approach */
"04LA0ZZ"  =  "1"  /* Occlusion of Left Renal Artery, Open Approach */
"04LA3CZ"  =  "1"  /* Occlusion of L Renal Art with Extralum Dev, Perc Approach */
"04LA3DZ"  =  "1"  /* Occlusion of L Renal Art with Intralum Dev, Perc Approach */
"04LA3ZZ"  =  "1"  /* Occlusion of Left Renal Artery, Percutaneous Approach */
"04LA4CZ"  =  "1"  /* Occlusion L Renal Art w Extralum Dev, Perc Endo */
"04LA4DZ"  =  "1"  /* Occlusion L Renal Art w Intralum Dev, Perc Endo */
"04LA4ZZ"  =  "1"  /* Occlusion of Left Renal Artery, Perc Endo Approach */
"04LB0CZ"  =  "1"  /* Occlusion of Inf Mesent Art with Extralum Dev, Open Approach */
"04LB0DZ"  =  "1"  /* Occlusion of Inf Mesent Art with Intralum Dev, Open Approach */
"04LB0ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery, Open Approach */
"04LB3CZ"  =  "1"  /* Occlusion of Inf Mesent Art with Extralum Dev, Perc Approach */
"04LB3DZ"  =  "1"  /* Occlusion of Inf Mesent Art with Intralum Dev, Perc Approach */
"04LB3ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery, Perc Approach */
"04LB4CZ"  =  "1"  /* Occlusion Inf Mesent Art w Extralum Dev, Perc Endo */
"04LB4DZ"  =  "1"  /* Occlusion Inf Mesent Art w Intralum Dev, Perc Endo */
"04LB4ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Artery, Perc Endo Approach */
"04LC0CZ"  =  "1"  /* Occlusion R Com Iliac Art w Extralum Dev, Open */
"04LC0DZ"  =  "1"  /* Occlusion R Com Iliac Art w Intralum Dev, Open */
"04LC0ZZ"  =  "1"  /* Occlusion of Right Common Iliac Artery, Open Approach */
"04LC3CZ"  =  "1"  /* Occlusion R Com Iliac Art w Extralum Dev, Perc */
"04LC3DZ"  =  "1"  /* Occlusion R Com Iliac Art w Intralum Dev, Perc */
"04LC3ZZ"  =  "1"  /* Occlusion of Right Common Iliac Artery, Perc Approach */
"04LC4CZ"  =  "1"  /* Occlusion R Com Iliac Art w Extralum Dev, Perc Endo */
"04LC4DZ"  =  "1"  /* Occlusion R Com Iliac Art w Intralum Dev, Perc Endo */
"04LC4ZZ"  =  "1"  /* Occlusion of Right Common Iliac Artery, Perc Endo Approach */
"04LD0CZ"  =  "1"  /* Occlusion L Com Iliac Art w Extralum Dev, Open */
"04LD0DZ"  =  "1"  /* Occlusion L Com Iliac Art w Intralum Dev, Open */
"04LD0ZZ"  =  "1"  /* Occlusion of Left Common Iliac Artery, Open Approach */
"04LD3CZ"  =  "1"  /* Occlusion L Com Iliac Art w Extralum Dev, Perc */
"04LD3DZ"  =  "1"  /* Occlusion L Com Iliac Art w Intralum Dev, Perc */
"04LD3ZZ"  =  "1"  /* Occlusion of Left Common Iliac Artery, Percutaneous Approach */
"04LD4CZ"  =  "1"  /* Occlusion L Com Iliac Art w Extralum Dev, Perc Endo */
"04LD4DZ"  =  "1"  /* Occlusion L Com Iliac Art w Intralum Dev, Perc Endo */
"04LD4ZZ"  =  "1"  /* Occlusion of Left Common Iliac Artery, Perc Endo Approach */
"04LE0CT"  =  "1"  /* Occlusion of R Uter A with Extralum Dev, Open Approach */
"04LE0CZ"  =  "1"  /* Occlusion R Int Iliac Art w Extralum Dev, Open */
"04LE0DT"  =  "1"  /* Occlusion of R Uter A with Intralum Dev, Open Approach */
"04LE0DZ"  =  "1"  /* Occlusion R Int Iliac Art w Intralum Dev, Open */
"04LE0ZT"  =  "1"  /* Occlusion of Right Uterine Artery, Open Approach */
"04LE0ZZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery, Open Approach */
"04LE3CT"  =  "1"  /* Occlusion of R Uter A with Extralum Dev, Perc Approach */
"04LE3CZ"  =  "1"  /* Occlusion R Int Iliac Art w Extralum Dev, Perc */
"04LE3DT"  =  "1"  /* Occlusion of R Uter A with Intralum Dev, Perc Approach */
"04LE3DZ"  =  "1"  /* Occlusion R Int Iliac Art w Intralum Dev, Perc */
"04LE3ZT"  =  "1"  /* Occlusion of Right Uterine Artery, Percutaneous Approach */
"04LE3ZZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery, Perc Approach */
"04LE4CT"  =  "1"  /* Occlusion of R Uter A with Extralum Dev, Perc Endo Approach */
"04LE4CZ"  =  "1"  /* Occlusion R Int Iliac Art w Extralum Dev, Perc Endo */
"04LE4DT"  =  "1"  /* Occlusion of R Uter A with Intralum Dev, Perc Endo Approach */
"04LE4DZ"  =  "1"  /* Occlusion R Int Iliac Art w Intralum Dev, Perc Endo */
"04LE4ZT"  =  "1"  /* Occlusion of Right Uterine Artery, Perc Endo Approach */
"04LE4ZZ"  =  "1"  /* Occlusion of Right Internal Iliac Artery, Perc Endo Approach */
"04LF0CU"  =  "1"  /* Occlusion of L Uter A with Extralum Dev, Open Approach */
"04LF0CZ"  =  "1"  /* Occlusion L Int Iliac Art w Extralum Dev, Open */
"04LF0DU"  =  "1"  /* Occlusion of L Uter A with Intralum Dev, Open Approach */
"04LF0DZ"  =  "1"  /* Occlusion L Int Iliac Art w Intralum Dev, Open */
"04LF0ZU"  =  "1"  /* Occlusion of Left Uterine Artery, Open Approach */
"04LF0ZZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery, Open Approach */
"04LF3CU"  =  "1"  /* Occlusion of L Uter A with Extralum Dev, Perc Approach */
"04LF3CZ"  =  "1"  /* Occlusion L Int Iliac Art w Extralum Dev, Perc */
"04LF3DU"  =  "1"  /* Occlusion of L Uter A with Intralum Dev, Perc Approach */
"04LF3DZ"  =  "1"  /* Occlusion L Int Iliac Art w Intralum Dev, Perc */
"04LF3ZU"  =  "1"  /* Occlusion of Left Uterine Artery, Percutaneous Approach */
"04LF3ZZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery, Perc Approach */
"04LF4CU"  =  "1"  /* Occlusion of L Uter A with Extralum Dev, Perc Endo Approach */
"04LF4CZ"  =  "1"  /* Occlusion L Int Iliac Art w Extralum Dev, Perc Endo */
"04LF4DU"  =  "1"  /* Occlusion of L Uter A with Intralum Dev, Perc Endo Approach */
"04LF4DZ"  =  "1"  /* Occlusion L Int Iliac Art w Intralum Dev, Perc Endo */
"04LF4ZU"  =  "1"  /* Occlusion of Left Uterine Artery, Perc Endo Approach */
"04LF4ZZ"  =  "1"  /* Occlusion of Left Internal Iliac Artery, Perc Endo Approach */
"04LH0CZ"  =  "1"  /* Occlusion R Ext Iliac Art w Extralum Dev, Open */
"04LH0DZ"  =  "1"  /* Occlusion R Ext Iliac Art w Intralum Dev, Open */
"04LH0ZZ"  =  "1"  /* Occlusion of Right External Iliac Artery, Open Approach */
"04LH3CZ"  =  "1"  /* Occlusion R Ext Iliac Art w Extralum Dev, Perc */
"04LH3DZ"  =  "1"  /* Occlusion R Ext Iliac Art w Intralum Dev, Perc */
"04LH3ZZ"  =  "1"  /* Occlusion of Right External Iliac Artery, Perc Approach */
"04LH4CZ"  =  "1"  /* Occlusion R Ext Iliac Art w Extralum Dev, Perc Endo */
"04LH4DZ"  =  "1"  /* Occlusion R Ext Iliac Art w Intralum Dev, Perc Endo */
"04LH4ZZ"  =  "1"  /* Occlusion of Right External Iliac Artery, Perc Endo Approach */
"04LJ0CZ"  =  "1"  /* Occlusion L Ext Iliac Art w Extralum Dev, Open */
"04LJ0DZ"  =  "1"  /* Occlusion L Ext Iliac Art w Intralum Dev, Open */
"04LJ0ZZ"  =  "1"  /* Occlusion of Left External Iliac Artery, Open Approach */
"04LJ3CZ"  =  "1"  /* Occlusion L Ext Iliac Art w Extralum Dev, Perc */
"04LJ3DZ"  =  "1"  /* Occlusion L Ext Iliac Art w Intralum Dev, Perc */
"04LJ3ZZ"  =  "1"  /* Occlusion of Left External Iliac Artery, Perc Approach */
"04LJ4CZ"  =  "1"  /* Occlusion L Ext Iliac Art w Extralum Dev, Perc Endo */
"04LJ4DZ"  =  "1"  /* Occlusion L Ext Iliac Art w Intralum Dev, Perc Endo */
"04LJ4ZZ"  =  "1"  /* Occlusion of Left External Iliac Artery, Perc Endo Approach */
"04LK0CZ"  =  "1"  /* Occlusion of R Femor Art with Extralum Dev, Open Approach */
"04LK0DZ"  =  "1"  /* Occlusion of R Femor Art with Intralum Dev, Open Approach */
"04LK0ZZ"  =  "1"  /* Occlusion of Right Femoral Artery, Open Approach */
"04LK3CZ"  =  "1"  /* Occlusion of R Femor Art with Extralum Dev, Perc Approach */
"04LK3DZ"  =  "1"  /* Occlusion of R Femor Art with Intralum Dev, Perc Approach */
"04LK3ZZ"  =  "1"  /* Occlusion of Right Femoral Artery, Percutaneous Approach */
"04LK4CZ"  =  "1"  /* Occlusion R Femor Art w Extralum Dev, Perc Endo */
"04LK4DZ"  =  "1"  /* Occlusion R Femor Art w Intralum Dev, Perc Endo */
"04LK4ZZ"  =  "1"  /* Occlusion of Right Femoral Artery, Perc Endo Approach */
"04LL0CZ"  =  "1"  /* Occlusion of L Femor Art with Extralum Dev, Open Approach */
"04LL0DZ"  =  "1"  /* Occlusion of L Femor Art with Intralum Dev, Open Approach */
"04LL0ZZ"  =  "1"  /* Occlusion of Left Femoral Artery, Open Approach */
"04LL3CZ"  =  "1"  /* Occlusion of L Femor Art with Extralum Dev, Perc Approach */
"04LL3DZ"  =  "1"  /* Occlusion of L Femor Art with Intralum Dev, Perc Approach */
"04LL3ZZ"  =  "1"  /* Occlusion of Left Femoral Artery, Percutaneous Approach */
"04LL4CZ"  =  "1"  /* Occlusion L Femor Art w Extralum Dev, Perc Endo */
"04LL4DZ"  =  "1"  /* Occlusion L Femor Art w Intralum Dev, Perc Endo */
"04LL4ZZ"  =  "1"  /* Occlusion of Left Femoral Artery, Perc Endo Approach */
"04LM0CZ"  =  "1"  /* Occlusion of R Poplit Art with Extralum Dev, Open Approach */
"04LM0DZ"  =  "1"  /* Occlusion of R Poplit Art with Intralum Dev, Open Approach */
"04LM0ZZ"  =  "1"  /* Occlusion of Right Popliteal Artery, Open Approach */
"04LM3CZ"  =  "1"  /* Occlusion of R Poplit Art with Extralum Dev, Perc Approach */
"04LM3DZ"  =  "1"  /* Occlusion of R Poplit Art with Intralum Dev, Perc Approach */
"04LM3ZZ"  =  "1"  /* Occlusion of Right Popliteal Artery, Percutaneous Approach */
"04LM4CZ"  =  "1"  /* Occlusion R Poplit Art w Extralum Dev, Perc Endo */
"04LM4DZ"  =  "1"  /* Occlusion R Poplit Art w Intralum Dev, Perc Endo */
"04LM4ZZ"  =  "1"  /* Occlusion of Right Popliteal Artery, Perc Endo Approach */
"04LN0CZ"  =  "1"  /* Occlusion of L Poplit Art with Extralum Dev, Open Approach */
"04LN0DZ"  =  "1"  /* Occlusion of L Poplit Art with Intralum Dev, Open Approach */
"04LN0ZZ"  =  "1"  /* Occlusion of Left Popliteal Artery, Open Approach */
"04LN3CZ"  =  "1"  /* Occlusion of L Poplit Art with Extralum Dev, Perc Approach */
"04LN3DZ"  =  "1"  /* Occlusion of L Poplit Art with Intralum Dev, Perc Approach */
"04LN3ZZ"  =  "1"  /* Occlusion of Left Popliteal Artery, Percutaneous Approach */
"04LN4CZ"  =  "1"  /* Occlusion L Poplit Art w Extralum Dev, Perc Endo */
"04LN4DZ"  =  "1"  /* Occlusion L Poplit Art w Intralum Dev, Perc Endo */
"04LN4ZZ"  =  "1"  /* Occlusion of Left Popliteal Artery, Perc Endo Approach */
"04LP0CZ"  =  "1"  /* Occlusion of R Ant Tib Art with Extralum Dev, Open Approach */
"04LP0DZ"  =  "1"  /* Occlusion of R Ant Tib Art with Intralum Dev, Open Approach */
"04LP0ZZ"  =  "1"  /* Occlusion of Right Anterior Tibial Artery, Open Approach */
"04LP3CZ"  =  "1"  /* Occlusion of R Ant Tib Art with Extralum Dev, Perc Approach */
"04LP3DZ"  =  "1"  /* Occlusion of R Ant Tib Art with Intralum Dev, Perc Approach */
"04LP3ZZ"  =  "1"  /* Occlusion of Right Anterior Tibial Artery, Perc Approach */
"04LP4CZ"  =  "1"  /* Occlusion R Ant Tib Art w Extralum Dev, Perc Endo */
"04LP4DZ"  =  "1"  /* Occlusion R Ant Tib Art w Intralum Dev, Perc Endo */
"04LP4ZZ"  =  "1"  /* Occlusion of R Ant Tib Art, Perc Endo Approach */
"04LQ0CZ"  =  "1"  /* Occlusion of L Ant Tib Art with Extralum Dev, Open Approach */
"04LQ0DZ"  =  "1"  /* Occlusion of L Ant Tib Art with Intralum Dev, Open Approach */
"04LQ0ZZ"  =  "1"  /* Occlusion of Left Anterior Tibial Artery, Open Approach */
"04LQ3CZ"  =  "1"  /* Occlusion of L Ant Tib Art with Extralum Dev, Perc Approach */
"04LQ3DZ"  =  "1"  /* Occlusion of L Ant Tib Art with Intralum Dev, Perc Approach */
"04LQ3ZZ"  =  "1"  /* Occlusion of Left Anterior Tibial Artery, Perc Approach */
"04LQ4CZ"  =  "1"  /* Occlusion L Ant Tib Art w Extralum Dev, Perc Endo */
"04LQ4DZ"  =  "1"  /* Occlusion L Ant Tib Art w Intralum Dev, Perc Endo */
"04LQ4ZZ"  =  "1"  /* Occlusion of Left Anterior Tibial Artery, Perc Endo Approach */
"04LR0CZ"  =  "1"  /* Occlusion of R Post Tib Art with Extralum Dev, Open Approach */
"04LR0DZ"  =  "1"  /* Occlusion of R Post Tib Art with Intralum Dev, Open Approach */
"04LR0ZZ"  =  "1"  /* Occlusion of Right Posterior Tibial Artery, Open Approach */
"04LR3CZ"  =  "1"  /* Occlusion of R Post Tib Art with Extralum Dev, Perc Approach */
"04LR3DZ"  =  "1"  /* Occlusion of R Post Tib Art with Intralum Dev, Perc Approach */
"04LR3ZZ"  =  "1"  /* Occlusion of Right Posterior Tibial Artery, Perc Approach */
"04LR4CZ"  =  "1"  /* Occlusion R Post Tib Art w Extralum Dev, Perc Endo */
"04LR4DZ"  =  "1"  /* Occlusion R Post Tib Art w Intralum Dev, Perc Endo */
"04LR4ZZ"  =  "1"  /* Occlusion of R Post Tib Art, Perc Endo Approach */
"04LS0CZ"  =  "1"  /* Occlusion of L Post Tib Art with Extralum Dev, Open Approach */
"04LS0DZ"  =  "1"  /* Occlusion of L Post Tib Art with Intralum Dev, Open Approach */
"04LS0ZZ"  =  "1"  /* Occlusion of Left Posterior Tibial Artery, Open Approach */
"04LS3CZ"  =  "1"  /* Occlusion of L Post Tib Art with Extralum Dev, Perc Approach */
"04LS3DZ"  =  "1"  /* Occlusion of L Post Tib Art with Intralum Dev, Perc Approach */
"04LS3ZZ"  =  "1"  /* Occlusion of Left Posterior Tibial Artery, Perc Approach */
"04LS4CZ"  =  "1"  /* Occlusion L Post Tib Art w Extralum Dev, Perc Endo */
"04LS4DZ"  =  "1"  /* Occlusion L Post Tib Art w Intralum Dev, Perc Endo */
"04LS4ZZ"  =  "1"  /* Occlusion of L Post Tib Art, Perc Endo Approach */
"04LT0CZ"  =  "1"  /* Occlusion of R Peroneal Art with Extralum Dev, Open Approach */
"04LT0DZ"  =  "1"  /* Occlusion of R Peroneal Art with Intralum Dev, Open Approach */
"04LT0ZZ"  =  "1"  /* Occlusion of Right Peroneal Artery, Open Approach */
"04LT3CZ"  =  "1"  /* Occlusion of R Peroneal Art with Extralum Dev, Perc Approach */
"04LT3DZ"  =  "1"  /* Occlusion of R Peroneal Art with Intralum Dev, Perc Approach */
"04LT3ZZ"  =  "1"  /* Occlusion of Right Peroneal Artery, Percutaneous Approach */
"04LT4CZ"  =  "1"  /* Occlusion R Peroneal Art w Extralum Dev, Perc Endo */
"04LT4DZ"  =  "1"  /* Occlusion R Peroneal Art w Intralum Dev, Perc Endo */
"04LT4ZZ"  =  "1"  /* Occlusion of Right Peroneal Artery, Perc Endo Approach */
"04LU0CZ"  =  "1"  /* Occlusion of L Peroneal Art with Extralum Dev, Open Approach */
"04LU0DZ"  =  "1"  /* Occlusion of L Peroneal Art with Intralum Dev, Open Approach */
"04LU0ZZ"  =  "1"  /* Occlusion of Left Peroneal Artery, Open Approach */
"04LU3CZ"  =  "1"  /* Occlusion of L Peroneal Art with Extralum Dev, Perc Approach */
"04LU3DZ"  =  "1"  /* Occlusion of L Peroneal Art with Intralum Dev, Perc Approach */
"04LU3ZZ"  =  "1"  /* Occlusion of Left Peroneal Artery, Percutaneous Approach */
"04LU4CZ"  =  "1"  /* Occlusion L Peroneal Art w Extralum Dev, Perc Endo */
"04LU4DZ"  =  "1"  /* Occlusion L Peroneal Art w Intralum Dev, Perc Endo */
"04LU4ZZ"  =  "1"  /* Occlusion of Left Peroneal Artery, Perc Endo Approach */
"04LV0CZ"  =  "1"  /* Occlusion of R Foot Art with Extralum Dev, Open Approach */
"04LV0DZ"  =  "1"  /* Occlusion of R Foot Art with Intralum Dev, Open Approach */
"04LV0ZZ"  =  "1"  /* Occlusion of Right Foot Artery, Open Approach */
"04LV3CZ"  =  "1"  /* Occlusion of R Foot Art with Extralum Dev, Perc Approach */
"04LV3DZ"  =  "1"  /* Occlusion of R Foot Art with Intralum Dev, Perc Approach */
"04LV3ZZ"  =  "1"  /* Occlusion of Right Foot Artery, Percutaneous Approach */
"04LV4CZ"  =  "1"  /* Occlusion R Foot Art w Extralum Dev, Perc Endo */
"04LV4DZ"  =  "1"  /* Occlusion R Foot Art w Intralum Dev, Perc Endo */
"04LV4ZZ"  =  "1"  /* Occlusion of Right Foot Artery, Perc Endo Approach */
"04LW0CZ"  =  "1"  /* Occlusion of L Foot Art with Extralum Dev, Open Approach */
"04LW0DZ"  =  "1"  /* Occlusion of L Foot Art with Intralum Dev, Open Approach */
"04LW0ZZ"  =  "1"  /* Occlusion of Left Foot Artery, Open Approach */
"04LW3CZ"  =  "1"  /* Occlusion of L Foot Art with Extralum Dev, Perc Approach */
"04LW3DZ"  =  "1"  /* Occlusion of L Foot Art with Intralum Dev, Perc Approach */
"04LW3ZZ"  =  "1"  /* Occlusion of Left Foot Artery, Percutaneous Approach */
"04LW4CZ"  =  "1"  /* Occlusion L Foot Art w Extralum Dev, Perc Endo */
"04LW4DZ"  =  "1"  /* Occlusion L Foot Art w Intralum Dev, Perc Endo */
"04LW4ZZ"  =  "1"  /* Occlusion of Left Foot Artery, Perc Endo Approach */
"04LY0CZ"  =  "1"  /* Occlusion of Lower Artery with Extralum Dev, Open Approach */
"04LY0DZ"  =  "1"  /* Occlusion of Lower Artery with Intralum Dev, Open Approach */
"04LY0ZZ"  =  "1"  /* Occlusion of Lower Artery, Open Approach */
"04LY3CZ"  =  "1"  /* Occlusion of Lower Artery with Extralum Dev, Perc Approach */
"04LY3DZ"  =  "1"  /* Occlusion of Lower Artery with Intralum Dev, Perc Approach */
"04LY3ZZ"  =  "1"  /* Occlusion of Lower Artery, Percutaneous Approach */
"04LY4CZ"  =  "1"  /* Occlusion of Low Art with Extralum Dev, Perc Endo Approach */
"04LY4DZ"  =  "1"  /* Occlusion of Low Art with Intralum Dev, Perc Endo Approach */
"04LY4ZZ"  =  "1"  /* Occlusion of Lower Artery, Percutaneous Endoscopic Approach */
"04N00ZZ"  =  "1"  /* Release Abdominal Aorta, Open Approach */
"04N03ZZ"  =  "1"  /* Release Abdominal Aorta, Percutaneous Approach */
"04N04ZZ"  =  "1"  /* Release Abdominal Aorta, Percutaneous Endoscopic Approach */
"04N10ZZ"  =  "1"  /* Release Celiac Artery, Open Approach */
"04N13ZZ"  =  "1"  /* Release Celiac Artery, Percutaneous Approach */
"04N14ZZ"  =  "1"  /* Release Celiac Artery, Percutaneous Endoscopic Approach */
"04N20ZZ"  =  "1"  /* Release Gastric Artery, Open Approach */
"04N23ZZ"  =  "1"  /* Release Gastric Artery, Percutaneous Approach */
"04N24ZZ"  =  "1"  /* Release Gastric Artery, Percutaneous Endoscopic Approach */
"04N30ZZ"  =  "1"  /* Release Hepatic Artery, Open Approach */
"04N33ZZ"  =  "1"  /* Release Hepatic Artery, Percutaneous Approach */
"04N34ZZ"  =  "1"  /* Release Hepatic Artery, Percutaneous Endoscopic Approach */
"04N40ZZ"  =  "1"  /* Release Splenic Artery, Open Approach */
"04N43ZZ"  =  "1"  /* Release Splenic Artery, Percutaneous Approach */
"04N44ZZ"  =  "1"  /* Release Splenic Artery, Percutaneous Endoscopic Approach */
"04N50ZZ"  =  "1"  /* Release Superior Mesenteric Artery, Open Approach */
"04N53ZZ"  =  "1"  /* Release Superior Mesenteric Artery, Percutaneous Approach */
"04N54ZZ"  =  "1"  /* Release Superior Mesenteric Artery, Perc Endo Approach */
"04N60ZZ"  =  "1"  /* Release Right Colic Artery, Open Approach */
"04N63ZZ"  =  "1"  /* Release Right Colic Artery, Percutaneous Approach */
"04N64ZZ"  =  "1"  /* Release Right Colic Artery, Percutaneous Endoscopic Approach */
"04N70ZZ"  =  "1"  /* Release Left Colic Artery, Open Approach */
"04N73ZZ"  =  "1"  /* Release Left Colic Artery, Percutaneous Approach */
"04N74ZZ"  =  "1"  /* Release Left Colic Artery, Percutaneous Endoscopic Approach */
"04N80ZZ"  =  "1"  /* Release Middle Colic Artery, Open Approach */
"04N83ZZ"  =  "1"  /* Release Middle Colic Artery, Percutaneous Approach */
"04N84ZZ"  =  "1"  /* Release Middle Colic Artery, Perc Endo Approach */
"04N90ZZ"  =  "1"  /* Release Right Renal Artery, Open Approach */
"04N93ZZ"  =  "1"  /* Release Right Renal Artery, Percutaneous Approach */
"04N94ZZ"  =  "1"  /* Release Right Renal Artery, Percutaneous Endoscopic Approach */
"04NA0ZZ"  =  "1"  /* Release Left Renal Artery, Open Approach */
"04NA3ZZ"  =  "1"  /* Release Left Renal Artery, Percutaneous Approach */
"04NA4ZZ"  =  "1"  /* Release Left Renal Artery, Percutaneous Endoscopic Approach */
"04NB0ZZ"  =  "1"  /* Release Inferior Mesenteric Artery, Open Approach */
"04NB3ZZ"  =  "1"  /* Release Inferior Mesenteric Artery, Percutaneous Approach */
"04NB4ZZ"  =  "1"  /* Release Inferior Mesenteric Artery, Perc Endo Approach */
"04NC0ZZ"  =  "1"  /* Release Right Common Iliac Artery, Open Approach */
"04NC3ZZ"  =  "1"  /* Release Right Common Iliac Artery, Percutaneous Approach */
"04NC4ZZ"  =  "1"  /* Release Right Common Iliac Artery, Perc Endo Approach */
"04ND0ZZ"  =  "1"  /* Release Left Common Iliac Artery, Open Approach */
"04ND3ZZ"  =  "1"  /* Release Left Common Iliac Artery, Percutaneous Approach */
"04ND4ZZ"  =  "1"  /* Release Left Common Iliac Artery, Perc Endo Approach */
"04NE0ZZ"  =  "1"  /* Release Right Internal Iliac Artery, Open Approach */
"04NE3ZZ"  =  "1"  /* Release Right Internal Iliac Artery, Percutaneous Approach */
"04NE4ZZ"  =  "1"  /* Release Right Internal Iliac Artery, Perc Endo Approach */
"04NF0ZZ"  =  "1"  /* Release Left Internal Iliac Artery, Open Approach */
"04NF3ZZ"  =  "1"  /* Release Left Internal Iliac Artery, Percutaneous Approach */
"04NF4ZZ"  =  "1"  /* Release Left Internal Iliac Artery, Perc Endo Approach */
"04NH0ZZ"  =  "1"  /* Release Right External Iliac Artery, Open Approach */
"04NH3ZZ"  =  "1"  /* Release Right External Iliac Artery, Percutaneous Approach */
"04NH4ZZ"  =  "1"  /* Release Right External Iliac Artery, Perc Endo Approach */
"04NJ0ZZ"  =  "1"  /* Release Left External Iliac Artery, Open Approach */
"04NJ3ZZ"  =  "1"  /* Release Left External Iliac Artery, Percutaneous Approach */
"04NJ4ZZ"  =  "1"  /* Release Left External Iliac Artery, Perc Endo Approach */
"04NK0ZZ"  =  "1"  /* Release Right Femoral Artery, Open Approach */
"04NK3ZZ"  =  "1"  /* Release Right Femoral Artery, Percutaneous Approach */
"04NK4ZZ"  =  "1"  /* Release Right Femoral Artery, Perc Endo Approach */
"04NL0ZZ"  =  "1"  /* Release Left Femoral Artery, Open Approach */
"04NL3ZZ"  =  "1"  /* Release Left Femoral Artery, Percutaneous Approach */
"04NL4ZZ"  =  "1"  /* Release Left Femoral Artery, Perc Endo Approach */
"04NM0ZZ"  =  "1"  /* Release Right Popliteal Artery, Open Approach */
"04NM3ZZ"  =  "1"  /* Release Right Popliteal Artery, Percutaneous Approach */
"04NM4ZZ"  =  "1"  /* Release Right Popliteal Artery, Perc Endo Approach */
"04NN0ZZ"  =  "1"  /* Release Left Popliteal Artery, Open Approach */
"04NN3ZZ"  =  "1"  /* Release Left Popliteal Artery, Percutaneous Approach */
"04NN4ZZ"  =  "1"  /* Release Left Popliteal Artery, Perc Endo Approach */
"04NP0ZZ"  =  "1"  /* Release Right Anterior Tibial Artery, Open Approach */
"04NP3ZZ"  =  "1"  /* Release Right Anterior Tibial Artery, Percutaneous Approach */
"04NP4ZZ"  =  "1"  /* Release Right Anterior Tibial Artery, Perc Endo Approach */
"04NQ0ZZ"  =  "1"  /* Release Left Anterior Tibial Artery, Open Approach */
"04NQ3ZZ"  =  "1"  /* Release Left Anterior Tibial Artery, Percutaneous Approach */
"04NQ4ZZ"  =  "1"  /* Release Left Anterior Tibial Artery, Perc Endo Approach */
"04NR0ZZ"  =  "1"  /* Release Right Posterior Tibial Artery, Open Approach */
"04NR3ZZ"  =  "1"  /* Release Right Posterior Tibial Artery, Percutaneous Approach */
"04NR4ZZ"  =  "1"  /* Release Right Posterior Tibial Artery, Perc Endo Approach */
"04NS0ZZ"  =  "1"  /* Release Left Posterior Tibial Artery, Open Approach */
"04NS3ZZ"  =  "1"  /* Release Left Posterior Tibial Artery, Percutaneous Approach */
"04NS4ZZ"  =  "1"  /* Release Left Posterior Tibial Artery, Perc Endo Approach */
"04NT0ZZ"  =  "1"  /* Release Right Peroneal Artery, Open Approach */
"04NT3ZZ"  =  "1"  /* Release Right Peroneal Artery, Percutaneous Approach */
"04NT4ZZ"  =  "1"  /* Release Right Peroneal Artery, Perc Endo Approach */
"04NU0ZZ"  =  "1"  /* Release Left Peroneal Artery, Open Approach */
"04NU3ZZ"  =  "1"  /* Release Left Peroneal Artery, Percutaneous Approach */
"04NU4ZZ"  =  "1"  /* Release Left Peroneal Artery, Perc Endo Approach */
"04NV0ZZ"  =  "1"  /* Release Right Foot Artery, Open Approach */
"04NV3ZZ"  =  "1"  /* Release Right Foot Artery, Percutaneous Approach */
"04NV4ZZ"  =  "1"  /* Release Right Foot Artery, Percutaneous Endoscopic Approach */
"04NW0ZZ"  =  "1"  /* Release Left Foot Artery, Open Approach */
"04NW3ZZ"  =  "1"  /* Release Left Foot Artery, Percutaneous Approach */
"04NW4ZZ"  =  "1"  /* Release Left Foot Artery, Percutaneous Endoscopic Approach */
"04NY0ZZ"  =  "1"  /* Release Lower Artery, Open Approach */
"04NY3ZZ"  =  "1"  /* Release Lower Artery, Percutaneous Approach */
"04NY4ZZ"  =  "1"  /* Release Lower Artery, Percutaneous Endoscopic Approach */
"04PY00Z"  =  "1"  /* Removal of Drainage Device from Lower Artery, Open Approach */
"04PY02Z"  =  "1"  /* Removal of Monitoring Device from Low Art, Open Approach */
"04PY03Z"  =  "1"  /* Removal of Infusion Device from Lower Artery, Open Approach */
"04PY07Z"  =  "1"  /* Removal of Autol Sub from Low Art, Open Approach */
"04PY0CZ"  =  "1"  /* Removal of Extraluminal Device from Low Art, Open Approach */
"04PY0DZ"  =  "1"  /* Removal of Intraluminal Device from Low Art, Open Approach */
"04PY0JZ"  =  "1"  /* Removal of Synthetic Substitute from Low Art, Open Approach */
"04PY0KZ"  =  "1"  /* Removal of Nonaut Sub from Low Art, Open Approach */
"04PY30Z"  =  "1"  /* Removal of Drainage Device from Lower Artery, Perc Approach */
"04PY32Z"  =  "1"  /* Removal of Monitoring Device from Low Art, Perc Approach */
"04PY33Z"  =  "1"  /* Removal of Infusion Device from Lower Artery, Perc Approach */
"04PY37Z"  =  "1"  /* Removal of Autol Sub from Low Art, Perc Approach */
"04PY3CZ"  =  "1"  /* Removal of Extraluminal Device from Low Art, Perc Approach */
"04PY3DZ"  =  "1"  /* Removal of Intraluminal Device from Low Art, Perc Approach */
"04PY3JZ"  =  "1"  /* Removal of Synthetic Substitute from Low Art, Perc Approach */
"04PY3KZ"  =  "1"  /* Removal of Nonaut Sub from Low Art, Perc Approach */
"04PY40Z"  =  "1"  /* Removal of Drainage Device from Low Art, Perc Endo Approach */
"04PY42Z"  =  "1"  /* Removal of Monitor Dev from Low Art, Perc Endo Approach */
"04PY43Z"  =  "1"  /* Removal of Infusion Device from Low Art, Perc Endo Approach */
"04PY47Z"  =  "1"  /* Removal of Autol Sub from Low Art, Perc Endo Approach */
"04PY4CZ"  =  "1"  /* Removal of Extralum Dev from Low Art, Perc Endo Approach */
"04PY4DZ"  =  "1"  /* Removal of Intralum Dev from Low Art, Perc Endo Approach */
"04PY4JZ"  =  "1"  /* Removal of Synth Sub from Low Art, Perc Endo Approach */
"04PY4KZ"  =  "1"  /* Removal of Nonaut Sub from Low Art, Perc Endo Approach */
"04Q00ZZ"  =  "1"  /* Repair Abdominal Aorta, Open Approach */
"04Q03ZZ"  =  "1"  /* Repair Abdominal Aorta, Percutaneous Approach */
"04Q04ZZ"  =  "1"  /* Repair Abdominal Aorta, Percutaneous Endoscopic Approach */
"04Q10ZZ"  =  "1"  /* Repair Celiac Artery, Open Approach */
"04Q13ZZ"  =  "1"  /* Repair Celiac Artery, Percutaneous Approach */
"04Q14ZZ"  =  "1"  /* Repair Celiac Artery, Percutaneous Endoscopic Approach */
"04Q20ZZ"  =  "1"  /* Repair Gastric Artery, Open Approach */
"04Q23ZZ"  =  "1"  /* Repair Gastric Artery, Percutaneous Approach */
"04Q24ZZ"  =  "1"  /* Repair Gastric Artery, Percutaneous Endoscopic Approach */
"04Q30ZZ"  =  "1"  /* Repair Hepatic Artery, Open Approach */
"04Q33ZZ"  =  "1"  /* Repair Hepatic Artery, Percutaneous Approach */
"04Q34ZZ"  =  "1"  /* Repair Hepatic Artery, Percutaneous Endoscopic Approach */
"04Q40ZZ"  =  "1"  /* Repair Splenic Artery, Open Approach */
"04Q43ZZ"  =  "1"  /* Repair Splenic Artery, Percutaneous Approach */
"04Q44ZZ"  =  "1"  /* Repair Splenic Artery, Percutaneous Endoscopic Approach */
"04Q50ZZ"  =  "1"  /* Repair Superior Mesenteric Artery, Open Approach */
"04Q53ZZ"  =  "1"  /* Repair Superior Mesenteric Artery, Percutaneous Approach */
"04Q54ZZ"  =  "1"  /* Repair Superior Mesenteric Artery, Perc Endo Approach */
"04Q60ZZ"  =  "1"  /* Repair Right Colic Artery, Open Approach */
"04Q63ZZ"  =  "1"  /* Repair Right Colic Artery, Percutaneous Approach */
"04Q64ZZ"  =  "1"  /* Repair Right Colic Artery, Percutaneous Endoscopic Approach */
"04Q70ZZ"  =  "1"  /* Repair Left Colic Artery, Open Approach */
"04Q73ZZ"  =  "1"  /* Repair Left Colic Artery, Percutaneous Approach */
"04Q74ZZ"  =  "1"  /* Repair Left Colic Artery, Percutaneous Endoscopic Approach */
"04Q80ZZ"  =  "1"  /* Repair Middle Colic Artery, Open Approach */
"04Q83ZZ"  =  "1"  /* Repair Middle Colic Artery, Percutaneous Approach */
"04Q84ZZ"  =  "1"  /* Repair Middle Colic Artery, Percutaneous Endoscopic Approach */
"04Q90ZZ"  =  "1"  /* Repair Right Renal Artery, Open Approach */
"04Q93ZZ"  =  "1"  /* Repair Right Renal Artery, Percutaneous Approach */
"04Q94ZZ"  =  "1"  /* Repair Right Renal Artery, Percutaneous Endoscopic Approach */
"04QA0ZZ"  =  "1"  /* Repair Left Renal Artery, Open Approach */
"04QA3ZZ"  =  "1"  /* Repair Left Renal Artery, Percutaneous Approach */
"04QA4ZZ"  =  "1"  /* Repair Left Renal Artery, Percutaneous Endoscopic Approach */
"04QB0ZZ"  =  "1"  /* Repair Inferior Mesenteric Artery, Open Approach */
"04QB3ZZ"  =  "1"  /* Repair Inferior Mesenteric Artery, Percutaneous Approach */
"04QB4ZZ"  =  "1"  /* Repair Inferior Mesenteric Artery, Perc Endo Approach */
"04QC0ZZ"  =  "1"  /* Repair Right Common Iliac Artery, Open Approach */
"04QC3ZZ"  =  "1"  /* Repair Right Common Iliac Artery, Percutaneous Approach */
"04QC4ZZ"  =  "1"  /* Repair Right Common Iliac Artery, Perc Endo Approach */
"04QD0ZZ"  =  "1"  /* Repair Left Common Iliac Artery, Open Approach */
"04QD3ZZ"  =  "1"  /* Repair Left Common Iliac Artery, Percutaneous Approach */
"04QD4ZZ"  =  "1"  /* Repair Left Common Iliac Artery, Perc Endo Approach */
"04QE0ZZ"  =  "1"  /* Repair Right Internal Iliac Artery, Open Approach */
"04QE3ZZ"  =  "1"  /* Repair Right Internal Iliac Artery, Percutaneous Approach */
"04QE4ZZ"  =  "1"  /* Repair Right Internal Iliac Artery, Perc Endo Approach */
"04QF0ZZ"  =  "1"  /* Repair Left Internal Iliac Artery, Open Approach */
"04QF3ZZ"  =  "1"  /* Repair Left Internal Iliac Artery, Percutaneous Approach */
"04QF4ZZ"  =  "1"  /* Repair Left Internal Iliac Artery, Perc Endo Approach */
"04QH0ZZ"  =  "1"  /* Repair Right External Iliac Artery, Open Approach */
"04QH3ZZ"  =  "1"  /* Repair Right External Iliac Artery, Percutaneous Approach */
"04QH4ZZ"  =  "1"  /* Repair Right External Iliac Artery, Perc Endo Approach */
"04QJ0ZZ"  =  "1"  /* Repair Left External Iliac Artery, Open Approach */
"04QJ3ZZ"  =  "1"  /* Repair Left External Iliac Artery, Percutaneous Approach */
"04QJ4ZZ"  =  "1"  /* Repair Left External Iliac Artery, Perc Endo Approach */
"04QK0ZZ"  =  "1"  /* Repair Right Femoral Artery, Open Approach */
"04QK3ZZ"  =  "1"  /* Repair Right Femoral Artery, Percutaneous Approach */
"04QK4ZZ"  =  "1"  /* Repair Right Femoral Artery, Perc Endo Approach */
"04QL0ZZ"  =  "1"  /* Repair Left Femoral Artery, Open Approach */
"04QL3ZZ"  =  "1"  /* Repair Left Femoral Artery, Percutaneous Approach */
"04QL4ZZ"  =  "1"  /* Repair Left Femoral Artery, Percutaneous Endoscopic Approach */
"04QM0ZZ"  =  "1"  /* Repair Right Popliteal Artery, Open Approach */
"04QM3ZZ"  =  "1"  /* Repair Right Popliteal Artery, Percutaneous Approach */
"04QM4ZZ"  =  "1"  /* Repair Right Popliteal Artery, Perc Endo Approach */
"04QN0ZZ"  =  "1"  /* Repair Left Popliteal Artery, Open Approach */
"04QN3ZZ"  =  "1"  /* Repair Left Popliteal Artery, Percutaneous Approach */
"04QN4ZZ"  =  "1"  /* Repair Left Popliteal Artery, Perc Endo Approach */
"04QP0ZZ"  =  "1"  /* Repair Right Anterior Tibial Artery, Open Approach */
"04QP3ZZ"  =  "1"  /* Repair Right Anterior Tibial Artery, Percutaneous Approach */
"04QP4ZZ"  =  "1"  /* Repair Right Anterior Tibial Artery, Perc Endo Approach */
"04QQ0ZZ"  =  "1"  /* Repair Left Anterior Tibial Artery, Open Approach */
"04QQ3ZZ"  =  "1"  /* Repair Left Anterior Tibial Artery, Percutaneous Approach */
"04QQ4ZZ"  =  "1"  /* Repair Left Anterior Tibial Artery, Perc Endo Approach */
"04QR0ZZ"  =  "1"  /* Repair Right Posterior Tibial Artery, Open Approach */
"04QR3ZZ"  =  "1"  /* Repair Right Posterior Tibial Artery, Percutaneous Approach */
"04QR4ZZ"  =  "1"  /* Repair Right Posterior Tibial Artery, Perc Endo Approach */
"04QS0ZZ"  =  "1"  /* Repair Left Posterior Tibial Artery, Open Approach */
"04QS3ZZ"  =  "1"  /* Repair Left Posterior Tibial Artery, Percutaneous Approach */
"04QS4ZZ"  =  "1"  /* Repair Left Posterior Tibial Artery, Perc Endo Approach */
"04QT0ZZ"  =  "1"  /* Repair Right Peroneal Artery, Open Approach */
"04QT3ZZ"  =  "1"  /* Repair Right Peroneal Artery, Percutaneous Approach */
"04QT4ZZ"  =  "1"  /* Repair Right Peroneal Artery, Perc Endo Approach */
"04QU0ZZ"  =  "1"  /* Repair Left Peroneal Artery, Open Approach */
"04QU3ZZ"  =  "1"  /* Repair Left Peroneal Artery, Percutaneous Approach */
"04QU4ZZ"  =  "1"  /* Repair Left Peroneal Artery, Perc Endo Approach */
"04QV0ZZ"  =  "1"  /* Repair Right Foot Artery, Open Approach */
"04QV3ZZ"  =  "1"  /* Repair Right Foot Artery, Percutaneous Approach */
"04QV4ZZ"  =  "1"  /* Repair Right Foot Artery, Percutaneous Endoscopic Approach */
"04QW0ZZ"  =  "1"  /* Repair Left Foot Artery, Open Approach */
"04QW3ZZ"  =  "1"  /* Repair Left Foot Artery, Percutaneous Approach */
"04QW4ZZ"  =  "1"  /* Repair Left Foot Artery, Percutaneous Endoscopic Approach */
"04QY0ZZ"  =  "1"  /* Repair Lower Artery, Open Approach */
"04QY3ZZ"  =  "1"  /* Repair Lower Artery, Percutaneous Approach */
"04QY4ZZ"  =  "1"  /* Repair Lower Artery, Percutaneous Endoscopic Approach */
"04R007Z"  =  "1"  /* Replacement of Abdominal Aorta with Autol Sub, Open Approach */
"04R00JZ"  =  "1"  /* Replacement of Abdominal Aorta with Synth Sub, Open Approach */
"04R00KZ"  =  "1"  /* Replacement of Abd Aorta with Nonaut Sub, Open Approach */
"04R047Z"  =  "1"  /* Replacement of Abd Aorta with Autol Sub, Perc Endo Approach */
"04R04JZ"  =  "1"  /* Replacement of Abd Aorta with Synth Sub, Perc Endo Approach */
"04R04KZ"  =  "1"  /* Replacement of Abd Aorta with Nonaut Sub, Perc Endo Approach */
"04R107Z"  =  "1"  /* Replacement of Celiac Artery with Autol Sub, Open Approach */
"04R10JZ"  =  "1"  /* Replacement of Celiac Artery with Synth Sub, Open Approach */
"04R10KZ"  =  "1"  /* Replacement of Celiac Artery with Nonaut Sub, Open Approach */
"04R147Z"  =  "1"  /* Replacement of Celiac Art with Autol Sub, Perc Endo Approach */
"04R14JZ"  =  "1"  /* Replacement of Celiac Art with Synth Sub, Perc Endo Approach */
"04R14KZ"  =  "1"  /* Replace of Celiac Art with Nonaut Sub, Perc Endo Approach */
"04R207Z"  =  "1"  /* Replacement of Gastric Artery with Autol Sub, Open Approach */
"04R20JZ"  =  "1"  /* Replacement of Gastric Artery with Synth Sub, Open Approach */
"04R20KZ"  =  "1"  /* Replacement of Gastric Artery with Nonaut Sub, Open Approach */
"04R247Z"  =  "1"  /* Replace of Gastric Art with Autol Sub, Perc Endo Approach */
"04R24JZ"  =  "1"  /* Replace of Gastric Art with Synth Sub, Perc Endo Approach */
"04R24KZ"  =  "1"  /* Replace of Gastric Art with Nonaut Sub, Perc Endo Approach */
"04R307Z"  =  "1"  /* Replacement of Hepatic Artery with Autol Sub, Open Approach */
"04R30JZ"  =  "1"  /* Replacement of Hepatic Artery with Synth Sub, Open Approach */
"04R30KZ"  =  "1"  /* Replacement of Hepatic Artery with Nonaut Sub, Open Approach */
"04R347Z"  =  "1"  /* Replace of Hepatic Art with Autol Sub, Perc Endo Approach */
"04R34JZ"  =  "1"  /* Replace of Hepatic Art with Synth Sub, Perc Endo Approach */
"04R34KZ"  =  "1"  /* Replace of Hepatic Art with Nonaut Sub, Perc Endo Approach */
"04R407Z"  =  "1"  /* Replacement of Splenic Artery with Autol Sub, Open Approach */
"04R40JZ"  =  "1"  /* Replacement of Splenic Artery with Synth Sub, Open Approach */
"04R40KZ"  =  "1"  /* Replacement of Splenic Artery with Nonaut Sub, Open Approach */
"04R447Z"  =  "1"  /* Replace of Splenic Art with Autol Sub, Perc Endo Approach */
"04R44JZ"  =  "1"  /* Replace of Splenic Art with Synth Sub, Perc Endo Approach */
"04R44KZ"  =  "1"  /* Replace of Splenic Art with Nonaut Sub, Perc Endo Approach */
"04R507Z"  =  "1"  /* Replacement of Sup Mesent Art with Autol Sub, Open Approach */
"04R50JZ"  =  "1"  /* Replacement of Sup Mesent Art with Synth Sub, Open Approach */
"04R50KZ"  =  "1"  /* Replacement of Sup Mesent Art with Nonaut Sub, Open Approach */
"04R547Z"  =  "1"  /* Replace of Sup Mesent Art with Autol Sub, Perc Endo Approach */
"04R54JZ"  =  "1"  /* Replace of Sup Mesent Art with Synth Sub, Perc Endo Approach */
"04R54KZ"  =  "1"  /* Replace Sup Mesent Art w Nonaut Sub, Perc Endo */
"04R607Z"  =  "1"  /* Replacement of R Colic Art with Autol Sub, Open Approach */
"04R60JZ"  =  "1"  /* Replacement of R Colic Art with Synth Sub, Open Approach */
"04R60KZ"  =  "1"  /* Replacement of R Colic Art with Nonaut Sub, Open Approach */
"04R647Z"  =  "1"  /* Replace of R Colic Art with Autol Sub, Perc Endo Approach */
"04R64JZ"  =  "1"  /* Replace of R Colic Art with Synth Sub, Perc Endo Approach */
"04R64KZ"  =  "1"  /* Replace of R Colic Art with Nonaut Sub, Perc Endo Approach */
"04R707Z"  =  "1"  /* Replacement of L Colic Art with Autol Sub, Open Approach */
"04R70JZ"  =  "1"  /* Replacement of L Colic Art with Synth Sub, Open Approach */
"04R70KZ"  =  "1"  /* Replacement of L Colic Art with Nonaut Sub, Open Approach */
"04R747Z"  =  "1"  /* Replace of L Colic Art with Autol Sub, Perc Endo Approach */
"04R74JZ"  =  "1"  /* Replace of L Colic Art with Synth Sub, Perc Endo Approach */
"04R74KZ"  =  "1"  /* Replace of L Colic Art with Nonaut Sub, Perc Endo Approach */
"04R807Z"  =  "1"  /* Replacement of Mid Colic Art with Autol Sub, Open Approach */
"04R80JZ"  =  "1"  /* Replacement of Mid Colic Art with Synth Sub, Open Approach */
"04R80KZ"  =  "1"  /* Replacement of Mid Colic Art with Nonaut Sub, Open Approach */
"04R847Z"  =  "1"  /* Replace of Mid Colic Art with Autol Sub, Perc Endo Approach */
"04R84JZ"  =  "1"  /* Replace of Mid Colic Art with Synth Sub, Perc Endo Approach */
"04R84KZ"  =  "1"  /* Replace of Mid Colic Art with Nonaut Sub, Perc Endo Approach */
"04R907Z"  =  "1"  /* Replacement of R Renal Art with Autol Sub, Open Approach */
"04R90JZ"  =  "1"  /* Replacement of R Renal Art with Synth Sub, Open Approach */
"04R90KZ"  =  "1"  /* Replacement of R Renal Art with Nonaut Sub, Open Approach */
"04R947Z"  =  "1"  /* Replace of R Renal Art with Autol Sub, Perc Endo Approach */
"04R94JZ"  =  "1"  /* Replace of R Renal Art with Synth Sub, Perc Endo Approach */
"04R94KZ"  =  "1"  /* Replace of R Renal Art with Nonaut Sub, Perc Endo Approach */
"04RA07Z"  =  "1"  /* Replacement of L Renal Art with Autol Sub, Open Approach */
"04RA0JZ"  =  "1"  /* Replacement of L Renal Art with Synth Sub, Open Approach */
"04RA0KZ"  =  "1"  /* Replacement of L Renal Art with Nonaut Sub, Open Approach */
"04RA47Z"  =  "1"  /* Replace of L Renal Art with Autol Sub, Perc Endo Approach */
"04RA4JZ"  =  "1"  /* Replace of L Renal Art with Synth Sub, Perc Endo Approach */
"04RA4KZ"  =  "1"  /* Replace of L Renal Art with Nonaut Sub, Perc Endo Approach */
"04RB07Z"  =  "1"  /* Replacement of Inf Mesent Art with Autol Sub, Open Approach */
"04RB0JZ"  =  "1"  /* Replacement of Inf Mesent Art with Synth Sub, Open Approach */
"04RB0KZ"  =  "1"  /* Replacement of Inf Mesent Art with Nonaut Sub, Open Approach */
"04RB47Z"  =  "1"  /* Replace of Inf Mesent Art with Autol Sub, Perc Endo Approach */
"04RB4JZ"  =  "1"  /* Replace of Inf Mesent Art with Synth Sub, Perc Endo Approach */
"04RB4KZ"  =  "1"  /* Replace Inf Mesent Art w Nonaut Sub, Perc Endo */
"04RC07Z"  =  "1"  /* Replacement of R Com Iliac Art with Autol Sub, Open Approach */
"04RC0JZ"  =  "1"  /* Replacement of R Com Iliac Art with Synth Sub, Open Approach */
"04RC0KZ"  =  "1"  /* Replace of R Com Iliac Art with Nonaut Sub, Open Approach */
"04RC47Z"  =  "1"  /* Replace R Com Iliac Art w Autol Sub, Perc Endo */
"04RC4JZ"  =  "1"  /* Replace R Com Iliac Art w Synth Sub, Perc Endo */
"04RC4KZ"  =  "1"  /* Replace R Com Iliac Art w Nonaut Sub, Perc Endo */
"04RD07Z"  =  "1"  /* Replacement of L Com Iliac Art with Autol Sub, Open Approach */
"04RD0JZ"  =  "1"  /* Replacement of L Com Iliac Art with Synth Sub, Open Approach */
"04RD0KZ"  =  "1"  /* Replace of L Com Iliac Art with Nonaut Sub, Open Approach */
"04RD47Z"  =  "1"  /* Replace L Com Iliac Art w Autol Sub, Perc Endo */
"04RD4JZ"  =  "1"  /* Replace L Com Iliac Art w Synth Sub, Perc Endo */
"04RD4KZ"  =  "1"  /* Replace L Com Iliac Art w Nonaut Sub, Perc Endo */
"04RE07Z"  =  "1"  /* Replacement of R Int Iliac Art with Autol Sub, Open Approach */
"04RE0JZ"  =  "1"  /* Replacement of R Int Iliac Art with Synth Sub, Open Approach */
"04RE0KZ"  =  "1"  /* Replace of R Int Iliac Art with Nonaut Sub, Open Approach */
"04RE47Z"  =  "1"  /* Replace R Int Iliac Art w Autol Sub, Perc Endo */
"04RE4JZ"  =  "1"  /* Replace R Int Iliac Art w Synth Sub, Perc Endo */
"04RE4KZ"  =  "1"  /* Replace R Int Iliac Art w Nonaut Sub, Perc Endo */
"04RF07Z"  =  "1"  /* Replacement of L Int Iliac Art with Autol Sub, Open Approach */
"04RF0JZ"  =  "1"  /* Replacement of L Int Iliac Art with Synth Sub, Open Approach */
"04RF0KZ"  =  "1"  /* Replace of L Int Iliac Art with Nonaut Sub, Open Approach */
"04RF47Z"  =  "1"  /* Replace L Int Iliac Art w Autol Sub, Perc Endo */
"04RF4JZ"  =  "1"  /* Replace L Int Iliac Art w Synth Sub, Perc Endo */
"04RF4KZ"  =  "1"  /* Replace L Int Iliac Art w Nonaut Sub, Perc Endo */
"04RH07Z"  =  "1"  /* Replacement of R Ext Iliac Art with Autol Sub, Open Approach */
"04RH0JZ"  =  "1"  /* Replacement of R Ext Iliac Art with Synth Sub, Open Approach */
"04RH0KZ"  =  "1"  /* Replace of R Ext Iliac Art with Nonaut Sub, Open Approach */
"04RH47Z"  =  "1"  /* Replace R Ext Iliac Art w Autol Sub, Perc Endo */
"04RH4JZ"  =  "1"  /* Replace R Ext Iliac Art w Synth Sub, Perc Endo */
"04RH4KZ"  =  "1"  /* Replace R Ext Iliac Art w Nonaut Sub, Perc Endo */
"04RJ07Z"  =  "1"  /* Replacement of L Ext Iliac Art with Autol Sub, Open Approach */
"04RJ0JZ"  =  "1"  /* Replacement of L Ext Iliac Art with Synth Sub, Open Approach */
"04RJ0KZ"  =  "1"  /* Replace of L Ext Iliac Art with Nonaut Sub, Open Approach */
"04RJ47Z"  =  "1"  /* Replace L Ext Iliac Art w Autol Sub, Perc Endo */
"04RJ4JZ"  =  "1"  /* Replace L Ext Iliac Art w Synth Sub, Perc Endo */
"04RJ4KZ"  =  "1"  /* Replace L Ext Iliac Art w Nonaut Sub, Perc Endo */
"04RK07Z"  =  "1"  /* Replacement of R Femor Art with Autol Sub, Open Approach */
"04RK0JZ"  =  "1"  /* Replacement of R Femor Art with Synth Sub, Open Approach */
"04RK0KZ"  =  "1"  /* Replacement of R Femor Art with Nonaut Sub, Open Approach */
"04RK47Z"  =  "1"  /* Replace of R Femor Art with Autol Sub, Perc Endo Approach */
"04RK4JZ"  =  "1"  /* Replace of R Femor Art with Synth Sub, Perc Endo Approach */
"04RK4KZ"  =  "1"  /* Replace of R Femor Art with Nonaut Sub, Perc Endo Approach */
"04RL07Z"  =  "1"  /* Replacement of L Femor Art with Autol Sub, Open Approach */
"04RL0JZ"  =  "1"  /* Replacement of L Femor Art with Synth Sub, Open Approach */
"04RL0KZ"  =  "1"  /* Replacement of L Femor Art with Nonaut Sub, Open Approach */
"04RL47Z"  =  "1"  /* Replace of L Femor Art with Autol Sub, Perc Endo Approach */
"04RL4JZ"  =  "1"  /* Replace of L Femor Art with Synth Sub, Perc Endo Approach */
"04RL4KZ"  =  "1"  /* Replace of L Femor Art with Nonaut Sub, Perc Endo Approach */
"04RM07Z"  =  "1"  /* Replacement of R Poplit Art with Autol Sub, Open Approach */
"04RM0JZ"  =  "1"  /* Replacement of R Poplit Art with Synth Sub, Open Approach */
"04RM0KZ"  =  "1"  /* Replacement of R Poplit Art with Nonaut Sub, Open Approach */
"04RM47Z"  =  "1"  /* Replace of R Poplit Art with Autol Sub, Perc Endo Approach */
"04RM4JZ"  =  "1"  /* Replace of R Poplit Art with Synth Sub, Perc Endo Approach */
"04RM4KZ"  =  "1"  /* Replace of R Poplit Art with Nonaut Sub, Perc Endo Approach */
"04RN07Z"  =  "1"  /* Replacement of L Poplit Art with Autol Sub, Open Approach */
"04RN0JZ"  =  "1"  /* Replacement of L Poplit Art with Synth Sub, Open Approach */
"04RN0KZ"  =  "1"  /* Replacement of L Poplit Art with Nonaut Sub, Open Approach */
"04RN47Z"  =  "1"  /* Replace of L Poplit Art with Autol Sub, Perc Endo Approach */
"04RN4JZ"  =  "1"  /* Replace of L Poplit Art with Synth Sub, Perc Endo Approach */
"04RN4KZ"  =  "1"  /* Replace of L Poplit Art with Nonaut Sub, Perc Endo Approach */
"04RP07Z"  =  "1"  /* Replacement of R Ant Tib Art with Autol Sub, Open Approach */
"04RP0JZ"  =  "1"  /* Replacement of R Ant Tib Art with Synth Sub, Open Approach */
"04RP0KZ"  =  "1"  /* Replacement of R Ant Tib Art with Nonaut Sub, Open Approach */
"04RP47Z"  =  "1"  /* Replace of R Ant Tib Art with Autol Sub, Perc Endo Approach */
"04RP4JZ"  =  "1"  /* Replace of R Ant Tib Art with Synth Sub, Perc Endo Approach */
"04RP4KZ"  =  "1"  /* Replace of R Ant Tib Art with Nonaut Sub, Perc Endo Approach */
"04RQ07Z"  =  "1"  /* Replacement of L Ant Tib Art with Autol Sub, Open Approach */
"04RQ0JZ"  =  "1"  /* Replacement of L Ant Tib Art with Synth Sub, Open Approach */
"04RQ0KZ"  =  "1"  /* Replacement of L Ant Tib Art with Nonaut Sub, Open Approach */
"04RQ47Z"  =  "1"  /* Replace of L Ant Tib Art with Autol Sub, Perc Endo Approach */
"04RQ4JZ"  =  "1"  /* Replace of L Ant Tib Art with Synth Sub, Perc Endo Approach */
"04RQ4KZ"  =  "1"  /* Replace of L Ant Tib Art with Nonaut Sub, Perc Endo Approach */
"04RR07Z"  =  "1"  /* Replacement of R Post Tib Art with Autol Sub, Open Approach */
"04RR0JZ"  =  "1"  /* Replacement of R Post Tib Art with Synth Sub, Open Approach */
"04RR0KZ"  =  "1"  /* Replacement of R Post Tib Art with Nonaut Sub, Open Approach */
"04RR47Z"  =  "1"  /* Replace of R Post Tib Art with Autol Sub, Perc Endo Approach */
"04RR4JZ"  =  "1"  /* Replace of R Post Tib Art with Synth Sub, Perc Endo Approach */
"04RR4KZ"  =  "1"  /* Replace R Post Tib Art w Nonaut Sub, Perc Endo */
"04RS07Z"  =  "1"  /* Replacement of L Post Tib Art with Autol Sub, Open Approach */
"04RS0JZ"  =  "1"  /* Replacement of L Post Tib Art with Synth Sub, Open Approach */
"04RS0KZ"  =  "1"  /* Replacement of L Post Tib Art with Nonaut Sub, Open Approach */
"04RS47Z"  =  "1"  /* Replace of L Post Tib Art with Autol Sub, Perc Endo Approach */
"04RS4JZ"  =  "1"  /* Replace of L Post Tib Art with Synth Sub, Perc Endo Approach */
"04RS4KZ"  =  "1"  /* Replace L Post Tib Art w Nonaut Sub, Perc Endo */
"04RT07Z"  =  "1"  /* Replacement of R Peroneal Art with Autol Sub, Open Approach */
"04RT0JZ"  =  "1"  /* Replacement of R Peroneal Art with Synth Sub, Open Approach */
"04RT0KZ"  =  "1"  /* Replacement of R Peroneal Art with Nonaut Sub, Open Approach */
"04RT47Z"  =  "1"  /* Replace of R Peroneal Art with Autol Sub, Perc Endo Approach */
"04RT4JZ"  =  "1"  /* Replace of R Peroneal Art with Synth Sub, Perc Endo Approach */
"04RT4KZ"  =  "1"  /* Replace R Peroneal Art w Nonaut Sub, Perc Endo */
"04RU07Z"  =  "1"  /* Replacement of L Peroneal Art with Autol Sub, Open Approach */
"04RU0JZ"  =  "1"  /* Replacement of L Peroneal Art with Synth Sub, Open Approach */
"04RU0KZ"  =  "1"  /* Replacement of L Peroneal Art with Nonaut Sub, Open Approach */
"04RU47Z"  =  "1"  /* Replace of L Peroneal Art with Autol Sub, Perc Endo Approach */
"04RU4JZ"  =  "1"  /* Replace of L Peroneal Art with Synth Sub, Perc Endo Approach */
"04RU4KZ"  =  "1"  /* Replace L Peroneal Art w Nonaut Sub, Perc Endo */
"04RV07Z"  =  "1"  /* Replacement of R Foot Art with Autol Sub, Open Approach */
"04RV0JZ"  =  "1"  /* Replacement of R Foot Art with Synth Sub, Open Approach */
"04RV0KZ"  =  "1"  /* Replacement of R Foot Art with Nonaut Sub, Open Approach */
"04RV47Z"  =  "1"  /* Replacement of R Foot Art with Autol Sub, Perc Endo Approach */
"04RV4JZ"  =  "1"  /* Replacement of R Foot Art with Synth Sub, Perc Endo Approach */
"04RV4KZ"  =  "1"  /* Replace of R Foot Art with Nonaut Sub, Perc Endo Approach */
"04RW07Z"  =  "1"  /* Replacement of L Foot Art with Autol Sub, Open Approach */
"04RW0JZ"  =  "1"  /* Replacement of L Foot Art with Synth Sub, Open Approach */
"04RW0KZ"  =  "1"  /* Replacement of L Foot Art with Nonaut Sub, Open Approach */
"04RW47Z"  =  "1"  /* Replacement of L Foot Art with Autol Sub, Perc Endo Approach */
"04RW4JZ"  =  "1"  /* Replacement of L Foot Art with Synth Sub, Perc Endo Approach */
"04RW4KZ"  =  "1"  /* Replace of L Foot Art with Nonaut Sub, Perc Endo Approach */
"04RY07Z"  =  "1"  /* Replacement of Lower Artery with Autol Sub, Open Approach */
"04RY0JZ"  =  "1"  /* Replacement of Lower Artery with Synth Sub, Open Approach */
"04RY0KZ"  =  "1"  /* Replacement of Lower Artery with Nonaut Sub, Open Approach */
"04RY47Z"  =  "1"  /* Replacement of Low Art with Autol Sub, Perc Endo Approach */
"04RY4JZ"  =  "1"  /* Replacement of Low Art with Synth Sub, Perc Endo Approach */
"04RY4KZ"  =  "1"  /* Replacement of Low Art with Nonaut Sub, Perc Endo Approach */
"04S00ZZ"  =  "1"  /* Reposition Abdominal Aorta, Open Approach */
"04S03ZZ"  =  "1"  /* Reposition Abdominal Aorta, Percutaneous Approach */
"04S04ZZ"  =  "1"  /* Reposition Abdominal Aorta, Percutaneous Endoscopic Approach */
"04S10ZZ"  =  "1"  /* Reposition Celiac Artery, Open Approach */
"04S13ZZ"  =  "1"  /* Reposition Celiac Artery, Percutaneous Approach */
"04S14ZZ"  =  "1"  /* Reposition Celiac Artery, Percutaneous Endoscopic Approach */
"04S20ZZ"  =  "1"  /* Reposition Gastric Artery, Open Approach */
"04S23ZZ"  =  "1"  /* Reposition Gastric Artery, Percutaneous Approach */
"04S24ZZ"  =  "1"  /* Reposition Gastric Artery, Percutaneous Endoscopic Approach */
"04S30ZZ"  =  "1"  /* Reposition Hepatic Artery, Open Approach */
"04S33ZZ"  =  "1"  /* Reposition Hepatic Artery, Percutaneous Approach */
"04S34ZZ"  =  "1"  /* Reposition Hepatic Artery, Percutaneous Endoscopic Approach */
"04S40ZZ"  =  "1"  /* Reposition Splenic Artery, Open Approach */
"04S43ZZ"  =  "1"  /* Reposition Splenic Artery, Percutaneous Approach */
"04S44ZZ"  =  "1"  /* Reposition Splenic Artery, Percutaneous Endoscopic Approach */
"04S50ZZ"  =  "1"  /* Reposition Superior Mesenteric Artery, Open Approach */
"04S53ZZ"  =  "1"  /* Reposition Superior Mesenteric Artery, Percutaneous Approach */
"04S54ZZ"  =  "1"  /* Reposition Superior Mesenteric Artery, Perc Endo Approach */
"04S60ZZ"  =  "1"  /* Reposition Right Colic Artery, Open Approach */
"04S63ZZ"  =  "1"  /* Reposition Right Colic Artery, Percutaneous Approach */
"04S64ZZ"  =  "1"  /* Reposition Right Colic Artery, Perc Endo Approach */
"04S70ZZ"  =  "1"  /* Reposition Left Colic Artery, Open Approach */
"04S73ZZ"  =  "1"  /* Reposition Left Colic Artery, Percutaneous Approach */
"04S74ZZ"  =  "1"  /* Reposition Left Colic Artery, Perc Endo Approach */
"04S80ZZ"  =  "1"  /* Reposition Middle Colic Artery, Open Approach */
"04S83ZZ"  =  "1"  /* Reposition Middle Colic Artery, Percutaneous Approach */
"04S84ZZ"  =  "1"  /* Reposition Middle Colic Artery, Perc Endo Approach */
"04S90ZZ"  =  "1"  /* Reposition Right Renal Artery, Open Approach */
"04S93ZZ"  =  "1"  /* Reposition Right Renal Artery, Percutaneous Approach */
"04S94ZZ"  =  "1"  /* Reposition Right Renal Artery, Perc Endo Approach */
"04SA0ZZ"  =  "1"  /* Reposition Left Renal Artery, Open Approach */
"04SA3ZZ"  =  "1"  /* Reposition Left Renal Artery, Percutaneous Approach */
"04SA4ZZ"  =  "1"  /* Reposition Left Renal Artery, Perc Endo Approach */
"04SB0ZZ"  =  "1"  /* Reposition Inferior Mesenteric Artery, Open Approach */
"04SB3ZZ"  =  "1"  /* Reposition Inferior Mesenteric Artery, Percutaneous Approach */
"04SB4ZZ"  =  "1"  /* Reposition Inferior Mesenteric Artery, Perc Endo Approach */
"04SC0ZZ"  =  "1"  /* Reposition Right Common Iliac Artery, Open Approach */
"04SC3ZZ"  =  "1"  /* Reposition Right Common Iliac Artery, Percutaneous Approach */
"04SC4ZZ"  =  "1"  /* Reposition Right Common Iliac Artery, Perc Endo Approach */
"04SD0ZZ"  =  "1"  /* Reposition Left Common Iliac Artery, Open Approach */
"04SD3ZZ"  =  "1"  /* Reposition Left Common Iliac Artery, Percutaneous Approach */
"04SD4ZZ"  =  "1"  /* Reposition Left Common Iliac Artery, Perc Endo Approach */
"04SE0ZZ"  =  "1"  /* Reposition Right Internal Iliac Artery, Open Approach */
"04SE3ZZ"  =  "1"  /* Reposition Right Internal Iliac Artery, Perc Approach */
"04SE4ZZ"  =  "1"  /* Reposition Right Internal Iliac Artery, Perc Endo Approach */
"04SF0ZZ"  =  "1"  /* Reposition Left Internal Iliac Artery, Open Approach */
"04SF3ZZ"  =  "1"  /* Reposition Left Internal Iliac Artery, Percutaneous Approach */
"04SF4ZZ"  =  "1"  /* Reposition Left Internal Iliac Artery, Perc Endo Approach */
"04SH0ZZ"  =  "1"  /* Reposition Right External Iliac Artery, Open Approach */
"04SH3ZZ"  =  "1"  /* Reposition Right External Iliac Artery, Perc Approach */
"04SH4ZZ"  =  "1"  /* Reposition Right External Iliac Artery, Perc Endo Approach */
"04SJ0ZZ"  =  "1"  /* Reposition Left External Iliac Artery, Open Approach */
"04SJ3ZZ"  =  "1"  /* Reposition Left External Iliac Artery, Percutaneous Approach */
"04SJ4ZZ"  =  "1"  /* Reposition Left External Iliac Artery, Perc Endo Approach */
"04SK0ZZ"  =  "1"  /* Reposition Right Femoral Artery, Open Approach */
"04SK3ZZ"  =  "1"  /* Reposition Right Femoral Artery, Percutaneous Approach */
"04SK4ZZ"  =  "1"  /* Reposition Right Femoral Artery, Perc Endo Approach */
"04SL0ZZ"  =  "1"  /* Reposition Left Femoral Artery, Open Approach */
"04SL3ZZ"  =  "1"  /* Reposition Left Femoral Artery, Percutaneous Approach */
"04SL4ZZ"  =  "1"  /* Reposition Left Femoral Artery, Perc Endo Approach */
"04SM0ZZ"  =  "1"  /* Reposition Right Popliteal Artery, Open Approach */
"04SM3ZZ"  =  "1"  /* Reposition Right Popliteal Artery, Percutaneous Approach */
"04SM4ZZ"  =  "1"  /* Reposition Right Popliteal Artery, Perc Endo Approach */
"04SN0ZZ"  =  "1"  /* Reposition Left Popliteal Artery, Open Approach */
"04SN3ZZ"  =  "1"  /* Reposition Left Popliteal Artery, Percutaneous Approach */
"04SN4ZZ"  =  "1"  /* Reposition Left Popliteal Artery, Perc Endo Approach */
"04SP0ZZ"  =  "1"  /* Reposition Right Anterior Tibial Artery, Open Approach */
"04SP3ZZ"  =  "1"  /* Reposition Right Anterior Tibial Artery, Perc Approach */
"04SP4ZZ"  =  "1"  /* Reposition Right Anterior Tibial Artery, Perc Endo Approach */
"04SQ0ZZ"  =  "1"  /* Reposition Left Anterior Tibial Artery, Open Approach */
"04SQ3ZZ"  =  "1"  /* Reposition Left Anterior Tibial Artery, Perc Approach */
"04SQ4ZZ"  =  "1"  /* Reposition Left Anterior Tibial Artery, Perc Endo Approach */
"04SR0ZZ"  =  "1"  /* Reposition Right Posterior Tibial Artery, Open Approach */
"04SR3ZZ"  =  "1"  /* Reposition Right Posterior Tibial Artery, Perc Approach */
"04SR4ZZ"  =  "1"  /* Reposition Right Posterior Tibial Artery, Perc Endo Approach */
"04SS0ZZ"  =  "1"  /* Reposition Left Posterior Tibial Artery, Open Approach */
"04SS3ZZ"  =  "1"  /* Reposition Left Posterior Tibial Artery, Perc Approach */
"04SS4ZZ"  =  "1"  /* Reposition Left Posterior Tibial Artery, Perc Endo Approach */
"04ST0ZZ"  =  "1"  /* Reposition Right Peroneal Artery, Open Approach */
"04ST3ZZ"  =  "1"  /* Reposition Right Peroneal Artery, Percutaneous Approach */
"04ST4ZZ"  =  "1"  /* Reposition Right Peroneal Artery, Perc Endo Approach */
"04SU0ZZ"  =  "1"  /* Reposition Left Peroneal Artery, Open Approach */
"04SU3ZZ"  =  "1"  /* Reposition Left Peroneal Artery, Percutaneous Approach */
"04SU4ZZ"  =  "1"  /* Reposition Left Peroneal Artery, Perc Endo Approach */
"04SV0ZZ"  =  "1"  /* Reposition Right Foot Artery, Open Approach */
"04SV3ZZ"  =  "1"  /* Reposition Right Foot Artery, Percutaneous Approach */
"04SV4ZZ"  =  "1"  /* Reposition Right Foot Artery, Perc Endo Approach */
"04SW0ZZ"  =  "1"  /* Reposition Left Foot Artery, Open Approach */
"04SW3ZZ"  =  "1"  /* Reposition Left Foot Artery, Percutaneous Approach */
"04SW4ZZ"  =  "1"  /* Reposition Left Foot Artery, Perc Endo Approach */
"04SY0ZZ"  =  "1"  /* Reposition Lower Artery, Open Approach */
"04SY3ZZ"  =  "1"  /* Reposition Lower Artery, Percutaneous Approach */
"04SY4ZZ"  =  "1"  /* Reposition Lower Artery, Percutaneous Endoscopic Approach */
"04U007Z"  =  "1"  /* Supplement Abdominal Aorta with Autol Sub, Open Approach */
"04U00JZ"  =  "1"  /* Supplement Abdominal Aorta with Synth Sub, Open Approach */
"04U00KZ"  =  "1"  /* Supplement Abdominal Aorta with Nonaut Sub, Open Approach */
"04U037Z"  =  "1"  /* Supplement Abdominal Aorta with Autol Sub, Perc Approach */
"04U03JZ"  =  "1"  /* Supplement Abdominal Aorta with Synth Sub, Perc Approach */
"04U03KZ"  =  "1"  /* Supplement Abdominal Aorta with Nonaut Sub, Perc Approach */
"04U047Z"  =  "1"  /* Supplement Abd Aorta with Autol Sub, Perc Endo Approach */
"04U04JZ"  =  "1"  /* Supplement Abd Aorta with Synth Sub, Perc Endo Approach */
"04U04KZ"  =  "1"  /* Supplement Abd Aorta with Nonaut Sub, Perc Endo Approach */
"04U107Z"  =  "1"  /* Supplement Celiac Artery with Autol Sub, Open Approach */
"04U10JZ"  =  "1"  /* Supplement Celiac Artery with Synth Sub, Open Approach */
"04U10KZ"  =  "1"  /* Supplement Celiac Artery with Nonaut Sub, Open Approach */
"04U137Z"  =  "1"  /* Supplement Celiac Artery with Autol Sub, Perc Approach */
"04U13JZ"  =  "1"  /* Supplement Celiac Artery with Synth Sub, Perc Approach */
"04U13KZ"  =  "1"  /* Supplement Celiac Artery with Nonaut Sub, Perc Approach */
"04U147Z"  =  "1"  /* Supplement Celiac Artery with Autol Sub, Perc Endo Approach */
"04U14JZ"  =  "1"  /* Supplement Celiac Artery with Synth Sub, Perc Endo Approach */
"04U14KZ"  =  "1"  /* Supplement Celiac Artery with Nonaut Sub, Perc Endo Approach */
"04U207Z"  =  "1"  /* Supplement Gastric Artery with Autol Sub, Open Approach */
"04U20JZ"  =  "1"  /* Supplement Gastric Artery with Synth Sub, Open Approach */
"04U20KZ"  =  "1"  /* Supplement Gastric Artery with Nonaut Sub, Open Approach */
"04U237Z"  =  "1"  /* Supplement Gastric Artery with Autol Sub, Perc Approach */
"04U23JZ"  =  "1"  /* Supplement Gastric Artery with Synth Sub, Perc Approach */
"04U23KZ"  =  "1"  /* Supplement Gastric Artery with Nonaut Sub, Perc Approach */
"04U247Z"  =  "1"  /* Supplement Gastric Artery with Autol Sub, Perc Endo Approach */
"04U24JZ"  =  "1"  /* Supplement Gastric Artery with Synth Sub, Perc Endo Approach */
"04U24KZ"  =  "1"  /* Supplement Gastric Art with Nonaut Sub, Perc Endo Approach */
"04U307Z"  =  "1"  /* Supplement Hepatic Artery with Autol Sub, Open Approach */
"04U30JZ"  =  "1"  /* Supplement Hepatic Artery with Synth Sub, Open Approach */
"04U30KZ"  =  "1"  /* Supplement Hepatic Artery with Nonaut Sub, Open Approach */
"04U337Z"  =  "1"  /* Supplement Hepatic Artery with Autol Sub, Perc Approach */
"04U33JZ"  =  "1"  /* Supplement Hepatic Artery with Synth Sub, Perc Approach */
"04U33KZ"  =  "1"  /* Supplement Hepatic Artery with Nonaut Sub, Perc Approach */
"04U347Z"  =  "1"  /* Supplement Hepatic Artery with Autol Sub, Perc Endo Approach */
"04U34JZ"  =  "1"  /* Supplement Hepatic Artery with Synth Sub, Perc Endo Approach */
"04U34KZ"  =  "1"  /* Supplement Hepatic Art with Nonaut Sub, Perc Endo Approach */
"04U407Z"  =  "1"  /* Supplement Splenic Artery with Autol Sub, Open Approach */
"04U40JZ"  =  "1"  /* Supplement Splenic Artery with Synth Sub, Open Approach */
"04U40KZ"  =  "1"  /* Supplement Splenic Artery with Nonaut Sub, Open Approach */
"04U437Z"  =  "1"  /* Supplement Splenic Artery with Autol Sub, Perc Approach */
"04U43JZ"  =  "1"  /* Supplement Splenic Artery with Synth Sub, Perc Approach */
"04U43KZ"  =  "1"  /* Supplement Splenic Artery with Nonaut Sub, Perc Approach */
"04U447Z"  =  "1"  /* Supplement Splenic Artery with Autol Sub, Perc Endo Approach */
"04U44JZ"  =  "1"  /* Supplement Splenic Artery with Synth Sub, Perc Endo Approach */
"04U44KZ"  =  "1"  /* Supplement Splenic Art with Nonaut Sub, Perc Endo Approach */
"04U507Z"  =  "1"  /* Supplement Sup Mesent Art with Autol Sub, Open Approach */
"04U50JZ"  =  "1"  /* Supplement Sup Mesent Art with Synth Sub, Open Approach */
"04U50KZ"  =  "1"  /* Supplement Sup Mesent Art with Nonaut Sub, Open Approach */
"04U537Z"  =  "1"  /* Supplement Sup Mesent Art with Autol Sub, Perc Approach */
"04U53JZ"  =  "1"  /* Supplement Sup Mesent Art with Synth Sub, Perc Approach */
"04U53KZ"  =  "1"  /* Supplement Sup Mesent Art with Nonaut Sub, Perc Approach */
"04U547Z"  =  "1"  /* Supplement Sup Mesent Art with Autol Sub, Perc Endo Approach */
"04U54JZ"  =  "1"  /* Supplement Sup Mesent Art with Synth Sub, Perc Endo Approach */
"04U54KZ"  =  "1"  /* Supplement Sup Mesent Art w Nonaut Sub, Perc Endo */
"04U607Z"  =  "1"  /* Supplement Right Colic Artery with Autol Sub, Open Approach */
"04U60JZ"  =  "1"  /* Supplement Right Colic Artery with Synth Sub, Open Approach */
"04U60KZ"  =  "1"  /* Supplement Right Colic Artery with Nonaut Sub, Open Approach */
"04U637Z"  =  "1"  /* Supplement Right Colic Artery with Autol Sub, Perc Approach */
"04U63JZ"  =  "1"  /* Supplement Right Colic Artery with Synth Sub, Perc Approach */
"04U63KZ"  =  "1"  /* Supplement Right Colic Artery with Nonaut Sub, Perc Approach */
"04U647Z"  =  "1"  /* Supplement R Colic Art with Autol Sub, Perc Endo Approach */
"04U64JZ"  =  "1"  /* Supplement R Colic Art with Synth Sub, Perc Endo Approach */
"04U64KZ"  =  "1"  /* Supplement R Colic Art with Nonaut Sub, Perc Endo Approach */
"04U707Z"  =  "1"  /* Supplement Left Colic Artery with Autol Sub, Open Approach */
"04U70JZ"  =  "1"  /* Supplement Left Colic Artery with Synth Sub, Open Approach */
"04U70KZ"  =  "1"  /* Supplement Left Colic Artery with Nonaut Sub, Open Approach */
"04U737Z"  =  "1"  /* Supplement Left Colic Artery with Autol Sub, Perc Approach */
"04U73JZ"  =  "1"  /* Supplement Left Colic Artery with Synth Sub, Perc Approach */
"04U73KZ"  =  "1"  /* Supplement Left Colic Artery with Nonaut Sub, Perc Approach */
"04U747Z"  =  "1"  /* Supplement L Colic Art with Autol Sub, Perc Endo Approach */
"04U74JZ"  =  "1"  /* Supplement L Colic Art with Synth Sub, Perc Endo Approach */
"04U74KZ"  =  "1"  /* Supplement L Colic Art with Nonaut Sub, Perc Endo Approach */
"04U807Z"  =  "1"  /* Supplement Middle Colic Artery with Autol Sub, Open Approach */
"04U80JZ"  =  "1"  /* Supplement Middle Colic Artery with Synth Sub, Open Approach */
"04U80KZ"  =  "1"  /* Supplement Mid Colic Art with Nonaut Sub, Open Approach */
"04U837Z"  =  "1"  /* Supplement Middle Colic Artery with Autol Sub, Perc Approach */
"04U83JZ"  =  "1"  /* Supplement Middle Colic Artery with Synth Sub, Perc Approach */
"04U83KZ"  =  "1"  /* Supplement Mid Colic Art with Nonaut Sub, Perc Approach */
"04U847Z"  =  "1"  /* Supplement Mid Colic Art with Autol Sub, Perc Endo Approach */
"04U84JZ"  =  "1"  /* Supplement Mid Colic Art with Synth Sub, Perc Endo Approach */
"04U84KZ"  =  "1"  /* Supplement Mid Colic Art with Nonaut Sub, Perc Endo Approach */
"04U907Z"  =  "1"  /* Supplement Right Renal Artery with Autol Sub, Open Approach */
"04U90JZ"  =  "1"  /* Supplement Right Renal Artery with Synth Sub, Open Approach */
"04U90KZ"  =  "1"  /* Supplement Right Renal Artery with Nonaut Sub, Open Approach */
"04U937Z"  =  "1"  /* Supplement Right Renal Artery with Autol Sub, Perc Approach */
"04U93JZ"  =  "1"  /* Supplement Right Renal Artery with Synth Sub, Perc Approach */
"04U93KZ"  =  "1"  /* Supplement Right Renal Artery with Nonaut Sub, Perc Approach */
"04U947Z"  =  "1"  /* Supplement R Renal Art with Autol Sub, Perc Endo Approach */
"04U94JZ"  =  "1"  /* Supplement R Renal Art with Synth Sub, Perc Endo Approach */
"04U94KZ"  =  "1"  /* Supplement R Renal Art with Nonaut Sub, Perc Endo Approach */
"04UA07Z"  =  "1"  /* Supplement Left Renal Artery with Autol Sub, Open Approach */
"04UA0JZ"  =  "1"  /* Supplement Left Renal Artery with Synth Sub, Open Approach */
"04UA0KZ"  =  "1"  /* Supplement Left Renal Artery with Nonaut Sub, Open Approach */
"04UA37Z"  =  "1"  /* Supplement Left Renal Artery with Autol Sub, Perc Approach */
"04UA3JZ"  =  "1"  /* Supplement Left Renal Artery with Synth Sub, Perc Approach */
"04UA3KZ"  =  "1"  /* Supplement Left Renal Artery with Nonaut Sub, Perc Approach */
"04UA47Z"  =  "1"  /* Supplement L Renal Art with Autol Sub, Perc Endo Approach */
"04UA4JZ"  =  "1"  /* Supplement L Renal Art with Synth Sub, Perc Endo Approach */
"04UA4KZ"  =  "1"  /* Supplement L Renal Art with Nonaut Sub, Perc Endo Approach */
"04UB07Z"  =  "1"  /* Supplement Inf Mesent Art with Autol Sub, Open Approach */
"04UB0JZ"  =  "1"  /* Supplement Inf Mesent Art with Synth Sub, Open Approach */
"04UB0KZ"  =  "1"  /* Supplement Inf Mesent Art with Nonaut Sub, Open Approach */
"04UB37Z"  =  "1"  /* Supplement Inf Mesent Art with Autol Sub, Perc Approach */
"04UB3JZ"  =  "1"  /* Supplement Inf Mesent Art with Synth Sub, Perc Approach */
"04UB3KZ"  =  "1"  /* Supplement Inf Mesent Art with Nonaut Sub, Perc Approach */
"04UB47Z"  =  "1"  /* Supplement Inf Mesent Art with Autol Sub, Perc Endo Approach */
"04UB4JZ"  =  "1"  /* Supplement Inf Mesent Art with Synth Sub, Perc Endo Approach */
"04UB4KZ"  =  "1"  /* Supplement Inf Mesent Art w Nonaut Sub, Perc Endo */
"04UC07Z"  =  "1"  /* Supplement R Com Iliac Art with Autol Sub, Open Approach */
"04UC0JZ"  =  "1"  /* Supplement R Com Iliac Art with Synth Sub, Open Approach */
"04UC0KZ"  =  "1"  /* Supplement R Com Iliac Art with Nonaut Sub, Open Approach */
"04UC37Z"  =  "1"  /* Supplement R Com Iliac Art with Autol Sub, Perc Approach */
"04UC3JZ"  =  "1"  /* Supplement R Com Iliac Art with Synth Sub, Perc Approach */
"04UC3KZ"  =  "1"  /* Supplement R Com Iliac Art with Nonaut Sub, Perc Approach */
"04UC47Z"  =  "1"  /* Supplement R Com Iliac Art w Autol Sub, Perc Endo */
"04UC4JZ"  =  "1"  /* Supplement R Com Iliac Art w Synth Sub, Perc Endo */
"04UC4KZ"  =  "1"  /* Supplement R Com Iliac Art w Nonaut Sub, Perc Endo */
"04UD07Z"  =  "1"  /* Supplement L Com Iliac Art with Autol Sub, Open Approach */
"04UD0JZ"  =  "1"  /* Supplement L Com Iliac Art with Synth Sub, Open Approach */
"04UD0KZ"  =  "1"  /* Supplement L Com Iliac Art with Nonaut Sub, Open Approach */
"04UD37Z"  =  "1"  /* Supplement L Com Iliac Art with Autol Sub, Perc Approach */
"04UD3JZ"  =  "1"  /* Supplement L Com Iliac Art with Synth Sub, Perc Approach */
"04UD3KZ"  =  "1"  /* Supplement L Com Iliac Art with Nonaut Sub, Perc Approach */
"04UD47Z"  =  "1"  /* Supplement L Com Iliac Art w Autol Sub, Perc Endo */
"04UD4JZ"  =  "1"  /* Supplement L Com Iliac Art w Synth Sub, Perc Endo */
"04UD4KZ"  =  "1"  /* Supplement L Com Iliac Art w Nonaut Sub, Perc Endo */
"04UE07Z"  =  "1"  /* Supplement R Int Iliac Art with Autol Sub, Open Approach */
"04UE0JZ"  =  "1"  /* Supplement R Int Iliac Art with Synth Sub, Open Approach */
"04UE0KZ"  =  "1"  /* Supplement R Int Iliac Art with Nonaut Sub, Open Approach */
"04UE37Z"  =  "1"  /* Supplement R Int Iliac Art with Autol Sub, Perc Approach */
"04UE3JZ"  =  "1"  /* Supplement R Int Iliac Art with Synth Sub, Perc Approach */
"04UE3KZ"  =  "1"  /* Supplement R Int Iliac Art with Nonaut Sub, Perc Approach */
"04UE47Z"  =  "1"  /* Supplement R Int Iliac Art w Autol Sub, Perc Endo */
"04UE4JZ"  =  "1"  /* Supplement R Int Iliac Art w Synth Sub, Perc Endo */
"04UE4KZ"  =  "1"  /* Supplement R Int Iliac Art w Nonaut Sub, Perc Endo */
"04UF07Z"  =  "1"  /* Supplement L Int Iliac Art with Autol Sub, Open Approach */
"04UF0JZ"  =  "1"  /* Supplement L Int Iliac Art with Synth Sub, Open Approach */
"04UF0KZ"  =  "1"  /* Supplement L Int Iliac Art with Nonaut Sub, Open Approach */
"04UF37Z"  =  "1"  /* Supplement L Int Iliac Art with Autol Sub, Perc Approach */
"04UF3JZ"  =  "1"  /* Supplement L Int Iliac Art with Synth Sub, Perc Approach */
"04UF3KZ"  =  "1"  /* Supplement L Int Iliac Art with Nonaut Sub, Perc Approach */
"04UF47Z"  =  "1"  /* Supplement L Int Iliac Art w Autol Sub, Perc Endo */
"04UF4JZ"  =  "1"  /* Supplement L Int Iliac Art w Synth Sub, Perc Endo */
"04UF4KZ"  =  "1"  /* Supplement L Int Iliac Art w Nonaut Sub, Perc Endo */
"04UH07Z"  =  "1"  /* Supplement R Ext Iliac Art with Autol Sub, Open Approach */
"04UH0JZ"  =  "1"  /* Supplement R Ext Iliac Art with Synth Sub, Open Approach */
"04UH0KZ"  =  "1"  /* Supplement R Ext Iliac Art with Nonaut Sub, Open Approach */
"04UH37Z"  =  "1"  /* Supplement R Ext Iliac Art with Autol Sub, Perc Approach */
"04UH3JZ"  =  "1"  /* Supplement R Ext Iliac Art with Synth Sub, Perc Approach */
"04UH3KZ"  =  "1"  /* Supplement R Ext Iliac Art with Nonaut Sub, Perc Approach */
"04UH47Z"  =  "1"  /* Supplement R Ext Iliac Art w Autol Sub, Perc Endo */
"04UH4JZ"  =  "1"  /* Supplement R Ext Iliac Art w Synth Sub, Perc Endo */
"04UH4KZ"  =  "1"  /* Supplement R Ext Iliac Art w Nonaut Sub, Perc Endo */
"04UJ07Z"  =  "1"  /* Supplement L Ext Iliac Art with Autol Sub, Open Approach */
"04UJ0JZ"  =  "1"  /* Supplement L Ext Iliac Art with Synth Sub, Open Approach */
"04UJ0KZ"  =  "1"  /* Supplement L Ext Iliac Art with Nonaut Sub, Open Approach */
"04UJ37Z"  =  "1"  /* Supplement L Ext Iliac Art with Autol Sub, Perc Approach */
"04UJ3JZ"  =  "1"  /* Supplement L Ext Iliac Art with Synth Sub, Perc Approach */
"04UJ3KZ"  =  "1"  /* Supplement L Ext Iliac Art with Nonaut Sub, Perc Approach */
"04UJ47Z"  =  "1"  /* Supplement L Ext Iliac Art w Autol Sub, Perc Endo */
"04UJ4JZ"  =  "1"  /* Supplement L Ext Iliac Art w Synth Sub, Perc Endo */
"04UJ4KZ"  =  "1"  /* Supplement L Ext Iliac Art w Nonaut Sub, Perc Endo */
"04UK07Z"  =  "1"  /* Supplement R Femor Art with Autol Sub, Open Approach */
"04UK0JZ"  =  "1"  /* Supplement R Femor Art with Synth Sub, Open Approach */
"04UK0KZ"  =  "1"  /* Supplement R Femor Art with Nonaut Sub, Open Approach */
"04UK37Z"  =  "1"  /* Supplement R Femor Art with Autol Sub, Perc Approach */
"04UK3JZ"  =  "1"  /* Supplement R Femor Art with Synth Sub, Perc Approach */
"04UK3KZ"  =  "1"  /* Supplement R Femor Art with Nonaut Sub, Perc Approach */
"04UK47Z"  =  "1"  /* Supplement R Femor Art with Autol Sub, Perc Endo Approach */
"04UK4JZ"  =  "1"  /* Supplement R Femor Art with Synth Sub, Perc Endo Approach */
"04UK4KZ"  =  "1"  /* Supplement R Femor Art with Nonaut Sub, Perc Endo Approach */
"04UL07Z"  =  "1"  /* Supplement Left Femoral Artery with Autol Sub, Open Approach */
"04UL0JZ"  =  "1"  /* Supplement Left Femoral Artery with Synth Sub, Open Approach */
"04UL0KZ"  =  "1"  /* Supplement L Femor Art with Nonaut Sub, Open Approach */
"04UL37Z"  =  "1"  /* Supplement Left Femoral Artery with Autol Sub, Perc Approach */
"04UL3JZ"  =  "1"  /* Supplement Left Femoral Artery with Synth Sub, Perc Approach */
"04UL3KZ"  =  "1"  /* Supplement L Femor Art with Nonaut Sub, Perc Approach */
"04UL47Z"  =  "1"  /* Supplement L Femor Art with Autol Sub, Perc Endo Approach */
"04UL4JZ"  =  "1"  /* Supplement L Femor Art with Synth Sub, Perc Endo Approach */
"04UL4KZ"  =  "1"  /* Supplement L Femor Art with Nonaut Sub, Perc Endo Approach */
"04UM07Z"  =  "1"  /* Supplement R Poplit Art with Autol Sub, Open Approach */
"04UM0JZ"  =  "1"  /* Supplement R Poplit Art with Synth Sub, Open Approach */
"04UM0KZ"  =  "1"  /* Supplement R Poplit Art with Nonaut Sub, Open Approach */
"04UM37Z"  =  "1"  /* Supplement R Poplit Art with Autol Sub, Perc Approach */
"04UM3JZ"  =  "1"  /* Supplement R Poplit Art with Synth Sub, Perc Approach */
"04UM3KZ"  =  "1"  /* Supplement R Poplit Art with Nonaut Sub, Perc Approach */
"04UM47Z"  =  "1"  /* Supplement R Poplit Art with Autol Sub, Perc Endo Approach */
"04UM4JZ"  =  "1"  /* Supplement R Poplit Art with Synth Sub, Perc Endo Approach */
"04UM4KZ"  =  "1"  /* Supplement R Poplit Art with Nonaut Sub, Perc Endo Approach */
"04UN07Z"  =  "1"  /* Supplement L Poplit Art with Autol Sub, Open Approach */
"04UN0JZ"  =  "1"  /* Supplement L Poplit Art with Synth Sub, Open Approach */
"04UN0KZ"  =  "1"  /* Supplement L Poplit Art with Nonaut Sub, Open Approach */
"04UN37Z"  =  "1"  /* Supplement L Poplit Art with Autol Sub, Perc Approach */
"04UN3JZ"  =  "1"  /* Supplement L Poplit Art with Synth Sub, Perc Approach */
"04UN3KZ"  =  "1"  /* Supplement L Poplit Art with Nonaut Sub, Perc Approach */
"04UN47Z"  =  "1"  /* Supplement L Poplit Art with Autol Sub, Perc Endo Approach */
"04UN4JZ"  =  "1"  /* Supplement L Poplit Art with Synth Sub, Perc Endo Approach */
"04UN4KZ"  =  "1"  /* Supplement L Poplit Art with Nonaut Sub, Perc Endo Approach */
"04UP07Z"  =  "1"  /* Supplement R Ant Tib Art with Autol Sub, Open Approach */
"04UP0JZ"  =  "1"  /* Supplement R Ant Tib Art with Synth Sub, Open Approach */
"04UP0KZ"  =  "1"  /* Supplement R Ant Tib Art with Nonaut Sub, Open Approach */
"04UP37Z"  =  "1"  /* Supplement R Ant Tib Art with Autol Sub, Perc Approach */
"04UP3JZ"  =  "1"  /* Supplement R Ant Tib Art with Synth Sub, Perc Approach */
"04UP3KZ"  =  "1"  /* Supplement R Ant Tib Art with Nonaut Sub, Perc Approach */
"04UP47Z"  =  "1"  /* Supplement R Ant Tib Art with Autol Sub, Perc Endo Approach */
"04UP4JZ"  =  "1"  /* Supplement R Ant Tib Art with Synth Sub, Perc Endo Approach */
"04UP4KZ"  =  "1"  /* Supplement R Ant Tib Art with Nonaut Sub, Perc Endo Approach */
"04UQ07Z"  =  "1"  /* Supplement L Ant Tib Art with Autol Sub, Open Approach */
"04UQ0JZ"  =  "1"  /* Supplement L Ant Tib Art with Synth Sub, Open Approach */
"04UQ0KZ"  =  "1"  /* Supplement L Ant Tib Art with Nonaut Sub, Open Approach */
"04UQ37Z"  =  "1"  /* Supplement L Ant Tib Art with Autol Sub, Perc Approach */
"04UQ3JZ"  =  "1"  /* Supplement L Ant Tib Art with Synth Sub, Perc Approach */
"04UQ3KZ"  =  "1"  /* Supplement L Ant Tib Art with Nonaut Sub, Perc Approach */
"04UQ47Z"  =  "1"  /* Supplement L Ant Tib Art with Autol Sub, Perc Endo Approach */
"04UQ4JZ"  =  "1"  /* Supplement L Ant Tib Art with Synth Sub, Perc Endo Approach */
"04UQ4KZ"  =  "1"  /* Supplement L Ant Tib Art with Nonaut Sub, Perc Endo Approach */
"04UR07Z"  =  "1"  /* Supplement R Post Tib Art with Autol Sub, Open Approach */
"04UR0JZ"  =  "1"  /* Supplement R Post Tib Art with Synth Sub, Open Approach */
"04UR0KZ"  =  "1"  /* Supplement R Post Tib Art with Nonaut Sub, Open Approach */
"04UR37Z"  =  "1"  /* Supplement R Post Tib Art with Autol Sub, Perc Approach */
"04UR3JZ"  =  "1"  /* Supplement R Post Tib Art with Synth Sub, Perc Approach */
"04UR3KZ"  =  "1"  /* Supplement R Post Tib Art with Nonaut Sub, Perc Approach */
"04UR47Z"  =  "1"  /* Supplement R Post Tib Art with Autol Sub, Perc Endo Approach */
"04UR4JZ"  =  "1"  /* Supplement R Post Tib Art with Synth Sub, Perc Endo Approach */
"04UR4KZ"  =  "1"  /* Supplement R Post Tib Art w Nonaut Sub, Perc Endo */
"04US07Z"  =  "1"  /* Supplement L Post Tib Art with Autol Sub, Open Approach */
"04US0JZ"  =  "1"  /* Supplement L Post Tib Art with Synth Sub, Open Approach */
"04US0KZ"  =  "1"  /* Supplement L Post Tib Art with Nonaut Sub, Open Approach */
"04US37Z"  =  "1"  /* Supplement L Post Tib Art with Autol Sub, Perc Approach */
"04US3JZ"  =  "1"  /* Supplement L Post Tib Art with Synth Sub, Perc Approach */
"04US3KZ"  =  "1"  /* Supplement L Post Tib Art with Nonaut Sub, Perc Approach */
"04US47Z"  =  "1"  /* Supplement L Post Tib Art with Autol Sub, Perc Endo Approach */
"04US4JZ"  =  "1"  /* Supplement L Post Tib Art with Synth Sub, Perc Endo Approach */
"04US4KZ"  =  "1"  /* Supplement L Post Tib Art w Nonaut Sub, Perc Endo */
"04UT07Z"  =  "1"  /* Supplement R Peroneal Art with Autol Sub, Open Approach */
"04UT0JZ"  =  "1"  /* Supplement R Peroneal Art with Synth Sub, Open Approach */
"04UT0KZ"  =  "1"  /* Supplement R Peroneal Art with Nonaut Sub, Open Approach */
"04UT37Z"  =  "1"  /* Supplement R Peroneal Art with Autol Sub, Perc Approach */
"04UT3JZ"  =  "1"  /* Supplement R Peroneal Art with Synth Sub, Perc Approach */
"04UT3KZ"  =  "1"  /* Supplement R Peroneal Art with Nonaut Sub, Perc Approach */
"04UT47Z"  =  "1"  /* Supplement R Peroneal Art with Autol Sub, Perc Endo Approach */
"04UT4JZ"  =  "1"  /* Supplement R Peroneal Art with Synth Sub, Perc Endo Approach */
"04UT4KZ"  =  "1"  /* Supplement R Peroneal Art w Nonaut Sub, Perc Endo */
"04UU07Z"  =  "1"  /* Supplement L Peroneal Art with Autol Sub, Open Approach */
"04UU0JZ"  =  "1"  /* Supplement L Peroneal Art with Synth Sub, Open Approach */
"04UU0KZ"  =  "1"  /* Supplement L Peroneal Art with Nonaut Sub, Open Approach */
"04UU37Z"  =  "1"  /* Supplement L Peroneal Art with Autol Sub, Perc Approach */
"04UU3JZ"  =  "1"  /* Supplement L Peroneal Art with Synth Sub, Perc Approach */
"04UU3KZ"  =  "1"  /* Supplement L Peroneal Art with Nonaut Sub, Perc Approach */
"04UU47Z"  =  "1"  /* Supplement L Peroneal Art with Autol Sub, Perc Endo Approach */
"04UU4JZ"  =  "1"  /* Supplement L Peroneal Art with Synth Sub, Perc Endo Approach */
"04UU4KZ"  =  "1"  /* Supplement L Peroneal Art w Nonaut Sub, Perc Endo */
"04UV07Z"  =  "1"  /* Supplement Right Foot Artery with Autol Sub, Open Approach */
"04UV0JZ"  =  "1"  /* Supplement Right Foot Artery with Synth Sub, Open Approach */
"04UV0KZ"  =  "1"  /* Supplement Right Foot Artery with Nonaut Sub, Open Approach */
"04UV37Z"  =  "1"  /* Supplement Right Foot Artery with Autol Sub, Perc Approach */
"04UV3JZ"  =  "1"  /* Supplement Right Foot Artery with Synth Sub, Perc Approach */
"04UV3KZ"  =  "1"  /* Supplement Right Foot Artery with Nonaut Sub, Perc Approach */
"04UV47Z"  =  "1"  /* Supplement R Foot Art with Autol Sub, Perc Endo Approach */
"04UV4JZ"  =  "1"  /* Supplement R Foot Art with Synth Sub, Perc Endo Approach */
"04UV4KZ"  =  "1"  /* Supplement R Foot Art with Nonaut Sub, Perc Endo Approach */
"04UW07Z"  =  "1"  /* Supplement Left Foot Artery with Autol Sub, Open Approach */
"04UW0JZ"  =  "1"  /* Supplement Left Foot Artery with Synth Sub, Open Approach */
"04UW0KZ"  =  "1"  /* Supplement Left Foot Artery with Nonaut Sub, Open Approach */
"04UW37Z"  =  "1"  /* Supplement Left Foot Artery with Autol Sub, Perc Approach */
"04UW3JZ"  =  "1"  /* Supplement Left Foot Artery with Synth Sub, Perc Approach */
"04UW3KZ"  =  "1"  /* Supplement Left Foot Artery with Nonaut Sub, Perc Approach */
"04UW47Z"  =  "1"  /* Supplement L Foot Art with Autol Sub, Perc Endo Approach */
"04UW4JZ"  =  "1"  /* Supplement L Foot Art with Synth Sub, Perc Endo Approach */
"04UW4KZ"  =  "1"  /* Supplement L Foot Art with Nonaut Sub, Perc Endo Approach */
"04UY07Z"  =  "1"  /* Supplement Lower Artery with Autol Sub, Open Approach */
"04UY0JZ"  =  "1"  /* Supplement Lower Artery with Synth Sub, Open Approach */
"04UY0KZ"  =  "1"  /* Supplement Lower Artery with Nonaut Sub, Open Approach */
"04UY37Z"  =  "1"  /* Supplement Lower Artery with Autol Sub, Perc Approach */
"04UY3JZ"  =  "1"  /* Supplement Lower Artery with Synth Sub, Perc Approach */
"04UY3KZ"  =  "1"  /* Supplement Lower Artery with Nonaut Sub, Perc Approach */
"04UY47Z"  =  "1"  /* Supplement Lower Artery with Autol Sub, Perc Endo Approach */
"04UY4JZ"  =  "1"  /* Supplement Lower Artery with Synth Sub, Perc Endo Approach */
"04UY4KZ"  =  "1"  /* Supplement Lower Artery with Nonaut Sub, Perc Endo Approach */
"04V00CZ"  =  "1"  /* Restriction of Abd Aorta with Extralum Dev, Open Approach */
"04V00DJ"  =  "1"  /* Restrict of Abd Aorta with Intralum Dev, Temp, Open Approach */
"04V00DZ"  =  "1"  /* Restriction of Abd Aorta with Intralum Dev, Open Approach */
"04V00ZZ"  =  "1"  /* Restriction of Abdominal Aorta, Open Approach */
"04V03CZ"  =  "1"  /* Restriction of Abd Aorta with Extralum Dev, Perc Approach */
"04V03DJ"  =  "1"  /* Restrict of Abd Aorta with Intralum Dev, Temp, Perc Approach */
"04V03DZ"  =  "1"  /* Restriction of Abd Aorta with Intralum Dev, Perc Approach */
"04V03ZZ"  =  "1"  /* Restriction of Abdominal Aorta, Percutaneous Approach */
"04V04CZ"  =  "1"  /* Restrict of Abd Aorta with Extralum Dev, Perc Endo Approach */
"04V04DJ"  =  "1"  /* Restrict Abd Aorta w Intralum Dev, Temp, Perc Endo */
"04V04DZ"  =  "1"  /* Restrict of Abd Aorta with Intralum Dev, Perc Endo Approach */
"04V04ZZ"  =  "1"  /* Restriction of Abdominal Aorta, Perc Endo Approach */
"04V10CZ"  =  "1"  /* Restriction of Celiac Art with Extralum Dev, Open Approach */
"04V10DZ"  =  "1"  /* Restriction of Celiac Art with Intralum Dev, Open Approach */
"04V10ZZ"  =  "1"  /* Restriction of Celiac Artery, Open Approach */
"04V13CZ"  =  "1"  /* Restriction of Celiac Art with Extralum Dev, Perc Approach */
"04V13DZ"  =  "1"  /* Restriction of Celiac Art with Intralum Dev, Perc Approach */
"04V13ZZ"  =  "1"  /* Restriction of Celiac Artery, Percutaneous Approach */
"04V14CZ"  =  "1"  /* Restrict of Celiac Art with Extralum Dev, Perc Endo Approach */
"04V14DZ"  =  "1"  /* Restrict of Celiac Art with Intralum Dev, Perc Endo Approach */
"04V14ZZ"  =  "1"  /* Restriction of Celiac Artery, Perc Endo Approach */
"04V20CZ"  =  "1"  /* Restriction of Gastric Art with Extralum Dev, Open Approach */
"04V20DZ"  =  "1"  /* Restriction of Gastric Art with Intralum Dev, Open Approach */
"04V20ZZ"  =  "1"  /* Restriction of Gastric Artery, Open Approach */
"04V23CZ"  =  "1"  /* Restriction of Gastric Art with Extralum Dev, Perc Approach */
"04V23DZ"  =  "1"  /* Restriction of Gastric Art with Intralum Dev, Perc Approach */
"04V23ZZ"  =  "1"  /* Restriction of Gastric Artery, Percutaneous Approach */
"04V24CZ"  =  "1"  /* Restrict Gastric Art w Extralum Dev, Perc Endo */
"04V24DZ"  =  "1"  /* Restrict Gastric Art w Intralum Dev, Perc Endo */
"04V24ZZ"  =  "1"  /* Restriction of Gastric Artery, Perc Endo Approach */
"04V30CZ"  =  "1"  /* Restriction of Hepatic Art with Extralum Dev, Open Approach */
"04V30DZ"  =  "1"  /* Restriction of Hepatic Art with Intralum Dev, Open Approach */
"04V30ZZ"  =  "1"  /* Restriction of Hepatic Artery, Open Approach */
"04V33CZ"  =  "1"  /* Restriction of Hepatic Art with Extralum Dev, Perc Approach */
"04V33DZ"  =  "1"  /* Restriction of Hepatic Art with Intralum Dev, Perc Approach */
"04V33ZZ"  =  "1"  /* Restriction of Hepatic Artery, Percutaneous Approach */
"04V34CZ"  =  "1"  /* Restrict Hepatic Art w Extralum Dev, Perc Endo */
"04V34DZ"  =  "1"  /* Restrict Hepatic Art w Intralum Dev, Perc Endo */
"04V34ZZ"  =  "1"  /* Restriction of Hepatic Artery, Perc Endo Approach */
"04V40CZ"  =  "1"  /* Restriction of Splenic Art with Extralum Dev, Open Approach */
"04V40DZ"  =  "1"  /* Restriction of Splenic Art with Intralum Dev, Open Approach */
"04V40ZZ"  =  "1"  /* Restriction of Splenic Artery, Open Approach */
"04V43CZ"  =  "1"  /* Restriction of Splenic Art with Extralum Dev, Perc Approach */
"04V43DZ"  =  "1"  /* Restriction of Splenic Art with Intralum Dev, Perc Approach */
"04V43ZZ"  =  "1"  /* Restriction of Splenic Artery, Percutaneous Approach */
"04V44CZ"  =  "1"  /* Restrict Splenic Art w Extralum Dev, Perc Endo */
"04V44DZ"  =  "1"  /* Restrict Splenic Art w Intralum Dev, Perc Endo */
"04V44ZZ"  =  "1"  /* Restriction of Splenic Artery, Perc Endo Approach */
"04V50CZ"  =  "1"  /* Restrict of Sup Mesent Art with Extralum Dev, Open Approach */
"04V50DZ"  =  "1"  /* Restrict of Sup Mesent Art with Intralum Dev, Open Approach */
"04V50ZZ"  =  "1"  /* Restriction of Superior Mesenteric Artery, Open Approach */
"04V53CZ"  =  "1"  /* Restrict of Sup Mesent Art with Extralum Dev, Perc Approach */
"04V53DZ"  =  "1"  /* Restrict of Sup Mesent Art with Intralum Dev, Perc Approach */
"04V53ZZ"  =  "1"  /* Restriction of Superior Mesenteric Artery, Perc Approach */
"04V54CZ"  =  "1"  /* Restrict Sup Mesent Art w Extralum Dev, Perc Endo */
"04V54DZ"  =  "1"  /* Restrict Sup Mesent Art w Intralum Dev, Perc Endo */
"04V54ZZ"  =  "1"  /* Restriction of Sup Mesent Art, Perc Endo Approach */
"04V60CZ"  =  "1"  /* Restriction of R Colic Art with Extralum Dev, Open Approach */
"04V60DZ"  =  "1"  /* Restriction of R Colic Art with Intralum Dev, Open Approach */
"04V60ZZ"  =  "1"  /* Restriction of Right Colic Artery, Open Approach */
"04V63CZ"  =  "1"  /* Restriction of R Colic Art with Extralum Dev, Perc Approach */
"04V63DZ"  =  "1"  /* Restriction of R Colic Art with Intralum Dev, Perc Approach */
"04V63ZZ"  =  "1"  /* Restriction of Right Colic Artery, Percutaneous Approach */
"04V64CZ"  =  "1"  /* Restrict R Colic Art w Extralum Dev, Perc Endo */
"04V64DZ"  =  "1"  /* Restrict R Colic Art w Intralum Dev, Perc Endo */
"04V64ZZ"  =  "1"  /* Restriction of Right Colic Artery, Perc Endo Approach */
"04V70CZ"  =  "1"  /* Restriction of L Colic Art with Extralum Dev, Open Approach */
"04V70DZ"  =  "1"  /* Restriction of L Colic Art with Intralum Dev, Open Approach */
"04V70ZZ"  =  "1"  /* Restriction of Left Colic Artery, Open Approach */
"04V73CZ"  =  "1"  /* Restriction of L Colic Art with Extralum Dev, Perc Approach */
"04V73DZ"  =  "1"  /* Restriction of L Colic Art with Intralum Dev, Perc Approach */
"04V73ZZ"  =  "1"  /* Restriction of Left Colic Artery, Percutaneous Approach */
"04V74CZ"  =  "1"  /* Restrict L Colic Art w Extralum Dev, Perc Endo */
"04V74DZ"  =  "1"  /* Restrict L Colic Art w Intralum Dev, Perc Endo */
"04V74ZZ"  =  "1"  /* Restriction of Left Colic Artery, Perc Endo Approach */
"04V80CZ"  =  "1"  /* Restrict of Mid Colic Art with Extralum Dev, Open Approach */
"04V80DZ"  =  "1"  /* Restrict of Mid Colic Art with Intralum Dev, Open Approach */
"04V80ZZ"  =  "1"  /* Restriction of Middle Colic Artery, Open Approach */
"04V83CZ"  =  "1"  /* Restrict of Mid Colic Art with Extralum Dev, Perc Approach */
"04V83DZ"  =  "1"  /* Restrict of Mid Colic Art with Intralum Dev, Perc Approach */
"04V83ZZ"  =  "1"  /* Restriction of Middle Colic Artery, Percutaneous Approach */
"04V84CZ"  =  "1"  /* Restrict Mid Colic Art w Extralum Dev, Perc Endo */
"04V84DZ"  =  "1"  /* Restrict Mid Colic Art w Intralum Dev, Perc Endo */
"04V84ZZ"  =  "1"  /* Restriction of Middle Colic Artery, Perc Endo Approach */
"04V90CZ"  =  "1"  /* Restriction of R Renal Art with Extralum Dev, Open Approach */
"04V90DZ"  =  "1"  /* Restriction of R Renal Art with Intralum Dev, Open Approach */
"04V90ZZ"  =  "1"  /* Restriction of Right Renal Artery, Open Approach */
"04V93CZ"  =  "1"  /* Restriction of R Renal Art with Extralum Dev, Perc Approach */
"04V93DZ"  =  "1"  /* Restriction of R Renal Art with Intralum Dev, Perc Approach */
"04V93ZZ"  =  "1"  /* Restriction of Right Renal Artery, Percutaneous Approach */
"04V94CZ"  =  "1"  /* Restrict R Renal Art w Extralum Dev, Perc Endo */
"04V94DZ"  =  "1"  /* Restrict R Renal Art w Intralum Dev, Perc Endo */
"04V94ZZ"  =  "1"  /* Restriction of Right Renal Artery, Perc Endo Approach */
"04VA0CZ"  =  "1"  /* Restriction of L Renal Art with Extralum Dev, Open Approach */
"04VA0DZ"  =  "1"  /* Restriction of L Renal Art with Intralum Dev, Open Approach */
"04VA0ZZ"  =  "1"  /* Restriction of Left Renal Artery, Open Approach */
"04VA3CZ"  =  "1"  /* Restriction of L Renal Art with Extralum Dev, Perc Approach */
"04VA3DZ"  =  "1"  /* Restriction of L Renal Art with Intralum Dev, Perc Approach */
"04VA3ZZ"  =  "1"  /* Restriction of Left Renal Artery, Percutaneous Approach */
"04VA4CZ"  =  "1"  /* Restrict L Renal Art w Extralum Dev, Perc Endo */
"04VA4DZ"  =  "1"  /* Restrict L Renal Art w Intralum Dev, Perc Endo */
"04VA4ZZ"  =  "1"  /* Restriction of Left Renal Artery, Perc Endo Approach */
"04VB0CZ"  =  "1"  /* Restrict of Inf Mesent Art with Extralum Dev, Open Approach */
"04VB0DZ"  =  "1"  /* Restrict of Inf Mesent Art with Intralum Dev, Open Approach */
"04VB0ZZ"  =  "1"  /* Restriction of Inferior Mesenteric Artery, Open Approach */
"04VB3CZ"  =  "1"  /* Restrict of Inf Mesent Art with Extralum Dev, Perc Approach */
"04VB3DZ"  =  "1"  /* Restrict of Inf Mesent Art with Intralum Dev, Perc Approach */
"04VB3ZZ"  =  "1"  /* Restriction of Inferior Mesenteric Artery, Perc Approach */
"04VB4CZ"  =  "1"  /* Restrict Inf Mesent Art w Extralum Dev, Perc Endo */
"04VB4DZ"  =  "1"  /* Restrict Inf Mesent Art w Intralum Dev, Perc Endo */
"04VB4ZZ"  =  "1"  /* Restriction of Inf Mesent Art, Perc Endo Approach */
"04VC0CZ"  =  "1"  /* Restrict of R Com Iliac Art with Extralum Dev, Open Approach */
"04VC0DZ"  =  "1"  /* Restrict of R Com Iliac Art with Intralum Dev, Open Approach */
"04VC0ZZ"  =  "1"  /* Restriction of Right Common Iliac Artery, Open Approach */
"04VC3CZ"  =  "1"  /* Restrict of R Com Iliac Art with Extralum Dev, Perc Approach */
"04VC3DZ"  =  "1"  /* Restrict of R Com Iliac Art with Intralum Dev, Perc Approach */
"04VC3ZZ"  =  "1"  /* Restriction of Right Common Iliac Artery, Perc Approach */
"04VC4CZ"  =  "1"  /* Restrict R Com Iliac Art w Extralum Dev, Perc Endo */
"04VC4DZ"  =  "1"  /* Restrict R Com Iliac Art w Intralum Dev, Perc Endo */
"04VC4ZZ"  =  "1"  /* Restriction of Right Common Iliac Artery, Perc Endo Approach */
"04VD0CZ"  =  "1"  /* Restrict of L Com Iliac Art with Extralum Dev, Open Approach */
"04VD0DZ"  =  "1"  /* Restrict of L Com Iliac Art with Intralum Dev, Open Approach */
"04VD0ZZ"  =  "1"  /* Restriction of Left Common Iliac Artery, Open Approach */
"04VD3CZ"  =  "1"  /* Restrict of L Com Iliac Art with Extralum Dev, Perc Approach */
"04VD3DZ"  =  "1"  /* Restrict of L Com Iliac Art with Intralum Dev, Perc Approach */
"04VD3ZZ"  =  "1"  /* Restriction of Left Common Iliac Artery, Perc Approach */
"04VD4CZ"  =  "1"  /* Restrict L Com Iliac Art w Extralum Dev, Perc Endo */
"04VD4DZ"  =  "1"  /* Restrict L Com Iliac Art w Intralum Dev, Perc Endo */
"04VD4ZZ"  =  "1"  /* Restriction of Left Common Iliac Artery, Perc Endo Approach */
"04VE0CZ"  =  "1"  /* Restrict of R Int Iliac Art with Extralum Dev, Open Approach */
"04VE0DZ"  =  "1"  /* Restrict of R Int Iliac Art with Intralum Dev, Open Approach */
"04VE0ZZ"  =  "1"  /* Restriction of Right Internal Iliac Artery, Open Approach */
"04VE3CZ"  =  "1"  /* Restrict of R Int Iliac Art with Extralum Dev, Perc Approach */
"04VE3DZ"  =  "1"  /* Restrict of R Int Iliac Art with Intralum Dev, Perc Approach */
"04VE3ZZ"  =  "1"  /* Restriction of Right Internal Iliac Artery, Perc Approach */
"04VE4CZ"  =  "1"  /* Restrict R Int Iliac Art w Extralum Dev, Perc Endo */
"04VE4DZ"  =  "1"  /* Restrict R Int Iliac Art w Intralum Dev, Perc Endo */
"04VE4ZZ"  =  "1"  /* Restriction of R Int Iliac Art, Perc Endo Approach */
"04VF0CZ"  =  "1"  /* Restrict of L Int Iliac Art with Extralum Dev, Open Approach */
"04VF0DZ"  =  "1"  /* Restrict of L Int Iliac Art with Intralum Dev, Open Approach */
"04VF0ZZ"  =  "1"  /* Restriction of Left Internal Iliac Artery, Open Approach */
"04VF3CZ"  =  "1"  /* Restrict of L Int Iliac Art with Extralum Dev, Perc Approach */
"04VF3DZ"  =  "1"  /* Restrict of L Int Iliac Art with Intralum Dev, Perc Approach */
"04VF3ZZ"  =  "1"  /* Restriction of Left Internal Iliac Artery, Perc Approach */
"04VF4CZ"  =  "1"  /* Restrict L Int Iliac Art w Extralum Dev, Perc Endo */
"04VF4DZ"  =  "1"  /* Restrict L Int Iliac Art w Intralum Dev, Perc Endo */
"04VF4ZZ"  =  "1"  /* Restriction of L Int Iliac Art, Perc Endo Approach */
"04VH0CZ"  =  "1"  /* Restrict of R Ext Iliac Art with Extralum Dev, Open Approach */
"04VH0DZ"  =  "1"  /* Restrict of R Ext Iliac Art with Intralum Dev, Open Approach */
"04VH0ZZ"  =  "1"  /* Restriction of Right External Iliac Artery, Open Approach */
"04VH3CZ"  =  "1"  /* Restrict of R Ext Iliac Art with Extralum Dev, Perc Approach */
"04VH3DZ"  =  "1"  /* Restrict of R Ext Iliac Art with Intralum Dev, Perc Approach */
"04VH3ZZ"  =  "1"  /* Restriction of Right External Iliac Artery, Perc Approach */
"04VH4CZ"  =  "1"  /* Restrict R Ext Iliac Art w Extralum Dev, Perc Endo */
"04VH4DZ"  =  "1"  /* Restrict R Ext Iliac Art w Intralum Dev, Perc Endo */
"04VH4ZZ"  =  "1"  /* Restriction of R Ext Iliac Art, Perc Endo Approach */
"04VJ0CZ"  =  "1"  /* Restrict of L Ext Iliac Art with Extralum Dev, Open Approach */
"04VJ0DZ"  =  "1"  /* Restrict of L Ext Iliac Art with Intralum Dev, Open Approach */
"04VJ0ZZ"  =  "1"  /* Restriction of Left External Iliac Artery, Open Approach */
"04VJ3CZ"  =  "1"  /* Restrict of L Ext Iliac Art with Extralum Dev, Perc Approach */
"04VJ3DZ"  =  "1"  /* Restrict of L Ext Iliac Art with Intralum Dev, Perc Approach */
"04VJ3ZZ"  =  "1"  /* Restriction of Left External Iliac Artery, Perc Approach */
"04VJ4CZ"  =  "1"  /* Restrict L Ext Iliac Art w Extralum Dev, Perc Endo */
"04VJ4DZ"  =  "1"  /* Restrict L Ext Iliac Art w Intralum Dev, Perc Endo */
"04VJ4ZZ"  =  "1"  /* Restriction of L Ext Iliac Art, Perc Endo Approach */
"04VK0CZ"  =  "1"  /* Restriction of R Femor Art with Extralum Dev, Open Approach */
"04VK0DZ"  =  "1"  /* Restriction of R Femor Art with Intralum Dev, Open Approach */
"04VK0ZZ"  =  "1"  /* Restriction of Right Femoral Artery, Open Approach */
"04VK3CZ"  =  "1"  /* Restriction of R Femor Art with Extralum Dev, Perc Approach */
"04VK3DZ"  =  "1"  /* Restriction of R Femor Art with Intralum Dev, Perc Approach */
"04VK3ZZ"  =  "1"  /* Restriction of Right Femoral Artery, Percutaneous Approach */
"04VK4CZ"  =  "1"  /* Restrict R Femor Art w Extralum Dev, Perc Endo */
"04VK4DZ"  =  "1"  /* Restrict R Femor Art w Intralum Dev, Perc Endo */
"04VK4ZZ"  =  "1"  /* Restriction of Right Femoral Artery, Perc Endo Approach */
"04VL0CZ"  =  "1"  /* Restriction of L Femor Art with Extralum Dev, Open Approach */
"04VL0DZ"  =  "1"  /* Restriction of L Femor Art with Intralum Dev, Open Approach */
"04VL0ZZ"  =  "1"  /* Restriction of Left Femoral Artery, Open Approach */
"04VL3CZ"  =  "1"  /* Restriction of L Femor Art with Extralum Dev, Perc Approach */
"04VL3DZ"  =  "1"  /* Restriction of L Femor Art with Intralum Dev, Perc Approach */
"04VL3ZZ"  =  "1"  /* Restriction of Left Femoral Artery, Percutaneous Approach */
"04VL4CZ"  =  "1"  /* Restrict L Femor Art w Extralum Dev, Perc Endo */
"04VL4DZ"  =  "1"  /* Restrict L Femor Art w Intralum Dev, Perc Endo */
"04VL4ZZ"  =  "1"  /* Restriction of Left Femoral Artery, Perc Endo Approach */
"04VM0CZ"  =  "1"  /* Restriction of R Poplit Art with Extralum Dev, Open Approach */
"04VM0DZ"  =  "1"  /* Restriction of R Poplit Art with Intralum Dev, Open Approach */
"04VM0ZZ"  =  "1"  /* Restriction of Right Popliteal Artery, Open Approach */
"04VM3CZ"  =  "1"  /* Restriction of R Poplit Art with Extralum Dev, Perc Approach */
"04VM3DZ"  =  "1"  /* Restriction of R Poplit Art with Intralum Dev, Perc Approach */
"04VM3ZZ"  =  "1"  /* Restriction of Right Popliteal Artery, Percutaneous Approach */
"04VM4CZ"  =  "1"  /* Restrict R Poplit Art w Extralum Dev, Perc Endo */
"04VM4DZ"  =  "1"  /* Restrict R Poplit Art w Intralum Dev, Perc Endo */
"04VM4ZZ"  =  "1"  /* Restriction of Right Popliteal Artery, Perc Endo Approach */
"04VN0CZ"  =  "1"  /* Restriction of L Poplit Art with Extralum Dev, Open Approach */
"04VN0DZ"  =  "1"  /* Restriction of L Poplit Art with Intralum Dev, Open Approach */
"04VN0ZZ"  =  "1"  /* Restriction of Left Popliteal Artery, Open Approach */
"04VN3CZ"  =  "1"  /* Restriction of L Poplit Art with Extralum Dev, Perc Approach */
"04VN3DZ"  =  "1"  /* Restriction of L Poplit Art with Intralum Dev, Perc Approach */
"04VN3ZZ"  =  "1"  /* Restriction of Left Popliteal Artery, Percutaneous Approach */
"04VN4CZ"  =  "1"  /* Restrict L Poplit Art w Extralum Dev, Perc Endo */
"04VN4DZ"  =  "1"  /* Restrict L Poplit Art w Intralum Dev, Perc Endo */
"04VN4ZZ"  =  "1"  /* Restriction of Left Popliteal Artery, Perc Endo Approach */
"04VP0CZ"  =  "1"  /* Restrict of R Ant Tib Art with Extralum Dev, Open Approach */
"04VP0DZ"  =  "1"  /* Restrict of R Ant Tib Art with Intralum Dev, Open Approach */
"04VP0ZZ"  =  "1"  /* Restriction of Right Anterior Tibial Artery, Open Approach */
"04VP3CZ"  =  "1"  /* Restrict of R Ant Tib Art with Extralum Dev, Perc Approach */
"04VP3DZ"  =  "1"  /* Restrict of R Ant Tib Art with Intralum Dev, Perc Approach */
"04VP3ZZ"  =  "1"  /* Restriction of Right Anterior Tibial Artery, Perc Approach */
"04VP4CZ"  =  "1"  /* Restrict R Ant Tib Art w Extralum Dev, Perc Endo */
"04VP4DZ"  =  "1"  /* Restrict R Ant Tib Art w Intralum Dev, Perc Endo */
"04VP4ZZ"  =  "1"  /* Restriction of R Ant Tib Art, Perc Endo Approach */
"04VQ0CZ"  =  "1"  /* Restrict of L Ant Tib Art with Extralum Dev, Open Approach */
"04VQ0DZ"  =  "1"  /* Restrict of L Ant Tib Art with Intralum Dev, Open Approach */
"04VQ0ZZ"  =  "1"  /* Restriction of Left Anterior Tibial Artery, Open Approach */
"04VQ3CZ"  =  "1"  /* Restrict of L Ant Tib Art with Extralum Dev, Perc Approach */
"04VQ3DZ"  =  "1"  /* Restrict of L Ant Tib Art with Intralum Dev, Perc Approach */
"04VQ3ZZ"  =  "1"  /* Restriction of Left Anterior Tibial Artery, Perc Approach */
"04VQ4CZ"  =  "1"  /* Restrict L Ant Tib Art w Extralum Dev, Perc Endo */
"04VQ4DZ"  =  "1"  /* Restrict L Ant Tib Art w Intralum Dev, Perc Endo */
"04VQ4ZZ"  =  "1"  /* Restriction of L Ant Tib Art, Perc Endo Approach */
"04VR0CZ"  =  "1"  /* Restrict of R Post Tib Art with Extralum Dev, Open Approach */
"04VR0DZ"  =  "1"  /* Restrict of R Post Tib Art with Intralum Dev, Open Approach */
"04VR0ZZ"  =  "1"  /* Restriction of Right Posterior Tibial Artery, Open Approach */
"04VR3CZ"  =  "1"  /* Restrict of R Post Tib Art with Extralum Dev, Perc Approach */
"04VR3DZ"  =  "1"  /* Restrict of R Post Tib Art with Intralum Dev, Perc Approach */
"04VR3ZZ"  =  "1"  /* Restriction of Right Posterior Tibial Artery, Perc Approach */
"04VR4CZ"  =  "1"  /* Restrict R Post Tib Art w Extralum Dev, Perc Endo */
"04VR4DZ"  =  "1"  /* Restrict R Post Tib Art w Intralum Dev, Perc Endo */
"04VR4ZZ"  =  "1"  /* Restriction of R Post Tib Art, Perc Endo Approach */
"04VS0CZ"  =  "1"  /* Restrict of L Post Tib Art with Extralum Dev, Open Approach */
"04VS0DZ"  =  "1"  /* Restrict of L Post Tib Art with Intralum Dev, Open Approach */
"04VS0ZZ"  =  "1"  /* Restriction of Left Posterior Tibial Artery, Open Approach */
"04VS3CZ"  =  "1"  /* Restrict of L Post Tib Art with Extralum Dev, Perc Approach */
"04VS3DZ"  =  "1"  /* Restrict of L Post Tib Art with Intralum Dev, Perc Approach */
"04VS3ZZ"  =  "1"  /* Restriction of Left Posterior Tibial Artery, Perc Approach */
"04VS4CZ"  =  "1"  /* Restrict L Post Tib Art w Extralum Dev, Perc Endo */
"04VS4DZ"  =  "1"  /* Restrict L Post Tib Art w Intralum Dev, Perc Endo */
"04VS4ZZ"  =  "1"  /* Restriction of L Post Tib Art, Perc Endo Approach */
"04VT0CZ"  =  "1"  /* Restrict of R Peroneal Art with Extralum Dev, Open Approach */
"04VT0DZ"  =  "1"  /* Restrict of R Peroneal Art with Intralum Dev, Open Approach */
"04VT0ZZ"  =  "1"  /* Restriction of Right Peroneal Artery, Open Approach */
"04VT3CZ"  =  "1"  /* Restrict of R Peroneal Art with Extralum Dev, Perc Approach */
"04VT3DZ"  =  "1"  /* Restrict of R Peroneal Art with Intralum Dev, Perc Approach */
"04VT3ZZ"  =  "1"  /* Restriction of Right Peroneal Artery, Percutaneous Approach */
"04VT4CZ"  =  "1"  /* Restrict R Peroneal Art w Extralum Dev, Perc Endo */
"04VT4DZ"  =  "1"  /* Restrict R Peroneal Art w Intralum Dev, Perc Endo */
"04VT4ZZ"  =  "1"  /* Restriction of Right Peroneal Artery, Perc Endo Approach */
"04VU0CZ"  =  "1"  /* Restrict of L Peroneal Art with Extralum Dev, Open Approach */
"04VU0DZ"  =  "1"  /* Restrict of L Peroneal Art with Intralum Dev, Open Approach */
"04VU0ZZ"  =  "1"  /* Restriction of Left Peroneal Artery, Open Approach */
"04VU3CZ"  =  "1"  /* Restrict of L Peroneal Art with Extralum Dev, Perc Approach */
"04VU3DZ"  =  "1"  /* Restrict of L Peroneal Art with Intralum Dev, Perc Approach */
"04VU3ZZ"  =  "1"  /* Restriction of Left Peroneal Artery, Percutaneous Approach */
"04VU4CZ"  =  "1"  /* Restrict L Peroneal Art w Extralum Dev, Perc Endo */
"04VU4DZ"  =  "1"  /* Restrict L Peroneal Art w Intralum Dev, Perc Endo */
"04VU4ZZ"  =  "1"  /* Restriction of Left Peroneal Artery, Perc Endo Approach */
"04VV0CZ"  =  "1"  /* Restriction of R Foot Art with Extralum Dev, Open Approach */
"04VV0DZ"  =  "1"  /* Restriction of R Foot Art with Intralum Dev, Open Approach */
"04VV0ZZ"  =  "1"  /* Restriction of Right Foot Artery, Open Approach */
"04VV3CZ"  =  "1"  /* Restriction of R Foot Art with Extralum Dev, Perc Approach */
"04VV3DZ"  =  "1"  /* Restriction of R Foot Art with Intralum Dev, Perc Approach */
"04VV3ZZ"  =  "1"  /* Restriction of Right Foot Artery, Percutaneous Approach */
"04VV4CZ"  =  "1"  /* Restrict of R Foot Art with Extralum Dev, Perc Endo Approach */
"04VV4DZ"  =  "1"  /* Restrict of R Foot Art with Intralum Dev, Perc Endo Approach */
"04VV4ZZ"  =  "1"  /* Restriction of Right Foot Artery, Perc Endo Approach */
"04VW0CZ"  =  "1"  /* Restriction of L Foot Art with Extralum Dev, Open Approach */
"04VW0DZ"  =  "1"  /* Restriction of L Foot Art with Intralum Dev, Open Approach */
"04VW0ZZ"  =  "1"  /* Restriction of Left Foot Artery, Open Approach */
"04VW3CZ"  =  "1"  /* Restriction of L Foot Art with Extralum Dev, Perc Approach */
"04VW3DZ"  =  "1"  /* Restriction of L Foot Art with Intralum Dev, Perc Approach */
"04VW3ZZ"  =  "1"  /* Restriction of Left Foot Artery, Percutaneous Approach */
"04VW4CZ"  =  "1"  /* Restrict of L Foot Art with Extralum Dev, Perc Endo Approach */
"04VW4DZ"  =  "1"  /* Restrict of L Foot Art with Intralum Dev, Perc Endo Approach */
"04VW4ZZ"  =  "1"  /* Restriction of Left Foot Artery, Perc Endo Approach */
"04VY0CZ"  =  "1"  /* Restriction of Lower Artery with Extralum Dev, Open Approach */
"04VY0DZ"  =  "1"  /* Restriction of Lower Artery with Intralum Dev, Open Approach */
"04VY0ZZ"  =  "1"  /* Restriction of Lower Artery, Open Approach */
"04VY3CZ"  =  "1"  /* Restriction of Lower Artery with Extralum Dev, Perc Approach */
"04VY3DZ"  =  "1"  /* Restriction of Lower Artery with Intralum Dev, Perc Approach */
"04VY3ZZ"  =  "1"  /* Restriction of Lower Artery, Percutaneous Approach */
"04VY4CZ"  =  "1"  /* Restriction of Low Art with Extralum Dev, Perc Endo Approach */
"04VY4DZ"  =  "1"  /* Restriction of Low Art with Intralum Dev, Perc Endo Approach */
"04VY4ZZ"  =  "1"  /* Restriction of Lower Artery, Perc Endo Approach */
"04WY00Z"  =  "1"  /* Revision of Drainage Device in Lower Artery, Open Approach */
"04WY02Z"  =  "1"  /* Revision of Monitoring Device in Lower Artery, Open Approach */
"04WY03Z"  =  "1"  /* Revision of Infusion Device in Lower Artery, Open Approach */
"04WY07Z"  =  "1"  /* Revision of Autol Sub in Low Art, Open Approach */
"04WY0CZ"  =  "1"  /* Revision of Extraluminal Device in Low Art, Open Approach */
"04WY0DZ"  =  "1"  /* Revision of Intraluminal Device in Low Art, Open Approach */
"04WY0JZ"  =  "1"  /* Revision of Synthetic Substitute in Low Art, Open Approach */
"04WY0KZ"  =  "1"  /* Revision of Nonaut Sub in Low Art, Open Approach */
"04WY30Z"  =  "1"  /* Revision of Drainage Device in Lower Artery, Perc Approach */
"04WY32Z"  =  "1"  /* Revision of Monitoring Device in Lower Artery, Perc Approach */
"04WY33Z"  =  "1"  /* Revision of Infusion Device in Lower Artery, Perc Approach */
"04WY37Z"  =  "1"  /* Revision of Autol Sub in Low Art, Perc Approach */
"04WY3CZ"  =  "1"  /* Revision of Extraluminal Device in Low Art, Perc Approach */
"04WY3DZ"  =  "1"  /* Revision of Intraluminal Device in Low Art, Perc Approach */
"04WY3JZ"  =  "1"  /* Revision of Synthetic Substitute in Low Art, Perc Approach */
"04WY3KZ"  =  "1"  /* Revision of Nonaut Sub in Low Art, Perc Approach */
"04WY40Z"  =  "1"  /* Revision of Drainage Device in Low Art, Perc Endo Approach */
"04WY42Z"  =  "1"  /* Revision of Monitoring Device in Low Art, Perc Endo Approach */
"04WY43Z"  =  "1"  /* Revision of Infusion Device in Low Art, Perc Endo Approach */
"04WY47Z"  =  "1"  /* Revision of Autol Sub in Low Art, Perc Endo Approach */
"04WY4CZ"  =  "1"  /* Revision of Extralum Dev in Low Art, Perc Endo Approach */
"04WY4DZ"  =  "1"  /* Revision of Intralum Dev in Low Art, Perc Endo Approach */
"04WY4JZ"  =  "1"  /* Revision of Synth Sub in Low Art, Perc Endo Approach */
"04WY4KZ"  =  "1"  /* Revision of Nonaut Sub in Low Art, Perc Endo Approach */
"051007Y"  =  "1"  /* Bypass Azygos Vein to Up Vein with Autol Sub, Open Approach */
"051009Y"  =  "1"  /* Bypass Azygos Vein to Up Vein with Autol Vn, Open Approach */
"05100AY"  =  "1"  /* Bypass Azygos Vein to Up Vein with Autol Art, Open Approach */
"05100JY"  =  "1"  /* Bypass Azygos Vein to Up Vein with Synth Sub, Open Approach */
"05100KY"  =  "1"  /* Bypass Azygos Vein to Up Vein with Nonaut Sub, Open Approach */
"05100ZY"  =  "1"  /* Bypass Azygos Vein to Upper Vein, Open Approach */
"051047Y"  =  "1"  /* Bypass Azygos Vein to Up Vein w Autol Sub, Perc Endo */
"051049Y"  =  "1"  /* Bypass Azygos Vein to Up Vein w Autol Vn, Perc Endo */
"05104AY"  =  "1"  /* Bypass Azygos Vein to Up Vein w Autol Art, Perc Endo */
"05104JY"  =  "1"  /* Bypass Azygos Vein to Up Vein w Synth Sub, Perc Endo */
"05104KY"  =  "1"  /* Bypass Azygos Vein to Up Vein w Nonaut Sub, Perc Endo */
"05104ZY"  =  "1"  /* Bypass Azygos Vein to Upper Vein, Perc Endo Approach */
"051107Y"  =  "1"  /* Bypass Hemiazygos Vein to Up Vein w Autol Sub, Open */
"051109Y"  =  "1"  /* Bypass Hemiazygos Vein to Up Vein w Autol Vn, Open */
"05110AY"  =  "1"  /* Bypass Hemiazygos Vein to Up Vein w Autol Art, Open */
"05110JY"  =  "1"  /* Bypass Hemiazygos Vein to Up Vein w Synth Sub, Open */
"05110KY"  =  "1"  /* Bypass Hemiazygos Vein to Up Vein w Nonaut Sub, Open */
"05110ZY"  =  "1"  /* Bypass Hemiazygos Vein to Upper Vein, Open Approach */
"051147Y"  =  "1"  /* Bypass Hemiazygos Vein to Up Vein w Autol Sub, Perc Endo */
"051149Y"  =  "1"  /* Bypass Hemiazygos Vein to Up Vein w Autol Vn, Perc Endo */
"05114AY"  =  "1"  /* Bypass Hemiazygos Vein to Up Vein w Autol Art, Perc Endo */
"05114JY"  =  "1"  /* Bypass Hemiazygos Vein to Up Vein w Synth Sub, Perc Endo */
"05114KY"  =  "1"  /* Bypass Hemiazygos Vein to Up Vein w Nonaut Sub, Perc Endo */
"05114ZY"  =  "1"  /* Bypass Hemiazygos Vein to Upper Vein, Perc Endo Approach */
"051307Y"  =  "1"  /* Bypass R Innom Vein to Up Vein with Autol Sub, Open Approach */
"051309Y"  =  "1"  /* Bypass R Innom Vein to Up Vein with Autol Vn, Open Approach */
"05130AY"  =  "1"  /* Bypass R Innom Vein to Up Vein with Autol Art, Open Approach */
"05130JY"  =  "1"  /* Bypass R Innom Vein to Up Vein with Synth Sub, Open Approach */
"05130KY"  =  "1"  /* Bypass R Innom Vein to Up Vein w Nonaut Sub, Open */
"05130ZY"  =  "1"  /* Bypass Right Innominate Vein to Upper Vein, Open Approach */
"051347Y"  =  "1"  /* Bypass R Innom Vein to Up Vein w Autol Sub, Perc Endo */
"051349Y"  =  "1"  /* Bypass R Innom Vein to Up Vein w Autol Vn, Perc Endo */
"05134AY"  =  "1"  /* Bypass R Innom Vein to Up Vein w Autol Art, Perc Endo */
"05134JY"  =  "1"  /* Bypass R Innom Vein to Up Vein w Synth Sub, Perc Endo */
"05134KY"  =  "1"  /* Bypass R Innom Vein to Up Vein w Nonaut Sub, Perc Endo */
"05134ZY"  =  "1"  /* Bypass Right Innominate Vein to Up Vein, Perc Endo Approach */
"051407Y"  =  "1"  /* Bypass L Innom Vein to Up Vein with Autol Sub, Open Approach */
"051409Y"  =  "1"  /* Bypass L Innom Vein to Up Vein with Autol Vn, Open Approach */
"05140AY"  =  "1"  /* Bypass L Innom Vein to Up Vein with Autol Art, Open Approach */
"05140JY"  =  "1"  /* Bypass L Innom Vein to Up Vein with Synth Sub, Open Approach */
"05140KY"  =  "1"  /* Bypass L Innom Vein to Up Vein w Nonaut Sub, Open */
"05140ZY"  =  "1"  /* Bypass Left Innominate Vein to Upper Vein, Open Approach */
"051447Y"  =  "1"  /* Bypass L Innom Vein to Up Vein w Autol Sub, Perc Endo */
"051449Y"  =  "1"  /* Bypass L Innom Vein to Up Vein w Autol Vn, Perc Endo */
"05144AY"  =  "1"  /* Bypass L Innom Vein to Up Vein w Autol Art, Perc Endo */
"05144JY"  =  "1"  /* Bypass L Innom Vein to Up Vein w Synth Sub, Perc Endo */
"05144KY"  =  "1"  /* Bypass L Innom Vein to Up Vein w Nonaut Sub, Perc Endo */
"05144ZY"  =  "1"  /* Bypass Left Innominate Vein to Up Vein, Perc Endo Approach */
"051507Y"  =  "1"  /* Bypass R Subclav Vein to Up Vein w Autol Sub, Open */
"051509Y"  =  "1"  /* Bypass R Subclav Vein to Up Vein w Autol Vn, Open */
"05150AY"  =  "1"  /* Bypass R Subclav Vein to Up Vein w Autol Art, Open */
"05150JY"  =  "1"  /* Bypass R Subclav Vein to Up Vein w Synth Sub, Open */
"05150KY"  =  "1"  /* Bypass R Subclav Vein to Up Vein w Nonaut Sub, Open */
"05150ZY"  =  "1"  /* Bypass Right Subclavian Vein to Upper Vein, Open Approach */
"051547Y"  =  "1"  /* Bypass R Subclav Vein to Up Vein w Autol Sub, Perc Endo */
"051549Y"  =  "1"  /* Bypass R Subclav Vein to Up Vein w Autol Vn, Perc Endo */
"05154AY"  =  "1"  /* Bypass R Subclav Vein to Up Vein w Autol Art, Perc Endo */
"05154JY"  =  "1"  /* Bypass R Subclav Vein to Up Vein w Synth Sub, Perc Endo */
"05154KY"  =  "1"  /* Bypass R Subclav Vein to Up Vein w Nonaut Sub, Perc Endo */
"05154ZY"  =  "1"  /* Bypass Right Subclavian Vein to Up Vein, Perc Endo Approach */
"051607Y"  =  "1"  /* Bypass L Subclav Vein to Up Vein w Autol Sub, Open */
"051609Y"  =  "1"  /* Bypass L Subclav Vein to Up Vein w Autol Vn, Open */
"05160AY"  =  "1"  /* Bypass L Subclav Vein to Up Vein w Autol Art, Open */
"05160JY"  =  "1"  /* Bypass L Subclav Vein to Up Vein w Synth Sub, Open */
"05160KY"  =  "1"  /* Bypass L Subclav Vein to Up Vein w Nonaut Sub, Open */
"05160ZY"  =  "1"  /* Bypass Left Subclavian Vein to Upper Vein, Open Approach */
"051647Y"  =  "1"  /* Bypass L Subclav Vein to Up Vein w Autol Sub, Perc Endo */
"051649Y"  =  "1"  /* Bypass L Subclav Vein to Up Vein w Autol Vn, Perc Endo */
"05164AY"  =  "1"  /* Bypass L Subclav Vein to Up Vein w Autol Art, Perc Endo */
"05164JY"  =  "1"  /* Bypass L Subclav Vein to Up Vein w Synth Sub, Perc Endo */
"05164KY"  =  "1"  /* Bypass L Subclav Vein to Up Vein w Nonaut Sub, Perc Endo */
"05164ZY"  =  "1"  /* Bypass Left Subclavian Vein to Up Vein, Perc Endo Approach */
"051707Y"  =  "1"  /* Bypass R Axilla Vein to Up Vein w Autol Sub, Open */
"051709Y"  =  "1"  /* Bypass R Axilla Vein to Up Vein with Autol Vn, Open Approach */
"05170AY"  =  "1"  /* Bypass R Axilla Vein to Up Vein w Autol Art, Open */
"05170JY"  =  "1"  /* Bypass R Axilla Vein to Up Vein w Synth Sub, Open */
"05170KY"  =  "1"  /* Bypass R Axilla Vein to Up Vein w Nonaut Sub, Open */
"05170ZY"  =  "1"  /* Bypass Right Axillary Vein to Upper Vein, Open Approach */
"051747Y"  =  "1"  /* Bypass R Axilla Vein to Up Vein w Autol Sub, Perc Endo */
"051749Y"  =  "1"  /* Bypass R Axilla Vein to Up Vein w Autol Vn, Perc Endo */
"05174AY"  =  "1"  /* Bypass R Axilla Vein to Up Vein w Autol Art, Perc Endo */
"05174JY"  =  "1"  /* Bypass R Axilla Vein to Up Vein w Synth Sub, Perc Endo */
"05174KY"  =  "1"  /* Bypass R Axilla Vein to Up Vein w Nonaut Sub, Perc Endo */
"05174ZY"  =  "1"  /* Bypass Right Axillary Vein to Upper Vein, Perc Endo Approach */
"051807Y"  =  "1"  /* Bypass L Axilla Vein to Up Vein w Autol Sub, Open */
"051809Y"  =  "1"  /* Bypass L Axilla Vein to Up Vein with Autol Vn, Open Approach */
"05180AY"  =  "1"  /* Bypass L Axilla Vein to Up Vein w Autol Art, Open */
"05180JY"  =  "1"  /* Bypass L Axilla Vein to Up Vein w Synth Sub, Open */
"05180KY"  =  "1"  /* Bypass L Axilla Vein to Up Vein w Nonaut Sub, Open */
"05180ZY"  =  "1"  /* Bypass Left Axillary Vein to Upper Vein, Open Approach */
"051847Y"  =  "1"  /* Bypass L Axilla Vein to Up Vein w Autol Sub, Perc Endo */
"051849Y"  =  "1"  /* Bypass L Axilla Vein to Up Vein w Autol Vn, Perc Endo */
"05184AY"  =  "1"  /* Bypass L Axilla Vein to Up Vein w Autol Art, Perc Endo */
"05184JY"  =  "1"  /* Bypass L Axilla Vein to Up Vein w Synth Sub, Perc Endo */
"05184KY"  =  "1"  /* Bypass L Axilla Vein to Up Vein w Nonaut Sub, Perc Endo */
"05184ZY"  =  "1"  /* Bypass Left Axillary Vein to Upper Vein, Perc Endo Approach */
"051907Y"  =  "1"  /* Bypass R Brach Vein to Up Vein with Autol Sub, Open Approach */
"051909Y"  =  "1"  /* Bypass R Brach Vein to Up Vein with Autol Vn, Open Approach */
"05190AY"  =  "1"  /* Bypass R Brach Vein to Up Vein with Autol Art, Open Approach */
"05190JY"  =  "1"  /* Bypass R Brach Vein to Up Vein with Synth Sub, Open Approach */
"05190KY"  =  "1"  /* Bypass R Brach Vein to Up Vein w Nonaut Sub, Open */
"05190ZY"  =  "1"  /* Bypass Right Brachial Vein to Upper Vein, Open Approach */
"051947Y"  =  "1"  /* Bypass R Brach Vein to Up Vein w Autol Sub, Perc Endo */
"051949Y"  =  "1"  /* Bypass R Brach Vein to Up Vein w Autol Vn, Perc Endo */
"05194AY"  =  "1"  /* Bypass R Brach Vein to Up Vein w Autol Art, Perc Endo */
"05194JY"  =  "1"  /* Bypass R Brach Vein to Up Vein w Synth Sub, Perc Endo */
"05194KY"  =  "1"  /* Bypass R Brach Vein to Up Vein w Nonaut Sub, Perc Endo */
"05194ZY"  =  "1"  /* Bypass Right Brachial Vein to Upper Vein, Perc Endo Approach */
"051A07Y"  =  "1"  /* Bypass L Brach Vein to Up Vein with Autol Sub, Open Approach */
"051A09Y"  =  "1"  /* Bypass L Brach Vein to Up Vein with Autol Vn, Open Approach */
"051A0AY"  =  "1"  /* Bypass L Brach Vein to Up Vein with Autol Art, Open Approach */
"051A0JY"  =  "1"  /* Bypass L Brach Vein to Up Vein with Synth Sub, Open Approach */
"051A0KY"  =  "1"  /* Bypass L Brach Vein to Up Vein w Nonaut Sub, Open */
"051A0ZY"  =  "1"  /* Bypass Left Brachial Vein to Upper Vein, Open Approach */
"051A47Y"  =  "1"  /* Bypass L Brach Vein to Up Vein w Autol Sub, Perc Endo */
"051A49Y"  =  "1"  /* Bypass L Brach Vein to Up Vein w Autol Vn, Perc Endo */
"051A4AY"  =  "1"  /* Bypass L Brach Vein to Up Vein w Autol Art, Perc Endo */
"051A4JY"  =  "1"  /* Bypass L Brach Vein to Up Vein w Synth Sub, Perc Endo */
"051A4KY"  =  "1"  /* Bypass L Brach Vein to Up Vein w Nonaut Sub, Perc Endo */
"051A4ZY"  =  "1"  /* Bypass Left Brachial Vein to Upper Vein, Perc Endo Approach */
"051B07Y"  =  "1"  /* Bypass R Basilic Vein to Up Vein w Autol Sub, Open */
"051B09Y"  =  "1"  /* Bypass R Basilic Vein to Up Vein w Autol Vn, Open */
"051B0AY"  =  "1"  /* Bypass R Basilic Vein to Up Vein w Autol Art, Open */
"051B0JY"  =  "1"  /* Bypass R Basilic Vein to Up Vein w Synth Sub, Open */
"051B0KY"  =  "1"  /* Bypass R Basilic Vein to Up Vein w Nonaut Sub, Open */
"051B0ZY"  =  "1"  /* Bypass Right Basilic Vein to Upper Vein, Open Approach */
"051B47Y"  =  "1"  /* Bypass R Basilic Vein to Up Vein w Autol Sub, Perc Endo */
"051B49Y"  =  "1"  /* Bypass R Basilic Vein to Up Vein w Autol Vn, Perc Endo */
"051B4AY"  =  "1"  /* Bypass R Basilic Vein to Up Vein w Autol Art, Perc Endo */
"051B4JY"  =  "1"  /* Bypass R Basilic Vein to Up Vein w Synth Sub, Perc Endo */
"051B4KY"  =  "1"  /* Bypass R Basilic Vein to Up Vein w Nonaut Sub, Perc Endo */
"051B4ZY"  =  "1"  /* Bypass Right Basilic Vein to Upper Vein, Perc Endo Approach */
"051C07Y"  =  "1"  /* Bypass L Basilic Vein to Up Vein w Autol Sub, Open */
"051C09Y"  =  "1"  /* Bypass L Basilic Vein to Up Vein w Autol Vn, Open */
"051C0AY"  =  "1"  /* Bypass L Basilic Vein to Up Vein w Autol Art, Open */
"051C0JY"  =  "1"  /* Bypass L Basilic Vein to Up Vein w Synth Sub, Open */
"051C0KY"  =  "1"  /* Bypass L Basilic Vein to Up Vein w Nonaut Sub, Open */
"051C0ZY"  =  "1"  /* Bypass Left Basilic Vein to Upper Vein, Open Approach */
"051C47Y"  =  "1"  /* Bypass L Basilic Vein to Up Vein w Autol Sub, Perc Endo */
"051C49Y"  =  "1"  /* Bypass L Basilic Vein to Up Vein w Autol Vn, Perc Endo */
"051C4AY"  =  "1"  /* Bypass L Basilic Vein to Up Vein w Autol Art, Perc Endo */
"051C4JY"  =  "1"  /* Bypass L Basilic Vein to Up Vein w Synth Sub, Perc Endo */
"051C4KY"  =  "1"  /* Bypass L Basilic Vein to Up Vein w Nonaut Sub, Perc Endo */
"051C4ZY"  =  "1"  /* Bypass Left Basilic Vein to Upper Vein, Perc Endo Approach */
"051D07Y"  =  "1"  /* Bypass R Cephalic Vein to Up Vein w Autol Sub, Open */
"051D09Y"  =  "1"  /* Bypass R Cephalic Vein to Up Vein w Autol Vn, Open */
"051D0AY"  =  "1"  /* Bypass R Cephalic Vein to Up Vein w Autol Art, Open */
"051D0JY"  =  "1"  /* Bypass R Cephalic Vein to Up Vein w Synth Sub, Open */
"051D0KY"  =  "1"  /* Bypass R Cephalic Vein to Up Vein w Nonaut Sub, Open */
"051D0ZY"  =  "1"  /* Bypass Right Cephalic Vein to Upper Vein, Open Approach */
"051D47Y"  =  "1"  /* Bypass R Cephalic Vein to Up Vein w Autol Sub, Perc Endo */
"051D49Y"  =  "1"  /* Bypass R Cephalic Vein to Up Vein w Autol Vn, Perc Endo */
"051D4AY"  =  "1"  /* Bypass R Cephalic Vein to Up Vein w Autol Art, Perc Endo */
"051D4JY"  =  "1"  /* Bypass R Cephalic Vein to Up Vein w Synth Sub, Perc Endo */
"051D4KY"  =  "1"  /* Bypass R Cephalic Vein to Up Vein w Nonaut Sub, Perc Endo */
"051D4ZY"  =  "1"  /* Bypass Right Cephalic Vein to Upper Vein, Perc Endo Approach */
"051F07Y"  =  "1"  /* Bypass L Cephalic Vein to Up Vein w Autol Sub, Open */
"051F09Y"  =  "1"  /* Bypass L Cephalic Vein to Up Vein w Autol Vn, Open */
"051F0AY"  =  "1"  /* Bypass L Cephalic Vein to Up Vein w Autol Art, Open */
"051F0JY"  =  "1"  /* Bypass L Cephalic Vein to Up Vein w Synth Sub, Open */
"051F0KY"  =  "1"  /* Bypass L Cephalic Vein to Up Vein w Nonaut Sub, Open */
"051F0ZY"  =  "1"  /* Bypass Left Cephalic Vein to Upper Vein, Open Approach */
"051F47Y"  =  "1"  /* Bypass L Cephalic Vein to Up Vein w Autol Sub, Perc Endo */
"051F49Y"  =  "1"  /* Bypass L Cephalic Vein to Up Vein w Autol Vn, Perc Endo */
"051F4AY"  =  "1"  /* Bypass L Cephalic Vein to Up Vein w Autol Art, Perc Endo */
"051F4JY"  =  "1"  /* Bypass L Cephalic Vein to Up Vein w Synth Sub, Perc Endo */
"051F4KY"  =  "1"  /* Bypass L Cephalic Vein to Up Vein w Nonaut Sub, Perc Endo */
"051F4ZY"  =  "1"  /* Bypass Left Cephalic Vein to Upper Vein, Perc Endo Approach */
"051G07Y"  =  "1"  /* Bypass R Hand Vein to Up Vein with Autol Sub, Open Approach */
"051G09Y"  =  "1"  /* Bypass R Hand Vein to Up Vein with Autol Vn, Open Approach */
"051G0AY"  =  "1"  /* Bypass R Hand Vein to Up Vein with Autol Art, Open Approach */
"051G0JY"  =  "1"  /* Bypass R Hand Vein to Up Vein with Synth Sub, Open Approach */
"051G0KY"  =  "1"  /* Bypass R Hand Vein to Up Vein with Nonaut Sub, Open Approach */
"051G0ZY"  =  "1"  /* Bypass Right Hand Vein to Upper Vein, Open Approach */
"051G47Y"  =  "1"  /* Bypass R Hand Vein to Up Vein w Autol Sub, Perc Endo */
"051G49Y"  =  "1"  /* Bypass R Hand Vein to Up Vein w Autol Vn, Perc Endo */
"051G4AY"  =  "1"  /* Bypass R Hand Vein to Up Vein w Autol Art, Perc Endo */
"051G4JY"  =  "1"  /* Bypass R Hand Vein to Up Vein w Synth Sub, Perc Endo */
"051G4KY"  =  "1"  /* Bypass R Hand Vein to Up Vein w Nonaut Sub, Perc Endo */
"051G4ZY"  =  "1"  /* Bypass Right Hand Vein to Upper Vein, Perc Endo Approach */
"051H07Y"  =  "1"  /* Bypass L Hand Vein to Up Vein with Autol Sub, Open Approach */
"051H09Y"  =  "1"  /* Bypass L Hand Vein to Up Vein with Autol Vn, Open Approach */
"051H0AY"  =  "1"  /* Bypass L Hand Vein to Up Vein with Autol Art, Open Approach */
"051H0JY"  =  "1"  /* Bypass L Hand Vein to Up Vein with Synth Sub, Open Approach */
"051H0KY"  =  "1"  /* Bypass L Hand Vein to Up Vein with Nonaut Sub, Open Approach */
"051H0ZY"  =  "1"  /* Bypass Left Hand Vein to Upper Vein, Open Approach */
"051H47Y"  =  "1"  /* Bypass L Hand Vein to Up Vein w Autol Sub, Perc Endo */
"051H49Y"  =  "1"  /* Bypass L Hand Vein to Up Vein w Autol Vn, Perc Endo */
"051H4AY"  =  "1"  /* Bypass L Hand Vein to Up Vein w Autol Art, Perc Endo */
"051H4JY"  =  "1"  /* Bypass L Hand Vein to Up Vein w Synth Sub, Perc Endo */
"051H4KY"  =  "1"  /* Bypass L Hand Vein to Up Vein w Nonaut Sub, Perc Endo */
"051H4ZY"  =  "1"  /* Bypass Left Hand Vein to Upper Vein, Perc Endo Approach */
"051L07Y"  =  "1"  /* Bypass Intracran Vein to Up Vein w Autol Sub, Open */
"051L09Y"  =  "1"  /* Bypass Intracran Vein to Up Vein w Autol Vn, Open */
"051L0AY"  =  "1"  /* Bypass Intracran Vein to Up Vein w Autol Art, Open */
"051L0JY"  =  "1"  /* Bypass Intracran Vein to Up Vein w Synth Sub, Open */
"051L0KY"  =  "1"  /* Bypass Intracran Vein to Up Vein w Nonaut Sub, Open */
"051L0ZY"  =  "1"  /* Bypass Intracranial Vein to Upper Vein, Open Approach */
"051L47Y"  =  "1"  /* Bypass Intracran Vein to Up Vein w Autol Sub, Perc Endo */
"051L49Y"  =  "1"  /* Bypass Intracran Vein to Up Vein w Autol Vn, Perc Endo */
"051L4AY"  =  "1"  /* Bypass Intracran Vein to Up Vein w Autol Art, Perc Endo */
"051L4JY"  =  "1"  /* Bypass Intracran Vein to Up Vein w Synth Sub, Perc Endo */
"051L4KY"  =  "1"  /* Bypass Intracran Vein to Up Vein w Nonaut Sub, Perc Endo */
"051L4ZY"  =  "1"  /* Bypass Intracranial Vein to Upper Vein, Perc Endo Approach */
"051M07Y"  =  "1"  /* Bypass R Int Jugular Vein to Up Vein w Autol Sub, Open */
"051M09Y"  =  "1"  /* Bypass R Int Jugular Vein to Up Vein w Autol Vn, Open */
"051M0AY"  =  "1"  /* Bypass R Int Jugular Vein to Up Vein w Autol Art, Open */
"051M0JY"  =  "1"  /* Bypass R Int Jugular Vein to Up Vein w Synth Sub, Open */
"051M0KY"  =  "1"  /* Bypass R Int Jugular Vein to Up Vein w Nonaut Sub, Open */
"051M0ZY"  =  "1"  /* Bypass Right Internal Jugular Vein to Up Vein, Open Approach */
"051M47Y"  =  "1"  /* Bypass R Int Jugular Vein to Up Vein w Autol Sub, Perc Endo */
"051M49Y"  =  "1"  /* Bypass R Int Jugular Vein to Up Vein w Autol Vn, Perc Endo */
"051M4AY"  =  "1"  /* Bypass R Int Jugular Vein to Up Vein w Autol Art, Perc Endo */
"051M4JY"  =  "1"  /* Bypass R Int Jugular Vein to Up Vein w Synth Sub, Perc Endo */
"051M4KY"  =  "1"  /* Bypass R Int Jugular Vein to Up Vein w Nonaut Sub, Perc Endo */
"051M4ZY"  =  "1"  /* Bypass R Int Jugular Vein to Up Vein, Perc Endo Approach */
"051N07Y"  =  "1"  /* Bypass L Int Jugular Vein to Up Vein w Autol Sub, Open */
"051N09Y"  =  "1"  /* Bypass L Int Jugular Vein to Up Vein w Autol Vn, Open */
"051N0AY"  =  "1"  /* Bypass L Int Jugular Vein to Up Vein w Autol Art, Open */
"051N0JY"  =  "1"  /* Bypass L Int Jugular Vein to Up Vein w Synth Sub, Open */
"051N0KY"  =  "1"  /* Bypass L Int Jugular Vein to Up Vein w Nonaut Sub, Open */
"051N0ZY"  =  "1"  /* Bypass Left Internal Jugular Vein to Up Vein, Open Approach */
"051N47Y"  =  "1"  /* Bypass L Int Jugular Vein to Up Vein w Autol Sub, Perc Endo */
"051N49Y"  =  "1"  /* Bypass L Int Jugular Vein to Up Vein w Autol Vn, Perc Endo */
"051N4AY"  =  "1"  /* Bypass L Int Jugular Vein to Up Vein w Autol Art, Perc Endo */
"051N4JY"  =  "1"  /* Bypass L Int Jugular Vein to Up Vein w Synth Sub, Perc Endo */
"051N4KY"  =  "1"  /* Bypass L Int Jugular Vein to Up Vein w Nonaut Sub, Perc Endo */
"051N4ZY"  =  "1"  /* Bypass L Int Jugular Vein to Up Vein, Perc Endo Approach */
"051P07Y"  =  "1"  /* Bypass R Ext Jugular Vein to Up Vein w Autol Sub, Open */
"051P09Y"  =  "1"  /* Bypass R Ext Jugular Vein to Up Vein w Autol Vn, Open */
"051P0AY"  =  "1"  /* Bypass R Ext Jugular Vein to Up Vein w Autol Art, Open */
"051P0JY"  =  "1"  /* Bypass R Ext Jugular Vein to Up Vein w Synth Sub, Open */
"051P0KY"  =  "1"  /* Bypass R Ext Jugular Vein to Up Vein w Nonaut Sub, Open */
"051P0ZY"  =  "1"  /* Bypass Right External Jugular Vein to Up Vein, Open Approach */
"051P47Y"  =  "1"  /* Bypass R Ext Jugular Vein to Up Vein w Autol Sub, Perc Endo */
"051P49Y"  =  "1"  /* Bypass R Ext Jugular Vein to Up Vein w Autol Vn, Perc Endo */
"051P4AY"  =  "1"  /* Bypass R Ext Jugular Vein to Up Vein w Autol Art, Perc Endo */
"051P4JY"  =  "1"  /* Bypass R Ext Jugular Vein to Up Vein w Synth Sub, Perc Endo */
"051P4KY"  =  "1"  /* Bypass R Ext Jugular Vein to Up Vein w Nonaut Sub, Perc Endo */
"051P4ZY"  =  "1"  /* Bypass R Ext Jugular Vein to Up Vein, Perc Endo Approach */
"051Q07Y"  =  "1"  /* Bypass L Ext Jugular Vein to Up Vein w Autol Sub, Open */
"051Q09Y"  =  "1"  /* Bypass L Ext Jugular Vein to Up Vein w Autol Vn, Open */
"051Q0AY"  =  "1"  /* Bypass L Ext Jugular Vein to Up Vein w Autol Art, Open */
"051Q0JY"  =  "1"  /* Bypass L Ext Jugular Vein to Up Vein w Synth Sub, Open */
"051Q0KY"  =  "1"  /* Bypass L Ext Jugular Vein to Up Vein w Nonaut Sub, Open */
"051Q0ZY"  =  "1"  /* Bypass Left External Jugular Vein to Up Vein, Open Approach */
"051Q47Y"  =  "1"  /* Bypass L Ext Jugular Vein to Up Vein w Autol Sub, Perc Endo */
"051Q49Y"  =  "1"  /* Bypass L Ext Jugular Vein to Up Vein w Autol Vn, Perc Endo */
"051Q4AY"  =  "1"  /* Bypass L Ext Jugular Vein to Up Vein w Autol Art, Perc Endo */
"051Q4JY"  =  "1"  /* Bypass L Ext Jugular Vein to Up Vein w Synth Sub, Perc Endo */
"051Q4KY"  =  "1"  /* Bypass L Ext Jugular Vein to Up Vein w Nonaut Sub, Perc Endo */
"051Q4ZY"  =  "1"  /* Bypass L Ext Jugular Vein to Up Vein, Perc Endo Approach */
"051R07Y"  =  "1"  /* Bypass R Verteb Vein to Up Vein w Autol Sub, Open */
"051R09Y"  =  "1"  /* Bypass R Verteb Vein to Up Vein with Autol Vn, Open Approach */
"051R0AY"  =  "1"  /* Bypass R Verteb Vein to Up Vein w Autol Art, Open */
"051R0JY"  =  "1"  /* Bypass R Verteb Vein to Up Vein w Synth Sub, Open */
"051R0KY"  =  "1"  /* Bypass R Verteb Vein to Up Vein w Nonaut Sub, Open */
"051R0ZY"  =  "1"  /* Bypass Right Vertebral Vein to Upper Vein, Open Approach */
"051R47Y"  =  "1"  /* Bypass R Verteb Vein to Up Vein w Autol Sub, Perc Endo */
"051R49Y"  =  "1"  /* Bypass R Verteb Vein to Up Vein w Autol Vn, Perc Endo */
"051R4AY"  =  "1"  /* Bypass R Verteb Vein to Up Vein w Autol Art, Perc Endo */
"051R4JY"  =  "1"  /* Bypass R Verteb Vein to Up Vein w Synth Sub, Perc Endo */
"051R4KY"  =  "1"  /* Bypass R Verteb Vein to Up Vein w Nonaut Sub, Perc Endo */
"051R4ZY"  =  "1"  /* Bypass Right Vertebral Vein to Up Vein, Perc Endo Approach */
"051S07Y"  =  "1"  /* Bypass L Verteb Vein to Up Vein w Autol Sub, Open */
"051S09Y"  =  "1"  /* Bypass L Verteb Vein to Up Vein with Autol Vn, Open Approach */
"051S0AY"  =  "1"  /* Bypass L Verteb Vein to Up Vein w Autol Art, Open */
"051S0JY"  =  "1"  /* Bypass L Verteb Vein to Up Vein w Synth Sub, Open */
"051S0KY"  =  "1"  /* Bypass L Verteb Vein to Up Vein w Nonaut Sub, Open */
"051S0ZY"  =  "1"  /* Bypass Left Vertebral Vein to Upper Vein, Open Approach */
"051S47Y"  =  "1"  /* Bypass L Verteb Vein to Up Vein w Autol Sub, Perc Endo */
"051S49Y"  =  "1"  /* Bypass L Verteb Vein to Up Vein w Autol Vn, Perc Endo */
"051S4AY"  =  "1"  /* Bypass L Verteb Vein to Up Vein w Autol Art, Perc Endo */
"051S4JY"  =  "1"  /* Bypass L Verteb Vein to Up Vein w Synth Sub, Perc Endo */
"051S4KY"  =  "1"  /* Bypass L Verteb Vein to Up Vein w Nonaut Sub, Perc Endo */
"051S4ZY"  =  "1"  /* Bypass Left Vertebral Vein to Upper Vein, Perc Endo Approach */
"051T07Y"  =  "1"  /* Bypass R Face Vein to Up Vein with Autol Sub, Open Approach */
"051T09Y"  =  "1"  /* Bypass R Face Vein to Up Vein with Autol Vn, Open Approach */
"051T0AY"  =  "1"  /* Bypass R Face Vein to Up Vein with Autol Art, Open Approach */
"051T0JY"  =  "1"  /* Bypass R Face Vein to Up Vein with Synth Sub, Open Approach */
"051T0KY"  =  "1"  /* Bypass R Face Vein to Up Vein with Nonaut Sub, Open Approach */
"051T0ZY"  =  "1"  /* Bypass Right Face Vein to Upper Vein, Open Approach */
"051T47Y"  =  "1"  /* Bypass R Face Vein to Up Vein w Autol Sub, Perc Endo */
"051T49Y"  =  "1"  /* Bypass R Face Vein to Up Vein w Autol Vn, Perc Endo */
"051T4AY"  =  "1"  /* Bypass R Face Vein to Up Vein w Autol Art, Perc Endo */
"051T4JY"  =  "1"  /* Bypass R Face Vein to Up Vein w Synth Sub, Perc Endo */
"051T4KY"  =  "1"  /* Bypass R Face Vein to Up Vein w Nonaut Sub, Perc Endo */
"051T4ZY"  =  "1"  /* Bypass Right Face Vein to Upper Vein, Perc Endo Approach */
"051V07Y"  =  "1"  /* Bypass L Face Vein to Up Vein with Autol Sub, Open Approach */
"051V09Y"  =  "1"  /* Bypass L Face Vein to Up Vein with Autol Vn, Open Approach */
"051V0AY"  =  "1"  /* Bypass L Face Vein to Up Vein with Autol Art, Open Approach */
"051V0JY"  =  "1"  /* Bypass L Face Vein to Up Vein with Synth Sub, Open Approach */
"051V0KY"  =  "1"  /* Bypass L Face Vein to Up Vein with Nonaut Sub, Open Approach */
"051V0ZY"  =  "1"  /* Bypass Left Face Vein to Upper Vein, Open Approach */
"051V47Y"  =  "1"  /* Bypass L Face Vein to Up Vein w Autol Sub, Perc Endo */
"051V49Y"  =  "1"  /* Bypass L Face Vein to Up Vein w Autol Vn, Perc Endo */
"051V4AY"  =  "1"  /* Bypass L Face Vein to Up Vein w Autol Art, Perc Endo */
"051V4JY"  =  "1"  /* Bypass L Face Vein to Up Vein w Synth Sub, Perc Endo */
"051V4KY"  =  "1"  /* Bypass L Face Vein to Up Vein w Nonaut Sub, Perc Endo */
"051V4ZY"  =  "1"  /* Bypass Left Face Vein to Upper Vein, Perc Endo Approach */
"05500ZZ"  =  "1"  /* Destruction of Azygos Vein, Open Approach */
"05503ZZ"  =  "1"  /* Destruction of Azygos Vein, Percutaneous Approach */
"05504ZZ"  =  "1"  /* Destruction of Azygos Vein, Percutaneous Endoscopic Approach */
"05510ZZ"  =  "1"  /* Destruction of Hemiazygos Vein, Open Approach */
"05513ZZ"  =  "1"  /* Destruction of Hemiazygos Vein, Percutaneous Approach */
"05514ZZ"  =  "1"  /* Destruction of Hemiazygos Vein, Perc Endo Approach */
"05530ZZ"  =  "1"  /* Destruction of Right Innominate Vein, Open Approach */
"05533ZZ"  =  "1"  /* Destruction of Right Innominate Vein, Percutaneous Approach */
"05534ZZ"  =  "1"  /* Destruction of Right Innominate Vein, Perc Endo Approach */
"05540ZZ"  =  "1"  /* Destruction of Left Innominate Vein, Open Approach */
"05543ZZ"  =  "1"  /* Destruction of Left Innominate Vein, Percutaneous Approach */
"05544ZZ"  =  "1"  /* Destruction of Left Innominate Vein, Perc Endo Approach */
"05550ZZ"  =  "1"  /* Destruction of Right Subclavian Vein, Open Approach */
"05553ZZ"  =  "1"  /* Destruction of Right Subclavian Vein, Percutaneous Approach */
"05554ZZ"  =  "1"  /* Destruction of Right Subclavian Vein, Perc Endo Approach */
"05560ZZ"  =  "1"  /* Destruction of Left Subclavian Vein, Open Approach */
"05563ZZ"  =  "1"  /* Destruction of Left Subclavian Vein, Percutaneous Approach */
"05564ZZ"  =  "1"  /* Destruction of Left Subclavian Vein, Perc Endo Approach */
"05570ZZ"  =  "1"  /* Destruction of Right Axillary Vein, Open Approach */
"05573ZZ"  =  "1"  /* Destruction of Right Axillary Vein, Percutaneous Approach */
"05574ZZ"  =  "1"  /* Destruction of Right Axillary Vein, Perc Endo Approach */
"05580ZZ"  =  "1"  /* Destruction of Left Axillary Vein, Open Approach */
"05583ZZ"  =  "1"  /* Destruction of Left Axillary Vein, Percutaneous Approach */
"05584ZZ"  =  "1"  /* Destruction of Left Axillary Vein, Perc Endo Approach */
"05590ZZ"  =  "1"  /* Destruction of Right Brachial Vein, Open Approach */
"05593ZZ"  =  "1"  /* Destruction of Right Brachial Vein, Percutaneous Approach */
"05594ZZ"  =  "1"  /* Destruction of Right Brachial Vein, Perc Endo Approach */
"055A0ZZ"  =  "1"  /* Destruction of Left Brachial Vein, Open Approach */
"055A3ZZ"  =  "1"  /* Destruction of Left Brachial Vein, Percutaneous Approach */
"055A4ZZ"  =  "1"  /* Destruction of Left Brachial Vein, Perc Endo Approach */
"055B0ZZ"  =  "1"  /* Destruction of Right Basilic Vein, Open Approach */
"055B3ZZ"  =  "1"  /* Destruction of Right Basilic Vein, Percutaneous Approach */
"055B4ZZ"  =  "1"  /* Destruction of Right Basilic Vein, Perc Endo Approach */
"055C0ZZ"  =  "1"  /* Destruction of Left Basilic Vein, Open Approach */
"055C3ZZ"  =  "1"  /* Destruction of Left Basilic Vein, Percutaneous Approach */
"055C4ZZ"  =  "1"  /* Destruction of Left Basilic Vein, Perc Endo Approach */
"055D0ZZ"  =  "1"  /* Destruction of Right Cephalic Vein, Open Approach */
"055D3ZZ"  =  "1"  /* Destruction of Right Cephalic Vein, Percutaneous Approach */
"055D4ZZ"  =  "1"  /* Destruction of Right Cephalic Vein, Perc Endo Approach */
"055F0ZZ"  =  "1"  /* Destruction of Left Cephalic Vein, Open Approach */
"055F3ZZ"  =  "1"  /* Destruction of Left Cephalic Vein, Percutaneous Approach */
"055F4ZZ"  =  "1"  /* Destruction of Left Cephalic Vein, Perc Endo Approach */
"055G0ZZ"  =  "1"  /* Destruction of Right Hand Vein, Open Approach */
"055G3ZZ"  =  "1"  /* Destruction of Right Hand Vein, Percutaneous Approach */
"055G4ZZ"  =  "1"  /* Destruction of Right Hand Vein, Perc Endo Approach */
"055H0ZZ"  =  "1"  /* Destruction of Left Hand Vein, Open Approach */
"055H3ZZ"  =  "1"  /* Destruction of Left Hand Vein, Percutaneous Approach */
"055H4ZZ"  =  "1"  /* Destruction of Left Hand Vein, Perc Endo Approach */
"055L0ZZ"  =  "1"  /* Destruction of Intracranial Vein, Open Approach */
"055L3ZZ"  =  "1"  /* Destruction of Intracranial Vein, Percutaneous Approach */
"055L4ZZ"  =  "1"  /* Destruction of Intracranial Vein, Perc Endo Approach */
"055M0ZZ"  =  "1"  /* Destruction of Right Internal Jugular Vein, Open Approach */
"055M3ZZ"  =  "1"  /* Destruction of Right Internal Jugular Vein, Perc Approach */
"055M4ZZ"  =  "1"  /* Destruction of R Int Jugular Vein, Perc Endo Approach */
"055N0ZZ"  =  "1"  /* Destruction of Left Internal Jugular Vein, Open Approach */
"055N3ZZ"  =  "1"  /* Destruction of Left Internal Jugular Vein, Perc Approach */
"055N4ZZ"  =  "1"  /* Destruction of L Int Jugular Vein, Perc Endo Approach */
"055P0ZZ"  =  "1"  /* Destruction of Right External Jugular Vein, Open Approach */
"055P3ZZ"  =  "1"  /* Destruction of Right External Jugular Vein, Perc Approach */
"055P4ZZ"  =  "1"  /* Destruction of R Ext Jugular Vein, Perc Endo Approach */
"055Q0ZZ"  =  "1"  /* Destruction of Left External Jugular Vein, Open Approach */
"055Q3ZZ"  =  "1"  /* Destruction of Left External Jugular Vein, Perc Approach */
"055Q4ZZ"  =  "1"  /* Destruction of L Ext Jugular Vein, Perc Endo Approach */
"055R0ZZ"  =  "1"  /* Destruction of Right Vertebral Vein, Open Approach */
"055R3ZZ"  =  "1"  /* Destruction of Right Vertebral Vein, Percutaneous Approach */
"055R4ZZ"  =  "1"  /* Destruction of Right Vertebral Vein, Perc Endo Approach */
"055S0ZZ"  =  "1"  /* Destruction of Left Vertebral Vein, Open Approach */
"055S3ZZ"  =  "1"  /* Destruction of Left Vertebral Vein, Percutaneous Approach */
"055S4ZZ"  =  "1"  /* Destruction of Left Vertebral Vein, Perc Endo Approach */
"055T0ZZ"  =  "1"  /* Destruction of Right Face Vein, Open Approach */
"055T3ZZ"  =  "1"  /* Destruction of Right Face Vein, Percutaneous Approach */
"055T4ZZ"  =  "1"  /* Destruction of Right Face Vein, Perc Endo Approach */
"055V0ZZ"  =  "1"  /* Destruction of Left Face Vein, Open Approach */
"055V3ZZ"  =  "1"  /* Destruction of Left Face Vein, Percutaneous Approach */
"055V4ZZ"  =  "1"  /* Destruction of Left Face Vein, Perc Endo Approach */
"055Y0ZZ"  =  "1"  /* Destruction of Upper Vein, Open Approach */
"055Y3ZZ"  =  "1"  /* Destruction of Upper Vein, Percutaneous Approach */
"055Y4ZZ"  =  "1"  /* Destruction of Upper Vein, Percutaneous Endoscopic Approach */
"05700DZ"  =  "1"  /* Dilation of Azygos Vein with Intralum Dev, Open Approach */
"05700ZZ"  =  "1"  /* Dilation of Azygos Vein, Open Approach */
"05703DZ"  =  "1"  /* Dilation of Azygos Vein with Intralum Dev, Perc Approach */
"05703ZZ"  =  "1"  /* Dilation of Azygos Vein, Percutaneous Approach */
"05704DZ"  =  "1"  /* Dilate of Azygos Vein with Intralum Dev, Perc Endo Approach */
"05704ZZ"  =  "1"  /* Dilation of Azygos Vein, Percutaneous Endoscopic Approach */
"05710DZ"  =  "1"  /* Dilation of Hemiazygos Vein with Intralum Dev, Open Approach */
"05710ZZ"  =  "1"  /* Dilation of Hemiazygos Vein, Open Approach */
"05713DZ"  =  "1"  /* Dilation of Hemiazygos Vein with Intralum Dev, Perc Approach */
"05713ZZ"  =  "1"  /* Dilation of Hemiazygos Vein, Percutaneous Approach */
"05714DZ"  =  "1"  /* Dilate Hemiazygos Vein w Intralum Dev, Perc Endo */
"05714ZZ"  =  "1"  /* Dilation of Hemiazygos Vein, Perc Endo Approach */
"05730DZ"  =  "1"  /* Dilation of R Innom Vein with Intralum Dev, Open Approach */
"05730ZZ"  =  "1"  /* Dilation of Right Innominate Vein, Open Approach */
"05733DZ"  =  "1"  /* Dilation of R Innom Vein with Intralum Dev, Perc Approach */
"05733ZZ"  =  "1"  /* Dilation of Right Innominate Vein, Percutaneous Approach */
"05734DZ"  =  "1"  /* Dilate of R Innom Vein with Intralum Dev, Perc Endo Approach */
"05734ZZ"  =  "1"  /* Dilation of Right Innominate Vein, Perc Endo Approach */
"05740DZ"  =  "1"  /* Dilation of L Innom Vein with Intralum Dev, Open Approach */
"05740ZZ"  =  "1"  /* Dilation of Left Innominate Vein, Open Approach */
"05743DZ"  =  "1"  /* Dilation of L Innom Vein with Intralum Dev, Perc Approach */
"05743ZZ"  =  "1"  /* Dilation of Left Innominate Vein, Percutaneous Approach */
"05744DZ"  =  "1"  /* Dilate of L Innom Vein with Intralum Dev, Perc Endo Approach */
"05744ZZ"  =  "1"  /* Dilation of Left Innominate Vein, Perc Endo Approach */
"05750DZ"  =  "1"  /* Dilation of R Subclav Vein with Intralum Dev, Open Approach */
"05750ZZ"  =  "1"  /* Dilation of Right Subclavian Vein, Open Approach */
"05753DZ"  =  "1"  /* Dilation of R Subclav Vein with Intralum Dev, Perc Approach */
"05753ZZ"  =  "1"  /* Dilation of Right Subclavian Vein, Percutaneous Approach */
"05754DZ"  =  "1"  /* Dilate R Subclav Vein w Intralum Dev, Perc Endo */
"05754ZZ"  =  "1"  /* Dilation of Right Subclavian Vein, Perc Endo Approach */
"05760DZ"  =  "1"  /* Dilation of L Subclav Vein with Intralum Dev, Open Approach */
"05760ZZ"  =  "1"  /* Dilation of Left Subclavian Vein, Open Approach */
"05763DZ"  =  "1"  /* Dilation of L Subclav Vein with Intralum Dev, Perc Approach */
"05763ZZ"  =  "1"  /* Dilation of Left Subclavian Vein, Percutaneous Approach */
"05764DZ"  =  "1"  /* Dilate L Subclav Vein w Intralum Dev, Perc Endo */
"05764ZZ"  =  "1"  /* Dilation of Left Subclavian Vein, Perc Endo Approach */
"05770DZ"  =  "1"  /* Dilation of R Axilla Vein with Intralum Dev, Open Approach */
"05770ZZ"  =  "1"  /* Dilation of Right Axillary Vein, Open Approach */
"05773DZ"  =  "1"  /* Dilation of R Axilla Vein with Intralum Dev, Perc Approach */
"05773ZZ"  =  "1"  /* Dilation of Right Axillary Vein, Percutaneous Approach */
"05774DZ"  =  "1"  /* Dilate R Axilla Vein w Intralum Dev, Perc Endo */
"05774ZZ"  =  "1"  /* Dilation of Right Axillary Vein, Perc Endo Approach */
"05780DZ"  =  "1"  /* Dilation of L Axilla Vein with Intralum Dev, Open Approach */
"05780ZZ"  =  "1"  /* Dilation of Left Axillary Vein, Open Approach */
"05783DZ"  =  "1"  /* Dilation of L Axilla Vein with Intralum Dev, Perc Approach */
"05783ZZ"  =  "1"  /* Dilation of Left Axillary Vein, Percutaneous Approach */
"05784DZ"  =  "1"  /* Dilate L Axilla Vein w Intralum Dev, Perc Endo */
"05784ZZ"  =  "1"  /* Dilation of Left Axillary Vein, Perc Endo Approach */
"05790DZ"  =  "1"  /* Dilation of R Brach Vein with Intralum Dev, Open Approach */
"05790ZZ"  =  "1"  /* Dilation of Right Brachial Vein, Open Approach */
"05793DZ"  =  "1"  /* Dilation of R Brach Vein with Intralum Dev, Perc Approach */
"05793ZZ"  =  "1"  /* Dilation of Right Brachial Vein, Percutaneous Approach */
"05794DZ"  =  "1"  /* Dilate of R Brach Vein with Intralum Dev, Perc Endo Approach */
"05794ZZ"  =  "1"  /* Dilation of Right Brachial Vein, Perc Endo Approach */
"057A0DZ"  =  "1"  /* Dilation of L Brach Vein with Intralum Dev, Open Approach */
"057A0ZZ"  =  "1"  /* Dilation of Left Brachial Vein, Open Approach */
"057A3DZ"  =  "1"  /* Dilation of L Brach Vein with Intralum Dev, Perc Approach */
"057A3ZZ"  =  "1"  /* Dilation of Left Brachial Vein, Percutaneous Approach */
"057A4DZ"  =  "1"  /* Dilate of L Brach Vein with Intralum Dev, Perc Endo Approach */
"057A4ZZ"  =  "1"  /* Dilation of Left Brachial Vein, Perc Endo Approach */
"057B0DZ"  =  "1"  /* Dilation of R Basilic Vein with Intralum Dev, Open Approach */
"057B0ZZ"  =  "1"  /* Dilation of Right Basilic Vein, Open Approach */
"057B3DZ"  =  "1"  /* Dilation of R Basilic Vein with Intralum Dev, Perc Approach */
"057B3ZZ"  =  "1"  /* Dilation of Right Basilic Vein, Percutaneous Approach */
"057B4DZ"  =  "1"  /* Dilate R Basilic Vein w Intralum Dev, Perc Endo */
"057B4ZZ"  =  "1"  /* Dilation of Right Basilic Vein, Perc Endo Approach */
"057C0DZ"  =  "1"  /* Dilation of L Basilic Vein with Intralum Dev, Open Approach */
"057C0ZZ"  =  "1"  /* Dilation of Left Basilic Vein, Open Approach */
"057C3DZ"  =  "1"  /* Dilation of L Basilic Vein with Intralum Dev, Perc Approach */
"057C3ZZ"  =  "1"  /* Dilation of Left Basilic Vein, Percutaneous Approach */
"057C4DZ"  =  "1"  /* Dilate L Basilic Vein w Intralum Dev, Perc Endo */
"057C4ZZ"  =  "1"  /* Dilation of Left Basilic Vein, Perc Endo Approach */
"057D0DZ"  =  "1"  /* Dilation of R Cephalic Vein with Intralum Dev, Open Approach */
"057D0ZZ"  =  "1"  /* Dilation of Right Cephalic Vein, Open Approach */
"057D3DZ"  =  "1"  /* Dilation of R Cephalic Vein with Intralum Dev, Perc Approach */
"057D3ZZ"  =  "1"  /* Dilation of Right Cephalic Vein, Percutaneous Approach */
"057D4DZ"  =  "1"  /* Dilate R Cephalic Vein w Intralum Dev, Perc Endo */
"057D4ZZ"  =  "1"  /* Dilation of Right Cephalic Vein, Perc Endo Approach */
"057F0DZ"  =  "1"  /* Dilation of L Cephalic Vein with Intralum Dev, Open Approach */
"057F0ZZ"  =  "1"  /* Dilation of Left Cephalic Vein, Open Approach */
"057F3DZ"  =  "1"  /* Dilation of L Cephalic Vein with Intralum Dev, Perc Approach */
"057F3ZZ"  =  "1"  /* Dilation of Left Cephalic Vein, Percutaneous Approach */
"057F4DZ"  =  "1"  /* Dilate L Cephalic Vein w Intralum Dev, Perc Endo */
"057F4ZZ"  =  "1"  /* Dilation of Left Cephalic Vein, Perc Endo Approach */
"057G0DZ"  =  "1"  /* Dilation of Right Hand Vein with Intralum Dev, Open Approach */
"057G0ZZ"  =  "1"  /* Dilation of Right Hand Vein, Open Approach */
"057G3DZ"  =  "1"  /* Dilation of Right Hand Vein with Intralum Dev, Perc Approach */
"057G3ZZ"  =  "1"  /* Dilation of Right Hand Vein, Percutaneous Approach */
"057G4DZ"  =  "1"  /* Dilate of R Hand Vein with Intralum Dev, Perc Endo Approach */
"057G4ZZ"  =  "1"  /* Dilation of Right Hand Vein, Perc Endo Approach */
"057H0DZ"  =  "1"  /* Dilation of Left Hand Vein with Intralum Dev, Open Approach */
"057H0ZZ"  =  "1"  /* Dilation of Left Hand Vein, Open Approach */
"057H3DZ"  =  "1"  /* Dilation of Left Hand Vein with Intralum Dev, Perc Approach */
"057H3ZZ"  =  "1"  /* Dilation of Left Hand Vein, Percutaneous Approach */
"057H4DZ"  =  "1"  /* Dilate of L Hand Vein with Intralum Dev, Perc Endo Approach */
"057H4ZZ"  =  "1"  /* Dilation of Left Hand Vein, Percutaneous Endoscopic Approach */
"057L0DZ"  =  "1"  /* Dilation of Intracran Vein with Intralum Dev, Open Approach */
"057L0ZZ"  =  "1"  /* Dilation of Intracranial Vein, Open Approach */
"057L3DZ"  =  "1"  /* Dilation of Intracran Vein with Intralum Dev, Perc Approach */
"057L3ZZ"  =  "1"  /* Dilation of Intracranial Vein, Percutaneous Approach */
"057L4DZ"  =  "1"  /* Dilate Intracran Vein w Intralum Dev, Perc Endo */
"057L4ZZ"  =  "1"  /* Dilation of Intracranial Vein, Perc Endo Approach */
"057M0DZ"  =  "1"  /* Dilate R Int Jugular Vein w Intralum Dev, Open */
"057M0ZZ"  =  "1"  /* Dilation of Right Internal Jugular Vein, Open Approach */
"057M3DZ"  =  "1"  /* Dilate R Int Jugular Vein w Intralum Dev, Perc */
"057M3ZZ"  =  "1"  /* Dilation of Right Internal Jugular Vein, Perc Approach */
"057M4DZ"  =  "1"  /* Dilate R Int Jugular Vein w Intralum Dev, Perc Endo */
"057M4ZZ"  =  "1"  /* Dilation of Right Internal Jugular Vein, Perc Endo Approach */
"057N0DZ"  =  "1"  /* Dilate L Int Jugular Vein w Intralum Dev, Open */
"057N0ZZ"  =  "1"  /* Dilation of Left Internal Jugular Vein, Open Approach */
"057N3DZ"  =  "1"  /* Dilate L Int Jugular Vein w Intralum Dev, Perc */
"057N3ZZ"  =  "1"  /* Dilation of Left Internal Jugular Vein, Perc Approach */
"057N4DZ"  =  "1"  /* Dilate L Int Jugular Vein w Intralum Dev, Perc Endo */
"057N4ZZ"  =  "1"  /* Dilation of Left Internal Jugular Vein, Perc Endo Approach */
"057P0DZ"  =  "1"  /* Dilate R Ext Jugular Vein w Intralum Dev, Open */
"057P0ZZ"  =  "1"  /* Dilation of Right External Jugular Vein, Open Approach */
"057P3DZ"  =  "1"  /* Dilate R Ext Jugular Vein w Intralum Dev, Perc */
"057P3ZZ"  =  "1"  /* Dilation of Right External Jugular Vein, Perc Approach */
"057P4DZ"  =  "1"  /* Dilate R Ext Jugular Vein w Intralum Dev, Perc Endo */
"057P4ZZ"  =  "1"  /* Dilation of Right External Jugular Vein, Perc Endo Approach */
"057Q0DZ"  =  "1"  /* Dilate L Ext Jugular Vein w Intralum Dev, Open */
"057Q0ZZ"  =  "1"  /* Dilation of Left External Jugular Vein, Open Approach */
"057Q3DZ"  =  "1"  /* Dilate L Ext Jugular Vein w Intralum Dev, Perc */
"057Q3ZZ"  =  "1"  /* Dilation of Left External Jugular Vein, Perc Approach */
"057Q4DZ"  =  "1"  /* Dilate L Ext Jugular Vein w Intralum Dev, Perc Endo */
"057Q4ZZ"  =  "1"  /* Dilation of Left External Jugular Vein, Perc Endo Approach */
"057R0DZ"  =  "1"  /* Dilation of R Verteb Vein with Intralum Dev, Open Approach */
"057R0ZZ"  =  "1"  /* Dilation of Right Vertebral Vein, Open Approach */
"057R3DZ"  =  "1"  /* Dilation of R Verteb Vein with Intralum Dev, Perc Approach */
"057R3ZZ"  =  "1"  /* Dilation of Right Vertebral Vein, Percutaneous Approach */
"057R4DZ"  =  "1"  /* Dilate R Verteb Vein w Intralum Dev, Perc Endo */
"057R4ZZ"  =  "1"  /* Dilation of Right Vertebral Vein, Perc Endo Approach */
"057S0DZ"  =  "1"  /* Dilation of L Verteb Vein with Intralum Dev, Open Approach */
"057S0ZZ"  =  "1"  /* Dilation of Left Vertebral Vein, Open Approach */
"057S3DZ"  =  "1"  /* Dilation of L Verteb Vein with Intralum Dev, Perc Approach */
"057S3ZZ"  =  "1"  /* Dilation of Left Vertebral Vein, Percutaneous Approach */
"057S4DZ"  =  "1"  /* Dilate L Verteb Vein w Intralum Dev, Perc Endo */
"057S4ZZ"  =  "1"  /* Dilation of Left Vertebral Vein, Perc Endo Approach */
"057T0DZ"  =  "1"  /* Dilation of Right Face Vein with Intralum Dev, Open Approach */
"057T0ZZ"  =  "1"  /* Dilation of Right Face Vein, Open Approach */
"057T3DZ"  =  "1"  /* Dilation of Right Face Vein with Intralum Dev, Perc Approach */
"057T3ZZ"  =  "1"  /* Dilation of Right Face Vein, Percutaneous Approach */
"057T4DZ"  =  "1"  /* Dilate of R Face Vein with Intralum Dev, Perc Endo Approach */
"057T4ZZ"  =  "1"  /* Dilation of Right Face Vein, Perc Endo Approach */
"057V0DZ"  =  "1"  /* Dilation of Left Face Vein with Intralum Dev, Open Approach */
"057V0ZZ"  =  "1"  /* Dilation of Left Face Vein, Open Approach */
"057V3DZ"  =  "1"  /* Dilation of Left Face Vein with Intralum Dev, Perc Approach */
"057V3ZZ"  =  "1"  /* Dilation of Left Face Vein, Percutaneous Approach */
"057V4DZ"  =  "1"  /* Dilate of L Face Vein with Intralum Dev, Perc Endo Approach */
"057V4ZZ"  =  "1"  /* Dilation of Left Face Vein, Percutaneous Endoscopic Approach */
"057Y0DZ"  =  "1"  /* Dilation of Upper Vein with Intralum Dev, Open Approach */
"057Y0ZZ"  =  "1"  /* Dilation of Upper Vein, Open Approach */
"057Y3DZ"  =  "1"  /* Dilation of Upper Vein with Intralum Dev, Perc Approach */
"057Y3ZZ"  =  "1"  /* Dilation of Upper Vein, Percutaneous Approach */
"057Y4DZ"  =  "1"  /* Dilation of Upper Vein with Intralum Dev, Perc Endo Approach */
"057Y4ZZ"  =  "1"  /* Dilation of Upper Vein, Percutaneous Endoscopic Approach */
"05900ZX"  =  "1"  /* Drainage of Azygos Vein, Open Approach, Diagnostic */
"05903ZX"  =  "1"  /* Drainage of Azygos Vein, Percutaneous Approach, Diagnostic */
"05904ZX"  =  "1"  /* Drainage of Azygos Vein, Perc Endo Approach, Diagn */
"05910ZX"  =  "1"  /* Drainage of Hemiazygos Vein, Open Approach, Diagnostic */
"05913ZX"  =  "1"  /* Drainage of Hemiazygos Vein, Percutaneous Approach, Diagn */
"05914ZX"  =  "1"  /* Drainage of Hemiazygos Vein, Perc Endo Approach, Diagn */
"05930ZX"  =  "1"  /* Drainage of Right Innominate Vein, Open Approach, Diagnostic */
"05933ZX"  =  "1"  /* Drainage of Right Innominate Vein, Perc Approach, Diagn */
"05934ZX"  =  "1"  /* Drainage of Right Innominate Vein, Perc Endo Approach, Diagn */
"05940ZX"  =  "1"  /* Drainage of Left Innominate Vein, Open Approach, Diagnostic */
"05943ZX"  =  "1"  /* Drainage of Left Innominate Vein, Perc Approach, Diagn */
"05944ZX"  =  "1"  /* Drainage of Left Innominate Vein, Perc Endo Approach, Diagn */
"05950ZX"  =  "1"  /* Drainage of Right Subclavian Vein, Open Approach, Diagnostic */
"05953ZX"  =  "1"  /* Drainage of Right Subclavian Vein, Perc Approach, Diagn */
"05954ZX"  =  "1"  /* Drainage of Right Subclavian Vein, Perc Endo Approach, Diagn */
"05960ZX"  =  "1"  /* Drainage of Left Subclavian Vein, Open Approach, Diagnostic */
"05963ZX"  =  "1"  /* Drainage of Left Subclavian Vein, Perc Approach, Diagn */
"05964ZX"  =  "1"  /* Drainage of Left Subclavian Vein, Perc Endo Approach, Diagn */
"05970ZX"  =  "1"  /* Drainage of Right Axillary Vein, Open Approach, Diagnostic */
"05973ZX"  =  "1"  /* Drainage of Right Axillary Vein, Perc Approach, Diagn */
"05974ZX"  =  "1"  /* Drainage of Right Axillary Vein, Perc Endo Approach, Diagn */
"05980ZX"  =  "1"  /* Drainage of Left Axillary Vein, Open Approach, Diagnostic */
"05983ZX"  =  "1"  /* Drainage of Left Axillary Vein, Percutaneous Approach, Diagn */
"05984ZX"  =  "1"  /* Drainage of Left Axillary Vein, Perc Endo Approach, Diagn */
"05990ZX"  =  "1"  /* Drainage of Right Brachial Vein, Open Approach, Diagnostic */
"05993ZX"  =  "1"  /* Drainage of Right Brachial Vein, Perc Approach, Diagn */
"05994ZX"  =  "1"  /* Drainage of Right Brachial Vein, Perc Endo Approach, Diagn */
"059A0ZX"  =  "1"  /* Drainage of Left Brachial Vein, Open Approach, Diagnostic */
"059A3ZX"  =  "1"  /* Drainage of Left Brachial Vein, Percutaneous Approach, Diagn */
"059A4ZX"  =  "1"  /* Drainage of Left Brachial Vein, Perc Endo Approach, Diagn */
"059B0ZX"  =  "1"  /* Drainage of Right Basilic Vein, Open Approach, Diagnostic */
"059B3ZX"  =  "1"  /* Drainage of Right Basilic Vein, Percutaneous Approach, Diagn */
"059B4ZX"  =  "1"  /* Drainage of Right Basilic Vein, Perc Endo Approach, Diagn */
"059C0ZX"  =  "1"  /* Drainage of Left Basilic Vein, Open Approach, Diagnostic */
"059C3ZX"  =  "1"  /* Drainage of Left Basilic Vein, Percutaneous Approach, Diagn */
"059C4ZX"  =  "1"  /* Drainage of Left Basilic Vein, Perc Endo Approach, Diagn */
"059D0ZX"  =  "1"  /* Drainage of Right Cephalic Vein, Open Approach, Diagnostic */
"059D3ZX"  =  "1"  /* Drainage of Right Cephalic Vein, Perc Approach, Diagn */
"059D4ZX"  =  "1"  /* Drainage of Right Cephalic Vein, Perc Endo Approach, Diagn */
"059F0ZX"  =  "1"  /* Drainage of Left Cephalic Vein, Open Approach, Diagnostic */
"059F3ZX"  =  "1"  /* Drainage of Left Cephalic Vein, Percutaneous Approach, Diagn */
"059F4ZX"  =  "1"  /* Drainage of Left Cephalic Vein, Perc Endo Approach, Diagn */
"059G0ZX"  =  "1"  /* Drainage of Right Hand Vein, Open Approach, Diagnostic */
"059G3ZX"  =  "1"  /* Drainage of Right Hand Vein, Percutaneous Approach, Diagn */
"059G4ZX"  =  "1"  /* Drainage of Right Hand Vein, Perc Endo Approach, Diagn */
"059H0ZX"  =  "1"  /* Drainage of Left Hand Vein, Open Approach, Diagnostic */
"059H3ZX"  =  "1"  /* Drainage of Left Hand Vein, Percutaneous Approach, Diagn */
"059H4ZX"  =  "1"  /* Drainage of Left Hand Vein, Perc Endo Approach, Diagn */
"059L0ZX"  =  "1"  /* Drainage of Intracranial Vein, Open Approach, Diagnostic */
"059L3ZX"  =  "1"  /* Drainage of Intracranial Vein, Percutaneous Approach, Diagn */
"059L4ZX"  =  "1"  /* Drainage of Intracranial Vein, Perc Endo Approach, Diagn */
"059M0ZX"  =  "1"  /* Drainage of R Int Jugular Vein, Open Approach, Diagn */
"059M3ZX"  =  "1"  /* Drainage of R Int Jugular Vein, Perc Approach, Diagn */
"059M4ZX"  =  "1"  /* Drainage of R Int Jugular Vein, Perc Endo Approach, Diagn */
"059N0ZX"  =  "1"  /* Drainage of Left Internal Jugular Vein, Open Approach, Diagn */
"059N3ZX"  =  "1"  /* Drainage of Left Internal Jugular Vein, Perc Approach, Diagn */
"059N4ZX"  =  "1"  /* Drainage of L Int Jugular Vein, Perc Endo Approach, Diagn */
"059P0ZX"  =  "1"  /* Drainage of R Ext Jugular Vein, Open Approach, Diagn */
"059P3ZX"  =  "1"  /* Drainage of R Ext Jugular Vein, Perc Approach, Diagn */
"059P4ZX"  =  "1"  /* Drainage of R Ext Jugular Vein, Perc Endo Approach, Diagn */
"059Q0ZX"  =  "1"  /* Drainage of Left External Jugular Vein, Open Approach, Diagn */
"059Q3ZX"  =  "1"  /* Drainage of Left External Jugular Vein, Perc Approach, Diagn */
"059Q4ZX"  =  "1"  /* Drainage of L Ext Jugular Vein, Perc Endo Approach, Diagn */
"059R0ZX"  =  "1"  /* Drainage of Right Vertebral Vein, Open Approach, Diagnostic */
"059R3ZX"  =  "1"  /* Drainage of Right Vertebral Vein, Perc Approach, Diagn */
"059R4ZX"  =  "1"  /* Drainage of Right Vertebral Vein, Perc Endo Approach, Diagn */
"059S0ZX"  =  "1"  /* Drainage of Left Vertebral Vein, Open Approach, Diagnostic */
"059S3ZX"  =  "1"  /* Drainage of Left Vertebral Vein, Perc Approach, Diagn */
"059S4ZX"  =  "1"  /* Drainage of Left Vertebral Vein, Perc Endo Approach, Diagn */
"059T0ZX"  =  "1"  /* Drainage of Right Face Vein, Open Approach, Diagnostic */
"059T3ZX"  =  "1"  /* Drainage of Right Face Vein, Percutaneous Approach, Diagn */
"059T4ZX"  =  "1"  /* Drainage of Right Face Vein, Perc Endo Approach, Diagn */
"059V0ZX"  =  "1"  /* Drainage of Left Face Vein, Open Approach, Diagnostic */
"059V3ZX"  =  "1"  /* Drainage of Left Face Vein, Percutaneous Approach, Diagn */
"059V4ZX"  =  "1"  /* Drainage of Left Face Vein, Perc Endo Approach, Diagn */
"059Y0ZX"  =  "1"  /* Drainage of Upper Vein, Open Approach, Diagnostic */
"059Y3ZX"  =  "1"  /* Drainage of Upper Vein, Percutaneous Approach, Diagnostic */
"059Y4ZX"  =  "1"  /* Drainage of Upper Vein, Perc Endo Approach, Diagn */
"05B00ZX"  =  "1"  /* Excision of Azygos Vein, Open Approach, Diagnostic */
"05B00ZZ"  =  "1"  /* Excision of Azygos Vein, Open Approach */
"05B03ZX"  =  "1"  /* Excision of Azygos Vein, Percutaneous Approach, Diagnostic */
"05B03ZZ"  =  "1"  /* Excision of Azygos Vein, Percutaneous Approach */
"05B04ZX"  =  "1"  /* Excision of Azygos Vein, Perc Endo Approach, Diagn */
"05B04ZZ"  =  "1"  /* Excision of Azygos Vein, Percutaneous Endoscopic Approach */
"05B10ZX"  =  "1"  /* Excision of Hemiazygos Vein, Open Approach, Diagnostic */
"05B10ZZ"  =  "1"  /* Excision of Hemiazygos Vein, Open Approach */
"05B13ZX"  =  "1"  /* Excision of Hemiazygos Vein, Percutaneous Approach, Diagn */
"05B13ZZ"  =  "1"  /* Excision of Hemiazygos Vein, Percutaneous Approach */
"05B14ZX"  =  "1"  /* Excision of Hemiazygos Vein, Perc Endo Approach, Diagn */
"05B14ZZ"  =  "1"  /* Excision of Hemiazygos Vein, Perc Endo Approach */
"05B30ZX"  =  "1"  /* Excision of Right Innominate Vein, Open Approach, Diagnostic */
"05B30ZZ"  =  "1"  /* Excision of Right Innominate Vein, Open Approach */
"05B33ZX"  =  "1"  /* Excision of Right Innominate Vein, Perc Approach, Diagn */
"05B33ZZ"  =  "1"  /* Excision of Right Innominate Vein, Percutaneous Approach */
"05B34ZX"  =  "1"  /* Excision of Right Innominate Vein, Perc Endo Approach, Diagn */
"05B34ZZ"  =  "1"  /* Excision of Right Innominate Vein, Perc Endo Approach */
"05B40ZX"  =  "1"  /* Excision of Left Innominate Vein, Open Approach, Diagnostic */
"05B40ZZ"  =  "1"  /* Excision of Left Innominate Vein, Open Approach */
"05B43ZX"  =  "1"  /* Excision of Left Innominate Vein, Perc Approach, Diagn */
"05B43ZZ"  =  "1"  /* Excision of Left Innominate Vein, Percutaneous Approach */
"05B44ZX"  =  "1"  /* Excision of Left Innominate Vein, Perc Endo Approach, Diagn */
"05B44ZZ"  =  "1"  /* Excision of Left Innominate Vein, Perc Endo Approach */
"05B50ZX"  =  "1"  /* Excision of Right Subclavian Vein, Open Approach, Diagnostic */
"05B50ZZ"  =  "1"  /* Excision of Right Subclavian Vein, Open Approach */
"05B53ZX"  =  "1"  /* Excision of Right Subclavian Vein, Perc Approach, Diagn */
"05B53ZZ"  =  "1"  /* Excision of Right Subclavian Vein, Percutaneous Approach */
"05B54ZX"  =  "1"  /* Excision of Right Subclavian Vein, Perc Endo Approach, Diagn */
"05B54ZZ"  =  "1"  /* Excision of Right Subclavian Vein, Perc Endo Approach */
"05B60ZX"  =  "1"  /* Excision of Left Subclavian Vein, Open Approach, Diagnostic */
"05B60ZZ"  =  "1"  /* Excision of Left Subclavian Vein, Open Approach */
"05B63ZX"  =  "1"  /* Excision of Left Subclavian Vein, Perc Approach, Diagn */
"05B63ZZ"  =  "1"  /* Excision of Left Subclavian Vein, Percutaneous Approach */
"05B64ZX"  =  "1"  /* Excision of Left Subclavian Vein, Perc Endo Approach, Diagn */
"05B64ZZ"  =  "1"  /* Excision of Left Subclavian Vein, Perc Endo Approach */
"05B70ZX"  =  "1"  /* Excision of Right Axillary Vein, Open Approach, Diagnostic */
"05B70ZZ"  =  "1"  /* Excision of Right Axillary Vein, Open Approach */
"05B73ZX"  =  "1"  /* Excision of Right Axillary Vein, Perc Approach, Diagn */
"05B73ZZ"  =  "1"  /* Excision of Right Axillary Vein, Percutaneous Approach */
"05B74ZX"  =  "1"  /* Excision of Right Axillary Vein, Perc Endo Approach, Diagn */
"05B74ZZ"  =  "1"  /* Excision of Right Axillary Vein, Perc Endo Approach */
"05B80ZX"  =  "1"  /* Excision of Left Axillary Vein, Open Approach, Diagnostic */
"05B80ZZ"  =  "1"  /* Excision of Left Axillary Vein, Open Approach */
"05B83ZX"  =  "1"  /* Excision of Left Axillary Vein, Percutaneous Approach, Diagn */
"05B83ZZ"  =  "1"  /* Excision of Left Axillary Vein, Percutaneous Approach */
"05B84ZX"  =  "1"  /* Excision of Left Axillary Vein, Perc Endo Approach, Diagn */
"05B84ZZ"  =  "1"  /* Excision of Left Axillary Vein, Perc Endo Approach */
"05B90ZX"  =  "1"  /* Excision of Right Brachial Vein, Open Approach, Diagnostic */
"05B90ZZ"  =  "1"  /* Excision of Right Brachial Vein, Open Approach */
"05B93ZX"  =  "1"  /* Excision of Right Brachial Vein, Perc Approach, Diagn */
"05B93ZZ"  =  "1"  /* Excision of Right Brachial Vein, Percutaneous Approach */
"05B94ZX"  =  "1"  /* Excision of Right Brachial Vein, Perc Endo Approach, Diagn */
"05B94ZZ"  =  "1"  /* Excision of Right Brachial Vein, Perc Endo Approach */
"05BA0ZX"  =  "1"  /* Excision of Left Brachial Vein, Open Approach, Diagnostic */
"05BA0ZZ"  =  "1"  /* Excision of Left Brachial Vein, Open Approach */
"05BA3ZX"  =  "1"  /* Excision of Left Brachial Vein, Percutaneous Approach, Diagn */
"05BA3ZZ"  =  "1"  /* Excision of Left Brachial Vein, Percutaneous Approach */
"05BA4ZX"  =  "1"  /* Excision of Left Brachial Vein, Perc Endo Approach, Diagn */
"05BA4ZZ"  =  "1"  /* Excision of Left Brachial Vein, Perc Endo Approach */
"05BB0ZX"  =  "1"  /* Excision of Right Basilic Vein, Open Approach, Diagnostic */
"05BB0ZZ"  =  "1"  /* Excision of Right Basilic Vein, Open Approach */
"05BB3ZX"  =  "1"  /* Excision of Right Basilic Vein, Percutaneous Approach, Diagn */
"05BB3ZZ"  =  "1"  /* Excision of Right Basilic Vein, Percutaneous Approach */
"05BB4ZX"  =  "1"  /* Excision of Right Basilic Vein, Perc Endo Approach, Diagn */
"05BB4ZZ"  =  "1"  /* Excision of Right Basilic Vein, Perc Endo Approach */
"05BC0ZX"  =  "1"  /* Excision of Left Basilic Vein, Open Approach, Diagnostic */
"05BC0ZZ"  =  "1"  /* Excision of Left Basilic Vein, Open Approach */
"05BC3ZX"  =  "1"  /* Excision of Left Basilic Vein, Percutaneous Approach, Diagn */
"05BC3ZZ"  =  "1"  /* Excision of Left Basilic Vein, Percutaneous Approach */
"05BC4ZX"  =  "1"  /* Excision of Left Basilic Vein, Perc Endo Approach, Diagn */
"05BC4ZZ"  =  "1"  /* Excision of Left Basilic Vein, Perc Endo Approach */
"05BD0ZX"  =  "1"  /* Excision of Right Cephalic Vein, Open Approach, Diagnostic */
"05BD0ZZ"  =  "1"  /* Excision of Right Cephalic Vein, Open Approach */
"05BD3ZX"  =  "1"  /* Excision of Right Cephalic Vein, Perc Approach, Diagn */
"05BD3ZZ"  =  "1"  /* Excision of Right Cephalic Vein, Percutaneous Approach */
"05BD4ZX"  =  "1"  /* Excision of Right Cephalic Vein, Perc Endo Approach, Diagn */
"05BD4ZZ"  =  "1"  /* Excision of Right Cephalic Vein, Perc Endo Approach */
"05BF0ZX"  =  "1"  /* Excision of Left Cephalic Vein, Open Approach, Diagnostic */
"05BF0ZZ"  =  "1"  /* Excision of Left Cephalic Vein, Open Approach */
"05BF3ZX"  =  "1"  /* Excision of Left Cephalic Vein, Percutaneous Approach, Diagn */
"05BF3ZZ"  =  "1"  /* Excision of Left Cephalic Vein, Percutaneous Approach */
"05BF4ZX"  =  "1"  /* Excision of Left Cephalic Vein, Perc Endo Approach, Diagn */
"05BF4ZZ"  =  "1"  /* Excision of Left Cephalic Vein, Perc Endo Approach */
"05BG0ZX"  =  "1"  /* Excision of Right Hand Vein, Open Approach, Diagnostic */
"05BG0ZZ"  =  "1"  /* Excision of Right Hand Vein, Open Approach */
"05BG3ZX"  =  "1"  /* Excision of Right Hand Vein, Percutaneous Approach, Diagn */
"05BG3ZZ"  =  "1"  /* Excision of Right Hand Vein, Percutaneous Approach */
"05BG4ZX"  =  "1"  /* Excision of Right Hand Vein, Perc Endo Approach, Diagn */
"05BG4ZZ"  =  "1"  /* Excision of Right Hand Vein, Perc Endo Approach */
"05BH0ZX"  =  "1"  /* Excision of Left Hand Vein, Open Approach, Diagnostic */
"05BH0ZZ"  =  "1"  /* Excision of Left Hand Vein, Open Approach */
"05BH3ZX"  =  "1"  /* Excision of Left Hand Vein, Percutaneous Approach, Diagn */
"05BH3ZZ"  =  "1"  /* Excision of Left Hand Vein, Percutaneous Approach */
"05BH4ZX"  =  "1"  /* Excision of Left Hand Vein, Perc Endo Approach, Diagn */
"05BH4ZZ"  =  "1"  /* Excision of Left Hand Vein, Percutaneous Endoscopic Approach */
"05BL0ZX"  =  "1"  /* Excision of Intracranial Vein, Open Approach, Diagnostic */
"05BL0ZZ"  =  "1"  /* Excision of Intracranial Vein, Open Approach */
"05BL3ZX"  =  "1"  /* Excision of Intracranial Vein, Percutaneous Approach, Diagn */
"05BL3ZZ"  =  "1"  /* Excision of Intracranial Vein, Percutaneous Approach */
"05BL4ZX"  =  "1"  /* Excision of Intracranial Vein, Perc Endo Approach, Diagn */
"05BL4ZZ"  =  "1"  /* Excision of Intracranial Vein, Perc Endo Approach */
"05BM0ZX"  =  "1"  /* Excision of R Int Jugular Vein, Open Approach, Diagn */
"05BM0ZZ"  =  "1"  /* Excision of Right Internal Jugular Vein, Open Approach */
"05BM3ZX"  =  "1"  /* Excision of R Int Jugular Vein, Perc Approach, Diagn */
"05BM3ZZ"  =  "1"  /* Excision of Right Internal Jugular Vein, Perc Approach */
"05BM4ZX"  =  "1"  /* Excision of R Int Jugular Vein, Perc Endo Approach, Diagn */
"05BM4ZZ"  =  "1"  /* Excision of Right Internal Jugular Vein, Perc Endo Approach */
"05BN0ZX"  =  "1"  /* Excision of Left Internal Jugular Vein, Open Approach, Diagn */
"05BN0ZZ"  =  "1"  /* Excision of Left Internal Jugular Vein, Open Approach */
"05BN3ZX"  =  "1"  /* Excision of Left Internal Jugular Vein, Perc Approach, Diagn */
"05BN3ZZ"  =  "1"  /* Excision of Left Internal Jugular Vein, Perc Approach */
"05BN4ZX"  =  "1"  /* Excision of L Int Jugular Vein, Perc Endo Approach, Diagn */
"05BN4ZZ"  =  "1"  /* Excision of Left Internal Jugular Vein, Perc Endo Approach */
"05BP0ZX"  =  "1"  /* Excision of R Ext Jugular Vein, Open Approach, Diagn */
"05BP0ZZ"  =  "1"  /* Excision of Right External Jugular Vein, Open Approach */
"05BP3ZX"  =  "1"  /* Excision of R Ext Jugular Vein, Perc Approach, Diagn */
"05BP3ZZ"  =  "1"  /* Excision of Right External Jugular Vein, Perc Approach */
"05BP4ZX"  =  "1"  /* Excision of R Ext Jugular Vein, Perc Endo Approach, Diagn */
"05BP4ZZ"  =  "1"  /* Excision of Right External Jugular Vein, Perc Endo Approach */
"05BQ0ZX"  =  "1"  /* Excision of Left External Jugular Vein, Open Approach, Diagn */
"05BQ0ZZ"  =  "1"  /* Excision of Left External Jugular Vein, Open Approach */
"05BQ3ZX"  =  "1"  /* Excision of Left External Jugular Vein, Perc Approach, Diagn */
"05BQ3ZZ"  =  "1"  /* Excision of Left External Jugular Vein, Perc Approach */
"05BQ4ZX"  =  "1"  /* Excision of L Ext Jugular Vein, Perc Endo Approach, Diagn */
"05BQ4ZZ"  =  "1"  /* Excision of Left External Jugular Vein, Perc Endo Approach */
"05BR0ZX"  =  "1"  /* Excision of Right Vertebral Vein, Open Approach, Diagnostic */
"05BR0ZZ"  =  "1"  /* Excision of Right Vertebral Vein, Open Approach */
"05BR3ZX"  =  "1"  /* Excision of Right Vertebral Vein, Perc Approach, Diagn */
"05BR3ZZ"  =  "1"  /* Excision of Right Vertebral Vein, Percutaneous Approach */
"05BR4ZX"  =  "1"  /* Excision of Right Vertebral Vein, Perc Endo Approach, Diagn */
"05BR4ZZ"  =  "1"  /* Excision of Right Vertebral Vein, Perc Endo Approach */
"05BS0ZX"  =  "1"  /* Excision of Left Vertebral Vein, Open Approach, Diagnostic */
"05BS0ZZ"  =  "1"  /* Excision of Left Vertebral Vein, Open Approach */
"05BS3ZX"  =  "1"  /* Excision of Left Vertebral Vein, Perc Approach, Diagn */
"05BS3ZZ"  =  "1"  /* Excision of Left Vertebral Vein, Percutaneous Approach */
"05BS4ZX"  =  "1"  /* Excision of Left Vertebral Vein, Perc Endo Approach, Diagn */
"05BS4ZZ"  =  "1"  /* Excision of Left Vertebral Vein, Perc Endo Approach */
"05BT0ZX"  =  "1"  /* Excision of Right Face Vein, Open Approach, Diagnostic */
"05BT0ZZ"  =  "1"  /* Excision of Right Face Vein, Open Approach */
"05BT3ZX"  =  "1"  /* Excision of Right Face Vein, Percutaneous Approach, Diagn */
"05BT3ZZ"  =  "1"  /* Excision of Right Face Vein, Percutaneous Approach */
"05BT4ZX"  =  "1"  /* Excision of Right Face Vein, Perc Endo Approach, Diagn */
"05BT4ZZ"  =  "1"  /* Excision of Right Face Vein, Perc Endo Approach */
"05BV0ZX"  =  "1"  /* Excision of Left Face Vein, Open Approach, Diagnostic */
"05BV0ZZ"  =  "1"  /* Excision of Left Face Vein, Open Approach */
"05BV3ZX"  =  "1"  /* Excision of Left Face Vein, Percutaneous Approach, Diagn */
"05BV3ZZ"  =  "1"  /* Excision of Left Face Vein, Percutaneous Approach */
"05BV4ZX"  =  "1"  /* Excision of Left Face Vein, Perc Endo Approach, Diagn */
"05BV4ZZ"  =  "1"  /* Excision of Left Face Vein, Percutaneous Endoscopic Approach */
"05BY0ZX"  =  "1"  /* Excision of Upper Vein, Open Approach, Diagnostic */
"05BY0ZZ"  =  "1"  /* Excision of Upper Vein, Open Approach */
"05BY3ZX"  =  "1"  /* Excision of Upper Vein, Percutaneous Approach, Diagnostic */
"05BY3ZZ"  =  "1"  /* Excision of Upper Vein, Percutaneous Approach */
"05BY4ZX"  =  "1"  /* Excision of Upper Vein, Perc Endo Approach, Diagn */
"05BY4ZZ"  =  "1"  /* Excision of Upper Vein, Percutaneous Endoscopic Approach */
"05C00ZZ"  =  "1"  /* Extirpation of Matter from Azygos Vein, Open Approach */
"05C03ZZ"  =  "1"  /* Extirpation of Matter from Azygos Vein, Perc Approach */
"05C04ZZ"  =  "1"  /* Extirpation of Matter from Azygos Vein, Perc Endo Approach */
"05C10ZZ"  =  "1"  /* Extirpation of Matter from Hemiazygos Vein, Open Approach */
"05C13ZZ"  =  "1"  /* Extirpation of Matter from Hemiazygos Vein, Perc Approach */
"05C14ZZ"  =  "1"  /* Extirpate of Matter from Hemiazygos Vein, Perc Endo Approach */
"05C30ZZ"  =  "1"  /* Extirpation of Matter from R Innom Vein, Open Approach */
"05C33ZZ"  =  "1"  /* Extirpation of Matter from R Innom Vein, Perc Approach */
"05C34ZZ"  =  "1"  /* Extirpation of Matter from R Innom Vein, Perc Endo Approach */
"05C40ZZ"  =  "1"  /* Extirpation of Matter from L Innom Vein, Open Approach */
"05C43ZZ"  =  "1"  /* Extirpation of Matter from L Innom Vein, Perc Approach */
"05C44ZZ"  =  "1"  /* Extirpation of Matter from L Innom Vein, Perc Endo Approach */
"05C50ZZ"  =  "1"  /* Extirpation of Matter from R Subclav Vein, Open Approach */
"05C53ZZ"  =  "1"  /* Extirpation of Matter from R Subclav Vein, Perc Approach */
"05C54ZZ"  =  "1"  /* Extirpate of Matter from R Subclav Vein, Perc Endo Approach */
"05C60ZZ"  =  "1"  /* Extirpation of Matter from L Subclav Vein, Open Approach */
"05C63ZZ"  =  "1"  /* Extirpation of Matter from L Subclav Vein, Perc Approach */
"05C64ZZ"  =  "1"  /* Extirpate of Matter from L Subclav Vein, Perc Endo Approach */
"05C70ZZ"  =  "1"  /* Extirpation of Matter from R Axilla Vein, Open Approach */
"05C73ZZ"  =  "1"  /* Extirpation of Matter from R Axilla Vein, Perc Approach */
"05C74ZZ"  =  "1"  /* Extirpation of Matter from R Axilla Vein, Perc Endo Approach */
"05C80ZZ"  =  "1"  /* Extirpation of Matter from Left Axillary Vein, Open Approach */
"05C83ZZ"  =  "1"  /* Extirpation of Matter from Left Axillary Vein, Perc Approach */
"05C84ZZ"  =  "1"  /* Extirpation of Matter from L Axilla Vein, Perc Endo Approach */
"05C90ZZ"  =  "1"  /* Extirpation of Matter from R Brach Vein, Open Approach */
"05C93ZZ"  =  "1"  /* Extirpation of Matter from R Brach Vein, Perc Approach */
"05C94ZZ"  =  "1"  /* Extirpation of Matter from R Brach Vein, Perc Endo Approach */
"05CA0ZZ"  =  "1"  /* Extirpation of Matter from Left Brachial Vein, Open Approach */
"05CA3ZZ"  =  "1"  /* Extirpation of Matter from Left Brachial Vein, Perc Approach */
"05CA4ZZ"  =  "1"  /* Extirpation of Matter from L Brach Vein, Perc Endo Approach */
"05CB0ZZ"  =  "1"  /* Extirpation of Matter from Right Basilic Vein, Open Approach */
"05CB3ZZ"  =  "1"  /* Extirpation of Matter from Right Basilic Vein, Perc Approach */
"05CB4ZZ"  =  "1"  /* Extirpate of Matter from R Basilic Vein, Perc Endo Approach */
"05CC0ZZ"  =  "1"  /* Extirpation of Matter from Left Basilic Vein, Open Approach */
"05CC3ZZ"  =  "1"  /* Extirpation of Matter from Left Basilic Vein, Perc Approach */
"05CC4ZZ"  =  "1"  /* Extirpate of Matter from L Basilic Vein, Perc Endo Approach */
"05CD0ZZ"  =  "1"  /* Extirpation of Matter from R Cephalic Vein, Open Approach */
"05CD3ZZ"  =  "1"  /* Extirpation of Matter from R Cephalic Vein, Perc Approach */
"05CD4ZZ"  =  "1"  /* Extirpate of Matter from R Cephalic Vein, Perc Endo Approach */
"05CF0ZZ"  =  "1"  /* Extirpation of Matter from Left Cephalic Vein, Open Approach */
"05CF3ZZ"  =  "1"  /* Extirpation of Matter from Left Cephalic Vein, Perc Approach */
"05CF4ZZ"  =  "1"  /* Extirpate of Matter from L Cephalic Vein, Perc Endo Approach */
"05CG0ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Vein, Open Approach */
"05CG3ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Vein, Perc Approach */
"05CG4ZZ"  =  "1"  /* Extirpation of Matter from R Hand Vein, Perc Endo Approach */
"05CH0ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Vein, Open Approach */
"05CH3ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Vein, Perc Approach */
"05CH4ZZ"  =  "1"  /* Extirpation of Matter from L Hand Vein, Perc Endo Approach */
"05CL0ZZ"  =  "1"  /* Extirpation of Matter from Intracranial Vein, Open Approach */
"05CL3ZZ"  =  "1"  /* Extirpation of Matter from Intracranial Vein, Perc Approach */
"05CL4ZZ"  =  "1"  /* Extirpate of Matter from Intracran Vein, Perc Endo Approach */
"05CM0ZZ"  =  "1"  /* Extirpation of Matter from R Int Jugular Vein, Open Approach */
"05CM3ZZ"  =  "1"  /* Extirpation of Matter from R Int Jugular Vein, Perc Approach */
"05CM4ZZ"  =  "1"  /* Extirpate matter from R Int Jugular Vein, Perc Endo */
"05CN0ZZ"  =  "1"  /* Extirpation of Matter from L Int Jugular Vein, Open Approach */
"05CN3ZZ"  =  "1"  /* Extirpation of Matter from L Int Jugular Vein, Perc Approach */
"05CN4ZZ"  =  "1"  /* Extirpate matter from L Int Jugular Vein, Perc Endo */
"05CP0ZZ"  =  "1"  /* Extirpation of Matter from R Ext Jugular Vein, Open Approach */
"05CP3ZZ"  =  "1"  /* Extirpation of Matter from R Ext Jugular Vein, Perc Approach */
"05CP4ZZ"  =  "1"  /* Extirpate matter from R Ext Jugular Vein, Perc Endo */
"05CQ0ZZ"  =  "1"  /* Extirpation of Matter from L Ext Jugular Vein, Open Approach */
"05CQ3ZZ"  =  "1"  /* Extirpation of Matter from L Ext Jugular Vein, Perc Approach */
"05CQ4ZZ"  =  "1"  /* Extirpate matter from L Ext Jugular Vein, Perc Endo */
"05CR0ZZ"  =  "1"  /* Extirpation of Matter from R Verteb Vein, Open Approach */
"05CR3ZZ"  =  "1"  /* Extirpation of Matter from R Verteb Vein, Perc Approach */
"05CR4ZZ"  =  "1"  /* Extirpation of Matter from R Verteb Vein, Perc Endo Approach */
"05CS0ZZ"  =  "1"  /* Extirpation of Matter from L Verteb Vein, Open Approach */
"05CS3ZZ"  =  "1"  /* Extirpation of Matter from L Verteb Vein, Perc Approach */
"05CS4ZZ"  =  "1"  /* Extirpation of Matter from L Verteb Vein, Perc Endo Approach */
"05CT0ZZ"  =  "1"  /* Extirpation of Matter from Right Face Vein, Open Approach */
"05CT3ZZ"  =  "1"  /* Extirpation of Matter from Right Face Vein, Perc Approach */
"05CT4ZZ"  =  "1"  /* Extirpation of Matter from R Face Vein, Perc Endo Approach */
"05CV0ZZ"  =  "1"  /* Extirpation of Matter from Left Face Vein, Open Approach */
"05CV3ZZ"  =  "1"  /* Extirpation of Matter from Left Face Vein, Perc Approach */
"05CV4ZZ"  =  "1"  /* Extirpation of Matter from L Face Vein, Perc Endo Approach */
"05CY0ZZ"  =  "1"  /* Extirpation of Matter from Upper Vein, Open Approach */
"05CY3ZZ"  =  "1"  /* Extirpation of Matter from Upper Vein, Percutaneous Approach */
"05CY4ZZ"  =  "1"  /* Extirpation of Matter from Upper Vein, Perc Endo Approach */
"05D90ZZ"  =  "1"  /* Extraction of Right Brachial Vein, Open Approach */
"05D93ZZ"  =  "1"  /* Extraction of Right Brachial Vein, Percutaneous Approach */
"05DA0ZZ"  =  "1"  /* Extraction of Left Brachial Vein, Open Approach */
"05DA3ZZ"  =  "1"  /* Extraction of Left Brachial Vein, Percutaneous Approach */
"05DB0ZZ"  =  "1"  /* Extraction of Right Basilic Vein, Open Approach */
"05DB3ZZ"  =  "1"  /* Extraction of Right Basilic Vein, Percutaneous Approach */
"05DC0ZZ"  =  "1"  /* Extraction of Left Basilic Vein, Open Approach */
"05DC3ZZ"  =  "1"  /* Extraction of Left Basilic Vein, Percutaneous Approach */
"05DD0ZZ"  =  "1"  /* Extraction of Right Cephalic Vein, Open Approach */
"05DD3ZZ"  =  "1"  /* Extraction of Right Cephalic Vein, Percutaneous Approach */
"05DF0ZZ"  =  "1"  /* Extraction of Left Cephalic Vein, Open Approach */
"05DF3ZZ"  =  "1"  /* Extraction of Left Cephalic Vein, Percutaneous Approach */
"05DG0ZZ"  =  "1"  /* Extraction of Right Hand Vein, Open Approach */
"05DG3ZZ"  =  "1"  /* Extraction of Right Hand Vein, Percutaneous Approach */
"05DH0ZZ"  =  "1"  /* Extraction of Left Hand Vein, Open Approach */
"05DH3ZZ"  =  "1"  /* Extraction of Left Hand Vein, Percutaneous Approach */
"05DY0ZZ"  =  "1"  /* Extraction of Upper Vein, Open Approach */
"05DY3ZZ"  =  "1"  /* Extraction of Upper Vein, Percutaneous Approach */
"05H00DZ"  =  "1"  /* Insertion of Intralum Dev into Azygos Vein, Open Approach */
"05H03DZ"  =  "1"  /* Insertion of Intralum Dev into Azygos Vein, Perc Approach */
"05H04DZ"  =  "1"  /* Insert of Intralum Dev into Azygos Vein, Perc Endo Approach */
"05H10DZ"  =  "1"  /* Insert of Intralum Dev into Hemiazygos Vein, Open Approach */
"05H13DZ"  =  "1"  /* Insert of Intralum Dev into Hemiazygos Vein, Perc Approach */
"05H14DZ"  =  "1"  /* Insert Intralum Dev in Hemiazygos Vein, Perc Endo */
"05H30DZ"  =  "1"  /* Insertion of Intralum Dev into R Innom Vein, Open Approach */
"05H33DZ"  =  "1"  /* Insertion of Intralum Dev into R Innom Vein, Perc Approach */
"05H34DZ"  =  "1"  /* Insert of Intralum Dev into R Innom Vein, Perc Endo Approach */
"05H40DZ"  =  "1"  /* Insertion of Intralum Dev into L Innom Vein, Open Approach */
"05H43DZ"  =  "1"  /* Insertion of Intralum Dev into L Innom Vein, Perc Approach */
"05H44DZ"  =  "1"  /* Insert of Intralum Dev into L Innom Vein, Perc Endo Approach */
"05H50DZ"  =  "1"  /* Insertion of Intralum Dev into R Subclav Vein, Open Approach */
"05H53DZ"  =  "1"  /* Insertion of Intralum Dev into R Subclav Vein, Perc Approach */
"05H54DZ"  =  "1"  /* Insert Intralum Dev in R Subclav Vein, Perc Endo */
"05H60DZ"  =  "1"  /* Insertion of Intralum Dev into L Subclav Vein, Open Approach */
"05H63DZ"  =  "1"  /* Insertion of Intralum Dev into L Subclav Vein, Perc Approach */
"05H64DZ"  =  "1"  /* Insert Intralum Dev in L Subclav Vein, Perc Endo */
"05H70DZ"  =  "1"  /* Insertion of Intralum Dev into R Axilla Vein, Open Approach */
"05H73DZ"  =  "1"  /* Insertion of Intralum Dev into R Axilla Vein, Perc Approach */
"05H74DZ"  =  "1"  /* Insert Intralum Dev in R Axilla Vein, Perc Endo */
"05H80DZ"  =  "1"  /* Insertion of Intralum Dev into L Axilla Vein, Open Approach */
"05H83DZ"  =  "1"  /* Insertion of Intralum Dev into L Axilla Vein, Perc Approach */
"05H84DZ"  =  "1"  /* Insert Intralum Dev in L Axilla Vein, Perc Endo */
"05H90DZ"  =  "1"  /* Insertion of Intralum Dev into R Brach Vein, Open Approach */
"05H93DZ"  =  "1"  /* Insertion of Intralum Dev into R Brach Vein, Perc Approach */
"05H94DZ"  =  "1"  /* Insert of Intralum Dev into R Brach Vein, Perc Endo Approach */
"05HA0DZ"  =  "1"  /* Insertion of Intralum Dev into L Brach Vein, Open Approach */
"05HA3DZ"  =  "1"  /* Insertion of Intralum Dev into L Brach Vein, Perc Approach */
"05HA4DZ"  =  "1"  /* Insert of Intralum Dev into L Brach Vein, Perc Endo Approach */
"05HB0DZ"  =  "1"  /* Insertion of Intralum Dev into R Basilic Vein, Open Approach */
"05HB3DZ"  =  "1"  /* Insertion of Intralum Dev into R Basilic Vein, Perc Approach */
"05HB4DZ"  =  "1"  /* Insert Intralum Dev in R Basilic Vein, Perc Endo */
"05HC0DZ"  =  "1"  /* Insertion of Intralum Dev into L Basilic Vein, Open Approach */
"05HC3DZ"  =  "1"  /* Insertion of Intralum Dev into L Basilic Vein, Perc Approach */
"05HC4DZ"  =  "1"  /* Insert Intralum Dev in L Basilic Vein, Perc Endo */
"05HD0DZ"  =  "1"  /* Insert of Intralum Dev into R Cephalic Vein, Open Approach */
"05HD3DZ"  =  "1"  /* Insert of Intralum Dev into R Cephalic Vein, Perc Approach */
"05HD4DZ"  =  "1"  /* Insert Intralum Dev in R Cephalic Vein, Perc Endo */
"05HF0DZ"  =  "1"  /* Insert of Intralum Dev into L Cephalic Vein, Open Approach */
"05HF3DZ"  =  "1"  /* Insert of Intralum Dev into L Cephalic Vein, Perc Approach */
"05HF4DZ"  =  "1"  /* Insert Intralum Dev in L Cephalic Vein, Perc Endo */
"05HG0DZ"  =  "1"  /* Insertion of Intralum Dev into R Hand Vein, Open Approach */
"05HG3DZ"  =  "1"  /* Insertion of Intralum Dev into R Hand Vein, Perc Approach */
"05HG4DZ"  =  "1"  /* Insert of Intralum Dev into R Hand Vein, Perc Endo Approach */
"05HH0DZ"  =  "1"  /* Insertion of Intralum Dev into L Hand Vein, Open Approach */
"05HH3DZ"  =  "1"  /* Insertion of Intralum Dev into L Hand Vein, Perc Approach */
"05HH4DZ"  =  "1"  /* Insert of Intralum Dev into L Hand Vein, Perc Endo Approach */
"05HL0DZ"  =  "1"  /* Insertion of Intralum Dev into Intracran Vein, Open Approach */
"05HL3DZ"  =  "1"  /* Insertion of Intralum Dev into Intracran Vein, Perc Approach */
"05HL4DZ"  =  "1"  /* Insert Intralum Dev in Intracran Vein, Perc Endo */
"05HM0DZ"  =  "1"  /* Insert Intralum Dev in R Int Jugular Vein, Open */
"05HM3DZ"  =  "1"  /* Insert Intralum Dev in R Int Jugular Vein, Perc */
"05HM4DZ"  =  "1"  /* Insert Intralum Dev in R Int Jugular Vein, Perc Endo */
"05HN0DZ"  =  "1"  /* Insert Intralum Dev in L Int Jugular Vein, Open */
"05HN3DZ"  =  "1"  /* Insert Intralum Dev in L Int Jugular Vein, Perc */
"05HN4DZ"  =  "1"  /* Insert Intralum Dev in L Int Jugular Vein, Perc Endo */
"05HP0DZ"  =  "1"  /* Insert Intralum Dev in R Ext Jugular Vein, Open */
"05HP3DZ"  =  "1"  /* Insert Intralum Dev in R Ext Jugular Vein, Perc */
"05HP4DZ"  =  "1"  /* Insert Intralum Dev in R Ext Jugular Vein, Perc Endo */
"05HQ0DZ"  =  "1"  /* Insert Intralum Dev in L Ext Jugular Vein, Open */
"05HQ3DZ"  =  "1"  /* Insert Intralum Dev in L Ext Jugular Vein, Perc */
"05HQ4DZ"  =  "1"  /* Insert Intralum Dev in L Ext Jugular Vein, Perc Endo */
"05HR0DZ"  =  "1"  /* Insertion of Intralum Dev into R Verteb Vein, Open Approach */
"05HR3DZ"  =  "1"  /* Insertion of Intralum Dev into R Verteb Vein, Perc Approach */
"05HR4DZ"  =  "1"  /* Insert Intralum Dev in R Verteb Vein, Perc Endo */
"05HS0DZ"  =  "1"  /* Insertion of Intralum Dev into L Verteb Vein, Open Approach */
"05HS3DZ"  =  "1"  /* Insertion of Intralum Dev into L Verteb Vein, Perc Approach */
"05HS4DZ"  =  "1"  /* Insert Intralum Dev in L Verteb Vein, Perc Endo */
"05HT0DZ"  =  "1"  /* Insertion of Intralum Dev into R Face Vein, Open Approach */
"05HT3DZ"  =  "1"  /* Insertion of Intralum Dev into R Face Vein, Perc Approach */
"05HT4DZ"  =  "1"  /* Insert of Intralum Dev into R Face Vein, Perc Endo Approach */
"05HV0DZ"  =  "1"  /* Insertion of Intralum Dev into L Face Vein, Open Approach */
"05HV3DZ"  =  "1"  /* Insertion of Intralum Dev into L Face Vein, Perc Approach */
"05HV4DZ"  =  "1"  /* Insert of Intralum Dev into L Face Vein, Perc Endo Approach */
"05HY02Z"  =  "1"  /* Insertion of Monitoring Device into Up Vein, Open Approach */
"05HY0DZ"  =  "1"  /* Insertion of Intraluminal Device into Up Vein, Open Approach */
"05HY32Z"  =  "1"  /* Insertion of Monitoring Device into Up Vein, Perc Approach */
"05HY3DZ"  =  "1"  /* Insertion of Intraluminal Device into Up Vein, Perc Approach */
"05HY42Z"  =  "1"  /* Insertion of Monitor Dev into Up Vein, Perc Endo Approach */
"05HY4DZ"  =  "1"  /* Insertion of Intralum Dev into Up Vein, Perc Endo Approach */
"05JY0ZZ"  =  "1"  /* Inspection of Upper Vein, Open Approach */
"05JY3ZZ"  =  "1"  /* Inspection of Upper Vein, Percutaneous Approach */
"05JY4ZZ"  =  "1"  /* Inspection of Upper Vein, Percutaneous Endoscopic Approach */
"05L00CZ"  =  "1"  /* Occlusion of Azygos Vein with Extralum Dev, Open Approach */
"05L00DZ"  =  "1"  /* Occlusion of Azygos Vein with Intralum Dev, Open Approach */
"05L00ZZ"  =  "1"  /* Occlusion of Azygos Vein, Open Approach */
"05L03CZ"  =  "1"  /* Occlusion of Azygos Vein with Extralum Dev, Perc Approach */
"05L03DZ"  =  "1"  /* Occlusion of Azygos Vein with Intralum Dev, Perc Approach */
"05L03ZZ"  =  "1"  /* Occlusion of Azygos Vein, Percutaneous Approach */
"05L04CZ"  =  "1"  /* Occlusion Azygos Vein w Extralum Dev, Perc Endo */
"05L04DZ"  =  "1"  /* Occlusion Azygos Vein w Intralum Dev, Perc Endo */
"05L04ZZ"  =  "1"  /* Occlusion of Azygos Vein, Percutaneous Endoscopic Approach */
"05L10CZ"  =  "1"  /* Occlusion Hemiazygos Vein w Extralum Dev, Open */
"05L10DZ"  =  "1"  /* Occlusion Hemiazygos Vein w Intralum Dev, Open */
"05L10ZZ"  =  "1"  /* Occlusion of Hemiazygos Vein, Open Approach */
"05L13CZ"  =  "1"  /* Occlusion Hemiazygos Vein w Extralum Dev, Perc */
"05L13DZ"  =  "1"  /* Occlusion Hemiazygos Vein w Intralum Dev, Perc */
"05L13ZZ"  =  "1"  /* Occlusion of Hemiazygos Vein, Percutaneous Approach */
"05L14CZ"  =  "1"  /* Occlusion Hemiazygos Vein w Extralum Dev, Perc Endo */
"05L14DZ"  =  "1"  /* Occlusion Hemiazygos Vein w Intralum Dev, Perc Endo */
"05L14ZZ"  =  "1"  /* Occlusion of Hemiazygos Vein, Perc Endo Approach */
"05L30CZ"  =  "1"  /* Occlusion of R Innom Vein with Extralum Dev, Open Approach */
"05L30DZ"  =  "1"  /* Occlusion of R Innom Vein with Intralum Dev, Open Approach */
"05L30ZZ"  =  "1"  /* Occlusion of Right Innominate Vein, Open Approach */
"05L33CZ"  =  "1"  /* Occlusion of R Innom Vein with Extralum Dev, Perc Approach */
"05L33DZ"  =  "1"  /* Occlusion of R Innom Vein with Intralum Dev, Perc Approach */
"05L33ZZ"  =  "1"  /* Occlusion of Right Innominate Vein, Percutaneous Approach */
"05L34CZ"  =  "1"  /* Occlusion R Innom Vein w Extralum Dev, Perc Endo */
"05L34DZ"  =  "1"  /* Occlusion R Innom Vein w Intralum Dev, Perc Endo */
"05L34ZZ"  =  "1"  /* Occlusion of Right Innominate Vein, Perc Endo Approach */
"05L40CZ"  =  "1"  /* Occlusion of L Innom Vein with Extralum Dev, Open Approach */
"05L40DZ"  =  "1"  /* Occlusion of L Innom Vein with Intralum Dev, Open Approach */
"05L40ZZ"  =  "1"  /* Occlusion of Left Innominate Vein, Open Approach */
"05L43CZ"  =  "1"  /* Occlusion of L Innom Vein with Extralum Dev, Perc Approach */
"05L43DZ"  =  "1"  /* Occlusion of L Innom Vein with Intralum Dev, Perc Approach */
"05L43ZZ"  =  "1"  /* Occlusion of Left Innominate Vein, Percutaneous Approach */
"05L44CZ"  =  "1"  /* Occlusion L Innom Vein w Extralum Dev, Perc Endo */
"05L44DZ"  =  "1"  /* Occlusion L Innom Vein w Intralum Dev, Perc Endo */
"05L44ZZ"  =  "1"  /* Occlusion of Left Innominate Vein, Perc Endo Approach */
"05L50CZ"  =  "1"  /* Occlusion of R Subclav Vein with Extralum Dev, Open Approach */
"05L50DZ"  =  "1"  /* Occlusion of R Subclav Vein with Intralum Dev, Open Approach */
"05L50ZZ"  =  "1"  /* Occlusion of Right Subclavian Vein, Open Approach */
"05L53CZ"  =  "1"  /* Occlusion of R Subclav Vein with Extralum Dev, Perc Approach */
"05L53DZ"  =  "1"  /* Occlusion of R Subclav Vein with Intralum Dev, Perc Approach */
"05L53ZZ"  =  "1"  /* Occlusion of Right Subclavian Vein, Percutaneous Approach */
"05L54CZ"  =  "1"  /* Occlusion R Subclav Vein w Extralum Dev, Perc Endo */
"05L54DZ"  =  "1"  /* Occlusion R Subclav Vein w Intralum Dev, Perc Endo */
"05L54ZZ"  =  "1"  /* Occlusion of Right Subclavian Vein, Perc Endo Approach */
"05L60CZ"  =  "1"  /* Occlusion of L Subclav Vein with Extralum Dev, Open Approach */
"05L60DZ"  =  "1"  /* Occlusion of L Subclav Vein with Intralum Dev, Open Approach */
"05L60ZZ"  =  "1"  /* Occlusion of Left Subclavian Vein, Open Approach */
"05L63CZ"  =  "1"  /* Occlusion of L Subclav Vein with Extralum Dev, Perc Approach */
"05L63DZ"  =  "1"  /* Occlusion of L Subclav Vein with Intralum Dev, Perc Approach */
"05L63ZZ"  =  "1"  /* Occlusion of Left Subclavian Vein, Percutaneous Approach */
"05L64CZ"  =  "1"  /* Occlusion L Subclav Vein w Extralum Dev, Perc Endo */
"05L64DZ"  =  "1"  /* Occlusion L Subclav Vein w Intralum Dev, Perc Endo */
"05L64ZZ"  =  "1"  /* Occlusion of Left Subclavian Vein, Perc Endo Approach */
"05L70CZ"  =  "1"  /* Occlusion of R Axilla Vein with Extralum Dev, Open Approach */
"05L70DZ"  =  "1"  /* Occlusion of R Axilla Vein with Intralum Dev, Open Approach */
"05L70ZZ"  =  "1"  /* Occlusion of Right Axillary Vein, Open Approach */
"05L73CZ"  =  "1"  /* Occlusion of R Axilla Vein with Extralum Dev, Perc Approach */
"05L73DZ"  =  "1"  /* Occlusion of R Axilla Vein with Intralum Dev, Perc Approach */
"05L73ZZ"  =  "1"  /* Occlusion of Right Axillary Vein, Percutaneous Approach */
"05L74CZ"  =  "1"  /* Occlusion R Axilla Vein w Extralum Dev, Perc Endo */
"05L74DZ"  =  "1"  /* Occlusion R Axilla Vein w Intralum Dev, Perc Endo */
"05L74ZZ"  =  "1"  /* Occlusion of Right Axillary Vein, Perc Endo Approach */
"05L80CZ"  =  "1"  /* Occlusion of L Axilla Vein with Extralum Dev, Open Approach */
"05L80DZ"  =  "1"  /* Occlusion of L Axilla Vein with Intralum Dev, Open Approach */
"05L80ZZ"  =  "1"  /* Occlusion of Left Axillary Vein, Open Approach */
"05L83CZ"  =  "1"  /* Occlusion of L Axilla Vein with Extralum Dev, Perc Approach */
"05L83DZ"  =  "1"  /* Occlusion of L Axilla Vein with Intralum Dev, Perc Approach */
"05L83ZZ"  =  "1"  /* Occlusion of Left Axillary Vein, Percutaneous Approach */
"05L84CZ"  =  "1"  /* Occlusion L Axilla Vein w Extralum Dev, Perc Endo */
"05L84DZ"  =  "1"  /* Occlusion L Axilla Vein w Intralum Dev, Perc Endo */
"05L84ZZ"  =  "1"  /* Occlusion of Left Axillary Vein, Perc Endo Approach */
"05L90CZ"  =  "1"  /* Occlusion of R Brach Vein with Extralum Dev, Open Approach */
"05L90DZ"  =  "1"  /* Occlusion of R Brach Vein with Intralum Dev, Open Approach */
"05L90ZZ"  =  "1"  /* Occlusion of Right Brachial Vein, Open Approach */
"05L93CZ"  =  "1"  /* Occlusion of R Brach Vein with Extralum Dev, Perc Approach */
"05L93DZ"  =  "1"  /* Occlusion of R Brach Vein with Intralum Dev, Perc Approach */
"05L93ZZ"  =  "1"  /* Occlusion of Right Brachial Vein, Percutaneous Approach */
"05L94CZ"  =  "1"  /* Occlusion R Brach Vein w Extralum Dev, Perc Endo */
"05L94DZ"  =  "1"  /* Occlusion R Brach Vein w Intralum Dev, Perc Endo */
"05L94ZZ"  =  "1"  /* Occlusion of Right Brachial Vein, Perc Endo Approach */
"05LA0CZ"  =  "1"  /* Occlusion of L Brach Vein with Extralum Dev, Open Approach */
"05LA0DZ"  =  "1"  /* Occlusion of L Brach Vein with Intralum Dev, Open Approach */
"05LA0ZZ"  =  "1"  /* Occlusion of Left Brachial Vein, Open Approach */
"05LA3CZ"  =  "1"  /* Occlusion of L Brach Vein with Extralum Dev, Perc Approach */
"05LA3DZ"  =  "1"  /* Occlusion of L Brach Vein with Intralum Dev, Perc Approach */
"05LA3ZZ"  =  "1"  /* Occlusion of Left Brachial Vein, Percutaneous Approach */
"05LA4CZ"  =  "1"  /* Occlusion L Brach Vein w Extralum Dev, Perc Endo */
"05LA4DZ"  =  "1"  /* Occlusion L Brach Vein w Intralum Dev, Perc Endo */
"05LA4ZZ"  =  "1"  /* Occlusion of Left Brachial Vein, Perc Endo Approach */
"05LB0CZ"  =  "1"  /* Occlusion of R Basilic Vein with Extralum Dev, Open Approach */
"05LB0DZ"  =  "1"  /* Occlusion of R Basilic Vein with Intralum Dev, Open Approach */
"05LB0ZZ"  =  "1"  /* Occlusion of Right Basilic Vein, Open Approach */
"05LB3CZ"  =  "1"  /* Occlusion of R Basilic Vein with Extralum Dev, Perc Approach */
"05LB3DZ"  =  "1"  /* Occlusion of R Basilic Vein with Intralum Dev, Perc Approach */
"05LB3ZZ"  =  "1"  /* Occlusion of Right Basilic Vein, Percutaneous Approach */
"05LB4CZ"  =  "1"  /* Occlusion R Basilic Vein w Extralum Dev, Perc Endo */
"05LB4DZ"  =  "1"  /* Occlusion R Basilic Vein w Intralum Dev, Perc Endo */
"05LB4ZZ"  =  "1"  /* Occlusion of Right Basilic Vein, Perc Endo Approach */
"05LC0CZ"  =  "1"  /* Occlusion of L Basilic Vein with Extralum Dev, Open Approach */
"05LC0DZ"  =  "1"  /* Occlusion of L Basilic Vein with Intralum Dev, Open Approach */
"05LC0ZZ"  =  "1"  /* Occlusion of Left Basilic Vein, Open Approach */
"05LC3CZ"  =  "1"  /* Occlusion of L Basilic Vein with Extralum Dev, Perc Approach */
"05LC3DZ"  =  "1"  /* Occlusion of L Basilic Vein with Intralum Dev, Perc Approach */
"05LC3ZZ"  =  "1"  /* Occlusion of Left Basilic Vein, Percutaneous Approach */
"05LC4CZ"  =  "1"  /* Occlusion L Basilic Vein w Extralum Dev, Perc Endo */
"05LC4DZ"  =  "1"  /* Occlusion L Basilic Vein w Intralum Dev, Perc Endo */
"05LC4ZZ"  =  "1"  /* Occlusion of Left Basilic Vein, Perc Endo Approach */
"05LD0CZ"  =  "1"  /* Occlusion R Cephalic Vein w Extralum Dev, Open */
"05LD0DZ"  =  "1"  /* Occlusion R Cephalic Vein w Intralum Dev, Open */
"05LD0ZZ"  =  "1"  /* Occlusion of Right Cephalic Vein, Open Approach */
"05LD3CZ"  =  "1"  /* Occlusion R Cephalic Vein w Extralum Dev, Perc */
"05LD3DZ"  =  "1"  /* Occlusion R Cephalic Vein w Intralum Dev, Perc */
"05LD3ZZ"  =  "1"  /* Occlusion of Right Cephalic Vein, Percutaneous Approach */
"05LD4CZ"  =  "1"  /* Occlusion R Cephalic Vein w Extralum Dev, Perc Endo */
"05LD4DZ"  =  "1"  /* Occlusion R Cephalic Vein w Intralum Dev, Perc Endo */
"05LD4ZZ"  =  "1"  /* Occlusion of Right Cephalic Vein, Perc Endo Approach */
"05LF0CZ"  =  "1"  /* Occlusion L Cephalic Vein w Extralum Dev, Open */
"05LF0DZ"  =  "1"  /* Occlusion L Cephalic Vein w Intralum Dev, Open */
"05LF0ZZ"  =  "1"  /* Occlusion of Left Cephalic Vein, Open Approach */
"05LF3CZ"  =  "1"  /* Occlusion L Cephalic Vein w Extralum Dev, Perc */
"05LF3DZ"  =  "1"  /* Occlusion L Cephalic Vein w Intralum Dev, Perc */
"05LF3ZZ"  =  "1"  /* Occlusion of Left Cephalic Vein, Percutaneous Approach */
"05LF4CZ"  =  "1"  /* Occlusion L Cephalic Vein w Extralum Dev, Perc Endo */
"05LF4DZ"  =  "1"  /* Occlusion L Cephalic Vein w Intralum Dev, Perc Endo */
"05LF4ZZ"  =  "1"  /* Occlusion of Left Cephalic Vein, Perc Endo Approach */
"05LG0CZ"  =  "1"  /* Occlusion of R Hand Vein with Extralum Dev, Open Approach */
"05LG0DZ"  =  "1"  /* Occlusion of R Hand Vein with Intralum Dev, Open Approach */
"05LG0ZZ"  =  "1"  /* Occlusion of Right Hand Vein, Open Approach */
"05LG3CZ"  =  "1"  /* Occlusion of R Hand Vein with Extralum Dev, Perc Approach */
"05LG3DZ"  =  "1"  /* Occlusion of R Hand Vein with Intralum Dev, Perc Approach */
"05LG3ZZ"  =  "1"  /* Occlusion of Right Hand Vein, Percutaneous Approach */
"05LG4CZ"  =  "1"  /* Occlusion R Hand Vein w Extralum Dev, Perc Endo */
"05LG4DZ"  =  "1"  /* Occlusion R Hand Vein w Intralum Dev, Perc Endo */
"05LG4ZZ"  =  "1"  /* Occlusion of Right Hand Vein, Perc Endo Approach */
"05LH0CZ"  =  "1"  /* Occlusion of Left Hand Vein with Extralum Dev, Open Approach */
"05LH0DZ"  =  "1"  /* Occlusion of Left Hand Vein with Intralum Dev, Open Approach */
"05LH0ZZ"  =  "1"  /* Occlusion of Left Hand Vein, Open Approach */
"05LH3CZ"  =  "1"  /* Occlusion of Left Hand Vein with Extralum Dev, Perc Approach */
"05LH3DZ"  =  "1"  /* Occlusion of Left Hand Vein with Intralum Dev, Perc Approach */
"05LH3ZZ"  =  "1"  /* Occlusion of Left Hand Vein, Percutaneous Approach */
"05LH4CZ"  =  "1"  /* Occlusion L Hand Vein w Extralum Dev, Perc Endo */
"05LH4DZ"  =  "1"  /* Occlusion L Hand Vein w Intralum Dev, Perc Endo */
"05LH4ZZ"  =  "1"  /* Occlusion of Left Hand Vein, Perc Endo Approach */
"05LL0CZ"  =  "1"  /* Occlusion of Intracran Vein with Extralum Dev, Open Approach */
"05LL0DZ"  =  "1"  /* Occlusion of Intracran Vein with Intralum Dev, Open Approach */
"05LL0ZZ"  =  "1"  /* Occlusion of Intracranial Vein, Open Approach */
"05LL3CZ"  =  "1"  /* Occlusion of Intracran Vein with Extralum Dev, Perc Approach */
"05LL3DZ"  =  "1"  /* Occlusion of Intracran Vein with Intralum Dev, Perc Approach */
"05LL3ZZ"  =  "1"  /* Occlusion of Intracranial Vein, Percutaneous Approach */
"05LL4CZ"  =  "1"  /* Occlusion Intracran Vein w Extralum Dev, Perc Endo */
"05LL4DZ"  =  "1"  /* Occlusion Intracran Vein w Intralum Dev, Perc Endo */
"05LL4ZZ"  =  "1"  /* Occlusion of Intracranial Vein, Perc Endo Approach */
"05LM0CZ"  =  "1"  /* Occlusion R Int Jugular Vein w Extralum Dev, Open */
"05LM0DZ"  =  "1"  /* Occlusion R Int Jugular Vein w Intralum Dev, Open */
"05LM0ZZ"  =  "1"  /* Occlusion of Right Internal Jugular Vein, Open Approach */
"05LM3CZ"  =  "1"  /* Occlusion R Int Jugular Vein w Extralum Dev, Perc */
"05LM3DZ"  =  "1"  /* Occlusion R Int Jugular Vein w Intralum Dev, Perc */
"05LM3ZZ"  =  "1"  /* Occlusion of Right Internal Jugular Vein, Perc Approach */
"05LM4CZ"  =  "1"  /* Occlusion R Int Jugular Vein w Extralum Dev, Perc Endo */
"05LM4DZ"  =  "1"  /* Occlusion R Int Jugular Vein w Intralum Dev, Perc Endo */
"05LM4ZZ"  =  "1"  /* Occlusion of Right Internal Jugular Vein, Perc Endo Approach */
"05LN0CZ"  =  "1"  /* Occlusion L Int Jugular Vein w Extralum Dev, Open */
"05LN0DZ"  =  "1"  /* Occlusion L Int Jugular Vein w Intralum Dev, Open */
"05LN0ZZ"  =  "1"  /* Occlusion of Left Internal Jugular Vein, Open Approach */
"05LN3CZ"  =  "1"  /* Occlusion L Int Jugular Vein w Extralum Dev, Perc */
"05LN3DZ"  =  "1"  /* Occlusion L Int Jugular Vein w Intralum Dev, Perc */
"05LN3ZZ"  =  "1"  /* Occlusion of Left Internal Jugular Vein, Perc Approach */
"05LN4CZ"  =  "1"  /* Occlusion L Int Jugular Vein w Extralum Dev, Perc Endo */
"05LN4DZ"  =  "1"  /* Occlusion L Int Jugular Vein w Intralum Dev, Perc Endo */
"05LN4ZZ"  =  "1"  /* Occlusion of Left Internal Jugular Vein, Perc Endo Approach */
"05LP0CZ"  =  "1"  /* Occlusion R Ext Jugular Vein w Extralum Dev, Open */
"05LP0DZ"  =  "1"  /* Occlusion R Ext Jugular Vein w Intralum Dev, Open */
"05LP0ZZ"  =  "1"  /* Occlusion of Right External Jugular Vein, Open Approach */
"05LP3CZ"  =  "1"  /* Occlusion R Ext Jugular Vein w Extralum Dev, Perc */
"05LP3DZ"  =  "1"  /* Occlusion R Ext Jugular Vein w Intralum Dev, Perc */
"05LP3ZZ"  =  "1"  /* Occlusion of Right External Jugular Vein, Perc Approach */
"05LP4CZ"  =  "1"  /* Occlusion R Ext Jugular Vein w Extralum Dev, Perc Endo */
"05LP4DZ"  =  "1"  /* Occlusion R Ext Jugular Vein w Intralum Dev, Perc Endo */
"05LP4ZZ"  =  "1"  /* Occlusion of Right External Jugular Vein, Perc Endo Approach */
"05LQ0CZ"  =  "1"  /* Occlusion L Ext Jugular Vein w Extralum Dev, Open */
"05LQ0DZ"  =  "1"  /* Occlusion L Ext Jugular Vein w Intralum Dev, Open */
"05LQ0ZZ"  =  "1"  /* Occlusion of Left External Jugular Vein, Open Approach */
"05LQ3CZ"  =  "1"  /* Occlusion L Ext Jugular Vein w Extralum Dev, Perc */
"05LQ3DZ"  =  "1"  /* Occlusion L Ext Jugular Vein w Intralum Dev, Perc */
"05LQ3ZZ"  =  "1"  /* Occlusion of Left External Jugular Vein, Perc Approach */
"05LQ4CZ"  =  "1"  /* Occlusion L Ext Jugular Vein w Extralum Dev, Perc Endo */
"05LQ4DZ"  =  "1"  /* Occlusion L Ext Jugular Vein w Intralum Dev, Perc Endo */
"05LQ4ZZ"  =  "1"  /* Occlusion of Left External Jugular Vein, Perc Endo Approach */
"05LR0CZ"  =  "1"  /* Occlusion of R Verteb Vein with Extralum Dev, Open Approach */
"05LR0DZ"  =  "1"  /* Occlusion of R Verteb Vein with Intralum Dev, Open Approach */
"05LR0ZZ"  =  "1"  /* Occlusion of Right Vertebral Vein, Open Approach */
"05LR3CZ"  =  "1"  /* Occlusion of R Verteb Vein with Extralum Dev, Perc Approach */
"05LR3DZ"  =  "1"  /* Occlusion of R Verteb Vein with Intralum Dev, Perc Approach */
"05LR3ZZ"  =  "1"  /* Occlusion of Right Vertebral Vein, Percutaneous Approach */
"05LR4CZ"  =  "1"  /* Occlusion R Verteb Vein w Extralum Dev, Perc Endo */
"05LR4DZ"  =  "1"  /* Occlusion R Verteb Vein w Intralum Dev, Perc Endo */
"05LR4ZZ"  =  "1"  /* Occlusion of Right Vertebral Vein, Perc Endo Approach */
"05LS0CZ"  =  "1"  /* Occlusion of L Verteb Vein with Extralum Dev, Open Approach */
"05LS0DZ"  =  "1"  /* Occlusion of L Verteb Vein with Intralum Dev, Open Approach */
"05LS0ZZ"  =  "1"  /* Occlusion of Left Vertebral Vein, Open Approach */
"05LS3CZ"  =  "1"  /* Occlusion of L Verteb Vein with Extralum Dev, Perc Approach */
"05LS3DZ"  =  "1"  /* Occlusion of L Verteb Vein with Intralum Dev, Perc Approach */
"05LS3ZZ"  =  "1"  /* Occlusion of Left Vertebral Vein, Percutaneous Approach */
"05LS4CZ"  =  "1"  /* Occlusion L Verteb Vein w Extralum Dev, Perc Endo */
"05LS4DZ"  =  "1"  /* Occlusion L Verteb Vein w Intralum Dev, Perc Endo */
"05LS4ZZ"  =  "1"  /* Occlusion of Left Vertebral Vein, Perc Endo Approach */
"05LT0CZ"  =  "1"  /* Occlusion of R Face Vein with Extralum Dev, Open Approach */
"05LT0DZ"  =  "1"  /* Occlusion of R Face Vein with Intralum Dev, Open Approach */
"05LT0ZZ"  =  "1"  /* Occlusion of Right Face Vein, Open Approach */
"05LT3CZ"  =  "1"  /* Occlusion of R Face Vein with Extralum Dev, Perc Approach */
"05LT3DZ"  =  "1"  /* Occlusion of R Face Vein with Intralum Dev, Perc Approach */
"05LT3ZZ"  =  "1"  /* Occlusion of Right Face Vein, Percutaneous Approach */
"05LT4CZ"  =  "1"  /* Occlusion R Face Vein w Extralum Dev, Perc Endo */
"05LT4DZ"  =  "1"  /* Occlusion R Face Vein w Intralum Dev, Perc Endo */
"05LT4ZZ"  =  "1"  /* Occlusion of Right Face Vein, Perc Endo Approach */
"05LV0CZ"  =  "1"  /* Occlusion of Left Face Vein with Extralum Dev, Open Approach */
"05LV0DZ"  =  "1"  /* Occlusion of Left Face Vein with Intralum Dev, Open Approach */
"05LV0ZZ"  =  "1"  /* Occlusion of Left Face Vein, Open Approach */
"05LV3CZ"  =  "1"  /* Occlusion of Left Face Vein with Extralum Dev, Perc Approach */
"05LV3DZ"  =  "1"  /* Occlusion of Left Face Vein with Intralum Dev, Perc Approach */
"05LV3ZZ"  =  "1"  /* Occlusion of Left Face Vein, Percutaneous Approach */
"05LV4CZ"  =  "1"  /* Occlusion L Face Vein w Extralum Dev, Perc Endo */
"05LV4DZ"  =  "1"  /* Occlusion L Face Vein w Intralum Dev, Perc Endo */
"05LV4ZZ"  =  "1"  /* Occlusion of Left Face Vein, Perc Endo Approach */
"05LY0CZ"  =  "1"  /* Occlusion of Upper Vein with Extralum Dev, Open Approach */
"05LY0DZ"  =  "1"  /* Occlusion of Upper Vein with Intralum Dev, Open Approach */
"05LY0ZZ"  =  "1"  /* Occlusion of Upper Vein, Open Approach */
"05LY3CZ"  =  "1"  /* Occlusion of Upper Vein with Extralum Dev, Perc Approach */
"05LY3DZ"  =  "1"  /* Occlusion of Upper Vein with Intralum Dev, Perc Approach */
"05LY3ZZ"  =  "1"  /* Occlusion of Upper Vein, Percutaneous Approach */
"05LY4CZ"  =  "1"  /* Occlusion of Up Vein with Extralum Dev, Perc Endo Approach */
"05LY4DZ"  =  "1"  /* Occlusion of Up Vein with Intralum Dev, Perc Endo Approach */
"05LY4ZZ"  =  "1"  /* Occlusion of Upper Vein, Percutaneous Endoscopic Approach */
"05N00ZZ"  =  "1"  /* Release Azygos Vein, Open Approach */
"05N03ZZ"  =  "1"  /* Release Azygos Vein, Percutaneous Approach */
"05N04ZZ"  =  "1"  /* Release Azygos Vein, Percutaneous Endoscopic Approach */
"05N10ZZ"  =  "1"  /* Release Hemiazygos Vein, Open Approach */
"05N13ZZ"  =  "1"  /* Release Hemiazygos Vein, Percutaneous Approach */
"05N14ZZ"  =  "1"  /* Release Hemiazygos Vein, Percutaneous Endoscopic Approach */
"05N30ZZ"  =  "1"  /* Release Right Innominate Vein, Open Approach */
"05N33ZZ"  =  "1"  /* Release Right Innominate Vein, Percutaneous Approach */
"05N34ZZ"  =  "1"  /* Release Right Innominate Vein, Perc Endo Approach */
"05N40ZZ"  =  "1"  /* Release Left Innominate Vein, Open Approach */
"05N43ZZ"  =  "1"  /* Release Left Innominate Vein, Percutaneous Approach */
"05N44ZZ"  =  "1"  /* Release Left Innominate Vein, Perc Endo Approach */
"05N50ZZ"  =  "1"  /* Release Right Subclavian Vein, Open Approach */
"05N53ZZ"  =  "1"  /* Release Right Subclavian Vein, Percutaneous Approach */
"05N54ZZ"  =  "1"  /* Release Right Subclavian Vein, Perc Endo Approach */
"05N60ZZ"  =  "1"  /* Release Left Subclavian Vein, Open Approach */
"05N63ZZ"  =  "1"  /* Release Left Subclavian Vein, Percutaneous Approach */
"05N64ZZ"  =  "1"  /* Release Left Subclavian Vein, Perc Endo Approach */
"05N70ZZ"  =  "1"  /* Release Right Axillary Vein, Open Approach */
"05N73ZZ"  =  "1"  /* Release Right Axillary Vein, Percutaneous Approach */
"05N74ZZ"  =  "1"  /* Release Right Axillary Vein, Perc Endo Approach */
"05N80ZZ"  =  "1"  /* Release Left Axillary Vein, Open Approach */
"05N83ZZ"  =  "1"  /* Release Left Axillary Vein, Percutaneous Approach */
"05N84ZZ"  =  "1"  /* Release Left Axillary Vein, Percutaneous Endoscopic Approach */
"05N90ZZ"  =  "1"  /* Release Right Brachial Vein, Open Approach */
"05N93ZZ"  =  "1"  /* Release Right Brachial Vein, Percutaneous Approach */
"05N94ZZ"  =  "1"  /* Release Right Brachial Vein, Perc Endo Approach */
"05NA0ZZ"  =  "1"  /* Release Left Brachial Vein, Open Approach */
"05NA3ZZ"  =  "1"  /* Release Left Brachial Vein, Percutaneous Approach */
"05NA4ZZ"  =  "1"  /* Release Left Brachial Vein, Percutaneous Endoscopic Approach */
"05NB0ZZ"  =  "1"  /* Release Right Basilic Vein, Open Approach */
"05NB3ZZ"  =  "1"  /* Release Right Basilic Vein, Percutaneous Approach */
"05NB4ZZ"  =  "1"  /* Release Right Basilic Vein, Percutaneous Endoscopic Approach */
"05NC0ZZ"  =  "1"  /* Release Left Basilic Vein, Open Approach */
"05NC3ZZ"  =  "1"  /* Release Left Basilic Vein, Percutaneous Approach */
"05NC4ZZ"  =  "1"  /* Release Left Basilic Vein, Percutaneous Endoscopic Approach */
"05ND0ZZ"  =  "1"  /* Release Right Cephalic Vein, Open Approach */
"05ND3ZZ"  =  "1"  /* Release Right Cephalic Vein, Percutaneous Approach */
"05ND4ZZ"  =  "1"  /* Release Right Cephalic Vein, Perc Endo Approach */
"05NF0ZZ"  =  "1"  /* Release Left Cephalic Vein, Open Approach */
"05NF3ZZ"  =  "1"  /* Release Left Cephalic Vein, Percutaneous Approach */
"05NF4ZZ"  =  "1"  /* Release Left Cephalic Vein, Percutaneous Endoscopic Approach */
"05NG0ZZ"  =  "1"  /* Release Right Hand Vein, Open Approach */
"05NG3ZZ"  =  "1"  /* Release Right Hand Vein, Percutaneous Approach */
"05NG4ZZ"  =  "1"  /* Release Right Hand Vein, Percutaneous Endoscopic Approach */
"05NH0ZZ"  =  "1"  /* Release Left Hand Vein, Open Approach */
"05NH3ZZ"  =  "1"  /* Release Left Hand Vein, Percutaneous Approach */
"05NH4ZZ"  =  "1"  /* Release Left Hand Vein, Percutaneous Endoscopic Approach */
"05NL0ZZ"  =  "1"  /* Release Intracranial Vein, Open Approach */
"05NL3ZZ"  =  "1"  /* Release Intracranial Vein, Percutaneous Approach */
"05NL4ZZ"  =  "1"  /* Release Intracranial Vein, Percutaneous Endoscopic Approach */
"05NM0ZZ"  =  "1"  /* Release Right Internal Jugular Vein, Open Approach */
"05NM3ZZ"  =  "1"  /* Release Right Internal Jugular Vein, Percutaneous Approach */
"05NM4ZZ"  =  "1"  /* Release Right Internal Jugular Vein, Perc Endo Approach */
"05NN0ZZ"  =  "1"  /* Release Left Internal Jugular Vein, Open Approach */
"05NN3ZZ"  =  "1"  /* Release Left Internal Jugular Vein, Percutaneous Approach */
"05NN4ZZ"  =  "1"  /* Release Left Internal Jugular Vein, Perc Endo Approach */
"05NP0ZZ"  =  "1"  /* Release Right External Jugular Vein, Open Approach */
"05NP3ZZ"  =  "1"  /* Release Right External Jugular Vein, Percutaneous Approach */
"05NP4ZZ"  =  "1"  /* Release Right External Jugular Vein, Perc Endo Approach */
"05NQ0ZZ"  =  "1"  /* Release Left External Jugular Vein, Open Approach */
"05NQ3ZZ"  =  "1"  /* Release Left External Jugular Vein, Percutaneous Approach */
"05NQ4ZZ"  =  "1"  /* Release Left External Jugular Vein, Perc Endo Approach */
"05NR0ZZ"  =  "1"  /* Release Right Vertebral Vein, Open Approach */
"05NR3ZZ"  =  "1"  /* Release Right Vertebral Vein, Percutaneous Approach */
"05NR4ZZ"  =  "1"  /* Release Right Vertebral Vein, Perc Endo Approach */
"05NS0ZZ"  =  "1"  /* Release Left Vertebral Vein, Open Approach */
"05NS3ZZ"  =  "1"  /* Release Left Vertebral Vein, Percutaneous Approach */
"05NS4ZZ"  =  "1"  /* Release Left Vertebral Vein, Perc Endo Approach */
"05NT0ZZ"  =  "1"  /* Release Right Face Vein, Open Approach */
"05NT3ZZ"  =  "1"  /* Release Right Face Vein, Percutaneous Approach */
"05NT4ZZ"  =  "1"  /* Release Right Face Vein, Percutaneous Endoscopic Approach */
"05NV0ZZ"  =  "1"  /* Release Left Face Vein, Open Approach */
"05NV3ZZ"  =  "1"  /* Release Left Face Vein, Percutaneous Approach */
"05NV4ZZ"  =  "1"  /* Release Left Face Vein, Percutaneous Endoscopic Approach */
"05NY0ZZ"  =  "1"  /* Release Upper Vein, Open Approach */
"05NY3ZZ"  =  "1"  /* Release Upper Vein, Percutaneous Approach */
"05NY4ZZ"  =  "1"  /* Release Upper Vein, Percutaneous Endoscopic Approach */
"05PY00Z"  =  "1"  /* Removal of Drainage Device from Upper Vein, Open Approach */
"05PY02Z"  =  "1"  /* Removal of Monitoring Device from Upper Vein, Open Approach */
"05PY03Z"  =  "1"  /* Removal of Infusion Device from Upper Vein, Open Approach */
"05PY07Z"  =  "1"  /* Removal of Autol Sub from Up Vein, Open Approach */
"05PY0CZ"  =  "1"  /* Removal of Extraluminal Device from Up Vein, Open Approach */
"05PY0DZ"  =  "1"  /* Removal of Intraluminal Device from Up Vein, Open Approach */
"05PY0JZ"  =  "1"  /* Removal of Synthetic Substitute from Up Vein, Open Approach */
"05PY0KZ"  =  "1"  /* Removal of Nonaut Sub from Up Vein, Open Approach */
"05PY30Z"  =  "1"  /* Removal of Drainage Device from Upper Vein, Perc Approach */
"05PY32Z"  =  "1"  /* Removal of Monitoring Device from Upper Vein, Perc Approach */
"05PY33Z"  =  "1"  /* Removal of Infusion Device from Upper Vein, Perc Approach */
"05PY37Z"  =  "1"  /* Removal of Autol Sub from Up Vein, Perc Approach */
"05PY3CZ"  =  "1"  /* Removal of Extraluminal Device from Up Vein, Perc Approach */
"05PY3DZ"  =  "1"  /* Removal of Intraluminal Device from Up Vein, Perc Approach */
"05PY3JZ"  =  "1"  /* Removal of Synthetic Substitute from Up Vein, Perc Approach */
"05PY3KZ"  =  "1"  /* Removal of Nonaut Sub from Up Vein, Perc Approach */
"05PY40Z"  =  "1"  /* Removal of Drainage Device from Up Vein, Perc Endo Approach */
"05PY42Z"  =  "1"  /* Removal of Monitor Dev from Up Vein, Perc Endo Approach */
"05PY43Z"  =  "1"  /* Removal of Infusion Device from Up Vein, Perc Endo Approach */
"05PY47Z"  =  "1"  /* Removal of Autol Sub from Up Vein, Perc Endo Approach */
"05PY4CZ"  =  "1"  /* Removal of Extralum Dev from Up Vein, Perc Endo Approach */
"05PY4DZ"  =  "1"  /* Removal of Intralum Dev from Up Vein, Perc Endo Approach */
"05PY4JZ"  =  "1"  /* Removal of Synth Sub from Up Vein, Perc Endo Approach */
"05PY4KZ"  =  "1"  /* Removal of Nonaut Sub from Up Vein, Perc Endo Approach */
"05Q00ZZ"  =  "1"  /* Repair Azygos Vein, Open Approach */
"05Q03ZZ"  =  "1"  /* Repair Azygos Vein, Percutaneous Approach */
"05Q04ZZ"  =  "1"  /* Repair Azygos Vein, Percutaneous Endoscopic Approach */
"05Q10ZZ"  =  "1"  /* Repair Hemiazygos Vein, Open Approach */
"05Q13ZZ"  =  "1"  /* Repair Hemiazygos Vein, Percutaneous Approach */
"05Q14ZZ"  =  "1"  /* Repair Hemiazygos Vein, Percutaneous Endoscopic Approach */
"05Q30ZZ"  =  "1"  /* Repair Right Innominate Vein, Open Approach */
"05Q33ZZ"  =  "1"  /* Repair Right Innominate Vein, Percutaneous Approach */
"05Q34ZZ"  =  "1"  /* Repair Right Innominate Vein, Perc Endo Approach */
"05Q40ZZ"  =  "1"  /* Repair Left Innominate Vein, Open Approach */
"05Q43ZZ"  =  "1"  /* Repair Left Innominate Vein, Percutaneous Approach */
"05Q44ZZ"  =  "1"  /* Repair Left Innominate Vein, Perc Endo Approach */
"05Q50ZZ"  =  "1"  /* Repair Right Subclavian Vein, Open Approach */
"05Q53ZZ"  =  "1"  /* Repair Right Subclavian Vein, Percutaneous Approach */
"05Q54ZZ"  =  "1"  /* Repair Right Subclavian Vein, Perc Endo Approach */
"05Q60ZZ"  =  "1"  /* Repair Left Subclavian Vein, Open Approach */
"05Q63ZZ"  =  "1"  /* Repair Left Subclavian Vein, Percutaneous Approach */
"05Q64ZZ"  =  "1"  /* Repair Left Subclavian Vein, Perc Endo Approach */
"05Q70ZZ"  =  "1"  /* Repair Right Axillary Vein, Open Approach */
"05Q73ZZ"  =  "1"  /* Repair Right Axillary Vein, Percutaneous Approach */
"05Q74ZZ"  =  "1"  /* Repair Right Axillary Vein, Percutaneous Endoscopic Approach */
"05Q80ZZ"  =  "1"  /* Repair Left Axillary Vein, Open Approach */
"05Q83ZZ"  =  "1"  /* Repair Left Axillary Vein, Percutaneous Approach */
"05Q84ZZ"  =  "1"  /* Repair Left Axillary Vein, Percutaneous Endoscopic Approach */
"05Q90ZZ"  =  "1"  /* Repair Right Brachial Vein, Open Approach */
"05Q93ZZ"  =  "1"  /* Repair Right Brachial Vein, Percutaneous Approach */
"05Q94ZZ"  =  "1"  /* Repair Right Brachial Vein, Percutaneous Endoscopic Approach */
"05QA0ZZ"  =  "1"  /* Repair Left Brachial Vein, Open Approach */
"05QA3ZZ"  =  "1"  /* Repair Left Brachial Vein, Percutaneous Approach */
"05QA4ZZ"  =  "1"  /* Repair Left Brachial Vein, Percutaneous Endoscopic Approach */
"05QB0ZZ"  =  "1"  /* Repair Right Basilic Vein, Open Approach */
"05QB3ZZ"  =  "1"  /* Repair Right Basilic Vein, Percutaneous Approach */
"05QB4ZZ"  =  "1"  /* Repair Right Basilic Vein, Percutaneous Endoscopic Approach */
"05QC0ZZ"  =  "1"  /* Repair Left Basilic Vein, Open Approach */
"05QC3ZZ"  =  "1"  /* Repair Left Basilic Vein, Percutaneous Approach */
"05QC4ZZ"  =  "1"  /* Repair Left Basilic Vein, Percutaneous Endoscopic Approach */
"05QD0ZZ"  =  "1"  /* Repair Right Cephalic Vein, Open Approach */
"05QD3ZZ"  =  "1"  /* Repair Right Cephalic Vein, Percutaneous Approach */
"05QD4ZZ"  =  "1"  /* Repair Right Cephalic Vein, Percutaneous Endoscopic Approach */
"05QF0ZZ"  =  "1"  /* Repair Left Cephalic Vein, Open Approach */
"05QF3ZZ"  =  "1"  /* Repair Left Cephalic Vein, Percutaneous Approach */
"05QF4ZZ"  =  "1"  /* Repair Left Cephalic Vein, Percutaneous Endoscopic Approach */
"05QG0ZZ"  =  "1"  /* Repair Right Hand Vein, Open Approach */
"05QG3ZZ"  =  "1"  /* Repair Right Hand Vein, Percutaneous Approach */
"05QG4ZZ"  =  "1"  /* Repair Right Hand Vein, Percutaneous Endoscopic Approach */
"05QH0ZZ"  =  "1"  /* Repair Left Hand Vein, Open Approach */
"05QH3ZZ"  =  "1"  /* Repair Left Hand Vein, Percutaneous Approach */
"05QH4ZZ"  =  "1"  /* Repair Left Hand Vein, Percutaneous Endoscopic Approach */
"05QL0ZZ"  =  "1"  /* Repair Intracranial Vein, Open Approach */
"05QL3ZZ"  =  "1"  /* Repair Intracranial Vein, Percutaneous Approach */
"05QL4ZZ"  =  "1"  /* Repair Intracranial Vein, Percutaneous Endoscopic Approach */
"05QM0ZZ"  =  "1"  /* Repair Right Internal Jugular Vein, Open Approach */
"05QM3ZZ"  =  "1"  /* Repair Right Internal Jugular Vein, Percutaneous Approach */
"05QM4ZZ"  =  "1"  /* Repair Right Internal Jugular Vein, Perc Endo Approach */
"05QN0ZZ"  =  "1"  /* Repair Left Internal Jugular Vein, Open Approach */
"05QN3ZZ"  =  "1"  /* Repair Left Internal Jugular Vein, Percutaneous Approach */
"05QN4ZZ"  =  "1"  /* Repair Left Internal Jugular Vein, Perc Endo Approach */
"05QP0ZZ"  =  "1"  /* Repair Right External Jugular Vein, Open Approach */
"05QP3ZZ"  =  "1"  /* Repair Right External Jugular Vein, Percutaneous Approach */
"05QP4ZZ"  =  "1"  /* Repair Right External Jugular Vein, Perc Endo Approach */
"05QQ0ZZ"  =  "1"  /* Repair Left External Jugular Vein, Open Approach */
"05QQ3ZZ"  =  "1"  /* Repair Left External Jugular Vein, Percutaneous Approach */
"05QQ4ZZ"  =  "1"  /* Repair Left External Jugular Vein, Perc Endo Approach */
"05QR0ZZ"  =  "1"  /* Repair Right Vertebral Vein, Open Approach */
"05QR3ZZ"  =  "1"  /* Repair Right Vertebral Vein, Percutaneous Approach */
"05QR4ZZ"  =  "1"  /* Repair Right Vertebral Vein, Perc Endo Approach */
"05QS0ZZ"  =  "1"  /* Repair Left Vertebral Vein, Open Approach */
"05QS3ZZ"  =  "1"  /* Repair Left Vertebral Vein, Percutaneous Approach */
"05QS4ZZ"  =  "1"  /* Repair Left Vertebral Vein, Percutaneous Endoscopic Approach */
"05QT0ZZ"  =  "1"  /* Repair Right Face Vein, Open Approach */
"05QT3ZZ"  =  "1"  /* Repair Right Face Vein, Percutaneous Approach */
"05QT4ZZ"  =  "1"  /* Repair Right Face Vein, Percutaneous Endoscopic Approach */
"05QV0ZZ"  =  "1"  /* Repair Left Face Vein, Open Approach */
"05QV3ZZ"  =  "1"  /* Repair Left Face Vein, Percutaneous Approach */
"05QV4ZZ"  =  "1"  /* Repair Left Face Vein, Percutaneous Endoscopic Approach */
"05QY0ZZ"  =  "1"  /* Repair Upper Vein, Open Approach */
"05QY3ZZ"  =  "1"  /* Repair Upper Vein, Percutaneous Approach */
"05QY4ZZ"  =  "1"  /* Repair Upper Vein, Percutaneous Endoscopic Approach */
"05R007Z"  =  "1"  /* Replacement of Azygos Vein with Autol Sub, Open Approach */
"05R00JZ"  =  "1"  /* Replacement of Azygos Vein with Synth Sub, Open Approach */
"05R00KZ"  =  "1"  /* Replacement of Azygos Vein with Nonaut Sub, Open Approach */
"05R047Z"  =  "1"  /* Replace of Azygos Vein with Autol Sub, Perc Endo Approach */
"05R04JZ"  =  "1"  /* Replace of Azygos Vein with Synth Sub, Perc Endo Approach */
"05R04KZ"  =  "1"  /* Replace of Azygos Vein with Nonaut Sub, Perc Endo Approach */
"05R107Z"  =  "1"  /* Replacement of Hemiazygos Vein with Autol Sub, Open Approach */
"05R10JZ"  =  "1"  /* Replacement of Hemiazygos Vein with Synth Sub, Open Approach */
"05R10KZ"  =  "1"  /* Replace of Hemiazygos Vein with Nonaut Sub, Open Approach */
"05R147Z"  =  "1"  /* Replace Hemiazygos Vein w Autol Sub, Perc Endo */
"05R14JZ"  =  "1"  /* Replace Hemiazygos Vein w Synth Sub, Perc Endo */
"05R14KZ"  =  "1"  /* Replace Hemiazygos Vein w Nonaut Sub, Perc Endo */
"05R307Z"  =  "1"  /* Replacement of R Innom Vein with Autol Sub, Open Approach */
"05R30JZ"  =  "1"  /* Replacement of R Innom Vein with Synth Sub, Open Approach */
"05R30KZ"  =  "1"  /* Replacement of R Innom Vein with Nonaut Sub, Open Approach */
"05R347Z"  =  "1"  /* Replace of R Innom Vein with Autol Sub, Perc Endo Approach */
"05R34JZ"  =  "1"  /* Replace of R Innom Vein with Synth Sub, Perc Endo Approach */
"05R34KZ"  =  "1"  /* Replace of R Innom Vein with Nonaut Sub, Perc Endo Approach */
"05R407Z"  =  "1"  /* Replacement of L Innom Vein with Autol Sub, Open Approach */
"05R40JZ"  =  "1"  /* Replacement of L Innom Vein with Synth Sub, Open Approach */
"05R40KZ"  =  "1"  /* Replacement of L Innom Vein with Nonaut Sub, Open Approach */
"05R447Z"  =  "1"  /* Replace of L Innom Vein with Autol Sub, Perc Endo Approach */
"05R44JZ"  =  "1"  /* Replace of L Innom Vein with Synth Sub, Perc Endo Approach */
"05R44KZ"  =  "1"  /* Replace of L Innom Vein with Nonaut Sub, Perc Endo Approach */
"05R507Z"  =  "1"  /* Replacement of R Subclav Vein with Autol Sub, Open Approach */
"05R50JZ"  =  "1"  /* Replacement of R Subclav Vein with Synth Sub, Open Approach */
"05R50KZ"  =  "1"  /* Replacement of R Subclav Vein with Nonaut Sub, Open Approach */
"05R547Z"  =  "1"  /* Replace of R Subclav Vein with Autol Sub, Perc Endo Approach */
"05R54JZ"  =  "1"  /* Replace of R Subclav Vein with Synth Sub, Perc Endo Approach */
"05R54KZ"  =  "1"  /* Replace R Subclav Vein w Nonaut Sub, Perc Endo */
"05R607Z"  =  "1"  /* Replacement of L Subclav Vein with Autol Sub, Open Approach */
"05R60JZ"  =  "1"  /* Replacement of L Subclav Vein with Synth Sub, Open Approach */
"05R60KZ"  =  "1"  /* Replacement of L Subclav Vein with Nonaut Sub, Open Approach */
"05R647Z"  =  "1"  /* Replace of L Subclav Vein with Autol Sub, Perc Endo Approach */
"05R64JZ"  =  "1"  /* Replace of L Subclav Vein with Synth Sub, Perc Endo Approach */
"05R64KZ"  =  "1"  /* Replace L Subclav Vein w Nonaut Sub, Perc Endo */
"05R707Z"  =  "1"  /* Replacement of R Axilla Vein with Autol Sub, Open Approach */
"05R70JZ"  =  "1"  /* Replacement of R Axilla Vein with Synth Sub, Open Approach */
"05R70KZ"  =  "1"  /* Replacement of R Axilla Vein with Nonaut Sub, Open Approach */
"05R747Z"  =  "1"  /* Replace of R Axilla Vein with Autol Sub, Perc Endo Approach */
"05R74JZ"  =  "1"  /* Replace of R Axilla Vein with Synth Sub, Perc Endo Approach */
"05R74KZ"  =  "1"  /* Replace of R Axilla Vein with Nonaut Sub, Perc Endo Approach */
"05R807Z"  =  "1"  /* Replacement of L Axilla Vein with Autol Sub, Open Approach */
"05R80JZ"  =  "1"  /* Replacement of L Axilla Vein with Synth Sub, Open Approach */
"05R80KZ"  =  "1"  /* Replacement of L Axilla Vein with Nonaut Sub, Open Approach */
"05R847Z"  =  "1"  /* Replace of L Axilla Vein with Autol Sub, Perc Endo Approach */
"05R84JZ"  =  "1"  /* Replace of L Axilla Vein with Synth Sub, Perc Endo Approach */
"05R84KZ"  =  "1"  /* Replace of L Axilla Vein with Nonaut Sub, Perc Endo Approach */
"05R907Z"  =  "1"  /* Replacement of R Brach Vein with Autol Sub, Open Approach */
"05R90JZ"  =  "1"  /* Replacement of R Brach Vein with Synth Sub, Open Approach */
"05R90KZ"  =  "1"  /* Replacement of R Brach Vein with Nonaut Sub, Open Approach */
"05R947Z"  =  "1"  /* Replace of R Brach Vein with Autol Sub, Perc Endo Approach */
"05R94JZ"  =  "1"  /* Replace of R Brach Vein with Synth Sub, Perc Endo Approach */
"05R94KZ"  =  "1"  /* Replace of R Brach Vein with Nonaut Sub, Perc Endo Approach */
"05RA07Z"  =  "1"  /* Replacement of L Brach Vein with Autol Sub, Open Approach */
"05RA0JZ"  =  "1"  /* Replacement of L Brach Vein with Synth Sub, Open Approach */
"05RA0KZ"  =  "1"  /* Replacement of L Brach Vein with Nonaut Sub, Open Approach */
"05RA47Z"  =  "1"  /* Replace of L Brach Vein with Autol Sub, Perc Endo Approach */
"05RA4JZ"  =  "1"  /* Replace of L Brach Vein with Synth Sub, Perc Endo Approach */
"05RA4KZ"  =  "1"  /* Replace of L Brach Vein with Nonaut Sub, Perc Endo Approach */
"05RB07Z"  =  "1"  /* Replacement of R Basilic Vein with Autol Sub, Open Approach */
"05RB0JZ"  =  "1"  /* Replacement of R Basilic Vein with Synth Sub, Open Approach */
"05RB0KZ"  =  "1"  /* Replacement of R Basilic Vein with Nonaut Sub, Open Approach */
"05RB47Z"  =  "1"  /* Replace of R Basilic Vein with Autol Sub, Perc Endo Approach */
"05RB4JZ"  =  "1"  /* Replace of R Basilic Vein with Synth Sub, Perc Endo Approach */
"05RB4KZ"  =  "1"  /* Replace R Basilic Vein w Nonaut Sub, Perc Endo */
"05RC07Z"  =  "1"  /* Replacement of L Basilic Vein with Autol Sub, Open Approach */
"05RC0JZ"  =  "1"  /* Replacement of L Basilic Vein with Synth Sub, Open Approach */
"05RC0KZ"  =  "1"  /* Replacement of L Basilic Vein with Nonaut Sub, Open Approach */
"05RC47Z"  =  "1"  /* Replace of L Basilic Vein with Autol Sub, Perc Endo Approach */
"05RC4JZ"  =  "1"  /* Replace of L Basilic Vein with Synth Sub, Perc Endo Approach */
"05RC4KZ"  =  "1"  /* Replace L Basilic Vein w Nonaut Sub, Perc Endo */
"05RD07Z"  =  "1"  /* Replacement of R Cephalic Vein with Autol Sub, Open Approach */
"05RD0JZ"  =  "1"  /* Replacement of R Cephalic Vein with Synth Sub, Open Approach */
"05RD0KZ"  =  "1"  /* Replace of R Cephalic Vein with Nonaut Sub, Open Approach */
"05RD47Z"  =  "1"  /* Replace R Cephalic Vein w Autol Sub, Perc Endo */
"05RD4JZ"  =  "1"  /* Replace R Cephalic Vein w Synth Sub, Perc Endo */
"05RD4KZ"  =  "1"  /* Replace R Cephalic Vein w Nonaut Sub, Perc Endo */
"05RF07Z"  =  "1"  /* Replacement of L Cephalic Vein with Autol Sub, Open Approach */
"05RF0JZ"  =  "1"  /* Replacement of L Cephalic Vein with Synth Sub, Open Approach */
"05RF0KZ"  =  "1"  /* Replace of L Cephalic Vein with Nonaut Sub, Open Approach */
"05RF47Z"  =  "1"  /* Replace L Cephalic Vein w Autol Sub, Perc Endo */
"05RF4JZ"  =  "1"  /* Replace L Cephalic Vein w Synth Sub, Perc Endo */
"05RF4KZ"  =  "1"  /* Replace L Cephalic Vein w Nonaut Sub, Perc Endo */
"05RG07Z"  =  "1"  /* Replacement of Right Hand Vein with Autol Sub, Open Approach */
"05RG0JZ"  =  "1"  /* Replacement of Right Hand Vein with Synth Sub, Open Approach */
"05RG0KZ"  =  "1"  /* Replacement of R Hand Vein with Nonaut Sub, Open Approach */
"05RG47Z"  =  "1"  /* Replace of R Hand Vein with Autol Sub, Perc Endo Approach */
"05RG4JZ"  =  "1"  /* Replace of R Hand Vein with Synth Sub, Perc Endo Approach */
"05RG4KZ"  =  "1"  /* Replace of R Hand Vein with Nonaut Sub, Perc Endo Approach */
"05RH07Z"  =  "1"  /* Replacement of Left Hand Vein with Autol Sub, Open Approach */
"05RH0JZ"  =  "1"  /* Replacement of Left Hand Vein with Synth Sub, Open Approach */
"05RH0KZ"  =  "1"  /* Replacement of Left Hand Vein with Nonaut Sub, Open Approach */
"05RH47Z"  =  "1"  /* Replace of L Hand Vein with Autol Sub, Perc Endo Approach */
"05RH4JZ"  =  "1"  /* Replace of L Hand Vein with Synth Sub, Perc Endo Approach */
"05RH4KZ"  =  "1"  /* Replace of L Hand Vein with Nonaut Sub, Perc Endo Approach */
"05RL07Z"  =  "1"  /* Replacement of Intracran Vein with Autol Sub, Open Approach */
"05RL0JZ"  =  "1"  /* Replacement of Intracran Vein with Synth Sub, Open Approach */
"05RL0KZ"  =  "1"  /* Replacement of Intracran Vein with Nonaut Sub, Open Approach */
"05RL47Z"  =  "1"  /* Replace of Intracran Vein with Autol Sub, Perc Endo Approach */
"05RL4JZ"  =  "1"  /* Replace of Intracran Vein with Synth Sub, Perc Endo Approach */
"05RL4KZ"  =  "1"  /* Replace Intracran Vein w Nonaut Sub, Perc Endo */
"05RM07Z"  =  "1"  /* Replace of R Int Jugular Vein with Autol Sub, Open Approach */
"05RM0JZ"  =  "1"  /* Replace of R Int Jugular Vein with Synth Sub, Open Approach */
"05RM0KZ"  =  "1"  /* Replace of R Int Jugular Vein with Nonaut Sub, Open Approach */
"05RM47Z"  =  "1"  /* Replace R Int Jugular Vein w Autol Sub, Perc Endo */
"05RM4JZ"  =  "1"  /* Replace R Int Jugular Vein w Synth Sub, Perc Endo */
"05RM4KZ"  =  "1"  /* Replace R Int Jugular Vein w Nonaut Sub, Perc Endo */
"05RN07Z"  =  "1"  /* Replace of L Int Jugular Vein with Autol Sub, Open Approach */
"05RN0JZ"  =  "1"  /* Replace of L Int Jugular Vein with Synth Sub, Open Approach */
"05RN0KZ"  =  "1"  /* Replace of L Int Jugular Vein with Nonaut Sub, Open Approach */
"05RN47Z"  =  "1"  /* Replace L Int Jugular Vein w Autol Sub, Perc Endo */
"05RN4JZ"  =  "1"  /* Replace L Int Jugular Vein w Synth Sub, Perc Endo */
"05RN4KZ"  =  "1"  /* Replace L Int Jugular Vein w Nonaut Sub, Perc Endo */
"05RP07Z"  =  "1"  /* Replace of R Ext Jugular Vein with Autol Sub, Open Approach */
"05RP0JZ"  =  "1"  /* Replace of R Ext Jugular Vein with Synth Sub, Open Approach */
"05RP0KZ"  =  "1"  /* Replace of R Ext Jugular Vein with Nonaut Sub, Open Approach */
"05RP47Z"  =  "1"  /* Replace R Ext Jugular Vein w Autol Sub, Perc Endo */
"05RP4JZ"  =  "1"  /* Replace R Ext Jugular Vein w Synth Sub, Perc Endo */
"05RP4KZ"  =  "1"  /* Replace R Ext Jugular Vein w Nonaut Sub, Perc Endo */
"05RQ07Z"  =  "1"  /* Replace of L Ext Jugular Vein with Autol Sub, Open Approach */
"05RQ0JZ"  =  "1"  /* Replace of L Ext Jugular Vein with Synth Sub, Open Approach */
"05RQ0KZ"  =  "1"  /* Replace of L Ext Jugular Vein with Nonaut Sub, Open Approach */
"05RQ47Z"  =  "1"  /* Replace L Ext Jugular Vein w Autol Sub, Perc Endo */
"05RQ4JZ"  =  "1"  /* Replace L Ext Jugular Vein w Synth Sub, Perc Endo */
"05RQ4KZ"  =  "1"  /* Replace L Ext Jugular Vein w Nonaut Sub, Perc Endo */
"05RR07Z"  =  "1"  /* Replacement of R Verteb Vein with Autol Sub, Open Approach */
"05RR0JZ"  =  "1"  /* Replacement of R Verteb Vein with Synth Sub, Open Approach */
"05RR0KZ"  =  "1"  /* Replacement of R Verteb Vein with Nonaut Sub, Open Approach */
"05RR47Z"  =  "1"  /* Replace of R Verteb Vein with Autol Sub, Perc Endo Approach */
"05RR4JZ"  =  "1"  /* Replace of R Verteb Vein with Synth Sub, Perc Endo Approach */
"05RR4KZ"  =  "1"  /* Replace of R Verteb Vein with Nonaut Sub, Perc Endo Approach */
"05RS07Z"  =  "1"  /* Replacement of L Verteb Vein with Autol Sub, Open Approach */
"05RS0JZ"  =  "1"  /* Replacement of L Verteb Vein with Synth Sub, Open Approach */
"05RS0KZ"  =  "1"  /* Replacement of L Verteb Vein with Nonaut Sub, Open Approach */
"05RS47Z"  =  "1"  /* Replace of L Verteb Vein with Autol Sub, Perc Endo Approach */
"05RS4JZ"  =  "1"  /* Replace of L Verteb Vein with Synth Sub, Perc Endo Approach */
"05RS4KZ"  =  "1"  /* Replace of L Verteb Vein with Nonaut Sub, Perc Endo Approach */
"05RT07Z"  =  "1"  /* Replacement of Right Face Vein with Autol Sub, Open Approach */
"05RT0JZ"  =  "1"  /* Replacement of Right Face Vein with Synth Sub, Open Approach */
"05RT0KZ"  =  "1"  /* Replacement of R Face Vein with Nonaut Sub, Open Approach */
"05RT47Z"  =  "1"  /* Replace of R Face Vein with Autol Sub, Perc Endo Approach */
"05RT4JZ"  =  "1"  /* Replace of R Face Vein with Synth Sub, Perc Endo Approach */
"05RT4KZ"  =  "1"  /* Replace of R Face Vein with Nonaut Sub, Perc Endo Approach */
"05RV07Z"  =  "1"  /* Replacement of Left Face Vein with Autol Sub, Open Approach */
"05RV0JZ"  =  "1"  /* Replacement of Left Face Vein with Synth Sub, Open Approach */
"05RV0KZ"  =  "1"  /* Replacement of Left Face Vein with Nonaut Sub, Open Approach */
"05RV47Z"  =  "1"  /* Replace of L Face Vein with Autol Sub, Perc Endo Approach */
"05RV4JZ"  =  "1"  /* Replace of L Face Vein with Synth Sub, Perc Endo Approach */
"05RV4KZ"  =  "1"  /* Replace of L Face Vein with Nonaut Sub, Perc Endo Approach */
"05RY07Z"  =  "1"  /* Replacement of Upper Vein with Autol Sub, Open Approach */
"05RY0JZ"  =  "1"  /* Replacement of Upper Vein with Synth Sub, Open Approach */
"05RY0KZ"  =  "1"  /* Replacement of Upper Vein with Nonaut Sub, Open Approach */
"05RY47Z"  =  "1"  /* Replacement of Upper Vein with Autol Sub, Perc Endo Approach */
"05RY4JZ"  =  "1"  /* Replacement of Upper Vein with Synth Sub, Perc Endo Approach */
"05RY4KZ"  =  "1"  /* Replacement of Up Vein with Nonaut Sub, Perc Endo Approach */
"05S00ZZ"  =  "1"  /* Reposition Azygos Vein, Open Approach */
"05S03ZZ"  =  "1"  /* Reposition Azygos Vein, Percutaneous Approach */
"05S04ZZ"  =  "1"  /* Reposition Azygos Vein, Percutaneous Endoscopic Approach */
"05S10ZZ"  =  "1"  /* Reposition Hemiazygos Vein, Open Approach */
"05S13ZZ"  =  "1"  /* Reposition Hemiazygos Vein, Percutaneous Approach */
"05S14ZZ"  =  "1"  /* Reposition Hemiazygos Vein, Percutaneous Endoscopic Approach */
"05S30ZZ"  =  "1"  /* Reposition Right Innominate Vein, Open Approach */
"05S33ZZ"  =  "1"  /* Reposition Right Innominate Vein, Percutaneous Approach */
"05S34ZZ"  =  "1"  /* Reposition Right Innominate Vein, Perc Endo Approach */
"05S40ZZ"  =  "1"  /* Reposition Left Innominate Vein, Open Approach */
"05S43ZZ"  =  "1"  /* Reposition Left Innominate Vein, Percutaneous Approach */
"05S44ZZ"  =  "1"  /* Reposition Left Innominate Vein, Perc Endo Approach */
"05S50ZZ"  =  "1"  /* Reposition Right Subclavian Vein, Open Approach */
"05S53ZZ"  =  "1"  /* Reposition Right Subclavian Vein, Percutaneous Approach */
"05S54ZZ"  =  "1"  /* Reposition Right Subclavian Vein, Perc Endo Approach */
"05S60ZZ"  =  "1"  /* Reposition Left Subclavian Vein, Open Approach */
"05S63ZZ"  =  "1"  /* Reposition Left Subclavian Vein, Percutaneous Approach */
"05S64ZZ"  =  "1"  /* Reposition Left Subclavian Vein, Perc Endo Approach */
"05S70ZZ"  =  "1"  /* Reposition Right Axillary Vein, Open Approach */
"05S73ZZ"  =  "1"  /* Reposition Right Axillary Vein, Percutaneous Approach */
"05S74ZZ"  =  "1"  /* Reposition Right Axillary Vein, Perc Endo Approach */
"05S80ZZ"  =  "1"  /* Reposition Left Axillary Vein, Open Approach */
"05S83ZZ"  =  "1"  /* Reposition Left Axillary Vein, Percutaneous Approach */
"05S84ZZ"  =  "1"  /* Reposition Left Axillary Vein, Perc Endo Approach */
"05S90ZZ"  =  "1"  /* Reposition Right Brachial Vein, Open Approach */
"05S93ZZ"  =  "1"  /* Reposition Right Brachial Vein, Percutaneous Approach */
"05S94ZZ"  =  "1"  /* Reposition Right Brachial Vein, Perc Endo Approach */
"05SA0ZZ"  =  "1"  /* Reposition Left Brachial Vein, Open Approach */
"05SA3ZZ"  =  "1"  /* Reposition Left Brachial Vein, Percutaneous Approach */
"05SA4ZZ"  =  "1"  /* Reposition Left Brachial Vein, Perc Endo Approach */
"05SB0ZZ"  =  "1"  /* Reposition Right Basilic Vein, Open Approach */
"05SB3ZZ"  =  "1"  /* Reposition Right Basilic Vein, Percutaneous Approach */
"05SB4ZZ"  =  "1"  /* Reposition Right Basilic Vein, Perc Endo Approach */
"05SC0ZZ"  =  "1"  /* Reposition Left Basilic Vein, Open Approach */
"05SC3ZZ"  =  "1"  /* Reposition Left Basilic Vein, Percutaneous Approach */
"05SC4ZZ"  =  "1"  /* Reposition Left Basilic Vein, Perc Endo Approach */
"05SD0ZZ"  =  "1"  /* Reposition Right Cephalic Vein, Open Approach */
"05SD3ZZ"  =  "1"  /* Reposition Right Cephalic Vein, Percutaneous Approach */
"05SD4ZZ"  =  "1"  /* Reposition Right Cephalic Vein, Perc Endo Approach */
"05SF0ZZ"  =  "1"  /* Reposition Left Cephalic Vein, Open Approach */
"05SF3ZZ"  =  "1"  /* Reposition Left Cephalic Vein, Percutaneous Approach */
"05SF4ZZ"  =  "1"  /* Reposition Left Cephalic Vein, Perc Endo Approach */
"05SG0ZZ"  =  "1"  /* Reposition Right Hand Vein, Open Approach */
"05SG3ZZ"  =  "1"  /* Reposition Right Hand Vein, Percutaneous Approach */
"05SG4ZZ"  =  "1"  /* Reposition Right Hand Vein, Percutaneous Endoscopic Approach */
"05SH0ZZ"  =  "1"  /* Reposition Left Hand Vein, Open Approach */
"05SH3ZZ"  =  "1"  /* Reposition Left Hand Vein, Percutaneous Approach */
"05SH4ZZ"  =  "1"  /* Reposition Left Hand Vein, Percutaneous Endoscopic Approach */
"05SL0ZZ"  =  "1"  /* Reposition Intracranial Vein, Open Approach */
"05SL3ZZ"  =  "1"  /* Reposition Intracranial Vein, Percutaneous Approach */
"05SL4ZZ"  =  "1"  /* Reposition Intracranial Vein, Perc Endo Approach */
"05SM0ZZ"  =  "1"  /* Reposition Right Internal Jugular Vein, Open Approach */
"05SM3ZZ"  =  "1"  /* Reposition Right Internal Jugular Vein, Perc Approach */
"05SM4ZZ"  =  "1"  /* Reposition Right Internal Jugular Vein, Perc Endo Approach */
"05SN0ZZ"  =  "1"  /* Reposition Left Internal Jugular Vein, Open Approach */
"05SN3ZZ"  =  "1"  /* Reposition Left Internal Jugular Vein, Percutaneous Approach */
"05SN4ZZ"  =  "1"  /* Reposition Left Internal Jugular Vein, Perc Endo Approach */
"05SP0ZZ"  =  "1"  /* Reposition Right External Jugular Vein, Open Approach */
"05SP3ZZ"  =  "1"  /* Reposition Right External Jugular Vein, Perc Approach */
"05SP4ZZ"  =  "1"  /* Reposition Right External Jugular Vein, Perc Endo Approach */
"05SQ0ZZ"  =  "1"  /* Reposition Left External Jugular Vein, Open Approach */
"05SQ3ZZ"  =  "1"  /* Reposition Left External Jugular Vein, Percutaneous Approach */
"05SQ4ZZ"  =  "1"  /* Reposition Left External Jugular Vein, Perc Endo Approach */
"05SR0ZZ"  =  "1"  /* Reposition Right Vertebral Vein, Open Approach */
"05SR3ZZ"  =  "1"  /* Reposition Right Vertebral Vein, Percutaneous Approach */
"05SR4ZZ"  =  "1"  /* Reposition Right Vertebral Vein, Perc Endo Approach */
"05SS0ZZ"  =  "1"  /* Reposition Left Vertebral Vein, Open Approach */
"05SS3ZZ"  =  "1"  /* Reposition Left Vertebral Vein, Percutaneous Approach */
"05SS4ZZ"  =  "1"  /* Reposition Left Vertebral Vein, Perc Endo Approach */
"05ST0ZZ"  =  "1"  /* Reposition Right Face Vein, Open Approach */
"05ST3ZZ"  =  "1"  /* Reposition Right Face Vein, Percutaneous Approach */
"05ST4ZZ"  =  "1"  /* Reposition Right Face Vein, Percutaneous Endoscopic Approach */
"05SV0ZZ"  =  "1"  /* Reposition Left Face Vein, Open Approach */
"05SV3ZZ"  =  "1"  /* Reposition Left Face Vein, Percutaneous Approach */
"05SV4ZZ"  =  "1"  /* Reposition Left Face Vein, Percutaneous Endoscopic Approach */
"05SY0ZZ"  =  "1"  /* Reposition Upper Vein, Open Approach */
"05SY3ZZ"  =  "1"  /* Reposition Upper Vein, Percutaneous Approach */
"05SY4ZZ"  =  "1"  /* Reposition Upper Vein, Percutaneous Endoscopic Approach */
"05U007Z"  =  "1"  /* Supplement Azygos Vein with Autol Sub, Open Approach */
"05U00JZ"  =  "1"  /* Supplement Azygos Vein with Synth Sub, Open Approach */
"05U00KZ"  =  "1"  /* Supplement Azygos Vein with Nonaut Sub, Open Approach */
"05U037Z"  =  "1"  /* Supplement Azygos Vein with Autol Sub, Perc Approach */
"05U03JZ"  =  "1"  /* Supplement Azygos Vein with Synth Sub, Perc Approach */
"05U03KZ"  =  "1"  /* Supplement Azygos Vein with Nonaut Sub, Perc Approach */
"05U047Z"  =  "1"  /* Supplement Azygos Vein with Autol Sub, Perc Endo Approach */
"05U04JZ"  =  "1"  /* Supplement Azygos Vein with Synth Sub, Perc Endo Approach */
"05U04KZ"  =  "1"  /* Supplement Azygos Vein with Nonaut Sub, Perc Endo Approach */
"05U107Z"  =  "1"  /* Supplement Hemiazygos Vein with Autol Sub, Open Approach */
"05U10JZ"  =  "1"  /* Supplement Hemiazygos Vein with Synth Sub, Open Approach */
"05U10KZ"  =  "1"  /* Supplement Hemiazygos Vein with Nonaut Sub, Open Approach */
"05U137Z"  =  "1"  /* Supplement Hemiazygos Vein with Autol Sub, Perc Approach */
"05U13JZ"  =  "1"  /* Supplement Hemiazygos Vein with Synth Sub, Perc Approach */
"05U13KZ"  =  "1"  /* Supplement Hemiazygos Vein with Nonaut Sub, Perc Approach */
"05U147Z"  =  "1"  /* Supplement Hemiazygos Vein w Autol Sub, Perc Endo */
"05U14JZ"  =  "1"  /* Supplement Hemiazygos Vein w Synth Sub, Perc Endo */
"05U14KZ"  =  "1"  /* Supplement Hemiazygos Vein w Nonaut Sub, Perc Endo */
"05U307Z"  =  "1"  /* Supplement R Innom Vein with Autol Sub, Open Approach */
"05U30JZ"  =  "1"  /* Supplement R Innom Vein with Synth Sub, Open Approach */
"05U30KZ"  =  "1"  /* Supplement R Innom Vein with Nonaut Sub, Open Approach */
"05U337Z"  =  "1"  /* Supplement R Innom Vein with Autol Sub, Perc Approach */
"05U33JZ"  =  "1"  /* Supplement R Innom Vein with Synth Sub, Perc Approach */
"05U33KZ"  =  "1"  /* Supplement R Innom Vein with Nonaut Sub, Perc Approach */
"05U347Z"  =  "1"  /* Supplement R Innom Vein with Autol Sub, Perc Endo Approach */
"05U34JZ"  =  "1"  /* Supplement R Innom Vein with Synth Sub, Perc Endo Approach */
"05U34KZ"  =  "1"  /* Supplement R Innom Vein with Nonaut Sub, Perc Endo Approach */
"05U407Z"  =  "1"  /* Supplement L Innom Vein with Autol Sub, Open Approach */
"05U40JZ"  =  "1"  /* Supplement L Innom Vein with Synth Sub, Open Approach */
"05U40KZ"  =  "1"  /* Supplement L Innom Vein with Nonaut Sub, Open Approach */
"05U437Z"  =  "1"  /* Supplement L Innom Vein with Autol Sub, Perc Approach */
"05U43JZ"  =  "1"  /* Supplement L Innom Vein with Synth Sub, Perc Approach */
"05U43KZ"  =  "1"  /* Supplement L Innom Vein with Nonaut Sub, Perc Approach */
"05U447Z"  =  "1"  /* Supplement L Innom Vein with Autol Sub, Perc Endo Approach */
"05U44JZ"  =  "1"  /* Supplement L Innom Vein with Synth Sub, Perc Endo Approach */
"05U44KZ"  =  "1"  /* Supplement L Innom Vein with Nonaut Sub, Perc Endo Approach */
"05U507Z"  =  "1"  /* Supplement R Subclav Vein with Autol Sub, Open Approach */
"05U50JZ"  =  "1"  /* Supplement R Subclav Vein with Synth Sub, Open Approach */
"05U50KZ"  =  "1"  /* Supplement R Subclav Vein with Nonaut Sub, Open Approach */
"05U537Z"  =  "1"  /* Supplement R Subclav Vein with Autol Sub, Perc Approach */
"05U53JZ"  =  "1"  /* Supplement R Subclav Vein with Synth Sub, Perc Approach */
"05U53KZ"  =  "1"  /* Supplement R Subclav Vein with Nonaut Sub, Perc Approach */
"05U547Z"  =  "1"  /* Supplement R Subclav Vein with Autol Sub, Perc Endo Approach */
"05U54JZ"  =  "1"  /* Supplement R Subclav Vein with Synth Sub, Perc Endo Approach */
"05U54KZ"  =  "1"  /* Supplement R Subclav Vein w Nonaut Sub, Perc Endo */
"05U607Z"  =  "1"  /* Supplement L Subclav Vein with Autol Sub, Open Approach */
"05U60JZ"  =  "1"  /* Supplement L Subclav Vein with Synth Sub, Open Approach */
"05U60KZ"  =  "1"  /* Supplement L Subclav Vein with Nonaut Sub, Open Approach */
"05U637Z"  =  "1"  /* Supplement L Subclav Vein with Autol Sub, Perc Approach */
"05U63JZ"  =  "1"  /* Supplement L Subclav Vein with Synth Sub, Perc Approach */
"05U63KZ"  =  "1"  /* Supplement L Subclav Vein with Nonaut Sub, Perc Approach */
"05U647Z"  =  "1"  /* Supplement L Subclav Vein with Autol Sub, Perc Endo Approach */
"05U64JZ"  =  "1"  /* Supplement L Subclav Vein with Synth Sub, Perc Endo Approach */
"05U64KZ"  =  "1"  /* Supplement L Subclav Vein w Nonaut Sub, Perc Endo */
"05U707Z"  =  "1"  /* Supplement Right Axillary Vein with Autol Sub, Open Approach */
"05U70JZ"  =  "1"  /* Supplement Right Axillary Vein with Synth Sub, Open Approach */
"05U70KZ"  =  "1"  /* Supplement R Axilla Vein with Nonaut Sub, Open Approach */
"05U737Z"  =  "1"  /* Supplement Right Axillary Vein with Autol Sub, Perc Approach */
"05U73JZ"  =  "1"  /* Supplement Right Axillary Vein with Synth Sub, Perc Approach */
"05U73KZ"  =  "1"  /* Supplement R Axilla Vein with Nonaut Sub, Perc Approach */
"05U747Z"  =  "1"  /* Supplement R Axilla Vein with Autol Sub, Perc Endo Approach */
"05U74JZ"  =  "1"  /* Supplement R Axilla Vein with Synth Sub, Perc Endo Approach */
"05U74KZ"  =  "1"  /* Supplement R Axilla Vein with Nonaut Sub, Perc Endo Approach */
"05U807Z"  =  "1"  /* Supplement Left Axillary Vein with Autol Sub, Open Approach */
"05U80JZ"  =  "1"  /* Supplement Left Axillary Vein with Synth Sub, Open Approach */
"05U80KZ"  =  "1"  /* Supplement Left Axillary Vein with Nonaut Sub, Open Approach */
"05U837Z"  =  "1"  /* Supplement Left Axillary Vein with Autol Sub, Perc Approach */
"05U83JZ"  =  "1"  /* Supplement Left Axillary Vein with Synth Sub, Perc Approach */
"05U83KZ"  =  "1"  /* Supplement Left Axillary Vein with Nonaut Sub, Perc Approach */
"05U847Z"  =  "1"  /* Supplement L Axilla Vein with Autol Sub, Perc Endo Approach */
"05U84JZ"  =  "1"  /* Supplement L Axilla Vein with Synth Sub, Perc Endo Approach */
"05U84KZ"  =  "1"  /* Supplement L Axilla Vein with Nonaut Sub, Perc Endo Approach */
"05U907Z"  =  "1"  /* Supplement Right Brachial Vein with Autol Sub, Open Approach */
"05U90JZ"  =  "1"  /* Supplement Right Brachial Vein with Synth Sub, Open Approach */
"05U90KZ"  =  "1"  /* Supplement R Brach Vein with Nonaut Sub, Open Approach */
"05U937Z"  =  "1"  /* Supplement Right Brachial Vein with Autol Sub, Perc Approach */
"05U93JZ"  =  "1"  /* Supplement Right Brachial Vein with Synth Sub, Perc Approach */
"05U93KZ"  =  "1"  /* Supplement R Brach Vein with Nonaut Sub, Perc Approach */
"05U947Z"  =  "1"  /* Supplement R Brach Vein with Autol Sub, Perc Endo Approach */
"05U94JZ"  =  "1"  /* Supplement R Brach Vein with Synth Sub, Perc Endo Approach */
"05U94KZ"  =  "1"  /* Supplement R Brach Vein with Nonaut Sub, Perc Endo Approach */
"05UA07Z"  =  "1"  /* Supplement Left Brachial Vein with Autol Sub, Open Approach */
"05UA0JZ"  =  "1"  /* Supplement Left Brachial Vein with Synth Sub, Open Approach */
"05UA0KZ"  =  "1"  /* Supplement Left Brachial Vein with Nonaut Sub, Open Approach */
"05UA37Z"  =  "1"  /* Supplement Left Brachial Vein with Autol Sub, Perc Approach */
"05UA3JZ"  =  "1"  /* Supplement Left Brachial Vein with Synth Sub, Perc Approach */
"05UA3KZ"  =  "1"  /* Supplement Left Brachial Vein with Nonaut Sub, Perc Approach */
"05UA47Z"  =  "1"  /* Supplement L Brach Vein with Autol Sub, Perc Endo Approach */
"05UA4JZ"  =  "1"  /* Supplement L Brach Vein with Synth Sub, Perc Endo Approach */
"05UA4KZ"  =  "1"  /* Supplement L Brach Vein with Nonaut Sub, Perc Endo Approach */
"05UB07Z"  =  "1"  /* Supplement Right Basilic Vein with Autol Sub, Open Approach */
"05UB0JZ"  =  "1"  /* Supplement Right Basilic Vein with Synth Sub, Open Approach */
"05UB0KZ"  =  "1"  /* Supplement Right Basilic Vein with Nonaut Sub, Open Approach */
"05UB37Z"  =  "1"  /* Supplement Right Basilic Vein with Autol Sub, Perc Approach */
"05UB3JZ"  =  "1"  /* Supplement Right Basilic Vein with Synth Sub, Perc Approach */
"05UB3KZ"  =  "1"  /* Supplement Right Basilic Vein with Nonaut Sub, Perc Approach */
"05UB47Z"  =  "1"  /* Supplement R Basilic Vein with Autol Sub, Perc Endo Approach */
"05UB4JZ"  =  "1"  /* Supplement R Basilic Vein with Synth Sub, Perc Endo Approach */
"05UB4KZ"  =  "1"  /* Supplement R Basilic Vein w Nonaut Sub, Perc Endo */
"05UC07Z"  =  "1"  /* Supplement Left Basilic Vein with Autol Sub, Open Approach */
"05UC0JZ"  =  "1"  /* Supplement Left Basilic Vein with Synth Sub, Open Approach */
"05UC0KZ"  =  "1"  /* Supplement Left Basilic Vein with Nonaut Sub, Open Approach */
"05UC37Z"  =  "1"  /* Supplement Left Basilic Vein with Autol Sub, Perc Approach */
"05UC3JZ"  =  "1"  /* Supplement Left Basilic Vein with Synth Sub, Perc Approach */
"05UC3KZ"  =  "1"  /* Supplement Left Basilic Vein with Nonaut Sub, Perc Approach */
"05UC47Z"  =  "1"  /* Supplement L Basilic Vein with Autol Sub, Perc Endo Approach */
"05UC4JZ"  =  "1"  /* Supplement L Basilic Vein with Synth Sub, Perc Endo Approach */
"05UC4KZ"  =  "1"  /* Supplement L Basilic Vein w Nonaut Sub, Perc Endo */
"05UD07Z"  =  "1"  /* Supplement Right Cephalic Vein with Autol Sub, Open Approach */
"05UD0JZ"  =  "1"  /* Supplement Right Cephalic Vein with Synth Sub, Open Approach */
"05UD0KZ"  =  "1"  /* Supplement R Cephalic Vein with Nonaut Sub, Open Approach */
"05UD37Z"  =  "1"  /* Supplement Right Cephalic Vein with Autol Sub, Perc Approach */
"05UD3JZ"  =  "1"  /* Supplement Right Cephalic Vein with Synth Sub, Perc Approach */
"05UD3KZ"  =  "1"  /* Supplement R Cephalic Vein with Nonaut Sub, Perc Approach */
"05UD47Z"  =  "1"  /* Supplement R Cephalic Vein w Autol Sub, Perc Endo */
"05UD4JZ"  =  "1"  /* Supplement R Cephalic Vein w Synth Sub, Perc Endo */
"05UD4KZ"  =  "1"  /* Supplement R Cephalic Vein w Nonaut Sub, Perc Endo */
"05UF07Z"  =  "1"  /* Supplement Left Cephalic Vein with Autol Sub, Open Approach */
"05UF0JZ"  =  "1"  /* Supplement Left Cephalic Vein with Synth Sub, Open Approach */
"05UF0KZ"  =  "1"  /* Supplement Left Cephalic Vein with Nonaut Sub, Open Approach */
"05UF37Z"  =  "1"  /* Supplement Left Cephalic Vein with Autol Sub, Perc Approach */
"05UF3JZ"  =  "1"  /* Supplement Left Cephalic Vein with Synth Sub, Perc Approach */
"05UF3KZ"  =  "1"  /* Supplement Left Cephalic Vein with Nonaut Sub, Perc Approach */
"05UF47Z"  =  "1"  /* Supplement L Cephalic Vein w Autol Sub, Perc Endo */
"05UF4JZ"  =  "1"  /* Supplement L Cephalic Vein w Synth Sub, Perc Endo */
"05UF4KZ"  =  "1"  /* Supplement L Cephalic Vein w Nonaut Sub, Perc Endo */
"05UG07Z"  =  "1"  /* Supplement Right Hand Vein with Autol Sub, Open Approach */
"05UG0JZ"  =  "1"  /* Supplement Right Hand Vein with Synth Sub, Open Approach */
"05UG0KZ"  =  "1"  /* Supplement Right Hand Vein with Nonaut Sub, Open Approach */
"05UG37Z"  =  "1"  /* Supplement Right Hand Vein with Autol Sub, Perc Approach */
"05UG3JZ"  =  "1"  /* Supplement Right Hand Vein with Synth Sub, Perc Approach */
"05UG3KZ"  =  "1"  /* Supplement Right Hand Vein with Nonaut Sub, Perc Approach */
"05UG47Z"  =  "1"  /* Supplement R Hand Vein with Autol Sub, Perc Endo Approach */
"05UG4JZ"  =  "1"  /* Supplement R Hand Vein with Synth Sub, Perc Endo Approach */
"05UG4KZ"  =  "1"  /* Supplement R Hand Vein with Nonaut Sub, Perc Endo Approach */
"05UH07Z"  =  "1"  /* Supplement Left Hand Vein with Autol Sub, Open Approach */
"05UH0JZ"  =  "1"  /* Supplement Left Hand Vein with Synth Sub, Open Approach */
"05UH0KZ"  =  "1"  /* Supplement Left Hand Vein with Nonaut Sub, Open Approach */
"05UH37Z"  =  "1"  /* Supplement Left Hand Vein with Autol Sub, Perc Approach */
"05UH3JZ"  =  "1"  /* Supplement Left Hand Vein with Synth Sub, Perc Approach */
"05UH3KZ"  =  "1"  /* Supplement Left Hand Vein with Nonaut Sub, Perc Approach */
"05UH47Z"  =  "1"  /* Supplement Left Hand Vein with Autol Sub, Perc Endo Approach */
"05UH4JZ"  =  "1"  /* Supplement Left Hand Vein with Synth Sub, Perc Endo Approach */
"05UH4KZ"  =  "1"  /* Supplement L Hand Vein with Nonaut Sub, Perc Endo Approach */
"05UL07Z"  =  "1"  /* Supplement Intracranial Vein with Autol Sub, Open Approach */
"05UL0JZ"  =  "1"  /* Supplement Intracranial Vein with Synth Sub, Open Approach */
"05UL0KZ"  =  "1"  /* Supplement Intracranial Vein with Nonaut Sub, Open Approach */
"05UL37Z"  =  "1"  /* Supplement Intracranial Vein with Autol Sub, Perc Approach */
"05UL3JZ"  =  "1"  /* Supplement Intracranial Vein with Synth Sub, Perc Approach */
"05UL3KZ"  =  "1"  /* Supplement Intracranial Vein with Nonaut Sub, Perc Approach */
"05UL47Z"  =  "1"  /* Supplement Intracran Vein with Autol Sub, Perc Endo Approach */
"05UL4JZ"  =  "1"  /* Supplement Intracran Vein with Synth Sub, Perc Endo Approach */
"05UL4KZ"  =  "1"  /* Supplement Intracran Vein w Nonaut Sub, Perc Endo */
"05UM07Z"  =  "1"  /* Supplement R Int Jugular Vein with Autol Sub, Open Approach */
"05UM0JZ"  =  "1"  /* Supplement R Int Jugular Vein with Synth Sub, Open Approach */
"05UM0KZ"  =  "1"  /* Supplement R Int Jugular Vein with Nonaut Sub, Open Approach */
"05UM37Z"  =  "1"  /* Supplement R Int Jugular Vein with Autol Sub, Perc Approach */
"05UM3JZ"  =  "1"  /* Supplement R Int Jugular Vein with Synth Sub, Perc Approach */
"05UM3KZ"  =  "1"  /* Supplement R Int Jugular Vein with Nonaut Sub, Perc Approach */
"05UM47Z"  =  "1"  /* Supplement R Int Jugular Vein w Autol Sub, Perc Endo */
"05UM4JZ"  =  "1"  /* Supplement R Int Jugular Vein w Synth Sub, Perc Endo */
"05UM4KZ"  =  "1"  /* Supplement R Int Jugular Vein w Nonaut Sub, Perc Endo */
"05UN07Z"  =  "1"  /* Supplement L Int Jugular Vein with Autol Sub, Open Approach */
"05UN0JZ"  =  "1"  /* Supplement L Int Jugular Vein with Synth Sub, Open Approach */
"05UN0KZ"  =  "1"  /* Supplement L Int Jugular Vein with Nonaut Sub, Open Approach */
"05UN37Z"  =  "1"  /* Supplement L Int Jugular Vein with Autol Sub, Perc Approach */
"05UN3JZ"  =  "1"  /* Supplement L Int Jugular Vein with Synth Sub, Perc Approach */
"05UN3KZ"  =  "1"  /* Supplement L Int Jugular Vein with Nonaut Sub, Perc Approach */
"05UN47Z"  =  "1"  /* Supplement L Int Jugular Vein w Autol Sub, Perc Endo */
"05UN4JZ"  =  "1"  /* Supplement L Int Jugular Vein w Synth Sub, Perc Endo */
"05UN4KZ"  =  "1"  /* Supplement L Int Jugular Vein w Nonaut Sub, Perc Endo */
"05UP07Z"  =  "1"  /* Supplement R Ext Jugular Vein with Autol Sub, Open Approach */
"05UP0JZ"  =  "1"  /* Supplement R Ext Jugular Vein with Synth Sub, Open Approach */
"05UP0KZ"  =  "1"  /* Supplement R Ext Jugular Vein with Nonaut Sub, Open Approach */
"05UP37Z"  =  "1"  /* Supplement R Ext Jugular Vein with Autol Sub, Perc Approach */
"05UP3JZ"  =  "1"  /* Supplement R Ext Jugular Vein with Synth Sub, Perc Approach */
"05UP3KZ"  =  "1"  /* Supplement R Ext Jugular Vein with Nonaut Sub, Perc Approach */
"05UP47Z"  =  "1"  /* Supplement R Ext Jugular Vein w Autol Sub, Perc Endo */
"05UP4JZ"  =  "1"  /* Supplement R Ext Jugular Vein w Synth Sub, Perc Endo */
"05UP4KZ"  =  "1"  /* Supplement R Ext Jugular Vein w Nonaut Sub, Perc Endo */
"05UQ07Z"  =  "1"  /* Supplement L Ext Jugular Vein with Autol Sub, Open Approach */
"05UQ0JZ"  =  "1"  /* Supplement L Ext Jugular Vein with Synth Sub, Open Approach */
"05UQ0KZ"  =  "1"  /* Supplement L Ext Jugular Vein with Nonaut Sub, Open Approach */
"05UQ37Z"  =  "1"  /* Supplement L Ext Jugular Vein with Autol Sub, Perc Approach */
"05UQ3JZ"  =  "1"  /* Supplement L Ext Jugular Vein with Synth Sub, Perc Approach */
"05UQ3KZ"  =  "1"  /* Supplement L Ext Jugular Vein with Nonaut Sub, Perc Approach */
"05UQ47Z"  =  "1"  /* Supplement L Ext Jugular Vein w Autol Sub, Perc Endo */
"05UQ4JZ"  =  "1"  /* Supplement L Ext Jugular Vein w Synth Sub, Perc Endo */
"05UQ4KZ"  =  "1"  /* Supplement L Ext Jugular Vein w Nonaut Sub, Perc Endo */
"05UR07Z"  =  "1"  /* Supplement R Verteb Vein with Autol Sub, Open Approach */
"05UR0JZ"  =  "1"  /* Supplement R Verteb Vein with Synth Sub, Open Approach */
"05UR0KZ"  =  "1"  /* Supplement R Verteb Vein with Nonaut Sub, Open Approach */
"05UR37Z"  =  "1"  /* Supplement R Verteb Vein with Autol Sub, Perc Approach */
"05UR3JZ"  =  "1"  /* Supplement R Verteb Vein with Synth Sub, Perc Approach */
"05UR3KZ"  =  "1"  /* Supplement R Verteb Vein with Nonaut Sub, Perc Approach */
"05UR47Z"  =  "1"  /* Supplement R Verteb Vein with Autol Sub, Perc Endo Approach */
"05UR4JZ"  =  "1"  /* Supplement R Verteb Vein with Synth Sub, Perc Endo Approach */
"05UR4KZ"  =  "1"  /* Supplement R Verteb Vein with Nonaut Sub, Perc Endo Approach */
"05US07Z"  =  "1"  /* Supplement Left Vertebral Vein with Autol Sub, Open Approach */
"05US0JZ"  =  "1"  /* Supplement Left Vertebral Vein with Synth Sub, Open Approach */
"05US0KZ"  =  "1"  /* Supplement L Verteb Vein with Nonaut Sub, Open Approach */
"05US37Z"  =  "1"  /* Supplement Left Vertebral Vein with Autol Sub, Perc Approach */
"05US3JZ"  =  "1"  /* Supplement Left Vertebral Vein with Synth Sub, Perc Approach */
"05US3KZ"  =  "1"  /* Supplement L Verteb Vein with Nonaut Sub, Perc Approach */
"05US47Z"  =  "1"  /* Supplement L Verteb Vein with Autol Sub, Perc Endo Approach */
"05US4JZ"  =  "1"  /* Supplement L Verteb Vein with Synth Sub, Perc Endo Approach */
"05US4KZ"  =  "1"  /* Supplement L Verteb Vein with Nonaut Sub, Perc Endo Approach */
"05UT07Z"  =  "1"  /* Supplement Right Face Vein with Autol Sub, Open Approach */
"05UT0JZ"  =  "1"  /* Supplement Right Face Vein with Synth Sub, Open Approach */
"05UT0KZ"  =  "1"  /* Supplement Right Face Vein with Nonaut Sub, Open Approach */
"05UT37Z"  =  "1"  /* Supplement Right Face Vein with Autol Sub, Perc Approach */
"05UT3JZ"  =  "1"  /* Supplement Right Face Vein with Synth Sub, Perc Approach */
"05UT3KZ"  =  "1"  /* Supplement Right Face Vein with Nonaut Sub, Perc Approach */
"05UT47Z"  =  "1"  /* Supplement R Face Vein with Autol Sub, Perc Endo Approach */
"05UT4JZ"  =  "1"  /* Supplement R Face Vein with Synth Sub, Perc Endo Approach */
"05UT4KZ"  =  "1"  /* Supplement R Face Vein with Nonaut Sub, Perc Endo Approach */
"05UV07Z"  =  "1"  /* Supplement Left Face Vein with Autol Sub, Open Approach */
"05UV0JZ"  =  "1"  /* Supplement Left Face Vein with Synth Sub, Open Approach */
"05UV0KZ"  =  "1"  /* Supplement Left Face Vein with Nonaut Sub, Open Approach */
"05UV37Z"  =  "1"  /* Supplement Left Face Vein with Autol Sub, Perc Approach */
"05UV3JZ"  =  "1"  /* Supplement Left Face Vein with Synth Sub, Perc Approach */
"05UV3KZ"  =  "1"  /* Supplement Left Face Vein with Nonaut Sub, Perc Approach */
"05UV47Z"  =  "1"  /* Supplement Left Face Vein with Autol Sub, Perc Endo Approach */
"05UV4JZ"  =  "1"  /* Supplement Left Face Vein with Synth Sub, Perc Endo Approach */
"05UV4KZ"  =  "1"  /* Supplement L Face Vein with Nonaut Sub, Perc Endo Approach */
"05UY07Z"  =  "1"  /* Supplement Upper Vein with Autol Sub, Open Approach */
"05UY0JZ"  =  "1"  /* Supplement Upper Vein with Synth Sub, Open Approach */
"05UY0KZ"  =  "1"  /* Supplement Upper Vein with Nonaut Sub, Open Approach */
"05UY37Z"  =  "1"  /* Supplement Upper Vein with Autol Sub, Perc Approach */
"05UY3JZ"  =  "1"  /* Supplement Upper Vein with Synth Sub, Perc Approach */
"05UY3KZ"  =  "1"  /* Supplement Upper Vein with Nonaut Sub, Perc Approach */
"05UY47Z"  =  "1"  /* Supplement Upper Vein with Autol Sub, Perc Endo Approach */
"05UY4JZ"  =  "1"  /* Supplement Upper Vein with Synth Sub, Perc Endo Approach */
"05UY4KZ"  =  "1"  /* Supplement Upper Vein with Nonaut Sub, Perc Endo Approach */
"05V00CZ"  =  "1"  /* Restriction of Azygos Vein with Extralum Dev, Open Approach */
"05V00DZ"  =  "1"  /* Restriction of Azygos Vein with Intralum Dev, Open Approach */
"05V00ZZ"  =  "1"  /* Restriction of Azygos Vein, Open Approach */
"05V03CZ"  =  "1"  /* Restriction of Azygos Vein with Extralum Dev, Perc Approach */
"05V03DZ"  =  "1"  /* Restriction of Azygos Vein with Intralum Dev, Perc Approach */
"05V03ZZ"  =  "1"  /* Restriction of Azygos Vein, Percutaneous Approach */
"05V04CZ"  =  "1"  /* Restrict Azygos Vein w Extralum Dev, Perc Endo */
"05V04DZ"  =  "1"  /* Restrict Azygos Vein w Intralum Dev, Perc Endo */
"05V04ZZ"  =  "1"  /* Restriction of Azygos Vein, Percutaneous Endoscopic Approach */
"05V10CZ"  =  "1"  /* Restrict of Hemiazygos Vein with Extralum Dev, Open Approach */
"05V10DZ"  =  "1"  /* Restrict of Hemiazygos Vein with Intralum Dev, Open Approach */
"05V10ZZ"  =  "1"  /* Restriction of Hemiazygos Vein, Open Approach */
"05V13CZ"  =  "1"  /* Restrict of Hemiazygos Vein with Extralum Dev, Perc Approach */
"05V13DZ"  =  "1"  /* Restrict of Hemiazygos Vein with Intralum Dev, Perc Approach */
"05V13ZZ"  =  "1"  /* Restriction of Hemiazygos Vein, Percutaneous Approach */
"05V14CZ"  =  "1"  /* Restrict Hemiazygos Vein w Extralum Dev, Perc Endo */
"05V14DZ"  =  "1"  /* Restrict Hemiazygos Vein w Intralum Dev, Perc Endo */
"05V14ZZ"  =  "1"  /* Restriction of Hemiazygos Vein, Perc Endo Approach */
"05V30CZ"  =  "1"  /* Restriction of R Innom Vein with Extralum Dev, Open Approach */
"05V30DZ"  =  "1"  /* Restriction of R Innom Vein with Intralum Dev, Open Approach */
"05V30ZZ"  =  "1"  /* Restriction of Right Innominate Vein, Open Approach */
"05V33CZ"  =  "1"  /* Restriction of R Innom Vein with Extralum Dev, Perc Approach */
"05V33DZ"  =  "1"  /* Restriction of R Innom Vein with Intralum Dev, Perc Approach */
"05V33ZZ"  =  "1"  /* Restriction of Right Innominate Vein, Percutaneous Approach */
"05V34CZ"  =  "1"  /* Restrict R Innom Vein w Extralum Dev, Perc Endo */
"05V34DZ"  =  "1"  /* Restrict R Innom Vein w Intralum Dev, Perc Endo */
"05V34ZZ"  =  "1"  /* Restriction of Right Innominate Vein, Perc Endo Approach */
"05V40CZ"  =  "1"  /* Restriction of L Innom Vein with Extralum Dev, Open Approach */
"05V40DZ"  =  "1"  /* Restriction of L Innom Vein with Intralum Dev, Open Approach */
"05V40ZZ"  =  "1"  /* Restriction of Left Innominate Vein, Open Approach */
"05V43CZ"  =  "1"  /* Restriction of L Innom Vein with Extralum Dev, Perc Approach */
"05V43DZ"  =  "1"  /* Restriction of L Innom Vein with Intralum Dev, Perc Approach */
"05V43ZZ"  =  "1"  /* Restriction of Left Innominate Vein, Percutaneous Approach */
"05V44CZ"  =  "1"  /* Restrict L Innom Vein w Extralum Dev, Perc Endo */
"05V44DZ"  =  "1"  /* Restrict L Innom Vein w Intralum Dev, Perc Endo */
"05V44ZZ"  =  "1"  /* Restriction of Left Innominate Vein, Perc Endo Approach */
"05V50CZ"  =  "1"  /* Restrict of R Subclav Vein with Extralum Dev, Open Approach */
"05V50DZ"  =  "1"  /* Restrict of R Subclav Vein with Intralum Dev, Open Approach */
"05V50ZZ"  =  "1"  /* Restriction of Right Subclavian Vein, Open Approach */
"05V53CZ"  =  "1"  /* Restrict of R Subclav Vein with Extralum Dev, Perc Approach */
"05V53DZ"  =  "1"  /* Restrict of R Subclav Vein with Intralum Dev, Perc Approach */
"05V53ZZ"  =  "1"  /* Restriction of Right Subclavian Vein, Percutaneous Approach */
"05V54CZ"  =  "1"  /* Restrict R Subclav Vein w Extralum Dev, Perc Endo */
"05V54DZ"  =  "1"  /* Restrict R Subclav Vein w Intralum Dev, Perc Endo */
"05V54ZZ"  =  "1"  /* Restriction of Right Subclavian Vein, Perc Endo Approach */
"05V60CZ"  =  "1"  /* Restrict of L Subclav Vein with Extralum Dev, Open Approach */
"05V60DZ"  =  "1"  /* Restrict of L Subclav Vein with Intralum Dev, Open Approach */
"05V60ZZ"  =  "1"  /* Restriction of Left Subclavian Vein, Open Approach */
"05V63CZ"  =  "1"  /* Restrict of L Subclav Vein with Extralum Dev, Perc Approach */
"05V63DZ"  =  "1"  /* Restrict of L Subclav Vein with Intralum Dev, Perc Approach */
"05V63ZZ"  =  "1"  /* Restriction of Left Subclavian Vein, Percutaneous Approach */
"05V64CZ"  =  "1"  /* Restrict L Subclav Vein w Extralum Dev, Perc Endo */
"05V64DZ"  =  "1"  /* Restrict L Subclav Vein w Intralum Dev, Perc Endo */
"05V64ZZ"  =  "1"  /* Restriction of Left Subclavian Vein, Perc Endo Approach */
"05V70CZ"  =  "1"  /* Restrict of R Axilla Vein with Extralum Dev, Open Approach */
"05V70DZ"  =  "1"  /* Restrict of R Axilla Vein with Intralum Dev, Open Approach */
"05V70ZZ"  =  "1"  /* Restriction of Right Axillary Vein, Open Approach */
"05V73CZ"  =  "1"  /* Restrict of R Axilla Vein with Extralum Dev, Perc Approach */
"05V73DZ"  =  "1"  /* Restrict of R Axilla Vein with Intralum Dev, Perc Approach */
"05V73ZZ"  =  "1"  /* Restriction of Right Axillary Vein, Percutaneous Approach */
"05V74CZ"  =  "1"  /* Restrict R Axilla Vein w Extralum Dev, Perc Endo */
"05V74DZ"  =  "1"  /* Restrict R Axilla Vein w Intralum Dev, Perc Endo */
"05V74ZZ"  =  "1"  /* Restriction of Right Axillary Vein, Perc Endo Approach */
"05V80CZ"  =  "1"  /* Restrict of L Axilla Vein with Extralum Dev, Open Approach */
"05V80DZ"  =  "1"  /* Restrict of L Axilla Vein with Intralum Dev, Open Approach */
"05V80ZZ"  =  "1"  /* Restriction of Left Axillary Vein, Open Approach */
"05V83CZ"  =  "1"  /* Restrict of L Axilla Vein with Extralum Dev, Perc Approach */
"05V83DZ"  =  "1"  /* Restrict of L Axilla Vein with Intralum Dev, Perc Approach */
"05V83ZZ"  =  "1"  /* Restriction of Left Axillary Vein, Percutaneous Approach */
"05V84CZ"  =  "1"  /* Restrict L Axilla Vein w Extralum Dev, Perc Endo */
"05V84DZ"  =  "1"  /* Restrict L Axilla Vein w Intralum Dev, Perc Endo */
"05V84ZZ"  =  "1"  /* Restriction of Left Axillary Vein, Perc Endo Approach */
"05V90CZ"  =  "1"  /* Restriction of R Brach Vein with Extralum Dev, Open Approach */
"05V90DZ"  =  "1"  /* Restriction of R Brach Vein with Intralum Dev, Open Approach */
"05V90ZZ"  =  "1"  /* Restriction of Right Brachial Vein, Open Approach */
"05V93CZ"  =  "1"  /* Restriction of R Brach Vein with Extralum Dev, Perc Approach */
"05V93DZ"  =  "1"  /* Restriction of R Brach Vein with Intralum Dev, Perc Approach */
"05V93ZZ"  =  "1"  /* Restriction of Right Brachial Vein, Percutaneous Approach */
"05V94CZ"  =  "1"  /* Restrict R Brach Vein w Extralum Dev, Perc Endo */
"05V94DZ"  =  "1"  /* Restrict R Brach Vein w Intralum Dev, Perc Endo */
"05V94ZZ"  =  "1"  /* Restriction of Right Brachial Vein, Perc Endo Approach */
"05VA0CZ"  =  "1"  /* Restriction of L Brach Vein with Extralum Dev, Open Approach */
"05VA0DZ"  =  "1"  /* Restriction of L Brach Vein with Intralum Dev, Open Approach */
"05VA0ZZ"  =  "1"  /* Restriction of Left Brachial Vein, Open Approach */
"05VA3CZ"  =  "1"  /* Restriction of L Brach Vein with Extralum Dev, Perc Approach */
"05VA3DZ"  =  "1"  /* Restriction of L Brach Vein with Intralum Dev, Perc Approach */
"05VA3ZZ"  =  "1"  /* Restriction of Left Brachial Vein, Percutaneous Approach */
"05VA4CZ"  =  "1"  /* Restrict L Brach Vein w Extralum Dev, Perc Endo */
"05VA4DZ"  =  "1"  /* Restrict L Brach Vein w Intralum Dev, Perc Endo */
"05VA4ZZ"  =  "1"  /* Restriction of Left Brachial Vein, Perc Endo Approach */
"05VB0CZ"  =  "1"  /* Restrict of R Basilic Vein with Extralum Dev, Open Approach */
"05VB0DZ"  =  "1"  /* Restrict of R Basilic Vein with Intralum Dev, Open Approach */
"05VB0ZZ"  =  "1"  /* Restriction of Right Basilic Vein, Open Approach */
"05VB3CZ"  =  "1"  /* Restrict of R Basilic Vein with Extralum Dev, Perc Approach */
"05VB3DZ"  =  "1"  /* Restrict of R Basilic Vein with Intralum Dev, Perc Approach */
"05VB3ZZ"  =  "1"  /* Restriction of Right Basilic Vein, Percutaneous Approach */
"05VB4CZ"  =  "1"  /* Restrict R Basilic Vein w Extralum Dev, Perc Endo */
"05VB4DZ"  =  "1"  /* Restrict R Basilic Vein w Intralum Dev, Perc Endo */
"05VB4ZZ"  =  "1"  /* Restriction of Right Basilic Vein, Perc Endo Approach */
"05VC0CZ"  =  "1"  /* Restrict of L Basilic Vein with Extralum Dev, Open Approach */
"05VC0DZ"  =  "1"  /* Restrict of L Basilic Vein with Intralum Dev, Open Approach */
"05VC0ZZ"  =  "1"  /* Restriction of Left Basilic Vein, Open Approach */
"05VC3CZ"  =  "1"  /* Restrict of L Basilic Vein with Extralum Dev, Perc Approach */
"05VC3DZ"  =  "1"  /* Restrict of L Basilic Vein with Intralum Dev, Perc Approach */
"05VC3ZZ"  =  "1"  /* Restriction of Left Basilic Vein, Percutaneous Approach */
"05VC4CZ"  =  "1"  /* Restrict L Basilic Vein w Extralum Dev, Perc Endo */
"05VC4DZ"  =  "1"  /* Restrict L Basilic Vein w Intralum Dev, Perc Endo */
"05VC4ZZ"  =  "1"  /* Restriction of Left Basilic Vein, Perc Endo Approach */
"05VD0CZ"  =  "1"  /* Restrict of R Cephalic Vein with Extralum Dev, Open Approach */
"05VD0DZ"  =  "1"  /* Restrict of R Cephalic Vein with Intralum Dev, Open Approach */
"05VD0ZZ"  =  "1"  /* Restriction of Right Cephalic Vein, Open Approach */
"05VD3CZ"  =  "1"  /* Restrict of R Cephalic Vein with Extralum Dev, Perc Approach */
"05VD3DZ"  =  "1"  /* Restrict of R Cephalic Vein with Intralum Dev, Perc Approach */
"05VD3ZZ"  =  "1"  /* Restriction of Right Cephalic Vein, Percutaneous Approach */
"05VD4CZ"  =  "1"  /* Restrict R Cephalic Vein w Extralum Dev, Perc Endo */
"05VD4DZ"  =  "1"  /* Restrict R Cephalic Vein w Intralum Dev, Perc Endo */
"05VD4ZZ"  =  "1"  /* Restriction of Right Cephalic Vein, Perc Endo Approach */
"05VF0CZ"  =  "1"  /* Restrict of L Cephalic Vein with Extralum Dev, Open Approach */
"05VF0DZ"  =  "1"  /* Restrict of L Cephalic Vein with Intralum Dev, Open Approach */
"05VF0ZZ"  =  "1"  /* Restriction of Left Cephalic Vein, Open Approach */
"05VF3CZ"  =  "1"  /* Restrict of L Cephalic Vein with Extralum Dev, Perc Approach */
"05VF3DZ"  =  "1"  /* Restrict of L Cephalic Vein with Intralum Dev, Perc Approach */
"05VF3ZZ"  =  "1"  /* Restriction of Left Cephalic Vein, Percutaneous Approach */
"05VF4CZ"  =  "1"  /* Restrict L Cephalic Vein w Extralum Dev, Perc Endo */
"05VF4DZ"  =  "1"  /* Restrict L Cephalic Vein w Intralum Dev, Perc Endo */
"05VF4ZZ"  =  "1"  /* Restriction of Left Cephalic Vein, Perc Endo Approach */
"05VG0CZ"  =  "1"  /* Restriction of R Hand Vein with Extralum Dev, Open Approach */
"05VG0DZ"  =  "1"  /* Restriction of R Hand Vein with Intralum Dev, Open Approach */
"05VG0ZZ"  =  "1"  /* Restriction of Right Hand Vein, Open Approach */
"05VG3CZ"  =  "1"  /* Restriction of R Hand Vein with Extralum Dev, Perc Approach */
"05VG3DZ"  =  "1"  /* Restriction of R Hand Vein with Intralum Dev, Perc Approach */
"05VG3ZZ"  =  "1"  /* Restriction of Right Hand Vein, Percutaneous Approach */
"05VG4CZ"  =  "1"  /* Restrict R Hand Vein w Extralum Dev, Perc Endo */
"05VG4DZ"  =  "1"  /* Restrict R Hand Vein w Intralum Dev, Perc Endo */
"05VG4ZZ"  =  "1"  /* Restriction of Right Hand Vein, Perc Endo Approach */
"05VH0CZ"  =  "1"  /* Restriction of L Hand Vein with Extralum Dev, Open Approach */
"05VH0DZ"  =  "1"  /* Restriction of L Hand Vein with Intralum Dev, Open Approach */
"05VH0ZZ"  =  "1"  /* Restriction of Left Hand Vein, Open Approach */
"05VH3CZ"  =  "1"  /* Restriction of L Hand Vein with Extralum Dev, Perc Approach */
"05VH3DZ"  =  "1"  /* Restriction of L Hand Vein with Intralum Dev, Perc Approach */
"05VH3ZZ"  =  "1"  /* Restriction of Left Hand Vein, Percutaneous Approach */
"05VH4CZ"  =  "1"  /* Restrict L Hand Vein w Extralum Dev, Perc Endo */
"05VH4DZ"  =  "1"  /* Restrict L Hand Vein w Intralum Dev, Perc Endo */
"05VH4ZZ"  =  "1"  /* Restriction of Left Hand Vein, Perc Endo Approach */
"05VL0CZ"  =  "1"  /* Restrict of Intracran Vein with Extralum Dev, Open Approach */
"05VL0DZ"  =  "1"  /* Restrict of Intracran Vein with Intralum Dev, Open Approach */
"05VL0ZZ"  =  "1"  /* Restriction of Intracranial Vein, Open Approach */
"05VL3CZ"  =  "1"  /* Restrict of Intracran Vein with Extralum Dev, Perc Approach */
"05VL3DZ"  =  "1"  /* Restrict of Intracran Vein with Intralum Dev, Perc Approach */
"05VL3ZZ"  =  "1"  /* Restriction of Intracranial Vein, Percutaneous Approach */
"05VL4CZ"  =  "1"  /* Restrict Intracran Vein w Extralum Dev, Perc Endo */
"05VL4DZ"  =  "1"  /* Restrict Intracran Vein w Intralum Dev, Perc Endo */
"05VL4ZZ"  =  "1"  /* Restriction of Intracranial Vein, Perc Endo Approach */
"05VM0CZ"  =  "1"  /* Restrict R Int Jugular Vein w Extralum Dev, Open */
"05VM0DZ"  =  "1"  /* Restrict R Int Jugular Vein w Intralum Dev, Open */
"05VM0ZZ"  =  "1"  /* Restriction of Right Internal Jugular Vein, Open Approach */
"05VM3CZ"  =  "1"  /* Restrict R Int Jugular Vein w Extralum Dev, Perc */
"05VM3DZ"  =  "1"  /* Restrict R Int Jugular Vein w Intralum Dev, Perc */
"05VM3ZZ"  =  "1"  /* Restriction of Right Internal Jugular Vein, Perc Approach */
"05VM4CZ"  =  "1"  /* Restrict R Int Jugular Vein w Extralum Dev, Perc Endo */
"05VM4DZ"  =  "1"  /* Restrict R Int Jugular Vein w Intralum Dev, Perc Endo */
"05VM4ZZ"  =  "1"  /* Restriction of R Int Jugular Vein, Perc Endo Approach */
"05VN0CZ"  =  "1"  /* Restrict L Int Jugular Vein w Extralum Dev, Open */
"05VN0DZ"  =  "1"  /* Restrict L Int Jugular Vein w Intralum Dev, Open */
"05VN0ZZ"  =  "1"  /* Restriction of Left Internal Jugular Vein, Open Approach */
"05VN3CZ"  =  "1"  /* Restrict L Int Jugular Vein w Extralum Dev, Perc */
"05VN3DZ"  =  "1"  /* Restrict L Int Jugular Vein w Intralum Dev, Perc */
"05VN3ZZ"  =  "1"  /* Restriction of Left Internal Jugular Vein, Perc Approach */
"05VN4CZ"  =  "1"  /* Restrict L Int Jugular Vein w Extralum Dev, Perc Endo */
"05VN4DZ"  =  "1"  /* Restrict L Int Jugular Vein w Intralum Dev, Perc Endo */
"05VN4ZZ"  =  "1"  /* Restriction of L Int Jugular Vein, Perc Endo Approach */
"05VP0CZ"  =  "1"  /* Restrict R Ext Jugular Vein w Extralum Dev, Open */
"05VP0DZ"  =  "1"  /* Restrict R Ext Jugular Vein w Intralum Dev, Open */
"05VP0ZZ"  =  "1"  /* Restriction of Right External Jugular Vein, Open Approach */
"05VP3CZ"  =  "1"  /* Restrict R Ext Jugular Vein w Extralum Dev, Perc */
"05VP3DZ"  =  "1"  /* Restrict R Ext Jugular Vein w Intralum Dev, Perc */
"05VP3ZZ"  =  "1"  /* Restriction of Right External Jugular Vein, Perc Approach */
"05VP4CZ"  =  "1"  /* Restrict R Ext Jugular Vein w Extralum Dev, Perc Endo */
"05VP4DZ"  =  "1"  /* Restrict R Ext Jugular Vein w Intralum Dev, Perc Endo */
"05VP4ZZ"  =  "1"  /* Restriction of R Ext Jugular Vein, Perc Endo Approach */
"05VQ0CZ"  =  "1"  /* Restrict L Ext Jugular Vein w Extralum Dev, Open */
"05VQ0DZ"  =  "1"  /* Restrict L Ext Jugular Vein w Intralum Dev, Open */
"05VQ0ZZ"  =  "1"  /* Restriction of Left External Jugular Vein, Open Approach */
"05VQ3CZ"  =  "1"  /* Restrict L Ext Jugular Vein w Extralum Dev, Perc */
"05VQ3DZ"  =  "1"  /* Restrict L Ext Jugular Vein w Intralum Dev, Perc */
"05VQ3ZZ"  =  "1"  /* Restriction of Left External Jugular Vein, Perc Approach */
"05VQ4CZ"  =  "1"  /* Restrict L Ext Jugular Vein w Extralum Dev, Perc Endo */
"05VQ4DZ"  =  "1"  /* Restrict L Ext Jugular Vein w Intralum Dev, Perc Endo */
"05VQ4ZZ"  =  "1"  /* Restriction of L Ext Jugular Vein, Perc Endo Approach */
"05VR0CZ"  =  "1"  /* Restrict of R Verteb Vein with Extralum Dev, Open Approach */
"05VR0DZ"  =  "1"  /* Restrict of R Verteb Vein with Intralum Dev, Open Approach */
"05VR0ZZ"  =  "1"  /* Restriction of Right Vertebral Vein, Open Approach */
"05VR3CZ"  =  "1"  /* Restrict of R Verteb Vein with Extralum Dev, Perc Approach */
"05VR3DZ"  =  "1"  /* Restrict of R Verteb Vein with Intralum Dev, Perc Approach */
"05VR3ZZ"  =  "1"  /* Restriction of Right Vertebral Vein, Percutaneous Approach */
"05VR4CZ"  =  "1"  /* Restrict R Verteb Vein w Extralum Dev, Perc Endo */
"05VR4DZ"  =  "1"  /* Restrict R Verteb Vein w Intralum Dev, Perc Endo */
"05VR4ZZ"  =  "1"  /* Restriction of Right Vertebral Vein, Perc Endo Approach */
"05VS0CZ"  =  "1"  /* Restrict of L Verteb Vein with Extralum Dev, Open Approach */
"05VS0DZ"  =  "1"  /* Restrict of L Verteb Vein with Intralum Dev, Open Approach */
"05VS0ZZ"  =  "1"  /* Restriction of Left Vertebral Vein, Open Approach */
"05VS3CZ"  =  "1"  /* Restrict of L Verteb Vein with Extralum Dev, Perc Approach */
"05VS3DZ"  =  "1"  /* Restrict of L Verteb Vein with Intralum Dev, Perc Approach */
"05VS3ZZ"  =  "1"  /* Restriction of Left Vertebral Vein, Percutaneous Approach */
"05VS4CZ"  =  "1"  /* Restrict L Verteb Vein w Extralum Dev, Perc Endo */
"05VS4DZ"  =  "1"  /* Restrict L Verteb Vein w Intralum Dev, Perc Endo */
"05VS4ZZ"  =  "1"  /* Restriction of Left Vertebral Vein, Perc Endo Approach */
"05VT0CZ"  =  "1"  /* Restriction of R Face Vein with Extralum Dev, Open Approach */
"05VT0DZ"  =  "1"  /* Restriction of R Face Vein with Intralum Dev, Open Approach */
"05VT0ZZ"  =  "1"  /* Restriction of Right Face Vein, Open Approach */
"05VT3CZ"  =  "1"  /* Restriction of R Face Vein with Extralum Dev, Perc Approach */
"05VT3DZ"  =  "1"  /* Restriction of R Face Vein with Intralum Dev, Perc Approach */
"05VT3ZZ"  =  "1"  /* Restriction of Right Face Vein, Percutaneous Approach */
"05VT4CZ"  =  "1"  /* Restrict R Face Vein w Extralum Dev, Perc Endo */
"05VT4DZ"  =  "1"  /* Restrict R Face Vein w Intralum Dev, Perc Endo */
"05VT4ZZ"  =  "1"  /* Restriction of Right Face Vein, Perc Endo Approach */
"05VV0CZ"  =  "1"  /* Restriction of L Face Vein with Extralum Dev, Open Approach */
"05VV0DZ"  =  "1"  /* Restriction of L Face Vein with Intralum Dev, Open Approach */
"05VV0ZZ"  =  "1"  /* Restriction of Left Face Vein, Open Approach */
"05VV3CZ"  =  "1"  /* Restriction of L Face Vein with Extralum Dev, Perc Approach */
"05VV3DZ"  =  "1"  /* Restriction of L Face Vein with Intralum Dev, Perc Approach */
"05VV3ZZ"  =  "1"  /* Restriction of Left Face Vein, Percutaneous Approach */
"05VV4CZ"  =  "1"  /* Restrict L Face Vein w Extralum Dev, Perc Endo */
"05VV4DZ"  =  "1"  /* Restrict L Face Vein w Intralum Dev, Perc Endo */
"05VV4ZZ"  =  "1"  /* Restriction of Left Face Vein, Perc Endo Approach */
"05VY0CZ"  =  "1"  /* Restriction of Upper Vein with Extralum Dev, Open Approach */
"05VY0DZ"  =  "1"  /* Restriction of Upper Vein with Intralum Dev, Open Approach */
"05VY0ZZ"  =  "1"  /* Restriction of Upper Vein, Open Approach */
"05VY3CZ"  =  "1"  /* Restriction of Upper Vein with Extralum Dev, Perc Approach */
"05VY3DZ"  =  "1"  /* Restriction of Upper Vein with Intralum Dev, Perc Approach */
"05VY3ZZ"  =  "1"  /* Restriction of Upper Vein, Percutaneous Approach */
"05VY4CZ"  =  "1"  /* Restriction of Up Vein with Extralum Dev, Perc Endo Approach */
"05VY4DZ"  =  "1"  /* Restriction of Up Vein with Intralum Dev, Perc Endo Approach */
"05VY4ZZ"  =  "1"  /* Restriction of Upper Vein, Percutaneous Endoscopic Approach */
"05WY00Z"  =  "1"  /* Revision of Drainage Device in Upper Vein, Open Approach */
"05WY02Z"  =  "1"  /* Revision of Monitoring Device in Upper Vein, Open Approach */
"05WY03Z"  =  "1"  /* Revision of Infusion Device in Upper Vein, Open Approach */
"05WY07Z"  =  "1"  /* Revision of Autol Sub in Up Vein, Open Approach */
"05WY0CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Vein, Open Approach */
"05WY0DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Vein, Open Approach */
"05WY0JZ"  =  "1"  /* Revision of Synthetic Substitute in Up Vein, Open Approach */
"05WY0KZ"  =  "1"  /* Revision of Nonaut Sub in Up Vein, Open Approach */
"05WY30Z"  =  "1"  /* Revision of Drainage Device in Upper Vein, Perc Approach */
"05WY32Z"  =  "1"  /* Revision of Monitoring Device in Upper Vein, Perc Approach */
"05WY33Z"  =  "1"  /* Revision of Infusion Device in Upper Vein, Perc Approach */
"05WY37Z"  =  "1"  /* Revision of Autol Sub in Up Vein, Perc Approach */
"05WY3CZ"  =  "1"  /* Revision of Extraluminal Device in Upper Vein, Perc Approach */
"05WY3DZ"  =  "1"  /* Revision of Intraluminal Device in Upper Vein, Perc Approach */
"05WY3JZ"  =  "1"  /* Revision of Synthetic Substitute in Up Vein, Perc Approach */
"05WY3KZ"  =  "1"  /* Revision of Nonaut Sub in Up Vein, Perc Approach */
"05WY40Z"  =  "1"  /* Revision of Drainage Device in Up Vein, Perc Endo Approach */
"05WY42Z"  =  "1"  /* Revision of Monitoring Device in Up Vein, Perc Endo Approach */
"05WY43Z"  =  "1"  /* Revision of Infusion Device in Up Vein, Perc Endo Approach */
"05WY47Z"  =  "1"  /* Revision of Autol Sub in Up Vein, Perc Endo Approach */
"05WY4CZ"  =  "1"  /* Revision of Extralum Dev in Up Vein, Perc Endo Approach */
"05WY4DZ"  =  "1"  /* Revision of Intralum Dev in Up Vein, Perc Endo Approach */
"05WY4JZ"  =  "1"  /* Revision of Synth Sub in Up Vein, Perc Endo Approach */
"05WY4KZ"  =  "1"  /* Revision of Nonaut Sub in Up Vein, Perc Endo Approach */
"0610075"  =  "1"  /* Bypass Inf Vena Cava to Sup Mesent V w Autol Sub, Open */
"0610076"  =  "1"  /* Bypass Inf Vena Cava to Inf Mesent V w Autol Sub, Open */
"061007Y"  =  "1"  /* Bypass Inf Vena Cava to Low Vein w Autol Sub, Open */
"0610095"  =  "1"  /* Bypass Inf Vena Cava to Sup Mesent V w Autol Vn, Open */
"0610096"  =  "1"  /* Bypass Inf Vena Cava to Inf Mesent V w Autol Vn, Open */
"061009Y"  =  "1"  /* Bypass Inf Vena Cava to Low Vein w Autol Vn, Open */
"06100A5"  =  "1"  /* Bypass Inf Vena Cava to Sup Mesent V w Autol Art, Open */
"06100A6"  =  "1"  /* Bypass Inf Vena Cava to Inf Mesent V w Autol Art, Open */
"06100AY"  =  "1"  /* Bypass Inf Vena Cava to Low Vein w Autol Art, Open */
"06100J5"  =  "1"  /* Bypass Inf Vena Cava to Sup Mesent V w Synth Sub, Open */
"06100J6"  =  "1"  /* Bypass Inf Vena Cava to Inf Mesent V w Synth Sub, Open */
"06100JY"  =  "1"  /* Bypass Inf Vena Cava to Low Vein w Synth Sub, Open */
"06100K5"  =  "1"  /* Bypass Inf Vena Cava to Sup Mesent V w Nonaut Sub, Open */
"06100K6"  =  "1"  /* Bypass Inf Vena Cava to Inf Mesent V w Nonaut Sub, Open */
"06100KY"  =  "1"  /* Bypass Inf Vena Cava to Low Vein w Nonaut Sub, Open */
"06100Z5"  =  "1"  /* Bypass Inferior Vena Cava to Sup Mesent V, Open Approach */
"06100Z6"  =  "1"  /* Bypass Inferior Vena Cava to Inf Mesent V, Open Approach */
"06100ZY"  =  "1"  /* Bypass Inferior Vena Cava to Lower Vein, Open Approach */
"0610475"  =  "1"  /* Bypass Inf Vena Cava to Sup Mesent V w Autol Sub, Perc Endo */
"0610476"  =  "1"  /* Bypass Inf Vena Cava to Inf Mesent V w Autol Sub, Perc Endo */
"061047Y"  =  "1"  /* Bypass Inf Vena Cava to Low Vein w Autol Sub, Perc Endo */
"0610495"  =  "1"  /* Bypass Inf Vena Cava to Sup Mesent V w Autol Vn, Perc Endo */
"0610496"  =  "1"  /* Bypass Inf Vena Cava to Inf Mesent V w Autol Vn, Perc Endo */
"061049Y"  =  "1"  /* Bypass Inf Vena Cava to Low Vein w Autol Vn, Perc Endo */
"06104A5"  =  "1"  /* Bypass Inf Vena Cava to Sup Mesent V w Autol Art, Perc Endo */
"06104A6"  =  "1"  /* Bypass Inf Vena Cava to Inf Mesent V w Autol Art, Perc Endo */
"06104AY"  =  "1"  /* Bypass Inf Vena Cava to Low Vein w Autol Art, Perc Endo */
"06104J5"  =  "1"  /* Bypass Inf Vena Cava to Sup Mesent V w Synth Sub, Perc Endo */
"06104J6"  =  "1"  /* Bypass Inf Vena Cava to Inf Mesent V w Synth Sub, Perc Endo */
"06104JY"  =  "1"  /* Bypass Inf Vena Cava to Low Vein w Synth Sub, Perc Endo */
"06104K5"  =  "1"  /* Bypass Inf Vena Cava to Sup Mesent V w Nonaut Sub, Perc Endo */
"06104K6"  =  "1"  /* Bypass Inf Vena Cava to Inf Mesent V w Nonaut Sub, Perc Endo */
"06104KY"  =  "1"  /* Bypass Inf Vena Cava to Low Vein w Nonaut Sub, Perc Endo */
"06104Z5"  =  "1"  /* Bypass Inf Vena Cava to Sup Mesent V, Perc Endo Approach */
"06104Z6"  =  "1"  /* Bypass Inf Vena Cava to Inf Mesent V, Perc Endo Approach */
"06104ZY"  =  "1"  /* Bypass Inferior Vena Cava to Lower Vein, Perc Endo Approach */
"0611079"  =  "1"  /* Bypass Splenic Vein to R Renal Vein w Autol Sub, Open */
"061107B"  =  "1"  /* Bypass Splenic Vein to L Renal Vein w Autol Sub, Open */
"061107Y"  =  "1"  /* Bypass Splenic Vein to Low Vein w Autol Sub, Open */
"0611099"  =  "1"  /* Bypass Splenic Vein to R Renal Vein w Autol Vn, Open */
"061109B"  =  "1"  /* Bypass Splenic Vein to L Renal Vein w Autol Vn, Open */
"061109Y"  =  "1"  /* Bypass Splenic Vein to Low Vein with Autol Vn, Open Approach */
"06110A9"  =  "1"  /* Bypass Splenic Vein to R Renal Vein w Autol Art, Open */
"06110AB"  =  "1"  /* Bypass Splenic Vein to L Renal Vein w Autol Art, Open */
"06110AY"  =  "1"  /* Bypass Splenic Vein to Low Vein w Autol Art, Open */
"06110J9"  =  "1"  /* Bypass Splenic Vein to R Renal Vein w Synth Sub, Open */
"06110JB"  =  "1"  /* Bypass Splenic Vein to L Renal Vein w Synth Sub, Open */
"06110JY"  =  "1"  /* Bypass Splenic Vein to Low Vein w Synth Sub, Open */
"06110K9"  =  "1"  /* Bypass Splenic Vein to R Renal Vein w Nonaut Sub, Open */
"06110KB"  =  "1"  /* Bypass Splenic Vein to L Renal Vein w Nonaut Sub, Open */
"06110KY"  =  "1"  /* Bypass Splenic Vein to Low Vein w Nonaut Sub, Open */
"06110Z9"  =  "1"  /* Bypass Splenic Vein to Right Renal Vein, Open Approach */
"06110ZB"  =  "1"  /* Bypass Splenic Vein to Left Renal Vein, Open Approach */
"06110ZY"  =  "1"  /* Bypass Splenic Vein to Lower Vein, Open Approach */
"0611479"  =  "1"  /* Bypass Splenic Vein to R Renal Vein w Autol Sub, Perc Endo */
"061147B"  =  "1"  /* Bypass Splenic Vein to L Renal Vein w Autol Sub, Perc Endo */
"061147Y"  =  "1"  /* Bypass Splenic Vein to Low Vein w Autol Sub, Perc Endo */
"0611499"  =  "1"  /* Bypass Splenic Vein to R Renal Vein w Autol Vn, Perc Endo */
"061149B"  =  "1"  /* Bypass Splenic Vein to L Renal Vein w Autol Vn, Perc Endo */
"061149Y"  =  "1"  /* Bypass Splenic Vein to Low Vein w Autol Vn, Perc Endo */
"06114A9"  =  "1"  /* Bypass Splenic Vein to R Renal Vein w Autol Art, Perc Endo */
"06114AB"  =  "1"  /* Bypass Splenic Vein to L Renal Vein w Autol Art, Perc Endo */
"06114AY"  =  "1"  /* Bypass Splenic Vein to Low Vein w Autol Art, Perc Endo */
"06114J9"  =  "1"  /* Bypass Splenic Vein to R Renal Vein w Synth Sub, Perc Endo */
"06114JB"  =  "1"  /* Bypass Splenic Vein to L Renal Vein w Synth Sub, Perc Endo */
"06114JY"  =  "1"  /* Bypass Splenic Vein to Low Vein w Synth Sub, Perc Endo */
"06114K9"  =  "1"  /* Bypass Splenic Vein to R Renal Vein w Nonaut Sub, Perc Endo */
"06114KB"  =  "1"  /* Bypass Splenic Vein to L Renal Vein w Nonaut Sub, Perc Endo */
"06114KY"  =  "1"  /* Bypass Splenic Vein to Low Vein w Nonaut Sub, Perc Endo */
"06114Z9"  =  "1"  /* Bypass Splenic Vein to Right Renal Vein, Perc Endo Approach */
"06114ZB"  =  "1"  /* Bypass Splenic Vein to Left Renal Vein, Perc Endo Approach */
"06114ZY"  =  "1"  /* Bypass Splenic Vein to Lower Vein, Perc Endo Approach */
"061207Y"  =  "1"  /* Bypass Gastric Vein to Low Vein w Autol Sub, Open */
"061209Y"  =  "1"  /* Bypass Gastric Vein to Low Vein with Autol Vn, Open Approach */
"06120AY"  =  "1"  /* Bypass Gastric Vein to Low Vein w Autol Art, Open */
"06120JY"  =  "1"  /* Bypass Gastric Vein to Low Vein w Synth Sub, Open */
"06120KY"  =  "1"  /* Bypass Gastric Vein to Low Vein w Nonaut Sub, Open */
"06120ZY"  =  "1"  /* Bypass Gastric Vein to Lower Vein, Open Approach */
"061247Y"  =  "1"  /* Bypass Gastric Vein to Low Vein w Autol Sub, Perc Endo */
"061249Y"  =  "1"  /* Bypass Gastric Vein to Low Vein w Autol Vn, Perc Endo */
"06124AY"  =  "1"  /* Bypass Gastric Vein to Low Vein w Autol Art, Perc Endo */
"06124JY"  =  "1"  /* Bypass Gastric Vein to Low Vein w Synth Sub, Perc Endo */
"06124KY"  =  "1"  /* Bypass Gastric Vein to Low Vein w Nonaut Sub, Perc Endo */
"06124ZY"  =  "1"  /* Bypass Gastric Vein to Lower Vein, Perc Endo Approach */
"061307Y"  =  "1"  /* Bypass Esophageal Vein to Low Vein w Autol Sub, Open */
"061309Y"  =  "1"  /* Bypass Esophageal Vein to Low Vein w Autol Vn, Open */
"06130AY"  =  "1"  /* Bypass Esophageal Vein to Low Vein w Autol Art, Open */
"06130JY"  =  "1"  /* Bypass Esophageal Vein to Low Vein w Synth Sub, Open */
"06130KY"  =  "1"  /* Bypass Esophageal Vein to Low Vein w Nonaut Sub, Open */
"06130ZY"  =  "1"  /* Bypass Esophageal Vein to Lower Vein, Open Approach */
"061347Y"  =  "1"  /* Bypass Esophageal Vein to Low Vein w Autol Sub, Perc Endo */
"061349Y"  =  "1"  /* Bypass Esophageal Vein to Low Vein w Autol Vn, Perc Endo */
"06134AY"  =  "1"  /* Bypass Esophageal Vein to Low Vein w Autol Art, Perc Endo */
"06134JY"  =  "1"  /* Bypass Esophageal Vein to Low Vein w Synth Sub, Perc Endo */
"06134KY"  =  "1"  /* Bypass Esophageal Vein to Low Vein w Nonaut Sub, Perc Endo */
"06134ZY"  =  "1"  /* Bypass Esophageal Vein to Lower Vein, Perc Endo Approach */
"061407Y"  =  "1"  /* Bypass Hepatic Vein to Low Vein w Autol Sub, Open */
"061409Y"  =  "1"  /* Bypass Hepatic Vein to Low Vein with Autol Vn, Open Approach */
"06140AY"  =  "1"  /* Bypass Hepatic Vein to Low Vein w Autol Art, Open */
"06140JY"  =  "1"  /* Bypass Hepatic Vein to Low Vein w Synth Sub, Open */
"06140KY"  =  "1"  /* Bypass Hepatic Vein to Low Vein w Nonaut Sub, Open */
"06140ZY"  =  "1"  /* Bypass Hepatic Vein to Lower Vein, Open Approach */
"061447Y"  =  "1"  /* Bypass Hepatic Vein to Low Vein w Autol Sub, Perc Endo */
"061449Y"  =  "1"  /* Bypass Hepatic Vein to Low Vein w Autol Vn, Perc Endo */
"06144AY"  =  "1"  /* Bypass Hepatic Vein to Low Vein w Autol Art, Perc Endo */
"06144JY"  =  "1"  /* Bypass Hepatic Vein to Low Vein w Synth Sub, Perc Endo */
"06144KY"  =  "1"  /* Bypass Hepatic Vein to Low Vein w Nonaut Sub, Perc Endo */
"06144ZY"  =  "1"  /* Bypass Hepatic Vein to Lower Vein, Perc Endo Approach */
"061507Y"  =  "1"  /* Bypass Sup Mesent Vein to Low Vein w Autol Sub, Open */
"061509Y"  =  "1"  /* Bypass Sup Mesent Vein to Low Vein w Autol Vn, Open */
"06150AY"  =  "1"  /* Bypass Sup Mesent Vein to Low Vein w Autol Art, Open */
"06150JY"  =  "1"  /* Bypass Sup Mesent Vein to Low Vein w Synth Sub, Open */
"06150KY"  =  "1"  /* Bypass Sup Mesent Vein to Low Vein w Nonaut Sub, Open */
"06150ZY"  =  "1"  /* Bypass Superior Mesenteric Vein to Lower Vein, Open Approach */
"061547Y"  =  "1"  /* Bypass Sup Mesent Vein to Low Vein w Autol Sub, Perc Endo */
"061549Y"  =  "1"  /* Bypass Sup Mesent Vein to Low Vein w Autol Vn, Perc Endo */
"06154AY"  =  "1"  /* Bypass Sup Mesent Vein to Low Vein w Autol Art, Perc Endo */
"06154JY"  =  "1"  /* Bypass Sup Mesent Vein to Low Vein w Synth Sub, Perc Endo */
"06154KY"  =  "1"  /* Bypass Sup Mesent Vein to Low Vein w Nonaut Sub, Perc Endo */
"06154ZY"  =  "1"  /* Bypass Sup Mesent Vein to Low Vein, Perc Endo Approach */
"061607Y"  =  "1"  /* Bypass Inf Mesent Vein to Low Vein w Autol Sub, Open */
"061609Y"  =  "1"  /* Bypass Inf Mesent Vein to Low Vein w Autol Vn, Open */
"06160AY"  =  "1"  /* Bypass Inf Mesent Vein to Low Vein w Autol Art, Open */
"06160JY"  =  "1"  /* Bypass Inf Mesent Vein to Low Vein w Synth Sub, Open */
"06160KY"  =  "1"  /* Bypass Inf Mesent Vein to Low Vein w Nonaut Sub, Open */
"06160ZY"  =  "1"  /* Bypass Inferior Mesenteric Vein to Lower Vein, Open Approach */
"061647Y"  =  "1"  /* Bypass Inf Mesent Vein to Low Vein w Autol Sub, Perc Endo */
"061649Y"  =  "1"  /* Bypass Inf Mesent Vein to Low Vein w Autol Vn, Perc Endo */
"06164AY"  =  "1"  /* Bypass Inf Mesent Vein to Low Vein w Autol Art, Perc Endo */
"06164JY"  =  "1"  /* Bypass Inf Mesent Vein to Low Vein w Synth Sub, Perc Endo */
"06164KY"  =  "1"  /* Bypass Inf Mesent Vein to Low Vein w Nonaut Sub, Perc Endo */
"06164ZY"  =  "1"  /* Bypass Inf Mesent Vein to Low Vein, Perc Endo Approach */
"061707Y"  =  "1"  /* Bypass Colic Vein to Low Vein with Autol Sub, Open Approach */
"061709Y"  =  "1"  /* Bypass Colic Vein to Lower Vein with Autol Vn, Open Approach */
"06170AY"  =  "1"  /* Bypass Colic Vein to Low Vein with Autol Art, Open Approach */
"06170JY"  =  "1"  /* Bypass Colic Vein to Low Vein with Synth Sub, Open Approach */
"06170KY"  =  "1"  /* Bypass Colic Vein to Low Vein with Nonaut Sub, Open Approach */
"06170ZY"  =  "1"  /* Bypass Colic Vein to Lower Vein, Open Approach */
"061747Y"  =  "1"  /* Bypass Colic Vein to Low Vein w Autol Sub, Perc Endo */
"061749Y"  =  "1"  /* Bypass Colic Vein to Low Vein w Autol Vn, Perc Endo */
"06174AY"  =  "1"  /* Bypass Colic Vein to Low Vein w Autol Art, Perc Endo */
"06174JY"  =  "1"  /* Bypass Colic Vein to Low Vein w Synth Sub, Perc Endo */
"06174KY"  =  "1"  /* Bypass Colic Vein to Low Vein w Nonaut Sub, Perc Endo */
"06174ZY"  =  "1"  /* Bypass Colic Vein to Lower Vein, Perc Endo Approach */
"0618079"  =  "1"  /* Bypass Portal Vein to R Renal Vein w Autol Sub, Open */
"061807B"  =  "1"  /* Bypass Portal Vein to L Renal Vein w Autol Sub, Open */
"061807Y"  =  "1"  /* Bypass Portal Vein to Low Vein with Autol Sub, Open Approach */
"0618099"  =  "1"  /* Bypass Portal Vein to R Renal Vein w Autol Vn, Open */
"061809B"  =  "1"  /* Bypass Portal Vein to L Renal Vein w Autol Vn, Open */
"061809Y"  =  "1"  /* Bypass Portal Vein to Low Vein with Autol Vn, Open Approach */
"06180A9"  =  "1"  /* Bypass Portal Vein to R Renal Vein w Autol Art, Open */
"06180AB"  =  "1"  /* Bypass Portal Vein to L Renal Vein w Autol Art, Open */
"06180AY"  =  "1"  /* Bypass Portal Vein to Low Vein with Autol Art, Open Approach */
"06180J9"  =  "1"  /* Bypass Portal Vein to R Renal Vein w Synth Sub, Open */
"06180JB"  =  "1"  /* Bypass Portal Vein to L Renal Vein w Synth Sub, Open */
"06180JY"  =  "1"  /* Bypass Portal Vein to Low Vein with Synth Sub, Open Approach */
"06180K9"  =  "1"  /* Bypass Portal Vein to R Renal Vein w Nonaut Sub, Open */
"06180KB"  =  "1"  /* Bypass Portal Vein to L Renal Vein w Nonaut Sub, Open */
"06180KY"  =  "1"  /* Bypass Portal Vein to Low Vein w Nonaut Sub, Open */
"06180Z9"  =  "1"  /* Bypass Portal Vein to Right Renal Vein, Open Approach */
"06180ZB"  =  "1"  /* Bypass Portal Vein to Left Renal Vein, Open Approach */
"06180ZY"  =  "1"  /* Bypass Portal Vein to Lower Vein, Open Approach */
"06183DY"  =  "1"  /* Bypass Portal Vein to Low Vein w Intralum Dev, Perc */
"0618479"  =  "1"  /* Bypass Portal Vein to R Renal Vein w Autol Sub, Perc Endo */
"061847B"  =  "1"  /* Bypass Portal Vein to L Renal Vein w Autol Sub, Perc Endo */
"061847Y"  =  "1"  /* Bypass Portal Vein to Low Vein w Autol Sub, Perc Endo */
"0618499"  =  "1"  /* Bypass Portal Vein to R Renal Vein w Autol Vn, Perc Endo */
"061849B"  =  "1"  /* Bypass Portal Vein to L Renal Vein w Autol Vn, Perc Endo */
"061849Y"  =  "1"  /* Bypass Portal Vein to Low Vein w Autol Vn, Perc Endo */
"06184A9"  =  "1"  /* Bypass Portal Vein to R Renal Vein w Autol Art, Perc Endo */
"06184AB"  =  "1"  /* Bypass Portal Vein to L Renal Vein w Autol Art, Perc Endo */
"06184AY"  =  "1"  /* Bypass Portal Vein to Low Vein w Autol Art, Perc Endo */
"06184DY"  =  "1"  /* Bypass Portal Vein to Low Vein w Intralum Dev, Perc Endo */
"06184J9"  =  "1"  /* Bypass Portal Vein to R Renal Vein w Synth Sub, Perc Endo */
"06184JB"  =  "1"  /* Bypass Portal Vein to L Renal Vein w Synth Sub, Perc Endo */
"06184JY"  =  "1"  /* Bypass Portal Vein to Low Vein w Synth Sub, Perc Endo */
"06184K9"  =  "1"  /* Bypass Portal Vein to R Renal Vein w Nonaut Sub, Perc Endo */
"06184KB"  =  "1"  /* Bypass Portal Vein to L Renal Vein w Nonaut Sub, Perc Endo */
"06184KY"  =  "1"  /* Bypass Portal Vein to Low Vein w Nonaut Sub, Perc Endo */
"06184Z9"  =  "1"  /* Bypass Portal Vein to Right Renal Vein, Perc Endo Approach */
"06184ZB"  =  "1"  /* Bypass Portal Vein to Left Renal Vein, Perc Endo Approach */
"06184ZY"  =  "1"  /* Bypass Portal Vein to Lower Vein, Perc Endo Approach */
"061907Y"  =  "1"  /* Bypass R Renal Vein to Low Vein w Autol Sub, Open */
"061909Y"  =  "1"  /* Bypass R Renal Vein to Low Vein with Autol Vn, Open Approach */
"06190AY"  =  "1"  /* Bypass R Renal Vein to Low Vein w Autol Art, Open */
"06190JY"  =  "1"  /* Bypass R Renal Vein to Low Vein w Synth Sub, Open */
"06190KY"  =  "1"  /* Bypass R Renal Vein to Low Vein w Nonaut Sub, Open */
"06190ZY"  =  "1"  /* Bypass Right Renal Vein to Lower Vein, Open Approach */
"061947Y"  =  "1"  /* Bypass R Renal Vein to Low Vein w Autol Sub, Perc Endo */
"061949Y"  =  "1"  /* Bypass R Renal Vein to Low Vein w Autol Vn, Perc Endo */
"06194AY"  =  "1"  /* Bypass R Renal Vein to Low Vein w Autol Art, Perc Endo */
"06194JY"  =  "1"  /* Bypass R Renal Vein to Low Vein w Synth Sub, Perc Endo */
"06194KY"  =  "1"  /* Bypass R Renal Vein to Low Vein w Nonaut Sub, Perc Endo */
"06194ZY"  =  "1"  /* Bypass Right Renal Vein to Lower Vein, Perc Endo Approach */
"061B07Y"  =  "1"  /* Bypass L Renal Vein to Low Vein w Autol Sub, Open */
"061B09Y"  =  "1"  /* Bypass L Renal Vein to Low Vein with Autol Vn, Open Approach */
"061B0AY"  =  "1"  /* Bypass L Renal Vein to Low Vein w Autol Art, Open */
"061B0JY"  =  "1"  /* Bypass L Renal Vein to Low Vein w Synth Sub, Open */
"061B0KY"  =  "1"  /* Bypass L Renal Vein to Low Vein w Nonaut Sub, Open */
"061B0ZY"  =  "1"  /* Bypass Left Renal Vein to Lower Vein, Open Approach */
"061B47Y"  =  "1"  /* Bypass L Renal Vein to Low Vein w Autol Sub, Perc Endo */
"061B49Y"  =  "1"  /* Bypass L Renal Vein to Low Vein w Autol Vn, Perc Endo */
"061B4AY"  =  "1"  /* Bypass L Renal Vein to Low Vein w Autol Art, Perc Endo */
"061B4JY"  =  "1"  /* Bypass L Renal Vein to Low Vein w Synth Sub, Perc Endo */
"061B4KY"  =  "1"  /* Bypass L Renal Vein to Low Vein w Nonaut Sub, Perc Endo */
"061B4ZY"  =  "1"  /* Bypass Left Renal Vein to Lower Vein, Perc Endo Approach */
"061C07Y"  =  "1"  /* Bypass R Com Iliac Vein to Low Vein w Autol Sub, Open */
"061C09Y"  =  "1"  /* Bypass R Com Iliac Vein to Low Vein w Autol Vn, Open */
"061C0AY"  =  "1"  /* Bypass R Com Iliac Vein to Low Vein w Autol Art, Open */
"061C0JY"  =  "1"  /* Bypass R Com Iliac Vein to Low Vein w Synth Sub, Open */
"061C0KY"  =  "1"  /* Bypass R Com Iliac Vein to Low Vein w Nonaut Sub, Open */
"061C0ZY"  =  "1"  /* Bypass Right Common Iliac Vein to Lower Vein, Open Approach */
"061C47Y"  =  "1"  /* Bypass R Com Iliac Vein to Low Vein w Autol Sub, Perc Endo */
"061C49Y"  =  "1"  /* Bypass R Com Iliac Vein to Low Vein w Autol Vn, Perc Endo */
"061C4AY"  =  "1"  /* Bypass R Com Iliac Vein to Low Vein w Autol Art, Perc Endo */
"061C4JY"  =  "1"  /* Bypass R Com Iliac Vein to Low Vein w Synth Sub, Perc Endo */
"061C4KY"  =  "1"  /* Bypass R Com Iliac Vein to Low Vein w Nonaut Sub, Perc Endo */
"061C4ZY"  =  "1"  /* Bypass R Com Iliac Vein to Low Vein, Perc Endo Approach */
"061D07Y"  =  "1"  /* Bypass L Com Iliac Vein to Low Vein w Autol Sub, Open */
"061D09Y"  =  "1"  /* Bypass L Com Iliac Vein to Low Vein w Autol Vn, Open */
"061D0AY"  =  "1"  /* Bypass L Com Iliac Vein to Low Vein w Autol Art, Open */
"061D0JY"  =  "1"  /* Bypass L Com Iliac Vein to Low Vein w Synth Sub, Open */
"061D0KY"  =  "1"  /* Bypass L Com Iliac Vein to Low Vein w Nonaut Sub, Open */
"061D0ZY"  =  "1"  /* Bypass Left Common Iliac Vein to Lower Vein, Open Approach */
"061D47Y"  =  "1"  /* Bypass L Com Iliac Vein to Low Vein w Autol Sub, Perc Endo */
"061D49Y"  =  "1"  /* Bypass L Com Iliac Vein to Low Vein w Autol Vn, Perc Endo */
"061D4AY"  =  "1"  /* Bypass L Com Iliac Vein to Low Vein w Autol Art, Perc Endo */
"061D4JY"  =  "1"  /* Bypass L Com Iliac Vein to Low Vein w Synth Sub, Perc Endo */
"061D4KY"  =  "1"  /* Bypass L Com Iliac Vein to Low Vein w Nonaut Sub, Perc Endo */
"061D4ZY"  =  "1"  /* Bypass L Com Iliac Vein to Low Vein, Perc Endo Approach */
"061F07Y"  =  "1"  /* Bypass R Ext Iliac Vein to Low Vein w Autol Sub, Open */
"061F09Y"  =  "1"  /* Bypass R Ext Iliac Vein to Low Vein w Autol Vn, Open */
"061F0AY"  =  "1"  /* Bypass R Ext Iliac Vein to Low Vein w Autol Art, Open */
"061F0JY"  =  "1"  /* Bypass R Ext Iliac Vein to Low Vein w Synth Sub, Open */
"061F0KY"  =  "1"  /* Bypass R Ext Iliac Vein to Low Vein w Nonaut Sub, Open */
"061F0ZY"  =  "1"  /* Bypass Right External Iliac Vein to Low Vein, Open Approach */
"061F47Y"  =  "1"  /* Bypass R Ext Iliac Vein to Low Vein w Autol Sub, Perc Endo */
"061F49Y"  =  "1"  /* Bypass R Ext Iliac Vein to Low Vein w Autol Vn, Perc Endo */
"061F4AY"  =  "1"  /* Bypass R Ext Iliac Vein to Low Vein w Autol Art, Perc Endo */
"061F4JY"  =  "1"  /* Bypass R Ext Iliac Vein to Low Vein w Synth Sub, Perc Endo */
"061F4KY"  =  "1"  /* Bypass R Ext Iliac Vein to Low Vein w Nonaut Sub, Perc Endo */
"061F4ZY"  =  "1"  /* Bypass R Ext Iliac Vein to Low Vein, Perc Endo Approach */
"061G07Y"  =  "1"  /* Bypass L Ext Iliac Vein to Low Vein w Autol Sub, Open */
"061G09Y"  =  "1"  /* Bypass L Ext Iliac Vein to Low Vein w Autol Vn, Open */
"061G0AY"  =  "1"  /* Bypass L Ext Iliac Vein to Low Vein w Autol Art, Open */
"061G0JY"  =  "1"  /* Bypass L Ext Iliac Vein to Low Vein w Synth Sub, Open */
"061G0KY"  =  "1"  /* Bypass L Ext Iliac Vein to Low Vein w Nonaut Sub, Open */
"061G0ZY"  =  "1"  /* Bypass Left External Iliac Vein to Lower Vein, Open Approach */
"061G47Y"  =  "1"  /* Bypass L Ext Iliac Vein to Low Vein w Autol Sub, Perc Endo */
"061G49Y"  =  "1"  /* Bypass L Ext Iliac Vein to Low Vein w Autol Vn, Perc Endo */
"061G4AY"  =  "1"  /* Bypass L Ext Iliac Vein to Low Vein w Autol Art, Perc Endo */
"061G4JY"  =  "1"  /* Bypass L Ext Iliac Vein to Low Vein w Synth Sub, Perc Endo */
"061G4KY"  =  "1"  /* Bypass L Ext Iliac Vein to Low Vein w Nonaut Sub, Perc Endo */
"061G4ZY"  =  "1"  /* Bypass L Ext Iliac Vein to Low Vein, Perc Endo Approach */
"061H07Y"  =  "1"  /* Bypass R Hypogast Vein to Low Vein w Autol Sub, Open */
"061H09Y"  =  "1"  /* Bypass R Hypogast Vein to Low Vein w Autol Vn, Open */
"061H0AY"  =  "1"  /* Bypass R Hypogast Vein to Low Vein w Autol Art, Open */
"061H0JY"  =  "1"  /* Bypass R Hypogast Vein to Low Vein w Synth Sub, Open */
"061H0KY"  =  "1"  /* Bypass R Hypogast Vein to Low Vein w Nonaut Sub, Open */
"061H0ZY"  =  "1"  /* Bypass Right Hypogastric Vein to Lower Vein, Open Approach */
"061H47Y"  =  "1"  /* Bypass R Hypogast Vein to Low Vein w Autol Sub, Perc Endo */
"061H49Y"  =  "1"  /* Bypass R Hypogast Vein to Low Vein w Autol Vn, Perc Endo */
"061H4AY"  =  "1"  /* Bypass R Hypogast Vein to Low Vein w Autol Art, Perc Endo */
"061H4JY"  =  "1"  /* Bypass R Hypogast Vein to Low Vein w Synth Sub, Perc Endo */
"061H4KY"  =  "1"  /* Bypass R Hypogast Vein to Low Vein w Nonaut Sub, Perc Endo */
"061H4ZY"  =  "1"  /* Bypass R Hypogast Vein to Low Vein, Perc Endo Approach */
"061J07Y"  =  "1"  /* Bypass L Hypogast Vein to Low Vein w Autol Sub, Open */
"061J09Y"  =  "1"  /* Bypass L Hypogast Vein to Low Vein w Autol Vn, Open */
"061J0AY"  =  "1"  /* Bypass L Hypogast Vein to Low Vein w Autol Art, Open */
"061J0JY"  =  "1"  /* Bypass L Hypogast Vein to Low Vein w Synth Sub, Open */
"061J0KY"  =  "1"  /* Bypass L Hypogast Vein to Low Vein w Nonaut Sub, Open */
"061J0ZY"  =  "1"  /* Bypass Left Hypogastric Vein to Lower Vein, Open Approach */
"061J47Y"  =  "1"  /* Bypass L Hypogast Vein to Low Vein w Autol Sub, Perc Endo */
"061J49Y"  =  "1"  /* Bypass L Hypogast Vein to Low Vein w Autol Vn, Perc Endo */
"061J4AY"  =  "1"  /* Bypass L Hypogast Vein to Low Vein w Autol Art, Perc Endo */
"061J4JY"  =  "1"  /* Bypass L Hypogast Vein to Low Vein w Synth Sub, Perc Endo */
"061J4KY"  =  "1"  /* Bypass L Hypogast Vein to Low Vein w Nonaut Sub, Perc Endo */
"061J4ZY"  =  "1"  /* Bypass Left Hypogastric Vein to Low Vein, Perc Endo Approach */
"061M07Y"  =  "1"  /* Bypass R Femor Vein to Low Vein w Autol Sub, Open */
"061M09Y"  =  "1"  /* Bypass R Femor Vein to Low Vein with Autol Vn, Open Approach */
"061M0AY"  =  "1"  /* Bypass R Femor Vein to Low Vein w Autol Art, Open */
"061M0JY"  =  "1"  /* Bypass R Femor Vein to Low Vein w Synth Sub, Open */
"061M0KY"  =  "1"  /* Bypass R Femor Vein to Low Vein w Nonaut Sub, Open */
"061M0ZY"  =  "1"  /* Bypass Right Femoral Vein to Lower Vein, Open Approach */
"061M47Y"  =  "1"  /* Bypass R Femor Vein to Low Vein w Autol Sub, Perc Endo */
"061M49Y"  =  "1"  /* Bypass R Femor Vein to Low Vein w Autol Vn, Perc Endo */
"061M4AY"  =  "1"  /* Bypass R Femor Vein to Low Vein w Autol Art, Perc Endo */
"061M4JY"  =  "1"  /* Bypass R Femor Vein to Low Vein w Synth Sub, Perc Endo */
"061M4KY"  =  "1"  /* Bypass R Femor Vein to Low Vein w Nonaut Sub, Perc Endo */
"061M4ZY"  =  "1"  /* Bypass Right Femoral Vein to Lower Vein, Perc Endo Approach */
"061N07Y"  =  "1"  /* Bypass L Femor Vein to Low Vein w Autol Sub, Open */
"061N09Y"  =  "1"  /* Bypass L Femor Vein to Low Vein with Autol Vn, Open Approach */
"061N0AY"  =  "1"  /* Bypass L Femor Vein to Low Vein w Autol Art, Open */
"061N0JY"  =  "1"  /* Bypass L Femor Vein to Low Vein w Synth Sub, Open */
"061N0KY"  =  "1"  /* Bypass L Femor Vein to Low Vein w Nonaut Sub, Open */
"061N0ZY"  =  "1"  /* Bypass Left Femoral Vein to Lower Vein, Open Approach */
"061N47Y"  =  "1"  /* Bypass L Femor Vein to Low Vein w Autol Sub, Perc Endo */
"061N49Y"  =  "1"  /* Bypass L Femor Vein to Low Vein w Autol Vn, Perc Endo */
"061N4AY"  =  "1"  /* Bypass L Femor Vein to Low Vein w Autol Art, Perc Endo */
"061N4JY"  =  "1"  /* Bypass L Femor Vein to Low Vein w Synth Sub, Perc Endo */
"061N4KY"  =  "1"  /* Bypass L Femor Vein to Low Vein w Nonaut Sub, Perc Endo */
"061N4ZY"  =  "1"  /* Bypass Left Femoral Vein to Lower Vein, Perc Endo Approach */
"061P07Y"  =  "1"  /* Bypass R Great Saphenous to Low Vein w Autol Sub, Open */
"061P09Y"  =  "1"  /* Bypass R Great Saphenous to Low Vein w Autol Vn, Open */
"061P0AY"  =  "1"  /* Bypass R Great Saphenous to Low Vein w Autol Art, Open */
"061P0JY"  =  "1"  /* Bypass R Great Saphenous to Low Vein w Synth Sub, Open */
"061P0KY"  =  "1"  /* Bypass R Great Saphenous to Low Vein w Nonaut Sub, Open */
"061P0ZY"  =  "1"  /* Bypass R Great Saphenous to Low Vein, Open Approach */
"061P47Y"  =  "1"  /* Bypass R Great Saphenous to Low Vein w Autol Sub, Perc Endo */
"061P49Y"  =  "1"  /* Bypass R Great Saphenous to Low Vein w Autol Vn, Perc Endo */
"061P4AY"  =  "1"  /* Bypass R Great Saphenous to Low Vein w Autol Art, Perc Endo */
"061P4JY"  =  "1"  /* Bypass R Great Saphenous to Low Vein w Synth Sub, Perc Endo */
"061P4KY"  =  "1"  /* Bypass R Great Saphenous to Low Vein w Nonaut Sub, Perc Endo */
"061P4ZY"  =  "1"  /* Bypass R Great Saphenous to Low Vein, Perc Endo Approach */
"061Q07Y"  =  "1"  /* Bypass L Great Saphenous to Low Vein w Autol Sub, Open */
"061Q09Y"  =  "1"  /* Bypass L Great Saphenous to Low Vein w Autol Vn, Open */
"061Q0AY"  =  "1"  /* Bypass L Great Saphenous to Low Vein w Autol Art, Open */
"061Q0JY"  =  "1"  /* Bypass L Great Saphenous to Low Vein w Synth Sub, Open */
"061Q0KY"  =  "1"  /* Bypass L Great Saphenous to Low Vein w Nonaut Sub, Open */
"061Q0ZY"  =  "1"  /* Bypass L Great Saphenous to Low Vein, Open Approach */
"061Q47Y"  =  "1"  /* Bypass L Great Saphenous to Low Vein w Autol Sub, Perc Endo */
"061Q49Y"  =  "1"  /* Bypass L Great Saphenous to Low Vein w Autol Vn, Perc Endo */
"061Q4AY"  =  "1"  /* Bypass L Great Saphenous to Low Vein w Autol Art, Perc Endo */
"061Q4JY"  =  "1"  /* Bypass L Great Saphenous to Low Vein w Synth Sub, Perc Endo */
"061Q4KY"  =  "1"  /* Bypass L Great Saphenous to Low Vein w Nonaut Sub, Perc Endo */
"061Q4ZY"  =  "1"  /* Bypass L Great Saphenous to Low Vein, Perc Endo Approach */
"061R07Y"  =  "1"  /* Bypass R Less Saphenous to Low Vein w Autol Sub, Open */
"061R09Y"  =  "1"  /* Bypass R Less Saphenous to Low Vein w Autol Vn, Open */
"061R0AY"  =  "1"  /* Bypass R Less Saphenous to Low Vein w Autol Art, Open */
"061R0JY"  =  "1"  /* Bypass R Less Saphenous to Low Vein w Synth Sub, Open */
"061R0KY"  =  "1"  /* Bypass R Less Saphenous to Low Vein w Nonaut Sub, Open */
"061R0ZY"  =  "1"  /* Bypass R Less Saphenous to Low Vein, Open Approach */
"061R47Y"  =  "1"  /* Bypass R Less Saphenous to Low Vein w Autol Sub, Perc Endo */
"061R49Y"  =  "1"  /* Bypass R Less Saphenous to Low Vein w Autol Vn, Perc Endo */
"061R4AY"  =  "1"  /* Bypass R Less Saphenous to Low Vein w Autol Art, Perc Endo */
"061R4JY"  =  "1"  /* Bypass R Less Saphenous to Low Vein w Synth Sub, Perc Endo */
"061R4KY"  =  "1"  /* Bypass R Less Saphenous to Low Vein w Nonaut Sub, Perc Endo */
"061R4ZY"  =  "1"  /* Bypass R Less Saphenous to Low Vein, Perc Endo Approach */
"061S07Y"  =  "1"  /* Bypass L Less Saphenous to Low Vein w Autol Sub, Open */
"061S09Y"  =  "1"  /* Bypass L Less Saphenous to Low Vein w Autol Vn, Open */
"061S0AY"  =  "1"  /* Bypass L Less Saphenous to Low Vein w Autol Art, Open */
"061S0JY"  =  "1"  /* Bypass L Less Saphenous to Low Vein w Synth Sub, Open */
"061S0KY"  =  "1"  /* Bypass L Less Saphenous to Low Vein w Nonaut Sub, Open */
"061S0ZY"  =  "1"  /* Bypass Left Lesser Saphenous Vein to Low Vein, Open Approach */
"061S47Y"  =  "1"  /* Bypass L Less Saphenous to Low Vein w Autol Sub, Perc Endo */
"061S49Y"  =  "1"  /* Bypass L Less Saphenous to Low Vein w Autol Vn, Perc Endo */
"061S4AY"  =  "1"  /* Bypass L Less Saphenous to Low Vein w Autol Art, Perc Endo */
"061S4JY"  =  "1"  /* Bypass L Less Saphenous to Low Vein w Synth Sub, Perc Endo */
"061S4KY"  =  "1"  /* Bypass L Less Saphenous to Low Vein w Nonaut Sub, Perc Endo */
"061S4ZY"  =  "1"  /* Bypass L Less Saphenous to Low Vein, Perc Endo Approach */
"061T07Y"  =  "1"  /* Bypass R Foot Vein to Low Vein with Autol Sub, Open Approach */
"061T09Y"  =  "1"  /* Bypass R Foot Vein to Low Vein with Autol Vn, Open Approach */
"061T0AY"  =  "1"  /* Bypass R Foot Vein to Low Vein with Autol Art, Open Approach */
"061T0JY"  =  "1"  /* Bypass R Foot Vein to Low Vein with Synth Sub, Open Approach */
"061T0KY"  =  "1"  /* Bypass R Foot Vein to Low Vein w Nonaut Sub, Open */
"061T0ZY"  =  "1"  /* Bypass Right Foot Vein to Lower Vein, Open Approach */
"061T47Y"  =  "1"  /* Bypass R Foot Vein to Low Vein w Autol Sub, Perc Endo */
"061T49Y"  =  "1"  /* Bypass R Foot Vein to Low Vein w Autol Vn, Perc Endo */
"061T4AY"  =  "1"  /* Bypass R Foot Vein to Low Vein w Autol Art, Perc Endo */
"061T4JY"  =  "1"  /* Bypass R Foot Vein to Low Vein w Synth Sub, Perc Endo */
"061T4KY"  =  "1"  /* Bypass R Foot Vein to Low Vein w Nonaut Sub, Perc Endo */
"061T4ZY"  =  "1"  /* Bypass Right Foot Vein to Lower Vein, Perc Endo Approach */
"061V07Y"  =  "1"  /* Bypass L Foot Vein to Low Vein with Autol Sub, Open Approach */
"061V09Y"  =  "1"  /* Bypass L Foot Vein to Low Vein with Autol Vn, Open Approach */
"061V0AY"  =  "1"  /* Bypass L Foot Vein to Low Vein with Autol Art, Open Approach */
"061V0JY"  =  "1"  /* Bypass L Foot Vein to Low Vein with Synth Sub, Open Approach */
"061V0KY"  =  "1"  /* Bypass L Foot Vein to Low Vein w Nonaut Sub, Open */
"061V0ZY"  =  "1"  /* Bypass Left Foot Vein to Lower Vein, Open Approach */
"061V47Y"  =  "1"  /* Bypass L Foot Vein to Low Vein w Autol Sub, Perc Endo */
"061V49Y"  =  "1"  /* Bypass L Foot Vein to Low Vein w Autol Vn, Perc Endo */
"061V4AY"  =  "1"  /* Bypass L Foot Vein to Low Vein w Autol Art, Perc Endo */
"061V4JY"  =  "1"  /* Bypass L Foot Vein to Low Vein w Synth Sub, Perc Endo */
"061V4KY"  =  "1"  /* Bypass L Foot Vein to Low Vein w Nonaut Sub, Perc Endo */
"061V4ZY"  =  "1"  /* Bypass Left Foot Vein to Lower Vein, Perc Endo Approach */
"06500ZZ"  =  "1"  /* Destruction of Inferior Vena Cava, Open Approach */
"06503ZZ"  =  "1"  /* Destruction of Inferior Vena Cava, Percutaneous Approach */
"06504ZZ"  =  "1"  /* Destruction of Inferior Vena Cava, Perc Endo Approach */
"06510ZZ"  =  "1"  /* Destruction of Splenic Vein, Open Approach */
"06513ZZ"  =  "1"  /* Destruction of Splenic Vein, Percutaneous Approach */
"06514ZZ"  =  "1"  /* Destruction of Splenic Vein, Perc Endo Approach */
"06520ZZ"  =  "1"  /* Destruction of Gastric Vein, Open Approach */
"06523ZZ"  =  "1"  /* Destruction of Gastric Vein, Percutaneous Approach */
"06524ZZ"  =  "1"  /* Destruction of Gastric Vein, Perc Endo Approach */
"06530ZZ"  =  "1"  /* Destruction of Esophageal Vein, Open Approach */
"06533ZZ"  =  "1"  /* Destruction of Esophageal Vein, Percutaneous Approach */
"06534ZZ"  =  "1"  /* Destruction of Esophageal Vein, Perc Endo Approach */
"06540ZZ"  =  "1"  /* Destruction of Hepatic Vein, Open Approach */
"06543ZZ"  =  "1"  /* Destruction of Hepatic Vein, Percutaneous Approach */
"06544ZZ"  =  "1"  /* Destruction of Hepatic Vein, Perc Endo Approach */
"06550ZZ"  =  "1"  /* Destruction of Superior Mesenteric Vein, Open Approach */
"06553ZZ"  =  "1"  /* Destruction of Superior Mesenteric Vein, Perc Approach */
"06554ZZ"  =  "1"  /* Destruction of Superior Mesenteric Vein, Perc Endo Approach */
"06560ZZ"  =  "1"  /* Destruction of Inferior Mesenteric Vein, Open Approach */
"06563ZZ"  =  "1"  /* Destruction of Inferior Mesenteric Vein, Perc Approach */
"06564ZZ"  =  "1"  /* Destruction of Inferior Mesenteric Vein, Perc Endo Approach */
"06570ZZ"  =  "1"  /* Destruction of Colic Vein, Open Approach */
"06573ZZ"  =  "1"  /* Destruction of Colic Vein, Percutaneous Approach */
"06574ZZ"  =  "1"  /* Destruction of Colic Vein, Percutaneous Endoscopic Approach */
"06580ZZ"  =  "1"  /* Destruction of Portal Vein, Open Approach */
"06583ZZ"  =  "1"  /* Destruction of Portal Vein, Percutaneous Approach */
"06584ZZ"  =  "1"  /* Destruction of Portal Vein, Percutaneous Endoscopic Approach */
"06590ZZ"  =  "1"  /* Destruction of Right Renal Vein, Open Approach */
"06593ZZ"  =  "1"  /* Destruction of Right Renal Vein, Percutaneous Approach */
"06594ZZ"  =  "1"  /* Destruction of Right Renal Vein, Perc Endo Approach */
"065B0ZZ"  =  "1"  /* Destruction of Left Renal Vein, Open Approach */
"065B3ZZ"  =  "1"  /* Destruction of Left Renal Vein, Percutaneous Approach */
"065B4ZZ"  =  "1"  /* Destruction of Left Renal Vein, Perc Endo Approach */
"065C0ZZ"  =  "1"  /* Destruction of Right Common Iliac Vein, Open Approach */
"065C3ZZ"  =  "1"  /* Destruction of Right Common Iliac Vein, Perc Approach */
"065C4ZZ"  =  "1"  /* Destruction of Right Common Iliac Vein, Perc Endo Approach */
"065D0ZZ"  =  "1"  /* Destruction of Left Common Iliac Vein, Open Approach */
"065D3ZZ"  =  "1"  /* Destruction of Left Common Iliac Vein, Percutaneous Approach */
"065D4ZZ"  =  "1"  /* Destruction of Left Common Iliac Vein, Perc Endo Approach */
"065F0ZZ"  =  "1"  /* Destruction of Right External Iliac Vein, Open Approach */
"065F3ZZ"  =  "1"  /* Destruction of Right External Iliac Vein, Perc Approach */
"065F4ZZ"  =  "1"  /* Destruction of Right External Iliac Vein, Perc Endo Approach */
"065G0ZZ"  =  "1"  /* Destruction of Left External Iliac Vein, Open Approach */
"065G3ZZ"  =  "1"  /* Destruction of Left External Iliac Vein, Perc Approach */
"065G4ZZ"  =  "1"  /* Destruction of Left External Iliac Vein, Perc Endo Approach */
"065H0ZZ"  =  "1"  /* Destruction of Right Hypogastric Vein, Open Approach */
"065H3ZZ"  =  "1"  /* Destruction of Right Hypogastric Vein, Percutaneous Approach */
"065H4ZZ"  =  "1"  /* Destruction of Right Hypogastric Vein, Perc Endo Approach */
"065J0ZZ"  =  "1"  /* Destruction of Left Hypogastric Vein, Open Approach */
"065J3ZZ"  =  "1"  /* Destruction of Left Hypogastric Vein, Percutaneous Approach */
"065J4ZZ"  =  "1"  /* Destruction of Left Hypogastric Vein, Perc Endo Approach */
"065M0ZZ"  =  "1"  /* Destruction of Right Femoral Vein, Open Approach */
"065M3ZZ"  =  "1"  /* Destruction of Right Femoral Vein, Percutaneous Approach */
"065M4ZZ"  =  "1"  /* Destruction of Right Femoral Vein, Perc Endo Approach */
"065N0ZZ"  =  "1"  /* Destruction of Left Femoral Vein, Open Approach */
"065N3ZZ"  =  "1"  /* Destruction of Left Femoral Vein, Percutaneous Approach */
"065N4ZZ"  =  "1"  /* Destruction of Left Femoral Vein, Perc Endo Approach */
"065P0ZZ"  =  "1"  /* Destruction of Right Greater Saphenous Vein, Open Approach */
"065P3ZZ"  =  "1"  /* Destruction of Right Greater Saphenous Vein, Perc Approach */
"065P4ZZ"  =  "1"  /* Destruction of R Great Saphenous, Perc Endo Approach */
"065Q0ZZ"  =  "1"  /* Destruction of Left Greater Saphenous Vein, Open Approach */
"065Q3ZZ"  =  "1"  /* Destruction of Left Greater Saphenous Vein, Perc Approach */
"065Q4ZZ"  =  "1"  /* Destruction of L Great Saphenous, Perc Endo Approach */
"065R0ZZ"  =  "1"  /* Destruction of Right Lesser Saphenous Vein, Open Approach */
"065R3ZZ"  =  "1"  /* Destruction of Right Lesser Saphenous Vein, Perc Approach */
"065R4ZZ"  =  "1"  /* Destruction of R Less Saphenous, Perc Endo Approach */
"065S0ZZ"  =  "1"  /* Destruction of Left Lesser Saphenous Vein, Open Approach */
"065S3ZZ"  =  "1"  /* Destruction of Left Lesser Saphenous Vein, Perc Approach */
"065S4ZZ"  =  "1"  /* Destruction of L Less Saphenous, Perc Endo Approach */
"065T0ZZ"  =  "1"  /* Destruction of Right Foot Vein, Open Approach */
"065T3ZZ"  =  "1"  /* Destruction of Right Foot Vein, Percutaneous Approach */
"065T4ZZ"  =  "1"  /* Destruction of Right Foot Vein, Perc Endo Approach */
"065V0ZZ"  =  "1"  /* Destruction of Left Foot Vein, Open Approach */
"065V3ZZ"  =  "1"  /* Destruction of Left Foot Vein, Percutaneous Approach */
"065V4ZZ"  =  "1"  /* Destruction of Left Foot Vein, Perc Endo Approach */
"065Y0ZC"  =  "1"  /* Destruction of Hemorrhoidal Plexus, Open Approach */
"065Y0ZZ"  =  "1"  /* Destruction of Lower Vein, Open Approach */
"065Y3ZC"  =  "1"  /* Destruction of Hemorrhoidal Plexus, Percutaneous Approach */
"065Y3ZZ"  =  "1"  /* Destruction of Lower Vein, Percutaneous Approach */
"065Y4ZC"  =  "1"  /* Destruction of Hemorrhoidal Plexus, Perc Endo Approach */
"065Y4ZZ"  =  "1"  /* Destruction of Lower Vein, Percutaneous Endoscopic Approach */
"06700DZ"  =  "1"  /* Dilation of Inf Vena Cava with Intralum Dev, Open Approach */
"06700ZZ"  =  "1"  /* Dilation of Inferior Vena Cava, Open Approach */
"06703DZ"  =  "1"  /* Dilation of Inf Vena Cava with Intralum Dev, Perc Approach */
"06703ZZ"  =  "1"  /* Dilation of Inferior Vena Cava, Percutaneous Approach */
"06704DZ"  =  "1"  /* Dilate Inf Vena Cava w Intralum Dev, Perc Endo */
"06704ZZ"  =  "1"  /* Dilation of Inferior Vena Cava, Perc Endo Approach */
"06710DZ"  =  "1"  /* Dilation of Splenic Vein with Intralum Dev, Open Approach */
"06710ZZ"  =  "1"  /* Dilation of Splenic Vein, Open Approach */
"06713DZ"  =  "1"  /* Dilation of Splenic Vein with Intralum Dev, Perc Approach */
"06713ZZ"  =  "1"  /* Dilation of Splenic Vein, Percutaneous Approach */
"06714DZ"  =  "1"  /* Dilate of Splenic Vein with Intralum Dev, Perc Endo Approach */
"06714ZZ"  =  "1"  /* Dilation of Splenic Vein, Percutaneous Endoscopic Approach */
"06720DZ"  =  "1"  /* Dilation of Gastric Vein with Intralum Dev, Open Approach */
"06720ZZ"  =  "1"  /* Dilation of Gastric Vein, Open Approach */
"06723DZ"  =  "1"  /* Dilation of Gastric Vein with Intralum Dev, Perc Approach */
"06723ZZ"  =  "1"  /* Dilation of Gastric Vein, Percutaneous Approach */
"06724DZ"  =  "1"  /* Dilate of Gastric Vein with Intralum Dev, Perc Endo Approach */
"06724ZZ"  =  "1"  /* Dilation of Gastric Vein, Percutaneous Endoscopic Approach */
"06730DZ"  =  "1"  /* Dilation of Esophageal Vein with Intralum Dev, Open Approach */
"06730ZZ"  =  "1"  /* Dilation of Esophageal Vein, Open Approach */
"06733DZ"  =  "1"  /* Dilation of Esophageal Vein with Intralum Dev, Perc Approach */
"06733ZZ"  =  "1"  /* Dilation of Esophageal Vein, Percutaneous Approach */
"06734DZ"  =  "1"  /* Dilate Esophageal Vein w Intralum Dev, Perc Endo */
"06734ZZ"  =  "1"  /* Dilation of Esophageal Vein, Perc Endo Approach */
"06740DZ"  =  "1"  /* Dilation of Hepatic Vein with Intralum Dev, Open Approach */
"06740ZZ"  =  "1"  /* Dilation of Hepatic Vein, Open Approach */
"06743DZ"  =  "1"  /* Dilation of Hepatic Vein with Intralum Dev, Perc Approach */
"06743ZZ"  =  "1"  /* Dilation of Hepatic Vein, Percutaneous Approach */
"06744DZ"  =  "1"  /* Dilate of Hepatic Vein with Intralum Dev, Perc Endo Approach */
"06744ZZ"  =  "1"  /* Dilation of Hepatic Vein, Percutaneous Endoscopic Approach */
"06750DZ"  =  "1"  /* Dilation of Sup Mesent Vein with Intralum Dev, Open Approach */
"06750ZZ"  =  "1"  /* Dilation of Superior Mesenteric Vein, Open Approach */
"06753DZ"  =  "1"  /* Dilation of Sup Mesent Vein with Intralum Dev, Perc Approach */
"06753ZZ"  =  "1"  /* Dilation of Superior Mesenteric Vein, Percutaneous Approach */
"06754DZ"  =  "1"  /* Dilate Sup Mesent Vein w Intralum Dev, Perc Endo */
"06754ZZ"  =  "1"  /* Dilation of Superior Mesenteric Vein, Perc Endo Approach */
"06760DZ"  =  "1"  /* Dilation of Inf Mesent Vein with Intralum Dev, Open Approach */
"06760ZZ"  =  "1"  /* Dilation of Inferior Mesenteric Vein, Open Approach */
"06763DZ"  =  "1"  /* Dilation of Inf Mesent Vein with Intralum Dev, Perc Approach */
"06763ZZ"  =  "1"  /* Dilation of Inferior Mesenteric Vein, Percutaneous Approach */
"06764DZ"  =  "1"  /* Dilate Inf Mesent Vein w Intralum Dev, Perc Endo */
"06764ZZ"  =  "1"  /* Dilation of Inferior Mesenteric Vein, Perc Endo Approach */
"06770DZ"  =  "1"  /* Dilation of Colic Vein with Intralum Dev, Open Approach */
"06770ZZ"  =  "1"  /* Dilation of Colic Vein, Open Approach */
"06773DZ"  =  "1"  /* Dilation of Colic Vein with Intralum Dev, Perc Approach */
"06773ZZ"  =  "1"  /* Dilation of Colic Vein, Percutaneous Approach */
"06774DZ"  =  "1"  /* Dilation of Colic Vein with Intralum Dev, Perc Endo Approach */
"06774ZZ"  =  "1"  /* Dilation of Colic Vein, Percutaneous Endoscopic Approach */
"06780DZ"  =  "1"  /* Dilation of Portal Vein with Intralum Dev, Open Approach */
"06780ZZ"  =  "1"  /* Dilation of Portal Vein, Open Approach */
"06783DZ"  =  "1"  /* Dilation of Portal Vein with Intralum Dev, Perc Approach */
"06783ZZ"  =  "1"  /* Dilation of Portal Vein, Percutaneous Approach */
"06784DZ"  =  "1"  /* Dilate of Portal Vein with Intralum Dev, Perc Endo Approach */
"06784ZZ"  =  "1"  /* Dilation of Portal Vein, Percutaneous Endoscopic Approach */
"06790DZ"  =  "1"  /* Dilation of R Renal Vein with Intralum Dev, Open Approach */
"06790ZZ"  =  "1"  /* Dilation of Right Renal Vein, Open Approach */
"06793DZ"  =  "1"  /* Dilation of R Renal Vein with Intralum Dev, Perc Approach */
"06793ZZ"  =  "1"  /* Dilation of Right Renal Vein, Percutaneous Approach */
"06794DZ"  =  "1"  /* Dilate of R Renal Vein with Intralum Dev, Perc Endo Approach */
"06794ZZ"  =  "1"  /* Dilation of Right Renal Vein, Perc Endo Approach */
"067B0DZ"  =  "1"  /* Dilation of Left Renal Vein with Intralum Dev, Open Approach */
"067B0ZZ"  =  "1"  /* Dilation of Left Renal Vein, Open Approach */
"067B3DZ"  =  "1"  /* Dilation of Left Renal Vein with Intralum Dev, Perc Approach */
"067B3ZZ"  =  "1"  /* Dilation of Left Renal Vein, Percutaneous Approach */
"067B4DZ"  =  "1"  /* Dilate of L Renal Vein with Intralum Dev, Perc Endo Approach */
"067B4ZZ"  =  "1"  /* Dilation of Left Renal Vein, Perc Endo Approach */
"067C0DZ"  =  "1"  /* Dilate of R Com Iliac Vein with Intralum Dev, Open Approach */
"067C0ZZ"  =  "1"  /* Dilation of Right Common Iliac Vein, Open Approach */
"067C3DZ"  =  "1"  /* Dilate of R Com Iliac Vein with Intralum Dev, Perc Approach */
"067C3ZZ"  =  "1"  /* Dilation of Right Common Iliac Vein, Percutaneous Approach */
"067C4DZ"  =  "1"  /* Dilate R Com Iliac Vein w Intralum Dev, Perc Endo */
"067C4ZZ"  =  "1"  /* Dilation of Right Common Iliac Vein, Perc Endo Approach */
"067D0DZ"  =  "1"  /* Dilate of L Com Iliac Vein with Intralum Dev, Open Approach */
"067D0ZZ"  =  "1"  /* Dilation of Left Common Iliac Vein, Open Approach */
"067D3DZ"  =  "1"  /* Dilate of L Com Iliac Vein with Intralum Dev, Perc Approach */
"067D3ZZ"  =  "1"  /* Dilation of Left Common Iliac Vein, Percutaneous Approach */
"067D4DZ"  =  "1"  /* Dilate L Com Iliac Vein w Intralum Dev, Perc Endo */
"067D4ZZ"  =  "1"  /* Dilation of Left Common Iliac Vein, Perc Endo Approach */
"067F0DZ"  =  "1"  /* Dilate of R Ext Iliac Vein with Intralum Dev, Open Approach */
"067F0ZZ"  =  "1"  /* Dilation of Right External Iliac Vein, Open Approach */
"067F3DZ"  =  "1"  /* Dilate of R Ext Iliac Vein with Intralum Dev, Perc Approach */
"067F3ZZ"  =  "1"  /* Dilation of Right External Iliac Vein, Percutaneous Approach */
"067F4DZ"  =  "1"  /* Dilate R Ext Iliac Vein w Intralum Dev, Perc Endo */
"067F4ZZ"  =  "1"  /* Dilation of Right External Iliac Vein, Perc Endo Approach */
"067G0DZ"  =  "1"  /* Dilate of L Ext Iliac Vein with Intralum Dev, Open Approach */
"067G0ZZ"  =  "1"  /* Dilation of Left External Iliac Vein, Open Approach */
"067G3DZ"  =  "1"  /* Dilate of L Ext Iliac Vein with Intralum Dev, Perc Approach */
"067G3ZZ"  =  "1"  /* Dilation of Left External Iliac Vein, Percutaneous Approach */
"067G4DZ"  =  "1"  /* Dilate L Ext Iliac Vein w Intralum Dev, Perc Endo */
"067G4ZZ"  =  "1"  /* Dilation of Left External Iliac Vein, Perc Endo Approach */
"067H0DZ"  =  "1"  /* Dilation of R Hypogast Vein with Intralum Dev, Open Approach */
"067H0ZZ"  =  "1"  /* Dilation of Right Hypogastric Vein, Open Approach */
"067H3DZ"  =  "1"  /* Dilation of R Hypogast Vein with Intralum Dev, Perc Approach */
"067H3ZZ"  =  "1"  /* Dilation of Right Hypogastric Vein, Percutaneous Approach */
"067H4DZ"  =  "1"  /* Dilate R Hypogast Vein w Intralum Dev, Perc Endo */
"067H4ZZ"  =  "1"  /* Dilation of Right Hypogastric Vein, Perc Endo Approach */
"067J0DZ"  =  "1"  /* Dilation of L Hypogast Vein with Intralum Dev, Open Approach */
"067J0ZZ"  =  "1"  /* Dilation of Left Hypogastric Vein, Open Approach */
"067J3DZ"  =  "1"  /* Dilation of L Hypogast Vein with Intralum Dev, Perc Approach */
"067J3ZZ"  =  "1"  /* Dilation of Left Hypogastric Vein, Percutaneous Approach */
"067J4DZ"  =  "1"  /* Dilate L Hypogast Vein w Intralum Dev, Perc Endo */
"067J4ZZ"  =  "1"  /* Dilation of Left Hypogastric Vein, Perc Endo Approach */
"067M0DZ"  =  "1"  /* Dilation of R Femor Vein with Intralum Dev, Open Approach */
"067M0ZZ"  =  "1"  /* Dilation of Right Femoral Vein, Open Approach */
"067M3DZ"  =  "1"  /* Dilation of R Femor Vein with Intralum Dev, Perc Approach */
"067M3ZZ"  =  "1"  /* Dilation of Right Femoral Vein, Percutaneous Approach */
"067M4DZ"  =  "1"  /* Dilate of R Femor Vein with Intralum Dev, Perc Endo Approach */
"067M4ZZ"  =  "1"  /* Dilation of Right Femoral Vein, Perc Endo Approach */
"067N0DZ"  =  "1"  /* Dilation of L Femor Vein with Intralum Dev, Open Approach */
"067N0ZZ"  =  "1"  /* Dilation of Left Femoral Vein, Open Approach */
"067N3DZ"  =  "1"  /* Dilation of L Femor Vein with Intralum Dev, Perc Approach */
"067N3ZZ"  =  "1"  /* Dilation of Left Femoral Vein, Percutaneous Approach */
"067N4DZ"  =  "1"  /* Dilate of L Femor Vein with Intralum Dev, Perc Endo Approach */
"067N4ZZ"  =  "1"  /* Dilation of Left Femoral Vein, Perc Endo Approach */
"067P0DZ"  =  "1"  /* Dilate of R Great Saphenous with Intralum Dev, Open Approach */
"067P0ZZ"  =  "1"  /* Dilation of Right Greater Saphenous Vein, Open Approach */
"067P3DZ"  =  "1"  /* Dilate of R Great Saphenous with Intralum Dev, Perc Approach */
"067P3ZZ"  =  "1"  /* Dilation of Right Greater Saphenous Vein, Perc Approach */
"067P4DZ"  =  "1"  /* Dilate R Great Saphenous w Intralum Dev, Perc Endo */
"067P4ZZ"  =  "1"  /* Dilation of Right Greater Saphenous Vein, Perc Endo Approach */
"067Q0DZ"  =  "1"  /* Dilate of L Great Saphenous with Intralum Dev, Open Approach */
"067Q0ZZ"  =  "1"  /* Dilation of Left Greater Saphenous Vein, Open Approach */
"067Q3DZ"  =  "1"  /* Dilate of L Great Saphenous with Intralum Dev, Perc Approach */
"067Q3ZZ"  =  "1"  /* Dilation of Left Greater Saphenous Vein, Perc Approach */
"067Q4DZ"  =  "1"  /* Dilate L Great Saphenous w Intralum Dev, Perc Endo */
"067Q4ZZ"  =  "1"  /* Dilation of Left Greater Saphenous Vein, Perc Endo Approach */
"067R0DZ"  =  "1"  /* Dilate of R Less Saphenous with Intralum Dev, Open Approach */
"067R0ZZ"  =  "1"  /* Dilation of Right Lesser Saphenous Vein, Open Approach */
"067R3DZ"  =  "1"  /* Dilate of R Less Saphenous with Intralum Dev, Perc Approach */
"067R3ZZ"  =  "1"  /* Dilation of Right Lesser Saphenous Vein, Perc Approach */
"067R4DZ"  =  "1"  /* Dilate R Less Saphenous w Intralum Dev, Perc Endo */
"067R4ZZ"  =  "1"  /* Dilation of Right Lesser Saphenous Vein, Perc Endo Approach */
"067S0DZ"  =  "1"  /* Dilate of L Less Saphenous with Intralum Dev, Open Approach */
"067S0ZZ"  =  "1"  /* Dilation of Left Lesser Saphenous Vein, Open Approach */
"067S3DZ"  =  "1"  /* Dilate of L Less Saphenous with Intralum Dev, Perc Approach */
"067S3ZZ"  =  "1"  /* Dilation of Left Lesser Saphenous Vein, Perc Approach */
"067S4DZ"  =  "1"  /* Dilate L Less Saphenous w Intralum Dev, Perc Endo */
"067S4ZZ"  =  "1"  /* Dilation of Left Lesser Saphenous Vein, Perc Endo Approach */
"067T0DZ"  =  "1"  /* Dilation of Right Foot Vein with Intralum Dev, Open Approach */
"067T0ZZ"  =  "1"  /* Dilation of Right Foot Vein, Open Approach */
"067T3DZ"  =  "1"  /* Dilation of Right Foot Vein with Intralum Dev, Perc Approach */
"067T3ZZ"  =  "1"  /* Dilation of Right Foot Vein, Percutaneous Approach */
"067T4DZ"  =  "1"  /* Dilate of R Foot Vein with Intralum Dev, Perc Endo Approach */
"067T4ZZ"  =  "1"  /* Dilation of Right Foot Vein, Perc Endo Approach */
"067V0DZ"  =  "1"  /* Dilation of Left Foot Vein with Intralum Dev, Open Approach */
"067V0ZZ"  =  "1"  /* Dilation of Left Foot Vein, Open Approach */
"067V3DZ"  =  "1"  /* Dilation of Left Foot Vein with Intralum Dev, Perc Approach */
"067V3ZZ"  =  "1"  /* Dilation of Left Foot Vein, Percutaneous Approach */
"067V4DZ"  =  "1"  /* Dilate of L Foot Vein with Intralum Dev, Perc Endo Approach */
"067V4ZZ"  =  "1"  /* Dilation of Left Foot Vein, Percutaneous Endoscopic Approach */
"067Y0DZ"  =  "1"  /* Dilation of Lower Vein with Intralum Dev, Open Approach */
"067Y0ZZ"  =  "1"  /* Dilation of Lower Vein, Open Approach */
"067Y3DZ"  =  "1"  /* Dilation of Lower Vein with Intralum Dev, Perc Approach */
"067Y3ZZ"  =  "1"  /* Dilation of Lower Vein, Percutaneous Approach */
"067Y4DZ"  =  "1"  /* Dilation of Lower Vein with Intralum Dev, Perc Endo Approach */
"067Y4ZZ"  =  "1"  /* Dilation of Lower Vein, Percutaneous Endoscopic Approach */
"06900ZX"  =  "1"  /* Drainage of Inferior Vena Cava, Open Approach, Diagnostic */
"06903ZX"  =  "1"  /* Drainage of Inferior Vena Cava, Percutaneous Approach, Diagn */
"06904ZX"  =  "1"  /* Drainage of Inferior Vena Cava, Perc Endo Approach, Diagn */
"06910ZX"  =  "1"  /* Drainage of Splenic Vein, Open Approach, Diagnostic */
"06913ZX"  =  "1"  /* Drainage of Splenic Vein, Percutaneous Approach, Diagnostic */
"06914ZX"  =  "1"  /* Drainage of Splenic Vein, Perc Endo Approach, Diagn */
"06920ZX"  =  "1"  /* Drainage of Gastric Vein, Open Approach, Diagnostic */
"06923ZX"  =  "1"  /* Drainage of Gastric Vein, Percutaneous Approach, Diagnostic */
"06924ZX"  =  "1"  /* Drainage of Gastric Vein, Perc Endo Approach, Diagn */
"069300Z"  =  "1"  /* Drainage of Esophageal Vein with Drain Dev, Open Approach */
"06930ZX"  =  "1"  /* Drainage of Esophageal Vein, Open Approach, Diagnostic */
"06930ZZ"  =  "1"  /* Drainage of Esophageal Vein, Open Approach */
"069330Z"  =  "1"  /* Drainage of Esophageal Vein with Drain Dev, Perc Approach */
"06933ZX"  =  "1"  /* Drainage of Esophageal Vein, Percutaneous Approach, Diagn */
"06933ZZ"  =  "1"  /* Drainage of Esophageal Vein, Percutaneous Approach */
"069340Z"  =  "1"  /* Drain of Esophageal Vein with Drain Dev, Perc Endo Approach */
"06934ZX"  =  "1"  /* Drainage of Esophageal Vein, Perc Endo Approach, Diagn */
"06934ZZ"  =  "1"  /* Drainage of Esophageal Vein, Perc Endo Approach */
"06940ZX"  =  "1"  /* Drainage of Hepatic Vein, Open Approach, Diagnostic */
"06943ZX"  =  "1"  /* Drainage of Hepatic Vein, Percutaneous Approach, Diagnostic */
"06944ZX"  =  "1"  /* Drainage of Hepatic Vein, Perc Endo Approach, Diagn */
"06950ZX"  =  "1"  /* Drainage of Superior Mesenteric Vein, Open Approach, Diagn */
"06953ZX"  =  "1"  /* Drainage of Superior Mesenteric Vein, Perc Approach, Diagn */
"06954ZX"  =  "1"  /* Drainage of Sup Mesent Vein, Perc Endo Approach, Diagn */
"06960ZX"  =  "1"  /* Drainage of Inferior Mesenteric Vein, Open Approach, Diagn */
"06963ZX"  =  "1"  /* Drainage of Inferior Mesenteric Vein, Perc Approach, Diagn */
"06964ZX"  =  "1"  /* Drainage of Inf Mesent Vein, Perc Endo Approach, Diagn */
"06970ZX"  =  "1"  /* Drainage of Colic Vein, Open Approach, Diagnostic */
"06973ZX"  =  "1"  /* Drainage of Colic Vein, Percutaneous Approach, Diagnostic */
"06974ZX"  =  "1"  /* Drainage of Colic Vein, Perc Endo Approach, Diagn */
"06980ZX"  =  "1"  /* Drainage of Portal Vein, Open Approach, Diagnostic */
"06983ZX"  =  "1"  /* Drainage of Portal Vein, Percutaneous Approach, Diagnostic */
"06984ZX"  =  "1"  /* Drainage of Portal Vein, Perc Endo Approach, Diagn */
"06990ZX"  =  "1"  /* Drainage of Right Renal Vein, Open Approach, Diagnostic */
"06993ZX"  =  "1"  /* Drainage of Right Renal Vein, Percutaneous Approach, Diagn */
"06994ZX"  =  "1"  /* Drainage of Right Renal Vein, Perc Endo Approach, Diagn */
"069B0ZX"  =  "1"  /* Drainage of Left Renal Vein, Open Approach, Diagnostic */
"069B3ZX"  =  "1"  /* Drainage of Left Renal Vein, Percutaneous Approach, Diagn */
"069B4ZX"  =  "1"  /* Drainage of Left Renal Vein, Perc Endo Approach, Diagn */
"069C0ZX"  =  "1"  /* Drainage of Right Common Iliac Vein, Open Approach, Diagn */
"069C3ZX"  =  "1"  /* Drainage of Right Common Iliac Vein, Perc Approach, Diagn */
"069C4ZX"  =  "1"  /* Drainage of R Com Iliac Vein, Perc Endo Approach, Diagn */
"069D0ZX"  =  "1"  /* Drainage of Left Common Iliac Vein, Open Approach, Diagn */
"069D3ZX"  =  "1"  /* Drainage of Left Common Iliac Vein, Perc Approach, Diagn */
"069D4ZX"  =  "1"  /* Drainage of L Com Iliac Vein, Perc Endo Approach, Diagn */
"069F0ZX"  =  "1"  /* Drainage of Right External Iliac Vein, Open Approach, Diagn */
"069F3ZX"  =  "1"  /* Drainage of Right External Iliac Vein, Perc Approach, Diagn */
"069F4ZX"  =  "1"  /* Drainage of R Ext Iliac Vein, Perc Endo Approach, Diagn */
"069G0ZX"  =  "1"  /* Drainage of Left External Iliac Vein, Open Approach, Diagn */
"069G3ZX"  =  "1"  /* Drainage of Left External Iliac Vein, Perc Approach, Diagn */
"069G4ZX"  =  "1"  /* Drainage of L Ext Iliac Vein, Perc Endo Approach, Diagn */
"069H0ZX"  =  "1"  /* Drainage of Right Hypogastric Vein, Open Approach, Diagn */
"069H3ZX"  =  "1"  /* Drainage of Right Hypogastric Vein, Perc Approach, Diagn */
"069H4ZX"  =  "1"  /* Drainage of R Hypogast Vein, Perc Endo Approach, Diagn */
"069J0ZX"  =  "1"  /* Drainage of Left Hypogastric Vein, Open Approach, Diagnostic */
"069J3ZX"  =  "1"  /* Drainage of Left Hypogastric Vein, Perc Approach, Diagn */
"069J4ZX"  =  "1"  /* Drainage of Left Hypogastric Vein, Perc Endo Approach, Diagn */
"069M0ZX"  =  "1"  /* Drainage of Right Femoral Vein, Open Approach, Diagnostic */
"069M3ZX"  =  "1"  /* Drainage of Right Femoral Vein, Percutaneous Approach, Diagn */
"069M4ZX"  =  "1"  /* Drainage of Right Femoral Vein, Perc Endo Approach, Diagn */
"069N0ZX"  =  "1"  /* Drainage of Left Femoral Vein, Open Approach, Diagnostic */
"069N3ZX"  =  "1"  /* Drainage of Left Femoral Vein, Percutaneous Approach, Diagn */
"069N4ZX"  =  "1"  /* Drainage of Left Femoral Vein, Perc Endo Approach, Diagn */
"069P0ZX"  =  "1"  /* Drainage of R Great Saphenous, Open Approach, Diagn */
"069P3ZX"  =  "1"  /* Drainage of R Great Saphenous, Perc Approach, Diagn */
"069P4ZX"  =  "1"  /* Drainage of R Great Saphenous, Perc Endo Approach, Diagn */
"069Q0ZX"  =  "1"  /* Drainage of L Great Saphenous, Open Approach, Diagn */
"069Q3ZX"  =  "1"  /* Drainage of L Great Saphenous, Perc Approach, Diagn */
"069Q4ZX"  =  "1"  /* Drainage of L Great Saphenous, Perc Endo Approach, Diagn */
"069R0ZX"  =  "1"  /* Drainage of R Less Saphenous, Open Approach, Diagn */
"069R3ZX"  =  "1"  /* Drainage of R Less Saphenous, Perc Approach, Diagn */
"069R4ZX"  =  "1"  /* Drainage of R Less Saphenous, Perc Endo Approach, Diagn */
"069S0ZX"  =  "1"  /* Drainage of Left Lesser Saphenous Vein, Open Approach, Diagn */
"069S3ZX"  =  "1"  /* Drainage of Left Lesser Saphenous Vein, Perc Approach, Diagn */
"069S4ZX"  =  "1"  /* Drainage of L Less Saphenous, Perc Endo Approach, Diagn */
"069T0ZX"  =  "1"  /* Drainage of Right Foot Vein, Open Approach, Diagnostic */
"069T3ZX"  =  "1"  /* Drainage of Right Foot Vein, Percutaneous Approach, Diagn */
"069T4ZX"  =  "1"  /* Drainage of Right Foot Vein, Perc Endo Approach, Diagn */
"069V0ZX"  =  "1"  /* Drainage of Left Foot Vein, Open Approach, Diagnostic */
"069V3ZX"  =  "1"  /* Drainage of Left Foot Vein, Percutaneous Approach, Diagn */
"069V4ZX"  =  "1"  /* Drainage of Left Foot Vein, Perc Endo Approach, Diagn */
"069Y0ZX"  =  "1"  /* Drainage of Lower Vein, Open Approach, Diagnostic */
"069Y3ZX"  =  "1"  /* Drainage of Lower Vein, Percutaneous Approach, Diagnostic */
"069Y4ZX"  =  "1"  /* Drainage of Lower Vein, Perc Endo Approach, Diagn */
"06B00ZX"  =  "1"  /* Excision of Inferior Vena Cava, Open Approach, Diagnostic */
"06B00ZZ"  =  "1"  /* Excision of Inferior Vena Cava, Open Approach */
"06B03ZX"  =  "1"  /* Excision of Inferior Vena Cava, Percutaneous Approach, Diagn */
"06B03ZZ"  =  "1"  /* Excision of Inferior Vena Cava, Percutaneous Approach */
"06B04ZX"  =  "1"  /* Excision of Inferior Vena Cava, Perc Endo Approach, Diagn */
"06B04ZZ"  =  "1"  /* Excision of Inferior Vena Cava, Perc Endo Approach */
"06B10ZX"  =  "1"  /* Excision of Splenic Vein, Open Approach, Diagnostic */
"06B10ZZ"  =  "1"  /* Excision of Splenic Vein, Open Approach */
"06B13ZX"  =  "1"  /* Excision of Splenic Vein, Percutaneous Approach, Diagnostic */
"06B13ZZ"  =  "1"  /* Excision of Splenic Vein, Percutaneous Approach */
"06B14ZX"  =  "1"  /* Excision of Splenic Vein, Perc Endo Approach, Diagn */
"06B14ZZ"  =  "1"  /* Excision of Splenic Vein, Percutaneous Endoscopic Approach */
"06B20ZX"  =  "1"  /* Excision of Gastric Vein, Open Approach, Diagnostic */
"06B20ZZ"  =  "1"  /* Excision of Gastric Vein, Open Approach */
"06B23ZX"  =  "1"  /* Excision of Gastric Vein, Percutaneous Approach, Diagnostic */
"06B23ZZ"  =  "1"  /* Excision of Gastric Vein, Percutaneous Approach */
"06B24ZX"  =  "1"  /* Excision of Gastric Vein, Perc Endo Approach, Diagn */
"06B24ZZ"  =  "1"  /* Excision of Gastric Vein, Percutaneous Endoscopic Approach */
"06B30ZX"  =  "1"  /* Excision of Esophageal Vein, Open Approach, Diagnostic */
"06B30ZZ"  =  "1"  /* Excision of Esophageal Vein, Open Approach */
"06B33ZX"  =  "1"  /* Excision of Esophageal Vein, Percutaneous Approach, Diagn */
"06B33ZZ"  =  "1"  /* Excision of Esophageal Vein, Percutaneous Approach */
"06B34ZX"  =  "1"  /* Excision of Esophageal Vein, Perc Endo Approach, Diagn */
"06B34ZZ"  =  "1"  /* Excision of Esophageal Vein, Perc Endo Approach */
"06B40ZX"  =  "1"  /* Excision of Hepatic Vein, Open Approach, Diagnostic */
"06B40ZZ"  =  "1"  /* Excision of Hepatic Vein, Open Approach */
"06B43ZX"  =  "1"  /* Excision of Hepatic Vein, Percutaneous Approach, Diagnostic */
"06B43ZZ"  =  "1"  /* Excision of Hepatic Vein, Percutaneous Approach */
"06B44ZX"  =  "1"  /* Excision of Hepatic Vein, Perc Endo Approach, Diagn */
"06B44ZZ"  =  "1"  /* Excision of Hepatic Vein, Percutaneous Endoscopic Approach */
"06B50ZX"  =  "1"  /* Excision of Superior Mesenteric Vein, Open Approach, Diagn */
"06B50ZZ"  =  "1"  /* Excision of Superior Mesenteric Vein, Open Approach */
"06B53ZX"  =  "1"  /* Excision of Superior Mesenteric Vein, Perc Approach, Diagn */
"06B53ZZ"  =  "1"  /* Excision of Superior Mesenteric Vein, Percutaneous Approach */
"06B54ZX"  =  "1"  /* Excision of Sup Mesent Vein, Perc Endo Approach, Diagn */
"06B54ZZ"  =  "1"  /* Excision of Superior Mesenteric Vein, Perc Endo Approach */
"06B60ZX"  =  "1"  /* Excision of Inferior Mesenteric Vein, Open Approach, Diagn */
"06B60ZZ"  =  "1"  /* Excision of Inferior Mesenteric Vein, Open Approach */
"06B63ZX"  =  "1"  /* Excision of Inferior Mesenteric Vein, Perc Approach, Diagn */
"06B63ZZ"  =  "1"  /* Excision of Inferior Mesenteric Vein, Percutaneous Approach */
"06B64ZX"  =  "1"  /* Excision of Inf Mesent Vein, Perc Endo Approach, Diagn */
"06B64ZZ"  =  "1"  /* Excision of Inferior Mesenteric Vein, Perc Endo Approach */
"06B70ZX"  =  "1"  /* Excision of Colic Vein, Open Approach, Diagnostic */
"06B70ZZ"  =  "1"  /* Excision of Colic Vein, Open Approach */
"06B73ZX"  =  "1"  /* Excision of Colic Vein, Percutaneous Approach, Diagnostic */
"06B73ZZ"  =  "1"  /* Excision of Colic Vein, Percutaneous Approach */
"06B74ZX"  =  "1"  /* Excision of Colic Vein, Perc Endo Approach, Diagn */
"06B74ZZ"  =  "1"  /* Excision of Colic Vein, Percutaneous Endoscopic Approach */
"06B80ZX"  =  "1"  /* Excision of Portal Vein, Open Approach, Diagnostic */
"06B80ZZ"  =  "1"  /* Excision of Portal Vein, Open Approach */
"06B83ZX"  =  "1"  /* Excision of Portal Vein, Percutaneous Approach, Diagnostic */
"06B83ZZ"  =  "1"  /* Excision of Portal Vein, Percutaneous Approach */
"06B84ZX"  =  "1"  /* Excision of Portal Vein, Perc Endo Approach, Diagn */
"06B84ZZ"  =  "1"  /* Excision of Portal Vein, Percutaneous Endoscopic Approach */
"06B90ZX"  =  "1"  /* Excision of Right Renal Vein, Open Approach, Diagnostic */
"06B90ZZ"  =  "1"  /* Excision of Right Renal Vein, Open Approach */
"06B93ZX"  =  "1"  /* Excision of Right Renal Vein, Percutaneous Approach, Diagn */
"06B93ZZ"  =  "1"  /* Excision of Right Renal Vein, Percutaneous Approach */
"06B94ZX"  =  "1"  /* Excision of Right Renal Vein, Perc Endo Approach, Diagn */
"06B94ZZ"  =  "1"  /* Excision of Right Renal Vein, Perc Endo Approach */
"06BB0ZX"  =  "1"  /* Excision of Left Renal Vein, Open Approach, Diagnostic */
"06BB0ZZ"  =  "1"  /* Excision of Left Renal Vein, Open Approach */
"06BB3ZX"  =  "1"  /* Excision of Left Renal Vein, Percutaneous Approach, Diagn */
"06BB3ZZ"  =  "1"  /* Excision of Left Renal Vein, Percutaneous Approach */
"06BB4ZX"  =  "1"  /* Excision of Left Renal Vein, Perc Endo Approach, Diagn */
"06BB4ZZ"  =  "1"  /* Excision of Left Renal Vein, Perc Endo Approach */
"06BC0ZX"  =  "1"  /* Excision of Right Common Iliac Vein, Open Approach, Diagn */
"06BC0ZZ"  =  "1"  /* Excision of Right Common Iliac Vein, Open Approach */
"06BC3ZX"  =  "1"  /* Excision of Right Common Iliac Vein, Perc Approach, Diagn */
"06BC3ZZ"  =  "1"  /* Excision of Right Common Iliac Vein, Percutaneous Approach */
"06BC4ZX"  =  "1"  /* Excision of R Com Iliac Vein, Perc Endo Approach, Diagn */
"06BC4ZZ"  =  "1"  /* Excision of Right Common Iliac Vein, Perc Endo Approach */
"06BD0ZX"  =  "1"  /* Excision of Left Common Iliac Vein, Open Approach, Diagn */
"06BD0ZZ"  =  "1"  /* Excision of Left Common Iliac Vein, Open Approach */
"06BD3ZX"  =  "1"  /* Excision of Left Common Iliac Vein, Perc Approach, Diagn */
"06BD3ZZ"  =  "1"  /* Excision of Left Common Iliac Vein, Percutaneous Approach */
"06BD4ZX"  =  "1"  /* Excision of L Com Iliac Vein, Perc Endo Approach, Diagn */
"06BD4ZZ"  =  "1"  /* Excision of Left Common Iliac Vein, Perc Endo Approach */
"06BF0ZX"  =  "1"  /* Excision of Right External Iliac Vein, Open Approach, Diagn */
"06BF0ZZ"  =  "1"  /* Excision of Right External Iliac Vein, Open Approach */
"06BF3ZX"  =  "1"  /* Excision of Right External Iliac Vein, Perc Approach, Diagn */
"06BF3ZZ"  =  "1"  /* Excision of Right External Iliac Vein, Percutaneous Approach */
"06BF4ZX"  =  "1"  /* Excision of R Ext Iliac Vein, Perc Endo Approach, Diagn */
"06BF4ZZ"  =  "1"  /* Excision of Right External Iliac Vein, Perc Endo Approach */
"06BG0ZX"  =  "1"  /* Excision of Left External Iliac Vein, Open Approach, Diagn */
"06BG0ZZ"  =  "1"  /* Excision of Left External Iliac Vein, Open Approach */
"06BG3ZX"  =  "1"  /* Excision of Left External Iliac Vein, Perc Approach, Diagn */
"06BG3ZZ"  =  "1"  /* Excision of Left External Iliac Vein, Percutaneous Approach */
"06BG4ZX"  =  "1"  /* Excision of L Ext Iliac Vein, Perc Endo Approach, Diagn */
"06BG4ZZ"  =  "1"  /* Excision of Left External Iliac Vein, Perc Endo Approach */
"06BH0ZX"  =  "1"  /* Excision of Right Hypogastric Vein, Open Approach, Diagn */
"06BH0ZZ"  =  "1"  /* Excision of Right Hypogastric Vein, Open Approach */
"06BH3ZX"  =  "1"  /* Excision of Right Hypogastric Vein, Perc Approach, Diagn */
"06BH3ZZ"  =  "1"  /* Excision of Right Hypogastric Vein, Percutaneous Approach */
"06BH4ZX"  =  "1"  /* Excision of R Hypogast Vein, Perc Endo Approach, Diagn */
"06BH4ZZ"  =  "1"  /* Excision of Right Hypogastric Vein, Perc Endo Approach */
"06BJ0ZX"  =  "1"  /* Excision of Left Hypogastric Vein, Open Approach, Diagnostic */
"06BJ0ZZ"  =  "1"  /* Excision of Left Hypogastric Vein, Open Approach */
"06BJ3ZX"  =  "1"  /* Excision of Left Hypogastric Vein, Perc Approach, Diagn */
"06BJ3ZZ"  =  "1"  /* Excision of Left Hypogastric Vein, Percutaneous Approach */
"06BJ4ZX"  =  "1"  /* Excision of Left Hypogastric Vein, Perc Endo Approach, Diagn */
"06BJ4ZZ"  =  "1"  /* Excision of Left Hypogastric Vein, Perc Endo Approach */
"06BM0ZX"  =  "1"  /* Excision of Right Femoral Vein, Open Approach, Diagnostic */
"06BM0ZZ"  =  "1"  /* Excision of Right Femoral Vein, Open Approach */
"06BM3ZX"  =  "1"  /* Excision of Right Femoral Vein, Percutaneous Approach, Diagn */
"06BM3ZZ"  =  "1"  /* Excision of Right Femoral Vein, Percutaneous Approach */
"06BM4ZX"  =  "1"  /* Excision of Right Femoral Vein, Perc Endo Approach, Diagn */
"06BM4ZZ"  =  "1"  /* Excision of Right Femoral Vein, Perc Endo Approach */
"06BN0ZX"  =  "1"  /* Excision of Left Femoral Vein, Open Approach, Diagnostic */
"06BN0ZZ"  =  "1"  /* Excision of Left Femoral Vein, Open Approach */
"06BN3ZX"  =  "1"  /* Excision of Left Femoral Vein, Percutaneous Approach, Diagn */
"06BN3ZZ"  =  "1"  /* Excision of Left Femoral Vein, Percutaneous Approach */
"06BN4ZX"  =  "1"  /* Excision of Left Femoral Vein, Perc Endo Approach, Diagn */
"06BN4ZZ"  =  "1"  /* Excision of Left Femoral Vein, Perc Endo Approach */
"06BP0ZX"  =  "1"  /* Excision of R Great Saphenous, Open Approach, Diagn */
"06BP0ZZ"  =  "1"  /* Excision of Right Greater Saphenous Vein, Open Approach */
"06BP3ZX"  =  "1"  /* Excision of R Great Saphenous, Perc Approach, Diagn */
"06BP3ZZ"  =  "1"  /* Excision of Right Greater Saphenous Vein, Perc Approach */
"06BP4ZX"  =  "1"  /* Excision of R Great Saphenous, Perc Endo Approach, Diagn */
"06BP4ZZ"  =  "1"  /* Excision of Right Greater Saphenous Vein, Perc Endo Approach */
"06BQ0ZX"  =  "1"  /* Excision of L Great Saphenous, Open Approach, Diagn */
"06BQ0ZZ"  =  "1"  /* Excision of Left Greater Saphenous Vein, Open Approach */
"06BQ3ZX"  =  "1"  /* Excision of L Great Saphenous, Perc Approach, Diagn */
"06BQ3ZZ"  =  "1"  /* Excision of Left Greater Saphenous Vein, Perc Approach */
"06BQ4ZX"  =  "1"  /* Excision of L Great Saphenous, Perc Endo Approach, Diagn */
"06BQ4ZZ"  =  "1"  /* Excision of Left Greater Saphenous Vein, Perc Endo Approach */
"06BR0ZX"  =  "1"  /* Excision of R Less Saphenous, Open Approach, Diagn */
"06BR0ZZ"  =  "1"  /* Excision of Right Lesser Saphenous Vein, Open Approach */
"06BR3ZX"  =  "1"  /* Excision of R Less Saphenous, Perc Approach, Diagn */
"06BR3ZZ"  =  "1"  /* Excision of Right Lesser Saphenous Vein, Perc Approach */
"06BR4ZX"  =  "1"  /* Excision of R Less Saphenous, Perc Endo Approach, Diagn */
"06BR4ZZ"  =  "1"  /* Excision of Right Lesser Saphenous Vein, Perc Endo Approach */
"06BS0ZX"  =  "1"  /* Excision of Left Lesser Saphenous Vein, Open Approach, Diagn */
"06BS0ZZ"  =  "1"  /* Excision of Left Lesser Saphenous Vein, Open Approach */
"06BS3ZX"  =  "1"  /* Excision of Left Lesser Saphenous Vein, Perc Approach, Diagn */
"06BS3ZZ"  =  "1"  /* Excision of Left Lesser Saphenous Vein, Perc Approach */
"06BS4ZX"  =  "1"  /* Excision of L Less Saphenous, Perc Endo Approach, Diagn */
"06BS4ZZ"  =  "1"  /* Excision of Left Lesser Saphenous Vein, Perc Endo Approach */
"06BT0ZX"  =  "1"  /* Excision of Right Foot Vein, Open Approach, Diagnostic */
"06BT0ZZ"  =  "1"  /* Excision of Right Foot Vein, Open Approach */
"06BT3ZX"  =  "1"  /* Excision of Right Foot Vein, Percutaneous Approach, Diagn */
"06BT3ZZ"  =  "1"  /* Excision of Right Foot Vein, Percutaneous Approach */
"06BT4ZX"  =  "1"  /* Excision of Right Foot Vein, Perc Endo Approach, Diagn */
"06BT4ZZ"  =  "1"  /* Excision of Right Foot Vein, Perc Endo Approach */
"06BV0ZX"  =  "1"  /* Excision of Left Foot Vein, Open Approach, Diagnostic */
"06BV0ZZ"  =  "1"  /* Excision of Left Foot Vein, Open Approach */
"06BV3ZX"  =  "1"  /* Excision of Left Foot Vein, Percutaneous Approach, Diagn */
"06BV3ZZ"  =  "1"  /* Excision of Left Foot Vein, Percutaneous Approach */
"06BV4ZX"  =  "1"  /* Excision of Left Foot Vein, Perc Endo Approach, Diagn */
"06BV4ZZ"  =  "1"  /* Excision of Left Foot Vein, Percutaneous Endoscopic Approach */
"06BY0ZC"  =  "1"  /* Excision of Hemorrhoidal Plexus, Open Approach */
"06BY0ZX"  =  "1"  /* Excision of Lower Vein, Open Approach, Diagnostic */
"06BY0ZZ"  =  "1"  /* Excision of Lower Vein, Open Approach */
"06BY3ZC"  =  "1"  /* Excision of Hemorrhoidal Plexus, Percutaneous Approach */
"06BY3ZX"  =  "1"  /* Excision of Lower Vein, Percutaneous Approach, Diagnostic */
"06BY3ZZ"  =  "1"  /* Excision of Lower Vein, Percutaneous Approach */
"06BY4ZC"  =  "1"  /* Excision of Hemorrhoidal Plexus, Perc Endo Approach */
"06BY4ZX"  =  "1"  /* Excision of Lower Vein, Perc Endo Approach, Diagn */
"06BY4ZZ"  =  "1"  /* Excision of Lower Vein, Percutaneous Endoscopic Approach */
"06C00ZZ"  =  "1"  /* Extirpation of Matter from Inferior Vena Cava, Open Approach */
"06C03ZZ"  =  "1"  /* Extirpation of Matter from Inferior Vena Cava, Perc Approach */
"06C04ZZ"  =  "1"  /* Extirpation of Matter from Inf Vena Cava, Perc Endo Approach */
"06C10ZZ"  =  "1"  /* Extirpation of Matter from Splenic Vein, Open Approach */
"06C13ZZ"  =  "1"  /* Extirpation of Matter from Splenic Vein, Perc Approach */
"06C14ZZ"  =  "1"  /* Extirpation of Matter from Splenic Vein, Perc Endo Approach */
"06C20ZZ"  =  "1"  /* Extirpation of Matter from Gastric Vein, Open Approach */
"06C23ZZ"  =  "1"  /* Extirpation of Matter from Gastric Vein, Perc Approach */
"06C24ZZ"  =  "1"  /* Extirpation of Matter from Gastric Vein, Perc Endo Approach */
"06C30ZZ"  =  "1"  /* Extirpation of Matter from Esophageal Vein, Open Approach */
"06C33ZZ"  =  "1"  /* Extirpation of Matter from Esophageal Vein, Perc Approach */
"06C34ZZ"  =  "1"  /* Extirpate of Matter from Esophageal Vein, Perc Endo Approach */
"06C40ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Vein, Open Approach */
"06C43ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Vein, Perc Approach */
"06C44ZZ"  =  "1"  /* Extirpation of Matter from Hepatic Vein, Perc Endo Approach */
"06C50ZZ"  =  "1"  /* Extirpation of Matter from Sup Mesent Vein, Open Approach */
"06C53ZZ"  =  "1"  /* Extirpation of Matter from Sup Mesent Vein, Perc Approach */
"06C54ZZ"  =  "1"  /* Extirpate of Matter from Sup Mesent Vein, Perc Endo Approach */
"06C60ZZ"  =  "1"  /* Extirpation of Matter from Inf Mesent Vein, Open Approach */
"06C63ZZ"  =  "1"  /* Extirpation of Matter from Inf Mesent Vein, Perc Approach */
"06C64ZZ"  =  "1"  /* Extirpate of Matter from Inf Mesent Vein, Perc Endo Approach */
"06C70ZZ"  =  "1"  /* Extirpation of Matter from Colic Vein, Open Approach */
"06C73ZZ"  =  "1"  /* Extirpation of Matter from Colic Vein, Percutaneous Approach */
"06C74ZZ"  =  "1"  /* Extirpation of Matter from Colic Vein, Perc Endo Approach */
"06C80ZZ"  =  "1"  /* Extirpation of Matter from Portal Vein, Open Approach */
"06C83ZZ"  =  "1"  /* Extirpation of Matter from Portal Vein, Perc Approach */
"06C84ZZ"  =  "1"  /* Extirpation of Matter from Portal Vein, Perc Endo Approach */
"06C90ZZ"  =  "1"  /* Extirpation of Matter from Right Renal Vein, Open Approach */
"06C93ZZ"  =  "1"  /* Extirpation of Matter from Right Renal Vein, Perc Approach */
"06C94ZZ"  =  "1"  /* Extirpation of Matter from R Renal Vein, Perc Endo Approach */
"06CB0ZZ"  =  "1"  /* Extirpation of Matter from Left Renal Vein, Open Approach */
"06CB3ZZ"  =  "1"  /* Extirpation of Matter from Left Renal Vein, Perc Approach */
"06CB4ZZ"  =  "1"  /* Extirpation of Matter from L Renal Vein, Perc Endo Approach */
"06CC0ZZ"  =  "1"  /* Extirpation of Matter from R Com Iliac Vein, Open Approach */
"06CC3ZZ"  =  "1"  /* Extirpation of Matter from R Com Iliac Vein, Perc Approach */
"06CC4ZZ"  =  "1"  /* Extirpate matter from R Com Iliac Vein, Perc Endo */
"06CD0ZZ"  =  "1"  /* Extirpation of Matter from L Com Iliac Vein, Open Approach */
"06CD3ZZ"  =  "1"  /* Extirpation of Matter from L Com Iliac Vein, Perc Approach */
"06CD4ZZ"  =  "1"  /* Extirpate matter from L Com Iliac Vein, Perc Endo */
"06CF0ZZ"  =  "1"  /* Extirpation of Matter from R Ext Iliac Vein, Open Approach */
"06CF3ZZ"  =  "1"  /* Extirpation of Matter from R Ext Iliac Vein, Perc Approach */
"06CF4ZZ"  =  "1"  /* Extirpate matter from R Ext Iliac Vein, Perc Endo */
"06CG0ZZ"  =  "1"  /* Extirpation of Matter from L Ext Iliac Vein, Open Approach */
"06CG3ZZ"  =  "1"  /* Extirpation of Matter from L Ext Iliac Vein, Perc Approach */
"06CG4ZZ"  =  "1"  /* Extirpate matter from L Ext Iliac Vein, Perc Endo */
"06CH0ZZ"  =  "1"  /* Extirpation of Matter from R Hypogast Vein, Open Approach */
"06CH3ZZ"  =  "1"  /* Extirpation of Matter from R Hypogast Vein, Perc Approach */
"06CH4ZZ"  =  "1"  /* Extirpate of Matter from R Hypogast Vein, Perc Endo Approach */
"06CJ0ZZ"  =  "1"  /* Extirpation of Matter from L Hypogast Vein, Open Approach */
"06CJ3ZZ"  =  "1"  /* Extirpation of Matter from L Hypogast Vein, Perc Approach */
"06CJ4ZZ"  =  "1"  /* Extirpate of Matter from L Hypogast Vein, Perc Endo Approach */
"06CM0ZZ"  =  "1"  /* Extirpation of Matter from Right Femoral Vein, Open Approach */
"06CM3ZZ"  =  "1"  /* Extirpation of Matter from Right Femoral Vein, Perc Approach */
"06CM4ZZ"  =  "1"  /* Extirpation of Matter from R Femor Vein, Perc Endo Approach */
"06CN0ZZ"  =  "1"  /* Extirpation of Matter from Left Femoral Vein, Open Approach */
"06CN3ZZ"  =  "1"  /* Extirpation of Matter from Left Femoral Vein, Perc Approach */
"06CN4ZZ"  =  "1"  /* Extirpation of Matter from L Femor Vein, Perc Endo Approach */
"06CP0ZZ"  =  "1"  /* Extirpation of Matter from R Great Saphenous, Open Approach */
"06CP3ZZ"  =  "1"  /* Extirpation of Matter from R Great Saphenous, Perc Approach */
"06CP4ZZ"  =  "1"  /* Extirpate matter from R Great Saphenous, Perc Endo */
"06CQ0ZZ"  =  "1"  /* Extirpation of Matter from L Great Saphenous, Open Approach */
"06CQ3ZZ"  =  "1"  /* Extirpation of Matter from L Great Saphenous, Perc Approach */
"06CQ4ZZ"  =  "1"  /* Extirpate matter from L Great Saphenous, Perc Endo */
"06CR0ZZ"  =  "1"  /* Extirpation of Matter from R Less Saphenous, Open Approach */
"06CR3ZZ"  =  "1"  /* Extirpation of Matter from R Less Saphenous, Perc Approach */
"06CR4ZZ"  =  "1"  /* Extirpate matter from R Less Saphenous, Perc Endo */
"06CS0ZZ"  =  "1"  /* Extirpation of Matter from L Less Saphenous, Open Approach */
"06CS3ZZ"  =  "1"  /* Extirpation of Matter from L Less Saphenous, Perc Approach */
"06CS4ZZ"  =  "1"  /* Extirpate matter from L Less Saphenous, Perc Endo */
"06CT0ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Vein, Open Approach */
"06CT3ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Vein, Perc Approach */
"06CT4ZZ"  =  "1"  /* Extirpation of Matter from R Foot Vein, Perc Endo Approach */
"06CV0ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Vein, Open Approach */
"06CV3ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Vein, Perc Approach */
"06CV4ZZ"  =  "1"  /* Extirpation of Matter from L Foot Vein, Perc Endo Approach */
"06CY0ZZ"  =  "1"  /* Extirpation of Matter from Lower Vein, Open Approach */
"06CY3ZZ"  =  "1"  /* Extirpation of Matter from Lower Vein, Percutaneous Approach */
"06CY4ZZ"  =  "1"  /* Extirpation of Matter from Lower Vein, Perc Endo Approach */
"06DM0ZZ"  =  "1"  /* Extraction of Right Femoral Vein, Open Approach */
"06DM3ZZ"  =  "1"  /* Extraction of Right Femoral Vein, Percutaneous Approach */
"06DM4ZZ"  =  "1"  /* Extraction of Right Femoral Vein, Perc Endo Approach */
"06DN0ZZ"  =  "1"  /* Extraction of Left Femoral Vein, Open Approach */
"06DN3ZZ"  =  "1"  /* Extraction of Left Femoral Vein, Percutaneous Approach */
"06DN4ZZ"  =  "1"  /* Extraction of Left Femoral Vein, Perc Endo Approach */
"06DP0ZZ"  =  "1"  /* Extraction of Right Greater Saphenous Vein, Open Approach */
"06DP3ZZ"  =  "1"  /* Extraction of Right Greater Saphenous Vein, Perc Approach */
"06DP4ZZ"  =  "1"  /* Extraction of R Great Saphenous, Perc Endo Approach */
"06DQ0ZZ"  =  "1"  /* Extraction of Left Greater Saphenous Vein, Open Approach */
"06DQ3ZZ"  =  "1"  /* Extraction of Left Greater Saphenous Vein, Perc Approach */
"06DQ4ZZ"  =  "1"  /* Extraction of L Great Saphenous, Perc Endo Approach */
"06DR0ZZ"  =  "1"  /* Extraction of Right Lesser Saphenous Vein, Open Approach */
"06DR3ZZ"  =  "1"  /* Extraction of Right Lesser Saphenous Vein, Perc Approach */
"06DR4ZZ"  =  "1"  /* Extraction of R Less Saphenous, Perc Endo Approach */
"06DS0ZZ"  =  "1"  /* Extraction of Left Lesser Saphenous Vein, Open Approach */
"06DS3ZZ"  =  "1"  /* Extraction of Left Lesser Saphenous Vein, Perc Approach */
"06DS4ZZ"  =  "1"  /* Extraction of Left Lesser Saphenous Vein, Perc Endo Approach */
"06DT0ZZ"  =  "1"  /* Extraction of Right Foot Vein, Open Approach */
"06DT3ZZ"  =  "1"  /* Extraction of Right Foot Vein, Percutaneous Approach */
"06DT4ZZ"  =  "1"  /* Extraction of Right Foot Vein, Perc Endo Approach */
"06DV0ZZ"  =  "1"  /* Extraction of Left Foot Vein, Open Approach */
"06DV3ZZ"  =  "1"  /* Extraction of Left Foot Vein, Percutaneous Approach */
"06DV4ZZ"  =  "1"  /* Extraction of Left Foot Vein, Perc Endo Approach */
"06DY0ZZ"  =  "1"  /* Extraction of Lower Vein, Open Approach */
"06DY3ZZ"  =  "1"  /* Extraction of Lower Vein, Percutaneous Approach */
"06DY4ZZ"  =  "1"  /* Extraction of Lower Vein, Percutaneous Endoscopic Approach */
"06H00DZ"  =  "1"  /* Insertion of Intralum Dev into Inf Vena Cava, Open Approach */
"06H03DZ"  =  "1"  /* Insertion of Intralum Dev into Inf Vena Cava, Perc Approach */
"06H04DZ"  =  "1"  /* Insert Intralum Dev in Inf Vena Cava, Perc Endo */
"06H10DZ"  =  "1"  /* Insertion of Intralum Dev into Splenic Vein, Open Approach */
"06H13DZ"  =  "1"  /* Insertion of Intralum Dev into Splenic Vein, Perc Approach */
"06H14DZ"  =  "1"  /* Insert of Intralum Dev into Splenic Vein, Perc Endo Approach */
"06H20DZ"  =  "1"  /* Insertion of Intralum Dev into Gastric Vein, Open Approach */
"06H23DZ"  =  "1"  /* Insertion of Intralum Dev into Gastric Vein, Perc Approach */
"06H24DZ"  =  "1"  /* Insert of Intralum Dev into Gastric Vein, Perc Endo Approach */
"06H30DZ"  =  "1"  /* Insert of Intralum Dev into Esophageal Vein, Open Approach */
"06H33DZ"  =  "1"  /* Insert of Intralum Dev into Esophageal Vein, Perc Approach */
"06H34DZ"  =  "1"  /* Insert Intralum Dev in Esophageal Vein, Perc Endo */
"06H40DZ"  =  "1"  /* Insertion of Intralum Dev into Hepatic Vein, Open Approach */
"06H43DZ"  =  "1"  /* Insertion of Intralum Dev into Hepatic Vein, Perc Approach */
"06H44DZ"  =  "1"  /* Insert of Intralum Dev into Hepatic Vein, Perc Endo Approach */
"06H50DZ"  =  "1"  /* Insert of Intralum Dev into Sup Mesent Vein, Open Approach */
"06H53DZ"  =  "1"  /* Insert of Intralum Dev into Sup Mesent Vein, Perc Approach */
"06H54DZ"  =  "1"  /* Insert Intralum Dev in Sup Mesent Vein, Perc Endo */
"06H60DZ"  =  "1"  /* Insert of Intralum Dev into Inf Mesent Vein, Open Approach */
"06H63DZ"  =  "1"  /* Insert of Intralum Dev into Inf Mesent Vein, Perc Approach */
"06H64DZ"  =  "1"  /* Insert Intralum Dev in Inf Mesent Vein, Perc Endo */
"06H70DZ"  =  "1"  /* Insertion of Intralum Dev into Colic Vein, Open Approach */
"06H73DZ"  =  "1"  /* Insertion of Intralum Dev into Colic Vein, Perc Approach */
"06H74DZ"  =  "1"  /* Insert of Intralum Dev into Colic Vein, Perc Endo Approach */
"06H80DZ"  =  "1"  /* Insertion of Intralum Dev into Portal Vein, Open Approach */
"06H83DZ"  =  "1"  /* Insertion of Intralum Dev into Portal Vein, Perc Approach */
"06H84DZ"  =  "1"  /* Insert of Intralum Dev into Portal Vein, Perc Endo Approach */
"06H90DZ"  =  "1"  /* Insertion of Intralum Dev into R Renal Vein, Open Approach */
"06H93DZ"  =  "1"  /* Insertion of Intralum Dev into R Renal Vein, Perc Approach */
"06H94DZ"  =  "1"  /* Insert of Intralum Dev into R Renal Vein, Perc Endo Approach */
"06HB0DZ"  =  "1"  /* Insertion of Intralum Dev into L Renal Vein, Open Approach */
"06HB3DZ"  =  "1"  /* Insertion of Intralum Dev into L Renal Vein, Perc Approach */
"06HB4DZ"  =  "1"  /* Insert of Intralum Dev into L Renal Vein, Perc Endo Approach */
"06HC0DZ"  =  "1"  /* Insert of Intralum Dev into R Com Iliac Vein, Open Approach */
"06HC3DZ"  =  "1"  /* Insert of Intralum Dev into R Com Iliac Vein, Perc Approach */
"06HC4DZ"  =  "1"  /* Insert Intralum Dev in R Com Iliac Vein, Perc Endo */
"06HD0DZ"  =  "1"  /* Insert of Intralum Dev into L Com Iliac Vein, Open Approach */
"06HD3DZ"  =  "1"  /* Insert of Intralum Dev into L Com Iliac Vein, Perc Approach */
"06HD4DZ"  =  "1"  /* Insert Intralum Dev in L Com Iliac Vein, Perc Endo */
"06HF0DZ"  =  "1"  /* Insert of Intralum Dev into R Ext Iliac Vein, Open Approach */
"06HF3DZ"  =  "1"  /* Insert of Intralum Dev into R Ext Iliac Vein, Perc Approach */
"06HF4DZ"  =  "1"  /* Insert Intralum Dev in R Ext Iliac Vein, Perc Endo */
"06HG0DZ"  =  "1"  /* Insert of Intralum Dev into L Ext Iliac Vein, Open Approach */
"06HG3DZ"  =  "1"  /* Insert of Intralum Dev into L Ext Iliac Vein, Perc Approach */
"06HG4DZ"  =  "1"  /* Insert Intralum Dev in L Ext Iliac Vein, Perc Endo */
"06HH0DZ"  =  "1"  /* Insert of Intralum Dev into R Hypogast Vein, Open Approach */
"06HH3DZ"  =  "1"  /* Insert of Intralum Dev into R Hypogast Vein, Perc Approach */
"06HH4DZ"  =  "1"  /* Insert Intralum Dev in R Hypogast Vein, Perc Endo */
"06HJ0DZ"  =  "1"  /* Insert of Intralum Dev into L Hypogast Vein, Open Approach */
"06HJ3DZ"  =  "1"  /* Insert of Intralum Dev into L Hypogast Vein, Perc Approach */
"06HJ4DZ"  =  "1"  /* Insert Intralum Dev in L Hypogast Vein, Perc Endo */
"06HM0DZ"  =  "1"  /* Insertion of Intralum Dev into R Femor Vein, Open Approach */
"06HM3DZ"  =  "1"  /* Insertion of Intralum Dev into R Femor Vein, Perc Approach */
"06HM4DZ"  =  "1"  /* Insert of Intralum Dev into R Femor Vein, Perc Endo Approach */
"06HN0DZ"  =  "1"  /* Insertion of Intralum Dev into L Femor Vein, Open Approach */
"06HN3DZ"  =  "1"  /* Insertion of Intralum Dev into L Femor Vein, Perc Approach */
"06HN4DZ"  =  "1"  /* Insert of Intralum Dev into L Femor Vein, Perc Endo Approach */
"06HP0DZ"  =  "1"  /* Insert of Intralum Dev into R Great Saphenous, Open Approach */
"06HP3DZ"  =  "1"  /* Insert of Intralum Dev into R Great Saphenous, Perc Approach */
"06HP4DZ"  =  "1"  /* Insert Intralum Dev in R Great Saphenous, Perc Endo */
"06HQ0DZ"  =  "1"  /* Insert of Intralum Dev into L Great Saphenous, Open Approach */
"06HQ3DZ"  =  "1"  /* Insert of Intralum Dev into L Great Saphenous, Perc Approach */
"06HQ4DZ"  =  "1"  /* Insert Intralum Dev in L Great Saphenous, Perc Endo */
"06HR0DZ"  =  "1"  /* Insert of Intralum Dev into R Less Saphenous, Open Approach */
"06HR3DZ"  =  "1"  /* Insert of Intralum Dev into R Less Saphenous, Perc Approach */
"06HR4DZ"  =  "1"  /* Insert Intralum Dev in R Less Saphenous, Perc Endo */
"06HS0DZ"  =  "1"  /* Insert of Intralum Dev into L Less Saphenous, Open Approach */
"06HS3DZ"  =  "1"  /* Insert of Intralum Dev into L Less Saphenous, Perc Approach */
"06HS4DZ"  =  "1"  /* Insert Intralum Dev in L Less Saphenous, Perc Endo */
"06HT0DZ"  =  "1"  /* Insertion of Intralum Dev into R Foot Vein, Open Approach */
"06HT3DZ"  =  "1"  /* Insertion of Intralum Dev into R Foot Vein, Perc Approach */
"06HT4DZ"  =  "1"  /* Insert of Intralum Dev into R Foot Vein, Perc Endo Approach */
"06HV0DZ"  =  "1"  /* Insertion of Intralum Dev into L Foot Vein, Open Approach */
"06HV3DZ"  =  "1"  /* Insertion of Intralum Dev into L Foot Vein, Perc Approach */
"06HV4DZ"  =  "1"  /* Insert of Intralum Dev into L Foot Vein, Perc Endo Approach */
"06HY02Z"  =  "1"  /* Insertion of Monitoring Device into Low Vein, Open Approach */
"06HY0DZ"  =  "1"  /* Insertion of Intralum Dev into Low Vein, Open Approach */
"06HY32Z"  =  "1"  /* Insertion of Monitoring Device into Low Vein, Perc Approach */
"06HY3DZ"  =  "1"  /* Insertion of Intralum Dev into Low Vein, Perc Approach */
"06HY42Z"  =  "1"  /* Insertion of Monitor Dev into Low Vein, Perc Endo Approach */
"06HY4DZ"  =  "1"  /* Insertion of Intralum Dev into Low Vein, Perc Endo Approach */
"06JY0ZZ"  =  "1"  /* Inspection of Lower Vein, Open Approach */
"06JY3ZZ"  =  "1"  /* Inspection of Lower Vein, Percutaneous Approach */
"06JY4ZZ"  =  "1"  /* Inspection of Lower Vein, Percutaneous Endoscopic Approach */
"06L00CZ"  =  "1"  /* Occlusion of Inf Vena Cava with Extralum Dev, Open Approach */
"06L00DZ"  =  "1"  /* Occlusion of Inf Vena Cava with Intralum Dev, Open Approach */
"06L00ZZ"  =  "1"  /* Occlusion of Inferior Vena Cava, Open Approach */
"06L03CZ"  =  "1"  /* Occlusion of Inf Vena Cava with Extralum Dev, Perc Approach */
"06L03DZ"  =  "1"  /* Occlusion of Inf Vena Cava with Intralum Dev, Perc Approach */
"06L03ZZ"  =  "1"  /* Occlusion of Inferior Vena Cava, Percutaneous Approach */
"06L04CZ"  =  "1"  /* Occlusion Inf Vena Cava w Extralum Dev, Perc Endo */
"06L04DZ"  =  "1"  /* Occlusion Inf Vena Cava w Intralum Dev, Perc Endo */
"06L04ZZ"  =  "1"  /* Occlusion of Inferior Vena Cava, Perc Endo Approach */
"06L10CZ"  =  "1"  /* Occlusion of Splenic Vein with Extralum Dev, Open Approach */
"06L10DZ"  =  "1"  /* Occlusion of Splenic Vein with Intralum Dev, Open Approach */
"06L10ZZ"  =  "1"  /* Occlusion of Splenic Vein, Open Approach */
"06L13CZ"  =  "1"  /* Occlusion of Splenic Vein with Extralum Dev, Perc Approach */
"06L13DZ"  =  "1"  /* Occlusion of Splenic Vein with Intralum Dev, Perc Approach */
"06L13ZZ"  =  "1"  /* Occlusion of Splenic Vein, Percutaneous Approach */
"06L14CZ"  =  "1"  /* Occlusion Splenic Vein w Extralum Dev, Perc Endo */
"06L14DZ"  =  "1"  /* Occlusion Splenic Vein w Intralum Dev, Perc Endo */
"06L14ZZ"  =  "1"  /* Occlusion of Splenic Vein, Percutaneous Endoscopic Approach */
"06L20CZ"  =  "1"  /* Occlusion of Gastric Vein with Extralum Dev, Open Approach */
"06L20DZ"  =  "1"  /* Occlusion of Gastric Vein with Intralum Dev, Open Approach */
"06L20ZZ"  =  "1"  /* Occlusion of Gastric Vein, Open Approach */
"06L23CZ"  =  "1"  /* Occlusion of Gastric Vein with Extralum Dev, Perc Approach */
"06L23DZ"  =  "1"  /* Occlusion of Gastric Vein with Intralum Dev, Perc Approach */
"06L23ZZ"  =  "1"  /* Occlusion of Gastric Vein, Percutaneous Approach */
"06L24CZ"  =  "1"  /* Occlusion Gastric Vein w Extralum Dev, Perc Endo */
"06L24DZ"  =  "1"  /* Occlusion Gastric Vein w Intralum Dev, Perc Endo */
"06L24ZZ"  =  "1"  /* Occlusion of Gastric Vein, Percutaneous Endoscopic Approach */
"06L30CZ"  =  "1"  /* Occlusion Esophageal Vein w Extralum Dev, Open */
"06L30DZ"  =  "1"  /* Occlusion Esophageal Vein w Intralum Dev, Open */
"06L30ZZ"  =  "1"  /* Occlusion of Esophageal Vein, Open Approach */
"06L33CZ"  =  "1"  /* Occlusion Esophageal Vein w Extralum Dev, Perc */
"06L33DZ"  =  "1"  /* Occlusion Esophageal Vein w Intralum Dev, Perc */
"06L33ZZ"  =  "1"  /* Occlusion of Esophageal Vein, Percutaneous Approach */
"06L34CZ"  =  "1"  /* Occlusion Esophageal Vein w Extralum Dev, Perc Endo */
"06L34DZ"  =  "1"  /* Occlusion Esophageal Vein w Intralum Dev, Perc Endo */
"06L34ZZ"  =  "1"  /* Occlusion of Esophageal Vein, Perc Endo Approach */
"06L40CZ"  =  "1"  /* Occlusion of Hepatic Vein with Extralum Dev, Open Approach */
"06L40DZ"  =  "1"  /* Occlusion of Hepatic Vein with Intralum Dev, Open Approach */
"06L40ZZ"  =  "1"  /* Occlusion of Hepatic Vein, Open Approach */
"06L43CZ"  =  "1"  /* Occlusion of Hepatic Vein with Extralum Dev, Perc Approach */
"06L43DZ"  =  "1"  /* Occlusion of Hepatic Vein with Intralum Dev, Perc Approach */
"06L43ZZ"  =  "1"  /* Occlusion of Hepatic Vein, Percutaneous Approach */
"06L44CZ"  =  "1"  /* Occlusion Hepatic Vein w Extralum Dev, Perc Endo */
"06L44DZ"  =  "1"  /* Occlusion Hepatic Vein w Intralum Dev, Perc Endo */
"06L44ZZ"  =  "1"  /* Occlusion of Hepatic Vein, Percutaneous Endoscopic Approach */
"06L50CZ"  =  "1"  /* Occlusion Sup Mesent Vein w Extralum Dev, Open */
"06L50DZ"  =  "1"  /* Occlusion Sup Mesent Vein w Intralum Dev, Open */
"06L50ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein, Open Approach */
"06L53CZ"  =  "1"  /* Occlusion Sup Mesent Vein w Extralum Dev, Perc */
"06L53DZ"  =  "1"  /* Occlusion Sup Mesent Vein w Intralum Dev, Perc */
"06L53ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein, Percutaneous Approach */
"06L54CZ"  =  "1"  /* Occlusion Sup Mesent Vein w Extralum Dev, Perc Endo */
"06L54DZ"  =  "1"  /* Occlusion Sup Mesent Vein w Intralum Dev, Perc Endo */
"06L54ZZ"  =  "1"  /* Occlusion of Superior Mesenteric Vein, Perc Endo Approach */
"06L60CZ"  =  "1"  /* Occlusion Inf Mesent Vein w Extralum Dev, Open */
"06L60DZ"  =  "1"  /* Occlusion Inf Mesent Vein w Intralum Dev, Open */
"06L60ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein, Open Approach */
"06L63CZ"  =  "1"  /* Occlusion Inf Mesent Vein w Extralum Dev, Perc */
"06L63DZ"  =  "1"  /* Occlusion Inf Mesent Vein w Intralum Dev, Perc */
"06L63ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein, Percutaneous Approach */
"06L64CZ"  =  "1"  /* Occlusion Inf Mesent Vein w Extralum Dev, Perc Endo */
"06L64DZ"  =  "1"  /* Occlusion Inf Mesent Vein w Intralum Dev, Perc Endo */
"06L64ZZ"  =  "1"  /* Occlusion of Inferior Mesenteric Vein, Perc Endo Approach */
"06L70CZ"  =  "1"  /* Occlusion of Colic Vein with Extralum Dev, Open Approach */
"06L70DZ"  =  "1"  /* Occlusion of Colic Vein with Intralum Dev, Open Approach */
"06L70ZZ"  =  "1"  /* Occlusion of Colic Vein, Open Approach */
"06L73CZ"  =  "1"  /* Occlusion of Colic Vein with Extralum Dev, Perc Approach */
"06L73DZ"  =  "1"  /* Occlusion of Colic Vein with Intralum Dev, Perc Approach */
"06L73ZZ"  =  "1"  /* Occlusion of Colic Vein, Percutaneous Approach */
"06L74CZ"  =  "1"  /* Occlusion Colic Vein w Extralum Dev, Perc Endo */
"06L74DZ"  =  "1"  /* Occlusion Colic Vein w Intralum Dev, Perc Endo */
"06L74ZZ"  =  "1"  /* Occlusion of Colic Vein, Percutaneous Endoscopic Approach */
"06L80CZ"  =  "1"  /* Occlusion of Portal Vein with Extralum Dev, Open Approach */
"06L80DZ"  =  "1"  /* Occlusion of Portal Vein with Intralum Dev, Open Approach */
"06L80ZZ"  =  "1"  /* Occlusion of Portal Vein, Open Approach */
"06L83CZ"  =  "1"  /* Occlusion of Portal Vein with Extralum Dev, Perc Approach */
"06L83DZ"  =  "1"  /* Occlusion of Portal Vein with Intralum Dev, Perc Approach */
"06L83ZZ"  =  "1"  /* Occlusion of Portal Vein, Percutaneous Approach */
"06L84CZ"  =  "1"  /* Occlusion Portal Vein w Extralum Dev, Perc Endo */
"06L84DZ"  =  "1"  /* Occlusion Portal Vein w Intralum Dev, Perc Endo */
"06L84ZZ"  =  "1"  /* Occlusion of Portal Vein, Percutaneous Endoscopic Approach */
"06L90CZ"  =  "1"  /* Occlusion of R Renal Vein with Extralum Dev, Open Approach */
"06L90DZ"  =  "1"  /* Occlusion of R Renal Vein with Intralum Dev, Open Approach */
"06L90ZZ"  =  "1"  /* Occlusion of Right Renal Vein, Open Approach */
"06L93CZ"  =  "1"  /* Occlusion of R Renal Vein with Extralum Dev, Perc Approach */
"06L93DZ"  =  "1"  /* Occlusion of R Renal Vein with Intralum Dev, Perc Approach */
"06L93ZZ"  =  "1"  /* Occlusion of Right Renal Vein, Percutaneous Approach */
"06L94CZ"  =  "1"  /* Occlusion R Renal Vein w Extralum Dev, Perc Endo */
"06L94DZ"  =  "1"  /* Occlusion R Renal Vein w Intralum Dev, Perc Endo */
"06L94ZZ"  =  "1"  /* Occlusion of Right Renal Vein, Perc Endo Approach */
"06LB0CZ"  =  "1"  /* Occlusion of L Renal Vein with Extralum Dev, Open Approach */
"06LB0DZ"  =  "1"  /* Occlusion of L Renal Vein with Intralum Dev, Open Approach */
"06LB0ZZ"  =  "1"  /* Occlusion of Left Renal Vein, Open Approach */
"06LB3CZ"  =  "1"  /* Occlusion of L Renal Vein with Extralum Dev, Perc Approach */
"06LB3DZ"  =  "1"  /* Occlusion of L Renal Vein with Intralum Dev, Perc Approach */
"06LB3ZZ"  =  "1"  /* Occlusion of Left Renal Vein, Percutaneous Approach */
"06LB4CZ"  =  "1"  /* Occlusion L Renal Vein w Extralum Dev, Perc Endo */
"06LB4DZ"  =  "1"  /* Occlusion L Renal Vein w Intralum Dev, Perc Endo */
"06LB4ZZ"  =  "1"  /* Occlusion of Left Renal Vein, Perc Endo Approach */
"06LC0CZ"  =  "1"  /* Occlusion R Com Iliac Vein w Extralum Dev, Open */
"06LC0DZ"  =  "1"  /* Occlusion R Com Iliac Vein w Intralum Dev, Open */
"06LC0ZZ"  =  "1"  /* Occlusion of Right Common Iliac Vein, Open Approach */
"06LC3CZ"  =  "1"  /* Occlusion R Com Iliac Vein w Extralum Dev, Perc */
"06LC3DZ"  =  "1"  /* Occlusion R Com Iliac Vein w Intralum Dev, Perc */
"06LC3ZZ"  =  "1"  /* Occlusion of Right Common Iliac Vein, Percutaneous Approach */
"06LC4CZ"  =  "1"  /* Occlusion R Com Iliac Vein w Extralum Dev, Perc Endo */
"06LC4DZ"  =  "1"  /* Occlusion R Com Iliac Vein w Intralum Dev, Perc Endo */
"06LC4ZZ"  =  "1"  /* Occlusion of Right Common Iliac Vein, Perc Endo Approach */
"06LD0CZ"  =  "1"  /* Occlusion L Com Iliac Vein w Extralum Dev, Open */
"06LD0DZ"  =  "1"  /* Occlusion L Com Iliac Vein w Intralum Dev, Open */
"06LD0ZZ"  =  "1"  /* Occlusion of Left Common Iliac Vein, Open Approach */
"06LD3CZ"  =  "1"  /* Occlusion L Com Iliac Vein w Extralum Dev, Perc */
"06LD3DZ"  =  "1"  /* Occlusion L Com Iliac Vein w Intralum Dev, Perc */
"06LD3ZZ"  =  "1"  /* Occlusion of Left Common Iliac Vein, Percutaneous Approach */
"06LD4CZ"  =  "1"  /* Occlusion L Com Iliac Vein w Extralum Dev, Perc Endo */
"06LD4DZ"  =  "1"  /* Occlusion L Com Iliac Vein w Intralum Dev, Perc Endo */
"06LD4ZZ"  =  "1"  /* Occlusion of Left Common Iliac Vein, Perc Endo Approach */
"06LF0CZ"  =  "1"  /* Occlusion R Ext Iliac Vein w Extralum Dev, Open */
"06LF0DZ"  =  "1"  /* Occlusion R Ext Iliac Vein w Intralum Dev, Open */
"06LF0ZZ"  =  "1"  /* Occlusion of Right External Iliac Vein, Open Approach */
"06LF3CZ"  =  "1"  /* Occlusion R Ext Iliac Vein w Extralum Dev, Perc */
"06LF3DZ"  =  "1"  /* Occlusion R Ext Iliac Vein w Intralum Dev, Perc */
"06LF3ZZ"  =  "1"  /* Occlusion of Right External Iliac Vein, Perc Approach */
"06LF4CZ"  =  "1"  /* Occlusion R Ext Iliac Vein w Extralum Dev, Perc Endo */
"06LF4DZ"  =  "1"  /* Occlusion R Ext Iliac Vein w Intralum Dev, Perc Endo */
"06LF4ZZ"  =  "1"  /* Occlusion of Right External Iliac Vein, Perc Endo Approach */
"06LG0CZ"  =  "1"  /* Occlusion L Ext Iliac Vein w Extralum Dev, Open */
"06LG0DZ"  =  "1"  /* Occlusion L Ext Iliac Vein w Intralum Dev, Open */
"06LG0ZZ"  =  "1"  /* Occlusion of Left External Iliac Vein, Open Approach */
"06LG3CZ"  =  "1"  /* Occlusion L Ext Iliac Vein w Extralum Dev, Perc */
"06LG3DZ"  =  "1"  /* Occlusion L Ext Iliac Vein w Intralum Dev, Perc */
"06LG3ZZ"  =  "1"  /* Occlusion of Left External Iliac Vein, Percutaneous Approach */
"06LG4CZ"  =  "1"  /* Occlusion L Ext Iliac Vein w Extralum Dev, Perc Endo */
"06LG4DZ"  =  "1"  /* Occlusion L Ext Iliac Vein w Intralum Dev, Perc Endo */
"06LG4ZZ"  =  "1"  /* Occlusion of Left External Iliac Vein, Perc Endo Approach */
"06LH0CZ"  =  "1"  /* Occlusion R Hypogast Vein w Extralum Dev, Open */
"06LH0DZ"  =  "1"  /* Occlusion R Hypogast Vein w Intralum Dev, Open */
"06LH0ZZ"  =  "1"  /* Occlusion of Right Hypogastric Vein, Open Approach */
"06LH3CZ"  =  "1"  /* Occlusion R Hypogast Vein w Extralum Dev, Perc */
"06LH3DZ"  =  "1"  /* Occlusion R Hypogast Vein w Intralum Dev, Perc */
"06LH3ZZ"  =  "1"  /* Occlusion of Right Hypogastric Vein, Percutaneous Approach */
"06LH4CZ"  =  "1"  /* Occlusion R Hypogast Vein w Extralum Dev, Perc Endo */
"06LH4DZ"  =  "1"  /* Occlusion R Hypogast Vein w Intralum Dev, Perc Endo */
"06LH4ZZ"  =  "1"  /* Occlusion of Right Hypogastric Vein, Perc Endo Approach */
"06LJ0CZ"  =  "1"  /* Occlusion L Hypogast Vein w Extralum Dev, Open */
"06LJ0DZ"  =  "1"  /* Occlusion L Hypogast Vein w Intralum Dev, Open */
"06LJ0ZZ"  =  "1"  /* Occlusion of Left Hypogastric Vein, Open Approach */
"06LJ3CZ"  =  "1"  /* Occlusion L Hypogast Vein w Extralum Dev, Perc */
"06LJ3DZ"  =  "1"  /* Occlusion L Hypogast Vein w Intralum Dev, Perc */
"06LJ3ZZ"  =  "1"  /* Occlusion of Left Hypogastric Vein, Percutaneous Approach */
"06LJ4CZ"  =  "1"  /* Occlusion L Hypogast Vein w Extralum Dev, Perc Endo */
"06LJ4DZ"  =  "1"  /* Occlusion L Hypogast Vein w Intralum Dev, Perc Endo */
"06LJ4ZZ"  =  "1"  /* Occlusion of Left Hypogastric Vein, Perc Endo Approach */
"06LM0CZ"  =  "1"  /* Occlusion of R Femor Vein with Extralum Dev, Open Approach */
"06LM0DZ"  =  "1"  /* Occlusion of R Femor Vein with Intralum Dev, Open Approach */
"06LM0ZZ"  =  "1"  /* Occlusion of Right Femoral Vein, Open Approach */
"06LM3CZ"  =  "1"  /* Occlusion of R Femor Vein with Extralum Dev, Perc Approach */
"06LM3DZ"  =  "1"  /* Occlusion of R Femor Vein with Intralum Dev, Perc Approach */
"06LM3ZZ"  =  "1"  /* Occlusion of Right Femoral Vein, Percutaneous Approach */
"06LM4CZ"  =  "1"  /* Occlusion R Femor Vein w Extralum Dev, Perc Endo */
"06LM4DZ"  =  "1"  /* Occlusion R Femor Vein w Intralum Dev, Perc Endo */
"06LM4ZZ"  =  "1"  /* Occlusion of Right Femoral Vein, Perc Endo Approach */
"06LN0CZ"  =  "1"  /* Occlusion of L Femor Vein with Extralum Dev, Open Approach */
"06LN0DZ"  =  "1"  /* Occlusion of L Femor Vein with Intralum Dev, Open Approach */
"06LN0ZZ"  =  "1"  /* Occlusion of Left Femoral Vein, Open Approach */
"06LN3CZ"  =  "1"  /* Occlusion of L Femor Vein with Extralum Dev, Perc Approach */
"06LN3DZ"  =  "1"  /* Occlusion of L Femor Vein with Intralum Dev, Perc Approach */
"06LN3ZZ"  =  "1"  /* Occlusion of Left Femoral Vein, Percutaneous Approach */
"06LN4CZ"  =  "1"  /* Occlusion L Femor Vein w Extralum Dev, Perc Endo */
"06LN4DZ"  =  "1"  /* Occlusion L Femor Vein w Intralum Dev, Perc Endo */
"06LN4ZZ"  =  "1"  /* Occlusion of Left Femoral Vein, Perc Endo Approach */
"06LP0CZ"  =  "1"  /* Occlusion R Great Saphenous w Extralum Dev, Open */
"06LP0DZ"  =  "1"  /* Occlusion R Great Saphenous w Intralum Dev, Open */
"06LP0ZZ"  =  "1"  /* Occlusion of Right Greater Saphenous Vein, Open Approach */
"06LP3CZ"  =  "1"  /* Occlusion R Great Saphenous w Extralum Dev, Perc */
"06LP3DZ"  =  "1"  /* Occlusion R Great Saphenous w Intralum Dev, Perc */
"06LP3ZZ"  =  "1"  /* Occlusion of Right Greater Saphenous Vein, Perc Approach */
"06LP4CZ"  =  "1"  /* Occlusion R Great Saphenous w Extralum Dev, Perc Endo */
"06LP4DZ"  =  "1"  /* Occlusion R Great Saphenous w Intralum Dev, Perc Endo */
"06LP4ZZ"  =  "1"  /* Occlusion of R Great Saphenous, Perc Endo Approach */
"06LQ0CZ"  =  "1"  /* Occlusion L Great Saphenous w Extralum Dev, Open */
"06LQ0DZ"  =  "1"  /* Occlusion L Great Saphenous w Intralum Dev, Open */
"06LQ0ZZ"  =  "1"  /* Occlusion of Left Greater Saphenous Vein, Open Approach */
"06LQ3CZ"  =  "1"  /* Occlusion L Great Saphenous w Extralum Dev, Perc */
"06LQ3DZ"  =  "1"  /* Occlusion L Great Saphenous w Intralum Dev, Perc */
"06LQ3ZZ"  =  "1"  /* Occlusion of Left Greater Saphenous Vein, Perc Approach */
"06LQ4CZ"  =  "1"  /* Occlusion L Great Saphenous w Extralum Dev, Perc Endo */
"06LQ4DZ"  =  "1"  /* Occlusion L Great Saphenous w Intralum Dev, Perc Endo */
"06LQ4ZZ"  =  "1"  /* Occlusion of Left Greater Saphenous Vein, Perc Endo Approach */
"06LR0CZ"  =  "1"  /* Occlusion R Less Saphenous w Extralum Dev, Open */
"06LR0DZ"  =  "1"  /* Occlusion R Less Saphenous w Intralum Dev, Open */
"06LR0ZZ"  =  "1"  /* Occlusion of Right Lesser Saphenous Vein, Open Approach */
"06LR3CZ"  =  "1"  /* Occlusion R Less Saphenous w Extralum Dev, Perc */
"06LR3DZ"  =  "1"  /* Occlusion R Less Saphenous w Intralum Dev, Perc */
"06LR3ZZ"  =  "1"  /* Occlusion of Right Lesser Saphenous Vein, Perc Approach */
"06LR4CZ"  =  "1"  /* Occlusion R Less Saphenous w Extralum Dev, Perc Endo */
"06LR4DZ"  =  "1"  /* Occlusion R Less Saphenous w Intralum Dev, Perc Endo */
"06LR4ZZ"  =  "1"  /* Occlusion of Right Lesser Saphenous Vein, Perc Endo Approach */
"06LS0CZ"  =  "1"  /* Occlusion L Less Saphenous w Extralum Dev, Open */
"06LS0DZ"  =  "1"  /* Occlusion L Less Saphenous w Intralum Dev, Open */
"06LS0ZZ"  =  "1"  /* Occlusion of Left Lesser Saphenous Vein, Open Approach */
"06LS3CZ"  =  "1"  /* Occlusion L Less Saphenous w Extralum Dev, Perc */
"06LS3DZ"  =  "1"  /* Occlusion L Less Saphenous w Intralum Dev, Perc */
"06LS3ZZ"  =  "1"  /* Occlusion of Left Lesser Saphenous Vein, Perc Approach */
"06LS4CZ"  =  "1"  /* Occlusion L Less Saphenous w Extralum Dev, Perc Endo */
"06LS4DZ"  =  "1"  /* Occlusion L Less Saphenous w Intralum Dev, Perc Endo */
"06LS4ZZ"  =  "1"  /* Occlusion of Left Lesser Saphenous Vein, Perc Endo Approach */
"06LT0CZ"  =  "1"  /* Occlusion of R Foot Vein with Extralum Dev, Open Approach */
"06LT0DZ"  =  "1"  /* Occlusion of R Foot Vein with Intralum Dev, Open Approach */
"06LT0ZZ"  =  "1"  /* Occlusion of Right Foot Vein, Open Approach */
"06LT3CZ"  =  "1"  /* Occlusion of R Foot Vein with Extralum Dev, Perc Approach */
"06LT3DZ"  =  "1"  /* Occlusion of R Foot Vein with Intralum Dev, Perc Approach */
"06LT3ZZ"  =  "1"  /* Occlusion of Right Foot Vein, Percutaneous Approach */
"06LT4CZ"  =  "1"  /* Occlusion R Foot Vein w Extralum Dev, Perc Endo */
"06LT4DZ"  =  "1"  /* Occlusion R Foot Vein w Intralum Dev, Perc Endo */
"06LT4ZZ"  =  "1"  /* Occlusion of Right Foot Vein, Perc Endo Approach */
"06LV0CZ"  =  "1"  /* Occlusion of Left Foot Vein with Extralum Dev, Open Approach */
"06LV0DZ"  =  "1"  /* Occlusion of Left Foot Vein with Intralum Dev, Open Approach */
"06LV0ZZ"  =  "1"  /* Occlusion of Left Foot Vein, Open Approach */
"06LV3CZ"  =  "1"  /* Occlusion of Left Foot Vein with Extralum Dev, Perc Approach */
"06LV3DZ"  =  "1"  /* Occlusion of Left Foot Vein with Intralum Dev, Perc Approach */
"06LV3ZZ"  =  "1"  /* Occlusion of Left Foot Vein, Percutaneous Approach */
"06LV4CZ"  =  "1"  /* Occlusion L Foot Vein w Extralum Dev, Perc Endo */
"06LV4DZ"  =  "1"  /* Occlusion L Foot Vein w Intralum Dev, Perc Endo */
"06LV4ZZ"  =  "1"  /* Occlusion of Left Foot Vein, Perc Endo Approach */
"06LY0CC"  =  "1"  /* Occlusion of Hemorr Plexus with Extralum Dev, Open Approach */
"06LY0CZ"  =  "1"  /* Occlusion of Lower Vein with Extralum Dev, Open Approach */
"06LY0DC"  =  "1"  /* Occlusion of Hemorr Plexus with Intralum Dev, Open Approach */
"06LY0DZ"  =  "1"  /* Occlusion of Lower Vein with Intralum Dev, Open Approach */
"06LY0ZC"  =  "1"  /* Occlusion of Hemorrhoidal Plexus, Open Approach */
"06LY0ZZ"  =  "1"  /* Occlusion of Lower Vein, Open Approach */
"06LY3CC"  =  "1"  /* Occlusion of Hemorr Plexus with Extralum Dev, Perc Approach */
"06LY3CZ"  =  "1"  /* Occlusion of Lower Vein with Extralum Dev, Perc Approach */
"06LY3DC"  =  "1"  /* Occlusion of Hemorr Plexus with Intralum Dev, Perc Approach */
"06LY3DZ"  =  "1"  /* Occlusion of Lower Vein with Intralum Dev, Perc Approach */
"06LY3ZC"  =  "1"  /* Occlusion of Hemorrhoidal Plexus, Percutaneous Approach */
"06LY3ZZ"  =  "1"  /* Occlusion of Lower Vein, Percutaneous Approach */
"06LY4CC"  =  "1"  /* Occlusion Hemorr Plexus w Extralum Dev, Perc Endo */
"06LY4CZ"  =  "1"  /* Occlusion of Low Vein with Extralum Dev, Perc Endo Approach */
"06LY4DC"  =  "1"  /* Occlusion Hemorr Plexus w Intralum Dev, Perc Endo */
"06LY4DZ"  =  "1"  /* Occlusion of Low Vein with Intralum Dev, Perc Endo Approach */
"06LY4ZC"  =  "1"  /* Occlusion of Hemorrhoidal Plexus, Perc Endo Approach */
"06LY4ZZ"  =  "1"  /* Occlusion of Lower Vein, Percutaneous Endoscopic Approach */
"06N00ZZ"  =  "1"  /* Release Inferior Vena Cava, Open Approach */
"06N03ZZ"  =  "1"  /* Release Inferior Vena Cava, Percutaneous Approach */
"06N04ZZ"  =  "1"  /* Release Inferior Vena Cava, Percutaneous Endoscopic Approach */
"06N10ZZ"  =  "1"  /* Release Splenic Vein, Open Approach */
"06N13ZZ"  =  "1"  /* Release Splenic Vein, Percutaneous Approach */
"06N14ZZ"  =  "1"  /* Release Splenic Vein, Percutaneous Endoscopic Approach */
"06N20ZZ"  =  "1"  /* Release Gastric Vein, Open Approach */
"06N23ZZ"  =  "1"  /* Release Gastric Vein, Percutaneous Approach */
"06N24ZZ"  =  "1"  /* Release Gastric Vein, Percutaneous Endoscopic Approach */
"06N30ZZ"  =  "1"  /* Release Esophageal Vein, Open Approach */
"06N33ZZ"  =  "1"  /* Release Esophageal Vein, Percutaneous Approach */
"06N34ZZ"  =  "1"  /* Release Esophageal Vein, Percutaneous Endoscopic Approach */
"06N40ZZ"  =  "1"  /* Release Hepatic Vein, Open Approach */
"06N43ZZ"  =  "1"  /* Release Hepatic Vein, Percutaneous Approach */
"06N44ZZ"  =  "1"  /* Release Hepatic Vein, Percutaneous Endoscopic Approach */
"06N50ZZ"  =  "1"  /* Release Superior Mesenteric Vein, Open Approach */
"06N53ZZ"  =  "1"  /* Release Superior Mesenteric Vein, Percutaneous Approach */
"06N54ZZ"  =  "1"  /* Release Superior Mesenteric Vein, Perc Endo Approach */
"06N60ZZ"  =  "1"  /* Release Inferior Mesenteric Vein, Open Approach */
"06N63ZZ"  =  "1"  /* Release Inferior Mesenteric Vein, Percutaneous Approach */
"06N64ZZ"  =  "1"  /* Release Inferior Mesenteric Vein, Perc Endo Approach */
"06N70ZZ"  =  "1"  /* Release Colic Vein, Open Approach */
"06N73ZZ"  =  "1"  /* Release Colic Vein, Percutaneous Approach */
"06N74ZZ"  =  "1"  /* Release Colic Vein, Percutaneous Endoscopic Approach */
"06N80ZZ"  =  "1"  /* Release Portal Vein, Open Approach */
"06N83ZZ"  =  "1"  /* Release Portal Vein, Percutaneous Approach */
"06N84ZZ"  =  "1"  /* Release Portal Vein, Percutaneous Endoscopic Approach */
"06N90ZZ"  =  "1"  /* Release Right Renal Vein, Open Approach */
"06N93ZZ"  =  "1"  /* Release Right Renal Vein, Percutaneous Approach */
"06N94ZZ"  =  "1"  /* Release Right Renal Vein, Percutaneous Endoscopic Approach */
"06NB0ZZ"  =  "1"  /* Release Left Renal Vein, Open Approach */
"06NB3ZZ"  =  "1"  /* Release Left Renal Vein, Percutaneous Approach */
"06NB4ZZ"  =  "1"  /* Release Left Renal Vein, Percutaneous Endoscopic Approach */
"06NC0ZZ"  =  "1"  /* Release Right Common Iliac Vein, Open Approach */
"06NC3ZZ"  =  "1"  /* Release Right Common Iliac Vein, Percutaneous Approach */
"06NC4ZZ"  =  "1"  /* Release Right Common Iliac Vein, Perc Endo Approach */
"06ND0ZZ"  =  "1"  /* Release Left Common Iliac Vein, Open Approach */
"06ND3ZZ"  =  "1"  /* Release Left Common Iliac Vein, Percutaneous Approach */
"06ND4ZZ"  =  "1"  /* Release Left Common Iliac Vein, Perc Endo Approach */
"06NF0ZZ"  =  "1"  /* Release Right External Iliac Vein, Open Approach */
"06NF3ZZ"  =  "1"  /* Release Right External Iliac Vein, Percutaneous Approach */
"06NF4ZZ"  =  "1"  /* Release Right External Iliac Vein, Perc Endo Approach */
"06NG0ZZ"  =  "1"  /* Release Left External Iliac Vein, Open Approach */
"06NG3ZZ"  =  "1"  /* Release Left External Iliac Vein, Percutaneous Approach */
"06NG4ZZ"  =  "1"  /* Release Left External Iliac Vein, Perc Endo Approach */
"06NH0ZZ"  =  "1"  /* Release Right Hypogastric Vein, Open Approach */
"06NH3ZZ"  =  "1"  /* Release Right Hypogastric Vein, Percutaneous Approach */
"06NH4ZZ"  =  "1"  /* Release Right Hypogastric Vein, Perc Endo Approach */
"06NJ0ZZ"  =  "1"  /* Release Left Hypogastric Vein, Open Approach */
"06NJ3ZZ"  =  "1"  /* Release Left Hypogastric Vein, Percutaneous Approach */
"06NJ4ZZ"  =  "1"  /* Release Left Hypogastric Vein, Perc Endo Approach */
"06NM0ZZ"  =  "1"  /* Release Right Femoral Vein, Open Approach */
"06NM3ZZ"  =  "1"  /* Release Right Femoral Vein, Percutaneous Approach */
"06NM4ZZ"  =  "1"  /* Release Right Femoral Vein, Percutaneous Endoscopic Approach */
"06NN0ZZ"  =  "1"  /* Release Left Femoral Vein, Open Approach */
"06NN3ZZ"  =  "1"  /* Release Left Femoral Vein, Percutaneous Approach */
"06NN4ZZ"  =  "1"  /* Release Left Femoral Vein, Percutaneous Endoscopic Approach */
"06NP0ZZ"  =  "1"  /* Release Right Greater Saphenous Vein, Open Approach */
"06NP3ZZ"  =  "1"  /* Release Right Greater Saphenous Vein, Percutaneous Approach */
"06NP4ZZ"  =  "1"  /* Release Right Greater Saphenous Vein, Perc Endo Approach */
"06NQ0ZZ"  =  "1"  /* Release Left Greater Saphenous Vein, Open Approach */
"06NQ3ZZ"  =  "1"  /* Release Left Greater Saphenous Vein, Percutaneous Approach */
"06NQ4ZZ"  =  "1"  /* Release Left Greater Saphenous Vein, Perc Endo Approach */
"06NR0ZZ"  =  "1"  /* Release Right Lesser Saphenous Vein, Open Approach */
"06NR3ZZ"  =  "1"  /* Release Right Lesser Saphenous Vein, Percutaneous Approach */
"06NR4ZZ"  =  "1"  /* Release Right Lesser Saphenous Vein, Perc Endo Approach */
"06NS0ZZ"  =  "1"  /* Release Left Lesser Saphenous Vein, Open Approach */
"06NS3ZZ"  =  "1"  /* Release Left Lesser Saphenous Vein, Percutaneous Approach */
"06NS4ZZ"  =  "1"  /* Release Left Lesser Saphenous Vein, Perc Endo Approach */
"06NT0ZZ"  =  "1"  /* Release Right Foot Vein, Open Approach */
"06NT3ZZ"  =  "1"  /* Release Right Foot Vein, Percutaneous Approach */
"06NT4ZZ"  =  "1"  /* Release Right Foot Vein, Percutaneous Endoscopic Approach */
"06NV0ZZ"  =  "1"  /* Release Left Foot Vein, Open Approach */
"06NV3ZZ"  =  "1"  /* Release Left Foot Vein, Percutaneous Approach */
"06NV4ZZ"  =  "1"  /* Release Left Foot Vein, Percutaneous Endoscopic Approach */
"06NY0ZZ"  =  "1"  /* Release Lower Vein, Open Approach */
"06NY3ZZ"  =  "1"  /* Release Lower Vein, Percutaneous Approach */
"06NY4ZZ"  =  "1"  /* Release Lower Vein, Percutaneous Endoscopic Approach */
"06PY00Z"  =  "1"  /* Removal of Drainage Device from Lower Vein, Open Approach */
"06PY02Z"  =  "1"  /* Removal of Monitoring Device from Lower Vein, Open Approach */
"06PY03Z"  =  "1"  /* Removal of Infusion Device from Lower Vein, Open Approach */
"06PY07Z"  =  "1"  /* Removal of Autol Sub from Low Vein, Open Approach */
"06PY0CZ"  =  "1"  /* Removal of Extraluminal Device from Low Vein, Open Approach */
"06PY0DZ"  =  "1"  /* Removal of Intraluminal Device from Low Vein, Open Approach */
"06PY0JZ"  =  "1"  /* Removal of Synthetic Substitute from Low Vein, Open Approach */
"06PY0KZ"  =  "1"  /* Removal of Nonaut Sub from Low Vein, Open Approach */
"06PY30Z"  =  "1"  /* Removal of Drainage Device from Lower Vein, Perc Approach */
"06PY32Z"  =  "1"  /* Removal of Monitoring Device from Lower Vein, Perc Approach */
"06PY33Z"  =  "1"  /* Removal of Infusion Device from Lower Vein, Perc Approach */
"06PY37Z"  =  "1"  /* Removal of Autol Sub from Low Vein, Perc Approach */
"06PY3CZ"  =  "1"  /* Removal of Extraluminal Device from Low Vein, Perc Approach */
"06PY3DZ"  =  "1"  /* Removal of Intraluminal Device from Low Vein, Perc Approach */
"06PY3JZ"  =  "1"  /* Removal of Synthetic Substitute from Low Vein, Perc Approach */
"06PY3KZ"  =  "1"  /* Removal of Nonaut Sub from Low Vein, Perc Approach */
"06PY40Z"  =  "1"  /* Removal of Drainage Device from Low Vein, Perc Endo Approach */
"06PY42Z"  =  "1"  /* Removal of Monitor Dev from Low Vein, Perc Endo Approach */
"06PY43Z"  =  "1"  /* Removal of Infusion Device from Low Vein, Perc Endo Approach */
"06PY47Z"  =  "1"  /* Removal of Autol Sub from Low Vein, Perc Endo Approach */
"06PY4CZ"  =  "1"  /* Removal of Extralum Dev from Low Vein, Perc Endo Approach */
"06PY4DZ"  =  "1"  /* Removal of Intralum Dev from Low Vein, Perc Endo Approach */
"06PY4JZ"  =  "1"  /* Removal of Synth Sub from Low Vein, Perc Endo Approach */
"06PY4KZ"  =  "1"  /* Removal of Nonaut Sub from Low Vein, Perc Endo Approach */
"06Q00ZZ"  =  "1"  /* Repair Inferior Vena Cava, Open Approach */
"06Q03ZZ"  =  "1"  /* Repair Inferior Vena Cava, Percutaneous Approach */
"06Q04ZZ"  =  "1"  /* Repair Inferior Vena Cava, Percutaneous Endoscopic Approach */
"06Q10ZZ"  =  "1"  /* Repair Splenic Vein, Open Approach */
"06Q13ZZ"  =  "1"  /* Repair Splenic Vein, Percutaneous Approach */
"06Q14ZZ"  =  "1"  /* Repair Splenic Vein, Percutaneous Endoscopic Approach */
"06Q20ZZ"  =  "1"  /* Repair Gastric Vein, Open Approach */
"06Q23ZZ"  =  "1"  /* Repair Gastric Vein, Percutaneous Approach */
"06Q24ZZ"  =  "1"  /* Repair Gastric Vein, Percutaneous Endoscopic Approach */
"06Q30ZZ"  =  "1"  /* Repair Esophageal Vein, Open Approach */
"06Q33ZZ"  =  "1"  /* Repair Esophageal Vein, Percutaneous Approach */
"06Q34ZZ"  =  "1"  /* Repair Esophageal Vein, Percutaneous Endoscopic Approach */
"06Q40ZZ"  =  "1"  /* Repair Hepatic Vein, Open Approach */
"06Q43ZZ"  =  "1"  /* Repair Hepatic Vein, Percutaneous Approach */
"06Q44ZZ"  =  "1"  /* Repair Hepatic Vein, Percutaneous Endoscopic Approach */
"06Q50ZZ"  =  "1"  /* Repair Superior Mesenteric Vein, Open Approach */
"06Q53ZZ"  =  "1"  /* Repair Superior Mesenteric Vein, Percutaneous Approach */
"06Q54ZZ"  =  "1"  /* Repair Superior Mesenteric Vein, Perc Endo Approach */
"06Q60ZZ"  =  "1"  /* Repair Inferior Mesenteric Vein, Open Approach */
"06Q63ZZ"  =  "1"  /* Repair Inferior Mesenteric Vein, Percutaneous Approach */
"06Q64ZZ"  =  "1"  /* Repair Inferior Mesenteric Vein, Perc Endo Approach */
"06Q70ZZ"  =  "1"  /* Repair Colic Vein, Open Approach */
"06Q73ZZ"  =  "1"  /* Repair Colic Vein, Percutaneous Approach */
"06Q74ZZ"  =  "1"  /* Repair Colic Vein, Percutaneous Endoscopic Approach */
"06Q80ZZ"  =  "1"  /* Repair Portal Vein, Open Approach */
"06Q83ZZ"  =  "1"  /* Repair Portal Vein, Percutaneous Approach */
"06Q84ZZ"  =  "1"  /* Repair Portal Vein, Percutaneous Endoscopic Approach */
"06Q90ZZ"  =  "1"  /* Repair Right Renal Vein, Open Approach */
"06Q93ZZ"  =  "1"  /* Repair Right Renal Vein, Percutaneous Approach */
"06Q94ZZ"  =  "1"  /* Repair Right Renal Vein, Percutaneous Endoscopic Approach */
"06QB0ZZ"  =  "1"  /* Repair Left Renal Vein, Open Approach */
"06QB3ZZ"  =  "1"  /* Repair Left Renal Vein, Percutaneous Approach */
"06QB4ZZ"  =  "1"  /* Repair Left Renal Vein, Percutaneous Endoscopic Approach */
"06QC0ZZ"  =  "1"  /* Repair Right Common Iliac Vein, Open Approach */
"06QC3ZZ"  =  "1"  /* Repair Right Common Iliac Vein, Percutaneous Approach */
"06QC4ZZ"  =  "1"  /* Repair Right Common Iliac Vein, Perc Endo Approach */
"06QD0ZZ"  =  "1"  /* Repair Left Common Iliac Vein, Open Approach */
"06QD3ZZ"  =  "1"  /* Repair Left Common Iliac Vein, Percutaneous Approach */
"06QD4ZZ"  =  "1"  /* Repair Left Common Iliac Vein, Perc Endo Approach */
"06QF0ZZ"  =  "1"  /* Repair Right External Iliac Vein, Open Approach */
"06QF3ZZ"  =  "1"  /* Repair Right External Iliac Vein, Percutaneous Approach */
"06QF4ZZ"  =  "1"  /* Repair Right External Iliac Vein, Perc Endo Approach */
"06QG0ZZ"  =  "1"  /* Repair Left External Iliac Vein, Open Approach */
"06QG3ZZ"  =  "1"  /* Repair Left External Iliac Vein, Percutaneous Approach */
"06QG4ZZ"  =  "1"  /* Repair Left External Iliac Vein, Perc Endo Approach */
"06QH0ZZ"  =  "1"  /* Repair Right Hypogastric Vein, Open Approach */
"06QH3ZZ"  =  "1"  /* Repair Right Hypogastric Vein, Percutaneous Approach */
"06QH4ZZ"  =  "1"  /* Repair Right Hypogastric Vein, Perc Endo Approach */
"06QJ0ZZ"  =  "1"  /* Repair Left Hypogastric Vein, Open Approach */
"06QJ3ZZ"  =  "1"  /* Repair Left Hypogastric Vein, Percutaneous Approach */
"06QJ4ZZ"  =  "1"  /* Repair Left Hypogastric Vein, Perc Endo Approach */
"06QM0ZZ"  =  "1"  /* Repair Right Femoral Vein, Open Approach */
"06QM3ZZ"  =  "1"  /* Repair Right Femoral Vein, Percutaneous Approach */
"06QM4ZZ"  =  "1"  /* Repair Right Femoral Vein, Percutaneous Endoscopic Approach */
"06QN0ZZ"  =  "1"  /* Repair Left Femoral Vein, Open Approach */
"06QN3ZZ"  =  "1"  /* Repair Left Femoral Vein, Percutaneous Approach */
"06QN4ZZ"  =  "1"  /* Repair Left Femoral Vein, Percutaneous Endoscopic Approach */
"06QP0ZZ"  =  "1"  /* Repair Right Greater Saphenous Vein, Open Approach */
"06QP3ZZ"  =  "1"  /* Repair Right Greater Saphenous Vein, Percutaneous Approach */
"06QP4ZZ"  =  "1"  /* Repair Right Greater Saphenous Vein, Perc Endo Approach */
"06QQ0ZZ"  =  "1"  /* Repair Left Greater Saphenous Vein, Open Approach */
"06QQ3ZZ"  =  "1"  /* Repair Left Greater Saphenous Vein, Percutaneous Approach */
"06QQ4ZZ"  =  "1"  /* Repair Left Greater Saphenous Vein, Perc Endo Approach */
"06QR0ZZ"  =  "1"  /* Repair Right Lesser Saphenous Vein, Open Approach */
"06QR3ZZ"  =  "1"  /* Repair Right Lesser Saphenous Vein, Percutaneous Approach */
"06QR4ZZ"  =  "1"  /* Repair Right Lesser Saphenous Vein, Perc Endo Approach */
"06QS0ZZ"  =  "1"  /* Repair Left Lesser Saphenous Vein, Open Approach */
"06QS3ZZ"  =  "1"  /* Repair Left Lesser Saphenous Vein, Percutaneous Approach */
"06QS4ZZ"  =  "1"  /* Repair Left Lesser Saphenous Vein, Perc Endo Approach */
"06QT0ZZ"  =  "1"  /* Repair Right Foot Vein, Open Approach */
"06QT3ZZ"  =  "1"  /* Repair Right Foot Vein, Percutaneous Approach */
"06QT4ZZ"  =  "1"  /* Repair Right Foot Vein, Percutaneous Endoscopic Approach */
"06QV0ZZ"  =  "1"  /* Repair Left Foot Vein, Open Approach */
"06QV3ZZ"  =  "1"  /* Repair Left Foot Vein, Percutaneous Approach */
"06QV4ZZ"  =  "1"  /* Repair Left Foot Vein, Percutaneous Endoscopic Approach */
"06QY0ZZ"  =  "1"  /* Repair Lower Vein, Open Approach */
"06QY3ZZ"  =  "1"  /* Repair Lower Vein, Percutaneous Approach */
"06QY4ZZ"  =  "1"  /* Repair Lower Vein, Percutaneous Endoscopic Approach */
"06R007Z"  =  "1"  /* Replacement of Inf Vena Cava with Autol Sub, Open Approach */
"06R00JZ"  =  "1"  /* Replacement of Inf Vena Cava with Synth Sub, Open Approach */
"06R00KZ"  =  "1"  /* Replacement of Inf Vena Cava with Nonaut Sub, Open Approach */
"06R047Z"  =  "1"  /* Replace of Inf Vena Cava with Autol Sub, Perc Endo Approach */
"06R04JZ"  =  "1"  /* Replace of Inf Vena Cava with Synth Sub, Perc Endo Approach */
"06R04KZ"  =  "1"  /* Replace of Inf Vena Cava with Nonaut Sub, Perc Endo Approach */
"06R107Z"  =  "1"  /* Replacement of Splenic Vein with Autol Sub, Open Approach */
"06R10JZ"  =  "1"  /* Replacement of Splenic Vein with Synth Sub, Open Approach */
"06R10KZ"  =  "1"  /* Replacement of Splenic Vein with Nonaut Sub, Open Approach */
"06R147Z"  =  "1"  /* Replace of Splenic Vein with Autol Sub, Perc Endo Approach */
"06R14JZ"  =  "1"  /* Replace of Splenic Vein with Synth Sub, Perc Endo Approach */
"06R14KZ"  =  "1"  /* Replace of Splenic Vein with Nonaut Sub, Perc Endo Approach */
"06R207Z"  =  "1"  /* Replacement of Gastric Vein with Autol Sub, Open Approach */
"06R20JZ"  =  "1"  /* Replacement of Gastric Vein with Synth Sub, Open Approach */
"06R20KZ"  =  "1"  /* Replacement of Gastric Vein with Nonaut Sub, Open Approach */
"06R247Z"  =  "1"  /* Replace of Gastric Vein with Autol Sub, Perc Endo Approach */
"06R24JZ"  =  "1"  /* Replace of Gastric Vein with Synth Sub, Perc Endo Approach */
"06R24KZ"  =  "1"  /* Replace of Gastric Vein with Nonaut Sub, Perc Endo Approach */
"06R307Z"  =  "1"  /* Replacement of Esophageal Vein with Autol Sub, Open Approach */
"06R30JZ"  =  "1"  /* Replacement of Esophageal Vein with Synth Sub, Open Approach */
"06R30KZ"  =  "1"  /* Replace of Esophageal Vein with Nonaut Sub, Open Approach */
"06R347Z"  =  "1"  /* Replace Esophageal Vein w Autol Sub, Perc Endo */
"06R34JZ"  =  "1"  /* Replace Esophageal Vein w Synth Sub, Perc Endo */
"06R34KZ"  =  "1"  /* Replace Esophageal Vein w Nonaut Sub, Perc Endo */
"06R407Z"  =  "1"  /* Replacement of Hepatic Vein with Autol Sub, Open Approach */
"06R40JZ"  =  "1"  /* Replacement of Hepatic Vein with Synth Sub, Open Approach */
"06R40KZ"  =  "1"  /* Replacement of Hepatic Vein with Nonaut Sub, Open Approach */
"06R447Z"  =  "1"  /* Replace of Hepatic Vein with Autol Sub, Perc Endo Approach */
"06R44JZ"  =  "1"  /* Replace of Hepatic Vein with Synth Sub, Perc Endo Approach */
"06R44KZ"  =  "1"  /* Replace of Hepatic Vein with Nonaut Sub, Perc Endo Approach */
"06R507Z"  =  "1"  /* Replacement of Sup Mesent Vein with Autol Sub, Open Approach */
"06R50JZ"  =  "1"  /* Replacement of Sup Mesent Vein with Synth Sub, Open Approach */
"06R50KZ"  =  "1"  /* Replace of Sup Mesent Vein with Nonaut Sub, Open Approach */
"06R547Z"  =  "1"  /* Replace Sup Mesent Vein w Autol Sub, Perc Endo */
"06R54JZ"  =  "1"  /* Replace Sup Mesent Vein w Synth Sub, Perc Endo */
"06R54KZ"  =  "1"  /* Replace Sup Mesent Vein w Nonaut Sub, Perc Endo */
"06R607Z"  =  "1"  /* Replacement of Inf Mesent Vein with Autol Sub, Open Approach */
"06R60JZ"  =  "1"  /* Replacement of Inf Mesent Vein with Synth Sub, Open Approach */
"06R60KZ"  =  "1"  /* Replace of Inf Mesent Vein with Nonaut Sub, Open Approach */
"06R647Z"  =  "1"  /* Replace Inf Mesent Vein w Autol Sub, Perc Endo */
"06R64JZ"  =  "1"  /* Replace Inf Mesent Vein w Synth Sub, Perc Endo */
"06R64KZ"  =  "1"  /* Replace Inf Mesent Vein w Nonaut Sub, Perc Endo */
"06R707Z"  =  "1"  /* Replacement of Colic Vein with Autol Sub, Open Approach */
"06R70JZ"  =  "1"  /* Replacement of Colic Vein with Synth Sub, Open Approach */
"06R70KZ"  =  "1"  /* Replacement of Colic Vein with Nonaut Sub, Open Approach */
"06R747Z"  =  "1"  /* Replacement of Colic Vein with Autol Sub, Perc Endo Approach */
"06R74JZ"  =  "1"  /* Replacement of Colic Vein with Synth Sub, Perc Endo Approach */
"06R74KZ"  =  "1"  /* Replace of Colic Vein with Nonaut Sub, Perc Endo Approach */
"06R807Z"  =  "1"  /* Replacement of Portal Vein with Autol Sub, Open Approach */
"06R80JZ"  =  "1"  /* Replacement of Portal Vein with Synth Sub, Open Approach */
"06R80KZ"  =  "1"  /* Replacement of Portal Vein with Nonaut Sub, Open Approach */
"06R847Z"  =  "1"  /* Replace of Portal Vein with Autol Sub, Perc Endo Approach */
"06R84JZ"  =  "1"  /* Replace of Portal Vein with Synth Sub, Perc Endo Approach */
"06R84KZ"  =  "1"  /* Replace of Portal Vein with Nonaut Sub, Perc Endo Approach */
"06R907Z"  =  "1"  /* Replacement of R Renal Vein with Autol Sub, Open Approach */
"06R90JZ"  =  "1"  /* Replacement of R Renal Vein with Synth Sub, Open Approach */
"06R90KZ"  =  "1"  /* Replacement of R Renal Vein with Nonaut Sub, Open Approach */
"06R947Z"  =  "1"  /* Replace of R Renal Vein with Autol Sub, Perc Endo Approach */
"06R94JZ"  =  "1"  /* Replace of R Renal Vein with Synth Sub, Perc Endo Approach */
"06R94KZ"  =  "1"  /* Replace of R Renal Vein with Nonaut Sub, Perc Endo Approach */
"06RB07Z"  =  "1"  /* Replacement of Left Renal Vein with Autol Sub, Open Approach */
"06RB0JZ"  =  "1"  /* Replacement of Left Renal Vein with Synth Sub, Open Approach */
"06RB0KZ"  =  "1"  /* Replacement of L Renal Vein with Nonaut Sub, Open Approach */
"06RB47Z"  =  "1"  /* Replace of L Renal Vein with Autol Sub, Perc Endo Approach */
"06RB4JZ"  =  "1"  /* Replace of L Renal Vein with Synth Sub, Perc Endo Approach */
"06RB4KZ"  =  "1"  /* Replace of L Renal Vein with Nonaut Sub, Perc Endo Approach */
"06RC07Z"  =  "1"  /* Replace of R Com Iliac Vein with Autol Sub, Open Approach */
"06RC0JZ"  =  "1"  /* Replace of R Com Iliac Vein with Synth Sub, Open Approach */
"06RC0KZ"  =  "1"  /* Replace of R Com Iliac Vein with Nonaut Sub, Open Approach */
"06RC47Z"  =  "1"  /* Replace R Com Iliac Vein w Autol Sub, Perc Endo */
"06RC4JZ"  =  "1"  /* Replace R Com Iliac Vein w Synth Sub, Perc Endo */
"06RC4KZ"  =  "1"  /* Replace R Com Iliac Vein w Nonaut Sub, Perc Endo */
"06RD07Z"  =  "1"  /* Replace of L Com Iliac Vein with Autol Sub, Open Approach */
"06RD0JZ"  =  "1"  /* Replace of L Com Iliac Vein with Synth Sub, Open Approach */
"06RD0KZ"  =  "1"  /* Replace of L Com Iliac Vein with Nonaut Sub, Open Approach */
"06RD47Z"  =  "1"  /* Replace L Com Iliac Vein w Autol Sub, Perc Endo */
"06RD4JZ"  =  "1"  /* Replace L Com Iliac Vein w Synth Sub, Perc Endo */
"06RD4KZ"  =  "1"  /* Replace L Com Iliac Vein w Nonaut Sub, Perc Endo */
"06RF07Z"  =  "1"  /* Replace of R Ext Iliac Vein with Autol Sub, Open Approach */
"06RF0JZ"  =  "1"  /* Replace of R Ext Iliac Vein with Synth Sub, Open Approach */
"06RF0KZ"  =  "1"  /* Replace of R Ext Iliac Vein with Nonaut Sub, Open Approach */
"06RF47Z"  =  "1"  /* Replace R Ext Iliac Vein w Autol Sub, Perc Endo */
"06RF4JZ"  =  "1"  /* Replace R Ext Iliac Vein w Synth Sub, Perc Endo */
"06RF4KZ"  =  "1"  /* Replace R Ext Iliac Vein w Nonaut Sub, Perc Endo */
"06RG07Z"  =  "1"  /* Replace of L Ext Iliac Vein with Autol Sub, Open Approach */
"06RG0JZ"  =  "1"  /* Replace of L Ext Iliac Vein with Synth Sub, Open Approach */
"06RG0KZ"  =  "1"  /* Replace of L Ext Iliac Vein with Nonaut Sub, Open Approach */
"06RG47Z"  =  "1"  /* Replace L Ext Iliac Vein w Autol Sub, Perc Endo */
"06RG4JZ"  =  "1"  /* Replace L Ext Iliac Vein w Synth Sub, Perc Endo */
"06RG4KZ"  =  "1"  /* Replace L Ext Iliac Vein w Nonaut Sub, Perc Endo */
"06RH07Z"  =  "1"  /* Replacement of R Hypogast Vein with Autol Sub, Open Approach */
"06RH0JZ"  =  "1"  /* Replacement of R Hypogast Vein with Synth Sub, Open Approach */
"06RH0KZ"  =  "1"  /* Replace of R Hypogast Vein with Nonaut Sub, Open Approach */
"06RH47Z"  =  "1"  /* Replace R Hypogast Vein w Autol Sub, Perc Endo */
"06RH4JZ"  =  "1"  /* Replace R Hypogast Vein w Synth Sub, Perc Endo */
"06RH4KZ"  =  "1"  /* Replace R Hypogast Vein w Nonaut Sub, Perc Endo */
"06RJ07Z"  =  "1"  /* Replacement of L Hypogast Vein with Autol Sub, Open Approach */
"06RJ0JZ"  =  "1"  /* Replacement of L Hypogast Vein with Synth Sub, Open Approach */
"06RJ0KZ"  =  "1"  /* Replace of L Hypogast Vein with Nonaut Sub, Open Approach */
"06RJ47Z"  =  "1"  /* Replace L Hypogast Vein w Autol Sub, Perc Endo */
"06RJ4JZ"  =  "1"  /* Replace L Hypogast Vein w Synth Sub, Perc Endo */
"06RJ4KZ"  =  "1"  /* Replace L Hypogast Vein w Nonaut Sub, Perc Endo */
"06RM07Z"  =  "1"  /* Replacement of R Femor Vein with Autol Sub, Open Approach */
"06RM0JZ"  =  "1"  /* Replacement of R Femor Vein with Synth Sub, Open Approach */
"06RM0KZ"  =  "1"  /* Replacement of R Femor Vein with Nonaut Sub, Open Approach */
"06RM47Z"  =  "1"  /* Replace of R Femor Vein with Autol Sub, Perc Endo Approach */
"06RM4JZ"  =  "1"  /* Replace of R Femor Vein with Synth Sub, Perc Endo Approach */
"06RM4KZ"  =  "1"  /* Replace of R Femor Vein with Nonaut Sub, Perc Endo Approach */
"06RN07Z"  =  "1"  /* Replacement of L Femor Vein with Autol Sub, Open Approach */
"06RN0JZ"  =  "1"  /* Replacement of L Femor Vein with Synth Sub, Open Approach */
"06RN0KZ"  =  "1"  /* Replacement of L Femor Vein with Nonaut Sub, Open Approach */
"06RN47Z"  =  "1"  /* Replace of L Femor Vein with Autol Sub, Perc Endo Approach */
"06RN4JZ"  =  "1"  /* Replace of L Femor Vein with Synth Sub, Perc Endo Approach */
"06RN4KZ"  =  "1"  /* Replace of L Femor Vein with Nonaut Sub, Perc Endo Approach */
"06RP07Z"  =  "1"  /* Replace of R Great Saphenous with Autol Sub, Open Approach */
"06RP0JZ"  =  "1"  /* Replace of R Great Saphenous with Synth Sub, Open Approach */
"06RP0KZ"  =  "1"  /* Replace of R Great Saphenous with Nonaut Sub, Open Approach */
"06RP47Z"  =  "1"  /* Replace R Great Saphenous w Autol Sub, Perc Endo */
"06RP4JZ"  =  "1"  /* Replace R Great Saphenous w Synth Sub, Perc Endo */
"06RP4KZ"  =  "1"  /* Replace R Great Saphenous w Nonaut Sub, Perc Endo */
"06RQ07Z"  =  "1"  /* Replace of L Great Saphenous with Autol Sub, Open Approach */
"06RQ0JZ"  =  "1"  /* Replace of L Great Saphenous with Synth Sub, Open Approach */
"06RQ0KZ"  =  "1"  /* Replace of L Great Saphenous with Nonaut Sub, Open Approach */
"06RQ47Z"  =  "1"  /* Replace L Great Saphenous w Autol Sub, Perc Endo */
"06RQ4JZ"  =  "1"  /* Replace L Great Saphenous w Synth Sub, Perc Endo */
"06RQ4KZ"  =  "1"  /* Replace L Great Saphenous w Nonaut Sub, Perc Endo */
"06RR07Z"  =  "1"  /* Replace of R Less Saphenous with Autol Sub, Open Approach */
"06RR0JZ"  =  "1"  /* Replace of R Less Saphenous with Synth Sub, Open Approach */
"06RR0KZ"  =  "1"  /* Replace of R Less Saphenous with Nonaut Sub, Open Approach */
"06RR47Z"  =  "1"  /* Replace R Less Saphenous w Autol Sub, Perc Endo */
"06RR4JZ"  =  "1"  /* Replace R Less Saphenous w Synth Sub, Perc Endo */
"06RR4KZ"  =  "1"  /* Replace R Less Saphenous w Nonaut Sub, Perc Endo */
"06RS07Z"  =  "1"  /* Replace of L Less Saphenous with Autol Sub, Open Approach */
"06RS0JZ"  =  "1"  /* Replace of L Less Saphenous with Synth Sub, Open Approach */
"06RS0KZ"  =  "1"  /* Replace of L Less Saphenous with Nonaut Sub, Open Approach */
"06RS47Z"  =  "1"  /* Replace L Less Saphenous w Autol Sub, Perc Endo */
"06RS4JZ"  =  "1"  /* Replace L Less Saphenous w Synth Sub, Perc Endo */
"06RS4KZ"  =  "1"  /* Replace L Less Saphenous w Nonaut Sub, Perc Endo */
"06RT07Z"  =  "1"  /* Replacement of Right Foot Vein with Autol Sub, Open Approach */
"06RT0JZ"  =  "1"  /* Replacement of Right Foot Vein with Synth Sub, Open Approach */
"06RT0KZ"  =  "1"  /* Replacement of R Foot Vein with Nonaut Sub, Open Approach */
"06RT47Z"  =  "1"  /* Replace of R Foot Vein with Autol Sub, Perc Endo Approach */
"06RT4JZ"  =  "1"  /* Replace of R Foot Vein with Synth Sub, Perc Endo Approach */
"06RT4KZ"  =  "1"  /* Replace of R Foot Vein with Nonaut Sub, Perc Endo Approach */
"06RV07Z"  =  "1"  /* Replacement of Left Foot Vein with Autol Sub, Open Approach */
"06RV0JZ"  =  "1"  /* Replacement of Left Foot Vein with Synth Sub, Open Approach */
"06RV0KZ"  =  "1"  /* Replacement of Left Foot Vein with Nonaut Sub, Open Approach */
"06RV47Z"  =  "1"  /* Replace of L Foot Vein with Autol Sub, Perc Endo Approach */
"06RV4JZ"  =  "1"  /* Replace of L Foot Vein with Synth Sub, Perc Endo Approach */
"06RV4KZ"  =  "1"  /* Replace of L Foot Vein with Nonaut Sub, Perc Endo Approach */
"06RY07Z"  =  "1"  /* Replacement of Lower Vein with Autol Sub, Open Approach */
"06RY0JZ"  =  "1"  /* Replacement of Lower Vein with Synth Sub, Open Approach */
"06RY0KZ"  =  "1"  /* Replacement of Lower Vein with Nonaut Sub, Open Approach */
"06RY47Z"  =  "1"  /* Replacement of Lower Vein with Autol Sub, Perc Endo Approach */
"06RY4JZ"  =  "1"  /* Replacement of Lower Vein with Synth Sub, Perc Endo Approach */
"06RY4KZ"  =  "1"  /* Replacement of Low Vein with Nonaut Sub, Perc Endo Approach */
"06S00ZZ"  =  "1"  /* Reposition Inferior Vena Cava, Open Approach */
"06S03ZZ"  =  "1"  /* Reposition Inferior Vena Cava, Percutaneous Approach */
"06S04ZZ"  =  "1"  /* Reposition Inferior Vena Cava, Perc Endo Approach */
"06S10ZZ"  =  "1"  /* Reposition Splenic Vein, Open Approach */
"06S13ZZ"  =  "1"  /* Reposition Splenic Vein, Percutaneous Approach */
"06S14ZZ"  =  "1"  /* Reposition Splenic Vein, Percutaneous Endoscopic Approach */
"06S20ZZ"  =  "1"  /* Reposition Gastric Vein, Open Approach */
"06S23ZZ"  =  "1"  /* Reposition Gastric Vein, Percutaneous Approach */
"06S24ZZ"  =  "1"  /* Reposition Gastric Vein, Percutaneous Endoscopic Approach */
"06S30ZZ"  =  "1"  /* Reposition Esophageal Vein, Open Approach */
"06S33ZZ"  =  "1"  /* Reposition Esophageal Vein, Percutaneous Approach */
"06S34ZZ"  =  "1"  /* Reposition Esophageal Vein, Percutaneous Endoscopic Approach */
"06S40ZZ"  =  "1"  /* Reposition Hepatic Vein, Open Approach */
"06S43ZZ"  =  "1"  /* Reposition Hepatic Vein, Percutaneous Approach */
"06S44ZZ"  =  "1"  /* Reposition Hepatic Vein, Percutaneous Endoscopic Approach */
"06S50ZZ"  =  "1"  /* Reposition Superior Mesenteric Vein, Open Approach */
"06S53ZZ"  =  "1"  /* Reposition Superior Mesenteric Vein, Percutaneous Approach */
"06S54ZZ"  =  "1"  /* Reposition Superior Mesenteric Vein, Perc Endo Approach */
"06S60ZZ"  =  "1"  /* Reposition Inferior Mesenteric Vein, Open Approach */
"06S63ZZ"  =  "1"  /* Reposition Inferior Mesenteric Vein, Percutaneous Approach */
"06S64ZZ"  =  "1"  /* Reposition Inferior Mesenteric Vein, Perc Endo Approach */
"06S70ZZ"  =  "1"  /* Reposition Colic Vein, Open Approach */
"06S73ZZ"  =  "1"  /* Reposition Colic Vein, Percutaneous Approach */
"06S74ZZ"  =  "1"  /* Reposition Colic Vein, Percutaneous Endoscopic Approach */
"06S80ZZ"  =  "1"  /* Reposition Portal Vein, Open Approach */
"06S83ZZ"  =  "1"  /* Reposition Portal Vein, Percutaneous Approach */
"06S84ZZ"  =  "1"  /* Reposition Portal Vein, Percutaneous Endoscopic Approach */
"06S90ZZ"  =  "1"  /* Reposition Right Renal Vein, Open Approach */
"06S93ZZ"  =  "1"  /* Reposition Right Renal Vein, Percutaneous Approach */
"06S94ZZ"  =  "1"  /* Reposition Right Renal Vein, Perc Endo Approach */
"06SB0ZZ"  =  "1"  /* Reposition Left Renal Vein, Open Approach */
"06SB3ZZ"  =  "1"  /* Reposition Left Renal Vein, Percutaneous Approach */
"06SB4ZZ"  =  "1"  /* Reposition Left Renal Vein, Percutaneous Endoscopic Approach */
"06SC0ZZ"  =  "1"  /* Reposition Right Common Iliac Vein, Open Approach */
"06SC3ZZ"  =  "1"  /* Reposition Right Common Iliac Vein, Percutaneous Approach */
"06SC4ZZ"  =  "1"  /* Reposition Right Common Iliac Vein, Perc Endo Approach */
"06SD0ZZ"  =  "1"  /* Reposition Left Common Iliac Vein, Open Approach */
"06SD3ZZ"  =  "1"  /* Reposition Left Common Iliac Vein, Percutaneous Approach */
"06SD4ZZ"  =  "1"  /* Reposition Left Common Iliac Vein, Perc Endo Approach */
"06SF0ZZ"  =  "1"  /* Reposition Right External Iliac Vein, Open Approach */
"06SF3ZZ"  =  "1"  /* Reposition Right External Iliac Vein, Percutaneous Approach */
"06SF4ZZ"  =  "1"  /* Reposition Right External Iliac Vein, Perc Endo Approach */
"06SG0ZZ"  =  "1"  /* Reposition Left External Iliac Vein, Open Approach */
"06SG3ZZ"  =  "1"  /* Reposition Left External Iliac Vein, Percutaneous Approach */
"06SG4ZZ"  =  "1"  /* Reposition Left External Iliac Vein, Perc Endo Approach */
"06SH0ZZ"  =  "1"  /* Reposition Right Hypogastric Vein, Open Approach */
"06SH3ZZ"  =  "1"  /* Reposition Right Hypogastric Vein, Percutaneous Approach */
"06SH4ZZ"  =  "1"  /* Reposition Right Hypogastric Vein, Perc Endo Approach */
"06SJ0ZZ"  =  "1"  /* Reposition Left Hypogastric Vein, Open Approach */
"06SJ3ZZ"  =  "1"  /* Reposition Left Hypogastric Vein, Percutaneous Approach */
"06SJ4ZZ"  =  "1"  /* Reposition Left Hypogastric Vein, Perc Endo Approach */
"06SM0ZZ"  =  "1"  /* Reposition Right Femoral Vein, Open Approach */
"06SM3ZZ"  =  "1"  /* Reposition Right Femoral Vein, Percutaneous Approach */
"06SM4ZZ"  =  "1"  /* Reposition Right Femoral Vein, Perc Endo Approach */
"06SN0ZZ"  =  "1"  /* Reposition Left Femoral Vein, Open Approach */
"06SN3ZZ"  =  "1"  /* Reposition Left Femoral Vein, Percutaneous Approach */
"06SN4ZZ"  =  "1"  /* Reposition Left Femoral Vein, Perc Endo Approach */
"06SP0ZZ"  =  "1"  /* Reposition Right Greater Saphenous Vein, Open Approach */
"06SP3ZZ"  =  "1"  /* Reposition Right Greater Saphenous Vein, Perc Approach */
"06SP4ZZ"  =  "1"  /* Reposition Right Greater Saphenous Vein, Perc Endo Approach */
"06SQ0ZZ"  =  "1"  /* Reposition Left Greater Saphenous Vein, Open Approach */
"06SQ3ZZ"  =  "1"  /* Reposition Left Greater Saphenous Vein, Perc Approach */
"06SQ4ZZ"  =  "1"  /* Reposition Left Greater Saphenous Vein, Perc Endo Approach */
"06SR0ZZ"  =  "1"  /* Reposition Right Lesser Saphenous Vein, Open Approach */
"06SR3ZZ"  =  "1"  /* Reposition Right Lesser Saphenous Vein, Perc Approach */
"06SR4ZZ"  =  "1"  /* Reposition Right Lesser Saphenous Vein, Perc Endo Approach */
"06SS0ZZ"  =  "1"  /* Reposition Left Lesser Saphenous Vein, Open Approach */
"06SS3ZZ"  =  "1"  /* Reposition Left Lesser Saphenous Vein, Percutaneous Approach */
"06SS4ZZ"  =  "1"  /* Reposition Left Lesser Saphenous Vein, Perc Endo Approach */
"06ST0ZZ"  =  "1"  /* Reposition Right Foot Vein, Open Approach */
"06ST3ZZ"  =  "1"  /* Reposition Right Foot Vein, Percutaneous Approach */
"06ST4ZZ"  =  "1"  /* Reposition Right Foot Vein, Percutaneous Endoscopic Approach */
"06SV0ZZ"  =  "1"  /* Reposition Left Foot Vein, Open Approach */
"06SV3ZZ"  =  "1"  /* Reposition Left Foot Vein, Percutaneous Approach */
"06SV4ZZ"  =  "1"  /* Reposition Left Foot Vein, Percutaneous Endoscopic Approach */
"06SY0ZZ"  =  "1"  /* Reposition Lower Vein, Open Approach */
"06SY3ZZ"  =  "1"  /* Reposition Lower Vein, Percutaneous Approach */
"06SY4ZZ"  =  "1"  /* Reposition Lower Vein, Percutaneous Endoscopic Approach */
"06U007Z"  =  "1"  /* Supplement Inferior Vena Cava with Autol Sub, Open Approach */
"06U00JZ"  =  "1"  /* Supplement Inferior Vena Cava with Synth Sub, Open Approach */
"06U00KZ"  =  "1"  /* Supplement Inferior Vena Cava with Nonaut Sub, Open Approach */
"06U037Z"  =  "1"  /* Supplement Inferior Vena Cava with Autol Sub, Perc Approach */
"06U03JZ"  =  "1"  /* Supplement Inferior Vena Cava with Synth Sub, Perc Approach */
"06U03KZ"  =  "1"  /* Supplement Inferior Vena Cava with Nonaut Sub, Perc Approach */
"06U047Z"  =  "1"  /* Supplement Inf Vena Cava with Autol Sub, Perc Endo Approach */
"06U04JZ"  =  "1"  /* Supplement Inf Vena Cava with Synth Sub, Perc Endo Approach */
"06U04KZ"  =  "1"  /* Supplement Inf Vena Cava with Nonaut Sub, Perc Endo Approach */
"06U107Z"  =  "1"  /* Supplement Splenic Vein with Autol Sub, Open Approach */
"06U10JZ"  =  "1"  /* Supplement Splenic Vein with Synth Sub, Open Approach */
"06U10KZ"  =  "1"  /* Supplement Splenic Vein with Nonaut Sub, Open Approach */
"06U137Z"  =  "1"  /* Supplement Splenic Vein with Autol Sub, Perc Approach */
"06U13JZ"  =  "1"  /* Supplement Splenic Vein with Synth Sub, Perc Approach */
"06U13KZ"  =  "1"  /* Supplement Splenic Vein with Nonaut Sub, Perc Approach */
"06U147Z"  =  "1"  /* Supplement Splenic Vein with Autol Sub, Perc Endo Approach */
"06U14JZ"  =  "1"  /* Supplement Splenic Vein with Synth Sub, Perc Endo Approach */
"06U14KZ"  =  "1"  /* Supplement Splenic Vein with Nonaut Sub, Perc Endo Approach */
"06U207Z"  =  "1"  /* Supplement Gastric Vein with Autol Sub, Open Approach */
"06U20JZ"  =  "1"  /* Supplement Gastric Vein with Synth Sub, Open Approach */
"06U20KZ"  =  "1"  /* Supplement Gastric Vein with Nonaut Sub, Open Approach */
"06U237Z"  =  "1"  /* Supplement Gastric Vein with Autol Sub, Perc Approach */
"06U23JZ"  =  "1"  /* Supplement Gastric Vein with Synth Sub, Perc Approach */
"06U23KZ"  =  "1"  /* Supplement Gastric Vein with Nonaut Sub, Perc Approach */
"06U247Z"  =  "1"  /* Supplement Gastric Vein with Autol Sub, Perc Endo Approach */
"06U24JZ"  =  "1"  /* Supplement Gastric Vein with Synth Sub, Perc Endo Approach */
"06U24KZ"  =  "1"  /* Supplement Gastric Vein with Nonaut Sub, Perc Endo Approach */
"06U307Z"  =  "1"  /* Supplement Esophageal Vein with Autol Sub, Open Approach */
"06U30JZ"  =  "1"  /* Supplement Esophageal Vein with Synth Sub, Open Approach */
"06U30KZ"  =  "1"  /* Supplement Esophageal Vein with Nonaut Sub, Open Approach */
"06U337Z"  =  "1"  /* Supplement Esophageal Vein with Autol Sub, Perc Approach */
"06U33JZ"  =  "1"  /* Supplement Esophageal Vein with Synth Sub, Perc Approach */
"06U33KZ"  =  "1"  /* Supplement Esophageal Vein with Nonaut Sub, Perc Approach */
"06U347Z"  =  "1"  /* Supplement Esophageal Vein w Autol Sub, Perc Endo */
"06U34JZ"  =  "1"  /* Supplement Esophageal Vein w Synth Sub, Perc Endo */
"06U34KZ"  =  "1"  /* Supplement Esophageal Vein w Nonaut Sub, Perc Endo */
"06U407Z"  =  "1"  /* Supplement Hepatic Vein with Autol Sub, Open Approach */
"06U40JZ"  =  "1"  /* Supplement Hepatic Vein with Synth Sub, Open Approach */
"06U40KZ"  =  "1"  /* Supplement Hepatic Vein with Nonaut Sub, Open Approach */
"06U437Z"  =  "1"  /* Supplement Hepatic Vein with Autol Sub, Perc Approach */
"06U43JZ"  =  "1"  /* Supplement Hepatic Vein with Synth Sub, Perc Approach */
"06U43KZ"  =  "1"  /* Supplement Hepatic Vein with Nonaut Sub, Perc Approach */
"06U447Z"  =  "1"  /* Supplement Hepatic Vein with Autol Sub, Perc Endo Approach */
"06U44JZ"  =  "1"  /* Supplement Hepatic Vein with Synth Sub, Perc Endo Approach */
"06U44KZ"  =  "1"  /* Supplement Hepatic Vein with Nonaut Sub, Perc Endo Approach */
"06U507Z"  =  "1"  /* Supplement Sup Mesent Vein with Autol Sub, Open Approach */
"06U50JZ"  =  "1"  /* Supplement Sup Mesent Vein with Synth Sub, Open Approach */
"06U50KZ"  =  "1"  /* Supplement Sup Mesent Vein with Nonaut Sub, Open Approach */
"06U537Z"  =  "1"  /* Supplement Sup Mesent Vein with Autol Sub, Perc Approach */
"06U53JZ"  =  "1"  /* Supplement Sup Mesent Vein with Synth Sub, Perc Approach */
"06U53KZ"  =  "1"  /* Supplement Sup Mesent Vein with Nonaut Sub, Perc Approach */
"06U547Z"  =  "1"  /* Supplement Sup Mesent Vein w Autol Sub, Perc Endo */
"06U54JZ"  =  "1"  /* Supplement Sup Mesent Vein w Synth Sub, Perc Endo */
"06U54KZ"  =  "1"  /* Supplement Sup Mesent Vein w Nonaut Sub, Perc Endo */
"06U607Z"  =  "1"  /* Supplement Inf Mesent Vein with Autol Sub, Open Approach */
"06U60JZ"  =  "1"  /* Supplement Inf Mesent Vein with Synth Sub, Open Approach */
"06U60KZ"  =  "1"  /* Supplement Inf Mesent Vein with Nonaut Sub, Open Approach */
"06U637Z"  =  "1"  /* Supplement Inf Mesent Vein with Autol Sub, Perc Approach */
"06U63JZ"  =  "1"  /* Supplement Inf Mesent Vein with Synth Sub, Perc Approach */
"06U63KZ"  =  "1"  /* Supplement Inf Mesent Vein with Nonaut Sub, Perc Approach */
"06U647Z"  =  "1"  /* Supplement Inf Mesent Vein w Autol Sub, Perc Endo */
"06U64JZ"  =  "1"  /* Supplement Inf Mesent Vein w Synth Sub, Perc Endo */
"06U64KZ"  =  "1"  /* Supplement Inf Mesent Vein w Nonaut Sub, Perc Endo */
"06U707Z"  =  "1"  /* Supplement Colic Vein with Autol Sub, Open Approach */
"06U70JZ"  =  "1"  /* Supplement Colic Vein with Synth Sub, Open Approach */
"06U70KZ"  =  "1"  /* Supplement Colic Vein with Nonaut Sub, Open Approach */
"06U737Z"  =  "1"  /* Supplement Colic Vein with Autol Sub, Perc Approach */
"06U73JZ"  =  "1"  /* Supplement Colic Vein with Synth Sub, Perc Approach */
"06U73KZ"  =  "1"  /* Supplement Colic Vein with Nonaut Sub, Perc Approach */
"06U747Z"  =  "1"  /* Supplement Colic Vein with Autol Sub, Perc Endo Approach */
"06U74JZ"  =  "1"  /* Supplement Colic Vein with Synth Sub, Perc Endo Approach */
"06U74KZ"  =  "1"  /* Supplement Colic Vein with Nonaut Sub, Perc Endo Approach */
"06U807Z"  =  "1"  /* Supplement Portal Vein with Autol Sub, Open Approach */
"06U80JZ"  =  "1"  /* Supplement Portal Vein with Synth Sub, Open Approach */
"06U80KZ"  =  "1"  /* Supplement Portal Vein with Nonaut Sub, Open Approach */
"06U837Z"  =  "1"  /* Supplement Portal Vein with Autol Sub, Perc Approach */
"06U83JZ"  =  "1"  /* Supplement Portal Vein with Synth Sub, Perc Approach */
"06U83KZ"  =  "1"  /* Supplement Portal Vein with Nonaut Sub, Perc Approach */
"06U847Z"  =  "1"  /* Supplement Portal Vein with Autol Sub, Perc Endo Approach */
"06U84JZ"  =  "1"  /* Supplement Portal Vein with Synth Sub, Perc Endo Approach */
"06U84KZ"  =  "1"  /* Supplement Portal Vein with Nonaut Sub, Perc Endo Approach */
"06U907Z"  =  "1"  /* Supplement Right Renal Vein with Autol Sub, Open Approach */
"06U90JZ"  =  "1"  /* Supplement Right Renal Vein with Synth Sub, Open Approach */
"06U90KZ"  =  "1"  /* Supplement Right Renal Vein with Nonaut Sub, Open Approach */
"06U937Z"  =  "1"  /* Supplement Right Renal Vein with Autol Sub, Perc Approach */
"06U93JZ"  =  "1"  /* Supplement Right Renal Vein with Synth Sub, Perc Approach */
"06U93KZ"  =  "1"  /* Supplement Right Renal Vein with Nonaut Sub, Perc Approach */
"06U947Z"  =  "1"  /* Supplement R Renal Vein with Autol Sub, Perc Endo Approach */
"06U94JZ"  =  "1"  /* Supplement R Renal Vein with Synth Sub, Perc Endo Approach */
"06U94KZ"  =  "1"  /* Supplement R Renal Vein with Nonaut Sub, Perc Endo Approach */
"06UB07Z"  =  "1"  /* Supplement Left Renal Vein with Autol Sub, Open Approach */
"06UB0JZ"  =  "1"  /* Supplement Left Renal Vein with Synth Sub, Open Approach */
"06UB0KZ"  =  "1"  /* Supplement Left Renal Vein with Nonaut Sub, Open Approach */
"06UB37Z"  =  "1"  /* Supplement Left Renal Vein with Autol Sub, Perc Approach */
"06UB3JZ"  =  "1"  /* Supplement Left Renal Vein with Synth Sub, Perc Approach */
"06UB3KZ"  =  "1"  /* Supplement Left Renal Vein with Nonaut Sub, Perc Approach */
"06UB47Z"  =  "1"  /* Supplement L Renal Vein with Autol Sub, Perc Endo Approach */
"06UB4JZ"  =  "1"  /* Supplement L Renal Vein with Synth Sub, Perc Endo Approach */
"06UB4KZ"  =  "1"  /* Supplement L Renal Vein with Nonaut Sub, Perc Endo Approach */
"06UC07Z"  =  "1"  /* Supplement R Com Iliac Vein with Autol Sub, Open Approach */
"06UC0JZ"  =  "1"  /* Supplement R Com Iliac Vein with Synth Sub, Open Approach */
"06UC0KZ"  =  "1"  /* Supplement R Com Iliac Vein with Nonaut Sub, Open Approach */
"06UC37Z"  =  "1"  /* Supplement R Com Iliac Vein with Autol Sub, Perc Approach */
"06UC3JZ"  =  "1"  /* Supplement R Com Iliac Vein with Synth Sub, Perc Approach */
"06UC3KZ"  =  "1"  /* Supplement R Com Iliac Vein with Nonaut Sub, Perc Approach */
"06UC47Z"  =  "1"  /* Supplement R Com Iliac Vein w Autol Sub, Perc Endo */
"06UC4JZ"  =  "1"  /* Supplement R Com Iliac Vein w Synth Sub, Perc Endo */
"06UC4KZ"  =  "1"  /* Supplement R Com Iliac Vein w Nonaut Sub, Perc Endo */
"06UD07Z"  =  "1"  /* Supplement L Com Iliac Vein with Autol Sub, Open Approach */
"06UD0JZ"  =  "1"  /* Supplement L Com Iliac Vein with Synth Sub, Open Approach */
"06UD0KZ"  =  "1"  /* Supplement L Com Iliac Vein with Nonaut Sub, Open Approach */
"06UD37Z"  =  "1"  /* Supplement L Com Iliac Vein with Autol Sub, Perc Approach */
"06UD3JZ"  =  "1"  /* Supplement L Com Iliac Vein with Synth Sub, Perc Approach */
"06UD3KZ"  =  "1"  /* Supplement L Com Iliac Vein with Nonaut Sub, Perc Approach */
"06UD47Z"  =  "1"  /* Supplement L Com Iliac Vein w Autol Sub, Perc Endo */
"06UD4JZ"  =  "1"  /* Supplement L Com Iliac Vein w Synth Sub, Perc Endo */
"06UD4KZ"  =  "1"  /* Supplement L Com Iliac Vein w Nonaut Sub, Perc Endo */
"06UF07Z"  =  "1"  /* Supplement R Ext Iliac Vein with Autol Sub, Open Approach */
"06UF0JZ"  =  "1"  /* Supplement R Ext Iliac Vein with Synth Sub, Open Approach */
"06UF0KZ"  =  "1"  /* Supplement R Ext Iliac Vein with Nonaut Sub, Open Approach */
"06UF37Z"  =  "1"  /* Supplement R Ext Iliac Vein with Autol Sub, Perc Approach */
"06UF3JZ"  =  "1"  /* Supplement R Ext Iliac Vein with Synth Sub, Perc Approach */
"06UF3KZ"  =  "1"  /* Supplement R Ext Iliac Vein with Nonaut Sub, Perc Approach */
"06UF47Z"  =  "1"  /* Supplement R Ext Iliac Vein w Autol Sub, Perc Endo */
"06UF4JZ"  =  "1"  /* Supplement R Ext Iliac Vein w Synth Sub, Perc Endo */
"06UF4KZ"  =  "1"  /* Supplement R Ext Iliac Vein w Nonaut Sub, Perc Endo */
"06UG07Z"  =  "1"  /* Supplement L Ext Iliac Vein with Autol Sub, Open Approach */
"06UG0JZ"  =  "1"  /* Supplement L Ext Iliac Vein with Synth Sub, Open Approach */
"06UG0KZ"  =  "1"  /* Supplement L Ext Iliac Vein with Nonaut Sub, Open Approach */
"06UG37Z"  =  "1"  /* Supplement L Ext Iliac Vein with Autol Sub, Perc Approach */
"06UG3JZ"  =  "1"  /* Supplement L Ext Iliac Vein with Synth Sub, Perc Approach */
"06UG3KZ"  =  "1"  /* Supplement L Ext Iliac Vein with Nonaut Sub, Perc Approach */
"06UG47Z"  =  "1"  /* Supplement L Ext Iliac Vein w Autol Sub, Perc Endo */
"06UG4JZ"  =  "1"  /* Supplement L Ext Iliac Vein w Synth Sub, Perc Endo */
"06UG4KZ"  =  "1"  /* Supplement L Ext Iliac Vein w Nonaut Sub, Perc Endo */
"06UH07Z"  =  "1"  /* Supplement R Hypogast Vein with Autol Sub, Open Approach */
"06UH0JZ"  =  "1"  /* Supplement R Hypogast Vein with Synth Sub, Open Approach */
"06UH0KZ"  =  "1"  /* Supplement R Hypogast Vein with Nonaut Sub, Open Approach */
"06UH37Z"  =  "1"  /* Supplement R Hypogast Vein with Autol Sub, Perc Approach */
"06UH3JZ"  =  "1"  /* Supplement R Hypogast Vein with Synth Sub, Perc Approach */
"06UH3KZ"  =  "1"  /* Supplement R Hypogast Vein with Nonaut Sub, Perc Approach */
"06UH47Z"  =  "1"  /* Supplement R Hypogast Vein w Autol Sub, Perc Endo */
"06UH4JZ"  =  "1"  /* Supplement R Hypogast Vein w Synth Sub, Perc Endo */
"06UH4KZ"  =  "1"  /* Supplement R Hypogast Vein w Nonaut Sub, Perc Endo */
"06UJ07Z"  =  "1"  /* Supplement L Hypogast Vein with Autol Sub, Open Approach */
"06UJ0JZ"  =  "1"  /* Supplement L Hypogast Vein with Synth Sub, Open Approach */
"06UJ0KZ"  =  "1"  /* Supplement L Hypogast Vein with Nonaut Sub, Open Approach */
"06UJ37Z"  =  "1"  /* Supplement L Hypogast Vein with Autol Sub, Perc Approach */
"06UJ3JZ"  =  "1"  /* Supplement L Hypogast Vein with Synth Sub, Perc Approach */
"06UJ3KZ"  =  "1"  /* Supplement L Hypogast Vein with Nonaut Sub, Perc Approach */
"06UJ47Z"  =  "1"  /* Supplement L Hypogast Vein w Autol Sub, Perc Endo */
"06UJ4JZ"  =  "1"  /* Supplement L Hypogast Vein w Synth Sub, Perc Endo */
"06UJ4KZ"  =  "1"  /* Supplement L Hypogast Vein w Nonaut Sub, Perc Endo */
"06UM07Z"  =  "1"  /* Supplement Right Femoral Vein with Autol Sub, Open Approach */
"06UM0JZ"  =  "1"  /* Supplement Right Femoral Vein with Synth Sub, Open Approach */
"06UM0KZ"  =  "1"  /* Supplement Right Femoral Vein with Nonaut Sub, Open Approach */
"06UM37Z"  =  "1"  /* Supplement Right Femoral Vein with Autol Sub, Perc Approach */
"06UM3JZ"  =  "1"  /* Supplement Right Femoral Vein with Synth Sub, Perc Approach */
"06UM3KZ"  =  "1"  /* Supplement Right Femoral Vein with Nonaut Sub, Perc Approach */
"06UM47Z"  =  "1"  /* Supplement R Femor Vein with Autol Sub, Perc Endo Approach */
"06UM4JZ"  =  "1"  /* Supplement R Femor Vein with Synth Sub, Perc Endo Approach */
"06UM4KZ"  =  "1"  /* Supplement R Femor Vein with Nonaut Sub, Perc Endo Approach */
"06UN07Z"  =  "1"  /* Supplement Left Femoral Vein with Autol Sub, Open Approach */
"06UN0JZ"  =  "1"  /* Supplement Left Femoral Vein with Synth Sub, Open Approach */
"06UN0KZ"  =  "1"  /* Supplement Left Femoral Vein with Nonaut Sub, Open Approach */
"06UN37Z"  =  "1"  /* Supplement Left Femoral Vein with Autol Sub, Perc Approach */
"06UN3JZ"  =  "1"  /* Supplement Left Femoral Vein with Synth Sub, Perc Approach */
"06UN3KZ"  =  "1"  /* Supplement Left Femoral Vein with Nonaut Sub, Perc Approach */
"06UN47Z"  =  "1"  /* Supplement L Femor Vein with Autol Sub, Perc Endo Approach */
"06UN4JZ"  =  "1"  /* Supplement L Femor Vein with Synth Sub, Perc Endo Approach */
"06UN4KZ"  =  "1"  /* Supplement L Femor Vein with Nonaut Sub, Perc Endo Approach */
"06UP07Z"  =  "1"  /* Supplement R Great Saphenous with Autol Sub, Open Approach */
"06UP0JZ"  =  "1"  /* Supplement R Great Saphenous with Synth Sub, Open Approach */
"06UP0KZ"  =  "1"  /* Supplement R Great Saphenous with Nonaut Sub, Open Approach */
"06UP37Z"  =  "1"  /* Supplement R Great Saphenous with Autol Sub, Perc Approach */
"06UP3JZ"  =  "1"  /* Supplement R Great Saphenous with Synth Sub, Perc Approach */
"06UP3KZ"  =  "1"  /* Supplement R Great Saphenous with Nonaut Sub, Perc Approach */
"06UP47Z"  =  "1"  /* Supplement R Great Saphenous w Autol Sub, Perc Endo */
"06UP4JZ"  =  "1"  /* Supplement R Great Saphenous w Synth Sub, Perc Endo */
"06UP4KZ"  =  "1"  /* Supplement R Great Saphenous w Nonaut Sub, Perc Endo */
"06UQ07Z"  =  "1"  /* Supplement L Great Saphenous with Autol Sub, Open Approach */
"06UQ0JZ"  =  "1"  /* Supplement L Great Saphenous with Synth Sub, Open Approach */
"06UQ0KZ"  =  "1"  /* Supplement L Great Saphenous with Nonaut Sub, Open Approach */
"06UQ37Z"  =  "1"  /* Supplement L Great Saphenous with Autol Sub, Perc Approach */
"06UQ3JZ"  =  "1"  /* Supplement L Great Saphenous with Synth Sub, Perc Approach */
"06UQ3KZ"  =  "1"  /* Supplement L Great Saphenous with Nonaut Sub, Perc Approach */
"06UQ47Z"  =  "1"  /* Supplement L Great Saphenous w Autol Sub, Perc Endo */
"06UQ4JZ"  =  "1"  /* Supplement L Great Saphenous w Synth Sub, Perc Endo */
"06UQ4KZ"  =  "1"  /* Supplement L Great Saphenous w Nonaut Sub, Perc Endo */
"06UR07Z"  =  "1"  /* Supplement R Less Saphenous with Autol Sub, Open Approach */
"06UR0JZ"  =  "1"  /* Supplement R Less Saphenous with Synth Sub, Open Approach */
"06UR0KZ"  =  "1"  /* Supplement R Less Saphenous with Nonaut Sub, Open Approach */
"06UR37Z"  =  "1"  /* Supplement R Less Saphenous with Autol Sub, Perc Approach */
"06UR3JZ"  =  "1"  /* Supplement R Less Saphenous with Synth Sub, Perc Approach */
"06UR3KZ"  =  "1"  /* Supplement R Less Saphenous with Nonaut Sub, Perc Approach */
"06UR47Z"  =  "1"  /* Supplement R Less Saphenous w Autol Sub, Perc Endo */
"06UR4JZ"  =  "1"  /* Supplement R Less Saphenous w Synth Sub, Perc Endo */
"06UR4KZ"  =  "1"  /* Supplement R Less Saphenous w Nonaut Sub, Perc Endo */
"06US07Z"  =  "1"  /* Supplement L Less Saphenous with Autol Sub, Open Approach */
"06US0JZ"  =  "1"  /* Supplement L Less Saphenous with Synth Sub, Open Approach */
"06US0KZ"  =  "1"  /* Supplement L Less Saphenous with Nonaut Sub, Open Approach */
"06US37Z"  =  "1"  /* Supplement L Less Saphenous with Autol Sub, Perc Approach */
"06US3JZ"  =  "1"  /* Supplement L Less Saphenous with Synth Sub, Perc Approach */
"06US3KZ"  =  "1"  /* Supplement L Less Saphenous with Nonaut Sub, Perc Approach */
"06US47Z"  =  "1"  /* Supplement L Less Saphenous w Autol Sub, Perc Endo */
"06US4JZ"  =  "1"  /* Supplement L Less Saphenous w Synth Sub, Perc Endo */
"06US4KZ"  =  "1"  /* Supplement L Less Saphenous w Nonaut Sub, Perc Endo */
"06UT07Z"  =  "1"  /* Supplement Right Foot Vein with Autol Sub, Open Approach */
"06UT0JZ"  =  "1"  /* Supplement Right Foot Vein with Synth Sub, Open Approach */
"06UT0KZ"  =  "1"  /* Supplement Right Foot Vein with Nonaut Sub, Open Approach */
"06UT37Z"  =  "1"  /* Supplement Right Foot Vein with Autol Sub, Perc Approach */
"06UT3JZ"  =  "1"  /* Supplement Right Foot Vein with Synth Sub, Perc Approach */
"06UT3KZ"  =  "1"  /* Supplement Right Foot Vein with Nonaut Sub, Perc Approach */
"06UT47Z"  =  "1"  /* Supplement R Foot Vein with Autol Sub, Perc Endo Approach */
"06UT4JZ"  =  "1"  /* Supplement R Foot Vein with Synth Sub, Perc Endo Approach */
"06UT4KZ"  =  "1"  /* Supplement R Foot Vein with Nonaut Sub, Perc Endo Approach */
"06UV07Z"  =  "1"  /* Supplement Left Foot Vein with Autol Sub, Open Approach */
"06UV0JZ"  =  "1"  /* Supplement Left Foot Vein with Synth Sub, Open Approach */
"06UV0KZ"  =  "1"  /* Supplement Left Foot Vein with Nonaut Sub, Open Approach */
"06UV37Z"  =  "1"  /* Supplement Left Foot Vein with Autol Sub, Perc Approach */
"06UV3JZ"  =  "1"  /* Supplement Left Foot Vein with Synth Sub, Perc Approach */
"06UV3KZ"  =  "1"  /* Supplement Left Foot Vein with Nonaut Sub, Perc Approach */
"06UV47Z"  =  "1"  /* Supplement Left Foot Vein with Autol Sub, Perc Endo Approach */
"06UV4JZ"  =  "1"  /* Supplement Left Foot Vein with Synth Sub, Perc Endo Approach */
"06UV4KZ"  =  "1"  /* Supplement L Foot Vein with Nonaut Sub, Perc Endo Approach */
"06UY07Z"  =  "1"  /* Supplement Lower Vein with Autol Sub, Open Approach */
"06UY0JZ"  =  "1"  /* Supplement Lower Vein with Synth Sub, Open Approach */
"06UY0KZ"  =  "1"  /* Supplement Lower Vein with Nonaut Sub, Open Approach */
"06UY37Z"  =  "1"  /* Supplement Lower Vein with Autol Sub, Perc Approach */
"06UY3JZ"  =  "1"  /* Supplement Lower Vein with Synth Sub, Perc Approach */
"06UY3KZ"  =  "1"  /* Supplement Lower Vein with Nonaut Sub, Perc Approach */
"06UY47Z"  =  "1"  /* Supplement Lower Vein with Autol Sub, Perc Endo Approach */
"06UY4JZ"  =  "1"  /* Supplement Lower Vein with Synth Sub, Perc Endo Approach */
"06UY4KZ"  =  "1"  /* Supplement Lower Vein with Nonaut Sub, Perc Endo Approach */
"06V00CZ"  =  "1"  /* Restrict of Inf Vena Cava with Extralum Dev, Open Approach */
"06V00DZ"  =  "1"  /* Restrict of Inf Vena Cava with Intralum Dev, Open Approach */
"06V00ZZ"  =  "1"  /* Restriction of Inferior Vena Cava, Open Approach */
"06V03CZ"  =  "1"  /* Restrict of Inf Vena Cava with Extralum Dev, Perc Approach */
"06V03DZ"  =  "1"  /* Restrict of Inf Vena Cava with Intralum Dev, Perc Approach */
"06V03ZZ"  =  "1"  /* Restriction of Inferior Vena Cava, Percutaneous Approach */
"06V04CZ"  =  "1"  /* Restrict Inf Vena Cava w Extralum Dev, Perc Endo */
"06V04DZ"  =  "1"  /* Restrict Inf Vena Cava w Intralum Dev, Perc Endo */
"06V04ZZ"  =  "1"  /* Restriction of Inferior Vena Cava, Perc Endo Approach */
"06V10CZ"  =  "1"  /* Restriction of Splenic Vein with Extralum Dev, Open Approach */
"06V10DZ"  =  "1"  /* Restriction of Splenic Vein with Intralum Dev, Open Approach */
"06V10ZZ"  =  "1"  /* Restriction of Splenic Vein, Open Approach */
"06V13CZ"  =  "1"  /* Restriction of Splenic Vein with Extralum Dev, Perc Approach */
"06V13DZ"  =  "1"  /* Restriction of Splenic Vein with Intralum Dev, Perc Approach */
"06V13ZZ"  =  "1"  /* Restriction of Splenic Vein, Percutaneous Approach */
"06V14CZ"  =  "1"  /* Restrict Splenic Vein w Extralum Dev, Perc Endo */
"06V14DZ"  =  "1"  /* Restrict Splenic Vein w Intralum Dev, Perc Endo */
"06V14ZZ"  =  "1"  /* Restriction of Splenic Vein, Perc Endo Approach */
"06V20CZ"  =  "1"  /* Restriction of Gastric Vein with Extralum Dev, Open Approach */
"06V20DZ"  =  "1"  /* Restriction of Gastric Vein with Intralum Dev, Open Approach */
"06V20ZZ"  =  "1"  /* Restriction of Gastric Vein, Open Approach */
"06V23CZ"  =  "1"  /* Restriction of Gastric Vein with Extralum Dev, Perc Approach */
"06V23DZ"  =  "1"  /* Restriction of Gastric Vein with Intralum Dev, Perc Approach */
"06V23ZZ"  =  "1"  /* Restriction of Gastric Vein, Percutaneous Approach */
"06V24CZ"  =  "1"  /* Restrict Gastric Vein w Extralum Dev, Perc Endo */
"06V24DZ"  =  "1"  /* Restrict Gastric Vein w Intralum Dev, Perc Endo */
"06V24ZZ"  =  "1"  /* Restriction of Gastric Vein, Perc Endo Approach */
"06V30CZ"  =  "1"  /* Restrict of Esophageal Vein with Extralum Dev, Open Approach */
"06V30DZ"  =  "1"  /* Restrict of Esophageal Vein with Intralum Dev, Open Approach */
"06V30ZZ"  =  "1"  /* Restriction of Esophageal Vein, Open Approach */
"06V33CZ"  =  "1"  /* Restrict of Esophageal Vein with Extralum Dev, Perc Approach */
"06V33DZ"  =  "1"  /* Restrict of Esophageal Vein with Intralum Dev, Perc Approach */
"06V33ZZ"  =  "1"  /* Restriction of Esophageal Vein, Percutaneous Approach */
"06V34CZ"  =  "1"  /* Restrict Esophageal Vein w Extralum Dev, Perc Endo */
"06V34DZ"  =  "1"  /* Restrict Esophageal Vein w Intralum Dev, Perc Endo */
"06V34ZZ"  =  "1"  /* Restriction of Esophageal Vein, Perc Endo Approach */
"06V40CZ"  =  "1"  /* Restriction of Hepatic Vein with Extralum Dev, Open Approach */
"06V40DZ"  =  "1"  /* Restriction of Hepatic Vein with Intralum Dev, Open Approach */
"06V40ZZ"  =  "1"  /* Restriction of Hepatic Vein, Open Approach */
"06V43CZ"  =  "1"  /* Restriction of Hepatic Vein with Extralum Dev, Perc Approach */
"06V43DZ"  =  "1"  /* Restriction of Hepatic Vein with Intralum Dev, Perc Approach */
"06V43ZZ"  =  "1"  /* Restriction of Hepatic Vein, Percutaneous Approach */
"06V44CZ"  =  "1"  /* Restrict Hepatic Vein w Extralum Dev, Perc Endo */
"06V44DZ"  =  "1"  /* Restrict Hepatic Vein w Intralum Dev, Perc Endo */
"06V44ZZ"  =  "1"  /* Restriction of Hepatic Vein, Perc Endo Approach */
"06V50CZ"  =  "1"  /* Restrict of Sup Mesent Vein with Extralum Dev, Open Approach */
"06V50DZ"  =  "1"  /* Restrict of Sup Mesent Vein with Intralum Dev, Open Approach */
"06V50ZZ"  =  "1"  /* Restriction of Superior Mesenteric Vein, Open Approach */
"06V53CZ"  =  "1"  /* Restrict of Sup Mesent Vein with Extralum Dev, Perc Approach */
"06V53DZ"  =  "1"  /* Restrict of Sup Mesent Vein with Intralum Dev, Perc Approach */
"06V53ZZ"  =  "1"  /* Restriction of Superior Mesenteric Vein, Perc Approach */
"06V54CZ"  =  "1"  /* Restrict Sup Mesent Vein w Extralum Dev, Perc Endo */
"06V54DZ"  =  "1"  /* Restrict Sup Mesent Vein w Intralum Dev, Perc Endo */
"06V54ZZ"  =  "1"  /* Restriction of Superior Mesenteric Vein, Perc Endo Approach */
"06V60CZ"  =  "1"  /* Restrict of Inf Mesent Vein with Extralum Dev, Open Approach */
"06V60DZ"  =  "1"  /* Restrict of Inf Mesent Vein with Intralum Dev, Open Approach */
"06V60ZZ"  =  "1"  /* Restriction of Inferior Mesenteric Vein, Open Approach */
"06V63CZ"  =  "1"  /* Restrict of Inf Mesent Vein with Extralum Dev, Perc Approach */
"06V63DZ"  =  "1"  /* Restrict of Inf Mesent Vein with Intralum Dev, Perc Approach */
"06V63ZZ"  =  "1"  /* Restriction of Inferior Mesenteric Vein, Perc Approach */
"06V64CZ"  =  "1"  /* Restrict Inf Mesent Vein w Extralum Dev, Perc Endo */
"06V64DZ"  =  "1"  /* Restrict Inf Mesent Vein w Intralum Dev, Perc Endo */
"06V64ZZ"  =  "1"  /* Restriction of Inferior Mesenteric Vein, Perc Endo Approach */
"06V70CZ"  =  "1"  /* Restriction of Colic Vein with Extralum Dev, Open Approach */
"06V70DZ"  =  "1"  /* Restriction of Colic Vein with Intralum Dev, Open Approach */
"06V70ZZ"  =  "1"  /* Restriction of Colic Vein, Open Approach */
"06V73CZ"  =  "1"  /* Restriction of Colic Vein with Extralum Dev, Perc Approach */
"06V73DZ"  =  "1"  /* Restriction of Colic Vein with Intralum Dev, Perc Approach */
"06V73ZZ"  =  "1"  /* Restriction of Colic Vein, Percutaneous Approach */
"06V74CZ"  =  "1"  /* Restrict of Colic Vein with Extralum Dev, Perc Endo Approach */
"06V74DZ"  =  "1"  /* Restrict of Colic Vein with Intralum Dev, Perc Endo Approach */
"06V74ZZ"  =  "1"  /* Restriction of Colic Vein, Percutaneous Endoscopic Approach */
"06V80CZ"  =  "1"  /* Restriction of Portal Vein with Extralum Dev, Open Approach */
"06V80DZ"  =  "1"  /* Restriction of Portal Vein with Intralum Dev, Open Approach */
"06V80ZZ"  =  "1"  /* Restriction of Portal Vein, Open Approach */
"06V83CZ"  =  "1"  /* Restriction of Portal Vein with Extralum Dev, Perc Approach */
"06V83DZ"  =  "1"  /* Restriction of Portal Vein with Intralum Dev, Perc Approach */
"06V83ZZ"  =  "1"  /* Restriction of Portal Vein, Percutaneous Approach */
"06V84CZ"  =  "1"  /* Restrict Portal Vein w Extralum Dev, Perc Endo */
"06V84DZ"  =  "1"  /* Restrict Portal Vein w Intralum Dev, Perc Endo */
"06V84ZZ"  =  "1"  /* Restriction of Portal Vein, Percutaneous Endoscopic Approach */
"06V90CZ"  =  "1"  /* Restriction of R Renal Vein with Extralum Dev, Open Approach */
"06V90DZ"  =  "1"  /* Restriction of R Renal Vein with Intralum Dev, Open Approach */
"06V90ZZ"  =  "1"  /* Restriction of Right Renal Vein, Open Approach */
"06V93CZ"  =  "1"  /* Restriction of R Renal Vein with Extralum Dev, Perc Approach */
"06V93DZ"  =  "1"  /* Restriction of R Renal Vein with Intralum Dev, Perc Approach */
"06V93ZZ"  =  "1"  /* Restriction of Right Renal Vein, Percutaneous Approach */
"06V94CZ"  =  "1"  /* Restrict R Renal Vein w Extralum Dev, Perc Endo */
"06V94DZ"  =  "1"  /* Restrict R Renal Vein w Intralum Dev, Perc Endo */
"06V94ZZ"  =  "1"  /* Restriction of Right Renal Vein, Perc Endo Approach */
"06VB0CZ"  =  "1"  /* Restriction of L Renal Vein with Extralum Dev, Open Approach */
"06VB0DZ"  =  "1"  /* Restriction of L Renal Vein with Intralum Dev, Open Approach */
"06VB0ZZ"  =  "1"  /* Restriction of Left Renal Vein, Open Approach */
"06VB3CZ"  =  "1"  /* Restriction of L Renal Vein with Extralum Dev, Perc Approach */
"06VB3DZ"  =  "1"  /* Restriction of L Renal Vein with Intralum Dev, Perc Approach */
"06VB3ZZ"  =  "1"  /* Restriction of Left Renal Vein, Percutaneous Approach */
"06VB4CZ"  =  "1"  /* Restrict L Renal Vein w Extralum Dev, Perc Endo */
"06VB4DZ"  =  "1"  /* Restrict L Renal Vein w Intralum Dev, Perc Endo */
"06VB4ZZ"  =  "1"  /* Restriction of Left Renal Vein, Perc Endo Approach */
"06VC0CZ"  =  "1"  /* Restrict R Com Iliac Vein w Extralum Dev, Open */
"06VC0DZ"  =  "1"  /* Restrict R Com Iliac Vein w Intralum Dev, Open */
"06VC0ZZ"  =  "1"  /* Restriction of Right Common Iliac Vein, Open Approach */
"06VC3CZ"  =  "1"  /* Restrict R Com Iliac Vein w Extralum Dev, Perc */
"06VC3DZ"  =  "1"  /* Restrict R Com Iliac Vein w Intralum Dev, Perc */
"06VC3ZZ"  =  "1"  /* Restriction of Right Common Iliac Vein, Perc Approach */
"06VC4CZ"  =  "1"  /* Restrict R Com Iliac Vein w Extralum Dev, Perc Endo */
"06VC4DZ"  =  "1"  /* Restrict R Com Iliac Vein w Intralum Dev, Perc Endo */
"06VC4ZZ"  =  "1"  /* Restriction of Right Common Iliac Vein, Perc Endo Approach */
"06VD0CZ"  =  "1"  /* Restrict L Com Iliac Vein w Extralum Dev, Open */
"06VD0DZ"  =  "1"  /* Restrict L Com Iliac Vein w Intralum Dev, Open */
"06VD0ZZ"  =  "1"  /* Restriction of Left Common Iliac Vein, Open Approach */
"06VD3CZ"  =  "1"  /* Restrict L Com Iliac Vein w Extralum Dev, Perc */
"06VD3DZ"  =  "1"  /* Restrict L Com Iliac Vein w Intralum Dev, Perc */
"06VD3ZZ"  =  "1"  /* Restriction of Left Common Iliac Vein, Percutaneous Approach */
"06VD4CZ"  =  "1"  /* Restrict L Com Iliac Vein w Extralum Dev, Perc Endo */
"06VD4DZ"  =  "1"  /* Restrict L Com Iliac Vein w Intralum Dev, Perc Endo */
"06VD4ZZ"  =  "1"  /* Restriction of Left Common Iliac Vein, Perc Endo Approach */
"06VF0CZ"  =  "1"  /* Restrict R Ext Iliac Vein w Extralum Dev, Open */
"06VF0DZ"  =  "1"  /* Restrict R Ext Iliac Vein w Intralum Dev, Open */
"06VF0ZZ"  =  "1"  /* Restriction of Right External Iliac Vein, Open Approach */
"06VF3CZ"  =  "1"  /* Restrict R Ext Iliac Vein w Extralum Dev, Perc */
"06VF3DZ"  =  "1"  /* Restrict R Ext Iliac Vein w Intralum Dev, Perc */
"06VF3ZZ"  =  "1"  /* Restriction of Right External Iliac Vein, Perc Approach */
"06VF4CZ"  =  "1"  /* Restrict R Ext Iliac Vein w Extralum Dev, Perc Endo */
"06VF4DZ"  =  "1"  /* Restrict R Ext Iliac Vein w Intralum Dev, Perc Endo */
"06VF4ZZ"  =  "1"  /* Restriction of Right External Iliac Vein, Perc Endo Approach */
"06VG0CZ"  =  "1"  /* Restrict L Ext Iliac Vein w Extralum Dev, Open */
"06VG0DZ"  =  "1"  /* Restrict L Ext Iliac Vein w Intralum Dev, Open */
"06VG0ZZ"  =  "1"  /* Restriction of Left External Iliac Vein, Open Approach */
"06VG3CZ"  =  "1"  /* Restrict L Ext Iliac Vein w Extralum Dev, Perc */
"06VG3DZ"  =  "1"  /* Restrict L Ext Iliac Vein w Intralum Dev, Perc */
"06VG3ZZ"  =  "1"  /* Restriction of Left External Iliac Vein, Perc Approach */
"06VG4CZ"  =  "1"  /* Restrict L Ext Iliac Vein w Extralum Dev, Perc Endo */
"06VG4DZ"  =  "1"  /* Restrict L Ext Iliac Vein w Intralum Dev, Perc Endo */
"06VG4ZZ"  =  "1"  /* Restriction of Left External Iliac Vein, Perc Endo Approach */
"06VH0CZ"  =  "1"  /* Restrict of R Hypogast Vein with Extralum Dev, Open Approach */
"06VH0DZ"  =  "1"  /* Restrict of R Hypogast Vein with Intralum Dev, Open Approach */
"06VH0ZZ"  =  "1"  /* Restriction of Right Hypogastric Vein, Open Approach */
"06VH3CZ"  =  "1"  /* Restrict of R Hypogast Vein with Extralum Dev, Perc Approach */
"06VH3DZ"  =  "1"  /* Restrict of R Hypogast Vein with Intralum Dev, Perc Approach */
"06VH3ZZ"  =  "1"  /* Restriction of Right Hypogastric Vein, Percutaneous Approach */
"06VH4CZ"  =  "1"  /* Restrict R Hypogast Vein w Extralum Dev, Perc Endo */
"06VH4DZ"  =  "1"  /* Restrict R Hypogast Vein w Intralum Dev, Perc Endo */
"06VH4ZZ"  =  "1"  /* Restriction of Right Hypogastric Vein, Perc Endo Approach */
"06VJ0CZ"  =  "1"  /* Restrict of L Hypogast Vein with Extralum Dev, Open Approach */
"06VJ0DZ"  =  "1"  /* Restrict of L Hypogast Vein with Intralum Dev, Open Approach */
"06VJ0ZZ"  =  "1"  /* Restriction of Left Hypogastric Vein, Open Approach */
"06VJ3CZ"  =  "1"  /* Restrict of L Hypogast Vein with Extralum Dev, Perc Approach */
"06VJ3DZ"  =  "1"  /* Restrict of L Hypogast Vein with Intralum Dev, Perc Approach */
"06VJ3ZZ"  =  "1"  /* Restriction of Left Hypogastric Vein, Percutaneous Approach */
"06VJ4CZ"  =  "1"  /* Restrict L Hypogast Vein w Extralum Dev, Perc Endo */
"06VJ4DZ"  =  "1"  /* Restrict L Hypogast Vein w Intralum Dev, Perc Endo */
"06VJ4ZZ"  =  "1"  /* Restriction of Left Hypogastric Vein, Perc Endo Approach */
"06VM0CZ"  =  "1"  /* Restriction of R Femor Vein with Extralum Dev, Open Approach */
"06VM0DZ"  =  "1"  /* Restriction of R Femor Vein with Intralum Dev, Open Approach */
"06VM0ZZ"  =  "1"  /* Restriction of Right Femoral Vein, Open Approach */
"06VM3CZ"  =  "1"  /* Restriction of R Femor Vein with Extralum Dev, Perc Approach */
"06VM3DZ"  =  "1"  /* Restriction of R Femor Vein with Intralum Dev, Perc Approach */
"06VM3ZZ"  =  "1"  /* Restriction of Right Femoral Vein, Percutaneous Approach */
"06VM4CZ"  =  "1"  /* Restrict R Femor Vein w Extralum Dev, Perc Endo */
"06VM4DZ"  =  "1"  /* Restrict R Femor Vein w Intralum Dev, Perc Endo */
"06VM4ZZ"  =  "1"  /* Restriction of Right Femoral Vein, Perc Endo Approach */
"06VN0CZ"  =  "1"  /* Restriction of L Femor Vein with Extralum Dev, Open Approach */
"06VN0DZ"  =  "1"  /* Restriction of L Femor Vein with Intralum Dev, Open Approach */
"06VN0ZZ"  =  "1"  /* Restriction of Left Femoral Vein, Open Approach */
"06VN3CZ"  =  "1"  /* Restriction of L Femor Vein with Extralum Dev, Perc Approach */
"06VN3DZ"  =  "1"  /* Restriction of L Femor Vein with Intralum Dev, Perc Approach */
"06VN3ZZ"  =  "1"  /* Restriction of Left Femoral Vein, Percutaneous Approach */
"06VN4CZ"  =  "1"  /* Restrict L Femor Vein w Extralum Dev, Perc Endo */
"06VN4DZ"  =  "1"  /* Restrict L Femor Vein w Intralum Dev, Perc Endo */
"06VN4ZZ"  =  "1"  /* Restriction of Left Femoral Vein, Perc Endo Approach */
"06VP0CZ"  =  "1"  /* Restrict R Great Saphenous w Extralum Dev, Open */
"06VP0DZ"  =  "1"  /* Restrict R Great Saphenous w Intralum Dev, Open */
"06VP0ZZ"  =  "1"  /* Restriction of Right Greater Saphenous Vein, Open Approach */
"06VP3CZ"  =  "1"  /* Restrict R Great Saphenous w Extralum Dev, Perc */
"06VP3DZ"  =  "1"  /* Restrict R Great Saphenous w Intralum Dev, Perc */
"06VP3ZZ"  =  "1"  /* Restriction of Right Greater Saphenous Vein, Perc Approach */
"06VP4CZ"  =  "1"  /* Restrict R Great Saphenous w Extralum Dev, Perc Endo */
"06VP4DZ"  =  "1"  /* Restrict R Great Saphenous w Intralum Dev, Perc Endo */
"06VP4ZZ"  =  "1"  /* Restriction of R Great Saphenous, Perc Endo Approach */
"06VQ0CZ"  =  "1"  /* Restrict L Great Saphenous w Extralum Dev, Open */
"06VQ0DZ"  =  "1"  /* Restrict L Great Saphenous w Intralum Dev, Open */
"06VQ0ZZ"  =  "1"  /* Restriction of Left Greater Saphenous Vein, Open Approach */
"06VQ3CZ"  =  "1"  /* Restrict L Great Saphenous w Extralum Dev, Perc */
"06VQ3DZ"  =  "1"  /* Restrict L Great Saphenous w Intralum Dev, Perc */
"06VQ3ZZ"  =  "1"  /* Restriction of Left Greater Saphenous Vein, Perc Approach */
"06VQ4CZ"  =  "1"  /* Restrict L Great Saphenous w Extralum Dev, Perc Endo */
"06VQ4DZ"  =  "1"  /* Restrict L Great Saphenous w Intralum Dev, Perc Endo */
"06VQ4ZZ"  =  "1"  /* Restriction of L Great Saphenous, Perc Endo Approach */
"06VR0CZ"  =  "1"  /* Restrict R Less Saphenous w Extralum Dev, Open */
"06VR0DZ"  =  "1"  /* Restrict R Less Saphenous w Intralum Dev, Open */
"06VR0ZZ"  =  "1"  /* Restriction of Right Lesser Saphenous Vein, Open Approach */
"06VR3CZ"  =  "1"  /* Restrict R Less Saphenous w Extralum Dev, Perc */
"06VR3DZ"  =  "1"  /* Restrict R Less Saphenous w Intralum Dev, Perc */
"06VR3ZZ"  =  "1"  /* Restriction of Right Lesser Saphenous Vein, Perc Approach */
"06VR4CZ"  =  "1"  /* Restrict R Less Saphenous w Extralum Dev, Perc Endo */
"06VR4DZ"  =  "1"  /* Restrict R Less Saphenous w Intralum Dev, Perc Endo */
"06VR4ZZ"  =  "1"  /* Restriction of R Less Saphenous, Perc Endo Approach */
"06VS0CZ"  =  "1"  /* Restrict L Less Saphenous w Extralum Dev, Open */
"06VS0DZ"  =  "1"  /* Restrict L Less Saphenous w Intralum Dev, Open */
"06VS0ZZ"  =  "1"  /* Restriction of Left Lesser Saphenous Vein, Open Approach */
"06VS3CZ"  =  "1"  /* Restrict L Less Saphenous w Extralum Dev, Perc */
"06VS3DZ"  =  "1"  /* Restrict L Less Saphenous w Intralum Dev, Perc */
"06VS3ZZ"  =  "1"  /* Restriction of Left Lesser Saphenous Vein, Perc Approach */
"06VS4CZ"  =  "1"  /* Restrict L Less Saphenous w Extralum Dev, Perc Endo */
"06VS4DZ"  =  "1"  /* Restrict L Less Saphenous w Intralum Dev, Perc Endo */
"06VS4ZZ"  =  "1"  /* Restriction of L Less Saphenous, Perc Endo Approach */
"06VT0CZ"  =  "1"  /* Restriction of R Foot Vein with Extralum Dev, Open Approach */
"06VT0DZ"  =  "1"  /* Restriction of R Foot Vein with Intralum Dev, Open Approach */
"06VT0ZZ"  =  "1"  /* Restriction of Right Foot Vein, Open Approach */
"06VT3CZ"  =  "1"  /* Restriction of R Foot Vein with Extralum Dev, Perc Approach */
"06VT3DZ"  =  "1"  /* Restriction of R Foot Vein with Intralum Dev, Perc Approach */
"06VT3ZZ"  =  "1"  /* Restriction of Right Foot Vein, Percutaneous Approach */
"06VT4CZ"  =  "1"  /* Restrict R Foot Vein w Extralum Dev, Perc Endo */
"06VT4DZ"  =  "1"  /* Restrict R Foot Vein w Intralum Dev, Perc Endo */
"06VT4ZZ"  =  "1"  /* Restriction of Right Foot Vein, Perc Endo Approach */
"06VV0CZ"  =  "1"  /* Restriction of L Foot Vein with Extralum Dev, Open Approach */
"06VV0DZ"  =  "1"  /* Restriction of L Foot Vein with Intralum Dev, Open Approach */
"06VV0ZZ"  =  "1"  /* Restriction of Left Foot Vein, Open Approach */
"06VV3CZ"  =  "1"  /* Restriction of L Foot Vein with Extralum Dev, Perc Approach */
"06VV3DZ"  =  "1"  /* Restriction of L Foot Vein with Intralum Dev, Perc Approach */
"06VV3ZZ"  =  "1"  /* Restriction of Left Foot Vein, Percutaneous Approach */
"06VV4CZ"  =  "1"  /* Restrict L Foot Vein w Extralum Dev, Perc Endo */
"06VV4DZ"  =  "1"  /* Restrict L Foot Vein w Intralum Dev, Perc Endo */
"06VV4ZZ"  =  "1"  /* Restriction of Left Foot Vein, Perc Endo Approach */
"06VY0CZ"  =  "1"  /* Restriction of Lower Vein with Extralum Dev, Open Approach */
"06VY0DZ"  =  "1"  /* Restriction of Lower Vein with Intralum Dev, Open Approach */
"06VY0ZZ"  =  "1"  /* Restriction of Lower Vein, Open Approach */
"06VY3CZ"  =  "1"  /* Restriction of Lower Vein with Extralum Dev, Perc Approach */
"06VY3DZ"  =  "1"  /* Restriction of Lower Vein with Intralum Dev, Perc Approach */
"06VY3ZZ"  =  "1"  /* Restriction of Lower Vein, Percutaneous Approach */
"06VY4CZ"  =  "1"  /* Restrict of Low Vein with Extralum Dev, Perc Endo Approach */
"06VY4DZ"  =  "1"  /* Restrict of Low Vein with Intralum Dev, Perc Endo Approach */
"06VY4ZZ"  =  "1"  /* Restriction of Lower Vein, Percutaneous Endoscopic Approach */
"06WY00Z"  =  "1"  /* Revision of Drainage Device in Lower Vein, Open Approach */
"06WY02Z"  =  "1"  /* Revision of Monitoring Device in Lower Vein, Open Approach */
"06WY03Z"  =  "1"  /* Revision of Infusion Device in Lower Vein, Open Approach */
"06WY07Z"  =  "1"  /* Revision of Autol Sub in Low Vein, Open Approach */
"06WY0CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Vein, Open Approach */
"06WY0DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Vein, Open Approach */
"06WY0JZ"  =  "1"  /* Revision of Synthetic Substitute in Low Vein, Open Approach */
"06WY0KZ"  =  "1"  /* Revision of Nonaut Sub in Low Vein, Open Approach */
"06WY30Z"  =  "1"  /* Revision of Drainage Device in Lower Vein, Perc Approach */
"06WY32Z"  =  "1"  /* Revision of Monitoring Device in Lower Vein, Perc Approach */
"06WY33Z"  =  "1"  /* Revision of Infusion Device in Lower Vein, Perc Approach */
"06WY37Z"  =  "1"  /* Revision of Autol Sub in Low Vein, Perc Approach */
"06WY3CZ"  =  "1"  /* Revision of Extraluminal Device in Lower Vein, Perc Approach */
"06WY3DZ"  =  "1"  /* Revision of Intraluminal Device in Lower Vein, Perc Approach */
"06WY3JZ"  =  "1"  /* Revision of Synthetic Substitute in Low Vein, Perc Approach */
"06WY3KZ"  =  "1"  /* Revision of Nonaut Sub in Low Vein, Perc Approach */
"06WY40Z"  =  "1"  /* Revision of Drainage Device in Low Vein, Perc Endo Approach */
"06WY42Z"  =  "1"  /* Revision of Monitor Dev in Low Vein, Perc Endo Approach */
"06WY43Z"  =  "1"  /* Revision of Infusion Device in Low Vein, Perc Endo Approach */
"06WY47Z"  =  "1"  /* Revision of Autol Sub in Low Vein, Perc Endo Approach */
"06WY4CZ"  =  "1"  /* Revision of Extralum Dev in Low Vein, Perc Endo Approach */
"06WY4DZ"  =  "1"  /* Revision of Intralum Dev in Low Vein, Perc Endo Approach */
"06WY4JZ"  =  "1"  /* Revision of Synth Sub in Low Vein, Perc Endo Approach */
"06WY4KZ"  =  "1"  /* Revision of Nonaut Sub in Low Vein, Perc Endo Approach */
"07500ZZ"  =  "1"  /* Destruction of Head Lymphatic, Open Approach */
"07503ZZ"  =  "1"  /* Destruction of Head Lymphatic, Percutaneous Approach */
"07504ZZ"  =  "1"  /* Destruction of Head Lymphatic, Perc Endo Approach */
"07510ZZ"  =  "1"  /* Destruction of Right Neck Lymphatic, Open Approach */
"07513ZZ"  =  "1"  /* Destruction of Right Neck Lymphatic, Percutaneous Approach */
"07514ZZ"  =  "1"  /* Destruction of Right Neck Lymphatic, Perc Endo Approach */
"07520ZZ"  =  "1"  /* Destruction of Left Neck Lymphatic, Open Approach */
"07523ZZ"  =  "1"  /* Destruction of Left Neck Lymphatic, Percutaneous Approach */
"07524ZZ"  =  "1"  /* Destruction of Left Neck Lymphatic, Perc Endo Approach */
"07530ZZ"  =  "1"  /* Destruction of R Up Extrem Lymph, Open Approach */
"07533ZZ"  =  "1"  /* Destruction of R Up Extrem Lymph, Perc Approach */
"07534ZZ"  =  "1"  /* Destruction of R Up Extrem Lymph, Perc Endo Approach */
"07540ZZ"  =  "1"  /* Destruction of Left Upper Extremity Lymphatic, Open Approach */
"07543ZZ"  =  "1"  /* Destruction of Left Upper Extremity Lymphatic, Perc Approach */
"07544ZZ"  =  "1"  /* Destruction of L Up Extrem Lymph, Perc Endo Approach */
"07550ZZ"  =  "1"  /* Destruction of Right Axillary Lymphatic, Open Approach */
"07553ZZ"  =  "1"  /* Destruction of Right Axillary Lymphatic, Perc Approach */
"07554ZZ"  =  "1"  /* Destruction of Right Axillary Lymphatic, Perc Endo Approach */
"07560ZZ"  =  "1"  /* Destruction of Left Axillary Lymphatic, Open Approach */
"07563ZZ"  =  "1"  /* Destruction of Left Axillary Lymphatic, Perc Approach */
"07564ZZ"  =  "1"  /* Destruction of Left Axillary Lymphatic, Perc Endo Approach */
"07570ZZ"  =  "1"  /* Destruction of Thorax Lymphatic, Open Approach */
"07573ZZ"  =  "1"  /* Destruction of Thorax Lymphatic, Percutaneous Approach */
"07574ZZ"  =  "1"  /* Destruction of Thorax Lymphatic, Perc Endo Approach */
"07580ZZ"  =  "1"  /* Destruction of Int Mamm, R Lymph, Open Approach */
"07583ZZ"  =  "1"  /* Destruction of Int Mamm, R Lymph, Perc Approach */
"07584ZZ"  =  "1"  /* Destruction of Int Mamm, R Lymph, Perc Endo Approach */
"07590ZZ"  =  "1"  /* Destruction of Int Mamm, L Lymph, Open Approach */
"07593ZZ"  =  "1"  /* Destruction of Int Mamm, L Lymph, Perc Approach */
"07594ZZ"  =  "1"  /* Destruction of Int Mamm, L Lymph, Perc Endo Approach */
"075B0ZZ"  =  "1"  /* Destruction of Mesenteric Lymphatic, Open Approach */
"075B3ZZ"  =  "1"  /* Destruction of Mesenteric Lymphatic, Percutaneous Approach */
"075B4ZZ"  =  "1"  /* Destruction of Mesenteric Lymphatic, Perc Endo Approach */
"075C0ZZ"  =  "1"  /* Destruction of Pelvis Lymphatic, Open Approach */
"075C3ZZ"  =  "1"  /* Destruction of Pelvis Lymphatic, Percutaneous Approach */
"075C4ZZ"  =  "1"  /* Destruction of Pelvis Lymphatic, Perc Endo Approach */
"075D0ZZ"  =  "1"  /* Destruction of Aortic Lymphatic, Open Approach */
"075D3ZZ"  =  "1"  /* Destruction of Aortic Lymphatic, Percutaneous Approach */
"075D4ZZ"  =  "1"  /* Destruction of Aortic Lymphatic, Perc Endo Approach */
"075F0ZZ"  =  "1"  /* Destruction of R Low Extrem Lymph, Open Approach */
"075F3ZZ"  =  "1"  /* Destruction of R Low Extrem Lymph, Perc Approach */
"075F4ZZ"  =  "1"  /* Destruction of R Low Extrem Lymph, Perc Endo Approach */
"075G0ZZ"  =  "1"  /* Destruction of Left Lower Extremity Lymphatic, Open Approach */
"075G3ZZ"  =  "1"  /* Destruction of Left Lower Extremity Lymphatic, Perc Approach */
"075G4ZZ"  =  "1"  /* Destruction of L Low Extrem Lymph, Perc Endo Approach */
"075H0ZZ"  =  "1"  /* Destruction of Right Inguinal Lymphatic, Open Approach */
"075H3ZZ"  =  "1"  /* Destruction of Right Inguinal Lymphatic, Perc Approach */
"075H4ZZ"  =  "1"  /* Destruction of Right Inguinal Lymphatic, Perc Endo Approach */
"075J0ZZ"  =  "1"  /* Destruction of Left Inguinal Lymphatic, Open Approach */
"075J3ZZ"  =  "1"  /* Destruction of Left Inguinal Lymphatic, Perc Approach */
"075J4ZZ"  =  "1"  /* Destruction of Left Inguinal Lymphatic, Perc Endo Approach */
"075K0ZZ"  =  "1"  /* Destruction of Thoracic Duct, Open Approach */
"075K3ZZ"  =  "1"  /* Destruction of Thoracic Duct, Percutaneous Approach */
"075K4ZZ"  =  "1"  /* Destruction of Thoracic Duct, Perc Endo Approach */
"075L0ZZ"  =  "1"  /* Destruction of Cisterna Chyli, Open Approach */
"075L3ZZ"  =  "1"  /* Destruction of Cisterna Chyli, Percutaneous Approach */
"075L4ZZ"  =  "1"  /* Destruction of Cisterna Chyli, Perc Endo Approach */
"075M0ZZ"  =  "1"  /* Destruction of Thymus, Open Approach */
"075M3ZZ"  =  "1"  /* Destruction of Thymus, Percutaneous Approach */
"075M4ZZ"  =  "1"  /* Destruction of Thymus, Percutaneous Endoscopic Approach */
"075P0ZZ"  =  "1"  /* Destruction of Spleen, Open Approach */
"075P3ZZ"  =  "1"  /* Destruction of Spleen, Percutaneous Approach */
"075P4ZZ"  =  "1"  /* Destruction of Spleen, Percutaneous Endoscopic Approach */
"079000Z"  =  "1"  /* Drainage of Head Lymphatic with Drain Dev, Open Approach */
"07900ZX"  =  "1"  /* Drainage of Head Lymphatic, Open Approach, Diagnostic */
"07900ZZ"  =  "1"  /* Drainage of Head Lymphatic, Open Approach */
"079030Z"  =  "1"  /* Drainage of Head Lymphatic with Drain Dev, Perc Approach */
"07903ZX"  =  "1"  /* Drainage of Head Lymphatic, Percutaneous Approach, Diagn */
"07903ZZ"  =  "1"  /* Drainage of Head Lymphatic, Percutaneous Approach */
"079040Z"  =  "1"  /* Drainage of Head Lymph with Drain Dev, Perc Endo Approach */
"07904ZX"  =  "1"  /* Drainage of Head Lymphatic, Perc Endo Approach, Diagn */
"07904ZZ"  =  "1"  /* Drainage of Head Lymphatic, Percutaneous Endoscopic Approach */
"079100Z"  =  "1"  /* Drainage of R Neck Lymph with Drain Dev, Open Approach */
"07910ZX"  =  "1"  /* Drainage of Right Neck Lymphatic, Open Approach, Diagnostic */
"07910ZZ"  =  "1"  /* Drainage of Right Neck Lymphatic, Open Approach */
"079130Z"  =  "1"  /* Drainage of R Neck Lymph with Drain Dev, Perc Approach */
"07913ZX"  =  "1"  /* Drainage of Right Neck Lymphatic, Perc Approach, Diagn */
"07913ZZ"  =  "1"  /* Drainage of Right Neck Lymphatic, Percutaneous Approach */
"079140Z"  =  "1"  /* Drainage of R Neck Lymph with Drain Dev, Perc Endo Approach */
"07914ZX"  =  "1"  /* Drainage of Right Neck Lymphatic, Perc Endo Approach, Diagn */
"07914ZZ"  =  "1"  /* Drainage of Right Neck Lymphatic, Perc Endo Approach */
"079200Z"  =  "1"  /* Drainage of L Neck Lymph with Drain Dev, Open Approach */
"07920ZX"  =  "1"  /* Drainage of Left Neck Lymphatic, Open Approach, Diagnostic */
"07920ZZ"  =  "1"  /* Drainage of Left Neck Lymphatic, Open Approach */
"079230Z"  =  "1"  /* Drainage of L Neck Lymph with Drain Dev, Perc Approach */
"07923ZX"  =  "1"  /* Drainage of Left Neck Lymphatic, Perc Approach, Diagn */
"07923ZZ"  =  "1"  /* Drainage of Left Neck Lymphatic, Percutaneous Approach */
"079240Z"  =  "1"  /* Drainage of L Neck Lymph with Drain Dev, Perc Endo Approach */
"07924ZX"  =  "1"  /* Drainage of Left Neck Lymphatic, Perc Endo Approach, Diagn */
"07924ZZ"  =  "1"  /* Drainage of Left Neck Lymphatic, Perc Endo Approach */
"079300Z"  =  "1"  /* Drainage of R Up Extrem Lymph with Drain Dev, Open Approach */
"07930ZX"  =  "1"  /* Drainage of R Up Extrem Lymph, Open Approach, Diagn */
"07930ZZ"  =  "1"  /* Drainage of Right Upper Extremity Lymphatic, Open Approach */
"079330Z"  =  "1"  /* Drainage of R Up Extrem Lymph with Drain Dev, Perc Approach */
"07933ZX"  =  "1"  /* Drainage of R Up Extrem Lymph, Perc Approach, Diagn */
"07933ZZ"  =  "1"  /* Drainage of Right Upper Extremity Lymphatic, Perc Approach */
"079340Z"  =  "1"  /* Drain R Up Extrem Lymph w Drain Dev, Perc Endo */
"07934ZX"  =  "1"  /* Drainage of R Up Extrem Lymph, Perc Endo Approach, Diagn */
"07934ZZ"  =  "1"  /* Drainage of R Up Extrem Lymph, Perc Endo Approach */
"079400Z"  =  "1"  /* Drainage of L Up Extrem Lymph with Drain Dev, Open Approach */
"07940ZX"  =  "1"  /* Drainage of L Up Extrem Lymph, Open Approach, Diagn */
"07940ZZ"  =  "1"  /* Drainage of Left Upper Extremity Lymphatic, Open Approach */
"079430Z"  =  "1"  /* Drainage of L Up Extrem Lymph with Drain Dev, Perc Approach */
"07943ZX"  =  "1"  /* Drainage of L Up Extrem Lymph, Perc Approach, Diagn */
"07943ZZ"  =  "1"  /* Drainage of Left Upper Extremity Lymphatic, Perc Approach */
"079440Z"  =  "1"  /* Drain L Up Extrem Lymph w Drain Dev, Perc Endo */
"07944ZX"  =  "1"  /* Drainage of L Up Extrem Lymph, Perc Endo Approach, Diagn */
"07944ZZ"  =  "1"  /* Drainage of L Up Extrem Lymph, Perc Endo Approach */
"079500Z"  =  "1"  /* Drainage of R Axilla Lymph with Drain Dev, Open Approach */
"07950ZX"  =  "1"  /* Drainage of Right Axillary Lymphatic, Open Approach, Diagn */
"07950ZZ"  =  "1"  /* Drainage of Right Axillary Lymphatic, Open Approach */
"079530Z"  =  "1"  /* Drainage of R Axilla Lymph with Drain Dev, Perc Approach */
"07953ZX"  =  "1"  /* Drainage of Right Axillary Lymphatic, Perc Approach, Diagn */
"07953ZZ"  =  "1"  /* Drainage of Right Axillary Lymphatic, Percutaneous Approach */
"079540Z"  =  "1"  /* Drain of R Axilla Lymph with Drain Dev, Perc Endo Approach */
"07954ZX"  =  "1"  /* Drainage of R Axilla Lymph, Perc Endo Approach, Diagn */
"07954ZZ"  =  "1"  /* Drainage of Right Axillary Lymphatic, Perc Endo Approach */
"079600Z"  =  "1"  /* Drainage of L Axilla Lymph with Drain Dev, Open Approach */
"07960ZX"  =  "1"  /* Drainage of Left Axillary Lymphatic, Open Approach, Diagn */
"07960ZZ"  =  "1"  /* Drainage of Left Axillary Lymphatic, Open Approach */
"079630Z"  =  "1"  /* Drainage of L Axilla Lymph with Drain Dev, Perc Approach */
"07963ZX"  =  "1"  /* Drainage of Left Axillary Lymphatic, Perc Approach, Diagn */
"07963ZZ"  =  "1"  /* Drainage of Left Axillary Lymphatic, Percutaneous Approach */
"079640Z"  =  "1"  /* Drain of L Axilla Lymph with Drain Dev, Perc Endo Approach */
"07964ZX"  =  "1"  /* Drainage of L Axilla Lymph, Perc Endo Approach, Diagn */
"07964ZZ"  =  "1"  /* Drainage of Left Axillary Lymphatic, Perc Endo Approach */
"079700Z"  =  "1"  /* Drainage of Thorax Lymphatic with Drain Dev, Open Approach */
"07970ZX"  =  "1"  /* Drainage of Thorax Lymphatic, Open Approach, Diagnostic */
"07970ZZ"  =  "1"  /* Drainage of Thorax Lymphatic, Open Approach */
"079730Z"  =  "1"  /* Drainage of Thorax Lymphatic with Drain Dev, Perc Approach */
"07973ZX"  =  "1"  /* Drainage of Thorax Lymphatic, Percutaneous Approach, Diagn */
"07973ZZ"  =  "1"  /* Drainage of Thorax Lymphatic, Percutaneous Approach */
"079740Z"  =  "1"  /* Drainage of Thorax Lymph with Drain Dev, Perc Endo Approach */
"07974ZX"  =  "1"  /* Drainage of Thorax Lymphatic, Perc Endo Approach, Diagn */
"07974ZZ"  =  "1"  /* Drainage of Thorax Lymphatic, Perc Endo Approach */
"079800Z"  =  "1"  /* Drainage of Int Mamm, R Lymph with Drain Dev, Open Approach */
"07980ZX"  =  "1"  /* Drainage of Int Mamm, R Lymph, Open Approach, Diagn */
"07980ZZ"  =  "1"  /* Drainage of Right Internal Mammary Lymphatic, Open Approach */
"079830Z"  =  "1"  /* Drainage of Int Mamm, R Lymph with Drain Dev, Perc Approach */
"07983ZX"  =  "1"  /* Drainage of Int Mamm, R Lymph, Perc Approach, Diagn */
"07983ZZ"  =  "1"  /* Drainage of Right Internal Mammary Lymphatic, Perc Approach */
"079840Z"  =  "1"  /* Drain Int Mamm, R Lymph w Drain Dev, Perc Endo */
"07984ZX"  =  "1"  /* Drainage of Int Mamm, R Lymph, Perc Endo Approach, Diagn */
"07984ZZ"  =  "1"  /* Drainage of Int Mamm, R Lymph, Perc Endo Approach */
"079900Z"  =  "1"  /* Drainage of Int Mamm, L Lymph with Drain Dev, Open Approach */
"07990ZX"  =  "1"  /* Drainage of Int Mamm, L Lymph, Open Approach, Diagn */
"07990ZZ"  =  "1"  /* Drainage of Left Internal Mammary Lymphatic, Open Approach */
"079930Z"  =  "1"  /* Drainage of Int Mamm, L Lymph with Drain Dev, Perc Approach */
"07993ZX"  =  "1"  /* Drainage of Int Mamm, L Lymph, Perc Approach, Diagn */
"07993ZZ"  =  "1"  /* Drainage of Left Internal Mammary Lymphatic, Perc Approach */
"079940Z"  =  "1"  /* Drain Int Mamm, L Lymph w Drain Dev, Perc Endo */
"07994ZX"  =  "1"  /* Drainage of Int Mamm, L Lymph, Perc Endo Approach, Diagn */
"07994ZZ"  =  "1"  /* Drainage of Int Mamm, L Lymph, Perc Endo Approach */
"079B00Z"  =  "1"  /* Drainage of Mesenteric Lymph with Drain Dev, Open Approach */
"079B0ZX"  =  "1"  /* Drainage of Mesenteric Lymphatic, Open Approach, Diagnostic */
"079B0ZZ"  =  "1"  /* Drainage of Mesenteric Lymphatic, Open Approach */
"079B30Z"  =  "1"  /* Drainage of Mesenteric Lymph with Drain Dev, Perc Approach */
"079B3ZX"  =  "1"  /* Drainage of Mesenteric Lymphatic, Perc Approach, Diagn */
"079B3ZZ"  =  "1"  /* Drainage of Mesenteric Lymphatic, Percutaneous Approach */
"079B40Z"  =  "1"  /* Drain of Mesenteric Lymph with Drain Dev, Perc Endo Approach */
"079B4ZX"  =  "1"  /* Drainage of Mesenteric Lymphatic, Perc Endo Approach, Diagn */
"079B4ZZ"  =  "1"  /* Drainage of Mesenteric Lymphatic, Perc Endo Approach */
"079C00Z"  =  "1"  /* Drainage of Pelvis Lymphatic with Drain Dev, Open Approach */
"079C0ZX"  =  "1"  /* Drainage of Pelvis Lymphatic, Open Approach, Diagnostic */
"079C0ZZ"  =  "1"  /* Drainage of Pelvis Lymphatic, Open Approach */
"079C30Z"  =  "1"  /* Drainage of Pelvis Lymphatic with Drain Dev, Perc Approach */
"079C3ZX"  =  "1"  /* Drainage of Pelvis Lymphatic, Percutaneous Approach, Diagn */
"079C3ZZ"  =  "1"  /* Drainage of Pelvis Lymphatic, Percutaneous Approach */
"079C40Z"  =  "1"  /* Drainage of Pelvis Lymph with Drain Dev, Perc Endo Approach */
"079C4ZX"  =  "1"  /* Drainage of Pelvis Lymphatic, Perc Endo Approach, Diagn */
"079C4ZZ"  =  "1"  /* Drainage of Pelvis Lymphatic, Perc Endo Approach */
"079D00Z"  =  "1"  /* Drainage of Aortic Lymphatic with Drain Dev, Open Approach */
"079D0ZX"  =  "1"  /* Drainage of Aortic Lymphatic, Open Approach, Diagnostic */
"079D0ZZ"  =  "1"  /* Drainage of Aortic Lymphatic, Open Approach */
"079D30Z"  =  "1"  /* Drainage of Aortic Lymphatic with Drain Dev, Perc Approach */
"079D3ZX"  =  "1"  /* Drainage of Aortic Lymphatic, Percutaneous Approach, Diagn */
"079D3ZZ"  =  "1"  /* Drainage of Aortic Lymphatic, Percutaneous Approach */
"079D40Z"  =  "1"  /* Drainage of Aortic Lymph with Drain Dev, Perc Endo Approach */
"079D4ZX"  =  "1"  /* Drainage of Aortic Lymphatic, Perc Endo Approach, Diagn */
"079D4ZZ"  =  "1"  /* Drainage of Aortic Lymphatic, Perc Endo Approach */
"079F00Z"  =  "1"  /* Drainage of R Low Extrem Lymph with Drain Dev, Open Approach */
"079F0ZX"  =  "1"  /* Drainage of R Low Extrem Lymph, Open Approach, Diagn */
"079F0ZZ"  =  "1"  /* Drainage of Right Lower Extremity Lymphatic, Open Approach */
"079F30Z"  =  "1"  /* Drainage of R Low Extrem Lymph with Drain Dev, Perc Approach */
"079F3ZX"  =  "1"  /* Drainage of R Low Extrem Lymph, Perc Approach, Diagn */
"079F3ZZ"  =  "1"  /* Drainage of Right Lower Extremity Lymphatic, Perc Approach */
"079F40Z"  =  "1"  /* Drain R Low Extrem Lymph w Drain Dev, Perc Endo */
"079F4ZX"  =  "1"  /* Drainage of R Low Extrem Lymph, Perc Endo Approach, Diagn */
"079F4ZZ"  =  "1"  /* Drainage of R Low Extrem Lymph, Perc Endo Approach */
"079G00Z"  =  "1"  /* Drainage of L Low Extrem Lymph with Drain Dev, Open Approach */
"079G0ZX"  =  "1"  /* Drainage of L Low Extrem Lymph, Open Approach, Diagn */
"079G0ZZ"  =  "1"  /* Drainage of Left Lower Extremity Lymphatic, Open Approach */
"079G30Z"  =  "1"  /* Drainage of L Low Extrem Lymph with Drain Dev, Perc Approach */
"079G3ZX"  =  "1"  /* Drainage of L Low Extrem Lymph, Perc Approach, Diagn */
"079G3ZZ"  =  "1"  /* Drainage of Left Lower Extremity Lymphatic, Perc Approach */
"079G40Z"  =  "1"  /* Drain L Low Extrem Lymph w Drain Dev, Perc Endo */
"079G4ZX"  =  "1"  /* Drainage of L Low Extrem Lymph, Perc Endo Approach, Diagn */
"079G4ZZ"  =  "1"  /* Drainage of L Low Extrem Lymph, Perc Endo Approach */
"079H00Z"  =  "1"  /* Drainage of R Inqnl Lymph with Drain Dev, Open Approach */
"079H0ZX"  =  "1"  /* Drainage of Right Inguinal Lymphatic, Open Approach, Diagn */
"079H0ZZ"  =  "1"  /* Drainage of Right Inguinal Lymphatic, Open Approach */
"079H30Z"  =  "1"  /* Drainage of R Inqnl Lymph with Drain Dev, Perc Approach */
"079H3ZX"  =  "1"  /* Drainage of Right Inguinal Lymphatic, Perc Approach, Diagn */
"079H3ZZ"  =  "1"  /* Drainage of Right Inguinal Lymphatic, Percutaneous Approach */
"079H40Z"  =  "1"  /* Drainage of R Inqnl Lymph with Drain Dev, Perc Endo Approach */
"079H4ZX"  =  "1"  /* Drainage of R Inqnl Lymph, Perc Endo Approach, Diagn */
"079H4ZZ"  =  "1"  /* Drainage of Right Inguinal Lymphatic, Perc Endo Approach */
"079J00Z"  =  "1"  /* Drainage of L Inqnl Lymph with Drain Dev, Open Approach */
"079J0ZX"  =  "1"  /* Drainage of Left Inguinal Lymphatic, Open Approach, Diagn */
"079J0ZZ"  =  "1"  /* Drainage of Left Inguinal Lymphatic, Open Approach */
"079J30Z"  =  "1"  /* Drainage of L Inqnl Lymph with Drain Dev, Perc Approach */
"079J3ZX"  =  "1"  /* Drainage of Left Inguinal Lymphatic, Perc Approach, Diagn */
"079J3ZZ"  =  "1"  /* Drainage of Left Inguinal Lymphatic, Percutaneous Approach */
"079J40Z"  =  "1"  /* Drainage of L Inqnl Lymph with Drain Dev, Perc Endo Approach */
"079J4ZX"  =  "1"  /* Drainage of L Inqnl Lymph, Perc Endo Approach, Diagn */
"079J4ZZ"  =  "1"  /* Drainage of Left Inguinal Lymphatic, Perc Endo Approach */
"079K00Z"  =  "1"  /* Drainage of Thoracic Duct with Drain Dev, Open Approach */
"079K0ZX"  =  "1"  /* Drainage of Thoracic Duct, Open Approach, Diagnostic */
"079K0ZZ"  =  "1"  /* Drainage of Thoracic Duct, Open Approach */
"079K30Z"  =  "1"  /* Drainage of Thoracic Duct with Drain Dev, Perc Approach */
"079K3ZX"  =  "1"  /* Drainage of Thoracic Duct, Percutaneous Approach, Diagnostic */
"079K3ZZ"  =  "1"  /* Drainage of Thoracic Duct, Percutaneous Approach */
"079K40Z"  =  "1"  /* Drainage of Thoracic Duct with Drain Dev, Perc Endo Approach */
"079K4ZX"  =  "1"  /* Drainage of Thoracic Duct, Perc Endo Approach, Diagn */
"079K4ZZ"  =  "1"  /* Drainage of Thoracic Duct, Percutaneous Endoscopic Approach */
"079L00Z"  =  "1"  /* Drainage of Cisterna Chyli with Drain Dev, Open Approach */
"079L0ZX"  =  "1"  /* Drainage of Cisterna Chyli, Open Approach, Diagnostic */
"079L0ZZ"  =  "1"  /* Drainage of Cisterna Chyli, Open Approach */
"079L30Z"  =  "1"  /* Drainage of Cisterna Chyli with Drain Dev, Perc Approach */
"079L3ZX"  =  "1"  /* Drainage of Cisterna Chyli, Percutaneous Approach, Diagn */
"079L3ZZ"  =  "1"  /* Drainage of Cisterna Chyli, Percutaneous Approach */
"079L40Z"  =  "1"  /* Drain of Cisterna Chyli with Drain Dev, Perc Endo Approach */
"079L4ZX"  =  "1"  /* Drainage of Cisterna Chyli, Perc Endo Approach, Diagn */
"079L4ZZ"  =  "1"  /* Drainage of Cisterna Chyli, Percutaneous Endoscopic Approach */
"079M00Z"  =  "1"  /* Drainage of Thymus with Drainage Device, Open Approach */
"079M0ZX"  =  "1"  /* Drainage of Thymus, Open Approach, Diagnostic */
"079M0ZZ"  =  "1"  /* Drainage of Thymus, Open Approach */
"079M30Z"  =  "1"  /* Drainage of Thymus with Drainage Device, Perc Approach */
"079M3ZX"  =  "1"  /* Drainage of Thymus, Percutaneous Approach, Diagnostic */
"079M3ZZ"  =  "1"  /* Drainage of Thymus, Percutaneous Approach */
"079M40Z"  =  "1"  /* Drainage of Thymus with Drainage Device, Perc Endo Approach */
"079M4ZX"  =  "1"  /* Drainage of Thymus, Percutaneous Endoscopic Approach, Diagn */
"079M4ZZ"  =  "1"  /* Drainage of Thymus, Percutaneous Endoscopic Approach */
"079P00Z"  =  "1"  /* Drainage of Spleen with Drainage Device, Open Approach */
"079P0ZX"  =  "1"  /* Drainage of Spleen, Open Approach, Diagnostic */
"079P0ZZ"  =  "1"  /* Drainage of Spleen, Open Approach */
"07B00ZX"  =  "1"  /* Excision of Head Lymphatic, Open Approach, Diagnostic */
"07B00ZZ"  =  "1"  /* Excision of Head Lymphatic, Open Approach */
"07B03ZX"  =  "1"  /* Excision of Head Lymphatic, Percutaneous Approach, Diagn */
"07B03ZZ"  =  "1"  /* Excision of Head Lymphatic, Percutaneous Approach */
"07B04ZX"  =  "1"  /* Excision of Head Lymphatic, Perc Endo Approach, Diagn */
"07B04ZZ"  =  "1"  /* Excision of Head Lymphatic, Percutaneous Endoscopic Approach */
"07B10ZX"  =  "1"  /* Excision of Right Neck Lymphatic, Open Approach, Diagnostic */
"07B10ZZ"  =  "1"  /* Excision of Right Neck Lymphatic, Open Approach */
"07B13ZX"  =  "1"  /* Excision of Right Neck Lymphatic, Perc Approach, Diagn */
"07B13ZZ"  =  "1"  /* Excision of Right Neck Lymphatic, Percutaneous Approach */
"07B14ZX"  =  "1"  /* Excision of Right Neck Lymphatic, Perc Endo Approach, Diagn */
"07B14ZZ"  =  "1"  /* Excision of Right Neck Lymphatic, Perc Endo Approach */
"07B20ZX"  =  "1"  /* Excision of Left Neck Lymphatic, Open Approach, Diagnostic */
"07B20ZZ"  =  "1"  /* Excision of Left Neck Lymphatic, Open Approach */
"07B23ZX"  =  "1"  /* Excision of Left Neck Lymphatic, Perc Approach, Diagn */
"07B23ZZ"  =  "1"  /* Excision of Left Neck Lymphatic, Percutaneous Approach */
"07B24ZX"  =  "1"  /* Excision of Left Neck Lymphatic, Perc Endo Approach, Diagn */
"07B24ZZ"  =  "1"  /* Excision of Left Neck Lymphatic, Perc Endo Approach */
"07B30ZX"  =  "1"  /* Excision of R Up Extrem Lymph, Open Approach, Diagn */
"07B30ZZ"  =  "1"  /* Excision of Right Upper Extremity Lymphatic, Open Approach */
"07B33ZX"  =  "1"  /* Excision of R Up Extrem Lymph, Perc Approach, Diagn */
"07B33ZZ"  =  "1"  /* Excision of Right Upper Extremity Lymphatic, Perc Approach */
"07B34ZX"  =  "1"  /* Excision of R Up Extrem Lymph, Perc Endo Approach, Diagn */
"07B34ZZ"  =  "1"  /* Excision of R Up Extrem Lymph, Perc Endo Approach */
"07B40ZX"  =  "1"  /* Excision of L Up Extrem Lymph, Open Approach, Diagn */
"07B40ZZ"  =  "1"  /* Excision of Left Upper Extremity Lymphatic, Open Approach */
"07B43ZX"  =  "1"  /* Excision of L Up Extrem Lymph, Perc Approach, Diagn */
"07B43ZZ"  =  "1"  /* Excision of Left Upper Extremity Lymphatic, Perc Approach */
"07B44ZX"  =  "1"  /* Excision of L Up Extrem Lymph, Perc Endo Approach, Diagn */
"07B44ZZ"  =  "1"  /* Excision of L Up Extrem Lymph, Perc Endo Approach */
"07B50ZX"  =  "1"  /* Excision of Right Axillary Lymphatic, Open Approach, Diagn */
"07B50ZZ"  =  "1"  /* Excision of Right Axillary Lymphatic, Open Approach */
"07B53ZX"  =  "1"  /* Excision of Right Axillary Lymphatic, Perc Approach, Diagn */
"07B53ZZ"  =  "1"  /* Excision of Right Axillary Lymphatic, Percutaneous Approach */
"07B54ZX"  =  "1"  /* Excision of R Axilla Lymph, Perc Endo Approach, Diagn */
"07B54ZZ"  =  "1"  /* Excision of Right Axillary Lymphatic, Perc Endo Approach */
"07B60ZX"  =  "1"  /* Excision of Left Axillary Lymphatic, Open Approach, Diagn */
"07B60ZZ"  =  "1"  /* Excision of Left Axillary Lymphatic, Open Approach */
"07B63ZX"  =  "1"  /* Excision of Left Axillary Lymphatic, Perc Approach, Diagn */
"07B63ZZ"  =  "1"  /* Excision of Left Axillary Lymphatic, Percutaneous Approach */
"07B64ZX"  =  "1"  /* Excision of L Axilla Lymph, Perc Endo Approach, Diagn */
"07B64ZZ"  =  "1"  /* Excision of Left Axillary Lymphatic, Perc Endo Approach */
"07B70ZX"  =  "1"  /* Excision of Thorax Lymphatic, Open Approach, Diagnostic */
"07B70ZZ"  =  "1"  /* Excision of Thorax Lymphatic, Open Approach */
"07B73ZX"  =  "1"  /* Excision of Thorax Lymphatic, Percutaneous Approach, Diagn */
"07B73ZZ"  =  "1"  /* Excision of Thorax Lymphatic, Percutaneous Approach */
"07B74ZX"  =  "1"  /* Excision of Thorax Lymphatic, Perc Endo Approach, Diagn */
"07B74ZZ"  =  "1"  /* Excision of Thorax Lymphatic, Perc Endo Approach */
"07B80ZX"  =  "1"  /* Excision of Int Mamm, R Lymph, Open Approach, Diagn */
"07B80ZZ"  =  "1"  /* Excision of Right Internal Mammary Lymphatic, Open Approach */
"07B83ZX"  =  "1"  /* Excision of Int Mamm, R Lymph, Perc Approach, Diagn */
"07B83ZZ"  =  "1"  /* Excision of Right Internal Mammary Lymphatic, Perc Approach */
"07B84ZX"  =  "1"  /* Excision of Int Mamm, R Lymph, Perc Endo Approach, Diagn */
"07B84ZZ"  =  "1"  /* Excision of Int Mamm, R Lymph, Perc Endo Approach */
"07B90ZX"  =  "1"  /* Excision of Int Mamm, L Lymph, Open Approach, Diagn */
"07B90ZZ"  =  "1"  /* Excision of Left Internal Mammary Lymphatic, Open Approach */
"07B93ZX"  =  "1"  /* Excision of Int Mamm, L Lymph, Perc Approach, Diagn */
"07B93ZZ"  =  "1"  /* Excision of Left Internal Mammary Lymphatic, Perc Approach */
"07B94ZX"  =  "1"  /* Excision of Int Mamm, L Lymph, Perc Endo Approach, Diagn */
"07B94ZZ"  =  "1"  /* Excision of Int Mamm, L Lymph, Perc Endo Approach */
"07BB0ZX"  =  "1"  /* Excision of Mesenteric Lymphatic, Open Approach, Diagnostic */
"07BB0ZZ"  =  "1"  /* Excision of Mesenteric Lymphatic, Open Approach */
"07BB3ZX"  =  "1"  /* Excision of Mesenteric Lymphatic, Perc Approach, Diagn */
"07BB3ZZ"  =  "1"  /* Excision of Mesenteric Lymphatic, Percutaneous Approach */
"07BB4ZX"  =  "1"  /* Excision of Mesenteric Lymphatic, Perc Endo Approach, Diagn */
"07BB4ZZ"  =  "1"  /* Excision of Mesenteric Lymphatic, Perc Endo Approach */
"07BC0ZX"  =  "1"  /* Excision of Pelvis Lymphatic, Open Approach, Diagnostic */
"07BC0ZZ"  =  "1"  /* Excision of Pelvis Lymphatic, Open Approach */
"07BC3ZX"  =  "1"  /* Excision of Pelvis Lymphatic, Percutaneous Approach, Diagn */
"07BC3ZZ"  =  "1"  /* Excision of Pelvis Lymphatic, Percutaneous Approach */
"07BC4ZX"  =  "1"  /* Excision of Pelvis Lymphatic, Perc Endo Approach, Diagn */
"07BC4ZZ"  =  "1"  /* Excision of Pelvis Lymphatic, Perc Endo Approach */
"07BD0ZX"  =  "1"  /* Excision of Aortic Lymphatic, Open Approach, Diagnostic */
"07BD0ZZ"  =  "1"  /* Excision of Aortic Lymphatic, Open Approach */
"07BD3ZX"  =  "1"  /* Excision of Aortic Lymphatic, Percutaneous Approach, Diagn */
"07BD3ZZ"  =  "1"  /* Excision of Aortic Lymphatic, Percutaneous Approach */
"07BD4ZX"  =  "1"  /* Excision of Aortic Lymphatic, Perc Endo Approach, Diagn */
"07BD4ZZ"  =  "1"  /* Excision of Aortic Lymphatic, Perc Endo Approach */
"07BF0ZX"  =  "1"  /* Excision of R Low Extrem Lymph, Open Approach, Diagn */
"07BF0ZZ"  =  "1"  /* Excision of Right Lower Extremity Lymphatic, Open Approach */
"07BF3ZX"  =  "1"  /* Excision of R Low Extrem Lymph, Perc Approach, Diagn */
"07BF3ZZ"  =  "1"  /* Excision of Right Lower Extremity Lymphatic, Perc Approach */
"07BF4ZX"  =  "1"  /* Excision of R Low Extrem Lymph, Perc Endo Approach, Diagn */
"07BF4ZZ"  =  "1"  /* Excision of R Low Extrem Lymph, Perc Endo Approach */
"07BG0ZX"  =  "1"  /* Excision of L Low Extrem Lymph, Open Approach, Diagn */
"07BG0ZZ"  =  "1"  /* Excision of Left Lower Extremity Lymphatic, Open Approach */
"07BG3ZX"  =  "1"  /* Excision of L Low Extrem Lymph, Perc Approach, Diagn */
"07BG3ZZ"  =  "1"  /* Excision of Left Lower Extremity Lymphatic, Perc Approach */
"07BG4ZX"  =  "1"  /* Excision of L Low Extrem Lymph, Perc Endo Approach, Diagn */
"07BG4ZZ"  =  "1"  /* Excision of L Low Extrem Lymph, Perc Endo Approach */
"07BH0ZX"  =  "1"  /* Excision of Right Inguinal Lymphatic, Open Approach, Diagn */
"07BH0ZZ"  =  "1"  /* Excision of Right Inguinal Lymphatic, Open Approach */
"07BH3ZX"  =  "1"  /* Excision of Right Inguinal Lymphatic, Perc Approach, Diagn */
"07BH3ZZ"  =  "1"  /* Excision of Right Inguinal Lymphatic, Percutaneous Approach */
"07BH4ZX"  =  "1"  /* Excision of R Inqnl Lymph, Perc Endo Approach, Diagn */
"07BH4ZZ"  =  "1"  /* Excision of Right Inguinal Lymphatic, Perc Endo Approach */
"07BJ0ZX"  =  "1"  /* Excision of Left Inguinal Lymphatic, Open Approach, Diagn */
"07BJ0ZZ"  =  "1"  /* Excision of Left Inguinal Lymphatic, Open Approach */
"07BJ3ZX"  =  "1"  /* Excision of Left Inguinal Lymphatic, Perc Approach, Diagn */
"07BJ3ZZ"  =  "1"  /* Excision of Left Inguinal Lymphatic, Percutaneous Approach */
"07BJ4ZX"  =  "1"  /* Excision of L Inqnl Lymph, Perc Endo Approach, Diagn */
"07BJ4ZZ"  =  "1"  /* Excision of Left Inguinal Lymphatic, Perc Endo Approach */
"07BK0ZX"  =  "1"  /* Excision of Thoracic Duct, Open Approach, Diagnostic */
"07BK0ZZ"  =  "1"  /* Excision of Thoracic Duct, Open Approach */
"07BK3ZX"  =  "1"  /* Excision of Thoracic Duct, Percutaneous Approach, Diagnostic */
"07BK3ZZ"  =  "1"  /* Excision of Thoracic Duct, Percutaneous Approach */
"07BK4ZX"  =  "1"  /* Excision of Thoracic Duct, Perc Endo Approach, Diagn */
"07BK4ZZ"  =  "1"  /* Excision of Thoracic Duct, Percutaneous Endoscopic Approach */
"07BL0ZX"  =  "1"  /* Excision of Cisterna Chyli, Open Approach, Diagnostic */
"07BL0ZZ"  =  "1"  /* Excision of Cisterna Chyli, Open Approach */
"07BL3ZX"  =  "1"  /* Excision of Cisterna Chyli, Percutaneous Approach, Diagn */
"07BL3ZZ"  =  "1"  /* Excision of Cisterna Chyli, Percutaneous Approach */
"07BL4ZX"  =  "1"  /* Excision of Cisterna Chyli, Perc Endo Approach, Diagn */
"07BL4ZZ"  =  "1"  /* Excision of Cisterna Chyli, Percutaneous Endoscopic Approach */
"07BM0ZX"  =  "1"  /* Excision of Thymus, Open Approach, Diagnostic */
"07BM0ZZ"  =  "1"  /* Excision of Thymus, Open Approach */
"07BM3ZX"  =  "1"  /* Excision of Thymus, Percutaneous Approach, Diagnostic */
"07BM3ZZ"  =  "1"  /* Excision of Thymus, Percutaneous Approach */
"07BM4ZX"  =  "1"  /* Excision of Thymus, Percutaneous Endoscopic Approach, Diagn */
"07BM4ZZ"  =  "1"  /* Excision of Thymus, Percutaneous Endoscopic Approach */
"07BP0ZX"  =  "1"  /* Excision of Spleen, Open Approach, Diagnostic */
"07BP0ZZ"  =  "1"  /* Excision of Spleen, Open Approach */
"07BP3ZZ"  =  "1"  /* Excision of Spleen, Percutaneous Approach */
"07BP4ZZ"  =  "1"  /* Excision of Spleen, Percutaneous Endoscopic Approach */
"07C00ZZ"  =  "1"  /* Extirpation of Matter from Head Lymphatic, Open Approach */
"07C03ZZ"  =  "1"  /* Extirpation of Matter from Head Lymphatic, Perc Approach */
"07C04ZZ"  =  "1"  /* Extirpation of Matter from Head Lymph, Perc Endo Approach */
"07C10ZZ"  =  "1"  /* Extirpation of Matter from R Neck Lymph, Open Approach */
"07C13ZZ"  =  "1"  /* Extirpation of Matter from R Neck Lymph, Perc Approach */
"07C14ZZ"  =  "1"  /* Extirpation of Matter from R Neck Lymph, Perc Endo Approach */
"07C20ZZ"  =  "1"  /* Extirpation of Matter from L Neck Lymph, Open Approach */
"07C23ZZ"  =  "1"  /* Extirpation of Matter from L Neck Lymph, Perc Approach */
"07C24ZZ"  =  "1"  /* Extirpation of Matter from L Neck Lymph, Perc Endo Approach */
"07C30ZZ"  =  "1"  /* Extirpation of Matter from R Up Extrem Lymph, Open Approach */
"07C33ZZ"  =  "1"  /* Extirpation of Matter from R Up Extrem Lymph, Perc Approach */
"07C34ZZ"  =  "1"  /* Extirpate matter from R Up Extrem Lymph, Perc Endo */
"07C40ZZ"  =  "1"  /* Extirpation of Matter from L Up Extrem Lymph, Open Approach */
"07C43ZZ"  =  "1"  /* Extirpation of Matter from L Up Extrem Lymph, Perc Approach */
"07C44ZZ"  =  "1"  /* Extirpate matter from L Up Extrem Lymph, Perc Endo */
"07C50ZZ"  =  "1"  /* Extirpation of Matter from R Axilla Lymph, Open Approach */
"07C53ZZ"  =  "1"  /* Extirpation of Matter from R Axilla Lymph, Perc Approach */
"07C54ZZ"  =  "1"  /* Extirpate of Matter from R Axilla Lymph, Perc Endo Approach */
"07C60ZZ"  =  "1"  /* Extirpation of Matter from L Axilla Lymph, Open Approach */
"07C63ZZ"  =  "1"  /* Extirpation of Matter from L Axilla Lymph, Perc Approach */
"07C64ZZ"  =  "1"  /* Extirpate of Matter from L Axilla Lymph, Perc Endo Approach */
"07C70ZZ"  =  "1"  /* Extirpation of Matter from Thorax Lymphatic, Open Approach */
"07C73ZZ"  =  "1"  /* Extirpation of Matter from Thorax Lymphatic, Perc Approach */
"07C74ZZ"  =  "1"  /* Extirpation of Matter from Thorax Lymph, Perc Endo Approach */
"07C80ZZ"  =  "1"  /* Extirpation of Matter from Int Mamm, R Lymph, Open Approach */
"07C83ZZ"  =  "1"  /* Extirpation of Matter from Int Mamm, R Lymph, Perc Approach */
"07C84ZZ"  =  "1"  /* Extirpate matter from Int Mamm, R Lymph, Perc Endo */
"07C90ZZ"  =  "1"  /* Extirpation of Matter from Int Mamm, L Lymph, Open Approach */
"07C93ZZ"  =  "1"  /* Extirpation of Matter from Int Mamm, L Lymph, Perc Approach */
"07C94ZZ"  =  "1"  /* Extirpate matter from Int Mamm, L Lymph, Perc Endo */
"07CB0ZZ"  =  "1"  /* Extirpation of Matter from Mesenteric Lymph, Open Approach */
"07CB3ZZ"  =  "1"  /* Extirpation of Matter from Mesenteric Lymph, Perc Approach */
"07CB4ZZ"  =  "1"  /* Extirpate matter from Mesenteric Lymph, Perc Endo */
"07CC0ZZ"  =  "1"  /* Extirpation of Matter from Pelvis Lymphatic, Open Approach */
"07CC3ZZ"  =  "1"  /* Extirpation of Matter from Pelvis Lymphatic, Perc Approach */
"07CC4ZZ"  =  "1"  /* Extirpation of Matter from Pelvis Lymph, Perc Endo Approach */
"07CD0ZZ"  =  "1"  /* Extirpation of Matter from Aortic Lymphatic, Open Approach */
"07CD3ZZ"  =  "1"  /* Extirpation of Matter from Aortic Lymphatic, Perc Approach */
"07CD4ZZ"  =  "1"  /* Extirpation of Matter from Aortic Lymph, Perc Endo Approach */
"07CF0ZZ"  =  "1"  /* Extirpation of Matter from R Low Extrem Lymph, Open Approach */
"07CF3ZZ"  =  "1"  /* Extirpation of Matter from R Low Extrem Lymph, Perc Approach */
"07CF4ZZ"  =  "1"  /* Extirpate matter from R Low Extrem Lymph, Perc Endo */
"07CG0ZZ"  =  "1"  /* Extirpation of Matter from L Low Extrem Lymph, Open Approach */
"07CG3ZZ"  =  "1"  /* Extirpation of Matter from L Low Extrem Lymph, Perc Approach */
"07CG4ZZ"  =  "1"  /* Extirpate matter from L Low Extrem Lymph, Perc Endo */
"07CH0ZZ"  =  "1"  /* Extirpation of Matter from R Inqnl Lymph, Open Approach */
"07CH3ZZ"  =  "1"  /* Extirpation of Matter from R Inqnl Lymph, Perc Approach */
"07CH4ZZ"  =  "1"  /* Extirpation of Matter from R Inqnl Lymph, Perc Endo Approach */
"07CJ0ZZ"  =  "1"  /* Extirpation of Matter from L Inqnl Lymph, Open Approach */
"07CJ3ZZ"  =  "1"  /* Extirpation of Matter from L Inqnl Lymph, Perc Approach */
"07CJ4ZZ"  =  "1"  /* Extirpation of Matter from L Inqnl Lymph, Perc Endo Approach */
"07CK0ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Duct, Open Approach */
"07CK3ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Duct, Perc Approach */
"07CK4ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Duct, Perc Endo Approach */
"07CL0ZZ"  =  "1"  /* Extirpation of Matter from Cisterna Chyli, Open Approach */
"07CL3ZZ"  =  "1"  /* Extirpation of Matter from Cisterna Chyli, Perc Approach */
"07CL4ZZ"  =  "1"  /* Extirpate of Matter from Cisterna Chyli, Perc Endo Approach */
"07CM0ZZ"  =  "1"  /* Extirpation of Matter from Thymus, Open Approach */
"07CM3ZZ"  =  "1"  /* Extirpation of Matter from Thymus, Percutaneous Approach */
"07CM4ZZ"  =  "1"  /* Extirpation of Matter from Thymus, Perc Endo Approach */
"07CP0ZZ"  =  "1"  /* Extirpation of Matter from Spleen, Open Approach */
"07JK0ZZ"  =  "1"  /* Inspection of Thoracic Duct, Open Approach */
"07JK3ZZ"  =  "1"  /* Inspection of Thoracic Duct, Percutaneous Approach */
"07JK4ZZ"  =  "1"  /* Inspection of Thoracic Duct, Perc Endo Approach */
"07JL0ZZ"  =  "1"  /* Inspection of Cisterna Chyli, Open Approach */
"07JL3ZZ"  =  "1"  /* Inspection of Cisterna Chyli, Percutaneous Approach */
"07JL4ZZ"  =  "1"  /* Inspection of Cisterna Chyli, Perc Endo Approach */
"07JM0ZZ"  =  "1"  /* Inspection of Thymus, Open Approach */
"07JM3ZZ"  =  "1"  /* Inspection of Thymus, Percutaneous Approach */
"07JM4ZZ"  =  "1"  /* Inspection of Thymus, Percutaneous Endoscopic Approach */
"07JN0ZZ"  =  "1"  /* Inspection of Lymphatic, Open Approach */
"07JN3ZZ"  =  "1"  /* Inspection of Lymphatic, Percutaneous Approach */
"07JN4ZZ"  =  "1"  /* Inspection of Lymphatic, Percutaneous Endoscopic Approach */
"07JP0ZZ"  =  "1"  /* Inspection of Spleen, Open Approach */
"07L00CZ"  =  "1"  /* Occlusion of Head Lymphatic with Extralum Dev, Open Approach */
"07L00DZ"  =  "1"  /* Occlusion of Head Lymphatic with Intralum Dev, Open Approach */
"07L00ZZ"  =  "1"  /* Occlusion of Head Lymphatic, Open Approach */
"07L03CZ"  =  "1"  /* Occlusion of Head Lymphatic with Extralum Dev, Perc Approach */
"07L03DZ"  =  "1"  /* Occlusion of Head Lymphatic with Intralum Dev, Perc Approach */
"07L03ZZ"  =  "1"  /* Occlusion of Head Lymphatic, Percutaneous Approach */
"07L04CZ"  =  "1"  /* Occlusion Head Lymph w Extralum Dev, Perc Endo */
"07L04DZ"  =  "1"  /* Occlusion Head Lymph w Intralum Dev, Perc Endo */
"07L04ZZ"  =  "1"  /* Occlusion of Head Lymphatic, Perc Endo Approach */
"07L10CZ"  =  "1"  /* Occlusion of R Neck Lymph with Extralum Dev, Open Approach */
"07L10DZ"  =  "1"  /* Occlusion of R Neck Lymph with Intralum Dev, Open Approach */
"07L10ZZ"  =  "1"  /* Occlusion of Right Neck Lymphatic, Open Approach */
"07L13CZ"  =  "1"  /* Occlusion of R Neck Lymph with Extralum Dev, Perc Approach */
"07L13DZ"  =  "1"  /* Occlusion of R Neck Lymph with Intralum Dev, Perc Approach */
"07L13ZZ"  =  "1"  /* Occlusion of Right Neck Lymphatic, Percutaneous Approach */
"07L14CZ"  =  "1"  /* Occlusion R Neck Lymph w Extralum Dev, Perc Endo */
"07L14DZ"  =  "1"  /* Occlusion R Neck Lymph w Intralum Dev, Perc Endo */
"07L14ZZ"  =  "1"  /* Occlusion of Right Neck Lymphatic, Perc Endo Approach */
"07L20CZ"  =  "1"  /* Occlusion of L Neck Lymph with Extralum Dev, Open Approach */
"07L20DZ"  =  "1"  /* Occlusion of L Neck Lymph with Intralum Dev, Open Approach */
"07L20ZZ"  =  "1"  /* Occlusion of Left Neck Lymphatic, Open Approach */
"07L23CZ"  =  "1"  /* Occlusion of L Neck Lymph with Extralum Dev, Perc Approach */
"07L23DZ"  =  "1"  /* Occlusion of L Neck Lymph with Intralum Dev, Perc Approach */
"07L23ZZ"  =  "1"  /* Occlusion of Left Neck Lymphatic, Percutaneous Approach */
"07L24CZ"  =  "1"  /* Occlusion L Neck Lymph w Extralum Dev, Perc Endo */
"07L24DZ"  =  "1"  /* Occlusion L Neck Lymph w Intralum Dev, Perc Endo */
"07L24ZZ"  =  "1"  /* Occlusion of Left Neck Lymphatic, Perc Endo Approach */
"07L30CZ"  =  "1"  /* Occlusion R Up Extrem Lymph w Extralum Dev, Open */
"07L30DZ"  =  "1"  /* Occlusion R Up Extrem Lymph w Intralum Dev, Open */
"07L30ZZ"  =  "1"  /* Occlusion of Right Upper Extremity Lymphatic, Open Approach */
"07L33CZ"  =  "1"  /* Occlusion R Up Extrem Lymph w Extralum Dev, Perc */
"07L33DZ"  =  "1"  /* Occlusion R Up Extrem Lymph w Intralum Dev, Perc */
"07L33ZZ"  =  "1"  /* Occlusion of Right Upper Extremity Lymphatic, Perc Approach */
"07L34CZ"  =  "1"  /* Occlusion R Up Extrem Lymph w Extralum Dev, Perc Endo */
"07L34DZ"  =  "1"  /* Occlusion R Up Extrem Lymph w Intralum Dev, Perc Endo */
"07L34ZZ"  =  "1"  /* Occlusion of R Up Extrem Lymph, Perc Endo Approach */
"07L40CZ"  =  "1"  /* Occlusion L Up Extrem Lymph w Extralum Dev, Open */
"07L40DZ"  =  "1"  /* Occlusion L Up Extrem Lymph w Intralum Dev, Open */
"07L40ZZ"  =  "1"  /* Occlusion of Left Upper Extremity Lymphatic, Open Approach */
"07L43CZ"  =  "1"  /* Occlusion L Up Extrem Lymph w Extralum Dev, Perc */
"07L43DZ"  =  "1"  /* Occlusion L Up Extrem Lymph w Intralum Dev, Perc */
"07L43ZZ"  =  "1"  /* Occlusion of Left Upper Extremity Lymphatic, Perc Approach */
"07L44CZ"  =  "1"  /* Occlusion L Up Extrem Lymph w Extralum Dev, Perc Endo */
"07L44DZ"  =  "1"  /* Occlusion L Up Extrem Lymph w Intralum Dev, Perc Endo */
"07L44ZZ"  =  "1"  /* Occlusion of L Up Extrem Lymph, Perc Endo Approach */
"07L50CZ"  =  "1"  /* Occlusion of R Axilla Lymph with Extralum Dev, Open Approach */
"07L50DZ"  =  "1"  /* Occlusion of R Axilla Lymph with Intralum Dev, Open Approach */
"07L50ZZ"  =  "1"  /* Occlusion of Right Axillary Lymphatic, Open Approach */
"07L53CZ"  =  "1"  /* Occlusion of R Axilla Lymph with Extralum Dev, Perc Approach */
"07L53DZ"  =  "1"  /* Occlusion of R Axilla Lymph with Intralum Dev, Perc Approach */
"07L53ZZ"  =  "1"  /* Occlusion of Right Axillary Lymphatic, Percutaneous Approach */
"07L54CZ"  =  "1"  /* Occlusion R Axilla Lymph w Extralum Dev, Perc Endo */
"07L54DZ"  =  "1"  /* Occlusion R Axilla Lymph w Intralum Dev, Perc Endo */
"07L54ZZ"  =  "1"  /* Occlusion of Right Axillary Lymphatic, Perc Endo Approach */
"07L60CZ"  =  "1"  /* Occlusion of L Axilla Lymph with Extralum Dev, Open Approach */
"07L60DZ"  =  "1"  /* Occlusion of L Axilla Lymph with Intralum Dev, Open Approach */
"07L60ZZ"  =  "1"  /* Occlusion of Left Axillary Lymphatic, Open Approach */
"07L63CZ"  =  "1"  /* Occlusion of L Axilla Lymph with Extralum Dev, Perc Approach */
"07L63DZ"  =  "1"  /* Occlusion of L Axilla Lymph with Intralum Dev, Perc Approach */
"07L63ZZ"  =  "1"  /* Occlusion of Left Axillary Lymphatic, Percutaneous Approach */
"07L64CZ"  =  "1"  /* Occlusion L Axilla Lymph w Extralum Dev, Perc Endo */
"07L64DZ"  =  "1"  /* Occlusion L Axilla Lymph w Intralum Dev, Perc Endo */
"07L64ZZ"  =  "1"  /* Occlusion of Left Axillary Lymphatic, Perc Endo Approach */
"07L70CZ"  =  "1"  /* Occlusion of Thorax Lymph with Extralum Dev, Open Approach */
"07L70DZ"  =  "1"  /* Occlusion of Thorax Lymph with Intralum Dev, Open Approach */
"07L70ZZ"  =  "1"  /* Occlusion of Thorax Lymphatic, Open Approach */
"07L73CZ"  =  "1"  /* Occlusion of Thorax Lymph with Extralum Dev, Perc Approach */
"07L73DZ"  =  "1"  /* Occlusion of Thorax Lymph with Intralum Dev, Perc Approach */
"07L73ZZ"  =  "1"  /* Occlusion of Thorax Lymphatic, Percutaneous Approach */
"07L74CZ"  =  "1"  /* Occlusion Thorax Lymph w Extralum Dev, Perc Endo */
"07L74DZ"  =  "1"  /* Occlusion Thorax Lymph w Intralum Dev, Perc Endo */
"07L74ZZ"  =  "1"  /* Occlusion of Thorax Lymphatic, Perc Endo Approach */
"07L80CZ"  =  "1"  /* Occlusion Int Mamm, R Lymph w Extralum Dev, Open */
"07L80DZ"  =  "1"  /* Occlusion Int Mamm, R Lymph w Intralum Dev, Open */
"07L80ZZ"  =  "1"  /* Occlusion of Right Internal Mammary Lymphatic, Open Approach */
"07L83CZ"  =  "1"  /* Occlusion Int Mamm, R Lymph w Extralum Dev, Perc */
"07L83DZ"  =  "1"  /* Occlusion Int Mamm, R Lymph w Intralum Dev, Perc */
"07L83ZZ"  =  "1"  /* Occlusion of Right Internal Mammary Lymphatic, Perc Approach */
"07L84CZ"  =  "1"  /* Occlusion Int Mamm, R Lymph w Extralum Dev, Perc Endo */
"07L84DZ"  =  "1"  /* Occlusion Int Mamm, R Lymph w Intralum Dev, Perc Endo */
"07L84ZZ"  =  "1"  /* Occlusion of Int Mamm, R Lymph, Perc Endo Approach */
"07L90CZ"  =  "1"  /* Occlusion Int Mamm, L Lymph w Extralum Dev, Open */
"07L90DZ"  =  "1"  /* Occlusion Int Mamm, L Lymph w Intralum Dev, Open */
"07L90ZZ"  =  "1"  /* Occlusion of Left Internal Mammary Lymphatic, Open Approach */
"07L93CZ"  =  "1"  /* Occlusion Int Mamm, L Lymph w Extralum Dev, Perc */
"07L93DZ"  =  "1"  /* Occlusion Int Mamm, L Lymph w Intralum Dev, Perc */
"07L93ZZ"  =  "1"  /* Occlusion of Left Internal Mammary Lymphatic, Perc Approach */
"07L94CZ"  =  "1"  /* Occlusion Int Mamm, L Lymph w Extralum Dev, Perc Endo */
"07L94DZ"  =  "1"  /* Occlusion Int Mamm, L Lymph w Intralum Dev, Perc Endo */
"07L94ZZ"  =  "1"  /* Occlusion of Int Mamm, L Lymph, Perc Endo Approach */
"07LB0CZ"  =  "1"  /* Occlusion Mesenteric Lymph w Extralum Dev, Open */
"07LB0DZ"  =  "1"  /* Occlusion Mesenteric Lymph w Intralum Dev, Open */
"07LB0ZZ"  =  "1"  /* Occlusion of Mesenteric Lymphatic, Open Approach */
"07LB3CZ"  =  "1"  /* Occlusion Mesenteric Lymph w Extralum Dev, Perc */
"07LB3DZ"  =  "1"  /* Occlusion Mesenteric Lymph w Intralum Dev, Perc */
"07LB3ZZ"  =  "1"  /* Occlusion of Mesenteric Lymphatic, Percutaneous Approach */
"07LB4CZ"  =  "1"  /* Occlusion Mesenteric Lymph w Extralum Dev, Perc Endo */
"07LB4DZ"  =  "1"  /* Occlusion Mesenteric Lymph w Intralum Dev, Perc Endo */
"07LB4ZZ"  =  "1"  /* Occlusion of Mesenteric Lymphatic, Perc Endo Approach */
"07LC0CZ"  =  "1"  /* Occlusion of Pelvis Lymph with Extralum Dev, Open Approach */
"07LC0DZ"  =  "1"  /* Occlusion of Pelvis Lymph with Intralum Dev, Open Approach */
"07LC0ZZ"  =  "1"  /* Occlusion of Pelvis Lymphatic, Open Approach */
"07LC3CZ"  =  "1"  /* Occlusion of Pelvis Lymph with Extralum Dev, Perc Approach */
"07LC3DZ"  =  "1"  /* Occlusion of Pelvis Lymph with Intralum Dev, Perc Approach */
"07LC3ZZ"  =  "1"  /* Occlusion of Pelvis Lymphatic, Percutaneous Approach */
"07LC4CZ"  =  "1"  /* Occlusion Pelvis Lymph w Extralum Dev, Perc Endo */
"07LC4DZ"  =  "1"  /* Occlusion Pelvis Lymph w Intralum Dev, Perc Endo */
"07LC4ZZ"  =  "1"  /* Occlusion of Pelvis Lymphatic, Perc Endo Approach */
"07LD0CZ"  =  "1"  /* Occlusion of Aortic Lymph with Extralum Dev, Open Approach */
"07LD0DZ"  =  "1"  /* Occlusion of Aortic Lymph with Intralum Dev, Open Approach */
"07LD0ZZ"  =  "1"  /* Occlusion of Aortic Lymphatic, Open Approach */
"07LD3CZ"  =  "1"  /* Occlusion of Aortic Lymph with Extralum Dev, Perc Approach */
"07LD3DZ"  =  "1"  /* Occlusion of Aortic Lymph with Intralum Dev, Perc Approach */
"07LD3ZZ"  =  "1"  /* Occlusion of Aortic Lymphatic, Percutaneous Approach */
"07LD4CZ"  =  "1"  /* Occlusion Aortic Lymph w Extralum Dev, Perc Endo */
"07LD4DZ"  =  "1"  /* Occlusion Aortic Lymph w Intralum Dev, Perc Endo */
"07LD4ZZ"  =  "1"  /* Occlusion of Aortic Lymphatic, Perc Endo Approach */
"07LF0CZ"  =  "1"  /* Occlusion R Low Extrem Lymph w Extralum Dev, Open */
"07LF0DZ"  =  "1"  /* Occlusion R Low Extrem Lymph w Intralum Dev, Open */
"07LF0ZZ"  =  "1"  /* Occlusion of Right Lower Extremity Lymphatic, Open Approach */
"07LF3CZ"  =  "1"  /* Occlusion R Low Extrem Lymph w Extralum Dev, Perc */
"07LF3DZ"  =  "1"  /* Occlusion R Low Extrem Lymph w Intralum Dev, Perc */
"07LF3ZZ"  =  "1"  /* Occlusion of Right Lower Extremity Lymphatic, Perc Approach */
"07LF4CZ"  =  "1"  /* Occlusion R Low Extrem Lymph w Extralum Dev, Perc Endo */
"07LF4DZ"  =  "1"  /* Occlusion R Low Extrem Lymph w Intralum Dev, Perc Endo */
"07LF4ZZ"  =  "1"  /* Occlusion of R Low Extrem Lymph, Perc Endo Approach */
"07LG0CZ"  =  "1"  /* Occlusion L Low Extrem Lymph w Extralum Dev, Open */
"07LG0DZ"  =  "1"  /* Occlusion L Low Extrem Lymph w Intralum Dev, Open */
"07LG0ZZ"  =  "1"  /* Occlusion of Left Lower Extremity Lymphatic, Open Approach */
"07LG3CZ"  =  "1"  /* Occlusion L Low Extrem Lymph w Extralum Dev, Perc */
"07LG3DZ"  =  "1"  /* Occlusion L Low Extrem Lymph w Intralum Dev, Perc */
"07LG3ZZ"  =  "1"  /* Occlusion of Left Lower Extremity Lymphatic, Perc Approach */
"07LG4CZ"  =  "1"  /* Occlusion L Low Extrem Lymph w Extralum Dev, Perc Endo */
"07LG4DZ"  =  "1"  /* Occlusion L Low Extrem Lymph w Intralum Dev, Perc Endo */
"07LG4ZZ"  =  "1"  /* Occlusion of L Low Extrem Lymph, Perc Endo Approach */
"07LH0CZ"  =  "1"  /* Occlusion of R Inqnl Lymph with Extralum Dev, Open Approach */
"07LH0DZ"  =  "1"  /* Occlusion of R Inqnl Lymph with Intralum Dev, Open Approach */
"07LH0ZZ"  =  "1"  /* Occlusion of Right Inguinal Lymphatic, Open Approach */
"07LH3CZ"  =  "1"  /* Occlusion of R Inqnl Lymph with Extralum Dev, Perc Approach */
"07LH3DZ"  =  "1"  /* Occlusion of R Inqnl Lymph with Intralum Dev, Perc Approach */
"07LH3ZZ"  =  "1"  /* Occlusion of Right Inguinal Lymphatic, Percutaneous Approach */
"07LH4CZ"  =  "1"  /* Occlusion R Inqnl Lymph w Extralum Dev, Perc Endo */
"07LH4DZ"  =  "1"  /* Occlusion R Inqnl Lymph w Intralum Dev, Perc Endo */
"07LH4ZZ"  =  "1"  /* Occlusion of Right Inguinal Lymphatic, Perc Endo Approach */
"07LJ0CZ"  =  "1"  /* Occlusion of L Inqnl Lymph with Extralum Dev, Open Approach */
"07LJ0DZ"  =  "1"  /* Occlusion of L Inqnl Lymph with Intralum Dev, Open Approach */
"07LJ0ZZ"  =  "1"  /* Occlusion of Left Inguinal Lymphatic, Open Approach */
"07LJ3CZ"  =  "1"  /* Occlusion of L Inqnl Lymph with Extralum Dev, Perc Approach */
"07LJ3DZ"  =  "1"  /* Occlusion of L Inqnl Lymph with Intralum Dev, Perc Approach */
"07LJ3ZZ"  =  "1"  /* Occlusion of Left Inguinal Lymphatic, Percutaneous Approach */
"07LJ4CZ"  =  "1"  /* Occlusion L Inqnl Lymph w Extralum Dev, Perc Endo */
"07LJ4DZ"  =  "1"  /* Occlusion L Inqnl Lymph w Intralum Dev, Perc Endo */
"07LJ4ZZ"  =  "1"  /* Occlusion of Left Inguinal Lymphatic, Perc Endo Approach */
"07LK0CZ"  =  "1"  /* Occlusion of Thoracic Duct with Extralum Dev, Open Approach */
"07LK0DZ"  =  "1"  /* Occlusion of Thoracic Duct with Intralum Dev, Open Approach */
"07LK0ZZ"  =  "1"  /* Occlusion of Thoracic Duct, Open Approach */
"07LK3CZ"  =  "1"  /* Occlusion of Thoracic Duct with Extralum Dev, Perc Approach */
"07LK3DZ"  =  "1"  /* Occlusion of Thoracic Duct with Intralum Dev, Perc Approach */
"07LK3ZZ"  =  "1"  /* Occlusion of Thoracic Duct, Percutaneous Approach */
"07LK4CZ"  =  "1"  /* Occlusion Thoracic Duct w Extralum Dev, Perc Endo */
"07LK4DZ"  =  "1"  /* Occlusion Thoracic Duct w Intralum Dev, Perc Endo */
"07LK4ZZ"  =  "1"  /* Occlusion of Thoracic Duct, Percutaneous Endoscopic Approach */
"07LL0CZ"  =  "1"  /* Occlusion of Cisterna Chyli with Extralum Dev, Open Approach */
"07LL0DZ"  =  "1"  /* Occlusion of Cisterna Chyli with Intralum Dev, Open Approach */
"07LL0ZZ"  =  "1"  /* Occlusion of Cisterna Chyli, Open Approach */
"07LL3CZ"  =  "1"  /* Occlusion of Cisterna Chyli with Extralum Dev, Perc Approach */
"07LL3DZ"  =  "1"  /* Occlusion of Cisterna Chyli with Intralum Dev, Perc Approach */
"07LL3ZZ"  =  "1"  /* Occlusion of Cisterna Chyli, Percutaneous Approach */
"07LL4CZ"  =  "1"  /* Occlusion Cisterna Chyli w Extralum Dev, Perc Endo */
"07LL4DZ"  =  "1"  /* Occlusion Cisterna Chyli w Intralum Dev, Perc Endo */
"07LL4ZZ"  =  "1"  /* Occlusion of Cisterna Chyli, Perc Endo Approach */
"07N00ZZ"  =  "1"  /* Release Head Lymphatic, Open Approach */
"07N03ZZ"  =  "1"  /* Release Head Lymphatic, Percutaneous Approach */
"07N04ZZ"  =  "1"  /* Release Head Lymphatic, Percutaneous Endoscopic Approach */
"07N10ZZ"  =  "1"  /* Release Right Neck Lymphatic, Open Approach */
"07N13ZZ"  =  "1"  /* Release Right Neck Lymphatic, Percutaneous Approach */
"07N14ZZ"  =  "1"  /* Release Right Neck Lymphatic, Perc Endo Approach */
"07N20ZZ"  =  "1"  /* Release Left Neck Lymphatic, Open Approach */
"07N23ZZ"  =  "1"  /* Release Left Neck Lymphatic, Percutaneous Approach */
"07N24ZZ"  =  "1"  /* Release Left Neck Lymphatic, Perc Endo Approach */
"07N30ZZ"  =  "1"  /* Release Right Upper Extremity Lymphatic, Open Approach */
"07N33ZZ"  =  "1"  /* Release Right Upper Extremity Lymphatic, Perc Approach */
"07N34ZZ"  =  "1"  /* Release Right Upper Extremity Lymphatic, Perc Endo Approach */
"07N40ZZ"  =  "1"  /* Release Left Upper Extremity Lymphatic, Open Approach */
"07N43ZZ"  =  "1"  /* Release Left Upper Extremity Lymphatic, Perc Approach */
"07N44ZZ"  =  "1"  /* Release Left Upper Extremity Lymphatic, Perc Endo Approach */
"07N50ZZ"  =  "1"  /* Release Right Axillary Lymphatic, Open Approach */
"07N53ZZ"  =  "1"  /* Release Right Axillary Lymphatic, Percutaneous Approach */
"07N54ZZ"  =  "1"  /* Release Right Axillary Lymphatic, Perc Endo Approach */
"07N60ZZ"  =  "1"  /* Release Left Axillary Lymphatic, Open Approach */
"07N63ZZ"  =  "1"  /* Release Left Axillary Lymphatic, Percutaneous Approach */
"07N64ZZ"  =  "1"  /* Release Left Axillary Lymphatic, Perc Endo Approach */
"07N70ZZ"  =  "1"  /* Release Thorax Lymphatic, Open Approach */
"07N73ZZ"  =  "1"  /* Release Thorax Lymphatic, Percutaneous Approach */
"07N74ZZ"  =  "1"  /* Release Thorax Lymphatic, Percutaneous Endoscopic Approach */
"07N80ZZ"  =  "1"  /* Release Right Internal Mammary Lymphatic, Open Approach */
"07N83ZZ"  =  "1"  /* Release Right Internal Mammary Lymphatic, Perc Approach */
"07N84ZZ"  =  "1"  /* Release Right Internal Mammary Lymphatic, Perc Endo Approach */
"07N90ZZ"  =  "1"  /* Release Left Internal Mammary Lymphatic, Open Approach */
"07N93ZZ"  =  "1"  /* Release Left Internal Mammary Lymphatic, Perc Approach */
"07N94ZZ"  =  "1"  /* Release Left Internal Mammary Lymphatic, Perc Endo Approach */
"07NB0ZZ"  =  "1"  /* Release Mesenteric Lymphatic, Open Approach */
"07NB3ZZ"  =  "1"  /* Release Mesenteric Lymphatic, Percutaneous Approach */
"07NB4ZZ"  =  "1"  /* Release Mesenteric Lymphatic, Perc Endo Approach */
"07NC0ZZ"  =  "1"  /* Release Pelvis Lymphatic, Open Approach */
"07NC3ZZ"  =  "1"  /* Release Pelvis Lymphatic, Percutaneous Approach */
"07NC4ZZ"  =  "1"  /* Release Pelvis Lymphatic, Percutaneous Endoscopic Approach */
"07ND0ZZ"  =  "1"  /* Release Aortic Lymphatic, Open Approach */
"07ND3ZZ"  =  "1"  /* Release Aortic Lymphatic, Percutaneous Approach */
"07ND4ZZ"  =  "1"  /* Release Aortic Lymphatic, Percutaneous Endoscopic Approach */
"07NF0ZZ"  =  "1"  /* Release Right Lower Extremity Lymphatic, Open Approach */
"07NF3ZZ"  =  "1"  /* Release Right Lower Extremity Lymphatic, Perc Approach */
"07NF4ZZ"  =  "1"  /* Release Right Lower Extremity Lymphatic, Perc Endo Approach */
"07NG0ZZ"  =  "1"  /* Release Left Lower Extremity Lymphatic, Open Approach */
"07NG3ZZ"  =  "1"  /* Release Left Lower Extremity Lymphatic, Perc Approach */
"07NG4ZZ"  =  "1"  /* Release Left Lower Extremity Lymphatic, Perc Endo Approach */
"07NH0ZZ"  =  "1"  /* Release Right Inguinal Lymphatic, Open Approach */
"07NH3ZZ"  =  "1"  /* Release Right Inguinal Lymphatic, Percutaneous Approach */
"07NH4ZZ"  =  "1"  /* Release Right Inguinal Lymphatic, Perc Endo Approach */
"07NJ0ZZ"  =  "1"  /* Release Left Inguinal Lymphatic, Open Approach */
"07NJ3ZZ"  =  "1"  /* Release Left Inguinal Lymphatic, Percutaneous Approach */
"07NJ4ZZ"  =  "1"  /* Release Left Inguinal Lymphatic, Perc Endo Approach */
"07NK0ZZ"  =  "1"  /* Release Thoracic Duct, Open Approach */
"07NK3ZZ"  =  "1"  /* Release Thoracic Duct, Percutaneous Approach */
"07NK4ZZ"  =  "1"  /* Release Thoracic Duct, Percutaneous Endoscopic Approach */
"07NL0ZZ"  =  "1"  /* Release Cisterna Chyli, Open Approach */
"07NL3ZZ"  =  "1"  /* Release Cisterna Chyli, Percutaneous Approach */
"07NL4ZZ"  =  "1"  /* Release Cisterna Chyli, Percutaneous Endoscopic Approach */
"07NM0ZZ"  =  "1"  /* Release Thymus, Open Approach */
"07NM3ZZ"  =  "1"  /* Release Thymus, Percutaneous Approach */
"07NM4ZZ"  =  "1"  /* Release Thymus, Percutaneous Endoscopic Approach */
"07NP0ZZ"  =  "1"  /* Release Spleen, Open Approach */
"07NP3ZZ"  =  "1"  /* Release Spleen, Percutaneous Approach */
"07NP4ZZ"  =  "1"  /* Release Spleen, Percutaneous Endoscopic Approach */
"07PK00Z"  =  "1"  /* Removal of Drainage Device from Thoracic Duct, Open Approach */
"07PK03Z"  =  "1"  /* Removal of Infusion Device from Thoracic Duct, Open Approach */
"07PK07Z"  =  "1"  /* Removal of Autol Sub from Thoracic Duct, Open Approach */
"07PK0CZ"  =  "1"  /* Removal of Extralum Dev from Thoracic Duct, Open Approach */
"07PK0DZ"  =  "1"  /* Removal of Intralum Dev from Thoracic Duct, Open Approach */
"07PK0JZ"  =  "1"  /* Removal of Synth Sub from Thoracic Duct, Open Approach */
"07PK0KZ"  =  "1"  /* Removal of Nonaut Sub from Thoracic Duct, Open Approach */
"07PK30Z"  =  "1"  /* Removal of Drainage Device from Thoracic Duct, Perc Approach */
"07PK33Z"  =  "1"  /* Removal of Infusion Device from Thoracic Duct, Perc Approach */
"07PK37Z"  =  "1"  /* Removal of Autol Sub from Thoracic Duct, Perc Approach */
"07PK3CZ"  =  "1"  /* Removal of Extralum Dev from Thoracic Duct, Perc Approach */
"07PK3DZ"  =  "1"  /* Removal of Intralum Dev from Thoracic Duct, Perc Approach */
"07PK3JZ"  =  "1"  /* Removal of Synth Sub from Thoracic Duct, Perc Approach */
"07PK3KZ"  =  "1"  /* Removal of Nonaut Sub from Thoracic Duct, Perc Approach */
"07PK40Z"  =  "1"  /* Removal of Drain Dev from Thoracic Duct, Perc Endo Approach */
"07PK43Z"  =  "1"  /* Remove Infusion Dev from Thoracic Duct, Perc Endo */
"07PK47Z"  =  "1"  /* Removal of Autol Sub from Thoracic Duct, Perc Endo Approach */
"07PK4CZ"  =  "1"  /* Remove Extralum Dev from Thoracic Duct, Perc Endo */
"07PK4DZ"  =  "1"  /* Remove Intralum Dev from Thoracic Duct, Perc Endo */
"07PK4JZ"  =  "1"  /* Removal of Synth Sub from Thoracic Duct, Perc Endo Approach */
"07PK4KZ"  =  "1"  /* Removal of Nonaut Sub from Thoracic Duct, Perc Endo Approach */
"07PL00Z"  =  "1"  /* Removal of Drain Dev from Cisterna Chyli, Open Approach */
"07PL03Z"  =  "1"  /* Removal of Infusion Dev from Cisterna Chyli, Open Approach */
"07PL07Z"  =  "1"  /* Removal of Autol Sub from Cisterna Chyli, Open Approach */
"07PL0CZ"  =  "1"  /* Removal of Extralum Dev from Cisterna Chyli, Open Approach */
"07PL0DZ"  =  "1"  /* Removal of Intralum Dev from Cisterna Chyli, Open Approach */
"07PL0JZ"  =  "1"  /* Removal of Synth Sub from Cisterna Chyli, Open Approach */
"07PL0KZ"  =  "1"  /* Removal of Nonaut Sub from Cisterna Chyli, Open Approach */
"07PL30Z"  =  "1"  /* Removal of Drain Dev from Cisterna Chyli, Perc Approach */
"07PL33Z"  =  "1"  /* Removal of Infusion Dev from Cisterna Chyli, Perc Approach */
"07PL37Z"  =  "1"  /* Removal of Autol Sub from Cisterna Chyli, Perc Approach */
"07PL3CZ"  =  "1"  /* Removal of Extralum Dev from Cisterna Chyli, Perc Approach */
"07PL3DZ"  =  "1"  /* Removal of Intralum Dev from Cisterna Chyli, Perc Approach */
"07PL3JZ"  =  "1"  /* Removal of Synth Sub from Cisterna Chyli, Perc Approach */
"07PL3KZ"  =  "1"  /* Removal of Nonaut Sub from Cisterna Chyli, Perc Approach */
"07PL40Z"  =  "1"  /* Removal of Drain Dev from Cisterna Chyli, Perc Endo Approach */
"07PL43Z"  =  "1"  /* Remove Infusion Dev from Cisterna Chyli, Perc Endo */
"07PL47Z"  =  "1"  /* Removal of Autol Sub from Cisterna Chyli, Perc Endo Approach */
"07PL4CZ"  =  "1"  /* Remove Extralum Dev from Cisterna Chyli, Perc Endo */
"07PL4DZ"  =  "1"  /* Remove Intralum Dev from Cisterna Chyli, Perc Endo */
"07PL4JZ"  =  "1"  /* Removal of Synth Sub from Cisterna Chyli, Perc Endo Approach */
"07PL4KZ"  =  "1"  /* Remove of Nonaut Sub from Cisterna Chyli, Perc Endo Approach */
"07PM00Z"  =  "1"  /* Removal of Drainage Device from Thymus, Open Approach */
"07PM03Z"  =  "1"  /* Removal of Infusion Device from Thymus, Open Approach */
"07PM30Z"  =  "1"  /* Removal of Drainage Device from Thymus, Perc Approach */
"07PM33Z"  =  "1"  /* Removal of Infusion Device from Thymus, Perc Approach */
"07PM40Z"  =  "1"  /* Removal of Drainage Device from Thymus, Perc Endo Approach */
"07PM43Z"  =  "1"  /* Removal of Infusion Device from Thymus, Perc Endo Approach */
"07PN00Z"  =  "1"  /* Removal of Drainage Device from Lymphatic, Open Approach */
"07PN03Z"  =  "1"  /* Removal of Infusion Device from Lymphatic, Open Approach */
"07PN07Z"  =  "1"  /* Removal of Autol Sub from Lymph, Open Approach */
"07PN0CZ"  =  "1"  /* Removal of Extraluminal Device from Lymphatic, Open Approach */
"07PN0DZ"  =  "1"  /* Removal of Intraluminal Device from Lymphatic, Open Approach */
"07PN0JZ"  =  "1"  /* Removal of Synthetic Substitute from Lymph, Open Approach */
"07PN0KZ"  =  "1"  /* Removal of Nonaut Sub from Lymph, Open Approach */
"07PN30Z"  =  "1"  /* Removal of Drainage Device from Lymphatic, Perc Approach */
"07PN33Z"  =  "1"  /* Removal of Infusion Device from Lymphatic, Perc Approach */
"07PN37Z"  =  "1"  /* Removal of Autol Sub from Lymph, Perc Approach */
"07PN3CZ"  =  "1"  /* Removal of Extraluminal Device from Lymphatic, Perc Approach */
"07PN3DZ"  =  "1"  /* Removal of Intraluminal Device from Lymphatic, Perc Approach */
"07PN3JZ"  =  "1"  /* Removal of Synthetic Substitute from Lymph, Perc Approach */
"07PN3KZ"  =  "1"  /* Removal of Nonaut Sub from Lymph, Perc Approach */
"07PN40Z"  =  "1"  /* Removal of Drainage Device from Lymph, Perc Endo Approach */
"07PN43Z"  =  "1"  /* Removal of Infusion Device from Lymph, Perc Endo Approach */
"07PN47Z"  =  "1"  /* Removal of Autol Sub from Lymph, Perc Endo Approach */
"07PN4CZ"  =  "1"  /* Removal of Extralum Dev from Lymph, Perc Endo Approach */
"07PN4DZ"  =  "1"  /* Removal of Intralum Dev from Lymph, Perc Endo Approach */
"07PN4JZ"  =  "1"  /* Removal of Synth Sub from Lymph, Perc Endo Approach */
"07PN4KZ"  =  "1"  /* Removal of Nonaut Sub from Lymph, Perc Endo Approach */
"07PP00Z"  =  "1"  /* Removal of Drainage Device from Spleen, Open Approach */
"07PP03Z"  =  "1"  /* Removal of Infusion Device from Spleen, Open Approach */
"07PP30Z"  =  "1"  /* Removal of Drainage Device from Spleen, Perc Approach */
"07PP33Z"  =  "1"  /* Removal of Infusion Device from Spleen, Perc Approach */
"07PP40Z"  =  "1"  /* Removal of Drainage Device from Spleen, Perc Endo Approach */
"07PP43Z"  =  "1"  /* Removal of Infusion Device from Spleen, Perc Endo Approach */
"07Q00ZZ"  =  "1"  /* Repair Head Lymphatic, Open Approach */
"07Q03ZZ"  =  "1"  /* Repair Head Lymphatic, Percutaneous Approach */
"07Q04ZZ"  =  "1"  /* Repair Head Lymphatic, Percutaneous Endoscopic Approach */
"07Q10ZZ"  =  "1"  /* Repair Right Neck Lymphatic, Open Approach */
"07Q13ZZ"  =  "1"  /* Repair Right Neck Lymphatic, Percutaneous Approach */
"07Q14ZZ"  =  "1"  /* Repair Right Neck Lymphatic, Perc Endo Approach */
"07Q20ZZ"  =  "1"  /* Repair Left Neck Lymphatic, Open Approach */
"07Q23ZZ"  =  "1"  /* Repair Left Neck Lymphatic, Percutaneous Approach */
"07Q24ZZ"  =  "1"  /* Repair Left Neck Lymphatic, Percutaneous Endoscopic Approach */
"07Q30ZZ"  =  "1"  /* Repair Right Upper Extremity Lymphatic, Open Approach */
"07Q33ZZ"  =  "1"  /* Repair Right Upper Extremity Lymphatic, Perc Approach */
"07Q34ZZ"  =  "1"  /* Repair Right Upper Extremity Lymphatic, Perc Endo Approach */
"07Q40ZZ"  =  "1"  /* Repair Left Upper Extremity Lymphatic, Open Approach */
"07Q43ZZ"  =  "1"  /* Repair Left Upper Extremity Lymphatic, Percutaneous Approach */
"07Q44ZZ"  =  "1"  /* Repair Left Upper Extremity Lymphatic, Perc Endo Approach */
"07Q50ZZ"  =  "1"  /* Repair Right Axillary Lymphatic, Open Approach */
"07Q53ZZ"  =  "1"  /* Repair Right Axillary Lymphatic, Percutaneous Approach */
"07Q54ZZ"  =  "1"  /* Repair Right Axillary Lymphatic, Perc Endo Approach */
"07Q60ZZ"  =  "1"  /* Repair Left Axillary Lymphatic, Open Approach */
"07Q63ZZ"  =  "1"  /* Repair Left Axillary Lymphatic, Percutaneous Approach */
"07Q64ZZ"  =  "1"  /* Repair Left Axillary Lymphatic, Perc Endo Approach */
"07Q70ZZ"  =  "1"  /* Repair Thorax Lymphatic, Open Approach */
"07Q73ZZ"  =  "1"  /* Repair Thorax Lymphatic, Percutaneous Approach */
"07Q74ZZ"  =  "1"  /* Repair Thorax Lymphatic, Percutaneous Endoscopic Approach */
"07Q80ZZ"  =  "1"  /* Repair Right Internal Mammary Lymphatic, Open Approach */
"07Q83ZZ"  =  "1"  /* Repair Right Internal Mammary Lymphatic, Perc Approach */
"07Q84ZZ"  =  "1"  /* Repair Right Internal Mammary Lymphatic, Perc Endo Approach */
"07Q90ZZ"  =  "1"  /* Repair Left Internal Mammary Lymphatic, Open Approach */
"07Q93ZZ"  =  "1"  /* Repair Left Internal Mammary Lymphatic, Perc Approach */
"07Q94ZZ"  =  "1"  /* Repair Left Internal Mammary Lymphatic, Perc Endo Approach */
"07QB0ZZ"  =  "1"  /* Repair Mesenteric Lymphatic, Open Approach */
"07QB3ZZ"  =  "1"  /* Repair Mesenteric Lymphatic, Percutaneous Approach */
"07QB4ZZ"  =  "1"  /* Repair Mesenteric Lymphatic, Perc Endo Approach */
"07QC0ZZ"  =  "1"  /* Repair Pelvis Lymphatic, Open Approach */
"07QC3ZZ"  =  "1"  /* Repair Pelvis Lymphatic, Percutaneous Approach */
"07QC4ZZ"  =  "1"  /* Repair Pelvis Lymphatic, Percutaneous Endoscopic Approach */
"07QD0ZZ"  =  "1"  /* Repair Aortic Lymphatic, Open Approach */
"07QD3ZZ"  =  "1"  /* Repair Aortic Lymphatic, Percutaneous Approach */
"07QD4ZZ"  =  "1"  /* Repair Aortic Lymphatic, Percutaneous Endoscopic Approach */
"07QF0ZZ"  =  "1"  /* Repair Right Lower Extremity Lymphatic, Open Approach */
"07QF3ZZ"  =  "1"  /* Repair Right Lower Extremity Lymphatic, Perc Approach */
"07QF4ZZ"  =  "1"  /* Repair Right Lower Extremity Lymphatic, Perc Endo Approach */
"07QG0ZZ"  =  "1"  /* Repair Left Lower Extremity Lymphatic, Open Approach */
"07QG3ZZ"  =  "1"  /* Repair Left Lower Extremity Lymphatic, Percutaneous Approach */
"07QG4ZZ"  =  "1"  /* Repair Left Lower Extremity Lymphatic, Perc Endo Approach */
"07QH0ZZ"  =  "1"  /* Repair Right Inguinal Lymphatic, Open Approach */
"07QH3ZZ"  =  "1"  /* Repair Right Inguinal Lymphatic, Percutaneous Approach */
"07QH4ZZ"  =  "1"  /* Repair Right Inguinal Lymphatic, Perc Endo Approach */
"07QJ0ZZ"  =  "1"  /* Repair Left Inguinal Lymphatic, Open Approach */
"07QJ3ZZ"  =  "1"  /* Repair Left Inguinal Lymphatic, Percutaneous Approach */
"07QJ4ZZ"  =  "1"  /* Repair Left Inguinal Lymphatic, Perc Endo Approach */
"07QK0ZZ"  =  "1"  /* Repair Thoracic Duct, Open Approach */
"07QK3ZZ"  =  "1"  /* Repair Thoracic Duct, Percutaneous Approach */
"07QK4ZZ"  =  "1"  /* Repair Thoracic Duct, Percutaneous Endoscopic Approach */
"07QL0ZZ"  =  "1"  /* Repair Cisterna Chyli, Open Approach */
"07QL3ZZ"  =  "1"  /* Repair Cisterna Chyli, Percutaneous Approach */
"07QL4ZZ"  =  "1"  /* Repair Cisterna Chyli, Percutaneous Endoscopic Approach */
"07QM0ZZ"  =  "1"  /* Repair Thymus, Open Approach */
"07QM3ZZ"  =  "1"  /* Repair Thymus, Percutaneous Approach */
"07QM4ZZ"  =  "1"  /* Repair Thymus, Percutaneous Endoscopic Approach */
"07QP0ZZ"  =  "1"  /* Repair Spleen, Open Approach */
"07QP3ZZ"  =  "1"  /* Repair Spleen, Percutaneous Approach */
"07QP4ZZ"  =  "1"  /* Repair Spleen, Percutaneous Endoscopic Approach */
"07SM0ZZ"  =  "1"  /* Reposition Thymus, Open Approach */
"07SP0ZZ"  =  "1"  /* Reposition Spleen, Open Approach */
"07T00ZZ"  =  "1"  /* Resection of Head Lymphatic, Open Approach */
"07T04ZZ"  =  "1"  /* Resection of Head Lymphatic, Perc Endo Approach */
"07T10ZZ"  =  "1"  /* Resection of Right Neck Lymphatic, Open Approach */
"07T14ZZ"  =  "1"  /* Resection of Right Neck Lymphatic, Perc Endo Approach */
"07T20ZZ"  =  "1"  /* Resection of Left Neck Lymphatic, Open Approach */
"07T24ZZ"  =  "1"  /* Resection of Left Neck Lymphatic, Perc Endo Approach */
"07T30ZZ"  =  "1"  /* Resection of Right Upper Extremity Lymphatic, Open Approach */
"07T34ZZ"  =  "1"  /* Resection of R Up Extrem Lymph, Perc Endo Approach */
"07T40ZZ"  =  "1"  /* Resection of Left Upper Extremity Lymphatic, Open Approach */
"07T44ZZ"  =  "1"  /* Resection of L Up Extrem Lymph, Perc Endo Approach */
"07T50ZZ"  =  "1"  /* Resection of Right Axillary Lymphatic, Open Approach */
"07T54ZZ"  =  "1"  /* Resection of Right Axillary Lymphatic, Perc Endo Approach */
"07T60ZZ"  =  "1"  /* Resection of Left Axillary Lymphatic, Open Approach */
"07T64ZZ"  =  "1"  /* Resection of Left Axillary Lymphatic, Perc Endo Approach */
"07T70ZZ"  =  "1"  /* Resection of Thorax Lymphatic, Open Approach */
"07T74ZZ"  =  "1"  /* Resection of Thorax Lymphatic, Perc Endo Approach */
"07T80ZZ"  =  "1"  /* Resection of Right Internal Mammary Lymphatic, Open Approach */
"07T84ZZ"  =  "1"  /* Resection of Int Mamm, R Lymph, Perc Endo Approach */
"07T90ZZ"  =  "1"  /* Resection of Left Internal Mammary Lymphatic, Open Approach */
"07T94ZZ"  =  "1"  /* Resection of Int Mamm, L Lymph, Perc Endo Approach */
"07TB0ZZ"  =  "1"  /* Resection of Mesenteric Lymphatic, Open Approach */
"07TB4ZZ"  =  "1"  /* Resection of Mesenteric Lymphatic, Perc Endo Approach */
"07TC0ZZ"  =  "1"  /* Resection of Pelvis Lymphatic, Open Approach */
"07TC4ZZ"  =  "1"  /* Resection of Pelvis Lymphatic, Perc Endo Approach */
"07TD0ZZ"  =  "1"  /* Resection of Aortic Lymphatic, Open Approach */
"07TD4ZZ"  =  "1"  /* Resection of Aortic Lymphatic, Perc Endo Approach */
"07TF0ZZ"  =  "1"  /* Resection of Right Lower Extremity Lymphatic, Open Approach */
"07TF4ZZ"  =  "1"  /* Resection of R Low Extrem Lymph, Perc Endo Approach */
"07TG0ZZ"  =  "1"  /* Resection of Left Lower Extremity Lymphatic, Open Approach */
"07TG4ZZ"  =  "1"  /* Resection of L Low Extrem Lymph, Perc Endo Approach */
"07TH0ZZ"  =  "1"  /* Resection of Right Inguinal Lymphatic, Open Approach */
"07TH4ZZ"  =  "1"  /* Resection of Right Inguinal Lymphatic, Perc Endo Approach */
"07TJ0ZZ"  =  "1"  /* Resection of Left Inguinal Lymphatic, Open Approach */
"07TJ4ZZ"  =  "1"  /* Resection of Left Inguinal Lymphatic, Perc Endo Approach */
"07TK0ZZ"  =  "1"  /* Resection of Thoracic Duct, Open Approach */
"07TK4ZZ"  =  "1"  /* Resection of Thoracic Duct, Percutaneous Endoscopic Approach */
"07TL0ZZ"  =  "1"  /* Resection of Cisterna Chyli, Open Approach */
"07TL4ZZ"  =  "1"  /* Resection of Cisterna Chyli, Perc Endo Approach */
"07TM0ZZ"  =  "1"  /* Resection of Thymus, Open Approach */
"07TM4ZZ"  =  "1"  /* Resection of Thymus, Percutaneous Endoscopic Approach */
"07TP0ZZ"  =  "1"  /* Resection of Spleen, Open Approach */
"07TP4ZZ"  =  "1"  /* Resection of Spleen, Percutaneous Endoscopic Approach */
"07U007Z"  =  "1"  /* Supplement Head Lymphatic with Autol Sub, Open Approach */
"07U00JZ"  =  "1"  /* Supplement Head Lymphatic with Synth Sub, Open Approach */
"07U00KZ"  =  "1"  /* Supplement Head Lymphatic with Nonaut Sub, Open Approach */
"07U047Z"  =  "1"  /* Supplement Head Lymphatic with Autol Sub, Perc Endo Approach */
"07U04JZ"  =  "1"  /* Supplement Head Lymphatic with Synth Sub, Perc Endo Approach */
"07U04KZ"  =  "1"  /* Supplement Head Lymph with Nonaut Sub, Perc Endo Approach */
"07U107Z"  =  "1"  /* Supplement R Neck Lymph with Autol Sub, Open Approach */
"07U10JZ"  =  "1"  /* Supplement R Neck Lymph with Synth Sub, Open Approach */
"07U10KZ"  =  "1"  /* Supplement R Neck Lymph with Nonaut Sub, Open Approach */
"07U147Z"  =  "1"  /* Supplement R Neck Lymph with Autol Sub, Perc Endo Approach */
"07U14JZ"  =  "1"  /* Supplement R Neck Lymph with Synth Sub, Perc Endo Approach */
"07U14KZ"  =  "1"  /* Supplement R Neck Lymph with Nonaut Sub, Perc Endo Approach */
"07U207Z"  =  "1"  /* Supplement Left Neck Lymphatic with Autol Sub, Open Approach */
"07U20JZ"  =  "1"  /* Supplement Left Neck Lymphatic with Synth Sub, Open Approach */
"07U20KZ"  =  "1"  /* Supplement L Neck Lymph with Nonaut Sub, Open Approach */
"07U247Z"  =  "1"  /* Supplement L Neck Lymph with Autol Sub, Perc Endo Approach */
"07U24JZ"  =  "1"  /* Supplement L Neck Lymph with Synth Sub, Perc Endo Approach */
"07U24KZ"  =  "1"  /* Supplement L Neck Lymph with Nonaut Sub, Perc Endo Approach */
"07U307Z"  =  "1"  /* Supplement R Up Extrem Lymph with Autol Sub, Open Approach */
"07U30JZ"  =  "1"  /* Supplement R Up Extrem Lymph with Synth Sub, Open Approach */
"07U30KZ"  =  "1"  /* Supplement R Up Extrem Lymph with Nonaut Sub, Open Approach */
"07U347Z"  =  "1"  /* Supplement R Up Extrem Lymph w Autol Sub, Perc Endo */
"07U34JZ"  =  "1"  /* Supplement R Up Extrem Lymph w Synth Sub, Perc Endo */
"07U34KZ"  =  "1"  /* Supplement R Up Extrem Lymph w Nonaut Sub, Perc Endo */
"07U407Z"  =  "1"  /* Supplement L Up Extrem Lymph with Autol Sub, Open Approach */
"07U40JZ"  =  "1"  /* Supplement L Up Extrem Lymph with Synth Sub, Open Approach */
"07U40KZ"  =  "1"  /* Supplement L Up Extrem Lymph with Nonaut Sub, Open Approach */
"07U447Z"  =  "1"  /* Supplement L Up Extrem Lymph w Autol Sub, Perc Endo */
"07U44JZ"  =  "1"  /* Supplement L Up Extrem Lymph w Synth Sub, Perc Endo */
"07U44KZ"  =  "1"  /* Supplement L Up Extrem Lymph w Nonaut Sub, Perc Endo */
"07U507Z"  =  "1"  /* Supplement R Axilla Lymph with Autol Sub, Open Approach */
"07U50JZ"  =  "1"  /* Supplement R Axilla Lymph with Synth Sub, Open Approach */
"07U50KZ"  =  "1"  /* Supplement R Axilla Lymph with Nonaut Sub, Open Approach */
"07U547Z"  =  "1"  /* Supplement R Axilla Lymph with Autol Sub, Perc Endo Approach */
"07U54JZ"  =  "1"  /* Supplement R Axilla Lymph with Synth Sub, Perc Endo Approach */
"07U54KZ"  =  "1"  /* Supplement R Axilla Lymph w Nonaut Sub, Perc Endo */
"07U607Z"  =  "1"  /* Supplement L Axilla Lymph with Autol Sub, Open Approach */
"07U60JZ"  =  "1"  /* Supplement L Axilla Lymph with Synth Sub, Open Approach */
"07U60KZ"  =  "1"  /* Supplement L Axilla Lymph with Nonaut Sub, Open Approach */
"07U647Z"  =  "1"  /* Supplement L Axilla Lymph with Autol Sub, Perc Endo Approach */
"07U64JZ"  =  "1"  /* Supplement L Axilla Lymph with Synth Sub, Perc Endo Approach */
"07U64KZ"  =  "1"  /* Supplement L Axilla Lymph w Nonaut Sub, Perc Endo */
"07U707Z"  =  "1"  /* Supplement Thorax Lymphatic with Autol Sub, Open Approach */
"07U70JZ"  =  "1"  /* Supplement Thorax Lymphatic with Synth Sub, Open Approach */
"07U70KZ"  =  "1"  /* Supplement Thorax Lymphatic with Nonaut Sub, Open Approach */
"07U747Z"  =  "1"  /* Supplement Thorax Lymph with Autol Sub, Perc Endo Approach */
"07U74JZ"  =  "1"  /* Supplement Thorax Lymph with Synth Sub, Perc Endo Approach */
"07U74KZ"  =  "1"  /* Supplement Thorax Lymph with Nonaut Sub, Perc Endo Approach */
"07U807Z"  =  "1"  /* Supplement Int Mamm, R Lymph with Autol Sub, Open Approach */
"07U80JZ"  =  "1"  /* Supplement Int Mamm, R Lymph with Synth Sub, Open Approach */
"07U80KZ"  =  "1"  /* Supplement Int Mamm, R Lymph with Nonaut Sub, Open Approach */
"07U847Z"  =  "1"  /* Supplement Int Mamm, R Lymph w Autol Sub, Perc Endo */
"07U84JZ"  =  "1"  /* Supplement Int Mamm, R Lymph w Synth Sub, Perc Endo */
"07U84KZ"  =  "1"  /* Supplement Int Mamm, R Lymph w Nonaut Sub, Perc Endo */
"07U907Z"  =  "1"  /* Supplement Int Mamm, L Lymph with Autol Sub, Open Approach */
"07U90JZ"  =  "1"  /* Supplement Int Mamm, L Lymph with Synth Sub, Open Approach */
"07U90KZ"  =  "1"  /* Supplement Int Mamm, L Lymph with Nonaut Sub, Open Approach */
"07U947Z"  =  "1"  /* Supplement Int Mamm, L Lymph w Autol Sub, Perc Endo */
"07U94JZ"  =  "1"  /* Supplement Int Mamm, L Lymph w Synth Sub, Perc Endo */
"07U94KZ"  =  "1"  /* Supplement Int Mamm, L Lymph w Nonaut Sub, Perc Endo */
"07UB07Z"  =  "1"  /* Supplement Mesenteric Lymph with Autol Sub, Open Approach */
"07UB0JZ"  =  "1"  /* Supplement Mesenteric Lymph with Synth Sub, Open Approach */
"07UB0KZ"  =  "1"  /* Supplement Mesenteric Lymph with Nonaut Sub, Open Approach */
"07UB47Z"  =  "1"  /* Supplement Mesenteric Lymph w Autol Sub, Perc Endo */
"07UB4JZ"  =  "1"  /* Supplement Mesenteric Lymph w Synth Sub, Perc Endo */
"07UB4KZ"  =  "1"  /* Supplement Mesenteric Lymph w Nonaut Sub, Perc Endo */
"07UC07Z"  =  "1"  /* Supplement Pelvis Lymphatic with Autol Sub, Open Approach */
"07UC0JZ"  =  "1"  /* Supplement Pelvis Lymphatic with Synth Sub, Open Approach */
"07UC0KZ"  =  "1"  /* Supplement Pelvis Lymphatic with Nonaut Sub, Open Approach */
"07UC47Z"  =  "1"  /* Supplement Pelvis Lymph with Autol Sub, Perc Endo Approach */
"07UC4JZ"  =  "1"  /* Supplement Pelvis Lymph with Synth Sub, Perc Endo Approach */
"07UC4KZ"  =  "1"  /* Supplement Pelvis Lymph with Nonaut Sub, Perc Endo Approach */
"07UD07Z"  =  "1"  /* Supplement Aortic Lymphatic with Autol Sub, Open Approach */
"07UD0JZ"  =  "1"  /* Supplement Aortic Lymphatic with Synth Sub, Open Approach */
"07UD0KZ"  =  "1"  /* Supplement Aortic Lymphatic with Nonaut Sub, Open Approach */
"07UD47Z"  =  "1"  /* Supplement Aortic Lymph with Autol Sub, Perc Endo Approach */
"07UD4JZ"  =  "1"  /* Supplement Aortic Lymph with Synth Sub, Perc Endo Approach */
"07UD4KZ"  =  "1"  /* Supplement Aortic Lymph with Nonaut Sub, Perc Endo Approach */
"07UF07Z"  =  "1"  /* Supplement R Low Extrem Lymph with Autol Sub, Open Approach */
"07UF0JZ"  =  "1"  /* Supplement R Low Extrem Lymph with Synth Sub, Open Approach */
"07UF0KZ"  =  "1"  /* Supplement R Low Extrem Lymph with Nonaut Sub, Open Approach */
"07UF47Z"  =  "1"  /* Supplement R Low Extrem Lymph w Autol Sub, Perc Endo */
"07UF4JZ"  =  "1"  /* Supplement R Low Extrem Lymph w Synth Sub, Perc Endo */
"07UF4KZ"  =  "1"  /* Supplement R Low Extrem Lymph w Nonaut Sub, Perc Endo */
"07UG07Z"  =  "1"  /* Supplement L Low Extrem Lymph with Autol Sub, Open Approach */
"07UG0JZ"  =  "1"  /* Supplement L Low Extrem Lymph with Synth Sub, Open Approach */
"07UG0KZ"  =  "1"  /* Supplement L Low Extrem Lymph with Nonaut Sub, Open Approach */
"07UG47Z"  =  "1"  /* Supplement L Low Extrem Lymph w Autol Sub, Perc Endo */
"07UG4JZ"  =  "1"  /* Supplement L Low Extrem Lymph w Synth Sub, Perc Endo */
"07UG4KZ"  =  "1"  /* Supplement L Low Extrem Lymph w Nonaut Sub, Perc Endo */
"07UH07Z"  =  "1"  /* Supplement R Inqnl Lymph with Autol Sub, Open Approach */
"07UH0JZ"  =  "1"  /* Supplement R Inqnl Lymph with Synth Sub, Open Approach */
"07UH0KZ"  =  "1"  /* Supplement R Inqnl Lymph with Nonaut Sub, Open Approach */
"07UH47Z"  =  "1"  /* Supplement R Inqnl Lymph with Autol Sub, Perc Endo Approach */
"07UH4JZ"  =  "1"  /* Supplement R Inqnl Lymph with Synth Sub, Perc Endo Approach */
"07UH4KZ"  =  "1"  /* Supplement R Inqnl Lymph with Nonaut Sub, Perc Endo Approach */
"07UJ07Z"  =  "1"  /* Supplement L Inqnl Lymph with Autol Sub, Open Approach */
"07UJ0JZ"  =  "1"  /* Supplement L Inqnl Lymph with Synth Sub, Open Approach */
"07UJ0KZ"  =  "1"  /* Supplement L Inqnl Lymph with Nonaut Sub, Open Approach */
"07UJ47Z"  =  "1"  /* Supplement L Inqnl Lymph with Autol Sub, Perc Endo Approach */
"07UJ4JZ"  =  "1"  /* Supplement L Inqnl Lymph with Synth Sub, Perc Endo Approach */
"07UJ4KZ"  =  "1"  /* Supplement L Inqnl Lymph with Nonaut Sub, Perc Endo Approach */
"07UK07Z"  =  "1"  /* Supplement Thoracic Duct with Autol Sub, Open Approach */
"07UK0JZ"  =  "1"  /* Supplement Thoracic Duct with Synth Sub, Open Approach */
"07UK0KZ"  =  "1"  /* Supplement Thoracic Duct with Nonaut Sub, Open Approach */
"07UK47Z"  =  "1"  /* Supplement Thoracic Duct with Autol Sub, Perc Endo Approach */
"07UK4JZ"  =  "1"  /* Supplement Thoracic Duct with Synth Sub, Perc Endo Approach */
"07UK4KZ"  =  "1"  /* Supplement Thoracic Duct with Nonaut Sub, Perc Endo Approach */
"07UL07Z"  =  "1"  /* Supplement Cisterna Chyli with Autol Sub, Open Approach */
"07UL0JZ"  =  "1"  /* Supplement Cisterna Chyli with Synth Sub, Open Approach */
"07UL0KZ"  =  "1"  /* Supplement Cisterna Chyli with Nonaut Sub, Open Approach */
"07UL47Z"  =  "1"  /* Supplement Cisterna Chyli with Autol Sub, Perc Endo Approach */
"07UL4JZ"  =  "1"  /* Supplement Cisterna Chyli with Synth Sub, Perc Endo Approach */
"07UL4KZ"  =  "1"  /* Supplement Cisterna Chyli w Nonaut Sub, Perc Endo */
"07V00CZ"  =  "1"  /* Restriction of Head Lymph with Extralum Dev, Open Approach */
"07V00DZ"  =  "1"  /* Restriction of Head Lymph with Intralum Dev, Open Approach */
"07V00ZZ"  =  "1"  /* Restriction of Head Lymphatic, Open Approach */
"07V03CZ"  =  "1"  /* Restriction of Head Lymph with Extralum Dev, Perc Approach */
"07V03DZ"  =  "1"  /* Restriction of Head Lymph with Intralum Dev, Perc Approach */
"07V03ZZ"  =  "1"  /* Restriction of Head Lymphatic, Percutaneous Approach */
"07V04CZ"  =  "1"  /* Restrict of Head Lymph with Extralum Dev, Perc Endo Approach */
"07V04DZ"  =  "1"  /* Restrict of Head Lymph with Intralum Dev, Perc Endo Approach */
"07V04ZZ"  =  "1"  /* Restriction of Head Lymphatic, Perc Endo Approach */
"07V10CZ"  =  "1"  /* Restriction of R Neck Lymph with Extralum Dev, Open Approach */
"07V10DZ"  =  "1"  /* Restriction of R Neck Lymph with Intralum Dev, Open Approach */
"07V10ZZ"  =  "1"  /* Restriction of Right Neck Lymphatic, Open Approach */
"07V13CZ"  =  "1"  /* Restriction of R Neck Lymph with Extralum Dev, Perc Approach */
"07V13DZ"  =  "1"  /* Restriction of R Neck Lymph with Intralum Dev, Perc Approach */
"07V13ZZ"  =  "1"  /* Restriction of Right Neck Lymphatic, Percutaneous Approach */
"07V14CZ"  =  "1"  /* Restrict R Neck Lymph w Extralum Dev, Perc Endo */
"07V14DZ"  =  "1"  /* Restrict R Neck Lymph w Intralum Dev, Perc Endo */
"07V14ZZ"  =  "1"  /* Restriction of Right Neck Lymphatic, Perc Endo Approach */
"07V20CZ"  =  "1"  /* Restriction of L Neck Lymph with Extralum Dev, Open Approach */
"07V20DZ"  =  "1"  /* Restriction of L Neck Lymph with Intralum Dev, Open Approach */
"07V20ZZ"  =  "1"  /* Restriction of Left Neck Lymphatic, Open Approach */
"07V23CZ"  =  "1"  /* Restriction of L Neck Lymph with Extralum Dev, Perc Approach */
"07V23DZ"  =  "1"  /* Restriction of L Neck Lymph with Intralum Dev, Perc Approach */
"07V23ZZ"  =  "1"  /* Restriction of Left Neck Lymphatic, Percutaneous Approach */
"07V24CZ"  =  "1"  /* Restrict L Neck Lymph w Extralum Dev, Perc Endo */
"07V24DZ"  =  "1"  /* Restrict L Neck Lymph w Intralum Dev, Perc Endo */
"07V24ZZ"  =  "1"  /* Restriction of Left Neck Lymphatic, Perc Endo Approach */
"07V30CZ"  =  "1"  /* Restrict R Up Extrem Lymph w Extralum Dev, Open */
"07V30DZ"  =  "1"  /* Restrict R Up Extrem Lymph w Intralum Dev, Open */
"07V30ZZ"  =  "1"  /* Restriction of R Up Extrem Lymph, Open Approach */
"07V33CZ"  =  "1"  /* Restrict R Up Extrem Lymph w Extralum Dev, Perc */
"07V33DZ"  =  "1"  /* Restrict R Up Extrem Lymph w Intralum Dev, Perc */
"07V33ZZ"  =  "1"  /* Restriction of R Up Extrem Lymph, Perc Approach */
"07V34CZ"  =  "1"  /* Restrict R Up Extrem Lymph w Extralum Dev, Perc Endo */
"07V34DZ"  =  "1"  /* Restrict R Up Extrem Lymph w Intralum Dev, Perc Endo */
"07V34ZZ"  =  "1"  /* Restriction of R Up Extrem Lymph, Perc Endo Approach */
"07V40CZ"  =  "1"  /* Restrict L Up Extrem Lymph w Extralum Dev, Open */
"07V40DZ"  =  "1"  /* Restrict L Up Extrem Lymph w Intralum Dev, Open */
"07V40ZZ"  =  "1"  /* Restriction of Left Upper Extremity Lymphatic, Open Approach */
"07V43CZ"  =  "1"  /* Restrict L Up Extrem Lymph w Extralum Dev, Perc */
"07V43DZ"  =  "1"  /* Restrict L Up Extrem Lymph w Intralum Dev, Perc */
"07V43ZZ"  =  "1"  /* Restriction of Left Upper Extremity Lymphatic, Perc Approach */
"07V44CZ"  =  "1"  /* Restrict L Up Extrem Lymph w Extralum Dev, Perc Endo */
"07V44DZ"  =  "1"  /* Restrict L Up Extrem Lymph w Intralum Dev, Perc Endo */
"07V44ZZ"  =  "1"  /* Restriction of L Up Extrem Lymph, Perc Endo Approach */
"07V50CZ"  =  "1"  /* Restrict of R Axilla Lymph with Extralum Dev, Open Approach */
"07V50DZ"  =  "1"  /* Restrict of R Axilla Lymph with Intralum Dev, Open Approach */
"07V50ZZ"  =  "1"  /* Restriction of Right Axillary Lymphatic, Open Approach */
"07V53CZ"  =  "1"  /* Restrict of R Axilla Lymph with Extralum Dev, Perc Approach */
"07V53DZ"  =  "1"  /* Restrict of R Axilla Lymph with Intralum Dev, Perc Approach */
"07V53ZZ"  =  "1"  /* Restriction of Right Axillary Lymphatic, Perc Approach */
"07V54CZ"  =  "1"  /* Restrict R Axilla Lymph w Extralum Dev, Perc Endo */
"07V54DZ"  =  "1"  /* Restrict R Axilla Lymph w Intralum Dev, Perc Endo */
"07V54ZZ"  =  "1"  /* Restriction of Right Axillary Lymphatic, Perc Endo Approach */
"07V60CZ"  =  "1"  /* Restrict of L Axilla Lymph with Extralum Dev, Open Approach */
"07V60DZ"  =  "1"  /* Restrict of L Axilla Lymph with Intralum Dev, Open Approach */
"07V60ZZ"  =  "1"  /* Restriction of Left Axillary Lymphatic, Open Approach */
"07V63CZ"  =  "1"  /* Restrict of L Axilla Lymph with Extralum Dev, Perc Approach */
"07V63DZ"  =  "1"  /* Restrict of L Axilla Lymph with Intralum Dev, Perc Approach */
"07V63ZZ"  =  "1"  /* Restriction of Left Axillary Lymphatic, Perc Approach */
"07V64CZ"  =  "1"  /* Restrict L Axilla Lymph w Extralum Dev, Perc Endo */
"07V64DZ"  =  "1"  /* Restrict L Axilla Lymph w Intralum Dev, Perc Endo */
"07V64ZZ"  =  "1"  /* Restriction of Left Axillary Lymphatic, Perc Endo Approach */
"07V70CZ"  =  "1"  /* Restriction of Thorax Lymph with Extralum Dev, Open Approach */
"07V70DZ"  =  "1"  /* Restriction of Thorax Lymph with Intralum Dev, Open Approach */
"07V70ZZ"  =  "1"  /* Restriction of Thorax Lymphatic, Open Approach */
"07V73CZ"  =  "1"  /* Restriction of Thorax Lymph with Extralum Dev, Perc Approach */
"07V73DZ"  =  "1"  /* Restriction of Thorax Lymph with Intralum Dev, Perc Approach */
"07V73ZZ"  =  "1"  /* Restriction of Thorax Lymphatic, Percutaneous Approach */
"07V74CZ"  =  "1"  /* Restrict Thorax Lymph w Extralum Dev, Perc Endo */
"07V74DZ"  =  "1"  /* Restrict Thorax Lymph w Intralum Dev, Perc Endo */
"07V74ZZ"  =  "1"  /* Restriction of Thorax Lymphatic, Perc Endo Approach */
"07V80CZ"  =  "1"  /* Restrict Int Mamm, R Lymph w Extralum Dev, Open */
"07V80DZ"  =  "1"  /* Restrict Int Mamm, R Lymph w Intralum Dev, Open */
"07V80ZZ"  =  "1"  /* Restriction of Int Mamm, R Lymph, Open Approach */
"07V83CZ"  =  "1"  /* Restrict Int Mamm, R Lymph w Extralum Dev, Perc */
"07V83DZ"  =  "1"  /* Restrict Int Mamm, R Lymph w Intralum Dev, Perc */
"07V83ZZ"  =  "1"  /* Restriction of Int Mamm, R Lymph, Perc Approach */
"07V84CZ"  =  "1"  /* Restrict Int Mamm, R Lymph w Extralum Dev, Perc Endo */
"07V84DZ"  =  "1"  /* Restrict Int Mamm, R Lymph w Intralum Dev, Perc Endo */
"07V84ZZ"  =  "1"  /* Restriction of Int Mamm, R Lymph, Perc Endo Approach */
"07V90CZ"  =  "1"  /* Restrict Int Mamm, L Lymph w Extralum Dev, Open */
"07V90DZ"  =  "1"  /* Restrict Int Mamm, L Lymph w Intralum Dev, Open */
"07V90ZZ"  =  "1"  /* Restriction of Int Mamm, L Lymph, Open Approach */
"07V93CZ"  =  "1"  /* Restrict Int Mamm, L Lymph w Extralum Dev, Perc */
"07V93DZ"  =  "1"  /* Restrict Int Mamm, L Lymph w Intralum Dev, Perc */
"07V93ZZ"  =  "1"  /* Restriction of Int Mamm, L Lymph, Perc Approach */
"07V94CZ"  =  "1"  /* Restrict Int Mamm, L Lymph w Extralum Dev, Perc Endo */
"07V94DZ"  =  "1"  /* Restrict Int Mamm, L Lymph w Intralum Dev, Perc Endo */
"07V94ZZ"  =  "1"  /* Restriction of Int Mamm, L Lymph, Perc Endo Approach */
"07VB0CZ"  =  "1"  /* Restrict Mesenteric Lymph w Extralum Dev, Open */
"07VB0DZ"  =  "1"  /* Restrict Mesenteric Lymph w Intralum Dev, Open */
"07VB0ZZ"  =  "1"  /* Restriction of Mesenteric Lymphatic, Open Approach */
"07VB3CZ"  =  "1"  /* Restrict Mesenteric Lymph w Extralum Dev, Perc */
"07VB3DZ"  =  "1"  /* Restrict Mesenteric Lymph w Intralum Dev, Perc */
"07VB3ZZ"  =  "1"  /* Restriction of Mesenteric Lymphatic, Percutaneous Approach */
"07VB4CZ"  =  "1"  /* Restrict Mesenteric Lymph w Extralum Dev, Perc Endo */
"07VB4DZ"  =  "1"  /* Restrict Mesenteric Lymph w Intralum Dev, Perc Endo */
"07VB4ZZ"  =  "1"  /* Restriction of Mesenteric Lymphatic, Perc Endo Approach */
"07VC0CZ"  =  "1"  /* Restriction of Pelvis Lymph with Extralum Dev, Open Approach */
"07VC0DZ"  =  "1"  /* Restriction of Pelvis Lymph with Intralum Dev, Open Approach */
"07VC0ZZ"  =  "1"  /* Restriction of Pelvis Lymphatic, Open Approach */
"07VC3CZ"  =  "1"  /* Restriction of Pelvis Lymph with Extralum Dev, Perc Approach */
"07VC3DZ"  =  "1"  /* Restriction of Pelvis Lymph with Intralum Dev, Perc Approach */
"07VC3ZZ"  =  "1"  /* Restriction of Pelvis Lymphatic, Percutaneous Approach */
"07VC4CZ"  =  "1"  /* Restrict Pelvis Lymph w Extralum Dev, Perc Endo */
"07VC4DZ"  =  "1"  /* Restrict Pelvis Lymph w Intralum Dev, Perc Endo */
"07VC4ZZ"  =  "1"  /* Restriction of Pelvis Lymphatic, Perc Endo Approach */
"07VD0CZ"  =  "1"  /* Restriction of Aortic Lymph with Extralum Dev, Open Approach */
"07VD0DZ"  =  "1"  /* Restriction of Aortic Lymph with Intralum Dev, Open Approach */
"07VD0ZZ"  =  "1"  /* Restriction of Aortic Lymphatic, Open Approach */
"07VD3CZ"  =  "1"  /* Restriction of Aortic Lymph with Extralum Dev, Perc Approach */
"07VD3DZ"  =  "1"  /* Restriction of Aortic Lymph with Intralum Dev, Perc Approach */
"07VD3ZZ"  =  "1"  /* Restriction of Aortic Lymphatic, Percutaneous Approach */
"07VD4CZ"  =  "1"  /* Restrict Aortic Lymph w Extralum Dev, Perc Endo */
"07VD4DZ"  =  "1"  /* Restrict Aortic Lymph w Intralum Dev, Perc Endo */
"07VD4ZZ"  =  "1"  /* Restriction of Aortic Lymphatic, Perc Endo Approach */
"07VF0CZ"  =  "1"  /* Restrict R Low Extrem Lymph w Extralum Dev, Open */
"07VF0DZ"  =  "1"  /* Restrict R Low Extrem Lymph w Intralum Dev, Open */
"07VF0ZZ"  =  "1"  /* Restriction of R Low Extrem Lymph, Open Approach */
"07VF3CZ"  =  "1"  /* Restrict R Low Extrem Lymph w Extralum Dev, Perc */
"07VF3DZ"  =  "1"  /* Restrict R Low Extrem Lymph w Intralum Dev, Perc */
"07VF3ZZ"  =  "1"  /* Restriction of R Low Extrem Lymph, Perc Approach */
"07VF4CZ"  =  "1"  /* Restrict R Low Extrem Lymph w Extralum Dev, Perc Endo */
"07VF4DZ"  =  "1"  /* Restrict R Low Extrem Lymph w Intralum Dev, Perc Endo */
"07VF4ZZ"  =  "1"  /* Restriction of R Low Extrem Lymph, Perc Endo Approach */
"07VG0CZ"  =  "1"  /* Restrict L Low Extrem Lymph w Extralum Dev, Open */
"07VG0DZ"  =  "1"  /* Restrict L Low Extrem Lymph w Intralum Dev, Open */
"07VG0ZZ"  =  "1"  /* Restriction of Left Lower Extremity Lymphatic, Open Approach */
"07VG3CZ"  =  "1"  /* Restrict L Low Extrem Lymph w Extralum Dev, Perc */
"07VG3DZ"  =  "1"  /* Restrict L Low Extrem Lymph w Intralum Dev, Perc */
"07VG3ZZ"  =  "1"  /* Restriction of Left Lower Extremity Lymphatic, Perc Approach */
"07VG4CZ"  =  "1"  /* Restrict L Low Extrem Lymph w Extralum Dev, Perc Endo */
"07VG4DZ"  =  "1"  /* Restrict L Low Extrem Lymph w Intralum Dev, Perc Endo */
"07VG4ZZ"  =  "1"  /* Restriction of L Low Extrem Lymph, Perc Endo Approach */
"07VH0CZ"  =  "1"  /* Restrict of R Inqnl Lymph with Extralum Dev, Open Approach */
"07VH0DZ"  =  "1"  /* Restrict of R Inqnl Lymph with Intralum Dev, Open Approach */
"07VH0ZZ"  =  "1"  /* Restriction of Right Inguinal Lymphatic, Open Approach */
"07VH3CZ"  =  "1"  /* Restrict of R Inqnl Lymph with Extralum Dev, Perc Approach */
"07VH3DZ"  =  "1"  /* Restrict of R Inqnl Lymph with Intralum Dev, Perc Approach */
"07VH3ZZ"  =  "1"  /* Restriction of Right Inguinal Lymphatic, Perc Approach */
"07VH4CZ"  =  "1"  /* Restrict R Inqnl Lymph w Extralum Dev, Perc Endo */
"07VH4DZ"  =  "1"  /* Restrict R Inqnl Lymph w Intralum Dev, Perc Endo */
"07VH4ZZ"  =  "1"  /* Restriction of Right Inguinal Lymphatic, Perc Endo Approach */
"07VJ0CZ"  =  "1"  /* Restrict of L Inqnl Lymph with Extralum Dev, Open Approach */
"07VJ0DZ"  =  "1"  /* Restrict of L Inqnl Lymph with Intralum Dev, Open Approach */
"07VJ0ZZ"  =  "1"  /* Restriction of Left Inguinal Lymphatic, Open Approach */
"07VJ3CZ"  =  "1"  /* Restrict of L Inqnl Lymph with Extralum Dev, Perc Approach */
"07VJ3DZ"  =  "1"  /* Restrict of L Inqnl Lymph with Intralum Dev, Perc Approach */
"07VJ3ZZ"  =  "1"  /* Restriction of Left Inguinal Lymphatic, Perc Approach */
"07VJ4CZ"  =  "1"  /* Restrict L Inqnl Lymph w Extralum Dev, Perc Endo */
"07VJ4DZ"  =  "1"  /* Restrict L Inqnl Lymph w Intralum Dev, Perc Endo */
"07VJ4ZZ"  =  "1"  /* Restriction of Left Inguinal Lymphatic, Perc Endo Approach */
"07VK0CZ"  =  "1"  /* Restrict of Thoracic Duct with Extralum Dev, Open Approach */
"07VK0DZ"  =  "1"  /* Restrict of Thoracic Duct with Intralum Dev, Open Approach */
"07VK0ZZ"  =  "1"  /* Restriction of Thoracic Duct, Open Approach */
"07VK3CZ"  =  "1"  /* Restrict of Thoracic Duct with Extralum Dev, Perc Approach */
"07VK3DZ"  =  "1"  /* Restrict of Thoracic Duct with Intralum Dev, Perc Approach */
"07VK3ZZ"  =  "1"  /* Restriction of Thoracic Duct, Percutaneous Approach */
"07VK4CZ"  =  "1"  /* Restrict Thoracic Duct w Extralum Dev, Perc Endo */
"07VK4DZ"  =  "1"  /* Restrict Thoracic Duct w Intralum Dev, Perc Endo */
"07VK4ZZ"  =  "1"  /* Restriction of Thoracic Duct, Perc Endo Approach */
"07VL0CZ"  =  "1"  /* Restrict of Cisterna Chyli with Extralum Dev, Open Approach */
"07VL0DZ"  =  "1"  /* Restrict of Cisterna Chyli with Intralum Dev, Open Approach */
"07VL0ZZ"  =  "1"  /* Restriction of Cisterna Chyli, Open Approach */
"07VL3CZ"  =  "1"  /* Restrict of Cisterna Chyli with Extralum Dev, Perc Approach */
"07VL3DZ"  =  "1"  /* Restrict of Cisterna Chyli with Intralum Dev, Perc Approach */
"07VL3ZZ"  =  "1"  /* Restriction of Cisterna Chyli, Percutaneous Approach */
"07VL4CZ"  =  "1"  /* Restrict Cisterna Chyli w Extralum Dev, Perc Endo */
"07VL4DZ"  =  "1"  /* Restrict Cisterna Chyli w Intralum Dev, Perc Endo */
"07VL4ZZ"  =  "1"  /* Restriction of Cisterna Chyli, Perc Endo Approach */
"07WK00Z"  =  "1"  /* Revision of Drainage Device in Thoracic Duct, Open Approach */
"07WK03Z"  =  "1"  /* Revision of Infusion Device in Thoracic Duct, Open Approach */
"07WK07Z"  =  "1"  /* Revision of Autol Sub in Thoracic Duct, Open Approach */
"07WK0CZ"  =  "1"  /* Revision of Extralum Dev in Thoracic Duct, Open Approach */
"07WK0DZ"  =  "1"  /* Revision of Intralum Dev in Thoracic Duct, Open Approach */
"07WK0JZ"  =  "1"  /* Revision of Synth Sub in Thoracic Duct, Open Approach */
"07WK0KZ"  =  "1"  /* Revision of Nonaut Sub in Thoracic Duct, Open Approach */
"07WK30Z"  =  "1"  /* Revision of Drainage Device in Thoracic Duct, Perc Approach */
"07WK33Z"  =  "1"  /* Revision of Infusion Device in Thoracic Duct, Perc Approach */
"07WK37Z"  =  "1"  /* Revision of Autol Sub in Thoracic Duct, Perc Approach */
"07WK3CZ"  =  "1"  /* Revision of Extralum Dev in Thoracic Duct, Perc Approach */
"07WK3DZ"  =  "1"  /* Revision of Intralum Dev in Thoracic Duct, Perc Approach */
"07WK3JZ"  =  "1"  /* Revision of Synth Sub in Thoracic Duct, Perc Approach */
"07WK3KZ"  =  "1"  /* Revision of Nonaut Sub in Thoracic Duct, Perc Approach */
"07WK40Z"  =  "1"  /* Revision of Drain Dev in Thoracic Duct, Perc Endo Approach */
"07WK43Z"  =  "1"  /* Revise of Infusion Dev in Thoracic Duct, Perc Endo Approach */
"07WK47Z"  =  "1"  /* Revision of Autol Sub in Thoracic Duct, Perc Endo Approach */
"07WK4CZ"  =  "1"  /* Revise of Extralum Dev in Thoracic Duct, Perc Endo Approach */
"07WK4DZ"  =  "1"  /* Revise of Intralum Dev in Thoracic Duct, Perc Endo Approach */
"07WK4JZ"  =  "1"  /* Revision of Synth Sub in Thoracic Duct, Perc Endo Approach */
"07WK4KZ"  =  "1"  /* Revision of Nonaut Sub in Thoracic Duct, Perc Endo Approach */
"07WL00Z"  =  "1"  /* Revision of Drainage Device in Cisterna Chyli, Open Approach */
"07WL03Z"  =  "1"  /* Revision of Infusion Device in Cisterna Chyli, Open Approach */
"07WL07Z"  =  "1"  /* Revision of Autol Sub in Cisterna Chyli, Open Approach */
"07WL0CZ"  =  "1"  /* Revision of Extralum Dev in Cisterna Chyli, Open Approach */
"07WL0DZ"  =  "1"  /* Revision of Intralum Dev in Cisterna Chyli, Open Approach */
"07WL0JZ"  =  "1"  /* Revision of Synth Sub in Cisterna Chyli, Open Approach */
"07WL0KZ"  =  "1"  /* Revision of Nonaut Sub in Cisterna Chyli, Open Approach */
"07WL30Z"  =  "1"  /* Revision of Drainage Device in Cisterna Chyli, Perc Approach */
"07WL33Z"  =  "1"  /* Revision of Infusion Device in Cisterna Chyli, Perc Approach */
"07WL37Z"  =  "1"  /* Revision of Autol Sub in Cisterna Chyli, Perc Approach */
"07WL3CZ"  =  "1"  /* Revision of Extralum Dev in Cisterna Chyli, Perc Approach */
"07WL3DZ"  =  "1"  /* Revision of Intralum Dev in Cisterna Chyli, Perc Approach */
"07WL3JZ"  =  "1"  /* Revision of Synth Sub in Cisterna Chyli, Perc Approach */
"07WL3KZ"  =  "1"  /* Revision of Nonaut Sub in Cisterna Chyli, Perc Approach */
"07WL40Z"  =  "1"  /* Revision of Drain Dev in Cisterna Chyli, Perc Endo Approach */
"07WL43Z"  =  "1"  /* Revise of Infusion Dev in Cisterna Chyli, Perc Endo Approach */
"07WL47Z"  =  "1"  /* Revision of Autol Sub in Cisterna Chyli, Perc Endo Approach */
"07WL4CZ"  =  "1"  /* Revise of Extralum Dev in Cisterna Chyli, Perc Endo Approach */
"07WL4DZ"  =  "1"  /* Revise of Intralum Dev in Cisterna Chyli, Perc Endo Approach */
"07WL4JZ"  =  "1"  /* Revision of Synth Sub in Cisterna Chyli, Perc Endo Approach */
"07WL4KZ"  =  "1"  /* Revision of Nonaut Sub in Cisterna Chyli, Perc Endo Approach */
"07WM00Z"  =  "1"  /* Revision of Drainage Device in Thymus, Open Approach */
"07WM03Z"  =  "1"  /* Revision of Infusion Device in Thymus, Open Approach */
"07WM30Z"  =  "1"  /* Revision of Drainage Device in Thymus, Percutaneous Approach */
"07WM33Z"  =  "1"  /* Revision of Infusion Device in Thymus, Percutaneous Approach */
"07WM40Z"  =  "1"  /* Revision of Drainage Device in Thymus, Perc Endo Approach */
"07WM43Z"  =  "1"  /* Revision of Infusion Device in Thymus, Perc Endo Approach */
"07WN00Z"  =  "1"  /* Revision of Drainage Device in Lymphatic, Open Approach */
"07WN03Z"  =  "1"  /* Revision of Infusion Device in Lymphatic, Open Approach */
"07WN07Z"  =  "1"  /* Revision of Autol Sub in Lymph, Open Approach */
"07WN0CZ"  =  "1"  /* Revision of Extraluminal Device in Lymphatic, Open Approach */
"07WN0DZ"  =  "1"  /* Revision of Intraluminal Device in Lymphatic, Open Approach */
"07WN0JZ"  =  "1"  /* Revision of Synthetic Substitute in Lymphatic, Open Approach */
"07WN0KZ"  =  "1"  /* Revision of Nonaut Sub in Lymph, Open Approach */
"07WN30Z"  =  "1"  /* Revision of Drainage Device in Lymphatic, Perc Approach */
"07WN33Z"  =  "1"  /* Revision of Infusion Device in Lymphatic, Perc Approach */
"07WN37Z"  =  "1"  /* Revision of Autol Sub in Lymph, Perc Approach */
"07WN3CZ"  =  "1"  /* Revision of Extraluminal Device in Lymphatic, Perc Approach */
"07WN3DZ"  =  "1"  /* Revision of Intraluminal Device in Lymphatic, Perc Approach */
"07WN3JZ"  =  "1"  /* Revision of Synthetic Substitute in Lymphatic, Perc Approach */
"07WN3KZ"  =  "1"  /* Revision of Nonaut Sub in Lymph, Perc Approach */
"07WN40Z"  =  "1"  /* Revision of Drainage Device in Lymphatic, Perc Endo Approach */
"07WN43Z"  =  "1"  /* Revision of Infusion Device in Lymphatic, Perc Endo Approach */
"07WN47Z"  =  "1"  /* Revision of Autol Sub in Lymph, Perc Endo Approach */
"07WN4CZ"  =  "1"  /* Revision of Extraluminal Device in Lymph, Perc Endo Approach */
"07WN4DZ"  =  "1"  /* Revision of Intraluminal Device in Lymph, Perc Endo Approach */
"07WN4JZ"  =  "1"  /* Revision of Synth Sub in Lymph, Perc Endo Approach */
"07WN4KZ"  =  "1"  /* Revision of Nonaut Sub in Lymph, Perc Endo Approach */
"07WP00Z"  =  "1"  /* Revision of Drainage Device in Spleen, Open Approach */
"07WP03Z"  =  "1"  /* Revision of Infusion Device in Spleen, Open Approach */
"07WP30Z"  =  "1"  /* Revision of Drainage Device in Spleen, Percutaneous Approach */
"07WP33Z"  =  "1"  /* Revision of Infusion Device in Spleen, Percutaneous Approach */
"07WP40Z"  =  "1"  /* Revision of Drainage Device in Spleen, Perc Endo Approach */
"07WP43Z"  =  "1"  /* Revision of Infusion Device in Spleen, Perc Endo Approach */
"07YM0Z0"  =  "1"  /* Transplantation of Thymus, Allogeneic, Open Approach */
"07YM0Z1"  =  "1"  /* Transplantation of Thymus, Syngeneic, Open Approach */
"07YM0Z2"  =  "1"  /* Transplantation of Thymus, Zooplastic, Open Approach */
"07YP0Z0"  =  "1"  /* Transplantation of Spleen, Allogeneic, Open Approach */
"07YP0Z1"  =  "1"  /* Transplantation of Spleen, Syngeneic, Open Approach */
"07YP0Z2"  =  "1"  /* Transplantation of Spleen, Zooplastic, Open Approach */
"08123J4"  =  "1"  /* Bypass R Ant Chamber to Sclera with Synth Sub, Perc Approach */
"08123K4"  =  "1"  /* Bypass R Ant Chamber to Sclera w Nonaut Sub, Perc */
"08123Z4"  =  "1"  /* Bypass Right Anterior Chamber to Sclera, Perc Approach */
"08133J4"  =  "1"  /* Bypass L Ant Chamber to Sclera with Synth Sub, Perc Approach */
"08133K4"  =  "1"  /* Bypass L Ant Chamber to Sclera w Nonaut Sub, Perc */
"08133Z4"  =  "1"  /* Bypass Left Anterior Chamber to Sclera, Perc Approach */
"081X0J3"  =  "1"  /* Bypass R Lacrml Duct to Nasal Cav w Synth Sub, Open */
"081X0K3"  =  "1"  /* Bypass R Lacrml Duct to Nasal Cav w Nonaut Sub, Open */
"081X0Z3"  =  "1"  /* Bypass Right Lacrimal Duct to Nasal Cavity, Open Approach */
"081X3J3"  =  "1"  /* Bypass R Lacrml Duct to Nasal Cav w Synth Sub, Perc */
"081X3K3"  =  "1"  /* Bypass R Lacrml Duct to Nasal Cav w Nonaut Sub, Perc */
"081X3Z3"  =  "1"  /* Bypass Right Lacrimal Duct to Nasal Cavity, Perc Approach */
"081Y0J3"  =  "1"  /* Bypass L Lacrml Duct to Nasal Cav w Synth Sub, Open */
"081Y0K3"  =  "1"  /* Bypass L Lacrml Duct to Nasal Cav w Nonaut Sub, Open */
"081Y0Z3"  =  "1"  /* Bypass Left Lacrimal Duct to Nasal Cavity, Open Approach */
"081Y3J3"  =  "1"  /* Bypass L Lacrml Duct to Nasal Cav w Synth Sub, Perc */
"081Y3K3"  =  "1"  /* Bypass L Lacrml Duct to Nasal Cav w Nonaut Sub, Perc */
"081Y3Z3"  =  "1"  /* Bypass Left Lacrimal Duct to Nasal Cavity, Perc Approach */
"0850XZZ"  =  "1"  /* Destruction of Right Eye, External Approach */
"0851XZZ"  =  "1"  /* Destruction of Left Eye, External Approach */
"08523ZZ"  =  "1"  /* Destruction of Right Anterior Chamber, Percutaneous Approach */
"08533ZZ"  =  "1"  /* Destruction of Left Anterior Chamber, Percutaneous Approach */
"08543ZZ"  =  "1"  /* Destruction of Right Vitreous, Percutaneous Approach */
"08553ZZ"  =  "1"  /* Destruction of Left Vitreous, Percutaneous Approach */
"0856XZZ"  =  "1"  /* Destruction of Right Sclera, External Approach */
"0857XZZ"  =  "1"  /* Destruction of Left Sclera, External Approach */
"0858XZZ"  =  "1"  /* Destruction of Right Cornea, External Approach */
"0859XZZ"  =  "1"  /* Destruction of Left Cornea, External Approach */
"085A0ZZ"  =  "1"  /* Destruction of Right Choroid, Open Approach */
"085A3ZZ"  =  "1"  /* Destruction of Right Choroid, Percutaneous Approach */
"085B0ZZ"  =  "1"  /* Destruction of Left Choroid, Open Approach */
"085B3ZZ"  =  "1"  /* Destruction of Left Choroid, Percutaneous Approach */
"085C3ZZ"  =  "1"  /* Destruction of Right Iris, Percutaneous Approach */
"085D3ZZ"  =  "1"  /* Destruction of Left Iris, Percutaneous Approach */
"085E3ZZ"  =  "1"  /* Destruction of Right Retina, Percutaneous Approach */
"085F3ZZ"  =  "1"  /* Destruction of Left Retina, Percutaneous Approach */
"085G3ZZ"  =  "1"  /* Destruction of Right Retinal Vessel, Percutaneous Approach */
"085H3ZZ"  =  "1"  /* Destruction of Left Retinal Vessel, Percutaneous Approach */
"085J3ZZ"  =  "1"  /* Destruction of Right Lens, Percutaneous Approach */
"085K3ZZ"  =  "1"  /* Destruction of Left Lens, Percutaneous Approach */
"085L0ZZ"  =  "1"  /* Destruction of Right Extraocular Muscle, Open Approach */
"085L3ZZ"  =  "1"  /* Destruction of Right Extraocular Muscle, Perc Approach */
"085M0ZZ"  =  "1"  /* Destruction of Left Extraocular Muscle, Open Approach */
"085M3ZZ"  =  "1"  /* Destruction of Left Extraocular Muscle, Perc Approach */
"085N0ZZ"  =  "1"  /* Destruction of Right Upper Eyelid, Open Approach */
"085N3ZZ"  =  "1"  /* Destruction of Right Upper Eyelid, Percutaneous Approach */
"085NXZZ"  =  "1"  /* Destruction of Right Upper Eyelid, External Approach */
"085P0ZZ"  =  "1"  /* Destruction of Left Upper Eyelid, Open Approach */
"085P3ZZ"  =  "1"  /* Destruction of Left Upper Eyelid, Percutaneous Approach */
"085PXZZ"  =  "1"  /* Destruction of Left Upper Eyelid, External Approach */
"085Q0ZZ"  =  "1"  /* Destruction of Right Lower Eyelid, Open Approach */
"085Q3ZZ"  =  "1"  /* Destruction of Right Lower Eyelid, Percutaneous Approach */
"085QXZZ"  =  "1"  /* Destruction of Right Lower Eyelid, External Approach */
"085R0ZZ"  =  "1"  /* Destruction of Left Lower Eyelid, Open Approach */
"085R3ZZ"  =  "1"  /* Destruction of Left Lower Eyelid, Percutaneous Approach */
"085RXZZ"  =  "1"  /* Destruction of Left Lower Eyelid, External Approach */
"085SXZZ"  =  "1"  /* Destruction of Right Conjunctiva, External Approach */
"085TXZZ"  =  "1"  /* Destruction of Left Conjunctiva, External Approach */
"085V0ZZ"  =  "1"  /* Destruction of Right Lacrimal Gland, Open Approach */
"085V3ZZ"  =  "1"  /* Destruction of Right Lacrimal Gland, Percutaneous Approach */
"085W0ZZ"  =  "1"  /* Destruction of Left Lacrimal Gland, Open Approach */
"085W3ZZ"  =  "1"  /* Destruction of Left Lacrimal Gland, Percutaneous Approach */
"085X0ZZ"  =  "1"  /* Destruction of Right Lacrimal Duct, Open Approach */
"085X3ZZ"  =  "1"  /* Destruction of Right Lacrimal Duct, Percutaneous Approach */
"085X7ZZ"  =  "1"  /* Destruction of Right Lacrimal Duct, Via Opening */
"085X8ZZ"  =  "1"  /* Destruction of Right Lacrimal Duct, Endo */
"085Y0ZZ"  =  "1"  /* Destruction of Left Lacrimal Duct, Open Approach */
"085Y3ZZ"  =  "1"  /* Destruction of Left Lacrimal Duct, Percutaneous Approach */
"085Y7ZZ"  =  "1"  /* Destruction of Left Lacrimal Duct, Via Opening */
"085Y8ZZ"  =  "1"  /* Destruction of Left Lacrimal Duct, Endo */
"087X0DZ"  =  "1"  /* Dilation of R Lacrml Duct with Intralum Dev, Open Approach */
"087X0ZZ"  =  "1"  /* Dilation of Right Lacrimal Duct, Open Approach */
"087X3DZ"  =  "1"  /* Dilation of R Lacrml Duct with Intralum Dev, Perc Approach */
"087X3ZZ"  =  "1"  /* Dilation of Right Lacrimal Duct, Percutaneous Approach */
"087X7DZ"  =  "1"  /* Dilation of R Lacrml Duct with Intralum Dev, Via Opening */
"087X7ZZ"  =  "1"  /* Dilation of Right Lacrimal Duct, Via Opening */
"087X8DZ"  =  "1"  /* Dilation of Right Lacrimal Duct with Intralum Dev, Endo */
"087X8ZZ"  =  "1"  /* Dilation of Right Lacrimal Duct, Endo */
"087Y0DZ"  =  "1"  /* Dilation of L Lacrml Duct with Intralum Dev, Open Approach */
"087Y0ZZ"  =  "1"  /* Dilation of Left Lacrimal Duct, Open Approach */
"087Y3DZ"  =  "1"  /* Dilation of L Lacrml Duct with Intralum Dev, Perc Approach */
"087Y3ZZ"  =  "1"  /* Dilation of Left Lacrimal Duct, Percutaneous Approach */
"087Y7DZ"  =  "1"  /* Dilation of L Lacrml Duct with Intralum Dev, Via Opening */
"087Y7ZZ"  =  "1"  /* Dilation of Left Lacrimal Duct, Via Opening */
"087Y8DZ"  =  "1"  /* Dilation of Left Lacrimal Duct with Intralum Dev, Endo */
"087Y8ZZ"  =  "1"  /* Dilation of Left Lacrimal Duct, Endo */
"0890X0Z"  =  "1"  /* Drainage of Right Eye with Drainage Device, Extern Approach */
"0890XZX"  =  "1"  /* Drainage of Right Eye, External Approach, Diagnostic */
"0890XZZ"  =  "1"  /* Drainage of Right Eye, External Approach */
"0891X0Z"  =  "1"  /* Drainage of Left Eye with Drainage Device, External Approach */
"0891XZX"  =  "1"  /* Drainage of Left Eye, External Approach, Diagnostic */
"0891XZZ"  =  "1"  /* Drainage of Left Eye, External Approach */
"089230Z"  =  "1"  /* Drainage of R Ant Chamber with Drain Dev, Perc Approach */
"08923ZX"  =  "1"  /* Drainage of Right Anterior Chamber, Perc Approach, Diagn */
"08923ZZ"  =  "1"  /* Drainage of Right Anterior Chamber, Percutaneous Approach */
"089330Z"  =  "1"  /* Drainage of L Ant Chamber with Drain Dev, Perc Approach */
"08933ZX"  =  "1"  /* Drainage of Left Anterior Chamber, Perc Approach, Diagn */
"08933ZZ"  =  "1"  /* Drainage of Left Anterior Chamber, Percutaneous Approach */
"089430Z"  =  "1"  /* Drainage of Right Vitreous with Drain Dev, Perc Approach */
"08943ZX"  =  "1"  /* Drainage of Right Vitreous, Percutaneous Approach, Diagn */
"08943ZZ"  =  "1"  /* Drainage of Right Vitreous, Percutaneous Approach */
"089530Z"  =  "1"  /* Drainage of Left Vitreous with Drain Dev, Perc Approach */
"08953ZX"  =  "1"  /* Drainage of Left Vitreous, Percutaneous Approach, Diagnostic */
"08953ZZ"  =  "1"  /* Drainage of Left Vitreous, Percutaneous Approach */
"0896X0Z"  =  "1"  /* Drainage of Right Sclera with Drain Dev, Extern Approach */
"0896XZX"  =  "1"  /* Drainage of Right Sclera, External Approach, Diagnostic */
"0896XZZ"  =  "1"  /* Drainage of Right Sclera, External Approach */
"0897X0Z"  =  "1"  /* Drainage of Left Sclera with Drain Dev, Extern Approach */
"0897XZX"  =  "1"  /* Drainage of Left Sclera, External Approach, Diagnostic */
"0897XZZ"  =  "1"  /* Drainage of Left Sclera, External Approach */
"0898X0Z"  =  "1"  /* Drainage of Right Cornea with Drain Dev, Extern Approach */
"0898XZX"  =  "1"  /* Drainage of Right Cornea, External Approach, Diagnostic */
"0898XZZ"  =  "1"  /* Drainage of Right Cornea, External Approach */
"0899X0Z"  =  "1"  /* Drainage of Left Cornea with Drain Dev, Extern Approach */
"0899XZX"  =  "1"  /* Drainage of Left Cornea, External Approach, Diagnostic */
"0899XZZ"  =  "1"  /* Drainage of Left Cornea, External Approach */
"089A00Z"  =  "1"  /* Drainage of Right Choroid with Drain Dev, Open Approach */
"089A0ZX"  =  "1"  /* Drainage of Right Choroid, Open Approach, Diagnostic */
"089A0ZZ"  =  "1"  /* Drainage of Right Choroid, Open Approach */
"089A30Z"  =  "1"  /* Drainage of Right Choroid with Drain Dev, Perc Approach */
"089A3ZX"  =  "1"  /* Drainage of Right Choroid, Percutaneous Approach, Diagnostic */
"089A3ZZ"  =  "1"  /* Drainage of Right Choroid, Percutaneous Approach */
"089B00Z"  =  "1"  /* Drainage of Left Choroid with Drainage Device, Open Approach */
"089B0ZX"  =  "1"  /* Drainage of Left Choroid, Open Approach, Diagnostic */
"089B0ZZ"  =  "1"  /* Drainage of Left Choroid, Open Approach */
"089B30Z"  =  "1"  /* Drainage of Left Choroid with Drainage Device, Perc Approach */
"089B3ZX"  =  "1"  /* Drainage of Left Choroid, Percutaneous Approach, Diagnostic */
"089B3ZZ"  =  "1"  /* Drainage of Left Choroid, Percutaneous Approach */
"089C30Z"  =  "1"  /* Drainage of Right Iris with Drainage Device, Perc Approach */
"089C3ZX"  =  "1"  /* Drainage of Right Iris, Percutaneous Approach, Diagnostic */
"089C3ZZ"  =  "1"  /* Drainage of Right Iris, Percutaneous Approach */
"089D30Z"  =  "1"  /* Drainage of Left Iris with Drainage Device, Perc Approach */
"089D3ZX"  =  "1"  /* Drainage of Left Iris, Percutaneous Approach, Diagnostic */
"089D3ZZ"  =  "1"  /* Drainage of Left Iris, Percutaneous Approach */
"089E30Z"  =  "1"  /* Drainage of Right Retina with Drainage Device, Perc Approach */
"089E3ZX"  =  "1"  /* Drainage of Right Retina, Percutaneous Approach, Diagnostic */
"089E3ZZ"  =  "1"  /* Drainage of Right Retina, Percutaneous Approach */
"089F30Z"  =  "1"  /* Drainage of Left Retina with Drainage Device, Perc Approach */
"089F3ZX"  =  "1"  /* Drainage of Left Retina, Percutaneous Approach, Diagnostic */
"089F3ZZ"  =  "1"  /* Drainage of Left Retina, Percutaneous Approach */
"089G30Z"  =  "1"  /* Drainage of R Retinal Vessel with Drain Dev, Perc Approach */
"089G3ZX"  =  "1"  /* Drainage of Right Retinal Vessel, Perc Approach, Diagn */
"089G3ZZ"  =  "1"  /* Drainage of Right Retinal Vessel, Percutaneous Approach */
"089H30Z"  =  "1"  /* Drainage of L Retinal Vessel with Drain Dev, Perc Approach */
"089H3ZX"  =  "1"  /* Drainage of Left Retinal Vessel, Perc Approach, Diagn */
"089H3ZZ"  =  "1"  /* Drainage of Left Retinal Vessel, Percutaneous Approach */
"089J30Z"  =  "1"  /* Drainage of Right Lens with Drainage Device, Perc Approach */
"089J3ZX"  =  "1"  /* Drainage of Right Lens, Percutaneous Approach, Diagnostic */
"089J3ZZ"  =  "1"  /* Drainage of Right Lens, Percutaneous Approach */
"089K30Z"  =  "1"  /* Drainage of Left Lens with Drainage Device, Perc Approach */
"089K3ZX"  =  "1"  /* Drainage of Left Lens, Percutaneous Approach, Diagnostic */
"089K3ZZ"  =  "1"  /* Drainage of Left Lens, Percutaneous Approach */
"089L00Z"  =  "1"  /* Drainage of R Extraoc Muscle with Drain Dev, Open Approach */
"089L0ZX"  =  "1"  /* Drainage of Right Extraocular Muscle, Open Approach, Diagn */
"089L0ZZ"  =  "1"  /* Drainage of Right Extraocular Muscle, Open Approach */
"089L30Z"  =  "1"  /* Drainage of R Extraoc Muscle with Drain Dev, Perc Approach */
"089L3ZX"  =  "1"  /* Drainage of Right Extraocular Muscle, Perc Approach, Diagn */
"089L3ZZ"  =  "1"  /* Drainage of Right Extraocular Muscle, Percutaneous Approach */
"089M00Z"  =  "1"  /* Drainage of L Extraoc Muscle with Drain Dev, Open Approach */
"089M0ZX"  =  "1"  /* Drainage of Left Extraocular Muscle, Open Approach, Diagn */
"089M0ZZ"  =  "1"  /* Drainage of Left Extraocular Muscle, Open Approach */
"089M30Z"  =  "1"  /* Drainage of L Extraoc Muscle with Drain Dev, Perc Approach */
"089M3ZX"  =  "1"  /* Drainage of Left Extraocular Muscle, Perc Approach, Diagn */
"089M3ZZ"  =  "1"  /* Drainage of Left Extraocular Muscle, Percutaneous Approach */
"089N0ZX"  =  "1"  /* Drainage of Right Upper Eyelid, Open Approach, Diagnostic */
"089N3ZX"  =  "1"  /* Drainage of Right Upper Eyelid, Percutaneous Approach, Diagn */
"089NXZX"  =  "1"  /* Drainage of Right Upper Eyelid, External Approach, Diagn */
"089P0ZX"  =  "1"  /* Drainage of Left Upper Eyelid, Open Approach, Diagnostic */
"089P3ZX"  =  "1"  /* Drainage of Left Upper Eyelid, Percutaneous Approach, Diagn */
"089PXZX"  =  "1"  /* Drainage of Left Upper Eyelid, External Approach, Diagnostic */
"089Q0ZX"  =  "1"  /* Drainage of Right Lower Eyelid, Open Approach, Diagnostic */
"089Q3ZX"  =  "1"  /* Drainage of Right Lower Eyelid, Percutaneous Approach, Diagn */
"089QXZX"  =  "1"  /* Drainage of Right Lower Eyelid, External Approach, Diagn */
"089R0ZX"  =  "1"  /* Drainage of Left Lower Eyelid, Open Approach, Diagnostic */
"089R3ZX"  =  "1"  /* Drainage of Left Lower Eyelid, Percutaneous Approach, Diagn */
"089RXZX"  =  "1"  /* Drainage of Left Lower Eyelid, External Approach, Diagnostic */
"089SX0Z"  =  "1"  /* Drainage of R Conjunctiva with Drain Dev, Extern Approach */
"089SXZX"  =  "1"  /* Drainage of Right Conjunctiva, External Approach, Diagnostic */
"089SXZZ"  =  "1"  /* Drainage of Right Conjunctiva, External Approach */
"089TX0Z"  =  "1"  /* Drainage of Left Conjunctiva with Drain Dev, Extern Approach */
"089TXZX"  =  "1"  /* Drainage of Left Conjunctiva, External Approach, Diagnostic */
"089TXZZ"  =  "1"  /* Drainage of Left Conjunctiva, External Approach */
"089V00Z"  =  "1"  /* Drainage of R Lacrml Gland with Drain Dev, Open Approach */
"089V0ZX"  =  "1"  /* Drainage of Right Lacrimal Gland, Open Approach, Diagnostic */
"089V0ZZ"  =  "1"  /* Drainage of Right Lacrimal Gland, Open Approach */
"089V30Z"  =  "1"  /* Drainage of R Lacrml Gland with Drain Dev, Perc Approach */
"089V3ZX"  =  "1"  /* Drainage of Right Lacrimal Gland, Perc Approach, Diagn */
"089V3ZZ"  =  "1"  /* Drainage of Right Lacrimal Gland, Percutaneous Approach */
"089W00Z"  =  "1"  /* Drainage of L Lacrml Gland with Drain Dev, Open Approach */
"089W0ZX"  =  "1"  /* Drainage of Left Lacrimal Gland, Open Approach, Diagnostic */
"089W0ZZ"  =  "1"  /* Drainage of Left Lacrimal Gland, Open Approach */
"089W30Z"  =  "1"  /* Drainage of L Lacrml Gland with Drain Dev, Perc Approach */
"089W3ZX"  =  "1"  /* Drainage of Left Lacrimal Gland, Perc Approach, Diagn */
"089W3ZZ"  =  "1"  /* Drainage of Left Lacrimal Gland, Percutaneous Approach */
"089X00Z"  =  "1"  /* Drainage of R Lacrml Duct with Drain Dev, Open Approach */
"089X0ZX"  =  "1"  /* Drainage of Right Lacrimal Duct, Open Approach, Diagnostic */
"089X0ZZ"  =  "1"  /* Drainage of Right Lacrimal Duct, Open Approach */
"089X30Z"  =  "1"  /* Drainage of R Lacrml Duct with Drain Dev, Perc Approach */
"089X3ZX"  =  "1"  /* Drainage of Right Lacrimal Duct, Perc Approach, Diagn */
"089X3ZZ"  =  "1"  /* Drainage of Right Lacrimal Duct, Percutaneous Approach */
"089X70Z"  =  "1"  /* Drainage of Right Lacrimal Duct with Drain Dev, Via Opening */
"089X7ZX"  =  "1"  /* Drainage of Right Lacrimal Duct, Via Opening, Diagn */
"089X7ZZ"  =  "1"  /* Drainage of Right Lacrimal Duct, Via Opening */
"089X80Z"  =  "1"  /* Drainage of Right Lacrimal Duct with Drainage Device, Endo */
"089X8ZX"  =  "1"  /* Drainage of Right Lacrimal Duct, Endo, Diagn */
"089X8ZZ"  =  "1"  /* Drainage of Right Lacrimal Duct, Endo */
"089Y00Z"  =  "1"  /* Drainage of Left Lacrimal Duct with Drain Dev, Open Approach */
"089Y0ZX"  =  "1"  /* Drainage of Left Lacrimal Duct, Open Approach, Diagnostic */
"089Y0ZZ"  =  "1"  /* Drainage of Left Lacrimal Duct, Open Approach */
"089Y30Z"  =  "1"  /* Drainage of Left Lacrimal Duct with Drain Dev, Perc Approach */
"089Y3ZX"  =  "1"  /* Drainage of Left Lacrimal Duct, Percutaneous Approach, Diagn */
"089Y3ZZ"  =  "1"  /* Drainage of Left Lacrimal Duct, Percutaneous Approach */
"089Y70Z"  =  "1"  /* Drainage of Left Lacrimal Duct with Drain Dev, Via Opening */
"089Y7ZX"  =  "1"  /* Drainage of Left Lacrimal Duct, Via Opening, Diagn */
"089Y7ZZ"  =  "1"  /* Drainage of Left Lacrimal Duct, Via Opening */
"089Y80Z"  =  "1"  /* Drainage of Left Lacrimal Duct with Drainage Device, Endo */
"089Y8ZX"  =  "1"  /* Drainage of Left Lacrimal Duct, Endo, Diagn */
"089Y8ZZ"  =  "1"  /* Drainage of Left Lacrimal Duct, Endo */
"08B00ZX"  =  "1"  /* Excision of Right Eye, Open Approach, Diagnostic */
"08B00ZZ"  =  "1"  /* Excision of Right Eye, Open Approach */
"08B03ZX"  =  "1"  /* Excision of Right Eye, Percutaneous Approach, Diagnostic */
"08B03ZZ"  =  "1"  /* Excision of Right Eye, Percutaneous Approach */
"08B0XZX"  =  "1"  /* Excision of Right Eye, External Approach, Diagnostic */
"08B0XZZ"  =  "1"  /* Excision of Right Eye, External Approach */
"08B10ZX"  =  "1"  /* Excision of Left Eye, Open Approach, Diagnostic */
"08B10ZZ"  =  "1"  /* Excision of Left Eye, Open Approach */
"08B13ZX"  =  "1"  /* Excision of Left Eye, Percutaneous Approach, Diagnostic */
"08B13ZZ"  =  "1"  /* Excision of Left Eye, Percutaneous Approach */
"08B1XZX"  =  "1"  /* Excision of Left Eye, External Approach, Diagnostic */
"08B1XZZ"  =  "1"  /* Excision of Left Eye, External Approach */
"08B43ZX"  =  "1"  /* Excision of Right Vitreous, Percutaneous Approach, Diagn */
"08B43ZZ"  =  "1"  /* Excision of Right Vitreous, Percutaneous Approach */
"08B53ZX"  =  "1"  /* Excision of Left Vitreous, Percutaneous Approach, Diagnostic */
"08B53ZZ"  =  "1"  /* Excision of Left Vitreous, Percutaneous Approach */
"08B6XZX"  =  "1"  /* Excision of Right Sclera, External Approach, Diagnostic */
"08B6XZZ"  =  "1"  /* Excision of Right Sclera, External Approach */
"08B7XZX"  =  "1"  /* Excision of Left Sclera, External Approach, Diagnostic */
"08B7XZZ"  =  "1"  /* Excision of Left Sclera, External Approach */
"08B8XZX"  =  "1"  /* Excision of Right Cornea, External Approach, Diagnostic */
"08B8XZZ"  =  "1"  /* Excision of Right Cornea, External Approach */
"08B9XZX"  =  "1"  /* Excision of Left Cornea, External Approach, Diagnostic */
"08B9XZZ"  =  "1"  /* Excision of Left Cornea, External Approach */
"08BA0ZX"  =  "1"  /* Excision of Right Choroid, Open Approach, Diagnostic */
"08BA0ZZ"  =  "1"  /* Excision of Right Choroid, Open Approach */
"08BA3ZX"  =  "1"  /* Excision of Right Choroid, Percutaneous Approach, Diagnostic */
"08BA3ZZ"  =  "1"  /* Excision of Right Choroid, Percutaneous Approach */
"08BB0ZX"  =  "1"  /* Excision of Left Choroid, Open Approach, Diagnostic */
"08BB0ZZ"  =  "1"  /* Excision of Left Choroid, Open Approach */
"08BB3ZX"  =  "1"  /* Excision of Left Choroid, Percutaneous Approach, Diagnostic */
"08BB3ZZ"  =  "1"  /* Excision of Left Choroid, Percutaneous Approach */
"08BC3ZX"  =  "1"  /* Excision of Right Iris, Percutaneous Approach, Diagnostic */
"08BC3ZZ"  =  "1"  /* Excision of Right Iris, Percutaneous Approach */
"08BD3ZX"  =  "1"  /* Excision of Left Iris, Percutaneous Approach, Diagnostic */
"08BD3ZZ"  =  "1"  /* Excision of Left Iris, Percutaneous Approach */
"08BE3ZX"  =  "1"  /* Excision of Right Retina, Percutaneous Approach, Diagnostic */
"08BE3ZZ"  =  "1"  /* Excision of Right Retina, Percutaneous Approach */
"08BF3ZX"  =  "1"  /* Excision of Left Retina, Percutaneous Approach, Diagnostic */
"08BF3ZZ"  =  "1"  /* Excision of Left Retina, Percutaneous Approach */
"08BJ3ZX"  =  "1"  /* Excision of Right Lens, Percutaneous Approach, Diagnostic */
"08BJ3ZZ"  =  "1"  /* Excision of Right Lens, Percutaneous Approach */
"08BK3ZX"  =  "1"  /* Excision of Left Lens, Percutaneous Approach, Diagnostic */
"08BK3ZZ"  =  "1"  /* Excision of Left Lens, Percutaneous Approach */
"08BL0ZX"  =  "1"  /* Excision of Right Extraocular Muscle, Open Approach, Diagn */
"08BL0ZZ"  =  "1"  /* Excision of Right Extraocular Muscle, Open Approach */
"08BL3ZX"  =  "1"  /* Excision of Right Extraocular Muscle, Perc Approach, Diagn */
"08BL3ZZ"  =  "1"  /* Excision of Right Extraocular Muscle, Percutaneous Approach */
"08BM0ZX"  =  "1"  /* Excision of Left Extraocular Muscle, Open Approach, Diagn */
"08BM0ZZ"  =  "1"  /* Excision of Left Extraocular Muscle, Open Approach */
"08BM3ZX"  =  "1"  /* Excision of Left Extraocular Muscle, Perc Approach, Diagn */
"08BM3ZZ"  =  "1"  /* Excision of Left Extraocular Muscle, Percutaneous Approach */
"08BN0ZX"  =  "1"  /* Excision of Right Upper Eyelid, Open Approach, Diagnostic */
"08BN0ZZ"  =  "1"  /* Excision of Right Upper Eyelid, Open Approach */
"08BN3ZX"  =  "1"  /* Excision of Right Upper Eyelid, Percutaneous Approach, Diagn */
"08BN3ZZ"  =  "1"  /* Excision of Right Upper Eyelid, Percutaneous Approach */
"08BNXZX"  =  "1"  /* Excision of Right Upper Eyelid, External Approach, Diagn */
"08BNXZZ"  =  "1"  /* Excision of Right Upper Eyelid, External Approach */
"08BP0ZX"  =  "1"  /* Excision of Left Upper Eyelid, Open Approach, Diagnostic */
"08BP0ZZ"  =  "1"  /* Excision of Left Upper Eyelid, Open Approach */
"08BP3ZX"  =  "1"  /* Excision of Left Upper Eyelid, Percutaneous Approach, Diagn */
"08BP3ZZ"  =  "1"  /* Excision of Left Upper Eyelid, Percutaneous Approach */
"08BPXZX"  =  "1"  /* Excision of Left Upper Eyelid, External Approach, Diagnostic */
"08BPXZZ"  =  "1"  /* Excision of Left Upper Eyelid, External Approach */
"08BQ0ZX"  =  "1"  /* Excision of Right Lower Eyelid, Open Approach, Diagnostic */
"08BQ0ZZ"  =  "1"  /* Excision of Right Lower Eyelid, Open Approach */
"08BQ3ZX"  =  "1"  /* Excision of Right Lower Eyelid, Percutaneous Approach, Diagn */
"08BQ3ZZ"  =  "1"  /* Excision of Right Lower Eyelid, Percutaneous Approach */
"08BQXZX"  =  "1"  /* Excision of Right Lower Eyelid, External Approach, Diagn */
"08BQXZZ"  =  "1"  /* Excision of Right Lower Eyelid, External Approach */
"08BR0ZX"  =  "1"  /* Excision of Left Lower Eyelid, Open Approach, Diagnostic */
"08BR0ZZ"  =  "1"  /* Excision of Left Lower Eyelid, Open Approach */
"08BR3ZX"  =  "1"  /* Excision of Left Lower Eyelid, Percutaneous Approach, Diagn */
"08BR3ZZ"  =  "1"  /* Excision of Left Lower Eyelid, Percutaneous Approach */
"08BRXZX"  =  "1"  /* Excision of Left Lower Eyelid, External Approach, Diagnostic */
"08BRXZZ"  =  "1"  /* Excision of Left Lower Eyelid, External Approach */
"08BSXZX"  =  "1"  /* Excision of Right Conjunctiva, External Approach, Diagnostic */
"08BSXZZ"  =  "1"  /* Excision of Right Conjunctiva, External Approach */
"08BTXZX"  =  "1"  /* Excision of Left Conjunctiva, External Approach, Diagnostic */
"08BTXZZ"  =  "1"  /* Excision of Left Conjunctiva, External Approach */
"08BV0ZX"  =  "1"  /* Excision of Right Lacrimal Gland, Open Approach, Diagnostic */
"08BV0ZZ"  =  "1"  /* Excision of Right Lacrimal Gland, Open Approach */
"08BV3ZX"  =  "1"  /* Excision of Right Lacrimal Gland, Perc Approach, Diagn */
"08BV3ZZ"  =  "1"  /* Excision of Right Lacrimal Gland, Percutaneous Approach */
"08BW0ZX"  =  "1"  /* Excision of Left Lacrimal Gland, Open Approach, Diagnostic */
"08BW0ZZ"  =  "1"  /* Excision of Left Lacrimal Gland, Open Approach */
"08BW3ZX"  =  "1"  /* Excision of Left Lacrimal Gland, Perc Approach, Diagn */
"08BW3ZZ"  =  "1"  /* Excision of Left Lacrimal Gland, Percutaneous Approach */
"08BX0ZX"  =  "1"  /* Excision of Right Lacrimal Duct, Open Approach, Diagnostic */
"08BX0ZZ"  =  "1"  /* Excision of Right Lacrimal Duct, Open Approach */
"08BX3ZX"  =  "1"  /* Excision of Right Lacrimal Duct, Perc Approach, Diagn */
"08BX3ZZ"  =  "1"  /* Excision of Right Lacrimal Duct, Percutaneous Approach */
"08BX7ZX"  =  "1"  /* Excision of Right Lacrimal Duct, Via Opening, Diagn */
"08BX7ZZ"  =  "1"  /* Excision of Right Lacrimal Duct, Via Opening */
"08BX8ZX"  =  "1"  /* Excision of Right Lacrimal Duct, Endo, Diagn */
"08BX8ZZ"  =  "1"  /* Excision of Right Lacrimal Duct, Endo */
"08BY0ZX"  =  "1"  /* Excision of Left Lacrimal Duct, Open Approach, Diagnostic */
"08BY0ZZ"  =  "1"  /* Excision of Left Lacrimal Duct, Open Approach */
"08BY3ZX"  =  "1"  /* Excision of Left Lacrimal Duct, Percutaneous Approach, Diagn */
"08BY3ZZ"  =  "1"  /* Excision of Left Lacrimal Duct, Percutaneous Approach */
"08BY7ZX"  =  "1"  /* Excision of Left Lacrimal Duct, Via Opening, Diagn */
"08BY7ZZ"  =  "1"  /* Excision of Left Lacrimal Duct, Via Opening */
"08BY8ZX"  =  "1"  /* Excision of Left Lacrimal Duct, Endo, Diagn */
"08BY8ZZ"  =  "1"  /* Excision of Left Lacrimal Duct, Endo */
"08C0XZZ"  =  "1"  /* Extirpation of Matter from Right Eye, External Approach */
"08C1XZZ"  =  "1"  /* Extirpation of Matter from Left Eye, External Approach */
"08C23ZZ"  =  "1"  /* Extirpation of Matter from R Ant Chamber, Perc Approach */
"08C33ZZ"  =  "1"  /* Extirpation of Matter from L Ant Chamber, Perc Approach */
"08C43ZZ"  =  "1"  /* Extirpation of Matter from Right Vitreous, Perc Approach */
"08C4XZZ"  =  "1"  /* Extirpation of Matter from Right Vitreous, External Approach */
"08C53ZZ"  =  "1"  /* Extirpation of Matter from Left Vitreous, Perc Approach */
"08C5XZZ"  =  "1"  /* Extirpation of Matter from Left Vitreous, External Approach */
"08C8XZZ"  =  "1"  /* Extirpation of Matter from Right Cornea, External Approach */
"08C9XZZ"  =  "1"  /* Extirpation of Matter from Left Cornea, External Approach */
"08CA0ZZ"  =  "1"  /* Extirpation of Matter from Right Choroid, Open Approach */
"08CA3ZZ"  =  "1"  /* Extirpation of Matter from Right Choroid, Perc Approach */
"08CAXZZ"  =  "1"  /* Extirpation of Matter from Right Choroid, External Approach */
"08CB0ZZ"  =  "1"  /* Extirpation of Matter from Left Choroid, Open Approach */
"08CB3ZZ"  =  "1"  /* Extirpation of Matter from Left Choroid, Perc Approach */
"08CBXZZ"  =  "1"  /* Extirpation of Matter from Left Choroid, External Approach */
"08CC3ZZ"  =  "1"  /* Extirpation of Matter from Right Iris, Percutaneous Approach */
"08CCXZZ"  =  "1"  /* Extirpation of Matter from Right Iris, External Approach */
"08CD3ZZ"  =  "1"  /* Extirpation of Matter from Left Iris, Percutaneous Approach */
"08CDXZZ"  =  "1"  /* Extirpation of Matter from Left Iris, External Approach */
"08CE3ZZ"  =  "1"  /* Extirpation of Matter from Right Retina, Perc Approach */
"08CEXZZ"  =  "1"  /* Extirpation of Matter from Right Retina, External Approach */
"08CF3ZZ"  =  "1"  /* Extirpation of Matter from Left Retina, Perc Approach */
"08CFXZZ"  =  "1"  /* Extirpation of Matter from Left Retina, External Approach */
"08CG3ZZ"  =  "1"  /* Extirpation of Matter from R Retinal Vessel, Perc Approach */
"08CGXZZ"  =  "1"  /* Extirpation of Matter from R Retinal Vessel, Extern Approach */
"08CH3ZZ"  =  "1"  /* Extirpation of Matter from L Retinal Vessel, Perc Approach */
"08CHXZZ"  =  "1"  /* Extirpation of Matter from L Retinal Vessel, Extern Approach */
"08CJ3ZZ"  =  "1"  /* Extirpation of Matter from Right Lens, Percutaneous Approach */
"08CJXZZ"  =  "1"  /* Extirpation of Matter from Right Lens, External Approach */
"08CK3ZZ"  =  "1"  /* Extirpation of Matter from Left Lens, Percutaneous Approach */
"08CKXZZ"  =  "1"  /* Extirpation of Matter from Left Lens, External Approach */
"08CL0ZZ"  =  "1"  /* Extirpation of Matter from R Extraoc Muscle, Open Approach */
"08CL3ZZ"  =  "1"  /* Extirpation of Matter from R Extraoc Muscle, Perc Approach */
"08CLXZZ"  =  "1"  /* Extirpation of Matter from R Extraoc Muscle, Extern Approach */
"08CM0ZZ"  =  "1"  /* Extirpation of Matter from L Extraoc Muscle, Open Approach */
"08CM3ZZ"  =  "1"  /* Extirpation of Matter from L Extraoc Muscle, Perc Approach */
"08CMXZZ"  =  "1"  /* Extirpation of Matter from L Extraoc Muscle, Extern Approach */
"08CSXZZ"  =  "1"  /* Extirpation of Matter from R Conjunctiva, Extern Approach */
"08CTXZZ"  =  "1"  /* Extirpation of Matter from Left Conjunctiva, Extern Approach */
"08CV0ZZ"  =  "1"  /* Extirpation of Matter from R Lacrml Gland, Open Approach */
"08CV3ZZ"  =  "1"  /* Extirpation of Matter from R Lacrml Gland, Perc Approach */
"08CVXZZ"  =  "1"  /* Extirpation of Matter from R Lacrml Gland, Extern Approach */
"08CW0ZZ"  =  "1"  /* Extirpation of Matter from L Lacrml Gland, Open Approach */
"08CW3ZZ"  =  "1"  /* Extirpation of Matter from L Lacrml Gland, Perc Approach */
"08CWXZZ"  =  "1"  /* Extirpation of Matter from L Lacrml Gland, Extern Approach */
"08CX0ZZ"  =  "1"  /* Extirpation of Matter from R Lacrml Duct, Open Approach */
"08CX3ZZ"  =  "1"  /* Extirpation of Matter from R Lacrml Duct, Perc Approach */
"08CX7ZZ"  =  "1"  /* Extirpation of Matter from Right Lacrimal Duct, Via Opening */
"08CX8ZZ"  =  "1"  /* Extirpation of Matter from Right Lacrimal Duct, Endo */
"08CY0ZZ"  =  "1"  /* Extirpation of Matter from Left Lacrimal Duct, Open Approach */
"08CY3ZZ"  =  "1"  /* Extirpation of Matter from Left Lacrimal Duct, Perc Approach */
"08CY7ZZ"  =  "1"  /* Extirpation of Matter from Left Lacrimal Duct, Via Opening */
"08CY8ZZ"  =  "1"  /* Extirpation of Matter from Left Lacrimal Duct, Endo */
"08D8XZX"  =  "1"  /* Extraction of Right Cornea, External Approach, Diagnostic */
"08D8XZZ"  =  "1"  /* Extraction of Right Cornea, External Approach */
"08D9XZX"  =  "1"  /* Extraction of Left Cornea, External Approach, Diagnostic */
"08D9XZZ"  =  "1"  /* Extraction of Left Cornea, External Approach */
"08DJ3ZZ"  =  "1"  /* Extraction of Right Lens, Percutaneous Approach */
"08DK3ZZ"  =  "1"  /* Extraction of Left Lens, Percutaneous Approach */
"08F43ZZ"  =  "1"  /* Fragmentation in Right Vitreous, Percutaneous Approach */
"08F53ZZ"  =  "1"  /* Fragmentation in Left Vitreous, Percutaneous Approach */
"08H005Z"  =  "1"  /* Insertion of Epiret Prosth into R Eye, Open Approach */
"08H031Z"  =  "1"  /* Insertion of Radioactive Element into R Eye, Perc Approach */
"08H033Z"  =  "1"  /* Insertion of Infusion Device into Right Eye, Perc Approach */
"08H0X1Z"  =  "1"  /* Insertion of Radioactive Element into R Eye, Extern Approach */
"08H0X3Z"  =  "1"  /* Insertion of Infusion Device into Right Eye, Extern Approach */
"08H105Z"  =  "1"  /* Insertion of Epiret Prosth into L Eye, Open Approach */
"08H131Z"  =  "1"  /* Insertion of Radioactive Element into L Eye, Perc Approach */
"08H133Z"  =  "1"  /* Insertion of Infusion Device into Left Eye, Perc Approach */
"08H1X1Z"  =  "1"  /* Insertion of Radioactive Element into L Eye, Extern Approach */
"08H1X3Z"  =  "1"  /* Insertion of Infusion Device into Left Eye, Extern Approach */
"08J0XZZ"  =  "1"  /* Inspection of Right Eye, External Approach */
"08J1XZZ"  =  "1"  /* Inspection of Left Eye, External Approach */
"08JJXZZ"  =  "1"  /* Inspection of Right Lens, External Approach */
"08JKXZZ"  =  "1"  /* Inspection of Left Lens, External Approach */
"08JL0ZZ"  =  "1"  /* Inspection of Right Extraocular Muscle, Open Approach */
"08JLXZZ"  =  "1"  /* Inspection of Right Extraocular Muscle, External Approach */
"08JM0ZZ"  =  "1"  /* Inspection of Left Extraocular Muscle, Open Approach */
"08JMXZZ"  =  "1"  /* Inspection of Left Extraocular Muscle, External Approach */
"08LX0CZ"  =  "1"  /* Occlusion of R Lacrml Duct with Extralum Dev, Open Approach */
"08LX0DZ"  =  "1"  /* Occlusion of R Lacrml Duct with Intralum Dev, Open Approach */
"08LX0ZZ"  =  "1"  /* Occlusion of Right Lacrimal Duct, Open Approach */
"08LX3CZ"  =  "1"  /* Occlusion of R Lacrml Duct with Extralum Dev, Perc Approach */
"08LX3DZ"  =  "1"  /* Occlusion of R Lacrml Duct with Intralum Dev, Perc Approach */
"08LX3ZZ"  =  "1"  /* Occlusion of Right Lacrimal Duct, Percutaneous Approach */
"08LX7DZ"  =  "1"  /* Occlusion of R Lacrml Duct with Intralum Dev, Via Opening */
"08LX7ZZ"  =  "1"  /* Occlusion of Right Lacrimal Duct, Via Opening */
"08LX8DZ"  =  "1"  /* Occlusion of Right Lacrimal Duct with Intralum Dev, Endo */
"08LX8ZZ"  =  "1"  /* Occlusion of Right Lacrimal Duct, Endo */
"08LY0CZ"  =  "1"  /* Occlusion of L Lacrml Duct with Extralum Dev, Open Approach */
"08LY0DZ"  =  "1"  /* Occlusion of L Lacrml Duct with Intralum Dev, Open Approach */
"08LY0ZZ"  =  "1"  /* Occlusion of Left Lacrimal Duct, Open Approach */
"08LY3CZ"  =  "1"  /* Occlusion of L Lacrml Duct with Extralum Dev, Perc Approach */
"08LY3DZ"  =  "1"  /* Occlusion of L Lacrml Duct with Intralum Dev, Perc Approach */
"08LY3ZZ"  =  "1"  /* Occlusion of Left Lacrimal Duct, Percutaneous Approach */
"08LY7DZ"  =  "1"  /* Occlusion of L Lacrml Duct with Intralum Dev, Via Opening */
"08LY7ZZ"  =  "1"  /* Occlusion of Left Lacrimal Duct, Via Opening */
"08LY8DZ"  =  "1"  /* Occlusion of Left Lacrimal Duct with Intralum Dev, Endo */
"08LY8ZZ"  =  "1"  /* Occlusion of Left Lacrimal Duct, Endo */
"08MNXZZ"  =  "1"  /* Reattachment of Right Upper Eyelid, External Approach */
"08MPXZZ"  =  "1"  /* Reattachment of Left Upper Eyelid, External Approach */
"08MQXZZ"  =  "1"  /* Reattachment of Right Lower Eyelid, External Approach */
"08MRXZZ"  =  "1"  /* Reattachment of Left Lower Eyelid, External Approach */
"08N0XZZ"  =  "1"  /* Release Right Eye, External Approach */
"08N1XZZ"  =  "1"  /* Release Left Eye, External Approach */
"08N23ZZ"  =  "1"  /* Release Right Anterior Chamber, Percutaneous Approach */
"08N33ZZ"  =  "1"  /* Release Left Anterior Chamber, Percutaneous Approach */
"08N43ZZ"  =  "1"  /* Release Right Vitreous, Percutaneous Approach */
"08N53ZZ"  =  "1"  /* Release Left Vitreous, Percutaneous Approach */
"08N6XZZ"  =  "1"  /* Release Right Sclera, External Approach */
"08N7XZZ"  =  "1"  /* Release Left Sclera, External Approach */
"08N8XZZ"  =  "1"  /* Release Right Cornea, External Approach */
"08N9XZZ"  =  "1"  /* Release Left Cornea, External Approach */
"08NA0ZZ"  =  "1"  /* Release Right Choroid, Open Approach */
"08NA3ZZ"  =  "1"  /* Release Right Choroid, Percutaneous Approach */
"08NB0ZZ"  =  "1"  /* Release Left Choroid, Open Approach */
"08NB3ZZ"  =  "1"  /* Release Left Choroid, Percutaneous Approach */
"08NC3ZZ"  =  "1"  /* Release Right Iris, Percutaneous Approach */
"08ND3ZZ"  =  "1"  /* Release Left Iris, Percutaneous Approach */
"08NE3ZZ"  =  "1"  /* Release Right Retina, Percutaneous Approach */
"08NF3ZZ"  =  "1"  /* Release Left Retina, Percutaneous Approach */
"08NG3ZZ"  =  "1"  /* Release Right Retinal Vessel, Percutaneous Approach */
"08NH3ZZ"  =  "1"  /* Release Left Retinal Vessel, Percutaneous Approach */
"08NJ3ZZ"  =  "1"  /* Release Right Lens, Percutaneous Approach */
"08NK3ZZ"  =  "1"  /* Release Left Lens, Percutaneous Approach */
"08NL0ZZ"  =  "1"  /* Release Right Extraocular Muscle, Open Approach */
"08NL3ZZ"  =  "1"  /* Release Right Extraocular Muscle, Percutaneous Approach */
"08NM0ZZ"  =  "1"  /* Release Left Extraocular Muscle, Open Approach */
"08NM3ZZ"  =  "1"  /* Release Left Extraocular Muscle, Percutaneous Approach */
"08NN0ZZ"  =  "1"  /* Release Right Upper Eyelid, Open Approach */
"08NN3ZZ"  =  "1"  /* Release Right Upper Eyelid, Percutaneous Approach */
"08NNXZZ"  =  "1"  /* Release Right Upper Eyelid, External Approach */
"08NP0ZZ"  =  "1"  /* Release Left Upper Eyelid, Open Approach */
"08NP3ZZ"  =  "1"  /* Release Left Upper Eyelid, Percutaneous Approach */
"08NPXZZ"  =  "1"  /* Release Left Upper Eyelid, External Approach */
"08NQ0ZZ"  =  "1"  /* Release Right Lower Eyelid, Open Approach */
"08NQ3ZZ"  =  "1"  /* Release Right Lower Eyelid, Percutaneous Approach */
"08NQXZZ"  =  "1"  /* Release Right Lower Eyelid, External Approach */
"08NR0ZZ"  =  "1"  /* Release Left Lower Eyelid, Open Approach */
"08NR3ZZ"  =  "1"  /* Release Left Lower Eyelid, Percutaneous Approach */
"08NRXZZ"  =  "1"  /* Release Left Lower Eyelid, External Approach */
"08NSXZZ"  =  "1"  /* Release Right Conjunctiva, External Approach */
"08NTXZZ"  =  "1"  /* Release Left Conjunctiva, External Approach */
"08NV0ZZ"  =  "1"  /* Release Right Lacrimal Gland, Open Approach */
"08NV3ZZ"  =  "1"  /* Release Right Lacrimal Gland, Percutaneous Approach */
"08NW0ZZ"  =  "1"  /* Release Left Lacrimal Gland, Open Approach */
"08NW3ZZ"  =  "1"  /* Release Left Lacrimal Gland, Percutaneous Approach */
"08NX0ZZ"  =  "1"  /* Release Right Lacrimal Duct, Open Approach */
"08NX3ZZ"  =  "1"  /* Release Right Lacrimal Duct, Percutaneous Approach */
"08NX7ZZ"  =  "1"  /* Release Right Lacrimal Duct, Via Opening */
"08NX8ZZ"  =  "1"  /* Release Right Lacrimal Duct, Endo */
"08NY0ZZ"  =  "1"  /* Release Left Lacrimal Duct, Open Approach */
"08NY3ZZ"  =  "1"  /* Release Left Lacrimal Duct, Percutaneous Approach */
"08NY7ZZ"  =  "1"  /* Release Left Lacrimal Duct, Via Opening */
"08NY8ZZ"  =  "1"  /* Release Left Lacrimal Duct, Endo */
"08P000Z"  =  "1"  /* Removal of Drainage Device from Right Eye, Open Approach */
"08P001Z"  =  "1"  /* Removal of Radioactive Element from Right Eye, Open Approach */
"08P003Z"  =  "1"  /* Removal of Infusion Device from Right Eye, Open Approach */
"08P007Z"  =  "1"  /* Removal of Autol Sub from R Eye, Open Approach */
"08P00CZ"  =  "1"  /* Removal of Extraluminal Device from Right Eye, Open Approach */
"08P00DZ"  =  "1"  /* Removal of Intraluminal Device from Right Eye, Open Approach */
"08P00JZ"  =  "1"  /* Removal of Synthetic Substitute from R Eye, Open Approach */
"08P00KZ"  =  "1"  /* Removal of Nonaut Sub from R Eye, Open Approach */
"08P030Z"  =  "1"  /* Removal of Drainage Device from Right Eye, Perc Approach */
"08P031Z"  =  "1"  /* Removal of Radioactive Element from Right Eye, Perc Approach */
"08P033Z"  =  "1"  /* Removal of Infusion Device from Right Eye, Perc Approach */
"08P037Z"  =  "1"  /* Removal of Autol Sub from R Eye, Perc Approach */
"08P03CZ"  =  "1"  /* Removal of Extraluminal Device from Right Eye, Perc Approach */
"08P03DZ"  =  "1"  /* Removal of Intraluminal Device from Right Eye, Perc Approach */
"08P03JZ"  =  "1"  /* Removal of Synthetic Substitute from R Eye, Perc Approach */
"08P03KZ"  =  "1"  /* Removal of Nonaut Sub from R Eye, Perc Approach */
"08P070Z"  =  "1"  /* Removal of Drainage Device from Right Eye, Via Opening */
"08P071Z"  =  "1"  /* Removal of Radioactive Element from Right Eye, Via Opening */
"08P073Z"  =  "1"  /* Removal of Infusion Device from Right Eye, Via Opening */
"08P077Z"  =  "1"  /* Removal of Autol Sub from R Eye, Via Opening */
"08P07CZ"  =  "1"  /* Removal of Extraluminal Device from Right Eye, Via Opening */
"08P07DZ"  =  "1"  /* Removal of Intraluminal Device from Right Eye, Via Opening */
"08P07JZ"  =  "1"  /* Removal of Synthetic Substitute from Right Eye, Via Opening */
"08P07KZ"  =  "1"  /* Removal of Nonaut Sub from R Eye, Via Opening */
"08P080Z"  =  "1"  /* Removal of Drainage Device from Right Eye, Endo */
"08P081Z"  =  "1"  /* Removal of Radioactive Element from Right Eye, Endo */
"08P083Z"  =  "1"  /* Removal of Infusion Device from Right Eye, Endo */
"08P087Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Right Eye, Endo */
"08P08CZ"  =  "1"  /* Removal of Extraluminal Device from Right Eye, Endo */
"08P08DZ"  =  "1"  /* Removal of Intraluminal Device from Right Eye, Endo */
"08P08JZ"  =  "1"  /* Removal of Synthetic Substitute from Right Eye, Endo */
"08P08KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from R Eye, Endo */
"08P0X1Z"  =  "1"  /* Removal of Radioactive Element from R Eye, Extern Approach */
"08P0X7Z"  =  "1"  /* Removal of Autol Sub from R Eye, Extern Approach */
"08P0XJZ"  =  "1"  /* Removal of Synthetic Substitute from R Eye, Extern Approach */
"08P0XKZ"  =  "1"  /* Removal of Nonaut Sub from R Eye, Extern Approach */
"08P100Z"  =  "1"  /* Removal of Drainage Device from Left Eye, Open Approach */
"08P101Z"  =  "1"  /* Removal of Radioactive Element from Left Eye, Open Approach */
"08P103Z"  =  "1"  /* Removal of Infusion Device from Left Eye, Open Approach */
"08P107Z"  =  "1"  /* Removal of Autol Sub from L Eye, Open Approach */
"08P10CZ"  =  "1"  /* Removal of Extraluminal Device from Left Eye, Open Approach */
"08P10DZ"  =  "1"  /* Removal of Intraluminal Device from Left Eye, Open Approach */
"08P10JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Eye, Open Approach */
"08P10KZ"  =  "1"  /* Removal of Nonaut Sub from L Eye, Open Approach */
"08P130Z"  =  "1"  /* Removal of Drainage Device from Left Eye, Perc Approach */
"08P131Z"  =  "1"  /* Removal of Radioactive Element from Left Eye, Perc Approach */
"08P133Z"  =  "1"  /* Removal of Infusion Device from Left Eye, Perc Approach */
"08P137Z"  =  "1"  /* Removal of Autol Sub from L Eye, Perc Approach */
"08P13CZ"  =  "1"  /* Removal of Extraluminal Device from Left Eye, Perc Approach */
"08P13DZ"  =  "1"  /* Removal of Intraluminal Device from Left Eye, Perc Approach */
"08P13JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Eye, Perc Approach */
"08P13KZ"  =  "1"  /* Removal of Nonaut Sub from L Eye, Perc Approach */
"08P170Z"  =  "1"  /* Removal of Drainage Device from Left Eye, Via Opening */
"08P171Z"  =  "1"  /* Removal of Radioactive Element from Left Eye, Via Opening */
"08P173Z"  =  "1"  /* Removal of Infusion Device from Left Eye, Via Opening */
"08P177Z"  =  "1"  /* Removal of Autol Sub from L Eye, Via Opening */
"08P17CZ"  =  "1"  /* Removal of Extraluminal Device from Left Eye, Via Opening */
"08P17DZ"  =  "1"  /* Removal of Intraluminal Device from Left Eye, Via Opening */
"08P17JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Eye, Via Opening */
"08P17KZ"  =  "1"  /* Removal of Nonaut Sub from L Eye, Via Opening */
"08P180Z"  =  "1"  /* Removal of Drainage Device from Left Eye, Endo */
"08P181Z"  =  "1"  /* Removal of Radioactive Element from Left Eye, Endo */
"08P183Z"  =  "1"  /* Removal of Infusion Device from Left Eye, Endo */
"08P187Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Left Eye, Endo */
"08P18CZ"  =  "1"  /* Removal of Extraluminal Device from Left Eye, Endo */
"08P18DZ"  =  "1"  /* Removal of Intraluminal Device from Left Eye, Endo */
"08P18JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Eye, Endo */
"08P18KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from L Eye, Endo */
"08P1X7Z"  =  "1"  /* Removal of Autol Sub from L Eye, Extern Approach */
"08P1XJZ"  =  "1"  /* Removal of Synthetic Substitute from L Eye, Extern Approach */
"08P1XKZ"  =  "1"  /* Removal of Nonaut Sub from L Eye, Extern Approach */
"08PJ3JZ"  =  "1"  /* Removal of Synthetic Substitute from R Lens, Perc Approach */
"08PK3JZ"  =  "1"  /* Removal of Synthetic Substitute from L Lens, Perc Approach */
"08PL00Z"  =  "1"  /* Removal of Drain Dev from R Extraoc Muscle, Open Approach */
"08PL07Z"  =  "1"  /* Removal of Autol Sub from R Extraoc Muscle, Open Approach */
"08PL0JZ"  =  "1"  /* Removal of Synth Sub from R Extraoc Muscle, Open Approach */
"08PL0KZ"  =  "1"  /* Removal of Nonaut Sub from R Extraoc Muscle, Open Approach */
"08PL30Z"  =  "1"  /* Removal of Drain Dev from R Extraoc Muscle, Perc Approach */
"08PL37Z"  =  "1"  /* Removal of Autol Sub from R Extraoc Muscle, Perc Approach */
"08PL3JZ"  =  "1"  /* Removal of Synth Sub from R Extraoc Muscle, Perc Approach */
"08PL3KZ"  =  "1"  /* Removal of Nonaut Sub from R Extraoc Muscle, Perc Approach */
"08PM00Z"  =  "1"  /* Removal of Drain Dev from L Extraoc Muscle, Open Approach */
"08PM07Z"  =  "1"  /* Removal of Autol Sub from L Extraoc Muscle, Open Approach */
"08PM0JZ"  =  "1"  /* Removal of Synth Sub from L Extraoc Muscle, Open Approach */
"08PM0KZ"  =  "1"  /* Removal of Nonaut Sub from L Extraoc Muscle, Open Approach */
"08PM30Z"  =  "1"  /* Removal of Drain Dev from L Extraoc Muscle, Perc Approach */
"08PM37Z"  =  "1"  /* Removal of Autol Sub from L Extraoc Muscle, Perc Approach */
"08PM3JZ"  =  "1"  /* Removal of Synth Sub from L Extraoc Muscle, Perc Approach */
"08PM3KZ"  =  "1"  /* Removal of Nonaut Sub from L Extraoc Muscle, Perc Approach */
"08Q0XZZ"  =  "1"  /* Repair Right Eye, External Approach */
"08Q1XZZ"  =  "1"  /* Repair Left Eye, External Approach */
"08Q23ZZ"  =  "1"  /* Repair Right Anterior Chamber, Percutaneous Approach */
"08Q33ZZ"  =  "1"  /* Repair Left Anterior Chamber, Percutaneous Approach */
"08Q43ZZ"  =  "1"  /* Repair Right Vitreous, Percutaneous Approach */
"08Q53ZZ"  =  "1"  /* Repair Left Vitreous, Percutaneous Approach */
"08Q6XZZ"  =  "1"  /* Repair Right Sclera, External Approach */
"08Q7XZZ"  =  "1"  /* Repair Left Sclera, External Approach */
"08Q8XZZ"  =  "1"  /* Repair Right Cornea, External Approach */
"08Q9XZZ"  =  "1"  /* Repair Left Cornea, External Approach */
"08QA0ZZ"  =  "1"  /* Repair Right Choroid, Open Approach */
"08QA3ZZ"  =  "1"  /* Repair Right Choroid, Percutaneous Approach */
"08QB0ZZ"  =  "1"  /* Repair Left Choroid, Open Approach */
"08QB3ZZ"  =  "1"  /* Repair Left Choroid, Percutaneous Approach */
"08QC3ZZ"  =  "1"  /* Repair Right Iris, Percutaneous Approach */
"08QD3ZZ"  =  "1"  /* Repair Left Iris, Percutaneous Approach */
"08QE3ZZ"  =  "1"  /* Repair Right Retina, Percutaneous Approach */
"08QF3ZZ"  =  "1"  /* Repair Left Retina, Percutaneous Approach */
"08QG3ZZ"  =  "1"  /* Repair Right Retinal Vessel, Percutaneous Approach */
"08QH3ZZ"  =  "1"  /* Repair Left Retinal Vessel, Percutaneous Approach */
"08QJ3ZZ"  =  "1"  /* Repair Right Lens, Percutaneous Approach */
"08QK3ZZ"  =  "1"  /* Repair Left Lens, Percutaneous Approach */
"08QL0ZZ"  =  "1"  /* Repair Right Extraocular Muscle, Open Approach */
"08QL3ZZ"  =  "1"  /* Repair Right Extraocular Muscle, Percutaneous Approach */
"08QM0ZZ"  =  "1"  /* Repair Left Extraocular Muscle, Open Approach */
"08QM3ZZ"  =  "1"  /* Repair Left Extraocular Muscle, Percutaneous Approach */
"08QSXZZ"  =  "1"  /* Repair Right Conjunctiva, External Approach */
"08QTXZZ"  =  "1"  /* Repair Left Conjunctiva, External Approach */
"08QV0ZZ"  =  "1"  /* Repair Right Lacrimal Gland, Open Approach */
"08QV3ZZ"  =  "1"  /* Repair Right Lacrimal Gland, Percutaneous Approach */
"08QW0ZZ"  =  "1"  /* Repair Left Lacrimal Gland, Open Approach */
"08QW3ZZ"  =  "1"  /* Repair Left Lacrimal Gland, Percutaneous Approach */
"08QX0ZZ"  =  "1"  /* Repair Right Lacrimal Duct, Open Approach */
"08QX3ZZ"  =  "1"  /* Repair Right Lacrimal Duct, Percutaneous Approach */
"08QX7ZZ"  =  "1"  /* Repair Right Lacrimal Duct, Via Opening */
"08QX8ZZ"  =  "1"  /* Repair Right Lacrimal Duct, Endo */
"08QY0ZZ"  =  "1"  /* Repair Left Lacrimal Duct, Open Approach */
"08QY3ZZ"  =  "1"  /* Repair Left Lacrimal Duct, Percutaneous Approach */
"08QY7ZZ"  =  "1"  /* Repair Left Lacrimal Duct, Via Natural or Artificial Opening */
"08QY8ZZ"  =  "1"  /* Repair Left Lacrimal Duct, Endo */
"08R007Z"  =  "1"  /* Replacement of Right Eye with Autol Sub, Open Approach */
"08R00JZ"  =  "1"  /* Replacement of Right Eye with Synth Sub, Open Approach */
"08R00KZ"  =  "1"  /* Replacement of Right Eye with Nonaut Sub, Open Approach */
"08R037Z"  =  "1"  /* Replacement of Right Eye with Autol Sub, Perc Approach */
"08R03JZ"  =  "1"  /* Replacement of Right Eye with Synth Sub, Perc Approach */
"08R03KZ"  =  "1"  /* Replacement of Right Eye with Nonaut Sub, Perc Approach */
"08R107Z"  =  "1"  /* Replacement of Left Eye with Autol Sub, Open Approach */
"08R10JZ"  =  "1"  /* Replacement of Left Eye with Synth Sub, Open Approach */
"08R10KZ"  =  "1"  /* Replacement of Left Eye with Nonaut Sub, Open Approach */
"08R137Z"  =  "1"  /* Replacement of Left Eye with Autol Sub, Perc Approach */
"08R13JZ"  =  "1"  /* Replacement of Left Eye with Synth Sub, Perc Approach */
"08R13KZ"  =  "1"  /* Replacement of Left Eye with Nonaut Sub, Perc Approach */
"08R437Z"  =  "1"  /* Replacement of Right Vitreous with Autol Sub, Perc Approach */
"08R43JZ"  =  "1"  /* Replacement of Right Vitreous with Synth Sub, Perc Approach */
"08R43KZ"  =  "1"  /* Replacement of Right Vitreous with Nonaut Sub, Perc Approach */
"08R537Z"  =  "1"  /* Replacement of Left Vitreous with Autol Sub, Perc Approach */
"08R53JZ"  =  "1"  /* Replacement of Left Vitreous with Synth Sub, Perc Approach */
"08R53KZ"  =  "1"  /* Replacement of Left Vitreous with Nonaut Sub, Perc Approach */
"08R6X7Z"  =  "1"  /* Replacement of Right Sclera with Autol Sub, Extern Approach */
"08R6XJZ"  =  "1"  /* Replacement of Right Sclera with Synth Sub, Extern Approach */
"08R6XKZ"  =  "1"  /* Replacement of Right Sclera with Nonaut Sub, Extern Approach */
"08R7X7Z"  =  "1"  /* Replacement of Left Sclera with Autol Sub, Extern Approach */
"08R7XJZ"  =  "1"  /* Replacement of Left Sclera with Synth Sub, Extern Approach */
"08R7XKZ"  =  "1"  /* Replacement of Left Sclera with Nonaut Sub, Extern Approach */
"08R837Z"  =  "1"  /* Replacement of Right Cornea with Autol Sub, Perc Approach */
"08R83JZ"  =  "1"  /* Replacement of Right Cornea with Synth Sub, Perc Approach */
"08R83KZ"  =  "1"  /* Replacement of Right Cornea with Nonaut Sub, Perc Approach */
"08R8X7Z"  =  "1"  /* Replacement of Right Cornea with Autol Sub, Extern Approach */
"08R8XJZ"  =  "1"  /* Replacement of Right Cornea with Synth Sub, Extern Approach */
"08R8XKZ"  =  "1"  /* Replacement of Right Cornea with Nonaut Sub, Extern Approach */
"08R937Z"  =  "1"  /* Replacement of Left Cornea with Autol Sub, Perc Approach */
"08R93JZ"  =  "1"  /* Replacement of Left Cornea with Synth Sub, Perc Approach */
"08R93KZ"  =  "1"  /* Replacement of Left Cornea with Nonaut Sub, Perc Approach */
"08R9X7Z"  =  "1"  /* Replacement of Left Cornea with Autol Sub, Extern Approach */
"08R9XJZ"  =  "1"  /* Replacement of Left Cornea with Synth Sub, Extern Approach */
"08R9XKZ"  =  "1"  /* Replacement of Left Cornea with Nonaut Sub, Extern Approach */
"08RA07Z"  =  "1"  /* Replacement of Right Choroid with Autol Sub, Open Approach */
"08RA0JZ"  =  "1"  /* Replacement of Right Choroid with Synth Sub, Open Approach */
"08RA0KZ"  =  "1"  /* Replacement of Right Choroid with Nonaut Sub, Open Approach */
"08RA37Z"  =  "1"  /* Replacement of Right Choroid with Autol Sub, Perc Approach */
"08RA3JZ"  =  "1"  /* Replacement of Right Choroid with Synth Sub, Perc Approach */
"08RA3KZ"  =  "1"  /* Replacement of Right Choroid with Nonaut Sub, Perc Approach */
"08RB07Z"  =  "1"  /* Replacement of Left Choroid with Autol Sub, Open Approach */
"08RB0JZ"  =  "1"  /* Replacement of Left Choroid with Synth Sub, Open Approach */
"08RB0KZ"  =  "1"  /* Replacement of Left Choroid with Nonaut Sub, Open Approach */
"08RB37Z"  =  "1"  /* Replacement of Left Choroid with Autol Sub, Perc Approach */
"08RB3JZ"  =  "1"  /* Replacement of Left Choroid with Synth Sub, Perc Approach */
"08RB3KZ"  =  "1"  /* Replacement of Left Choroid with Nonaut Sub, Perc Approach */
"08RC37Z"  =  "1"  /* Replacement of Right Iris with Autol Sub, Perc Approach */
"08RC3JZ"  =  "1"  /* Replacement of Right Iris with Synth Sub, Perc Approach */
"08RC3KZ"  =  "1"  /* Replacement of Right Iris with Nonaut Sub, Perc Approach */
"08RD37Z"  =  "1"  /* Replacement of Left Iris with Autol Sub, Perc Approach */
"08RD3JZ"  =  "1"  /* Replacement of Left Iris with Synth Sub, Perc Approach */
"08RD3KZ"  =  "1"  /* Replacement of Left Iris with Nonaut Sub, Perc Approach */
"08RG37Z"  =  "1"  /* Replace of R Retinal Vessel with Autol Sub, Perc Approach */
"08RG3JZ"  =  "1"  /* Replace of R Retinal Vessel with Synth Sub, Perc Approach */
"08RG3KZ"  =  "1"  /* Replace of R Retinal Vessel with Nonaut Sub, Perc Approach */
"08RH37Z"  =  "1"  /* Replace of L Retinal Vessel with Autol Sub, Perc Approach */
"08RH3JZ"  =  "1"  /* Replace of L Retinal Vessel with Synth Sub, Perc Approach */
"08RH3KZ"  =  "1"  /* Replace of L Retinal Vessel with Nonaut Sub, Perc Approach */
"08RJ30Z"  =  "1"  /* Replacement of R Lens with Intraoc Telescp, Perc Approach */
"08RJ37Z"  =  "1"  /* Replacement of Right Lens with Autol Sub, Perc Approach */
"08RJ3JZ"  =  "1"  /* Replacement of Right Lens with Synth Sub, Perc Approach */
"08RJ3KZ"  =  "1"  /* Replacement of Right Lens with Nonaut Sub, Perc Approach */
"08RK30Z"  =  "1"  /* Replacement of Left Lens with Intraoc Telescp, Perc Approach */
"08RK37Z"  =  "1"  /* Replacement of Left Lens with Autol Sub, Perc Approach */
"08RK3JZ"  =  "1"  /* Replacement of Left Lens with Synth Sub, Perc Approach */
"08RK3KZ"  =  "1"  /* Replacement of Left Lens with Nonaut Sub, Perc Approach */
"08RN07Z"  =  "1"  /* Replacement of R Up Eyelid with Autol Sub, Open Approach */
"08RN0JZ"  =  "1"  /* Replacement of R Up Eyelid with Synth Sub, Open Approach */
"08RN0KZ"  =  "1"  /* Replacement of R Up Eyelid with Nonaut Sub, Open Approach */
"08RN37Z"  =  "1"  /* Replacement of R Up Eyelid with Autol Sub, Perc Approach */
"08RN3JZ"  =  "1"  /* Replacement of R Up Eyelid with Synth Sub, Perc Approach */
"08RN3KZ"  =  "1"  /* Replacement of R Up Eyelid with Nonaut Sub, Perc Approach */
"08RNX7Z"  =  "1"  /* Replacement of R Up Eyelid with Autol Sub, Extern Approach */
"08RNXJZ"  =  "1"  /* Replacement of R Up Eyelid with Synth Sub, Extern Approach */
"08RNXKZ"  =  "1"  /* Replacement of R Up Eyelid with Nonaut Sub, Extern Approach */
"08RP07Z"  =  "1"  /* Replacement of L Up Eyelid with Autol Sub, Open Approach */
"08RP0JZ"  =  "1"  /* Replacement of L Up Eyelid with Synth Sub, Open Approach */
"08RP0KZ"  =  "1"  /* Replacement of L Up Eyelid with Nonaut Sub, Open Approach */
"08RP37Z"  =  "1"  /* Replacement of L Up Eyelid with Autol Sub, Perc Approach */
"08RP3JZ"  =  "1"  /* Replacement of L Up Eyelid with Synth Sub, Perc Approach */
"08RP3KZ"  =  "1"  /* Replacement of L Up Eyelid with Nonaut Sub, Perc Approach */
"08RPX7Z"  =  "1"  /* Replacement of L Up Eyelid with Autol Sub, Extern Approach */
"08RPXJZ"  =  "1"  /* Replacement of L Up Eyelid with Synth Sub, Extern Approach */
"08RPXKZ"  =  "1"  /* Replacement of L Up Eyelid with Nonaut Sub, Extern Approach */
"08RQ07Z"  =  "1"  /* Replacement of R Low Eyelid with Autol Sub, Open Approach */
"08RQ0JZ"  =  "1"  /* Replacement of R Low Eyelid with Synth Sub, Open Approach */
"08RQ0KZ"  =  "1"  /* Replacement of R Low Eyelid with Nonaut Sub, Open Approach */
"08RQ37Z"  =  "1"  /* Replacement of R Low Eyelid with Autol Sub, Perc Approach */
"08RQ3JZ"  =  "1"  /* Replacement of R Low Eyelid with Synth Sub, Perc Approach */
"08RQ3KZ"  =  "1"  /* Replacement of R Low Eyelid with Nonaut Sub, Perc Approach */
"08RQX7Z"  =  "1"  /* Replacement of R Low Eyelid with Autol Sub, Extern Approach */
"08RQXJZ"  =  "1"  /* Replacement of R Low Eyelid with Synth Sub, Extern Approach */
"08RQXKZ"  =  "1"  /* Replacement of R Low Eyelid with Nonaut Sub, Extern Approach */
"08RR07Z"  =  "1"  /* Replacement of L Low Eyelid with Autol Sub, Open Approach */
"08RR0JZ"  =  "1"  /* Replacement of L Low Eyelid with Synth Sub, Open Approach */
"08RR0KZ"  =  "1"  /* Replacement of L Low Eyelid with Nonaut Sub, Open Approach */
"08RR37Z"  =  "1"  /* Replacement of L Low Eyelid with Autol Sub, Perc Approach */
"08RR3JZ"  =  "1"  /* Replacement of L Low Eyelid with Synth Sub, Perc Approach */
"08RR3KZ"  =  "1"  /* Replacement of L Low Eyelid with Nonaut Sub, Perc Approach */
"08RRX7Z"  =  "1"  /* Replacement of L Low Eyelid with Autol Sub, Extern Approach */
"08RRXJZ"  =  "1"  /* Replacement of L Low Eyelid with Synth Sub, Extern Approach */
"08RRXKZ"  =  "1"  /* Replacement of L Low Eyelid with Nonaut Sub, Extern Approach */
"08RSX7Z"  =  "1"  /* Replacement of R Conjunctiva with Autol Sub, Extern Approach */
"08RSXJZ"  =  "1"  /* Replacement of R Conjunctiva with Synth Sub, Extern Approach */
"08RSXKZ"  =  "1"  /* Replace of R Conjunctiva with Nonaut Sub, Extern Approach */
"08RTX7Z"  =  "1"  /* Replacement of L Conjunctiva with Autol Sub, Extern Approach */
"08RTXJZ"  =  "1"  /* Replacement of L Conjunctiva with Synth Sub, Extern Approach */
"08RTXKZ"  =  "1"  /* Replace of L Conjunctiva with Nonaut Sub, Extern Approach */
"08RX07Z"  =  "1"  /* Replacement of R Lacrml Duct with Autol Sub, Open Approach */
"08RX0JZ"  =  "1"  /* Replacement of R Lacrml Duct with Synth Sub, Open Approach */
"08RX0KZ"  =  "1"  /* Replacement of R Lacrml Duct with Nonaut Sub, Open Approach */
"08RX37Z"  =  "1"  /* Replacement of R Lacrml Duct with Autol Sub, Perc Approach */
"08RX3JZ"  =  "1"  /* Replacement of R Lacrml Duct with Synth Sub, Perc Approach */
"08RX3KZ"  =  "1"  /* Replacement of R Lacrml Duct with Nonaut Sub, Perc Approach */
"08RX77Z"  =  "1"  /* Replacement of R Lacrml Duct with Autol Sub, Via Opening */
"08RX7JZ"  =  "1"  /* Replacement of R Lacrml Duct with Synth Sub, Via Opening */
"08RX7KZ"  =  "1"  /* Replacement of R Lacrml Duct with Nonaut Sub, Via Opening */
"08RX87Z"  =  "1"  /* Replacement of Right Lacrimal Duct with Autol Sub, Endo */
"08RX8JZ"  =  "1"  /* Replacement of Right Lacrimal Duct with Synth Sub, Endo */
"08RX8KZ"  =  "1"  /* Replacement of Right Lacrimal Duct with Nonaut Sub, Endo */
"08RY07Z"  =  "1"  /* Replacement of L Lacrml Duct with Autol Sub, Open Approach */
"08RY0JZ"  =  "1"  /* Replacement of L Lacrml Duct with Synth Sub, Open Approach */
"08RY0KZ"  =  "1"  /* Replacement of L Lacrml Duct with Nonaut Sub, Open Approach */
"08RY37Z"  =  "1"  /* Replacement of L Lacrml Duct with Autol Sub, Perc Approach */
"08RY3JZ"  =  "1"  /* Replacement of L Lacrml Duct with Synth Sub, Perc Approach */
"08RY3KZ"  =  "1"  /* Replacement of L Lacrml Duct with Nonaut Sub, Perc Approach */
"08RY77Z"  =  "1"  /* Replacement of L Lacrml Duct with Autol Sub, Via Opening */
"08RY7JZ"  =  "1"  /* Replacement of L Lacrml Duct with Synth Sub, Via Opening */
"08RY7KZ"  =  "1"  /* Replacement of L Lacrml Duct with Nonaut Sub, Via Opening */
"08RY87Z"  =  "1"  /* Replacement of Left Lacrimal Duct with Autol Sub, Endo */
"08RY8JZ"  =  "1"  /* Replacement of Left Lacrimal Duct with Synth Sub, Endo */
"08RY8KZ"  =  "1"  /* Replacement of Left Lacrimal Duct with Nonaut Sub, Endo */
"08SC3ZZ"  =  "1"  /* Reposition Right Iris, Percutaneous Approach */
"08SD3ZZ"  =  "1"  /* Reposition Left Iris, Percutaneous Approach */
"08SG3ZZ"  =  "1"  /* Reposition Right Retinal Vessel, Percutaneous Approach */
"08SH3ZZ"  =  "1"  /* Reposition Left Retinal Vessel, Percutaneous Approach */
"08SJ3ZZ"  =  "1"  /* Reposition Right Lens, Percutaneous Approach */
"08SK3ZZ"  =  "1"  /* Reposition Left Lens, Percutaneous Approach */
"08SL0ZZ"  =  "1"  /* Reposition Right Extraocular Muscle, Open Approach */
"08SL3ZZ"  =  "1"  /* Reposition Right Extraocular Muscle, Percutaneous Approach */
"08SM0ZZ"  =  "1"  /* Reposition Left Extraocular Muscle, Open Approach */
"08SM3ZZ"  =  "1"  /* Reposition Left Extraocular Muscle, Percutaneous Approach */
"08SN0ZZ"  =  "1"  /* Reposition Right Upper Eyelid, Open Approach */
"08SN3ZZ"  =  "1"  /* Reposition Right Upper Eyelid, Percutaneous Approach */
"08SNXZZ"  =  "1"  /* Reposition Right Upper Eyelid, External Approach */
"08SP0ZZ"  =  "1"  /* Reposition Left Upper Eyelid, Open Approach */
"08SP3ZZ"  =  "1"  /* Reposition Left Upper Eyelid, Percutaneous Approach */
"08SPXZZ"  =  "1"  /* Reposition Left Upper Eyelid, External Approach */
"08SQ0ZZ"  =  "1"  /* Reposition Right Lower Eyelid, Open Approach */
"08SQ3ZZ"  =  "1"  /* Reposition Right Lower Eyelid, Percutaneous Approach */
"08SQXZZ"  =  "1"  /* Reposition Right Lower Eyelid, External Approach */
"08SR0ZZ"  =  "1"  /* Reposition Left Lower Eyelid, Open Approach */
"08SR3ZZ"  =  "1"  /* Reposition Left Lower Eyelid, Percutaneous Approach */
"08SRXZZ"  =  "1"  /* Reposition Left Lower Eyelid, External Approach */
"08SV0ZZ"  =  "1"  /* Reposition Right Lacrimal Gland, Open Approach */
"08SV3ZZ"  =  "1"  /* Reposition Right Lacrimal Gland, Percutaneous Approach */
"08SW0ZZ"  =  "1"  /* Reposition Left Lacrimal Gland, Open Approach */
"08SW3ZZ"  =  "1"  /* Reposition Left Lacrimal Gland, Percutaneous Approach */
"08SX0ZZ"  =  "1"  /* Reposition Right Lacrimal Duct, Open Approach */
"08SX3ZZ"  =  "1"  /* Reposition Right Lacrimal Duct, Percutaneous Approach */
"08SX7ZZ"  =  "1"  /* Reposition Right Lacrimal Duct, Via Opening */
"08SX8ZZ"  =  "1"  /* Reposition Right Lacrimal Duct, Endo */
"08SY0ZZ"  =  "1"  /* Reposition Left Lacrimal Duct, Open Approach */
"08SY3ZZ"  =  "1"  /* Reposition Left Lacrimal Duct, Percutaneous Approach */
"08SY7ZZ"  =  "1"  /* Reposition Left Lacrimal Duct, Via Opening */
"08SY8ZZ"  =  "1"  /* Reposition Left Lacrimal Duct, Endo */
"08T0XZZ"  =  "1"  /* Resection of Right Eye, External Approach */
"08T1XZZ"  =  "1"  /* Resection of Left Eye, External Approach */
"08T43ZZ"  =  "1"  /* Resection of Right Vitreous, Percutaneous Approach */
"08T53ZZ"  =  "1"  /* Resection of Left Vitreous, Percutaneous Approach */
"08T8XZZ"  =  "1"  /* Resection of Right Cornea, External Approach */
"08T9XZZ"  =  "1"  /* Resection of Left Cornea, External Approach */
"08TC3ZZ"  =  "1"  /* Resection of Right Iris, Percutaneous Approach */
"08TD3ZZ"  =  "1"  /* Resection of Left Iris, Percutaneous Approach */
"08TJ3ZZ"  =  "1"  /* Resection of Right Lens, Percutaneous Approach */
"08TK3ZZ"  =  "1"  /* Resection of Left Lens, Percutaneous Approach */
"08TL0ZZ"  =  "1"  /* Resection of Right Extraocular Muscle, Open Approach */
"08TL3ZZ"  =  "1"  /* Resection of Right Extraocular Muscle, Percutaneous Approach */
"08TM0ZZ"  =  "1"  /* Resection of Left Extraocular Muscle, Open Approach */
"08TM3ZZ"  =  "1"  /* Resection of Left Extraocular Muscle, Percutaneous Approach */
"08TN0ZZ"  =  "1"  /* Resection of Right Upper Eyelid, Open Approach */
"08TNXZZ"  =  "1"  /* Resection of Right Upper Eyelid, External Approach */
"08TP0ZZ"  =  "1"  /* Resection of Left Upper Eyelid, Open Approach */
"08TPXZZ"  =  "1"  /* Resection of Left Upper Eyelid, External Approach */
"08TQ0ZZ"  =  "1"  /* Resection of Right Lower Eyelid, Open Approach */
"08TQXZZ"  =  "1"  /* Resection of Right Lower Eyelid, External Approach */
"08TR0ZZ"  =  "1"  /* Resection of Left Lower Eyelid, Open Approach */
"08TRXZZ"  =  "1"  /* Resection of Left Lower Eyelid, External Approach */
"08TV0ZZ"  =  "1"  /* Resection of Right Lacrimal Gland, Open Approach */
"08TV3ZZ"  =  "1"  /* Resection of Right Lacrimal Gland, Percutaneous Approach */
"08TW0ZZ"  =  "1"  /* Resection of Left Lacrimal Gland, Open Approach */
"08TW3ZZ"  =  "1"  /* Resection of Left Lacrimal Gland, Percutaneous Approach */
"08TX0ZZ"  =  "1"  /* Resection of Right Lacrimal Duct, Open Approach */
"08TX3ZZ"  =  "1"  /* Resection of Right Lacrimal Duct, Percutaneous Approach */
"08TX7ZZ"  =  "1"  /* Resection of Right Lacrimal Duct, Via Opening */
"08TX8ZZ"  =  "1"  /* Resection of Right Lacrimal Duct, Endo */
"08TY0ZZ"  =  "1"  /* Resection of Left Lacrimal Duct, Open Approach */
"08TY3ZZ"  =  "1"  /* Resection of Left Lacrimal Duct, Percutaneous Approach */
"08TY7ZZ"  =  "1"  /* Resection of Left Lacrimal Duct, Via Opening */
"08TY8ZZ"  =  "1"  /* Resection of Left Lacrimal Duct, Endo */
"08U007Z"  =  "1"  /* Supplement of Right Eye with Autol Sub, Open Approach */
"08U00JZ"  =  "1"  /* Supplement of Right Eye with Synth Sub, Open Approach */
"08U00KZ"  =  "1"  /* Supplement of Right Eye with Nonaut Sub, Open Approach */
"08U037Z"  =  "1"  /* Supplement of Right Eye with Autol Sub, Perc Approach */
"08U03JZ"  =  "1"  /* Supplement of Right Eye with Synth Sub, Perc Approach */
"08U03KZ"  =  "1"  /* Supplement of Right Eye with Nonaut Sub, Perc Approach */
"08U107Z"  =  "1"  /* Supplement of Left Eye with Autol Sub, Open Approach */
"08U10JZ"  =  "1"  /* Supplement of Left Eye with Synth Sub, Open Approach */
"08U10KZ"  =  "1"  /* Supplement of Left Eye with Nonaut Sub, Open Approach */
"08U137Z"  =  "1"  /* Supplement of Left Eye with Autol Sub, Perc Approach */
"08U13JZ"  =  "1"  /* Supplement of Left Eye with Synth Sub, Perc Approach */
"08U13KZ"  =  "1"  /* Supplement of Left Eye with Nonaut Sub, Perc Approach */
"08U807Z"  =  "1"  /* Supplement Right Cornea with Autol Sub, Open Approach */
"08U80JZ"  =  "1"  /* Supplement Right Cornea with Synth Sub, Open Approach */
"08U80KZ"  =  "1"  /* Supplement Right Cornea with Nonaut Sub, Open Approach */
"08U837Z"  =  "1"  /* Supplement Right Cornea with Autol Sub, Perc Approach */
"08U83JZ"  =  "1"  /* Supplement Right Cornea with Synth Sub, Perc Approach */
"08U83KZ"  =  "1"  /* Supplement Right Cornea with Nonaut Sub, Perc Approach */
"08U8X7Z"  =  "1"  /* Supplement Right Cornea with Autol Sub, Extern Approach */
"08U8XJZ"  =  "1"  /* Supplement Right Cornea with Synth Sub, Extern Approach */
"08U8XKZ"  =  "1"  /* Supplement Right Cornea with Nonaut Sub, Extern Approach */
"08U907Z"  =  "1"  /* Supplement Left Cornea with Autol Sub, Open Approach */
"08U90JZ"  =  "1"  /* Supplement Left Cornea with Synth Sub, Open Approach */
"08U90KZ"  =  "1"  /* Supplement Left Cornea with Nonaut Sub, Open Approach */
"08U937Z"  =  "1"  /* Supplement Left Cornea with Autol Sub, Perc Approach */
"08U93JZ"  =  "1"  /* Supplement Left Cornea with Synth Sub, Perc Approach */
"08U93KZ"  =  "1"  /* Supplement Left Cornea with Nonaut Sub, Perc Approach */
"08U9X7Z"  =  "1"  /* Supplement Left Cornea with Autol Sub, Extern Approach */
"08U9XJZ"  =  "1"  /* Supplement Left Cornea with Synth Sub, Extern Approach */
"08U9XKZ"  =  "1"  /* Supplement Left Cornea with Nonaut Sub, Extern Approach */
"08UC07Z"  =  "1"  /* Supplement Right Iris with Autol Sub, Open Approach */
"08UC0JZ"  =  "1"  /* Supplement Right Iris with Synth Sub, Open Approach */
"08UC0KZ"  =  "1"  /* Supplement Right Iris with Nonaut Sub, Open Approach */
"08UC37Z"  =  "1"  /* Supplement Right Iris with Autol Sub, Perc Approach */
"08UC3JZ"  =  "1"  /* Supplement Right Iris with Synth Sub, Perc Approach */
"08UC3KZ"  =  "1"  /* Supplement Right Iris with Nonaut Sub, Perc Approach */
"08UD07Z"  =  "1"  /* Supplement Left Iris with Autol Sub, Open Approach */
"08UD0JZ"  =  "1"  /* Supplement Left Iris with Synth Sub, Open Approach */
"08UD0KZ"  =  "1"  /* Supplement Left Iris with Nonaut Sub, Open Approach */
"08UD37Z"  =  "1"  /* Supplement Left Iris with Autol Sub, Perc Approach */
"08UD3JZ"  =  "1"  /* Supplement Left Iris with Synth Sub, Perc Approach */
"08UD3KZ"  =  "1"  /* Supplement Left Iris with Nonaut Sub, Perc Approach */
"08UE07Z"  =  "1"  /* Supplement Right Retina with Autol Sub, Open Approach */
"08UE0JZ"  =  "1"  /* Supplement Right Retina with Synth Sub, Open Approach */
"08UE0KZ"  =  "1"  /* Supplement Right Retina with Nonaut Sub, Open Approach */
"08UE37Z"  =  "1"  /* Supplement Right Retina with Autol Sub, Perc Approach */
"08UE3JZ"  =  "1"  /* Supplement Right Retina with Synth Sub, Perc Approach */
"08UE3KZ"  =  "1"  /* Supplement Right Retina with Nonaut Sub, Perc Approach */
"08UF07Z"  =  "1"  /* Supplement Left Retina with Autol Sub, Open Approach */
"08UF0JZ"  =  "1"  /* Supplement Left Retina with Synth Sub, Open Approach */
"08UF0KZ"  =  "1"  /* Supplement Left Retina with Nonaut Sub, Open Approach */
"08UF37Z"  =  "1"  /* Supplement Left Retina with Autol Sub, Perc Approach */
"08UF3JZ"  =  "1"  /* Supplement Left Retina with Synth Sub, Perc Approach */
"08UF3KZ"  =  "1"  /* Supplement Left Retina with Nonaut Sub, Perc Approach */
"08UG07Z"  =  "1"  /* Supplement R Retinal Vessel with Autol Sub, Open Approach */
"08UG0JZ"  =  "1"  /* Supplement R Retinal Vessel with Synth Sub, Open Approach */
"08UG0KZ"  =  "1"  /* Supplement R Retinal Vessel with Nonaut Sub, Open Approach */
"08UG37Z"  =  "1"  /* Supplement R Retinal Vessel with Autol Sub, Perc Approach */
"08UG3JZ"  =  "1"  /* Supplement R Retinal Vessel with Synth Sub, Perc Approach */
"08UG3KZ"  =  "1"  /* Supplement R Retinal Vessel with Nonaut Sub, Perc Approach */
"08UH07Z"  =  "1"  /* Supplement Left Retinal Vessel with Autol Sub, Open Approach */
"08UH0JZ"  =  "1"  /* Supplement Left Retinal Vessel with Synth Sub, Open Approach */
"08UH0KZ"  =  "1"  /* Supplement L Retinal Vessel with Nonaut Sub, Open Approach */
"08UH37Z"  =  "1"  /* Supplement Left Retinal Vessel with Autol Sub, Perc Approach */
"08UH3JZ"  =  "1"  /* Supplement Left Retinal Vessel with Synth Sub, Perc Approach */
"08UH3KZ"  =  "1"  /* Supplement L Retinal Vessel with Nonaut Sub, Perc Approach */
"08UL07Z"  =  "1"  /* Supplement R Extraoc Muscle with Autol Sub, Open Approach */
"08UL0JZ"  =  "1"  /* Supplement R Extraoc Muscle with Synth Sub, Open Approach */
"08UL0KZ"  =  "1"  /* Supplement R Extraoc Muscle with Nonaut Sub, Open Approach */
"08UL37Z"  =  "1"  /* Supplement R Extraoc Muscle with Autol Sub, Perc Approach */
"08UL3JZ"  =  "1"  /* Supplement R Extraoc Muscle with Synth Sub, Perc Approach */
"08UL3KZ"  =  "1"  /* Supplement R Extraoc Muscle with Nonaut Sub, Perc Approach */
"08UM07Z"  =  "1"  /* Supplement L Extraoc Muscle with Autol Sub, Open Approach */
"08UM0JZ"  =  "1"  /* Supplement L Extraoc Muscle with Synth Sub, Open Approach */
"08UM0KZ"  =  "1"  /* Supplement L Extraoc Muscle with Nonaut Sub, Open Approach */
"08UM37Z"  =  "1"  /* Supplement L Extraoc Muscle with Autol Sub, Perc Approach */
"08UM3JZ"  =  "1"  /* Supplement L Extraoc Muscle with Synth Sub, Perc Approach */
"08UM3KZ"  =  "1"  /* Supplement L Extraoc Muscle with Nonaut Sub, Perc Approach */
"08UN07Z"  =  "1"  /* Supplement Right Upper Eyelid with Autol Sub, Open Approach */
"08UN0JZ"  =  "1"  /* Supplement Right Upper Eyelid with Synth Sub, Open Approach */
"08UN0KZ"  =  "1"  /* Supplement Right Upper Eyelid with Nonaut Sub, Open Approach */
"08UN37Z"  =  "1"  /* Supplement Right Upper Eyelid with Autol Sub, Perc Approach */
"08UN3JZ"  =  "1"  /* Supplement Right Upper Eyelid with Synth Sub, Perc Approach */
"08UN3KZ"  =  "1"  /* Supplement Right Upper Eyelid with Nonaut Sub, Perc Approach */
"08UNX7Z"  =  "1"  /* Supplement R Up Eyelid with Autol Sub, Extern Approach */
"08UNXJZ"  =  "1"  /* Supplement R Up Eyelid with Synth Sub, Extern Approach */
"08UNXKZ"  =  "1"  /* Supplement R Up Eyelid with Nonaut Sub, Extern Approach */
"08UP07Z"  =  "1"  /* Supplement Left Upper Eyelid with Autol Sub, Open Approach */
"08UP0JZ"  =  "1"  /* Supplement Left Upper Eyelid with Synth Sub, Open Approach */
"08UP0KZ"  =  "1"  /* Supplement Left Upper Eyelid with Nonaut Sub, Open Approach */
"08UP37Z"  =  "1"  /* Supplement Left Upper Eyelid with Autol Sub, Perc Approach */
"08UP3JZ"  =  "1"  /* Supplement Left Upper Eyelid with Synth Sub, Perc Approach */
"08UP3KZ"  =  "1"  /* Supplement Left Upper Eyelid with Nonaut Sub, Perc Approach */
"08UPX7Z"  =  "1"  /* Supplement Left Upper Eyelid with Autol Sub, Extern Approach */
"08UPXJZ"  =  "1"  /* Supplement Left Upper Eyelid with Synth Sub, Extern Approach */
"08UPXKZ"  =  "1"  /* Supplement L Up Eyelid with Nonaut Sub, Extern Approach */
"08UQ07Z"  =  "1"  /* Supplement Right Lower Eyelid with Autol Sub, Open Approach */
"08UQ0JZ"  =  "1"  /* Supplement Right Lower Eyelid with Synth Sub, Open Approach */
"08UQ0KZ"  =  "1"  /* Supplement Right Lower Eyelid with Nonaut Sub, Open Approach */
"08UQ37Z"  =  "1"  /* Supplement Right Lower Eyelid with Autol Sub, Perc Approach */
"08UQ3JZ"  =  "1"  /* Supplement Right Lower Eyelid with Synth Sub, Perc Approach */
"08UQ3KZ"  =  "1"  /* Supplement Right Lower Eyelid with Nonaut Sub, Perc Approach */
"08UQX7Z"  =  "1"  /* Supplement R Low Eyelid with Autol Sub, Extern Approach */
"08UQXJZ"  =  "1"  /* Supplement R Low Eyelid with Synth Sub, Extern Approach */
"08UQXKZ"  =  "1"  /* Supplement R Low Eyelid with Nonaut Sub, Extern Approach */
"08UR07Z"  =  "1"  /* Supplement Left Lower Eyelid with Autol Sub, Open Approach */
"08UR0JZ"  =  "1"  /* Supplement Left Lower Eyelid with Synth Sub, Open Approach */
"08UR0KZ"  =  "1"  /* Supplement Left Lower Eyelid with Nonaut Sub, Open Approach */
"08UR37Z"  =  "1"  /* Supplement Left Lower Eyelid with Autol Sub, Perc Approach */
"08UR3JZ"  =  "1"  /* Supplement Left Lower Eyelid with Synth Sub, Perc Approach */
"08UR3KZ"  =  "1"  /* Supplement Left Lower Eyelid with Nonaut Sub, Perc Approach */
"08URX7Z"  =  "1"  /* Supplement Left Lower Eyelid with Autol Sub, Extern Approach */
"08URXJZ"  =  "1"  /* Supplement Left Lower Eyelid with Synth Sub, Extern Approach */
"08URXKZ"  =  "1"  /* Supplement L Low Eyelid with Nonaut Sub, Extern Approach */
"08UX07Z"  =  "1"  /* Supplement Right Lacrimal Duct with Autol Sub, Open Approach */
"08UX0JZ"  =  "1"  /* Supplement Right Lacrimal Duct with Synth Sub, Open Approach */
"08UX0KZ"  =  "1"  /* Supplement R Lacrml Duct with Nonaut Sub, Open Approach */
"08UX37Z"  =  "1"  /* Supplement Right Lacrimal Duct with Autol Sub, Perc Approach */
"08UX3JZ"  =  "1"  /* Supplement Right Lacrimal Duct with Synth Sub, Perc Approach */
"08UX3KZ"  =  "1"  /* Supplement R Lacrml Duct with Nonaut Sub, Perc Approach */
"08UX77Z"  =  "1"  /* Supplement Right Lacrimal Duct with Autol Sub, Via Opening */
"08UX7JZ"  =  "1"  /* Supplement Right Lacrimal Duct with Synth Sub, Via Opening */
"08UX7KZ"  =  "1"  /* Supplement Right Lacrimal Duct with Nonaut Sub, Via Opening */
"08UX87Z"  =  "1"  /* Supplement Right Lacrimal Duct with Autol Sub, Endo */
"08UX8JZ"  =  "1"  /* Supplement Right Lacrimal Duct with Synth Sub, Endo */
"08UX8KZ"  =  "1"  /* Supplement Right Lacrimal Duct with Nonaut Sub, Endo */
"08UY07Z"  =  "1"  /* Supplement Left Lacrimal Duct with Autol Sub, Open Approach */
"08UY0JZ"  =  "1"  /* Supplement Left Lacrimal Duct with Synth Sub, Open Approach */
"08UY0KZ"  =  "1"  /* Supplement Left Lacrimal Duct with Nonaut Sub, Open Approach */
"08UY37Z"  =  "1"  /* Supplement Left Lacrimal Duct with Autol Sub, Perc Approach */
"08UY3JZ"  =  "1"  /* Supplement Left Lacrimal Duct with Synth Sub, Perc Approach */
"08UY3KZ"  =  "1"  /* Supplement Left Lacrimal Duct with Nonaut Sub, Perc Approach */
"08UY77Z"  =  "1"  /* Supplement Left Lacrimal Duct with Autol Sub, Via Opening */
"08UY7JZ"  =  "1"  /* Supplement Left Lacrimal Duct with Synth Sub, Via Opening */
"08UY7KZ"  =  "1"  /* Supplement Left Lacrimal Duct with Nonaut Sub, Via Opening */
"08UY87Z"  =  "1"  /* Supplement Left Lacrimal Duct with Autol Sub, Endo */
"08UY8JZ"  =  "1"  /* Supplement Left Lacrimal Duct with Synth Sub, Endo */
"08UY8KZ"  =  "1"  /* Supplement Left Lacrimal Duct with Nonaut Sub, Endo */
"08VX0CZ"  =  "1"  /* Restrict of R Lacrml Duct with Extralum Dev, Open Approach */
"08VX0DZ"  =  "1"  /* Restrict of R Lacrml Duct with Intralum Dev, Open Approach */
"08VX0ZZ"  =  "1"  /* Restriction of Right Lacrimal Duct, Open Approach */
"08VX3CZ"  =  "1"  /* Restrict of R Lacrml Duct with Extralum Dev, Perc Approach */
"08VX3DZ"  =  "1"  /* Restrict of R Lacrml Duct with Intralum Dev, Perc Approach */
"08VX3ZZ"  =  "1"  /* Restriction of Right Lacrimal Duct, Percutaneous Approach */
"08VX7DZ"  =  "1"  /* Restriction of R Lacrml Duct with Intralum Dev, Via Opening */
"08VX7ZZ"  =  "1"  /* Restriction of Right Lacrimal Duct, Via Opening */
"08VX8DZ"  =  "1"  /* Restriction of Right Lacrimal Duct with Intralum Dev, Endo */
"08VX8ZZ"  =  "1"  /* Restriction of Right Lacrimal Duct, Endo */
"08VY0CZ"  =  "1"  /* Restrict of L Lacrml Duct with Extralum Dev, Open Approach */
"08VY0DZ"  =  "1"  /* Restrict of L Lacrml Duct with Intralum Dev, Open Approach */
"08VY0ZZ"  =  "1"  /* Restriction of Left Lacrimal Duct, Open Approach */
"08VY3CZ"  =  "1"  /* Restrict of L Lacrml Duct with Extralum Dev, Perc Approach */
"08VY3DZ"  =  "1"  /* Restrict of L Lacrml Duct with Intralum Dev, Perc Approach */
"08VY3ZZ"  =  "1"  /* Restriction of Left Lacrimal Duct, Percutaneous Approach */
"08VY7DZ"  =  "1"  /* Restriction of L Lacrml Duct with Intralum Dev, Via Opening */
"08VY7ZZ"  =  "1"  /* Restriction of Left Lacrimal Duct, Via Opening */
"08VY8DZ"  =  "1"  /* Restriction of Left Lacrimal Duct with Intralum Dev, Endo */
"08VY8ZZ"  =  "1"  /* Restriction of Left Lacrimal Duct, Endo */
"08W000Z"  =  "1"  /* Revision of Drainage Device in Right Eye, Open Approach */
"08W003Z"  =  "1"  /* Revision of Infusion Device in Right Eye, Open Approach */
"08W007Z"  =  "1"  /* Revision of Autol Sub in R Eye, Open Approach */
"08W00CZ"  =  "1"  /* Revision of Extraluminal Device in Right Eye, Open Approach */
"08W00DZ"  =  "1"  /* Revision of Intraluminal Device in Right Eye, Open Approach */
"08W00JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Eye, Open Approach */
"08W00KZ"  =  "1"  /* Revision of Nonaut Sub in R Eye, Open Approach */
"08W030Z"  =  "1"  /* Revision of Drainage Device in Right Eye, Perc Approach */
"08W033Z"  =  "1"  /* Revision of Infusion Device in Right Eye, Perc Approach */
"08W037Z"  =  "1"  /* Revision of Autol Sub in R Eye, Perc Approach */
"08W03CZ"  =  "1"  /* Revision of Extraluminal Device in Right Eye, Perc Approach */
"08W03DZ"  =  "1"  /* Revision of Intraluminal Device in Right Eye, Perc Approach */
"08W03JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Eye, Perc Approach */
"08W03KZ"  =  "1"  /* Revision of Nonaut Sub in R Eye, Perc Approach */
"08W070Z"  =  "1"  /* Revision of Drainage Device in Right Eye, Via Opening */
"08W073Z"  =  "1"  /* Revision of Infusion Device in Right Eye, Via Opening */
"08W077Z"  =  "1"  /* Revision of Autol Sub in R Eye, Via Opening */
"08W07CZ"  =  "1"  /* Revision of Extraluminal Device in Right Eye, Via Opening */
"08W07DZ"  =  "1"  /* Revision of Intraluminal Device in Right Eye, Via Opening */
"08W07JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Eye, Via Opening */
"08W07KZ"  =  "1"  /* Revision of Nonaut Sub in R Eye, Via Opening */
"08W080Z"  =  "1"  /* Revision of Drainage Device in Right Eye, Endo */
"08W083Z"  =  "1"  /* Revision of Infusion Device in Right Eye, Endo */
"08W087Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Right Eye, Endo */
"08W08CZ"  =  "1"  /* Revision of Extraluminal Device in Right Eye, Endo */
"08W08DZ"  =  "1"  /* Revision of Intraluminal Device in Right Eye, Endo */
"08W08JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Eye, Endo */
"08W08KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in R Eye, Endo */
"08W100Z"  =  "1"  /* Revision of Drainage Device in Left Eye, Open Approach */
"08W103Z"  =  "1"  /* Revision of Infusion Device in Left Eye, Open Approach */
"08W107Z"  =  "1"  /* Revision of Autol Sub in L Eye, Open Approach */
"08W10CZ"  =  "1"  /* Revision of Extraluminal Device in Left Eye, Open Approach */
"08W10DZ"  =  "1"  /* Revision of Intraluminal Device in Left Eye, Open Approach */
"08W10JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Eye, Open Approach */
"08W10KZ"  =  "1"  /* Revision of Nonaut Sub in L Eye, Open Approach */
"08W130Z"  =  "1"  /* Revision of Drainage Device in Left Eye, Perc Approach */
"08W133Z"  =  "1"  /* Revision of Infusion Device in Left Eye, Perc Approach */
"08W137Z"  =  "1"  /* Revision of Autol Sub in L Eye, Perc Approach */
"08W13CZ"  =  "1"  /* Revision of Extraluminal Device in Left Eye, Perc Approach */
"08W13DZ"  =  "1"  /* Revision of Intraluminal Device in Left Eye, Perc Approach */
"08W13JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Eye, Perc Approach */
"08W13KZ"  =  "1"  /* Revision of Nonaut Sub in L Eye, Perc Approach */
"08W170Z"  =  "1"  /* Revision of Drainage Device in Left Eye, Via Opening */
"08W173Z"  =  "1"  /* Revision of Infusion Device in Left Eye, Via Opening */
"08W177Z"  =  "1"  /* Revision of Autol Sub in L Eye, Via Opening */
"08W17CZ"  =  "1"  /* Revision of Extraluminal Device in Left Eye, Via Opening */
"08W17DZ"  =  "1"  /* Revision of Intraluminal Device in Left Eye, Via Opening */
"08W17JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Eye, Via Opening */
"08W17KZ"  =  "1"  /* Revision of Nonaut Sub in L Eye, Via Opening */
"08W180Z"  =  "1"  /* Revision of Drainage Device in Left Eye, Endo */
"08W183Z"  =  "1"  /* Revision of Infusion Device in Left Eye, Endo */
"08W187Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Left Eye, Endo */
"08W18CZ"  =  "1"  /* Revision of Extraluminal Device in Left Eye, Endo */
"08W18DZ"  =  "1"  /* Revision of Intraluminal Device in Left Eye, Endo */
"08W18JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Eye, Endo */
"08W18KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in L Eye, Endo */
"08WJ3JZ"  =  "1"  /* Revision of Synthetic Substitute in R Lens, Perc Approach */
"08WK3JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Lens, Perc Approach */
"08WL00Z"  =  "1"  /* Revision of Drain Dev in R Extraoc Muscle, Open Approach */
"08WL07Z"  =  "1"  /* Revision of Autol Sub in R Extraoc Muscle, Open Approach */
"08WL0JZ"  =  "1"  /* Revision of Synth Sub in R Extraoc Muscle, Open Approach */
"08WL0KZ"  =  "1"  /* Revision of Nonaut Sub in R Extraoc Muscle, Open Approach */
"08WL30Z"  =  "1"  /* Revision of Drain Dev in R Extraoc Muscle, Perc Approach */
"08WL37Z"  =  "1"  /* Revision of Autol Sub in R Extraoc Muscle, Perc Approach */
"08WL3JZ"  =  "1"  /* Revision of Synth Sub in R Extraoc Muscle, Perc Approach */
"08WL3KZ"  =  "1"  /* Revision of Nonaut Sub in R Extraoc Muscle, Perc Approach */
"08WM00Z"  =  "1"  /* Revision of Drain Dev in L Extraoc Muscle, Open Approach */
"08WM07Z"  =  "1"  /* Revision of Autol Sub in L Extraoc Muscle, Open Approach */
"08WM0JZ"  =  "1"  /* Revision of Synth Sub in L Extraoc Muscle, Open Approach */
"08WM0KZ"  =  "1"  /* Revision of Nonaut Sub in L Extraoc Muscle, Open Approach */
"08WM30Z"  =  "1"  /* Revision of Drain Dev in L Extraoc Muscle, Perc Approach */
"08WM37Z"  =  "1"  /* Revision of Autol Sub in L Extraoc Muscle, Perc Approach */
"08WM3JZ"  =  "1"  /* Revision of Synth Sub in L Extraoc Muscle, Perc Approach */
"08WM3KZ"  =  "1"  /* Revision of Nonaut Sub in L Extraoc Muscle, Perc Approach */
"08XL0ZZ"  =  "1"  /* Transfer Right Extraocular Muscle, Open Approach */
"08XL3ZZ"  =  "1"  /* Transfer Right Extraocular Muscle, Percutaneous Approach */
"08XM0ZZ"  =  "1"  /* Transfer Left Extraocular Muscle, Open Approach */
"08XM3ZZ"  =  "1"  /* Transfer Left Extraocular Muscle, Percutaneous Approach */
"090007Z"  =  "1"  /* Alteration of R Ext Ear with Autol Sub, Open Approach */
"09000JZ"  =  "1"  /* Alteration of R Ext Ear with Synth Sub, Open Approach */
"09000KZ"  =  "1"  /* Alteration of R Ext Ear with Nonaut Sub, Open Approach */
"09000ZZ"  =  "1"  /* Alteration of Right External Ear, Open Approach */
"090037Z"  =  "1"  /* Alteration of R Ext Ear with Autol Sub, Perc Approach */
"09003JZ"  =  "1"  /* Alteration of R Ext Ear with Synth Sub, Perc Approach */
"09003KZ"  =  "1"  /* Alteration of R Ext Ear with Nonaut Sub, Perc Approach */
"09003ZZ"  =  "1"  /* Alteration of Right External Ear, Percutaneous Approach */
"090047Z"  =  "1"  /* Alteration of R Ext Ear with Autol Sub, Perc Endo Approach */
"09004JZ"  =  "1"  /* Alteration of R Ext Ear with Synth Sub, Perc Endo Approach */
"09004KZ"  =  "1"  /* Alteration of R Ext Ear with Nonaut Sub, Perc Endo Approach */
"09004ZZ"  =  "1"  /* Alteration of Right External Ear, Perc Endo Approach */
"0900X7Z"  =  "1"  /* Alteration of R Ext Ear with Autol Sub, Extern Approach */
"0900XJZ"  =  "1"  /* Alteration of R Ext Ear with Synth Sub, Extern Approach */
"0900XKZ"  =  "1"  /* Alteration of R Ext Ear with Nonaut Sub, Extern Approach */
"0900XZZ"  =  "1"  /* Alteration of Right External Ear, External Approach */
"090107Z"  =  "1"  /* Alteration of L Ext Ear with Autol Sub, Open Approach */
"09010JZ"  =  "1"  /* Alteration of L Ext Ear with Synth Sub, Open Approach */
"09010KZ"  =  "1"  /* Alteration of L Ext Ear with Nonaut Sub, Open Approach */
"09010ZZ"  =  "1"  /* Alteration of Left External Ear, Open Approach */
"090137Z"  =  "1"  /* Alteration of L Ext Ear with Autol Sub, Perc Approach */
"09013JZ"  =  "1"  /* Alteration of L Ext Ear with Synth Sub, Perc Approach */
"09013KZ"  =  "1"  /* Alteration of L Ext Ear with Nonaut Sub, Perc Approach */
"09013ZZ"  =  "1"  /* Alteration of Left External Ear, Percutaneous Approach */
"090147Z"  =  "1"  /* Alteration of L Ext Ear with Autol Sub, Perc Endo Approach */
"09014JZ"  =  "1"  /* Alteration of L Ext Ear with Synth Sub, Perc Endo Approach */
"09014KZ"  =  "1"  /* Alteration of L Ext Ear with Nonaut Sub, Perc Endo Approach */
"09014ZZ"  =  "1"  /* Alteration of Left External Ear, Perc Endo Approach */
"0901X7Z"  =  "1"  /* Alteration of L Ext Ear with Autol Sub, Extern Approach */
"0901XJZ"  =  "1"  /* Alteration of L Ext Ear with Synth Sub, Extern Approach */
"0901XKZ"  =  "1"  /* Alteration of L Ext Ear with Nonaut Sub, Extern Approach */
"0901XZZ"  =  "1"  /* Alteration of Left External Ear, External Approach */
"090207Z"  =  "1"  /* Alteration of Bi Ext Ear with Autol Sub, Open Approach */
"09020JZ"  =  "1"  /* Alteration of Bi Ext Ear with Synth Sub, Open Approach */
"09020KZ"  =  "1"  /* Alteration of Bi Ext Ear with Nonaut Sub, Open Approach */
"09020ZZ"  =  "1"  /* Alteration of Bilateral External Ear, Open Approach */
"090237Z"  =  "1"  /* Alteration of Bi Ext Ear with Autol Sub, Perc Approach */
"09023JZ"  =  "1"  /* Alteration of Bi Ext Ear with Synth Sub, Perc Approach */
"09023KZ"  =  "1"  /* Alteration of Bi Ext Ear with Nonaut Sub, Perc Approach */
"09023ZZ"  =  "1"  /* Alteration of Bilateral External Ear, Percutaneous Approach */
"090247Z"  =  "1"  /* Alteration of Bi Ext Ear with Autol Sub, Perc Endo Approach */
"09024JZ"  =  "1"  /* Alteration of Bi Ext Ear with Synth Sub, Perc Endo Approach */
"09024KZ"  =  "1"  /* Alteration of Bi Ext Ear with Nonaut Sub, Perc Endo Approach */
"09024ZZ"  =  "1"  /* Alteration of Bilateral External Ear, Perc Endo Approach */
"0902X7Z"  =  "1"  /* Alteration of Bi Ext Ear with Autol Sub, Extern Approach */
"0902XJZ"  =  "1"  /* Alteration of Bi Ext Ear with Synth Sub, Extern Approach */
"0902XKZ"  =  "1"  /* Alteration of Bi Ext Ear with Nonaut Sub, Extern Approach */
"0902XZZ"  =  "1"  /* Alteration of Bilateral External Ear, External Approach */
"090K07Z"  =  "1"  /* Alteration of Nose with Autol Sub, Open Approach */
"090K0JZ"  =  "1"  /* Alteration of Nose with Synthetic Substitute, Open Approach */
"090K0KZ"  =  "1"  /* Alteration of Nose with Nonaut Sub, Open Approach */
"090K0ZZ"  =  "1"  /* Alteration of Nose, Open Approach */
"090K37Z"  =  "1"  /* Alteration of Nose with Autol Sub, Perc Approach */
"090K3JZ"  =  "1"  /* Alteration of Nose with Synthetic Substitute, Perc Approach */
"090K3KZ"  =  "1"  /* Alteration of Nose with Nonaut Sub, Perc Approach */
"090K3ZZ"  =  "1"  /* Alteration of Nose, Percutaneous Approach */
"090K47Z"  =  "1"  /* Alteration of Nose with Autol Sub, Perc Endo Approach */
"090K4JZ"  =  "1"  /* Alteration of Nose with Synth Sub, Perc Endo Approach */
"090K4KZ"  =  "1"  /* Alteration of Nose with Nonaut Sub, Perc Endo Approach */
"090K4ZZ"  =  "1"  /* Alteration of Nose, Percutaneous Endoscopic Approach */
"090KX7Z"  =  "1"  /* Alteration of Nose with Autol Sub, Extern Approach */
"090KXJZ"  =  "1"  /* Alteration of Nose with Synth Sub, Extern Approach */
"090KXKZ"  =  "1"  /* Alteration of Nose with Nonaut Sub, Extern Approach */
"090KXZZ"  =  "1"  /* Alteration of Nose, External Approach */
"091D070"  =  "1"  /* Bypass R Inner Ear to Endolymphatic w Autol Sub, Open */
"091D0J0"  =  "1"  /* Bypass R Inner Ear to Endolymphatic w Synth Sub, Open */
"091D0K0"  =  "1"  /* Bypass R Inner Ear to Endolymphatic w Nonaut Sub, Open */
"091D0Z0"  =  "1"  /* Bypass Right Inner Ear to Endolymphatic, Open Approach */
"091E070"  =  "1"  /* Bypass L Inner Ear to Endolymphatic w Autol Sub, Open */
"091E0J0"  =  "1"  /* Bypass L Inner Ear to Endolymphatic w Synth Sub, Open */
"091E0K0"  =  "1"  /* Bypass L Inner Ear to Endolymphatic w Nonaut Sub, Open */
"091E0Z0"  =  "1"  /* Bypass Left Inner Ear to Endolymphatic, Open Approach */
"09550ZZ"  =  "1"  /* Destruction of Right Middle Ear, Open Approach */
"09560ZZ"  =  "1"  /* Destruction of Left Middle Ear, Open Approach */
"09570ZZ"  =  "1"  /* Destruction of Right Tympanic Membrane, Open Approach */
"09573ZZ"  =  "1"  /* Destruction of Right Tympanic Membrane, Perc Approach */
"09574ZZ"  =  "1"  /* Destruction of Right Tympanic Membrane, Perc Endo Approach */
"09577ZZ"  =  "1"  /* Destruction of Right Tympanic Membrane, Via Opening */
"09578ZZ"  =  "1"  /* Destruction of Right Tympanic Membrane, Endo */
"09580ZZ"  =  "1"  /* Destruction of Left Tympanic Membrane, Open Approach */
"09583ZZ"  =  "1"  /* Destruction of Left Tympanic Membrane, Percutaneous Approach */
"09584ZZ"  =  "1"  /* Destruction of Left Tympanic Membrane, Perc Endo Approach */
"09587ZZ"  =  "1"  /* Destruction of Left Tympanic Membrane, Via Opening */
"09588ZZ"  =  "1"  /* Destruction of Left Tympanic Membrane, Endo */
"09590ZZ"  =  "1"  /* Destruction of Right Auditory Ossicle, Open Approach */
"095A0ZZ"  =  "1"  /* Destruction of Left Auditory Ossicle, Open Approach */
"095B0ZZ"  =  "1"  /* Destruction of Right Mastoid Sinus, Open Approach */
"095B3ZZ"  =  "1"  /* Destruction of Right Mastoid Sinus, Percutaneous Approach */
"095B4ZZ"  =  "1"  /* Destruction of Right Mastoid Sinus, Perc Endo Approach */
"095C0ZZ"  =  "1"  /* Destruction of Left Mastoid Sinus, Open Approach */
"095C3ZZ"  =  "1"  /* Destruction of Left Mastoid Sinus, Percutaneous Approach */
"095C4ZZ"  =  "1"  /* Destruction of Left Mastoid Sinus, Perc Endo Approach */
"095D0ZZ"  =  "1"  /* Destruction of Right Inner Ear, Open Approach */
"095E0ZZ"  =  "1"  /* Destruction of Left Inner Ear, Open Approach */
"095L0ZZ"  =  "1"  /* Destruction of Nasal Turbinate, Open Approach */
"095L3ZZ"  =  "1"  /* Destruction of Nasal Turbinate, Percutaneous Approach */
"095L4ZZ"  =  "1"  /* Destruction of Nasal Turbinate, Perc Endo Approach */
"095L7ZZ"  =  "1"  /* Destruction of Nasal Turbinate, Via Opening */
"095L8ZZ"  =  "1"  /* Destruction of Nasal Turbinate, Endo */
"095N0ZZ"  =  "1"  /* Destruction of Nasopharynx, Open Approach */
"095N3ZZ"  =  "1"  /* Destruction of Nasopharynx, Percutaneous Approach */
"095N4ZZ"  =  "1"  /* Destruction of Nasopharynx, Percutaneous Endoscopic Approach */
"095N7ZZ"  =  "1"  /* Destruction of Nasopharynx, Via Opening */
"095N8ZZ"  =  "1"  /* Destruction of Nasopharynx, Endo */
"095P0ZZ"  =  "1"  /* Destruction of Accessory Sinus, Open Approach */
"095P3ZZ"  =  "1"  /* Destruction of Accessory Sinus, Percutaneous Approach */
"095P4ZZ"  =  "1"  /* Destruction of Accessory Sinus, Perc Endo Approach */
"095Q0ZZ"  =  "1"  /* Destruction of Right Maxillary Sinus, Open Approach */
"095Q3ZZ"  =  "1"  /* Destruction of Right Maxillary Sinus, Percutaneous Approach */
"095Q4ZZ"  =  "1"  /* Destruction of Right Maxillary Sinus, Perc Endo Approach */
"095R0ZZ"  =  "1"  /* Destruction of Left Maxillary Sinus, Open Approach */
"095R3ZZ"  =  "1"  /* Destruction of Left Maxillary Sinus, Percutaneous Approach */
"095R4ZZ"  =  "1"  /* Destruction of Left Maxillary Sinus, Perc Endo Approach */
"095S0ZZ"  =  "1"  /* Destruction of Right Frontal Sinus, Open Approach */
"095S3ZZ"  =  "1"  /* Destruction of Right Frontal Sinus, Percutaneous Approach */
"095S4ZZ"  =  "1"  /* Destruction of Right Frontal Sinus, Perc Endo Approach */
"095T0ZZ"  =  "1"  /* Destruction of Left Frontal Sinus, Open Approach */
"095T3ZZ"  =  "1"  /* Destruction of Left Frontal Sinus, Percutaneous Approach */
"095T4ZZ"  =  "1"  /* Destruction of Left Frontal Sinus, Perc Endo Approach */
"095U0ZZ"  =  "1"  /* Destruction of Right Ethmoid Sinus, Open Approach */
"095U3ZZ"  =  "1"  /* Destruction of Right Ethmoid Sinus, Percutaneous Approach */
"095U4ZZ"  =  "1"  /* Destruction of Right Ethmoid Sinus, Perc Endo Approach */
"095V0ZZ"  =  "1"  /* Destruction of Left Ethmoid Sinus, Open Approach */
"095V3ZZ"  =  "1"  /* Destruction of Left Ethmoid Sinus, Percutaneous Approach */
"095V4ZZ"  =  "1"  /* Destruction of Left Ethmoid Sinus, Perc Endo Approach */
"095W0ZZ"  =  "1"  /* Destruction of Right Sphenoid Sinus, Open Approach */
"095W3ZZ"  =  "1"  /* Destruction of Right Sphenoid Sinus, Percutaneous Approach */
"095W4ZZ"  =  "1"  /* Destruction of Right Sphenoid Sinus, Perc Endo Approach */
"095X0ZZ"  =  "1"  /* Destruction of Left Sphenoid Sinus, Open Approach */
"095X3ZZ"  =  "1"  /* Destruction of Left Sphenoid Sinus, Percutaneous Approach */
"095X4ZZ"  =  "1"  /* Destruction of Left Sphenoid Sinus, Perc Endo Approach */
"098L0ZZ"  =  "1"  /* Division of Nasal Turbinate, Open Approach */
"098L3ZZ"  =  "1"  /* Division of Nasal Turbinate, Percutaneous Approach */
"098L4ZZ"  =  "1"  /* Division of Nasal Turbinate, Perc Endo Approach */
"098L7ZZ"  =  "1"  /* Division of Nasal Turbinate, Via Opening */
"098L8ZZ"  =  "1"  /* Division of Nasal Turbinate, Endo */
"099500Z"  =  "1"  /* Drainage of Right Middle Ear with Drain Dev, Open Approach */
"09950ZX"  =  "1"  /* Drainage of Right Middle Ear, Open Approach, Diagnostic */
"099600Z"  =  "1"  /* Drainage of Left Middle Ear with Drain Dev, Open Approach */
"09960ZX"  =  "1"  /* Drainage of Left Middle Ear, Open Approach, Diagnostic */
"099700Z"  =  "1"  /* Drain of R Tympanic Membrane with Drain Dev, Open Approach */
"09970ZX"  =  "1"  /* Drainage of Right Tympanic Membrane, Open Approach, Diagn */
"099730Z"  =  "1"  /* Drain of R Tympanic Membrane with Drain Dev, Perc Approach */
"09973ZX"  =  "1"  /* Drainage of Right Tympanic Membrane, Perc Approach, Diagn */
"099740Z"  =  "1"  /* Drain R Tympanic Membrane w Drain Dev, Perc Endo */
"09974ZX"  =  "1"  /* Drainage of R Tympanic Membrane, Perc Endo Approach, Diagn */
"099770Z"  =  "1"  /* Drainage of R Tympanic Membrane with Drain Dev, Via Opening */
"09977ZX"  =  "1"  /* Drainage of Right Tympanic Membrane, Via Opening, Diagn */
"099780Z"  =  "1"  /* Drainage of Right Tympanic Membrane with Drain Dev, Endo */
"09978ZX"  =  "1"  /* Drainage of Right Tympanic Membrane, Endo, Diagn */
"099800Z"  =  "1"  /* Drain of L Tympanic Membrane with Drain Dev, Open Approach */
"09980ZX"  =  "1"  /* Drainage of Left Tympanic Membrane, Open Approach, Diagn */
"099830Z"  =  "1"  /* Drain of L Tympanic Membrane with Drain Dev, Perc Approach */
"09983ZX"  =  "1"  /* Drainage of Left Tympanic Membrane, Perc Approach, Diagn */
"099840Z"  =  "1"  /* Drain L Tympanic Membrane w Drain Dev, Perc Endo */
"09984ZX"  =  "1"  /* Drainage of L Tympanic Membrane, Perc Endo Approach, Diagn */
"099870Z"  =  "1"  /* Drainage of L Tympanic Membrane with Drain Dev, Via Opening */
"09987ZX"  =  "1"  /* Drainage of Left Tympanic Membrane, Via Opening, Diagn */
"099880Z"  =  "1"  /* Drainage of Left Tympanic Membrane with Drain Dev, Endo */
"09988ZX"  =  "1"  /* Drainage of Left Tympanic Membrane, Endo, Diagn */
"099900Z"  =  "1"  /* Drainage of R Auditory Ossicle with Drain Dev, Open Approach */
"09990ZX"  =  "1"  /* Drainage of Right Auditory Ossicle, Open Approach, Diagn */
"09990ZZ"  =  "1"  /* Drainage of Right Auditory Ossicle, Open Approach */
"099A00Z"  =  "1"  /* Drainage of L Auditory Ossicle with Drain Dev, Open Approach */
"099A0ZX"  =  "1"  /* Drainage of Left Auditory Ossicle, Open Approach, Diagnostic */
"099A0ZZ"  =  "1"  /* Drainage of Left Auditory Ossicle, Open Approach */
"099B00Z"  =  "1"  /* Drainage of R Mastoid Sinus with Drain Dev, Open Approach */
"099B0ZX"  =  "1"  /* Drainage of Right Mastoid Sinus, Open Approach, Diagnostic */
"099B0ZZ"  =  "1"  /* Drainage of Right Mastoid Sinus, Open Approach */
"099B30Z"  =  "1"  /* Drainage of R Mastoid Sinus with Drain Dev, Perc Approach */
"099B3ZX"  =  "1"  /* Drainage of Right Mastoid Sinus, Perc Approach, Diagn */
"099B3ZZ"  =  "1"  /* Drainage of Right Mastoid Sinus, Percutaneous Approach */
"099B40Z"  =  "1"  /* Drain of R Mastoid Sinus with Drain Dev, Perc Endo Approach */
"099B4ZX"  =  "1"  /* Drainage of Right Mastoid Sinus, Perc Endo Approach, Diagn */
"099B4ZZ"  =  "1"  /* Drainage of Right Mastoid Sinus, Perc Endo Approach */
"099C00Z"  =  "1"  /* Drainage of Left Mastoid Sinus with Drain Dev, Open Approach */
"099C0ZX"  =  "1"  /* Drainage of Left Mastoid Sinus, Open Approach, Diagnostic */
"099C0ZZ"  =  "1"  /* Drainage of Left Mastoid Sinus, Open Approach */
"099C30Z"  =  "1"  /* Drainage of Left Mastoid Sinus with Drain Dev, Perc Approach */
"099C3ZX"  =  "1"  /* Drainage of Left Mastoid Sinus, Percutaneous Approach, Diagn */
"099C3ZZ"  =  "1"  /* Drainage of Left Mastoid Sinus, Percutaneous Approach */
"099C40Z"  =  "1"  /* Drain of L Mastoid Sinus with Drain Dev, Perc Endo Approach */
"099C4ZX"  =  "1"  /* Drainage of Left Mastoid Sinus, Perc Endo Approach, Diagn */
"099C4ZZ"  =  "1"  /* Drainage of Left Mastoid Sinus, Perc Endo Approach */
"099D00Z"  =  "1"  /* Drainage of Right Inner Ear with Drain Dev, Open Approach */
"099D0ZX"  =  "1"  /* Drainage of Right Inner Ear, Open Approach, Diagnostic */
"099D0ZZ"  =  "1"  /* Drainage of Right Inner Ear, Open Approach */
"099E00Z"  =  "1"  /* Drainage of Left Inner Ear with Drain Dev, Open Approach */
"099E0ZX"  =  "1"  /* Drainage of Left Inner Ear, Open Approach, Diagnostic */
"099E0ZZ"  =  "1"  /* Drainage of Left Inner Ear, Open Approach */
"099F0ZX"  =  "1"  /* Drainage of Right Eustachian Tube, Open Approach, Diagnostic */
"099F3ZX"  =  "1"  /* Drainage of Right Eustachian Tube, Perc Approach, Diagn */
"099F4ZX"  =  "1"  /* Drainage of Right Eustachian Tube, Perc Endo Approach, Diagn */
"099F7ZX"  =  "1"  /* Drainage of Right Eustachian Tube, Via Opening, Diagn */
"099F8ZX"  =  "1"  /* Drainage of Right Eustachian Tube, Endo, Diagn */
"099G0ZX"  =  "1"  /* Drainage of Left Eustachian Tube, Open Approach, Diagnostic */
"099G3ZX"  =  "1"  /* Drainage of Left Eustachian Tube, Perc Approach, Diagn */
"099G4ZX"  =  "1"  /* Drainage of Left Eustachian Tube, Perc Endo Approach, Diagn */
"099G7ZX"  =  "1"  /* Drainage of Left Eustachian Tube, Via Opening, Diagn */
"099G8ZX"  =  "1"  /* Drainage of Left Eustachian Tube, Endo, Diagn */
"099N00Z"  =  "1"  /* Drainage of Nasopharynx with Drainage Device, Open Approach */
"099N0ZZ"  =  "1"  /* Drainage of Nasopharynx, Open Approach */
"099N30Z"  =  "1"  /* Drainage of Nasopharynx with Drainage Device, Perc Approach */
"099N3ZZ"  =  "1"  /* Drainage of Nasopharynx, Percutaneous Approach */
"099N40Z"  =  "1"  /* Drainage of Nasopharynx with Drain Dev, Perc Endo Approach */
"099N4ZZ"  =  "1"  /* Drainage of Nasopharynx, Percutaneous Endoscopic Approach */
"099N70Z"  =  "1"  /* Drainage of Nasopharynx with Drainage Device, Via Opening */
"099N7ZZ"  =  "1"  /* Drainage of Nasopharynx, Via Natural or Artificial Opening */
"099N80Z"  =  "1"  /* Drainage of Nasopharynx with Drainage Device, Endo */
"099N8ZZ"  =  "1"  /* Drainage of Nasopharynx, Endo */
"099P00Z"  =  "1"  /* Drainage of Accessory Sinus with Drain Dev, Open Approach */
"099P0ZX"  =  "1"  /* Drainage of Accessory Sinus, Open Approach, Diagnostic */
"099P0ZZ"  =  "1"  /* Drainage of Accessory Sinus, Open Approach */
"099Q00Z"  =  "1"  /* Drainage of R Maxillary Sinus with Drain Dev, Open Approach */
"099Q0ZX"  =  "1"  /* Drainage of Right Maxillary Sinus, Open Approach, Diagnostic */
"099Q0ZZ"  =  "1"  /* Drainage of Right Maxillary Sinus, Open Approach */
"099R00Z"  =  "1"  /* Drainage of L Maxillary Sinus with Drain Dev, Open Approach */
"099R0ZX"  =  "1"  /* Drainage of Left Maxillary Sinus, Open Approach, Diagnostic */
"099R0ZZ"  =  "1"  /* Drainage of Left Maxillary Sinus, Open Approach */
"099S00Z"  =  "1"  /* Drainage of R Frntl Sinus with Drain Dev, Open Approach */
"099S0ZX"  =  "1"  /* Drainage of Right Frontal Sinus, Open Approach, Diagnostic */
"099S0ZZ"  =  "1"  /* Drainage of Right Frontal Sinus, Open Approach */
"099T00Z"  =  "1"  /* Drainage of Left Frontal Sinus with Drain Dev, Open Approach */
"099T0ZX"  =  "1"  /* Drainage of Left Frontal Sinus, Open Approach, Diagnostic */
"099T0ZZ"  =  "1"  /* Drainage of Left Frontal Sinus, Open Approach */
"099U00Z"  =  "1"  /* Drainage of R Ethmoid Sinus with Drain Dev, Open Approach */
"099U0ZX"  =  "1"  /* Drainage of Right Ethmoid Sinus, Open Approach, Diagnostic */
"099U0ZZ"  =  "1"  /* Drainage of Right Ethmoid Sinus, Open Approach */
"099V00Z"  =  "1"  /* Drainage of Left Ethmoid Sinus with Drain Dev, Open Approach */
"099V0ZX"  =  "1"  /* Drainage of Left Ethmoid Sinus, Open Approach, Diagnostic */
"099V0ZZ"  =  "1"  /* Drainage of Left Ethmoid Sinus, Open Approach */
"099W00Z"  =  "1"  /* Drainage of R Sphenoid Sinus with Drain Dev, Open Approach */
"099W0ZX"  =  "1"  /* Drainage of Right Sphenoid Sinus, Open Approach, Diagnostic */
"099W0ZZ"  =  "1"  /* Drainage of Right Sphenoid Sinus, Open Approach */
"099X00Z"  =  "1"  /* Drainage of L Sphenoid Sinus with Drain Dev, Open Approach */
"099X0ZX"  =  "1"  /* Drainage of Left Sphenoid Sinus, Open Approach, Diagnostic */
"099X0ZZ"  =  "1"  /* Drainage of Left Sphenoid Sinus, Open Approach */
"09B50ZX"  =  "1"  /* Excision of Right Middle Ear, Open Approach, Diagnostic */
"09B50ZZ"  =  "1"  /* Excision of Right Middle Ear, Open Approach */
"09B60ZX"  =  "1"  /* Excision of Left Middle Ear, Open Approach, Diagnostic */
"09B60ZZ"  =  "1"  /* Excision of Left Middle Ear, Open Approach */
"09B70ZX"  =  "1"  /* Excision of Right Tympanic Membrane, Open Approach, Diagn */
"09B70ZZ"  =  "1"  /* Excision of Right Tympanic Membrane, Open Approach */
"09B73ZX"  =  "1"  /* Excision of Right Tympanic Membrane, Perc Approach, Diagn */
"09B73ZZ"  =  "1"  /* Excision of Right Tympanic Membrane, Percutaneous Approach */
"09B74ZX"  =  "1"  /* Excision of R Tympanic Membrane, Perc Endo Approach, Diagn */
"09B74ZZ"  =  "1"  /* Excision of Right Tympanic Membrane, Perc Endo Approach */
"09B77ZX"  =  "1"  /* Excision of Right Tympanic Membrane, Via Opening, Diagn */
"09B77ZZ"  =  "1"  /* Excision of Right Tympanic Membrane, Via Opening */
"09B78ZX"  =  "1"  /* Excision of Right Tympanic Membrane, Endo, Diagn */
"09B78ZZ"  =  "1"  /* Excision of Right Tympanic Membrane, Endo */
"09B80ZX"  =  "1"  /* Excision of Left Tympanic Membrane, Open Approach, Diagn */
"09B80ZZ"  =  "1"  /* Excision of Left Tympanic Membrane, Open Approach */
"09B83ZX"  =  "1"  /* Excision of Left Tympanic Membrane, Perc Approach, Diagn */
"09B83ZZ"  =  "1"  /* Excision of Left Tympanic Membrane, Percutaneous Approach */
"09B84ZX"  =  "1"  /* Excision of L Tympanic Membrane, Perc Endo Approach, Diagn */
"09B84ZZ"  =  "1"  /* Excision of Left Tympanic Membrane, Perc Endo Approach */
"09B87ZX"  =  "1"  /* Excision of Left Tympanic Membrane, Via Opening, Diagn */
"09B87ZZ"  =  "1"  /* Excision of Left Tympanic Membrane, Via Opening */
"09B88ZX"  =  "1"  /* Excision of Left Tympanic Membrane, Endo, Diagn */
"09B88ZZ"  =  "1"  /* Excision of Left Tympanic Membrane, Endo */
"09B90ZX"  =  "1"  /* Excision of Right Auditory Ossicle, Open Approach, Diagn */
"09B90ZZ"  =  "1"  /* Excision of Right Auditory Ossicle, Open Approach */
"09BA0ZX"  =  "1"  /* Excision of Left Auditory Ossicle, Open Approach, Diagnostic */
"09BA0ZZ"  =  "1"  /* Excision of Left Auditory Ossicle, Open Approach */
"09BB0ZX"  =  "1"  /* Excision of Right Mastoid Sinus, Open Approach, Diagnostic */
"09BB0ZZ"  =  "1"  /* Excision of Right Mastoid Sinus, Open Approach */
"09BB3ZX"  =  "1"  /* Excision of Right Mastoid Sinus, Perc Approach, Diagn */
"09BB3ZZ"  =  "1"  /* Excision of Right Mastoid Sinus, Percutaneous Approach */
"09BB4ZX"  =  "1"  /* Excision of Right Mastoid Sinus, Perc Endo Approach, Diagn */
"09BB4ZZ"  =  "1"  /* Excision of Right Mastoid Sinus, Perc Endo Approach */
"09BC0ZX"  =  "1"  /* Excision of Left Mastoid Sinus, Open Approach, Diagnostic */
"09BC0ZZ"  =  "1"  /* Excision of Left Mastoid Sinus, Open Approach */
"09BC3ZX"  =  "1"  /* Excision of Left Mastoid Sinus, Percutaneous Approach, Diagn */
"09BC3ZZ"  =  "1"  /* Excision of Left Mastoid Sinus, Percutaneous Approach */
"09BC4ZX"  =  "1"  /* Excision of Left Mastoid Sinus, Perc Endo Approach, Diagn */
"09BC4ZZ"  =  "1"  /* Excision of Left Mastoid Sinus, Perc Endo Approach */
"09BD0ZX"  =  "1"  /* Excision of Right Inner Ear, Open Approach, Diagnostic */
"09BD0ZZ"  =  "1"  /* Excision of Right Inner Ear, Open Approach */
"09BE0ZX"  =  "1"  /* Excision of Left Inner Ear, Open Approach, Diagnostic */
"09BE0ZZ"  =  "1"  /* Excision of Left Inner Ear, Open Approach */
"09BL0ZZ"  =  "1"  /* Excision of Nasal Turbinate, Open Approach */
"09BL3ZZ"  =  "1"  /* Excision of Nasal Turbinate, Percutaneous Approach */
"09BL4ZZ"  =  "1"  /* Excision of Nasal Turbinate, Perc Endo Approach */
"09BL7ZZ"  =  "1"  /* Excision of Nasal Turbinate, Via Opening */
"09BL8ZZ"  =  "1"  /* Excision of Nasal Turbinate, Endo */
"09BM0ZZ"  =  "1"  /* Excision of Nasal Septum, Open Approach */
"09BM3ZZ"  =  "1"  /* Excision of Nasal Septum, Percutaneous Approach */
"09BM4ZZ"  =  "1"  /* Excision of Nasal Septum, Percutaneous Endoscopic Approach */
"09BN0ZZ"  =  "1"  /* Excision of Nasopharynx, Open Approach */
"09BN3ZZ"  =  "1"  /* Excision of Nasopharynx, Percutaneous Approach */
"09BN4ZZ"  =  "1"  /* Excision of Nasopharynx, Percutaneous Endoscopic Approach */
"09BN7ZZ"  =  "1"  /* Excision of Nasopharynx, Via Natural or Artificial Opening */
"09BN8ZZ"  =  "1"  /* Excision of Nasopharynx, Endo */
"09BP0ZX"  =  "1"  /* Excision of Accessory Sinus, Open Approach, Diagnostic */
"09BP0ZZ"  =  "1"  /* Excision of Accessory Sinus, Open Approach */
"09BP3ZZ"  =  "1"  /* Excision of Accessory Sinus, Percutaneous Approach */
"09BP4ZZ"  =  "1"  /* Excision of Accessory Sinus, Perc Endo Approach */
"09BQ0ZX"  =  "1"  /* Excision of Right Maxillary Sinus, Open Approach, Diagnostic */
"09BQ0ZZ"  =  "1"  /* Excision of Right Maxillary Sinus, Open Approach */
"09BQ3ZZ"  =  "1"  /* Excision of Right Maxillary Sinus, Percutaneous Approach */
"09BQ4ZZ"  =  "1"  /* Excision of Right Maxillary Sinus, Perc Endo Approach */
"09BR0ZX"  =  "1"  /* Excision of Left Maxillary Sinus, Open Approach, Diagnostic */
"09BR0ZZ"  =  "1"  /* Excision of Left Maxillary Sinus, Open Approach */
"09BR3ZZ"  =  "1"  /* Excision of Left Maxillary Sinus, Percutaneous Approach */
"09BR4ZZ"  =  "1"  /* Excision of Left Maxillary Sinus, Perc Endo Approach */
"09BS0ZX"  =  "1"  /* Excision of Right Frontal Sinus, Open Approach, Diagnostic */
"09BS0ZZ"  =  "1"  /* Excision of Right Frontal Sinus, Open Approach */
"09BS3ZZ"  =  "1"  /* Excision of Right Frontal Sinus, Percutaneous Approach */
"09BS4ZZ"  =  "1"  /* Excision of Right Frontal Sinus, Perc Endo Approach */
"09BT0ZX"  =  "1"  /* Excision of Left Frontal Sinus, Open Approach, Diagnostic */
"09BT0ZZ"  =  "1"  /* Excision of Left Frontal Sinus, Open Approach */
"09BT3ZZ"  =  "1"  /* Excision of Left Frontal Sinus, Percutaneous Approach */
"09BT4ZZ"  =  "1"  /* Excision of Left Frontal Sinus, Perc Endo Approach */
"09BU0ZX"  =  "1"  /* Excision of Right Ethmoid Sinus, Open Approach, Diagnostic */
"09BU0ZZ"  =  "1"  /* Excision of Right Ethmoid Sinus, Open Approach */
"09BU3ZZ"  =  "1"  /* Excision of Right Ethmoid Sinus, Percutaneous Approach */
"09BU4ZZ"  =  "1"  /* Excision of Right Ethmoid Sinus, Perc Endo Approach */
"09BV0ZX"  =  "1"  /* Excision of Left Ethmoid Sinus, Open Approach, Diagnostic */
"09BV0ZZ"  =  "1"  /* Excision of Left Ethmoid Sinus, Open Approach */
"09BV3ZZ"  =  "1"  /* Excision of Left Ethmoid Sinus, Percutaneous Approach */
"09BV4ZZ"  =  "1"  /* Excision of Left Ethmoid Sinus, Perc Endo Approach */
"09BW0ZX"  =  "1"  /* Excision of Right Sphenoid Sinus, Open Approach, Diagnostic */
"09BW0ZZ"  =  "1"  /* Excision of Right Sphenoid Sinus, Open Approach */
"09BW3ZZ"  =  "1"  /* Excision of Right Sphenoid Sinus, Percutaneous Approach */
"09BW4ZZ"  =  "1"  /* Excision of Right Sphenoid Sinus, Perc Endo Approach */
"09BX0ZX"  =  "1"  /* Excision of Left Sphenoid Sinus, Open Approach, Diagnostic */
"09BX0ZZ"  =  "1"  /* Excision of Left Sphenoid Sinus, Open Approach */
"09BX3ZZ"  =  "1"  /* Excision of Left Sphenoid Sinus, Percutaneous Approach */
"09BX4ZZ"  =  "1"  /* Excision of Left Sphenoid Sinus, Perc Endo Approach */
"09C50ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Ear, Open Approach */
"09C60ZZ"  =  "1"  /* Extirpation of Matter from Left Middle Ear, Open Approach */
"09C90ZZ"  =  "1"  /* Extirpation of Matter from R Auditory Ossicle, Open Approach */
"09CA0ZZ"  =  "1"  /* Extirpation of Matter from L Auditory Ossicle, Open Approach */
"09CB0ZZ"  =  "1"  /* Extirpation of Matter from R Mastoid Sinus, Open Approach */
"09CB3ZZ"  =  "1"  /* Extirpation of Matter from R Mastoid Sinus, Perc Approach */
"09CB4ZZ"  =  "1"  /* Extirpate of Matter from R Mastoid Sinus, Perc Endo Approach */
"09CC0ZZ"  =  "1"  /* Extirpation of Matter from Left Mastoid Sinus, Open Approach */
"09CC3ZZ"  =  "1"  /* Extirpation of Matter from Left Mastoid Sinus, Perc Approach */
"09CC4ZZ"  =  "1"  /* Extirpate of Matter from L Mastoid Sinus, Perc Endo Approach */
"09CD0ZZ"  =  "1"  /* Extirpation of Matter from Right Inner Ear, Open Approach */
"09CE0ZZ"  =  "1"  /* Extirpation of Matter from Left Inner Ear, Open Approach */
"09CN0ZZ"  =  "1"  /* Extirpation of Matter from Nasopharynx, Open Approach */
"09CN3ZZ"  =  "1"  /* Extirpation of Matter from Nasopharynx, Perc Approach */
"09CN4ZZ"  =  "1"  /* Extirpation of Matter from Nasopharynx, Perc Endo Approach */
"09CN7ZZ"  =  "1"  /* Extirpation of Matter from Nasopharynx, Via Opening */
"09CN8ZZ"  =  "1"  /* Extirpation of Matter from Nasopharynx, Endo */
"09CP0ZZ"  =  "1"  /* Extirpation of Matter from Accessory Sinus, Open Approach */
"09CP3ZZ"  =  "1"  /* Extirpation of Matter from Accessory Sinus, Perc Approach */
"09CP4ZZ"  =  "1"  /* Extirpate of Matter from Accessory Sinus, Perc Endo Approach */
"09CQ0ZZ"  =  "1"  /* Extirpation of Matter from R Maxillary Sinus, Open Approach */
"09CQ3ZZ"  =  "1"  /* Extirpation of Matter from R Maxillary Sinus, Perc Approach */
"09CQ4ZZ"  =  "1"  /* Extirpate matter from R Maxillary Sinus, Perc Endo */
"09CR0ZZ"  =  "1"  /* Extirpation of Matter from L Maxillary Sinus, Open Approach */
"09CR3ZZ"  =  "1"  /* Extirpation of Matter from L Maxillary Sinus, Perc Approach */
"09CR4ZZ"  =  "1"  /* Extirpate matter from L Maxillary Sinus, Perc Endo */
"09CS0ZZ"  =  "1"  /* Extirpation of Matter from R Frntl Sinus, Open Approach */
"09CS3ZZ"  =  "1"  /* Extirpation of Matter from R Frntl Sinus, Perc Approach */
"09CS4ZZ"  =  "1"  /* Extirpation of Matter from R Frntl Sinus, Perc Endo Approach */
"09CT0ZZ"  =  "1"  /* Extirpation of Matter from Left Frontal Sinus, Open Approach */
"09CT3ZZ"  =  "1"  /* Extirpation of Matter from Left Frontal Sinus, Perc Approach */
"09CT4ZZ"  =  "1"  /* Extirpation of Matter from L Frntl Sinus, Perc Endo Approach */
"09CU0ZZ"  =  "1"  /* Extirpation of Matter from R Ethmoid Sinus, Open Approach */
"09CU3ZZ"  =  "1"  /* Extirpation of Matter from R Ethmoid Sinus, Perc Approach */
"09CU4ZZ"  =  "1"  /* Extirpate of Matter from R Ethmoid Sinus, Perc Endo Approach */
"09CV0ZZ"  =  "1"  /* Extirpation of Matter from Left Ethmoid Sinus, Open Approach */
"09CV3ZZ"  =  "1"  /* Extirpation of Matter from Left Ethmoid Sinus, Perc Approach */
"09CV4ZZ"  =  "1"  /* Extirpate of Matter from L Ethmoid Sinus, Perc Endo Approach */
"09CW0ZZ"  =  "1"  /* Extirpation of Matter from R Sphenoid Sinus, Open Approach */
"09CW3ZZ"  =  "1"  /* Extirpation of Matter from R Sphenoid Sinus, Perc Approach */
"09CW4ZZ"  =  "1"  /* Extirpate matter from R Sphenoid Sinus, Perc Endo */
"09CX0ZZ"  =  "1"  /* Extirpation of Matter from L Sphenoid Sinus, Open Approach */
"09CX3ZZ"  =  "1"  /* Extirpation of Matter from L Sphenoid Sinus, Perc Approach */
"09CX4ZZ"  =  "1"  /* Extirpate matter from L Sphenoid Sinus, Perc Endo */
"09D70ZZ"  =  "1"  /* Extraction of Right Tympanic Membrane, Open Approach */
"09D73ZZ"  =  "1"  /* Extraction of Right Tympanic Membrane, Percutaneous Approach */
"09D74ZZ"  =  "1"  /* Extraction of Right Tympanic Membrane, Perc Endo Approach */
"09D77ZZ"  =  "1"  /* Extraction of Right Tympanic Membrane, Via Opening */
"09D78ZZ"  =  "1"  /* Extraction of Right Tympanic Membrane, Endo */
"09D80ZZ"  =  "1"  /* Extraction of Left Tympanic Membrane, Open Approach */
"09D83ZZ"  =  "1"  /* Extraction of Left Tympanic Membrane, Percutaneous Approach */
"09D84ZZ"  =  "1"  /* Extraction of Left Tympanic Membrane, Perc Endo Approach */
"09D87ZZ"  =  "1"  /* Extraction of Left Tympanic Membrane, Via Opening */
"09D88ZZ"  =  "1"  /* Extraction of Left Tympanic Membrane, Endo */
"09D90ZZ"  =  "1"  /* Extraction of Right Auditory Ossicle, Open Approach */
"09DA0ZZ"  =  "1"  /* Extraction of Left Auditory Ossicle, Open Approach */
"09DB0ZZ"  =  "1"  /* Extraction of Right Mastoid Sinus, Open Approach */
"09DB3ZZ"  =  "1"  /* Extraction of Right Mastoid Sinus, Percutaneous Approach */
"09DB4ZZ"  =  "1"  /* Extraction of Right Mastoid Sinus, Perc Endo Approach */
"09DC0ZZ"  =  "1"  /* Extraction of Left Mastoid Sinus, Open Approach */
"09DC3ZZ"  =  "1"  /* Extraction of Left Mastoid Sinus, Percutaneous Approach */
"09DC4ZZ"  =  "1"  /* Extraction of Left Mastoid Sinus, Perc Endo Approach */
"09DL0ZZ"  =  "1"  /* Extraction of Nasal Turbinate, Open Approach */
"09DL3ZZ"  =  "1"  /* Extraction of Nasal Turbinate, Percutaneous Approach */
"09DL4ZZ"  =  "1"  /* Extraction of Nasal Turbinate, Perc Endo Approach */
"09DL7ZZ"  =  "1"  /* Extraction of Nasal Turbinate, Via Opening */
"09DL8ZZ"  =  "1"  /* Extraction of Nasal Turbinate, Endo */
"09DM0ZZ"  =  "1"  /* Extraction of Nasal Septum, Open Approach */
"09DM3ZZ"  =  "1"  /* Extraction of Nasal Septum, Percutaneous Approach */
"09DM4ZZ"  =  "1"  /* Extraction of Nasal Septum, Percutaneous Endoscopic Approach */
"09DP0ZZ"  =  "1"  /* Extraction of Accessory Sinus, Open Approach */
"09DP3ZZ"  =  "1"  /* Extraction of Accessory Sinus, Percutaneous Approach */
"09DP4ZZ"  =  "1"  /* Extraction of Accessory Sinus, Perc Endo Approach */
"09DQ0ZZ"  =  "1"  /* Extraction of Right Maxillary Sinus, Open Approach */
"09DQ3ZZ"  =  "1"  /* Extraction of Right Maxillary Sinus, Percutaneous Approach */
"09DQ4ZZ"  =  "1"  /* Extraction of Right Maxillary Sinus, Perc Endo Approach */
"09DR0ZZ"  =  "1"  /* Extraction of Left Maxillary Sinus, Open Approach */
"09DR3ZZ"  =  "1"  /* Extraction of Left Maxillary Sinus, Percutaneous Approach */
"09DR4ZZ"  =  "1"  /* Extraction of Left Maxillary Sinus, Perc Endo Approach */
"09DS0ZZ"  =  "1"  /* Extraction of Right Frontal Sinus, Open Approach */
"09DS3ZZ"  =  "1"  /* Extraction of Right Frontal Sinus, Percutaneous Approach */
"09DS4ZZ"  =  "1"  /* Extraction of Right Frontal Sinus, Perc Endo Approach */
"09DT0ZZ"  =  "1"  /* Extraction of Left Frontal Sinus, Open Approach */
"09DT3ZZ"  =  "1"  /* Extraction of Left Frontal Sinus, Percutaneous Approach */
"09DT4ZZ"  =  "1"  /* Extraction of Left Frontal Sinus, Perc Endo Approach */
"09DU0ZZ"  =  "1"  /* Extraction of Right Ethmoid Sinus, Open Approach */
"09DU3ZZ"  =  "1"  /* Extraction of Right Ethmoid Sinus, Percutaneous Approach */
"09DU4ZZ"  =  "1"  /* Extraction of Right Ethmoid Sinus, Perc Endo Approach */
"09DV0ZZ"  =  "1"  /* Extraction of Left Ethmoid Sinus, Open Approach */
"09DV3ZZ"  =  "1"  /* Extraction of Left Ethmoid Sinus, Percutaneous Approach */
"09DV4ZZ"  =  "1"  /* Extraction of Left Ethmoid Sinus, Perc Endo Approach */
"09DW0ZZ"  =  "1"  /* Extraction of Right Sphenoid Sinus, Open Approach */
"09DW3ZZ"  =  "1"  /* Extraction of Right Sphenoid Sinus, Percutaneous Approach */
"09DW4ZZ"  =  "1"  /* Extraction of Right Sphenoid Sinus, Perc Endo Approach */
"09DX0ZZ"  =  "1"  /* Extraction of Left Sphenoid Sinus, Open Approach */
"09DX3ZZ"  =  "1"  /* Extraction of Left Sphenoid Sinus, Percutaneous Approach */
"09DX4ZZ"  =  "1"  /* Extraction of Left Sphenoid Sinus, Perc Endo Approach */
"09HD04Z"  =  "1"  /* Insert Bone Condct Hear Dev in R Inner Ear, Open */
"09HD05Z"  =  "1"  /* Insert Singl Chan Cochl Prosth in R Inner Ear, Open */
"09HD06Z"  =  "1"  /* Insert Mult Chan Cochl Prosth in R Inner Ear, Open */
"09HD0SZ"  =  "1"  /* Insertion of Hearing Device into R Inner Ear, Open Approach */
"09HD34Z"  =  "1"  /* Insert Bone Condct Hear Dev in R Inner Ear, Perc */
"09HD35Z"  =  "1"  /* Insert Singl Chan Cochl Prosth in R Inner Ear, Perc */
"09HD36Z"  =  "1"  /* Insert Mult Chan Cochl Prosth in R Inner Ear, Perc */
"09HD3SZ"  =  "1"  /* Insertion of Hearing Device into R Inner Ear, Perc Approach */
"09HD44Z"  =  "1"  /* Insert Bone Condct Hear Dev in R Inner Ear, Perc Endo */
"09HD45Z"  =  "1"  /* Insert Singl Chan Cochl Prosth in R Inner Ear, Perc Endo */
"09HD46Z"  =  "1"  /* Insert Mult Chan Cochl Prosth in R Inner Ear, Perc Endo */
"09HD4SZ"  =  "1"  /* Insertion of Hear Dev into R Inner Ear, Perc Endo Approach */
"09HE04Z"  =  "1"  /* Insert Bone Condct Hear Dev in L Inner Ear, Open */
"09HE05Z"  =  "1"  /* Insert Singl Chan Cochl Prosth in L Inner Ear, Open */
"09HE06Z"  =  "1"  /* Insert Mult Chan Cochl Prosth in L Inner Ear, Open */
"09HE0SZ"  =  "1"  /* Insertion of Hearing Device into L Inner Ear, Open Approach */
"09HE34Z"  =  "1"  /* Insert Bone Condct Hear Dev in L Inner Ear, Perc */
"09HE35Z"  =  "1"  /* Insert Singl Chan Cochl Prosth in L Inner Ear, Perc */
"09HE36Z"  =  "1"  /* Insert Mult Chan Cochl Prosth in L Inner Ear, Perc */
"09HE3SZ"  =  "1"  /* Insertion of Hearing Device into L Inner Ear, Perc Approach */
"09HE44Z"  =  "1"  /* Insert Bone Condct Hear Dev in L Inner Ear, Perc Endo */
"09HE45Z"  =  "1"  /* Insert Singl Chan Cochl Prosth in L Inner Ear, Perc Endo */
"09HE46Z"  =  "1"  /* Insert Mult Chan Cochl Prosth in L Inner Ear, Perc Endo */
"09HE4SZ"  =  "1"  /* Insertion of Hear Dev into L Inner Ear, Perc Endo Approach */
"09J70ZZ"  =  "1"  /* Inspection of Right Tympanic Membrane, Open Approach */
"09J73ZZ"  =  "1"  /* Inspection of Right Tympanic Membrane, Percutaneous Approach */
"09J74ZZ"  =  "1"  /* Inspection of Right Tympanic Membrane, Perc Endo Approach */
"09J77ZZ"  =  "1"  /* Inspection of Right Tympanic Membrane, Via Opening */
"09J7XZZ"  =  "1"  /* Inspection of Right Tympanic Membrane, External Approach */
"09J80ZZ"  =  "1"  /* Inspection of Left Tympanic Membrane, Open Approach */
"09J83ZZ"  =  "1"  /* Inspection of Left Tympanic Membrane, Percutaneous Approach */
"09J84ZZ"  =  "1"  /* Inspection of Left Tympanic Membrane, Perc Endo Approach */
"09J87ZZ"  =  "1"  /* Inspection of Left Tympanic Membrane, Via Opening */
"09J8XZZ"  =  "1"  /* Inspection of Left Tympanic Membrane, External Approach */
"09JD0ZZ"  =  "1"  /* Inspection of Right Inner Ear, Open Approach */
"09JD3ZZ"  =  "1"  /* Inspection of Right Inner Ear, Percutaneous Approach */
"09JD4ZZ"  =  "1"  /* Inspection of Right Inner Ear, Perc Endo Approach */
"09JDXZZ"  =  "1"  /* Inspection of Right Inner Ear, External Approach */
"09JE0ZZ"  =  "1"  /* Inspection of Left Inner Ear, Open Approach */
"09JE3ZZ"  =  "1"  /* Inspection of Left Inner Ear, Percutaneous Approach */
"09JE4ZZ"  =  "1"  /* Inspection of Left Inner Ear, Perc Endo Approach */
"09JEXZZ"  =  "1"  /* Inspection of Left Inner Ear, External Approach */
"09M0XZZ"  =  "1"  /* Reattachment of Right External Ear, External Approach */
"09M1XZZ"  =  "1"  /* Reattachment of Left External Ear, External Approach */
"09MKXZZ"  =  "1"  /* Reattachment of Nose, External Approach */
"09N00ZZ"  =  "1"  /* Release Right External Ear, Open Approach */
"09N03ZZ"  =  "1"  /* Release Right External Ear, Percutaneous Approach */
"09N04ZZ"  =  "1"  /* Release Right External Ear, Percutaneous Endoscopic Approach */
"09N0XZZ"  =  "1"  /* Release Right External Ear, External Approach */
"09N10ZZ"  =  "1"  /* Release Left External Ear, Open Approach */
"09N13ZZ"  =  "1"  /* Release Left External Ear, Percutaneous Approach */
"09N14ZZ"  =  "1"  /* Release Left External Ear, Percutaneous Endoscopic Approach */
"09N1XZZ"  =  "1"  /* Release Left External Ear, External Approach */
"09N30ZZ"  =  "1"  /* Release Right External Auditory Canal, Open Approach */
"09N33ZZ"  =  "1"  /* Release Right External Auditory Canal, Percutaneous Approach */
"09N34ZZ"  =  "1"  /* Release Right External Auditory Canal, Perc Endo Approach */
"09N37ZZ"  =  "1"  /* Release Right External Auditory Canal, Via Opening */
"09N38ZZ"  =  "1"  /* Release Right External Auditory Canal, Endo */
"09N3XZZ"  =  "1"  /* Release Right External Auditory Canal, External Approach */
"09N40ZZ"  =  "1"  /* Release Left External Auditory Canal, Open Approach */
"09N43ZZ"  =  "1"  /* Release Left External Auditory Canal, Percutaneous Approach */
"09N44ZZ"  =  "1"  /* Release Left External Auditory Canal, Perc Endo Approach */
"09N47ZZ"  =  "1"  /* Release Left External Auditory Canal, Via Opening */
"09N48ZZ"  =  "1"  /* Release Left External Auditory Canal, Endo */
"09N4XZZ"  =  "1"  /* Release Left External Auditory Canal, External Approach */
"09N50ZZ"  =  "1"  /* Release Right Middle Ear, Open Approach */
"09N60ZZ"  =  "1"  /* Release Left Middle Ear, Open Approach */
"09N70ZZ"  =  "1"  /* Release Right Tympanic Membrane, Open Approach */
"09N73ZZ"  =  "1"  /* Release Right Tympanic Membrane, Percutaneous Approach */
"09N74ZZ"  =  "1"  /* Release Right Tympanic Membrane, Perc Endo Approach */
"09N77ZZ"  =  "1"  /* Release Right Tympanic Membrane, Via Opening */
"09N78ZZ"  =  "1"  /* Release Right Tympanic Membrane, Endo */
"09N80ZZ"  =  "1"  /* Release Left Tympanic Membrane, Open Approach */
"09N83ZZ"  =  "1"  /* Release Left Tympanic Membrane, Percutaneous Approach */
"09N84ZZ"  =  "1"  /* Release Left Tympanic Membrane, Perc Endo Approach */
"09N87ZZ"  =  "1"  /* Release Left Tympanic Membrane, Via Opening */
"09N88ZZ"  =  "1"  /* Release Left Tympanic Membrane, Endo */
"09N90ZZ"  =  "1"  /* Release Right Auditory Ossicle, Open Approach */
"09NA0ZZ"  =  "1"  /* Release Left Auditory Ossicle, Open Approach */
"09NB0ZZ"  =  "1"  /* Release Right Mastoid Sinus, Open Approach */
"09NB3ZZ"  =  "1"  /* Release Right Mastoid Sinus, Percutaneous Approach */
"09NB4ZZ"  =  "1"  /* Release Right Mastoid Sinus, Perc Endo Approach */
"09NC0ZZ"  =  "1"  /* Release Left Mastoid Sinus, Open Approach */
"09NC3ZZ"  =  "1"  /* Release Left Mastoid Sinus, Percutaneous Approach */
"09NC4ZZ"  =  "1"  /* Release Left Mastoid Sinus, Percutaneous Endoscopic Approach */
"09ND0ZZ"  =  "1"  /* Release Right Inner Ear, Open Approach */
"09NE0ZZ"  =  "1"  /* Release Left Inner Ear, Open Approach */
"09NN0ZZ"  =  "1"  /* Release Nasopharynx, Open Approach */
"09NN3ZZ"  =  "1"  /* Release Nasopharynx, Percutaneous Approach */
"09NN4ZZ"  =  "1"  /* Release Nasopharynx, Percutaneous Endoscopic Approach */
"09NN7ZZ"  =  "1"  /* Release Nasopharynx, Via Natural or Artificial Opening */
"09NN8ZZ"  =  "1"  /* Release Nasopharynx, Endo */
"09NP0ZZ"  =  "1"  /* Release Accessory Sinus, Open Approach */
"09NP3ZZ"  =  "1"  /* Release Accessory Sinus, Percutaneous Approach */
"09NP4ZZ"  =  "1"  /* Release Accessory Sinus, Percutaneous Endoscopic Approach */
"09NQ0ZZ"  =  "1"  /* Release Right Maxillary Sinus, Open Approach */
"09NQ3ZZ"  =  "1"  /* Release Right Maxillary Sinus, Percutaneous Approach */
"09NQ4ZZ"  =  "1"  /* Release Right Maxillary Sinus, Perc Endo Approach */
"09NR0ZZ"  =  "1"  /* Release Left Maxillary Sinus, Open Approach */
"09NR3ZZ"  =  "1"  /* Release Left Maxillary Sinus, Percutaneous Approach */
"09NR4ZZ"  =  "1"  /* Release Left Maxillary Sinus, Perc Endo Approach */
"09NS0ZZ"  =  "1"  /* Release Right Frontal Sinus, Open Approach */
"09NS3ZZ"  =  "1"  /* Release Right Frontal Sinus, Percutaneous Approach */
"09NS4ZZ"  =  "1"  /* Release Right Frontal Sinus, Perc Endo Approach */
"09NT0ZZ"  =  "1"  /* Release Left Frontal Sinus, Open Approach */
"09NT3ZZ"  =  "1"  /* Release Left Frontal Sinus, Percutaneous Approach */
"09NT4ZZ"  =  "1"  /* Release Left Frontal Sinus, Percutaneous Endoscopic Approach */
"09NU0ZZ"  =  "1"  /* Release Right Ethmoid Sinus, Open Approach */
"09NU3ZZ"  =  "1"  /* Release Right Ethmoid Sinus, Percutaneous Approach */
"09NU4ZZ"  =  "1"  /* Release Right Ethmoid Sinus, Perc Endo Approach */
"09NV0ZZ"  =  "1"  /* Release Left Ethmoid Sinus, Open Approach */
"09NV3ZZ"  =  "1"  /* Release Left Ethmoid Sinus, Percutaneous Approach */
"09NV4ZZ"  =  "1"  /* Release Left Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09NW0ZZ"  =  "1"  /* Release Right Sphenoid Sinus, Open Approach */
"09NW3ZZ"  =  "1"  /* Release Right Sphenoid Sinus, Percutaneous Approach */
"09NW4ZZ"  =  "1"  /* Release Right Sphenoid Sinus, Perc Endo Approach */
"09NX0ZZ"  =  "1"  /* Release Left Sphenoid Sinus, Open Approach */
"09NX3ZZ"  =  "1"  /* Release Left Sphenoid Sinus, Percutaneous Approach */
"09NX4ZZ"  =  "1"  /* Release Left Sphenoid Sinus, Perc Endo Approach */
"09PD0SZ"  =  "1"  /* Removal of Hearing Device from R Inner Ear, Open Approach */
"09PD7SZ"  =  "1"  /* Removal of Hearing Device from Right Inner Ear, Via Opening */
"09PD8SZ"  =  "1"  /* Removal of Hearing Device from Right Inner Ear, Endo */
"09PE0SZ"  =  "1"  /* Removal of Hearing Device from Left Inner Ear, Open Approach */
"09PE7SZ"  =  "1"  /* Removal of Hearing Device from Left Inner Ear, Via Opening */
"09PE8SZ"  =  "1"  /* Removal of Hearing Device from Left Inner Ear, Endo */
"09PH00Z"  =  "1"  /* Removal of Drainage Device from Right Ear, Open Approach */
"09PH07Z"  =  "1"  /* Removal of Autol Sub from R Ear, Open Approach */
"09PH0DZ"  =  "1"  /* Removal of Intraluminal Device from Right Ear, Open Approach */
"09PH0JZ"  =  "1"  /* Removal of Synthetic Substitute from R Ear, Open Approach */
"09PH0KZ"  =  "1"  /* Removal of Nonaut Sub from R Ear, Open Approach */
"09PH37Z"  =  "1"  /* Removal of Autol Sub from R Ear, Perc Approach */
"09PH3DZ"  =  "1"  /* Removal of Intraluminal Device from Right Ear, Perc Approach */
"09PH47Z"  =  "1"  /* Removal of Autol Sub from R Ear, Perc Endo Approach */
"09PH4DZ"  =  "1"  /* Removal of Intralum Dev from R Ear, Perc Endo Approach */
"09PH77Z"  =  "1"  /* Removal of Autol Sub from R Ear, Via Opening */
"09PH7JZ"  =  "1"  /* Removal of Synthetic Substitute from Right Ear, Via Opening */
"09PH7KZ"  =  "1"  /* Removal of Nonaut Sub from R Ear, Via Opening */
"09PH87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Right Ear, Endo */
"09PH8JZ"  =  "1"  /* Removal of Synthetic Substitute from Right Ear, Endo */
"09PH8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from R Ear, Endo */
"09PJ00Z"  =  "1"  /* Removal of Drainage Device from Left Ear, Open Approach */
"09PJ07Z"  =  "1"  /* Removal of Autol Sub from L Ear, Open Approach */
"09PJ0DZ"  =  "1"  /* Removal of Intraluminal Device from Left Ear, Open Approach */
"09PJ0JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Ear, Open Approach */
"09PJ0KZ"  =  "1"  /* Removal of Nonaut Sub from L Ear, Open Approach */
"09PJ37Z"  =  "1"  /* Removal of Autol Sub from L Ear, Perc Approach */
"09PJ3DZ"  =  "1"  /* Removal of Intraluminal Device from Left Ear, Perc Approach */
"09PJ47Z"  =  "1"  /* Removal of Autol Sub from L Ear, Perc Endo Approach */
"09PJ4DZ"  =  "1"  /* Removal of Intralum Dev from L Ear, Perc Endo Approach */
"09PJ77Z"  =  "1"  /* Removal of Autol Sub from L Ear, Via Opening */
"09PJ7JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Ear, Via Opening */
"09PJ7KZ"  =  "1"  /* Removal of Nonaut Sub from L Ear, Via Opening */
"09PJ87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Left Ear, Endo */
"09PJ8JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Ear, Endo */
"09PJ8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from L Ear, Endo */
"09PY00Z"  =  "1"  /* Removal of Drainage Device from Sinus, Open Approach */
"09PY30Z"  =  "1"  /* Removal of Drainage Device from Sinus, Percutaneous Approach */
"09PY40Z"  =  "1"  /* Removal of Drainage Device from Sinus, Perc Endo Approach */
"09Q00ZZ"  =  "1"  /* Repair Right External Ear, Open Approach */
"09Q03ZZ"  =  "1"  /* Repair Right External Ear, Percutaneous Approach */
"09Q04ZZ"  =  "1"  /* Repair Right External Ear, Percutaneous Endoscopic Approach */
"09Q10ZZ"  =  "1"  /* Repair Left External Ear, Open Approach */
"09Q13ZZ"  =  "1"  /* Repair Left External Ear, Percutaneous Approach */
"09Q14ZZ"  =  "1"  /* Repair Left External Ear, Percutaneous Endoscopic Approach */
"09Q20ZZ"  =  "1"  /* Repair Bilateral External Ear, Open Approach */
"09Q23ZZ"  =  "1"  /* Repair Bilateral External Ear, Percutaneous Approach */
"09Q24ZZ"  =  "1"  /* Repair Bilateral External Ear, Perc Endo Approach */
"09Q30ZZ"  =  "1"  /* Repair Right External Auditory Canal, Open Approach */
"09Q33ZZ"  =  "1"  /* Repair Right External Auditory Canal, Percutaneous Approach */
"09Q34ZZ"  =  "1"  /* Repair Right External Auditory Canal, Perc Endo Approach */
"09Q37ZZ"  =  "1"  /* Repair Right External Auditory Canal, Via Opening */
"09Q38ZZ"  =  "1"  /* Repair Right External Auditory Canal, Endo */
"09Q40ZZ"  =  "1"  /* Repair Left External Auditory Canal, Open Approach */
"09Q43ZZ"  =  "1"  /* Repair Left External Auditory Canal, Percutaneous Approach */
"09Q44ZZ"  =  "1"  /* Repair Left External Auditory Canal, Perc Endo Approach */
"09Q47ZZ"  =  "1"  /* Repair Left External Auditory Canal, Via Opening */
"09Q48ZZ"  =  "1"  /* Repair Left External Auditory Canal, Endo */
"09Q50ZZ"  =  "1"  /* Repair Right Middle Ear, Open Approach */
"09Q60ZZ"  =  "1"  /* Repair Left Middle Ear, Open Approach */
"09Q70ZZ"  =  "1"  /* Repair Right Tympanic Membrane, Open Approach */
"09Q73ZZ"  =  "1"  /* Repair Right Tympanic Membrane, Percutaneous Approach */
"09Q74ZZ"  =  "1"  /* Repair Right Tympanic Membrane, Perc Endo Approach */
"09Q77ZZ"  =  "1"  /* Repair Right Tympanic Membrane, Via Opening */
"09Q78ZZ"  =  "1"  /* Repair Right Tympanic Membrane, Endo */
"09Q80ZZ"  =  "1"  /* Repair Left Tympanic Membrane, Open Approach */
"09Q83ZZ"  =  "1"  /* Repair Left Tympanic Membrane, Percutaneous Approach */
"09Q84ZZ"  =  "1"  /* Repair Left Tympanic Membrane, Perc Endo Approach */
"09Q87ZZ"  =  "1"  /* Repair Left Tympanic Membrane, Via Opening */
"09Q88ZZ"  =  "1"  /* Repair Left Tympanic Membrane, Endo */
"09Q90ZZ"  =  "1"  /* Repair Right Auditory Ossicle, Open Approach */
"09QA0ZZ"  =  "1"  /* Repair Left Auditory Ossicle, Open Approach */
"09QB0ZZ"  =  "1"  /* Repair Right Mastoid Sinus, Open Approach */
"09QB3ZZ"  =  "1"  /* Repair Right Mastoid Sinus, Percutaneous Approach */
"09QB4ZZ"  =  "1"  /* Repair Right Mastoid Sinus, Percutaneous Endoscopic Approach */
"09QC0ZZ"  =  "1"  /* Repair Left Mastoid Sinus, Open Approach */
"09QC3ZZ"  =  "1"  /* Repair Left Mastoid Sinus, Percutaneous Approach */
"09QC4ZZ"  =  "1"  /* Repair Left Mastoid Sinus, Percutaneous Endoscopic Approach */
"09QD0ZZ"  =  "1"  /* Repair Right Inner Ear, Open Approach */
"09QE0ZZ"  =  "1"  /* Repair Left Inner Ear, Open Approach */
"09QK0ZZ"  =  "1"  /* Repair Nose, Open Approach */
"09QK3ZZ"  =  "1"  /* Repair Nose, Percutaneous Approach */
"09QK4ZZ"  =  "1"  /* Repair Nose, Percutaneous Endoscopic Approach */
"09QKXZZ"  =  "1"  /* Repair Nose, External Approach */
"09QL0ZZ"  =  "1"  /* Repair Nasal Turbinate, Open Approach */
"09QL3ZZ"  =  "1"  /* Repair Nasal Turbinate, Percutaneous Approach */
"09QL4ZZ"  =  "1"  /* Repair Nasal Turbinate, Percutaneous Endoscopic Approach */
"09QL7ZZ"  =  "1"  /* Repair Nasal Turbinate, Via Natural or Artificial Opening */
"09QL8ZZ"  =  "1"  /* Repair Nasal Turbinate, Endo */
"09QM0ZZ"  =  "1"  /* Repair Nasal Septum, Open Approach */
"09QM3ZZ"  =  "1"  /* Repair Nasal Septum, Percutaneous Approach */
"09QM4ZZ"  =  "1"  /* Repair Nasal Septum, Percutaneous Endoscopic Approach */
"09QN0ZZ"  =  "1"  /* Repair Nasopharynx, Open Approach */
"09QN3ZZ"  =  "1"  /* Repair Nasopharynx, Percutaneous Approach */
"09QN4ZZ"  =  "1"  /* Repair Nasopharynx, Percutaneous Endoscopic Approach */
"09QN7ZZ"  =  "1"  /* Repair Nasopharynx, Via Natural or Artificial Opening */
"09QN8ZZ"  =  "1"  /* Repair Nasopharynx, Endo */
"09QP0ZZ"  =  "1"  /* Repair Accessory Sinus, Open Approach */
"09QP3ZZ"  =  "1"  /* Repair Accessory Sinus, Percutaneous Approach */
"09QP4ZZ"  =  "1"  /* Repair Accessory Sinus, Percutaneous Endoscopic Approach */
"09QQ0ZZ"  =  "1"  /* Repair Right Maxillary Sinus, Open Approach */
"09QQ3ZZ"  =  "1"  /* Repair Right Maxillary Sinus, Percutaneous Approach */
"09QQ4ZZ"  =  "1"  /* Repair Right Maxillary Sinus, Perc Endo Approach */
"09QR0ZZ"  =  "1"  /* Repair Left Maxillary Sinus, Open Approach */
"09QR3ZZ"  =  "1"  /* Repair Left Maxillary Sinus, Percutaneous Approach */
"09QR4ZZ"  =  "1"  /* Repair Left Maxillary Sinus, Perc Endo Approach */
"09QS0ZZ"  =  "1"  /* Repair Right Frontal Sinus, Open Approach */
"09QS3ZZ"  =  "1"  /* Repair Right Frontal Sinus, Percutaneous Approach */
"09QS4ZZ"  =  "1"  /* Repair Right Frontal Sinus, Percutaneous Endoscopic Approach */
"09QT0ZZ"  =  "1"  /* Repair Left Frontal Sinus, Open Approach */
"09QT3ZZ"  =  "1"  /* Repair Left Frontal Sinus, Percutaneous Approach */
"09QT4ZZ"  =  "1"  /* Repair Left Frontal Sinus, Percutaneous Endoscopic Approach */
"09QU0ZZ"  =  "1"  /* Repair Right Ethmoid Sinus, Open Approach */
"09QU3ZZ"  =  "1"  /* Repair Right Ethmoid Sinus, Percutaneous Approach */
"09QU4ZZ"  =  "1"  /* Repair Right Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09QV0ZZ"  =  "1"  /* Repair Left Ethmoid Sinus, Open Approach */
"09QV3ZZ"  =  "1"  /* Repair Left Ethmoid Sinus, Percutaneous Approach */
"09QV4ZZ"  =  "1"  /* Repair Left Ethmoid Sinus, Percutaneous Endoscopic Approach */
"09QW0ZZ"  =  "1"  /* Repair Right Sphenoid Sinus, Open Approach */
"09QW3ZZ"  =  "1"  /* Repair Right Sphenoid Sinus, Percutaneous Approach */
"09QW4ZZ"  =  "1"  /* Repair Right Sphenoid Sinus, Perc Endo Approach */
"09QX0ZZ"  =  "1"  /* Repair Left Sphenoid Sinus, Open Approach */
"09QX3ZZ"  =  "1"  /* Repair Left Sphenoid Sinus, Percutaneous Approach */
"09QX4ZZ"  =  "1"  /* Repair Left Sphenoid Sinus, Percutaneous Endoscopic Approach */
"09R007Z"  =  "1"  /* Replacement of R Ext Ear with Autol Sub, Open Approach */
"09R00JZ"  =  "1"  /* Replacement of R Ext Ear with Synth Sub, Open Approach */
"09R00KZ"  =  "1"  /* Replacement of R Ext Ear with Nonaut Sub, Open Approach */
"09R0X7Z"  =  "1"  /* Replacement of R Ext Ear with Autol Sub, Extern Approach */
"09R0XJZ"  =  "1"  /* Replacement of R Ext Ear with Synth Sub, Extern Approach */
"09R0XKZ"  =  "1"  /* Replacement of R Ext Ear with Nonaut Sub, Extern Approach */
"09R107Z"  =  "1"  /* Replacement of L Ext Ear with Autol Sub, Open Approach */
"09R10JZ"  =  "1"  /* Replacement of L Ext Ear with Synth Sub, Open Approach */
"09R10KZ"  =  "1"  /* Replacement of L Ext Ear with Nonaut Sub, Open Approach */
"09R1X7Z"  =  "1"  /* Replacement of L Ext Ear with Autol Sub, Extern Approach */
"09R1XJZ"  =  "1"  /* Replacement of L Ext Ear with Synth Sub, Extern Approach */
"09R1XKZ"  =  "1"  /* Replacement of L Ext Ear with Nonaut Sub, Extern Approach */
"09R207Z"  =  "1"  /* Replacement of Bi Ext Ear with Autol Sub, Open Approach */
"09R20JZ"  =  "1"  /* Replacement of Bi Ext Ear with Synth Sub, Open Approach */
"09R20KZ"  =  "1"  /* Replacement of Bi Ext Ear with Nonaut Sub, Open Approach */
"09R2X7Z"  =  "1"  /* Replacement of Bi Ext Ear with Autol Sub, Extern Approach */
"09R2XJZ"  =  "1"  /* Replacement of Bi Ext Ear with Synth Sub, Extern Approach */
"09R2XKZ"  =  "1"  /* Replacement of Bi Ext Ear with Nonaut Sub, Extern Approach */
"09R507Z"  =  "1"  /* Replacement of R Mid Ear with Autol Sub, Open Approach */
"09R50JZ"  =  "1"  /* Replacement of R Mid Ear with Synth Sub, Open Approach */
"09R50KZ"  =  "1"  /* Replacement of R Mid Ear with Nonaut Sub, Open Approach */
"09R607Z"  =  "1"  /* Replacement of Left Middle Ear with Autol Sub, Open Approach */
"09R60JZ"  =  "1"  /* Replacement of Left Middle Ear with Synth Sub, Open Approach */
"09R60KZ"  =  "1"  /* Replacement of L Mid Ear with Nonaut Sub, Open Approach */
"09R707Z"  =  "1"  /* Replace of R Tympanic Membrane with Autol Sub, Open Approach */
"09R70JZ"  =  "1"  /* Replace of R Tympanic Membrane with Synth Sub, Open Approach */
"09R70KZ"  =  "1"  /* Replace R Tympanic Membrane w Nonaut Sub, Open */
"09R777Z"  =  "1"  /* Replace of R Tympanic Membrane with Autol Sub, Via Opening */
"09R77JZ"  =  "1"  /* Replace of R Tympanic Membrane with Synth Sub, Via Opening */
"09R77KZ"  =  "1"  /* Replace of R Tympanic Membrane with Nonaut Sub, Via Opening */
"09R787Z"  =  "1"  /* Replacement of Right Tympanic Membrane with Autol Sub, Endo */
"09R78JZ"  =  "1"  /* Replacement of Right Tympanic Membrane with Synth Sub, Endo */
"09R78KZ"  =  "1"  /* Replacement of Right Tympanic Membrane with Nonaut Sub, Endo */
"09R807Z"  =  "1"  /* Replace of L Tympanic Membrane with Autol Sub, Open Approach */
"09R80JZ"  =  "1"  /* Replace of L Tympanic Membrane with Synth Sub, Open Approach */
"09R80KZ"  =  "1"  /* Replace L Tympanic Membrane w Nonaut Sub, Open */
"09R877Z"  =  "1"  /* Replace of L Tympanic Membrane with Autol Sub, Via Opening */
"09R87JZ"  =  "1"  /* Replace of L Tympanic Membrane with Synth Sub, Via Opening */
"09R87KZ"  =  "1"  /* Replace of L Tympanic Membrane with Nonaut Sub, Via Opening */
"09R887Z"  =  "1"  /* Replacement of Left Tympanic Membrane with Autol Sub, Endo */
"09R88JZ"  =  "1"  /* Replacement of Left Tympanic Membrane with Synth Sub, Endo */
"09R88KZ"  =  "1"  /* Replacement of Left Tympanic Membrane with Nonaut Sub, Endo */
"09R907Z"  =  "1"  /* Replace of R Auditory Ossicle with Autol Sub, Open Approach */
"09R90JZ"  =  "1"  /* Replace of R Auditory Ossicle with Synth Sub, Open Approach */
"09R90KZ"  =  "1"  /* Replace of R Auditory Ossicle with Nonaut Sub, Open Approach */
"09RA07Z"  =  "1"  /* Replace of L Auditory Ossicle with Autol Sub, Open Approach */
"09RA0JZ"  =  "1"  /* Replace of L Auditory Ossicle with Synth Sub, Open Approach */
"09RA0KZ"  =  "1"  /* Replace of L Auditory Ossicle with Nonaut Sub, Open Approach */
"09RD07Z"  =  "1"  /* Replacement of Right Inner Ear with Autol Sub, Open Approach */
"09RD0JZ"  =  "1"  /* Replacement of Right Inner Ear with Synth Sub, Open Approach */
"09RD0KZ"  =  "1"  /* Replacement of R Inner Ear with Nonaut Sub, Open Approach */
"09RE07Z"  =  "1"  /* Replacement of Left Inner Ear with Autol Sub, Open Approach */
"09RE0JZ"  =  "1"  /* Replacement of Left Inner Ear with Synth Sub, Open Approach */
"09RE0KZ"  =  "1"  /* Replacement of Left Inner Ear with Nonaut Sub, Open Approach */
"09RK07Z"  =  "1"  /* Replacement of Nose with Autol Sub, Open Approach */
"09RK0JZ"  =  "1"  /* Replacement of Nose with Synthetic Substitute, Open Approach */
"09RK0KZ"  =  "1"  /* Replacement of Nose with Nonaut Sub, Open Approach */
"09RKX7Z"  =  "1"  /* Replacement of Nose with Autol Sub, Extern Approach */
"09RKXJZ"  =  "1"  /* Replacement of Nose with Synth Sub, Extern Approach */
"09RKXKZ"  =  "1"  /* Replacement of Nose with Nonaut Sub, Extern Approach */
"09RL07Z"  =  "1"  /* Replacement of Nasal Turbinate with Autol Sub, Open Approach */
"09RL0JZ"  =  "1"  /* Replacement of Nasal Turbinate with Synth Sub, Open Approach */
"09RL0KZ"  =  "1"  /* Replace of Nasal Turbinate with Nonaut Sub, Open Approach */
"09RL37Z"  =  "1"  /* Replacement of Nasal Turbinate with Autol Sub, Perc Approach */
"09RL3JZ"  =  "1"  /* Replacement of Nasal Turbinate with Synth Sub, Perc Approach */
"09RL3KZ"  =  "1"  /* Replace of Nasal Turbinate with Nonaut Sub, Perc Approach */
"09RL47Z"  =  "1"  /* Replace Nasal Turbinate w Autol Sub, Perc Endo */
"09RL4JZ"  =  "1"  /* Replace Nasal Turbinate w Synth Sub, Perc Endo */
"09RL4KZ"  =  "1"  /* Replace Nasal Turbinate w Nonaut Sub, Perc Endo */
"09RL77Z"  =  "1"  /* Replacement of Nasal Turbinate with Autol Sub, Via Opening */
"09RL7JZ"  =  "1"  /* Replacement of Nasal Turbinate with Synth Sub, Via Opening */
"09RL7KZ"  =  "1"  /* Replacement of Nasal Turbinate with Nonaut Sub, Via Opening */
"09RL87Z"  =  "1"  /* Replacement of Nasal Turbinate with Autol Sub, Endo */
"09RL8JZ"  =  "1"  /* Replacement of Nasal Turbinate with Synth Sub, Endo */
"09RL8KZ"  =  "1"  /* Replacement of Nasal Turbinate with Nonaut Sub, Endo */
"09RM07Z"  =  "1"  /* Replacement of Nasal Septum with Autol Sub, Open Approach */
"09RM0JZ"  =  "1"  /* Replacement of Nasal Septum with Synth Sub, Open Approach */
"09RM0KZ"  =  "1"  /* Replacement of Nasal Septum with Nonaut Sub, Open Approach */
"09RM37Z"  =  "1"  /* Replacement of Nasal Septum with Autol Sub, Perc Approach */
"09RM3JZ"  =  "1"  /* Replacement of Nasal Septum with Synth Sub, Perc Approach */
"09RM3KZ"  =  "1"  /* Replacement of Nasal Septum with Nonaut Sub, Perc Approach */
"09RM47Z"  =  "1"  /* Replace of Nasal Septum with Autol Sub, Perc Endo Approach */
"09RM4JZ"  =  "1"  /* Replace of Nasal Septum with Synth Sub, Perc Endo Approach */
"09RM4KZ"  =  "1"  /* Replace of Nasal Septum with Nonaut Sub, Perc Endo Approach */
"09RN07Z"  =  "1"  /* Replacement of Nasopharynx with Autol Sub, Open Approach */
"09RN0JZ"  =  "1"  /* Replacement of Nasopharynx with Synth Sub, Open Approach */
"09RN0KZ"  =  "1"  /* Replacement of Nasopharynx with Nonaut Sub, Open Approach */
"09RN77Z"  =  "1"  /* Replacement of Nasopharynx with Autol Sub, Via Opening */
"09RN7JZ"  =  "1"  /* Replacement of Nasopharynx with Synth Sub, Via Opening */
"09RN7KZ"  =  "1"  /* Replacement of Nasopharynx with Nonaut Sub, Via Opening */
"09RN87Z"  =  "1"  /* Replacement of Nasopharynx with Autol Sub, Endo */
"09RN8JZ"  =  "1"  /* Replacement of Nasopharynx with Synthetic Substitute, Endo */
"09RN8KZ"  =  "1"  /* Replacement of Nasopharynx with Nonaut Sub, Endo */
"09S00ZZ"  =  "1"  /* Reposition Right External Ear, Open Approach */
"09S04ZZ"  =  "1"  /* Reposition Right External Ear, Perc Endo Approach */
"09S0XZZ"  =  "1"  /* Reposition Right External Ear, External Approach */
"09S10ZZ"  =  "1"  /* Reposition Left External Ear, Open Approach */
"09S14ZZ"  =  "1"  /* Reposition Left External Ear, Perc Endo Approach */
"09S1XZZ"  =  "1"  /* Reposition Left External Ear, External Approach */
"09S20ZZ"  =  "1"  /* Reposition Bilateral External Ear, Open Approach */
"09S24ZZ"  =  "1"  /* Reposition Bilateral External Ear, Perc Endo Approach */
"09S2XZZ"  =  "1"  /* Reposition Bilateral External Ear, External Approach */
"09S70ZZ"  =  "1"  /* Reposition Right Tympanic Membrane, Open Approach */
"09S74ZZ"  =  "1"  /* Reposition Right Tympanic Membrane, Perc Endo Approach */
"09S77ZZ"  =  "1"  /* Reposition Right Tympanic Membrane, Via Opening */
"09S78ZZ"  =  "1"  /* Reposition Right Tympanic Membrane, Endo */
"09S80ZZ"  =  "1"  /* Reposition Left Tympanic Membrane, Open Approach */
"09S84ZZ"  =  "1"  /* Reposition Left Tympanic Membrane, Perc Endo Approach */
"09S87ZZ"  =  "1"  /* Reposition Left Tympanic Membrane, Via Opening */
"09S88ZZ"  =  "1"  /* Reposition Left Tympanic Membrane, Endo */
"09S90ZZ"  =  "1"  /* Reposition Right Auditory Ossicle, Open Approach */
"09S94ZZ"  =  "1"  /* Reposition Right Auditory Ossicle, Perc Endo Approach */
"09SA0ZZ"  =  "1"  /* Reposition Left Auditory Ossicle, Open Approach */
"09SA4ZZ"  =  "1"  /* Reposition Left Auditory Ossicle, Perc Endo Approach */
"09SK0ZZ"  =  "1"  /* Reposition Nose, Open Approach */
"09SK4ZZ"  =  "1"  /* Reposition Nose, Percutaneous Endoscopic Approach */
"09SKXZZ"  =  "1"  /* Reposition Nose, External Approach */
"09SL0ZZ"  =  "1"  /* Reposition Nasal Turbinate, Open Approach */
"09SL4ZZ"  =  "1"  /* Reposition Nasal Turbinate, Percutaneous Endoscopic Approach */
"09SL7ZZ"  =  "1"  /* Reposition Nasal Turbinate, Via Opening */
"09SL8ZZ"  =  "1"  /* Reposition Nasal Turbinate, Endo */
"09SM0ZZ"  =  "1"  /* Reposition Nasal Septum, Open Approach */
"09SM4ZZ"  =  "1"  /* Reposition Nasal Septum, Percutaneous Endoscopic Approach */
"09T00ZZ"  =  "1"  /* Resection of Right External Ear, Open Approach */
"09T04ZZ"  =  "1"  /* Resection of Right External Ear, Perc Endo Approach */
"09T0XZZ"  =  "1"  /* Resection of Right External Ear, External Approach */
"09T10ZZ"  =  "1"  /* Resection of Left External Ear, Open Approach */
"09T14ZZ"  =  "1"  /* Resection of Left External Ear, Perc Endo Approach */
"09T1XZZ"  =  "1"  /* Resection of Left External Ear, External Approach */
"09T50ZZ"  =  "1"  /* Resection of Right Middle Ear, Open Approach */
"09T60ZZ"  =  "1"  /* Resection of Left Middle Ear, Open Approach */
"09T70ZZ"  =  "1"  /* Resection of Right Tympanic Membrane, Open Approach */
"09T74ZZ"  =  "1"  /* Resection of Right Tympanic Membrane, Perc Endo Approach */
"09T77ZZ"  =  "1"  /* Resection of Right Tympanic Membrane, Via Opening */
"09T78ZZ"  =  "1"  /* Resection of Right Tympanic Membrane, Endo */
"09T80ZZ"  =  "1"  /* Resection of Left Tympanic Membrane, Open Approach */
"09T84ZZ"  =  "1"  /* Resection of Left Tympanic Membrane, Perc Endo Approach */
"09T87ZZ"  =  "1"  /* Resection of Left Tympanic Membrane, Via Opening */
"09T88ZZ"  =  "1"  /* Resection of Left Tympanic Membrane, Endo */
"09T90ZZ"  =  "1"  /* Resection of Right Auditory Ossicle, Open Approach */
"09TA0ZZ"  =  "1"  /* Resection of Left Auditory Ossicle, Open Approach */
"09TB0ZZ"  =  "1"  /* Resection of Right Mastoid Sinus, Open Approach */
"09TB4ZZ"  =  "1"  /* Resection of Right Mastoid Sinus, Perc Endo Approach */
"09TC0ZZ"  =  "1"  /* Resection of Left Mastoid Sinus, Open Approach */
"09TC4ZZ"  =  "1"  /* Resection of Left Mastoid Sinus, Perc Endo Approach */
"09TD0ZZ"  =  "1"  /* Resection of Right Inner Ear, Open Approach */
"09TE0ZZ"  =  "1"  /* Resection of Left Inner Ear, Open Approach */
"09TK0ZZ"  =  "1"  /* Resection of Nose, Open Approach */
"09TK4ZZ"  =  "1"  /* Resection of Nose, Percutaneous Endoscopic Approach */
"09TKXZZ"  =  "1"  /* Resection of Nose, External Approach */
"09TL0ZZ"  =  "1"  /* Resection of Nasal Turbinate, Open Approach */
"09TL4ZZ"  =  "1"  /* Resection of Nasal Turbinate, Perc Endo Approach */
"09TL7ZZ"  =  "1"  /* Resection of Nasal Turbinate, Via Opening */
"09TL8ZZ"  =  "1"  /* Resection of Nasal Turbinate, Endo */
"09TM0ZZ"  =  "1"  /* Resection of Nasal Septum, Open Approach */
"09TM4ZZ"  =  "1"  /* Resection of Nasal Septum, Percutaneous Endoscopic Approach */
"09TN0ZZ"  =  "1"  /* Resection of Nasopharynx, Open Approach */
"09TN4ZZ"  =  "1"  /* Resection of Nasopharynx, Percutaneous Endoscopic Approach */
"09TN7ZZ"  =  "1"  /* Resection of Nasopharynx, Via Natural or Artificial Opening */
"09TN8ZZ"  =  "1"  /* Resection of Nasopharynx, Endo */
"09TP0ZZ"  =  "1"  /* Resection of Accessory Sinus, Open Approach */
"09TP4ZZ"  =  "1"  /* Resection of Accessory Sinus, Perc Endo Approach */
"09TQ0ZZ"  =  "1"  /* Resection of Right Maxillary Sinus, Open Approach */
"09TQ4ZZ"  =  "1"  /* Resection of Right Maxillary Sinus, Perc Endo Approach */
"09TR0ZZ"  =  "1"  /* Resection of Left Maxillary Sinus, Open Approach */
"09TR4ZZ"  =  "1"  /* Resection of Left Maxillary Sinus, Perc Endo Approach */
"09TS0ZZ"  =  "1"  /* Resection of Right Frontal Sinus, Open Approach */
"09TS4ZZ"  =  "1"  /* Resection of Right Frontal Sinus, Perc Endo Approach */
"09TT0ZZ"  =  "1"  /* Resection of Left Frontal Sinus, Open Approach */
"09TT4ZZ"  =  "1"  /* Resection of Left Frontal Sinus, Perc Endo Approach */
"09TU0ZZ"  =  "1"  /* Resection of Right Ethmoid Sinus, Open Approach */
"09TU4ZZ"  =  "1"  /* Resection of Right Ethmoid Sinus, Perc Endo Approach */
"09TV0ZZ"  =  "1"  /* Resection of Left Ethmoid Sinus, Open Approach */
"09TV4ZZ"  =  "1"  /* Resection of Left Ethmoid Sinus, Perc Endo Approach */
"09TW0ZZ"  =  "1"  /* Resection of Right Sphenoid Sinus, Open Approach */
"09TW4ZZ"  =  "1"  /* Resection of Right Sphenoid Sinus, Perc Endo Approach */
"09TX0ZZ"  =  "1"  /* Resection of Left Sphenoid Sinus, Open Approach */
"09TX4ZZ"  =  "1"  /* Resection of Left Sphenoid Sinus, Perc Endo Approach */
"09U007Z"  =  "1"  /* Supplement Right External Ear with Autol Sub, Open Approach */
"09U00JZ"  =  "1"  /* Supplement Right External Ear with Synth Sub, Open Approach */
"09U00KZ"  =  "1"  /* Supplement Right External Ear with Nonaut Sub, Open Approach */
"09U0X7Z"  =  "1"  /* Supplement R Ext Ear with Autol Sub, Extern Approach */
"09U0XJZ"  =  "1"  /* Supplement R Ext Ear with Synth Sub, Extern Approach */
"09U0XKZ"  =  "1"  /* Supplement R Ext Ear with Nonaut Sub, Extern Approach */
"09U107Z"  =  "1"  /* Supplement Left External Ear with Autol Sub, Open Approach */
"09U10JZ"  =  "1"  /* Supplement Left External Ear with Synth Sub, Open Approach */
"09U10KZ"  =  "1"  /* Supplement Left External Ear with Nonaut Sub, Open Approach */
"09U1X7Z"  =  "1"  /* Supplement Left External Ear with Autol Sub, Extern Approach */
"09U1XJZ"  =  "1"  /* Supplement Left External Ear with Synth Sub, Extern Approach */
"09U1XKZ"  =  "1"  /* Supplement L Ext Ear with Nonaut Sub, Extern Approach */
"09U207Z"  =  "1"  /* Supplement Bi Ext Ear with Autol Sub, Open Approach */
"09U20JZ"  =  "1"  /* Supplement Bi Ext Ear with Synth Sub, Open Approach */
"09U20KZ"  =  "1"  /* Supplement Bi Ext Ear with Nonaut Sub, Open Approach */
"09U2X7Z"  =  "1"  /* Supplement Bi Ext Ear with Autol Sub, Extern Approach */
"09U2XJZ"  =  "1"  /* Supplement Bi Ext Ear with Synth Sub, Extern Approach */
"09U2XKZ"  =  "1"  /* Supplement Bi Ext Ear with Nonaut Sub, Extern Approach */
"09U507Z"  =  "1"  /* Supplement Right Middle Ear with Autol Sub, Open Approach */
"09U50JZ"  =  "1"  /* Supplement Right Middle Ear with Synth Sub, Open Approach */
"09U50KZ"  =  "1"  /* Supplement Right Middle Ear with Nonaut Sub, Open Approach */
"09U607Z"  =  "1"  /* Supplement Left Middle Ear with Autol Sub, Open Approach */
"09U60JZ"  =  "1"  /* Supplement Left Middle Ear with Synth Sub, Open Approach */
"09U60KZ"  =  "1"  /* Supplement Left Middle Ear with Nonaut Sub, Open Approach */
"09U707Z"  =  "1"  /* Supplement R Tympanic Membrane with Autol Sub, Open Approach */
"09U70JZ"  =  "1"  /* Supplement R Tympanic Membrane with Synth Sub, Open Approach */
"09U70KZ"  =  "1"  /* Supplement R Tympanic Membrane w Nonaut Sub, Open */
"09U777Z"  =  "1"  /* Supplement R Tympanic Membrane with Autol Sub, Via Opening */
"09U77JZ"  =  "1"  /* Supplement R Tympanic Membrane with Synth Sub, Via Opening */
"09U77KZ"  =  "1"  /* Supplement R Tympanic Membrane with Nonaut Sub, Via Opening */
"09U787Z"  =  "1"  /* Supplement Right Tympanic Membrane with Autol Sub, Endo */
"09U78JZ"  =  "1"  /* Supplement Right Tympanic Membrane with Synth Sub, Endo */
"09U78KZ"  =  "1"  /* Supplement Right Tympanic Membrane with Nonaut Sub, Endo */
"09U807Z"  =  "1"  /* Supplement L Tympanic Membrane with Autol Sub, Open Approach */
"09U80JZ"  =  "1"  /* Supplement L Tympanic Membrane with Synth Sub, Open Approach */
"09U80KZ"  =  "1"  /* Supplement L Tympanic Membrane w Nonaut Sub, Open */
"09U877Z"  =  "1"  /* Supplement L Tympanic Membrane with Autol Sub, Via Opening */
"09U87JZ"  =  "1"  /* Supplement L Tympanic Membrane with Synth Sub, Via Opening */
"09U87KZ"  =  "1"  /* Supplement L Tympanic Membrane with Nonaut Sub, Via Opening */
"09U887Z"  =  "1"  /* Supplement Left Tympanic Membrane with Autol Sub, Endo */
"09U88JZ"  =  "1"  /* Supplement Left Tympanic Membrane with Synth Sub, Endo */
"09U88KZ"  =  "1"  /* Supplement Left Tympanic Membrane with Nonaut Sub, Endo */
"09U907Z"  =  "1"  /* Supplement R Auditory Ossicle with Autol Sub, Open Approach */
"09U90JZ"  =  "1"  /* Supplement R Auditory Ossicle with Synth Sub, Open Approach */
"09U90KZ"  =  "1"  /* Supplement R Auditory Ossicle with Nonaut Sub, Open Approach */
"09UA07Z"  =  "1"  /* Supplement L Auditory Ossicle with Autol Sub, Open Approach */
"09UA0JZ"  =  "1"  /* Supplement L Auditory Ossicle with Synth Sub, Open Approach */
"09UA0KZ"  =  "1"  /* Supplement L Auditory Ossicle with Nonaut Sub, Open Approach */
"09UD07Z"  =  "1"  /* Supplement Right Inner Ear with Autol Sub, Open Approach */
"09UD0JZ"  =  "1"  /* Supplement Right Inner Ear with Synth Sub, Open Approach */
"09UD0KZ"  =  "1"  /* Supplement Right Inner Ear with Nonaut Sub, Open Approach */
"09UE07Z"  =  "1"  /* Supplement Left Inner Ear with Autol Sub, Open Approach */
"09UE0JZ"  =  "1"  /* Supplement Left Inner Ear with Synth Sub, Open Approach */
"09UE0KZ"  =  "1"  /* Supplement Left Inner Ear with Nonaut Sub, Open Approach */
"09UK07Z"  =  "1"  /* Supplement Nose with Autol Sub, Open Approach */
"09UK0JZ"  =  "1"  /* Supplement Nose with Synthetic Substitute, Open Approach */
"09UK0KZ"  =  "1"  /* Supplement Nose with Nonaut Sub, Open Approach */
"09UKX7Z"  =  "1"  /* Supplement Nose with Autol Sub, Extern Approach */
"09UKXJZ"  =  "1"  /* Supplement Nose with Synthetic Substitute, External Approach */
"09UKXKZ"  =  "1"  /* Supplement Nose with Nonaut Sub, Extern Approach */
"09UL07Z"  =  "1"  /* Supplement Nasal Turbinate with Autol Sub, Open Approach */
"09UL0JZ"  =  "1"  /* Supplement Nasal Turbinate with Synth Sub, Open Approach */
"09UL0KZ"  =  "1"  /* Supplement Nasal Turbinate with Nonaut Sub, Open Approach */
"09UL37Z"  =  "1"  /* Supplement Nasal Turbinate with Autol Sub, Perc Approach */
"09UL3JZ"  =  "1"  /* Supplement Nasal Turbinate with Synth Sub, Perc Approach */
"09UL3KZ"  =  "1"  /* Supplement Nasal Turbinate with Nonaut Sub, Perc Approach */
"09UL47Z"  =  "1"  /* Supplement Nasal Turbinate w Autol Sub, Perc Endo */
"09UL4JZ"  =  "1"  /* Supplement Nasal Turbinate w Synth Sub, Perc Endo */
"09UL4KZ"  =  "1"  /* Supplement Nasal Turbinate w Nonaut Sub, Perc Endo */
"09UL77Z"  =  "1"  /* Supplement Nasal Turbinate with Autol Sub, Via Opening */
"09UL7JZ"  =  "1"  /* Supplement Nasal Turbinate with Synth Sub, Via Opening */
"09UL7KZ"  =  "1"  /* Supplement Nasal Turbinate with Nonaut Sub, Via Opening */
"09UL87Z"  =  "1"  /* Supplement Nasal Turbinate with Autol Sub, Endo */
"09UL8JZ"  =  "1"  /* Supplement Nasal Turbinate with Synthetic Substitute, Endo */
"09UL8KZ"  =  "1"  /* Supplement Nasal Turbinate with Nonaut Sub, Endo */
"09UM07Z"  =  "1"  /* Supplement Nasal Septum with Autol Sub, Open Approach */
"09UM0JZ"  =  "1"  /* Supplement Nasal Septum with Synth Sub, Open Approach */
"09UM0KZ"  =  "1"  /* Supplement Nasal Septum with Nonaut Sub, Open Approach */
"09UM37Z"  =  "1"  /* Supplement Nasal Septum with Autol Sub, Perc Approach */
"09UM3JZ"  =  "1"  /* Supplement Nasal Septum with Synth Sub, Perc Approach */
"09UM3KZ"  =  "1"  /* Supplement Nasal Septum with Nonaut Sub, Perc Approach */
"09UM47Z"  =  "1"  /* Supplement Nasal Septum with Autol Sub, Perc Endo Approach */
"09UM4JZ"  =  "1"  /* Supplement Nasal Septum with Synth Sub, Perc Endo Approach */
"09UM4KZ"  =  "1"  /* Supplement Nasal Septum with Nonaut Sub, Perc Endo Approach */
"09UN07Z"  =  "1"  /* Supplement Nasopharynx with Autol Sub, Open Approach */
"09UN0JZ"  =  "1"  /* Supplement Nasopharynx with Synth Sub, Open Approach */
"09UN0KZ"  =  "1"  /* Supplement Nasopharynx with Nonaut Sub, Open Approach */
"09UN77Z"  =  "1"  /* Supplement Nasopharynx with Autol Sub, Via Opening */
"09UN7JZ"  =  "1"  /* Supplement Nasopharynx with Synth Sub, Via Opening */
"09UN7KZ"  =  "1"  /* Supplement Nasopharynx with Nonaut Sub, Via Opening */
"09UN87Z"  =  "1"  /* Supplement Nasopharynx with Autol Sub, Endo */
"09UN8JZ"  =  "1"  /* Supplement Nasopharynx with Synthetic Substitute, Endo */
"09UN8KZ"  =  "1"  /* Supplement Nasopharynx with Nonaut Sub, Endo */
"09W707Z"  =  "1"  /* Revision of Autol Sub in R Tympanic Membrane, Open Approach */
"09W70JZ"  =  "1"  /* Revision of Synth Sub in R Tympanic Membrane, Open Approach */
"09W70KZ"  =  "1"  /* Revision of Nonaut Sub in R Tympanic Membrane, Open Approach */
"09W777Z"  =  "1"  /* Revision of Autol Sub in R Tympanic Membrane, Via Opening */
"09W77JZ"  =  "1"  /* Revision of Synth Sub in R Tympanic Membrane, Via Opening */
"09W77KZ"  =  "1"  /* Revision of Nonaut Sub in R Tympanic Membrane, Via Opening */
"09W787Z"  =  "1"  /* Revision of Autol Sub in R Tympanic Membrane, Endo */
"09W78JZ"  =  "1"  /* Revision of Synth Sub in R Tympanic Membrane, Endo */
"09W78KZ"  =  "1"  /* Revision of Nonaut Sub in R Tympanic Membrane, Endo */
"09W807Z"  =  "1"  /* Revision of Autol Sub in L Tympanic Membrane, Open Approach */
"09W80JZ"  =  "1"  /* Revision of Synth Sub in L Tympanic Membrane, Open Approach */
"09W80KZ"  =  "1"  /* Revision of Nonaut Sub in L Tympanic Membrane, Open Approach */
"09W877Z"  =  "1"  /* Revision of Autol Sub in L Tympanic Membrane, Via Opening */
"09W87JZ"  =  "1"  /* Revision of Synth Sub in L Tympanic Membrane, Via Opening */
"09W87KZ"  =  "1"  /* Revision of Nonaut Sub in L Tympanic Membrane, Via Opening */
"09W887Z"  =  "1"  /* Revision of Autol Sub in L Tympanic Membrane, Endo */
"09W88JZ"  =  "1"  /* Revision of Synth Sub in L Tympanic Membrane, Endo */
"09W88KZ"  =  "1"  /* Revision of Nonaut Sub in L Tympanic Membrane, Endo */
"09W907Z"  =  "1"  /* Revision of Autol Sub in R Auditory Ossicle, Open Approach */
"09W90JZ"  =  "1"  /* Revision of Synth Sub in R Auditory Ossicle, Open Approach */
"09W90KZ"  =  "1"  /* Revision of Nonaut Sub in R Auditory Ossicle, Open Approach */
"09W977Z"  =  "1"  /* Revision of Autol Sub in R Auditory Ossicle, Via Opening */
"09W97JZ"  =  "1"  /* Revision of Synth Sub in R Auditory Ossicle, Via Opening */
"09W97KZ"  =  "1"  /* Revision of Nonaut Sub in R Auditory Ossicle, Via Opening */
"09W987Z"  =  "1"  /* Revision of Autol Sub in R Auditory Ossicle, Endo */
"09W98JZ"  =  "1"  /* Revision of Synthetic Substitute in R Auditory Ossicle, Endo */
"09W98KZ"  =  "1"  /* Revision of Nonaut Sub in R Auditory Ossicle, Endo */
"09WA07Z"  =  "1"  /* Revision of Autol Sub in L Auditory Ossicle, Open Approach */
"09WA0JZ"  =  "1"  /* Revision of Synth Sub in L Auditory Ossicle, Open Approach */
"09WA0KZ"  =  "1"  /* Revision of Nonaut Sub in L Auditory Ossicle, Open Approach */
"09WA77Z"  =  "1"  /* Revision of Autol Sub in L Auditory Ossicle, Via Opening */
"09WA7JZ"  =  "1"  /* Revision of Synth Sub in L Auditory Ossicle, Via Opening */
"09WA7KZ"  =  "1"  /* Revision of Nonaut Sub in L Auditory Ossicle, Via Opening */
"09WA87Z"  =  "1"  /* Revision of Autol Sub in L Auditory Ossicle, Endo */
"09WA8JZ"  =  "1"  /* Revision of Synthetic Substitute in L Auditory Ossicle, Endo */
"09WA8KZ"  =  "1"  /* Revision of Nonaut Sub in L Auditory Ossicle, Endo */
"09WD0SZ"  =  "1"  /* Revision of Hearing Device in Right Inner Ear, Open Approach */
"09WD7SZ"  =  "1"  /* Revision of Hearing Device in Right Inner Ear, Via Opening */
"09WD8SZ"  =  "1"  /* Revision of Hearing Device in Right Inner Ear, Endo */
"09WE0SZ"  =  "1"  /* Revision of Hearing Device in Left Inner Ear, Open Approach */
"09WE7SZ"  =  "1"  /* Revision of Hearing Device in Left Inner Ear, Via Opening */
"09WE8SZ"  =  "1"  /* Revision of Hearing Device in Left Inner Ear, Endo */
"09WH00Z"  =  "1"  /* Revision of Drainage Device in Right Ear, Open Approach */
"09WH07Z"  =  "1"  /* Revision of Autol Sub in R Ear, Open Approach */
"09WH0DZ"  =  "1"  /* Revision of Intraluminal Device in Right Ear, Open Approach */
"09WH0JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Ear, Open Approach */
"09WH0KZ"  =  "1"  /* Revision of Nonaut Sub in R Ear, Open Approach */
"09WH30Z"  =  "1"  /* Revision of Drainage Device in Right Ear, Perc Approach */
"09WH37Z"  =  "1"  /* Revision of Autol Sub in R Ear, Perc Approach */
"09WH3DZ"  =  "1"  /* Revision of Intraluminal Device in Right Ear, Perc Approach */
"09WH40Z"  =  "1"  /* Revision of Drainage Device in Right Ear, Perc Endo Approach */
"09WH47Z"  =  "1"  /* Revision of Autol Sub in R Ear, Perc Endo Approach */
"09WH4DZ"  =  "1"  /* Revision of Intraluminal Device in R Ear, Perc Endo Approach */
"09WH70Z"  =  "1"  /* Revision of Drainage Device in Right Ear, Via Opening */
"09WH77Z"  =  "1"  /* Revision of Autol Sub in R Ear, Via Opening */
"09WH7JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Ear, Via Opening */
"09WH7KZ"  =  "1"  /* Revision of Nonaut Sub in R Ear, Via Opening */
"09WH80Z"  =  "1"  /* Revision of Drainage Device in Right Ear, Endo */
"09WH87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Right Ear, Endo */
"09WH8JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Ear, Endo */
"09WH8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in R Ear, Endo */
"09WJ00Z"  =  "1"  /* Revision of Drainage Device in Left Ear, Open Approach */
"09WJ07Z"  =  "1"  /* Revision of Autol Sub in L Ear, Open Approach */
"09WJ0DZ"  =  "1"  /* Revision of Intraluminal Device in Left Ear, Open Approach */
"09WJ0JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Ear, Open Approach */
"09WJ0KZ"  =  "1"  /* Revision of Nonaut Sub in L Ear, Open Approach */
"09WJ30Z"  =  "1"  /* Revision of Drainage Device in Left Ear, Perc Approach */
"09WJ37Z"  =  "1"  /* Revision of Autol Sub in L Ear, Perc Approach */
"09WJ3DZ"  =  "1"  /* Revision of Intraluminal Device in Left Ear, Perc Approach */
"09WJ40Z"  =  "1"  /* Revision of Drainage Device in Left Ear, Perc Endo Approach */
"09WJ47Z"  =  "1"  /* Revision of Autol Sub in L Ear, Perc Endo Approach */
"09WJ4DZ"  =  "1"  /* Revision of Intraluminal Device in L Ear, Perc Endo Approach */
"09WJ70Z"  =  "1"  /* Revision of Drainage Device in Left Ear, Via Opening */
"09WJ77Z"  =  "1"  /* Revision of Autol Sub in L Ear, Via Opening */
"09WJ7JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Ear, Via Opening */
"09WJ7KZ"  =  "1"  /* Revision of Nonaut Sub in L Ear, Via Opening */
"09WJ80Z"  =  "1"  /* Revision of Drainage Device in Left Ear, Endo */
"09WJ87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Left Ear, Endo */
"09WJ8JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Ear, Endo */
"09WJ8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in L Ear, Endo */
"09WY00Z"  =  "1"  /* Revision of Drainage Device in Sinus, Open Approach */
"09WY30Z"  =  "1"  /* Revision of Drainage Device in Sinus, Percutaneous Approach */
"09WY40Z"  =  "1"  /* Revision of Drainage Device in Sinus, Perc Endo Approach */
"0B110F4"  =  "1"  /* Bypass Trachea to Cutaneous with Trach Dev, Open Approach */
"0B110Z4"  =  "1"  /* Bypass Trachea to Cutaneous, Open Approach */
"0B114F4"  =  "1"  /* Bypass Trachea to Cutan with Trach Dev, Perc Endo Approach */
"0B114Z4"  =  "1"  /* Bypass Trachea to Cutaneous, Perc Endo Approach */
"0B510ZZ"  =  "1"  /* Destruction of Trachea, Open Approach */
"0B513ZZ"  =  "1"  /* Destruction of Trachea, Percutaneous Approach */
"0B514ZZ"  =  "1"  /* Destruction of Trachea, Percutaneous Endoscopic Approach */
"0B517ZZ"  =  "1"  /* Destruction of Trachea, Via Natural or Artificial Opening */
"0B518ZZ"  =  "1"  /* Destruction of Trachea, Endo */
"0B520ZZ"  =  "1"  /* Destruction of Carina, Open Approach */
"0B523ZZ"  =  "1"  /* Destruction of Carina, Percutaneous Approach */
"0B524ZZ"  =  "1"  /* Destruction of Carina, Percutaneous Endoscopic Approach */
"0B527ZZ"  =  "1"  /* Destruction of Carina, Via Natural or Artificial Opening */
"0B528ZZ"  =  "1"  /* Destruction of Carina, Endo */
"0B530ZZ"  =  "1"  /* Destruction of Right Main Bronchus, Open Approach */
"0B533ZZ"  =  "1"  /* Destruction of Right Main Bronchus, Percutaneous Approach */
"0B537ZZ"  =  "1"  /* Destruction of Right Main Bronchus, Via Opening */
"0B538ZZ"  =  "1"  /* Destruction of Right Main Bronchus, Endo */
"0B540ZZ"  =  "1"  /* Destruction of Right Upper Lobe Bronchus, Open Approach */
"0B543ZZ"  =  "1"  /* Destruction of Right Upper Lobe Bronchus, Perc Approach */
"0B547ZZ"  =  "1"  /* Destruction of Right Upper Lobe Bronchus, Via Opening */
"0B548ZZ"  =  "1"  /* Destruction of Right Upper Lobe Bronchus, Endo */
"0B550ZZ"  =  "1"  /* Destruction of Right Middle Lobe Bronchus, Open Approach */
"0B553ZZ"  =  "1"  /* Destruction of Right Middle Lobe Bronchus, Perc Approach */
"0B557ZZ"  =  "1"  /* Destruction of Right Middle Lobe Bronchus, Via Opening */
"0B558ZZ"  =  "1"  /* Destruction of Right Middle Lobe Bronchus, Endo */
"0B560ZZ"  =  "1"  /* Destruction of Right Lower Lobe Bronchus, Open Approach */
"0B563ZZ"  =  "1"  /* Destruction of Right Lower Lobe Bronchus, Perc Approach */
"0B567ZZ"  =  "1"  /* Destruction of Right Lower Lobe Bronchus, Via Opening */
"0B568ZZ"  =  "1"  /* Destruction of Right Lower Lobe Bronchus, Endo */
"0B570ZZ"  =  "1"  /* Destruction of Left Main Bronchus, Open Approach */
"0B573ZZ"  =  "1"  /* Destruction of Left Main Bronchus, Percutaneous Approach */
"0B577ZZ"  =  "1"  /* Destruction of Left Main Bronchus, Via Opening */
"0B578ZZ"  =  "1"  /* Destruction of Left Main Bronchus, Endo */
"0B580ZZ"  =  "1"  /* Destruction of Left Upper Lobe Bronchus, Open Approach */
"0B583ZZ"  =  "1"  /* Destruction of Left Upper Lobe Bronchus, Perc Approach */
"0B587ZZ"  =  "1"  /* Destruction of Left Upper Lobe Bronchus, Via Opening */
"0B588ZZ"  =  "1"  /* Destruction of Left Upper Lobe Bronchus, Endo */
"0B590ZZ"  =  "1"  /* Destruction of Lingula Bronchus, Open Approach */
"0B593ZZ"  =  "1"  /* Destruction of Lingula Bronchus, Percutaneous Approach */
"0B597ZZ"  =  "1"  /* Destruction of Lingula Bronchus, Via Opening */
"0B598ZZ"  =  "1"  /* Destruction of Lingula Bronchus, Endo */
"0B5B0ZZ"  =  "1"  /* Destruction of Left Lower Lobe Bronchus, Open Approach */
"0B5B3ZZ"  =  "1"  /* Destruction of Left Lower Lobe Bronchus, Perc Approach */
"0B5B7ZZ"  =  "1"  /* Destruction of Left Lower Lobe Bronchus, Via Opening */
"0B5B8ZZ"  =  "1"  /* Destruction of Left Lower Lobe Bronchus, Endo */
"0B5C0ZZ"  =  "1"  /* Destruction of Right Upper Lung Lobe, Open Approach */
"0B5C3ZZ"  =  "1"  /* Destruction of Right Upper Lung Lobe, Percutaneous Approach */
"0B5C4ZZ"  =  "1"  /* Destruction of Right Upper Lung Lobe, Perc Endo Approach */
"0B5C7ZZ"  =  "1"  /* Destruction of Right Upper Lung Lobe, Via Opening */
"0B5D0ZZ"  =  "1"  /* Destruction of Right Middle Lung Lobe, Open Approach */
"0B5D3ZZ"  =  "1"  /* Destruction of Right Middle Lung Lobe, Percutaneous Approach */
"0B5D4ZZ"  =  "1"  /* Destruction of Right Middle Lung Lobe, Perc Endo Approach */
"0B5D7ZZ"  =  "1"  /* Destruction of Right Middle Lung Lobe, Via Opening */
"0B5F0ZZ"  =  "1"  /* Destruction of Right Lower Lung Lobe, Open Approach */
"0B5F3ZZ"  =  "1"  /* Destruction of Right Lower Lung Lobe, Percutaneous Approach */
"0B5F4ZZ"  =  "1"  /* Destruction of Right Lower Lung Lobe, Perc Endo Approach */
"0B5F7ZZ"  =  "1"  /* Destruction of Right Lower Lung Lobe, Via Opening */
"0B5G0ZZ"  =  "1"  /* Destruction of Left Upper Lung Lobe, Open Approach */
"0B5G3ZZ"  =  "1"  /* Destruction of Left Upper Lung Lobe, Percutaneous Approach */
"0B5G4ZZ"  =  "1"  /* Destruction of Left Upper Lung Lobe, Perc Endo Approach */
"0B5G7ZZ"  =  "1"  /* Destruction of Left Upper Lung Lobe, Via Opening */
"0B5H0ZZ"  =  "1"  /* Destruction of Lung Lingula, Open Approach */
"0B5H3ZZ"  =  "1"  /* Destruction of Lung Lingula, Percutaneous Approach */
"0B5H4ZZ"  =  "1"  /* Destruction of Lung Lingula, Perc Endo Approach */
"0B5H7ZZ"  =  "1"  /* Destruction of Lung Lingula, Via Opening */
"0B5J0ZZ"  =  "1"  /* Destruction of Left Lower Lung Lobe, Open Approach */
"0B5J3ZZ"  =  "1"  /* Destruction of Left Lower Lung Lobe, Percutaneous Approach */
"0B5J4ZZ"  =  "1"  /* Destruction of Left Lower Lung Lobe, Perc Endo Approach */
"0B5J7ZZ"  =  "1"  /* Destruction of Left Lower Lung Lobe, Via Opening */
"0B5K0ZZ"  =  "1"  /* Destruction of Right Lung, Open Approach */
"0B5K3ZZ"  =  "1"  /* Destruction of Right Lung, Percutaneous Approach */
"0B5K4ZZ"  =  "1"  /* Destruction of Right Lung, Percutaneous Endoscopic Approach */
"0B5K7ZZ"  =  "1"  /* Destruction of Right Lung, Via Natural or Artificial Opening */
"0B5L0ZZ"  =  "1"  /* Destruction of Left Lung, Open Approach */
"0B5L3ZZ"  =  "1"  /* Destruction of Left Lung, Percutaneous Approach */
"0B5L4ZZ"  =  "1"  /* Destruction of Left Lung, Percutaneous Endoscopic Approach */
"0B5L7ZZ"  =  "1"  /* Destruction of Left Lung, Via Natural or Artificial Opening */
"0B5M0ZZ"  =  "1"  /* Destruction of Bilateral Lungs, Open Approach */
"0B5M3ZZ"  =  "1"  /* Destruction of Bilateral Lungs, Percutaneous Approach */
"0B5M4ZZ"  =  "1"  /* Destruction of Bilateral Lungs, Perc Endo Approach */
"0B5M7ZZ"  =  "1"  /* Destruction of Bilateral Lungs, Via Opening */
"0B5N0ZZ"  =  "1"  /* Destruction of Right Pleura, Open Approach */
"0B5N3ZZ"  =  "1"  /* Destruction of Right Pleura, Percutaneous Approach */
"0B5N4ZZ"  =  "1"  /* Destruction of Right Pleura, Perc Endo Approach */
"0B5P0ZZ"  =  "1"  /* Destruction of Left Pleura, Open Approach */
"0B5P3ZZ"  =  "1"  /* Destruction of Left Pleura, Percutaneous Approach */
"0B5P4ZZ"  =  "1"  /* Destruction of Left Pleura, Percutaneous Endoscopic Approach */
"0B5R0ZZ"  =  "1"  /* Destruction of Right Diaphragm, Open Approach */
"0B5R3ZZ"  =  "1"  /* Destruction of Right Diaphragm, Percutaneous Approach */
"0B5R4ZZ"  =  "1"  /* Destruction of Right Diaphragm, Perc Endo Approach */
"0B5S0ZZ"  =  "1"  /* Destruction of Left Diaphragm, Open Approach */
"0B5S3ZZ"  =  "1"  /* Destruction of Left Diaphragm, Percutaneous Approach */
"0B5S4ZZ"  =  "1"  /* Destruction of Left Diaphragm, Perc Endo Approach */
"0B710DZ"  =  "1"  /* Dilation of Trachea with Intraluminal Device, Open Approach */
"0B710ZZ"  =  "1"  /* Dilation of Trachea, Open Approach */
"0B713DZ"  =  "1"  /* Dilation of Trachea with Intraluminal Device, Perc Approach */
"0B713ZZ"  =  "1"  /* Dilation of Trachea, Percutaneous Approach */
"0B714DZ"  =  "1"  /* Dilation of Trachea with Intralum Dev, Perc Endo Approach */
"0B714ZZ"  =  "1"  /* Dilation of Trachea, Percutaneous Endoscopic Approach */
"0B717DZ"  =  "1"  /* Dilation of Trachea with Intraluminal Device, Via Opening */
"0B717ZZ"  =  "1"  /* Dilation of Trachea, Via Natural or Artificial Opening */
"0B718DZ"  =  "1"  /* Dilation of Trachea with Intraluminal Device, Endo */
"0B718ZZ"  =  "1"  /* Dilation of Trachea, Endo */
"0B720DZ"  =  "1"  /* Dilation of Carina with Intraluminal Device, Open Approach */
"0B720ZZ"  =  "1"  /* Dilation of Carina, Open Approach */
"0B723DZ"  =  "1"  /* Dilation of Carina with Intraluminal Device, Perc Approach */
"0B723ZZ"  =  "1"  /* Dilation of Carina, Percutaneous Approach */
"0B724DZ"  =  "1"  /* Dilation of Carina with Intralum Dev, Perc Endo Approach */
"0B724ZZ"  =  "1"  /* Dilation of Carina, Percutaneous Endoscopic Approach */
"0B727DZ"  =  "1"  /* Dilation of Carina with Intraluminal Device, Via Opening */
"0B727ZZ"  =  "1"  /* Dilation of Carina, Via Natural or Artificial Opening */
"0B728DZ"  =  "1"  /* Dilation of Carina with Intraluminal Device, Endo */
"0B728ZZ"  =  "1"  /* Dilation of Carina, Endo */
"0B9100Z"  =  "1"  /* Drainage of Trachea with Drainage Device, Open Approach */
"0B910ZX"  =  "1"  /* Drainage of Trachea, Open Approach, Diagnostic */
"0B910ZZ"  =  "1"  /* Drainage of Trachea, Open Approach */
"0B9130Z"  =  "1"  /* Drainage of Trachea with Drainage Device, Perc Approach */
"0B913ZZ"  =  "1"  /* Drainage of Trachea, Percutaneous Approach */
"0B9140Z"  =  "1"  /* Drainage of Trachea with Drainage Device, Perc Endo Approach */
"0B914ZZ"  =  "1"  /* Drainage of Trachea, Percutaneous Endoscopic Approach */
"0B9170Z"  =  "1"  /* Drainage of Trachea with Drainage Device, Via Opening */
"0B917ZZ"  =  "1"  /* Drainage of Trachea, Via Natural or Artificial Opening */
"0B9180Z"  =  "1"  /* Drainage of Trachea with Drainage Device, Endo */
"0B918ZZ"  =  "1"  /* Drainage of Trachea, Endo */
"0B9200Z"  =  "1"  /* Drainage of Carina with Drainage Device, Open Approach */
"0B920ZX"  =  "1"  /* Drainage of Carina, Open Approach, Diagnostic */
"0B920ZZ"  =  "1"  /* Drainage of Carina, Open Approach */
"0B9230Z"  =  "1"  /* Drainage of Carina with Drainage Device, Perc Approach */
"0B923ZZ"  =  "1"  /* Drainage of Carina, Percutaneous Approach */
"0B9240Z"  =  "1"  /* Drainage of Carina with Drainage Device, Perc Endo Approach */
"0B924ZZ"  =  "1"  /* Drainage of Carina, Percutaneous Endoscopic Approach */
"0B9270Z"  =  "1"  /* Drainage of Carina with Drainage Device, Via Opening */
"0B927ZZ"  =  "1"  /* Drainage of Carina, Via Natural or Artificial Opening */
"0B9280Z"  =  "1"  /* Drainage of Carina with Drainage Device, Endo */
"0B928ZZ"  =  "1"  /* Drainage of Carina, Endo */
"0B9300Z"  =  "1"  /* Drainage of R Main Bronc with Drain Dev, Open Approach */
"0B930ZX"  =  "1"  /* Drainage of Right Main Bronchus, Open Approach, Diagnostic */
"0B930ZZ"  =  "1"  /* Drainage of Right Main Bronchus, Open Approach */
"0B9330Z"  =  "1"  /* Drainage of R Main Bronc with Drain Dev, Perc Approach */
"0B933ZZ"  =  "1"  /* Drainage of Right Main Bronchus, Percutaneous Approach */
"0B9340Z"  =  "1"  /* Drainage of R Main Bronc with Drain Dev, Perc Endo Approach */
"0B934ZZ"  =  "1"  /* Drainage of Right Main Bronchus, Perc Endo Approach */
"0B9370Z"  =  "1"  /* Drainage of Right Main Bronchus with Drain Dev, Via Opening */
"0B937ZZ"  =  "1"  /* Drainage of Right Main Bronchus, Via Opening */
"0B9380Z"  =  "1"  /* Drainage of Right Main Bronchus with Drainage Device, Endo */
"0B938ZZ"  =  "1"  /* Drainage of Right Main Bronchus, Endo */
"0B9400Z"  =  "1"  /* Drainage of R Up Lobe Bronc with Drain Dev, Open Approach */
"0B940ZX"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Open Approach, Diagn */
"0B940ZZ"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Open Approach */
"0B9430Z"  =  "1"  /* Drainage of R Up Lobe Bronc with Drain Dev, Perc Approach */
"0B943ZZ"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Percutaneous Approach */
"0B9440Z"  =  "1"  /* Drain of R Up Lobe Bronc with Drain Dev, Perc Endo Approach */
"0B944ZZ"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Perc Endo Approach */
"0B9470Z"  =  "1"  /* Drainage of R Up Lobe Bronc with Drain Dev, Via Opening */
"0B947ZZ"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Via Opening */
"0B9480Z"  =  "1"  /* Drainage of Right Upper Lobe Bronchus with Drain Dev, Endo */
"0B948ZZ"  =  "1"  /* Drainage of Right Upper Lobe Bronchus, Endo */
"0B9500Z"  =  "1"  /* Drainage of R Mid Lobe Bronc with Drain Dev, Open Approach */
"0B950ZX"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Open Approach, Diagn */
"0B950ZZ"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Open Approach */
"0B9530Z"  =  "1"  /* Drainage of R Mid Lobe Bronc with Drain Dev, Perc Approach */
"0B953ZZ"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Perc Approach */
"0B9540Z"  =  "1"  /* Drain of R Mid Lobe Bronc with Drain Dev, Perc Endo Approach */
"0B954ZZ"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Perc Endo Approach */
"0B9570Z"  =  "1"  /* Drainage of R Mid Lobe Bronc with Drain Dev, Via Opening */
"0B957ZZ"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Via Opening */
"0B9580Z"  =  "1"  /* Drainage of Right Middle Lobe Bronchus with Drain Dev, Endo */
"0B958ZZ"  =  "1"  /* Drainage of Right Middle Lobe Bronchus, Endo */
"0B9600Z"  =  "1"  /* Drainage of R Low Lobe Bronc with Drain Dev, Open Approach */
"0B960ZX"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Open Approach, Diagn */
"0B960ZZ"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Open Approach */
"0B9630Z"  =  "1"  /* Drainage of R Low Lobe Bronc with Drain Dev, Perc Approach */
"0B963ZZ"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Percutaneous Approach */
"0B9640Z"  =  "1"  /* Drain of R Low Lobe Bronc with Drain Dev, Perc Endo Approach */
"0B964ZZ"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Perc Endo Approach */
"0B9670Z"  =  "1"  /* Drainage of R Low Lobe Bronc with Drain Dev, Via Opening */
"0B967ZZ"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Via Opening */
"0B9680Z"  =  "1"  /* Drainage of Right Lower Lobe Bronchus with Drain Dev, Endo */
"0B968ZZ"  =  "1"  /* Drainage of Right Lower Lobe Bronchus, Endo */
"0B9700Z"  =  "1"  /* Drainage of Left Main Bronchus with Drain Dev, Open Approach */
"0B970ZX"  =  "1"  /* Drainage of Left Main Bronchus, Open Approach, Diagnostic */
"0B970ZZ"  =  "1"  /* Drainage of Left Main Bronchus, Open Approach */
"0B9730Z"  =  "1"  /* Drainage of Left Main Bronchus with Drain Dev, Perc Approach */
"0B973ZZ"  =  "1"  /* Drainage of Left Main Bronchus, Percutaneous Approach */
"0B9740Z"  =  "1"  /* Drainage of L Main Bronc with Drain Dev, Perc Endo Approach */
"0B974ZZ"  =  "1"  /* Drainage of Left Main Bronchus, Perc Endo Approach */
"0B9770Z"  =  "1"  /* Drainage of Left Main Bronchus with Drain Dev, Via Opening */
"0B977ZZ"  =  "1"  /* Drainage of Left Main Bronchus, Via Opening */
"0B9780Z"  =  "1"  /* Drainage of Left Main Bronchus with Drainage Device, Endo */
"0B978ZZ"  =  "1"  /* Drainage of Left Main Bronchus, Endo */
"0B9800Z"  =  "1"  /* Drainage of L Up Lobe Bronc with Drain Dev, Open Approach */
"0B980ZX"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Open Approach, Diagn */
"0B980ZZ"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Open Approach */
"0B9830Z"  =  "1"  /* Drainage of L Up Lobe Bronc with Drain Dev, Perc Approach */
"0B983ZZ"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Percutaneous Approach */
"0B9840Z"  =  "1"  /* Drain of L Up Lobe Bronc with Drain Dev, Perc Endo Approach */
"0B984ZZ"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Perc Endo Approach */
"0B9870Z"  =  "1"  /* Drainage of L Up Lobe Bronc with Drain Dev, Via Opening */
"0B987ZZ"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Via Opening */
"0B9880Z"  =  "1"  /* Drainage of Left Upper Lobe Bronchus with Drain Dev, Endo */
"0B988ZZ"  =  "1"  /* Drainage of Left Upper Lobe Bronchus, Endo */
"0B9900Z"  =  "1"  /* Drainage of Lingula Bronchus with Drain Dev, Open Approach */
"0B990ZX"  =  "1"  /* Drainage of Lingula Bronchus, Open Approach, Diagnostic */
"0B990ZZ"  =  "1"  /* Drainage of Lingula Bronchus, Open Approach */
"0B9930Z"  =  "1"  /* Drainage of Lingula Bronchus with Drain Dev, Perc Approach */
"0B993ZZ"  =  "1"  /* Drainage of Lingula Bronchus, Percutaneous Approach */
"0B9940Z"  =  "1"  /* Drainage of Lingula Bronc with Drain Dev, Perc Endo Approach */
"0B994ZZ"  =  "1"  /* Drainage of Lingula Bronchus, Perc Endo Approach */
"0B9970Z"  =  "1"  /* Drainage of Lingula Bronchus with Drain Dev, Via Opening */
"0B997ZZ"  =  "1"  /* Drainage of Lingula Bronchus, Via Opening */
"0B9980Z"  =  "1"  /* Drainage of Lingula Bronchus with Drainage Device, Endo */
"0B998ZZ"  =  "1"  /* Drainage of Lingula Bronchus, Endo */
"0B9B00Z"  =  "1"  /* Drainage of L Low Lobe Bronc with Drain Dev, Open Approach */
"0B9B0ZX"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Open Approach, Diagn */
"0B9B0ZZ"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Open Approach */
"0B9B30Z"  =  "1"  /* Drainage of L Low Lobe Bronc with Drain Dev, Perc Approach */
"0B9B3ZZ"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Percutaneous Approach */
"0B9B40Z"  =  "1"  /* Drain of L Low Lobe Bronc with Drain Dev, Perc Endo Approach */
"0B9B4ZZ"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Perc Endo Approach */
"0B9B70Z"  =  "1"  /* Drainage of L Low Lobe Bronc with Drain Dev, Via Opening */
"0B9B7ZZ"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Via Opening */
"0B9B80Z"  =  "1"  /* Drainage of Left Lower Lobe Bronchus with Drain Dev, Endo */
"0B9B8ZZ"  =  "1"  /* Drainage of Left Lower Lobe Bronchus, Endo */
"0B9C00Z"  =  "1"  /* Drainage of R Up Lung Lobe with Drain Dev, Open Approach */
"0B9C0ZX"  =  "1"  /* Drainage of Right Upper Lung Lobe, Open Approach, Diagnostic */
"0B9C0ZZ"  =  "1"  /* Drainage of Right Upper Lung Lobe, Open Approach */
"0B9C30Z"  =  "1"  /* Drainage of R Up Lung Lobe with Drain Dev, Perc Approach */
"0B9C3ZZ"  =  "1"  /* Drainage of Right Upper Lung Lobe, Percutaneous Approach */
"0B9C40Z"  =  "1"  /* Drain of R Up Lung Lobe with Drain Dev, Perc Endo Approach */
"0B9C4ZZ"  =  "1"  /* Drainage of Right Upper Lung Lobe, Perc Endo Approach */
"0B9C70Z"  =  "1"  /* Drainage of R Up Lung Lobe with Drain Dev, Via Opening */
"0B9C7ZZ"  =  "1"  /* Drainage of Right Upper Lung Lobe, Via Opening */
"0B9C80Z"  =  "1"  /* Drainage of Right Upper Lung Lobe with Drainage Device, Endo */
"0B9C8ZX"  =  "1"  /* Drainage of Right Upper Lung Lobe, Endo, Diagn */
"0B9C8ZZ"  =  "1"  /* Drainage of Right Upper Lung Lobe, Endo */
"0B9D00Z"  =  "1"  /* Drainage of R Mid Lung Lobe with Drain Dev, Open Approach */
"0B9D0ZX"  =  "1"  /* Drainage of Right Middle Lung Lobe, Open Approach, Diagn */
"0B9D0ZZ"  =  "1"  /* Drainage of Right Middle Lung Lobe, Open Approach */
"0B9D30Z"  =  "1"  /* Drainage of R Mid Lung Lobe with Drain Dev, Perc Approach */
"0B9D3ZZ"  =  "1"  /* Drainage of Right Middle Lung Lobe, Percutaneous Approach */
"0B9D40Z"  =  "1"  /* Drain of R Mid Lung Lobe with Drain Dev, Perc Endo Approach */
"0B9D4ZZ"  =  "1"  /* Drainage of Right Middle Lung Lobe, Perc Endo Approach */
"0B9D70Z"  =  "1"  /* Drainage of R Mid Lung Lobe with Drain Dev, Via Opening */
"0B9D7ZZ"  =  "1"  /* Drainage of Right Middle Lung Lobe, Via Opening */
"0B9D80Z"  =  "1"  /* Drainage of Right Middle Lung Lobe with Drain Dev, Endo */
"0B9D8ZX"  =  "1"  /* Drainage of Right Middle Lung Lobe, Endo, Diagn */
"0B9D8ZZ"  =  "1"  /* Drainage of Right Middle Lung Lobe, Endo */
"0B9F00Z"  =  "1"  /* Drainage of R Low Lung Lobe with Drain Dev, Open Approach */
"0B9F0ZX"  =  "1"  /* Drainage of Right Lower Lung Lobe, Open Approach, Diagnostic */
"0B9F0ZZ"  =  "1"  /* Drainage of Right Lower Lung Lobe, Open Approach */
"0B9F30Z"  =  "1"  /* Drainage of R Low Lung Lobe with Drain Dev, Perc Approach */
"0B9F3ZZ"  =  "1"  /* Drainage of Right Lower Lung Lobe, Percutaneous Approach */
"0B9F40Z"  =  "1"  /* Drain of R Low Lung Lobe with Drain Dev, Perc Endo Approach */
"0B9F4ZZ"  =  "1"  /* Drainage of Right Lower Lung Lobe, Perc Endo Approach */
"0B9F70Z"  =  "1"  /* Drainage of R Low Lung Lobe with Drain Dev, Via Opening */
"0B9F7ZZ"  =  "1"  /* Drainage of Right Lower Lung Lobe, Via Opening */
"0B9F80Z"  =  "1"  /* Drainage of Right Lower Lung Lobe with Drainage Device, Endo */
"0B9F8ZX"  =  "1"  /* Drainage of Right Lower Lung Lobe, Endo, Diagn */
"0B9F8ZZ"  =  "1"  /* Drainage of Right Lower Lung Lobe, Endo */
"0B9G00Z"  =  "1"  /* Drainage of L Up Lung Lobe with Drain Dev, Open Approach */
"0B9G0ZX"  =  "1"  /* Drainage of Left Upper Lung Lobe, Open Approach, Diagnostic */
"0B9G0ZZ"  =  "1"  /* Drainage of Left Upper Lung Lobe, Open Approach */
"0B9G30Z"  =  "1"  /* Drainage of L Up Lung Lobe with Drain Dev, Perc Approach */
"0B9G3ZZ"  =  "1"  /* Drainage of Left Upper Lung Lobe, Percutaneous Approach */
"0B9G40Z"  =  "1"  /* Drain of L Up Lung Lobe with Drain Dev, Perc Endo Approach */
"0B9G4ZZ"  =  "1"  /* Drainage of Left Upper Lung Lobe, Perc Endo Approach */
"0B9G70Z"  =  "1"  /* Drainage of Left Upper Lung Lobe with Drain Dev, Via Opening */
"0B9G7ZZ"  =  "1"  /* Drainage of Left Upper Lung Lobe, Via Opening */
"0B9G80Z"  =  "1"  /* Drainage of Left Upper Lung Lobe with Drainage Device, Endo */
"0B9G8ZX"  =  "1"  /* Drainage of Left Upper Lung Lobe, Endo, Diagn */
"0B9G8ZZ"  =  "1"  /* Drainage of Left Upper Lung Lobe, Endo */
"0B9H00Z"  =  "1"  /* Drainage of Lung Lingula with Drainage Device, Open Approach */
"0B9H0ZX"  =  "1"  /* Drainage of Lung Lingula, Open Approach, Diagnostic */
"0B9H0ZZ"  =  "1"  /* Drainage of Lung Lingula, Open Approach */
"0B9H30Z"  =  "1"  /* Drainage of Lung Lingula with Drainage Device, Perc Approach */
"0B9H3ZZ"  =  "1"  /* Drainage of Lung Lingula, Percutaneous Approach */
"0B9H40Z"  =  "1"  /* Drainage of Lung Lingula with Drain Dev, Perc Endo Approach */
"0B9H4ZZ"  =  "1"  /* Drainage of Lung Lingula, Percutaneous Endoscopic Approach */
"0B9H70Z"  =  "1"  /* Drainage of Lung Lingula with Drainage Device, Via Opening */
"0B9H7ZZ"  =  "1"  /* Drainage of Lung Lingula, Via Natural or Artificial Opening */
"0B9H80Z"  =  "1"  /* Drainage of Lung Lingula with Drainage Device, Endo */
"0B9H8ZX"  =  "1"  /* Drainage of Lung Lingula, Endo, Diagn */
"0B9H8ZZ"  =  "1"  /* Drainage of Lung Lingula, Endo */
"0B9J00Z"  =  "1"  /* Drainage of L Low Lung Lobe with Drain Dev, Open Approach */
"0B9J0ZX"  =  "1"  /* Drainage of Left Lower Lung Lobe, Open Approach, Diagnostic */
"0B9J0ZZ"  =  "1"  /* Drainage of Left Lower Lung Lobe, Open Approach */
"0B9J30Z"  =  "1"  /* Drainage of L Low Lung Lobe with Drain Dev, Perc Approach */
"0B9J3ZZ"  =  "1"  /* Drainage of Left Lower Lung Lobe, Percutaneous Approach */
"0B9J40Z"  =  "1"  /* Drain of L Low Lung Lobe with Drain Dev, Perc Endo Approach */
"0B9J4ZZ"  =  "1"  /* Drainage of Left Lower Lung Lobe, Perc Endo Approach */
"0B9J70Z"  =  "1"  /* Drainage of Left Lower Lung Lobe with Drain Dev, Via Opening */
"0B9J7ZZ"  =  "1"  /* Drainage of Left Lower Lung Lobe, Via Opening */
"0B9J80Z"  =  "1"  /* Drainage of Left Lower Lung Lobe with Drainage Device, Endo */
"0B9J8ZX"  =  "1"  /* Drainage of Left Lower Lung Lobe, Endo, Diagn */
"0B9J8ZZ"  =  "1"  /* Drainage of Left Lower Lung Lobe, Endo */
"0B9K00Z"  =  "1"  /* Drainage of Right Lung with Drainage Device, Open Approach */
"0B9K0ZX"  =  "1"  /* Drainage of Right Lung, Open Approach, Diagnostic */
"0B9K0ZZ"  =  "1"  /* Drainage of Right Lung, Open Approach */
"0B9K30Z"  =  "1"  /* Drainage of Right Lung with Drainage Device, Perc Approach */
"0B9K3ZZ"  =  "1"  /* Drainage of Right Lung, Percutaneous Approach */
"0B9K40Z"  =  "1"  /* Drainage of Right Lung with Drain Dev, Perc Endo Approach */
"0B9K4ZZ"  =  "1"  /* Drainage of Right Lung, Percutaneous Endoscopic Approach */
"0B9K70Z"  =  "1"  /* Drainage of Right Lung with Drainage Device, Via Opening */
"0B9K7ZZ"  =  "1"  /* Drainage of Right Lung, Via Natural or Artificial Opening */
"0B9K80Z"  =  "1"  /* Drainage of Right Lung with Drainage Device, Endo */
"0B9K8ZX"  =  "1"  /* Drainage of Right Lung, Endo, Diagn */
"0B9K8ZZ"  =  "1"  /* Drainage of Right Lung, Endo */
"0B9L00Z"  =  "1"  /* Drainage of Left Lung with Drainage Device, Open Approach */
"0B9L0ZX"  =  "1"  /* Drainage of Left Lung, Open Approach, Diagnostic */
"0B9L0ZZ"  =  "1"  /* Drainage of Left Lung, Open Approach */
"0B9L30Z"  =  "1"  /* Drainage of Left Lung with Drainage Device, Perc Approach */
"0B9L3ZZ"  =  "1"  /* Drainage of Left Lung, Percutaneous Approach */
"0B9L40Z"  =  "1"  /* Drainage of Left Lung with Drain Dev, Perc Endo Approach */
"0B9L4ZZ"  =  "1"  /* Drainage of Left Lung, Percutaneous Endoscopic Approach */
"0B9L70Z"  =  "1"  /* Drainage of Left Lung with Drainage Device, Via Opening */
"0B9L7ZZ"  =  "1"  /* Drainage of Left Lung, Via Natural or Artificial Opening */
"0B9L80Z"  =  "1"  /* Drainage of Left Lung with Drainage Device, Endo */
"0B9L8ZX"  =  "1"  /* Drainage of Left Lung, Endo, Diagn */
"0B9L8ZZ"  =  "1"  /* Drainage of Left Lung, Endo */
"0B9M00Z"  =  "1"  /* Drainage of Bilateral Lungs with Drain Dev, Open Approach */
"0B9M0ZX"  =  "1"  /* Drainage of Bilateral Lungs, Open Approach, Diagnostic */
"0B9M0ZZ"  =  "1"  /* Drainage of Bilateral Lungs, Open Approach */
"0B9M30Z"  =  "1"  /* Drainage of Bilateral Lungs with Drain Dev, Perc Approach */
"0B9M3ZZ"  =  "1"  /* Drainage of Bilateral Lungs, Percutaneous Approach */
"0B9M40Z"  =  "1"  /* Drainage of Bi Lungs with Drain Dev, Perc Endo Approach */
"0B9M4ZZ"  =  "1"  /* Drainage of Bilateral Lungs, Perc Endo Approach */
"0B9M70Z"  =  "1"  /* Drainage of Bilateral Lungs with Drain Dev, Via Opening */
"0B9M7ZZ"  =  "1"  /* Drainage of Bilateral Lungs, Via Opening */
"0B9M80Z"  =  "1"  /* Drainage of Bilateral Lungs with Drainage Device, Endo */
"0B9M8ZX"  =  "1"  /* Drainage of Bilateral Lungs, Endo, Diagn */
"0B9M8ZZ"  =  "1"  /* Drainage of Bilateral Lungs, Endo */
"0B9N40Z"  =  "1"  /* Drainage of Right Pleura with Drain Dev, Perc Endo Approach */
"0B9N4ZZ"  =  "1"  /* Drainage of Right Pleura, Percutaneous Endoscopic Approach */
"0B9P40Z"  =  "1"  /* Drainage of Left Pleura with Drain Dev, Perc Endo Approach */
"0B9P4ZZ"  =  "1"  /* Drainage of Left Pleura, Percutaneous Endoscopic Approach */
"0B9R00Z"  =  "1"  /* Drainage of Right Diaphragm with Drain Dev, Open Approach */
"0B9R0ZX"  =  "1"  /* Drainage of Right Diaphragm, Open Approach, Diagnostic */
"0B9R0ZZ"  =  "1"  /* Drainage of Right Diaphragm, Open Approach */
"0B9R30Z"  =  "1"  /* Drainage of Right Diaphragm with Drain Dev, Perc Approach */
"0B9R3ZX"  =  "1"  /* Drainage of Right Diaphragm, Percutaneous Approach, Diagn */
"0B9R3ZZ"  =  "1"  /* Drainage of Right Diaphragm, Percutaneous Approach */
"0B9R40Z"  =  "1"  /* Drainage of R Diaphragm with Drain Dev, Perc Endo Approach */
"0B9R4ZX"  =  "1"  /* Drainage of Right Diaphragm, Perc Endo Approach, Diagn */
"0B9R4ZZ"  =  "1"  /* Drainage of Right Diaphragm, Perc Endo Approach */
"0B9S00Z"  =  "1"  /* Drainage of Left Diaphragm with Drain Dev, Open Approach */
"0B9S0ZX"  =  "1"  /* Drainage of Left Diaphragm, Open Approach, Diagnostic */
"0B9S0ZZ"  =  "1"  /* Drainage of Left Diaphragm, Open Approach */
"0B9S30Z"  =  "1"  /* Drainage of Left Diaphragm with Drain Dev, Perc Approach */
"0B9S3ZX"  =  "1"  /* Drainage of Left Diaphragm, Percutaneous Approach, Diagn */
"0B9S3ZZ"  =  "1"  /* Drainage of Left Diaphragm, Percutaneous Approach */
"0B9S40Z"  =  "1"  /* Drainage of L Diaphragm with Drain Dev, Perc Endo Approach */
"0B9S4ZX"  =  "1"  /* Drainage of Left Diaphragm, Perc Endo Approach, Diagn */
"0B9S4ZZ"  =  "1"  /* Drainage of Left Diaphragm, Percutaneous Endoscopic Approach */
"0BB10ZX"  =  "1"  /* Excision of Trachea, Open Approach, Diagnostic */
"0BB10ZZ"  =  "1"  /* Excision of Trachea, Open Approach */
"0BB13ZZ"  =  "1"  /* Excision of Trachea, Percutaneous Approach */
"0BB14ZZ"  =  "1"  /* Excision of Trachea, Percutaneous Endoscopic Approach */
"0BB17ZZ"  =  "1"  /* Excision of Trachea, Via Natural or Artificial Opening */
"0BB18ZZ"  =  "1"  /* Excision of Trachea, Endo */
"0BB20ZX"  =  "1"  /* Excision of Carina, Open Approach, Diagnostic */
"0BB20ZZ"  =  "1"  /* Excision of Carina, Open Approach */
"0BB23ZZ"  =  "1"  /* Excision of Carina, Percutaneous Approach */
"0BB24ZZ"  =  "1"  /* Excision of Carina, Percutaneous Endoscopic Approach */
"0BB27ZZ"  =  "1"  /* Excision of Carina, Via Natural or Artificial Opening */
"0BB28ZZ"  =  "1"  /* Excision of Carina, Endo */
"0BB30ZX"  =  "1"  /* Excision of Right Main Bronchus, Open Approach, Diagnostic */
"0BB30ZZ"  =  "1"  /* Excision of Right Main Bronchus, Open Approach */
"0BB33ZZ"  =  "1"  /* Excision of Right Main Bronchus, Percutaneous Approach */
"0BB37ZZ"  =  "1"  /* Excision of Right Main Bronchus, Via Opening */
"0BB40ZX"  =  "1"  /* Excision of Right Upper Lobe Bronchus, Open Approach, Diagn */
"0BB40ZZ"  =  "1"  /* Excision of Right Upper Lobe Bronchus, Open Approach */
"0BB43ZZ"  =  "1"  /* Excision of Right Upper Lobe Bronchus, Percutaneous Approach */
"0BB47ZZ"  =  "1"  /* Excision of Right Upper Lobe Bronchus, Via Opening */
"0BB50ZX"  =  "1"  /* Excision of Right Middle Lobe Bronchus, Open Approach, Diagn */
"0BB50ZZ"  =  "1"  /* Excision of Right Middle Lobe Bronchus, Open Approach */
"0BB53ZZ"  =  "1"  /* Excision of Right Middle Lobe Bronchus, Perc Approach */
"0BB57ZZ"  =  "1"  /* Excision of Right Middle Lobe Bronchus, Via Opening */
"0BB60ZX"  =  "1"  /* Excision of Right Lower Lobe Bronchus, Open Approach, Diagn */
"0BB60ZZ"  =  "1"  /* Excision of Right Lower Lobe Bronchus, Open Approach */
"0BB63ZZ"  =  "1"  /* Excision of Right Lower Lobe Bronchus, Percutaneous Approach */
"0BB67ZZ"  =  "1"  /* Excision of Right Lower Lobe Bronchus, Via Opening */
"0BB70ZX"  =  "1"  /* Excision of Left Main Bronchus, Open Approach, Diagnostic */
"0BB70ZZ"  =  "1"  /* Excision of Left Main Bronchus, Open Approach */
"0BB73ZZ"  =  "1"  /* Excision of Left Main Bronchus, Percutaneous Approach */
"0BB77ZZ"  =  "1"  /* Excision of Left Main Bronchus, Via Opening */
"0BB80ZX"  =  "1"  /* Excision of Left Upper Lobe Bronchus, Open Approach, Diagn */
"0BB80ZZ"  =  "1"  /* Excision of Left Upper Lobe Bronchus, Open Approach */
"0BB83ZZ"  =  "1"  /* Excision of Left Upper Lobe Bronchus, Percutaneous Approach */
"0BB87ZZ"  =  "1"  /* Excision of Left Upper Lobe Bronchus, Via Opening */
"0BB90ZX"  =  "1"  /* Excision of Lingula Bronchus, Open Approach, Diagnostic */
"0BB90ZZ"  =  "1"  /* Excision of Lingula Bronchus, Open Approach */
"0BB93ZZ"  =  "1"  /* Excision of Lingula Bronchus, Percutaneous Approach */
"0BB97ZZ"  =  "1"  /* Excision of Lingula Bronchus, Via Opening */
"0BBB0ZX"  =  "1"  /* Excision of Left Lower Lobe Bronchus, Open Approach, Diagn */
"0BBB0ZZ"  =  "1"  /* Excision of Left Lower Lobe Bronchus, Open Approach */
"0BBB3ZZ"  =  "1"  /* Excision of Left Lower Lobe Bronchus, Percutaneous Approach */
"0BBB7ZZ"  =  "1"  /* Excision of Left Lower Lobe Bronchus, Via Opening */
"0BBC0ZX"  =  "1"  /* Excision of Right Upper Lung Lobe, Open Approach, Diagnostic */
"0BBC0ZZ"  =  "1"  /* Excision of Right Upper Lung Lobe, Open Approach */
"0BBC3ZZ"  =  "1"  /* Excision of Right Upper Lung Lobe, Percutaneous Approach */
"0BBC4ZX"  =  "1"  /* Excision of Right Upper Lung Lobe, Perc Endo Approach, Diagn */
"0BBC4ZZ"  =  "1"  /* Excision of Right Upper Lung Lobe, Perc Endo Approach */
"0BBC7ZX"  =  "1"  /* Excision of Right Upper Lung Lobe, Via Opening, Diagn */
"0BBC7ZZ"  =  "1"  /* Excision of Right Upper Lung Lobe, Via Opening */
"0BBC8ZX"  =  "1"  /* Excision of Right Upper Lung Lobe, Endo, Diagn */
"0BBD0ZX"  =  "1"  /* Excision of Right Middle Lung Lobe, Open Approach, Diagn */
"0BBD0ZZ"  =  "1"  /* Excision of Right Middle Lung Lobe, Open Approach */
"0BBD3ZZ"  =  "1"  /* Excision of Right Middle Lung Lobe, Percutaneous Approach */
"0BBD4ZX"  =  "1"  /* Excision of R Mid Lung Lobe, Perc Endo Approach, Diagn */
"0BBD4ZZ"  =  "1"  /* Excision of Right Middle Lung Lobe, Perc Endo Approach */
"0BBD7ZX"  =  "1"  /* Excision of Right Middle Lung Lobe, Via Opening, Diagn */
"0BBD7ZZ"  =  "1"  /* Excision of Right Middle Lung Lobe, Via Opening */
"0BBD8ZX"  =  "1"  /* Excision of Right Middle Lung Lobe, Endo, Diagn */
"0BBF0ZX"  =  "1"  /* Excision of Right Lower Lung Lobe, Open Approach, Diagnostic */
"0BBF0ZZ"  =  "1"  /* Excision of Right Lower Lung Lobe, Open Approach */
"0BBF3ZZ"  =  "1"  /* Excision of Right Lower Lung Lobe, Percutaneous Approach */
"0BBF4ZX"  =  "1"  /* Excision of Right Lower Lung Lobe, Perc Endo Approach, Diagn */
"0BBF4ZZ"  =  "1"  /* Excision of Right Lower Lung Lobe, Perc Endo Approach */
"0BBF7ZX"  =  "1"  /* Excision of Right Lower Lung Lobe, Via Opening, Diagn */
"0BBF7ZZ"  =  "1"  /* Excision of Right Lower Lung Lobe, Via Opening */
"0BBF8ZX"  =  "1"  /* Excision of Right Lower Lung Lobe, Endo, Diagn */
"0BBG0ZX"  =  "1"  /* Excision of Left Upper Lung Lobe, Open Approach, Diagnostic */
"0BBG0ZZ"  =  "1"  /* Excision of Left Upper Lung Lobe, Open Approach */
"0BBG3ZZ"  =  "1"  /* Excision of Left Upper Lung Lobe, Percutaneous Approach */
"0BBG4ZX"  =  "1"  /* Excision of Left Upper Lung Lobe, Perc Endo Approach, Diagn */
"0BBG4ZZ"  =  "1"  /* Excision of Left Upper Lung Lobe, Perc Endo Approach */
"0BBG7ZX"  =  "1"  /* Excision of Left Upper Lung Lobe, Via Opening, Diagn */
"0BBG7ZZ"  =  "1"  /* Excision of Left Upper Lung Lobe, Via Opening */
"0BBG8ZX"  =  "1"  /* Excision of Left Upper Lung Lobe, Endo, Diagn */
"0BBH0ZX"  =  "1"  /* Excision of Lung Lingula, Open Approach, Diagnostic */
"0BBH0ZZ"  =  "1"  /* Excision of Lung Lingula, Open Approach */
"0BBH3ZZ"  =  "1"  /* Excision of Lung Lingula, Percutaneous Approach */
"0BBH4ZX"  =  "1"  /* Excision of Lung Lingula, Perc Endo Approach, Diagn */
"0BBH4ZZ"  =  "1"  /* Excision of Lung Lingula, Percutaneous Endoscopic Approach */
"0BBH7ZX"  =  "1"  /* Excision of Lung Lingula, Via Opening, Diagn */
"0BBH7ZZ"  =  "1"  /* Excision of Lung Lingula, Via Natural or Artificial Opening */
"0BBH8ZX"  =  "1"  /* Excision of Lung Lingula, Endo, Diagn */
"0BBJ0ZX"  =  "1"  /* Excision of Left Lower Lung Lobe, Open Approach, Diagnostic */
"0BBJ0ZZ"  =  "1"  /* Excision of Left Lower Lung Lobe, Open Approach */
"0BBJ3ZZ"  =  "1"  /* Excision of Left Lower Lung Lobe, Percutaneous Approach */
"0BBJ4ZX"  =  "1"  /* Excision of Left Lower Lung Lobe, Perc Endo Approach, Diagn */
"0BBJ4ZZ"  =  "1"  /* Excision of Left Lower Lung Lobe, Perc Endo Approach */
"0BBJ7ZX"  =  "1"  /* Excision of Left Lower Lung Lobe, Via Opening, Diagn */
"0BBJ7ZZ"  =  "1"  /* Excision of Left Lower Lung Lobe, Via Opening */
"0BBJ8ZX"  =  "1"  /* Excision of Left Lower Lung Lobe, Endo, Diagn */
"0BBK0ZX"  =  "1"  /* Excision of Right Lung, Open Approach, Diagnostic */
"0BBK0ZZ"  =  "1"  /* Excision of Right Lung, Open Approach */
"0BBK3ZZ"  =  "1"  /* Excision of Right Lung, Percutaneous Approach */
"0BBK4ZX"  =  "1"  /* Excision of Right Lung, Perc Endo Approach, Diagn */
"0BBK4ZZ"  =  "1"  /* Excision of Right Lung, Percutaneous Endoscopic Approach */
"0BBK7ZX"  =  "1"  /* Excision of Right Lung, Via Opening, Diagn */
"0BBK7ZZ"  =  "1"  /* Excision of Right Lung, Via Natural or Artificial Opening */
"0BBK8ZX"  =  "1"  /* Excision of Right Lung, Endo, Diagn */
"0BBL0ZX"  =  "1"  /* Excision of Left Lung, Open Approach, Diagnostic */
"0BBL0ZZ"  =  "1"  /* Excision of Left Lung, Open Approach */
"0BBL3ZZ"  =  "1"  /* Excision of Left Lung, Percutaneous Approach */
"0BBL4ZX"  =  "1"  /* Excision of Left Lung, Perc Endo Approach, Diagn */
"0BBL4ZZ"  =  "1"  /* Excision of Left Lung, Percutaneous Endoscopic Approach */
"0BBL7ZX"  =  "1"  /* Excision of Left Lung, Via Opening, Diagn */
"0BBL7ZZ"  =  "1"  /* Excision of Left Lung, Via Natural or Artificial Opening */
"0BBL8ZX"  =  "1"  /* Excision of Left Lung, Endo, Diagn */
"0BBM0ZX"  =  "1"  /* Excision of Bilateral Lungs, Open Approach, Diagnostic */
"0BBM0ZZ"  =  "1"  /* Excision of Bilateral Lungs, Open Approach */
"0BBM3ZZ"  =  "1"  /* Excision of Bilateral Lungs, Percutaneous Approach */
"0BBM4ZX"  =  "1"  /* Excision of Bilateral Lungs, Perc Endo Approach, Diagn */
"0BBM7ZX"  =  "1"  /* Excision of Bilateral Lungs, Via Opening, Diagn */
"0BBM7ZZ"  =  "1"  /* Excision of Bilateral Lungs, Via Opening */
"0BBM8ZX"  =  "1"  /* Excision of Bilateral Lungs, Endo, Diagn */
"0BBN0ZZ"  =  "1"  /* Excision of Right Pleura, Open Approach */
"0BBN3ZZ"  =  "1"  /* Excision of Right Pleura, Percutaneous Approach */
"0BBN4ZX"  =  "1"  /* Excision of Right Pleura, Perc Endo Approach, Diagn */
"0BBN4ZZ"  =  "1"  /* Excision of Right Pleura, Percutaneous Endoscopic Approach */
"0BBP0ZZ"  =  "1"  /* Excision of Left Pleura, Open Approach */
"0BBP3ZZ"  =  "1"  /* Excision of Left Pleura, Percutaneous Approach */
"0BBP4ZX"  =  "1"  /* Excision of Left Pleura, Perc Endo Approach, Diagn */
"0BBP4ZZ"  =  "1"  /* Excision of Left Pleura, Percutaneous Endoscopic Approach */
"0BBR0ZX"  =  "1"  /* Excision of Right Diaphragm, Open Approach, Diagnostic */
"0BBR0ZZ"  =  "1"  /* Excision of Right Diaphragm, Open Approach */
"0BBR3ZX"  =  "1"  /* Excision of Right Diaphragm, Percutaneous Approach, Diagn */
"0BBR3ZZ"  =  "1"  /* Excision of Right Diaphragm, Percutaneous Approach */
"0BBR4ZX"  =  "1"  /* Excision of Right Diaphragm, Perc Endo Approach, Diagn */
"0BBR4ZZ"  =  "1"  /* Excision of Right Diaphragm, Perc Endo Approach */
"0BBS0ZX"  =  "1"  /* Excision of Left Diaphragm, Open Approach, Diagnostic */
"0BBS0ZZ"  =  "1"  /* Excision of Left Diaphragm, Open Approach */
"0BBS3ZX"  =  "1"  /* Excision of Left Diaphragm, Percutaneous Approach, Diagn */
"0BBS3ZZ"  =  "1"  /* Excision of Left Diaphragm, Percutaneous Approach */
"0BBS4ZX"  =  "1"  /* Excision of Left Diaphragm, Perc Endo Approach, Diagn */
"0BBS4ZZ"  =  "1"  /* Excision of Left Diaphragm, Percutaneous Endoscopic Approach */
"0BC10ZZ"  =  "1"  /* Extirpation of Matter from Trachea, Open Approach */
"0BC13ZZ"  =  "1"  /* Extirpation of Matter from Trachea, Percutaneous Approach */
"0BC14ZZ"  =  "1"  /* Extirpation of Matter from Trachea, Perc Endo Approach */
"0BC20ZZ"  =  "1"  /* Extirpation of Matter from Carina, Open Approach */
"0BC23ZZ"  =  "1"  /* Extirpation of Matter from Carina, Percutaneous Approach */
"0BC24ZZ"  =  "1"  /* Extirpation of Matter from Carina, Perc Endo Approach */
"0BC30ZZ"  =  "1"  /* Extirpation of Matter from R Main Bronc, Open Approach */
"0BC33ZZ"  =  "1"  /* Extirpation of Matter from R Main Bronc, Perc Approach */
"0BC34ZZ"  =  "1"  /* Extirpation of Matter from R Main Bronc, Perc Endo Approach */
"0BC40ZZ"  =  "1"  /* Extirpation of Matter from R Up Lobe Bronc, Open Approach */
"0BC43ZZ"  =  "1"  /* Extirpation of Matter from R Up Lobe Bronc, Perc Approach */
"0BC44ZZ"  =  "1"  /* Extirpate of Matter from R Up Lobe Bronc, Perc Endo Approach */
"0BC50ZZ"  =  "1"  /* Extirpation of Matter from R Mid Lobe Bronc, Open Approach */
"0BC53ZZ"  =  "1"  /* Extirpation of Matter from R Mid Lobe Bronc, Perc Approach */
"0BC54ZZ"  =  "1"  /* Extirpate matter from R Mid Lobe Bronc, Perc Endo */
"0BC60ZZ"  =  "1"  /* Extirpation of Matter from R Low Lobe Bronc, Open Approach */
"0BC63ZZ"  =  "1"  /* Extirpation of Matter from R Low Lobe Bronc, Perc Approach */
"0BC64ZZ"  =  "1"  /* Extirpate matter from R Low Lobe Bronc, Perc Endo */
"0BC70ZZ"  =  "1"  /* Extirpation of Matter from Left Main Bronchus, Open Approach */
"0BC73ZZ"  =  "1"  /* Extirpation of Matter from Left Main Bronchus, Perc Approach */
"0BC74ZZ"  =  "1"  /* Extirpation of Matter from L Main Bronc, Perc Endo Approach */
"0BC80ZZ"  =  "1"  /* Extirpation of Matter from L Up Lobe Bronc, Open Approach */
"0BC83ZZ"  =  "1"  /* Extirpation of Matter from L Up Lobe Bronc, Perc Approach */
"0BC84ZZ"  =  "1"  /* Extirpate of Matter from L Up Lobe Bronc, Perc Endo Approach */
"0BC90ZZ"  =  "1"  /* Extirpation of Matter from Lingula Bronchus, Open Approach */
"0BC93ZZ"  =  "1"  /* Extirpation of Matter from Lingula Bronchus, Perc Approach */
"0BC94ZZ"  =  "1"  /* Extirpation of Matter from Lingula Bronc, Perc Endo Approach */
"0BCB0ZZ"  =  "1"  /* Extirpation of Matter from L Low Lobe Bronc, Open Approach */
"0BCB3ZZ"  =  "1"  /* Extirpation of Matter from L Low Lobe Bronc, Perc Approach */
"0BCB4ZZ"  =  "1"  /* Extirpate matter from L Low Lobe Bronc, Perc Endo */
"0BCC0ZZ"  =  "1"  /* Extirpation of Matter from R Up Lung Lobe, Open Approach */
"0BCC3ZZ"  =  "1"  /* Extirpation of Matter from R Up Lung Lobe, Perc Approach */
"0BCC4ZZ"  =  "1"  /* Extirpate of Matter from R Up Lung Lobe, Perc Endo Approach */
"0BCC7ZZ"  =  "1"  /* Extirpation of Matter from R Up Lung Lobe, Via Opening */
"0BCC8ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Lung Lobe, Endo */
"0BCD0ZZ"  =  "1"  /* Extirpation of Matter from R Mid Lung Lobe, Open Approach */
"0BCD3ZZ"  =  "1"  /* Extirpation of Matter from R Mid Lung Lobe, Perc Approach */
"0BCD4ZZ"  =  "1"  /* Extirpate of Matter from R Mid Lung Lobe, Perc Endo Approach */
"0BCD7ZZ"  =  "1"  /* Extirpation of Matter from R Mid Lung Lobe, Via Opening */
"0BCD8ZZ"  =  "1"  /* Extirpation of Matter from Right Middle Lung Lobe, Endo */
"0BCF0ZZ"  =  "1"  /* Extirpation of Matter from R Low Lung Lobe, Open Approach */
"0BCF3ZZ"  =  "1"  /* Extirpation of Matter from R Low Lung Lobe, Perc Approach */
"0BCF4ZZ"  =  "1"  /* Extirpate of Matter from R Low Lung Lobe, Perc Endo Approach */
"0BCF7ZZ"  =  "1"  /* Extirpation of Matter from R Low Lung Lobe, Via Opening */
"0BCF8ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Lung Lobe, Endo */
"0BCG0ZZ"  =  "1"  /* Extirpation of Matter from L Up Lung Lobe, Open Approach */
"0BCG3ZZ"  =  "1"  /* Extirpation of Matter from L Up Lung Lobe, Perc Approach */
"0BCG4ZZ"  =  "1"  /* Extirpate of Matter from L Up Lung Lobe, Perc Endo Approach */
"0BCG7ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lung Lobe, Via Opening */
"0BCG8ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Lung Lobe, Endo */
"0BCH0ZZ"  =  "1"  /* Extirpation of Matter from Lung Lingula, Open Approach */
"0BCH3ZZ"  =  "1"  /* Extirpation of Matter from Lung Lingula, Perc Approach */
"0BCH4ZZ"  =  "1"  /* Extirpation of Matter from Lung Lingula, Perc Endo Approach */
"0BCH7ZZ"  =  "1"  /* Extirpation of Matter from Lung Lingula, Via Opening */
"0BCH8ZZ"  =  "1"  /* Extirpation of Matter from Lung Lingula, Endo */
"0BCJ0ZZ"  =  "1"  /* Extirpation of Matter from L Low Lung Lobe, Open Approach */
"0BCJ3ZZ"  =  "1"  /* Extirpation of Matter from L Low Lung Lobe, Perc Approach */
"0BCJ4ZZ"  =  "1"  /* Extirpate of Matter from L Low Lung Lobe, Perc Endo Approach */
"0BCJ7ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lung Lobe, Via Opening */
"0BCJ8ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Lung Lobe, Endo */
"0BCK0ZZ"  =  "1"  /* Extirpation of Matter from Right Lung, Open Approach */
"0BCK3ZZ"  =  "1"  /* Extirpation of Matter from Right Lung, Percutaneous Approach */
"0BCK4ZZ"  =  "1"  /* Extirpation of Matter from Right Lung, Perc Endo Approach */
"0BCK7ZZ"  =  "1"  /* Extirpation of Matter from Right Lung, Via Opening */
"0BCK8ZZ"  =  "1"  /* Extirpation of Matter from Right Lung, Endo */
"0BCL0ZZ"  =  "1"  /* Extirpation of Matter from Left Lung, Open Approach */
"0BCL3ZZ"  =  "1"  /* Extirpation of Matter from Left Lung, Percutaneous Approach */
"0BCL4ZZ"  =  "1"  /* Extirpation of Matter from Left Lung, Perc Endo Approach */
"0BCL7ZZ"  =  "1"  /* Extirpation of Matter from Left Lung, Via Opening */
"0BCL8ZZ"  =  "1"  /* Extirpation of Matter from Left Lung, Endo */
"0BCM0ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Lungs, Open Approach */
"0BCM3ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Lungs, Perc Approach */
"0BCM4ZZ"  =  "1"  /* Extirpation of Matter from Bi Lungs, Perc Endo Approach */
"0BCM7ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Lungs, Via Opening */
"0BCM8ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Lungs, Endo */
"0BCR0ZZ"  =  "1"  /* Extirpation of Matter from Right Diaphragm, Open Approach */
"0BCR3ZZ"  =  "1"  /* Extirpation of Matter from Right Diaphragm, Perc Approach */
"0BCR4ZZ"  =  "1"  /* Extirpation of Matter from R Diaphragm, Perc Endo Approach */
"0BCS0ZZ"  =  "1"  /* Extirpation of Matter from Left Diaphragm, Open Approach */
"0BCS3ZZ"  =  "1"  /* Extirpation of Matter from Left Diaphragm, Perc Approach */
"0BCS4ZZ"  =  "1"  /* Extirpation of Matter from L Diaphragm, Perc Endo Approach */
"0BDN0ZX"  =  "1"  /* Extraction of Right Pleura, Open Approach, Diagnostic */
"0BDN0ZZ"  =  "1"  /* Extraction of Right Pleura, Open Approach */
"0BDN3ZX"  =  "1"  /* Extraction of Right Pleura, Percutaneous Approach, Diagn */
"0BDN3ZZ"  =  "1"  /* Extraction of Right Pleura, Percutaneous Approach */
"0BDN4ZX"  =  "1"  /* Extraction of Right Pleura, Perc Endo Approach, Diagn */
"0BDN4ZZ"  =  "1"  /* Extraction of Right Pleura, Percutaneous Endoscopic Approach */
"0BDP0ZX"  =  "1"  /* Extraction of Left Pleura, Open Approach, Diagnostic */
"0BDP0ZZ"  =  "1"  /* Extraction of Left Pleura, Open Approach */
"0BDP3ZX"  =  "1"  /* Extraction of Left Pleura, Percutaneous Approach, Diagnostic */
"0BDP3ZZ"  =  "1"  /* Extraction of Left Pleura, Percutaneous Approach */
"0BDP4ZX"  =  "1"  /* Extraction of Left Pleura, Perc Endo Approach, Diagn */
"0BDP4ZZ"  =  "1"  /* Extraction of Left Pleura, Percutaneous Endoscopic Approach */
"0BF10ZZ"  =  "1"  /* Fragmentation in Trachea, Open Approach */
"0BF13ZZ"  =  "1"  /* Fragmentation in Trachea, Percutaneous Approach */
"0BF14ZZ"  =  "1"  /* Fragmentation in Trachea, Percutaneous Endoscopic Approach */
"0BF17ZZ"  =  "1"  /* Fragmentation in Trachea, Via Natural or Artificial Opening */
"0BF18ZZ"  =  "1"  /* Fragmentation in Trachea, Endo */
"0BF20ZZ"  =  "1"  /* Fragmentation in Carina, Open Approach */
"0BF23ZZ"  =  "1"  /* Fragmentation in Carina, Percutaneous Approach */
"0BF24ZZ"  =  "1"  /* Fragmentation in Carina, Percutaneous Endoscopic Approach */
"0BF27ZZ"  =  "1"  /* Fragmentation in Carina, Via Natural or Artificial Opening */
"0BF28ZZ"  =  "1"  /* Fragmentation in Carina, Endo */
"0BF30ZZ"  =  "1"  /* Fragmentation in Right Main Bronchus, Open Approach */
"0BF33ZZ"  =  "1"  /* Fragmentation in Right Main Bronchus, Percutaneous Approach */
"0BF34ZZ"  =  "1"  /* Fragmentation in Right Main Bronchus, Perc Endo Approach */
"0BF37ZZ"  =  "1"  /* Fragmentation in Right Main Bronchus, Via Opening */
"0BF38ZZ"  =  "1"  /* Fragmentation in Right Main Bronchus, Endo */
"0BF40ZZ"  =  "1"  /* Fragmentation in Right Upper Lobe Bronchus, Open Approach */
"0BF43ZZ"  =  "1"  /* Fragmentation in Right Upper Lobe Bronchus, Perc Approach */
"0BF44ZZ"  =  "1"  /* Fragmentation in R Up Lobe Bronc, Perc Endo Approach */
"0BF47ZZ"  =  "1"  /* Fragmentation in Right Upper Lobe Bronchus, Via Opening */
"0BF48ZZ"  =  "1"  /* Fragmentation in Right Upper Lobe Bronchus, Endo */
"0BF50ZZ"  =  "1"  /* Fragmentation in Right Middle Lobe Bronchus, Open Approach */
"0BF53ZZ"  =  "1"  /* Fragmentation in Right Middle Lobe Bronchus, Perc Approach */
"0BF54ZZ"  =  "1"  /* Fragmentation in R Mid Lobe Bronc, Perc Endo Approach */
"0BF57ZZ"  =  "1"  /* Fragmentation in Right Middle Lobe Bronchus, Via Opening */
"0BF58ZZ"  =  "1"  /* Fragmentation in Right Middle Lobe Bronchus, Endo */
"0BF60ZZ"  =  "1"  /* Fragmentation in Right Lower Lobe Bronchus, Open Approach */
"0BF63ZZ"  =  "1"  /* Fragmentation in Right Lower Lobe Bronchus, Perc Approach */
"0BF64ZZ"  =  "1"  /* Fragmentation in R Low Lobe Bronc, Perc Endo Approach */
"0BF67ZZ"  =  "1"  /* Fragmentation in Right Lower Lobe Bronchus, Via Opening */
"0BF68ZZ"  =  "1"  /* Fragmentation in Right Lower Lobe Bronchus, Endo */
"0BF70ZZ"  =  "1"  /* Fragmentation in Left Main Bronchus, Open Approach */
"0BF73ZZ"  =  "1"  /* Fragmentation in Left Main Bronchus, Percutaneous Approach */
"0BF74ZZ"  =  "1"  /* Fragmentation in Left Main Bronchus, Perc Endo Approach */
"0BF77ZZ"  =  "1"  /* Fragmentation in Left Main Bronchus, Via Opening */
"0BF78ZZ"  =  "1"  /* Fragmentation in Left Main Bronchus, Endo */
"0BF80ZZ"  =  "1"  /* Fragmentation in Left Upper Lobe Bronchus, Open Approach */
"0BF83ZZ"  =  "1"  /* Fragmentation in Left Upper Lobe Bronchus, Perc Approach */
"0BF84ZZ"  =  "1"  /* Fragmentation in L Up Lobe Bronc, Perc Endo Approach */
"0BF87ZZ"  =  "1"  /* Fragmentation in Left Upper Lobe Bronchus, Via Opening */
"0BF88ZZ"  =  "1"  /* Fragmentation in Left Upper Lobe Bronchus, Endo */
"0BF90ZZ"  =  "1"  /* Fragmentation in Lingula Bronchus, Open Approach */
"0BF93ZZ"  =  "1"  /* Fragmentation in Lingula Bronchus, Percutaneous Approach */
"0BF94ZZ"  =  "1"  /* Fragmentation in Lingula Bronchus, Perc Endo Approach */
"0BF97ZZ"  =  "1"  /* Fragmentation in Lingula Bronchus, Via Opening */
"0BF98ZZ"  =  "1"  /* Fragmentation in Lingula Bronchus, Endo */
"0BFB0ZZ"  =  "1"  /* Fragmentation in Left Lower Lobe Bronchus, Open Approach */
"0BFB3ZZ"  =  "1"  /* Fragmentation in Left Lower Lobe Bronchus, Perc Approach */
"0BFB4ZZ"  =  "1"  /* Fragmentation in L Low Lobe Bronc, Perc Endo Approach */
"0BFB7ZZ"  =  "1"  /* Fragmentation in Left Lower Lobe Bronchus, Via Opening */
"0BFB8ZZ"  =  "1"  /* Fragmentation in Left Lower Lobe Bronchus, Endo */
"0BH001Z"  =  "1"  /* Insert Radioact Elem in Tracheobronc Tree, Open */
"0BH002Z"  =  "1"  /* Insert of Monitor Dev into Tracheobronc Tree, Open Approach */
"0BH003Z"  =  "1"  /* Insert of Infusion Dev into Tracheobronc Tree, Open Approach */
"0BH00DZ"  =  "1"  /* Insert of Intralum Dev into Tracheobronc Tree, Open Approach */
"0BH031Z"  =  "1"  /* Insert Radioact Elem in Tracheobronc Tree, Perc */
"0BH032Z"  =  "1"  /* Insert of Monitor Dev into Tracheobronc Tree, Perc Approach */
"0BH033Z"  =  "1"  /* Insert of Infusion Dev into Tracheobronc Tree, Perc Approach */
"0BH03DZ"  =  "1"  /* Insert of Intralum Dev into Tracheobronc Tree, Perc Approach */
"0BH041Z"  =  "1"  /* Insert Radioact Elem in Tracheobronc Tree, Perc Endo */
"0BH042Z"  =  "1"  /* Insert Monitor Dev in Tracheobronc Tree, Perc Endo */
"0BH043Z"  =  "1"  /* Insert Infusion Dev in Tracheobronc Tree, Perc Endo */
"0BH04DZ"  =  "1"  /* Insert Intralum Dev in Tracheobronc Tree, Perc Endo */
"0BH071Z"  =  "1"  /* Insert of Radioact Elem into Tracheobronc Tree, Via Opening */
"0BH081Z"  =  "1"  /* Insertion of Radioact Elem into Tracheobronc Tree, Endo */
"0BH102Z"  =  "1"  /* Insertion of Monitoring Device into Trachea, Open Approach */
"0BH10DZ"  =  "1"  /* Insertion of Intraluminal Device into Trachea, Open Approach */
"0BH13DZ"  =  "1"  /* Insertion of Intraluminal Device into Trachea, Perc Approach */
"0BH14DZ"  =  "1"  /* Insertion of Intralum Dev into Trachea, Perc Endo Approach */
"0BH172Z"  =  "1"  /* Insertion of Monitoring Device into Trachea, Via Opening */
"0BH17DZ"  =  "1"  /* Insertion of Intraluminal Device into Trachea, Via Opening */
"0BH182Z"  =  "1"  /* Insertion of Monitoring Device into Trachea, Endo */
"0BH18DZ"  =  "1"  /* Insertion of Intraluminal Device into Trachea, Endo */
"0BH30GZ"  =  "1"  /* Insert of Endobronch Valve into R Main Bronc, Open Approach */
"0BH33GZ"  =  "1"  /* Insert of Endobronch Valve into R Main Bronc, Perc Approach */
"0BH34GZ"  =  "1"  /* Insert Endobronch Valve in R Main Bronc, Perc Endo */
"0BH37GZ"  =  "1"  /* Insertion of Endobronch Valve into R Main Bronc, Via Opening */
"0BH40GZ"  =  "1"  /* Insert Endobronch Valve in R Up Lobe Bronc, Open */
"0BH43GZ"  =  "1"  /* Insert Endobronch Valve in R Up Lobe Bronc, Perc */
"0BH44GZ"  =  "1"  /* Insert Endobronch Valve in R Up Lobe Bronc, Perc Endo */
"0BH47GZ"  =  "1"  /* Insert of Endobronch Valve into R Up Lobe Bronc, Via Opening */
"0BH50GZ"  =  "1"  /* Insert Endobronch Valve in R Mid Lobe Bronc, Open */
"0BH53GZ"  =  "1"  /* Insert Endobronch Valve in R Mid Lobe Bronc, Perc */
"0BH54GZ"  =  "1"  /* Insert Endobronch Valve in R Mid Lobe Bronc, Perc Endo */
"0BH57GZ"  =  "1"  /* Insert Endobronch Valve in R Mid Lobe Bronc, Via Opening */
"0BH60GZ"  =  "1"  /* Insert Endobronch Valve in R Low Lobe Bronc, Open */
"0BH63GZ"  =  "1"  /* Insert Endobronch Valve in R Low Lobe Bronc, Perc */
"0BH64GZ"  =  "1"  /* Insert Endobronch Valve in R Low Lobe Bronc, Perc Endo */
"0BH67GZ"  =  "1"  /* Insert Endobronch Valve in R Low Lobe Bronc, Via Opening */
"0BH70GZ"  =  "1"  /* Insert of Endobronch Valve into L Main Bronc, Open Approach */
"0BH73GZ"  =  "1"  /* Insert of Endobronch Valve into L Main Bronc, Perc Approach */
"0BH74GZ"  =  "1"  /* Insert Endobronch Valve in L Main Bronc, Perc Endo */
"0BH77GZ"  =  "1"  /* Insertion of Endobronch Valve into L Main Bronc, Via Opening */
"0BH80GZ"  =  "1"  /* Insert Endobronch Valve in L Up Lobe Bronc, Open */
"0BH83GZ"  =  "1"  /* Insert Endobronch Valve in L Up Lobe Bronc, Perc */
"0BH84GZ"  =  "1"  /* Insert Endobronch Valve in L Up Lobe Bronc, Perc Endo */
"0BH87GZ"  =  "1"  /* Insert of Endobronch Valve into L Up Lobe Bronc, Via Opening */
"0BH90GZ"  =  "1"  /* Insert of Endobronch Valve into Lingula Bronc, Open Approach */
"0BH93GZ"  =  "1"  /* Insert of Endobronch Valve into Lingula Bronc, Perc Approach */
"0BH94GZ"  =  "1"  /* Insert Endobronch Valve in Lingula Bronc, Perc Endo */
"0BH97GZ"  =  "1"  /* Insert of Endobronch Valve into Lingula Bronc, Via Opening */
"0BHB0GZ"  =  "1"  /* Insert Endobronch Valve in L Low Lobe Bronc, Open */
"0BHB3GZ"  =  "1"  /* Insert Endobronch Valve in L Low Lobe Bronc, Perc */
"0BHB4GZ"  =  "1"  /* Insert Endobronch Valve in L Low Lobe Bronc, Perc Endo */
"0BHB7GZ"  =  "1"  /* Insert Endobronch Valve in L Low Lobe Bronc, Via Opening */
"0BHK01Z"  =  "1"  /* Insertion of Radioactive Element into R Lung, Open Approach */
"0BHK02Z"  =  "1"  /* Insertion of Monitoring Device into R Lung, Open Approach */
"0BHK03Z"  =  "1"  /* Insertion of Infusion Device into Right Lung, Open Approach */
"0BHK31Z"  =  "1"  /* Insertion of Radioactive Element into R Lung, Perc Approach */
"0BHK32Z"  =  "1"  /* Insertion of Monitoring Device into R Lung, Perc Approach */
"0BHK33Z"  =  "1"  /* Insertion of Infusion Device into Right Lung, Perc Approach */
"0BHK41Z"  =  "1"  /* Insertion of Radioact Elem into R Lung, Perc Endo Approach */
"0BHK42Z"  =  "1"  /* Insertion of Monitor Dev into R Lung, Perc Endo Approach */
"0BHK43Z"  =  "1"  /* Insertion of Infusion Device into R Lung, Perc Endo Approach */
"0BHK71Z"  =  "1"  /* Insertion of Radioactive Element into R Lung, Via Opening */
"0BHK72Z"  =  "1"  /* Insertion of Monitoring Device into Right Lung, Via Opening */
"0BHK73Z"  =  "1"  /* Insertion of Infusion Device into Right Lung, Via Opening */
"0BHK81Z"  =  "1"  /* Insertion of Radioactive Element into Right Lung, Endo */
"0BHK82Z"  =  "1"  /* Insertion of Monitoring Device into Right Lung, Endo */
"0BHK83Z"  =  "1"  /* Insertion of Infusion Device into Right Lung, Endo */
"0BHL01Z"  =  "1"  /* Insertion of Radioactive Element into L Lung, Open Approach */
"0BHL02Z"  =  "1"  /* Insertion of Monitoring Device into Left Lung, Open Approach */
"0BHL03Z"  =  "1"  /* Insertion of Infusion Device into Left Lung, Open Approach */
"0BHL31Z"  =  "1"  /* Insertion of Radioactive Element into L Lung, Perc Approach */
"0BHL32Z"  =  "1"  /* Insertion of Monitoring Device into Left Lung, Perc Approach */
"0BHL33Z"  =  "1"  /* Insertion of Infusion Device into Left Lung, Perc Approach */
"0BHL41Z"  =  "1"  /* Insertion of Radioact Elem into L Lung, Perc Endo Approach */
"0BHL42Z"  =  "1"  /* Insertion of Monitor Dev into L Lung, Perc Endo Approach */
"0BHL43Z"  =  "1"  /* Insertion of Infusion Device into L Lung, Perc Endo Approach */
"0BHL71Z"  =  "1"  /* Insertion of Radioactive Element into Left Lung, Via Opening */
"0BHL72Z"  =  "1"  /* Insertion of Monitoring Device into Left Lung, Via Opening */
"0BHL73Z"  =  "1"  /* Insertion of Infusion Device into Left Lung, Via Opening */
"0BHL81Z"  =  "1"  /* Insertion of Radioactive Element into Left Lung, Endo */
"0BHL82Z"  =  "1"  /* Insertion of Monitoring Device into Left Lung, Endo */
"0BHL83Z"  =  "1"  /* Insertion of Infusion Device into Left Lung, Endo */
"0BHR02Z"  =  "1"  /* Insertion of Monitor Dev into R Diaphragm, Open Approach */
"0BHR0MZ"  =  "1"  /* Insertion of Diaphragm Lead into R Diaphragm, Open Approach */
"0BHR32Z"  =  "1"  /* Insertion of Monitor Dev into R Diaphragm, Perc Approach */
"0BHR3MZ"  =  "1"  /* Insertion of Diaphragm Lead into R Diaphragm, Perc Approach */
"0BHR42Z"  =  "1"  /* Insert of Monitor Dev into R Diaphragm, Perc Endo Approach */
"0BHR4MZ"  =  "1"  /* Insert Diaphragm Lead in R Diaphragm, Perc Endo */
"0BHS02Z"  =  "1"  /* Insertion of Monitor Dev into L Diaphragm, Open Approach */
"0BHS0MZ"  =  "1"  /* Insertion of Diaphragm Lead into L Diaphragm, Open Approach */
"0BHS32Z"  =  "1"  /* Insertion of Monitor Dev into L Diaphragm, Perc Approach */
"0BHS3MZ"  =  "1"  /* Insertion of Diaphragm Lead into L Diaphragm, Perc Approach */
"0BHS42Z"  =  "1"  /* Insert of Monitor Dev into L Diaphragm, Perc Endo Approach */
"0BHS4MZ"  =  "1"  /* Insert Diaphragm Lead in L Diaphragm, Perc Endo */
"0BJ00ZZ"  =  "1"  /* Inspection of Tracheobronchial Tree, Open Approach */
"0BJ03ZZ"  =  "1"  /* Inspection of Tracheobronchial Tree, Percutaneous Approach */
"0BJ04ZZ"  =  "1"  /* Inspection of Tracheobronchial Tree, Perc Endo Approach */
"0BJ07ZZ"  =  "1"  /* Inspection of Tracheobronchial Tree, Via Opening */
"0BJ0XZZ"  =  "1"  /* Inspection of Tracheobronchial Tree, External Approach */
"0BJ10ZZ"  =  "1"  /* Inspection of Trachea, Open Approach */
"0BJK0ZZ"  =  "1"  /* Inspection of Right Lung, Open Approach */
"0BJK3ZZ"  =  "1"  /* Inspection of Right Lung, Percutaneous Approach */
"0BJK4ZZ"  =  "1"  /* Inspection of Right Lung, Percutaneous Endoscopic Approach */
"0BJK7ZZ"  =  "1"  /* Inspection of Right Lung, Via Natural or Artificial Opening */
"0BJKXZZ"  =  "1"  /* Inspection of Right Lung, External Approach */
"0BJL0ZZ"  =  "1"  /* Inspection of Left Lung, Open Approach */
"0BJL3ZZ"  =  "1"  /* Inspection of Left Lung, Percutaneous Approach */
"0BJL4ZZ"  =  "1"  /* Inspection of Left Lung, Percutaneous Endoscopic Approach */
"0BJL7ZZ"  =  "1"  /* Inspection of Left Lung, Via Natural or Artificial Opening */
"0BJLXZZ"  =  "1"  /* Inspection of Left Lung, External Approach */
"0BJQ0ZZ"  =  "1"  /* Inspection of Pleura, Open Approach */
"0BJQ3ZZ"  =  "1"  /* Inspection of Pleura, Percutaneous Approach */
"0BJQ4ZZ"  =  "1"  /* Inspection of Pleura, Percutaneous Endoscopic Approach */
"0BJQ7ZZ"  =  "1"  /* Inspection of Pleura, Via Natural or Artificial Opening */
"0BJQ8ZZ"  =  "1"  /* Inspection of Pleura, Endo */
"0BJQXZZ"  =  "1"  /* Inspection of Pleura, External Approach */
"0BJT0ZZ"  =  "1"  /* Inspection of Diaphragm, Open Approach */
"0BJT3ZZ"  =  "1"  /* Inspection of Diaphragm, Percutaneous Approach */
"0BJT4ZZ"  =  "1"  /* Inspection of Diaphragm, Percutaneous Endoscopic Approach */
"0BJT7ZZ"  =  "1"  /* Inspection of Diaphragm, Via Natural or Artificial Opening */
"0BJT8ZZ"  =  "1"  /* Inspection of Diaphragm, Endo */
"0BJTXZZ"  =  "1"  /* Inspection of Diaphragm, External Approach */
"0BL10CZ"  =  "1"  /* Occlusion of Trachea with Extraluminal Device, Open Approach */
"0BL10DZ"  =  "1"  /* Occlusion of Trachea with Intraluminal Device, Open Approach */
"0BL10ZZ"  =  "1"  /* Occlusion of Trachea, Open Approach */
"0BL13CZ"  =  "1"  /* Occlusion of Trachea with Extraluminal Device, Perc Approach */
"0BL13DZ"  =  "1"  /* Occlusion of Trachea with Intraluminal Device, Perc Approach */
"0BL13ZZ"  =  "1"  /* Occlusion of Trachea, Percutaneous Approach */
"0BL14CZ"  =  "1"  /* Occlusion of Trachea with Extralum Dev, Perc Endo Approach */
"0BL14DZ"  =  "1"  /* Occlusion of Trachea with Intralum Dev, Perc Endo Approach */
"0BL14ZZ"  =  "1"  /* Occlusion of Trachea, Percutaneous Endoscopic Approach */
"0BL17DZ"  =  "1"  /* Occlusion of Trachea with Intraluminal Device, Via Opening */
"0BL17ZZ"  =  "1"  /* Occlusion of Trachea, Via Natural or Artificial Opening */
"0BL18DZ"  =  "1"  /* Occlusion of Trachea with Intraluminal Device, Endo */
"0BL18ZZ"  =  "1"  /* Occlusion of Trachea, Endo */
"0BL20CZ"  =  "1"  /* Occlusion of Carina with Extraluminal Device, Open Approach */
"0BL20DZ"  =  "1"  /* Occlusion of Carina with Intraluminal Device, Open Approach */
"0BL20ZZ"  =  "1"  /* Occlusion of Carina, Open Approach */
"0BL23CZ"  =  "1"  /* Occlusion of Carina with Extraluminal Device, Perc Approach */
"0BL23DZ"  =  "1"  /* Occlusion of Carina with Intraluminal Device, Perc Approach */
"0BL23ZZ"  =  "1"  /* Occlusion of Carina, Percutaneous Approach */
"0BL24CZ"  =  "1"  /* Occlusion of Carina with Extralum Dev, Perc Endo Approach */
"0BL24DZ"  =  "1"  /* Occlusion of Carina with Intralum Dev, Perc Endo Approach */
"0BL24ZZ"  =  "1"  /* Occlusion of Carina, Percutaneous Endoscopic Approach */
"0BL27DZ"  =  "1"  /* Occlusion of Carina with Intraluminal Device, Via Opening */
"0BL27ZZ"  =  "1"  /* Occlusion of Carina, Via Natural or Artificial Opening */
"0BL28DZ"  =  "1"  /* Occlusion of Carina with Intraluminal Device, Endo */
"0BL28ZZ"  =  "1"  /* Occlusion of Carina, Endo */
"0BL30CZ"  =  "1"  /* Occlusion of R Main Bronc with Extralum Dev, Open Approach */
"0BL30DZ"  =  "1"  /* Occlusion of R Main Bronc with Intralum Dev, Open Approach */
"0BL30ZZ"  =  "1"  /* Occlusion of Right Main Bronchus, Open Approach */
"0BL33CZ"  =  "1"  /* Occlusion of R Main Bronc with Extralum Dev, Perc Approach */
"0BL33DZ"  =  "1"  /* Occlusion of R Main Bronc with Intralum Dev, Perc Approach */
"0BL33ZZ"  =  "1"  /* Occlusion of Right Main Bronchus, Percutaneous Approach */
"0BL34CZ"  =  "1"  /* Occlusion R Main Bronc w Extralum Dev, Perc Endo */
"0BL34DZ"  =  "1"  /* Occlusion R Main Bronc w Intralum Dev, Perc Endo */
"0BL34ZZ"  =  "1"  /* Occlusion of Right Main Bronchus, Perc Endo Approach */
"0BL37DZ"  =  "1"  /* Occlusion of R Main Bronc with Intralum Dev, Via Opening */
"0BL37ZZ"  =  "1"  /* Occlusion of Right Main Bronchus, Via Opening */
"0BL38DZ"  =  "1"  /* Occlusion of Right Main Bronchus with Intralum Dev, Endo */
"0BL38ZZ"  =  "1"  /* Occlusion of Right Main Bronchus, Endo */
"0BL40CZ"  =  "1"  /* Occlusion R Up Lobe Bronc w Extralum Dev, Open */
"0BL40DZ"  =  "1"  /* Occlusion R Up Lobe Bronc w Intralum Dev, Open */
"0BL40ZZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus, Open Approach */
"0BL43CZ"  =  "1"  /* Occlusion R Up Lobe Bronc w Extralum Dev, Perc */
"0BL43DZ"  =  "1"  /* Occlusion R Up Lobe Bronc w Intralum Dev, Perc */
"0BL43ZZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus, Perc Approach */
"0BL44CZ"  =  "1"  /* Occlusion R Up Lobe Bronc w Extralum Dev, Perc Endo */
"0BL44DZ"  =  "1"  /* Occlusion R Up Lobe Bronc w Intralum Dev, Perc Endo */
"0BL44ZZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus, Perc Endo Approach */
"0BL47DZ"  =  "1"  /* Occlusion of R Up Lobe Bronc with Intralum Dev, Via Opening */
"0BL47ZZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus, Via Opening */
"0BL48DZ"  =  "1"  /* Occlusion of R Up Lobe Bronc with Intralum Dev, Endo */
"0BL48ZZ"  =  "1"  /* Occlusion of Right Upper Lobe Bronchus, Endo */
"0BL50CZ"  =  "1"  /* Occlusion R Mid Lobe Bronc w Extralum Dev, Open */
"0BL50DZ"  =  "1"  /* Occlusion R Mid Lobe Bronc w Intralum Dev, Open */
"0BL50ZZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus, Open Approach */
"0BL53CZ"  =  "1"  /* Occlusion R Mid Lobe Bronc w Extralum Dev, Perc */
"0BL53DZ"  =  "1"  /* Occlusion R Mid Lobe Bronc w Intralum Dev, Perc */
"0BL53ZZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus, Perc Approach */
"0BL54CZ"  =  "1"  /* Occlusion R Mid Lobe Bronc w Extralum Dev, Perc Endo */
"0BL54DZ"  =  "1"  /* Occlusion R Mid Lobe Bronc w Intralum Dev, Perc Endo */
"0BL54ZZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus, Perc Endo Approach */
"0BL57DZ"  =  "1"  /* Occlusion of R Mid Lobe Bronc with Intralum Dev, Via Opening */
"0BL57ZZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus, Via Opening */
"0BL58DZ"  =  "1"  /* Occlusion of R Mid Lobe Bronc with Intralum Dev, Endo */
"0BL58ZZ"  =  "1"  /* Occlusion of Right Middle Lobe Bronchus, Endo */
"0BL60CZ"  =  "1"  /* Occlusion R Low Lobe Bronc w Extralum Dev, Open */
"0BL60DZ"  =  "1"  /* Occlusion R Low Lobe Bronc w Intralum Dev, Open */
"0BL60ZZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus, Open Approach */
"0BL63CZ"  =  "1"  /* Occlusion R Low Lobe Bronc w Extralum Dev, Perc */
"0BL63DZ"  =  "1"  /* Occlusion R Low Lobe Bronc w Intralum Dev, Perc */
"0BL63ZZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus, Perc Approach */
"0BL64CZ"  =  "1"  /* Occlusion R Low Lobe Bronc w Extralum Dev, Perc Endo */
"0BL64DZ"  =  "1"  /* Occlusion R Low Lobe Bronc w Intralum Dev, Perc Endo */
"0BL64ZZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus, Perc Endo Approach */
"0BL67DZ"  =  "1"  /* Occlusion of R Low Lobe Bronc with Intralum Dev, Via Opening */
"0BL67ZZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus, Via Opening */
"0BL68DZ"  =  "1"  /* Occlusion of R Low Lobe Bronc with Intralum Dev, Endo */
"0BL68ZZ"  =  "1"  /* Occlusion of Right Lower Lobe Bronchus, Endo */
"0BL70CZ"  =  "1"  /* Occlusion of L Main Bronc with Extralum Dev, Open Approach */
"0BL70DZ"  =  "1"  /* Occlusion of L Main Bronc with Intralum Dev, Open Approach */
"0BL70ZZ"  =  "1"  /* Occlusion of Left Main Bronchus, Open Approach */
"0BL73CZ"  =  "1"  /* Occlusion of L Main Bronc with Extralum Dev, Perc Approach */
"0BL73DZ"  =  "1"  /* Occlusion of L Main Bronc with Intralum Dev, Perc Approach */
"0BL73ZZ"  =  "1"  /* Occlusion of Left Main Bronchus, Percutaneous Approach */
"0BL74CZ"  =  "1"  /* Occlusion L Main Bronc w Extralum Dev, Perc Endo */
"0BL74DZ"  =  "1"  /* Occlusion L Main Bronc w Intralum Dev, Perc Endo */
"0BL74ZZ"  =  "1"  /* Occlusion of Left Main Bronchus, Perc Endo Approach */
"0BL77DZ"  =  "1"  /* Occlusion of L Main Bronc with Intralum Dev, Via Opening */
"0BL77ZZ"  =  "1"  /* Occlusion of Left Main Bronchus, Via Opening */
"0BL78DZ"  =  "1"  /* Occlusion of Left Main Bronchus with Intralum Dev, Endo */
"0BL78ZZ"  =  "1"  /* Occlusion of Left Main Bronchus, Endo */
"0BL80CZ"  =  "1"  /* Occlusion L Up Lobe Bronc w Extralum Dev, Open */
"0BL80DZ"  =  "1"  /* Occlusion L Up Lobe Bronc w Intralum Dev, Open */
"0BL80ZZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus, Open Approach */
"0BL83CZ"  =  "1"  /* Occlusion L Up Lobe Bronc w Extralum Dev, Perc */
"0BL83DZ"  =  "1"  /* Occlusion L Up Lobe Bronc w Intralum Dev, Perc */
"0BL83ZZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus, Percutaneous Approach */
"0BL84CZ"  =  "1"  /* Occlusion L Up Lobe Bronc w Extralum Dev, Perc Endo */
"0BL84DZ"  =  "1"  /* Occlusion L Up Lobe Bronc w Intralum Dev, Perc Endo */
"0BL84ZZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus, Perc Endo Approach */
"0BL87DZ"  =  "1"  /* Occlusion of L Up Lobe Bronc with Intralum Dev, Via Opening */
"0BL87ZZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus, Via Opening */
"0BL88DZ"  =  "1"  /* Occlusion of L Up Lobe Bronc with Intralum Dev, Endo */
"0BL88ZZ"  =  "1"  /* Occlusion of Left Upper Lobe Bronchus, Endo */
"0BL90CZ"  =  "1"  /* Occlusion of Lingula Bronc with Extralum Dev, Open Approach */
"0BL90DZ"  =  "1"  /* Occlusion of Lingula Bronc with Intralum Dev, Open Approach */
"0BL90ZZ"  =  "1"  /* Occlusion of Lingula Bronchus, Open Approach */
"0BL93CZ"  =  "1"  /* Occlusion of Lingula Bronc with Extralum Dev, Perc Approach */
"0BL93DZ"  =  "1"  /* Occlusion of Lingula Bronc with Intralum Dev, Perc Approach */
"0BL93ZZ"  =  "1"  /* Occlusion of Lingula Bronchus, Percutaneous Approach */
"0BL94CZ"  =  "1"  /* Occlusion Lingula Bronc w Extralum Dev, Perc Endo */
"0BL94DZ"  =  "1"  /* Occlusion Lingula Bronc w Intralum Dev, Perc Endo */
"0BL94ZZ"  =  "1"  /* Occlusion of Lingula Bronchus, Perc Endo Approach */
"0BL97DZ"  =  "1"  /* Occlusion of Lingula Bronchus with Intralum Dev, Via Opening */
"0BL97ZZ"  =  "1"  /* Occlusion of Lingula Bronchus, Via Opening */
"0BL98DZ"  =  "1"  /* Occlusion of Lingula Bronchus with Intraluminal Device, Endo */
"0BL98ZZ"  =  "1"  /* Occlusion of Lingula Bronchus, Endo */
"0BLB0CZ"  =  "1"  /* Occlusion L Low Lobe Bronc w Extralum Dev, Open */
"0BLB0DZ"  =  "1"  /* Occlusion L Low Lobe Bronc w Intralum Dev, Open */
"0BLB0ZZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus, Open Approach */
"0BLB3CZ"  =  "1"  /* Occlusion L Low Lobe Bronc w Extralum Dev, Perc */
"0BLB3DZ"  =  "1"  /* Occlusion L Low Lobe Bronc w Intralum Dev, Perc */
"0BLB3ZZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus, Percutaneous Approach */
"0BLB4CZ"  =  "1"  /* Occlusion L Low Lobe Bronc w Extralum Dev, Perc Endo */
"0BLB4DZ"  =  "1"  /* Occlusion L Low Lobe Bronc w Intralum Dev, Perc Endo */
"0BLB4ZZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus, Perc Endo Approach */
"0BLB7DZ"  =  "1"  /* Occlusion of L Low Lobe Bronc with Intralum Dev, Via Opening */
"0BLB7ZZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus, Via Opening */
"0BLB8DZ"  =  "1"  /* Occlusion of L Low Lobe Bronc with Intralum Dev, Endo */
"0BLB8ZZ"  =  "1"  /* Occlusion of Left Lower Lobe Bronchus, Endo */
"0BM10ZZ"  =  "1"  /* Reattachment of Trachea, Open Approach */
"0BM20ZZ"  =  "1"  /* Reattachment of Carina, Open Approach */
"0BM30ZZ"  =  "1"  /* Reattachment of Right Main Bronchus, Open Approach */
"0BM40ZZ"  =  "1"  /* Reattachment of Right Upper Lobe Bronchus, Open Approach */
"0BM50ZZ"  =  "1"  /* Reattachment of Right Middle Lobe Bronchus, Open Approach */
"0BM60ZZ"  =  "1"  /* Reattachment of Right Lower Lobe Bronchus, Open Approach */
"0BM70ZZ"  =  "1"  /* Reattachment of Left Main Bronchus, Open Approach */
"0BM80ZZ"  =  "1"  /* Reattachment of Left Upper Lobe Bronchus, Open Approach */
"0BM90ZZ"  =  "1"  /* Reattachment of Lingula Bronchus, Open Approach */
"0BMB0ZZ"  =  "1"  /* Reattachment of Left Lower Lobe Bronchus, Open Approach */
"0BMC0ZZ"  =  "1"  /* Reattachment of Right Upper Lung Lobe, Open Approach */
"0BMD0ZZ"  =  "1"  /* Reattachment of Right Middle Lung Lobe, Open Approach */
"0BMF0ZZ"  =  "1"  /* Reattachment of Right Lower Lung Lobe, Open Approach */
"0BMG0ZZ"  =  "1"  /* Reattachment of Left Upper Lung Lobe, Open Approach */
"0BMH0ZZ"  =  "1"  /* Reattachment of Lung Lingula, Open Approach */
"0BMJ0ZZ"  =  "1"  /* Reattachment of Left Lower Lung Lobe, Open Approach */
"0BMK0ZZ"  =  "1"  /* Reattachment of Right Lung, Open Approach */
"0BML0ZZ"  =  "1"  /* Reattachment of Left Lung, Open Approach */
"0BMR0ZZ"  =  "1"  /* Reattachment of Right Diaphragm, Open Approach */
"0BMS0ZZ"  =  "1"  /* Reattachment of Left Diaphragm, Open Approach */
"0BN10ZZ"  =  "1"  /* Release Trachea, Open Approach */
"0BN13ZZ"  =  "1"  /* Release Trachea, Percutaneous Approach */
"0BN14ZZ"  =  "1"  /* Release Trachea, Percutaneous Endoscopic Approach */
"0BN17ZZ"  =  "1"  /* Release Trachea, Via Natural or Artificial Opening */
"0BN18ZZ"  =  "1"  /* Release Trachea, Endo */
"0BN20ZZ"  =  "1"  /* Release Carina, Open Approach */
"0BN23ZZ"  =  "1"  /* Release Carina, Percutaneous Approach */
"0BN24ZZ"  =  "1"  /* Release Carina, Percutaneous Endoscopic Approach */
"0BN27ZZ"  =  "1"  /* Release Carina, Via Natural or Artificial Opening */
"0BN28ZZ"  =  "1"  /* Release Carina, Via Natural or Artificial Opening Endoscopic */
"0BN30ZZ"  =  "1"  /* Release Right Main Bronchus, Open Approach */
"0BN33ZZ"  =  "1"  /* Release Right Main Bronchus, Percutaneous Approach */
"0BN34ZZ"  =  "1"  /* Release Right Main Bronchus, Perc Endo Approach */
"0BN37ZZ"  =  "1"  /* Release Right Main Bronchus, Via Opening */
"0BN38ZZ"  =  "1"  /* Release Right Main Bronchus, Endo */
"0BN40ZZ"  =  "1"  /* Release Right Upper Lobe Bronchus, Open Approach */
"0BN43ZZ"  =  "1"  /* Release Right Upper Lobe Bronchus, Percutaneous Approach */
"0BN44ZZ"  =  "1"  /* Release Right Upper Lobe Bronchus, Perc Endo Approach */
"0BN47ZZ"  =  "1"  /* Release Right Upper Lobe Bronchus, Via Opening */
"0BN48ZZ"  =  "1"  /* Release Right Upper Lobe Bronchus, Endo */
"0BN50ZZ"  =  "1"  /* Release Right Middle Lobe Bronchus, Open Approach */
"0BN53ZZ"  =  "1"  /* Release Right Middle Lobe Bronchus, Percutaneous Approach */
"0BN54ZZ"  =  "1"  /* Release Right Middle Lobe Bronchus, Perc Endo Approach */
"0BN57ZZ"  =  "1"  /* Release Right Middle Lobe Bronchus, Via Opening */
"0BN58ZZ"  =  "1"  /* Release Right Middle Lobe Bronchus, Endo */
"0BN60ZZ"  =  "1"  /* Release Right Lower Lobe Bronchus, Open Approach */
"0BN63ZZ"  =  "1"  /* Release Right Lower Lobe Bronchus, Percutaneous Approach */
"0BN64ZZ"  =  "1"  /* Release Right Lower Lobe Bronchus, Perc Endo Approach */
"0BN67ZZ"  =  "1"  /* Release Right Lower Lobe Bronchus, Via Opening */
"0BN68ZZ"  =  "1"  /* Release Right Lower Lobe Bronchus, Endo */
"0BN70ZZ"  =  "1"  /* Release Left Main Bronchus, Open Approach */
"0BN73ZZ"  =  "1"  /* Release Left Main Bronchus, Percutaneous Approach */
"0BN74ZZ"  =  "1"  /* Release Left Main Bronchus, Percutaneous Endoscopic Approach */
"0BN77ZZ"  =  "1"  /* Release Left Main Bronchus, Via Opening */
"0BN78ZZ"  =  "1"  /* Release Left Main Bronchus, Endo */
"0BN80ZZ"  =  "1"  /* Release Left Upper Lobe Bronchus, Open Approach */
"0BN83ZZ"  =  "1"  /* Release Left Upper Lobe Bronchus, Percutaneous Approach */
"0BN84ZZ"  =  "1"  /* Release Left Upper Lobe Bronchus, Perc Endo Approach */
"0BN87ZZ"  =  "1"  /* Release Left Upper Lobe Bronchus, Via Opening */
"0BN88ZZ"  =  "1"  /* Release Left Upper Lobe Bronchus, Endo */
"0BN90ZZ"  =  "1"  /* Release Lingula Bronchus, Open Approach */
"0BN93ZZ"  =  "1"  /* Release Lingula Bronchus, Percutaneous Approach */
"0BN94ZZ"  =  "1"  /* Release Lingula Bronchus, Percutaneous Endoscopic Approach */
"0BN97ZZ"  =  "1"  /* Release Lingula Bronchus, Via Natural or Artificial Opening */
"0BN98ZZ"  =  "1"  /* Release Lingula Bronchus, Endo */
"0BNB0ZZ"  =  "1"  /* Release Left Lower Lobe Bronchus, Open Approach */
"0BNB3ZZ"  =  "1"  /* Release Left Lower Lobe Bronchus, Percutaneous Approach */
"0BNB4ZZ"  =  "1"  /* Release Left Lower Lobe Bronchus, Perc Endo Approach */
"0BNB7ZZ"  =  "1"  /* Release Left Lower Lobe Bronchus, Via Opening */
"0BNB8ZZ"  =  "1"  /* Release Left Lower Lobe Bronchus, Endo */
"0BNC0ZZ"  =  "1"  /* Release Right Upper Lung Lobe, Open Approach */
"0BNC3ZZ"  =  "1"  /* Release Right Upper Lung Lobe, Percutaneous Approach */
"0BNC4ZZ"  =  "1"  /* Release Right Upper Lung Lobe, Perc Endo Approach */
"0BNC7ZZ"  =  "1"  /* Release Right Upper Lung Lobe, Via Opening */
"0BNC8ZZ"  =  "1"  /* Release Right Upper Lung Lobe, Endo */
"0BND0ZZ"  =  "1"  /* Release Right Middle Lung Lobe, Open Approach */
"0BND3ZZ"  =  "1"  /* Release Right Middle Lung Lobe, Percutaneous Approach */
"0BND4ZZ"  =  "1"  /* Release Right Middle Lung Lobe, Perc Endo Approach */
"0BND7ZZ"  =  "1"  /* Release Right Middle Lung Lobe, Via Opening */
"0BND8ZZ"  =  "1"  /* Release Right Middle Lung Lobe, Endo */
"0BNF0ZZ"  =  "1"  /* Release Right Lower Lung Lobe, Open Approach */
"0BNF3ZZ"  =  "1"  /* Release Right Lower Lung Lobe, Percutaneous Approach */
"0BNF4ZZ"  =  "1"  /* Release Right Lower Lung Lobe, Perc Endo Approach */
"0BNF7ZZ"  =  "1"  /* Release Right Lower Lung Lobe, Via Opening */
"0BNF8ZZ"  =  "1"  /* Release Right Lower Lung Lobe, Endo */
"0BNG0ZZ"  =  "1"  /* Release Left Upper Lung Lobe, Open Approach */
"0BNG3ZZ"  =  "1"  /* Release Left Upper Lung Lobe, Percutaneous Approach */
"0BNG4ZZ"  =  "1"  /* Release Left Upper Lung Lobe, Perc Endo Approach */
"0BNG7ZZ"  =  "1"  /* Release Left Upper Lung Lobe, Via Opening */
"0BNG8ZZ"  =  "1"  /* Release Left Upper Lung Lobe, Endo */
"0BNH0ZZ"  =  "1"  /* Release Lung Lingula, Open Approach */
"0BNH3ZZ"  =  "1"  /* Release Lung Lingula, Percutaneous Approach */
"0BNH4ZZ"  =  "1"  /* Release Lung Lingula, Percutaneous Endoscopic Approach */
"0BNH7ZZ"  =  "1"  /* Release Lung Lingula, Via Natural or Artificial Opening */
"0BNH8ZZ"  =  "1"  /* Release Lung Lingula, Endo */
"0BNJ0ZZ"  =  "1"  /* Release Left Lower Lung Lobe, Open Approach */
"0BNJ3ZZ"  =  "1"  /* Release Left Lower Lung Lobe, Percutaneous Approach */
"0BNJ4ZZ"  =  "1"  /* Release Left Lower Lung Lobe, Perc Endo Approach */
"0BNJ7ZZ"  =  "1"  /* Release Left Lower Lung Lobe, Via Opening */
"0BNJ8ZZ"  =  "1"  /* Release Left Lower Lung Lobe, Endo */
"0BNK0ZZ"  =  "1"  /* Release Right Lung, Open Approach */
"0BNK3ZZ"  =  "1"  /* Release Right Lung, Percutaneous Approach */
"0BNK4ZZ"  =  "1"  /* Release Right Lung, Percutaneous Endoscopic Approach */
"0BNK7ZZ"  =  "1"  /* Release Right Lung, Via Natural or Artificial Opening */
"0BNK8ZZ"  =  "1"  /* Release Right Lung, Endo */
"0BNL0ZZ"  =  "1"  /* Release Left Lung, Open Approach */
"0BNL3ZZ"  =  "1"  /* Release Left Lung, Percutaneous Approach */
"0BNL4ZZ"  =  "1"  /* Release Left Lung, Percutaneous Endoscopic Approach */
"0BNL7ZZ"  =  "1"  /* Release Left Lung, Via Natural or Artificial Opening */
"0BNL8ZZ"  =  "1"  /* Release Left Lung, Endo */
"0BNM0ZZ"  =  "1"  /* Release Bilateral Lungs, Open Approach */
"0BNM3ZZ"  =  "1"  /* Release Bilateral Lungs, Percutaneous Approach */
"0BNM4ZZ"  =  "1"  /* Release Bilateral Lungs, Percutaneous Endoscopic Approach */
"0BNM7ZZ"  =  "1"  /* Release Bilateral Lungs, Via Natural or Artificial Opening */
"0BNM8ZZ"  =  "1"  /* Release Bilateral Lungs, Endo */
"0BNN0ZZ"  =  "1"  /* Release Right Pleura, Open Approach */
"0BNN3ZZ"  =  "1"  /* Release Right Pleura, Percutaneous Approach */
"0BNN4ZZ"  =  "1"  /* Release Right Pleura, Percutaneous Endoscopic Approach */
"0BNP0ZZ"  =  "1"  /* Release Left Pleura, Open Approach */
"0BNP3ZZ"  =  "1"  /* Release Left Pleura, Percutaneous Approach */
"0BNP4ZZ"  =  "1"  /* Release Left Pleura, Percutaneous Endoscopic Approach */
"0BNR0ZZ"  =  "1"  /* Release Right Diaphragm, Open Approach */
"0BNR3ZZ"  =  "1"  /* Release Right Diaphragm, Percutaneous Approach */
"0BNR4ZZ"  =  "1"  /* Release Right Diaphragm, Percutaneous Endoscopic Approach */
"0BNS0ZZ"  =  "1"  /* Release Left Diaphragm, Open Approach */
"0BNS3ZZ"  =  "1"  /* Release Left Diaphragm, Percutaneous Approach */
"0BNS4ZZ"  =  "1"  /* Release Left Diaphragm, Percutaneous Endoscopic Approach */
"0BP000Z"  =  "1"  /* Removal of Drain Dev from Tracheobronc Tree, Open Approach */
"0BP001Z"  =  "1"  /* Remove Radioact Elem from Tracheobronc Tree, Open */
"0BP002Z"  =  "1"  /* Removal of Monitor Dev from Tracheobronc Tree, Open Approach */
"0BP003Z"  =  "1"  /* Remove of Infusion Dev from Tracheobronc Tree, Open Approach */
"0BP007Z"  =  "1"  /* Removal of Autol Sub from Tracheobronc Tree, Open Approach */
"0BP00CZ"  =  "1"  /* Remove of Extralum Dev from Tracheobronc Tree, Open Approach */
"0BP00DZ"  =  "1"  /* Remove of Intralum Dev from Tracheobronc Tree, Open Approach */
"0BP00JZ"  =  "1"  /* Removal of Synth Sub from Tracheobronc Tree, Open Approach */
"0BP00KZ"  =  "1"  /* Removal of Nonaut Sub from Tracheobronc Tree, Open Approach */
"0BP030Z"  =  "1"  /* Removal of Drain Dev from Tracheobronc Tree, Perc Approach */
"0BP031Z"  =  "1"  /* Remove Radioact Elem from Tracheobronc Tree, Perc */
"0BP032Z"  =  "1"  /* Removal of Monitor Dev from Tracheobronc Tree, Perc Approach */
"0BP033Z"  =  "1"  /* Remove of Infusion Dev from Tracheobronc Tree, Perc Approach */
"0BP037Z"  =  "1"  /* Removal of Autol Sub from Tracheobronc Tree, Perc Approach */
"0BP03CZ"  =  "1"  /* Remove of Extralum Dev from Tracheobronc Tree, Perc Approach */
"0BP03DZ"  =  "1"  /* Remove of Intralum Dev from Tracheobronc Tree, Perc Approach */
"0BP03JZ"  =  "1"  /* Removal of Synth Sub from Tracheobronc Tree, Perc Approach */
"0BP03KZ"  =  "1"  /* Removal of Nonaut Sub from Tracheobronc Tree, Perc Approach */
"0BP040Z"  =  "1"  /* Remove Drain Dev from Tracheobronc Tree, Perc Endo */
"0BP041Z"  =  "1"  /* Remove Radioact Elem from Tracheobronc Tree, Perc Endo */
"0BP042Z"  =  "1"  /* Remove Monitor Dev from Tracheobronc Tree, Perc Endo */
"0BP043Z"  =  "1"  /* Remove Infusion Dev from Tracheobronc Tree, Perc Endo */
"0BP047Z"  =  "1"  /* Remove Autol Sub from Tracheobronc Tree, Perc Endo */
"0BP04CZ"  =  "1"  /* Remove Extralum Dev from Tracheobronc Tree, Perc Endo */
"0BP04DZ"  =  "1"  /* Remove Intralum Dev from Tracheobronc Tree, Perc Endo */
"0BP04JZ"  =  "1"  /* Remove Synth Sub from Tracheobronc Tree, Perc Endo */
"0BP04KZ"  =  "1"  /* Remove Nonaut Sub from Tracheobronc Tree, Perc Endo */
"0BP070Z"  =  "1"  /* Removal of Drain Dev from Tracheobronc Tree, Via Opening */
"0BP071Z"  =  "1"  /* Removal of Radioact Elem from Tracheobronc Tree, Via Opening */
"0BP072Z"  =  "1"  /* Removal of Monitor Dev from Tracheobronc Tree, Via Opening */
"0BP073Z"  =  "1"  /* Removal of Infusion Dev from Tracheobronc Tree, Via Opening */
"0BP077Z"  =  "1"  /* Removal of Autol Sub from Tracheobronc Tree, Via Opening */
"0BP07CZ"  =  "1"  /* Removal of Extralum Dev from Tracheobronc Tree, Via Opening */
"0BP07DZ"  =  "1"  /* Removal of Intralum Dev from Tracheobronc Tree, Via Opening */
"0BP07JZ"  =  "1"  /* Removal of Synth Sub from Tracheobronc Tree, Via Opening */
"0BP07KZ"  =  "1"  /* Removal of Nonaut Sub from Tracheobronc Tree, Via Opening */
"0BP080Z"  =  "1"  /* Removal of Drainage Device from Tracheobronchial Tree, Endo */
"0BP081Z"  =  "1"  /* Removal of Radioactive Element from Tracheobronc Tree, Endo */
"0BP082Z"  =  "1"  /* Removal of Monitoring Device from Tracheobronc Tree, Endo */
"0BP083Z"  =  "1"  /* Removal of Infusion Device from Tracheobronchial Tree, Endo */
"0BP087Z"  =  "1"  /* Removal of Autol Sub from Tracheobronc Tree, Endo */
"0BP08CZ"  =  "1"  /* Removal of Extraluminal Device from Tracheobronc Tree, Endo */
"0BP08DZ"  =  "1"  /* Removal of Intraluminal Device from Tracheobronc Tree, Endo */
"0BP08JZ"  =  "1"  /* Removal of Synthetic Substitute from Tracheobronc Tree, Endo */
"0BP08KZ"  =  "1"  /* Removal of Nonaut Sub from Tracheobronc Tree, Endo */
"0BP100Z"  =  "1"  /* Removal of Drainage Device from Trachea, Open Approach */
"0BP102Z"  =  "1"  /* Removal of Monitoring Device from Trachea, Open Approach */
"0BP107Z"  =  "1"  /* Removal of Autol Sub from Trachea, Open Approach */
"0BP10CZ"  =  "1"  /* Removal of Extraluminal Device from Trachea, Open Approach */
"0BP10DZ"  =  "1"  /* Removal of Intraluminal Device from Trachea, Open Approach */
"0BP10FZ"  =  "1"  /* Removal of Tracheostomy Device from Trachea, Open Approach */
"0BP10JZ"  =  "1"  /* Removal of Synthetic Substitute from Trachea, Open Approach */
"0BP10KZ"  =  "1"  /* Removal of Nonaut Sub from Trachea, Open Approach */
"0BP130Z"  =  "1"  /* Removal of Drainage Device from Trachea, Perc Approach */
"0BP132Z"  =  "1"  /* Removal of Monitoring Device from Trachea, Perc Approach */
"0BP137Z"  =  "1"  /* Removal of Autol Sub from Trachea, Perc Approach */
"0BP13CZ"  =  "1"  /* Removal of Extraluminal Device from Trachea, Perc Approach */
"0BP13DZ"  =  "1"  /* Removal of Intraluminal Device from Trachea, Perc Approach */
"0BP13FZ"  =  "1"  /* Removal of Tracheostomy Device from Trachea, Perc Approach */
"0BP13JZ"  =  "1"  /* Removal of Synthetic Substitute from Trachea, Perc Approach */
"0BP13KZ"  =  "1"  /* Removal of Nonaut Sub from Trachea, Perc Approach */
"0BP140Z"  =  "1"  /* Removal of Drainage Device from Trachea, Perc Endo Approach */
"0BP142Z"  =  "1"  /* Removal of Monitor Dev from Trachea, Perc Endo Approach */
"0BP147Z"  =  "1"  /* Removal of Autol Sub from Trachea, Perc Endo Approach */
"0BP14CZ"  =  "1"  /* Removal of Extralum Dev from Trachea, Perc Endo Approach */
"0BP14DZ"  =  "1"  /* Removal of Intralum Dev from Trachea, Perc Endo Approach */
"0BP14FZ"  =  "1"  /* Removal of Trach Dev from Trachea, Perc Endo Approach */
"0BP14JZ"  =  "1"  /* Removal of Synth Sub from Trachea, Perc Endo Approach */
"0BP14KZ"  =  "1"  /* Removal of Nonaut Sub from Trachea, Perc Endo Approach */
"0BP170Z"  =  "1"  /* Removal of Drainage Device from Trachea, Via Opening */
"0BP172Z"  =  "1"  /* Removal of Monitoring Device from Trachea, Via Opening */
"0BP177Z"  =  "1"  /* Removal of Autol Sub from Trachea, Via Opening */
"0BP17CZ"  =  "1"  /* Removal of Extraluminal Device from Trachea, Via Opening */
"0BP17DZ"  =  "1"  /* Removal of Intraluminal Device from Trachea, Via Opening */
"0BP17FZ"  =  "1"  /* Removal of Tracheostomy Device from Trachea, Via Opening */
"0BP17JZ"  =  "1"  /* Removal of Synthetic Substitute from Trachea, Via Opening */
"0BP17KZ"  =  "1"  /* Removal of Nonaut Sub from Trachea, Via Opening */
"0BP180Z"  =  "1"  /* Removal of Drainage Device from Trachea, Endo */
"0BP182Z"  =  "1"  /* Removal of Monitoring Device from Trachea, Endo */
"0BP187Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Trachea, Endo */
"0BP18CZ"  =  "1"  /* Removal of Extraluminal Device from Trachea, Endo */
"0BP18DZ"  =  "1"  /* Removal of Intraluminal Device from Trachea, Endo */
"0BP18FZ"  =  "1"  /* Removal of Tracheostomy Device from Trachea, Endo */
"0BP18JZ"  =  "1"  /* Removal of Synthetic Substitute from Trachea, Endo */
"0BP18KZ"  =  "1"  /* Removal of Nonaut Sub from Trachea, Endo */
"0BPK00Z"  =  "1"  /* Removal of Drainage Device from Right Lung, Open Approach */
"0BPK01Z"  =  "1"  /* Removal of Radioactive Element from R Lung, Open Approach */
"0BPK02Z"  =  "1"  /* Removal of Monitoring Device from Right Lung, Open Approach */
"0BPK03Z"  =  "1"  /* Removal of Infusion Device from Right Lung, Open Approach */
"0BPK30Z"  =  "1"  /* Removal of Drainage Device from Right Lung, Perc Approach */
"0BPK31Z"  =  "1"  /* Removal of Radioactive Element from R Lung, Perc Approach */
"0BPK32Z"  =  "1"  /* Removal of Monitoring Device from Right Lung, Perc Approach */
"0BPK33Z"  =  "1"  /* Removal of Infusion Device from Right Lung, Perc Approach */
"0BPK40Z"  =  "1"  /* Removal of Drainage Device from R Lung, Perc Endo Approach */
"0BPK41Z"  =  "1"  /* Removal of Radioact Elem from R Lung, Perc Endo Approach */
"0BPK42Z"  =  "1"  /* Removal of Monitoring Device from R Lung, Perc Endo Approach */
"0BPK43Z"  =  "1"  /* Removal of Infusion Device from R Lung, Perc Endo Approach */
"0BPK70Z"  =  "1"  /* Removal of Drainage Device from Right Lung, Via Opening */
"0BPK71Z"  =  "1"  /* Removal of Radioactive Element from Right Lung, Via Opening */
"0BPK72Z"  =  "1"  /* Removal of Monitoring Device from Right Lung, Via Opening */
"0BPK73Z"  =  "1"  /* Removal of Infusion Device from Right Lung, Via Opening */
"0BPK80Z"  =  "1"  /* Removal of Drainage Device from Right Lung, Endo */
"0BPK81Z"  =  "1"  /* Removal of Radioactive Element from Right Lung, Endo */
"0BPK82Z"  =  "1"  /* Removal of Monitoring Device from Right Lung, Endo */
"0BPK83Z"  =  "1"  /* Removal of Infusion Device from Right Lung, Endo */
"0BPL00Z"  =  "1"  /* Removal of Drainage Device from Left Lung, Open Approach */
"0BPL01Z"  =  "1"  /* Removal of Radioactive Element from Left Lung, Open Approach */
"0BPL02Z"  =  "1"  /* Removal of Monitoring Device from Left Lung, Open Approach */
"0BPL03Z"  =  "1"  /* Removal of Infusion Device from Left Lung, Open Approach */
"0BPL30Z"  =  "1"  /* Removal of Drainage Device from Left Lung, Perc Approach */
"0BPL31Z"  =  "1"  /* Removal of Radioactive Element from Left Lung, Perc Approach */
"0BPL32Z"  =  "1"  /* Removal of Monitoring Device from Left Lung, Perc Approach */
"0BPL33Z"  =  "1"  /* Removal of Infusion Device from Left Lung, Perc Approach */
"0BPL40Z"  =  "1"  /* Removal of Drainage Device from L Lung, Perc Endo Approach */
"0BPL41Z"  =  "1"  /* Removal of Radioact Elem from L Lung, Perc Endo Approach */
"0BPL42Z"  =  "1"  /* Removal of Monitoring Device from L Lung, Perc Endo Approach */
"0BPL43Z"  =  "1"  /* Removal of Infusion Device from L Lung, Perc Endo Approach */
"0BPL70Z"  =  "1"  /* Removal of Drainage Device from Left Lung, Via Opening */
"0BPL71Z"  =  "1"  /* Removal of Radioactive Element from Left Lung, Via Opening */
"0BPL72Z"  =  "1"  /* Removal of Monitoring Device from Left Lung, Via Opening */
"0BPL73Z"  =  "1"  /* Removal of Infusion Device from Left Lung, Via Opening */
"0BPL80Z"  =  "1"  /* Removal of Drainage Device from Left Lung, Endo */
"0BPL81Z"  =  "1"  /* Removal of Radioactive Element from Left Lung, Endo */
"0BPL82Z"  =  "1"  /* Removal of Monitoring Device from Left Lung, Endo */
"0BPL83Z"  =  "1"  /* Removal of Infusion Device from Left Lung, Endo */
"0BPT00Z"  =  "1"  /* Removal of Drainage Device from Diaphragm, Open Approach */
"0BPT02Z"  =  "1"  /* Removal of Monitoring Device from Diaphragm, Open Approach */
"0BPT07Z"  =  "1"  /* Removal of Autol Sub from Diaphragm, Open Approach */
"0BPT0JZ"  =  "1"  /* Removal of Synth Sub from Diaphragm, Open Approach */
"0BPT0KZ"  =  "1"  /* Removal of Nonaut Sub from Diaphragm, Open Approach */
"0BPT0MZ"  =  "1"  /* Removal of Diaphragm Lead from Diaphragm, Open Approach */
"0BPT30Z"  =  "1"  /* Removal of Drainage Device from Diaphragm, Perc Approach */
"0BPT32Z"  =  "1"  /* Removal of Monitoring Device from Diaphragm, Perc Approach */
"0BPT37Z"  =  "1"  /* Removal of Autol Sub from Diaphragm, Perc Approach */
"0BPT3JZ"  =  "1"  /* Removal of Synth Sub from Diaphragm, Perc Approach */
"0BPT3KZ"  =  "1"  /* Removal of Nonaut Sub from Diaphragm, Perc Approach */
"0BPT3MZ"  =  "1"  /* Removal of Diaphragm Lead from Diaphragm, Perc Approach */
"0BPT40Z"  =  "1"  /* Removal of Drain Dev from Diaphragm, Perc Endo Approach */
"0BPT42Z"  =  "1"  /* Removal of Monitor Dev from Diaphragm, Perc Endo Approach */
"0BPT47Z"  =  "1"  /* Removal of Autol Sub from Diaphragm, Perc Endo Approach */
"0BPT4JZ"  =  "1"  /* Removal of Synth Sub from Diaphragm, Perc Endo Approach */
"0BPT4KZ"  =  "1"  /* Removal of Nonaut Sub from Diaphragm, Perc Endo Approach */
"0BPT4MZ"  =  "1"  /* Removal of Diaphragm Lead from Diaphragm, Perc Endo Approach */
"0BPT70Z"  =  "1"  /* Removal of Drainage Device from Diaphragm, Via Opening */
"0BPT72Z"  =  "1"  /* Removal of Monitoring Device from Diaphragm, Via Opening */
"0BPT77Z"  =  "1"  /* Removal of Autol Sub from Diaphragm, Via Opening */
"0BPT7JZ"  =  "1"  /* Removal of Synthetic Substitute from Diaphragm, Via Opening */
"0BPT7KZ"  =  "1"  /* Removal of Nonaut Sub from Diaphragm, Via Opening */
"0BPT7MZ"  =  "1"  /* Removal of Diaphragm Lead from Diaphragm, Via Opening */
"0BPT80Z"  =  "1"  /* Removal of Drainage Device from Diaphragm, Endo */
"0BPT82Z"  =  "1"  /* Removal of Monitoring Device from Diaphragm, Endo */
"0BPT87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Diaphragm, Endo */
"0BPT8JZ"  =  "1"  /* Removal of Synthetic Substitute from Diaphragm, Endo */
"0BPT8KZ"  =  "1"  /* Removal of Nonaut Sub from Diaphragm, Endo */
"0BPT8MZ"  =  "1"  /* Removal of Diaphragmatic Pacemaker Lead from Diaphragm, Endo */
"0BQ10ZZ"  =  "1"  /* Repair Trachea, Open Approach */
"0BQ13ZZ"  =  "1"  /* Repair Trachea, Percutaneous Approach */
"0BQ14ZZ"  =  "1"  /* Repair Trachea, Percutaneous Endoscopic Approach */
"0BQ17ZZ"  =  "1"  /* Repair Trachea, Via Natural or Artificial Opening */
"0BQ18ZZ"  =  "1"  /* Repair Trachea, Via Natural or Artificial Opening Endoscopic */
"0BQ20ZZ"  =  "1"  /* Repair Carina, Open Approach */
"0BQ23ZZ"  =  "1"  /* Repair Carina, Percutaneous Approach */
"0BQ24ZZ"  =  "1"  /* Repair Carina, Percutaneous Endoscopic Approach */
"0BQ27ZZ"  =  "1"  /* Repair Carina, Via Natural or Artificial Opening */
"0BQ28ZZ"  =  "1"  /* Repair Carina, Via Natural or Artificial Opening Endoscopic */
"0BQ30ZZ"  =  "1"  /* Repair Right Main Bronchus, Open Approach */
"0BQ33ZZ"  =  "1"  /* Repair Right Main Bronchus, Percutaneous Approach */
"0BQ34ZZ"  =  "1"  /* Repair Right Main Bronchus, Percutaneous Endoscopic Approach */
"0BQ37ZZ"  =  "1"  /* Repair Right Main Bronchus, Via Opening */
"0BQ38ZZ"  =  "1"  /* Repair Right Main Bronchus, Endo */
"0BQ40ZZ"  =  "1"  /* Repair Right Upper Lobe Bronchus, Open Approach */
"0BQ43ZZ"  =  "1"  /* Repair Right Upper Lobe Bronchus, Percutaneous Approach */
"0BQ44ZZ"  =  "1"  /* Repair Right Upper Lobe Bronchus, Perc Endo Approach */
"0BQ47ZZ"  =  "1"  /* Repair Right Upper Lobe Bronchus, Via Opening */
"0BQ48ZZ"  =  "1"  /* Repair Right Upper Lobe Bronchus, Endo */
"0BQ50ZZ"  =  "1"  /* Repair Right Middle Lobe Bronchus, Open Approach */
"0BQ53ZZ"  =  "1"  /* Repair Right Middle Lobe Bronchus, Percutaneous Approach */
"0BQ54ZZ"  =  "1"  /* Repair Right Middle Lobe Bronchus, Perc Endo Approach */
"0BQ57ZZ"  =  "1"  /* Repair Right Middle Lobe Bronchus, Via Opening */
"0BQ58ZZ"  =  "1"  /* Repair Right Middle Lobe Bronchus, Endo */
"0BQ60ZZ"  =  "1"  /* Repair Right Lower Lobe Bronchus, Open Approach */
"0BQ63ZZ"  =  "1"  /* Repair Right Lower Lobe Bronchus, Percutaneous Approach */
"0BQ64ZZ"  =  "1"  /* Repair Right Lower Lobe Bronchus, Perc Endo Approach */
"0BQ67ZZ"  =  "1"  /* Repair Right Lower Lobe Bronchus, Via Opening */
"0BQ68ZZ"  =  "1"  /* Repair Right Lower Lobe Bronchus, Endo */
"0BQ70ZZ"  =  "1"  /* Repair Left Main Bronchus, Open Approach */
"0BQ73ZZ"  =  "1"  /* Repair Left Main Bronchus, Percutaneous Approach */
"0BQ74ZZ"  =  "1"  /* Repair Left Main Bronchus, Percutaneous Endoscopic Approach */
"0BQ77ZZ"  =  "1"  /* Repair Left Main Bronchus, Via Natural or Artificial Opening */
"0BQ78ZZ"  =  "1"  /* Repair Left Main Bronchus, Endo */
"0BQ80ZZ"  =  "1"  /* Repair Left Upper Lobe Bronchus, Open Approach */
"0BQ83ZZ"  =  "1"  /* Repair Left Upper Lobe Bronchus, Percutaneous Approach */
"0BQ84ZZ"  =  "1"  /* Repair Left Upper Lobe Bronchus, Perc Endo Approach */
"0BQ87ZZ"  =  "1"  /* Repair Left Upper Lobe Bronchus, Via Opening */
"0BQ88ZZ"  =  "1"  /* Repair Left Upper Lobe Bronchus, Endo */
"0BQ90ZZ"  =  "1"  /* Repair Lingula Bronchus, Open Approach */
"0BQ93ZZ"  =  "1"  /* Repair Lingula Bronchus, Percutaneous Approach */
"0BQ94ZZ"  =  "1"  /* Repair Lingula Bronchus, Percutaneous Endoscopic Approach */
"0BQ97ZZ"  =  "1"  /* Repair Lingula Bronchus, Via Natural or Artificial Opening */
"0BQ98ZZ"  =  "1"  /* Repair Lingula Bronchus, Endo */
"0BQB0ZZ"  =  "1"  /* Repair Left Lower Lobe Bronchus, Open Approach */
"0BQB3ZZ"  =  "1"  /* Repair Left Lower Lobe Bronchus, Percutaneous Approach */
"0BQB4ZZ"  =  "1"  /* Repair Left Lower Lobe Bronchus, Perc Endo Approach */
"0BQB7ZZ"  =  "1"  /* Repair Left Lower Lobe Bronchus, Via Opening */
"0BQB8ZZ"  =  "1"  /* Repair Left Lower Lobe Bronchus, Endo */
"0BQC0ZZ"  =  "1"  /* Repair Right Upper Lung Lobe, Open Approach */
"0BQC3ZZ"  =  "1"  /* Repair Right Upper Lung Lobe, Percutaneous Approach */
"0BQC4ZZ"  =  "1"  /* Repair Right Upper Lung Lobe, Perc Endo Approach */
"0BQC7ZZ"  =  "1"  /* Repair Right Upper Lung Lobe, Via Opening */
"0BQC8ZZ"  =  "1"  /* Repair Right Upper Lung Lobe, Endo */
"0BQD0ZZ"  =  "1"  /* Repair Right Middle Lung Lobe, Open Approach */
"0BQD3ZZ"  =  "1"  /* Repair Right Middle Lung Lobe, Percutaneous Approach */
"0BQD4ZZ"  =  "1"  /* Repair Right Middle Lung Lobe, Perc Endo Approach */
"0BQD7ZZ"  =  "1"  /* Repair Right Middle Lung Lobe, Via Opening */
"0BQD8ZZ"  =  "1"  /* Repair Right Middle Lung Lobe, Endo */
"0BQF0ZZ"  =  "1"  /* Repair Right Lower Lung Lobe, Open Approach */
"0BQF3ZZ"  =  "1"  /* Repair Right Lower Lung Lobe, Percutaneous Approach */
"0BQF4ZZ"  =  "1"  /* Repair Right Lower Lung Lobe, Perc Endo Approach */
"0BQF7ZZ"  =  "1"  /* Repair Right Lower Lung Lobe, Via Opening */
"0BQF8ZZ"  =  "1"  /* Repair Right Lower Lung Lobe, Endo */
"0BQG0ZZ"  =  "1"  /* Repair Left Upper Lung Lobe, Open Approach */
"0BQG3ZZ"  =  "1"  /* Repair Left Upper Lung Lobe, Percutaneous Approach */
"0BQG4ZZ"  =  "1"  /* Repair Left Upper Lung Lobe, Perc Endo Approach */
"0BQG7ZZ"  =  "1"  /* Repair Left Upper Lung Lobe, Via Opening */
"0BQG8ZZ"  =  "1"  /* Repair Left Upper Lung Lobe, Endo */
"0BQH0ZZ"  =  "1"  /* Repair Lung Lingula, Open Approach */
"0BQH3ZZ"  =  "1"  /* Repair Lung Lingula, Percutaneous Approach */
"0BQH4ZZ"  =  "1"  /* Repair Lung Lingula, Percutaneous Endoscopic Approach */
"0BQH7ZZ"  =  "1"  /* Repair Lung Lingula, Via Natural or Artificial Opening */
"0BQH8ZZ"  =  "1"  /* Repair Lung Lingula, Endo */
"0BQJ0ZZ"  =  "1"  /* Repair Left Lower Lung Lobe, Open Approach */
"0BQJ3ZZ"  =  "1"  /* Repair Left Lower Lung Lobe, Percutaneous Approach */
"0BQJ4ZZ"  =  "1"  /* Repair Left Lower Lung Lobe, Perc Endo Approach */
"0BQJ7ZZ"  =  "1"  /* Repair Left Lower Lung Lobe, Via Opening */
"0BQJ8ZZ"  =  "1"  /* Repair Left Lower Lung Lobe, Endo */
"0BQK0ZZ"  =  "1"  /* Repair Right Lung, Open Approach */
"0BQK3ZZ"  =  "1"  /* Repair Right Lung, Percutaneous Approach */
"0BQK4ZZ"  =  "1"  /* Repair Right Lung, Percutaneous Endoscopic Approach */
"0BQK7ZZ"  =  "1"  /* Repair Right Lung, Via Natural or Artificial Opening */
"0BQK8ZZ"  =  "1"  /* Repair Right Lung, Endo */
"0BQL0ZZ"  =  "1"  /* Repair Left Lung, Open Approach */
"0BQL3ZZ"  =  "1"  /* Repair Left Lung, Percutaneous Approach */
"0BQL4ZZ"  =  "1"  /* Repair Left Lung, Percutaneous Endoscopic Approach */
"0BQL7ZZ"  =  "1"  /* Repair Left Lung, Via Natural or Artificial Opening */
"0BQL8ZZ"  =  "1"  /* Repair Left Lung, Endo */
"0BQM0ZZ"  =  "1"  /* Repair Bilateral Lungs, Open Approach */
"0BQM3ZZ"  =  "1"  /* Repair Bilateral Lungs, Percutaneous Approach */
"0BQM4ZZ"  =  "1"  /* Repair Bilateral Lungs, Percutaneous Endoscopic Approach */
"0BQM7ZZ"  =  "1"  /* Repair Bilateral Lungs, Via Natural or Artificial Opening */
"0BQM8ZZ"  =  "1"  /* Repair Bilateral Lungs, Endo */
"0BQN0ZZ"  =  "1"  /* Repair Right Pleura, Open Approach */
"0BQN3ZZ"  =  "1"  /* Repair Right Pleura, Percutaneous Approach */
"0BQN4ZZ"  =  "1"  /* Repair Right Pleura, Percutaneous Endoscopic Approach */
"0BQP0ZZ"  =  "1"  /* Repair Left Pleura, Open Approach */
"0BQP3ZZ"  =  "1"  /* Repair Left Pleura, Percutaneous Approach */
"0BQP4ZZ"  =  "1"  /* Repair Left Pleura, Percutaneous Endoscopic Approach */
"0BQR0ZZ"  =  "1"  /* Repair Right Diaphragm, Open Approach */
"0BQR3ZZ"  =  "1"  /* Repair Right Diaphragm, Percutaneous Approach */
"0BQR4ZZ"  =  "1"  /* Repair Right Diaphragm, Percutaneous Endoscopic Approach */
"0BQS0ZZ"  =  "1"  /* Repair Left Diaphragm, Open Approach */
"0BQS3ZZ"  =  "1"  /* Repair Left Diaphragm, Percutaneous Approach */
"0BQS4ZZ"  =  "1"  /* Repair Left Diaphragm, Percutaneous Endoscopic Approach */
"0BS10ZZ"  =  "1"  /* Reposition Trachea, Open Approach */
"0BS20ZZ"  =  "1"  /* Reposition Carina, Open Approach */
"0BS30ZZ"  =  "1"  /* Reposition Right Main Bronchus, Open Approach */
"0BS40ZZ"  =  "1"  /* Reposition Right Upper Lobe Bronchus, Open Approach */
"0BS50ZZ"  =  "1"  /* Reposition Right Middle Lobe Bronchus, Open Approach */
"0BS60ZZ"  =  "1"  /* Reposition Right Lower Lobe Bronchus, Open Approach */
"0BS70ZZ"  =  "1"  /* Reposition Left Main Bronchus, Open Approach */
"0BS80ZZ"  =  "1"  /* Reposition Left Upper Lobe Bronchus, Open Approach */
"0BS90ZZ"  =  "1"  /* Reposition Lingula Bronchus, Open Approach */
"0BSB0ZZ"  =  "1"  /* Reposition Left Lower Lobe Bronchus, Open Approach */
"0BSC0ZZ"  =  "1"  /* Reposition Right Upper Lung Lobe, Open Approach */
"0BSD0ZZ"  =  "1"  /* Reposition Right Middle Lung Lobe, Open Approach */
"0BSF0ZZ"  =  "1"  /* Reposition Right Lower Lung Lobe, Open Approach */
"0BSG0ZZ"  =  "1"  /* Reposition Left Upper Lung Lobe, Open Approach */
"0BSH0ZZ"  =  "1"  /* Reposition Lung Lingula, Open Approach */
"0BSJ0ZZ"  =  "1"  /* Reposition Left Lower Lung Lobe, Open Approach */
"0BSK0ZZ"  =  "1"  /* Reposition Right Lung, Open Approach */
"0BSL0ZZ"  =  "1"  /* Reposition Left Lung, Open Approach */
"0BSR0ZZ"  =  "1"  /* Reposition Right Diaphragm, Open Approach */
"0BSS0ZZ"  =  "1"  /* Reposition Left Diaphragm, Open Approach */
"0BT10ZZ"  =  "1"  /* Resection of Trachea, Open Approach */
"0BT14ZZ"  =  "1"  /* Resection of Trachea, Percutaneous Endoscopic Approach */
"0BT20ZZ"  =  "1"  /* Resection of Carina, Open Approach */
"0BT24ZZ"  =  "1"  /* Resection of Carina, Percutaneous Endoscopic Approach */
"0BT30ZZ"  =  "1"  /* Resection of Right Main Bronchus, Open Approach */
"0BT34ZZ"  =  "1"  /* Resection of Right Main Bronchus, Perc Endo Approach */
"0BT40ZZ"  =  "1"  /* Resection of Right Upper Lobe Bronchus, Open Approach */
"0BT44ZZ"  =  "1"  /* Resection of Right Upper Lobe Bronchus, Perc Endo Approach */
"0BT50ZZ"  =  "1"  /* Resection of Right Middle Lobe Bronchus, Open Approach */
"0BT54ZZ"  =  "1"  /* Resection of Right Middle Lobe Bronchus, Perc Endo Approach */
"0BT60ZZ"  =  "1"  /* Resection of Right Lower Lobe Bronchus, Open Approach */
"0BT64ZZ"  =  "1"  /* Resection of Right Lower Lobe Bronchus, Perc Endo Approach */
"0BT70ZZ"  =  "1"  /* Resection of Left Main Bronchus, Open Approach */
"0BT74ZZ"  =  "1"  /* Resection of Left Main Bronchus, Perc Endo Approach */
"0BT80ZZ"  =  "1"  /* Resection of Left Upper Lobe Bronchus, Open Approach */
"0BT84ZZ"  =  "1"  /* Resection of Left Upper Lobe Bronchus, Perc Endo Approach */
"0BT90ZZ"  =  "1"  /* Resection of Lingula Bronchus, Open Approach */
"0BT94ZZ"  =  "1"  /* Resection of Lingula Bronchus, Perc Endo Approach */
"0BTB0ZZ"  =  "1"  /* Resection of Left Lower Lobe Bronchus, Open Approach */
"0BTB4ZZ"  =  "1"  /* Resection of Left Lower Lobe Bronchus, Perc Endo Approach */
"0BTC0ZZ"  =  "1"  /* Resection of Right Upper Lung Lobe, Open Approach */
"0BTC4ZZ"  =  "1"  /* Resection of Right Upper Lung Lobe, Perc Endo Approach */
"0BTD0ZZ"  =  "1"  /* Resection of Right Middle Lung Lobe, Open Approach */
"0BTD4ZZ"  =  "1"  /* Resection of Right Middle Lung Lobe, Perc Endo Approach */
"0BTF0ZZ"  =  "1"  /* Resection of Right Lower Lung Lobe, Open Approach */
"0BTF4ZZ"  =  "1"  /* Resection of Right Lower Lung Lobe, Perc Endo Approach */
"0BTG0ZZ"  =  "1"  /* Resection of Left Upper Lung Lobe, Open Approach */
"0BTG4ZZ"  =  "1"  /* Resection of Left Upper Lung Lobe, Perc Endo Approach */
"0BTH0ZZ"  =  "1"  /* Resection of Lung Lingula, Open Approach */
"0BTH4ZZ"  =  "1"  /* Resection of Lung Lingula, Percutaneous Endoscopic Approach */
"0BTJ0ZZ"  =  "1"  /* Resection of Left Lower Lung Lobe, Open Approach */
"0BTJ4ZZ"  =  "1"  /* Resection of Left Lower Lung Lobe, Perc Endo Approach */
"0BTK0ZZ"  =  "1"  /* Resection of Right Lung, Open Approach */
"0BTK4ZZ"  =  "1"  /* Resection of Right Lung, Percutaneous Endoscopic Approach */
"0BTL0ZZ"  =  "1"  /* Resection of Left Lung, Open Approach */
"0BTL4ZZ"  =  "1"  /* Resection of Left Lung, Percutaneous Endoscopic Approach */
"0BTM0ZZ"  =  "1"  /* Resection of Bilateral Lungs, Open Approach */
"0BTM4ZZ"  =  "1"  /* Resection of Bilateral Lungs, Perc Endo Approach */
"0BTR0ZZ"  =  "1"  /* Resection of Right Diaphragm, Open Approach */
"0BTR4ZZ"  =  "1"  /* Resection of Right Diaphragm, Perc Endo Approach */
"0BTS0ZZ"  =  "1"  /* Resection of Left Diaphragm, Open Approach */
"0BTS4ZZ"  =  "1"  /* Resection of Left Diaphragm, Perc Endo Approach */
"0BU107Z"  =  "1"  /* Supplement Trachea with Autol Sub, Open Approach */
"0BU10JZ"  =  "1"  /* Supplement Trachea with Synthetic Substitute, Open Approach */
"0BU10KZ"  =  "1"  /* Supplement Trachea with Nonaut Sub, Open Approach */
"0BU147Z"  =  "1"  /* Supplement Trachea with Autol Sub, Perc Endo Approach */
"0BU14JZ"  =  "1"  /* Supplement Trachea with Synth Sub, Perc Endo Approach */
"0BU14KZ"  =  "1"  /* Supplement Trachea with Nonaut Sub, Perc Endo Approach */
"0BU207Z"  =  "1"  /* Supplement Carina with Autol Sub, Open Approach */
"0BU20JZ"  =  "1"  /* Supplement Carina with Synthetic Substitute, Open Approach */
"0BU20KZ"  =  "1"  /* Supplement Carina with Nonaut Sub, Open Approach */
"0BU247Z"  =  "1"  /* Supplement Carina with Autol Sub, Perc Endo Approach */
"0BU24JZ"  =  "1"  /* Supplement Carina with Synth Sub, Perc Endo Approach */
"0BU24KZ"  =  "1"  /* Supplement Carina with Nonaut Sub, Perc Endo Approach */
"0BU307Z"  =  "1"  /* Supplement Right Main Bronchus with Autol Sub, Open Approach */
"0BU30JZ"  =  "1"  /* Supplement Right Main Bronchus with Synth Sub, Open Approach */
"0BU30KZ"  =  "1"  /* Supplement R Main Bronc with Nonaut Sub, Open Approach */
"0BU347Z"  =  "1"  /* Supplement R Main Bronc with Autol Sub, Perc Endo Approach */
"0BU34JZ"  =  "1"  /* Supplement R Main Bronc with Synth Sub, Perc Endo Approach */
"0BU34KZ"  =  "1"  /* Supplement R Main Bronc with Nonaut Sub, Perc Endo Approach */
"0BU407Z"  =  "1"  /* Supplement R Up Lobe Bronc with Autol Sub, Open Approach */
"0BU40JZ"  =  "1"  /* Supplement R Up Lobe Bronc with Synth Sub, Open Approach */
"0BU40KZ"  =  "1"  /* Supplement R Up Lobe Bronc with Nonaut Sub, Open Approach */
"0BU447Z"  =  "1"  /* Supplement R Up Lobe Bronc w Autol Sub, Perc Endo */
"0BU44JZ"  =  "1"  /* Supplement R Up Lobe Bronc w Synth Sub, Perc Endo */
"0BU44KZ"  =  "1"  /* Supplement R Up Lobe Bronc w Nonaut Sub, Perc Endo */
"0BU507Z"  =  "1"  /* Supplement R Mid Lobe Bronc with Autol Sub, Open Approach */
"0BU50JZ"  =  "1"  /* Supplement R Mid Lobe Bronc with Synth Sub, Open Approach */
"0BU50KZ"  =  "1"  /* Supplement R Mid Lobe Bronc with Nonaut Sub, Open Approach */
"0BU547Z"  =  "1"  /* Supplement R Mid Lobe Bronc w Autol Sub, Perc Endo */
"0BU54JZ"  =  "1"  /* Supplement R Mid Lobe Bronc w Synth Sub, Perc Endo */
"0BU54KZ"  =  "1"  /* Supplement R Mid Lobe Bronc w Nonaut Sub, Perc Endo */
"0BU607Z"  =  "1"  /* Supplement R Low Lobe Bronc with Autol Sub, Open Approach */
"0BU60JZ"  =  "1"  /* Supplement R Low Lobe Bronc with Synth Sub, Open Approach */
"0BU60KZ"  =  "1"  /* Supplement R Low Lobe Bronc with Nonaut Sub, Open Approach */
"0BU647Z"  =  "1"  /* Supplement R Low Lobe Bronc w Autol Sub, Perc Endo */
"0BU64JZ"  =  "1"  /* Supplement R Low Lobe Bronc w Synth Sub, Perc Endo */
"0BU64KZ"  =  "1"  /* Supplement R Low Lobe Bronc w Nonaut Sub, Perc Endo */
"0BU707Z"  =  "1"  /* Supplement Left Main Bronchus with Autol Sub, Open Approach */
"0BU70JZ"  =  "1"  /* Supplement Left Main Bronchus with Synth Sub, Open Approach */
"0BU70KZ"  =  "1"  /* Supplement Left Main Bronchus with Nonaut Sub, Open Approach */
"0BU747Z"  =  "1"  /* Supplement L Main Bronc with Autol Sub, Perc Endo Approach */
"0BU74JZ"  =  "1"  /* Supplement L Main Bronc with Synth Sub, Perc Endo Approach */
"0BU74KZ"  =  "1"  /* Supplement L Main Bronc with Nonaut Sub, Perc Endo Approach */
"0BU807Z"  =  "1"  /* Supplement L Up Lobe Bronc with Autol Sub, Open Approach */
"0BU80JZ"  =  "1"  /* Supplement L Up Lobe Bronc with Synth Sub, Open Approach */
"0BU80KZ"  =  "1"  /* Supplement L Up Lobe Bronc with Nonaut Sub, Open Approach */
"0BU847Z"  =  "1"  /* Supplement L Up Lobe Bronc w Autol Sub, Perc Endo */
"0BU84JZ"  =  "1"  /* Supplement L Up Lobe Bronc w Synth Sub, Perc Endo */
"0BU84KZ"  =  "1"  /* Supplement L Up Lobe Bronc w Nonaut Sub, Perc Endo */
"0BU907Z"  =  "1"  /* Supplement Lingula Bronchus with Autol Sub, Open Approach */
"0BU90JZ"  =  "1"  /* Supplement Lingula Bronchus with Synth Sub, Open Approach */
"0BU90KZ"  =  "1"  /* Supplement Lingula Bronchus with Nonaut Sub, Open Approach */
"0BU947Z"  =  "1"  /* Supplement Lingula Bronc with Autol Sub, Perc Endo Approach */
"0BU94JZ"  =  "1"  /* Supplement Lingula Bronc with Synth Sub, Perc Endo Approach */
"0BU94KZ"  =  "1"  /* Supplement Lingula Bronc with Nonaut Sub, Perc Endo Approach */
"0BUB07Z"  =  "1"  /* Supplement L Low Lobe Bronc with Autol Sub, Open Approach */
"0BUB0JZ"  =  "1"  /* Supplement L Low Lobe Bronc with Synth Sub, Open Approach */
"0BUB0KZ"  =  "1"  /* Supplement L Low Lobe Bronc with Nonaut Sub, Open Approach */
"0BUB47Z"  =  "1"  /* Supplement L Low Lobe Bronc w Autol Sub, Perc Endo */
"0BUB4JZ"  =  "1"  /* Supplement L Low Lobe Bronc w Synth Sub, Perc Endo */
"0BUB4KZ"  =  "1"  /* Supplement L Low Lobe Bronc w Nonaut Sub, Perc Endo */
"0BUR07Z"  =  "1"  /* Supplement Right Diaphragm with Autol Sub, Open Approach */
"0BUR0JZ"  =  "1"  /* Supplement Right Diaphragm with Synth Sub, Open Approach */
"0BUR0KZ"  =  "1"  /* Supplement Right Diaphragm with Nonaut Sub, Open Approach */
"0BUR47Z"  =  "1"  /* Supplement R Diaphragm with Autol Sub, Perc Endo Approach */
"0BUR4JZ"  =  "1"  /* Supplement R Diaphragm with Synth Sub, Perc Endo Approach */
"0BUR4KZ"  =  "1"  /* Supplement R Diaphragm with Nonaut Sub, Perc Endo Approach */
"0BUS07Z"  =  "1"  /* Supplement Left Diaphragm with Autol Sub, Open Approach */
"0BUS0JZ"  =  "1"  /* Supplement Left Diaphragm with Synth Sub, Open Approach */
"0BUS0KZ"  =  "1"  /* Supplement Left Diaphragm with Nonaut Sub, Open Approach */
"0BUS47Z"  =  "1"  /* Supplement Left Diaphragm with Autol Sub, Perc Endo Approach */
"0BUS4JZ"  =  "1"  /* Supplement Left Diaphragm with Synth Sub, Perc Endo Approach */
"0BUS4KZ"  =  "1"  /* Supplement L Diaphragm with Nonaut Sub, Perc Endo Approach */
"0BV10CZ"  =  "1"  /* Restriction of Trachea with Extralum Dev, Open Approach */
"0BV10DZ"  =  "1"  /* Restriction of Trachea with Intralum Dev, Open Approach */
"0BV10ZZ"  =  "1"  /* Restriction of Trachea, Open Approach */
"0BV13CZ"  =  "1"  /* Restriction of Trachea with Extralum Dev, Perc Approach */
"0BV13DZ"  =  "1"  /* Restriction of Trachea with Intralum Dev, Perc Approach */
"0BV13ZZ"  =  "1"  /* Restriction of Trachea, Percutaneous Approach */
"0BV14CZ"  =  "1"  /* Restriction of Trachea with Extralum Dev, Perc Endo Approach */
"0BV14DZ"  =  "1"  /* Restriction of Trachea with Intralum Dev, Perc Endo Approach */
"0BV14ZZ"  =  "1"  /* Restriction of Trachea, Percutaneous Endoscopic Approach */
"0BV17DZ"  =  "1"  /* Restriction of Trachea with Intraluminal Device, Via Opening */
"0BV17ZZ"  =  "1"  /* Restriction of Trachea, Via Natural or Artificial Opening */
"0BV18DZ"  =  "1"  /* Restriction of Trachea with Intraluminal Device, Endo */
"0BV18ZZ"  =  "1"  /* Restriction of Trachea, Endo */
"0BV20CZ"  =  "1"  /* Restriction of Carina with Extralum Dev, Open Approach */
"0BV20DZ"  =  "1"  /* Restriction of Carina with Intralum Dev, Open Approach */
"0BV20ZZ"  =  "1"  /* Restriction of Carina, Open Approach */
"0BV23CZ"  =  "1"  /* Restriction of Carina with Extralum Dev, Perc Approach */
"0BV23DZ"  =  "1"  /* Restriction of Carina with Intralum Dev, Perc Approach */
"0BV23ZZ"  =  "1"  /* Restriction of Carina, Percutaneous Approach */
"0BV24CZ"  =  "1"  /* Restriction of Carina with Extralum Dev, Perc Endo Approach */
"0BV24DZ"  =  "1"  /* Restriction of Carina with Intralum Dev, Perc Endo Approach */
"0BV24ZZ"  =  "1"  /* Restriction of Carina, Percutaneous Endoscopic Approach */
"0BV27DZ"  =  "1"  /* Restriction of Carina with Intraluminal Device, Via Opening */
"0BV27ZZ"  =  "1"  /* Restriction of Carina, Via Natural or Artificial Opening */
"0BV28DZ"  =  "1"  /* Restriction of Carina with Intraluminal Device, Endo */
"0BV28ZZ"  =  "1"  /* Restriction of Carina, Endo */
"0BV30CZ"  =  "1"  /* Restriction of R Main Bronc with Extralum Dev, Open Approach */
"0BV30DZ"  =  "1"  /* Restriction of R Main Bronc with Intralum Dev, Open Approach */
"0BV30ZZ"  =  "1"  /* Restriction of Right Main Bronchus, Open Approach */
"0BV33CZ"  =  "1"  /* Restriction of R Main Bronc with Extralum Dev, Perc Approach */
"0BV33DZ"  =  "1"  /* Restriction of R Main Bronc with Intralum Dev, Perc Approach */
"0BV33ZZ"  =  "1"  /* Restriction of Right Main Bronchus, Percutaneous Approach */
"0BV34CZ"  =  "1"  /* Restrict R Main Bronc w Extralum Dev, Perc Endo */
"0BV34DZ"  =  "1"  /* Restrict R Main Bronc w Intralum Dev, Perc Endo */
"0BV34ZZ"  =  "1"  /* Restriction of Right Main Bronchus, Perc Endo Approach */
"0BV37DZ"  =  "1"  /* Restriction of R Main Bronc with Intralum Dev, Via Opening */
"0BV37ZZ"  =  "1"  /* Restriction of Right Main Bronchus, Via Opening */
"0BV38DZ"  =  "1"  /* Restriction of Right Main Bronchus with Intralum Dev, Endo */
"0BV38ZZ"  =  "1"  /* Restriction of Right Main Bronchus, Endo */
"0BV40CZ"  =  "1"  /* Restrict of R Up Lobe Bronc with Extralum Dev, Open Approach */
"0BV40DZ"  =  "1"  /* Restrict of R Up Lobe Bronc with Intralum Dev, Open Approach */
"0BV40ZZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus, Open Approach */
"0BV43CZ"  =  "1"  /* Restrict of R Up Lobe Bronc with Extralum Dev, Perc Approach */
"0BV43DZ"  =  "1"  /* Restrict of R Up Lobe Bronc with Intralum Dev, Perc Approach */
"0BV43ZZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus, Perc Approach */
"0BV44CZ"  =  "1"  /* Restrict R Up Lobe Bronc w Extralum Dev, Perc Endo */
"0BV44DZ"  =  "1"  /* Restrict R Up Lobe Bronc w Intralum Dev, Perc Endo */
"0BV44ZZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus, Perc Endo Approach */
"0BV47DZ"  =  "1"  /* Restrict of R Up Lobe Bronc with Intralum Dev, Via Opening */
"0BV47ZZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus, Via Opening */
"0BV48DZ"  =  "1"  /* Restriction of R Up Lobe Bronc with Intralum Dev, Endo */
"0BV48ZZ"  =  "1"  /* Restriction of Right Upper Lobe Bronchus, Endo */
"0BV50CZ"  =  "1"  /* Restrict R Mid Lobe Bronc w Extralum Dev, Open */
"0BV50DZ"  =  "1"  /* Restrict R Mid Lobe Bronc w Intralum Dev, Open */
"0BV50ZZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus, Open Approach */
"0BV53CZ"  =  "1"  /* Restrict R Mid Lobe Bronc w Extralum Dev, Perc */
"0BV53DZ"  =  "1"  /* Restrict R Mid Lobe Bronc w Intralum Dev, Perc */
"0BV53ZZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus, Perc Approach */
"0BV54CZ"  =  "1"  /* Restrict R Mid Lobe Bronc w Extralum Dev, Perc Endo */
"0BV54DZ"  =  "1"  /* Restrict R Mid Lobe Bronc w Intralum Dev, Perc Endo */
"0BV54ZZ"  =  "1"  /* Restriction of R Mid Lobe Bronc, Perc Endo Approach */
"0BV57DZ"  =  "1"  /* Restrict of R Mid Lobe Bronc with Intralum Dev, Via Opening */
"0BV57ZZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus, Via Opening */
"0BV58DZ"  =  "1"  /* Restriction of R Mid Lobe Bronc with Intralum Dev, Endo */
"0BV58ZZ"  =  "1"  /* Restriction of Right Middle Lobe Bronchus, Endo */
"0BV60CZ"  =  "1"  /* Restrict R Low Lobe Bronc w Extralum Dev, Open */
"0BV60DZ"  =  "1"  /* Restrict R Low Lobe Bronc w Intralum Dev, Open */
"0BV60ZZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus, Open Approach */
"0BV63CZ"  =  "1"  /* Restrict R Low Lobe Bronc w Extralum Dev, Perc */
"0BV63DZ"  =  "1"  /* Restrict R Low Lobe Bronc w Intralum Dev, Perc */
"0BV63ZZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus, Perc Approach */
"0BV64CZ"  =  "1"  /* Restrict R Low Lobe Bronc w Extralum Dev, Perc Endo */
"0BV64DZ"  =  "1"  /* Restrict R Low Lobe Bronc w Intralum Dev, Perc Endo */
"0BV64ZZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus, Perc Endo Approach */
"0BV67DZ"  =  "1"  /* Restrict of R Low Lobe Bronc with Intralum Dev, Via Opening */
"0BV67ZZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus, Via Opening */
"0BV68DZ"  =  "1"  /* Restriction of R Low Lobe Bronc with Intralum Dev, Endo */
"0BV68ZZ"  =  "1"  /* Restriction of Right Lower Lobe Bronchus, Endo */
"0BV70CZ"  =  "1"  /* Restriction of L Main Bronc with Extralum Dev, Open Approach */
"0BV70DZ"  =  "1"  /* Restriction of L Main Bronc with Intralum Dev, Open Approach */
"0BV70ZZ"  =  "1"  /* Restriction of Left Main Bronchus, Open Approach */
"0BV73CZ"  =  "1"  /* Restriction of L Main Bronc with Extralum Dev, Perc Approach */
"0BV73DZ"  =  "1"  /* Restriction of L Main Bronc with Intralum Dev, Perc Approach */
"0BV73ZZ"  =  "1"  /* Restriction of Left Main Bronchus, Percutaneous Approach */
"0BV74CZ"  =  "1"  /* Restrict L Main Bronc w Extralum Dev, Perc Endo */
"0BV74DZ"  =  "1"  /* Restrict L Main Bronc w Intralum Dev, Perc Endo */
"0BV74ZZ"  =  "1"  /* Restriction of Left Main Bronchus, Perc Endo Approach */
"0BV77DZ"  =  "1"  /* Restriction of L Main Bronc with Intralum Dev, Via Opening */
"0BV77ZZ"  =  "1"  /* Restriction of Left Main Bronchus, Via Opening */
"0BV78DZ"  =  "1"  /* Restriction of Left Main Bronchus with Intralum Dev, Endo */
"0BV78ZZ"  =  "1"  /* Restriction of Left Main Bronchus, Endo */
"0BV80CZ"  =  "1"  /* Restrict of L Up Lobe Bronc with Extralum Dev, Open Approach */
"0BV80DZ"  =  "1"  /* Restrict of L Up Lobe Bronc with Intralum Dev, Open Approach */
"0BV80ZZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus, Open Approach */
"0BV83CZ"  =  "1"  /* Restrict of L Up Lobe Bronc with Extralum Dev, Perc Approach */
"0BV83DZ"  =  "1"  /* Restrict of L Up Lobe Bronc with Intralum Dev, Perc Approach */
"0BV83ZZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus, Perc Approach */
"0BV84CZ"  =  "1"  /* Restrict L Up Lobe Bronc w Extralum Dev, Perc Endo */
"0BV84DZ"  =  "1"  /* Restrict L Up Lobe Bronc w Intralum Dev, Perc Endo */
"0BV84ZZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus, Perc Endo Approach */
"0BV87DZ"  =  "1"  /* Restrict of L Up Lobe Bronc with Intralum Dev, Via Opening */
"0BV87ZZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus, Via Opening */
"0BV88DZ"  =  "1"  /* Restriction of L Up Lobe Bronc with Intralum Dev, Endo */
"0BV88ZZ"  =  "1"  /* Restriction of Left Upper Lobe Bronchus, Endo */
"0BV90CZ"  =  "1"  /* Restrict of Lingula Bronc with Extralum Dev, Open Approach */
"0BV90DZ"  =  "1"  /* Restrict of Lingula Bronc with Intralum Dev, Open Approach */
"0BV90ZZ"  =  "1"  /* Restriction of Lingula Bronchus, Open Approach */
"0BV93CZ"  =  "1"  /* Restrict of Lingula Bronc with Extralum Dev, Perc Approach */
"0BV93DZ"  =  "1"  /* Restrict of Lingula Bronc with Intralum Dev, Perc Approach */
"0BV93ZZ"  =  "1"  /* Restriction of Lingula Bronchus, Percutaneous Approach */
"0BV94CZ"  =  "1"  /* Restrict Lingula Bronc w Extralum Dev, Perc Endo */
"0BV94DZ"  =  "1"  /* Restrict Lingula Bronc w Intralum Dev, Perc Endo */
"0BV94ZZ"  =  "1"  /* Restriction of Lingula Bronchus, Perc Endo Approach */
"0BV97DZ"  =  "1"  /* Restriction of Lingula Bronc with Intralum Dev, Via Opening */
"0BV97ZZ"  =  "1"  /* Restriction of Lingula Bronchus, Via Opening */
"0BV98DZ"  =  "1"  /* Restriction of Lingula Bronchus with Intralum Dev, Endo */
"0BV98ZZ"  =  "1"  /* Restriction of Lingula Bronchus, Endo */
"0BVB0CZ"  =  "1"  /* Restrict L Low Lobe Bronc w Extralum Dev, Open */
"0BVB0DZ"  =  "1"  /* Restrict L Low Lobe Bronc w Intralum Dev, Open */
"0BVB0ZZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus, Open Approach */
"0BVB3CZ"  =  "1"  /* Restrict L Low Lobe Bronc w Extralum Dev, Perc */
"0BVB3DZ"  =  "1"  /* Restrict L Low Lobe Bronc w Intralum Dev, Perc */
"0BVB3ZZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus, Perc Approach */
"0BVB4CZ"  =  "1"  /* Restrict L Low Lobe Bronc w Extralum Dev, Perc Endo */
"0BVB4DZ"  =  "1"  /* Restrict L Low Lobe Bronc w Intralum Dev, Perc Endo */
"0BVB4ZZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus, Perc Endo Approach */
"0BVB7DZ"  =  "1"  /* Restrict of L Low Lobe Bronc with Intralum Dev, Via Opening */
"0BVB7ZZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus, Via Opening */
"0BVB8DZ"  =  "1"  /* Restriction of L Low Lobe Bronc with Intralum Dev, Endo */
"0BVB8ZZ"  =  "1"  /* Restriction of Left Lower Lobe Bronchus, Endo */
"0BW000Z"  =  "1"  /* Revision of Drain Dev in Tracheobronc Tree, Open Approach */
"0BW002Z"  =  "1"  /* Revision of Monitor Dev in Tracheobronc Tree, Open Approach */
"0BW003Z"  =  "1"  /* Revision of Infusion Dev in Tracheobronc Tree, Open Approach */
"0BW007Z"  =  "1"  /* Revision of Autol Sub in Tracheobronc Tree, Open Approach */
"0BW00CZ"  =  "1"  /* Revision of Extralum Dev in Tracheobronc Tree, Open Approach */
"0BW00DZ"  =  "1"  /* Revision of Intralum Dev in Tracheobronc Tree, Open Approach */
"0BW00JZ"  =  "1"  /* Revision of Synth Sub in Tracheobronc Tree, Open Approach */
"0BW00KZ"  =  "1"  /* Revision of Nonaut Sub in Tracheobronc Tree, Open Approach */
"0BW030Z"  =  "1"  /* Revision of Drain Dev in Tracheobronc Tree, Perc Approach */
"0BW032Z"  =  "1"  /* Revision of Monitor Dev in Tracheobronc Tree, Perc Approach */
"0BW033Z"  =  "1"  /* Revision of Infusion Dev in Tracheobronc Tree, Perc Approach */
"0BW037Z"  =  "1"  /* Revision of Autol Sub in Tracheobronc Tree, Perc Approach */
"0BW03CZ"  =  "1"  /* Revision of Extralum Dev in Tracheobronc Tree, Perc Approach */
"0BW03DZ"  =  "1"  /* Revision of Intralum Dev in Tracheobronc Tree, Perc Approach */
"0BW03JZ"  =  "1"  /* Revision of Synth Sub in Tracheobronc Tree, Perc Approach */
"0BW03KZ"  =  "1"  /* Revision of Nonaut Sub in Tracheobronc Tree, Perc Approach */
"0BW040Z"  =  "1"  /* Revise of Drain Dev in Tracheobronc Tree, Perc Endo Approach */
"0BW042Z"  =  "1"  /* Revise Monitor Dev in Tracheobronc Tree, Perc Endo */
"0BW043Z"  =  "1"  /* Revise Infusion Dev in Tracheobronc Tree, Perc Endo */
"0BW047Z"  =  "1"  /* Revise of Autol Sub in Tracheobronc Tree, Perc Endo Approach */
"0BW04CZ"  =  "1"  /* Revise Extralum Dev in Tracheobronc Tree, Perc Endo */
"0BW04DZ"  =  "1"  /* Revise Intralum Dev in Tracheobronc Tree, Perc Endo */
"0BW04JZ"  =  "1"  /* Revise of Synth Sub in Tracheobronc Tree, Perc Endo Approach */
"0BW04KZ"  =  "1"  /* Revise Nonaut Sub in Tracheobronc Tree, Perc Endo */
"0BW070Z"  =  "1"  /* Revision of Drain Dev in Tracheobronc Tree, Via Opening */
"0BW072Z"  =  "1"  /* Revision of Monitor Dev in Tracheobronc Tree, Via Opening */
"0BW073Z"  =  "1"  /* Revision of Infusion Dev in Tracheobronc Tree, Via Opening */
"0BW077Z"  =  "1"  /* Revision of Autol Sub in Tracheobronc Tree, Via Opening */
"0BW07CZ"  =  "1"  /* Revision of Extralum Dev in Tracheobronc Tree, Via Opening */
"0BW07DZ"  =  "1"  /* Revision of Intralum Dev in Tracheobronc Tree, Via Opening */
"0BW07JZ"  =  "1"  /* Revision of Synth Sub in Tracheobronc Tree, Via Opening */
"0BW07KZ"  =  "1"  /* Revision of Nonaut Sub in Tracheobronc Tree, Via Opening */
"0BW080Z"  =  "1"  /* Revision of Drainage Device in Tracheobronchial Tree, Endo */
"0BW082Z"  =  "1"  /* Revision of Monitoring Device in Tracheobronchial Tree, Endo */
"0BW083Z"  =  "1"  /* Revision of Infusion Device in Tracheobronchial Tree, Endo */
"0BW087Z"  =  "1"  /* Revision of Autol Sub in Tracheobronc Tree, Endo */
"0BW08CZ"  =  "1"  /* Revision of Extraluminal Device in Tracheobronc Tree, Endo */
"0BW08DZ"  =  "1"  /* Revision of Intraluminal Device in Tracheobronc Tree, Endo */
"0BW08JZ"  =  "1"  /* Revision of Synthetic Substitute in Tracheobronc Tree, Endo */
"0BW08KZ"  =  "1"  /* Revision of Nonaut Sub in Tracheobronc Tree, Endo */
"0BW100Z"  =  "1"  /* Revision of Drainage Device in Trachea, Open Approach */
"0BW102Z"  =  "1"  /* Revision of Monitoring Device in Trachea, Open Approach */
"0BW107Z"  =  "1"  /* Revision of Autol Sub in Trachea, Open Approach */
"0BW10CZ"  =  "1"  /* Revision of Extraluminal Device in Trachea, Open Approach */
"0BW10DZ"  =  "1"  /* Revision of Intraluminal Device in Trachea, Open Approach */
"0BW10FZ"  =  "1"  /* Revision of Tracheostomy Device in Trachea, Open Approach */
"0BW10JZ"  =  "1"  /* Revision of Synthetic Substitute in Trachea, Open Approach */
"0BW10KZ"  =  "1"  /* Revision of Nonaut Sub in Trachea, Open Approach */
"0BW130Z"  =  "1"  /* Revision of Drainage Device in Trachea, Perc Approach */
"0BW132Z"  =  "1"  /* Revision of Monitoring Device in Trachea, Perc Approach */
"0BW137Z"  =  "1"  /* Revision of Autol Sub in Trachea, Perc Approach */
"0BW13CZ"  =  "1"  /* Revision of Extraluminal Device in Trachea, Perc Approach */
"0BW13DZ"  =  "1"  /* Revision of Intraluminal Device in Trachea, Perc Approach */
"0BW13FZ"  =  "1"  /* Revision of Tracheostomy Device in Trachea, Perc Approach */
"0BW13JZ"  =  "1"  /* Revision of Synthetic Substitute in Trachea, Perc Approach */
"0BW13KZ"  =  "1"  /* Revision of Nonaut Sub in Trachea, Perc Approach */
"0BW140Z"  =  "1"  /* Revision of Drainage Device in Trachea, Perc Endo Approach */
"0BW142Z"  =  "1"  /* Revision of Monitoring Device in Trachea, Perc Endo Approach */
"0BW147Z"  =  "1"  /* Revision of Autol Sub in Trachea, Perc Endo Approach */
"0BW14CZ"  =  "1"  /* Revision of Extralum Dev in Trachea, Perc Endo Approach */
"0BW14DZ"  =  "1"  /* Revision of Intralum Dev in Trachea, Perc Endo Approach */
"0BW14FZ"  =  "1"  /* Revision of Trach Dev in Trachea, Perc Endo Approach */
"0BW14JZ"  =  "1"  /* Revision of Synth Sub in Trachea, Perc Endo Approach */
"0BW14KZ"  =  "1"  /* Revision of Nonaut Sub in Trachea, Perc Endo Approach */
"0BW170Z"  =  "1"  /* Revision of Drainage Device in Trachea, Via Opening */
"0BW172Z"  =  "1"  /* Revision of Monitoring Device in Trachea, Via Opening */
"0BW177Z"  =  "1"  /* Revision of Autol Sub in Trachea, Via Opening */
"0BW17CZ"  =  "1"  /* Revision of Extraluminal Device in Trachea, Via Opening */
"0BW17DZ"  =  "1"  /* Revision of Intraluminal Device in Trachea, Via Opening */
"0BW17FZ"  =  "1"  /* Revision of Tracheostomy Device in Trachea, Via Opening */
"0BW17JZ"  =  "1"  /* Revision of Synthetic Substitute in Trachea, Via Opening */
"0BW17KZ"  =  "1"  /* Revision of Nonaut Sub in Trachea, Via Opening */
"0BW180Z"  =  "1"  /* Revision of Drainage Device in Trachea, Endo */
"0BW182Z"  =  "1"  /* Revision of Monitoring Device in Trachea, Endo */
"0BW187Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Trachea, Endo */
"0BW18CZ"  =  "1"  /* Revision of Extraluminal Device in Trachea, Endo */
"0BW18DZ"  =  "1"  /* Revision of Intraluminal Device in Trachea, Endo */
"0BW18FZ"  =  "1"  /* Revision of Tracheostomy Device in Trachea, Endo */
"0BW18JZ"  =  "1"  /* Revision of Synthetic Substitute in Trachea, Endo */
"0BW18KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Trachea, Endo */
"0BWK00Z"  =  "1"  /* Revision of Drainage Device in Right Lung, Open Approach */
"0BWK02Z"  =  "1"  /* Revision of Monitoring Device in Right Lung, Open Approach */
"0BWK03Z"  =  "1"  /* Revision of Infusion Device in Right Lung, Open Approach */
"0BWK30Z"  =  "1"  /* Revision of Drainage Device in Right Lung, Perc Approach */
"0BWK32Z"  =  "1"  /* Revision of Monitoring Device in Right Lung, Perc Approach */
"0BWK33Z"  =  "1"  /* Revision of Infusion Device in Right Lung, Perc Approach */
"0BWK40Z"  =  "1"  /* Revision of Drainage Device in R Lung, Perc Endo Approach */
"0BWK42Z"  =  "1"  /* Revision of Monitoring Device in R Lung, Perc Endo Approach */
"0BWK43Z"  =  "1"  /* Revision of Infusion Device in R Lung, Perc Endo Approach */
"0BWK70Z"  =  "1"  /* Revision of Drainage Device in Right Lung, Via Opening */
"0BWK72Z"  =  "1"  /* Revision of Monitoring Device in Right Lung, Via Opening */
"0BWK73Z"  =  "1"  /* Revision of Infusion Device in Right Lung, Via Opening */
"0BWK80Z"  =  "1"  /* Revision of Drainage Device in Right Lung, Endo */
"0BWK82Z"  =  "1"  /* Revision of Monitoring Device in Right Lung, Endo */
"0BWK83Z"  =  "1"  /* Revision of Infusion Device in Right Lung, Endo */
"0BWL00Z"  =  "1"  /* Revision of Drainage Device in Left Lung, Open Approach */
"0BWL02Z"  =  "1"  /* Revision of Monitoring Device in Left Lung, Open Approach */
"0BWL03Z"  =  "1"  /* Revision of Infusion Device in Left Lung, Open Approach */
"0BWL30Z"  =  "1"  /* Revision of Drainage Device in Left Lung, Perc Approach */
"0BWL32Z"  =  "1"  /* Revision of Monitoring Device in Left Lung, Perc Approach */
"0BWL33Z"  =  "1"  /* Revision of Infusion Device in Left Lung, Perc Approach */
"0BWL40Z"  =  "1"  /* Revision of Drainage Device in Left Lung, Perc Endo Approach */
"0BWL42Z"  =  "1"  /* Revision of Monitoring Device in L Lung, Perc Endo Approach */
"0BWL43Z"  =  "1"  /* Revision of Infusion Device in Left Lung, Perc Endo Approach */
"0BWL70Z"  =  "1"  /* Revision of Drainage Device in Left Lung, Via Opening */
"0BWL72Z"  =  "1"  /* Revision of Monitoring Device in Left Lung, Via Opening */
"0BWL73Z"  =  "1"  /* Revision of Infusion Device in Left Lung, Via Opening */
"0BWL80Z"  =  "1"  /* Revision of Drainage Device in Left Lung, Endo */
"0BWL82Z"  =  "1"  /* Revision of Monitoring Device in Left Lung, Endo */
"0BWL83Z"  =  "1"  /* Revision of Infusion Device in Left Lung, Endo */
"0BWT00Z"  =  "1"  /* Revision of Drainage Device in Diaphragm, Open Approach */
"0BWT02Z"  =  "1"  /* Revision of Monitoring Device in Diaphragm, Open Approach */
"0BWT07Z"  =  "1"  /* Revision of Autol Sub in Diaphragm, Open Approach */
"0BWT0JZ"  =  "1"  /* Revision of Synthetic Substitute in Diaphragm, Open Approach */
"0BWT0KZ"  =  "1"  /* Revision of Nonaut Sub in Diaphragm, Open Approach */
"0BWT0MZ"  =  "1"  /* Revision of Diaphragm Lead in Diaphragm, Open Approach */
"0BWT30Z"  =  "1"  /* Revision of Drainage Device in Diaphragm, Perc Approach */
"0BWT32Z"  =  "1"  /* Revision of Monitoring Device in Diaphragm, Perc Approach */
"0BWT37Z"  =  "1"  /* Revision of Autol Sub in Diaphragm, Perc Approach */
"0BWT3JZ"  =  "1"  /* Revision of Synthetic Substitute in Diaphragm, Perc Approach */
"0BWT3KZ"  =  "1"  /* Revision of Nonaut Sub in Diaphragm, Perc Approach */
"0BWT3MZ"  =  "1"  /* Revision of Diaphragm Lead in Diaphragm, Perc Approach */
"0BWT40Z"  =  "1"  /* Revision of Drainage Device in Diaphragm, Perc Endo Approach */
"0BWT42Z"  =  "1"  /* Revision of Monitor Dev in Diaphragm, Perc Endo Approach */
"0BWT47Z"  =  "1"  /* Revision of Autol Sub in Diaphragm, Perc Endo Approach */
"0BWT4JZ"  =  "1"  /* Revision of Synth Sub in Diaphragm, Perc Endo Approach */
"0BWT4KZ"  =  "1"  /* Revision of Nonaut Sub in Diaphragm, Perc Endo Approach */
"0BWT4MZ"  =  "1"  /* Revision of Diaphragm Lead in Diaphragm, Perc Endo Approach */
"0BWT70Z"  =  "1"  /* Revision of Drainage Device in Diaphragm, Via Opening */
"0BWT72Z"  =  "1"  /* Revision of Monitoring Device in Diaphragm, Via Opening */
"0BWT77Z"  =  "1"  /* Revision of Autol Sub in Diaphragm, Via Opening */
"0BWT7JZ"  =  "1"  /* Revision of Synthetic Substitute in Diaphragm, Via Opening */
"0BWT7KZ"  =  "1"  /* Revision of Nonaut Sub in Diaphragm, Via Opening */
"0BWT7MZ"  =  "1"  /* Revision of Diaphragm Lead in Diaphragm, Via Opening */
"0BWT80Z"  =  "1"  /* Revision of Drainage Device in Diaphragm, Endo */
"0BWT82Z"  =  "1"  /* Revision of Monitoring Device in Diaphragm, Endo */
"0BWT87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Diaphragm, Endo */
"0BWT8JZ"  =  "1"  /* Revision of Synthetic Substitute in Diaphragm, Endo */
"0BWT8KZ"  =  "1"  /* Revision of Nonaut Sub in Diaphragm, Endo */
"0BWT8MZ"  =  "1"  /* Revision of Diaphragmatic Pacemaker Lead in Diaphragm, Endo */
"0BYC0Z0"  =  "1"  /* Transplantation of R Up Lung Lobe, Allogen, Open Approach */
"0BYC0Z1"  =  "1"  /* Transplantation of R Up Lung Lobe, Syngen, Open Approach */
"0BYC0Z2"  =  "1"  /* Transplantation of R Up Lung Lobe, Zooplast, Open Approach */
"0BYD0Z0"  =  "1"  /* Transplantation of R Mid Lung Lobe, Allogen, Open Approach */
"0BYD0Z1"  =  "1"  /* Transplantation of R Mid Lung Lobe, Syngen, Open Approach */
"0BYD0Z2"  =  "1"  /* Transplantation of R Mid Lung Lobe, Zooplast, Open Approach */
"0BYF0Z0"  =  "1"  /* Transplantation of R Low Lung Lobe, Allogen, Open Approach */
"0BYF0Z1"  =  "1"  /* Transplantation of R Low Lung Lobe, Syngen, Open Approach */
"0BYF0Z2"  =  "1"  /* Transplantation of R Low Lung Lobe, Zooplast, Open Approach */
"0BYG0Z0"  =  "1"  /* Transplantation of L Up Lung Lobe, Allogen, Open Approach */
"0BYG0Z1"  =  "1"  /* Transplantation of L Up Lung Lobe, Syngen, Open Approach */
"0BYG0Z2"  =  "1"  /* Transplantation of L Up Lung Lobe, Zooplast, Open Approach */
"0BYH0Z0"  =  "1"  /* Transplantation of Lung Lingula, Allogeneic, Open Approach */
"0BYH0Z1"  =  "1"  /* Transplantation of Lung Lingula, Syngeneic, Open Approach */
"0BYH0Z2"  =  "1"  /* Transplantation of Lung Lingula, Zooplastic, Open Approach */
"0BYJ0Z0"  =  "1"  /* Transplantation of L Low Lung Lobe, Allogen, Open Approach */
"0BYJ0Z1"  =  "1"  /* Transplantation of L Low Lung Lobe, Syngen, Open Approach */
"0BYJ0Z2"  =  "1"  /* Transplantation of L Low Lung Lobe, Zooplast, Open Approach */
"0BYK0Z0"  =  "1"  /* Transplantation of Right Lung, Allogeneic, Open Approach */
"0BYK0Z1"  =  "1"  /* Transplantation of Right Lung, Syngeneic, Open Approach */
"0BYK0Z2"  =  "1"  /* Transplantation of Right Lung, Zooplastic, Open Approach */
"0BYL0Z0"  =  "1"  /* Transplantation of Left Lung, Allogeneic, Open Approach */
"0BYL0Z1"  =  "1"  /* Transplantation of Left Lung, Syngeneic, Open Approach */
"0BYL0Z2"  =  "1"  /* Transplantation of Left Lung, Zooplastic, Open Approach */
"0BYM0Z0"  =  "1"  /* Transplantation of Bilateral Lungs, Allogen, Open Approach */
"0BYM0Z1"  =  "1"  /* Transplantation of Bilateral Lungs, Syngeneic, Open Approach */
"0BYM0Z2"  =  "1"  /* Transplantation of Bilateral Lungs, Zooplast, Open Approach */
"0C00X7Z"  =  "1"  /* Alteration of Upper Lip with Autol Sub, Extern Approach */
"0C00XJZ"  =  "1"  /* Alteration of Upper Lip with Synth Sub, Extern Approach */
"0C00XKZ"  =  "1"  /* Alteration of Upper Lip with Nonaut Sub, Extern Approach */
"0C00XZZ"  =  "1"  /* Alteration of Upper Lip, External Approach */
"0C01X7Z"  =  "1"  /* Alteration of Lower Lip with Autol Sub, Extern Approach */
"0C01XJZ"  =  "1"  /* Alteration of Lower Lip with Synth Sub, Extern Approach */
"0C01XKZ"  =  "1"  /* Alteration of Lower Lip with Nonaut Sub, Extern Approach */
"0C01XZZ"  =  "1"  /* Alteration of Lower Lip, External Approach */
"0C500ZZ"  =  "1"  /* Destruction of Upper Lip, Open Approach */
"0C503ZZ"  =  "1"  /* Destruction of Upper Lip, Percutaneous Approach */
"0C50XZZ"  =  "1"  /* Destruction of Upper Lip, External Approach */
"0C510ZZ"  =  "1"  /* Destruction of Lower Lip, Open Approach */
"0C513ZZ"  =  "1"  /* Destruction of Lower Lip, Percutaneous Approach */
"0C51XZZ"  =  "1"  /* Destruction of Lower Lip, External Approach */
"0C520ZZ"  =  "1"  /* Destruction of Hard Palate, Open Approach */
"0C523ZZ"  =  "1"  /* Destruction of Hard Palate, Percutaneous Approach */
"0C52XZZ"  =  "1"  /* Destruction of Hard Palate, External Approach */
"0C530ZZ"  =  "1"  /* Destruction of Soft Palate, Open Approach */
"0C533ZZ"  =  "1"  /* Destruction of Soft Palate, Percutaneous Approach */
"0C53XZZ"  =  "1"  /* Destruction of Soft Palate, External Approach */
"0C540ZZ"  =  "1"  /* Destruction of Buccal Mucosa, Open Approach */
"0C543ZZ"  =  "1"  /* Destruction of Buccal Mucosa, Percutaneous Approach */
"0C54XZZ"  =  "1"  /* Destruction of Buccal Mucosa, External Approach */
"0C570ZZ"  =  "1"  /* Destruction of Tongue, Open Approach */
"0C573ZZ"  =  "1"  /* Destruction of Tongue, Percutaneous Approach */
"0C57XZZ"  =  "1"  /* Destruction of Tongue, External Approach */
"0C580ZZ"  =  "1"  /* Destruction of Right Parotid Gland, Open Approach */
"0C583ZZ"  =  "1"  /* Destruction of Right Parotid Gland, Percutaneous Approach */
"0C590ZZ"  =  "1"  /* Destruction of Left Parotid Gland, Open Approach */
"0C593ZZ"  =  "1"  /* Destruction of Left Parotid Gland, Percutaneous Approach */
"0C5B0ZZ"  =  "1"  /* Destruction of Right Parotid Duct, Open Approach */
"0C5B3ZZ"  =  "1"  /* Destruction of Right Parotid Duct, Percutaneous Approach */
"0C5C0ZZ"  =  "1"  /* Destruction of Left Parotid Duct, Open Approach */
"0C5C3ZZ"  =  "1"  /* Destruction of Left Parotid Duct, Percutaneous Approach */
"0C5D0ZZ"  =  "1"  /* Destruction of Right Sublingual Gland, Open Approach */
"0C5D3ZZ"  =  "1"  /* Destruction of Right Sublingual Gland, Percutaneous Approach */
"0C5F0ZZ"  =  "1"  /* Destruction of Left Sublingual Gland, Open Approach */
"0C5F3ZZ"  =  "1"  /* Destruction of Left Sublingual Gland, Percutaneous Approach */
"0C5G0ZZ"  =  "1"  /* Destruction of Right Submaxillary Gland, Open Approach */
"0C5G3ZZ"  =  "1"  /* Destruction of Right Submaxillary Gland, Perc Approach */
"0C5H0ZZ"  =  "1"  /* Destruction of Left Submaxillary Gland, Open Approach */
"0C5H3ZZ"  =  "1"  /* Destruction of Left Submaxillary Gland, Perc Approach */
"0C5J0ZZ"  =  "1"  /* Destruction of Minor Salivary Gland, Open Approach */
"0C5J3ZZ"  =  "1"  /* Destruction of Minor Salivary Gland, Percutaneous Approach */
"0C5M0ZZ"  =  "1"  /* Destruction of Pharynx, Open Approach */
"0C5M3ZZ"  =  "1"  /* Destruction of Pharynx, Percutaneous Approach */
"0C5M4ZZ"  =  "1"  /* Destruction of Pharynx, Percutaneous Endoscopic Approach */
"0C5M7ZZ"  =  "1"  /* Destruction of Pharynx, Via Natural or Artificial Opening */
"0C5M8ZZ"  =  "1"  /* Destruction of Pharynx, Endo */
"0C5N0ZZ"  =  "1"  /* Destruction of Uvula, Open Approach */
"0C5N3ZZ"  =  "1"  /* Destruction of Uvula, Percutaneous Approach */
"0C5NXZZ"  =  "1"  /* Destruction of Uvula, External Approach */
"0C5P0ZZ"  =  "1"  /* Destruction of Tonsils, Open Approach */
"0C5P3ZZ"  =  "1"  /* Destruction of Tonsils, Percutaneous Approach */
"0C5PXZZ"  =  "1"  /* Destruction of Tonsils, External Approach */
"0C5Q0ZZ"  =  "1"  /* Destruction of Adenoids, Open Approach */
"0C5Q3ZZ"  =  "1"  /* Destruction of Adenoids, Percutaneous Approach */
"0C5QXZZ"  =  "1"  /* Destruction of Adenoids, External Approach */
"0C5R0ZZ"  =  "1"  /* Destruction of Epiglottis, Open Approach */
"0C5R3ZZ"  =  "1"  /* Destruction of Epiglottis, Percutaneous Approach */
"0C5R4ZZ"  =  "1"  /* Destruction of Epiglottis, Percutaneous Endoscopic Approach */
"0C5R7ZZ"  =  "1"  /* Destruction of Epiglottis, Via Natural or Artificial Opening */
"0C5R8ZZ"  =  "1"  /* Destruction of Epiglottis, Endo */
"0C5S0ZZ"  =  "1"  /* Destruction of Larynx, Open Approach */
"0C5S3ZZ"  =  "1"  /* Destruction of Larynx, Percutaneous Approach */
"0C5S4ZZ"  =  "1"  /* Destruction of Larynx, Percutaneous Endoscopic Approach */
"0C5S7ZZ"  =  "1"  /* Destruction of Larynx, Via Natural or Artificial Opening */
"0C5S8ZZ"  =  "1"  /* Destruction of Larynx, Endo */
"0C5T0ZZ"  =  "1"  /* Destruction of Right Vocal Cord, Open Approach */
"0C5T3ZZ"  =  "1"  /* Destruction of Right Vocal Cord, Percutaneous Approach */
"0C5T4ZZ"  =  "1"  /* Destruction of Right Vocal Cord, Perc Endo Approach */
"0C5T7ZZ"  =  "1"  /* Destruction of Right Vocal Cord, Via Opening */
"0C5T8ZZ"  =  "1"  /* Destruction of Right Vocal Cord, Endo */
"0C5V0ZZ"  =  "1"  /* Destruction of Left Vocal Cord, Open Approach */
"0C5V3ZZ"  =  "1"  /* Destruction of Left Vocal Cord, Percutaneous Approach */
"0C5V4ZZ"  =  "1"  /* Destruction of Left Vocal Cord, Perc Endo Approach */
"0C5V7ZZ"  =  "1"  /* Destruction of Left Vocal Cord, Via Opening */
"0C5V8ZZ"  =  "1"  /* Destruction of Left Vocal Cord, Endo */
"0C7S0DZ"  =  "1"  /* Dilation of Larynx with Intraluminal Device, Open Approach */
"0C7S0ZZ"  =  "1"  /* Dilation of Larynx, Open Approach */
"0C7S3DZ"  =  "1"  /* Dilation of Larynx with Intraluminal Device, Perc Approach */
"0C7S3ZZ"  =  "1"  /* Dilation of Larynx, Percutaneous Approach */
"0C7S4DZ"  =  "1"  /* Dilation of Larynx with Intralum Dev, Perc Endo Approach */
"0C7S4ZZ"  =  "1"  /* Dilation of Larynx, Percutaneous Endoscopic Approach */
"0C7S7DZ"  =  "1"  /* Dilation of Larynx with Intraluminal Device, Via Opening */
"0C7S7ZZ"  =  "1"  /* Dilation of Larynx, Via Natural or Artificial Opening */
"0C7S8DZ"  =  "1"  /* Dilation of Larynx with Intraluminal Device, Endo */
"0C7S8ZZ"  =  "1"  /* Dilation of Larynx, Endo */
"0C9000Z"  =  "1"  /* Drainage of Upper Lip with Drainage Device, Open Approach */
"0C900ZZ"  =  "1"  /* Drainage of Upper Lip, Open Approach */
"0C9030Z"  =  "1"  /* Drainage of Upper Lip with Drainage Device, Perc Approach */
"0C903ZZ"  =  "1"  /* Drainage of Upper Lip, Percutaneous Approach */
"0C90X0Z"  =  "1"  /* Drainage of Upper Lip with Drainage Device, Extern Approach */
"0C90XZZ"  =  "1"  /* Drainage of Upper Lip, External Approach */
"0C9100Z"  =  "1"  /* Drainage of Lower Lip with Drainage Device, Open Approach */
"0C910ZZ"  =  "1"  /* Drainage of Lower Lip, Open Approach */
"0C9130Z"  =  "1"  /* Drainage of Lower Lip with Drainage Device, Perc Approach */
"0C913ZZ"  =  "1"  /* Drainage of Lower Lip, Percutaneous Approach */
"0C91X0Z"  =  "1"  /* Drainage of Lower Lip with Drainage Device, Extern Approach */
"0C91XZZ"  =  "1"  /* Drainage of Lower Lip, External Approach */
"0C9200Z"  =  "1"  /* Drainage of Hard Palate with Drainage Device, Open Approach */
"0C920ZX"  =  "1"  /* Drainage of Hard Palate, Open Approach, Diagnostic */
"0C920ZZ"  =  "1"  /* Drainage of Hard Palate, Open Approach */
"0C9230Z"  =  "1"  /* Drainage of Hard Palate with Drainage Device, Perc Approach */
"0C923ZX"  =  "1"  /* Drainage of Hard Palate, Percutaneous Approach, Diagnostic */
"0C923ZZ"  =  "1"  /* Drainage of Hard Palate, Percutaneous Approach */
"0C92X0Z"  =  "1"  /* Drainage of Hard Palate with Drain Dev, Extern Approach */
"0C92XZX"  =  "1"  /* Drainage of Hard Palate, External Approach, Diagnostic */
"0C92XZZ"  =  "1"  /* Drainage of Hard Palate, External Approach */
"0C9300Z"  =  "1"  /* Drainage of Soft Palate with Drainage Device, Open Approach */
"0C930ZX"  =  "1"  /* Drainage of Soft Palate, Open Approach, Diagnostic */
"0C930ZZ"  =  "1"  /* Drainage of Soft Palate, Open Approach */
"0C9330Z"  =  "1"  /* Drainage of Soft Palate with Drainage Device, Perc Approach */
"0C933ZX"  =  "1"  /* Drainage of Soft Palate, Percutaneous Approach, Diagnostic */
"0C933ZZ"  =  "1"  /* Drainage of Soft Palate, Percutaneous Approach */
"0C93X0Z"  =  "1"  /* Drainage of Soft Palate with Drain Dev, Extern Approach */
"0C93XZX"  =  "1"  /* Drainage of Soft Palate, External Approach, Diagnostic */
"0C93XZZ"  =  "1"  /* Drainage of Soft Palate, External Approach */
"0C9400Z"  =  "1"  /* Drainage of Buccal Mucosa with Drain Dev, Open Approach */
"0C940ZZ"  =  "1"  /* Drainage of Buccal Mucosa, Open Approach */
"0C9430Z"  =  "1"  /* Drainage of Buccal Mucosa with Drain Dev, Perc Approach */
"0C943ZZ"  =  "1"  /* Drainage of Buccal Mucosa, Percutaneous Approach */
"0C94X0Z"  =  "1"  /* Drainage of Buccal Mucosa with Drain Dev, Extern Approach */
"0C94XZZ"  =  "1"  /* Drainage of Buccal Mucosa, External Approach */
"0C9700Z"  =  "1"  /* Drainage of Tongue with Drainage Device, Open Approach */
"0C970ZX"  =  "1"  /* Drainage of Tongue, Open Approach, Diagnostic */
"0C970ZZ"  =  "1"  /* Drainage of Tongue, Open Approach */
"0C9730Z"  =  "1"  /* Drainage of Tongue with Drainage Device, Perc Approach */
"0C973ZZ"  =  "1"  /* Drainage of Tongue, Percutaneous Approach */
"0C97X0Z"  =  "1"  /* Drainage of Tongue with Drainage Device, External Approach */
"0C97XZZ"  =  "1"  /* Drainage of Tongue, External Approach */
"0C980ZX"  =  "1"  /* Drainage of Right Parotid Gland, Open Approach, Diagnostic */
"0C990ZX"  =  "1"  /* Drainage of Left Parotid Gland, Open Approach, Diagnostic */
"0C9B0ZX"  =  "1"  /* Drainage of Right Parotid Duct, Open Approach, Diagnostic */
"0C9C0ZX"  =  "1"  /* Drainage of Left Parotid Duct, Open Approach, Diagnostic */
"0C9D0ZX"  =  "1"  /* Drainage of Right Sublingual Gland, Open Approach, Diagn */
"0C9F0ZX"  =  "1"  /* Drainage of Left Sublingual Gland, Open Approach, Diagnostic */
"0C9G0ZX"  =  "1"  /* Drainage of Right Submaxillary Gland, Open Approach, Diagn */
"0C9H0ZX"  =  "1"  /* Drainage of Left Submaxillary Gland, Open Approach, Diagn */
"0C9J0ZX"  =  "1"  /* Drainage of Minor Salivary Gland, Open Approach, Diagnostic */
"0C9M00Z"  =  "1"  /* Drainage of Pharynx with Drainage Device, Open Approach */
"0C9M0ZZ"  =  "1"  /* Drainage of Pharynx, Open Approach */
"0C9M30Z"  =  "1"  /* Drainage of Pharynx with Drainage Device, Perc Approach */
"0C9M3ZZ"  =  "1"  /* Drainage of Pharynx, Percutaneous Approach */
"0C9M40Z"  =  "1"  /* Drainage of Pharynx with Drainage Device, Perc Endo Approach */
"0C9M4ZZ"  =  "1"  /* Drainage of Pharynx, Percutaneous Endoscopic Approach */
"0C9M70Z"  =  "1"  /* Drainage of Pharynx with Drainage Device, Via Opening */
"0C9M7ZZ"  =  "1"  /* Drainage of Pharynx, Via Natural or Artificial Opening */
"0C9M80Z"  =  "1"  /* Drainage of Pharynx with Drainage Device, Endo */
"0C9M8ZZ"  =  "1"  /* Drainage of Pharynx, Endo */
"0C9N00Z"  =  "1"  /* Drainage of Uvula with Drainage Device, Open Approach */
"0C9N0ZX"  =  "1"  /* Drainage of Uvula, Open Approach, Diagnostic */
"0C9N0ZZ"  =  "1"  /* Drainage of Uvula, Open Approach */
"0C9N30Z"  =  "1"  /* Drainage of Uvula with Drainage Device, Perc Approach */
"0C9N3ZX"  =  "1"  /* Drainage of Uvula, Percutaneous Approach, Diagnostic */
"0C9N3ZZ"  =  "1"  /* Drainage of Uvula, Percutaneous Approach */
"0C9NX0Z"  =  "1"  /* Drainage of Uvula with Drainage Device, External Approach */
"0C9NXZX"  =  "1"  /* Drainage of Uvula, External Approach, Diagnostic */
"0C9NXZZ"  =  "1"  /* Drainage of Uvula, External Approach */
"0C9P00Z"  =  "1"  /* Drainage of Tonsils with Drainage Device, Open Approach */
"0C9P0ZX"  =  "1"  /* Drainage of Tonsils, Open Approach, Diagnostic */
"0C9P0ZZ"  =  "1"  /* Drainage of Tonsils, Open Approach */
"0C9P30Z"  =  "1"  /* Drainage of Tonsils with Drainage Device, Perc Approach */
"0C9P3ZX"  =  "1"  /* Drainage of Tonsils, Percutaneous Approach, Diagnostic */
"0C9P3ZZ"  =  "1"  /* Drainage of Tonsils, Percutaneous Approach */
"0C9PX0Z"  =  "1"  /* Drainage of Tonsils with Drainage Device, External Approach */
"0C9PXZX"  =  "1"  /* Drainage of Tonsils, External Approach, Diagnostic */
"0C9PXZZ"  =  "1"  /* Drainage of Tonsils, External Approach */
"0C9Q00Z"  =  "1"  /* Drainage of Adenoids with Drainage Device, Open Approach */
"0C9Q0ZX"  =  "1"  /* Drainage of Adenoids, Open Approach, Diagnostic */
"0C9Q0ZZ"  =  "1"  /* Drainage of Adenoids, Open Approach */
"0C9Q30Z"  =  "1"  /* Drainage of Adenoids with Drainage Device, Perc Approach */
"0C9Q3ZX"  =  "1"  /* Drainage of Adenoids, Percutaneous Approach, Diagnostic */
"0C9Q3ZZ"  =  "1"  /* Drainage of Adenoids, Percutaneous Approach */
"0C9QX0Z"  =  "1"  /* Drainage of Adenoids with Drainage Device, External Approach */
"0C9QXZX"  =  "1"  /* Drainage of Adenoids, External Approach, Diagnostic */
"0C9QXZZ"  =  "1"  /* Drainage of Adenoids, External Approach */
"0C9R00Z"  =  "1"  /* Drainage of Epiglottis with Drainage Device, Open Approach */
"0C9R0ZX"  =  "1"  /* Drainage of Epiglottis, Open Approach, Diagnostic */
"0C9R0ZZ"  =  "1"  /* Drainage of Epiglottis, Open Approach */
"0C9R30Z"  =  "1"  /* Drainage of Epiglottis with Drainage Device, Perc Approach */
"0C9R3ZZ"  =  "1"  /* Drainage of Epiglottis, Percutaneous Approach */
"0C9R40Z"  =  "1"  /* Drainage of Epiglottis with Drain Dev, Perc Endo Approach */
"0C9R4ZZ"  =  "1"  /* Drainage of Epiglottis, Percutaneous Endoscopic Approach */
"0C9R70Z"  =  "1"  /* Drainage of Epiglottis with Drainage Device, Via Opening */
"0C9R7ZZ"  =  "1"  /* Drainage of Epiglottis, Via Natural or Artificial Opening */
"0C9R80Z"  =  "1"  /* Drainage of Epiglottis with Drainage Device, Endo */
"0C9R8ZZ"  =  "1"  /* Drainage of Epiglottis, Endo */
"0C9S00Z"  =  "1"  /* Drainage of Larynx with Drainage Device, Open Approach */
"0C9S0ZX"  =  "1"  /* Drainage of Larynx, Open Approach, Diagnostic */
"0C9S0ZZ"  =  "1"  /* Drainage of Larynx, Open Approach */
"0C9S30Z"  =  "1"  /* Drainage of Larynx with Drainage Device, Perc Approach */
"0C9S3ZZ"  =  "1"  /* Drainage of Larynx, Percutaneous Approach */
"0C9S40Z"  =  "1"  /* Drainage of Larynx with Drainage Device, Perc Endo Approach */
"0C9S4ZZ"  =  "1"  /* Drainage of Larynx, Percutaneous Endoscopic Approach */
"0C9S70Z"  =  "1"  /* Drainage of Larynx with Drainage Device, Via Opening */
"0C9S7ZZ"  =  "1"  /* Drainage of Larynx, Via Natural or Artificial Opening */
"0C9S80Z"  =  "1"  /* Drainage of Larynx with Drainage Device, Endo */
"0C9S8ZZ"  =  "1"  /* Drainage of Larynx, Endo */
"0C9T00Z"  =  "1"  /* Drainage of Right Vocal Cord with Drain Dev, Open Approach */
"0C9T0ZX"  =  "1"  /* Drainage of Right Vocal Cord, Open Approach, Diagnostic */
"0C9T0ZZ"  =  "1"  /* Drainage of Right Vocal Cord, Open Approach */
"0C9T30Z"  =  "1"  /* Drainage of Right Vocal Cord with Drain Dev, Perc Approach */
"0C9T3ZZ"  =  "1"  /* Drainage of Right Vocal Cord, Percutaneous Approach */
"0C9T40Z"  =  "1"  /* Drainage of R Vocal Cord with Drain Dev, Perc Endo Approach */
"0C9T4ZZ"  =  "1"  /* Drainage of Right Vocal Cord, Perc Endo Approach */
"0C9T70Z"  =  "1"  /* Drainage of Right Vocal Cord with Drain Dev, Via Opening */
"0C9T7ZZ"  =  "1"  /* Drainage of Right Vocal Cord, Via Opening */
"0C9T80Z"  =  "1"  /* Drainage of Right Vocal Cord with Drainage Device, Endo */
"0C9T8ZZ"  =  "1"  /* Drainage of Right Vocal Cord, Endo */
"0C9V00Z"  =  "1"  /* Drainage of Left Vocal Cord with Drain Dev, Open Approach */
"0C9V0ZX"  =  "1"  /* Drainage of Left Vocal Cord, Open Approach, Diagnostic */
"0C9V0ZZ"  =  "1"  /* Drainage of Left Vocal Cord, Open Approach */
"0C9V30Z"  =  "1"  /* Drainage of Left Vocal Cord with Drain Dev, Perc Approach */
"0C9V3ZZ"  =  "1"  /* Drainage of Left Vocal Cord, Percutaneous Approach */
"0C9V40Z"  =  "1"  /* Drainage of L Vocal Cord with Drain Dev, Perc Endo Approach */
"0C9V4ZZ"  =  "1"  /* Drainage of Left Vocal Cord, Perc Endo Approach */
"0C9V70Z"  =  "1"  /* Drainage of Left Vocal Cord with Drain Dev, Via Opening */
"0C9V7ZZ"  =  "1"  /* Drainage of Left Vocal Cord, Via Opening */
"0C9V80Z"  =  "1"  /* Drainage of Left Vocal Cord with Drainage Device, Endo */
"0C9V8ZZ"  =  "1"  /* Drainage of Left Vocal Cord, Endo */
"0CB00ZZ"  =  "1"  /* Excision of Upper Lip, Open Approach */
"0CB03ZZ"  =  "1"  /* Excision of Upper Lip, Percutaneous Approach */
"0CB0XZZ"  =  "1"  /* Excision of Upper Lip, External Approach */
"0CB10ZZ"  =  "1"  /* Excision of Lower Lip, Open Approach */
"0CB13ZZ"  =  "1"  /* Excision of Lower Lip, Percutaneous Approach */
"0CB1XZZ"  =  "1"  /* Excision of Lower Lip, External Approach */
"0CB20ZX"  =  "1"  /* Excision of Hard Palate, Open Approach, Diagnostic */
"0CB20ZZ"  =  "1"  /* Excision of Hard Palate, Open Approach */
"0CB23ZX"  =  "1"  /* Excision of Hard Palate, Percutaneous Approach, Diagnostic */
"0CB23ZZ"  =  "1"  /* Excision of Hard Palate, Percutaneous Approach */
"0CB2XZX"  =  "1"  /* Excision of Hard Palate, External Approach, Diagnostic */
"0CB2XZZ"  =  "1"  /* Excision of Hard Palate, External Approach */
"0CB30ZX"  =  "1"  /* Excision of Soft Palate, Open Approach, Diagnostic */
"0CB30ZZ"  =  "1"  /* Excision of Soft Palate, Open Approach */
"0CB33ZX"  =  "1"  /* Excision of Soft Palate, Percutaneous Approach, Diagnostic */
"0CB33ZZ"  =  "1"  /* Excision of Soft Palate, Percutaneous Approach */
"0CB3XZX"  =  "1"  /* Excision of Soft Palate, External Approach, Diagnostic */
"0CB3XZZ"  =  "1"  /* Excision of Soft Palate, External Approach */
"0CB40ZZ"  =  "1"  /* Excision of Buccal Mucosa, Open Approach */
"0CB43ZZ"  =  "1"  /* Excision of Buccal Mucosa, Percutaneous Approach */
"0CB4XZZ"  =  "1"  /* Excision of Buccal Mucosa, External Approach */
"0CB70ZX"  =  "1"  /* Excision of Tongue, Open Approach, Diagnostic */
"0CB70ZZ"  =  "1"  /* Excision of Tongue, Open Approach */
"0CB73ZZ"  =  "1"  /* Excision of Tongue, Percutaneous Approach */
"0CB7XZZ"  =  "1"  /* Excision of Tongue, External Approach */
"0CB80ZX"  =  "1"  /* Excision of Right Parotid Gland, Open Approach, Diagnostic */
"0CB80ZZ"  =  "1"  /* Excision of Right Parotid Gland, Open Approach */
"0CB83ZZ"  =  "1"  /* Excision of Right Parotid Gland, Percutaneous Approach */
"0CB90ZX"  =  "1"  /* Excision of Left Parotid Gland, Open Approach, Diagnostic */
"0CB90ZZ"  =  "1"  /* Excision of Left Parotid Gland, Open Approach */
"0CB93ZZ"  =  "1"  /* Excision of Left Parotid Gland, Percutaneous Approach */
"0CBB0ZX"  =  "1"  /* Excision of Right Parotid Duct, Open Approach, Diagnostic */
"0CBB0ZZ"  =  "1"  /* Excision of Right Parotid Duct, Open Approach */
"0CBB3ZZ"  =  "1"  /* Excision of Right Parotid Duct, Percutaneous Approach */
"0CBC0ZX"  =  "1"  /* Excision of Left Parotid Duct, Open Approach, Diagnostic */
"0CBC0ZZ"  =  "1"  /* Excision of Left Parotid Duct, Open Approach */
"0CBC3ZZ"  =  "1"  /* Excision of Left Parotid Duct, Percutaneous Approach */
"0CBD0ZX"  =  "1"  /* Excision of Right Sublingual Gland, Open Approach, Diagn */
"0CBD0ZZ"  =  "1"  /* Excision of Right Sublingual Gland, Open Approach */
"0CBD3ZZ"  =  "1"  /* Excision of Right Sublingual Gland, Percutaneous Approach */
"0CBF0ZX"  =  "1"  /* Excision of Left Sublingual Gland, Open Approach, Diagnostic */
"0CBF0ZZ"  =  "1"  /* Excision of Left Sublingual Gland, Open Approach */
"0CBF3ZZ"  =  "1"  /* Excision of Left Sublingual Gland, Percutaneous Approach */
"0CBG0ZX"  =  "1"  /* Excision of Right Submaxillary Gland, Open Approach, Diagn */
"0CBG0ZZ"  =  "1"  /* Excision of Right Submaxillary Gland, Open Approach */
"0CBG3ZZ"  =  "1"  /* Excision of Right Submaxillary Gland, Percutaneous Approach */
"0CBH0ZX"  =  "1"  /* Excision of Left Submaxillary Gland, Open Approach, Diagn */
"0CBH0ZZ"  =  "1"  /* Excision of Left Submaxillary Gland, Open Approach */
"0CBH3ZZ"  =  "1"  /* Excision of Left Submaxillary Gland, Percutaneous Approach */
"0CBJ0ZX"  =  "1"  /* Excision of Minor Salivary Gland, Open Approach, Diagnostic */
"0CBJ0ZZ"  =  "1"  /* Excision of Minor Salivary Gland, Open Approach */
"0CBJ3ZZ"  =  "1"  /* Excision of Minor Salivary Gland, Percutaneous Approach */
"0CBM0ZZ"  =  "1"  /* Excision of Pharynx, Open Approach */
"0CBM3ZZ"  =  "1"  /* Excision of Pharynx, Percutaneous Approach */
"0CBM4ZZ"  =  "1"  /* Excision of Pharynx, Percutaneous Endoscopic Approach */
"0CBM7ZZ"  =  "1"  /* Excision of Pharynx, Via Natural or Artificial Opening */
"0CBM8ZZ"  =  "1"  /* Excision of Pharynx, Endo */
"0CBN0ZX"  =  "1"  /* Excision of Uvula, Open Approach, Diagnostic */
"0CBN0ZZ"  =  "1"  /* Excision of Uvula, Open Approach */
"0CBN3ZX"  =  "1"  /* Excision of Uvula, Percutaneous Approach, Diagnostic */
"0CBN3ZZ"  =  "1"  /* Excision of Uvula, Percutaneous Approach */
"0CBNXZX"  =  "1"  /* Excision of Uvula, External Approach, Diagnostic */
"0CBNXZZ"  =  "1"  /* Excision of Uvula, External Approach */
"0CBP0ZX"  =  "1"  /* Excision of Tonsils, Open Approach, Diagnostic */
"0CBP0ZZ"  =  "1"  /* Excision of Tonsils, Open Approach */
"0CBP3ZX"  =  "1"  /* Excision of Tonsils, Percutaneous Approach, Diagnostic */
"0CBP3ZZ"  =  "1"  /* Excision of Tonsils, Percutaneous Approach */
"0CBPXZX"  =  "1"  /* Excision of Tonsils, External Approach, Diagnostic */
"0CBPXZZ"  =  "1"  /* Excision of Tonsils, External Approach */
"0CBQ0ZX"  =  "1"  /* Excision of Adenoids, Open Approach, Diagnostic */
"0CBQ0ZZ"  =  "1"  /* Excision of Adenoids, Open Approach */
"0CBQ3ZX"  =  "1"  /* Excision of Adenoids, Percutaneous Approach, Diagnostic */
"0CBQ3ZZ"  =  "1"  /* Excision of Adenoids, Percutaneous Approach */
"0CBQXZX"  =  "1"  /* Excision of Adenoids, External Approach, Diagnostic */
"0CBQXZZ"  =  "1"  /* Excision of Adenoids, External Approach */
"0CBR0ZX"  =  "1"  /* Excision of Epiglottis, Open Approach, Diagnostic */
"0CBR0ZZ"  =  "1"  /* Excision of Epiglottis, Open Approach */
"0CBR3ZZ"  =  "1"  /* Excision of Epiglottis, Percutaneous Approach */
"0CBR4ZZ"  =  "1"  /* Excision of Epiglottis, Percutaneous Endoscopic Approach */
"0CBR7ZZ"  =  "1"  /* Excision of Epiglottis, Via Natural or Artificial Opening */
"0CBR8ZZ"  =  "1"  /* Excision of Epiglottis, Endo */
"0CBS0ZX"  =  "1"  /* Excision of Larynx, Open Approach, Diagnostic */
"0CBS0ZZ"  =  "1"  /* Excision of Larynx, Open Approach */
"0CBS3ZZ"  =  "1"  /* Excision of Larynx, Percutaneous Approach */
"0CBS4ZZ"  =  "1"  /* Excision of Larynx, Percutaneous Endoscopic Approach */
"0CBS7ZZ"  =  "1"  /* Excision of Larynx, Via Natural or Artificial Opening */
"0CBS8ZZ"  =  "1"  /* Excision of Larynx, Endo */
"0CBT0ZX"  =  "1"  /* Excision of Right Vocal Cord, Open Approach, Diagnostic */
"0CBT0ZZ"  =  "1"  /* Excision of Right Vocal Cord, Open Approach */
"0CBT3ZZ"  =  "1"  /* Excision of Right Vocal Cord, Percutaneous Approach */
"0CBT4ZZ"  =  "1"  /* Excision of Right Vocal Cord, Perc Endo Approach */
"0CBT7ZZ"  =  "1"  /* Excision of Right Vocal Cord, Via Opening */
"0CBT8ZZ"  =  "1"  /* Excision of Right Vocal Cord, Endo */
"0CBV0ZX"  =  "1"  /* Excision of Left Vocal Cord, Open Approach, Diagnostic */
"0CBV0ZZ"  =  "1"  /* Excision of Left Vocal Cord, Open Approach */
"0CBV3ZZ"  =  "1"  /* Excision of Left Vocal Cord, Percutaneous Approach */
"0CBV4ZZ"  =  "1"  /* Excision of Left Vocal Cord, Perc Endo Approach */
"0CBV7ZZ"  =  "1"  /* Excision of Left Vocal Cord, Via Opening */
"0CBV8ZZ"  =  "1"  /* Excision of Left Vocal Cord, Endo */
"0CC00ZZ"  =  "1"  /* Extirpation of Matter from Upper Lip, Open Approach */
"0CC03ZZ"  =  "1"  /* Extirpation of Matter from Upper Lip, Percutaneous Approach */
"0CC10ZZ"  =  "1"  /* Extirpation of Matter from Lower Lip, Open Approach */
"0CC13ZZ"  =  "1"  /* Extirpation of Matter from Lower Lip, Percutaneous Approach */
"0CC20ZZ"  =  "1"  /* Extirpation of Matter from Hard Palate, Open Approach */
"0CC23ZZ"  =  "1"  /* Extirpation of Matter from Hard Palate, Perc Approach */
"0CC30ZZ"  =  "1"  /* Extirpation of Matter from Soft Palate, Open Approach */
"0CC33ZZ"  =  "1"  /* Extirpation of Matter from Soft Palate, Perc Approach */
"0CC40ZZ"  =  "1"  /* Extirpation of Matter from Buccal Mucosa, Open Approach */
"0CC43ZZ"  =  "1"  /* Extirpation of Matter from Buccal Mucosa, Perc Approach */
"0CC70ZZ"  =  "1"  /* Extirpation of Matter from Tongue, Open Approach */
"0CC73ZZ"  =  "1"  /* Extirpation of Matter from Tongue, Percutaneous Approach */
"0CCM0ZZ"  =  "1"  /* Extirpation of Matter from Pharynx, Open Approach */
"0CCM3ZZ"  =  "1"  /* Extirpation of Matter from Pharynx, Percutaneous Approach */
"0CCM4ZZ"  =  "1"  /* Extirpation of Matter from Pharynx, Perc Endo Approach */
"0CCN0ZZ"  =  "1"  /* Extirpation of Matter from Uvula, Open Approach */
"0CCN3ZZ"  =  "1"  /* Extirpation of Matter from Uvula, Percutaneous Approach */
"0CCP0ZZ"  =  "1"  /* Extirpation of Matter from Tonsils, Open Approach */
"0CCP3ZZ"  =  "1"  /* Extirpation of Matter from Tonsils, Percutaneous Approach */
"0CCQ0ZZ"  =  "1"  /* Extirpation of Matter from Adenoids, Open Approach */
"0CCQ3ZZ"  =  "1"  /* Extirpation of Matter from Adenoids, Percutaneous Approach */
"0CCR0ZZ"  =  "1"  /* Extirpation of Matter from Epiglottis, Open Approach */
"0CCR3ZZ"  =  "1"  /* Extirpation of Matter from Epiglottis, Percutaneous Approach */
"0CCR4ZZ"  =  "1"  /* Extirpation of Matter from Epiglottis, Perc Endo Approach */
"0CCR7ZZ"  =  "1"  /* Extirpation of Matter from Epiglottis, Via Opening */
"0CCR8ZZ"  =  "1"  /* Extirpation of Matter from Epiglottis, Endo */
"0CCS0ZZ"  =  "1"  /* Extirpation of Matter from Larynx, Open Approach */
"0CCS3ZZ"  =  "1"  /* Extirpation of Matter from Larynx, Percutaneous Approach */
"0CCS4ZZ"  =  "1"  /* Extirpation of Matter from Larynx, Perc Endo Approach */
"0CCT0ZZ"  =  "1"  /* Extirpation of Matter from Right Vocal Cord, Open Approach */
"0CCT3ZZ"  =  "1"  /* Extirpation of Matter from Right Vocal Cord, Perc Approach */
"0CCT4ZZ"  =  "1"  /* Extirpation of Matter from R Vocal Cord, Perc Endo Approach */
"0CCT7ZZ"  =  "1"  /* Extirpation of Matter from Right Vocal Cord, Via Opening */
"0CCT8ZZ"  =  "1"  /* Extirpation of Matter from Right Vocal Cord, Endo */
"0CCV0ZZ"  =  "1"  /* Extirpation of Matter from Left Vocal Cord, Open Approach */
"0CCV3ZZ"  =  "1"  /* Extirpation of Matter from Left Vocal Cord, Perc Approach */
"0CCV4ZZ"  =  "1"  /* Extirpation of Matter from L Vocal Cord, Perc Endo Approach */
"0CCV7ZZ"  =  "1"  /* Extirpation of Matter from Left Vocal Cord, Via Opening */
"0CCV8ZZ"  =  "1"  /* Extirpation of Matter from Left Vocal Cord, Endo */
"0CDT0ZZ"  =  "1"  /* Extraction of Right Vocal Cord, Open Approach */
"0CDT3ZZ"  =  "1"  /* Extraction of Right Vocal Cord, Percutaneous Approach */
"0CDT4ZZ"  =  "1"  /* Extraction of Right Vocal Cord, Perc Endo Approach */
"0CDT7ZZ"  =  "1"  /* Extraction of Right Vocal Cord, Via Opening */
"0CDT8ZZ"  =  "1"  /* Extraction of Right Vocal Cord, Endo */
"0CDV0ZZ"  =  "1"  /* Extraction of Left Vocal Cord, Open Approach */
"0CDV3ZZ"  =  "1"  /* Extraction of Left Vocal Cord, Percutaneous Approach */
"0CDV4ZZ"  =  "1"  /* Extraction of Left Vocal Cord, Perc Endo Approach */
"0CDV7ZZ"  =  "1"  /* Extraction of Left Vocal Cord, Via Opening */
"0CDV8ZZ"  =  "1"  /* Extraction of Left Vocal Cord, Endo */
"0CH701Z"  =  "1"  /* Insertion of Radioactive Element into Tongue, Open Approach */
"0CH731Z"  =  "1"  /* Insertion of Radioactive Element into Tongue, Perc Approach */
"0CH7X1Z"  =  "1"  /* Insertion of Radioact Elem into Tongue, Extern Approach */
"0CLB0CZ"  =  "1"  /* Occlusion of R Parotid Duct with Extralum Dev, Open Approach */
"0CLB0DZ"  =  "1"  /* Occlusion of R Parotid Duct with Intralum Dev, Open Approach */
"0CLB0ZZ"  =  "1"  /* Occlusion of Right Parotid Duct, Open Approach */
"0CLB3CZ"  =  "1"  /* Occlusion of R Parotid Duct with Extralum Dev, Perc Approach */
"0CLB3DZ"  =  "1"  /* Occlusion of R Parotid Duct with Intralum Dev, Perc Approach */
"0CLB3ZZ"  =  "1"  /* Occlusion of Right Parotid Duct, Percutaneous Approach */
"0CLB4CZ"  =  "1"  /* Occlusion R Parotid Duct w Extralum Dev, Perc Endo */
"0CLB4DZ"  =  "1"  /* Occlusion R Parotid Duct w Intralum Dev, Perc Endo */
"0CLB4ZZ"  =  "1"  /* Occlusion of Right Parotid Duct, Perc Endo Approach */
"0CLB7DZ"  =  "1"  /* Occlusion of R Parotid Duct with Intralum Dev, Via Opening */
"0CLB7ZZ"  =  "1"  /* Occlusion of Right Parotid Duct, Via Opening */
"0CLB8DZ"  =  "1"  /* Occlusion of Right Parotid Duct with Intralum Dev, Endo */
"0CLB8ZZ"  =  "1"  /* Occlusion of Right Parotid Duct, Endo */
"0CLC0CZ"  =  "1"  /* Occlusion of L Parotid Duct with Extralum Dev, Open Approach */
"0CLC0DZ"  =  "1"  /* Occlusion of L Parotid Duct with Intralum Dev, Open Approach */
"0CLC0ZZ"  =  "1"  /* Occlusion of Left Parotid Duct, Open Approach */
"0CLC3CZ"  =  "1"  /* Occlusion of L Parotid Duct with Extralum Dev, Perc Approach */
"0CLC3DZ"  =  "1"  /* Occlusion of L Parotid Duct with Intralum Dev, Perc Approach */
"0CLC3ZZ"  =  "1"  /* Occlusion of Left Parotid Duct, Percutaneous Approach */
"0CLC4CZ"  =  "1"  /* Occlusion L Parotid Duct w Extralum Dev, Perc Endo */
"0CLC4DZ"  =  "1"  /* Occlusion L Parotid Duct w Intralum Dev, Perc Endo */
"0CLC4ZZ"  =  "1"  /* Occlusion of Left Parotid Duct, Perc Endo Approach */
"0CLC7DZ"  =  "1"  /* Occlusion of L Parotid Duct with Intralum Dev, Via Opening */
"0CLC7ZZ"  =  "1"  /* Occlusion of Left Parotid Duct, Via Opening */
"0CLC8DZ"  =  "1"  /* Occlusion of Left Parotid Duct with Intralum Dev, Endo */
"0CLC8ZZ"  =  "1"  /* Occlusion of Left Parotid Duct, Endo */
"0CM00ZZ"  =  "1"  /* Reattachment of Upper Lip, Open Approach */
"0CM10ZZ"  =  "1"  /* Reattachment of Lower Lip, Open Approach */
"0CM30ZZ"  =  "1"  /* Reattachment of Soft Palate, Open Approach */
"0CM70ZZ"  =  "1"  /* Reattachment of Tongue, Open Approach */
"0CMN0ZZ"  =  "1"  /* Reattachment of Uvula, Open Approach */
"0CN20ZZ"  =  "1"  /* Release Hard Palate, Open Approach */
"0CN23ZZ"  =  "1"  /* Release Hard Palate, Percutaneous Approach */
"0CN2XZZ"  =  "1"  /* Release Hard Palate, External Approach */
"0CN30ZZ"  =  "1"  /* Release Soft Palate, Open Approach */
"0CN33ZZ"  =  "1"  /* Release Soft Palate, Percutaneous Approach */
"0CN3XZZ"  =  "1"  /* Release Soft Palate, External Approach */
"0CN40ZZ"  =  "1"  /* Release Buccal Mucosa, Open Approach */
"0CN43ZZ"  =  "1"  /* Release Buccal Mucosa, Percutaneous Approach */
"0CN4XZZ"  =  "1"  /* Release Buccal Mucosa, External Approach */
"0CN80ZZ"  =  "1"  /* Release Right Parotid Gland, Open Approach */
"0CN83ZZ"  =  "1"  /* Release Right Parotid Gland, Percutaneous Approach */
"0CN90ZZ"  =  "1"  /* Release Left Parotid Gland, Open Approach */
"0CN93ZZ"  =  "1"  /* Release Left Parotid Gland, Percutaneous Approach */
"0CNB0ZZ"  =  "1"  /* Release Right Parotid Duct, Open Approach */
"0CNB3ZZ"  =  "1"  /* Release Right Parotid Duct, Percutaneous Approach */
"0CNC0ZZ"  =  "1"  /* Release Left Parotid Duct, Open Approach */
"0CNC3ZZ"  =  "1"  /* Release Left Parotid Duct, Percutaneous Approach */
"0CND0ZZ"  =  "1"  /* Release Right Sublingual Gland, Open Approach */
"0CND3ZZ"  =  "1"  /* Release Right Sublingual Gland, Percutaneous Approach */
"0CNF0ZZ"  =  "1"  /* Release Left Sublingual Gland, Open Approach */
"0CNF3ZZ"  =  "1"  /* Release Left Sublingual Gland, Percutaneous Approach */
"0CNG0ZZ"  =  "1"  /* Release Right Submaxillary Gland, Open Approach */
"0CNG3ZZ"  =  "1"  /* Release Right Submaxillary Gland, Percutaneous Approach */
"0CNH0ZZ"  =  "1"  /* Release Left Submaxillary Gland, Open Approach */
"0CNH3ZZ"  =  "1"  /* Release Left Submaxillary Gland, Percutaneous Approach */
"0CNJ0ZZ"  =  "1"  /* Release Minor Salivary Gland, Open Approach */
"0CNJ3ZZ"  =  "1"  /* Release Minor Salivary Gland, Percutaneous Approach */
"0CNM0ZZ"  =  "1"  /* Release Pharynx, Open Approach */
"0CNM3ZZ"  =  "1"  /* Release Pharynx, Percutaneous Approach */
"0CNM4ZZ"  =  "1"  /* Release Pharynx, Percutaneous Endoscopic Approach */
"0CNM7ZZ"  =  "1"  /* Release Pharynx, Via Natural or Artificial Opening */
"0CNM8ZZ"  =  "1"  /* Release Pharynx, Endo */
"0CNN0ZZ"  =  "1"  /* Release Uvula, Open Approach */
"0CNN3ZZ"  =  "1"  /* Release Uvula, Percutaneous Approach */
"0CNNXZZ"  =  "1"  /* Release Uvula, External Approach */
"0CNP0ZZ"  =  "1"  /* Release Tonsils, Open Approach */
"0CNP3ZZ"  =  "1"  /* Release Tonsils, Percutaneous Approach */
"0CNPXZZ"  =  "1"  /* Release Tonsils, External Approach */
"0CNQ0ZZ"  =  "1"  /* Release Adenoids, Open Approach */
"0CNQ3ZZ"  =  "1"  /* Release Adenoids, Percutaneous Approach */
"0CNQXZZ"  =  "1"  /* Release Adenoids, External Approach */
"0CNR0ZZ"  =  "1"  /* Release Epiglottis, Open Approach */
"0CNR3ZZ"  =  "1"  /* Release Epiglottis, Percutaneous Approach */
"0CNR4ZZ"  =  "1"  /* Release Epiglottis, Percutaneous Endoscopic Approach */
"0CNR7ZZ"  =  "1"  /* Release Epiglottis, Via Natural or Artificial Opening */
"0CNR8ZZ"  =  "1"  /* Release Epiglottis, Endo */
"0CNS0ZZ"  =  "1"  /* Release Larynx, Open Approach */
"0CNS3ZZ"  =  "1"  /* Release Larynx, Percutaneous Approach */
"0CNS4ZZ"  =  "1"  /* Release Larynx, Percutaneous Endoscopic Approach */
"0CNS7ZZ"  =  "1"  /* Release Larynx, Via Natural or Artificial Opening */
"0CNS8ZZ"  =  "1"  /* Release Larynx, Via Natural or Artificial Opening Endoscopic */
"0CNT0ZZ"  =  "1"  /* Release Right Vocal Cord, Open Approach */
"0CNT3ZZ"  =  "1"  /* Release Right Vocal Cord, Percutaneous Approach */
"0CNT4ZZ"  =  "1"  /* Release Right Vocal Cord, Percutaneous Endoscopic Approach */
"0CNT7ZZ"  =  "1"  /* Release Right Vocal Cord, Via Natural or Artificial Opening */
"0CNT8ZZ"  =  "1"  /* Release Right Vocal Cord, Endo */
"0CNV0ZZ"  =  "1"  /* Release Left Vocal Cord, Open Approach */
"0CNV3ZZ"  =  "1"  /* Release Left Vocal Cord, Percutaneous Approach */
"0CNV4ZZ"  =  "1"  /* Release Left Vocal Cord, Percutaneous Endoscopic Approach */
"0CNV7ZZ"  =  "1"  /* Release Left Vocal Cord, Via Natural or Artificial Opening */
"0CNV8ZZ"  =  "1"  /* Release Left Vocal Cord, Endo */
"0CPS00Z"  =  "1"  /* Removal of Drainage Device from Larynx, Open Approach */
"0CPS07Z"  =  "1"  /* Removal of Autol Sub from Larynx, Open Approach */
"0CPS0DZ"  =  "1"  /* Removal of Intraluminal Device from Larynx, Open Approach */
"0CPS0JZ"  =  "1"  /* Removal of Synthetic Substitute from Larynx, Open Approach */
"0CPS0KZ"  =  "1"  /* Removal of Nonaut Sub from Larynx, Open Approach */
"0CPS30Z"  =  "1"  /* Removal of Drainage Device from Larynx, Perc Approach */
"0CPS37Z"  =  "1"  /* Removal of Autol Sub from Larynx, Perc Approach */
"0CPS3DZ"  =  "1"  /* Removal of Intraluminal Device from Larynx, Perc Approach */
"0CPS3JZ"  =  "1"  /* Removal of Synthetic Substitute from Larynx, Perc Approach */
"0CPS3KZ"  =  "1"  /* Removal of Nonaut Sub from Larynx, Perc Approach */
"0CPS70Z"  =  "1"  /* Removal of Drainage Device from Larynx, Via Opening */
"0CPS77Z"  =  "1"  /* Removal of Autol Sub from Larynx, Via Opening */
"0CPS7DZ"  =  "1"  /* Removal of Intraluminal Device from Larynx, Via Opening */
"0CPS7JZ"  =  "1"  /* Removal of Synthetic Substitute from Larynx, Via Opening */
"0CPS7KZ"  =  "1"  /* Removal of Nonaut Sub from Larynx, Via Opening */
"0CPS80Z"  =  "1"  /* Removal of Drainage Device from Larynx, Endo */
"0CPS87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Larynx, Endo */
"0CPS8DZ"  =  "1"  /* Removal of Intraluminal Device from Larynx, Endo */
"0CPS8JZ"  =  "1"  /* Removal of Synthetic Substitute from Larynx, Endo */
"0CPS8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Larynx, Endo */
"0CPY00Z"  =  "1"  /* Removal of Drainage Device from Mouth/Throat, Open Approach */
"0CPY01Z"  =  "1"  /* Removal of Radioact Elem from Mouth/Throat, Open Approach */
"0CPY07Z"  =  "1"  /* Removal of Autol Sub from Mouth/Throat, Open Approach */
"0CPY0DZ"  =  "1"  /* Removal of Intralum Dev from Mouth/Throat, Open Approach */
"0CPY0JZ"  =  "1"  /* Removal of Synth Sub from Mouth/Throat, Open Approach */
"0CPY0KZ"  =  "1"  /* Removal of Nonaut Sub from Mouth/Throat, Open Approach */
"0CPY30Z"  =  "1"  /* Removal of Drainage Device from Mouth/Throat, Perc Approach */
"0CPY31Z"  =  "1"  /* Removal of Radioact Elem from Mouth/Throat, Perc Approach */
"0CPY37Z"  =  "1"  /* Removal of Autol Sub from Mouth/Throat, Perc Approach */
"0CPY3DZ"  =  "1"  /* Removal of Intralum Dev from Mouth/Throat, Perc Approach */
"0CPY3JZ"  =  "1"  /* Removal of Synth Sub from Mouth/Throat, Perc Approach */
"0CPY3KZ"  =  "1"  /* Removal of Nonaut Sub from Mouth/Throat, Perc Approach */
"0CPY71Z"  =  "1"  /* Removal of Radioact Elem from Mouth/Throat, Via Opening */
"0CPY77Z"  =  "1"  /* Removal of Autol Sub from Mouth/Throat, Via Opening */
"0CPY7JZ"  =  "1"  /* Removal of Synth Sub from Mouth/Throat, Via Opening */
"0CPY7KZ"  =  "1"  /* Removal of Nonaut Sub from Mouth/Throat, Via Opening */
"0CPY81Z"  =  "1"  /* Removal of Radioactive Element from Mouth and Throat, Endo */
"0CPY87Z"  =  "1"  /* Removal of Autol Sub from Mouth/Throat, Endo */
"0CPY8JZ"  =  "1"  /* Removal of Synthetic Substitute from Mouth and Throat, Endo */
"0CPY8KZ"  =  "1"  /* Removal of Nonaut Sub from Mouth/Throat, Endo */
"0CQ00ZZ"  =  "1"  /* Repair Upper Lip, Open Approach */
"0CQ03ZZ"  =  "1"  /* Repair Upper Lip, Percutaneous Approach */
"0CQ10ZZ"  =  "1"  /* Repair Lower Lip, Open Approach */
"0CQ13ZZ"  =  "1"  /* Repair Lower Lip, Percutaneous Approach */
"0CQ20ZZ"  =  "1"  /* Repair Hard Palate, Open Approach */
"0CQ23ZZ"  =  "1"  /* Repair Hard Palate, Percutaneous Approach */
"0CQ2XZZ"  =  "1"  /* Repair Hard Palate, External Approach */
"0CQ30ZZ"  =  "1"  /* Repair Soft Palate, Open Approach */
"0CQ33ZZ"  =  "1"  /* Repair Soft Palate, Percutaneous Approach */
"0CQ3XZZ"  =  "1"  /* Repair Soft Palate, External Approach */
"0CQ40ZZ"  =  "1"  /* Repair Buccal Mucosa, Open Approach */
"0CQ43ZZ"  =  "1"  /* Repair Buccal Mucosa, Percutaneous Approach */
"0CQ4XZZ"  =  "1"  /* Repair Buccal Mucosa, External Approach */
"0CQ70ZZ"  =  "1"  /* Repair Tongue, Open Approach */
"0CQ73ZZ"  =  "1"  /* Repair Tongue, Percutaneous Approach */
"0CQ7XZZ"  =  "1"  /* Repair Tongue, External Approach */
"0CQ80ZZ"  =  "1"  /* Repair Right Parotid Gland, Open Approach */
"0CQ83ZZ"  =  "1"  /* Repair Right Parotid Gland, Percutaneous Approach */
"0CQ90ZZ"  =  "1"  /* Repair Left Parotid Gland, Open Approach */
"0CQ93ZZ"  =  "1"  /* Repair Left Parotid Gland, Percutaneous Approach */
"0CQB0ZZ"  =  "1"  /* Repair Right Parotid Duct, Open Approach */
"0CQB3ZZ"  =  "1"  /* Repair Right Parotid Duct, Percutaneous Approach */
"0CQC0ZZ"  =  "1"  /* Repair Left Parotid Duct, Open Approach */
"0CQC3ZZ"  =  "1"  /* Repair Left Parotid Duct, Percutaneous Approach */
"0CQD0ZZ"  =  "1"  /* Repair Right Sublingual Gland, Open Approach */
"0CQD3ZZ"  =  "1"  /* Repair Right Sublingual Gland, Percutaneous Approach */
"0CQF0ZZ"  =  "1"  /* Repair Left Sublingual Gland, Open Approach */
"0CQF3ZZ"  =  "1"  /* Repair Left Sublingual Gland, Percutaneous Approach */
"0CQG0ZZ"  =  "1"  /* Repair Right Submaxillary Gland, Open Approach */
"0CQG3ZZ"  =  "1"  /* Repair Right Submaxillary Gland, Percutaneous Approach */
"0CQH0ZZ"  =  "1"  /* Repair Left Submaxillary Gland, Open Approach */
"0CQH3ZZ"  =  "1"  /* Repair Left Submaxillary Gland, Percutaneous Approach */
"0CQJ0ZZ"  =  "1"  /* Repair Minor Salivary Gland, Open Approach */
"0CQJ3ZZ"  =  "1"  /* Repair Minor Salivary Gland, Percutaneous Approach */
"0CQM0ZZ"  =  "1"  /* Repair Pharynx, Open Approach */
"0CQM3ZZ"  =  "1"  /* Repair Pharynx, Percutaneous Approach */
"0CQM4ZZ"  =  "1"  /* Repair Pharynx, Percutaneous Endoscopic Approach */
"0CQM7ZZ"  =  "1"  /* Repair Pharynx, Via Natural or Artificial Opening */
"0CQM8ZZ"  =  "1"  /* Repair Pharynx, Via Natural or Artificial Opening Endoscopic */
"0CQN0ZZ"  =  "1"  /* Repair Uvula, Open Approach */
"0CQN3ZZ"  =  "1"  /* Repair Uvula, Percutaneous Approach */
"0CQNXZZ"  =  "1"  /* Repair Uvula, External Approach */
"0CQP0ZZ"  =  "1"  /* Repair Tonsils, Open Approach */
"0CQP3ZZ"  =  "1"  /* Repair Tonsils, Percutaneous Approach */
"0CQPXZZ"  =  "1"  /* Repair Tonsils, External Approach */
"0CQQ0ZZ"  =  "1"  /* Repair Adenoids, Open Approach */
"0CQQ3ZZ"  =  "1"  /* Repair Adenoids, Percutaneous Approach */
"0CQQXZZ"  =  "1"  /* Repair Adenoids, External Approach */
"0CQR0ZZ"  =  "1"  /* Repair Epiglottis, Open Approach */
"0CQR3ZZ"  =  "1"  /* Repair Epiglottis, Percutaneous Approach */
"0CQR4ZZ"  =  "1"  /* Repair Epiglottis, Percutaneous Endoscopic Approach */
"0CQR7ZZ"  =  "1"  /* Repair Epiglottis, Via Natural or Artificial Opening */
"0CQR8ZZ"  =  "1"  /* Repair Epiglottis, Endo */
"0CQS0ZZ"  =  "1"  /* Repair Larynx, Open Approach */
"0CQS3ZZ"  =  "1"  /* Repair Larynx, Percutaneous Approach */
"0CQS4ZZ"  =  "1"  /* Repair Larynx, Percutaneous Endoscopic Approach */
"0CQS7ZZ"  =  "1"  /* Repair Larynx, Via Natural or Artificial Opening */
"0CQS8ZZ"  =  "1"  /* Repair Larynx, Via Natural or Artificial Opening Endoscopic */
"0CQT0ZZ"  =  "1"  /* Repair Right Vocal Cord, Open Approach */
"0CQT3ZZ"  =  "1"  /* Repair Right Vocal Cord, Percutaneous Approach */
"0CQT4ZZ"  =  "1"  /* Repair Right Vocal Cord, Percutaneous Endoscopic Approach */
"0CQT7ZZ"  =  "1"  /* Repair Right Vocal Cord, Via Natural or Artificial Opening */
"0CQT8ZZ"  =  "1"  /* Repair Right Vocal Cord, Endo */
"0CQV0ZZ"  =  "1"  /* Repair Left Vocal Cord, Open Approach */
"0CQV3ZZ"  =  "1"  /* Repair Left Vocal Cord, Percutaneous Approach */
"0CQV4ZZ"  =  "1"  /* Repair Left Vocal Cord, Percutaneous Endoscopic Approach */
"0CQV7ZZ"  =  "1"  /* Repair Left Vocal Cord, Via Natural or Artificial Opening */
"0CQV8ZZ"  =  "1"  /* Repair Left Vocal Cord, Endo */
"0CR007Z"  =  "1"  /* Replacement of Upper Lip with Autol Sub, Open Approach */
"0CR00JZ"  =  "1"  /* Replacement of Upper Lip with Synth Sub, Open Approach */
"0CR00KZ"  =  "1"  /* Replacement of Upper Lip with Nonaut Sub, Open Approach */
"0CR037Z"  =  "1"  /* Replacement of Upper Lip with Autol Sub, Perc Approach */
"0CR03JZ"  =  "1"  /* Replacement of Upper Lip with Synth Sub, Perc Approach */
"0CR03KZ"  =  "1"  /* Replacement of Upper Lip with Nonaut Sub, Perc Approach */
"0CR0X7Z"  =  "1"  /* Replacement of Upper Lip with Autol Sub, Extern Approach */
"0CR0XJZ"  =  "1"  /* Replacement of Upper Lip with Synth Sub, Extern Approach */
"0CR0XKZ"  =  "1"  /* Replacement of Upper Lip with Nonaut Sub, Extern Approach */
"0CR107Z"  =  "1"  /* Replacement of Lower Lip with Autol Sub, Open Approach */
"0CR10JZ"  =  "1"  /* Replacement of Lower Lip with Synth Sub, Open Approach */
"0CR10KZ"  =  "1"  /* Replacement of Lower Lip with Nonaut Sub, Open Approach */
"0CR137Z"  =  "1"  /* Replacement of Lower Lip with Autol Sub, Perc Approach */
"0CR13JZ"  =  "1"  /* Replacement of Lower Lip with Synth Sub, Perc Approach */
"0CR13KZ"  =  "1"  /* Replacement of Lower Lip with Nonaut Sub, Perc Approach */
"0CR1X7Z"  =  "1"  /* Replacement of Lower Lip with Autol Sub, Extern Approach */
"0CR1XJZ"  =  "1"  /* Replacement of Lower Lip with Synth Sub, Extern Approach */
"0CR1XKZ"  =  "1"  /* Replacement of Lower Lip with Nonaut Sub, Extern Approach */
"0CR207Z"  =  "1"  /* Replacement of Hard Palate with Autol Sub, Open Approach */
"0CR20JZ"  =  "1"  /* Replacement of Hard Palate with Synth Sub, Open Approach */
"0CR20KZ"  =  "1"  /* Replacement of Hard Palate with Nonaut Sub, Open Approach */
"0CR237Z"  =  "1"  /* Replacement of Hard Palate with Autol Sub, Perc Approach */
"0CR23JZ"  =  "1"  /* Replacement of Hard Palate with Synth Sub, Perc Approach */
"0CR23KZ"  =  "1"  /* Replacement of Hard Palate with Nonaut Sub, Perc Approach */
"0CR2X7Z"  =  "1"  /* Replacement of Hard Palate with Autol Sub, Extern Approach */
"0CR2XJZ"  =  "1"  /* Replacement of Hard Palate with Synth Sub, Extern Approach */
"0CR2XKZ"  =  "1"  /* Replacement of Hard Palate with Nonaut Sub, Extern Approach */
"0CR307Z"  =  "1"  /* Replacement of Soft Palate with Autol Sub, Open Approach */
"0CR30JZ"  =  "1"  /* Replacement of Soft Palate with Synth Sub, Open Approach */
"0CR30KZ"  =  "1"  /* Replacement of Soft Palate with Nonaut Sub, Open Approach */
"0CR337Z"  =  "1"  /* Replacement of Soft Palate with Autol Sub, Perc Approach */
"0CR33JZ"  =  "1"  /* Replacement of Soft Palate with Synth Sub, Perc Approach */
"0CR33KZ"  =  "1"  /* Replacement of Soft Palate with Nonaut Sub, Perc Approach */
"0CR3X7Z"  =  "1"  /* Replacement of Soft Palate with Autol Sub, Extern Approach */
"0CR3XJZ"  =  "1"  /* Replacement of Soft Palate with Synth Sub, Extern Approach */
"0CR3XKZ"  =  "1"  /* Replacement of Soft Palate with Nonaut Sub, Extern Approach */
"0CR407Z"  =  "1"  /* Replacement of Buccal Mucosa with Autol Sub, Open Approach */
"0CR40JZ"  =  "1"  /* Replacement of Buccal Mucosa with Synth Sub, Open Approach */
"0CR40KZ"  =  "1"  /* Replacement of Buccal Mucosa with Nonaut Sub, Open Approach */
"0CR437Z"  =  "1"  /* Replacement of Buccal Mucosa with Autol Sub, Perc Approach */
"0CR43JZ"  =  "1"  /* Replacement of Buccal Mucosa with Synth Sub, Perc Approach */
"0CR43KZ"  =  "1"  /* Replacement of Buccal Mucosa with Nonaut Sub, Perc Approach */
"0CR4X7Z"  =  "1"  /* Replacement of Buccal Mucosa with Autol Sub, Extern Approach */
"0CR4XJZ"  =  "1"  /* Replacement of Buccal Mucosa with Synth Sub, Extern Approach */
"0CR4XKZ"  =  "1"  /* Replace of Buccal Mucosa with Nonaut Sub, Extern Approach */
"0CR507Z"  =  "1"  /* Replacement of Upper Gingiva with Autol Sub, Open Approach */
"0CR50JZ"  =  "1"  /* Replacement of Upper Gingiva with Synth Sub, Open Approach */
"0CR50KZ"  =  "1"  /* Replacement of Upper Gingiva with Nonaut Sub, Open Approach */
"0CR537Z"  =  "1"  /* Replacement of Upper Gingiva with Autol Sub, Perc Approach */
"0CR53JZ"  =  "1"  /* Replacement of Upper Gingiva with Synth Sub, Perc Approach */
"0CR53KZ"  =  "1"  /* Replacement of Upper Gingiva with Nonaut Sub, Perc Approach */
"0CR5X7Z"  =  "1"  /* Replacement of Upper Gingiva with Autol Sub, Extern Approach */
"0CR5XJZ"  =  "1"  /* Replacement of Upper Gingiva with Synth Sub, Extern Approach */
"0CR5XKZ"  =  "1"  /* Replacement of Up Gingiva with Nonaut Sub, Extern Approach */
"0CR607Z"  =  "1"  /* Replacement of Lower Gingiva with Autol Sub, Open Approach */
"0CR60JZ"  =  "1"  /* Replacement of Lower Gingiva with Synth Sub, Open Approach */
"0CR60KZ"  =  "1"  /* Replacement of Lower Gingiva with Nonaut Sub, Open Approach */
"0CR637Z"  =  "1"  /* Replacement of Lower Gingiva with Autol Sub, Perc Approach */
"0CR63JZ"  =  "1"  /* Replacement of Lower Gingiva with Synth Sub, Perc Approach */
"0CR63KZ"  =  "1"  /* Replacement of Lower Gingiva with Nonaut Sub, Perc Approach */
"0CR6X7Z"  =  "1"  /* Replacement of Lower Gingiva with Autol Sub, Extern Approach */
"0CR6XJZ"  =  "1"  /* Replacement of Lower Gingiva with Synth Sub, Extern Approach */
"0CR6XKZ"  =  "1"  /* Replacement of Low Gingiva with Nonaut Sub, Extern Approach */
"0CR707Z"  =  "1"  /* Replacement of Tongue with Autol Sub, Open Approach */
"0CR70JZ"  =  "1"  /* Replacement of Tongue with Synth Sub, Open Approach */
"0CR70KZ"  =  "1"  /* Replacement of Tongue with Nonaut Sub, Open Approach */
"0CR737Z"  =  "1"  /* Replacement of Tongue with Autol Sub, Perc Approach */
"0CR73JZ"  =  "1"  /* Replacement of Tongue with Synth Sub, Perc Approach */
"0CR73KZ"  =  "1"  /* Replacement of Tongue with Nonaut Sub, Perc Approach */
"0CR7X7Z"  =  "1"  /* Replacement of Tongue with Autol Sub, Extern Approach */
"0CR7XJZ"  =  "1"  /* Replacement of Tongue with Synth Sub, Extern Approach */
"0CR7XKZ"  =  "1"  /* Replacement of Tongue with Nonaut Sub, Extern Approach */
"0CRB07Z"  =  "1"  /* Replacement of R Parotid Duct with Autol Sub, Open Approach */
"0CRB0JZ"  =  "1"  /* Replacement of R Parotid Duct with Synth Sub, Open Approach */
"0CRB0KZ"  =  "1"  /* Replacement of R Parotid Duct with Nonaut Sub, Open Approach */
"0CRB37Z"  =  "1"  /* Replacement of R Parotid Duct with Autol Sub, Perc Approach */
"0CRB3JZ"  =  "1"  /* Replacement of R Parotid Duct with Synth Sub, Perc Approach */
"0CRB3KZ"  =  "1"  /* Replacement of R Parotid Duct with Nonaut Sub, Perc Approach */
"0CRC07Z"  =  "1"  /* Replacement of L Parotid Duct with Autol Sub, Open Approach */
"0CRC0JZ"  =  "1"  /* Replacement of L Parotid Duct with Synth Sub, Open Approach */
"0CRC0KZ"  =  "1"  /* Replacement of L Parotid Duct with Nonaut Sub, Open Approach */
"0CRC37Z"  =  "1"  /* Replacement of L Parotid Duct with Autol Sub, Perc Approach */
"0CRC3JZ"  =  "1"  /* Replacement of L Parotid Duct with Synth Sub, Perc Approach */
"0CRC3KZ"  =  "1"  /* Replacement of L Parotid Duct with Nonaut Sub, Perc Approach */
"0CRM07Z"  =  "1"  /* Replacement of Pharynx with Autol Sub, Open Approach */
"0CRM0JZ"  =  "1"  /* Replacement of Pharynx with Synth Sub, Open Approach */
"0CRM0KZ"  =  "1"  /* Replacement of Pharynx with Nonaut Sub, Open Approach */
"0CRM77Z"  =  "1"  /* Replacement of Pharynx with Autol Sub, Via Opening */
"0CRM7JZ"  =  "1"  /* Replacement of Pharynx with Synth Sub, Via Opening */
"0CRM7KZ"  =  "1"  /* Replacement of Pharynx with Nonaut Sub, Via Opening */
"0CRM87Z"  =  "1"  /* Replacement of Pharynx with Autol Sub, Endo */
"0CRM8JZ"  =  "1"  /* Replacement of Pharynx with Synthetic Substitute, Endo */
"0CRM8KZ"  =  "1"  /* Replacement of Pharynx with Nonaut Sub, Endo */
"0CRN07Z"  =  "1"  /* Replacement of Uvula with Autol Sub, Open Approach */
"0CRN0JZ"  =  "1"  /* Replacement of Uvula with Synth Sub, Open Approach */
"0CRN0KZ"  =  "1"  /* Replacement of Uvula with Nonaut Sub, Open Approach */
"0CRN37Z"  =  "1"  /* Replacement of Uvula with Autol Sub, Perc Approach */
"0CRN3JZ"  =  "1"  /* Replacement of Uvula with Synth Sub, Perc Approach */
"0CRN3KZ"  =  "1"  /* Replacement of Uvula with Nonaut Sub, Perc Approach */
"0CRNX7Z"  =  "1"  /* Replacement of Uvula with Autol Sub, Extern Approach */
"0CRNXJZ"  =  "1"  /* Replacement of Uvula with Synth Sub, Extern Approach */
"0CRNXKZ"  =  "1"  /* Replacement of Uvula with Nonaut Sub, Extern Approach */
"0CRR07Z"  =  "1"  /* Replacement of Epiglottis with Autol Sub, Open Approach */
"0CRR0JZ"  =  "1"  /* Replacement of Epiglottis with Synth Sub, Open Approach */
"0CRR0KZ"  =  "1"  /* Replacement of Epiglottis with Nonaut Sub, Open Approach */
"0CRR77Z"  =  "1"  /* Replacement of Epiglottis with Autol Sub, Via Opening */
"0CRR7JZ"  =  "1"  /* Replacement of Epiglottis with Synth Sub, Via Opening */
"0CRR7KZ"  =  "1"  /* Replacement of Epiglottis with Nonaut Sub, Via Opening */
"0CRR87Z"  =  "1"  /* Replacement of Epiglottis with Autol Sub, Endo */
"0CRR8JZ"  =  "1"  /* Replacement of Epiglottis with Synthetic Substitute, Endo */
"0CRR8KZ"  =  "1"  /* Replacement of Epiglottis with Nonaut Sub, Endo */
"0CRS07Z"  =  "1"  /* Replacement of Larynx with Autol Sub, Open Approach */
"0CRS0JZ"  =  "1"  /* Replacement of Larynx with Synth Sub, Open Approach */
"0CRS0KZ"  =  "1"  /* Replacement of Larynx with Nonaut Sub, Open Approach */
"0CRS77Z"  =  "1"  /* Replacement of Larynx with Autol Sub, Via Opening */
"0CRS7JZ"  =  "1"  /* Replacement of Larynx with Synthetic Substitute, Via Opening */
"0CRS7KZ"  =  "1"  /* Replacement of Larynx with Nonaut Sub, Via Opening */
"0CRS87Z"  =  "1"  /* Replacement of Larynx with Autol Sub, Endo */
"0CRS8JZ"  =  "1"  /* Replacement of Larynx with Synthetic Substitute, Endo */
"0CRS8KZ"  =  "1"  /* Replacement of Larynx with Nonaut Sub, Endo */
"0CRT07Z"  =  "1"  /* Replacement of R Vocal Cord with Autol Sub, Open Approach */
"0CRT0JZ"  =  "1"  /* Replacement of R Vocal Cord with Synth Sub, Open Approach */
"0CRT0KZ"  =  "1"  /* Replacement of R Vocal Cord with Nonaut Sub, Open Approach */
"0CRT77Z"  =  "1"  /* Replacement of Right Vocal Cord with Autol Sub, Via Opening */
"0CRT7JZ"  =  "1"  /* Replacement of Right Vocal Cord with Synth Sub, Via Opening */
"0CRT7KZ"  =  "1"  /* Replacement of Right Vocal Cord with Nonaut Sub, Via Opening */
"0CRT87Z"  =  "1"  /* Replacement of Right Vocal Cord with Autol Sub, Endo */
"0CRT8JZ"  =  "1"  /* Replacement of Right Vocal Cord with Synth Sub, Endo */
"0CRT8KZ"  =  "1"  /* Replacement of Right Vocal Cord with Nonaut Sub, Endo */
"0CRV07Z"  =  "1"  /* Replacement of Left Vocal Cord with Autol Sub, Open Approach */
"0CRV0JZ"  =  "1"  /* Replacement of Left Vocal Cord with Synth Sub, Open Approach */
"0CRV0KZ"  =  "1"  /* Replacement of L Vocal Cord with Nonaut Sub, Open Approach */
"0CRV77Z"  =  "1"  /* Replacement of Left Vocal Cord with Autol Sub, Via Opening */
"0CRV7JZ"  =  "1"  /* Replacement of Left Vocal Cord with Synth Sub, Via Opening */
"0CRV7KZ"  =  "1"  /* Replacement of Left Vocal Cord with Nonaut Sub, Via Opening */
"0CRV87Z"  =  "1"  /* Replacement of Left Vocal Cord with Autol Sub, Endo */
"0CRV8JZ"  =  "1"  /* Replacement of Left Vocal Cord with Synth Sub, Endo */
"0CRV8KZ"  =  "1"  /* Replacement of Left Vocal Cord with Nonaut Sub, Endo */
"0CS00ZZ"  =  "1"  /* Reposition Upper Lip, Open Approach */
"0CS0XZZ"  =  "1"  /* Reposition Upper Lip, External Approach */
"0CS10ZZ"  =  "1"  /* Reposition Lower Lip, Open Approach */
"0CS1XZZ"  =  "1"  /* Reposition Lower Lip, External Approach */
"0CS20ZZ"  =  "1"  /* Reposition Hard Palate, Open Approach */
"0CS2XZZ"  =  "1"  /* Reposition Hard Palate, External Approach */
"0CS30ZZ"  =  "1"  /* Reposition Soft Palate, Open Approach */
"0CS3XZZ"  =  "1"  /* Reposition Soft Palate, External Approach */
"0CS70ZZ"  =  "1"  /* Reposition Tongue, Open Approach */
"0CS7XZZ"  =  "1"  /* Reposition Tongue, External Approach */
"0CSB0ZZ"  =  "1"  /* Reposition Right Parotid Duct, Open Approach */
"0CSB3ZZ"  =  "1"  /* Reposition Right Parotid Duct, Percutaneous Approach */
"0CSC0ZZ"  =  "1"  /* Reposition Left Parotid Duct, Open Approach */
"0CSC3ZZ"  =  "1"  /* Reposition Left Parotid Duct, Percutaneous Approach */
"0CSN0ZZ"  =  "1"  /* Reposition Uvula, Open Approach */
"0CSNXZZ"  =  "1"  /* Reposition Uvula, External Approach */
"0CSR0ZZ"  =  "1"  /* Reposition Epiglottis, Open Approach */
"0CSR7ZZ"  =  "1"  /* Reposition Epiglottis, Via Natural or Artificial Opening */
"0CSR8ZZ"  =  "1"  /* Reposition Epiglottis, Endo */
"0CST0ZZ"  =  "1"  /* Reposition Right Vocal Cord, Open Approach */
"0CST7ZZ"  =  "1"  /* Reposition Right Vocal Cord, Via Opening */
"0CST8ZZ"  =  "1"  /* Reposition Right Vocal Cord, Endo */
"0CSV0ZZ"  =  "1"  /* Reposition Left Vocal Cord, Open Approach */
"0CSV7ZZ"  =  "1"  /* Reposition Left Vocal Cord, Via Opening */
"0CSV8ZZ"  =  "1"  /* Reposition Left Vocal Cord, Endo */
"0CT00ZZ"  =  "1"  /* Resection of Upper Lip, Open Approach */
"0CT0XZZ"  =  "1"  /* Resection of Upper Lip, External Approach */
"0CT10ZZ"  =  "1"  /* Resection of Lower Lip, Open Approach */
"0CT1XZZ"  =  "1"  /* Resection of Lower Lip, External Approach */
"0CT20ZZ"  =  "1"  /* Resection of Hard Palate, Open Approach */
"0CT2XZZ"  =  "1"  /* Resection of Hard Palate, External Approach */
"0CT30ZZ"  =  "1"  /* Resection of Soft Palate, Open Approach */
"0CT3XZZ"  =  "1"  /* Resection of Soft Palate, External Approach */
"0CT70ZZ"  =  "1"  /* Resection of Tongue, Open Approach */
"0CT7XZZ"  =  "1"  /* Resection of Tongue, External Approach */
"0CT80ZZ"  =  "1"  /* Resection of Right Parotid Gland, Open Approach */
"0CT90ZZ"  =  "1"  /* Resection of Left Parotid Gland, Open Approach */
"0CTB0ZZ"  =  "1"  /* Resection of Right Parotid Duct, Open Approach */
"0CTC0ZZ"  =  "1"  /* Resection of Left Parotid Duct, Open Approach */
"0CTD0ZZ"  =  "1"  /* Resection of Right Sublingual Gland, Open Approach */
"0CTF0ZZ"  =  "1"  /* Resection of Left Sublingual Gland, Open Approach */
"0CTG0ZZ"  =  "1"  /* Resection of Right Submaxillary Gland, Open Approach */
"0CTH0ZZ"  =  "1"  /* Resection of Left Submaxillary Gland, Open Approach */
"0CTJ0ZZ"  =  "1"  /* Resection of Minor Salivary Gland, Open Approach */
"0CTM0ZZ"  =  "1"  /* Resection of Pharynx, Open Approach */
"0CTM4ZZ"  =  "1"  /* Resection of Pharynx, Percutaneous Endoscopic Approach */
"0CTM7ZZ"  =  "1"  /* Resection of Pharynx, Via Natural or Artificial Opening */
"0CTM8ZZ"  =  "1"  /* Resection of Pharynx, Endo */
"0CTN0ZZ"  =  "1"  /* Resection of Uvula, Open Approach */
"0CTNXZZ"  =  "1"  /* Resection of Uvula, External Approach */
"0CTP0ZZ"  =  "1"  /* Resection of Tonsils, Open Approach */
"0CTPXZZ"  =  "1"  /* Resection of Tonsils, External Approach */
"0CTQ0ZZ"  =  "1"  /* Resection of Adenoids, Open Approach */
"0CTQXZZ"  =  "1"  /* Resection of Adenoids, External Approach */
"0CTR0ZZ"  =  "1"  /* Resection of Epiglottis, Open Approach */
"0CTR4ZZ"  =  "1"  /* Resection of Epiglottis, Percutaneous Endoscopic Approach */
"0CTR7ZZ"  =  "1"  /* Resection of Epiglottis, Via Natural or Artificial Opening */
"0CTR8ZZ"  =  "1"  /* Resection of Epiglottis, Endo */
"0CTS0ZZ"  =  "1"  /* Resection of Larynx, Open Approach */
"0CTS4ZZ"  =  "1"  /* Resection of Larynx, Percutaneous Endoscopic Approach */
"0CTS7ZZ"  =  "1"  /* Resection of Larynx, Via Natural or Artificial Opening */
"0CTS8ZZ"  =  "1"  /* Resection of Larynx, Endo */
"0CTT0ZZ"  =  "1"  /* Resection of Right Vocal Cord, Open Approach */
"0CTT4ZZ"  =  "1"  /* Resection of Right Vocal Cord, Perc Endo Approach */
"0CTT7ZZ"  =  "1"  /* Resection of Right Vocal Cord, Via Opening */
"0CTT8ZZ"  =  "1"  /* Resection of Right Vocal Cord, Endo */
"0CTV0ZZ"  =  "1"  /* Resection of Left Vocal Cord, Open Approach */
"0CTV4ZZ"  =  "1"  /* Resection of Left Vocal Cord, Perc Endo Approach */
"0CTV7ZZ"  =  "1"  /* Resection of Left Vocal Cord, Via Opening */
"0CTV8ZZ"  =  "1"  /* Resection of Left Vocal Cord, Endo */
"0CU007Z"  =  "1"  /* Supplement Upper Lip with Autol Sub, Open Approach */
"0CU00JZ"  =  "1"  /* Supplement Upper Lip with Synth Sub, Open Approach */
"0CU00KZ"  =  "1"  /* Supplement Upper Lip with Nonaut Sub, Open Approach */
"0CU037Z"  =  "1"  /* Supplement Upper Lip with Autol Sub, Perc Approach */
"0CU03JZ"  =  "1"  /* Supplement Upper Lip with Synth Sub, Perc Approach */
"0CU03KZ"  =  "1"  /* Supplement Upper Lip with Nonaut Sub, Perc Approach */
"0CU0X7Z"  =  "1"  /* Supplement Upper Lip with Autol Sub, Extern Approach */
"0CU0XJZ"  =  "1"  /* Supplement Upper Lip with Synth Sub, Extern Approach */
"0CU0XKZ"  =  "1"  /* Supplement Upper Lip with Nonaut Sub, Extern Approach */
"0CU107Z"  =  "1"  /* Supplement Lower Lip with Autol Sub, Open Approach */
"0CU10JZ"  =  "1"  /* Supplement Lower Lip with Synth Sub, Open Approach */
"0CU10KZ"  =  "1"  /* Supplement Lower Lip with Nonaut Sub, Open Approach */
"0CU137Z"  =  "1"  /* Supplement Lower Lip with Autol Sub, Perc Approach */
"0CU13JZ"  =  "1"  /* Supplement Lower Lip with Synth Sub, Perc Approach */
"0CU13KZ"  =  "1"  /* Supplement Lower Lip with Nonaut Sub, Perc Approach */
"0CU1X7Z"  =  "1"  /* Supplement Lower Lip with Autol Sub, Extern Approach */
"0CU1XJZ"  =  "1"  /* Supplement Lower Lip with Synth Sub, Extern Approach */
"0CU1XKZ"  =  "1"  /* Supplement Lower Lip with Nonaut Sub, Extern Approach */
"0CU207Z"  =  "1"  /* Supplement Hard Palate with Autol Sub, Open Approach */
"0CU20KZ"  =  "1"  /* Supplement Hard Palate with Nonaut Sub, Open Approach */
"0CU237Z"  =  "1"  /* Supplement Hard Palate with Autol Sub, Perc Approach */
"0CU23KZ"  =  "1"  /* Supplement Hard Palate with Nonaut Sub, Perc Approach */
"0CU2X7Z"  =  "1"  /* Supplement Hard Palate with Autol Sub, Extern Approach */
"0CU2XJZ"  =  "1"  /* Supplement Hard Palate with Synth Sub, Extern Approach */
"0CU2XKZ"  =  "1"  /* Supplement Hard Palate with Nonaut Sub, Extern Approach */
"0CU307Z"  =  "1"  /* Supplement Soft Palate with Autol Sub, Open Approach */
"0CU30JZ"  =  "1"  /* Supplement Soft Palate with Synth Sub, Open Approach */
"0CU30KZ"  =  "1"  /* Supplement Soft Palate with Nonaut Sub, Open Approach */
"0CU337Z"  =  "1"  /* Supplement Soft Palate with Autol Sub, Perc Approach */
"0CU33JZ"  =  "1"  /* Supplement Soft Palate with Synth Sub, Perc Approach */
"0CU33KZ"  =  "1"  /* Supplement Soft Palate with Nonaut Sub, Perc Approach */
"0CU3X7Z"  =  "1"  /* Supplement Soft Palate with Autol Sub, Extern Approach */
"0CU3XJZ"  =  "1"  /* Supplement Soft Palate with Synth Sub, Extern Approach */
"0CU3XKZ"  =  "1"  /* Supplement Soft Palate with Nonaut Sub, Extern Approach */
"0CU407Z"  =  "1"  /* Supplement Buccal Mucosa with Autol Sub, Open Approach */
"0CU40JZ"  =  "1"  /* Supplement Buccal Mucosa with Synth Sub, Open Approach */
"0CU40KZ"  =  "1"  /* Supplement Buccal Mucosa with Nonaut Sub, Open Approach */
"0CU437Z"  =  "1"  /* Supplement Buccal Mucosa with Autol Sub, Perc Approach */
"0CU43JZ"  =  "1"  /* Supplement Buccal Mucosa with Synth Sub, Perc Approach */
"0CU43KZ"  =  "1"  /* Supplement Buccal Mucosa with Nonaut Sub, Perc Approach */
"0CU4X7Z"  =  "1"  /* Supplement Buccal Mucosa with Autol Sub, Extern Approach */
"0CU4XJZ"  =  "1"  /* Supplement Buccal Mucosa with Synth Sub, Extern Approach */
"0CU4XKZ"  =  "1"  /* Supplement Buccal Mucosa with Nonaut Sub, Extern Approach */
"0CU507Z"  =  "1"  /* Supplement Upper Gingiva with Autol Sub, Open Approach */
"0CU50JZ"  =  "1"  /* Supplement Upper Gingiva with Synth Sub, Open Approach */
"0CU50KZ"  =  "1"  /* Supplement Upper Gingiva with Nonaut Sub, Open Approach */
"0CU537Z"  =  "1"  /* Supplement Upper Gingiva with Autol Sub, Perc Approach */
"0CU53JZ"  =  "1"  /* Supplement Upper Gingiva with Synth Sub, Perc Approach */
"0CU53KZ"  =  "1"  /* Supplement Upper Gingiva with Nonaut Sub, Perc Approach */
"0CU5X7Z"  =  "1"  /* Supplement Upper Gingiva with Autol Sub, Extern Approach */
"0CU5XJZ"  =  "1"  /* Supplement Upper Gingiva with Synth Sub, Extern Approach */
"0CU5XKZ"  =  "1"  /* Supplement Upper Gingiva with Nonaut Sub, Extern Approach */
"0CU607Z"  =  "1"  /* Supplement Lower Gingiva with Autol Sub, Open Approach */
"0CU60JZ"  =  "1"  /* Supplement Lower Gingiva with Synth Sub, Open Approach */
"0CU60KZ"  =  "1"  /* Supplement Lower Gingiva with Nonaut Sub, Open Approach */
"0CU637Z"  =  "1"  /* Supplement Lower Gingiva with Autol Sub, Perc Approach */
"0CU63JZ"  =  "1"  /* Supplement Lower Gingiva with Synth Sub, Perc Approach */
"0CU63KZ"  =  "1"  /* Supplement Lower Gingiva with Nonaut Sub, Perc Approach */
"0CU6X7Z"  =  "1"  /* Supplement Lower Gingiva with Autol Sub, Extern Approach */
"0CU6XJZ"  =  "1"  /* Supplement Lower Gingiva with Synth Sub, Extern Approach */
"0CU6XKZ"  =  "1"  /* Supplement Lower Gingiva with Nonaut Sub, Extern Approach */
"0CU707Z"  =  "1"  /* Supplement Tongue with Autol Sub, Open Approach */
"0CU70JZ"  =  "1"  /* Supplement Tongue with Synthetic Substitute, Open Approach */
"0CU70KZ"  =  "1"  /* Supplement Tongue with Nonaut Sub, Open Approach */
"0CU737Z"  =  "1"  /* Supplement Tongue with Autol Sub, Perc Approach */
"0CU73JZ"  =  "1"  /* Supplement Tongue with Synthetic Substitute, Perc Approach */
"0CU73KZ"  =  "1"  /* Supplement Tongue with Nonaut Sub, Perc Approach */
"0CU7X7Z"  =  "1"  /* Supplement Tongue with Autol Sub, Extern Approach */
"0CU7XJZ"  =  "1"  /* Supplement Tongue with Synthetic Substitute, Extern Approach */
"0CU7XKZ"  =  "1"  /* Supplement Tongue with Nonaut Sub, Extern Approach */
"0CUM07Z"  =  "1"  /* Supplement Pharynx with Autol Sub, Open Approach */
"0CUM0JZ"  =  "1"  /* Supplement Pharynx with Synthetic Substitute, Open Approach */
"0CUM0KZ"  =  "1"  /* Supplement Pharynx with Nonaut Sub, Open Approach */
"0CUM77Z"  =  "1"  /* Supplement Pharynx with Autol Sub, Via Opening */
"0CUM7JZ"  =  "1"  /* Supplement Pharynx with Synthetic Substitute, Via Opening */
"0CUM7KZ"  =  "1"  /* Supplement Pharynx with Nonaut Sub, Via Opening */
"0CUM87Z"  =  "1"  /* Supplement Pharynx with Autologous Tissue Substitute, Endo */
"0CUM8JZ"  =  "1"  /* Supplement Pharynx with Synthetic Substitute, Endo */
"0CUM8KZ"  =  "1"  /* Supplement Pharynx with Nonaut Sub, Endo */
"0CUN07Z"  =  "1"  /* Supplement Uvula with Autol Sub, Open Approach */
"0CUN0JZ"  =  "1"  /* Supplement Uvula with Synthetic Substitute, Open Approach */
"0CUN0KZ"  =  "1"  /* Supplement Uvula with Nonaut Sub, Open Approach */
"0CUN37Z"  =  "1"  /* Supplement Uvula with Autol Sub, Perc Approach */
"0CUN3JZ"  =  "1"  /* Supplement Uvula with Synthetic Substitute, Perc Approach */
"0CUN3KZ"  =  "1"  /* Supplement Uvula with Nonaut Sub, Perc Approach */
"0CUNX7Z"  =  "1"  /* Supplement Uvula with Autol Sub, Extern Approach */
"0CUNXJZ"  =  "1"  /* Supplement Uvula with Synthetic Substitute, Extern Approach */
"0CUNXKZ"  =  "1"  /* Supplement Uvula with Nonaut Sub, Extern Approach */
"0CUR07Z"  =  "1"  /* Supplement Epiglottis with Autol Sub, Open Approach */
"0CUR0JZ"  =  "1"  /* Supplement Epiglottis with Synth Sub, Open Approach */
"0CUR0KZ"  =  "1"  /* Supplement Epiglottis with Nonaut Sub, Open Approach */
"0CUR77Z"  =  "1"  /* Supplement Epiglottis with Autol Sub, Via Opening */
"0CUR7JZ"  =  "1"  /* Supplement Epiglottis with Synthetic Substitute, Via Opening */
"0CUR7KZ"  =  "1"  /* Supplement Epiglottis with Nonaut Sub, Via Opening */
"0CUR87Z"  =  "1"  /* Supplement Epiglottis with Autol Sub, Endo */
"0CUR8JZ"  =  "1"  /* Supplement Epiglottis with Synthetic Substitute, Endo */
"0CUR8KZ"  =  "1"  /* Supplement Epiglottis with Nonaut Sub, Endo */
"0CUS07Z"  =  "1"  /* Supplement Larynx with Autol Sub, Open Approach */
"0CUS0JZ"  =  "1"  /* Supplement Larynx with Synthetic Substitute, Open Approach */
"0CUS0KZ"  =  "1"  /* Supplement Larynx with Nonaut Sub, Open Approach */
"0CUS77Z"  =  "1"  /* Supplement Larynx with Autol Sub, Via Opening */
"0CUS7JZ"  =  "1"  /* Supplement Larynx with Synthetic Substitute, Via Opening */
"0CUS7KZ"  =  "1"  /* Supplement Larynx with Nonaut Sub, Via Opening */
"0CUS87Z"  =  "1"  /* Supplement Larynx with Autologous Tissue Substitute, Endo */
"0CUS8JZ"  =  "1"  /* Supplement Larynx with Synthetic Substitute, Endo */
"0CUS8KZ"  =  "1"  /* Supplement Larynx with Nonautologous Tissue Substitute, Endo */
"0CUT07Z"  =  "1"  /* Supplement Right Vocal Cord with Autol Sub, Open Approach */
"0CUT0JZ"  =  "1"  /* Supplement Right Vocal Cord with Synth Sub, Open Approach */
"0CUT0KZ"  =  "1"  /* Supplement Right Vocal Cord with Nonaut Sub, Open Approach */
"0CUT77Z"  =  "1"  /* Supplement Right Vocal Cord with Autol Sub, Via Opening */
"0CUT7JZ"  =  "1"  /* Supplement Right Vocal Cord with Synth Sub, Via Opening */
"0CUT7KZ"  =  "1"  /* Supplement Right Vocal Cord with Nonaut Sub, Via Opening */
"0CUT87Z"  =  "1"  /* Supplement Right Vocal Cord with Autol Sub, Endo */
"0CUT8JZ"  =  "1"  /* Supplement Right Vocal Cord with Synthetic Substitute, Endo */
"0CUT8KZ"  =  "1"  /* Supplement Right Vocal Cord with Nonaut Sub, Endo */
"0CUV07Z"  =  "1"  /* Supplement Left Vocal Cord with Autol Sub, Open Approach */
"0CUV0JZ"  =  "1"  /* Supplement Left Vocal Cord with Synth Sub, Open Approach */
"0CUV0KZ"  =  "1"  /* Supplement Left Vocal Cord with Nonaut Sub, Open Approach */
"0CUV77Z"  =  "1"  /* Supplement Left Vocal Cord with Autol Sub, Via Opening */
"0CUV7JZ"  =  "1"  /* Supplement Left Vocal Cord with Synth Sub, Via Opening */
"0CUV7KZ"  =  "1"  /* Supplement Left Vocal Cord with Nonaut Sub, Via Opening */
"0CUV87Z"  =  "1"  /* Supplement Left Vocal Cord with Autol Sub, Endo */
"0CUV8JZ"  =  "1"  /* Supplement Left Vocal Cord with Synthetic Substitute, Endo */
"0CUV8KZ"  =  "1"  /* Supplement Left Vocal Cord with Nonaut Sub, Endo */
"0CVB0CZ"  =  "1"  /* Restrict of R Parotid Duct with Extralum Dev, Open Approach */
"0CVB0DZ"  =  "1"  /* Restrict of R Parotid Duct with Intralum Dev, Open Approach */
"0CVB0ZZ"  =  "1"  /* Restriction of Right Parotid Duct, Open Approach */
"0CVB3CZ"  =  "1"  /* Restrict of R Parotid Duct with Extralum Dev, Perc Approach */
"0CVB3DZ"  =  "1"  /* Restrict of R Parotid Duct with Intralum Dev, Perc Approach */
"0CVB3ZZ"  =  "1"  /* Restriction of Right Parotid Duct, Percutaneous Approach */
"0CVB7DZ"  =  "1"  /* Restriction of R Parotid Duct with Intralum Dev, Via Opening */
"0CVB7ZZ"  =  "1"  /* Restriction of Right Parotid Duct, Via Opening */
"0CVB8DZ"  =  "1"  /* Restriction of Right Parotid Duct with Intralum Dev, Endo */
"0CVB8ZZ"  =  "1"  /* Restriction of Right Parotid Duct, Endo */
"0CVC0CZ"  =  "1"  /* Restrict of L Parotid Duct with Extralum Dev, Open Approach */
"0CVC0DZ"  =  "1"  /* Restrict of L Parotid Duct with Intralum Dev, Open Approach */
"0CVC0ZZ"  =  "1"  /* Restriction of Left Parotid Duct, Open Approach */
"0CVC3CZ"  =  "1"  /* Restrict of L Parotid Duct with Extralum Dev, Perc Approach */
"0CVC3DZ"  =  "1"  /* Restrict of L Parotid Duct with Intralum Dev, Perc Approach */
"0CVC3ZZ"  =  "1"  /* Restriction of Left Parotid Duct, Percutaneous Approach */
"0CVC7DZ"  =  "1"  /* Restriction of L Parotid Duct with Intralum Dev, Via Opening */
"0CVC7ZZ"  =  "1"  /* Restriction of Left Parotid Duct, Via Opening */
"0CVC8DZ"  =  "1"  /* Restriction of Left Parotid Duct with Intralum Dev, Endo */
"0CVC8ZZ"  =  "1"  /* Restriction of Left Parotid Duct, Endo */
"0CWS00Z"  =  "1"  /* Revision of Drainage Device in Larynx, Open Approach */
"0CWS07Z"  =  "1"  /* Revision of Autol Sub in Larynx, Open Approach */
"0CWS0DZ"  =  "1"  /* Revision of Intraluminal Device in Larynx, Open Approach */
"0CWS0JZ"  =  "1"  /* Revision of Synthetic Substitute in Larynx, Open Approach */
"0CWS0KZ"  =  "1"  /* Revision of Nonaut Sub in Larynx, Open Approach */
"0CWS30Z"  =  "1"  /* Revision of Drainage Device in Larynx, Percutaneous Approach */
"0CWS37Z"  =  "1"  /* Revision of Autol Sub in Larynx, Perc Approach */
"0CWS3DZ"  =  "1"  /* Revision of Intraluminal Device in Larynx, Perc Approach */
"0CWS3JZ"  =  "1"  /* Revision of Synthetic Substitute in Larynx, Perc Approach */
"0CWS3KZ"  =  "1"  /* Revision of Nonaut Sub in Larynx, Perc Approach */
"0CWS70Z"  =  "1"  /* Revision of Drainage Device in Larynx, Via Opening */
"0CWS77Z"  =  "1"  /* Revision of Autol Sub in Larynx, Via Opening */
"0CWS7DZ"  =  "1"  /* Revision of Intraluminal Device in Larynx, Via Opening */
"0CWS7JZ"  =  "1"  /* Revision of Synthetic Substitute in Larynx, Via Opening */
"0CWS7KZ"  =  "1"  /* Revision of Nonaut Sub in Larynx, Via Opening */
"0CWS80Z"  =  "1"  /* Revision of Drainage Device in Larynx, Endo */
"0CWS87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Larynx, Endo */
"0CWS8DZ"  =  "1"  /* Revision of Intraluminal Device in Larynx, Endo */
"0CWS8JZ"  =  "1"  /* Revision of Synthetic Substitute in Larynx, Endo */
"0CWS8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Larynx, Endo */
"0CWY00Z"  =  "1"  /* Revision of Drainage Device in Mouth/Throat, Open Approach */
"0CWY01Z"  =  "1"  /* Revision of Radioact Elem in Mouth/Throat, Open Approach */
"0CWY0DZ"  =  "1"  /* Revision of Intralum Dev in Mouth/Throat, Open Approach */
"0CWY0JZ"  =  "1"  /* Revision of Synth Sub in Mouth/Throat, Open Approach */
"0CWY0KZ"  =  "1"  /* Revision of Nonaut Sub in Mouth/Throat, Open Approach */
"0CWY30Z"  =  "1"  /* Revision of Drainage Device in Mouth/Throat, Perc Approach */
"0CWY31Z"  =  "1"  /* Revision of Radioact Elem in Mouth/Throat, Perc Approach */
"0CWY37Z"  =  "1"  /* Revision of Autol Sub in Mouth/Throat, Perc Approach */
"0CWY3DZ"  =  "1"  /* Revision of Intralum Dev in Mouth/Throat, Perc Approach */
"0CWY3JZ"  =  "1"  /* Revision of Synth Sub in Mouth/Throat, Perc Approach */
"0CWY3KZ"  =  "1"  /* Revision of Nonaut Sub in Mouth/Throat, Perc Approach */
"0CWY70Z"  =  "1"  /* Revision of Drainage Device in Mouth and Throat, Via Opening */
"0CWY71Z"  =  "1"  /* Revision of Radioactive Element in Mouth/Throat, Via Opening */
"0CWY77Z"  =  "1"  /* Revision of Autol Sub in Mouth/Throat, Via Opening */
"0CWY7DZ"  =  "1"  /* Revision of Intraluminal Device in Mouth/Throat, Via Opening */
"0CWY7JZ"  =  "1"  /* Revision of Synth Sub in Mouth/Throat, Via Opening */
"0CWY7KZ"  =  "1"  /* Revision of Nonaut Sub in Mouth/Throat, Via Opening */
"0CWY80Z"  =  "1"  /* Revision of Drainage Device in Mouth and Throat, Endo */
"0CWY81Z"  =  "1"  /* Revision of Radioactive Element in Mouth and Throat, Endo */
"0CWY87Z"  =  "1"  /* Revision of Autol Sub in Mouth/Throat, Endo */
"0CWY8DZ"  =  "1"  /* Revision of Intraluminal Device in Mouth and Throat, Endo */
"0CWY8JZ"  =  "1"  /* Revision of Synthetic Substitute in Mouth and Throat, Endo */
"0CWY8KZ"  =  "1"  /* Revision of Nonaut Sub in Mouth/Throat, Endo */
"0CX00ZZ"  =  "1"  /* Transfer Upper Lip, Open Approach */
"0CX0XZZ"  =  "1"  /* Transfer Upper Lip, External Approach */
"0CX10ZZ"  =  "1"  /* Transfer Lower Lip, Open Approach */
"0CX1XZZ"  =  "1"  /* Transfer Lower Lip, External Approach */
"0CX30ZZ"  =  "1"  /* Transfer Soft Palate, Open Approach */
"0CX3XZZ"  =  "1"  /* Transfer Soft Palate, External Approach */
"0CX40ZZ"  =  "1"  /* Transfer Buccal Mucosa, Open Approach */
"0CX4XZZ"  =  "1"  /* Transfer Buccal Mucosa, External Approach */
"0CX50ZZ"  =  "1"  /* Transfer Upper Gingiva, Open Approach */
"0CX5XZZ"  =  "1"  /* Transfer Upper Gingiva, External Approach */
"0CX60ZZ"  =  "1"  /* Transfer Lower Gingiva, Open Approach */
"0CX6XZZ"  =  "1"  /* Transfer Lower Gingiva, External Approach */
"0CX70ZZ"  =  "1"  /* Transfer Tongue, Open Approach */
"0CX7XZZ"  =  "1"  /* Transfer Tongue, External Approach */
"0D11074"  =  "1"  /* Bypass Up Esophag to Cutan with Autol Sub, Open Approach */
"0D11076"  =  "1"  /* Bypass Up Esophag to Stomach with Autol Sub, Open Approach */
"0D11079"  =  "1"  /* Bypass Up Esophag to Duoden with Autol Sub, Open Approach */
"0D1107A"  =  "1"  /* Bypass Up Esophag to Jejunum with Autol Sub, Open Approach */
"0D1107B"  =  "1"  /* Bypass Up Esophag to Ileum with Autol Sub, Open Approach */
"0D110J4"  =  "1"  /* Bypass Up Esophag to Cutan with Synth Sub, Open Approach */
"0D110J6"  =  "1"  /* Bypass Up Esophag to Stomach with Synth Sub, Open Approach */
"0D110J9"  =  "1"  /* Bypass Up Esophag to Duoden with Synth Sub, Open Approach */
"0D110JA"  =  "1"  /* Bypass Up Esophag to Jejunum with Synth Sub, Open Approach */
"0D110JB"  =  "1"  /* Bypass Up Esophag to Ileum with Synth Sub, Open Approach */
"0D110K4"  =  "1"  /* Bypass Up Esophag to Cutan with Nonaut Sub, Open Approach */
"0D110K6"  =  "1"  /* Bypass Up Esophag to Stomach with Nonaut Sub, Open Approach */
"0D110K9"  =  "1"  /* Bypass Up Esophag to Duoden with Nonaut Sub, Open Approach */
"0D110KA"  =  "1"  /* Bypass Up Esophag to Jejunum with Nonaut Sub, Open Approach */
"0D110KB"  =  "1"  /* Bypass Up Esophag to Ileum with Nonaut Sub, Open Approach */
"0D110Z4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous, Open Approach */
"0D110Z6"  =  "1"  /* Bypass Upper Esophagus to Stomach, Open Approach */
"0D110Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Open Approach */
"0D110ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Open Approach */
"0D110ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Open Approach */
"0D113J4"  =  "1"  /* Bypass Up Esophag to Cutan with Synth Sub, Perc Approach */
"0D11474"  =  "1"  /* Bypass Up Esophag to Cutan w Autol Sub, Perc Endo */
"0D11476"  =  "1"  /* Bypass Up Esophag to Stomach w Autol Sub, Perc Endo */
"0D11479"  =  "1"  /* Bypass Up Esophag to Duoden w Autol Sub, Perc Endo */
"0D1147A"  =  "1"  /* Bypass Up Esophag to Jejunum w Autol Sub, Perc Endo */
"0D1147B"  =  "1"  /* Bypass Up Esophag to Ileum w Autol Sub, Perc Endo */
"0D114J4"  =  "1"  /* Bypass Up Esophag to Cutan w Synth Sub, Perc Endo */
"0D114J6"  =  "1"  /* Bypass Up Esophag to Stomach w Synth Sub, Perc Endo */
"0D114J9"  =  "1"  /* Bypass Up Esophag to Duoden w Synth Sub, Perc Endo */
"0D114JA"  =  "1"  /* Bypass Up Esophag to Jejunum w Synth Sub, Perc Endo */
"0D114JB"  =  "1"  /* Bypass Up Esophag to Ileum w Synth Sub, Perc Endo */
"0D114K4"  =  "1"  /* Bypass Up Esophag to Cutan w Nonaut Sub, Perc Endo */
"0D114K6"  =  "1"  /* Bypass Up Esophag to Stomach w Nonaut Sub, Perc Endo */
"0D114K9"  =  "1"  /* Bypass Up Esophag to Duoden w Nonaut Sub, Perc Endo */
"0D114KA"  =  "1"  /* Bypass Up Esophag to Jejunum w Nonaut Sub, Perc Endo */
"0D114KB"  =  "1"  /* Bypass Up Esophag to Ileum w Nonaut Sub, Perc Endo */
"0D114Z4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous, Perc Endo Approach */
"0D114Z6"  =  "1"  /* Bypass Upper Esophagus to Stomach, Perc Endo Approach */
"0D114Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Perc Endo Approach */
"0D114ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Perc Endo Approach */
"0D114ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Perc Endo Approach */
"0D11874"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Autol Sub, Endo */
"0D11876"  =  "1"  /* Bypass Upper Esophagus to Stomach with Autol Sub, Endo */
"0D11879"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Autol Sub, Endo */
"0D1187A"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Autol Sub, Endo */
"0D1187B"  =  "1"  /* Bypass Upper Esophagus to Ileum with Autol Sub, Endo */
"0D118J4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Synth Sub, Endo */
"0D118J6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Synth Sub, Endo */
"0D118J9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Synth Sub, Endo */
"0D118JA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Synth Sub, Endo */
"0D118JB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Synth Sub, Endo */
"0D118K4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Nonaut Sub, Endo */
"0D118K6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Nonaut Sub, Endo */
"0D118K9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Nonaut Sub, Endo */
"0D118KA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Nonaut Sub, Endo */
"0D118KB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Nonaut Sub, Endo */
"0D118Z4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous, Endo */
"0D118Z6"  =  "1"  /* Bypass Upper Esophagus to Stomach, Endo */
"0D118Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Endo */
"0D118ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Endo */
"0D118ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Endo */
"0D12074"  =  "1"  /* Bypass Mid Esophag to Cutan with Autol Sub, Open Approach */
"0D12076"  =  "1"  /* Bypass Mid Esophag to Stomach with Autol Sub, Open Approach */
"0D12079"  =  "1"  /* Bypass Mid Esophag to Duoden with Autol Sub, Open Approach */
"0D1207A"  =  "1"  /* Bypass Mid Esophag to Jejunum with Autol Sub, Open Approach */
"0D1207B"  =  "1"  /* Bypass Mid Esophag to Ileum with Autol Sub, Open Approach */
"0D120J4"  =  "1"  /* Bypass Mid Esophag to Cutan with Synth Sub, Open Approach */
"0D120J6"  =  "1"  /* Bypass Mid Esophag to Stomach with Synth Sub, Open Approach */
"0D120J9"  =  "1"  /* Bypass Mid Esophag to Duoden with Synth Sub, Open Approach */
"0D120JA"  =  "1"  /* Bypass Mid Esophag to Jejunum with Synth Sub, Open Approach */
"0D120JB"  =  "1"  /* Bypass Mid Esophag to Ileum with Synth Sub, Open Approach */
"0D120K4"  =  "1"  /* Bypass Mid Esophag to Cutan with Nonaut Sub, Open Approach */
"0D120K6"  =  "1"  /* Bypass Mid Esophag to Stomach with Nonaut Sub, Open Approach */
"0D120K9"  =  "1"  /* Bypass Mid Esophag to Duoden with Nonaut Sub, Open Approach */
"0D120KA"  =  "1"  /* Bypass Mid Esophag to Jejunum with Nonaut Sub, Open Approach */
"0D120KB"  =  "1"  /* Bypass Mid Esophag to Ileum with Nonaut Sub, Open Approach */
"0D120Z4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous, Open Approach */
"0D120Z6"  =  "1"  /* Bypass Middle Esophagus to Stomach, Open Approach */
"0D120Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Open Approach */
"0D120ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Open Approach */
"0D120ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Open Approach */
"0D123J4"  =  "1"  /* Bypass Mid Esophag to Cutan with Synth Sub, Perc Approach */
"0D12474"  =  "1"  /* Bypass Mid Esophag to Cutan w Autol Sub, Perc Endo */
"0D12476"  =  "1"  /* Bypass Mid Esophag to Stomach w Autol Sub, Perc Endo */
"0D12479"  =  "1"  /* Bypass Mid Esophag to Duoden w Autol Sub, Perc Endo */
"0D1247A"  =  "1"  /* Bypass Mid Esophag to Jejunum w Autol Sub, Perc Endo */
"0D1247B"  =  "1"  /* Bypass Mid Esophag to Ileum w Autol Sub, Perc Endo */
"0D124J4"  =  "1"  /* Bypass Mid Esophag to Cutan w Synth Sub, Perc Endo */
"0D124J6"  =  "1"  /* Bypass Mid Esophag to Stomach w Synth Sub, Perc Endo */
"0D124J9"  =  "1"  /* Bypass Mid Esophag to Duoden w Synth Sub, Perc Endo */
"0D124JA"  =  "1"  /* Bypass Mid Esophag to Jejunum w Synth Sub, Perc Endo */
"0D124JB"  =  "1"  /* Bypass Mid Esophag to Ileum w Synth Sub, Perc Endo */
"0D124K4"  =  "1"  /* Bypass Mid Esophag to Cutan w Nonaut Sub, Perc Endo */
"0D124K6"  =  "1"  /* Bypass Mid Esophag to Stomach w Nonaut Sub, Perc Endo */
"0D124K9"  =  "1"  /* Bypass Mid Esophag to Duoden w Nonaut Sub, Perc Endo */
"0D124KA"  =  "1"  /* Bypass Mid Esophag to Jejunum w Nonaut Sub, Perc Endo */
"0D124KB"  =  "1"  /* Bypass Mid Esophag to Ileum w Nonaut Sub, Perc Endo */
"0D124Z4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous, Perc Endo Approach */
"0D124Z6"  =  "1"  /* Bypass Middle Esophagus to Stomach, Perc Endo Approach */
"0D124Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Perc Endo Approach */
"0D124ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Perc Endo Approach */
"0D124ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Perc Endo Approach */
"0D12874"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Autol Sub, Endo */
"0D12876"  =  "1"  /* Bypass Middle Esophagus to Stomach with Autol Sub, Endo */
"0D12879"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Autol Sub, Endo */
"0D1287A"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Autol Sub, Endo */
"0D1287B"  =  "1"  /* Bypass Middle Esophagus to Ileum with Autol Sub, Endo */
"0D128J4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Synth Sub, Endo */
"0D128J6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Synth Sub, Endo */
"0D128J9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Synth Sub, Endo */
"0D128JA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Synth Sub, Endo */
"0D128JB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Synth Sub, Endo */
"0D128K4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Nonaut Sub, Endo */
"0D128K6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Nonaut Sub, Endo */
"0D128K9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Nonaut Sub, Endo */
"0D128KA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Nonaut Sub, Endo */
"0D128KB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Nonaut Sub, Endo */
"0D128Z4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous, Endo */
"0D128Z6"  =  "1"  /* Bypass Middle Esophagus to Stomach, Endo */
"0D128Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Endo */
"0D128ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Endo */
"0D128ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Endo */
"0D13074"  =  "1"  /* Bypass Low Esophag to Cutan with Autol Sub, Open Approach */
"0D13076"  =  "1"  /* Bypass Low Esophag to Stomach with Autol Sub, Open Approach */
"0D13079"  =  "1"  /* Bypass Low Esophag to Duoden with Autol Sub, Open Approach */
"0D1307A"  =  "1"  /* Bypass Low Esophag to Jejunum with Autol Sub, Open Approach */
"0D1307B"  =  "1"  /* Bypass Low Esophag to Ileum with Autol Sub, Open Approach */
"0D130J4"  =  "1"  /* Bypass Low Esophag to Cutan with Synth Sub, Open Approach */
"0D130J6"  =  "1"  /* Bypass Low Esophag to Stomach with Synth Sub, Open Approach */
"0D130J9"  =  "1"  /* Bypass Low Esophag to Duoden with Synth Sub, Open Approach */
"0D130JA"  =  "1"  /* Bypass Low Esophag to Jejunum with Synth Sub, Open Approach */
"0D130JB"  =  "1"  /* Bypass Low Esophag to Ileum with Synth Sub, Open Approach */
"0D130K4"  =  "1"  /* Bypass Low Esophag to Cutan with Nonaut Sub, Open Approach */
"0D130K6"  =  "1"  /* Bypass Low Esophag to Stomach with Nonaut Sub, Open Approach */
"0D130K9"  =  "1"  /* Bypass Low Esophag to Duoden with Nonaut Sub, Open Approach */
"0D130KA"  =  "1"  /* Bypass Low Esophag to Jejunum with Nonaut Sub, Open Approach */
"0D130KB"  =  "1"  /* Bypass Low Esophag to Ileum with Nonaut Sub, Open Approach */
"0D130Z4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous, Open Approach */
"0D130Z6"  =  "1"  /* Bypass Lower Esophagus to Stomach, Open Approach */
"0D130Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Open Approach */
"0D130ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Open Approach */
"0D130ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Open Approach */
"0D133J4"  =  "1"  /* Bypass Low Esophag to Cutan with Synth Sub, Perc Approach */
"0D13474"  =  "1"  /* Bypass Low Esophag to Cutan w Autol Sub, Perc Endo */
"0D13476"  =  "1"  /* Bypass Low Esophag to Stomach w Autol Sub, Perc Endo */
"0D13479"  =  "1"  /* Bypass Low Esophag to Duoden w Autol Sub, Perc Endo */
"0D1347A"  =  "1"  /* Bypass Low Esophag to Jejunum w Autol Sub, Perc Endo */
"0D1347B"  =  "1"  /* Bypass Low Esophag to Ileum w Autol Sub, Perc Endo */
"0D134J4"  =  "1"  /* Bypass Low Esophag to Cutan w Synth Sub, Perc Endo */
"0D134J6"  =  "1"  /* Bypass Low Esophag to Stomach w Synth Sub, Perc Endo */
"0D134J9"  =  "1"  /* Bypass Low Esophag to Duoden w Synth Sub, Perc Endo */
"0D134JA"  =  "1"  /* Bypass Low Esophag to Jejunum w Synth Sub, Perc Endo */
"0D134JB"  =  "1"  /* Bypass Low Esophag to Ileum w Synth Sub, Perc Endo */
"0D134K4"  =  "1"  /* Bypass Low Esophag to Cutan w Nonaut Sub, Perc Endo */
"0D134K6"  =  "1"  /* Bypass Low Esophag to Stomach w Nonaut Sub, Perc Endo */
"0D134K9"  =  "1"  /* Bypass Low Esophag to Duoden w Nonaut Sub, Perc Endo */
"0D134KA"  =  "1"  /* Bypass Low Esophag to Jejunum w Nonaut Sub, Perc Endo */
"0D134KB"  =  "1"  /* Bypass Low Esophag to Ileum w Nonaut Sub, Perc Endo */
"0D134Z4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous, Perc Endo Approach */
"0D134Z6"  =  "1"  /* Bypass Lower Esophagus to Stomach, Perc Endo Approach */
"0D134Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Perc Endo Approach */
"0D134ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Perc Endo Approach */
"0D134ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Perc Endo Approach */
"0D13874"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Autol Sub, Endo */
"0D13876"  =  "1"  /* Bypass Lower Esophagus to Stomach with Autol Sub, Endo */
"0D13879"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Autol Sub, Endo */
"0D1387A"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Autol Sub, Endo */
"0D1387B"  =  "1"  /* Bypass Lower Esophagus to Ileum with Autol Sub, Endo */
"0D138J4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Synth Sub, Endo */
"0D138J6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Synth Sub, Endo */
"0D138J9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Synth Sub, Endo */
"0D138JA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Synth Sub, Endo */
"0D138JB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Synth Sub, Endo */
"0D138K4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Nonaut Sub, Endo */
"0D138K6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Nonaut Sub, Endo */
"0D138K9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Nonaut Sub, Endo */
"0D138KA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Nonaut Sub, Endo */
"0D138KB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Nonaut Sub, Endo */
"0D138Z4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous, Endo */
"0D138Z6"  =  "1"  /* Bypass Lower Esophagus to Stomach, Endo */
"0D138Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Endo */
"0D138ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Endo */
"0D138ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Endo */
"0D15074"  =  "1"  /* Bypass Esophagus to Cutaneous with Autol Sub, Open Approach */
"0D15076"  =  "1"  /* Bypass Esophagus to Stomach with Autol Sub, Open Approach */
"0D15079"  =  "1"  /* Bypass Esophagus to Duodenum with Autol Sub, Open Approach */
"0D1507A"  =  "1"  /* Bypass Esophagus to Jejunum with Autol Sub, Open Approach */
"0D1507B"  =  "1"  /* Bypass Esophagus to Ileum with Autol Sub, Open Approach */
"0D150J4"  =  "1"  /* Bypass Esophagus to Cutaneous with Synth Sub, Open Approach */
"0D150J6"  =  "1"  /* Bypass Esophagus to Stomach with Synth Sub, Open Approach */
"0D150J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synth Sub, Open Approach */
"0D150JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synth Sub, Open Approach */
"0D150JB"  =  "1"  /* Bypass Esophagus to Ileum with Synth Sub, Open Approach */
"0D150K4"  =  "1"  /* Bypass Esophagus to Cutaneous with Nonaut Sub, Open Approach */
"0D150K6"  =  "1"  /* Bypass Esophagus to Stomach with Nonaut Sub, Open Approach */
"0D150K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonaut Sub, Open Approach */
"0D150KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonaut Sub, Open Approach */
"0D150KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonaut Sub, Open Approach */
"0D150Z4"  =  "1"  /* Bypass Esophagus to Cutaneous, Open Approach */
"0D150Z6"  =  "1"  /* Bypass Esophagus to Stomach, Open Approach */
"0D150Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Open Approach */
"0D150ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Open Approach */
"0D150ZB"  =  "1"  /* Bypass Esophagus to Ileum, Open Approach */
"0D153J4"  =  "1"  /* Bypass Esophagus to Cutaneous with Synth Sub, Perc Approach */
"0D15474"  =  "1"  /* Bypass Esophagus to Cutan with Autol Sub, Perc Endo Approach */
"0D15476"  =  "1"  /* Bypass Esophag to Stomach with Autol Sub, Perc Endo Approach */
"0D15479"  =  "1"  /* Bypass Esophag to Duoden with Autol Sub, Perc Endo Approach */
"0D1547A"  =  "1"  /* Bypass Esophag to Jejunum with Autol Sub, Perc Endo Approach */
"0D1547B"  =  "1"  /* Bypass Esophagus to Ileum with Autol Sub, Perc Endo Approach */
"0D154J4"  =  "1"  /* Bypass Esophagus to Cutan with Synth Sub, Perc Endo Approach */
"0D154J6"  =  "1"  /* Bypass Esophag to Stomach with Synth Sub, Perc Endo Approach */
"0D154J9"  =  "1"  /* Bypass Esophag to Duoden with Synth Sub, Perc Endo Approach */
"0D154JA"  =  "1"  /* Bypass Esophag to Jejunum with Synth Sub, Perc Endo Approach */
"0D154JB"  =  "1"  /* Bypass Esophagus to Ileum with Synth Sub, Perc Endo Approach */
"0D154K4"  =  "1"  /* Bypass Esophag to Cutan with Nonaut Sub, Perc Endo Approach */
"0D154K6"  =  "1"  /* Bypass Esophag to Stomach w Nonaut Sub, Perc Endo */
"0D154K9"  =  "1"  /* Bypass Esophag to Duoden with Nonaut Sub, Perc Endo Approach */
"0D154KA"  =  "1"  /* Bypass Esophag to Jejunum w Nonaut Sub, Perc Endo */
"0D154KB"  =  "1"  /* Bypass Esophag to Ileum with Nonaut Sub, Perc Endo Approach */
"0D154Z4"  =  "1"  /* Bypass Esophagus to Cutaneous, Perc Endo Approach */
"0D154Z6"  =  "1"  /* Bypass Esophagus to Stomach, Perc Endo Approach */
"0D154Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Perc Endo Approach */
"0D154ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Perc Endo Approach */
"0D154ZB"  =  "1"  /* Bypass Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D15874"  =  "1"  /* Bypass Esophagus to Cutaneous with Autol Sub, Endo */
"0D15876"  =  "1"  /* Bypass Esophagus to Stomach with Autol Sub, Endo */
"0D15879"  =  "1"  /* Bypass Esophagus to Duodenum with Autol Sub, Endo */
"0D1587A"  =  "1"  /* Bypass Esophagus to Jejunum with Autol Sub, Endo */
"0D1587B"  =  "1"  /* Bypass Esophagus to Ileum with Autol Sub, Endo */
"0D158J4"  =  "1"  /* Bypass Esophagus to Cutaneous with Synth Sub, Endo */
"0D158J6"  =  "1"  /* Bypass Esophagus to Stomach with Synthetic Substitute, Endo */
"0D158J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synthetic Substitute, Endo */
"0D158JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synthetic Substitute, Endo */
"0D158JB"  =  "1"  /* Bypass Esophagus to Ileum with Synthetic Substitute, Endo */
"0D158K4"  =  "1"  /* Bypass Esophagus to Cutaneous with Nonaut Sub, Endo */
"0D158K6"  =  "1"  /* Bypass Esophagus to Stomach with Nonaut Sub, Endo */
"0D158K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonaut Sub, Endo */
"0D158KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonaut Sub, Endo */
"0D158KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonaut Sub, Endo */
"0D158Z4"  =  "1"  /* Bypass Esophagus to Cutaneous, Endo */
"0D158Z6"  =  "1"  /* Bypass Esophagus to Stomach, Endo */
"0D158Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Endo */
"0D158ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Endo */
"0D158ZB"  =  "1"  /* Bypass Esophagus to Ileum, Endo */
"0D16079"  =  "1"  /* Bypass Stomach to Duodenum with Autol Sub, Open Approach */
"0D1607A"  =  "1"  /* Bypass Stomach to Jejunum with Autol Sub, Open Approach */
"0D1607B"  =  "1"  /* Bypass Stomach to Ileum with Autol Sub, Open Approach */
"0D1607L"  =  "1"  /* Bypass Stomach to Trans Colon with Autol Sub, Open Approach */
"0D160J9"  =  "1"  /* Bypass Stomach to Duodenum with Synth Sub, Open Approach */
"0D160JA"  =  "1"  /* Bypass Stomach to Jejunum with Synth Sub, Open Approach */
"0D160JB"  =  "1"  /* Bypass Stomach to Ileum with Synth Sub, Open Approach */
"0D160JL"  =  "1"  /* Bypass Stomach to Trans Colon with Synth Sub, Open Approach */
"0D160K9"  =  "1"  /* Bypass Stomach to Duodenum with Nonaut Sub, Open Approach */
"0D160KA"  =  "1"  /* Bypass Stomach to Jejunum with Nonaut Sub, Open Approach */
"0D160KB"  =  "1"  /* Bypass Stomach to Ileum with Nonaut Sub, Open Approach */
"0D160KL"  =  "1"  /* Bypass Stomach to Trans Colon with Nonaut Sub, Open Approach */
"0D160Z9"  =  "1"  /* Bypass Stomach to Duodenum, Open Approach */
"0D160ZA"  =  "1"  /* Bypass Stomach to Jejunum, Open Approach */
"0D160ZB"  =  "1"  /* Bypass Stomach to Ileum, Open Approach */
"0D160ZL"  =  "1"  /* Bypass Stomach to Transverse Colon, Open Approach */
"0D16479"  =  "1"  /* Bypass Stomach to Duoden with Autol Sub, Perc Endo Approach */
"0D1647A"  =  "1"  /* Bypass Stomach to Jejunum with Autol Sub, Perc Endo Approach */
"0D1647B"  =  "1"  /* Bypass Stomach to Ileum with Autol Sub, Perc Endo Approach */
"0D1647L"  =  "1"  /* Bypass Stomach to Trans Colon w Autol Sub, Perc Endo */
"0D164J9"  =  "1"  /* Bypass Stomach to Duoden with Synth Sub, Perc Endo Approach */
"0D164JA"  =  "1"  /* Bypass Stomach to Jejunum with Synth Sub, Perc Endo Approach */
"0D164JB"  =  "1"  /* Bypass Stomach to Ileum with Synth Sub, Perc Endo Approach */
"0D164JL"  =  "1"  /* Bypass Stomach to Trans Colon w Synth Sub, Perc Endo */
"0D164K9"  =  "1"  /* Bypass Stomach to Duoden with Nonaut Sub, Perc Endo Approach */
"0D164KA"  =  "1"  /* Bypass Stomach to Jejunum w Nonaut Sub, Perc Endo */
"0D164KB"  =  "1"  /* Bypass Stomach to Ileum with Nonaut Sub, Perc Endo Approach */
"0D164KL"  =  "1"  /* Bypass Stomach to Trans Colon w Nonaut Sub, Perc Endo */
"0D164Z9"  =  "1"  /* Bypass Stomach to Duodenum, Percutaneous Endoscopic Approach */
"0D164ZA"  =  "1"  /* Bypass Stomach to Jejunum, Percutaneous Endoscopic Approach */
"0D164ZB"  =  "1"  /* Bypass Stomach to Ileum, Percutaneous Endoscopic Approach */
"0D164ZL"  =  "1"  /* Bypass Stomach to Transverse Colon, Perc Endo Approach */
"0D16879"  =  "1"  /* Bypass Stomach to Duodenum with Autol Sub, Endo */
"0D1687A"  =  "1"  /* Bypass Stomach to Jejunum with Autol Sub, Endo */
"0D1687B"  =  "1"  /* Bypass Stomach to Ileum with Autol Sub, Endo */
"0D1687L"  =  "1"  /* Bypass Stomach to Transverse Colon with Autol Sub, Endo */
"0D168J9"  =  "1"  /* Bypass Stomach to Duodenum with Synthetic Substitute, Endo */
"0D168JA"  =  "1"  /* Bypass Stomach to Jejunum with Synthetic Substitute, Endo */
"0D168JB"  =  "1"  /* Bypass Stomach to Ileum with Synthetic Substitute, Endo */
"0D168JL"  =  "1"  /* Bypass Stomach to Transverse Colon with Synth Sub, Endo */
"0D168K9"  =  "1"  /* Bypass Stomach to Duodenum with Nonaut Sub, Endo */
"0D168KA"  =  "1"  /* Bypass Stomach to Jejunum with Nonaut Sub, Endo */
"0D168KB"  =  "1"  /* Bypass Stomach to Ileum with Nonaut Sub, Endo */
"0D168KL"  =  "1"  /* Bypass Stomach to Transverse Colon with Nonaut Sub, Endo */
"0D168Z9"  =  "1"  /* Bypass Stomach to Duodenum, Endo */
"0D168ZA"  =  "1"  /* Bypass Stomach to Jejunum, Endo */
"0D168ZB"  =  "1"  /* Bypass Stomach to Ileum, Endo */
"0D168ZL"  =  "1"  /* Bypass Stomach to Transverse Colon, Endo */
"0D19074"  =  "1"  /* Bypass Duodenum to Cutaneous with Autol Sub, Open Approach */
"0D19079"  =  "1"  /* Bypass Duodenum to Duodenum with Autol Sub, Open Approach */
"0D1907A"  =  "1"  /* Bypass Duodenum to Jejunum with Autol Sub, Open Approach */
"0D1907B"  =  "1"  /* Bypass Duodenum to Ileum with Autol Sub, Open Approach */
"0D1907L"  =  "1"  /* Bypass Duodenum to Trans Colon with Autol Sub, Open Approach */
"0D190J4"  =  "1"  /* Bypass Duodenum to Cutaneous with Synth Sub, Open Approach */
"0D190J9"  =  "1"  /* Bypass Duodenum to Duodenum with Synth Sub, Open Approach */
"0D190JA"  =  "1"  /* Bypass Duodenum to Jejunum with Synth Sub, Open Approach */
"0D190JB"  =  "1"  /* Bypass Duodenum to Ileum with Synth Sub, Open Approach */
"0D190JL"  =  "1"  /* Bypass Duodenum to Trans Colon with Synth Sub, Open Approach */
"0D190K4"  =  "1"  /* Bypass Duodenum to Cutaneous with Nonaut Sub, Open Approach */
"0D190K9"  =  "1"  /* Bypass Duodenum to Duodenum with Nonaut Sub, Open Approach */
"0D190KA"  =  "1"  /* Bypass Duodenum to Jejunum with Nonaut Sub, Open Approach */
"0D190KB"  =  "1"  /* Bypass Duodenum to Ileum with Nonaut Sub, Open Approach */
"0D190KL"  =  "1"  /* Bypass Duodenum to Trans Colon w Nonaut Sub, Open */
"0D190Z4"  =  "1"  /* Bypass Duodenum to Cutaneous, Open Approach */
"0D190Z9"  =  "1"  /* Bypass Duodenum to Duodenum, Open Approach */
"0D190ZA"  =  "1"  /* Bypass Duodenum to Jejunum, Open Approach */
"0D190ZB"  =  "1"  /* Bypass Duodenum to Ileum, Open Approach */
"0D190ZL"  =  "1"  /* Bypass Duodenum to Transverse Colon, Open Approach */
"0D193J4"  =  "1"  /* Bypass Duodenum to Cutaneous with Synth Sub, Perc Approach */
"0D19474"  =  "1"  /* Bypass Duodenum to Cutan with Autol Sub, Perc Endo Approach */
"0D19479"  =  "1"  /* Bypass Duodenum to Duoden with Autol Sub, Perc Endo Approach */
"0D1947A"  =  "1"  /* Bypass Duodenum to Jejunum w Autol Sub, Perc Endo */
"0D1947B"  =  "1"  /* Bypass Duodenum to Ileum with Autol Sub, Perc Endo Approach */
"0D1947L"  =  "1"  /* Bypass Duodenum to Trans Colon w Autol Sub, Perc Endo */
"0D194J4"  =  "1"  /* Bypass Duodenum to Cutan with Synth Sub, Perc Endo Approach */
"0D194J9"  =  "1"  /* Bypass Duodenum to Duoden with Synth Sub, Perc Endo Approach */
"0D194JA"  =  "1"  /* Bypass Duodenum to Jejunum w Synth Sub, Perc Endo */
"0D194JB"  =  "1"  /* Bypass Duodenum to Ileum with Synth Sub, Perc Endo Approach */
"0D194JL"  =  "1"  /* Bypass Duodenum to Trans Colon w Synth Sub, Perc Endo */
"0D194K4"  =  "1"  /* Bypass Duodenum to Cutan with Nonaut Sub, Perc Endo Approach */
"0D194K9"  =  "1"  /* Bypass Duodenum to Duoden w Nonaut Sub, Perc Endo */
"0D194KA"  =  "1"  /* Bypass Duodenum to Jejunum w Nonaut Sub, Perc Endo */
"0D194KB"  =  "1"  /* Bypass Duodenum to Ileum with Nonaut Sub, Perc Endo Approach */
"0D194KL"  =  "1"  /* Bypass Duodenum to Trans Colon w Nonaut Sub, Perc Endo */
"0D194Z4"  =  "1"  /* Bypass Duodenum to Cutaneous, Perc Endo Approach */
"0D194Z9"  =  "1"  /* Bypass Duodenum to Duodenum, Perc Endo Approach */
"0D194ZA"  =  "1"  /* Bypass Duodenum to Jejunum, Percutaneous Endoscopic Approach */
"0D194ZB"  =  "1"  /* Bypass Duodenum to Ileum, Percutaneous Endoscopic Approach */
"0D194ZL"  =  "1"  /* Bypass Duodenum to Transverse Colon, Perc Endo Approach */
"0D19874"  =  "1"  /* Bypass Duodenum to Cutaneous with Autol Sub, Endo */
"0D19879"  =  "1"  /* Bypass Duodenum to Duodenum with Autol Sub, Endo */
"0D1987A"  =  "1"  /* Bypass Duodenum to Jejunum with Autol Sub, Endo */
"0D1987B"  =  "1"  /* Bypass Duodenum to Ileum with Autol Sub, Endo */
"0D1987L"  =  "1"  /* Bypass Duodenum to Transverse Colon with Autol Sub, Endo */
"0D198J4"  =  "1"  /* Bypass Duodenum to Cutaneous with Synthetic Substitute, Endo */
"0D198J9"  =  "1"  /* Bypass Duodenum to Duodenum with Synthetic Substitute, Endo */
"0D198JA"  =  "1"  /* Bypass Duodenum to Jejunum with Synthetic Substitute, Endo */
"0D198JB"  =  "1"  /* Bypass Duodenum to Ileum with Synthetic Substitute, Endo */
"0D198JL"  =  "1"  /* Bypass Duodenum to Transverse Colon with Synth Sub, Endo */
"0D198K4"  =  "1"  /* Bypass Duodenum to Cutaneous with Nonaut Sub, Endo */
"0D198K9"  =  "1"  /* Bypass Duodenum to Duodenum with Nonaut Sub, Endo */
"0D198KA"  =  "1"  /* Bypass Duodenum to Jejunum with Nonaut Sub, Endo */
"0D198KB"  =  "1"  /* Bypass Duodenum to Ileum with Nonaut Sub, Endo */
"0D198KL"  =  "1"  /* Bypass Duodenum to Transverse Colon with Nonaut Sub, Endo */
"0D198Z4"  =  "1"  /* Bypass Duodenum to Cutaneous, Endo */
"0D198Z9"  =  "1"  /* Bypass Duodenum to Duodenum, Endo */
"0D198ZA"  =  "1"  /* Bypass Duodenum to Jejunum, Endo */
"0D198ZB"  =  "1"  /* Bypass Duodenum to Ileum, Endo */
"0D198ZL"  =  "1"  /* Bypass Duodenum to Transverse Colon, Endo */
"0D1A074"  =  "1"  /* Bypass Jejunum to Cutaneous with Autol Sub, Open Approach */
"0D1A07A"  =  "1"  /* Bypass Jejunum to Jejunum with Autol Sub, Open Approach */
"0D1A07B"  =  "1"  /* Bypass Jejunum to Ileum with Autol Sub, Open Approach */
"0D1A07H"  =  "1"  /* Bypass Jejunum to Cecum with Autol Sub, Open Approach */
"0D1A07K"  =  "1"  /* Bypass Jejunum to Asc Colon with Autol Sub, Open Approach */
"0D1A07L"  =  "1"  /* Bypass Jejunum to Trans Colon with Autol Sub, Open Approach */
"0D1A07M"  =  "1"  /* Bypass Jejunum to Desc Colon with Autol Sub, Open Approach */
"0D1A07N"  =  "1"  /* Bypass Jejunum to Sigm Colon with Autol Sub, Open Approach */
"0D1A07P"  =  "1"  /* Bypass Jejunum to Rectum with Autol Sub, Open Approach */
"0D1A07Q"  =  "1"  /* Bypass Jejunum to Anus with Autol Sub, Open Approach */
"0D1A0J4"  =  "1"  /* Bypass Jejunum to Cutaneous with Synth Sub, Open Approach */
"0D1A0JA"  =  "1"  /* Bypass Jejunum to Jejunum with Synth Sub, Open Approach */
"0D1A0JB"  =  "1"  /* Bypass Jejunum to Ileum with Synth Sub, Open Approach */
"0D1A0JH"  =  "1"  /* Bypass Jejunum to Cecum with Synth Sub, Open Approach */
"0D1A0JK"  =  "1"  /* Bypass Jejunum to Asc Colon with Synth Sub, Open Approach */
"0D1A0JL"  =  "1"  /* Bypass Jejunum to Trans Colon with Synth Sub, Open Approach */
"0D1A0JM"  =  "1"  /* Bypass Jejunum to Desc Colon with Synth Sub, Open Approach */
"0D1A0JN"  =  "1"  /* Bypass Jejunum to Sigm Colon with Synth Sub, Open Approach */
"0D1A0JP"  =  "1"  /* Bypass Jejunum to Rectum with Synth Sub, Open Approach */
"0D1A0JQ"  =  "1"  /* Bypass Jejunum to Anus with Synth Sub, Open Approach */
"0D1A0K4"  =  "1"  /* Bypass Jejunum to Cutaneous with Nonaut Sub, Open Approach */
"0D1A0KA"  =  "1"  /* Bypass Jejunum to Jejunum with Nonaut Sub, Open Approach */
"0D1A0KB"  =  "1"  /* Bypass Jejunum to Ileum with Nonaut Sub, Open Approach */
"0D1A0KH"  =  "1"  /* Bypass Jejunum to Cecum with Nonaut Sub, Open Approach */
"0D1A0KK"  =  "1"  /* Bypass Jejunum to Asc Colon with Nonaut Sub, Open Approach */
"0D1A0KL"  =  "1"  /* Bypass Jejunum to Trans Colon with Nonaut Sub, Open Approach */
"0D1A0KM"  =  "1"  /* Bypass Jejunum to Desc Colon with Nonaut Sub, Open Approach */
"0D1A0KN"  =  "1"  /* Bypass Jejunum to Sigm Colon with Nonaut Sub, Open Approach */
"0D1A0KP"  =  "1"  /* Bypass Jejunum to Rectum with Nonaut Sub, Open Approach */
"0D1A0KQ"  =  "1"  /* Bypass Jejunum to Anus with Nonaut Sub, Open Approach */
"0D1A0Z4"  =  "1"  /* Bypass Jejunum to Cutaneous, Open Approach */
"0D1A0ZA"  =  "1"  /* Bypass Jejunum to Jejunum, Open Approach */
"0D1A0ZB"  =  "1"  /* Bypass Jejunum to Ileum, Open Approach */
"0D1A0ZH"  =  "1"  /* Bypass Jejunum to Cecum, Open Approach */
"0D1A0ZK"  =  "1"  /* Bypass Jejunum to Ascending Colon, Open Approach */
"0D1A0ZL"  =  "1"  /* Bypass Jejunum to Transverse Colon, Open Approach */
"0D1A0ZM"  =  "1"  /* Bypass Jejunum to Descending Colon, Open Approach */
"0D1A0ZN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon, Open Approach */
"0D1A0ZP"  =  "1"  /* Bypass Jejunum to Rectum, Open Approach */
"0D1A0ZQ"  =  "1"  /* Bypass Jejunum to Anus, Open Approach */
"0D1A3J4"  =  "1"  /* Bypass Jejunum to Cutaneous with Synth Sub, Perc Approach */
"0D1A474"  =  "1"  /* Bypass Jejunum to Cutan with Autol Sub, Perc Endo Approach */
"0D1A47A"  =  "1"  /* Bypass Jejunum to Jejunum with Autol Sub, Perc Endo Approach */
"0D1A47B"  =  "1"  /* Bypass Jejunum to Ileum with Autol Sub, Perc Endo Approach */
"0D1A47H"  =  "1"  /* Bypass Jejunum to Cecum with Autol Sub, Perc Endo Approach */
"0D1A47K"  =  "1"  /* Bypass Jejunum to Asc Colon w Autol Sub, Perc Endo */
"0D1A47L"  =  "1"  /* Bypass Jejunum to Trans Colon w Autol Sub, Perc Endo */
"0D1A47M"  =  "1"  /* Bypass Jejunum to Desc Colon w Autol Sub, Perc Endo */
"0D1A47N"  =  "1"  /* Bypass Jejunum to Sigm Colon w Autol Sub, Perc Endo */
"0D1A47P"  =  "1"  /* Bypass Jejunum to Rectum with Autol Sub, Perc Endo Approach */
"0D1A47Q"  =  "1"  /* Bypass Jejunum to Anus with Autol Sub, Perc Endo Approach */
"0D1A4J4"  =  "1"  /* Bypass Jejunum to Cutan with Synth Sub, Perc Endo Approach */
"0D1A4JA"  =  "1"  /* Bypass Jejunum to Jejunum with Synth Sub, Perc Endo Approach */
"0D1A4JB"  =  "1"  /* Bypass Jejunum to Ileum with Synth Sub, Perc Endo Approach */
"0D1A4JH"  =  "1"  /* Bypass Jejunum to Cecum with Synth Sub, Perc Endo Approach */
"0D1A4JK"  =  "1"  /* Bypass Jejunum to Asc Colon w Synth Sub, Perc Endo */
"0D1A4JL"  =  "1"  /* Bypass Jejunum to Trans Colon w Synth Sub, Perc Endo */
"0D1A4JM"  =  "1"  /* Bypass Jejunum to Desc Colon w Synth Sub, Perc Endo */
"0D1A4JN"  =  "1"  /* Bypass Jejunum to Sigm Colon w Synth Sub, Perc Endo */
"0D1A4JP"  =  "1"  /* Bypass Jejunum to Rectum with Synth Sub, Perc Endo Approach */
"0D1A4JQ"  =  "1"  /* Bypass Jejunum to Anus with Synth Sub, Perc Endo Approach */
"0D1A4K4"  =  "1"  /* Bypass Jejunum to Cutan with Nonaut Sub, Perc Endo Approach */
"0D1A4KA"  =  "1"  /* Bypass Jejunum to Jejunum w Nonaut Sub, Perc Endo */
"0D1A4KB"  =  "1"  /* Bypass Jejunum to Ileum with Nonaut Sub, Perc Endo Approach */
"0D1A4KH"  =  "1"  /* Bypass Jejunum to Cecum with Nonaut Sub, Perc Endo Approach */
"0D1A4KK"  =  "1"  /* Bypass Jejunum to Asc Colon w Nonaut Sub, Perc Endo */
"0D1A4KL"  =  "1"  /* Bypass Jejunum to Trans Colon w Nonaut Sub, Perc Endo */
"0D1A4KM"  =  "1"  /* Bypass Jejunum to Desc Colon w Nonaut Sub, Perc Endo */
"0D1A4KN"  =  "1"  /* Bypass Jejunum to Sigm Colon w Nonaut Sub, Perc Endo */
"0D1A4KP"  =  "1"  /* Bypass Jejunum to Rectum with Nonaut Sub, Perc Endo Approach */
"0D1A4KQ"  =  "1"  /* Bypass Jejunum to Anus with Nonaut Sub, Perc Endo Approach */
"0D1A4Z4"  =  "1"  /* Bypass Jejunum to Cutaneous, Perc Endo Approach */
"0D1A4ZA"  =  "1"  /* Bypass Jejunum to Jejunum, Percutaneous Endoscopic Approach */
"0D1A4ZB"  =  "1"  /* Bypass Jejunum to Ileum, Percutaneous Endoscopic Approach */
"0D1A4ZH"  =  "1"  /* Bypass Jejunum to Cecum, Percutaneous Endoscopic Approach */
"0D1A4ZK"  =  "1"  /* Bypass Jejunum to Ascending Colon, Perc Endo Approach */
"0D1A4ZL"  =  "1"  /* Bypass Jejunum to Transverse Colon, Perc Endo Approach */
"0D1A4ZM"  =  "1"  /* Bypass Jejunum to Descending Colon, Perc Endo Approach */
"0D1A4ZN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon, Perc Endo Approach */
"0D1A4ZP"  =  "1"  /* Bypass Jejunum to Rectum, Percutaneous Endoscopic Approach */
"0D1A4ZQ"  =  "1"  /* Bypass Jejunum to Anus, Percutaneous Endoscopic Approach */
"0D1A874"  =  "1"  /* Bypass Jejunum to Cutaneous with Autol Sub, Endo */
"0D1A87A"  =  "1"  /* Bypass Jejunum to Jejunum with Autol Sub, Endo */
"0D1A87B"  =  "1"  /* Bypass Jejunum to Ileum with Autol Sub, Endo */
"0D1A87H"  =  "1"  /* Bypass Jejunum to Cecum with Autol Sub, Endo */
"0D1A87K"  =  "1"  /* Bypass Jejunum to Ascending Colon with Autol Sub, Endo */
"0D1A87L"  =  "1"  /* Bypass Jejunum to Transverse Colon with Autol Sub, Endo */
"0D1A87M"  =  "1"  /* Bypass Jejunum to Descending Colon with Autol Sub, Endo */
"0D1A87N"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Autol Sub, Endo */
"0D1A87P"  =  "1"  /* Bypass Jejunum to Rectum with Autol Sub, Endo */
"0D1A87Q"  =  "1"  /* Bypass Jejunum to Anus with Autol Sub, Endo */
"0D1A8J4"  =  "1"  /* Bypass Jejunum to Cutaneous with Synthetic Substitute, Endo */
"0D1A8JA"  =  "1"  /* Bypass Jejunum to Jejunum with Synthetic Substitute, Endo */
"0D1A8JB"  =  "1"  /* Bypass Jejunum to Ileum with Synthetic Substitute, Endo */
"0D1A8JH"  =  "1"  /* Bypass Jejunum to Cecum with Synthetic Substitute, Endo */
"0D1A8JK"  =  "1"  /* Bypass Jejunum to Ascending Colon with Synth Sub, Endo */
"0D1A8JL"  =  "1"  /* Bypass Jejunum to Transverse Colon with Synth Sub, Endo */
"0D1A8JM"  =  "1"  /* Bypass Jejunum to Descending Colon with Synth Sub, Endo */
"0D1A8JN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Synth Sub, Endo */
"0D1A8JP"  =  "1"  /* Bypass Jejunum to Rectum with Synthetic Substitute, Endo */
"0D1A8JQ"  =  "1"  /* Bypass Jejunum to Anus with Synthetic Substitute, Endo */
"0D1A8K4"  =  "1"  /* Bypass Jejunum to Cutaneous with Nonaut Sub, Endo */
"0D1A8KA"  =  "1"  /* Bypass Jejunum to Jejunum with Nonaut Sub, Endo */
"0D1A8KB"  =  "1"  /* Bypass Jejunum to Ileum with Nonaut Sub, Endo */
"0D1A8KH"  =  "1"  /* Bypass Jejunum to Cecum with Nonaut Sub, Endo */
"0D1A8KK"  =  "1"  /* Bypass Jejunum to Ascending Colon with Nonaut Sub, Endo */
"0D1A8KL"  =  "1"  /* Bypass Jejunum to Transverse Colon with Nonaut Sub, Endo */
"0D1A8KM"  =  "1"  /* Bypass Jejunum to Descending Colon with Nonaut Sub, Endo */
"0D1A8KN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon with Nonaut Sub, Endo */
"0D1A8KP"  =  "1"  /* Bypass Jejunum to Rectum with Nonaut Sub, Endo */
"0D1A8KQ"  =  "1"  /* Bypass Jejunum to Anus with Nonaut Sub, Endo */
"0D1A8Z4"  =  "1"  /* Bypass Jejunum to Cutaneous, Endo */
"0D1A8ZA"  =  "1"  /* Bypass Jejunum to Jejunum, Endo */
"0D1A8ZB"  =  "1"  /* Bypass Jejunum to Ileum, Endo */
"0D1A8ZH"  =  "1"  /* Bypass Jejunum to Cecum, Endo */
"0D1A8ZK"  =  "1"  /* Bypass Jejunum to Ascending Colon, Endo */
"0D1A8ZL"  =  "1"  /* Bypass Jejunum to Transverse Colon, Endo */
"0D1A8ZM"  =  "1"  /* Bypass Jejunum to Descending Colon, Endo */
"0D1A8ZN"  =  "1"  /* Bypass Jejunum to Sigmoid Colon, Endo */
"0D1A8ZP"  =  "1"  /* Bypass Jejunum to Rectum, Endo */
"0D1A8ZQ"  =  "1"  /* Bypass Jejunum to Anus, Endo */
"0D1B074"  =  "1"  /* Bypass Ileum to Cutaneous with Autol Sub, Open Approach */
"0D1B07B"  =  "1"  /* Bypass Ileum to Ileum with Autol Sub, Open Approach */
"0D1B07H"  =  "1"  /* Bypass Ileum to Cecum with Autol Sub, Open Approach */
"0D1B07K"  =  "1"  /* Bypass Ileum to Asc Colon with Autol Sub, Open Approach */
"0D1B07L"  =  "1"  /* Bypass Ileum to Trans Colon with Autol Sub, Open Approach */
"0D1B07M"  =  "1"  /* Bypass Ileum to Desc Colon with Autol Sub, Open Approach */
"0D1B07N"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Autol Sub, Open Approach */
"0D1B07P"  =  "1"  /* Bypass Ileum to Rectum with Autol Sub, Open Approach */
"0D1B07Q"  =  "1"  /* Bypass Ileum to Anus with Autol Sub, Open Approach */
"0D1B0J4"  =  "1"  /* Bypass Ileum to Cutaneous with Synth Sub, Open Approach */
"0D1B0JB"  =  "1"  /* Bypass Ileum to Ileum with Synth Sub, Open Approach */
"0D1B0JH"  =  "1"  /* Bypass Ileum to Cecum with Synth Sub, Open Approach */
"0D1B0JK"  =  "1"  /* Bypass Ileum to Asc Colon with Synth Sub, Open Approach */
"0D1B0JL"  =  "1"  /* Bypass Ileum to Trans Colon with Synth Sub, Open Approach */
"0D1B0JM"  =  "1"  /* Bypass Ileum to Desc Colon with Synth Sub, Open Approach */
"0D1B0JN"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Synth Sub, Open Approach */
"0D1B0JP"  =  "1"  /* Bypass Ileum to Rectum with Synth Sub, Open Approach */
"0D1B0JQ"  =  "1"  /* Bypass Ileum to Anus with Synth Sub, Open Approach */
"0D1B0K4"  =  "1"  /* Bypass Ileum to Cutaneous with Nonaut Sub, Open Approach */
"0D1B0KB"  =  "1"  /* Bypass Ileum to Ileum with Nonaut Sub, Open Approach */
"0D1B0KH"  =  "1"  /* Bypass Ileum to Cecum with Nonaut Sub, Open Approach */
"0D1B0KK"  =  "1"  /* Bypass Ileum to Asc Colon with Nonaut Sub, Open Approach */
"0D1B0KL"  =  "1"  /* Bypass Ileum to Trans Colon with Nonaut Sub, Open Approach */
"0D1B0KM"  =  "1"  /* Bypass Ileum to Desc Colon with Nonaut Sub, Open Approach */
"0D1B0KN"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Nonaut Sub, Open Approach */
"0D1B0KP"  =  "1"  /* Bypass Ileum to Rectum with Nonaut Sub, Open Approach */
"0D1B0KQ"  =  "1"  /* Bypass Ileum to Anus with Nonaut Sub, Open Approach */
"0D1B0Z4"  =  "1"  /* Bypass Ileum to Cutaneous, Open Approach */
"0D1B0ZB"  =  "1"  /* Bypass Ileum to Ileum, Open Approach */
"0D1B0ZH"  =  "1"  /* Bypass Ileum to Cecum, Open Approach */
"0D1B0ZK"  =  "1"  /* Bypass Ileum to Ascending Colon, Open Approach */
"0D1B0ZL"  =  "1"  /* Bypass Ileum to Transverse Colon, Open Approach */
"0D1B0ZM"  =  "1"  /* Bypass Ileum to Descending Colon, Open Approach */
"0D1B0ZN"  =  "1"  /* Bypass Ileum to Sigmoid Colon, Open Approach */
"0D1B0ZP"  =  "1"  /* Bypass Ileum to Rectum, Open Approach */
"0D1B0ZQ"  =  "1"  /* Bypass Ileum to Anus, Open Approach */
"0D1B3J4"  =  "1"  /* Bypass Ileum to Cutaneous with Synth Sub, Perc Approach */
"0D1B474"  =  "1"  /* Bypass Ileum to Cutaneous with Autol Sub, Perc Endo Approach */
"0D1B47B"  =  "1"  /* Bypass Ileum to Ileum with Autol Sub, Perc Endo Approach */
"0D1B47H"  =  "1"  /* Bypass Ileum to Cecum with Autol Sub, Perc Endo Approach */
"0D1B47K"  =  "1"  /* Bypass Ileum to Asc Colon with Autol Sub, Perc Endo Approach */
"0D1B47L"  =  "1"  /* Bypass Ileum to Trans Colon w Autol Sub, Perc Endo */
"0D1B47M"  =  "1"  /* Bypass Ileum to Desc Colon w Autol Sub, Perc Endo */
"0D1B47N"  =  "1"  /* Bypass Ileum to Sigm Colon w Autol Sub, Perc Endo */
"0D1B47P"  =  "1"  /* Bypass Ileum to Rectum with Autol Sub, Perc Endo Approach */
"0D1B47Q"  =  "1"  /* Bypass Ileum to Anus with Autol Sub, Perc Endo Approach */
"0D1B4J4"  =  "1"  /* Bypass Ileum to Cutaneous with Synth Sub, Perc Endo Approach */
"0D1B4JB"  =  "1"  /* Bypass Ileum to Ileum with Synth Sub, Perc Endo Approach */
"0D1B4JH"  =  "1"  /* Bypass Ileum to Cecum with Synth Sub, Perc Endo Approach */
"0D1B4JK"  =  "1"  /* Bypass Ileum to Asc Colon with Synth Sub, Perc Endo Approach */
"0D1B4JL"  =  "1"  /* Bypass Ileum to Trans Colon w Synth Sub, Perc Endo */
"0D1B4JM"  =  "1"  /* Bypass Ileum to Desc Colon w Synth Sub, Perc Endo */
"0D1B4JN"  =  "1"  /* Bypass Ileum to Sigm Colon w Synth Sub, Perc Endo */
"0D1B4JP"  =  "1"  /* Bypass Ileum to Rectum with Synth Sub, Perc Endo Approach */
"0D1B4JQ"  =  "1"  /* Bypass Ileum to Anus with Synth Sub, Perc Endo Approach */
"0D1B4K4"  =  "1"  /* Bypass Ileum to Cutan with Nonaut Sub, Perc Endo Approach */
"0D1B4KB"  =  "1"  /* Bypass Ileum to Ileum with Nonaut Sub, Perc Endo Approach */
"0D1B4KH"  =  "1"  /* Bypass Ileum to Cecum with Nonaut Sub, Perc Endo Approach */
"0D1B4KK"  =  "1"  /* Bypass Ileum to Asc Colon w Nonaut Sub, Perc Endo */
"0D1B4KL"  =  "1"  /* Bypass Ileum to Trans Colon w Nonaut Sub, Perc Endo */
"0D1B4KM"  =  "1"  /* Bypass Ileum to Desc Colon w Nonaut Sub, Perc Endo */
"0D1B4KN"  =  "1"  /* Bypass Ileum to Sigm Colon w Nonaut Sub, Perc Endo */
"0D1B4KP"  =  "1"  /* Bypass Ileum to Rectum with Nonaut Sub, Perc Endo Approach */
"0D1B4KQ"  =  "1"  /* Bypass Ileum to Anus with Nonaut Sub, Perc Endo Approach */
"0D1B4Z4"  =  "1"  /* Bypass Ileum to Cutaneous, Percutaneous Endoscopic Approach */
"0D1B4ZB"  =  "1"  /* Bypass Ileum to Ileum, Percutaneous Endoscopic Approach */
"0D1B4ZH"  =  "1"  /* Bypass Ileum to Cecum, Percutaneous Endoscopic Approach */
"0D1B4ZK"  =  "1"  /* Bypass Ileum to Ascending Colon, Perc Endo Approach */
"0D1B4ZL"  =  "1"  /* Bypass Ileum to Transverse Colon, Perc Endo Approach */
"0D1B4ZM"  =  "1"  /* Bypass Ileum to Descending Colon, Perc Endo Approach */
"0D1B4ZN"  =  "1"  /* Bypass Ileum to Sigmoid Colon, Perc Endo Approach */
"0D1B4ZP"  =  "1"  /* Bypass Ileum to Rectum, Percutaneous Endoscopic Approach */
"0D1B4ZQ"  =  "1"  /* Bypass Ileum to Anus, Percutaneous Endoscopic Approach */
"0D1B874"  =  "1"  /* Bypass Ileum to Cutaneous with Autol Sub, Endo */
"0D1B87B"  =  "1"  /* Bypass Ileum to Ileum with Autol Sub, Endo */
"0D1B87H"  =  "1"  /* Bypass Ileum to Cecum with Autol Sub, Endo */
"0D1B87K"  =  "1"  /* Bypass Ileum to Ascending Colon with Autol Sub, Endo */
"0D1B87L"  =  "1"  /* Bypass Ileum to Transverse Colon with Autol Sub, Endo */
"0D1B87M"  =  "1"  /* Bypass Ileum to Descending Colon with Autol Sub, Endo */
"0D1B87N"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Autol Sub, Endo */
"0D1B87P"  =  "1"  /* Bypass Ileum to Rectum with Autol Sub, Endo */
"0D1B87Q"  =  "1"  /* Bypass Ileum to Anus with Autologous Tissue Substitute, Endo */
"0D1B8J4"  =  "1"  /* Bypass Ileum to Cutaneous with Synthetic Substitute, Endo */
"0D1B8JB"  =  "1"  /* Bypass Ileum to Ileum with Synthetic Substitute, Endo */
"0D1B8JH"  =  "1"  /* Bypass Ileum to Cecum with Synthetic Substitute, Endo */
"0D1B8JK"  =  "1"  /* Bypass Ileum to Ascending Colon with Synth Sub, Endo */
"0D1B8JL"  =  "1"  /* Bypass Ileum to Transverse Colon with Synth Sub, Endo */
"0D1B8JM"  =  "1"  /* Bypass Ileum to Descending Colon with Synth Sub, Endo */
"0D1B8JN"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Synth Sub, Endo */
"0D1B8JP"  =  "1"  /* Bypass Ileum to Rectum with Synthetic Substitute, Endo */
"0D1B8JQ"  =  "1"  /* Bypass Ileum to Anus with Synthetic Substitute, Endo */
"0D1B8K4"  =  "1"  /* Bypass Ileum to Cutaneous with Nonaut Sub, Endo */
"0D1B8KB"  =  "1"  /* Bypass Ileum to Ileum with Nonaut Sub, Endo */
"0D1B8KH"  =  "1"  /* Bypass Ileum to Cecum with Nonaut Sub, Endo */
"0D1B8KK"  =  "1"  /* Bypass Ileum to Ascending Colon with Nonaut Sub, Endo */
"0D1B8KL"  =  "1"  /* Bypass Ileum to Transverse Colon with Nonaut Sub, Endo */
"0D1B8KM"  =  "1"  /* Bypass Ileum to Descending Colon with Nonaut Sub, Endo */
"0D1B8KN"  =  "1"  /* Bypass Ileum to Sigmoid Colon with Nonaut Sub, Endo */
"0D1B8KP"  =  "1"  /* Bypass Ileum to Rectum with Nonaut Sub, Endo */
"0D1B8KQ"  =  "1"  /* Bypass Ileum to Anus with Nonaut Sub, Endo */
"0D1B8Z4"  =  "1"  /* Bypass Ileum to Cutaneous, Endo */
"0D1B8ZB"  =  "1"  /* Bypass Ileum to Ileum, Endo */
"0D1B8ZH"  =  "1"  /* Bypass Ileum to Cecum, Endo */
"0D1B8ZK"  =  "1"  /* Bypass Ileum to Ascending Colon, Endo */
"0D1B8ZL"  =  "1"  /* Bypass Ileum to Transverse Colon, Endo */
"0D1B8ZM"  =  "1"  /* Bypass Ileum to Descending Colon, Endo */
"0D1B8ZN"  =  "1"  /* Bypass Ileum to Sigmoid Colon, Endo */
"0D1B8ZP"  =  "1"  /* Bypass Ileum to Rectum, Endo */
"0D1B8ZQ"  =  "1"  /* Bypass Ileum to Anus, Endo */
"0D1H074"  =  "1"  /* Bypass Cecum to Cutaneous with Autol Sub, Open Approach */
"0D1H07H"  =  "1"  /* Bypass Cecum to Cecum with Autol Sub, Open Approach */
"0D1H07K"  =  "1"  /* Bypass Cecum to Asc Colon with Autol Sub, Open Approach */
"0D1H07L"  =  "1"  /* Bypass Cecum to Trans Colon with Autol Sub, Open Approach */
"0D1H07M"  =  "1"  /* Bypass Cecum to Desc Colon with Autol Sub, Open Approach */
"0D1H07N"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Autol Sub, Open Approach */
"0D1H07P"  =  "1"  /* Bypass Cecum to Rectum with Autol Sub, Open Approach */
"0D1H0J4"  =  "1"  /* Bypass Cecum to Cutaneous with Synth Sub, Open Approach */
"0D1H0JH"  =  "1"  /* Bypass Cecum to Cecum with Synth Sub, Open Approach */
"0D1H0JK"  =  "1"  /* Bypass Cecum to Asc Colon with Synth Sub, Open Approach */
"0D1H0JL"  =  "1"  /* Bypass Cecum to Trans Colon with Synth Sub, Open Approach */
"0D1H0JM"  =  "1"  /* Bypass Cecum to Desc Colon with Synth Sub, Open Approach */
"0D1H0JN"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Synth Sub, Open Approach */
"0D1H0JP"  =  "1"  /* Bypass Cecum to Rectum with Synth Sub, Open Approach */
"0D1H0K4"  =  "1"  /* Bypass Cecum to Cutaneous with Nonaut Sub, Open Approach */
"0D1H0KH"  =  "1"  /* Bypass Cecum to Cecum with Nonaut Sub, Open Approach */
"0D1H0KK"  =  "1"  /* Bypass Cecum to Asc Colon with Nonaut Sub, Open Approach */
"0D1H0KL"  =  "1"  /* Bypass Cecum to Trans Colon with Nonaut Sub, Open Approach */
"0D1H0KM"  =  "1"  /* Bypass Cecum to Desc Colon with Nonaut Sub, Open Approach */
"0D1H0KN"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Nonaut Sub, Open Approach */
"0D1H0KP"  =  "1"  /* Bypass Cecum to Rectum with Nonaut Sub, Open Approach */
"0D1H0Z4"  =  "1"  /* Bypass Cecum to Cutaneous, Open Approach */
"0D1H0ZH"  =  "1"  /* Bypass Cecum to Cecum, Open Approach */
"0D1H0ZK"  =  "1"  /* Bypass Cecum to Ascending Colon, Open Approach */
"0D1H0ZL"  =  "1"  /* Bypass Cecum to Transverse Colon, Open Approach */
"0D1H0ZM"  =  "1"  /* Bypass Cecum to Descending Colon, Open Approach */
"0D1H0ZN"  =  "1"  /* Bypass Cecum to Sigmoid Colon, Open Approach */
"0D1H0ZP"  =  "1"  /* Bypass Cecum to Rectum, Open Approach */
"0D1H3J4"  =  "1"  /* Bypass Cecum to Cutaneous with Synth Sub, Perc Approach */
"0D1H474"  =  "1"  /* Bypass Cecum to Cutaneous with Autol Sub, Perc Endo Approach */
"0D1H47H"  =  "1"  /* Bypass Cecum to Cecum with Autol Sub, Perc Endo Approach */
"0D1H47K"  =  "1"  /* Bypass Cecum to Asc Colon with Autol Sub, Perc Endo Approach */
"0D1H47L"  =  "1"  /* Bypass Cecum to Trans Colon w Autol Sub, Perc Endo */
"0D1H47M"  =  "1"  /* Bypass Cecum to Desc Colon w Autol Sub, Perc Endo */
"0D1H47N"  =  "1"  /* Bypass Cecum to Sigm Colon w Autol Sub, Perc Endo */
"0D1H47P"  =  "1"  /* Bypass Cecum to Rectum with Autol Sub, Perc Endo Approach */
"0D1H4J4"  =  "1"  /* Bypass Cecum to Cutaneous with Synth Sub, Perc Endo Approach */
"0D1H4JH"  =  "1"  /* Bypass Cecum to Cecum with Synth Sub, Perc Endo Approach */
"0D1H4JK"  =  "1"  /* Bypass Cecum to Asc Colon with Synth Sub, Perc Endo Approach */
"0D1H4JL"  =  "1"  /* Bypass Cecum to Trans Colon w Synth Sub, Perc Endo */
"0D1H4JM"  =  "1"  /* Bypass Cecum to Desc Colon w Synth Sub, Perc Endo */
"0D1H4JN"  =  "1"  /* Bypass Cecum to Sigm Colon w Synth Sub, Perc Endo */
"0D1H4JP"  =  "1"  /* Bypass Cecum to Rectum with Synth Sub, Perc Endo Approach */
"0D1H4K4"  =  "1"  /* Bypass Cecum to Cutan with Nonaut Sub, Perc Endo Approach */
"0D1H4KH"  =  "1"  /* Bypass Cecum to Cecum with Nonaut Sub, Perc Endo Approach */
"0D1H4KK"  =  "1"  /* Bypass Cecum to Asc Colon w Nonaut Sub, Perc Endo */
"0D1H4KL"  =  "1"  /* Bypass Cecum to Trans Colon w Nonaut Sub, Perc Endo */
"0D1H4KM"  =  "1"  /* Bypass Cecum to Desc Colon w Nonaut Sub, Perc Endo */
"0D1H4KN"  =  "1"  /* Bypass Cecum to Sigm Colon w Nonaut Sub, Perc Endo */
"0D1H4KP"  =  "1"  /* Bypass Cecum to Rectum with Nonaut Sub, Perc Endo Approach */
"0D1H4Z4"  =  "1"  /* Bypass Cecum to Cutaneous, Percutaneous Endoscopic Approach */
"0D1H4ZH"  =  "1"  /* Bypass Cecum to Cecum, Percutaneous Endoscopic Approach */
"0D1H4ZK"  =  "1"  /* Bypass Cecum to Ascending Colon, Perc Endo Approach */
"0D1H4ZL"  =  "1"  /* Bypass Cecum to Transverse Colon, Perc Endo Approach */
"0D1H4ZM"  =  "1"  /* Bypass Cecum to Descending Colon, Perc Endo Approach */
"0D1H4ZN"  =  "1"  /* Bypass Cecum to Sigmoid Colon, Perc Endo Approach */
"0D1H4ZP"  =  "1"  /* Bypass Cecum to Rectum, Percutaneous Endoscopic Approach */
"0D1H874"  =  "1"  /* Bypass Cecum to Cutaneous with Autol Sub, Endo */
"0D1H87H"  =  "1"  /* Bypass Cecum to Cecum with Autol Sub, Endo */
"0D1H87K"  =  "1"  /* Bypass Cecum to Ascending Colon with Autol Sub, Endo */
"0D1H87L"  =  "1"  /* Bypass Cecum to Transverse Colon with Autol Sub, Endo */
"0D1H87M"  =  "1"  /* Bypass Cecum to Descending Colon with Autol Sub, Endo */
"0D1H87N"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Autol Sub, Endo */
"0D1H87P"  =  "1"  /* Bypass Cecum to Rectum with Autol Sub, Endo */
"0D1H8J4"  =  "1"  /* Bypass Cecum to Cutaneous with Synthetic Substitute, Endo */
"0D1H8JH"  =  "1"  /* Bypass Cecum to Cecum with Synthetic Substitute, Endo */
"0D1H8JK"  =  "1"  /* Bypass Cecum to Ascending Colon with Synth Sub, Endo */
"0D1H8JL"  =  "1"  /* Bypass Cecum to Transverse Colon with Synth Sub, Endo */
"0D1H8JM"  =  "1"  /* Bypass Cecum to Descending Colon with Synth Sub, Endo */
"0D1H8JN"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Synth Sub, Endo */
"0D1H8JP"  =  "1"  /* Bypass Cecum to Rectum with Synthetic Substitute, Endo */
"0D1H8K4"  =  "1"  /* Bypass Cecum to Cutaneous with Nonaut Sub, Endo */
"0D1H8KH"  =  "1"  /* Bypass Cecum to Cecum with Nonaut Sub, Endo */
"0D1H8KK"  =  "1"  /* Bypass Cecum to Ascending Colon with Nonaut Sub, Endo */
"0D1H8KL"  =  "1"  /* Bypass Cecum to Transverse Colon with Nonaut Sub, Endo */
"0D1H8KM"  =  "1"  /* Bypass Cecum to Descending Colon with Nonaut Sub, Endo */
"0D1H8KN"  =  "1"  /* Bypass Cecum to Sigmoid Colon with Nonaut Sub, Endo */
"0D1H8KP"  =  "1"  /* Bypass Cecum to Rectum with Nonaut Sub, Endo */
"0D1H8Z4"  =  "1"  /* Bypass Cecum to Cutaneous, Endo */
"0D1H8ZH"  =  "1"  /* Bypass Cecum to Cecum, Endo */
"0D1H8ZK"  =  "1"  /* Bypass Cecum to Ascending Colon, Endo */
"0D1H8ZL"  =  "1"  /* Bypass Cecum to Transverse Colon, Endo */
"0D1H8ZM"  =  "1"  /* Bypass Cecum to Descending Colon, Endo */
"0D1H8ZN"  =  "1"  /* Bypass Cecum to Sigmoid Colon, Endo */
"0D1H8ZP"  =  "1"  /* Bypass Cecum to Rectum, Endo */
"0D1K074"  =  "1"  /* Bypass Asc Colon to Cutan with Autol Sub, Open Approach */
"0D1K07K"  =  "1"  /* Bypass Asc Colon to Asc Colon with Autol Sub, Open Approach */
"0D1K07L"  =  "1"  /* Bypass Asc Colon to Trans Colon w Autol Sub, Open */
"0D1K07M"  =  "1"  /* Bypass Asc Colon to Desc Colon with Autol Sub, Open Approach */
"0D1K07N"  =  "1"  /* Bypass Asc Colon to Sigm Colon with Autol Sub, Open Approach */
"0D1K07P"  =  "1"  /* Bypass Asc Colon to Rectum with Autol Sub, Open Approach */
"0D1K0J4"  =  "1"  /* Bypass Asc Colon to Cutan with Synth Sub, Open Approach */
"0D1K0JK"  =  "1"  /* Bypass Asc Colon to Asc Colon with Synth Sub, Open Approach */
"0D1K0JL"  =  "1"  /* Bypass Asc Colon to Trans Colon w Synth Sub, Open */
"0D1K0JM"  =  "1"  /* Bypass Asc Colon to Desc Colon with Synth Sub, Open Approach */
"0D1K0JN"  =  "1"  /* Bypass Asc Colon to Sigm Colon with Synth Sub, Open Approach */
"0D1K0JP"  =  "1"  /* Bypass Asc Colon to Rectum with Synth Sub, Open Approach */
"0D1K0K4"  =  "1"  /* Bypass Asc Colon to Cutan with Nonaut Sub, Open Approach */
"0D1K0KK"  =  "1"  /* Bypass Asc Colon to Asc Colon with Nonaut Sub, Open Approach */
"0D1K0KL"  =  "1"  /* Bypass Asc Colon to Trans Colon w Nonaut Sub, Open */
"0D1K0KM"  =  "1"  /* Bypass Asc Colon to Desc Colon w Nonaut Sub, Open */
"0D1K0KN"  =  "1"  /* Bypass Asc Colon to Sigm Colon w Nonaut Sub, Open */
"0D1K0KP"  =  "1"  /* Bypass Asc Colon to Rectum with Nonaut Sub, Open Approach */
"0D1K0Z4"  =  "1"  /* Bypass Ascending Colon to Cutaneous, Open Approach */
"0D1K0ZK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon, Open Approach */
"0D1K0ZL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon, Open Approach */
"0D1K0ZM"  =  "1"  /* Bypass Ascending Colon to Descending Colon, Open Approach */
"0D1K0ZN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon, Open Approach */
"0D1K0ZP"  =  "1"  /* Bypass Ascending Colon to Rectum, Open Approach */
"0D1K3J4"  =  "1"  /* Bypass Asc Colon to Cutan with Synth Sub, Perc Approach */
"0D1K474"  =  "1"  /* Bypass Asc Colon to Cutan with Autol Sub, Perc Endo Approach */
"0D1K47K"  =  "1"  /* Bypass Asc Colon to Asc Colon w Autol Sub, Perc Endo */
"0D1K47L"  =  "1"  /* Bypass Asc Colon to Trans Colon w Autol Sub, Perc Endo */
"0D1K47M"  =  "1"  /* Bypass Asc Colon to Desc Colon w Autol Sub, Perc Endo */
"0D1K47N"  =  "1"  /* Bypass Asc Colon to Sigm Colon w Autol Sub, Perc Endo */
"0D1K47P"  =  "1"  /* Bypass Asc Colon to Rectum w Autol Sub, Perc Endo */
"0D1K4J4"  =  "1"  /* Bypass Asc Colon to Cutan with Synth Sub, Perc Endo Approach */
"0D1K4JK"  =  "1"  /* Bypass Asc Colon to Asc Colon w Synth Sub, Perc Endo */
"0D1K4JL"  =  "1"  /* Bypass Asc Colon to Trans Colon w Synth Sub, Perc Endo */
"0D1K4JM"  =  "1"  /* Bypass Asc Colon to Desc Colon w Synth Sub, Perc Endo */
"0D1K4JN"  =  "1"  /* Bypass Asc Colon to Sigm Colon w Synth Sub, Perc Endo */
"0D1K4JP"  =  "1"  /* Bypass Asc Colon to Rectum w Synth Sub, Perc Endo */
"0D1K4K4"  =  "1"  /* Bypass Asc Colon to Cutan w Nonaut Sub, Perc Endo */
"0D1K4KK"  =  "1"  /* Bypass Asc Colon to Asc Colon w Nonaut Sub, Perc Endo */
"0D1K4KL"  =  "1"  /* Bypass Asc Colon to Trans Colon w Nonaut Sub, Perc Endo */
"0D1K4KM"  =  "1"  /* Bypass Asc Colon to Desc Colon w Nonaut Sub, Perc Endo */
"0D1K4KN"  =  "1"  /* Bypass Asc Colon to Sigm Colon w Nonaut Sub, Perc Endo */
"0D1K4KP"  =  "1"  /* Bypass Asc Colon to Rectum w Nonaut Sub, Perc Endo */
"0D1K4Z4"  =  "1"  /* Bypass Ascending Colon to Cutaneous, Perc Endo Approach */
"0D1K4ZK"  =  "1"  /* Bypass Ascending Colon to Asc Colon, Perc Endo Approach */
"0D1K4ZL"  =  "1"  /* Bypass Ascending Colon to Trans Colon, Perc Endo Approach */
"0D1K4ZM"  =  "1"  /* Bypass Ascending Colon to Desc Colon, Perc Endo Approach */
"0D1K4ZN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon, Perc Endo Approach */
"0D1K4ZP"  =  "1"  /* Bypass Ascending Colon to Rectum, Perc Endo Approach */
"0D1K874"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Autol Sub, Endo */
"0D1K87K"  =  "1"  /* Bypass Ascending Colon to Asc Colon with Autol Sub, Endo */
"0D1K87L"  =  "1"  /* Bypass Ascending Colon to Trans Colon with Autol Sub, Endo */
"0D1K87M"  =  "1"  /* Bypass Ascending Colon to Desc Colon with Autol Sub, Endo */
"0D1K87N"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Autol Sub, Endo */
"0D1K87P"  =  "1"  /* Bypass Ascending Colon to Rectum with Autol Sub, Endo */
"0D1K8J4"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Synth Sub, Endo */
"0D1K8JK"  =  "1"  /* Bypass Ascending Colon to Asc Colon with Synth Sub, Endo */
"0D1K8JL"  =  "1"  /* Bypass Ascending Colon to Trans Colon with Synth Sub, Endo */
"0D1K8JM"  =  "1"  /* Bypass Ascending Colon to Desc Colon with Synth Sub, Endo */
"0D1K8JN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon with Synth Sub, Endo */
"0D1K8JP"  =  "1"  /* Bypass Ascending Colon to Rectum with Synth Sub, Endo */
"0D1K8K4"  =  "1"  /* Bypass Ascending Colon to Cutaneous with Nonaut Sub, Endo */
"0D1K8KK"  =  "1"  /* Bypass Ascending Colon to Asc Colon with Nonaut Sub, Endo */
"0D1K8KL"  =  "1"  /* Bypass Ascending Colon to Trans Colon with Nonaut Sub, Endo */
"0D1K8KM"  =  "1"  /* Bypass Ascending Colon to Desc Colon with Nonaut Sub, Endo */
"0D1K8KN"  =  "1"  /* Bypass Ascending Colon to Sigm Colon with Nonaut Sub, Endo */
"0D1K8KP"  =  "1"  /* Bypass Ascending Colon to Rectum with Nonaut Sub, Endo */
"0D1K8Z4"  =  "1"  /* Bypass Ascending Colon to Cutaneous, Endo */
"0D1K8ZK"  =  "1"  /* Bypass Ascending Colon to Ascending Colon, Endo */
"0D1K8ZL"  =  "1"  /* Bypass Ascending Colon to Transverse Colon, Endo */
"0D1K8ZM"  =  "1"  /* Bypass Ascending Colon to Descending Colon, Endo */
"0D1K8ZN"  =  "1"  /* Bypass Ascending Colon to Sigmoid Colon, Endo */
"0D1K8ZP"  =  "1"  /* Bypass Ascending Colon to Rectum, Endo */
"0D1L074"  =  "1"  /* Bypass Trans Colon to Cutan with Autol Sub, Open Approach */
"0D1L07L"  =  "1"  /* Bypass Trans Colon to Trans Colon w Autol Sub, Open */
"0D1L07M"  =  "1"  /* Bypass Trans Colon to Desc Colon w Autol Sub, Open */
"0D1L07N"  =  "1"  /* Bypass Trans Colon to Sigm Colon w Autol Sub, Open */
"0D1L07P"  =  "1"  /* Bypass Trans Colon to Rectum with Autol Sub, Open Approach */
"0D1L0J4"  =  "1"  /* Bypass Trans Colon to Cutan with Synth Sub, Open Approach */
"0D1L0JL"  =  "1"  /* Bypass Trans Colon to Trans Colon w Synth Sub, Open */
"0D1L0JM"  =  "1"  /* Bypass Trans Colon to Desc Colon w Synth Sub, Open */
"0D1L0JN"  =  "1"  /* Bypass Trans Colon to Sigm Colon w Synth Sub, Open */
"0D1L0JP"  =  "1"  /* Bypass Trans Colon to Rectum with Synth Sub, Open Approach */
"0D1L0K4"  =  "1"  /* Bypass Trans Colon to Cutan with Nonaut Sub, Open Approach */
"0D1L0KL"  =  "1"  /* Bypass Trans Colon to Trans Colon w Nonaut Sub, Open */
"0D1L0KM"  =  "1"  /* Bypass Trans Colon to Desc Colon w Nonaut Sub, Open */
"0D1L0KN"  =  "1"  /* Bypass Trans Colon to Sigm Colon w Nonaut Sub, Open */
"0D1L0KP"  =  "1"  /* Bypass Trans Colon to Rectum with Nonaut Sub, Open Approach */
"0D1L0Z4"  =  "1"  /* Bypass Transverse Colon to Cutaneous, Open Approach */
"0D1L0ZL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon, Open Approach */
"0D1L0ZM"  =  "1"  /* Bypass Transverse Colon to Descending Colon, Open Approach */
"0D1L0ZN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon, Open Approach */
"0D1L0ZP"  =  "1"  /* Bypass Transverse Colon to Rectum, Open Approach */
"0D1L3J4"  =  "1"  /* Bypass Trans Colon to Cutan with Synth Sub, Perc Approach */
"0D1L474"  =  "1"  /* Bypass Trans Colon to Cutan w Autol Sub, Perc Endo */
"0D1L47L"  =  "1"  /* Bypass Trans Colon to Trans Colon w Autol Sub, Perc Endo */
"0D1L47M"  =  "1"  /* Bypass Trans Colon to Desc Colon w Autol Sub, Perc Endo */
"0D1L47N"  =  "1"  /* Bypass Trans Colon to Sigm Colon w Autol Sub, Perc Endo */
"0D1L47P"  =  "1"  /* Bypass Trans Colon to Rectum w Autol Sub, Perc Endo */
"0D1L4J4"  =  "1"  /* Bypass Trans Colon to Cutan w Synth Sub, Perc Endo */
"0D1L4JL"  =  "1"  /* Bypass Trans Colon to Trans Colon w Synth Sub, Perc Endo */
"0D1L4JM"  =  "1"  /* Bypass Trans Colon to Desc Colon w Synth Sub, Perc Endo */
"0D1L4JN"  =  "1"  /* Bypass Trans Colon to Sigm Colon w Synth Sub, Perc Endo */
"0D1L4JP"  =  "1"  /* Bypass Trans Colon to Rectum w Synth Sub, Perc Endo */
"0D1L4K4"  =  "1"  /* Bypass Trans Colon to Cutan w Nonaut Sub, Perc Endo */
"0D1L4KL"  =  "1"  /* Bypass Trans Colon to Trans Colon w Nonaut Sub, Perc Endo */
"0D1L4KM"  =  "1"  /* Bypass Trans Colon to Desc Colon w Nonaut Sub, Perc Endo */
"0D1L4KN"  =  "1"  /* Bypass Trans Colon to Sigm Colon w Nonaut Sub, Perc Endo */
"0D1L4KP"  =  "1"  /* Bypass Trans Colon to Rectum w Nonaut Sub, Perc Endo */
"0D1L4Z4"  =  "1"  /* Bypass Transverse Colon to Cutaneous, Perc Endo Approach */
"0D1L4ZL"  =  "1"  /* Bypass Transverse Colon to Trans Colon, Perc Endo Approach */
"0D1L4ZM"  =  "1"  /* Bypass Transverse Colon to Desc Colon, Perc Endo Approach */
"0D1L4ZN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon, Perc Endo Approach */
"0D1L4ZP"  =  "1"  /* Bypass Transverse Colon to Rectum, Perc Endo Approach */
"0D1L874"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Autol Sub, Endo */
"0D1L87L"  =  "1"  /* Bypass Transverse Colon to Trans Colon with Autol Sub, Endo */
"0D1L87M"  =  "1"  /* Bypass Transverse Colon to Desc Colon with Autol Sub, Endo */
"0D1L87N"  =  "1"  /* Bypass Transverse Colon to Sigm Colon with Autol Sub, Endo */
"0D1L87P"  =  "1"  /* Bypass Transverse Colon to Rectum with Autol Sub, Endo */
"0D1L8J4"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Synth Sub, Endo */
"0D1L8JL"  =  "1"  /* Bypass Transverse Colon to Trans Colon with Synth Sub, Endo */
"0D1L8JM"  =  "1"  /* Bypass Transverse Colon to Desc Colon with Synth Sub, Endo */
"0D1L8JN"  =  "1"  /* Bypass Transverse Colon to Sigm Colon with Synth Sub, Endo */
"0D1L8JP"  =  "1"  /* Bypass Transverse Colon to Rectum with Synth Sub, Endo */
"0D1L8K4"  =  "1"  /* Bypass Transverse Colon to Cutaneous with Nonaut Sub, Endo */
"0D1L8KL"  =  "1"  /* Bypass Transverse Colon to Trans Colon with Nonaut Sub, Endo */
"0D1L8KM"  =  "1"  /* Bypass Transverse Colon to Desc Colon with Nonaut Sub, Endo */
"0D1L8KN"  =  "1"  /* Bypass Transverse Colon to Sigm Colon with Nonaut Sub, Endo */
"0D1L8KP"  =  "1"  /* Bypass Transverse Colon to Rectum with Nonaut Sub, Endo */
"0D1L8Z4"  =  "1"  /* Bypass Transverse Colon to Cutaneous, Endo */
"0D1L8ZL"  =  "1"  /* Bypass Transverse Colon to Transverse Colon, Endo */
"0D1L8ZM"  =  "1"  /* Bypass Transverse Colon to Descending Colon, Endo */
"0D1L8ZN"  =  "1"  /* Bypass Transverse Colon to Sigmoid Colon, Endo */
"0D1L8ZP"  =  "1"  /* Bypass Transverse Colon to Rectum, Endo */
"0D1M074"  =  "1"  /* Bypass Descend Colon to Cutan with Autol Sub, Open Approach */
"0D1M07M"  =  "1"  /* Bypass Descend Colon to Desc Colon w Autol Sub, Open */
"0D1M07N"  =  "1"  /* Bypass Descend Colon to Sigm Colon w Autol Sub, Open */
"0D1M07P"  =  "1"  /* Bypass Descend Colon to Rectum with Autol Sub, Open Approach */
"0D1M0J4"  =  "1"  /* Bypass Descend Colon to Cutan with Synth Sub, Open Approach */
"0D1M0JM"  =  "1"  /* Bypass Descend Colon to Desc Colon w Synth Sub, Open */
"0D1M0JN"  =  "1"  /* Bypass Descend Colon to Sigm Colon w Synth Sub, Open */
"0D1M0JP"  =  "1"  /* Bypass Descend Colon to Rectum with Synth Sub, Open Approach */
"0D1M0K4"  =  "1"  /* Bypass Descend Colon to Cutan with Nonaut Sub, Open Approach */
"0D1M0KM"  =  "1"  /* Bypass Descend Colon to Desc Colon w Nonaut Sub, Open */
"0D1M0KN"  =  "1"  /* Bypass Descend Colon to Sigm Colon w Nonaut Sub, Open */
"0D1M0KP"  =  "1"  /* Bypass Descend Colon to Rectum w Nonaut Sub, Open */
"0D1M0Z4"  =  "1"  /* Bypass Descending Colon to Cutaneous, Open Approach */
"0D1M0ZM"  =  "1"  /* Bypass Descending Colon to Descending Colon, Open Approach */
"0D1M0ZN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon, Open Approach */
"0D1M0ZP"  =  "1"  /* Bypass Descending Colon to Rectum, Open Approach */
"0D1M3J4"  =  "1"  /* Bypass Descend Colon to Cutan with Synth Sub, Perc Approach */
"0D1M474"  =  "1"  /* Bypass Descend Colon to Cutan w Autol Sub, Perc Endo */
"0D1M47M"  =  "1"  /* Bypass Descend Colon to Desc Colon w Autol Sub, Perc Endo */
"0D1M47N"  =  "1"  /* Bypass Descend Colon to Sigm Colon w Autol Sub, Perc Endo */
"0D1M47P"  =  "1"  /* Bypass Descend Colon to Rectum w Autol Sub, Perc Endo */
"0D1M4J4"  =  "1"  /* Bypass Descend Colon to Cutan w Synth Sub, Perc Endo */
"0D1M4JM"  =  "1"  /* Bypass Descend Colon to Desc Colon w Synth Sub, Perc Endo */
"0D1M4JN"  =  "1"  /* Bypass Descend Colon to Sigm Colon w Synth Sub, Perc Endo */
"0D1M4JP"  =  "1"  /* Bypass Descend Colon to Rectum w Synth Sub, Perc Endo */
"0D1M4K4"  =  "1"  /* Bypass Descend Colon to Cutan w Nonaut Sub, Perc Endo */
"0D1M4KM"  =  "1"  /* Bypass Descend Colon to Desc Colon w Nonaut Sub, Perc Endo */
"0D1M4KN"  =  "1"  /* Bypass Descend Colon to Sigm Colon w Nonaut Sub, Perc Endo */
"0D1M4KP"  =  "1"  /* Bypass Descend Colon to Rectum w Nonaut Sub, Perc Endo */
"0D1M4Z4"  =  "1"  /* Bypass Descending Colon to Cutaneous, Perc Endo Approach */
"0D1M4ZM"  =  "1"  /* Bypass Descending Colon to Desc Colon, Perc Endo Approach */
"0D1M4ZN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon, Perc Endo Approach */
"0D1M4ZP"  =  "1"  /* Bypass Descending Colon to Rectum, Perc Endo Approach */
"0D1M874"  =  "1"  /* Bypass Descending Colon to Cutaneous with Autol Sub, Endo */
"0D1M87M"  =  "1"  /* Bypass Descending Colon to Desc Colon with Autol Sub, Endo */
"0D1M87N"  =  "1"  /* Bypass Descending Colon to Sigm Colon with Autol Sub, Endo */
"0D1M87P"  =  "1"  /* Bypass Descending Colon to Rectum with Autol Sub, Endo */
"0D1M8J4"  =  "1"  /* Bypass Descending Colon to Cutaneous with Synth Sub, Endo */
"0D1M8JM"  =  "1"  /* Bypass Descending Colon to Desc Colon with Synth Sub, Endo */
"0D1M8JN"  =  "1"  /* Bypass Descending Colon to Sigm Colon with Synth Sub, Endo */
"0D1M8JP"  =  "1"  /* Bypass Descending Colon to Rectum with Synth Sub, Endo */
"0D1M8K4"  =  "1"  /* Bypass Descending Colon to Cutaneous with Nonaut Sub, Endo */
"0D1M8KM"  =  "1"  /* Bypass Descending Colon to Desc Colon with Nonaut Sub, Endo */
"0D1M8KN"  =  "1"  /* Bypass Descending Colon to Sigm Colon with Nonaut Sub, Endo */
"0D1M8KP"  =  "1"  /* Bypass Descending Colon to Rectum with Nonaut Sub, Endo */
"0D1M8Z4"  =  "1"  /* Bypass Descending Colon to Cutaneous, Endo */
"0D1M8ZM"  =  "1"  /* Bypass Descending Colon to Descending Colon, Endo */
"0D1M8ZN"  =  "1"  /* Bypass Descending Colon to Sigmoid Colon, Endo */
"0D1M8ZP"  =  "1"  /* Bypass Descending Colon to Rectum, Endo */
"0D1N074"  =  "1"  /* Bypass Sigmoid Colon to Cutan with Autol Sub, Open Approach */
"0D1N07N"  =  "1"  /* Bypass Sigmoid Colon to Sigm Colon w Autol Sub, Open */
"0D1N07P"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Autol Sub, Open Approach */
"0D1N0J4"  =  "1"  /* Bypass Sigmoid Colon to Cutan with Synth Sub, Open Approach */
"0D1N0JN"  =  "1"  /* Bypass Sigmoid Colon to Sigm Colon w Synth Sub, Open */
"0D1N0JP"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Synth Sub, Open Approach */
"0D1N0K4"  =  "1"  /* Bypass Sigmoid Colon to Cutan with Nonaut Sub, Open Approach */
"0D1N0KN"  =  "1"  /* Bypass Sigmoid Colon to Sigm Colon w Nonaut Sub, Open */
"0D1N0KP"  =  "1"  /* Bypass Sigmoid Colon to Rectum w Nonaut Sub, Open */
"0D1N0Z4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous, Open Approach */
"0D1N0ZN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon, Open Approach */
"0D1N0ZP"  =  "1"  /* Bypass Sigmoid Colon to Rectum, Open Approach */
"0D1N3J4"  =  "1"  /* Bypass Sigmoid Colon to Cutan with Synth Sub, Perc Approach */
"0D1N474"  =  "1"  /* Bypass Sigmoid Colon to Cutan w Autol Sub, Perc Endo */
"0D1N47N"  =  "1"  /* Bypass Sigmoid Colon to Sigm Colon w Autol Sub, Perc Endo */
"0D1N47P"  =  "1"  /* Bypass Sigmoid Colon to Rectum w Autol Sub, Perc Endo */
"0D1N4J4"  =  "1"  /* Bypass Sigmoid Colon to Cutan w Synth Sub, Perc Endo */
"0D1N4JN"  =  "1"  /* Bypass Sigmoid Colon to Sigm Colon w Synth Sub, Perc Endo */
"0D1N4JP"  =  "1"  /* Bypass Sigmoid Colon to Rectum w Synth Sub, Perc Endo */
"0D1N4K4"  =  "1"  /* Bypass Sigmoid Colon to Cutan w Nonaut Sub, Perc Endo */
"0D1N4KN"  =  "1"  /* Bypass Sigmoid Colon to Sigm Colon w Nonaut Sub, Perc Endo */
"0D1N4KP"  =  "1"  /* Bypass Sigmoid Colon to Rectum w Nonaut Sub, Perc Endo */
"0D1N4Z4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous, Perc Endo Approach */
"0D1N4ZN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon, Perc Endo Approach */
"0D1N4ZP"  =  "1"  /* Bypass Sigmoid Colon to Rectum, Perc Endo Approach */
"0D1N874"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Autol Sub, Endo */
"0D1N87N"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Autol Sub, Endo */
"0D1N87P"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Autol Sub, Endo */
"0D1N8J4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Synth Sub, Endo */
"0D1N8JN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Synth Sub, Endo */
"0D1N8JP"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Synth Sub, Endo */
"0D1N8K4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous with Nonaut Sub, Endo */
"0D1N8KN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon with Nonaut Sub, Endo */
"0D1N8KP"  =  "1"  /* Bypass Sigmoid Colon to Rectum with Nonaut Sub, Endo */
"0D1N8Z4"  =  "1"  /* Bypass Sigmoid Colon to Cutaneous, Endo */
"0D1N8ZN"  =  "1"  /* Bypass Sigmoid Colon to Sigmoid Colon, Endo */
"0D1N8ZP"  =  "1"  /* Bypass Sigmoid Colon to Rectum, Endo */
"0D510ZZ"  =  "1"  /* Destruction of Upper Esophagus, Open Approach */
"0D513ZZ"  =  "1"  /* Destruction of Upper Esophagus, Percutaneous Approach */
"0D517ZZ"  =  "1"  /* Destruction of Upper Esophagus, Via Opening */
"0D520ZZ"  =  "1"  /* Destruction of Middle Esophagus, Open Approach */
"0D523ZZ"  =  "1"  /* Destruction of Middle Esophagus, Percutaneous Approach */
"0D527ZZ"  =  "1"  /* Destruction of Middle Esophagus, Via Opening */
"0D530ZZ"  =  "1"  /* Destruction of Lower Esophagus, Open Approach */
"0D533ZZ"  =  "1"  /* Destruction of Lower Esophagus, Percutaneous Approach */
"0D537ZZ"  =  "1"  /* Destruction of Lower Esophagus, Via Opening */
"0D540ZZ"  =  "1"  /* Destruction of Esophagogastric Junction, Open Approach */
"0D543ZZ"  =  "1"  /* Destruction of Esophagogastric Junction, Perc Approach */
"0D547ZZ"  =  "1"  /* Destruction of Esophagogastric Junction, Via Opening */
"0D550ZZ"  =  "1"  /* Destruction of Esophagus, Open Approach */
"0D553ZZ"  =  "1"  /* Destruction of Esophagus, Percutaneous Approach */
"0D557ZZ"  =  "1"  /* Destruction of Esophagus, Via Natural or Artificial Opening */
"0D560ZZ"  =  "1"  /* Destruction of Stomach, Open Approach */
"0D563ZZ"  =  "1"  /* Destruction of Stomach, Percutaneous Approach */
"0D567ZZ"  =  "1"  /* Destruction of Stomach, Via Natural or Artificial Opening */
"0D570ZZ"  =  "1"  /* Destruction of Stomach, Pylorus, Open Approach */
"0D573ZZ"  =  "1"  /* Destruction of Stomach, Pylorus, Percutaneous Approach */
"0D577ZZ"  =  "1"  /* Destruction of Stomach, Pylorus, Via Opening */
"0D580ZZ"  =  "1"  /* Destruction of Small Intestine, Open Approach */
"0D583ZZ"  =  "1"  /* Destruction of Small Intestine, Percutaneous Approach */
"0D584ZZ"  =  "1"  /* Destruction of Small Intestine, Perc Endo Approach */
"0D587ZZ"  =  "1"  /* Destruction of Small Intestine, Via Opening */
"0D588ZZ"  =  "1"  /* Destruction of Small Intestine, Endo */
"0D590ZZ"  =  "1"  /* Destruction of Duodenum, Open Approach */
"0D593ZZ"  =  "1"  /* Destruction of Duodenum, Percutaneous Approach */
"0D597ZZ"  =  "1"  /* Destruction of Duodenum, Via Natural or Artificial Opening */
"0D5A0ZZ"  =  "1"  /* Destruction of Jejunum, Open Approach */
"0D5A3ZZ"  =  "1"  /* Destruction of Jejunum, Percutaneous Approach */
"0D5A4ZZ"  =  "1"  /* Destruction of Jejunum, Percutaneous Endoscopic Approach */
"0D5A7ZZ"  =  "1"  /* Destruction of Jejunum, Via Natural or Artificial Opening */
"0D5A8ZZ"  =  "1"  /* Destruction of Jejunum, Endo */
"0D5B0ZZ"  =  "1"  /* Destruction of Ileum, Open Approach */
"0D5B3ZZ"  =  "1"  /* Destruction of Ileum, Percutaneous Approach */
"0D5B4ZZ"  =  "1"  /* Destruction of Ileum, Percutaneous Endoscopic Approach */
"0D5B7ZZ"  =  "1"  /* Destruction of Ileum, Via Natural or Artificial Opening */
"0D5B8ZZ"  =  "1"  /* Destruction of Ileum, Endo */
"0D5C0ZZ"  =  "1"  /* Destruction of Ileocecal Valve, Open Approach */
"0D5C3ZZ"  =  "1"  /* Destruction of Ileocecal Valve, Percutaneous Approach */
"0D5C4ZZ"  =  "1"  /* Destruction of Ileocecal Valve, Perc Endo Approach */
"0D5C7ZZ"  =  "1"  /* Destruction of Ileocecal Valve, Via Opening */
"0D5C8ZZ"  =  "1"  /* Destruction of Ileocecal Valve, Endo */
"0D5E0ZZ"  =  "1"  /* Destruction of Large Intestine, Open Approach */
"0D5E3ZZ"  =  "1"  /* Destruction of Large Intestine, Percutaneous Approach */
"0D5E7ZZ"  =  "1"  /* Destruction of Large Intestine, Via Opening */
"0D5F0ZZ"  =  "1"  /* Destruction of Right Large Intestine, Open Approach */
"0D5F3ZZ"  =  "1"  /* Destruction of Right Large Intestine, Percutaneous Approach */
"0D5F7ZZ"  =  "1"  /* Destruction of Right Large Intestine, Via Opening */
"0D5G0ZZ"  =  "1"  /* Destruction of Left Large Intestine, Open Approach */
"0D5G3ZZ"  =  "1"  /* Destruction of Left Large Intestine, Percutaneous Approach */
"0D5G7ZZ"  =  "1"  /* Destruction of Left Large Intestine, Via Opening */
"0D5H0ZZ"  =  "1"  /* Destruction of Cecum, Open Approach */
"0D5H3ZZ"  =  "1"  /* Destruction of Cecum, Percutaneous Approach */
"0D5H7ZZ"  =  "1"  /* Destruction of Cecum, Via Natural or Artificial Opening */
"0D5J0ZZ"  =  "1"  /* Destruction of Appendix, Open Approach */
"0D5J3ZZ"  =  "1"  /* Destruction of Appendix, Percutaneous Approach */
"0D5J4ZZ"  =  "1"  /* Destruction of Appendix, Percutaneous Endoscopic Approach */
"0D5J7ZZ"  =  "1"  /* Destruction of Appendix, Via Natural or Artificial Opening */
"0D5J8ZZ"  =  "1"  /* Destruction of Appendix, Endo */
"0D5K0ZZ"  =  "1"  /* Destruction of Ascending Colon, Open Approach */
"0D5K3ZZ"  =  "1"  /* Destruction of Ascending Colon, Percutaneous Approach */
"0D5K7ZZ"  =  "1"  /* Destruction of Ascending Colon, Via Opening */
"0D5L0ZZ"  =  "1"  /* Destruction of Transverse Colon, Open Approach */
"0D5L3ZZ"  =  "1"  /* Destruction of Transverse Colon, Percutaneous Approach */
"0D5L7ZZ"  =  "1"  /* Destruction of Transverse Colon, Via Opening */
"0D5M0ZZ"  =  "1"  /* Destruction of Descending Colon, Open Approach */
"0D5M3ZZ"  =  "1"  /* Destruction of Descending Colon, Percutaneous Approach */
"0D5M7ZZ"  =  "1"  /* Destruction of Descending Colon, Via Opening */
"0D5N0ZZ"  =  "1"  /* Destruction of Sigmoid Colon, Open Approach */
"0D5N3ZZ"  =  "1"  /* Destruction of Sigmoid Colon, Percutaneous Approach */
"0D5N7ZZ"  =  "1"  /* Destruction of Sigmoid Colon, Via Opening */
"0D5Q0ZZ"  =  "1"  /* Destruction of Anus, Open Approach */
"0D5Q3ZZ"  =  "1"  /* Destruction of Anus, Percutaneous Approach */
"0D5Q7ZZ"  =  "1"  /* Destruction of Anus, Via Natural or Artificial Opening */
"0D5QXZZ"  =  "1"  /* Destruction of Anus, External Approach */
"0D5R0ZZ"  =  "1"  /* Destruction of Anal Sphincter, Open Approach */
"0D5R3ZZ"  =  "1"  /* Destruction of Anal Sphincter, Percutaneous Approach */
"0D5S0ZZ"  =  "1"  /* Destruction of Greater Omentum, Open Approach */
"0D5S3ZZ"  =  "1"  /* Destruction of Greater Omentum, Percutaneous Approach */
"0D5S4ZZ"  =  "1"  /* Destruction of Greater Omentum, Perc Endo Approach */
"0D5T0ZZ"  =  "1"  /* Destruction of Lesser Omentum, Open Approach */
"0D5T3ZZ"  =  "1"  /* Destruction of Lesser Omentum, Percutaneous Approach */
"0D5T4ZZ"  =  "1"  /* Destruction of Lesser Omentum, Perc Endo Approach */
"0D5V0ZZ"  =  "1"  /* Destruction of Mesentery, Open Approach */
"0D5V3ZZ"  =  "1"  /* Destruction of Mesentery, Percutaneous Approach */
"0D5V4ZZ"  =  "1"  /* Destruction of Mesentery, Percutaneous Endoscopic Approach */
"0D5W0ZZ"  =  "1"  /* Destruction of Peritoneum, Open Approach */
"0D5W3ZZ"  =  "1"  /* Destruction of Peritoneum, Percutaneous Approach */
"0D5W4ZZ"  =  "1"  /* Destruction of Peritoneum, Percutaneous Endoscopic Approach */
"0D710DZ"  =  "1"  /* Dilation of Upper Esophagus with Intralum Dev, Open Approach */
"0D710ZZ"  =  "1"  /* Dilation of Upper Esophagus, Open Approach */
"0D713DZ"  =  "1"  /* Dilation of Upper Esophagus with Intralum Dev, Perc Approach */
"0D713ZZ"  =  "1"  /* Dilation of Upper Esophagus, Percutaneous Approach */
"0D714DZ"  =  "1"  /* Dilation of Up Esophag with Intralum Dev, Perc Endo Approach */
"0D714ZZ"  =  "1"  /* Dilation of Upper Esophagus, Perc Endo Approach */
"0D720DZ"  =  "1"  /* Dilation of Mid Esophag with Intralum Dev, Open Approach */
"0D720ZZ"  =  "1"  /* Dilation of Middle Esophagus, Open Approach */
"0D723DZ"  =  "1"  /* Dilation of Mid Esophag with Intralum Dev, Perc Approach */
"0D723ZZ"  =  "1"  /* Dilation of Middle Esophagus, Percutaneous Approach */
"0D724DZ"  =  "1"  /* Dilate of Mid Esophag with Intralum Dev, Perc Endo Approach */
"0D724ZZ"  =  "1"  /* Dilation of Middle Esophagus, Perc Endo Approach */
"0D730DZ"  =  "1"  /* Dilation of Lower Esophagus with Intralum Dev, Open Approach */
"0D730ZZ"  =  "1"  /* Dilation of Lower Esophagus, Open Approach */
"0D733DZ"  =  "1"  /* Dilation of Lower Esophagus with Intralum Dev, Perc Approach */
"0D733ZZ"  =  "1"  /* Dilation of Lower Esophagus, Percutaneous Approach */
"0D734DZ"  =  "1"  /* Dilate of Low Esophag with Intralum Dev, Perc Endo Approach */
"0D734ZZ"  =  "1"  /* Dilation of Lower Esophagus, Perc Endo Approach */
"0D740DZ"  =  "1"  /* Dilate of Esophagast Junct with Intralum Dev, Open Approach */
"0D740ZZ"  =  "1"  /* Dilation of Esophagogastric Junction, Open Approach */
"0D743DZ"  =  "1"  /* Dilate of Esophagast Junct with Intralum Dev, Perc Approach */
"0D743ZZ"  =  "1"  /* Dilation of Esophagogastric Junction, Percutaneous Approach */
"0D744DZ"  =  "1"  /* Dilate Esophagast Junct w Intralum Dev, Perc Endo */
"0D744ZZ"  =  "1"  /* Dilation of Esophagogastric Junction, Perc Endo Approach */
"0D750DZ"  =  "1"  /* Dilation of Esophagus with Intralum Dev, Open Approach */
"0D750ZZ"  =  "1"  /* Dilation of Esophagus, Open Approach */
"0D753DZ"  =  "1"  /* Dilation of Esophagus with Intralum Dev, Perc Approach */
"0D753ZZ"  =  "1"  /* Dilation of Esophagus, Percutaneous Approach */
"0D754DZ"  =  "1"  /* Dilation of Esophagus with Intralum Dev, Perc Endo Approach */
"0D754ZZ"  =  "1"  /* Dilation of Esophagus, Percutaneous Endoscopic Approach */
"0D760DZ"  =  "1"  /* Dilation of Stomach with Intraluminal Device, Open Approach */
"0D760ZZ"  =  "1"  /* Dilation of Stomach, Open Approach */
"0D763DZ"  =  "1"  /* Dilation of Stomach with Intraluminal Device, Perc Approach */
"0D763ZZ"  =  "1"  /* Dilation of Stomach, Percutaneous Approach */
"0D764DZ"  =  "1"  /* Dilation of Stomach with Intralum Dev, Perc Endo Approach */
"0D764ZZ"  =  "1"  /* Dilation of Stomach, Percutaneous Endoscopic Approach */
"0D767DZ"  =  "1"  /* Dilation of Stomach with Intraluminal Device, Via Opening */
"0D767ZZ"  =  "1"  /* Dilation of Stomach, Via Natural or Artificial Opening */
"0D768DZ"  =  "1"  /* Dilation of Stomach with Intraluminal Device, Endo */
"0D768ZZ"  =  "1"  /* Dilation of Stomach, Endo */
"0D770DZ"  =  "1"  /* Dilation of Stomach, Pylrs with Intralum Dev, Open Approach */
"0D770ZZ"  =  "1"  /* Dilation of Stomach, Pylorus, Open Approach */
"0D773DZ"  =  "1"  /* Dilation of Stomach, Pylrs with Intralum Dev, Perc Approach */
"0D773ZZ"  =  "1"  /* Dilation of Stomach, Pylorus, Percutaneous Approach */
"0D774ZZ"  =  "1"  /* Dilation of Stomach, Pylorus, Perc Endo Approach */
"0D777DZ"  =  "1"  /* Dilation of Stomach, Pylorus with Intralum Dev, Via Opening */
"0D777ZZ"  =  "1"  /* Dilation of Stomach, Pylorus, Via Opening */
"0D778ZZ"  =  "1"  /* Dilation of Stomach, Pylorus, Endo */
"0D780ZZ"  =  "1"  /* Dilation of Small Intestine, Open Approach */
"0D783ZZ"  =  "1"  /* Dilation of Small Intestine, Percutaneous Approach */
"0D784ZZ"  =  "1"  /* Dilation of Small Intestine, Perc Endo Approach */
"0D790ZZ"  =  "1"  /* Dilation of Duodenum, Open Approach */
"0D793ZZ"  =  "1"  /* Dilation of Duodenum, Percutaneous Approach */
"0D794ZZ"  =  "1"  /* Dilation of Duodenum, Percutaneous Endoscopic Approach */
"0D7A0ZZ"  =  "1"  /* Dilation of Jejunum, Open Approach */
"0D7A3ZZ"  =  "1"  /* Dilation of Jejunum, Percutaneous Approach */
"0D7A4ZZ"  =  "1"  /* Dilation of Jejunum, Percutaneous Endoscopic Approach */
"0D7B0ZZ"  =  "1"  /* Dilation of Ileum, Open Approach */
"0D7B3ZZ"  =  "1"  /* Dilation of Ileum, Percutaneous Approach */
"0D7B4ZZ"  =  "1"  /* Dilation of Ileum, Percutaneous Endoscopic Approach */
"0D7C0ZZ"  =  "1"  /* Dilation of Ileocecal Valve, Open Approach */
"0D7C3ZZ"  =  "1"  /* Dilation of Ileocecal Valve, Percutaneous Approach */
"0D7C4ZZ"  =  "1"  /* Dilation of Ileocecal Valve, Perc Endo Approach */
"0D7E0ZZ"  =  "1"  /* Dilation of Large Intestine, Open Approach */
"0D7E3ZZ"  =  "1"  /* Dilation of Large Intestine, Percutaneous Approach */
"0D7E4ZZ"  =  "1"  /* Dilation of Large Intestine, Perc Endo Approach */
"0D7F0ZZ"  =  "1"  /* Dilation of Right Large Intestine, Open Approach */
"0D7F3ZZ"  =  "1"  /* Dilation of Right Large Intestine, Percutaneous Approach */
"0D7F4ZZ"  =  "1"  /* Dilation of Right Large Intestine, Perc Endo Approach */
"0D7G0ZZ"  =  "1"  /* Dilation of Left Large Intestine, Open Approach */
"0D7G3ZZ"  =  "1"  /* Dilation of Left Large Intestine, Percutaneous Approach */
"0D7G4ZZ"  =  "1"  /* Dilation of Left Large Intestine, Perc Endo Approach */
"0D7H0ZZ"  =  "1"  /* Dilation of Cecum, Open Approach */
"0D7H3ZZ"  =  "1"  /* Dilation of Cecum, Percutaneous Approach */
"0D7H4ZZ"  =  "1"  /* Dilation of Cecum, Percutaneous Endoscopic Approach */
"0D7K0ZZ"  =  "1"  /* Dilation of Ascending Colon, Open Approach */
"0D7K3ZZ"  =  "1"  /* Dilation of Ascending Colon, Percutaneous Approach */
"0D7K4ZZ"  =  "1"  /* Dilation of Ascending Colon, Perc Endo Approach */
"0D7L0ZZ"  =  "1"  /* Dilation of Transverse Colon, Open Approach */
"0D7L3ZZ"  =  "1"  /* Dilation of Transverse Colon, Percutaneous Approach */
"0D7L4ZZ"  =  "1"  /* Dilation of Transverse Colon, Perc Endo Approach */
"0D7M0ZZ"  =  "1"  /* Dilation of Descending Colon, Open Approach */
"0D7M3ZZ"  =  "1"  /* Dilation of Descending Colon, Percutaneous Approach */
"0D7M4ZZ"  =  "1"  /* Dilation of Descending Colon, Perc Endo Approach */
"0D7N0ZZ"  =  "1"  /* Dilation of Sigmoid Colon, Open Approach */
"0D7N3ZZ"  =  "1"  /* Dilation of Sigmoid Colon, Percutaneous Approach */
"0D7N4ZZ"  =  "1"  /* Dilation of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D7P0DZ"  =  "1"  /* Dilation of Rectum with Intraluminal Device, Open Approach */
"0D7P0ZZ"  =  "1"  /* Dilation of Rectum, Open Approach */
"0D7P3DZ"  =  "1"  /* Dilation of Rectum with Intraluminal Device, Perc Approach */
"0D7P3ZZ"  =  "1"  /* Dilation of Rectum, Percutaneous Approach */
"0D7P4DZ"  =  "1"  /* Dilation of Rectum with Intralum Dev, Perc Endo Approach */
"0D7P4ZZ"  =  "1"  /* Dilation of Rectum, Percutaneous Endoscopic Approach */
"0D7Q0DZ"  =  "1"  /* Dilation of Anus with Intraluminal Device, Open Approach */
"0D7Q0ZZ"  =  "1"  /* Dilation of Anus, Open Approach */
"0D7Q3DZ"  =  "1"  /* Dilation of Anus with Intraluminal Device, Perc Approach */
"0D7Q3ZZ"  =  "1"  /* Dilation of Anus, Percutaneous Approach */
"0D7Q4DZ"  =  "1"  /* Dilation of Anus with Intralum Dev, Perc Endo Approach */
"0D7Q4ZZ"  =  "1"  /* Dilation of Anus, Percutaneous Endoscopic Approach */
"0D840ZZ"  =  "1"  /* Division of Esophagogastric Junction, Open Approach */
"0D843ZZ"  =  "1"  /* Division of Esophagogastric Junction, Percutaneous Approach */
"0D844ZZ"  =  "1"  /* Division of Esophagogastric Junction, Perc Endo Approach */
"0D847ZZ"  =  "1"  /* Division of Esophagogastric Junction, Via Opening */
"0D848ZZ"  =  "1"  /* Division of Esophagogastric Junction, Endo */
"0D870ZZ"  =  "1"  /* Division of Stomach, Pylorus, Open Approach */
"0D873ZZ"  =  "1"  /* Division of Stomach, Pylorus, Percutaneous Approach */
"0D874ZZ"  =  "1"  /* Division of Stomach, Pylorus, Perc Endo Approach */
"0D877ZZ"  =  "1"  /* Division of Stomach, Pylorus, Via Opening */
"0D878ZZ"  =  "1"  /* Division of Stomach, Pylorus, Endo */
"0D8R0ZZ"  =  "1"  /* Division of Anal Sphincter, Open Approach */
"0D8R3ZZ"  =  "1"  /* Division of Anal Sphincter, Percutaneous Approach */
"0D9100Z"  =  "1"  /* Drainage of Upper Esophagus with Drain Dev, Open Approach */
"0D910ZX"  =  "1"  /* Drainage of Upper Esophagus, Open Approach, Diagnostic */
"0D910ZZ"  =  "1"  /* Drainage of Upper Esophagus, Open Approach */
"0D9130Z"  =  "1"  /* Drainage of Upper Esophagus with Drain Dev, Perc Approach */
"0D913ZZ"  =  "1"  /* Drainage of Upper Esophagus, Percutaneous Approach */
"0D9140Z"  =  "1"  /* Drainage of Up Esophag with Drain Dev, Perc Endo Approach */
"0D914ZZ"  =  "1"  /* Drainage of Upper Esophagus, Perc Endo Approach */
"0D9170Z"  =  "1"  /* Drainage of Upper Esophagus with Drain Dev, Via Opening */
"0D917ZZ"  =  "1"  /* Drainage of Upper Esophagus, Via Opening */
"0D9180Z"  =  "1"  /* Drainage of Upper Esophagus with Drainage Device, Endo */
"0D918ZZ"  =  "1"  /* Drainage of Upper Esophagus, Endo */
"0D9200Z"  =  "1"  /* Drainage of Middle Esophagus with Drain Dev, Open Approach */
"0D920ZX"  =  "1"  /* Drainage of Middle Esophagus, Open Approach, Diagnostic */
"0D920ZZ"  =  "1"  /* Drainage of Middle Esophagus, Open Approach */
"0D9230Z"  =  "1"  /* Drainage of Middle Esophagus with Drain Dev, Perc Approach */
"0D923ZZ"  =  "1"  /* Drainage of Middle Esophagus, Percutaneous Approach */
"0D9240Z"  =  "1"  /* Drainage of Mid Esophag with Drain Dev, Perc Endo Approach */
"0D924ZZ"  =  "1"  /* Drainage of Middle Esophagus, Perc Endo Approach */
"0D9270Z"  =  "1"  /* Drainage of Middle Esophagus with Drain Dev, Via Opening */
"0D927ZZ"  =  "1"  /* Drainage of Middle Esophagus, Via Opening */
"0D9280Z"  =  "1"  /* Drainage of Middle Esophagus with Drainage Device, Endo */
"0D928ZZ"  =  "1"  /* Drainage of Middle Esophagus, Endo */
"0D9300Z"  =  "1"  /* Drainage of Lower Esophagus with Drain Dev, Open Approach */
"0D930ZX"  =  "1"  /* Drainage of Lower Esophagus, Open Approach, Diagnostic */
"0D930ZZ"  =  "1"  /* Drainage of Lower Esophagus, Open Approach */
"0D9330Z"  =  "1"  /* Drainage of Lower Esophagus with Drain Dev, Perc Approach */
"0D933ZZ"  =  "1"  /* Drainage of Lower Esophagus, Percutaneous Approach */
"0D9340Z"  =  "1"  /* Drainage of Low Esophag with Drain Dev, Perc Endo Approach */
"0D934ZZ"  =  "1"  /* Drainage of Lower Esophagus, Perc Endo Approach */
"0D9370Z"  =  "1"  /* Drainage of Lower Esophagus with Drain Dev, Via Opening */
"0D937ZZ"  =  "1"  /* Drainage of Lower Esophagus, Via Opening */
"0D9380Z"  =  "1"  /* Drainage of Lower Esophagus with Drainage Device, Endo */
"0D938ZZ"  =  "1"  /* Drainage of Lower Esophagus, Endo */
"0D9400Z"  =  "1"  /* Drainage of Esophagast Junct with Drain Dev, Open Approach */
"0D940ZX"  =  "1"  /* Drainage of Esophagogastric Junction, Open Approach, Diagn */
"0D940ZZ"  =  "1"  /* Drainage of Esophagogastric Junction, Open Approach */
"0D9430Z"  =  "1"  /* Drainage of Esophagast Junct with Drain Dev, Perc Approach */
"0D943ZZ"  =  "1"  /* Drainage of Esophagogastric Junction, Percutaneous Approach */
"0D9440Z"  =  "1"  /* Drain of Esophagast Junct with Drain Dev, Perc Endo Approach */
"0D944ZZ"  =  "1"  /* Drainage of Esophagogastric Junction, Perc Endo Approach */
"0D9470Z"  =  "1"  /* Drainage of Esophagast Junct with Drain Dev, Via Opening */
"0D947ZZ"  =  "1"  /* Drainage of Esophagogastric Junction, Via Opening */
"0D9480Z"  =  "1"  /* Drainage of Esophagogastric Junction with Drain Dev, Endo */
"0D948ZZ"  =  "1"  /* Drainage of Esophagogastric Junction, Endo */
"0D9500Z"  =  "1"  /* Drainage of Esophagus with Drainage Device, Open Approach */
"0D950ZX"  =  "1"  /* Drainage of Esophagus, Open Approach, Diagnostic */
"0D950ZZ"  =  "1"  /* Drainage of Esophagus, Open Approach */
"0D9530Z"  =  "1"  /* Drainage of Esophagus with Drainage Device, Perc Approach */
"0D953ZZ"  =  "1"  /* Drainage of Esophagus, Percutaneous Approach */
"0D9540Z"  =  "1"  /* Drainage of Esophagus with Drain Dev, Perc Endo Approach */
"0D954ZZ"  =  "1"  /* Drainage of Esophagus, Percutaneous Endoscopic Approach */
"0D9570Z"  =  "1"  /* Drainage of Esophagus with Drainage Device, Via Opening */
"0D957ZZ"  =  "1"  /* Drainage of Esophagus, Via Natural or Artificial Opening */
"0D9580Z"  =  "1"  /* Drainage of Esophagus with Drainage Device, Endo */
"0D958ZZ"  =  "1"  /* Drainage of Esophagus, Endo */
"0D9600Z"  =  "1"  /* Drainage of Stomach with Drainage Device, Open Approach */
"0D960ZX"  =  "1"  /* Drainage of Stomach, Open Approach, Diagnostic */
"0D960ZZ"  =  "1"  /* Drainage of Stomach, Open Approach */
"0D9630Z"  =  "1"  /* Drainage of Stomach with Drainage Device, Perc Approach */
"0D963ZZ"  =  "1"  /* Drainage of Stomach, Percutaneous Approach */
"0D9640Z"  =  "1"  /* Drainage of Stomach with Drainage Device, Perc Endo Approach */
"0D964ZZ"  =  "1"  /* Drainage of Stomach, Percutaneous Endoscopic Approach */
"0D967ZZ"  =  "1"  /* Drainage of Stomach, Via Natural or Artificial Opening */
"0D968ZZ"  =  "1"  /* Drainage of Stomach, Endo */
"0D9700Z"  =  "1"  /* Drainage of Stomach, Pylorus with Drain Dev, Open Approach */
"0D970ZX"  =  "1"  /* Drainage of Stomach, Pylorus, Open Approach, Diagnostic */
"0D970ZZ"  =  "1"  /* Drainage of Stomach, Pylorus, Open Approach */
"0D9730Z"  =  "1"  /* Drainage of Stomach, Pylorus with Drain Dev, Perc Approach */
"0D973ZZ"  =  "1"  /* Drainage of Stomach, Pylorus, Percutaneous Approach */
"0D9740Z"  =  "1"  /* Drain of Stomach, Pylrs with Drain Dev, Perc Endo Approach */
"0D974ZZ"  =  "1"  /* Drainage of Stomach, Pylorus, Perc Endo Approach */
"0D977ZZ"  =  "1"  /* Drainage of Stomach, Pylorus, Via Opening */
"0D978ZZ"  =  "1"  /* Drainage of Stomach, Pylorus, Endo */
"0D9800Z"  =  "1"  /* Drainage of Small Intestine with Drain Dev, Open Approach */
"0D980ZX"  =  "1"  /* Drainage of Small Intestine, Open Approach, Diagnostic */
"0D980ZZ"  =  "1"  /* Drainage of Small Intestine, Open Approach */
"0D9830Z"  =  "1"  /* Drainage of Small Intestine with Drain Dev, Perc Approach */
"0D983ZZ"  =  "1"  /* Drainage of Small Intestine, Percutaneous Approach */
"0D9840Z"  =  "1"  /* Drainage of Small Intest with Drain Dev, Perc Endo Approach */
"0D984ZZ"  =  "1"  /* Drainage of Small Intestine, Perc Endo Approach */
"0D987ZZ"  =  "1"  /* Drainage of Small Intestine, Via Opening */
"0D988ZZ"  =  "1"  /* Drainage of Small Intestine, Endo */
"0D9900Z"  =  "1"  /* Drainage of Duodenum with Drainage Device, Open Approach */
"0D990ZX"  =  "1"  /* Drainage of Duodenum, Open Approach, Diagnostic */
"0D990ZZ"  =  "1"  /* Drainage of Duodenum, Open Approach */
"0D9930Z"  =  "1"  /* Drainage of Duodenum with Drainage Device, Perc Approach */
"0D993ZZ"  =  "1"  /* Drainage of Duodenum, Percutaneous Approach */
"0D9940Z"  =  "1"  /* Drainage of Duodenum with Drain Dev, Perc Endo Approach */
"0D994ZZ"  =  "1"  /* Drainage of Duodenum, Percutaneous Endoscopic Approach */
"0D997ZZ"  =  "1"  /* Drainage of Duodenum, Via Natural or Artificial Opening */
"0D998ZZ"  =  "1"  /* Drainage of Duodenum, Endo */
"0D9A00Z"  =  "1"  /* Drainage of Jejunum with Drainage Device, Open Approach */
"0D9A0ZX"  =  "1"  /* Drainage of Jejunum, Open Approach, Diagnostic */
"0D9A0ZZ"  =  "1"  /* Drainage of Jejunum, Open Approach */
"0D9A30Z"  =  "1"  /* Drainage of Jejunum with Drainage Device, Perc Approach */
"0D9A3ZZ"  =  "1"  /* Drainage of Jejunum, Percutaneous Approach */
"0D9A40Z"  =  "1"  /* Drainage of Jejunum with Drainage Device, Perc Endo Approach */
"0D9A4ZZ"  =  "1"  /* Drainage of Jejunum, Percutaneous Endoscopic Approach */
"0D9A7ZZ"  =  "1"  /* Drainage of Jejunum, Via Natural or Artificial Opening */
"0D9A8ZZ"  =  "1"  /* Drainage of Jejunum, Endo */
"0D9B00Z"  =  "1"  /* Drainage of Ileum with Drainage Device, Open Approach */
"0D9B0ZX"  =  "1"  /* Drainage of Ileum, Open Approach, Diagnostic */
"0D9B0ZZ"  =  "1"  /* Drainage of Ileum, Open Approach */
"0D9B30Z"  =  "1"  /* Drainage of Ileum with Drainage Device, Perc Approach */
"0D9B3ZZ"  =  "1"  /* Drainage of Ileum, Percutaneous Approach */
"0D9B40Z"  =  "1"  /* Drainage of Ileum with Drainage Device, Perc Endo Approach */
"0D9B4ZZ"  =  "1"  /* Drainage of Ileum, Percutaneous Endoscopic Approach */
"0D9B7ZZ"  =  "1"  /* Drainage of Ileum, Via Natural or Artificial Opening */
"0D9B8ZZ"  =  "1"  /* Drainage of Ileum, Endo */
"0D9C00Z"  =  "1"  /* Drainage of Ileocecal Valve with Drain Dev, Open Approach */
"0D9C0ZX"  =  "1"  /* Drainage of Ileocecal Valve, Open Approach, Diagnostic */
"0D9C0ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Open Approach */
"0D9C30Z"  =  "1"  /* Drainage of Ileocecal Valve with Drain Dev, Perc Approach */
"0D9C3ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Percutaneous Approach */
"0D9C40Z"  =  "1"  /* Drain of Ileocecal Valve with Drain Dev, Perc Endo Approach */
"0D9C4ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Perc Endo Approach */
"0D9C70Z"  =  "1"  /* Drainage of Ileocecal Valve with Drain Dev, Via Opening */
"0D9C7ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Via Opening */
"0D9C80Z"  =  "1"  /* Drainage of Ileocecal Valve with Drainage Device, Endo */
"0D9C8ZZ"  =  "1"  /* Drainage of Ileocecal Valve, Endo */
"0D9E00Z"  =  "1"  /* Drainage of Large Intestine with Drain Dev, Open Approach */
"0D9E0ZX"  =  "1"  /* Drainage of Large Intestine, Open Approach, Diagnostic */
"0D9E0ZZ"  =  "1"  /* Drainage of Large Intestine, Open Approach */
"0D9E30Z"  =  "1"  /* Drainage of Large Intestine with Drain Dev, Perc Approach */
"0D9E3ZZ"  =  "1"  /* Drainage of Large Intestine, Percutaneous Approach */
"0D9E40Z"  =  "1"  /* Drainage of Lg Intest with Drain Dev, Perc Endo Approach */
"0D9E4ZZ"  =  "1"  /* Drainage of Large Intestine, Perc Endo Approach */
"0D9E7ZZ"  =  "1"  /* Drainage of Large Intestine, Via Opening */
"0D9E8ZZ"  =  "1"  /* Drainage of Large Intestine, Endo */
"0D9F00Z"  =  "1"  /* Drainage of R Lg Intest with Drain Dev, Open Approach */
"0D9F0ZX"  =  "1"  /* Drainage of Right Large Intestine, Open Approach, Diagnostic */
"0D9F0ZZ"  =  "1"  /* Drainage of Right Large Intestine, Open Approach */
"0D9F30Z"  =  "1"  /* Drainage of R Lg Intest with Drain Dev, Perc Approach */
"0D9F3ZZ"  =  "1"  /* Drainage of Right Large Intestine, Percutaneous Approach */
"0D9F40Z"  =  "1"  /* Drainage of R Lg Intest with Drain Dev, Perc Endo Approach */
"0D9F4ZZ"  =  "1"  /* Drainage of Right Large Intestine, Perc Endo Approach */
"0D9F7ZZ"  =  "1"  /* Drainage of Right Large Intestine, Via Opening */
"0D9F8ZZ"  =  "1"  /* Drainage of Right Large Intestine, Endo */
"0D9G00Z"  =  "1"  /* Drainage of L Lg Intest with Drain Dev, Open Approach */
"0D9G0ZX"  =  "1"  /* Drainage of Left Large Intestine, Open Approach, Diagnostic */
"0D9G0ZZ"  =  "1"  /* Drainage of Left Large Intestine, Open Approach */
"0D9G30Z"  =  "1"  /* Drainage of L Lg Intest with Drain Dev, Perc Approach */
"0D9G3ZZ"  =  "1"  /* Drainage of Left Large Intestine, Percutaneous Approach */
"0D9G40Z"  =  "1"  /* Drainage of L Lg Intest with Drain Dev, Perc Endo Approach */
"0D9G4ZZ"  =  "1"  /* Drainage of Left Large Intestine, Perc Endo Approach */
"0D9G7ZZ"  =  "1"  /* Drainage of Left Large Intestine, Via Opening */
"0D9G8ZZ"  =  "1"  /* Drainage of Left Large Intestine, Endo */
"0D9H00Z"  =  "1"  /* Drainage of Cecum with Drainage Device, Open Approach */
"0D9H0ZX"  =  "1"  /* Drainage of Cecum, Open Approach, Diagnostic */
"0D9H0ZZ"  =  "1"  /* Drainage of Cecum, Open Approach */
"0D9H30Z"  =  "1"  /* Drainage of Cecum with Drainage Device, Perc Approach */
"0D9H3ZZ"  =  "1"  /* Drainage of Cecum, Percutaneous Approach */
"0D9H40Z"  =  "1"  /* Drainage of Cecum with Drainage Device, Perc Endo Approach */
"0D9H4ZZ"  =  "1"  /* Drainage of Cecum, Percutaneous Endoscopic Approach */
"0D9H7ZZ"  =  "1"  /* Drainage of Cecum, Via Natural or Artificial Opening */
"0D9H8ZZ"  =  "1"  /* Drainage of Cecum, Endo */
"0D9J00Z"  =  "1"  /* Drainage of Appendix with Drainage Device, Open Approach */
"0D9J0ZX"  =  "1"  /* Drainage of Appendix, Open Approach, Diagnostic */
"0D9J0ZZ"  =  "1"  /* Drainage of Appendix, Open Approach */
"0D9J30Z"  =  "1"  /* Drainage of Appendix with Drainage Device, Perc Approach */
"0D9J3ZX"  =  "1"  /* Drainage of Appendix, Percutaneous Approach, Diagnostic */
"0D9J3ZZ"  =  "1"  /* Drainage of Appendix, Percutaneous Approach */
"0D9J40Z"  =  "1"  /* Drainage of Appendix with Drain Dev, Perc Endo Approach */
"0D9J4ZX"  =  "1"  /* Drainage of Appendix, Perc Endo Approach, Diagn */
"0D9J4ZZ"  =  "1"  /* Drainage of Appendix, Percutaneous Endoscopic Approach */
"0D9J70Z"  =  "1"  /* Drainage of Appendix with Drainage Device, Via Opening */
"0D9J7ZX"  =  "1"  /* Drainage of Appendix, Via Opening, Diagn */
"0D9J7ZZ"  =  "1"  /* Drainage of Appendix, Via Natural or Artificial Opening */
"0D9J80Z"  =  "1"  /* Drainage of Appendix with Drainage Device, Endo */
"0D9J8ZX"  =  "1"  /* Drainage of Appendix, Endo, Diagn */
"0D9J8ZZ"  =  "1"  /* Drainage of Appendix, Endo */
"0D9K00Z"  =  "1"  /* Drainage of Ascending Colon with Drain Dev, Open Approach */
"0D9K0ZX"  =  "1"  /* Drainage of Ascending Colon, Open Approach, Diagnostic */
"0D9K0ZZ"  =  "1"  /* Drainage of Ascending Colon, Open Approach */
"0D9K30Z"  =  "1"  /* Drainage of Ascending Colon with Drain Dev, Perc Approach */
"0D9K3ZZ"  =  "1"  /* Drainage of Ascending Colon, Percutaneous Approach */
"0D9K40Z"  =  "1"  /* Drainage of Asc Colon with Drain Dev, Perc Endo Approach */
"0D9K4ZZ"  =  "1"  /* Drainage of Ascending Colon, Perc Endo Approach */
"0D9K7ZZ"  =  "1"  /* Drainage of Ascending Colon, Via Opening */
"0D9K8ZZ"  =  "1"  /* Drainage of Ascending Colon, Endo */
"0D9L00Z"  =  "1"  /* Drainage of Transverse Colon with Drain Dev, Open Approach */
"0D9L0ZX"  =  "1"  /* Drainage of Transverse Colon, Open Approach, Diagnostic */
"0D9L0ZZ"  =  "1"  /* Drainage of Transverse Colon, Open Approach */
"0D9L30Z"  =  "1"  /* Drainage of Transverse Colon with Drain Dev, Perc Approach */
"0D9L3ZZ"  =  "1"  /* Drainage of Transverse Colon, Percutaneous Approach */
"0D9L40Z"  =  "1"  /* Drainage of Trans Colon with Drain Dev, Perc Endo Approach */
"0D9L4ZZ"  =  "1"  /* Drainage of Transverse Colon, Perc Endo Approach */
"0D9L7ZZ"  =  "1"  /* Drainage of Transverse Colon, Via Opening */
"0D9L8ZZ"  =  "1"  /* Drainage of Transverse Colon, Endo */
"0D9M00Z"  =  "1"  /* Drainage of Descending Colon with Drain Dev, Open Approach */
"0D9M0ZX"  =  "1"  /* Drainage of Descending Colon, Open Approach, Diagnostic */
"0D9M0ZZ"  =  "1"  /* Drainage of Descending Colon, Open Approach */
"0D9M30Z"  =  "1"  /* Drainage of Descending Colon with Drain Dev, Perc Approach */
"0D9M3ZZ"  =  "1"  /* Drainage of Descending Colon, Percutaneous Approach */
"0D9M40Z"  =  "1"  /* Drainage of Descend Colon with Drain Dev, Perc Endo Approach */
"0D9M4ZZ"  =  "1"  /* Drainage of Descending Colon, Perc Endo Approach */
"0D9M7ZZ"  =  "1"  /* Drainage of Descending Colon, Via Opening */
"0D9M8ZZ"  =  "1"  /* Drainage of Descending Colon, Endo */
"0D9N00Z"  =  "1"  /* Drainage of Sigmoid Colon with Drain Dev, Open Approach */
"0D9N0ZX"  =  "1"  /* Drainage of Sigmoid Colon, Open Approach, Diagnostic */
"0D9N0ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Open Approach */
"0D9N30Z"  =  "1"  /* Drainage of Sigmoid Colon with Drain Dev, Perc Approach */
"0D9N3ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Percutaneous Approach */
"0D9N40Z"  =  "1"  /* Drainage of Sigmoid Colon with Drain Dev, Perc Endo Approach */
"0D9N4ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0D9N7ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Via Natural or Artificial Opening */
"0D9N8ZZ"  =  "1"  /* Drainage of Sigmoid Colon, Endo */
"0D9P00Z"  =  "1"  /* Drainage of Rectum with Drainage Device, Open Approach */
"0D9P0ZX"  =  "1"  /* Drainage of Rectum, Open Approach, Diagnostic */
"0D9P0ZZ"  =  "1"  /* Drainage of Rectum, Open Approach */
"0D9P30Z"  =  "1"  /* Drainage of Rectum with Drainage Device, Perc Approach */
"0D9P3ZZ"  =  "1"  /* Drainage of Rectum, Percutaneous Approach */
"0D9P40Z"  =  "1"  /* Drainage of Rectum with Drainage Device, Perc Endo Approach */
"0D9P4ZZ"  =  "1"  /* Drainage of Rectum, Percutaneous Endoscopic Approach */
"0D9P7ZZ"  =  "1"  /* Drainage of Rectum, Via Natural or Artificial Opening */
"0D9P8ZZ"  =  "1"  /* Drainage of Rectum, Endo */
"0D9Q00Z"  =  "1"  /* Drainage of Anus with Drainage Device, Open Approach */
"0D9Q0ZZ"  =  "1"  /* Drainage of Anus, Open Approach */
"0D9Q30Z"  =  "1"  /* Drainage of Anus with Drainage Device, Percutaneous Approach */
"0D9Q3ZZ"  =  "1"  /* Drainage of Anus, Percutaneous Approach */
"0D9Q40Z"  =  "1"  /* Drainage of Anus with Drainage Device, Perc Endo Approach */
"0D9Q4ZZ"  =  "1"  /* Drainage of Anus, Percutaneous Endoscopic Approach */
"0D9Q70Z"  =  "1"  /* Drainage of Anus with Drainage Device, Via Opening */
"0D9Q7ZZ"  =  "1"  /* Drainage of Anus, Via Natural or Artificial Opening */
"0D9Q80Z"  =  "1"  /* Drainage of Anus with Drainage Device, Endo */
"0D9Q8ZZ"  =  "1"  /* Drainage of Anus, Endo */
"0D9QX0Z"  =  "1"  /* Drainage of Anus with Drainage Device, External Approach */
"0D9QXZZ"  =  "1"  /* Drainage of Anus, External Approach */
"0D9R00Z"  =  "1"  /* Drainage of Anal Sphincter with Drain Dev, Open Approach */
"0D9R0ZZ"  =  "1"  /* Drainage of Anal Sphincter, Open Approach */
"0D9R30Z"  =  "1"  /* Drainage of Anal Sphincter with Drain Dev, Perc Approach */
"0D9R3ZZ"  =  "1"  /* Drainage of Anal Sphincter, Percutaneous Approach */
"0D9R40Z"  =  "1"  /* Drain of Anal Sphincter with Drain Dev, Perc Endo Approach */
"0D9R4ZZ"  =  "1"  /* Drainage of Anal Sphincter, Percutaneous Endoscopic Approach */
"0D9S00Z"  =  "1"  /* Drainage of Greater Omentum with Drain Dev, Open Approach */
"0D9S0ZX"  =  "1"  /* Drainage of Greater Omentum, Open Approach, Diagnostic */
"0D9S0ZZ"  =  "1"  /* Drainage of Greater Omentum, Open Approach */
"0D9S3ZX"  =  "1"  /* Drainage of Greater Omentum, Percutaneous Approach, Diagn */
"0D9S4ZX"  =  "1"  /* Drainage of Greater Omentum, Perc Endo Approach, Diagn */
"0D9T00Z"  =  "1"  /* Drainage of Lesser Omentum with Drain Dev, Open Approach */
"0D9T0ZX"  =  "1"  /* Drainage of Lesser Omentum, Open Approach, Diagnostic */
"0D9T0ZZ"  =  "1"  /* Drainage of Lesser Omentum, Open Approach */
"0D9T3ZX"  =  "1"  /* Drainage of Lesser Omentum, Percutaneous Approach, Diagn */
"0D9T4ZX"  =  "1"  /* Drainage of Lesser Omentum, Perc Endo Approach, Diagn */
"0D9V00Z"  =  "1"  /* Drainage of Mesentery with Drainage Device, Open Approach */
"0D9V0ZX"  =  "1"  /* Drainage of Mesentery, Open Approach, Diagnostic */
"0D9V0ZZ"  =  "1"  /* Drainage of Mesentery, Open Approach */
"0D9V3ZX"  =  "1"  /* Drainage of Mesentery, Percutaneous Approach, Diagnostic */
"0D9V4ZX"  =  "1"  /* Drainage of Mesentery, Perc Endo Approach, Diagn */
"0D9W00Z"  =  "1"  /* Drainage of Peritoneum with Drainage Device, Open Approach */
"0D9W0ZX"  =  "1"  /* Drainage of Peritoneum, Open Approach, Diagnostic */
"0D9W0ZZ"  =  "1"  /* Drainage of Peritoneum, Open Approach */
"0D9W3ZX"  =  "1"  /* Drainage of Peritoneum, Percutaneous Approach, Diagnostic */
"0D9W4ZX"  =  "1"  /* Drainage of Peritoneum, Perc Endo Approach, Diagn */
"0DB10ZX"  =  "1"  /* Excision of Upper Esophagus, Open Approach, Diagnostic */
"0DB10ZZ"  =  "1"  /* Excision of Upper Esophagus, Open Approach */
"0DB13ZZ"  =  "1"  /* Excision of Upper Esophagus, Percutaneous Approach */
"0DB17ZZ"  =  "1"  /* Excision of Upper Esophagus, Via Opening */
"0DB20ZX"  =  "1"  /* Excision of Middle Esophagus, Open Approach, Diagnostic */
"0DB20ZZ"  =  "1"  /* Excision of Middle Esophagus, Open Approach */
"0DB23ZZ"  =  "1"  /* Excision of Middle Esophagus, Percutaneous Approach */
"0DB27ZZ"  =  "1"  /* Excision of Middle Esophagus, Via Opening */
"0DB30ZX"  =  "1"  /* Excision of Lower Esophagus, Open Approach, Diagnostic */
"0DB30ZZ"  =  "1"  /* Excision of Lower Esophagus, Open Approach */
"0DB33ZZ"  =  "1"  /* Excision of Lower Esophagus, Percutaneous Approach */
"0DB37ZZ"  =  "1"  /* Excision of Lower Esophagus, Via Opening */
"0DB40ZX"  =  "1"  /* Excision of Esophagogastric Junction, Open Approach, Diagn */
"0DB40ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Open Approach */
"0DB43ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Percutaneous Approach */
"0DB44ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Perc Endo Approach */
"0DB47ZZ"  =  "1"  /* Excision of Esophagogastric Junction, Via Opening */
"0DB50ZX"  =  "1"  /* Excision of Esophagus, Open Approach, Diagnostic */
"0DB50ZZ"  =  "1"  /* Excision of Esophagus, Open Approach */
"0DB53ZZ"  =  "1"  /* Excision of Esophagus, Percutaneous Approach */
"0DB57ZZ"  =  "1"  /* Excision of Esophagus, Via Natural or Artificial Opening */
"0DB60Z3"  =  "1"  /* Excision of Stomach, Open Approach, Vertical */
"0DB60ZX"  =  "1"  /* Excision of Stomach, Open Approach, Diagnostic */
"0DB60ZZ"  =  "1"  /* Excision of Stomach, Open Approach */
"0DB63Z3"  =  "1"  /* Excision of Stomach, Percutaneous Approach, Vertical */
"0DB63ZZ"  =  "1"  /* Excision of Stomach, Percutaneous Approach */
"0DB64Z3"  =  "1"  /* Excision of Stomach, Percutaneous Endoscopic Approach, Vert */
"0DB67Z3"  =  "1"  /* Excision of Stomach, Via Natural or Artificial Opening, Vert */
"0DB67ZZ"  =  "1"  /* Excision of Stomach, Via Natural or Artificial Opening */
"0DB68Z3"  =  "1"  /* Excision of Stomach, Endo, Vert */
"0DB70ZX"  =  "1"  /* Excision of Stomach, Pylorus, Open Approach, Diagnostic */
"0DB70ZZ"  =  "1"  /* Excision of Stomach, Pylorus, Open Approach */
"0DB73ZZ"  =  "1"  /* Excision of Stomach, Pylorus, Percutaneous Approach */
"0DB77ZZ"  =  "1"  /* Excision of Stomach, Pylorus, Via Opening */
"0DB80ZX"  =  "1"  /* Excision of Small Intestine, Open Approach, Diagnostic */
"0DB80ZZ"  =  "1"  /* Excision of Small Intestine, Open Approach */
"0DB83ZZ"  =  "1"  /* Excision of Small Intestine, Percutaneous Approach */
"0DB84ZZ"  =  "1"  /* Excision of Small Intestine, Perc Endo Approach */
"0DB87ZZ"  =  "1"  /* Excision of Small Intestine, Via Opening */
"0DB88ZZ"  =  "1"  /* Excision of Small Intestine, Endo */
"0DB90ZX"  =  "1"  /* Excision of Duodenum, Open Approach, Diagnostic */
"0DB90ZZ"  =  "1"  /* Excision of Duodenum, Open Approach */
"0DB93ZZ"  =  "1"  /* Excision of Duodenum, Percutaneous Approach */
"0DB97ZZ"  =  "1"  /* Excision of Duodenum, Via Natural or Artificial Opening */
"0DBA0ZX"  =  "1"  /* Excision of Jejunum, Open Approach, Diagnostic */
"0DBA0ZZ"  =  "1"  /* Excision of Jejunum, Open Approach */
"0DBA3ZZ"  =  "1"  /* Excision of Jejunum, Percutaneous Approach */
"0DBA4ZZ"  =  "1"  /* Excision of Jejunum, Percutaneous Endoscopic Approach */
"0DBA7ZZ"  =  "1"  /* Excision of Jejunum, Via Natural or Artificial Opening */
"0DBA8ZZ"  =  "1"  /* Excision of Jejunum, Endo */
"0DBB0ZX"  =  "1"  /* Excision of Ileum, Open Approach, Diagnostic */
"0DBB0ZZ"  =  "1"  /* Excision of Ileum, Open Approach */
"0DBB3ZZ"  =  "1"  /* Excision of Ileum, Percutaneous Approach */
"0DBB4ZZ"  =  "1"  /* Excision of Ileum, Percutaneous Endoscopic Approach */
"0DBB7ZZ"  =  "1"  /* Excision of Ileum, Via Natural or Artificial Opening */
"0DBB8ZZ"  =  "1"  /* Excision of Ileum, Endo */
"0DBC0ZX"  =  "1"  /* Excision of Ileocecal Valve, Open Approach, Diagnostic */
"0DBC0ZZ"  =  "1"  /* Excision of Ileocecal Valve, Open Approach */
"0DBC3ZZ"  =  "1"  /* Excision of Ileocecal Valve, Percutaneous Approach */
"0DBC4ZZ"  =  "1"  /* Excision of Ileocecal Valve, Perc Endo Approach */
"0DBC7ZZ"  =  "1"  /* Excision of Ileocecal Valve, Via Opening */
"0DBC8ZZ"  =  "1"  /* Excision of Ileocecal Valve, Endo */
"0DBE0ZX"  =  "1"  /* Excision of Large Intestine, Open Approach, Diagnostic */
"0DBE0ZZ"  =  "1"  /* Excision of Large Intestine, Open Approach */
"0DBE3ZZ"  =  "1"  /* Excision of Large Intestine, Percutaneous Approach */
"0DBE4ZZ"  =  "1"  /* Excision of Large Intestine, Perc Endo Approach */
"0DBE7ZZ"  =  "1"  /* Excision of Large Intestine, Via Opening */
"0DBF0ZX"  =  "1"  /* Excision of Right Large Intestine, Open Approach, Diagnostic */
"0DBF0ZZ"  =  "1"  /* Excision of Right Large Intestine, Open Approach */
"0DBF3ZZ"  =  "1"  /* Excision of Right Large Intestine, Percutaneous Approach */
"0DBF4ZZ"  =  "1"  /* Excision of Right Large Intestine, Perc Endo Approach */
"0DBF7ZZ"  =  "1"  /* Excision of Right Large Intestine, Via Opening */
"0DBG0ZX"  =  "1"  /* Excision of Left Large Intestine, Open Approach, Diagnostic */
"0DBG0ZZ"  =  "1"  /* Excision of Left Large Intestine, Open Approach */
"0DBG3ZZ"  =  "1"  /* Excision of Left Large Intestine, Percutaneous Approach */
"0DBG4ZZ"  =  "1"  /* Excision of Left Large Intestine, Perc Endo Approach */
"0DBG7ZZ"  =  "1"  /* Excision of Left Large Intestine, Via Opening */
"0DBH0ZX"  =  "1"  /* Excision of Cecum, Open Approach, Diagnostic */
"0DBH0ZZ"  =  "1"  /* Excision of Cecum, Open Approach */
"0DBH3ZZ"  =  "1"  /* Excision of Cecum, Percutaneous Approach */
"0DBH4ZZ"  =  "1"  /* Excision of Cecum, Percutaneous Endoscopic Approach */
"0DBH7ZZ"  =  "1"  /* Excision of Cecum, Via Natural or Artificial Opening */
"0DBJ0ZX"  =  "1"  /* Excision of Appendix, Open Approach, Diagnostic */
"0DBJ0ZZ"  =  "1"  /* Excision of Appendix, Open Approach */
"0DBJ3ZX"  =  "1"  /* Excision of Appendix, Percutaneous Approach, Diagnostic */
"0DBJ3ZZ"  =  "1"  /* Excision of Appendix, Percutaneous Approach */
"0DBJ4ZX"  =  "1"  /* Excision of Appendix, Perc Endo Approach, Diagn */
"0DBJ4ZZ"  =  "1"  /* Excision of Appendix, Percutaneous Endoscopic Approach */
"0DBJ7ZX"  =  "1"  /* Excision of Appendix, Via Opening, Diagn */
"0DBJ7ZZ"  =  "1"  /* Excision of Appendix, Via Natural or Artificial Opening */
"0DBJ8ZX"  =  "1"  /* Excision of Appendix, Endo, Diagn */
"0DBJ8ZZ"  =  "1"  /* Excision of Appendix, Endo */
"0DBK0ZX"  =  "1"  /* Excision of Ascending Colon, Open Approach, Diagnostic */
"0DBK0ZZ"  =  "1"  /* Excision of Ascending Colon, Open Approach */
"0DBK3ZZ"  =  "1"  /* Excision of Ascending Colon, Percutaneous Approach */
"0DBK4ZZ"  =  "1"  /* Excision of Ascending Colon, Perc Endo Approach */
"0DBK7ZZ"  =  "1"  /* Excision of Ascending Colon, Via Opening */
"0DBL0ZX"  =  "1"  /* Excision of Transverse Colon, Open Approach, Diagnostic */
"0DBL0ZZ"  =  "1"  /* Excision of Transverse Colon, Open Approach */
"0DBL3ZZ"  =  "1"  /* Excision of Transverse Colon, Percutaneous Approach */
"0DBL4ZZ"  =  "1"  /* Excision of Transverse Colon, Perc Endo Approach */
"0DBL7ZZ"  =  "1"  /* Excision of Transverse Colon, Via Opening */
"0DBM0ZX"  =  "1"  /* Excision of Descending Colon, Open Approach, Diagnostic */
"0DBM0ZZ"  =  "1"  /* Excision of Descending Colon, Open Approach */
"0DBM3ZZ"  =  "1"  /* Excision of Descending Colon, Percutaneous Approach */
"0DBM4ZZ"  =  "1"  /* Excision of Descending Colon, Perc Endo Approach */
"0DBM7ZZ"  =  "1"  /* Excision of Descending Colon, Via Opening */
"0DBN0ZX"  =  "1"  /* Excision of Sigmoid Colon, Open Approach, Diagnostic */
"0DBN0ZZ"  =  "1"  /* Excision of Sigmoid Colon, Open Approach */
"0DBN3ZZ"  =  "1"  /* Excision of Sigmoid Colon, Percutaneous Approach */
"0DBN4ZZ"  =  "1"  /* Excision of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DBN7ZZ"  =  "1"  /* Excision of Sigmoid Colon, Via Natural or Artificial Opening */
"0DBP0ZX"  =  "1"  /* Excision of Rectum, Open Approach, Diagnostic */
"0DBP0ZZ"  =  "1"  /* Excision of Rectum, Open Approach */
"0DBP3ZZ"  =  "1"  /* Excision of Rectum, Percutaneous Approach */
"0DBP4ZZ"  =  "1"  /* Excision of Rectum, Percutaneous Endoscopic Approach */
"0DBP7ZZ"  =  "1"  /* Excision of Rectum, Via Natural or Artificial Opening */
"0DBQ0ZZ"  =  "1"  /* Excision of Anus, Open Approach */
"0DBQ3ZZ"  =  "1"  /* Excision of Anus, Percutaneous Approach */
"0DBQ4ZZ"  =  "1"  /* Excision of Anus, Percutaneous Endoscopic Approach */
"0DBQ7ZZ"  =  "1"  /* Excision of Anus, Via Natural or Artificial Opening */
"0DBQ8ZZ"  =  "1"  /* Excision of Anus, Endo */
"0DBQXZZ"  =  "1"  /* Excision of Anus, External Approach */
"0DBR0ZZ"  =  "1"  /* Excision of Anal Sphincter, Open Approach */
"0DBR3ZZ"  =  "1"  /* Excision of Anal Sphincter, Percutaneous Approach */
"0DBR4ZZ"  =  "1"  /* Excision of Anal Sphincter, Percutaneous Endoscopic Approach */
"0DBS0ZX"  =  "1"  /* Excision of Greater Omentum, Open Approach, Diagnostic */
"0DBS0ZZ"  =  "1"  /* Excision of Greater Omentum, Open Approach */
"0DBS3ZZ"  =  "1"  /* Excision of Greater Omentum, Percutaneous Approach */
"0DBS4ZZ"  =  "1"  /* Excision of Greater Omentum, Perc Endo Approach */
"0DBT0ZX"  =  "1"  /* Excision of Lesser Omentum, Open Approach, Diagnostic */
"0DBT0ZZ"  =  "1"  /* Excision of Lesser Omentum, Open Approach */
"0DBT3ZZ"  =  "1"  /* Excision of Lesser Omentum, Percutaneous Approach */
"0DBT4ZZ"  =  "1"  /* Excision of Lesser Omentum, Percutaneous Endoscopic Approach */
"0DBV0ZX"  =  "1"  /* Excision of Mesentery, Open Approach, Diagnostic */
"0DBV0ZZ"  =  "1"  /* Excision of Mesentery, Open Approach */
"0DBV3ZZ"  =  "1"  /* Excision of Mesentery, Percutaneous Approach */
"0DBV4ZZ"  =  "1"  /* Excision of Mesentery, Percutaneous Endoscopic Approach */
"0DBW0ZX"  =  "1"  /* Excision of Peritoneum, Open Approach, Diagnostic */
"0DBW0ZZ"  =  "1"  /* Excision of Peritoneum, Open Approach */
"0DBW3ZZ"  =  "1"  /* Excision of Peritoneum, Percutaneous Approach */
"0DBW4ZZ"  =  "1"  /* Excision of Peritoneum, Percutaneous Endoscopic Approach */
"0DC10ZZ"  =  "1"  /* Extirpation of Matter from Upper Esophagus, Open Approach */
"0DC13ZZ"  =  "1"  /* Extirpation of Matter from Upper Esophagus, Perc Approach */
"0DC14ZZ"  =  "1"  /* Extirpation of Matter from Up Esophag, Perc Endo Approach */
"0DC20ZZ"  =  "1"  /* Extirpation of Matter from Middle Esophagus, Open Approach */
"0DC23ZZ"  =  "1"  /* Extirpation of Matter from Middle Esophagus, Perc Approach */
"0DC24ZZ"  =  "1"  /* Extirpation of Matter from Mid Esophag, Perc Endo Approach */
"0DC30ZZ"  =  "1"  /* Extirpation of Matter from Lower Esophagus, Open Approach */
"0DC33ZZ"  =  "1"  /* Extirpation of Matter from Lower Esophagus, Perc Approach */
"0DC34ZZ"  =  "1"  /* Extirpation of Matter from Low Esophag, Perc Endo Approach */
"0DC40ZZ"  =  "1"  /* Extirpation of Matter from Esophagast Junct, Open Approach */
"0DC43ZZ"  =  "1"  /* Extirpation of Matter from Esophagast Junct, Perc Approach */
"0DC44ZZ"  =  "1"  /* Extirpate matter from Esophagast Junct, Perc Endo */
"0DC50ZZ"  =  "1"  /* Extirpation of Matter from Esophagus, Open Approach */
"0DC53ZZ"  =  "1"  /* Extirpation of Matter from Esophagus, Percutaneous Approach */
"0DC54ZZ"  =  "1"  /* Extirpation of Matter from Esophagus, Perc Endo Approach */
"0DC60ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Open Approach */
"0DC63ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Percutaneous Approach */
"0DC64ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Perc Endo Approach */
"0DC70ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Pylorus, Open Approach */
"0DC73ZZ"  =  "1"  /* Extirpation of Matter from Stomach, Pylorus, Perc Approach */
"0DC74ZZ"  =  "1"  /* Extirpate of Matter from Stomach, Pylrs, Perc Endo Approach */
"0DC80ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Open Approach */
"0DC83ZZ"  =  "1"  /* Extirpation of Matter from Small Intestine, Perc Approach */
"0DC84ZZ"  =  "1"  /* Extirpation of Matter from Small Intest, Perc Endo Approach */
"0DC90ZZ"  =  "1"  /* Extirpation of Matter from Duodenum, Open Approach */
"0DC93ZZ"  =  "1"  /* Extirpation of Matter from Duodenum, Percutaneous Approach */
"0DC94ZZ"  =  "1"  /* Extirpation of Matter from Duodenum, Perc Endo Approach */
"0DCA0ZZ"  =  "1"  /* Extirpation of Matter from Jejunum, Open Approach */
"0DCA3ZZ"  =  "1"  /* Extirpation of Matter from Jejunum, Percutaneous Approach */
"0DCA4ZZ"  =  "1"  /* Extirpation of Matter from Jejunum, Perc Endo Approach */
"0DCB0ZZ"  =  "1"  /* Extirpation of Matter from Ileum, Open Approach */
"0DCB3ZZ"  =  "1"  /* Extirpation of Matter from Ileum, Percutaneous Approach */
"0DCB4ZZ"  =  "1"  /* Extirpation of Matter from Ileum, Perc Endo Approach */
"0DCC0ZZ"  =  "1"  /* Extirpation of Matter from Ileocecal Valve, Open Approach */
"0DCC3ZZ"  =  "1"  /* Extirpation of Matter from Ileocecal Valve, Perc Approach */
"0DCC4ZZ"  =  "1"  /* Extirpate of Matter from Ileocecal Valve, Perc Endo Approach */
"0DCE0ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Open Approach */
"0DCE3ZZ"  =  "1"  /* Extirpation of Matter from Large Intestine, Perc Approach */
"0DCE4ZZ"  =  "1"  /* Extirpation of Matter from Lg Intest, Perc Endo Approach */
"0DCF0ZZ"  =  "1"  /* Extirpation of Matter from R Lg Intest, Open Approach */
"0DCF3ZZ"  =  "1"  /* Extirpation of Matter from R Lg Intest, Perc Approach */
"0DCF4ZZ"  =  "1"  /* Extirpation of Matter from R Lg Intest, Perc Endo Approach */
"0DCG0ZZ"  =  "1"  /* Extirpation of Matter from L Lg Intest, Open Approach */
"0DCG3ZZ"  =  "1"  /* Extirpation of Matter from L Lg Intest, Perc Approach */
"0DCG4ZZ"  =  "1"  /* Extirpation of Matter from L Lg Intest, Perc Endo Approach */
"0DCH0ZZ"  =  "1"  /* Extirpation of Matter from Cecum, Open Approach */
"0DCH3ZZ"  =  "1"  /* Extirpation of Matter from Cecum, Percutaneous Approach */
"0DCH4ZZ"  =  "1"  /* Extirpation of Matter from Cecum, Perc Endo Approach */
"0DCJ0ZZ"  =  "1"  /* Extirpation of Matter from Appendix, Open Approach */
"0DCJ3ZZ"  =  "1"  /* Extirpation of Matter from Appendix, Percutaneous Approach */
"0DCJ4ZZ"  =  "1"  /* Extirpation of Matter from Appendix, Perc Endo Approach */
"0DCJ7ZZ"  =  "1"  /* Extirpation of Matter from Appendix, Via Opening */
"0DCJ8ZZ"  =  "1"  /* Extirpation of Matter from Appendix, Endo */
"0DCK0ZZ"  =  "1"  /* Extirpation of Matter from Ascending Colon, Open Approach */
"0DCK3ZZ"  =  "1"  /* Extirpation of Matter from Ascending Colon, Perc Approach */
"0DCK4ZZ"  =  "1"  /* Extirpation of Matter from Asc Colon, Perc Endo Approach */
"0DCL0ZZ"  =  "1"  /* Extirpation of Matter from Transverse Colon, Open Approach */
"0DCL3ZZ"  =  "1"  /* Extirpation of Matter from Transverse Colon, Perc Approach */
"0DCL4ZZ"  =  "1"  /* Extirpation of Matter from Trans Colon, Perc Endo Approach */
"0DCM0ZZ"  =  "1"  /* Extirpation of Matter from Descending Colon, Open Approach */
"0DCM3ZZ"  =  "1"  /* Extirpation of Matter from Descending Colon, Perc Approach */
"0DCM4ZZ"  =  "1"  /* Extirpation of Matter from Descend Colon, Perc Endo Approach */
"0DCN0ZZ"  =  "1"  /* Extirpation of Matter from Sigmoid Colon, Open Approach */
"0DCN3ZZ"  =  "1"  /* Extirpation of Matter from Sigmoid Colon, Perc Approach */
"0DCN4ZZ"  =  "1"  /* Extirpation of Matter from Sigmoid Colon, Perc Endo Approach */
"0DCP0ZZ"  =  "1"  /* Extirpation of Matter from Rectum, Open Approach */
"0DCP3ZZ"  =  "1"  /* Extirpation of Matter from Rectum, Percutaneous Approach */
"0DCP4ZZ"  =  "1"  /* Extirpation of Matter from Rectum, Perc Endo Approach */
"0DCQ0ZZ"  =  "1"  /* Extirpation of Matter from Anus, Open Approach */
"0DCQ3ZZ"  =  "1"  /* Extirpation of Matter from Anus, Percutaneous Approach */
"0DCQ4ZZ"  =  "1"  /* Extirpation of Matter from Anus, Perc Endo Approach */
"0DCR0ZZ"  =  "1"  /* Extirpation of Matter from Anal Sphincter, Open Approach */
"0DCR3ZZ"  =  "1"  /* Extirpation of Matter from Anal Sphincter, Perc Approach */
"0DCR4ZZ"  =  "1"  /* Extirpate of Matter from Anal Sphincter, Perc Endo Approach */
"0DCS0ZZ"  =  "1"  /* Extirpation of Matter from Greater Omentum, Open Approach */
"0DCS3ZZ"  =  "1"  /* Extirpation of Matter from Greater Omentum, Perc Approach */
"0DCS4ZZ"  =  "1"  /* Extirpation of Matter from Great Omentum, Perc Endo Approach */
"0DCT0ZZ"  =  "1"  /* Extirpation of Matter from Lesser Omentum, Open Approach */
"0DCT3ZZ"  =  "1"  /* Extirpation of Matter from Lesser Omentum, Perc Approach */
"0DCT4ZZ"  =  "1"  /* Extirpation of Matter from Less Omentum, Perc Endo Approach */
"0DCV0ZZ"  =  "1"  /* Extirpation of Matter from Mesentery, Open Approach */
"0DCV3ZZ"  =  "1"  /* Extirpation of Matter from Mesentery, Percutaneous Approach */
"0DCV4ZZ"  =  "1"  /* Extirpation of Matter from Mesentery, Perc Endo Approach */
"0DCW0ZZ"  =  "1"  /* Extirpation of Matter from Peritoneum, Open Approach */
"0DCW3ZZ"  =  "1"  /* Extirpation of Matter from Peritoneum, Percutaneous Approach */
"0DCW4ZZ"  =  "1"  /* Extirpation of Matter from Peritoneum, Perc Endo Approach */
"0DF50ZZ"  =  "1"  /* Fragmentation in Esophagus, Open Approach */
"0DF53ZZ"  =  "1"  /* Fragmentation in Esophagus, Percutaneous Approach */
"0DF54ZZ"  =  "1"  /* Fragmentation in Esophagus, Percutaneous Endoscopic Approach */
"0DF57ZZ"  =  "1"  /* Fragmentation in Esophagus, Via Opening */
"0DF58ZZ"  =  "1"  /* Fragmentation in Esophagus, Endo */
"0DF60ZZ"  =  "1"  /* Fragmentation in Stomach, Open Approach */
"0DF63ZZ"  =  "1"  /* Fragmentation in Stomach, Percutaneous Approach */
"0DF64ZZ"  =  "1"  /* Fragmentation in Stomach, Percutaneous Endoscopic Approach */
"0DF67ZZ"  =  "1"  /* Fragmentation in Stomach, Via Natural or Artificial Opening */
"0DF68ZZ"  =  "1"  /* Fragmentation in Stomach, Endo */
"0DF80ZZ"  =  "1"  /* Fragmentation in Small Intestine, Open Approach */
"0DF83ZZ"  =  "1"  /* Fragmentation in Small Intestine, Percutaneous Approach */
"0DF84ZZ"  =  "1"  /* Fragmentation in Small Intestine, Perc Endo Approach */
"0DF87ZZ"  =  "1"  /* Fragmentation in Small Intestine, Via Opening */
"0DF88ZZ"  =  "1"  /* Fragmentation in Small Intestine, Endo */
"0DF90ZZ"  =  "1"  /* Fragmentation in Duodenum, Open Approach */
"0DF93ZZ"  =  "1"  /* Fragmentation in Duodenum, Percutaneous Approach */
"0DF94ZZ"  =  "1"  /* Fragmentation in Duodenum, Percutaneous Endoscopic Approach */
"0DF97ZZ"  =  "1"  /* Fragmentation in Duodenum, Via Natural or Artificial Opening */
"0DF98ZZ"  =  "1"  /* Fragmentation in Duodenum, Endo */
"0DFA0ZZ"  =  "1"  /* Fragmentation in Jejunum, Open Approach */
"0DFA3ZZ"  =  "1"  /* Fragmentation in Jejunum, Percutaneous Approach */
"0DFA4ZZ"  =  "1"  /* Fragmentation in Jejunum, Percutaneous Endoscopic Approach */
"0DFA7ZZ"  =  "1"  /* Fragmentation in Jejunum, Via Natural or Artificial Opening */
"0DFA8ZZ"  =  "1"  /* Fragmentation in Jejunum, Endo */
"0DFB0ZZ"  =  "1"  /* Fragmentation in Ileum, Open Approach */
"0DFB3ZZ"  =  "1"  /* Fragmentation in Ileum, Percutaneous Approach */
"0DFB4ZZ"  =  "1"  /* Fragmentation in Ileum, Percutaneous Endoscopic Approach */
"0DFB7ZZ"  =  "1"  /* Fragmentation in Ileum, Via Natural or Artificial Opening */
"0DFB8ZZ"  =  "1"  /* Fragmentation in Ileum, Endo */
"0DFE0ZZ"  =  "1"  /* Fragmentation in Large Intestine, Open Approach */
"0DFE3ZZ"  =  "1"  /* Fragmentation in Large Intestine, Percutaneous Approach */
"0DFE4ZZ"  =  "1"  /* Fragmentation in Large Intestine, Perc Endo Approach */
"0DFE7ZZ"  =  "1"  /* Fragmentation in Large Intestine, Via Opening */
"0DFE8ZZ"  =  "1"  /* Fragmentation in Large Intestine, Endo */
"0DFF0ZZ"  =  "1"  /* Fragmentation in Right Large Intestine, Open Approach */
"0DFF3ZZ"  =  "1"  /* Fragmentation in Right Large Intestine, Perc Approach */
"0DFF4ZZ"  =  "1"  /* Fragmentation in Right Large Intestine, Perc Endo Approach */
"0DFF7ZZ"  =  "1"  /* Fragmentation in Right Large Intestine, Via Opening */
"0DFF8ZZ"  =  "1"  /* Fragmentation in Right Large Intestine, Endo */
"0DFG0ZZ"  =  "1"  /* Fragmentation in Left Large Intestine, Open Approach */
"0DFG3ZZ"  =  "1"  /* Fragmentation in Left Large Intestine, Percutaneous Approach */
"0DFG4ZZ"  =  "1"  /* Fragmentation in Left Large Intestine, Perc Endo Approach */
"0DFG7ZZ"  =  "1"  /* Fragmentation in Left Large Intestine, Via Opening */
"0DFG8ZZ"  =  "1"  /* Fragmentation in Left Large Intestine, Endo */
"0DFH0ZZ"  =  "1"  /* Fragmentation in Cecum, Open Approach */
"0DFH3ZZ"  =  "1"  /* Fragmentation in Cecum, Percutaneous Approach */
"0DFH4ZZ"  =  "1"  /* Fragmentation in Cecum, Percutaneous Endoscopic Approach */
"0DFH7ZZ"  =  "1"  /* Fragmentation in Cecum, Via Natural or Artificial Opening */
"0DFH8ZZ"  =  "1"  /* Fragmentation in Cecum, Endo */
"0DFJ0ZZ"  =  "1"  /* Fragmentation in Appendix, Open Approach */
"0DFJ3ZZ"  =  "1"  /* Fragmentation in Appendix, Percutaneous Approach */
"0DFJ4ZZ"  =  "1"  /* Fragmentation in Appendix, Percutaneous Endoscopic Approach */
"0DFJ7ZZ"  =  "1"  /* Fragmentation in Appendix, Via Natural or Artificial Opening */
"0DFJ8ZZ"  =  "1"  /* Fragmentation in Appendix, Endo */
"0DFK0ZZ"  =  "1"  /* Fragmentation in Ascending Colon, Open Approach */
"0DFK3ZZ"  =  "1"  /* Fragmentation in Ascending Colon, Percutaneous Approach */
"0DFK4ZZ"  =  "1"  /* Fragmentation in Ascending Colon, Perc Endo Approach */
"0DFK7ZZ"  =  "1"  /* Fragmentation in Ascending Colon, Via Opening */
"0DFK8ZZ"  =  "1"  /* Fragmentation in Ascending Colon, Endo */
"0DFL0ZZ"  =  "1"  /* Fragmentation in Transverse Colon, Open Approach */
"0DFL3ZZ"  =  "1"  /* Fragmentation in Transverse Colon, Percutaneous Approach */
"0DFL4ZZ"  =  "1"  /* Fragmentation in Transverse Colon, Perc Endo Approach */
"0DFL7ZZ"  =  "1"  /* Fragmentation in Transverse Colon, Via Opening */
"0DFL8ZZ"  =  "1"  /* Fragmentation in Transverse Colon, Endo */
"0DFM0ZZ"  =  "1"  /* Fragmentation in Descending Colon, Open Approach */
"0DFM3ZZ"  =  "1"  /* Fragmentation in Descending Colon, Percutaneous Approach */
"0DFM4ZZ"  =  "1"  /* Fragmentation in Descending Colon, Perc Endo Approach */
"0DFM7ZZ"  =  "1"  /* Fragmentation in Descending Colon, Via Opening */
"0DFM8ZZ"  =  "1"  /* Fragmentation in Descending Colon, Endo */
"0DFN0ZZ"  =  "1"  /* Fragmentation in Sigmoid Colon, Open Approach */
"0DFN3ZZ"  =  "1"  /* Fragmentation in Sigmoid Colon, Percutaneous Approach */
"0DFN4ZZ"  =  "1"  /* Fragmentation in Sigmoid Colon, Perc Endo Approach */
"0DFN7ZZ"  =  "1"  /* Fragmentation in Sigmoid Colon, Via Opening */
"0DFN8ZZ"  =  "1"  /* Fragmentation in Sigmoid Colon, Endo */
"0DFP0ZZ"  =  "1"  /* Fragmentation in Rectum, Open Approach */
"0DFP3ZZ"  =  "1"  /* Fragmentation in Rectum, Percutaneous Approach */
"0DFP4ZZ"  =  "1"  /* Fragmentation in Rectum, Percutaneous Endoscopic Approach */
"0DFP7ZZ"  =  "1"  /* Fragmentation in Rectum, Via Natural or Artificial Opening */
"0DFP8ZZ"  =  "1"  /* Fragmentation in Rectum, Endo */
"0DFQ0ZZ"  =  "1"  /* Fragmentation in Anus, Open Approach */
"0DFQ3ZZ"  =  "1"  /* Fragmentation in Anus, Percutaneous Approach */
"0DFQ4ZZ"  =  "1"  /* Fragmentation in Anus, Percutaneous Endoscopic Approach */
"0DFQ7ZZ"  =  "1"  /* Fragmentation in Anus, Via Natural or Artificial Opening */
"0DFQ8ZZ"  =  "1"  /* Fragmentation in Anus, Endo */
"0DH501Z"  =  "1"  /* Insertion of Radioactive Element into Esophag, Open Approach */
"0DH502Z"  =  "1"  /* Insertion of Monitoring Device into Esophagus, Open Approach */
"0DH503Z"  =  "1"  /* Insertion of Infusion Device into Esophagus, Open Approach */
"0DH531Z"  =  "1"  /* Insertion of Radioactive Element into Esophag, Perc Approach */
"0DH532Z"  =  "1"  /* Insertion of Monitoring Device into Esophagus, Perc Approach */
"0DH533Z"  =  "1"  /* Insertion of Infusion Device into Esophagus, Perc Approach */
"0DH541Z"  =  "1"  /* Insertion of Radioact Elem into Esophag, Perc Endo Approach */
"0DH542Z"  =  "1"  /* Insertion of Monitor Dev into Esophag, Perc Endo Approach */
"0DH543Z"  =  "1"  /* Insertion of Infusion Dev into Esophag, Perc Endo Approach */
"0DH571Z"  =  "1"  /* Insertion of Radioactive Element into Esophagus, Via Opening */
"0DH572Z"  =  "1"  /* Insertion of Monitoring Device into Esophagus, Via Opening */
"0DH573Z"  =  "1"  /* Insertion of Infusion Device into Esophagus, Via Opening */
"0DH581Z"  =  "1"  /* Insertion of Radioactive Element into Esophagus, Endo */
"0DH582Z"  =  "1"  /* Insertion of Monitoring Device into Esophagus, Endo */
"0DH583Z"  =  "1"  /* Insertion of Infusion Device into Esophagus, Endo */
"0DH602Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Open Approach */
"0DH603Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Open Approach */
"0DH60DZ"  =  "1"  /* Insertion of Intraluminal Device into Stomach, Open Approach */
"0DH60MZ"  =  "1"  /* Insertion of Stimulator Lead into Stomach, Open Approach */
"0DH60UZ"  =  "1"  /* Insertion of Feeding Device into Stomach, Open Approach */
"0DH632Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Perc Approach */
"0DH633Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Perc Approach */
"0DH63DZ"  =  "1"  /* Insertion of Intraluminal Device into Stomach, Perc Approach */
"0DH63MZ"  =  "1"  /* Insertion of Stimulator Lead into Stomach, Perc Approach */
"0DH642Z"  =  "1"  /* Insertion of Monitor Dev into Stomach, Perc Endo Approach */
"0DH643Z"  =  "1"  /* Insertion of Infusion Dev into Stomach, Perc Endo Approach */
"0DH64DZ"  =  "1"  /* Insertion of Intralum Dev into Stomach, Perc Endo Approach */
"0DH64MZ"  =  "1"  /* Insertion of Stim Lead into Stomach, Perc Endo Approach */
"0DH672Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Via Opening */
"0DH673Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Via Opening */
"0DH67DZ"  =  "1"  /* Insertion of Intraluminal Device into Stomach, Via Opening */
"0DH67UZ"  =  "1"  /* Insertion of Feeding Device into Stomach, Via Opening */
"0DH682Z"  =  "1"  /* Insertion of Monitoring Device into Stomach, Endo */
"0DH683Z"  =  "1"  /* Insertion of Infusion Device into Stomach, Endo */
"0DH68DZ"  =  "1"  /* Insertion of Intraluminal Device into Stomach, Endo */
"0DH68UZ"  =  "1"  /* Insertion of Feeding Device into Stomach, Endo */
"0DH802Z"  =  "1"  /* Insertion of Monitor Dev into Small Intest, Open Approach */
"0DH803Z"  =  "1"  /* Insertion of Infusion Dev into Small Intest, Open Approach */
"0DH832Z"  =  "1"  /* Insertion of Monitor Dev into Small Intest, Perc Approach */
"0DH833Z"  =  "1"  /* Insertion of Infusion Dev into Small Intest, Perc Approach */
"0DH842Z"  =  "1"  /* Insert of Monitor Dev into Small Intest, Perc Endo Approach */
"0DH843Z"  =  "1"  /* Insert of Infusion Dev into Small Intest, Perc Endo Approach */
"0DH872Z"  =  "1"  /* Insertion of Monitor Dev into Small Intest, Via Opening */
"0DH873Z"  =  "1"  /* Insertion of Infusion Device into Small Intest, Via Opening */
"0DH882Z"  =  "1"  /* Insertion of Monitoring Device into Small Intestine, Endo */
"0DH883Z"  =  "1"  /* Insertion of Infusion Device into Small Intestine, Endo */
"0DH902Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Open Approach */
"0DH903Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Open Approach */
"0DH932Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Perc Approach */
"0DH933Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Perc Approach */
"0DH942Z"  =  "1"  /* Insertion of Monitor Dev into Duodenum, Perc Endo Approach */
"0DH943Z"  =  "1"  /* Insertion of Infusion Dev into Duodenum, Perc Endo Approach */
"0DH972Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Via Opening */
"0DH973Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Via Opening */
"0DH982Z"  =  "1"  /* Insertion of Monitoring Device into Duodenum, Endo */
"0DH983Z"  =  "1"  /* Insertion of Infusion Device into Duodenum, Endo */
"0DHA02Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Open Approach */
"0DHA03Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Open Approach */
"0DHA32Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Perc Approach */
"0DHA33Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Perc Approach */
"0DHA42Z"  =  "1"  /* Insertion of Monitor Dev into Jejunum, Perc Endo Approach */
"0DHA43Z"  =  "1"  /* Insertion of Infusion Dev into Jejunum, Perc Endo Approach */
"0DHA72Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Via Opening */
"0DHA73Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Via Opening */
"0DHA82Z"  =  "1"  /* Insertion of Monitoring Device into Jejunum, Endo */
"0DHA83Z"  =  "1"  /* Insertion of Infusion Device into Jejunum, Endo */
"0DHB02Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Open Approach */
"0DHB03Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Open Approach */
"0DHB32Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Perc Approach */
"0DHB33Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Perc Approach */
"0DHB42Z"  =  "1"  /* Insertion of Monitor Dev into Ileum, Perc Endo Approach */
"0DHB43Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Perc Endo Approach */
"0DHB72Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Via Opening */
"0DHB73Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Via Opening */
"0DHB82Z"  =  "1"  /* Insertion of Monitoring Device into Ileum, Endo */
"0DHB83Z"  =  "1"  /* Insertion of Infusion Device into Ileum, Endo */
"0DHP01Z"  =  "1"  /* Insertion of Radioactive Element into Rectum, Open Approach */
"0DHP31Z"  =  "1"  /* Insertion of Radioactive Element into Rectum, Perc Approach */
"0DHP41Z"  =  "1"  /* Insertion of Radioact Elem into Rectum, Perc Endo Approach */
"0DHP71Z"  =  "1"  /* Insertion of Radioactive Element into Rectum, Via Opening */
"0DHP81Z"  =  "1"  /* Insertion of Radioactive Element into Rectum, Endo */
"0DHQ0DZ"  =  "1"  /* Insertion of Intraluminal Device into Anus, Open Approach */
"0DHQ0LZ"  =  "1"  /* Insertion of Artificial Sphincter into Anus, Open Approach */
"0DHQ3DZ"  =  "1"  /* Insertion of Intraluminal Device into Anus, Perc Approach */
"0DHQ3LZ"  =  "1"  /* Insertion of Artificial Sphincter into Anus, Perc Approach */
"0DHQ4DZ"  =  "1"  /* Insertion of Intralum Dev into Anus, Perc Endo Approach */
"0DHQ4LZ"  =  "1"  /* Insertion of Artif Sphincter into Anus, Perc Endo Approach */
"0DHQ7DZ"  =  "1"  /* Insertion of Intraluminal Device into Anus, Via Opening */
"0DHQ8DZ"  =  "1"  /* Insertion of Intraluminal Device into Anus, Endo */
"0DHR0MZ"  =  "1"  /* Insertion of Stim Lead into Anal Sphincter, Open Approach */
"0DHR3MZ"  =  "1"  /* Insertion of Stim Lead into Anal Sphincter, Perc Approach */
"0DHR4MZ"  =  "1"  /* Insert of Stim Lead into Anal Sphincter, Perc Endo Approach */
"0DJ00ZZ"  =  "1"  /* Inspection of Upper Intestinal Tract, Open Approach */
"0DJ04ZZ"  =  "1"  /* Inspection of Upper Intestinal Tract, Perc Endo Approach */
"0DJ60ZZ"  =  "1"  /* Inspection of Stomach, Open Approach */
"0DJ64ZZ"  =  "1"  /* Inspection of Stomach, Percutaneous Endoscopic Approach */
"0DJD0ZZ"  =  "1"  /* Inspection of Lower Intestinal Tract, Open Approach */
"0DJD4ZZ"  =  "1"  /* Inspection of Lower Intestinal Tract, Perc Endo Approach */
"0DJU0ZZ"  =  "1"  /* Inspection of Omentum, Open Approach */
"0DJU3ZZ"  =  "1"  /* Inspection of Omentum, Percutaneous Approach */
"0DJU4ZZ"  =  "1"  /* Inspection of Omentum, Percutaneous Endoscopic Approach */
"0DJV0ZZ"  =  "1"  /* Inspection of Mesentery, Open Approach */
"0DJV3ZZ"  =  "1"  /* Inspection of Mesentery, Percutaneous Approach */
"0DJV4ZZ"  =  "1"  /* Inspection of Mesentery, Percutaneous Endoscopic Approach */
"0DJW0ZZ"  =  "1"  /* Inspection of Peritoneum, Open Approach */
"0DJW3ZZ"  =  "1"  /* Inspection of Peritoneum, Percutaneous Approach */
"0DJW4ZZ"  =  "1"  /* Inspection of Peritoneum, Percutaneous Endoscopic Approach */
"0DL60CZ"  =  "1"  /* Occlusion of Stomach with Extraluminal Device, Open Approach */
"0DL60DZ"  =  "1"  /* Occlusion of Stomach with Intraluminal Device, Open Approach */
"0DL60ZZ"  =  "1"  /* Occlusion of Stomach, Open Approach */
"0DL63CZ"  =  "1"  /* Occlusion of Stomach with Extraluminal Device, Perc Approach */
"0DL63DZ"  =  "1"  /* Occlusion of Stomach with Intraluminal Device, Perc Approach */
"0DL63ZZ"  =  "1"  /* Occlusion of Stomach, Percutaneous Approach */
"0DL64CZ"  =  "1"  /* Occlusion of Stomach with Extralum Dev, Perc Endo Approach */
"0DL64DZ"  =  "1"  /* Occlusion of Stomach with Intralum Dev, Perc Endo Approach */
"0DL64ZZ"  =  "1"  /* Occlusion of Stomach, Percutaneous Endoscopic Approach */
"0DL67DZ"  =  "1"  /* Occlusion of Stomach with Intraluminal Device, Via Opening */
"0DL67ZZ"  =  "1"  /* Occlusion of Stomach, Via Natural or Artificial Opening */
"0DL68DZ"  =  "1"  /* Occlusion of Stomach with Intraluminal Device, Endo */
"0DL68ZZ"  =  "1"  /* Occlusion of Stomach, Endo */
"0DL70CZ"  =  "1"  /* Occlusion of Stomach, Pylrs with Extralum Dev, Open Approach */
"0DL70DZ"  =  "1"  /* Occlusion of Stomach, Pylrs with Intralum Dev, Open Approach */
"0DL70ZZ"  =  "1"  /* Occlusion of Stomach, Pylorus, Open Approach */
"0DL73CZ"  =  "1"  /* Occlusion of Stomach, Pylrs with Extralum Dev, Perc Approach */
"0DL73DZ"  =  "1"  /* Occlusion of Stomach, Pylrs with Intralum Dev, Perc Approach */
"0DL73ZZ"  =  "1"  /* Occlusion of Stomach, Pylorus, Percutaneous Approach */
"0DL74CZ"  =  "1"  /* Occlusion Stomach, Pylrs w Extralum Dev, Perc Endo */
"0DL74DZ"  =  "1"  /* Occlusion Stomach, Pylrs w Intralum Dev, Perc Endo */
"0DL74ZZ"  =  "1"  /* Occlusion of Stomach, Pylorus, Perc Endo Approach */
"0DL77DZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Intralum Dev, Via Opening */
"0DL77ZZ"  =  "1"  /* Occlusion of Stomach, Pylorus, Via Opening */
"0DL78DZ"  =  "1"  /* Occlusion of Stomach, Pylorus with Intraluminal Device, Endo */
"0DL78ZZ"  =  "1"  /* Occlusion of Stomach, Pylorus, Endo */
"0DL80CZ"  =  "1"  /* Occlusion of Small Intest with Extralum Dev, Open Approach */
"0DL80DZ"  =  "1"  /* Occlusion of Small Intest with Intralum Dev, Open Approach */
"0DL80ZZ"  =  "1"  /* Occlusion of Small Intestine, Open Approach */
"0DL83CZ"  =  "1"  /* Occlusion of Small Intest with Extralum Dev, Perc Approach */
"0DL83DZ"  =  "1"  /* Occlusion of Small Intest with Intralum Dev, Perc Approach */
"0DL83ZZ"  =  "1"  /* Occlusion of Small Intestine, Percutaneous Approach */
"0DL84CZ"  =  "1"  /* Occlusion Small Intest w Extralum Dev, Perc Endo */
"0DL84DZ"  =  "1"  /* Occlusion Small Intest w Intralum Dev, Perc Endo */
"0DL84ZZ"  =  "1"  /* Occlusion of Small Intestine, Perc Endo Approach */
"0DL87DZ"  =  "1"  /* Occlusion of Small Intestine with Intralum Dev, Via Opening */
"0DL87ZZ"  =  "1"  /* Occlusion of Small Intestine, Via Opening */
"0DL88DZ"  =  "1"  /* Occlusion of Small Intestine with Intraluminal Device, Endo */
"0DL88ZZ"  =  "1"  /* Occlusion of Small Intestine, Endo */
"0DL90CZ"  =  "1"  /* Occlusion of Duodenum with Extralum Dev, Open Approach */
"0DL90DZ"  =  "1"  /* Occlusion of Duodenum with Intralum Dev, Open Approach */
"0DL90ZZ"  =  "1"  /* Occlusion of Duodenum, Open Approach */
"0DL93CZ"  =  "1"  /* Occlusion of Duodenum with Extralum Dev, Perc Approach */
"0DL93DZ"  =  "1"  /* Occlusion of Duodenum with Intralum Dev, Perc Approach */
"0DL93ZZ"  =  "1"  /* Occlusion of Duodenum, Percutaneous Approach */
"0DL94CZ"  =  "1"  /* Occlusion of Duodenum with Extralum Dev, Perc Endo Approach */
"0DL94DZ"  =  "1"  /* Occlusion of Duodenum with Intralum Dev, Perc Endo Approach */
"0DL94ZZ"  =  "1"  /* Occlusion of Duodenum, Percutaneous Endoscopic Approach */
"0DL97DZ"  =  "1"  /* Occlusion of Duodenum with Intraluminal Device, Via Opening */
"0DL97ZZ"  =  "1"  /* Occlusion of Duodenum, Via Natural or Artificial Opening */
"0DL98DZ"  =  "1"  /* Occlusion of Duodenum with Intraluminal Device, Endo */
"0DL98ZZ"  =  "1"  /* Occlusion of Duodenum, Endo */
"0DLA0CZ"  =  "1"  /* Occlusion of Jejunum with Extraluminal Device, Open Approach */
"0DLA0DZ"  =  "1"  /* Occlusion of Jejunum with Intraluminal Device, Open Approach */
"0DLA0ZZ"  =  "1"  /* Occlusion of Jejunum, Open Approach */
"0DLA3CZ"  =  "1"  /* Occlusion of Jejunum with Extraluminal Device, Perc Approach */
"0DLA3DZ"  =  "1"  /* Occlusion of Jejunum with Intraluminal Device, Perc Approach */
"0DLA3ZZ"  =  "1"  /* Occlusion of Jejunum, Percutaneous Approach */
"0DLA4CZ"  =  "1"  /* Occlusion of Jejunum with Extralum Dev, Perc Endo Approach */
"0DLA4DZ"  =  "1"  /* Occlusion of Jejunum with Intralum Dev, Perc Endo Approach */
"0DLA4ZZ"  =  "1"  /* Occlusion of Jejunum, Percutaneous Endoscopic Approach */
"0DLA7DZ"  =  "1"  /* Occlusion of Jejunum with Intraluminal Device, Via Opening */
"0DLA7ZZ"  =  "1"  /* Occlusion of Jejunum, Via Natural or Artificial Opening */
"0DLA8DZ"  =  "1"  /* Occlusion of Jejunum with Intraluminal Device, Endo */
"0DLA8ZZ"  =  "1"  /* Occlusion of Jejunum, Endo */
"0DLB0CZ"  =  "1"  /* Occlusion of Ileum with Extraluminal Device, Open Approach */
"0DLB0DZ"  =  "1"  /* Occlusion of Ileum with Intraluminal Device, Open Approach */
"0DLB0ZZ"  =  "1"  /* Occlusion of Ileum, Open Approach */
"0DLB3CZ"  =  "1"  /* Occlusion of Ileum with Extraluminal Device, Perc Approach */
"0DLB3DZ"  =  "1"  /* Occlusion of Ileum with Intraluminal Device, Perc Approach */
"0DLB3ZZ"  =  "1"  /* Occlusion of Ileum, Percutaneous Approach */
"0DLB4CZ"  =  "1"  /* Occlusion of Ileum with Extralum Dev, Perc Endo Approach */
"0DLB4DZ"  =  "1"  /* Occlusion of Ileum with Intralum Dev, Perc Endo Approach */
"0DLB4ZZ"  =  "1"  /* Occlusion of Ileum, Percutaneous Endoscopic Approach */
"0DLB7DZ"  =  "1"  /* Occlusion of Ileum with Intraluminal Device, Via Opening */
"0DLB7ZZ"  =  "1"  /* Occlusion of Ileum, Via Natural or Artificial Opening */
"0DLB8DZ"  =  "1"  /* Occlusion of Ileum with Intraluminal Device, Endo */
"0DLB8ZZ"  =  "1"  /* Occlusion of Ileum, Endo */
"0DLC0CZ"  =  "1"  /* Occlusion Ileocecal Valve w Extralum Dev, Open */
"0DLC0DZ"  =  "1"  /* Occlusion Ileocecal Valve w Intralum Dev, Open */
"0DLC0ZZ"  =  "1"  /* Occlusion of Ileocecal Valve, Open Approach */
"0DLC3CZ"  =  "1"  /* Occlusion Ileocecal Valve w Extralum Dev, Perc */
"0DLC3DZ"  =  "1"  /* Occlusion Ileocecal Valve w Intralum Dev, Perc */
"0DLC3ZZ"  =  "1"  /* Occlusion of Ileocecal Valve, Percutaneous Approach */
"0DLC4CZ"  =  "1"  /* Occlusion Ileocecal Valve w Extralum Dev, Perc Endo */
"0DLC4DZ"  =  "1"  /* Occlusion Ileocecal Valve w Intralum Dev, Perc Endo */
"0DLC4ZZ"  =  "1"  /* Occlusion of Ileocecal Valve, Perc Endo Approach */
"0DLC7DZ"  =  "1"  /* Occlusion of Ileocecal Valve with Intralum Dev, Via Opening */
"0DLC7ZZ"  =  "1"  /* Occlusion of Ileocecal Valve, Via Opening */
"0DLC8DZ"  =  "1"  /* Occlusion of Ileocecal Valve with Intraluminal Device, Endo */
"0DLC8ZZ"  =  "1"  /* Occlusion of Ileocecal Valve, Endo */
"0DLE0CZ"  =  "1"  /* Occlusion of Lg Intest with Extralum Dev, Open Approach */
"0DLE0DZ"  =  "1"  /* Occlusion of Lg Intest with Intralum Dev, Open Approach */
"0DLE0ZZ"  =  "1"  /* Occlusion of Large Intestine, Open Approach */
"0DLE3CZ"  =  "1"  /* Occlusion of Lg Intest with Extralum Dev, Perc Approach */
"0DLE3DZ"  =  "1"  /* Occlusion of Lg Intest with Intralum Dev, Perc Approach */
"0DLE3ZZ"  =  "1"  /* Occlusion of Large Intestine, Percutaneous Approach */
"0DLE4CZ"  =  "1"  /* Occlusion of Lg Intest with Extralum Dev, Perc Endo Approach */
"0DLE4DZ"  =  "1"  /* Occlusion of Lg Intest with Intralum Dev, Perc Endo Approach */
"0DLE4ZZ"  =  "1"  /* Occlusion of Large Intestine, Perc Endo Approach */
"0DLE7DZ"  =  "1"  /* Occlusion of Large Intestine with Intralum Dev, Via Opening */
"0DLE7ZZ"  =  "1"  /* Occlusion of Large Intestine, Via Opening */
"0DLE8DZ"  =  "1"  /* Occlusion of Large Intestine with Intraluminal Device, Endo */
"0DLE8ZZ"  =  "1"  /* Occlusion of Large Intestine, Endo */
"0DLF0CZ"  =  "1"  /* Occlusion of R Lg Intest with Extralum Dev, Open Approach */
"0DLF0DZ"  =  "1"  /* Occlusion of R Lg Intest with Intralum Dev, Open Approach */
"0DLF0ZZ"  =  "1"  /* Occlusion of Right Large Intestine, Open Approach */
"0DLF3CZ"  =  "1"  /* Occlusion of R Lg Intest with Extralum Dev, Perc Approach */
"0DLF3DZ"  =  "1"  /* Occlusion of R Lg Intest with Intralum Dev, Perc Approach */
"0DLF3ZZ"  =  "1"  /* Occlusion of Right Large Intestine, Percutaneous Approach */
"0DLF4CZ"  =  "1"  /* Occlusion R Lg Intest w Extralum Dev, Perc Endo */
"0DLF4DZ"  =  "1"  /* Occlusion R Lg Intest w Intralum Dev, Perc Endo */
"0DLF4ZZ"  =  "1"  /* Occlusion of Right Large Intestine, Perc Endo Approach */
"0DLF7DZ"  =  "1"  /* Occlusion of R Lg Intest with Intralum Dev, Via Opening */
"0DLF7ZZ"  =  "1"  /* Occlusion of Right Large Intestine, Via Opening */
"0DLF8DZ"  =  "1"  /* Occlusion of Right Large Intestine with Intralum Dev, Endo */
"0DLF8ZZ"  =  "1"  /* Occlusion of Right Large Intestine, Endo */
"0DLG0CZ"  =  "1"  /* Occlusion of L Lg Intest with Extralum Dev, Open Approach */
"0DLG0DZ"  =  "1"  /* Occlusion of L Lg Intest with Intralum Dev, Open Approach */
"0DLG0ZZ"  =  "1"  /* Occlusion of Left Large Intestine, Open Approach */
"0DLG3CZ"  =  "1"  /* Occlusion of L Lg Intest with Extralum Dev, Perc Approach */
"0DLG3DZ"  =  "1"  /* Occlusion of L Lg Intest with Intralum Dev, Perc Approach */
"0DLG3ZZ"  =  "1"  /* Occlusion of Left Large Intestine, Percutaneous Approach */
"0DLG4CZ"  =  "1"  /* Occlusion L Lg Intest w Extralum Dev, Perc Endo */
"0DLG4DZ"  =  "1"  /* Occlusion L Lg Intest w Intralum Dev, Perc Endo */
"0DLG4ZZ"  =  "1"  /* Occlusion of Left Large Intestine, Perc Endo Approach */
"0DLG7DZ"  =  "1"  /* Occlusion of L Lg Intest with Intralum Dev, Via Opening */
"0DLG7ZZ"  =  "1"  /* Occlusion of Left Large Intestine, Via Opening */
"0DLG8DZ"  =  "1"  /* Occlusion of Left Large Intestine with Intralum Dev, Endo */
"0DLG8ZZ"  =  "1"  /* Occlusion of Left Large Intestine, Endo */
"0DLH0CZ"  =  "1"  /* Occlusion of Cecum with Extraluminal Device, Open Approach */
"0DLH0DZ"  =  "1"  /* Occlusion of Cecum with Intraluminal Device, Open Approach */
"0DLH0ZZ"  =  "1"  /* Occlusion of Cecum, Open Approach */
"0DLH3CZ"  =  "1"  /* Occlusion of Cecum with Extraluminal Device, Perc Approach */
"0DLH3DZ"  =  "1"  /* Occlusion of Cecum with Intraluminal Device, Perc Approach */
"0DLH3ZZ"  =  "1"  /* Occlusion of Cecum, Percutaneous Approach */
"0DLH4CZ"  =  "1"  /* Occlusion of Cecum with Extralum Dev, Perc Endo Approach */
"0DLH4DZ"  =  "1"  /* Occlusion of Cecum with Intralum Dev, Perc Endo Approach */
"0DLH4ZZ"  =  "1"  /* Occlusion of Cecum, Percutaneous Endoscopic Approach */
"0DLH7DZ"  =  "1"  /* Occlusion of Cecum with Intraluminal Device, Via Opening */
"0DLH7ZZ"  =  "1"  /* Occlusion of Cecum, Via Natural or Artificial Opening */
"0DLH8DZ"  =  "1"  /* Occlusion of Cecum with Intraluminal Device, Endo */
"0DLH8ZZ"  =  "1"  /* Occlusion of Cecum, Endo */
"0DLK0CZ"  =  "1"  /* Occlusion of Asc Colon with Extralum Dev, Open Approach */
"0DLK0DZ"  =  "1"  /* Occlusion of Asc Colon with Intralum Dev, Open Approach */
"0DLK0ZZ"  =  "1"  /* Occlusion of Ascending Colon, Open Approach */
"0DLK3CZ"  =  "1"  /* Occlusion of Asc Colon with Extralum Dev, Perc Approach */
"0DLK3DZ"  =  "1"  /* Occlusion of Asc Colon with Intralum Dev, Perc Approach */
"0DLK3ZZ"  =  "1"  /* Occlusion of Ascending Colon, Percutaneous Approach */
"0DLK4CZ"  =  "1"  /* Occlusion of Asc Colon with Extralum Dev, Perc Endo Approach */
"0DLK4DZ"  =  "1"  /* Occlusion of Asc Colon with Intralum Dev, Perc Endo Approach */
"0DLK4ZZ"  =  "1"  /* Occlusion of Ascending Colon, Perc Endo Approach */
"0DLK7DZ"  =  "1"  /* Occlusion of Ascending Colon with Intralum Dev, Via Opening */
"0DLK7ZZ"  =  "1"  /* Occlusion of Ascending Colon, Via Opening */
"0DLK8DZ"  =  "1"  /* Occlusion of Ascending Colon with Intraluminal Device, Endo */
"0DLK8ZZ"  =  "1"  /* Occlusion of Ascending Colon, Endo */
"0DLL0CZ"  =  "1"  /* Occlusion of Trans Colon with Extralum Dev, Open Approach */
"0DLL0DZ"  =  "1"  /* Occlusion of Trans Colon with Intralum Dev, Open Approach */
"0DLL0ZZ"  =  "1"  /* Occlusion of Transverse Colon, Open Approach */
"0DLL3CZ"  =  "1"  /* Occlusion of Trans Colon with Extralum Dev, Perc Approach */
"0DLL3DZ"  =  "1"  /* Occlusion of Trans Colon with Intralum Dev, Perc Approach */
"0DLL3ZZ"  =  "1"  /* Occlusion of Transverse Colon, Percutaneous Approach */
"0DLL4CZ"  =  "1"  /* Occlusion Trans Colon w Extralum Dev, Perc Endo */
"0DLL4DZ"  =  "1"  /* Occlusion Trans Colon w Intralum Dev, Perc Endo */
"0DLL4ZZ"  =  "1"  /* Occlusion of Transverse Colon, Perc Endo Approach */
"0DLL7DZ"  =  "1"  /* Occlusion of Transverse Colon with Intralum Dev, Via Opening */
"0DLL7ZZ"  =  "1"  /* Occlusion of Transverse Colon, Via Opening */
"0DLL8DZ"  =  "1"  /* Occlusion of Transverse Colon with Intraluminal Device, Endo */
"0DLL8ZZ"  =  "1"  /* Occlusion of Transverse Colon, Endo */
"0DLM0CZ"  =  "1"  /* Occlusion of Descend Colon with Extralum Dev, Open Approach */
"0DLM0DZ"  =  "1"  /* Occlusion of Descend Colon with Intralum Dev, Open Approach */
"0DLM0ZZ"  =  "1"  /* Occlusion of Descending Colon, Open Approach */
"0DLM3CZ"  =  "1"  /* Occlusion of Descend Colon with Extralum Dev, Perc Approach */
"0DLM3DZ"  =  "1"  /* Occlusion of Descend Colon with Intralum Dev, Perc Approach */
"0DLM3ZZ"  =  "1"  /* Occlusion of Descending Colon, Percutaneous Approach */
"0DLM4CZ"  =  "1"  /* Occlusion Descend Colon w Extralum Dev, Perc Endo */
"0DLM4DZ"  =  "1"  /* Occlusion Descend Colon w Intralum Dev, Perc Endo */
"0DLM4ZZ"  =  "1"  /* Occlusion of Descending Colon, Perc Endo Approach */
"0DLM7DZ"  =  "1"  /* Occlusion of Descending Colon with Intralum Dev, Via Opening */
"0DLM7ZZ"  =  "1"  /* Occlusion of Descending Colon, Via Opening */
"0DLM8DZ"  =  "1"  /* Occlusion of Descending Colon with Intraluminal Device, Endo */
"0DLM8ZZ"  =  "1"  /* Occlusion of Descending Colon, Endo */
"0DLN0CZ"  =  "1"  /* Occlusion of Sigmoid Colon with Extralum Dev, Open Approach */
"0DLN0DZ"  =  "1"  /* Occlusion of Sigmoid Colon with Intralum Dev, Open Approach */
"0DLN0ZZ"  =  "1"  /* Occlusion of Sigmoid Colon, Open Approach */
"0DLN3CZ"  =  "1"  /* Occlusion of Sigmoid Colon with Extralum Dev, Perc Approach */
"0DLN3DZ"  =  "1"  /* Occlusion of Sigmoid Colon with Intralum Dev, Perc Approach */
"0DLN3ZZ"  =  "1"  /* Occlusion of Sigmoid Colon, Percutaneous Approach */
"0DLN4CZ"  =  "1"  /* Occlusion Sigmoid Colon w Extralum Dev, Perc Endo */
"0DLN4DZ"  =  "1"  /* Occlusion Sigmoid Colon w Intralum Dev, Perc Endo */
"0DLN4ZZ"  =  "1"  /* Occlusion of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DLN7DZ"  =  "1"  /* Occlusion of Sigmoid Colon with Intralum Dev, Via Opening */
"0DLN7ZZ"  =  "1"  /* Occlusion of Sigmoid Colon, Via Opening */
"0DLN8DZ"  =  "1"  /* Occlusion of Sigmoid Colon with Intraluminal Device, Endo */
"0DLN8ZZ"  =  "1"  /* Occlusion of Sigmoid Colon, Endo */
"0DLP0CZ"  =  "1"  /* Occlusion of Rectum with Extraluminal Device, Open Approach */
"0DLP0DZ"  =  "1"  /* Occlusion of Rectum with Intraluminal Device, Open Approach */
"0DLP0ZZ"  =  "1"  /* Occlusion of Rectum, Open Approach */
"0DLP3CZ"  =  "1"  /* Occlusion of Rectum with Extraluminal Device, Perc Approach */
"0DLP3DZ"  =  "1"  /* Occlusion of Rectum with Intraluminal Device, Perc Approach */
"0DLP3ZZ"  =  "1"  /* Occlusion of Rectum, Percutaneous Approach */
"0DLP4CZ"  =  "1"  /* Occlusion of Rectum with Extralum Dev, Perc Endo Approach */
"0DLP4DZ"  =  "1"  /* Occlusion of Rectum with Intralum Dev, Perc Endo Approach */
"0DLP4ZZ"  =  "1"  /* Occlusion of Rectum, Percutaneous Endoscopic Approach */
"0DLP7DZ"  =  "1"  /* Occlusion of Rectum with Intraluminal Device, Via Opening */
"0DLP7ZZ"  =  "1"  /* Occlusion of Rectum, Via Natural or Artificial Opening */
"0DLP8DZ"  =  "1"  /* Occlusion of Rectum with Intraluminal Device, Endo */
"0DLP8ZZ"  =  "1"  /* Occlusion of Rectum, Endo */
"0DLQ0CZ"  =  "1"  /* Occlusion of Anus with Extraluminal Device, Open Approach */
"0DLQ0DZ"  =  "1"  /* Occlusion of Anus with Intraluminal Device, Open Approach */
"0DLQ0ZZ"  =  "1"  /* Occlusion of Anus, Open Approach */
"0DLQ3CZ"  =  "1"  /* Occlusion of Anus with Extraluminal Device, Perc Approach */
"0DLQ3DZ"  =  "1"  /* Occlusion of Anus with Intraluminal Device, Perc Approach */
"0DLQ3ZZ"  =  "1"  /* Occlusion of Anus, Percutaneous Approach */
"0DLQ4CZ"  =  "1"  /* Occlusion of Anus with Extralum Dev, Perc Endo Approach */
"0DLQ4DZ"  =  "1"  /* Occlusion of Anus with Intralum Dev, Perc Endo Approach */
"0DLQ4ZZ"  =  "1"  /* Occlusion of Anus, Percutaneous Endoscopic Approach */
"0DLQ7DZ"  =  "1"  /* Occlusion of Anus with Intraluminal Device, Via Opening */
"0DLQ7ZZ"  =  "1"  /* Occlusion of Anus, Via Natural or Artificial Opening */
"0DLQ8DZ"  =  "1"  /* Occlusion of Anus with Intraluminal Device, Endo */
"0DLQ8ZZ"  =  "1"  /* Occlusion of Anus, Endo */
"0DLQXCZ"  =  "1"  /* Occlusion of Anus with Extraluminal Device, Extern Approach */
"0DLQXDZ"  =  "1"  /* Occlusion of Anus with Intraluminal Device, Extern Approach */
"0DLQXZZ"  =  "1"  /* Occlusion of Anus, External Approach */
"0DM50ZZ"  =  "1"  /* Reattachment of Esophagus, Open Approach */
"0DM54ZZ"  =  "1"  /* Reattachment of Esophagus, Percutaneous Endoscopic Approach */
"0DM60ZZ"  =  "1"  /* Reattachment of Stomach, Open Approach */
"0DM64ZZ"  =  "1"  /* Reattachment of Stomach, Percutaneous Endoscopic Approach */
"0DM80ZZ"  =  "1"  /* Reattachment of Small Intestine, Open Approach */
"0DM84ZZ"  =  "1"  /* Reattachment of Small Intestine, Perc Endo Approach */
"0DM90ZZ"  =  "1"  /* Reattachment of Duodenum, Open Approach */
"0DM94ZZ"  =  "1"  /* Reattachment of Duodenum, Percutaneous Endoscopic Approach */
"0DMA0ZZ"  =  "1"  /* Reattachment of Jejunum, Open Approach */
"0DMA4ZZ"  =  "1"  /* Reattachment of Jejunum, Percutaneous Endoscopic Approach */
"0DMB0ZZ"  =  "1"  /* Reattachment of Ileum, Open Approach */
"0DMB4ZZ"  =  "1"  /* Reattachment of Ileum, Percutaneous Endoscopic Approach */
"0DME0ZZ"  =  "1"  /* Reattachment of Large Intestine, Open Approach */
"0DME4ZZ"  =  "1"  /* Reattachment of Large Intestine, Perc Endo Approach */
"0DMF0ZZ"  =  "1"  /* Reattachment of Right Large Intestine, Open Approach */
"0DMF4ZZ"  =  "1"  /* Reattachment of Right Large Intestine, Perc Endo Approach */
"0DMG0ZZ"  =  "1"  /* Reattachment of Left Large Intestine, Open Approach */
"0DMG4ZZ"  =  "1"  /* Reattachment of Left Large Intestine, Perc Endo Approach */
"0DMH0ZZ"  =  "1"  /* Reattachment of Cecum, Open Approach */
"0DMH4ZZ"  =  "1"  /* Reattachment of Cecum, Percutaneous Endoscopic Approach */
"0DMK0ZZ"  =  "1"  /* Reattachment of Ascending Colon, Open Approach */
"0DMK4ZZ"  =  "1"  /* Reattachment of Ascending Colon, Perc Endo Approach */
"0DML0ZZ"  =  "1"  /* Reattachment of Transverse Colon, Open Approach */
"0DML4ZZ"  =  "1"  /* Reattachment of Transverse Colon, Perc Endo Approach */
"0DMM0ZZ"  =  "1"  /* Reattachment of Descending Colon, Open Approach */
"0DMM4ZZ"  =  "1"  /* Reattachment of Descending Colon, Perc Endo Approach */
"0DMN0ZZ"  =  "1"  /* Reattachment of Sigmoid Colon, Open Approach */
"0DMN4ZZ"  =  "1"  /* Reattachment of Sigmoid Colon, Perc Endo Approach */
"0DMP0ZZ"  =  "1"  /* Reattachment of Rectum, Open Approach */
"0DMP4ZZ"  =  "1"  /* Reattachment of Rectum, Percutaneous Endoscopic Approach */
"0DN10ZZ"  =  "1"  /* Release Upper Esophagus, Open Approach */
"0DN13ZZ"  =  "1"  /* Release Upper Esophagus, Percutaneous Approach */
"0DN14ZZ"  =  "1"  /* Release Upper Esophagus, Percutaneous Endoscopic Approach */
"0DN17ZZ"  =  "1"  /* Release Upper Esophagus, Via Natural or Artificial Opening */
"0DN18ZZ"  =  "1"  /* Release Upper Esophagus, Endo */
"0DN20ZZ"  =  "1"  /* Release Middle Esophagus, Open Approach */
"0DN23ZZ"  =  "1"  /* Release Middle Esophagus, Percutaneous Approach */
"0DN24ZZ"  =  "1"  /* Release Middle Esophagus, Percutaneous Endoscopic Approach */
"0DN27ZZ"  =  "1"  /* Release Middle Esophagus, Via Natural or Artificial Opening */
"0DN28ZZ"  =  "1"  /* Release Middle Esophagus, Endo */
"0DN30ZZ"  =  "1"  /* Release Lower Esophagus, Open Approach */
"0DN33ZZ"  =  "1"  /* Release Lower Esophagus, Percutaneous Approach */
"0DN34ZZ"  =  "1"  /* Release Lower Esophagus, Percutaneous Endoscopic Approach */
"0DN37ZZ"  =  "1"  /* Release Lower Esophagus, Via Natural or Artificial Opening */
"0DN38ZZ"  =  "1"  /* Release Lower Esophagus, Endo */
"0DN40ZZ"  =  "1"  /* Release Esophagogastric Junction, Open Approach */
"0DN43ZZ"  =  "1"  /* Release Esophagogastric Junction, Percutaneous Approach */
"0DN44ZZ"  =  "1"  /* Release Esophagogastric Junction, Perc Endo Approach */
"0DN47ZZ"  =  "1"  /* Release Esophagogastric Junction, Via Opening */
"0DN48ZZ"  =  "1"  /* Release Esophagogastric Junction, Endo */
"0DN50ZZ"  =  "1"  /* Release Esophagus, Open Approach */
"0DN53ZZ"  =  "1"  /* Release Esophagus, Percutaneous Approach */
"0DN54ZZ"  =  "1"  /* Release Esophagus, Percutaneous Endoscopic Approach */
"0DN57ZZ"  =  "1"  /* Release Esophagus, Via Natural or Artificial Opening */
"0DN58ZZ"  =  "1"  /* Release Esophagus, Endo */
"0DN60ZZ"  =  "1"  /* Release Stomach, Open Approach */
"0DN63ZZ"  =  "1"  /* Release Stomach, Percutaneous Approach */
"0DN64ZZ"  =  "1"  /* Release Stomach, Percutaneous Endoscopic Approach */
"0DN67ZZ"  =  "1"  /* Release Stomach, Via Natural or Artificial Opening */
"0DN68ZZ"  =  "1"  /* Release Stomach, Endo */
"0DN70ZZ"  =  "1"  /* Release Stomach, Pylorus, Open Approach */
"0DN73ZZ"  =  "1"  /* Release Stomach, Pylorus, Percutaneous Approach */
"0DN74ZZ"  =  "1"  /* Release Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0DN77ZZ"  =  "1"  /* Release Stomach, Pylorus, Via Natural or Artificial Opening */
"0DN78ZZ"  =  "1"  /* Release Stomach, Pylorus, Endo */
"0DN80ZZ"  =  "1"  /* Release Small Intestine, Open Approach */
"0DN83ZZ"  =  "1"  /* Release Small Intestine, Percutaneous Approach */
"0DN84ZZ"  =  "1"  /* Release Small Intestine, Percutaneous Endoscopic Approach */
"0DN90ZZ"  =  "1"  /* Release Duodenum, Open Approach */
"0DN93ZZ"  =  "1"  /* Release Duodenum, Percutaneous Approach */
"0DN94ZZ"  =  "1"  /* Release Duodenum, Percutaneous Endoscopic Approach */
"0DNA0ZZ"  =  "1"  /* Release Jejunum, Open Approach */
"0DNA3ZZ"  =  "1"  /* Release Jejunum, Percutaneous Approach */
"0DNA4ZZ"  =  "1"  /* Release Jejunum, Percutaneous Endoscopic Approach */
"0DNB0ZZ"  =  "1"  /* Release Ileum, Open Approach */
"0DNB3ZZ"  =  "1"  /* Release Ileum, Percutaneous Approach */
"0DNB4ZZ"  =  "1"  /* Release Ileum, Percutaneous Endoscopic Approach */
"0DNC0ZZ"  =  "1"  /* Release Ileocecal Valve, Open Approach */
"0DNC3ZZ"  =  "1"  /* Release Ileocecal Valve, Percutaneous Approach */
"0DNC4ZZ"  =  "1"  /* Release Ileocecal Valve, Percutaneous Endoscopic Approach */
"0DNC7ZZ"  =  "1"  /* Release Ileocecal Valve, Via Natural or Artificial Opening */
"0DNC8ZZ"  =  "1"  /* Release Ileocecal Valve, Endo */
"0DNE0ZZ"  =  "1"  /* Release Large Intestine, Open Approach */
"0DNE3ZZ"  =  "1"  /* Release Large Intestine, Percutaneous Approach */
"0DNE4ZZ"  =  "1"  /* Release Large Intestine, Percutaneous Endoscopic Approach */
"0DNF0ZZ"  =  "1"  /* Release Right Large Intestine, Open Approach */
"0DNF3ZZ"  =  "1"  /* Release Right Large Intestine, Percutaneous Approach */
"0DNF4ZZ"  =  "1"  /* Release Right Large Intestine, Perc Endo Approach */
"0DNG0ZZ"  =  "1"  /* Release Left Large Intestine, Open Approach */
"0DNG3ZZ"  =  "1"  /* Release Left Large Intestine, Percutaneous Approach */
"0DNG4ZZ"  =  "1"  /* Release Left Large Intestine, Perc Endo Approach */
"0DNH0ZZ"  =  "1"  /* Release Cecum, Open Approach */
"0DNH3ZZ"  =  "1"  /* Release Cecum, Percutaneous Approach */
"0DNH4ZZ"  =  "1"  /* Release Cecum, Percutaneous Endoscopic Approach */
"0DNJ0ZZ"  =  "1"  /* Release Appendix, Open Approach */
"0DNJ3ZZ"  =  "1"  /* Release Appendix, Percutaneous Approach */
"0DNJ4ZZ"  =  "1"  /* Release Appendix, Percutaneous Endoscopic Approach */
"0DNJ7ZZ"  =  "1"  /* Release Appendix, Via Natural or Artificial Opening */
"0DNJ8ZZ"  =  "1"  /* Release Appendix, Endo */
"0DNK0ZZ"  =  "1"  /* Release Ascending Colon, Open Approach */
"0DNK3ZZ"  =  "1"  /* Release Ascending Colon, Percutaneous Approach */
"0DNK4ZZ"  =  "1"  /* Release Ascending Colon, Percutaneous Endoscopic Approach */
"0DNL0ZZ"  =  "1"  /* Release Transverse Colon, Open Approach */
"0DNL3ZZ"  =  "1"  /* Release Transverse Colon, Percutaneous Approach */
"0DNL4ZZ"  =  "1"  /* Release Transverse Colon, Percutaneous Endoscopic Approach */
"0DNM0ZZ"  =  "1"  /* Release Descending Colon, Open Approach */
"0DNM3ZZ"  =  "1"  /* Release Descending Colon, Percutaneous Approach */
"0DNM4ZZ"  =  "1"  /* Release Descending Colon, Percutaneous Endoscopic Approach */
"0DNN0ZZ"  =  "1"  /* Release Sigmoid Colon, Open Approach */
"0DNN3ZZ"  =  "1"  /* Release Sigmoid Colon, Percutaneous Approach */
"0DNN4ZZ"  =  "1"  /* Release Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DNP0ZZ"  =  "1"  /* Release Rectum, Open Approach */
"0DNP3ZZ"  =  "1"  /* Release Rectum, Percutaneous Approach */
"0DNP4ZZ"  =  "1"  /* Release Rectum, Percutaneous Endoscopic Approach */
"0DNP7ZZ"  =  "1"  /* Release Rectum, Via Natural or Artificial Opening */
"0DNP8ZZ"  =  "1"  /* Release Rectum, Via Natural or Artificial Opening Endoscopic */
"0DNQ0ZZ"  =  "1"  /* Release Anus, Open Approach */
"0DNQ3ZZ"  =  "1"  /* Release Anus, Percutaneous Approach */
"0DNQ4ZZ"  =  "1"  /* Release Anus, Percutaneous Endoscopic Approach */
"0DNQ7ZZ"  =  "1"  /* Release Anus, Via Natural or Artificial Opening */
"0DNQ8ZZ"  =  "1"  /* Release Anus, Via Natural or Artificial Opening Endoscopic */
"0DNQXZZ"  =  "1"  /* Release Anus, External Approach */
"0DNR0ZZ"  =  "1"  /* Release Anal Sphincter, Open Approach */
"0DNR3ZZ"  =  "1"  /* Release Anal Sphincter, Percutaneous Approach */
"0DNR4ZZ"  =  "1"  /* Release Anal Sphincter, Percutaneous Endoscopic Approach */
"0DNS0ZZ"  =  "1"  /* Release Greater Omentum, Open Approach */
"0DNS3ZZ"  =  "1"  /* Release Greater Omentum, Percutaneous Approach */
"0DNS4ZZ"  =  "1"  /* Release Greater Omentum, Percutaneous Endoscopic Approach */
"0DNT0ZZ"  =  "1"  /* Release Lesser Omentum, Open Approach */
"0DNT3ZZ"  =  "1"  /* Release Lesser Omentum, Percutaneous Approach */
"0DNT4ZZ"  =  "1"  /* Release Lesser Omentum, Percutaneous Endoscopic Approach */
"0DNV0ZZ"  =  "1"  /* Release Mesentery, Open Approach */
"0DNV3ZZ"  =  "1"  /* Release Mesentery, Percutaneous Approach */
"0DNV4ZZ"  =  "1"  /* Release Mesentery, Percutaneous Endoscopic Approach */
"0DNW0ZZ"  =  "1"  /* Release Peritoneum, Open Approach */
"0DNW3ZZ"  =  "1"  /* Release Peritoneum, Percutaneous Approach */
"0DNW4ZZ"  =  "1"  /* Release Peritoneum, Percutaneous Endoscopic Approach */
"0DP000Z"  =  "1"  /* Removal of Drain Dev from Up Intest Tract, Open Approach */
"0DP002Z"  =  "1"  /* Removal of Monitor Dev from Up Intest Tract, Open Approach */
"0DP003Z"  =  "1"  /* Removal of Infusion Dev from Up Intest Tract, Open Approach */
"0DP007Z"  =  "1"  /* Removal of Autol Sub from Up Intest Tract, Open Approach */
"0DP00CZ"  =  "1"  /* Removal of Extralum Dev from Up Intest Tract, Open Approach */
"0DP00DZ"  =  "1"  /* Removal of Intralum Dev from Up Intest Tract, Open Approach */
"0DP00JZ"  =  "1"  /* Removal of Synth Sub from Up Intest Tract, Open Approach */
"0DP00KZ"  =  "1"  /* Removal of Nonaut Sub from Up Intest Tract, Open Approach */
"0DP00UZ"  =  "1"  /* Removal of Feeding Dev from Up Intest Tract, Open Approach */
"0DP030Z"  =  "1"  /* Removal of Drain Dev from Up Intest Tract, Perc Approach */
"0DP032Z"  =  "1"  /* Removal of Monitor Dev from Up Intest Tract, Perc Approach */
"0DP033Z"  =  "1"  /* Removal of Infusion Dev from Up Intest Tract, Perc Approach */
"0DP037Z"  =  "1"  /* Removal of Autol Sub from Up Intest Tract, Perc Approach */
"0DP03CZ"  =  "1"  /* Removal of Extralum Dev from Up Intest Tract, Perc Approach */
"0DP03DZ"  =  "1"  /* Removal of Intralum Dev from Up Intest Tract, Perc Approach */
"0DP03JZ"  =  "1"  /* Removal of Synth Sub from Up Intest Tract, Perc Approach */
"0DP03KZ"  =  "1"  /* Removal of Nonaut Sub from Up Intest Tract, Perc Approach */
"0DP03UZ"  =  "1"  /* Removal of Feeding Dev from Up Intest Tract, Perc Approach */
"0DP040Z"  =  "1"  /* Remove of Drain Dev from Up Intest Tract, Perc Endo Approach */
"0DP042Z"  =  "1"  /* Remove Monitor Dev from Up Intest Tract, Perc Endo */
"0DP043Z"  =  "1"  /* Remove Infusion Dev from Up Intest Tract, Perc Endo */
"0DP047Z"  =  "1"  /* Remove of Autol Sub from Up Intest Tract, Perc Endo Approach */
"0DP04CZ"  =  "1"  /* Remove Extralum Dev from Up Intest Tract, Perc Endo */
"0DP04DZ"  =  "1"  /* Remove Intralum Dev from Up Intest Tract, Perc Endo */
"0DP04JZ"  =  "1"  /* Remove of Synth Sub from Up Intest Tract, Perc Endo Approach */
"0DP04KZ"  =  "1"  /* Remove Nonaut Sub from Up Intest Tract, Perc Endo */
"0DP04UZ"  =  "1"  /* Remove Feeding Dev from Up Intest Tract, Perc Endo */
"0DP070Z"  =  "1"  /* Removal of Drainage Device from Up Intest Tract, Via Opening */
"0DP072Z"  =  "1"  /* Removal of Monitor Dev from Up Intest Tract, Via Opening */
"0DP073Z"  =  "1"  /* Removal of Infusion Device from Up Intest Tract, Via Opening */
"0DP077Z"  =  "1"  /* Removal of Autol Sub from Up Intest Tract, Via Opening */
"0DP07CZ"  =  "1"  /* Removal of Extralum Dev from Up Intest Tract, Via Opening */
"0DP07DZ"  =  "1"  /* Removal of Intralum Dev from Up Intest Tract, Via Opening */
"0DP07JZ"  =  "1"  /* Removal of Synth Sub from Up Intest Tract, Via Opening */
"0DP07KZ"  =  "1"  /* Removal of Nonaut Sub from Up Intest Tract, Via Opening */
"0DP07UZ"  =  "1"  /* Removal of Feeding Device from Up Intest Tract, Via Opening */
"0DP080Z"  =  "1"  /* Removal of Drainage Device from Upper Intestinal Tract, Endo */
"0DP082Z"  =  "1"  /* Removal of Monitoring Device from Up Intest Tract, Endo */
"0DP083Z"  =  "1"  /* Removal of Infusion Device from Upper Intestinal Tract, Endo */
"0DP087Z"  =  "1"  /* Removal of Autol Sub from Up Intest Tract, Endo */
"0DP08CZ"  =  "1"  /* Removal of Extraluminal Device from Up Intest Tract, Endo */
"0DP08DZ"  =  "1"  /* Removal of Intraluminal Device from Up Intest Tract, Endo */
"0DP08JZ"  =  "1"  /* Removal of Synthetic Substitute from Up Intest Tract, Endo */
"0DP08KZ"  =  "1"  /* Removal of Nonaut Sub from Up Intest Tract, Endo */
"0DP08UZ"  =  "1"  /* Removal of Feeding Device from Upper Intestinal Tract, Endo */
"0DP501Z"  =  "1"  /* Removal of Radioactive Element from Esophagus, Open Approach */
"0DP502Z"  =  "1"  /* Removal of Monitoring Device from Esophagus, Open Approach */
"0DP503Z"  =  "1"  /* Removal of Infusion Device from Esophagus, Open Approach */
"0DP50UZ"  =  "1"  /* Removal of Feeding Device from Esophagus, Open Approach */
"0DP531Z"  =  "1"  /* Removal of Radioactive Element from Esophagus, Perc Approach */
"0DP532Z"  =  "1"  /* Removal of Monitoring Device from Esophagus, Perc Approach */
"0DP533Z"  =  "1"  /* Removal of Infusion Device from Esophagus, Perc Approach */
"0DP53UZ"  =  "1"  /* Removal of Feeding Device from Esophagus, Perc Approach */
"0DP541Z"  =  "1"  /* Removal of Radioact Elem from Esophag, Perc Endo Approach */
"0DP542Z"  =  "1"  /* Removal of Monitor Dev from Esophag, Perc Endo Approach */
"0DP543Z"  =  "1"  /* Removal of Infusion Device from Esophag, Perc Endo Approach */
"0DP54UZ"  =  "1"  /* Removal of Feeding Device from Esophagus, Perc Endo Approach */
"0DP57DZ"  =  "1"  /* Removal of Intraluminal Device from Esophagus, Via Opening */
"0DP58DZ"  =  "1"  /* Removal of Intraluminal Device from Esophagus, Endo */
"0DP600Z"  =  "1"  /* Removal of Drainage Device from Stomach, Open Approach */
"0DP602Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Open Approach */
"0DP603Z"  =  "1"  /* Removal of Infusion Device from Stomach, Open Approach */
"0DP607Z"  =  "1"  /* Removal of Autol Sub from Stomach, Open Approach */
"0DP60CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Open Approach */
"0DP60DZ"  =  "1"  /* Removal of Intraluminal Device from Stomach, Open Approach */
"0DP60JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Open Approach */
"0DP60KZ"  =  "1"  /* Removal of Nonaut Sub from Stomach, Open Approach */
"0DP60MZ"  =  "1"  /* Removal of Stimulator Lead from Stomach, Open Approach */
"0DP60UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Open Approach */
"0DP630Z"  =  "1"  /* Removal of Drainage Device from Stomach, Perc Approach */
"0DP632Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Perc Approach */
"0DP633Z"  =  "1"  /* Removal of Infusion Device from Stomach, Perc Approach */
"0DP637Z"  =  "1"  /* Removal of Autol Sub from Stomach, Perc Approach */
"0DP63CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Perc Approach */
"0DP63DZ"  =  "1"  /* Removal of Intraluminal Device from Stomach, Perc Approach */
"0DP63JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Perc Approach */
"0DP63KZ"  =  "1"  /* Removal of Nonaut Sub from Stomach, Perc Approach */
"0DP63MZ"  =  "1"  /* Removal of Stimulator Lead from Stomach, Perc Approach */
"0DP63UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Perc Approach */
"0DP640Z"  =  "1"  /* Removal of Drainage Device from Stomach, Perc Endo Approach */
"0DP642Z"  =  "1"  /* Removal of Monitor Dev from Stomach, Perc Endo Approach */
"0DP643Z"  =  "1"  /* Removal of Infusion Device from Stomach, Perc Endo Approach */
"0DP647Z"  =  "1"  /* Removal of Autol Sub from Stomach, Perc Endo Approach */
"0DP64CZ"  =  "1"  /* Removal of Extralum Dev from Stomach, Perc Endo Approach */
"0DP64DZ"  =  "1"  /* Removal of Intralum Dev from Stomach, Perc Endo Approach */
"0DP64JZ"  =  "1"  /* Removal of Synth Sub from Stomach, Perc Endo Approach */
"0DP64KZ"  =  "1"  /* Removal of Nonaut Sub from Stomach, Perc Endo Approach */
"0DP64MZ"  =  "1"  /* Removal of Stimulator Lead from Stomach, Perc Endo Approach */
"0DP64UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Perc Endo Approach */
"0DP670Z"  =  "1"  /* Removal of Drainage Device from Stomach, Via Opening */
"0DP672Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Via Opening */
"0DP673Z"  =  "1"  /* Removal of Infusion Device from Stomach, Via Opening */
"0DP677Z"  =  "1"  /* Removal of Autol Sub from Stomach, Via Opening */
"0DP67CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Via Opening */
"0DP67JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Via Opening */
"0DP67KZ"  =  "1"  /* Removal of Nonaut Sub from Stomach, Via Opening */
"0DP67UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Via Opening */
"0DP680Z"  =  "1"  /* Removal of Drainage Device from Stomach, Endo */
"0DP682Z"  =  "1"  /* Removal of Monitoring Device from Stomach, Endo */
"0DP683Z"  =  "1"  /* Removal of Infusion Device from Stomach, Endo */
"0DP687Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Stomach, Endo */
"0DP68CZ"  =  "1"  /* Removal of Extraluminal Device from Stomach, Endo */
"0DP68JZ"  =  "1"  /* Removal of Synthetic Substitute from Stomach, Endo */
"0DP68KZ"  =  "1"  /* Removal of Nonaut Sub from Stomach, Endo */
"0DP68UZ"  =  "1"  /* Removal of Feeding Device from Stomach, Endo */
"0DPD00Z"  =  "1"  /* Removal of Drain Dev from Low Intest Tract, Open Approach */
"0DPD02Z"  =  "1"  /* Removal of Monitor Dev from Low Intest Tract, Open Approach */
"0DPD03Z"  =  "1"  /* Removal of Infusion Dev from Low Intest Tract, Open Approach */
"0DPD07Z"  =  "1"  /* Removal of Autol Sub from Low Intest Tract, Open Approach */
"0DPD0CZ"  =  "1"  /* Removal of Extralum Dev from Low Intest Tract, Open Approach */
"0DPD0DZ"  =  "1"  /* Removal of Intralum Dev from Low Intest Tract, Open Approach */
"0DPD0JZ"  =  "1"  /* Removal of Synth Sub from Low Intest Tract, Open Approach */
"0DPD0KZ"  =  "1"  /* Removal of Nonaut Sub from Low Intest Tract, Open Approach */
"0DPD0UZ"  =  "1"  /* Removal of Feeding Dev from Low Intest Tract, Open Approach */
"0DPD30Z"  =  "1"  /* Removal of Drain Dev from Low Intest Tract, Perc Approach */
"0DPD32Z"  =  "1"  /* Removal of Monitor Dev from Low Intest Tract, Perc Approach */
"0DPD33Z"  =  "1"  /* Removal of Infusion Dev from Low Intest Tract, Perc Approach */
"0DPD37Z"  =  "1"  /* Removal of Autol Sub from Low Intest Tract, Perc Approach */
"0DPD3CZ"  =  "1"  /* Removal of Extralum Dev from Low Intest Tract, Perc Approach */
"0DPD3DZ"  =  "1"  /* Removal of Intralum Dev from Low Intest Tract, Perc Approach */
"0DPD3JZ"  =  "1"  /* Removal of Synth Sub from Low Intest Tract, Perc Approach */
"0DPD3KZ"  =  "1"  /* Removal of Nonaut Sub from Low Intest Tract, Perc Approach */
"0DPD3UZ"  =  "1"  /* Removal of Feeding Dev from Low Intest Tract, Perc Approach */
"0DPD40Z"  =  "1"  /* Remove Drain Dev from Low Intest Tract, Perc Endo */
"0DPD42Z"  =  "1"  /* Remove Monitor Dev from Low Intest Tract, Perc Endo */
"0DPD43Z"  =  "1"  /* Remove Infusion Dev from Low Intest Tract, Perc Endo */
"0DPD47Z"  =  "1"  /* Remove Autol Sub from Low Intest Tract, Perc Endo */
"0DPD4CZ"  =  "1"  /* Remove Extralum Dev from Low Intest Tract, Perc Endo */
"0DPD4DZ"  =  "1"  /* Remove Intralum Dev from Low Intest Tract, Perc Endo */
"0DPD4JZ"  =  "1"  /* Remove Synth Sub from Low Intest Tract, Perc Endo */
"0DPD4KZ"  =  "1"  /* Remove Nonaut Sub from Low Intest Tract, Perc Endo */
"0DPD4UZ"  =  "1"  /* Remove Feeding Dev from Low Intest Tract, Perc Endo */
"0DPD70Z"  =  "1"  /* Removal of Drain Dev from Low Intest Tract, Via Opening */
"0DPD72Z"  =  "1"  /* Removal of Monitor Dev from Low Intest Tract, Via Opening */
"0DPD73Z"  =  "1"  /* Removal of Infusion Dev from Low Intest Tract, Via Opening */
"0DPD77Z"  =  "1"  /* Removal of Autol Sub from Low Intest Tract, Via Opening */
"0DPD7CZ"  =  "1"  /* Removal of Extralum Dev from Low Intest Tract, Via Opening */
"0DPD7DZ"  =  "1"  /* Removal of Intralum Dev from Low Intest Tract, Via Opening */
"0DPD7JZ"  =  "1"  /* Removal of Synth Sub from Low Intest Tract, Via Opening */
"0DPD7KZ"  =  "1"  /* Removal of Nonaut Sub from Low Intest Tract, Via Opening */
"0DPD7UZ"  =  "1"  /* Removal of Feeding Device from Low Intest Tract, Via Opening */
"0DPD80Z"  =  "1"  /* Removal of Drainage Device from Lower Intestinal Tract, Endo */
"0DPD82Z"  =  "1"  /* Removal of Monitoring Device from Low Intest Tract, Endo */
"0DPD83Z"  =  "1"  /* Removal of Infusion Device from Lower Intestinal Tract, Endo */
"0DPD87Z"  =  "1"  /* Removal of Autol Sub from Low Intest Tract, Endo */
"0DPD8CZ"  =  "1"  /* Removal of Extraluminal Device from Low Intest Tract, Endo */
"0DPD8DZ"  =  "1"  /* Removal of Intraluminal Device from Low Intest Tract, Endo */
"0DPD8JZ"  =  "1"  /* Removal of Synthetic Substitute from Low Intest Tract, Endo */
"0DPD8KZ"  =  "1"  /* Removal of Nonaut Sub from Low Intest Tract, Endo */
"0DPD8UZ"  =  "1"  /* Removal of Feeding Device from Lower Intestinal Tract, Endo */
"0DPP01Z"  =  "1"  /* Removal of Radioactive Element from Rectum, Open Approach */
"0DPP31Z"  =  "1"  /* Removal of Radioactive Element from Rectum, Perc Approach */
"0DPP41Z"  =  "1"  /* Removal of Radioact Elem from Rectum, Perc Endo Approach */
"0DPQ0LZ"  =  "1"  /* Removal of Artificial Sphincter from Anus, Open Approach */
"0DPQ3LZ"  =  "1"  /* Removal of Artificial Sphincter from Anus, Perc Approach */
"0DPQ4LZ"  =  "1"  /* Removal of Artif Sphincter from Anus, Perc Endo Approach */
"0DPQ7LZ"  =  "1"  /* Removal of Artificial Sphincter from Anus, Via Opening */
"0DPQ8LZ"  =  "1"  /* Removal of Artificial Sphincter from Anus, Endo */
"0DPR0MZ"  =  "1"  /* Removal of Stim Lead from Anal Sphincter, Open Approach */
"0DPR3MZ"  =  "1"  /* Removal of Stim Lead from Anal Sphincter, Perc Approach */
"0DPR4MZ"  =  "1"  /* Removal of Stim Lead from Anal Sphincter, Perc Endo Approach */
"0DPU00Z"  =  "1"  /* Removal of Drainage Device from Omentum, Open Approach */
"0DPU01Z"  =  "1"  /* Removal of Radioactive Element from Omentum, Open Approach */
"0DPU07Z"  =  "1"  /* Removal of Autol Sub from Omentum, Open Approach */
"0DPU0JZ"  =  "1"  /* Removal of Synthetic Substitute from Omentum, Open Approach */
"0DPU0KZ"  =  "1"  /* Removal of Nonaut Sub from Omentum, Open Approach */
"0DPU30Z"  =  "1"  /* Removal of Drainage Device from Omentum, Perc Approach */
"0DPU31Z"  =  "1"  /* Removal of Radioactive Element from Omentum, Perc Approach */
"0DPU37Z"  =  "1"  /* Removal of Autol Sub from Omentum, Perc Approach */
"0DPU3JZ"  =  "1"  /* Removal of Synthetic Substitute from Omentum, Perc Approach */
"0DPU3KZ"  =  "1"  /* Removal of Nonaut Sub from Omentum, Perc Approach */
"0DPU40Z"  =  "1"  /* Removal of Drainage Device from Omentum, Perc Endo Approach */
"0DPU41Z"  =  "1"  /* Removal of Radioact Elem from Omentum, Perc Endo Approach */
"0DPU47Z"  =  "1"  /* Removal of Autol Sub from Omentum, Perc Endo Approach */
"0DPU4JZ"  =  "1"  /* Removal of Synth Sub from Omentum, Perc Endo Approach */
"0DPU4KZ"  =  "1"  /* Removal of Nonaut Sub from Omentum, Perc Endo Approach */
"0DPV00Z"  =  "1"  /* Removal of Drainage Device from Mesentery, Open Approach */
"0DPV01Z"  =  "1"  /* Removal of Radioactive Element from Mesentery, Open Approach */
"0DPV07Z"  =  "1"  /* Removal of Autol Sub from Mesentery, Open Approach */
"0DPV0JZ"  =  "1"  /* Removal of Synth Sub from Mesentery, Open Approach */
"0DPV0KZ"  =  "1"  /* Removal of Nonaut Sub from Mesentery, Open Approach */
"0DPV30Z"  =  "1"  /* Removal of Drainage Device from Mesentery, Perc Approach */
"0DPV31Z"  =  "1"  /* Removal of Radioactive Element from Mesentery, Perc Approach */
"0DPV37Z"  =  "1"  /* Removal of Autol Sub from Mesentery, Perc Approach */
"0DPV3JZ"  =  "1"  /* Removal of Synth Sub from Mesentery, Perc Approach */
"0DPV3KZ"  =  "1"  /* Removal of Nonaut Sub from Mesentery, Perc Approach */
"0DPV40Z"  =  "1"  /* Removal of Drain Dev from Mesentery, Perc Endo Approach */
"0DPV41Z"  =  "1"  /* Removal of Radioact Elem from Mesentery, Perc Endo Approach */
"0DPV47Z"  =  "1"  /* Removal of Autol Sub from Mesentery, Perc Endo Approach */
"0DPV4JZ"  =  "1"  /* Removal of Synth Sub from Mesentery, Perc Endo Approach */
"0DPV4KZ"  =  "1"  /* Removal of Nonaut Sub from Mesentery, Perc Endo Approach */
"0DPW00Z"  =  "1"  /* Removal of Drainage Device from Peritoneum, Open Approach */
"0DPW01Z"  =  "1"  /* Removal of Radioact Elem from Peritoneum, Open Approach */
"0DPW07Z"  =  "1"  /* Removal of Autol Sub from Peritoneum, Open Approach */
"0DPW0JZ"  =  "1"  /* Removal of Synth Sub from Peritoneum, Open Approach */
"0DPW0KZ"  =  "1"  /* Removal of Nonaut Sub from Peritoneum, Open Approach */
"0DPW30Z"  =  "1"  /* Removal of Drainage Device from Peritoneum, Perc Approach */
"0DPW31Z"  =  "1"  /* Removal of Radioact Elem from Peritoneum, Perc Approach */
"0DPW37Z"  =  "1"  /* Removal of Autol Sub from Peritoneum, Perc Approach */
"0DPW3JZ"  =  "1"  /* Removal of Synth Sub from Peritoneum, Perc Approach */
"0DPW3KZ"  =  "1"  /* Removal of Nonaut Sub from Peritoneum, Perc Approach */
"0DPW40Z"  =  "1"  /* Removal of Drain Dev from Peritoneum, Perc Endo Approach */
"0DPW41Z"  =  "1"  /* Removal of Radioact Elem from Peritoneum, Perc Endo Approach */
"0DPW47Z"  =  "1"  /* Removal of Autol Sub from Peritoneum, Perc Endo Approach */
"0DPW4JZ"  =  "1"  /* Removal of Synth Sub from Peritoneum, Perc Endo Approach */
"0DPW4KZ"  =  "1"  /* Removal of Nonaut Sub from Peritoneum, Perc Endo Approach */
"0DQ10ZZ"  =  "1"  /* Repair Upper Esophagus, Open Approach */
"0DQ13ZZ"  =  "1"  /* Repair Upper Esophagus, Percutaneous Approach */
"0DQ14ZZ"  =  "1"  /* Repair Upper Esophagus, Percutaneous Endoscopic Approach */
"0DQ17ZZ"  =  "1"  /* Repair Upper Esophagus, Via Natural or Artificial Opening */
"0DQ18ZZ"  =  "1"  /* Repair Upper Esophagus, Endo */
"0DQ20ZZ"  =  "1"  /* Repair Middle Esophagus, Open Approach */
"0DQ23ZZ"  =  "1"  /* Repair Middle Esophagus, Percutaneous Approach */
"0DQ24ZZ"  =  "1"  /* Repair Middle Esophagus, Percutaneous Endoscopic Approach */
"0DQ27ZZ"  =  "1"  /* Repair Middle Esophagus, Via Natural or Artificial Opening */
"0DQ28ZZ"  =  "1"  /* Repair Middle Esophagus, Endo */
"0DQ30ZZ"  =  "1"  /* Repair Lower Esophagus, Open Approach */
"0DQ33ZZ"  =  "1"  /* Repair Lower Esophagus, Percutaneous Approach */
"0DQ34ZZ"  =  "1"  /* Repair Lower Esophagus, Percutaneous Endoscopic Approach */
"0DQ37ZZ"  =  "1"  /* Repair Lower Esophagus, Via Natural or Artificial Opening */
"0DQ38ZZ"  =  "1"  /* Repair Lower Esophagus, Endo */
"0DQ40ZZ"  =  "1"  /* Repair Esophagogastric Junction, Open Approach */
"0DQ43ZZ"  =  "1"  /* Repair Esophagogastric Junction, Percutaneous Approach */
"0DQ44ZZ"  =  "1"  /* Repair Esophagogastric Junction, Perc Endo Approach */
"0DQ47ZZ"  =  "1"  /* Repair Esophagogastric Junction, Via Opening */
"0DQ48ZZ"  =  "1"  /* Repair Esophagogastric Junction, Endo */
"0DQ50ZZ"  =  "1"  /* Repair Esophagus, Open Approach */
"0DQ53ZZ"  =  "1"  /* Repair Esophagus, Percutaneous Approach */
"0DQ54ZZ"  =  "1"  /* Repair Esophagus, Percutaneous Endoscopic Approach */
"0DQ57ZZ"  =  "1"  /* Repair Esophagus, Via Natural or Artificial Opening */
"0DQ58ZZ"  =  "1"  /* Repair Esophagus, Endo */
"0DQ60ZZ"  =  "1"  /* Repair Stomach, Open Approach */
"0DQ63ZZ"  =  "1"  /* Repair Stomach, Percutaneous Approach */
"0DQ64ZZ"  =  "1"  /* Repair Stomach, Percutaneous Endoscopic Approach */
"0DQ67ZZ"  =  "1"  /* Repair Stomach, Via Natural or Artificial Opening */
"0DQ68ZZ"  =  "1"  /* Repair Stomach, Via Natural or Artificial Opening Endoscopic */
"0DQ70ZZ"  =  "1"  /* Repair Stomach, Pylorus, Open Approach */
"0DQ73ZZ"  =  "1"  /* Repair Stomach, Pylorus, Percutaneous Approach */
"0DQ74ZZ"  =  "1"  /* Repair Stomach, Pylorus, Percutaneous Endoscopic Approach */
"0DQ77ZZ"  =  "1"  /* Repair Stomach, Pylorus, Via Natural or Artificial Opening */
"0DQ78ZZ"  =  "1"  /* Repair Stomach, Pylorus, Endo */
"0DQ80ZZ"  =  "1"  /* Repair Small Intestine, Open Approach */
"0DQ83ZZ"  =  "1"  /* Repair Small Intestine, Percutaneous Approach */
"0DQ84ZZ"  =  "1"  /* Repair Small Intestine, Percutaneous Endoscopic Approach */
"0DQ87ZZ"  =  "1"  /* Repair Small Intestine, Via Natural or Artificial Opening */
"0DQ88ZZ"  =  "1"  /* Repair Small Intestine, Endo */
"0DQ90ZZ"  =  "1"  /* Repair Duodenum, Open Approach */
"0DQ93ZZ"  =  "1"  /* Repair Duodenum, Percutaneous Approach */
"0DQ94ZZ"  =  "1"  /* Repair Duodenum, Percutaneous Endoscopic Approach */
"0DQ97ZZ"  =  "1"  /* Repair Duodenum, Via Natural or Artificial Opening */
"0DQ98ZZ"  =  "1"  /* Repair Duodenum, Endo */
"0DQA0ZZ"  =  "1"  /* Repair Jejunum, Open Approach */
"0DQA3ZZ"  =  "1"  /* Repair Jejunum, Percutaneous Approach */
"0DQA4ZZ"  =  "1"  /* Repair Jejunum, Percutaneous Endoscopic Approach */
"0DQA7ZZ"  =  "1"  /* Repair Jejunum, Via Natural or Artificial Opening */
"0DQA8ZZ"  =  "1"  /* Repair Jejunum, Via Natural or Artificial Opening Endoscopic */
"0DQB0ZZ"  =  "1"  /* Repair Ileum, Open Approach */
"0DQB3ZZ"  =  "1"  /* Repair Ileum, Percutaneous Approach */
"0DQB4ZZ"  =  "1"  /* Repair Ileum, Percutaneous Endoscopic Approach */
"0DQB7ZZ"  =  "1"  /* Repair Ileum, Via Natural or Artificial Opening */
"0DQB8ZZ"  =  "1"  /* Repair Ileum, Via Natural or Artificial Opening Endoscopic */
"0DQC0ZZ"  =  "1"  /* Repair Ileocecal Valve, Open Approach */
"0DQC3ZZ"  =  "1"  /* Repair Ileocecal Valve, Percutaneous Approach */
"0DQC4ZZ"  =  "1"  /* Repair Ileocecal Valve, Percutaneous Endoscopic Approach */
"0DQC7ZZ"  =  "1"  /* Repair Ileocecal Valve, Via Natural or Artificial Opening */
"0DQC8ZZ"  =  "1"  /* Repair Ileocecal Valve, Endo */
"0DQE0ZZ"  =  "1"  /* Repair Large Intestine, Open Approach */
"0DQE3ZZ"  =  "1"  /* Repair Large Intestine, Percutaneous Approach */
"0DQE4ZZ"  =  "1"  /* Repair Large Intestine, Percutaneous Endoscopic Approach */
"0DQE7ZZ"  =  "1"  /* Repair Large Intestine, Via Natural or Artificial Opening */
"0DQE8ZZ"  =  "1"  /* Repair Large Intestine, Endo */
"0DQF0ZZ"  =  "1"  /* Repair Right Large Intestine, Open Approach */
"0DQF3ZZ"  =  "1"  /* Repair Right Large Intestine, Percutaneous Approach */
"0DQF4ZZ"  =  "1"  /* Repair Right Large Intestine, Perc Endo Approach */
"0DQF7ZZ"  =  "1"  /* Repair Right Large Intestine, Via Opening */
"0DQF8ZZ"  =  "1"  /* Repair Right Large Intestine, Endo */
"0DQG0ZZ"  =  "1"  /* Repair Left Large Intestine, Open Approach */
"0DQG3ZZ"  =  "1"  /* Repair Left Large Intestine, Percutaneous Approach */
"0DQG4ZZ"  =  "1"  /* Repair Left Large Intestine, Perc Endo Approach */
"0DQG7ZZ"  =  "1"  /* Repair Left Large Intestine, Via Opening */
"0DQG8ZZ"  =  "1"  /* Repair Left Large Intestine, Endo */
"0DQH0ZZ"  =  "1"  /* Repair Cecum, Open Approach */
"0DQH3ZZ"  =  "1"  /* Repair Cecum, Percutaneous Approach */
"0DQH4ZZ"  =  "1"  /* Repair Cecum, Percutaneous Endoscopic Approach */
"0DQH7ZZ"  =  "1"  /* Repair Cecum, Via Natural or Artificial Opening */
"0DQH8ZZ"  =  "1"  /* Repair Cecum, Via Natural or Artificial Opening Endoscopic */
"0DQJ0ZZ"  =  "1"  /* Repair Appendix, Open Approach */
"0DQJ3ZZ"  =  "1"  /* Repair Appendix, Percutaneous Approach */
"0DQJ4ZZ"  =  "1"  /* Repair Appendix, Percutaneous Endoscopic Approach */
"0DQJ7ZZ"  =  "1"  /* Repair Appendix, Via Natural or Artificial Opening */
"0DQJ8ZZ"  =  "1"  /* Repair Appendix, Endo */
"0DQK0ZZ"  =  "1"  /* Repair Ascending Colon, Open Approach */
"0DQK3ZZ"  =  "1"  /* Repair Ascending Colon, Percutaneous Approach */
"0DQK4ZZ"  =  "1"  /* Repair Ascending Colon, Percutaneous Endoscopic Approach */
"0DQK7ZZ"  =  "1"  /* Repair Ascending Colon, Via Natural or Artificial Opening */
"0DQK8ZZ"  =  "1"  /* Repair Ascending Colon, Endo */
"0DQL0ZZ"  =  "1"  /* Repair Transverse Colon, Open Approach */
"0DQL3ZZ"  =  "1"  /* Repair Transverse Colon, Percutaneous Approach */
"0DQL4ZZ"  =  "1"  /* Repair Transverse Colon, Percutaneous Endoscopic Approach */
"0DQL7ZZ"  =  "1"  /* Repair Transverse Colon, Via Natural or Artificial Opening */
"0DQL8ZZ"  =  "1"  /* Repair Transverse Colon, Endo */
"0DQM0ZZ"  =  "1"  /* Repair Descending Colon, Open Approach */
"0DQM3ZZ"  =  "1"  /* Repair Descending Colon, Percutaneous Approach */
"0DQM4ZZ"  =  "1"  /* Repair Descending Colon, Percutaneous Endoscopic Approach */
"0DQM7ZZ"  =  "1"  /* Repair Descending Colon, Via Natural or Artificial Opening */
"0DQM8ZZ"  =  "1"  /* Repair Descending Colon, Endo */
"0DQN0ZZ"  =  "1"  /* Repair Sigmoid Colon, Open Approach */
"0DQN3ZZ"  =  "1"  /* Repair Sigmoid Colon, Percutaneous Approach */
"0DQN4ZZ"  =  "1"  /* Repair Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DQN7ZZ"  =  "1"  /* Repair Sigmoid Colon, Via Natural or Artificial Opening */
"0DQN8ZZ"  =  "1"  /* Repair Sigmoid Colon, Endo */
"0DQP0ZZ"  =  "1"  /* Repair Rectum, Open Approach */
"0DQP3ZZ"  =  "1"  /* Repair Rectum, Percutaneous Approach */
"0DQP4ZZ"  =  "1"  /* Repair Rectum, Percutaneous Endoscopic Approach */
"0DQP7ZZ"  =  "1"  /* Repair Rectum, Via Natural or Artificial Opening */
"0DQP8ZZ"  =  "1"  /* Repair Rectum, Via Natural or Artificial Opening Endoscopic */
"0DQQ0ZZ"  =  "1"  /* Repair Anus, Open Approach */
"0DQQ3ZZ"  =  "1"  /* Repair Anus, Percutaneous Approach */
"0DQQ4ZZ"  =  "1"  /* Repair Anus, Percutaneous Endoscopic Approach */
"0DQQ7ZZ"  =  "1"  /* Repair Anus, Via Natural or Artificial Opening */
"0DQQ8ZZ"  =  "1"  /* Repair Anus, Via Natural or Artificial Opening Endoscopic */
"0DQQXZZ"  =  "1"  /* Repair Anus, External Approach */
"0DQR0ZZ"  =  "1"  /* Repair Anal Sphincter, Open Approach */
"0DQR3ZZ"  =  "1"  /* Repair Anal Sphincter, Percutaneous Approach */
"0DQR4ZZ"  =  "1"  /* Repair Anal Sphincter, Percutaneous Endoscopic Approach */
"0DQS0ZZ"  =  "1"  /* Repair Greater Omentum, Open Approach */
"0DQS3ZZ"  =  "1"  /* Repair Greater Omentum, Percutaneous Approach */
"0DQS4ZZ"  =  "1"  /* Repair Greater Omentum, Percutaneous Endoscopic Approach */
"0DQT0ZZ"  =  "1"  /* Repair Lesser Omentum, Open Approach */
"0DQT3ZZ"  =  "1"  /* Repair Lesser Omentum, Percutaneous Approach */
"0DQT4ZZ"  =  "1"  /* Repair Lesser Omentum, Percutaneous Endoscopic Approach */
"0DQV0ZZ"  =  "1"  /* Repair Mesentery, Open Approach */
"0DQV3ZZ"  =  "1"  /* Repair Mesentery, Percutaneous Approach */
"0DQV4ZZ"  =  "1"  /* Repair Mesentery, Percutaneous Endoscopic Approach */
"0DQW0ZZ"  =  "1"  /* Repair Peritoneum, Open Approach */
"0DQW3ZZ"  =  "1"  /* Repair Peritoneum, Percutaneous Approach */
"0DQW4ZZ"  =  "1"  /* Repair Peritoneum, Percutaneous Endoscopic Approach */
"0DR507Z"  =  "1"  /* Replacement of Esophagus with Autol Sub, Open Approach */
"0DR50JZ"  =  "1"  /* Replacement of Esophagus with Synth Sub, Open Approach */
"0DR50KZ"  =  "1"  /* Replacement of Esophagus with Nonaut Sub, Open Approach */
"0DR547Z"  =  "1"  /* Replacement of Esophagus with Autol Sub, Perc Endo Approach */
"0DR54JZ"  =  "1"  /* Replacement of Esophagus with Synth Sub, Perc Endo Approach */
"0DR54KZ"  =  "1"  /* Replacement of Esophagus with Nonaut Sub, Perc Endo Approach */
"0DR577Z"  =  "1"  /* Replacement of Esophagus with Autol Sub, Via Opening */
"0DR57JZ"  =  "1"  /* Replacement of Esophagus with Synth Sub, Via Opening */
"0DR57KZ"  =  "1"  /* Replacement of Esophagus with Nonaut Sub, Via Opening */
"0DR587Z"  =  "1"  /* Replacement of Esophagus with Autol Sub, Endo */
"0DR58JZ"  =  "1"  /* Replacement of Esophagus with Synthetic Substitute, Endo */
"0DR58KZ"  =  "1"  /* Replacement of Esophagus with Nonaut Sub, Endo */
"0DRR07Z"  =  "1"  /* Replacement of Anal Sphincter with Autol Sub, Open Approach */
"0DRR0JZ"  =  "1"  /* Replacement of Anal Sphincter with Synth Sub, Open Approach */
"0DRR0KZ"  =  "1"  /* Replacement of Anal Sphincter with Nonaut Sub, Open Approach */
"0DRR47Z"  =  "1"  /* Replace of Anal Sphincter with Autol Sub, Perc Endo Approach */
"0DRR4JZ"  =  "1"  /* Replace of Anal Sphincter with Synth Sub, Perc Endo Approach */
"0DRR4KZ"  =  "1"  /* Replace Anal Sphincter w Nonaut Sub, Perc Endo */
"0DRS07Z"  =  "1"  /* Replacement of Greater Omentum with Autol Sub, Open Approach */
"0DRS0JZ"  =  "1"  /* Replacement of Greater Omentum with Synth Sub, Open Approach */
"0DRS0KZ"  =  "1"  /* Replacement of Great Omentum with Nonaut Sub, Open Approach */
"0DRS47Z"  =  "1"  /* Replace of Great Omentum with Autol Sub, Perc Endo Approach */
"0DRS4JZ"  =  "1"  /* Replace of Great Omentum with Synth Sub, Perc Endo Approach */
"0DRS4KZ"  =  "1"  /* Replace of Great Omentum with Nonaut Sub, Perc Endo Approach */
"0DRT07Z"  =  "1"  /* Replacement of Lesser Omentum with Autol Sub, Open Approach */
"0DRT0JZ"  =  "1"  /* Replacement of Lesser Omentum with Synth Sub, Open Approach */
"0DRT0KZ"  =  "1"  /* Replacement of Lesser Omentum with Nonaut Sub, Open Approach */
"0DRT47Z"  =  "1"  /* Replace of Less Omentum with Autol Sub, Perc Endo Approach */
"0DRT4JZ"  =  "1"  /* Replace of Less Omentum with Synth Sub, Perc Endo Approach */
"0DRT4KZ"  =  "1"  /* Replace of Less Omentum with Nonaut Sub, Perc Endo Approach */
"0DRV07Z"  =  "1"  /* Replacement of Mesentery with Autol Sub, Open Approach */
"0DRV0JZ"  =  "1"  /* Replacement of Mesentery with Synth Sub, Open Approach */
"0DRV0KZ"  =  "1"  /* Replacement of Mesentery with Nonaut Sub, Open Approach */
"0DRV47Z"  =  "1"  /* Replacement of Mesentery with Autol Sub, Perc Endo Approach */
"0DRV4JZ"  =  "1"  /* Replacement of Mesentery with Synth Sub, Perc Endo Approach */
"0DRV4KZ"  =  "1"  /* Replacement of Mesentery with Nonaut Sub, Perc Endo Approach */
"0DRW07Z"  =  "1"  /* Replacement of Peritoneum with Autol Sub, Open Approach */
"0DRW0JZ"  =  "1"  /* Replacement of Peritoneum with Synth Sub, Open Approach */
"0DRW0KZ"  =  "1"  /* Replacement of Peritoneum with Nonaut Sub, Open Approach */
"0DRW47Z"  =  "1"  /* Replacement of Peritoneum with Autol Sub, Perc Endo Approach */
"0DRW4JZ"  =  "1"  /* Replacement of Peritoneum with Synth Sub, Perc Endo Approach */
"0DRW4KZ"  =  "1"  /* Replace of Peritoneum with Nonaut Sub, Perc Endo Approach */
"0DS50ZZ"  =  "1"  /* Reposition Esophagus, Open Approach */
"0DS54ZZ"  =  "1"  /* Reposition Esophagus, Percutaneous Endoscopic Approach */
"0DS57ZZ"  =  "1"  /* Reposition Esophagus, Via Natural or Artificial Opening */
"0DS58ZZ"  =  "1"  /* Reposition Esophagus, Endo */
"0DS5XZZ"  =  "1"  /* Reposition Esophagus, External Approach */
"0DS60ZZ"  =  "1"  /* Reposition Stomach, Open Approach */
"0DS64ZZ"  =  "1"  /* Reposition Stomach, Percutaneous Endoscopic Approach */
"0DS67ZZ"  =  "1"  /* Reposition Stomach, Via Natural or Artificial Opening */
"0DS68ZZ"  =  "1"  /* Reposition Stomach, Endo */
"0DS6XZZ"  =  "1"  /* Reposition Stomach, External Approach */
"0DS90ZZ"  =  "1"  /* Reposition Duodenum, Open Approach */
"0DS94ZZ"  =  "1"  /* Reposition Duodenum, Percutaneous Endoscopic Approach */
"0DS97ZZ"  =  "1"  /* Reposition Duodenum, Via Natural or Artificial Opening */
"0DS98ZZ"  =  "1"  /* Reposition Duodenum, Endo */
"0DSA0ZZ"  =  "1"  /* Reposition Jejunum, Open Approach */
"0DSA4ZZ"  =  "1"  /* Reposition Jejunum, Percutaneous Endoscopic Approach */
"0DSA7ZZ"  =  "1"  /* Reposition Jejunum, Via Natural or Artificial Opening */
"0DSA8ZZ"  =  "1"  /* Reposition Jejunum, Endo */
"0DSB0ZZ"  =  "1"  /* Reposition Ileum, Open Approach */
"0DSB4ZZ"  =  "1"  /* Reposition Ileum, Percutaneous Endoscopic Approach */
"0DSB7ZZ"  =  "1"  /* Reposition Ileum, Via Natural or Artificial Opening */
"0DSB8ZZ"  =  "1"  /* Reposition Ileum, Endo */
"0DSH0ZZ"  =  "1"  /* Reposition Cecum, Open Approach */
"0DSH4ZZ"  =  "1"  /* Reposition Cecum, Percutaneous Endoscopic Approach */
"0DSH7ZZ"  =  "1"  /* Reposition Cecum, Via Natural or Artificial Opening */
"0DSH8ZZ"  =  "1"  /* Reposition Cecum, Endo */
"0DSK0ZZ"  =  "1"  /* Reposition Ascending Colon, Open Approach */
"0DSK4ZZ"  =  "1"  /* Reposition Ascending Colon, Percutaneous Endoscopic Approach */
"0DSK7ZZ"  =  "1"  /* Reposition Ascending Colon, Via Opening */
"0DSK8ZZ"  =  "1"  /* Reposition Ascending Colon, Endo */
"0DSL0ZZ"  =  "1"  /* Reposition Transverse Colon, Open Approach */
"0DSL4ZZ"  =  "1"  /* Reposition Transverse Colon, Perc Endo Approach */
"0DSL7ZZ"  =  "1"  /* Reposition Transverse Colon, Via Opening */
"0DSL8ZZ"  =  "1"  /* Reposition Transverse Colon, Endo */
"0DSM0ZZ"  =  "1"  /* Reposition Descending Colon, Open Approach */
"0DSM4ZZ"  =  "1"  /* Reposition Descending Colon, Perc Endo Approach */
"0DSM7ZZ"  =  "1"  /* Reposition Descending Colon, Via Opening */
"0DSM8ZZ"  =  "1"  /* Reposition Descending Colon, Endo */
"0DSN0ZZ"  =  "1"  /* Reposition Sigmoid Colon, Open Approach */
"0DSN4ZZ"  =  "1"  /* Reposition Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DSN7ZZ"  =  "1"  /* Reposition Sigmoid Colon, Via Natural or Artificial Opening */
"0DSN8ZZ"  =  "1"  /* Reposition Sigmoid Colon, Endo */
"0DSP0ZZ"  =  "1"  /* Reposition Rectum, Open Approach */
"0DSP4ZZ"  =  "1"  /* Reposition Rectum, Percutaneous Endoscopic Approach */
"0DSP7ZZ"  =  "1"  /* Reposition Rectum, Via Natural or Artificial Opening */
"0DSP8ZZ"  =  "1"  /* Reposition Rectum, Endo */
"0DSQ0ZZ"  =  "1"  /* Reposition Anus, Open Approach */
"0DSQ4ZZ"  =  "1"  /* Reposition Anus, Percutaneous Endoscopic Approach */
"0DSQ7ZZ"  =  "1"  /* Reposition Anus, Via Natural or Artificial Opening */
"0DSQ8ZZ"  =  "1"  /* Reposition Anus, Endo */
"0DSQXZZ"  =  "1"  /* Reposition Anus, External Approach */
"0DT10ZZ"  =  "1"  /* Resection of Upper Esophagus, Open Approach */
"0DT14ZZ"  =  "1"  /* Resection of Upper Esophagus, Perc Endo Approach */
"0DT17ZZ"  =  "1"  /* Resection of Upper Esophagus, Via Opening */
"0DT18ZZ"  =  "1"  /* Resection of Upper Esophagus, Endo */
"0DT20ZZ"  =  "1"  /* Resection of Middle Esophagus, Open Approach */
"0DT24ZZ"  =  "1"  /* Resection of Middle Esophagus, Perc Endo Approach */
"0DT27ZZ"  =  "1"  /* Resection of Middle Esophagus, Via Opening */
"0DT28ZZ"  =  "1"  /* Resection of Middle Esophagus, Endo */
"0DT30ZZ"  =  "1"  /* Resection of Lower Esophagus, Open Approach */
"0DT34ZZ"  =  "1"  /* Resection of Lower Esophagus, Perc Endo Approach */
"0DT37ZZ"  =  "1"  /* Resection of Lower Esophagus, Via Opening */
"0DT38ZZ"  =  "1"  /* Resection of Lower Esophagus, Endo */
"0DT40ZZ"  =  "1"  /* Resection of Esophagogastric Junction, Open Approach */
"0DT44ZZ"  =  "1"  /* Resection of Esophagogastric Junction, Perc Endo Approach */
"0DT47ZZ"  =  "1"  /* Resection of Esophagogastric Junction, Via Opening */
"0DT48ZZ"  =  "1"  /* Resection of Esophagogastric Junction, Endo */
"0DT50ZZ"  =  "1"  /* Resection of Esophagus, Open Approach */
"0DT54ZZ"  =  "1"  /* Resection of Esophagus, Percutaneous Endoscopic Approach */
"0DT57ZZ"  =  "1"  /* Resection of Esophagus, Via Natural or Artificial Opening */
"0DT58ZZ"  =  "1"  /* Resection of Esophagus, Endo */
"0DT60ZZ"  =  "1"  /* Resection of Stomach, Open Approach */
"0DT64ZZ"  =  "1"  /* Resection of Stomach, Percutaneous Endoscopic Approach */
"0DT67ZZ"  =  "1"  /* Resection of Stomach, Via Natural or Artificial Opening */
"0DT68ZZ"  =  "1"  /* Resection of Stomach, Endo */
"0DT70ZZ"  =  "1"  /* Resection of Stomach, Pylorus, Open Approach */
"0DT74ZZ"  =  "1"  /* Resection of Stomach, Pylorus, Perc Endo Approach */
"0DT77ZZ"  =  "1"  /* Resection of Stomach, Pylorus, Via Opening */
"0DT78ZZ"  =  "1"  /* Resection of Stomach, Pylorus, Endo */
"0DT80ZZ"  =  "1"  /* Resection of Small Intestine, Open Approach */
"0DT84ZZ"  =  "1"  /* Resection of Small Intestine, Perc Endo Approach */
"0DT87ZZ"  =  "1"  /* Resection of Small Intestine, Via Opening */
"0DT88ZZ"  =  "1"  /* Resection of Small Intestine, Endo */
"0DT90ZZ"  =  "1"  /* Resection of Duodenum, Open Approach */
"0DT94ZZ"  =  "1"  /* Resection of Duodenum, Percutaneous Endoscopic Approach */
"0DT97ZZ"  =  "1"  /* Resection of Duodenum, Via Natural or Artificial Opening */
"0DT98ZZ"  =  "1"  /* Resection of Duodenum, Endo */
"0DTA0ZZ"  =  "1"  /* Resection of Jejunum, Open Approach */
"0DTA4ZZ"  =  "1"  /* Resection of Jejunum, Percutaneous Endoscopic Approach */
"0DTA7ZZ"  =  "1"  /* Resection of Jejunum, Via Natural or Artificial Opening */
"0DTA8ZZ"  =  "1"  /* Resection of Jejunum, Endo */
"0DTB0ZZ"  =  "1"  /* Resection of Ileum, Open Approach */
"0DTB4ZZ"  =  "1"  /* Resection of Ileum, Percutaneous Endoscopic Approach */
"0DTB7ZZ"  =  "1"  /* Resection of Ileum, Via Natural or Artificial Opening */
"0DTB8ZZ"  =  "1"  /* Resection of Ileum, Endo */
"0DTC0ZZ"  =  "1"  /* Resection of Ileocecal Valve, Open Approach */
"0DTC4ZZ"  =  "1"  /* Resection of Ileocecal Valve, Perc Endo Approach */
"0DTC7ZZ"  =  "1"  /* Resection of Ileocecal Valve, Via Opening */
"0DTC8ZZ"  =  "1"  /* Resection of Ileocecal Valve, Endo */
"0DTE0ZZ"  =  "1"  /* Resection of Large Intestine, Open Approach */
"0DTE4ZZ"  =  "1"  /* Resection of Large Intestine, Perc Endo Approach */
"0DTE7ZZ"  =  "1"  /* Resection of Large Intestine, Via Opening */
"0DTE8ZZ"  =  "1"  /* Resection of Large Intestine, Endo */
"0DTF0ZZ"  =  "1"  /* Resection of Right Large Intestine, Open Approach */
"0DTF4ZZ"  =  "1"  /* Resection of Right Large Intestine, Perc Endo Approach */
"0DTF7ZZ"  =  "1"  /* Resection of Right Large Intestine, Via Opening */
"0DTF8ZZ"  =  "1"  /* Resection of Right Large Intestine, Endo */
"0DTG0ZZ"  =  "1"  /* Resection of Left Large Intestine, Open Approach */
"0DTG4ZZ"  =  "1"  /* Resection of Left Large Intestine, Perc Endo Approach */
"0DTG7ZZ"  =  "1"  /* Resection of Left Large Intestine, Via Opening */
"0DTG8ZZ"  =  "1"  /* Resection of Left Large Intestine, Endo */
"0DTH0ZZ"  =  "1"  /* Resection of Cecum, Open Approach */
"0DTH4ZZ"  =  "1"  /* Resection of Cecum, Percutaneous Endoscopic Approach */
"0DTH7ZZ"  =  "1"  /* Resection of Cecum, Via Natural or Artificial Opening */
"0DTH8ZZ"  =  "1"  /* Resection of Cecum, Endo */
"0DTJ0ZZ"  =  "1"  /* Resection of Appendix, Open Approach */
"0DTJ4ZZ"  =  "1"  /* Resection of Appendix, Percutaneous Endoscopic Approach */
"0DTJ7ZZ"  =  "1"  /* Resection of Appendix, Via Natural or Artificial Opening */
"0DTJ8ZZ"  =  "1"  /* Resection of Appendix, Endo */
"0DTK0ZZ"  =  "1"  /* Resection of Ascending Colon, Open Approach */
"0DTK4ZZ"  =  "1"  /* Resection of Ascending Colon, Perc Endo Approach */
"0DTK7ZZ"  =  "1"  /* Resection of Ascending Colon, Via Opening */
"0DTK8ZZ"  =  "1"  /* Resection of Ascending Colon, Endo */
"0DTL0ZZ"  =  "1"  /* Resection of Transverse Colon, Open Approach */
"0DTL4ZZ"  =  "1"  /* Resection of Transverse Colon, Perc Endo Approach */
"0DTL7ZZ"  =  "1"  /* Resection of Transverse Colon, Via Opening */
"0DTL8ZZ"  =  "1"  /* Resection of Transverse Colon, Endo */
"0DTM0ZZ"  =  "1"  /* Resection of Descending Colon, Open Approach */
"0DTM4ZZ"  =  "1"  /* Resection of Descending Colon, Perc Endo Approach */
"0DTM7ZZ"  =  "1"  /* Resection of Descending Colon, Via Opening */
"0DTM8ZZ"  =  "1"  /* Resection of Descending Colon, Endo */
"0DTN0ZZ"  =  "1"  /* Resection of Sigmoid Colon, Open Approach */
"0DTN4ZZ"  =  "1"  /* Resection of Sigmoid Colon, Percutaneous Endoscopic Approach */
"0DTN7ZZ"  =  "1"  /* Resection of Sigmoid Colon, Via Opening */
"0DTN8ZZ"  =  "1"  /* Resection of Sigmoid Colon, Endo */
"0DTP0ZZ"  =  "1"  /* Resection of Rectum, Open Approach */
"0DTP4ZZ"  =  "1"  /* Resection of Rectum, Percutaneous Endoscopic Approach */
"0DTP7ZZ"  =  "1"  /* Resection of Rectum, Via Natural or Artificial Opening */
"0DTP8ZZ"  =  "1"  /* Resection of Rectum, Endo */
"0DTQ0ZZ"  =  "1"  /* Resection of Anus, Open Approach */
"0DTQ4ZZ"  =  "1"  /* Resection of Anus, Percutaneous Endoscopic Approach */
"0DTQ7ZZ"  =  "1"  /* Resection of Anus, Via Natural or Artificial Opening */
"0DTQ8ZZ"  =  "1"  /* Resection of Anus, Endo */
"0DTR0ZZ"  =  "1"  /* Resection of Anal Sphincter, Open Approach */
"0DTR4ZZ"  =  "1"  /* Resection of Anal Sphincter, Perc Endo Approach */
"0DTS0ZZ"  =  "1"  /* Resection of Greater Omentum, Open Approach */
"0DTS4ZZ"  =  "1"  /* Resection of Greater Omentum, Perc Endo Approach */
"0DTT0ZZ"  =  "1"  /* Resection of Lesser Omentum, Open Approach */
"0DTT4ZZ"  =  "1"  /* Resection of Lesser Omentum, Perc Endo Approach */
"0DU107Z"  =  "1"  /* Supplement Upper Esophagus with Autol Sub, Open Approach */
"0DU10JZ"  =  "1"  /* Supplement Upper Esophagus with Synth Sub, Open Approach */
"0DU10KZ"  =  "1"  /* Supplement Upper Esophagus with Nonaut Sub, Open Approach */
"0DU147Z"  =  "1"  /* Supplement Up Esophag with Autol Sub, Perc Endo Approach */
"0DU14JZ"  =  "1"  /* Supplement Up Esophag with Synth Sub, Perc Endo Approach */
"0DU14KZ"  =  "1"  /* Supplement Up Esophag with Nonaut Sub, Perc Endo Approach */
"0DU177Z"  =  "1"  /* Supplement Upper Esophagus with Autol Sub, Via Opening */
"0DU17JZ"  =  "1"  /* Supplement Upper Esophagus with Synth Sub, Via Opening */
"0DU17KZ"  =  "1"  /* Supplement Upper Esophagus with Nonaut Sub, Via Opening */
"0DU187Z"  =  "1"  /* Supplement Upper Esophagus with Autol Sub, Endo */
"0DU18JZ"  =  "1"  /* Supplement Upper Esophagus with Synthetic Substitute, Endo */
"0DU18KZ"  =  "1"  /* Supplement Upper Esophagus with Nonaut Sub, Endo */
"0DU207Z"  =  "1"  /* Supplement Middle Esophagus with Autol Sub, Open Approach */
"0DU20JZ"  =  "1"  /* Supplement Middle Esophagus with Synth Sub, Open Approach */
"0DU20KZ"  =  "1"  /* Supplement Middle Esophagus with Nonaut Sub, Open Approach */
"0DU247Z"  =  "1"  /* Supplement Mid Esophag with Autol Sub, Perc Endo Approach */
"0DU24JZ"  =  "1"  /* Supplement Mid Esophag with Synth Sub, Perc Endo Approach */
"0DU24KZ"  =  "1"  /* Supplement Mid Esophag with Nonaut Sub, Perc Endo Approach */
"0DU277Z"  =  "1"  /* Supplement Middle Esophagus with Autol Sub, Via Opening */
"0DU27JZ"  =  "1"  /* Supplement Middle Esophagus with Synth Sub, Via Opening */
"0DU27KZ"  =  "1"  /* Supplement Middle Esophagus with Nonaut Sub, Via Opening */
"0DU287Z"  =  "1"  /* Supplement Middle Esophagus with Autol Sub, Endo */
"0DU28JZ"  =  "1"  /* Supplement Middle Esophagus with Synthetic Substitute, Endo */
"0DU28KZ"  =  "1"  /* Supplement Middle Esophagus with Nonaut Sub, Endo */
"0DU307Z"  =  "1"  /* Supplement Lower Esophagus with Autol Sub, Open Approach */
"0DU30JZ"  =  "1"  /* Supplement Lower Esophagus with Synth Sub, Open Approach */
"0DU30KZ"  =  "1"  /* Supplement Lower Esophagus with Nonaut Sub, Open Approach */
"0DU347Z"  =  "1"  /* Supplement Low Esophag with Autol Sub, Perc Endo Approach */
"0DU34JZ"  =  "1"  /* Supplement Low Esophag with Synth Sub, Perc Endo Approach */
"0DU34KZ"  =  "1"  /* Supplement Low Esophag with Nonaut Sub, Perc Endo Approach */
"0DU377Z"  =  "1"  /* Supplement Lower Esophagus with Autol Sub, Via Opening */
"0DU37JZ"  =  "1"  /* Supplement Lower Esophagus with Synth Sub, Via Opening */
"0DU37KZ"  =  "1"  /* Supplement Lower Esophagus with Nonaut Sub, Via Opening */
"0DU387Z"  =  "1"  /* Supplement Lower Esophagus with Autol Sub, Endo */
"0DU38JZ"  =  "1"  /* Supplement Lower Esophagus with Synthetic Substitute, Endo */
"0DU38KZ"  =  "1"  /* Supplement Lower Esophagus with Nonaut Sub, Endo */
"0DU407Z"  =  "1"  /* Supplement Esophagast Junct with Autol Sub, Open Approach */
"0DU40JZ"  =  "1"  /* Supplement Esophagast Junct with Synth Sub, Open Approach */
"0DU40KZ"  =  "1"  /* Supplement Esophagast Junct with Nonaut Sub, Open Approach */
"0DU447Z"  =  "1"  /* Supplement Esophagast Junct w Autol Sub, Perc Endo */
"0DU44JZ"  =  "1"  /* Supplement Esophagast Junct w Synth Sub, Perc Endo */
"0DU44KZ"  =  "1"  /* Supplement Esophagast Junct w Nonaut Sub, Perc Endo */
"0DU477Z"  =  "1"  /* Supplement Esophagast Junct with Autol Sub, Via Opening */
"0DU47JZ"  =  "1"  /* Supplement Esophagast Junct with Synth Sub, Via Opening */
"0DU47KZ"  =  "1"  /* Supplement Esophagast Junct with Nonaut Sub, Via Opening */
"0DU487Z"  =  "1"  /* Supplement Esophagogastric Junction with Autol Sub, Endo */
"0DU48JZ"  =  "1"  /* Supplement Esophagogastric Junction with Synth Sub, Endo */
"0DU48KZ"  =  "1"  /* Supplement Esophagogastric Junction with Nonaut Sub, Endo */
"0DU507Z"  =  "1"  /* Supplement Esophagus with Autol Sub, Open Approach */
"0DU50JZ"  =  "1"  /* Supplement Esophagus with Synth Sub, Open Approach */
"0DU50KZ"  =  "1"  /* Supplement Esophagus with Nonaut Sub, Open Approach */
"0DU547Z"  =  "1"  /* Supplement Esophagus with Autol Sub, Perc Endo Approach */
"0DU54JZ"  =  "1"  /* Supplement Esophagus with Synth Sub, Perc Endo Approach */
"0DU54KZ"  =  "1"  /* Supplement Esophagus with Nonaut Sub, Perc Endo Approach */
"0DU577Z"  =  "1"  /* Supplement Esophagus with Autol Sub, Via Opening */
"0DU57JZ"  =  "1"  /* Supplement Esophagus with Synthetic Substitute, Via Opening */
"0DU57KZ"  =  "1"  /* Supplement Esophagus with Nonaut Sub, Via Opening */
"0DU587Z"  =  "1"  /* Supplement Esophagus with Autologous Tissue Substitute, Endo */
"0DU58JZ"  =  "1"  /* Supplement Esophagus with Synthetic Substitute, Endo */
"0DU58KZ"  =  "1"  /* Supplement Esophagus with Nonaut Sub, Endo */
"0DU607Z"  =  "1"  /* Supplement Stomach with Autol Sub, Open Approach */
"0DU60JZ"  =  "1"  /* Supplement Stomach with Synthetic Substitute, Open Approach */
"0DU60KZ"  =  "1"  /* Supplement Stomach with Nonaut Sub, Open Approach */
"0DU647Z"  =  "1"  /* Supplement Stomach with Autol Sub, Perc Endo Approach */
"0DU64JZ"  =  "1"  /* Supplement Stomach with Synth Sub, Perc Endo Approach */
"0DU64KZ"  =  "1"  /* Supplement Stomach with Nonaut Sub, Perc Endo Approach */
"0DU677Z"  =  "1"  /* Supplement Stomach with Autol Sub, Via Opening */
"0DU67JZ"  =  "1"  /* Supplement Stomach with Synthetic Substitute, Via Opening */
"0DU67KZ"  =  "1"  /* Supplement Stomach with Nonaut Sub, Via Opening */
"0DU687Z"  =  "1"  /* Supplement Stomach with Autologous Tissue Substitute, Endo */
"0DU68JZ"  =  "1"  /* Supplement Stomach with Synthetic Substitute, Endo */
"0DU68KZ"  =  "1"  /* Supplement Stomach with Nonaut Sub, Endo */
"0DU707Z"  =  "1"  /* Supplement Stomach, Pylorus with Autol Sub, Open Approach */
"0DU70JZ"  =  "1"  /* Supplement Stomach, Pylorus with Synth Sub, Open Approach */
"0DU70KZ"  =  "1"  /* Supplement Stomach, Pylorus with Nonaut Sub, Open Approach */
"0DU747Z"  =  "1"  /* Supplement Stomach, Pylrs with Autol Sub, Perc Endo Approach */
"0DU74JZ"  =  "1"  /* Supplement Stomach, Pylrs with Synth Sub, Perc Endo Approach */
"0DU74KZ"  =  "1"  /* Supplement Stomach, Pylrs w Nonaut Sub, Perc Endo */
"0DU777Z"  =  "1"  /* Supplement Stomach, Pylorus with Autol Sub, Via Opening */
"0DU77JZ"  =  "1"  /* Supplement Stomach, Pylorus with Synth Sub, Via Opening */
"0DU77KZ"  =  "1"  /* Supplement Stomach, Pylorus with Nonaut Sub, Via Opening */
"0DU787Z"  =  "1"  /* Supplement Stomach, Pylorus with Autol Sub, Endo */
"0DU78JZ"  =  "1"  /* Supplement Stomach, Pylorus with Synthetic Substitute, Endo */
"0DU78KZ"  =  "1"  /* Supplement Stomach, Pylorus with Nonaut Sub, Endo */
"0DU807Z"  =  "1"  /* Supplement Small Intestine with Autol Sub, Open Approach */
"0DU80JZ"  =  "1"  /* Supplement Small Intestine with Synth Sub, Open Approach */
"0DU80KZ"  =  "1"  /* Supplement Small Intestine with Nonaut Sub, Open Approach */
"0DU847Z"  =  "1"  /* Supplement Small Intest with Autol Sub, Perc Endo Approach */
"0DU84JZ"  =  "1"  /* Supplement Small Intest with Synth Sub, Perc Endo Approach */
"0DU84KZ"  =  "1"  /* Supplement Small Intest with Nonaut Sub, Perc Endo Approach */
"0DU877Z"  =  "1"  /* Supplement Small Intestine with Autol Sub, Via Opening */
"0DU87JZ"  =  "1"  /* Supplement Small Intestine with Synth Sub, Via Opening */
"0DU87KZ"  =  "1"  /* Supplement Small Intestine with Nonaut Sub, Via Opening */
"0DU887Z"  =  "1"  /* Supplement Small Intestine with Autol Sub, Endo */
"0DU88JZ"  =  "1"  /* Supplement Small Intestine with Synthetic Substitute, Endo */
"0DU88KZ"  =  "1"  /* Supplement Small Intestine with Nonaut Sub, Endo */
"0DU907Z"  =  "1"  /* Supplement Duodenum with Autol Sub, Open Approach */
"0DU90JZ"  =  "1"  /* Supplement Duodenum with Synthetic Substitute, Open Approach */
"0DU90KZ"  =  "1"  /* Supplement Duodenum with Nonaut Sub, Open Approach */
"0DU947Z"  =  "1"  /* Supplement Duodenum with Autol Sub, Perc Endo Approach */
"0DU94JZ"  =  "1"  /* Supplement Duodenum with Synth Sub, Perc Endo Approach */
"0DU94KZ"  =  "1"  /* Supplement Duodenum with Nonaut Sub, Perc Endo Approach */
"0DU977Z"  =  "1"  /* Supplement Duodenum with Autol Sub, Via Opening */
"0DU97JZ"  =  "1"  /* Supplement Duodenum with Synthetic Substitute, Via Opening */
"0DU97KZ"  =  "1"  /* Supplement Duodenum with Nonaut Sub, Via Opening */
"0DU987Z"  =  "1"  /* Supplement Duodenum with Autologous Tissue Substitute, Endo */
"0DU98JZ"  =  "1"  /* Supplement Duodenum with Synthetic Substitute, Endo */
"0DU98KZ"  =  "1"  /* Supplement Duodenum with Nonaut Sub, Endo */
"0DUA07Z"  =  "1"  /* Supplement Jejunum with Autol Sub, Open Approach */
"0DUA0JZ"  =  "1"  /* Supplement Jejunum with Synthetic Substitute, Open Approach */
"0DUA0KZ"  =  "1"  /* Supplement Jejunum with Nonaut Sub, Open Approach */
"0DUA47Z"  =  "1"  /* Supplement Jejunum with Autol Sub, Perc Endo Approach */
"0DUA4JZ"  =  "1"  /* Supplement Jejunum with Synth Sub, Perc Endo Approach */
"0DUA4KZ"  =  "1"  /* Supplement Jejunum with Nonaut Sub, Perc Endo Approach */
"0DUA77Z"  =  "1"  /* Supplement Jejunum with Autol Sub, Via Opening */
"0DUA7JZ"  =  "1"  /* Supplement Jejunum with Synthetic Substitute, Via Opening */
"0DUA7KZ"  =  "1"  /* Supplement Jejunum with Nonaut Sub, Via Opening */
"0DUA87Z"  =  "1"  /* Supplement Jejunum with Autologous Tissue Substitute, Endo */
"0DUA8JZ"  =  "1"  /* Supplement Jejunum with Synthetic Substitute, Endo */
"0DUA8KZ"  =  "1"  /* Supplement Jejunum with Nonaut Sub, Endo */
"0DUB07Z"  =  "1"  /* Supplement Ileum with Autol Sub, Open Approach */
"0DUB0JZ"  =  "1"  /* Supplement Ileum with Synthetic Substitute, Open Approach */
"0DUB0KZ"  =  "1"  /* Supplement Ileum with Nonaut Sub, Open Approach */
"0DUB47Z"  =  "1"  /* Supplement Ileum with Autol Sub, Perc Endo Approach */
"0DUB4JZ"  =  "1"  /* Supplement Ileum with Synth Sub, Perc Endo Approach */
"0DUB4KZ"  =  "1"  /* Supplement Ileum with Nonaut Sub, Perc Endo Approach */
"0DUB77Z"  =  "1"  /* Supplement Ileum with Autol Sub, Via Opening */
"0DUB7JZ"  =  "1"  /* Supplement Ileum with Synthetic Substitute, Via Opening */
"0DUB7KZ"  =  "1"  /* Supplement Ileum with Nonaut Sub, Via Opening */
"0DUB87Z"  =  "1"  /* Supplement Ileum with Autologous Tissue Substitute, Endo */
"0DUB8JZ"  =  "1"  /* Supplement Ileum with Synthetic Substitute, Endo */
"0DUB8KZ"  =  "1"  /* Supplement Ileum with Nonautologous Tissue Substitute, Endo */
"0DUC07Z"  =  "1"  /* Supplement Ileocecal Valve with Autol Sub, Open Approach */
"0DUC0JZ"  =  "1"  /* Supplement Ileocecal Valve with Synth Sub, Open Approach */
"0DUC0KZ"  =  "1"  /* Supplement Ileocecal Valve with Nonaut Sub, Open Approach */
"0DUC47Z"  =  "1"  /* Supplement Ileocecal Valve w Autol Sub, Perc Endo */
"0DUC4JZ"  =  "1"  /* Supplement Ileocecal Valve w Synth Sub, Perc Endo */
"0DUC4KZ"  =  "1"  /* Supplement Ileocecal Valve w Nonaut Sub, Perc Endo */
"0DUC77Z"  =  "1"  /* Supplement Ileocecal Valve with Autol Sub, Via Opening */
"0DUC7JZ"  =  "1"  /* Supplement Ileocecal Valve with Synth Sub, Via Opening */
"0DUC7KZ"  =  "1"  /* Supplement Ileocecal Valve with Nonaut Sub, Via Opening */
"0DUC87Z"  =  "1"  /* Supplement Ileocecal Valve with Autol Sub, Endo */
"0DUC8JZ"  =  "1"  /* Supplement Ileocecal Valve with Synthetic Substitute, Endo */
"0DUC8KZ"  =  "1"  /* Supplement Ileocecal Valve with Nonaut Sub, Endo */
"0DUE07Z"  =  "1"  /* Supplement Large Intestine with Autol Sub, Open Approach */
"0DUE0JZ"  =  "1"  /* Supplement Large Intestine with Synth Sub, Open Approach */
"0DUE0KZ"  =  "1"  /* Supplement Large Intestine with Nonaut Sub, Open Approach */
"0DUE47Z"  =  "1"  /* Supplement Lg Intest with Autol Sub, Perc Endo Approach */
"0DUE4JZ"  =  "1"  /* Supplement Lg Intest with Synth Sub, Perc Endo Approach */
"0DUE4KZ"  =  "1"  /* Supplement Lg Intest with Nonaut Sub, Perc Endo Approach */
"0DUE77Z"  =  "1"  /* Supplement Large Intestine with Autol Sub, Via Opening */
"0DUE7JZ"  =  "1"  /* Supplement Large Intestine with Synth Sub, Via Opening */
"0DUE7KZ"  =  "1"  /* Supplement Large Intestine with Nonaut Sub, Via Opening */
"0DUE87Z"  =  "1"  /* Supplement Large Intestine with Autol Sub, Endo */
"0DUE8JZ"  =  "1"  /* Supplement Large Intestine with Synthetic Substitute, Endo */
"0DUE8KZ"  =  "1"  /* Supplement Large Intestine with Nonaut Sub, Endo */
"0DUF07Z"  =  "1"  /* Supplement R Lg Intest with Autol Sub, Open Approach */
"0DUF0JZ"  =  "1"  /* Supplement R Lg Intest with Synth Sub, Open Approach */
"0DUF0KZ"  =  "1"  /* Supplement R Lg Intest with Nonaut Sub, Open Approach */
"0DUF47Z"  =  "1"  /* Supplement R Lg Intest with Autol Sub, Perc Endo Approach */
"0DUF4JZ"  =  "1"  /* Supplement R Lg Intest with Synth Sub, Perc Endo Approach */
"0DUF4KZ"  =  "1"  /* Supplement R Lg Intest with Nonaut Sub, Perc Endo Approach */
"0DUF77Z"  =  "1"  /* Supplement Right Large Intestine with Autol Sub, Via Opening */
"0DUF7JZ"  =  "1"  /* Supplement Right Large Intestine with Synth Sub, Via Opening */
"0DUF7KZ"  =  "1"  /* Supplement R Lg Intest with Nonaut Sub, Via Opening */
"0DUF87Z"  =  "1"  /* Supplement Right Large Intestine with Autol Sub, Endo */
"0DUF8JZ"  =  "1"  /* Supplement Right Large Intestine with Synth Sub, Endo */
"0DUF8KZ"  =  "1"  /* Supplement Right Large Intestine with Nonaut Sub, Endo */
"0DUG07Z"  =  "1"  /* Supplement L Lg Intest with Autol Sub, Open Approach */
"0DUG0JZ"  =  "1"  /* Supplement L Lg Intest with Synth Sub, Open Approach */
"0DUG0KZ"  =  "1"  /* Supplement L Lg Intest with Nonaut Sub, Open Approach */
"0DUG47Z"  =  "1"  /* Supplement L Lg Intest with Autol Sub, Perc Endo Approach */
"0DUG4JZ"  =  "1"  /* Supplement L Lg Intest with Synth Sub, Perc Endo Approach */
"0DUG4KZ"  =  "1"  /* Supplement L Lg Intest with Nonaut Sub, Perc Endo Approach */
"0DUG77Z"  =  "1"  /* Supplement Left Large Intestine with Autol Sub, Via Opening */
"0DUG7JZ"  =  "1"  /* Supplement Left Large Intestine with Synth Sub, Via Opening */
"0DUG7KZ"  =  "1"  /* Supplement Left Large Intestine with Nonaut Sub, Via Opening */
"0DUG87Z"  =  "1"  /* Supplement Left Large Intestine with Autol Sub, Endo */
"0DUG8JZ"  =  "1"  /* Supplement Left Large Intestine with Synth Sub, Endo */
"0DUG8KZ"  =  "1"  /* Supplement Left Large Intestine with Nonaut Sub, Endo */
"0DUH07Z"  =  "1"  /* Supplement Cecum with Autol Sub, Open Approach */
"0DUH0JZ"  =  "1"  /* Supplement Cecum with Synthetic Substitute, Open Approach */
"0DUH0KZ"  =  "1"  /* Supplement Cecum with Nonaut Sub, Open Approach */
"0DUH47Z"  =  "1"  /* Supplement Cecum with Autol Sub, Perc Endo Approach */
"0DUH4JZ"  =  "1"  /* Supplement Cecum with Synth Sub, Perc Endo Approach */
"0DUH4KZ"  =  "1"  /* Supplement Cecum with Nonaut Sub, Perc Endo Approach */
"0DUH77Z"  =  "1"  /* Supplement Cecum with Autol Sub, Via Opening */
"0DUH7JZ"  =  "1"  /* Supplement Cecum with Synthetic Substitute, Via Opening */
"0DUH7KZ"  =  "1"  /* Supplement Cecum with Nonaut Sub, Via Opening */
"0DUH87Z"  =  "1"  /* Supplement Cecum with Autologous Tissue Substitute, Endo */
"0DUH8JZ"  =  "1"  /* Supplement Cecum with Synthetic Substitute, Endo */
"0DUH8KZ"  =  "1"  /* Supplement Cecum with Nonautologous Tissue Substitute, Endo */
"0DUK07Z"  =  "1"  /* Supplement Ascending Colon with Autol Sub, Open Approach */
"0DUK0JZ"  =  "1"  /* Supplement Ascending Colon with Synth Sub, Open Approach */
"0DUK0KZ"  =  "1"  /* Supplement Ascending Colon with Nonaut Sub, Open Approach */
"0DUK47Z"  =  "1"  /* Supplement Asc Colon with Autol Sub, Perc Endo Approach */
"0DUK4JZ"  =  "1"  /* Supplement Asc Colon with Synth Sub, Perc Endo Approach */
"0DUK4KZ"  =  "1"  /* Supplement Asc Colon with Nonaut Sub, Perc Endo Approach */
"0DUK77Z"  =  "1"  /* Supplement Ascending Colon with Autol Sub, Via Opening */
"0DUK7JZ"  =  "1"  /* Supplement Ascending Colon with Synth Sub, Via Opening */
"0DUK7KZ"  =  "1"  /* Supplement Ascending Colon with Nonaut Sub, Via Opening */
"0DUK87Z"  =  "1"  /* Supplement Ascending Colon with Autol Sub, Endo */
"0DUK8JZ"  =  "1"  /* Supplement Ascending Colon with Synthetic Substitute, Endo */
"0DUK8KZ"  =  "1"  /* Supplement Ascending Colon with Nonaut Sub, Endo */
"0DUL07Z"  =  "1"  /* Supplement Transverse Colon with Autol Sub, Open Approach */
"0DUL0JZ"  =  "1"  /* Supplement Transverse Colon with Synth Sub, Open Approach */
"0DUL0KZ"  =  "1"  /* Supplement Transverse Colon with Nonaut Sub, Open Approach */
"0DUL47Z"  =  "1"  /* Supplement Trans Colon with Autol Sub, Perc Endo Approach */
"0DUL4JZ"  =  "1"  /* Supplement Trans Colon with Synth Sub, Perc Endo Approach */
"0DUL4KZ"  =  "1"  /* Supplement Trans Colon with Nonaut Sub, Perc Endo Approach */
"0DUL77Z"  =  "1"  /* Supplement Transverse Colon with Autol Sub, Via Opening */
"0DUL7JZ"  =  "1"  /* Supplement Transverse Colon with Synth Sub, Via Opening */
"0DUL7KZ"  =  "1"  /* Supplement Transverse Colon with Nonaut Sub, Via Opening */
"0DUL87Z"  =  "1"  /* Supplement Transverse Colon with Autol Sub, Endo */
"0DUL8JZ"  =  "1"  /* Supplement Transverse Colon with Synthetic Substitute, Endo */
"0DUL8KZ"  =  "1"  /* Supplement Transverse Colon with Nonaut Sub, Endo */
"0DUM07Z"  =  "1"  /* Supplement Descending Colon with Autol Sub, Open Approach */
"0DUM0JZ"  =  "1"  /* Supplement Descending Colon with Synth Sub, Open Approach */
"0DUM0KZ"  =  "1"  /* Supplement Descending Colon with Nonaut Sub, Open Approach */
"0DUM47Z"  =  "1"  /* Supplement Descend Colon with Autol Sub, Perc Endo Approach */
"0DUM4JZ"  =  "1"  /* Supplement Descend Colon with Synth Sub, Perc Endo Approach */
"0DUM4KZ"  =  "1"  /* Supplement Descend Colon with Nonaut Sub, Perc Endo Approach */
"0DUM77Z"  =  "1"  /* Supplement Descending Colon with Autol Sub, Via Opening */
"0DUM7JZ"  =  "1"  /* Supplement Descending Colon with Synth Sub, Via Opening */
"0DUM7KZ"  =  "1"  /* Supplement Descending Colon with Nonaut Sub, Via Opening */
"0DUM87Z"  =  "1"  /* Supplement Descending Colon with Autol Sub, Endo */
"0DUM8JZ"  =  "1"  /* Supplement Descending Colon with Synthetic Substitute, Endo */
"0DUM8KZ"  =  "1"  /* Supplement Descending Colon with Nonaut Sub, Endo */
"0DUN07Z"  =  "1"  /* Supplement Sigmoid Colon with Autol Sub, Open Approach */
"0DUN0JZ"  =  "1"  /* Supplement Sigmoid Colon with Synth Sub, Open Approach */
"0DUN0KZ"  =  "1"  /* Supplement Sigmoid Colon with Nonaut Sub, Open Approach */
"0DUN47Z"  =  "1"  /* Supplement Sigmoid Colon with Autol Sub, Perc Endo Approach */
"0DUN4JZ"  =  "1"  /* Supplement Sigmoid Colon with Synth Sub, Perc Endo Approach */
"0DUN4KZ"  =  "1"  /* Supplement Sigmoid Colon with Nonaut Sub, Perc Endo Approach */
"0DUN77Z"  =  "1"  /* Supplement Sigmoid Colon with Autol Sub, Via Opening */
"0DUN7JZ"  =  "1"  /* Supplement Sigmoid Colon with Synth Sub, Via Opening */
"0DUN7KZ"  =  "1"  /* Supplement Sigmoid Colon with Nonaut Sub, Via Opening */
"0DUN87Z"  =  "1"  /* Supplement Sigmoid Colon with Autol Sub, Endo */
"0DUN8JZ"  =  "1"  /* Supplement Sigmoid Colon with Synthetic Substitute, Endo */
"0DUN8KZ"  =  "1"  /* Supplement Sigmoid Colon with Nonaut Sub, Endo */
"0DUP07Z"  =  "1"  /* Supplement Rectum with Autol Sub, Open Approach */
"0DUP0JZ"  =  "1"  /* Supplement Rectum with Synthetic Substitute, Open Approach */
"0DUP0KZ"  =  "1"  /* Supplement Rectum with Nonaut Sub, Open Approach */
"0DUP47Z"  =  "1"  /* Supplement Rectum with Autol Sub, Perc Endo Approach */
"0DUP4JZ"  =  "1"  /* Supplement Rectum with Synth Sub, Perc Endo Approach */
"0DUP4KZ"  =  "1"  /* Supplement Rectum with Nonaut Sub, Perc Endo Approach */
"0DUP77Z"  =  "1"  /* Supplement Rectum with Autol Sub, Via Opening */
"0DUP7JZ"  =  "1"  /* Supplement Rectum with Synthetic Substitute, Via Opening */
"0DUP7KZ"  =  "1"  /* Supplement Rectum with Nonaut Sub, Via Opening */
"0DUP87Z"  =  "1"  /* Supplement Rectum with Autologous Tissue Substitute, Endo */
"0DUP8JZ"  =  "1"  /* Supplement Rectum with Synthetic Substitute, Endo */
"0DUP8KZ"  =  "1"  /* Supplement Rectum with Nonautologous Tissue Substitute, Endo */
"0DUQ07Z"  =  "1"  /* Supplement Anus with Autol Sub, Open Approach */
"0DUQ0JZ"  =  "1"  /* Supplement Anus with Synthetic Substitute, Open Approach */
"0DUQ0KZ"  =  "1"  /* Supplement Anus with Nonaut Sub, Open Approach */
"0DUQ47Z"  =  "1"  /* Supplement Anus with Autol Sub, Perc Endo Approach */
"0DUQ4JZ"  =  "1"  /* Supplement Anus with Synth Sub, Perc Endo Approach */
"0DUQ4KZ"  =  "1"  /* Supplement Anus with Nonaut Sub, Perc Endo Approach */
"0DUQ77Z"  =  "1"  /* Supplement Anus with Autol Sub, Via Opening */
"0DUQ7JZ"  =  "1"  /* Supplement Anus with Synthetic Substitute, Via Opening */
"0DUQ7KZ"  =  "1"  /* Supplement Anus with Nonaut Sub, Via Opening */
"0DUQ87Z"  =  "1"  /* Supplement Anus with Autologous Tissue Substitute, Endo */
"0DUQ8JZ"  =  "1"  /* Supplement Anus with Synthetic Substitute, Endo */
"0DUQ8KZ"  =  "1"  /* Supplement Anus with Nonautologous Tissue Substitute, Endo */
"0DUQX7Z"  =  "1"  /* Supplement Anus with Autol Sub, Extern Approach */
"0DUQXJZ"  =  "1"  /* Supplement Anus with Synthetic Substitute, External Approach */
"0DUQXKZ"  =  "1"  /* Supplement Anus with Nonaut Sub, Extern Approach */
"0DUR07Z"  =  "1"  /* Supplement Anal Sphincter with Autol Sub, Open Approach */
"0DUR0JZ"  =  "1"  /* Supplement Anal Sphincter with Synth Sub, Open Approach */
"0DUR0KZ"  =  "1"  /* Supplement Anal Sphincter with Nonaut Sub, Open Approach */
"0DUR47Z"  =  "1"  /* Supplement Anal Sphincter with Autol Sub, Perc Endo Approach */
"0DUR4JZ"  =  "1"  /* Supplement Anal Sphincter with Synth Sub, Perc Endo Approach */
"0DUR4KZ"  =  "1"  /* Supplement Anal Sphincter w Nonaut Sub, Perc Endo */
"0DUS07Z"  =  "1"  /* Supplement Greater Omentum with Autol Sub, Open Approach */
"0DUS0JZ"  =  "1"  /* Supplement Greater Omentum with Synth Sub, Open Approach */
"0DUS0KZ"  =  "1"  /* Supplement Greater Omentum with Nonaut Sub, Open Approach */
"0DUS47Z"  =  "1"  /* Supplement Great Omentum with Autol Sub, Perc Endo Approach */
"0DUS4JZ"  =  "1"  /* Supplement Great Omentum with Synth Sub, Perc Endo Approach */
"0DUS4KZ"  =  "1"  /* Supplement Great Omentum with Nonaut Sub, Perc Endo Approach */
"0DUT07Z"  =  "1"  /* Supplement Lesser Omentum with Autol Sub, Open Approach */
"0DUT0JZ"  =  "1"  /* Supplement Lesser Omentum with Synth Sub, Open Approach */
"0DUT0KZ"  =  "1"  /* Supplement Lesser Omentum with Nonaut Sub, Open Approach */
"0DUT47Z"  =  "1"  /* Supplement Lesser Omentum with Autol Sub, Perc Endo Approach */
"0DUT4JZ"  =  "1"  /* Supplement Lesser Omentum with Synth Sub, Perc Endo Approach */
"0DUT4KZ"  =  "1"  /* Supplement Less Omentum with Nonaut Sub, Perc Endo Approach */
"0DUV07Z"  =  "1"  /* Supplement Mesentery with Autol Sub, Open Approach */
"0DUV0JZ"  =  "1"  /* Supplement Mesentery with Synth Sub, Open Approach */
"0DUV0KZ"  =  "1"  /* Supplement Mesentery with Nonaut Sub, Open Approach */
"0DUV47Z"  =  "1"  /* Supplement Mesentery with Autol Sub, Perc Endo Approach */
"0DUV4JZ"  =  "1"  /* Supplement Mesentery with Synth Sub, Perc Endo Approach */
"0DUV4KZ"  =  "1"  /* Supplement Mesentery with Nonaut Sub, Perc Endo Approach */
"0DUW07Z"  =  "1"  /* Supplement Peritoneum with Autol Sub, Open Approach */
"0DUW0JZ"  =  "1"  /* Supplement Peritoneum with Synth Sub, Open Approach */
"0DUW0KZ"  =  "1"  /* Supplement Peritoneum with Nonaut Sub, Open Approach */
"0DUW47Z"  =  "1"  /* Supplement Peritoneum with Autol Sub, Perc Endo Approach */
"0DUW4JZ"  =  "1"  /* Supplement Peritoneum with Synth Sub, Perc Endo Approach */
"0DUW4KZ"  =  "1"  /* Supplement Peritoneum with Nonaut Sub, Perc Endo Approach */
"0DV10CZ"  =  "1"  /* Restriction of Up Esophag with Extralum Dev, Open Approach */
"0DV10DZ"  =  "1"  /* Restriction of Up Esophag with Intralum Dev, Open Approach */
"0DV10ZZ"  =  "1"  /* Restriction of Upper Esophagus, Open Approach */
"0DV13CZ"  =  "1"  /* Restriction of Up Esophag with Extralum Dev, Perc Approach */
"0DV13DZ"  =  "1"  /* Restriction of Up Esophag with Intralum Dev, Perc Approach */
"0DV13ZZ"  =  "1"  /* Restriction of Upper Esophagus, Percutaneous Approach */
"0DV14CZ"  =  "1"  /* Restrict of Up Esophag with Extralum Dev, Perc Endo Approach */
"0DV14DZ"  =  "1"  /* Restrict of Up Esophag with Intralum Dev, Perc Endo Approach */
"0DV14ZZ"  =  "1"  /* Restriction of Upper Esophagus, Perc Endo Approach */
"0DV17DZ"  =  "1"  /* Restriction of Up Esophag with Intralum Dev, Via Opening */
"0DV17ZZ"  =  "1"  /* Restriction of Upper Esophagus, Via Opening */
"0DV18DZ"  =  "1"  /* Restriction of Upper Esophagus with Intralum Dev, Endo */
"0DV18ZZ"  =  "1"  /* Restriction of Upper Esophagus, Endo */
"0DV20CZ"  =  "1"  /* Restriction of Mid Esophag with Extralum Dev, Open Approach */
"0DV20DZ"  =  "1"  /* Restriction of Mid Esophag with Intralum Dev, Open Approach */
"0DV20ZZ"  =  "1"  /* Restriction of Middle Esophagus, Open Approach */
"0DV23CZ"  =  "1"  /* Restriction of Mid Esophag with Extralum Dev, Perc Approach */
"0DV23DZ"  =  "1"  /* Restriction of Mid Esophag with Intralum Dev, Perc Approach */
"0DV23ZZ"  =  "1"  /* Restriction of Middle Esophagus, Percutaneous Approach */
"0DV24CZ"  =  "1"  /* Restrict Mid Esophag w Extralum Dev, Perc Endo */
"0DV24DZ"  =  "1"  /* Restrict Mid Esophag w Intralum Dev, Perc Endo */
"0DV24ZZ"  =  "1"  /* Restriction of Middle Esophagus, Perc Endo Approach */
"0DV27DZ"  =  "1"  /* Restriction of Mid Esophag with Intralum Dev, Via Opening */
"0DV27ZZ"  =  "1"  /* Restriction of Middle Esophagus, Via Opening */
"0DV28DZ"  =  "1"  /* Restriction of Middle Esophagus with Intralum Dev, Endo */
"0DV28ZZ"  =  "1"  /* Restriction of Middle Esophagus, Endo */
"0DV30CZ"  =  "1"  /* Restriction of Low Esophag with Extralum Dev, Open Approach */
"0DV30DZ"  =  "1"  /* Restriction of Low Esophag with Intralum Dev, Open Approach */
"0DV30ZZ"  =  "1"  /* Restriction of Lower Esophagus, Open Approach */
"0DV33CZ"  =  "1"  /* Restriction of Low Esophag with Extralum Dev, Perc Approach */
"0DV33DZ"  =  "1"  /* Restriction of Low Esophag with Intralum Dev, Perc Approach */
"0DV33ZZ"  =  "1"  /* Restriction of Lower Esophagus, Percutaneous Approach */
"0DV34CZ"  =  "1"  /* Restrict Low Esophag w Extralum Dev, Perc Endo */
"0DV34DZ"  =  "1"  /* Restrict Low Esophag w Intralum Dev, Perc Endo */
"0DV34ZZ"  =  "1"  /* Restriction of Lower Esophagus, Perc Endo Approach */
"0DV37DZ"  =  "1"  /* Restriction of Low Esophag with Intralum Dev, Via Opening */
"0DV37ZZ"  =  "1"  /* Restriction of Lower Esophagus, Via Opening */
"0DV38DZ"  =  "1"  /* Restriction of Lower Esophagus with Intralum Dev, Endo */
"0DV38ZZ"  =  "1"  /* Restriction of Lower Esophagus, Endo */
"0DV40CZ"  =  "1"  /* Restrict Esophagast Junct w Extralum Dev, Open */
"0DV40DZ"  =  "1"  /* Restrict Esophagast Junct w Intralum Dev, Open */
"0DV40ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Open Approach */
"0DV43CZ"  =  "1"  /* Restrict Esophagast Junct w Extralum Dev, Perc */
"0DV43DZ"  =  "1"  /* Restrict Esophagast Junct w Intralum Dev, Perc */
"0DV43ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Perc Approach */
"0DV44CZ"  =  "1"  /* Restrict Esophagast Junct w Extralum Dev, Perc Endo */
"0DV44DZ"  =  "1"  /* Restrict Esophagast Junct w Intralum Dev, Perc Endo */
"0DV44ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Perc Endo Approach */
"0DV47DZ"  =  "1"  /* Restrict of Esophagast Junct with Intralum Dev, Via Opening */
"0DV47ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Via Opening */
"0DV48DZ"  =  "1"  /* Restriction of Esophagast Junct with Intralum Dev, Endo */
"0DV48ZZ"  =  "1"  /* Restriction of Esophagogastric Junction, Endo */
"0DV50CZ"  =  "1"  /* Restriction of Esophagus with Extralum Dev, Open Approach */
"0DV50DZ"  =  "1"  /* Restriction of Esophagus with Intralum Dev, Open Approach */
"0DV50ZZ"  =  "1"  /* Restriction of Esophagus, Open Approach */
"0DV53CZ"  =  "1"  /* Restriction of Esophagus with Extralum Dev, Perc Approach */
"0DV53DZ"  =  "1"  /* Restriction of Esophagus with Intralum Dev, Perc Approach */
"0DV53ZZ"  =  "1"  /* Restriction of Esophagus, Percutaneous Approach */
"0DV54CZ"  =  "1"  /* Restriction of Esophag with Extralum Dev, Perc Endo Approach */
"0DV54DZ"  =  "1"  /* Restriction of Esophag with Intralum Dev, Perc Endo Approach */
"0DV54ZZ"  =  "1"  /* Restriction of Esophagus, Percutaneous Endoscopic Approach */
"0DV57DZ"  =  "1"  /* Restriction of Esophagus with Intralum Dev, Via Opening */
"0DV57ZZ"  =  "1"  /* Restriction of Esophagus, Via Natural or Artificial Opening */
"0DV58DZ"  =  "1"  /* Restriction of Esophagus with Intraluminal Device, Endo */
"0DV58ZZ"  =  "1"  /* Restriction of Esophagus, Endo */
"0DV60CZ"  =  "1"  /* Restriction of Stomach with Extralum Dev, Open Approach */
"0DV60DZ"  =  "1"  /* Restriction of Stomach with Intralum Dev, Open Approach */
"0DV60ZZ"  =  "1"  /* Restriction of Stomach, Open Approach */
"0DV63CZ"  =  "1"  /* Restriction of Stomach with Extralum Dev, Perc Approach */
"0DV63DZ"  =  "1"  /* Restriction of Stomach with Intralum Dev, Perc Approach */
"0DV63ZZ"  =  "1"  /* Restriction of Stomach, Percutaneous Approach */
"0DV64CZ"  =  "1"  /* Restriction of Stomach with Extralum Dev, Perc Endo Approach */
"0DV64DZ"  =  "1"  /* Restriction of Stomach with Intralum Dev, Perc Endo Approach */
"0DV64ZZ"  =  "1"  /* Restriction of Stomach, Percutaneous Endoscopic Approach */
"0DV67ZZ"  =  "1"  /* Restriction of Stomach, Via Natural or Artificial Opening */
"0DV68ZZ"  =  "1"  /* Restriction of Stomach, Endo */
"0DV70CZ"  =  "1"  /* Restrict of Stomach, Pylrs with Extralum Dev, Open Approach */
"0DV70DZ"  =  "1"  /* Restrict of Stomach, Pylrs with Intralum Dev, Open Approach */
"0DV70ZZ"  =  "1"  /* Restriction of Stomach, Pylorus, Open Approach */
"0DV73CZ"  =  "1"  /* Restrict of Stomach, Pylrs with Extralum Dev, Perc Approach */
"0DV73DZ"  =  "1"  /* Restrict of Stomach, Pylrs with Intralum Dev, Perc Approach */
"0DV73ZZ"  =  "1"  /* Restriction of Stomach, Pylorus, Percutaneous Approach */
"0DV74CZ"  =  "1"  /* Restrict Stomach, Pylrs w Extralum Dev, Perc Endo */
"0DV74DZ"  =  "1"  /* Restrict Stomach, Pylrs w Intralum Dev, Perc Endo */
"0DV74ZZ"  =  "1"  /* Restriction of Stomach, Pylorus, Perc Endo Approach */
"0DV77DZ"  =  "1"  /* Restriction of Stomach, Pylrs with Intralum Dev, Via Opening */
"0DV77ZZ"  =  "1"  /* Restriction of Stomach, Pylorus, Via Opening */
"0DV78DZ"  =  "1"  /* Restriction of Stomach, Pylorus with Intralum Dev, Endo */
"0DV78ZZ"  =  "1"  /* Restriction of Stomach, Pylorus, Endo */
"0DV80CZ"  =  "1"  /* Restriction of Small Intest with Extralum Dev, Open Approach */
"0DV80DZ"  =  "1"  /* Restriction of Small Intest with Intralum Dev, Open Approach */
"0DV80ZZ"  =  "1"  /* Restriction of Small Intestine, Open Approach */
"0DV83CZ"  =  "1"  /* Restriction of Small Intest with Extralum Dev, Perc Approach */
"0DV83DZ"  =  "1"  /* Restriction of Small Intest with Intralum Dev, Perc Approach */
"0DV83ZZ"  =  "1"  /* Restriction of Small Intestine, Percutaneous Approach */
"0DV84CZ"  =  "1"  /* Restrict Small Intest w Extralum Dev, Perc Endo */
"0DV84DZ"  =  "1"  /* Restrict Small Intest w Intralum Dev, Perc Endo */
"0DV84ZZ"  =  "1"  /* Restriction of Small Intestine, Perc Endo Approach */
"0DV87DZ"  =  "1"  /* Restriction of Small Intest with Intralum Dev, Via Opening */
"0DV87ZZ"  =  "1"  /* Restriction of Small Intestine, Via Opening */
"0DV88DZ"  =  "1"  /* Restriction of Small Intestine with Intralum Dev, Endo */
"0DV88ZZ"  =  "1"  /* Restriction of Small Intestine, Endo */
"0DV90CZ"  =  "1"  /* Restriction of Duodenum with Extralum Dev, Open Approach */
"0DV90DZ"  =  "1"  /* Restriction of Duodenum with Intralum Dev, Open Approach */
"0DV90ZZ"  =  "1"  /* Restriction of Duodenum, Open Approach */
"0DV93CZ"  =  "1"  /* Restriction of Duodenum with Extralum Dev, Perc Approach */
"0DV93DZ"  =  "1"  /* Restriction of Duodenum with Intralum Dev, Perc Approach */
"0DV93ZZ"  =  "1"  /* Restriction of Duodenum, Percutaneous Approach */
"0DV94CZ"  =  "1"  /* Restrict of Duodenum with Extralum Dev, Perc Endo Approach */
"0DV94DZ"  =  "1"  /* Restrict of Duodenum with Intralum Dev, Perc Endo Approach */
"0DV94ZZ"  =  "1"  /* Restriction of Duodenum, Percutaneous Endoscopic Approach */
"0DV97DZ"  =  "1"  /* Restriction of Duodenum with Intralum Dev, Via Opening */
"0DV97ZZ"  =  "1"  /* Restriction of Duodenum, Via Natural or Artificial Opening */
"0DV98DZ"  =  "1"  /* Restriction of Duodenum with Intraluminal Device, Endo */
"0DV98ZZ"  =  "1"  /* Restriction of Duodenum, Endo */
"0DVA0CZ"  =  "1"  /* Restriction of Jejunum with Extralum Dev, Open Approach */
"0DVA0DZ"  =  "1"  /* Restriction of Jejunum with Intralum Dev, Open Approach */
"0DVA0ZZ"  =  "1"  /* Restriction of Jejunum, Open Approach */
"0DVA3CZ"  =  "1"  /* Restriction of Jejunum with Extralum Dev, Perc Approach */
"0DVA3DZ"  =  "1"  /* Restriction of Jejunum with Intralum Dev, Perc Approach */
"0DVA3ZZ"  =  "1"  /* Restriction of Jejunum, Percutaneous Approach */
"0DVA4CZ"  =  "1"  /* Restriction of Jejunum with Extralum Dev, Perc Endo Approach */
"0DVA4DZ"  =  "1"  /* Restriction of Jejunum with Intralum Dev, Perc Endo Approach */
"0DVA4ZZ"  =  "1"  /* Restriction of Jejunum, Percutaneous Endoscopic Approach */
"0DVA7DZ"  =  "1"  /* Restriction of Jejunum with Intraluminal Device, Via Opening */
"0DVA7ZZ"  =  "1"  /* Restriction of Jejunum, Via Natural or Artificial Opening */
"0DVA8DZ"  =  "1"  /* Restriction of Jejunum with Intraluminal Device, Endo */
"0DVA8ZZ"  =  "1"  /* Restriction of Jejunum, Endo */
"0DVB0CZ"  =  "1"  /* Restriction of Ileum with Extraluminal Device, Open Approach */
"0DVB0DZ"  =  "1"  /* Restriction of Ileum with Intraluminal Device, Open Approach */
"0DVB0ZZ"  =  "1"  /* Restriction of Ileum, Open Approach */
"0DVB3CZ"  =  "1"  /* Restriction of Ileum with Extraluminal Device, Perc Approach */
"0DVB3DZ"  =  "1"  /* Restriction of Ileum with Intraluminal Device, Perc Approach */
"0DVB3ZZ"  =  "1"  /* Restriction of Ileum, Percutaneous Approach */
"0DVB4CZ"  =  "1"  /* Restriction of Ileum with Extralum Dev, Perc Endo Approach */
"0DVB4DZ"  =  "1"  /* Restriction of Ileum with Intralum Dev, Perc Endo Approach */
"0DVB4ZZ"  =  "1"  /* Restriction of Ileum, Percutaneous Endoscopic Approach */
"0DVB7DZ"  =  "1"  /* Restriction of Ileum with Intraluminal Device, Via Opening */
"0DVB7ZZ"  =  "1"  /* Restriction of Ileum, Via Natural or Artificial Opening */
"0DVB8DZ"  =  "1"  /* Restriction of Ileum with Intraluminal Device, Endo */
"0DVB8ZZ"  =  "1"  /* Restriction of Ileum, Endo */
"0DVC0CZ"  =  "1"  /* Restrict of Ileocecal Valve with Extralum Dev, Open Approach */
"0DVC0DZ"  =  "1"  /* Restrict of Ileocecal Valve with Intralum Dev, Open Approach */
"0DVC0ZZ"  =  "1"  /* Restriction of Ileocecal Valve, Open Approach */
"0DVC3CZ"  =  "1"  /* Restrict of Ileocecal Valve with Extralum Dev, Perc Approach */
"0DVC3DZ"  =  "1"  /* Restrict of Ileocecal Valve with Intralum Dev, Perc Approach */
"0DVC3ZZ"  =  "1"  /* Restriction of Ileocecal Valve, Percutaneous Approach */
"0DVC4CZ"  =  "1"  /* Restrict Ileocecal Valve w Extralum Dev, Perc Endo */
"0DVC4DZ"  =  "1"  /* Restrict Ileocecal Valve w Intralum Dev, Perc Endo */
"0DVC4ZZ"  =  "1"  /* Restriction of Ileocecal Valve, Perc Endo Approach */
"0DVC7DZ"  =  "1"  /* Restrict of Ileocecal Valve with Intralum Dev, Via Opening */
"0DVC7ZZ"  =  "1"  /* Restriction of Ileocecal Valve, Via Opening */
"0DVC8DZ"  =  "1"  /* Restriction of Ileocecal Valve with Intralum Dev, Endo */
"0DVC8ZZ"  =  "1"  /* Restriction of Ileocecal Valve, Endo */
"0DVE0CZ"  =  "1"  /* Restriction of Lg Intest with Extralum Dev, Open Approach */
"0DVE0DZ"  =  "1"  /* Restriction of Lg Intest with Intralum Dev, Open Approach */
"0DVE0ZZ"  =  "1"  /* Restriction of Large Intestine, Open Approach */
"0DVE3CZ"  =  "1"  /* Restriction of Lg Intest with Extralum Dev, Perc Approach */
"0DVE3DZ"  =  "1"  /* Restriction of Lg Intest with Intralum Dev, Perc Approach */
"0DVE3ZZ"  =  "1"  /* Restriction of Large Intestine, Percutaneous Approach */
"0DVE4CZ"  =  "1"  /* Restrict of Lg Intest with Extralum Dev, Perc Endo Approach */
"0DVE4DZ"  =  "1"  /* Restrict of Lg Intest with Intralum Dev, Perc Endo Approach */
"0DVE4ZZ"  =  "1"  /* Restriction of Large Intestine, Perc Endo Approach */
"0DVE7DZ"  =  "1"  /* Restriction of Lg Intest with Intralum Dev, Via Opening */
"0DVE7ZZ"  =  "1"  /* Restriction of Large Intestine, Via Opening */
"0DVE8DZ"  =  "1"  /* Restriction of Large Intestine with Intralum Dev, Endo */
"0DVE8ZZ"  =  "1"  /* Restriction of Large Intestine, Endo */
"0DVF0CZ"  =  "1"  /* Restriction of R Lg Intest with Extralum Dev, Open Approach */
"0DVF0DZ"  =  "1"  /* Restriction of R Lg Intest with Intralum Dev, Open Approach */
"0DVF0ZZ"  =  "1"  /* Restriction of Right Large Intestine, Open Approach */
"0DVF3CZ"  =  "1"  /* Restriction of R Lg Intest with Extralum Dev, Perc Approach */
"0DVF3DZ"  =  "1"  /* Restriction of R Lg Intest with Intralum Dev, Perc Approach */
"0DVF3ZZ"  =  "1"  /* Restriction of Right Large Intestine, Percutaneous Approach */
"0DVF4CZ"  =  "1"  /* Restrict R Lg Intest w Extralum Dev, Perc Endo */
"0DVF4DZ"  =  "1"  /* Restrict R Lg Intest w Intralum Dev, Perc Endo */
"0DVF4ZZ"  =  "1"  /* Restriction of Right Large Intestine, Perc Endo Approach */
"0DVF7DZ"  =  "1"  /* Restriction of R Lg Intest with Intralum Dev, Via Opening */
"0DVF7ZZ"  =  "1"  /* Restriction of Right Large Intestine, Via Opening */
"0DVF8DZ"  =  "1"  /* Restriction of Right Large Intestine with Intralum Dev, Endo */
"0DVF8ZZ"  =  "1"  /* Restriction of Right Large Intestine, Endo */
"0DVG0CZ"  =  "1"  /* Restriction of L Lg Intest with Extralum Dev, Open Approach */
"0DVG0DZ"  =  "1"  /* Restriction of L Lg Intest with Intralum Dev, Open Approach */
"0DVG0ZZ"  =  "1"  /* Restriction of Left Large Intestine, Open Approach */
"0DVG3CZ"  =  "1"  /* Restriction of L Lg Intest with Extralum Dev, Perc Approach */
"0DVG3DZ"  =  "1"  /* Restriction of L Lg Intest with Intralum Dev, Perc Approach */
"0DVG3ZZ"  =  "1"  /* Restriction of Left Large Intestine, Percutaneous Approach */
"0DVG4CZ"  =  "1"  /* Restrict L Lg Intest w Extralum Dev, Perc Endo */
"0DVG4DZ"  =  "1"  /* Restrict L Lg Intest w Intralum Dev, Perc Endo */
"0DVG4ZZ"  =  "1"  /* Restriction of Left Large Intestine, Perc Endo Approach */
"0DVG7DZ"  =  "1"  /* Restriction of L Lg Intest with Intralum Dev, Via Opening */
"0DVG7ZZ"  =  "1"  /* Restriction of Left Large Intestine, Via Opening */
"0DVG8DZ"  =  "1"  /* Restriction of Left Large Intestine with Intralum Dev, Endo */
"0DVG8ZZ"  =  "1"  /* Restriction of Left Large Intestine, Endo */
"0DVH0CZ"  =  "1"  /* Restriction of Cecum with Extraluminal Device, Open Approach */
"0DVH0DZ"  =  "1"  /* Restriction of Cecum with Intraluminal Device, Open Approach */
"0DVH0ZZ"  =  "1"  /* Restriction of Cecum, Open Approach */
"0DVH3CZ"  =  "1"  /* Restriction of Cecum with Extraluminal Device, Perc Approach */
"0DVH3DZ"  =  "1"  /* Restriction of Cecum with Intraluminal Device, Perc Approach */
"0DVH3ZZ"  =  "1"  /* Restriction of Cecum, Percutaneous Approach */
"0DVH4CZ"  =  "1"  /* Restriction of Cecum with Extralum Dev, Perc Endo Approach */
"0DVH4DZ"  =  "1"  /* Restriction of Cecum with Intralum Dev, Perc Endo Approach */
"0DVH4ZZ"  =  "1"  /* Restriction of Cecum, Percutaneous Endoscopic Approach */
"0DVH7DZ"  =  "1"  /* Restriction of Cecum with Intraluminal Device, Via Opening */
"0DVH7ZZ"  =  "1"  /* Restriction of Cecum, Via Natural or Artificial Opening */
"0DVH8DZ"  =  "1"  /* Restriction of Cecum with Intraluminal Device, Endo */
"0DVH8ZZ"  =  "1"  /* Restriction of Cecum, Endo */
"0DVK0CZ"  =  "1"  /* Restriction of Asc Colon with Extralum Dev, Open Approach */
"0DVK0DZ"  =  "1"  /* Restriction of Asc Colon with Intralum Dev, Open Approach */
"0DVK0ZZ"  =  "1"  /* Restriction of Ascending Colon, Open Approach */
"0DVK3CZ"  =  "1"  /* Restriction of Asc Colon with Extralum Dev, Perc Approach */
"0DVK3DZ"  =  "1"  /* Restriction of Asc Colon with Intralum Dev, Perc Approach */
"0DVK3ZZ"  =  "1"  /* Restriction of Ascending Colon, Percutaneous Approach */
"0DVK4CZ"  =  "1"  /* Restrict of Asc Colon with Extralum Dev, Perc Endo Approach */
"0DVK4DZ"  =  "1"  /* Restrict of Asc Colon with Intralum Dev, Perc Endo Approach */
"0DVK4ZZ"  =  "1"  /* Restriction of Ascending Colon, Perc Endo Approach */
"0DVK7DZ"  =  "1"  /* Restriction of Asc Colon with Intralum Dev, Via Opening */
"0DVK7ZZ"  =  "1"  /* Restriction of Ascending Colon, Via Opening */
"0DVK8DZ"  =  "1"  /* Restriction of Ascending Colon with Intralum Dev, Endo */
"0DVK8ZZ"  =  "1"  /* Restriction of Ascending Colon, Endo */
"0DVL0CZ"  =  "1"  /* Restriction of Trans Colon with Extralum Dev, Open Approach */
"0DVL0DZ"  =  "1"  /* Restriction of Trans Colon with Intralum Dev, Open Approach */
"0DVL0ZZ"  =  "1"  /* Restriction of Transverse Colon, Open Approach */
"0DVL3CZ"  =  "1"  /* Restriction of Trans Colon with Extralum Dev, Perc Approach */
"0DVL3DZ"  =  "1"  /* Restriction of Trans Colon with Intralum Dev, Perc Approach */
"0DVL3ZZ"  =  "1"  /* Restriction of Transverse Colon, Percutaneous Approach */
"0DVL4CZ"  =  "1"  /* Restrict Trans Colon w Extralum Dev, Perc Endo */
"0DVL4DZ"  =  "1"  /* Restrict Trans Colon w Intralum Dev, Perc Endo */
"0DVL4ZZ"  =  "1"  /* Restriction of Transverse Colon, Perc Endo Approach */
"0DVL7DZ"  =  "1"  /* Restriction of Trans Colon with Intralum Dev, Via Opening */
"0DVL7ZZ"  =  "1"  /* Restriction of Transverse Colon, Via Opening */
"0DVL8DZ"  =  "1"  /* Restriction of Transverse Colon with Intralum Dev, Endo */
"0DVL8ZZ"  =  "1"  /* Restriction of Transverse Colon, Endo */
"0DVM0CZ"  =  "1"  /* Restrict of Descend Colon with Extralum Dev, Open Approach */
"0DVM0DZ"  =  "1"  /* Restrict of Descend Colon with Intralum Dev, Open Approach */
"0DVM0ZZ"  =  "1"  /* Restriction of Descending Colon, Open Approach */
"0DVM3CZ"  =  "1"  /* Restrict of Descend Colon with Extralum Dev, Perc Approach */
"0DVM3DZ"  =  "1"  /* Restrict of Descend Colon with Intralum Dev, Perc Approach */
"0DVM3ZZ"  =  "1"  /* Restriction of Descending Colon, Percutaneous Approach */
"0DVM4CZ"  =  "1"  /* Restrict Descend Colon w Extralum Dev, Perc Endo */
"0DVM4DZ"  =  "1"  /* Restrict Descend Colon w Intralum Dev, Perc Endo */
"0DVM4ZZ"  =  "1"  /* Restriction of Descending Colon, Perc Endo Approach */
"0DVM7DZ"  =  "1"  /* Restriction of Descend Colon with Intralum Dev, Via Opening */
"0DVM7ZZ"  =  "1"  /* Restriction of Descending Colon, Via Opening */
"0DVM8DZ"  =  "1"  /* Restriction of Descending Colon with Intralum Dev, Endo */
"0DVM8ZZ"  =  "1"  /* Restriction of Descending Colon, Endo */
"0DVN0CZ"  =  "1"  /* Restrict of Sigmoid Colon with Extralum Dev, Open Approach */
"0DVN0DZ"  =  "1"  /* Restrict of Sigmoid Colon with Intralum Dev, Open Approach */
"0DVN0ZZ"  =  "1"  /* Restriction of Sigmoid Colon, Open Approach */
"0DVN3CZ"  =  "1"  /* Restrict of Sigmoid Colon with Extralum Dev, Perc Approach */
"0DVN3DZ"  =  "1"  /* Restrict of Sigmoid Colon with Intralum Dev, Perc Approach */
"0DVN3ZZ"  =  "1"  /* Restriction of Sigmoid Colon, Percutaneous Approach */
"0DVN4CZ"  =  "1"  /* Restrict Sigmoid Colon w Extralum Dev, Perc Endo */
"0DVN4DZ"  =  "1"  /* Restrict Sigmoid Colon w Intralum Dev, Perc Endo */
"0DVN4ZZ"  =  "1"  /* Restriction of Sigmoid Colon, Perc Endo Approach */
"0DVN7DZ"  =  "1"  /* Restriction of Sigmoid Colon with Intralum Dev, Via Opening */
"0DVN7ZZ"  =  "1"  /* Restriction of Sigmoid Colon, Via Opening */
"0DVN8DZ"  =  "1"  /* Restriction of Sigmoid Colon with Intraluminal Device, Endo */
"0DVN8ZZ"  =  "1"  /* Restriction of Sigmoid Colon, Endo */
"0DVP0CZ"  =  "1"  /* Restriction of Rectum with Extralum Dev, Open Approach */
"0DVP0DZ"  =  "1"  /* Restriction of Rectum with Intralum Dev, Open Approach */
"0DVP0ZZ"  =  "1"  /* Restriction of Rectum, Open Approach */
"0DVP3CZ"  =  "1"  /* Restriction of Rectum with Extralum Dev, Perc Approach */
"0DVP3DZ"  =  "1"  /* Restriction of Rectum with Intralum Dev, Perc Approach */
"0DVP3ZZ"  =  "1"  /* Restriction of Rectum, Percutaneous Approach */
"0DVP4CZ"  =  "1"  /* Restriction of Rectum with Extralum Dev, Perc Endo Approach */
"0DVP4DZ"  =  "1"  /* Restriction of Rectum with Intralum Dev, Perc Endo Approach */
"0DVP4ZZ"  =  "1"  /* Restriction of Rectum, Percutaneous Endoscopic Approach */
"0DVP7DZ"  =  "1"  /* Restriction of Rectum with Intraluminal Device, Via Opening */
"0DVP7ZZ"  =  "1"  /* Restriction of Rectum, Via Natural or Artificial Opening */
"0DVP8DZ"  =  "1"  /* Restriction of Rectum with Intraluminal Device, Endo */
"0DVP8ZZ"  =  "1"  /* Restriction of Rectum, Endo */
"0DVQ0CZ"  =  "1"  /* Restriction of Anus with Extraluminal Device, Open Approach */
"0DVQ0DZ"  =  "1"  /* Restriction of Anus with Intraluminal Device, Open Approach */
"0DVQ0ZZ"  =  "1"  /* Restriction of Anus, Open Approach */
"0DVQ3CZ"  =  "1"  /* Restriction of Anus with Extraluminal Device, Perc Approach */
"0DVQ3DZ"  =  "1"  /* Restriction of Anus with Intraluminal Device, Perc Approach */
"0DVQ3ZZ"  =  "1"  /* Restriction of Anus, Percutaneous Approach */
"0DVQ4CZ"  =  "1"  /* Restriction of Anus with Extralum Dev, Perc Endo Approach */
"0DVQ4DZ"  =  "1"  /* Restriction of Anus with Intralum Dev, Perc Endo Approach */
"0DVQ4ZZ"  =  "1"  /* Restriction of Anus, Percutaneous Endoscopic Approach */
"0DVQ7DZ"  =  "1"  /* Restriction of Anus with Intraluminal Device, Via Opening */
"0DVQ7ZZ"  =  "1"  /* Restriction of Anus, Via Natural or Artificial Opening */
"0DVQ8DZ"  =  "1"  /* Restriction of Anus with Intraluminal Device, Endo */
"0DVQ8ZZ"  =  "1"  /* Restriction of Anus, Endo */
"0DVQXCZ"  =  "1"  /* Restriction of Anus with Extralum Dev, Extern Approach */
"0DVQXDZ"  =  "1"  /* Restriction of Anus with Intralum Dev, Extern Approach */
"0DVQXZZ"  =  "1"  /* Restriction of Anus, External Approach */
"0DW000Z"  =  "1"  /* Revision of Drain Dev in Up Intest Tract, Open Approach */
"0DW002Z"  =  "1"  /* Revision of Monitor Dev in Up Intest Tract, Open Approach */
"0DW003Z"  =  "1"  /* Revision of Infusion Dev in Up Intest Tract, Open Approach */
"0DW007Z"  =  "1"  /* Revision of Autol Sub in Up Intest Tract, Open Approach */
"0DW00CZ"  =  "1"  /* Revision of Extralum Dev in Up Intest Tract, Open Approach */
"0DW00DZ"  =  "1"  /* Revision of Intralum Dev in Up Intest Tract, Open Approach */
"0DW00JZ"  =  "1"  /* Revision of Synth Sub in Up Intest Tract, Open Approach */
"0DW00KZ"  =  "1"  /* Revision of Nonaut Sub in Up Intest Tract, Open Approach */
"0DW00UZ"  =  "1"  /* Revision of Feeding Device in Up Intest Tract, Open Approach */
"0DW030Z"  =  "1"  /* Revision of Drain Dev in Up Intest Tract, Perc Approach */
"0DW032Z"  =  "1"  /* Revision of Monitor Dev in Up Intest Tract, Perc Approach */
"0DW033Z"  =  "1"  /* Revision of Infusion Dev in Up Intest Tract, Perc Approach */
"0DW037Z"  =  "1"  /* Revision of Autol Sub in Up Intest Tract, Perc Approach */
"0DW03CZ"  =  "1"  /* Revision of Extralum Dev in Up Intest Tract, Perc Approach */
"0DW03DZ"  =  "1"  /* Revision of Intralum Dev in Up Intest Tract, Perc Approach */
"0DW03JZ"  =  "1"  /* Revision of Synth Sub in Up Intest Tract, Perc Approach */
"0DW03KZ"  =  "1"  /* Revision of Nonaut Sub in Up Intest Tract, Perc Approach */
"0DW03UZ"  =  "1"  /* Revision of Feeding Device in Up Intest Tract, Perc Approach */
"0DW040Z"  =  "1"  /* Revision of Drain Dev in Up Intest Tract, Perc Endo Approach */
"0DW042Z"  =  "1"  /* Revise of Monitor Dev in Up Intest Tract, Perc Endo Approach */
"0DW043Z"  =  "1"  /* Revise Infusion Dev in Up Intest Tract, Perc Endo */
"0DW047Z"  =  "1"  /* Revision of Autol Sub in Up Intest Tract, Perc Endo Approach */
"0DW04CZ"  =  "1"  /* Revise Extralum Dev in Up Intest Tract, Perc Endo */
"0DW04DZ"  =  "1"  /* Revise Intralum Dev in Up Intest Tract, Perc Endo */
"0DW04JZ"  =  "1"  /* Revision of Synth Sub in Up Intest Tract, Perc Endo Approach */
"0DW04KZ"  =  "1"  /* Revise of Nonaut Sub in Up Intest Tract, Perc Endo Approach */
"0DW04UZ"  =  "1"  /* Revise of Feeding Dev in Up Intest Tract, Perc Endo Approach */
"0DW070Z"  =  "1"  /* Revision of Drainage Device in Up Intest Tract, Via Opening */
"0DW072Z"  =  "1"  /* Revision of Monitor Dev in Up Intest Tract, Via Opening */
"0DW073Z"  =  "1"  /* Revision of Infusion Device in Up Intest Tract, Via Opening */
"0DW077Z"  =  "1"  /* Revision of Autol Sub in Up Intest Tract, Via Opening */
"0DW07CZ"  =  "1"  /* Revision of Extralum Dev in Up Intest Tract, Via Opening */
"0DW07DZ"  =  "1"  /* Revision of Intralum Dev in Up Intest Tract, Via Opening */
"0DW07JZ"  =  "1"  /* Revision of Synth Sub in Up Intest Tract, Via Opening */
"0DW07KZ"  =  "1"  /* Revision of Nonaut Sub in Up Intest Tract, Via Opening */
"0DW07UZ"  =  "1"  /* Revision of Feeding Device in Up Intest Tract, Via Opening */
"0DW080Z"  =  "1"  /* Revision of Drainage Device in Upper Intestinal Tract, Endo */
"0DW082Z"  =  "1"  /* Revision of Monitoring Device in Up Intest Tract, Endo */
"0DW083Z"  =  "1"  /* Revision of Infusion Device in Upper Intestinal Tract, Endo */
"0DW087Z"  =  "1"  /* Revision of Autol Sub in Up Intest Tract, Endo */
"0DW08CZ"  =  "1"  /* Revision of Extraluminal Device in Up Intest Tract, Endo */
"0DW08DZ"  =  "1"  /* Revision of Intraluminal Device in Up Intest Tract, Endo */
"0DW08JZ"  =  "1"  /* Revision of Synthetic Substitute in Up Intest Tract, Endo */
"0DW08KZ"  =  "1"  /* Revision of Nonaut Sub in Up Intest Tract, Endo */
"0DW08UZ"  =  "1"  /* Revision of Feeding Device in Upper Intestinal Tract, Endo */
"0DW57DZ"  =  "1"  /* Revision of Intraluminal Device in Esophagus, Via Opening */
"0DW58DZ"  =  "1"  /* Revision of Intraluminal Device in Esophagus, Endo */
"0DW600Z"  =  "1"  /* Revision of Drainage Device in Stomach, Open Approach */
"0DW602Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Open Approach */
"0DW603Z"  =  "1"  /* Revision of Infusion Device in Stomach, Open Approach */
"0DW607Z"  =  "1"  /* Revision of Autol Sub in Stomach, Open Approach */
"0DW60CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Open Approach */
"0DW60DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Open Approach */
"0DW60JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Open Approach */
"0DW60KZ"  =  "1"  /* Revision of Nonaut Sub in Stomach, Open Approach */
"0DW60MZ"  =  "1"  /* Revision of Stimulator Lead in Stomach, Open Approach */
"0DW60UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Open Approach */
"0DW630Z"  =  "1"  /* Revision of Drainage Device in Stomach, Perc Approach */
"0DW632Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Perc Approach */
"0DW633Z"  =  "1"  /* Revision of Infusion Device in Stomach, Perc Approach */
"0DW637Z"  =  "1"  /* Revision of Autol Sub in Stomach, Perc Approach */
"0DW63CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Perc Approach */
"0DW63DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Perc Approach */
"0DW63JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Perc Approach */
"0DW63KZ"  =  "1"  /* Revision of Nonaut Sub in Stomach, Perc Approach */
"0DW63MZ"  =  "1"  /* Revision of Stimulator Lead in Stomach, Perc Approach */
"0DW63UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Percutaneous Approach */
"0DW640Z"  =  "1"  /* Revision of Drainage Device in Stomach, Perc Endo Approach */
"0DW642Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Perc Endo Approach */
"0DW643Z"  =  "1"  /* Revision of Infusion Device in Stomach, Perc Endo Approach */
"0DW647Z"  =  "1"  /* Revision of Autol Sub in Stomach, Perc Endo Approach */
"0DW64CZ"  =  "1"  /* Revision of Extralum Dev in Stomach, Perc Endo Approach */
"0DW64DZ"  =  "1"  /* Revision of Intralum Dev in Stomach, Perc Endo Approach */
"0DW64JZ"  =  "1"  /* Revision of Synth Sub in Stomach, Perc Endo Approach */
"0DW64KZ"  =  "1"  /* Revision of Nonaut Sub in Stomach, Perc Endo Approach */
"0DW64MZ"  =  "1"  /* Revision of Stimulator Lead in Stomach, Perc Endo Approach */
"0DW64UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Perc Endo Approach */
"0DW670Z"  =  "1"  /* Revision of Drainage Device in Stomach, Via Opening */
"0DW672Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Via Opening */
"0DW673Z"  =  "1"  /* Revision of Infusion Device in Stomach, Via Opening */
"0DW677Z"  =  "1"  /* Revision of Autol Sub in Stomach, Via Opening */
"0DW67CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Via Opening */
"0DW67DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Via Opening */
"0DW67JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Via Opening */
"0DW67KZ"  =  "1"  /* Revision of Nonaut Sub in Stomach, Via Opening */
"0DW67UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Via Opening */
"0DW680Z"  =  "1"  /* Revision of Drainage Device in Stomach, Endo */
"0DW682Z"  =  "1"  /* Revision of Monitoring Device in Stomach, Endo */
"0DW683Z"  =  "1"  /* Revision of Infusion Device in Stomach, Endo */
"0DW687Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Stomach, Endo */
"0DW68CZ"  =  "1"  /* Revision of Extraluminal Device in Stomach, Endo */
"0DW68DZ"  =  "1"  /* Revision of Intraluminal Device in Stomach, Endo */
"0DW68JZ"  =  "1"  /* Revision of Synthetic Substitute in Stomach, Endo */
"0DW68KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Stomach, Endo */
"0DW68UZ"  =  "1"  /* Revision of Feeding Device in Stomach, Endo */
"0DW807Z"  =  "1"  /* Revision of Autol Sub in Small Intest, Open Approach */
"0DW80JZ"  =  "1"  /* Revision of Synth Sub in Small Intest, Open Approach */
"0DW80KZ"  =  "1"  /* Revision of Nonaut Sub in Small Intest, Open Approach */
"0DW847Z"  =  "1"  /* Revision of Autol Sub in Small Intest, Perc Endo Approach */
"0DW84JZ"  =  "1"  /* Revision of Synth Sub in Small Intest, Perc Endo Approach */
"0DW84KZ"  =  "1"  /* Revision of Nonaut Sub in Small Intest, Perc Endo Approach */
"0DW877Z"  =  "1"  /* Revision of Autol Sub in Small Intest, Via Opening */
"0DW87JZ"  =  "1"  /* Revision of Synth Sub in Small Intest, Via Opening */
"0DW87KZ"  =  "1"  /* Revision of Nonaut Sub in Small Intest, Via Opening */
"0DW887Z"  =  "1"  /* Revision of Autol Sub in Small Intest, Endo */
"0DW88JZ"  =  "1"  /* Revision of Synthetic Substitute in Small Intestine, Endo */
"0DW88KZ"  =  "1"  /* Revision of Nonaut Sub in Small Intest, Endo */
"0DWD00Z"  =  "1"  /* Revision of Drain Dev in Low Intest Tract, Open Approach */
"0DWD02Z"  =  "1"  /* Revision of Monitor Dev in Low Intest Tract, Open Approach */
"0DWD03Z"  =  "1"  /* Revision of Infusion Dev in Low Intest Tract, Open Approach */
"0DWD07Z"  =  "1"  /* Revision of Autol Sub in Low Intest Tract, Open Approach */
"0DWD0CZ"  =  "1"  /* Revision of Extralum Dev in Low Intest Tract, Open Approach */
"0DWD0DZ"  =  "1"  /* Revision of Intralum Dev in Low Intest Tract, Open Approach */
"0DWD0JZ"  =  "1"  /* Revision of Synth Sub in Low Intest Tract, Open Approach */
"0DWD0KZ"  =  "1"  /* Revision of Nonaut Sub in Low Intest Tract, Open Approach */
"0DWD0UZ"  =  "1"  /* Revision of Feeding Dev in Low Intest Tract, Open Approach */
"0DWD30Z"  =  "1"  /* Revision of Drain Dev in Low Intest Tract, Perc Approach */
"0DWD32Z"  =  "1"  /* Revision of Monitor Dev in Low Intest Tract, Perc Approach */
"0DWD33Z"  =  "1"  /* Revision of Infusion Dev in Low Intest Tract, Perc Approach */
"0DWD37Z"  =  "1"  /* Revision of Autol Sub in Low Intest Tract, Perc Approach */
"0DWD3CZ"  =  "1"  /* Revision of Extralum Dev in Low Intest Tract, Perc Approach */
"0DWD3DZ"  =  "1"  /* Revision of Intralum Dev in Low Intest Tract, Perc Approach */
"0DWD3JZ"  =  "1"  /* Revision of Synth Sub in Low Intest Tract, Perc Approach */
"0DWD3KZ"  =  "1"  /* Revision of Nonaut Sub in Low Intest Tract, Perc Approach */
"0DWD3UZ"  =  "1"  /* Revision of Feeding Dev in Low Intest Tract, Perc Approach */
"0DWD40Z"  =  "1"  /* Revise of Drain Dev in Low Intest Tract, Perc Endo Approach */
"0DWD42Z"  =  "1"  /* Revise Monitor Dev in Low Intest Tract, Perc Endo */
"0DWD43Z"  =  "1"  /* Revise Infusion Dev in Low Intest Tract, Perc Endo */
"0DWD47Z"  =  "1"  /* Revise of Autol Sub in Low Intest Tract, Perc Endo Approach */
"0DWD4CZ"  =  "1"  /* Revise Extralum Dev in Low Intest Tract, Perc Endo */
"0DWD4DZ"  =  "1"  /* Revise Intralum Dev in Low Intest Tract, Perc Endo */
"0DWD4JZ"  =  "1"  /* Revise of Synth Sub in Low Intest Tract, Perc Endo Approach */
"0DWD4KZ"  =  "1"  /* Revise of Nonaut Sub in Low Intest Tract, Perc Endo Approach */
"0DWD4UZ"  =  "1"  /* Revise Feeding Dev in Low Intest Tract, Perc Endo */
"0DWD70Z"  =  "1"  /* Revision of Drainage Device in Low Intest Tract, Via Opening */
"0DWD72Z"  =  "1"  /* Revision of Monitor Dev in Low Intest Tract, Via Opening */
"0DWD73Z"  =  "1"  /* Revision of Infusion Device in Low Intest Tract, Via Opening */
"0DWD77Z"  =  "1"  /* Revision of Autol Sub in Low Intest Tract, Via Opening */
"0DWD7CZ"  =  "1"  /* Revision of Extralum Dev in Low Intest Tract, Via Opening */
"0DWD7DZ"  =  "1"  /* Revision of Intralum Dev in Low Intest Tract, Via Opening */
"0DWD7JZ"  =  "1"  /* Revision of Synth Sub in Low Intest Tract, Via Opening */
"0DWD7KZ"  =  "1"  /* Revision of Nonaut Sub in Low Intest Tract, Via Opening */
"0DWD7UZ"  =  "1"  /* Revision of Feeding Device in Low Intest Tract, Via Opening */
"0DWD80Z"  =  "1"  /* Revision of Drainage Device in Lower Intestinal Tract, Endo */
"0DWD82Z"  =  "1"  /* Revision of Monitoring Device in Low Intest Tract, Endo */
"0DWD83Z"  =  "1"  /* Revision of Infusion Device in Lower Intestinal Tract, Endo */
"0DWD87Z"  =  "1"  /* Revision of Autol Sub in Low Intest Tract, Endo */
"0DWD8CZ"  =  "1"  /* Revision of Extraluminal Device in Low Intest Tract, Endo */
"0DWD8DZ"  =  "1"  /* Revision of Intraluminal Device in Low Intest Tract, Endo */
"0DWD8JZ"  =  "1"  /* Revision of Synthetic Substitute in Low Intest Tract, Endo */
"0DWD8KZ"  =  "1"  /* Revision of Nonaut Sub in Low Intest Tract, Endo */
"0DWD8UZ"  =  "1"  /* Revision of Feeding Device in Lower Intestinal Tract, Endo */
"0DWE07Z"  =  "1"  /* Revision of Autol Sub in Lg Intest, Open Approach */
"0DWE0JZ"  =  "1"  /* Revision of Synthetic Substitute in Lg Intest, Open Approach */
"0DWE0KZ"  =  "1"  /* Revision of Nonaut Sub in Lg Intest, Open Approach */
"0DWE47Z"  =  "1"  /* Revision of Autol Sub in Lg Intest, Perc Endo Approach */
"0DWE4JZ"  =  "1"  /* Revision of Synth Sub in Lg Intest, Perc Endo Approach */
"0DWE4KZ"  =  "1"  /* Revision of Nonaut Sub in Lg Intest, Perc Endo Approach */
"0DWE77Z"  =  "1"  /* Revision of Autol Sub in Lg Intest, Via Opening */
"0DWE7JZ"  =  "1"  /* Revision of Synthetic Substitute in Lg Intest, Via Opening */
"0DWE7KZ"  =  "1"  /* Revision of Nonaut Sub in Lg Intest, Via Opening */
"0DWE87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Lg Intest, Endo */
"0DWE8JZ"  =  "1"  /* Revision of Synthetic Substitute in Large Intestine, Endo */
"0DWE8KZ"  =  "1"  /* Revision of Nonaut Sub in Lg Intest, Endo */
"0DWQ0LZ"  =  "1"  /* Revision of Artificial Sphincter in Anus, Open Approach */
"0DWQ3LZ"  =  "1"  /* Revision of Artificial Sphincter in Anus, Perc Approach */
"0DWQ4LZ"  =  "1"  /* Revision of Artificial Sphincter in Anus, Perc Endo Approach */
"0DWQ7LZ"  =  "1"  /* Revision of Artificial Sphincter in Anus, Via Opening */
"0DWQ8LZ"  =  "1"  /* Revision of Artificial Sphincter in Anus, Endo */
"0DWR0MZ"  =  "1"  /* Revision of Stimulator Lead in Anal Sphincter, Open Approach */
"0DWR3MZ"  =  "1"  /* Revision of Stimulator Lead in Anal Sphincter, Perc Approach */
"0DWR4MZ"  =  "1"  /* Revision of Stim Lead in Anal Sphincter, Perc Endo Approach */
"0DWU07Z"  =  "1"  /* Revision of Autol Sub in Omentum, Open Approach */
"0DWU0JZ"  =  "1"  /* Revision of Synthetic Substitute in Omentum, Open Approach */
"0DWU0KZ"  =  "1"  /* Revision of Nonaut Sub in Omentum, Open Approach */
"0DWU37Z"  =  "1"  /* Revision of Autol Sub in Omentum, Perc Approach */
"0DWU3JZ"  =  "1"  /* Revision of Synthetic Substitute in Omentum, Perc Approach */
"0DWU3KZ"  =  "1"  /* Revision of Nonaut Sub in Omentum, Perc Approach */
"0DWU47Z"  =  "1"  /* Revision of Autol Sub in Omentum, Perc Endo Approach */
"0DWU4JZ"  =  "1"  /* Revision of Synth Sub in Omentum, Perc Endo Approach */
"0DWU4KZ"  =  "1"  /* Revision of Nonaut Sub in Omentum, Perc Endo Approach */
"0DWV07Z"  =  "1"  /* Revision of Autol Sub in Mesentery, Open Approach */
"0DWV0JZ"  =  "1"  /* Revision of Synthetic Substitute in Mesentery, Open Approach */
"0DWV0KZ"  =  "1"  /* Revision of Nonaut Sub in Mesentery, Open Approach */
"0DWV37Z"  =  "1"  /* Revision of Autol Sub in Mesentery, Perc Approach */
"0DWV3JZ"  =  "1"  /* Revision of Synthetic Substitute in Mesentery, Perc Approach */
"0DWV3KZ"  =  "1"  /* Revision of Nonaut Sub in Mesentery, Perc Approach */
"0DWV47Z"  =  "1"  /* Revision of Autol Sub in Mesentery, Perc Endo Approach */
"0DWV4JZ"  =  "1"  /* Revision of Synth Sub in Mesentery, Perc Endo Approach */
"0DWV4KZ"  =  "1"  /* Revision of Nonaut Sub in Mesentery, Perc Endo Approach */
"0DWW07Z"  =  "1"  /* Revision of Autol Sub in Peritoneum, Open Approach */
"0DWW0JZ"  =  "1"  /* Revision of Synth Sub in Peritoneum, Open Approach */
"0DWW0KZ"  =  "1"  /* Revision of Nonaut Sub in Peritoneum, Open Approach */
"0DWW37Z"  =  "1"  /* Revision of Autol Sub in Peritoneum, Perc Approach */
"0DWW3JZ"  =  "1"  /* Revision of Synth Sub in Peritoneum, Perc Approach */
"0DWW3KZ"  =  "1"  /* Revision of Nonaut Sub in Peritoneum, Perc Approach */
"0DWW47Z"  =  "1"  /* Revision of Autol Sub in Peritoneum, Perc Endo Approach */
"0DWW4JZ"  =  "1"  /* Revision of Synth Sub in Peritoneum, Perc Endo Approach */
"0DWW4KZ"  =  "1"  /* Revision of Nonaut Sub in Peritoneum, Perc Endo Approach */
"0DX60Z5"  =  "1"  /* Transfer Stomach to Esophagus, Open Approach */
"0DX64Z5"  =  "1"  /* Transfer Stomach to Esophagus, Perc Endo Approach */
"0DX80Z5"  =  "1"  /* Transfer Small Intestine to Esophagus, Open Approach */
"0DX84Z5"  =  "1"  /* Transfer Small Intestine to Esophagus, Perc Endo Approach */
"0DXE0Z5"  =  "1"  /* Transfer Large Intestine to Esophagus, Open Approach */
"0DXE4Z5"  =  "1"  /* Transfer Large Intestine to Esophagus, Perc Endo Approach */
"0DY60Z0"  =  "1"  /* Transplantation of Stomach, Allogeneic, Open Approach */
"0DY60Z1"  =  "1"  /* Transplantation of Stomach, Syngeneic, Open Approach */
"0DY60Z2"  =  "1"  /* Transplantation of Stomach, Zooplastic, Open Approach */
"0DY80Z0"  =  "1"  /* Transplantation of Small Intestine, Allogen, Open Approach */
"0DY80Z1"  =  "1"  /* Transplantation of Small Intestine, Syngeneic, Open Approach */
"0DY80Z2"  =  "1"  /* Transplantation of Small Intestine, Zooplast, Open Approach */
"0DYE0Z0"  =  "1"  /* Transplantation of Large Intestine, Allogen, Open Approach */
"0DYE0Z1"  =  "1"  /* Transplantation of Large Intestine, Syngeneic, Open Approach */
"0DYE0Z2"  =  "1"  /* Transplantation of Large Intestine, Zooplast, Open Approach */
"0F140D3"  =  "1"  /* Bypass Gallbladder to Duoden w Intralum Dev, Open */
"0F140D4"  =  "1"  /* Bypass Gallbladder to Stomach w Intralum Dev, Open */
"0F140D5"  =  "1"  /* Bypass Gallbladder to R Hep Duc w Intralum Dev, Open */
"0F140D6"  =  "1"  /* Bypass Gallbladder to L Hep Duc w Intralum Dev, Open */
"0F140D7"  =  "1"  /* Bypass Gallbladder to Caud Duct w Intralum Dev, Open */
"0F140D8"  =  "1"  /* Bypass Gallbladder to Cyst Duct w Intralum Dev, Open */
"0F140D9"  =  "1"  /* Bypass Gallbladder to CBD with Intralum Dev, Open Approach */
"0F140DB"  =  "1"  /* Bypass Gallbladder to Sm Int w Intralum Dev, Open */
"0F140Z3"  =  "1"  /* Bypass Gallbladder to Duodenum, Open Approach */
"0F140Z4"  =  "1"  /* Bypass Gallbladder to Stomach, Open Approach */
"0F140Z5"  =  "1"  /* Bypass Gallbladder to Right Hepatic Duct, Open Approach */
"0F140Z6"  =  "1"  /* Bypass Gallbladder to Left Hepatic Duct, Open Approach */
"0F140Z7"  =  "1"  /* Bypass Gallbladder to Caudate Hepatic Duct, Open Approach */
"0F140Z8"  =  "1"  /* Bypass Gallbladder to Cystic Duct, Open Approach */
"0F140Z9"  =  "1"  /* Bypass Gallbladder to Common Bile Duct, Open Approach */
"0F140ZB"  =  "1"  /* Bypass Gallbladder to Small Intestine, Open Approach */
"0F144D3"  =  "1"  /* Bypass Gallbladder to Duoden w Intralum Dev, Perc Endo */
"0F144D4"  =  "1"  /* Bypass Gallbladder to Stomach w Intralum Dev, Perc Endo */
"0F144D5"  =  "1"  /* Bypass Gallbladder to R Hep Duc w Intralum Dev, Perc Endo */
"0F144D6"  =  "1"  /* Bypass Gallbladder to L Hep Duc w Intralum Dev, Perc Endo */
"0F144D7"  =  "1"  /* Bypass Gallbladder to Caud Duct w Intralum Dev, Perc Endo */
"0F144D8"  =  "1"  /* Bypass Gallbladder to Cyst Duct w Intralum Dev, Perc Endo */
"0F144D9"  =  "1"  /* Bypass Gallbladder to CBD w Intralum Dev, Perc Endo */
"0F144DB"  =  "1"  /* Bypass Gallbladder to Sm Int w Intralum Dev, Perc Endo */
"0F144Z3"  =  "1"  /* Bypass Gallbladder to Duodenum, Perc Endo Approach */
"0F144Z4"  =  "1"  /* Bypass Gallbladder to Stomach, Perc Endo Approach */
"0F144Z5"  =  "1"  /* Bypass Gallbladder to Right Hepatic Duct, Perc Endo Approach */
"0F144Z6"  =  "1"  /* Bypass Gallbladder to Left Hepatic Duct, Perc Endo Approach */
"0F144Z7"  =  "1"  /* Bypass Gallbladder to Caud Duct, Perc Endo Approach */
"0F144Z8"  =  "1"  /* Bypass Gallbladder to Cystic Duct, Perc Endo Approach */
"0F144Z9"  =  "1"  /* Bypass Gallbladder to Common Bile Duct, Perc Endo Approach */
"0F144ZB"  =  "1"  /* Bypass Gallbladder to Small Intestine, Perc Endo Approach */
"0F150D3"  =  "1"  /* Bypass R Hepatic Duct to Duoden w Intralum Dev, Open */
"0F150D4"  =  "1"  /* Bypass R Hepatic Duct to Stomach w Intralum Dev, Open */
"0F150D5"  =  "1"  /* Bypass R Hepatic Duct to R Hep Duc w Intralum Dev, Open */
"0F150D6"  =  "1"  /* Bypass R Hepatic Duct to L Hep Duc w Intralum Dev, Open */
"0F150D7"  =  "1"  /* Bypass R Hepatic Duct to Caud Duct w Intralum Dev, Open */
"0F150D8"  =  "1"  /* Bypass R Hepatic Duct to Cyst Duct w Intralum Dev, Open */
"0F150D9"  =  "1"  /* Bypass R Hepatic Duct to CBD w Intralum Dev, Open */
"0F150DB"  =  "1"  /* Bypass R Hepatic Duct to Sm Int w Intralum Dev, Open */
"0F150Z3"  =  "1"  /* Bypass Right Hepatic Duct to Duodenum, Open Approach */
"0F150Z4"  =  "1"  /* Bypass Right Hepatic Duct to Stomach, Open Approach */
"0F150Z5"  =  "1"  /* Bypass Right Hepatic Duct to R Hep Duc, Open Approach */
"0F150Z6"  =  "1"  /* Bypass Right Hepatic Duct to L Hep Duc, Open Approach */
"0F150Z7"  =  "1"  /* Bypass Right Hepatic Duct to Caud Duct, Open Approach */
"0F150Z8"  =  "1"  /* Bypass Right Hepatic Duct to Cystic Duct, Open Approach */
"0F150Z9"  =  "1"  /* Bypass Right Hepatic Duct to Common Bile Duct, Open Approach */
"0F150ZB"  =  "1"  /* Bypass Right Hepatic Duct to Small Intestine, Open Approach */
"0F154D3"  =  "1"  /* Bypass R Hepatic Duct to Duoden w Intralum Dev, Perc Endo */
"0F154D4"  =  "1"  /* Bypass R Hepatic Duct to Stomach w Intralum Dev, Perc Endo */
"0F154D5"  =  "1"  /* Bypass R Hepatic Duct to R Hep Duc w Intralum Dev, Perc Endo */
"0F154D6"  =  "1"  /* Bypass R Hepatic Duct to L Hep Duc w Intralum Dev, Perc Endo */
"0F154D7"  =  "1"  /* Bypass R Hepatic Duct to Caud Duct w Intralum Dev, Perc Endo */
"0F154D8"  =  "1"  /* Bypass R Hepatic Duct to Cyst Duct w Intralum Dev, Perc Endo */
"0F154D9"  =  "1"  /* Bypass R Hepatic Duct to CBD w Intralum Dev, Perc Endo */
"0F154DB"  =  "1"  /* Bypass R Hepatic Duct to Sm Int w Intralum Dev, Perc Endo */
"0F154Z3"  =  "1"  /* Bypass Right Hepatic Duct to Duodenum, Perc Endo Approach */
"0F154Z4"  =  "1"  /* Bypass Right Hepatic Duct to Stomach, Perc Endo Approach */
"0F154Z5"  =  "1"  /* Bypass Right Hepatic Duct to R Hep Duc, Perc Endo Approach */
"0F154Z6"  =  "1"  /* Bypass Right Hepatic Duct to L Hep Duc, Perc Endo Approach */
"0F154Z7"  =  "1"  /* Bypass Right Hepatic Duct to Caud Duct, Perc Endo Approach */
"0F154Z8"  =  "1"  /* Bypass Right Hepatic Duct to Cystic Duct, Perc Endo Approach */
"0F154Z9"  =  "1"  /* Bypass Right Hepatic Duct to CBD, Perc Endo Approach */
"0F154ZB"  =  "1"  /* Bypass Right Hepatic Duct to Sm Int, Perc Endo Approach */
"0F160D3"  =  "1"  /* Bypass L Hepatic Duct to Duoden w Intralum Dev, Open */
"0F160D4"  =  "1"  /* Bypass L Hepatic Duct to Stomach w Intralum Dev, Open */
"0F160D5"  =  "1"  /* Bypass L Hepatic Duct to R Hep Duc w Intralum Dev, Open */
"0F160D6"  =  "1"  /* Bypass L Hepatic Duct to L Hep Duc w Intralum Dev, Open */
"0F160D7"  =  "1"  /* Bypass L Hepatic Duct to Caud Duct w Intralum Dev, Open */
"0F160D8"  =  "1"  /* Bypass L Hepatic Duct to Cyst Duct w Intralum Dev, Open */
"0F160D9"  =  "1"  /* Bypass L Hepatic Duct to CBD w Intralum Dev, Open */
"0F160DB"  =  "1"  /* Bypass L Hepatic Duct to Sm Int w Intralum Dev, Open */
"0F160Z3"  =  "1"  /* Bypass Left Hepatic Duct to Duodenum, Open Approach */
"0F160Z4"  =  "1"  /* Bypass Left Hepatic Duct to Stomach, Open Approach */
"0F160Z5"  =  "1"  /* Bypass Left Hepatic Duct to R Hep Duc, Open Approach */
"0F160Z6"  =  "1"  /* Bypass Left Hepatic Duct to Left Hepatic Duct, Open Approach */
"0F160Z7"  =  "1"  /* Bypass Left Hepatic Duct to Caud Duct, Open Approach */
"0F160Z8"  =  "1"  /* Bypass Left Hepatic Duct to Cystic Duct, Open Approach */
"0F160Z9"  =  "1"  /* Bypass Left Hepatic Duct to Common Bile Duct, Open Approach */
"0F160ZB"  =  "1"  /* Bypass Left Hepatic Duct to Small Intestine, Open Approach */
"0F164D3"  =  "1"  /* Bypass L Hepatic Duct to Duoden w Intralum Dev, Perc Endo */
"0F164D4"  =  "1"  /* Bypass L Hepatic Duct to Stomach w Intralum Dev, Perc Endo */
"0F164D5"  =  "1"  /* Bypass L Hepatic Duct to R Hep Duc w Intralum Dev, Perc Endo */
"0F164D6"  =  "1"  /* Bypass L Hepatic Duct to L Hep Duc w Intralum Dev, Perc Endo */
"0F164D7"  =  "1"  /* Bypass L Hepatic Duct to Caud Duct w Intralum Dev, Perc Endo */
"0F164D8"  =  "1"  /* Bypass L Hepatic Duct to Cyst Duct w Intralum Dev, Perc Endo */
"0F164D9"  =  "1"  /* Bypass L Hepatic Duct to CBD w Intralum Dev, Perc Endo */
"0F164DB"  =  "1"  /* Bypass L Hepatic Duct to Sm Int w Intralum Dev, Perc Endo */
"0F164Z3"  =  "1"  /* Bypass Left Hepatic Duct to Duodenum, Perc Endo Approach */
"0F164Z4"  =  "1"  /* Bypass Left Hepatic Duct to Stomach, Perc Endo Approach */
"0F164Z5"  =  "1"  /* Bypass Left Hepatic Duct to R Hep Duc, Perc Endo Approach */
"0F164Z6"  =  "1"  /* Bypass Left Hepatic Duct to L Hep Duc, Perc Endo Approach */
"0F164Z7"  =  "1"  /* Bypass Left Hepatic Duct to Caud Duct, Perc Endo Approach */
"0F164Z8"  =  "1"  /* Bypass Left Hepatic Duct to Cystic Duct, Perc Endo Approach */
"0F164Z9"  =  "1"  /* Bypass Left Hepatic Duct to CBD, Perc Endo Approach */
"0F164ZB"  =  "1"  /* Bypass Left Hepatic Duct to Sm Int, Perc Endo Approach */
"0F180D3"  =  "1"  /* Bypass Cystic Duct to Duoden w Intralum Dev, Open */
"0F180D4"  =  "1"  /* Bypass Cystic Duct to Stomach w Intralum Dev, Open */
"0F180D5"  =  "1"  /* Bypass Cystic Duct to R Hep Duc w Intralum Dev, Open */
"0F180D6"  =  "1"  /* Bypass Cystic Duct to L Hep Duc w Intralum Dev, Open */
"0F180D7"  =  "1"  /* Bypass Cystic Duct to Caud Duct w Intralum Dev, Open */
"0F180D8"  =  "1"  /* Bypass Cystic Duct to Cyst Duct w Intralum Dev, Open */
"0F180D9"  =  "1"  /* Bypass Cystic Duct to CBD with Intralum Dev, Open Approach */
"0F180DB"  =  "1"  /* Bypass Cystic Duct to Sm Int w Intralum Dev, Open */
"0F180Z3"  =  "1"  /* Bypass Cystic Duct to Duodenum, Open Approach */
"0F180Z4"  =  "1"  /* Bypass Cystic Duct to Stomach, Open Approach */
"0F180Z5"  =  "1"  /* Bypass Cystic Duct to Right Hepatic Duct, Open Approach */
"0F180Z6"  =  "1"  /* Bypass Cystic Duct to Left Hepatic Duct, Open Approach */
"0F180Z7"  =  "1"  /* Bypass Cystic Duct to Caudate Hepatic Duct, Open Approach */
"0F180Z8"  =  "1"  /* Bypass Cystic Duct to Cystic Duct, Open Approach */
"0F180Z9"  =  "1"  /* Bypass Cystic Duct to Common Bile Duct, Open Approach */
"0F180ZB"  =  "1"  /* Bypass Cystic Duct to Small Intestine, Open Approach */
"0F184D3"  =  "1"  /* Bypass Cystic Duct to Duoden w Intralum Dev, Perc Endo */
"0F184D4"  =  "1"  /* Bypass Cystic Duct to Stomach w Intralum Dev, Perc Endo */
"0F184D5"  =  "1"  /* Bypass Cystic Duct to R Hep Duc w Intralum Dev, Perc Endo */
"0F184D6"  =  "1"  /* Bypass Cystic Duct to L Hep Duc w Intralum Dev, Perc Endo */
"0F184D7"  =  "1"  /* Bypass Cystic Duct to Caud Duct w Intralum Dev, Perc Endo */
"0F184D8"  =  "1"  /* Bypass Cystic Duct to Cyst Duct w Intralum Dev, Perc Endo */
"0F184D9"  =  "1"  /* Bypass Cystic Duct to CBD w Intralum Dev, Perc Endo */
"0F184DB"  =  "1"  /* Bypass Cystic Duct to Sm Int w Intralum Dev, Perc Endo */
"0F184Z3"  =  "1"  /* Bypass Cystic Duct to Duodenum, Perc Endo Approach */
"0F184Z4"  =  "1"  /* Bypass Cystic Duct to Stomach, Perc Endo Approach */
"0F184Z5"  =  "1"  /* Bypass Cystic Duct to Right Hepatic Duct, Perc Endo Approach */
"0F184Z6"  =  "1"  /* Bypass Cystic Duct to Left Hepatic Duct, Perc Endo Approach */
"0F184Z7"  =  "1"  /* Bypass Cystic Duct to Caud Duct, Perc Endo Approach */
"0F184Z8"  =  "1"  /* Bypass Cystic Duct to Cystic Duct, Perc Endo Approach */
"0F184Z9"  =  "1"  /* Bypass Cystic Duct to Common Bile Duct, Perc Endo Approach */
"0F184ZB"  =  "1"  /* Bypass Cystic Duct to Small Intestine, Perc Endo Approach */
"0F190D3"  =  "1"  /* Bypass Com Bile Duct to Duoden w Intralum Dev, Open */
"0F190D4"  =  "1"  /* Bypass Com Bile Duct to Stomach w Intralum Dev, Open */
"0F190D5"  =  "1"  /* Bypass Com Bile Duct to R Hep Duc w Intralum Dev, Open */
"0F190D6"  =  "1"  /* Bypass Com Bile Duct to L Hep Duc w Intralum Dev, Open */
"0F190D7"  =  "1"  /* Bypass Com Bile Duct to Caud Duct w Intralum Dev, Open */
"0F190D8"  =  "1"  /* Bypass Com Bile Duct to Cyst Duct w Intralum Dev, Open */
"0F190D9"  =  "1"  /* Bypass Com Bile Duct to CBD with Intralum Dev, Open Approach */
"0F190DB"  =  "1"  /* Bypass Com Bile Duct to Sm Int w Intralum Dev, Open */
"0F190Z3"  =  "1"  /* Bypass Common Bile Duct to Duodenum, Open Approach */
"0F190Z4"  =  "1"  /* Bypass Common Bile Duct to Stomach, Open Approach */
"0F190Z5"  =  "1"  /* Bypass Common Bile Duct to Right Hepatic Duct, Open Approach */
"0F190Z6"  =  "1"  /* Bypass Common Bile Duct to Left Hepatic Duct, Open Approach */
"0F190Z7"  =  "1"  /* Bypass Common Bile Duct to Caud Duct, Open Approach */
"0F190Z8"  =  "1"  /* Bypass Common Bile Duct to Cystic Duct, Open Approach */
"0F190Z9"  =  "1"  /* Bypass Common Bile Duct to Common Bile Duct, Open Approach */
"0F190ZB"  =  "1"  /* Bypass Common Bile Duct to Small Intestine, Open Approach */
"0F194D3"  =  "1"  /* Bypass Com Bile Duct to Duoden w Intralum Dev, Perc Endo */
"0F194D4"  =  "1"  /* Bypass Com Bile Duct to Stomach w Intralum Dev, Perc Endo */
"0F194D5"  =  "1"  /* Bypass Com Bile Duct to R Hep Duc w Intralum Dev, Perc Endo */
"0F194D6"  =  "1"  /* Bypass Com Bile Duct to L Hep Duc w Intralum Dev, Perc Endo */
"0F194D7"  =  "1"  /* Bypass Com Bile Duct to Caud Duct w Intralum Dev, Perc Endo */
"0F194D8"  =  "1"  /* Bypass Com Bile Duct to Cyst Duct w Intralum Dev, Perc Endo */
"0F194D9"  =  "1"  /* Bypass Com Bile Duct to CBD w Intralum Dev, Perc Endo */
"0F194DB"  =  "1"  /* Bypass Com Bile Duct to Sm Int w Intralum Dev, Perc Endo */
"0F194Z3"  =  "1"  /* Bypass Common Bile Duct to Duodenum, Perc Endo Approach */
"0F194Z4"  =  "1"  /* Bypass Common Bile Duct to Stomach, Perc Endo Approach */
"0F194Z5"  =  "1"  /* Bypass Common Bile Duct to R Hep Duc, Perc Endo Approach */
"0F194Z6"  =  "1"  /* Bypass Common Bile Duct to L Hep Duc, Perc Endo Approach */
"0F194Z7"  =  "1"  /* Bypass Common Bile Duct to Caud Duct, Perc Endo Approach */
"0F194Z8"  =  "1"  /* Bypass Common Bile Duct to Cystic Duct, Perc Endo Approach */
"0F194Z9"  =  "1"  /* Bypass Common Bile Duct to CBD, Perc Endo Approach */
"0F194ZB"  =  "1"  /* Bypass Common Bile Duct to Sm Int, Perc Endo Approach */
"0F1D0D3"  =  "1"  /* Bypass Pancreat Duct to Duoden w Intralum Dev, Open */
"0F1D0DB"  =  "1"  /* Bypass Pancreat Duct to Sm Int w Intralum Dev, Open */
"0F1D0DC"  =  "1"  /* Bypass Pancreat Duct to Lg Int w Intralum Dev, Open */
"0F1D0Z3"  =  "1"  /* Bypass Pancreatic Duct to Duodenum, Open Approach */
"0F1D0ZB"  =  "1"  /* Bypass Pancreatic Duct to Small Intestine, Open Approach */
"0F1D0ZC"  =  "1"  /* Bypass Pancreatic Duct to Large Intestine, Open Approach */
"0F1D4D3"  =  "1"  /* Bypass Pancreat Duct to Duoden w Intralum Dev, Perc Endo */
"0F1D4DB"  =  "1"  /* Bypass Pancreat Duct to Sm Int w Intralum Dev, Perc Endo */
"0F1D4DC"  =  "1"  /* Bypass Pancreat Duct to Lg Int w Intralum Dev, Perc Endo */
"0F1D4Z3"  =  "1"  /* Bypass Pancreatic Duct to Duodenum, Perc Endo Approach */
"0F1D4ZB"  =  "1"  /* Bypass Pancreatic Duct to Sm Int, Perc Endo Approach */
"0F1D4ZC"  =  "1"  /* Bypass Pancreatic Duct to Lg Int, Perc Endo Approach */
"0F1F0D3"  =  "1"  /* Bypass Access Pancr Duct to Duoden w Intralum Dev, Open */
"0F1F0DB"  =  "1"  /* Bypass Access Pancr Duct to Sm Int w Intralum Dev, Open */
"0F1F0DC"  =  "1"  /* Bypass Access Pancr Duct to Lg Int w Intralum Dev, Open */
"0F1F0Z3"  =  "1"  /* Bypass Accessory Pancreatic Duct to Duodenum, Open Approach */
"0F1F0ZB"  =  "1"  /* Bypass Accessory Pancreatic Duct to Sm Int, Open Approach */
"0F1F0ZC"  =  "1"  /* Bypass Accessory Pancreatic Duct to Lg Int, Open Approach */
"0F1F4D3"  =  "1"  /* Bypass Access Pancr Duct to Duoden w Intralum Dev, Perc Endo */
"0F1F4DB"  =  "1"  /* Bypass Access Pancr Duct to Sm Int w Intralum Dev, Perc Endo */
"0F1F4DC"  =  "1"  /* Bypass Access Pancr Duct to Lg Int w Intralum Dev, Perc Endo */
"0F1F4Z3"  =  "1"  /* Bypass Access Pancr Duct to Duoden, Perc Endo Approach */
"0F1F4ZB"  =  "1"  /* Bypass Access Pancr Duct to Sm Int, Perc Endo Approach */
"0F1F4ZC"  =  "1"  /* Bypass Access Pancr Duct to Lg Int, Perc Endo Approach */
"0F1G0D3"  =  "1"  /* Bypass Pancreas to Duodenum with Intralum Dev, Open Approach */
"0F1G0DB"  =  "1"  /* Bypass Pancreas to Sm Int with Intralum Dev, Open Approach */
"0F1G0DC"  =  "1"  /* Bypass Pancreas to Lg Int with Intralum Dev, Open Approach */
"0F1G0Z3"  =  "1"  /* Bypass Pancreas to Duodenum, Open Approach */
"0F1G0ZB"  =  "1"  /* Bypass Pancreas to Small Intestine, Open Approach */
"0F1G0ZC"  =  "1"  /* Bypass Pancreas to Large Intestine, Open Approach */
"0F1G4D3"  =  "1"  /* Bypass Pancreas to Duoden w Intralum Dev, Perc Endo */
"0F1G4DB"  =  "1"  /* Bypass Pancreas to Sm Int w Intralum Dev, Perc Endo */
"0F1G4DC"  =  "1"  /* Bypass Pancreas to Lg Int w Intralum Dev, Perc Endo */
"0F1G4Z3"  =  "1"  /* Bypass Pancreas to Duodenum, Perc Endo Approach */
"0F1G4ZB"  =  "1"  /* Bypass Pancreas to Small Intestine, Perc Endo Approach */
"0F1G4ZC"  =  "1"  /* Bypass Pancreas to Large Intestine, Perc Endo Approach */
"0F500ZZ"  =  "1"  /* Destruction of Liver, Open Approach */
"0F503ZZ"  =  "1"  /* Destruction of Liver, Percutaneous Approach */
"0F504ZZ"  =  "1"  /* Destruction of Liver, Percutaneous Endoscopic Approach */
"0F510ZZ"  =  "1"  /* Destruction of Right Lobe Liver, Open Approach */
"0F513ZZ"  =  "1"  /* Destruction of Right Lobe Liver, Percutaneous Approach */
"0F514ZZ"  =  "1"  /* Destruction of Right Lobe Liver, Perc Endo Approach */
"0F520ZZ"  =  "1"  /* Destruction of Left Lobe Liver, Open Approach */
"0F523ZZ"  =  "1"  /* Destruction of Left Lobe Liver, Percutaneous Approach */
"0F524ZZ"  =  "1"  /* Destruction of Left Lobe Liver, Perc Endo Approach */
"0F540ZZ"  =  "1"  /* Destruction of Gallbladder, Open Approach */
"0F543ZZ"  =  "1"  /* Destruction of Gallbladder, Percutaneous Approach */
"0F544ZZ"  =  "1"  /* Destruction of Gallbladder, Percutaneous Endoscopic Approach */
"0F550ZZ"  =  "1"  /* Destruction of Right Hepatic Duct, Open Approach */
"0F553ZZ"  =  "1"  /* Destruction of Right Hepatic Duct, Percutaneous Approach */
"0F557ZZ"  =  "1"  /* Destruction of Right Hepatic Duct, Via Opening */
"0F560ZZ"  =  "1"  /* Destruction of Left Hepatic Duct, Open Approach */
"0F563ZZ"  =  "1"  /* Destruction of Left Hepatic Duct, Percutaneous Approach */
"0F567ZZ"  =  "1"  /* Destruction of Left Hepatic Duct, Via Opening */
"0F580ZZ"  =  "1"  /* Destruction of Cystic Duct, Open Approach */
"0F583ZZ"  =  "1"  /* Destruction of Cystic Duct, Percutaneous Approach */
"0F587ZZ"  =  "1"  /* Destruction of Cystic Duct, Via Opening */
"0F590ZZ"  =  "1"  /* Destruction of Common Bile Duct, Open Approach */
"0F593ZZ"  =  "1"  /* Destruction of Common Bile Duct, Percutaneous Approach */
"0F597ZZ"  =  "1"  /* Destruction of Common Bile Duct, Via Opening */
"0F5C0ZZ"  =  "1"  /* Destruction of Ampulla of Vater, Open Approach */
"0F5C3ZZ"  =  "1"  /* Destruction of Ampulla of Vater, Percutaneous Approach */
"0F5C7ZZ"  =  "1"  /* Destruction of Ampulla of Vater, Via Opening */
"0F5D0ZZ"  =  "1"  /* Destruction of Pancreatic Duct, Open Approach */
"0F5D3ZZ"  =  "1"  /* Destruction of Pancreatic Duct, Percutaneous Approach */
"0F5D7ZZ"  =  "1"  /* Destruction of Pancreatic Duct, Via Opening */
"0F5F0ZZ"  =  "1"  /* Destruction of Accessory Pancreatic Duct, Open Approach */
"0F5F3ZZ"  =  "1"  /* Destruction of Accessory Pancreatic Duct, Perc Approach */
"0F5F7ZZ"  =  "1"  /* Destruction of Accessory Pancreatic Duct, Via Opening */
"0F5G0ZZ"  =  "1"  /* Destruction of Pancreas, Open Approach */
"0F5G3ZZ"  =  "1"  /* Destruction of Pancreas, Percutaneous Approach */
"0F750DZ"  =  "1"  /* Dilation of R Hepatic Duct with Intralum Dev, Open Approach */
"0F750ZZ"  =  "1"  /* Dilation of Right Hepatic Duct, Open Approach */
"0F757ZZ"  =  "1"  /* Dilation of Right Hepatic Duct, Via Opening */
"0F760DZ"  =  "1"  /* Dilation of L Hepatic Duct with Intralum Dev, Open Approach */
"0F760ZZ"  =  "1"  /* Dilation of Left Hepatic Duct, Open Approach */
"0F767ZZ"  =  "1"  /* Dilation of Left Hepatic Duct, Via Opening */
"0F780DZ"  =  "1"  /* Dilation of Cystic Duct with Intralum Dev, Open Approach */
"0F780ZZ"  =  "1"  /* Dilation of Cystic Duct, Open Approach */
"0F787ZZ"  =  "1"  /* Dilation of Cystic Duct, Via Natural or Artificial Opening */
"0F790DZ"  =  "1"  /* Dilation of Com Bile Duct with Intralum Dev, Open Approach */
"0F790ZZ"  =  "1"  /* Dilation of Common Bile Duct, Open Approach */
"0F797ZZ"  =  "1"  /* Dilation of Common Bile Duct, Via Opening */
"0F7C0DZ"  =  "1"  /* Dilation of Ampulla Vater with Intralum Dev, Open Approach */
"0F7C0ZZ"  =  "1"  /* Dilation of Ampulla of Vater, Open Approach */
"0F7C3DZ"  =  "1"  /* Dilation of Ampulla Vater with Intralum Dev, Perc Approach */
"0F7C3ZZ"  =  "1"  /* Dilation of Ampulla of Vater, Percutaneous Approach */
"0F7C4DZ"  =  "1"  /* Dilate Ampulla Vater w Intralum Dev, Perc Endo */
"0F7C4ZZ"  =  "1"  /* Dilation of Ampulla of Vater, Perc Endo Approach */
"0F7C7DZ"  =  "1"  /* Dilation of Ampulla of Vater with Intralum Dev, Via Opening */
"0F7C7ZZ"  =  "1"  /* Dilation of Ampulla of Vater, Via Opening */
"0F7D0DZ"  =  "1"  /* Dilation of Pancreatic Duct with Intralum Dev, Open Approach */
"0F7D0ZZ"  =  "1"  /* Dilation of Pancreatic Duct, Open Approach */
"0F7D3DZ"  =  "1"  /* Dilation of Pancreatic Duct with Intralum Dev, Perc Approach */
"0F7D3ZZ"  =  "1"  /* Dilation of Pancreatic Duct, Percutaneous Approach */
"0F7D7ZZ"  =  "1"  /* Dilation of Pancreatic Duct, Via Opening */
"0F7F0DZ"  =  "1"  /* Dilate of Access Pancr Duct with Intralum Dev, Open Approach */
"0F7F0ZZ"  =  "1"  /* Dilation of Accessory Pancreatic Duct, Open Approach */
"0F7F3DZ"  =  "1"  /* Dilate of Access Pancr Duct with Intralum Dev, Perc Approach */
"0F7F3ZZ"  =  "1"  /* Dilation of Accessory Pancreatic Duct, Percutaneous Approach */
"0F7F7DZ"  =  "1"  /* Dilation of Access Pancr Duct with Intralum Dev, Via Opening */
"0F7F7ZZ"  =  "1"  /* Dilation of Accessory Pancreatic Duct, Via Opening */
"0F8G0ZZ"  =  "1"  /* Division of Pancreas, Open Approach */
"0F8G3ZZ"  =  "1"  /* Division of Pancreas, Percutaneous Approach */
"0F8G4ZZ"  =  "1"  /* Division of Pancreas, Percutaneous Endoscopic Approach */
"0F9000Z"  =  "1"  /* Drainage of Liver with Drainage Device, Open Approach */
"0F900ZX"  =  "1"  /* Drainage of Liver, Open Approach, Diagnostic */
"0F900ZZ"  =  "1"  /* Drainage of Liver, Open Approach */
"0F9100Z"  =  "1"  /* Drainage of Right Lobe Liver with Drain Dev, Open Approach */
"0F910ZX"  =  "1"  /* Drainage of Right Lobe Liver, Open Approach, Diagnostic */
"0F910ZZ"  =  "1"  /* Drainage of Right Lobe Liver, Open Approach */
"0F9200Z"  =  "1"  /* Drainage of Left Lobe Liver with Drain Dev, Open Approach */
"0F920ZX"  =  "1"  /* Drainage of Left Lobe Liver, Open Approach, Diagnostic */
"0F920ZZ"  =  "1"  /* Drainage of Left Lobe Liver, Open Approach */
"0F9400Z"  =  "1"  /* Drainage of Gallbladder with Drainage Device, Open Approach */
"0F940ZX"  =  "1"  /* Drainage of Gallbladder, Open Approach, Diagnostic */
"0F940ZZ"  =  "1"  /* Drainage of Gallbladder, Open Approach */
"0F9430Z"  =  "1"  /* Drainage of Gallbladder with Drainage Device, Perc Approach */
"0F9500Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drain Dev, Open Approach */
"0F950ZX"  =  "1"  /* Drainage of Right Hepatic Duct, Open Approach, Diagnostic */
"0F950ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Open Approach */
"0F9530Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drain Dev, Perc Approach */
"0F953ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Percutaneous Approach */
"0F9540Z"  =  "1"  /* Drain of R Hepatic Duct with Drain Dev, Perc Endo Approach */
"0F954ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Perc Endo Approach */
"0F9570Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drain Dev, Via Opening */
"0F957ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Via Opening */
"0F9580Z"  =  "1"  /* Drainage of Right Hepatic Duct with Drainage Device, Endo */
"0F958ZZ"  =  "1"  /* Drainage of Right Hepatic Duct, Endo */
"0F9600Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drain Dev, Open Approach */
"0F960ZX"  =  "1"  /* Drainage of Left Hepatic Duct, Open Approach, Diagnostic */
"0F960ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Open Approach */
"0F9630Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drain Dev, Perc Approach */
"0F963ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Percutaneous Approach */
"0F9640Z"  =  "1"  /* Drain of L Hepatic Duct with Drain Dev, Perc Endo Approach */
"0F964ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Perc Endo Approach */
"0F9670Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drain Dev, Via Opening */
"0F967ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Via Opening */
"0F9680Z"  =  "1"  /* Drainage of Left Hepatic Duct with Drainage Device, Endo */
"0F968ZZ"  =  "1"  /* Drainage of Left Hepatic Duct, Endo */
"0F9800Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Open Approach */
"0F980ZX"  =  "1"  /* Drainage of Cystic Duct, Open Approach, Diagnostic */
"0F980ZZ"  =  "1"  /* Drainage of Cystic Duct, Open Approach */
"0F9830Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Perc Approach */
"0F983ZZ"  =  "1"  /* Drainage of Cystic Duct, Percutaneous Approach */
"0F9840Z"  =  "1"  /* Drainage of Cystic Duct with Drain Dev, Perc Endo Approach */
"0F984ZZ"  =  "1"  /* Drainage of Cystic Duct, Percutaneous Endoscopic Approach */
"0F9870Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Via Opening */
"0F987ZZ"  =  "1"  /* Drainage of Cystic Duct, Via Natural or Artificial Opening */
"0F9880Z"  =  "1"  /* Drainage of Cystic Duct with Drainage Device, Endo */
"0F988ZZ"  =  "1"  /* Drainage of Cystic Duct, Endo */
"0F9900Z"  =  "1"  /* Drainage of Common Bile Duct with Drain Dev, Open Approach */
"0F990ZX"  =  "1"  /* Drainage of Common Bile Duct, Open Approach, Diagnostic */
"0F990ZZ"  =  "1"  /* Drainage of Common Bile Duct, Open Approach */
"0F9930Z"  =  "1"  /* Drainage of Common Bile Duct with Drain Dev, Perc Approach */
"0F9940Z"  =  "1"  /* Drainage of Com Bile Duct with Drain Dev, Perc Endo Approach */
"0F9970Z"  =  "1"  /* Drainage of Common Bile Duct with Drain Dev, Via Opening */
"0F9C00Z"  =  "1"  /* Drainage of Ampulla of Vater with Drain Dev, Open Approach */
"0F9C0ZX"  =  "1"  /* Drainage of Ampulla of Vater, Open Approach, Diagnostic */
"0F9C0ZZ"  =  "1"  /* Drainage of Ampulla of Vater, Open Approach */
"0F9C30Z"  =  "1"  /* Drainage of Ampulla of Vater with Drain Dev, Perc Approach */
"0F9C3ZZ"  =  "1"  /* Drainage of Ampulla of Vater, Percutaneous Approach */
"0F9C70Z"  =  "1"  /* Drainage of Ampulla of Vater with Drain Dev, Via Opening */
"0F9C7ZZ"  =  "1"  /* Drainage of Ampulla of Vater, Via Opening */
"0F9D00Z"  =  "1"  /* Drainage of Pancreatic Duct with Drain Dev, Open Approach */
"0F9D0ZX"  =  "1"  /* Drainage of Pancreatic Duct, Open Approach, Diagnostic */
"0F9D0ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Open Approach */
"0F9D30Z"  =  "1"  /* Drainage of Pancreatic Duct with Drain Dev, Perc Approach */
"0F9D3ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Percutaneous Approach */
"0F9D40Z"  =  "1"  /* Drainage of Pancreat Duct with Drain Dev, Perc Endo Approach */
"0F9D4ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Perc Endo Approach */
"0F9D70Z"  =  "1"  /* Drainage of Pancreatic Duct with Drain Dev, Via Opening */
"0F9D7ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Via Opening */
"0F9D8ZZ"  =  "1"  /* Drainage of Pancreatic Duct, Endo */
"0F9F00Z"  =  "1"  /* Drainage of Access Pancr Duct with Drain Dev, Open Approach */
"0F9F0ZX"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Open Approach, Diagn */
"0F9F0ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Open Approach */
"0F9F30Z"  =  "1"  /* Drainage of Access Pancr Duct with Drain Dev, Perc Approach */
"0F9F3ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Percutaneous Approach */
"0F9F40Z"  =  "1"  /* Drain Access Pancr Duct w Drain Dev, Perc Endo */
"0F9F4ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Perc Endo Approach */
"0F9F70Z"  =  "1"  /* Drainage of Access Pancr Duct with Drain Dev, Via Opening */
"0F9F7ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Via Opening */
"0F9F8ZZ"  =  "1"  /* Drainage of Accessory Pancreatic Duct, Endo */
"0F9G00Z"  =  "1"  /* Drainage of Pancreas with Drainage Device, Open Approach */
"0F9G0ZX"  =  "1"  /* Drainage of Pancreas, Open Approach, Diagnostic */
"0F9G0ZZ"  =  "1"  /* Drainage of Pancreas, Open Approach */
"0F9G30Z"  =  "1"  /* Drainage of Pancreas with Drainage Device, Perc Approach */
"0F9G3ZZ"  =  "1"  /* Drainage of Pancreas, Percutaneous Approach */
"0F9G40Z"  =  "1"  /* Drainage of Pancreas with Drain Dev, Perc Endo Approach */
"0F9G4ZZ"  =  "1"  /* Drainage of Pancreas, Percutaneous Endoscopic Approach */
"0FB00ZX"  =  "1"  /* Excision of Liver, Open Approach, Diagnostic */
"0FB00ZZ"  =  "1"  /* Excision of Liver, Open Approach */
"0FB03ZZ"  =  "1"  /* Excision of Liver, Percutaneous Approach */
"0FB04ZX"  =  "1"  /* Excision of Liver, Percutaneous Endoscopic Approach, Diagn */
"0FB04ZZ"  =  "1"  /* Excision of Liver, Percutaneous Endoscopic Approach */
"0FB10ZX"  =  "1"  /* Excision of Right Lobe Liver, Open Approach, Diagnostic */
"0FB10ZZ"  =  "1"  /* Excision of Right Lobe Liver, Open Approach */
"0FB13ZZ"  =  "1"  /* Excision of Right Lobe Liver, Percutaneous Approach */
"0FB14ZX"  =  "1"  /* Excision of Right Lobe Liver, Perc Endo Approach, Diagn */
"0FB14ZZ"  =  "1"  /* Excision of Right Lobe Liver, Perc Endo Approach */
"0FB20ZX"  =  "1"  /* Excision of Left Lobe Liver, Open Approach, Diagnostic */
"0FB20ZZ"  =  "1"  /* Excision of Left Lobe Liver, Open Approach */
"0FB23ZZ"  =  "1"  /* Excision of Left Lobe Liver, Percutaneous Approach */
"0FB24ZX"  =  "1"  /* Excision of Left Lobe Liver, Perc Endo Approach, Diagn */
"0FB24ZZ"  =  "1"  /* Excision of Left Lobe Liver, Perc Endo Approach */
"0FB40ZX"  =  "1"  /* Excision of Gallbladder, Open Approach, Diagnostic */
"0FB40ZZ"  =  "1"  /* Excision of Gallbladder, Open Approach */
"0FB43ZZ"  =  "1"  /* Excision of Gallbladder, Percutaneous Approach */
"0FB44ZZ"  =  "1"  /* Excision of Gallbladder, Percutaneous Endoscopic Approach */
"0FB50ZX"  =  "1"  /* Excision of Right Hepatic Duct, Open Approach, Diagnostic */
"0FB50ZZ"  =  "1"  /* Excision of Right Hepatic Duct, Open Approach */
"0FB53ZZ"  =  "1"  /* Excision of Right Hepatic Duct, Percutaneous Approach */
"0FB57ZZ"  =  "1"  /* Excision of Right Hepatic Duct, Via Opening */
"0FB60ZX"  =  "1"  /* Excision of Left Hepatic Duct, Open Approach, Diagnostic */
"0FB60ZZ"  =  "1"  /* Excision of Left Hepatic Duct, Open Approach */
"0FB63ZZ"  =  "1"  /* Excision of Left Hepatic Duct, Percutaneous Approach */
"0FB67ZZ"  =  "1"  /* Excision of Left Hepatic Duct, Via Opening */
"0FB80ZX"  =  "1"  /* Excision of Cystic Duct, Open Approach, Diagnostic */
"0FB80ZZ"  =  "1"  /* Excision of Cystic Duct, Open Approach */
"0FB83ZZ"  =  "1"  /* Excision of Cystic Duct, Percutaneous Approach */
"0FB87ZZ"  =  "1"  /* Excision of Cystic Duct, Via Natural or Artificial Opening */
"0FB90ZX"  =  "1"  /* Excision of Common Bile Duct, Open Approach, Diagnostic */
"0FB90ZZ"  =  "1"  /* Excision of Common Bile Duct, Open Approach */
"0FB93ZZ"  =  "1"  /* Excision of Common Bile Duct, Percutaneous Approach */
"0FB97ZZ"  =  "1"  /* Excision of Common Bile Duct, Via Opening */
"0FBC0ZX"  =  "1"  /* Excision of Ampulla of Vater, Open Approach, Diagnostic */
"0FBC0ZZ"  =  "1"  /* Excision of Ampulla of Vater, Open Approach */
"0FBC3ZZ"  =  "1"  /* Excision of Ampulla of Vater, Percutaneous Approach */
"0FBC7ZZ"  =  "1"  /* Excision of Ampulla of Vater, Via Opening */
"0FBD0ZX"  =  "1"  /* Excision of Pancreatic Duct, Open Approach, Diagnostic */
"0FBD0ZZ"  =  "1"  /* Excision of Pancreatic Duct, Open Approach */
"0FBD3ZZ"  =  "1"  /* Excision of Pancreatic Duct, Percutaneous Approach */
"0FBD7ZZ"  =  "1"  /* Excision of Pancreatic Duct, Via Opening */
"0FBF0ZX"  =  "1"  /* Excision of Accessory Pancreatic Duct, Open Approach, Diagn */
"0FBF0ZZ"  =  "1"  /* Excision of Accessory Pancreatic Duct, Open Approach */
"0FBF3ZZ"  =  "1"  /* Excision of Accessory Pancreatic Duct, Percutaneous Approach */
"0FBF7ZZ"  =  "1"  /* Excision of Accessory Pancreatic Duct, Via Opening */
"0FBG0ZX"  =  "1"  /* Excision of Pancreas, Open Approach, Diagnostic */
"0FBG0ZZ"  =  "1"  /* Excision of Pancreas, Open Approach */
"0FBG3ZZ"  =  "1"  /* Excision of Pancreas, Percutaneous Approach */
"0FBG4ZZ"  =  "1"  /* Excision of Pancreas, Percutaneous Endoscopic Approach */
"0FC00ZZ"  =  "1"  /* Extirpation of Matter from Liver, Open Approach */
"0FC03ZZ"  =  "1"  /* Extirpation of Matter from Liver, Percutaneous Approach */
"0FC04ZZ"  =  "1"  /* Extirpation of Matter from Liver, Perc Endo Approach */
"0FC10ZZ"  =  "1"  /* Extirpation of Matter from Right Lobe Liver, Open Approach */
"0FC13ZZ"  =  "1"  /* Extirpation of Matter from Right Lobe Liver, Perc Approach */
"0FC14ZZ"  =  "1"  /* Extirpation of Matter from R Lobe Liver, Perc Endo Approach */
"0FC20ZZ"  =  "1"  /* Extirpation of Matter from Left Lobe Liver, Open Approach */
"0FC23ZZ"  =  "1"  /* Extirpation of Matter from Left Lobe Liver, Perc Approach */
"0FC24ZZ"  =  "1"  /* Extirpation of Matter from L Lobe Liver, Perc Endo Approach */
"0FC40ZZ"  =  "1"  /* Extirpation of Matter from Gallbladder, Open Approach */
"0FC43ZZ"  =  "1"  /* Extirpation of Matter from Gallbladder, Perc Approach */
"0FC44ZZ"  =  "1"  /* Extirpation of Matter from Gallbladder, Perc Endo Approach */
"0FC50ZZ"  =  "1"  /* Extirpation of Matter from Right Hepatic Duct, Open Approach */
"0FC60ZZ"  =  "1"  /* Extirpation of Matter from Left Hepatic Duct, Open Approach */
"0FC80ZZ"  =  "1"  /* Extirpation of Matter from Cystic Duct, Open Approach */
"0FC90ZZ"  =  "1"  /* Extirpation of Matter from Common Bile Duct, Open Approach */
"0FCC0ZZ"  =  "1"  /* Extirpation of Matter from Ampulla of Vater, Open Approach */
"0FCC3ZZ"  =  "1"  /* Extirpation of Matter from Ampulla of Vater, Perc Approach */
"0FCC7ZZ"  =  "1"  /* Extirpation of Matter from Ampulla of Vater, Via Opening */
"0FCD0ZZ"  =  "1"  /* Extirpation of Matter from Pancreatic Duct, Open Approach */
"0FCD7ZZ"  =  "1"  /* Extirpation of Matter from Pancreatic Duct, Via Opening */
"0FCF0ZZ"  =  "1"  /* Extirpation of Matter from Access Pancr Duct, Open Approach */
"0FCF7ZZ"  =  "1"  /* Extirpation of Matter from Access Pancr Duct, Via Opening */
"0FCG0ZZ"  =  "1"  /* Extirpation of Matter from Pancreas, Open Approach */
"0FCG3ZZ"  =  "1"  /* Extirpation of Matter from Pancreas, Percutaneous Approach */
"0FCG4ZZ"  =  "1"  /* Extirpation of Matter from Pancreas, Perc Endo Approach */
"0FF40ZZ"  =  "1"  /* Fragmentation in Gallbladder, Open Approach */
"0FF43ZZ"  =  "1"  /* Fragmentation in Gallbladder, Percutaneous Approach */
"0FF44ZZ"  =  "1"  /* Fragmentation in Gallbladder, Perc Endo Approach */
"0FF47ZZ"  =  "1"  /* Fragmentation in Gallbladder, Via Opening */
"0FF50ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Open Approach */
"0FF53ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Percutaneous Approach */
"0FF54ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Perc Endo Approach */
"0FF57ZZ"  =  "1"  /* Fragmentation in Right Hepatic Duct, Via Opening */
"0FF60ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Open Approach */
"0FF63ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Percutaneous Approach */
"0FF64ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Perc Endo Approach */
"0FF67ZZ"  =  "1"  /* Fragmentation in Left Hepatic Duct, Via Opening */
"0FF80ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Open Approach */
"0FF83ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Percutaneous Approach */
"0FF84ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Perc Endo Approach */
"0FF87ZZ"  =  "1"  /* Fragmentation in Cystic Duct, Via Opening */
"0FF90ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Open Approach */
"0FF93ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Percutaneous Approach */
"0FF94ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Perc Endo Approach */
"0FF97ZZ"  =  "1"  /* Fragmentation in Common Bile Duct, Via Opening */
"0FFC0ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Open Approach */
"0FFC3ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Percutaneous Approach */
"0FFC4ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Perc Endo Approach */
"0FFC7ZZ"  =  "1"  /* Fragmentation in Ampulla of Vater, Via Opening */
"0FFD0ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Open Approach */
"0FFD3ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Percutaneous Approach */
"0FFD4ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Perc Endo Approach */
"0FFD7ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Via Opening */
"0FFD8ZZ"  =  "1"  /* Fragmentation in Pancreatic Duct, Endo */
"0FFF0ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Open Approach */
"0FFF3ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Perc Approach */
"0FFF4ZZ"  =  "1"  /* Fragmentation in Access Pancr Duct, Perc Endo Approach */
"0FFF7ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Via Opening */
"0FFF8ZZ"  =  "1"  /* Fragmentation in Accessory Pancreatic Duct, Endo */
"0FH002Z"  =  "1"  /* Insertion of Monitoring Device into Liver, Open Approach */
"0FH032Z"  =  "1"  /* Insertion of Monitoring Device into Liver, Perc Approach */
"0FH042Z"  =  "1"  /* Insertion of Monitor Dev into Liver, Perc Endo Approach */
"0FH102Z"  =  "1"  /* Insertion of Monitor Dev into R Lobe Liver, Open Approach */
"0FH132Z"  =  "1"  /* Insertion of Monitor Dev into R Lobe Liver, Perc Approach */
"0FH142Z"  =  "1"  /* Insert of Monitor Dev into R Lobe Liver, Perc Endo Approach */
"0FH202Z"  =  "1"  /* Insertion of Monitor Dev into L Lobe Liver, Open Approach */
"0FH232Z"  =  "1"  /* Insertion of Monitor Dev into L Lobe Liver, Perc Approach */
"0FH242Z"  =  "1"  /* Insert of Monitor Dev into L Lobe Liver, Perc Endo Approach */
"0FH402Z"  =  "1"  /* Insertion of Monitor Dev into Gallbladder, Open Approach */
"0FH432Z"  =  "1"  /* Insertion of Monitor Dev into Gallbladder, Perc Approach */
"0FH442Z"  =  "1"  /* Insert of Monitor Dev into Gallbladder, Perc Endo Approach */
"0FHB01Z"  =  "1"  /* Insert of Radioact Elem into Hepatobil Duct, Open Approach */
"0FHB02Z"  =  "1"  /* Insertion of Monitor Dev into Hepatobil Duct, Open Approach */
"0FHB0DZ"  =  "1"  /* Insertion of Intralum Dev into Hepatobil Duct, Open Approach */
"0FHB31Z"  =  "1"  /* Insert of Radioact Elem into Hepatobil Duct, Perc Approach */
"0FHB32Z"  =  "1"  /* Insertion of Monitor Dev into Hepatobil Duct, Perc Approach */
"0FHB3DZ"  =  "1"  /* Insertion of Intralum Dev into Hepatobil Duct, Perc Approach */
"0FHB41Z"  =  "1"  /* Insert Radioact Elem in Hepatobil Duct, Perc Endo */
"0FHB42Z"  =  "1"  /* Insert Monitor Dev in Hepatobil Duct, Perc Endo */
"0FHB71Z"  =  "1"  /* Insertion of Radioact Elem into Hepatobil Duct, Via Opening */
"0FHB72Z"  =  "1"  /* Insertion of Monitor Dev into Hepatobil Duct, Via Opening */
"0FHB7DZ"  =  "1"  /* Insertion of Intralum Dev into Hepatobil Duct, Via Opening */
"0FHB81Z"  =  "1"  /* Insertion of Radioactive Element into Hepatobil Duct, Endo */
"0FHB82Z"  =  "1"  /* Insertion of Monitoring Device into Hepatobiliary Duct, Endo */
"0FHD01Z"  =  "1"  /* Insertion of Radioact Elem into Pancreat Duct, Open Approach */
"0FHD02Z"  =  "1"  /* Insertion of Monitor Dev into Pancreat Duct, Open Approach */
"0FHD0DZ"  =  "1"  /* Insertion of Intralum Dev into Pancreat Duct, Open Approach */
"0FHD31Z"  =  "1"  /* Insertion of Radioact Elem into Pancreat Duct, Perc Approach */
"0FHD32Z"  =  "1"  /* Insertion of Monitor Dev into Pancreat Duct, Perc Approach */
"0FHD3DZ"  =  "1"  /* Insertion of Intralum Dev into Pancreat Duct, Perc Approach */
"0FHD41Z"  =  "1"  /* Insert Radioact Elem in Pancreat Duct, Perc Endo */
"0FHD42Z"  =  "1"  /* Insert of Monitor Dev into Pancreat Duct, Perc Endo Approach */
"0FHD71Z"  =  "1"  /* Insertion of Radioact Elem into Pancreat Duct, Via Opening */
"0FHD72Z"  =  "1"  /* Insertion of Monitor Dev into Pancreat Duct, Via Opening */
"0FHD7DZ"  =  "1"  /* Insertion of Intralum Dev into Pancreat Duct, Via Opening */
"0FHD81Z"  =  "1"  /* Insertion of Radioactive Element into Pancreatic Duct, Endo */
"0FHD82Z"  =  "1"  /* Insertion of Monitoring Device into Pancreatic Duct, Endo */
"0FHG02Z"  =  "1"  /* Insertion of Monitoring Device into Pancreas, Open Approach */
"0FHG32Z"  =  "1"  /* Insertion of Monitoring Device into Pancreas, Perc Approach */
"0FHG42Z"  =  "1"  /* Insertion of Monitor Dev into Pancreas, Perc Endo Approach */
"0FJ00ZZ"  =  "1"  /* Inspection of Liver, Open Approach */
"0FJ03ZZ"  =  "1"  /* Inspection of Liver, Percutaneous Approach */
"0FJ04ZZ"  =  "1"  /* Inspection of Liver, Percutaneous Endoscopic Approach */
"0FJ40ZZ"  =  "1"  /* Inspection of Gallbladder, Open Approach */
"0FJ43ZZ"  =  "1"  /* Inspection of Gallbladder, Percutaneous Approach */
"0FJ44ZZ"  =  "1"  /* Inspection of Gallbladder, Percutaneous Endoscopic Approach */
"0FJB0ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Open Approach */
"0FJB3ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Percutaneous Approach */
"0FJB4ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Perc Endo Approach */
"0FJB7ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Via Opening */
"0FJB8ZZ"  =  "1"  /* Inspection of Hepatobiliary Duct, Endo */
"0FJD0ZZ"  =  "1"  /* Inspection of Pancreatic Duct, Open Approach */
"0FJD3ZZ"  =  "1"  /* Inspection of Pancreatic Duct, Percutaneous Approach */
"0FJD4ZZ"  =  "1"  /* Inspection of Pancreatic Duct, Perc Endo Approach */
"0FJD7ZZ"  =  "1"  /* Inspection of Pancreatic Duct, Via Opening */
"0FJD8ZZ"  =  "1"  /* Inspection of Pancreatic Duct, Endo */
"0FJG0ZZ"  =  "1"  /* Inspection of Pancreas, Open Approach */
"0FJG3ZZ"  =  "1"  /* Inspection of Pancreas, Percutaneous Approach */
"0FJG4ZZ"  =  "1"  /* Inspection of Pancreas, Percutaneous Endoscopic Approach */
"0FL50CZ"  =  "1"  /* Occlusion of R Hepatic Duct with Extralum Dev, Open Approach */
"0FL50DZ"  =  "1"  /* Occlusion of R Hepatic Duct with Intralum Dev, Open Approach */
"0FL50ZZ"  =  "1"  /* Occlusion of Right Hepatic Duct, Open Approach */
"0FL60CZ"  =  "1"  /* Occlusion of L Hepatic Duct with Extralum Dev, Open Approach */
"0FL60DZ"  =  "1"  /* Occlusion of L Hepatic Duct with Intralum Dev, Open Approach */
"0FL60ZZ"  =  "1"  /* Occlusion of Left Hepatic Duct, Open Approach */
"0FL80CZ"  =  "1"  /* Occlusion of Cystic Duct with Extralum Dev, Open Approach */
"0FL80DZ"  =  "1"  /* Occlusion of Cystic Duct with Intralum Dev, Open Approach */
"0FL80ZZ"  =  "1"  /* Occlusion of Cystic Duct, Open Approach */
"0FL90CZ"  =  "1"  /* Occlusion of Com Bile Duct with Extralum Dev, Open Approach */
"0FL90DZ"  =  "1"  /* Occlusion of Com Bile Duct with Intralum Dev, Open Approach */
"0FL90ZZ"  =  "1"  /* Occlusion of Common Bile Duct, Open Approach */
"0FLC0CZ"  =  "1"  /* Occlusion of Ampulla Vater with Extralum Dev, Open Approach */
"0FLC0DZ"  =  "1"  /* Occlusion of Ampulla Vater with Intralum Dev, Open Approach */
"0FLC0ZZ"  =  "1"  /* Occlusion of Ampulla of Vater, Open Approach */
"0FLC3CZ"  =  "1"  /* Occlusion of Ampulla Vater with Extralum Dev, Perc Approach */
"0FLC3DZ"  =  "1"  /* Occlusion of Ampulla Vater with Intralum Dev, Perc Approach */
"0FLC3ZZ"  =  "1"  /* Occlusion of Ampulla of Vater, Percutaneous Approach */
"0FLC4CZ"  =  "1"  /* Occlusion Ampulla Vater w Extralum Dev, Perc Endo */
"0FLC4DZ"  =  "1"  /* Occlusion Ampulla Vater w Intralum Dev, Perc Endo */
"0FLC4ZZ"  =  "1"  /* Occlusion of Ampulla of Vater, Perc Endo Approach */
"0FLC7DZ"  =  "1"  /* Occlusion of Ampulla of Vater with Intralum Dev, Via Opening */
"0FLC7ZZ"  =  "1"  /* Occlusion of Ampulla of Vater, Via Opening */
"0FLC8DZ"  =  "1"  /* Occlusion of Ampulla of Vater with Intraluminal Device, Endo */
"0FLC8ZZ"  =  "1"  /* Occlusion of Ampulla of Vater, Endo */
"0FLD0CZ"  =  "1"  /* Occlusion of Pancreat Duct with Extralum Dev, Open Approach */
"0FLD0DZ"  =  "1"  /* Occlusion of Pancreat Duct with Intralum Dev, Open Approach */
"0FLD0ZZ"  =  "1"  /* Occlusion of Pancreatic Duct, Open Approach */
"0FLD3CZ"  =  "1"  /* Occlusion of Pancreat Duct with Extralum Dev, Perc Approach */
"0FLD3DZ"  =  "1"  /* Occlusion of Pancreat Duct with Intralum Dev, Perc Approach */
"0FLD3ZZ"  =  "1"  /* Occlusion of Pancreatic Duct, Percutaneous Approach */
"0FLD4CZ"  =  "1"  /* Occlusion Pancreat Duct w Extralum Dev, Perc Endo */
"0FLD4DZ"  =  "1"  /* Occlusion Pancreat Duct w Intralum Dev, Perc Endo */
"0FLD4ZZ"  =  "1"  /* Occlusion of Pancreatic Duct, Perc Endo Approach */
"0FLD7DZ"  =  "1"  /* Occlusion of Pancreatic Duct with Intralum Dev, Via Opening */
"0FLD7ZZ"  =  "1"  /* Occlusion of Pancreatic Duct, Via Opening */
"0FLD8DZ"  =  "1"  /* Occlusion of Pancreatic Duct with Intraluminal Device, Endo */
"0FLD8ZZ"  =  "1"  /* Occlusion of Pancreatic Duct, Endo */
"0FLF0CZ"  =  "1"  /* Occlusion Access Pancr Duct w Extralum Dev, Open */
"0FLF0DZ"  =  "1"  /* Occlusion Access Pancr Duct w Intralum Dev, Open */
"0FLF0ZZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct, Open Approach */
"0FLF3CZ"  =  "1"  /* Occlusion Access Pancr Duct w Extralum Dev, Perc */
"0FLF3DZ"  =  "1"  /* Occlusion Access Pancr Duct w Intralum Dev, Perc */
"0FLF3ZZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct, Perc Approach */
"0FLF4CZ"  =  "1"  /* Occlusion Access Pancr Duct w Extralum Dev, Perc Endo */
"0FLF4DZ"  =  "1"  /* Occlusion Access Pancr Duct w Intralum Dev, Perc Endo */
"0FLF4ZZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct, Perc Endo Approach */
"0FLF7DZ"  =  "1"  /* Occlusion Access Pancr Duct w Intralum Dev, Via Opening */
"0FLF7ZZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct, Via Opening */
"0FLF8DZ"  =  "1"  /* Occlusion of Access Pancr Duct with Intralum Dev, Endo */
"0FLF8ZZ"  =  "1"  /* Occlusion of Accessory Pancreatic Duct, Endo */
"0FM00ZZ"  =  "1"  /* Reattachment of Liver, Open Approach */
"0FM04ZZ"  =  "1"  /* Reattachment of Liver, Percutaneous Endoscopic Approach */
"0FM10ZZ"  =  "1"  /* Reattachment of Right Lobe Liver, Open Approach */
"0FM14ZZ"  =  "1"  /* Reattachment of Right Lobe Liver, Perc Endo Approach */
"0FM20ZZ"  =  "1"  /* Reattachment of Left Lobe Liver, Open Approach */
"0FM24ZZ"  =  "1"  /* Reattachment of Left Lobe Liver, Perc Endo Approach */
"0FM40ZZ"  =  "1"  /* Reattachment of Gallbladder, Open Approach */
"0FM50ZZ"  =  "1"  /* Reattachment of Right Hepatic Duct, Open Approach */
"0FM60ZZ"  =  "1"  /* Reattachment of Left Hepatic Duct, Open Approach */
"0FM80ZZ"  =  "1"  /* Reattachment of Cystic Duct, Open Approach */
"0FM90ZZ"  =  "1"  /* Reattachment of Common Bile Duct, Open Approach */
"0FMC0ZZ"  =  "1"  /* Reattachment of Ampulla of Vater, Open Approach */
"0FMC4ZZ"  =  "1"  /* Reattachment of Ampulla of Vater, Perc Endo Approach */
"0FMD0ZZ"  =  "1"  /* Reattachment of Pancreatic Duct, Open Approach */
"0FMD4ZZ"  =  "1"  /* Reattachment of Pancreatic Duct, Perc Endo Approach */
"0FMF0ZZ"  =  "1"  /* Reattachment of Accessory Pancreatic Duct, Open Approach */
"0FMF4ZZ"  =  "1"  /* Reattachment of Access Pancr Duct, Perc Endo Approach */
"0FMG0ZZ"  =  "1"  /* Reattachment of Pancreas, Open Approach */
"0FMG4ZZ"  =  "1"  /* Reattachment of Pancreas, Percutaneous Endoscopic Approach */
"0FN00ZZ"  =  "1"  /* Release Liver, Open Approach */
"0FN03ZZ"  =  "1"  /* Release Liver, Percutaneous Approach */
"0FN04ZZ"  =  "1"  /* Release Liver, Percutaneous Endoscopic Approach */
"0FN10ZZ"  =  "1"  /* Release Right Lobe Liver, Open Approach */
"0FN13ZZ"  =  "1"  /* Release Right Lobe Liver, Percutaneous Approach */
"0FN14ZZ"  =  "1"  /* Release Right Lobe Liver, Percutaneous Endoscopic Approach */
"0FN20ZZ"  =  "1"  /* Release Left Lobe Liver, Open Approach */
"0FN23ZZ"  =  "1"  /* Release Left Lobe Liver, Percutaneous Approach */
"0FN24ZZ"  =  "1"  /* Release Left Lobe Liver, Percutaneous Endoscopic Approach */
"0FN40ZZ"  =  "1"  /* Release Gallbladder, Open Approach */
"0FN43ZZ"  =  "1"  /* Release Gallbladder, Percutaneous Approach */
"0FN44ZZ"  =  "1"  /* Release Gallbladder, Percutaneous Endoscopic Approach */
"0FN50ZZ"  =  "1"  /* Release Right Hepatic Duct, Open Approach */
"0FN53ZZ"  =  "1"  /* Release Right Hepatic Duct, Percutaneous Approach */
"0FN54ZZ"  =  "1"  /* Release Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0FN57ZZ"  =  "1"  /* Release Right Hepatic Duct, Via Opening */
"0FN58ZZ"  =  "1"  /* Release Right Hepatic Duct, Endo */
"0FN60ZZ"  =  "1"  /* Release Left Hepatic Duct, Open Approach */
"0FN63ZZ"  =  "1"  /* Release Left Hepatic Duct, Percutaneous Approach */
"0FN64ZZ"  =  "1"  /* Release Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0FN67ZZ"  =  "1"  /* Release Left Hepatic Duct, Via Natural or Artificial Opening */
"0FN68ZZ"  =  "1"  /* Release Left Hepatic Duct, Endo */
"0FN80ZZ"  =  "1"  /* Release Cystic Duct, Open Approach */
"0FN83ZZ"  =  "1"  /* Release Cystic Duct, Percutaneous Approach */
"0FN84ZZ"  =  "1"  /* Release Cystic Duct, Percutaneous Endoscopic Approach */
"0FN87ZZ"  =  "1"  /* Release Cystic Duct, Via Natural or Artificial Opening */
"0FN88ZZ"  =  "1"  /* Release Cystic Duct, Endo */
"0FN90ZZ"  =  "1"  /* Release Common Bile Duct, Open Approach */
"0FN93ZZ"  =  "1"  /* Release Common Bile Duct, Percutaneous Approach */
"0FN94ZZ"  =  "1"  /* Release Common Bile Duct, Percutaneous Endoscopic Approach */
"0FN97ZZ"  =  "1"  /* Release Common Bile Duct, Via Natural or Artificial Opening */
"0FN98ZZ"  =  "1"  /* Release Common Bile Duct, Endo */
"0FNC0ZZ"  =  "1"  /* Release Ampulla of Vater, Open Approach */
"0FNC3ZZ"  =  "1"  /* Release Ampulla of Vater, Percutaneous Approach */
"0FNC4ZZ"  =  "1"  /* Release Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FNC7ZZ"  =  "1"  /* Release Ampulla of Vater, Via Natural or Artificial Opening */
"0FNC8ZZ"  =  "1"  /* Release Ampulla of Vater, Endo */
"0FND0ZZ"  =  "1"  /* Release Pancreatic Duct, Open Approach */
"0FND3ZZ"  =  "1"  /* Release Pancreatic Duct, Percutaneous Approach */
"0FND4ZZ"  =  "1"  /* Release Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FND7ZZ"  =  "1"  /* Release Pancreatic Duct, Via Natural or Artificial Opening */
"0FND8ZZ"  =  "1"  /* Release Pancreatic Duct, Endo */
"0FNF0ZZ"  =  "1"  /* Release Accessory Pancreatic Duct, Open Approach */
"0FNF3ZZ"  =  "1"  /* Release Accessory Pancreatic Duct, Percutaneous Approach */
"0FNF4ZZ"  =  "1"  /* Release Accessory Pancreatic Duct, Perc Endo Approach */
"0FNF7ZZ"  =  "1"  /* Release Accessory Pancreatic Duct, Via Opening */
"0FNF8ZZ"  =  "1"  /* Release Accessory Pancreatic Duct, Endo */
"0FNG0ZZ"  =  "1"  /* Release Pancreas, Open Approach */
"0FNG3ZZ"  =  "1"  /* Release Pancreas, Percutaneous Approach */
"0FNG4ZZ"  =  "1"  /* Release Pancreas, Percutaneous Endoscopic Approach */
"0FP000Z"  =  "1"  /* Removal of Drainage Device from Liver, Open Approach */
"0FP002Z"  =  "1"  /* Removal of Monitoring Device from Liver, Open Approach */
"0FP003Z"  =  "1"  /* Removal of Infusion Device from Liver, Open Approach */
"0FP030Z"  =  "1"  /* Removal of Drainage Device from Liver, Percutaneous Approach */
"0FP032Z"  =  "1"  /* Removal of Monitoring Device from Liver, Perc Approach */
"0FP033Z"  =  "1"  /* Removal of Infusion Device from Liver, Percutaneous Approach */
"0FP040Z"  =  "1"  /* Removal of Drainage Device from Liver, Perc Endo Approach */
"0FP042Z"  =  "1"  /* Removal of Monitoring Device from Liver, Perc Endo Approach */
"0FP043Z"  =  "1"  /* Removal of Infusion Device from Liver, Perc Endo Approach */
"0FP400Z"  =  "1"  /* Removal of Drainage Device from Gallbladder, Open Approach */
"0FP402Z"  =  "1"  /* Removal of Monitoring Device from Gallbladder, Open Approach */
"0FP403Z"  =  "1"  /* Removal of Infusion Device from Gallbladder, Open Approach */
"0FP40DZ"  =  "1"  /* Removal of Intralum Dev from Gallbladder, Open Approach */
"0FP430Z"  =  "1"  /* Removal of Drainage Device from Gallbladder, Perc Approach */
"0FP432Z"  =  "1"  /* Removal of Monitoring Device from Gallbladder, Perc Approach */
"0FP433Z"  =  "1"  /* Removal of Infusion Device from Gallbladder, Perc Approach */
"0FP43DZ"  =  "1"  /* Removal of Intralum Dev from Gallbladder, Perc Approach */
"0FP440Z"  =  "1"  /* Removal of Drain Dev from Gallbladder, Perc Endo Approach */
"0FP442Z"  =  "1"  /* Removal of Monitor Dev from Gallbladder, Perc Endo Approach */
"0FP443Z"  =  "1"  /* Removal of Infusion Dev from Gallbladder, Perc Endo Approach */
"0FP44DZ"  =  "1"  /* Removal of Intralum Dev from Gallbladder, Perc Endo Approach */
"0FPB00Z"  =  "1"  /* Removal of Drain Dev from Hepatobil Duct, Open Approach */
"0FPB01Z"  =  "1"  /* Removal of Radioact Elem from Hepatobil Duct, Open Approach */
"0FPB02Z"  =  "1"  /* Removal of Monitor Dev from Hepatobil Duct, Open Approach */
"0FPB03Z"  =  "1"  /* Removal of Infusion Dev from Hepatobil Duct, Open Approach */
"0FPB07Z"  =  "1"  /* Removal of Autol Sub from Hepatobil Duct, Open Approach */
"0FPB0CZ"  =  "1"  /* Removal of Extralum Dev from Hepatobil Duct, Open Approach */
"0FPB0DZ"  =  "1"  /* Removal of Intralum Dev from Hepatobil Duct, Open Approach */
"0FPB0JZ"  =  "1"  /* Removal of Synth Sub from Hepatobil Duct, Open Approach */
"0FPB0KZ"  =  "1"  /* Removal of Nonaut Sub from Hepatobil Duct, Open Approach */
"0FPB30Z"  =  "1"  /* Removal of Drain Dev from Hepatobil Duct, Perc Approach */
"0FPB31Z"  =  "1"  /* Removal of Radioact Elem from Hepatobil Duct, Perc Approach */
"0FPB32Z"  =  "1"  /* Removal of Monitor Dev from Hepatobil Duct, Perc Approach */
"0FPB33Z"  =  "1"  /* Removal of Infusion Dev from Hepatobil Duct, Perc Approach */
"0FPB37Z"  =  "1"  /* Removal of Autol Sub from Hepatobil Duct, Perc Approach */
"0FPB3CZ"  =  "1"  /* Removal of Extralum Dev from Hepatobil Duct, Perc Approach */
"0FPB3DZ"  =  "1"  /* Removal of Intralum Dev from Hepatobil Duct, Perc Approach */
"0FPB3JZ"  =  "1"  /* Removal of Synth Sub from Hepatobil Duct, Perc Approach */
"0FPB3KZ"  =  "1"  /* Removal of Nonaut Sub from Hepatobil Duct, Perc Approach */
"0FPB40Z"  =  "1"  /* Removal of Drain Dev from Hepatobil Duct, Perc Endo Approach */
"0FPB41Z"  =  "1"  /* Remove Radioact Elem from Hepatobil Duct, Perc Endo */
"0FPB42Z"  =  "1"  /* Remove Monitor Dev from Hepatobil Duct, Perc Endo */
"0FPB43Z"  =  "1"  /* Remove Infusion Dev from Hepatobil Duct, Perc Endo */
"0FPB47Z"  =  "1"  /* Removal of Autol Sub from Hepatobil Duct, Perc Endo Approach */
"0FPB4CZ"  =  "1"  /* Remove Extralum Dev from Hepatobil Duct, Perc Endo */
"0FPB4DZ"  =  "1"  /* Remove Intralum Dev from Hepatobil Duct, Perc Endo */
"0FPB4JZ"  =  "1"  /* Removal of Synth Sub from Hepatobil Duct, Perc Endo Approach */
"0FPB4KZ"  =  "1"  /* Remove of Nonaut Sub from Hepatobil Duct, Perc Endo Approach */
"0FPB70Z"  =  "1"  /* Removal of Drainage Device from Hepatobil Duct, Via Opening */
"0FPB71Z"  =  "1"  /* Removal of Radioact Elem from Hepatobil Duct, Via Opening */
"0FPB72Z"  =  "1"  /* Removal of Monitor Dev from Hepatobil Duct, Via Opening */
"0FPB73Z"  =  "1"  /* Removal of Infusion Device from Hepatobil Duct, Via Opening */
"0FPB77Z"  =  "1"  /* Removal of Autol Sub from Hepatobil Duct, Via Opening */
"0FPB7CZ"  =  "1"  /* Removal of Extralum Dev from Hepatobil Duct, Via Opening */
"0FPB7DZ"  =  "1"  /* Removal of Intralum Dev from Hepatobil Duct, Via Opening */
"0FPB7JZ"  =  "1"  /* Removal of Synth Sub from Hepatobil Duct, Via Opening */
"0FPB7KZ"  =  "1"  /* Removal of Nonaut Sub from Hepatobil Duct, Via Opening */
"0FPB80Z"  =  "1"  /* Removal of Drainage Device from Hepatobiliary Duct, Endo */
"0FPB81Z"  =  "1"  /* Removal of Radioactive Element from Hepatobiliary Duct, Endo */
"0FPB82Z"  =  "1"  /* Removal of Monitoring Device from Hepatobiliary Duct, Endo */
"0FPB83Z"  =  "1"  /* Removal of Infusion Device from Hepatobiliary Duct, Endo */
"0FPB87Z"  =  "1"  /* Removal of Autol Sub from Hepatobil Duct, Endo */
"0FPB8CZ"  =  "1"  /* Removal of Extraluminal Device from Hepatobiliary Duct, Endo */
"0FPB8DZ"  =  "1"  /* Removal of Intraluminal Device from Hepatobiliary Duct, Endo */
"0FPB8JZ"  =  "1"  /* Removal of Synthetic Substitute from Hepatobil Duct, Endo */
"0FPB8KZ"  =  "1"  /* Removal of Nonaut Sub from Hepatobil Duct, Endo */
"0FPD00Z"  =  "1"  /* Removal of Drainage Device from Pancreat Duct, Open Approach */
"0FPD01Z"  =  "1"  /* Removal of Radioact Elem from Pancreat Duct, Open Approach */
"0FPD02Z"  =  "1"  /* Removal of Monitor Dev from Pancreat Duct, Open Approach */
"0FPD03Z"  =  "1"  /* Removal of Infusion Device from Pancreat Duct, Open Approach */
"0FPD07Z"  =  "1"  /* Removal of Autol Sub from Pancreat Duct, Open Approach */
"0FPD0CZ"  =  "1"  /* Removal of Extralum Dev from Pancreat Duct, Open Approach */
"0FPD0DZ"  =  "1"  /* Removal of Intralum Dev from Pancreat Duct, Open Approach */
"0FPD0JZ"  =  "1"  /* Removal of Synth Sub from Pancreat Duct, Open Approach */
"0FPD0KZ"  =  "1"  /* Removal of Nonaut Sub from Pancreat Duct, Open Approach */
"0FPD30Z"  =  "1"  /* Removal of Drainage Device from Pancreat Duct, Perc Approach */
"0FPD31Z"  =  "1"  /* Removal of Radioact Elem from Pancreat Duct, Perc Approach */
"0FPD32Z"  =  "1"  /* Removal of Monitor Dev from Pancreat Duct, Perc Approach */
"0FPD33Z"  =  "1"  /* Removal of Infusion Device from Pancreat Duct, Perc Approach */
"0FPD37Z"  =  "1"  /* Removal of Autol Sub from Pancreat Duct, Perc Approach */
"0FPD3CZ"  =  "1"  /* Removal of Extralum Dev from Pancreat Duct, Perc Approach */
"0FPD3DZ"  =  "1"  /* Removal of Intralum Dev from Pancreat Duct, Perc Approach */
"0FPD3JZ"  =  "1"  /* Removal of Synth Sub from Pancreat Duct, Perc Approach */
"0FPD3KZ"  =  "1"  /* Removal of Nonaut Sub from Pancreat Duct, Perc Approach */
"0FPD40Z"  =  "1"  /* Removal of Drain Dev from Pancreat Duct, Perc Endo Approach */
"0FPD41Z"  =  "1"  /* Remove Radioact Elem from Pancreat Duct, Perc Endo */
"0FPD42Z"  =  "1"  /* Remove of Monitor Dev from Pancreat Duct, Perc Endo Approach */
"0FPD43Z"  =  "1"  /* Remove Infusion Dev from Pancreat Duct, Perc Endo */
"0FPD47Z"  =  "1"  /* Removal of Autol Sub from Pancreat Duct, Perc Endo Approach */
"0FPD4CZ"  =  "1"  /* Remove Extralum Dev from Pancreat Duct, Perc Endo */
"0FPD4DZ"  =  "1"  /* Remove Intralum Dev from Pancreat Duct, Perc Endo */
"0FPD4JZ"  =  "1"  /* Removal of Synth Sub from Pancreat Duct, Perc Endo Approach */
"0FPD4KZ"  =  "1"  /* Removal of Nonaut Sub from Pancreat Duct, Perc Endo Approach */
"0FPD70Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Via Opening */
"0FPD71Z"  =  "1"  /* Removal of Radioact Elem from Pancreat Duct, Via Opening */
"0FPD72Z"  =  "1"  /* Removal of Monitoring Device from Pancreat Duct, Via Opening */
"0FPD73Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Via Opening */
"0FPD77Z"  =  "1"  /* Removal of Autol Sub from Pancreat Duct, Via Opening */
"0FPD7CZ"  =  "1"  /* Removal of Extralum Dev from Pancreat Duct, Via Opening */
"0FPD7DZ"  =  "1"  /* Removal of Intralum Dev from Pancreat Duct, Via Opening */
"0FPD7JZ"  =  "1"  /* Removal of Synth Sub from Pancreat Duct, Via Opening */
"0FPD7KZ"  =  "1"  /* Removal of Nonaut Sub from Pancreat Duct, Via Opening */
"0FPD80Z"  =  "1"  /* Removal of Drainage Device from Pancreatic Duct, Endo */
"0FPD81Z"  =  "1"  /* Removal of Radioactive Element from Pancreatic Duct, Endo */
"0FPD82Z"  =  "1"  /* Removal of Monitoring Device from Pancreatic Duct, Endo */
"0FPD83Z"  =  "1"  /* Removal of Infusion Device from Pancreatic Duct, Endo */
"0FPD87Z"  =  "1"  /* Removal of Autol Sub from Pancreat Duct, Endo */
"0FPD8CZ"  =  "1"  /* Removal of Extraluminal Device from Pancreatic Duct, Endo */
"0FPD8DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreatic Duct, Endo */
"0FPD8JZ"  =  "1"  /* Removal of Synthetic Substitute from Pancreatic Duct, Endo */
"0FPD8KZ"  =  "1"  /* Removal of Nonaut Sub from Pancreat Duct, Endo */
"0FPG00Z"  =  "1"  /* Removal of Drainage Device from Pancreas, Open Approach */
"0FPG02Z"  =  "1"  /* Removal of Monitoring Device from Pancreas, Open Approach */
"0FPG03Z"  =  "1"  /* Removal of Infusion Device from Pancreas, Open Approach */
"0FPG0DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreas, Open Approach */
"0FPG30Z"  =  "1"  /* Removal of Drainage Device from Pancreas, Perc Approach */
"0FPG32Z"  =  "1"  /* Removal of Monitoring Device from Pancreas, Perc Approach */
"0FPG33Z"  =  "1"  /* Removal of Infusion Device from Pancreas, Perc Approach */
"0FPG3DZ"  =  "1"  /* Removal of Intraluminal Device from Pancreas, Perc Approach */
"0FPG40Z"  =  "1"  /* Removal of Drainage Device from Pancreas, Perc Endo Approach */
"0FPG42Z"  =  "1"  /* Removal of Monitor Dev from Pancreas, Perc Endo Approach */
"0FPG43Z"  =  "1"  /* Removal of Infusion Device from Pancreas, Perc Endo Approach */
"0FPG4DZ"  =  "1"  /* Removal of Intralum Dev from Pancreas, Perc Endo Approach */
"0FPGXDZ"  =  "1"  /* Removal of Intralum Dev from Pancreas, Extern Approach */
"0FQ00ZZ"  =  "1"  /* Repair Liver, Open Approach */
"0FQ03ZZ"  =  "1"  /* Repair Liver, Percutaneous Approach */
"0FQ04ZZ"  =  "1"  /* Repair Liver, Percutaneous Endoscopic Approach */
"0FQ10ZZ"  =  "1"  /* Repair Right Lobe Liver, Open Approach */
"0FQ13ZZ"  =  "1"  /* Repair Right Lobe Liver, Percutaneous Approach */
"0FQ14ZZ"  =  "1"  /* Repair Right Lobe Liver, Percutaneous Endoscopic Approach */
"0FQ20ZZ"  =  "1"  /* Repair Left Lobe Liver, Open Approach */
"0FQ23ZZ"  =  "1"  /* Repair Left Lobe Liver, Percutaneous Approach */
"0FQ24ZZ"  =  "1"  /* Repair Left Lobe Liver, Percutaneous Endoscopic Approach */
"0FQ40ZZ"  =  "1"  /* Repair Gallbladder, Open Approach */
"0FQ43ZZ"  =  "1"  /* Repair Gallbladder, Percutaneous Approach */
"0FQ44ZZ"  =  "1"  /* Repair Gallbladder, Percutaneous Endoscopic Approach */
"0FQ50ZZ"  =  "1"  /* Repair Right Hepatic Duct, Open Approach */
"0FQ53ZZ"  =  "1"  /* Repair Right Hepatic Duct, Percutaneous Approach */
"0FQ54ZZ"  =  "1"  /* Repair Right Hepatic Duct, Percutaneous Endoscopic Approach */
"0FQ57ZZ"  =  "1"  /* Repair Right Hepatic Duct, Via Natural or Artificial Opening */
"0FQ58ZZ"  =  "1"  /* Repair Right Hepatic Duct, Endo */
"0FQ60ZZ"  =  "1"  /* Repair Left Hepatic Duct, Open Approach */
"0FQ63ZZ"  =  "1"  /* Repair Left Hepatic Duct, Percutaneous Approach */
"0FQ64ZZ"  =  "1"  /* Repair Left Hepatic Duct, Percutaneous Endoscopic Approach */
"0FQ67ZZ"  =  "1"  /* Repair Left Hepatic Duct, Via Natural or Artificial Opening */
"0FQ68ZZ"  =  "1"  /* Repair Left Hepatic Duct, Endo */
"0FQ80ZZ"  =  "1"  /* Repair Cystic Duct, Open Approach */
"0FQ83ZZ"  =  "1"  /* Repair Cystic Duct, Percutaneous Approach */
"0FQ84ZZ"  =  "1"  /* Repair Cystic Duct, Percutaneous Endoscopic Approach */
"0FQ87ZZ"  =  "1"  /* Repair Cystic Duct, Via Natural or Artificial Opening */
"0FQ88ZZ"  =  "1"  /* Repair Cystic Duct, Endo */
"0FQ90ZZ"  =  "1"  /* Repair Common Bile Duct, Open Approach */
"0FQ93ZZ"  =  "1"  /* Repair Common Bile Duct, Percutaneous Approach */
"0FQ94ZZ"  =  "1"  /* Repair Common Bile Duct, Percutaneous Endoscopic Approach */
"0FQ97ZZ"  =  "1"  /* Repair Common Bile Duct, Via Natural or Artificial Opening */
"0FQ98ZZ"  =  "1"  /* Repair Common Bile Duct, Endo */
"0FQC0ZZ"  =  "1"  /* Repair Ampulla of Vater, Open Approach */
"0FQC3ZZ"  =  "1"  /* Repair Ampulla of Vater, Percutaneous Approach */
"0FQC4ZZ"  =  "1"  /* Repair Ampulla of Vater, Percutaneous Endoscopic Approach */
"0FQC7ZZ"  =  "1"  /* Repair Ampulla of Vater, Via Natural or Artificial Opening */
"0FQC8ZZ"  =  "1"  /* Repair Ampulla of Vater, Endo */
"0FQD0ZZ"  =  "1"  /* Repair Pancreatic Duct, Open Approach */
"0FQD3ZZ"  =  "1"  /* Repair Pancreatic Duct, Percutaneous Approach */
"0FQD4ZZ"  =  "1"  /* Repair Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FQD7ZZ"  =  "1"  /* Repair Pancreatic Duct, Via Natural or Artificial Opening */
"0FQD8ZZ"  =  "1"  /* Repair Pancreatic Duct, Endo */
"0FQF0ZZ"  =  "1"  /* Repair Accessory Pancreatic Duct, Open Approach */
"0FQF3ZZ"  =  "1"  /* Repair Accessory Pancreatic Duct, Percutaneous Approach */
"0FQF4ZZ"  =  "1"  /* Repair Accessory Pancreatic Duct, Perc Endo Approach */
"0FQF7ZZ"  =  "1"  /* Repair Accessory Pancreatic Duct, Via Opening */
"0FQF8ZZ"  =  "1"  /* Repair Accessory Pancreatic Duct, Endo */
"0FQG0ZZ"  =  "1"  /* Repair Pancreas, Open Approach */
"0FQG3ZZ"  =  "1"  /* Repair Pancreas, Percutaneous Approach */
"0FQG4ZZ"  =  "1"  /* Repair Pancreas, Percutaneous Endoscopic Approach */
"0FR507Z"  =  "1"  /* Replacement of R Hepatic Duct with Autol Sub, Open Approach */
"0FR50JZ"  =  "1"  /* Replacement of R Hepatic Duct with Synth Sub, Open Approach */
"0FR50KZ"  =  "1"  /* Replacement of R Hepatic Duct with Nonaut Sub, Open Approach */
"0FR547Z"  =  "1"  /* Replace of R Hepatic Duct with Autol Sub, Perc Endo Approach */
"0FR54JZ"  =  "1"  /* Replace of R Hepatic Duct with Synth Sub, Perc Endo Approach */
"0FR54KZ"  =  "1"  /* Replace R Hepatic Duct w Nonaut Sub, Perc Endo */
"0FR607Z"  =  "1"  /* Replacement of L Hepatic Duct with Autol Sub, Open Approach */
"0FR60JZ"  =  "1"  /* Replacement of L Hepatic Duct with Synth Sub, Open Approach */
"0FR60KZ"  =  "1"  /* Replacement of L Hepatic Duct with Nonaut Sub, Open Approach */
"0FR647Z"  =  "1"  /* Replace of L Hepatic Duct with Autol Sub, Perc Endo Approach */
"0FR64JZ"  =  "1"  /* Replace of L Hepatic Duct with Synth Sub, Perc Endo Approach */
"0FR64KZ"  =  "1"  /* Replace L Hepatic Duct w Nonaut Sub, Perc Endo */
"0FR807Z"  =  "1"  /* Replacement of Cystic Duct with Autol Sub, Open Approach */
"0FR80JZ"  =  "1"  /* Replacement of Cystic Duct with Synth Sub, Open Approach */
"0FR80KZ"  =  "1"  /* Replacement of Cystic Duct with Nonaut Sub, Open Approach */
"0FR847Z"  =  "1"  /* Replace of Cystic Duct with Autol Sub, Perc Endo Approach */
"0FR84JZ"  =  "1"  /* Replace of Cystic Duct with Synth Sub, Perc Endo Approach */
"0FR84KZ"  =  "1"  /* Replace of Cystic Duct with Nonaut Sub, Perc Endo Approach */
"0FR907Z"  =  "1"  /* Replacement of Com Bile Duct with Autol Sub, Open Approach */
"0FR90JZ"  =  "1"  /* Replacement of Com Bile Duct with Synth Sub, Open Approach */
"0FR90KZ"  =  "1"  /* Replacement of Com Bile Duct with Nonaut Sub, Open Approach */
"0FR947Z"  =  "1"  /* Replace of Com Bile Duct with Autol Sub, Perc Endo Approach */
"0FR94JZ"  =  "1"  /* Replace of Com Bile Duct with Synth Sub, Perc Endo Approach */
"0FR94KZ"  =  "1"  /* Replace of Com Bile Duct with Nonaut Sub, Perc Endo Approach */
"0FRC07Z"  =  "1"  /* Replacement of Ampulla Vater with Autol Sub, Open Approach */
"0FRC0JZ"  =  "1"  /* Replacement of Ampulla Vater with Synth Sub, Open Approach */
"0FRC0KZ"  =  "1"  /* Replacement of Ampulla Vater with Nonaut Sub, Open Approach */
"0FRC47Z"  =  "1"  /* Replace of Ampulla Vater with Autol Sub, Perc Endo Approach */
"0FRC4JZ"  =  "1"  /* Replace of Ampulla Vater with Synth Sub, Perc Endo Approach */
"0FRC4KZ"  =  "1"  /* Replace of Ampulla Vater with Nonaut Sub, Perc Endo Approach */
"0FRD07Z"  =  "1"  /* Replacement of Pancreatic Duct with Autol Sub, Open Approach */
"0FRD0JZ"  =  "1"  /* Replacement of Pancreatic Duct with Synth Sub, Open Approach */
"0FRD0KZ"  =  "1"  /* Replacement of Pancreat Duct with Nonaut Sub, Open Approach */
"0FRD47Z"  =  "1"  /* Replace of Pancreat Duct with Autol Sub, Perc Endo Approach */
"0FRD4JZ"  =  "1"  /* Replace of Pancreat Duct with Synth Sub, Perc Endo Approach */
"0FRD4KZ"  =  "1"  /* Replace of Pancreat Duct with Nonaut Sub, Perc Endo Approach */
"0FRF07Z"  =  "1"  /* Replace of Access Pancr Duct with Autol Sub, Open Approach */
"0FRF0JZ"  =  "1"  /* Replace of Access Pancr Duct with Synth Sub, Open Approach */
"0FRF0KZ"  =  "1"  /* Replace of Access Pancr Duct with Nonaut Sub, Open Approach */
"0FRF47Z"  =  "1"  /* Replace Access Pancr Duct w Autol Sub, Perc Endo */
"0FRF4JZ"  =  "1"  /* Replace Access Pancr Duct w Synth Sub, Perc Endo */
"0FRF4KZ"  =  "1"  /* Replace Access Pancr Duct w Nonaut Sub, Perc Endo */
"0FS00ZZ"  =  "1"  /* Reposition Liver, Open Approach */
"0FS04ZZ"  =  "1"  /* Reposition Liver, Percutaneous Endoscopic Approach */
"0FS40ZZ"  =  "1"  /* Reposition Gallbladder, Open Approach */
"0FS44ZZ"  =  "1"  /* Reposition Gallbladder, Percutaneous Endoscopic Approach */
"0FS50ZZ"  =  "1"  /* Reposition Right Hepatic Duct, Open Approach */
"0FS54ZZ"  =  "1"  /* Reposition Right Hepatic Duct, Perc Endo Approach */
"0FS60ZZ"  =  "1"  /* Reposition Left Hepatic Duct, Open Approach */
"0FS64ZZ"  =  "1"  /* Reposition Left Hepatic Duct, Perc Endo Approach */
"0FS80ZZ"  =  "1"  /* Reposition Cystic Duct, Open Approach */
"0FS84ZZ"  =  "1"  /* Reposition Cystic Duct, Percutaneous Endoscopic Approach */
"0FS90ZZ"  =  "1"  /* Reposition Common Bile Duct, Open Approach */
"0FS94ZZ"  =  "1"  /* Reposition Common Bile Duct, Perc Endo Approach */
"0FSC0ZZ"  =  "1"  /* Reposition Ampulla of Vater, Open Approach */
"0FSC4ZZ"  =  "1"  /* Reposition Ampulla of Vater, Perc Endo Approach */
"0FSD0ZZ"  =  "1"  /* Reposition Pancreatic Duct, Open Approach */
"0FSD4ZZ"  =  "1"  /* Reposition Pancreatic Duct, Percutaneous Endoscopic Approach */
"0FSF0ZZ"  =  "1"  /* Reposition Accessory Pancreatic Duct, Open Approach */
"0FSF4ZZ"  =  "1"  /* Reposition Accessory Pancreatic Duct, Perc Endo Approach */
"0FSG0ZZ"  =  "1"  /* Reposition Pancreas, Open Approach */
"0FSG4ZZ"  =  "1"  /* Reposition Pancreas, Percutaneous Endoscopic Approach */
"0FT00ZZ"  =  "1"  /* Resection of Liver, Open Approach */
"0FT04ZZ"  =  "1"  /* Resection of Liver, Percutaneous Endoscopic Approach */
"0FT10ZZ"  =  "1"  /* Resection of Right Lobe Liver, Open Approach */
"0FT14ZZ"  =  "1"  /* Resection of Right Lobe Liver, Perc Endo Approach */
"0FT20ZZ"  =  "1"  /* Resection of Left Lobe Liver, Open Approach */
"0FT24ZZ"  =  "1"  /* Resection of Left Lobe Liver, Perc Endo Approach */
"0FT40ZZ"  =  "1"  /* Resection of Gallbladder, Open Approach */
"0FT44ZZ"  =  "1"  /* Resection of Gallbladder, Percutaneous Endoscopic Approach */
"0FT50ZZ"  =  "1"  /* Resection of Right Hepatic Duct, Open Approach */
"0FT54ZZ"  =  "1"  /* Resection of Right Hepatic Duct, Perc Endo Approach */
"0FT57ZZ"  =  "1"  /* Resection of Right Hepatic Duct, Via Opening */
"0FT58ZZ"  =  "1"  /* Resection of Right Hepatic Duct, Endo */
"0FT60ZZ"  =  "1"  /* Resection of Left Hepatic Duct, Open Approach */
"0FT64ZZ"  =  "1"  /* Resection of Left Hepatic Duct, Perc Endo Approach */
"0FT67ZZ"  =  "1"  /* Resection of Left Hepatic Duct, Via Opening */
"0FT68ZZ"  =  "1"  /* Resection of Left Hepatic Duct, Endo */
"0FT80ZZ"  =  "1"  /* Resection of Cystic Duct, Open Approach */
"0FT84ZZ"  =  "1"  /* Resection of Cystic Duct, Percutaneous Endoscopic Approach */
"0FT87ZZ"  =  "1"  /* Resection of Cystic Duct, Via Natural or Artificial Opening */
"0FT88ZZ"  =  "1"  /* Resection of Cystic Duct, Endo */
"0FT90ZZ"  =  "1"  /* Resection of Common Bile Duct, Open Approach */
"0FT94ZZ"  =  "1"  /* Resection of Common Bile Duct, Perc Endo Approach */
"0FT97ZZ"  =  "1"  /* Resection of Common Bile Duct, Via Opening */
"0FT98ZZ"  =  "1"  /* Resection of Common Bile Duct, Endo */
"0FTC0ZZ"  =  "1"  /* Resection of Ampulla of Vater, Open Approach */
"0FTC4ZZ"  =  "1"  /* Resection of Ampulla of Vater, Perc Endo Approach */
"0FTC7ZZ"  =  "1"  /* Resection of Ampulla of Vater, Via Opening */
"0FTC8ZZ"  =  "1"  /* Resection of Ampulla of Vater, Endo */
"0FTD0ZZ"  =  "1"  /* Resection of Pancreatic Duct, Open Approach */
"0FTD7ZZ"  =  "1"  /* Resection of Pancreatic Duct, Via Opening */
"0FTF0ZZ"  =  "1"  /* Resection of Accessory Pancreatic Duct, Open Approach */
"0FTF7ZZ"  =  "1"  /* Resection of Accessory Pancreatic Duct, Via Opening */
"0FTG0ZZ"  =  "1"  /* Resection of Pancreas, Open Approach */
"0FTG4ZZ"  =  "1"  /* Resection of Pancreas, Percutaneous Endoscopic Approach */
"0FU507Z"  =  "1"  /* Supplement Right Hepatic Duct with Autol Sub, Open Approach */
"0FU50JZ"  =  "1"  /* Supplement Right Hepatic Duct with Synth Sub, Open Approach */
"0FU50KZ"  =  "1"  /* Supplement Right Hepatic Duct with Nonaut Sub, Open Approach */
"0FU537Z"  =  "1"  /* Supplement Right Hepatic Duct with Autol Sub, Perc Approach */
"0FU53JZ"  =  "1"  /* Supplement Right Hepatic Duct with Synth Sub, Perc Approach */
"0FU53KZ"  =  "1"  /* Supplement Right Hepatic Duct with Nonaut Sub, Perc Approach */
"0FU547Z"  =  "1"  /* Supplement R Hepatic Duct with Autol Sub, Perc Endo Approach */
"0FU54JZ"  =  "1"  /* Supplement R Hepatic Duct with Synth Sub, Perc Endo Approach */
"0FU54KZ"  =  "1"  /* Supplement R Hepatic Duct w Nonaut Sub, Perc Endo */
"0FU607Z"  =  "1"  /* Supplement Left Hepatic Duct with Autol Sub, Open Approach */
"0FU60JZ"  =  "1"  /* Supplement Left Hepatic Duct with Synth Sub, Open Approach */
"0FU60KZ"  =  "1"  /* Supplement Left Hepatic Duct with Nonaut Sub, Open Approach */
"0FU637Z"  =  "1"  /* Supplement Left Hepatic Duct with Autol Sub, Perc Approach */
"0FU63JZ"  =  "1"  /* Supplement Left Hepatic Duct with Synth Sub, Perc Approach */
"0FU63KZ"  =  "1"  /* Supplement Left Hepatic Duct with Nonaut Sub, Perc Approach */
"0FU647Z"  =  "1"  /* Supplement L Hepatic Duct with Autol Sub, Perc Endo Approach */
"0FU64JZ"  =  "1"  /* Supplement L Hepatic Duct with Synth Sub, Perc Endo Approach */
"0FU64KZ"  =  "1"  /* Supplement L Hepatic Duct w Nonaut Sub, Perc Endo */
"0FU807Z"  =  "1"  /* Supplement Cystic Duct with Autol Sub, Open Approach */
"0FU80JZ"  =  "1"  /* Supplement Cystic Duct with Synth Sub, Open Approach */
"0FU80KZ"  =  "1"  /* Supplement Cystic Duct with Nonaut Sub, Open Approach */
"0FU837Z"  =  "1"  /* Supplement Cystic Duct with Autol Sub, Perc Approach */
"0FU83JZ"  =  "1"  /* Supplement Cystic Duct with Synth Sub, Perc Approach */
"0FU83KZ"  =  "1"  /* Supplement Cystic Duct with Nonaut Sub, Perc Approach */
"0FU847Z"  =  "1"  /* Supplement Cystic Duct with Autol Sub, Perc Endo Approach */
"0FU84JZ"  =  "1"  /* Supplement Cystic Duct with Synth Sub, Perc Endo Approach */
"0FU84KZ"  =  "1"  /* Supplement Cystic Duct with Nonaut Sub, Perc Endo Approach */
"0FU907Z"  =  "1"  /* Supplement Common Bile Duct with Autol Sub, Open Approach */
"0FU90JZ"  =  "1"  /* Supplement Common Bile Duct with Synth Sub, Open Approach */
"0FU90KZ"  =  "1"  /* Supplement Common Bile Duct with Nonaut Sub, Open Approach */
"0FU937Z"  =  "1"  /* Supplement Common Bile Duct with Autol Sub, Perc Approach */
"0FU93JZ"  =  "1"  /* Supplement Common Bile Duct with Synth Sub, Perc Approach */
"0FU93KZ"  =  "1"  /* Supplement Common Bile Duct with Nonaut Sub, Perc Approach */
"0FU947Z"  =  "1"  /* Supplement Com Bile Duct with Autol Sub, Perc Endo Approach */
"0FU94JZ"  =  "1"  /* Supplement Com Bile Duct with Synth Sub, Perc Endo Approach */
"0FU94KZ"  =  "1"  /* Supplement Com Bile Duct with Nonaut Sub, Perc Endo Approach */
"0FUC07Z"  =  "1"  /* Supplement Ampulla of Vater with Autol Sub, Open Approach */
"0FUC0JZ"  =  "1"  /* Supplement Ampulla of Vater with Synth Sub, Open Approach */
"0FUC0KZ"  =  "1"  /* Supplement Ampulla of Vater with Nonaut Sub, Open Approach */
"0FUC37Z"  =  "1"  /* Supplement Ampulla of Vater with Autol Sub, Perc Approach */
"0FUC3JZ"  =  "1"  /* Supplement Ampulla of Vater with Synth Sub, Perc Approach */
"0FUC3KZ"  =  "1"  /* Supplement Ampulla of Vater with Nonaut Sub, Perc Approach */
"0FUC47Z"  =  "1"  /* Supplement Ampulla Vater with Autol Sub, Perc Endo Approach */
"0FUC4JZ"  =  "1"  /* Supplement Ampulla Vater with Synth Sub, Perc Endo Approach */
"0FUC4KZ"  =  "1"  /* Supplement Ampulla Vater with Nonaut Sub, Perc Endo Approach */
"0FUD07Z"  =  "1"  /* Supplement Pancreatic Duct with Autol Sub, Open Approach */
"0FUD0JZ"  =  "1"  /* Supplement Pancreatic Duct with Synth Sub, Open Approach */
"0FUD0KZ"  =  "1"  /* Supplement Pancreatic Duct with Nonaut Sub, Open Approach */
"0FUD37Z"  =  "1"  /* Supplement Pancreatic Duct with Autol Sub, Perc Approach */
"0FUD3JZ"  =  "1"  /* Supplement Pancreatic Duct with Synth Sub, Perc Approach */
"0FUD3KZ"  =  "1"  /* Supplement Pancreatic Duct with Nonaut Sub, Perc Approach */
"0FUD47Z"  =  "1"  /* Supplement Pancreat Duct with Autol Sub, Perc Endo Approach */
"0FUD4JZ"  =  "1"  /* Supplement Pancreat Duct with Synth Sub, Perc Endo Approach */
"0FUD4KZ"  =  "1"  /* Supplement Pancreat Duct with Nonaut Sub, Perc Endo Approach */
"0FUF07Z"  =  "1"  /* Supplement Access Pancr Duct with Autol Sub, Open Approach */
"0FUF0JZ"  =  "1"  /* Supplement Access Pancr Duct with Synth Sub, Open Approach */
"0FUF0KZ"  =  "1"  /* Supplement Access Pancr Duct with Nonaut Sub, Open Approach */
"0FUF37Z"  =  "1"  /* Supplement Access Pancr Duct with Autol Sub, Perc Approach */
"0FUF3JZ"  =  "1"  /* Supplement Access Pancr Duct with Synth Sub, Perc Approach */
"0FUF3KZ"  =  "1"  /* Supplement Access Pancr Duct with Nonaut Sub, Perc Approach */
"0FUF47Z"  =  "1"  /* Supplement Access Pancr Duct w Autol Sub, Perc Endo */
"0FUF4JZ"  =  "1"  /* Supplement Access Pancr Duct w Synth Sub, Perc Endo */
"0FUF4KZ"  =  "1"  /* Supplement Access Pancr Duct w Nonaut Sub, Perc Endo */
"0FV50CZ"  =  "1"  /* Restrict of R Hepatic Duct with Extralum Dev, Open Approach */
"0FV50DZ"  =  "1"  /* Restrict of R Hepatic Duct with Intralum Dev, Open Approach */
"0FV50ZZ"  =  "1"  /* Restriction of Right Hepatic Duct, Open Approach */
"0FV60CZ"  =  "1"  /* Restrict of L Hepatic Duct with Extralum Dev, Open Approach */
"0FV60DZ"  =  "1"  /* Restrict of L Hepatic Duct with Intralum Dev, Open Approach */
"0FV60ZZ"  =  "1"  /* Restriction of Left Hepatic Duct, Open Approach */
"0FV80CZ"  =  "1"  /* Restriction of Cystic Duct with Extralum Dev, Open Approach */
"0FV80DZ"  =  "1"  /* Restriction of Cystic Duct with Intralum Dev, Open Approach */
"0FV80ZZ"  =  "1"  /* Restriction of Cystic Duct, Open Approach */
"0FV90CZ"  =  "1"  /* Restrict of Com Bile Duct with Extralum Dev, Open Approach */
"0FV90DZ"  =  "1"  /* Restrict of Com Bile Duct with Intralum Dev, Open Approach */
"0FV90ZZ"  =  "1"  /* Restriction of Common Bile Duct, Open Approach */
"0FVC0CZ"  =  "1"  /* Restrict of Ampulla Vater with Extralum Dev, Open Approach */
"0FVC0DZ"  =  "1"  /* Restrict of Ampulla Vater with Intralum Dev, Open Approach */
"0FVC0ZZ"  =  "1"  /* Restriction of Ampulla of Vater, Open Approach */
"0FVC3CZ"  =  "1"  /* Restrict of Ampulla Vater with Extralum Dev, Perc Approach */
"0FVC3DZ"  =  "1"  /* Restrict of Ampulla Vater with Intralum Dev, Perc Approach */
"0FVC3ZZ"  =  "1"  /* Restriction of Ampulla of Vater, Percutaneous Approach */
"0FVC4CZ"  =  "1"  /* Restrict Ampulla Vater w Extralum Dev, Perc Endo */
"0FVC4DZ"  =  "1"  /* Restrict Ampulla Vater w Intralum Dev, Perc Endo */
"0FVC4ZZ"  =  "1"  /* Restriction of Ampulla of Vater, Perc Endo Approach */
"0FVC7DZ"  =  "1"  /* Restriction of Ampulla Vater with Intralum Dev, Via Opening */
"0FVC7ZZ"  =  "1"  /* Restriction of Ampulla of Vater, Via Opening */
"0FVC8DZ"  =  "1"  /* Restriction of Ampulla of Vater with Intralum Dev, Endo */
"0FVC8ZZ"  =  "1"  /* Restriction of Ampulla of Vater, Endo */
"0FVD0CZ"  =  "1"  /* Restrict of Pancreat Duct with Extralum Dev, Open Approach */
"0FVD0DZ"  =  "1"  /* Restrict of Pancreat Duct with Intralum Dev, Open Approach */
"0FVD0ZZ"  =  "1"  /* Restriction of Pancreatic Duct, Open Approach */
"0FVD3CZ"  =  "1"  /* Restrict of Pancreat Duct with Extralum Dev, Perc Approach */
"0FVD3DZ"  =  "1"  /* Restrict of Pancreat Duct with Intralum Dev, Perc Approach */
"0FVD3ZZ"  =  "1"  /* Restriction of Pancreatic Duct, Percutaneous Approach */
"0FVD4CZ"  =  "1"  /* Restrict Pancreat Duct w Extralum Dev, Perc Endo */
"0FVD4DZ"  =  "1"  /* Restrict Pancreat Duct w Intralum Dev, Perc Endo */
"0FVD4ZZ"  =  "1"  /* Restriction of Pancreatic Duct, Perc Endo Approach */
"0FVD7DZ"  =  "1"  /* Restriction of Pancreat Duct with Intralum Dev, Via Opening */
"0FVD7ZZ"  =  "1"  /* Restriction of Pancreatic Duct, Via Opening */
"0FVD8DZ"  =  "1"  /* Restriction of Pancreatic Duct with Intralum Dev, Endo */
"0FVD8ZZ"  =  "1"  /* Restriction of Pancreatic Duct, Endo */
"0FVF0CZ"  =  "1"  /* Restrict Access Pancr Duct w Extralum Dev, Open */
"0FVF0DZ"  =  "1"  /* Restrict Access Pancr Duct w Intralum Dev, Open */
"0FVF0ZZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct, Open Approach */
"0FVF3CZ"  =  "1"  /* Restrict Access Pancr Duct w Extralum Dev, Perc */
"0FVF3DZ"  =  "1"  /* Restrict Access Pancr Duct w Intralum Dev, Perc */
"0FVF3ZZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct, Perc Approach */
"0FVF4CZ"  =  "1"  /* Restrict Access Pancr Duct w Extralum Dev, Perc Endo */
"0FVF4DZ"  =  "1"  /* Restrict Access Pancr Duct w Intralum Dev, Perc Endo */
"0FVF4ZZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct, Perc Endo Approach */
"0FVF7DZ"  =  "1"  /* Restrict of Access Pancr Duct with Intralum Dev, Via Opening */
"0FVF7ZZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct, Via Opening */
"0FVF8DZ"  =  "1"  /* Restriction of Access Pancr Duct with Intralum Dev, Endo */
"0FVF8ZZ"  =  "1"  /* Restriction of Accessory Pancreatic Duct, Endo */
"0FW000Z"  =  "1"  /* Revision of Drainage Device in Liver, Open Approach */
"0FW002Z"  =  "1"  /* Revision of Monitoring Device in Liver, Open Approach */
"0FW003Z"  =  "1"  /* Revision of Infusion Device in Liver, Open Approach */
"0FW030Z"  =  "1"  /* Revision of Drainage Device in Liver, Percutaneous Approach */
"0FW032Z"  =  "1"  /* Revision of Monitoring Device in Liver, Perc Approach */
"0FW033Z"  =  "1"  /* Revision of Infusion Device in Liver, Percutaneous Approach */
"0FW040Z"  =  "1"  /* Revision of Drainage Device in Liver, Perc Endo Approach */
"0FW042Z"  =  "1"  /* Revision of Monitoring Device in Liver, Perc Endo Approach */
"0FW043Z"  =  "1"  /* Revision of Infusion Device in Liver, Perc Endo Approach */
"0FW400Z"  =  "1"  /* Revision of Drainage Device in Gallbladder, Open Approach */
"0FW402Z"  =  "1"  /* Revision of Monitoring Device in Gallbladder, Open Approach */
"0FW403Z"  =  "1"  /* Revision of Infusion Device in Gallbladder, Open Approach */
"0FW40DZ"  =  "1"  /* Revision of Intralum Dev in Gallbladder, Open Approach */
"0FW430Z"  =  "1"  /* Revision of Drainage Device in Gallbladder, Perc Approach */
"0FW432Z"  =  "1"  /* Revision of Monitoring Device in Gallbladder, Perc Approach */
"0FW433Z"  =  "1"  /* Revision of Infusion Device in Gallbladder, Perc Approach */
"0FW43DZ"  =  "1"  /* Revision of Intralum Dev in Gallbladder, Perc Approach */
"0FW440Z"  =  "1"  /* Revision of Drain Dev in Gallbladder, Perc Endo Approach */
"0FW442Z"  =  "1"  /* Revision of Monitor Dev in Gallbladder, Perc Endo Approach */
"0FW443Z"  =  "1"  /* Revision of Infusion Dev in Gallbladder, Perc Endo Approach */
"0FW44DZ"  =  "1"  /* Revision of Intralum Dev in Gallbladder, Perc Endo Approach */
"0FWB00Z"  =  "1"  /* Revision of Drainage Device in Hepatobil Duct, Open Approach */
"0FWB02Z"  =  "1"  /* Revision of Monitor Dev in Hepatobil Duct, Open Approach */
"0FWB03Z"  =  "1"  /* Revision of Infusion Device in Hepatobil Duct, Open Approach */
"0FWB07Z"  =  "1"  /* Revision of Autol Sub in Hepatobil Duct, Open Approach */
"0FWB0CZ"  =  "1"  /* Revision of Extralum Dev in Hepatobil Duct, Open Approach */
"0FWB0DZ"  =  "1"  /* Revision of Intralum Dev in Hepatobil Duct, Open Approach */
"0FWB0JZ"  =  "1"  /* Revision of Synth Sub in Hepatobil Duct, Open Approach */
"0FWB0KZ"  =  "1"  /* Revision of Nonaut Sub in Hepatobil Duct, Open Approach */
"0FWB30Z"  =  "1"  /* Revision of Drainage Device in Hepatobil Duct, Perc Approach */
"0FWB32Z"  =  "1"  /* Revision of Monitor Dev in Hepatobil Duct, Perc Approach */
"0FWB33Z"  =  "1"  /* Revision of Infusion Device in Hepatobil Duct, Perc Approach */
"0FWB37Z"  =  "1"  /* Revision of Autol Sub in Hepatobil Duct, Perc Approach */
"0FWB3CZ"  =  "1"  /* Revision of Extralum Dev in Hepatobil Duct, Perc Approach */
"0FWB3DZ"  =  "1"  /* Revision of Intralum Dev in Hepatobil Duct, Perc Approach */
"0FWB3JZ"  =  "1"  /* Revision of Synth Sub in Hepatobil Duct, Perc Approach */
"0FWB3KZ"  =  "1"  /* Revision of Nonaut Sub in Hepatobil Duct, Perc Approach */
"0FWB40Z"  =  "1"  /* Revision of Drain Dev in Hepatobil Duct, Perc Endo Approach */
"0FWB42Z"  =  "1"  /* Revise of Monitor Dev in Hepatobil Duct, Perc Endo Approach */
"0FWB43Z"  =  "1"  /* Revise of Infusion Dev in Hepatobil Duct, Perc Endo Approach */
"0FWB47Z"  =  "1"  /* Revision of Autol Sub in Hepatobil Duct, Perc Endo Approach */
"0FWB4CZ"  =  "1"  /* Revise of Extralum Dev in Hepatobil Duct, Perc Endo Approach */
"0FWB4DZ"  =  "1"  /* Revise of Intralum Dev in Hepatobil Duct, Perc Endo Approach */
"0FWB4JZ"  =  "1"  /* Revision of Synth Sub in Hepatobil Duct, Perc Endo Approach */
"0FWB4KZ"  =  "1"  /* Revision of Nonaut Sub in Hepatobil Duct, Perc Endo Approach */
"0FWB70Z"  =  "1"  /* Revision of Drainage Device in Hepatobil Duct, Via Opening */
"0FWB72Z"  =  "1"  /* Revision of Monitoring Device in Hepatobil Duct, Via Opening */
"0FWB73Z"  =  "1"  /* Revision of Infusion Device in Hepatobil Duct, Via Opening */
"0FWB77Z"  =  "1"  /* Revision of Autol Sub in Hepatobil Duct, Via Opening */
"0FWB7CZ"  =  "1"  /* Revision of Extralum Dev in Hepatobil Duct, Via Opening */
"0FWB7DZ"  =  "1"  /* Revision of Intralum Dev in Hepatobil Duct, Via Opening */
"0FWB7JZ"  =  "1"  /* Revision of Synth Sub in Hepatobil Duct, Via Opening */
"0FWB7KZ"  =  "1"  /* Revision of Nonaut Sub in Hepatobil Duct, Via Opening */
"0FWB80Z"  =  "1"  /* Revision of Drainage Device in Hepatobiliary Duct, Endo */
"0FWB82Z"  =  "1"  /* Revision of Monitoring Device in Hepatobiliary Duct, Endo */
"0FWB83Z"  =  "1"  /* Revision of Infusion Device in Hepatobiliary Duct, Endo */
"0FWB87Z"  =  "1"  /* Revision of Autol Sub in Hepatobil Duct, Endo */
"0FWB8CZ"  =  "1"  /* Revision of Extraluminal Device in Hepatobiliary Duct, Endo */
"0FWB8DZ"  =  "1"  /* Revision of Intraluminal Device in Hepatobiliary Duct, Endo */
"0FWB8JZ"  =  "1"  /* Revision of Synthetic Substitute in Hepatobiliary Duct, Endo */
"0FWB8KZ"  =  "1"  /* Revision of Nonaut Sub in Hepatobil Duct, Endo */
"0FWD00Z"  =  "1"  /* Revision of Drainage Device in Pancreat Duct, Open Approach */
"0FWD02Z"  =  "1"  /* Revision of Monitor Dev in Pancreat Duct, Open Approach */
"0FWD03Z"  =  "1"  /* Revision of Infusion Device in Pancreat Duct, Open Approach */
"0FWD07Z"  =  "1"  /* Revision of Autol Sub in Pancreat Duct, Open Approach */
"0FWD0CZ"  =  "1"  /* Revision of Extralum Dev in Pancreat Duct, Open Approach */
"0FWD0DZ"  =  "1"  /* Revision of Intralum Dev in Pancreat Duct, Open Approach */
"0FWD0JZ"  =  "1"  /* Revision of Synth Sub in Pancreat Duct, Open Approach */
"0FWD0KZ"  =  "1"  /* Revision of Nonaut Sub in Pancreat Duct, Open Approach */
"0FWD30Z"  =  "1"  /* Revision of Drainage Device in Pancreat Duct, Perc Approach */
"0FWD32Z"  =  "1"  /* Revision of Monitor Dev in Pancreat Duct, Perc Approach */
"0FWD33Z"  =  "1"  /* Revision of Infusion Device in Pancreat Duct, Perc Approach */
"0FWD37Z"  =  "1"  /* Revision of Autol Sub in Pancreat Duct, Perc Approach */
"0FWD3CZ"  =  "1"  /* Revision of Extralum Dev in Pancreat Duct, Perc Approach */
"0FWD3DZ"  =  "1"  /* Revision of Intralum Dev in Pancreat Duct, Perc Approach */
"0FWD3JZ"  =  "1"  /* Revision of Synth Sub in Pancreat Duct, Perc Approach */
"0FWD3KZ"  =  "1"  /* Revision of Nonaut Sub in Pancreat Duct, Perc Approach */
"0FWD40Z"  =  "1"  /* Revision of Drain Dev in Pancreat Duct, Perc Endo Approach */
"0FWD42Z"  =  "1"  /* Revision of Monitor Dev in Pancreat Duct, Perc Endo Approach */
"0FWD43Z"  =  "1"  /* Revise of Infusion Dev in Pancreat Duct, Perc Endo Approach */
"0FWD47Z"  =  "1"  /* Revision of Autol Sub in Pancreat Duct, Perc Endo Approach */
"0FWD4CZ"  =  "1"  /* Revise of Extralum Dev in Pancreat Duct, Perc Endo Approach */
"0FWD4DZ"  =  "1"  /* Revise of Intralum Dev in Pancreat Duct, Perc Endo Approach */
"0FWD4JZ"  =  "1"  /* Revision of Synth Sub in Pancreat Duct, Perc Endo Approach */
"0FWD4KZ"  =  "1"  /* Revision of Nonaut Sub in Pancreat Duct, Perc Endo Approach */
"0FWD70Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Via Opening */
"0FWD72Z"  =  "1"  /* Revision of Monitoring Device in Pancreat Duct, Via Opening */
"0FWD73Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Via Opening */
"0FWD77Z"  =  "1"  /* Revision of Autol Sub in Pancreat Duct, Via Opening */
"0FWD7CZ"  =  "1"  /* Revision of Extralum Dev in Pancreat Duct, Via Opening */
"0FWD7DZ"  =  "1"  /* Revision of Intralum Dev in Pancreat Duct, Via Opening */
"0FWD7JZ"  =  "1"  /* Revision of Synth Sub in Pancreat Duct, Via Opening */
"0FWD7KZ"  =  "1"  /* Revision of Nonaut Sub in Pancreat Duct, Via Opening */
"0FWD80Z"  =  "1"  /* Revision of Drainage Device in Pancreatic Duct, Endo */
"0FWD82Z"  =  "1"  /* Revision of Monitoring Device in Pancreatic Duct, Endo */
"0FWD83Z"  =  "1"  /* Revision of Infusion Device in Pancreatic Duct, Endo */
"0FWD87Z"  =  "1"  /* Revision of Autol Sub in Pancreat Duct, Endo */
"0FWD8CZ"  =  "1"  /* Revision of Extraluminal Device in Pancreatic Duct, Endo */
"0FWD8DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreatic Duct, Endo */
"0FWD8JZ"  =  "1"  /* Revision of Synthetic Substitute in Pancreatic Duct, Endo */
"0FWD8KZ"  =  "1"  /* Revision of Nonaut Sub in Pancreat Duct, Endo */
"0FWG00Z"  =  "1"  /* Revision of Drainage Device in Pancreas, Open Approach */
"0FWG02Z"  =  "1"  /* Revision of Monitoring Device in Pancreas, Open Approach */
"0FWG03Z"  =  "1"  /* Revision of Infusion Device in Pancreas, Open Approach */
"0FWG0DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreas, Open Approach */
"0FWG30Z"  =  "1"  /* Revision of Drainage Device in Pancreas, Perc Approach */
"0FWG32Z"  =  "1"  /* Revision of Monitoring Device in Pancreas, Perc Approach */
"0FWG33Z"  =  "1"  /* Revision of Infusion Device in Pancreas, Perc Approach */
"0FWG3DZ"  =  "1"  /* Revision of Intraluminal Device in Pancreas, Perc Approach */
"0FWG40Z"  =  "1"  /* Revision of Drainage Device in Pancreas, Perc Endo Approach */
"0FWG42Z"  =  "1"  /* Revision of Monitor Dev in Pancreas, Perc Endo Approach */
"0FWG43Z"  =  "1"  /* Revision of Infusion Device in Pancreas, Perc Endo Approach */
"0FWG4DZ"  =  "1"  /* Revision of Intralum Dev in Pancreas, Perc Endo Approach */
"0FY00Z0"  =  "1"  /* Transplantation of Liver, Allogeneic, Open Approach */
"0FY00Z1"  =  "1"  /* Transplantation of Liver, Syngeneic, Open Approach */
"0FY00Z2"  =  "1"  /* Transplantation of Liver, Zooplastic, Open Approach */
"0FYG0Z0"  =  "1"  /* Transplantation of Pancreas, Allogeneic, Open Approach */
"0FYG0Z1"  =  "1"  /* Transplantation of Pancreas, Syngeneic, Open Approach */
"0FYG0Z2"  =  "1"  /* Transplantation of Pancreas, Zooplastic, Open Approach */
"0G500ZZ"  =  "1"  /* Destruction of Pituitary Gland, Open Approach */
"0G503ZZ"  =  "1"  /* Destruction of Pituitary Gland, Percutaneous Approach */
"0G504ZZ"  =  "1"  /* Destruction of Pituitary Gland, Perc Endo Approach */
"0G510ZZ"  =  "1"  /* Destruction of Pineal Body, Open Approach */
"0G513ZZ"  =  "1"  /* Destruction of Pineal Body, Percutaneous Approach */
"0G514ZZ"  =  "1"  /* Destruction of Pineal Body, Percutaneous Endoscopic Approach */
"0G520ZZ"  =  "1"  /* Destruction of Left Adrenal Gland, Open Approach */
"0G523ZZ"  =  "1"  /* Destruction of Left Adrenal Gland, Percutaneous Approach */
"0G524ZZ"  =  "1"  /* Destruction of Left Adrenal Gland, Perc Endo Approach */
"0G530ZZ"  =  "1"  /* Destruction of Right Adrenal Gland, Open Approach */
"0G533ZZ"  =  "1"  /* Destruction of Right Adrenal Gland, Percutaneous Approach */
"0G534ZZ"  =  "1"  /* Destruction of Right Adrenal Gland, Perc Endo Approach */
"0G540ZZ"  =  "1"  /* Destruction of Bilateral Adrenal Glands, Open Approach */
"0G543ZZ"  =  "1"  /* Destruction of Bilateral Adrenal Glands, Perc Approach */
"0G544ZZ"  =  "1"  /* Destruction of Bilateral Adrenal Glands, Perc Endo Approach */
"0G5G0ZZ"  =  "1"  /* Destruction of Left Thyroid Gland Lobe, Open Approach */
"0G5G3ZZ"  =  "1"  /* Destruction of Left Thyroid Gland Lobe, Perc Approach */
"0G5G4ZZ"  =  "1"  /* Destruction of Left Thyroid Gland Lobe, Perc Endo Approach */
"0G5H0ZZ"  =  "1"  /* Destruction of Right Thyroid Gland Lobe, Open Approach */
"0G5H3ZZ"  =  "1"  /* Destruction of Right Thyroid Gland Lobe, Perc Approach */
"0G5H4ZZ"  =  "1"  /* Destruction of Right Thyroid Gland Lobe, Perc Endo Approach */
"0G5K0ZZ"  =  "1"  /* Destruction of Thyroid Gland, Open Approach */
"0G5K3ZZ"  =  "1"  /* Destruction of Thyroid Gland, Percutaneous Approach */
"0G5K4ZZ"  =  "1"  /* Destruction of Thyroid Gland, Perc Endo Approach */
"0G5L0ZZ"  =  "1"  /* Destruction of R Sup Parathyroid, Open Approach */
"0G5L3ZZ"  =  "1"  /* Destruction of R Sup Parathyroid, Perc Approach */
"0G5L4ZZ"  =  "1"  /* Destruction of R Sup Parathyroid, Perc Endo Approach */
"0G5M0ZZ"  =  "1"  /* Destruction of L Sup Parathyroid, Open Approach */
"0G5M3ZZ"  =  "1"  /* Destruction of L Sup Parathyroid, Perc Approach */
"0G5M4ZZ"  =  "1"  /* Destruction of L Sup Parathyroid, Perc Endo Approach */
"0G5N0ZZ"  =  "1"  /* Destruction of R Inf Parathyroid, Open Approach */
"0G5N3ZZ"  =  "1"  /* Destruction of R Inf Parathyroid, Perc Approach */
"0G5N4ZZ"  =  "1"  /* Destruction of R Inf Parathyroid, Perc Endo Approach */
"0G5P0ZZ"  =  "1"  /* Destruction of L Inf Parathyroid, Open Approach */
"0G5P3ZZ"  =  "1"  /* Destruction of L Inf Parathyroid, Perc Approach */
"0G5P4ZZ"  =  "1"  /* Destruction of L Inf Parathyroid, Perc Endo Approach */
"0G5Q0ZZ"  =  "1"  /* Destruction of Multiple Parathyroid Glands, Open Approach */
"0G5Q3ZZ"  =  "1"  /* Destruction of Multiple Parathyroid Glands, Perc Approach */
"0G5Q4ZZ"  =  "1"  /* Destruction of Mult Parathyroid, Perc Endo Approach */
"0G5R0ZZ"  =  "1"  /* Destruction of Parathyroid Gland, Open Approach */
"0G5R3ZZ"  =  "1"  /* Destruction of Parathyroid Gland, Percutaneous Approach */
"0G5R4ZZ"  =  "1"  /* Destruction of Parathyroid Gland, Perc Endo Approach */
"0G800ZZ"  =  "1"  /* Division of Pituitary Gland, Open Approach */
"0G803ZZ"  =  "1"  /* Division of Pituitary Gland, Percutaneous Approach */
"0G804ZZ"  =  "1"  /* Division of Pituitary Gland, Perc Endo Approach */
"0G8J0ZZ"  =  "1"  /* Division of Thyroid Gland Isthmus, Open Approach */
"0G8J3ZZ"  =  "1"  /* Division of Thyroid Gland Isthmus, Percutaneous Approach */
"0G8J4ZZ"  =  "1"  /* Division of Thyroid Gland Isthmus, Perc Endo Approach */
"0G9000Z"  =  "1"  /* Drainage of Pituitary Gland with Drain Dev, Open Approach */
"0G900ZX"  =  "1"  /* Drainage of Pituitary Gland, Open Approach, Diagnostic */
"0G900ZZ"  =  "1"  /* Drainage of Pituitary Gland, Open Approach */
"0G9030Z"  =  "1"  /* Drainage of Pituitary Gland with Drain Dev, Perc Approach */
"0G903ZX"  =  "1"  /* Drainage of Pituitary Gland, Percutaneous Approach, Diagn */
"0G903ZZ"  =  "1"  /* Drainage of Pituitary Gland, Percutaneous Approach */
"0G9040Z"  =  "1"  /* Drain of Pituitary Gland with Drain Dev, Perc Endo Approach */
"0G904ZX"  =  "1"  /* Drainage of Pituitary Gland, Perc Endo Approach, Diagn */
"0G904ZZ"  =  "1"  /* Drainage of Pituitary Gland, Perc Endo Approach */
"0G9100Z"  =  "1"  /* Drainage of Pineal Body with Drainage Device, Open Approach */
"0G910ZX"  =  "1"  /* Drainage of Pineal Body, Open Approach, Diagnostic */
"0G910ZZ"  =  "1"  /* Drainage of Pineal Body, Open Approach */
"0G9130Z"  =  "1"  /* Drainage of Pineal Body with Drainage Device, Perc Approach */
"0G913ZX"  =  "1"  /* Drainage of Pineal Body, Percutaneous Approach, Diagnostic */
"0G913ZZ"  =  "1"  /* Drainage of Pineal Body, Percutaneous Approach */
"0G9140Z"  =  "1"  /* Drainage of Pineal Body with Drain Dev, Perc Endo Approach */
"0G914ZX"  =  "1"  /* Drainage of Pineal Body, Perc Endo Approach, Diagn */
"0G914ZZ"  =  "1"  /* Drainage of Pineal Body, Percutaneous Endoscopic Approach */
"0G9200Z"  =  "1"  /* Drainage of Left Adrenal Gland with Drain Dev, Open Approach */
"0G920ZX"  =  "1"  /* Drainage of Left Adrenal Gland, Open Approach, Diagnostic */
"0G920ZZ"  =  "1"  /* Drainage of Left Adrenal Gland, Open Approach */
"0G9230Z"  =  "1"  /* Drainage of Left Adrenal Gland with Drain Dev, Perc Approach */
"0G923ZZ"  =  "1"  /* Drainage of Left Adrenal Gland, Percutaneous Approach */
"0G9240Z"  =  "1"  /* Drain of L Adrenal Gland with Drain Dev, Perc Endo Approach */
"0G924ZZ"  =  "1"  /* Drainage of Left Adrenal Gland, Perc Endo Approach */
"0G9300Z"  =  "1"  /* Drainage of R Adrenal Gland with Drain Dev, Open Approach */
"0G930ZX"  =  "1"  /* Drainage of Right Adrenal Gland, Open Approach, Diagnostic */
"0G930ZZ"  =  "1"  /* Drainage of Right Adrenal Gland, Open Approach */
"0G9330Z"  =  "1"  /* Drainage of R Adrenal Gland with Drain Dev, Perc Approach */
"0G933ZZ"  =  "1"  /* Drainage of Right Adrenal Gland, Percutaneous Approach */
"0G9340Z"  =  "1"  /* Drain of R Adrenal Gland with Drain Dev, Perc Endo Approach */
"0G934ZZ"  =  "1"  /* Drainage of Right Adrenal Gland, Perc Endo Approach */
"0G9400Z"  =  "1"  /* Drainage of Bi Adrenal Gland with Drain Dev, Open Approach */
"0G940ZX"  =  "1"  /* Drainage of Bilateral Adrenal Glands, Open Approach, Diagn */
"0G940ZZ"  =  "1"  /* Drainage of Bilateral Adrenal Glands, Open Approach */
"0G9430Z"  =  "1"  /* Drainage of Bi Adrenal Gland with Drain Dev, Perc Approach */
"0G943ZZ"  =  "1"  /* Drainage of Bilateral Adrenal Glands, Percutaneous Approach */
"0G9440Z"  =  "1"  /* Drain of Bi Adrenal Gland with Drain Dev, Perc Endo Approach */
"0G944ZZ"  =  "1"  /* Drainage of Bilateral Adrenal Glands, Perc Endo Approach */
"0G9G00Z"  =  "1"  /* Drainage of L Thyroid Lobe with Drain Dev, Open Approach */
"0G9G0ZX"  =  "1"  /* Drainage of Left Thyroid Gland Lobe, Open Approach, Diagn */
"0G9G0ZZ"  =  "1"  /* Drainage of Left Thyroid Gland Lobe, Open Approach */
"0G9H00Z"  =  "1"  /* Drainage of R Thyroid Lobe with Drain Dev, Open Approach */
"0G9H0ZX"  =  "1"  /* Drainage of Right Thyroid Gland Lobe, Open Approach, Diagn */
"0G9H0ZZ"  =  "1"  /* Drainage of Right Thyroid Gland Lobe, Open Approach */
"0G9K00Z"  =  "1"  /* Drainage of Thyroid Gland with Drain Dev, Open Approach */
"0G9K0ZX"  =  "1"  /* Drainage of Thyroid Gland, Open Approach, Diagnostic */
"0G9K0ZZ"  =  "1"  /* Drainage of Thyroid Gland, Open Approach */
"0G9L00Z"  =  "1"  /* Drainage of R Sup Parathyroid with Drain Dev, Open Approach */
"0G9L0ZX"  =  "1"  /* Drainage of R Sup Parathyroid, Open Approach, Diagn */
"0G9L0ZZ"  =  "1"  /* Drainage of Right Superior Parathyroid Gland, Open Approach */
"0G9L3ZX"  =  "1"  /* Drainage of R Sup Parathyroid, Perc Approach, Diagn */
"0G9L4ZX"  =  "1"  /* Drainage of R Sup Parathyroid, Perc Endo Approach, Diagn */
"0G9M00Z"  =  "1"  /* Drainage of L Sup Parathyroid with Drain Dev, Open Approach */
"0G9M0ZX"  =  "1"  /* Drainage of L Sup Parathyroid, Open Approach, Diagn */
"0G9M0ZZ"  =  "1"  /* Drainage of Left Superior Parathyroid Gland, Open Approach */
"0G9M3ZX"  =  "1"  /* Drainage of L Sup Parathyroid, Perc Approach, Diagn */
"0G9M4ZX"  =  "1"  /* Drainage of L Sup Parathyroid, Perc Endo Approach, Diagn */
"0G9N00Z"  =  "1"  /* Drainage of R Inf Parathyroid with Drain Dev, Open Approach */
"0G9N0ZX"  =  "1"  /* Drainage of R Inf Parathyroid, Open Approach, Diagn */
"0G9N0ZZ"  =  "1"  /* Drainage of Right Inferior Parathyroid Gland, Open Approach */
"0G9N3ZX"  =  "1"  /* Drainage of R Inf Parathyroid, Perc Approach, Diagn */
"0G9N4ZX"  =  "1"  /* Drainage of R Inf Parathyroid, Perc Endo Approach, Diagn */
"0G9P00Z"  =  "1"  /* Drainage of L Inf Parathyroid with Drain Dev, Open Approach */
"0G9P0ZX"  =  "1"  /* Drainage of L Inf Parathyroid, Open Approach, Diagn */
"0G9P0ZZ"  =  "1"  /* Drainage of Left Inferior Parathyroid Gland, Open Approach */
"0G9P3ZX"  =  "1"  /* Drainage of L Inf Parathyroid, Perc Approach, Diagn */
"0G9P4ZX"  =  "1"  /* Drainage of L Inf Parathyroid, Perc Endo Approach, Diagn */
"0G9Q00Z"  =  "1"  /* Drainage of Mult Parathyroid with Drain Dev, Open Approach */
"0G9Q0ZX"  =  "1"  /* Drainage of Mult Parathyroid, Open Approach, Diagn */
"0G9Q0ZZ"  =  "1"  /* Drainage of Multiple Parathyroid Glands, Open Approach */
"0G9Q3ZX"  =  "1"  /* Drainage of Mult Parathyroid, Perc Approach, Diagn */
"0G9Q4ZX"  =  "1"  /* Drainage of Mult Parathyroid, Perc Endo Approach, Diagn */
"0G9R00Z"  =  "1"  /* Drainage of Parathyroid Gland with Drain Dev, Open Approach */
"0G9R0ZX"  =  "1"  /* Drainage of Parathyroid Gland, Open Approach, Diagnostic */
"0G9R0ZZ"  =  "1"  /* Drainage of Parathyroid Gland, Open Approach */
"0G9R3ZX"  =  "1"  /* Drainage of Parathyroid Gland, Percutaneous Approach, Diagn */
"0G9R4ZX"  =  "1"  /* Drainage of Parathyroid Gland, Perc Endo Approach, Diagn */
"0GB00ZX"  =  "1"  /* Excision of Pituitary Gland, Open Approach, Diagnostic */
"0GB00ZZ"  =  "1"  /* Excision of Pituitary Gland, Open Approach */
"0GB03ZX"  =  "1"  /* Excision of Pituitary Gland, Percutaneous Approach, Diagn */
"0GB03ZZ"  =  "1"  /* Excision of Pituitary Gland, Percutaneous Approach */
"0GB04ZX"  =  "1"  /* Excision of Pituitary Gland, Perc Endo Approach, Diagn */
"0GB04ZZ"  =  "1"  /* Excision of Pituitary Gland, Perc Endo Approach */
"0GB10ZX"  =  "1"  /* Excision of Pineal Body, Open Approach, Diagnostic */
"0GB10ZZ"  =  "1"  /* Excision of Pineal Body, Open Approach */
"0GB13ZX"  =  "1"  /* Excision of Pineal Body, Percutaneous Approach, Diagnostic */
"0GB13ZZ"  =  "1"  /* Excision of Pineal Body, Percutaneous Approach */
"0GB14ZX"  =  "1"  /* Excision of Pineal Body, Perc Endo Approach, Diagn */
"0GB14ZZ"  =  "1"  /* Excision of Pineal Body, Percutaneous Endoscopic Approach */
"0GB20ZX"  =  "1"  /* Excision of Left Adrenal Gland, Open Approach, Diagnostic */
"0GB20ZZ"  =  "1"  /* Excision of Left Adrenal Gland, Open Approach */
"0GB23ZZ"  =  "1"  /* Excision of Left Adrenal Gland, Percutaneous Approach */
"0GB24ZZ"  =  "1"  /* Excision of Left Adrenal Gland, Perc Endo Approach */
"0GB30ZX"  =  "1"  /* Excision of Right Adrenal Gland, Open Approach, Diagnostic */
"0GB30ZZ"  =  "1"  /* Excision of Right Adrenal Gland, Open Approach */
"0GB33ZZ"  =  "1"  /* Excision of Right Adrenal Gland, Percutaneous Approach */
"0GB34ZZ"  =  "1"  /* Excision of Right Adrenal Gland, Perc Endo Approach */
"0GB40ZX"  =  "1"  /* Excision of Bilateral Adrenal Glands, Open Approach, Diagn */
"0GB40ZZ"  =  "1"  /* Excision of Bilateral Adrenal Glands, Open Approach */
"0GB43ZZ"  =  "1"  /* Excision of Bilateral Adrenal Glands, Percutaneous Approach */
"0GB44ZZ"  =  "1"  /* Excision of Bilateral Adrenal Glands, Perc Endo Approach */
"0GBG0ZX"  =  "1"  /* Excision of Left Thyroid Gland Lobe, Open Approach, Diagn */
"0GBG0ZZ"  =  "1"  /* Excision of Left Thyroid Gland Lobe, Open Approach */
"0GBG3ZZ"  =  "1"  /* Excision of Left Thyroid Gland Lobe, Percutaneous Approach */
"0GBG4ZZ"  =  "1"  /* Excision of Left Thyroid Gland Lobe, Perc Endo Approach */
"0GBH0ZX"  =  "1"  /* Excision of Right Thyroid Gland Lobe, Open Approach, Diagn */
"0GBH0ZZ"  =  "1"  /* Excision of Right Thyroid Gland Lobe, Open Approach */
"0GBH3ZZ"  =  "1"  /* Excision of Right Thyroid Gland Lobe, Percutaneous Approach */
"0GBH4ZZ"  =  "1"  /* Excision of Right Thyroid Gland Lobe, Perc Endo Approach */
"0GBL0ZX"  =  "1"  /* Excision of R Sup Parathyroid, Open Approach, Diagn */
"0GBL0ZZ"  =  "1"  /* Excision of Right Superior Parathyroid Gland, Open Approach */
"0GBL3ZX"  =  "1"  /* Excision of R Sup Parathyroid, Perc Approach, Diagn */
"0GBL3ZZ"  =  "1"  /* Excision of Right Superior Parathyroid Gland, Perc Approach */
"0GBL4ZX"  =  "1"  /* Excision of R Sup Parathyroid, Perc Endo Approach, Diagn */
"0GBL4ZZ"  =  "1"  /* Excision of R Sup Parathyroid, Perc Endo Approach */
"0GBM0ZX"  =  "1"  /* Excision of L Sup Parathyroid, Open Approach, Diagn */
"0GBM0ZZ"  =  "1"  /* Excision of Left Superior Parathyroid Gland, Open Approach */
"0GBM3ZX"  =  "1"  /* Excision of L Sup Parathyroid, Perc Approach, Diagn */
"0GBM3ZZ"  =  "1"  /* Excision of Left Superior Parathyroid Gland, Perc Approach */
"0GBM4ZX"  =  "1"  /* Excision of L Sup Parathyroid, Perc Endo Approach, Diagn */
"0GBM4ZZ"  =  "1"  /* Excision of L Sup Parathyroid, Perc Endo Approach */
"0GBN0ZX"  =  "1"  /* Excision of R Inf Parathyroid, Open Approach, Diagn */
"0GBN0ZZ"  =  "1"  /* Excision of Right Inferior Parathyroid Gland, Open Approach */
"0GBN3ZX"  =  "1"  /* Excision of R Inf Parathyroid, Perc Approach, Diagn */
"0GBN3ZZ"  =  "1"  /* Excision of Right Inferior Parathyroid Gland, Perc Approach */
"0GBN4ZX"  =  "1"  /* Excision of R Inf Parathyroid, Perc Endo Approach, Diagn */
"0GBN4ZZ"  =  "1"  /* Excision of R Inf Parathyroid, Perc Endo Approach */
"0GBP0ZX"  =  "1"  /* Excision of L Inf Parathyroid, Open Approach, Diagn */
"0GBP0ZZ"  =  "1"  /* Excision of Left Inferior Parathyroid Gland, Open Approach */
"0GBP3ZX"  =  "1"  /* Excision of L Inf Parathyroid, Perc Approach, Diagn */
"0GBP3ZZ"  =  "1"  /* Excision of Left Inferior Parathyroid Gland, Perc Approach */
"0GBP4ZX"  =  "1"  /* Excision of L Inf Parathyroid, Perc Endo Approach, Diagn */
"0GBP4ZZ"  =  "1"  /* Excision of L Inf Parathyroid, Perc Endo Approach */
"0GBQ0ZX"  =  "1"  /* Excision of Mult Parathyroid, Open Approach, Diagn */
"0GBQ0ZZ"  =  "1"  /* Excision of Multiple Parathyroid Glands, Open Approach */
"0GBQ3ZX"  =  "1"  /* Excision of Mult Parathyroid, Perc Approach, Diagn */
"0GBQ3ZZ"  =  "1"  /* Excision of Multiple Parathyroid Glands, Perc Approach */
"0GBQ4ZX"  =  "1"  /* Excision of Mult Parathyroid, Perc Endo Approach, Diagn */
"0GBQ4ZZ"  =  "1"  /* Excision of Multiple Parathyroid Glands, Perc Endo Approach */
"0GBR0ZX"  =  "1"  /* Excision of Parathyroid Gland, Open Approach, Diagnostic */
"0GBR0ZZ"  =  "1"  /* Excision of Parathyroid Gland, Open Approach */
"0GBR3ZX"  =  "1"  /* Excision of Parathyroid Gland, Percutaneous Approach, Diagn */
"0GBR3ZZ"  =  "1"  /* Excision of Parathyroid Gland, Percutaneous Approach */
"0GBR4ZX"  =  "1"  /* Excision of Parathyroid Gland, Perc Endo Approach, Diagn */
"0GBR4ZZ"  =  "1"  /* Excision of Parathyroid Gland, Perc Endo Approach */
"0GC00ZZ"  =  "1"  /* Extirpation of Matter from Pituitary Gland, Open Approach */
"0GC03ZZ"  =  "1"  /* Extirpation of Matter from Pituitary Gland, Perc Approach */
"0GC04ZZ"  =  "1"  /* Extirpate of Matter from Pituitary Gland, Perc Endo Approach */
"0GC10ZZ"  =  "1"  /* Extirpation of Matter from Pineal Body, Open Approach */
"0GC13ZZ"  =  "1"  /* Extirpation of Matter from Pineal Body, Perc Approach */
"0GC14ZZ"  =  "1"  /* Extirpation of Matter from Pineal Body, Perc Endo Approach */
"0GC20ZZ"  =  "1"  /* Extirpation of Matter from Left Adrenal Gland, Open Approach */
"0GC23ZZ"  =  "1"  /* Extirpation of Matter from Left Adrenal Gland, Perc Approach */
"0GC24ZZ"  =  "1"  /* Extirpate of Matter from L Adrenal Gland, Perc Endo Approach */
"0GC30ZZ"  =  "1"  /* Extirpation of Matter from R Adrenal Gland, Open Approach */
"0GC33ZZ"  =  "1"  /* Extirpation of Matter from R Adrenal Gland, Perc Approach */
"0GC34ZZ"  =  "1"  /* Extirpate of Matter from R Adrenal Gland, Perc Endo Approach */
"0GC40ZZ"  =  "1"  /* Extirpation of Matter from Bi Adrenal Gland, Open Approach */
"0GC43ZZ"  =  "1"  /* Extirpation of Matter from Bi Adrenal Gland, Perc Approach */
"0GC44ZZ"  =  "1"  /* Extirpate matter from Bi Adrenal Gland, Perc Endo */
"0GCG0ZZ"  =  "1"  /* Extirpation of Matter from L Thyroid Lobe, Open Approach */
"0GCG3ZZ"  =  "1"  /* Extirpation of Matter from L Thyroid Lobe, Perc Approach */
"0GCG4ZZ"  =  "1"  /* Extirpate of Matter from L Thyroid Lobe, Perc Endo Approach */
"0GCH0ZZ"  =  "1"  /* Extirpation of Matter from R Thyroid Lobe, Open Approach */
"0GCH3ZZ"  =  "1"  /* Extirpation of Matter from R Thyroid Lobe, Perc Approach */
"0GCH4ZZ"  =  "1"  /* Extirpate of Matter from R Thyroid Lobe, Perc Endo Approach */
"0GCK0ZZ"  =  "1"  /* Extirpation of Matter from Thyroid Gland, Open Approach */
"0GCK3ZZ"  =  "1"  /* Extirpation of Matter from Thyroid Gland, Perc Approach */
"0GCK4ZZ"  =  "1"  /* Extirpation of Matter from Thyroid Gland, Perc Endo Approach */
"0GCL0ZZ"  =  "1"  /* Extirpation of Matter from R Sup Parathyroid, Open Approach */
"0GCL3ZZ"  =  "1"  /* Extirpation of Matter from R Sup Parathyroid, Perc Approach */
"0GCL4ZZ"  =  "1"  /* Extirpate matter from R Sup Parathyroid, Perc Endo */
"0GCM0ZZ"  =  "1"  /* Extirpation of Matter from L Sup Parathyroid, Open Approach */
"0GCM3ZZ"  =  "1"  /* Extirpation of Matter from L Sup Parathyroid, Perc Approach */
"0GCM4ZZ"  =  "1"  /* Extirpate matter from L Sup Parathyroid, Perc Endo */
"0GCN0ZZ"  =  "1"  /* Extirpation of Matter from R Inf Parathyroid, Open Approach */
"0GCN3ZZ"  =  "1"  /* Extirpation of Matter from R Inf Parathyroid, Perc Approach */
"0GCN4ZZ"  =  "1"  /* Extirpate matter from R Inf Parathyroid, Perc Endo */
"0GCP0ZZ"  =  "1"  /* Extirpation of Matter from L Inf Parathyroid, Open Approach */
"0GCP3ZZ"  =  "1"  /* Extirpation of Matter from L Inf Parathyroid, Perc Approach */
"0GCP4ZZ"  =  "1"  /* Extirpate matter from L Inf Parathyroid, Perc Endo */
"0GCQ0ZZ"  =  "1"  /* Extirpation of Matter from Mult Parathyroid, Open Approach */
"0GCQ3ZZ"  =  "1"  /* Extirpation of Matter from Mult Parathyroid, Perc Approach */
"0GCQ4ZZ"  =  "1"  /* Extirpate matter from Mult Parathyroid, Perc Endo */
"0GCR0ZZ"  =  "1"  /* Extirpation of Matter from Parathyroid Gland, Open Approach */
"0GCR3ZZ"  =  "1"  /* Extirpation of Matter from Parathyroid Gland, Perc Approach */
"0GCR4ZZ"  =  "1"  /* Extirpate matter from Parathyroid Gland, Perc Endo */
"0GHS02Z"  =  "1"  /* Insertion of Monitor Dev into Endocrine Gland, Open Approach */
"0GHS03Z"  =  "1"  /* Insert of Infusion Dev into Endocrine Gland, Open Approach */
"0GHS32Z"  =  "1"  /* Insertion of Monitor Dev into Endocrine Gland, Perc Approach */
"0GHS33Z"  =  "1"  /* Insert of Infusion Dev into Endocrine Gland, Perc Approach */
"0GHS42Z"  =  "1"  /* Insert Monitor Dev in Endocrine Gland, Perc Endo */
"0GHS43Z"  =  "1"  /* Insert Infusion Dev in Endocrine Gland, Perc Endo */
"0GJ00ZZ"  =  "1"  /* Inspection of Pituitary Gland, Open Approach */
"0GJ03ZZ"  =  "1"  /* Inspection of Pituitary Gland, Percutaneous Approach */
"0GJ04ZZ"  =  "1"  /* Inspection of Pituitary Gland, Perc Endo Approach */
"0GJ10ZZ"  =  "1"  /* Inspection of Pineal Body, Open Approach */
"0GJ13ZZ"  =  "1"  /* Inspection of Pineal Body, Percutaneous Approach */
"0GJ14ZZ"  =  "1"  /* Inspection of Pineal Body, Percutaneous Endoscopic Approach */
"0GJ50ZZ"  =  "1"  /* Inspection of Adrenal Gland, Open Approach */
"0GJ53ZZ"  =  "1"  /* Inspection of Adrenal Gland, Percutaneous Approach */
"0GJ54ZZ"  =  "1"  /* Inspection of Adrenal Gland, Perc Endo Approach */
"0GJK0ZZ"  =  "1"  /* Inspection of Thyroid Gland, Open Approach */
"0GJK3ZZ"  =  "1"  /* Inspection of Thyroid Gland, Percutaneous Approach */
"0GJK4ZZ"  =  "1"  /* Inspection of Thyroid Gland, Perc Endo Approach */
"0GJR0ZZ"  =  "1"  /* Inspection of Parathyroid Gland, Open Approach */
"0GJR3ZZ"  =  "1"  /* Inspection of Parathyroid Gland, Percutaneous Approach */
"0GJR4ZZ"  =  "1"  /* Inspection of Parathyroid Gland, Perc Endo Approach */
"0GJS0ZZ"  =  "1"  /* Inspection of Endocrine Gland, Open Approach */
"0GJS3ZZ"  =  "1"  /* Inspection of Endocrine Gland, Percutaneous Approach */
"0GJS4ZZ"  =  "1"  /* Inspection of Endocrine Gland, Perc Endo Approach */
"0GM20ZZ"  =  "1"  /* Reattachment of Left Adrenal Gland, Open Approach */
"0GM24ZZ"  =  "1"  /* Reattachment of Left Adrenal Gland, Perc Endo Approach */
"0GM30ZZ"  =  "1"  /* Reattachment of Right Adrenal Gland, Open Approach */
"0GM34ZZ"  =  "1"  /* Reattachment of Right Adrenal Gland, Perc Endo Approach */
"0GMG0ZZ"  =  "1"  /* Reattachment of Left Thyroid Gland Lobe, Open Approach */
"0GMG4ZZ"  =  "1"  /* Reattachment of Left Thyroid Gland Lobe, Perc Endo Approach */
"0GMH0ZZ"  =  "1"  /* Reattachment of Right Thyroid Gland Lobe, Open Approach */
"0GMH4ZZ"  =  "1"  /* Reattachment of Right Thyroid Gland Lobe, Perc Endo Approach */
"0GML0ZZ"  =  "1"  /* Reattachment of R Sup Parathyroid, Open Approach */
"0GML4ZZ"  =  "1"  /* Reattachment of R Sup Parathyroid, Perc Endo Approach */
"0GMM0ZZ"  =  "1"  /* Reattachment of L Sup Parathyroid, Open Approach */
"0GMM4ZZ"  =  "1"  /* Reattachment of L Sup Parathyroid, Perc Endo Approach */
"0GMN0ZZ"  =  "1"  /* Reattachment of R Inf Parathyroid, Open Approach */
"0GMN4ZZ"  =  "1"  /* Reattachment of R Inf Parathyroid, Perc Endo Approach */
"0GMP0ZZ"  =  "1"  /* Reattachment of L Inf Parathyroid, Open Approach */
"0GMP4ZZ"  =  "1"  /* Reattachment of L Inf Parathyroid, Perc Endo Approach */
"0GMQ0ZZ"  =  "1"  /* Reattachment of Multiple Parathyroid Glands, Open Approach */
"0GMQ4ZZ"  =  "1"  /* Reattachment of Mult Parathyroid, Perc Endo Approach */
"0GMR0ZZ"  =  "1"  /* Reattachment of Parathyroid Gland, Open Approach */
"0GMR4ZZ"  =  "1"  /* Reattachment of Parathyroid Gland, Perc Endo Approach */
"0GN00ZZ"  =  "1"  /* Release Pituitary Gland, Open Approach */
"0GN03ZZ"  =  "1"  /* Release Pituitary Gland, Percutaneous Approach */
"0GN04ZZ"  =  "1"  /* Release Pituitary Gland, Percutaneous Endoscopic Approach */
"0GN10ZZ"  =  "1"  /* Release Pineal Body, Open Approach */
"0GN13ZZ"  =  "1"  /* Release Pineal Body, Percutaneous Approach */
"0GN14ZZ"  =  "1"  /* Release Pineal Body, Percutaneous Endoscopic Approach */
"0GN20ZZ"  =  "1"  /* Release Left Adrenal Gland, Open Approach */
"0GN23ZZ"  =  "1"  /* Release Left Adrenal Gland, Percutaneous Approach */
"0GN24ZZ"  =  "1"  /* Release Left Adrenal Gland, Percutaneous Endoscopic Approach */
"0GN30ZZ"  =  "1"  /* Release Right Adrenal Gland, Open Approach */
"0GN33ZZ"  =  "1"  /* Release Right Adrenal Gland, Percutaneous Approach */
"0GN34ZZ"  =  "1"  /* Release Right Adrenal Gland, Perc Endo Approach */
"0GN40ZZ"  =  "1"  /* Release Bilateral Adrenal Glands, Open Approach */
"0GN43ZZ"  =  "1"  /* Release Bilateral Adrenal Glands, Percutaneous Approach */
"0GN44ZZ"  =  "1"  /* Release Bilateral Adrenal Glands, Perc Endo Approach */
"0GNG0ZZ"  =  "1"  /* Release Left Thyroid Gland Lobe, Open Approach */
"0GNG3ZZ"  =  "1"  /* Release Left Thyroid Gland Lobe, Percutaneous Approach */
"0GNG4ZZ"  =  "1"  /* Release Left Thyroid Gland Lobe, Perc Endo Approach */
"0GNH0ZZ"  =  "1"  /* Release Right Thyroid Gland Lobe, Open Approach */
"0GNH3ZZ"  =  "1"  /* Release Right Thyroid Gland Lobe, Percutaneous Approach */
"0GNH4ZZ"  =  "1"  /* Release Right Thyroid Gland Lobe, Perc Endo Approach */
"0GNK0ZZ"  =  "1"  /* Release Thyroid Gland, Open Approach */
"0GNK3ZZ"  =  "1"  /* Release Thyroid Gland, Percutaneous Approach */
"0GNK4ZZ"  =  "1"  /* Release Thyroid Gland, Percutaneous Endoscopic Approach */
"0GNL0ZZ"  =  "1"  /* Release Right Superior Parathyroid Gland, Open Approach */
"0GNL3ZZ"  =  "1"  /* Release Right Superior Parathyroid Gland, Perc Approach */
"0GNL4ZZ"  =  "1"  /* Release Right Superior Parathyroid Gland, Perc Endo Approach */
"0GNM0ZZ"  =  "1"  /* Release Left Superior Parathyroid Gland, Open Approach */
"0GNM3ZZ"  =  "1"  /* Release Left Superior Parathyroid Gland, Perc Approach */
"0GNM4ZZ"  =  "1"  /* Release Left Superior Parathyroid Gland, Perc Endo Approach */
"0GNN0ZZ"  =  "1"  /* Release Right Inferior Parathyroid Gland, Open Approach */
"0GNN3ZZ"  =  "1"  /* Release Right Inferior Parathyroid Gland, Perc Approach */
"0GNN4ZZ"  =  "1"  /* Release Right Inferior Parathyroid Gland, Perc Endo Approach */
"0GNP0ZZ"  =  "1"  /* Release Left Inferior Parathyroid Gland, Open Approach */
"0GNP3ZZ"  =  "1"  /* Release Left Inferior Parathyroid Gland, Perc Approach */
"0GNP4ZZ"  =  "1"  /* Release Left Inferior Parathyroid Gland, Perc Endo Approach */
"0GNQ0ZZ"  =  "1"  /* Release Multiple Parathyroid Glands, Open Approach */
"0GNQ3ZZ"  =  "1"  /* Release Multiple Parathyroid Glands, Percutaneous Approach */
"0GNQ4ZZ"  =  "1"  /* Release Multiple Parathyroid Glands, Perc Endo Approach */
"0GNR0ZZ"  =  "1"  /* Release Parathyroid Gland, Open Approach */
"0GNR3ZZ"  =  "1"  /* Release Parathyroid Gland, Percutaneous Approach */
"0GNR4ZZ"  =  "1"  /* Release Parathyroid Gland, Percutaneous Endoscopic Approach */
"0GP000Z"  =  "1"  /* Removal of Drain Dev from Pituitary Gland, Open Approach */
"0GP030Z"  =  "1"  /* Removal of Drain Dev from Pituitary Gland, Perc Approach */
"0GP040Z"  =  "1"  /* Remove of Drain Dev from Pituitary Gland, Perc Endo Approach */
"0GP100Z"  =  "1"  /* Removal of Drainage Device from Pineal Body, Open Approach */
"0GP130Z"  =  "1"  /* Removal of Drainage Device from Pineal Body, Perc Approach */
"0GP140Z"  =  "1"  /* Removal of Drain Dev from Pineal Body, Perc Endo Approach */
"0GP500Z"  =  "1"  /* Removal of Drainage Device from Adrenal Gland, Open Approach */
"0GP530Z"  =  "1"  /* Removal of Drainage Device from Adrenal Gland, Perc Approach */
"0GP540Z"  =  "1"  /* Removal of Drain Dev from Adrenal Gland, Perc Endo Approach */
"0GPK00Z"  =  "1"  /* Removal of Drainage Device from Thyroid Gland, Open Approach */
"0GPK30Z"  =  "1"  /* Removal of Drainage Device from Thyroid Gland, Perc Approach */
"0GPK40Z"  =  "1"  /* Removal of Drain Dev from Thyroid Gland, Perc Endo Approach */
"0GPR00Z"  =  "1"  /* Removal of Drain Dev from Parathyroid Gland, Open Approach */
"0GPR30Z"  =  "1"  /* Removal of Drain Dev from Parathyroid Gland, Perc Approach */
"0GPR40Z"  =  "1"  /* Remove Drain Dev from Parathyroid Gland, Perc Endo */
"0GQ00ZZ"  =  "1"  /* Repair Pituitary Gland, Open Approach */
"0GQ03ZZ"  =  "1"  /* Repair Pituitary Gland, Percutaneous Approach */
"0GQ04ZZ"  =  "1"  /* Repair Pituitary Gland, Percutaneous Endoscopic Approach */
"0GQ10ZZ"  =  "1"  /* Repair Pineal Body, Open Approach */
"0GQ13ZZ"  =  "1"  /* Repair Pineal Body, Percutaneous Approach */
"0GQ14ZZ"  =  "1"  /* Repair Pineal Body, Percutaneous Endoscopic Approach */
"0GQ20ZZ"  =  "1"  /* Repair Left Adrenal Gland, Open Approach */
"0GQ23ZZ"  =  "1"  /* Repair Left Adrenal Gland, Percutaneous Approach */
"0GQ24ZZ"  =  "1"  /* Repair Left Adrenal Gland, Percutaneous Endoscopic Approach */
"0GQ30ZZ"  =  "1"  /* Repair Right Adrenal Gland, Open Approach */
"0GQ33ZZ"  =  "1"  /* Repair Right Adrenal Gland, Percutaneous Approach */
"0GQ34ZZ"  =  "1"  /* Repair Right Adrenal Gland, Percutaneous Endoscopic Approach */
"0GQ40ZZ"  =  "1"  /* Repair Bilateral Adrenal Glands, Open Approach */
"0GQ43ZZ"  =  "1"  /* Repair Bilateral Adrenal Glands, Percutaneous Approach */
"0GQ44ZZ"  =  "1"  /* Repair Bilateral Adrenal Glands, Perc Endo Approach */
"0GQG0ZZ"  =  "1"  /* Repair Left Thyroid Gland Lobe, Open Approach */
"0GQG3ZZ"  =  "1"  /* Repair Left Thyroid Gland Lobe, Percutaneous Approach */
"0GQG4ZZ"  =  "1"  /* Repair Left Thyroid Gland Lobe, Perc Endo Approach */
"0GQH0ZZ"  =  "1"  /* Repair Right Thyroid Gland Lobe, Open Approach */
"0GQH3ZZ"  =  "1"  /* Repair Right Thyroid Gland Lobe, Percutaneous Approach */
"0GQH4ZZ"  =  "1"  /* Repair Right Thyroid Gland Lobe, Perc Endo Approach */
"0GQJ0ZZ"  =  "1"  /* Repair Thyroid Gland Isthmus, Open Approach */
"0GQJ3ZZ"  =  "1"  /* Repair Thyroid Gland Isthmus, Percutaneous Approach */
"0GQJ4ZZ"  =  "1"  /* Repair Thyroid Gland Isthmus, Perc Endo Approach */
"0GQK0ZZ"  =  "1"  /* Repair Thyroid Gland, Open Approach */
"0GQK3ZZ"  =  "1"  /* Repair Thyroid Gland, Percutaneous Approach */
"0GQK4ZZ"  =  "1"  /* Repair Thyroid Gland, Percutaneous Endoscopic Approach */
"0GQL0ZZ"  =  "1"  /* Repair Right Superior Parathyroid Gland, Open Approach */
"0GQL3ZZ"  =  "1"  /* Repair Right Superior Parathyroid Gland, Perc Approach */
"0GQL4ZZ"  =  "1"  /* Repair Right Superior Parathyroid Gland, Perc Endo Approach */
"0GQM0ZZ"  =  "1"  /* Repair Left Superior Parathyroid Gland, Open Approach */
"0GQM3ZZ"  =  "1"  /* Repair Left Superior Parathyroid Gland, Perc Approach */
"0GQM4ZZ"  =  "1"  /* Repair Left Superior Parathyroid Gland, Perc Endo Approach */
"0GQN0ZZ"  =  "1"  /* Repair Right Inferior Parathyroid Gland, Open Approach */
"0GQN3ZZ"  =  "1"  /* Repair Right Inferior Parathyroid Gland, Perc Approach */
"0GQN4ZZ"  =  "1"  /* Repair Right Inferior Parathyroid Gland, Perc Endo Approach */
"0GQP0ZZ"  =  "1"  /* Repair Left Inferior Parathyroid Gland, Open Approach */
"0GQP3ZZ"  =  "1"  /* Repair Left Inferior Parathyroid Gland, Perc Approach */
"0GQP4ZZ"  =  "1"  /* Repair Left Inferior Parathyroid Gland, Perc Endo Approach */
"0GQQ0ZZ"  =  "1"  /* Repair Multiple Parathyroid Glands, Open Approach */
"0GQQ3ZZ"  =  "1"  /* Repair Multiple Parathyroid Glands, Percutaneous Approach */
"0GQQ4ZZ"  =  "1"  /* Repair Multiple Parathyroid Glands, Perc Endo Approach */
"0GQR0ZZ"  =  "1"  /* Repair Parathyroid Gland, Open Approach */
"0GQR3ZZ"  =  "1"  /* Repair Parathyroid Gland, Percutaneous Approach */
"0GQR4ZZ"  =  "1"  /* Repair Parathyroid Gland, Percutaneous Endoscopic Approach */
"0GS20ZZ"  =  "1"  /* Reposition Left Adrenal Gland, Open Approach */
"0GS24ZZ"  =  "1"  /* Reposition Left Adrenal Gland, Perc Endo Approach */
"0GS30ZZ"  =  "1"  /* Reposition Right Adrenal Gland, Open Approach */
"0GS34ZZ"  =  "1"  /* Reposition Right Adrenal Gland, Perc Endo Approach */
"0GSG0ZZ"  =  "1"  /* Reposition Left Thyroid Gland Lobe, Open Approach */
"0GSG4ZZ"  =  "1"  /* Reposition Left Thyroid Gland Lobe, Perc Endo Approach */
"0GSH0ZZ"  =  "1"  /* Reposition Right Thyroid Gland Lobe, Open Approach */
"0GSH4ZZ"  =  "1"  /* Reposition Right Thyroid Gland Lobe, Perc Endo Approach */
"0GSL0ZZ"  =  "1"  /* Reposition Right Superior Parathyroid Gland, Open Approach */
"0GSL4ZZ"  =  "1"  /* Reposition R Sup Parathyroid, Perc Endo Approach */
"0GSM0ZZ"  =  "1"  /* Reposition Left Superior Parathyroid Gland, Open Approach */
"0GSM4ZZ"  =  "1"  /* Reposition L Sup Parathyroid, Perc Endo Approach */
"0GSN0ZZ"  =  "1"  /* Reposition Right Inferior Parathyroid Gland, Open Approach */
"0GSN4ZZ"  =  "1"  /* Reposition R Inf Parathyroid, Perc Endo Approach */
"0GSP0ZZ"  =  "1"  /* Reposition Left Inferior Parathyroid Gland, Open Approach */
"0GSP4ZZ"  =  "1"  /* Reposition L Inf Parathyroid, Perc Endo Approach */
"0GSQ0ZZ"  =  "1"  /* Reposition Multiple Parathyroid Glands, Open Approach */
"0GSQ4ZZ"  =  "1"  /* Reposition Multiple Parathyroid Glands, Perc Endo Approach */
"0GSR0ZZ"  =  "1"  /* Reposition Parathyroid Gland, Open Approach */
"0GSR4ZZ"  =  "1"  /* Reposition Parathyroid Gland, Perc Endo Approach */
"0GT00ZZ"  =  "1"  /* Resection of Pituitary Gland, Open Approach */
"0GT04ZZ"  =  "1"  /* Resection of Pituitary Gland, Perc Endo Approach */
"0GT10ZZ"  =  "1"  /* Resection of Pineal Body, Open Approach */
"0GT14ZZ"  =  "1"  /* Resection of Pineal Body, Percutaneous Endoscopic Approach */
"0GT20ZZ"  =  "1"  /* Resection of Left Adrenal Gland, Open Approach */
"0GT24ZZ"  =  "1"  /* Resection of Left Adrenal Gland, Perc Endo Approach */
"0GT30ZZ"  =  "1"  /* Resection of Right Adrenal Gland, Open Approach */
"0GT34ZZ"  =  "1"  /* Resection of Right Adrenal Gland, Perc Endo Approach */
"0GT40ZZ"  =  "1"  /* Resection of Bilateral Adrenal Glands, Open Approach */
"0GT44ZZ"  =  "1"  /* Resection of Bilateral Adrenal Glands, Perc Endo Approach */
"0GTG0ZZ"  =  "1"  /* Resection of Left Thyroid Gland Lobe, Open Approach */
"0GTG4ZZ"  =  "1"  /* Resection of Left Thyroid Gland Lobe, Perc Endo Approach */
"0GTH0ZZ"  =  "1"  /* Resection of Right Thyroid Gland Lobe, Open Approach */
"0GTH4ZZ"  =  "1"  /* Resection of Right Thyroid Gland Lobe, Perc Endo Approach */
"0GTK0ZZ"  =  "1"  /* Resection of Thyroid Gland, Open Approach */
"0GTK4ZZ"  =  "1"  /* Resection of Thyroid Gland, Percutaneous Endoscopic Approach */
"0GTL0ZZ"  =  "1"  /* Resection of Right Superior Parathyroid Gland, Open Approach */
"0GTL4ZZ"  =  "1"  /* Resection of R Sup Parathyroid, Perc Endo Approach */
"0GTM0ZZ"  =  "1"  /* Resection of Left Superior Parathyroid Gland, Open Approach */
"0GTM4ZZ"  =  "1"  /* Resection of L Sup Parathyroid, Perc Endo Approach */
"0GTN0ZZ"  =  "1"  /* Resection of Right Inferior Parathyroid Gland, Open Approach */
"0GTN4ZZ"  =  "1"  /* Resection of R Inf Parathyroid, Perc Endo Approach */
"0GTP0ZZ"  =  "1"  /* Resection of Left Inferior Parathyroid Gland, Open Approach */
"0GTP4ZZ"  =  "1"  /* Resection of L Inf Parathyroid, Perc Endo Approach */
"0GTQ0ZZ"  =  "1"  /* Resection of Multiple Parathyroid Glands, Open Approach */
"0GTQ4ZZ"  =  "1"  /* Resection of Multiple Parathyroid Glands, Perc Endo Approach */
"0GTR0ZZ"  =  "1"  /* Resection of Parathyroid Gland, Open Approach */
"0GTR4ZZ"  =  "1"  /* Resection of Parathyroid Gland, Perc Endo Approach */
"0GW000Z"  =  "1"  /* Revision of Drain Dev in Pituitary Gland, Open Approach */
"0GW030Z"  =  "1"  /* Revision of Drain Dev in Pituitary Gland, Perc Approach */
"0GW040Z"  =  "1"  /* Revision of Drain Dev in Pituitary Gland, Perc Endo Approach */
"0GW100Z"  =  "1"  /* Revision of Drainage Device in Pineal Body, Open Approach */
"0GW130Z"  =  "1"  /* Revision of Drainage Device in Pineal Body, Perc Approach */
"0GW140Z"  =  "1"  /* Revision of Drain Dev in Pineal Body, Perc Endo Approach */
"0GW500Z"  =  "1"  /* Revision of Drainage Device in Adrenal Gland, Open Approach */
"0GW530Z"  =  "1"  /* Revision of Drainage Device in Adrenal Gland, Perc Approach */
"0GW540Z"  =  "1"  /* Revision of Drain Dev in Adrenal Gland, Perc Endo Approach */
"0GWK00Z"  =  "1"  /* Revision of Drainage Device in Thyroid Gland, Open Approach */
"0GWK30Z"  =  "1"  /* Revision of Drainage Device in Thyroid Gland, Perc Approach */
"0GWK40Z"  =  "1"  /* Revision of Drain Dev in Thyroid Gland, Perc Endo Approach */
"0GWR00Z"  =  "1"  /* Revision of Drain Dev in Parathyroid Gland, Open Approach */
"0GWR30Z"  =  "1"  /* Revision of Drain Dev in Parathyroid Gland, Perc Approach */
"0GWR40Z"  =  "1"  /* Revise of Drain Dev in Parathyroid Gland, Perc Endo Approach */
"0H0T07Z"  =  "1"  /* Alteration of Right Breast with Autol Sub, Open Approach */
"0H0T0JZ"  =  "1"  /* Alteration of Right Breast with Synth Sub, Open Approach */
"0H0T0KZ"  =  "1"  /* Alteration of Right Breast with Nonaut Sub, Open Approach */
"0H0T0ZZ"  =  "1"  /* Alteration of Right Breast, Open Approach */
"0H0T37Z"  =  "1"  /* Alteration of Right Breast with Autol Sub, Perc Approach */
"0H0T3JZ"  =  "1"  /* Alteration of Right Breast with Synth Sub, Perc Approach */
"0H0T3KZ"  =  "1"  /* Alteration of Right Breast with Nonaut Sub, Perc Approach */
"0H0T3ZZ"  =  "1"  /* Alteration of Right Breast, Percutaneous Approach */
"0H0TX7Z"  =  "1"  /* Alteration of Right Breast with Autol Sub, Extern Approach */
"0H0TXJZ"  =  "1"  /* Alteration of Right Breast with Synth Sub, Extern Approach */
"0H0TXKZ"  =  "1"  /* Alteration of Right Breast with Nonaut Sub, Extern Approach */
"0H0TXZZ"  =  "1"  /* Alteration of Right Breast, External Approach */
"0H0U07Z"  =  "1"  /* Alteration of Left Breast with Autol Sub, Open Approach */
"0H0U0JZ"  =  "1"  /* Alteration of Left Breast with Synth Sub, Open Approach */
"0H0U0KZ"  =  "1"  /* Alteration of Left Breast with Nonaut Sub, Open Approach */
"0H0U0ZZ"  =  "1"  /* Alteration of Left Breast, Open Approach */
"0H0U37Z"  =  "1"  /* Alteration of Left Breast with Autol Sub, Perc Approach */
"0H0U3JZ"  =  "1"  /* Alteration of Left Breast with Synth Sub, Perc Approach */
"0H0U3KZ"  =  "1"  /* Alteration of Left Breast with Nonaut Sub, Perc Approach */
"0H0U3ZZ"  =  "1"  /* Alteration of Left Breast, Percutaneous Approach */
"0H0UX7Z"  =  "1"  /* Alteration of Left Breast with Autol Sub, Extern Approach */
"0H0UXJZ"  =  "1"  /* Alteration of Left Breast with Synth Sub, Extern Approach */
"0H0UXKZ"  =  "1"  /* Alteration of Left Breast with Nonaut Sub, Extern Approach */
"0H0UXZZ"  =  "1"  /* Alteration of Left Breast, External Approach */
"0H0V07Z"  =  "1"  /* Alteration of Bilateral Breast with Autol Sub, Open Approach */
"0H0V0JZ"  =  "1"  /* Alteration of Bilateral Breast with Synth Sub, Open Approach */
"0H0V0KZ"  =  "1"  /* Alteration of Bi Breast with Nonaut Sub, Open Approach */
"0H0V0ZZ"  =  "1"  /* Alteration of Bilateral Breast, Open Approach */
"0H0V37Z"  =  "1"  /* Alteration of Bilateral Breast with Autol Sub, Perc Approach */
"0H0V3JZ"  =  "1"  /* Alteration of Bilateral Breast with Synth Sub, Perc Approach */
"0H0V3KZ"  =  "1"  /* Alteration of Bi Breast with Nonaut Sub, Perc Approach */
"0H0V3ZZ"  =  "1"  /* Alteration of Bilateral Breast, Percutaneous Approach */
"0H0VX7Z"  =  "1"  /* Alteration of Bi Breast with Autol Sub, Extern Approach */
"0H0VXJZ"  =  "1"  /* Alteration of Bi Breast with Synth Sub, Extern Approach */
"0H0VXKZ"  =  "1"  /* Alteration of Bi Breast with Nonaut Sub, Extern Approach */
"0H0VXZZ"  =  "1"  /* Alteration of Bilateral Breast, External Approach */
"0H5T0ZZ"  =  "1"  /* Destruction of Right Breast, Open Approach */
"0H5T3ZZ"  =  "1"  /* Destruction of Right Breast, Percutaneous Approach */
"0H5T7ZZ"  =  "1"  /* Destruction of Right Breast, Via Opening */
"0H5T8ZZ"  =  "1"  /* Destruction of Right Breast, Endo */
"0H5TXZZ"  =  "1"  /* Destruction of Right Breast, External Approach */
"0H5U0ZZ"  =  "1"  /* Destruction of Left Breast, Open Approach */
"0H5U3ZZ"  =  "1"  /* Destruction of Left Breast, Percutaneous Approach */
"0H5U7ZZ"  =  "1"  /* Destruction of Left Breast, Via Opening */
"0H5U8ZZ"  =  "1"  /* Destruction of Left Breast, Endo */
"0H5UXZZ"  =  "1"  /* Destruction of Left Breast, External Approach */
"0H5V0ZZ"  =  "1"  /* Destruction of Bilateral Breast, Open Approach */
"0H5V3ZZ"  =  "1"  /* Destruction of Bilateral Breast, Percutaneous Approach */
"0H5V7ZZ"  =  "1"  /* Destruction of Bilateral Breast, Via Opening */
"0H5V8ZZ"  =  "1"  /* Destruction of Bilateral Breast, Endo */
"0H5VXZZ"  =  "1"  /* Destruction of Bilateral Breast, External Approach */
"0H5W0ZZ"  =  "1"  /* Destruction of Right Nipple, Open Approach */
"0H5W3ZZ"  =  "1"  /* Destruction of Right Nipple, Percutaneous Approach */
"0H5W7ZZ"  =  "1"  /* Destruction of Right Nipple, Via Opening */
"0H5W8ZZ"  =  "1"  /* Destruction of Right Nipple, Endo */
"0H5WXZZ"  =  "1"  /* Destruction of Right Nipple, External Approach */
"0H5X0ZZ"  =  "1"  /* Destruction of Left Nipple, Open Approach */
"0H5X3ZZ"  =  "1"  /* Destruction of Left Nipple, Percutaneous Approach */
"0H5X7ZZ"  =  "1"  /* Destruction of Left Nipple, Via Opening */
"0H5X8ZZ"  =  "1"  /* Destruction of Left Nipple, Endo */
"0H5XXZZ"  =  "1"  /* Destruction of Left Nipple, External Approach */
"0H80XZZ"  =  "1"  /* Division of Scalp Skin, External Approach */
"0H81XZZ"  =  "1"  /* Division of Face Skin, External Approach */
"0H84XZZ"  =  "1"  /* Division of Neck Skin, External Approach */
"0H85XZZ"  =  "1"  /* Division of Chest Skin, External Approach */
"0H86XZZ"  =  "1"  /* Division of Back Skin, External Approach */
"0H87XZZ"  =  "1"  /* Division of Abdomen Skin, External Approach */
"0H88XZZ"  =  "1"  /* Division of Buttock Skin, External Approach */
"0H89XZZ"  =  "1"  /* Division of Perineum Skin, External Approach */
"0H8AXZZ"  =  "1"  /* Division of Genitalia Skin, External Approach */
"0H8BXZZ"  =  "1"  /* Division of Right Upper Arm Skin, External Approach */
"0H8CXZZ"  =  "1"  /* Division of Left Upper Arm Skin, External Approach */
"0H8DXZZ"  =  "1"  /* Division of Right Lower Arm Skin, External Approach */
"0H8EXZZ"  =  "1"  /* Division of Left Lower Arm Skin, External Approach */
"0H8FXZZ"  =  "1"  /* Division of Right Hand Skin, External Approach */
"0H8GXZZ"  =  "1"  /* Division of Left Hand Skin, External Approach */
"0H8HXZZ"  =  "1"  /* Division of Right Upper Leg Skin, External Approach */
"0H8JXZZ"  =  "1"  /* Division of Left Upper Leg Skin, External Approach */
"0H8KXZZ"  =  "1"  /* Division of Right Lower Leg Skin, External Approach */
"0H8LXZZ"  =  "1"  /* Division of Left Lower Leg Skin, External Approach */
"0H8MXZZ"  =  "1"  /* Division of Right Foot Skin, External Approach */
"0H8NXZZ"  =  "1"  /* Division of Left Foot Skin, External Approach */
"0H99X0Z"  =  "1"  /* Drainage of Perineum Skin with Drain Dev, Extern Approach */
"0H99XZZ"  =  "1"  /* Drainage of Perineum Skin, External Approach */
"0H9T0ZX"  =  "1"  /* Drainage of Right Breast, Open Approach, Diagnostic */
"0H9U0ZX"  =  "1"  /* Drainage of Left Breast, Open Approach, Diagnostic */
"0H9V0ZX"  =  "1"  /* Drainage of Bilateral Breast, Open Approach, Diagnostic */
"0H9W0ZX"  =  "1"  /* Drainage of Right Nipple, Open Approach, Diagnostic */
"0H9X0ZX"  =  "1"  /* Drainage of Left Nipple, Open Approach, Diagnostic */
"0HB0XZZ"  =  "1"  /* Excision of Scalp Skin, External Approach */
"0HB1XZZ"  =  "1"  /* Excision of Face Skin, External Approach */
"0HB4XZZ"  =  "1"  /* Excision of Neck Skin, External Approach */
"0HB5XZZ"  =  "1"  /* Excision of Chest Skin, External Approach */
"0HB6XZZ"  =  "1"  /* Excision of Back Skin, External Approach */
"0HB7XZZ"  =  "1"  /* Excision of Abdomen Skin, External Approach */
"0HB8XZZ"  =  "1"  /* Excision of Buttock Skin, External Approach */
"0HBAXZZ"  =  "1"  /* Excision of Genitalia Skin, External Approach */
"0HBBXZZ"  =  "1"  /* Excision of Right Upper Arm Skin, External Approach */
"0HBCXZZ"  =  "1"  /* Excision of Left Upper Arm Skin, External Approach */
"0HBDXZZ"  =  "1"  /* Excision of Right Lower Arm Skin, External Approach */
"0HBEXZZ"  =  "1"  /* Excision of Left Lower Arm Skin, External Approach */
"0HBFXZZ"  =  "1"  /* Excision of Right Hand Skin, External Approach */
"0HBGXZZ"  =  "1"  /* Excision of Left Hand Skin, External Approach */
"0HBHXZZ"  =  "1"  /* Excision of Right Upper Leg Skin, External Approach */
"0HBJXZZ"  =  "1"  /* Excision of Left Upper Leg Skin, External Approach */
"0HBKXZZ"  =  "1"  /* Excision of Right Lower Leg Skin, External Approach */
"0HBLXZZ"  =  "1"  /* Excision of Left Lower Leg Skin, External Approach */
"0HBMXZZ"  =  "1"  /* Excision of Right Foot Skin, External Approach */
"0HBNXZZ"  =  "1"  /* Excision of Left Foot Skin, External Approach */
"0HBT0ZX"  =  "1"  /* Excision of Right Breast, Open Approach, Diagnostic */
"0HBT0ZZ"  =  "1"  /* Excision of Right Breast, Open Approach */
"0HBT3ZZ"  =  "1"  /* Excision of Right Breast, Percutaneous Approach */
"0HBT7ZZ"  =  "1"  /* Excision of Right Breast, Via Natural or Artificial Opening */
"0HBT8ZZ"  =  "1"  /* Excision of Right Breast, Endo */
"0HBTXZZ"  =  "1"  /* Excision of Right Breast, External Approach */
"0HBU0ZX"  =  "1"  /* Excision of Left Breast, Open Approach, Diagnostic */
"0HBU0ZZ"  =  "1"  /* Excision of Left Breast, Open Approach */
"0HBU3ZZ"  =  "1"  /* Excision of Left Breast, Percutaneous Approach */
"0HBU7ZZ"  =  "1"  /* Excision of Left Breast, Via Natural or Artificial Opening */
"0HBU8ZZ"  =  "1"  /* Excision of Left Breast, Endo */
"0HBUXZZ"  =  "1"  /* Excision of Left Breast, External Approach */
"0HBV0ZX"  =  "1"  /* Excision of Bilateral Breast, Open Approach, Diagnostic */
"0HBV0ZZ"  =  "1"  /* Excision of Bilateral Breast, Open Approach */
"0HBV3ZZ"  =  "1"  /* Excision of Bilateral Breast, Percutaneous Approach */
"0HBV7ZZ"  =  "1"  /* Excision of Bilateral Breast, Via Opening */
"0HBV8ZZ"  =  "1"  /* Excision of Bilateral Breast, Endo */
"0HBVXZZ"  =  "1"  /* Excision of Bilateral Breast, External Approach */
"0HBW0ZX"  =  "1"  /* Excision of Right Nipple, Open Approach, Diagnostic */
"0HBW0ZZ"  =  "1"  /* Excision of Right Nipple, Open Approach */
"0HBW3ZZ"  =  "1"  /* Excision of Right Nipple, Percutaneous Approach */
"0HBW7ZZ"  =  "1"  /* Excision of Right Nipple, Via Natural or Artificial Opening */
"0HBW8ZZ"  =  "1"  /* Excision of Right Nipple, Endo */
"0HBWXZZ"  =  "1"  /* Excision of Right Nipple, External Approach */
"0HBX0ZX"  =  "1"  /* Excision of Left Nipple, Open Approach, Diagnostic */
"0HBX0ZZ"  =  "1"  /* Excision of Left Nipple, Open Approach */
"0HBX3ZZ"  =  "1"  /* Excision of Left Nipple, Percutaneous Approach */
"0HBX7ZZ"  =  "1"  /* Excision of Left Nipple, Via Natural or Artificial Opening */
"0HBX8ZZ"  =  "1"  /* Excision of Left Nipple, Endo */
"0HBXXZZ"  =  "1"  /* Excision of Left Nipple, External Approach */
"0HBY0ZX"  =  "1"  /* Excision of Supernumerary Breast, Open Approach, Diagnostic */
"0HBY0ZZ"  =  "1"  /* Excision of Supernumerary Breast, Open Approach */
"0HBY3ZZ"  =  "1"  /* Excision of Supernumerary Breast, Percutaneous Approach */
"0HBY7ZZ"  =  "1"  /* Excision of Supernumerary Breast, Via Opening */
"0HBY8ZZ"  =  "1"  /* Excision of Supernumerary Breast, Endo */
"0HBYXZZ"  =  "1"  /* Excision of Supernumerary Breast, External Approach */
"0HHT01Z"  =  "1"  /* Insertion of Radioact Elem into R Breast, Open Approach */
"0HHT0NZ"  =  "1"  /* Insertion of Tissue Expander into R Breast, Open Approach */
"0HHT31Z"  =  "1"  /* Insertion of Radioact Elem into R Breast, Perc Approach */
"0HHT3NZ"  =  "1"  /* Insertion of Tissue Expander into R Breast, Perc Approach */
"0HHT71Z"  =  "1"  /* Insertion of Radioactive Element into R Breast, Via Opening */
"0HHT7NZ"  =  "1"  /* Insertion of Tissue Expander into Right Breast, Via Opening */
"0HHT81Z"  =  "1"  /* Insertion of Radioactive Element into Right Breast, Endo */
"0HHT8NZ"  =  "1"  /* Insertion of Tissue Expander into Right Breast, Endo */
"0HHTX1Z"  =  "1"  /* Insertion of Radioact Elem into R Breast, Extern Approach */
"0HHU01Z"  =  "1"  /* Insertion of Radioact Elem into L Breast, Open Approach */
"0HHU0NZ"  =  "1"  /* Insertion of Tissue Expander into Left Breast, Open Approach */
"0HHU31Z"  =  "1"  /* Insertion of Radioact Elem into L Breast, Perc Approach */
"0HHU3NZ"  =  "1"  /* Insertion of Tissue Expander into Left Breast, Perc Approach */
"0HHU71Z"  =  "1"  /* Insertion of Radioactive Element into L Breast, Via Opening */
"0HHU7NZ"  =  "1"  /* Insertion of Tissue Expander into Left Breast, Via Opening */
"0HHU81Z"  =  "1"  /* Insertion of Radioactive Element into Left Breast, Endo */
"0HHU8NZ"  =  "1"  /* Insertion of Tissue Expander into Left Breast, Endo */
"0HHUX1Z"  =  "1"  /* Insertion of Radioact Elem into L Breast, Extern Approach */
"0HHV01Z"  =  "1"  /* Insertion of Radioact Elem into Bi Breast, Open Approach */
"0HHV0NZ"  =  "1"  /* Insertion of Tissue Expander into Bi Breast, Open Approach */
"0HHV31Z"  =  "1"  /* Insertion of Radioact Elem into Bi Breast, Perc Approach */
"0HHV3NZ"  =  "1"  /* Insertion of Tissue Expander into Bi Breast, Perc Approach */
"0HHV71Z"  =  "1"  /* Insertion of Radioactive Element into Bi Breast, Via Opening */
"0HHV7NZ"  =  "1"  /* Insertion of Tissue Expander into Bi Breast, Via Opening */
"0HHV81Z"  =  "1"  /* Insertion of Radioactive Element into Bilateral Breast, Endo */
"0HHV8NZ"  =  "1"  /* Insertion of Tissue Expander into Bilateral Breast, Endo */
"0HHVX1Z"  =  "1"  /* Insertion of Radioact Elem into Bi Breast, Extern Approach */
"0HHW01Z"  =  "1"  /* Insertion of Radioact Elem into R Nipple, Open Approach */
"0HHW0NZ"  =  "1"  /* Insertion of Tissue Expander into R Nipple, Open Approach */
"0HHW31Z"  =  "1"  /* Insertion of Radioact Elem into R Nipple, Perc Approach */
"0HHW3NZ"  =  "1"  /* Insertion of Tissue Expander into R Nipple, Perc Approach */
"0HHW71Z"  =  "1"  /* Insertion of Radioactive Element into R Nipple, Via Opening */
"0HHW7NZ"  =  "1"  /* Insertion of Tissue Expander into Right Nipple, Via Opening */
"0HHW81Z"  =  "1"  /* Insertion of Radioactive Element into Right Nipple, Endo */
"0HHW8NZ"  =  "1"  /* Insertion of Tissue Expander into Right Nipple, Endo */
"0HHWX1Z"  =  "1"  /* Insertion of Radioact Elem into R Nipple, Extern Approach */
"0HHX01Z"  =  "1"  /* Insertion of Radioact Elem into L Nipple, Open Approach */
"0HHX0NZ"  =  "1"  /* Insertion of Tissue Expander into Left Nipple, Open Approach */
"0HHX31Z"  =  "1"  /* Insertion of Radioact Elem into L Nipple, Perc Approach */
"0HHX3NZ"  =  "1"  /* Insertion of Tissue Expander into Left Nipple, Perc Approach */
"0HHX71Z"  =  "1"  /* Insertion of Radioactive Element into L Nipple, Via Opening */
"0HHX7NZ"  =  "1"  /* Insertion of Tissue Expander into Left Nipple, Via Opening */
"0HHX81Z"  =  "1"  /* Insertion of Radioactive Element into Left Nipple, Endo */
"0HHX8NZ"  =  "1"  /* Insertion of Tissue Expander into Left Nipple, Endo */
"0HHXX1Z"  =  "1"  /* Insertion of Radioact Elem into L Nipple, Extern Approach */
"0HM1XZZ"  =  "1"  /* Reattachment of Face Skin, External Approach */
"0HM2XZZ"  =  "1"  /* Reattachment of Right Ear Skin, External Approach */
"0HM3XZZ"  =  "1"  /* Reattachment of Left Ear Skin, External Approach */
"0HM4XZZ"  =  "1"  /* Reattachment of Neck Skin, External Approach */
"0HM5XZZ"  =  "1"  /* Reattachment of Chest Skin, External Approach */
"0HM6XZZ"  =  "1"  /* Reattachment of Back Skin, External Approach */
"0HM7XZZ"  =  "1"  /* Reattachment of Abdomen Skin, External Approach */
"0HM8XZZ"  =  "1"  /* Reattachment of Buttock Skin, External Approach */
"0HM9XZZ"  =  "1"  /* Reattachment of Perineum Skin, External Approach */
"0HMAXZZ"  =  "1"  /* Reattachment of Genitalia Skin, External Approach */
"0HMBXZZ"  =  "1"  /* Reattachment of Right Upper Arm Skin, External Approach */
"0HMCXZZ"  =  "1"  /* Reattachment of Left Upper Arm Skin, External Approach */
"0HMDXZZ"  =  "1"  /* Reattachment of Right Lower Arm Skin, External Approach */
"0HMEXZZ"  =  "1"  /* Reattachment of Left Lower Arm Skin, External Approach */
"0HMFXZZ"  =  "1"  /* Reattachment of Right Hand Skin, External Approach */
"0HMGXZZ"  =  "1"  /* Reattachment of Left Hand Skin, External Approach */
"0HMHXZZ"  =  "1"  /* Reattachment of Right Upper Leg Skin, External Approach */
"0HMJXZZ"  =  "1"  /* Reattachment of Left Upper Leg Skin, External Approach */
"0HMKXZZ"  =  "1"  /* Reattachment of Right Lower Leg Skin, External Approach */
"0HMLXZZ"  =  "1"  /* Reattachment of Left Lower Leg Skin, External Approach */
"0HMMXZZ"  =  "1"  /* Reattachment of Right Foot Skin, External Approach */
"0HMNXZZ"  =  "1"  /* Reattachment of Left Foot Skin, External Approach */
"0HMTXZZ"  =  "1"  /* Reattachment of Right Breast, External Approach */
"0HMUXZZ"  =  "1"  /* Reattachment of Left Breast, External Approach */
"0HMVXZZ"  =  "1"  /* Reattachment of Bilateral Breast, External Approach */
"0HMWXZZ"  =  "1"  /* Reattachment of Right Nipple, External Approach */
"0HMXXZZ"  =  "1"  /* Reattachment of Left Nipple, External Approach */
"0HN0XZZ"  =  "1"  /* Release Scalp Skin, External Approach */
"0HN1XZZ"  =  "1"  /* Release Face Skin, External Approach */
"0HN2XZZ"  =  "1"  /* Release Right Ear Skin, External Approach */
"0HN3XZZ"  =  "1"  /* Release Left Ear Skin, External Approach */
"0HN4XZZ"  =  "1"  /* Release Neck Skin, External Approach */
"0HN5XZZ"  =  "1"  /* Release Chest Skin, External Approach */
"0HN6XZZ"  =  "1"  /* Release Back Skin, External Approach */
"0HN7XZZ"  =  "1"  /* Release Abdomen Skin, External Approach */
"0HN8XZZ"  =  "1"  /* Release Buttock Skin, External Approach */
"0HN9XZZ"  =  "1"  /* Release Perineum Skin, External Approach */
"0HNAXZZ"  =  "1"  /* Release Genitalia Skin, External Approach */
"0HNBXZZ"  =  "1"  /* Release Right Upper Arm Skin, External Approach */
"0HNCXZZ"  =  "1"  /* Release Left Upper Arm Skin, External Approach */
"0HNDXZZ"  =  "1"  /* Release Right Lower Arm Skin, External Approach */
"0HNEXZZ"  =  "1"  /* Release Left Lower Arm Skin, External Approach */
"0HNFXZZ"  =  "1"  /* Release Right Hand Skin, External Approach */
"0HNGXZZ"  =  "1"  /* Release Left Hand Skin, External Approach */
"0HNHXZZ"  =  "1"  /* Release Right Upper Leg Skin, External Approach */
"0HNJXZZ"  =  "1"  /* Release Left Upper Leg Skin, External Approach */
"0HNKXZZ"  =  "1"  /* Release Right Lower Leg Skin, External Approach */
"0HNLXZZ"  =  "1"  /* Release Left Lower Leg Skin, External Approach */
"0HNMXZZ"  =  "1"  /* Release Right Foot Skin, External Approach */
"0HNNXZZ"  =  "1"  /* Release Left Foot Skin, External Approach */
"0HNQXZZ"  =  "1"  /* Release Finger Nail, External Approach */
"0HNRXZZ"  =  "1"  /* Release Toe Nail, External Approach */
"0HNT0ZZ"  =  "1"  /* Release Right Breast, Open Approach */
"0HNT3ZZ"  =  "1"  /* Release Right Breast, Percutaneous Approach */
"0HNT7ZZ"  =  "1"  /* Release Right Breast, Via Natural or Artificial Opening */
"0HNT8ZZ"  =  "1"  /* Release Right Breast, Endo */
"0HNTXZZ"  =  "1"  /* Release Right Breast, External Approach */
"0HNU0ZZ"  =  "1"  /* Release Left Breast, Open Approach */
"0HNU3ZZ"  =  "1"  /* Release Left Breast, Percutaneous Approach */
"0HNU7ZZ"  =  "1"  /* Release Left Breast, Via Natural or Artificial Opening */
"0HNU8ZZ"  =  "1"  /* Release Left Breast, Endo */
"0HNUXZZ"  =  "1"  /* Release Left Breast, External Approach */
"0HNV0ZZ"  =  "1"  /* Release Bilateral Breast, Open Approach */
"0HNV3ZZ"  =  "1"  /* Release Bilateral Breast, Percutaneous Approach */
"0HNV7ZZ"  =  "1"  /* Release Bilateral Breast, Via Natural or Artificial Opening */
"0HNV8ZZ"  =  "1"  /* Release Bilateral Breast, Endo */
"0HNVXZZ"  =  "1"  /* Release Bilateral Breast, External Approach */
"0HNW0ZZ"  =  "1"  /* Release Right Nipple, Open Approach */
"0HNW3ZZ"  =  "1"  /* Release Right Nipple, Percutaneous Approach */
"0HNW7ZZ"  =  "1"  /* Release Right Nipple, Via Natural or Artificial Opening */
"0HNW8ZZ"  =  "1"  /* Release Right Nipple, Endo */
"0HNWXZZ"  =  "1"  /* Release Right Nipple, External Approach */
"0HNX0ZZ"  =  "1"  /* Release Left Nipple, Open Approach */
"0HNX3ZZ"  =  "1"  /* Release Left Nipple, Percutaneous Approach */
"0HNX7ZZ"  =  "1"  /* Release Left Nipple, Via Natural or Artificial Opening */
"0HNX8ZZ"  =  "1"  /* Release Left Nipple, Endo */
"0HNXXZZ"  =  "1"  /* Release Left Nipple, External Approach */
"0HPT0JZ"  =  "1"  /* Removal of Synthetic Substitute from R Breast, Open Approach */
"0HPT0NZ"  =  "1"  /* Removal of Tissue Expander from Right Breast, Open Approach */
"0HPT3JZ"  =  "1"  /* Removal of Synthetic Substitute from R Breast, Perc Approach */
"0HPT3NZ"  =  "1"  /* Removal of Tissue Expander from Right Breast, Perc Approach */
"0HPU0JZ"  =  "1"  /* Removal of Synthetic Substitute from L Breast, Open Approach */
"0HPU0NZ"  =  "1"  /* Removal of Tissue Expander from Left Breast, Open Approach */
"0HPU3JZ"  =  "1"  /* Removal of Synthetic Substitute from L Breast, Perc Approach */
"0HPU3NZ"  =  "1"  /* Removal of Tissue Expander from Left Breast, Perc Approach */
"0HQQXZZ"  =  "1"  /* Repair Finger Nail, External Approach */
"0HQRXZZ"  =  "1"  /* Repair Toe Nail, External Approach */
"0HQT0ZZ"  =  "1"  /* Repair Right Breast, Open Approach */
"0HQT3ZZ"  =  "1"  /* Repair Right Breast, Percutaneous Approach */
"0HQT7ZZ"  =  "1"  /* Repair Right Breast, Via Natural or Artificial Opening */
"0HQT8ZZ"  =  "1"  /* Repair Right Breast, Endo */
"0HQU0ZZ"  =  "1"  /* Repair Left Breast, Open Approach */
"0HQU3ZZ"  =  "1"  /* Repair Left Breast, Percutaneous Approach */
"0HQU7ZZ"  =  "1"  /* Repair Left Breast, Via Natural or Artificial Opening */
"0HQU8ZZ"  =  "1"  /* Repair Left Breast, Endo */
"0HQV0ZZ"  =  "1"  /* Repair Bilateral Breast, Open Approach */
"0HQV3ZZ"  =  "1"  /* Repair Bilateral Breast, Percutaneous Approach */
"0HQV7ZZ"  =  "1"  /* Repair Bilateral Breast, Via Natural or Artificial Opening */
"0HQV8ZZ"  =  "1"  /* Repair Bilateral Breast, Endo */
"0HQW0ZZ"  =  "1"  /* Repair Right Nipple, Open Approach */
"0HQW3ZZ"  =  "1"  /* Repair Right Nipple, Percutaneous Approach */
"0HQW7ZZ"  =  "1"  /* Repair Right Nipple, Via Natural or Artificial Opening */
"0HQW8ZZ"  =  "1"  /* Repair Right Nipple, Endo */
"0HQWXZZ"  =  "1"  /* Repair Right Nipple, External Approach */
"0HQX0ZZ"  =  "1"  /* Repair Left Nipple, Open Approach */
"0HQX3ZZ"  =  "1"  /* Repair Left Nipple, Percutaneous Approach */
"0HQX7ZZ"  =  "1"  /* Repair Left Nipple, Via Natural or Artificial Opening */
"0HQX8ZZ"  =  "1"  /* Repair Left Nipple, Endo */
"0HQXXZZ"  =  "1"  /* Repair Left Nipple, External Approach */
"0HQY0ZZ"  =  "1"  /* Repair Supernumerary Breast, Open Approach */
"0HQY3ZZ"  =  "1"  /* Repair Supernumerary Breast, Percutaneous Approach */
"0HQY7ZZ"  =  "1"  /* Repair Supernumerary Breast, Via Opening */
"0HQY8ZZ"  =  "1"  /* Repair Supernumerary Breast, Endo */
"0HR0X73"  =  "1"  /* Replace Scalp Skin w Autol Sub, Full Thick, Extern */
"0HR0X74"  =  "1"  /* Replace Scalp Skin w Autol Sub, Part Thick, Extern */
"0HR0XJ3"  =  "1"  /* Replace Scalp Skin w Synth Sub, Full Thick, Extern */
"0HR0XJ4"  =  "1"  /* Replace Scalp Skin w Synth Sub, Part Thick, Extern */
"0HR0XJZ"  =  "1"  /* Replacement of Scalp Skin with Synth Sub, Extern Approach */
"0HR0XK3"  =  "1"  /* Replace Scalp Skin w Nonaut Sub, Full Thick, Extern */
"0HR0XK4"  =  "1"  /* Replace Scalp Skin w Nonaut Sub, Part Thick, Extern */
"0HR1X73"  =  "1"  /* Replace Face Skin w Autol Sub, Full Thick, Extern */
"0HR1X74"  =  "1"  /* Replace Face Skin w Autol Sub, Part Thick, Extern */
"0HR1XJ3"  =  "1"  /* Replace Face Skin w Synth Sub, Full Thick, Extern */
"0HR1XJ4"  =  "1"  /* Replace Face Skin w Synth Sub, Part Thick, Extern */
"0HR1XJZ"  =  "1"  /* Replacement of Face Skin with Synth Sub, Extern Approach */
"0HR1XK3"  =  "1"  /* Replace Face Skin w Nonaut Sub, Full Thick, Extern */
"0HR1XK4"  =  "1"  /* Replace Face Skin w Nonaut Sub, Part Thick, Extern */
"0HR2X73"  =  "1"  /* Replace R Ear Skin w Autol Sub, Full Thick, Extern */
"0HR2X74"  =  "1"  /* Replace R Ear Skin w Autol Sub, Part Thick, Extern */
"0HR2XJ3"  =  "1"  /* Replace R Ear Skin w Synth Sub, Full Thick, Extern */
"0HR2XJ4"  =  "1"  /* Replace R Ear Skin w Synth Sub, Part Thick, Extern */
"0HR2XJZ"  =  "1"  /* Replacement of R Ear Skin with Synth Sub, Extern Approach */
"0HR2XK3"  =  "1"  /* Replace R Ear Skin w Nonaut Sub, Full Thick, Extern */
"0HR2XK4"  =  "1"  /* Replace R Ear Skin w Nonaut Sub, Part Thick, Extern */
"0HR3X73"  =  "1"  /* Replace L Ear Skin w Autol Sub, Full Thick, Extern */
"0HR3X74"  =  "1"  /* Replace L Ear Skin w Autol Sub, Part Thick, Extern */
"0HR3XJ3"  =  "1"  /* Replace L Ear Skin w Synth Sub, Full Thick, Extern */
"0HR3XJ4"  =  "1"  /* Replace L Ear Skin w Synth Sub, Part Thick, Extern */
"0HR3XJZ"  =  "1"  /* Replacement of Left Ear Skin with Synth Sub, Extern Approach */
"0HR3XK3"  =  "1"  /* Replace L Ear Skin w Nonaut Sub, Full Thick, Extern */
"0HR3XK4"  =  "1"  /* Replace L Ear Skin w Nonaut Sub, Part Thick, Extern */
"0HR4X73"  =  "1"  /* Replace Neck Skin w Autol Sub, Full Thick, Extern */
"0HR4X74"  =  "1"  /* Replace Neck Skin w Autol Sub, Part Thick, Extern */
"0HR4XJ3"  =  "1"  /* Replace Neck Skin w Synth Sub, Full Thick, Extern */
"0HR4XJ4"  =  "1"  /* Replace Neck Skin w Synth Sub, Part Thick, Extern */
"0HR4XJZ"  =  "1"  /* Replacement of Neck Skin with Synth Sub, Extern Approach */
"0HR4XK3"  =  "1"  /* Replace Neck Skin w Nonaut Sub, Full Thick, Extern */
"0HR4XK4"  =  "1"  /* Replace Neck Skin w Nonaut Sub, Part Thick, Extern */
"0HR5X73"  =  "1"  /* Replace Chest Skin w Autol Sub, Full Thick, Extern */
"0HR5X74"  =  "1"  /* Replace Chest Skin w Autol Sub, Part Thick, Extern */
"0HR5XJ3"  =  "1"  /* Replace Chest Skin w Synth Sub, Full Thick, Extern */
"0HR5XJ4"  =  "1"  /* Replace Chest Skin w Synth Sub, Part Thick, Extern */
"0HR5XJZ"  =  "1"  /* Replacement of Chest Skin with Synth Sub, Extern Approach */
"0HR5XK3"  =  "1"  /* Replace Chest Skin w Nonaut Sub, Full Thick, Extern */
"0HR5XK4"  =  "1"  /* Replace Chest Skin w Nonaut Sub, Part Thick, Extern */
"0HR6X73"  =  "1"  /* Replace Back Skin w Autol Sub, Full Thick, Extern */
"0HR6X74"  =  "1"  /* Replace Back Skin w Autol Sub, Part Thick, Extern */
"0HR6XJ3"  =  "1"  /* Replace Back Skin w Synth Sub, Full Thick, Extern */
"0HR6XJ4"  =  "1"  /* Replace Back Skin w Synth Sub, Part Thick, Extern */
"0HR6XJZ"  =  "1"  /* Replacement of Back Skin with Synth Sub, Extern Approach */
"0HR6XK3"  =  "1"  /* Replace Back Skin w Nonaut Sub, Full Thick, Extern */
"0HR6XK4"  =  "1"  /* Replace Back Skin w Nonaut Sub, Part Thick, Extern */
"0HR7X73"  =  "1"  /* Replace Abd Skin w Autol Sub, Full Thick, Extern */
"0HR7X74"  =  "1"  /* Replace Abd Skin w Autol Sub, Part Thick, Extern */
"0HR7XJ3"  =  "1"  /* Replace Abd Skin w Synth Sub, Full Thick, Extern */
"0HR7XJ4"  =  "1"  /* Replace Abd Skin w Synth Sub, Part Thick, Extern */
"0HR7XJZ"  =  "1"  /* Replacement of Abdomen Skin with Synth Sub, Extern Approach */
"0HR7XK3"  =  "1"  /* Replace Abd Skin w Nonaut Sub, Full Thick, Extern */
"0HR7XK4"  =  "1"  /* Replace Abd Skin w Nonaut Sub, Part Thick, Extern */
"0HR8X73"  =  "1"  /* Replace Buttock Skin w Autol Sub, Full Thick, Extern */
"0HR8X74"  =  "1"  /* Replace Buttock Skin w Autol Sub, Part Thick, Extern */
"0HR8XJ3"  =  "1"  /* Replace Buttock Skin w Synth Sub, Full Thick, Extern */
"0HR8XJ4"  =  "1"  /* Replace Buttock Skin w Synth Sub, Part Thick, Extern */
"0HR8XJZ"  =  "1"  /* Replacement of Buttock Skin with Synth Sub, Extern Approach */
"0HR8XK3"  =  "1"  /* Replace Buttock Skin w Nonaut Sub, Full Thick, Extern */
"0HR8XK4"  =  "1"  /* Replace Buttock Skin w Nonaut Sub, Part Thick, Extern */
"0HR9X73"  =  "1"  /* Replace Perineum Skin w Autol Sub, Full Thick, Extern */
"0HR9X74"  =  "1"  /* Replace Perineum Skin w Autol Sub, Part Thick, Extern */
"0HR9XJ3"  =  "1"  /* Replace Perineum Skin w Synth Sub, Full Thick, Extern */
"0HR9XJ4"  =  "1"  /* Replace Perineum Skin w Synth Sub, Part Thick, Extern */
"0HR9XJZ"  =  "1"  /* Replacement of Perineum Skin with Synth Sub, Extern Approach */
"0HR9XK3"  =  "1"  /* Replace Perineum Skin w Nonaut Sub, Full Thick, Extern */
"0HR9XK4"  =  "1"  /* Replace Perineum Skin w Nonaut Sub, Part Thick, Extern */
"0HRAX73"  =  "1"  /* Replace Genitalia Skin w Autol Sub, Full Thick, Extern */
"0HRAX74"  =  "1"  /* Replace Genitalia Skin w Autol Sub, Part Thick, Extern */
"0HRAXJ3"  =  "1"  /* Replace Genitalia Skin w Synth Sub, Full Thick, Extern */
"0HRAXJ4"  =  "1"  /* Replace Genitalia Skin w Synth Sub, Part Thick, Extern */
"0HRAXJZ"  =  "1"  /* Replace of Genitalia Skin with Synth Sub, Extern Approach */
"0HRAXK3"  =  "1"  /* Replace Genitalia Skin w Nonaut Sub, Full Thick, Extern */
"0HRAXK4"  =  "1"  /* Replace Genitalia Skin w Nonaut Sub, Part Thick, Extern */
"0HRBX73"  =  "1"  /* Replace R Up Arm Skin w Autol Sub, Full Thick, Extern */
"0HRBX74"  =  "1"  /* Replace R Up Arm Skin w Autol Sub, Part Thick, Extern */
"0HRBXJ3"  =  "1"  /* Replace R Up Arm Skin w Synth Sub, Full Thick, Extern */
"0HRBXJ4"  =  "1"  /* Replace R Up Arm Skin w Synth Sub, Part Thick, Extern */
"0HRBXJZ"  =  "1"  /* Replacement of R Up Arm Skin with Synth Sub, Extern Approach */
"0HRBXK3"  =  "1"  /* Replace R Up Arm Skin w Nonaut Sub, Full Thick, Extern */
"0HRBXK4"  =  "1"  /* Replace R Up Arm Skin w Nonaut Sub, Part Thick, Extern */
"0HRCX73"  =  "1"  /* Replace L Up Arm Skin w Autol Sub, Full Thick, Extern */
"0HRCX74"  =  "1"  /* Replace L Up Arm Skin w Autol Sub, Part Thick, Extern */
"0HRCXJ3"  =  "1"  /* Replace L Up Arm Skin w Synth Sub, Full Thick, Extern */
"0HRCXJ4"  =  "1"  /* Replace L Up Arm Skin w Synth Sub, Part Thick, Extern */
"0HRCXJZ"  =  "1"  /* Replacement of L Up Arm Skin with Synth Sub, Extern Approach */
"0HRCXK3"  =  "1"  /* Replace L Up Arm Skin w Nonaut Sub, Full Thick, Extern */
"0HRCXK4"  =  "1"  /* Replace L Up Arm Skin w Nonaut Sub, Part Thick, Extern */
"0HRDX73"  =  "1"  /* Replace R Low Arm Skin w Autol Sub, Full Thick, Extern */
"0HRDX74"  =  "1"  /* Replace R Low Arm Skin w Autol Sub, Part Thick, Extern */
"0HRDXJ3"  =  "1"  /* Replace R Low Arm Skin w Synth Sub, Full Thick, Extern */
"0HRDXJ4"  =  "1"  /* Replace R Low Arm Skin w Synth Sub, Part Thick, Extern */
"0HRDXJZ"  =  "1"  /* Replace of R Low Arm Skin with Synth Sub, Extern Approach */
"0HRDXK3"  =  "1"  /* Replace R Low Arm Skin w Nonaut Sub, Full Thick, Extern */
"0HRDXK4"  =  "1"  /* Replace R Low Arm Skin w Nonaut Sub, Part Thick, Extern */
"0HREX73"  =  "1"  /* Replace L Low Arm Skin w Autol Sub, Full Thick, Extern */
"0HREX74"  =  "1"  /* Replace L Low Arm Skin w Autol Sub, Part Thick, Extern */
"0HREXJ3"  =  "1"  /* Replace L Low Arm Skin w Synth Sub, Full Thick, Extern */
"0HREXJ4"  =  "1"  /* Replace L Low Arm Skin w Synth Sub, Part Thick, Extern */
"0HREXJZ"  =  "1"  /* Replace of L Low Arm Skin with Synth Sub, Extern Approach */
"0HREXK3"  =  "1"  /* Replace L Low Arm Skin w Nonaut Sub, Full Thick, Extern */
"0HREXK4"  =  "1"  /* Replace L Low Arm Skin w Nonaut Sub, Part Thick, Extern */
"0HRFX73"  =  "1"  /* Replace R Hand Skin w Autol Sub, Full Thick, Extern */
"0HRFX74"  =  "1"  /* Replace R Hand Skin w Autol Sub, Part Thick, Extern */
"0HRFXJ3"  =  "1"  /* Replace R Hand Skin w Synth Sub, Full Thick, Extern */
"0HRFXJ4"  =  "1"  /* Replace R Hand Skin w Synth Sub, Part Thick, Extern */
"0HRFXJZ"  =  "1"  /* Replacement of R Hand Skin with Synth Sub, Extern Approach */
"0HRFXK3"  =  "1"  /* Replace R Hand Skin w Nonaut Sub, Full Thick, Extern */
"0HRFXK4"  =  "1"  /* Replace R Hand Skin w Nonaut Sub, Part Thick, Extern */
"0HRGX73"  =  "1"  /* Replace L Hand Skin w Autol Sub, Full Thick, Extern */
"0HRGX74"  =  "1"  /* Replace L Hand Skin w Autol Sub, Part Thick, Extern */
"0HRGXJ3"  =  "1"  /* Replace L Hand Skin w Synth Sub, Full Thick, Extern */
"0HRGXJ4"  =  "1"  /* Replace L Hand Skin w Synth Sub, Part Thick, Extern */
"0HRGXJZ"  =  "1"  /* Replacement of L Hand Skin with Synth Sub, Extern Approach */
"0HRGXK3"  =  "1"  /* Replace L Hand Skin w Nonaut Sub, Full Thick, Extern */
"0HRGXK4"  =  "1"  /* Replace L Hand Skin w Nonaut Sub, Part Thick, Extern */
"0HRHX73"  =  "1"  /* Replace R Up Leg Skin w Autol Sub, Full Thick, Extern */
"0HRHX74"  =  "1"  /* Replace R Up Leg Skin w Autol Sub, Part Thick, Extern */
"0HRHXJ3"  =  "1"  /* Replace R Up Leg Skin w Synth Sub, Full Thick, Extern */
"0HRHXJ4"  =  "1"  /* Replace R Up Leg Skin w Synth Sub, Part Thick, Extern */
"0HRHXJZ"  =  "1"  /* Replacement of R Up Leg Skin with Synth Sub, Extern Approach */
"0HRHXK3"  =  "1"  /* Replace R Up Leg Skin w Nonaut Sub, Full Thick, Extern */
"0HRHXK4"  =  "1"  /* Replace R Up Leg Skin w Nonaut Sub, Part Thick, Extern */
"0HRJX73"  =  "1"  /* Replace L Up Leg Skin w Autol Sub, Full Thick, Extern */
"0HRJX74"  =  "1"  /* Replace L Up Leg Skin w Autol Sub, Part Thick, Extern */
"0HRJXJ3"  =  "1"  /* Replace L Up Leg Skin w Synth Sub, Full Thick, Extern */
"0HRJXJ4"  =  "1"  /* Replace L Up Leg Skin w Synth Sub, Part Thick, Extern */
"0HRJXJZ"  =  "1"  /* Replacement of L Up Leg Skin with Synth Sub, Extern Approach */
"0HRJXK3"  =  "1"  /* Replace L Up Leg Skin w Nonaut Sub, Full Thick, Extern */
"0HRJXK4"  =  "1"  /* Replace L Up Leg Skin w Nonaut Sub, Part Thick, Extern */
"0HRKX73"  =  "1"  /* Replace R Low Leg Skin w Autol Sub, Full Thick, Extern */
"0HRKX74"  =  "1"  /* Replace R Low Leg Skin w Autol Sub, Part Thick, Extern */
"0HRKXJ3"  =  "1"  /* Replace R Low Leg Skin w Synth Sub, Full Thick, Extern */
"0HRKXJ4"  =  "1"  /* Replace R Low Leg Skin w Synth Sub, Part Thick, Extern */
"0HRKXJZ"  =  "1"  /* Replace of R Low Leg Skin with Synth Sub, Extern Approach */
"0HRKXK3"  =  "1"  /* Replace R Low Leg Skin w Nonaut Sub, Full Thick, Extern */
"0HRKXK4"  =  "1"  /* Replace R Low Leg Skin w Nonaut Sub, Part Thick, Extern */
"0HRLX73"  =  "1"  /* Replace L Low Leg Skin w Autol Sub, Full Thick, Extern */
"0HRLX74"  =  "1"  /* Replace L Low Leg Skin w Autol Sub, Part Thick, Extern */
"0HRLXJ3"  =  "1"  /* Replace L Low Leg Skin w Synth Sub, Full Thick, Extern */
"0HRLXJ4"  =  "1"  /* Replace L Low Leg Skin w Synth Sub, Part Thick, Extern */
"0HRLXJZ"  =  "1"  /* Replace of L Low Leg Skin with Synth Sub, Extern Approach */
"0HRLXK3"  =  "1"  /* Replace L Low Leg Skin w Nonaut Sub, Full Thick, Extern */
"0HRLXK4"  =  "1"  /* Replace L Low Leg Skin w Nonaut Sub, Part Thick, Extern */
"0HRMX73"  =  "1"  /* Replace R Foot Skin w Autol Sub, Full Thick, Extern */
"0HRMX74"  =  "1"  /* Replace R Foot Skin w Autol Sub, Part Thick, Extern */
"0HRMXJ3"  =  "1"  /* Replace R Foot Skin w Synth Sub, Full Thick, Extern */
"0HRMXJ4"  =  "1"  /* Replace R Foot Skin w Synth Sub, Part Thick, Extern */
"0HRMXJZ"  =  "1"  /* Replacement of R Foot Skin with Synth Sub, Extern Approach */
"0HRMXK3"  =  "1"  /* Replace R Foot Skin w Nonaut Sub, Full Thick, Extern */
"0HRMXK4"  =  "1"  /* Replace R Foot Skin w Nonaut Sub, Part Thick, Extern */
"0HRNX73"  =  "1"  /* Replace L Foot Skin w Autol Sub, Full Thick, Extern */
"0HRNX74"  =  "1"  /* Replace L Foot Skin w Autol Sub, Part Thick, Extern */
"0HRNXJ3"  =  "1"  /* Replace L Foot Skin w Synth Sub, Full Thick, Extern */
"0HRNXJ4"  =  "1"  /* Replace L Foot Skin w Synth Sub, Part Thick, Extern */
"0HRNXJZ"  =  "1"  /* Replacement of L Foot Skin with Synth Sub, Extern Approach */
"0HRNXK3"  =  "1"  /* Replace L Foot Skin w Nonaut Sub, Full Thick, Extern */
"0HRNXK4"  =  "1"  /* Replace L Foot Skin w Nonaut Sub, Part Thick, Extern */
"0HRQX7Z"  =  "1"  /* Replacement of Finger Nail with Autol Sub, Extern Approach */
"0HRQXJZ"  =  "1"  /* Replacement of Finger Nail with Synth Sub, Extern Approach */
"0HRQXKZ"  =  "1"  /* Replacement of Finger Nail with Nonaut Sub, Extern Approach */
"0HRRX7Z"  =  "1"  /* Replacement of Toe Nail with Autol Sub, Extern Approach */
"0HRRXJZ"  =  "1"  /* Replacement of Toe Nail with Synth Sub, Extern Approach */
"0HRRXKZ"  =  "1"  /* Replacement of Toe Nail with Nonaut Sub, Extern Approach */
"0HRSXJZ"  =  "1"  /* Replacement of Hair with Synth Sub, Extern Approach */
"0HRSXKZ"  =  "1"  /* Replacement of Hair with Nonaut Sub, Extern Approach */
"0HRT075"  =  "1"  /* Replacement of R Breast using Lat Dorsi Flap, Open Approach */
"0HRT076"  =  "1"  /* Replacement of Right Breast using TRAM Flap, Open Approach */
"0HRT077"  =  "1"  /* Replacement of Right Breast using DIEP Flap, Open Approach */
"0HRT078"  =  "1"  /* Replacement of Right Breast using SIEA Flap, Open Approach */
"0HRT079"  =  "1"  /* Replacement of Right Breast using GAP Flap, Open Approach */
"0HRT07Z"  =  "1"  /* Replacement of Right Breast with Autol Sub, Open Approach */
"0HRT0JZ"  =  "1"  /* Replacement of Right Breast with Synth Sub, Open Approach */
"0HRT0KZ"  =  "1"  /* Replacement of Right Breast with Nonaut Sub, Open Approach */
"0HRT37Z"  =  "1"  /* Replacement of Right Breast with Autol Sub, Perc Approach */
"0HRT3JZ"  =  "1"  /* Replacement of Right Breast with Synth Sub, Perc Approach */
"0HRT3KZ"  =  "1"  /* Replacement of Right Breast with Nonaut Sub, Perc Approach */
"0HRTX7Z"  =  "1"  /* Replacement of Right Breast with Autol Sub, Extern Approach */
"0HRTXJZ"  =  "1"  /* Replacement of Right Breast with Synth Sub, Extern Approach */
"0HRTXKZ"  =  "1"  /* Replacement of Right Breast with Nonaut Sub, Extern Approach */
"0HRU075"  =  "1"  /* Replacement of L Breast using Lat Dorsi Flap, Open Approach */
"0HRU076"  =  "1"  /* Replacement of Left Breast using TRAM Flap, Open Approach */
"0HRU077"  =  "1"  /* Replacement of Left Breast using DIEP Flap, Open Approach */
"0HRU078"  =  "1"  /* Replacement of Left Breast using SIEA Flap, Open Approach */
"0HRU079"  =  "1"  /* Replacement of Left Breast using GAP Flap, Open Approach */
"0HRU07Z"  =  "1"  /* Replacement of Left Breast with Autol Sub, Open Approach */
"0HRU0JZ"  =  "1"  /* Replacement of Left Breast with Synth Sub, Open Approach */
"0HRU0KZ"  =  "1"  /* Replacement of Left Breast with Nonaut Sub, Open Approach */
"0HRU37Z"  =  "1"  /* Replacement of Left Breast with Autol Sub, Perc Approach */
"0HRU3JZ"  =  "1"  /* Replacement of Left Breast with Synth Sub, Perc Approach */
"0HRU3KZ"  =  "1"  /* Replacement of Left Breast with Nonaut Sub, Perc Approach */
"0HRUX7Z"  =  "1"  /* Replacement of Left Breast with Autol Sub, Extern Approach */
"0HRUXJZ"  =  "1"  /* Replacement of Left Breast with Synth Sub, Extern Approach */
"0HRUXKZ"  =  "1"  /* Replacement of Left Breast with Nonaut Sub, Extern Approach */
"0HRV075"  =  "1"  /* Replacement of Bi Breast using Lat Dorsi Flap, Open Approach */
"0HRV076"  =  "1"  /* Replacement of Bi Breast using TRAM Flap, Open Approach */
"0HRV077"  =  "1"  /* Replacement of Bi Breast using DIEP Flap, Open Approach */
"0HRV078"  =  "1"  /* Replacement of Bi Breast using SIEA Flap, Open Approach */
"0HRV079"  =  "1"  /* Replacement of Bi Breast using GAP Flap, Open Approach */
"0HRV07Z"  =  "1"  /* Replacement of Bi Breast with Autol Sub, Open Approach */
"0HRV0JZ"  =  "1"  /* Replacement of Bi Breast with Synth Sub, Open Approach */
"0HRV0KZ"  =  "1"  /* Replacement of Bi Breast with Nonaut Sub, Open Approach */
"0HRV37Z"  =  "1"  /* Replacement of Bi Breast with Autol Sub, Perc Approach */
"0HRV3JZ"  =  "1"  /* Replacement of Bi Breast with Synth Sub, Perc Approach */
"0HRV3KZ"  =  "1"  /* Replacement of Bi Breast with Nonaut Sub, Perc Approach */
"0HRVX7Z"  =  "1"  /* Replacement of Bi Breast with Autol Sub, Extern Approach */
"0HRVXJZ"  =  "1"  /* Replacement of Bi Breast with Synth Sub, Extern Approach */
"0HRVXKZ"  =  "1"  /* Replacement of Bi Breast with Nonaut Sub, Extern Approach */
"0HRW07Z"  =  "1"  /* Replacement of Right Nipple with Autol Sub, Open Approach */
"0HRW0JZ"  =  "1"  /* Replacement of Right Nipple with Synth Sub, Open Approach */
"0HRW0KZ"  =  "1"  /* Replacement of Right Nipple with Nonaut Sub, Open Approach */
"0HRW37Z"  =  "1"  /* Replacement of Right Nipple with Autol Sub, Perc Approach */
"0HRW3JZ"  =  "1"  /* Replacement of Right Nipple with Synth Sub, Perc Approach */
"0HRW3KZ"  =  "1"  /* Replacement of Right Nipple with Nonaut Sub, Perc Approach */
"0HRWX7Z"  =  "1"  /* Replacement of Right Nipple with Autol Sub, Extern Approach */
"0HRWXJZ"  =  "1"  /* Replacement of Right Nipple with Synth Sub, Extern Approach */
"0HRWXKZ"  =  "1"  /* Replacement of Right Nipple with Nonaut Sub, Extern Approach */
"0HRX07Z"  =  "1"  /* Replacement of Left Nipple with Autol Sub, Open Approach */
"0HRX0JZ"  =  "1"  /* Replacement of Left Nipple with Synth Sub, Open Approach */
"0HRX0KZ"  =  "1"  /* Replacement of Left Nipple with Nonaut Sub, Open Approach */
"0HRX37Z"  =  "1"  /* Replacement of Left Nipple with Autol Sub, Perc Approach */
"0HRX3JZ"  =  "1"  /* Replacement of Left Nipple with Synth Sub, Perc Approach */
"0HRX3KZ"  =  "1"  /* Replacement of Left Nipple with Nonaut Sub, Perc Approach */
"0HRXX7Z"  =  "1"  /* Replacement of Left Nipple with Autol Sub, Extern Approach */
"0HRXXJZ"  =  "1"  /* Replacement of Left Nipple with Synth Sub, Extern Approach */
"0HRXXKZ"  =  "1"  /* Replacement of Left Nipple with Nonaut Sub, Extern Approach */
"0HST0ZZ"  =  "1"  /* Reposition Right Breast, Open Approach */
"0HSU0ZZ"  =  "1"  /* Reposition Left Breast, Open Approach */
"0HSV0ZZ"  =  "1"  /* Reposition Bilateral Breast, Open Approach */
"0HSWXZZ"  =  "1"  /* Reposition Right Nipple, External Approach */
"0HSXXZZ"  =  "1"  /* Reposition Left Nipple, External Approach */
"0HTT0ZZ"  =  "1"  /* Resection of Right Breast, Open Approach */
"0HTU0ZZ"  =  "1"  /* Resection of Left Breast, Open Approach */
"0HTV0ZZ"  =  "1"  /* Resection of Bilateral Breast, Open Approach */
"0HTWXZZ"  =  "1"  /* Resection of Right Nipple, External Approach */
"0HTXXZZ"  =  "1"  /* Resection of Left Nipple, External Approach */
"0HTY0ZZ"  =  "1"  /* Resection of Supernumerary Breast, Open Approach */
"0HUT07Z"  =  "1"  /* Supplement Right Breast with Autol Sub, Open Approach */
"0HUT0JZ"  =  "1"  /* Supplement Right Breast with Synth Sub, Open Approach */
"0HUT0KZ"  =  "1"  /* Supplement Right Breast with Nonaut Sub, Open Approach */
"0HUT37Z"  =  "1"  /* Supplement Right Breast with Autol Sub, Perc Approach */
"0HUT3JZ"  =  "1"  /* Supplement Right Breast with Synth Sub, Perc Approach */
"0HUT3KZ"  =  "1"  /* Supplement Right Breast with Nonaut Sub, Perc Approach */
"0HUT77Z"  =  "1"  /* Supplement Right Breast with Autol Sub, Via Opening */
"0HUT7JZ"  =  "1"  /* Supplement Right Breast with Synth Sub, Via Opening */
"0HUT7KZ"  =  "1"  /* Supplement Right Breast with Nonaut Sub, Via Opening */
"0HUT87Z"  =  "1"  /* Supplement Right Breast with Autol Sub, Endo */
"0HUT8JZ"  =  "1"  /* Supplement Right Breast with Synthetic Substitute, Endo */
"0HUT8KZ"  =  "1"  /* Supplement Right Breast with Nonaut Sub, Endo */
"0HUTX7Z"  =  "1"  /* Supplement Right Breast with Autol Sub, Extern Approach */
"0HUTXJZ"  =  "1"  /* Supplement Right Breast with Synth Sub, Extern Approach */
"0HUTXKZ"  =  "1"  /* Supplement Right Breast with Nonaut Sub, Extern Approach */
"0HUU07Z"  =  "1"  /* Supplement Left Breast with Autol Sub, Open Approach */
"0HUU0JZ"  =  "1"  /* Supplement Left Breast with Synth Sub, Open Approach */
"0HUU0KZ"  =  "1"  /* Supplement Left Breast with Nonaut Sub, Open Approach */
"0HUU37Z"  =  "1"  /* Supplement Left Breast with Autol Sub, Perc Approach */
"0HUU3JZ"  =  "1"  /* Supplement Left Breast with Synth Sub, Perc Approach */
"0HUU3KZ"  =  "1"  /* Supplement Left Breast with Nonaut Sub, Perc Approach */
"0HUU77Z"  =  "1"  /* Supplement Left Breast with Autol Sub, Via Opening */
"0HUU7JZ"  =  "1"  /* Supplement Left Breast with Synth Sub, Via Opening */
"0HUU7KZ"  =  "1"  /* Supplement Left Breast with Nonaut Sub, Via Opening */
"0HUU87Z"  =  "1"  /* Supplement Left Breast with Autol Sub, Endo */
"0HUU8JZ"  =  "1"  /* Supplement Left Breast with Synthetic Substitute, Endo */
"0HUU8KZ"  =  "1"  /* Supplement Left Breast with Nonaut Sub, Endo */
"0HUUX7Z"  =  "1"  /* Supplement Left Breast with Autol Sub, Extern Approach */
"0HUUXJZ"  =  "1"  /* Supplement Left Breast with Synth Sub, Extern Approach */
"0HUUXKZ"  =  "1"  /* Supplement Left Breast with Nonaut Sub, Extern Approach */
"0HUV07Z"  =  "1"  /* Supplement Bilateral Breast with Autol Sub, Open Approach */
"0HUV0JZ"  =  "1"  /* Supplement Bilateral Breast with Synth Sub, Open Approach */
"0HUV0KZ"  =  "1"  /* Supplement Bilateral Breast with Nonaut Sub, Open Approach */
"0HUV37Z"  =  "1"  /* Supplement Bilateral Breast with Autol Sub, Perc Approach */
"0HUV3JZ"  =  "1"  /* Supplement Bilateral Breast with Synth Sub, Perc Approach */
"0HUV3KZ"  =  "1"  /* Supplement Bilateral Breast with Nonaut Sub, Perc Approach */
"0HUV77Z"  =  "1"  /* Supplement Bilateral Breast with Autol Sub, Via Opening */
"0HUV7JZ"  =  "1"  /* Supplement Bilateral Breast with Synth Sub, Via Opening */
"0HUV7KZ"  =  "1"  /* Supplement Bilateral Breast with Nonaut Sub, Via Opening */
"0HUV87Z"  =  "1"  /* Supplement Bilateral Breast with Autol Sub, Endo */
"0HUV8JZ"  =  "1"  /* Supplement Bilateral Breast with Synthetic Substitute, Endo */
"0HUV8KZ"  =  "1"  /* Supplement Bilateral Breast with Nonaut Sub, Endo */
"0HUVX7Z"  =  "1"  /* Supplement Bilateral Breast with Autol Sub, Extern Approach */
"0HUVXJZ"  =  "1"  /* Supplement Bilateral Breast with Synth Sub, Extern Approach */
"0HUVXKZ"  =  "1"  /* Supplement Bilateral Breast with Nonaut Sub, Extern Approach */
"0HUW07Z"  =  "1"  /* Supplement Right Nipple with Autol Sub, Open Approach */
"0HUW0JZ"  =  "1"  /* Supplement Right Nipple with Synth Sub, Open Approach */
"0HUW0KZ"  =  "1"  /* Supplement Right Nipple with Nonaut Sub, Open Approach */
"0HUW37Z"  =  "1"  /* Supplement Right Nipple with Autol Sub, Perc Approach */
"0HUW3JZ"  =  "1"  /* Supplement Right Nipple with Synth Sub, Perc Approach */
"0HUW3KZ"  =  "1"  /* Supplement Right Nipple with Nonaut Sub, Perc Approach */
"0HUW77Z"  =  "1"  /* Supplement Right Nipple with Autol Sub, Via Opening */
"0HUW7JZ"  =  "1"  /* Supplement Right Nipple with Synth Sub, Via Opening */
"0HUW7KZ"  =  "1"  /* Supplement Right Nipple with Nonaut Sub, Via Opening */
"0HUW87Z"  =  "1"  /* Supplement Right Nipple with Autol Sub, Endo */
"0HUW8JZ"  =  "1"  /* Supplement Right Nipple with Synthetic Substitute, Endo */
"0HUW8KZ"  =  "1"  /* Supplement Right Nipple with Nonaut Sub, Endo */
"0HUWX7Z"  =  "1"  /* Supplement Right Nipple with Autol Sub, Extern Approach */
"0HUWXJZ"  =  "1"  /* Supplement Right Nipple with Synth Sub, Extern Approach */
"0HUWXKZ"  =  "1"  /* Supplement Right Nipple with Nonaut Sub, Extern Approach */
"0HUX07Z"  =  "1"  /* Supplement Left Nipple with Autol Sub, Open Approach */
"0HUX0JZ"  =  "1"  /* Supplement Left Nipple with Synth Sub, Open Approach */
"0HUX0KZ"  =  "1"  /* Supplement Left Nipple with Nonaut Sub, Open Approach */
"0HUX37Z"  =  "1"  /* Supplement Left Nipple with Autol Sub, Perc Approach */
"0HUX3JZ"  =  "1"  /* Supplement Left Nipple with Synth Sub, Perc Approach */
"0HUX3KZ"  =  "1"  /* Supplement Left Nipple with Nonaut Sub, Perc Approach */
"0HUX77Z"  =  "1"  /* Supplement Left Nipple with Autol Sub, Via Opening */
"0HUX7JZ"  =  "1"  /* Supplement Left Nipple with Synth Sub, Via Opening */
"0HUX7KZ"  =  "1"  /* Supplement Left Nipple with Nonaut Sub, Via Opening */
"0HUX87Z"  =  "1"  /* Supplement Left Nipple with Autol Sub, Endo */
"0HUX8JZ"  =  "1"  /* Supplement Left Nipple with Synthetic Substitute, Endo */
"0HUX8KZ"  =  "1"  /* Supplement Left Nipple with Nonaut Sub, Endo */
"0HUXX7Z"  =  "1"  /* Supplement Left Nipple with Autol Sub, Extern Approach */
"0HUXXJZ"  =  "1"  /* Supplement Left Nipple with Synth Sub, Extern Approach */
"0HUXXKZ"  =  "1"  /* Supplement Left Nipple with Nonaut Sub, Extern Approach */
"0HWT0JZ"  =  "1"  /* Revision of Synthetic Substitute in R Breast, Open Approach */
"0HWT3JZ"  =  "1"  /* Revision of Synthetic Substitute in R Breast, Perc Approach */
"0HWU0JZ"  =  "1"  /* Revision of Synthetic Substitute in L Breast, Open Approach */
"0HWU3JZ"  =  "1"  /* Revision of Synthetic Substitute in L Breast, Perc Approach */
"0HX0XZZ"  =  "1"  /* Transfer Scalp Skin, External Approach */
"0HX1XZZ"  =  "1"  /* Transfer Face Skin, External Approach */
"0HX2XZZ"  =  "1"  /* Transfer Right Ear Skin, External Approach */
"0HX3XZZ"  =  "1"  /* Transfer Left Ear Skin, External Approach */
"0HX4XZZ"  =  "1"  /* Transfer Neck Skin, External Approach */
"0HX5XZZ"  =  "1"  /* Transfer Chest Skin, External Approach */
"0HX6XZZ"  =  "1"  /* Transfer Back Skin, External Approach */
"0HX7XZZ"  =  "1"  /* Transfer Abdomen Skin, External Approach */
"0HX8XZZ"  =  "1"  /* Transfer Buttock Skin, External Approach */
"0HX9XZZ"  =  "1"  /* Transfer Perineum Skin, External Approach */
"0HXAXZZ"  =  "1"  /* Transfer Genitalia Skin, External Approach */
"0HXBXZZ"  =  "1"  /* Transfer Right Upper Arm Skin, External Approach */
"0HXCXZZ"  =  "1"  /* Transfer Left Upper Arm Skin, External Approach */
"0HXDXZZ"  =  "1"  /* Transfer Right Lower Arm Skin, External Approach */
"0HXEXZZ"  =  "1"  /* Transfer Left Lower Arm Skin, External Approach */
"0HXFXZZ"  =  "1"  /* Transfer Right Hand Skin, External Approach */
"0HXGXZZ"  =  "1"  /* Transfer Left Hand Skin, External Approach */
"0HXHXZZ"  =  "1"  /* Transfer Right Upper Leg Skin, External Approach */
"0HXJXZZ"  =  "1"  /* Transfer Left Upper Leg Skin, External Approach */
"0HXKXZZ"  =  "1"  /* Transfer Right Lower Leg Skin, External Approach */
"0HXLXZZ"  =  "1"  /* Transfer Left Lower Leg Skin, External Approach */
"0HXMXZZ"  =  "1"  /* Transfer Right Foot Skin, External Approach */
"0HXNXZZ"  =  "1"  /* Transfer Left Foot Skin, External Approach */
"0J010ZZ"  =  "1"  /* Alteration of Face Subcu/Fascia, Open Approach */
"0J013ZZ"  =  "1"  /* Alteration of Face Subcu/Fascia, Perc Approach */
"0J040ZZ"  =  "1"  /* Alteration of Ant Neck Subcu/Fascia, Open Approach */
"0J043ZZ"  =  "1"  /* Alteration of Ant Neck Subcu/Fascia, Perc Approach */
"0J050ZZ"  =  "1"  /* Alteration of Post Neck Subcu/Fascia, Open Approach */
"0J053ZZ"  =  "1"  /* Alteration of Post Neck Subcu/Fascia, Perc Approach */
"0J060ZZ"  =  "1"  /* Alteration of Chest Subcu/Fascia, Open Approach */
"0J063ZZ"  =  "1"  /* Alteration of Chest Subcu/Fascia, Perc Approach */
"0J070ZZ"  =  "1"  /* Alteration of Back Subcu/Fascia, Open Approach */
"0J073ZZ"  =  "1"  /* Alteration of Back Subcu/Fascia, Perc Approach */
"0J080ZZ"  =  "1"  /* Alteration of Abd Subcu/Fascia, Open Approach */
"0J083ZZ"  =  "1"  /* Alteration of Abd Subcu/Fascia, Perc Approach */
"0J090ZZ"  =  "1"  /* Alteration of Buttock Subcu/Fascia, Open Approach */
"0J093ZZ"  =  "1"  /* Alteration of Buttock Subcu/Fascia, Perc Approach */
"0J0D0ZZ"  =  "1"  /* Alteration of R Up Arm Subcu/Fascia, Open Approach */
"0J0D3ZZ"  =  "1"  /* Alteration of R Up Arm Subcu/Fascia, Perc Approach */
"0J0F0ZZ"  =  "1"  /* Alteration of L Up Arm Subcu/Fascia, Open Approach */
"0J0F3ZZ"  =  "1"  /* Alteration of L Up Arm Subcu/Fascia, Perc Approach */
"0J0G0ZZ"  =  "1"  /* Alteration of R Low Arm Subcu/Fascia, Open Approach */
"0J0G3ZZ"  =  "1"  /* Alteration of R Low Arm Subcu/Fascia, Perc Approach */
"0J0H0ZZ"  =  "1"  /* Alteration of L Low Arm Subcu/Fascia, Open Approach */
"0J0H3ZZ"  =  "1"  /* Alteration of L Low Arm Subcu/Fascia, Perc Approach */
"0J0L0ZZ"  =  "1"  /* Alteration of R Up Leg Subcu/Fascia, Open Approach */
"0J0L3ZZ"  =  "1"  /* Alteration of R Up Leg Subcu/Fascia, Perc Approach */
"0J0M0ZZ"  =  "1"  /* Alteration of L Up Leg Subcu/Fascia, Open Approach */
"0J0M3ZZ"  =  "1"  /* Alteration of L Up Leg Subcu/Fascia, Perc Approach */
"0J0N0ZZ"  =  "1"  /* Alteration of R Low Leg Subcu/Fascia, Open Approach */
"0J0N3ZZ"  =  "1"  /* Alteration of R Low Leg Subcu/Fascia, Perc Approach */
"0J0P0ZZ"  =  "1"  /* Alteration of L Low Leg Subcu/Fascia, Open Approach */
"0J0P3ZZ"  =  "1"  /* Alteration of L Low Leg Subcu/Fascia, Perc Approach */
"0J800ZZ"  =  "1"  /* Division of Scalp Subcu/Fascia, Open Approach */
"0J803ZZ"  =  "1"  /* Division of Scalp Subcu/Fascia, Perc Approach */
"0J810ZZ"  =  "1"  /* Division of Face Subcu/Fascia, Open Approach */
"0J813ZZ"  =  "1"  /* Division of Face Subcu/Fascia, Perc Approach */
"0J840ZZ"  =  "1"  /* Division of Ant Neck Subcu/Fascia, Open Approach */
"0J843ZZ"  =  "1"  /* Division of Ant Neck Subcu/Fascia, Perc Approach */
"0J850ZZ"  =  "1"  /* Division of Post Neck Subcu/Fascia, Open Approach */
"0J853ZZ"  =  "1"  /* Division of Post Neck Subcu/Fascia, Perc Approach */
"0J860ZZ"  =  "1"  /* Division of Chest Subcu/Fascia, Open Approach */
"0J863ZZ"  =  "1"  /* Division of Chest Subcu/Fascia, Perc Approach */
"0J870ZZ"  =  "1"  /* Division of Back Subcu/Fascia, Open Approach */
"0J873ZZ"  =  "1"  /* Division of Back Subcu/Fascia, Perc Approach */
"0J880ZZ"  =  "1"  /* Division of Abd Subcu/Fascia, Open Approach */
"0J883ZZ"  =  "1"  /* Division of Abd Subcu/Fascia, Perc Approach */
"0J890ZZ"  =  "1"  /* Division of Buttock Subcu/Fascia, Open Approach */
"0J893ZZ"  =  "1"  /* Division of Buttock Subcu/Fascia, Perc Approach */
"0J8B0ZZ"  =  "1"  /* Division of Perineum Subcu/Fascia, Open Approach */
"0J8B3ZZ"  =  "1"  /* Division of Perineum Subcu/Fascia, Perc Approach */
"0J8C0ZZ"  =  "1"  /* Division of Pelvic Subcu/Fascia, Open Approach */
"0J8C3ZZ"  =  "1"  /* Division of Pelvic Subcu/Fascia, Perc Approach */
"0J8D0ZZ"  =  "1"  /* Division of R Up Arm Subcu/Fascia, Open Approach */
"0J8D3ZZ"  =  "1"  /* Division of R Up Arm Subcu/Fascia, Perc Approach */
"0J8F0ZZ"  =  "1"  /* Division of L Up Arm Subcu/Fascia, Open Approach */
"0J8F3ZZ"  =  "1"  /* Division of L Up Arm Subcu/Fascia, Perc Approach */
"0J8G0ZZ"  =  "1"  /* Division of R Low Arm Subcu/Fascia, Open Approach */
"0J8G3ZZ"  =  "1"  /* Division of R Low Arm Subcu/Fascia, Perc Approach */
"0J8H0ZZ"  =  "1"  /* Division of L Low Arm Subcu/Fascia, Open Approach */
"0J8H3ZZ"  =  "1"  /* Division of L Low Arm Subcu/Fascia, Perc Approach */
"0J8J0ZZ"  =  "1"  /* Division of R Hand Subcu/Fascia, Open Approach */
"0J8J3ZZ"  =  "1"  /* Division of R Hand Subcu/Fascia, Perc Approach */
"0J8K0ZZ"  =  "1"  /* Division of L Hand Subcu/Fascia, Open Approach */
"0J8K3ZZ"  =  "1"  /* Division of L Hand Subcu/Fascia, Perc Approach */
"0J8L0ZZ"  =  "1"  /* Division of R Up Leg Subcu/Fascia, Open Approach */
"0J8L3ZZ"  =  "1"  /* Division of R Up Leg Subcu/Fascia, Perc Approach */
"0J8M0ZZ"  =  "1"  /* Division of L Up Leg Subcu/Fascia, Open Approach */
"0J8M3ZZ"  =  "1"  /* Division of L Up Leg Subcu/Fascia, Perc Approach */
"0J8N0ZZ"  =  "1"  /* Division of R Low Leg Subcu/Fascia, Open Approach */
"0J8N3ZZ"  =  "1"  /* Division of R Low Leg Subcu/Fascia, Perc Approach */
"0J8P0ZZ"  =  "1"  /* Division of L Low Leg Subcu/Fascia, Open Approach */
"0J8P3ZZ"  =  "1"  /* Division of L Low Leg Subcu/Fascia, Perc Approach */
"0J8Q0ZZ"  =  "1"  /* Division of R Foot Subcu/Fascia, Open Approach */
"0J8Q3ZZ"  =  "1"  /* Division of R Foot Subcu/Fascia, Perc Approach */
"0J8R0ZZ"  =  "1"  /* Division of L Foot Subcu/Fascia, Open Approach */
"0J8R3ZZ"  =  "1"  /* Division of L Foot Subcu/Fascia, Perc Approach */
"0J8S0ZZ"  =  "1"  /* Division of Head & Neck Subcu/Fascia, Open Approach */
"0J8S3ZZ"  =  "1"  /* Division of Head & Neck Subcu/Fascia, Perc Approach */
"0J8T0ZZ"  =  "1"  /* Division of Trunk Subcu/Fascia, Open Approach */
"0J8T3ZZ"  =  "1"  /* Division of Trunk Subcu/Fascia, Perc Approach */
"0J8V0ZZ"  =  "1"  /* Division of Up Extrem Subcu/Fascia, Open Approach */
"0J8V3ZZ"  =  "1"  /* Division of Up Extrem Subcu/Fascia, Perc Approach */
"0J8W0ZZ"  =  "1"  /* Division of Low Extrem Subcu/Fascia, Open Approach */
"0J8W3ZZ"  =  "1"  /* Division of Low Extrem Subcu/Fascia, Perc Approach */
"0J900ZZ"  =  "1"  /* Drainage of Scalp Subcu/Fascia, Open Approach */
"0J9100Z"  =  "1"  /* Drainage of Face Subcu/Fascia with Drain Dev, Open Approach */
"0J910ZZ"  =  "1"  /* Drainage of Face Subcu/Fascia, Open Approach */
"0J9130Z"  =  "1"  /* Drainage of Face Subcu/Fascia with Drain Dev, Perc Approach */
"0J940ZZ"  =  "1"  /* Drainage of Ant Neck Subcu/Fascia, Open Approach */
"0J950ZZ"  =  "1"  /* Drainage of Post Neck Subcu/Fascia, Open Approach */
"0J960ZZ"  =  "1"  /* Drainage of Chest Subcu/Fascia, Open Approach */
"0J970ZZ"  =  "1"  /* Drainage of Back Subcu/Fascia, Open Approach */
"0J980ZZ"  =  "1"  /* Drainage of Abd Subcu/Fascia, Open Approach */
"0J990ZZ"  =  "1"  /* Drainage of Buttock Subcu/Fascia, Open Approach */
"0J9B0ZZ"  =  "1"  /* Drainage of Perineum Subcu/Fascia, Open Approach */
"0J9C0ZZ"  =  "1"  /* Drainage of Pelvic Subcu/Fascia, Open Approach */
"0J9D0ZZ"  =  "1"  /* Drainage of R Up Arm Subcu/Fascia, Open Approach */
"0J9F0ZZ"  =  "1"  /* Drainage of L Up Arm Subcu/Fascia, Open Approach */
"0J9G0ZZ"  =  "1"  /* Drainage of R Low Arm Subcu/Fascia, Open Approach */
"0J9H0ZZ"  =  "1"  /* Drainage of L Low Arm Subcu/Fascia, Open Approach */
"0J9J00Z"  =  "1"  /* Drain of R Hand Subcu/Fascia with Drain Dev, Open Approach */
"0J9J0ZZ"  =  "1"  /* Drainage of R Hand Subcu/Fascia, Open Approach */
"0J9J30Z"  =  "1"  /* Drain of R Hand Subcu/Fascia with Drain Dev, Perc Approach */
"0J9K00Z"  =  "1"  /* Drain of L Hand Subcu/Fascia with Drain Dev, Open Approach */
"0J9K0ZZ"  =  "1"  /* Drainage of L Hand Subcu/Fascia, Open Approach */
"0J9K30Z"  =  "1"  /* Drain of L Hand Subcu/Fascia with Drain Dev, Perc Approach */
"0J9L0ZZ"  =  "1"  /* Drainage of R Up Leg Subcu/Fascia, Open Approach */
"0J9M0ZZ"  =  "1"  /* Drainage of L Up Leg Subcu/Fascia, Open Approach */
"0J9N0ZZ"  =  "1"  /* Drainage of R Low Leg Subcu/Fascia, Open Approach */
"0J9P0ZZ"  =  "1"  /* Drainage of L Low Leg Subcu/Fascia, Open Approach */
"0J9Q0ZZ"  =  "1"  /* Drainage of R Foot Subcu/Fascia, Open Approach */
"0J9R0ZZ"  =  "1"  /* Drainage of L Foot Subcu/Fascia, Open Approach */
"0JB00ZZ"  =  "1"  /* Excision of Scalp Subcu/Fascia, Open Approach */
"0JB10ZZ"  =  "1"  /* Excision of Face Subcu/Fascia, Open Approach */
"0JB13ZZ"  =  "1"  /* Excision of Face Subcu/Fascia, Perc Approach */
"0JB40ZZ"  =  "1"  /* Excision of Ant Neck Subcu/Fascia, Open Approach */
"0JB50ZZ"  =  "1"  /* Excision of Post Neck Subcu/Fascia, Open Approach */
"0JB60ZZ"  =  "1"  /* Excision of Chest Subcu/Fascia, Open Approach */
"0JB70ZZ"  =  "1"  /* Excision of Back Subcu/Fascia, Open Approach */
"0JB80ZZ"  =  "1"  /* Excision of Abd Subcu/Fascia, Open Approach */
"0JB90ZZ"  =  "1"  /* Excision of Buttock Subcu/Fascia, Open Approach */
"0JBB0ZZ"  =  "1"  /* Excision of Perineum Subcu/Fascia, Open Approach */
"0JBC0ZZ"  =  "1"  /* Excision of Pelvic Subcu/Fascia, Open Approach */
"0JBD0ZZ"  =  "1"  /* Excision of R Up Arm Subcu/Fascia, Open Approach */
"0JBF0ZZ"  =  "1"  /* Excision of L Up Arm Subcu/Fascia, Open Approach */
"0JBG0ZZ"  =  "1"  /* Excision of R Low Arm Subcu/Fascia, Open Approach */
"0JBH0ZZ"  =  "1"  /* Excision of L Low Arm Subcu/Fascia, Open Approach */
"0JBJ0ZZ"  =  "1"  /* Excision of R Hand Subcu/Fascia, Open Approach */
"0JBJ3ZZ"  =  "1"  /* Excision of R Hand Subcu/Fascia, Perc Approach */
"0JBK0ZZ"  =  "1"  /* Excision of L Hand Subcu/Fascia, Open Approach */
"0JBK3ZZ"  =  "1"  /* Excision of L Hand Subcu/Fascia, Perc Approach */
"0JBL0ZZ"  =  "1"  /* Excision of R Up Leg Subcu/Fascia, Open Approach */
"0JBM0ZZ"  =  "1"  /* Excision of L Up Leg Subcu/Fascia, Open Approach */
"0JBN0ZZ"  =  "1"  /* Excision of R Low Leg Subcu/Fascia, Open Approach */
"0JBP0ZZ"  =  "1"  /* Excision of L Low Leg Subcu/Fascia, Open Approach */
"0JBQ0ZZ"  =  "1"  /* Excision of R Foot Subcu/Fascia, Open Approach */
"0JBR0ZZ"  =  "1"  /* Excision of L Foot Subcu/Fascia, Open Approach */
"0JD00ZZ"  =  "1"  /* Extraction of Scalp Subcu/Fascia, Open Approach */
"0JD03ZZ"  =  "1"  /* Extraction of Scalp Subcu/Fascia, Perc Approach */
"0JD10ZZ"  =  "1"  /* Extraction of Face Subcu/Fascia, Open Approach */
"0JD13ZZ"  =  "1"  /* Extraction of Face Subcu/Fascia, Perc Approach */
"0JD40ZZ"  =  "1"  /* Extraction of Ant Neck Subcu/Fascia, Open Approach */
"0JD43ZZ"  =  "1"  /* Extraction of Ant Neck Subcu/Fascia, Perc Approach */
"0JD50ZZ"  =  "1"  /* Extraction of Post Neck Subcu/Fascia, Open Approach */
"0JD53ZZ"  =  "1"  /* Extraction of Post Neck Subcu/Fascia, Perc Approach */
"0JD60ZZ"  =  "1"  /* Extraction of Chest Subcu/Fascia, Open Approach */
"0JD63ZZ"  =  "1"  /* Extraction of Chest Subcu/Fascia, Perc Approach */
"0JD70ZZ"  =  "1"  /* Extraction of Back Subcu/Fascia, Open Approach */
"0JD73ZZ"  =  "1"  /* Extraction of Back Subcu/Fascia, Perc Approach */
"0JD80ZZ"  =  "1"  /* Extraction of Abd Subcu/Fascia, Open Approach */
"0JD83ZZ"  =  "1"  /* Extraction of Abd Subcu/Fascia, Perc Approach */
"0JD90ZZ"  =  "1"  /* Extraction of Buttock Subcu/Fascia, Open Approach */
"0JD93ZZ"  =  "1"  /* Extraction of Buttock Subcu/Fascia, Perc Approach */
"0JDB0ZZ"  =  "1"  /* Extraction of Perineum Subcu/Fascia, Open Approach */
"0JDB3ZZ"  =  "1"  /* Extraction of Perineum Subcu/Fascia, Perc Approach */
"0JDC0ZZ"  =  "1"  /* Extraction of Pelvic Subcu/Fascia, Open Approach */
"0JDC3ZZ"  =  "1"  /* Extraction of Pelvic Subcu/Fascia, Perc Approach */
"0JDD0ZZ"  =  "1"  /* Extraction of R Up Arm Subcu/Fascia, Open Approach */
"0JDD3ZZ"  =  "1"  /* Extraction of R Up Arm Subcu/Fascia, Perc Approach */
"0JDF0ZZ"  =  "1"  /* Extraction of L Up Arm Subcu/Fascia, Open Approach */
"0JDF3ZZ"  =  "1"  /* Extraction of L Up Arm Subcu/Fascia, Perc Approach */
"0JDG0ZZ"  =  "1"  /* Extraction of R Low Arm Subcu/Fascia, Open Approach */
"0JDG3ZZ"  =  "1"  /* Extraction of R Low Arm Subcu/Fascia, Perc Approach */
"0JDH0ZZ"  =  "1"  /* Extraction of L Low Arm Subcu/Fascia, Open Approach */
"0JDH3ZZ"  =  "1"  /* Extraction of L Low Arm Subcu/Fascia, Perc Approach */
"0JDJ0ZZ"  =  "1"  /* Extraction of R Hand Subcu/Fascia, Open Approach */
"0JDJ3ZZ"  =  "1"  /* Extraction of R Hand Subcu/Fascia, Perc Approach */
"0JDK0ZZ"  =  "1"  /* Extraction of L Hand Subcu/Fascia, Open Approach */
"0JDK3ZZ"  =  "1"  /* Extraction of L Hand Subcu/Fascia, Perc Approach */
"0JDL0ZZ"  =  "1"  /* Extraction of R Up Leg Subcu/Fascia, Open Approach */
"0JDL3ZZ"  =  "1"  /* Extraction of R Up Leg Subcu/Fascia, Perc Approach */
"0JDM0ZZ"  =  "1"  /* Extraction of L Up Leg Subcu/Fascia, Open Approach */
"0JDM3ZZ"  =  "1"  /* Extraction of L Up Leg Subcu/Fascia, Perc Approach */
"0JDN0ZZ"  =  "1"  /* Extraction of R Low Leg Subcu/Fascia, Open Approach */
"0JDN3ZZ"  =  "1"  /* Extraction of R Low Leg Subcu/Fascia, Perc Approach */
"0JDP0ZZ"  =  "1"  /* Extraction of L Low Leg Subcu/Fascia, Open Approach */
"0JDP3ZZ"  =  "1"  /* Extraction of L Low Leg Subcu/Fascia, Perc Approach */
"0JDQ0ZZ"  =  "1"  /* Extraction of R Foot Subcu/Fascia, Open Approach */
"0JDQ3ZZ"  =  "1"  /* Extraction of R Foot Subcu/Fascia, Perc Approach */
"0JDR0ZZ"  =  "1"  /* Extraction of L Foot Subcu/Fascia, Open Approach */
"0JDR3ZZ"  =  "1"  /* Extraction of L Foot Subcu/Fascia, Perc Approach */
"0JH00NZ"  =  "1"  /* Insert Tissue Expander in Scalp Subcu/Fascia, Open */
"0JH03NZ"  =  "1"  /* Insert Tissue Expander in Scalp Subcu/Fascia, Perc */
"0JH10NZ"  =  "1"  /* Insert Tissue Expander in Face Subcu/Fascia, Open */
"0JH13NZ"  =  "1"  /* Insert Tissue Expander in Face Subcu/Fascia, Perc */
"0JH40NZ"  =  "1"  /* Insert Tissue Expander in Ant Neck Subcu/Fascia, Open */
"0JH43NZ"  =  "1"  /* Insert Tissue Expander in Ant Neck Subcu/Fascia, Perc */
"0JH50NZ"  =  "1"  /* Insert Tissue Expander in Post Neck Subcu/Fascia, Open */
"0JH53NZ"  =  "1"  /* Insert Tissue Expander in Post Neck Subcu/Fascia, Perc */
"0JH600Z"  =  "1"  /* Insert Hemodyn Monitor Dev in Chest Subcu/Fascia, Open */
"0JH607Z"  =  "1"  /* Insert Card Rsync Pace Puls Gen in Chest Subcu/Fascia, Open */
"0JH608Z"  =  "1"  /* Insert of Defib Gen into Chest Subcu/Fascia, Open Approach */
"0JH609Z"  =  "1"  /* Insert Card Rsync Defib Puls Gen in Chest Subcu/Fascia, Open */
"0JH60AZ"  =  "1"  /* Insert Contractil Mod Dev in Chest Subcu/Fascia, Open */
"0JH60BZ"  =  "1"  /* Insert Singl Array Stim Gen in Chest Subcu/Fascia, Open */
"0JH60CZ"  =  "1"  /* Insert Singl Array Rchg Stim Gen in Chest Subcu/Fascia, Open */
"0JH60DZ"  =  "1"  /* Insert Mult Array Stim Gen in Chest Subcu/Fascia, Open */
"0JH60EZ"  =  "1"  /* Insert Mult Array Rechg Stim Gen in Chest Subcu/Fascia, Open */
"0JH60MZ"  =  "1"  /* Insertion of Stim Gen into Chest Subcu/Fascia, Open Approach */
"0JH60NZ"  =  "1"  /* Insert Tissue Expander in Chest Subcu/Fascia, Open */
"0JH60PZ"  =  "1"  /* Insert Card Rhythm Dev in Chest Subcu/Fascia, Open */
"0JH60VZ"  =  "1"  /* Insert Infusion Pump in Chest Subcu/Fascia, Open */
"0JH630Z"  =  "1"  /* Insert Hemodyn Monitor Dev in Chest Subcu/Fascia, Perc */
"0JH637Z"  =  "1"  /* Insert Card Rsync Pace Puls Gen in Chest Subcu/Fascia, Perc */
"0JH638Z"  =  "1"  /* Insert of Defib Gen into Chest Subcu/Fascia, Perc Approach */
"0JH639Z"  =  "1"  /* Insert Card Rsync Defib Puls Gen in Chest Subcu/Fascia, Perc */
"0JH63AZ"  =  "1"  /* Insert Contractil Mod Dev in Chest Subcu/Fascia, Perc */
"0JH63BZ"  =  "1"  /* Insert Singl Array Stim Gen in Chest Subcu/Fascia, Perc */
"0JH63CZ"  =  "1"  /* Insert Singl Array Rchg Stim Gen in Chest Subcu/Fascia, Perc */
"0JH63DZ"  =  "1"  /* Insert Mult Array Stim Gen in Chest Subcu/Fascia, Perc */
"0JH63EZ"  =  "1"  /* Insert Mult Array Rechg Stim Gen in Chest Subcu/Fascia, Perc */
"0JH63MZ"  =  "1"  /* Insertion of Stim Gen into Chest Subcu/Fascia, Perc Approach */
"0JH63NZ"  =  "1"  /* Insert Tissue Expander in Chest Subcu/Fascia, Perc */
"0JH63PZ"  =  "1"  /* Insert Card Rhythm Dev in Chest Subcu/Fascia, Perc */
"0JH63VZ"  =  "1"  /* Insert Infusion Pump in Chest Subcu/Fascia, Perc */
"0JH70BZ"  =  "1"  /* Insert Singl Array Stim Gen in Back Subcu/Fascia, Open */
"0JH70CZ"  =  "1"  /* Insert Singl Array Rchg Stim Gen in Back Subcu/Fascia, Open */
"0JH70DZ"  =  "1"  /* Insert Mult Array Stim Gen in Back Subcu/Fascia, Open */
"0JH70EZ"  =  "1"  /* Insert Mult Array Rechg Stim Gen in Back Subcu/Fascia, Open */
"0JH70MZ"  =  "1"  /* Insertion of Stim Gen into Back Subcu/Fascia, Open Approach */
"0JH70NZ"  =  "1"  /* Insert Tissue Expander in Back Subcu/Fascia, Open */
"0JH70VZ"  =  "1"  /* Insert Infusion Pump in Back Subcu/Fascia, Open */
"0JH73BZ"  =  "1"  /* Insert Singl Array Stim Gen in Back Subcu/Fascia, Perc */
"0JH73CZ"  =  "1"  /* Insert Singl Array Rchg Stim Gen in Back Subcu/Fascia, Perc */
"0JH73DZ"  =  "1"  /* Insert Mult Array Stim Gen in Back Subcu/Fascia, Perc */
"0JH73EZ"  =  "1"  /* Insert Mult Array Rechg Stim Gen in Back Subcu/Fascia, Perc */
"0JH73MZ"  =  "1"  /* Insertion of Stim Gen into Back Subcu/Fascia, Perc Approach */
"0JH73NZ"  =  "1"  /* Insert Tissue Expander in Back Subcu/Fascia, Perc */
"0JH73VZ"  =  "1"  /* Insert Infusion Pump in Back Subcu/Fascia, Perc */
"0JH800Z"  =  "1"  /* Insert Hemodyn Monitor Dev in Abd Subcu/Fascia, Open */
"0JH807Z"  =  "1"  /* Insert Card Rsync Pace Puls Gen in Abd Subcu/Fascia, Open */
"0JH808Z"  =  "1"  /* Insertion of Defib Gen into Abd Subcu/Fascia, Open Approach */
"0JH809Z"  =  "1"  /* Insert Card Rsync Defib Puls Gen in Abd Subcu/Fascia, Open */
"0JH80AZ"  =  "1"  /* Insert Contractil Mod Dev in Abd Subcu/Fascia, Open */
"0JH80BZ"  =  "1"  /* Insert Singl Array Stim Gen in Abd Subcu/Fascia, Open */
"0JH80CZ"  =  "1"  /* Insert Singl Array Rchg Stim Gen in Abd Subcu/Fascia, Open */
"0JH80DZ"  =  "1"  /* Insert Mult Array Stim Gen in Abd Subcu/Fascia, Open */
"0JH80EZ"  =  "1"  /* Insert Mult Array Rechg Stim Gen in Abd Subcu/Fascia, Open */
"0JH80MZ"  =  "1"  /* Insertion of Stim Gen into Abd Subcu/Fascia, Open Approach */
"0JH80NZ"  =  "1"  /* Insert Tissue Expander in Abd Subcu/Fascia, Open */
"0JH80PZ"  =  "1"  /* Insert Card Rhythm Dev in Abd Subcu/Fascia, Open */
"0JH80VZ"  =  "1"  /* Insert of Infusion Pump into Abd Subcu/Fascia, Open Approach */
"0JH830Z"  =  "1"  /* Insert Hemodyn Monitor Dev in Abd Subcu/Fascia, Perc */
"0JH837Z"  =  "1"  /* Insert Card Rsync Pace Puls Gen in Abd Subcu/Fascia, Perc */
"0JH838Z"  =  "1"  /* Insertion of Defib Gen into Abd Subcu/Fascia, Perc Approach */
"0JH839Z"  =  "1"  /* Insert Card Rsync Defib Puls Gen in Abd Subcu/Fascia, Perc */
"0JH83AZ"  =  "1"  /* Insert Contractil Mod Dev in Abd Subcu/Fascia, Perc */
"0JH83BZ"  =  "1"  /* Insert Singl Array Stim Gen in Abd Subcu/Fascia, Perc */
"0JH83CZ"  =  "1"  /* Insert Singl Array Rchg Stim Gen in Abd Subcu/Fascia, Perc */
"0JH83DZ"  =  "1"  /* Insert Mult Array Stim Gen in Abd Subcu/Fascia, Perc */
"0JH83EZ"  =  "1"  /* Insert Mult Array Rechg Stim Gen in Abd Subcu/Fascia, Perc */
"0JH83MZ"  =  "1"  /* Insertion of Stim Gen into Abd Subcu/Fascia, Perc Approach */
"0JH83NZ"  =  "1"  /* Insert Tissue Expander in Abd Subcu/Fascia, Perc */
"0JH83PZ"  =  "1"  /* Insert Card Rhythm Dev in Abd Subcu/Fascia, Perc */
"0JH83VZ"  =  "1"  /* Insert of Infusion Pump into Abd Subcu/Fascia, Perc Approach */
"0JH90NZ"  =  "1"  /* Insert Tissue Expander in Buttock Subcu/Fascia, Open */
"0JH93NZ"  =  "1"  /* Insert Tissue Expander in Buttock Subcu/Fascia, Perc */
"0JHB0NZ"  =  "1"  /* Insert Tissue Expander in Perineum Subcu/Fascia, Open */
"0JHB3NZ"  =  "1"  /* Insert Tissue Expander in Perineum Subcu/Fascia, Perc */
"0JHC0NZ"  =  "1"  /* Insert Tissue Expander in Pelvic Subcu/Fascia, Open */
"0JHC3NZ"  =  "1"  /* Insert Tissue Expander in Pelvic Subcu/Fascia, Perc */
"0JHD0NZ"  =  "1"  /* Insert Tissue Expander in R Up Arm Subcu/Fascia, Open */
"0JHD3NZ"  =  "1"  /* Insert Tissue Expander in R Up Arm Subcu/Fascia, Perc */
"0JHF0NZ"  =  "1"  /* Insert Tissue Expander in L Up Arm Subcu/Fascia, Open */
"0JHF3NZ"  =  "1"  /* Insert Tissue Expander in L Up Arm Subcu/Fascia, Perc */
"0JHG0NZ"  =  "1"  /* Insert Tissue Expander in R Low Arm Subcu/Fascia, Open */
"0JHG3NZ"  =  "1"  /* Insert Tissue Expander in R Low Arm Subcu/Fascia, Perc */
"0JHH0NZ"  =  "1"  /* Insert Tissue Expander in L Low Arm Subcu/Fascia, Open */
"0JHH3NZ"  =  "1"  /* Insert Tissue Expander in L Low Arm Subcu/Fascia, Perc */
"0JHJ0NZ"  =  "1"  /* Insert Tissue Expander in R Hand Subcu/Fascia, Open */
"0JHJ3NZ"  =  "1"  /* Insert Tissue Expander in R Hand Subcu/Fascia, Perc */
"0JHK0NZ"  =  "1"  /* Insert Tissue Expander in L Hand Subcu/Fascia, Open */
"0JHK3NZ"  =  "1"  /* Insert Tissue Expander in L Hand Subcu/Fascia, Perc */
"0JHL0NZ"  =  "1"  /* Insert Tissue Expander in R Up Leg Subcu/Fascia, Open */
"0JHL3NZ"  =  "1"  /* Insert Tissue Expander in R Up Leg Subcu/Fascia, Perc */
"0JHM0NZ"  =  "1"  /* Insert Tissue Expander in L Up Leg Subcu/Fascia, Open */
"0JHM3NZ"  =  "1"  /* Insert Tissue Expander in L Up Leg Subcu/Fascia, Perc */
"0JHN0NZ"  =  "1"  /* Insert Tissue Expander in R Low Leg Subcu/Fascia, Open */
"0JHN3NZ"  =  "1"  /* Insert Tissue Expander in R Low Leg Subcu/Fascia, Perc */
"0JHP0NZ"  =  "1"  /* Insert Tissue Expander in L Low Leg Subcu/Fascia, Open */
"0JHP3NZ"  =  "1"  /* Insert Tissue Expander in L Low Leg Subcu/Fascia, Perc */
"0JHQ0NZ"  =  "1"  /* Insert Tissue Expander in R Foot Subcu/Fascia, Open */
"0JHQ3NZ"  =  "1"  /* Insert Tissue Expander in R Foot Subcu/Fascia, Perc */
"0JHR0NZ"  =  "1"  /* Insert Tissue Expander in L Foot Subcu/Fascia, Open */
"0JHR3NZ"  =  "1"  /* Insert Tissue Expander in L Foot Subcu/Fascia, Perc */
"0JHS01Z"  =  "1"  /* Insert Radioact Elem in Head & Neck Subcu/Fascia, Open */
"0JHS31Z"  =  "1"  /* Insert Radioact Elem in Head & Neck Subcu/Fascia, Perc */
"0JHT01Z"  =  "1"  /* Insert Radioact Elem in Trunk Subcu/Fascia, Open */
"0JHT0VZ"  =  "1"  /* Insert Infusion Pump in Trunk Subcu/Fascia, Open */
"0JHT31Z"  =  "1"  /* Insert Radioact Elem in Trunk Subcu/Fascia, Perc */
"0JHT3VZ"  =  "1"  /* Insert Infusion Pump in Trunk Subcu/Fascia, Perc */
"0JHV01Z"  =  "1"  /* Insert Radioact Elem in Up Extrem Subcu/Fascia, Open */
"0JHV31Z"  =  "1"  /* Insert Radioact Elem in Up Extrem Subcu/Fascia, Perc */
"0JHW01Z"  =  "1"  /* Insert Radioact Elem in Low Extrem Subcu/Fascia, Open */
"0JHW31Z"  =  "1"  /* Insert Radioact Elem in Low Extrem Subcu/Fascia, Perc */
"0JN00ZZ"  =  "1"  /* Release Scalp Subcutaneous Tissue and Fascia, Open Approach */
"0JN03ZZ"  =  "1"  /* Release Scalp Subcutaneous Tissue and Fascia, Perc Approach */
"0JN10ZZ"  =  "1"  /* Release Face Subcutaneous Tissue and Fascia, Open Approach */
"0JN13ZZ"  =  "1"  /* Release Face Subcutaneous Tissue and Fascia, Perc Approach */
"0JN40ZZ"  =  "1"  /* Release Ant Neck Subcu/Fascia, Open Approach */
"0JN43ZZ"  =  "1"  /* Release Ant Neck Subcu/Fascia, Perc Approach */
"0JN50ZZ"  =  "1"  /* Release Post Neck Subcu/Fascia, Open Approach */
"0JN53ZZ"  =  "1"  /* Release Post Neck Subcu/Fascia, Perc Approach */
"0JN60ZZ"  =  "1"  /* Release Chest Subcutaneous Tissue and Fascia, Open Approach */
"0JN63ZZ"  =  "1"  /* Release Chest Subcutaneous Tissue and Fascia, Perc Approach */
"0JN70ZZ"  =  "1"  /* Release Back Subcutaneous Tissue and Fascia, Open Approach */
"0JN73ZZ"  =  "1"  /* Release Back Subcutaneous Tissue and Fascia, Perc Approach */
"0JN80ZZ"  =  "1"  /* Release Abd Subcu/Fascia, Open Approach */
"0JN83ZZ"  =  "1"  /* Release Abd Subcu/Fascia, Perc Approach */
"0JN90ZZ"  =  "1"  /* Release Buttock Subcu/Fascia, Open Approach */
"0JN93ZZ"  =  "1"  /* Release Buttock Subcu/Fascia, Perc Approach */
"0JNB0ZZ"  =  "1"  /* Release Perineum Subcu/Fascia, Open Approach */
"0JNB3ZZ"  =  "1"  /* Release Perineum Subcu/Fascia, Perc Approach */
"0JNC0ZZ"  =  "1"  /* Release Pelvic Subcu/Fascia, Open Approach */
"0JNC3ZZ"  =  "1"  /* Release Pelvic Subcu/Fascia, Perc Approach */
"0JND0ZZ"  =  "1"  /* Release R Up Arm Subcu/Fascia, Open Approach */
"0JND3ZZ"  =  "1"  /* Release R Up Arm Subcu/Fascia, Perc Approach */
"0JNF0ZZ"  =  "1"  /* Release L Up Arm Subcu/Fascia, Open Approach */
"0JNF3ZZ"  =  "1"  /* Release L Up Arm Subcu/Fascia, Perc Approach */
"0JNG0ZZ"  =  "1"  /* Release R Low Arm Subcu/Fascia, Open Approach */
"0JNG3ZZ"  =  "1"  /* Release R Low Arm Subcu/Fascia, Perc Approach */
"0JNH0ZZ"  =  "1"  /* Release L Low Arm Subcu/Fascia, Open Approach */
"0JNH3ZZ"  =  "1"  /* Release L Low Arm Subcu/Fascia, Perc Approach */
"0JNJ0ZZ"  =  "1"  /* Release R Hand Subcu/Fascia, Open Approach */
"0JNJ3ZZ"  =  "1"  /* Release R Hand Subcu/Fascia, Perc Approach */
"0JNK0ZZ"  =  "1"  /* Release L Hand Subcu/Fascia, Open Approach */
"0JNK3ZZ"  =  "1"  /* Release L Hand Subcu/Fascia, Perc Approach */
"0JNL0ZZ"  =  "1"  /* Release R Up Leg Subcu/Fascia, Open Approach */
"0JNL3ZZ"  =  "1"  /* Release R Up Leg Subcu/Fascia, Perc Approach */
"0JNM0ZZ"  =  "1"  /* Release L Up Leg Subcu/Fascia, Open Approach */
"0JNM3ZZ"  =  "1"  /* Release L Up Leg Subcu/Fascia, Perc Approach */
"0JNN0ZZ"  =  "1"  /* Release R Low Leg Subcu/Fascia, Open Approach */
"0JNN3ZZ"  =  "1"  /* Release R Low Leg Subcu/Fascia, Perc Approach */
"0JNP0ZZ"  =  "1"  /* Release L Low Leg Subcu/Fascia, Open Approach */
"0JNP3ZZ"  =  "1"  /* Release L Low Leg Subcu/Fascia, Perc Approach */
"0JNQ0ZZ"  =  "1"  /* Release R Foot Subcu/Fascia, Open Approach */
"0JNQ3ZZ"  =  "1"  /* Release R Foot Subcu/Fascia, Perc Approach */
"0JNR0ZZ"  =  "1"  /* Release L Foot Subcu/Fascia, Open Approach */
"0JNR3ZZ"  =  "1"  /* Release L Foot Subcu/Fascia, Perc Approach */
"0JPT0PZ"  =  "1"  /* Remove Card Rhythm Dev from Trunk Subcu/Fascia, Open */
"0JPT3PZ"  =  "1"  /* Remove Card Rhythm Dev from Trunk Subcu/Fascia, Perc */
"0JQ00ZZ"  =  "1"  /* Repair Scalp Subcutaneous Tissue and Fascia, Open Approach */
"0JQ03ZZ"  =  "1"  /* Repair Scalp Subcutaneous Tissue and Fascia, Perc Approach */
"0JQ10ZZ"  =  "1"  /* Repair Face Subcutaneous Tissue and Fascia, Open Approach */
"0JQ13ZZ"  =  "1"  /* Repair Face Subcutaneous Tissue and Fascia, Perc Approach */
"0JQ40ZZ"  =  "1"  /* Repair Ant Neck Subcu/Fascia, Open Approach */
"0JQ43ZZ"  =  "1"  /* Repair Ant Neck Subcu/Fascia, Perc Approach */
"0JQ50ZZ"  =  "1"  /* Repair Post Neck Subcu/Fascia, Open Approach */
"0JQ53ZZ"  =  "1"  /* Repair Post Neck Subcu/Fascia, Perc Approach */
"0JQ60ZZ"  =  "1"  /* Repair Chest Subcutaneous Tissue and Fascia, Open Approach */
"0JQ63ZZ"  =  "1"  /* Repair Chest Subcutaneous Tissue and Fascia, Perc Approach */
"0JQ70ZZ"  =  "1"  /* Repair Back Subcutaneous Tissue and Fascia, Open Approach */
"0JQ73ZZ"  =  "1"  /* Repair Back Subcutaneous Tissue and Fascia, Perc Approach */
"0JQ80ZZ"  =  "1"  /* Repair Abdomen Subcutaneous Tissue and Fascia, Open Approach */
"0JQ83ZZ"  =  "1"  /* Repair Abdomen Subcutaneous Tissue and Fascia, Perc Approach */
"0JQ90ZZ"  =  "1"  /* Repair Buttock Subcutaneous Tissue and Fascia, Open Approach */
"0JQ93ZZ"  =  "1"  /* Repair Buttock Subcutaneous Tissue and Fascia, Perc Approach */
"0JQB0ZZ"  =  "1"  /* Repair Perineum Subcu/Fascia, Open Approach */
"0JQB3ZZ"  =  "1"  /* Repair Perineum Subcu/Fascia, Perc Approach */
"0JQC0ZZ"  =  "1"  /* Repair Pelvic Subcu/Fascia, Open Approach */
"0JQC3ZZ"  =  "1"  /* Repair Pelvic Subcu/Fascia, Perc Approach */
"0JQD0ZZ"  =  "1"  /* Repair R Up Arm Subcu/Fascia, Open Approach */
"0JQD3ZZ"  =  "1"  /* Repair R Up Arm Subcu/Fascia, Perc Approach */
"0JQF0ZZ"  =  "1"  /* Repair L Up Arm Subcu/Fascia, Open Approach */
"0JQF3ZZ"  =  "1"  /* Repair L Up Arm Subcu/Fascia, Perc Approach */
"0JQG0ZZ"  =  "1"  /* Repair R Low Arm Subcu/Fascia, Open Approach */
"0JQG3ZZ"  =  "1"  /* Repair R Low Arm Subcu/Fascia, Perc Approach */
"0JQH0ZZ"  =  "1"  /* Repair L Low Arm Subcu/Fascia, Open Approach */
"0JQH3ZZ"  =  "1"  /* Repair L Low Arm Subcu/Fascia, Perc Approach */
"0JQJ0ZZ"  =  "1"  /* Repair R Hand Subcu/Fascia, Open Approach */
"0JQJ3ZZ"  =  "1"  /* Repair R Hand Subcu/Fascia, Perc Approach */
"0JQK0ZZ"  =  "1"  /* Repair L Hand Subcu/Fascia, Open Approach */
"0JQK3ZZ"  =  "1"  /* Repair L Hand Subcu/Fascia, Perc Approach */
"0JQL0ZZ"  =  "1"  /* Repair R Up Leg Subcu/Fascia, Open Approach */
"0JQL3ZZ"  =  "1"  /* Repair R Up Leg Subcu/Fascia, Perc Approach */
"0JQM0ZZ"  =  "1"  /* Repair L Up Leg Subcu/Fascia, Open Approach */
"0JQM3ZZ"  =  "1"  /* Repair L Up Leg Subcu/Fascia, Perc Approach */
"0JQN0ZZ"  =  "1"  /* Repair R Low Leg Subcu/Fascia, Open Approach */
"0JQN3ZZ"  =  "1"  /* Repair R Low Leg Subcu/Fascia, Perc Approach */
"0JQP0ZZ"  =  "1"  /* Repair L Low Leg Subcu/Fascia, Open Approach */
"0JQP3ZZ"  =  "1"  /* Repair L Low Leg Subcu/Fascia, Perc Approach */
"0JQQ0ZZ"  =  "1"  /* Repair R Foot Subcu/Fascia, Open Approach */
"0JQQ3ZZ"  =  "1"  /* Repair R Foot Subcu/Fascia, Perc Approach */
"0JQR0ZZ"  =  "1"  /* Repair L Foot Subcu/Fascia, Open Approach */
"0JQR3ZZ"  =  "1"  /* Repair L Foot Subcu/Fascia, Perc Approach */
"0JR007Z"  =  "1"  /* Replace of Scalp Subcu/Fascia with Autol Sub, Open Approach */
"0JR00JZ"  =  "1"  /* Replace of Scalp Subcu/Fascia with Synth Sub, Open Approach */
"0JR00KZ"  =  "1"  /* Replace of Scalp Subcu/Fascia with Nonaut Sub, Open Approach */
"0JR037Z"  =  "1"  /* Replace of Scalp Subcu/Fascia with Autol Sub, Perc Approach */
"0JR03JZ"  =  "1"  /* Replace of Scalp Subcu/Fascia with Synth Sub, Perc Approach */
"0JR03KZ"  =  "1"  /* Replace of Scalp Subcu/Fascia with Nonaut Sub, Perc Approach */
"0JR107Z"  =  "1"  /* Replace of Face Subcu/Fascia with Autol Sub, Open Approach */
"0JR10JZ"  =  "1"  /* Replace of Face Subcu/Fascia with Synth Sub, Open Approach */
"0JR10KZ"  =  "1"  /* Replace of Face Subcu/Fascia with Nonaut Sub, Open Approach */
"0JR137Z"  =  "1"  /* Replace of Face Subcu/Fascia with Autol Sub, Perc Approach */
"0JR13JZ"  =  "1"  /* Replace of Face Subcu/Fascia with Synth Sub, Perc Approach */
"0JR13KZ"  =  "1"  /* Replace of Face Subcu/Fascia with Nonaut Sub, Perc Approach */
"0JR407Z"  =  "1"  /* Replace Ant Neck Subcu/Fascia w Autol Sub, Open */
"0JR40JZ"  =  "1"  /* Replace Ant Neck Subcu/Fascia w Synth Sub, Open */
"0JR40KZ"  =  "1"  /* Replace Ant Neck Subcu/Fascia w Nonaut Sub, Open */
"0JR437Z"  =  "1"  /* Replace Ant Neck Subcu/Fascia w Autol Sub, Perc */
"0JR43JZ"  =  "1"  /* Replace Ant Neck Subcu/Fascia w Synth Sub, Perc */
"0JR43KZ"  =  "1"  /* Replace Ant Neck Subcu/Fascia w Nonaut Sub, Perc */
"0JR507Z"  =  "1"  /* Replace Post Neck Subcu/Fascia w Autol Sub, Open */
"0JR50JZ"  =  "1"  /* Replace Post Neck Subcu/Fascia w Synth Sub, Open */
"0JR50KZ"  =  "1"  /* Replace Post Neck Subcu/Fascia w Nonaut Sub, Open */
"0JR537Z"  =  "1"  /* Replace Post Neck Subcu/Fascia w Autol Sub, Perc */
"0JR53JZ"  =  "1"  /* Replace Post Neck Subcu/Fascia w Synth Sub, Perc */
"0JR53KZ"  =  "1"  /* Replace Post Neck Subcu/Fascia w Nonaut Sub, Perc */
"0JR607Z"  =  "1"  /* Replace of Chest Subcu/Fascia with Autol Sub, Open Approach */
"0JR60JZ"  =  "1"  /* Replace of Chest Subcu/Fascia with Synth Sub, Open Approach */
"0JR60KZ"  =  "1"  /* Replace of Chest Subcu/Fascia with Nonaut Sub, Open Approach */
"0JR637Z"  =  "1"  /* Replace of Chest Subcu/Fascia with Autol Sub, Perc Approach */
"0JR63JZ"  =  "1"  /* Replace of Chest Subcu/Fascia with Synth Sub, Perc Approach */
"0JR63KZ"  =  "1"  /* Replace of Chest Subcu/Fascia with Nonaut Sub, Perc Approach */
"0JR707Z"  =  "1"  /* Replace of Back Subcu/Fascia with Autol Sub, Open Approach */
"0JR70JZ"  =  "1"  /* Replace of Back Subcu/Fascia with Synth Sub, Open Approach */
"0JR70KZ"  =  "1"  /* Replace of Back Subcu/Fascia with Nonaut Sub, Open Approach */
"0JR737Z"  =  "1"  /* Replace of Back Subcu/Fascia with Autol Sub, Perc Approach */
"0JR73JZ"  =  "1"  /* Replace of Back Subcu/Fascia with Synth Sub, Perc Approach */
"0JR73KZ"  =  "1"  /* Replace of Back Subcu/Fascia with Nonaut Sub, Perc Approach */
"0JR807Z"  =  "1"  /* Replace of Abd Subcu/Fascia with Autol Sub, Open Approach */
"0JR80JZ"  =  "1"  /* Replace of Abd Subcu/Fascia with Synth Sub, Open Approach */
"0JR80KZ"  =  "1"  /* Replace of Abd Subcu/Fascia with Nonaut Sub, Open Approach */
"0JR837Z"  =  "1"  /* Replace of Abd Subcu/Fascia with Autol Sub, Perc Approach */
"0JR83JZ"  =  "1"  /* Replace of Abd Subcu/Fascia with Synth Sub, Perc Approach */
"0JR83KZ"  =  "1"  /* Replace of Abd Subcu/Fascia with Nonaut Sub, Perc Approach */
"0JR907Z"  =  "1"  /* Replace Buttock Subcu/Fascia w Autol Sub, Open */
"0JR90JZ"  =  "1"  /* Replace Buttock Subcu/Fascia w Synth Sub, Open */
"0JR90KZ"  =  "1"  /* Replace Buttock Subcu/Fascia w Nonaut Sub, Open */
"0JR937Z"  =  "1"  /* Replace Buttock Subcu/Fascia w Autol Sub, Perc */
"0JR93JZ"  =  "1"  /* Replace Buttock Subcu/Fascia w Synth Sub, Perc */
"0JR93KZ"  =  "1"  /* Replace Buttock Subcu/Fascia w Nonaut Sub, Perc */
"0JRB07Z"  =  "1"  /* Replace Perineum Subcu/Fascia w Autol Sub, Open */
"0JRB0JZ"  =  "1"  /* Replace Perineum Subcu/Fascia w Synth Sub, Open */
"0JRB0KZ"  =  "1"  /* Replace Perineum Subcu/Fascia w Nonaut Sub, Open */
"0JRB37Z"  =  "1"  /* Replace Perineum Subcu/Fascia w Autol Sub, Perc */
"0JRB3JZ"  =  "1"  /* Replace Perineum Subcu/Fascia w Synth Sub, Perc */
"0JRB3KZ"  =  "1"  /* Replace Perineum Subcu/Fascia w Nonaut Sub, Perc */
"0JRC07Z"  =  "1"  /* Replace of Pelvic Subcu/Fascia with Autol Sub, Open Approach */
"0JRC0JZ"  =  "1"  /* Replace of Pelvic Subcu/Fascia with Synth Sub, Open Approach */
"0JRC0KZ"  =  "1"  /* Replace Pelvic Subcu/Fascia w Nonaut Sub, Open */
"0JRC37Z"  =  "1"  /* Replace of Pelvic Subcu/Fascia with Autol Sub, Perc Approach */
"0JRC3JZ"  =  "1"  /* Replace of Pelvic Subcu/Fascia with Synth Sub, Perc Approach */
"0JRC3KZ"  =  "1"  /* Replace Pelvic Subcu/Fascia w Nonaut Sub, Perc */
"0JRD07Z"  =  "1"  /* Replace R Up Arm Subcu/Fascia w Autol Sub, Open */
"0JRD0JZ"  =  "1"  /* Replace R Up Arm Subcu/Fascia w Synth Sub, Open */
"0JRD0KZ"  =  "1"  /* Replace R Up Arm Subcu/Fascia w Nonaut Sub, Open */
"0JRD37Z"  =  "1"  /* Replace R Up Arm Subcu/Fascia w Autol Sub, Perc */
"0JRD3JZ"  =  "1"  /* Replace R Up Arm Subcu/Fascia w Synth Sub, Perc */
"0JRD3KZ"  =  "1"  /* Replace R Up Arm Subcu/Fascia w Nonaut Sub, Perc */
"0JRF07Z"  =  "1"  /* Replace L Up Arm Subcu/Fascia w Autol Sub, Open */
"0JRF0JZ"  =  "1"  /* Replace L Up Arm Subcu/Fascia w Synth Sub, Open */
"0JRF0KZ"  =  "1"  /* Replace L Up Arm Subcu/Fascia w Nonaut Sub, Open */
"0JRF37Z"  =  "1"  /* Replace L Up Arm Subcu/Fascia w Autol Sub, Perc */
"0JRF3JZ"  =  "1"  /* Replace L Up Arm Subcu/Fascia w Synth Sub, Perc */
"0JRF3KZ"  =  "1"  /* Replace L Up Arm Subcu/Fascia w Nonaut Sub, Perc */
"0JRG07Z"  =  "1"  /* Replace R Low Arm Subcu/Fascia w Autol Sub, Open */
"0JRG0JZ"  =  "1"  /* Replace R Low Arm Subcu/Fascia w Synth Sub, Open */
"0JRG0KZ"  =  "1"  /* Replace R Low Arm Subcu/Fascia w Nonaut Sub, Open */
"0JRG37Z"  =  "1"  /* Replace R Low Arm Subcu/Fascia w Autol Sub, Perc */
"0JRG3JZ"  =  "1"  /* Replace R Low Arm Subcu/Fascia w Synth Sub, Perc */
"0JRG3KZ"  =  "1"  /* Replace R Low Arm Subcu/Fascia w Nonaut Sub, Perc */
"0JRH07Z"  =  "1"  /* Replace L Low Arm Subcu/Fascia w Autol Sub, Open */
"0JRH0JZ"  =  "1"  /* Replace L Low Arm Subcu/Fascia w Synth Sub, Open */
"0JRH0KZ"  =  "1"  /* Replace L Low Arm Subcu/Fascia w Nonaut Sub, Open */
"0JRH37Z"  =  "1"  /* Replace L Low Arm Subcu/Fascia w Autol Sub, Perc */
"0JRH3JZ"  =  "1"  /* Replace L Low Arm Subcu/Fascia w Synth Sub, Perc */
"0JRH3KZ"  =  "1"  /* Replace L Low Arm Subcu/Fascia w Nonaut Sub, Perc */
"0JRJ07Z"  =  "1"  /* Replace of R Hand Subcu/Fascia with Autol Sub, Open Approach */
"0JRJ0JZ"  =  "1"  /* Replace of R Hand Subcu/Fascia with Synth Sub, Open Approach */
"0JRJ0KZ"  =  "1"  /* Replace R Hand Subcu/Fascia w Nonaut Sub, Open */
"0JRJ37Z"  =  "1"  /* Replace of R Hand Subcu/Fascia with Autol Sub, Perc Approach */
"0JRJ3JZ"  =  "1"  /* Replace of R Hand Subcu/Fascia with Synth Sub, Perc Approach */
"0JRJ3KZ"  =  "1"  /* Replace R Hand Subcu/Fascia w Nonaut Sub, Perc */
"0JRK07Z"  =  "1"  /* Replace of L Hand Subcu/Fascia with Autol Sub, Open Approach */
"0JRK0JZ"  =  "1"  /* Replace of L Hand Subcu/Fascia with Synth Sub, Open Approach */
"0JRK0KZ"  =  "1"  /* Replace L Hand Subcu/Fascia w Nonaut Sub, Open */
"0JRK37Z"  =  "1"  /* Replace of L Hand Subcu/Fascia with Autol Sub, Perc Approach */
"0JRK3JZ"  =  "1"  /* Replace of L Hand Subcu/Fascia with Synth Sub, Perc Approach */
"0JRK3KZ"  =  "1"  /* Replace L Hand Subcu/Fascia w Nonaut Sub, Perc */
"0JRL07Z"  =  "1"  /* Replace R Up Leg Subcu/Fascia w Autol Sub, Open */
"0JRL0JZ"  =  "1"  /* Replace R Up Leg Subcu/Fascia w Synth Sub, Open */
"0JRL0KZ"  =  "1"  /* Replace R Up Leg Subcu/Fascia w Nonaut Sub, Open */
"0JRL37Z"  =  "1"  /* Replace R Up Leg Subcu/Fascia w Autol Sub, Perc */
"0JRL3JZ"  =  "1"  /* Replace R Up Leg Subcu/Fascia w Synth Sub, Perc */
"0JRL3KZ"  =  "1"  /* Replace R Up Leg Subcu/Fascia w Nonaut Sub, Perc */
"0JRM07Z"  =  "1"  /* Replace L Up Leg Subcu/Fascia w Autol Sub, Open */
"0JRM0JZ"  =  "1"  /* Replace L Up Leg Subcu/Fascia w Synth Sub, Open */
"0JRM0KZ"  =  "1"  /* Replace L Up Leg Subcu/Fascia w Nonaut Sub, Open */
"0JRM37Z"  =  "1"  /* Replace L Up Leg Subcu/Fascia w Autol Sub, Perc */
"0JRM3JZ"  =  "1"  /* Replace L Up Leg Subcu/Fascia w Synth Sub, Perc */
"0JRM3KZ"  =  "1"  /* Replace L Up Leg Subcu/Fascia w Nonaut Sub, Perc */
"0JRN07Z"  =  "1"  /* Replace R Low Leg Subcu/Fascia w Autol Sub, Open */
"0JRN0JZ"  =  "1"  /* Replace R Low Leg Subcu/Fascia w Synth Sub, Open */
"0JRN0KZ"  =  "1"  /* Replace R Low Leg Subcu/Fascia w Nonaut Sub, Open */
"0JRN37Z"  =  "1"  /* Replace R Low Leg Subcu/Fascia w Autol Sub, Perc */
"0JRN3JZ"  =  "1"  /* Replace R Low Leg Subcu/Fascia w Synth Sub, Perc */
"0JRN3KZ"  =  "1"  /* Replace R Low Leg Subcu/Fascia w Nonaut Sub, Perc */
"0JRP07Z"  =  "1"  /* Replace L Low Leg Subcu/Fascia w Autol Sub, Open */
"0JRP0JZ"  =  "1"  /* Replace L Low Leg Subcu/Fascia w Synth Sub, Open */
"0JRP0KZ"  =  "1"  /* Replace L Low Leg Subcu/Fascia w Nonaut Sub, Open */
"0JRP37Z"  =  "1"  /* Replace L Low Leg Subcu/Fascia w Autol Sub, Perc */
"0JRP3JZ"  =  "1"  /* Replace L Low Leg Subcu/Fascia w Synth Sub, Perc */
"0JRP3KZ"  =  "1"  /* Replace L Low Leg Subcu/Fascia w Nonaut Sub, Perc */
"0JRQ07Z"  =  "1"  /* Replace of R Foot Subcu/Fascia with Autol Sub, Open Approach */
"0JRQ0JZ"  =  "1"  /* Replace of R Foot Subcu/Fascia with Synth Sub, Open Approach */
"0JRQ0KZ"  =  "1"  /* Replace R Foot Subcu/Fascia w Nonaut Sub, Open */
"0JRQ37Z"  =  "1"  /* Replace of R Foot Subcu/Fascia with Autol Sub, Perc Approach */
"0JRQ3JZ"  =  "1"  /* Replace of R Foot Subcu/Fascia with Synth Sub, Perc Approach */
"0JRQ3KZ"  =  "1"  /* Replace R Foot Subcu/Fascia w Nonaut Sub, Perc */
"0JRR07Z"  =  "1"  /* Replace of L Foot Subcu/Fascia with Autol Sub, Open Approach */
"0JRR0JZ"  =  "1"  /* Replace of L Foot Subcu/Fascia with Synth Sub, Open Approach */
"0JRR0KZ"  =  "1"  /* Replace L Foot Subcu/Fascia w Nonaut Sub, Open */
"0JRR37Z"  =  "1"  /* Replace of L Foot Subcu/Fascia with Autol Sub, Perc Approach */
"0JRR3JZ"  =  "1"  /* Replace of L Foot Subcu/Fascia with Synth Sub, Perc Approach */
"0JRR3KZ"  =  "1"  /* Replace L Foot Subcu/Fascia w Nonaut Sub, Perc */
"0JU007Z"  =  "1"  /* Supplement Scalp Subcu/Fascia w Autol Sub, Open */
"0JU00JZ"  =  "1"  /* Supplement Scalp Subcu/Fascia w Synth Sub, Open */
"0JU00KZ"  =  "1"  /* Supplement Scalp Subcu/Fascia w Nonaut Sub, Open */
"0JU037Z"  =  "1"  /* Supplement Scalp Subcu/Fascia w Autol Sub, Perc */
"0JU03JZ"  =  "1"  /* Supplement Scalp Subcu/Fascia w Synth Sub, Perc */
"0JU03KZ"  =  "1"  /* Supplement Scalp Subcu/Fascia w Nonaut Sub, Perc */
"0JU107Z"  =  "1"  /* Supplement Face Subcu/Fascia w Autol Sub, Open */
"0JU10JZ"  =  "1"  /* Supplement Face Subcu/Fascia w Synth Sub, Open */
"0JU10KZ"  =  "1"  /* Supplement Face Subcu/Fascia w Nonaut Sub, Open */
"0JU137Z"  =  "1"  /* Supplement Face Subcu/Fascia w Autol Sub, Perc */
"0JU13JZ"  =  "1"  /* Supplement Face Subcu/Fascia w Synth Sub, Perc */
"0JU13KZ"  =  "1"  /* Supplement Face Subcu/Fascia w Nonaut Sub, Perc */
"0JU407Z"  =  "1"  /* Supplement Ant Neck Subcu/Fascia w Autol Sub, Open */
"0JU40JZ"  =  "1"  /* Supplement Ant Neck Subcu/Fascia w Synth Sub, Open */
"0JU40KZ"  =  "1"  /* Supplement Ant Neck Subcu/Fascia w Nonaut Sub, Open */
"0JU437Z"  =  "1"  /* Supplement Ant Neck Subcu/Fascia w Autol Sub, Perc */
"0JU43JZ"  =  "1"  /* Supplement Ant Neck Subcu/Fascia w Synth Sub, Perc */
"0JU43KZ"  =  "1"  /* Supplement Ant Neck Subcu/Fascia w Nonaut Sub, Perc */
"0JU507Z"  =  "1"  /* Supplement Post Neck Subcu/Fascia w Autol Sub, Open */
"0JU50JZ"  =  "1"  /* Supplement Post Neck Subcu/Fascia w Synth Sub, Open */
"0JU50KZ"  =  "1"  /* Supplement Post Neck Subcu/Fascia w Nonaut Sub, Open */
"0JU537Z"  =  "1"  /* Supplement Post Neck Subcu/Fascia w Autol Sub, Perc */
"0JU53JZ"  =  "1"  /* Supplement Post Neck Subcu/Fascia w Synth Sub, Perc */
"0JU53KZ"  =  "1"  /* Supplement Post Neck Subcu/Fascia w Nonaut Sub, Perc */
"0JU607Z"  =  "1"  /* Supplement Chest Subcu/Fascia w Autol Sub, Open */
"0JU60JZ"  =  "1"  /* Supplement Chest Subcu/Fascia w Synth Sub, Open */
"0JU60KZ"  =  "1"  /* Supplement Chest Subcu/Fascia w Nonaut Sub, Open */
"0JU637Z"  =  "1"  /* Supplement Chest Subcu/Fascia w Autol Sub, Perc */
"0JU63JZ"  =  "1"  /* Supplement Chest Subcu/Fascia w Synth Sub, Perc */
"0JU63KZ"  =  "1"  /* Supplement Chest Subcu/Fascia w Nonaut Sub, Perc */
"0JU707Z"  =  "1"  /* Supplement Back Subcu/Fascia w Autol Sub, Open */
"0JU70JZ"  =  "1"  /* Supplement Back Subcu/Fascia w Synth Sub, Open */
"0JU70KZ"  =  "1"  /* Supplement Back Subcu/Fascia w Nonaut Sub, Open */
"0JU737Z"  =  "1"  /* Supplement Back Subcu/Fascia w Autol Sub, Perc */
"0JU73JZ"  =  "1"  /* Supplement Back Subcu/Fascia w Synth Sub, Perc */
"0JU73KZ"  =  "1"  /* Supplement Back Subcu/Fascia w Nonaut Sub, Perc */
"0JU807Z"  =  "1"  /* Supplement of Abd Subcu/Fascia with Autol Sub, Open Approach */
"0JU80JZ"  =  "1"  /* Supplement of Abd Subcu/Fascia with Synth Sub, Open Approach */
"0JU80KZ"  =  "1"  /* Supplement Abd Subcu/Fascia w Nonaut Sub, Open */
"0JU837Z"  =  "1"  /* Supplement of Abd Subcu/Fascia with Autol Sub, Perc Approach */
"0JU83JZ"  =  "1"  /* Supplement of Abd Subcu/Fascia with Synth Sub, Perc Approach */
"0JU83KZ"  =  "1"  /* Supplement Abd Subcu/Fascia w Nonaut Sub, Perc */
"0JU907Z"  =  "1"  /* Supplement Buttock Subcu/Fascia w Autol Sub, Open */
"0JU90JZ"  =  "1"  /* Supplement Buttock Subcu/Fascia w Synth Sub, Open */
"0JU90KZ"  =  "1"  /* Supplement Buttock Subcu/Fascia w Nonaut Sub, Open */
"0JU937Z"  =  "1"  /* Supplement Buttock Subcu/Fascia w Autol Sub, Perc */
"0JU93JZ"  =  "1"  /* Supplement Buttock Subcu/Fascia w Synth Sub, Perc */
"0JU93KZ"  =  "1"  /* Supplement Buttock Subcu/Fascia w Nonaut Sub, Perc */
"0JUB07Z"  =  "1"  /* Supplement Perineum Subcu/Fascia w Autol Sub, Open */
"0JUB0JZ"  =  "1"  /* Supplement Perineum Subcu/Fascia w Synth Sub, Open */
"0JUB0KZ"  =  "1"  /* Supplement Perineum Subcu/Fascia w Nonaut Sub, Open */
"0JUB37Z"  =  "1"  /* Supplement Perineum Subcu/Fascia w Autol Sub, Perc */
"0JUB3JZ"  =  "1"  /* Supplement Perineum Subcu/Fascia w Synth Sub, Perc */
"0JUB3KZ"  =  "1"  /* Supplement Perineum Subcu/Fascia w Nonaut Sub, Perc */
"0JUC07Z"  =  "1"  /* Supplement Pelvic Subcu/Fascia w Autol Sub, Open */
"0JUC0JZ"  =  "1"  /* Supplement Pelvic Subcu/Fascia w Synth Sub, Open */
"0JUC0KZ"  =  "1"  /* Supplement Pelvic Subcu/Fascia w Nonaut Sub, Open */
"0JUC37Z"  =  "1"  /* Supplement Pelvic Subcu/Fascia w Autol Sub, Perc */
"0JUC3JZ"  =  "1"  /* Supplement Pelvic Subcu/Fascia w Synth Sub, Perc */
"0JUC3KZ"  =  "1"  /* Supplement Pelvic Subcu/Fascia w Nonaut Sub, Perc */
"0JUD07Z"  =  "1"  /* Supplement R Up Arm Subcu/Fascia w Autol Sub, Open */
"0JUD0JZ"  =  "1"  /* Supplement R Up Arm Subcu/Fascia w Synth Sub, Open */
"0JUD0KZ"  =  "1"  /* Supplement R Up Arm Subcu/Fascia w Nonaut Sub, Open */
"0JUD37Z"  =  "1"  /* Supplement R Up Arm Subcu/Fascia w Autol Sub, Perc */
"0JUD3JZ"  =  "1"  /* Supplement R Up Arm Subcu/Fascia w Synth Sub, Perc */
"0JUD3KZ"  =  "1"  /* Supplement R Up Arm Subcu/Fascia w Nonaut Sub, Perc */
"0JUF07Z"  =  "1"  /* Supplement L Up Arm Subcu/Fascia w Autol Sub, Open */
"0JUF0JZ"  =  "1"  /* Supplement L Up Arm Subcu/Fascia w Synth Sub, Open */
"0JUF0KZ"  =  "1"  /* Supplement L Up Arm Subcu/Fascia w Nonaut Sub, Open */
"0JUF37Z"  =  "1"  /* Supplement L Up Arm Subcu/Fascia w Autol Sub, Perc */
"0JUF3JZ"  =  "1"  /* Supplement L Up Arm Subcu/Fascia w Synth Sub, Perc */
"0JUF3KZ"  =  "1"  /* Supplement L Up Arm Subcu/Fascia w Nonaut Sub, Perc */
"0JUG07Z"  =  "1"  /* Supplement R Low Arm Subcu/Fascia w Autol Sub, Open */
"0JUG0JZ"  =  "1"  /* Supplement R Low Arm Subcu/Fascia w Synth Sub, Open */
"0JUG0KZ"  =  "1"  /* Supplement R Low Arm Subcu/Fascia w Nonaut Sub, Open */
"0JUG37Z"  =  "1"  /* Supplement R Low Arm Subcu/Fascia w Autol Sub, Perc */
"0JUG3JZ"  =  "1"  /* Supplement R Low Arm Subcu/Fascia w Synth Sub, Perc */
"0JUG3KZ"  =  "1"  /* Supplement R Low Arm Subcu/Fascia w Nonaut Sub, Perc */
"0JUH07Z"  =  "1"  /* Supplement L Low Arm Subcu/Fascia w Autol Sub, Open */
"0JUH0JZ"  =  "1"  /* Supplement L Low Arm Subcu/Fascia w Synth Sub, Open */
"0JUH0KZ"  =  "1"  /* Supplement L Low Arm Subcu/Fascia w Nonaut Sub, Open */
"0JUH37Z"  =  "1"  /* Supplement L Low Arm Subcu/Fascia w Autol Sub, Perc */
"0JUH3JZ"  =  "1"  /* Supplement L Low Arm Subcu/Fascia w Synth Sub, Perc */
"0JUH3KZ"  =  "1"  /* Supplement L Low Arm Subcu/Fascia w Nonaut Sub, Perc */
"0JUJ07Z"  =  "1"  /* Supplement R Hand Subcu/Fascia w Autol Sub, Open */
"0JUJ0JZ"  =  "1"  /* Supplement R Hand Subcu/Fascia w Synth Sub, Open */
"0JUJ0KZ"  =  "1"  /* Supplement R Hand Subcu/Fascia w Nonaut Sub, Open */
"0JUJ37Z"  =  "1"  /* Supplement R Hand Subcu/Fascia w Autol Sub, Perc */
"0JUJ3JZ"  =  "1"  /* Supplement R Hand Subcu/Fascia w Synth Sub, Perc */
"0JUJ3KZ"  =  "1"  /* Supplement R Hand Subcu/Fascia w Nonaut Sub, Perc */
"0JUK07Z"  =  "1"  /* Supplement L Hand Subcu/Fascia w Autol Sub, Open */
"0JUK0JZ"  =  "1"  /* Supplement L Hand Subcu/Fascia w Synth Sub, Open */
"0JUK0KZ"  =  "1"  /* Supplement L Hand Subcu/Fascia w Nonaut Sub, Open */
"0JUK37Z"  =  "1"  /* Supplement L Hand Subcu/Fascia w Autol Sub, Perc */
"0JUK3JZ"  =  "1"  /* Supplement L Hand Subcu/Fascia w Synth Sub, Perc */
"0JUK3KZ"  =  "1"  /* Supplement L Hand Subcu/Fascia w Nonaut Sub, Perc */
"0JUL07Z"  =  "1"  /* Supplement R Up Leg Subcu/Fascia w Autol Sub, Open */
"0JUL0JZ"  =  "1"  /* Supplement R Up Leg Subcu/Fascia w Synth Sub, Open */
"0JUL0KZ"  =  "1"  /* Supplement R Up Leg Subcu/Fascia w Nonaut Sub, Open */
"0JUL37Z"  =  "1"  /* Supplement R Up Leg Subcu/Fascia w Autol Sub, Perc */
"0JUL3JZ"  =  "1"  /* Supplement R Up Leg Subcu/Fascia w Synth Sub, Perc */
"0JUL3KZ"  =  "1"  /* Supplement R Up Leg Subcu/Fascia w Nonaut Sub, Perc */
"0JUM07Z"  =  "1"  /* Supplement L Up Leg Subcu/Fascia w Autol Sub, Open */
"0JUM0JZ"  =  "1"  /* Supplement L Up Leg Subcu/Fascia w Synth Sub, Open */
"0JUM0KZ"  =  "1"  /* Supplement L Up Leg Subcu/Fascia w Nonaut Sub, Open */
"0JUM37Z"  =  "1"  /* Supplement L Up Leg Subcu/Fascia w Autol Sub, Perc */
"0JUM3JZ"  =  "1"  /* Supplement L Up Leg Subcu/Fascia w Synth Sub, Perc */
"0JUM3KZ"  =  "1"  /* Supplement L Up Leg Subcu/Fascia w Nonaut Sub, Perc */
"0JUN07Z"  =  "1"  /* Supplement R Low Leg Subcu/Fascia w Autol Sub, Open */
"0JUN0JZ"  =  "1"  /* Supplement R Low Leg Subcu/Fascia w Synth Sub, Open */
"0JUN0KZ"  =  "1"  /* Supplement R Low Leg Subcu/Fascia w Nonaut Sub, Open */
"0JUN37Z"  =  "1"  /* Supplement R Low Leg Subcu/Fascia w Autol Sub, Perc */
"0JUN3JZ"  =  "1"  /* Supplement R Low Leg Subcu/Fascia w Synth Sub, Perc */
"0JUN3KZ"  =  "1"  /* Supplement R Low Leg Subcu/Fascia w Nonaut Sub, Perc */
"0JUP07Z"  =  "1"  /* Supplement L Low Leg Subcu/Fascia w Autol Sub, Open */
"0JUP0JZ"  =  "1"  /* Supplement L Low Leg Subcu/Fascia w Synth Sub, Open */
"0JUP0KZ"  =  "1"  /* Supplement L Low Leg Subcu/Fascia w Nonaut Sub, Open */
"0JUP37Z"  =  "1"  /* Supplement L Low Leg Subcu/Fascia w Autol Sub, Perc */
"0JUP3JZ"  =  "1"  /* Supplement L Low Leg Subcu/Fascia w Synth Sub, Perc */
"0JUP3KZ"  =  "1"  /* Supplement L Low Leg Subcu/Fascia w Nonaut Sub, Perc */
"0JUQ07Z"  =  "1"  /* Supplement R Foot Subcu/Fascia w Autol Sub, Open */
"0JUQ0JZ"  =  "1"  /* Supplement R Foot Subcu/Fascia w Synth Sub, Open */
"0JUQ0KZ"  =  "1"  /* Supplement R Foot Subcu/Fascia w Nonaut Sub, Open */
"0JUQ37Z"  =  "1"  /* Supplement R Foot Subcu/Fascia w Autol Sub, Perc */
"0JUQ3JZ"  =  "1"  /* Supplement R Foot Subcu/Fascia w Synth Sub, Perc */
"0JUQ3KZ"  =  "1"  /* Supplement R Foot Subcu/Fascia w Nonaut Sub, Perc */
"0JUR07Z"  =  "1"  /* Supplement L Foot Subcu/Fascia w Autol Sub, Open */
"0JUR0JZ"  =  "1"  /* Supplement L Foot Subcu/Fascia w Synth Sub, Open */
"0JUR0KZ"  =  "1"  /* Supplement L Foot Subcu/Fascia w Nonaut Sub, Open */
"0JUR37Z"  =  "1"  /* Supplement L Foot Subcu/Fascia w Autol Sub, Perc */
"0JUR3JZ"  =  "1"  /* Supplement L Foot Subcu/Fascia w Synth Sub, Perc */
"0JUR3KZ"  =  "1"  /* Supplement L Foot Subcu/Fascia w Nonaut Sub, Perc */
"0JWT0MZ"  =  "1"  /* Revision of Stim Gen in Trunk Subcu/Fascia, Open Approach */
"0JWT0PZ"  =  "1"  /* Revise Card Rhythm Dev in Trunk Subcu/Fascia, Open */
"0JWT3MZ"  =  "1"  /* Revision of Stim Gen in Trunk Subcu/Fascia, Perc Approach */
"0JWT3PZ"  =  "1"  /* Revise Card Rhythm Dev in Trunk Subcu/Fascia, Perc */
"0JWTXMZ"  =  "1"  /* Revision of Stim Gen in Trunk Subcu/Fascia, Extern Approach */
"0JX00ZB"  =  "1"  /* Transfer Scalp Subcu/Fascia with Skin, Subcu, Open Approach */
"0JX00ZC"  =  "1"  /* Transfer Scalp Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JX00ZZ"  =  "1"  /* Transfer Scalp Subcutaneous Tissue and Fascia, Open Approach */
"0JX03ZB"  =  "1"  /* Transfer Scalp Subcu/Fascia with Skin, Subcu, Perc Approach */
"0JX03ZC"  =  "1"  /* Transfer Scalp Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JX03ZZ"  =  "1"  /* Transfer Scalp Subcutaneous Tissue and Fascia, Perc Approach */
"0JX10ZB"  =  "1"  /* Transfer Face Subcu/Fascia with Skin, Subcu, Open Approach */
"0JX10ZC"  =  "1"  /* Transfer Face Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JX10ZZ"  =  "1"  /* Transfer Face Subcutaneous Tissue and Fascia, Open Approach */
"0JX13ZB"  =  "1"  /* Transfer Face Subcu/Fascia with Skin, Subcu, Perc Approach */
"0JX13ZC"  =  "1"  /* Transfer Face Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JX13ZZ"  =  "1"  /* Transfer Face Subcutaneous Tissue and Fascia, Perc Approach */
"0JX40ZB"  =  "1"  /* Transfer Ant Neck Subcu/Fascia w Skin, Subcu, Open */
"0JX40ZC"  =  "1"  /* Transfer Ant Neck Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JX40ZZ"  =  "1"  /* Transfer Ant Neck Subcu/Fascia, Open Approach */
"0JX43ZB"  =  "1"  /* Transfer Ant Neck Subcu/Fascia w Skin, Subcu, Perc */
"0JX43ZC"  =  "1"  /* Transfer Ant Neck Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JX43ZZ"  =  "1"  /* Transfer Ant Neck Subcu/Fascia, Perc Approach */
"0JX50ZB"  =  "1"  /* Transfer Post Neck Subcu/Fascia w Skin, Subcu, Open */
"0JX50ZC"  =  "1"  /* Transfer Post Neck Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JX50ZZ"  =  "1"  /* Transfer Post Neck Subcu/Fascia, Open Approach */
"0JX53ZB"  =  "1"  /* Transfer Post Neck Subcu/Fascia w Skin, Subcu, Perc */
"0JX53ZC"  =  "1"  /* Transfer Post Neck Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JX53ZZ"  =  "1"  /* Transfer Post Neck Subcu/Fascia, Perc Approach */
"0JX60ZB"  =  "1"  /* Transfer Chest Subcu/Fascia with Skin, Subcu, Open Approach */
"0JX60ZC"  =  "1"  /* Transfer Chest Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JX60ZZ"  =  "1"  /* Transfer Chest Subcutaneous Tissue and Fascia, Open Approach */
"0JX63ZB"  =  "1"  /* Transfer Chest Subcu/Fascia with Skin, Subcu, Perc Approach */
"0JX63ZC"  =  "1"  /* Transfer Chest Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JX63ZZ"  =  "1"  /* Transfer Chest Subcutaneous Tissue and Fascia, Perc Approach */
"0JX70ZB"  =  "1"  /* Transfer Back Subcu/Fascia with Skin, Subcu, Open Approach */
"0JX70ZC"  =  "1"  /* Transfer Back Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JX70ZZ"  =  "1"  /* Transfer Back Subcutaneous Tissue and Fascia, Open Approach */
"0JX73ZB"  =  "1"  /* Transfer Back Subcu/Fascia with Skin, Subcu, Perc Approach */
"0JX73ZC"  =  "1"  /* Transfer Back Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JX73ZZ"  =  "1"  /* Transfer Back Subcutaneous Tissue and Fascia, Perc Approach */
"0JX80ZB"  =  "1"  /* Transfer Abd Subcu/Fascia with Skin, Subcu, Open Approach */
"0JX80ZC"  =  "1"  /* Transfer Abd Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JX80ZZ"  =  "1"  /* Transfer Abd Subcu/Fascia, Open Approach */
"0JX83ZB"  =  "1"  /* Transfer Abd Subcu/Fascia with Skin, Subcu, Perc Approach */
"0JX83ZC"  =  "1"  /* Transfer Abd Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JX83ZZ"  =  "1"  /* Transfer Abd Subcu/Fascia, Perc Approach */
"0JX90ZB"  =  "1"  /* Transfer Buttock Subcu/Fascia w Skin, Subcu, Open */
"0JX90ZC"  =  "1"  /* Transfer Buttock Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JX90ZZ"  =  "1"  /* Transfer Buttock Subcu/Fascia, Open Approach */
"0JX93ZB"  =  "1"  /* Transfer Buttock Subcu/Fascia w Skin, Subcu, Perc */
"0JX93ZC"  =  "1"  /* Transfer Buttock Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JX93ZZ"  =  "1"  /* Transfer Buttock Subcu/Fascia, Perc Approach */
"0JXB0ZB"  =  "1"  /* Transfer Perineum Subcu/Fascia w Skin, Subcu, Open */
"0JXB0ZC"  =  "1"  /* Transfer Perineum Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JXB0ZZ"  =  "1"  /* Transfer Perineum Subcu/Fascia, Open Approach */
"0JXB3ZB"  =  "1"  /* Transfer Perineum Subcu/Fascia w Skin, Subcu, Perc */
"0JXB3ZC"  =  "1"  /* Transfer Perineum Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JXB3ZZ"  =  "1"  /* Transfer Perineum Subcu/Fascia, Perc Approach */
"0JXC0ZB"  =  "1"  /* Transfer Pelvic Subcu/Fascia with Skin, Subcu, Open Approach */
"0JXC0ZC"  =  "1"  /* Transfer Pelvic Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JXC0ZZ"  =  "1"  /* Transfer Pelvic Subcu/Fascia, Open Approach */
"0JXC3ZB"  =  "1"  /* Transfer Pelvic Subcu/Fascia with Skin, Subcu, Perc Approach */
"0JXC3ZC"  =  "1"  /* Transfer Pelvic Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JXC3ZZ"  =  "1"  /* Transfer Pelvic Subcu/Fascia, Perc Approach */
"0JXD0ZB"  =  "1"  /* Transfer R Up Arm Subcu/Fascia w Skin, Subcu, Open */
"0JXD0ZC"  =  "1"  /* Transfer R Up Arm Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JXD0ZZ"  =  "1"  /* Transfer R Up Arm Subcu/Fascia, Open Approach */
"0JXD3ZB"  =  "1"  /* Transfer R Up Arm Subcu/Fascia w Skin, Subcu, Perc */
"0JXD3ZC"  =  "1"  /* Transfer R Up Arm Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JXD3ZZ"  =  "1"  /* Transfer R Up Arm Subcu/Fascia, Perc Approach */
"0JXF0ZB"  =  "1"  /* Transfer L Up Arm Subcu/Fascia w Skin, Subcu, Open */
"0JXF0ZC"  =  "1"  /* Transfer L Up Arm Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JXF0ZZ"  =  "1"  /* Transfer L Up Arm Subcu/Fascia, Open Approach */
"0JXF3ZB"  =  "1"  /* Transfer L Up Arm Subcu/Fascia w Skin, Subcu, Perc */
"0JXF3ZC"  =  "1"  /* Transfer L Up Arm Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JXF3ZZ"  =  "1"  /* Transfer L Up Arm Subcu/Fascia, Perc Approach */
"0JXG0ZB"  =  "1"  /* Transfer R Low Arm Subcu/Fascia w Skin, Subcu, Open */
"0JXG0ZC"  =  "1"  /* Transfer R Low Arm Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JXG0ZZ"  =  "1"  /* Transfer R Low Arm Subcu/Fascia, Open Approach */
"0JXG3ZB"  =  "1"  /* Transfer R Low Arm Subcu/Fascia w Skin, Subcu, Perc */
"0JXG3ZC"  =  "1"  /* Transfer R Low Arm Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JXG3ZZ"  =  "1"  /* Transfer R Low Arm Subcu/Fascia, Perc Approach */
"0JXH0ZB"  =  "1"  /* Transfer L Low Arm Subcu/Fascia w Skin, Subcu, Open */
"0JXH0ZC"  =  "1"  /* Transfer L Low Arm Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JXH0ZZ"  =  "1"  /* Transfer L Low Arm Subcu/Fascia, Open Approach */
"0JXH3ZB"  =  "1"  /* Transfer L Low Arm Subcu/Fascia w Skin, Subcu, Perc */
"0JXH3ZC"  =  "1"  /* Transfer L Low Arm Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JXH3ZZ"  =  "1"  /* Transfer L Low Arm Subcu/Fascia, Perc Approach */
"0JXJ0ZB"  =  "1"  /* Transfer R Hand Subcu/Fascia with Skin, Subcu, Open Approach */
"0JXJ0ZC"  =  "1"  /* Transfer R Hand Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JXJ0ZZ"  =  "1"  /* Transfer R Hand Subcu/Fascia, Open Approach */
"0JXJ3ZB"  =  "1"  /* Transfer R Hand Subcu/Fascia with Skin, Subcu, Perc Approach */
"0JXJ3ZC"  =  "1"  /* Transfer R Hand Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JXJ3ZZ"  =  "1"  /* Transfer R Hand Subcu/Fascia, Perc Approach */
"0JXK0ZB"  =  "1"  /* Transfer L Hand Subcu/Fascia with Skin, Subcu, Open Approach */
"0JXK0ZC"  =  "1"  /* Transfer L Hand Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JXK0ZZ"  =  "1"  /* Transfer L Hand Subcu/Fascia, Open Approach */
"0JXK3ZB"  =  "1"  /* Transfer L Hand Subcu/Fascia with Skin, Subcu, Perc Approach */
"0JXK3ZC"  =  "1"  /* Transfer L Hand Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JXK3ZZ"  =  "1"  /* Transfer L Hand Subcu/Fascia, Perc Approach */
"0JXL0ZB"  =  "1"  /* Transfer R Up Leg Subcu/Fascia w Skin, Subcu, Open */
"0JXL0ZC"  =  "1"  /* Transfer R Up Leg Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JXL0ZZ"  =  "1"  /* Transfer R Up Leg Subcu/Fascia, Open Approach */
"0JXL3ZB"  =  "1"  /* Transfer R Up Leg Subcu/Fascia w Skin, Subcu, Perc */
"0JXL3ZC"  =  "1"  /* Transfer R Up Leg Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JXL3ZZ"  =  "1"  /* Transfer R Up Leg Subcu/Fascia, Perc Approach */
"0JXM0ZB"  =  "1"  /* Transfer L Up Leg Subcu/Fascia w Skin, Subcu, Open */
"0JXM0ZC"  =  "1"  /* Transfer L Up Leg Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JXM0ZZ"  =  "1"  /* Transfer L Up Leg Subcu/Fascia, Open Approach */
"0JXM3ZB"  =  "1"  /* Transfer L Up Leg Subcu/Fascia w Skin, Subcu, Perc */
"0JXM3ZC"  =  "1"  /* Transfer L Up Leg Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JXM3ZZ"  =  "1"  /* Transfer L Up Leg Subcu/Fascia, Perc Approach */
"0JXN0ZB"  =  "1"  /* Transfer R Low Leg Subcu/Fascia w Skin, Subcu, Open */
"0JXN0ZC"  =  "1"  /* Transfer R Low Leg Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JXN0ZZ"  =  "1"  /* Transfer R Low Leg Subcu/Fascia, Open Approach */
"0JXN3ZB"  =  "1"  /* Transfer R Low Leg Subcu/Fascia w Skin, Subcu, Perc */
"0JXN3ZC"  =  "1"  /* Transfer R Low Leg Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JXN3ZZ"  =  "1"  /* Transfer R Low Leg Subcu/Fascia, Perc Approach */
"0JXP0ZB"  =  "1"  /* Transfer L Low Leg Subcu/Fascia w Skin, Subcu, Open */
"0JXP0ZC"  =  "1"  /* Transfer L Low Leg Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JXP0ZZ"  =  "1"  /* Transfer L Low Leg Subcu/Fascia, Open Approach */
"0JXP3ZB"  =  "1"  /* Transfer L Low Leg Subcu/Fascia w Skin, Subcu, Perc */
"0JXP3ZC"  =  "1"  /* Transfer L Low Leg Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JXP3ZZ"  =  "1"  /* Transfer L Low Leg Subcu/Fascia, Perc Approach */
"0JXQ0ZB"  =  "1"  /* Transfer R Foot Subcu/Fascia with Skin, Subcu, Open Approach */
"0JXQ0ZC"  =  "1"  /* Transfer R Foot Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JXQ0ZZ"  =  "1"  /* Transfer R Foot Subcu/Fascia, Open Approach */
"0JXQ3ZB"  =  "1"  /* Transfer R Foot Subcu/Fascia with Skin, Subcu, Perc Approach */
"0JXQ3ZC"  =  "1"  /* Transfer R Foot Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JXQ3ZZ"  =  "1"  /* Transfer R Foot Subcu/Fascia, Perc Approach */
"0JXR0ZB"  =  "1"  /* Transfer L Foot Subcu/Fascia with Skin, Subcu, Open Approach */
"0JXR0ZC"  =  "1"  /* Transfer L Foot Subcu/Fascia w Skin, Subcu, Fascia, Open */
"0JXR0ZZ"  =  "1"  /* Transfer L Foot Subcu/Fascia, Open Approach */
"0JXR3ZB"  =  "1"  /* Transfer L Foot Subcu/Fascia with Skin, Subcu, Perc Approach */
"0JXR3ZC"  =  "1"  /* Transfer L Foot Subcu/Fascia w Skin, Subcu, Fascia, Perc */
"0JXR3ZZ"  =  "1"  /* Transfer L Foot Subcu/Fascia, Perc Approach */
"0K500ZZ"  =  "1"  /* Destruction of Head Muscle, Open Approach */
"0K503ZZ"  =  "1"  /* Destruction of Head Muscle, Percutaneous Approach */
"0K504ZZ"  =  "1"  /* Destruction of Head Muscle, Percutaneous Endoscopic Approach */
"0K510ZZ"  =  "1"  /* Destruction of Facial Muscle, Open Approach */
"0K513ZZ"  =  "1"  /* Destruction of Facial Muscle, Percutaneous Approach */
"0K514ZZ"  =  "1"  /* Destruction of Facial Muscle, Perc Endo Approach */
"0K520ZZ"  =  "1"  /* Destruction of Right Neck Muscle, Open Approach */
"0K523ZZ"  =  "1"  /* Destruction of Right Neck Muscle, Percutaneous Approach */
"0K524ZZ"  =  "1"  /* Destruction of Right Neck Muscle, Perc Endo Approach */
"0K530ZZ"  =  "1"  /* Destruction of Left Neck Muscle, Open Approach */
"0K533ZZ"  =  "1"  /* Destruction of Left Neck Muscle, Percutaneous Approach */
"0K534ZZ"  =  "1"  /* Destruction of Left Neck Muscle, Perc Endo Approach */
"0K540ZZ"  =  "1"  /* Destruction of Tongue, Palate, Pharynx Muscle, Open Approach */
"0K543ZZ"  =  "1"  /* Destruction of Tongue, Palate, Pharynx Muscle, Perc Approach */
"0K544ZZ"  =  "1"  /* Destruction of Tongue/Palate/Phar Muscle, Perc Endo Approach */
"0K550ZZ"  =  "1"  /* Destruction of Right Shoulder Muscle, Open Approach */
"0K553ZZ"  =  "1"  /* Destruction of Right Shoulder Muscle, Percutaneous Approach */
"0K554ZZ"  =  "1"  /* Destruction of Right Shoulder Muscle, Perc Endo Approach */
"0K560ZZ"  =  "1"  /* Destruction of Left Shoulder Muscle, Open Approach */
"0K563ZZ"  =  "1"  /* Destruction of Left Shoulder Muscle, Percutaneous Approach */
"0K564ZZ"  =  "1"  /* Destruction of Left Shoulder Muscle, Perc Endo Approach */
"0K570ZZ"  =  "1"  /* Destruction of Right Upper Arm Muscle, Open Approach */
"0K573ZZ"  =  "1"  /* Destruction of Right Upper Arm Muscle, Percutaneous Approach */
"0K574ZZ"  =  "1"  /* Destruction of Right Upper Arm Muscle, Perc Endo Approach */
"0K580ZZ"  =  "1"  /* Destruction of Left Upper Arm Muscle, Open Approach */
"0K583ZZ"  =  "1"  /* Destruction of Left Upper Arm Muscle, Percutaneous Approach */
"0K584ZZ"  =  "1"  /* Destruction of Left Upper Arm Muscle, Perc Endo Approach */
"0K590ZZ"  =  "1"  /* Destruction of R Low Arm & Wrist Muscle, Open Approach */
"0K593ZZ"  =  "1"  /* Destruction of R Low Arm & Wrist Muscle, Perc Approach */
"0K594ZZ"  =  "1"  /* Destruction of R Low Arm & Wrist Muscle, Perc Endo Approach */
"0K5B0ZZ"  =  "1"  /* Destruction of L Low Arm & Wrist Muscle, Open Approach */
"0K5B3ZZ"  =  "1"  /* Destruction of L Low Arm & Wrist Muscle, Perc Approach */
"0K5B4ZZ"  =  "1"  /* Destruction of L Low Arm & Wrist Muscle, Perc Endo Approach */
"0K5C0ZZ"  =  "1"  /* Destruction of Right Hand Muscle, Open Approach */
"0K5C3ZZ"  =  "1"  /* Destruction of Right Hand Muscle, Percutaneous Approach */
"0K5C4ZZ"  =  "1"  /* Destruction of Right Hand Muscle, Perc Endo Approach */
"0K5D0ZZ"  =  "1"  /* Destruction of Left Hand Muscle, Open Approach */
"0K5D3ZZ"  =  "1"  /* Destruction of Left Hand Muscle, Percutaneous Approach */
"0K5D4ZZ"  =  "1"  /* Destruction of Left Hand Muscle, Perc Endo Approach */
"0K5F0ZZ"  =  "1"  /* Destruction of Right Trunk Muscle, Open Approach */
"0K5F3ZZ"  =  "1"  /* Destruction of Right Trunk Muscle, Percutaneous Approach */
"0K5F4ZZ"  =  "1"  /* Destruction of Right Trunk Muscle, Perc Endo Approach */
"0K5G0ZZ"  =  "1"  /* Destruction of Left Trunk Muscle, Open Approach */
"0K5G3ZZ"  =  "1"  /* Destruction of Left Trunk Muscle, Percutaneous Approach */
"0K5G4ZZ"  =  "1"  /* Destruction of Left Trunk Muscle, Perc Endo Approach */
"0K5H0ZZ"  =  "1"  /* Destruction of Right Thorax Muscle, Open Approach */
"0K5H3ZZ"  =  "1"  /* Destruction of Right Thorax Muscle, Percutaneous Approach */
"0K5H4ZZ"  =  "1"  /* Destruction of Right Thorax Muscle, Perc Endo Approach */
"0K5J0ZZ"  =  "1"  /* Destruction of Left Thorax Muscle, Open Approach */
"0K5J3ZZ"  =  "1"  /* Destruction of Left Thorax Muscle, Percutaneous Approach */
"0K5J4ZZ"  =  "1"  /* Destruction of Left Thorax Muscle, Perc Endo Approach */
"0K5K0ZZ"  =  "1"  /* Destruction of Right Abdomen Muscle, Open Approach */
"0K5K3ZZ"  =  "1"  /* Destruction of Right Abdomen Muscle, Percutaneous Approach */
"0K5K4ZZ"  =  "1"  /* Destruction of Right Abdomen Muscle, Perc Endo Approach */
"0K5L0ZZ"  =  "1"  /* Destruction of Left Abdomen Muscle, Open Approach */
"0K5L3ZZ"  =  "1"  /* Destruction of Left Abdomen Muscle, Percutaneous Approach */
"0K5L4ZZ"  =  "1"  /* Destruction of Left Abdomen Muscle, Perc Endo Approach */
"0K5M0ZZ"  =  "1"  /* Destruction of Perineum Muscle, Open Approach */
"0K5M3ZZ"  =  "1"  /* Destruction of Perineum Muscle, Percutaneous Approach */
"0K5M4ZZ"  =  "1"  /* Destruction of Perineum Muscle, Perc Endo Approach */
"0K5N0ZZ"  =  "1"  /* Destruction of Right Hip Muscle, Open Approach */
"0K5N3ZZ"  =  "1"  /* Destruction of Right Hip Muscle, Percutaneous Approach */
"0K5N4ZZ"  =  "1"  /* Destruction of Right Hip Muscle, Perc Endo Approach */
"0K5P0ZZ"  =  "1"  /* Destruction of Left Hip Muscle, Open Approach */
"0K5P3ZZ"  =  "1"  /* Destruction of Left Hip Muscle, Percutaneous Approach */
"0K5P4ZZ"  =  "1"  /* Destruction of Left Hip Muscle, Perc Endo Approach */
"0K5Q0ZZ"  =  "1"  /* Destruction of Right Upper Leg Muscle, Open Approach */
"0K5Q3ZZ"  =  "1"  /* Destruction of Right Upper Leg Muscle, Percutaneous Approach */
"0K5Q4ZZ"  =  "1"  /* Destruction of Right Upper Leg Muscle, Perc Endo Approach */
"0K5R0ZZ"  =  "1"  /* Destruction of Left Upper Leg Muscle, Open Approach */
"0K5R3ZZ"  =  "1"  /* Destruction of Left Upper Leg Muscle, Percutaneous Approach */
"0K5R4ZZ"  =  "1"  /* Destruction of Left Upper Leg Muscle, Perc Endo Approach */
"0K5S0ZZ"  =  "1"  /* Destruction of Right Lower Leg Muscle, Open Approach */
"0K5S3ZZ"  =  "1"  /* Destruction of Right Lower Leg Muscle, Percutaneous Approach */
"0K5S4ZZ"  =  "1"  /* Destruction of Right Lower Leg Muscle, Perc Endo Approach */
"0K5T0ZZ"  =  "1"  /* Destruction of Left Lower Leg Muscle, Open Approach */
"0K5T3ZZ"  =  "1"  /* Destruction of Left Lower Leg Muscle, Percutaneous Approach */
"0K5T4ZZ"  =  "1"  /* Destruction of Left Lower Leg Muscle, Perc Endo Approach */
"0K5V0ZZ"  =  "1"  /* Destruction of Right Foot Muscle, Open Approach */
"0K5V3ZZ"  =  "1"  /* Destruction of Right Foot Muscle, Percutaneous Approach */
"0K5V4ZZ"  =  "1"  /* Destruction of Right Foot Muscle, Perc Endo Approach */
"0K5W0ZZ"  =  "1"  /* Destruction of Left Foot Muscle, Open Approach */
"0K5W3ZZ"  =  "1"  /* Destruction of Left Foot Muscle, Percutaneous Approach */
"0K5W4ZZ"  =  "1"  /* Destruction of Left Foot Muscle, Perc Endo Approach */
"0K800ZZ"  =  "1"  /* Division of Head Muscle, Open Approach */
"0K803ZZ"  =  "1"  /* Division of Head Muscle, Percutaneous Approach */
"0K804ZZ"  =  "1"  /* Division of Head Muscle, Percutaneous Endoscopic Approach */
"0K810ZZ"  =  "1"  /* Division of Facial Muscle, Open Approach */
"0K813ZZ"  =  "1"  /* Division of Facial Muscle, Percutaneous Approach */
"0K814ZZ"  =  "1"  /* Division of Facial Muscle, Percutaneous Endoscopic Approach */
"0K820ZZ"  =  "1"  /* Division of Right Neck Muscle, Open Approach */
"0K823ZZ"  =  "1"  /* Division of Right Neck Muscle, Percutaneous Approach */
"0K824ZZ"  =  "1"  /* Division of Right Neck Muscle, Perc Endo Approach */
"0K830ZZ"  =  "1"  /* Division of Left Neck Muscle, Open Approach */
"0K833ZZ"  =  "1"  /* Division of Left Neck Muscle, Percutaneous Approach */
"0K834ZZ"  =  "1"  /* Division of Left Neck Muscle, Perc Endo Approach */
"0K840ZZ"  =  "1"  /* Division of Tongue, Palate, Pharynx Muscle, Open Approach */
"0K843ZZ"  =  "1"  /* Division of Tongue, Palate, Pharynx Muscle, Perc Approach */
"0K844ZZ"  =  "1"  /* Division of Tongue/Palate/Phar Muscle, Perc Endo Approach */
"0K850ZZ"  =  "1"  /* Division of Right Shoulder Muscle, Open Approach */
"0K853ZZ"  =  "1"  /* Division of Right Shoulder Muscle, Percutaneous Approach */
"0K854ZZ"  =  "1"  /* Division of Right Shoulder Muscle, Perc Endo Approach */
"0K860ZZ"  =  "1"  /* Division of Left Shoulder Muscle, Open Approach */
"0K863ZZ"  =  "1"  /* Division of Left Shoulder Muscle, Percutaneous Approach */
"0K864ZZ"  =  "1"  /* Division of Left Shoulder Muscle, Perc Endo Approach */
"0K870ZZ"  =  "1"  /* Division of Right Upper Arm Muscle, Open Approach */
"0K873ZZ"  =  "1"  /* Division of Right Upper Arm Muscle, Percutaneous Approach */
"0K874ZZ"  =  "1"  /* Division of Right Upper Arm Muscle, Perc Endo Approach */
"0K880ZZ"  =  "1"  /* Division of Left Upper Arm Muscle, Open Approach */
"0K883ZZ"  =  "1"  /* Division of Left Upper Arm Muscle, Percutaneous Approach */
"0K884ZZ"  =  "1"  /* Division of Left Upper Arm Muscle, Perc Endo Approach */
"0K890ZZ"  =  "1"  /* Division of Right Lower Arm and Wrist Muscle, Open Approach */
"0K893ZZ"  =  "1"  /* Division of Right Lower Arm and Wrist Muscle, Perc Approach */
"0K894ZZ"  =  "1"  /* Division of R Low Arm & Wrist Muscle, Perc Endo Approach */
"0K8B0ZZ"  =  "1"  /* Division of Left Lower Arm and Wrist Muscle, Open Approach */
"0K8B3ZZ"  =  "1"  /* Division of Left Lower Arm and Wrist Muscle, Perc Approach */
"0K8B4ZZ"  =  "1"  /* Division of L Low Arm & Wrist Muscle, Perc Endo Approach */
"0K8C0ZZ"  =  "1"  /* Division of Right Hand Muscle, Open Approach */
"0K8C3ZZ"  =  "1"  /* Division of Right Hand Muscle, Percutaneous Approach */
"0K8C4ZZ"  =  "1"  /* Division of Right Hand Muscle, Perc Endo Approach */
"0K8D0ZZ"  =  "1"  /* Division of Left Hand Muscle, Open Approach */
"0K8D3ZZ"  =  "1"  /* Division of Left Hand Muscle, Percutaneous Approach */
"0K8D4ZZ"  =  "1"  /* Division of Left Hand Muscle, Perc Endo Approach */
"0K8F0ZZ"  =  "1"  /* Division of Right Trunk Muscle, Open Approach */
"0K8F3ZZ"  =  "1"  /* Division of Right Trunk Muscle, Percutaneous Approach */
"0K8F4ZZ"  =  "1"  /* Division of Right Trunk Muscle, Perc Endo Approach */
"0K8G0ZZ"  =  "1"  /* Division of Left Trunk Muscle, Open Approach */
"0K8G3ZZ"  =  "1"  /* Division of Left Trunk Muscle, Percutaneous Approach */
"0K8G4ZZ"  =  "1"  /* Division of Left Trunk Muscle, Perc Endo Approach */
"0K8H0ZZ"  =  "1"  /* Division of Right Thorax Muscle, Open Approach */
"0K8H3ZZ"  =  "1"  /* Division of Right Thorax Muscle, Percutaneous Approach */
"0K8H4ZZ"  =  "1"  /* Division of Right Thorax Muscle, Perc Endo Approach */
"0K8J0ZZ"  =  "1"  /* Division of Left Thorax Muscle, Open Approach */
"0K8J3ZZ"  =  "1"  /* Division of Left Thorax Muscle, Percutaneous Approach */
"0K8J4ZZ"  =  "1"  /* Division of Left Thorax Muscle, Perc Endo Approach */
"0K8K0ZZ"  =  "1"  /* Division of Right Abdomen Muscle, Open Approach */
"0K8K3ZZ"  =  "1"  /* Division of Right Abdomen Muscle, Percutaneous Approach */
"0K8K4ZZ"  =  "1"  /* Division of Right Abdomen Muscle, Perc Endo Approach */
"0K8L0ZZ"  =  "1"  /* Division of Left Abdomen Muscle, Open Approach */
"0K8L3ZZ"  =  "1"  /* Division of Left Abdomen Muscle, Percutaneous Approach */
"0K8L4ZZ"  =  "1"  /* Division of Left Abdomen Muscle, Perc Endo Approach */
"0K8M0ZZ"  =  "1"  /* Division of Perineum Muscle, Open Approach */
"0K8M3ZZ"  =  "1"  /* Division of Perineum Muscle, Percutaneous Approach */
"0K8M4ZZ"  =  "1"  /* Division of Perineum Muscle, Perc Endo Approach */
"0K8N0ZZ"  =  "1"  /* Division of Right Hip Muscle, Open Approach */
"0K8N3ZZ"  =  "1"  /* Division of Right Hip Muscle, Percutaneous Approach */
"0K8N4ZZ"  =  "1"  /* Division of Right Hip Muscle, Perc Endo Approach */
"0K8P0ZZ"  =  "1"  /* Division of Left Hip Muscle, Open Approach */
"0K8P3ZZ"  =  "1"  /* Division of Left Hip Muscle, Percutaneous Approach */
"0K8P4ZZ"  =  "1"  /* Division of Left Hip Muscle, Perc Endo Approach */
"0K8Q0ZZ"  =  "1"  /* Division of Right Upper Leg Muscle, Open Approach */
"0K8Q3ZZ"  =  "1"  /* Division of Right Upper Leg Muscle, Percutaneous Approach */
"0K8Q4ZZ"  =  "1"  /* Division of Right Upper Leg Muscle, Perc Endo Approach */
"0K8R0ZZ"  =  "1"  /* Division of Left Upper Leg Muscle, Open Approach */
"0K8R3ZZ"  =  "1"  /* Division of Left Upper Leg Muscle, Percutaneous Approach */
"0K8R4ZZ"  =  "1"  /* Division of Left Upper Leg Muscle, Perc Endo Approach */
"0K8S0ZZ"  =  "1"  /* Division of Right Lower Leg Muscle, Open Approach */
"0K8S3ZZ"  =  "1"  /* Division of Right Lower Leg Muscle, Percutaneous Approach */
"0K8S4ZZ"  =  "1"  /* Division of Right Lower Leg Muscle, Perc Endo Approach */
"0K8T0ZZ"  =  "1"  /* Division of Left Lower Leg Muscle, Open Approach */
"0K8T3ZZ"  =  "1"  /* Division of Left Lower Leg Muscle, Percutaneous Approach */
"0K8T4ZZ"  =  "1"  /* Division of Left Lower Leg Muscle, Perc Endo Approach */
"0K8V0ZZ"  =  "1"  /* Division of Right Foot Muscle, Open Approach */
"0K8V3ZZ"  =  "1"  /* Division of Right Foot Muscle, Percutaneous Approach */
"0K8V4ZZ"  =  "1"  /* Division of Right Foot Muscle, Perc Endo Approach */
"0K8W0ZZ"  =  "1"  /* Division of Left Foot Muscle, Open Approach */
"0K8W3ZZ"  =  "1"  /* Division of Left Foot Muscle, Percutaneous Approach */
"0K8W4ZZ"  =  "1"  /* Division of Left Foot Muscle, Perc Endo Approach */
"0K9000Z"  =  "1"  /* Drainage of Head Muscle with Drainage Device, Open Approach */
"0K900ZX"  =  "1"  /* Drainage of Head Muscle, Open Approach, Diagnostic */
"0K900ZZ"  =  "1"  /* Drainage of Head Muscle, Open Approach */
"0K9030Z"  =  "1"  /* Drainage of Head Muscle with Drainage Device, Perc Approach */
"0K903ZX"  =  "1"  /* Drainage of Head Muscle, Percutaneous Approach, Diagnostic */
"0K9040Z"  =  "1"  /* Drainage of Head Muscle with Drain Dev, Perc Endo Approach */
"0K904ZX"  =  "1"  /* Drainage of Head Muscle, Perc Endo Approach, Diagn */
"0K904ZZ"  =  "1"  /* Drainage of Head Muscle, Percutaneous Endoscopic Approach */
"0K9100Z"  =  "1"  /* Drainage of Facial Muscle with Drain Dev, Open Approach */
"0K910ZX"  =  "1"  /* Drainage of Facial Muscle, Open Approach, Diagnostic */
"0K910ZZ"  =  "1"  /* Drainage of Facial Muscle, Open Approach */
"0K9130Z"  =  "1"  /* Drainage of Facial Muscle with Drain Dev, Perc Approach */
"0K913ZX"  =  "1"  /* Drainage of Facial Muscle, Percutaneous Approach, Diagnostic */
"0K9140Z"  =  "1"  /* Drainage of Facial Muscle with Drain Dev, Perc Endo Approach */
"0K914ZX"  =  "1"  /* Drainage of Facial Muscle, Perc Endo Approach, Diagn */
"0K914ZZ"  =  "1"  /* Drainage of Facial Muscle, Percutaneous Endoscopic Approach */
"0K9200Z"  =  "1"  /* Drainage of Right Neck Muscle with Drain Dev, Open Approach */
"0K920ZX"  =  "1"  /* Drainage of Right Neck Muscle, Open Approach, Diagnostic */
"0K920ZZ"  =  "1"  /* Drainage of Right Neck Muscle, Open Approach */
"0K9230Z"  =  "1"  /* Drainage of Right Neck Muscle with Drain Dev, Perc Approach */
"0K923ZX"  =  "1"  /* Drainage of Right Neck Muscle, Percutaneous Approach, Diagn */
"0K9240Z"  =  "1"  /* Drainage of R Neck Muscle with Drain Dev, Perc Endo Approach */
"0K924ZX"  =  "1"  /* Drainage of Right Neck Muscle, Perc Endo Approach, Diagn */
"0K924ZZ"  =  "1"  /* Drainage of Right Neck Muscle, Perc Endo Approach */
"0K9300Z"  =  "1"  /* Drainage of Left Neck Muscle with Drain Dev, Open Approach */
"0K930ZX"  =  "1"  /* Drainage of Left Neck Muscle, Open Approach, Diagnostic */
"0K930ZZ"  =  "1"  /* Drainage of Left Neck Muscle, Open Approach */
"0K9330Z"  =  "1"  /* Drainage of Left Neck Muscle with Drain Dev, Perc Approach */
"0K933ZX"  =  "1"  /* Drainage of Left Neck Muscle, Percutaneous Approach, Diagn */
"0K9340Z"  =  "1"  /* Drainage of L Neck Muscle with Drain Dev, Perc Endo Approach */
"0K934ZX"  =  "1"  /* Drainage of Left Neck Muscle, Perc Endo Approach, Diagn */
"0K934ZZ"  =  "1"  /* Drainage of Left Neck Muscle, Perc Endo Approach */
"0K9400Z"  =  "1"  /* Drain Tongue/Palate/Phar Muscle w Drain Dev, Open */
"0K940ZX"  =  "1"  /* Drainage of Tongue/Palate/Phar Muscle, Open Approach, Diagn */
"0K940ZZ"  =  "1"  /* Drainage of Tongue, Palate, Pharynx Muscle, Open Approach */
"0K9430Z"  =  "1"  /* Drain Tongue/Palate/Phar Muscle w Drain Dev, Perc */
"0K943ZX"  =  "1"  /* Drainage of Tongue/Palate/Phar Muscle, Perc Approach, Diagn */
"0K9440Z"  =  "1"  /* Drain Tongue/Palate/Phar Muscle w Drain Dev, Perc Endo */
"0K944ZX"  =  "1"  /* Drain Tongue/Palate/Phar Muscle, Perc Endo, Diagn */
"0K944ZZ"  =  "1"  /* Drainage of Tongue/Palate/Phar Muscle, Perc Endo Approach */
"0K9500Z"  =  "1"  /* Drainage of R Shoulder Muscle with Drain Dev, Open Approach */
"0K950ZX"  =  "1"  /* Drainage of Right Shoulder Muscle, Open Approach, Diagnostic */
"0K950ZZ"  =  "1"  /* Drainage of Right Shoulder Muscle, Open Approach */
"0K9530Z"  =  "1"  /* Drainage of R Shoulder Muscle with Drain Dev, Perc Approach */
"0K953ZX"  =  "1"  /* Drainage of Right Shoulder Muscle, Perc Approach, Diagn */
"0K9540Z"  =  "1"  /* Drain R Shoulder Muscle w Drain Dev, Perc Endo */
"0K954ZX"  =  "1"  /* Drainage of Right Shoulder Muscle, Perc Endo Approach, Diagn */
"0K954ZZ"  =  "1"  /* Drainage of Right Shoulder Muscle, Perc Endo Approach */
"0K9600Z"  =  "1"  /* Drainage of L Shoulder Muscle with Drain Dev, Open Approach */
"0K960ZX"  =  "1"  /* Drainage of Left Shoulder Muscle, Open Approach, Diagnostic */
"0K960ZZ"  =  "1"  /* Drainage of Left Shoulder Muscle, Open Approach */
"0K9630Z"  =  "1"  /* Drainage of L Shoulder Muscle with Drain Dev, Perc Approach */
"0K963ZX"  =  "1"  /* Drainage of Left Shoulder Muscle, Perc Approach, Diagn */
"0K9640Z"  =  "1"  /* Drain L Shoulder Muscle w Drain Dev, Perc Endo */
"0K964ZX"  =  "1"  /* Drainage of Left Shoulder Muscle, Perc Endo Approach, Diagn */
"0K964ZZ"  =  "1"  /* Drainage of Left Shoulder Muscle, Perc Endo Approach */
"0K9700Z"  =  "1"  /* Drainage of R Up Arm Muscle with Drain Dev, Open Approach */
"0K970ZX"  =  "1"  /* Drainage of Right Upper Arm Muscle, Open Approach, Diagn */
"0K970ZZ"  =  "1"  /* Drainage of Right Upper Arm Muscle, Open Approach */
"0K9730Z"  =  "1"  /* Drainage of R Up Arm Muscle with Drain Dev, Perc Approach */
"0K973ZX"  =  "1"  /* Drainage of Right Upper Arm Muscle, Perc Approach, Diagn */
"0K9740Z"  =  "1"  /* Drain of R Up Arm Muscle with Drain Dev, Perc Endo Approach */
"0K974ZX"  =  "1"  /* Drainage of R Up Arm Muscle, Perc Endo Approach, Diagn */
"0K974ZZ"  =  "1"  /* Drainage of Right Upper Arm Muscle, Perc Endo Approach */
"0K9800Z"  =  "1"  /* Drainage of L Up Arm Muscle with Drain Dev, Open Approach */
"0K980ZX"  =  "1"  /* Drainage of Left Upper Arm Muscle, Open Approach, Diagnostic */
"0K980ZZ"  =  "1"  /* Drainage of Left Upper Arm Muscle, Open Approach */
"0K9830Z"  =  "1"  /* Drainage of L Up Arm Muscle with Drain Dev, Perc Approach */
"0K983ZX"  =  "1"  /* Drainage of Left Upper Arm Muscle, Perc Approach, Diagn */
"0K9840Z"  =  "1"  /* Drain of L Up Arm Muscle with Drain Dev, Perc Endo Approach */
"0K984ZX"  =  "1"  /* Drainage of Left Upper Arm Muscle, Perc Endo Approach, Diagn */
"0K984ZZ"  =  "1"  /* Drainage of Left Upper Arm Muscle, Perc Endo Approach */
"0K9900Z"  =  "1"  /* Drain R Low Arm & Wrist Muscle w Drain Dev, Open */
"0K990ZX"  =  "1"  /* Drainage of R Low Arm & Wrist Muscle, Open Approach, Diagn */
"0K990ZZ"  =  "1"  /* Drainage of Right Lower Arm and Wrist Muscle, Open Approach */
"0K9930Z"  =  "1"  /* Drain R Low Arm & Wrist Muscle w Drain Dev, Perc */
"0K993ZX"  =  "1"  /* Drainage of R Low Arm & Wrist Muscle, Perc Approach, Diagn */
"0K9940Z"  =  "1"  /* Drain R Low Arm & Wrist Muscle w Drain Dev, Perc Endo */
"0K994ZX"  =  "1"  /* Drain of R Low Arm & Wrist Muscle, Perc Endo Approach, Diagn */
"0K994ZZ"  =  "1"  /* Drainage of R Low Arm & Wrist Muscle, Perc Endo Approach */
"0K9B00Z"  =  "1"  /* Drain L Low Arm & Wrist Muscle w Drain Dev, Open */
"0K9B0ZX"  =  "1"  /* Drainage of L Low Arm & Wrist Muscle, Open Approach, Diagn */
"0K9B0ZZ"  =  "1"  /* Drainage of Left Lower Arm and Wrist Muscle, Open Approach */
"0K9B30Z"  =  "1"  /* Drain L Low Arm & Wrist Muscle w Drain Dev, Perc */
"0K9B3ZX"  =  "1"  /* Drainage of L Low Arm & Wrist Muscle, Perc Approach, Diagn */
"0K9B40Z"  =  "1"  /* Drain L Low Arm & Wrist Muscle w Drain Dev, Perc Endo */
"0K9B4ZX"  =  "1"  /* Drain of L Low Arm & Wrist Muscle, Perc Endo Approach, Diagn */
"0K9B4ZZ"  =  "1"  /* Drainage of L Low Arm & Wrist Muscle, Perc Endo Approach */
"0K9C00Z"  =  "1"  /* Drainage of Right Hand Muscle with Drain Dev, Open Approach */
"0K9C0ZX"  =  "1"  /* Drainage of Right Hand Muscle, Open Approach, Diagnostic */
"0K9C0ZZ"  =  "1"  /* Drainage of Right Hand Muscle, Open Approach */
"0K9C30Z"  =  "1"  /* Drainage of Right Hand Muscle with Drain Dev, Perc Approach */
"0K9C3ZX"  =  "1"  /* Drainage of Right Hand Muscle, Percutaneous Approach, Diagn */
"0K9C40Z"  =  "1"  /* Drainage of R Hand Muscle with Drain Dev, Perc Endo Approach */
"0K9C4ZX"  =  "1"  /* Drainage of Right Hand Muscle, Perc Endo Approach, Diagn */
"0K9D00Z"  =  "1"  /* Drainage of Left Hand Muscle with Drain Dev, Open Approach */
"0K9D0ZX"  =  "1"  /* Drainage of Left Hand Muscle, Open Approach, Diagnostic */
"0K9D0ZZ"  =  "1"  /* Drainage of Left Hand Muscle, Open Approach */
"0K9D30Z"  =  "1"  /* Drainage of Left Hand Muscle with Drain Dev, Perc Approach */
"0K9D3ZX"  =  "1"  /* Drainage of Left Hand Muscle, Percutaneous Approach, Diagn */
"0K9D40Z"  =  "1"  /* Drainage of L Hand Muscle with Drain Dev, Perc Endo Approach */
"0K9D4ZX"  =  "1"  /* Drainage of Left Hand Muscle, Perc Endo Approach, Diagn */
"0K9F00Z"  =  "1"  /* Drainage of Right Trunk Muscle with Drain Dev, Open Approach */
"0K9F0ZX"  =  "1"  /* Drainage of Right Trunk Muscle, Open Approach, Diagnostic */
"0K9F0ZZ"  =  "1"  /* Drainage of Right Trunk Muscle, Open Approach */
"0K9F30Z"  =  "1"  /* Drainage of Right Trunk Muscle with Drain Dev, Perc Approach */
"0K9F3ZX"  =  "1"  /* Drainage of Right Trunk Muscle, Percutaneous Approach, Diagn */
"0K9F40Z"  =  "1"  /* Drain of R Trunk Muscle with Drain Dev, Perc Endo Approach */
"0K9F4ZX"  =  "1"  /* Drainage of Right Trunk Muscle, Perc Endo Approach, Diagn */
"0K9F4ZZ"  =  "1"  /* Drainage of Right Trunk Muscle, Perc Endo Approach */
"0K9G00Z"  =  "1"  /* Drainage of Left Trunk Muscle with Drain Dev, Open Approach */
"0K9G0ZX"  =  "1"  /* Drainage of Left Trunk Muscle, Open Approach, Diagnostic */
"0K9G0ZZ"  =  "1"  /* Drainage of Left Trunk Muscle, Open Approach */
"0K9G30Z"  =  "1"  /* Drainage of Left Trunk Muscle with Drain Dev, Perc Approach */
"0K9G3ZX"  =  "1"  /* Drainage of Left Trunk Muscle, Percutaneous Approach, Diagn */
"0K9G40Z"  =  "1"  /* Drain of L Trunk Muscle with Drain Dev, Perc Endo Approach */
"0K9G4ZX"  =  "1"  /* Drainage of Left Trunk Muscle, Perc Endo Approach, Diagn */
"0K9G4ZZ"  =  "1"  /* Drainage of Left Trunk Muscle, Perc Endo Approach */
"0K9H00Z"  =  "1"  /* Drainage of R Thorax Muscle with Drain Dev, Open Approach */
"0K9H0ZX"  =  "1"  /* Drainage of Right Thorax Muscle, Open Approach, Diagnostic */
"0K9H0ZZ"  =  "1"  /* Drainage of Right Thorax Muscle, Open Approach */
"0K9H30Z"  =  "1"  /* Drainage of R Thorax Muscle with Drain Dev, Perc Approach */
"0K9H3ZX"  =  "1"  /* Drainage of Right Thorax Muscle, Perc Approach, Diagn */
"0K9H40Z"  =  "1"  /* Drain of R Thorax Muscle with Drain Dev, Perc Endo Approach */
"0K9H4ZX"  =  "1"  /* Drainage of Right Thorax Muscle, Perc Endo Approach, Diagn */
"0K9H4ZZ"  =  "1"  /* Drainage of Right Thorax Muscle, Perc Endo Approach */
"0K9J00Z"  =  "1"  /* Drainage of Left Thorax Muscle with Drain Dev, Open Approach */
"0K9J0ZX"  =  "1"  /* Drainage of Left Thorax Muscle, Open Approach, Diagnostic */
"0K9J0ZZ"  =  "1"  /* Drainage of Left Thorax Muscle, Open Approach */
"0K9J30Z"  =  "1"  /* Drainage of Left Thorax Muscle with Drain Dev, Perc Approach */
"0K9J3ZX"  =  "1"  /* Drainage of Left Thorax Muscle, Percutaneous Approach, Diagn */
"0K9J40Z"  =  "1"  /* Drain of L Thorax Muscle with Drain Dev, Perc Endo Approach */
"0K9J4ZX"  =  "1"  /* Drainage of Left Thorax Muscle, Perc Endo Approach, Diagn */
"0K9J4ZZ"  =  "1"  /* Drainage of Left Thorax Muscle, Perc Endo Approach */
"0K9K00Z"  =  "1"  /* Drainage of R Abd Muscle with Drain Dev, Open Approach */
"0K9K0ZX"  =  "1"  /* Drainage of Right Abdomen Muscle, Open Approach, Diagnostic */
"0K9K0ZZ"  =  "1"  /* Drainage of Right Abdomen Muscle, Open Approach */
"0K9K30Z"  =  "1"  /* Drainage of R Abd Muscle with Drain Dev, Perc Approach */
"0K9K3ZX"  =  "1"  /* Drainage of Right Abdomen Muscle, Perc Approach, Diagn */
"0K9K40Z"  =  "1"  /* Drainage of R Abd Muscle with Drain Dev, Perc Endo Approach */
"0K9K4ZX"  =  "1"  /* Drainage of Right Abdomen Muscle, Perc Endo Approach, Diagn */
"0K9K4ZZ"  =  "1"  /* Drainage of Right Abdomen Muscle, Perc Endo Approach */
"0K9L00Z"  =  "1"  /* Drainage of L Abd Muscle with Drain Dev, Open Approach */
"0K9L0ZX"  =  "1"  /* Drainage of Left Abdomen Muscle, Open Approach, Diagnostic */
"0K9L0ZZ"  =  "1"  /* Drainage of Left Abdomen Muscle, Open Approach */
"0K9L30Z"  =  "1"  /* Drainage of L Abd Muscle with Drain Dev, Perc Approach */
"0K9L3ZX"  =  "1"  /* Drainage of Left Abdomen Muscle, Perc Approach, Diagn */
"0K9L40Z"  =  "1"  /* Drainage of L Abd Muscle with Drain Dev, Perc Endo Approach */
"0K9L4ZX"  =  "1"  /* Drainage of Left Abdomen Muscle, Perc Endo Approach, Diagn */
"0K9L4ZZ"  =  "1"  /* Drainage of Left Abdomen Muscle, Perc Endo Approach */
"0K9M00Z"  =  "1"  /* Drainage of Perineum Muscle with Drain Dev, Open Approach */
"0K9M0ZX"  =  "1"  /* Drainage of Perineum Muscle, Open Approach, Diagnostic */
"0K9M0ZZ"  =  "1"  /* Drainage of Perineum Muscle, Open Approach */
"0K9M30Z"  =  "1"  /* Drainage of Perineum Muscle with Drain Dev, Perc Approach */
"0K9M3ZX"  =  "1"  /* Drainage of Perineum Muscle, Percutaneous Approach, Diagn */
"0K9M40Z"  =  "1"  /* Drain of Perineum Muscle with Drain Dev, Perc Endo Approach */
"0K9M4ZX"  =  "1"  /* Drainage of Perineum Muscle, Perc Endo Approach, Diagn */
"0K9M4ZZ"  =  "1"  /* Drainage of Perineum Muscle, Perc Endo Approach */
"0K9N00Z"  =  "1"  /* Drainage of Right Hip Muscle with Drain Dev, Open Approach */
"0K9N0ZX"  =  "1"  /* Drainage of Right Hip Muscle, Open Approach, Diagnostic */
"0K9N0ZZ"  =  "1"  /* Drainage of Right Hip Muscle, Open Approach */
"0K9N30Z"  =  "1"  /* Drainage of Right Hip Muscle with Drain Dev, Perc Approach */
"0K9N3ZX"  =  "1"  /* Drainage of Right Hip Muscle, Percutaneous Approach, Diagn */
"0K9N40Z"  =  "1"  /* Drainage of R Hip Muscle with Drain Dev, Perc Endo Approach */
"0K9N4ZX"  =  "1"  /* Drainage of Right Hip Muscle, Perc Endo Approach, Diagn */
"0K9N4ZZ"  =  "1"  /* Drainage of Right Hip Muscle, Perc Endo Approach */
"0K9P00Z"  =  "1"  /* Drainage of Left Hip Muscle with Drain Dev, Open Approach */
"0K9P0ZX"  =  "1"  /* Drainage of Left Hip Muscle, Open Approach, Diagnostic */
"0K9P0ZZ"  =  "1"  /* Drainage of Left Hip Muscle, Open Approach */
"0K9P30Z"  =  "1"  /* Drainage of Left Hip Muscle with Drain Dev, Perc Approach */
"0K9P3ZX"  =  "1"  /* Drainage of Left Hip Muscle, Percutaneous Approach, Diagn */
"0K9P40Z"  =  "1"  /* Drainage of L Hip Muscle with Drain Dev, Perc Endo Approach */
"0K9P4ZX"  =  "1"  /* Drainage of Left Hip Muscle, Perc Endo Approach, Diagn */
"0K9P4ZZ"  =  "1"  /* Drainage of Left Hip Muscle, Perc Endo Approach */
"0K9Q00Z"  =  "1"  /* Drainage of R Up Leg Muscle with Drain Dev, Open Approach */
"0K9Q0ZX"  =  "1"  /* Drainage of Right Upper Leg Muscle, Open Approach, Diagn */
"0K9Q0ZZ"  =  "1"  /* Drainage of Right Upper Leg Muscle, Open Approach */
"0K9Q30Z"  =  "1"  /* Drainage of R Up Leg Muscle with Drain Dev, Perc Approach */
"0K9Q3ZX"  =  "1"  /* Drainage of Right Upper Leg Muscle, Perc Approach, Diagn */
"0K9Q40Z"  =  "1"  /* Drain of R Up Leg Muscle with Drain Dev, Perc Endo Approach */
"0K9Q4ZX"  =  "1"  /* Drainage of R Up Leg Muscle, Perc Endo Approach, Diagn */
"0K9Q4ZZ"  =  "1"  /* Drainage of Right Upper Leg Muscle, Perc Endo Approach */
"0K9R00Z"  =  "1"  /* Drainage of L Up Leg Muscle with Drain Dev, Open Approach */
"0K9R0ZX"  =  "1"  /* Drainage of Left Upper Leg Muscle, Open Approach, Diagnostic */
"0K9R0ZZ"  =  "1"  /* Drainage of Left Upper Leg Muscle, Open Approach */
"0K9R30Z"  =  "1"  /* Drainage of L Up Leg Muscle with Drain Dev, Perc Approach */
"0K9R3ZX"  =  "1"  /* Drainage of Left Upper Leg Muscle, Perc Approach, Diagn */
"0K9R40Z"  =  "1"  /* Drain of L Up Leg Muscle with Drain Dev, Perc Endo Approach */
"0K9R4ZX"  =  "1"  /* Drainage of Left Upper Leg Muscle, Perc Endo Approach, Diagn */
"0K9R4ZZ"  =  "1"  /* Drainage of Left Upper Leg Muscle, Perc Endo Approach */
"0K9S00Z"  =  "1"  /* Drainage of R Low Leg Muscle with Drain Dev, Open Approach */
"0K9S0ZX"  =  "1"  /* Drainage of Right Lower Leg Muscle, Open Approach, Diagn */
"0K9S0ZZ"  =  "1"  /* Drainage of Right Lower Leg Muscle, Open Approach */
"0K9S30Z"  =  "1"  /* Drainage of R Low Leg Muscle with Drain Dev, Perc Approach */
"0K9S3ZX"  =  "1"  /* Drainage of Right Lower Leg Muscle, Perc Approach, Diagn */
"0K9S40Z"  =  "1"  /* Drain of R Low Leg Muscle with Drain Dev, Perc Endo Approach */
"0K9S4ZX"  =  "1"  /* Drainage of R Low Leg Muscle, Perc Endo Approach, Diagn */
"0K9S4ZZ"  =  "1"  /* Drainage of Right Lower Leg Muscle, Perc Endo Approach */
"0K9T00Z"  =  "1"  /* Drainage of L Low Leg Muscle with Drain Dev, Open Approach */
"0K9T0ZX"  =  "1"  /* Drainage of Left Lower Leg Muscle, Open Approach, Diagnostic */
"0K9T0ZZ"  =  "1"  /* Drainage of Left Lower Leg Muscle, Open Approach */
"0K9T30Z"  =  "1"  /* Drainage of L Low Leg Muscle with Drain Dev, Perc Approach */
"0K9T3ZX"  =  "1"  /* Drainage of Left Lower Leg Muscle, Perc Approach, Diagn */
"0K9T40Z"  =  "1"  /* Drain of L Low Leg Muscle with Drain Dev, Perc Endo Approach */
"0K9T4ZX"  =  "1"  /* Drainage of Left Lower Leg Muscle, Perc Endo Approach, Diagn */
"0K9T4ZZ"  =  "1"  /* Drainage of Left Lower Leg Muscle, Perc Endo Approach */
"0K9V00Z"  =  "1"  /* Drainage of Right Foot Muscle with Drain Dev, Open Approach */
"0K9V0ZX"  =  "1"  /* Drainage of Right Foot Muscle, Open Approach, Diagnostic */
"0K9V0ZZ"  =  "1"  /* Drainage of Right Foot Muscle, Open Approach */
"0K9V30Z"  =  "1"  /* Drainage of Right Foot Muscle with Drain Dev, Perc Approach */
"0K9V3ZX"  =  "1"  /* Drainage of Right Foot Muscle, Percutaneous Approach, Diagn */
"0K9V40Z"  =  "1"  /* Drainage of R Foot Muscle with Drain Dev, Perc Endo Approach */
"0K9V4ZX"  =  "1"  /* Drainage of Right Foot Muscle, Perc Endo Approach, Diagn */
"0K9V4ZZ"  =  "1"  /* Drainage of Right Foot Muscle, Perc Endo Approach */
"0K9W00Z"  =  "1"  /* Drainage of Left Foot Muscle with Drain Dev, Open Approach */
"0K9W0ZX"  =  "1"  /* Drainage of Left Foot Muscle, Open Approach, Diagnostic */
"0K9W0ZZ"  =  "1"  /* Drainage of Left Foot Muscle, Open Approach */
"0K9W30Z"  =  "1"  /* Drainage of Left Foot Muscle with Drain Dev, Perc Approach */
"0K9W3ZX"  =  "1"  /* Drainage of Left Foot Muscle, Percutaneous Approach, Diagn */
"0K9W40Z"  =  "1"  /* Drainage of L Foot Muscle with Drain Dev, Perc Endo Approach */
"0K9W4ZX"  =  "1"  /* Drainage of Left Foot Muscle, Perc Endo Approach, Diagn */
"0K9W4ZZ"  =  "1"  /* Drainage of Left Foot Muscle, Perc Endo Approach */
"0KB00ZX"  =  "1"  /* Excision of Head Muscle, Open Approach, Diagnostic */
"0KB00ZZ"  =  "1"  /* Excision of Head Muscle, Open Approach */
"0KB03ZX"  =  "1"  /* Excision of Head Muscle, Percutaneous Approach, Diagnostic */
"0KB03ZZ"  =  "1"  /* Excision of Head Muscle, Percutaneous Approach */
"0KB04ZX"  =  "1"  /* Excision of Head Muscle, Perc Endo Approach, Diagn */
"0KB04ZZ"  =  "1"  /* Excision of Head Muscle, Percutaneous Endoscopic Approach */
"0KB10ZX"  =  "1"  /* Excision of Facial Muscle, Open Approach, Diagnostic */
"0KB10ZZ"  =  "1"  /* Excision of Facial Muscle, Open Approach */
"0KB13ZX"  =  "1"  /* Excision of Facial Muscle, Percutaneous Approach, Diagnostic */
"0KB13ZZ"  =  "1"  /* Excision of Facial Muscle, Percutaneous Approach */
"0KB14ZX"  =  "1"  /* Excision of Facial Muscle, Perc Endo Approach, Diagn */
"0KB14ZZ"  =  "1"  /* Excision of Facial Muscle, Percutaneous Endoscopic Approach */
"0KB20ZX"  =  "1"  /* Excision of Right Neck Muscle, Open Approach, Diagnostic */
"0KB20ZZ"  =  "1"  /* Excision of Right Neck Muscle, Open Approach */
"0KB23ZX"  =  "1"  /* Excision of Right Neck Muscle, Percutaneous Approach, Diagn */
"0KB23ZZ"  =  "1"  /* Excision of Right Neck Muscle, Percutaneous Approach */
"0KB24ZX"  =  "1"  /* Excision of Right Neck Muscle, Perc Endo Approach, Diagn */
"0KB24ZZ"  =  "1"  /* Excision of Right Neck Muscle, Perc Endo Approach */
"0KB30ZX"  =  "1"  /* Excision of Left Neck Muscle, Open Approach, Diagnostic */
"0KB30ZZ"  =  "1"  /* Excision of Left Neck Muscle, Open Approach */
"0KB33ZX"  =  "1"  /* Excision of Left Neck Muscle, Percutaneous Approach, Diagn */
"0KB33ZZ"  =  "1"  /* Excision of Left Neck Muscle, Percutaneous Approach */
"0KB34ZX"  =  "1"  /* Excision of Left Neck Muscle, Perc Endo Approach, Diagn */
"0KB34ZZ"  =  "1"  /* Excision of Left Neck Muscle, Perc Endo Approach */
"0KB40ZX"  =  "1"  /* Excision of Tongue/Palate/Phar Muscle, Open Approach, Diagn */
"0KB40ZZ"  =  "1"  /* Excision of Tongue, Palate, Pharynx Muscle, Open Approach */
"0KB43ZX"  =  "1"  /* Excision of Tongue/Palate/Phar Muscle, Perc Approach, Diagn */
"0KB43ZZ"  =  "1"  /* Excision of Tongue, Palate, Pharynx Muscle, Perc Approach */
"0KB44ZX"  =  "1"  /* Excise Tongue/Palate/Phar Muscle, Perc Endo, Diagn */
"0KB44ZZ"  =  "1"  /* Excision of Tongue/Palate/Phar Muscle, Perc Endo Approach */
"0KB50ZX"  =  "1"  /* Excision of Right Shoulder Muscle, Open Approach, Diagnostic */
"0KB50ZZ"  =  "1"  /* Excision of Right Shoulder Muscle, Open Approach */
"0KB53ZX"  =  "1"  /* Excision of Right Shoulder Muscle, Perc Approach, Diagn */
"0KB53ZZ"  =  "1"  /* Excision of Right Shoulder Muscle, Percutaneous Approach */
"0KB54ZX"  =  "1"  /* Excision of Right Shoulder Muscle, Perc Endo Approach, Diagn */
"0KB54ZZ"  =  "1"  /* Excision of Right Shoulder Muscle, Perc Endo Approach */
"0KB60ZX"  =  "1"  /* Excision of Left Shoulder Muscle, Open Approach, Diagnostic */
"0KB60ZZ"  =  "1"  /* Excision of Left Shoulder Muscle, Open Approach */
"0KB63ZX"  =  "1"  /* Excision of Left Shoulder Muscle, Perc Approach, Diagn */
"0KB63ZZ"  =  "1"  /* Excision of Left Shoulder Muscle, Percutaneous Approach */
"0KB64ZX"  =  "1"  /* Excision of Left Shoulder Muscle, Perc Endo Approach, Diagn */
"0KB64ZZ"  =  "1"  /* Excision of Left Shoulder Muscle, Perc Endo Approach */
"0KB70ZX"  =  "1"  /* Excision of Right Upper Arm Muscle, Open Approach, Diagn */
"0KB70ZZ"  =  "1"  /* Excision of Right Upper Arm Muscle, Open Approach */
"0KB73ZX"  =  "1"  /* Excision of Right Upper Arm Muscle, Perc Approach, Diagn */
"0KB73ZZ"  =  "1"  /* Excision of Right Upper Arm Muscle, Percutaneous Approach */
"0KB74ZX"  =  "1"  /* Excision of R Up Arm Muscle, Perc Endo Approach, Diagn */
"0KB74ZZ"  =  "1"  /* Excision of Right Upper Arm Muscle, Perc Endo Approach */
"0KB80ZX"  =  "1"  /* Excision of Left Upper Arm Muscle, Open Approach, Diagnostic */
"0KB80ZZ"  =  "1"  /* Excision of Left Upper Arm Muscle, Open Approach */
"0KB83ZX"  =  "1"  /* Excision of Left Upper Arm Muscle, Perc Approach, Diagn */
"0KB83ZZ"  =  "1"  /* Excision of Left Upper Arm Muscle, Percutaneous Approach */
"0KB84ZX"  =  "1"  /* Excision of Left Upper Arm Muscle, Perc Endo Approach, Diagn */
"0KB84ZZ"  =  "1"  /* Excision of Left Upper Arm Muscle, Perc Endo Approach */
"0KB90ZX"  =  "1"  /* Excision of R Low Arm & Wrist Muscle, Open Approach, Diagn */
"0KB90ZZ"  =  "1"  /* Excision of Right Lower Arm and Wrist Muscle, Open Approach */
"0KB93ZX"  =  "1"  /* Excision of R Low Arm & Wrist Muscle, Perc Approach, Diagn */
"0KB93ZZ"  =  "1"  /* Excision of Right Lower Arm and Wrist Muscle, Perc Approach */
"0KB94ZX"  =  "1"  /* Excise R Low Arm & Wrist Muscle, Perc Endo, Diagn */
"0KB94ZZ"  =  "1"  /* Excision of R Low Arm & Wrist Muscle, Perc Endo Approach */
"0KBB0ZX"  =  "1"  /* Excision of L Low Arm & Wrist Muscle, Open Approach, Diagn */
"0KBB0ZZ"  =  "1"  /* Excision of Left Lower Arm and Wrist Muscle, Open Approach */
"0KBB3ZX"  =  "1"  /* Excision of L Low Arm & Wrist Muscle, Perc Approach, Diagn */
"0KBB3ZZ"  =  "1"  /* Excision of Left Lower Arm and Wrist Muscle, Perc Approach */
"0KBB4ZX"  =  "1"  /* Excise L Low Arm & Wrist Muscle, Perc Endo, Diagn */
"0KBB4ZZ"  =  "1"  /* Excision of L Low Arm & Wrist Muscle, Perc Endo Approach */
"0KBC0ZX"  =  "1"  /* Excision of Right Hand Muscle, Open Approach, Diagnostic */
"0KBC0ZZ"  =  "1"  /* Excision of Right Hand Muscle, Open Approach */
"0KBC3ZX"  =  "1"  /* Excision of Right Hand Muscle, Percutaneous Approach, Diagn */
"0KBC3ZZ"  =  "1"  /* Excision of Right Hand Muscle, Percutaneous Approach */
"0KBC4ZX"  =  "1"  /* Excision of Right Hand Muscle, Perc Endo Approach, Diagn */
"0KBC4ZZ"  =  "1"  /* Excision of Right Hand Muscle, Perc Endo Approach */
"0KBD0ZX"  =  "1"  /* Excision of Left Hand Muscle, Open Approach, Diagnostic */
"0KBD0ZZ"  =  "1"  /* Excision of Left Hand Muscle, Open Approach */
"0KBD3ZX"  =  "1"  /* Excision of Left Hand Muscle, Percutaneous Approach, Diagn */
"0KBD3ZZ"  =  "1"  /* Excision of Left Hand Muscle, Percutaneous Approach */
"0KBD4ZX"  =  "1"  /* Excision of Left Hand Muscle, Perc Endo Approach, Diagn */
"0KBD4ZZ"  =  "1"  /* Excision of Left Hand Muscle, Perc Endo Approach */
"0KBF0ZX"  =  "1"  /* Excision of Right Trunk Muscle, Open Approach, Diagnostic */
"0KBF0ZZ"  =  "1"  /* Excision of Right Trunk Muscle, Open Approach */
"0KBF3ZX"  =  "1"  /* Excision of Right Trunk Muscle, Percutaneous Approach, Diagn */
"0KBF3ZZ"  =  "1"  /* Excision of Right Trunk Muscle, Percutaneous Approach */
"0KBF4ZX"  =  "1"  /* Excision of Right Trunk Muscle, Perc Endo Approach, Diagn */
"0KBF4ZZ"  =  "1"  /* Excision of Right Trunk Muscle, Perc Endo Approach */
"0KBG0ZX"  =  "1"  /* Excision of Left Trunk Muscle, Open Approach, Diagnostic */
"0KBG0ZZ"  =  "1"  /* Excision of Left Trunk Muscle, Open Approach */
"0KBG3ZX"  =  "1"  /* Excision of Left Trunk Muscle, Percutaneous Approach, Diagn */
"0KBG3ZZ"  =  "1"  /* Excision of Left Trunk Muscle, Percutaneous Approach */
"0KBG4ZX"  =  "1"  /* Excision of Left Trunk Muscle, Perc Endo Approach, Diagn */
"0KBG4ZZ"  =  "1"  /* Excision of Left Trunk Muscle, Perc Endo Approach */
"0KBH0ZX"  =  "1"  /* Excision of Right Thorax Muscle, Open Approach, Diagnostic */
"0KBH0ZZ"  =  "1"  /* Excision of Right Thorax Muscle, Open Approach */
"0KBH3ZX"  =  "1"  /* Excision of Right Thorax Muscle, Perc Approach, Diagn */
"0KBH3ZZ"  =  "1"  /* Excision of Right Thorax Muscle, Percutaneous Approach */
"0KBH4ZX"  =  "1"  /* Excision of Right Thorax Muscle, Perc Endo Approach, Diagn */
"0KBH4ZZ"  =  "1"  /* Excision of Right Thorax Muscle, Perc Endo Approach */
"0KBJ0ZX"  =  "1"  /* Excision of Left Thorax Muscle, Open Approach, Diagnostic */
"0KBJ0ZZ"  =  "1"  /* Excision of Left Thorax Muscle, Open Approach */
"0KBJ3ZX"  =  "1"  /* Excision of Left Thorax Muscle, Percutaneous Approach, Diagn */
"0KBJ3ZZ"  =  "1"  /* Excision of Left Thorax Muscle, Percutaneous Approach */
"0KBJ4ZX"  =  "1"  /* Excision of Left Thorax Muscle, Perc Endo Approach, Diagn */
"0KBJ4ZZ"  =  "1"  /* Excision of Left Thorax Muscle, Perc Endo Approach */
"0KBK0ZX"  =  "1"  /* Excision of Right Abdomen Muscle, Open Approach, Diagnostic */
"0KBK0ZZ"  =  "1"  /* Excision of Right Abdomen Muscle, Open Approach */
"0KBK3ZX"  =  "1"  /* Excision of Right Abdomen Muscle, Perc Approach, Diagn */
"0KBK3ZZ"  =  "1"  /* Excision of Right Abdomen Muscle, Percutaneous Approach */
"0KBK4ZX"  =  "1"  /* Excision of Right Abdomen Muscle, Perc Endo Approach, Diagn */
"0KBK4ZZ"  =  "1"  /* Excision of Right Abdomen Muscle, Perc Endo Approach */
"0KBL0ZX"  =  "1"  /* Excision of Left Abdomen Muscle, Open Approach, Diagnostic */
"0KBL0ZZ"  =  "1"  /* Excision of Left Abdomen Muscle, Open Approach */
"0KBL3ZX"  =  "1"  /* Excision of Left Abdomen Muscle, Perc Approach, Diagn */
"0KBL3ZZ"  =  "1"  /* Excision of Left Abdomen Muscle, Percutaneous Approach */
"0KBL4ZX"  =  "1"  /* Excision of Left Abdomen Muscle, Perc Endo Approach, Diagn */
"0KBL4ZZ"  =  "1"  /* Excision of Left Abdomen Muscle, Perc Endo Approach */
"0KBM0ZX"  =  "1"  /* Excision of Perineum Muscle, Open Approach, Diagnostic */
"0KBM0ZZ"  =  "1"  /* Excision of Perineum Muscle, Open Approach */
"0KBM3ZX"  =  "1"  /* Excision of Perineum Muscle, Percutaneous Approach, Diagn */
"0KBM3ZZ"  =  "1"  /* Excision of Perineum Muscle, Percutaneous Approach */
"0KBM4ZX"  =  "1"  /* Excision of Perineum Muscle, Perc Endo Approach, Diagn */
"0KBM4ZZ"  =  "1"  /* Excision of Perineum Muscle, Perc Endo Approach */
"0KBN0ZX"  =  "1"  /* Excision of Right Hip Muscle, Open Approach, Diagnostic */
"0KBN0ZZ"  =  "1"  /* Excision of Right Hip Muscle, Open Approach */
"0KBN3ZX"  =  "1"  /* Excision of Right Hip Muscle, Percutaneous Approach, Diagn */
"0KBN3ZZ"  =  "1"  /* Excision of Right Hip Muscle, Percutaneous Approach */
"0KBN4ZX"  =  "1"  /* Excision of Right Hip Muscle, Perc Endo Approach, Diagn */
"0KBN4ZZ"  =  "1"  /* Excision of Right Hip Muscle, Perc Endo Approach */
"0KBP0ZX"  =  "1"  /* Excision of Left Hip Muscle, Open Approach, Diagnostic */
"0KBP0ZZ"  =  "1"  /* Excision of Left Hip Muscle, Open Approach */
"0KBP3ZX"  =  "1"  /* Excision of Left Hip Muscle, Percutaneous Approach, Diagn */
"0KBP3ZZ"  =  "1"  /* Excision of Left Hip Muscle, Percutaneous Approach */
"0KBP4ZX"  =  "1"  /* Excision of Left Hip Muscle, Perc Endo Approach, Diagn */
"0KBP4ZZ"  =  "1"  /* Excision of Left Hip Muscle, Perc Endo Approach */
"0KBQ0ZX"  =  "1"  /* Excision of Right Upper Leg Muscle, Open Approach, Diagn */
"0KBQ0ZZ"  =  "1"  /* Excision of Right Upper Leg Muscle, Open Approach */
"0KBQ3ZX"  =  "1"  /* Excision of Right Upper Leg Muscle, Perc Approach, Diagn */
"0KBQ3ZZ"  =  "1"  /* Excision of Right Upper Leg Muscle, Percutaneous Approach */
"0KBQ4ZX"  =  "1"  /* Excision of R Up Leg Muscle, Perc Endo Approach, Diagn */
"0KBQ4ZZ"  =  "1"  /* Excision of Right Upper Leg Muscle, Perc Endo Approach */
"0KBR0ZX"  =  "1"  /* Excision of Left Upper Leg Muscle, Open Approach, Diagnostic */
"0KBR0ZZ"  =  "1"  /* Excision of Left Upper Leg Muscle, Open Approach */
"0KBR3ZX"  =  "1"  /* Excision of Left Upper Leg Muscle, Perc Approach, Diagn */
"0KBR3ZZ"  =  "1"  /* Excision of Left Upper Leg Muscle, Percutaneous Approach */
"0KBR4ZX"  =  "1"  /* Excision of Left Upper Leg Muscle, Perc Endo Approach, Diagn */
"0KBR4ZZ"  =  "1"  /* Excision of Left Upper Leg Muscle, Perc Endo Approach */
"0KBS0ZX"  =  "1"  /* Excision of Right Lower Leg Muscle, Open Approach, Diagn */
"0KBS0ZZ"  =  "1"  /* Excision of Right Lower Leg Muscle, Open Approach */
"0KBS3ZX"  =  "1"  /* Excision of Right Lower Leg Muscle, Perc Approach, Diagn */
"0KBS3ZZ"  =  "1"  /* Excision of Right Lower Leg Muscle, Percutaneous Approach */
"0KBS4ZX"  =  "1"  /* Excision of R Low Leg Muscle, Perc Endo Approach, Diagn */
"0KBS4ZZ"  =  "1"  /* Excision of Right Lower Leg Muscle, Perc Endo Approach */
"0KBT0ZX"  =  "1"  /* Excision of Left Lower Leg Muscle, Open Approach, Diagnostic */
"0KBT0ZZ"  =  "1"  /* Excision of Left Lower Leg Muscle, Open Approach */
"0KBT3ZX"  =  "1"  /* Excision of Left Lower Leg Muscle, Perc Approach, Diagn */
"0KBT3ZZ"  =  "1"  /* Excision of Left Lower Leg Muscle, Percutaneous Approach */
"0KBT4ZX"  =  "1"  /* Excision of Left Lower Leg Muscle, Perc Endo Approach, Diagn */
"0KBT4ZZ"  =  "1"  /* Excision of Left Lower Leg Muscle, Perc Endo Approach */
"0KBV0ZX"  =  "1"  /* Excision of Right Foot Muscle, Open Approach, Diagnostic */
"0KBV0ZZ"  =  "1"  /* Excision of Right Foot Muscle, Open Approach */
"0KBV3ZX"  =  "1"  /* Excision of Right Foot Muscle, Percutaneous Approach, Diagn */
"0KBV3ZZ"  =  "1"  /* Excision of Right Foot Muscle, Percutaneous Approach */
"0KBV4ZX"  =  "1"  /* Excision of Right Foot Muscle, Perc Endo Approach, Diagn */
"0KBV4ZZ"  =  "1"  /* Excision of Right Foot Muscle, Perc Endo Approach */
"0KBW0ZX"  =  "1"  /* Excision of Left Foot Muscle, Open Approach, Diagnostic */
"0KBW0ZZ"  =  "1"  /* Excision of Left Foot Muscle, Open Approach */
"0KBW3ZX"  =  "1"  /* Excision of Left Foot Muscle, Percutaneous Approach, Diagn */
"0KBW3ZZ"  =  "1"  /* Excision of Left Foot Muscle, Percutaneous Approach */
"0KBW4ZX"  =  "1"  /* Excision of Left Foot Muscle, Perc Endo Approach, Diagn */
"0KBW4ZZ"  =  "1"  /* Excision of Left Foot Muscle, Perc Endo Approach */
"0KC00ZZ"  =  "1"  /* Extirpation of Matter from Head Muscle, Open Approach */
"0KC03ZZ"  =  "1"  /* Extirpation of Matter from Head Muscle, Perc Approach */
"0KC04ZZ"  =  "1"  /* Extirpation of Matter from Head Muscle, Perc Endo Approach */
"0KC10ZZ"  =  "1"  /* Extirpation of Matter from Facial Muscle, Open Approach */
"0KC13ZZ"  =  "1"  /* Extirpation of Matter from Facial Muscle, Perc Approach */
"0KC14ZZ"  =  "1"  /* Extirpation of Matter from Facial Muscle, Perc Endo Approach */
"0KC20ZZ"  =  "1"  /* Extirpation of Matter from Right Neck Muscle, Open Approach */
"0KC23ZZ"  =  "1"  /* Extirpation of Matter from Right Neck Muscle, Perc Approach */
"0KC24ZZ"  =  "1"  /* Extirpation of Matter from R Neck Muscle, Perc Endo Approach */
"0KC30ZZ"  =  "1"  /* Extirpation of Matter from Left Neck Muscle, Open Approach */
"0KC33ZZ"  =  "1"  /* Extirpation of Matter from Left Neck Muscle, Perc Approach */
"0KC34ZZ"  =  "1"  /* Extirpation of Matter from L Neck Muscle, Perc Endo Approach */
"0KC40ZZ"  =  "1"  /* Extirpate matter from Tongue/Palate/Phar Muscle, Open */
"0KC43ZZ"  =  "1"  /* Extirpate matter from Tongue/Palate/Phar Muscle, Perc */
"0KC44ZZ"  =  "1"  /* Extirpate matter from Tongue/Palate/Phar Muscle, Perc Endo */
"0KC50ZZ"  =  "1"  /* Extirpation of Matter from R Shoulder Muscle, Open Approach */
"0KC53ZZ"  =  "1"  /* Extirpation of Matter from R Shoulder Muscle, Perc Approach */
"0KC54ZZ"  =  "1"  /* Extirpate matter from R Shoulder Muscle, Perc Endo */
"0KC60ZZ"  =  "1"  /* Extirpation of Matter from L Shoulder Muscle, Open Approach */
"0KC63ZZ"  =  "1"  /* Extirpation of Matter from L Shoulder Muscle, Perc Approach */
"0KC64ZZ"  =  "1"  /* Extirpate matter from L Shoulder Muscle, Perc Endo */
"0KC70ZZ"  =  "1"  /* Extirpation of Matter from R Up Arm Muscle, Open Approach */
"0KC73ZZ"  =  "1"  /* Extirpation of Matter from R Up Arm Muscle, Perc Approach */
"0KC74ZZ"  =  "1"  /* Extirpate of Matter from R Up Arm Muscle, Perc Endo Approach */
"0KC80ZZ"  =  "1"  /* Extirpation of Matter from L Up Arm Muscle, Open Approach */
"0KC83ZZ"  =  "1"  /* Extirpation of Matter from L Up Arm Muscle, Perc Approach */
"0KC84ZZ"  =  "1"  /* Extirpate of Matter from L Up Arm Muscle, Perc Endo Approach */
"0KC90ZZ"  =  "1"  /* Extirpate matter from R Low Arm & Wrist Muscle, Open */
"0KC93ZZ"  =  "1"  /* Extirpate matter from R Low Arm & Wrist Muscle, Perc */
"0KC94ZZ"  =  "1"  /* Extirpate matter from R Low Arm & Wrist Muscle, Perc Endo */
"0KCB0ZZ"  =  "1"  /* Extirpate matter from L Low Arm & Wrist Muscle, Open */
"0KCB3ZZ"  =  "1"  /* Extirpate matter from L Low Arm & Wrist Muscle, Perc */
"0KCB4ZZ"  =  "1"  /* Extirpate matter from L Low Arm & Wrist Muscle, Perc Endo */
"0KCC0ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Muscle, Open Approach */
"0KCC3ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Muscle, Perc Approach */
"0KCC4ZZ"  =  "1"  /* Extirpation of Matter from R Hand Muscle, Perc Endo Approach */
"0KCD0ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Muscle, Open Approach */
"0KCD3ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Muscle, Perc Approach */
"0KCD4ZZ"  =  "1"  /* Extirpation of Matter from L Hand Muscle, Perc Endo Approach */
"0KCF0ZZ"  =  "1"  /* Extirpation of Matter from Right Trunk Muscle, Open Approach */
"0KCF3ZZ"  =  "1"  /* Extirpation of Matter from Right Trunk Muscle, Perc Approach */
"0KCF4ZZ"  =  "1"  /* Extirpate of Matter from R Trunk Muscle, Perc Endo Approach */
"0KCG0ZZ"  =  "1"  /* Extirpation of Matter from Left Trunk Muscle, Open Approach */
"0KCG3ZZ"  =  "1"  /* Extirpation of Matter from Left Trunk Muscle, Perc Approach */
"0KCG4ZZ"  =  "1"  /* Extirpate of Matter from L Trunk Muscle, Perc Endo Approach */
"0KCH0ZZ"  =  "1"  /* Extirpation of Matter from R Thorax Muscle, Open Approach */
"0KCH3ZZ"  =  "1"  /* Extirpation of Matter from R Thorax Muscle, Perc Approach */
"0KCH4ZZ"  =  "1"  /* Extirpate of Matter from R Thorax Muscle, Perc Endo Approach */
"0KCJ0ZZ"  =  "1"  /* Extirpation of Matter from Left Thorax Muscle, Open Approach */
"0KCJ3ZZ"  =  "1"  /* Extirpation of Matter from Left Thorax Muscle, Perc Approach */
"0KCJ4ZZ"  =  "1"  /* Extirpate of Matter from L Thorax Muscle, Perc Endo Approach */
"0KCK0ZZ"  =  "1"  /* Extirpation of Matter from R Abd Muscle, Open Approach */
"0KCK3ZZ"  =  "1"  /* Extirpation of Matter from R Abd Muscle, Perc Approach */
"0KCK4ZZ"  =  "1"  /* Extirpation of Matter from R Abd Muscle, Perc Endo Approach */
"0KCL0ZZ"  =  "1"  /* Extirpation of Matter from L Abd Muscle, Open Approach */
"0KCL3ZZ"  =  "1"  /* Extirpation of Matter from L Abd Muscle, Perc Approach */
"0KCL4ZZ"  =  "1"  /* Extirpation of Matter from L Abd Muscle, Perc Endo Approach */
"0KCM0ZZ"  =  "1"  /* Extirpation of Matter from Perineum Muscle, Open Approach */
"0KCM3ZZ"  =  "1"  /* Extirpation of Matter from Perineum Muscle, Perc Approach */
"0KCM4ZZ"  =  "1"  /* Extirpate of Matter from Perineum Muscle, Perc Endo Approach */
"0KCN0ZZ"  =  "1"  /* Extirpation of Matter from Right Hip Muscle, Open Approach */
"0KCN3ZZ"  =  "1"  /* Extirpation of Matter from Right Hip Muscle, Perc Approach */
"0KCN4ZZ"  =  "1"  /* Extirpation of Matter from R Hip Muscle, Perc Endo Approach */
"0KCP0ZZ"  =  "1"  /* Extirpation of Matter from Left Hip Muscle, Open Approach */
"0KCP3ZZ"  =  "1"  /* Extirpation of Matter from Left Hip Muscle, Perc Approach */
"0KCP4ZZ"  =  "1"  /* Extirpation of Matter from L Hip Muscle, Perc Endo Approach */
"0KCQ0ZZ"  =  "1"  /* Extirpation of Matter from R Up Leg Muscle, Open Approach */
"0KCQ3ZZ"  =  "1"  /* Extirpation of Matter from R Up Leg Muscle, Perc Approach */
"0KCQ4ZZ"  =  "1"  /* Extirpate of Matter from R Up Leg Muscle, Perc Endo Approach */
"0KCR0ZZ"  =  "1"  /* Extirpation of Matter from L Up Leg Muscle, Open Approach */
"0KCR3ZZ"  =  "1"  /* Extirpation of Matter from L Up Leg Muscle, Perc Approach */
"0KCR4ZZ"  =  "1"  /* Extirpate of Matter from L Up Leg Muscle, Perc Endo Approach */
"0KCS0ZZ"  =  "1"  /* Extirpation of Matter from R Low Leg Muscle, Open Approach */
"0KCS3ZZ"  =  "1"  /* Extirpation of Matter from R Low Leg Muscle, Perc Approach */
"0KCS4ZZ"  =  "1"  /* Extirpate matter from R Low Leg Muscle, Perc Endo */
"0KCT0ZZ"  =  "1"  /* Extirpation of Matter from L Low Leg Muscle, Open Approach */
"0KCT3ZZ"  =  "1"  /* Extirpation of Matter from L Low Leg Muscle, Perc Approach */
"0KCT4ZZ"  =  "1"  /* Extirpate matter from L Low Leg Muscle, Perc Endo */
"0KCV0ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Muscle, Open Approach */
"0KCV3ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Muscle, Perc Approach */
"0KCV4ZZ"  =  "1"  /* Extirpation of Matter from R Foot Muscle, Perc Endo Approach */
"0KCW0ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Muscle, Open Approach */
"0KCW3ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Muscle, Perc Approach */
"0KCW4ZZ"  =  "1"  /* Extirpation of Matter from L Foot Muscle, Perc Endo Approach */
"0KHX0MZ"  =  "1"  /* Insertion of Stimulator Lead into Up Muscle, Open Approach */
"0KHX3MZ"  =  "1"  /* Insertion of Stimulator Lead into Up Muscle, Perc Approach */
"0KHX4MZ"  =  "1"  /* Insertion of Stim Lead into Up Muscle, Perc Endo Approach */
"0KHY0MZ"  =  "1"  /* Insertion of Stimulator Lead into Low Muscle, Open Approach */
"0KHY3MZ"  =  "1"  /* Insertion of Stimulator Lead into Low Muscle, Perc Approach */
"0KHY4MZ"  =  "1"  /* Insertion of Stim Lead into Low Muscle, Perc Endo Approach */
"0KJX0ZZ"  =  "1"  /* Inspection of Upper Muscle, Open Approach */
"0KJX3ZZ"  =  "1"  /* Inspection of Upper Muscle, Percutaneous Approach */
"0KJX4ZZ"  =  "1"  /* Inspection of Upper Muscle, Percutaneous Endoscopic Approach */
"0KJY0ZZ"  =  "1"  /* Inspection of Lower Muscle, Open Approach */
"0KJY3ZZ"  =  "1"  /* Inspection of Lower Muscle, Percutaneous Approach */
"0KJY4ZZ"  =  "1"  /* Inspection of Lower Muscle, Percutaneous Endoscopic Approach */
"0KM00ZZ"  =  "1"  /* Reattachment of Head Muscle, Open Approach */
"0KM04ZZ"  =  "1"  /* Reattachment of Head Muscle, Perc Endo Approach */
"0KM10ZZ"  =  "1"  /* Reattachment of Facial Muscle, Open Approach */
"0KM14ZZ"  =  "1"  /* Reattachment of Facial Muscle, Perc Endo Approach */
"0KM20ZZ"  =  "1"  /* Reattachment of Right Neck Muscle, Open Approach */
"0KM24ZZ"  =  "1"  /* Reattachment of Right Neck Muscle, Perc Endo Approach */
"0KM30ZZ"  =  "1"  /* Reattachment of Left Neck Muscle, Open Approach */
"0KM34ZZ"  =  "1"  /* Reattachment of Left Neck Muscle, Perc Endo Approach */
"0KM40ZZ"  =  "1"  /* Reattachment of Tongue/Palate/Phar Muscle, Open Approach */
"0KM44ZZ"  =  "1"  /* Reattach of Tongue/Palate/Phar Muscle, Perc Endo Approach */
"0KM50ZZ"  =  "1"  /* Reattachment of Right Shoulder Muscle, Open Approach */
"0KM54ZZ"  =  "1"  /* Reattachment of Right Shoulder Muscle, Perc Endo Approach */
"0KM60ZZ"  =  "1"  /* Reattachment of Left Shoulder Muscle, Open Approach */
"0KM64ZZ"  =  "1"  /* Reattachment of Left Shoulder Muscle, Perc Endo Approach */
"0KM70ZZ"  =  "1"  /* Reattachment of Right Upper Arm Muscle, Open Approach */
"0KM74ZZ"  =  "1"  /* Reattachment of Right Upper Arm Muscle, Perc Endo Approach */
"0KM80ZZ"  =  "1"  /* Reattachment of Left Upper Arm Muscle, Open Approach */
"0KM84ZZ"  =  "1"  /* Reattachment of Left Upper Arm Muscle, Perc Endo Approach */
"0KM90ZZ"  =  "1"  /* Reattachment of R Low Arm & Wrist Muscle, Open Approach */
"0KM94ZZ"  =  "1"  /* Reattachment of R Low Arm & Wrist Muscle, Perc Endo Approach */
"0KMB0ZZ"  =  "1"  /* Reattachment of L Low Arm & Wrist Muscle, Open Approach */
"0KMB4ZZ"  =  "1"  /* Reattachment of L Low Arm & Wrist Muscle, Perc Endo Approach */
"0KMC0ZZ"  =  "1"  /* Reattachment of Right Hand Muscle, Open Approach */
"0KMC4ZZ"  =  "1"  /* Reattachment of Right Hand Muscle, Perc Endo Approach */
"0KMD0ZZ"  =  "1"  /* Reattachment of Left Hand Muscle, Open Approach */
"0KMD4ZZ"  =  "1"  /* Reattachment of Left Hand Muscle, Perc Endo Approach */
"0KMF0ZZ"  =  "1"  /* Reattachment of Right Trunk Muscle, Open Approach */
"0KMF4ZZ"  =  "1"  /* Reattachment of Right Trunk Muscle, Perc Endo Approach */
"0KMG0ZZ"  =  "1"  /* Reattachment of Left Trunk Muscle, Open Approach */
"0KMG4ZZ"  =  "1"  /* Reattachment of Left Trunk Muscle, Perc Endo Approach */
"0KMH0ZZ"  =  "1"  /* Reattachment of Right Thorax Muscle, Open Approach */
"0KMH4ZZ"  =  "1"  /* Reattachment of Right Thorax Muscle, Perc Endo Approach */
"0KMJ0ZZ"  =  "1"  /* Reattachment of Left Thorax Muscle, Open Approach */
"0KMJ4ZZ"  =  "1"  /* Reattachment of Left Thorax Muscle, Perc Endo Approach */
"0KMK0ZZ"  =  "1"  /* Reattachment of Right Abdomen Muscle, Open Approach */
"0KMK4ZZ"  =  "1"  /* Reattachment of Right Abdomen Muscle, Perc Endo Approach */
"0KML0ZZ"  =  "1"  /* Reattachment of Left Abdomen Muscle, Open Approach */
"0KML4ZZ"  =  "1"  /* Reattachment of Left Abdomen Muscle, Perc Endo Approach */
"0KMM0ZZ"  =  "1"  /* Reattachment of Perineum Muscle, Open Approach */
"0KMM4ZZ"  =  "1"  /* Reattachment of Perineum Muscle, Perc Endo Approach */
"0KMN0ZZ"  =  "1"  /* Reattachment of Right Hip Muscle, Open Approach */
"0KMN4ZZ"  =  "1"  /* Reattachment of Right Hip Muscle, Perc Endo Approach */
"0KMP0ZZ"  =  "1"  /* Reattachment of Left Hip Muscle, Open Approach */
"0KMP4ZZ"  =  "1"  /* Reattachment of Left Hip Muscle, Perc Endo Approach */
"0KMQ0ZZ"  =  "1"  /* Reattachment of Right Upper Leg Muscle, Open Approach */
"0KMQ4ZZ"  =  "1"  /* Reattachment of Right Upper Leg Muscle, Perc Endo Approach */
"0KMR0ZZ"  =  "1"  /* Reattachment of Left Upper Leg Muscle, Open Approach */
"0KMR4ZZ"  =  "1"  /* Reattachment of Left Upper Leg Muscle, Perc Endo Approach */
"0KMS0ZZ"  =  "1"  /* Reattachment of Right Lower Leg Muscle, Open Approach */
"0KMS4ZZ"  =  "1"  /* Reattachment of Right Lower Leg Muscle, Perc Endo Approach */
"0KMT0ZZ"  =  "1"  /* Reattachment of Left Lower Leg Muscle, Open Approach */
"0KMT4ZZ"  =  "1"  /* Reattachment of Left Lower Leg Muscle, Perc Endo Approach */
"0KMV0ZZ"  =  "1"  /* Reattachment of Right Foot Muscle, Open Approach */
"0KMV4ZZ"  =  "1"  /* Reattachment of Right Foot Muscle, Perc Endo Approach */
"0KMW0ZZ"  =  "1"  /* Reattachment of Left Foot Muscle, Open Approach */
"0KMW4ZZ"  =  "1"  /* Reattachment of Left Foot Muscle, Perc Endo Approach */
"0KN00ZZ"  =  "1"  /* Release Head Muscle, Open Approach */
"0KN03ZZ"  =  "1"  /* Release Head Muscle, Percutaneous Approach */
"0KN04ZZ"  =  "1"  /* Release Head Muscle, Percutaneous Endoscopic Approach */
"0KN10ZZ"  =  "1"  /* Release Facial Muscle, Open Approach */
"0KN13ZZ"  =  "1"  /* Release Facial Muscle, Percutaneous Approach */
"0KN14ZZ"  =  "1"  /* Release Facial Muscle, Percutaneous Endoscopic Approach */
"0KN20ZZ"  =  "1"  /* Release Right Neck Muscle, Open Approach */
"0KN23ZZ"  =  "1"  /* Release Right Neck Muscle, Percutaneous Approach */
"0KN24ZZ"  =  "1"  /* Release Right Neck Muscle, Percutaneous Endoscopic Approach */
"0KN30ZZ"  =  "1"  /* Release Left Neck Muscle, Open Approach */
"0KN33ZZ"  =  "1"  /* Release Left Neck Muscle, Percutaneous Approach */
"0KN34ZZ"  =  "1"  /* Release Left Neck Muscle, Percutaneous Endoscopic Approach */
"0KN40ZZ"  =  "1"  /* Release Tongue, Palate, Pharynx Muscle, Open Approach */
"0KN43ZZ"  =  "1"  /* Release Tongue, Palate, Pharynx Muscle, Perc Approach */
"0KN44ZZ"  =  "1"  /* Release Tongue, Palate, Pharynx Muscle, Perc Endo Approach */
"0KN50ZZ"  =  "1"  /* Release Right Shoulder Muscle, Open Approach */
"0KN53ZZ"  =  "1"  /* Release Right Shoulder Muscle, Percutaneous Approach */
"0KN54ZZ"  =  "1"  /* Release Right Shoulder Muscle, Perc Endo Approach */
"0KN60ZZ"  =  "1"  /* Release Left Shoulder Muscle, Open Approach */
"0KN63ZZ"  =  "1"  /* Release Left Shoulder Muscle, Percutaneous Approach */
"0KN64ZZ"  =  "1"  /* Release Left Shoulder Muscle, Perc Endo Approach */
"0KN70ZZ"  =  "1"  /* Release Right Upper Arm Muscle, Open Approach */
"0KN73ZZ"  =  "1"  /* Release Right Upper Arm Muscle, Percutaneous Approach */
"0KN74ZZ"  =  "1"  /* Release Right Upper Arm Muscle, Perc Endo Approach */
"0KN80ZZ"  =  "1"  /* Release Left Upper Arm Muscle, Open Approach */
"0KN83ZZ"  =  "1"  /* Release Left Upper Arm Muscle, Percutaneous Approach */
"0KN84ZZ"  =  "1"  /* Release Left Upper Arm Muscle, Perc Endo Approach */
"0KN90ZZ"  =  "1"  /* Release Right Lower Arm and Wrist Muscle, Open Approach */
"0KN93ZZ"  =  "1"  /* Release Right Lower Arm and Wrist Muscle, Perc Approach */
"0KN94ZZ"  =  "1"  /* Release Right Lower Arm and Wrist Muscle, Perc Endo Approach */
"0KNB0ZZ"  =  "1"  /* Release Left Lower Arm and Wrist Muscle, Open Approach */
"0KNB3ZZ"  =  "1"  /* Release Left Lower Arm and Wrist Muscle, Perc Approach */
"0KNB4ZZ"  =  "1"  /* Release Left Lower Arm and Wrist Muscle, Perc Endo Approach */
"0KNC0ZZ"  =  "1"  /* Release Right Hand Muscle, Open Approach */
"0KNC3ZZ"  =  "1"  /* Release Right Hand Muscle, Percutaneous Approach */
"0KNC4ZZ"  =  "1"  /* Release Right Hand Muscle, Percutaneous Endoscopic Approach */
"0KND0ZZ"  =  "1"  /* Release Left Hand Muscle, Open Approach */
"0KND3ZZ"  =  "1"  /* Release Left Hand Muscle, Percutaneous Approach */
"0KND4ZZ"  =  "1"  /* Release Left Hand Muscle, Percutaneous Endoscopic Approach */
"0KNF0ZZ"  =  "1"  /* Release Right Trunk Muscle, Open Approach */
"0KNF3ZZ"  =  "1"  /* Release Right Trunk Muscle, Percutaneous Approach */
"0KNF4ZZ"  =  "1"  /* Release Right Trunk Muscle, Percutaneous Endoscopic Approach */
"0KNG0ZZ"  =  "1"  /* Release Left Trunk Muscle, Open Approach */
"0KNG3ZZ"  =  "1"  /* Release Left Trunk Muscle, Percutaneous Approach */
"0KNG4ZZ"  =  "1"  /* Release Left Trunk Muscle, Percutaneous Endoscopic Approach */
"0KNH0ZZ"  =  "1"  /* Release Right Thorax Muscle, Open Approach */
"0KNH3ZZ"  =  "1"  /* Release Right Thorax Muscle, Percutaneous Approach */
"0KNH4ZZ"  =  "1"  /* Release Right Thorax Muscle, Perc Endo Approach */
"0KNJ0ZZ"  =  "1"  /* Release Left Thorax Muscle, Open Approach */
"0KNJ3ZZ"  =  "1"  /* Release Left Thorax Muscle, Percutaneous Approach */
"0KNJ4ZZ"  =  "1"  /* Release Left Thorax Muscle, Percutaneous Endoscopic Approach */
"0KNK0ZZ"  =  "1"  /* Release Right Abdomen Muscle, Open Approach */
"0KNK3ZZ"  =  "1"  /* Release Right Abdomen Muscle, Percutaneous Approach */
"0KNK4ZZ"  =  "1"  /* Release Right Abdomen Muscle, Perc Endo Approach */
"0KNL0ZZ"  =  "1"  /* Release Left Abdomen Muscle, Open Approach */
"0KNL3ZZ"  =  "1"  /* Release Left Abdomen Muscle, Percutaneous Approach */
"0KNL4ZZ"  =  "1"  /* Release Left Abdomen Muscle, Perc Endo Approach */
"0KNM0ZZ"  =  "1"  /* Release Perineum Muscle, Open Approach */
"0KNM3ZZ"  =  "1"  /* Release Perineum Muscle, Percutaneous Approach */
"0KNM4ZZ"  =  "1"  /* Release Perineum Muscle, Percutaneous Endoscopic Approach */
"0KNN0ZZ"  =  "1"  /* Release Right Hip Muscle, Open Approach */
"0KNN3ZZ"  =  "1"  /* Release Right Hip Muscle, Percutaneous Approach */
"0KNN4ZZ"  =  "1"  /* Release Right Hip Muscle, Percutaneous Endoscopic Approach */
"0KNP0ZZ"  =  "1"  /* Release Left Hip Muscle, Open Approach */
"0KNP3ZZ"  =  "1"  /* Release Left Hip Muscle, Percutaneous Approach */
"0KNP4ZZ"  =  "1"  /* Release Left Hip Muscle, Percutaneous Endoscopic Approach */
"0KNQ0ZZ"  =  "1"  /* Release Right Upper Leg Muscle, Open Approach */
"0KNQ3ZZ"  =  "1"  /* Release Right Upper Leg Muscle, Percutaneous Approach */
"0KNQ4ZZ"  =  "1"  /* Release Right Upper Leg Muscle, Perc Endo Approach */
"0KNR0ZZ"  =  "1"  /* Release Left Upper Leg Muscle, Open Approach */
"0KNR3ZZ"  =  "1"  /* Release Left Upper Leg Muscle, Percutaneous Approach */
"0KNR4ZZ"  =  "1"  /* Release Left Upper Leg Muscle, Perc Endo Approach */
"0KNS0ZZ"  =  "1"  /* Release Right Lower Leg Muscle, Open Approach */
"0KNS3ZZ"  =  "1"  /* Release Right Lower Leg Muscle, Percutaneous Approach */
"0KNS4ZZ"  =  "1"  /* Release Right Lower Leg Muscle, Perc Endo Approach */
"0KNT0ZZ"  =  "1"  /* Release Left Lower Leg Muscle, Open Approach */
"0KNT3ZZ"  =  "1"  /* Release Left Lower Leg Muscle, Percutaneous Approach */
"0KNT4ZZ"  =  "1"  /* Release Left Lower Leg Muscle, Perc Endo Approach */
"0KNV0ZZ"  =  "1"  /* Release Right Foot Muscle, Open Approach */
"0KNV3ZZ"  =  "1"  /* Release Right Foot Muscle, Percutaneous Approach */
"0KNV4ZZ"  =  "1"  /* Release Right Foot Muscle, Percutaneous Endoscopic Approach */
"0KNW0ZZ"  =  "1"  /* Release Left Foot Muscle, Open Approach */
"0KNW3ZZ"  =  "1"  /* Release Left Foot Muscle, Percutaneous Approach */
"0KNW4ZZ"  =  "1"  /* Release Left Foot Muscle, Percutaneous Endoscopic Approach */
"0KPX00Z"  =  "1"  /* Removal of Drainage Device from Upper Muscle, Open Approach */
"0KPX07Z"  =  "1"  /* Removal of Autol Sub from Up Muscle, Open Approach */
"0KPX0JZ"  =  "1"  /* Removal of Synth Sub from Up Muscle, Open Approach */
"0KPX0KZ"  =  "1"  /* Removal of Nonaut Sub from Up Muscle, Open Approach */
"0KPX0MZ"  =  "1"  /* Removal of Stimulator Lead from Upper Muscle, Open Approach */
"0KPX30Z"  =  "1"  /* Removal of Drainage Device from Upper Muscle, Perc Approach */
"0KPX37Z"  =  "1"  /* Removal of Autol Sub from Up Muscle, Perc Approach */
"0KPX3JZ"  =  "1"  /* Removal of Synth Sub from Up Muscle, Perc Approach */
"0KPX3KZ"  =  "1"  /* Removal of Nonaut Sub from Up Muscle, Perc Approach */
"0KPX3MZ"  =  "1"  /* Removal of Stimulator Lead from Upper Muscle, Perc Approach */
"0KPX40Z"  =  "1"  /* Removal of Drain Dev from Up Muscle, Perc Endo Approach */
"0KPX47Z"  =  "1"  /* Removal of Autol Sub from Up Muscle, Perc Endo Approach */
"0KPX4JZ"  =  "1"  /* Removal of Synth Sub from Up Muscle, Perc Endo Approach */
"0KPX4KZ"  =  "1"  /* Removal of Nonaut Sub from Up Muscle, Perc Endo Approach */
"0KPX4MZ"  =  "1"  /* Removal of Stim Lead from Up Muscle, Perc Endo Approach */
"0KPY00Z"  =  "1"  /* Removal of Drainage Device from Lower Muscle, Open Approach */
"0KPY07Z"  =  "1"  /* Removal of Autol Sub from Low Muscle, Open Approach */
"0KPY0JZ"  =  "1"  /* Removal of Synth Sub from Low Muscle, Open Approach */
"0KPY0KZ"  =  "1"  /* Removal of Nonaut Sub from Low Muscle, Open Approach */
"0KPY0MZ"  =  "1"  /* Removal of Stimulator Lead from Lower Muscle, Open Approach */
"0KPY30Z"  =  "1"  /* Removal of Drainage Device from Lower Muscle, Perc Approach */
"0KPY37Z"  =  "1"  /* Removal of Autol Sub from Low Muscle, Perc Approach */
"0KPY3JZ"  =  "1"  /* Removal of Synth Sub from Low Muscle, Perc Approach */
"0KPY3KZ"  =  "1"  /* Removal of Nonaut Sub from Low Muscle, Perc Approach */
"0KPY3MZ"  =  "1"  /* Removal of Stimulator Lead from Lower Muscle, Perc Approach */
"0KPY40Z"  =  "1"  /* Removal of Drain Dev from Low Muscle, Perc Endo Approach */
"0KPY47Z"  =  "1"  /* Removal of Autol Sub from Low Muscle, Perc Endo Approach */
"0KPY4JZ"  =  "1"  /* Removal of Synth Sub from Low Muscle, Perc Endo Approach */
"0KPY4KZ"  =  "1"  /* Removal of Nonaut Sub from Low Muscle, Perc Endo Approach */
"0KPY4MZ"  =  "1"  /* Removal of Stim Lead from Low Muscle, Perc Endo Approach */
"0KQ00ZZ"  =  "1"  /* Repair Head Muscle, Open Approach */
"0KQ03ZZ"  =  "1"  /* Repair Head Muscle, Percutaneous Approach */
"0KQ04ZZ"  =  "1"  /* Repair Head Muscle, Percutaneous Endoscopic Approach */
"0KQ10ZZ"  =  "1"  /* Repair Facial Muscle, Open Approach */
"0KQ13ZZ"  =  "1"  /* Repair Facial Muscle, Percutaneous Approach */
"0KQ14ZZ"  =  "1"  /* Repair Facial Muscle, Percutaneous Endoscopic Approach */
"0KQ20ZZ"  =  "1"  /* Repair Right Neck Muscle, Open Approach */
"0KQ23ZZ"  =  "1"  /* Repair Right Neck Muscle, Percutaneous Approach */
"0KQ24ZZ"  =  "1"  /* Repair Right Neck Muscle, Percutaneous Endoscopic Approach */
"0KQ30ZZ"  =  "1"  /* Repair Left Neck Muscle, Open Approach */
"0KQ33ZZ"  =  "1"  /* Repair Left Neck Muscle, Percutaneous Approach */
"0KQ34ZZ"  =  "1"  /* Repair Left Neck Muscle, Percutaneous Endoscopic Approach */
"0KQ40ZZ"  =  "1"  /* Repair Tongue, Palate, Pharynx Muscle, Open Approach */
"0KQ43ZZ"  =  "1"  /* Repair Tongue, Palate, Pharynx Muscle, Percutaneous Approach */
"0KQ44ZZ"  =  "1"  /* Repair Tongue, Palate, Pharynx Muscle, Perc Endo Approach */
"0KQ50ZZ"  =  "1"  /* Repair Right Shoulder Muscle, Open Approach */
"0KQ53ZZ"  =  "1"  /* Repair Right Shoulder Muscle, Percutaneous Approach */
"0KQ54ZZ"  =  "1"  /* Repair Right Shoulder Muscle, Perc Endo Approach */
"0KQ60ZZ"  =  "1"  /* Repair Left Shoulder Muscle, Open Approach */
"0KQ63ZZ"  =  "1"  /* Repair Left Shoulder Muscle, Percutaneous Approach */
"0KQ64ZZ"  =  "1"  /* Repair Left Shoulder Muscle, Perc Endo Approach */
"0KQ70ZZ"  =  "1"  /* Repair Right Upper Arm Muscle, Open Approach */
"0KQ73ZZ"  =  "1"  /* Repair Right Upper Arm Muscle, Percutaneous Approach */
"0KQ74ZZ"  =  "1"  /* Repair Right Upper Arm Muscle, Perc Endo Approach */
"0KQ80ZZ"  =  "1"  /* Repair Left Upper Arm Muscle, Open Approach */
"0KQ83ZZ"  =  "1"  /* Repair Left Upper Arm Muscle, Percutaneous Approach */
"0KQ84ZZ"  =  "1"  /* Repair Left Upper Arm Muscle, Perc Endo Approach */
"0KQ90ZZ"  =  "1"  /* Repair Right Lower Arm and Wrist Muscle, Open Approach */
"0KQ93ZZ"  =  "1"  /* Repair Right Lower Arm and Wrist Muscle, Perc Approach */
"0KQ94ZZ"  =  "1"  /* Repair Right Lower Arm and Wrist Muscle, Perc Endo Approach */
"0KQB0ZZ"  =  "1"  /* Repair Left Lower Arm and Wrist Muscle, Open Approach */
"0KQB3ZZ"  =  "1"  /* Repair Left Lower Arm and Wrist Muscle, Perc Approach */
"0KQB4ZZ"  =  "1"  /* Repair Left Lower Arm and Wrist Muscle, Perc Endo Approach */
"0KQC0ZZ"  =  "1"  /* Repair Right Hand Muscle, Open Approach */
"0KQC3ZZ"  =  "1"  /* Repair Right Hand Muscle, Percutaneous Approach */
"0KQC4ZZ"  =  "1"  /* Repair Right Hand Muscle, Percutaneous Endoscopic Approach */
"0KQD0ZZ"  =  "1"  /* Repair Left Hand Muscle, Open Approach */
"0KQD3ZZ"  =  "1"  /* Repair Left Hand Muscle, Percutaneous Approach */
"0KQD4ZZ"  =  "1"  /* Repair Left Hand Muscle, Percutaneous Endoscopic Approach */
"0KQF0ZZ"  =  "1"  /* Repair Right Trunk Muscle, Open Approach */
"0KQF3ZZ"  =  "1"  /* Repair Right Trunk Muscle, Percutaneous Approach */
"0KQF4ZZ"  =  "1"  /* Repair Right Trunk Muscle, Percutaneous Endoscopic Approach */
"0KQG0ZZ"  =  "1"  /* Repair Left Trunk Muscle, Open Approach */
"0KQG3ZZ"  =  "1"  /* Repair Left Trunk Muscle, Percutaneous Approach */
"0KQG4ZZ"  =  "1"  /* Repair Left Trunk Muscle, Percutaneous Endoscopic Approach */
"0KQH0ZZ"  =  "1"  /* Repair Right Thorax Muscle, Open Approach */
"0KQH3ZZ"  =  "1"  /* Repair Right Thorax Muscle, Percutaneous Approach */
"0KQH4ZZ"  =  "1"  /* Repair Right Thorax Muscle, Percutaneous Endoscopic Approach */
"0KQJ0ZZ"  =  "1"  /* Repair Left Thorax Muscle, Open Approach */
"0KQJ3ZZ"  =  "1"  /* Repair Left Thorax Muscle, Percutaneous Approach */
"0KQJ4ZZ"  =  "1"  /* Repair Left Thorax Muscle, Percutaneous Endoscopic Approach */
"0KQK0ZZ"  =  "1"  /* Repair Right Abdomen Muscle, Open Approach */
"0KQK3ZZ"  =  "1"  /* Repair Right Abdomen Muscle, Percutaneous Approach */
"0KQK4ZZ"  =  "1"  /* Repair Right Abdomen Muscle, Perc Endo Approach */
"0KQL0ZZ"  =  "1"  /* Repair Left Abdomen Muscle, Open Approach */
"0KQL3ZZ"  =  "1"  /* Repair Left Abdomen Muscle, Percutaneous Approach */
"0KQL4ZZ"  =  "1"  /* Repair Left Abdomen Muscle, Percutaneous Endoscopic Approach */
"0KQM0ZZ"  =  "1"  /* Repair Perineum Muscle, Open Approach */
"0KQM3ZZ"  =  "1"  /* Repair Perineum Muscle, Percutaneous Approach */
"0KQM4ZZ"  =  "1"  /* Repair Perineum Muscle, Percutaneous Endoscopic Approach */
"0KQN0ZZ"  =  "1"  /* Repair Right Hip Muscle, Open Approach */
"0KQN3ZZ"  =  "1"  /* Repair Right Hip Muscle, Percutaneous Approach */
"0KQN4ZZ"  =  "1"  /* Repair Right Hip Muscle, Percutaneous Endoscopic Approach */
"0KQP0ZZ"  =  "1"  /* Repair Left Hip Muscle, Open Approach */
"0KQP3ZZ"  =  "1"  /* Repair Left Hip Muscle, Percutaneous Approach */
"0KQP4ZZ"  =  "1"  /* Repair Left Hip Muscle, Percutaneous Endoscopic Approach */
"0KQQ0ZZ"  =  "1"  /* Repair Right Upper Leg Muscle, Open Approach */
"0KQQ3ZZ"  =  "1"  /* Repair Right Upper Leg Muscle, Percutaneous Approach */
"0KQQ4ZZ"  =  "1"  /* Repair Right Upper Leg Muscle, Perc Endo Approach */
"0KQR0ZZ"  =  "1"  /* Repair Left Upper Leg Muscle, Open Approach */
"0KQR3ZZ"  =  "1"  /* Repair Left Upper Leg Muscle, Percutaneous Approach */
"0KQR4ZZ"  =  "1"  /* Repair Left Upper Leg Muscle, Perc Endo Approach */
"0KQS0ZZ"  =  "1"  /* Repair Right Lower Leg Muscle, Open Approach */
"0KQS3ZZ"  =  "1"  /* Repair Right Lower Leg Muscle, Percutaneous Approach */
"0KQS4ZZ"  =  "1"  /* Repair Right Lower Leg Muscle, Perc Endo Approach */
"0KQT0ZZ"  =  "1"  /* Repair Left Lower Leg Muscle, Open Approach */
"0KQT3ZZ"  =  "1"  /* Repair Left Lower Leg Muscle, Percutaneous Approach */
"0KQT4ZZ"  =  "1"  /* Repair Left Lower Leg Muscle, Perc Endo Approach */
"0KQV0ZZ"  =  "1"  /* Repair Right Foot Muscle, Open Approach */
"0KQV3ZZ"  =  "1"  /* Repair Right Foot Muscle, Percutaneous Approach */
"0KQV4ZZ"  =  "1"  /* Repair Right Foot Muscle, Percutaneous Endoscopic Approach */
"0KQW0ZZ"  =  "1"  /* Repair Left Foot Muscle, Open Approach */
"0KQW3ZZ"  =  "1"  /* Repair Left Foot Muscle, Percutaneous Approach */
"0KQW4ZZ"  =  "1"  /* Repair Left Foot Muscle, Percutaneous Endoscopic Approach */
"0KS00ZZ"  =  "1"  /* Reposition Head Muscle, Open Approach */
"0KS04ZZ"  =  "1"  /* Reposition Head Muscle, Percutaneous Endoscopic Approach */
"0KS10ZZ"  =  "1"  /* Reposition Facial Muscle, Open Approach */
"0KS14ZZ"  =  "1"  /* Reposition Facial Muscle, Percutaneous Endoscopic Approach */
"0KS20ZZ"  =  "1"  /* Reposition Right Neck Muscle, Open Approach */
"0KS24ZZ"  =  "1"  /* Reposition Right Neck Muscle, Perc Endo Approach */
"0KS30ZZ"  =  "1"  /* Reposition Left Neck Muscle, Open Approach */
"0KS34ZZ"  =  "1"  /* Reposition Left Neck Muscle, Perc Endo Approach */
"0KS40ZZ"  =  "1"  /* Reposition Tongue, Palate, Pharynx Muscle, Open Approach */
"0KS44ZZ"  =  "1"  /* Reposition Tongue/Palate/Phar Muscle, Perc Endo Approach */
"0KS50ZZ"  =  "1"  /* Reposition Right Shoulder Muscle, Open Approach */
"0KS54ZZ"  =  "1"  /* Reposition Right Shoulder Muscle, Perc Endo Approach */
"0KS60ZZ"  =  "1"  /* Reposition Left Shoulder Muscle, Open Approach */
"0KS64ZZ"  =  "1"  /* Reposition Left Shoulder Muscle, Perc Endo Approach */
"0KS70ZZ"  =  "1"  /* Reposition Right Upper Arm Muscle, Open Approach */
"0KS74ZZ"  =  "1"  /* Reposition Right Upper Arm Muscle, Perc Endo Approach */
"0KS80ZZ"  =  "1"  /* Reposition Left Upper Arm Muscle, Open Approach */
"0KS84ZZ"  =  "1"  /* Reposition Left Upper Arm Muscle, Perc Endo Approach */
"0KS90ZZ"  =  "1"  /* Reposition Right Lower Arm and Wrist Muscle, Open Approach */
"0KS94ZZ"  =  "1"  /* Reposition R Low Arm & Wrist Muscle, Perc Endo Approach */
"0KSB0ZZ"  =  "1"  /* Reposition Left Lower Arm and Wrist Muscle, Open Approach */
"0KSB4ZZ"  =  "1"  /* Reposition L Low Arm & Wrist Muscle, Perc Endo Approach */
"0KSC0ZZ"  =  "1"  /* Reposition Right Hand Muscle, Open Approach */
"0KSC4ZZ"  =  "1"  /* Reposition Right Hand Muscle, Perc Endo Approach */
"0KSD0ZZ"  =  "1"  /* Reposition Left Hand Muscle, Open Approach */
"0KSD4ZZ"  =  "1"  /* Reposition Left Hand Muscle, Perc Endo Approach */
"0KSF0ZZ"  =  "1"  /* Reposition Right Trunk Muscle, Open Approach */
"0KSF4ZZ"  =  "1"  /* Reposition Right Trunk Muscle, Perc Endo Approach */
"0KSG0ZZ"  =  "1"  /* Reposition Left Trunk Muscle, Open Approach */
"0KSG4ZZ"  =  "1"  /* Reposition Left Trunk Muscle, Perc Endo Approach */
"0KSH0ZZ"  =  "1"  /* Reposition Right Thorax Muscle, Open Approach */
"0KSH4ZZ"  =  "1"  /* Reposition Right Thorax Muscle, Perc Endo Approach */
"0KSJ0ZZ"  =  "1"  /* Reposition Left Thorax Muscle, Open Approach */
"0KSJ4ZZ"  =  "1"  /* Reposition Left Thorax Muscle, Perc Endo Approach */
"0KSK0ZZ"  =  "1"  /* Reposition Right Abdomen Muscle, Open Approach */
"0KSK4ZZ"  =  "1"  /* Reposition Right Abdomen Muscle, Perc Endo Approach */
"0KSL0ZZ"  =  "1"  /* Reposition Left Abdomen Muscle, Open Approach */
"0KSL4ZZ"  =  "1"  /* Reposition Left Abdomen Muscle, Perc Endo Approach */
"0KSM0ZZ"  =  "1"  /* Reposition Perineum Muscle, Open Approach */
"0KSM4ZZ"  =  "1"  /* Reposition Perineum Muscle, Percutaneous Endoscopic Approach */
"0KSN0ZZ"  =  "1"  /* Reposition Right Hip Muscle, Open Approach */
"0KSN4ZZ"  =  "1"  /* Reposition Right Hip Muscle, Perc Endo Approach */
"0KSP0ZZ"  =  "1"  /* Reposition Left Hip Muscle, Open Approach */
"0KSP4ZZ"  =  "1"  /* Reposition Left Hip Muscle, Percutaneous Endoscopic Approach */
"0KSQ0ZZ"  =  "1"  /* Reposition Right Upper Leg Muscle, Open Approach */
"0KSQ4ZZ"  =  "1"  /* Reposition Right Upper Leg Muscle, Perc Endo Approach */
"0KSR0ZZ"  =  "1"  /* Reposition Left Upper Leg Muscle, Open Approach */
"0KSR4ZZ"  =  "1"  /* Reposition Left Upper Leg Muscle, Perc Endo Approach */
"0KSS0ZZ"  =  "1"  /* Reposition Right Lower Leg Muscle, Open Approach */
"0KSS4ZZ"  =  "1"  /* Reposition Right Lower Leg Muscle, Perc Endo Approach */
"0KST0ZZ"  =  "1"  /* Reposition Left Lower Leg Muscle, Open Approach */
"0KST4ZZ"  =  "1"  /* Reposition Left Lower Leg Muscle, Perc Endo Approach */
"0KSV0ZZ"  =  "1"  /* Reposition Right Foot Muscle, Open Approach */
"0KSV4ZZ"  =  "1"  /* Reposition Right Foot Muscle, Perc Endo Approach */
"0KSW0ZZ"  =  "1"  /* Reposition Left Foot Muscle, Open Approach */
"0KSW4ZZ"  =  "1"  /* Reposition Left Foot Muscle, Perc Endo Approach */
"0KT00ZZ"  =  "1"  /* Resection of Head Muscle, Open Approach */
"0KT04ZZ"  =  "1"  /* Resection of Head Muscle, Percutaneous Endoscopic Approach */
"0KT10ZZ"  =  "1"  /* Resection of Facial Muscle, Open Approach */
"0KT14ZZ"  =  "1"  /* Resection of Facial Muscle, Percutaneous Endoscopic Approach */
"0KT20ZZ"  =  "1"  /* Resection of Right Neck Muscle, Open Approach */
"0KT24ZZ"  =  "1"  /* Resection of Right Neck Muscle, Perc Endo Approach */
"0KT30ZZ"  =  "1"  /* Resection of Left Neck Muscle, Open Approach */
"0KT34ZZ"  =  "1"  /* Resection of Left Neck Muscle, Perc Endo Approach */
"0KT40ZZ"  =  "1"  /* Resection of Tongue, Palate, Pharynx Muscle, Open Approach */
"0KT44ZZ"  =  "1"  /* Resection of Tongue/Palate/Phar Muscle, Perc Endo Approach */
"0KT50ZZ"  =  "1"  /* Resection of Right Shoulder Muscle, Open Approach */
"0KT54ZZ"  =  "1"  /* Resection of Right Shoulder Muscle, Perc Endo Approach */
"0KT60ZZ"  =  "1"  /* Resection of Left Shoulder Muscle, Open Approach */
"0KT64ZZ"  =  "1"  /* Resection of Left Shoulder Muscle, Perc Endo Approach */
"0KT70ZZ"  =  "1"  /* Resection of Right Upper Arm Muscle, Open Approach */
"0KT74ZZ"  =  "1"  /* Resection of Right Upper Arm Muscle, Perc Endo Approach */
"0KT80ZZ"  =  "1"  /* Resection of Left Upper Arm Muscle, Open Approach */
"0KT84ZZ"  =  "1"  /* Resection of Left Upper Arm Muscle, Perc Endo Approach */
"0KT90ZZ"  =  "1"  /* Resection of Right Lower Arm and Wrist Muscle, Open Approach */
"0KT94ZZ"  =  "1"  /* Resection of R Low Arm & Wrist Muscle, Perc Endo Approach */
"0KTB0ZZ"  =  "1"  /* Resection of Left Lower Arm and Wrist Muscle, Open Approach */
"0KTB4ZZ"  =  "1"  /* Resection of L Low Arm & Wrist Muscle, Perc Endo Approach */
"0KTC0ZZ"  =  "1"  /* Resection of Right Hand Muscle, Open Approach */
"0KTC4ZZ"  =  "1"  /* Resection of Right Hand Muscle, Perc Endo Approach */
"0KTD0ZZ"  =  "1"  /* Resection of Left Hand Muscle, Open Approach */
"0KTD4ZZ"  =  "1"  /* Resection of Left Hand Muscle, Perc Endo Approach */
"0KTF0ZZ"  =  "1"  /* Resection of Right Trunk Muscle, Open Approach */
"0KTF4ZZ"  =  "1"  /* Resection of Right Trunk Muscle, Perc Endo Approach */
"0KTG0ZZ"  =  "1"  /* Resection of Left Trunk Muscle, Open Approach */
"0KTG4ZZ"  =  "1"  /* Resection of Left Trunk Muscle, Perc Endo Approach */
"0KTH0ZZ"  =  "1"  /* Resection of Right Thorax Muscle, Open Approach */
"0KTH4ZZ"  =  "1"  /* Resection of Right Thorax Muscle, Perc Endo Approach */
"0KTJ0ZZ"  =  "1"  /* Resection of Left Thorax Muscle, Open Approach */
"0KTJ4ZZ"  =  "1"  /* Resection of Left Thorax Muscle, Perc Endo Approach */
"0KTK0ZZ"  =  "1"  /* Resection of Right Abdomen Muscle, Open Approach */
"0KTK4ZZ"  =  "1"  /* Resection of Right Abdomen Muscle, Perc Endo Approach */
"0KTL0ZZ"  =  "1"  /* Resection of Left Abdomen Muscle, Open Approach */
"0KTL4ZZ"  =  "1"  /* Resection of Left Abdomen Muscle, Perc Endo Approach */
"0KTM0ZZ"  =  "1"  /* Resection of Perineum Muscle, Open Approach */
"0KTM4ZZ"  =  "1"  /* Resection of Perineum Muscle, Perc Endo Approach */
"0KTN0ZZ"  =  "1"  /* Resection of Right Hip Muscle, Open Approach */
"0KTN4ZZ"  =  "1"  /* Resection of Right Hip Muscle, Perc Endo Approach */
"0KTP0ZZ"  =  "1"  /* Resection of Left Hip Muscle, Open Approach */
"0KTP4ZZ"  =  "1"  /* Resection of Left Hip Muscle, Perc Endo Approach */
"0KTQ0ZZ"  =  "1"  /* Resection of Right Upper Leg Muscle, Open Approach */
"0KTQ4ZZ"  =  "1"  /* Resection of Right Upper Leg Muscle, Perc Endo Approach */
"0KTR0ZZ"  =  "1"  /* Resection of Left Upper Leg Muscle, Open Approach */
"0KTR4ZZ"  =  "1"  /* Resection of Left Upper Leg Muscle, Perc Endo Approach */
"0KTS0ZZ"  =  "1"  /* Resection of Right Lower Leg Muscle, Open Approach */
"0KTS4ZZ"  =  "1"  /* Resection of Right Lower Leg Muscle, Perc Endo Approach */
"0KTT0ZZ"  =  "1"  /* Resection of Left Lower Leg Muscle, Open Approach */
"0KTT4ZZ"  =  "1"  /* Resection of Left Lower Leg Muscle, Perc Endo Approach */
"0KTV0ZZ"  =  "1"  /* Resection of Right Foot Muscle, Open Approach */
"0KTV4ZZ"  =  "1"  /* Resection of Right Foot Muscle, Perc Endo Approach */
"0KTW0ZZ"  =  "1"  /* Resection of Left Foot Muscle, Open Approach */
"0KTW4ZZ"  =  "1"  /* Resection of Left Foot Muscle, Perc Endo Approach */
"0KU007Z"  =  "1"  /* Supplement Head Muscle with Autol Sub, Open Approach */
"0KU00JZ"  =  "1"  /* Supplement Head Muscle with Synth Sub, Open Approach */
"0KU00KZ"  =  "1"  /* Supplement Head Muscle with Nonaut Sub, Open Approach */
"0KU047Z"  =  "1"  /* Supplement Head Muscle with Autol Sub, Perc Endo Approach */
"0KU04JZ"  =  "1"  /* Supplement Head Muscle with Synth Sub, Perc Endo Approach */
"0KU04KZ"  =  "1"  /* Supplement Head Muscle with Nonaut Sub, Perc Endo Approach */
"0KU107Z"  =  "1"  /* Supplement Facial Muscle with Autol Sub, Open Approach */
"0KU10JZ"  =  "1"  /* Supplement Facial Muscle with Synth Sub, Open Approach */
"0KU10KZ"  =  "1"  /* Supplement Facial Muscle with Nonaut Sub, Open Approach */
"0KU147Z"  =  "1"  /* Supplement Facial Muscle with Autol Sub, Perc Endo Approach */
"0KU14JZ"  =  "1"  /* Supplement Facial Muscle with Synth Sub, Perc Endo Approach */
"0KU14KZ"  =  "1"  /* Supplement Facial Muscle with Nonaut Sub, Perc Endo Approach */
"0KU207Z"  =  "1"  /* Supplement Right Neck Muscle with Autol Sub, Open Approach */
"0KU20JZ"  =  "1"  /* Supplement Right Neck Muscle with Synth Sub, Open Approach */
"0KU20KZ"  =  "1"  /* Supplement Right Neck Muscle with Nonaut Sub, Open Approach */
"0KU247Z"  =  "1"  /* Supplement R Neck Muscle with Autol Sub, Perc Endo Approach */
"0KU24JZ"  =  "1"  /* Supplement R Neck Muscle with Synth Sub, Perc Endo Approach */
"0KU24KZ"  =  "1"  /* Supplement R Neck Muscle with Nonaut Sub, Perc Endo Approach */
"0KU307Z"  =  "1"  /* Supplement Left Neck Muscle with Autol Sub, Open Approach */
"0KU30JZ"  =  "1"  /* Supplement Left Neck Muscle with Synth Sub, Open Approach */
"0KU30KZ"  =  "1"  /* Supplement Left Neck Muscle with Nonaut Sub, Open Approach */
"0KU347Z"  =  "1"  /* Supplement L Neck Muscle with Autol Sub, Perc Endo Approach */
"0KU34JZ"  =  "1"  /* Supplement L Neck Muscle with Synth Sub, Perc Endo Approach */
"0KU34KZ"  =  "1"  /* Supplement L Neck Muscle with Nonaut Sub, Perc Endo Approach */
"0KU407Z"  =  "1"  /* Supplement Tongue/Palate/Phar Muscle w Autol Sub, Open */
"0KU40JZ"  =  "1"  /* Supplement Tongue/Palate/Phar Muscle w Synth Sub, Open */
"0KU40KZ"  =  "1"  /* Supplement Tongue/Palate/Phar Muscle w Nonaut Sub, Open */
"0KU447Z"  =  "1"  /* Supplement Tongue/Palate/Phar Muscle w Autol Sub, Perc Endo */
"0KU44JZ"  =  "1"  /* Supplement Tongue/Palate/Phar Muscle w Synth Sub, Perc Endo */
"0KU44KZ"  =  "1"  /* Supplement Tongue/Palate/Phar Muscle w Nonaut Sub, Perc Endo */
"0KU507Z"  =  "1"  /* Supplement R Shoulder Muscle with Autol Sub, Open Approach */
"0KU50JZ"  =  "1"  /* Supplement R Shoulder Muscle with Synth Sub, Open Approach */
"0KU50KZ"  =  "1"  /* Supplement R Shoulder Muscle with Nonaut Sub, Open Approach */
"0KU547Z"  =  "1"  /* Supplement R Shoulder Muscle w Autol Sub, Perc Endo */
"0KU54JZ"  =  "1"  /* Supplement R Shoulder Muscle w Synth Sub, Perc Endo */
"0KU54KZ"  =  "1"  /* Supplement R Shoulder Muscle w Nonaut Sub, Perc Endo */
"0KU607Z"  =  "1"  /* Supplement L Shoulder Muscle with Autol Sub, Open Approach */
"0KU60JZ"  =  "1"  /* Supplement L Shoulder Muscle with Synth Sub, Open Approach */
"0KU60KZ"  =  "1"  /* Supplement L Shoulder Muscle with Nonaut Sub, Open Approach */
"0KU647Z"  =  "1"  /* Supplement L Shoulder Muscle w Autol Sub, Perc Endo */
"0KU64JZ"  =  "1"  /* Supplement L Shoulder Muscle w Synth Sub, Perc Endo */
"0KU64KZ"  =  "1"  /* Supplement L Shoulder Muscle w Nonaut Sub, Perc Endo */
"0KU707Z"  =  "1"  /* Supplement R Up Arm Muscle with Autol Sub, Open Approach */
"0KU70JZ"  =  "1"  /* Supplement R Up Arm Muscle with Synth Sub, Open Approach */
"0KU70KZ"  =  "1"  /* Supplement R Up Arm Muscle with Nonaut Sub, Open Approach */
"0KU747Z"  =  "1"  /* Supplement R Up Arm Muscle w Autol Sub, Perc Endo */
"0KU74JZ"  =  "1"  /* Supplement R Up Arm Muscle w Synth Sub, Perc Endo */
"0KU74KZ"  =  "1"  /* Supplement R Up Arm Muscle w Nonaut Sub, Perc Endo */
"0KU807Z"  =  "1"  /* Supplement L Up Arm Muscle with Autol Sub, Open Approach */
"0KU80JZ"  =  "1"  /* Supplement L Up Arm Muscle with Synth Sub, Open Approach */
"0KU80KZ"  =  "1"  /* Supplement L Up Arm Muscle with Nonaut Sub, Open Approach */
"0KU847Z"  =  "1"  /* Supplement L Up Arm Muscle w Autol Sub, Perc Endo */
"0KU84JZ"  =  "1"  /* Supplement L Up Arm Muscle w Synth Sub, Perc Endo */
"0KU84KZ"  =  "1"  /* Supplement L Up Arm Muscle w Nonaut Sub, Perc Endo */
"0KU907Z"  =  "1"  /* Supplement R Low Arm & Wrist Muscle w Autol Sub, Open */
"0KU90JZ"  =  "1"  /* Supplement R Low Arm & Wrist Muscle w Synth Sub, Open */
"0KU90KZ"  =  "1"  /* Supplement R Low Arm & Wrist Muscle w Nonaut Sub, Open */
"0KU947Z"  =  "1"  /* Supplement R Low Arm & Wrist Muscle w Autol Sub, Perc Endo */
"0KU94JZ"  =  "1"  /* Supplement R Low Arm & Wrist Muscle w Synth Sub, Perc Endo */
"0KU94KZ"  =  "1"  /* Supplement R Low Arm & Wrist Muscle w Nonaut Sub, Perc Endo */
"0KUB07Z"  =  "1"  /* Supplement L Low Arm & Wrist Muscle w Autol Sub, Open */
"0KUB0JZ"  =  "1"  /* Supplement L Low Arm & Wrist Muscle w Synth Sub, Open */
"0KUB0KZ"  =  "1"  /* Supplement L Low Arm & Wrist Muscle w Nonaut Sub, Open */
"0KUB47Z"  =  "1"  /* Supplement L Low Arm & Wrist Muscle w Autol Sub, Perc Endo */
"0KUB4JZ"  =  "1"  /* Supplement L Low Arm & Wrist Muscle w Synth Sub, Perc Endo */
"0KUB4KZ"  =  "1"  /* Supplement L Low Arm & Wrist Muscle w Nonaut Sub, Perc Endo */
"0KUC07Z"  =  "1"  /* Supplement Right Hand Muscle with Autol Sub, Open Approach */
"0KUC0JZ"  =  "1"  /* Supplement Right Hand Muscle with Synth Sub, Open Approach */
"0KUC0KZ"  =  "1"  /* Supplement Right Hand Muscle with Nonaut Sub, Open Approach */
"0KUC47Z"  =  "1"  /* Supplement R Hand Muscle with Autol Sub, Perc Endo Approach */
"0KUC4JZ"  =  "1"  /* Supplement R Hand Muscle with Synth Sub, Perc Endo Approach */
"0KUC4KZ"  =  "1"  /* Supplement R Hand Muscle with Nonaut Sub, Perc Endo Approach */
"0KUD07Z"  =  "1"  /* Supplement Left Hand Muscle with Autol Sub, Open Approach */
"0KUD0JZ"  =  "1"  /* Supplement Left Hand Muscle with Synth Sub, Open Approach */
"0KUD0KZ"  =  "1"  /* Supplement Left Hand Muscle with Nonaut Sub, Open Approach */
"0KUD47Z"  =  "1"  /* Supplement L Hand Muscle with Autol Sub, Perc Endo Approach */
"0KUD4JZ"  =  "1"  /* Supplement L Hand Muscle with Synth Sub, Perc Endo Approach */
"0KUD4KZ"  =  "1"  /* Supplement L Hand Muscle with Nonaut Sub, Perc Endo Approach */
"0KUF07Z"  =  "1"  /* Supplement Right Trunk Muscle with Autol Sub, Open Approach */
"0KUF0JZ"  =  "1"  /* Supplement Right Trunk Muscle with Synth Sub, Open Approach */
"0KUF0KZ"  =  "1"  /* Supplement Right Trunk Muscle with Nonaut Sub, Open Approach */
"0KUF47Z"  =  "1"  /* Supplement R Trunk Muscle with Autol Sub, Perc Endo Approach */
"0KUF4JZ"  =  "1"  /* Supplement R Trunk Muscle with Synth Sub, Perc Endo Approach */
"0KUF4KZ"  =  "1"  /* Supplement R Trunk Muscle w Nonaut Sub, Perc Endo */
"0KUG07Z"  =  "1"  /* Supplement Left Trunk Muscle with Autol Sub, Open Approach */
"0KUG0JZ"  =  "1"  /* Supplement Left Trunk Muscle with Synth Sub, Open Approach */
"0KUG0KZ"  =  "1"  /* Supplement Left Trunk Muscle with Nonaut Sub, Open Approach */
"0KUG47Z"  =  "1"  /* Supplement L Trunk Muscle with Autol Sub, Perc Endo Approach */
"0KUG4JZ"  =  "1"  /* Supplement L Trunk Muscle with Synth Sub, Perc Endo Approach */
"0KUG4KZ"  =  "1"  /* Supplement L Trunk Muscle w Nonaut Sub, Perc Endo */
"0KUH07Z"  =  "1"  /* Supplement Right Thorax Muscle with Autol Sub, Open Approach */
"0KUH0JZ"  =  "1"  /* Supplement Right Thorax Muscle with Synth Sub, Open Approach */
"0KUH0KZ"  =  "1"  /* Supplement R Thorax Muscle with Nonaut Sub, Open Approach */
"0KUH47Z"  =  "1"  /* Supplement R Thorax Muscle w Autol Sub, Perc Endo */
"0KUH4JZ"  =  "1"  /* Supplement R Thorax Muscle w Synth Sub, Perc Endo */
"0KUH4KZ"  =  "1"  /* Supplement R Thorax Muscle w Nonaut Sub, Perc Endo */
"0KUJ07Z"  =  "1"  /* Supplement Left Thorax Muscle with Autol Sub, Open Approach */
"0KUJ0JZ"  =  "1"  /* Supplement Left Thorax Muscle with Synth Sub, Open Approach */
"0KUJ0KZ"  =  "1"  /* Supplement Left Thorax Muscle with Nonaut Sub, Open Approach */
"0KUJ47Z"  =  "1"  /* Supplement L Thorax Muscle w Autol Sub, Perc Endo */
"0KUJ4JZ"  =  "1"  /* Supplement L Thorax Muscle w Synth Sub, Perc Endo */
"0KUJ4KZ"  =  "1"  /* Supplement L Thorax Muscle w Nonaut Sub, Perc Endo */
"0KUK07Z"  =  "1"  /* Supplement R Abd Muscle with Autol Sub, Open Approach */
"0KUK0JZ"  =  "1"  /* Supplement R Abd Muscle with Synth Sub, Open Approach */
"0KUK0KZ"  =  "1"  /* Supplement R Abd Muscle with Nonaut Sub, Open Approach */
"0KUK47Z"  =  "1"  /* Supplement R Abd Muscle with Autol Sub, Perc Endo Approach */
"0KUK4JZ"  =  "1"  /* Supplement R Abd Muscle with Synth Sub, Perc Endo Approach */
"0KUK4KZ"  =  "1"  /* Supplement R Abd Muscle with Nonaut Sub, Perc Endo Approach */
"0KUL07Z"  =  "1"  /* Supplement Left Abdomen Muscle with Autol Sub, Open Approach */
"0KUL0JZ"  =  "1"  /* Supplement Left Abdomen Muscle with Synth Sub, Open Approach */
"0KUL0KZ"  =  "1"  /* Supplement L Abd Muscle with Nonaut Sub, Open Approach */
"0KUL47Z"  =  "1"  /* Supplement L Abd Muscle with Autol Sub, Perc Endo Approach */
"0KUL4JZ"  =  "1"  /* Supplement L Abd Muscle with Synth Sub, Perc Endo Approach */
"0KUL4KZ"  =  "1"  /* Supplement L Abd Muscle with Nonaut Sub, Perc Endo Approach */
"0KUM07Z"  =  "1"  /* Supplement Perineum Muscle with Autol Sub, Open Approach */
"0KUM0JZ"  =  "1"  /* Supplement Perineum Muscle with Synth Sub, Open Approach */
"0KUM0KZ"  =  "1"  /* Supplement Perineum Muscle with Nonaut Sub, Open Approach */
"0KUM47Z"  =  "1"  /* Supplement Perineum Muscle w Autol Sub, Perc Endo */
"0KUM4JZ"  =  "1"  /* Supplement Perineum Muscle w Synth Sub, Perc Endo */
"0KUM4KZ"  =  "1"  /* Supplement Perineum Muscle w Nonaut Sub, Perc Endo */
"0KUN07Z"  =  "1"  /* Supplement Right Hip Muscle with Autol Sub, Open Approach */
"0KUN0JZ"  =  "1"  /* Supplement Right Hip Muscle with Synth Sub, Open Approach */
"0KUN0KZ"  =  "1"  /* Supplement Right Hip Muscle with Nonaut Sub, Open Approach */
"0KUN47Z"  =  "1"  /* Supplement R Hip Muscle with Autol Sub, Perc Endo Approach */
"0KUN4JZ"  =  "1"  /* Supplement R Hip Muscle with Synth Sub, Perc Endo Approach */
"0KUN4KZ"  =  "1"  /* Supplement R Hip Muscle with Nonaut Sub, Perc Endo Approach */
"0KUP07Z"  =  "1"  /* Supplement Left Hip Muscle with Autol Sub, Open Approach */
"0KUP0JZ"  =  "1"  /* Supplement Left Hip Muscle with Synth Sub, Open Approach */
"0KUP0KZ"  =  "1"  /* Supplement Left Hip Muscle with Nonaut Sub, Open Approach */
"0KUP47Z"  =  "1"  /* Supplement L Hip Muscle with Autol Sub, Perc Endo Approach */
"0KUP4JZ"  =  "1"  /* Supplement L Hip Muscle with Synth Sub, Perc Endo Approach */
"0KUP4KZ"  =  "1"  /* Supplement L Hip Muscle with Nonaut Sub, Perc Endo Approach */
"0KUQ07Z"  =  "1"  /* Supplement R Up Leg Muscle with Autol Sub, Open Approach */
"0KUQ0JZ"  =  "1"  /* Supplement R Up Leg Muscle with Synth Sub, Open Approach */
"0KUQ0KZ"  =  "1"  /* Supplement R Up Leg Muscle with Nonaut Sub, Open Approach */
"0KUQ47Z"  =  "1"  /* Supplement R Up Leg Muscle w Autol Sub, Perc Endo */
"0KUQ4JZ"  =  "1"  /* Supplement R Up Leg Muscle w Synth Sub, Perc Endo */
"0KUQ4KZ"  =  "1"  /* Supplement R Up Leg Muscle w Nonaut Sub, Perc Endo */
"0KUR07Z"  =  "1"  /* Supplement L Up Leg Muscle with Autol Sub, Open Approach */
"0KUR0JZ"  =  "1"  /* Supplement L Up Leg Muscle with Synth Sub, Open Approach */
"0KUR0KZ"  =  "1"  /* Supplement L Up Leg Muscle with Nonaut Sub, Open Approach */
"0KUR47Z"  =  "1"  /* Supplement L Up Leg Muscle w Autol Sub, Perc Endo */
"0KUR4JZ"  =  "1"  /* Supplement L Up Leg Muscle w Synth Sub, Perc Endo */
"0KUR4KZ"  =  "1"  /* Supplement L Up Leg Muscle w Nonaut Sub, Perc Endo */
"0KUS07Z"  =  "1"  /* Supplement R Low Leg Muscle with Autol Sub, Open Approach */
"0KUS0JZ"  =  "1"  /* Supplement R Low Leg Muscle with Synth Sub, Open Approach */
"0KUS0KZ"  =  "1"  /* Supplement R Low Leg Muscle with Nonaut Sub, Open Approach */
"0KUS47Z"  =  "1"  /* Supplement R Low Leg Muscle w Autol Sub, Perc Endo */
"0KUS4JZ"  =  "1"  /* Supplement R Low Leg Muscle w Synth Sub, Perc Endo */
"0KUS4KZ"  =  "1"  /* Supplement R Low Leg Muscle w Nonaut Sub, Perc Endo */
"0KUT07Z"  =  "1"  /* Supplement L Low Leg Muscle with Autol Sub, Open Approach */
"0KUT0JZ"  =  "1"  /* Supplement L Low Leg Muscle with Synth Sub, Open Approach */
"0KUT0KZ"  =  "1"  /* Supplement L Low Leg Muscle with Nonaut Sub, Open Approach */
"0KUT47Z"  =  "1"  /* Supplement L Low Leg Muscle w Autol Sub, Perc Endo */
"0KUT4JZ"  =  "1"  /* Supplement L Low Leg Muscle w Synth Sub, Perc Endo */
"0KUT4KZ"  =  "1"  /* Supplement L Low Leg Muscle w Nonaut Sub, Perc Endo */
"0KUV07Z"  =  "1"  /* Supplement Right Foot Muscle with Autol Sub, Open Approach */
"0KUV0JZ"  =  "1"  /* Supplement Right Foot Muscle with Synth Sub, Open Approach */
"0KUV0KZ"  =  "1"  /* Supplement Right Foot Muscle with Nonaut Sub, Open Approach */
"0KUV47Z"  =  "1"  /* Supplement R Foot Muscle with Autol Sub, Perc Endo Approach */
"0KUV4JZ"  =  "1"  /* Supplement R Foot Muscle with Synth Sub, Perc Endo Approach */
"0KUV4KZ"  =  "1"  /* Supplement R Foot Muscle with Nonaut Sub, Perc Endo Approach */
"0KUW07Z"  =  "1"  /* Supplement Left Foot Muscle with Autol Sub, Open Approach */
"0KUW0JZ"  =  "1"  /* Supplement Left Foot Muscle with Synth Sub, Open Approach */
"0KUW0KZ"  =  "1"  /* Supplement Left Foot Muscle with Nonaut Sub, Open Approach */
"0KUW47Z"  =  "1"  /* Supplement L Foot Muscle with Autol Sub, Perc Endo Approach */
"0KUW4JZ"  =  "1"  /* Supplement L Foot Muscle with Synth Sub, Perc Endo Approach */
"0KUW4KZ"  =  "1"  /* Supplement L Foot Muscle with Nonaut Sub, Perc Endo Approach */
"0KWX00Z"  =  "1"  /* Revision of Drainage Device in Upper Muscle, Open Approach */
"0KWX07Z"  =  "1"  /* Revision of Autol Sub in Up Muscle, Open Approach */
"0KWX0JZ"  =  "1"  /* Revision of Synthetic Substitute in Up Muscle, Open Approach */
"0KWX0KZ"  =  "1"  /* Revision of Nonaut Sub in Up Muscle, Open Approach */
"0KWX0MZ"  =  "1"  /* Revision of Stimulator Lead in Upper Muscle, Open Approach */
"0KWX30Z"  =  "1"  /* Revision of Drainage Device in Upper Muscle, Perc Approach */
"0KWX37Z"  =  "1"  /* Revision of Autol Sub in Up Muscle, Perc Approach */
"0KWX3JZ"  =  "1"  /* Revision of Synthetic Substitute in Up Muscle, Perc Approach */
"0KWX3KZ"  =  "1"  /* Revision of Nonaut Sub in Up Muscle, Perc Approach */
"0KWX3MZ"  =  "1"  /* Revision of Stimulator Lead in Upper Muscle, Perc Approach */
"0KWX40Z"  =  "1"  /* Revision of Drainage Device in Up Muscle, Perc Endo Approach */
"0KWX47Z"  =  "1"  /* Revision of Autol Sub in Up Muscle, Perc Endo Approach */
"0KWX4JZ"  =  "1"  /* Revision of Synth Sub in Up Muscle, Perc Endo Approach */
"0KWX4KZ"  =  "1"  /* Revision of Nonaut Sub in Up Muscle, Perc Endo Approach */
"0KWX4MZ"  =  "1"  /* Revision of Stimulator Lead in Up Muscle, Perc Endo Approach */
"0KWY00Z"  =  "1"  /* Revision of Drainage Device in Lower Muscle, Open Approach */
"0KWY07Z"  =  "1"  /* Revision of Autol Sub in Low Muscle, Open Approach */
"0KWY0JZ"  =  "1"  /* Revision of Synth Sub in Low Muscle, Open Approach */
"0KWY0KZ"  =  "1"  /* Revision of Nonaut Sub in Low Muscle, Open Approach */
"0KWY0MZ"  =  "1"  /* Revision of Stimulator Lead in Lower Muscle, Open Approach */
"0KWY30Z"  =  "1"  /* Revision of Drainage Device in Lower Muscle, Perc Approach */
"0KWY37Z"  =  "1"  /* Revision of Autol Sub in Low Muscle, Perc Approach */
"0KWY3JZ"  =  "1"  /* Revision of Synth Sub in Low Muscle, Perc Approach */
"0KWY3KZ"  =  "1"  /* Revision of Nonaut Sub in Low Muscle, Perc Approach */
"0KWY3MZ"  =  "1"  /* Revision of Stimulator Lead in Lower Muscle, Perc Approach */
"0KWY40Z"  =  "1"  /* Revision of Drain Dev in Low Muscle, Perc Endo Approach */
"0KWY47Z"  =  "1"  /* Revision of Autol Sub in Low Muscle, Perc Endo Approach */
"0KWY4JZ"  =  "1"  /* Revision of Synth Sub in Low Muscle, Perc Endo Approach */
"0KWY4KZ"  =  "1"  /* Revision of Nonaut Sub in Low Muscle, Perc Endo Approach */
"0KWY4MZ"  =  "1"  /* Revision of Stim Lead in Low Muscle, Perc Endo Approach */
"0KX00Z0"  =  "1"  /* Transfer Head Muscle with Skin, Open Approach */
"0KX00Z1"  =  "1"  /* Transfer Head Muscle with Subcutaneous Tissue, Open Approach */
"0KX00Z2"  =  "1"  /* Transfer Head Muscle with Skin, Subcu, Open Approach */
"0KX00ZZ"  =  "1"  /* Transfer Head Muscle, Open Approach */
"0KX04Z0"  =  "1"  /* Transfer Head Muscle with Skin, Perc Endo Approach */
"0KX04Z1"  =  "1"  /* Transfer Head Muscle with Subcu, Perc Endo Approach */
"0KX04Z2"  =  "1"  /* Transfer Head Muscle with Skin, Subcu, Perc Endo Approach */
"0KX04ZZ"  =  "1"  /* Transfer Head Muscle, Percutaneous Endoscopic Approach */
"0KX10Z0"  =  "1"  /* Transfer Facial Muscle with Skin, Open Approach */
"0KX10Z1"  =  "1"  /* Transfer Facial Muscle with Subcu, Open Approach */
"0KX10Z2"  =  "1"  /* Transfer Facial Muscle with Skin, Subcu, Open Approach */
"0KX10ZZ"  =  "1"  /* Transfer Facial Muscle, Open Approach */
"0KX14Z0"  =  "1"  /* Transfer Facial Muscle with Skin, Perc Endo Approach */
"0KX14Z1"  =  "1"  /* Transfer Facial Muscle with Subcu, Perc Endo Approach */
"0KX14Z2"  =  "1"  /* Transfer Facial Muscle with Skin, Subcu, Perc Endo Approach */
"0KX14ZZ"  =  "1"  /* Transfer Facial Muscle, Percutaneous Endoscopic Approach */
"0KX20Z0"  =  "1"  /* Transfer Right Neck Muscle with Skin, Open Approach */
"0KX20Z1"  =  "1"  /* Transfer Right Neck Muscle with Subcu, Open Approach */
"0KX20Z2"  =  "1"  /* Transfer Right Neck Muscle with Skin, Subcu, Open Approach */
"0KX20ZZ"  =  "1"  /* Transfer Right Neck Muscle, Open Approach */
"0KX24Z0"  =  "1"  /* Transfer Right Neck Muscle with Skin, Perc Endo Approach */
"0KX24Z1"  =  "1"  /* Transfer Right Neck Muscle with Subcu, Perc Endo Approach */
"0KX24Z2"  =  "1"  /* Transfer R Neck Muscle with Skin, Subcu, Perc Endo Approach */
"0KX24ZZ"  =  "1"  /* Transfer Right Neck Muscle, Percutaneous Endoscopic Approach */
"0KX30Z0"  =  "1"  /* Transfer Left Neck Muscle with Skin, Open Approach */
"0KX30Z1"  =  "1"  /* Transfer Left Neck Muscle with Subcu, Open Approach */
"0KX30Z2"  =  "1"  /* Transfer Left Neck Muscle with Skin, Subcu, Open Approach */
"0KX30ZZ"  =  "1"  /* Transfer Left Neck Muscle, Open Approach */
"0KX34Z0"  =  "1"  /* Transfer Left Neck Muscle with Skin, Perc Endo Approach */
"0KX34Z1"  =  "1"  /* Transfer Left Neck Muscle with Subcu, Perc Endo Approach */
"0KX34Z2"  =  "1"  /* Transfer L Neck Muscle with Skin, Subcu, Perc Endo Approach */
"0KX34ZZ"  =  "1"  /* Transfer Left Neck Muscle, Percutaneous Endoscopic Approach */
"0KX40Z0"  =  "1"  /* Transfer Tongue/Palate/Phar Muscle with Skin, Open Approach */
"0KX40Z1"  =  "1"  /* Transfer Tongue/Palate/Phar Muscle with Subcu, Open Approach */
"0KX40Z2"  =  "1"  /* Transfer Tongue/Palate/Phar Muscle w Skin, Subcu, Open */
"0KX40ZZ"  =  "1"  /* Transfer Tongue, Palate, Pharynx Muscle, Open Approach */
"0KX44Z0"  =  "1"  /* Transfer Tongue/Palate/Phar Muscle w Skin, Perc Endo */
"0KX44Z1"  =  "1"  /* Transfer Tongue/Palate/Phar Muscle w Subcu, Perc Endo */
"0KX44Z2"  =  "1"  /* Transfer Tongue/Palate/Phar Muscle w Skin, Subcu, Perc Endo */
"0KX44ZZ"  =  "1"  /* Transfer Tongue, Palate, Pharynx Muscle, Perc Endo Approach */
"0KX50Z0"  =  "1"  /* Transfer Right Shoulder Muscle with Skin, Open Approach */
"0KX50Z1"  =  "1"  /* Transfer Right Shoulder Muscle with Subcu, Open Approach */
"0KX50Z2"  =  "1"  /* Transfer R Shoulder Muscle with Skin, Subcu, Open Approach */
"0KX50ZZ"  =  "1"  /* Transfer Right Shoulder Muscle, Open Approach */
"0KX54Z0"  =  "1"  /* Transfer Right Shoulder Muscle with Skin, Perc Endo Approach */
"0KX54Z1"  =  "1"  /* Transfer R Shoulder Muscle with Subcu, Perc Endo Approach */
"0KX54Z2"  =  "1"  /* Transfer R Shoulder Muscle w Skin, Subcu, Perc Endo */
"0KX54ZZ"  =  "1"  /* Transfer Right Shoulder Muscle, Perc Endo Approach */
"0KX60Z0"  =  "1"  /* Transfer Left Shoulder Muscle with Skin, Open Approach */
"0KX60Z1"  =  "1"  /* Transfer Left Shoulder Muscle with Subcu, Open Approach */
"0KX60Z2"  =  "1"  /* Transfer L Shoulder Muscle with Skin, Subcu, Open Approach */
"0KX60ZZ"  =  "1"  /* Transfer Left Shoulder Muscle, Open Approach */
"0KX64Z0"  =  "1"  /* Transfer Left Shoulder Muscle with Skin, Perc Endo Approach */
"0KX64Z1"  =  "1"  /* Transfer Left Shoulder Muscle with Subcu, Perc Endo Approach */
"0KX64Z2"  =  "1"  /* Transfer L Shoulder Muscle w Skin, Subcu, Perc Endo */
"0KX64ZZ"  =  "1"  /* Transfer Left Shoulder Muscle, Perc Endo Approach */
"0KX70Z0"  =  "1"  /* Transfer Right Upper Arm Muscle with Skin, Open Approach */
"0KX70Z1"  =  "1"  /* Transfer Right Upper Arm Muscle with Subcu, Open Approach */
"0KX70Z2"  =  "1"  /* Transfer R Up Arm Muscle with Skin, Subcu, Open Approach */
"0KX70ZZ"  =  "1"  /* Transfer Right Upper Arm Muscle, Open Approach */
"0KX74Z0"  =  "1"  /* Transfer R Up Arm Muscle with Skin, Perc Endo Approach */
"0KX74Z1"  =  "1"  /* Transfer R Up Arm Muscle with Subcu, Perc Endo Approach */
"0KX74Z2"  =  "1"  /* Transfer R Up Arm Muscle w Skin, Subcu, Perc Endo */
"0KX74ZZ"  =  "1"  /* Transfer Right Upper Arm Muscle, Perc Endo Approach */
"0KX80Z0"  =  "1"  /* Transfer Left Upper Arm Muscle with Skin, Open Approach */
"0KX80Z1"  =  "1"  /* Transfer Left Upper Arm Muscle with Subcu, Open Approach */
"0KX80Z2"  =  "1"  /* Transfer L Up Arm Muscle with Skin, Subcu, Open Approach */
"0KX80ZZ"  =  "1"  /* Transfer Left Upper Arm Muscle, Open Approach */
"0KX84Z0"  =  "1"  /* Transfer Left Upper Arm Muscle with Skin, Perc Endo Approach */
"0KX84Z1"  =  "1"  /* Transfer L Up Arm Muscle with Subcu, Perc Endo Approach */
"0KX84Z2"  =  "1"  /* Transfer L Up Arm Muscle w Skin, Subcu, Perc Endo */
"0KX84ZZ"  =  "1"  /* Transfer Left Upper Arm Muscle, Perc Endo Approach */
"0KX90Z0"  =  "1"  /* Transfer R Low Arm & Wrist Muscle with Skin, Open Approach */
"0KX90Z1"  =  "1"  /* Transfer R Low Arm & Wrist Muscle with Subcu, Open Approach */
"0KX90Z2"  =  "1"  /* Transfer R Low Arm & Wrist Muscle w Skin, Subcu, Open */
"0KX90ZZ"  =  "1"  /* Transfer Right Lower Arm and Wrist Muscle, Open Approach */
"0KX94Z0"  =  "1"  /* Transfer R Low Arm & Wrist Muscle w Skin, Perc Endo */
"0KX94Z1"  =  "1"  /* Transfer R Low Arm & Wrist Muscle w Subcu, Perc Endo */
"0KX94Z2"  =  "1"  /* Transfer R Low Arm & Wrist Muscle w Skin, Subcu, Perc Endo */
"0KX94ZZ"  =  "1"  /* Transfer R Low Arm & Wrist Muscle, Perc Endo Approach */
"0KXB0Z0"  =  "1"  /* Transfer L Low Arm & Wrist Muscle with Skin, Open Approach */
"0KXB0Z1"  =  "1"  /* Transfer L Low Arm & Wrist Muscle with Subcu, Open Approach */
"0KXB0Z2"  =  "1"  /* Transfer L Low Arm & Wrist Muscle w Skin, Subcu, Open */
"0KXB0ZZ"  =  "1"  /* Transfer Left Lower Arm and Wrist Muscle, Open Approach */
"0KXB4Z0"  =  "1"  /* Transfer L Low Arm & Wrist Muscle w Skin, Perc Endo */
"0KXB4Z1"  =  "1"  /* Transfer L Low Arm & Wrist Muscle w Subcu, Perc Endo */
"0KXB4Z2"  =  "1"  /* Transfer L Low Arm & Wrist Muscle w Skin, Subcu, Perc Endo */
"0KXB4ZZ"  =  "1"  /* Transfer Left Lower Arm and Wrist Muscle, Perc Endo Approach */
"0KXC0Z0"  =  "1"  /* Transfer Right Hand Muscle with Skin, Open Approach */
"0KXC0Z1"  =  "1"  /* Transfer Right Hand Muscle with Subcu, Open Approach */
"0KXC0Z2"  =  "1"  /* Transfer Right Hand Muscle with Skin, Subcu, Open Approach */
"0KXC0ZZ"  =  "1"  /* Transfer Right Hand Muscle, Open Approach */
"0KXC4Z0"  =  "1"  /* Transfer Right Hand Muscle with Skin, Perc Endo Approach */
"0KXC4Z1"  =  "1"  /* Transfer Right Hand Muscle with Subcu, Perc Endo Approach */
"0KXC4Z2"  =  "1"  /* Transfer R Hand Muscle with Skin, Subcu, Perc Endo Approach */
"0KXC4ZZ"  =  "1"  /* Transfer Right Hand Muscle, Percutaneous Endoscopic Approach */
"0KXD0Z0"  =  "1"  /* Transfer Left Hand Muscle with Skin, Open Approach */
"0KXD0Z1"  =  "1"  /* Transfer Left Hand Muscle with Subcu, Open Approach */
"0KXD0Z2"  =  "1"  /* Transfer Left Hand Muscle with Skin, Subcu, Open Approach */
"0KXD0ZZ"  =  "1"  /* Transfer Left Hand Muscle, Open Approach */
"0KXD4Z0"  =  "1"  /* Transfer Left Hand Muscle with Skin, Perc Endo Approach */
"0KXD4Z1"  =  "1"  /* Transfer Left Hand Muscle with Subcu, Perc Endo Approach */
"0KXD4Z2"  =  "1"  /* Transfer L Hand Muscle with Skin, Subcu, Perc Endo Approach */
"0KXD4ZZ"  =  "1"  /* Transfer Left Hand Muscle, Percutaneous Endoscopic Approach */
"0KXF0Z0"  =  "1"  /* Transfer Right Trunk Muscle with Skin, Open Approach */
"0KXF0Z1"  =  "1"  /* Transfer Right Trunk Muscle with Subcu, Open Approach */
"0KXF0Z2"  =  "1"  /* Transfer Right Trunk Muscle with Skin, Subcu, Open Approach */
"0KXF0ZZ"  =  "1"  /* Transfer Right Trunk Muscle, Open Approach */
"0KXF4Z0"  =  "1"  /* Transfer Right Trunk Muscle with Skin, Perc Endo Approach */
"0KXF4Z1"  =  "1"  /* Transfer Right Trunk Muscle with Subcu, Perc Endo Approach */
"0KXF4Z2"  =  "1"  /* Transfer R Trunk Muscle with Skin, Subcu, Perc Endo Approach */
"0KXF4ZZ"  =  "1"  /* Transfer Right Trunk Muscle, Perc Endo Approach */
"0KXG0Z0"  =  "1"  /* Transfer Left Trunk Muscle with Skin, Open Approach */
"0KXG0Z1"  =  "1"  /* Transfer Left Trunk Muscle with Subcu, Open Approach */
"0KXG0Z2"  =  "1"  /* Transfer Left Trunk Muscle with Skin, Subcu, Open Approach */
"0KXG0ZZ"  =  "1"  /* Transfer Left Trunk Muscle, Open Approach */
"0KXG4Z0"  =  "1"  /* Transfer Left Trunk Muscle with Skin, Perc Endo Approach */
"0KXG4Z1"  =  "1"  /* Transfer Left Trunk Muscle with Subcu, Perc Endo Approach */
"0KXG4Z2"  =  "1"  /* Transfer L Trunk Muscle with Skin, Subcu, Perc Endo Approach */
"0KXG4ZZ"  =  "1"  /* Transfer Left Trunk Muscle, Percutaneous Endoscopic Approach */
"0KXH0Z0"  =  "1"  /* Transfer Right Thorax Muscle with Skin, Open Approach */
"0KXH0Z1"  =  "1"  /* Transfer Right Thorax Muscle with Subcu, Open Approach */
"0KXH0Z2"  =  "1"  /* Transfer Right Thorax Muscle with Skin, Subcu, Open Approach */
"0KXH0ZZ"  =  "1"  /* Transfer Right Thorax Muscle, Open Approach */
"0KXH4Z0"  =  "1"  /* Transfer Right Thorax Muscle with Skin, Perc Endo Approach */
"0KXH4Z1"  =  "1"  /* Transfer Right Thorax Muscle with Subcu, Perc Endo Approach */
"0KXH4Z2"  =  "1"  /* Transfer R Thorax Muscle w Skin, Subcu, Perc Endo */
"0KXH4ZZ"  =  "1"  /* Transfer Right Thorax Muscle, Perc Endo Approach */
"0KXJ0Z0"  =  "1"  /* Transfer Left Thorax Muscle with Skin, Open Approach */
"0KXJ0Z1"  =  "1"  /* Transfer Left Thorax Muscle with Subcu, Open Approach */
"0KXJ0Z2"  =  "1"  /* Transfer Left Thorax Muscle with Skin, Subcu, Open Approach */
"0KXJ0ZZ"  =  "1"  /* Transfer Left Thorax Muscle, Open Approach */
"0KXJ4Z0"  =  "1"  /* Transfer Left Thorax Muscle with Skin, Perc Endo Approach */
"0KXJ4Z1"  =  "1"  /* Transfer Left Thorax Muscle with Subcu, Perc Endo Approach */
"0KXJ4Z2"  =  "1"  /* Transfer L Thorax Muscle w Skin, Subcu, Perc Endo */
"0KXJ4ZZ"  =  "1"  /* Transfer Left Thorax Muscle, Perc Endo Approach */
"0KXK0Z0"  =  "1"  /* Transfer Right Abdomen Muscle with Skin, Open Approach */
"0KXK0Z1"  =  "1"  /* Transfer Right Abdomen Muscle with Subcu, Open Approach */
"0KXK0Z2"  =  "1"  /* Transfer R Abd Muscle with Skin, Subcu, Open Approach */
"0KXK0Z6"  =  "1"  /* Transfer Right Abdomen Muscle, TRAM Flap, Open Approach */
"0KXK0ZZ"  =  "1"  /* Transfer Right Abdomen Muscle, Open Approach */
"0KXK4Z0"  =  "1"  /* Transfer Right Abdomen Muscle with Skin, Perc Endo Approach */
"0KXK4Z1"  =  "1"  /* Transfer Right Abdomen Muscle with Subcu, Perc Endo Approach */
"0KXK4Z2"  =  "1"  /* Transfer R Abd Muscle with Skin, Subcu, Perc Endo Approach */
"0KXK4Z6"  =  "1"  /* Transfer Right Abdomen Muscle, TRAM Flap, Perc Endo Approach */
"0KXK4ZZ"  =  "1"  /* Transfer Right Abdomen Muscle, Perc Endo Approach */
"0KXL0Z0"  =  "1"  /* Transfer Left Abdomen Muscle with Skin, Open Approach */
"0KXL0Z1"  =  "1"  /* Transfer Left Abdomen Muscle with Subcu, Open Approach */
"0KXL0Z2"  =  "1"  /* Transfer Left Abdomen Muscle with Skin, Subcu, Open Approach */
"0KXL0Z6"  =  "1"  /* Transfer Left Abdomen Muscle, TRAM Flap, Open Approach */
"0KXL0ZZ"  =  "1"  /* Transfer Left Abdomen Muscle, Open Approach */
"0KXL4Z0"  =  "1"  /* Transfer Left Abdomen Muscle with Skin, Perc Endo Approach */
"0KXL4Z1"  =  "1"  /* Transfer Left Abdomen Muscle with Subcu, Perc Endo Approach */
"0KXL4Z2"  =  "1"  /* Transfer L Abd Muscle with Skin, Subcu, Perc Endo Approach */
"0KXL4Z6"  =  "1"  /* Transfer Left Abdomen Muscle, TRAM Flap, Perc Endo Approach */
"0KXL4ZZ"  =  "1"  /* Transfer Left Abdomen Muscle, Perc Endo Approach */
"0KXM0Z0"  =  "1"  /* Transfer Perineum Muscle with Skin, Open Approach */
"0KXM0Z1"  =  "1"  /* Transfer Perineum Muscle with Subcu, Open Approach */
"0KXM0Z2"  =  "1"  /* Transfer Perineum Muscle with Skin, Subcu, Open Approach */
"0KXM0ZZ"  =  "1"  /* Transfer Perineum Muscle, Open Approach */
"0KXM4Z0"  =  "1"  /* Transfer Perineum Muscle with Skin, Perc Endo Approach */
"0KXM4Z1"  =  "1"  /* Transfer Perineum Muscle with Subcu, Perc Endo Approach */
"0KXM4Z2"  =  "1"  /* Transfer Perineum Muscle w Skin, Subcu, Perc Endo */
"0KXM4ZZ"  =  "1"  /* Transfer Perineum Muscle, Percutaneous Endoscopic Approach */
"0KXN0Z0"  =  "1"  /* Transfer Right Hip Muscle with Skin, Open Approach */
"0KXN0Z1"  =  "1"  /* Transfer Right Hip Muscle with Subcu, Open Approach */
"0KXN0Z2"  =  "1"  /* Transfer Right Hip Muscle with Skin, Subcu, Open Approach */
"0KXN0ZZ"  =  "1"  /* Transfer Right Hip Muscle, Open Approach */
"0KXN4Z0"  =  "1"  /* Transfer Right Hip Muscle with Skin, Perc Endo Approach */
"0KXN4Z1"  =  "1"  /* Transfer Right Hip Muscle with Subcu, Perc Endo Approach */
"0KXN4Z2"  =  "1"  /* Transfer R Hip Muscle with Skin, Subcu, Perc Endo Approach */
"0KXN4ZZ"  =  "1"  /* Transfer Right Hip Muscle, Percutaneous Endoscopic Approach */
"0KXP0Z0"  =  "1"  /* Transfer Left Hip Muscle with Skin, Open Approach */
"0KXP0Z1"  =  "1"  /* Transfer Left Hip Muscle with Subcu, Open Approach */
"0KXP0Z2"  =  "1"  /* Transfer Left Hip Muscle with Skin, Subcu, Open Approach */
"0KXP0ZZ"  =  "1"  /* Transfer Left Hip Muscle, Open Approach */
"0KXP4Z0"  =  "1"  /* Transfer Left Hip Muscle with Skin, Perc Endo Approach */
"0KXP4Z1"  =  "1"  /* Transfer Left Hip Muscle with Subcu, Perc Endo Approach */
"0KXP4Z2"  =  "1"  /* Transfer L Hip Muscle with Skin, Subcu, Perc Endo Approach */
"0KXP4ZZ"  =  "1"  /* Transfer Left Hip Muscle, Percutaneous Endoscopic Approach */
"0KXQ0Z0"  =  "1"  /* Transfer Right Upper Leg Muscle with Skin, Open Approach */
"0KXQ0Z1"  =  "1"  /* Transfer Right Upper Leg Muscle with Subcu, Open Approach */
"0KXQ0Z2"  =  "1"  /* Transfer R Up Leg Muscle with Skin, Subcu, Open Approach */
"0KXQ0ZZ"  =  "1"  /* Transfer Right Upper Leg Muscle, Open Approach */
"0KXQ4Z0"  =  "1"  /* Transfer R Up Leg Muscle with Skin, Perc Endo Approach */
"0KXQ4Z1"  =  "1"  /* Transfer R Up Leg Muscle with Subcu, Perc Endo Approach */
"0KXQ4Z2"  =  "1"  /* Transfer R Up Leg Muscle w Skin, Subcu, Perc Endo */
"0KXQ4ZZ"  =  "1"  /* Transfer Right Upper Leg Muscle, Perc Endo Approach */
"0KXR0Z0"  =  "1"  /* Transfer Left Upper Leg Muscle with Skin, Open Approach */
"0KXR0Z1"  =  "1"  /* Transfer Left Upper Leg Muscle with Subcu, Open Approach */
"0KXR0Z2"  =  "1"  /* Transfer L Up Leg Muscle with Skin, Subcu, Open Approach */
"0KXR0ZZ"  =  "1"  /* Transfer Left Upper Leg Muscle, Open Approach */
"0KXR4Z0"  =  "1"  /* Transfer Left Upper Leg Muscle with Skin, Perc Endo Approach */
"0KXR4Z1"  =  "1"  /* Transfer L Up Leg Muscle with Subcu, Perc Endo Approach */
"0KXR4Z2"  =  "1"  /* Transfer L Up Leg Muscle w Skin, Subcu, Perc Endo */
"0KXR4ZZ"  =  "1"  /* Transfer Left Upper Leg Muscle, Perc Endo Approach */
"0KXS0Z0"  =  "1"  /* Transfer Right Lower Leg Muscle with Skin, Open Approach */
"0KXS0Z1"  =  "1"  /* Transfer Right Lower Leg Muscle with Subcu, Open Approach */
"0KXS0Z2"  =  "1"  /* Transfer R Low Leg Muscle with Skin, Subcu, Open Approach */
"0KXS0ZZ"  =  "1"  /* Transfer Right Lower Leg Muscle, Open Approach */
"0KXS4Z0"  =  "1"  /* Transfer R Low Leg Muscle with Skin, Perc Endo Approach */
"0KXS4Z1"  =  "1"  /* Transfer R Low Leg Muscle with Subcu, Perc Endo Approach */
"0KXS4Z2"  =  "1"  /* Transfer R Low Leg Muscle w Skin, Subcu, Perc Endo */
"0KXS4ZZ"  =  "1"  /* Transfer Right Lower Leg Muscle, Perc Endo Approach */
"0KXT0Z0"  =  "1"  /* Transfer Left Lower Leg Muscle with Skin, Open Approach */
"0KXT0Z1"  =  "1"  /* Transfer Left Lower Leg Muscle with Subcu, Open Approach */
"0KXT0Z2"  =  "1"  /* Transfer L Low Leg Muscle with Skin, Subcu, Open Approach */
"0KXT0ZZ"  =  "1"  /* Transfer Left Lower Leg Muscle, Open Approach */
"0KXT4Z0"  =  "1"  /* Transfer Left Lower Leg Muscle with Skin, Perc Endo Approach */
"0KXT4Z1"  =  "1"  /* Transfer L Low Leg Muscle with Subcu, Perc Endo Approach */
"0KXT4Z2"  =  "1"  /* Transfer L Low Leg Muscle w Skin, Subcu, Perc Endo */
"0KXT4ZZ"  =  "1"  /* Transfer Left Lower Leg Muscle, Perc Endo Approach */
"0KXV0Z0"  =  "1"  /* Transfer Right Foot Muscle with Skin, Open Approach */
"0KXV0Z1"  =  "1"  /* Transfer Right Foot Muscle with Subcu, Open Approach */
"0KXV0Z2"  =  "1"  /* Transfer Right Foot Muscle with Skin, Subcu, Open Approach */
"0KXV0ZZ"  =  "1"  /* Transfer Right Foot Muscle, Open Approach */
"0KXV4Z0"  =  "1"  /* Transfer Right Foot Muscle with Skin, Perc Endo Approach */
"0KXV4Z1"  =  "1"  /* Transfer Right Foot Muscle with Subcu, Perc Endo Approach */
"0KXV4Z2"  =  "1"  /* Transfer R Foot Muscle with Skin, Subcu, Perc Endo Approach */
"0KXV4ZZ"  =  "1"  /* Transfer Right Foot Muscle, Percutaneous Endoscopic Approach */
"0KXW0Z0"  =  "1"  /* Transfer Left Foot Muscle with Skin, Open Approach */
"0KXW0Z1"  =  "1"  /* Transfer Left Foot Muscle with Subcu, Open Approach */
"0KXW0Z2"  =  "1"  /* Transfer Left Foot Muscle with Skin, Subcu, Open Approach */
"0KXW0ZZ"  =  "1"  /* Transfer Left Foot Muscle, Open Approach */
"0KXW4Z0"  =  "1"  /* Transfer Left Foot Muscle with Skin, Perc Endo Approach */
"0KXW4Z1"  =  "1"  /* Transfer Left Foot Muscle with Subcu, Perc Endo Approach */
"0KXW4Z2"  =  "1"  /* Transfer L Foot Muscle with Skin, Subcu, Perc Endo Approach */
"0KXW4ZZ"  =  "1"  /* Transfer Left Foot Muscle, Percutaneous Endoscopic Approach */
"0L500ZZ"  =  "1"  /* Destruction of Head and Neck Tendon, Open Approach */
"0L503ZZ"  =  "1"  /* Destruction of Head and Neck Tendon, Percutaneous Approach */
"0L504ZZ"  =  "1"  /* Destruction of Head and Neck Tendon, Perc Endo Approach */
"0L510ZZ"  =  "1"  /* Destruction of Right Shoulder Tendon, Open Approach */
"0L513ZZ"  =  "1"  /* Destruction of Right Shoulder Tendon, Percutaneous Approach */
"0L514ZZ"  =  "1"  /* Destruction of Right Shoulder Tendon, Perc Endo Approach */
"0L520ZZ"  =  "1"  /* Destruction of Left Shoulder Tendon, Open Approach */
"0L523ZZ"  =  "1"  /* Destruction of Left Shoulder Tendon, Percutaneous Approach */
"0L524ZZ"  =  "1"  /* Destruction of Left Shoulder Tendon, Perc Endo Approach */
"0L530ZZ"  =  "1"  /* Destruction of Right Upper Arm Tendon, Open Approach */
"0L533ZZ"  =  "1"  /* Destruction of Right Upper Arm Tendon, Percutaneous Approach */
"0L534ZZ"  =  "1"  /* Destruction of Right Upper Arm Tendon, Perc Endo Approach */
"0L540ZZ"  =  "1"  /* Destruction of Left Upper Arm Tendon, Open Approach */
"0L543ZZ"  =  "1"  /* Destruction of Left Upper Arm Tendon, Percutaneous Approach */
"0L544ZZ"  =  "1"  /* Destruction of Left Upper Arm Tendon, Perc Endo Approach */
"0L550ZZ"  =  "1"  /* Destruction of R Low Arm & Wrist Tendon, Open Approach */
"0L553ZZ"  =  "1"  /* Destruction of R Low Arm & Wrist Tendon, Perc Approach */
"0L554ZZ"  =  "1"  /* Destruction of R Low Arm & Wrist Tendon, Perc Endo Approach */
"0L560ZZ"  =  "1"  /* Destruction of L Low Arm & Wrist Tendon, Open Approach */
"0L563ZZ"  =  "1"  /* Destruction of L Low Arm & Wrist Tendon, Perc Approach */
"0L564ZZ"  =  "1"  /* Destruction of L Low Arm & Wrist Tendon, Perc Endo Approach */
"0L570ZZ"  =  "1"  /* Destruction of Right Hand Tendon, Open Approach */
"0L573ZZ"  =  "1"  /* Destruction of Right Hand Tendon, Percutaneous Approach */
"0L574ZZ"  =  "1"  /* Destruction of Right Hand Tendon, Perc Endo Approach */
"0L580ZZ"  =  "1"  /* Destruction of Left Hand Tendon, Open Approach */
"0L583ZZ"  =  "1"  /* Destruction of Left Hand Tendon, Percutaneous Approach */
"0L584ZZ"  =  "1"  /* Destruction of Left Hand Tendon, Perc Endo Approach */
"0L590ZZ"  =  "1"  /* Destruction of Right Trunk Tendon, Open Approach */
"0L593ZZ"  =  "1"  /* Destruction of Right Trunk Tendon, Percutaneous Approach */
"0L594ZZ"  =  "1"  /* Destruction of Right Trunk Tendon, Perc Endo Approach */
"0L5B0ZZ"  =  "1"  /* Destruction of Left Trunk Tendon, Open Approach */
"0L5B3ZZ"  =  "1"  /* Destruction of Left Trunk Tendon, Percutaneous Approach */
"0L5B4ZZ"  =  "1"  /* Destruction of Left Trunk Tendon, Perc Endo Approach */
"0L5C0ZZ"  =  "1"  /* Destruction of Right Thorax Tendon, Open Approach */
"0L5C3ZZ"  =  "1"  /* Destruction of Right Thorax Tendon, Percutaneous Approach */
"0L5C4ZZ"  =  "1"  /* Destruction of Right Thorax Tendon, Perc Endo Approach */
"0L5D0ZZ"  =  "1"  /* Destruction of Left Thorax Tendon, Open Approach */
"0L5D3ZZ"  =  "1"  /* Destruction of Left Thorax Tendon, Percutaneous Approach */
"0L5D4ZZ"  =  "1"  /* Destruction of Left Thorax Tendon, Perc Endo Approach */
"0L5F0ZZ"  =  "1"  /* Destruction of Right Abdomen Tendon, Open Approach */
"0L5F3ZZ"  =  "1"  /* Destruction of Right Abdomen Tendon, Percutaneous Approach */
"0L5F4ZZ"  =  "1"  /* Destruction of Right Abdomen Tendon, Perc Endo Approach */
"0L5G0ZZ"  =  "1"  /* Destruction of Left Abdomen Tendon, Open Approach */
"0L5G3ZZ"  =  "1"  /* Destruction of Left Abdomen Tendon, Percutaneous Approach */
"0L5G4ZZ"  =  "1"  /* Destruction of Left Abdomen Tendon, Perc Endo Approach */
"0L5H0ZZ"  =  "1"  /* Destruction of Perineum Tendon, Open Approach */
"0L5H3ZZ"  =  "1"  /* Destruction of Perineum Tendon, Percutaneous Approach */
"0L5H4ZZ"  =  "1"  /* Destruction of Perineum Tendon, Perc Endo Approach */
"0L5J0ZZ"  =  "1"  /* Destruction of Right Hip Tendon, Open Approach */
"0L5J3ZZ"  =  "1"  /* Destruction of Right Hip Tendon, Percutaneous Approach */
"0L5J4ZZ"  =  "1"  /* Destruction of Right Hip Tendon, Perc Endo Approach */
"0L5K0ZZ"  =  "1"  /* Destruction of Left Hip Tendon, Open Approach */
"0L5K3ZZ"  =  "1"  /* Destruction of Left Hip Tendon, Percutaneous Approach */
"0L5K4ZZ"  =  "1"  /* Destruction of Left Hip Tendon, Perc Endo Approach */
"0L5L0ZZ"  =  "1"  /* Destruction of Right Upper Leg Tendon, Open Approach */
"0L5L3ZZ"  =  "1"  /* Destruction of Right Upper Leg Tendon, Percutaneous Approach */
"0L5L4ZZ"  =  "1"  /* Destruction of Right Upper Leg Tendon, Perc Endo Approach */
"0L5M0ZZ"  =  "1"  /* Destruction of Left Upper Leg Tendon, Open Approach */
"0L5M3ZZ"  =  "1"  /* Destruction of Left Upper Leg Tendon, Percutaneous Approach */
"0L5M4ZZ"  =  "1"  /* Destruction of Left Upper Leg Tendon, Perc Endo Approach */
"0L5N0ZZ"  =  "1"  /* Destruction of Right Lower Leg Tendon, Open Approach */
"0L5N3ZZ"  =  "1"  /* Destruction of Right Lower Leg Tendon, Percutaneous Approach */
"0L5N4ZZ"  =  "1"  /* Destruction of Right Lower Leg Tendon, Perc Endo Approach */
"0L5P0ZZ"  =  "1"  /* Destruction of Left Lower Leg Tendon, Open Approach */
"0L5P3ZZ"  =  "1"  /* Destruction of Left Lower Leg Tendon, Percutaneous Approach */
"0L5P4ZZ"  =  "1"  /* Destruction of Left Lower Leg Tendon, Perc Endo Approach */
"0L5Q0ZZ"  =  "1"  /* Destruction of Right Knee Tendon, Open Approach */
"0L5Q3ZZ"  =  "1"  /* Destruction of Right Knee Tendon, Percutaneous Approach */
"0L5Q4ZZ"  =  "1"  /* Destruction of Right Knee Tendon, Perc Endo Approach */
"0L5R0ZZ"  =  "1"  /* Destruction of Left Knee Tendon, Open Approach */
"0L5R3ZZ"  =  "1"  /* Destruction of Left Knee Tendon, Percutaneous Approach */
"0L5R4ZZ"  =  "1"  /* Destruction of Left Knee Tendon, Perc Endo Approach */
"0L5S0ZZ"  =  "1"  /* Destruction of Right Ankle Tendon, Open Approach */
"0L5S3ZZ"  =  "1"  /* Destruction of Right Ankle Tendon, Percutaneous Approach */
"0L5S4ZZ"  =  "1"  /* Destruction of Right Ankle Tendon, Perc Endo Approach */
"0L5T0ZZ"  =  "1"  /* Destruction of Left Ankle Tendon, Open Approach */
"0L5T3ZZ"  =  "1"  /* Destruction of Left Ankle Tendon, Percutaneous Approach */
"0L5T4ZZ"  =  "1"  /* Destruction of Left Ankle Tendon, Perc Endo Approach */
"0L5V0ZZ"  =  "1"  /* Destruction of Right Foot Tendon, Open Approach */
"0L5V3ZZ"  =  "1"  /* Destruction of Right Foot Tendon, Percutaneous Approach */
"0L5V4ZZ"  =  "1"  /* Destruction of Right Foot Tendon, Perc Endo Approach */
"0L5W0ZZ"  =  "1"  /* Destruction of Left Foot Tendon, Open Approach */
"0L5W3ZZ"  =  "1"  /* Destruction of Left Foot Tendon, Percutaneous Approach */
"0L5W4ZZ"  =  "1"  /* Destruction of Left Foot Tendon, Perc Endo Approach */
"0L800ZZ"  =  "1"  /* Division of Head and Neck Tendon, Open Approach */
"0L803ZZ"  =  "1"  /* Division of Head and Neck Tendon, Percutaneous Approach */
"0L804ZZ"  =  "1"  /* Division of Head and Neck Tendon, Perc Endo Approach */
"0L810ZZ"  =  "1"  /* Division of Right Shoulder Tendon, Open Approach */
"0L813ZZ"  =  "1"  /* Division of Right Shoulder Tendon, Percutaneous Approach */
"0L814ZZ"  =  "1"  /* Division of Right Shoulder Tendon, Perc Endo Approach */
"0L820ZZ"  =  "1"  /* Division of Left Shoulder Tendon, Open Approach */
"0L823ZZ"  =  "1"  /* Division of Left Shoulder Tendon, Percutaneous Approach */
"0L824ZZ"  =  "1"  /* Division of Left Shoulder Tendon, Perc Endo Approach */
"0L830ZZ"  =  "1"  /* Division of Right Upper Arm Tendon, Open Approach */
"0L833ZZ"  =  "1"  /* Division of Right Upper Arm Tendon, Percutaneous Approach */
"0L834ZZ"  =  "1"  /* Division of Right Upper Arm Tendon, Perc Endo Approach */
"0L840ZZ"  =  "1"  /* Division of Left Upper Arm Tendon, Open Approach */
"0L843ZZ"  =  "1"  /* Division of Left Upper Arm Tendon, Percutaneous Approach */
"0L844ZZ"  =  "1"  /* Division of Left Upper Arm Tendon, Perc Endo Approach */
"0L850ZZ"  =  "1"  /* Division of Right Lower Arm and Wrist Tendon, Open Approach */
"0L853ZZ"  =  "1"  /* Division of Right Lower Arm and Wrist Tendon, Perc Approach */
"0L854ZZ"  =  "1"  /* Division of R Low Arm & Wrist Tendon, Perc Endo Approach */
"0L860ZZ"  =  "1"  /* Division of Left Lower Arm and Wrist Tendon, Open Approach */
"0L863ZZ"  =  "1"  /* Division of Left Lower Arm and Wrist Tendon, Perc Approach */
"0L864ZZ"  =  "1"  /* Division of L Low Arm & Wrist Tendon, Perc Endo Approach */
"0L870ZZ"  =  "1"  /* Division of Right Hand Tendon, Open Approach */
"0L873ZZ"  =  "1"  /* Division of Right Hand Tendon, Percutaneous Approach */
"0L874ZZ"  =  "1"  /* Division of Right Hand Tendon, Perc Endo Approach */
"0L880ZZ"  =  "1"  /* Division of Left Hand Tendon, Open Approach */
"0L883ZZ"  =  "1"  /* Division of Left Hand Tendon, Percutaneous Approach */
"0L884ZZ"  =  "1"  /* Division of Left Hand Tendon, Perc Endo Approach */
"0L890ZZ"  =  "1"  /* Division of Right Trunk Tendon, Open Approach */
"0L893ZZ"  =  "1"  /* Division of Right Trunk Tendon, Percutaneous Approach */
"0L894ZZ"  =  "1"  /* Division of Right Trunk Tendon, Perc Endo Approach */
"0L8B0ZZ"  =  "1"  /* Division of Left Trunk Tendon, Open Approach */
"0L8B3ZZ"  =  "1"  /* Division of Left Trunk Tendon, Percutaneous Approach */
"0L8B4ZZ"  =  "1"  /* Division of Left Trunk Tendon, Perc Endo Approach */
"0L8C0ZZ"  =  "1"  /* Division of Right Thorax Tendon, Open Approach */
"0L8C3ZZ"  =  "1"  /* Division of Right Thorax Tendon, Percutaneous Approach */
"0L8C4ZZ"  =  "1"  /* Division of Right Thorax Tendon, Perc Endo Approach */
"0L8D0ZZ"  =  "1"  /* Division of Left Thorax Tendon, Open Approach */
"0L8D3ZZ"  =  "1"  /* Division of Left Thorax Tendon, Percutaneous Approach */
"0L8D4ZZ"  =  "1"  /* Division of Left Thorax Tendon, Perc Endo Approach */
"0L8F0ZZ"  =  "1"  /* Division of Right Abdomen Tendon, Open Approach */
"0L8F3ZZ"  =  "1"  /* Division of Right Abdomen Tendon, Percutaneous Approach */
"0L8F4ZZ"  =  "1"  /* Division of Right Abdomen Tendon, Perc Endo Approach */
"0L8G0ZZ"  =  "1"  /* Division of Left Abdomen Tendon, Open Approach */
"0L8G3ZZ"  =  "1"  /* Division of Left Abdomen Tendon, Percutaneous Approach */
"0L8G4ZZ"  =  "1"  /* Division of Left Abdomen Tendon, Perc Endo Approach */
"0L8H0ZZ"  =  "1"  /* Division of Perineum Tendon, Open Approach */
"0L8H3ZZ"  =  "1"  /* Division of Perineum Tendon, Percutaneous Approach */
"0L8H4ZZ"  =  "1"  /* Division of Perineum Tendon, Perc Endo Approach */
"0L8J0ZZ"  =  "1"  /* Division of Right Hip Tendon, Open Approach */
"0L8J3ZZ"  =  "1"  /* Division of Right Hip Tendon, Percutaneous Approach */
"0L8J4ZZ"  =  "1"  /* Division of Right Hip Tendon, Perc Endo Approach */
"0L8K0ZZ"  =  "1"  /* Division of Left Hip Tendon, Open Approach */
"0L8K3ZZ"  =  "1"  /* Division of Left Hip Tendon, Percutaneous Approach */
"0L8K4ZZ"  =  "1"  /* Division of Left Hip Tendon, Perc Endo Approach */
"0L8L0ZZ"  =  "1"  /* Division of Right Upper Leg Tendon, Open Approach */
"0L8L3ZZ"  =  "1"  /* Division of Right Upper Leg Tendon, Percutaneous Approach */
"0L8L4ZZ"  =  "1"  /* Division of Right Upper Leg Tendon, Perc Endo Approach */
"0L8M0ZZ"  =  "1"  /* Division of Left Upper Leg Tendon, Open Approach */
"0L8M3ZZ"  =  "1"  /* Division of Left Upper Leg Tendon, Percutaneous Approach */
"0L8M4ZZ"  =  "1"  /* Division of Left Upper Leg Tendon, Perc Endo Approach */
"0L8N0ZZ"  =  "1"  /* Division of Right Lower Leg Tendon, Open Approach */
"0L8N3ZZ"  =  "1"  /* Division of Right Lower Leg Tendon, Percutaneous Approach */
"0L8N4ZZ"  =  "1"  /* Division of Right Lower Leg Tendon, Perc Endo Approach */
"0L8P0ZZ"  =  "1"  /* Division of Left Lower Leg Tendon, Open Approach */
"0L8P3ZZ"  =  "1"  /* Division of Left Lower Leg Tendon, Percutaneous Approach */
"0L8P4ZZ"  =  "1"  /* Division of Left Lower Leg Tendon, Perc Endo Approach */
"0L8Q0ZZ"  =  "1"  /* Division of Right Knee Tendon, Open Approach */
"0L8Q3ZZ"  =  "1"  /* Division of Right Knee Tendon, Percutaneous Approach */
"0L8Q4ZZ"  =  "1"  /* Division of Right Knee Tendon, Perc Endo Approach */
"0L8R0ZZ"  =  "1"  /* Division of Left Knee Tendon, Open Approach */
"0L8R3ZZ"  =  "1"  /* Division of Left Knee Tendon, Percutaneous Approach */
"0L8R4ZZ"  =  "1"  /* Division of Left Knee Tendon, Perc Endo Approach */
"0L8S0ZZ"  =  "1"  /* Division of Right Ankle Tendon, Open Approach */
"0L8S3ZZ"  =  "1"  /* Division of Right Ankle Tendon, Percutaneous Approach */
"0L8S4ZZ"  =  "1"  /* Division of Right Ankle Tendon, Perc Endo Approach */
"0L8T0ZZ"  =  "1"  /* Division of Left Ankle Tendon, Open Approach */
"0L8T3ZZ"  =  "1"  /* Division of Left Ankle Tendon, Percutaneous Approach */
"0L8T4ZZ"  =  "1"  /* Division of Left Ankle Tendon, Perc Endo Approach */
"0L8V0ZZ"  =  "1"  /* Division of Right Foot Tendon, Open Approach */
"0L8V3ZZ"  =  "1"  /* Division of Right Foot Tendon, Percutaneous Approach */
"0L8V4ZZ"  =  "1"  /* Division of Right Foot Tendon, Perc Endo Approach */
"0L8W0ZZ"  =  "1"  /* Division of Left Foot Tendon, Open Approach */
"0L8W3ZZ"  =  "1"  /* Division of Left Foot Tendon, Percutaneous Approach */
"0L8W4ZZ"  =  "1"  /* Division of Left Foot Tendon, Perc Endo Approach */
"0L9000Z"  =  "1"  /* Drainage of Head & Neck Tendon with Drain Dev, Open Approach */
"0L900ZX"  =  "1"  /* Drainage of Head and Neck Tendon, Open Approach, Diagnostic */
"0L900ZZ"  =  "1"  /* Drainage of Head and Neck Tendon, Open Approach */
"0L9030Z"  =  "1"  /* Drainage of Head & Neck Tendon with Drain Dev, Perc Approach */
"0L903ZX"  =  "1"  /* Drainage of Head and Neck Tendon, Perc Approach, Diagn */
"0L903ZZ"  =  "1"  /* Drainage of Head and Neck Tendon, Percutaneous Approach */
"0L9040Z"  =  "1"  /* Drain Head & Neck Tendon w Drain Dev, Perc Endo */
"0L904ZX"  =  "1"  /* Drainage of Head and Neck Tendon, Perc Endo Approach, Diagn */
"0L904ZZ"  =  "1"  /* Drainage of Head and Neck Tendon, Perc Endo Approach */
"0L9100Z"  =  "1"  /* Drainage of R Shoulder Tendon with Drain Dev, Open Approach */
"0L910ZX"  =  "1"  /* Drainage of Right Shoulder Tendon, Open Approach, Diagnostic */
"0L910ZZ"  =  "1"  /* Drainage of Right Shoulder Tendon, Open Approach */
"0L9130Z"  =  "1"  /* Drainage of R Shoulder Tendon with Drain Dev, Perc Approach */
"0L913ZX"  =  "1"  /* Drainage of Right Shoulder Tendon, Perc Approach, Diagn */
"0L913ZZ"  =  "1"  /* Drainage of Right Shoulder Tendon, Percutaneous Approach */
"0L9140Z"  =  "1"  /* Drain R Shoulder Tendon w Drain Dev, Perc Endo */
"0L914ZX"  =  "1"  /* Drainage of Right Shoulder Tendon, Perc Endo Approach, Diagn */
"0L914ZZ"  =  "1"  /* Drainage of Right Shoulder Tendon, Perc Endo Approach */
"0L9200Z"  =  "1"  /* Drainage of L Shoulder Tendon with Drain Dev, Open Approach */
"0L920ZX"  =  "1"  /* Drainage of Left Shoulder Tendon, Open Approach, Diagnostic */
"0L920ZZ"  =  "1"  /* Drainage of Left Shoulder Tendon, Open Approach */
"0L9230Z"  =  "1"  /* Drainage of L Shoulder Tendon with Drain Dev, Perc Approach */
"0L923ZX"  =  "1"  /* Drainage of Left Shoulder Tendon, Perc Approach, Diagn */
"0L923ZZ"  =  "1"  /* Drainage of Left Shoulder Tendon, Percutaneous Approach */
"0L9240Z"  =  "1"  /* Drain L Shoulder Tendon w Drain Dev, Perc Endo */
"0L924ZX"  =  "1"  /* Drainage of Left Shoulder Tendon, Perc Endo Approach, Diagn */
"0L924ZZ"  =  "1"  /* Drainage of Left Shoulder Tendon, Perc Endo Approach */
"0L9300Z"  =  "1"  /* Drainage of R Up Arm Tendon with Drain Dev, Open Approach */
"0L930ZX"  =  "1"  /* Drainage of Right Upper Arm Tendon, Open Approach, Diagn */
"0L930ZZ"  =  "1"  /* Drainage of Right Upper Arm Tendon, Open Approach */
"0L9330Z"  =  "1"  /* Drainage of R Up Arm Tendon with Drain Dev, Perc Approach */
"0L933ZX"  =  "1"  /* Drainage of Right Upper Arm Tendon, Perc Approach, Diagn */
"0L933ZZ"  =  "1"  /* Drainage of Right Upper Arm Tendon, Percutaneous Approach */
"0L9340Z"  =  "1"  /* Drain of R Up Arm Tendon with Drain Dev, Perc Endo Approach */
"0L934ZX"  =  "1"  /* Drainage of R Up Arm Tendon, Perc Endo Approach, Diagn */
"0L934ZZ"  =  "1"  /* Drainage of Right Upper Arm Tendon, Perc Endo Approach */
"0L9400Z"  =  "1"  /* Drainage of L Up Arm Tendon with Drain Dev, Open Approach */
"0L940ZX"  =  "1"  /* Drainage of Left Upper Arm Tendon, Open Approach, Diagnostic */
"0L940ZZ"  =  "1"  /* Drainage of Left Upper Arm Tendon, Open Approach */
"0L9430Z"  =  "1"  /* Drainage of L Up Arm Tendon with Drain Dev, Perc Approach */
"0L943ZX"  =  "1"  /* Drainage of Left Upper Arm Tendon, Perc Approach, Diagn */
"0L943ZZ"  =  "1"  /* Drainage of Left Upper Arm Tendon, Percutaneous Approach */
"0L9440Z"  =  "1"  /* Drain of L Up Arm Tendon with Drain Dev, Perc Endo Approach */
"0L944ZX"  =  "1"  /* Drainage of Left Upper Arm Tendon, Perc Endo Approach, Diagn */
"0L944ZZ"  =  "1"  /* Drainage of Left Upper Arm Tendon, Perc Endo Approach */
"0L9500Z"  =  "1"  /* Drain R Low Arm & Wrist Tendon w Drain Dev, Open */
"0L950ZX"  =  "1"  /* Drainage of R Low Arm & Wrist Tendon, Open Approach, Diagn */
"0L950ZZ"  =  "1"  /* Drainage of Right Lower Arm and Wrist Tendon, Open Approach */
"0L9530Z"  =  "1"  /* Drain R Low Arm & Wrist Tendon w Drain Dev, Perc */
"0L953ZX"  =  "1"  /* Drainage of R Low Arm & Wrist Tendon, Perc Approach, Diagn */
"0L953ZZ"  =  "1"  /* Drainage of Right Lower Arm and Wrist Tendon, Perc Approach */
"0L9540Z"  =  "1"  /* Drain R Low Arm & Wrist Tendon w Drain Dev, Perc Endo */
"0L954ZX"  =  "1"  /* Drain of R Low Arm & Wrist Tendon, Perc Endo Approach, Diagn */
"0L954ZZ"  =  "1"  /* Drainage of R Low Arm & Wrist Tendon, Perc Endo Approach */
"0L9600Z"  =  "1"  /* Drain L Low Arm & Wrist Tendon w Drain Dev, Open */
"0L960ZX"  =  "1"  /* Drainage of L Low Arm & Wrist Tendon, Open Approach, Diagn */
"0L960ZZ"  =  "1"  /* Drainage of Left Lower Arm and Wrist Tendon, Open Approach */
"0L9630Z"  =  "1"  /* Drain L Low Arm & Wrist Tendon w Drain Dev, Perc */
"0L963ZX"  =  "1"  /* Drainage of L Low Arm & Wrist Tendon, Perc Approach, Diagn */
"0L963ZZ"  =  "1"  /* Drainage of Left Lower Arm and Wrist Tendon, Perc Approach */
"0L9640Z"  =  "1"  /* Drain L Low Arm & Wrist Tendon w Drain Dev, Perc Endo */
"0L964ZX"  =  "1"  /* Drain of L Low Arm & Wrist Tendon, Perc Endo Approach, Diagn */
"0L964ZZ"  =  "1"  /* Drainage of L Low Arm & Wrist Tendon, Perc Endo Approach */
"0L9700Z"  =  "1"  /* Drainage of Right Hand Tendon with Drain Dev, Open Approach */
"0L970ZX"  =  "1"  /* Drainage of Right Hand Tendon, Open Approach, Diagnostic */
"0L970ZZ"  =  "1"  /* Drainage of Right Hand Tendon, Open Approach */
"0L9730Z"  =  "1"  /* Drainage of Right Hand Tendon with Drain Dev, Perc Approach */
"0L973ZX"  =  "1"  /* Drainage of Right Hand Tendon, Percutaneous Approach, Diagn */
"0L9740Z"  =  "1"  /* Drainage of R Hand Tendon with Drain Dev, Perc Endo Approach */
"0L974ZX"  =  "1"  /* Drainage of Right Hand Tendon, Perc Endo Approach, Diagn */
"0L9800Z"  =  "1"  /* Drainage of Left Hand Tendon with Drain Dev, Open Approach */
"0L980ZX"  =  "1"  /* Drainage of Left Hand Tendon, Open Approach, Diagnostic */
"0L980ZZ"  =  "1"  /* Drainage of Left Hand Tendon, Open Approach */
"0L9830Z"  =  "1"  /* Drainage of Left Hand Tendon with Drain Dev, Perc Approach */
"0L983ZX"  =  "1"  /* Drainage of Left Hand Tendon, Percutaneous Approach, Diagn */
"0L9840Z"  =  "1"  /* Drainage of L Hand Tendon with Drain Dev, Perc Endo Approach */
"0L984ZX"  =  "1"  /* Drainage of Left Hand Tendon, Perc Endo Approach, Diagn */
"0L9900Z"  =  "1"  /* Drainage of Right Trunk Tendon with Drain Dev, Open Approach */
"0L990ZX"  =  "1"  /* Drainage of Right Trunk Tendon, Open Approach, Diagnostic */
"0L990ZZ"  =  "1"  /* Drainage of Right Trunk Tendon, Open Approach */
"0L9930Z"  =  "1"  /* Drainage of Right Trunk Tendon with Drain Dev, Perc Approach */
"0L993ZX"  =  "1"  /* Drainage of Right Trunk Tendon, Percutaneous Approach, Diagn */
"0L993ZZ"  =  "1"  /* Drainage of Right Trunk Tendon, Percutaneous Approach */
"0L9940Z"  =  "1"  /* Drain of R Trunk Tendon with Drain Dev, Perc Endo Approach */
"0L994ZX"  =  "1"  /* Drainage of Right Trunk Tendon, Perc Endo Approach, Diagn */
"0L994ZZ"  =  "1"  /* Drainage of Right Trunk Tendon, Perc Endo Approach */
"0L9B00Z"  =  "1"  /* Drainage of Left Trunk Tendon with Drain Dev, Open Approach */
"0L9B0ZX"  =  "1"  /* Drainage of Left Trunk Tendon, Open Approach, Diagnostic */
"0L9B0ZZ"  =  "1"  /* Drainage of Left Trunk Tendon, Open Approach */
"0L9B30Z"  =  "1"  /* Drainage of Left Trunk Tendon with Drain Dev, Perc Approach */
"0L9B3ZX"  =  "1"  /* Drainage of Left Trunk Tendon, Percutaneous Approach, Diagn */
"0L9B3ZZ"  =  "1"  /* Drainage of Left Trunk Tendon, Percutaneous Approach */
"0L9B40Z"  =  "1"  /* Drain of L Trunk Tendon with Drain Dev, Perc Endo Approach */
"0L9B4ZX"  =  "1"  /* Drainage of Left Trunk Tendon, Perc Endo Approach, Diagn */
"0L9B4ZZ"  =  "1"  /* Drainage of Left Trunk Tendon, Perc Endo Approach */
"0L9C00Z"  =  "1"  /* Drainage of R Thorax Tendon with Drain Dev, Open Approach */
"0L9C0ZX"  =  "1"  /* Drainage of Right Thorax Tendon, Open Approach, Diagnostic */
"0L9C0ZZ"  =  "1"  /* Drainage of Right Thorax Tendon, Open Approach */
"0L9C30Z"  =  "1"  /* Drainage of R Thorax Tendon with Drain Dev, Perc Approach */
"0L9C3ZX"  =  "1"  /* Drainage of Right Thorax Tendon, Perc Approach, Diagn */
"0L9C3ZZ"  =  "1"  /* Drainage of Right Thorax Tendon, Percutaneous Approach */
"0L9C40Z"  =  "1"  /* Drain of R Thorax Tendon with Drain Dev, Perc Endo Approach */
"0L9C4ZX"  =  "1"  /* Drainage of Right Thorax Tendon, Perc Endo Approach, Diagn */
"0L9C4ZZ"  =  "1"  /* Drainage of Right Thorax Tendon, Perc Endo Approach */
"0L9D00Z"  =  "1"  /* Drainage of Left Thorax Tendon with Drain Dev, Open Approach */
"0L9D0ZX"  =  "1"  /* Drainage of Left Thorax Tendon, Open Approach, Diagnostic */
"0L9D0ZZ"  =  "1"  /* Drainage of Left Thorax Tendon, Open Approach */
"0L9D30Z"  =  "1"  /* Drainage of Left Thorax Tendon with Drain Dev, Perc Approach */
"0L9D3ZX"  =  "1"  /* Drainage of Left Thorax Tendon, Percutaneous Approach, Diagn */
"0L9D3ZZ"  =  "1"  /* Drainage of Left Thorax Tendon, Percutaneous Approach */
"0L9D40Z"  =  "1"  /* Drain of L Thorax Tendon with Drain Dev, Perc Endo Approach */
"0L9D4ZX"  =  "1"  /* Drainage of Left Thorax Tendon, Perc Endo Approach, Diagn */
"0L9D4ZZ"  =  "1"  /* Drainage of Left Thorax Tendon, Perc Endo Approach */
"0L9F00Z"  =  "1"  /* Drainage of R Abd Tendon with Drain Dev, Open Approach */
"0L9F0ZX"  =  "1"  /* Drainage of Right Abdomen Tendon, Open Approach, Diagnostic */
"0L9F0ZZ"  =  "1"  /* Drainage of Right Abdomen Tendon, Open Approach */
"0L9F30Z"  =  "1"  /* Drainage of R Abd Tendon with Drain Dev, Perc Approach */
"0L9F3ZX"  =  "1"  /* Drainage of Right Abdomen Tendon, Perc Approach, Diagn */
"0L9F3ZZ"  =  "1"  /* Drainage of Right Abdomen Tendon, Percutaneous Approach */
"0L9F40Z"  =  "1"  /* Drainage of R Abd Tendon with Drain Dev, Perc Endo Approach */
"0L9F4ZX"  =  "1"  /* Drainage of Right Abdomen Tendon, Perc Endo Approach, Diagn */
"0L9F4ZZ"  =  "1"  /* Drainage of Right Abdomen Tendon, Perc Endo Approach */
"0L9G00Z"  =  "1"  /* Drainage of L Abd Tendon with Drain Dev, Open Approach */
"0L9G0ZX"  =  "1"  /* Drainage of Left Abdomen Tendon, Open Approach, Diagnostic */
"0L9G0ZZ"  =  "1"  /* Drainage of Left Abdomen Tendon, Open Approach */
"0L9G30Z"  =  "1"  /* Drainage of L Abd Tendon with Drain Dev, Perc Approach */
"0L9G3ZX"  =  "1"  /* Drainage of Left Abdomen Tendon, Perc Approach, Diagn */
"0L9G3ZZ"  =  "1"  /* Drainage of Left Abdomen Tendon, Percutaneous Approach */
"0L9G40Z"  =  "1"  /* Drainage of L Abd Tendon with Drain Dev, Perc Endo Approach */
"0L9G4ZX"  =  "1"  /* Drainage of Left Abdomen Tendon, Perc Endo Approach, Diagn */
"0L9G4ZZ"  =  "1"  /* Drainage of Left Abdomen Tendon, Perc Endo Approach */
"0L9H00Z"  =  "1"  /* Drainage of Perineum Tendon with Drain Dev, Open Approach */
"0L9H0ZX"  =  "1"  /* Drainage of Perineum Tendon, Open Approach, Diagnostic */
"0L9H0ZZ"  =  "1"  /* Drainage of Perineum Tendon, Open Approach */
"0L9H30Z"  =  "1"  /* Drainage of Perineum Tendon with Drain Dev, Perc Approach */
"0L9H3ZX"  =  "1"  /* Drainage of Perineum Tendon, Percutaneous Approach, Diagn */
"0L9H3ZZ"  =  "1"  /* Drainage of Perineum Tendon, Percutaneous Approach */
"0L9H40Z"  =  "1"  /* Drain of Perineum Tendon with Drain Dev, Perc Endo Approach */
"0L9H4ZX"  =  "1"  /* Drainage of Perineum Tendon, Perc Endo Approach, Diagn */
"0L9H4ZZ"  =  "1"  /* Drainage of Perineum Tendon, Perc Endo Approach */
"0L9J00Z"  =  "1"  /* Drainage of Right Hip Tendon with Drain Dev, Open Approach */
"0L9J0ZX"  =  "1"  /* Drainage of Right Hip Tendon, Open Approach, Diagnostic */
"0L9J0ZZ"  =  "1"  /* Drainage of Right Hip Tendon, Open Approach */
"0L9J30Z"  =  "1"  /* Drainage of Right Hip Tendon with Drain Dev, Perc Approach */
"0L9J3ZX"  =  "1"  /* Drainage of Right Hip Tendon, Percutaneous Approach, Diagn */
"0L9J3ZZ"  =  "1"  /* Drainage of Right Hip Tendon, Percutaneous Approach */
"0L9J40Z"  =  "1"  /* Drainage of R Hip Tendon with Drain Dev, Perc Endo Approach */
"0L9J4ZX"  =  "1"  /* Drainage of Right Hip Tendon, Perc Endo Approach, Diagn */
"0L9J4ZZ"  =  "1"  /* Drainage of Right Hip Tendon, Perc Endo Approach */
"0L9K00Z"  =  "1"  /* Drainage of Left Hip Tendon with Drain Dev, Open Approach */
"0L9K0ZX"  =  "1"  /* Drainage of Left Hip Tendon, Open Approach, Diagnostic */
"0L9K0ZZ"  =  "1"  /* Drainage of Left Hip Tendon, Open Approach */
"0L9K30Z"  =  "1"  /* Drainage of Left Hip Tendon with Drain Dev, Perc Approach */
"0L9K3ZX"  =  "1"  /* Drainage of Left Hip Tendon, Percutaneous Approach, Diagn */
"0L9K3ZZ"  =  "1"  /* Drainage of Left Hip Tendon, Percutaneous Approach */
"0L9K40Z"  =  "1"  /* Drainage of L Hip Tendon with Drain Dev, Perc Endo Approach */
"0L9K4ZX"  =  "1"  /* Drainage of Left Hip Tendon, Perc Endo Approach, Diagn */
"0L9K4ZZ"  =  "1"  /* Drainage of Left Hip Tendon, Perc Endo Approach */
"0L9L00Z"  =  "1"  /* Drainage of R Up Leg Tendon with Drain Dev, Open Approach */
"0L9L0ZX"  =  "1"  /* Drainage of Right Upper Leg Tendon, Open Approach, Diagn */
"0L9L0ZZ"  =  "1"  /* Drainage of Right Upper Leg Tendon, Open Approach */
"0L9L30Z"  =  "1"  /* Drainage of R Up Leg Tendon with Drain Dev, Perc Approach */
"0L9L3ZX"  =  "1"  /* Drainage of Right Upper Leg Tendon, Perc Approach, Diagn */
"0L9L3ZZ"  =  "1"  /* Drainage of Right Upper Leg Tendon, Percutaneous Approach */
"0L9L40Z"  =  "1"  /* Drain of R Up Leg Tendon with Drain Dev, Perc Endo Approach */
"0L9L4ZX"  =  "1"  /* Drainage of R Up Leg Tendon, Perc Endo Approach, Diagn */
"0L9L4ZZ"  =  "1"  /* Drainage of Right Upper Leg Tendon, Perc Endo Approach */
"0L9M00Z"  =  "1"  /* Drainage of L Up Leg Tendon with Drain Dev, Open Approach */
"0L9M0ZX"  =  "1"  /* Drainage of Left Upper Leg Tendon, Open Approach, Diagnostic */
"0L9M0ZZ"  =  "1"  /* Drainage of Left Upper Leg Tendon, Open Approach */
"0L9M30Z"  =  "1"  /* Drainage of L Up Leg Tendon with Drain Dev, Perc Approach */
"0L9M3ZX"  =  "1"  /* Drainage of Left Upper Leg Tendon, Perc Approach, Diagn */
"0L9M3ZZ"  =  "1"  /* Drainage of Left Upper Leg Tendon, Percutaneous Approach */
"0L9M40Z"  =  "1"  /* Drain of L Up Leg Tendon with Drain Dev, Perc Endo Approach */
"0L9M4ZX"  =  "1"  /* Drainage of Left Upper Leg Tendon, Perc Endo Approach, Diagn */
"0L9M4ZZ"  =  "1"  /* Drainage of Left Upper Leg Tendon, Perc Endo Approach */
"0L9N00Z"  =  "1"  /* Drainage of R Low Leg Tendon with Drain Dev, Open Approach */
"0L9N0ZX"  =  "1"  /* Drainage of Right Lower Leg Tendon, Open Approach, Diagn */
"0L9N0ZZ"  =  "1"  /* Drainage of Right Lower Leg Tendon, Open Approach */
"0L9N30Z"  =  "1"  /* Drainage of R Low Leg Tendon with Drain Dev, Perc Approach */
"0L9N3ZX"  =  "1"  /* Drainage of Right Lower Leg Tendon, Perc Approach, Diagn */
"0L9N3ZZ"  =  "1"  /* Drainage of Right Lower Leg Tendon, Percutaneous Approach */
"0L9N40Z"  =  "1"  /* Drain of R Low Leg Tendon with Drain Dev, Perc Endo Approach */
"0L9N4ZX"  =  "1"  /* Drainage of R Low Leg Tendon, Perc Endo Approach, Diagn */
"0L9N4ZZ"  =  "1"  /* Drainage of Right Lower Leg Tendon, Perc Endo Approach */
"0L9P00Z"  =  "1"  /* Drainage of L Low Leg Tendon with Drain Dev, Open Approach */
"0L9P0ZX"  =  "1"  /* Drainage of Left Lower Leg Tendon, Open Approach, Diagnostic */
"0L9P0ZZ"  =  "1"  /* Drainage of Left Lower Leg Tendon, Open Approach */
"0L9P30Z"  =  "1"  /* Drainage of L Low Leg Tendon with Drain Dev, Perc Approach */
"0L9P3ZX"  =  "1"  /* Drainage of Left Lower Leg Tendon, Perc Approach, Diagn */
"0L9P3ZZ"  =  "1"  /* Drainage of Left Lower Leg Tendon, Percutaneous Approach */
"0L9P40Z"  =  "1"  /* Drain of L Low Leg Tendon with Drain Dev, Perc Endo Approach */
"0L9P4ZX"  =  "1"  /* Drainage of Left Lower Leg Tendon, Perc Endo Approach, Diagn */
"0L9P4ZZ"  =  "1"  /* Drainage of Left Lower Leg Tendon, Perc Endo Approach */
"0L9Q00Z"  =  "1"  /* Drainage of Right Knee Tendon with Drain Dev, Open Approach */
"0L9Q0ZX"  =  "1"  /* Drainage of Right Knee Tendon, Open Approach, Diagnostic */
"0L9Q0ZZ"  =  "1"  /* Drainage of Right Knee Tendon, Open Approach */
"0L9Q30Z"  =  "1"  /* Drainage of Right Knee Tendon with Drain Dev, Perc Approach */
"0L9Q3ZX"  =  "1"  /* Drainage of Right Knee Tendon, Percutaneous Approach, Diagn */
"0L9Q3ZZ"  =  "1"  /* Drainage of Right Knee Tendon, Percutaneous Approach */
"0L9Q40Z"  =  "1"  /* Drainage of R Knee Tendon with Drain Dev, Perc Endo Approach */
"0L9Q4ZX"  =  "1"  /* Drainage of Right Knee Tendon, Perc Endo Approach, Diagn */
"0L9Q4ZZ"  =  "1"  /* Drainage of Right Knee Tendon, Perc Endo Approach */
"0L9R00Z"  =  "1"  /* Drainage of Left Knee Tendon with Drain Dev, Open Approach */
"0L9R0ZX"  =  "1"  /* Drainage of Left Knee Tendon, Open Approach, Diagnostic */
"0L9R0ZZ"  =  "1"  /* Drainage of Left Knee Tendon, Open Approach */
"0L9R30Z"  =  "1"  /* Drainage of Left Knee Tendon with Drain Dev, Perc Approach */
"0L9R3ZX"  =  "1"  /* Drainage of Left Knee Tendon, Percutaneous Approach, Diagn */
"0L9R3ZZ"  =  "1"  /* Drainage of Left Knee Tendon, Percutaneous Approach */
"0L9R40Z"  =  "1"  /* Drainage of L Knee Tendon with Drain Dev, Perc Endo Approach */
"0L9R4ZX"  =  "1"  /* Drainage of Left Knee Tendon, Perc Endo Approach, Diagn */
"0L9R4ZZ"  =  "1"  /* Drainage of Left Knee Tendon, Perc Endo Approach */
"0L9S00Z"  =  "1"  /* Drainage of Right Ankle Tendon with Drain Dev, Open Approach */
"0L9S0ZX"  =  "1"  /* Drainage of Right Ankle Tendon, Open Approach, Diagnostic */
"0L9S0ZZ"  =  "1"  /* Drainage of Right Ankle Tendon, Open Approach */
"0L9S30Z"  =  "1"  /* Drainage of Right Ankle Tendon with Drain Dev, Perc Approach */
"0L9S3ZX"  =  "1"  /* Drainage of Right Ankle Tendon, Percutaneous Approach, Diagn */
"0L9S3ZZ"  =  "1"  /* Drainage of Right Ankle Tendon, Percutaneous Approach */
"0L9S40Z"  =  "1"  /* Drain of R Ankle Tendon with Drain Dev, Perc Endo Approach */
"0L9S4ZX"  =  "1"  /* Drainage of Right Ankle Tendon, Perc Endo Approach, Diagn */
"0L9S4ZZ"  =  "1"  /* Drainage of Right Ankle Tendon, Perc Endo Approach */
"0L9T00Z"  =  "1"  /* Drainage of Left Ankle Tendon with Drain Dev, Open Approach */
"0L9T0ZX"  =  "1"  /* Drainage of Left Ankle Tendon, Open Approach, Diagnostic */
"0L9T0ZZ"  =  "1"  /* Drainage of Left Ankle Tendon, Open Approach */
"0L9T30Z"  =  "1"  /* Drainage of Left Ankle Tendon with Drain Dev, Perc Approach */
"0L9T3ZX"  =  "1"  /* Drainage of Left Ankle Tendon, Percutaneous Approach, Diagn */
"0L9T3ZZ"  =  "1"  /* Drainage of Left Ankle Tendon, Percutaneous Approach */
"0L9T40Z"  =  "1"  /* Drain of L Ankle Tendon with Drain Dev, Perc Endo Approach */
"0L9T4ZX"  =  "1"  /* Drainage of Left Ankle Tendon, Perc Endo Approach, Diagn */
"0L9T4ZZ"  =  "1"  /* Drainage of Left Ankle Tendon, Perc Endo Approach */
"0L9V00Z"  =  "1"  /* Drainage of Right Foot Tendon with Drain Dev, Open Approach */
"0L9V0ZX"  =  "1"  /* Drainage of Right Foot Tendon, Open Approach, Diagnostic */
"0L9V0ZZ"  =  "1"  /* Drainage of Right Foot Tendon, Open Approach */
"0L9V30Z"  =  "1"  /* Drainage of Right Foot Tendon with Drain Dev, Perc Approach */
"0L9V3ZX"  =  "1"  /* Drainage of Right Foot Tendon, Percutaneous Approach, Diagn */
"0L9V3ZZ"  =  "1"  /* Drainage of Right Foot Tendon, Percutaneous Approach */
"0L9V40Z"  =  "1"  /* Drainage of R Foot Tendon with Drain Dev, Perc Endo Approach */
"0L9V4ZX"  =  "1"  /* Drainage of Right Foot Tendon, Perc Endo Approach, Diagn */
"0L9V4ZZ"  =  "1"  /* Drainage of Right Foot Tendon, Perc Endo Approach */
"0L9W00Z"  =  "1"  /* Drainage of Left Foot Tendon with Drain Dev, Open Approach */
"0L9W0ZX"  =  "1"  /* Drainage of Left Foot Tendon, Open Approach, Diagnostic */
"0L9W0ZZ"  =  "1"  /* Drainage of Left Foot Tendon, Open Approach */
"0L9W30Z"  =  "1"  /* Drainage of Left Foot Tendon with Drain Dev, Perc Approach */
"0L9W3ZX"  =  "1"  /* Drainage of Left Foot Tendon, Percutaneous Approach, Diagn */
"0L9W3ZZ"  =  "1"  /* Drainage of Left Foot Tendon, Percutaneous Approach */
"0L9W40Z"  =  "1"  /* Drainage of L Foot Tendon with Drain Dev, Perc Endo Approach */
"0L9W4ZX"  =  "1"  /* Drainage of Left Foot Tendon, Perc Endo Approach, Diagn */
"0L9W4ZZ"  =  "1"  /* Drainage of Left Foot Tendon, Perc Endo Approach */
"0LB00ZX"  =  "1"  /* Excision of Head and Neck Tendon, Open Approach, Diagnostic */
"0LB00ZZ"  =  "1"  /* Excision of Head and Neck Tendon, Open Approach */
"0LB03ZX"  =  "1"  /* Excision of Head and Neck Tendon, Perc Approach, Diagn */
"0LB03ZZ"  =  "1"  /* Excision of Head and Neck Tendon, Percutaneous Approach */
"0LB04ZX"  =  "1"  /* Excision of Head and Neck Tendon, Perc Endo Approach, Diagn */
"0LB04ZZ"  =  "1"  /* Excision of Head and Neck Tendon, Perc Endo Approach */
"0LB10ZX"  =  "1"  /* Excision of Right Shoulder Tendon, Open Approach, Diagnostic */
"0LB10ZZ"  =  "1"  /* Excision of Right Shoulder Tendon, Open Approach */
"0LB13ZX"  =  "1"  /* Excision of Right Shoulder Tendon, Perc Approach, Diagn */
"0LB13ZZ"  =  "1"  /* Excision of Right Shoulder Tendon, Percutaneous Approach */
"0LB14ZX"  =  "1"  /* Excision of Right Shoulder Tendon, Perc Endo Approach, Diagn */
"0LB14ZZ"  =  "1"  /* Excision of Right Shoulder Tendon, Perc Endo Approach */
"0LB20ZX"  =  "1"  /* Excision of Left Shoulder Tendon, Open Approach, Diagnostic */
"0LB20ZZ"  =  "1"  /* Excision of Left Shoulder Tendon, Open Approach */
"0LB23ZX"  =  "1"  /* Excision of Left Shoulder Tendon, Perc Approach, Diagn */
"0LB23ZZ"  =  "1"  /* Excision of Left Shoulder Tendon, Percutaneous Approach */
"0LB24ZX"  =  "1"  /* Excision of Left Shoulder Tendon, Perc Endo Approach, Diagn */
"0LB24ZZ"  =  "1"  /* Excision of Left Shoulder Tendon, Perc Endo Approach */
"0LB30ZX"  =  "1"  /* Excision of Right Upper Arm Tendon, Open Approach, Diagn */
"0LB30ZZ"  =  "1"  /* Excision of Right Upper Arm Tendon, Open Approach */
"0LB33ZX"  =  "1"  /* Excision of Right Upper Arm Tendon, Perc Approach, Diagn */
"0LB33ZZ"  =  "1"  /* Excision of Right Upper Arm Tendon, Percutaneous Approach */
"0LB34ZX"  =  "1"  /* Excision of R Up Arm Tendon, Perc Endo Approach, Diagn */
"0LB34ZZ"  =  "1"  /* Excision of Right Upper Arm Tendon, Perc Endo Approach */
"0LB40ZX"  =  "1"  /* Excision of Left Upper Arm Tendon, Open Approach, Diagnostic */
"0LB40ZZ"  =  "1"  /* Excision of Left Upper Arm Tendon, Open Approach */
"0LB43ZX"  =  "1"  /* Excision of Left Upper Arm Tendon, Perc Approach, Diagn */
"0LB43ZZ"  =  "1"  /* Excision of Left Upper Arm Tendon, Percutaneous Approach */
"0LB44ZX"  =  "1"  /* Excision of Left Upper Arm Tendon, Perc Endo Approach, Diagn */
"0LB44ZZ"  =  "1"  /* Excision of Left Upper Arm Tendon, Perc Endo Approach */
"0LB50ZX"  =  "1"  /* Excision of R Low Arm & Wrist Tendon, Open Approach, Diagn */
"0LB50ZZ"  =  "1"  /* Excision of Right Lower Arm and Wrist Tendon, Open Approach */
"0LB53ZX"  =  "1"  /* Excision of R Low Arm & Wrist Tendon, Perc Approach, Diagn */
"0LB53ZZ"  =  "1"  /* Excision of Right Lower Arm and Wrist Tendon, Perc Approach */
"0LB54ZX"  =  "1"  /* Excise R Low Arm & Wrist Tendon, Perc Endo, Diagn */
"0LB54ZZ"  =  "1"  /* Excision of R Low Arm & Wrist Tendon, Perc Endo Approach */
"0LB60ZX"  =  "1"  /* Excision of L Low Arm & Wrist Tendon, Open Approach, Diagn */
"0LB60ZZ"  =  "1"  /* Excision of Left Lower Arm and Wrist Tendon, Open Approach */
"0LB63ZX"  =  "1"  /* Excision of L Low Arm & Wrist Tendon, Perc Approach, Diagn */
"0LB63ZZ"  =  "1"  /* Excision of Left Lower Arm and Wrist Tendon, Perc Approach */
"0LB64ZX"  =  "1"  /* Excise L Low Arm & Wrist Tendon, Perc Endo, Diagn */
"0LB64ZZ"  =  "1"  /* Excision of L Low Arm & Wrist Tendon, Perc Endo Approach */
"0LB70ZX"  =  "1"  /* Excision of Right Hand Tendon, Open Approach, Diagnostic */
"0LB70ZZ"  =  "1"  /* Excision of Right Hand Tendon, Open Approach */
"0LB73ZX"  =  "1"  /* Excision of Right Hand Tendon, Percutaneous Approach, Diagn */
"0LB73ZZ"  =  "1"  /* Excision of Right Hand Tendon, Percutaneous Approach */
"0LB74ZX"  =  "1"  /* Excision of Right Hand Tendon, Perc Endo Approach, Diagn */
"0LB74ZZ"  =  "1"  /* Excision of Right Hand Tendon, Perc Endo Approach */
"0LB80ZX"  =  "1"  /* Excision of Left Hand Tendon, Open Approach, Diagnostic */
"0LB80ZZ"  =  "1"  /* Excision of Left Hand Tendon, Open Approach */
"0LB83ZX"  =  "1"  /* Excision of Left Hand Tendon, Percutaneous Approach, Diagn */
"0LB83ZZ"  =  "1"  /* Excision of Left Hand Tendon, Percutaneous Approach */
"0LB84ZX"  =  "1"  /* Excision of Left Hand Tendon, Perc Endo Approach, Diagn */
"0LB84ZZ"  =  "1"  /* Excision of Left Hand Tendon, Perc Endo Approach */
"0LB90ZX"  =  "1"  /* Excision of Right Trunk Tendon, Open Approach, Diagnostic */
"0LB90ZZ"  =  "1"  /* Excision of Right Trunk Tendon, Open Approach */
"0LB93ZX"  =  "1"  /* Excision of Right Trunk Tendon, Percutaneous Approach, Diagn */
"0LB93ZZ"  =  "1"  /* Excision of Right Trunk Tendon, Percutaneous Approach */
"0LB94ZX"  =  "1"  /* Excision of Right Trunk Tendon, Perc Endo Approach, Diagn */
"0LB94ZZ"  =  "1"  /* Excision of Right Trunk Tendon, Perc Endo Approach */
"0LBB0ZX"  =  "1"  /* Excision of Left Trunk Tendon, Open Approach, Diagnostic */
"0LBB0ZZ"  =  "1"  /* Excision of Left Trunk Tendon, Open Approach */
"0LBB3ZX"  =  "1"  /* Excision of Left Trunk Tendon, Percutaneous Approach, Diagn */
"0LBB3ZZ"  =  "1"  /* Excision of Left Trunk Tendon, Percutaneous Approach */
"0LBB4ZX"  =  "1"  /* Excision of Left Trunk Tendon, Perc Endo Approach, Diagn */
"0LBB4ZZ"  =  "1"  /* Excision of Left Trunk Tendon, Perc Endo Approach */
"0LBC0ZX"  =  "1"  /* Excision of Right Thorax Tendon, Open Approach, Diagnostic */
"0LBC0ZZ"  =  "1"  /* Excision of Right Thorax Tendon, Open Approach */
"0LBC3ZX"  =  "1"  /* Excision of Right Thorax Tendon, Perc Approach, Diagn */
"0LBC3ZZ"  =  "1"  /* Excision of Right Thorax Tendon, Percutaneous Approach */
"0LBC4ZX"  =  "1"  /* Excision of Right Thorax Tendon, Perc Endo Approach, Diagn */
"0LBC4ZZ"  =  "1"  /* Excision of Right Thorax Tendon, Perc Endo Approach */
"0LBD0ZX"  =  "1"  /* Excision of Left Thorax Tendon, Open Approach, Diagnostic */
"0LBD0ZZ"  =  "1"  /* Excision of Left Thorax Tendon, Open Approach */
"0LBD3ZX"  =  "1"  /* Excision of Left Thorax Tendon, Percutaneous Approach, Diagn */
"0LBD3ZZ"  =  "1"  /* Excision of Left Thorax Tendon, Percutaneous Approach */
"0LBD4ZX"  =  "1"  /* Excision of Left Thorax Tendon, Perc Endo Approach, Diagn */
"0LBD4ZZ"  =  "1"  /* Excision of Left Thorax Tendon, Perc Endo Approach */
"0LBF0ZX"  =  "1"  /* Excision of Right Abdomen Tendon, Open Approach, Diagnostic */
"0LBF0ZZ"  =  "1"  /* Excision of Right Abdomen Tendon, Open Approach */
"0LBF3ZX"  =  "1"  /* Excision of Right Abdomen Tendon, Perc Approach, Diagn */
"0LBF3ZZ"  =  "1"  /* Excision of Right Abdomen Tendon, Percutaneous Approach */
"0LBF4ZX"  =  "1"  /* Excision of Right Abdomen Tendon, Perc Endo Approach, Diagn */
"0LBF4ZZ"  =  "1"  /* Excision of Right Abdomen Tendon, Perc Endo Approach */
"0LBG0ZX"  =  "1"  /* Excision of Left Abdomen Tendon, Open Approach, Diagnostic */
"0LBG0ZZ"  =  "1"  /* Excision of Left Abdomen Tendon, Open Approach */
"0LBG3ZX"  =  "1"  /* Excision of Left Abdomen Tendon, Perc Approach, Diagn */
"0LBG3ZZ"  =  "1"  /* Excision of Left Abdomen Tendon, Percutaneous Approach */
"0LBG4ZX"  =  "1"  /* Excision of Left Abdomen Tendon, Perc Endo Approach, Diagn */
"0LBG4ZZ"  =  "1"  /* Excision of Left Abdomen Tendon, Perc Endo Approach */
"0LBH0ZX"  =  "1"  /* Excision of Perineum Tendon, Open Approach, Diagnostic */
"0LBH0ZZ"  =  "1"  /* Excision of Perineum Tendon, Open Approach */
"0LBH3ZX"  =  "1"  /* Excision of Perineum Tendon, Percutaneous Approach, Diagn */
"0LBH3ZZ"  =  "1"  /* Excision of Perineum Tendon, Percutaneous Approach */
"0LBH4ZX"  =  "1"  /* Excision of Perineum Tendon, Perc Endo Approach, Diagn */
"0LBH4ZZ"  =  "1"  /* Excision of Perineum Tendon, Perc Endo Approach */
"0LBJ0ZX"  =  "1"  /* Excision of Right Hip Tendon, Open Approach, Diagnostic */
"0LBJ0ZZ"  =  "1"  /* Excision of Right Hip Tendon, Open Approach */
"0LBJ3ZX"  =  "1"  /* Excision of Right Hip Tendon, Percutaneous Approach, Diagn */
"0LBJ3ZZ"  =  "1"  /* Excision of Right Hip Tendon, Percutaneous Approach */
"0LBJ4ZX"  =  "1"  /* Excision of Right Hip Tendon, Perc Endo Approach, Diagn */
"0LBJ4ZZ"  =  "1"  /* Excision of Right Hip Tendon, Perc Endo Approach */
"0LBK0ZX"  =  "1"  /* Excision of Left Hip Tendon, Open Approach, Diagnostic */
"0LBK0ZZ"  =  "1"  /* Excision of Left Hip Tendon, Open Approach */
"0LBK3ZX"  =  "1"  /* Excision of Left Hip Tendon, Percutaneous Approach, Diagn */
"0LBK3ZZ"  =  "1"  /* Excision of Left Hip Tendon, Percutaneous Approach */
"0LBK4ZX"  =  "1"  /* Excision of Left Hip Tendon, Perc Endo Approach, Diagn */
"0LBK4ZZ"  =  "1"  /* Excision of Left Hip Tendon, Perc Endo Approach */
"0LBL0ZX"  =  "1"  /* Excision of Right Upper Leg Tendon, Open Approach, Diagn */
"0LBL0ZZ"  =  "1"  /* Excision of Right Upper Leg Tendon, Open Approach */
"0LBL3ZX"  =  "1"  /* Excision of Right Upper Leg Tendon, Perc Approach, Diagn */
"0LBL3ZZ"  =  "1"  /* Excision of Right Upper Leg Tendon, Percutaneous Approach */
"0LBL4ZX"  =  "1"  /* Excision of R Up Leg Tendon, Perc Endo Approach, Diagn */
"0LBL4ZZ"  =  "1"  /* Excision of Right Upper Leg Tendon, Perc Endo Approach */
"0LBM0ZX"  =  "1"  /* Excision of Left Upper Leg Tendon, Open Approach, Diagnostic */
"0LBM0ZZ"  =  "1"  /* Excision of Left Upper Leg Tendon, Open Approach */
"0LBM3ZX"  =  "1"  /* Excision of Left Upper Leg Tendon, Perc Approach, Diagn */
"0LBM3ZZ"  =  "1"  /* Excision of Left Upper Leg Tendon, Percutaneous Approach */
"0LBM4ZX"  =  "1"  /* Excision of Left Upper Leg Tendon, Perc Endo Approach, Diagn */
"0LBM4ZZ"  =  "1"  /* Excision of Left Upper Leg Tendon, Perc Endo Approach */
"0LBN0ZX"  =  "1"  /* Excision of Right Lower Leg Tendon, Open Approach, Diagn */
"0LBN0ZZ"  =  "1"  /* Excision of Right Lower Leg Tendon, Open Approach */
"0LBN3ZX"  =  "1"  /* Excision of Right Lower Leg Tendon, Perc Approach, Diagn */
"0LBN3ZZ"  =  "1"  /* Excision of Right Lower Leg Tendon, Percutaneous Approach */
"0LBN4ZX"  =  "1"  /* Excision of R Low Leg Tendon, Perc Endo Approach, Diagn */
"0LBN4ZZ"  =  "1"  /* Excision of Right Lower Leg Tendon, Perc Endo Approach */
"0LBP0ZX"  =  "1"  /* Excision of Left Lower Leg Tendon, Open Approach, Diagnostic */
"0LBP0ZZ"  =  "1"  /* Excision of Left Lower Leg Tendon, Open Approach */
"0LBP3ZX"  =  "1"  /* Excision of Left Lower Leg Tendon, Perc Approach, Diagn */
"0LBP3ZZ"  =  "1"  /* Excision of Left Lower Leg Tendon, Percutaneous Approach */
"0LBP4ZX"  =  "1"  /* Excision of Left Lower Leg Tendon, Perc Endo Approach, Diagn */
"0LBP4ZZ"  =  "1"  /* Excision of Left Lower Leg Tendon, Perc Endo Approach */
"0LBQ0ZX"  =  "1"  /* Excision of Right Knee Tendon, Open Approach, Diagnostic */
"0LBQ0ZZ"  =  "1"  /* Excision of Right Knee Tendon, Open Approach */
"0LBQ3ZX"  =  "1"  /* Excision of Right Knee Tendon, Percutaneous Approach, Diagn */
"0LBQ3ZZ"  =  "1"  /* Excision of Right Knee Tendon, Percutaneous Approach */
"0LBQ4ZX"  =  "1"  /* Excision of Right Knee Tendon, Perc Endo Approach, Diagn */
"0LBQ4ZZ"  =  "1"  /* Excision of Right Knee Tendon, Perc Endo Approach */
"0LBR0ZX"  =  "1"  /* Excision of Left Knee Tendon, Open Approach, Diagnostic */
"0LBR0ZZ"  =  "1"  /* Excision of Left Knee Tendon, Open Approach */
"0LBR3ZX"  =  "1"  /* Excision of Left Knee Tendon, Percutaneous Approach, Diagn */
"0LBR3ZZ"  =  "1"  /* Excision of Left Knee Tendon, Percutaneous Approach */
"0LBR4ZX"  =  "1"  /* Excision of Left Knee Tendon, Perc Endo Approach, Diagn */
"0LBR4ZZ"  =  "1"  /* Excision of Left Knee Tendon, Perc Endo Approach */
"0LBS0ZX"  =  "1"  /* Excision of Right Ankle Tendon, Open Approach, Diagnostic */
"0LBS0ZZ"  =  "1"  /* Excision of Right Ankle Tendon, Open Approach */
"0LBS3ZX"  =  "1"  /* Excision of Right Ankle Tendon, Percutaneous Approach, Diagn */
"0LBS3ZZ"  =  "1"  /* Excision of Right Ankle Tendon, Percutaneous Approach */
"0LBS4ZX"  =  "1"  /* Excision of Right Ankle Tendon, Perc Endo Approach, Diagn */
"0LBS4ZZ"  =  "1"  /* Excision of Right Ankle Tendon, Perc Endo Approach */
"0LBT0ZX"  =  "1"  /* Excision of Left Ankle Tendon, Open Approach, Diagnostic */
"0LBT0ZZ"  =  "1"  /* Excision of Left Ankle Tendon, Open Approach */
"0LBT3ZX"  =  "1"  /* Excision of Left Ankle Tendon, Percutaneous Approach, Diagn */
"0LBT3ZZ"  =  "1"  /* Excision of Left Ankle Tendon, Percutaneous Approach */
"0LBT4ZX"  =  "1"  /* Excision of Left Ankle Tendon, Perc Endo Approach, Diagn */
"0LBT4ZZ"  =  "1"  /* Excision of Left Ankle Tendon, Perc Endo Approach */
"0LBV0ZX"  =  "1"  /* Excision of Right Foot Tendon, Open Approach, Diagnostic */
"0LBV0ZZ"  =  "1"  /* Excision of Right Foot Tendon, Open Approach */
"0LBV3ZX"  =  "1"  /* Excision of Right Foot Tendon, Percutaneous Approach, Diagn */
"0LBV3ZZ"  =  "1"  /* Excision of Right Foot Tendon, Percutaneous Approach */
"0LBV4ZX"  =  "1"  /* Excision of Right Foot Tendon, Perc Endo Approach, Diagn */
"0LBV4ZZ"  =  "1"  /* Excision of Right Foot Tendon, Perc Endo Approach */
"0LBW0ZX"  =  "1"  /* Excision of Left Foot Tendon, Open Approach, Diagnostic */
"0LBW0ZZ"  =  "1"  /* Excision of Left Foot Tendon, Open Approach */
"0LBW3ZX"  =  "1"  /* Excision of Left Foot Tendon, Percutaneous Approach, Diagn */
"0LBW3ZZ"  =  "1"  /* Excision of Left Foot Tendon, Percutaneous Approach */
"0LBW4ZX"  =  "1"  /* Excision of Left Foot Tendon, Perc Endo Approach, Diagn */
"0LBW4ZZ"  =  "1"  /* Excision of Left Foot Tendon, Perc Endo Approach */
"0LC00ZZ"  =  "1"  /* Extirpation of Matter from Head & Neck Tendon, Open Approach */
"0LC03ZZ"  =  "1"  /* Extirpation of Matter from Head & Neck Tendon, Perc Approach */
"0LC04ZZ"  =  "1"  /* Extirpate matter from Head & Neck Tendon, Perc Endo */
"0LC10ZZ"  =  "1"  /* Extirpation of Matter from R Shoulder Tendon, Open Approach */
"0LC13ZZ"  =  "1"  /* Extirpation of Matter from R Shoulder Tendon, Perc Approach */
"0LC14ZZ"  =  "1"  /* Extirpate matter from R Shoulder Tendon, Perc Endo */
"0LC20ZZ"  =  "1"  /* Extirpation of Matter from L Shoulder Tendon, Open Approach */
"0LC23ZZ"  =  "1"  /* Extirpation of Matter from L Shoulder Tendon, Perc Approach */
"0LC24ZZ"  =  "1"  /* Extirpate matter from L Shoulder Tendon, Perc Endo */
"0LC30ZZ"  =  "1"  /* Extirpation of Matter from R Up Arm Tendon, Open Approach */
"0LC33ZZ"  =  "1"  /* Extirpation of Matter from R Up Arm Tendon, Perc Approach */
"0LC34ZZ"  =  "1"  /* Extirpate of Matter from R Up Arm Tendon, Perc Endo Approach */
"0LC40ZZ"  =  "1"  /* Extirpation of Matter from L Up Arm Tendon, Open Approach */
"0LC43ZZ"  =  "1"  /* Extirpation of Matter from L Up Arm Tendon, Perc Approach */
"0LC44ZZ"  =  "1"  /* Extirpate of Matter from L Up Arm Tendon, Perc Endo Approach */
"0LC50ZZ"  =  "1"  /* Extirpate matter from R Low Arm & Wrist Tendon, Open */
"0LC53ZZ"  =  "1"  /* Extirpate matter from R Low Arm & Wrist Tendon, Perc */
"0LC54ZZ"  =  "1"  /* Extirpate matter from R Low Arm & Wrist Tendon, Perc Endo */
"0LC60ZZ"  =  "1"  /* Extirpate matter from L Low Arm & Wrist Tendon, Open */
"0LC63ZZ"  =  "1"  /* Extirpate matter from L Low Arm & Wrist Tendon, Perc */
"0LC64ZZ"  =  "1"  /* Extirpate matter from L Low Arm & Wrist Tendon, Perc Endo */
"0LC70ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Tendon, Open Approach */
"0LC73ZZ"  =  "1"  /* Extirpation of Matter from Right Hand Tendon, Perc Approach */
"0LC74ZZ"  =  "1"  /* Extirpation of Matter from R Hand Tendon, Perc Endo Approach */
"0LC80ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Tendon, Open Approach */
"0LC83ZZ"  =  "1"  /* Extirpation of Matter from Left Hand Tendon, Perc Approach */
"0LC84ZZ"  =  "1"  /* Extirpation of Matter from L Hand Tendon, Perc Endo Approach */
"0LC90ZZ"  =  "1"  /* Extirpation of Matter from Right Trunk Tendon, Open Approach */
"0LC93ZZ"  =  "1"  /* Extirpation of Matter from Right Trunk Tendon, Perc Approach */
"0LC94ZZ"  =  "1"  /* Extirpate of Matter from R Trunk Tendon, Perc Endo Approach */
"0LCB0ZZ"  =  "1"  /* Extirpation of Matter from Left Trunk Tendon, Open Approach */
"0LCB3ZZ"  =  "1"  /* Extirpation of Matter from Left Trunk Tendon, Perc Approach */
"0LCB4ZZ"  =  "1"  /* Extirpate of Matter from L Trunk Tendon, Perc Endo Approach */
"0LCC0ZZ"  =  "1"  /* Extirpation of Matter from R Thorax Tendon, Open Approach */
"0LCC3ZZ"  =  "1"  /* Extirpation of Matter from R Thorax Tendon, Perc Approach */
"0LCC4ZZ"  =  "1"  /* Extirpate of Matter from R Thorax Tendon, Perc Endo Approach */
"0LCD0ZZ"  =  "1"  /* Extirpation of Matter from Left Thorax Tendon, Open Approach */
"0LCD3ZZ"  =  "1"  /* Extirpation of Matter from Left Thorax Tendon, Perc Approach */
"0LCD4ZZ"  =  "1"  /* Extirpate of Matter from L Thorax Tendon, Perc Endo Approach */
"0LCF0ZZ"  =  "1"  /* Extirpation of Matter from R Abd Tendon, Open Approach */
"0LCF3ZZ"  =  "1"  /* Extirpation of Matter from R Abd Tendon, Perc Approach */
"0LCF4ZZ"  =  "1"  /* Extirpation of Matter from R Abd Tendon, Perc Endo Approach */
"0LCG0ZZ"  =  "1"  /* Extirpation of Matter from L Abd Tendon, Open Approach */
"0LCG3ZZ"  =  "1"  /* Extirpation of Matter from L Abd Tendon, Perc Approach */
"0LCG4ZZ"  =  "1"  /* Extirpation of Matter from L Abd Tendon, Perc Endo Approach */
"0LCH0ZZ"  =  "1"  /* Extirpation of Matter from Perineum Tendon, Open Approach */
"0LCH3ZZ"  =  "1"  /* Extirpation of Matter from Perineum Tendon, Perc Approach */
"0LCH4ZZ"  =  "1"  /* Extirpate of Matter from Perineum Tendon, Perc Endo Approach */
"0LCJ0ZZ"  =  "1"  /* Extirpation of Matter from Right Hip Tendon, Open Approach */
"0LCJ3ZZ"  =  "1"  /* Extirpation of Matter from Right Hip Tendon, Perc Approach */
"0LCJ4ZZ"  =  "1"  /* Extirpation of Matter from R Hip Tendon, Perc Endo Approach */
"0LCK0ZZ"  =  "1"  /* Extirpation of Matter from Left Hip Tendon, Open Approach */
"0LCK3ZZ"  =  "1"  /* Extirpation of Matter from Left Hip Tendon, Perc Approach */
"0LCK4ZZ"  =  "1"  /* Extirpation of Matter from L Hip Tendon, Perc Endo Approach */
"0LCL0ZZ"  =  "1"  /* Extirpation of Matter from R Up Leg Tendon, Open Approach */
"0LCL3ZZ"  =  "1"  /* Extirpation of Matter from R Up Leg Tendon, Perc Approach */
"0LCL4ZZ"  =  "1"  /* Extirpate of Matter from R Up Leg Tendon, Perc Endo Approach */
"0LCM0ZZ"  =  "1"  /* Extirpation of Matter from L Up Leg Tendon, Open Approach */
"0LCM3ZZ"  =  "1"  /* Extirpation of Matter from L Up Leg Tendon, Perc Approach */
"0LCM4ZZ"  =  "1"  /* Extirpate of Matter from L Up Leg Tendon, Perc Endo Approach */
"0LCN0ZZ"  =  "1"  /* Extirpation of Matter from R Low Leg Tendon, Open Approach */
"0LCN3ZZ"  =  "1"  /* Extirpation of Matter from R Low Leg Tendon, Perc Approach */
"0LCN4ZZ"  =  "1"  /* Extirpate matter from R Low Leg Tendon, Perc Endo */
"0LCP0ZZ"  =  "1"  /* Extirpation of Matter from L Low Leg Tendon, Open Approach */
"0LCP3ZZ"  =  "1"  /* Extirpation of Matter from L Low Leg Tendon, Perc Approach */
"0LCP4ZZ"  =  "1"  /* Extirpate matter from L Low Leg Tendon, Perc Endo */
"0LCQ0ZZ"  =  "1"  /* Extirpation of Matter from Right Knee Tendon, Open Approach */
"0LCQ3ZZ"  =  "1"  /* Extirpation of Matter from Right Knee Tendon, Perc Approach */
"0LCQ4ZZ"  =  "1"  /* Extirpation of Matter from R Knee Tendon, Perc Endo Approach */
"0LCR0ZZ"  =  "1"  /* Extirpation of Matter from Left Knee Tendon, Open Approach */
"0LCR3ZZ"  =  "1"  /* Extirpation of Matter from Left Knee Tendon, Perc Approach */
"0LCR4ZZ"  =  "1"  /* Extirpation of Matter from L Knee Tendon, Perc Endo Approach */
"0LCS0ZZ"  =  "1"  /* Extirpation of Matter from Right Ankle Tendon, Open Approach */
"0LCS3ZZ"  =  "1"  /* Extirpation of Matter from Right Ankle Tendon, Perc Approach */
"0LCS4ZZ"  =  "1"  /* Extirpate of Matter from R Ankle Tendon, Perc Endo Approach */
"0LCT0ZZ"  =  "1"  /* Extirpation of Matter from Left Ankle Tendon, Open Approach */
"0LCT3ZZ"  =  "1"  /* Extirpation of Matter from Left Ankle Tendon, Perc Approach */
"0LCT4ZZ"  =  "1"  /* Extirpate of Matter from L Ankle Tendon, Perc Endo Approach */
"0LCV0ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Tendon, Open Approach */
"0LCV3ZZ"  =  "1"  /* Extirpation of Matter from Right Foot Tendon, Perc Approach */
"0LCV4ZZ"  =  "1"  /* Extirpation of Matter from R Foot Tendon, Perc Endo Approach */
"0LCW0ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Tendon, Open Approach */
"0LCW3ZZ"  =  "1"  /* Extirpation of Matter from Left Foot Tendon, Perc Approach */
"0LCW4ZZ"  =  "1"  /* Extirpation of Matter from L Foot Tendon, Perc Endo Approach */
"0LJX0ZZ"  =  "1"  /* Inspection of Upper Tendon, Open Approach */
"0LJX3ZZ"  =  "1"  /* Inspection of Upper Tendon, Percutaneous Approach */
"0LJX4ZZ"  =  "1"  /* Inspection of Upper Tendon, Percutaneous Endoscopic Approach */
"0LJY0ZZ"  =  "1"  /* Inspection of Lower Tendon, Open Approach */
"0LJY3ZZ"  =  "1"  /* Inspection of Lower Tendon, Percutaneous Approach */
"0LJY4ZZ"  =  "1"  /* Inspection of Lower Tendon, Percutaneous Endoscopic Approach */
"0LM00ZZ"  =  "1"  /* Reattachment of Head and Neck Tendon, Open Approach */
"0LM04ZZ"  =  "1"  /* Reattachment of Head and Neck Tendon, Perc Endo Approach */
"0LM10ZZ"  =  "1"  /* Reattachment of Right Shoulder Tendon, Open Approach */
"0LM14ZZ"  =  "1"  /* Reattachment of Right Shoulder Tendon, Perc Endo Approach */
"0LM20ZZ"  =  "1"  /* Reattachment of Left Shoulder Tendon, Open Approach */
"0LM24ZZ"  =  "1"  /* Reattachment of Left Shoulder Tendon, Perc Endo Approach */
"0LM30ZZ"  =  "1"  /* Reattachment of Right Upper Arm Tendon, Open Approach */
"0LM34ZZ"  =  "1"  /* Reattachment of Right Upper Arm Tendon, Perc Endo Approach */
"0LM40ZZ"  =  "1"  /* Reattachment of Left Upper Arm Tendon, Open Approach */
"0LM44ZZ"  =  "1"  /* Reattachment of Left Upper Arm Tendon, Perc Endo Approach */
"0LM50ZZ"  =  "1"  /* Reattachment of R Low Arm & Wrist Tendon, Open Approach */
"0LM54ZZ"  =  "1"  /* Reattachment of R Low Arm & Wrist Tendon, Perc Endo Approach */
"0LM60ZZ"  =  "1"  /* Reattachment of L Low Arm & Wrist Tendon, Open Approach */
"0LM64ZZ"  =  "1"  /* Reattachment of L Low Arm & Wrist Tendon, Perc Endo Approach */
"0LM70ZZ"  =  "1"  /* Reattachment of Right Hand Tendon, Open Approach */
"0LM74ZZ"  =  "1"  /* Reattachment of Right Hand Tendon, Perc Endo Approach */
"0LM80ZZ"  =  "1"  /* Reattachment of Left Hand Tendon, Open Approach */
"0LM84ZZ"  =  "1"  /* Reattachment of Left Hand Tendon, Perc Endo Approach */
"0LM90ZZ"  =  "1"  /* Reattachment of Right Trunk Tendon, Open Approach */
"0LM94ZZ"  =  "1"  /* Reattachment of Right Trunk Tendon, Perc Endo Approach */
"0LMB0ZZ"  =  "1"  /* Reattachment of Left Trunk Tendon, Open Approach */
"0LMB4ZZ"  =  "1"  /* Reattachment of Left Trunk Tendon, Perc Endo Approach */
"0LMC0ZZ"  =  "1"  /* Reattachment of Right Thorax Tendon, Open Approach */
"0LMC4ZZ"  =  "1"  /* Reattachment of Right Thorax Tendon, Perc Endo Approach */
"0LMD0ZZ"  =  "1"  /* Reattachment of Left Thorax Tendon, Open Approach */
"0LMD4ZZ"  =  "1"  /* Reattachment of Left Thorax Tendon, Perc Endo Approach */
"0LMF0ZZ"  =  "1"  /* Reattachment of Right Abdomen Tendon, Open Approach */
"0LMF4ZZ"  =  "1"  /* Reattachment of Right Abdomen Tendon, Perc Endo Approach */
"0LMG0ZZ"  =  "1"  /* Reattachment of Left Abdomen Tendon, Open Approach */
"0LMG4ZZ"  =  "1"  /* Reattachment of Left Abdomen Tendon, Perc Endo Approach */
"0LMH0ZZ"  =  "1"  /* Reattachment of Perineum Tendon, Open Approach */
"0LMH4ZZ"  =  "1"  /* Reattachment of Perineum Tendon, Perc Endo Approach */
"0LMJ0ZZ"  =  "1"  /* Reattachment of Right Hip Tendon, Open Approach */
"0LMJ4ZZ"  =  "1"  /* Reattachment of Right Hip Tendon, Perc Endo Approach */
"0LMK0ZZ"  =  "1"  /* Reattachment of Left Hip Tendon, Open Approach */
"0LMK4ZZ"  =  "1"  /* Reattachment of Left Hip Tendon, Perc Endo Approach */
"0LML0ZZ"  =  "1"  /* Reattachment of Right Upper Leg Tendon, Open Approach */
"0LML4ZZ"  =  "1"  /* Reattachment of Right Upper Leg Tendon, Perc Endo Approach */
"0LMM0ZZ"  =  "1"  /* Reattachment of Left Upper Leg Tendon, Open Approach */
"0LMM4ZZ"  =  "1"  /* Reattachment of Left Upper Leg Tendon, Perc Endo Approach */
"0LMN0ZZ"  =  "1"  /* Reattachment of Right Lower Leg Tendon, Open Approach */
"0LMN4ZZ"  =  "1"  /* Reattachment of Right Lower Leg Tendon, Perc Endo Approach */
"0LMP0ZZ"  =  "1"  /* Reattachment of Left Lower Leg Tendon, Open Approach */
"0LMP4ZZ"  =  "1"  /* Reattachment of Left Lower Leg Tendon, Perc Endo Approach */
"0LMQ0ZZ"  =  "1"  /* Reattachment of Right Knee Tendon, Open Approach */
"0LMQ4ZZ"  =  "1"  /* Reattachment of Right Knee Tendon, Perc Endo Approach */
"0LMR0ZZ"  =  "1"  /* Reattachment of Left Knee Tendon, Open Approach */
"0LMR4ZZ"  =  "1"  /* Reattachment of Left Knee Tendon, Perc Endo Approach */
"0LMS0ZZ"  =  "1"  /* Reattachment of Right Ankle Tendon, Open Approach */
"0LMS4ZZ"  =  "1"  /* Reattachment of Right Ankle Tendon, Perc Endo Approach */
"0LMT0ZZ"  =  "1"  /* Reattachment of Left Ankle Tendon, Open Approach */
"0LMT4ZZ"  =  "1"  /* Reattachment of Left Ankle Tendon, Perc Endo Approach */
"0LMV0ZZ"  =  "1"  /* Reattachment of Right Foot Tendon, Open Approach */
"0LMV4ZZ"  =  "1"  /* Reattachment of Right Foot Tendon, Perc Endo Approach */
"0LMW0ZZ"  =  "1"  /* Reattachment of Left Foot Tendon, Open Approach */
"0LMW4ZZ"  =  "1"  /* Reattachment of Left Foot Tendon, Perc Endo Approach */
"0LN00ZZ"  =  "1"  /* Release Head and Neck Tendon, Open Approach */
"0LN03ZZ"  =  "1"  /* Release Head and Neck Tendon, Percutaneous Approach */
"0LN04ZZ"  =  "1"  /* Release Head and Neck Tendon, Perc Endo Approach */
"0LN10ZZ"  =  "1"  /* Release Right Shoulder Tendon, Open Approach */
"0LN13ZZ"  =  "1"  /* Release Right Shoulder Tendon, Percutaneous Approach */
"0LN14ZZ"  =  "1"  /* Release Right Shoulder Tendon, Perc Endo Approach */
"0LN20ZZ"  =  "1"  /* Release Left Shoulder Tendon, Open Approach */
"0LN23ZZ"  =  "1"  /* Release Left Shoulder Tendon, Percutaneous Approach */
"0LN24ZZ"  =  "1"  /* Release Left Shoulder Tendon, Perc Endo Approach */
"0LN30ZZ"  =  "1"  /* Release Right Upper Arm Tendon, Open Approach */
"0LN33ZZ"  =  "1"  /* Release Right Upper Arm Tendon, Percutaneous Approach */
"0LN34ZZ"  =  "1"  /* Release Right Upper Arm Tendon, Perc Endo Approach */
"0LN40ZZ"  =  "1"  /* Release Left Upper Arm Tendon, Open Approach */
"0LN43ZZ"  =  "1"  /* Release Left Upper Arm Tendon, Percutaneous Approach */
"0LN44ZZ"  =  "1"  /* Release Left Upper Arm Tendon, Perc Endo Approach */
"0LN50ZZ"  =  "1"  /* Release Right Lower Arm and Wrist Tendon, Open Approach */
"0LN53ZZ"  =  "1"  /* Release Right Lower Arm and Wrist Tendon, Perc Approach */
"0LN54ZZ"  =  "1"  /* Release Right Lower Arm and Wrist Tendon, Perc Endo Approach */
"0LN60ZZ"  =  "1"  /* Release Left Lower Arm and Wrist Tendon, Open Approach */
"0LN63ZZ"  =  "1"  /* Release Left Lower Arm and Wrist Tendon, Perc Approach */
"0LN64ZZ"  =  "1"  /* Release Left Lower Arm and Wrist Tendon, Perc Endo Approach */
"0LN70ZZ"  =  "1"  /* Release Right Hand Tendon, Open Approach */
"0LN73ZZ"  =  "1"  /* Release Right Hand Tendon, Percutaneous Approach */
"0LN74ZZ"  =  "1"  /* Release Right Hand Tendon, Percutaneous Endoscopic Approach */
"0LN80ZZ"  =  "1"  /* Release Left Hand Tendon, Open Approach */
"0LN83ZZ"  =  "1"  /* Release Left Hand Tendon, Percutaneous Approach */
"0LN84ZZ"  =  "1"  /* Release Left Hand Tendon, Percutaneous Endoscopic Approach */
"0LN90ZZ"  =  "1"  /* Release Right Trunk Tendon, Open Approach */
"0LN93ZZ"  =  "1"  /* Release Right Trunk Tendon, Percutaneous Approach */
"0LN94ZZ"  =  "1"  /* Release Right Trunk Tendon, Percutaneous Endoscopic Approach */
"0LNB0ZZ"  =  "1"  /* Release Left Trunk Tendon, Open Approach */
"0LNB3ZZ"  =  "1"  /* Release Left Trunk Tendon, Percutaneous Approach */
"0LNB4ZZ"  =  "1"  /* Release Left Trunk Tendon, Percutaneous Endoscopic Approach */
"0LNC0ZZ"  =  "1"  /* Release Right Thorax Tendon, Open Approach */
"0LNC3ZZ"  =  "1"  /* Release Right Thorax Tendon, Percutaneous Approach */
"0LNC4ZZ"  =  "1"  /* Release Right Thorax Tendon, Perc Endo Approach */
"0LND0ZZ"  =  "1"  /* Release Left Thorax Tendon, Open Approach */
"0LND3ZZ"  =  "1"  /* Release Left Thorax Tendon, Percutaneous Approach */
"0LND4ZZ"  =  "1"  /* Release Left Thorax Tendon, Percutaneous Endoscopic Approach */
"0LNF0ZZ"  =  "1"  /* Release Right Abdomen Tendon, Open Approach */
"0LNF3ZZ"  =  "1"  /* Release Right Abdomen Tendon, Percutaneous Approach */
"0LNF4ZZ"  =  "1"  /* Release Right Abdomen Tendon, Perc Endo Approach */
"0LNG0ZZ"  =  "1"  /* Release Left Abdomen Tendon, Open Approach */
"0LNG3ZZ"  =  "1"  /* Release Left Abdomen Tendon, Percutaneous Approach */
"0LNG4ZZ"  =  "1"  /* Release Left Abdomen Tendon, Perc Endo Approach */
"0LNH0ZZ"  =  "1"  /* Release Perineum Tendon, Open Approach */
"0LNH3ZZ"  =  "1"  /* Release Perineum Tendon, Percutaneous Approach */
"0LNH4ZZ"  =  "1"  /* Release Perineum Tendon, Percutaneous Endoscopic Approach */
"0LNJ0ZZ"  =  "1"  /* Release Right Hip Tendon, Open Approach */
"0LNJ3ZZ"  =  "1"  /* Release Right Hip Tendon, Percutaneous Approach */
"0LNJ4ZZ"  =  "1"  /* Release Right Hip Tendon, Percutaneous Endoscopic Approach */
"0LNK0ZZ"  =  "1"  /* Release Left Hip Tendon, Open Approach */
"0LNK3ZZ"  =  "1"  /* Release Left Hip Tendon, Percutaneous Approach */
"0LNK4ZZ"  =  "1"  /* Release Left Hip Tendon, Percutaneous Endoscopic Approach */
"0LNL0ZZ"  =  "1"  /* Release Right Upper Leg Tendon, Open Approach */
"0LNL3ZZ"  =  "1"  /* Release Right Upper Leg Tendon, Percutaneous Approach */
"0LNL4ZZ"  =  "1"  /* Release Right Upper Leg Tendon, Perc Endo Approach */
"0LNM0ZZ"  =  "1"  /* Release Left Upper Leg Tendon, Open Approach */
"0LNM3ZZ"  =  "1"  /* Release Left Upper Leg Tendon, Percutaneous Approach */
"0LNM4ZZ"  =  "1"  /* Release Left Upper Leg Tendon, Perc Endo Approach */
"0LNN0ZZ"  =  "1"  /* Release Right Lower Leg Tendon, Open Approach */
"0LNN3ZZ"  =  "1"  /* Release Right Lower Leg Tendon, Percutaneous Approach */
"0LNN4ZZ"  =  "1"  /* Release Right Lower Leg Tendon, Perc Endo Approach */
"0LNP0ZZ"  =  "1"  /* Release Left Lower Leg Tendon, Open Approach */
"0LNP3ZZ"  =  "1"  /* Release Left Lower Leg Tendon, Percutaneous Approach */
"0LNP4ZZ"  =  "1"  /* Release Left Lower Leg Tendon, Perc Endo Approach */
"0LNQ0ZZ"  =  "1"  /* Release Right Knee Tendon, Open Approach */
"0LNQ3ZZ"  =  "1"  /* Release Right Knee Tendon, Percutaneous Approach */
"0LNQ4ZZ"  =  "1"  /* Release Right Knee Tendon, Percutaneous Endoscopic Approach */
"0LNR0ZZ"  =  "1"  /* Release Left Knee Tendon, Open Approach */
"0LNR3ZZ"  =  "1"  /* Release Left Knee Tendon, Percutaneous Approach */
"0LNR4ZZ"  =  "1"  /* Release Left Knee Tendon, Percutaneous Endoscopic Approach */
"0LNS0ZZ"  =  "1"  /* Release Right Ankle Tendon, Open Approach */
"0LNS3ZZ"  =  "1"  /* Release Right Ankle Tendon, Percutaneous Approach */
"0LNS4ZZ"  =  "1"  /* Release Right Ankle Tendon, Percutaneous Endoscopic Approach */
"0LNT0ZZ"  =  "1"  /* Release Left Ankle Tendon, Open Approach */
"0LNT3ZZ"  =  "1"  /* Release Left Ankle Tendon, Percutaneous Approach */
"0LNT4ZZ"  =  "1"  /* Release Left Ankle Tendon, Percutaneous Endoscopic Approach */
"0LNV0ZZ"  =  "1"  /* Release Right Foot Tendon, Open Approach */
"0LNV3ZZ"  =  "1"  /* Release Right Foot Tendon, Percutaneous Approach */
"0LNV4ZZ"  =  "1"  /* Release Right Foot Tendon, Percutaneous Endoscopic Approach */
"0LNW0ZZ"  =  "1"  /* Release Left Foot Tendon, Open Approach */
"0LNW3ZZ"  =  "1"  /* Release Left Foot Tendon, Percutaneous Approach */
"0LNW4ZZ"  =  "1"  /* Release Left Foot Tendon, Percutaneous Endoscopic Approach */
"0LPX00Z"  =  "1"  /* Removal of Drainage Device from Upper Tendon, Open Approach */
"0LPX07Z"  =  "1"  /* Removal of Autol Sub from Up Tendon, Open Approach */
"0LPX0JZ"  =  "1"  /* Removal of Synth Sub from Up Tendon, Open Approach */
"0LPX0KZ"  =  "1"  /* Removal of Nonaut Sub from Up Tendon, Open Approach */
"0LPX30Z"  =  "1"  /* Removal of Drainage Device from Upper Tendon, Perc Approach */
"0LPX37Z"  =  "1"  /* Removal of Autol Sub from Up Tendon, Perc Approach */
"0LPX3JZ"  =  "1"  /* Removal of Synth Sub from Up Tendon, Perc Approach */
"0LPX3KZ"  =  "1"  /* Removal of Nonaut Sub from Up Tendon, Perc Approach */
"0LPX40Z"  =  "1"  /* Removal of Drain Dev from Up Tendon, Perc Endo Approach */
"0LPX47Z"  =  "1"  /* Removal of Autol Sub from Up Tendon, Perc Endo Approach */
"0LPX4JZ"  =  "1"  /* Removal of Synth Sub from Up Tendon, Perc Endo Approach */
"0LPX4KZ"  =  "1"  /* Removal of Nonaut Sub from Up Tendon, Perc Endo Approach */
"0LPY00Z"  =  "1"  /* Removal of Drainage Device from Lower Tendon, Open Approach */
"0LPY07Z"  =  "1"  /* Removal of Autol Sub from Low Tendon, Open Approach */
"0LPY0JZ"  =  "1"  /* Removal of Synth Sub from Low Tendon, Open Approach */
"0LPY0KZ"  =  "1"  /* Removal of Nonaut Sub from Low Tendon, Open Approach */
"0LPY30Z"  =  "1"  /* Removal of Drainage Device from Lower Tendon, Perc Approach */
"0LPY37Z"  =  "1"  /* Removal of Autol Sub from Low Tendon, Perc Approach */
"0LPY3JZ"  =  "1"  /* Removal of Synth Sub from Low Tendon, Perc Approach */
"0LPY3KZ"  =  "1"  /* Removal of Nonaut Sub from Low Tendon, Perc Approach */
"0LPY40Z"  =  "1"  /* Removal of Drain Dev from Low Tendon, Perc Endo Approach */
"0LPY47Z"  =  "1"  /* Removal of Autol Sub from Low Tendon, Perc Endo Approach */
"0LPY4JZ"  =  "1"  /* Removal of Synth Sub from Low Tendon, Perc Endo Approach */
"0LPY4KZ"  =  "1"  /* Removal of Nonaut Sub from Low Tendon, Perc Endo Approach */
"0LQ00ZZ"  =  "1"  /* Repair Head and Neck Tendon, Open Approach */
"0LQ03ZZ"  =  "1"  /* Repair Head and Neck Tendon, Percutaneous Approach */
"0LQ04ZZ"  =  "1"  /* Repair Head and Neck Tendon, Perc Endo Approach */
"0LQ10ZZ"  =  "1"  /* Repair Right Shoulder Tendon, Open Approach */
"0LQ13ZZ"  =  "1"  /* Repair Right Shoulder Tendon, Percutaneous Approach */
"0LQ14ZZ"  =  "1"  /* Repair Right Shoulder Tendon, Perc Endo Approach */
"0LQ20ZZ"  =  "1"  /* Repair Left Shoulder Tendon, Open Approach */
"0LQ23ZZ"  =  "1"  /* Repair Left Shoulder Tendon, Percutaneous Approach */
"0LQ24ZZ"  =  "1"  /* Repair Left Shoulder Tendon, Perc Endo Approach */
"0LQ30ZZ"  =  "1"  /* Repair Right Upper Arm Tendon, Open Approach */
"0LQ33ZZ"  =  "1"  /* Repair Right Upper Arm Tendon, Percutaneous Approach */
"0LQ34ZZ"  =  "1"  /* Repair Right Upper Arm Tendon, Perc Endo Approach */
"0LQ40ZZ"  =  "1"  /* Repair Left Upper Arm Tendon, Open Approach */
"0LQ43ZZ"  =  "1"  /* Repair Left Upper Arm Tendon, Percutaneous Approach */
"0LQ44ZZ"  =  "1"  /* Repair Left Upper Arm Tendon, Perc Endo Approach */
"0LQ50ZZ"  =  "1"  /* Repair Right Lower Arm and Wrist Tendon, Open Approach */
"0LQ53ZZ"  =  "1"  /* Repair Right Lower Arm and Wrist Tendon, Perc Approach */
"0LQ54ZZ"  =  "1"  /* Repair Right Lower Arm and Wrist Tendon, Perc Endo Approach */
"0LQ60ZZ"  =  "1"  /* Repair Left Lower Arm and Wrist Tendon, Open Approach */
"0LQ63ZZ"  =  "1"  /* Repair Left Lower Arm and Wrist Tendon, Perc Approach */
"0LQ64ZZ"  =  "1"  /* Repair Left Lower Arm and Wrist Tendon, Perc Endo Approach */
"0LQ70ZZ"  =  "1"  /* Repair Right Hand Tendon, Open Approach */
"0LQ73ZZ"  =  "1"  /* Repair Right Hand Tendon, Percutaneous Approach */
"0LQ74ZZ"  =  "1"  /* Repair Right Hand Tendon, Percutaneous Endoscopic Approach */
"0LQ80ZZ"  =  "1"  /* Repair Left Hand Tendon, Open Approach */
"0LQ83ZZ"  =  "1"  /* Repair Left Hand Tendon, Percutaneous Approach */
"0LQ84ZZ"  =  "1"  /* Repair Left Hand Tendon, Percutaneous Endoscopic Approach */
"0LQ90ZZ"  =  "1"  /* Repair Right Trunk Tendon, Open Approach */
"0LQ93ZZ"  =  "1"  /* Repair Right Trunk Tendon, Percutaneous Approach */
"0LQ94ZZ"  =  "1"  /* Repair Right Trunk Tendon, Percutaneous Endoscopic Approach */
"0LQB0ZZ"  =  "1"  /* Repair Left Trunk Tendon, Open Approach */
"0LQB3ZZ"  =  "1"  /* Repair Left Trunk Tendon, Percutaneous Approach */
"0LQB4ZZ"  =  "1"  /* Repair Left Trunk Tendon, Percutaneous Endoscopic Approach */
"0LQC0ZZ"  =  "1"  /* Repair Right Thorax Tendon, Open Approach */
"0LQC3ZZ"  =  "1"  /* Repair Right Thorax Tendon, Percutaneous Approach */
"0LQC4ZZ"  =  "1"  /* Repair Right Thorax Tendon, Percutaneous Endoscopic Approach */
"0LQD0ZZ"  =  "1"  /* Repair Left Thorax Tendon, Open Approach */
"0LQD3ZZ"  =  "1"  /* Repair Left Thorax Tendon, Percutaneous Approach */
"0LQD4ZZ"  =  "1"  /* Repair Left Thorax Tendon, Percutaneous Endoscopic Approach */
"0LQF0ZZ"  =  "1"  /* Repair Right Abdomen Tendon, Open Approach */
"0LQF3ZZ"  =  "1"  /* Repair Right Abdomen Tendon, Percutaneous Approach */
"0LQF4ZZ"  =  "1"  /* Repair Right Abdomen Tendon, Perc Endo Approach */
"0LQG0ZZ"  =  "1"  /* Repair Left Abdomen Tendon, Open Approach */
"0LQG3ZZ"  =  "1"  /* Repair Left Abdomen Tendon, Percutaneous Approach */
"0LQG4ZZ"  =  "1"  /* Repair Left Abdomen Tendon, Percutaneous Endoscopic Approach */
"0LQH0ZZ"  =  "1"  /* Repair Perineum Tendon, Open Approach */
"0LQH3ZZ"  =  "1"  /* Repair Perineum Tendon, Percutaneous Approach */
"0LQH4ZZ"  =  "1"  /* Repair Perineum Tendon, Percutaneous Endoscopic Approach */
"0LQJ0ZZ"  =  "1"  /* Repair Right Hip Tendon, Open Approach */
"0LQJ3ZZ"  =  "1"  /* Repair Right Hip Tendon, Percutaneous Approach */
"0LQJ4ZZ"  =  "1"  /* Repair Right Hip Tendon, Percutaneous Endoscopic Approach */
"0LQK0ZZ"  =  "1"  /* Repair Left Hip Tendon, Open Approach */
"0LQK3ZZ"  =  "1"  /* Repair Left Hip Tendon, Percutaneous Approach */
"0LQK4ZZ"  =  "1"  /* Repair Left Hip Tendon, Percutaneous Endoscopic Approach */
"0LQL0ZZ"  =  "1"  /* Repair Right Upper Leg Tendon, Open Approach */
"0LQL3ZZ"  =  "1"  /* Repair Right Upper Leg Tendon, Percutaneous Approach */
"0LQL4ZZ"  =  "1"  /* Repair Right Upper Leg Tendon, Perc Endo Approach */
"0LQM0ZZ"  =  "1"  /* Repair Left Upper Leg Tendon, Open Approach */
"0LQM3ZZ"  =  "1"  /* Repair Left Upper Leg Tendon, Percutaneous Approach */
"0LQM4ZZ"  =  "1"  /* Repair Left Upper Leg Tendon, Perc Endo Approach */
"0LQN0ZZ"  =  "1"  /* Repair Right Lower Leg Tendon, Open Approach */
"0LQN3ZZ"  =  "1"  /* Repair Right Lower Leg Tendon, Percutaneous Approach */
"0LQN4ZZ"  =  "1"  /* Repair Right Lower Leg Tendon, Perc Endo Approach */
"0LQP0ZZ"  =  "1"  /* Repair Left Lower Leg Tendon, Open Approach */
"0LQP3ZZ"  =  "1"  /* Repair Left Lower Leg Tendon, Percutaneous Approach */
"0LQP4ZZ"  =  "1"  /* Repair Left Lower Leg Tendon, Perc Endo Approach */
"0LQQ0ZZ"  =  "1"  /* Repair Right Knee Tendon, Open Approach */
"0LQQ3ZZ"  =  "1"  /* Repair Right Knee Tendon, Percutaneous Approach */
"0LQQ4ZZ"  =  "1"  /* Repair Right Knee Tendon, Percutaneous Endoscopic Approach */
"0LQR0ZZ"  =  "1"  /* Repair Left Knee Tendon, Open Approach */
"0LQR3ZZ"  =  "1"  /* Repair Left Knee Tendon, Percutaneous Approach */
"0LQR4ZZ"  =  "1"  /* Repair Left Knee Tendon, Percutaneous Endoscopic Approach */
"0LQS0ZZ"  =  "1"  /* Repair Right Ankle Tendon, Open Approach */
"0LQS3ZZ"  =  "1"  /* Repair Right Ankle Tendon, Percutaneous Approach */
"0LQS4ZZ"  =  "1"  /* Repair Right Ankle Tendon, Percutaneous Endoscopic Approach */
"0LQT0ZZ"  =  "1"  /* Repair Left Ankle Tendon, Open Approach */
"0LQT3ZZ"  =  "1"  /* Repair Left Ankle Tendon, Percutaneous Approach */
"0LQT4ZZ"  =  "1"  /* Repair Left Ankle Tendon, Percutaneous Endoscopic Approach */
"0LQV0ZZ"  =  "1"  /* Repair Right Foot Tendon, Open Approach */
"0LQV3ZZ"  =  "1"  /* Repair Right Foot Tendon, Percutaneous Approach */
"0LQV4ZZ"  =  "1"  /* Repair Right Foot Tendon, Percutaneous Endoscopic Approach */
"0LQW0ZZ"  =  "1"  /* Repair Left Foot Tendon, Open Approach */
"0LQW3ZZ"  =  "1"  /* Repair Left Foot Tendon, Percutaneous Approach */
"0LQW4ZZ"  =  "1"  /* Repair Left Foot Tendon, Percutaneous Endoscopic Approach */
"0LR007Z"  =  "1"  /* Replace of Head & Neck Tendon with Autol Sub, Open Approach */
"0LR00JZ"  =  "1"  /* Replace of Head & Neck Tendon with Synth Sub, Open Approach */
"0LR00KZ"  =  "1"  /* Replace of Head & Neck Tendon with Nonaut Sub, Open Approach */
"0LR047Z"  =  "1"  /* Replace Head & Neck Tendon w Autol Sub, Perc Endo */
"0LR04JZ"  =  "1"  /* Replace Head & Neck Tendon w Synth Sub, Perc Endo */
"0LR04KZ"  =  "1"  /* Replace Head & Neck Tendon w Nonaut Sub, Perc Endo */
"0LR107Z"  =  "1"  /* Replace of R Shoulder Tendon with Autol Sub, Open Approach */
"0LR10JZ"  =  "1"  /* Replace of R Shoulder Tendon with Synth Sub, Open Approach */
"0LR10KZ"  =  "1"  /* Replace of R Shoulder Tendon with Nonaut Sub, Open Approach */
"0LR147Z"  =  "1"  /* Replace R Shoulder Tendon w Autol Sub, Perc Endo */
"0LR14JZ"  =  "1"  /* Replace R Shoulder Tendon w Synth Sub, Perc Endo */
"0LR14KZ"  =  "1"  /* Replace R Shoulder Tendon w Nonaut Sub, Perc Endo */
"0LR207Z"  =  "1"  /* Replace of L Shoulder Tendon with Autol Sub, Open Approach */
"0LR20JZ"  =  "1"  /* Replace of L Shoulder Tendon with Synth Sub, Open Approach */
"0LR20KZ"  =  "1"  /* Replace of L Shoulder Tendon with Nonaut Sub, Open Approach */
"0LR247Z"  =  "1"  /* Replace L Shoulder Tendon w Autol Sub, Perc Endo */
"0LR24JZ"  =  "1"  /* Replace L Shoulder Tendon w Synth Sub, Perc Endo */
"0LR24KZ"  =  "1"  /* Replace L Shoulder Tendon w Nonaut Sub, Perc Endo */
"0LR307Z"  =  "1"  /* Replacement of R Up Arm Tendon with Autol Sub, Open Approach */
"0LR30JZ"  =  "1"  /* Replacement of R Up Arm Tendon with Synth Sub, Open Approach */
"0LR30KZ"  =  "1"  /* Replace of R Up Arm Tendon with Nonaut Sub, Open Approach */
"0LR347Z"  =  "1"  /* Replace R Up Arm Tendon w Autol Sub, Perc Endo */
"0LR34JZ"  =  "1"  /* Replace R Up Arm Tendon w Synth Sub, Perc Endo */
"0LR34KZ"  =  "1"  /* Replace R Up Arm Tendon w Nonaut Sub, Perc Endo */
"0LR407Z"  =  "1"  /* Replacement of L Up Arm Tendon with Autol Sub, Open Approach */
"0LR40JZ"  =  "1"  /* Replacement of L Up Arm Tendon with Synth Sub, Open Approach */
"0LR40KZ"  =  "1"  /* Replace of L Up Arm Tendon with Nonaut Sub, Open Approach */
"0LR447Z"  =  "1"  /* Replace L Up Arm Tendon w Autol Sub, Perc Endo */
"0LR44JZ"  =  "1"  /* Replace L Up Arm Tendon w Synth Sub, Perc Endo */
"0LR44KZ"  =  "1"  /* Replace L Up Arm Tendon w Nonaut Sub, Perc Endo */
"0LR507Z"  =  "1"  /* Replace R Low Arm & Wrist Tendon w Autol Sub, Open */
"0LR50JZ"  =  "1"  /* Replace R Low Arm & Wrist Tendon w Synth Sub, Open */
"0LR50KZ"  =  "1"  /* Replace R Low Arm & Wrist Tendon w Nonaut Sub, Open */
"0LR547Z"  =  "1"  /* Replace R Low Arm & Wrist Tendon w Autol Sub, Perc Endo */
"0LR54JZ"  =  "1"  /* Replace R Low Arm & Wrist Tendon w Synth Sub, Perc Endo */
"0LR54KZ"  =  "1"  /* Replace R Low Arm & Wrist Tendon w Nonaut Sub, Perc Endo */
"0LR607Z"  =  "1"  /* Replace L Low Arm & Wrist Tendon w Autol Sub, Open */
"0LR60JZ"  =  "1"  /* Replace L Low Arm & Wrist Tendon w Synth Sub, Open */
"0LR60KZ"  =  "1"  /* Replace L Low Arm & Wrist Tendon w Nonaut Sub, Open */
"0LR647Z"  =  "1"  /* Replace L Low Arm & Wrist Tendon w Autol Sub, Perc Endo */
"0LR64JZ"  =  "1"  /* Replace L Low Arm & Wrist Tendon w Synth Sub, Perc Endo */
"0LR64KZ"  =  "1"  /* Replace L Low Arm & Wrist Tendon w Nonaut Sub, Perc Endo */
"0LR707Z"  =  "1"  /* Replacement of R Hand Tendon with Autol Sub, Open Approach */
"0LR70JZ"  =  "1"  /* Replacement of R Hand Tendon with Synth Sub, Open Approach */
"0LR70KZ"  =  "1"  /* Replacement of R Hand Tendon with Nonaut Sub, Open Approach */
"0LR747Z"  =  "1"  /* Replace of R Hand Tendon with Autol Sub, Perc Endo Approach */
"0LR74JZ"  =  "1"  /* Replace of R Hand Tendon with Synth Sub, Perc Endo Approach */
"0LR74KZ"  =  "1"  /* Replace of R Hand Tendon with Nonaut Sub, Perc Endo Approach */
"0LR807Z"  =  "1"  /* Replacement of L Hand Tendon with Autol Sub, Open Approach */
"0LR80JZ"  =  "1"  /* Replacement of L Hand Tendon with Synth Sub, Open Approach */
"0LR80KZ"  =  "1"  /* Replacement of L Hand Tendon with Nonaut Sub, Open Approach */
"0LR847Z"  =  "1"  /* Replace of L Hand Tendon with Autol Sub, Perc Endo Approach */
"0LR84JZ"  =  "1"  /* Replace of L Hand Tendon with Synth Sub, Perc Endo Approach */
"0LR84KZ"  =  "1"  /* Replace of L Hand Tendon with Nonaut Sub, Perc Endo Approach */
"0LR907Z"  =  "1"  /* Replacement of R Trunk Tendon with Autol Sub, Open Approach */
"0LR90JZ"  =  "1"  /* Replacement of R Trunk Tendon with Synth Sub, Open Approach */
"0LR90KZ"  =  "1"  /* Replacement of R Trunk Tendon with Nonaut Sub, Open Approach */
"0LR947Z"  =  "1"  /* Replace of R Trunk Tendon with Autol Sub, Perc Endo Approach */
"0LR94JZ"  =  "1"  /* Replace of R Trunk Tendon with Synth Sub, Perc Endo Approach */
"0LR94KZ"  =  "1"  /* Replace R Trunk Tendon w Nonaut Sub, Perc Endo */
"0LRB07Z"  =  "1"  /* Replacement of L Trunk Tendon with Autol Sub, Open Approach */
"0LRB0JZ"  =  "1"  /* Replacement of L Trunk Tendon with Synth Sub, Open Approach */
"0LRB0KZ"  =  "1"  /* Replacement of L Trunk Tendon with Nonaut Sub, Open Approach */
"0LRB47Z"  =  "1"  /* Replace of L Trunk Tendon with Autol Sub, Perc Endo Approach */
"0LRB4JZ"  =  "1"  /* Replace of L Trunk Tendon with Synth Sub, Perc Endo Approach */
"0LRB4KZ"  =  "1"  /* Replace L Trunk Tendon w Nonaut Sub, Perc Endo */
"0LRC07Z"  =  "1"  /* Replacement of R Thorax Tendon with Autol Sub, Open Approach */
"0LRC0JZ"  =  "1"  /* Replacement of R Thorax Tendon with Synth Sub, Open Approach */
"0LRC0KZ"  =  "1"  /* Replace of R Thorax Tendon with Nonaut Sub, Open Approach */
"0LRC47Z"  =  "1"  /* Replace R Thorax Tendon w Autol Sub, Perc Endo */
"0LRC4JZ"  =  "1"  /* Replace R Thorax Tendon w Synth Sub, Perc Endo */
"0LRC4KZ"  =  "1"  /* Replace R Thorax Tendon w Nonaut Sub, Perc Endo */
"0LRD07Z"  =  "1"  /* Replacement of L Thorax Tendon with Autol Sub, Open Approach */
"0LRD0JZ"  =  "1"  /* Replacement of L Thorax Tendon with Synth Sub, Open Approach */
"0LRD0KZ"  =  "1"  /* Replace of L Thorax Tendon with Nonaut Sub, Open Approach */
"0LRD47Z"  =  "1"  /* Replace L Thorax Tendon w Autol Sub, Perc Endo */
"0LRD4JZ"  =  "1"  /* Replace L Thorax Tendon w Synth Sub, Perc Endo */
"0LRD4KZ"  =  "1"  /* Replace L Thorax Tendon w Nonaut Sub, Perc Endo */
"0LRF07Z"  =  "1"  /* Replacement of R Abd Tendon with Autol Sub, Open Approach */
"0LRF0JZ"  =  "1"  /* Replacement of R Abd Tendon with Synth Sub, Open Approach */
"0LRF0KZ"  =  "1"  /* Replacement of R Abd Tendon with Nonaut Sub, Open Approach */
"0LRF47Z"  =  "1"  /* Replace of R Abd Tendon with Autol Sub, Perc Endo Approach */
"0LRF4JZ"  =  "1"  /* Replace of R Abd Tendon with Synth Sub, Perc Endo Approach */
"0LRF4KZ"  =  "1"  /* Replace of R Abd Tendon with Nonaut Sub, Perc Endo Approach */
"0LRG07Z"  =  "1"  /* Replacement of L Abd Tendon with Autol Sub, Open Approach */
"0LRG0JZ"  =  "1"  /* Replacement of L Abd Tendon with Synth Sub, Open Approach */
"0LRG0KZ"  =  "1"  /* Replacement of L Abd Tendon with Nonaut Sub, Open Approach */
"0LRG47Z"  =  "1"  /* Replace of L Abd Tendon with Autol Sub, Perc Endo Approach */
"0LRG4JZ"  =  "1"  /* Replace of L Abd Tendon with Synth Sub, Perc Endo Approach */
"0LRG4KZ"  =  "1"  /* Replace of L Abd Tendon with Nonaut Sub, Perc Endo Approach */
"0LRH07Z"  =  "1"  /* Replacement of Perineum Tendon with Autol Sub, Open Approach */
"0LRH0JZ"  =  "1"  /* Replacement of Perineum Tendon with Synth Sub, Open Approach */
"0LRH0KZ"  =  "1"  /* Replace of Perineum Tendon with Nonaut Sub, Open Approach */
"0LRH47Z"  =  "1"  /* Replace Perineum Tendon w Autol Sub, Perc Endo */
"0LRH4JZ"  =  "1"  /* Replace Perineum Tendon w Synth Sub, Perc Endo */
"0LRH4KZ"  =  "1"  /* Replace Perineum Tendon w Nonaut Sub, Perc Endo */
"0LRJ07Z"  =  "1"  /* Replacement of R Hip Tendon with Autol Sub, Open Approach */
"0LRJ0JZ"  =  "1"  /* Replacement of R Hip Tendon with Synth Sub, Open Approach */
"0LRJ0KZ"  =  "1"  /* Replacement of R Hip Tendon with Nonaut Sub, Open Approach */
"0LRJ47Z"  =  "1"  /* Replace of R Hip Tendon with Autol Sub, Perc Endo Approach */
"0LRJ4JZ"  =  "1"  /* Replace of R Hip Tendon with Synth Sub, Perc Endo Approach */
"0LRJ4KZ"  =  "1"  /* Replace of R Hip Tendon with Nonaut Sub, Perc Endo Approach */
"0LRK07Z"  =  "1"  /* Replacement of Left Hip Tendon with Autol Sub, Open Approach */
"0LRK0JZ"  =  "1"  /* Replacement of Left Hip Tendon with Synth Sub, Open Approach */
"0LRK0KZ"  =  "1"  /* Replacement of L Hip Tendon with Nonaut Sub, Open Approach */
"0LRK47Z"  =  "1"  /* Replace of L Hip Tendon with Autol Sub, Perc Endo Approach */
"0LRK4JZ"  =  "1"  /* Replace of L Hip Tendon with Synth Sub, Perc Endo Approach */
"0LRK4KZ"  =  "1"  /* Replace of L Hip Tendon with Nonaut Sub, Perc Endo Approach */
"0LRL07Z"  =  "1"  /* Replacement of R Up Leg Tendon with Autol Sub, Open Approach */
"0LRL0JZ"  =  "1"  /* Replacement of R Up Leg Tendon with Synth Sub, Open Approach */
"0LRL0KZ"  =  "1"  /* Replace of R Up Leg Tendon with Nonaut Sub, Open Approach */
"0LRL47Z"  =  "1"  /* Replace R Up Leg Tendon w Autol Sub, Perc Endo */
"0LRL4JZ"  =  "1"  /* Replace R Up Leg Tendon w Synth Sub, Perc Endo */
"0LRL4KZ"  =  "1"  /* Replace R Up Leg Tendon w Nonaut Sub, Perc Endo */
"0LRM07Z"  =  "1"  /* Replacement of L Up Leg Tendon with Autol Sub, Open Approach */
"0LRM0JZ"  =  "1"  /* Replacement of L Up Leg Tendon with Synth Sub, Open Approach */
"0LRM0KZ"  =  "1"  /* Replace of L Up Leg Tendon with Nonaut Sub, Open Approach */
"0LRM47Z"  =  "1"  /* Replace L Up Leg Tendon w Autol Sub, Perc Endo */
"0LRM4JZ"  =  "1"  /* Replace L Up Leg Tendon w Synth Sub, Perc Endo */
"0LRM4KZ"  =  "1"  /* Replace L Up Leg Tendon w Nonaut Sub, Perc Endo */
"0LRN07Z"  =  "1"  /* Replace of R Low Leg Tendon with Autol Sub, Open Approach */
"0LRN0JZ"  =  "1"  /* Replace of R Low Leg Tendon with Synth Sub, Open Approach */
"0LRN0KZ"  =  "1"  /* Replace of R Low Leg Tendon with Nonaut Sub, Open Approach */
"0LRN47Z"  =  "1"  /* Replace R Low Leg Tendon w Autol Sub, Perc Endo */
"0LRN4JZ"  =  "1"  /* Replace R Low Leg Tendon w Synth Sub, Perc Endo */
"0LRN4KZ"  =  "1"  /* Replace R Low Leg Tendon w Nonaut Sub, Perc Endo */
"0LRP07Z"  =  "1"  /* Replace of L Low Leg Tendon with Autol Sub, Open Approach */
"0LRP0JZ"  =  "1"  /* Replace of L Low Leg Tendon with Synth Sub, Open Approach */
"0LRP0KZ"  =  "1"  /* Replace of L Low Leg Tendon with Nonaut Sub, Open Approach */
"0LRP47Z"  =  "1"  /* Replace L Low Leg Tendon w Autol Sub, Perc Endo */
"0LRP4JZ"  =  "1"  /* Replace L Low Leg Tendon w Synth Sub, Perc Endo */
"0LRP4KZ"  =  "1"  /* Replace L Low Leg Tendon w Nonaut Sub, Perc Endo */
"0LRQ07Z"  =  "1"  /* Replacement of R Knee Tendon with Autol Sub, Open Approach */
"0LRQ0JZ"  =  "1"  /* Replacement of R Knee Tendon with Synth Sub, Open Approach */
"0LRQ0KZ"  =  "1"  /* Replacement of R Knee Tendon with Nonaut Sub, Open Approach */
"0LRQ47Z"  =  "1"  /* Replace of R Knee Tendon with Autol Sub, Perc Endo Approach */
"0LRQ4JZ"  =  "1"  /* Replace of R Knee Tendon with Synth Sub, Perc Endo Approach */
"0LRQ4KZ"  =  "1"  /* Replace of R Knee Tendon with Nonaut Sub, Perc Endo Approach */
"0LRR07Z"  =  "1"  /* Replacement of L Knee Tendon with Autol Sub, Open Approach */
"0LRR0JZ"  =  "1"  /* Replacement of L Knee Tendon with Synth Sub, Open Approach */
"0LRR0KZ"  =  "1"  /* Replacement of L Knee Tendon with Nonaut Sub, Open Approach */
"0LRR47Z"  =  "1"  /* Replace of L Knee Tendon with Autol Sub, Perc Endo Approach */
"0LRR4JZ"  =  "1"  /* Replace of L Knee Tendon with Synth Sub, Perc Endo Approach */
"0LRR4KZ"  =  "1"  /* Replace of L Knee Tendon with Nonaut Sub, Perc Endo Approach */
"0LRS07Z"  =  "1"  /* Replacement of R Ankle Tendon with Autol Sub, Open Approach */
"0LRS0JZ"  =  "1"  /* Replacement of R Ankle Tendon with Synth Sub, Open Approach */
"0LRS0KZ"  =  "1"  /* Replacement of R Ankle Tendon with Nonaut Sub, Open Approach */
"0LRS47Z"  =  "1"  /* Replace of R Ankle Tendon with Autol Sub, Perc Endo Approach */
"0LRS4JZ"  =  "1"  /* Replace of R Ankle Tendon with Synth Sub, Perc Endo Approach */
"0LRS4KZ"  =  "1"  /* Replace R Ankle Tendon w Nonaut Sub, Perc Endo */
"0LRT07Z"  =  "1"  /* Replacement of L Ankle Tendon with Autol Sub, Open Approach */
"0LRT0JZ"  =  "1"  /* Replacement of L Ankle Tendon with Synth Sub, Open Approach */
"0LRT0KZ"  =  "1"  /* Replacement of L Ankle Tendon with Nonaut Sub, Open Approach */
"0LRT47Z"  =  "1"  /* Replace of L Ankle Tendon with Autol Sub, Perc Endo Approach */
"0LRT4JZ"  =  "1"  /* Replace of L Ankle Tendon with Synth Sub, Perc Endo Approach */
"0LRT4KZ"  =  "1"  /* Replace L Ankle Tendon w Nonaut Sub, Perc Endo */
"0LRV07Z"  =  "1"  /* Replacement of R Foot Tendon with Autol Sub, Open Approach */
"0LRV0JZ"  =  "1"  /* Replacement of R Foot Tendon with Synth Sub, Open Approach */
"0LRV0KZ"  =  "1"  /* Replacement of R Foot Tendon with Nonaut Sub, Open Approach */
"0LRV47Z"  =  "1"  /* Replace of R Foot Tendon with Autol Sub, Perc Endo Approach */
"0LRV4JZ"  =  "1"  /* Replace of R Foot Tendon with Synth Sub, Perc Endo Approach */
"0LRV4KZ"  =  "1"  /* Replace of R Foot Tendon with Nonaut Sub, Perc Endo Approach */
"0LRW07Z"  =  "1"  /* Replacement of L Foot Tendon with Autol Sub, Open Approach */
"0LRW0JZ"  =  "1"  /* Replacement of L Foot Tendon with Synth Sub, Open Approach */
"0LRW0KZ"  =  "1"  /* Replacement of L Foot Tendon with Nonaut Sub, Open Approach */
"0LRW47Z"  =  "1"  /* Replace of L Foot Tendon with Autol Sub, Perc Endo Approach */
"0LRW4JZ"  =  "1"  /* Replace of L Foot Tendon with Synth Sub, Perc Endo Approach */
"0LRW4KZ"  =  "1"  /* Replace of L Foot Tendon with Nonaut Sub, Perc Endo Approach */
"0LS00ZZ"  =  "1"  /* Reposition Head and Neck Tendon, Open Approach */
"0LS04ZZ"  =  "1"  /* Reposition Head and Neck Tendon, Perc Endo Approach */
"0LS10ZZ"  =  "1"  /* Reposition Right Shoulder Tendon, Open Approach */
"0LS14ZZ"  =  "1"  /* Reposition Right Shoulder Tendon, Perc Endo Approach */
"0LS20ZZ"  =  "1"  /* Reposition Left Shoulder Tendon, Open Approach */
"0LS24ZZ"  =  "1"  /* Reposition Left Shoulder Tendon, Perc Endo Approach */
"0LS30ZZ"  =  "1"  /* Reposition Right Upper Arm Tendon, Open Approach */
"0LS34ZZ"  =  "1"  /* Reposition Right Upper Arm Tendon, Perc Endo Approach */
"0LS40ZZ"  =  "1"  /* Reposition Left Upper Arm Tendon, Open Approach */
"0LS44ZZ"  =  "1"  /* Reposition Left Upper Arm Tendon, Perc Endo Approach */
"0LS50ZZ"  =  "1"  /* Reposition Right Lower Arm and Wrist Tendon, Open Approach */
"0LS54ZZ"  =  "1"  /* Reposition R Low Arm & Wrist Tendon, Perc Endo Approach */
"0LS60ZZ"  =  "1"  /* Reposition Left Lower Arm and Wrist Tendon, Open Approach */
"0LS64ZZ"  =  "1"  /* Reposition L Low Arm & Wrist Tendon, Perc Endo Approach */
"0LS70ZZ"  =  "1"  /* Reposition Right Hand Tendon, Open Approach */
"0LS74ZZ"  =  "1"  /* Reposition Right Hand Tendon, Perc Endo Approach */
"0LS80ZZ"  =  "1"  /* Reposition Left Hand Tendon, Open Approach */
"0LS84ZZ"  =  "1"  /* Reposition Left Hand Tendon, Perc Endo Approach */
"0LS90ZZ"  =  "1"  /* Reposition Right Trunk Tendon, Open Approach */
"0LS94ZZ"  =  "1"  /* Reposition Right Trunk Tendon, Perc Endo Approach */
"0LSB0ZZ"  =  "1"  /* Reposition Left Trunk Tendon, Open Approach */
"0LSB4ZZ"  =  "1"  /* Reposition Left Trunk Tendon, Perc Endo Approach */
"0LSC0ZZ"  =  "1"  /* Reposition Right Thorax Tendon, Open Approach */
"0LSC4ZZ"  =  "1"  /* Reposition Right Thorax Tendon, Perc Endo Approach */
"0LSD0ZZ"  =  "1"  /* Reposition Left Thorax Tendon, Open Approach */
"0LSD4ZZ"  =  "1"  /* Reposition Left Thorax Tendon, Perc Endo Approach */
"0LSF0ZZ"  =  "1"  /* Reposition Right Abdomen Tendon, Open Approach */
"0LSF4ZZ"  =  "1"  /* Reposition Right Abdomen Tendon, Perc Endo Approach */
"0LSG0ZZ"  =  "1"  /* Reposition Left Abdomen Tendon, Open Approach */
"0LSG4ZZ"  =  "1"  /* Reposition Left Abdomen Tendon, Perc Endo Approach */
"0LSH0ZZ"  =  "1"  /* Reposition Perineum Tendon, Open Approach */
"0LSH4ZZ"  =  "1"  /* Reposition Perineum Tendon, Percutaneous Endoscopic Approach */
"0LSJ0ZZ"  =  "1"  /* Reposition Right Hip Tendon, Open Approach */
"0LSJ4ZZ"  =  "1"  /* Reposition Right Hip Tendon, Perc Endo Approach */
"0LSK0ZZ"  =  "1"  /* Reposition Left Hip Tendon, Open Approach */
"0LSK4ZZ"  =  "1"  /* Reposition Left Hip Tendon, Percutaneous Endoscopic Approach */
"0LSL0ZZ"  =  "1"  /* Reposition Right Upper Leg Tendon, Open Approach */
"0LSL4ZZ"  =  "1"  /* Reposition Right Upper Leg Tendon, Perc Endo Approach */
"0LSM0ZZ"  =  "1"  /* Reposition Left Upper Leg Tendon, Open Approach */
"0LSM4ZZ"  =  "1"  /* Reposition Left Upper Leg Tendon, Perc Endo Approach */
"0LSN0ZZ"  =  "1"  /* Reposition Right Lower Leg Tendon, Open Approach */
"0LSN4ZZ"  =  "1"  /* Reposition Right Lower Leg Tendon, Perc Endo Approach */
"0LSP0ZZ"  =  "1"  /* Reposition Left Lower Leg Tendon, Open Approach */
"0LSP4ZZ"  =  "1"  /* Reposition Left Lower Leg Tendon, Perc Endo Approach */
"0LSQ0ZZ"  =  "1"  /* Reposition Right Knee Tendon, Open Approach */
"0LSQ4ZZ"  =  "1"  /* Reposition Right Knee Tendon, Perc Endo Approach */
"0LSR0ZZ"  =  "1"  /* Reposition Left Knee Tendon, Open Approach */
"0LSR4ZZ"  =  "1"  /* Reposition Left Knee Tendon, Perc Endo Approach */
"0LSS0ZZ"  =  "1"  /* Reposition Right Ankle Tendon, Open Approach */
"0LSS4ZZ"  =  "1"  /* Reposition Right Ankle Tendon, Perc Endo Approach */
"0LST0ZZ"  =  "1"  /* Reposition Left Ankle Tendon, Open Approach */
"0LST4ZZ"  =  "1"  /* Reposition Left Ankle Tendon, Perc Endo Approach */
"0LSV0ZZ"  =  "1"  /* Reposition Right Foot Tendon, Open Approach */
"0LSV4ZZ"  =  "1"  /* Reposition Right Foot Tendon, Perc Endo Approach */
"0LSW0ZZ"  =  "1"  /* Reposition Left Foot Tendon, Open Approach */
"0LSW4ZZ"  =  "1"  /* Reposition Left Foot Tendon, Perc Endo Approach */
"0LT00ZZ"  =  "1"  /* Resection of Head and Neck Tendon, Open Approach */
"0LT04ZZ"  =  "1"  /* Resection of Head and Neck Tendon, Perc Endo Approach */
"0LT10ZZ"  =  "1"  /* Resection of Right Shoulder Tendon, Open Approach */
"0LT14ZZ"  =  "1"  /* Resection of Right Shoulder Tendon, Perc Endo Approach */
"0LT20ZZ"  =  "1"  /* Resection of Left Shoulder Tendon, Open Approach */
"0LT24ZZ"  =  "1"  /* Resection of Left Shoulder Tendon, Perc Endo Approach */
"0LT30ZZ"  =  "1"  /* Resection of Right Upper Arm Tendon, Open Approach */
"0LT34ZZ"  =  "1"  /* Resection of Right Upper Arm Tendon, Perc Endo Approach */
"0LT40ZZ"  =  "1"  /* Resection of Left Upper Arm Tendon, Open Approach */
"0LT44ZZ"  =  "1"  /* Resection of Left Upper Arm Tendon, Perc Endo Approach */
"0LT50ZZ"  =  "1"  /* Resection of Right Lower Arm and Wrist Tendon, Open Approach */
"0LT54ZZ"  =  "1"  /* Resection of R Low Arm & Wrist Tendon, Perc Endo Approach */
"0LT60ZZ"  =  "1"  /* Resection of Left Lower Arm and Wrist Tendon, Open Approach */
"0LT64ZZ"  =  "1"  /* Resection of L Low Arm & Wrist Tendon, Perc Endo Approach */
"0LT70ZZ"  =  "1"  /* Resection of Right Hand Tendon, Open Approach */
"0LT74ZZ"  =  "1"  /* Resection of Right Hand Tendon, Perc Endo Approach */
"0LT80ZZ"  =  "1"  /* Resection of Left Hand Tendon, Open Approach */
"0LT84ZZ"  =  "1"  /* Resection of Left Hand Tendon, Perc Endo Approach */
"0LT90ZZ"  =  "1"  /* Resection of Right Trunk Tendon, Open Approach */
"0LT94ZZ"  =  "1"  /* Resection of Right Trunk Tendon, Perc Endo Approach */
"0LTB0ZZ"  =  "1"  /* Resection of Left Trunk Tendon, Open Approach */
"0LTB4ZZ"  =  "1"  /* Resection of Left Trunk Tendon, Perc Endo Approach */
"0LTC0ZZ"  =  "1"  /* Resection of Right Thorax Tendon, Open Approach */
"0LTC4ZZ"  =  "1"  /* Resection of Right Thorax Tendon, Perc Endo Approach */
"0LTD0ZZ"  =  "1"  /* Resection of Left Thorax Tendon, Open Approach */
"0LTD4ZZ"  =  "1"  /* Resection of Left Thorax Tendon, Perc Endo Approach */
"0LTF0ZZ"  =  "1"  /* Resection of Right Abdomen Tendon, Open Approach */
"0LTF4ZZ"  =  "1"  /* Resection of Right Abdomen Tendon, Perc Endo Approach */
"0LTG0ZZ"  =  "1"  /* Resection of Left Abdomen Tendon, Open Approach */
"0LTG4ZZ"  =  "1"  /* Resection of Left Abdomen Tendon, Perc Endo Approach */
"0LTH0ZZ"  =  "1"  /* Resection of Perineum Tendon, Open Approach */
"0LTH4ZZ"  =  "1"  /* Resection of Perineum Tendon, Perc Endo Approach */
"0LTJ0ZZ"  =  "1"  /* Resection of Right Hip Tendon, Open Approach */
"0LTJ4ZZ"  =  "1"  /* Resection of Right Hip Tendon, Perc Endo Approach */
"0LTK0ZZ"  =  "1"  /* Resection of Left Hip Tendon, Open Approach */
"0LTK4ZZ"  =  "1"  /* Resection of Left Hip Tendon, Perc Endo Approach */
"0LTL0ZZ"  =  "1"  /* Resection of Right Upper Leg Tendon, Open Approach */
"0LTL4ZZ"  =  "1"  /* Resection of Right Upper Leg Tendon, Perc Endo Approach */
"0LTM0ZZ"  =  "1"  /* Resection of Left Upper Leg Tendon, Open Approach */
"0LTM4ZZ"  =  "1"  /* Resection of Left Upper Leg Tendon, Perc Endo Approach */
"0LTN0ZZ"  =  "1"  /* Resection of Right Lower Leg Tendon, Open Approach */
"0LTN4ZZ"  =  "1"  /* Resection of Right Lower Leg Tendon, Perc Endo Approach */
"0LTP0ZZ"  =  "1"  /* Resection of Left Lower Leg Tendon, Open Approach */
"0LTP4ZZ"  =  "1"  /* Resection of Left Lower Leg Tendon, Perc Endo Approach */
"0LTQ0ZZ"  =  "1"  /* Resection of Right Knee Tendon, Open Approach */
"0LTQ4ZZ"  =  "1"  /* Resection of Right Knee Tendon, Perc Endo Approach */
"0LTR0ZZ"  =  "1"  /* Resection of Left Knee Tendon, Open Approach */
"0LTR4ZZ"  =  "1"  /* Resection of Left Knee Tendon, Perc Endo Approach */
"0LTS0ZZ"  =  "1"  /* Resection of Right Ankle Tendon, Open Approach */
"0LTS4ZZ"  =  "1"  /* Resection of Right Ankle Tendon, Perc Endo Approach */
"0LTT0ZZ"  =  "1"  /* Resection of Left Ankle Tendon, Open Approach */
"0LTT4ZZ"  =  "1"  /* Resection of Left Ankle Tendon, Perc Endo Approach */
"0LTV0ZZ"  =  "1"  /* Resection of Right Foot Tendon, Open Approach */
"0LTV4ZZ"  =  "1"  /* Resection of Right Foot Tendon, Perc Endo Approach */
"0LTW0ZZ"  =  "1"  /* Resection of Left Foot Tendon, Open Approach */
"0LTW4ZZ"  =  "1"  /* Resection of Left Foot Tendon, Perc Endo Approach */
"0LU007Z"  =  "1"  /* Supplement Head & Neck Tendon with Autol Sub, Open Approach */
"0LU00JZ"  =  "1"  /* Supplement Head & Neck Tendon with Synth Sub, Open Approach */
"0LU00KZ"  =  "1"  /* Supplement Head & Neck Tendon with Nonaut Sub, Open Approach */
"0LU047Z"  =  "1"  /* Supplement Head & Neck Tendon w Autol Sub, Perc Endo */
"0LU04JZ"  =  "1"  /* Supplement Head & Neck Tendon w Synth Sub, Perc Endo */
"0LU04KZ"  =  "1"  /* Supplement Head & Neck Tendon w Nonaut Sub, Perc Endo */
"0LU107Z"  =  "1"  /* Supplement R Shoulder Tendon with Autol Sub, Open Approach */
"0LU10JZ"  =  "1"  /* Supplement R Shoulder Tendon with Synth Sub, Open Approach */
"0LU10KZ"  =  "1"  /* Supplement R Shoulder Tendon with Nonaut Sub, Open Approach */
"0LU147Z"  =  "1"  /* Supplement R Shoulder Tendon w Autol Sub, Perc Endo */
"0LU14JZ"  =  "1"  /* Supplement R Shoulder Tendon w Synth Sub, Perc Endo */
"0LU14KZ"  =  "1"  /* Supplement R Shoulder Tendon w Nonaut Sub, Perc Endo */
"0LU207Z"  =  "1"  /* Supplement L Shoulder Tendon with Autol Sub, Open Approach */
"0LU20JZ"  =  "1"  /* Supplement L Shoulder Tendon with Synth Sub, Open Approach */
"0LU20KZ"  =  "1"  /* Supplement L Shoulder Tendon with Nonaut Sub, Open Approach */
"0LU247Z"  =  "1"  /* Supplement L Shoulder Tendon w Autol Sub, Perc Endo */
"0LU24JZ"  =  "1"  /* Supplement L Shoulder Tendon w Synth Sub, Perc Endo */
"0LU24KZ"  =  "1"  /* Supplement L Shoulder Tendon w Nonaut Sub, Perc Endo */
"0LU307Z"  =  "1"  /* Supplement R Up Arm Tendon with Autol Sub, Open Approach */
"0LU30JZ"  =  "1"  /* Supplement R Up Arm Tendon with Synth Sub, Open Approach */
"0LU30KZ"  =  "1"  /* Supplement R Up Arm Tendon with Nonaut Sub, Open Approach */
"0LU347Z"  =  "1"  /* Supplement R Up Arm Tendon w Autol Sub, Perc Endo */
"0LU34JZ"  =  "1"  /* Supplement R Up Arm Tendon w Synth Sub, Perc Endo */
"0LU34KZ"  =  "1"  /* Supplement R Up Arm Tendon w Nonaut Sub, Perc Endo */
"0LU407Z"  =  "1"  /* Supplement L Up Arm Tendon with Autol Sub, Open Approach */
"0LU40JZ"  =  "1"  /* Supplement L Up Arm Tendon with Synth Sub, Open Approach */
"0LU40KZ"  =  "1"  /* Supplement L Up Arm Tendon with Nonaut Sub, Open Approach */
"0LU447Z"  =  "1"  /* Supplement L Up Arm Tendon w Autol Sub, Perc Endo */
"0LU44JZ"  =  "1"  /* Supplement L Up Arm Tendon w Synth Sub, Perc Endo */
"0LU44KZ"  =  "1"  /* Supplement L Up Arm Tendon w Nonaut Sub, Perc Endo */
"0LU507Z"  =  "1"  /* Supplement R Low Arm & Wrist Tendon w Autol Sub, Open */
"0LU50JZ"  =  "1"  /* Supplement R Low Arm & Wrist Tendon w Synth Sub, Open */
"0LU50KZ"  =  "1"  /* Supplement R Low Arm & Wrist Tendon w Nonaut Sub, Open */
"0LU547Z"  =  "1"  /* Supplement R Low Arm & Wrist Tendon w Autol Sub, Perc Endo */
"0LU54JZ"  =  "1"  /* Supplement R Low Arm & Wrist Tendon w Synth Sub, Perc Endo */
"0LU54KZ"  =  "1"  /* Supplement R Low Arm & Wrist Tendon w Nonaut Sub, Perc Endo */
"0LU607Z"  =  "1"  /* Supplement L Low Arm & Wrist Tendon w Autol Sub, Open */
"0LU60JZ"  =  "1"  /* Supplement L Low Arm & Wrist Tendon w Synth Sub, Open */
"0LU60KZ"  =  "1"  /* Supplement L Low Arm & Wrist Tendon w Nonaut Sub, Open */
"0LU647Z"  =  "1"  /* Supplement L Low Arm & Wrist Tendon w Autol Sub, Perc Endo */
"0LU64JZ"  =  "1"  /* Supplement L Low Arm & Wrist Tendon w Synth Sub, Perc Endo */
"0LU64KZ"  =  "1"  /* Supplement L Low Arm & Wrist Tendon w Nonaut Sub, Perc Endo */
"0LU707Z"  =  "1"  /* Supplement Right Hand Tendon with Autol Sub, Open Approach */
"0LU70JZ"  =  "1"  /* Supplement Right Hand Tendon with Synth Sub, Open Approach */
"0LU70KZ"  =  "1"  /* Supplement Right Hand Tendon with Nonaut Sub, Open Approach */
"0LU747Z"  =  "1"  /* Supplement R Hand Tendon with Autol Sub, Perc Endo Approach */
"0LU74JZ"  =  "1"  /* Supplement R Hand Tendon with Synth Sub, Perc Endo Approach */
"0LU74KZ"  =  "1"  /* Supplement R Hand Tendon with Nonaut Sub, Perc Endo Approach */
"0LU807Z"  =  "1"  /* Supplement Left Hand Tendon with Autol Sub, Open Approach */
"0LU80JZ"  =  "1"  /* Supplement Left Hand Tendon with Synth Sub, Open Approach */
"0LU80KZ"  =  "1"  /* Supplement Left Hand Tendon with Nonaut Sub, Open Approach */
"0LU847Z"  =  "1"  /* Supplement L Hand Tendon with Autol Sub, Perc Endo Approach */
"0LU84JZ"  =  "1"  /* Supplement L Hand Tendon with Synth Sub, Perc Endo Approach */
"0LU84KZ"  =  "1"  /* Supplement L Hand Tendon with Nonaut Sub, Perc Endo Approach */
"0LU907Z"  =  "1"  /* Supplement Right Trunk Tendon with Autol Sub, Open Approach */
"0LU90JZ"  =  "1"  /* Supplement Right Trunk Tendon with Synth Sub, Open Approach */
"0LU90KZ"  =  "1"  /* Supplement Right Trunk Tendon with Nonaut Sub, Open Approach */
"0LU947Z"  =  "1"  /* Supplement R Trunk Tendon with Autol Sub, Perc Endo Approach */
"0LU94JZ"  =  "1"  /* Supplement R Trunk Tendon with Synth Sub, Perc Endo Approach */
"0LU94KZ"  =  "1"  /* Supplement R Trunk Tendon w Nonaut Sub, Perc Endo */
"0LUB07Z"  =  "1"  /* Supplement Left Trunk Tendon with Autol Sub, Open Approach */
"0LUB0JZ"  =  "1"  /* Supplement Left Trunk Tendon with Synth Sub, Open Approach */
"0LUB0KZ"  =  "1"  /* Supplement Left Trunk Tendon with Nonaut Sub, Open Approach */
"0LUB47Z"  =  "1"  /* Supplement L Trunk Tendon with Autol Sub, Perc Endo Approach */
"0LUB4JZ"  =  "1"  /* Supplement L Trunk Tendon with Synth Sub, Perc Endo Approach */
"0LUB4KZ"  =  "1"  /* Supplement L Trunk Tendon w Nonaut Sub, Perc Endo */
"0LUC07Z"  =  "1"  /* Supplement Right Thorax Tendon with Autol Sub, Open Approach */
"0LUC0JZ"  =  "1"  /* Supplement Right Thorax Tendon with Synth Sub, Open Approach */
"0LUC0KZ"  =  "1"  /* Supplement R Thorax Tendon with Nonaut Sub, Open Approach */
"0LUC47Z"  =  "1"  /* Supplement R Thorax Tendon w Autol Sub, Perc Endo */
"0LUC4JZ"  =  "1"  /* Supplement R Thorax Tendon w Synth Sub, Perc Endo */
"0LUC4KZ"  =  "1"  /* Supplement R Thorax Tendon w Nonaut Sub, Perc Endo */
"0LUD07Z"  =  "1"  /* Supplement Left Thorax Tendon with Autol Sub, Open Approach */
"0LUD0JZ"  =  "1"  /* Supplement Left Thorax Tendon with Synth Sub, Open Approach */
"0LUD0KZ"  =  "1"  /* Supplement Left Thorax Tendon with Nonaut Sub, Open Approach */
"0LUD47Z"  =  "1"  /* Supplement L Thorax Tendon w Autol Sub, Perc Endo */
"0LUD4JZ"  =  "1"  /* Supplement L Thorax Tendon w Synth Sub, Perc Endo */
"0LUD4KZ"  =  "1"  /* Supplement L Thorax Tendon w Nonaut Sub, Perc Endo */
"0LUF07Z"  =  "1"  /* Supplement R Abd Tendon with Autol Sub, Open Approach */
"0LUF0JZ"  =  "1"  /* Supplement R Abd Tendon with Synth Sub, Open Approach */
"0LUF0KZ"  =  "1"  /* Supplement R Abd Tendon with Nonaut Sub, Open Approach */
"0LUF47Z"  =  "1"  /* Supplement R Abd Tendon with Autol Sub, Perc Endo Approach */
"0LUF4JZ"  =  "1"  /* Supplement R Abd Tendon with Synth Sub, Perc Endo Approach */
"0LUF4KZ"  =  "1"  /* Supplement R Abd Tendon with Nonaut Sub, Perc Endo Approach */
"0LUG07Z"  =  "1"  /* Supplement Left Abdomen Tendon with Autol Sub, Open Approach */
"0LUG0JZ"  =  "1"  /* Supplement Left Abdomen Tendon with Synth Sub, Open Approach */
"0LUG0KZ"  =  "1"  /* Supplement L Abd Tendon with Nonaut Sub, Open Approach */
"0LUG47Z"  =  "1"  /* Supplement L Abd Tendon with Autol Sub, Perc Endo Approach */
"0LUG4JZ"  =  "1"  /* Supplement L Abd Tendon with Synth Sub, Perc Endo Approach */
"0LUG4KZ"  =  "1"  /* Supplement L Abd Tendon with Nonaut Sub, Perc Endo Approach */
"0LUH07Z"  =  "1"  /* Supplement Perineum Tendon with Autol Sub, Open Approach */
"0LUH0JZ"  =  "1"  /* Supplement Perineum Tendon with Synth Sub, Open Approach */
"0LUH0KZ"  =  "1"  /* Supplement Perineum Tendon with Nonaut Sub, Open Approach */
"0LUH47Z"  =  "1"  /* Supplement Perineum Tendon w Autol Sub, Perc Endo */
"0LUH4JZ"  =  "1"  /* Supplement Perineum Tendon w Synth Sub, Perc Endo */
"0LUH4KZ"  =  "1"  /* Supplement Perineum Tendon w Nonaut Sub, Perc Endo */
"0LUJ07Z"  =  "1"  /* Supplement Right Hip Tendon with Autol Sub, Open Approach */
"0LUJ0JZ"  =  "1"  /* Supplement Right Hip Tendon with Synth Sub, Open Approach */
"0LUJ0KZ"  =  "1"  /* Supplement Right Hip Tendon with Nonaut Sub, Open Approach */
"0LUJ47Z"  =  "1"  /* Supplement R Hip Tendon with Autol Sub, Perc Endo Approach */
"0LUJ4JZ"  =  "1"  /* Supplement R Hip Tendon with Synth Sub, Perc Endo Approach */
"0LUJ4KZ"  =  "1"  /* Supplement R Hip Tendon with Nonaut Sub, Perc Endo Approach */
"0LUK07Z"  =  "1"  /* Supplement Left Hip Tendon with Autol Sub, Open Approach */
"0LUK0JZ"  =  "1"  /* Supplement Left Hip Tendon with Synth Sub, Open Approach */
"0LUK0KZ"  =  "1"  /* Supplement Left Hip Tendon with Nonaut Sub, Open Approach */
"0LUK47Z"  =  "1"  /* Supplement L Hip Tendon with Autol Sub, Perc Endo Approach */
"0LUK4JZ"  =  "1"  /* Supplement L Hip Tendon with Synth Sub, Perc Endo Approach */
"0LUK4KZ"  =  "1"  /* Supplement L Hip Tendon with Nonaut Sub, Perc Endo Approach */
"0LUL07Z"  =  "1"  /* Supplement R Up Leg Tendon with Autol Sub, Open Approach */
"0LUL0JZ"  =  "1"  /* Supplement R Up Leg Tendon with Synth Sub, Open Approach */
"0LUL0KZ"  =  "1"  /* Supplement R Up Leg Tendon with Nonaut Sub, Open Approach */
"0LUL47Z"  =  "1"  /* Supplement R Up Leg Tendon w Autol Sub, Perc Endo */
"0LUL4JZ"  =  "1"  /* Supplement R Up Leg Tendon w Synth Sub, Perc Endo */
"0LUL4KZ"  =  "1"  /* Supplement R Up Leg Tendon w Nonaut Sub, Perc Endo */
"0LUM07Z"  =  "1"  /* Supplement L Up Leg Tendon with Autol Sub, Open Approach */
"0LUM0JZ"  =  "1"  /* Supplement L Up Leg Tendon with Synth Sub, Open Approach */
"0LUM0KZ"  =  "1"  /* Supplement L Up Leg Tendon with Nonaut Sub, Open Approach */
"0LUM47Z"  =  "1"  /* Supplement L Up Leg Tendon w Autol Sub, Perc Endo */
"0LUM4JZ"  =  "1"  /* Supplement L Up Leg Tendon w Synth Sub, Perc Endo */
"0LUM4KZ"  =  "1"  /* Supplement L Up Leg Tendon w Nonaut Sub, Perc Endo */
"0LUN07Z"  =  "1"  /* Supplement R Low Leg Tendon with Autol Sub, Open Approach */
"0LUN0JZ"  =  "1"  /* Supplement R Low Leg Tendon with Synth Sub, Open Approach */
"0LUN0KZ"  =  "1"  /* Supplement R Low Leg Tendon with Nonaut Sub, Open Approach */
"0LUN47Z"  =  "1"  /* Supplement R Low Leg Tendon w Autol Sub, Perc Endo */
"0LUN4JZ"  =  "1"  /* Supplement R Low Leg Tendon w Synth Sub, Perc Endo */
"0LUN4KZ"  =  "1"  /* Supplement R Low Leg Tendon w Nonaut Sub, Perc Endo */
"0LUP07Z"  =  "1"  /* Supplement L Low Leg Tendon with Autol Sub, Open Approach */
"0LUP0JZ"  =  "1"  /* Supplement L Low Leg Tendon with Synth Sub, Open Approach */
"0LUP0KZ"  =  "1"  /* Supplement L Low Leg Tendon with Nonaut Sub, Open Approach */
"0LUP47Z"  =  "1"  /* Supplement L Low Leg Tendon w Autol Sub, Perc Endo */
"0LUP4JZ"  =  "1"  /* Supplement L Low Leg Tendon w Synth Sub, Perc Endo */
"0LUP4KZ"  =  "1"  /* Supplement L Low Leg Tendon w Nonaut Sub, Perc Endo */
"0LUQ07Z"  =  "1"  /* Supplement Right Knee Tendon with Autol Sub, Open Approach */
"0LUQ0JZ"  =  "1"  /* Supplement Right Knee Tendon with Synth Sub, Open Approach */
"0LUQ0KZ"  =  "1"  /* Supplement Right Knee Tendon with Nonaut Sub, Open Approach */
"0LUQ47Z"  =  "1"  /* Supplement R Knee Tendon with Autol Sub, Perc Endo Approach */
"0LUQ4JZ"  =  "1"  /* Supplement R Knee Tendon with Synth Sub, Perc Endo Approach */
"0LUQ4KZ"  =  "1"  /* Supplement R Knee Tendon with Nonaut Sub, Perc Endo Approach */
"0LUR07Z"  =  "1"  /* Supplement Left Knee Tendon with Autol Sub, Open Approach */
"0LUR0JZ"  =  "1"  /* Supplement Left Knee Tendon with Synth Sub, Open Approach */
"0LUR0KZ"  =  "1"  /* Supplement Left Knee Tendon with Nonaut Sub, Open Approach */
"0LUR47Z"  =  "1"  /* Supplement L Knee Tendon with Autol Sub, Perc Endo Approach */
"0LUR4JZ"  =  "1"  /* Supplement L Knee Tendon with Synth Sub, Perc Endo Approach */
"0LUR4KZ"  =  "1"  /* Supplement L Knee Tendon with Nonaut Sub, Perc Endo Approach */
"0LUS07Z"  =  "1"  /* Supplement Right Ankle Tendon with Autol Sub, Open Approach */
"0LUS0JZ"  =  "1"  /* Supplement Right Ankle Tendon with Synth Sub, Open Approach */
"0LUS0KZ"  =  "1"  /* Supplement Right Ankle Tendon with Nonaut Sub, Open Approach */
"0LUS47Z"  =  "1"  /* Supplement R Ankle Tendon with Autol Sub, Perc Endo Approach */
"0LUS4JZ"  =  "1"  /* Supplement R Ankle Tendon with Synth Sub, Perc Endo Approach */
"0LUS4KZ"  =  "1"  /* Supplement R Ankle Tendon w Nonaut Sub, Perc Endo */
"0LUT07Z"  =  "1"  /* Supplement Left Ankle Tendon with Autol Sub, Open Approach */
"0LUT0JZ"  =  "1"  /* Supplement Left Ankle Tendon with Synth Sub, Open Approach */
"0LUT0KZ"  =  "1"  /* Supplement Left Ankle Tendon with Nonaut Sub, Open Approach */
"0LUT47Z"  =  "1"  /* Supplement L Ankle Tendon with Autol Sub, Perc Endo Approach */
"0LUT4JZ"  =  "1"  /* Supplement L Ankle Tendon with Synth Sub, Perc Endo Approach */
"0LUT4KZ"  =  "1"  /* Supplement L Ankle Tendon w Nonaut Sub, Perc Endo */
"0LUV07Z"  =  "1"  /* Supplement Right Foot Tendon with Autol Sub, Open Approach */
"0LUV0JZ"  =  "1"  /* Supplement Right Foot Tendon with Synth Sub, Open Approach */
"0LUV0KZ"  =  "1"  /* Supplement Right Foot Tendon with Nonaut Sub, Open Approach */
"0LUV47Z"  =  "1"  /* Supplement R Foot Tendon with Autol Sub, Perc Endo Approach */
"0LUV4JZ"  =  "1"  /* Supplement R Foot Tendon with Synth Sub, Perc Endo Approach */
"0LUV4KZ"  =  "1"  /* Supplement R Foot Tendon with Nonaut Sub, Perc Endo Approach */
"0LUW07Z"  =  "1"  /* Supplement Left Foot Tendon with Autol Sub, Open Approach */
"0LUW0JZ"  =  "1"  /* Supplement Left Foot Tendon with Synth Sub, Open Approach */
"0LUW0KZ"  =  "1"  /* Supplement Left Foot Tendon with Nonaut Sub, Open Approach */
"0LUW47Z"  =  "1"  /* Supplement L Foot Tendon with Autol Sub, Perc Endo Approach */
"0LUW4JZ"  =  "1"  /* Supplement L Foot Tendon with Synth Sub, Perc Endo Approach */
"0LUW4KZ"  =  "1"  /* Supplement L Foot Tendon with Nonaut Sub, Perc Endo Approach */
"0LWX00Z"  =  "1"  /* Revision of Drainage Device in Upper Tendon, Open Approach */
"0LWX07Z"  =  "1"  /* Revision of Autol Sub in Up Tendon, Open Approach */
"0LWX0JZ"  =  "1"  /* Revision of Synthetic Substitute in Up Tendon, Open Approach */
"0LWX0KZ"  =  "1"  /* Revision of Nonaut Sub in Up Tendon, Open Approach */
"0LWX30Z"  =  "1"  /* Revision of Drainage Device in Upper Tendon, Perc Approach */
"0LWX37Z"  =  "1"  /* Revision of Autol Sub in Up Tendon, Perc Approach */
"0LWX3JZ"  =  "1"  /* Revision of Synthetic Substitute in Up Tendon, Perc Approach */
"0LWX3KZ"  =  "1"  /* Revision of Nonaut Sub in Up Tendon, Perc Approach */
"0LWX40Z"  =  "1"  /* Revision of Drainage Device in Up Tendon, Perc Endo Approach */
"0LWX47Z"  =  "1"  /* Revision of Autol Sub in Up Tendon, Perc Endo Approach */
"0LWX4JZ"  =  "1"  /* Revision of Synth Sub in Up Tendon, Perc Endo Approach */
"0LWX4KZ"  =  "1"  /* Revision of Nonaut Sub in Up Tendon, Perc Endo Approach */
"0LWY00Z"  =  "1"  /* Revision of Drainage Device in Lower Tendon, Open Approach */
"0LWY07Z"  =  "1"  /* Revision of Autol Sub in Low Tendon, Open Approach */
"0LWY0JZ"  =  "1"  /* Revision of Synth Sub in Low Tendon, Open Approach */
"0LWY0KZ"  =  "1"  /* Revision of Nonaut Sub in Low Tendon, Open Approach */
"0LWY30Z"  =  "1"  /* Revision of Drainage Device in Lower Tendon, Perc Approach */
"0LWY37Z"  =  "1"  /* Revision of Autol Sub in Low Tendon, Perc Approach */
"0LWY3JZ"  =  "1"  /* Revision of Synth Sub in Low Tendon, Perc Approach */
"0LWY3KZ"  =  "1"  /* Revision of Nonaut Sub in Low Tendon, Perc Approach */
"0LWY40Z"  =  "1"  /* Revision of Drain Dev in Low Tendon, Perc Endo Approach */
"0LWY47Z"  =  "1"  /* Revision of Autol Sub in Low Tendon, Perc Endo Approach */
"0LWY4JZ"  =  "1"  /* Revision of Synth Sub in Low Tendon, Perc Endo Approach */
"0LWY4KZ"  =  "1"  /* Revision of Nonaut Sub in Low Tendon, Perc Endo Approach */
"0LX00ZZ"  =  "1"  /* Transfer Head and Neck Tendon, Open Approach */
"0LX04ZZ"  =  "1"  /* Transfer Head and Neck Tendon, Perc Endo Approach */
"0LX10ZZ"  =  "1"  /* Transfer Right Shoulder Tendon, Open Approach */
"0LX14ZZ"  =  "1"  /* Transfer Right Shoulder Tendon, Perc Endo Approach */
"0LX20ZZ"  =  "1"  /* Transfer Left Shoulder Tendon, Open Approach */
"0LX24ZZ"  =  "1"  /* Transfer Left Shoulder Tendon, Perc Endo Approach */
"0LX30ZZ"  =  "1"  /* Transfer Right Upper Arm Tendon, Open Approach */
"0LX34ZZ"  =  "1"  /* Transfer Right Upper Arm Tendon, Perc Endo Approach */
"0LX40ZZ"  =  "1"  /* Transfer Left Upper Arm Tendon, Open Approach */
"0LX44ZZ"  =  "1"  /* Transfer Left Upper Arm Tendon, Perc Endo Approach */
"0LX50ZZ"  =  "1"  /* Transfer Right Lower Arm and Wrist Tendon, Open Approach */
"0LX54ZZ"  =  "1"  /* Transfer R Low Arm & Wrist Tendon, Perc Endo Approach */
"0LX60ZZ"  =  "1"  /* Transfer Left Lower Arm and Wrist Tendon, Open Approach */
"0LX64ZZ"  =  "1"  /* Transfer Left Lower Arm and Wrist Tendon, Perc Endo Approach */
"0LX70ZZ"  =  "1"  /* Transfer Right Hand Tendon, Open Approach */
"0LX74ZZ"  =  "1"  /* Transfer Right Hand Tendon, Percutaneous Endoscopic Approach */
"0LX80ZZ"  =  "1"  /* Transfer Left Hand Tendon, Open Approach */
"0LX84ZZ"  =  "1"  /* Transfer Left Hand Tendon, Percutaneous Endoscopic Approach */
"0LX90ZZ"  =  "1"  /* Transfer Right Trunk Tendon, Open Approach */
"0LX94ZZ"  =  "1"  /* Transfer Right Trunk Tendon, Perc Endo Approach */
"0LXB0ZZ"  =  "1"  /* Transfer Left Trunk Tendon, Open Approach */
"0LXB4ZZ"  =  "1"  /* Transfer Left Trunk Tendon, Percutaneous Endoscopic Approach */
"0LXC0ZZ"  =  "1"  /* Transfer Right Thorax Tendon, Open Approach */
"0LXC4ZZ"  =  "1"  /* Transfer Right Thorax Tendon, Perc Endo Approach */
"0LXD0ZZ"  =  "1"  /* Transfer Left Thorax Tendon, Open Approach */
"0LXD4ZZ"  =  "1"  /* Transfer Left Thorax Tendon, Perc Endo Approach */
"0LXF0ZZ"  =  "1"  /* Transfer Right Abdomen Tendon, Open Approach */
"0LXF4ZZ"  =  "1"  /* Transfer Right Abdomen Tendon, Perc Endo Approach */
"0LXG0ZZ"  =  "1"  /* Transfer Left Abdomen Tendon, Open Approach */
"0LXG4ZZ"  =  "1"  /* Transfer Left Abdomen Tendon, Perc Endo Approach */
"0LXH0ZZ"  =  "1"  /* Transfer Perineum Tendon, Open Approach */
"0LXH4ZZ"  =  "1"  /* Transfer Perineum Tendon, Percutaneous Endoscopic Approach */
"0LXJ0ZZ"  =  "1"  /* Transfer Right Hip Tendon, Open Approach */
"0LXJ4ZZ"  =  "1"  /* Transfer Right Hip Tendon, Percutaneous Endoscopic Approach */
"0LXK0ZZ"  =  "1"  /* Transfer Left Hip Tendon, Open Approach */
"0LXK4ZZ"  =  "1"  /* Transfer Left Hip Tendon, Percutaneous Endoscopic Approach */
"0LXL0ZZ"  =  "1"  /* Transfer Right Upper Leg Tendon, Open Approach */
"0LXL4ZZ"  =  "1"  /* Transfer Right Upper Leg Tendon, Perc Endo Approach */
"0LXM0ZZ"  =  "1"  /* Transfer Left Upper Leg Tendon, Open Approach */
"0LXM4ZZ"  =  "1"  /* Transfer Left Upper Leg Tendon, Perc Endo Approach */
"0LXN0ZZ"  =  "1"  /* Transfer Right Lower Leg Tendon, Open Approach */
"0LXN4ZZ"  =  "1"  /* Transfer Right Lower Leg Tendon, Perc Endo Approach */
"0LXP0ZZ"  =  "1"  /* Transfer Left Lower Leg Tendon, Open Approach */
"0LXP4ZZ"  =  "1"  /* Transfer Left Lower Leg Tendon, Perc Endo Approach */
"0LXQ0ZZ"  =  "1"  /* Transfer Right Knee Tendon, Open Approach */
"0LXQ4ZZ"  =  "1"  /* Transfer Right Knee Tendon, Percutaneous Endoscopic Approach */
"0LXR0ZZ"  =  "1"  /* Transfer Left Knee Tendon, Open Approach */
"0LXR4ZZ"  =  "1"  /* Transfer Left Knee Tendon, Percutaneous Endoscopic Approach */
"0LXS0ZZ"  =  "1"  /* Transfer Right Ankle Tendon, Open Approach */
"0LXS4ZZ"  =  "1"  /* Transfer Right Ankle Tendon, Perc Endo Approach */
"0LXT0ZZ"  =  "1"  /* Transfer Left Ankle Tendon, Open Approach */
"0LXT4ZZ"  =  "1"  /* Transfer Left Ankle Tendon, Percutaneous Endoscopic Approach */
"0LXV0ZZ"  =  "1"  /* Transfer Right Foot Tendon, Open Approach */
"0LXV4ZZ"  =  "1"  /* Transfer Right Foot Tendon, Percutaneous Endoscopic Approach */
"0LXW0ZZ"  =  "1"  /* Transfer Left Foot Tendon, Open Approach */
"0LXW4ZZ"  =  "1"  /* Transfer Left Foot Tendon, Percutaneous Endoscopic Approach */
"0M500ZZ"  =  "1"  /* Destruction of Head & Neck Bursa/Lig, Open Approach */
"0M503ZZ"  =  "1"  /* Destruction of Head & Neck Bursa/Lig, Perc Approach */
"0M504ZZ"  =  "1"  /* Destruction of Head & Neck Bursa/Lig, Perc Endo Approach */
"0M510ZZ"  =  "1"  /* Destruction of R Shoulder Bursa/Lig, Open Approach */
"0M513ZZ"  =  "1"  /* Destruction of R Shoulder Bursa/Lig, Perc Approach */
"0M514ZZ"  =  "1"  /* Destruction of R Shoulder Bursa/Lig, Perc Endo Approach */
"0M520ZZ"  =  "1"  /* Destruction of L Shoulder Bursa/Lig, Open Approach */
"0M523ZZ"  =  "1"  /* Destruction of L Shoulder Bursa/Lig, Perc Approach */
"0M524ZZ"  =  "1"  /* Destruction of L Shoulder Bursa/Lig, Perc Endo Approach */
"0M530ZZ"  =  "1"  /* Destruction of Right Elbow Bursa and Ligament, Open Approach */
"0M533ZZ"  =  "1"  /* Destruction of Right Elbow Bursa and Ligament, Perc Approach */
"0M534ZZ"  =  "1"  /* Destruction of R Elbow Bursa/Lig, Perc Endo Approach */
"0M540ZZ"  =  "1"  /* Destruction of Left Elbow Bursa and Ligament, Open Approach */
"0M543ZZ"  =  "1"  /* Destruction of Left Elbow Bursa and Ligament, Perc Approach */
"0M544ZZ"  =  "1"  /* Destruction of L Elbow Bursa/Lig, Perc Endo Approach */
"0M550ZZ"  =  "1"  /* Destruction of Right Wrist Bursa and Ligament, Open Approach */
"0M553ZZ"  =  "1"  /* Destruction of Right Wrist Bursa and Ligament, Perc Approach */
"0M554ZZ"  =  "1"  /* Destruction of R Wrist Bursa/Lig, Perc Endo Approach */
"0M560ZZ"  =  "1"  /* Destruction of Left Wrist Bursa and Ligament, Open Approach */
"0M563ZZ"  =  "1"  /* Destruction of Left Wrist Bursa and Ligament, Perc Approach */
"0M564ZZ"  =  "1"  /* Destruction of L Wrist Bursa/Lig, Perc Endo Approach */
"0M570ZZ"  =  "1"  /* Destruction of Right Hand Bursa and Ligament, Open Approach */
"0M573ZZ"  =  "1"  /* Destruction of Right Hand Bursa and Ligament, Perc Approach */
"0M574ZZ"  =  "1"  /* Destruction of R Hand Bursa/Lig, Perc Endo Approach */
"0M580ZZ"  =  "1"  /* Destruction of Left Hand Bursa and Ligament, Open Approach */
"0M583ZZ"  =  "1"  /* Destruction of Left Hand Bursa and Ligament, Perc Approach */
"0M584ZZ"  =  "1"  /* Destruction of L Hand Bursa/Lig, Perc Endo Approach */
"0M590ZZ"  =  "1"  /* Destruction of R Up Extrem Bursa/Lig, Open Approach */
"0M593ZZ"  =  "1"  /* Destruction of R Up Extrem Bursa/Lig, Perc Approach */
"0M594ZZ"  =  "1"  /* Destruction of R Up Extrem Bursa/Lig, Perc Endo Approach */
"0M5B0ZZ"  =  "1"  /* Destruction of L Up Extrem Bursa/Lig, Open Approach */
"0M5B3ZZ"  =  "1"  /* Destruction of L Up Extrem Bursa/Lig, Perc Approach */
"0M5B4ZZ"  =  "1"  /* Destruction of L Up Extrem Bursa/Lig, Perc Endo Approach */
"0M5C0ZZ"  =  "1"  /* Destruction of Right Trunk Bursa and Ligament, Open Approach */
"0M5C3ZZ"  =  "1"  /* Destruction of Right Trunk Bursa and Ligament, Perc Approach */
"0M5C4ZZ"  =  "1"  /* Destruction of R Trunk Bursa/Lig, Perc Endo Approach */
"0M5D0ZZ"  =  "1"  /* Destruction of Left Trunk Bursa and Ligament, Open Approach */
"0M5D3ZZ"  =  "1"  /* Destruction of Left Trunk Bursa and Ligament, Perc Approach */
"0M5D4ZZ"  =  "1"  /* Destruction of L Trunk Bursa/Lig, Perc Endo Approach */
"0M5F0ZZ"  =  "1"  /* Destruction of R Thorax Bursa/Lig, Open Approach */
"0M5F3ZZ"  =  "1"  /* Destruction of R Thorax Bursa/Lig, Perc Approach */
"0M5F4ZZ"  =  "1"  /* Destruction of R Thorax Bursa/Lig, Perc Endo Approach */
"0M5G0ZZ"  =  "1"  /* Destruction of Left Thorax Bursa and Ligament, Open Approach */
"0M5G3ZZ"  =  "1"  /* Destruction of Left Thorax Bursa and Ligament, Perc Approach */
"0M5G4ZZ"  =  "1"  /* Destruction of L Thorax Bursa/Lig, Perc Endo Approach */
"0M5H0ZZ"  =  "1"  /* Destruction of R Abd Bursa/Lig, Open Approach */
"0M5H3ZZ"  =  "1"  /* Destruction of R Abd Bursa/Lig, Perc Approach */
"0M5H4ZZ"  =  "1"  /* Destruction of R Abd Bursa/Lig, Perc Endo Approach */
"0M5J0ZZ"  =  "1"  /* Destruction of L Abd Bursa/Lig, Open Approach */
"0M5J3ZZ"  =  "1"  /* Destruction of L Abd Bursa/Lig, Perc Approach */
"0M5J4ZZ"  =  "1"  /* Destruction of L Abd Bursa/Lig, Perc Endo Approach */
"0M5K0ZZ"  =  "1"  /* Destruction of Perineum Bursa and Ligament, Open Approach */
"0M5K3ZZ"  =  "1"  /* Destruction of Perineum Bursa and Ligament, Perc Approach */
"0M5K4ZZ"  =  "1"  /* Destruction of Perineum Bursa/Lig, Perc Endo Approach */
"0M5L0ZZ"  =  "1"  /* Destruction of Right Hip Bursa and Ligament, Open Approach */
"0M5L3ZZ"  =  "1"  /* Destruction of Right Hip Bursa and Ligament, Perc Approach */
"0M5L4ZZ"  =  "1"  /* Destruction of R Hip Bursa/Lig, Perc Endo Approach */
"0M5M0ZZ"  =  "1"  /* Destruction of Left Hip Bursa and Ligament, Open Approach */
"0M5M3ZZ"  =  "1"  /* Destruction of Left Hip Bursa and Ligament, Perc Approach */
"0M5M4ZZ"  =  "1"  /* Destruction of L Hip Bursa/Lig, Perc Endo Approach */
"0M5N0ZZ"  =  "1"  /* Destruction of Right Knee Bursa and Ligament, Open Approach */
"0M5N3ZZ"  =  "1"  /* Destruction of Right Knee Bursa and Ligament, Perc Approach */
"0M5N4ZZ"  =  "1"  /* Destruction of R Knee Bursa/Lig, Perc Endo Approach */
"0M5P0ZZ"  =  "1"  /* Destruction of Left Knee Bursa and Ligament, Open Approach */
"0M5P3ZZ"  =  "1"  /* Destruction of Left Knee Bursa and Ligament, Perc Approach */
"0M5P4ZZ"  =  "1"  /* Destruction of L Knee Bursa/Lig, Perc Endo Approach */
"0M5Q0ZZ"  =  "1"  /* Destruction of Right Ankle Bursa and Ligament, Open Approach */
"0M5Q3ZZ"  =  "1"  /* Destruction of Right Ankle Bursa and Ligament, Perc Approach */
"0M5Q4ZZ"  =  "1"  /* Destruction of R Ankle Bursa/Lig, Perc Endo Approach */
"0M5R0ZZ"  =  "1"  /* Destruction of Left Ankle Bursa and Ligament, Open Approach */
"0M5R3ZZ"  =  "1"  /* Destruction of Left Ankle Bursa and Ligament, Perc Approach */
"0M5R4ZZ"  =  "1"  /* Destruction of L Ankle Bursa/Lig, Perc Endo Approach */
"0M5S0ZZ"  =  "1"  /* Destruction of Right Foot Bursa and Ligament, Open Approach */
"0M5S3ZZ"  =  "1"  /* Destruction of Right Foot Bursa and Ligament, Perc Approach */
"0M5S4ZZ"  =  "1"  /* Destruction of R Foot Bursa/Lig, Perc Endo Approach */
"0M5T0ZZ"  =  "1"  /* Destruction of Left Foot Bursa and Ligament, Open Approach */
"0M5T3ZZ"  =  "1"  /* Destruction of Left Foot Bursa and Ligament, Perc Approach */
"0M5T4ZZ"  =  "1"  /* Destruction of L Foot Bursa/Lig, Perc Endo Approach */
"0M5V0ZZ"  =  "1"  /* Destruction of R Low Extrem Bursa/Lig, Open Approach */
"0M5V3ZZ"  =  "1"  /* Destruction of R Low Extrem Bursa/Lig, Perc Approach */
"0M5V4ZZ"  =  "1"  /* Destruction of R Low Extrem Bursa/Lig, Perc Endo Approach */
"0M5W0ZZ"  =  "1"  /* Destruction of L Low Extrem Bursa/Lig, Open Approach */
"0M5W3ZZ"  =  "1"  /* Destruction of L Low Extrem Bursa/Lig, Perc Approach */
"0M5W4ZZ"  =  "1"  /* Destruction of L Low Extrem Bursa/Lig, Perc Endo Approach */
"0M800ZZ"  =  "1"  /* Division of Head and Neck Bursa and Ligament, Open Approach */
"0M803ZZ"  =  "1"  /* Division of Head and Neck Bursa and Ligament, Perc Approach */
"0M804ZZ"  =  "1"  /* Division of Head & Neck Bursa/Lig, Perc Endo Approach */
"0M810ZZ"  =  "1"  /* Division of Right Shoulder Bursa and Ligament, Open Approach */
"0M813ZZ"  =  "1"  /* Division of Right Shoulder Bursa and Ligament, Perc Approach */
"0M814ZZ"  =  "1"  /* Division of R Shoulder Bursa/Lig, Perc Endo Approach */
"0M820ZZ"  =  "1"  /* Division of Left Shoulder Bursa and Ligament, Open Approach */
"0M823ZZ"  =  "1"  /* Division of Left Shoulder Bursa and Ligament, Perc Approach */
"0M824ZZ"  =  "1"  /* Division of L Shoulder Bursa/Lig, Perc Endo Approach */
"0M830ZZ"  =  "1"  /* Division of Right Elbow Bursa and Ligament, Open Approach */
"0M833ZZ"  =  "1"  /* Division of Right Elbow Bursa and Ligament, Perc Approach */
"0M834ZZ"  =  "1"  /* Division of R Elbow Bursa/Lig, Perc Endo Approach */
"0M840ZZ"  =  "1"  /* Division of Left Elbow Bursa and Ligament, Open Approach */
"0M843ZZ"  =  "1"  /* Division of Left Elbow Bursa and Ligament, Perc Approach */
"0M844ZZ"  =  "1"  /* Division of L Elbow Bursa/Lig, Perc Endo Approach */
"0M870ZZ"  =  "1"  /* Division of Right Hand Bursa and Ligament, Open Approach */
"0M873ZZ"  =  "1"  /* Division of Right Hand Bursa and Ligament, Perc Approach */
"0M874ZZ"  =  "1"  /* Division of R Hand Bursa/Lig, Perc Endo Approach */
"0M880ZZ"  =  "1"  /* Division of Left Hand Bursa and Ligament, Open Approach */
"0M883ZZ"  =  "1"  /* Division of Left Hand Bursa and Ligament, Perc Approach */
"0M884ZZ"  =  "1"  /* Division of Left Hand Bursa and Ligament, Perc Endo Approach */
"0M890ZZ"  =  "1"  /* Division of R Up Extrem Bursa/Lig, Open Approach */
"0M893ZZ"  =  "1"  /* Division of R Up Extrem Bursa/Lig, Perc Approach */
"0M894ZZ"  =  "1"  /* Division of R Up Extrem Bursa/Lig, Perc Endo Approach */
"0M8B0ZZ"  =  "1"  /* Division of L Up Extrem Bursa/Lig, Open Approach */
"0M8B3ZZ"  =  "1"  /* Division of L Up Extrem Bursa/Lig, Perc Approach */
"0M8B4ZZ"  =  "1"  /* Division of L Up Extrem Bursa/Lig, Perc Endo Approach */
"0M8C0ZZ"  =  "1"  /* Division of Right Trunk Bursa and Ligament, Open Approach */
"0M8C3ZZ"  =  "1"  /* Division of Right Trunk Bursa and Ligament, Perc Approach */
"0M8C4ZZ"  =  "1"  /* Division of R Trunk Bursa/Lig, Perc Endo Approach */
"0M8D0ZZ"  =  "1"  /* Division of Left Trunk Bursa and Ligament, Open Approach */
"0M8D3ZZ"  =  "1"  /* Division of Left Trunk Bursa and Ligament, Perc Approach */
"0M8D4ZZ"  =  "1"  /* Division of L Trunk Bursa/Lig, Perc Endo Approach */
"0M8F0ZZ"  =  "1"  /* Division of Right Thorax Bursa and Ligament, Open Approach */
"0M8F3ZZ"  =  "1"  /* Division of Right Thorax Bursa and Ligament, Perc Approach */
"0M8F4ZZ"  =  "1"  /* Division of R Thorax Bursa/Lig, Perc Endo Approach */
"0M8G0ZZ"  =  "1"  /* Division of Left Thorax Bursa and Ligament, Open Approach */
"0M8G3ZZ"  =  "1"  /* Division of Left Thorax Bursa and Ligament, Perc Approach */
"0M8G4ZZ"  =  "1"  /* Division of L Thorax Bursa/Lig, Perc Endo Approach */
"0M8H0ZZ"  =  "1"  /* Division of Right Abdomen Bursa and Ligament, Open Approach */
"0M8H3ZZ"  =  "1"  /* Division of Right Abdomen Bursa and Ligament, Perc Approach */
"0M8H4ZZ"  =  "1"  /* Division of R Abd Bursa/Lig, Perc Endo Approach */
"0M8J0ZZ"  =  "1"  /* Division of Left Abdomen Bursa and Ligament, Open Approach */
"0M8J3ZZ"  =  "1"  /* Division of Left Abdomen Bursa and Ligament, Perc Approach */
"0M8J4ZZ"  =  "1"  /* Division of L Abd Bursa/Lig, Perc Endo Approach */
"0M8K0ZZ"  =  "1"  /* Division of Perineum Bursa and Ligament, Open Approach */
"0M8K3ZZ"  =  "1"  /* Division of Perineum Bursa and Ligament, Perc Approach */
"0M8K4ZZ"  =  "1"  /* Division of Perineum Bursa and Ligament, Perc Endo Approach */
"0M8L0ZZ"  =  "1"  /* Division of Right Hip Bursa and Ligament, Open Approach */
"0M8L3ZZ"  =  "1"  /* Division of Right Hip Bursa and Ligament, Perc Approach */
"0M8L4ZZ"  =  "1"  /* Division of Right Hip Bursa and Ligament, Perc Endo Approach */
"0M8M0ZZ"  =  "1"  /* Division of Left Hip Bursa and Ligament, Open Approach */
"0M8M3ZZ"  =  "1"  /* Division of Left Hip Bursa and Ligament, Perc Approach */
"0M8M4ZZ"  =  "1"  /* Division of Left Hip Bursa and Ligament, Perc Endo Approach */
"0M8N0ZZ"  =  "1"  /* Division of Right Knee Bursa and Ligament, Open Approach */
"0M8N3ZZ"  =  "1"  /* Division of Right Knee Bursa and Ligament, Perc Approach */
"0M8N4ZZ"  =  "1"  /* Division of R Knee Bursa/Lig, Perc Endo Approach */
"0M8P0ZZ"  =  "1"  /* Division of Left Knee Bursa and Ligament, Open Approach */
"0M8P3ZZ"  =  "1"  /* Division of Left Knee Bursa and Ligament, Perc Approach */
"0M8P4ZZ"  =  "1"  /* Division of Left Knee Bursa and Ligament, Perc Endo Approach */
"0M8Q0ZZ"  =  "1"  /* Division of Right Ankle Bursa and Ligament, Open Approach */
"0M8Q3ZZ"  =  "1"  /* Division of Right Ankle Bursa and Ligament, Perc Approach */
"0M8Q4ZZ"  =  "1"  /* Division of R Ankle Bursa/Lig, Perc Endo Approach */
"0M8R0ZZ"  =  "1"  /* Division of Left Ankle Bursa and Ligament, Open Approach */
"0M8R3ZZ"  =  "1"  /* Division of Left Ankle Bursa and Ligament, Perc Approach */
"0M8R4ZZ"  =  "1"  /* Division of L Ankle Bursa/Lig, Perc Endo Approach */
"0M8S0ZZ"  =  "1"  /* Division of Right Foot Bursa and Ligament, Open Approach */
"0M8S3ZZ"  =  "1"  /* Division of Right Foot Bursa and Ligament, Perc Approach */
"0M8S4ZZ"  =  "1"  /* Division of R Foot Bursa/Lig, Perc Endo Approach */
"0M8T0ZZ"  =  "1"  /* Division of Left Foot Bursa and Ligament, Open Approach */
"0M8T3ZZ"  =  "1"  /* Division of Left Foot Bursa and Ligament, Perc Approach */
"0M8T4ZZ"  =  "1"  /* Division of Left Foot Bursa and Ligament, Perc Endo Approach */
"0M8V0ZZ"  =  "1"  /* Division of R Low Extrem Bursa/Lig, Open Approach */
"0M8V3ZZ"  =  "1"  /* Division of R Low Extrem Bursa/Lig, Perc Approach */
"0M8V4ZZ"  =  "1"  /* Division of R Low Extrem Bursa/Lig, Perc Endo Approach */
"0M8W0ZZ"  =  "1"  /* Division of L Low Extrem Bursa/Lig, Open Approach */
"0M8W3ZZ"  =  "1"  /* Division of L Low Extrem Bursa/Lig, Perc Approach */
"0M8W4ZZ"  =  "1"  /* Division of L Low Extrem Bursa/Lig, Perc Endo Approach */
"0M9000Z"  =  "1"  /* Drain of Head & Neck Bursa/Lig with Drain Dev, Open Approach */
"0M900ZZ"  =  "1"  /* Drainage of Head and Neck Bursa and Ligament, Open Approach */
"0M9030Z"  =  "1"  /* Drain of Head & Neck Bursa/Lig with Drain Dev, Perc Approach */
"0M9040Z"  =  "1"  /* Drain Head & Neck Bursa/Lig w Drain Dev, Perc Endo */
"0M9100Z"  =  "1"  /* Drain of R Shoulder Bursa/Lig with Drain Dev, Open Approach */
"0M910ZZ"  =  "1"  /* Drainage of Right Shoulder Bursa and Ligament, Open Approach */
"0M914ZZ"  =  "1"  /* Drainage of R Shoulder Bursa/Lig, Perc Endo Approach */
"0M9200Z"  =  "1"  /* Drain of L Shoulder Bursa/Lig with Drain Dev, Open Approach */
"0M920ZZ"  =  "1"  /* Drainage of Left Shoulder Bursa and Ligament, Open Approach */
"0M924ZZ"  =  "1"  /* Drainage of L Shoulder Bursa/Lig, Perc Endo Approach */
"0M9300Z"  =  "1"  /* Drainage of R Elbow Bursa/Lig with Drain Dev, Open Approach */
"0M930ZZ"  =  "1"  /* Drainage of Right Elbow Bursa and Ligament, Open Approach */
"0M934ZZ"  =  "1"  /* Drainage of R Elbow Bursa/Lig, Perc Endo Approach */
"0M9400Z"  =  "1"  /* Drainage of L Elbow Bursa/Lig with Drain Dev, Open Approach */
"0M940ZZ"  =  "1"  /* Drainage of Left Elbow Bursa and Ligament, Open Approach */
"0M944ZZ"  =  "1"  /* Drainage of L Elbow Bursa/Lig, Perc Endo Approach */
"0M9500Z"  =  "1"  /* Drainage of R Wrist Bursa/Lig with Drain Dev, Open Approach */
"0M950ZZ"  =  "1"  /* Drainage of Right Wrist Bursa and Ligament, Open Approach */
"0M9530Z"  =  "1"  /* Drainage of R Wrist Bursa/Lig with Drain Dev, Perc Approach */
"0M9540Z"  =  "1"  /* Drain R Wrist Bursa/Lig w Drain Dev, Perc Endo */
"0M9600Z"  =  "1"  /* Drainage of L Wrist Bursa/Lig with Drain Dev, Open Approach */
"0M960ZZ"  =  "1"  /* Drainage of Left Wrist Bursa and Ligament, Open Approach */
"0M9630Z"  =  "1"  /* Drainage of L Wrist Bursa/Lig with Drain Dev, Perc Approach */
"0M9640Z"  =  "1"  /* Drain L Wrist Bursa/Lig w Drain Dev, Perc Endo */
"0M9700Z"  =  "1"  /* Drainage of R Hand Bursa/Lig with Drain Dev, Open Approach */
"0M970ZZ"  =  "1"  /* Drainage of Right Hand Bursa and Ligament, Open Approach */
"0M9800Z"  =  "1"  /* Drainage of L Hand Bursa/Lig with Drain Dev, Open Approach */
"0M980ZZ"  =  "1"  /* Drainage of Left Hand Bursa and Ligament, Open Approach */
"0M9900Z"  =  "1"  /* Drain of R Up Extrem Bursa/Lig with Drain Dev, Open Approach */
"0M990ZX"  =  "1"  /* Drainage of R Up Extrem Bursa/Lig, Open Approach, Diagn */
"0M990ZZ"  =  "1"  /* Drainage of R Up Extrem Bursa/Lig, Open Approach */
"0M993ZX"  =  "1"  /* Drainage of R Up Extrem Bursa/Lig, Perc Approach, Diagn */
"0M994ZX"  =  "1"  /* Drainage of R Up Extrem Bursa/Lig, Perc Endo Approach, Diagn */
"0M9B00Z"  =  "1"  /* Drain of L Up Extrem Bursa/Lig with Drain Dev, Open Approach */
"0M9B0ZX"  =  "1"  /* Drainage of L Up Extrem Bursa/Lig, Open Approach, Diagn */
"0M9B0ZZ"  =  "1"  /* Drainage of L Up Extrem Bursa/Lig, Open Approach */
"0M9B3ZX"  =  "1"  /* Drainage of L Up Extrem Bursa/Lig, Perc Approach, Diagn */
"0M9B4ZX"  =  "1"  /* Drainage of L Up Extrem Bursa/Lig, Perc Endo Approach, Diagn */
"0M9C00Z"  =  "1"  /* Drainage of R Trunk Bursa/Lig with Drain Dev, Open Approach */
"0M9C0ZZ"  =  "1"  /* Drainage of Right Trunk Bursa and Ligament, Open Approach */
"0M9D00Z"  =  "1"  /* Drainage of L Trunk Bursa/Lig with Drain Dev, Open Approach */
"0M9D0ZZ"  =  "1"  /* Drainage of Left Trunk Bursa and Ligament, Open Approach */
"0M9F00Z"  =  "1"  /* Drainage of R Thorax Bursa/Lig with Drain Dev, Open Approach */
"0M9F0ZZ"  =  "1"  /* Drainage of Right Thorax Bursa and Ligament, Open Approach */
"0M9G00Z"  =  "1"  /* Drainage of L Thorax Bursa/Lig with Drain Dev, Open Approach */
"0M9G0ZZ"  =  "1"  /* Drainage of Left Thorax Bursa and Ligament, Open Approach */
"0M9H00Z"  =  "1"  /* Drainage of R Abd Bursa/Lig with Drain Dev, Open Approach */
"0M9H0ZX"  =  "1"  /* Drainage of R Abd Bursa/Lig, Open Approach, Diagn */
"0M9H0ZZ"  =  "1"  /* Drainage of Right Abdomen Bursa and Ligament, Open Approach */
"0M9H3ZX"  =  "1"  /* Drainage of R Abd Bursa/Lig, Perc Approach, Diagn */
"0M9H4ZX"  =  "1"  /* Drainage of R Abd Bursa/Lig, Perc Endo Approach, Diagn */
"0M9J00Z"  =  "1"  /* Drainage of L Abd Bursa/Lig with Drain Dev, Open Approach */
"0M9J0ZX"  =  "1"  /* Drainage of L Abd Bursa/Lig, Open Approach, Diagn */
"0M9J0ZZ"  =  "1"  /* Drainage of Left Abdomen Bursa and Ligament, Open Approach */
"0M9J3ZX"  =  "1"  /* Drainage of L Abd Bursa/Lig, Perc Approach, Diagn */
"0M9J4ZX"  =  "1"  /* Drainage of L Abd Bursa/Lig, Perc Endo Approach, Diagn */
"0M9K00Z"  =  "1"  /* Drainage of Perineum Bursa/Lig with Drain Dev, Open Approach */
"0M9K0ZX"  =  "1"  /* Drainage of Perineum Bursa/Lig, Open Approach, Diagn */
"0M9K0ZZ"  =  "1"  /* Drainage of Perineum Bursa and Ligament, Open Approach */
"0M9K3ZX"  =  "1"  /* Drainage of Perineum Bursa/Lig, Perc Approach, Diagn */
"0M9K4ZX"  =  "1"  /* Drainage of Perineum Bursa/Lig, Perc Endo Approach, Diagn */
"0M9L00Z"  =  "1"  /* Drainage of R Hip Bursa/Lig with Drain Dev, Open Approach */
"0M9L0ZZ"  =  "1"  /* Drainage of Right Hip Bursa and Ligament, Open Approach */
"0M9L4ZZ"  =  "1"  /* Drainage of Right Hip Bursa and Ligament, Perc Endo Approach */
"0M9M00Z"  =  "1"  /* Drainage of L Hip Bursa/Lig with Drain Dev, Open Approach */
"0M9M0ZZ"  =  "1"  /* Drainage of Left Hip Bursa and Ligament, Open Approach */
"0M9M4ZZ"  =  "1"  /* Drainage of Left Hip Bursa and Ligament, Perc Endo Approach */
"0M9N00Z"  =  "1"  /* Drainage of R Knee Bursa/Lig with Drain Dev, Open Approach */
"0M9N0ZZ"  =  "1"  /* Drainage of Right Knee Bursa and Ligament, Open Approach */
"0M9N30Z"  =  "1"  /* Drainage of R Knee Bursa/Lig with Drain Dev, Perc Approach */
"0M9N40Z"  =  "1"  /* Drain of R Knee Bursa/Lig with Drain Dev, Perc Endo Approach */
"0M9P00Z"  =  "1"  /* Drainage of L Knee Bursa/Lig with Drain Dev, Open Approach */
"0M9P0ZZ"  =  "1"  /* Drainage of Left Knee Bursa and Ligament, Open Approach */
"0M9P30Z"  =  "1"  /* Drainage of L Knee Bursa/Lig with Drain Dev, Perc Approach */
"0M9P40Z"  =  "1"  /* Drain of L Knee Bursa/Lig with Drain Dev, Perc Endo Approach */
"0M9Q00Z"  =  "1"  /* Drainage of R Ankle Bursa/Lig with Drain Dev, Open Approach */
"0M9Q0ZZ"  =  "1"  /* Drainage of Right Ankle Bursa and Ligament, Open Approach */
"0M9Q30Z"  =  "1"  /* Drainage of R Ankle Bursa/Lig with Drain Dev, Perc Approach */
"0M9Q40Z"  =  "1"  /* Drain R Ankle Bursa/Lig w Drain Dev, Perc Endo */
"0M9R00Z"  =  "1"  /* Drainage of L Ankle Bursa/Lig with Drain Dev, Open Approach */
"0M9R0ZZ"  =  "1"  /* Drainage of Left Ankle Bursa and Ligament, Open Approach */
"0M9R30Z"  =  "1"  /* Drainage of L Ankle Bursa/Lig with Drain Dev, Perc Approach */
"0M9R40Z"  =  "1"  /* Drain L Ankle Bursa/Lig w Drain Dev, Perc Endo */
"0M9S00Z"  =  "1"  /* Drainage of R Foot Bursa/Lig with Drain Dev, Open Approach */
"0M9S0ZZ"  =  "1"  /* Drainage of Right Foot Bursa and Ligament, Open Approach */
"0M9S30Z"  =  "1"  /* Drainage of R Foot Bursa/Lig with Drain Dev, Perc Approach */
"0M9S40Z"  =  "1"  /* Drain of R Foot Bursa/Lig with Drain Dev, Perc Endo Approach */
"0M9T00Z"  =  "1"  /* Drainage of L Foot Bursa/Lig with Drain Dev, Open Approach */
"0M9T0ZZ"  =  "1"  /* Drainage of Left Foot Bursa and Ligament, Open Approach */
"0M9T30Z"  =  "1"  /* Drainage of L Foot Bursa/Lig with Drain Dev, Perc Approach */
"0M9T40Z"  =  "1"  /* Drain of L Foot Bursa/Lig with Drain Dev, Perc Endo Approach */
"0M9V00Z"  =  "1"  /* Drain R Low Extrem Bursa/Lig w Drain Dev, Open */
"0M9V0ZX"  =  "1"  /* Drainage of R Low Extrem Bursa/Lig, Open Approach, Diagn */
"0M9V0ZZ"  =  "1"  /* Drainage of R Low Extrem Bursa/Lig, Open Approach */
"0M9V3ZX"  =  "1"  /* Drainage of R Low Extrem Bursa/Lig, Perc Approach, Diagn */
"0M9V4ZX"  =  "1"  /* Drain of R Low Extrem Bursa/Lig, Perc Endo Approach, Diagn */
"0M9W00Z"  =  "1"  /* Drain L Low Extrem Bursa/Lig w Drain Dev, Open */
"0M9W0ZX"  =  "1"  /* Drainage of L Low Extrem Bursa/Lig, Open Approach, Diagn */
"0M9W0ZZ"  =  "1"  /* Drainage of L Low Extrem Bursa/Lig, Open Approach */
"0M9W3ZX"  =  "1"  /* Drainage of L Low Extrem Bursa/Lig, Perc Approach, Diagn */
"0M9W4ZX"  =  "1"  /* Drain of L Low Extrem Bursa/Lig, Perc Endo Approach, Diagn */
"0MB00ZZ"  =  "1"  /* Excision of Head and Neck Bursa and Ligament, Open Approach */
"0MB03ZZ"  =  "1"  /* Excision of Head and Neck Bursa and Ligament, Perc Approach */
"0MB04ZZ"  =  "1"  /* Excision of Head & Neck Bursa/Lig, Perc Endo Approach */
"0MB10ZZ"  =  "1"  /* Excision of Right Shoulder Bursa and Ligament, Open Approach */
"0MB13ZZ"  =  "1"  /* Excision of Right Shoulder Bursa and Ligament, Perc Approach */
"0MB14ZZ"  =  "1"  /* Excision of R Shoulder Bursa/Lig, Perc Endo Approach */
"0MB20ZZ"  =  "1"  /* Excision of Left Shoulder Bursa and Ligament, Open Approach */
"0MB23ZZ"  =  "1"  /* Excision of Left Shoulder Bursa and Ligament, Perc Approach */
"0MB24ZZ"  =  "1"  /* Excision of L Shoulder Bursa/Lig, Perc Endo Approach */
"0MB30ZZ"  =  "1"  /* Excision of Right Elbow Bursa and Ligament, Open Approach */
"0MB33ZZ"  =  "1"  /* Excision of Right Elbow Bursa and Ligament, Perc Approach */
"0MB34ZZ"  =  "1"  /* Excision of R Elbow Bursa/Lig, Perc Endo Approach */
"0MB40ZZ"  =  "1"  /* Excision of Left Elbow Bursa and Ligament, Open Approach */
"0MB43ZZ"  =  "1"  /* Excision of Left Elbow Bursa and Ligament, Perc Approach */
"0MB44ZZ"  =  "1"  /* Excision of L Elbow Bursa/Lig, Perc Endo Approach */
"0MB50ZZ"  =  "1"  /* Excision of Right Wrist Bursa and Ligament, Open Approach */
"0MB53ZZ"  =  "1"  /* Excision of Right Wrist Bursa and Ligament, Perc Approach */
"0MB54ZZ"  =  "1"  /* Excision of R Wrist Bursa/Lig, Perc Endo Approach */
"0MB60ZZ"  =  "1"  /* Excision of Left Wrist Bursa and Ligament, Open Approach */
"0MB63ZZ"  =  "1"  /* Excision of Left Wrist Bursa and Ligament, Perc Approach */
"0MB64ZZ"  =  "1"  /* Excision of L Wrist Bursa/Lig, Perc Endo Approach */
"0MB70ZZ"  =  "1"  /* Excision of Right Hand Bursa and Ligament, Open Approach */
"0MB73ZZ"  =  "1"  /* Excision of Right Hand Bursa and Ligament, Perc Approach */
"0MB74ZZ"  =  "1"  /* Excision of R Hand Bursa/Lig, Perc Endo Approach */
"0MB80ZZ"  =  "1"  /* Excision of Left Hand Bursa and Ligament, Open Approach */
"0MB83ZZ"  =  "1"  /* Excision of Left Hand Bursa and Ligament, Perc Approach */
"0MB84ZZ"  =  "1"  /* Excision of Left Hand Bursa and Ligament, Perc Endo Approach */
"0MB90ZX"  =  "1"  /* Excision of R Up Extrem Bursa/Lig, Open Approach, Diagn */
"0MB90ZZ"  =  "1"  /* Excision of R Up Extrem Bursa/Lig, Open Approach */
"0MB93ZX"  =  "1"  /* Excision of R Up Extrem Bursa/Lig, Perc Approach, Diagn */
"0MB93ZZ"  =  "1"  /* Excision of R Up Extrem Bursa/Lig, Perc Approach */
"0MB94ZZ"  =  "1"  /* Excision of R Up Extrem Bursa/Lig, Perc Endo Approach */
"0MBB0ZZ"  =  "1"  /* Excision of L Up Extrem Bursa/Lig, Open Approach */
"0MBB3ZZ"  =  "1"  /* Excision of L Up Extrem Bursa/Lig, Perc Approach */
"0MBB4ZZ"  =  "1"  /* Excision of L Up Extrem Bursa/Lig, Perc Endo Approach */
"0MBC0ZZ"  =  "1"  /* Excision of Right Trunk Bursa and Ligament, Open Approach */
"0MBC3ZZ"  =  "1"  /* Excision of Right Trunk Bursa and Ligament, Perc Approach */
"0MBC4ZZ"  =  "1"  /* Excision of R Trunk Bursa/Lig, Perc Endo Approach */
"0MBD0ZZ"  =  "1"  /* Excision of Left Trunk Bursa and Ligament, Open Approach */
"0MBD3ZZ"  =  "1"  /* Excision of Left Trunk Bursa and Ligament, Perc Approach */
"0MBD4ZZ"  =  "1"  /* Excision of L Trunk Bursa/Lig, Perc Endo Approach */
"0MBF0ZZ"  =  "1"  /* Excision of Right Thorax Bursa and Ligament, Open Approach */
"0MBF3ZZ"  =  "1"  /* Excision of Right Thorax Bursa and Ligament, Perc Approach */
"0MBF4ZZ"  =  "1"  /* Excision of R Thorax Bursa/Lig, Perc Endo Approach */
"0MBG0ZZ"  =  "1"  /* Excision of Left Thorax Bursa and Ligament, Open Approach */
"0MBG3ZZ"  =  "1"  /* Excision of Left Thorax Bursa and Ligament, Perc Approach */
"0MBG4ZZ"  =  "1"  /* Excision of L Thorax Bursa/Lig, Perc Endo Approach */
"0MBH0ZX"  =  "1"  /* Excision of R Abd Bursa/Lig, Open Approach, Diagn */
"0MBH0ZZ"  =  "1"  /* Excision of Right Abdomen Bursa and Ligament, Open Approach */
"0MBH3ZX"  =  "1"  /* Excision of R Abd Bursa/Lig, Perc Approach, Diagn */
"0MBH3ZZ"  =  "1"  /* Excision of Right Abdomen Bursa and Ligament, Perc Approach */
"0MBH4ZX"  =  "1"  /* Excision of R Abd Bursa/Lig, Perc Endo Approach, Diagn */
"0MBH4ZZ"  =  "1"  /* Excision of R Abd Bursa/Lig, Perc Endo Approach */
"0MBJ0ZX"  =  "1"  /* Excision of L Abd Bursa/Lig, Open Approach, Diagn */
"0MBJ0ZZ"  =  "1"  /* Excision of Left Abdomen Bursa and Ligament, Open Approach */
"0MBJ3ZX"  =  "1"  /* Excision of L Abd Bursa/Lig, Perc Approach, Diagn */
"0MBJ3ZZ"  =  "1"  /* Excision of Left Abdomen Bursa and Ligament, Perc Approach */
"0MBJ4ZX"  =  "1"  /* Excision of L Abd Bursa/Lig, Perc Endo Approach, Diagn */
"0MBJ4ZZ"  =  "1"  /* Excision of L Abd Bursa/Lig, Perc Endo Approach */
"0MBK0ZX"  =  "1"  /* Excision of Perineum Bursa/Lig, Open Approach, Diagn */
"0MBK0ZZ"  =  "1"  /* Excision of Perineum Bursa and Ligament, Open Approach */
"0MBK3ZX"  =  "1"  /* Excision of Perineum Bursa/Lig, Perc Approach, Diagn */
"0MBK3ZZ"  =  "1"  /* Excision of Perineum Bursa and Ligament, Perc Approach */
"0MBK4ZX"  =  "1"  /* Excision of Perineum Bursa/Lig, Perc Endo Approach, Diagn */
"0MBK4ZZ"  =  "1"  /* Excision of Perineum Bursa and Ligament, Perc Endo Approach */
"0MBL0ZZ"  =  "1"  /* Excision of Right Hip Bursa and Ligament, Open Approach */
"0MBL3ZZ"  =  "1"  /* Excision of Right Hip Bursa and Ligament, Perc Approach */
"0MBL4ZZ"  =  "1"  /* Excision of Right Hip Bursa and Ligament, Perc Endo Approach */
"0MBM0ZZ"  =  "1"  /* Excision of Left Hip Bursa and Ligament, Open Approach */
"0MBM3ZZ"  =  "1"  /* Excision of Left Hip Bursa and Ligament, Perc Approach */
"0MBM4ZZ"  =  "1"  /* Excision of Left Hip Bursa and Ligament, Perc Endo Approach */
"0MBN0ZZ"  =  "1"  /* Excision of Right Knee Bursa and Ligament, Open Approach */
"0MBN3ZZ"  =  "1"  /* Excision of Right Knee Bursa and Ligament, Perc Approach */
"0MBN4ZZ"  =  "1"  /* Excision of R Knee Bursa/Lig, Perc Endo Approach */
"0MBP0ZZ"  =  "1"  /* Excision of Left Knee Bursa and Ligament, Open Approach */
"0MBP3ZZ"  =  "1"  /* Excision of Left Knee Bursa and Ligament, Perc Approach */
"0MBP4ZZ"  =  "1"  /* Excision of Left Knee Bursa and Ligament, Perc Endo Approach */
"0MBQ0ZZ"  =  "1"  /* Excision of Right Ankle Bursa and Ligament, Open Approach */
"0MBQ3ZZ"  =  "1"  /* Excision of Right Ankle Bursa and Ligament, Perc Approach */
"0MBQ4ZZ"  =  "1"  /* Excision of R Ankle Bursa/Lig, Perc Endo Approach */
"0MBR0ZZ"  =  "1"  /* Excision of Left Ankle Bursa and Ligament, Open Approach */
"0MBR3ZZ"  =  "1"  /* Excision of Left Ankle Bursa and Ligament, Perc Approach */
"0MBR4ZZ"  =  "1"  /* Excision of L Ankle Bursa/Lig, Perc Endo Approach */
"0MBS0ZZ"  =  "1"  /* Excision of Right Foot Bursa and Ligament, Open Approach */
"0MBS3ZZ"  =  "1"  /* Excision of Right Foot Bursa and Ligament, Perc Approach */
"0MBS4ZZ"  =  "1"  /* Excision of R Foot Bursa/Lig, Perc Endo Approach */
"0MBT0ZZ"  =  "1"  /* Excision of Left Foot Bursa and Ligament, Open Approach */
"0MBT3ZZ"  =  "1"  /* Excision of Left Foot Bursa and Ligament, Perc Approach */
"0MBT4ZZ"  =  "1"  /* Excision of Left Foot Bursa and Ligament, Perc Endo Approach */
"0MBV0ZX"  =  "1"  /* Excision of R Low Extrem Bursa/Lig, Open Approach, Diagn */
"0MBV0ZZ"  =  "1"  /* Excision of R Low Extrem Bursa/Lig, Open Approach */
"0MBV3ZX"  =  "1"  /* Excision of R Low Extrem Bursa/Lig, Perc Approach, Diagn */
"0MBV3ZZ"  =  "1"  /* Excision of R Low Extrem Bursa/Lig, Perc Approach */
"0MBV4ZX"  =  "1"  /* Excise of R Low Extrem Bursa/Lig, Perc Endo Approach, Diagn */
"0MBV4ZZ"  =  "1"  /* Excision of R Low Extrem Bursa/Lig, Perc Endo Approach */
"0MBW0ZX"  =  "1"  /* Excision of L Low Extrem Bursa/Lig, Open Approach, Diagn */
"0MBW0ZZ"  =  "1"  /* Excision of L Low Extrem Bursa/Lig, Open Approach */
"0MBW3ZX"  =  "1"  /* Excision of L Low Extrem Bursa/Lig, Perc Approach, Diagn */
"0MBW3ZZ"  =  "1"  /* Excision of L Low Extrem Bursa/Lig, Perc Approach */
"0MBW4ZX"  =  "1"  /* Excise of L Low Extrem Bursa/Lig, Perc Endo Approach, Diagn */
"0MBW4ZZ"  =  "1"  /* Excision of L Low Extrem Bursa/Lig, Perc Endo Approach */
"0MC00ZZ"  =  "1"  /* Extirpate matter from Head & Neck Bursa/Lig, Open */
"0MC03ZZ"  =  "1"  /* Extirpate matter from Head & Neck Bursa/Lig, Perc */
"0MC04ZZ"  =  "1"  /* Extirpate matter from Head & Neck Bursa/Lig, Perc Endo */
"0MC10ZZ"  =  "1"  /* Extirpate of Matter from R Shoulder Bursa/Lig, Open Approach */
"0MC13ZZ"  =  "1"  /* Extirpate of Matter from R Shoulder Bursa/Lig, Perc Approach */
"0MC14ZZ"  =  "1"  /* Extirpate matter from R Shoulder Bursa/Lig, Perc Endo */
"0MC20ZZ"  =  "1"  /* Extirpate of Matter from L Shoulder Bursa/Lig, Open Approach */
"0MC23ZZ"  =  "1"  /* Extirpate of Matter from L Shoulder Bursa/Lig, Perc Approach */
"0MC24ZZ"  =  "1"  /* Extirpate matter from L Shoulder Bursa/Lig, Perc Endo */
"0MC30ZZ"  =  "1"  /* Extirpation of Matter from R Elbow Bursa/Lig, Open Approach */
"0MC33ZZ"  =  "1"  /* Extirpation of Matter from R Elbow Bursa/Lig, Perc Approach */
"0MC34ZZ"  =  "1"  /* Extirpate matter from R Elbow Bursa/Lig, Perc Endo */
"0MC40ZZ"  =  "1"  /* Extirpation of Matter from L Elbow Bursa/Lig, Open Approach */
"0MC43ZZ"  =  "1"  /* Extirpation of Matter from L Elbow Bursa/Lig, Perc Approach */
"0MC44ZZ"  =  "1"  /* Extirpate matter from L Elbow Bursa/Lig, Perc Endo */
"0MC50ZZ"  =  "1"  /* Extirpation of Matter from R Wrist Bursa/Lig, Open Approach */
"0MC53ZZ"  =  "1"  /* Extirpation of Matter from R Wrist Bursa/Lig, Perc Approach */
"0MC54ZZ"  =  "1"  /* Extirpate matter from R Wrist Bursa/Lig, Perc Endo */
"0MC60ZZ"  =  "1"  /* Extirpation of Matter from L Wrist Bursa/Lig, Open Approach */
"0MC63ZZ"  =  "1"  /* Extirpation of Matter from L Wrist Bursa/Lig, Perc Approach */
"0MC64ZZ"  =  "1"  /* Extirpate matter from L Wrist Bursa/Lig, Perc Endo */
"0MC70ZZ"  =  "1"  /* Extirpation of Matter from R Hand Bursa/Lig, Open Approach */
"0MC73ZZ"  =  "1"  /* Extirpation of Matter from R Hand Bursa/Lig, Perc Approach */
"0MC74ZZ"  =  "1"  /* Extirpate matter from R Hand Bursa/Lig, Perc Endo */
"0MC80ZZ"  =  "1"  /* Extirpation of Matter from L Hand Bursa/Lig, Open Approach */
"0MC83ZZ"  =  "1"  /* Extirpation of Matter from L Hand Bursa/Lig, Perc Approach */
"0MC84ZZ"  =  "1"  /* Extirpate matter from L Hand Bursa/Lig, Perc Endo */
"0MC90ZZ"  =  "1"  /* Extirpate matter from R Up Extrem Bursa/Lig, Open */
"0MC93ZZ"  =  "1"  /* Extirpate matter from R Up Extrem Bursa/Lig, Perc */
"0MC94ZZ"  =  "1"  /* Extirpate matter from R Up Extrem Bursa/Lig, Perc Endo */
"0MCB0ZZ"  =  "1"  /* Extirpate matter from L Up Extrem Bursa/Lig, Open */
"0MCB3ZZ"  =  "1"  /* Extirpate matter from L Up Extrem Bursa/Lig, Perc */
"0MCB4ZZ"  =  "1"  /* Extirpate matter from L Up Extrem Bursa/Lig, Perc Endo */
"0MCC0ZZ"  =  "1"  /* Extirpation of Matter from R Trunk Bursa/Lig, Open Approach */
"0MCC3ZZ"  =  "1"  /* Extirpation of Matter from R Trunk Bursa/Lig, Perc Approach */
"0MCC4ZZ"  =  "1"  /* Extirpate matter from R Trunk Bursa/Lig, Perc Endo */
"0MCD0ZZ"  =  "1"  /* Extirpation of Matter from L Trunk Bursa/Lig, Open Approach */
"0MCD3ZZ"  =  "1"  /* Extirpation of Matter from L Trunk Bursa/Lig, Perc Approach */
"0MCD4ZZ"  =  "1"  /* Extirpate matter from L Trunk Bursa/Lig, Perc Endo */
"0MCF0ZZ"  =  "1"  /* Extirpation of Matter from R Thorax Bursa/Lig, Open Approach */
"0MCF3ZZ"  =  "1"  /* Extirpation of Matter from R Thorax Bursa/Lig, Perc Approach */
"0MCF4ZZ"  =  "1"  /* Extirpate matter from R Thorax Bursa/Lig, Perc Endo */
"0MCG0ZZ"  =  "1"  /* Extirpation of Matter from L Thorax Bursa/Lig, Open Approach */
"0MCG3ZZ"  =  "1"  /* Extirpation of Matter from L Thorax Bursa/Lig, Perc Approach */
"0MCG4ZZ"  =  "1"  /* Extirpate matter from L Thorax Bursa/Lig, Perc Endo */
"0MCH0ZZ"  =  "1"  /* Extirpation of Matter from R Abd Bursa/Lig, Open Approach */
"0MCH3ZZ"  =  "1"  /* Extirpation of Matter from R Abd Bursa/Lig, Perc Approach */
"0MCH4ZZ"  =  "1"  /* Extirpate of Matter from R Abd Bursa/Lig, Perc Endo Approach */
"0MCJ0ZZ"  =  "1"  /* Extirpation of Matter from L Abd Bursa/Lig, Open Approach */
"0MCJ3ZZ"  =  "1"  /* Extirpation of Matter from L Abd Bursa/Lig, Perc Approach */
"0MCJ4ZZ"  =  "1"  /* Extirpate of Matter from L Abd Bursa/Lig, Perc Endo Approach */
"0MCK0ZZ"  =  "1"  /* Extirpation of Matter from Perineum Bursa/Lig, Open Approach */
"0MCK3ZZ"  =  "1"  /* Extirpation of Matter from Perineum Bursa/Lig, Perc Approach */
"0MCK4ZZ"  =  "1"  /* Extirpate matter from Perineum Bursa/Lig, Perc Endo */
"0MCL0ZZ"  =  "1"  /* Extirpation of Matter from R Hip Bursa/Lig, Open Approach */
"0MCL3ZZ"  =  "1"  /* Extirpation of Matter from R Hip Bursa/Lig, Perc Approach */
"0MCL4ZZ"  =  "1"  /* Extirpate of Matter from R Hip Bursa/Lig, Perc Endo Approach */
"0MCM0ZZ"  =  "1"  /* Extirpation of Matter from L Hip Bursa/Lig, Open Approach */
"0MCM3ZZ"  =  "1"  /* Extirpation of Matter from L Hip Bursa/Lig, Perc Approach */
"0MCM4ZZ"  =  "1"  /* Extirpate of Matter from L Hip Bursa/Lig, Perc Endo Approach */
"0MCN0ZZ"  =  "1"  /* Extirpation of Matter from R Knee Bursa/Lig, Open Approach */
"0MCN3ZZ"  =  "1"  /* Extirpation of Matter from R Knee Bursa/Lig, Perc Approach */
"0MCN4ZZ"  =  "1"  /* Extirpate matter from R Knee Bursa/Lig, Perc Endo */
"0MCP0ZZ"  =  "1"  /* Extirpation of Matter from L Knee Bursa/Lig, Open Approach */
"0MCP3ZZ"  =  "1"  /* Extirpation of Matter from L Knee Bursa/Lig, Perc Approach */
"0MCP4ZZ"  =  "1"  /* Extirpate matter from L Knee Bursa/Lig, Perc Endo */
"0MCQ0ZZ"  =  "1"  /* Extirpation of Matter from R Ankle Bursa/Lig, Open Approach */
"0MCQ3ZZ"  =  "1"  /* Extirpation of Matter from R Ankle Bursa/Lig, Perc Approach */
"0MCQ4ZZ"  =  "1"  /* Extirpate matter from R Ankle Bursa/Lig, Perc Endo */
"0MCR0ZZ"  =  "1"  /* Extirpation of Matter from L Ankle Bursa/Lig, Open Approach */
"0MCR3ZZ"  =  "1"  /* Extirpation of Matter from L Ankle Bursa/Lig, Perc Approach */
"0MCR4ZZ"  =  "1"  /* Extirpate matter from L Ankle Bursa/Lig, Perc Endo */
"0MCS0ZZ"  =  "1"  /* Extirpation of Matter from R Foot Bursa/Lig, Open Approach */
"0MCS3ZZ"  =  "1"  /* Extirpation of Matter from R Foot Bursa/Lig, Perc Approach */
"0MCS4ZZ"  =  "1"  /* Extirpate matter from R Foot Bursa/Lig, Perc Endo */
"0MCT0ZZ"  =  "1"  /* Extirpation of Matter from L Foot Bursa/Lig, Open Approach */
"0MCT3ZZ"  =  "1"  /* Extirpation of Matter from L Foot Bursa/Lig, Perc Approach */
"0MCT4ZZ"  =  "1"  /* Extirpate matter from L Foot Bursa/Lig, Perc Endo */
"0MCV0ZZ"  =  "1"  /* Extirpate matter from R Low Extrem Bursa/Lig, Open */
"0MCV3ZZ"  =  "1"  /* Extirpate matter from R Low Extrem Bursa/Lig, Perc */
"0MCV4ZZ"  =  "1"  /* Extirpate matter from R Low Extrem Bursa/Lig, Perc Endo */
"0MCW0ZZ"  =  "1"  /* Extirpate matter from L Low Extrem Bursa/Lig, Open */
"0MCW3ZZ"  =  "1"  /* Extirpate matter from L Low Extrem Bursa/Lig, Perc */
"0MCW4ZZ"  =  "1"  /* Extirpate matter from L Low Extrem Bursa/Lig, Perc Endo */
"0MD00ZZ"  =  "1"  /* Extraction of Head & Neck Bursa/Lig, Open Approach */
"0MD03ZZ"  =  "1"  /* Extraction of Head & Neck Bursa/Lig, Perc Approach */
"0MD04ZZ"  =  "1"  /* Extraction of Head & Neck Bursa/Lig, Perc Endo Approach */
"0MD10ZZ"  =  "1"  /* Extraction of R Shoulder Bursa/Lig, Open Approach */
"0MD13ZZ"  =  "1"  /* Extraction of R Shoulder Bursa/Lig, Perc Approach */
"0MD14ZZ"  =  "1"  /* Extraction of R Shoulder Bursa/Lig, Perc Endo Approach */
"0MD20ZZ"  =  "1"  /* Extraction of L Shoulder Bursa/Lig, Open Approach */
"0MD23ZZ"  =  "1"  /* Extraction of L Shoulder Bursa/Lig, Perc Approach */
"0MD24ZZ"  =  "1"  /* Extraction of L Shoulder Bursa/Lig, Perc Endo Approach */
"0MD30ZZ"  =  "1"  /* Extraction of Right Elbow Bursa and Ligament, Open Approach */
"0MD33ZZ"  =  "1"  /* Extraction of Right Elbow Bursa and Ligament, Perc Approach */
"0MD34ZZ"  =  "1"  /* Extraction of R Elbow Bursa/Lig, Perc Endo Approach */
"0MD40ZZ"  =  "1"  /* Extraction of Left Elbow Bursa and Ligament, Open Approach */
"0MD43ZZ"  =  "1"  /* Extraction of Left Elbow Bursa and Ligament, Perc Approach */
"0MD44ZZ"  =  "1"  /* Extraction of L Elbow Bursa/Lig, Perc Endo Approach */
"0MD50ZZ"  =  "1"  /* Extraction of Right Wrist Bursa and Ligament, Open Approach */
"0MD53ZZ"  =  "1"  /* Extraction of Right Wrist Bursa and Ligament, Perc Approach */
"0MD54ZZ"  =  "1"  /* Extraction of R Wrist Bursa/Lig, Perc Endo Approach */
"0MD60ZZ"  =  "1"  /* Extraction of Left Wrist Bursa and Ligament, Open Approach */
"0MD63ZZ"  =  "1"  /* Extraction of Left Wrist Bursa and Ligament, Perc Approach */
"0MD64ZZ"  =  "1"  /* Extraction of L Wrist Bursa/Lig, Perc Endo Approach */
"0MD70ZZ"  =  "1"  /* Extraction of Right Hand Bursa and Ligament, Open Approach */
"0MD73ZZ"  =  "1"  /* Extraction of Right Hand Bursa and Ligament, Perc Approach */
"0MD74ZZ"  =  "1"  /* Extraction of R Hand Bursa/Lig, Perc Endo Approach */
"0MD80ZZ"  =  "1"  /* Extraction of Left Hand Bursa and Ligament, Open Approach */
"0MD83ZZ"  =  "1"  /* Extraction of Left Hand Bursa and Ligament, Perc Approach */
"0MD84ZZ"  =  "1"  /* Extraction of L Hand Bursa/Lig, Perc Endo Approach */
"0MD90ZZ"  =  "1"  /* Extraction of R Up Extrem Bursa/Lig, Open Approach */
"0MD93ZZ"  =  "1"  /* Extraction of R Up Extrem Bursa/Lig, Perc Approach */
"0MD94ZZ"  =  "1"  /* Extraction of R Up Extrem Bursa/Lig, Perc Endo Approach */
"0MDB0ZZ"  =  "1"  /* Extraction of L Up Extrem Bursa/Lig, Open Approach */
"0MDB3ZZ"  =  "1"  /* Extraction of L Up Extrem Bursa/Lig, Perc Approach */
"0MDB4ZZ"  =  "1"  /* Extraction of L Up Extrem Bursa/Lig, Perc Endo Approach */
"0MDC0ZZ"  =  "1"  /* Extraction of Right Trunk Bursa and Ligament, Open Approach */
"0MDC3ZZ"  =  "1"  /* Extraction of Right Trunk Bursa and Ligament, Perc Approach */
"0MDC4ZZ"  =  "1"  /* Extraction of R Trunk Bursa/Lig, Perc Endo Approach */
"0MDD0ZZ"  =  "1"  /* Extraction of Left Trunk Bursa and Ligament, Open Approach */
"0MDD3ZZ"  =  "1"  /* Extraction of Left Trunk Bursa and Ligament, Perc Approach */
"0MDD4ZZ"  =  "1"  /* Extraction of L Trunk Bursa/Lig, Perc Endo Approach */
"0MDF0ZZ"  =  "1"  /* Extraction of Right Thorax Bursa and Ligament, Open Approach */
"0MDF3ZZ"  =  "1"  /* Extraction of Right Thorax Bursa and Ligament, Perc Approach */
"0MDF4ZZ"  =  "1"  /* Extraction of R Thorax Bursa/Lig, Perc Endo Approach */
"0MDG0ZZ"  =  "1"  /* Extraction of Left Thorax Bursa and Ligament, Open Approach */
"0MDG3ZZ"  =  "1"  /* Extraction of Left Thorax Bursa and Ligament, Perc Approach */
"0MDG4ZZ"  =  "1"  /* Extraction of L Thorax Bursa/Lig, Perc Endo Approach */
"0MDH0ZZ"  =  "1"  /* Extraction of R Abd Bursa/Lig, Open Approach */
"0MDH3ZZ"  =  "1"  /* Extraction of R Abd Bursa/Lig, Perc Approach */
"0MDH4ZZ"  =  "1"  /* Extraction of R Abd Bursa/Lig, Perc Endo Approach */
"0MDJ0ZZ"  =  "1"  /* Extraction of Left Abdomen Bursa and Ligament, Open Approach */
"0MDJ3ZZ"  =  "1"  /* Extraction of Left Abdomen Bursa and Ligament, Perc Approach */
"0MDJ4ZZ"  =  "1"  /* Extraction of L Abd Bursa/Lig, Perc Endo Approach */
"0MDK0ZZ"  =  "1"  /* Extraction of Perineum Bursa and Ligament, Open Approach */
"0MDK3ZZ"  =  "1"  /* Extraction of Perineum Bursa and Ligament, Perc Approach */
"0MDK4ZZ"  =  "1"  /* Extraction of Perineum Bursa/Lig, Perc Endo Approach */
"0MDL0ZZ"  =  "1"  /* Extraction of Right Hip Bursa and Ligament, Open Approach */
"0MDL3ZZ"  =  "1"  /* Extraction of Right Hip Bursa and Ligament, Perc Approach */
"0MDL4ZZ"  =  "1"  /* Extraction of R Hip Bursa/Lig, Perc Endo Approach */
"0MDM0ZZ"  =  "1"  /* Extraction of Left Hip Bursa and Ligament, Open Approach */
"0MDM3ZZ"  =  "1"  /* Extraction of Left Hip Bursa and Ligament, Perc Approach */
"0MDM4ZZ"  =  "1"  /* Extraction of L Hip Bursa/Lig, Perc Endo Approach */
"0MDN0ZZ"  =  "1"  /* Extraction of Right Knee Bursa and Ligament, Open Approach */
"0MDN3ZZ"  =  "1"  /* Extraction of Right Knee Bursa and Ligament, Perc Approach */
"0MDN4ZZ"  =  "1"  /* Extraction of R Knee Bursa/Lig, Perc Endo Approach */
"0MDP0ZZ"  =  "1"  /* Extraction of Left Knee Bursa and Ligament, Open Approach */
"0MDP3ZZ"  =  "1"  /* Extraction of Left Knee Bursa and Ligament, Perc Approach */
"0MDP4ZZ"  =  "1"  /* Extraction of L Knee Bursa/Lig, Perc Endo Approach */
"0MDQ0ZZ"  =  "1"  /* Extraction of Right Ankle Bursa and Ligament, Open Approach */
"0MDQ3ZZ"  =  "1"  /* Extraction of Right Ankle Bursa and Ligament, Perc Approach */
"0MDQ4ZZ"  =  "1"  /* Extraction of R Ankle Bursa/Lig, Perc Endo Approach */
"0MDR0ZZ"  =  "1"  /* Extraction of Left Ankle Bursa and Ligament, Open Approach */
"0MDR3ZZ"  =  "1"  /* Extraction of Left Ankle Bursa and Ligament, Perc Approach */
"0MDR4ZZ"  =  "1"  /* Extraction of L Ankle Bursa/Lig, Perc Endo Approach */
"0MDS0ZZ"  =  "1"  /* Extraction of Right Foot Bursa and Ligament, Open Approach */
"0MDS3ZZ"  =  "1"  /* Extraction of Right Foot Bursa and Ligament, Perc Approach */
"0MDS4ZZ"  =  "1"  /* Extraction of R Foot Bursa/Lig, Perc Endo Approach */
"0MDT0ZZ"  =  "1"  /* Extraction of Left Foot Bursa and Ligament, Open Approach */
"0MDT3ZZ"  =  "1"  /* Extraction of Left Foot Bursa and Ligament, Perc Approach */
"0MDT4ZZ"  =  "1"  /* Extraction of L Foot Bursa/Lig, Perc Endo Approach */
"0MDV0ZZ"  =  "1"  /* Extraction of R Low Extrem Bursa/Lig, Open Approach */
"0MDV3ZZ"  =  "1"  /* Extraction of R Low Extrem Bursa/Lig, Perc Approach */
"0MDV4ZZ"  =  "1"  /* Extraction of R Low Extrem Bursa/Lig, Perc Endo Approach */
"0MDW0ZZ"  =  "1"  /* Extraction of L Low Extrem Bursa/Lig, Open Approach */
"0MDW3ZZ"  =  "1"  /* Extraction of L Low Extrem Bursa/Lig, Perc Approach */
"0MDW4ZZ"  =  "1"  /* Extraction of L Low Extrem Bursa/Lig, Perc Endo Approach */
"0MJX0ZZ"  =  "1"  /* Inspection of Upper Bursa and Ligament, Open Approach */
"0MJX3ZZ"  =  "1"  /* Inspection of Upper Bursa and Ligament, Perc Approach */
"0MJX4ZZ"  =  "1"  /* Inspection of Upper Bursa and Ligament, Perc Endo Approach */
"0MJY0ZZ"  =  "1"  /* Inspection of Lower Bursa and Ligament, Open Approach */
"0MJY3ZZ"  =  "1"  /* Inspection of Lower Bursa and Ligament, Perc Approach */
"0MJY4ZZ"  =  "1"  /* Inspection of Lower Bursa and Ligament, Perc Endo Approach */
"0MM00ZZ"  =  "1"  /* Reattachment of Head & Neck Bursa/Lig, Open Approach */
"0MM04ZZ"  =  "1"  /* Reattachment of Head & Neck Bursa/Lig, Perc Endo Approach */
"0MM10ZZ"  =  "1"  /* Reattachment of R Shoulder Bursa/Lig, Open Approach */
"0MM14ZZ"  =  "1"  /* Reattachment of R Shoulder Bursa/Lig, Perc Endo Approach */
"0MM20ZZ"  =  "1"  /* Reattachment of L Shoulder Bursa/Lig, Open Approach */
"0MM24ZZ"  =  "1"  /* Reattachment of L Shoulder Bursa/Lig, Perc Endo Approach */
"0MM30ZZ"  =  "1"  /* Reattachment of R Elbow Bursa/Lig, Open Approach */
"0MM34ZZ"  =  "1"  /* Reattachment of R Elbow Bursa/Lig, Perc Endo Approach */
"0MM40ZZ"  =  "1"  /* Reattachment of Left Elbow Bursa and Ligament, Open Approach */
"0MM44ZZ"  =  "1"  /* Reattachment of L Elbow Bursa/Lig, Perc Endo Approach */
"0MM50ZZ"  =  "1"  /* Reattachment of R Wrist Bursa/Lig, Open Approach */
"0MM54ZZ"  =  "1"  /* Reattachment of R Wrist Bursa/Lig, Perc Endo Approach */
"0MM60ZZ"  =  "1"  /* Reattachment of Left Wrist Bursa and Ligament, Open Approach */
"0MM64ZZ"  =  "1"  /* Reattachment of L Wrist Bursa/Lig, Perc Endo Approach */
"0MM70ZZ"  =  "1"  /* Reattachment of Right Hand Bursa and Ligament, Open Approach */
"0MM74ZZ"  =  "1"  /* Reattachment of R Hand Bursa/Lig, Perc Endo Approach */
"0MM80ZZ"  =  "1"  /* Reattachment of Left Hand Bursa and Ligament, Open Approach */
"0MM84ZZ"  =  "1"  /* Reattachment of L Hand Bursa/Lig, Perc Endo Approach */
"0MM90ZZ"  =  "1"  /* Reattachment of R Up Extrem Bursa/Lig, Open Approach */
"0MM94ZZ"  =  "1"  /* Reattachment of R Up Extrem Bursa/Lig, Perc Endo Approach */
"0MMB0ZZ"  =  "1"  /* Reattachment of L Up Extrem Bursa/Lig, Open Approach */
"0MMB4ZZ"  =  "1"  /* Reattachment of L Up Extrem Bursa/Lig, Perc Endo Approach */
"0MMC0ZZ"  =  "1"  /* Reattachment of R Trunk Bursa/Lig, Open Approach */
"0MMC4ZZ"  =  "1"  /* Reattachment of R Trunk Bursa/Lig, Perc Endo Approach */
"0MMD0ZZ"  =  "1"  /* Reattachment of Left Trunk Bursa and Ligament, Open Approach */
"0MMD4ZZ"  =  "1"  /* Reattachment of L Trunk Bursa/Lig, Perc Endo Approach */
"0MMF0ZZ"  =  "1"  /* Reattachment of R Thorax Bursa/Lig, Open Approach */
"0MMF4ZZ"  =  "1"  /* Reattachment of R Thorax Bursa/Lig, Perc Endo Approach */
"0MMG0ZZ"  =  "1"  /* Reattachment of L Thorax Bursa/Lig, Open Approach */
"0MMG4ZZ"  =  "1"  /* Reattachment of L Thorax Bursa/Lig, Perc Endo Approach */
"0MMH0ZZ"  =  "1"  /* Reattachment of R Abd Bursa/Lig, Open Approach */
"0MMH4ZZ"  =  "1"  /* Reattachment of R Abd Bursa/Lig, Perc Endo Approach */
"0MMJ0ZZ"  =  "1"  /* Reattachment of L Abd Bursa/Lig, Open Approach */
"0MMJ4ZZ"  =  "1"  /* Reattachment of L Abd Bursa/Lig, Perc Endo Approach */
"0MMK0ZZ"  =  "1"  /* Reattachment of Perineum Bursa and Ligament, Open Approach */
"0MMK4ZZ"  =  "1"  /* Reattachment of Perineum Bursa/Lig, Perc Endo Approach */
"0MML0ZZ"  =  "1"  /* Reattachment of Right Hip Bursa and Ligament, Open Approach */
"0MML4ZZ"  =  "1"  /* Reattachment of R Hip Bursa/Lig, Perc Endo Approach */
"0MMM0ZZ"  =  "1"  /* Reattachment of Left Hip Bursa and Ligament, Open Approach */
"0MMM4ZZ"  =  "1"  /* Reattachment of L Hip Bursa/Lig, Perc Endo Approach */
"0MMN0ZZ"  =  "1"  /* Reattachment of Right Knee Bursa and Ligament, Open Approach */
"0MMN4ZZ"  =  "1"  /* Reattachment of R Knee Bursa/Lig, Perc Endo Approach */
"0MMP0ZZ"  =  "1"  /* Reattachment of Left Knee Bursa and Ligament, Open Approach */
"0MMP4ZZ"  =  "1"  /* Reattachment of L Knee Bursa/Lig, Perc Endo Approach */
"0MMQ0ZZ"  =  "1"  /* Reattachment of R Ankle Bursa/Lig, Open Approach */
"0MMQ4ZZ"  =  "1"  /* Reattachment of R Ankle Bursa/Lig, Perc Endo Approach */
"0MMR0ZZ"  =  "1"  /* Reattachment of Left Ankle Bursa and Ligament, Open Approach */
"0MMR4ZZ"  =  "1"  /* Reattachment of L Ankle Bursa/Lig, Perc Endo Approach */
"0MMS0ZZ"  =  "1"  /* Reattachment of Right Foot Bursa and Ligament, Open Approach */
"0MMS4ZZ"  =  "1"  /* Reattachment of R Foot Bursa/Lig, Perc Endo Approach */
"0MMT0ZZ"  =  "1"  /* Reattachment of Left Foot Bursa and Ligament, Open Approach */
"0MMT4ZZ"  =  "1"  /* Reattachment of L Foot Bursa/Lig, Perc Endo Approach */
"0MMV0ZZ"  =  "1"  /* Reattachment of R Low Extrem Bursa/Lig, Open Approach */
"0MMV4ZZ"  =  "1"  /* Reattachment of R Low Extrem Bursa/Lig, Perc Endo Approach */
"0MMW0ZZ"  =  "1"  /* Reattachment of L Low Extrem Bursa/Lig, Open Approach */
"0MMW4ZZ"  =  "1"  /* Reattachment of L Low Extrem Bursa/Lig, Perc Endo Approach */
"0MN00ZZ"  =  "1"  /* Release Head and Neck Bursa and Ligament, Open Approach */
"0MN03ZZ"  =  "1"  /* Release Head and Neck Bursa and Ligament, Perc Approach */
"0MN04ZZ"  =  "1"  /* Release Head and Neck Bursa and Ligament, Perc Endo Approach */
"0MN0XZZ"  =  "1"  /* Release Head and Neck Bursa and Ligament, External Approach */
"0MN10ZZ"  =  "1"  /* Release Right Shoulder Bursa and Ligament, Open Approach */
"0MN13ZZ"  =  "1"  /* Release Right Shoulder Bursa and Ligament, Perc Approach */
"0MN14ZZ"  =  "1"  /* Release R Shoulder Bursa/Lig, Perc Endo Approach */
"0MN1XZZ"  =  "1"  /* Release Right Shoulder Bursa and Ligament, External Approach */
"0MN20ZZ"  =  "1"  /* Release Left Shoulder Bursa and Ligament, Open Approach */
"0MN23ZZ"  =  "1"  /* Release Left Shoulder Bursa and Ligament, Perc Approach */
"0MN24ZZ"  =  "1"  /* Release Left Shoulder Bursa and Ligament, Perc Endo Approach */
"0MN2XZZ"  =  "1"  /* Release Left Shoulder Bursa and Ligament, External Approach */
"0MN30ZZ"  =  "1"  /* Release Right Elbow Bursa and Ligament, Open Approach */
"0MN33ZZ"  =  "1"  /* Release Right Elbow Bursa and Ligament, Perc Approach */
"0MN34ZZ"  =  "1"  /* Release Right Elbow Bursa and Ligament, Perc Endo Approach */
"0MN3XZZ"  =  "1"  /* Release Right Elbow Bursa and Ligament, External Approach */
"0MN40ZZ"  =  "1"  /* Release Left Elbow Bursa and Ligament, Open Approach */
"0MN43ZZ"  =  "1"  /* Release Left Elbow Bursa and Ligament, Percutaneous Approach */
"0MN44ZZ"  =  "1"  /* Release Left Elbow Bursa and Ligament, Perc Endo Approach */
"0MN4XZZ"  =  "1"  /* Release Left Elbow Bursa and Ligament, External Approach */
"0MN50ZZ"  =  "1"  /* Release Right Wrist Bursa and Ligament, Open Approach */
"0MN53ZZ"  =  "1"  /* Release Right Wrist Bursa and Ligament, Perc Approach */
"0MN54ZZ"  =  "1"  /* Release Right Wrist Bursa and Ligament, Perc Endo Approach */
"0MN5XZZ"  =  "1"  /* Release Right Wrist Bursa and Ligament, External Approach */
"0MN60ZZ"  =  "1"  /* Release Left Wrist Bursa and Ligament, Open Approach */
"0MN63ZZ"  =  "1"  /* Release Left Wrist Bursa and Ligament, Percutaneous Approach */
"0MN64ZZ"  =  "1"  /* Release Left Wrist Bursa and Ligament, Perc Endo Approach */
"0MN6XZZ"  =  "1"  /* Release Left Wrist Bursa and Ligament, External Approach */
"0MN70ZZ"  =  "1"  /* Release Right Hand Bursa and Ligament, Open Approach */
"0MN73ZZ"  =  "1"  /* Release Right Hand Bursa and Ligament, Percutaneous Approach */
"0MN74ZZ"  =  "1"  /* Release Right Hand Bursa and Ligament, Perc Endo Approach */
"0MN7XZZ"  =  "1"  /* Release Right Hand Bursa and Ligament, External Approach */
"0MN80ZZ"  =  "1"  /* Release Left Hand Bursa and Ligament, Open Approach */
"0MN83ZZ"  =  "1"  /* Release Left Hand Bursa and Ligament, Percutaneous Approach */
"0MN84ZZ"  =  "1"  /* Release Left Hand Bursa and Ligament, Perc Endo Approach */
"0MN8XZZ"  =  "1"  /* Release Left Hand Bursa and Ligament, External Approach */
"0MN90ZZ"  =  "1"  /* Release R Up Extrem Bursa/Lig, Open Approach */
"0MN93ZZ"  =  "1"  /* Release R Up Extrem Bursa/Lig, Perc Approach */
"0MN94ZZ"  =  "1"  /* Release R Up Extrem Bursa/Lig, Perc Endo Approach */
"0MN9XZZ"  =  "1"  /* Release R Up Extrem Bursa/Lig, Extern Approach */
"0MNB0ZZ"  =  "1"  /* Release L Up Extrem Bursa/Lig, Open Approach */
"0MNB3ZZ"  =  "1"  /* Release L Up Extrem Bursa/Lig, Perc Approach */
"0MNB4ZZ"  =  "1"  /* Release L Up Extrem Bursa/Lig, Perc Endo Approach */
"0MNBXZZ"  =  "1"  /* Release L Up Extrem Bursa/Lig, Extern Approach */
"0MNC0ZZ"  =  "1"  /* Release Right Trunk Bursa and Ligament, Open Approach */
"0MNC3ZZ"  =  "1"  /* Release Right Trunk Bursa and Ligament, Perc Approach */
"0MNC4ZZ"  =  "1"  /* Release Right Trunk Bursa and Ligament, Perc Endo Approach */
"0MNCXZZ"  =  "1"  /* Release Right Trunk Bursa and Ligament, External Approach */
"0MND0ZZ"  =  "1"  /* Release Left Trunk Bursa and Ligament, Open Approach */
"0MND3ZZ"  =  "1"  /* Release Left Trunk Bursa and Ligament, Percutaneous Approach */
"0MND4ZZ"  =  "1"  /* Release Left Trunk Bursa and Ligament, Perc Endo Approach */
"0MNDXZZ"  =  "1"  /* Release Left Trunk Bursa and Ligament, External Approach */
"0MNF0ZZ"  =  "1"  /* Release Right Thorax Bursa and Ligament, Open Approach */
"0MNF3ZZ"  =  "1"  /* Release Right Thorax Bursa and Ligament, Perc Approach */
"0MNF4ZZ"  =  "1"  /* Release Right Thorax Bursa and Ligament, Perc Endo Approach */
"0MNFXZZ"  =  "1"  /* Release Right Thorax Bursa and Ligament, External Approach */
"0MNG0ZZ"  =  "1"  /* Release Left Thorax Bursa and Ligament, Open Approach */
"0MNG3ZZ"  =  "1"  /* Release Left Thorax Bursa and Ligament, Perc Approach */
"0MNG4ZZ"  =  "1"  /* Release Left Thorax Bursa and Ligament, Perc Endo Approach */
"0MNGXZZ"  =  "1"  /* Release Left Thorax Bursa and Ligament, External Approach */
"0MNH0ZZ"  =  "1"  /* Release Right Abdomen Bursa and Ligament, Open Approach */
"0MNH3ZZ"  =  "1"  /* Release Right Abdomen Bursa and Ligament, Perc Approach */
"0MNH4ZZ"  =  "1"  /* Release Right Abdomen Bursa and Ligament, Perc Endo Approach */
"0MNHXZZ"  =  "1"  /* Release Right Abdomen Bursa and Ligament, External Approach */
"0MNJ0ZZ"  =  "1"  /* Release Left Abdomen Bursa and Ligament, Open Approach */
"0MNJ3ZZ"  =  "1"  /* Release Left Abdomen Bursa and Ligament, Perc Approach */
"0MNJ4ZZ"  =  "1"  /* Release Left Abdomen Bursa and Ligament, Perc Endo Approach */
"0MNJXZZ"  =  "1"  /* Release Left Abdomen Bursa and Ligament, External Approach */
"0MNK0ZZ"  =  "1"  /* Release Perineum Bursa and Ligament, Open Approach */
"0MNK3ZZ"  =  "1"  /* Release Perineum Bursa and Ligament, Percutaneous Approach */
"0MNK4ZZ"  =  "1"  /* Release Perineum Bursa and Ligament, Perc Endo Approach */
"0MNKXZZ"  =  "1"  /* Release Perineum Bursa and Ligament, External Approach */
"0MNL0ZZ"  =  "1"  /* Release Right Hip Bursa and Ligament, Open Approach */
"0MNL3ZZ"  =  "1"  /* Release Right Hip Bursa and Ligament, Percutaneous Approach */
"0MNL4ZZ"  =  "1"  /* Release Right Hip Bursa and Ligament, Perc Endo Approach */
"0MNLXZZ"  =  "1"  /* Release Right Hip Bursa and Ligament, External Approach */
"0MNM0ZZ"  =  "1"  /* Release Left Hip Bursa and Ligament, Open Approach */
"0MNM3ZZ"  =  "1"  /* Release Left Hip Bursa and Ligament, Percutaneous Approach */
"0MNM4ZZ"  =  "1"  /* Release Left Hip Bursa and Ligament, Perc Endo Approach */
"0MNMXZZ"  =  "1"  /* Release Left Hip Bursa and Ligament, External Approach */
"0MNN0ZZ"  =  "1"  /* Release Right Knee Bursa and Ligament, Open Approach */
"0MNN3ZZ"  =  "1"  /* Release Right Knee Bursa and Ligament, Percutaneous Approach */
"0MNN4ZZ"  =  "1"  /* Release Right Knee Bursa and Ligament, Perc Endo Approach */
"0MNNXZZ"  =  "1"  /* Release Right Knee Bursa and Ligament, External Approach */
"0MNP0ZZ"  =  "1"  /* Release Left Knee Bursa and Ligament, Open Approach */
"0MNP3ZZ"  =  "1"  /* Release Left Knee Bursa and Ligament, Percutaneous Approach */
"0MNP4ZZ"  =  "1"  /* Release Left Knee Bursa and Ligament, Perc Endo Approach */
"0MNPXZZ"  =  "1"  /* Release Left Knee Bursa and Ligament, External Approach */
"0MNQ0ZZ"  =  "1"  /* Release Right Ankle Bursa and Ligament, Open Approach */
"0MNQ3ZZ"  =  "1"  /* Release Right Ankle Bursa and Ligament, Perc Approach */
"0MNQ4ZZ"  =  "1"  /* Release Right Ankle Bursa and Ligament, Perc Endo Approach */
"0MNQXZZ"  =  "1"  /* Release Right Ankle Bursa and Ligament, External Approach */
"0MNR0ZZ"  =  "1"  /* Release Left Ankle Bursa and Ligament, Open Approach */
"0MNR3ZZ"  =  "1"  /* Release Left Ankle Bursa and Ligament, Percutaneous Approach */
"0MNR4ZZ"  =  "1"  /* Release Left Ankle Bursa and Ligament, Perc Endo Approach */
"0MNRXZZ"  =  "1"  /* Release Left Ankle Bursa and Ligament, External Approach */
"0MNS0ZZ"  =  "1"  /* Release Right Foot Bursa and Ligament, Open Approach */
"0MNS3ZZ"  =  "1"  /* Release Right Foot Bursa and Ligament, Percutaneous Approach */
"0MNS4ZZ"  =  "1"  /* Release Right Foot Bursa and Ligament, Perc Endo Approach */
"0MNSXZZ"  =  "1"  /* Release Right Foot Bursa and Ligament, External Approach */
"0MNT0ZZ"  =  "1"  /* Release Left Foot Bursa and Ligament, Open Approach */
"0MNT3ZZ"  =  "1"  /* Release Left Foot Bursa and Ligament, Percutaneous Approach */
"0MNT4ZZ"  =  "1"  /* Release Left Foot Bursa and Ligament, Perc Endo Approach */
"0MNTXZZ"  =  "1"  /* Release Left Foot Bursa and Ligament, External Approach */
"0MNV0ZZ"  =  "1"  /* Release R Low Extrem Bursa/Lig, Open Approach */
"0MNV3ZZ"  =  "1"  /* Release R Low Extrem Bursa/Lig, Perc Approach */
"0MNV4ZZ"  =  "1"  /* Release R Low Extrem Bursa/Lig, Perc Endo Approach */
"0MNVXZZ"  =  "1"  /* Release R Low Extrem Bursa/Lig, Extern Approach */
"0MNW0ZZ"  =  "1"  /* Release L Low Extrem Bursa/Lig, Open Approach */
"0MNW3ZZ"  =  "1"  /* Release L Low Extrem Bursa/Lig, Perc Approach */
"0MNW4ZZ"  =  "1"  /* Release L Low Extrem Bursa/Lig, Perc Endo Approach */
"0MNWXZZ"  =  "1"  /* Release L Low Extrem Bursa/Lig, Extern Approach */
"0MPX00Z"  =  "1"  /* Removal of Drainage Device from Up Bursa/Lig, Open Approach */
"0MPX07Z"  =  "1"  /* Removal of Autol Sub from Up Bursa/Lig, Open Approach */
"0MPX0JZ"  =  "1"  /* Removal of Synth Sub from Up Bursa/Lig, Open Approach */
"0MPX0KZ"  =  "1"  /* Removal of Nonaut Sub from Up Bursa/Lig, Open Approach */
"0MPX30Z"  =  "1"  /* Removal of Drainage Device from Up Bursa/Lig, Perc Approach */
"0MPX37Z"  =  "1"  /* Removal of Autol Sub from Up Bursa/Lig, Perc Approach */
"0MPX3JZ"  =  "1"  /* Removal of Synth Sub from Up Bursa/Lig, Perc Approach */
"0MPX3KZ"  =  "1"  /* Removal of Nonaut Sub from Up Bursa/Lig, Perc Approach */
"0MPX40Z"  =  "1"  /* Removal of Drain Dev from Up Bursa/Lig, Perc Endo Approach */
"0MPX47Z"  =  "1"  /* Removal of Autol Sub from Up Bursa/Lig, Perc Endo Approach */
"0MPX4JZ"  =  "1"  /* Removal of Synth Sub from Up Bursa/Lig, Perc Endo Approach */
"0MPX4KZ"  =  "1"  /* Removal of Nonaut Sub from Up Bursa/Lig, Perc Endo Approach */
"0MPY00Z"  =  "1"  /* Removal of Drainage Device from Low Bursa/Lig, Open Approach */
"0MPY07Z"  =  "1"  /* Removal of Autol Sub from Low Bursa/Lig, Open Approach */
"0MPY0JZ"  =  "1"  /* Removal of Synth Sub from Low Bursa/Lig, Open Approach */
"0MPY0KZ"  =  "1"  /* Removal of Nonaut Sub from Low Bursa/Lig, Open Approach */
"0MPY30Z"  =  "1"  /* Removal of Drainage Device from Low Bursa/Lig, Perc Approach */
"0MPY37Z"  =  "1"  /* Removal of Autol Sub from Low Bursa/Lig, Perc Approach */
"0MPY3JZ"  =  "1"  /* Removal of Synth Sub from Low Bursa/Lig, Perc Approach */
"0MPY3KZ"  =  "1"  /* Removal of Nonaut Sub from Low Bursa/Lig, Perc Approach */
"0MPY40Z"  =  "1"  /* Removal of Drain Dev from Low Bursa/Lig, Perc Endo Approach */
"0MPY47Z"  =  "1"  /* Removal of Autol Sub from Low Bursa/Lig, Perc Endo Approach */
"0MPY4JZ"  =  "1"  /* Removal of Synth Sub from Low Bursa/Lig, Perc Endo Approach */
"0MPY4KZ"  =  "1"  /* Removal of Nonaut Sub from Low Bursa/Lig, Perc Endo Approach */
"0MQ00ZZ"  =  "1"  /* Repair Head and Neck Bursa and Ligament, Open Approach */
"0MQ03ZZ"  =  "1"  /* Repair Head and Neck Bursa and Ligament, Perc Approach */
"0MQ04ZZ"  =  "1"  /* Repair Head and Neck Bursa and Ligament, Perc Endo Approach */
"0MQ10ZZ"  =  "1"  /* Repair Right Shoulder Bursa and Ligament, Open Approach */
"0MQ13ZZ"  =  "1"  /* Repair Right Shoulder Bursa and Ligament, Perc Approach */
"0MQ14ZZ"  =  "1"  /* Repair Right Shoulder Bursa and Ligament, Perc Endo Approach */
"0MQ20ZZ"  =  "1"  /* Repair Left Shoulder Bursa and Ligament, Open Approach */
"0MQ23ZZ"  =  "1"  /* Repair Left Shoulder Bursa and Ligament, Perc Approach */
"0MQ24ZZ"  =  "1"  /* Repair Left Shoulder Bursa and Ligament, Perc Endo Approach */
"0MQ30ZZ"  =  "1"  /* Repair Right Elbow Bursa and Ligament, Open Approach */
"0MQ33ZZ"  =  "1"  /* Repair Right Elbow Bursa and Ligament, Percutaneous Approach */
"0MQ34ZZ"  =  "1"  /* Repair Right Elbow Bursa and Ligament, Perc Endo Approach */
"0MQ40ZZ"  =  "1"  /* Repair Left Elbow Bursa and Ligament, Open Approach */
"0MQ43ZZ"  =  "1"  /* Repair Left Elbow Bursa and Ligament, Percutaneous Approach */
"0MQ44ZZ"  =  "1"  /* Repair Left Elbow Bursa and Ligament, Perc Endo Approach */
"0MQ50ZZ"  =  "1"  /* Repair Right Wrist Bursa and Ligament, Open Approach */
"0MQ53ZZ"  =  "1"  /* Repair Right Wrist Bursa and Ligament, Percutaneous Approach */
"0MQ54ZZ"  =  "1"  /* Repair Right Wrist Bursa and Ligament, Perc Endo Approach */
"0MQ60ZZ"  =  "1"  /* Repair Left Wrist Bursa and Ligament, Open Approach */
"0MQ63ZZ"  =  "1"  /* Repair Left Wrist Bursa and Ligament, Percutaneous Approach */
"0MQ64ZZ"  =  "1"  /* Repair Left Wrist Bursa and Ligament, Perc Endo Approach */
"0MQ70ZZ"  =  "1"  /* Repair Right Hand Bursa and Ligament, Open Approach */
"0MQ73ZZ"  =  "1"  /* Repair Right Hand Bursa and Ligament, Percutaneous Approach */
"0MQ74ZZ"  =  "1"  /* Repair Right Hand Bursa and Ligament, Perc Endo Approach */
"0MQ80ZZ"  =  "1"  /* Repair Left Hand Bursa and Ligament, Open Approach */
"0MQ83ZZ"  =  "1"  /* Repair Left Hand Bursa and Ligament, Percutaneous Approach */
"0MQ84ZZ"  =  "1"  /* Repair Left Hand Bursa and Ligament, Perc Endo Approach */
"0MQ90ZZ"  =  "1"  /* Repair R Up Extrem Bursa/Lig, Open Approach */
"0MQ93ZZ"  =  "1"  /* Repair R Up Extrem Bursa/Lig, Perc Approach */
"0MQ94ZZ"  =  "1"  /* Repair R Up Extrem Bursa/Lig, Perc Endo Approach */
"0MQB0ZZ"  =  "1"  /* Repair L Up Extrem Bursa/Lig, Open Approach */
"0MQB3ZZ"  =  "1"  /* Repair L Up Extrem Bursa/Lig, Perc Approach */
"0MQB4ZZ"  =  "1"  /* Repair L Up Extrem Bursa/Lig, Perc Endo Approach */
"0MQC0ZZ"  =  "1"  /* Repair Right Trunk Bursa and Ligament, Open Approach */
"0MQC3ZZ"  =  "1"  /* Repair Right Trunk Bursa and Ligament, Percutaneous Approach */
"0MQC4ZZ"  =  "1"  /* Repair Right Trunk Bursa and Ligament, Perc Endo Approach */
"0MQD0ZZ"  =  "1"  /* Repair Left Trunk Bursa and Ligament, Open Approach */
"0MQD3ZZ"  =  "1"  /* Repair Left Trunk Bursa and Ligament, Percutaneous Approach */
"0MQD4ZZ"  =  "1"  /* Repair Left Trunk Bursa and Ligament, Perc Endo Approach */
"0MQF0ZZ"  =  "1"  /* Repair Right Thorax Bursa and Ligament, Open Approach */
"0MQF3ZZ"  =  "1"  /* Repair Right Thorax Bursa and Ligament, Perc Approach */
"0MQF4ZZ"  =  "1"  /* Repair Right Thorax Bursa and Ligament, Perc Endo Approach */
"0MQG0ZZ"  =  "1"  /* Repair Left Thorax Bursa and Ligament, Open Approach */
"0MQG3ZZ"  =  "1"  /* Repair Left Thorax Bursa and Ligament, Percutaneous Approach */
"0MQG4ZZ"  =  "1"  /* Repair Left Thorax Bursa and Ligament, Perc Endo Approach */
"0MQH0ZZ"  =  "1"  /* Repair Right Abdomen Bursa and Ligament, Open Approach */
"0MQH3ZZ"  =  "1"  /* Repair Right Abdomen Bursa and Ligament, Perc Approach */
"0MQH4ZZ"  =  "1"  /* Repair Right Abdomen Bursa and Ligament, Perc Endo Approach */
"0MQJ0ZZ"  =  "1"  /* Repair Left Abdomen Bursa and Ligament, Open Approach */
"0MQJ3ZZ"  =  "1"  /* Repair Left Abdomen Bursa and Ligament, Perc Approach */
"0MQJ4ZZ"  =  "1"  /* Repair Left Abdomen Bursa and Ligament, Perc Endo Approach */
"0MQK0ZZ"  =  "1"  /* Repair Perineum Bursa and Ligament, Open Approach */
"0MQK3ZZ"  =  "1"  /* Repair Perineum Bursa and Ligament, Percutaneous Approach */
"0MQK4ZZ"  =  "1"  /* Repair Perineum Bursa and Ligament, Perc Endo Approach */
"0MQL0ZZ"  =  "1"  /* Repair Right Hip Bursa and Ligament, Open Approach */
"0MQL3ZZ"  =  "1"  /* Repair Right Hip Bursa and Ligament, Percutaneous Approach */
"0MQL4ZZ"  =  "1"  /* Repair Right Hip Bursa and Ligament, Perc Endo Approach */
"0MQM0ZZ"  =  "1"  /* Repair Left Hip Bursa and Ligament, Open Approach */
"0MQM3ZZ"  =  "1"  /* Repair Left Hip Bursa and Ligament, Percutaneous Approach */
"0MQM4ZZ"  =  "1"  /* Repair Left Hip Bursa and Ligament, Perc Endo Approach */
"0MQN0ZZ"  =  "1"  /* Repair Right Knee Bursa and Ligament, Open Approach */
"0MQN3ZZ"  =  "1"  /* Repair Right Knee Bursa and Ligament, Percutaneous Approach */
"0MQN4ZZ"  =  "1"  /* Repair Right Knee Bursa and Ligament, Perc Endo Approach */
"0MQP0ZZ"  =  "1"  /* Repair Left Knee Bursa and Ligament, Open Approach */
"0MQP3ZZ"  =  "1"  /* Repair Left Knee Bursa and Ligament, Percutaneous Approach */
"0MQP4ZZ"  =  "1"  /* Repair Left Knee Bursa and Ligament, Perc Endo Approach */
"0MQQ0ZZ"  =  "1"  /* Repair Right Ankle Bursa and Ligament, Open Approach */
"0MQQ3ZZ"  =  "1"  /* Repair Right Ankle Bursa and Ligament, Percutaneous Approach */
"0MQQ4ZZ"  =  "1"  /* Repair Right Ankle Bursa and Ligament, Perc Endo Approach */
"0MQR0ZZ"  =  "1"  /* Repair Left Ankle Bursa and Ligament, Open Approach */
"0MQR3ZZ"  =  "1"  /* Repair Left Ankle Bursa and Ligament, Percutaneous Approach */
"0MQR4ZZ"  =  "1"  /* Repair Left Ankle Bursa and Ligament, Perc Endo Approach */
"0MQS0ZZ"  =  "1"  /* Repair Right Foot Bursa and Ligament, Open Approach */
"0MQS3ZZ"  =  "1"  /* Repair Right Foot Bursa and Ligament, Percutaneous Approach */
"0MQS4ZZ"  =  "1"  /* Repair Right Foot Bursa and Ligament, Perc Endo Approach */
"0MQT0ZZ"  =  "1"  /* Repair Left Foot Bursa and Ligament, Open Approach */
"0MQT3ZZ"  =  "1"  /* Repair Left Foot Bursa and Ligament, Percutaneous Approach */
"0MQT4ZZ"  =  "1"  /* Repair Left Foot Bursa and Ligament, Perc Endo Approach */
"0MQV0ZZ"  =  "1"  /* Repair R Low Extrem Bursa/Lig, Open Approach */
"0MQV3ZZ"  =  "1"  /* Repair R Low Extrem Bursa/Lig, Perc Approach */
"0MQV4ZZ"  =  "1"  /* Repair R Low Extrem Bursa/Lig, Perc Endo Approach */
"0MQW0ZZ"  =  "1"  /* Repair L Low Extrem Bursa/Lig, Open Approach */
"0MQW3ZZ"  =  "1"  /* Repair L Low Extrem Bursa/Lig, Perc Approach */
"0MQW4ZZ"  =  "1"  /* Repair L Low Extrem Bursa/Lig, Perc Endo Approach */
"0MS00ZZ"  =  "1"  /* Reposition Head and Neck Bursa and Ligament, Open Approach */
"0MS04ZZ"  =  "1"  /* Reposition Head & Neck Bursa/Lig, Perc Endo Approach */
"0MS10ZZ"  =  "1"  /* Reposition Right Shoulder Bursa and Ligament, Open Approach */
"0MS14ZZ"  =  "1"  /* Reposition R Shoulder Bursa/Lig, Perc Endo Approach */
"0MS20ZZ"  =  "1"  /* Reposition Left Shoulder Bursa and Ligament, Open Approach */
"0MS24ZZ"  =  "1"  /* Reposition L Shoulder Bursa/Lig, Perc Endo Approach */
"0MS30ZZ"  =  "1"  /* Reposition Right Elbow Bursa and Ligament, Open Approach */
"0MS34ZZ"  =  "1"  /* Reposition R Elbow Bursa/Lig, Perc Endo Approach */
"0MS40ZZ"  =  "1"  /* Reposition Left Elbow Bursa and Ligament, Open Approach */
"0MS44ZZ"  =  "1"  /* Reposition Left Elbow Bursa and Ligament, Perc Endo Approach */
"0MS50ZZ"  =  "1"  /* Reposition Right Wrist Bursa and Ligament, Open Approach */
"0MS54ZZ"  =  "1"  /* Reposition R Wrist Bursa/Lig, Perc Endo Approach */
"0MS60ZZ"  =  "1"  /* Reposition Left Wrist Bursa and Ligament, Open Approach */
"0MS64ZZ"  =  "1"  /* Reposition Left Wrist Bursa and Ligament, Perc Endo Approach */
"0MS70ZZ"  =  "1"  /* Reposition Right Hand Bursa and Ligament, Open Approach */
"0MS74ZZ"  =  "1"  /* Reposition Right Hand Bursa and Ligament, Perc Endo Approach */
"0MS80ZZ"  =  "1"  /* Reposition Left Hand Bursa and Ligament, Open Approach */
"0MS84ZZ"  =  "1"  /* Reposition Left Hand Bursa and Ligament, Perc Endo Approach */
"0MS90ZZ"  =  "1"  /* Reposition R Up Extrem Bursa/Lig, Open Approach */
"0MS94ZZ"  =  "1"  /* Reposition R Up Extrem Bursa/Lig, Perc Endo Approach */
"0MSB0ZZ"  =  "1"  /* Reposition L Up Extrem Bursa/Lig, Open Approach */
"0MSB4ZZ"  =  "1"  /* Reposition L Up Extrem Bursa/Lig, Perc Endo Approach */
"0MSC0ZZ"  =  "1"  /* Reposition Right Trunk Bursa and Ligament, Open Approach */
"0MSC4ZZ"  =  "1"  /* Reposition R Trunk Bursa/Lig, Perc Endo Approach */
"0MSD0ZZ"  =  "1"  /* Reposition Left Trunk Bursa and Ligament, Open Approach */
"0MSD4ZZ"  =  "1"  /* Reposition Left Trunk Bursa and Ligament, Perc Endo Approach */
"0MSF0ZZ"  =  "1"  /* Reposition Right Thorax Bursa and Ligament, Open Approach */
"0MSF4ZZ"  =  "1"  /* Reposition R Thorax Bursa/Lig, Perc Endo Approach */
"0MSG0ZZ"  =  "1"  /* Reposition Left Thorax Bursa and Ligament, Open Approach */
"0MSG4ZZ"  =  "1"  /* Reposition L Thorax Bursa/Lig, Perc Endo Approach */
"0MSH0ZZ"  =  "1"  /* Reposition Right Abdomen Bursa and Ligament, Open Approach */
"0MSH4ZZ"  =  "1"  /* Reposition R Abd Bursa/Lig, Perc Endo Approach */
"0MSJ0ZZ"  =  "1"  /* Reposition Left Abdomen Bursa and Ligament, Open Approach */
"0MSJ4ZZ"  =  "1"  /* Reposition L Abd Bursa/Lig, Perc Endo Approach */
"0MSK0ZZ"  =  "1"  /* Reposition Perineum Bursa and Ligament, Open Approach */
"0MSK4ZZ"  =  "1"  /* Reposition Perineum Bursa and Ligament, Perc Endo Approach */
"0MSL0ZZ"  =  "1"  /* Reposition Right Hip Bursa and Ligament, Open Approach */
"0MSL4ZZ"  =  "1"  /* Reposition Right Hip Bursa and Ligament, Perc Endo Approach */
"0MSM0ZZ"  =  "1"  /* Reposition Left Hip Bursa and Ligament, Open Approach */
"0MSM4ZZ"  =  "1"  /* Reposition Left Hip Bursa and Ligament, Perc Endo Approach */
"0MSN0ZZ"  =  "1"  /* Reposition Right Knee Bursa and Ligament, Open Approach */
"0MSN4ZZ"  =  "1"  /* Reposition Right Knee Bursa and Ligament, Perc Endo Approach */
"0MSP0ZZ"  =  "1"  /* Reposition Left Knee Bursa and Ligament, Open Approach */
"0MSP4ZZ"  =  "1"  /* Reposition Left Knee Bursa and Ligament, Perc Endo Approach */
"0MSQ0ZZ"  =  "1"  /* Reposition Right Ankle Bursa and Ligament, Open Approach */
"0MSQ4ZZ"  =  "1"  /* Reposition R Ankle Bursa/Lig, Perc Endo Approach */
"0MSR0ZZ"  =  "1"  /* Reposition Left Ankle Bursa and Ligament, Open Approach */
"0MSR4ZZ"  =  "1"  /* Reposition Left Ankle Bursa and Ligament, Perc Endo Approach */
"0MSS0ZZ"  =  "1"  /* Reposition Right Foot Bursa and Ligament, Open Approach */
"0MSS4ZZ"  =  "1"  /* Reposition Right Foot Bursa and Ligament, Perc Endo Approach */
"0MST0ZZ"  =  "1"  /* Reposition Left Foot Bursa and Ligament, Open Approach */
"0MST4ZZ"  =  "1"  /* Reposition Left Foot Bursa and Ligament, Perc Endo Approach */
"0MSV0ZZ"  =  "1"  /* Reposition R Low Extrem Bursa/Lig, Open Approach */
"0MSV4ZZ"  =  "1"  /* Reposition R Low Extrem Bursa/Lig, Perc Endo Approach */
"0MSW0ZZ"  =  "1"  /* Reposition L Low Extrem Bursa/Lig, Open Approach */
"0MSW4ZZ"  =  "1"  /* Reposition L Low Extrem Bursa/Lig, Perc Endo Approach */
"0MT00ZZ"  =  "1"  /* Resection of Head and Neck Bursa and Ligament, Open Approach */
"0MT04ZZ"  =  "1"  /* Resection of Head & Neck Bursa/Lig, Perc Endo Approach */
"0MT10ZZ"  =  "1"  /* Resection of R Shoulder Bursa/Lig, Open Approach */
"0MT14ZZ"  =  "1"  /* Resection of R Shoulder Bursa/Lig, Perc Endo Approach */
"0MT20ZZ"  =  "1"  /* Resection of Left Shoulder Bursa and Ligament, Open Approach */
"0MT24ZZ"  =  "1"  /* Resection of L Shoulder Bursa/Lig, Perc Endo Approach */
"0MT30ZZ"  =  "1"  /* Resection of Right Elbow Bursa and Ligament, Open Approach */
"0MT34ZZ"  =  "1"  /* Resection of R Elbow Bursa/Lig, Perc Endo Approach */
"0MT40ZZ"  =  "1"  /* Resection of Left Elbow Bursa and Ligament, Open Approach */
"0MT44ZZ"  =  "1"  /* Resection of L Elbow Bursa/Lig, Perc Endo Approach */
"0MT50ZZ"  =  "1"  /* Resection of Right Wrist Bursa and Ligament, Open Approach */
"0MT54ZZ"  =  "1"  /* Resection of R Wrist Bursa/Lig, Perc Endo Approach */
"0MT60ZZ"  =  "1"  /* Resection of Left Wrist Bursa and Ligament, Open Approach */
"0MT64ZZ"  =  "1"  /* Resection of L Wrist Bursa/Lig, Perc Endo Approach */
"0MT70ZZ"  =  "1"  /* Resection of Right Hand Bursa and Ligament, Open Approach */
"0MT74ZZ"  =  "1"  /* Resection of R Hand Bursa/Lig, Perc Endo Approach */
"0MT80ZZ"  =  "1"  /* Resection of Left Hand Bursa and Ligament, Open Approach */
"0MT84ZZ"  =  "1"  /* Resection of L Hand Bursa/Lig, Perc Endo Approach */
"0MT90ZZ"  =  "1"  /* Resection of R Up Extrem Bursa/Lig, Open Approach */
"0MT94ZZ"  =  "1"  /* Resection of R Up Extrem Bursa/Lig, Perc Endo Approach */
"0MTB0ZZ"  =  "1"  /* Resection of L Up Extrem Bursa/Lig, Open Approach */
"0MTB4ZZ"  =  "1"  /* Resection of L Up Extrem Bursa/Lig, Perc Endo Approach */
"0MTC0ZZ"  =  "1"  /* Resection of Right Trunk Bursa and Ligament, Open Approach */
"0MTC4ZZ"  =  "1"  /* Resection of R Trunk Bursa/Lig, Perc Endo Approach */
"0MTD0ZZ"  =  "1"  /* Resection of Left Trunk Bursa and Ligament, Open Approach */
"0MTD4ZZ"  =  "1"  /* Resection of L Trunk Bursa/Lig, Perc Endo Approach */
"0MTF0ZZ"  =  "1"  /* Resection of Right Thorax Bursa and Ligament, Open Approach */
"0MTF4ZZ"  =  "1"  /* Resection of R Thorax Bursa/Lig, Perc Endo Approach */
"0MTG0ZZ"  =  "1"  /* Resection of Left Thorax Bursa and Ligament, Open Approach */
"0MTG4ZZ"  =  "1"  /* Resection of L Thorax Bursa/Lig, Perc Endo Approach */
"0MTH0ZZ"  =  "1"  /* Resection of Right Abdomen Bursa and Ligament, Open Approach */
"0MTH4ZZ"  =  "1"  /* Resection of R Abd Bursa/Lig, Perc Endo Approach */
"0MTJ0ZZ"  =  "1"  /* Resection of Left Abdomen Bursa and Ligament, Open Approach */
"0MTJ4ZZ"  =  "1"  /* Resection of L Abd Bursa/Lig, Perc Endo Approach */
"0MTK0ZZ"  =  "1"  /* Resection of Perineum Bursa and Ligament, Open Approach */
"0MTK4ZZ"  =  "1"  /* Resection of Perineum Bursa and Ligament, Perc Endo Approach */
"0MTL0ZZ"  =  "1"  /* Resection of Right Hip Bursa and Ligament, Open Approach */
"0MTL4ZZ"  =  "1"  /* Resection of R Hip Bursa/Lig, Perc Endo Approach */
"0MTM0ZZ"  =  "1"  /* Resection of Left Hip Bursa and Ligament, Open Approach */
"0MTM4ZZ"  =  "1"  /* Resection of Left Hip Bursa and Ligament, Perc Endo Approach */
"0MTN0ZZ"  =  "1"  /* Resection of Right Knee Bursa and Ligament, Open Approach */
"0MTN4ZZ"  =  "1"  /* Resection of R Knee Bursa/Lig, Perc Endo Approach */
"0MTP0ZZ"  =  "1"  /* Resection of Left Knee Bursa and Ligament, Open Approach */
"0MTP4ZZ"  =  "1"  /* Resection of L Knee Bursa/Lig, Perc Endo Approach */
"0MTQ0ZZ"  =  "1"  /* Resection of Right Ankle Bursa and Ligament, Open Approach */
"0MTQ4ZZ"  =  "1"  /* Resection of R Ankle Bursa/Lig, Perc Endo Approach */
"0MTR0ZZ"  =  "1"  /* Resection of Left Ankle Bursa and Ligament, Open Approach */
"0MTR4ZZ"  =  "1"  /* Resection of L Ankle Bursa/Lig, Perc Endo Approach */
"0MTS0ZZ"  =  "1"  /* Resection of Right Foot Bursa and Ligament, Open Approach */
"0MTS4ZZ"  =  "1"  /* Resection of R Foot Bursa/Lig, Perc Endo Approach */
"0MTT0ZZ"  =  "1"  /* Resection of Left Foot Bursa and Ligament, Open Approach */
"0MTT4ZZ"  =  "1"  /* Resection of L Foot Bursa/Lig, Perc Endo Approach */
"0MTV0ZZ"  =  "1"  /* Resection of R Low Extrem Bursa/Lig, Open Approach */
"0MTV4ZZ"  =  "1"  /* Resection of R Low Extrem Bursa/Lig, Perc Endo Approach */
"0MTW0ZZ"  =  "1"  /* Resection of L Low Extrem Bursa/Lig, Open Approach */
"0MTW4ZZ"  =  "1"  /* Resection of L Low Extrem Bursa/Lig, Perc Endo Approach */
"0MU007Z"  =  "1"  /* Supplement Head & Neck Bursa/Lig w Autol Sub, Open */
"0MU00JZ"  =  "1"  /* Supplement Head & Neck Bursa/Lig w Synth Sub, Open */
"0MU00KZ"  =  "1"  /* Supplement Head & Neck Bursa/Lig w Nonaut Sub, Open */
"0MU047Z"  =  "1"  /* Supplement Head & Neck Bursa/Lig w Autol Sub, Perc Endo */
"0MU04JZ"  =  "1"  /* Supplement Head & Neck Bursa/Lig w Synth Sub, Perc Endo */
"0MU04KZ"  =  "1"  /* Supplement Head & Neck Bursa/Lig w Nonaut Sub, Perc Endo */
"0MU107Z"  =  "1"  /* Supplement R Shoulder Bursa/Lig w Autol Sub, Open */
"0MU10JZ"  =  "1"  /* Supplement R Shoulder Bursa/Lig w Synth Sub, Open */
"0MU10KZ"  =  "1"  /* Supplement R Shoulder Bursa/Lig w Nonaut Sub, Open */
"0MU147Z"  =  "1"  /* Supplement R Shoulder Bursa/Lig w Autol Sub, Perc Endo */
"0MU14JZ"  =  "1"  /* Supplement R Shoulder Bursa/Lig w Synth Sub, Perc Endo */
"0MU14KZ"  =  "1"  /* Supplement R Shoulder Bursa/Lig w Nonaut Sub, Perc Endo */
"0MU207Z"  =  "1"  /* Supplement L Shoulder Bursa/Lig w Autol Sub, Open */
"0MU20JZ"  =  "1"  /* Supplement L Shoulder Bursa/Lig w Synth Sub, Open */
"0MU20KZ"  =  "1"  /* Supplement L Shoulder Bursa/Lig w Nonaut Sub, Open */
"0MU247Z"  =  "1"  /* Supplement L Shoulder Bursa/Lig w Autol Sub, Perc Endo */
"0MU24JZ"  =  "1"  /* Supplement L Shoulder Bursa/Lig w Synth Sub, Perc Endo */
"0MU24KZ"  =  "1"  /* Supplement L Shoulder Bursa/Lig w Nonaut Sub, Perc Endo */
"0MU307Z"  =  "1"  /* Supplement R Elbow Bursa/Lig with Autol Sub, Open Approach */
"0MU30JZ"  =  "1"  /* Supplement R Elbow Bursa/Lig with Synth Sub, Open Approach */
"0MU30KZ"  =  "1"  /* Supplement R Elbow Bursa/Lig with Nonaut Sub, Open Approach */
"0MU347Z"  =  "1"  /* Supplement R Elbow Bursa/Lig w Autol Sub, Perc Endo */
"0MU34JZ"  =  "1"  /* Supplement R Elbow Bursa/Lig w Synth Sub, Perc Endo */
"0MU34KZ"  =  "1"  /* Supplement R Elbow Bursa/Lig w Nonaut Sub, Perc Endo */
"0MU407Z"  =  "1"  /* Supplement L Elbow Bursa/Lig with Autol Sub, Open Approach */
"0MU40JZ"  =  "1"  /* Supplement L Elbow Bursa/Lig with Synth Sub, Open Approach */
"0MU40KZ"  =  "1"  /* Supplement L Elbow Bursa/Lig with Nonaut Sub, Open Approach */
"0MU447Z"  =  "1"  /* Supplement L Elbow Bursa/Lig w Autol Sub, Perc Endo */
"0MU44JZ"  =  "1"  /* Supplement L Elbow Bursa/Lig w Synth Sub, Perc Endo */
"0MU44KZ"  =  "1"  /* Supplement L Elbow Bursa/Lig w Nonaut Sub, Perc Endo */
"0MU507Z"  =  "1"  /* Supplement R Wrist Bursa/Lig with Autol Sub, Open Approach */
"0MU50JZ"  =  "1"  /* Supplement R Wrist Bursa/Lig with Synth Sub, Open Approach */
"0MU50KZ"  =  "1"  /* Supplement R Wrist Bursa/Lig with Nonaut Sub, Open Approach */
"0MU547Z"  =  "1"  /* Supplement R Wrist Bursa/Lig w Autol Sub, Perc Endo */
"0MU54JZ"  =  "1"  /* Supplement R Wrist Bursa/Lig w Synth Sub, Perc Endo */
"0MU54KZ"  =  "1"  /* Supplement R Wrist Bursa/Lig w Nonaut Sub, Perc Endo */
"0MU607Z"  =  "1"  /* Supplement L Wrist Bursa/Lig with Autol Sub, Open Approach */
"0MU60JZ"  =  "1"  /* Supplement L Wrist Bursa/Lig with Synth Sub, Open Approach */
"0MU60KZ"  =  "1"  /* Supplement L Wrist Bursa/Lig with Nonaut Sub, Open Approach */
"0MU647Z"  =  "1"  /* Supplement L Wrist Bursa/Lig w Autol Sub, Perc Endo */
"0MU64JZ"  =  "1"  /* Supplement L Wrist Bursa/Lig w Synth Sub, Perc Endo */
"0MU64KZ"  =  "1"  /* Supplement L Wrist Bursa/Lig w Nonaut Sub, Perc Endo */
"0MU707Z"  =  "1"  /* Supplement R Hand Bursa/Lig with Autol Sub, Open Approach */
"0MU70JZ"  =  "1"  /* Supplement R Hand Bursa/Lig with Synth Sub, Open Approach */
"0MU70KZ"  =  "1"  /* Supplement R Hand Bursa/Lig with Nonaut Sub, Open Approach */
"0MU747Z"  =  "1"  /* Supplement R Hand Bursa/Lig w Autol Sub, Perc Endo */
"0MU74JZ"  =  "1"  /* Supplement R Hand Bursa/Lig w Synth Sub, Perc Endo */
"0MU74KZ"  =  "1"  /* Supplement R Hand Bursa/Lig w Nonaut Sub, Perc Endo */
"0MU807Z"  =  "1"  /* Supplement L Hand Bursa/Lig with Autol Sub, Open Approach */
"0MU80JZ"  =  "1"  /* Supplement L Hand Bursa/Lig with Synth Sub, Open Approach */
"0MU80KZ"  =  "1"  /* Supplement L Hand Bursa/Lig with Nonaut Sub, Open Approach */
"0MU847Z"  =  "1"  /* Supplement L Hand Bursa/Lig w Autol Sub, Perc Endo */
"0MU84JZ"  =  "1"  /* Supplement L Hand Bursa/Lig w Synth Sub, Perc Endo */
"0MU84KZ"  =  "1"  /* Supplement L Hand Bursa/Lig w Nonaut Sub, Perc Endo */
"0MU907Z"  =  "1"  /* Supplement R Up Extrem Bursa/Lig w Autol Sub, Open */
"0MU90JZ"  =  "1"  /* Supplement R Up Extrem Bursa/Lig w Synth Sub, Open */
"0MU90KZ"  =  "1"  /* Supplement R Up Extrem Bursa/Lig w Nonaut Sub, Open */
"0MU947Z"  =  "1"  /* Supplement R Up Extrem Bursa/Lig w Autol Sub, Perc Endo */
"0MU94JZ"  =  "1"  /* Supplement R Up Extrem Bursa/Lig w Synth Sub, Perc Endo */
"0MU94KZ"  =  "1"  /* Supplement R Up Extrem Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUB07Z"  =  "1"  /* Supplement L Up Extrem Bursa/Lig w Autol Sub, Open */
"0MUB0JZ"  =  "1"  /* Supplement L Up Extrem Bursa/Lig w Synth Sub, Open */
"0MUB0KZ"  =  "1"  /* Supplement L Up Extrem Bursa/Lig w Nonaut Sub, Open */
"0MUB47Z"  =  "1"  /* Supplement L Up Extrem Bursa/Lig w Autol Sub, Perc Endo */
"0MUB4JZ"  =  "1"  /* Supplement L Up Extrem Bursa/Lig w Synth Sub, Perc Endo */
"0MUB4KZ"  =  "1"  /* Supplement L Up Extrem Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUC07Z"  =  "1"  /* Supplement R Trunk Bursa/Lig with Autol Sub, Open Approach */
"0MUC0JZ"  =  "1"  /* Supplement R Trunk Bursa/Lig with Synth Sub, Open Approach */
"0MUC0KZ"  =  "1"  /* Supplement R Trunk Bursa/Lig with Nonaut Sub, Open Approach */
"0MUC47Z"  =  "1"  /* Supplement R Trunk Bursa/Lig w Autol Sub, Perc Endo */
"0MUC4JZ"  =  "1"  /* Supplement R Trunk Bursa/Lig w Synth Sub, Perc Endo */
"0MUC4KZ"  =  "1"  /* Supplement R Trunk Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUD07Z"  =  "1"  /* Supplement L Trunk Bursa/Lig with Autol Sub, Open Approach */
"0MUD0JZ"  =  "1"  /* Supplement L Trunk Bursa/Lig with Synth Sub, Open Approach */
"0MUD0KZ"  =  "1"  /* Supplement L Trunk Bursa/Lig with Nonaut Sub, Open Approach */
"0MUD47Z"  =  "1"  /* Supplement L Trunk Bursa/Lig w Autol Sub, Perc Endo */
"0MUD4JZ"  =  "1"  /* Supplement L Trunk Bursa/Lig w Synth Sub, Perc Endo */
"0MUD4KZ"  =  "1"  /* Supplement L Trunk Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUF07Z"  =  "1"  /* Supplement R Thorax Bursa/Lig with Autol Sub, Open Approach */
"0MUF0JZ"  =  "1"  /* Supplement R Thorax Bursa/Lig with Synth Sub, Open Approach */
"0MUF0KZ"  =  "1"  /* Supplement R Thorax Bursa/Lig with Nonaut Sub, Open Approach */
"0MUF47Z"  =  "1"  /* Supplement R Thorax Bursa/Lig w Autol Sub, Perc Endo */
"0MUF4JZ"  =  "1"  /* Supplement R Thorax Bursa/Lig w Synth Sub, Perc Endo */
"0MUF4KZ"  =  "1"  /* Supplement R Thorax Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUG07Z"  =  "1"  /* Supplement L Thorax Bursa/Lig with Autol Sub, Open Approach */
"0MUG0JZ"  =  "1"  /* Supplement L Thorax Bursa/Lig with Synth Sub, Open Approach */
"0MUG0KZ"  =  "1"  /* Supplement L Thorax Bursa/Lig with Nonaut Sub, Open Approach */
"0MUG47Z"  =  "1"  /* Supplement L Thorax Bursa/Lig w Autol Sub, Perc Endo */
"0MUG4JZ"  =  "1"  /* Supplement L Thorax Bursa/Lig w Synth Sub, Perc Endo */
"0MUG4KZ"  =  "1"  /* Supplement L Thorax Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUH07Z"  =  "1"  /* Supplement R Abd Bursa/Lig with Autol Sub, Open Approach */
"0MUH0JZ"  =  "1"  /* Supplement R Abd Bursa/Lig with Synth Sub, Open Approach */
"0MUH0KZ"  =  "1"  /* Supplement R Abd Bursa/Lig with Nonaut Sub, Open Approach */
"0MUH47Z"  =  "1"  /* Supplement R Abd Bursa/Lig w Autol Sub, Perc Endo */
"0MUH4JZ"  =  "1"  /* Supplement R Abd Bursa/Lig w Synth Sub, Perc Endo */
"0MUH4KZ"  =  "1"  /* Supplement R Abd Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUJ07Z"  =  "1"  /* Supplement L Abd Bursa/Lig with Autol Sub, Open Approach */
"0MUJ0JZ"  =  "1"  /* Supplement L Abd Bursa/Lig with Synth Sub, Open Approach */
"0MUJ0KZ"  =  "1"  /* Supplement L Abd Bursa/Lig with Nonaut Sub, Open Approach */
"0MUJ47Z"  =  "1"  /* Supplement L Abd Bursa/Lig w Autol Sub, Perc Endo */
"0MUJ4JZ"  =  "1"  /* Supplement L Abd Bursa/Lig w Synth Sub, Perc Endo */
"0MUJ4KZ"  =  "1"  /* Supplement L Abd Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUK07Z"  =  "1"  /* Supplement Perineum Bursa/Lig with Autol Sub, Open Approach */
"0MUK0JZ"  =  "1"  /* Supplement Perineum Bursa/Lig with Synth Sub, Open Approach */
"0MUK0KZ"  =  "1"  /* Supplement Perineum Bursa/Lig with Nonaut Sub, Open Approach */
"0MUK47Z"  =  "1"  /* Supplement Perineum Bursa/Lig w Autol Sub, Perc Endo */
"0MUK4JZ"  =  "1"  /* Supplement Perineum Bursa/Lig w Synth Sub, Perc Endo */
"0MUK4KZ"  =  "1"  /* Supplement Perineum Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUL07Z"  =  "1"  /* Supplement R Hip Bursa/Lig with Autol Sub, Open Approach */
"0MUL0JZ"  =  "1"  /* Supplement R Hip Bursa/Lig with Synth Sub, Open Approach */
"0MUL0KZ"  =  "1"  /* Supplement R Hip Bursa/Lig with Nonaut Sub, Open Approach */
"0MUL47Z"  =  "1"  /* Supplement R Hip Bursa/Lig w Autol Sub, Perc Endo */
"0MUL4JZ"  =  "1"  /* Supplement R Hip Bursa/Lig w Synth Sub, Perc Endo */
"0MUL4KZ"  =  "1"  /* Supplement R Hip Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUM07Z"  =  "1"  /* Supplement L Hip Bursa/Lig with Autol Sub, Open Approach */
"0MUM0JZ"  =  "1"  /* Supplement L Hip Bursa/Lig with Synth Sub, Open Approach */
"0MUM0KZ"  =  "1"  /* Supplement L Hip Bursa/Lig with Nonaut Sub, Open Approach */
"0MUM47Z"  =  "1"  /* Supplement L Hip Bursa/Lig w Autol Sub, Perc Endo */
"0MUM4JZ"  =  "1"  /* Supplement L Hip Bursa/Lig w Synth Sub, Perc Endo */
"0MUM4KZ"  =  "1"  /* Supplement L Hip Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUN07Z"  =  "1"  /* Supplement R Knee Bursa/Lig with Autol Sub, Open Approach */
"0MUN0JZ"  =  "1"  /* Supplement R Knee Bursa/Lig with Synth Sub, Open Approach */
"0MUN0KZ"  =  "1"  /* Supplement R Knee Bursa/Lig with Nonaut Sub, Open Approach */
"0MUN47Z"  =  "1"  /* Supplement R Knee Bursa/Lig w Autol Sub, Perc Endo */
"0MUN4JZ"  =  "1"  /* Supplement R Knee Bursa/Lig w Synth Sub, Perc Endo */
"0MUN4KZ"  =  "1"  /* Supplement R Knee Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUP07Z"  =  "1"  /* Supplement L Knee Bursa/Lig with Autol Sub, Open Approach */
"0MUP0JZ"  =  "1"  /* Supplement L Knee Bursa/Lig with Synth Sub, Open Approach */
"0MUP0KZ"  =  "1"  /* Supplement L Knee Bursa/Lig with Nonaut Sub, Open Approach */
"0MUP47Z"  =  "1"  /* Supplement L Knee Bursa/Lig w Autol Sub, Perc Endo */
"0MUP4JZ"  =  "1"  /* Supplement L Knee Bursa/Lig w Synth Sub, Perc Endo */
"0MUP4KZ"  =  "1"  /* Supplement L Knee Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUQ07Z"  =  "1"  /* Supplement R Ankle Bursa/Lig with Autol Sub, Open Approach */
"0MUQ0JZ"  =  "1"  /* Supplement R Ankle Bursa/Lig with Synth Sub, Open Approach */
"0MUQ0KZ"  =  "1"  /* Supplement R Ankle Bursa/Lig with Nonaut Sub, Open Approach */
"0MUQ47Z"  =  "1"  /* Supplement R Ankle Bursa/Lig w Autol Sub, Perc Endo */
"0MUQ4JZ"  =  "1"  /* Supplement R Ankle Bursa/Lig w Synth Sub, Perc Endo */
"0MUQ4KZ"  =  "1"  /* Supplement R Ankle Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUR07Z"  =  "1"  /* Supplement L Ankle Bursa/Lig with Autol Sub, Open Approach */
"0MUR0JZ"  =  "1"  /* Supplement L Ankle Bursa/Lig with Synth Sub, Open Approach */
"0MUR0KZ"  =  "1"  /* Supplement L Ankle Bursa/Lig with Nonaut Sub, Open Approach */
"0MUR47Z"  =  "1"  /* Supplement L Ankle Bursa/Lig w Autol Sub, Perc Endo */
"0MUR4JZ"  =  "1"  /* Supplement L Ankle Bursa/Lig w Synth Sub, Perc Endo */
"0MUR4KZ"  =  "1"  /* Supplement L Ankle Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUS07Z"  =  "1"  /* Supplement R Foot Bursa/Lig with Autol Sub, Open Approach */
"0MUS0JZ"  =  "1"  /* Supplement R Foot Bursa/Lig with Synth Sub, Open Approach */
"0MUS0KZ"  =  "1"  /* Supplement R Foot Bursa/Lig with Nonaut Sub, Open Approach */
"0MUS47Z"  =  "1"  /* Supplement R Foot Bursa/Lig w Autol Sub, Perc Endo */
"0MUS4JZ"  =  "1"  /* Supplement R Foot Bursa/Lig w Synth Sub, Perc Endo */
"0MUS4KZ"  =  "1"  /* Supplement R Foot Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUT07Z"  =  "1"  /* Supplement L Foot Bursa/Lig with Autol Sub, Open Approach */
"0MUT0JZ"  =  "1"  /* Supplement L Foot Bursa/Lig with Synth Sub, Open Approach */
"0MUT0KZ"  =  "1"  /* Supplement L Foot Bursa/Lig with Nonaut Sub, Open Approach */
"0MUT47Z"  =  "1"  /* Supplement L Foot Bursa/Lig w Autol Sub, Perc Endo */
"0MUT4JZ"  =  "1"  /* Supplement L Foot Bursa/Lig w Synth Sub, Perc Endo */
"0MUT4KZ"  =  "1"  /* Supplement L Foot Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUV07Z"  =  "1"  /* Supplement R Low Extrem Bursa/Lig w Autol Sub, Open */
"0MUV0JZ"  =  "1"  /* Supplement R Low Extrem Bursa/Lig w Synth Sub, Open */
"0MUV0KZ"  =  "1"  /* Supplement R Low Extrem Bursa/Lig w Nonaut Sub, Open */
"0MUV47Z"  =  "1"  /* Supplement R Low Extrem Bursa/Lig w Autol Sub, Perc Endo */
"0MUV4JZ"  =  "1"  /* Supplement R Low Extrem Bursa/Lig w Synth Sub, Perc Endo */
"0MUV4KZ"  =  "1"  /* Supplement R Low Extrem Bursa/Lig w Nonaut Sub, Perc Endo */
"0MUW07Z"  =  "1"  /* Supplement L Low Extrem Bursa/Lig w Autol Sub, Open */
"0MUW0JZ"  =  "1"  /* Supplement L Low Extrem Bursa/Lig w Synth Sub, Open */
"0MUW0KZ"  =  "1"  /* Supplement L Low Extrem Bursa/Lig w Nonaut Sub, Open */
"0MUW47Z"  =  "1"  /* Supplement L Low Extrem Bursa/Lig w Autol Sub, Perc Endo */
"0MUW4JZ"  =  "1"  /* Supplement L Low Extrem Bursa/Lig w Synth Sub, Perc Endo */
"0MUW4KZ"  =  "1"  /* Supplement L Low Extrem Bursa/Lig w Nonaut Sub, Perc Endo */
"0MWX00Z"  =  "1"  /* Revision of Drainage Device in Up Bursa/Lig, Open Approach */
"0MWX07Z"  =  "1"  /* Revision of Autol Sub in Up Bursa/Lig, Open Approach */
"0MWX0JZ"  =  "1"  /* Revision of Synth Sub in Up Bursa/Lig, Open Approach */
"0MWX0KZ"  =  "1"  /* Revision of Nonaut Sub in Up Bursa/Lig, Open Approach */
"0MWX30Z"  =  "1"  /* Revision of Drainage Device in Up Bursa/Lig, Perc Approach */
"0MWX37Z"  =  "1"  /* Revision of Autol Sub in Up Bursa/Lig, Perc Approach */
"0MWX3JZ"  =  "1"  /* Revision of Synth Sub in Up Bursa/Lig, Perc Approach */
"0MWX3KZ"  =  "1"  /* Revision of Nonaut Sub in Up Bursa/Lig, Perc Approach */
"0MWX40Z"  =  "1"  /* Revision of Drain Dev in Up Bursa/Lig, Perc Endo Approach */
"0MWX47Z"  =  "1"  /* Revision of Autol Sub in Up Bursa/Lig, Perc Endo Approach */
"0MWX4JZ"  =  "1"  /* Revision of Synth Sub in Up Bursa/Lig, Perc Endo Approach */
"0MWX4KZ"  =  "1"  /* Revision of Nonaut Sub in Up Bursa/Lig, Perc Endo Approach */
"0MWY00Z"  =  "1"  /* Revision of Drainage Device in Low Bursa/Lig, Open Approach */
"0MWY07Z"  =  "1"  /* Revision of Autol Sub in Low Bursa/Lig, Open Approach */
"0MWY0JZ"  =  "1"  /* Revision of Synth Sub in Low Bursa/Lig, Open Approach */
"0MWY0KZ"  =  "1"  /* Revision of Nonaut Sub in Low Bursa/Lig, Open Approach */
"0MWY30Z"  =  "1"  /* Revision of Drainage Device in Low Bursa/Lig, Perc Approach */
"0MWY37Z"  =  "1"  /* Revision of Autol Sub in Low Bursa/Lig, Perc Approach */
"0MWY3JZ"  =  "1"  /* Revision of Synth Sub in Low Bursa/Lig, Perc Approach */
"0MWY3KZ"  =  "1"  /* Revision of Nonaut Sub in Low Bursa/Lig, Perc Approach */
"0MWY40Z"  =  "1"  /* Revision of Drain Dev in Low Bursa/Lig, Perc Endo Approach */
"0MWY47Z"  =  "1"  /* Revision of Autol Sub in Low Bursa/Lig, Perc Endo Approach */
"0MWY4JZ"  =  "1"  /* Revision of Synth Sub in Low Bursa/Lig, Perc Endo Approach */
"0MWY4KZ"  =  "1"  /* Revision of Nonaut Sub in Low Bursa/Lig, Perc Endo Approach */
"0MX00ZZ"  =  "1"  /* Transfer Head and Neck Bursa and Ligament, Open Approach */
"0MX04ZZ"  =  "1"  /* Transfer Head & Neck Bursa/Lig, Perc Endo Approach */
"0MX10ZZ"  =  "1"  /* Transfer Right Shoulder Bursa and Ligament, Open Approach */
"0MX14ZZ"  =  "1"  /* Transfer R Shoulder Bursa/Lig, Perc Endo Approach */
"0MX20ZZ"  =  "1"  /* Transfer Left Shoulder Bursa and Ligament, Open Approach */
"0MX24ZZ"  =  "1"  /* Transfer L Shoulder Bursa/Lig, Perc Endo Approach */
"0MX30ZZ"  =  "1"  /* Transfer Right Elbow Bursa and Ligament, Open Approach */
"0MX34ZZ"  =  "1"  /* Transfer Right Elbow Bursa and Ligament, Perc Endo Approach */
"0MX40ZZ"  =  "1"  /* Transfer Left Elbow Bursa and Ligament, Open Approach */
"0MX44ZZ"  =  "1"  /* Transfer Left Elbow Bursa and Ligament, Perc Endo Approach */
"0MX50ZZ"  =  "1"  /* Transfer Right Wrist Bursa and Ligament, Open Approach */
"0MX54ZZ"  =  "1"  /* Transfer Right Wrist Bursa and Ligament, Perc Endo Approach */
"0MX60ZZ"  =  "1"  /* Transfer Left Wrist Bursa and Ligament, Open Approach */
"0MX64ZZ"  =  "1"  /* Transfer Left Wrist Bursa and Ligament, Perc Endo Approach */
"0MX70ZZ"  =  "1"  /* Transfer Right Hand Bursa and Ligament, Open Approach */
"0MX74ZZ"  =  "1"  /* Transfer Right Hand Bursa and Ligament, Perc Endo Approach */
"0MX80ZZ"  =  "1"  /* Transfer Left Hand Bursa and Ligament, Open Approach */
"0MX84ZZ"  =  "1"  /* Transfer Left Hand Bursa and Ligament, Perc Endo Approach */
"0MX90ZZ"  =  "1"  /* Transfer R Up Extrem Bursa/Lig, Open Approach */
"0MX94ZZ"  =  "1"  /* Transfer R Up Extrem Bursa/Lig, Perc Endo Approach */
"0MXB0ZZ"  =  "1"  /* Transfer L Up Extrem Bursa/Lig, Open Approach */
"0MXB4ZZ"  =  "1"  /* Transfer L Up Extrem Bursa/Lig, Perc Endo Approach */
"0MXC0ZZ"  =  "1"  /* Transfer Right Trunk Bursa and Ligament, Open Approach */
"0MXC4ZZ"  =  "1"  /* Transfer Right Trunk Bursa and Ligament, Perc Endo Approach */
"0MXD0ZZ"  =  "1"  /* Transfer Left Trunk Bursa and Ligament, Open Approach */
"0MXD4ZZ"  =  "1"  /* Transfer Left Trunk Bursa and Ligament, Perc Endo Approach */
"0MXF0ZZ"  =  "1"  /* Transfer Right Thorax Bursa and Ligament, Open Approach */
"0MXF4ZZ"  =  "1"  /* Transfer Right Thorax Bursa and Ligament, Perc Endo Approach */
"0MXG0ZZ"  =  "1"  /* Transfer Left Thorax Bursa and Ligament, Open Approach */
"0MXG4ZZ"  =  "1"  /* Transfer Left Thorax Bursa and Ligament, Perc Endo Approach */
"0MXH0ZZ"  =  "1"  /* Transfer Right Abdomen Bursa and Ligament, Open Approach */
"0MXH4ZZ"  =  "1"  /* Transfer R Abd Bursa/Lig, Perc Endo Approach */
"0MXJ0ZZ"  =  "1"  /* Transfer Left Abdomen Bursa and Ligament, Open Approach */
"0MXJ4ZZ"  =  "1"  /* Transfer Left Abdomen Bursa and Ligament, Perc Endo Approach */
"0MXK0ZZ"  =  "1"  /* Transfer Perineum Bursa and Ligament, Open Approach */
"0MXK4ZZ"  =  "1"  /* Transfer Perineum Bursa and Ligament, Perc Endo Approach */
"0MXL0ZZ"  =  "1"  /* Transfer Right Hip Bursa and Ligament, Open Approach */
"0MXL4ZZ"  =  "1"  /* Transfer Right Hip Bursa and Ligament, Perc Endo Approach */
"0MXM0ZZ"  =  "1"  /* Transfer Left Hip Bursa and Ligament, Open Approach */
"0MXM4ZZ"  =  "1"  /* Transfer Left Hip Bursa and Ligament, Perc Endo Approach */
"0MXN0ZZ"  =  "1"  /* Transfer Right Knee Bursa and Ligament, Open Approach */
"0MXN4ZZ"  =  "1"  /* Transfer Right Knee Bursa and Ligament, Perc Endo Approach */
"0MXP0ZZ"  =  "1"  /* Transfer Left Knee Bursa and Ligament, Open Approach */
"0MXP4ZZ"  =  "1"  /* Transfer Left Knee Bursa and Ligament, Perc Endo Approach */
"0MXQ0ZZ"  =  "1"  /* Transfer Right Ankle Bursa and Ligament, Open Approach */
"0MXQ4ZZ"  =  "1"  /* Transfer Right Ankle Bursa and Ligament, Perc Endo Approach */
"0MXR0ZZ"  =  "1"  /* Transfer Left Ankle Bursa and Ligament, Open Approach */
"0MXR4ZZ"  =  "1"  /* Transfer Left Ankle Bursa and Ligament, Perc Endo Approach */
"0MXS0ZZ"  =  "1"  /* Transfer Right Foot Bursa and Ligament, Open Approach */
"0MXS4ZZ"  =  "1"  /* Transfer Right Foot Bursa and Ligament, Perc Endo Approach */
"0MXT0ZZ"  =  "1"  /* Transfer Left Foot Bursa and Ligament, Open Approach */
"0MXT4ZZ"  =  "1"  /* Transfer Left Foot Bursa and Ligament, Perc Endo Approach */
"0MXV0ZZ"  =  "1"  /* Transfer R Low Extrem Bursa/Lig, Open Approach */
"0MXV4ZZ"  =  "1"  /* Transfer R Low Extrem Bursa/Lig, Perc Endo Approach */
"0MXW0ZZ"  =  "1"  /* Transfer L Low Extrem Bursa/Lig, Open Approach */
"0MXW4ZZ"  =  "1"  /* Transfer L Low Extrem Bursa/Lig, Perc Endo Approach */
"0N500ZZ"  =  "1"  /* Destruction of Skull, Open Approach */
"0N503ZZ"  =  "1"  /* Destruction of Skull, Percutaneous Approach */
"0N504ZZ"  =  "1"  /* Destruction of Skull, Percutaneous Endoscopic Approach */
"0N510ZZ"  =  "1"  /* Destruction of Right Frontal Bone, Open Approach */
"0N513ZZ"  =  "1"  /* Destruction of Right Frontal Bone, Percutaneous Approach */
"0N514ZZ"  =  "1"  /* Destruction of Right Frontal Bone, Perc Endo Approach */
"0N520ZZ"  =  "1"  /* Destruction of Left Frontal Bone, Open Approach */
"0N523ZZ"  =  "1"  /* Destruction of Left Frontal Bone, Percutaneous Approach */
"0N524ZZ"  =  "1"  /* Destruction of Left Frontal Bone, Perc Endo Approach */
"0N530ZZ"  =  "1"  /* Destruction of Right Parietal Bone, Open Approach */
"0N533ZZ"  =  "1"  /* Destruction of Right Parietal Bone, Percutaneous Approach */
"0N534ZZ"  =  "1"  /* Destruction of Right Parietal Bone, Perc Endo Approach */
"0N540ZZ"  =  "1"  /* Destruction of Left Parietal Bone, Open Approach */
"0N543ZZ"  =  "1"  /* Destruction of Left Parietal Bone, Percutaneous Approach */
"0N544ZZ"  =  "1"  /* Destruction of Left Parietal Bone, Perc Endo Approach */
"0N550ZZ"  =  "1"  /* Destruction of Right Temporal Bone, Open Approach */
"0N553ZZ"  =  "1"  /* Destruction of Right Temporal Bone, Percutaneous Approach */
"0N554ZZ"  =  "1"  /* Destruction of Right Temporal Bone, Perc Endo Approach */
"0N560ZZ"  =  "1"  /* Destruction of Left Temporal Bone, Open Approach */
"0N563ZZ"  =  "1"  /* Destruction of Left Temporal Bone, Percutaneous Approach */
"0N564ZZ"  =  "1"  /* Destruction of Left Temporal Bone, Perc Endo Approach */
"0N570ZZ"  =  "1"  /* Destruction of Right Occipital Bone, Open Approach */
"0N573ZZ"  =  "1"  /* Destruction of Right Occipital Bone, Percutaneous Approach */
"0N574ZZ"  =  "1"  /* Destruction of Right Occipital Bone, Perc Endo Approach */
"0N580ZZ"  =  "1"  /* Destruction of Left Occipital Bone, Open Approach */
"0N583ZZ"  =  "1"  /* Destruction of Left Occipital Bone, Percutaneous Approach */
"0N584ZZ"  =  "1"  /* Destruction of Left Occipital Bone, Perc Endo Approach */
"0N5B0ZZ"  =  "1"  /* Destruction of Nasal Bone, Open Approach */
"0N5B3ZZ"  =  "1"  /* Destruction of Nasal Bone, Percutaneous Approach */
"0N5B4ZZ"  =  "1"  /* Destruction of Nasal Bone, Percutaneous Endoscopic Approach */
"0N5C0ZZ"  =  "1"  /* Destruction of Right Sphenoid Bone, Open Approach */
"0N5C3ZZ"  =  "1"  /* Destruction of Right Sphenoid Bone, Percutaneous Approach */
"0N5C4ZZ"  =  "1"  /* Destruction of Right Sphenoid Bone, Perc Endo Approach */
"0N5D0ZZ"  =  "1"  /* Destruction of Left Sphenoid Bone, Open Approach */
"0N5D3ZZ"  =  "1"  /* Destruction of Left Sphenoid Bone, Percutaneous Approach */
"0N5D4ZZ"  =  "1"  /* Destruction of Left Sphenoid Bone, Perc Endo Approach */
"0N5F0ZZ"  =  "1"  /* Destruction of Right Ethmoid Bone, Open Approach */
"0N5F3ZZ"  =  "1"  /* Destruction of Right Ethmoid Bone, Percutaneous Approach */
"0N5F4ZZ"  =  "1"  /* Destruction of Right Ethmoid Bone, Perc Endo Approach */
"0N5G0ZZ"  =  "1"  /* Destruction of Left Ethmoid Bone, Open Approach */
"0N5G3ZZ"  =  "1"  /* Destruction of Left Ethmoid Bone, Percutaneous Approach */
"0N5G4ZZ"  =  "1"  /* Destruction of Left Ethmoid Bone, Perc Endo Approach */
"0N5H0ZZ"  =  "1"  /* Destruction of Right Lacrimal Bone, Open Approach */
"0N5H3ZZ"  =  "1"  /* Destruction of Right Lacrimal Bone, Percutaneous Approach */
"0N5H4ZZ"  =  "1"  /* Destruction of Right Lacrimal Bone, Perc Endo Approach */
"0N5J0ZZ"  =  "1"  /* Destruction of Left Lacrimal Bone, Open Approach */
"0N5J3ZZ"  =  "1"  /* Destruction of Left Lacrimal Bone, Percutaneous Approach */
"0N5J4ZZ"  =  "1"  /* Destruction of Left Lacrimal Bone, Perc Endo Approach */
"0N5K0ZZ"  =  "1"  /* Destruction of Right Palatine Bone, Open Approach */
"0N5K3ZZ"  =  "1"  /* Destruction of Right Palatine Bone, Percutaneous Approach */
"0N5K4ZZ"  =  "1"  /* Destruction of Right Palatine Bone, Perc Endo Approach */
"0N5L0ZZ"  =  "1"  /* Destruction of Left Palatine Bone, Open Approach */
"0N5L3ZZ"  =  "1"  /* Destruction of Left Palatine Bone, Percutaneous Approach */
"0N5L4ZZ"  =  "1"  /* Destruction of Left Palatine Bone, Perc Endo Approach */
"0N5M0ZZ"  =  "1"  /* Destruction of Right Zygomatic Bone, Open Approach */
"0N5M3ZZ"  =  "1"  /* Destruction of Right Zygomatic Bone, Percutaneous Approach */
"0N5M4ZZ"  =  "1"  /* Destruction of Right Zygomatic Bone, Perc Endo Approach */
"0N5N0ZZ"  =  "1"  /* Destruction of Left Zygomatic Bone, Open Approach */
"0N5N3ZZ"  =  "1"  /* Destruction of Left Zygomatic Bone, Percutaneous Approach */
"0N5N4ZZ"  =  "1"  /* Destruction of Left Zygomatic Bone, Perc Endo Approach */
"0N5P0ZZ"  =  "1"  /* Destruction of Right Orbit, Open Approach */
"0N5P3ZZ"  =  "1"  /* Destruction of Right Orbit, Percutaneous Approach */
"0N5P4ZZ"  =  "1"  /* Destruction of Right Orbit, Percutaneous Endoscopic Approach */
"0N5Q0ZZ"  =  "1"  /* Destruction of Left Orbit, Open Approach */
"0N5Q3ZZ"  =  "1"  /* Destruction of Left Orbit, Percutaneous Approach */
"0N5Q4ZZ"  =  "1"  /* Destruction of Left Orbit, Percutaneous Endoscopic Approach */
"0N5R0ZZ"  =  "1"  /* Destruction of Right Maxilla, Open Approach */
"0N5R3ZZ"  =  "1"  /* Destruction of Right Maxilla, Percutaneous Approach */
"0N5R4ZZ"  =  "1"  /* Destruction of Right Maxilla, Perc Endo Approach */
"0N5S0ZZ"  =  "1"  /* Destruction of Left Maxilla, Open Approach */
"0N5S3ZZ"  =  "1"  /* Destruction of Left Maxilla, Percutaneous Approach */
"0N5S4ZZ"  =  "1"  /* Destruction of Left Maxilla, Perc Endo Approach */
"0N5T0ZZ"  =  "1"  /* Destruction of Right Mandible, Open Approach */
"0N5T3ZZ"  =  "1"  /* Destruction of Right Mandible, Percutaneous Approach */
"0N5T4ZZ"  =  "1"  /* Destruction of Right Mandible, Perc Endo Approach */
"0N5V0ZZ"  =  "1"  /* Destruction of Left Mandible, Open Approach */
"0N5V3ZZ"  =  "1"  /* Destruction of Left Mandible, Percutaneous Approach */
"0N5V4ZZ"  =  "1"  /* Destruction of Left Mandible, Perc Endo Approach */
"0N5X0ZZ"  =  "1"  /* Destruction of Hyoid Bone, Open Approach */
"0N5X3ZZ"  =  "1"  /* Destruction of Hyoid Bone, Percutaneous Approach */
"0N5X4ZZ"  =  "1"  /* Destruction of Hyoid Bone, Percutaneous Endoscopic Approach */
"0N800ZZ"  =  "1"  /* Division of Skull, Open Approach */
"0N803ZZ"  =  "1"  /* Division of Skull, Percutaneous Approach */
"0N804ZZ"  =  "1"  /* Division of Skull, Percutaneous Endoscopic Approach */
"0N810ZZ"  =  "1"  /* Division of Right Frontal Bone, Open Approach */
"0N813ZZ"  =  "1"  /* Division of Right Frontal Bone, Percutaneous Approach */
"0N814ZZ"  =  "1"  /* Division of Right Frontal Bone, Perc Endo Approach */
"0N820ZZ"  =  "1"  /* Division of Left Frontal Bone, Open Approach */
"0N823ZZ"  =  "1"  /* Division of Left Frontal Bone, Percutaneous Approach */
"0N824ZZ"  =  "1"  /* Division of Left Frontal Bone, Perc Endo Approach */
"0N830ZZ"  =  "1"  /* Division of Right Parietal Bone, Open Approach */
"0N833ZZ"  =  "1"  /* Division of Right Parietal Bone, Percutaneous Approach */
"0N834ZZ"  =  "1"  /* Division of Right Parietal Bone, Perc Endo Approach */
"0N840ZZ"  =  "1"  /* Division of Left Parietal Bone, Open Approach */
"0N843ZZ"  =  "1"  /* Division of Left Parietal Bone, Percutaneous Approach */
"0N844ZZ"  =  "1"  /* Division of Left Parietal Bone, Perc Endo Approach */
"0N850ZZ"  =  "1"  /* Division of Right Temporal Bone, Open Approach */
"0N853ZZ"  =  "1"  /* Division of Right Temporal Bone, Percutaneous Approach */
"0N854ZZ"  =  "1"  /* Division of Right Temporal Bone, Perc Endo Approach */
"0N860ZZ"  =  "1"  /* Division of Left Temporal Bone, Open Approach */
"0N863ZZ"  =  "1"  /* Division of Left Temporal Bone, Percutaneous Approach */
"0N864ZZ"  =  "1"  /* Division of Left Temporal Bone, Perc Endo Approach */
"0N870ZZ"  =  "1"  /* Division of Right Occipital Bone, Open Approach */
"0N873ZZ"  =  "1"  /* Division of Right Occipital Bone, Percutaneous Approach */
"0N874ZZ"  =  "1"  /* Division of Right Occipital Bone, Perc Endo Approach */
"0N880ZZ"  =  "1"  /* Division of Left Occipital Bone, Open Approach */
"0N883ZZ"  =  "1"  /* Division of Left Occipital Bone, Percutaneous Approach */
"0N884ZZ"  =  "1"  /* Division of Left Occipital Bone, Perc Endo Approach */
"0N8C0ZZ"  =  "1"  /* Division of Right Sphenoid Bone, Open Approach */
"0N8C3ZZ"  =  "1"  /* Division of Right Sphenoid Bone, Percutaneous Approach */
"0N8C4ZZ"  =  "1"  /* Division of Right Sphenoid Bone, Perc Endo Approach */
"0N8D0ZZ"  =  "1"  /* Division of Left Sphenoid Bone, Open Approach */
"0N8D3ZZ"  =  "1"  /* Division of Left Sphenoid Bone, Percutaneous Approach */
"0N8D4ZZ"  =  "1"  /* Division of Left Sphenoid Bone, Perc Endo Approach */
"0N8F0ZZ"  =  "1"  /* Division of Right Ethmoid Bone, Open Approach */
"0N8F3ZZ"  =  "1"  /* Division of Right Ethmoid Bone, Percutaneous Approach */
"0N8F4ZZ"  =  "1"  /* Division of Right Ethmoid Bone, Perc Endo Approach */
"0N8G0ZZ"  =  "1"  /* Division of Left Ethmoid Bone, Open Approach */
"0N8G3ZZ"  =  "1"  /* Division of Left Ethmoid Bone, Percutaneous Approach */
"0N8G4ZZ"  =  "1"  /* Division of Left Ethmoid Bone, Perc Endo Approach */
"0N8H0ZZ"  =  "1"  /* Division of Right Lacrimal Bone, Open Approach */
"0N8H3ZZ"  =  "1"  /* Division of Right Lacrimal Bone, Percutaneous Approach */
"0N8H4ZZ"  =  "1"  /* Division of Right Lacrimal Bone, Perc Endo Approach */
"0N8J0ZZ"  =  "1"  /* Division of Left Lacrimal Bone, Open Approach */
"0N8J3ZZ"  =  "1"  /* Division of Left Lacrimal Bone, Percutaneous Approach */
"0N8J4ZZ"  =  "1"  /* Division of Left Lacrimal Bone, Perc Endo Approach */
"0N8K0ZZ"  =  "1"  /* Division of Right Palatine Bone, Open Approach */
"0N8K3ZZ"  =  "1"  /* Division of Right Palatine Bone, Percutaneous Approach */
"0N8K4ZZ"  =  "1"  /* Division of Right Palatine Bone, Perc Endo Approach */
"0N8L0ZZ"  =  "1"  /* Division of Left Palatine Bone, Open Approach */
"0N8L3ZZ"  =  "1"  /* Division of Left Palatine Bone, Percutaneous Approach */
"0N8L4ZZ"  =  "1"  /* Division of Left Palatine Bone, Perc Endo Approach */
"0N8M0ZZ"  =  "1"  /* Division of Right Zygomatic Bone, Open Approach */
"0N8M3ZZ"  =  "1"  /* Division of Right Zygomatic Bone, Percutaneous Approach */
"0N8M4ZZ"  =  "1"  /* Division of Right Zygomatic Bone, Perc Endo Approach */
"0N8N0ZZ"  =  "1"  /* Division of Left Zygomatic Bone, Open Approach */
"0N8N3ZZ"  =  "1"  /* Division of Left Zygomatic Bone, Percutaneous Approach */
"0N8N4ZZ"  =  "1"  /* Division of Left Zygomatic Bone, Perc Endo Approach */
"0N8P0ZZ"  =  "1"  /* Division of Right Orbit, Open Approach */
"0N8P3ZZ"  =  "1"  /* Division of Right Orbit, Percutaneous Approach */
"0N8P4ZZ"  =  "1"  /* Division of Right Orbit, Percutaneous Endoscopic Approach */
"0N8Q0ZZ"  =  "1"  /* Division of Left Orbit, Open Approach */
"0N8Q3ZZ"  =  "1"  /* Division of Left Orbit, Percutaneous Approach */
"0N8Q4ZZ"  =  "1"  /* Division of Left Orbit, Percutaneous Endoscopic Approach */
"0N8R0ZZ"  =  "1"  /* Division of Right Maxilla, Open Approach */
"0N8R3ZZ"  =  "1"  /* Division of Right Maxilla, Percutaneous Approach */
"0N8R4ZZ"  =  "1"  /* Division of Right Maxilla, Percutaneous Endoscopic Approach */
"0N8S0ZZ"  =  "1"  /* Division of Left Maxilla, Open Approach */
"0N8S3ZZ"  =  "1"  /* Division of Left Maxilla, Percutaneous Approach */
"0N8S4ZZ"  =  "1"  /* Division of Left Maxilla, Percutaneous Endoscopic Approach */
"0N8T0ZZ"  =  "1"  /* Division of Right Mandible, Open Approach */
"0N8T3ZZ"  =  "1"  /* Division of Right Mandible, Percutaneous Approach */
"0N8T4ZZ"  =  "1"  /* Division of Right Mandible, Percutaneous Endoscopic Approach */
"0N8V0ZZ"  =  "1"  /* Division of Left Mandible, Open Approach */
"0N8V3ZZ"  =  "1"  /* Division of Left Mandible, Percutaneous Approach */
"0N8V4ZZ"  =  "1"  /* Division of Left Mandible, Percutaneous Endoscopic Approach */
"0N8X0ZZ"  =  "1"  /* Division of Hyoid Bone, Open Approach */
"0N8X3ZZ"  =  "1"  /* Division of Hyoid Bone, Percutaneous Approach */
"0N8X4ZZ"  =  "1"  /* Division of Hyoid Bone, Percutaneous Endoscopic Approach */
"0N9000Z"  =  "1"  /* Drainage of Skull with Drainage Device, Open Approach */
"0N900ZX"  =  "1"  /* Drainage of Skull, Open Approach, Diagnostic */
"0N900ZZ"  =  "1"  /* Drainage of Skull, Open Approach */
"0N9030Z"  =  "1"  /* Drainage of Skull with Drainage Device, Perc Approach */
"0N903ZX"  =  "1"  /* Drainage of Skull, Percutaneous Approach, Diagnostic */
"0N903ZZ"  =  "1"  /* Drainage of Skull, Percutaneous Approach */
"0N9040Z"  =  "1"  /* Drainage of Skull with Drainage Device, Perc Endo Approach */
"0N904ZX"  =  "1"  /* Drainage of Skull, Percutaneous Endoscopic Approach, Diagn */
"0N904ZZ"  =  "1"  /* Drainage of Skull, Percutaneous Endoscopic Approach */
"0N9100Z"  =  "1"  /* Drainage of Right Frontal Bone with Drain Dev, Open Approach */
"0N910ZX"  =  "1"  /* Drainage of Right Frontal Bone, Open Approach, Diagnostic */
"0N910ZZ"  =  "1"  /* Drainage of Right Frontal Bone, Open Approach */
"0N9130Z"  =  "1"  /* Drainage of Right Frontal Bone with Drain Dev, Perc Approach */
"0N913ZX"  =  "1"  /* Drainage of Right Frontal Bone, Percutaneous Approach, Diagn */
"0N913ZZ"  =  "1"  /* Drainage of Right Frontal Bone, Percutaneous Approach */
"0N9140Z"  =  "1"  /* Drain of R Frontal Bone with Drain Dev, Perc Endo Approach */
"0N914ZX"  =  "1"  /* Drainage of Right Frontal Bone, Perc Endo Approach, Diagn */
"0N914ZZ"  =  "1"  /* Drainage of Right Frontal Bone, Perc Endo Approach */
"0N9200Z"  =  "1"  /* Drainage of Left Frontal Bone with Drain Dev, Open Approach */
"0N920ZX"  =  "1"  /* Drainage of Left Frontal Bone, Open Approach, Diagnostic */
"0N920ZZ"  =  "1"  /* Drainage of Left Frontal Bone, Open Approach */
"0N9230Z"  =  "1"  /* Drainage of Left Frontal Bone with Drain Dev, Perc Approach */
"0N923ZX"  =  "1"  /* Drainage of Left Frontal Bone, Percutaneous Approach, Diagn */
"0N923ZZ"  =  "1"  /* Drainage of Left Frontal Bone, Percutaneous Approach */
"0N9240Z"  =  "1"  /* Drain of L Frontal Bone with Drain Dev, Perc Endo Approach */
"0N924ZX"  =  "1"  /* Drainage of Left Frontal Bone, Perc Endo Approach, Diagn */
"0N924ZZ"  =  "1"  /* Drainage of Left Frontal Bone, Perc Endo Approach */
"0N9300Z"  =  "1"  /* Drainage of R Parietal Bone with Drain Dev, Open Approach */
"0N930ZX"  =  "1"  /* Drainage of Right Parietal Bone, Open Approach, Diagnostic */
"0N930ZZ"  =  "1"  /* Drainage of Right Parietal Bone, Open Approach */
"0N9330Z"  =  "1"  /* Drainage of R Parietal Bone with Drain Dev, Perc Approach */
"0N933ZX"  =  "1"  /* Drainage of Right Parietal Bone, Perc Approach, Diagn */
"0N933ZZ"  =  "1"  /* Drainage of Right Parietal Bone, Percutaneous Approach */
"0N9340Z"  =  "1"  /* Drain of R Parietal Bone with Drain Dev, Perc Endo Approach */
"0N934ZX"  =  "1"  /* Drainage of Right Parietal Bone, Perc Endo Approach, Diagn */
"0N934ZZ"  =  "1"  /* Drainage of Right Parietal Bone, Perc Endo Approach */
"0N9400Z"  =  "1"  /* Drainage of Left Parietal Bone with Drain Dev, Open Approach */
"0N940ZX"  =  "1"  /* Drainage of Left Parietal Bone, Open Approach, Diagnostic */
"0N940ZZ"  =  "1"  /* Drainage of Left Parietal Bone, Open Approach */
"0N9430Z"  =  "1"  /* Drainage of Left Parietal Bone with Drain Dev, Perc Approach */
"0N943ZX"  =  "1"  /* Drainage of Left Parietal Bone, Percutaneous Approach, Diagn */
"0N943ZZ"  =  "1"  /* Drainage of Left Parietal Bone, Percutaneous Approach */
"0N9440Z"  =  "1"  /* Drain of L Parietal Bone with Drain Dev, Perc Endo Approach */
"0N944ZX"  =  "1"  /* Drainage of Left Parietal Bone, Perc Endo Approach, Diagn */
"0N944ZZ"  =  "1"  /* Drainage of Left Parietal Bone, Perc Endo Approach */
"0N9500Z"  =  "1"  /* Drainage of R Temporal Bone with Drain Dev, Open Approach */
"0N950ZX"  =  "1"  /* Drainage of Right Temporal Bone, Open Approach, Diagnostic */
"0N950ZZ"  =  "1"  /* Drainage of Right Temporal Bone, Open Approach */
"0N9530Z"  =  "1"  /* Drainage of R Temporal Bone with Drain Dev, Perc Approach */
"0N953ZX"  =  "1"  /* Drainage of Right Temporal Bone, Perc Approach, Diagn */
"0N953ZZ"  =  "1"  /* Drainage of Right Temporal Bone, Percutaneous Approach */
"0N9540Z"  =  "1"  /* Drain of R Temporal Bone with Drain Dev, Perc Endo Approach */
"0N954ZX"  =  "1"  /* Drainage of Right Temporal Bone, Perc Endo Approach, Diagn */
"0N954ZZ"  =  "1"  /* Drainage of Right Temporal Bone, Perc Endo Approach */
"0N9600Z"  =  "1"  /* Drainage of Left Temporal Bone with Drain Dev, Open Approach */
"0N960ZX"  =  "1"  /* Drainage of Left Temporal Bone, Open Approach, Diagnostic */
"0N960ZZ"  =  "1"  /* Drainage of Left Temporal Bone, Open Approach */
"0N9630Z"  =  "1"  /* Drainage of Left Temporal Bone with Drain Dev, Perc Approach */
"0N963ZX"  =  "1"  /* Drainage of Left Temporal Bone, Percutaneous Approach, Diagn */
"0N963ZZ"  =  "1"  /* Drainage of Left Temporal Bone, Percutaneous Approach */
"0N9640Z"  =  "1"  /* Drain of L Temporal Bone with Drain Dev, Perc Endo Approach */
"0N964ZX"  =  "1"  /* Drainage of Left Temporal Bone, Perc Endo Approach, Diagn */
"0N964ZZ"  =  "1"  /* Drainage of Left Temporal Bone, Perc Endo Approach */
"0N9700Z"  =  "1"  /* Drainage of R Occipital Bone with Drain Dev, Open Approach */
"0N970ZX"  =  "1"  /* Drainage of Right Occipital Bone, Open Approach, Diagnostic */
"0N970ZZ"  =  "1"  /* Drainage of Right Occipital Bone, Open Approach */
"0N9730Z"  =  "1"  /* Drainage of R Occipital Bone with Drain Dev, Perc Approach */
"0N973ZX"  =  "1"  /* Drainage of Right Occipital Bone, Perc Approach, Diagn */
"0N973ZZ"  =  "1"  /* Drainage of Right Occipital Bone, Percutaneous Approach */
"0N9740Z"  =  "1"  /* Drain of R Occipital Bone with Drain Dev, Perc Endo Approach */
"0N974ZX"  =  "1"  /* Drainage of Right Occipital Bone, Perc Endo Approach, Diagn */
"0N974ZZ"  =  "1"  /* Drainage of Right Occipital Bone, Perc Endo Approach */
"0N9800Z"  =  "1"  /* Drainage of L Occipital Bone with Drain Dev, Open Approach */
"0N980ZX"  =  "1"  /* Drainage of Left Occipital Bone, Open Approach, Diagnostic */
"0N980ZZ"  =  "1"  /* Drainage of Left Occipital Bone, Open Approach */
"0N9830Z"  =  "1"  /* Drainage of L Occipital Bone with Drain Dev, Perc Approach */
"0N983ZX"  =  "1"  /* Drainage of Left Occipital Bone, Perc Approach, Diagn */
"0N983ZZ"  =  "1"  /* Drainage of Left Occipital Bone, Percutaneous Approach */
"0N9840Z"  =  "1"  /* Drain of L Occipital Bone with Drain Dev, Perc Endo Approach */
"0N984ZX"  =  "1"  /* Drainage of Left Occipital Bone, Perc Endo Approach, Diagn */
"0N984ZZ"  =  "1"  /* Drainage of Left Occipital Bone, Perc Endo Approach */
"0N9C00Z"  =  "1"  /* Drainage of R Sphenoid Bone with Drain Dev, Open Approach */
"0N9C0ZX"  =  "1"  /* Drainage of Right Sphenoid Bone, Open Approach, Diagnostic */
"0N9C0ZZ"  =  "1"  /* Drainage of Right Sphenoid Bone, Open Approach */
"0N9C30Z"  =  "1"  /* Drainage of R Sphenoid Bone with Drain Dev, Perc Approach */
"0N9C3ZX"  =  "1"  /* Drainage of Right Sphenoid Bone, Perc Approach, Diagn */
"0N9C3ZZ"  =  "1"  /* Drainage of Right Sphenoid Bone, Percutaneous Approach */
"0N9C40Z"  =  "1"  /* Drain of R Sphenoid Bone with Drain Dev, Perc Endo Approach */
"0N9C4ZX"  =  "1"  /* Drainage of Right Sphenoid Bone, Perc Endo Approach, Diagn */
"0N9C4ZZ"  =  "1"  /* Drainage of Right Sphenoid Bone, Perc Endo Approach */
"0N9D00Z"  =  "1"  /* Drainage of Left Sphenoid Bone with Drain Dev, Open Approach */
"0N9D0ZX"  =  "1"  /* Drainage of Left Sphenoid Bone, Open Approach, Diagnostic */
"0N9D0ZZ"  =  "1"  /* Drainage of Left Sphenoid Bone, Open Approach */
"0N9D30Z"  =  "1"  /* Drainage of Left Sphenoid Bone with Drain Dev, Perc Approach */
"0N9D3ZX"  =  "1"  /* Drainage of Left Sphenoid Bone, Percutaneous Approach, Diagn */
"0N9D3ZZ"  =  "1"  /* Drainage of Left Sphenoid Bone, Percutaneous Approach */
"0N9D40Z"  =  "1"  /* Drain of L Sphenoid Bone with Drain Dev, Perc Endo Approach */
"0N9D4ZX"  =  "1"  /* Drainage of Left Sphenoid Bone, Perc Endo Approach, Diagn */
"0N9D4ZZ"  =  "1"  /* Drainage of Left Sphenoid Bone, Perc Endo Approach */
"0N9F00Z"  =  "1"  /* Drainage of Right Ethmoid Bone with Drain Dev, Open Approach */
"0N9F0ZX"  =  "1"  /* Drainage of Right Ethmoid Bone, Open Approach, Diagnostic */
"0N9F0ZZ"  =  "1"  /* Drainage of Right Ethmoid Bone, Open Approach */
"0N9F30Z"  =  "1"  /* Drainage of Right Ethmoid Bone with Drain Dev, Perc Approach */
"0N9F3ZX"  =  "1"  /* Drainage of Right Ethmoid Bone, Percutaneous Approach, Diagn */
"0N9F3ZZ"  =  "1"  /* Drainage of Right Ethmoid Bone, Percutaneous Approach */
"0N9F40Z"  =  "1"  /* Drain of R Ethmoid Bone with Drain Dev, Perc Endo Approach */
"0N9F4ZX"  =  "1"  /* Drainage of Right Ethmoid Bone, Perc Endo Approach, Diagn */
"0N9F4ZZ"  =  "1"  /* Drainage of Right Ethmoid Bone, Perc Endo Approach */
"0N9G00Z"  =  "1"  /* Drainage of Left Ethmoid Bone with Drain Dev, Open Approach */
"0N9G0ZX"  =  "1"  /* Drainage of Left Ethmoid Bone, Open Approach, Diagnostic */
"0N9G0ZZ"  =  "1"  /* Drainage of Left Ethmoid Bone, Open Approach */
"0N9G30Z"  =  "1"  /* Drainage of Left Ethmoid Bone with Drain Dev, Perc Approach */
"0N9G3ZX"  =  "1"  /* Drainage of Left Ethmoid Bone, Percutaneous Approach, Diagn */
"0N9G3ZZ"  =  "1"  /* Drainage of Left Ethmoid Bone, Percutaneous Approach */
"0N9G40Z"  =  "1"  /* Drain of L Ethmoid Bone with Drain Dev, Perc Endo Approach */
"0N9G4ZX"  =  "1"  /* Drainage of Left Ethmoid Bone, Perc Endo Approach, Diagn */
"0N9G4ZZ"  =  "1"  /* Drainage of Left Ethmoid Bone, Perc Endo Approach */
"0N9H00Z"  =  "1"  /* Drainage of R Lacrimal Bone with Drain Dev, Open Approach */
"0N9H0ZX"  =  "1"  /* Drainage of Right Lacrimal Bone, Open Approach, Diagnostic */
"0N9H0ZZ"  =  "1"  /* Drainage of Right Lacrimal Bone, Open Approach */
"0N9H30Z"  =  "1"  /* Drainage of R Lacrimal Bone with Drain Dev, Perc Approach */
"0N9H3ZX"  =  "1"  /* Drainage of Right Lacrimal Bone, Perc Approach, Diagn */
"0N9H3ZZ"  =  "1"  /* Drainage of Right Lacrimal Bone, Percutaneous Approach */
"0N9H40Z"  =  "1"  /* Drain of R Lacrimal Bone with Drain Dev, Perc Endo Approach */
"0N9H4ZX"  =  "1"  /* Drainage of Right Lacrimal Bone, Perc Endo Approach, Diagn */
"0N9H4ZZ"  =  "1"  /* Drainage of Right Lacrimal Bone, Perc Endo Approach */
"0N9J00Z"  =  "1"  /* Drainage of Left Lacrimal Bone with Drain Dev, Open Approach */
"0N9J0ZX"  =  "1"  /* Drainage of Left Lacrimal Bone, Open Approach, Diagnostic */
"0N9J0ZZ"  =  "1"  /* Drainage of Left Lacrimal Bone, Open Approach */
"0N9J30Z"  =  "1"  /* Drainage of Left Lacrimal Bone with Drain Dev, Perc Approach */
"0N9J3ZX"  =  "1"  /* Drainage of Left Lacrimal Bone, Percutaneous Approach, Diagn */
"0N9J3ZZ"  =  "1"  /* Drainage of Left Lacrimal Bone, Percutaneous Approach */
"0N9J40Z"  =  "1"  /* Drain of L Lacrimal Bone with Drain Dev, Perc Endo Approach */
"0N9J4ZX"  =  "1"  /* Drainage of Left Lacrimal Bone, Perc Endo Approach, Diagn */
"0N9J4ZZ"  =  "1"  /* Drainage of Left Lacrimal Bone, Perc Endo Approach */
"0N9K00Z"  =  "1"  /* Drainage of R Palatine Bone with Drain Dev, Open Approach */
"0N9K0ZX"  =  "1"  /* Drainage of Right Palatine Bone, Open Approach, Diagnostic */
"0N9K0ZZ"  =  "1"  /* Drainage of Right Palatine Bone, Open Approach */
"0N9K30Z"  =  "1"  /* Drainage of R Palatine Bone with Drain Dev, Perc Approach */
"0N9K3ZX"  =  "1"  /* Drainage of Right Palatine Bone, Perc Approach, Diagn */
"0N9K3ZZ"  =  "1"  /* Drainage of Right Palatine Bone, Percutaneous Approach */
"0N9K40Z"  =  "1"  /* Drain of R Palatine Bone with Drain Dev, Perc Endo Approach */
"0N9K4ZX"  =  "1"  /* Drainage of Right Palatine Bone, Perc Endo Approach, Diagn */
"0N9K4ZZ"  =  "1"  /* Drainage of Right Palatine Bone, Perc Endo Approach */
"0N9L00Z"  =  "1"  /* Drainage of Left Palatine Bone with Drain Dev, Open Approach */
"0N9L0ZX"  =  "1"  /* Drainage of Left Palatine Bone, Open Approach, Diagnostic */
"0N9L0ZZ"  =  "1"  /* Drainage of Left Palatine Bone, Open Approach */
"0N9L30Z"  =  "1"  /* Drainage of Left Palatine Bone with Drain Dev, Perc Approach */
"0N9L3ZX"  =  "1"  /* Drainage of Left Palatine Bone, Percutaneous Approach, Diagn */
"0N9L3ZZ"  =  "1"  /* Drainage of Left Palatine Bone, Percutaneous Approach */
"0N9L40Z"  =  "1"  /* Drain of L Palatine Bone with Drain Dev, Perc Endo Approach */
"0N9L4ZX"  =  "1"  /* Drainage of Left Palatine Bone, Perc Endo Approach, Diagn */
"0N9L4ZZ"  =  "1"  /* Drainage of Left Palatine Bone, Perc Endo Approach */
"0N9M00Z"  =  "1"  /* Drainage of R Zygomatic Bone with Drain Dev, Open Approach */
"0N9M0ZX"  =  "1"  /* Drainage of Right Zygomatic Bone, Open Approach, Diagnostic */
"0N9M0ZZ"  =  "1"  /* Drainage of Right Zygomatic Bone, Open Approach */
"0N9M30Z"  =  "1"  /* Drainage of R Zygomatic Bone with Drain Dev, Perc Approach */
"0N9M3ZX"  =  "1"  /* Drainage of Right Zygomatic Bone, Perc Approach, Diagn */
"0N9M3ZZ"  =  "1"  /* Drainage of Right Zygomatic Bone, Percutaneous Approach */
"0N9M40Z"  =  "1"  /* Drain of R Zygomatic Bone with Drain Dev, Perc Endo Approach */
"0N9M4ZX"  =  "1"  /* Drainage of Right Zygomatic Bone, Perc Endo Approach, Diagn */
"0N9M4ZZ"  =  "1"  /* Drainage of Right Zygomatic Bone, Perc Endo Approach */
"0N9N00Z"  =  "1"  /* Drainage of L Zygomatic Bone with Drain Dev, Open Approach */
"0N9N0ZX"  =  "1"  /* Drainage of Left Zygomatic Bone, Open Approach, Diagnostic */
"0N9N0ZZ"  =  "1"  /* Drainage of Left Zygomatic Bone, Open Approach */
"0N9N30Z"  =  "1"  /* Drainage of L Zygomatic Bone with Drain Dev, Perc Approach */
"0N9N3ZX"  =  "1"  /* Drainage of Left Zygomatic Bone, Perc Approach, Diagn */
"0N9N3ZZ"  =  "1"  /* Drainage of Left Zygomatic Bone, Percutaneous Approach */
"0N9N40Z"  =  "1"  /* Drain of L Zygomatic Bone with Drain Dev, Perc Endo Approach */
"0N9N4ZX"  =  "1"  /* Drainage of Left Zygomatic Bone, Perc Endo Approach, Diagn */
"0N9N4ZZ"  =  "1"  /* Drainage of Left Zygomatic Bone, Perc Endo Approach */
"0N9P00Z"  =  "1"  /* Drainage of Right Orbit with Drainage Device, Open Approach */
"0N9P0ZX"  =  "1"  /* Drainage of Right Orbit, Open Approach, Diagnostic */
"0N9P0ZZ"  =  "1"  /* Drainage of Right Orbit, Open Approach */
"0N9P30Z"  =  "1"  /* Drainage of Right Orbit with Drainage Device, Perc Approach */
"0N9P3ZX"  =  "1"  /* Drainage of Right Orbit, Percutaneous Approach, Diagnostic */
"0N9P3ZZ"  =  "1"  /* Drainage of Right Orbit, Percutaneous Approach */
"0N9P40Z"  =  "1"  /* Drainage of Right Orbit with Drain Dev, Perc Endo Approach */
"0N9P4ZX"  =  "1"  /* Drainage of Right Orbit, Perc Endo Approach, Diagn */
"0N9P4ZZ"  =  "1"  /* Drainage of Right Orbit, Percutaneous Endoscopic Approach */
"0N9Q00Z"  =  "1"  /* Drainage of Left Orbit with Drainage Device, Open Approach */
"0N9Q0ZX"  =  "1"  /* Drainage of Left Orbit, Open Approach, Diagnostic */
"0N9Q0ZZ"  =  "1"  /* Drainage of Left Orbit, Open Approach */
"0N9Q30Z"  =  "1"  /* Drainage of Left Orbit with Drainage Device, Perc Approach */
"0N9Q3ZX"  =  "1"  /* Drainage of Left Orbit, Percutaneous Approach, Diagnostic */
"0N9Q3ZZ"  =  "1"  /* Drainage of Left Orbit, Percutaneous Approach */
"0N9Q40Z"  =  "1"  /* Drainage of Left Orbit with Drain Dev, Perc Endo Approach */
"0N9Q4ZX"  =  "1"  /* Drainage of Left Orbit, Perc Endo Approach, Diagn */
"0N9Q4ZZ"  =  "1"  /* Drainage of Left Orbit, Percutaneous Endoscopic Approach */
"0N9R0ZX"  =  "1"  /* Drainage of Right Maxilla, Open Approach, Diagnostic */
"0N9R3ZX"  =  "1"  /* Drainage of Right Maxilla, Percutaneous Approach, Diagnostic */
"0N9R4ZX"  =  "1"  /* Drainage of Right Maxilla, Perc Endo Approach, Diagn */
"0N9S0ZX"  =  "1"  /* Drainage of Left Maxilla, Open Approach, Diagnostic */
"0N9S3ZX"  =  "1"  /* Drainage of Left Maxilla, Percutaneous Approach, Diagnostic */
"0N9S4ZX"  =  "1"  /* Drainage of Left Maxilla, Perc Endo Approach, Diagn */
"0N9T0ZX"  =  "1"  /* Drainage of Right Mandible, Open Approach, Diagnostic */
"0N9T3ZX"  =  "1"  /* Drainage of Right Mandible, Percutaneous Approach, Diagn */
"0N9T4ZX"  =  "1"  /* Drainage of Right Mandible, Perc Endo Approach, Diagn */
"0N9V0ZX"  =  "1"  /* Drainage of Left Mandible, Open Approach, Diagnostic */
"0N9V3ZX"  =  "1"  /* Drainage of Left Mandible, Percutaneous Approach, Diagnostic */
"0N9V4ZX"  =  "1"  /* Drainage of Left Mandible, Perc Endo Approach, Diagn */
"0N9X00Z"  =  "1"  /* Drainage of Hyoid Bone with Drainage Device, Open Approach */
"0N9X0ZX"  =  "1"  /* Drainage of Hyoid Bone, Open Approach, Diagnostic */
"0N9X0ZZ"  =  "1"  /* Drainage of Hyoid Bone, Open Approach */
"0N9X30Z"  =  "1"  /* Drainage of Hyoid Bone with Drainage Device, Perc Approach */
"0N9X3ZX"  =  "1"  /* Drainage of Hyoid Bone, Percutaneous Approach, Diagnostic */
"0N9X3ZZ"  =  "1"  /* Drainage of Hyoid Bone, Percutaneous Approach */
"0N9X40Z"  =  "1"  /* Drainage of Hyoid Bone with Drain Dev, Perc Endo Approach */
"0N9X4ZX"  =  "1"  /* Drainage of Hyoid Bone, Perc Endo Approach, Diagn */
"0N9X4ZZ"  =  "1"  /* Drainage of Hyoid Bone, Percutaneous Endoscopic Approach */
"0NB00ZX"  =  "1"  /* Excision of Skull, Open Approach, Diagnostic */
"0NB00ZZ"  =  "1"  /* Excision of Skull, Open Approach */
"0NB03ZX"  =  "1"  /* Excision of Skull, Percutaneous Approach, Diagnostic */
"0NB03ZZ"  =  "1"  /* Excision of Skull, Percutaneous Approach */
"0NB04ZX"  =  "1"  /* Excision of Skull, Percutaneous Endoscopic Approach, Diagn */
"0NB04ZZ"  =  "1"  /* Excision of Skull, Percutaneous Endoscopic Approach */
"0NB10ZX"  =  "1"  /* Excision of Right Frontal Bone, Open Approach, Diagnostic */
"0NB10ZZ"  =  "1"  /* Excision of Right Frontal Bone, Open Approach */
"0NB13ZX"  =  "1"  /* Excision of Right Frontal Bone, Percutaneous Approach, Diagn */
"0NB13ZZ"  =  "1"  /* Excision of Right Frontal Bone, Percutaneous Approach */
"0NB14ZX"  =  "1"  /* Excision of Right Frontal Bone, Perc Endo Approach, Diagn */
"0NB14ZZ"  =  "1"  /* Excision of Right Frontal Bone, Perc Endo Approach */
"0NB20ZX"  =  "1"  /* Excision of Left Frontal Bone, Open Approach, Diagnostic */
"0NB20ZZ"  =  "1"  /* Excision of Left Frontal Bone, Open Approach */
"0NB23ZX"  =  "1"  /* Excision of Left Frontal Bone, Percutaneous Approach, Diagn */
"0NB23ZZ"  =  "1"  /* Excision of Left Frontal Bone, Percutaneous Approach */
"0NB24ZX"  =  "1"  /* Excision of Left Frontal Bone, Perc Endo Approach, Diagn */
"0NB24ZZ"  =  "1"  /* Excision of Left Frontal Bone, Perc Endo Approach */
"0NB30ZX"  =  "1"  /* Excision of Right Parietal Bone, Open Approach, Diagnostic */
"0NB30ZZ"  =  "1"  /* Excision of Right Parietal Bone, Open Approach */
"0NB33ZX"  =  "1"  /* Excision of Right Parietal Bone, Perc Approach, Diagn */
"0NB33ZZ"  =  "1"  /* Excision of Right Parietal Bone, Percutaneous Approach */
"0NB34ZX"  =  "1"  /* Excision of Right Parietal Bone, Perc Endo Approach, Diagn */
"0NB34ZZ"  =  "1"  /* Excision of Right Parietal Bone, Perc Endo Approach */
"0NB40ZX"  =  "1"  /* Excision of Left Parietal Bone, Open Approach, Diagnostic */
"0NB40ZZ"  =  "1"  /* Excision of Left Parietal Bone, Open Approach */
"0NB43ZX"  =  "1"  /* Excision of Left Parietal Bone, Percutaneous Approach, Diagn */
"0NB43ZZ"  =  "1"  /* Excision of Left Parietal Bone, Percutaneous Approach */
"0NB44ZX"  =  "1"  /* Excision of Left Parietal Bone, Perc Endo Approach, Diagn */
"0NB44ZZ"  =  "1"  /* Excision of Left Parietal Bone, Perc Endo Approach */
"0NB50ZX"  =  "1"  /* Excision of Right Temporal Bone, Open Approach, Diagnostic */
"0NB50ZZ"  =  "1"  /* Excision of Right Temporal Bone, Open Approach */
"0NB53ZX"  =  "1"  /* Excision of Right Temporal Bone, Perc Approach, Diagn */
"0NB53ZZ"  =  "1"  /* Excision of Right Temporal Bone, Percutaneous Approach */
"0NB54ZX"  =  "1"  /* Excision of Right Temporal Bone, Perc Endo Approach, Diagn */
"0NB54ZZ"  =  "1"  /* Excision of Right Temporal Bone, Perc Endo Approach */
"0NB60ZX"  =  "1"  /* Excision of Left Temporal Bone, Open Approach, Diagnostic */
"0NB60ZZ"  =  "1"  /* Excision of Left Temporal Bone, Open Approach */
"0NB63ZX"  =  "1"  /* Excision of Left Temporal Bone, Percutaneous Approach, Diagn */
"0NB63ZZ"  =  "1"  /* Excision of Left Temporal Bone, Percutaneous Approach */
"0NB64ZX"  =  "1"  /* Excision of Left Temporal Bone, Perc Endo Approach, Diagn */
"0NB64ZZ"  =  "1"  /* Excision of Left Temporal Bone, Perc Endo Approach */
"0NB70ZX"  =  "1"  /* Excision of Right Occipital Bone, Open Approach, Diagnostic */
"0NB70ZZ"  =  "1"  /* Excision of Right Occipital Bone, Open Approach */
"0NB73ZX"  =  "1"  /* Excision of Right Occipital Bone, Perc Approach, Diagn */
"0NB73ZZ"  =  "1"  /* Excision of Right Occipital Bone, Percutaneous Approach */
"0NB74ZX"  =  "1"  /* Excision of Right Occipital Bone, Perc Endo Approach, Diagn */
"0NB74ZZ"  =  "1"  /* Excision of Right Occipital Bone, Perc Endo Approach */
"0NB80ZX"  =  "1"  /* Excision of Left Occipital Bone, Open Approach, Diagnostic */
"0NB80ZZ"  =  "1"  /* Excision of Left Occipital Bone, Open Approach */
"0NB83ZX"  =  "1"  /* Excision of Left Occipital Bone, Perc Approach, Diagn */
"0NB83ZZ"  =  "1"  /* Excision of Left Occipital Bone, Percutaneous Approach */
"0NB84ZX"  =  "1"  /* Excision of Left Occipital Bone, Perc Endo Approach, Diagn */
"0NB84ZZ"  =  "1"  /* Excision of Left Occipital Bone, Perc Endo Approach */
"0NBB0ZZ"  =  "1"  /* Excision of Nasal Bone, Open Approach */
"0NBB3ZZ"  =  "1"  /* Excision of Nasal Bone, Percutaneous Approach */
"0NBB4ZZ"  =  "1"  /* Excision of Nasal Bone, Percutaneous Endoscopic Approach */
"0NBC0ZX"  =  "1"  /* Excision of Right Sphenoid Bone, Open Approach, Diagnostic */
"0NBC0ZZ"  =  "1"  /* Excision of Right Sphenoid Bone, Open Approach */
"0NBC3ZX"  =  "1"  /* Excision of Right Sphenoid Bone, Perc Approach, Diagn */
"0NBC3ZZ"  =  "1"  /* Excision of Right Sphenoid Bone, Percutaneous Approach */
"0NBC4ZX"  =  "1"  /* Excision of Right Sphenoid Bone, Perc Endo Approach, Diagn */
"0NBC4ZZ"  =  "1"  /* Excision of Right Sphenoid Bone, Perc Endo Approach */
"0NBD0ZX"  =  "1"  /* Excision of Left Sphenoid Bone, Open Approach, Diagnostic */
"0NBD0ZZ"  =  "1"  /* Excision of Left Sphenoid Bone, Open Approach */
"0NBD3ZX"  =  "1"  /* Excision of Left Sphenoid Bone, Percutaneous Approach, Diagn */
"0NBD3ZZ"  =  "1"  /* Excision of Left Sphenoid Bone, Percutaneous Approach */
"0NBD4ZX"  =  "1"  /* Excision of Left Sphenoid Bone, Perc Endo Approach, Diagn */
"0NBD4ZZ"  =  "1"  /* Excision of Left Sphenoid Bone, Perc Endo Approach */
"0NBF0ZX"  =  "1"  /* Excision of Right Ethmoid Bone, Open Approach, Diagnostic */
"0NBF0ZZ"  =  "1"  /* Excision of Right Ethmoid Bone, Open Approach */
"0NBF3ZX"  =  "1"  /* Excision of Right Ethmoid Bone, Percutaneous Approach, Diagn */
"0NBF3ZZ"  =  "1"  /* Excision of Right Ethmoid Bone, Percutaneous Approach */
"0NBF4ZX"  =  "1"  /* Excision of Right Ethmoid Bone, Perc Endo Approach, Diagn */
"0NBF4ZZ"  =  "1"  /* Excision of Right Ethmoid Bone, Perc Endo Approach */
"0NBG0ZX"  =  "1"  /* Excision of Left Ethmoid Bone, Open Approach, Diagnostic */
"0NBG0ZZ"  =  "1"  /* Excision of Left Ethmoid Bone, Open Approach */
"0NBG3ZX"  =  "1"  /* Excision of Left Ethmoid Bone, Percutaneous Approach, Diagn */
"0NBG3ZZ"  =  "1"  /* Excision of Left Ethmoid Bone, Percutaneous Approach */
"0NBG4ZX"  =  "1"  /* Excision of Left Ethmoid Bone, Perc Endo Approach, Diagn */
"0NBG4ZZ"  =  "1"  /* Excision of Left Ethmoid Bone, Perc Endo Approach */
"0NBH0ZX"  =  "1"  /* Excision of Right Lacrimal Bone, Open Approach, Diagnostic */
"0NBH0ZZ"  =  "1"  /* Excision of Right Lacrimal Bone, Open Approach */
"0NBH3ZX"  =  "1"  /* Excision of Right Lacrimal Bone, Perc Approach, Diagn */
"0NBH3ZZ"  =  "1"  /* Excision of Right Lacrimal Bone, Percutaneous Approach */
"0NBH4ZX"  =  "1"  /* Excision of Right Lacrimal Bone, Perc Endo Approach, Diagn */
"0NBH4ZZ"  =  "1"  /* Excision of Right Lacrimal Bone, Perc Endo Approach */
"0NBJ0ZX"  =  "1"  /* Excision of Left Lacrimal Bone, Open Approach, Diagnostic */
"0NBJ0ZZ"  =  "1"  /* Excision of Left Lacrimal Bone, Open Approach */
"0NBJ3ZX"  =  "1"  /* Excision of Left Lacrimal Bone, Percutaneous Approach, Diagn */
"0NBJ3ZZ"  =  "1"  /* Excision of Left Lacrimal Bone, Percutaneous Approach */
"0NBJ4ZX"  =  "1"  /* Excision of Left Lacrimal Bone, Perc Endo Approach, Diagn */
"0NBJ4ZZ"  =  "1"  /* Excision of Left Lacrimal Bone, Perc Endo Approach */
"0NBK0ZX"  =  "1"  /* Excision of Right Palatine Bone, Open Approach, Diagnostic */
"0NBK0ZZ"  =  "1"  /* Excision of Right Palatine Bone, Open Approach */
"0NBK3ZX"  =  "1"  /* Excision of Right Palatine Bone, Perc Approach, Diagn */
"0NBK3ZZ"  =  "1"  /* Excision of Right Palatine Bone, Percutaneous Approach */
"0NBK4ZX"  =  "1"  /* Excision of Right Palatine Bone, Perc Endo Approach, Diagn */
"0NBK4ZZ"  =  "1"  /* Excision of Right Palatine Bone, Perc Endo Approach */
"0NBL0ZX"  =  "1"  /* Excision of Left Palatine Bone, Open Approach, Diagnostic */
"0NBL0ZZ"  =  "1"  /* Excision of Left Palatine Bone, Open Approach */
"0NBL3ZX"  =  "1"  /* Excision of Left Palatine Bone, Percutaneous Approach, Diagn */
"0NBL3ZZ"  =  "1"  /* Excision of Left Palatine Bone, Percutaneous Approach */
"0NBL4ZX"  =  "1"  /* Excision of Left Palatine Bone, Perc Endo Approach, Diagn */
"0NBL4ZZ"  =  "1"  /* Excision of Left Palatine Bone, Perc Endo Approach */
"0NBM0ZX"  =  "1"  /* Excision of Right Zygomatic Bone, Open Approach, Diagnostic */
"0NBM0ZZ"  =  "1"  /* Excision of Right Zygomatic Bone, Open Approach */
"0NBM3ZX"  =  "1"  /* Excision of Right Zygomatic Bone, Perc Approach, Diagn */
"0NBM3ZZ"  =  "1"  /* Excision of Right Zygomatic Bone, Percutaneous Approach */
"0NBM4ZX"  =  "1"  /* Excision of Right Zygomatic Bone, Perc Endo Approach, Diagn */
"0NBM4ZZ"  =  "1"  /* Excision of Right Zygomatic Bone, Perc Endo Approach */
"0NBN0ZX"  =  "1"  /* Excision of Left Zygomatic Bone, Open Approach, Diagnostic */
"0NBN0ZZ"  =  "1"  /* Excision of Left Zygomatic Bone, Open Approach */
"0NBN3ZX"  =  "1"  /* Excision of Left Zygomatic Bone, Perc Approach, Diagn */
"0NBN3ZZ"  =  "1"  /* Excision of Left Zygomatic Bone, Percutaneous Approach */
"0NBN4ZX"  =  "1"  /* Excision of Left Zygomatic Bone, Perc Endo Approach, Diagn */
"0NBN4ZZ"  =  "1"  /* Excision of Left Zygomatic Bone, Perc Endo Approach */
"0NBP0ZX"  =  "1"  /* Excision of Right Orbit, Open Approach, Diagnostic */
"0NBP0ZZ"  =  "1"  /* Excision of Right Orbit, Open Approach */
"0NBP3ZX"  =  "1"  /* Excision of Right Orbit, Percutaneous Approach, Diagnostic */
"0NBP3ZZ"  =  "1"  /* Excision of Right Orbit, Percutaneous Approach */
"0NBP4ZX"  =  "1"  /* Excision of Right Orbit, Perc Endo Approach, Diagn */
"0NBP4ZZ"  =  "1"  /* Excision of Right Orbit, Percutaneous Endoscopic Approach */
"0NBQ0ZX"  =  "1"  /* Excision of Left Orbit, Open Approach, Diagnostic */
"0NBQ0ZZ"  =  "1"  /* Excision of Left Orbit, Open Approach */
"0NBQ3ZX"  =  "1"  /* Excision of Left Orbit, Percutaneous Approach, Diagnostic */
"0NBQ3ZZ"  =  "1"  /* Excision of Left Orbit, Percutaneous Approach */
"0NBQ4ZX"  =  "1"  /* Excision of Left Orbit, Perc Endo Approach, Diagn */
"0NBQ4ZZ"  =  "1"  /* Excision of Left Orbit, Percutaneous Endoscopic Approach */
"0NBR0ZZ"  =  "1"  /* Excision of Right Maxilla, Open Approach */
"0NBR3ZZ"  =  "1"  /* Excision of Right Maxilla, Percutaneous Approach */
"0NBR4ZZ"  =  "1"  /* Excision of Right Maxilla, Percutaneous Endoscopic Approach */
"0NBS0ZZ"  =  "1"  /* Excision of Left Maxilla, Open Approach */
"0NBS3ZZ"  =  "1"  /* Excision of Left Maxilla, Percutaneous Approach */
"0NBS4ZZ"  =  "1"  /* Excision of Left Maxilla, Percutaneous Endoscopic Approach */
"0NBT0ZZ"  =  "1"  /* Excision of Right Mandible, Open Approach */
"0NBT3ZZ"  =  "1"  /* Excision of Right Mandible, Percutaneous Approach */
"0NBT4ZZ"  =  "1"  /* Excision of Right Mandible, Percutaneous Endoscopic Approach */
"0NBV0ZZ"  =  "1"  /* Excision of Left Mandible, Open Approach */
"0NBV3ZZ"  =  "1"  /* Excision of Left Mandible, Percutaneous Approach */
"0NBV4ZZ"  =  "1"  /* Excision of Left Mandible, Percutaneous Endoscopic Approach */
"0NBX0ZX"  =  "1"  /* Excision of Hyoid Bone, Open Approach, Diagnostic */
"0NBX0ZZ"  =  "1"  /* Excision of Hyoid Bone, Open Approach */
"0NBX3ZX"  =  "1"  /* Excision of Hyoid Bone, Percutaneous Approach, Diagnostic */
"0NBX3ZZ"  =  "1"  /* Excision of Hyoid Bone, Percutaneous Approach */
"0NBX4ZX"  =  "1"  /* Excision of Hyoid Bone, Perc Endo Approach, Diagn */
"0NBX4ZZ"  =  "1"  /* Excision of Hyoid Bone, Percutaneous Endoscopic Approach */
"0NC10ZZ"  =  "1"  /* Extirpation of Matter from Right Frontal Bone, Open Approach */
"0NC13ZZ"  =  "1"  /* Extirpation of Matter from Right Frontal Bone, Perc Approach */
"0NC14ZZ"  =  "1"  /* Extirpate of Matter from R Frontal Bone, Perc Endo Approach */
"0NC20ZZ"  =  "1"  /* Extirpation of Matter from Left Frontal Bone, Open Approach */
"0NC23ZZ"  =  "1"  /* Extirpation of Matter from Left Frontal Bone, Perc Approach */
"0NC24ZZ"  =  "1"  /* Extirpate of Matter from L Frontal Bone, Perc Endo Approach */
"0NC30ZZ"  =  "1"  /* Extirpation of Matter from R Parietal Bone, Open Approach */
"0NC33ZZ"  =  "1"  /* Extirpation of Matter from R Parietal Bone, Perc Approach */
"0NC34ZZ"  =  "1"  /* Extirpate of Matter from R Parietal Bone, Perc Endo Approach */
"0NC40ZZ"  =  "1"  /* Extirpation of Matter from Left Parietal Bone, Open Approach */
"0NC43ZZ"  =  "1"  /* Extirpation of Matter from Left Parietal Bone, Perc Approach */
"0NC44ZZ"  =  "1"  /* Extirpate of Matter from L Parietal Bone, Perc Endo Approach */
"0NC50ZZ"  =  "1"  /* Extirpation of Matter from R Temporal Bone, Open Approach */
"0NC53ZZ"  =  "1"  /* Extirpation of Matter from R Temporal Bone, Perc Approach */
"0NC54ZZ"  =  "1"  /* Extirpate of Matter from R Temporal Bone, Perc Endo Approach */
"0NC60ZZ"  =  "1"  /* Extirpation of Matter from Left Temporal Bone, Open Approach */
"0NC63ZZ"  =  "1"  /* Extirpation of Matter from Left Temporal Bone, Perc Approach */
"0NC64ZZ"  =  "1"  /* Extirpate of Matter from L Temporal Bone, Perc Endo Approach */
"0NC70ZZ"  =  "1"  /* Extirpation of Matter from R Occipital Bone, Open Approach */
"0NC73ZZ"  =  "1"  /* Extirpation of Matter from R Occipital Bone, Perc Approach */
"0NC74ZZ"  =  "1"  /* Extirpate matter from R Occipital Bone, Perc Endo */
"0NC80ZZ"  =  "1"  /* Extirpation of Matter from L Occipital Bone, Open Approach */
"0NC83ZZ"  =  "1"  /* Extirpation of Matter from L Occipital Bone, Perc Approach */
"0NC84ZZ"  =  "1"  /* Extirpate matter from L Occipital Bone, Perc Endo */
"0NCC0ZZ"  =  "1"  /* Extirpation of Matter from R Sphenoid Bone, Open Approach */
"0NCC3ZZ"  =  "1"  /* Extirpation of Matter from R Sphenoid Bone, Perc Approach */
"0NCC4ZZ"  =  "1"  /* Extirpate of Matter from R Sphenoid Bone, Perc Endo Approach */
"0NCD0ZZ"  =  "1"  /* Extirpation of Matter from Left Sphenoid Bone, Open Approach */
"0NCD3ZZ"  =  "1"  /* Extirpation of Matter from Left Sphenoid Bone, Perc Approach */
"0NCD4ZZ"  =  "1"  /* Extirpate of Matter from L Sphenoid Bone, Perc Endo Approach */
"0NCF0ZZ"  =  "1"  /* Extirpation of Matter from Right Ethmoid Bone, Open Approach */
"0NCF3ZZ"  =  "1"  /* Extirpation of Matter from Right Ethmoid Bone, Perc Approach */
"0NCF4ZZ"  =  "1"  /* Extirpate of Matter from R Ethmoid Bone, Perc Endo Approach */
"0NCG0ZZ"  =  "1"  /* Extirpation of Matter from Left Ethmoid Bone, Open Approach */
"0NCG3ZZ"  =  "1"  /* Extirpation of Matter from Left Ethmoid Bone, Perc Approach */
"0NCG4ZZ"  =  "1"  /* Extirpate of Matter from L Ethmoid Bone, Perc Endo Approach */
"0NCH0ZZ"  =  "1"  /* Extirpation of Matter from R Lacrimal Bone, Open Approach */
"0NCH3ZZ"  =  "1"  /* Extirpation of Matter from R Lacrimal Bone, Perc Approach */
"0NCH4ZZ"  =  "1"  /* Extirpate of Matter from R Lacrimal Bone, Perc Endo Approach */
"0NCJ0ZZ"  =  "1"  /* Extirpation of Matter from Left Lacrimal Bone, Open Approach */
"0NCJ3ZZ"  =  "1"  /* Extirpation of Matter from Left Lacrimal Bone, Perc Approach */
"0NCJ4ZZ"  =  "1"  /* Extirpate of Matter from L Lacrimal Bone, Perc Endo Approach */
"0NCK0ZZ"  =  "1"  /* Extirpation of Matter from R Palatine Bone, Open Approach */
"0NCK3ZZ"  =  "1"  /* Extirpation of Matter from R Palatine Bone, Perc Approach */
"0NCK4ZZ"  =  "1"  /* Extirpate of Matter from R Palatine Bone, Perc Endo Approach */
"0NCL0ZZ"  =  "1"  /* Extirpation of Matter from Left Palatine Bone, Open Approach */
"0NCL3ZZ"  =  "1"  /* Extirpation of Matter from Left Palatine Bone, Perc Approach */
"0NCL4ZZ"  =  "1"  /* Extirpate of Matter from L Palatine Bone, Perc Endo Approach */
"0NCM0ZZ"  =  "1"  /* Extirpation of Matter from R Zygomatic Bone, Open Approach */
"0NCM3ZZ"  =  "1"  /* Extirpation of Matter from R Zygomatic Bone, Perc Approach */
"0NCM4ZZ"  =  "1"  /* Extirpate matter from R Zygomatic Bone, Perc Endo */
"0NCN0ZZ"  =  "1"  /* Extirpation of Matter from L Zygomatic Bone, Open Approach */
"0NCN3ZZ"  =  "1"  /* Extirpation of Matter from L Zygomatic Bone, Perc Approach */
"0NCN4ZZ"  =  "1"  /* Extirpate matter from L Zygomatic Bone, Perc Endo */
"0NCP0ZZ"  =  "1"  /* Extirpation of Matter from Right Orbit, Open Approach */
"0NCP3ZZ"  =  "1"  /* Extirpation of Matter from Right Orbit, Perc Approach */
"0NCP4ZZ"  =  "1"  /* Extirpation of Matter from Right Orbit, Perc Endo Approach */
"0NCQ0ZZ"  =  "1"  /* Extirpation of Matter from Left Orbit, Open Approach */
"0NCQ3ZZ"  =  "1"  /* Extirpation of Matter from Left Orbit, Percutaneous Approach */
"0NCQ4ZZ"  =  "1"  /* Extirpation of Matter from Left Orbit, Perc Endo Approach */
"0NCX0ZZ"  =  "1"  /* Extirpation of Matter from Hyoid Bone, Open Approach */
"0NCX3ZZ"  =  "1"  /* Extirpation of Matter from Hyoid Bone, Percutaneous Approach */
"0NCX4ZZ"  =  "1"  /* Extirpation of Matter from Hyoid Bone, Perc Endo Approach */
"0NH004Z"  =  "1"  /* Insertion of Int Fix into Skull, Open Approach */
"0NH00MZ"  =  "1"  /* Insertion of Bone Stim into Skull, Open Approach */
"0NH00NZ"  =  "1"  /* Insertion of Neurostim into Skull, Open Approach */
"0NH034Z"  =  "1"  /* Insertion of Int Fix into Skull, Perc Approach */
"0NH03MZ"  =  "1"  /* Insertion of Bone Stim into Skull, Perc Approach */
"0NH044Z"  =  "1"  /* Insertion of Int Fix into Skull, Perc Endo Approach */
"0NH04MZ"  =  "1"  /* Insertion of Bone Stim into Skull, Perc Endo Approach */
"0NH104Z"  =  "1"  /* Insertion of Int Fix into R Frontal Bone, Open Approach */
"0NH134Z"  =  "1"  /* Insertion of Int Fix into R Frontal Bone, Perc Approach */
"0NH144Z"  =  "1"  /* Insertion of Int Fix into R Frontal Bone, Perc Endo Approach */
"0NH204Z"  =  "1"  /* Insertion of Int Fix into L Frontal Bone, Open Approach */
"0NH234Z"  =  "1"  /* Insertion of Int Fix into L Frontal Bone, Perc Approach */
"0NH244Z"  =  "1"  /* Insertion of Int Fix into L Frontal Bone, Perc Endo Approach */
"0NH304Z"  =  "1"  /* Insertion of Int Fix into R Parietal Bone, Open Approach */
"0NH334Z"  =  "1"  /* Insertion of Int Fix into R Parietal Bone, Perc Approach */
"0NH344Z"  =  "1"  /* Insert of Int Fix into R Parietal Bone, Perc Endo Approach */
"0NH404Z"  =  "1"  /* Insertion of Int Fix into L Parietal Bone, Open Approach */
"0NH434Z"  =  "1"  /* Insertion of Int Fix into L Parietal Bone, Perc Approach */
"0NH444Z"  =  "1"  /* Insert of Int Fix into L Parietal Bone, Perc Endo Approach */
"0NH504Z"  =  "1"  /* Insertion of Int Fix into R Temporal Bone, Open Approach */
"0NH50SZ"  =  "1"  /* Insertion of Hear Dev into R Temporal Bone, Open Approach */
"0NH534Z"  =  "1"  /* Insertion of Int Fix into R Temporal Bone, Perc Approach */
"0NH53SZ"  =  "1"  /* Insertion of Hear Dev into R Temporal Bone, Perc Approach */
"0NH544Z"  =  "1"  /* Insert of Int Fix into R Temporal Bone, Perc Endo Approach */
"0NH54SZ"  =  "1"  /* Insert of Hear Dev into R Temporal Bone, Perc Endo Approach */
"0NH604Z"  =  "1"  /* Insertion of Int Fix into L Temporal Bone, Open Approach */
"0NH60SZ"  =  "1"  /* Insertion of Hear Dev into L Temporal Bone, Open Approach */
"0NH634Z"  =  "1"  /* Insertion of Int Fix into L Temporal Bone, Perc Approach */
"0NH63SZ"  =  "1"  /* Insertion of Hear Dev into L Temporal Bone, Perc Approach */
"0NH644Z"  =  "1"  /* Insert of Int Fix into L Temporal Bone, Perc Endo Approach */
"0NH64SZ"  =  "1"  /* Insert of Hear Dev into L Temporal Bone, Perc Endo Approach */
"0NH704Z"  =  "1"  /* Insertion of Int Fix into R Occipital Bone, Open Approach */
"0NH734Z"  =  "1"  /* Insertion of Int Fix into R Occipital Bone, Perc Approach */
"0NH744Z"  =  "1"  /* Insert of Int Fix into R Occipital Bone, Perc Endo Approach */
"0NH804Z"  =  "1"  /* Insertion of Int Fix into L Occipital Bone, Open Approach */
"0NH834Z"  =  "1"  /* Insertion of Int Fix into L Occipital Bone, Perc Approach */
"0NH844Z"  =  "1"  /* Insert of Int Fix into L Occipital Bone, Perc Endo Approach */
"0NHC04Z"  =  "1"  /* Insertion of Int Fix into R Sphenoid Bone, Open Approach */
"0NHC34Z"  =  "1"  /* Insertion of Int Fix into R Sphenoid Bone, Perc Approach */
"0NHC44Z"  =  "1"  /* Insert of Int Fix into R Sphenoid Bone, Perc Endo Approach */
"0NHD04Z"  =  "1"  /* Insertion of Int Fix into L Sphenoid Bone, Open Approach */
"0NHD34Z"  =  "1"  /* Insertion of Int Fix into L Sphenoid Bone, Perc Approach */
"0NHD44Z"  =  "1"  /* Insert of Int Fix into L Sphenoid Bone, Perc Endo Approach */
"0NHF04Z"  =  "1"  /* Insertion of Int Fix into R Ethmoid Bone, Open Approach */
"0NHF34Z"  =  "1"  /* Insertion of Int Fix into R Ethmoid Bone, Perc Approach */
"0NHF44Z"  =  "1"  /* Insertion of Int Fix into R Ethmoid Bone, Perc Endo Approach */
"0NHG04Z"  =  "1"  /* Insertion of Int Fix into L Ethmoid Bone, Open Approach */
"0NHG34Z"  =  "1"  /* Insertion of Int Fix into L Ethmoid Bone, Perc Approach */
"0NHG44Z"  =  "1"  /* Insertion of Int Fix into L Ethmoid Bone, Perc Endo Approach */
"0NHH04Z"  =  "1"  /* Insertion of Int Fix into R Lacrimal Bone, Open Approach */
"0NHH34Z"  =  "1"  /* Insertion of Int Fix into R Lacrimal Bone, Perc Approach */
"0NHH44Z"  =  "1"  /* Insert of Int Fix into R Lacrimal Bone, Perc Endo Approach */
"0NHJ04Z"  =  "1"  /* Insertion of Int Fix into L Lacrimal Bone, Open Approach */
"0NHJ34Z"  =  "1"  /* Insertion of Int Fix into L Lacrimal Bone, Perc Approach */
"0NHJ44Z"  =  "1"  /* Insert of Int Fix into L Lacrimal Bone, Perc Endo Approach */
"0NHK04Z"  =  "1"  /* Insertion of Int Fix into R Palatine Bone, Open Approach */
"0NHK34Z"  =  "1"  /* Insertion of Int Fix into R Palatine Bone, Perc Approach */
"0NHK44Z"  =  "1"  /* Insert of Int Fix into R Palatine Bone, Perc Endo Approach */
"0NHL04Z"  =  "1"  /* Insertion of Int Fix into L Palatine Bone, Open Approach */
"0NHL34Z"  =  "1"  /* Insertion of Int Fix into L Palatine Bone, Perc Approach */
"0NHL44Z"  =  "1"  /* Insert of Int Fix into L Palatine Bone, Perc Endo Approach */
"0NHM04Z"  =  "1"  /* Insertion of Int Fix into R Zygomatic Bone, Open Approach */
"0NHM34Z"  =  "1"  /* Insertion of Int Fix into R Zygomatic Bone, Perc Approach */
"0NHM44Z"  =  "1"  /* Insert of Int Fix into R Zygomatic Bone, Perc Endo Approach */
"0NHN04Z"  =  "1"  /* Insertion of Int Fix into L Zygomatic Bone, Open Approach */
"0NHN34Z"  =  "1"  /* Insertion of Int Fix into L Zygomatic Bone, Perc Approach */
"0NHN44Z"  =  "1"  /* Insert of Int Fix into L Zygomatic Bone, Perc Endo Approach */
"0NHP04Z"  =  "1"  /* Insertion of Int Fix into R Orbit, Open Approach */
"0NHP34Z"  =  "1"  /* Insertion of Int Fix into R Orbit, Perc Approach */
"0NHP44Z"  =  "1"  /* Insertion of Int Fix into R Orbit, Perc Endo Approach */
"0NHQ04Z"  =  "1"  /* Insertion of Int Fix into L Orbit, Open Approach */
"0NHQ34Z"  =  "1"  /* Insertion of Int Fix into L Orbit, Perc Approach */
"0NHQ44Z"  =  "1"  /* Insertion of Int Fix into L Orbit, Perc Endo Approach */
"0NHR04Z"  =  "1"  /* Insertion of Int Fix into R Maxilla, Open Approach */
"0NHR05Z"  =  "1"  /* Insertion of Ext Fix into R Maxilla, Open Approach */
"0NHR34Z"  =  "1"  /* Insertion of Int Fix into R Maxilla, Perc Approach */
"0NHR35Z"  =  "1"  /* Insertion of Ext Fix into R Maxilla, Perc Approach */
"0NHR44Z"  =  "1"  /* Insertion of Int Fix into R Maxilla, Perc Endo Approach */
"0NHR45Z"  =  "1"  /* Insertion of Ext Fix into R Maxilla, Perc Endo Approach */
"0NHS04Z"  =  "1"  /* Insertion of Int Fix into L Maxilla, Open Approach */
"0NHS05Z"  =  "1"  /* Insertion of Ext Fix into L Maxilla, Open Approach */
"0NHS34Z"  =  "1"  /* Insertion of Int Fix into L Maxilla, Perc Approach */
"0NHS35Z"  =  "1"  /* Insertion of Ext Fix into L Maxilla, Perc Approach */
"0NHS44Z"  =  "1"  /* Insertion of Int Fix into L Maxilla, Perc Endo Approach */
"0NHS45Z"  =  "1"  /* Insertion of Ext Fix into L Maxilla, Perc Endo Approach */
"0NHT04Z"  =  "1"  /* Insertion of Int Fix into R Mandible, Open Approach */
"0NHT05Z"  =  "1"  /* Insertion of Ext Fix into R Mandible, Open Approach */
"0NHT34Z"  =  "1"  /* Insertion of Int Fix into R Mandible, Perc Approach */
"0NHT35Z"  =  "1"  /* Insertion of Ext Fix into R Mandible, Perc Approach */
"0NHT44Z"  =  "1"  /* Insertion of Int Fix into R Mandible, Perc Endo Approach */
"0NHT45Z"  =  "1"  /* Insertion of Ext Fix into R Mandible, Perc Endo Approach */
"0NHV04Z"  =  "1"  /* Insertion of Int Fix into L Mandible, Open Approach */
"0NHV05Z"  =  "1"  /* Insertion of Ext Fix into L Mandible, Open Approach */
"0NHV34Z"  =  "1"  /* Insertion of Int Fix into L Mandible, Perc Approach */
"0NHV35Z"  =  "1"  /* Insertion of Ext Fix into L Mandible, Perc Approach */
"0NHV44Z"  =  "1"  /* Insertion of Int Fix into L Mandible, Perc Endo Approach */
"0NHV45Z"  =  "1"  /* Insertion of Ext Fix into L Mandible, Perc Endo Approach */
"0NHW0MZ"  =  "1"  /* Insertion of Bone Stim into Facial Bone, Open Approach */
"0NHW3MZ"  =  "1"  /* Insertion of Bone Stim into Facial Bone, Perc Approach */
"0NHW4MZ"  =  "1"  /* Insertion of Bone Stim into Facial Bone, Perc Endo Approach */
"0NHX04Z"  =  "1"  /* Insertion of Int Fix into Hyoid Bone, Open Approach */
"0NHX34Z"  =  "1"  /* Insertion of Int Fix into Hyoid Bone, Perc Approach */
"0NHX44Z"  =  "1"  /* Insertion of Int Fix into Hyoid Bone, Perc Endo Approach */
"0NJ00ZZ"  =  "1"  /* Inspection of Skull, Open Approach */
"0NJ03ZZ"  =  "1"  /* Inspection of Skull, Percutaneous Approach */
"0NJ04ZZ"  =  "1"  /* Inspection of Skull, Percutaneous Endoscopic Approach */
"0NJB0ZZ"  =  "1"  /* Inspection of Nasal Bone, Open Approach */
"0NJB3ZZ"  =  "1"  /* Inspection of Nasal Bone, Percutaneous Approach */
"0NJB4ZZ"  =  "1"  /* Inspection of Nasal Bone, Percutaneous Endoscopic Approach */
"0NJW0ZZ"  =  "1"  /* Inspection of Facial Bone, Open Approach */
"0NJW3ZZ"  =  "1"  /* Inspection of Facial Bone, Percutaneous Approach */
"0NJW4ZZ"  =  "1"  /* Inspection of Facial Bone, Percutaneous Endoscopic Approach */
"0NN10ZZ"  =  "1"  /* Release Right Frontal Bone, Open Approach */
"0NN13ZZ"  =  "1"  /* Release Right Frontal Bone, Percutaneous Approach */
"0NN14ZZ"  =  "1"  /* Release Right Frontal Bone, Percutaneous Endoscopic Approach */
"0NN20ZZ"  =  "1"  /* Release Left Frontal Bone, Open Approach */
"0NN23ZZ"  =  "1"  /* Release Left Frontal Bone, Percutaneous Approach */
"0NN24ZZ"  =  "1"  /* Release Left Frontal Bone, Percutaneous Endoscopic Approach */
"0NN30ZZ"  =  "1"  /* Release Right Parietal Bone, Open Approach */
"0NN33ZZ"  =  "1"  /* Release Right Parietal Bone, Percutaneous Approach */
"0NN34ZZ"  =  "1"  /* Release Right Parietal Bone, Perc Endo Approach */
"0NN40ZZ"  =  "1"  /* Release Left Parietal Bone, Open Approach */
"0NN43ZZ"  =  "1"  /* Release Left Parietal Bone, Percutaneous Approach */
"0NN44ZZ"  =  "1"  /* Release Left Parietal Bone, Percutaneous Endoscopic Approach */
"0NN50ZZ"  =  "1"  /* Release Right Temporal Bone, Open Approach */
"0NN53ZZ"  =  "1"  /* Release Right Temporal Bone, Percutaneous Approach */
"0NN54ZZ"  =  "1"  /* Release Right Temporal Bone, Perc Endo Approach */
"0NN60ZZ"  =  "1"  /* Release Left Temporal Bone, Open Approach */
"0NN63ZZ"  =  "1"  /* Release Left Temporal Bone, Percutaneous Approach */
"0NN64ZZ"  =  "1"  /* Release Left Temporal Bone, Percutaneous Endoscopic Approach */
"0NN70ZZ"  =  "1"  /* Release Right Occipital Bone, Open Approach */
"0NN73ZZ"  =  "1"  /* Release Right Occipital Bone, Percutaneous Approach */
"0NN74ZZ"  =  "1"  /* Release Right Occipital Bone, Perc Endo Approach */
"0NN80ZZ"  =  "1"  /* Release Left Occipital Bone, Open Approach */
"0NN83ZZ"  =  "1"  /* Release Left Occipital Bone, Percutaneous Approach */
"0NN84ZZ"  =  "1"  /* Release Left Occipital Bone, Perc Endo Approach */
"0NNC0ZZ"  =  "1"  /* Release Right Sphenoid Bone, Open Approach */
"0NNC3ZZ"  =  "1"  /* Release Right Sphenoid Bone, Percutaneous Approach */
"0NNC4ZZ"  =  "1"  /* Release Right Sphenoid Bone, Perc Endo Approach */
"0NND0ZZ"  =  "1"  /* Release Left Sphenoid Bone, Open Approach */
"0NND3ZZ"  =  "1"  /* Release Left Sphenoid Bone, Percutaneous Approach */
"0NND4ZZ"  =  "1"  /* Release Left Sphenoid Bone, Percutaneous Endoscopic Approach */
"0NNF0ZZ"  =  "1"  /* Release Right Ethmoid Bone, Open Approach */
"0NNF3ZZ"  =  "1"  /* Release Right Ethmoid Bone, Percutaneous Approach */
"0NNF4ZZ"  =  "1"  /* Release Right Ethmoid Bone, Percutaneous Endoscopic Approach */
"0NNG0ZZ"  =  "1"  /* Release Left Ethmoid Bone, Open Approach */
"0NNG3ZZ"  =  "1"  /* Release Left Ethmoid Bone, Percutaneous Approach */
"0NNG4ZZ"  =  "1"  /* Release Left Ethmoid Bone, Percutaneous Endoscopic Approach */
"0NNH0ZZ"  =  "1"  /* Release Right Lacrimal Bone, Open Approach */
"0NNH3ZZ"  =  "1"  /* Release Right Lacrimal Bone, Percutaneous Approach */
"0NNH4ZZ"  =  "1"  /* Release Right Lacrimal Bone, Perc Endo Approach */
"0NNJ0ZZ"  =  "1"  /* Release Left Lacrimal Bone, Open Approach */
"0NNJ3ZZ"  =  "1"  /* Release Left Lacrimal Bone, Percutaneous Approach */
"0NNJ4ZZ"  =  "1"  /* Release Left Lacrimal Bone, Percutaneous Endoscopic Approach */
"0NNK0ZZ"  =  "1"  /* Release Right Palatine Bone, Open Approach */
"0NNK3ZZ"  =  "1"  /* Release Right Palatine Bone, Percutaneous Approach */
"0NNK4ZZ"  =  "1"  /* Release Right Palatine Bone, Perc Endo Approach */
"0NNL0ZZ"  =  "1"  /* Release Left Palatine Bone, Open Approach */
"0NNL3ZZ"  =  "1"  /* Release Left Palatine Bone, Percutaneous Approach */
"0NNL4ZZ"  =  "1"  /* Release Left Palatine Bone, Percutaneous Endoscopic Approach */
"0NNM0ZZ"  =  "1"  /* Release Right Zygomatic Bone, Open Approach */
"0NNM3ZZ"  =  "1"  /* Release Right Zygomatic Bone, Percutaneous Approach */
"0NNM4ZZ"  =  "1"  /* Release Right Zygomatic Bone, Perc Endo Approach */
"0NNN0ZZ"  =  "1"  /* Release Left Zygomatic Bone, Open Approach */
"0NNN3ZZ"  =  "1"  /* Release Left Zygomatic Bone, Percutaneous Approach */
"0NNN4ZZ"  =  "1"  /* Release Left Zygomatic Bone, Perc Endo Approach */
"0NNP0ZZ"  =  "1"  /* Release Right Orbit, Open Approach */
"0NNP3ZZ"  =  "1"  /* Release Right Orbit, Percutaneous Approach */
"0NNP4ZZ"  =  "1"  /* Release Right Orbit, Percutaneous Endoscopic Approach */
"0NNQ0ZZ"  =  "1"  /* Release Left Orbit, Open Approach */
"0NNQ3ZZ"  =  "1"  /* Release Left Orbit, Percutaneous Approach */
"0NNQ4ZZ"  =  "1"  /* Release Left Orbit, Percutaneous Endoscopic Approach */
"0NNR0ZZ"  =  "1"  /* Release Right Maxilla, Open Approach */
"0NNR3ZZ"  =  "1"  /* Release Right Maxilla, Percutaneous Approach */
"0NNR4ZZ"  =  "1"  /* Release Right Maxilla, Percutaneous Endoscopic Approach */
"0NNS0ZZ"  =  "1"  /* Release Left Maxilla, Open Approach */
"0NNS3ZZ"  =  "1"  /* Release Left Maxilla, Percutaneous Approach */
"0NNS4ZZ"  =  "1"  /* Release Left Maxilla, Percutaneous Endoscopic Approach */
"0NNT0ZZ"  =  "1"  /* Release Right Mandible, Open Approach */
"0NNT3ZZ"  =  "1"  /* Release Right Mandible, Percutaneous Approach */
"0NNT4ZZ"  =  "1"  /* Release Right Mandible, Percutaneous Endoscopic Approach */
"0NNV0ZZ"  =  "1"  /* Release Left Mandible, Open Approach */
"0NNV3ZZ"  =  "1"  /* Release Left Mandible, Percutaneous Approach */
"0NNV4ZZ"  =  "1"  /* Release Left Mandible, Percutaneous Endoscopic Approach */
"0NNX0ZZ"  =  "1"  /* Release Hyoid Bone, Open Approach */
"0NNX3ZZ"  =  "1"  /* Release Hyoid Bone, Percutaneous Approach */
"0NNX4ZZ"  =  "1"  /* Release Hyoid Bone, Percutaneous Endoscopic Approach */
"0NP000Z"  =  "1"  /* Removal of Drainage Device from Skull, Open Approach */
"0NP004Z"  =  "1"  /* Removal of Int Fix from Skull, Open Approach */
"0NP005Z"  =  "1"  /* Removal of Ext Fix from Skull, Open Approach */
"0NP007Z"  =  "1"  /* Removal of Autol Sub from Skull, Open Approach */
"0NP00JZ"  =  "1"  /* Removal of Synthetic Substitute from Skull, Open Approach */
"0NP00KZ"  =  "1"  /* Removal of Nonaut Sub from Skull, Open Approach */
"0NP00MZ"  =  "1"  /* Removal of Bone Growth Stimulator from Skull, Open Approach */
"0NP00NZ"  =  "1"  /* Removal of Neurostim from Skull, Open Approach */
"0NP00SZ"  =  "1"  /* Removal of Hearing Device from Skull, Open Approach */
"0NP030Z"  =  "1"  /* Removal of Drainage Device from Skull, Percutaneous Approach */
"0NP034Z"  =  "1"  /* Removal of Int Fix from Skull, Perc Approach */
"0NP037Z"  =  "1"  /* Removal of Autol Sub from Skull, Perc Approach */
"0NP03JZ"  =  "1"  /* Removal of Synthetic Substitute from Skull, Perc Approach */
"0NP03KZ"  =  "1"  /* Removal of Nonaut Sub from Skull, Perc Approach */
"0NP03MZ"  =  "1"  /* Removal of Bone Growth Stimulator from Skull, Perc Approach */
"0NP03SZ"  =  "1"  /* Removal of Hearing Device from Skull, Percutaneous Approach */
"0NP040Z"  =  "1"  /* Removal of Drainage Device from Skull, Perc Endo Approach */
"0NP044Z"  =  "1"  /* Removal of Int Fix from Skull, Perc Endo Approach */
"0NP047Z"  =  "1"  /* Removal of Autol Sub from Skull, Perc Endo Approach */
"0NP04JZ"  =  "1"  /* Removal of Synth Sub from Skull, Perc Endo Approach */
"0NP04KZ"  =  "1"  /* Removal of Nonaut Sub from Skull, Perc Endo Approach */
"0NP04MZ"  =  "1"  /* Removal of Bone Stim from Skull, Perc Endo Approach */
"0NP04SZ"  =  "1"  /* Removal of Hearing Device from Skull, Perc Endo Approach */
"0NP0X4Z"  =  "1"  /* Removal of Int Fix from Skull, Extern Approach */
"0NP0XMZ"  =  "1"  /* Removal of Bone Stim from Skull, Extern Approach */
"0NP0XSZ"  =  "1"  /* Removal of Hearing Device from Skull, External Approach */
"0NPW00Z"  =  "1"  /* Removal of Drainage Device from Facial Bone, Open Approach */
"0NPW04Z"  =  "1"  /* Removal of Int Fix from Facial Bone, Open Approach */
"0NPW07Z"  =  "1"  /* Removal of Autol Sub from Facial Bone, Open Approach */
"0NPW0JZ"  =  "1"  /* Removal of Synth Sub from Facial Bone, Open Approach */
"0NPW0KZ"  =  "1"  /* Removal of Nonaut Sub from Facial Bone, Open Approach */
"0NPW0MZ"  =  "1"  /* Removal of Bone Stim from Facial Bone, Open Approach */
"0NPW30Z"  =  "1"  /* Removal of Drainage Device from Facial Bone, Perc Approach */
"0NPW34Z"  =  "1"  /* Removal of Int Fix from Facial Bone, Perc Approach */
"0NPW37Z"  =  "1"  /* Removal of Autol Sub from Facial Bone, Perc Approach */
"0NPW3JZ"  =  "1"  /* Removal of Synth Sub from Facial Bone, Perc Approach */
"0NPW3KZ"  =  "1"  /* Removal of Nonaut Sub from Facial Bone, Perc Approach */
"0NPW3MZ"  =  "1"  /* Removal of Bone Stim from Facial Bone, Perc Approach */
"0NPW40Z"  =  "1"  /* Removal of Drain Dev from Facial Bone, Perc Endo Approach */
"0NPW44Z"  =  "1"  /* Removal of Int Fix from Facial Bone, Perc Endo Approach */
"0NPW47Z"  =  "1"  /* Removal of Autol Sub from Facial Bone, Perc Endo Approach */
"0NPW4JZ"  =  "1"  /* Removal of Synth Sub from Facial Bone, Perc Endo Approach */
"0NPW4KZ"  =  "1"  /* Removal of Nonaut Sub from Facial Bone, Perc Endo Approach */
"0NPW4MZ"  =  "1"  /* Removal of Bone Stim from Facial Bone, Perc Endo Approach */
"0NPWX4Z"  =  "1"  /* Removal of Int Fix from Facial Bone, Extern Approach */
"0NQ00ZZ"  =  "1"  /* Repair Skull, Open Approach */
"0NQ03ZZ"  =  "1"  /* Repair Skull, Percutaneous Approach */
"0NQ04ZZ"  =  "1"  /* Repair Skull, Percutaneous Endoscopic Approach */
"0NQ0XZZ"  =  "1"  /* Repair Skull, External Approach */
"0NQ10ZZ"  =  "1"  /* Repair Right Frontal Bone, Open Approach */
"0NQ13ZZ"  =  "1"  /* Repair Right Frontal Bone, Percutaneous Approach */
"0NQ14ZZ"  =  "1"  /* Repair Right Frontal Bone, Percutaneous Endoscopic Approach */
"0NQ1XZZ"  =  "1"  /* Repair Right Frontal Bone, External Approach */
"0NQ20ZZ"  =  "1"  /* Repair Left Frontal Bone, Open Approach */
"0NQ23ZZ"  =  "1"  /* Repair Left Frontal Bone, Percutaneous Approach */
"0NQ24ZZ"  =  "1"  /* Repair Left Frontal Bone, Percutaneous Endoscopic Approach */
"0NQ2XZZ"  =  "1"  /* Repair Left Frontal Bone, External Approach */
"0NQ30ZZ"  =  "1"  /* Repair Right Parietal Bone, Open Approach */
"0NQ33ZZ"  =  "1"  /* Repair Right Parietal Bone, Percutaneous Approach */
"0NQ34ZZ"  =  "1"  /* Repair Right Parietal Bone, Percutaneous Endoscopic Approach */
"0NQ3XZZ"  =  "1"  /* Repair Right Parietal Bone, External Approach */
"0NQ40ZZ"  =  "1"  /* Repair Left Parietal Bone, Open Approach */
"0NQ43ZZ"  =  "1"  /* Repair Left Parietal Bone, Percutaneous Approach */
"0NQ44ZZ"  =  "1"  /* Repair Left Parietal Bone, Percutaneous Endoscopic Approach */
"0NQ4XZZ"  =  "1"  /* Repair Left Parietal Bone, External Approach */
"0NQ50ZZ"  =  "1"  /* Repair Right Temporal Bone, Open Approach */
"0NQ53ZZ"  =  "1"  /* Repair Right Temporal Bone, Percutaneous Approach */
"0NQ54ZZ"  =  "1"  /* Repair Right Temporal Bone, Percutaneous Endoscopic Approach */
"0NQ5XZZ"  =  "1"  /* Repair Right Temporal Bone, External Approach */
"0NQ60ZZ"  =  "1"  /* Repair Left Temporal Bone, Open Approach */
"0NQ63ZZ"  =  "1"  /* Repair Left Temporal Bone, Percutaneous Approach */
"0NQ64ZZ"  =  "1"  /* Repair Left Temporal Bone, Percutaneous Endoscopic Approach */
"0NQ6XZZ"  =  "1"  /* Repair Left Temporal Bone, External Approach */
"0NQ70ZZ"  =  "1"  /* Repair Right Occipital Bone, Open Approach */
"0NQ73ZZ"  =  "1"  /* Repair Right Occipital Bone, Percutaneous Approach */
"0NQ74ZZ"  =  "1"  /* Repair Right Occipital Bone, Perc Endo Approach */
"0NQ7XZZ"  =  "1"  /* Repair Right Occipital Bone, External Approach */
"0NQ80ZZ"  =  "1"  /* Repair Left Occipital Bone, Open Approach */
"0NQ83ZZ"  =  "1"  /* Repair Left Occipital Bone, Percutaneous Approach */
"0NQ84ZZ"  =  "1"  /* Repair Left Occipital Bone, Percutaneous Endoscopic Approach */
"0NQ8XZZ"  =  "1"  /* Repair Left Occipital Bone, External Approach */
"0NQB0ZZ"  =  "1"  /* Repair Nasal Bone, Open Approach */
"0NQB3ZZ"  =  "1"  /* Repair Nasal Bone, Percutaneous Approach */
"0NQB4ZZ"  =  "1"  /* Repair Nasal Bone, Percutaneous Endoscopic Approach */
"0NQBXZZ"  =  "1"  /* Repair Nasal Bone, External Approach */
"0NQC0ZZ"  =  "1"  /* Repair Right Sphenoid Bone, Open Approach */
"0NQC3ZZ"  =  "1"  /* Repair Right Sphenoid Bone, Percutaneous Approach */
"0NQC4ZZ"  =  "1"  /* Repair Right Sphenoid Bone, Percutaneous Endoscopic Approach */
"0NQCXZZ"  =  "1"  /* Repair Right Sphenoid Bone, External Approach */
"0NQD0ZZ"  =  "1"  /* Repair Left Sphenoid Bone, Open Approach */
"0NQD3ZZ"  =  "1"  /* Repair Left Sphenoid Bone, Percutaneous Approach */
"0NQD4ZZ"  =  "1"  /* Repair Left Sphenoid Bone, Percutaneous Endoscopic Approach */
"0NQDXZZ"  =  "1"  /* Repair Left Sphenoid Bone, External Approach */
"0NQF0ZZ"  =  "1"  /* Repair Right Ethmoid Bone, Open Approach */
"0NQF3ZZ"  =  "1"  /* Repair Right Ethmoid Bone, Percutaneous Approach */
"0NQF4ZZ"  =  "1"  /* Repair Right Ethmoid Bone, Percutaneous Endoscopic Approach */
"0NQFXZZ"  =  "1"  /* Repair Right Ethmoid Bone, External Approach */
"0NQG0ZZ"  =  "1"  /* Repair Left Ethmoid Bone, Open Approach */
"0NQG3ZZ"  =  "1"  /* Repair Left Ethmoid Bone, Percutaneous Approach */
"0NQG4ZZ"  =  "1"  /* Repair Left Ethmoid Bone, Percutaneous Endoscopic Approach */
"0NQGXZZ"  =  "1"  /* Repair Left Ethmoid Bone, External Approach */
"0NQH0ZZ"  =  "1"  /* Repair Right Lacrimal Bone, Open Approach */
"0NQH3ZZ"  =  "1"  /* Repair Right Lacrimal Bone, Percutaneous Approach */
"0NQH4ZZ"  =  "1"  /* Repair Right Lacrimal Bone, Percutaneous Endoscopic Approach */
"0NQHXZZ"  =  "1"  /* Repair Right Lacrimal Bone, External Approach */
"0NQJ0ZZ"  =  "1"  /* Repair Left Lacrimal Bone, Open Approach */
"0NQJ3ZZ"  =  "1"  /* Repair Left Lacrimal Bone, Percutaneous Approach */
"0NQJ4ZZ"  =  "1"  /* Repair Left Lacrimal Bone, Percutaneous Endoscopic Approach */
"0NQJXZZ"  =  "1"  /* Repair Left Lacrimal Bone, External Approach */
"0NQK0ZZ"  =  "1"  /* Repair Right Palatine Bone, Open Approach */
"0NQK3ZZ"  =  "1"  /* Repair Right Palatine Bone, Percutaneous Approach */
"0NQK4ZZ"  =  "1"  /* Repair Right Palatine Bone, Percutaneous Endoscopic Approach */
"0NQKXZZ"  =  "1"  /* Repair Right Palatine Bone, External Approach */
"0NQL0ZZ"  =  "1"  /* Repair Left Palatine Bone, Open Approach */
"0NQL3ZZ"  =  "1"  /* Repair Left Palatine Bone, Percutaneous Approach */
"0NQL4ZZ"  =  "1"  /* Repair Left Palatine Bone, Percutaneous Endoscopic Approach */
"0NQLXZZ"  =  "1"  /* Repair Left Palatine Bone, External Approach */
"0NQM0ZZ"  =  "1"  /* Repair Right Zygomatic Bone, Open Approach */
"0NQM3ZZ"  =  "1"  /* Repair Right Zygomatic Bone, Percutaneous Approach */
"0NQM4ZZ"  =  "1"  /* Repair Right Zygomatic Bone, Perc Endo Approach */
"0NQMXZZ"  =  "1"  /* Repair Right Zygomatic Bone, External Approach */
"0NQN0ZZ"  =  "1"  /* Repair Left Zygomatic Bone, Open Approach */
"0NQN3ZZ"  =  "1"  /* Repair Left Zygomatic Bone, Percutaneous Approach */
"0NQN4ZZ"  =  "1"  /* Repair Left Zygomatic Bone, Percutaneous Endoscopic Approach */
"0NQNXZZ"  =  "1"  /* Repair Left Zygomatic Bone, External Approach */
"0NQP0ZZ"  =  "1"  /* Repair Right Orbit, Open Approach */
"0NQP3ZZ"  =  "1"  /* Repair Right Orbit, Percutaneous Approach */
"0NQP4ZZ"  =  "1"  /* Repair Right Orbit, Percutaneous Endoscopic Approach */
"0NQPXZZ"  =  "1"  /* Repair Right Orbit, External Approach */
"0NQQ0ZZ"  =  "1"  /* Repair Left Orbit, Open Approach */
"0NQQ3ZZ"  =  "1"  /* Repair Left Orbit, Percutaneous Approach */
"0NQQ4ZZ"  =  "1"  /* Repair Left Orbit, Percutaneous Endoscopic Approach */
"0NQQXZZ"  =  "1"  /* Repair Left Orbit, External Approach */
"0NQR0ZZ"  =  "1"  /* Repair Right Maxilla, Open Approach */
"0NQR3ZZ"  =  "1"  /* Repair Right Maxilla, Percutaneous Approach */
"0NQR4ZZ"  =  "1"  /* Repair Right Maxilla, Percutaneous Endoscopic Approach */
"0NQRXZZ"  =  "1"  /* Repair Right Maxilla, External Approach */
"0NQS0ZZ"  =  "1"  /* Repair Left Maxilla, Open Approach */
"0NQS3ZZ"  =  "1"  /* Repair Left Maxilla, Percutaneous Approach */
"0NQS4ZZ"  =  "1"  /* Repair Left Maxilla, Percutaneous Endoscopic Approach */
"0NQSXZZ"  =  "1"  /* Repair Left Maxilla, External Approach */
"0NQT0ZZ"  =  "1"  /* Repair Right Mandible, Open Approach */
"0NQT3ZZ"  =  "1"  /* Repair Right Mandible, Percutaneous Approach */
"0NQT4ZZ"  =  "1"  /* Repair Right Mandible, Percutaneous Endoscopic Approach */
"0NQTXZZ"  =  "1"  /* Repair Right Mandible, External Approach */
"0NQV0ZZ"  =  "1"  /* Repair Left Mandible, Open Approach */
"0NQV3ZZ"  =  "1"  /* Repair Left Mandible, Percutaneous Approach */
"0NQV4ZZ"  =  "1"  /* Repair Left Mandible, Percutaneous Endoscopic Approach */
"0NQVXZZ"  =  "1"  /* Repair Left Mandible, External Approach */
"0NQX0ZZ"  =  "1"  /* Repair Hyoid Bone, Open Approach */
"0NQX3ZZ"  =  "1"  /* Repair Hyoid Bone, Percutaneous Approach */
"0NQX4ZZ"  =  "1"  /* Repair Hyoid Bone, Percutaneous Endoscopic Approach */
"0NQXXZZ"  =  "1"  /* Repair Hyoid Bone, External Approach */
"0NR007Z"  =  "1"  /* Replacement of Skull with Autol Sub, Open Approach */
"0NR00JZ"  =  "1"  /* Replacement of Skull with Synth Sub, Open Approach */
"0NR00KZ"  =  "1"  /* Replacement of Skull with Nonaut Sub, Open Approach */
"0NR037Z"  =  "1"  /* Replacement of Skull with Autol Sub, Perc Approach */
"0NR03JZ"  =  "1"  /* Replacement of Skull with Synth Sub, Perc Approach */
"0NR03KZ"  =  "1"  /* Replacement of Skull with Nonaut Sub, Perc Approach */
"0NR047Z"  =  "1"  /* Replacement of Skull with Autol Sub, Perc Endo Approach */
"0NR04JZ"  =  "1"  /* Replacement of Skull with Synth Sub, Perc Endo Approach */
"0NR04KZ"  =  "1"  /* Replacement of Skull with Nonaut Sub, Perc Endo Approach */
"0NR107Z"  =  "1"  /* Replacement of R Frontal Bone with Autol Sub, Open Approach */
"0NR10JZ"  =  "1"  /* Replacement of R Frontal Bone with Synth Sub, Open Approach */
"0NR10KZ"  =  "1"  /* Replacement of R Frontal Bone with Nonaut Sub, Open Approach */
"0NR137Z"  =  "1"  /* Replacement of R Frontal Bone with Autol Sub, Perc Approach */
"0NR13JZ"  =  "1"  /* Replacement of R Frontal Bone with Synth Sub, Perc Approach */
"0NR13KZ"  =  "1"  /* Replacement of R Frontal Bone with Nonaut Sub, Perc Approach */
"0NR147Z"  =  "1"  /* Replace of R Frontal Bone with Autol Sub, Perc Endo Approach */
"0NR14JZ"  =  "1"  /* Replace of R Frontal Bone with Synth Sub, Perc Endo Approach */
"0NR14KZ"  =  "1"  /* Replace R Frontal Bone w Nonaut Sub, Perc Endo */
"0NR207Z"  =  "1"  /* Replacement of L Frontal Bone with Autol Sub, Open Approach */
"0NR20JZ"  =  "1"  /* Replacement of L Frontal Bone with Synth Sub, Open Approach */
"0NR20KZ"  =  "1"  /* Replacement of L Frontal Bone with Nonaut Sub, Open Approach */
"0NR237Z"  =  "1"  /* Replacement of L Frontal Bone with Autol Sub, Perc Approach */
"0NR23JZ"  =  "1"  /* Replacement of L Frontal Bone with Synth Sub, Perc Approach */
"0NR23KZ"  =  "1"  /* Replacement of L Frontal Bone with Nonaut Sub, Perc Approach */
"0NR247Z"  =  "1"  /* Replace of L Frontal Bone with Autol Sub, Perc Endo Approach */
"0NR24JZ"  =  "1"  /* Replace of L Frontal Bone with Synth Sub, Perc Endo Approach */
"0NR24KZ"  =  "1"  /* Replace L Frontal Bone w Nonaut Sub, Perc Endo */
"0NR307Z"  =  "1"  /* Replacement of R Parietal Bone with Autol Sub, Open Approach */
"0NR30JZ"  =  "1"  /* Replacement of R Parietal Bone with Synth Sub, Open Approach */
"0NR30KZ"  =  "1"  /* Replace of R Parietal Bone with Nonaut Sub, Open Approach */
"0NR337Z"  =  "1"  /* Replacement of R Parietal Bone with Autol Sub, Perc Approach */
"0NR33JZ"  =  "1"  /* Replacement of R Parietal Bone with Synth Sub, Perc Approach */
"0NR33KZ"  =  "1"  /* Replace of R Parietal Bone with Nonaut Sub, Perc Approach */
"0NR347Z"  =  "1"  /* Replace R Parietal Bone w Autol Sub, Perc Endo */
"0NR34JZ"  =  "1"  /* Replace R Parietal Bone w Synth Sub, Perc Endo */
"0NR34KZ"  =  "1"  /* Replace R Parietal Bone w Nonaut Sub, Perc Endo */
"0NR407Z"  =  "1"  /* Replacement of L Parietal Bone with Autol Sub, Open Approach */
"0NR40JZ"  =  "1"  /* Replacement of L Parietal Bone with Synth Sub, Open Approach */
"0NR40KZ"  =  "1"  /* Replace of L Parietal Bone with Nonaut Sub, Open Approach */
"0NR437Z"  =  "1"  /* Replacement of L Parietal Bone with Autol Sub, Perc Approach */
"0NR43JZ"  =  "1"  /* Replacement of L Parietal Bone with Synth Sub, Perc Approach */
"0NR43KZ"  =  "1"  /* Replace of L Parietal Bone with Nonaut Sub, Perc Approach */
"0NR447Z"  =  "1"  /* Replace L Parietal Bone w Autol Sub, Perc Endo */
"0NR44JZ"  =  "1"  /* Replace L Parietal Bone w Synth Sub, Perc Endo */
"0NR44KZ"  =  "1"  /* Replace L Parietal Bone w Nonaut Sub, Perc Endo */
"0NR507Z"  =  "1"  /* Replacement of R Temporal Bone with Autol Sub, Open Approach */
"0NR50JZ"  =  "1"  /* Replacement of R Temporal Bone with Synth Sub, Open Approach */
"0NR50KZ"  =  "1"  /* Replace of R Temporal Bone with Nonaut Sub, Open Approach */
"0NR537Z"  =  "1"  /* Replacement of R Temporal Bone with Autol Sub, Perc Approach */
"0NR53JZ"  =  "1"  /* Replacement of R Temporal Bone with Synth Sub, Perc Approach */
"0NR53KZ"  =  "1"  /* Replace of R Temporal Bone with Nonaut Sub, Perc Approach */
"0NR547Z"  =  "1"  /* Replace R Temporal Bone w Autol Sub, Perc Endo */
"0NR54JZ"  =  "1"  /* Replace R Temporal Bone w Synth Sub, Perc Endo */
"0NR54KZ"  =  "1"  /* Replace R Temporal Bone w Nonaut Sub, Perc Endo */
"0NR607Z"  =  "1"  /* Replacement of L Temporal Bone with Autol Sub, Open Approach */
"0NR60JZ"  =  "1"  /* Replacement of L Temporal Bone with Synth Sub, Open Approach */
"0NR60KZ"  =  "1"  /* Replace of L Temporal Bone with Nonaut Sub, Open Approach */
"0NR637Z"  =  "1"  /* Replacement of L Temporal Bone with Autol Sub, Perc Approach */
"0NR63JZ"  =  "1"  /* Replacement of L Temporal Bone with Synth Sub, Perc Approach */
"0NR63KZ"  =  "1"  /* Replace of L Temporal Bone with Nonaut Sub, Perc Approach */
"0NR647Z"  =  "1"  /* Replace L Temporal Bone w Autol Sub, Perc Endo */
"0NR64JZ"  =  "1"  /* Replace L Temporal Bone w Synth Sub, Perc Endo */
"0NR64KZ"  =  "1"  /* Replace L Temporal Bone w Nonaut Sub, Perc Endo */
"0NR707Z"  =  "1"  /* Replace of R Occipital Bone with Autol Sub, Open Approach */
"0NR70JZ"  =  "1"  /* Replace of R Occipital Bone with Synth Sub, Open Approach */
"0NR70KZ"  =  "1"  /* Replace of R Occipital Bone with Nonaut Sub, Open Approach */
"0NR737Z"  =  "1"  /* Replace of R Occipital Bone with Autol Sub, Perc Approach */
"0NR73JZ"  =  "1"  /* Replace of R Occipital Bone with Synth Sub, Perc Approach */
"0NR73KZ"  =  "1"  /* Replace of R Occipital Bone with Nonaut Sub, Perc Approach */
"0NR747Z"  =  "1"  /* Replace R Occipital Bone w Autol Sub, Perc Endo */
"0NR74JZ"  =  "1"  /* Replace R Occipital Bone w Synth Sub, Perc Endo */
"0NR74KZ"  =  "1"  /* Replace R Occipital Bone w Nonaut Sub, Perc Endo */
"0NR807Z"  =  "1"  /* Replace of L Occipital Bone with Autol Sub, Open Approach */
"0NR80JZ"  =  "1"  /* Replace of L Occipital Bone with Synth Sub, Open Approach */
"0NR80KZ"  =  "1"  /* Replace of L Occipital Bone with Nonaut Sub, Open Approach */
"0NR837Z"  =  "1"  /* Replace of L Occipital Bone with Autol Sub, Perc Approach */
"0NR83JZ"  =  "1"  /* Replace of L Occipital Bone with Synth Sub, Perc Approach */
"0NR83KZ"  =  "1"  /* Replace of L Occipital Bone with Nonaut Sub, Perc Approach */
"0NR847Z"  =  "1"  /* Replace L Occipital Bone w Autol Sub, Perc Endo */
"0NR84JZ"  =  "1"  /* Replace L Occipital Bone w Synth Sub, Perc Endo */
"0NR84KZ"  =  "1"  /* Replace L Occipital Bone w Nonaut Sub, Perc Endo */
"0NRB07Z"  =  "1"  /* Replacement of Nasal Bone with Autol Sub, Open Approach */
"0NRB0JZ"  =  "1"  /* Replacement of Nasal Bone with Synth Sub, Open Approach */
"0NRB0KZ"  =  "1"  /* Replacement of Nasal Bone with Nonaut Sub, Open Approach */
"0NRB37Z"  =  "1"  /* Replacement of Nasal Bone with Autol Sub, Perc Approach */
"0NRB3JZ"  =  "1"  /* Replacement of Nasal Bone with Synth Sub, Perc Approach */
"0NRB3KZ"  =  "1"  /* Replacement of Nasal Bone with Nonaut Sub, Perc Approach */
"0NRB47Z"  =  "1"  /* Replacement of Nasal Bone with Autol Sub, Perc Endo Approach */
"0NRB4JZ"  =  "1"  /* Replacement of Nasal Bone with Synth Sub, Perc Endo Approach */
"0NRB4KZ"  =  "1"  /* Replace of Nasal Bone with Nonaut Sub, Perc Endo Approach */
"0NRC07Z"  =  "1"  /* Replacement of R Sphenoid Bone with Autol Sub, Open Approach */
"0NRC0JZ"  =  "1"  /* Replacement of R Sphenoid Bone with Synth Sub, Open Approach */
"0NRC0KZ"  =  "1"  /* Replace of R Sphenoid Bone with Nonaut Sub, Open Approach */
"0NRC37Z"  =  "1"  /* Replacement of R Sphenoid Bone with Autol Sub, Perc Approach */
"0NRC3JZ"  =  "1"  /* Replacement of R Sphenoid Bone with Synth Sub, Perc Approach */
"0NRC3KZ"  =  "1"  /* Replace of R Sphenoid Bone with Nonaut Sub, Perc Approach */
"0NRC47Z"  =  "1"  /* Replace R Sphenoid Bone w Autol Sub, Perc Endo */
"0NRC4JZ"  =  "1"  /* Replace R Sphenoid Bone w Synth Sub, Perc Endo */
"0NRC4KZ"  =  "1"  /* Replace R Sphenoid Bone w Nonaut Sub, Perc Endo */
"0NRD07Z"  =  "1"  /* Replacement of L Sphenoid Bone with Autol Sub, Open Approach */
"0NRD0JZ"  =  "1"  /* Replacement of L Sphenoid Bone with Synth Sub, Open Approach */
"0NRD0KZ"  =  "1"  /* Replace of L Sphenoid Bone with Nonaut Sub, Open Approach */
"0NRD37Z"  =  "1"  /* Replacement of L Sphenoid Bone with Autol Sub, Perc Approach */
"0NRD3JZ"  =  "1"  /* Replacement of L Sphenoid Bone with Synth Sub, Perc Approach */
"0NRD3KZ"  =  "1"  /* Replace of L Sphenoid Bone with Nonaut Sub, Perc Approach */
"0NRD47Z"  =  "1"  /* Replace L Sphenoid Bone w Autol Sub, Perc Endo */
"0NRD4JZ"  =  "1"  /* Replace L Sphenoid Bone w Synth Sub, Perc Endo */
"0NRD4KZ"  =  "1"  /* Replace L Sphenoid Bone w Nonaut Sub, Perc Endo */
"0NRF07Z"  =  "1"  /* Replacement of R Ethmoid Bone with Autol Sub, Open Approach */
"0NRF0JZ"  =  "1"  /* Replacement of R Ethmoid Bone with Synth Sub, Open Approach */
"0NRF0KZ"  =  "1"  /* Replacement of R Ethmoid Bone with Nonaut Sub, Open Approach */
"0NRF37Z"  =  "1"  /* Replacement of R Ethmoid Bone with Autol Sub, Perc Approach */
"0NRF3JZ"  =  "1"  /* Replacement of R Ethmoid Bone with Synth Sub, Perc Approach */
"0NRF3KZ"  =  "1"  /* Replacement of R Ethmoid Bone with Nonaut Sub, Perc Approach */
"0NRF47Z"  =  "1"  /* Replace of R Ethmoid Bone with Autol Sub, Perc Endo Approach */
"0NRF4JZ"  =  "1"  /* Replace of R Ethmoid Bone with Synth Sub, Perc Endo Approach */
"0NRF4KZ"  =  "1"  /* Replace R Ethmoid Bone w Nonaut Sub, Perc Endo */
"0NRG07Z"  =  "1"  /* Replacement of L Ethmoid Bone with Autol Sub, Open Approach */
"0NRG0JZ"  =  "1"  /* Replacement of L Ethmoid Bone with Synth Sub, Open Approach */
"0NRG0KZ"  =  "1"  /* Replacement of L Ethmoid Bone with Nonaut Sub, Open Approach */
"0NRG37Z"  =  "1"  /* Replacement of L Ethmoid Bone with Autol Sub, Perc Approach */
"0NRG3JZ"  =  "1"  /* Replacement of L Ethmoid Bone with Synth Sub, Perc Approach */
"0NRG3KZ"  =  "1"  /* Replacement of L Ethmoid Bone with Nonaut Sub, Perc Approach */
"0NRG47Z"  =  "1"  /* Replace of L Ethmoid Bone with Autol Sub, Perc Endo Approach */
"0NRG4JZ"  =  "1"  /* Replace of L Ethmoid Bone with Synth Sub, Perc Endo Approach */
"0NRG4KZ"  =  "1"  /* Replace L Ethmoid Bone w Nonaut Sub, Perc Endo */
"0NRH07Z"  =  "1"  /* Replacement of R Lacrimal Bone with Autol Sub, Open Approach */
"0NRH0JZ"  =  "1"  /* Replacement of R Lacrimal Bone with Synth Sub, Open Approach */
"0NRH0KZ"  =  "1"  /* Replace of R Lacrimal Bone with Nonaut Sub, Open Approach */
"0NRH37Z"  =  "1"  /* Replacement of R Lacrimal Bone with Autol Sub, Perc Approach */
"0NRH3JZ"  =  "1"  /* Replacement of R Lacrimal Bone with Synth Sub, Perc Approach */
"0NRH3KZ"  =  "1"  /* Replace of R Lacrimal Bone with Nonaut Sub, Perc Approach */
"0NRH47Z"  =  "1"  /* Replace R Lacrimal Bone w Autol Sub, Perc Endo */
"0NRH4JZ"  =  "1"  /* Replace R Lacrimal Bone w Synth Sub, Perc Endo */
"0NRH4KZ"  =  "1"  /* Replace R Lacrimal Bone w Nonaut Sub, Perc Endo */
"0NRJ07Z"  =  "1"  /* Replacement of L Lacrimal Bone with Autol Sub, Open Approach */
"0NRJ0JZ"  =  "1"  /* Replacement of L Lacrimal Bone with Synth Sub, Open Approach */
"0NRJ0KZ"  =  "1"  /* Replace of L Lacrimal Bone with Nonaut Sub, Open Approach */
"0NRJ37Z"  =  "1"  /* Replacement of L Lacrimal Bone with Autol Sub, Perc Approach */
"0NRJ3JZ"  =  "1"  /* Replacement of L Lacrimal Bone with Synth Sub, Perc Approach */
"0NRJ3KZ"  =  "1"  /* Replace of L Lacrimal Bone with Nonaut Sub, Perc Approach */
"0NRJ47Z"  =  "1"  /* Replace L Lacrimal Bone w Autol Sub, Perc Endo */
"0NRJ4JZ"  =  "1"  /* Replace L Lacrimal Bone w Synth Sub, Perc Endo */
"0NRJ4KZ"  =  "1"  /* Replace L Lacrimal Bone w Nonaut Sub, Perc Endo */
"0NRK07Z"  =  "1"  /* Replacement of R Palatine Bone with Autol Sub, Open Approach */
"0NRK0JZ"  =  "1"  /* Replacement of R Palatine Bone with Synth Sub, Open Approach */
"0NRK0KZ"  =  "1"  /* Replace of R Palatine Bone with Nonaut Sub, Open Approach */
"0NRK37Z"  =  "1"  /* Replacement of R Palatine Bone with Autol Sub, Perc Approach */
"0NRK3JZ"  =  "1"  /* Replacement of R Palatine Bone with Synth Sub, Perc Approach */
"0NRK3KZ"  =  "1"  /* Replace of R Palatine Bone with Nonaut Sub, Perc Approach */
"0NRK47Z"  =  "1"  /* Replace R Palatine Bone w Autol Sub, Perc Endo */
"0NRK4JZ"  =  "1"  /* Replace R Palatine Bone w Synth Sub, Perc Endo */
"0NRK4KZ"  =  "1"  /* Replace R Palatine Bone w Nonaut Sub, Perc Endo */
"0NRL07Z"  =  "1"  /* Replacement of L Palatine Bone with Autol Sub, Open Approach */
"0NRL0JZ"  =  "1"  /* Replacement of L Palatine Bone with Synth Sub, Open Approach */
"0NRL0KZ"  =  "1"  /* Replace of L Palatine Bone with Nonaut Sub, Open Approach */
"0NRL37Z"  =  "1"  /* Replacement of L Palatine Bone with Autol Sub, Perc Approach */
"0NRL3JZ"  =  "1"  /* Replacement of L Palatine Bone with Synth Sub, Perc Approach */
"0NRL3KZ"  =  "1"  /* Replace of L Palatine Bone with Nonaut Sub, Perc Approach */
"0NRL47Z"  =  "1"  /* Replace L Palatine Bone w Autol Sub, Perc Endo */
"0NRL4JZ"  =  "1"  /* Replace L Palatine Bone w Synth Sub, Perc Endo */
"0NRL4KZ"  =  "1"  /* Replace L Palatine Bone w Nonaut Sub, Perc Endo */
"0NRM07Z"  =  "1"  /* Replace of R Zygomatic Bone with Autol Sub, Open Approach */
"0NRM0JZ"  =  "1"  /* Replace of R Zygomatic Bone with Synth Sub, Open Approach */
"0NRM0KZ"  =  "1"  /* Replace of R Zygomatic Bone with Nonaut Sub, Open Approach */
"0NRM37Z"  =  "1"  /* Replace of R Zygomatic Bone with Autol Sub, Perc Approach */
"0NRM3JZ"  =  "1"  /* Replace of R Zygomatic Bone with Synth Sub, Perc Approach */
"0NRM3KZ"  =  "1"  /* Replace of R Zygomatic Bone with Nonaut Sub, Perc Approach */
"0NRM47Z"  =  "1"  /* Replace R Zygomatic Bone w Autol Sub, Perc Endo */
"0NRM4JZ"  =  "1"  /* Replace R Zygomatic Bone w Synth Sub, Perc Endo */
"0NRM4KZ"  =  "1"  /* Replace R Zygomatic Bone w Nonaut Sub, Perc Endo */
"0NRN07Z"  =  "1"  /* Replace of L Zygomatic Bone with Autol Sub, Open Approach */
"0NRN0JZ"  =  "1"  /* Replace of L Zygomatic Bone with Synth Sub, Open Approach */
"0NRN0KZ"  =  "1"  /* Replace of L Zygomatic Bone with Nonaut Sub, Open Approach */
"0NRN37Z"  =  "1"  /* Replace of L Zygomatic Bone with Autol Sub, Perc Approach */
"0NRN3JZ"  =  "1"  /* Replace of L Zygomatic Bone with Synth Sub, Perc Approach */
"0NRN3KZ"  =  "1"  /* Replace of L Zygomatic Bone with Nonaut Sub, Perc Approach */
"0NRN47Z"  =  "1"  /* Replace L Zygomatic Bone w Autol Sub, Perc Endo */
"0NRN4JZ"  =  "1"  /* Replace L Zygomatic Bone w Synth Sub, Perc Endo */
"0NRN4KZ"  =  "1"  /* Replace L Zygomatic Bone w Nonaut Sub, Perc Endo */
"0NRP07Z"  =  "1"  /* Replacement of Right Orbit with Autol Sub, Open Approach */
"0NRP0JZ"  =  "1"  /* Replacement of Right Orbit with Synth Sub, Open Approach */
"0NRP0KZ"  =  "1"  /* Replacement of Right Orbit with Nonaut Sub, Open Approach */
"0NRP37Z"  =  "1"  /* Replacement of Right Orbit with Autol Sub, Perc Approach */
"0NRP3JZ"  =  "1"  /* Replacement of Right Orbit with Synth Sub, Perc Approach */
"0NRP3KZ"  =  "1"  /* Replacement of Right Orbit with Nonaut Sub, Perc Approach */
"0NRP47Z"  =  "1"  /* Replacement of R Orbit with Autol Sub, Perc Endo Approach */
"0NRP4JZ"  =  "1"  /* Replacement of R Orbit with Synth Sub, Perc Endo Approach */
"0NRP4KZ"  =  "1"  /* Replacement of R Orbit with Nonaut Sub, Perc Endo Approach */
"0NRQ07Z"  =  "1"  /* Replacement of Left Orbit with Autol Sub, Open Approach */
"0NRQ0JZ"  =  "1"  /* Replacement of Left Orbit with Synth Sub, Open Approach */
"0NRQ0KZ"  =  "1"  /* Replacement of Left Orbit with Nonaut Sub, Open Approach */
"0NRQ37Z"  =  "1"  /* Replacement of Left Orbit with Autol Sub, Perc Approach */
"0NRQ3JZ"  =  "1"  /* Replacement of Left Orbit with Synth Sub, Perc Approach */
"0NRQ3KZ"  =  "1"  /* Replacement of Left Orbit with Nonaut Sub, Perc Approach */
"0NRQ47Z"  =  "1"  /* Replacement of Left Orbit with Autol Sub, Perc Endo Approach */
"0NRQ4JZ"  =  "1"  /* Replacement of Left Orbit with Synth Sub, Perc Endo Approach */
"0NRQ4KZ"  =  "1"  /* Replacement of L Orbit with Nonaut Sub, Perc Endo Approach */
"0NRR07Z"  =  "1"  /* Replacement of Right Maxilla with Autol Sub, Open Approach */
"0NRR0JZ"  =  "1"  /* Replacement of Right Maxilla with Synth Sub, Open Approach */
"0NRR0KZ"  =  "1"  /* Replacement of Right Maxilla with Nonaut Sub, Open Approach */
"0NRR37Z"  =  "1"  /* Replacement of Right Maxilla with Autol Sub, Perc Approach */
"0NRR3JZ"  =  "1"  /* Replacement of Right Maxilla with Synth Sub, Perc Approach */
"0NRR3KZ"  =  "1"  /* Replacement of Right Maxilla with Nonaut Sub, Perc Approach */
"0NRR47Z"  =  "1"  /* Replacement of R Maxilla with Autol Sub, Perc Endo Approach */
"0NRR4JZ"  =  "1"  /* Replacement of R Maxilla with Synth Sub, Perc Endo Approach */
"0NRR4KZ"  =  "1"  /* Replacement of R Maxilla with Nonaut Sub, Perc Endo Approach */
"0NRS07Z"  =  "1"  /* Replacement of Left Maxilla with Autol Sub, Open Approach */
"0NRS0JZ"  =  "1"  /* Replacement of Left Maxilla with Synth Sub, Open Approach */
"0NRS0KZ"  =  "1"  /* Replacement of Left Maxilla with Nonaut Sub, Open Approach */
"0NRS37Z"  =  "1"  /* Replacement of Left Maxilla with Autol Sub, Perc Approach */
"0NRS3JZ"  =  "1"  /* Replacement of Left Maxilla with Synth Sub, Perc Approach */
"0NRS3KZ"  =  "1"  /* Replacement of Left Maxilla with Nonaut Sub, Perc Approach */
"0NRS47Z"  =  "1"  /* Replacement of L Maxilla with Autol Sub, Perc Endo Approach */
"0NRS4JZ"  =  "1"  /* Replacement of L Maxilla with Synth Sub, Perc Endo Approach */
"0NRS4KZ"  =  "1"  /* Replacement of L Maxilla with Nonaut Sub, Perc Endo Approach */
"0NRT07Z"  =  "1"  /* Replacement of Right Mandible with Autol Sub, Open Approach */
"0NRT0JZ"  =  "1"  /* Replacement of Right Mandible with Synth Sub, Open Approach */
"0NRT0KZ"  =  "1"  /* Replacement of Right Mandible with Nonaut Sub, Open Approach */
"0NRT37Z"  =  "1"  /* Replacement of Right Mandible with Autol Sub, Perc Approach */
"0NRT3JZ"  =  "1"  /* Replacement of Right Mandible with Synth Sub, Perc Approach */
"0NRT3KZ"  =  "1"  /* Replacement of Right Mandible with Nonaut Sub, Perc Approach */
"0NRT47Z"  =  "1"  /* Replacement of R Mandible with Autol Sub, Perc Endo Approach */
"0NRT4JZ"  =  "1"  /* Replacement of R Mandible with Synth Sub, Perc Endo Approach */
"0NRT4KZ"  =  "1"  /* Replace of R Mandible with Nonaut Sub, Perc Endo Approach */
"0NRV07Z"  =  "1"  /* Replacement of Left Mandible with Autol Sub, Open Approach */
"0NRV0JZ"  =  "1"  /* Replacement of Left Mandible with Synth Sub, Open Approach */
"0NRV0KZ"  =  "1"  /* Replacement of Left Mandible with Nonaut Sub, Open Approach */
"0NRV37Z"  =  "1"  /* Replacement of Left Mandible with Autol Sub, Perc Approach */
"0NRV3JZ"  =  "1"  /* Replacement of Left Mandible with Synth Sub, Perc Approach */
"0NRV3KZ"  =  "1"  /* Replacement of Left Mandible with Nonaut Sub, Perc Approach */
"0NRV47Z"  =  "1"  /* Replacement of L Mandible with Autol Sub, Perc Endo Approach */
"0NRV4JZ"  =  "1"  /* Replacement of L Mandible with Synth Sub, Perc Endo Approach */
"0NRV4KZ"  =  "1"  /* Replace of L Mandible with Nonaut Sub, Perc Endo Approach */
"0NRX07Z"  =  "1"  /* Replacement of Hyoid Bone with Autol Sub, Open Approach */
"0NRX0JZ"  =  "1"  /* Replacement of Hyoid Bone with Synth Sub, Open Approach */
"0NRX0KZ"  =  "1"  /* Replacement of Hyoid Bone with Nonaut Sub, Open Approach */
"0NRX37Z"  =  "1"  /* Replacement of Hyoid Bone with Autol Sub, Perc Approach */
"0NRX3JZ"  =  "1"  /* Replacement of Hyoid Bone with Synth Sub, Perc Approach */
"0NRX3KZ"  =  "1"  /* Replacement of Hyoid Bone with Nonaut Sub, Perc Approach */
"0NRX47Z"  =  "1"  /* Replacement of Hyoid Bone with Autol Sub, Perc Endo Approach */
"0NRX4JZ"  =  "1"  /* Replacement of Hyoid Bone with Synth Sub, Perc Endo Approach */
"0NRX4KZ"  =  "1"  /* Replace of Hyoid Bone with Nonaut Sub, Perc Endo Approach */
"0NS004Z"  =  "1"  /* Reposition Skull with Int Fix, Open Approach */
"0NS005Z"  =  "1"  /* Reposition Skull with Ext Fix, Open Approach */
"0NS00ZZ"  =  "1"  /* Reposition Skull, Open Approach */
"0NS034Z"  =  "1"  /* Reposition Skull with Int Fix, Perc Approach */
"0NS035Z"  =  "1"  /* Reposition Skull with Ext Fix, Perc Approach */
"0NS03ZZ"  =  "1"  /* Reposition Skull, Percutaneous Approach */
"0NS044Z"  =  "1"  /* Reposition Skull with Int Fix, Perc Endo Approach */
"0NS045Z"  =  "1"  /* Reposition Skull with Ext Fix, Perc Endo Approach */
"0NS04ZZ"  =  "1"  /* Reposition Skull, Percutaneous Endoscopic Approach */
"0NS0XZZ"  =  "1"  /* Reposition Skull, External Approach */
"0NS104Z"  =  "1"  /* Reposition Right Frontal Bone with Int Fix, Open Approach */
"0NS10ZZ"  =  "1"  /* Reposition Right Frontal Bone, Open Approach */
"0NS134Z"  =  "1"  /* Reposition Right Frontal Bone with Int Fix, Perc Approach */
"0NS13ZZ"  =  "1"  /* Reposition Right Frontal Bone, Percutaneous Approach */
"0NS144Z"  =  "1"  /* Reposition R Frontal Bone with Int Fix, Perc Endo Approach */
"0NS14ZZ"  =  "1"  /* Reposition Right Frontal Bone, Perc Endo Approach */
"0NS1XZZ"  =  "1"  /* Reposition Right Frontal Bone, External Approach */
"0NS204Z"  =  "1"  /* Reposition Left Frontal Bone with Int Fix, Open Approach */
"0NS20ZZ"  =  "1"  /* Reposition Left Frontal Bone, Open Approach */
"0NS234Z"  =  "1"  /* Reposition Left Frontal Bone with Int Fix, Perc Approach */
"0NS23ZZ"  =  "1"  /* Reposition Left Frontal Bone, Percutaneous Approach */
"0NS244Z"  =  "1"  /* Reposition L Frontal Bone with Int Fix, Perc Endo Approach */
"0NS24ZZ"  =  "1"  /* Reposition Left Frontal Bone, Perc Endo Approach */
"0NS2XZZ"  =  "1"  /* Reposition Left Frontal Bone, External Approach */
"0NS304Z"  =  "1"  /* Reposition Right Parietal Bone with Int Fix, Open Approach */
"0NS30ZZ"  =  "1"  /* Reposition Right Parietal Bone, Open Approach */
"0NS334Z"  =  "1"  /* Reposition Right Parietal Bone with Int Fix, Perc Approach */
"0NS33ZZ"  =  "1"  /* Reposition Right Parietal Bone, Percutaneous Approach */
"0NS344Z"  =  "1"  /* Reposition R Parietal Bone with Int Fix, Perc Endo Approach */
"0NS34ZZ"  =  "1"  /* Reposition Right Parietal Bone, Perc Endo Approach */
"0NS3XZZ"  =  "1"  /* Reposition Right Parietal Bone, External Approach */
"0NS404Z"  =  "1"  /* Reposition Left Parietal Bone with Int Fix, Open Approach */
"0NS40ZZ"  =  "1"  /* Reposition Left Parietal Bone, Open Approach */
"0NS434Z"  =  "1"  /* Reposition Left Parietal Bone with Int Fix, Perc Approach */
"0NS43ZZ"  =  "1"  /* Reposition Left Parietal Bone, Percutaneous Approach */
"0NS444Z"  =  "1"  /* Reposition L Parietal Bone with Int Fix, Perc Endo Approach */
"0NS44ZZ"  =  "1"  /* Reposition Left Parietal Bone, Perc Endo Approach */
"0NS4XZZ"  =  "1"  /* Reposition Left Parietal Bone, External Approach */
"0NS504Z"  =  "1"  /* Reposition Right Temporal Bone with Int Fix, Open Approach */
"0NS50ZZ"  =  "1"  /* Reposition Right Temporal Bone, Open Approach */
"0NS534Z"  =  "1"  /* Reposition Right Temporal Bone with Int Fix, Perc Approach */
"0NS53ZZ"  =  "1"  /* Reposition Right Temporal Bone, Percutaneous Approach */
"0NS544Z"  =  "1"  /* Reposition R Temporal Bone with Int Fix, Perc Endo Approach */
"0NS54ZZ"  =  "1"  /* Reposition Right Temporal Bone, Perc Endo Approach */
"0NS5XZZ"  =  "1"  /* Reposition Right Temporal Bone, External Approach */
"0NS604Z"  =  "1"  /* Reposition Left Temporal Bone with Int Fix, Open Approach */
"0NS60ZZ"  =  "1"  /* Reposition Left Temporal Bone, Open Approach */
"0NS634Z"  =  "1"  /* Reposition Left Temporal Bone with Int Fix, Perc Approach */
"0NS63ZZ"  =  "1"  /* Reposition Left Temporal Bone, Percutaneous Approach */
"0NS644Z"  =  "1"  /* Reposition L Temporal Bone with Int Fix, Perc Endo Approach */
"0NS64ZZ"  =  "1"  /* Reposition Left Temporal Bone, Perc Endo Approach */
"0NS6XZZ"  =  "1"  /* Reposition Left Temporal Bone, External Approach */
"0NS704Z"  =  "1"  /* Reposition Right Occipital Bone with Int Fix, Open Approach */
"0NS70ZZ"  =  "1"  /* Reposition Right Occipital Bone, Open Approach */
"0NS734Z"  =  "1"  /* Reposition Right Occipital Bone with Int Fix, Perc Approach */
"0NS73ZZ"  =  "1"  /* Reposition Right Occipital Bone, Percutaneous Approach */
"0NS744Z"  =  "1"  /* Reposition R Occipital Bone with Int Fix, Perc Endo Approach */
"0NS74ZZ"  =  "1"  /* Reposition Right Occipital Bone, Perc Endo Approach */
"0NS7XZZ"  =  "1"  /* Reposition Right Occipital Bone, External Approach */
"0NS804Z"  =  "1"  /* Reposition Left Occipital Bone with Int Fix, Open Approach */
"0NS80ZZ"  =  "1"  /* Reposition Left Occipital Bone, Open Approach */
"0NS834Z"  =  "1"  /* Reposition Left Occipital Bone with Int Fix, Perc Approach */
"0NS83ZZ"  =  "1"  /* Reposition Left Occipital Bone, Percutaneous Approach */
"0NS844Z"  =  "1"  /* Reposition L Occipital Bone with Int Fix, Perc Endo Approach */
"0NS84ZZ"  =  "1"  /* Reposition Left Occipital Bone, Perc Endo Approach */
"0NS8XZZ"  =  "1"  /* Reposition Left Occipital Bone, External Approach */
"0NSB04Z"  =  "1"  /* Reposition Nasal Bone with Int Fix, Open Approach */
"0NSB0ZZ"  =  "1"  /* Reposition Nasal Bone, Open Approach */
"0NSC04Z"  =  "1"  /* Reposition Right Sphenoid Bone with Int Fix, Open Approach */
"0NSC0ZZ"  =  "1"  /* Reposition Right Sphenoid Bone, Open Approach */
"0NSD04Z"  =  "1"  /* Reposition Left Sphenoid Bone with Int Fix, Open Approach */
"0NSD0ZZ"  =  "1"  /* Reposition Left Sphenoid Bone, Open Approach */
"0NSF04Z"  =  "1"  /* Reposition Right Ethmoid Bone with Int Fix, Open Approach */
"0NSF0ZZ"  =  "1"  /* Reposition Right Ethmoid Bone, Open Approach */
"0NSG04Z"  =  "1"  /* Reposition Left Ethmoid Bone with Int Fix, Open Approach */
"0NSG0ZZ"  =  "1"  /* Reposition Left Ethmoid Bone, Open Approach */
"0NSH04Z"  =  "1"  /* Reposition Right Lacrimal Bone with Int Fix, Open Approach */
"0NSH0ZZ"  =  "1"  /* Reposition Right Lacrimal Bone, Open Approach */
"0NSJ04Z"  =  "1"  /* Reposition Left Lacrimal Bone with Int Fix, Open Approach */
"0NSJ0ZZ"  =  "1"  /* Reposition Left Lacrimal Bone, Open Approach */
"0NSK04Z"  =  "1"  /* Reposition Right Palatine Bone with Int Fix, Open Approach */
"0NSK0ZZ"  =  "1"  /* Reposition Right Palatine Bone, Open Approach */
"0NSL04Z"  =  "1"  /* Reposition Left Palatine Bone with Int Fix, Open Approach */
"0NSL0ZZ"  =  "1"  /* Reposition Left Palatine Bone, Open Approach */
"0NSM04Z"  =  "1"  /* Reposition Right Zygomatic Bone with Int Fix, Open Approach */
"0NSM0ZZ"  =  "1"  /* Reposition Right Zygomatic Bone, Open Approach */
"0NSN04Z"  =  "1"  /* Reposition Left Zygomatic Bone with Int Fix, Open Approach */
"0NSN0ZZ"  =  "1"  /* Reposition Left Zygomatic Bone, Open Approach */
"0NSP04Z"  =  "1"  /* Reposition Right Orbit with Int Fix, Open Approach */
"0NSP0ZZ"  =  "1"  /* Reposition Right Orbit, Open Approach */
"0NSQ04Z"  =  "1"  /* Reposition Left Orbit with Int Fix, Open Approach */
"0NSQ0ZZ"  =  "1"  /* Reposition Left Orbit, Open Approach */
"0NSR04Z"  =  "1"  /* Reposition Right Maxilla with Int Fix, Open Approach */
"0NSR05Z"  =  "1"  /* Reposition Right Maxilla with Ext Fix, Open Approach */
"0NSR0ZZ"  =  "1"  /* Reposition Right Maxilla, Open Approach */
"0NSS04Z"  =  "1"  /* Reposition Left Maxilla with Int Fix, Open Approach */
"0NSS05Z"  =  "1"  /* Reposition Left Maxilla with Ext Fix, Open Approach */
"0NSS0ZZ"  =  "1"  /* Reposition Left Maxilla, Open Approach */
"0NST04Z"  =  "1"  /* Reposition Right Mandible with Int Fix, Open Approach */
"0NST05Z"  =  "1"  /* Reposition Right Mandible with Ext Fix, Open Approach */
"0NST0ZZ"  =  "1"  /* Reposition Right Mandible, Open Approach */
"0NSV04Z"  =  "1"  /* Reposition Left Mandible with Int Fix, Open Approach */
"0NSV05Z"  =  "1"  /* Reposition Left Mandible with Ext Fix, Open Approach */
"0NSV0ZZ"  =  "1"  /* Reposition Left Mandible, Open Approach */
"0NSX04Z"  =  "1"  /* Reposition Hyoid Bone with Int Fix, Open Approach */
"0NSX0ZZ"  =  "1"  /* Reposition Hyoid Bone, Open Approach */
"0NT10ZZ"  =  "1"  /* Resection of Right Frontal Bone, Open Approach */
"0NT20ZZ"  =  "1"  /* Resection of Left Frontal Bone, Open Approach */
"0NT30ZZ"  =  "1"  /* Resection of Right Parietal Bone, Open Approach */
"0NT40ZZ"  =  "1"  /* Resection of Left Parietal Bone, Open Approach */
"0NT50ZZ"  =  "1"  /* Resection of Right Temporal Bone, Open Approach */
"0NT60ZZ"  =  "1"  /* Resection of Left Temporal Bone, Open Approach */
"0NT70ZZ"  =  "1"  /* Resection of Right Occipital Bone, Open Approach */
"0NT80ZZ"  =  "1"  /* Resection of Left Occipital Bone, Open Approach */
"0NTB0ZZ"  =  "1"  /* Resection of Nasal Bone, Open Approach */
"0NTC0ZZ"  =  "1"  /* Resection of Right Sphenoid Bone, Open Approach */
"0NTD0ZZ"  =  "1"  /* Resection of Left Sphenoid Bone, Open Approach */
"0NTF0ZZ"  =  "1"  /* Resection of Right Ethmoid Bone, Open Approach */
"0NTG0ZZ"  =  "1"  /* Resection of Left Ethmoid Bone, Open Approach */
"0NTH0ZZ"  =  "1"  /* Resection of Right Lacrimal Bone, Open Approach */
"0NTJ0ZZ"  =  "1"  /* Resection of Left Lacrimal Bone, Open Approach */
"0NTK0ZZ"  =  "1"  /* Resection of Right Palatine Bone, Open Approach */
"0NTL0ZZ"  =  "1"  /* Resection of Left Palatine Bone, Open Approach */
"0NTM0ZZ"  =  "1"  /* Resection of Right Zygomatic Bone, Open Approach */
"0NTN0ZZ"  =  "1"  /* Resection of Left Zygomatic Bone, Open Approach */
"0NTP0ZZ"  =  "1"  /* Resection of Right Orbit, Open Approach */
"0NTQ0ZZ"  =  "1"  /* Resection of Left Orbit, Open Approach */
"0NTR0ZZ"  =  "1"  /* Resection of Right Maxilla, Open Approach */
"0NTS0ZZ"  =  "1"  /* Resection of Left Maxilla, Open Approach */
"0NTT0ZZ"  =  "1"  /* Resection of Right Mandible, Open Approach */
"0NTV0ZZ"  =  "1"  /* Resection of Left Mandible, Open Approach */
"0NTX0ZZ"  =  "1"  /* Resection of Hyoid Bone, Open Approach */
"0NU007Z"  =  "1"  /* Supplement Skull with Autol Sub, Open Approach */
"0NU00JZ"  =  "1"  /* Supplement Skull with Synthetic Substitute, Open Approach */
"0NU00KZ"  =  "1"  /* Supplement Skull with Nonaut Sub, Open Approach */
"0NU037Z"  =  "1"  /* Supplement Skull with Autol Sub, Perc Approach */
"0NU03JZ"  =  "1"  /* Supplement Skull with Synthetic Substitute, Perc Approach */
"0NU03KZ"  =  "1"  /* Supplement Skull with Nonaut Sub, Perc Approach */
"0NU047Z"  =  "1"  /* Supplement Skull with Autol Sub, Perc Endo Approach */
"0NU04JZ"  =  "1"  /* Supplement Skull with Synth Sub, Perc Endo Approach */
"0NU04KZ"  =  "1"  /* Supplement Skull with Nonaut Sub, Perc Endo Approach */
"0NU107Z"  =  "1"  /* Supplement Right Frontal Bone with Autol Sub, Open Approach */
"0NU10JZ"  =  "1"  /* Supplement Right Frontal Bone with Synth Sub, Open Approach */
"0NU10KZ"  =  "1"  /* Supplement Right Frontal Bone with Nonaut Sub, Open Approach */
"0NU137Z"  =  "1"  /* Supplement Right Frontal Bone with Autol Sub, Perc Approach */
"0NU13JZ"  =  "1"  /* Supplement Right Frontal Bone with Synth Sub, Perc Approach */
"0NU13KZ"  =  "1"  /* Supplement Right Frontal Bone with Nonaut Sub, Perc Approach */
"0NU147Z"  =  "1"  /* Supplement R Frontal Bone with Autol Sub, Perc Endo Approach */
"0NU14JZ"  =  "1"  /* Supplement R Frontal Bone with Synth Sub, Perc Endo Approach */
"0NU14KZ"  =  "1"  /* Supplement R Frontal Bone w Nonaut Sub, Perc Endo */
"0NU207Z"  =  "1"  /* Supplement Left Frontal Bone with Autol Sub, Open Approach */
"0NU20JZ"  =  "1"  /* Supplement Left Frontal Bone with Synth Sub, Open Approach */
"0NU20KZ"  =  "1"  /* Supplement Left Frontal Bone with Nonaut Sub, Open Approach */
"0NU237Z"  =  "1"  /* Supplement Left Frontal Bone with Autol Sub, Perc Approach */
"0NU23JZ"  =  "1"  /* Supplement Left Frontal Bone with Synth Sub, Perc Approach */
"0NU23KZ"  =  "1"  /* Supplement Left Frontal Bone with Nonaut Sub, Perc Approach */
"0NU247Z"  =  "1"  /* Supplement L Frontal Bone with Autol Sub, Perc Endo Approach */
"0NU24JZ"  =  "1"  /* Supplement L Frontal Bone with Synth Sub, Perc Endo Approach */
"0NU24KZ"  =  "1"  /* Supplement L Frontal Bone w Nonaut Sub, Perc Endo */
"0NU307Z"  =  "1"  /* Supplement Right Parietal Bone with Autol Sub, Open Approach */
"0NU30JZ"  =  "1"  /* Supplement Right Parietal Bone with Synth Sub, Open Approach */
"0NU30KZ"  =  "1"  /* Supplement R Parietal Bone with Nonaut Sub, Open Approach */
"0NU337Z"  =  "1"  /* Supplement Right Parietal Bone with Autol Sub, Perc Approach */
"0NU33JZ"  =  "1"  /* Supplement Right Parietal Bone with Synth Sub, Perc Approach */
"0NU33KZ"  =  "1"  /* Supplement R Parietal Bone with Nonaut Sub, Perc Approach */
"0NU347Z"  =  "1"  /* Supplement R Parietal Bone w Autol Sub, Perc Endo */
"0NU34JZ"  =  "1"  /* Supplement R Parietal Bone w Synth Sub, Perc Endo */
"0NU34KZ"  =  "1"  /* Supplement R Parietal Bone w Nonaut Sub, Perc Endo */
"0NU407Z"  =  "1"  /* Supplement Left Parietal Bone with Autol Sub, Open Approach */
"0NU40JZ"  =  "1"  /* Supplement Left Parietal Bone with Synth Sub, Open Approach */
"0NU40KZ"  =  "1"  /* Supplement Left Parietal Bone with Nonaut Sub, Open Approach */
"0NU437Z"  =  "1"  /* Supplement Left Parietal Bone with Autol Sub, Perc Approach */
"0NU43JZ"  =  "1"  /* Supplement Left Parietal Bone with Synth Sub, Perc Approach */
"0NU43KZ"  =  "1"  /* Supplement Left Parietal Bone with Nonaut Sub, Perc Approach */
"0NU447Z"  =  "1"  /* Supplement L Parietal Bone w Autol Sub, Perc Endo */
"0NU44JZ"  =  "1"  /* Supplement L Parietal Bone w Synth Sub, Perc Endo */
"0NU44KZ"  =  "1"  /* Supplement L Parietal Bone w Nonaut Sub, Perc Endo */
"0NU507Z"  =  "1"  /* Supplement Right Temporal Bone with Autol Sub, Open Approach */
"0NU50JZ"  =  "1"  /* Supplement Right Temporal Bone with Synth Sub, Open Approach */
"0NU50KZ"  =  "1"  /* Supplement R Temporal Bone with Nonaut Sub, Open Approach */
"0NU537Z"  =  "1"  /* Supplement Right Temporal Bone with Autol Sub, Perc Approach */
"0NU53JZ"  =  "1"  /* Supplement Right Temporal Bone with Synth Sub, Perc Approach */
"0NU53KZ"  =  "1"  /* Supplement R Temporal Bone with Nonaut Sub, Perc Approach */
"0NU547Z"  =  "1"  /* Supplement R Temporal Bone w Autol Sub, Perc Endo */
"0NU54JZ"  =  "1"  /* Supplement R Temporal Bone w Synth Sub, Perc Endo */
"0NU54KZ"  =  "1"  /* Supplement R Temporal Bone w Nonaut Sub, Perc Endo */
"0NU607Z"  =  "1"  /* Supplement Left Temporal Bone with Autol Sub, Open Approach */
"0NU60JZ"  =  "1"  /* Supplement Left Temporal Bone with Synth Sub, Open Approach */
"0NU60KZ"  =  "1"  /* Supplement Left Temporal Bone with Nonaut Sub, Open Approach */
"0NU637Z"  =  "1"  /* Supplement Left Temporal Bone with Autol Sub, Perc Approach */
"0NU63JZ"  =  "1"  /* Supplement Left Temporal Bone with Synth Sub, Perc Approach */
"0NU63KZ"  =  "1"  /* Supplement Left Temporal Bone with Nonaut Sub, Perc Approach */
"0NU647Z"  =  "1"  /* Supplement L Temporal Bone w Autol Sub, Perc Endo */
"0NU64JZ"  =  "1"  /* Supplement L Temporal Bone w Synth Sub, Perc Endo */
"0NU64KZ"  =  "1"  /* Supplement L Temporal Bone w Nonaut Sub, Perc Endo */
"0NU707Z"  =  "1"  /* Supplement R Occipital Bone with Autol Sub, Open Approach */
"0NU70JZ"  =  "1"  /* Supplement R Occipital Bone with Synth Sub, Open Approach */
"0NU70KZ"  =  "1"  /* Supplement R Occipital Bone with Nonaut Sub, Open Approach */
"0NU737Z"  =  "1"  /* Supplement R Occipital Bone with Autol Sub, Perc Approach */
"0NU73JZ"  =  "1"  /* Supplement R Occipital Bone with Synth Sub, Perc Approach */
"0NU73KZ"  =  "1"  /* Supplement R Occipital Bone with Nonaut Sub, Perc Approach */
"0NU747Z"  =  "1"  /* Supplement R Occipital Bone w Autol Sub, Perc Endo */
"0NU74JZ"  =  "1"  /* Supplement R Occipital Bone w Synth Sub, Perc Endo */
"0NU74KZ"  =  "1"  /* Supplement R Occipital Bone w Nonaut Sub, Perc Endo */
"0NU807Z"  =  "1"  /* Supplement Left Occipital Bone with Autol Sub, Open Approach */
"0NU80JZ"  =  "1"  /* Supplement Left Occipital Bone with Synth Sub, Open Approach */
"0NU80KZ"  =  "1"  /* Supplement L Occipital Bone with Nonaut Sub, Open Approach */
"0NU837Z"  =  "1"  /* Supplement Left Occipital Bone with Autol Sub, Perc Approach */
"0NU83JZ"  =  "1"  /* Supplement Left Occipital Bone with Synth Sub, Perc Approach */
"0NU83KZ"  =  "1"  /* Supplement L Occipital Bone with Nonaut Sub, Perc Approach */
"0NU847Z"  =  "1"  /* Supplement L Occipital Bone w Autol Sub, Perc Endo */
"0NU84JZ"  =  "1"  /* Supplement L Occipital Bone w Synth Sub, Perc Endo */
"0NU84KZ"  =  "1"  /* Supplement L Occipital Bone w Nonaut Sub, Perc Endo */
"0NUB07Z"  =  "1"  /* Supplement Nasal Bone with Autol Sub, Open Approach */
"0NUB0JZ"  =  "1"  /* Supplement Nasal Bone with Synth Sub, Open Approach */
"0NUB0KZ"  =  "1"  /* Supplement Nasal Bone with Nonaut Sub, Open Approach */
"0NUB37Z"  =  "1"  /* Supplement Nasal Bone with Autol Sub, Perc Approach */
"0NUB3JZ"  =  "1"  /* Supplement Nasal Bone with Synth Sub, Perc Approach */
"0NUB3KZ"  =  "1"  /* Supplement Nasal Bone with Nonaut Sub, Perc Approach */
"0NUB47Z"  =  "1"  /* Supplement Nasal Bone with Autol Sub, Perc Endo Approach */
"0NUB4JZ"  =  "1"  /* Supplement Nasal Bone with Synth Sub, Perc Endo Approach */
"0NUB4KZ"  =  "1"  /* Supplement Nasal Bone with Nonaut Sub, Perc Endo Approach */
"0NUC07Z"  =  "1"  /* Supplement Right Sphenoid Bone with Autol Sub, Open Approach */
"0NUC0JZ"  =  "1"  /* Supplement Right Sphenoid Bone with Synth Sub, Open Approach */
"0NUC0KZ"  =  "1"  /* Supplement R Sphenoid Bone with Nonaut Sub, Open Approach */
"0NUC37Z"  =  "1"  /* Supplement Right Sphenoid Bone with Autol Sub, Perc Approach */
"0NUC3JZ"  =  "1"  /* Supplement Right Sphenoid Bone with Synth Sub, Perc Approach */
"0NUC3KZ"  =  "1"  /* Supplement R Sphenoid Bone with Nonaut Sub, Perc Approach */
"0NUC47Z"  =  "1"  /* Supplement R Sphenoid Bone w Autol Sub, Perc Endo */
"0NUC4JZ"  =  "1"  /* Supplement R Sphenoid Bone w Synth Sub, Perc Endo */
"0NUC4KZ"  =  "1"  /* Supplement R Sphenoid Bone w Nonaut Sub, Perc Endo */
"0NUD07Z"  =  "1"  /* Supplement Left Sphenoid Bone with Autol Sub, Open Approach */
"0NUD0JZ"  =  "1"  /* Supplement Left Sphenoid Bone with Synth Sub, Open Approach */
"0NUD0KZ"  =  "1"  /* Supplement Left Sphenoid Bone with Nonaut Sub, Open Approach */
"0NUD37Z"  =  "1"  /* Supplement Left Sphenoid Bone with Autol Sub, Perc Approach */
"0NUD3JZ"  =  "1"  /* Supplement Left Sphenoid Bone with Synth Sub, Perc Approach */
"0NUD3KZ"  =  "1"  /* Supplement Left Sphenoid Bone with Nonaut Sub, Perc Approach */
"0NUD47Z"  =  "1"  /* Supplement L Sphenoid Bone w Autol Sub, Perc Endo */
"0NUD4JZ"  =  "1"  /* Supplement L Sphenoid Bone w Synth Sub, Perc Endo */
"0NUD4KZ"  =  "1"  /* Supplement L Sphenoid Bone w Nonaut Sub, Perc Endo */
"0NUF07Z"  =  "1"  /* Supplement Right Ethmoid Bone with Autol Sub, Open Approach */
"0NUF0JZ"  =  "1"  /* Supplement Right Ethmoid Bone with Synth Sub, Open Approach */
"0NUF0KZ"  =  "1"  /* Supplement Right Ethmoid Bone with Nonaut Sub, Open Approach */
"0NUF37Z"  =  "1"  /* Supplement Right Ethmoid Bone with Autol Sub, Perc Approach */
"0NUF3JZ"  =  "1"  /* Supplement Right Ethmoid Bone with Synth Sub, Perc Approach */
"0NUF3KZ"  =  "1"  /* Supplement Right Ethmoid Bone with Nonaut Sub, Perc Approach */
"0NUF47Z"  =  "1"  /* Supplement R Ethmoid Bone with Autol Sub, Perc Endo Approach */
"0NUF4JZ"  =  "1"  /* Supplement R Ethmoid Bone with Synth Sub, Perc Endo Approach */
"0NUF4KZ"  =  "1"  /* Supplement R Ethmoid Bone w Nonaut Sub, Perc Endo */
"0NUG07Z"  =  "1"  /* Supplement Left Ethmoid Bone with Autol Sub, Open Approach */
"0NUG0JZ"  =  "1"  /* Supplement Left Ethmoid Bone with Synth Sub, Open Approach */
"0NUG0KZ"  =  "1"  /* Supplement Left Ethmoid Bone with Nonaut Sub, Open Approach */
"0NUG37Z"  =  "1"  /* Supplement Left Ethmoid Bone with Autol Sub, Perc Approach */
"0NUG3JZ"  =  "1"  /* Supplement Left Ethmoid Bone with Synth Sub, Perc Approach */
"0NUG3KZ"  =  "1"  /* Supplement Left Ethmoid Bone with Nonaut Sub, Perc Approach */
"0NUG47Z"  =  "1"  /* Supplement L Ethmoid Bone with Autol Sub, Perc Endo Approach */
"0NUG4JZ"  =  "1"  /* Supplement L Ethmoid Bone with Synth Sub, Perc Endo Approach */
"0NUG4KZ"  =  "1"  /* Supplement L Ethmoid Bone w Nonaut Sub, Perc Endo */
"0NUH07Z"  =  "1"  /* Supplement Right Lacrimal Bone with Autol Sub, Open Approach */
"0NUH0JZ"  =  "1"  /* Supplement Right Lacrimal Bone with Synth Sub, Open Approach */
"0NUH0KZ"  =  "1"  /* Supplement R Lacrimal Bone with Nonaut Sub, Open Approach */
"0NUH37Z"  =  "1"  /* Supplement Right Lacrimal Bone with Autol Sub, Perc Approach */
"0NUH3JZ"  =  "1"  /* Supplement Right Lacrimal Bone with Synth Sub, Perc Approach */
"0NUH3KZ"  =  "1"  /* Supplement R Lacrimal Bone with Nonaut Sub, Perc Approach */
"0NUH47Z"  =  "1"  /* Supplement R Lacrimal Bone w Autol Sub, Perc Endo */
"0NUH4JZ"  =  "1"  /* Supplement R Lacrimal Bone w Synth Sub, Perc Endo */
"0NUH4KZ"  =  "1"  /* Supplement R Lacrimal Bone w Nonaut Sub, Perc Endo */
"0NUJ07Z"  =  "1"  /* Supplement Left Lacrimal Bone with Autol Sub, Open Approach */
"0NUJ0JZ"  =  "1"  /* Supplement Left Lacrimal Bone with Synth Sub, Open Approach */
"0NUJ0KZ"  =  "1"  /* Supplement Left Lacrimal Bone with Nonaut Sub, Open Approach */
"0NUJ37Z"  =  "1"  /* Supplement Left Lacrimal Bone with Autol Sub, Perc Approach */
"0NUJ3JZ"  =  "1"  /* Supplement Left Lacrimal Bone with Synth Sub, Perc Approach */
"0NUJ3KZ"  =  "1"  /* Supplement Left Lacrimal Bone with Nonaut Sub, Perc Approach */
"0NUJ47Z"  =  "1"  /* Supplement L Lacrimal Bone w Autol Sub, Perc Endo */
"0NUJ4JZ"  =  "1"  /* Supplement L Lacrimal Bone w Synth Sub, Perc Endo */
"0NUJ4KZ"  =  "1"  /* Supplement L Lacrimal Bone w Nonaut Sub, Perc Endo */
"0NUK07Z"  =  "1"  /* Supplement Right Palatine Bone with Autol Sub, Open Approach */
"0NUK0JZ"  =  "1"  /* Supplement Right Palatine Bone with Synth Sub, Open Approach */
"0NUK0KZ"  =  "1"  /* Supplement R Palatine Bone with Nonaut Sub, Open Approach */
"0NUK37Z"  =  "1"  /* Supplement Right Palatine Bone with Autol Sub, Perc Approach */
"0NUK3JZ"  =  "1"  /* Supplement Right Palatine Bone with Synth Sub, Perc Approach */
"0NUK3KZ"  =  "1"  /* Supplement R Palatine Bone with Nonaut Sub, Perc Approach */
"0NUK47Z"  =  "1"  /* Supplement R Palatine Bone w Autol Sub, Perc Endo */
"0NUK4JZ"  =  "1"  /* Supplement R Palatine Bone w Synth Sub, Perc Endo */
"0NUK4KZ"  =  "1"  /* Supplement R Palatine Bone w Nonaut Sub, Perc Endo */
"0NUL07Z"  =  "1"  /* Supplement Left Palatine Bone with Autol Sub, Open Approach */
"0NUL0JZ"  =  "1"  /* Supplement Left Palatine Bone with Synth Sub, Open Approach */
"0NUL0KZ"  =  "1"  /* Supplement Left Palatine Bone with Nonaut Sub, Open Approach */
"0NUL37Z"  =  "1"  /* Supplement Left Palatine Bone with Autol Sub, Perc Approach */
"0NUL3JZ"  =  "1"  /* Supplement Left Palatine Bone with Synth Sub, Perc Approach */
"0NUL3KZ"  =  "1"  /* Supplement Left Palatine Bone with Nonaut Sub, Perc Approach */
"0NUL47Z"  =  "1"  /* Supplement L Palatine Bone w Autol Sub, Perc Endo */
"0NUL4JZ"  =  "1"  /* Supplement L Palatine Bone w Synth Sub, Perc Endo */
"0NUL4KZ"  =  "1"  /* Supplement L Palatine Bone w Nonaut Sub, Perc Endo */
"0NUM07Z"  =  "1"  /* Supplement R Zygomatic Bone with Autol Sub, Open Approach */
"0NUM0JZ"  =  "1"  /* Supplement R Zygomatic Bone with Synth Sub, Open Approach */
"0NUM0KZ"  =  "1"  /* Supplement R Zygomatic Bone with Nonaut Sub, Open Approach */
"0NUM37Z"  =  "1"  /* Supplement R Zygomatic Bone with Autol Sub, Perc Approach */
"0NUM3JZ"  =  "1"  /* Supplement R Zygomatic Bone with Synth Sub, Perc Approach */
"0NUM3KZ"  =  "1"  /* Supplement R Zygomatic Bone with Nonaut Sub, Perc Approach */
"0NUM47Z"  =  "1"  /* Supplement R Zygomatic Bone w Autol Sub, Perc Endo */
"0NUM4JZ"  =  "1"  /* Supplement R Zygomatic Bone w Synth Sub, Perc Endo */
"0NUM4KZ"  =  "1"  /* Supplement R Zygomatic Bone w Nonaut Sub, Perc Endo */
"0NUN07Z"  =  "1"  /* Supplement Left Zygomatic Bone with Autol Sub, Open Approach */
"0NUN0JZ"  =  "1"  /* Supplement Left Zygomatic Bone with Synth Sub, Open Approach */
"0NUN0KZ"  =  "1"  /* Supplement L Zygomatic Bone with Nonaut Sub, Open Approach */
"0NUN37Z"  =  "1"  /* Supplement Left Zygomatic Bone with Autol Sub, Perc Approach */
"0NUN3JZ"  =  "1"  /* Supplement Left Zygomatic Bone with Synth Sub, Perc Approach */
"0NUN3KZ"  =  "1"  /* Supplement L Zygomatic Bone with Nonaut Sub, Perc Approach */
"0NUN47Z"  =  "1"  /* Supplement L Zygomatic Bone w Autol Sub, Perc Endo */
"0NUN4JZ"  =  "1"  /* Supplement L Zygomatic Bone w Synth Sub, Perc Endo */
"0NUN4KZ"  =  "1"  /* Supplement L Zygomatic Bone w Nonaut Sub, Perc Endo */
"0NUP07Z"  =  "1"  /* Supplement Right Orbit with Autol Sub, Open Approach */
"0NUP0JZ"  =  "1"  /* Supplement Right Orbit with Synth Sub, Open Approach */
"0NUP0KZ"  =  "1"  /* Supplement Right Orbit with Nonaut Sub, Open Approach */
"0NUP37Z"  =  "1"  /* Supplement Right Orbit with Autol Sub, Perc Approach */
"0NUP3JZ"  =  "1"  /* Supplement Right Orbit with Synth Sub, Perc Approach */
"0NUP3KZ"  =  "1"  /* Supplement Right Orbit with Nonaut Sub, Perc Approach */
"0NUP47Z"  =  "1"  /* Supplement Right Orbit with Autol Sub, Perc Endo Approach */
"0NUP4JZ"  =  "1"  /* Supplement Right Orbit with Synth Sub, Perc Endo Approach */
"0NUP4KZ"  =  "1"  /* Supplement Right Orbit with Nonaut Sub, Perc Endo Approach */
"0NUQ07Z"  =  "1"  /* Supplement Left Orbit with Autol Sub, Open Approach */
"0NUQ0JZ"  =  "1"  /* Supplement Left Orbit with Synth Sub, Open Approach */
"0NUQ0KZ"  =  "1"  /* Supplement Left Orbit with Nonaut Sub, Open Approach */
"0NUQ37Z"  =  "1"  /* Supplement Left Orbit with Autol Sub, Perc Approach */
"0NUQ3JZ"  =  "1"  /* Supplement Left Orbit with Synth Sub, Perc Approach */
"0NUQ3KZ"  =  "1"  /* Supplement Left Orbit with Nonaut Sub, Perc Approach */
"0NUQ47Z"  =  "1"  /* Supplement Left Orbit with Autol Sub, Perc Endo Approach */
"0NUQ4JZ"  =  "1"  /* Supplement Left Orbit with Synth Sub, Perc Endo Approach */
"0NUQ4KZ"  =  "1"  /* Supplement Left Orbit with Nonaut Sub, Perc Endo Approach */
"0NUR07Z"  =  "1"  /* Supplement Right Maxilla with Autol Sub, Open Approach */
"0NUR0JZ"  =  "1"  /* Supplement Right Maxilla with Synth Sub, Open Approach */
"0NUR0KZ"  =  "1"  /* Supplement Right Maxilla with Nonaut Sub, Open Approach */
"0NUR37Z"  =  "1"  /* Supplement Right Maxilla with Autol Sub, Perc Approach */
"0NUR3JZ"  =  "1"  /* Supplement Right Maxilla with Synth Sub, Perc Approach */
"0NUR3KZ"  =  "1"  /* Supplement Right Maxilla with Nonaut Sub, Perc Approach */
"0NUR47Z"  =  "1"  /* Supplement Right Maxilla with Autol Sub, Perc Endo Approach */
"0NUR4JZ"  =  "1"  /* Supplement Right Maxilla with Synth Sub, Perc Endo Approach */
"0NUR4KZ"  =  "1"  /* Supplement Right Maxilla with Nonaut Sub, Perc Endo Approach */
"0NUS07Z"  =  "1"  /* Supplement Left Maxilla with Autol Sub, Open Approach */
"0NUS0JZ"  =  "1"  /* Supplement Left Maxilla with Synth Sub, Open Approach */
"0NUS0KZ"  =  "1"  /* Supplement Left Maxilla with Nonaut Sub, Open Approach */
"0NUS37Z"  =  "1"  /* Supplement Left Maxilla with Autol Sub, Perc Approach */
"0NUS3JZ"  =  "1"  /* Supplement Left Maxilla with Synth Sub, Perc Approach */
"0NUS3KZ"  =  "1"  /* Supplement Left Maxilla with Nonaut Sub, Perc Approach */
"0NUS47Z"  =  "1"  /* Supplement Left Maxilla with Autol Sub, Perc Endo Approach */
"0NUS4JZ"  =  "1"  /* Supplement Left Maxilla with Synth Sub, Perc Endo Approach */
"0NUS4KZ"  =  "1"  /* Supplement Left Maxilla with Nonaut Sub, Perc Endo Approach */
"0NUT07Z"  =  "1"  /* Supplement Right Mandible with Autol Sub, Open Approach */
"0NUT0JZ"  =  "1"  /* Supplement Right Mandible with Synth Sub, Open Approach */
"0NUT0KZ"  =  "1"  /* Supplement Right Mandible with Nonaut Sub, Open Approach */
"0NUT37Z"  =  "1"  /* Supplement Right Mandible with Autol Sub, Perc Approach */
"0NUT3JZ"  =  "1"  /* Supplement Right Mandible with Synth Sub, Perc Approach */
"0NUT3KZ"  =  "1"  /* Supplement Right Mandible with Nonaut Sub, Perc Approach */
"0NUT47Z"  =  "1"  /* Supplement Right Mandible with Autol Sub, Perc Endo Approach */
"0NUT4JZ"  =  "1"  /* Supplement Right Mandible with Synth Sub, Perc Endo Approach */
"0NUT4KZ"  =  "1"  /* Supplement R Mandible with Nonaut Sub, Perc Endo Approach */
"0NUV07Z"  =  "1"  /* Supplement Left Mandible with Autol Sub, Open Approach */
"0NUV0JZ"  =  "1"  /* Supplement Left Mandible with Synth Sub, Open Approach */
"0NUV0KZ"  =  "1"  /* Supplement Left Mandible with Nonaut Sub, Open Approach */
"0NUV37Z"  =  "1"  /* Supplement Left Mandible with Autol Sub, Perc Approach */
"0NUV3JZ"  =  "1"  /* Supplement Left Mandible with Synth Sub, Perc Approach */
"0NUV3KZ"  =  "1"  /* Supplement Left Mandible with Nonaut Sub, Perc Approach */
"0NUV47Z"  =  "1"  /* Supplement Left Mandible with Autol Sub, Perc Endo Approach */
"0NUV4JZ"  =  "1"  /* Supplement Left Mandible with Synth Sub, Perc Endo Approach */
"0NUV4KZ"  =  "1"  /* Supplement Left Mandible with Nonaut Sub, Perc Endo Approach */
"0NUX07Z"  =  "1"  /* Supplement Hyoid Bone with Autol Sub, Open Approach */
"0NUX0JZ"  =  "1"  /* Supplement Hyoid Bone with Synth Sub, Open Approach */
"0NUX0KZ"  =  "1"  /* Supplement Hyoid Bone with Nonaut Sub, Open Approach */
"0NUX37Z"  =  "1"  /* Supplement Hyoid Bone with Autol Sub, Perc Approach */
"0NUX3JZ"  =  "1"  /* Supplement Hyoid Bone with Synth Sub, Perc Approach */
"0NUX3KZ"  =  "1"  /* Supplement Hyoid Bone with Nonaut Sub, Perc Approach */
"0NUX47Z"  =  "1"  /* Supplement Hyoid Bone with Autol Sub, Perc Endo Approach */
"0NUX4JZ"  =  "1"  /* Supplement Hyoid Bone with Synth Sub, Perc Endo Approach */
"0NUX4KZ"  =  "1"  /* Supplement Hyoid Bone with Nonaut Sub, Perc Endo Approach */
"0NW000Z"  =  "1"  /* Revision of Drainage Device in Skull, Open Approach */
"0NW004Z"  =  "1"  /* Revision of Internal Fixation Device in Skull, Open Approach */
"0NW005Z"  =  "1"  /* Revision of External Fixation Device in Skull, Open Approach */
"0NW007Z"  =  "1"  /* Revision of Autol Sub in Skull, Open Approach */
"0NW00JZ"  =  "1"  /* Revision of Synthetic Substitute in Skull, Open Approach */
"0NW00KZ"  =  "1"  /* Revision of Nonaut Sub in Skull, Open Approach */
"0NW00MZ"  =  "1"  /* Revision of Bone Growth Stimulator in Skull, Open Approach */
"0NW00NZ"  =  "1"  /* Revision of Neurostim in Skull, Open Approach */
"0NW00SZ"  =  "1"  /* Revision of Hearing Device in Skull, Open Approach */
"0NW030Z"  =  "1"  /* Revision of Drainage Device in Skull, Percutaneous Approach */
"0NW034Z"  =  "1"  /* Revision of Internal Fixation Device in Skull, Perc Approach */
"0NW035Z"  =  "1"  /* Revision of External Fixation Device in Skull, Perc Approach */
"0NW037Z"  =  "1"  /* Revision of Autol Sub in Skull, Perc Approach */
"0NW03JZ"  =  "1"  /* Revision of Synthetic Substitute in Skull, Perc Approach */
"0NW03KZ"  =  "1"  /* Revision of Nonaut Sub in Skull, Perc Approach */
"0NW03MZ"  =  "1"  /* Revision of Bone Growth Stimulator in Skull, Perc Approach */
"0NW03SZ"  =  "1"  /* Revision of Hearing Device in Skull, Percutaneous Approach */
"0NW040Z"  =  "1"  /* Revision of Drainage Device in Skull, Perc Endo Approach */
"0NW044Z"  =  "1"  /* Revision of Int Fix in Skull, Perc Endo Approach */
"0NW045Z"  =  "1"  /* Revision of Ext Fix in Skull, Perc Endo Approach */
"0NW047Z"  =  "1"  /* Revision of Autol Sub in Skull, Perc Endo Approach */
"0NW04JZ"  =  "1"  /* Revision of Synth Sub in Skull, Perc Endo Approach */
"0NW04KZ"  =  "1"  /* Revision of Nonaut Sub in Skull, Perc Endo Approach */
"0NW04MZ"  =  "1"  /* Revision of Bone Stim in Skull, Perc Endo Approach */
"0NW04SZ"  =  "1"  /* Revision of Hearing Device in Skull, Perc Endo Approach */
"0NWW00Z"  =  "1"  /* Revision of Drainage Device in Facial Bone, Open Approach */
"0NWW04Z"  =  "1"  /* Revision of Int Fix in Facial Bone, Open Approach */
"0NWW07Z"  =  "1"  /* Revision of Autol Sub in Facial Bone, Open Approach */
"0NWW0JZ"  =  "1"  /* Revision of Synth Sub in Facial Bone, Open Approach */
"0NWW0KZ"  =  "1"  /* Revision of Nonaut Sub in Facial Bone, Open Approach */
"0NWW0MZ"  =  "1"  /* Revision of Bone Stim in Facial Bone, Open Approach */
"0NWW30Z"  =  "1"  /* Revision of Drainage Device in Facial Bone, Perc Approach */
"0NWW34Z"  =  "1"  /* Revision of Int Fix in Facial Bone, Perc Approach */
"0NWW37Z"  =  "1"  /* Revision of Autol Sub in Facial Bone, Perc Approach */
"0NWW3JZ"  =  "1"  /* Revision of Synth Sub in Facial Bone, Perc Approach */
"0NWW3KZ"  =  "1"  /* Revision of Nonaut Sub in Facial Bone, Perc Approach */
"0NWW3MZ"  =  "1"  /* Revision of Bone Stim in Facial Bone, Perc Approach */
"0NWW40Z"  =  "1"  /* Revision of Drain Dev in Facial Bone, Perc Endo Approach */
"0NWW44Z"  =  "1"  /* Revision of Int Fix in Facial Bone, Perc Endo Approach */
"0NWW47Z"  =  "1"  /* Revision of Autol Sub in Facial Bone, Perc Endo Approach */
"0NWW4JZ"  =  "1"  /* Revision of Synth Sub in Facial Bone, Perc Endo Approach */
"0NWW4KZ"  =  "1"  /* Revision of Nonaut Sub in Facial Bone, Perc Endo Approach */
"0NWW4MZ"  =  "1"  /* Revision of Bone Stim in Facial Bone, Perc Endo Approach */
"0P500ZZ"  =  "1"  /* Destruction of Sternum, Open Approach */
"0P503ZZ"  =  "1"  /* Destruction of Sternum, Percutaneous Approach */
"0P504ZZ"  =  "1"  /* Destruction of Sternum, Percutaneous Endoscopic Approach */
"0P510ZZ"  =  "1"  /* Destruction of Right Rib, Open Approach */
"0P513ZZ"  =  "1"  /* Destruction of Right Rib, Percutaneous Approach */
"0P514ZZ"  =  "1"  /* Destruction of Right Rib, Percutaneous Endoscopic Approach */
"0P520ZZ"  =  "1"  /* Destruction of Left Rib, Open Approach */
"0P523ZZ"  =  "1"  /* Destruction of Left Rib, Percutaneous Approach */
"0P524ZZ"  =  "1"  /* Destruction of Left Rib, Percutaneous Endoscopic Approach */
"0P530ZZ"  =  "1"  /* Destruction of Cervical Vertebra, Open Approach */
"0P533ZZ"  =  "1"  /* Destruction of Cervical Vertebra, Percutaneous Approach */
"0P534ZZ"  =  "1"  /* Destruction of Cervical Vertebra, Perc Endo Approach */
"0P540ZZ"  =  "1"  /* Destruction of Thoracic Vertebra, Open Approach */
"0P543ZZ"  =  "1"  /* Destruction of Thoracic Vertebra, Percutaneous Approach */
"0P544ZZ"  =  "1"  /* Destruction of Thoracic Vertebra, Perc Endo Approach */
"0P550ZZ"  =  "1"  /* Destruction of Right Scapula, Open Approach */
"0P553ZZ"  =  "1"  /* Destruction of Right Scapula, Percutaneous Approach */
"0P554ZZ"  =  "1"  /* Destruction of Right Scapula, Perc Endo Approach */
"0P560ZZ"  =  "1"  /* Destruction of Left Scapula, Open Approach */
"0P563ZZ"  =  "1"  /* Destruction of Left Scapula, Percutaneous Approach */
"0P564ZZ"  =  "1"  /* Destruction of Left Scapula, Perc Endo Approach */
"0P570ZZ"  =  "1"  /* Destruction of Right Glenoid Cavity, Open Approach */
"0P573ZZ"  =  "1"  /* Destruction of Right Glenoid Cavity, Percutaneous Approach */
"0P574ZZ"  =  "1"  /* Destruction of Right Glenoid Cavity, Perc Endo Approach */
"0P580ZZ"  =  "1"  /* Destruction of Left Glenoid Cavity, Open Approach */
"0P583ZZ"  =  "1"  /* Destruction of Left Glenoid Cavity, Percutaneous Approach */
"0P584ZZ"  =  "1"  /* Destruction of Left Glenoid Cavity, Perc Endo Approach */
"0P590ZZ"  =  "1"  /* Destruction of Right Clavicle, Open Approach */
"0P593ZZ"  =  "1"  /* Destruction of Right Clavicle, Percutaneous Approach */
"0P594ZZ"  =  "1"  /* Destruction of Right Clavicle, Perc Endo Approach */
"0P5B0ZZ"  =  "1"  /* Destruction of Left Clavicle, Open Approach */
"0P5B3ZZ"  =  "1"  /* Destruction of Left Clavicle, Percutaneous Approach */
"0P5B4ZZ"  =  "1"  /* Destruction of Left Clavicle, Perc Endo Approach */
"0P5C0ZZ"  =  "1"  /* Destruction of Right Humeral Head, Open Approach */
"0P5C3ZZ"  =  "1"  /* Destruction of Right Humeral Head, Percutaneous Approach */
"0P5C4ZZ"  =  "1"  /* Destruction of Right Humeral Head, Perc Endo Approach */
"0P5D0ZZ"  =  "1"  /* Destruction of Left Humeral Head, Open Approach */
"0P5D3ZZ"  =  "1"  /* Destruction of Left Humeral Head, Percutaneous Approach */
"0P5D4ZZ"  =  "1"  /* Destruction of Left Humeral Head, Perc Endo Approach */
"0P5F0ZZ"  =  "1"  /* Destruction of Right Humeral Shaft, Open Approach */
"0P5F3ZZ"  =  "1"  /* Destruction of Right Humeral Shaft, Percutaneous Approach */
"0P5F4ZZ"  =  "1"  /* Destruction of Right Humeral Shaft, Perc Endo Approach */
"0P5G0ZZ"  =  "1"  /* Destruction of Left Humeral Shaft, Open Approach */
"0P5G3ZZ"  =  "1"  /* Destruction of Left Humeral Shaft, Percutaneous Approach */
"0P5G4ZZ"  =  "1"  /* Destruction of Left Humeral Shaft, Perc Endo Approach */
"0P5H0ZZ"  =  "1"  /* Destruction of Right Radius, Open Approach */
"0P5H3ZZ"  =  "1"  /* Destruction of Right Radius, Percutaneous Approach */
"0P5H4ZZ"  =  "1"  /* Destruction of Right Radius, Perc Endo Approach */
"0P5J0ZZ"  =  "1"  /* Destruction of Left Radius, Open Approach */
"0P5J3ZZ"  =  "1"  /* Destruction of Left Radius, Percutaneous Approach */
"0P5J4ZZ"  =  "1"  /* Destruction of Left Radius, Percutaneous Endoscopic Approach */
"0P5K0ZZ"  =  "1"  /* Destruction of Right Ulna, Open Approach */
"0P5K3ZZ"  =  "1"  /* Destruction of Right Ulna, Percutaneous Approach */
"0P5K4ZZ"  =  "1"  /* Destruction of Right Ulna, Percutaneous Endoscopic Approach */
"0P5L0ZZ"  =  "1"  /* Destruction of Left Ulna, Open Approach */
"0P5L3ZZ"  =  "1"  /* Destruction of Left Ulna, Percutaneous Approach */
"0P5L4ZZ"  =  "1"  /* Destruction of Left Ulna, Percutaneous Endoscopic Approach */
"0P5M0ZZ"  =  "1"  /* Destruction of Right Carpal, Open Approach */
"0P5M3ZZ"  =  "1"  /* Destruction of Right Carpal, Percutaneous Approach */
"0P5M4ZZ"  =  "1"  /* Destruction of Right Carpal, Perc Endo Approach */
"0P5N0ZZ"  =  "1"  /* Destruction of Left Carpal, Open Approach */
"0P5N3ZZ"  =  "1"  /* Destruction of Left Carpal, Percutaneous Approach */
"0P5N4ZZ"  =  "1"  /* Destruction of Left Carpal, Percutaneous Endoscopic Approach */
"0P5P0ZZ"  =  "1"  /* Destruction of Right Metacarpal, Open Approach */
"0P5P3ZZ"  =  "1"  /* Destruction of Right Metacarpal, Percutaneous Approach */
"0P5P4ZZ"  =  "1"  /* Destruction of Right Metacarpal, Perc Endo Approach */
"0P5Q0ZZ"  =  "1"  /* Destruction of Left Metacarpal, Open Approach */
"0P5Q3ZZ"  =  "1"  /* Destruction of Left Metacarpal, Percutaneous Approach */
"0P5Q4ZZ"  =  "1"  /* Destruction of Left Metacarpal, Perc Endo Approach */
"0P5R0ZZ"  =  "1"  /* Destruction of Right Thumb Phalanx, Open Approach */
"0P5R3ZZ"  =  "1"  /* Destruction of Right Thumb Phalanx, Percutaneous Approach */
"0P5R4ZZ"  =  "1"  /* Destruction of Right Thumb Phalanx, Perc Endo Approach */
"0P5S0ZZ"  =  "1"  /* Destruction of Left Thumb Phalanx, Open Approach */
"0P5S3ZZ"  =  "1"  /* Destruction of Left Thumb Phalanx, Percutaneous Approach */
"0P5S4ZZ"  =  "1"  /* Destruction of Left Thumb Phalanx, Perc Endo Approach */
"0P5T0ZZ"  =  "1"  /* Destruction of Right Finger Phalanx, Open Approach */
"0P5T3ZZ"  =  "1"  /* Destruction of Right Finger Phalanx, Percutaneous Approach */
"0P5T4ZZ"  =  "1"  /* Destruction of Right Finger Phalanx, Perc Endo Approach */
"0P5V0ZZ"  =  "1"  /* Destruction of Left Finger Phalanx, Open Approach */
"0P5V3ZZ"  =  "1"  /* Destruction of Left Finger Phalanx, Percutaneous Approach */
"0P5V4ZZ"  =  "1"  /* Destruction of Left Finger Phalanx, Perc Endo Approach */
"0P800ZZ"  =  "1"  /* Division of Sternum, Open Approach */
"0P803ZZ"  =  "1"  /* Division of Sternum, Percutaneous Approach */
"0P804ZZ"  =  "1"  /* Division of Sternum, Percutaneous Endoscopic Approach */
"0P810ZZ"  =  "1"  /* Division of Right Rib, Open Approach */
"0P813ZZ"  =  "1"  /* Division of Right Rib, Percutaneous Approach */
"0P814ZZ"  =  "1"  /* Division of Right Rib, Percutaneous Endoscopic Approach */
"0P820ZZ"  =  "1"  /* Division of Left Rib, Open Approach */
"0P823ZZ"  =  "1"  /* Division of Left Rib, Percutaneous Approach */
"0P824ZZ"  =  "1"  /* Division of Left Rib, Percutaneous Endoscopic Approach */
"0P830ZZ"  =  "1"  /* Division of Cervical Vertebra, Open Approach */
"0P833ZZ"  =  "1"  /* Division of Cervical Vertebra, Percutaneous Approach */
"0P834ZZ"  =  "1"  /* Division of Cervical Vertebra, Perc Endo Approach */
"0P840ZZ"  =  "1"  /* Division of Thoracic Vertebra, Open Approach */
"0P843ZZ"  =  "1"  /* Division of Thoracic Vertebra, Percutaneous Approach */
"0P844ZZ"  =  "1"  /* Division of Thoracic Vertebra, Perc Endo Approach */
"0P850ZZ"  =  "1"  /* Division of Right Scapula, Open Approach */
"0P853ZZ"  =  "1"  /* Division of Right Scapula, Percutaneous Approach */
"0P854ZZ"  =  "1"  /* Division of Right Scapula, Percutaneous Endoscopic Approach */
"0P860ZZ"  =  "1"  /* Division of Left Scapula, Open Approach */
"0P863ZZ"  =  "1"  /* Division of Left Scapula, Percutaneous Approach */
"0P864ZZ"  =  "1"  /* Division of Left Scapula, Percutaneous Endoscopic Approach */
"0P870ZZ"  =  "1"  /* Division of Right Glenoid Cavity, Open Approach */
"0P873ZZ"  =  "1"  /* Division of Right Glenoid Cavity, Percutaneous Approach */
"0P874ZZ"  =  "1"  /* Division of Right Glenoid Cavity, Perc Endo Approach */
"0P880ZZ"  =  "1"  /* Division of Left Glenoid Cavity, Open Approach */
"0P883ZZ"  =  "1"  /* Division of Left Glenoid Cavity, Percutaneous Approach */
"0P884ZZ"  =  "1"  /* Division of Left Glenoid Cavity, Perc Endo Approach */
"0P890ZZ"  =  "1"  /* Division of Right Clavicle, Open Approach */
"0P893ZZ"  =  "1"  /* Division of Right Clavicle, Percutaneous Approach */
"0P894ZZ"  =  "1"  /* Division of Right Clavicle, Percutaneous Endoscopic Approach */
"0P8B0ZZ"  =  "1"  /* Division of Left Clavicle, Open Approach */
"0P8B3ZZ"  =  "1"  /* Division of Left Clavicle, Percutaneous Approach */
"0P8B4ZZ"  =  "1"  /* Division of Left Clavicle, Percutaneous Endoscopic Approach */
"0P8C0ZZ"  =  "1"  /* Division of Right Humeral Head, Open Approach */
"0P8C3ZZ"  =  "1"  /* Division of Right Humeral Head, Percutaneous Approach */
"0P8C4ZZ"  =  "1"  /* Division of Right Humeral Head, Perc Endo Approach */
"0P8D0ZZ"  =  "1"  /* Division of Left Humeral Head, Open Approach */
"0P8D3ZZ"  =  "1"  /* Division of Left Humeral Head, Percutaneous Approach */
"0P8D4ZZ"  =  "1"  /* Division of Left Humeral Head, Perc Endo Approach */
"0P8F0ZZ"  =  "1"  /* Division of Right Humeral Shaft, Open Approach */
"0P8F3ZZ"  =  "1"  /* Division of Right Humeral Shaft, Percutaneous Approach */
"0P8F4ZZ"  =  "1"  /* Division of Right Humeral Shaft, Perc Endo Approach */
"0P8G0ZZ"  =  "1"  /* Division of Left Humeral Shaft, Open Approach */
"0P8G3ZZ"  =  "1"  /* Division of Left Humeral Shaft, Percutaneous Approach */
"0P8G4ZZ"  =  "1"  /* Division of Left Humeral Shaft, Perc Endo Approach */
"0P8H0ZZ"  =  "1"  /* Division of Right Radius, Open Approach */
"0P8H3ZZ"  =  "1"  /* Division of Right Radius, Percutaneous Approach */
"0P8H4ZZ"  =  "1"  /* Division of Right Radius, Percutaneous Endoscopic Approach */
"0P8J0ZZ"  =  "1"  /* Division of Left Radius, Open Approach */
"0P8J3ZZ"  =  "1"  /* Division of Left Radius, Percutaneous Approach */
"0P8J4ZZ"  =  "1"  /* Division of Left Radius, Percutaneous Endoscopic Approach */
"0P8K0ZZ"  =  "1"  /* Division of Right Ulna, Open Approach */
"0P8K3ZZ"  =  "1"  /* Division of Right Ulna, Percutaneous Approach */
"0P8K4ZZ"  =  "1"  /* Division of Right Ulna, Percutaneous Endoscopic Approach */
"0P8L0ZZ"  =  "1"  /* Division of Left Ulna, Open Approach */
"0P8L3ZZ"  =  "1"  /* Division of Left Ulna, Percutaneous Approach */
"0P8L4ZZ"  =  "1"  /* Division of Left Ulna, Percutaneous Endoscopic Approach */
"0P8M0ZZ"  =  "1"  /* Division of Right Carpal, Open Approach */
"0P8M3ZZ"  =  "1"  /* Division of Right Carpal, Percutaneous Approach */
"0P8M4ZZ"  =  "1"  /* Division of Right Carpal, Percutaneous Endoscopic Approach */
"0P8N0ZZ"  =  "1"  /* Division of Left Carpal, Open Approach */
"0P8N3ZZ"  =  "1"  /* Division of Left Carpal, Percutaneous Approach */
"0P8N4ZZ"  =  "1"  /* Division of Left Carpal, Percutaneous Endoscopic Approach */
"0P8P0ZZ"  =  "1"  /* Division of Right Metacarpal, Open Approach */
"0P8P3ZZ"  =  "1"  /* Division of Right Metacarpal, Percutaneous Approach */
"0P8P4ZZ"  =  "1"  /* Division of Right Metacarpal, Perc Endo Approach */
"0P8Q0ZZ"  =  "1"  /* Division of Left Metacarpal, Open Approach */
"0P8Q3ZZ"  =  "1"  /* Division of Left Metacarpal, Percutaneous Approach */
"0P8Q4ZZ"  =  "1"  /* Division of Left Metacarpal, Perc Endo Approach */
"0P8R0ZZ"  =  "1"  /* Division of Right Thumb Phalanx, Open Approach */
"0P8R3ZZ"  =  "1"  /* Division of Right Thumb Phalanx, Percutaneous Approach */
"0P8R4ZZ"  =  "1"  /* Division of Right Thumb Phalanx, Perc Endo Approach */
"0P8S0ZZ"  =  "1"  /* Division of Left Thumb Phalanx, Open Approach */
"0P8S3ZZ"  =  "1"  /* Division of Left Thumb Phalanx, Percutaneous Approach */
"0P8S4ZZ"  =  "1"  /* Division of Left Thumb Phalanx, Perc Endo Approach */
"0P8T0ZZ"  =  "1"  /* Division of Right Finger Phalanx, Open Approach */
"0P8T3ZZ"  =  "1"  /* Division of Right Finger Phalanx, Percutaneous Approach */
"0P8T4ZZ"  =  "1"  /* Division of Right Finger Phalanx, Perc Endo Approach */
"0P8V0ZZ"  =  "1"  /* Division of Left Finger Phalanx, Open Approach */
"0P8V3ZZ"  =  "1"  /* Division of Left Finger Phalanx, Percutaneous Approach */
"0P8V4ZZ"  =  "1"  /* Division of Left Finger Phalanx, Perc Endo Approach */
"0P9000Z"  =  "1"  /* Drainage of Sternum with Drainage Device, Open Approach */
"0P900ZX"  =  "1"  /* Drainage of Sternum, Open Approach, Diagnostic */
"0P900ZZ"  =  "1"  /* Drainage of Sternum, Open Approach */
"0P9030Z"  =  "1"  /* Drainage of Sternum with Drainage Device, Perc Approach */
"0P903ZX"  =  "1"  /* Drainage of Sternum, Percutaneous Approach, Diagnostic */
"0P903ZZ"  =  "1"  /* Drainage of Sternum, Percutaneous Approach */
"0P9040Z"  =  "1"  /* Drainage of Sternum with Drainage Device, Perc Endo Approach */
"0P904ZX"  =  "1"  /* Drainage of Sternum, Percutaneous Endoscopic Approach, Diagn */
"0P904ZZ"  =  "1"  /* Drainage of Sternum, Percutaneous Endoscopic Approach */
"0P9100Z"  =  "1"  /* Drainage of Right Rib with Drainage Device, Open Approach */
"0P910ZX"  =  "1"  /* Drainage of Right Rib, Open Approach, Diagnostic */
"0P910ZZ"  =  "1"  /* Drainage of Right Rib, Open Approach */
"0P9130Z"  =  "1"  /* Drainage of Right Rib with Drainage Device, Perc Approach */
"0P913ZX"  =  "1"  /* Drainage of Right Rib, Percutaneous Approach, Diagnostic */
"0P913ZZ"  =  "1"  /* Drainage of Right Rib, Percutaneous Approach */
"0P9140Z"  =  "1"  /* Drainage of Right Rib with Drain Dev, Perc Endo Approach */
"0P914ZX"  =  "1"  /* Drainage of Right Rib, Perc Endo Approach, Diagn */
"0P914ZZ"  =  "1"  /* Drainage of Right Rib, Percutaneous Endoscopic Approach */
"0P9200Z"  =  "1"  /* Drainage of Left Rib with Drainage Device, Open Approach */
"0P920ZX"  =  "1"  /* Drainage of Left Rib, Open Approach, Diagnostic */
"0P920ZZ"  =  "1"  /* Drainage of Left Rib, Open Approach */
"0P9230Z"  =  "1"  /* Drainage of Left Rib with Drainage Device, Perc Approach */
"0P923ZX"  =  "1"  /* Drainage of Left Rib, Percutaneous Approach, Diagnostic */
"0P923ZZ"  =  "1"  /* Drainage of Left Rib, Percutaneous Approach */
"0P9240Z"  =  "1"  /* Drainage of Left Rib with Drain Dev, Perc Endo Approach */
"0P924ZX"  =  "1"  /* Drainage of Left Rib, Perc Endo Approach, Diagn */
"0P924ZZ"  =  "1"  /* Drainage of Left Rib, Percutaneous Endoscopic Approach */
"0P9300Z"  =  "1"  /* Drainage of Cervical Vertebra with Drain Dev, Open Approach */
"0P930ZX"  =  "1"  /* Drainage of Cervical Vertebra, Open Approach, Diagnostic */
"0P930ZZ"  =  "1"  /* Drainage of Cervical Vertebra, Open Approach */
"0P9330Z"  =  "1"  /* Drainage of Cervical Vertebra with Drain Dev, Perc Approach */
"0P933ZX"  =  "1"  /* Drainage of Cervical Vertebra, Percutaneous Approach, Diagn */
"0P933ZZ"  =  "1"  /* Drainage of Cervical Vertebra, Percutaneous Approach */
"0P9340Z"  =  "1"  /* Drain of Cervcal Vertebra with Drain Dev, Perc Endo Approach */
"0P934ZX"  =  "1"  /* Drainage of Cervical Vertebra, Perc Endo Approach, Diagn */
"0P934ZZ"  =  "1"  /* Drainage of Cervical Vertebra, Perc Endo Approach */
"0P9400Z"  =  "1"  /* Drainage of Thoracic Vertebra with Drain Dev, Open Approach */
"0P940ZX"  =  "1"  /* Drainage of Thoracic Vertebra, Open Approach, Diagnostic */
"0P940ZZ"  =  "1"  /* Drainage of Thoracic Vertebra, Open Approach */
"0P9430Z"  =  "1"  /* Drainage of Thoracic Vertebra with Drain Dev, Perc Approach */
"0P943ZX"  =  "1"  /* Drainage of Thoracic Vertebra, Percutaneous Approach, Diagn */
"0P943ZZ"  =  "1"  /* Drainage of Thoracic Vertebra, Percutaneous Approach */
"0P9440Z"  =  "1"  /* Drainage of Thor Vertebra with Drain Dev, Perc Endo Approach */
"0P944ZX"  =  "1"  /* Drainage of Thoracic Vertebra, Perc Endo Approach, Diagn */
"0P944ZZ"  =  "1"  /* Drainage of Thoracic Vertebra, Perc Endo Approach */
"0P9500Z"  =  "1"  /* Drainage of Right Scapula with Drain Dev, Open Approach */
"0P950ZX"  =  "1"  /* Drainage of Right Scapula, Open Approach, Diagnostic */
"0P950ZZ"  =  "1"  /* Drainage of Right Scapula, Open Approach */
"0P9530Z"  =  "1"  /* Drainage of Right Scapula with Drain Dev, Perc Approach */
"0P953ZX"  =  "1"  /* Drainage of Right Scapula, Percutaneous Approach, Diagnostic */
"0P953ZZ"  =  "1"  /* Drainage of Right Scapula, Percutaneous Approach */
"0P9540Z"  =  "1"  /* Drainage of Right Scapula with Drain Dev, Perc Endo Approach */
"0P954ZX"  =  "1"  /* Drainage of Right Scapula, Perc Endo Approach, Diagn */
"0P954ZZ"  =  "1"  /* Drainage of Right Scapula, Percutaneous Endoscopic Approach */
"0P9600Z"  =  "1"  /* Drainage of Left Scapula with Drainage Device, Open Approach */
"0P960ZX"  =  "1"  /* Drainage of Left Scapula, Open Approach, Diagnostic */
"0P960ZZ"  =  "1"  /* Drainage of Left Scapula, Open Approach */
"0P9630Z"  =  "1"  /* Drainage of Left Scapula with Drainage Device, Perc Approach */
"0P963ZX"  =  "1"  /* Drainage of Left Scapula, Percutaneous Approach, Diagnostic */
"0P963ZZ"  =  "1"  /* Drainage of Left Scapula, Percutaneous Approach */
"0P9640Z"  =  "1"  /* Drainage of Left Scapula with Drain Dev, Perc Endo Approach */
"0P964ZX"  =  "1"  /* Drainage of Left Scapula, Perc Endo Approach, Diagn */
"0P964ZZ"  =  "1"  /* Drainage of Left Scapula, Percutaneous Endoscopic Approach */
"0P9700Z"  =  "1"  /* Drainage of R Glenoid Cav with Drain Dev, Open Approach */
"0P970ZX"  =  "1"  /* Drainage of Right Glenoid Cavity, Open Approach, Diagnostic */
"0P970ZZ"  =  "1"  /* Drainage of Right Glenoid Cavity, Open Approach */
"0P9730Z"  =  "1"  /* Drainage of R Glenoid Cav with Drain Dev, Perc Approach */
"0P973ZX"  =  "1"  /* Drainage of Right Glenoid Cavity, Perc Approach, Diagn */
"0P973ZZ"  =  "1"  /* Drainage of Right Glenoid Cavity, Percutaneous Approach */
"0P9740Z"  =  "1"  /* Drainage of R Glenoid Cav with Drain Dev, Perc Endo Approach */
"0P974ZX"  =  "1"  /* Drainage of Right Glenoid Cavity, Perc Endo Approach, Diagn */
"0P974ZZ"  =  "1"  /* Drainage of Right Glenoid Cavity, Perc Endo Approach */
"0P9800Z"  =  "1"  /* Drainage of L Glenoid Cav with Drain Dev, Open Approach */
"0P980ZX"  =  "1"  /* Drainage of Left Glenoid Cavity, Open Approach, Diagnostic */
"0P980ZZ"  =  "1"  /* Drainage of Left Glenoid Cavity, Open Approach */
"0P9830Z"  =  "1"  /* Drainage of L Glenoid Cav with Drain Dev, Perc Approach */
"0P983ZX"  =  "1"  /* Drainage of Left Glenoid Cavity, Perc Approach, Diagn */
"0P983ZZ"  =  "1"  /* Drainage of Left Glenoid Cavity, Percutaneous Approach */
"0P9840Z"  =  "1"  /* Drainage of L Glenoid Cav with Drain Dev, Perc Endo Approach */
"0P984ZX"  =  "1"  /* Drainage of Left Glenoid Cavity, Perc Endo Approach, Diagn */
"0P984ZZ"  =  "1"  /* Drainage of Left Glenoid Cavity, Perc Endo Approach */
"0P9900Z"  =  "1"  /* Drainage of Right Clavicle with Drain Dev, Open Approach */
"0P990ZX"  =  "1"  /* Drainage of Right Clavicle, Open Approach, Diagnostic */
"0P990ZZ"  =  "1"  /* Drainage of Right Clavicle, Open Approach */
"0P9930Z"  =  "1"  /* Drainage of Right Clavicle with Drain Dev, Perc Approach */
"0P993ZX"  =  "1"  /* Drainage of Right Clavicle, Percutaneous Approach, Diagn */
"0P993ZZ"  =  "1"  /* Drainage of Right Clavicle, Percutaneous Approach */
"0P9940Z"  =  "1"  /* Drainage of R Clavicle with Drain Dev, Perc Endo Approach */
"0P994ZX"  =  "1"  /* Drainage of Right Clavicle, Perc Endo Approach, Diagn */
"0P994ZZ"  =  "1"  /* Drainage of Right Clavicle, Percutaneous Endoscopic Approach */
"0P9B00Z"  =  "1"  /* Drainage of Left Clavicle with Drain Dev, Open Approach */
"0P9B0ZX"  =  "1"  /* Drainage of Left Clavicle, Open Approach, Diagnostic */
"0P9B0ZZ"  =  "1"  /* Drainage of Left Clavicle, Open Approach */
"0P9B30Z"  =  "1"  /* Drainage of Left Clavicle with Drain Dev, Perc Approach */
"0P9B3ZX"  =  "1"  /* Drainage of Left Clavicle, Percutaneous Approach, Diagnostic */
"0P9B3ZZ"  =  "1"  /* Drainage of Left Clavicle, Percutaneous Approach */
"0P9B40Z"  =  "1"  /* Drainage of Left Clavicle with Drain Dev, Perc Endo Approach */
"0P9B4ZX"  =  "1"  /* Drainage of Left Clavicle, Perc Endo Approach, Diagn */
"0P9B4ZZ"  =  "1"  /* Drainage of Left Clavicle, Percutaneous Endoscopic Approach */
"0P9C00Z"  =  "1"  /* Drainage of Right Humeral Head with Drain Dev, Open Approach */
"0P9C0ZX"  =  "1"  /* Drainage of Right Humeral Head, Open Approach, Diagnostic */
"0P9C0ZZ"  =  "1"  /* Drainage of Right Humeral Head, Open Approach */
"0P9C30Z"  =  "1"  /* Drainage of Right Humeral Head with Drain Dev, Perc Approach */
"0P9C3ZX"  =  "1"  /* Drainage of Right Humeral Head, Percutaneous Approach, Diagn */
"0P9C3ZZ"  =  "1"  /* Drainage of Right Humeral Head, Percutaneous Approach */
"0P9C40Z"  =  "1"  /* Drain of R Humeral Head with Drain Dev, Perc Endo Approach */
"0P9C4ZX"  =  "1"  /* Drainage of Right Humeral Head, Perc Endo Approach, Diagn */
"0P9C4ZZ"  =  "1"  /* Drainage of Right Humeral Head, Perc Endo Approach */
"0P9D00Z"  =  "1"  /* Drainage of Left Humeral Head with Drain Dev, Open Approach */
"0P9D0ZX"  =  "1"  /* Drainage of Left Humeral Head, Open Approach, Diagnostic */
"0P9D0ZZ"  =  "1"  /* Drainage of Left Humeral Head, Open Approach */
"0P9D30Z"  =  "1"  /* Drainage of Left Humeral Head with Drain Dev, Perc Approach */
"0P9D3ZX"  =  "1"  /* Drainage of Left Humeral Head, Percutaneous Approach, Diagn */
"0P9D3ZZ"  =  "1"  /* Drainage of Left Humeral Head, Percutaneous Approach */
"0P9D40Z"  =  "1"  /* Drain of L Humeral Head with Drain Dev, Perc Endo Approach */
"0P9D4ZX"  =  "1"  /* Drainage of Left Humeral Head, Perc Endo Approach, Diagn */
"0P9D4ZZ"  =  "1"  /* Drainage of Left Humeral Head, Perc Endo Approach */
"0P9F00Z"  =  "1"  /* Drainage of R Humeral Shaft with Drain Dev, Open Approach */
"0P9F0ZX"  =  "1"  /* Drainage of Right Humeral Shaft, Open Approach, Diagnostic */
"0P9F0ZZ"  =  "1"  /* Drainage of Right Humeral Shaft, Open Approach */
"0P9F30Z"  =  "1"  /* Drainage of R Humeral Shaft with Drain Dev, Perc Approach */
"0P9F3ZX"  =  "1"  /* Drainage of Right Humeral Shaft, Perc Approach, Diagn */
"0P9F3ZZ"  =  "1"  /* Drainage of Right Humeral Shaft, Percutaneous Approach */
"0P9F40Z"  =  "1"  /* Drain of R Humeral Shaft with Drain Dev, Perc Endo Approach */
"0P9F4ZX"  =  "1"  /* Drainage of Right Humeral Shaft, Perc Endo Approach, Diagn */
"0P9F4ZZ"  =  "1"  /* Drainage of Right Humeral Shaft, Perc Endo Approach */
"0P9G00Z"  =  "1"  /* Drainage of Left Humeral Shaft with Drain Dev, Open Approach */
"0P9G0ZX"  =  "1"  /* Drainage of Left Humeral Shaft, Open Approach, Diagnostic */
"0P9G0ZZ"  =  "1"  /* Drainage of Left Humeral Shaft, Open Approach */
"0P9G30Z"  =  "1"  /* Drainage of Left Humeral Shaft with Drain Dev, Perc Approach */
"0P9G3ZX"  =  "1"  /* Drainage of Left Humeral Shaft, Percutaneous Approach, Diagn */
"0P9G3ZZ"  =  "1"  /* Drainage of Left Humeral Shaft, Percutaneous Approach */
"0P9G40Z"  =  "1"  /* Drain of L Humeral Shaft with Drain Dev, Perc Endo Approach */
"0P9G4ZX"  =  "1"  /* Drainage of Left Humeral Shaft, Perc Endo Approach, Diagn */
"0P9G4ZZ"  =  "1"  /* Drainage of Left Humeral Shaft, Perc Endo Approach */
"0P9H00Z"  =  "1"  /* Drainage of Right Radius with Drainage Device, Open Approach */
"0P9H0ZX"  =  "1"  /* Drainage of Right Radius, Open Approach, Diagnostic */
"0P9H0ZZ"  =  "1"  /* Drainage of Right Radius, Open Approach */
"0P9H30Z"  =  "1"  /* Drainage of Right Radius with Drainage Device, Perc Approach */
"0P9H3ZX"  =  "1"  /* Drainage of Right Radius, Percutaneous Approach, Diagnostic */
"0P9H3ZZ"  =  "1"  /* Drainage of Right Radius, Percutaneous Approach */
"0P9H40Z"  =  "1"  /* Drainage of Right Radius with Drain Dev, Perc Endo Approach */
"0P9H4ZX"  =  "1"  /* Drainage of Right Radius, Perc Endo Approach, Diagn */
"0P9H4ZZ"  =  "1"  /* Drainage of Right Radius, Percutaneous Endoscopic Approach */
"0P9J00Z"  =  "1"  /* Drainage of Left Radius with Drainage Device, Open Approach */
"0P9J0ZX"  =  "1"  /* Drainage of Left Radius, Open Approach, Diagnostic */
"0P9J0ZZ"  =  "1"  /* Drainage of Left Radius, Open Approach */
"0P9J30Z"  =  "1"  /* Drainage of Left Radius with Drainage Device, Perc Approach */
"0P9J3ZX"  =  "1"  /* Drainage of Left Radius, Percutaneous Approach, Diagnostic */
"0P9J3ZZ"  =  "1"  /* Drainage of Left Radius, Percutaneous Approach */
"0P9J40Z"  =  "1"  /* Drainage of Left Radius with Drain Dev, Perc Endo Approach */
"0P9J4ZX"  =  "1"  /* Drainage of Left Radius, Perc Endo Approach, Diagn */
"0P9J4ZZ"  =  "1"  /* Drainage of Left Radius, Percutaneous Endoscopic Approach */
"0P9K00Z"  =  "1"  /* Drainage of Right Ulna with Drainage Device, Open Approach */
"0P9K0ZX"  =  "1"  /* Drainage of Right Ulna, Open Approach, Diagnostic */
"0P9K0ZZ"  =  "1"  /* Drainage of Right Ulna, Open Approach */
"0P9K30Z"  =  "1"  /* Drainage of Right Ulna with Drainage Device, Perc Approach */
"0P9K3ZX"  =  "1"  /* Drainage of Right Ulna, Percutaneous Approach, Diagnostic */
"0P9K3ZZ"  =  "1"  /* Drainage of Right Ulna, Percutaneous Approach */
"0P9K40Z"  =  "1"  /* Drainage of Right Ulna with Drain Dev, Perc Endo Approach */
"0P9K4ZX"  =  "1"  /* Drainage of Right Ulna, Perc Endo Approach, Diagn */
"0P9K4ZZ"  =  "1"  /* Drainage of Right Ulna, Percutaneous Endoscopic Approach */
"0P9L00Z"  =  "1"  /* Drainage of Left Ulna with Drainage Device, Open Approach */
"0P9L0ZX"  =  "1"  /* Drainage of Left Ulna, Open Approach, Diagnostic */
"0P9L0ZZ"  =  "1"  /* Drainage of Left Ulna, Open Approach */
"0P9L30Z"  =  "1"  /* Drainage of Left Ulna with Drainage Device, Perc Approach */
"0P9L3ZX"  =  "1"  /* Drainage of Left Ulna, Percutaneous Approach, Diagnostic */
"0P9L3ZZ"  =  "1"  /* Drainage of Left Ulna, Percutaneous Approach */
"0P9L40Z"  =  "1"  /* Drainage of Left Ulna with Drain Dev, Perc Endo Approach */
"0P9L4ZX"  =  "1"  /* Drainage of Left Ulna, Perc Endo Approach, Diagn */
"0P9L4ZZ"  =  "1"  /* Drainage of Left Ulna, Percutaneous Endoscopic Approach */
"0P9M00Z"  =  "1"  /* Drainage of Right Carpal with Drainage Device, Open Approach */
"0P9M0ZX"  =  "1"  /* Drainage of Right Carpal, Open Approach, Diagnostic */
"0P9M0ZZ"  =  "1"  /* Drainage of Right Carpal, Open Approach */
"0P9M30Z"  =  "1"  /* Drainage of Right Carpal with Drainage Device, Perc Approach */
"0P9M3ZX"  =  "1"  /* Drainage of Right Carpal, Percutaneous Approach, Diagnostic */
"0P9M3ZZ"  =  "1"  /* Drainage of Right Carpal, Percutaneous Approach */
"0P9M40Z"  =  "1"  /* Drainage of Right Carpal with Drain Dev, Perc Endo Approach */
"0P9M4ZX"  =  "1"  /* Drainage of Right Carpal, Perc Endo Approach, Diagn */
"0P9M4ZZ"  =  "1"  /* Drainage of Right Carpal, Percutaneous Endoscopic Approach */
"0P9N00Z"  =  "1"  /* Drainage of Left Carpal with Drainage Device, Open Approach */
"0P9N0ZX"  =  "1"  /* Drainage of Left Carpal, Open Approach, Diagnostic */
"0P9N0ZZ"  =  "1"  /* Drainage of Left Carpal, Open Approach */
"0P9N30Z"  =  "1"  /* Drainage of Left Carpal with Drainage Device, Perc Approach */
"0P9N3ZX"  =  "1"  /* Drainage of Left Carpal, Percutaneous Approach, Diagnostic */
"0P9N3ZZ"  =  "1"  /* Drainage of Left Carpal, Percutaneous Approach */
"0P9N40Z"  =  "1"  /* Drainage of Left Carpal with Drain Dev, Perc Endo Approach */
"0P9N4ZX"  =  "1"  /* Drainage of Left Carpal, Perc Endo Approach, Diagn */
"0P9N4ZZ"  =  "1"  /* Drainage of Left Carpal, Percutaneous Endoscopic Approach */
"0P9P00Z"  =  "1"  /* Drainage of Right Metacarpal with Drain Dev, Open Approach */
"0P9P0ZX"  =  "1"  /* Drainage of Right Metacarpal, Open Approach, Diagnostic */
"0P9P0ZZ"  =  "1"  /* Drainage of Right Metacarpal, Open Approach */
"0P9P30Z"  =  "1"  /* Drainage of Right Metacarpal with Drain Dev, Perc Approach */
"0P9P3ZX"  =  "1"  /* Drainage of Right Metacarpal, Percutaneous Approach, Diagn */
"0P9P3ZZ"  =  "1"  /* Drainage of Right Metacarpal, Percutaneous Approach */
"0P9P40Z"  =  "1"  /* Drainage of R Metacarpal with Drain Dev, Perc Endo Approach */
"0P9P4ZX"  =  "1"  /* Drainage of Right Metacarpal, Perc Endo Approach, Diagn */
"0P9P4ZZ"  =  "1"  /* Drainage of Right Metacarpal, Perc Endo Approach */
"0P9Q00Z"  =  "1"  /* Drainage of Left Metacarpal with Drain Dev, Open Approach */
"0P9Q0ZX"  =  "1"  /* Drainage of Left Metacarpal, Open Approach, Diagnostic */
"0P9Q0ZZ"  =  "1"  /* Drainage of Left Metacarpal, Open Approach */
"0P9Q30Z"  =  "1"  /* Drainage of Left Metacarpal with Drain Dev, Perc Approach */
"0P9Q3ZX"  =  "1"  /* Drainage of Left Metacarpal, Percutaneous Approach, Diagn */
"0P9Q3ZZ"  =  "1"  /* Drainage of Left Metacarpal, Percutaneous Approach */
"0P9Q40Z"  =  "1"  /* Drainage of L Metacarpal with Drain Dev, Perc Endo Approach */
"0P9Q4ZX"  =  "1"  /* Drainage of Left Metacarpal, Perc Endo Approach, Diagn */
"0P9Q4ZZ"  =  "1"  /* Drainage of Left Metacarpal, Perc Endo Approach */
"0P9R00Z"  =  "1"  /* Drainage of R Thumb Phalanx with Drain Dev, Open Approach */
"0P9R0ZX"  =  "1"  /* Drainage of Right Thumb Phalanx, Open Approach, Diagnostic */
"0P9R0ZZ"  =  "1"  /* Drainage of Right Thumb Phalanx, Open Approach */
"0P9R30Z"  =  "1"  /* Drainage of R Thumb Phalanx with Drain Dev, Perc Approach */
"0P9R3ZX"  =  "1"  /* Drainage of Right Thumb Phalanx, Perc Approach, Diagn */
"0P9R3ZZ"  =  "1"  /* Drainage of Right Thumb Phalanx, Percutaneous Approach */
"0P9R40Z"  =  "1"  /* Drain of R Thumb Phalanx with Drain Dev, Perc Endo Approach */
"0P9R4ZX"  =  "1"  /* Drainage of Right Thumb Phalanx, Perc Endo Approach, Diagn */
"0P9R4ZZ"  =  "1"  /* Drainage of Right Thumb Phalanx, Perc Endo Approach */
"0P9S00Z"  =  "1"  /* Drainage of Left Thumb Phalanx with Drain Dev, Open Approach */
"0P9S0ZX"  =  "1"  /* Drainage of Left Thumb Phalanx, Open Approach, Diagnostic */
"0P9S0ZZ"  =  "1"  /* Drainage of Left Thumb Phalanx, Open Approach */
"0P9S30Z"  =  "1"  /* Drainage of Left Thumb Phalanx with Drain Dev, Perc Approach */
"0P9S3ZX"  =  "1"  /* Drainage of Left Thumb Phalanx, Percutaneous Approach, Diagn */
"0P9S3ZZ"  =  "1"  /* Drainage of Left Thumb Phalanx, Percutaneous Approach */
"0P9S40Z"  =  "1"  /* Drain of L Thumb Phalanx with Drain Dev, Perc Endo Approach */
"0P9S4ZX"  =  "1"  /* Drainage of Left Thumb Phalanx, Perc Endo Approach, Diagn */
"0P9S4ZZ"  =  "1"  /* Drainage of Left Thumb Phalanx, Perc Endo Approach */
"0P9T00Z"  =  "1"  /* Drainage of R Finger Phalanx with Drain Dev, Open Approach */
"0P9T0ZX"  =  "1"  /* Drainage of Right Finger Phalanx, Open Approach, Diagnostic */
"0P9T0ZZ"  =  "1"  /* Drainage of Right Finger Phalanx, Open Approach */
"0P9T30Z"  =  "1"  /* Drainage of R Finger Phalanx with Drain Dev, Perc Approach */
"0P9T3ZX"  =  "1"  /* Drainage of Right Finger Phalanx, Perc Approach, Diagn */
"0P9T3ZZ"  =  "1"  /* Drainage of Right Finger Phalanx, Percutaneous Approach */
"0P9T40Z"  =  "1"  /* Drain of R Finger Phalanx with Drain Dev, Perc Endo Approach */
"0P9T4ZX"  =  "1"  /* Drainage of Right Finger Phalanx, Perc Endo Approach, Diagn */
"0P9T4ZZ"  =  "1"  /* Drainage of Right Finger Phalanx, Perc Endo Approach */
"0P9V00Z"  =  "1"  /* Drainage of L Finger Phalanx with Drain Dev, Open Approach */
"0P9V0ZX"  =  "1"  /* Drainage of Left Finger Phalanx, Open Approach, Diagnostic */
"0P9V0ZZ"  =  "1"  /* Drainage of Left Finger Phalanx, Open Approach */
"0P9V30Z"  =  "1"  /* Drainage of L Finger Phalanx with Drain Dev, Perc Approach */
"0P9V3ZX"  =  "1"  /* Drainage of Left Finger Phalanx, Perc Approach, Diagn */
"0P9V3ZZ"  =  "1"  /* Drainage of Left Finger Phalanx, Percutaneous Approach */
"0P9V40Z"  =  "1"  /* Drain of L Finger Phalanx with Drain Dev, Perc Endo Approach */
"0P9V4ZX"  =  "1"  /* Drainage of Left Finger Phalanx, Perc Endo Approach, Diagn */
"0P9V4ZZ"  =  "1"  /* Drainage of Left Finger Phalanx, Perc Endo Approach */
"0PB00ZX"  =  "1"  /* Excision of Sternum, Open Approach, Diagnostic */
"0PB00ZZ"  =  "1"  /* Excision of Sternum, Open Approach */
"0PB03ZX"  =  "1"  /* Excision of Sternum, Percutaneous Approach, Diagnostic */
"0PB03ZZ"  =  "1"  /* Excision of Sternum, Percutaneous Approach */
"0PB04ZX"  =  "1"  /* Excision of Sternum, Percutaneous Endoscopic Approach, Diagn */
"0PB04ZZ"  =  "1"  /* Excision of Sternum, Percutaneous Endoscopic Approach */
"0PB10ZX"  =  "1"  /* Excision of Right Rib, Open Approach, Diagnostic */
"0PB10ZZ"  =  "1"  /* Excision of Right Rib, Open Approach */
"0PB13ZX"  =  "1"  /* Excision of Right Rib, Percutaneous Approach, Diagnostic */
"0PB13ZZ"  =  "1"  /* Excision of Right Rib, Percutaneous Approach */
"0PB14ZX"  =  "1"  /* Excision of Right Rib, Perc Endo Approach, Diagn */
"0PB14ZZ"  =  "1"  /* Excision of Right Rib, Percutaneous Endoscopic Approach */
"0PB20ZX"  =  "1"  /* Excision of Left Rib, Open Approach, Diagnostic */
"0PB20ZZ"  =  "1"  /* Excision of Left Rib, Open Approach */
"0PB23ZX"  =  "1"  /* Excision of Left Rib, Percutaneous Approach, Diagnostic */
"0PB23ZZ"  =  "1"  /* Excision of Left Rib, Percutaneous Approach */
"0PB24ZX"  =  "1"  /* Excision of Left Rib, Perc Endo Approach, Diagn */
"0PB24ZZ"  =  "1"  /* Excision of Left Rib, Percutaneous Endoscopic Approach */
"0PB30ZX"  =  "1"  /* Excision of Cervical Vertebra, Open Approach, Diagnostic */
"0PB30ZZ"  =  "1"  /* Excision of Cervical Vertebra, Open Approach */
"0PB33ZX"  =  "1"  /* Excision of Cervical Vertebra, Percutaneous Approach, Diagn */
"0PB33ZZ"  =  "1"  /* Excision of Cervical Vertebra, Percutaneous Approach */
"0PB34ZX"  =  "1"  /* Excision of Cervical Vertebra, Perc Endo Approach, Diagn */
"0PB34ZZ"  =  "1"  /* Excision of Cervical Vertebra, Perc Endo Approach */
"0PB40ZX"  =  "1"  /* Excision of Thoracic Vertebra, Open Approach, Diagnostic */
"0PB40ZZ"  =  "1"  /* Excision of Thoracic Vertebra, Open Approach */
"0PB43ZX"  =  "1"  /* Excision of Thoracic Vertebra, Percutaneous Approach, Diagn */
"0PB43ZZ"  =  "1"  /* Excision of Thoracic Vertebra, Percutaneous Approach */
"0PB44ZX"  =  "1"  /* Excision of Thoracic Vertebra, Perc Endo Approach, Diagn */
"0PB44ZZ"  =  "1"  /* Excision of Thoracic Vertebra, Perc Endo Approach */
"0PB50ZX"  =  "1"  /* Excision of Right Scapula, Open Approach, Diagnostic */
"0PB50ZZ"  =  "1"  /* Excision of Right Scapula, Open Approach */
"0PB53ZX"  =  "1"  /* Excision of Right Scapula, Percutaneous Approach, Diagnostic */
"0PB53ZZ"  =  "1"  /* Excision of Right Scapula, Percutaneous Approach */
"0PB54ZX"  =  "1"  /* Excision of Right Scapula, Perc Endo Approach, Diagn */
"0PB54ZZ"  =  "1"  /* Excision of Right Scapula, Percutaneous Endoscopic Approach */
"0PB60ZX"  =  "1"  /* Excision of Left Scapula, Open Approach, Diagnostic */
"0PB60ZZ"  =  "1"  /* Excision of Left Scapula, Open Approach */
"0PB63ZX"  =  "1"  /* Excision of Left Scapula, Percutaneous Approach, Diagnostic */
"0PB63ZZ"  =  "1"  /* Excision of Left Scapula, Percutaneous Approach */
"0PB64ZX"  =  "1"  /* Excision of Left Scapula, Perc Endo Approach, Diagn */
"0PB64ZZ"  =  "1"  /* Excision of Left Scapula, Percutaneous Endoscopic Approach */
"0PB70ZX"  =  "1"  /* Excision of Right Glenoid Cavity, Open Approach, Diagnostic */
"0PB70ZZ"  =  "1"  /* Excision of Right Glenoid Cavity, Open Approach */
"0PB73ZX"  =  "1"  /* Excision of Right Glenoid Cavity, Perc Approach, Diagn */
"0PB73ZZ"  =  "1"  /* Excision of Right Glenoid Cavity, Percutaneous Approach */
"0PB74ZX"  =  "1"  /* Excision of Right Glenoid Cavity, Perc Endo Approach, Diagn */
"0PB74ZZ"  =  "1"  /* Excision of Right Glenoid Cavity, Perc Endo Approach */
"0PB80ZX"  =  "1"  /* Excision of Left Glenoid Cavity, Open Approach, Diagnostic */
"0PB80ZZ"  =  "1"  /* Excision of Left Glenoid Cavity, Open Approach */
"0PB83ZX"  =  "1"  /* Excision of Left Glenoid Cavity, Perc Approach, Diagn */
"0PB83ZZ"  =  "1"  /* Excision of Left Glenoid Cavity, Percutaneous Approach */
"0PB84ZX"  =  "1"  /* Excision of Left Glenoid Cavity, Perc Endo Approach, Diagn */
"0PB84ZZ"  =  "1"  /* Excision of Left Glenoid Cavity, Perc Endo Approach */
"0PB90ZX"  =  "1"  /* Excision of Right Clavicle, Open Approach, Diagnostic */
"0PB90ZZ"  =  "1"  /* Excision of Right Clavicle, Open Approach */
"0PB93ZX"  =  "1"  /* Excision of Right Clavicle, Percutaneous Approach, Diagn */
"0PB93ZZ"  =  "1"  /* Excision of Right Clavicle, Percutaneous Approach */
"0PB94ZX"  =  "1"  /* Excision of Right Clavicle, Perc Endo Approach, Diagn */
"0PB94ZZ"  =  "1"  /* Excision of Right Clavicle, Percutaneous Endoscopic Approach */
"0PBB0ZX"  =  "1"  /* Excision of Left Clavicle, Open Approach, Diagnostic */
"0PBB0ZZ"  =  "1"  /* Excision of Left Clavicle, Open Approach */
"0PBB3ZX"  =  "1"  /* Excision of Left Clavicle, Percutaneous Approach, Diagnostic */
"0PBB3ZZ"  =  "1"  /* Excision of Left Clavicle, Percutaneous Approach */
"0PBB4ZX"  =  "1"  /* Excision of Left Clavicle, Perc Endo Approach, Diagn */
"0PBB4ZZ"  =  "1"  /* Excision of Left Clavicle, Percutaneous Endoscopic Approach */
"0PBC0ZX"  =  "1"  /* Excision of Right Humeral Head, Open Approach, Diagnostic */
"0PBC0ZZ"  =  "1"  /* Excision of Right Humeral Head, Open Approach */
"0PBC3ZX"  =  "1"  /* Excision of Right Humeral Head, Percutaneous Approach, Diagn */
"0PBC3ZZ"  =  "1"  /* Excision of Right Humeral Head, Percutaneous Approach */
"0PBC4ZX"  =  "1"  /* Excision of Right Humeral Head, Perc Endo Approach, Diagn */
"0PBC4ZZ"  =  "1"  /* Excision of Right Humeral Head, Perc Endo Approach */
"0PBD0ZX"  =  "1"  /* Excision of Left Humeral Head, Open Approach, Diagnostic */
"0PBD0ZZ"  =  "1"  /* Excision of Left Humeral Head, Open Approach */
"0PBD3ZX"  =  "1"  /* Excision of Left Humeral Head, Percutaneous Approach, Diagn */
"0PBD3ZZ"  =  "1"  /* Excision of Left Humeral Head, Percutaneous Approach */
"0PBD4ZX"  =  "1"  /* Excision of Left Humeral Head, Perc Endo Approach, Diagn */
"0PBD4ZZ"  =  "1"  /* Excision of Left Humeral Head, Perc Endo Approach */
"0PBF0ZX"  =  "1"  /* Excision of Right Humeral Shaft, Open Approach, Diagnostic */
"0PBF0ZZ"  =  "1"  /* Excision of Right Humeral Shaft, Open Approach */
"0PBF3ZX"  =  "1"  /* Excision of Right Humeral Shaft, Perc Approach, Diagn */
"0PBF3ZZ"  =  "1"  /* Excision of Right Humeral Shaft, Percutaneous Approach */
"0PBF4ZX"  =  "1"  /* Excision of Right Humeral Shaft, Perc Endo Approach, Diagn */
"0PBF4ZZ"  =  "1"  /* Excision of Right Humeral Shaft, Perc Endo Approach */
"0PBG0ZX"  =  "1"  /* Excision of Left Humeral Shaft, Open Approach, Diagnostic */
"0PBG0ZZ"  =  "1"  /* Excision of Left Humeral Shaft, Open Approach */
"0PBG3ZX"  =  "1"  /* Excision of Left Humeral Shaft, Percutaneous Approach, Diagn */
"0PBG3ZZ"  =  "1"  /* Excision of Left Humeral Shaft, Percutaneous Approach */
"0PBG4ZX"  =  "1"  /* Excision of Left Humeral Shaft, Perc Endo Approach, Diagn */
"0PBG4ZZ"  =  "1"  /* Excision of Left Humeral Shaft, Perc Endo Approach */
"0PBH0ZX"  =  "1"  /* Excision of Right Radius, Open Approach, Diagnostic */
"0PBH0ZZ"  =  "1"  /* Excision of Right Radius, Open Approach */
"0PBH3ZX"  =  "1"  /* Excision of Right Radius, Percutaneous Approach, Diagnostic */
"0PBH3ZZ"  =  "1"  /* Excision of Right Radius, Percutaneous Approach */
"0PBH4ZX"  =  "1"  /* Excision of Right Radius, Perc Endo Approach, Diagn */
"0PBH4ZZ"  =  "1"  /* Excision of Right Radius, Percutaneous Endoscopic Approach */
"0PBJ0ZX"  =  "1"  /* Excision of Left Radius, Open Approach, Diagnostic */
"0PBJ0ZZ"  =  "1"  /* Excision of Left Radius, Open Approach */
"0PBJ3ZX"  =  "1"  /* Excision of Left Radius, Percutaneous Approach, Diagnostic */
"0PBJ3ZZ"  =  "1"  /* Excision of Left Radius, Percutaneous Approach */
"0PBJ4ZX"  =  "1"  /* Excision of Left Radius, Perc Endo Approach, Diagn */
"0PBJ4ZZ"  =  "1"  /* Excision of Left Radius, Percutaneous Endoscopic Approach */
"0PBK0ZX"  =  "1"  /* Excision of Right Ulna, Open Approach, Diagnostic */
"0PBK0ZZ"  =  "1"  /* Excision of Right Ulna, Open Approach */
"0PBK3ZX"  =  "1"  /* Excision of Right Ulna, Percutaneous Approach, Diagnostic */
"0PBK3ZZ"  =  "1"  /* Excision of Right Ulna, Percutaneous Approach */
"0PBK4ZX"  =  "1"  /* Excision of Right Ulna, Perc Endo Approach, Diagn */
"0PBK4ZZ"  =  "1"  /* Excision of Right Ulna, Percutaneous Endoscopic Approach */
"0PBL0ZX"  =  "1"  /* Excision of Left Ulna, Open Approach, Diagnostic */
"0PBL0ZZ"  =  "1"  /* Excision of Left Ulna, Open Approach */
"0PBL3ZX"  =  "1"  /* Excision of Left Ulna, Percutaneous Approach, Diagnostic */
"0PBL3ZZ"  =  "1"  /* Excision of Left Ulna, Percutaneous Approach */
"0PBL4ZX"  =  "1"  /* Excision of Left Ulna, Perc Endo Approach, Diagn */
"0PBL4ZZ"  =  "1"  /* Excision of Left Ulna, Percutaneous Endoscopic Approach */
"0PBM0ZX"  =  "1"  /* Excision of Right Carpal, Open Approach, Diagnostic */
"0PBM0ZZ"  =  "1"  /* Excision of Right Carpal, Open Approach */
"0PBM3ZX"  =  "1"  /* Excision of Right Carpal, Percutaneous Approach, Diagnostic */
"0PBM3ZZ"  =  "1"  /* Excision of Right Carpal, Percutaneous Approach */
"0PBM4ZX"  =  "1"  /* Excision of Right Carpal, Perc Endo Approach, Diagn */
"0PBM4ZZ"  =  "1"  /* Excision of Right Carpal, Percutaneous Endoscopic Approach */
"0PBN0ZX"  =  "1"  /* Excision of Left Carpal, Open Approach, Diagnostic */
"0PBN0ZZ"  =  "1"  /* Excision of Left Carpal, Open Approach */
"0PBN3ZX"  =  "1"  /* Excision of Left Carpal, Percutaneous Approach, Diagnostic */
"0PBN3ZZ"  =  "1"  /* Excision of Left Carpal, Percutaneous Approach */
"0PBN4ZX"  =  "1"  /* Excision of Left Carpal, Perc Endo Approach, Diagn */
"0PBN4ZZ"  =  "1"  /* Excision of Left Carpal, Percutaneous Endoscopic Approach */
"0PBP0ZX"  =  "1"  /* Excision of Right Metacarpal, Open Approach, Diagnostic */
"0PBP0ZZ"  =  "1"  /* Excision of Right Metacarpal, Open Approach */
"0PBP3ZX"  =  "1"  /* Excision of Right Metacarpal, Percutaneous Approach, Diagn */
"0PBP3ZZ"  =  "1"  /* Excision of Right Metacarpal, Percutaneous Approach */
"0PBP4ZX"  =  "1"  /* Excision of Right Metacarpal, Perc Endo Approach, Diagn */
"0PBP4ZZ"  =  "1"  /* Excision of Right Metacarpal, Perc Endo Approach */
"0PBQ0ZX"  =  "1"  /* Excision of Left Metacarpal, Open Approach, Diagnostic */
"0PBQ0ZZ"  =  "1"  /* Excision of Left Metacarpal, Open Approach */
"0PBQ3ZX"  =  "1"  /* Excision of Left Metacarpal, Percutaneous Approach, Diagn */
"0PBQ3ZZ"  =  "1"  /* Excision of Left Metacarpal, Percutaneous Approach */
"0PBQ4ZX"  =  "1"  /* Excision of Left Metacarpal, Perc Endo Approach, Diagn */
"0PBQ4ZZ"  =  "1"  /* Excision of Left Metacarpal, Perc Endo Approach */
"0PBR0ZX"  =  "1"  /* Excision of Right Thumb Phalanx, Open Approach, Diagnostic */
"0PBR0ZZ"  =  "1"  /* Excision of Right Thumb Phalanx, Open Approach */
"0PBR3ZX"  =  "1"  /* Excision of Right Thumb Phalanx, Perc Approach, Diagn */
"0PBR3ZZ"  =  "1"  /* Excision of Right Thumb Phalanx, Percutaneous Approach */
"0PBR4ZX"  =  "1"  /* Excision of Right Thumb Phalanx, Perc Endo Approach, Diagn */
"0PBR4ZZ"  =  "1"  /* Excision of Right Thumb Phalanx, Perc Endo Approach */
"0PBS0ZX"  =  "1"  /* Excision of Left Thumb Phalanx, Open Approach, Diagnostic */
"0PBS0ZZ"  =  "1"  /* Excision of Left Thumb Phalanx, Open Approach */
"0PBS3ZX"  =  "1"  /* Excision of Left Thumb Phalanx, Percutaneous Approach, Diagn */
"0PBS3ZZ"  =  "1"  /* Excision of Left Thumb Phalanx, Percutaneous Approach */
"0PBS4ZX"  =  "1"  /* Excision of Left Thumb Phalanx, Perc Endo Approach, Diagn */
"0PBS4ZZ"  =  "1"  /* Excision of Left Thumb Phalanx, Perc Endo Approach */
"0PBT0ZX"  =  "1"  /* Excision of Right Finger Phalanx, Open Approach, Diagnostic */
"0PBT0ZZ"  =  "1"  /* Excision of Right Finger Phalanx, Open Approach */
"0PBT3ZX"  =  "1"  /* Excision of Right Finger Phalanx, Perc Approach, Diagn */
"0PBT3ZZ"  =  "1"  /* Excision of Right Finger Phalanx, Percutaneous Approach */
"0PBT4ZX"  =  "1"  /* Excision of Right Finger Phalanx, Perc Endo Approach, Diagn */
"0PBT4ZZ"  =  "1"  /* Excision of Right Finger Phalanx, Perc Endo Approach */
"0PBV0ZX"  =  "1"  /* Excision of Left Finger Phalanx, Open Approach, Diagnostic */
"0PBV0ZZ"  =  "1"  /* Excision of Left Finger Phalanx, Open Approach */
"0PBV3ZX"  =  "1"  /* Excision of Left Finger Phalanx, Perc Approach, Diagn */
"0PBV3ZZ"  =  "1"  /* Excision of Left Finger Phalanx, Percutaneous Approach */
"0PBV4ZX"  =  "1"  /* Excision of Left Finger Phalanx, Perc Endo Approach, Diagn */
"0PBV4ZZ"  =  "1"  /* Excision of Left Finger Phalanx, Perc Endo Approach */
"0PC00ZZ"  =  "1"  /* Extirpation of Matter from Sternum, Open Approach */
"0PC03ZZ"  =  "1"  /* Extirpation of Matter from Sternum, Percutaneous Approach */
"0PC04ZZ"  =  "1"  /* Extirpation of Matter from Sternum, Perc Endo Approach */
"0PC10ZZ"  =  "1"  /* Extirpation of Matter from Right Rib, Open Approach */
"0PC13ZZ"  =  "1"  /* Extirpation of Matter from Right Rib, Percutaneous Approach */
"0PC14ZZ"  =  "1"  /* Extirpation of Matter from Right Rib, Perc Endo Approach */
"0PC20ZZ"  =  "1"  /* Extirpation of Matter from Left Rib, Open Approach */
"0PC23ZZ"  =  "1"  /* Extirpation of Matter from Left Rib, Percutaneous Approach */
"0PC24ZZ"  =  "1"  /* Extirpation of Matter from Left Rib, Perc Endo Approach */
"0PC30ZZ"  =  "1"  /* Extirpation of Matter from Cervical Vertebra, Open Approach */
"0PC33ZZ"  =  "1"  /* Extirpation of Matter from Cervical Vertebra, Perc Approach */
"0PC34ZZ"  =  "1"  /* Extirpate matter from Cervcal Vertebra, Perc Endo */
"0PC40ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Vertebra, Open Approach */
"0PC43ZZ"  =  "1"  /* Extirpation of Matter from Thoracic Vertebra, Perc Approach */
"0PC44ZZ"  =  "1"  /* Extirpation of Matter from Thor Vertebra, Perc Endo Approach */
"0PC50ZZ"  =  "1"  /* Extirpation of Matter from Right Scapula, Open Approach */
"0PC53ZZ"  =  "1"  /* Extirpation of Matter from Right Scapula, Perc Approach */
"0PC54ZZ"  =  "1"  /* Extirpation of Matter from Right Scapula, Perc Endo Approach */
"0PC60ZZ"  =  "1"  /* Extirpation of Matter from Left Scapula, Open Approach */
"0PC63ZZ"  =  "1"  /* Extirpation of Matter from Left Scapula, Perc Approach */
"0PC64ZZ"  =  "1"  /* Extirpation of Matter from Left Scapula, Perc Endo Approach */
"0PC70ZZ"  =  "1"  /* Extirpation of Matter from R Glenoid Cav, Open Approach */
"0PC73ZZ"  =  "1"  /* Extirpation of Matter from R Glenoid Cav, Perc Approach */
"0PC74ZZ"  =  "1"  /* Extirpation of Matter from R Glenoid Cav, Perc Endo Approach */
"0PC80ZZ"  =  "1"  /* Extirpation of Matter from L Glenoid Cav, Open Approach */
"0PC83ZZ"  =  "1"  /* Extirpation of Matter from L Glenoid Cav, Perc Approach */
"0PC84ZZ"  =  "1"  /* Extirpation of Matter from L Glenoid Cav, Perc Endo Approach */
"0PC90ZZ"  =  "1"  /* Extirpation of Matter from Right Clavicle, Open Approach */
"0PC93ZZ"  =  "1"  /* Extirpation of Matter from Right Clavicle, Perc Approach */
"0PC94ZZ"  =  "1"  /* Extirpation of Matter from R Clavicle, Perc Endo Approach */
"0PCB0ZZ"  =  "1"  /* Extirpation of Matter from Left Clavicle, Open Approach */
"0PCB3ZZ"  =  "1"  /* Extirpation of Matter from Left Clavicle, Perc Approach */
"0PCB4ZZ"  =  "1"  /* Extirpation of Matter from Left Clavicle, Perc Endo Approach */
"0PCC0ZZ"  =  "1"  /* Extirpation of Matter from Right Humeral Head, Open Approach */
"0PCC3ZZ"  =  "1"  /* Extirpation of Matter from Right Humeral Head, Perc Approach */
"0PCC4ZZ"  =  "1"  /* Extirpate of Matter from R Humeral Head, Perc Endo Approach */
"0PCD0ZZ"  =  "1"  /* Extirpation of Matter from Left Humeral Head, Open Approach */
"0PCD3ZZ"  =  "1"  /* Extirpation of Matter from Left Humeral Head, Perc Approach */
"0PCD4ZZ"  =  "1"  /* Extirpate of Matter from L Humeral Head, Perc Endo Approach */
"0PCF0ZZ"  =  "1"  /* Extirpation of Matter from R Humeral Shaft, Open Approach */
"0PCF3ZZ"  =  "1"  /* Extirpation of Matter from R Humeral Shaft, Perc Approach */
"0PCF4ZZ"  =  "1"  /* Extirpate of Matter from R Humeral Shaft, Perc Endo Approach */
"0PCG0ZZ"  =  "1"  /* Extirpation of Matter from Left Humeral Shaft, Open Approach */
"0PCG3ZZ"  =  "1"  /* Extirpation of Matter from Left Humeral Shaft, Perc Approach */
"0PCG4ZZ"  =  "1"  /* Extirpate of Matter from L Humeral Shaft, Perc Endo Approach */
"0PCH0ZZ"  =  "1"  /* Extirpation of Matter from Right Radius, Open Approach */
"0PCH3ZZ"  =  "1"  /* Extirpation of Matter from Right Radius, Perc Approach */
"0PCH4ZZ"  =  "1"  /* Extirpation of Matter from Right Radius, Perc Endo Approach */
"0PCJ0ZZ"  =  "1"  /* Extirpation of Matter from Left Radius, Open Approach */
"0PCJ3ZZ"  =  "1"  /* Extirpation of Matter from Left Radius, Perc Approach */
"0PCJ4ZZ"  =  "1"  /* Extirpation of Matter from Left Radius, Perc Endo Approach */
"0PCK0ZZ"  =  "1"  /* Extirpation of Matter from Right Ulna, Open Approach */
"0PCK3ZZ"  =  "1"  /* Extirpation of Matter from Right Ulna, Percutaneous Approach */
"0PCK4ZZ"  =  "1"  /* Extirpation of Matter from Right Ulna, Perc Endo Approach */
"0PCL0ZZ"  =  "1"  /* Extirpation of Matter from Left Ulna, Open Approach */
"0PCL3ZZ"  =  "1"  /* Extirpation of Matter from Left Ulna, Percutaneous Approach */
"0PCL4ZZ"  =  "1"  /* Extirpation of Matter from Left Ulna, Perc Endo Approach */
"0PCM0ZZ"  =  "1"  /* Extirpation of Matter from Right Carpal, Open Approach */
"0PCM3ZZ"  =  "1"  /* Extirpation of Matter from Right Carpal, Perc Approach */
"0PCM4ZZ"  =  "1"  /* Extirpation of Matter from Right Carpal, Perc Endo Approach */
"0PCN0ZZ"  =  "1"  /* Extirpation of Matter from Left Carpal, Open Approach */
"0PCN3ZZ"  =  "1"  /* Extirpation of Matter from Left Carpal, Perc Approach */
"0PCN4ZZ"  =  "1"  /* Extirpation of Matter from Left Carpal, Perc Endo Approach */
"0PCP0ZZ"  =  "1"  /* Extirpation of Matter from Right Metacarpal, Open Approach */
"0PCP3ZZ"  =  "1"  /* Extirpation of Matter from Right Metacarpal, Perc Approach */
"0PCP4ZZ"  =  "1"  /* Extirpation of Matter from R Metacarpal, Perc Endo Approach */
"0PCQ0ZZ"  =  "1"  /* Extirpation of Matter from Left Metacarpal, Open Approach */
"0PCQ3ZZ"  =  "1"  /* Extirpation of Matter from Left Metacarpal, Perc Approach */
"0PCQ4ZZ"  =  "1"  /* Extirpation of Matter from L Metacarpal, Perc Endo Approach */
"0PCR0ZZ"  =  "1"  /* Extirpation of Matter from R Thumb Phalanx, Open Approach */
"0PCR3ZZ"  =  "1"  /* Extirpation of Matter from R Thumb Phalanx, Perc Approach */
"0PCR4ZZ"  =  "1"  /* Extirpate of Matter from R Thumb Phalanx, Perc Endo Approach */
"0PCS0ZZ"  =  "1"  /* Extirpation of Matter from Left Thumb Phalanx, Open Approach */
"0PCS3ZZ"  =  "1"  /* Extirpation of Matter from Left Thumb Phalanx, Perc Approach */
"0PCS4ZZ"  =  "1"  /* Extirpate of Matter from L Thumb Phalanx, Perc Endo Approach */
"0PCT0ZZ"  =  "1"  /* Extirpation of Matter from R Finger Phalanx, Open Approach */
"0PCT3ZZ"  =  "1"  /* Extirpation of Matter from R Finger Phalanx, Perc Approach */
"0PCT4ZZ"  =  "1"  /* Extirpate matter from R Finger Phalanx, Perc Endo */
"0PCV0ZZ"  =  "1"  /* Extirpation of Matter from L Finger Phalanx, Open Approach */
"0PCV3ZZ"  =  "1"  /* Extirpation of Matter from L Finger Phalanx, Perc Approach */
"0PCV4ZZ"  =  "1"  /* Extirpate matter from L Finger Phalanx, Perc Endo */
"0PH000Z"  =  "1"  /* Insertion of Rigid Plate Fix into Sternum, Open Approach */
"0PH004Z"  =  "1"  /* Insertion of Int Fix into Sternum, Open Approach */
"0PH030Z"  =  "1"  /* Insertion of Rigid Plate Fix into Sternum, Perc Approach */
"0PH034Z"  =  "1"  /* Insertion of Int Fix into Sternum, Perc Approach */
"0PH040Z"  =  "1"  /* Insert of Rigid Plate Fix into Sternum, Perc Endo Approach */
"0PH044Z"  =  "1"  /* Insertion of Int Fix into Sternum, Perc Endo Approach */
"0PH104Z"  =  "1"  /* Insertion of Int Fix into R Rib, Open Approach */
"0PH134Z"  =  "1"  /* Insertion of Int Fix into R Rib, Perc Approach */
"0PH144Z"  =  "1"  /* Insertion of Int Fix into R Rib, Perc Endo Approach */
"0PH204Z"  =  "1"  /* Insertion of Int Fix into L Rib, Open Approach */
"0PH234Z"  =  "1"  /* Insertion of Int Fix into L Rib, Perc Approach */
"0PH244Z"  =  "1"  /* Insertion of Int Fix into L Rib, Perc Endo Approach */
"0PH304Z"  =  "1"  /* Insertion of Int Fix into Cervcal Vertebra, Open Approach */
"0PH334Z"  =  "1"  /* Insertion of Int Fix into Cervcal Vertebra, Perc Approach */
"0PH344Z"  =  "1"  /* Insert of Int Fix into Cervcal Vertebra, Perc Endo Approach */
"0PH404Z"  =  "1"  /* Insertion of Int Fix into Thor Vertebra, Open Approach */
"0PH434Z"  =  "1"  /* Insertion of Int Fix into Thor Vertebra, Perc Approach */
"0PH444Z"  =  "1"  /* Insertion of Int Fix into Thor Vertebra, Perc Endo Approach */
"0PH504Z"  =  "1"  /* Insertion of Int Fix into R Scapula, Open Approach */
"0PH534Z"  =  "1"  /* Insertion of Int Fix into R Scapula, Perc Approach */
"0PH544Z"  =  "1"  /* Insertion of Int Fix into R Scapula, Perc Endo Approach */
"0PH604Z"  =  "1"  /* Insertion of Int Fix into L Scapula, Open Approach */
"0PH634Z"  =  "1"  /* Insertion of Int Fix into L Scapula, Perc Approach */
"0PH644Z"  =  "1"  /* Insertion of Int Fix into L Scapula, Perc Endo Approach */
"0PH704Z"  =  "1"  /* Insertion of Int Fix into R Glenoid Cav, Open Approach */
"0PH734Z"  =  "1"  /* Insertion of Int Fix into R Glenoid Cav, Perc Approach */
"0PH744Z"  =  "1"  /* Insertion of Int Fix into R Glenoid Cav, Perc Endo Approach */
"0PH804Z"  =  "1"  /* Insertion of Int Fix into L Glenoid Cav, Open Approach */
"0PH834Z"  =  "1"  /* Insertion of Int Fix into L Glenoid Cav, Perc Approach */
"0PH844Z"  =  "1"  /* Insertion of Int Fix into L Glenoid Cav, Perc Endo Approach */
"0PH904Z"  =  "1"  /* Insertion of Int Fix into R Clavicle, Open Approach */
"0PH934Z"  =  "1"  /* Insertion of Int Fix into R Clavicle, Perc Approach */
"0PH944Z"  =  "1"  /* Insertion of Int Fix into R Clavicle, Perc Endo Approach */
"0PHB04Z"  =  "1"  /* Insertion of Int Fix into L Clavicle, Open Approach */
"0PHB34Z"  =  "1"  /* Insertion of Int Fix into L Clavicle, Perc Approach */
"0PHB44Z"  =  "1"  /* Insertion of Int Fix into L Clavicle, Perc Endo Approach */
"0PHC04Z"  =  "1"  /* Insertion of Int Fix into R Humeral Head, Open Approach */
"0PHC05Z"  =  "1"  /* Insertion of Ext Fix into R Humeral Head, Open Approach */
"0PHC06Z"  =  "1"  /* Insertion of Intramed Fix into R Humeral Head, Open Approach */
"0PHC0BZ"  =  "1"  /* Insert of Monopln Ext Fix into R Humeral Head, Open Approach */
"0PHC0CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Humeral Head, Open Approach */
"0PHC0DZ"  =  "1"  /* Insert of Hybrid Ext Fix into R Humeral Head, Open Approach */
"0PHC34Z"  =  "1"  /* Insertion of Int Fix into R Humeral Head, Perc Approach */
"0PHC35Z"  =  "1"  /* Insertion of Ext Fix into R Humeral Head, Perc Approach */
"0PHC36Z"  =  "1"  /* Insertion of Intramed Fix into R Humeral Head, Perc Approach */
"0PHC3BZ"  =  "1"  /* Insert of Monopln Ext Fix into R Humeral Head, Perc Approach */
"0PHC3CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Humeral Head, Perc Approach */
"0PHC3DZ"  =  "1"  /* Insert of Hybrid Ext Fix into R Humeral Head, Perc Approach */
"0PHC44Z"  =  "1"  /* Insertion of Int Fix into R Humeral Head, Perc Endo Approach */
"0PHC45Z"  =  "1"  /* Insertion of Ext Fix into R Humeral Head, Perc Endo Approach */
"0PHC46Z"  =  "1"  /* Insert Intramed Fix in R Humeral Head, Perc Endo */
"0PHC4BZ"  =  "1"  /* Insert Monopln Ext Fix in R Humeral Head, Perc Endo */
"0PHC4CZ"  =  "1"  /* Insert Ring Ext Fix in R Humeral Head, Perc Endo */
"0PHC4DZ"  =  "1"  /* Insert Hybrid Ext Fix in R Humeral Head, Perc Endo */
"0PHD04Z"  =  "1"  /* Insertion of Int Fix into L Humeral Head, Open Approach */
"0PHD05Z"  =  "1"  /* Insertion of Ext Fix into L Humeral Head, Open Approach */
"0PHD06Z"  =  "1"  /* Insertion of Intramed Fix into L Humeral Head, Open Approach */
"0PHD0BZ"  =  "1"  /* Insert of Monopln Ext Fix into L Humeral Head, Open Approach */
"0PHD0CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Humeral Head, Open Approach */
"0PHD0DZ"  =  "1"  /* Insert of Hybrid Ext Fix into L Humeral Head, Open Approach */
"0PHD34Z"  =  "1"  /* Insertion of Int Fix into L Humeral Head, Perc Approach */
"0PHD35Z"  =  "1"  /* Insertion of Ext Fix into L Humeral Head, Perc Approach */
"0PHD36Z"  =  "1"  /* Insertion of Intramed Fix into L Humeral Head, Perc Approach */
"0PHD3BZ"  =  "1"  /* Insert of Monopln Ext Fix into L Humeral Head, Perc Approach */
"0PHD3CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Humeral Head, Perc Approach */
"0PHD3DZ"  =  "1"  /* Insert of Hybrid Ext Fix into L Humeral Head, Perc Approach */
"0PHD44Z"  =  "1"  /* Insertion of Int Fix into L Humeral Head, Perc Endo Approach */
"0PHD45Z"  =  "1"  /* Insertion of Ext Fix into L Humeral Head, Perc Endo Approach */
"0PHD46Z"  =  "1"  /* Insert Intramed Fix in L Humeral Head, Perc Endo */
"0PHD4BZ"  =  "1"  /* Insert Monopln Ext Fix in L Humeral Head, Perc Endo */
"0PHD4CZ"  =  "1"  /* Insert Ring Ext Fix in L Humeral Head, Perc Endo */
"0PHD4DZ"  =  "1"  /* Insert Hybrid Ext Fix in L Humeral Head, Perc Endo */
"0PHF04Z"  =  "1"  /* Insertion of Int Fix into R Humeral Shaft, Open Approach */
"0PHF05Z"  =  "1"  /* Insertion of Ext Fix into R Humeral Shaft, Open Approach */
"0PHF06Z"  =  "1"  /* Insert of Intramed Fix into R Humeral Shaft, Open Approach */
"0PHF0BZ"  =  "1"  /* Insert Monopln Ext Fix in R Humeral Shaft, Open */
"0PHF0CZ"  =  "1"  /* Insert of Ring Ext Fix into R Humeral Shaft, Open Approach */
"0PHF0DZ"  =  "1"  /* Insert of Hybrid Ext Fix into R Humeral Shaft, Open Approach */
"0PHF34Z"  =  "1"  /* Insertion of Int Fix into R Humeral Shaft, Perc Approach */
"0PHF35Z"  =  "1"  /* Insertion of Ext Fix into R Humeral Shaft, Perc Approach */
"0PHF36Z"  =  "1"  /* Insert of Intramed Fix into R Humeral Shaft, Perc Approach */
"0PHF3BZ"  =  "1"  /* Insert Monopln Ext Fix in R Humeral Shaft, Perc */
"0PHF3CZ"  =  "1"  /* Insert of Ring Ext Fix into R Humeral Shaft, Perc Approach */
"0PHF3DZ"  =  "1"  /* Insert of Hybrid Ext Fix into R Humeral Shaft, Perc Approach */
"0PHF44Z"  =  "1"  /* Insert of Int Fix into R Humeral Shaft, Perc Endo Approach */
"0PHF45Z"  =  "1"  /* Insert of Ext Fix into R Humeral Shaft, Perc Endo Approach */
"0PHF46Z"  =  "1"  /* Insert Intramed Fix in R Humeral Shaft, Perc Endo */
"0PHF4BZ"  =  "1"  /* Insert Monopln Ext Fix in R Humeral Shaft, Perc Endo */
"0PHF4CZ"  =  "1"  /* Insert Ring Ext Fix in R Humeral Shaft, Perc Endo */
"0PHF4DZ"  =  "1"  /* Insert Hybrid Ext Fix in R Humeral Shaft, Perc Endo */
"0PHG04Z"  =  "1"  /* Insertion of Int Fix into L Humeral Shaft, Open Approach */
"0PHG05Z"  =  "1"  /* Insertion of Ext Fix into L Humeral Shaft, Open Approach */
"0PHG06Z"  =  "1"  /* Insert of Intramed Fix into L Humeral Shaft, Open Approach */
"0PHG0BZ"  =  "1"  /* Insert Monopln Ext Fix in L Humeral Shaft, Open */
"0PHG0CZ"  =  "1"  /* Insert of Ring Ext Fix into L Humeral Shaft, Open Approach */
"0PHG0DZ"  =  "1"  /* Insert of Hybrid Ext Fix into L Humeral Shaft, Open Approach */
"0PHG34Z"  =  "1"  /* Insertion of Int Fix into L Humeral Shaft, Perc Approach */
"0PHG35Z"  =  "1"  /* Insertion of Ext Fix into L Humeral Shaft, Perc Approach */
"0PHG36Z"  =  "1"  /* Insert of Intramed Fix into L Humeral Shaft, Perc Approach */
"0PHG3BZ"  =  "1"  /* Insert Monopln Ext Fix in L Humeral Shaft, Perc */
"0PHG3CZ"  =  "1"  /* Insert of Ring Ext Fix into L Humeral Shaft, Perc Approach */
"0PHG3DZ"  =  "1"  /* Insert of Hybrid Ext Fix into L Humeral Shaft, Perc Approach */
"0PHG44Z"  =  "1"  /* Insert of Int Fix into L Humeral Shaft, Perc Endo Approach */
"0PHG45Z"  =  "1"  /* Insert of Ext Fix into L Humeral Shaft, Perc Endo Approach */
"0PHG46Z"  =  "1"  /* Insert Intramed Fix in L Humeral Shaft, Perc Endo */
"0PHG4BZ"  =  "1"  /* Insert Monopln Ext Fix in L Humeral Shaft, Perc Endo */
"0PHG4CZ"  =  "1"  /* Insert Ring Ext Fix in L Humeral Shaft, Perc Endo */
"0PHG4DZ"  =  "1"  /* Insert Hybrid Ext Fix in L Humeral Shaft, Perc Endo */
"0PHH04Z"  =  "1"  /* Insertion of Int Fix into R Radius, Open Approach */
"0PHH05Z"  =  "1"  /* Insertion of Ext Fix into R Radius, Open Approach */
"0PHH06Z"  =  "1"  /* Insertion of Intramed Fix into R Radius, Open Approach */
"0PHH0BZ"  =  "1"  /* Insertion of Monopln Ext Fix into R Radius, Open Approach */
"0PHH0CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Radius, Open Approach */
"0PHH0DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into R Radius, Open Approach */
"0PHH34Z"  =  "1"  /* Insertion of Int Fix into R Radius, Perc Approach */
"0PHH35Z"  =  "1"  /* Insertion of Ext Fix into R Radius, Perc Approach */
"0PHH36Z"  =  "1"  /* Insertion of Intramed Fix into R Radius, Perc Approach */
"0PHH3BZ"  =  "1"  /* Insertion of Monopln Ext Fix into R Radius, Perc Approach */
"0PHH3CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Radius, Perc Approach */
"0PHH3DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into R Radius, Perc Approach */
"0PHH44Z"  =  "1"  /* Insertion of Int Fix into R Radius, Perc Endo Approach */
"0PHH45Z"  =  "1"  /* Insertion of Ext Fix into R Radius, Perc Endo Approach */
"0PHH46Z"  =  "1"  /* Insertion of Intramed Fix into R Radius, Perc Endo Approach */
"0PHH4BZ"  =  "1"  /* Insert of Monopln Ext Fix into R Radius, Perc Endo Approach */
"0PHH4CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Radius, Perc Endo Approach */
"0PHH4DZ"  =  "1"  /* Insert of Hybrid Ext Fix into R Radius, Perc Endo Approach */
"0PHJ04Z"  =  "1"  /* Insertion of Int Fix into L Radius, Open Approach */
"0PHJ05Z"  =  "1"  /* Insertion of Ext Fix into L Radius, Open Approach */
"0PHJ06Z"  =  "1"  /* Insertion of Intramed Fix into L Radius, Open Approach */
"0PHJ0BZ"  =  "1"  /* Insertion of Monopln Ext Fix into L Radius, Open Approach */
"0PHJ0CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Radius, Open Approach */
"0PHJ0DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into L Radius, Open Approach */
"0PHJ34Z"  =  "1"  /* Insertion of Int Fix into L Radius, Perc Approach */
"0PHJ35Z"  =  "1"  /* Insertion of Ext Fix into L Radius, Perc Approach */
"0PHJ36Z"  =  "1"  /* Insertion of Intramed Fix into L Radius, Perc Approach */
"0PHJ3BZ"  =  "1"  /* Insertion of Monopln Ext Fix into L Radius, Perc Approach */
"0PHJ3CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Radius, Perc Approach */
"0PHJ3DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into L Radius, Perc Approach */
"0PHJ44Z"  =  "1"  /* Insertion of Int Fix into L Radius, Perc Endo Approach */
"0PHJ45Z"  =  "1"  /* Insertion of Ext Fix into L Radius, Perc Endo Approach */
"0PHJ46Z"  =  "1"  /* Insertion of Intramed Fix into L Radius, Perc Endo Approach */
"0PHJ4BZ"  =  "1"  /* Insert of Monopln Ext Fix into L Radius, Perc Endo Approach */
"0PHJ4CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Radius, Perc Endo Approach */
"0PHJ4DZ"  =  "1"  /* Insert of Hybrid Ext Fix into L Radius, Perc Endo Approach */
"0PHK04Z"  =  "1"  /* Insertion of Int Fix into R Ulna, Open Approach */
"0PHK05Z"  =  "1"  /* Insertion of Ext Fix into R Ulna, Open Approach */
"0PHK06Z"  =  "1"  /* Insertion of Intramed Fix into R Ulna, Open Approach */
"0PHK0BZ"  =  "1"  /* Insertion of Monopln Ext Fix into R Ulna, Open Approach */
"0PHK0CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Ulna, Open Approach */
"0PHK0DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into R Ulna, Open Approach */
"0PHK34Z"  =  "1"  /* Insertion of Int Fix into R Ulna, Perc Approach */
"0PHK35Z"  =  "1"  /* Insertion of Ext Fix into R Ulna, Perc Approach */
"0PHK36Z"  =  "1"  /* Insertion of Intramed Fix into R Ulna, Perc Approach */
"0PHK3BZ"  =  "1"  /* Insertion of Monopln Ext Fix into R Ulna, Perc Approach */
"0PHK3CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Ulna, Perc Approach */
"0PHK3DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into R Ulna, Perc Approach */
"0PHK44Z"  =  "1"  /* Insertion of Int Fix into R Ulna, Perc Endo Approach */
"0PHK45Z"  =  "1"  /* Insertion of Ext Fix into R Ulna, Perc Endo Approach */
"0PHK46Z"  =  "1"  /* Insertion of Intramed Fix into R Ulna, Perc Endo Approach */
"0PHK4BZ"  =  "1"  /* Insertion of Monopln Ext Fix into R Ulna, Perc Endo Approach */
"0PHK4CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Ulna, Perc Endo Approach */
"0PHK4DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into R Ulna, Perc Endo Approach */
"0PHL04Z"  =  "1"  /* Insertion of Int Fix into L Ulna, Open Approach */
"0PHL05Z"  =  "1"  /* Insertion of Ext Fix into L Ulna, Open Approach */
"0PHL06Z"  =  "1"  /* Insertion of Intramed Fix into L Ulna, Open Approach */
"0PHL0BZ"  =  "1"  /* Insertion of Monopln Ext Fix into L Ulna, Open Approach */
"0PHL0CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Ulna, Open Approach */
"0PHL0DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into L Ulna, Open Approach */
"0PHL34Z"  =  "1"  /* Insertion of Int Fix into L Ulna, Perc Approach */
"0PHL35Z"  =  "1"  /* Insertion of Ext Fix into L Ulna, Perc Approach */
"0PHL36Z"  =  "1"  /* Insertion of Intramed Fix into L Ulna, Perc Approach */
"0PHL3BZ"  =  "1"  /* Insertion of Monopln Ext Fix into L Ulna, Perc Approach */
"0PHL3CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Ulna, Perc Approach */
"0PHL3DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into L Ulna, Perc Approach */
"0PHL44Z"  =  "1"  /* Insertion of Int Fix into L Ulna, Perc Endo Approach */
"0PHL45Z"  =  "1"  /* Insertion of Ext Fix into L Ulna, Perc Endo Approach */
"0PHL46Z"  =  "1"  /* Insertion of Intramed Fix into L Ulna, Perc Endo Approach */
"0PHL4BZ"  =  "1"  /* Insertion of Monopln Ext Fix into L Ulna, Perc Endo Approach */
"0PHL4CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Ulna, Perc Endo Approach */
"0PHL4DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into L Ulna, Perc Endo Approach */
"0PHM04Z"  =  "1"  /* Insertion of Int Fix into R Carpal, Open Approach */
"0PHM05Z"  =  "1"  /* Insertion of Ext Fix into R Carpal, Open Approach */
"0PHM34Z"  =  "1"  /* Insertion of Int Fix into R Carpal, Perc Approach */
"0PHM35Z"  =  "1"  /* Insertion of Ext Fix into R Carpal, Perc Approach */
"0PHM44Z"  =  "1"  /* Insertion of Int Fix into R Carpal, Perc Endo Approach */
"0PHM45Z"  =  "1"  /* Insertion of Ext Fix into R Carpal, Perc Endo Approach */
"0PHN04Z"  =  "1"  /* Insertion of Int Fix into L Carpal, Open Approach */
"0PHN05Z"  =  "1"  /* Insertion of Ext Fix into L Carpal, Open Approach */
"0PHN34Z"  =  "1"  /* Insertion of Int Fix into L Carpal, Perc Approach */
"0PHN35Z"  =  "1"  /* Insertion of Ext Fix into L Carpal, Perc Approach */
"0PHN44Z"  =  "1"  /* Insertion of Int Fix into L Carpal, Perc Endo Approach */
"0PHN45Z"  =  "1"  /* Insertion of Ext Fix into L Carpal, Perc Endo Approach */
"0PHP04Z"  =  "1"  /* Insertion of Int Fix into R Metacarpal, Open Approach */
"0PHP05Z"  =  "1"  /* Insertion of Ext Fix into R Metacarpal, Open Approach */
"0PHP34Z"  =  "1"  /* Insertion of Int Fix into R Metacarpal, Perc Approach */
"0PHP35Z"  =  "1"  /* Insertion of Ext Fix into R Metacarpal, Perc Approach */
"0PHP44Z"  =  "1"  /* Insertion of Int Fix into R Metacarpal, Perc Endo Approach */
"0PHP45Z"  =  "1"  /* Insertion of Ext Fix into R Metacarpal, Perc Endo Approach */
"0PHQ04Z"  =  "1"  /* Insertion of Int Fix into L Metacarpal, Open Approach */
"0PHQ05Z"  =  "1"  /* Insertion of Ext Fix into L Metacarpal, Open Approach */
"0PHQ34Z"  =  "1"  /* Insertion of Int Fix into L Metacarpal, Perc Approach */
"0PHQ35Z"  =  "1"  /* Insertion of Ext Fix into L Metacarpal, Perc Approach */
"0PHQ44Z"  =  "1"  /* Insertion of Int Fix into L Metacarpal, Perc Endo Approach */
"0PHQ45Z"  =  "1"  /* Insertion of Ext Fix into L Metacarpal, Perc Endo Approach */
"0PHR04Z"  =  "1"  /* Insertion of Int Fix into R Thumb Phalanx, Open Approach */
"0PHR05Z"  =  "1"  /* Insertion of Ext Fix into R Thumb Phalanx, Open Approach */
"0PHR34Z"  =  "1"  /* Insertion of Int Fix into R Thumb Phalanx, Perc Approach */
"0PHR35Z"  =  "1"  /* Insertion of Ext Fix into R Thumb Phalanx, Perc Approach */
"0PHR44Z"  =  "1"  /* Insert of Int Fix into R Thumb Phalanx, Perc Endo Approach */
"0PHR45Z"  =  "1"  /* Insert of Ext Fix into R Thumb Phalanx, Perc Endo Approach */
"0PHS04Z"  =  "1"  /* Insertion of Int Fix into L Thumb Phalanx, Open Approach */
"0PHS05Z"  =  "1"  /* Insertion of Ext Fix into L Thumb Phalanx, Open Approach */
"0PHS34Z"  =  "1"  /* Insertion of Int Fix into L Thumb Phalanx, Perc Approach */
"0PHS35Z"  =  "1"  /* Insertion of Ext Fix into L Thumb Phalanx, Perc Approach */
"0PHS44Z"  =  "1"  /* Insert of Int Fix into L Thumb Phalanx, Perc Endo Approach */
"0PHS45Z"  =  "1"  /* Insert of Ext Fix into L Thumb Phalanx, Perc Endo Approach */
"0PHT04Z"  =  "1"  /* Insertion of Int Fix into R Finger Phalanx, Open Approach */
"0PHT05Z"  =  "1"  /* Insertion of Ext Fix into R Finger Phalanx, Open Approach */
"0PHT34Z"  =  "1"  /* Insertion of Int Fix into R Finger Phalanx, Perc Approach */
"0PHT35Z"  =  "1"  /* Insertion of Ext Fix into R Finger Phalanx, Perc Approach */
"0PHT44Z"  =  "1"  /* Insert of Int Fix into R Finger Phalanx, Perc Endo Approach */
"0PHT45Z"  =  "1"  /* Insert of Ext Fix into R Finger Phalanx, Perc Endo Approach */
"0PHV04Z"  =  "1"  /* Insertion of Int Fix into L Finger Phalanx, Open Approach */
"0PHV05Z"  =  "1"  /* Insertion of Ext Fix into L Finger Phalanx, Open Approach */
"0PHV34Z"  =  "1"  /* Insertion of Int Fix into L Finger Phalanx, Perc Approach */
"0PHV35Z"  =  "1"  /* Insertion of Ext Fix into L Finger Phalanx, Perc Approach */
"0PHV44Z"  =  "1"  /* Insert of Int Fix into L Finger Phalanx, Perc Endo Approach */
"0PHV45Z"  =  "1"  /* Insert of Ext Fix into L Finger Phalanx, Perc Endo Approach */
"0PHY0MZ"  =  "1"  /* Insertion of Bone Stim into Up Bone, Open Approach */
"0PHY3MZ"  =  "1"  /* Insertion of Bone Stim into Up Bone, Perc Approach */
"0PHY4MZ"  =  "1"  /* Insertion of Bone Stim into Up Bone, Perc Endo Approach */
"0PJY0ZZ"  =  "1"  /* Inspection of Upper Bone, Open Approach */
"0PJY3ZZ"  =  "1"  /* Inspection of Upper Bone, Percutaneous Approach */
"0PJY4ZZ"  =  "1"  /* Inspection of Upper Bone, Percutaneous Endoscopic Approach */
"0PN00ZZ"  =  "1"  /* Release Sternum, Open Approach */
"0PN03ZZ"  =  "1"  /* Release Sternum, Percutaneous Approach */
"0PN04ZZ"  =  "1"  /* Release Sternum, Percutaneous Endoscopic Approach */
"0PN10ZZ"  =  "1"  /* Release Right Rib, Open Approach */
"0PN13ZZ"  =  "1"  /* Release Right Rib, Percutaneous Approach */
"0PN14ZZ"  =  "1"  /* Release Right Rib, Percutaneous Endoscopic Approach */
"0PN20ZZ"  =  "1"  /* Release Left Rib, Open Approach */
"0PN23ZZ"  =  "1"  /* Release Left Rib, Percutaneous Approach */
"0PN24ZZ"  =  "1"  /* Release Left Rib, Percutaneous Endoscopic Approach */
"0PN30ZZ"  =  "1"  /* Release Cervical Vertebra, Open Approach */
"0PN33ZZ"  =  "1"  /* Release Cervical Vertebra, Percutaneous Approach */
"0PN34ZZ"  =  "1"  /* Release Cervical Vertebra, Percutaneous Endoscopic Approach */
"0PN40ZZ"  =  "1"  /* Release Thoracic Vertebra, Open Approach */
"0PN43ZZ"  =  "1"  /* Release Thoracic Vertebra, Percutaneous Approach */
"0PN44ZZ"  =  "1"  /* Release Thoracic Vertebra, Percutaneous Endoscopic Approach */
"0PN50ZZ"  =  "1"  /* Release Right Scapula, Open Approach */
"0PN53ZZ"  =  "1"  /* Release Right Scapula, Percutaneous Approach */
"0PN54ZZ"  =  "1"  /* Release Right Scapula, Percutaneous Endoscopic Approach */
"0PN60ZZ"  =  "1"  /* Release Left Scapula, Open Approach */
"0PN63ZZ"  =  "1"  /* Release Left Scapula, Percutaneous Approach */
"0PN64ZZ"  =  "1"  /* Release Left Scapula, Percutaneous Endoscopic Approach */
"0PN70ZZ"  =  "1"  /* Release Right Glenoid Cavity, Open Approach */
"0PN73ZZ"  =  "1"  /* Release Right Glenoid Cavity, Percutaneous Approach */
"0PN74ZZ"  =  "1"  /* Release Right Glenoid Cavity, Perc Endo Approach */
"0PN80ZZ"  =  "1"  /* Release Left Glenoid Cavity, Open Approach */
"0PN83ZZ"  =  "1"  /* Release Left Glenoid Cavity, Percutaneous Approach */
"0PN84ZZ"  =  "1"  /* Release Left Glenoid Cavity, Perc Endo Approach */
"0PN90ZZ"  =  "1"  /* Release Right Clavicle, Open Approach */
"0PN93ZZ"  =  "1"  /* Release Right Clavicle, Percutaneous Approach */
"0PN94ZZ"  =  "1"  /* Release Right Clavicle, Percutaneous Endoscopic Approach */
"0PNB0ZZ"  =  "1"  /* Release Left Clavicle, Open Approach */
"0PNB3ZZ"  =  "1"  /* Release Left Clavicle, Percutaneous Approach */
"0PNB4ZZ"  =  "1"  /* Release Left Clavicle, Percutaneous Endoscopic Approach */
"0PNC0ZZ"  =  "1"  /* Release Right Humeral Head, Open Approach */
"0PNC3ZZ"  =  "1"  /* Release Right Humeral Head, Percutaneous Approach */
"0PNC4ZZ"  =  "1"  /* Release Right Humeral Head, Percutaneous Endoscopic Approach */
"0PND0ZZ"  =  "1"  /* Release Left Humeral Head, Open Approach */
"0PND3ZZ"  =  "1"  /* Release Left Humeral Head, Percutaneous Approach */
"0PND4ZZ"  =  "1"  /* Release Left Humeral Head, Percutaneous Endoscopic Approach */
"0PNF0ZZ"  =  "1"  /* Release Right Humeral Shaft, Open Approach */
"0PNF3ZZ"  =  "1"  /* Release Right Humeral Shaft, Percutaneous Approach */
"0PNF4ZZ"  =  "1"  /* Release Right Humeral Shaft, Perc Endo Approach */
"0PNG0ZZ"  =  "1"  /* Release Left Humeral Shaft, Open Approach */
"0PNG3ZZ"  =  "1"  /* Release Left Humeral Shaft, Percutaneous Approach */
"0PNG4ZZ"  =  "1"  /* Release Left Humeral Shaft, Percutaneous Endoscopic Approach */
"0PNH0ZZ"  =  "1"  /* Release Right Radius, Open Approach */
"0PNH3ZZ"  =  "1"  /* Release Right Radius, Percutaneous Approach */
"0PNH4ZZ"  =  "1"  /* Release Right Radius, Percutaneous Endoscopic Approach */
"0PNJ0ZZ"  =  "1"  /* Release Left Radius, Open Approach */
"0PNJ3ZZ"  =  "1"  /* Release Left Radius, Percutaneous Approach */
"0PNJ4ZZ"  =  "1"  /* Release Left Radius, Percutaneous Endoscopic Approach */
"0PNK0ZZ"  =  "1"  /* Release Right Ulna, Open Approach */
"0PNK3ZZ"  =  "1"  /* Release Right Ulna, Percutaneous Approach */
"0PNK4ZZ"  =  "1"  /* Release Right Ulna, Percutaneous Endoscopic Approach */
"0PNL0ZZ"  =  "1"  /* Release Left Ulna, Open Approach */
"0PNL3ZZ"  =  "1"  /* Release Left Ulna, Percutaneous Approach */
"0PNL4ZZ"  =  "1"  /* Release Left Ulna, Percutaneous Endoscopic Approach */
"0PNM0ZZ"  =  "1"  /* Release Right Carpal, Open Approach */
"0PNM3ZZ"  =  "1"  /* Release Right Carpal, Percutaneous Approach */
"0PNM4ZZ"  =  "1"  /* Release Right Carpal, Percutaneous Endoscopic Approach */
"0PNN0ZZ"  =  "1"  /* Release Left Carpal, Open Approach */
"0PNN3ZZ"  =  "1"  /* Release Left Carpal, Percutaneous Approach */
"0PNN4ZZ"  =  "1"  /* Release Left Carpal, Percutaneous Endoscopic Approach */
"0PNP0ZZ"  =  "1"  /* Release Right Metacarpal, Open Approach */
"0PNP3ZZ"  =  "1"  /* Release Right Metacarpal, Percutaneous Approach */
"0PNP4ZZ"  =  "1"  /* Release Right Metacarpal, Percutaneous Endoscopic Approach */
"0PNQ0ZZ"  =  "1"  /* Release Left Metacarpal, Open Approach */
"0PNQ3ZZ"  =  "1"  /* Release Left Metacarpal, Percutaneous Approach */
"0PNQ4ZZ"  =  "1"  /* Release Left Metacarpal, Percutaneous Endoscopic Approach */
"0PNR0ZZ"  =  "1"  /* Release Right Thumb Phalanx, Open Approach */
"0PNR3ZZ"  =  "1"  /* Release Right Thumb Phalanx, Percutaneous Approach */
"0PNR4ZZ"  =  "1"  /* Release Right Thumb Phalanx, Perc Endo Approach */
"0PNS0ZZ"  =  "1"  /* Release Left Thumb Phalanx, Open Approach */
"0PNS3ZZ"  =  "1"  /* Release Left Thumb Phalanx, Percutaneous Approach */
"0PNS4ZZ"  =  "1"  /* Release Left Thumb Phalanx, Percutaneous Endoscopic Approach */
"0PNT0ZZ"  =  "1"  /* Release Right Finger Phalanx, Open Approach */
"0PNT3ZZ"  =  "1"  /* Release Right Finger Phalanx, Percutaneous Approach */
"0PNT4ZZ"  =  "1"  /* Release Right Finger Phalanx, Perc Endo Approach */
"0PNV0ZZ"  =  "1"  /* Release Left Finger Phalanx, Open Approach */
"0PNV3ZZ"  =  "1"  /* Release Left Finger Phalanx, Percutaneous Approach */
"0PNV4ZZ"  =  "1"  /* Release Left Finger Phalanx, Perc Endo Approach */
"0PP004Z"  =  "1"  /* Removal of Int Fix from Sternum, Open Approach */
"0PP007Z"  =  "1"  /* Removal of Autol Sub from Sternum, Open Approach */
"0PP00JZ"  =  "1"  /* Removal of Synthetic Substitute from Sternum, Open Approach */
"0PP00KZ"  =  "1"  /* Removal of Nonaut Sub from Sternum, Open Approach */
"0PP034Z"  =  "1"  /* Removal of Int Fix from Sternum, Perc Approach */
"0PP037Z"  =  "1"  /* Removal of Autol Sub from Sternum, Perc Approach */
"0PP03JZ"  =  "1"  /* Removal of Synthetic Substitute from Sternum, Perc Approach */
"0PP03KZ"  =  "1"  /* Removal of Nonaut Sub from Sternum, Perc Approach */
"0PP044Z"  =  "1"  /* Removal of Int Fix from Sternum, Perc Endo Approach */
"0PP047Z"  =  "1"  /* Removal of Autol Sub from Sternum, Perc Endo Approach */
"0PP04JZ"  =  "1"  /* Removal of Synth Sub from Sternum, Perc Endo Approach */
"0PP04KZ"  =  "1"  /* Removal of Nonaut Sub from Sternum, Perc Endo Approach */
"0PP104Z"  =  "1"  /* Removal of Int Fix from R Rib, Open Approach */
"0PP107Z"  =  "1"  /* Removal of Autol Sub from R Rib, Open Approach */
"0PP10JZ"  =  "1"  /* Removal of Synthetic Substitute from R Rib, Open Approach */
"0PP10KZ"  =  "1"  /* Removal of Nonaut Sub from R Rib, Open Approach */
"0PP134Z"  =  "1"  /* Removal of Int Fix from R Rib, Perc Approach */
"0PP137Z"  =  "1"  /* Removal of Autol Sub from R Rib, Perc Approach */
"0PP13JZ"  =  "1"  /* Removal of Synthetic Substitute from R Rib, Perc Approach */
"0PP13KZ"  =  "1"  /* Removal of Nonaut Sub from R Rib, Perc Approach */
"0PP144Z"  =  "1"  /* Removal of Int Fix from R Rib, Perc Endo Approach */
"0PP147Z"  =  "1"  /* Removal of Autol Sub from R Rib, Perc Endo Approach */
"0PP14JZ"  =  "1"  /* Removal of Synth Sub from R Rib, Perc Endo Approach */
"0PP14KZ"  =  "1"  /* Removal of Nonaut Sub from R Rib, Perc Endo Approach */
"0PP204Z"  =  "1"  /* Removal of Int Fix from L Rib, Open Approach */
"0PP207Z"  =  "1"  /* Removal of Autol Sub from L Rib, Open Approach */
"0PP20JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Rib, Open Approach */
"0PP20KZ"  =  "1"  /* Removal of Nonaut Sub from L Rib, Open Approach */
"0PP234Z"  =  "1"  /* Removal of Int Fix from L Rib, Perc Approach */
"0PP237Z"  =  "1"  /* Removal of Autol Sub from L Rib, Perc Approach */
"0PP23JZ"  =  "1"  /* Removal of Synthetic Substitute from Left Rib, Perc Approach */
"0PP23KZ"  =  "1"  /* Removal of Nonaut Sub from L Rib, Perc Approach */
"0PP244Z"  =  "1"  /* Removal of Int Fix from L Rib, Perc Endo Approach */
"0PP247Z"  =  "1"  /* Removal of Autol Sub from L Rib, Perc Endo Approach */
"0PP24JZ"  =  "1"  /* Removal of Synth Sub from L Rib, Perc Endo Approach */
"0PP24KZ"  =  "1"  /* Removal of Nonaut Sub from L Rib, Perc Endo Approach */
"0PP304Z"  =  "1"  /* Removal of Int Fix from Cervcal Vertebra, Open Approach */
"0PP307Z"  =  "1"  /* Removal of Autol Sub from Cervcal Vertebra, Open Approach */
"0PP30JZ"  =  "1"  /* Removal of Synth Sub from Cervcal Vertebra, Open Approach */
"0PP30KZ"  =  "1"  /* Removal of Nonaut Sub from Cervcal Vertebra, Open Approach */
"0PP334Z"  =  "1"  /* Removal of Int Fix from Cervcal Vertebra, Perc Approach */
"0PP337Z"  =  "1"  /* Removal of Autol Sub from Cervcal Vertebra, Perc Approach */
"0PP33JZ"  =  "1"  /* Removal of Synth Sub from Cervcal Vertebra, Perc Approach */
"0PP33KZ"  =  "1"  /* Removal of Nonaut Sub from Cervcal Vertebra, Perc Approach */
"0PP344Z"  =  "1"  /* Removal of Int Fix from Cervcal Vertebra, Perc Endo Approach */
"0PP347Z"  =  "1"  /* Remove Autol Sub from Cervcal Vertebra, Perc Endo */
"0PP34JZ"  =  "1"  /* Remove Synth Sub from Cervcal Vertebra, Perc Endo */
"0PP34KZ"  =  "1"  /* Remove Nonaut Sub from Cervcal Vertebra, Perc Endo */
"0PP404Z"  =  "1"  /* Removal of Int Fix from Thor Vertebra, Open Approach */
"0PP407Z"  =  "1"  /* Removal of Autol Sub from Thor Vertebra, Open Approach */
"0PP40JZ"  =  "1"  /* Removal of Synth Sub from Thor Vertebra, Open Approach */
"0PP40KZ"  =  "1"  /* Removal of Nonaut Sub from Thor Vertebra, Open Approach */
"0PP434Z"  =  "1"  /* Removal of Int Fix from Thor Vertebra, Perc Approach */
"0PP437Z"  =  "1"  /* Removal of Autol Sub from Thor Vertebra, Perc Approach */
"0PP43JZ"  =  "1"  /* Removal of Synth Sub from Thor Vertebra, Perc Approach */
"0PP43KZ"  =  "1"  /* Removal of Nonaut Sub from Thor Vertebra, Perc Approach */
"0PP444Z"  =  "1"  /* Removal of Int Fix from Thor Vertebra, Perc Endo Approach */
"0PP447Z"  =  "1"  /* Removal of Autol Sub from Thor Vertebra, Perc Endo Approach */
"0PP44JZ"  =  "1"  /* Removal of Synth Sub from Thor Vertebra, Perc Endo Approach */
"0PP44KZ"  =  "1"  /* Removal of Nonaut Sub from Thor Vertebra, Perc Endo Approach */
"0PP504Z"  =  "1"  /* Removal of Int Fix from R Scapula, Open Approach */
"0PP507Z"  =  "1"  /* Removal of Autol Sub from R Scapula, Open Approach */
"0PP50JZ"  =  "1"  /* Removal of Synth Sub from R Scapula, Open Approach */
"0PP50KZ"  =  "1"  /* Removal of Nonaut Sub from R Scapula, Open Approach */
"0PP534Z"  =  "1"  /* Removal of Int Fix from R Scapula, Perc Approach */
"0PP537Z"  =  "1"  /* Removal of Autol Sub from R Scapula, Perc Approach */
"0PP53JZ"  =  "1"  /* Removal of Synth Sub from R Scapula, Perc Approach */
"0PP53KZ"  =  "1"  /* Removal of Nonaut Sub from R Scapula, Perc Approach */
"0PP544Z"  =  "1"  /* Removal of Int Fix from R Scapula, Perc Endo Approach */
"0PP547Z"  =  "1"  /* Removal of Autol Sub from R Scapula, Perc Endo Approach */
"0PP54JZ"  =  "1"  /* Removal of Synth Sub from R Scapula, Perc Endo Approach */
"0PP54KZ"  =  "1"  /* Removal of Nonaut Sub from R Scapula, Perc Endo Approach */
"0PP604Z"  =  "1"  /* Removal of Int Fix from L Scapula, Open Approach */
"0PP607Z"  =  "1"  /* Removal of Autol Sub from L Scapula, Open Approach */
"0PP60JZ"  =  "1"  /* Removal of Synth Sub from L Scapula, Open Approach */
"0PP60KZ"  =  "1"  /* Removal of Nonaut Sub from L Scapula, Open Approach */
"0PP634Z"  =  "1"  /* Removal of Int Fix from L Scapula, Perc Approach */
"0PP637Z"  =  "1"  /* Removal of Autol Sub from L Scapula, Perc Approach */
"0PP63JZ"  =  "1"  /* Removal of Synth Sub from L Scapula, Perc Approach */
"0PP63KZ"  =  "1"  /* Removal of Nonaut Sub from L Scapula, Perc Approach */
"0PP644Z"  =  "1"  /* Removal of Int Fix from L Scapula, Perc Endo Approach */
"0PP647Z"  =  "1"  /* Removal of Autol Sub from L Scapula, Perc Endo Approach */
"0PP64JZ"  =  "1"  /* Removal of Synth Sub from L Scapula, Perc Endo Approach */
"0PP64KZ"  =  "1"  /* Removal of Nonaut Sub from L Scapula, Perc Endo Approach */
"0PP704Z"  =  "1"  /* Removal of Int Fix from R Glenoid Cav, Open Approach */
"0PP707Z"  =  "1"  /* Removal of Autol Sub from R Glenoid Cav, Open Approach */
"0PP70JZ"  =  "1"  /* Removal of Synth Sub from R Glenoid Cav, Open Approach */
"0PP70KZ"  =  "1"  /* Removal of Nonaut Sub from R Glenoid Cav, Open Approach */
"0PP734Z"  =  "1"  /* Removal of Int Fix from R Glenoid Cav, Perc Approach */
"0PP737Z"  =  "1"  /* Removal of Autol Sub from R Glenoid Cav, Perc Approach */
"0PP73JZ"  =  "1"  /* Removal of Synth Sub from R Glenoid Cav, Perc Approach */
"0PP73KZ"  =  "1"  /* Removal of Nonaut Sub from R Glenoid Cav, Perc Approach */
"0PP744Z"  =  "1"  /* Removal of Int Fix from R Glenoid Cav, Perc Endo Approach */
"0PP747Z"  =  "1"  /* Removal of Autol Sub from R Glenoid Cav, Perc Endo Approach */
"0PP74JZ"  =  "1"  /* Removal of Synth Sub from R Glenoid Cav, Perc Endo Approach */
"0PP74KZ"  =  "1"  /* Removal of Nonaut Sub from R Glenoid Cav, Perc Endo Approach */
"0PP804Z"  =  "1"  /* Removal of Int Fix from L Glenoid Cav, Open Approach */
"0PP807Z"  =  "1"  /* Removal of Autol Sub from L Glenoid Cav, Open Approach */
"0PP80JZ"  =  "1"  /* Removal of Synth Sub from L Glenoid Cav, Open Approach */
"0PP80KZ"  =  "1"  /* Removal of Nonaut Sub from L Glenoid Cav, Open Approach */
"0PP834Z"  =  "1"  /* Removal of Int Fix from L Glenoid Cav, Perc Approach */
"0PP837Z"  =  "1"  /* Removal of Autol Sub from L Glenoid Cav, Perc Approach */
"0PP83JZ"  =  "1"  /* Removal of Synth Sub from L Glenoid Cav, Perc Approach */
"0PP83KZ"  =  "1"  /* Removal of Nonaut Sub from L Glenoid Cav, Perc Approach */
"0PP844Z"  =  "1"  /* Removal of Int Fix from L Glenoid Cav, Perc Endo Approach */
"0PP847Z"  =  "1"  /* Removal of Autol Sub from L Glenoid Cav, Perc Endo Approach */
"0PP84JZ"  =  "1"  /* Removal of Synth Sub from L Glenoid Cav, Perc Endo Approach */
"0PP84KZ"  =  "1"  /* Removal of Nonaut Sub from L Glenoid Cav, Perc Endo Approach */
"0PP904Z"  =  "1"  /* Removal of Int Fix from R Clavicle, Open Approach */
"0PP907Z"  =  "1"  /* Removal of Autol Sub from R Clavicle, Open Approach */
"0PP90JZ"  =  "1"  /* Removal of Synth Sub from R Clavicle, Open Approach */
"0PP90KZ"  =  "1"  /* Removal of Nonaut Sub from R Clavicle, Open Approach */
"0PP934Z"  =  "1"  /* Removal of Int Fix from R Clavicle, Perc Approach */
"0PP937Z"  =  "1"  /* Removal of Autol Sub from R Clavicle, Perc Approach */
"0PP93JZ"  =  "1"  /* Removal of Synth Sub from R Clavicle, Perc Approach */
"0PP93KZ"  =  "1"  /* Removal of Nonaut Sub from R Clavicle, Perc Approach */
"0PP944Z"  =  "1"  /* Removal of Int Fix from R Clavicle, Perc Endo Approach */
"0PP947Z"  =  "1"  /* Removal of Autol Sub from R Clavicle, Perc Endo Approach */
"0PP94JZ"  =  "1"  /* Removal of Synth Sub from R Clavicle, Perc Endo Approach */
"0PP94KZ"  =  "1"  /* Removal of Nonaut Sub from R Clavicle, Perc Endo Approach */
"0PPB04Z"  =  "1"  /* Removal of Int Fix from L Clavicle, Open Approach */
"0PPB07Z"  =  "1"  /* Removal of Autol Sub from L Clavicle, Open Approach */
"0PPB0JZ"  =  "1"  /* Removal of Synth Sub from L Clavicle, Open Approach */
"0PPB0KZ"  =  "1"  /* Removal of Nonaut Sub from L Clavicle, Open Approach */
"0PPB34Z"  =  "1"  /* Removal of Int Fix from L Clavicle, Perc Approach */
"0PPB37Z"  =  "1"  /* Removal of Autol Sub from L Clavicle, Perc Approach */
"0PPB3JZ"  =  "1"  /* Removal of Synth Sub from L Clavicle, Perc Approach */
"0PPB3KZ"  =  "1"  /* Removal of Nonaut Sub from L Clavicle, Perc Approach */
"0PPB44Z"  =  "1"  /* Removal of Int Fix from L Clavicle, Perc Endo Approach */
"0PPB47Z"  =  "1"  /* Removal of Autol Sub from L Clavicle, Perc Endo Approach */
"0PPB4JZ"  =  "1"  /* Removal of Synth Sub from L Clavicle, Perc Endo Approach */
"0PPB4KZ"  =  "1"  /* Removal of Nonaut Sub from L Clavicle, Perc Endo Approach */
"0PPC04Z"  =  "1"  /* Removal of Int Fix from R Humeral Head, Open Approach */
"0PPC05Z"  =  "1"  /* Removal of Ext Fix from R Humeral Head, Open Approach */
"0PPC07Z"  =  "1"  /* Removal of Autol Sub from R Humeral Head, Open Approach */
"0PPC0JZ"  =  "1"  /* Removal of Synth Sub from R Humeral Head, Open Approach */
"0PPC0KZ"  =  "1"  /* Removal of Nonaut Sub from R Humeral Head, Open Approach */
"0PPC34Z"  =  "1"  /* Removal of Int Fix from R Humeral Head, Perc Approach */
"0PPC35Z"  =  "1"  /* Removal of Ext Fix from R Humeral Head, Perc Approach */
"0PPC37Z"  =  "1"  /* Removal of Autol Sub from R Humeral Head, Perc Approach */
"0PPC3JZ"  =  "1"  /* Removal of Synth Sub from R Humeral Head, Perc Approach */
"0PPC3KZ"  =  "1"  /* Removal of Nonaut Sub from R Humeral Head, Perc Approach */
"0PPC44Z"  =  "1"  /* Removal of Int Fix from R Humeral Head, Perc Endo Approach */
"0PPC45Z"  =  "1"  /* Removal of Ext Fix from R Humeral Head, Perc Endo Approach */
"0PPC47Z"  =  "1"  /* Removal of Autol Sub from R Humeral Head, Perc Endo Approach */
"0PPC4JZ"  =  "1"  /* Removal of Synth Sub from R Humeral Head, Perc Endo Approach */
"0PPC4KZ"  =  "1"  /* Remove of Nonaut Sub from R Humeral Head, Perc Endo Approach */
"0PPD04Z"  =  "1"  /* Removal of Int Fix from L Humeral Head, Open Approach */
"0PPD05Z"  =  "1"  /* Removal of Ext Fix from L Humeral Head, Open Approach */
"0PPD07Z"  =  "1"  /* Removal of Autol Sub from L Humeral Head, Open Approach */
"0PPD0JZ"  =  "1"  /* Removal of Synth Sub from L Humeral Head, Open Approach */
"0PPD0KZ"  =  "1"  /* Removal of Nonaut Sub from L Humeral Head, Open Approach */
"0PPD34Z"  =  "1"  /* Removal of Int Fix from L Humeral Head, Perc Approach */
"0PPD35Z"  =  "1"  /* Removal of Ext Fix from L Humeral Head, Perc Approach */
"0PPD37Z"  =  "1"  /* Removal of Autol Sub from L Humeral Head, Perc Approach */
"0PPD3JZ"  =  "1"  /* Removal of Synth Sub from L Humeral Head, Perc Approach */
"0PPD3KZ"  =  "1"  /* Removal of Nonaut Sub from L Humeral Head, Perc Approach */
"0PPD44Z"  =  "1"  /* Removal of Int Fix from L Humeral Head, Perc Endo Approach */
"0PPD45Z"  =  "1"  /* Removal of Ext Fix from L Humeral Head, Perc Endo Approach */
"0PPD47Z"  =  "1"  /* Removal of Autol Sub from L Humeral Head, Perc Endo Approach */
"0PPD4JZ"  =  "1"  /* Removal of Synth Sub from L Humeral Head, Perc Endo Approach */
"0PPD4KZ"  =  "1"  /* Remove of Nonaut Sub from L Humeral Head, Perc Endo Approach */
"0PPF04Z"  =  "1"  /* Removal of Int Fix from R Humeral Shaft, Open Approach */
"0PPF05Z"  =  "1"  /* Removal of Ext Fix from R Humeral Shaft, Open Approach */
"0PPF07Z"  =  "1"  /* Removal of Autol Sub from R Humeral Shaft, Open Approach */
"0PPF0JZ"  =  "1"  /* Removal of Synth Sub from R Humeral Shaft, Open Approach */
"0PPF0KZ"  =  "1"  /* Removal of Nonaut Sub from R Humeral Shaft, Open Approach */
"0PPF34Z"  =  "1"  /* Removal of Int Fix from R Humeral Shaft, Perc Approach */
"0PPF35Z"  =  "1"  /* Removal of Ext Fix from R Humeral Shaft, Perc Approach */
"0PPF37Z"  =  "1"  /* Removal of Autol Sub from R Humeral Shaft, Perc Approach */
"0PPF3JZ"  =  "1"  /* Removal of Synth Sub from R Humeral Shaft, Perc Approach */
"0PPF3KZ"  =  "1"  /* Removal of Nonaut Sub from R Humeral Shaft, Perc Approach */
"0PPF44Z"  =  "1"  /* Removal of Int Fix from R Humeral Shaft, Perc Endo Approach */
"0PPF45Z"  =  "1"  /* Removal of Ext Fix from R Humeral Shaft, Perc Endo Approach */
"0PPF47Z"  =  "1"  /* Remove of Autol Sub from R Humeral Shaft, Perc Endo Approach */
"0PPF4JZ"  =  "1"  /* Remove of Synth Sub from R Humeral Shaft, Perc Endo Approach */
"0PPF4KZ"  =  "1"  /* Remove Nonaut Sub from R Humeral Shaft, Perc Endo */
"0PPG04Z"  =  "1"  /* Removal of Int Fix from L Humeral Shaft, Open Approach */
"0PPG05Z"  =  "1"  /* Removal of Ext Fix from L Humeral Shaft, Open Approach */
"0PPG07Z"  =  "1"  /* Removal of Autol Sub from L Humeral Shaft, Open Approach */
"0PPG0JZ"  =  "1"  /* Removal of Synth Sub from L Humeral Shaft, Open Approach */
"0PPG0KZ"  =  "1"  /* Removal of Nonaut Sub from L Humeral Shaft, Open Approach */
"0PPG34Z"  =  "1"  /* Removal of Int Fix from L Humeral Shaft, Perc Approach */
"0PPG35Z"  =  "1"  /* Removal of Ext Fix from L Humeral Shaft, Perc Approach */
"0PPG37Z"  =  "1"  /* Removal of Autol Sub from L Humeral Shaft, Perc Approach */
"0PPG3JZ"  =  "1"  /* Removal of Synth Sub from L Humeral Shaft, Perc Approach */
"0PPG3KZ"  =  "1"  /* Removal of Nonaut Sub from L Humeral Shaft, Perc Approach */
"0PPG44Z"  =  "1"  /* Removal of Int Fix from L Humeral Shaft, Perc Endo Approach */
"0PPG45Z"  =  "1"  /* Removal of Ext Fix from L Humeral Shaft, Perc Endo Approach */
"0PPG47Z"  =  "1"  /* Remove of Autol Sub from L Humeral Shaft, Perc Endo Approach */
"0PPG4JZ"  =  "1"  /* Remove of Synth Sub from L Humeral Shaft, Perc Endo Approach */
"0PPG4KZ"  =  "1"  /* Remove Nonaut Sub from L Humeral Shaft, Perc Endo */
"0PPH04Z"  =  "1"  /* Removal of Int Fix from R Radius, Open Approach */
"0PPH05Z"  =  "1"  /* Removal of Ext Fix from R Radius, Open Approach */
"0PPH07Z"  =  "1"  /* Removal of Autol Sub from R Radius, Open Approach */
"0PPH0JZ"  =  "1"  /* Removal of Synthetic Substitute from R Radius, Open Approach */
"0PPH0KZ"  =  "1"  /* Removal of Nonaut Sub from R Radius, Open Approach */
"0PPH34Z"  =  "1"  /* Removal of Int Fix from R Radius, Perc Approach */
"0PPH35Z"  =  "1"  /* Removal of Ext Fix from R Radius, Perc Approach */
"0PPH37Z"  =  "1"  /* Removal of Autol Sub from R Radius, Perc Approach */
"0PPH3JZ"  =  "1"  /* Removal of Synthetic Substitute from R Radius, Perc Approach */
"0PPH3KZ"  =  "1"  /* Removal of Nonaut Sub from R Radius, Perc Approach */
"0PPH44Z"  =  "1"  /* Removal of Int Fix from R Radius, Perc Endo Approach */
"0PPH45Z"  =  "1"  /* Removal of Ext Fix from R Radius, Perc Endo Approach */
"0PPH47Z"  =  "1"  /* Removal of Autol Sub from R Radius, Perc Endo Approach */
"0PPH4JZ"  =  "1"  /* Removal of Synth Sub from R Radius, Perc Endo Approach */
"0PPH4KZ"  =  "1"  /* Removal of Nonaut Sub from R Radius, Perc Endo Approach */
"0PPJ04Z"  =  "1"  /* Removal of Int Fix from L Radius, Open Approach */
"0PPJ05Z"  =  "1"  /* Removal of Ext Fix from L Radius, Open Approach */
"0PPJ07Z"  =  "1"  /* Removal of Autol Sub from L Radius, Open Approach */
"0PPJ0JZ"  =  "1"  /* Removal of Synthetic Substitute from L Radius, Open Approach */
"0PPJ0KZ"  =  "1"  /* Removal of Nonaut Sub from L Radius, Open Approach */
"0PPJ34Z"  =  "1"  /* Removal of Int Fix from L Radius, Perc Approach */
"0PPJ35Z"  =  "1"  /* Removal of Ext Fix from L Radius, Perc Approach */
"0PPJ37Z"  =  "1"  /* Removal of Autol Sub from L Radius, Perc Approach */
"0PPJ3JZ"  =  "1"  /* Removal of Synthetic Substitute from L Radius, Perc Approach */
"0PPJ3KZ"  =  "1"  /* Removal of Nonaut Sub from L Radius, Perc Approach */
"0PPJ44Z"  =  "1"  /* Removal of Int Fix from L Radius, Perc Endo Approach */
"0PPJ45Z"  =  "1"  /* Removal of Ext Fix from L Radius, Perc Endo Approach */
"0PPJ47Z"  =  "1"  /* Removal of Autol Sub from L Radius, Perc Endo Approach */
"0PPJ4JZ"  =  "1"  /* Removal of Synth Sub from L Radius, Perc Endo Approach */
"0PPJ4KZ"  =  "1"  /* Removal of Nonaut Sub from L Radius, Perc Endo Approach */
"0PPK04Z"  =  "1"  /* Removal of Int Fix from R Ulna, Open Approach */
"0PPK05Z"  =  "1"  /* Removal of Ext Fix from R Ulna, Open Approach */
"0PPK07Z"  =  "1"  /* Removal of Autol Sub from R Ulna, Open Approach */
"0PPK0JZ"  =  "1"  /* Removal of Synthetic Substitute from R Ulna, Open Approach */
"0PPK0KZ"  =  "1"  /* Removal of Nonaut Sub from R Ulna, Open Approach */
"0PPK34Z"  =  "1"  /* Removal of Int Fix from R Ulna, Perc Approach */
"0PPK35Z"  =  "1"  /* Removal of Ext Fix from R Ulna, Perc Approach */
"0PPK37Z"  =  "1"  /* Removal of Autol Sub from R Ulna, Perc Approach */
"0PPK3JZ"  =  "1"  /* Removal of Synthetic Substitute from R Ulna, Perc Approach */
"0PPK3KZ"  =  "1"  /* Removal of Nonaut Sub from R Ulna, Perc Approach */
"0PPK44Z"  =  "1"  /* Removal of Int Fix from R Ulna, Perc Endo Approach */
"0PPK45Z"  =  "1"  /* Removal of Ext Fix from R Ulna, Perc Endo Approach */
"0PPK47Z"  =  "1"  /* Removal of Autol Sub from R Ulna, Perc Endo Approach */
"0PPK4JZ"  =  "1"  /* Removal of Synth Sub from R Ulna, Perc Endo Approach */
"0PPK4KZ"  =  "1"  /* Removal of Nonaut Sub from R Ulna, Perc Endo Approach */
"0PPL04Z"  =  "1"  /* Removal of Int Fix from L Ulna, Open Approach */
"0PPL05Z"  =  "1"  /* Removal of Ext Fix from L Ulna, Open Approach */
"0PPL07Z"  =  "1"  /* Removal of Autol Sub from L Ulna, Open Approach */
"0PPL0JZ"  =  "1"  /* Removal of Synthetic Substitute from L Ulna, Open Approach */
"0PPL0KZ"  =  "1"  /* Removal of Nonaut Sub from L Ulna, Open Approach */
"0PPL34Z"  =  "1"  /* Removal of Int Fix from L Ulna, Perc Approach */
"0PPL35Z"  =  "1"  /* Removal of Ext Fix from L Ulna, Perc Approach */
"0PPL37Z"  =  "1"  /* Removal of Autol Sub from L Ulna, Perc Approach */
"0PPL3JZ"  =  "1"  /* Removal of Synthetic Substitute from L Ulna, Perc Approach */
"0PPL3KZ"  =  "1"  /* Removal of Nonaut Sub from L Ulna, Perc Approach */
"0PPL44Z"  =  "1"  /* Removal of Int Fix from L Ulna, Perc Endo Approach */
"0PPL45Z"  =  "1"  /* Removal of Ext Fix from L Ulna, Perc Endo Approach */
"0PPL47Z"  =  "1"  /* Removal of Autol Sub from L Ulna, Perc Endo Approach */
"0PPL4JZ"  =  "1"  /* Removal of Synth Sub from L Ulna, Perc Endo Approach */
"0PPL4KZ"  =  "1"  /* Removal of Nonaut Sub from L Ulna, Perc Endo Approach */
"0PPM04Z"  =  "1"  /* Removal of Int Fix from R Carpal, Open Approach */
"0PPM05Z"  =  "1"  /* Removal of Ext Fix from R Carpal, Open Approach */
"0PPM07Z"  =  "1"  /* Removal of Autol Sub from R Carpal, Open Approach */
"0PPM0JZ"  =  "1"  /* Removal of Synthetic Substitute from R Carpal, Open Approach */
"0PPM0KZ"  =  "1"  /* Removal of Nonaut Sub from R Carpal, Open Approach */
"0PPM34Z"  =  "1"  /* Removal of Int Fix from R Carpal, Perc Approach */
"0PPM35Z"  =  "1"  /* Removal of Ext Fix from R Carpal, Perc Approach */
"0PPM37Z"  =  "1"  /* Removal of Autol Sub from R Carpal, Perc Approach */
"0PPM3JZ"  =  "1"  /* Removal of Synthetic Substitute from R Carpal, Perc Approach */
"0PPM3KZ"  =  "1"  /* Removal of Nonaut Sub from R Carpal, Perc Approach */
"0PPM44Z"  =  "1"  /* Removal of Int Fix from R Carpal, Perc Endo Approach */
"0PPM45Z"  =  "1"  /* Removal of Ext Fix from R Carpal, Perc Endo Approach */
"0PPM47Z"  =  "1"  /* Removal of Autol Sub from R Carpal, Perc Endo Approach */
"0PPM4JZ"  =  "1"  /* Removal of Synth Sub from R Carpal, Perc Endo Approach */
"0PPM4KZ"  =  "1"  /* Removal of Nonaut Sub from R Carpal, Perc Endo Approach */
"0PPN04Z"  =  "1"  /* Removal of Int Fix from L Carpal, Open Approach */
"0PPN05Z"  =  "1"  /* Removal of Ext Fix from L Carpal, Open Approach */
"0PPN07Z"  =  "1"  /* Removal of Autol Sub from L Carpal, Open Approach */
"0PPN0JZ"  =  "1"  /* Removal of Synthetic Substitute from L Carpal, Open Approach */
"0PPN0KZ"  =  "1"  /* Removal of Nonaut Sub from L Carpal, Open Approach */
"0PPN34Z"  =  "1"  /* Removal of Int Fix from L Carpal, Perc Approach */
"0PPN35Z"  =  "1"  /* Removal of Ext Fix from L Carpal, Perc Approach */
"0PPN37Z"  =  "1"  /* Removal of Autol Sub from L Carpal, Perc Approach */
"0PPN3JZ"  =  "1"  /* Removal of Synthetic Substitute from L Carpal, Perc Approach */
"0PPN3KZ"  =  "1"  /* Removal of Nonaut Sub from L Carpal, Perc Approach */
"0PPN44Z"  =  "1"  /* Removal of Int Fix from L Carpal, Perc Endo Approach */
"0PPN45Z"  =  "1"  /* Removal of Ext Fix from L Carpal, Perc Endo Approach */
"0PPN47Z"  =  "1"  /* Removal of Autol Sub from L Carpal, Perc Endo Approach */
"0PPN4JZ"  =  "1"  /* Removal of Synth Sub from L Carpal, Perc Endo Approach */
"0PPN4KZ"  =  "1"  /* Removal of Nonaut Sub from L Carpal, Perc Endo Approach */
"0PPP04Z"  =  "1"  /* Removal of Int Fix from R Metacarpal, Open Approach */
"0PPP05Z"  =  "1"  /* Removal of Ext Fix from R Metacarpal, Open Approach */
"0PPP07Z"  =  "1"  /* Removal of Autol Sub from R Metacarpal, Open Approach */
"0PPP0JZ"  =  "1"  /* Removal of Synth Sub from R Metacarpal, Open Approach */
"0PPP0KZ"  =  "1"  /* Removal of Nonaut Sub from R Metacarpal, Open Approach */
"0PPP34Z"  =  "1"  /* Removal of Int Fix from R Metacarpal, Perc Approach */
"0PPP35Z"  =  "1"  /* Removal of Ext Fix from R Metacarpal, Perc Approach */
"0PPP37Z"  =  "1"  /* Removal of Autol Sub from R Metacarpal, Perc Approach */
"0PPP3JZ"  =  "1"  /* Removal of Synth Sub from R Metacarpal, Perc Approach */
"0PPP3KZ"  =  "1"  /* Removal of Nonaut Sub from R Metacarpal, Perc Approach */
"0PPP44Z"  =  "1"  /* Removal of Int Fix from R Metacarpal, Perc Endo Approach */
"0PPP45Z"  =  "1"  /* Removal of Ext Fix from R Metacarpal, Perc Endo Approach */
"0PPP47Z"  =  "1"  /* Removal of Autol Sub from R Metacarpal, Perc Endo Approach */
"0PPP4JZ"  =  "1"  /* Removal of Synth Sub from R Metacarpal, Perc Endo Approach */
"0PPP4KZ"  =  "1"  /* Removal of Nonaut Sub from R Metacarpal, Perc Endo Approach */
"0PPQ04Z"  =  "1"  /* Removal of Int Fix from L Metacarpal, Open Approach */
"0PPQ05Z"  =  "1"  /* Removal of Ext Fix from L Metacarpal, Open Approach */
"0PPQ07Z"  =  "1"  /* Removal of Autol Sub from L Metacarpal, Open Approach */
"0PPQ0JZ"  =  "1"  /* Removal of Synth Sub from L Metacarpal, Open Approach */
"0PPQ0KZ"  =  "1"  /* Removal of Nonaut Sub from L Metacarpal, Open Approach */
"0PPQ34Z"  =  "1"  /* Removal of Int Fix from L Metacarpal, Perc Approach */
"0PPQ35Z"  =  "1"  /* Removal of Ext Fix from L Metacarpal, Perc Approach */
"0PPQ37Z"  =  "1"  /* Removal of Autol Sub from L Metacarpal, Perc Approach */
"0PPQ3JZ"  =  "1"  /* Removal of Synth Sub from L Metacarpal, Perc Approach */
"0PPQ3KZ"  =  "1"  /* Removal of Nonaut Sub from L Metacarpal, Perc Approach */
"0PPQ44Z"  =  "1"  /* Removal of Int Fix from L Metacarpal, Perc Endo Approach */
"0PPQ45Z"  =  "1"  /* Removal of Ext Fix from L Metacarpal, Perc Endo Approach */
"0PPQ47Z"  =  "1"  /* Removal of Autol Sub from L Metacarpal, Perc Endo Approach */
"0PPQ4JZ"  =  "1"  /* Removal of Synth Sub from L Metacarpal, Perc Endo Approach */
"0PPQ4KZ"  =  "1"  /* Removal of Nonaut Sub from L Metacarpal, Perc Endo Approach */
"0PPR04Z"  =  "1"  /* Removal of Int Fix from R Thumb Phalanx, Open Approach */
"0PPR05Z"  =  "1"  /* Removal of Ext Fix from R Thumb Phalanx, Open Approach */
"0PPR07Z"  =  "1"  /* Removal of Autol Sub from R Thumb Phalanx, Open Approach */
"0PPR0JZ"  =  "1"  /* Removal of Synth Sub from R Thumb Phalanx, Open Approach */
"0PPR0KZ"  =  "1"  /* Removal of Nonaut Sub from R Thumb Phalanx, Open Approach */
"0PPR34Z"  =  "1"  /* Removal of Int Fix from R Thumb Phalanx, Perc Approach */
"0PPR35Z"  =  "1"  /* Removal of Ext Fix from R Thumb Phalanx, Perc Approach */
"0PPR37Z"  =  "1"  /* Removal of Autol Sub from R Thumb Phalanx, Perc Approach */
"0PPR3JZ"  =  "1"  /* Removal of Synth Sub from R Thumb Phalanx, Perc Approach */
"0PPR3KZ"  =  "1"  /* Removal of Nonaut Sub from R Thumb Phalanx, Perc Approach */
"0PPR44Z"  =  "1"  /* Removal of Int Fix from R Thumb Phalanx, Perc Endo Approach */
"0PPR45Z"  =  "1"  /* Removal of Ext Fix from R Thumb Phalanx, Perc Endo Approach */
"0PPR47Z"  =  "1"  /* Remove of Autol Sub from R Thumb Phalanx, Perc Endo Approach */
"0PPR4JZ"  =  "1"  /* Remove of Synth Sub from R Thumb Phalanx, Perc Endo Approach */
"0PPR4KZ"  =  "1"  /* Remove Nonaut Sub from R Thumb Phalanx, Perc Endo */
"0PPS04Z"  =  "1"  /* Removal of Int Fix from L Thumb Phalanx, Open Approach */
"0PPS05Z"  =  "1"  /* Removal of Ext Fix from L Thumb Phalanx, Open Approach */
"0PPS07Z"  =  "1"  /* Removal of Autol Sub from L Thumb Phalanx, Open Approach */
"0PPS0JZ"  =  "1"  /* Removal of Synth Sub from L Thumb Phalanx, Open Approach */
"0PPS0KZ"  =  "1"  /* Removal of Nonaut Sub from L Thumb Phalanx, Open Approach */
"0PPS34Z"  =  "1"  /* Removal of Int Fix from L Thumb Phalanx, Perc Approach */
"0PPS35Z"  =  "1"  /* Removal of Ext Fix from L Thumb Phalanx, Perc Approach */
"0PPS37Z"  =  "1"  /* Removal of Autol Sub from L Thumb Phalanx, Perc Approach */
"0PPS3JZ"  =  "1"  /* Removal of Synth Sub from L Thumb Phalanx, Perc Approach */
"0PPS3KZ"  =  "1"  /* Removal of Nonaut Sub from L Thumb Phalanx, Perc Approach */
"0PPS44Z"  =  "1"  /* Removal of Int Fix from L Thumb Phalanx, Perc Endo Approach */
"0PPS45Z"  =  "1"  /* Removal of Ext Fix from L Thumb Phalanx, Perc Endo Approach */
"0PPS47Z"  =  "1"  /* Remove of Autol Sub from L Thumb Phalanx, Perc Endo Approach */
"0PPS4JZ"  =  "1"  /* Remove of Synth Sub from L Thumb Phalanx, Perc Endo Approach */
"0PPS4KZ"  =  "1"  /* Remove Nonaut Sub from L Thumb Phalanx, Perc Endo */
"0PPT04Z"  =  "1"  /* Removal of Int Fix from R Finger Phalanx, Open Approach */
"0PPT05Z"  =  "1"  /* Removal of Ext Fix from R Finger Phalanx, Open Approach */
"0PPT07Z"  =  "1"  /* Removal of Autol Sub from R Finger Phalanx, Open Approach */
"0PPT0JZ"  =  "1"  /* Removal of Synth Sub from R Finger Phalanx, Open Approach */
"0PPT0KZ"  =  "1"  /* Removal of Nonaut Sub from R Finger Phalanx, Open Approach */
"0PPT34Z"  =  "1"  /* Removal of Int Fix from R Finger Phalanx, Perc Approach */
"0PPT35Z"  =  "1"  /* Removal of Ext Fix from R Finger Phalanx, Perc Approach */
"0PPT37Z"  =  "1"  /* Removal of Autol Sub from R Finger Phalanx, Perc Approach */
"0PPT3JZ"  =  "1"  /* Removal of Synth Sub from R Finger Phalanx, Perc Approach */
"0PPT3KZ"  =  "1"  /* Removal of Nonaut Sub from R Finger Phalanx, Perc Approach */
"0PPT44Z"  =  "1"  /* Removal of Int Fix from R Finger Phalanx, Perc Endo Approach */
"0PPT45Z"  =  "1"  /* Removal of Ext Fix from R Finger Phalanx, Perc Endo Approach */
"0PPT47Z"  =  "1"  /* Remove Autol Sub from R Finger Phalanx, Perc Endo */
"0PPT4JZ"  =  "1"  /* Remove Synth Sub from R Finger Phalanx, Perc Endo */
"0PPT4KZ"  =  "1"  /* Remove Nonaut Sub from R Finger Phalanx, Perc Endo */
"0PPV04Z"  =  "1"  /* Removal of Int Fix from L Finger Phalanx, Open Approach */
"0PPV05Z"  =  "1"  /* Removal of Ext Fix from L Finger Phalanx, Open Approach */
"0PPV07Z"  =  "1"  /* Removal of Autol Sub from L Finger Phalanx, Open Approach */
"0PPV0JZ"  =  "1"  /* Removal of Synth Sub from L Finger Phalanx, Open Approach */
"0PPV0KZ"  =  "1"  /* Removal of Nonaut Sub from L Finger Phalanx, Open Approach */
"0PPV34Z"  =  "1"  /* Removal of Int Fix from L Finger Phalanx, Perc Approach */
"0PPV35Z"  =  "1"  /* Removal of Ext Fix from L Finger Phalanx, Perc Approach */
"0PPV37Z"  =  "1"  /* Removal of Autol Sub from L Finger Phalanx, Perc Approach */
"0PPV3JZ"  =  "1"  /* Removal of Synth Sub from L Finger Phalanx, Perc Approach */
"0PPV3KZ"  =  "1"  /* Removal of Nonaut Sub from L Finger Phalanx, Perc Approach */
"0PPV44Z"  =  "1"  /* Removal of Int Fix from L Finger Phalanx, Perc Endo Approach */
"0PPV45Z"  =  "1"  /* Removal of Ext Fix from L Finger Phalanx, Perc Endo Approach */
"0PPV47Z"  =  "1"  /* Remove Autol Sub from L Finger Phalanx, Perc Endo */
"0PPV4JZ"  =  "1"  /* Remove Synth Sub from L Finger Phalanx, Perc Endo */
"0PPV4KZ"  =  "1"  /* Remove Nonaut Sub from L Finger Phalanx, Perc Endo */
"0PPY00Z"  =  "1"  /* Removal of Drainage Device from Upper Bone, Open Approach */
"0PPY0MZ"  =  "1"  /* Removal of Bone Stim from Up Bone, Open Approach */
"0PPY30Z"  =  "1"  /* Removal of Drainage Device from Upper Bone, Perc Approach */
"0PPY3MZ"  =  "1"  /* Removal of Bone Stim from Up Bone, Perc Approach */
"0PPY40Z"  =  "1"  /* Removal of Drainage Device from Up Bone, Perc Endo Approach */
"0PPY4MZ"  =  "1"  /* Removal of Bone Stim from Up Bone, Perc Endo Approach */
"0PQ00ZZ"  =  "1"  /* Repair Sternum, Open Approach */
"0PQ03ZZ"  =  "1"  /* Repair Sternum, Percutaneous Approach */
"0PQ04ZZ"  =  "1"  /* Repair Sternum, Percutaneous Endoscopic Approach */
"0PQ0XZZ"  =  "1"  /* Repair Sternum, External Approach */
"0PQ10ZZ"  =  "1"  /* Repair Right Rib, Open Approach */
"0PQ13ZZ"  =  "1"  /* Repair Right Rib, Percutaneous Approach */
"0PQ14ZZ"  =  "1"  /* Repair Right Rib, Percutaneous Endoscopic Approach */
"0PQ1XZZ"  =  "1"  /* Repair Right Rib, External Approach */
"0PQ20ZZ"  =  "1"  /* Repair Left Rib, Open Approach */
"0PQ23ZZ"  =  "1"  /* Repair Left Rib, Percutaneous Approach */
"0PQ24ZZ"  =  "1"  /* Repair Left Rib, Percutaneous Endoscopic Approach */
"0PQ2XZZ"  =  "1"  /* Repair Left Rib, External Approach */
"0PQ30ZZ"  =  "1"  /* Repair Cervical Vertebra, Open Approach */
"0PQ33ZZ"  =  "1"  /* Repair Cervical Vertebra, Percutaneous Approach */
"0PQ34ZZ"  =  "1"  /* Repair Cervical Vertebra, Percutaneous Endoscopic Approach */
"0PQ3XZZ"  =  "1"  /* Repair Cervical Vertebra, External Approach */
"0PQ40ZZ"  =  "1"  /* Repair Thoracic Vertebra, Open Approach */
"0PQ43ZZ"  =  "1"  /* Repair Thoracic Vertebra, Percutaneous Approach */
"0PQ44ZZ"  =  "1"  /* Repair Thoracic Vertebra, Percutaneous Endoscopic Approach */
"0PQ4XZZ"  =  "1"  /* Repair Thoracic Vertebra, External Approach */
"0PQ50ZZ"  =  "1"  /* Repair Right Scapula, Open Approach */
"0PQ53ZZ"  =  "1"  /* Repair Right Scapula, Percutaneous Approach */
"0PQ54ZZ"  =  "1"  /* Repair Right Scapula, Percutaneous Endoscopic Approach */
"0PQ5XZZ"  =  "1"  /* Repair Right Scapula, External Approach */
"0PQ60ZZ"  =  "1"  /* Repair Left Scapula, Open Approach */
"0PQ63ZZ"  =  "1"  /* Repair Left Scapula, Percutaneous Approach */
"0PQ64ZZ"  =  "1"  /* Repair Left Scapula, Percutaneous Endoscopic Approach */
"0PQ6XZZ"  =  "1"  /* Repair Left Scapula, External Approach */
"0PQ70ZZ"  =  "1"  /* Repair Right Glenoid Cavity, Open Approach */
"0PQ73ZZ"  =  "1"  /* Repair Right Glenoid Cavity, Percutaneous Approach */
"0PQ74ZZ"  =  "1"  /* Repair Right Glenoid Cavity, Perc Endo Approach */
"0PQ7XZZ"  =  "1"  /* Repair Right Glenoid Cavity, External Approach */
"0PQ80ZZ"  =  "1"  /* Repair Left Glenoid Cavity, Open Approach */
"0PQ83ZZ"  =  "1"  /* Repair Left Glenoid Cavity, Percutaneous Approach */
"0PQ84ZZ"  =  "1"  /* Repair Left Glenoid Cavity, Percutaneous Endoscopic Approach */
"0PQ8XZZ"  =  "1"  /* Repair Left Glenoid Cavity, External Approach */
"0PQ90ZZ"  =  "1"  /* Repair Right Clavicle, Open Approach */
"0PQ93ZZ"  =  "1"  /* Repair Right Clavicle, Percutaneous Approach */
"0PQ94ZZ"  =  "1"  /* Repair Right Clavicle, Percutaneous Endoscopic Approach */
"0PQ9XZZ"  =  "1"  /* Repair Right Clavicle, External Approach */
"0PQB0ZZ"  =  "1"  /* Repair Left Clavicle, Open Approach */
"0PQB3ZZ"  =  "1"  /* Repair Left Clavicle, Percutaneous Approach */
"0PQB4ZZ"  =  "1"  /* Repair Left Clavicle, Percutaneous Endoscopic Approach */
"0PQBXZZ"  =  "1"  /* Repair Left Clavicle, External Approach */
"0PQC0ZZ"  =  "1"  /* Repair Right Humeral Head, Open Approach */
"0PQC3ZZ"  =  "1"  /* Repair Right Humeral Head, Percutaneous Approach */
"0PQC4ZZ"  =  "1"  /* Repair Right Humeral Head, Percutaneous Endoscopic Approach */
"0PQCXZZ"  =  "1"  /* Repair Right Humeral Head, External Approach */
"0PQD0ZZ"  =  "1"  /* Repair Left Humeral Head, Open Approach */
"0PQD3ZZ"  =  "1"  /* Repair Left Humeral Head, Percutaneous Approach */
"0PQD4ZZ"  =  "1"  /* Repair Left Humeral Head, Percutaneous Endoscopic Approach */
"0PQDXZZ"  =  "1"  /* Repair Left Humeral Head, External Approach */
"0PQF0ZZ"  =  "1"  /* Repair Right Humeral Shaft, Open Approach */
"0PQF3ZZ"  =  "1"  /* Repair Right Humeral Shaft, Percutaneous Approach */
"0PQF4ZZ"  =  "1"  /* Repair Right Humeral Shaft, Percutaneous Endoscopic Approach */
"0PQFXZZ"  =  "1"  /* Repair Right Humeral Shaft, External Approach */
"0PQG0ZZ"  =  "1"  /* Repair Left Humeral Shaft, Open Approach */
"0PQG3ZZ"  =  "1"  /* Repair Left Humeral Shaft, Percutaneous Approach */
"0PQG4ZZ"  =  "1"  /* Repair Left Humeral Shaft, Percutaneous Endoscopic Approach */
"0PQGXZZ"  =  "1"  /* Repair Left Humeral Shaft, External Approach */
"0PQH0ZZ"  =  "1"  /* Repair Right Radius, Open Approach */
"0PQH3ZZ"  =  "1"  /* Repair Right Radius, Percutaneous Approach */
"0PQH4ZZ"  =  "1"  /* Repair Right Radius, Percutaneous Endoscopic Approach */
"0PQHXZZ"  =  "1"  /* Repair Right Radius, External Approach */
"0PQJ0ZZ"  =  "1"  /* Repair Left Radius, Open Approach */
"0PQJ3ZZ"  =  "1"  /* Repair Left Radius, Percutaneous Approach */
"0PQJ4ZZ"  =  "1"  /* Repair Left Radius, Percutaneous Endoscopic Approach */
"0PQJXZZ"  =  "1"  /* Repair Left Radius, External Approach */
"0PQK0ZZ"  =  "1"  /* Repair Right Ulna, Open Approach */
"0PQK3ZZ"  =  "1"  /* Repair Right Ulna, Percutaneous Approach */
"0PQK4ZZ"  =  "1"  /* Repair Right Ulna, Percutaneous Endoscopic Approach */
"0PQKXZZ"  =  "1"  /* Repair Right Ulna, External Approach */
"0PQL0ZZ"  =  "1"  /* Repair Left Ulna, Open Approach */
"0PQL3ZZ"  =  "1"  /* Repair Left Ulna, Percutaneous Approach */
"0PQL4ZZ"  =  "1"  /* Repair Left Ulna, Percutaneous Endoscopic Approach */
"0PQLXZZ"  =  "1"  /* Repair Left Ulna, External Approach */
"0PQM0ZZ"  =  "1"  /* Repair Right Carpal, Open Approach */
"0PQM3ZZ"  =  "1"  /* Repair Right Carpal, Percutaneous Approach */
"0PQM4ZZ"  =  "1"  /* Repair Right Carpal, Percutaneous Endoscopic Approach */
"0PQMXZZ"  =  "1"  /* Repair Right Carpal, External Approach */
"0PQN0ZZ"  =  "1"  /* Repair Left Carpal, Open Approach */
"0PQN3ZZ"  =  "1"  /* Repair Left Carpal, Percutaneous Approach */
"0PQN4ZZ"  =  "1"  /* Repair Left Carpal, Percutaneous Endoscopic Approach */
"0PQNXZZ"  =  "1"  /* Repair Left Carpal, External Approach */
"0PQP0ZZ"  =  "1"  /* Repair Right Metacarpal, Open Approach */
"0PQP3ZZ"  =  "1"  /* Repair Right Metacarpal, Percutaneous Approach */
"0PQP4ZZ"  =  "1"  /* Repair Right Metacarpal, Percutaneous Endoscopic Approach */
"0PQPXZZ"  =  "1"  /* Repair Right Metacarpal, External Approach */
"0PQQ0ZZ"  =  "1"  /* Repair Left Metacarpal, Open Approach */
"0PQQ3ZZ"  =  "1"  /* Repair Left Metacarpal, Percutaneous Approach */
"0PQQ4ZZ"  =  "1"  /* Repair Left Metacarpal, Percutaneous Endoscopic Approach */
"0PQQXZZ"  =  "1"  /* Repair Left Metacarpal, External Approach */
"0PQR0ZZ"  =  "1"  /* Repair Right Thumb Phalanx, Open Approach */
"0PQR3ZZ"  =  "1"  /* Repair Right Thumb Phalanx, Percutaneous Approach */
"0PQR4ZZ"  =  "1"  /* Repair Right Thumb Phalanx, Percutaneous Endoscopic Approach */
"0PQRXZZ"  =  "1"  /* Repair Right Thumb Phalanx, External Approach */
"0PQS0ZZ"  =  "1"  /* Repair Left Thumb Phalanx, Open Approach */
"0PQS3ZZ"  =  "1"  /* Repair Left Thumb Phalanx, Percutaneous Approach */
"0PQS4ZZ"  =  "1"  /* Repair Left Thumb Phalanx, Percutaneous Endoscopic Approach */
"0PQSXZZ"  =  "1"  /* Repair Left Thumb Phalanx, External Approach */
"0PQT0ZZ"  =  "1"  /* Repair Right Finger Phalanx, Open Approach */
"0PQT3ZZ"  =  "1"  /* Repair Right Finger Phalanx, Percutaneous Approach */
"0PQT4ZZ"  =  "1"  /* Repair Right Finger Phalanx, Perc Endo Approach */
"0PQTXZZ"  =  "1"  /* Repair Right Finger Phalanx, External Approach */
"0PQV0ZZ"  =  "1"  /* Repair Left Finger Phalanx, Open Approach */
"0PQV3ZZ"  =  "1"  /* Repair Left Finger Phalanx, Percutaneous Approach */
"0PQV4ZZ"  =  "1"  /* Repair Left Finger Phalanx, Percutaneous Endoscopic Approach */
"0PQVXZZ"  =  "1"  /* Repair Left Finger Phalanx, External Approach */
"0PR007Z"  =  "1"  /* Replacement of Sternum with Autol Sub, Open Approach */
"0PR00JZ"  =  "1"  /* Replacement of Sternum with Synth Sub, Open Approach */
"0PR00KZ"  =  "1"  /* Replacement of Sternum with Nonaut Sub, Open Approach */
"0PR037Z"  =  "1"  /* Replacement of Sternum with Autol Sub, Perc Approach */
"0PR03JZ"  =  "1"  /* Replacement of Sternum with Synth Sub, Perc Approach */
"0PR03KZ"  =  "1"  /* Replacement of Sternum with Nonaut Sub, Perc Approach */
"0PR047Z"  =  "1"  /* Replacement of Sternum with Autol Sub, Perc Endo Approach */
"0PR04JZ"  =  "1"  /* Replacement of Sternum with Synth Sub, Perc Endo Approach */
"0PR04KZ"  =  "1"  /* Replacement of Sternum with Nonaut Sub, Perc Endo Approach */
"0PR107Z"  =  "1"  /* Replacement of Right Rib with Autol Sub, Open Approach */
"0PR10JZ"  =  "1"  /* Replacement of Right Rib with Synth Sub, Open Approach */
"0PR10KZ"  =  "1"  /* Replacement of Right Rib with Nonaut Sub, Open Approach */
"0PR137Z"  =  "1"  /* Replacement of Right Rib with Autol Sub, Perc Approach */
"0PR13JZ"  =  "1"  /* Replacement of Right Rib with Synth Sub, Perc Approach */
"0PR13KZ"  =  "1"  /* Replacement of Right Rib with Nonaut Sub, Perc Approach */
"0PR147Z"  =  "1"  /* Replacement of Right Rib with Autol Sub, Perc Endo Approach */
"0PR14JZ"  =  "1"  /* Replacement of Right Rib with Synth Sub, Perc Endo Approach */
"0PR14KZ"  =  "1"  /* Replacement of Right Rib with Nonaut Sub, Perc Endo Approach */
"0PR207Z"  =  "1"  /* Replacement of Left Rib with Autol Sub, Open Approach */
"0PR20JZ"  =  "1"  /* Replacement of Left Rib with Synth Sub, Open Approach */
"0PR20KZ"  =  "1"  /* Replacement of Left Rib with Nonaut Sub, Open Approach */
"0PR237Z"  =  "1"  /* Replacement of Left Rib with Autol Sub, Perc Approach */
"0PR23JZ"  =  "1"  /* Replacement of Left Rib with Synth Sub, Perc Approach */
"0PR23KZ"  =  "1"  /* Replacement of Left Rib with Nonaut Sub, Perc Approach */
"0PR247Z"  =  "1"  /* Replacement of Left Rib with Autol Sub, Perc Endo Approach */
"0PR24JZ"  =  "1"  /* Replacement of Left Rib with Synth Sub, Perc Endo Approach */
"0PR24KZ"  =  "1"  /* Replacement of Left Rib with Nonaut Sub, Perc Endo Approach */
"0PR307Z"  =  "1"  /* Replace of Cervcal Vertebra with Autol Sub, Open Approach */
"0PR30JZ"  =  "1"  /* Replace of Cervcal Vertebra with Synth Sub, Open Approach */
"0PR30KZ"  =  "1"  /* Replace of Cervcal Vertebra with Nonaut Sub, Open Approach */
"0PR337Z"  =  "1"  /* Replace of Cervcal Vertebra with Autol Sub, Perc Approach */
"0PR33JZ"  =  "1"  /* Replace of Cervcal Vertebra with Synth Sub, Perc Approach */
"0PR33KZ"  =  "1"  /* Replace of Cervcal Vertebra with Nonaut Sub, Perc Approach */
"0PR347Z"  =  "1"  /* Replace Cervcal Vertebra w Autol Sub, Perc Endo */
"0PR34JZ"  =  "1"  /* Replace Cervcal Vertebra w Synth Sub, Perc Endo */
"0PR34KZ"  =  "1"  /* Replace Cervcal Vertebra w Nonaut Sub, Perc Endo */
"0PR407Z"  =  "1"  /* Replacement of Thor Vertebra with Autol Sub, Open Approach */
"0PR40JZ"  =  "1"  /* Replacement of Thor Vertebra with Synth Sub, Open Approach */
"0PR40KZ"  =  "1"  /* Replacement of Thor Vertebra with Nonaut Sub, Open Approach */
"0PR437Z"  =  "1"  /* Replacement of Thor Vertebra with Autol Sub, Perc Approach */
"0PR43JZ"  =  "1"  /* Replacement of Thor Vertebra with Synth Sub, Perc Approach */
"0PR43KZ"  =  "1"  /* Replacement of Thor Vertebra with Nonaut Sub, Perc Approach */
"0PR447Z"  =  "1"  /* Replace of Thor Vertebra with Autol Sub, Perc Endo Approach */
"0PR44JZ"  =  "1"  /* Replace of Thor Vertebra with Synth Sub, Perc Endo Approach */
"0PR44KZ"  =  "1"  /* Replace of Thor Vertebra with Nonaut Sub, Perc Endo Approach */
"0PR507Z"  =  "1"  /* Replacement of Right Scapula with Autol Sub, Open Approach */
"0PR50JZ"  =  "1"  /* Replacement of Right Scapula with Synth Sub, Open Approach */
"0PR50KZ"  =  "1"  /* Replacement of Right Scapula with Nonaut Sub, Open Approach */
"0PR537Z"  =  "1"  /* Replacement of Right Scapula with Autol Sub, Perc Approach */
"0PR53JZ"  =  "1"  /* Replacement of Right Scapula with Synth Sub, Perc Approach */
"0PR53KZ"  =  "1"  /* Replacement of Right Scapula with Nonaut Sub, Perc Approach */
"0PR547Z"  =  "1"  /* Replacement of R Scapula with Autol Sub, Perc Endo Approach */
"0PR54JZ"  =  "1"  /* Replacement of R Scapula with Synth Sub, Perc Endo Approach */
"0PR54KZ"  =  "1"  /* Replacement of R Scapula with Nonaut Sub, Perc Endo Approach */
"0PR607Z"  =  "1"  /* Replacement of Left Scapula with Autol Sub, Open Approach */
"0PR60JZ"  =  "1"  /* Replacement of Left Scapula with Synth Sub, Open Approach */
"0PR60KZ"  =  "1"  /* Replacement of Left Scapula with Nonaut Sub, Open Approach */
"0PR637Z"  =  "1"  /* Replacement of Left Scapula with Autol Sub, Perc Approach */
"0PR63JZ"  =  "1"  /* Replacement of Left Scapula with Synth Sub, Perc Approach */
"0PR63KZ"  =  "1"  /* Replacement of Left Scapula with Nonaut Sub, Perc Approach */
"0PR647Z"  =  "1"  /* Replacement of L Scapula with Autol Sub, Perc Endo Approach */
"0PR64JZ"  =  "1"  /* Replacement of L Scapula with Synth Sub, Perc Endo Approach */
"0PR64KZ"  =  "1"  /* Replacement of L Scapula with Nonaut Sub, Perc Endo Approach */
"0PR707Z"  =  "1"  /* Replacement of R Glenoid Cav with Autol Sub, Open Approach */
"0PR70JZ"  =  "1"  /* Replacement of R Glenoid Cav with Synth Sub, Open Approach */
"0PR70KZ"  =  "1"  /* Replacement of R Glenoid Cav with Nonaut Sub, Open Approach */
"0PR737Z"  =  "1"  /* Replacement of R Glenoid Cav with Autol Sub, Perc Approach */
"0PR73JZ"  =  "1"  /* Replacement of R Glenoid Cav with Synth Sub, Perc Approach */
"0PR73KZ"  =  "1"  /* Replacement of R Glenoid Cav with Nonaut Sub, Perc Approach */
"0PR747Z"  =  "1"  /* Replace of R Glenoid Cav with Autol Sub, Perc Endo Approach */
"0PR74JZ"  =  "1"  /* Replace of R Glenoid Cav with Synth Sub, Perc Endo Approach */
"0PR74KZ"  =  "1"  /* Replace of R Glenoid Cav with Nonaut Sub, Perc Endo Approach */
"0PR807Z"  =  "1"  /* Replacement of L Glenoid Cav with Autol Sub, Open Approach */
"0PR80JZ"  =  "1"  /* Replacement of L Glenoid Cav with Synth Sub, Open Approach */
"0PR80KZ"  =  "1"  /* Replacement of L Glenoid Cav with Nonaut Sub, Open Approach */
"0PR837Z"  =  "1"  /* Replacement of L Glenoid Cav with Autol Sub, Perc Approach */
"0PR83JZ"  =  "1"  /* Replacement of L Glenoid Cav with Synth Sub, Perc Approach */
"0PR83KZ"  =  "1"  /* Replacement of L Glenoid Cav with Nonaut Sub, Perc Approach */
"0PR847Z"  =  "1"  /* Replace of L Glenoid Cav with Autol Sub, Perc Endo Approach */
"0PR84JZ"  =  "1"  /* Replace of L Glenoid Cav with Synth Sub, Perc Endo Approach */
"0PR84KZ"  =  "1"  /* Replace of L Glenoid Cav with Nonaut Sub, Perc Endo Approach */
"0PR907Z"  =  "1"  /* Replacement of Right Clavicle with Autol Sub, Open Approach */
"0PR90JZ"  =  "1"  /* Replacement of Right Clavicle with Synth Sub, Open Approach */
"0PR90KZ"  =  "1"  /* Replacement of Right Clavicle with Nonaut Sub, Open Approach */
"0PR937Z"  =  "1"  /* Replacement of Right Clavicle with Autol Sub, Perc Approach */
"0PR93JZ"  =  "1"  /* Replacement of Right Clavicle with Synth Sub, Perc Approach */
"0PR93KZ"  =  "1"  /* Replacement of Right Clavicle with Nonaut Sub, Perc Approach */
"0PR947Z"  =  "1"  /* Replacement of R Clavicle with Autol Sub, Perc Endo Approach */
"0PR94JZ"  =  "1"  /* Replacement of R Clavicle with Synth Sub, Perc Endo Approach */
"0PR94KZ"  =  "1"  /* Replace of R Clavicle with Nonaut Sub, Perc Endo Approach */
"0PRB07Z"  =  "1"  /* Replacement of Left Clavicle with Autol Sub, Open Approach */
"0PRB0JZ"  =  "1"  /* Replacement of Left Clavicle with Synth Sub, Open Approach */
"0PRB0KZ"  =  "1"  /* Replacement of Left Clavicle with Nonaut Sub, Open Approach */
"0PRB37Z"  =  "1"  /* Replacement of Left Clavicle with Autol Sub, Perc Approach */
"0PRB3JZ"  =  "1"  /* Replacement of Left Clavicle with Synth Sub, Perc Approach */
"0PRB3KZ"  =  "1"  /* Replacement of Left Clavicle with Nonaut Sub, Perc Approach */
"0PRB47Z"  =  "1"  /* Replacement of L Clavicle with Autol Sub, Perc Endo Approach */
"0PRB4JZ"  =  "1"  /* Replacement of L Clavicle with Synth Sub, Perc Endo Approach */
"0PRB4KZ"  =  "1"  /* Replace of L Clavicle with Nonaut Sub, Perc Endo Approach */
"0PRC07Z"  =  "1"  /* Replacement of R Humeral Head with Autol Sub, Open Approach */
"0PRC0KZ"  =  "1"  /* Replacement of R Humeral Head with Nonaut Sub, Open Approach */
"0PRC37Z"  =  "1"  /* Replacement of R Humeral Head with Autol Sub, Perc Approach */
"0PRC3JZ"  =  "1"  /* Replacement of R Humeral Head with Synth Sub, Perc Approach */
"0PRC3KZ"  =  "1"  /* Replacement of R Humeral Head with Nonaut Sub, Perc Approach */
"0PRC47Z"  =  "1"  /* Replace of R Humeral Head with Autol Sub, Perc Endo Approach */
"0PRC4JZ"  =  "1"  /* Replace of R Humeral Head with Synth Sub, Perc Endo Approach */
"0PRC4KZ"  =  "1"  /* Replace R Humeral Head w Nonaut Sub, Perc Endo */
"0PRD07Z"  =  "1"  /* Replacement of L Humeral Head with Autol Sub, Open Approach */
"0PRD0KZ"  =  "1"  /* Replacement of L Humeral Head with Nonaut Sub, Open Approach */
"0PRD37Z"  =  "1"  /* Replacement of L Humeral Head with Autol Sub, Perc Approach */
"0PRD3JZ"  =  "1"  /* Replacement of L Humeral Head with Synth Sub, Perc Approach */
"0PRD3KZ"  =  "1"  /* Replacement of L Humeral Head with Nonaut Sub, Perc Approach */
"0PRD47Z"  =  "1"  /* Replace of L Humeral Head with Autol Sub, Perc Endo Approach */
"0PRD4JZ"  =  "1"  /* Replace of L Humeral Head with Synth Sub, Perc Endo Approach */
"0PRD4KZ"  =  "1"  /* Replace L Humeral Head w Nonaut Sub, Perc Endo */
"0PRF07Z"  =  "1"  /* Replacement of R Humeral Shaft with Autol Sub, Open Approach */
"0PRF0JZ"  =  "1"  /* Replacement of R Humeral Shaft with Synth Sub, Open Approach */
"0PRF0KZ"  =  "1"  /* Replace of R Humeral Shaft with Nonaut Sub, Open Approach */
"0PRF37Z"  =  "1"  /* Replacement of R Humeral Shaft with Autol Sub, Perc Approach */
"0PRF3JZ"  =  "1"  /* Replacement of R Humeral Shaft with Synth Sub, Perc Approach */
"0PRF3KZ"  =  "1"  /* Replace of R Humeral Shaft with Nonaut Sub, Perc Approach */
"0PRF47Z"  =  "1"  /* Replace R Humeral Shaft w Autol Sub, Perc Endo */
"0PRF4JZ"  =  "1"  /* Replace R Humeral Shaft w Synth Sub, Perc Endo */
"0PRF4KZ"  =  "1"  /* Replace R Humeral Shaft w Nonaut Sub, Perc Endo */
"0PRG07Z"  =  "1"  /* Replacement of L Humeral Shaft with Autol Sub, Open Approach */
"0PRG0JZ"  =  "1"  /* Replacement of L Humeral Shaft with Synth Sub, Open Approach */
"0PRG0KZ"  =  "1"  /* Replace of L Humeral Shaft with Nonaut Sub, Open Approach */
"0PRG37Z"  =  "1"  /* Replacement of L Humeral Shaft with Autol Sub, Perc Approach */
"0PRG3JZ"  =  "1"  /* Replacement of L Humeral Shaft with Synth Sub, Perc Approach */
"0PRG3KZ"  =  "1"  /* Replace of L Humeral Shaft with Nonaut Sub, Perc Approach */
"0PRG47Z"  =  "1"  /* Replace L Humeral Shaft w Autol Sub, Perc Endo */
"0PRG4JZ"  =  "1"  /* Replace L Humeral Shaft w Synth Sub, Perc Endo */
"0PRG4KZ"  =  "1"  /* Replace L Humeral Shaft w Nonaut Sub, Perc Endo */
"0PRH07Z"  =  "1"  /* Replacement of Right Radius with Autol Sub, Open Approach */
"0PRH0JZ"  =  "1"  /* Replacement of Right Radius with Synth Sub, Open Approach */
"0PRH0KZ"  =  "1"  /* Replacement of Right Radius with Nonaut Sub, Open Approach */
"0PRH37Z"  =  "1"  /* Replacement of Right Radius with Autol Sub, Perc Approach */
"0PRH3JZ"  =  "1"  /* Replacement of Right Radius with Synth Sub, Perc Approach */
"0PRH3KZ"  =  "1"  /* Replacement of Right Radius with Nonaut Sub, Perc Approach */
"0PRH47Z"  =  "1"  /* Replacement of R Radius with Autol Sub, Perc Endo Approach */
"0PRH4JZ"  =  "1"  /* Replacement of R Radius with Synth Sub, Perc Endo Approach */
"0PRH4KZ"  =  "1"  /* Replacement of R Radius with Nonaut Sub, Perc Endo Approach */
"0PRJ07Z"  =  "1"  /* Replacement of Left Radius with Autol Sub, Open Approach */
"0PRJ0JZ"  =  "1"  /* Replacement of Left Radius with Synth Sub, Open Approach */
"0PRJ0KZ"  =  "1"  /* Replacement of Left Radius with Nonaut Sub, Open Approach */
"0PRJ37Z"  =  "1"  /* Replacement of Left Radius with Autol Sub, Perc Approach */
"0PRJ3JZ"  =  "1"  /* Replacement of Left Radius with Synth Sub, Perc Approach */
"0PRJ3KZ"  =  "1"  /* Replacement of Left Radius with Nonaut Sub, Perc Approach */
"0PRJ47Z"  =  "1"  /* Replacement of L Radius with Autol Sub, Perc Endo Approach */
"0PRJ4JZ"  =  "1"  /* Replacement of L Radius with Synth Sub, Perc Endo Approach */
"0PRJ4KZ"  =  "1"  /* Replacement of L Radius with Nonaut Sub, Perc Endo Approach */
"0PRK07Z"  =  "1"  /* Replacement of Right Ulna with Autol Sub, Open Approach */
"0PRK0JZ"  =  "1"  /* Replacement of Right Ulna with Synth Sub, Open Approach */
"0PRK0KZ"  =  "1"  /* Replacement of Right Ulna with Nonaut Sub, Open Approach */
"0PRK37Z"  =  "1"  /* Replacement of Right Ulna with Autol Sub, Perc Approach */
"0PRK3JZ"  =  "1"  /* Replacement of Right Ulna with Synth Sub, Perc Approach */
"0PRK3KZ"  =  "1"  /* Replacement of Right Ulna with Nonaut Sub, Perc Approach */
"0PRK47Z"  =  "1"  /* Replacement of Right Ulna with Autol Sub, Perc Endo Approach */
"0PRK4JZ"  =  "1"  /* Replacement of Right Ulna with Synth Sub, Perc Endo Approach */
"0PRK4KZ"  =  "1"  /* Replacement of R Ulna with Nonaut Sub, Perc Endo Approach */
"0PRL07Z"  =  "1"  /* Replacement of Left Ulna with Autol Sub, Open Approach */
"0PRL0JZ"  =  "1"  /* Replacement of Left Ulna with Synth Sub, Open Approach */
"0PRL0KZ"  =  "1"  /* Replacement of Left Ulna with Nonaut Sub, Open Approach */
"0PRL37Z"  =  "1"  /* Replacement of Left Ulna with Autol Sub, Perc Approach */
"0PRL3JZ"  =  "1"  /* Replacement of Left Ulna with Synth Sub, Perc Approach */
"0PRL3KZ"  =  "1"  /* Replacement of Left Ulna with Nonaut Sub, Perc Approach */
"0PRL47Z"  =  "1"  /* Replacement of Left Ulna with Autol Sub, Perc Endo Approach */
"0PRL4JZ"  =  "1"  /* Replacement of Left Ulna with Synth Sub, Perc Endo Approach */
"0PRL4KZ"  =  "1"  /* Replacement of Left Ulna with Nonaut Sub, Perc Endo Approach */
"0PRM07Z"  =  "1"  /* Replacement of Right Carpal with Autol Sub, Open Approach */
"0PRM0JZ"  =  "1"  /* Replacement of Right Carpal with Synth Sub, Open Approach */
"0PRM0KZ"  =  "1"  /* Replacement of Right Carpal with Nonaut Sub, Open Approach */
"0PRM37Z"  =  "1"  /* Replacement of Right Carpal with Autol Sub, Perc Approach */
"0PRM3JZ"  =  "1"  /* Replacement of Right Carpal with Synth Sub, Perc Approach */
"0PRM3KZ"  =  "1"  /* Replacement of Right Carpal with Nonaut Sub, Perc Approach */
"0PRM47Z"  =  "1"  /* Replacement of R Carpal with Autol Sub, Perc Endo Approach */
"0PRM4JZ"  =  "1"  /* Replacement of R Carpal with Synth Sub, Perc Endo Approach */
"0PRM4KZ"  =  "1"  /* Replacement of R Carpal with Nonaut Sub, Perc Endo Approach */
"0PRN07Z"  =  "1"  /* Replacement of Left Carpal with Autol Sub, Open Approach */
"0PRN0JZ"  =  "1"  /* Replacement of Left Carpal with Synth Sub, Open Approach */
"0PRN0KZ"  =  "1"  /* Replacement of Left Carpal with Nonaut Sub, Open Approach */
"0PRN37Z"  =  "1"  /* Replacement of Left Carpal with Autol Sub, Perc Approach */
"0PRN3JZ"  =  "1"  /* Replacement of Left Carpal with Synth Sub, Perc Approach */
"0PRN3KZ"  =  "1"  /* Replacement of Left Carpal with Nonaut Sub, Perc Approach */
"0PRN47Z"  =  "1"  /* Replacement of L Carpal with Autol Sub, Perc Endo Approach */
"0PRN4JZ"  =  "1"  /* Replacement of L Carpal with Synth Sub, Perc Endo Approach */
"0PRN4KZ"  =  "1"  /* Replacement of L Carpal with Nonaut Sub, Perc Endo Approach */
"0PRP07Z"  =  "1"  /* Replacement of R Metacarpal with Autol Sub, Open Approach */
"0PRP0JZ"  =  "1"  /* Replacement of R Metacarpal with Synth Sub, Open Approach */
"0PRP0KZ"  =  "1"  /* Replacement of R Metacarpal with Nonaut Sub, Open Approach */
"0PRP37Z"  =  "1"  /* Replacement of R Metacarpal with Autol Sub, Perc Approach */
"0PRP3JZ"  =  "1"  /* Replacement of R Metacarpal with Synth Sub, Perc Approach */
"0PRP3KZ"  =  "1"  /* Replacement of R Metacarpal with Nonaut Sub, Perc Approach */
"0PRP47Z"  =  "1"  /* Replace of R Metacarpal with Autol Sub, Perc Endo Approach */
"0PRP4JZ"  =  "1"  /* Replace of R Metacarpal with Synth Sub, Perc Endo Approach */
"0PRP4KZ"  =  "1"  /* Replace of R Metacarpal with Nonaut Sub, Perc Endo Approach */
"0PRQ07Z"  =  "1"  /* Replacement of Left Metacarpal with Autol Sub, Open Approach */
"0PRQ0JZ"  =  "1"  /* Replacement of Left Metacarpal with Synth Sub, Open Approach */
"0PRQ0KZ"  =  "1"  /* Replacement of L Metacarpal with Nonaut Sub, Open Approach */
"0PRQ37Z"  =  "1"  /* Replacement of Left Metacarpal with Autol Sub, Perc Approach */
"0PRQ3JZ"  =  "1"  /* Replacement of Left Metacarpal with Synth Sub, Perc Approach */
"0PRQ3KZ"  =  "1"  /* Replacement of L Metacarpal with Nonaut Sub, Perc Approach */
"0PRQ47Z"  =  "1"  /* Replace of L Metacarpal with Autol Sub, Perc Endo Approach */
"0PRQ4JZ"  =  "1"  /* Replace of L Metacarpal with Synth Sub, Perc Endo Approach */
"0PRQ4KZ"  =  "1"  /* Replace of L Metacarpal with Nonaut Sub, Perc Endo Approach */
"0PRR07Z"  =  "1"  /* Replacement of R Thumb Phalanx with Autol Sub, Open Approach */
"0PRR0JZ"  =  "1"  /* Replacement of R Thumb Phalanx with Synth Sub, Open Approach */
"0PRR0KZ"  =  "1"  /* Replace of R Thumb Phalanx with Nonaut Sub, Open Approach */
"0PRR37Z"  =  "1"  /* Replacement of R Thumb Phalanx with Autol Sub, Perc Approach */
"0PRR3JZ"  =  "1"  /* Replacement of R Thumb Phalanx with Synth Sub, Perc Approach */
"0PRR3KZ"  =  "1"  /* Replace of R Thumb Phalanx with Nonaut Sub, Perc Approach */
"0PRR47Z"  =  "1"  /* Replace R Thumb Phalanx w Autol Sub, Perc Endo */
"0PRR4JZ"  =  "1"  /* Replace R Thumb Phalanx w Synth Sub, Perc Endo */
"0PRR4KZ"  =  "1"  /* Replace R Thumb Phalanx w Nonaut Sub, Perc Endo */
"0PRS07Z"  =  "1"  /* Replacement of L Thumb Phalanx with Autol Sub, Open Approach */
"0PRS0JZ"  =  "1"  /* Replacement of L Thumb Phalanx with Synth Sub, Open Approach */
"0PRS0KZ"  =  "1"  /* Replace of L Thumb Phalanx with Nonaut Sub, Open Approach */
"0PRS37Z"  =  "1"  /* Replacement of L Thumb Phalanx with Autol Sub, Perc Approach */
"0PRS3JZ"  =  "1"  /* Replacement of L Thumb Phalanx with Synth Sub, Perc Approach */
"0PRS3KZ"  =  "1"  /* Replace of L Thumb Phalanx with Nonaut Sub, Perc Approach */
"0PRS47Z"  =  "1"  /* Replace L Thumb Phalanx w Autol Sub, Perc Endo */
"0PRS4JZ"  =  "1"  /* Replace L Thumb Phalanx w Synth Sub, Perc Endo */
"0PRS4KZ"  =  "1"  /* Replace L Thumb Phalanx w Nonaut Sub, Perc Endo */
"0PRT07Z"  =  "1"  /* Replace of R Finger Phalanx with Autol Sub, Open Approach */
"0PRT0JZ"  =  "1"  /* Replace of R Finger Phalanx with Synth Sub, Open Approach */
"0PRT0KZ"  =  "1"  /* Replace of R Finger Phalanx with Nonaut Sub, Open Approach */
"0PRT37Z"  =  "1"  /* Replace of R Finger Phalanx with Autol Sub, Perc Approach */
"0PRT3JZ"  =  "1"  /* Replace of R Finger Phalanx with Synth Sub, Perc Approach */
"0PRT3KZ"  =  "1"  /* Replace of R Finger Phalanx with Nonaut Sub, Perc Approach */
"0PRT47Z"  =  "1"  /* Replace R Finger Phalanx w Autol Sub, Perc Endo */
"0PRT4JZ"  =  "1"  /* Replace R Finger Phalanx w Synth Sub, Perc Endo */
"0PRT4KZ"  =  "1"  /* Replace R Finger Phalanx w Nonaut Sub, Perc Endo */
"0PRV07Z"  =  "1"  /* Replace of L Finger Phalanx with Autol Sub, Open Approach */
"0PRV0JZ"  =  "1"  /* Replace of L Finger Phalanx with Synth Sub, Open Approach */
"0PRV0KZ"  =  "1"  /* Replace of L Finger Phalanx with Nonaut Sub, Open Approach */
"0PRV37Z"  =  "1"  /* Replace of L Finger Phalanx with Autol Sub, Perc Approach */
"0PRV3JZ"  =  "1"  /* Replace of L Finger Phalanx with Synth Sub, Perc Approach */
"0PRV3KZ"  =  "1"  /* Replace of L Finger Phalanx with Nonaut Sub, Perc Approach */
"0PRV47Z"  =  "1"  /* Replace L Finger Phalanx w Autol Sub, Perc Endo */
"0PRV4JZ"  =  "1"  /* Replace L Finger Phalanx w Synth Sub, Perc Endo */
"0PRV4KZ"  =  "1"  /* Replace L Finger Phalanx w Nonaut Sub, Perc Endo */
"0PS000Z"  =  "1"  /* Reposition Sternum with Rigid Plate Fix, Open Approach */
"0PS004Z"  =  "1"  /* Reposition Sternum with Int Fix, Open Approach */
"0PS00ZZ"  =  "1"  /* Reposition Sternum, Open Approach */
"0PS030Z"  =  "1"  /* Reposition Sternum with Rigid Plate Fix, Perc Approach */
"0PS034Z"  =  "1"  /* Reposition Sternum with Int Fix, Perc Approach */
"0PS040Z"  =  "1"  /* Reposition Sternum with Rigid Plate Fix, Perc Endo Approach */
"0PS044Z"  =  "1"  /* Reposition Sternum with Int Fix, Perc Endo Approach */
"0PS104Z"  =  "1"  /* Reposition Right Rib with Int Fix, Open Approach */
"0PS10ZZ"  =  "1"  /* Reposition Right Rib, Open Approach */
"0PS134Z"  =  "1"  /* Reposition Right Rib with Int Fix, Perc Approach */
"0PS144Z"  =  "1"  /* Reposition Right Rib with Int Fix, Perc Endo Approach */
"0PS204Z"  =  "1"  /* Reposition Left Rib with Int Fix, Open Approach */
"0PS20ZZ"  =  "1"  /* Reposition Left Rib, Open Approach */
"0PS234Z"  =  "1"  /* Reposition Left Rib with Int Fix, Perc Approach */
"0PS244Z"  =  "1"  /* Reposition Left Rib with Int Fix, Perc Endo Approach */
"0PS304Z"  =  "1"  /* Reposition Cervical Vertebra with Int Fix, Open Approach */
"0PS30ZZ"  =  "1"  /* Reposition Cervical Vertebra, Open Approach */
"0PS334Z"  =  "1"  /* Reposition Cervical Vertebra with Int Fix, Perc Approach */
"0PS33ZZ"  =  "1"  /* Reposition Cervical Vertebra, Percutaneous Approach */
"0PS344Z"  =  "1"  /* Reposition Cervcal Vertebra with Int Fix, Perc Endo Approach */
"0PS34ZZ"  =  "1"  /* Reposition Cervical Vertebra, Perc Endo Approach */
"0PS3XZZ"  =  "1"  /* Reposition Cervical Vertebra, External Approach */
"0PS404Z"  =  "1"  /* Reposition Thoracic Vertebra with Int Fix, Open Approach */
"0PS40ZZ"  =  "1"  /* Reposition Thoracic Vertebra, Open Approach */
"0PS434Z"  =  "1"  /* Reposition Thoracic Vertebra with Int Fix, Perc Approach */
"0PS43ZZ"  =  "1"  /* Reposition Thoracic Vertebra, Percutaneous Approach */
"0PS444Z"  =  "1"  /* Reposition Thor Vertebra with Int Fix, Perc Endo Approach */
"0PS44ZZ"  =  "1"  /* Reposition Thoracic Vertebra, Perc Endo Approach */
"0PS4XZZ"  =  "1"  /* Reposition Thoracic Vertebra, External Approach */
"0PS504Z"  =  "1"  /* Reposition Right Scapula with Int Fix, Open Approach */
"0PS50ZZ"  =  "1"  /* Reposition Right Scapula, Open Approach */
"0PS534Z"  =  "1"  /* Reposition Right Scapula with Int Fix, Perc Approach */
"0PS544Z"  =  "1"  /* Reposition Right Scapula with Int Fix, Perc Endo Approach */
"0PS604Z"  =  "1"  /* Reposition Left Scapula with Int Fix, Open Approach */
"0PS60ZZ"  =  "1"  /* Reposition Left Scapula, Open Approach */
"0PS634Z"  =  "1"  /* Reposition Left Scapula with Int Fix, Perc Approach */
"0PS644Z"  =  "1"  /* Reposition Left Scapula with Int Fix, Perc Endo Approach */
"0PS704Z"  =  "1"  /* Reposition Right Glenoid Cavity with Int Fix, Open Approach */
"0PS70ZZ"  =  "1"  /* Reposition Right Glenoid Cavity, Open Approach */
"0PS734Z"  =  "1"  /* Reposition Right Glenoid Cavity with Int Fix, Perc Approach */
"0PS744Z"  =  "1"  /* Reposition R Glenoid Cav with Int Fix, Perc Endo Approach */
"0PS804Z"  =  "1"  /* Reposition Left Glenoid Cavity with Int Fix, Open Approach */
"0PS80ZZ"  =  "1"  /* Reposition Left Glenoid Cavity, Open Approach */
"0PS834Z"  =  "1"  /* Reposition Left Glenoid Cavity with Int Fix, Perc Approach */
"0PS844Z"  =  "1"  /* Reposition L Glenoid Cav with Int Fix, Perc Endo Approach */
"0PS904Z"  =  "1"  /* Reposition Right Clavicle with Int Fix, Open Approach */
"0PS90ZZ"  =  "1"  /* Reposition Right Clavicle, Open Approach */
"0PS934Z"  =  "1"  /* Reposition Right Clavicle with Int Fix, Perc Approach */
"0PS944Z"  =  "1"  /* Reposition Right Clavicle with Int Fix, Perc Endo Approach */
"0PSB04Z"  =  "1"  /* Reposition Left Clavicle with Int Fix, Open Approach */
"0PSB0ZZ"  =  "1"  /* Reposition Left Clavicle, Open Approach */
"0PSB34Z"  =  "1"  /* Reposition Left Clavicle with Int Fix, Perc Approach */
"0PSB44Z"  =  "1"  /* Reposition Left Clavicle with Int Fix, Perc Endo Approach */
"0PSC04Z"  =  "1"  /* Reposition Right Humeral Head with Int Fix, Open Approach */
"0PSC05Z"  =  "1"  /* Reposition Right Humeral Head with Ext Fix, Open Approach */
"0PSC06Z"  =  "1"  /* Reposition R Humeral Head with Intramed Fix, Open Approach */
"0PSC0BZ"  =  "1"  /* Reposition R Humeral Head w Monopln Ext Fix, Open */
"0PSC0CZ"  =  "1"  /* Reposition R Humeral Head with Ring Ext Fix, Open Approach */
"0PSC0DZ"  =  "1"  /* Reposition R Humeral Head with Hybrid Ext Fix, Open Approach */
"0PSC0ZZ"  =  "1"  /* Reposition Right Humeral Head, Open Approach */
"0PSC34Z"  =  "1"  /* Reposition Right Humeral Head with Int Fix, Perc Approach */
"0PSC35Z"  =  "1"  /* Reposition Right Humeral Head with Ext Fix, Perc Approach */
"0PSC36Z"  =  "1"  /* Reposition R Humeral Head with Intramed Fix, Perc Approach */
"0PSC3BZ"  =  "1"  /* Reposition R Humeral Head w Monopln Ext Fix, Perc */
"0PSC3CZ"  =  "1"  /* Reposition R Humeral Head with Ring Ext Fix, Perc Approach */
"0PSC3DZ"  =  "1"  /* Reposition R Humeral Head with Hybrid Ext Fix, Perc Approach */
"0PSC44Z"  =  "1"  /* Reposition R Humeral Head with Int Fix, Perc Endo Approach */
"0PSC45Z"  =  "1"  /* Reposition R Humeral Head with Ext Fix, Perc Endo Approach */
"0PSC46Z"  =  "1"  /* Reposition R Humeral Head w Intramed Fix, Perc Endo */
"0PSC4BZ"  =  "1"  /* Reposition R Humeral Head w Monopln Ext Fix, Perc Endo */
"0PSC4CZ"  =  "1"  /* Reposition R Humeral Head w Ring Ext Fix, Perc Endo */
"0PSC4DZ"  =  "1"  /* Reposition R Humeral Head w Hybrid Ext Fix, Perc Endo */
"0PSD04Z"  =  "1"  /* Reposition Left Humeral Head with Int Fix, Open Approach */
"0PSD05Z"  =  "1"  /* Reposition Left Humeral Head with Ext Fix, Open Approach */
"0PSD06Z"  =  "1"  /* Reposition L Humeral Head with Intramed Fix, Open Approach */
"0PSD0BZ"  =  "1"  /* Reposition L Humeral Head w Monopln Ext Fix, Open */
"0PSD0CZ"  =  "1"  /* Reposition L Humeral Head with Ring Ext Fix, Open Approach */
"0PSD0DZ"  =  "1"  /* Reposition L Humeral Head with Hybrid Ext Fix, Open Approach */
"0PSD0ZZ"  =  "1"  /* Reposition Left Humeral Head, Open Approach */
"0PSD34Z"  =  "1"  /* Reposition Left Humeral Head with Int Fix, Perc Approach */
"0PSD35Z"  =  "1"  /* Reposition Left Humeral Head with Ext Fix, Perc Approach */
"0PSD36Z"  =  "1"  /* Reposition L Humeral Head with Intramed Fix, Perc Approach */
"0PSD3BZ"  =  "1"  /* Reposition L Humeral Head w Monopln Ext Fix, Perc */
"0PSD3CZ"  =  "1"  /* Reposition L Humeral Head with Ring Ext Fix, Perc Approach */
"0PSD3DZ"  =  "1"  /* Reposition L Humeral Head with Hybrid Ext Fix, Perc Approach */
"0PSD44Z"  =  "1"  /* Reposition L Humeral Head with Int Fix, Perc Endo Approach */
"0PSD45Z"  =  "1"  /* Reposition L Humeral Head with Ext Fix, Perc Endo Approach */
"0PSD46Z"  =  "1"  /* Reposition L Humeral Head w Intramed Fix, Perc Endo */
"0PSD4BZ"  =  "1"  /* Reposition L Humeral Head w Monopln Ext Fix, Perc Endo */
"0PSD4CZ"  =  "1"  /* Reposition L Humeral Head w Ring Ext Fix, Perc Endo */
"0PSD4DZ"  =  "1"  /* Reposition L Humeral Head w Hybrid Ext Fix, Perc Endo */
"0PSF04Z"  =  "1"  /* Reposition Right Humeral Shaft with Int Fix, Open Approach */
"0PSF05Z"  =  "1"  /* Reposition Right Humeral Shaft with Ext Fix, Open Approach */
"0PSF06Z"  =  "1"  /* Reposition R Humeral Shaft with Intramed Fix, Open Approach */
"0PSF0BZ"  =  "1"  /* Reposition R Humeral Shaft w Monopln Ext Fix, Open */
"0PSF0CZ"  =  "1"  /* Reposition R Humeral Shaft with Ring Ext Fix, Open Approach */
"0PSF0DZ"  =  "1"  /* Reposition R Humeral Shaft w Hybrid Ext Fix, Open */
"0PSF0ZZ"  =  "1"  /* Reposition Right Humeral Shaft, Open Approach */
"0PSF34Z"  =  "1"  /* Reposition Right Humeral Shaft with Int Fix, Perc Approach */
"0PSF35Z"  =  "1"  /* Reposition Right Humeral Shaft with Ext Fix, Perc Approach */
"0PSF36Z"  =  "1"  /* Reposition R Humeral Shaft with Intramed Fix, Perc Approach */
"0PSF3BZ"  =  "1"  /* Reposition R Humeral Shaft w Monopln Ext Fix, Perc */
"0PSF3CZ"  =  "1"  /* Reposition R Humeral Shaft with Ring Ext Fix, Perc Approach */
"0PSF3DZ"  =  "1"  /* Reposition R Humeral Shaft w Hybrid Ext Fix, Perc */
"0PSF44Z"  =  "1"  /* Reposition R Humeral Shaft with Int Fix, Perc Endo Approach */
"0PSF45Z"  =  "1"  /* Reposition R Humeral Shaft with Ext Fix, Perc Endo Approach */
"0PSF46Z"  =  "1"  /* Reposition R Humeral Shaft w Intramed Fix, Perc Endo */
"0PSF4BZ"  =  "1"  /* Reposition R Humeral Shaft w Monopln Ext Fix, Perc Endo */
"0PSF4CZ"  =  "1"  /* Reposition R Humeral Shaft w Ring Ext Fix, Perc Endo */
"0PSF4DZ"  =  "1"  /* Reposition R Humeral Shaft w Hybrid Ext Fix, Perc Endo */
"0PSG04Z"  =  "1"  /* Reposition Left Humeral Shaft with Int Fix, Open Approach */
"0PSG05Z"  =  "1"  /* Reposition Left Humeral Shaft with Ext Fix, Open Approach */
"0PSG06Z"  =  "1"  /* Reposition L Humeral Shaft with Intramed Fix, Open Approach */
"0PSG0BZ"  =  "1"  /* Reposition L Humeral Shaft w Monopln Ext Fix, Open */
"0PSG0CZ"  =  "1"  /* Reposition L Humeral Shaft with Ring Ext Fix, Open Approach */
"0PSG0DZ"  =  "1"  /* Reposition L Humeral Shaft w Hybrid Ext Fix, Open */
"0PSG0ZZ"  =  "1"  /* Reposition Left Humeral Shaft, Open Approach */
"0PSG34Z"  =  "1"  /* Reposition Left Humeral Shaft with Int Fix, Perc Approach */
"0PSG35Z"  =  "1"  /* Reposition Left Humeral Shaft with Ext Fix, Perc Approach */
"0PSG36Z"  =  "1"  /* Reposition L Humeral Shaft with Intramed Fix, Perc Approach */
"0PSG3BZ"  =  "1"  /* Reposition L Humeral Shaft w Monopln Ext Fix, Perc */
"0PSG3CZ"  =  "1"  /* Reposition L Humeral Shaft with Ring Ext Fix, Perc Approach */
"0PSG3DZ"  =  "1"  /* Reposition L Humeral Shaft w Hybrid Ext Fix, Perc */
"0PSG44Z"  =  "1"  /* Reposition L Humeral Shaft with Int Fix, Perc Endo Approach */
"0PSG45Z"  =  "1"  /* Reposition L Humeral Shaft with Ext Fix, Perc Endo Approach */
"0PSG46Z"  =  "1"  /* Reposition L Humeral Shaft w Intramed Fix, Perc Endo */
"0PSG4BZ"  =  "1"  /* Reposition L Humeral Shaft w Monopln Ext Fix, Perc Endo */
"0PSG4CZ"  =  "1"  /* Reposition L Humeral Shaft w Ring Ext Fix, Perc Endo */
"0PSG4DZ"  =  "1"  /* Reposition L Humeral Shaft w Hybrid Ext Fix, Perc Endo */
"0PSH04Z"  =  "1"  /* Reposition Right Radius with Int Fix, Open Approach */
"0PSH05Z"  =  "1"  /* Reposition Right Radius with Ext Fix, Open Approach */
"0PSH06Z"  =  "1"  /* Reposition Right Radius with Intramed Fix, Open Approach */
"0PSH0BZ"  =  "1"  /* Reposition Right Radius with Monopln Ext Fix, Open Approach */
"0PSH0CZ"  =  "1"  /* Reposition Right Radius with Ring Ext Fix, Open Approach */
"0PSH0DZ"  =  "1"  /* Reposition Right Radius with Hybrid Ext Fix, Open Approach */
"0PSH0ZZ"  =  "1"  /* Reposition Right Radius, Open Approach */
"0PSH34Z"  =  "1"  /* Reposition Right Radius with Int Fix, Perc Approach */
"0PSH35Z"  =  "1"  /* Reposition Right Radius with Ext Fix, Perc Approach */
"0PSH36Z"  =  "1"  /* Reposition Right Radius with Intramed Fix, Perc Approach */
"0PSH3BZ"  =  "1"  /* Reposition Right Radius with Monopln Ext Fix, Perc Approach */
"0PSH3CZ"  =  "1"  /* Reposition Right Radius with Ring Ext Fix, Perc Approach */
"0PSH3DZ"  =  "1"  /* Reposition Right Radius with Hybrid Ext Fix, Perc Approach */
"0PSH44Z"  =  "1"  /* Reposition Right Radius with Int Fix, Perc Endo Approach */
"0PSH45Z"  =  "1"  /* Reposition Right Radius with Ext Fix, Perc Endo Approach */
"0PSH46Z"  =  "1"  /* Reposition R Radius with Intramed Fix, Perc Endo Approach */
"0PSH4BZ"  =  "1"  /* Reposition R Radius with Monopln Ext Fix, Perc Endo Approach */
"0PSH4CZ"  =  "1"  /* Reposition R Radius with Ring Ext Fix, Perc Endo Approach */
"0PSH4DZ"  =  "1"  /* Reposition R Radius with Hybrid Ext Fix, Perc Endo Approach */
"0PSJ04Z"  =  "1"  /* Reposition Left Radius with Int Fix, Open Approach */
"0PSJ05Z"  =  "1"  /* Reposition Left Radius with Ext Fix, Open Approach */
"0PSJ06Z"  =  "1"  /* Reposition Left Radius with Intramed Fix, Open Approach */
"0PSJ0BZ"  =  "1"  /* Reposition Left Radius with Monopln Ext Fix, Open Approach */
"0PSJ0CZ"  =  "1"  /* Reposition Left Radius with Ring Ext Fix, Open Approach */
"0PSJ0DZ"  =  "1"  /* Reposition Left Radius with Hybrid Ext Fix, Open Approach */
"0PSJ0ZZ"  =  "1"  /* Reposition Left Radius, Open Approach */
"0PSJ34Z"  =  "1"  /* Reposition Left Radius with Int Fix, Perc Approach */
"0PSJ35Z"  =  "1"  /* Reposition Left Radius with Ext Fix, Perc Approach */
"0PSJ36Z"  =  "1"  /* Reposition Left Radius with Intramed Fix, Perc Approach */
"0PSJ3BZ"  =  "1"  /* Reposition Left Radius with Monopln Ext Fix, Perc Approach */
"0PSJ3CZ"  =  "1"  /* Reposition Left Radius with Ring Ext Fix, Perc Approach */
"0PSJ3DZ"  =  "1"  /* Reposition Left Radius with Hybrid Ext Fix, Perc Approach */
"0PSJ44Z"  =  "1"  /* Reposition Left Radius with Int Fix, Perc Endo Approach */
"0PSJ45Z"  =  "1"  /* Reposition Left Radius with Ext Fix, Perc Endo Approach */
"0PSJ46Z"  =  "1"  /* Reposition Left Radius with Intramed Fix, Perc Endo Approach */
"0PSJ4BZ"  =  "1"  /* Reposition L Radius with Monopln Ext Fix, Perc Endo Approach */
"0PSJ4CZ"  =  "1"  /* Reposition Left Radius with Ring Ext Fix, Perc Endo Approach */
"0PSJ4DZ"  =  "1"  /* Reposition L Radius with Hybrid Ext Fix, Perc Endo Approach */
"0PSK04Z"  =  "1"  /* Reposition Right Ulna with Int Fix, Open Approach */
"0PSK05Z"  =  "1"  /* Reposition Right Ulna with Ext Fix, Open Approach */
"0PSK06Z"  =  "1"  /* Reposition Right Ulna with Intramed Fix, Open Approach */
"0PSK0BZ"  =  "1"  /* Reposition Right Ulna with Monopln Ext Fix, Open Approach */
"0PSK0CZ"  =  "1"  /* Reposition Right Ulna with Ring Ext Fix, Open Approach */
"0PSK0DZ"  =  "1"  /* Reposition Right Ulna with Hybrid Ext Fix, Open Approach */
"0PSK0ZZ"  =  "1"  /* Reposition Right Ulna, Open Approach */
"0PSK34Z"  =  "1"  /* Reposition Right Ulna with Int Fix, Perc Approach */
"0PSK35Z"  =  "1"  /* Reposition Right Ulna with Ext Fix, Perc Approach */
"0PSK36Z"  =  "1"  /* Reposition Right Ulna with Intramed Fix, Perc Approach */
"0PSK3BZ"  =  "1"  /* Reposition Right Ulna with Monopln Ext Fix, Perc Approach */
"0PSK3CZ"  =  "1"  /* Reposition Right Ulna with Ring Ext Fix, Perc Approach */
"0PSK3DZ"  =  "1"  /* Reposition Right Ulna with Hybrid Ext Fix, Perc Approach */
"0PSK44Z"  =  "1"  /* Reposition Right Ulna with Int Fix, Perc Endo Approach */
"0PSK45Z"  =  "1"  /* Reposition Right Ulna with Ext Fix, Perc Endo Approach */
"0PSK46Z"  =  "1"  /* Reposition Right Ulna with Intramed Fix, Perc Endo Approach */
"0PSK4BZ"  =  "1"  /* Reposition R Ulna with Monopln Ext Fix, Perc Endo Approach */
"0PSK4CZ"  =  "1"  /* Reposition Right Ulna with Ring Ext Fix, Perc Endo Approach */
"0PSK4DZ"  =  "1"  /* Reposition R Ulna with Hybrid Ext Fix, Perc Endo Approach */
"0PSL04Z"  =  "1"  /* Reposition Left Ulna with Int Fix, Open Approach */
"0PSL05Z"  =  "1"  /* Reposition Left Ulna with Ext Fix, Open Approach */
"0PSL06Z"  =  "1"  /* Reposition Left Ulna with Intramed Fix, Open Approach */
"0PSL0BZ"  =  "1"  /* Reposition Left Ulna with Monopln Ext Fix, Open Approach */
"0PSL0CZ"  =  "1"  /* Reposition Left Ulna with Ring Ext Fix, Open Approach */
"0PSL0DZ"  =  "1"  /* Reposition Left Ulna with Hybrid Ext Fix, Open Approach */
"0PSL0ZZ"  =  "1"  /* Reposition Left Ulna, Open Approach */
"0PSL34Z"  =  "1"  /* Reposition Left Ulna with Int Fix, Perc Approach */
"0PSL35Z"  =  "1"  /* Reposition Left Ulna with Ext Fix, Perc Approach */
"0PSL36Z"  =  "1"  /* Reposition Left Ulna with Intramed Fix, Perc Approach */
"0PSL3BZ"  =  "1"  /* Reposition Left Ulna with Monopln Ext Fix, Perc Approach */
"0PSL3CZ"  =  "1"  /* Reposition Left Ulna with Ring Ext Fix, Perc Approach */
"0PSL3DZ"  =  "1"  /* Reposition Left Ulna with Hybrid Ext Fix, Perc Approach */
"0PSL44Z"  =  "1"  /* Reposition Left Ulna with Int Fix, Perc Endo Approach */
"0PSL45Z"  =  "1"  /* Reposition Left Ulna with Ext Fix, Perc Endo Approach */
"0PSL46Z"  =  "1"  /* Reposition Left Ulna with Intramed Fix, Perc Endo Approach */
"0PSL4BZ"  =  "1"  /* Reposition L Ulna with Monopln Ext Fix, Perc Endo Approach */
"0PSL4CZ"  =  "1"  /* Reposition Left Ulna with Ring Ext Fix, Perc Endo Approach */
"0PSL4DZ"  =  "1"  /* Reposition Left Ulna with Hybrid Ext Fix, Perc Endo Approach */
"0PSM04Z"  =  "1"  /* Reposition Right Carpal with Int Fix, Open Approach */
"0PSM05Z"  =  "1"  /* Reposition Right Carpal with Ext Fix, Open Approach */
"0PSM0ZZ"  =  "1"  /* Reposition Right Carpal, Open Approach */
"0PSM34Z"  =  "1"  /* Reposition Right Carpal with Int Fix, Perc Approach */
"0PSM35Z"  =  "1"  /* Reposition Right Carpal with Ext Fix, Perc Approach */
"0PSM44Z"  =  "1"  /* Reposition Right Carpal with Int Fix, Perc Endo Approach */
"0PSM45Z"  =  "1"  /* Reposition Right Carpal with Ext Fix, Perc Endo Approach */
"0PSN04Z"  =  "1"  /* Reposition Left Carpal with Int Fix, Open Approach */
"0PSN05Z"  =  "1"  /* Reposition Left Carpal with Ext Fix, Open Approach */
"0PSN0ZZ"  =  "1"  /* Reposition Left Carpal, Open Approach */
"0PSN34Z"  =  "1"  /* Reposition Left Carpal with Int Fix, Perc Approach */
"0PSN35Z"  =  "1"  /* Reposition Left Carpal with Ext Fix, Perc Approach */
"0PSN44Z"  =  "1"  /* Reposition Left Carpal with Int Fix, Perc Endo Approach */
"0PSN45Z"  =  "1"  /* Reposition Left Carpal with Ext Fix, Perc Endo Approach */
"0PSP04Z"  =  "1"  /* Reposition Right Metacarpal with Int Fix, Open Approach */
"0PSP05Z"  =  "1"  /* Reposition Right Metacarpal with Ext Fix, Open Approach */
"0PSP0ZZ"  =  "1"  /* Reposition Right Metacarpal, Open Approach */
"0PSP34Z"  =  "1"  /* Reposition Right Metacarpal with Int Fix, Perc Approach */
"0PSP35Z"  =  "1"  /* Reposition Right Metacarpal with Ext Fix, Perc Approach */
"0PSP44Z"  =  "1"  /* Reposition Right Metacarpal with Int Fix, Perc Endo Approach */
"0PSP45Z"  =  "1"  /* Reposition Right Metacarpal with Ext Fix, Perc Endo Approach */
"0PSQ04Z"  =  "1"  /* Reposition Left Metacarpal with Int Fix, Open Approach */
"0PSQ05Z"  =  "1"  /* Reposition Left Metacarpal with Ext Fix, Open Approach */
"0PSQ0ZZ"  =  "1"  /* Reposition Left Metacarpal, Open Approach */
"0PSQ34Z"  =  "1"  /* Reposition Left Metacarpal with Int Fix, Perc Approach */
"0PSQ35Z"  =  "1"  /* Reposition Left Metacarpal with Ext Fix, Perc Approach */
"0PSQ44Z"  =  "1"  /* Reposition Left Metacarpal with Int Fix, Perc Endo Approach */
"0PSQ45Z"  =  "1"  /* Reposition Left Metacarpal with Ext Fix, Perc Endo Approach */
"0PSR04Z"  =  "1"  /* Reposition Right Thumb Phalanx with Int Fix, Open Approach */
"0PSR05Z"  =  "1"  /* Reposition Right Thumb Phalanx with Ext Fix, Open Approach */
"0PSR0ZZ"  =  "1"  /* Reposition Right Thumb Phalanx, Open Approach */
"0PSR34Z"  =  "1"  /* Reposition Right Thumb Phalanx with Int Fix, Perc Approach */
"0PSR35Z"  =  "1"  /* Reposition Right Thumb Phalanx with Ext Fix, Perc Approach */
"0PSR44Z"  =  "1"  /* Reposition R Thumb Phalanx with Int Fix, Perc Endo Approach */
"0PSR45Z"  =  "1"  /* Reposition R Thumb Phalanx with Ext Fix, Perc Endo Approach */
"0PSS04Z"  =  "1"  /* Reposition Left Thumb Phalanx with Int Fix, Open Approach */
"0PSS05Z"  =  "1"  /* Reposition Left Thumb Phalanx with Ext Fix, Open Approach */
"0PSS0ZZ"  =  "1"  /* Reposition Left Thumb Phalanx, Open Approach */
"0PSS34Z"  =  "1"  /* Reposition Left Thumb Phalanx with Int Fix, Perc Approach */
"0PSS35Z"  =  "1"  /* Reposition Left Thumb Phalanx with Ext Fix, Perc Approach */
"0PSS44Z"  =  "1"  /* Reposition L Thumb Phalanx with Int Fix, Perc Endo Approach */
"0PSS45Z"  =  "1"  /* Reposition L Thumb Phalanx with Ext Fix, Perc Endo Approach */
"0PST04Z"  =  "1"  /* Reposition Right Finger Phalanx with Int Fix, Open Approach */
"0PST05Z"  =  "1"  /* Reposition Right Finger Phalanx with Ext Fix, Open Approach */
"0PST0ZZ"  =  "1"  /* Reposition Right Finger Phalanx, Open Approach */
"0PST34Z"  =  "1"  /* Reposition Right Finger Phalanx with Int Fix, Perc Approach */
"0PST35Z"  =  "1"  /* Reposition Right Finger Phalanx with Ext Fix, Perc Approach */
"0PST44Z"  =  "1"  /* Reposition R Finger Phalanx with Int Fix, Perc Endo Approach */
"0PST45Z"  =  "1"  /* Reposition R Finger Phalanx with Ext Fix, Perc Endo Approach */
"0PSV04Z"  =  "1"  /* Reposition Left Finger Phalanx with Int Fix, Open Approach */
"0PSV05Z"  =  "1"  /* Reposition Left Finger Phalanx with Ext Fix, Open Approach */
"0PSV0ZZ"  =  "1"  /* Reposition Left Finger Phalanx, Open Approach */
"0PSV34Z"  =  "1"  /* Reposition Left Finger Phalanx with Int Fix, Perc Approach */
"0PSV35Z"  =  "1"  /* Reposition Left Finger Phalanx with Ext Fix, Perc Approach */
"0PSV44Z"  =  "1"  /* Reposition L Finger Phalanx with Int Fix, Perc Endo Approach */
"0PSV45Z"  =  "1"  /* Reposition L Finger Phalanx with Ext Fix, Perc Endo Approach */
"0PT00ZZ"  =  "1"  /* Resection of Sternum, Open Approach */
"0PT10ZZ"  =  "1"  /* Resection of Right Rib, Open Approach */
"0PT20ZZ"  =  "1"  /* Resection of Left Rib, Open Approach */
"0PT50ZZ"  =  "1"  /* Resection of Right Scapula, Open Approach */
"0PT60ZZ"  =  "1"  /* Resection of Left Scapula, Open Approach */
"0PT70ZZ"  =  "1"  /* Resection of Right Glenoid Cavity, Open Approach */
"0PT80ZZ"  =  "1"  /* Resection of Left Glenoid Cavity, Open Approach */
"0PT90ZZ"  =  "1"  /* Resection of Right Clavicle, Open Approach */
"0PTB0ZZ"  =  "1"  /* Resection of Left Clavicle, Open Approach */
"0PTC0ZZ"  =  "1"  /* Resection of Right Humeral Head, Open Approach */
"0PTD0ZZ"  =  "1"  /* Resection of Left Humeral Head, Open Approach */
"0PTF0ZZ"  =  "1"  /* Resection of Right Humeral Shaft, Open Approach */
"0PTG0ZZ"  =  "1"  /* Resection of Left Humeral Shaft, Open Approach */
"0PTH0ZZ"  =  "1"  /* Resection of Right Radius, Open Approach */
"0PTJ0ZZ"  =  "1"  /* Resection of Left Radius, Open Approach */
"0PTK0ZZ"  =  "1"  /* Resection of Right Ulna, Open Approach */
"0PTL0ZZ"  =  "1"  /* Resection of Left Ulna, Open Approach */
"0PTM0ZZ"  =  "1"  /* Resection of Right Carpal, Open Approach */
"0PTN0ZZ"  =  "1"  /* Resection of Left Carpal, Open Approach */
"0PTP0ZZ"  =  "1"  /* Resection of Right Metacarpal, Open Approach */
"0PTQ0ZZ"  =  "1"  /* Resection of Left Metacarpal, Open Approach */
"0PTR0ZZ"  =  "1"  /* Resection of Right Thumb Phalanx, Open Approach */
"0PTS0ZZ"  =  "1"  /* Resection of Left Thumb Phalanx, Open Approach */
"0PTT0ZZ"  =  "1"  /* Resection of Right Finger Phalanx, Open Approach */
"0PTV0ZZ"  =  "1"  /* Resection of Left Finger Phalanx, Open Approach */
"0PU007Z"  =  "1"  /* Supplement Sternum with Autol Sub, Open Approach */
"0PU00JZ"  =  "1"  /* Supplement Sternum with Synthetic Substitute, Open Approach */
"0PU00KZ"  =  "1"  /* Supplement Sternum with Nonaut Sub, Open Approach */
"0PU037Z"  =  "1"  /* Supplement Sternum with Autol Sub, Perc Approach */
"0PU03JZ"  =  "1"  /* Supplement Sternum with Synthetic Substitute, Perc Approach */
"0PU03KZ"  =  "1"  /* Supplement Sternum with Nonaut Sub, Perc Approach */
"0PU047Z"  =  "1"  /* Supplement Sternum with Autol Sub, Perc Endo Approach */
"0PU04JZ"  =  "1"  /* Supplement Sternum with Synth Sub, Perc Endo Approach */
"0PU04KZ"  =  "1"  /* Supplement Sternum with Nonaut Sub, Perc Endo Approach */
"0PU107Z"  =  "1"  /* Supplement Right Rib with Autol Sub, Open Approach */
"0PU10JZ"  =  "1"  /* Supplement Right Rib with Synth Sub, Open Approach */
"0PU10KZ"  =  "1"  /* Supplement Right Rib with Nonaut Sub, Open Approach */
"0PU137Z"  =  "1"  /* Supplement Right Rib with Autol Sub, Perc Approach */
"0PU13JZ"  =  "1"  /* Supplement Right Rib with Synth Sub, Perc Approach */
"0PU13KZ"  =  "1"  /* Supplement Right Rib with Nonaut Sub, Perc Approach */
"0PU147Z"  =  "1"  /* Supplement Right Rib with Autol Sub, Perc Endo Approach */
"0PU14JZ"  =  "1"  /* Supplement Right Rib with Synth Sub, Perc Endo Approach */
"0PU14KZ"  =  "1"  /* Supplement Right Rib with Nonaut Sub, Perc Endo Approach */
"0PU207Z"  =  "1"  /* Supplement Left Rib with Autol Sub, Open Approach */
"0PU20JZ"  =  "1"  /* Supplement Left Rib with Synthetic Substitute, Open Approach */
"0PU20KZ"  =  "1"  /* Supplement Left Rib with Nonaut Sub, Open Approach */
"0PU237Z"  =  "1"  /* Supplement Left Rib with Autol Sub, Perc Approach */
"0PU23JZ"  =  "1"  /* Supplement Left Rib with Synthetic Substitute, Perc Approach */
"0PU23KZ"  =  "1"  /* Supplement Left Rib with Nonaut Sub, Perc Approach */
"0PU247Z"  =  "1"  /* Supplement Left Rib with Autol Sub, Perc Endo Approach */
"0PU24JZ"  =  "1"  /* Supplement Left Rib with Synth Sub, Perc Endo Approach */
"0PU24KZ"  =  "1"  /* Supplement Left Rib with Nonaut Sub, Perc Endo Approach */
"0PU307Z"  =  "1"  /* Supplement Cervical Vertebra with Autol Sub, Open Approach */
"0PU30JZ"  =  "1"  /* Supplement Cervical Vertebra with Synth Sub, Open Approach */
"0PU30KZ"  =  "1"  /* Supplement Cervical Vertebra with Nonaut Sub, Open Approach */
"0PU337Z"  =  "1"  /* Supplement Cervical Vertebra with Autol Sub, Perc Approach */
"0PU33JZ"  =  "1"  /* Supplement Cervical Vertebra with Synth Sub, Perc Approach */
"0PU33KZ"  =  "1"  /* Supplement Cervical Vertebra with Nonaut Sub, Perc Approach */
"0PU347Z"  =  "1"  /* Supplement Cervcal Vertebra w Autol Sub, Perc Endo */
"0PU34JZ"  =  "1"  /* Supplement Cervcal Vertebra w Synth Sub, Perc Endo */
"0PU34KZ"  =  "1"  /* Supplement Cervcal Vertebra w Nonaut Sub, Perc Endo */
"0PU407Z"  =  "1"  /* Supplement Thoracic Vertebra with Autol Sub, Open Approach */
"0PU40JZ"  =  "1"  /* Supplement Thoracic Vertebra with Synth Sub, Open Approach */
"0PU40KZ"  =  "1"  /* Supplement Thoracic Vertebra with Nonaut Sub, Open Approach */
"0PU437Z"  =  "1"  /* Supplement Thoracic Vertebra with Autol Sub, Perc Approach */
"0PU43JZ"  =  "1"  /* Supplement Thoracic Vertebra with Synth Sub, Perc Approach */
"0PU43KZ"  =  "1"  /* Supplement Thoracic Vertebra with Nonaut Sub, Perc Approach */
"0PU447Z"  =  "1"  /* Supplement Thor Vertebra with Autol Sub, Perc Endo Approach */
"0PU44JZ"  =  "1"  /* Supplement Thor Vertebra with Synth Sub, Perc Endo Approach */
"0PU44KZ"  =  "1"  /* Supplement Thor Vertebra with Nonaut Sub, Perc Endo Approach */
"0PU507Z"  =  "1"  /* Supplement Right Scapula with Autol Sub, Open Approach */
"0PU50JZ"  =  "1"  /* Supplement Right Scapula with Synth Sub, Open Approach */
"0PU50KZ"  =  "1"  /* Supplement Right Scapula with Nonaut Sub, Open Approach */
"0PU537Z"  =  "1"  /* Supplement Right Scapula with Autol Sub, Perc Approach */
"0PU53JZ"  =  "1"  /* Supplement Right Scapula with Synth Sub, Perc Approach */
"0PU53KZ"  =  "1"  /* Supplement Right Scapula with Nonaut Sub, Perc Approach */
"0PU547Z"  =  "1"  /* Supplement Right Scapula with Autol Sub, Perc Endo Approach */
"0PU54JZ"  =  "1"  /* Supplement Right Scapula with Synth Sub, Perc Endo Approach */
"0PU54KZ"  =  "1"  /* Supplement Right Scapula with Nonaut Sub, Perc Endo Approach */
"0PU607Z"  =  "1"  /* Supplement Left Scapula with Autol Sub, Open Approach */
"0PU60JZ"  =  "1"  /* Supplement Left Scapula with Synth Sub, Open Approach */
"0PU60KZ"  =  "1"  /* Supplement Left Scapula with Nonaut Sub, Open Approach */
"0PU637Z"  =  "1"  /* Supplement Left Scapula with Autol Sub, Perc Approach */
"0PU63JZ"  =  "1"  /* Supplement Left Scapula with Synth Sub, Perc Approach */
"0PU63KZ"  =  "1"  /* Supplement Left Scapula with Nonaut Sub, Perc Approach */
"0PU647Z"  =  "1"  /* Supplement Left Scapula with Autol Sub, Perc Endo Approach */
"0PU64JZ"  =  "1"  /* Supplement Left Scapula with Synth Sub, Perc Endo Approach */
"0PU64KZ"  =  "1"  /* Supplement Left Scapula with Nonaut Sub, Perc Endo Approach */
"0PU707Z"  =  "1"  /* Supplement R Glenoid Cav with Autol Sub, Open Approach */
"0PU70JZ"  =  "1"  /* Supplement R Glenoid Cav with Synth Sub, Open Approach */
"0PU70KZ"  =  "1"  /* Supplement R Glenoid Cav with Nonaut Sub, Open Approach */
"0PU737Z"  =  "1"  /* Supplement R Glenoid Cav with Autol Sub, Perc Approach */
"0PU73JZ"  =  "1"  /* Supplement R Glenoid Cav with Synth Sub, Perc Approach */
"0PU73KZ"  =  "1"  /* Supplement R Glenoid Cav with Nonaut Sub, Perc Approach */
"0PU747Z"  =  "1"  /* Supplement R Glenoid Cav with Autol Sub, Perc Endo Approach */
"0PU74JZ"  =  "1"  /* Supplement R Glenoid Cav with Synth Sub, Perc Endo Approach */
"0PU74KZ"  =  "1"  /* Supplement R Glenoid Cav with Nonaut Sub, Perc Endo Approach */
"0PU807Z"  =  "1"  /* Supplement Left Glenoid Cavity with Autol Sub, Open Approach */
"0PU80JZ"  =  "1"  /* Supplement Left Glenoid Cavity with Synth Sub, Open Approach */
"0PU80KZ"  =  "1"  /* Supplement L Glenoid Cav with Nonaut Sub, Open Approach */
"0PU837Z"  =  "1"  /* Supplement Left Glenoid Cavity with Autol Sub, Perc Approach */
"0PU83JZ"  =  "1"  /* Supplement Left Glenoid Cavity with Synth Sub, Perc Approach */
"0PU83KZ"  =  "1"  /* Supplement L Glenoid Cav with Nonaut Sub, Perc Approach */
"0PU847Z"  =  "1"  /* Supplement L Glenoid Cav with Autol Sub, Perc Endo Approach */
"0PU84JZ"  =  "1"  /* Supplement L Glenoid Cav with Synth Sub, Perc Endo Approach */
"0PU84KZ"  =  "1"  /* Supplement L Glenoid Cav with Nonaut Sub, Perc Endo Approach */
"0PU907Z"  =  "1"  /* Supplement Right Clavicle with Autol Sub, Open Approach */
"0PU90JZ"  =  "1"  /* Supplement Right Clavicle with Synth Sub, Open Approach */
"0PU90KZ"  =  "1"  /* Supplement Right Clavicle with Nonaut Sub, Open Approach */
"0PU937Z"  =  "1"  /* Supplement Right Clavicle with Autol Sub, Perc Approach */
"0PU93JZ"  =  "1"  /* Supplement Right Clavicle with Synth Sub, Perc Approach */
"0PU93KZ"  =  "1"  /* Supplement Right Clavicle with Nonaut Sub, Perc Approach */
"0PU947Z"  =  "1"  /* Supplement Right Clavicle with Autol Sub, Perc Endo Approach */
"0PU94JZ"  =  "1"  /* Supplement Right Clavicle with Synth Sub, Perc Endo Approach */
"0PU94KZ"  =  "1"  /* Supplement R Clavicle with Nonaut Sub, Perc Endo Approach */
"0PUB07Z"  =  "1"  /* Supplement Left Clavicle with Autol Sub, Open Approach */
"0PUB0JZ"  =  "1"  /* Supplement Left Clavicle with Synth Sub, Open Approach */
"0PUB0KZ"  =  "1"  /* Supplement Left Clavicle with Nonaut Sub, Open Approach */
"0PUB37Z"  =  "1"  /* Supplement Left Clavicle with Autol Sub, Perc Approach */
"0PUB3JZ"  =  "1"  /* Supplement Left Clavicle with Synth Sub, Perc Approach */
"0PUB3KZ"  =  "1"  /* Supplement Left Clavicle with Nonaut Sub, Perc Approach */
"0PUB47Z"  =  "1"  /* Supplement Left Clavicle with Autol Sub, Perc Endo Approach */
"0PUB4JZ"  =  "1"  /* Supplement Left Clavicle with Synth Sub, Perc Endo Approach */
"0PUB4KZ"  =  "1"  /* Supplement Left Clavicle with Nonaut Sub, Perc Endo Approach */
"0PUC07Z"  =  "1"  /* Supplement Right Humeral Head with Autol Sub, Open Approach */
"0PUC0JZ"  =  "1"  /* Supplement Right Humeral Head with Synth Sub, Open Approach */
"0PUC0KZ"  =  "1"  /* Supplement Right Humeral Head with Nonaut Sub, Open Approach */
"0PUC37Z"  =  "1"  /* Supplement Right Humeral Head with Autol Sub, Perc Approach */
"0PUC3JZ"  =  "1"  /* Supplement Right Humeral Head with Synth Sub, Perc Approach */
"0PUC3KZ"  =  "1"  /* Supplement Right Humeral Head with Nonaut Sub, Perc Approach */
"0PUC47Z"  =  "1"  /* Supplement R Humeral Head with Autol Sub, Perc Endo Approach */
"0PUC4JZ"  =  "1"  /* Supplement R Humeral Head with Synth Sub, Perc Endo Approach */
"0PUC4KZ"  =  "1"  /* Supplement R Humeral Head w Nonaut Sub, Perc Endo */
"0PUD07Z"  =  "1"  /* Supplement Left Humeral Head with Autol Sub, Open Approach */
"0PUD0JZ"  =  "1"  /* Supplement Left Humeral Head with Synth Sub, Open Approach */
"0PUD0KZ"  =  "1"  /* Supplement Left Humeral Head with Nonaut Sub, Open Approach */
"0PUD37Z"  =  "1"  /* Supplement Left Humeral Head with Autol Sub, Perc Approach */
"0PUD3JZ"  =  "1"  /* Supplement Left Humeral Head with Synth Sub, Perc Approach */
"0PUD3KZ"  =  "1"  /* Supplement Left Humeral Head with Nonaut Sub, Perc Approach */
"0PUD47Z"  =  "1"  /* Supplement L Humeral Head with Autol Sub, Perc Endo Approach */
"0PUD4JZ"  =  "1"  /* Supplement L Humeral Head with Synth Sub, Perc Endo Approach */
"0PUD4KZ"  =  "1"  /* Supplement L Humeral Head w Nonaut Sub, Perc Endo */
"0PUF07Z"  =  "1"  /* Supplement Right Humeral Shaft with Autol Sub, Open Approach */
"0PUF0JZ"  =  "1"  /* Supplement Right Humeral Shaft with Synth Sub, Open Approach */
"0PUF0KZ"  =  "1"  /* Supplement R Humeral Shaft with Nonaut Sub, Open Approach */
"0PUF37Z"  =  "1"  /* Supplement Right Humeral Shaft with Autol Sub, Perc Approach */
"0PUF3JZ"  =  "1"  /* Supplement Right Humeral Shaft with Synth Sub, Perc Approach */
"0PUF3KZ"  =  "1"  /* Supplement R Humeral Shaft with Nonaut Sub, Perc Approach */
"0PUF47Z"  =  "1"  /* Supplement R Humeral Shaft w Autol Sub, Perc Endo */
"0PUF4JZ"  =  "1"  /* Supplement R Humeral Shaft w Synth Sub, Perc Endo */
"0PUF4KZ"  =  "1"  /* Supplement R Humeral Shaft w Nonaut Sub, Perc Endo */
"0PUG07Z"  =  "1"  /* Supplement Left Humeral Shaft with Autol Sub, Open Approach */
"0PUG0JZ"  =  "1"  /* Supplement Left Humeral Shaft with Synth Sub, Open Approach */
"0PUG0KZ"  =  "1"  /* Supplement Left Humeral Shaft with Nonaut Sub, Open Approach */
"0PUG37Z"  =  "1"  /* Supplement Left Humeral Shaft with Autol Sub, Perc Approach */
"0PUG3JZ"  =  "1"  /* Supplement Left Humeral Shaft with Synth Sub, Perc Approach */
"0PUG3KZ"  =  "1"  /* Supplement Left Humeral Shaft with Nonaut Sub, Perc Approach */
"0PUG47Z"  =  "1"  /* Supplement L Humeral Shaft w Autol Sub, Perc Endo */
"0PUG4JZ"  =  "1"  /* Supplement L Humeral Shaft w Synth Sub, Perc Endo */
"0PUG4KZ"  =  "1"  /* Supplement L Humeral Shaft w Nonaut Sub, Perc Endo */
"0PUH07Z"  =  "1"  /* Supplement Right Radius with Autol Sub, Open Approach */
"0PUH0JZ"  =  "1"  /* Supplement Right Radius with Synth Sub, Open Approach */
"0PUH0KZ"  =  "1"  /* Supplement Right Radius with Nonaut Sub, Open Approach */
"0PUH37Z"  =  "1"  /* Supplement Right Radius with Autol Sub, Perc Approach */
"0PUH3JZ"  =  "1"  /* Supplement Right Radius with Synth Sub, Perc Approach */
"0PUH3KZ"  =  "1"  /* Supplement Right Radius with Nonaut Sub, Perc Approach */
"0PUH47Z"  =  "1"  /* Supplement Right Radius with Autol Sub, Perc Endo Approach */
"0PUH4JZ"  =  "1"  /* Supplement Right Radius with Synth Sub, Perc Endo Approach */
"0PUH4KZ"  =  "1"  /* Supplement Right Radius with Nonaut Sub, Perc Endo Approach */
"0PUJ07Z"  =  "1"  /* Supplement Left Radius with Autol Sub, Open Approach */
"0PUJ0JZ"  =  "1"  /* Supplement Left Radius with Synth Sub, Open Approach */
"0PUJ0KZ"  =  "1"  /* Supplement Left Radius with Nonaut Sub, Open Approach */
"0PUJ37Z"  =  "1"  /* Supplement Left Radius with Autol Sub, Perc Approach */
"0PUJ3JZ"  =  "1"  /* Supplement Left Radius with Synth Sub, Perc Approach */
"0PUJ3KZ"  =  "1"  /* Supplement Left Radius with Nonaut Sub, Perc Approach */
"0PUJ47Z"  =  "1"  /* Supplement Left Radius with Autol Sub, Perc Endo Approach */
"0PUJ4JZ"  =  "1"  /* Supplement Left Radius with Synth Sub, Perc Endo Approach */
"0PUJ4KZ"  =  "1"  /* Supplement Left Radius with Nonaut Sub, Perc Endo Approach */
"0PUK07Z"  =  "1"  /* Supplement Right Ulna with Autol Sub, Open Approach */
"0PUK0JZ"  =  "1"  /* Supplement Right Ulna with Synth Sub, Open Approach */
"0PUK0KZ"  =  "1"  /* Supplement Right Ulna with Nonaut Sub, Open Approach */
"0PUK37Z"  =  "1"  /* Supplement Right Ulna with Autol Sub, Perc Approach */
"0PUK3JZ"  =  "1"  /* Supplement Right Ulna with Synth Sub, Perc Approach */
"0PUK3KZ"  =  "1"  /* Supplement Right Ulna with Nonaut Sub, Perc Approach */
"0PUK47Z"  =  "1"  /* Supplement Right Ulna with Autol Sub, Perc Endo Approach */
"0PUK4JZ"  =  "1"  /* Supplement Right Ulna with Synth Sub, Perc Endo Approach */
"0PUK4KZ"  =  "1"  /* Supplement Right Ulna with Nonaut Sub, Perc Endo Approach */
"0PUL07Z"  =  "1"  /* Supplement Left Ulna with Autol Sub, Open Approach */
"0PUL0JZ"  =  "1"  /* Supplement Left Ulna with Synth Sub, Open Approach */
"0PUL0KZ"  =  "1"  /* Supplement Left Ulna with Nonaut Sub, Open Approach */
"0PUL37Z"  =  "1"  /* Supplement Left Ulna with Autol Sub, Perc Approach */
"0PUL3JZ"  =  "1"  /* Supplement Left Ulna with Synth Sub, Perc Approach */
"0PUL3KZ"  =  "1"  /* Supplement Left Ulna with Nonaut Sub, Perc Approach */
"0PUL47Z"  =  "1"  /* Supplement Left Ulna with Autol Sub, Perc Endo Approach */
"0PUL4JZ"  =  "1"  /* Supplement Left Ulna with Synth Sub, Perc Endo Approach */
"0PUL4KZ"  =  "1"  /* Supplement Left Ulna with Nonaut Sub, Perc Endo Approach */
"0PUM07Z"  =  "1"  /* Supplement Right Carpal with Autol Sub, Open Approach */
"0PUM0JZ"  =  "1"  /* Supplement Right Carpal with Synth Sub, Open Approach */
"0PUM0KZ"  =  "1"  /* Supplement Right Carpal with Nonaut Sub, Open Approach */
"0PUM37Z"  =  "1"  /* Supplement Right Carpal with Autol Sub, Perc Approach */
"0PUM3JZ"  =  "1"  /* Supplement Right Carpal with Synth Sub, Perc Approach */
"0PUM3KZ"  =  "1"  /* Supplement Right Carpal with Nonaut Sub, Perc Approach */
"0PUM47Z"  =  "1"  /* Supplement Right Carpal with Autol Sub, Perc Endo Approach */
"0PUM4JZ"  =  "1"  /* Supplement Right Carpal with Synth Sub, Perc Endo Approach */
"0PUM4KZ"  =  "1"  /* Supplement Right Carpal with Nonaut Sub, Perc Endo Approach */
"0PUN07Z"  =  "1"  /* Supplement Left Carpal with Autol Sub, Open Approach */
"0PUN0JZ"  =  "1"  /* Supplement Left Carpal with Synth Sub, Open Approach */
"0PUN0KZ"  =  "1"  /* Supplement Left Carpal with Nonaut Sub, Open Approach */
"0PUN37Z"  =  "1"  /* Supplement Left Carpal with Autol Sub, Perc Approach */
"0PUN3JZ"  =  "1"  /* Supplement Left Carpal with Synth Sub, Perc Approach */
"0PUN3KZ"  =  "1"  /* Supplement Left Carpal with Nonaut Sub, Perc Approach */
"0PUN47Z"  =  "1"  /* Supplement Left Carpal with Autol Sub, Perc Endo Approach */
"0PUN4JZ"  =  "1"  /* Supplement Left Carpal with Synth Sub, Perc Endo Approach */
"0PUN4KZ"  =  "1"  /* Supplement Left Carpal with Nonaut Sub, Perc Endo Approach */
"0PUP07Z"  =  "1"  /* Supplement Right Metacarpal with Autol Sub, Open Approach */
"0PUP0JZ"  =  "1"  /* Supplement Right Metacarpal with Synth Sub, Open Approach */
"0PUP0KZ"  =  "1"  /* Supplement Right Metacarpal with Nonaut Sub, Open Approach */
"0PUP37Z"  =  "1"  /* Supplement Right Metacarpal with Autol Sub, Perc Approach */
"0PUP3JZ"  =  "1"  /* Supplement Right Metacarpal with Synth Sub, Perc Approach */
"0PUP3KZ"  =  "1"  /* Supplement Right Metacarpal with Nonaut Sub, Perc Approach */
"0PUP47Z"  =  "1"  /* Supplement R Metacarpal with Autol Sub, Perc Endo Approach */
"0PUP4JZ"  =  "1"  /* Supplement R Metacarpal with Synth Sub, Perc Endo Approach */
"0PUP4KZ"  =  "1"  /* Supplement R Metacarpal with Nonaut Sub, Perc Endo Approach */
"0PUQ07Z"  =  "1"  /* Supplement Left Metacarpal with Autol Sub, Open Approach */
"0PUQ0JZ"  =  "1"  /* Supplement Left Metacarpal with Synth Sub, Open Approach */
"0PUQ0KZ"  =  "1"  /* Supplement Left Metacarpal with Nonaut Sub, Open Approach */
"0PUQ37Z"  =  "1"  /* Supplement Left Metacarpal with Autol Sub, Perc Approach */
"0PUQ3JZ"  =  "1"  /* Supplement Left Metacarpal with Synth Sub, Perc Approach */
"0PUQ3KZ"  =  "1"  /* Supplement Left Metacarpal with Nonaut Sub, Perc Approach */
"0PUQ47Z"  =  "1"  /* Supplement L Metacarpal with Autol Sub, Perc Endo Approach */
"0PUQ4JZ"  =  "1"  /* Supplement L Metacarpal with Synth Sub, Perc Endo Approach */
"0PUQ4KZ"  =  "1"  /* Supplement L Metacarpal with Nonaut Sub, Perc Endo Approach */
"0PUR07Z"  =  "1"  /* Supplement Right Thumb Phalanx with Autol Sub, Open Approach */
"0PUR0JZ"  =  "1"  /* Supplement Right Thumb Phalanx with Synth Sub, Open Approach */
"0PUR0KZ"  =  "1"  /* Supplement R Thumb Phalanx with Nonaut Sub, Open Approach */
"0PUR37Z"  =  "1"  /* Supplement Right Thumb Phalanx with Autol Sub, Perc Approach */
"0PUR3JZ"  =  "1"  /* Supplement Right Thumb Phalanx with Synth Sub, Perc Approach */
"0PUR3KZ"  =  "1"  /* Supplement R Thumb Phalanx with Nonaut Sub, Perc Approach */
"0PUR47Z"  =  "1"  /* Supplement R Thumb Phalanx w Autol Sub, Perc Endo */
"0PUR4JZ"  =  "1"  /* Supplement R Thumb Phalanx w Synth Sub, Perc Endo */
"0PUR4KZ"  =  "1"  /* Supplement R Thumb Phalanx w Nonaut Sub, Perc Endo */
"0PUS07Z"  =  "1"  /* Supplement Left Thumb Phalanx with Autol Sub, Open Approach */
"0PUS0JZ"  =  "1"  /* Supplement Left Thumb Phalanx with Synth Sub, Open Approach */
"0PUS0KZ"  =  "1"  /* Supplement Left Thumb Phalanx with Nonaut Sub, Open Approach */
"0PUS37Z"  =  "1"  /* Supplement Left Thumb Phalanx with Autol Sub, Perc Approach */
"0PUS3JZ"  =  "1"  /* Supplement Left Thumb Phalanx with Synth Sub, Perc Approach */
"0PUS3KZ"  =  "1"  /* Supplement Left Thumb Phalanx with Nonaut Sub, Perc Approach */
"0PUS47Z"  =  "1"  /* Supplement L Thumb Phalanx w Autol Sub, Perc Endo */
"0PUS4JZ"  =  "1"  /* Supplement L Thumb Phalanx w Synth Sub, Perc Endo */
"0PUS4KZ"  =  "1"  /* Supplement L Thumb Phalanx w Nonaut Sub, Perc Endo */
"0PUT07Z"  =  "1"  /* Supplement R Finger Phalanx with Autol Sub, Open Approach */
"0PUT0JZ"  =  "1"  /* Supplement R Finger Phalanx with Synth Sub, Open Approach */
"0PUT0KZ"  =  "1"  /* Supplement R Finger Phalanx with Nonaut Sub, Open Approach */
"0PUT37Z"  =  "1"  /* Supplement R Finger Phalanx with Autol Sub, Perc Approach */
"0PUT3JZ"  =  "1"  /* Supplement R Finger Phalanx with Synth Sub, Perc Approach */
"0PUT3KZ"  =  "1"  /* Supplement R Finger Phalanx with Nonaut Sub, Perc Approach */
"0PUT47Z"  =  "1"  /* Supplement R Finger Phalanx w Autol Sub, Perc Endo */
"0PUT4JZ"  =  "1"  /* Supplement R Finger Phalanx w Synth Sub, Perc Endo */
"0PUT4KZ"  =  "1"  /* Supplement R Finger Phalanx w Nonaut Sub, Perc Endo */
"0PUV07Z"  =  "1"  /* Supplement Left Finger Phalanx with Autol Sub, Open Approach */
"0PUV0JZ"  =  "1"  /* Supplement Left Finger Phalanx with Synth Sub, Open Approach */
"0PUV0KZ"  =  "1"  /* Supplement L Finger Phalanx with Nonaut Sub, Open Approach */
"0PUV37Z"  =  "1"  /* Supplement Left Finger Phalanx with Autol Sub, Perc Approach */
"0PUV3JZ"  =  "1"  /* Supplement Left Finger Phalanx with Synth Sub, Perc Approach */
"0PUV3KZ"  =  "1"  /* Supplement L Finger Phalanx with Nonaut Sub, Perc Approach */
"0PUV47Z"  =  "1"  /* Supplement L Finger Phalanx w Autol Sub, Perc Endo */
"0PUV4JZ"  =  "1"  /* Supplement L Finger Phalanx w Synth Sub, Perc Endo */
"0PUV4KZ"  =  "1"  /* Supplement L Finger Phalanx w Nonaut Sub, Perc Endo */
"0PW004Z"  =  "1"  /* Revision of Int Fix in Sternum, Open Approach */
"0PW007Z"  =  "1"  /* Revision of Autol Sub in Sternum, Open Approach */
"0PW00JZ"  =  "1"  /* Revision of Synthetic Substitute in Sternum, Open Approach */
"0PW00KZ"  =  "1"  /* Revision of Nonaut Sub in Sternum, Open Approach */
"0PW034Z"  =  "1"  /* Revision of Int Fix in Sternum, Perc Approach */
"0PW037Z"  =  "1"  /* Revision of Autol Sub in Sternum, Perc Approach */
"0PW03JZ"  =  "1"  /* Revision of Synthetic Substitute in Sternum, Perc Approach */
"0PW03KZ"  =  "1"  /* Revision of Nonaut Sub in Sternum, Perc Approach */
"0PW044Z"  =  "1"  /* Revision of Int Fix in Sternum, Perc Endo Approach */
"0PW047Z"  =  "1"  /* Revision of Autol Sub in Sternum, Perc Endo Approach */
"0PW04JZ"  =  "1"  /* Revision of Synth Sub in Sternum, Perc Endo Approach */
"0PW04KZ"  =  "1"  /* Revision of Nonaut Sub in Sternum, Perc Endo Approach */
"0PW104Z"  =  "1"  /* Revision of Internal Fixation Device in R Rib, Open Approach */
"0PW107Z"  =  "1"  /* Revision of Autol Sub in R Rib, Open Approach */
"0PW10JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Rib, Open Approach */
"0PW10KZ"  =  "1"  /* Revision of Nonaut Sub in R Rib, Open Approach */
"0PW134Z"  =  "1"  /* Revision of Internal Fixation Device in R Rib, Perc Approach */
"0PW137Z"  =  "1"  /* Revision of Autol Sub in R Rib, Perc Approach */
"0PW13JZ"  =  "1"  /* Revision of Synthetic Substitute in Right Rib, Perc Approach */
"0PW13KZ"  =  "1"  /* Revision of Nonaut Sub in R Rib, Perc Approach */
"0PW144Z"  =  "1"  /* Revision of Int Fix in R Rib, Perc Endo Approach */
"0PW147Z"  =  "1"  /* Revision of Autol Sub in R Rib, Perc Endo Approach */
"0PW14JZ"  =  "1"  /* Revision of Synth Sub in R Rib, Perc Endo Approach */
"0PW14KZ"  =  "1"  /* Revision of Nonaut Sub in R Rib, Perc Endo Approach */
"0PW204Z"  =  "1"  /* Revision of Internal Fixation Device in L Rib, Open Approach */
"0PW207Z"  =  "1"  /* Revision of Autol Sub in L Rib, Open Approach */
"0PW20JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Rib, Open Approach */
"0PW20KZ"  =  "1"  /* Revision of Nonaut Sub in L Rib, Open Approach */
"0PW234Z"  =  "1"  /* Revision of Internal Fixation Device in L Rib, Perc Approach */
"0PW237Z"  =  "1"  /* Revision of Autol Sub in L Rib, Perc Approach */
"0PW23JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Rib, Perc Approach */
"0PW23KZ"  =  "1"  /* Revision of Nonaut Sub in L Rib, Perc Approach */
"0PW244Z"  =  "1"  /* Revision of Int Fix in L Rib, Perc Endo Approach */
"0PW247Z"  =  "1"  /* Revision of Autol Sub in L Rib, Perc Endo Approach */
"0PW24JZ"  =  "1"  /* Revision of Synth Sub in L Rib, Perc Endo Approach */
"0PW24KZ"  =  "1"  /* Revision of Nonaut Sub in L Rib, Perc Endo Approach */
"0PW304Z"  =  "1"  /* Revision of Int Fix in Cervcal Vertebra, Open Approach */
"0PW307Z"  =  "1"  /* Revision of Autol Sub in Cervcal Vertebra, Open Approach */
"0PW30JZ"  =  "1"  /* Revision of Synth Sub in Cervcal Vertebra, Open Approach */
"0PW30KZ"  =  "1"  /* Revision of Nonaut Sub in Cervcal Vertebra, Open Approach */
"0PW334Z"  =  "1"  /* Revision of Int Fix in Cervcal Vertebra, Perc Approach */
"0PW337Z"  =  "1"  /* Revision of Autol Sub in Cervcal Vertebra, Perc Approach */
"0PW33JZ"  =  "1"  /* Revision of Synth Sub in Cervcal Vertebra, Perc Approach */
"0PW33KZ"  =  "1"  /* Revision of Nonaut Sub in Cervcal Vertebra, Perc Approach */
"0PW344Z"  =  "1"  /* Revision of Int Fix in Cervcal Vertebra, Perc Endo Approach */
"0PW347Z"  =  "1"  /* Revise of Autol Sub in Cervcal Vertebra, Perc Endo Approach */
"0PW34JZ"  =  "1"  /* Revise of Synth Sub in Cervcal Vertebra, Perc Endo Approach */
"0PW34KZ"  =  "1"  /* Revise of Nonaut Sub in Cervcal Vertebra, Perc Endo Approach */
"0PW404Z"  =  "1"  /* Revision of Int Fix in Thor Vertebra, Open Approach */
"0PW407Z"  =  "1"  /* Revision of Autol Sub in Thor Vertebra, Open Approach */
"0PW40JZ"  =  "1"  /* Revision of Synth Sub in Thor Vertebra, Open Approach */
"0PW40KZ"  =  "1"  /* Revision of Nonaut Sub in Thor Vertebra, Open Approach */
"0PW434Z"  =  "1"  /* Revision of Int Fix in Thor Vertebra, Perc Approach */
"0PW437Z"  =  "1"  /* Revision of Autol Sub in Thor Vertebra, Perc Approach */
"0PW43JZ"  =  "1"  /* Revision of Synth Sub in Thor Vertebra, Perc Approach */
"0PW43KZ"  =  "1"  /* Revision of Nonaut Sub in Thor Vertebra, Perc Approach */
"0PW444Z"  =  "1"  /* Revision of Int Fix in Thor Vertebra, Perc Endo Approach */
"0PW447Z"  =  "1"  /* Revision of Autol Sub in Thor Vertebra, Perc Endo Approach */
"0PW44JZ"  =  "1"  /* Revision of Synth Sub in Thor Vertebra, Perc Endo Approach */
"0PW44KZ"  =  "1"  /* Revision of Nonaut Sub in Thor Vertebra, Perc Endo Approach */
"0PW504Z"  =  "1"  /* Revision of Int Fix in R Scapula, Open Approach */
"0PW507Z"  =  "1"  /* Revision of Autol Sub in R Scapula, Open Approach */
"0PW50JZ"  =  "1"  /* Revision of Synthetic Substitute in R Scapula, Open Approach */
"0PW50KZ"  =  "1"  /* Revision of Nonaut Sub in R Scapula, Open Approach */
"0PW534Z"  =  "1"  /* Revision of Int Fix in R Scapula, Perc Approach */
"0PW537Z"  =  "1"  /* Revision of Autol Sub in R Scapula, Perc Approach */
"0PW53JZ"  =  "1"  /* Revision of Synthetic Substitute in R Scapula, Perc Approach */
"0PW53KZ"  =  "1"  /* Revision of Nonaut Sub in R Scapula, Perc Approach */
"0PW544Z"  =  "1"  /* Revision of Int Fix in R Scapula, Perc Endo Approach */
"0PW547Z"  =  "1"  /* Revision of Autol Sub in R Scapula, Perc Endo Approach */
"0PW54JZ"  =  "1"  /* Revision of Synth Sub in R Scapula, Perc Endo Approach */
"0PW54KZ"  =  "1"  /* Revision of Nonaut Sub in R Scapula, Perc Endo Approach */
"0PW604Z"  =  "1"  /* Revision of Int Fix in L Scapula, Open Approach */
"0PW607Z"  =  "1"  /* Revision of Autol Sub in L Scapula, Open Approach */
"0PW60JZ"  =  "1"  /* Revision of Synthetic Substitute in L Scapula, Open Approach */
"0PW60KZ"  =  "1"  /* Revision of Nonaut Sub in L Scapula, Open Approach */
"0PW634Z"  =  "1"  /* Revision of Int Fix in L Scapula, Perc Approach */
"0PW637Z"  =  "1"  /* Revision of Autol Sub in L Scapula, Perc Approach */
"0PW63JZ"  =  "1"  /* Revision of Synthetic Substitute in L Scapula, Perc Approach */
"0PW63KZ"  =  "1"  /* Revision of Nonaut Sub in L Scapula, Perc Approach */
"0PW644Z"  =  "1"  /* Revision of Int Fix in L Scapula, Perc Endo Approach */
"0PW647Z"  =  "1"  /* Revision of Autol Sub in L Scapula, Perc Endo Approach */
"0PW64JZ"  =  "1"  /* Revision of Synth Sub in L Scapula, Perc Endo Approach */
"0PW64KZ"  =  "1"  /* Revision of Nonaut Sub in L Scapula, Perc Endo Approach */
"0PW704Z"  =  "1"  /* Revision of Int Fix in R Glenoid Cav, Open Approach */
"0PW707Z"  =  "1"  /* Revision of Autol Sub in R Glenoid Cav, Open Approach */
"0PW70JZ"  =  "1"  /* Revision of Synth Sub in R Glenoid Cav, Open Approach */
"0PW70KZ"  =  "1"  /* Revision of Nonaut Sub in R Glenoid Cav, Open Approach */
"0PW734Z"  =  "1"  /* Revision of Int Fix in R Glenoid Cav, Perc Approach */
"0PW737Z"  =  "1"  /* Revision of Autol Sub in R Glenoid Cav, Perc Approach */
"0PW73JZ"  =  "1"  /* Revision of Synth Sub in R Glenoid Cav, Perc Approach */
"0PW73KZ"  =  "1"  /* Revision of Nonaut Sub in R Glenoid Cav, Perc Approach */
"0PW744Z"  =  "1"  /* Revision of Int Fix in R Glenoid Cav, Perc Endo Approach */
"0PW747Z"  =  "1"  /* Revision of Autol Sub in R Glenoid Cav, Perc Endo Approach */
"0PW74JZ"  =  "1"  /* Revision of Synth Sub in R Glenoid Cav, Perc Endo Approach */
"0PW74KZ"  =  "1"  /* Revision of Nonaut Sub in R Glenoid Cav, Perc Endo Approach */
"0PW804Z"  =  "1"  /* Revision of Int Fix in L Glenoid Cav, Open Approach */
"0PW807Z"  =  "1"  /* Revision of Autol Sub in L Glenoid Cav, Open Approach */
"0PW80JZ"  =  "1"  /* Revision of Synth Sub in L Glenoid Cav, Open Approach */
"0PW80KZ"  =  "1"  /* Revision of Nonaut Sub in L Glenoid Cav, Open Approach */
"0PW834Z"  =  "1"  /* Revision of Int Fix in L Glenoid Cav, Perc Approach */
"0PW837Z"  =  "1"  /* Revision of Autol Sub in L Glenoid Cav, Perc Approach */
"0PW83JZ"  =  "1"  /* Revision of Synth Sub in L Glenoid Cav, Perc Approach */
"0PW83KZ"  =  "1"  /* Revision of Nonaut Sub in L Glenoid Cav, Perc Approach */
"0PW844Z"  =  "1"  /* Revision of Int Fix in L Glenoid Cav, Perc Endo Approach */
"0PW847Z"  =  "1"  /* Revision of Autol Sub in L Glenoid Cav, Perc Endo Approach */
"0PW84JZ"  =  "1"  /* Revision of Synth Sub in L Glenoid Cav, Perc Endo Approach */
"0PW84KZ"  =  "1"  /* Revision of Nonaut Sub in L Glenoid Cav, Perc Endo Approach */
"0PW904Z"  =  "1"  /* Revision of Int Fix in R Clavicle, Open Approach */
"0PW907Z"  =  "1"  /* Revision of Autol Sub in R Clavicle, Open Approach */
"0PW90JZ"  =  "1"  /* Revision of Synth Sub in R Clavicle, Open Approach */
"0PW90KZ"  =  "1"  /* Revision of Nonaut Sub in R Clavicle, Open Approach */
"0PW934Z"  =  "1"  /* Revision of Int Fix in R Clavicle, Perc Approach */
"0PW937Z"  =  "1"  /* Revision of Autol Sub in R Clavicle, Perc Approach */
"0PW93JZ"  =  "1"  /* Revision of Synth Sub in R Clavicle, Perc Approach */
"0PW93KZ"  =  "1"  /* Revision of Nonaut Sub in R Clavicle, Perc Approach */
"0PW944Z"  =  "1"  /* Revision of Int Fix in R Clavicle, Perc Endo Approach */
"0PW947Z"  =  "1"  /* Revision of Autol Sub in R Clavicle, Perc Endo Approach */
"0PW94JZ"  =  "1"  /* Revision of Synth Sub in R Clavicle, Perc Endo Approach */
"0PW94KZ"  =  "1"  /* Revision of Nonaut Sub in R Clavicle, Perc Endo Approach */
"0PWB04Z"  =  "1"  /* Revision of Int Fix in L Clavicle, Open Approach */
"0PWB07Z"  =  "1"  /* Revision of Autol Sub in L Clavicle, Open Approach */
"0PWB0JZ"  =  "1"  /* Revision of Synth Sub in L Clavicle, Open Approach */
"0PWB0KZ"  =  "1"  /* Revision of Nonaut Sub in L Clavicle, Open Approach */
"0PWB34Z"  =  "1"  /* Revision of Int Fix in L Clavicle, Perc Approach */
"0PWB37Z"  =  "1"  /* Revision of Autol Sub in L Clavicle, Perc Approach */
"0PWB3JZ"  =  "1"  /* Revision of Synth Sub in L Clavicle, Perc Approach */
"0PWB3KZ"  =  "1"  /* Revision of Nonaut Sub in L Clavicle, Perc Approach */
"0PWB44Z"  =  "1"  /* Revision of Int Fix in L Clavicle, Perc Endo Approach */
"0PWB47Z"  =  "1"  /* Revision of Autol Sub in L Clavicle, Perc Endo Approach */
"0PWB4JZ"  =  "1"  /* Revision of Synth Sub in L Clavicle, Perc Endo Approach */
"0PWB4KZ"  =  "1"  /* Revision of Nonaut Sub in L Clavicle, Perc Endo Approach */
"0PWC04Z"  =  "1"  /* Revision of Int Fix in R Humeral Head, Open Approach */
"0PWC05Z"  =  "1"  /* Revision of Ext Fix in R Humeral Head, Open Approach */
"0PWC07Z"  =  "1"  /* Revision of Autol Sub in R Humeral Head, Open Approach */
"0PWC0JZ"  =  "1"  /* Revision of Synth Sub in R Humeral Head, Open Approach */
"0PWC0KZ"  =  "1"  /* Revision of Nonaut Sub in R Humeral Head, Open Approach */
"0PWC34Z"  =  "1"  /* Revision of Int Fix in R Humeral Head, Perc Approach */
"0PWC35Z"  =  "1"  /* Revision of Ext Fix in R Humeral Head, Perc Approach */
"0PWC37Z"  =  "1"  /* Revision of Autol Sub in R Humeral Head, Perc Approach */
"0PWC3JZ"  =  "1"  /* Revision of Synth Sub in R Humeral Head, Perc Approach */
"0PWC3KZ"  =  "1"  /* Revision of Nonaut Sub in R Humeral Head, Perc Approach */
"0PWC44Z"  =  "1"  /* Revision of Int Fix in R Humeral Head, Perc Endo Approach */
"0PWC45Z"  =  "1"  /* Revision of Ext Fix in R Humeral Head, Perc Endo Approach */
"0PWC47Z"  =  "1"  /* Revision of Autol Sub in R Humeral Head, Perc Endo Approach */
"0PWC4JZ"  =  "1"  /* Revision of Synth Sub in R Humeral Head, Perc Endo Approach */
"0PWC4KZ"  =  "1"  /* Revision of Nonaut Sub in R Humeral Head, Perc Endo Approach */
"0PWD04Z"  =  "1"  /* Revision of Int Fix in L Humeral Head, Open Approach */
"0PWD05Z"  =  "1"  /* Revision of Ext Fix in L Humeral Head, Open Approach */
"0PWD07Z"  =  "1"  /* Revision of Autol Sub in L Humeral Head, Open Approach */
"0PWD0JZ"  =  "1"  /* Revision of Synth Sub in L Humeral Head, Open Approach */
"0PWD0KZ"  =  "1"  /* Revision of Nonaut Sub in L Humeral Head, Open Approach */
"0PWD34Z"  =  "1"  /* Revision of Int Fix in L Humeral Head, Perc Approach */
"0PWD35Z"  =  "1"  /* Revision of Ext Fix in L Humeral Head, Perc Approach */
"0PWD37Z"  =  "1"  /* Revision of Autol Sub in L Humeral Head, Perc Approach */
"0PWD3JZ"  =  "1"  /* Revision of Synth Sub in L Humeral Head, Perc Approach */
"0PWD3KZ"  =  "1"  /* Revision of Nonaut Sub in L Humeral Head, Perc Approach */
"0PWD44Z"  =  "1"  /* Revision of Int Fix in L Humeral Head, Perc Endo Approach */
"0PWD45Z"  =  "1"  /* Revision of Ext Fix in L Humeral Head, Perc Endo Approach */
"0PWD47Z"  =  "1"  /* Revision of Autol Sub in L Humeral Head, Perc Endo Approach */
"0PWD4JZ"  =  "1"  /* Revision of Synth Sub in L Humeral Head, Perc Endo Approach */
"0PWD4KZ"  =  "1"  /* Revision of Nonaut Sub in L Humeral Head, Perc Endo Approach */
"0PWF04Z"  =  "1"  /* Revision of Int Fix in R Humeral Shaft, Open Approach */
"0PWF05Z"  =  "1"  /* Revision of Ext Fix in R Humeral Shaft, Open Approach */
"0PWF07Z"  =  "1"  /* Revision of Autol Sub in R Humeral Shaft, Open Approach */
"0PWF0JZ"  =  "1"  /* Revision of Synth Sub in R Humeral Shaft, Open Approach */
"0PWF0KZ"  =  "1"  /* Revision of Nonaut Sub in R Humeral Shaft, Open Approach */
"0PWF34Z"  =  "1"  /* Revision of Int Fix in R Humeral Shaft, Perc Approach */
"0PWF35Z"  =  "1"  /* Revision of Ext Fix in R Humeral Shaft, Perc Approach */
"0PWF37Z"  =  "1"  /* Revision of Autol Sub in R Humeral Shaft, Perc Approach */
"0PWF3JZ"  =  "1"  /* Revision of Synth Sub in R Humeral Shaft, Perc Approach */
"0PWF3KZ"  =  "1"  /* Revision of Nonaut Sub in R Humeral Shaft, Perc Approach */
"0PWF44Z"  =  "1"  /* Revision of Int Fix in R Humeral Shaft, Perc Endo Approach */
"0PWF45Z"  =  "1"  /* Revision of Ext Fix in R Humeral Shaft, Perc Endo Approach */
"0PWF47Z"  =  "1"  /* Revision of Autol Sub in R Humeral Shaft, Perc Endo Approach */
"0PWF4JZ"  =  "1"  /* Revision of Synth Sub in R Humeral Shaft, Perc Endo Approach */
"0PWF4KZ"  =  "1"  /* Revise of Nonaut Sub in R Humeral Shaft, Perc Endo Approach */
"0PWG04Z"  =  "1"  /* Revision of Int Fix in L Humeral Shaft, Open Approach */
"0PWG05Z"  =  "1"  /* Revision of Ext Fix in L Humeral Shaft, Open Approach */
"0PWG07Z"  =  "1"  /* Revision of Autol Sub in L Humeral Shaft, Open Approach */
"0PWG0JZ"  =  "1"  /* Revision of Synth Sub in L Humeral Shaft, Open Approach */
"0PWG0KZ"  =  "1"  /* Revision of Nonaut Sub in L Humeral Shaft, Open Approach */
"0PWG34Z"  =  "1"  /* Revision of Int Fix in L Humeral Shaft, Perc Approach */
"0PWG35Z"  =  "1"  /* Revision of Ext Fix in L Humeral Shaft, Perc Approach */
"0PWG37Z"  =  "1"  /* Revision of Autol Sub in L Humeral Shaft, Perc Approach */
"0PWG3JZ"  =  "1"  /* Revision of Synth Sub in L Humeral Shaft, Perc Approach */
"0PWG3KZ"  =  "1"  /* Revision of Nonaut Sub in L Humeral Shaft, Perc Approach */
"0PWG44Z"  =  "1"  /* Revision of Int Fix in L Humeral Shaft, Perc Endo Approach */
"0PWG45Z"  =  "1"  /* Revision of Ext Fix in L Humeral Shaft, Perc Endo Approach */
"0PWG47Z"  =  "1"  /* Revision of Autol Sub in L Humeral Shaft, Perc Endo Approach */
"0PWG4JZ"  =  "1"  /* Revision of Synth Sub in L Humeral Shaft, Perc Endo Approach */
"0PWG4KZ"  =  "1"  /* Revise of Nonaut Sub in L Humeral Shaft, Perc Endo Approach */
"0PWH04Z"  =  "1"  /* Revision of Int Fix in R Radius, Open Approach */
"0PWH05Z"  =  "1"  /* Revision of Ext Fix in R Radius, Open Approach */
"0PWH07Z"  =  "1"  /* Revision of Autol Sub in R Radius, Open Approach */
"0PWH0JZ"  =  "1"  /* Revision of Synthetic Substitute in R Radius, Open Approach */
"0PWH0KZ"  =  "1"  /* Revision of Nonaut Sub in R Radius, Open Approach */
"0PWH34Z"  =  "1"  /* Revision of Int Fix in R Radius, Perc Approach */
"0PWH35Z"  =  "1"  /* Revision of Ext Fix in R Radius, Perc Approach */
"0PWH37Z"  =  "1"  /* Revision of Autol Sub in R Radius, Perc Approach */
"0PWH3JZ"  =  "1"  /* Revision of Synthetic Substitute in R Radius, Perc Approach */
"0PWH3KZ"  =  "1"  /* Revision of Nonaut Sub in R Radius, Perc Approach */
"0PWH44Z"  =  "1"  /* Revision of Int Fix in R Radius, Perc Endo Approach */
"0PWH45Z"  =  "1"  /* Revision of Ext Fix in R Radius, Perc Endo Approach */
"0PWH47Z"  =  "1"  /* Revision of Autol Sub in R Radius, Perc Endo Approach */
"0PWH4JZ"  =  "1"  /* Revision of Synth Sub in R Radius, Perc Endo Approach */
"0PWH4KZ"  =  "1"  /* Revision of Nonaut Sub in R Radius, Perc Endo Approach */
"0PWJ04Z"  =  "1"  /* Revision of Int Fix in L Radius, Open Approach */
"0PWJ05Z"  =  "1"  /* Revision of Ext Fix in L Radius, Open Approach */
"0PWJ07Z"  =  "1"  /* Revision of Autol Sub in L Radius, Open Approach */
"0PWJ0JZ"  =  "1"  /* Revision of Synthetic Substitute in L Radius, Open Approach */
"0PWJ0KZ"  =  "1"  /* Revision of Nonaut Sub in L Radius, Open Approach */
"0PWJ34Z"  =  "1"  /* Revision of Int Fix in L Radius, Perc Approach */
"0PWJ35Z"  =  "1"  /* Revision of Ext Fix in L Radius, Perc Approach */
"0PWJ37Z"  =  "1"  /* Revision of Autol Sub in L Radius, Perc Approach */
"0PWJ3JZ"  =  "1"  /* Revision of Synthetic Substitute in L Radius, Perc Approach */
"0PWJ3KZ"  =  "1"  /* Revision of Nonaut Sub in L Radius, Perc Approach */
"0PWJ44Z"  =  "1"  /* Revision of Int Fix in L Radius, Perc Endo Approach */
"0PWJ45Z"  =  "1"  /* Revision of Ext Fix in L Radius, Perc Endo Approach */
"0PWJ47Z"  =  "1"  /* Revision of Autol Sub in L Radius, Perc Endo Approach */
"0PWJ4JZ"  =  "1"  /* Revision of Synth Sub in L Radius, Perc Endo Approach */
"0PWJ4KZ"  =  "1"  /* Revision of Nonaut Sub in L Radius, Perc Endo Approach */
"0PWK04Z"  =  "1"  /* Revision of Int Fix in R Ulna, Open Approach */
"0PWK05Z"  =  "1"  /* Revision of Ext Fix in R Ulna, Open Approach */
"0PWK07Z"  =  "1"  /* Revision of Autol Sub in R Ulna, Open Approach */
"0PWK0JZ"  =  "1"  /* Revision of Synthetic Substitute in R Ulna, Open Approach */
"0PWK0KZ"  =  "1"  /* Revision of Nonaut Sub in R Ulna, Open Approach */
"0PWK34Z"  =  "1"  /* Revision of Int Fix in R Ulna, Perc Approach */
"0PWK35Z"  =  "1"  /* Revision of Ext Fix in R Ulna, Perc Approach */
"0PWK37Z"  =  "1"  /* Revision of Autol Sub in R Ulna, Perc Approach */
"0PWK3JZ"  =  "1"  /* Revision of Synthetic Substitute in R Ulna, Perc Approach */
"0PWK3KZ"  =  "1"  /* Revision of Nonaut Sub in R Ulna, Perc Approach */
"0PWK44Z"  =  "1"  /* Revision of Int Fix in R Ulna, Perc Endo Approach */
"0PWK45Z"  =  "1"  /* Revision of Ext Fix in R Ulna, Perc Endo Approach */
"0PWK47Z"  =  "1"  /* Revision of Autol Sub in R Ulna, Perc Endo Approach */
"0PWK4JZ"  =  "1"  /* Revision of Synth Sub in R Ulna, Perc Endo Approach */
"0PWK4KZ"  =  "1"  /* Revision of Nonaut Sub in R Ulna, Perc Endo Approach */
"0PWL04Z"  =  "1"  /* Revision of Int Fix in L Ulna, Open Approach */
"0PWL05Z"  =  "1"  /* Revision of Ext Fix in L Ulna, Open Approach */
"0PWL07Z"  =  "1"  /* Revision of Autol Sub in L Ulna, Open Approach */
"0PWL0JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Ulna, Open Approach */
"0PWL0KZ"  =  "1"  /* Revision of Nonaut Sub in L Ulna, Open Approach */
"0PWL34Z"  =  "1"  /* Revision of Int Fix in L Ulna, Perc Approach */
"0PWL35Z"  =  "1"  /* Revision of Ext Fix in L Ulna, Perc Approach */
"0PWL37Z"  =  "1"  /* Revision of Autol Sub in L Ulna, Perc Approach */
"0PWL3JZ"  =  "1"  /* Revision of Synthetic Substitute in Left Ulna, Perc Approach */
"0PWL3KZ"  =  "1"  /* Revision of Nonaut Sub in L Ulna, Perc Approach */
"0PWL44Z"  =  "1"  /* Revision of Int Fix in L Ulna, Perc Endo Approach */
"0PWL45Z"  =  "1"  /* Revision of Ext Fix in L Ulna, Perc Endo Approach */
"0PWL47Z"  =  "1"  /* Revision of Autol Sub in L Ulna, Perc Endo Approach */
"0PWL4JZ"  =  "1"  /* Revision of Synth Sub in L Ulna, Perc Endo Approach */
"0PWL4KZ"  =  "1"  /* Revision of Nonaut Sub in L Ulna, Perc Endo Approach */
"0PWM04Z"  =  "1"  /* Revision of Int Fix in R Carpal, Open Approach */
"0PWM05Z"  =  "1"  /* Revision of Ext Fix in R Carpal, Open Approach */
"0PWM07Z"  =  "1"  /* Revision of Autol Sub in R Carpal, Open Approach */
"0PWM0JZ"  =  "1"  /* Revision of Synthetic Substitute in R Carpal, Open Approach */
"0PWM0KZ"  =  "1"  /* Revision of Nonaut Sub in R Carpal, Open Approach */
"0PWM34Z"  =  "1"  /* Revision of Int Fix in R Carpal, Perc Approach */
"0PWM35Z"  =  "1"  /* Revision of Ext Fix in R Carpal, Perc Approach */
"0PWM37Z"  =  "1"  /* Revision of Autol Sub in R Carpal, Perc Approach */
"0PWM3JZ"  =  "1"  /* Revision of Synthetic Substitute in R Carpal, Perc Approach */
"0PWM3KZ"  =  "1"  /* Revision of Nonaut Sub in R Carpal, Perc Approach */
"0PWM44Z"  =  "1"  /* Revision of Int Fix in R Carpal, Perc Endo Approach */
"0PWM45Z"  =  "1"  /* Revision of Ext Fix in R Carpal, Perc Endo Approach */
"0PWM47Z"  =  "1"  /* Revision of Autol Sub in R Carpal, Perc Endo Approach */
"0PWM4JZ"  =  "1"  /* Revision of Synth Sub in R Carpal, Perc Endo Approach */
"0PWM4KZ"  =  "1"  /* Revision of Nonaut Sub in R Carpal, Perc Endo Approach */
"0PWN04Z"  =  "1"  /* Revision of Int Fix in L Carpal, Open Approach */
"0PWN05Z"  =  "1"  /* Revision of Ext Fix in L Carpal, Open Approach */
"0PWN07Z"  =  "1"  /* Revision of Autol Sub in L Carpal, Open Approach */
"0PWN0JZ"  =  "1"  /* Revision of Synthetic Substitute in L Carpal, Open Approach */
"0PWN0KZ"  =  "1"  /* Revision of Nonaut Sub in L Carpal, Open Approach */
"0PWN34Z"  =  "1"  /* Revision of Int Fix in L Carpal, Perc Approach */
"0PWN35Z"  =  "1"  /* Revision of Ext Fix in L Carpal, Perc Approach */
"0PWN37Z"  =  "1"  /* Revision of Autol Sub in L Carpal, Perc Approach */
"0PWN3JZ"  =  "1"  /* Revision of Synthetic Substitute in L Carpal, Perc Approach */
"0PWN3KZ"  =  "1"  /* Revision of Nonaut Sub in L Carpal, Perc Approach */
"0PWN44Z"  =  "1"  /* Revision of Int Fix in L Carpal, Perc Endo Approach */
"0PWN45Z"  =  "1"  /* Revision of Ext Fix in L Carpal, Perc Endo Approach */
"0PWN47Z"  =  "1"  /* Revision of Autol Sub in L Carpal, Perc Endo Approach */
"0PWN4JZ"  =  "1"  /* Revision of Synth Sub in L Carpal, Perc Endo Approach */
"0PWN4KZ"  =  "1"  /* Revision of Nonaut Sub in L Carpal, Perc Endo Approach */
"0PWP04Z"  =  "1"  /* Revision of Int Fix in R Metacarpal, Open Approach */
"0PWP05Z"  =  "1"  /* Revision of Ext Fix in R Metacarpal, Open Approach */
"0PWP07Z"  =  "1"  /* Revision of Autol Sub in R Metacarpal, Open Approach */
"0PWP0JZ"  =  "1"  /* Revision of Synth Sub in R Metacarpal, Open Approach */
"0PWP0KZ"  =  "1"  /* Revision of Nonaut Sub in R Metacarpal, Open Approach */
"0PWP34Z"  =  "1"  /* Revision of Int Fix in R Metacarpal, Perc Approach */
"0PWP35Z"  =  "1"  /* Revision of Ext Fix in R Metacarpal, Perc Approach */
"0PWP37Z"  =  "1"  /* Revision of Autol Sub in R Metacarpal, Perc Approach */
"0PWP3JZ"  =  "1"  /* Revision of Synth Sub in R Metacarpal, Perc Approach */
"0PWP3KZ"  =  "1"  /* Revision of Nonaut Sub in R Metacarpal, Perc Approach */
"0PWP44Z"  =  "1"  /* Revision of Int Fix in R Metacarpal, Perc Endo Approach */
"0PWP45Z"  =  "1"  /* Revision of Ext Fix in R Metacarpal, Perc Endo Approach */
"0PWP47Z"  =  "1"  /* Revision of Autol Sub in R Metacarpal, Perc Endo Approach */
"0PWP4JZ"  =  "1"  /* Revision of Synth Sub in R Metacarpal, Perc Endo Approach */
"0PWP4KZ"  =  "1"  /* Revision of Nonaut Sub in R Metacarpal, Perc Endo Approach */
"0PWQ04Z"  =  "1"  /* Revision of Int Fix in L Metacarpal, Open Approach */
"0PWQ05Z"  =  "1"  /* Revision of Ext Fix in L Metacarpal, Open Approach */
"0PWQ07Z"  =  "1"  /* Revision of Autol Sub in L Metacarpal, Open Approach */
"0PWQ0JZ"  =  "1"  /* Revision of Synth Sub in L Metacarpal, Open Approach */
"0PWQ0KZ"  =  "1"  /* Revision of Nonaut Sub in L Metacarpal, Open Approach */
"0PWQ34Z"  =  "1"  /* Revision of Int Fix in L Metacarpal, Perc Approach */
"0PWQ35Z"  =  "1"  /* Revision of Ext Fix in L Metacarpal, Perc Approach */
"0PWQ37Z"  =  "1"  /* Revision of Autol Sub in L Metacarpal, Perc Approach */
"0PWQ3JZ"  =  "1"  /* Revision of Synth Sub in L Metacarpal, Perc Approach */
"0PWQ3KZ"  =  "1"  /* Revision of Nonaut Sub in L Metacarpal, Perc Approach */
"0PWQ44Z"  =  "1"  /* Revision of Int Fix in L Metacarpal, Perc Endo Approach */
"0PWQ45Z"  =  "1"  /* Revision of Ext Fix in L Metacarpal, Perc Endo Approach */
"0PWQ47Z"  =  "1"  /* Revision of Autol Sub in L Metacarpal, Perc Endo Approach */
"0PWQ4JZ"  =  "1"  /* Revision of Synth Sub in L Metacarpal, Perc Endo Approach */
"0PWQ4KZ"  =  "1"  /* Revision of Nonaut Sub in L Metacarpal, Perc Endo Approach */
"0PWR04Z"  =  "1"  /* Revision of Int Fix in R Thumb Phalanx, Open Approach */
"0PWR05Z"  =  "1"  /* Revision of Ext Fix in R Thumb Phalanx, Open Approach */
"0PWR07Z"  =  "1"  /* Revision of Autol Sub in R Thumb Phalanx, Open Approach */
"0PWR0JZ"  =  "1"  /* Revision of Synth Sub in R Thumb Phalanx, Open Approach */
"0PWR0KZ"  =  "1"  /* Revision of Nonaut Sub in R Thumb Phalanx, Open Approach */
"0PWR34Z"  =  "1"  /* Revision of Int Fix in R Thumb Phalanx, Perc Approach */
"0PWR35Z"  =  "1"  /* Revision of Ext Fix in R Thumb Phalanx, Perc Approach */
"0PWR37Z"  =  "1"  /* Revision of Autol Sub in R Thumb Phalanx, Perc Approach */
"0PWR3JZ"  =  "1"  /* Revision of Synth Sub in R Thumb Phalanx, Perc Approach */
"0PWR3KZ"  =  "1"  /* Revision of Nonaut Sub in R Thumb Phalanx, Perc Approach */
"0PWR44Z"  =  "1"  /* Revision of Int Fix in R Thumb Phalanx, Perc Endo Approach */
"0PWR45Z"  =  "1"  /* Revision of Ext Fix in R Thumb Phalanx, Perc Endo Approach */
"0PWR47Z"  =  "1"  /* Revision of Autol Sub in R Thumb Phalanx, Perc Endo Approach */
"0PWR4JZ"  =  "1"  /* Revision of Synth Sub in R Thumb Phalanx, Perc Endo Approach */
"0PWR4KZ"  =  "1"  /* Revise of Nonaut Sub in R Thumb Phalanx, Perc Endo Approach */
"0PWS04Z"  =  "1"  /* Revision of Int Fix in L Thumb Phalanx, Open Approach */
"0PWS05Z"  =  "1"  /* Revision of Ext Fix in L Thumb Phalanx, Open Approach */
"0PWS07Z"  =  "1"  /* Revision of Autol Sub in L Thumb Phalanx, Open Approach */
"0PWS0JZ"  =  "1"  /* Revision of Synth Sub in L Thumb Phalanx, Open Approach */
"0PWS0KZ"  =  "1"  /* Revision of Nonaut Sub in L Thumb Phalanx, Open Approach */
"0PWS34Z"  =  "1"  /* Revision of Int Fix in L Thumb Phalanx, Perc Approach */
"0PWS35Z"  =  "1"  /* Revision of Ext Fix in L Thumb Phalanx, Perc Approach */
"0PWS37Z"  =  "1"  /* Revision of Autol Sub in L Thumb Phalanx, Perc Approach */
"0PWS3JZ"  =  "1"  /* Revision of Synth Sub in L Thumb Phalanx, Perc Approach */
"0PWS3KZ"  =  "1"  /* Revision of Nonaut Sub in L Thumb Phalanx, Perc Approach */
"0PWS44Z"  =  "1"  /* Revision of Int Fix in L Thumb Phalanx, Perc Endo Approach */
"0PWS45Z"  =  "1"  /* Revision of Ext Fix in L Thumb Phalanx, Perc Endo Approach */
"0PWS47Z"  =  "1"  /* Revision of Autol Sub in L Thumb Phalanx, Perc Endo Approach */
"0PWS4JZ"  =  "1"  /* Revision of Synth Sub in L Thumb Phalanx, Perc Endo Approach */
"0PWS4KZ"  =  "1"  /* Revise of Nonaut Sub in L Thumb Phalanx, Perc Endo Approach */
"0PWT04Z"  =  "1"  /* Revision of Int Fix in R Finger Phalanx, Open Approach */
"0PWT05Z"  =  "1"  /* Revision of Ext Fix in R Finger Phalanx, Open Approach */
"0PWT07Z"  =  "1"  /* Revision of Autol Sub in R Finger Phalanx, Open Approach */
"0PWT0JZ"  =  "1"  /* Revision of Synth Sub in R Finger Phalanx, Open Approach */
"0PWT0KZ"  =  "1"  /* Revision of Nonaut Sub in R Finger Phalanx, Open Approach */
"0PWT34Z"  =  "1"  /* Revision of Int Fix in R Finger Phalanx, Perc Approach */
"0PWT35Z"  =  "1"  /* Revision of Ext Fix in R Finger Phalanx, Perc Approach */
"0PWT37Z"  =  "1"  /* Revision of Autol Sub in R Finger Phalanx, Perc Approach */
"0PWT3JZ"  =  "1"  /* Revision of Synth Sub in R Finger Phalanx, Perc Approach */
"0PWT3KZ"  =  "1"  /* Revision of Nonaut Sub in R Finger Phalanx, Perc Approach */
"0PWT44Z"  =  "1"  /* Revision of Int Fix in R Finger Phalanx, Perc Endo Approach */
"0PWT45Z"  =  "1"  /* Revision of Ext Fix in R Finger Phalanx, Perc Endo Approach */
"0PWT47Z"  =  "1"  /* Revise of Autol Sub in R Finger Phalanx, Perc Endo Approach */
"0PWT4JZ"  =  "1"  /* Revise of Synth Sub in R Finger Phalanx, Perc Endo Approach */
"0PWT4KZ"  =  "1"  /* Revise of Nonaut Sub in R Finger Phalanx, Perc Endo Approach */
"0PWV04Z"  =  "1"  /* Revision of Int Fix in L Finger Phalanx, Open Approach */
"0PWV05Z"  =  "1"  /* Revision of Ext Fix in L Finger Phalanx, Open Approach */
"0PWV07Z"  =  "1"  /* Revision of Autol Sub in L Finger Phalanx, Open Approach */
"0PWV0JZ"  =  "1"  /* Revision of Synth Sub in L Finger Phalanx, Open Approach */
"0PWV0KZ"  =  "1"  /* Revision of Nonaut Sub in L Finger Phalanx, Open Approach */
"0PWV34Z"  =  "1"  /* Revision of Int Fix in L Finger Phalanx, Perc Approach */
"0PWV35Z"  =  "1"  /* Revision of Ext Fix in L Finger Phalanx, Perc Approach */
"0PWV37Z"  =  "1"  /* Revision of Autol Sub in L Finger Phalanx, Perc Approach */
"0PWV3JZ"  =  "1"  /* Revision of Synth Sub in L Finger Phalanx, Perc Approach */
"0PWV3KZ"  =  "1"  /* Revision of Nonaut Sub in L Finger Phalanx, Perc Approach */
"0PWV44Z"  =  "1"  /* Revision of Int Fix in L Finger Phalanx, Perc Endo Approach */
"0PWV45Z"  =  "1"  /* Revision of Ext Fix in L Finger Phalanx, Perc Endo Approach */
"0PWV47Z"  =  "1"  /* Revise of Autol Sub in L Finger Phalanx, Perc Endo Approach */
"0PWV4JZ"  =  "1"  /* Revise of Synth Sub in L Finger Phalanx, Perc Endo Approach */
"0PWV4KZ"  =  "1"  /* Revise of Nonaut Sub in L Finger Phalanx, Perc Endo Approach */
"0PWY00Z"  =  "1"  /* Revision of Drainage Device in Upper Bone, Open Approach */
"0PWY0MZ"  =  "1"  /* Revision of Bone Growth Stimulator in Up Bone, Open Approach */
"0PWY30Z"  =  "1"  /* Revision of Drainage Device in Upper Bone, Perc Approach */
"0PWY3MZ"  =  "1"  /* Revision of Bone Growth Stimulator in Up Bone, Perc Approach */
"0PWY40Z"  =  "1"  /* Revision of Drainage Device in Up Bone, Perc Endo Approach */
"0PWY4MZ"  =  "1"  /* Revision of Bone Stim in Up Bone, Perc Endo Approach */
"0Q500ZZ"  =  "1"  /* Destruction of Lumbar Vertebra, Open Approach */
"0Q503ZZ"  =  "1"  /* Destruction of Lumbar Vertebra, Percutaneous Approach */
"0Q504ZZ"  =  "1"  /* Destruction of Lumbar Vertebra, Perc Endo Approach */
"0Q510ZZ"  =  "1"  /* Destruction of Sacrum, Open Approach */
"0Q513ZZ"  =  "1"  /* Destruction of Sacrum, Percutaneous Approach */
"0Q514ZZ"  =  "1"  /* Destruction of Sacrum, Percutaneous Endoscopic Approach */
"0Q520ZZ"  =  "1"  /* Destruction of Right Pelvic Bone, Open Approach */
"0Q523ZZ"  =  "1"  /* Destruction of Right Pelvic Bone, Percutaneous Approach */
"0Q524ZZ"  =  "1"  /* Destruction of Right Pelvic Bone, Perc Endo Approach */
"0Q530ZZ"  =  "1"  /* Destruction of Left Pelvic Bone, Open Approach */
"0Q533ZZ"  =  "1"  /* Destruction of Left Pelvic Bone, Percutaneous Approach */
"0Q534ZZ"  =  "1"  /* Destruction of Left Pelvic Bone, Perc Endo Approach */
"0Q540ZZ"  =  "1"  /* Destruction of Right Acetabulum, Open Approach */
"0Q543ZZ"  =  "1"  /* Destruction of Right Acetabulum, Percutaneous Approach */
"0Q544ZZ"  =  "1"  /* Destruction of Right Acetabulum, Perc Endo Approach */
"0Q550ZZ"  =  "1"  /* Destruction of Left Acetabulum, Open Approach */
"0Q553ZZ"  =  "1"  /* Destruction of Left Acetabulum, Percutaneous Approach */
"0Q554ZZ"  =  "1"  /* Destruction of Left Acetabulum, Perc Endo Approach */
"0Q560ZZ"  =  "1"  /* Destruction of Right Upper Femur, Open Approach */
"0Q563ZZ"  =  "1"  /* Destruction of Right Upper Femur, Percutaneous Approach */
"0Q564ZZ"  =  "1"  /* Destruction of Right Upper Femur, Perc Endo Approach */
"0Q570ZZ"  =  "1"  /* Destruction of Left Upper Femur, Open Approach */
"0Q573ZZ"  =  "1"  /* Destruction of Left Upper Femur, Percutaneous Approach */
"0Q574ZZ"  =  "1"  /* Destruction of Left Upper Femur, Perc Endo Approach */
"0Q580ZZ"  =  "1"  /* Destruction of Right Femoral Shaft, Open Approach */
"0Q583ZZ"  =  "1"  /* Destruction of Right Femoral Shaft, Percutaneous Approach */
"0Q584ZZ"  =  "1"  /* Destruction of Right Femoral Shaft, Perc Endo Approach */
"0Q590ZZ"  =  "1"  /* Destruction of Left Femoral Shaft, Open Approach */
"0Q593ZZ"  =  "1"  /* Destruction of Left Femoral Shaft, Percutaneous Approach */
"0Q594ZZ"  =  "1"  /* Destruction of Left Femoral Shaft, Perc Endo Approach */
"0Q5B0ZZ"  =  "1"  /* Destruction of Right Lower Femur, Open Approach */
"0Q5B3ZZ"  =  "1"  /* Destruction of Right Lower Femur, Percutaneous Approach */
"0Q5B4ZZ"  =  "1"  /* Destruction of Right Lower Femur, Perc Endo Approach */
"0Q5C0ZZ"  =  "1"  /* Destruction of Left Lower Femur, Open Approach */
"0Q5C3ZZ"  =  "1"  /* Destruction of Left Lower Femur, Percutaneous Approach */
"0Q5C4ZZ"  =  "1"  /* Destruction of Left Lower Femur, Perc Endo Approach */
"0Q5D0ZZ"  =  "1"  /* Destruction of Right Patella, Open Approach */
"0Q5D3ZZ"  =  "1"  /* Destruction of Right Patella, Percutaneous Approach */
"0Q5D4ZZ"  =  "1"  /* Destruction of Right Patella, Perc Endo Approach */
"0Q5F0ZZ"  =  "1"  /* Destruction of Left Patella, Open Approach */
"0Q5F3ZZ"  =  "1"  /* Destruction of Left Patella, Percutaneous Approach */
"0Q5F4ZZ"  =  "1"  /* Destruction of Left Patella, Perc Endo Approach */
"0Q5G0ZZ"  =  "1"  /* Destruction of Right Tibia, Open Approach */
"0Q5G3ZZ"  =  "1"  /* Destruction of Right Tibia, Percutaneous Approach */
"0Q5G4ZZ"  =  "1"  /* Destruction of Right Tibia, Percutaneous Endoscopic Approach */
"0Q5H0ZZ"  =  "1"  /* Destruction of Left Tibia, Open Approach */
"0Q5H3ZZ"  =  "1"  /* Destruction of Left Tibia, Percutaneous Approach */
"0Q5H4ZZ"  =  "1"  /* Destruction of Left Tibia, Percutaneous Endoscopic Approach */
"0Q5J0ZZ"  =  "1"  /* Destruction of Right Fibula, Open Approach */
"0Q5J3ZZ"  =  "1"  /* Destruction of Right Fibula, Percutaneous Approach */
"0Q5J4ZZ"  =  "1"  /* Destruction of Right Fibula, Perc Endo Approach */
"0Q5K0ZZ"  =  "1"  /* Destruction of Left Fibula, Open Approach */
"0Q5K3ZZ"  =  "1"  /* Destruction of Left Fibula, Percutaneous Approach */
"0Q5K4ZZ"  =  "1"  /* Destruction of Left Fibula, Percutaneous Endoscopic Approach */
"0Q5L0ZZ"  =  "1"  /* Destruction of Right Tarsal, Open Approach */
"0Q5L3ZZ"  =  "1"  /* Destruction of Right Tarsal, Percutaneous Approach */
"0Q5L4ZZ"  =  "1"  /* Destruction of Right Tarsal, Perc Endo Approach */
"0Q5M0ZZ"  =  "1"  /* Destruction of Left Tarsal, Open Approach */
"0Q5M3ZZ"  =  "1"  /* Destruction of Left Tarsal, Percutaneous Approach */
"0Q5M4ZZ"  =  "1"  /* Destruction of Left Tarsal, Percutaneous Endoscopic Approach */
"0Q5N0ZZ"  =  "1"  /* Destruction of Right Metatarsal, Open Approach */
"0Q5N3ZZ"  =  "1"  /* Destruction of Right Metatarsal, Percutaneous Approach */
"0Q5N4ZZ"  =  "1"  /* Destruction of Right Metatarsal, Perc Endo Approach */
"0Q5P0ZZ"  =  "1"  /* Destruction of Left Metatarsal, Open Approach */
"0Q5P3ZZ"  =  "1"  /* Destruction of Left Metatarsal, Percutaneous Approach */
"0Q5P4ZZ"  =  "1"  /* Destruction of Left Metatarsal, Perc Endo Approach */
"0Q5Q0ZZ"  =  "1"  /* Destruction of Right Toe Phalanx, Open Approach */
"0Q5Q3ZZ"  =  "1"  /* Destruction of Right Toe Phalanx, Percutaneous Approach */
"0Q5Q4ZZ"  =  "1"  /* Destruction of Right Toe Phalanx, Perc Endo Approach */
"0Q5R0ZZ"  =  "1"  /* Destruction of Left Toe Phalanx, Open Approach */
"0Q5R3ZZ"  =  "1"  /* Destruction of Left Toe Phalanx, Percutaneous Approach */
"0Q5R4ZZ"  =  "1"  /* Destruction of Left Toe Phalanx, Perc Endo Approach */
"0Q5S0ZZ"  =  "1"  /* Destruction of Coccyx, Open Approach */
"0Q5S3ZZ"  =  "1"  /* Destruction of Coccyx, Percutaneous Approach */
"0Q5S4ZZ"  =  "1"  /* Destruction of Coccyx, Percutaneous Endoscopic Approach */
"0Q800ZZ"  =  "1"  /* Division of Lumbar Vertebra, Open Approach */
"0Q803ZZ"  =  "1"  /* Division of Lumbar Vertebra, Percutaneous Approach */
"0Q804ZZ"  =  "1"  /* Division of Lumbar Vertebra, Perc Endo Approach */
"0Q810ZZ"  =  "1"  /* Division of Sacrum, Open Approach */
"0Q813ZZ"  =  "1"  /* Division of Sacrum, Percutaneous Approach */
"0Q814ZZ"  =  "1"  /* Division of Sacrum, Percutaneous Endoscopic Approach */
"0Q820ZZ"  =  "1"  /* Division of Right Pelvic Bone, Open Approach */
"0Q823ZZ"  =  "1"  /* Division of Right Pelvic Bone, Percutaneous Approach */
"0Q824ZZ"  =  "1"  /* Division of Right Pelvic Bone, Perc Endo Approach */
"0Q830ZZ"  =  "1"  /* Division of Left Pelvic Bone, Open Approach */
"0Q833ZZ"  =  "1"  /* Division of Left Pelvic Bone, Percutaneous Approach */
"0Q834ZZ"  =  "1"  /* Division of Left Pelvic Bone, Perc Endo Approach */
"0Q840ZZ"  =  "1"  /* Division of Right Acetabulum, Open Approach */
"0Q843ZZ"  =  "1"  /* Division of Right Acetabulum, Percutaneous Approach */
"0Q844ZZ"  =  "1"  /* Division of Right Acetabulum, Perc Endo Approach */
"0Q850ZZ"  =  "1"  /* Division of Left Acetabulum, Open Approach */
"0Q853ZZ"  =  "1"  /* Division of Left Acetabulum, Percutaneous Approach */
"0Q854ZZ"  =  "1"  /* Division of Left Acetabulum, Perc Endo Approach */
"0Q860ZZ"  =  "1"  /* Division of Right Upper Femur, Open Approach */
"0Q863ZZ"  =  "1"  /* Division of Right Upper Femur, Percutaneous Approach */
"0Q864ZZ"  =  "1"  /* Division of Right Upper Femur, Perc Endo Approach */
"0Q870ZZ"  =  "1"  /* Division of Left Upper Femur, Open Approach */
"0Q873ZZ"  =  "1"  /* Division of Left Upper Femur, Percutaneous Approach */
"0Q874ZZ"  =  "1"  /* Division of Left Upper Femur, Perc Endo Approach */
"0Q880ZZ"  =  "1"  /* Division of Right Femoral Shaft, Open Approach */
"0Q883ZZ"  =  "1"  /* Division of Right Femoral Shaft, Percutaneous Approach */
"0Q884ZZ"  =  "1"  /* Division of Right Femoral Shaft, Perc Endo Approach */
"0Q890ZZ"  =  "1"  /* Division of Left Femoral Shaft, Open Approach */
"0Q893ZZ"  =  "1"  /* Division of Left Femoral Shaft, Percutaneous Approach */
"0Q894ZZ"  =  "1"  /* Division of Left Femoral Shaft, Perc Endo Approach */
"0Q8B0ZZ"  =  "1"  /* Division of Right Lower Femur, Open Approach */
"0Q8B3ZZ"  =  "1"  /* Division of Right Lower Femur, Percutaneous Approach */
"0Q8B4ZZ"  =  "1"  /* Division of Right Lower Femur, Perc Endo Approach */
"0Q8C0ZZ"  =  "1"  /* Division of Left Lower Femur, Open Approach */
"0Q8C3ZZ"  =  "1"  /* Division of Left Lower Femur, Percutaneous Approach */
"0Q8C4ZZ"  =  "1"  /* Division of Left Lower Femur, Perc Endo Approach */
"0Q8D0ZZ"  =  "1"  /* Division of Right Patella, Open Approach */
"0Q8D3ZZ"  =  "1"  /* Division of Right Patella, Percutaneous Approach */
"0Q8D4ZZ"  =  "1"  /* Division of Right Patella, Percutaneous Endoscopic Approach */
"0Q8F0ZZ"  =  "1"  /* Division of Left Patella, Open Approach */
"0Q8F3ZZ"  =  "1"  /* Division of Left Patella, Percutaneous Approach */
"0Q8F4ZZ"  =  "1"  /* Division of Left Patella, Percutaneous Endoscopic Approach */
"0Q8G0ZZ"  =  "1"  /* Division of Right Tibia, Open Approach */
"0Q8G3ZZ"  =  "1"  /* Division of Right Tibia, Percutaneous Approach */
"0Q8G4ZZ"  =  "1"  /* Division of Right Tibia, Percutaneous Endoscopic Approach */
"0Q8H0ZZ"  =  "1"  /* Division of Left Tibia, Open Approach */
"0Q8H3ZZ"  =  "1"  /* Division of Left Tibia, Percutaneous Approach */
"0Q8H4ZZ"  =  "1"  /* Division of Left Tibia, Percutaneous Endoscopic Approach */
"0Q8J0ZZ"  =  "1"  /* Division of Right Fibula, Open Approach */
"0Q8J3ZZ"  =  "1"  /* Division of Right Fibula, Percutaneous Approach */
"0Q8J4ZZ"  =  "1"  /* Division of Right Fibula, Percutaneous Endoscopic Approach */
"0Q8K0ZZ"  =  "1"  /* Division of Left Fibula, Open Approach */
"0Q8K3ZZ"  =  "1"  /* Division of Left Fibula, Percutaneous Approach */
"0Q8K4ZZ"  =  "1"  /* Division of Left Fibula, Percutaneous Endoscopic Approach */
"0Q8L0ZZ"  =  "1"  /* Division of Right Tarsal, Open Approach */
"0Q8L3ZZ"  =  "1"  /* Division of Right Tarsal, Percutaneous Approach */
"0Q8L4ZZ"  =  "1"  /* Division of Right Tarsal, Percutaneous Endoscopic Approach */
"0Q8M0ZZ"  =  "1"  /* Division of Left Tarsal, Open Approach */
"0Q8M3ZZ"  =  "1"  /* Division of Left Tarsal, Percutaneous Approach */
"0Q8M4ZZ"  =  "1"  /* Division of Left Tarsal, Percutaneous Endoscopic Approach */
"0Q8N0ZZ"  =  "1"  /* Division of Right Metatarsal, Open Approach */
"0Q8N3ZZ"  =  "1"  /* Division of Right Metatarsal, Percutaneous Approach */
"0Q8N4ZZ"  =  "1"  /* Division of Right Metatarsal, Perc Endo Approach */
"0Q8P0ZZ"  =  "1"  /* Division of Left Metatarsal, Open Approach */
"0Q8P3ZZ"  =  "1"  /* Division of Left Metatarsal, Percutaneous Approach */
"0Q8P4ZZ"  =  "1"  /* Division of Left Metatarsal, Perc Endo Approach */
"0Q8Q0ZZ"  =  "1"  /* Division of Right Toe Phalanx, Open Approach */
"0Q8Q3ZZ"  =  "1"  /* Division of Right Toe Phalanx, Percutaneous Approach */
"0Q8Q4ZZ"  =  "1"  /* Division of Right Toe Phalanx, Perc Endo Approach */
"0Q8R0ZZ"  =  "1"  /* Division of Left Toe Phalanx, Open Approach */
"0Q8R3ZZ"  =  "1"  /* Division of Left Toe Phalanx, Percutaneous Approach */
"0Q8R4ZZ"  =  "1"  /* Division of Left Toe Phalanx, Perc Endo Approach */
"0Q8S0ZZ"  =  "1"  /* Division of Coccyx, Open Approach */
"0Q8S3ZZ"  =  "1"  /* Division of Coccyx, Percutaneous Approach */
"0Q8S4ZZ"  =  "1"  /* Division of Coccyx, Percutaneous Endoscopic Approach */
"0Q9000Z"  =  "1"  /* Drainage of Lumbar Vertebra with Drain Dev, Open Approach */
"0Q900ZX"  =  "1"  /* Drainage of Lumbar Vertebra, Open Approach, Diagnostic */
"0Q900ZZ"  =  "1"  /* Drainage of Lumbar Vertebra, Open Approach */
"0Q9030Z"  =  "1"  /* Drainage of Lumbar Vertebra with Drain Dev, Perc Approach */
"0Q903ZX"  =  "1"  /* Drainage of Lumbar Vertebra, Percutaneous Approach, Diagn */
"0Q903ZZ"  =  "1"  /* Drainage of Lumbar Vertebra, Percutaneous Approach */
"0Q9040Z"  =  "1"  /* Drainage of Lum Vertebra with Drain Dev, Perc Endo Approach */
"0Q904ZX"  =  "1"  /* Drainage of Lumbar Vertebra, Perc Endo Approach, Diagn */
"0Q904ZZ"  =  "1"  /* Drainage of Lumbar Vertebra, Perc Endo Approach */
"0Q9100Z"  =  "1"  /* Drainage of Sacrum with Drainage Device, Open Approach */
"0Q910ZX"  =  "1"  /* Drainage of Sacrum, Open Approach, Diagnostic */
"0Q910ZZ"  =  "1"  /* Drainage of Sacrum, Open Approach */
"0Q9130Z"  =  "1"  /* Drainage of Sacrum with Drainage Device, Perc Approach */
"0Q913ZX"  =  "1"  /* Drainage of Sacrum, Percutaneous Approach, Diagnostic */
"0Q913ZZ"  =  "1"  /* Drainage of Sacrum, Percutaneous Approach */
"0Q9140Z"  =  "1"  /* Drainage of Sacrum with Drainage Device, Perc Endo Approach */
"0Q914ZX"  =  "1"  /* Drainage of Sacrum, Percutaneous Endoscopic Approach, Diagn */
"0Q914ZZ"  =  "1"  /* Drainage of Sacrum, Percutaneous Endoscopic Approach */
"0Q9200Z"  =  "1"  /* Drainage of Right Pelvic Bone with Drain Dev, Open Approach */
"0Q920ZX"  =  "1"  /* Drainage of Right Pelvic Bone, Open Approach, Diagnostic */
"0Q920ZZ"  =  "1"  /* Drainage of Right Pelvic Bone, Open Approach */
"0Q9230Z"  =  "1"  /* Drainage of Right Pelvic Bone with Drain Dev, Perc Approach */
"0Q923ZX"  =  "1"  /* Drainage of Right Pelvic Bone, Percutaneous Approach, Diagn */
"0Q923ZZ"  =  "1"  /* Drainage of Right Pelvic Bone, Percutaneous Approach */
"0Q9240Z"  =  "1"  /* Drainage of R Pelvic Bone with Drain Dev, Perc Endo Approach */
"0Q924ZX"  =  "1"  /* Drainage of Right Pelvic Bone, Perc Endo Approach, Diagn */
"0Q924ZZ"  =  "1"  /* Drainage of Right Pelvic Bone, Perc Endo Approach */
"0Q9300Z"  =  "1"  /* Drainage of Left Pelvic Bone with Drain Dev, Open Approach */
"0Q930ZX"  =  "1"  /* Drainage of Left Pelvic Bone, Open Approach, Diagnostic */
"0Q930ZZ"  =  "1"  /* Drainage of Left Pelvic Bone, Open Approach */
"0Q9330Z"  =  "1"  /* Drainage of Left Pelvic Bone with Drain Dev, Perc Approach */
"0Q933ZX"  =  "1"  /* Drainage of Left Pelvic Bone, Percutaneous Approach, Diagn */
"0Q933ZZ"  =  "1"  /* Drainage of Left Pelvic Bone, Percutaneous Approach */
"0Q9340Z"  =  "1"  /* Drainage of L Pelvic Bone with Drain Dev, Perc Endo Approach */
"0Q934ZX"  =  "1"  /* Drainage of Left Pelvic Bone, Perc Endo Approach, Diagn */
"0Q934ZZ"  =  "1"  /* Drainage of Left Pelvic Bone, Perc Endo Approach */
"0Q9400Z"  =  "1"  /* Drainage of Right Acetabulum with Drain Dev, Open Approach */
"0Q940ZX"  =  "1"  /* Drainage of Right Acetabulum, Open Approach, Diagnostic */
"0Q940ZZ"  =  "1"  /* Drainage of Right Acetabulum, Open Approach */
"0Q9430Z"  =  "1"  /* Drainage of Right Acetabulum with Drain Dev, Perc Approach */
"0Q943ZX"  =  "1"  /* Drainage of Right Acetabulum, Percutaneous Approach, Diagn */
"0Q943ZZ"  =  "1"  /* Drainage of Right Acetabulum, Percutaneous Approach */
"0Q9440Z"  =  "1"  /* Drainage of R Acetabulum with Drain Dev, Perc Endo Approach */
"0Q944ZX"  =  "1"  /* Drainage of Right Acetabulum, Perc Endo Approach, Diagn */
"0Q944ZZ"  =  "1"  /* Drainage of Right Acetabulum, Perc Endo Approach */
"0Q9500Z"  =  "1"  /* Drainage of Left Acetabulum with Drain Dev, Open Approach */
"0Q950ZX"  =  "1"  /* Drainage of Left Acetabulum, Open Approach, Diagnostic */
"0Q950ZZ"  =  "1"  /* Drainage of Left Acetabulum, Open Approach */
"0Q9530Z"  =  "1"  /* Drainage of Left Acetabulum with Drain Dev, Perc Approach */
"0Q953ZX"  =  "1"  /* Drainage of Left Acetabulum, Percutaneous Approach, Diagn */
"0Q953ZZ"  =  "1"  /* Drainage of Left Acetabulum, Percutaneous Approach */
"0Q9540Z"  =  "1"  /* Drainage of L Acetabulum with Drain Dev, Perc Endo Approach */
"0Q954ZX"  =  "1"  /* Drainage of Left Acetabulum, Perc Endo Approach, Diagn */
"0Q954ZZ"  =  "1"  /* Drainage of Left Acetabulum, Perc Endo Approach */
"0Q9600Z"  =  "1"  /* Drainage of Right Upper Femur with Drain Dev, Open Approach */
"0Q960ZX"  =  "1"  /* Drainage of Right Upper Femur, Open Approach, Diagnostic */
"0Q960ZZ"  =  "1"  /* Drainage of Right Upper Femur, Open Approach */
"0Q9630Z"  =  "1"  /* Drainage of Right Upper Femur with Drain Dev, Perc Approach */
"0Q963ZX"  =  "1"  /* Drainage of Right Upper Femur, Percutaneous Approach, Diagn */
"0Q963ZZ"  =  "1"  /* Drainage of Right Upper Femur, Percutaneous Approach */
"0Q9640Z"  =  "1"  /* Drainage of R Up Femur with Drain Dev, Perc Endo Approach */
"0Q964ZX"  =  "1"  /* Drainage of Right Upper Femur, Perc Endo Approach, Diagn */
"0Q964ZZ"  =  "1"  /* Drainage of Right Upper Femur, Perc Endo Approach */
"0Q9700Z"  =  "1"  /* Drainage of Left Upper Femur with Drain Dev, Open Approach */
"0Q970ZX"  =  "1"  /* Drainage of Left Upper Femur, Open Approach, Diagnostic */
"0Q970ZZ"  =  "1"  /* Drainage of Left Upper Femur, Open Approach */
"0Q9730Z"  =  "1"  /* Drainage of Left Upper Femur with Drain Dev, Perc Approach */
"0Q973ZX"  =  "1"  /* Drainage of Left Upper Femur, Percutaneous Approach, Diagn */
"0Q973ZZ"  =  "1"  /* Drainage of Left Upper Femur, Percutaneous Approach */
"0Q9740Z"  =  "1"  /* Drainage of L Up Femur with Drain Dev, Perc Endo Approach */
"0Q974ZX"  =  "1"  /* Drainage of Left Upper Femur, Perc Endo Approach, Diagn */
"0Q974ZZ"  =  "1"  /* Drainage of Left Upper Femur, Perc Endo Approach */
"0Q9800Z"  =  "1"  /* Drainage of R Femur Shaft with Drain Dev, Open Approach */
"0Q980ZX"  =  "1"  /* Drainage of Right Femoral Shaft, Open Approach, Diagnostic */
"0Q980ZZ"  =  "1"  /* Drainage of Right Femoral Shaft, Open Approach */
"0Q9830Z"  =  "1"  /* Drainage of R Femur Shaft with Drain Dev, Perc Approach */
"0Q983ZX"  =  "1"  /* Drainage of Right Femoral Shaft, Perc Approach, Diagn */
"0Q983ZZ"  =  "1"  /* Drainage of Right Femoral Shaft, Percutaneous Approach */
"0Q9840Z"  =  "1"  /* Drainage of R Femur Shaft with Drain Dev, Perc Endo Approach */
"0Q984ZX"  =  "1"  /* Drainage of Right Femoral Shaft, Perc Endo Approach, Diagn */
"0Q984ZZ"  =  "1"  /* Drainage of Right Femoral Shaft, Perc Endo Approach */
"0Q9900Z"  =  "1"  /* Drainage of Left Femoral Shaft with Drain Dev, Open Approach */
"0Q990ZX"  =  "1"  /* Drainage of Left Femoral Shaft, Open Approach, Diagnostic */
"0Q990ZZ"  =  "1"  /* Drainage of Left Femoral Shaft, Open Approach */
"0Q9930Z"  =  "1"  /* Drainage of Left Femoral Shaft with Drain Dev, Perc Approach */
"0Q993ZX"  =  "1"  /* Drainage of Left Femoral Shaft, Percutaneous Approach, Diagn */
"0Q993ZZ"  =  "1"  /* Drainage of Left Femoral Shaft, Percutaneous Approach */
"0Q9940Z"  =  "1"  /* Drainage of L Femur Shaft with Drain Dev, Perc Endo Approach */
"0Q994ZX"  =  "1"  /* Drainage of Left Femoral Shaft, Perc Endo Approach, Diagn */
"0Q994ZZ"  =  "1"  /* Drainage of Left Femoral Shaft, Perc Endo Approach */
"0Q9B00Z"  =  "1"  /* Drainage of Right Lower Femur with Drain Dev, Open Approach */
"0Q9B0ZX"  =  "1"  /* Drainage of Right Lower Femur, Open Approach, Diagnostic */
"0Q9B0ZZ"  =  "1"  /* Drainage of Right Lower Femur, Open Approach */
"0Q9B30Z"  =  "1"  /* Drainage of Right Lower Femur with Drain Dev, Perc Approach */
"0Q9B3ZX"  =  "1"  /* Drainage of Right Lower Femur, Percutaneous Approach, Diagn */
"0Q9B3ZZ"  =  "1"  /* Drainage of Right Lower Femur, Percutaneous Approach */
"0Q9B40Z"  =  "1"  /* Drainage of R Low Femur with Drain Dev, Perc Endo Approach */
"0Q9B4ZX"  =  "1"  /* Drainage of Right Lower Femur, Perc Endo Approach, Diagn */
"0Q9B4ZZ"  =  "1"  /* Drainage of Right Lower Femur, Perc Endo Approach */
"0Q9C00Z"  =  "1"  /* Drainage of Left Lower Femur with Drain Dev, Open Approach */
"0Q9C0ZX"  =  "1"  /* Drainage of Left Lower Femur, Open Approach, Diagnostic */
"0Q9C0ZZ"  =  "1"  /* Drainage of Left Lower Femur, Open Approach */
"0Q9C30Z"  =  "1"  /* Drainage of Left Lower Femur with Drain Dev, Perc Approach */
"0Q9C3ZX"  =  "1"  /* Drainage of Left Lower Femur, Percutaneous Approach, Diagn */
"0Q9C3ZZ"  =  "1"  /* Drainage of Left Lower Femur, Percutaneous Approach */
"0Q9C40Z"  =  "1"  /* Drainage of L Low Femur with Drain Dev, Perc Endo Approach */
"0Q9C4ZX"  =  "1"  /* Drainage of Left Lower Femur, Perc Endo Approach, Diagn */
"0Q9C4ZZ"  =  "1"  /* Drainage of Left Lower Femur, Perc Endo Approach */
"0Q9D00Z"  =  "1"  /* Drainage of Right Patella with Drain Dev, Open Approach */
"0Q9D0ZX"  =  "1"  /* Drainage of Right Patella, Open Approach, Diagnostic */
"0Q9D0ZZ"  =  "1"  /* Drainage of Right Patella, Open Approach */
"0Q9D30Z"  =  "1"  /* Drainage of Right Patella with Drain Dev, Perc Approach */
"0Q9D3ZX"  =  "1"  /* Drainage of Right Patella, Percutaneous Approach, Diagnostic */
"0Q9D3ZZ"  =  "1"  /* Drainage of Right Patella, Percutaneous Approach */
"0Q9D40Z"  =  "1"  /* Drainage of Right Patella with Drain Dev, Perc Endo Approach */
"0Q9D4ZX"  =  "1"  /* Drainage of Right Patella, Perc Endo Approach, Diagn */
"0Q9D4ZZ"  =  "1"  /* Drainage of Right Patella, Percutaneous Endoscopic Approach */
"0Q9F00Z"  =  "1"  /* Drainage of Left Patella with Drainage Device, Open Approach */
"0Q9F0ZX"  =  "1"  /* Drainage of Left Patella, Open Approach, Diagnostic */
"0Q9F0ZZ"  =  "1"  /* Drainage of Left Patella, Open Approach */
"0Q9F30Z"  =  "1"  /* Drainage of Left Patella with Drainage Device, Perc Approach */
"0Q9F3ZX"  =  "1"  /* Drainage of Left Patella, Percutaneous Approach, Diagnostic */
"0Q9F3ZZ"  =  "1"  /* Drainage of Left Patella, Percutaneous Approach */
"0Q9F40Z"  =  "1"  /* Drainage of Left Patella with Drain Dev, Perc Endo Approach */
"0Q9F4ZX"  =  "1"  /* Drainage of Left Patella, Perc Endo Approach, Diagn */
"0Q9F4ZZ"  =  "1"  /* Drainage of Left Patella, Percutaneous Endoscopic Approach */
"0Q9G00Z"  =  "1"  /* Drainage of Right Tibia with Drainage Device, Open Approach */
"0Q9G0ZX"  =  "1"  /* Drainage of Right Tibia, Open Approach, Diagnostic */
"0Q9G0ZZ"  =  "1"  /* Drainage of Right Tibia, Open Approach */
"0Q9G30Z"  =  "1"  /* Drainage of Right Tibia with Drainage Device, Perc Approach */
"0Q9G3ZX"  =  "1"  /* Drainage of Right Tibia, Percutaneous Approach, Diagnostic */
"0Q9G3ZZ"  =  "1"  /* Drainage of Right Tibia, Percutaneous Approach */
"0Q9G40Z"  =  "1"  /* Drainage of Right Tibia with Drain Dev, Perc Endo Approach */
"0Q9G4ZX"  =  "1"  /* Drainage of Right Tibia, Perc Endo Approach, Diagn */
"0Q9G4ZZ"  =  "1"  /* Drainage of Right Tibia, Percutaneous Endoscopic Approach */
"0Q9H00Z"  =  "1"  /* Drainage of Left Tibia with Drainage Device, Open Approach */
"0Q9H0ZX"  =  "1"  /* Drainage of Left Tibia, Open Approach, Diagnostic */
"0Q9H0ZZ"  =  "1"  /* Drainage of Left Tibia, Open Approach */
"0Q9H30Z"  =  "1"  /* Drainage of Left Tibia with Drainage Device, Perc Approach */
"0Q9H3ZX"  =  "1"  /* Drainage of Left Tibia, Percutaneous Approach, Diagnostic */
"0Q9H3ZZ"  =  "1"  /* Drainage of Left Tibia, Percutaneous Approach */
"0Q9H40Z"  =  "1"  /* Drainage of Left Tibia with Drain Dev, Perc Endo Approach */
"0Q9H4ZX"  =  "1"  /* Drainage of Left Tibia, Perc Endo Approach, Diagn */
"0Q9H4ZZ"  =  "1"  /* Drainage of Left Tibia, Percutaneous Endoscopic Approach */
"0Q9J00Z"  =  "1"  /* Drainage of Right Fibula with Drainage Device, Open Approach */
"0Q9J0ZX"  =  "1"  /* Drainage of Right Fibula, Open Approach, Diagnostic */
"0Q9J0ZZ"  =  "1"  /* Drainage of Right Fibula, Open Approach */
"0Q9J30Z"  =  "1"  /* Drainage of Right Fibula with Drainage Device, Perc Approach */
"0Q9J3ZX"  =  "1"  /* Drainage of Right Fibula, Percutaneous Approach, Diagnostic */
"0Q9J3ZZ"  =  "1"  /* Drainage of Right Fibula, Percutaneous Approach */
"0Q9J40Z"  =  "1"  /* Drainage of Right Fibula with Drain Dev, Perc Endo Approach */
"0Q9J4ZX"  =  "1"  /* Drainage of Right Fibula, Perc Endo Approach, Diagn */
"0Q9J4ZZ"  =  "1"  /* Drainage of Right Fibula, Percutaneous Endoscopic Approach */
"0Q9K00Z"  =  "1"  /* Drainage of Left Fibula with Drainage Device, Open Approach */
"0Q9K0ZX"  =  "1"  /* Drainage of Left Fibula, Open Approach, Diagnostic */
"0Q9K0ZZ"  =  "1"  /* Drainage of Left Fibula, Open Approach */
"0Q9K30Z"  =  "1"  /* Drainage of Left Fibula with Drainage Device, Perc Approach */
"0Q9K3ZX"  =  "1"  /* Drainage of Left Fibula, Percutaneous Approach, Diagnostic */
"0Q9K3ZZ"  =  "1"  /* Drainage of Left Fibula, Percutaneous Approach */
"0Q9K40Z"  =  "1"  /* Drainage of Left Fibula with Drain Dev, Perc Endo Approach */
"0Q9K4ZX"  =  "1"  /* Drainage of Left Fibula, Perc Endo Approach, Diagn */
"0Q9K4ZZ"  =  "1"  /* Drainage of Left Fibula, Percutaneous Endoscopic Approach */
"0Q9L00Z"  =  "1"  /* Drainage of Right Tarsal with Drainage Device, Open Approach */
"0Q9L0ZX"  =  "1"  /* Drainage of Right Tarsal, Open Approach, Diagnostic */
"0Q9L0ZZ"  =  "1"  /* Drainage of Right Tarsal, Open Approach */
"0Q9L30Z"  =  "1"  /* Drainage of Right Tarsal with Drainage Device, Perc Approach */
"0Q9L3ZX"  =  "1"  /* Drainage of Right Tarsal, Percutaneous Approach, Diagnostic */
"0Q9L3ZZ"  =  "1"  /* Drainage of Right Tarsal, Percutaneous Approach */
"0Q9L40Z"  =  "1"  /* Drainage of Right Tarsal with Drain Dev, Perc Endo Approach */
"0Q9L4ZX"  =  "1"  /* Drainage of Right Tarsal, Perc Endo Approach, Diagn */
"0Q9L4ZZ"  =  "1"  /* Drainage of Right Tarsal, Percutaneous Endoscopic Approach */
"0Q9M00Z"  =  "1"  /* Drainage of Left Tarsal with Drainage Device, Open Approach */
"0Q9M0ZX"  =  "1"  /* Drainage of Left Tarsal, Open Approach, Diagnostic */
"0Q9M0ZZ"  =  "1"  /* Drainage of Left Tarsal, Open Approach */
"0Q9M30Z"  =  "1"  /* Drainage of Left Tarsal with Drainage Device, Perc Approach */
"0Q9M3ZX"  =  "1"  /* Drainage of Left Tarsal, Percutaneous Approach, Diagnostic */
"0Q9M3ZZ"  =  "1"  /* Drainage of Left Tarsal, Percutaneous Approach */
"0Q9M40Z"  =  "1"  /* Drainage of Left Tarsal with Drain Dev, Perc Endo Approach */
"0Q9M4ZX"  =  "1"  /* Drainage of Left Tarsal, Perc Endo Approach, Diagn */
"0Q9M4ZZ"  =  "1"  /* Drainage of Left Tarsal, Percutaneous Endoscopic Approach */
"0Q9N00Z"  =  "1"  /* Drainage of Right Metatarsal with Drain Dev, Open Approach */
"0Q9N0ZX"  =  "1"  /* Drainage of Right Metatarsal, Open Approach, Diagnostic */
"0Q9N0ZZ"  =  "1"  /* Drainage of Right Metatarsal, Open Approach */
"0Q9N30Z"  =  "1"  /* Drainage of Right Metatarsal with Drain Dev, Perc Approach */
"0Q9N3ZX"  =  "1"  /* Drainage of Right Metatarsal, Percutaneous Approach, Diagn */
"0Q9N3ZZ"  =  "1"  /* Drainage of Right Metatarsal, Percutaneous Approach */
"0Q9N40Z"  =  "1"  /* Drainage of R Metatarsal with Drain Dev, Perc Endo Approach */
"0Q9N4ZX"  =  "1"  /* Drainage of Right Metatarsal, Perc Endo Approach, Diagn */
"0Q9N4ZZ"  =  "1"  /* Drainage of Right Metatarsal, Perc Endo Approach */
"0Q9P00Z"  =  "1"  /* Drainage of Left Metatarsal with Drain Dev, Open Approach */
"0Q9P0ZX"  =  "1"  /* Drainage of Left Metatarsal, Open Approach, Diagnostic */
"0Q9P0ZZ"  =  "1"  /* Drainage of Left Metatarsal, Open Approach */
"0Q9P30Z"  =  "1"  /* Drainage of Left Metatarsal with Drain Dev, Perc Approach */
"0Q9P3ZX"  =  "1"  /* Drainage of Left Metatarsal, Percutaneous Approach, Diagn */
"0Q9P3ZZ"  =  "1"  /* Drainage of Left Metatarsal, Percutaneous Approach */
"0Q9P40Z"  =  "1"  /* Drainage of L Metatarsal with Drain Dev, Perc Endo Approach */
"0Q9P4ZX"  =  "1"  /* Drainage of Left Metatarsal, Perc Endo Approach, Diagn */
"0Q9P4ZZ"  =  "1"  /* Drainage of Left Metatarsal, Perc Endo Approach */
"0Q9Q00Z"  =  "1"  /* Drainage of Right Toe Phalanx with Drain Dev, Open Approach */
"0Q9Q0ZX"  =  "1"  /* Drainage of Right Toe Phalanx, Open Approach, Diagnostic */
"0Q9Q0ZZ"  =  "1"  /* Drainage of Right Toe Phalanx, Open Approach */
"0Q9Q30Z"  =  "1"  /* Drainage of Right Toe Phalanx with Drain Dev, Perc Approach */
"0Q9Q3ZX"  =  "1"  /* Drainage of Right Toe Phalanx, Percutaneous Approach, Diagn */
"0Q9Q3ZZ"  =  "1"  /* Drainage of Right Toe Phalanx, Percutaneous Approach */
"0Q9Q40Z"  =  "1"  /* Drainage of R Toe Phalanx with Drain Dev, Perc Endo Approach */
"0Q9Q4ZX"  =  "1"  /* Drainage of Right Toe Phalanx, Perc Endo Approach, Diagn */
"0Q9Q4ZZ"  =  "1"  /* Drainage of Right Toe Phalanx, Perc Endo Approach */
"0Q9R00Z"  =  "1"  /* Drainage of Left Toe Phalanx with Drain Dev, Open Approach */
"0Q9R0ZX"  =  "1"  /* Drainage of Left Toe Phalanx, Open Approach, Diagnostic */
"0Q9R0ZZ"  =  "1"  /* Drainage of Left Toe Phalanx, Open Approach */
"0Q9R30Z"  =  "1"  /* Drainage of Left Toe Phalanx with Drain Dev, Perc Approach */
"0Q9R3ZX"  =  "1"  /* Drainage of Left Toe Phalanx, Percutaneous Approach, Diagn */
"0Q9R3ZZ"  =  "1"  /* Drainage of Left Toe Phalanx, Percutaneous Approach */
"0Q9R40Z"  =  "1"  /* Drainage of L Toe Phalanx with Drain Dev, Perc Endo Approach */
"0Q9R4ZX"  =  "1"  /* Drainage of Left Toe Phalanx, Perc Endo Approach, Diagn */
"0Q9R4ZZ"  =  "1"  /* Drainage of Left Toe Phalanx, Perc Endo Approach */
"0Q9S00Z"  =  "1"  /* Drainage of Coccyx with Drainage Device, Open Approach */
"0Q9S0ZX"  =  "1"  /* Drainage of Coccyx, Open Approach, Diagnostic */
"0Q9S0ZZ"  =  "1"  /* Drainage of Coccyx, Open Approach */
"0Q9S30Z"  =  "1"  /* Drainage of Coccyx with Drainage Device, Perc Approach */
"0Q9S3ZX"  =  "1"  /* Drainage of Coccyx, Percutaneous Approach, Diagnostic */
"0Q9S3ZZ"  =  "1"  /* Drainage of Coccyx, Percutaneous Approach */
"0Q9S40Z"  =  "1"  /* Drainage of Coccyx with Drainage Device, Perc Endo Approach */
"0Q9S4ZX"  =  "1"  /* Drainage of Coccyx, Percutaneous Endoscopic Approach, Diagn */
"0Q9S4ZZ"  =  "1"  /* Drainage of Coccyx, Percutaneous Endoscopic Approach */
"0QB00ZX"  =  "1"  /* Excision of Lumbar Vertebra, Open Approach, Diagnostic */
"0QB00ZZ"  =  "1"  /* Excision of Lumbar Vertebra, Open Approach */
"0QB03ZX"  =  "1"  /* Excision of Lumbar Vertebra, Percutaneous Approach, Diagn */
"0QB03ZZ"  =  "1"  /* Excision of Lumbar Vertebra, Percutaneous Approach */
"0QB04ZX"  =  "1"  /* Excision of Lumbar Vertebra, Perc Endo Approach, Diagn */
"0QB04ZZ"  =  "1"  /* Excision of Lumbar Vertebra, Perc Endo Approach */
"0QB10ZX"  =  "1"  /* Excision of Sacrum, Open Approach, Diagnostic */
"0QB10ZZ"  =  "1"  /* Excision of Sacrum, Open Approach */
"0QB13ZX"  =  "1"  /* Excision of Sacrum, Percutaneous Approach, Diagnostic */
"0QB13ZZ"  =  "1"  /* Excision of Sacrum, Percutaneous Approach */
"0QB14ZX"  =  "1"  /* Excision of Sacrum, Percutaneous Endoscopic Approach, Diagn */
"0QB14ZZ"  =  "1"  /* Excision of Sacrum, Percutaneous Endoscopic Approach */
"0QB20ZX"  =  "1"  /* Excision of Right Pelvic Bone, Open Approach, Diagnostic */
"0QB20ZZ"  =  "1"  /* Excision of Right Pelvic Bone, Open Approach */
"0QB23ZX"  =  "1"  /* Excision of Right Pelvic Bone, Percutaneous Approach, Diagn */
"0QB23ZZ"  =  "1"  /* Excision of Right Pelvic Bone, Percutaneous Approach */
"0QB24ZX"  =  "1"  /* Excision of Right Pelvic Bone, Perc Endo Approach, Diagn */
"0QB24ZZ"  =  "1"  /* Excision of Right Pelvic Bone, Perc Endo Approach */
"0QB30ZX"  =  "1"  /* Excision of Left Pelvic Bone, Open Approach, Diagnostic */
"0QB30ZZ"  =  "1"  /* Excision of Left Pelvic Bone, Open Approach */
"0QB33ZX"  =  "1"  /* Excision of Left Pelvic Bone, Percutaneous Approach, Diagn */
"0QB33ZZ"  =  "1"  /* Excision of Left Pelvic Bone, Percutaneous Approach */
"0QB34ZX"  =  "1"  /* Excision of Left Pelvic Bone, Perc Endo Approach, Diagn */
"0QB34ZZ"  =  "1"  /* Excision of Left Pelvic Bone, Perc Endo Approach */
"0QB40ZX"  =  "1"  /* Excision of Right Acetabulum, Open Approach, Diagnostic */
"0QB40ZZ"  =  "1"  /* Excision of Right Acetabulum, Open Approach */
"0QB43ZX"  =  "1"  /* Excision of Right Acetabulum, Percutaneous Approach, Diagn */
"0QB43ZZ"  =  "1"  /* Excision of Right Acetabulum, Percutaneous Approach */
"0QB44ZX"  =  "1"  /* Excision of Right Acetabulum, Perc Endo Approach, Diagn */
"0QB44ZZ"  =  "1"  /* Excision of Right Acetabulum, Perc Endo Approach */
"0QB50ZX"  =  "1"  /* Excision of Left Acetabulum, Open Approach, Diagnostic */
"0QB50ZZ"  =  "1"  /* Excision of Left Acetabulum, Open Approach */
"0QB53ZX"  =  "1"  /* Excision of Left Acetabulum, Percutaneous Approach, Diagn */
"0QB53ZZ"  =  "1"  /* Excision of Left Acetabulum, Percutaneous Approach */
"0QB54ZX"  =  "1"  /* Excision of Left Acetabulum, Perc Endo Approach, Diagn */
"0QB54ZZ"  =  "1"  /* Excision of Left Acetabulum, Perc Endo Approach */
"0QB60ZX"  =  "1"  /* Excision of Right Upper Femur, Open Approach, Diagnostic */
"0QB60ZZ"  =  "1"  /* Excision of Right Upper Femur, Open Approach */
"0QB63ZX"  =  "1"  /* Excision of Right Upper Femur, Percutaneous Approach, Diagn */
"0QB63ZZ"  =  "1"  /* Excision of Right Upper Femur, Percutaneous Approach */
"0QB64ZX"  =  "1"  /* Excision of Right Upper Femur, Perc Endo Approach, Diagn */
"0QB64ZZ"  =  "1"  /* Excision of Right Upper Femur, Perc Endo Approach */
"0QB70ZX"  =  "1"  /* Excision of Left Upper Femur, Open Approach, Diagnostic */
"0QB70ZZ"  =  "1"  /* Excision of Left Upper Femur, Open Approach */
"0QB73ZX"  =  "1"  /* Excision of Left Upper Femur, Percutaneous Approach, Diagn */
"0QB73ZZ"  =  "1"  /* Excision of Left Upper Femur, Percutaneous Approach */
"0QB74ZX"  =  "1"  /* Excision of Left Upper Femur, Perc Endo Approach, Diagn */
"0QB74ZZ"  =  "1"  /* Excision of Left Upper Femur, Perc Endo Approach */
"0QB80ZX"  =  "1"  /* Excision of Right Femoral Shaft, Open Approach, Diagnostic */
"0QB80ZZ"  =  "1"  /* Excision of Right Femoral Shaft, Open Approach */
"0QB83ZX"  =  "1"  /* Excision of Right Femoral Shaft, Perc Approach, Diagn */
"0QB83ZZ"  =  "1"  /* Excision of Right Femoral Shaft, Percutaneous Approach */
"0QB84ZX"  =  "1"  /* Excision of Right Femoral Shaft, Perc Endo Approach, Diagn */
"0QB84ZZ"  =  "1"  /* Excision of Right Femoral Shaft, Perc Endo Approach */
"0QB90ZX"  =  "1"  /* Excision of Left Femoral Shaft, Open Approach, Diagnostic */
"0QB90ZZ"  =  "1"  /* Excision of Left Femoral Shaft, Open Approach */
"0QB93ZX"  =  "1"  /* Excision of Left Femoral Shaft, Percutaneous Approach, Diagn */
"0QB93ZZ"  =  "1"  /* Excision of Left Femoral Shaft, Percutaneous Approach */
"0QB94ZX"  =  "1"  /* Excision of Left Femoral Shaft, Perc Endo Approach, Diagn */
"0QB94ZZ"  =  "1"  /* Excision of Left Femoral Shaft, Perc Endo Approach */
"0QBB0ZX"  =  "1"  /* Excision of Right Lower Femur, Open Approach, Diagnostic */
"0QBB0ZZ"  =  "1"  /* Excision of Right Lower Femur, Open Approach */
"0QBB3ZX"  =  "1"  /* Excision of Right Lower Femur, Percutaneous Approach, Diagn */
"0QBB3ZZ"  =  "1"  /* Excision of Right Lower Femur, Percutaneous Approach */
"0QBB4ZX"  =  "1"  /* Excision of Right Lower Femur, Perc Endo Approach, Diagn */
"0QBB4ZZ"  =  "1"  /* Excision of Right Lower Femur, Perc Endo Approach */
"0QBC0ZX"  =  "1"  /* Excision of Left Lower Femur, Open Approach, Diagnostic */
"0QBC0ZZ"  =  "1"  /* Excision of Left Lower Femur, Open Approach */
"0QBC3ZX"  =  "1"  /* Excision of Left Lower Femur, Percutaneous Approach, Diagn */
"0QBC3ZZ"  =  "1"  /* Excision of Left Lower Femur, Percutaneous Approach */
"0QBC4ZX"  =  "1"  /* Excision of Left Lower Femur, Perc Endo Approach, Diagn */
"0QBC4ZZ"  =  "1"  /* Excision of Left Lower Femur, Perc Endo Approach */
"0QBD0ZX"  =  "1"  /* Excision of Right Patella, Open Approach, Diagnostic */
"0QBD0ZZ"  =  "1"  /* Excision of Right Patella, Open Approach */
"0QBD3ZX"  =  "1"  /* Excision of Right Patella, Percutaneous Approach, Diagnostic */
"0QBD3ZZ"  =  "1"  /* Excision of Right Patella, Percutaneous Approach */
"0QBD4ZX"  =  "1"  /* Excision of Right Patella, Perc Endo Approach, Diagn */
"0QBD4ZZ"  =  "1"  /* Excision of Right Patella, Percutaneous Endoscopic Approach */
"0QBF0ZX"  =  "1"  /* Excision of Left Patella, Open Approach, Diagnostic */
"0QBF0ZZ"  =  "1"  /* Excision of Left Patella, Open Approach */
"0QBF3ZX"  =  "1"  /* Excision of Left Patella, Percutaneous Approach, Diagnostic */
"0QBF3ZZ"  =  "1"  /* Excision of Left Patella, Percutaneous Approach */
"0QBF4ZX"  =  "1"  /* Excision of Left Patella, Perc Endo Approach, Diagn */
"0QBF4ZZ"  =  "1"  /* Excision of Left Patella, Percutaneous Endoscopic Approach */
"0QBG0ZX"  =  "1"  /* Excision of Right Tibia, Open Approach, Diagnostic */
"0QBG0ZZ"  =  "1"  /* Excision of Right Tibia, Open Approach */
"0QBG3ZX"  =  "1"  /* Excision of Right Tibia, Percutaneous Approach, Diagnostic */
"0QBG3ZZ"  =  "1"  /* Excision of Right Tibia, Percutaneous Approach */
"0QBG4ZX"  =  "1"  /* Excision of Right Tibia, Perc Endo Approach, Diagn */
"0QBG4ZZ"  =  "1"  /* Excision of Right Tibia, Percutaneous Endoscopic Approach */
"0QBH0ZX"  =  "1"  /* Excision of Left Tibia, Open Approach, Diagnostic */
"0QBH0ZZ"  =  "1"  /* Excision of Left Tibia, Open Approach */
"0QBH3ZX"  =  "1"  /* Excision of Left Tibia, Percutaneous Approach, Diagnostic */
"0QBH3ZZ"  =  "1"  /* Excision of Left Tibia, Percutaneous Approach */
"0QBH4ZX"  =  "1"  /* Excision of Left Tibia, Perc Endo Approach, Diagn */
"0QBH4ZZ"  =  "1"  /* Excision of Left Tibia, Percutaneous Endoscopic Approach */
"0QBJ0ZX"  =  "1"  /* Excision of Right Fibula, Open Approach, Diagnostic */
"0QBJ0ZZ"  =  "1"  /* Excision of Right Fibula, Open Approach */
"0QBJ3ZX"  =  "1"  /* Excision of Right Fibula, Percutaneous Approach, Diagnostic */
"0QBJ3ZZ"  =  "1"  /* Excision of Right Fibula, Percutaneous Approach */
"0QBJ4ZX"  =  "1"  /* Excision of Right Fibula, Perc Endo Approach, Diagn */
"0QBJ4ZZ"  =  "1"  /* Excision of Right Fibula, Percutaneous Endoscopic Approach */
"0QBK0ZX"  =  "1"  /* Excision of Left Fibula, Open Approach, Diagnostic */
"0QBK0ZZ"  =  "1"  /* Excision of Left Fibula, Open Approach */
"0QBK3ZX"  =  "1"  /* Excision of Left Fibula, Percutaneous Approach, Diagnostic */
"0QBK3ZZ"  =  "1"  /* Excision of Left Fibula, Percutaneous Approach */
"0QBK4ZX"  =  "1"  /* Excision of Left Fibula, Perc Endo Approach, Diagn */
"0QBK4ZZ"  =  "1"  /* Excision of Left Fibula, Percutaneous Endoscopic Approach */
"0QBL0ZX"  =  "1"  /* Excision of Right Tarsal, Open Approach, Diagnostic */
"0QBL0ZZ"  =  "1"  /* Excision of Right Tarsal, Open Approach */
"0QBL3ZX"  =  "1"  /* Excision of Right Tarsal, Percutaneous Approach, Diagnostic */
"0QBL3ZZ"  =  "1"  /* Excision of Right Tarsal, Percutaneous Approach */
"0QBL4ZX"  =  "1"  /* Excision of Right Tarsal, Perc Endo Approach, Diagn */
"0QBL4ZZ"  =  "1"  /* Excision of Right Tarsal, Percutaneous Endoscopic Approach */
"0QBM0ZX"  =  "1"  /* Excision of Left Tarsal, Open Approach, Diagnostic */
"0QBM0ZZ"  =  "1"  /* Excision of Left Tarsal, Open Approach */
"0QBM3ZX"  =  "1"  /* Excision of Left Tarsal, Percutaneous Approach, Diagnostic */
"0QBM3ZZ"  =  "1"  /* Excision of Left Tarsal, Percutaneous Approach */
"0QBM4ZX"  =  "1"  /* Excision of Left Tarsal, Perc Endo Approach, Diagn */
"0QBM4ZZ"  =  "1"  /* Excision of Left Tarsal, Percutaneous Endoscopic Approach */
"0QBN0ZX"  =  "1"  /* Excision of Right Metatarsal, Open Approach, Diagnostic */
"0QBN0ZZ"  =  "1"  /* Excision of Right Metatarsal, Open Approach */
"0QBN3ZX"  =  "1"  /* Excision of Right Metatarsal, Percutaneous Approach, Diagn */
"0QBN3ZZ"  =  "1"  /* Excision of Right Metatarsal, Percutaneous Approach */
"0QBN4ZX"  =  "1"  /* Excision of Right Metatarsal, Perc Endo Approach, Diagn */
"0QBN4ZZ"  =  "1"  /* Excision of Right Metatarsal, Perc Endo Approach */
"0QBP0ZX"  =  "1"  /* Excision of Left Metatarsal, Open Approach, Diagnostic */
"0QBP0ZZ"  =  "1"  /* Excision of Left Metatarsal, Open Approach */
"0QBP3ZX"  =  "1"  /* Excision of Left Metatarsal, Percutaneous Approach, Diagn */
"0QBP3ZZ"  =  "1"  /* Excision of Left Metatarsal, Percutaneous Approach */
"0QBP4ZX"  =  "1"  /* Excision of Left Metatarsal, Perc Endo Approach, Diagn */
"0QBP4ZZ"  =  "1"  /* Excision of Left Metatarsal, Perc Endo Approach */
"0QBQ0ZX"  =  "1"  /* Excision of Right Toe Phalanx, Open Approach, Diagnostic */
"0QBQ0ZZ"  =  "1"  /* Excision of Right Toe Phalanx, Open Approach */
"0QBQ3ZX"  =  "1"  /* Excision of Right Toe Phalanx, Percutaneous Approach, Diagn */
"0QBQ3ZZ"  =  "1"  /* Excision of Right Toe Phalanx, Percutaneous Approach */
"0QBQ4ZX"  =  "1"  /* Excision of Right Toe Phalanx, Perc Endo Approach, Diagn */
"0QBQ4ZZ"  =  "1"  /* Excision of Right Toe Phalanx, Perc Endo Approach */
"0QBR0ZX"  =  "1"  /* Excision of Left Toe Phalanx, Open Approach, Diagnostic */
"0QBR0ZZ"  =  "1"  /* Excision of Left Toe Phalanx, Open Approach */
"0QBR3ZX"  =  "1"  /* Excision of Left Toe Phalanx, Percutaneous Approach, Diagn */
"0QBR3ZZ"  =  "1"  /* Excision of Left Toe Phalanx, Percutaneous Approach */
"0QBR4ZX"  =  "1"  /* Excision of Left Toe Phalanx, Perc Endo Approach, Diagn */
"0QBR4ZZ"  =  "1"  /* Excision of Left Toe Phalanx, Perc Endo Approach */
"0QBS0ZX"  =  "1"  /* Excision of Coccyx, Open Approach, Diagnostic */
"0QBS0ZZ"  =  "1"  /* Excision of Coccyx, Open Approach */
"0QBS3ZX"  =  "1"  /* Excision of Coccyx, Percutaneous Approach, Diagnostic */
"0QBS3ZZ"  =  "1"  /* Excision of Coccyx, Percutaneous Approach */
"0QBS4ZX"  =  "1"  /* Excision of Coccyx, Percutaneous Endoscopic Approach, Diagn */
"0QBS4ZZ"  =  "1"  /* Excision of Coccyx, Percutaneous Endoscopic Approach */
"0QC00ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Vertebra, Open Approach */
"0QC03ZZ"  =  "1"  /* Extirpation of Matter from Lumbar Vertebra, Perc Approach */
"0QC04ZZ"  =  "1"  /* Extirpation of Matter from Lum Vertebra, Perc Endo Approach */
"0QC10ZZ"  =  "1"  /* Extirpation of Matter from Sacrum, Open Approach */
"0QC13ZZ"  =  "1"  /* Extirpation of Matter from Sacrum, Percutaneous Approach */
"0QC14ZZ"  =  "1"  /* Extirpation of Matter from Sacrum, Perc Endo Approach */
"0QC20ZZ"  =  "1"  /* Extirpation of Matter from Right Pelvic Bone, Open Approach */
"0QC23ZZ"  =  "1"  /* Extirpation of Matter from Right Pelvic Bone, Perc Approach */
"0QC24ZZ"  =  "1"  /* Extirpation of Matter from R Pelvic Bone, Perc Endo Approach */
"0QC30ZZ"  =  "1"  /* Extirpation of Matter from Left Pelvic Bone, Open Approach */
"0QC33ZZ"  =  "1"  /* Extirpation of Matter from Left Pelvic Bone, Perc Approach */
"0QC34ZZ"  =  "1"  /* Extirpation of Matter from L Pelvic Bone, Perc Endo Approach */
"0QC40ZZ"  =  "1"  /* Extirpation of Matter from Right Acetabulum, Open Approach */
"0QC43ZZ"  =  "1"  /* Extirpation of Matter from Right Acetabulum, Perc Approach */
"0QC44ZZ"  =  "1"  /* Extirpation of Matter from R Acetabulum, Perc Endo Approach */
"0QC50ZZ"  =  "1"  /* Extirpation of Matter from Left Acetabulum, Open Approach */
"0QC53ZZ"  =  "1"  /* Extirpation of Matter from Left Acetabulum, Perc Approach */
"0QC54ZZ"  =  "1"  /* Extirpation of Matter from L Acetabulum, Perc Endo Approach */
"0QC60ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Femur, Open Approach */
"0QC63ZZ"  =  "1"  /* Extirpation of Matter from Right Upper Femur, Perc Approach */
"0QC64ZZ"  =  "1"  /* Extirpation of Matter from R Up Femur, Perc Endo Approach */
"0QC70ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Femur, Open Approach */
"0QC73ZZ"  =  "1"  /* Extirpation of Matter from Left Upper Femur, Perc Approach */
"0QC74ZZ"  =  "1"  /* Extirpation of Matter from L Up Femur, Perc Endo Approach */
"0QC80ZZ"  =  "1"  /* Extirpation of Matter from R Femur Shaft, Open Approach */
"0QC83ZZ"  =  "1"  /* Extirpation of Matter from R Femur Shaft, Perc Approach */
"0QC84ZZ"  =  "1"  /* Extirpation of Matter from R Femur Shaft, Perc Endo Approach */
"0QC90ZZ"  =  "1"  /* Extirpation of Matter from Left Femoral Shaft, Open Approach */
"0QC93ZZ"  =  "1"  /* Extirpation of Matter from Left Femoral Shaft, Perc Approach */
"0QC94ZZ"  =  "1"  /* Extirpation of Matter from L Femur Shaft, Perc Endo Approach */
"0QCB0ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Femur, Open Approach */
"0QCB3ZZ"  =  "1"  /* Extirpation of Matter from Right Lower Femur, Perc Approach */
"0QCB4ZZ"  =  "1"  /* Extirpation of Matter from R Low Femur, Perc Endo Approach */
"0QCC0ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Femur, Open Approach */
"0QCC3ZZ"  =  "1"  /* Extirpation of Matter from Left Lower Femur, Perc Approach */
"0QCC4ZZ"  =  "1"  /* Extirpation of Matter from L Low Femur, Perc Endo Approach */
"0QCD0ZZ"  =  "1"  /* Extirpation of Matter from Right Patella, Open Approach */
"0QCD3ZZ"  =  "1"  /* Extirpation of Matter from Right Patella, Perc Approach */
"0QCD4ZZ"  =  "1"  /* Extirpation of Matter from Right Patella, Perc Endo Approach */
"0QCF0ZZ"  =  "1"  /* Extirpation of Matter from Left Patella, Open Approach */
"0QCF3ZZ"  =  "1"  /* Extirpation of Matter from Left Patella, Perc Approach */
"0QCF4ZZ"  =  "1"  /* Extirpation of Matter from Left Patella, Perc Endo Approach */
"0QCG0ZZ"  =  "1"  /* Extirpation of Matter from Right Tibia, Open Approach */
"0QCG3ZZ"  =  "1"  /* Extirpation of Matter from Right Tibia, Perc Approach */
"0QCG4ZZ"  =  "1"  /* Extirpation of Matter from Right Tibia, Perc Endo Approach */
"0QCH0ZZ"  =  "1"  /* Extirpation of Matter from Left Tibia, Open Approach */
"0QCH3ZZ"  =  "1"  /* Extirpation of Matter from Left Tibia, Percutaneous Approach */
"0QCH4ZZ"  =  "1"  /* Extirpation of Matter from Left Tibia, Perc Endo Approach */
"0QCJ0ZZ"  =  "1"  /* Extirpation of Matter from Right Fibula, Open Approach */
"0QCJ3ZZ"  =  "1"  /* Extirpation of Matter from Right Fibula, Perc Approach */
"0QCJ4ZZ"  =  "1"  /* Extirpation of Matter from Right Fibula, Perc Endo Approach */
"0QCK0ZZ"  =  "1"  /* Extirpation of Matter from Left Fibula, Open Approach */
"0QCK3ZZ"  =  "1"  /* Extirpation of Matter from Left Fibula, Perc Approach */
"0QCK4ZZ"  =  "1"  /* Extirpation of Matter from Left Fibula, Perc Endo Approach */
"0QCL0ZZ"  =  "1"  /* Extirpation of Matter from Right Tarsal, Open Approach */
"0QCL3ZZ"  =  "1"  /* Extirpation of Matter from Right Tarsal, Perc Approach */
"0QCL4ZZ"  =  "1"  /* Extirpation of Matter from Right Tarsal, Perc Endo Approach */
"0QCM0ZZ"  =  "1"  /* Extirpation of Matter from Left Tarsal, Open Approach */
"0QCM3ZZ"  =  "1"  /* Extirpation of Matter from Left Tarsal, Perc Approach */
"0QCM4ZZ"  =  "1"  /* Extirpation of Matter from Left Tarsal, Perc Endo Approach */
"0QCN0ZZ"  =  "1"  /* Extirpation of Matter from Right Metatarsal, Open Approach */
"0QCN3ZZ"  =  "1"  /* Extirpation of Matter from Right Metatarsal, Perc Approach */
"0QCN4ZZ"  =  "1"  /* Extirpation of Matter from R Metatarsal, Perc Endo Approach */
"0QCP0ZZ"  =  "1"  /* Extirpation of Matter from Left Metatarsal, Open Approach */
"0QCP3ZZ"  =  "1"  /* Extirpation of Matter from Left Metatarsal, Perc Approach */
"0QCP4ZZ"  =  "1"  /* Extirpation of Matter from L Metatarsal, Perc Endo Approach */
"0QCQ0ZZ"  =  "1"  /* Extirpation of Matter from Right Toe Phalanx, Open Approach */
"0QCQ3ZZ"  =  "1"  /* Extirpation of Matter from Right Toe Phalanx, Perc Approach */
"0QCQ4ZZ"  =  "1"  /* Extirpation of Matter from R Toe Phalanx, Perc Endo Approach */
"0QCR0ZZ"  =  "1"  /* Extirpation of Matter from Left Toe Phalanx, Open Approach */
"0QCR3ZZ"  =  "1"  /* Extirpation of Matter from Left Toe Phalanx, Perc Approach */
"0QCR4ZZ"  =  "1"  /* Extirpation of Matter from L Toe Phalanx, Perc Endo Approach */
"0QCS0ZZ"  =  "1"  /* Extirpation of Matter from Coccyx, Open Approach */
"0QCS3ZZ"  =  "1"  /* Extirpation of Matter from Coccyx, Percutaneous Approach */
"0QCS4ZZ"  =  "1"  /* Extirpation of Matter from Coccyx, Perc Endo Approach */
"0QH004Z"  =  "1"  /* Insertion of Int Fix into Lum Vertebra, Open Approach */
"0QH005Z"  =  "1"  /* Insertion of Ext Fix into Lum Vertebra, Open Approach */
"0QH034Z"  =  "1"  /* Insertion of Int Fix into Lum Vertebra, Perc Approach */
"0QH035Z"  =  "1"  /* Insertion of Ext Fix into Lum Vertebra, Perc Approach */
"0QH044Z"  =  "1"  /* Insertion of Int Fix into Lum Vertebra, Perc Endo Approach */
"0QH045Z"  =  "1"  /* Insertion of Ext Fix into Lum Vertebra, Perc Endo Approach */
"0QH104Z"  =  "1"  /* Insertion of Int Fix into Sacrum, Open Approach */
"0QH105Z"  =  "1"  /* Insertion of Ext Fix into Sacrum, Open Approach */
"0QH134Z"  =  "1"  /* Insertion of Int Fix into Sacrum, Perc Approach */
"0QH135Z"  =  "1"  /* Insertion of Ext Fix into Sacrum, Perc Approach */
"0QH144Z"  =  "1"  /* Insertion of Int Fix into Sacrum, Perc Endo Approach */
"0QH145Z"  =  "1"  /* Insertion of Ext Fix into Sacrum, Perc Endo Approach */
"0QH204Z"  =  "1"  /* Insertion of Int Fix into R Pelvic Bone, Open Approach */
"0QH205Z"  =  "1"  /* Insertion of Ext Fix into R Pelvic Bone, Open Approach */
"0QH234Z"  =  "1"  /* Insertion of Int Fix into R Pelvic Bone, Perc Approach */
"0QH235Z"  =  "1"  /* Insertion of Ext Fix into R Pelvic Bone, Perc Approach */
"0QH244Z"  =  "1"  /* Insertion of Int Fix into R Pelvic Bone, Perc Endo Approach */
"0QH245Z"  =  "1"  /* Insertion of Ext Fix into R Pelvic Bone, Perc Endo Approach */
"0QH304Z"  =  "1"  /* Insertion of Int Fix into L Pelvic Bone, Open Approach */
"0QH305Z"  =  "1"  /* Insertion of Ext Fix into L Pelvic Bone, Open Approach */
"0QH334Z"  =  "1"  /* Insertion of Int Fix into L Pelvic Bone, Perc Approach */
"0QH335Z"  =  "1"  /* Insertion of Ext Fix into L Pelvic Bone, Perc Approach */
"0QH344Z"  =  "1"  /* Insertion of Int Fix into L Pelvic Bone, Perc Endo Approach */
"0QH345Z"  =  "1"  /* Insertion of Ext Fix into L Pelvic Bone, Perc Endo Approach */
"0QH404Z"  =  "1"  /* Insertion of Int Fix into R Acetabulum, Open Approach */
"0QH405Z"  =  "1"  /* Insertion of Ext Fix into R Acetabulum, Open Approach */
"0QH434Z"  =  "1"  /* Insertion of Int Fix into R Acetabulum, Perc Approach */
"0QH435Z"  =  "1"  /* Insertion of Ext Fix into R Acetabulum, Perc Approach */
"0QH444Z"  =  "1"  /* Insertion of Int Fix into R Acetabulum, Perc Endo Approach */
"0QH445Z"  =  "1"  /* Insertion of Ext Fix into R Acetabulum, Perc Endo Approach */
"0QH504Z"  =  "1"  /* Insertion of Int Fix into L Acetabulum, Open Approach */
"0QH505Z"  =  "1"  /* Insertion of Ext Fix into L Acetabulum, Open Approach */
"0QH534Z"  =  "1"  /* Insertion of Int Fix into L Acetabulum, Perc Approach */
"0QH535Z"  =  "1"  /* Insertion of Ext Fix into L Acetabulum, Perc Approach */
"0QH544Z"  =  "1"  /* Insertion of Int Fix into L Acetabulum, Perc Endo Approach */
"0QH545Z"  =  "1"  /* Insertion of Ext Fix into L Acetabulum, Perc Endo Approach */
"0QH604Z"  =  "1"  /* Insertion of Int Fix into R Up Femur, Open Approach */
"0QH605Z"  =  "1"  /* Insertion of Ext Fix into R Up Femur, Open Approach */
"0QH606Z"  =  "1"  /* Insertion of Intramed Fix into R Up Femur, Open Approach */
"0QH60BZ"  =  "1"  /* Insertion of Monopln Ext Fix into R Up Femur, Open Approach */
"0QH60CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Up Femur, Open Approach */
"0QH60DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into R Up Femur, Open Approach */
"0QH634Z"  =  "1"  /* Insertion of Int Fix into R Up Femur, Perc Approach */
"0QH635Z"  =  "1"  /* Insertion of Ext Fix into R Up Femur, Perc Approach */
"0QH636Z"  =  "1"  /* Insertion of Intramed Fix into R Up Femur, Perc Approach */
"0QH63BZ"  =  "1"  /* Insertion of Monopln Ext Fix into R Up Femur, Perc Approach */
"0QH63CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Up Femur, Perc Approach */
"0QH63DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into R Up Femur, Perc Approach */
"0QH644Z"  =  "1"  /* Insertion of Int Fix into R Up Femur, Perc Endo Approach */
"0QH645Z"  =  "1"  /* Insertion of Ext Fix into R Up Femur, Perc Endo Approach */
"0QH646Z"  =  "1"  /* Insert of Intramed Fix into R Up Femur, Perc Endo Approach */
"0QH64BZ"  =  "1"  /* Insert Monopln Ext Fix in R Up Femur, Perc Endo */
"0QH64CZ"  =  "1"  /* Insert of Ring Ext Fix into R Up Femur, Perc Endo Approach */
"0QH64DZ"  =  "1"  /* Insert of Hybrid Ext Fix into R Up Femur, Perc Endo Approach */
"0QH704Z"  =  "1"  /* Insertion of Int Fix into L Up Femur, Open Approach */
"0QH705Z"  =  "1"  /* Insertion of Ext Fix into L Up Femur, Open Approach */
"0QH706Z"  =  "1"  /* Insertion of Intramed Fix into L Up Femur, Open Approach */
"0QH70BZ"  =  "1"  /* Insertion of Monopln Ext Fix into L Up Femur, Open Approach */
"0QH70CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Up Femur, Open Approach */
"0QH70DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into L Up Femur, Open Approach */
"0QH734Z"  =  "1"  /* Insertion of Int Fix into L Up Femur, Perc Approach */
"0QH735Z"  =  "1"  /* Insertion of Ext Fix into L Up Femur, Perc Approach */
"0QH736Z"  =  "1"  /* Insertion of Intramed Fix into L Up Femur, Perc Approach */
"0QH73BZ"  =  "1"  /* Insertion of Monopln Ext Fix into L Up Femur, Perc Approach */
"0QH73CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Up Femur, Perc Approach */
"0QH73DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into L Up Femur, Perc Approach */
"0QH744Z"  =  "1"  /* Insertion of Int Fix into L Up Femur, Perc Endo Approach */
"0QH745Z"  =  "1"  /* Insertion of Ext Fix into L Up Femur, Perc Endo Approach */
"0QH746Z"  =  "1"  /* Insert of Intramed Fix into L Up Femur, Perc Endo Approach */
"0QH74BZ"  =  "1"  /* Insert Monopln Ext Fix in L Up Femur, Perc Endo */
"0QH74CZ"  =  "1"  /* Insert of Ring Ext Fix into L Up Femur, Perc Endo Approach */
"0QH74DZ"  =  "1"  /* Insert of Hybrid Ext Fix into L Up Femur, Perc Endo Approach */
"0QH804Z"  =  "1"  /* Insertion of Int Fix into R Femur Shaft, Open Approach */
"0QH805Z"  =  "1"  /* Insertion of Ext Fix into R Femur Shaft, Open Approach */
"0QH806Z"  =  "1"  /* Insertion of Intramed Fix into R Femur Shaft, Open Approach */
"0QH80BZ"  =  "1"  /* Insert of Monopln Ext Fix into R Femur Shaft, Open Approach */
"0QH80CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Femur Shaft, Open Approach */
"0QH80DZ"  =  "1"  /* Insert of Hybrid Ext Fix into R Femur Shaft, Open Approach */
"0QH834Z"  =  "1"  /* Insertion of Int Fix into R Femur Shaft, Perc Approach */
"0QH835Z"  =  "1"  /* Insertion of Ext Fix into R Femur Shaft, Perc Approach */
"0QH836Z"  =  "1"  /* Insertion of Intramed Fix into R Femur Shaft, Perc Approach */
"0QH83BZ"  =  "1"  /* Insert of Monopln Ext Fix into R Femur Shaft, Perc Approach */
"0QH83CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Femur Shaft, Perc Approach */
"0QH83DZ"  =  "1"  /* Insert of Hybrid Ext Fix into R Femur Shaft, Perc Approach */
"0QH844Z"  =  "1"  /* Insertion of Int Fix into R Femur Shaft, Perc Endo Approach */
"0QH845Z"  =  "1"  /* Insertion of Ext Fix into R Femur Shaft, Perc Endo Approach */
"0QH846Z"  =  "1"  /* Insert Intramed Fix in R Femur Shaft, Perc Endo */
"0QH84BZ"  =  "1"  /* Insert Monopln Ext Fix in R Femur Shaft, Perc Endo */
"0QH84CZ"  =  "1"  /* Insert Ring Ext Fix in R Femur Shaft, Perc Endo */
"0QH84DZ"  =  "1"  /* Insert Hybrid Ext Fix in R Femur Shaft, Perc Endo */
"0QH904Z"  =  "1"  /* Insertion of Int Fix into L Femur Shaft, Open Approach */
"0QH905Z"  =  "1"  /* Insertion of Ext Fix into L Femur Shaft, Open Approach */
"0QH906Z"  =  "1"  /* Insertion of Intramed Fix into L Femur Shaft, Open Approach */
"0QH90BZ"  =  "1"  /* Insert of Monopln Ext Fix into L Femur Shaft, Open Approach */
"0QH90CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Femur Shaft, Open Approach */
"0QH90DZ"  =  "1"  /* Insert of Hybrid Ext Fix into L Femur Shaft, Open Approach */
"0QH934Z"  =  "1"  /* Insertion of Int Fix into L Femur Shaft, Perc Approach */
"0QH935Z"  =  "1"  /* Insertion of Ext Fix into L Femur Shaft, Perc Approach */
"0QH936Z"  =  "1"  /* Insertion of Intramed Fix into L Femur Shaft, Perc Approach */
"0QH93BZ"  =  "1"  /* Insert of Monopln Ext Fix into L Femur Shaft, Perc Approach */
"0QH93CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Femur Shaft, Perc Approach */
"0QH93DZ"  =  "1"  /* Insert of Hybrid Ext Fix into L Femur Shaft, Perc Approach */
"0QH944Z"  =  "1"  /* Insertion of Int Fix into L Femur Shaft, Perc Endo Approach */
"0QH945Z"  =  "1"  /* Insertion of Ext Fix into L Femur Shaft, Perc Endo Approach */
"0QH946Z"  =  "1"  /* Insert Intramed Fix in L Femur Shaft, Perc Endo */
"0QH94BZ"  =  "1"  /* Insert Monopln Ext Fix in L Femur Shaft, Perc Endo */
"0QH94CZ"  =  "1"  /* Insert Ring Ext Fix in L Femur Shaft, Perc Endo */
"0QH94DZ"  =  "1"  /* Insert Hybrid Ext Fix in L Femur Shaft, Perc Endo */
"0QHB04Z"  =  "1"  /* Insertion of Int Fix into R Low Femur, Open Approach */
"0QHB05Z"  =  "1"  /* Insertion of Ext Fix into R Low Femur, Open Approach */
"0QHB06Z"  =  "1"  /* Insertion of Intramed Fix into R Low Femur, Open Approach */
"0QHB0BZ"  =  "1"  /* Insertion of Monopln Ext Fix into R Low Femur, Open Approach */
"0QHB0CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Low Femur, Open Approach */
"0QHB0DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into R Low Femur, Open Approach */
"0QHB34Z"  =  "1"  /* Insertion of Int Fix into R Low Femur, Perc Approach */
"0QHB35Z"  =  "1"  /* Insertion of Ext Fix into R Low Femur, Perc Approach */
"0QHB36Z"  =  "1"  /* Insertion of Intramed Fix into R Low Femur, Perc Approach */
"0QHB3BZ"  =  "1"  /* Insertion of Monopln Ext Fix into R Low Femur, Perc Approach */
"0QHB3CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Low Femur, Perc Approach */
"0QHB3DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into R Low Femur, Perc Approach */
"0QHB44Z"  =  "1"  /* Insertion of Int Fix into R Low Femur, Perc Endo Approach */
"0QHB45Z"  =  "1"  /* Insertion of Ext Fix into R Low Femur, Perc Endo Approach */
"0QHB46Z"  =  "1"  /* Insert of Intramed Fix into R Low Femur, Perc Endo Approach */
"0QHB4BZ"  =  "1"  /* Insert Monopln Ext Fix in R Low Femur, Perc Endo */
"0QHB4CZ"  =  "1"  /* Insert of Ring Ext Fix into R Low Femur, Perc Endo Approach */
"0QHB4DZ"  =  "1"  /* Insert Hybrid Ext Fix in R Low Femur, Perc Endo */
"0QHC04Z"  =  "1"  /* Insertion of Int Fix into L Low Femur, Open Approach */
"0QHC05Z"  =  "1"  /* Insertion of Ext Fix into L Low Femur, Open Approach */
"0QHC06Z"  =  "1"  /* Insertion of Intramed Fix into L Low Femur, Open Approach */
"0QHC0BZ"  =  "1"  /* Insertion of Monopln Ext Fix into L Low Femur, Open Approach */
"0QHC0CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Low Femur, Open Approach */
"0QHC0DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into L Low Femur, Open Approach */
"0QHC34Z"  =  "1"  /* Insertion of Int Fix into L Low Femur, Perc Approach */
"0QHC35Z"  =  "1"  /* Insertion of Ext Fix into L Low Femur, Perc Approach */
"0QHC36Z"  =  "1"  /* Insertion of Intramed Fix into L Low Femur, Perc Approach */
"0QHC3BZ"  =  "1"  /* Insertion of Monopln Ext Fix into L Low Femur, Perc Approach */
"0QHC3CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Low Femur, Perc Approach */
"0QHC3DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into L Low Femur, Perc Approach */
"0QHC44Z"  =  "1"  /* Insertion of Int Fix into L Low Femur, Perc Endo Approach */
"0QHC45Z"  =  "1"  /* Insertion of Ext Fix into L Low Femur, Perc Endo Approach */
"0QHC46Z"  =  "1"  /* Insert of Intramed Fix into L Low Femur, Perc Endo Approach */
"0QHC4BZ"  =  "1"  /* Insert Monopln Ext Fix in L Low Femur, Perc Endo */
"0QHC4CZ"  =  "1"  /* Insert of Ring Ext Fix into L Low Femur, Perc Endo Approach */
"0QHC4DZ"  =  "1"  /* Insert Hybrid Ext Fix in L Low Femur, Perc Endo */
"0QHD04Z"  =  "1"  /* Insertion of Int Fix into R Patella, Open Approach */
"0QHD05Z"  =  "1"  /* Insertion of Ext Fix into R Patella, Open Approach */
"0QHD34Z"  =  "1"  /* Insertion of Int Fix into R Patella, Perc Approach */
"0QHD35Z"  =  "1"  /* Insertion of Ext Fix into R Patella, Perc Approach */
"0QHD44Z"  =  "1"  /* Insertion of Int Fix into R Patella, Perc Endo Approach */
"0QHD45Z"  =  "1"  /* Insertion of Ext Fix into R Patella, Perc Endo Approach */
"0QHF04Z"  =  "1"  /* Insertion of Int Fix into L Patella, Open Approach */
"0QHF05Z"  =  "1"  /* Insertion of Ext Fix into L Patella, Open Approach */
"0QHF34Z"  =  "1"  /* Insertion of Int Fix into L Patella, Perc Approach */
"0QHF35Z"  =  "1"  /* Insertion of Ext Fix into L Patella, Perc Approach */
"0QHF44Z"  =  "1"  /* Insertion of Int Fix into L Patella, Perc Endo Approach */
"0QHF45Z"  =  "1"  /* Insertion of Ext Fix into L Patella, Perc Endo Approach */
"0QHG04Z"  =  "1"  /* Insertion of Int Fix into R Tibia, Open Approach */
"0QHG05Z"  =  "1"  /* Insertion of Ext Fix into R Tibia, Open Approach */
"0QHG06Z"  =  "1"  /* Insertion of Intramed Fix into R Tibia, Open Approach */
"0QHG0BZ"  =  "1"  /* Insertion of Monopln Ext Fix into R Tibia, Open Approach */
"0QHG0CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Tibia, Open Approach */
"0QHG0DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into R Tibia, Open Approach */
"0QHG34Z"  =  "1"  /* Insertion of Int Fix into R Tibia, Perc Approach */
"0QHG35Z"  =  "1"  /* Insertion of Ext Fix into R Tibia, Perc Approach */
"0QHG36Z"  =  "1"  /* Insertion of Intramed Fix into R Tibia, Perc Approach */
"0QHG3BZ"  =  "1"  /* Insertion of Monopln Ext Fix into R Tibia, Perc Approach */
"0QHG3CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Tibia, Perc Approach */
"0QHG3DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into R Tibia, Perc Approach */
"0QHG44Z"  =  "1"  /* Insertion of Int Fix into R Tibia, Perc Endo Approach */
"0QHG45Z"  =  "1"  /* Insertion of Ext Fix into R Tibia, Perc Endo Approach */
"0QHG46Z"  =  "1"  /* Insertion of Intramed Fix into R Tibia, Perc Endo Approach */
"0QHG4BZ"  =  "1"  /* Insert of Monopln Ext Fix into R Tibia, Perc Endo Approach */
"0QHG4CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Tibia, Perc Endo Approach */
"0QHG4DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into R Tibia, Perc Endo Approach */
"0QHH04Z"  =  "1"  /* Insertion of Int Fix into L Tibia, Open Approach */
"0QHH05Z"  =  "1"  /* Insertion of Ext Fix into L Tibia, Open Approach */
"0QHH06Z"  =  "1"  /* Insertion of Intramed Fix into L Tibia, Open Approach */
"0QHH0BZ"  =  "1"  /* Insertion of Monopln Ext Fix into L Tibia, Open Approach */
"0QHH0CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Tibia, Open Approach */
"0QHH0DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into L Tibia, Open Approach */
"0QHH34Z"  =  "1"  /* Insertion of Int Fix into L Tibia, Perc Approach */
"0QHH35Z"  =  "1"  /* Insertion of Ext Fix into L Tibia, Perc Approach */
"0QHH36Z"  =  "1"  /* Insertion of Intramed Fix into L Tibia, Perc Approach */
"0QHH3BZ"  =  "1"  /* Insertion of Monopln Ext Fix into L Tibia, Perc Approach */
"0QHH3CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Tibia, Perc Approach */
"0QHH3DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into L Tibia, Perc Approach */
"0QHH44Z"  =  "1"  /* Insertion of Int Fix into L Tibia, Perc Endo Approach */
"0QHH45Z"  =  "1"  /* Insertion of Ext Fix into L Tibia, Perc Endo Approach */
"0QHH46Z"  =  "1"  /* Insertion of Intramed Fix into L Tibia, Perc Endo Approach */
"0QHH4BZ"  =  "1"  /* Insert of Monopln Ext Fix into L Tibia, Perc Endo Approach */
"0QHH4CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Tibia, Perc Endo Approach */
"0QHH4DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into L Tibia, Perc Endo Approach */
"0QHJ04Z"  =  "1"  /* Insertion of Int Fix into R Fibula, Open Approach */
"0QHJ05Z"  =  "1"  /* Insertion of Ext Fix into R Fibula, Open Approach */
"0QHJ06Z"  =  "1"  /* Insertion of Intramed Fix into R Fibula, Open Approach */
"0QHJ0BZ"  =  "1"  /* Insertion of Monopln Ext Fix into R Fibula, Open Approach */
"0QHJ0CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Fibula, Open Approach */
"0QHJ0DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into R Fibula, Open Approach */
"0QHJ34Z"  =  "1"  /* Insertion of Int Fix into R Fibula, Perc Approach */
"0QHJ35Z"  =  "1"  /* Insertion of Ext Fix into R Fibula, Perc Approach */
"0QHJ36Z"  =  "1"  /* Insertion of Intramed Fix into R Fibula, Perc Approach */
"0QHJ3BZ"  =  "1"  /* Insertion of Monopln Ext Fix into R Fibula, Perc Approach */
"0QHJ3CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Fibula, Perc Approach */
"0QHJ3DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into R Fibula, Perc Approach */
"0QHJ44Z"  =  "1"  /* Insertion of Int Fix into R Fibula, Perc Endo Approach */
"0QHJ45Z"  =  "1"  /* Insertion of Ext Fix into R Fibula, Perc Endo Approach */
"0QHJ46Z"  =  "1"  /* Insertion of Intramed Fix into R Fibula, Perc Endo Approach */
"0QHJ4BZ"  =  "1"  /* Insert of Monopln Ext Fix into R Fibula, Perc Endo Approach */
"0QHJ4CZ"  =  "1"  /* Insertion of Ring Ext Fix into R Fibula, Perc Endo Approach */
"0QHJ4DZ"  =  "1"  /* Insert of Hybrid Ext Fix into R Fibula, Perc Endo Approach */
"0QHK04Z"  =  "1"  /* Insertion of Int Fix into L Fibula, Open Approach */
"0QHK05Z"  =  "1"  /* Insertion of Ext Fix into L Fibula, Open Approach */
"0QHK06Z"  =  "1"  /* Insertion of Intramed Fix into L Fibula, Open Approach */
"0QHK0BZ"  =  "1"  /* Insertion of Monopln Ext Fix into L Fibula, Open Approach */
"0QHK0CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Fibula, Open Approach */
"0QHK0DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into L Fibula, Open Approach */
"0QHK34Z"  =  "1"  /* Insertion of Int Fix into L Fibula, Perc Approach */
"0QHK35Z"  =  "1"  /* Insertion of Ext Fix into L Fibula, Perc Approach */
"0QHK36Z"  =  "1"  /* Insertion of Intramed Fix into L Fibula, Perc Approach */
"0QHK3BZ"  =  "1"  /* Insertion of Monopln Ext Fix into L Fibula, Perc Approach */
"0QHK3CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Fibula, Perc Approach */
"0QHK3DZ"  =  "1"  /* Insertion of Hybrid Ext Fix into L Fibula, Perc Approach */
"0QHK44Z"  =  "1"  /* Insertion of Int Fix into L Fibula, Perc Endo Approach */
"0QHK45Z"  =  "1"  /* Insertion of Ext Fix into L Fibula, Perc Endo Approach */
"0QHK46Z"  =  "1"  /* Insertion of Intramed Fix into L Fibula, Perc Endo Approach */
"0QHK4BZ"  =  "1"  /* Insert of Monopln Ext Fix into L Fibula, Perc Endo Approach */
"0QHK4CZ"  =  "1"  /* Insertion of Ring Ext Fix into L Fibula, Perc Endo Approach */
"0QHK4DZ"  =  "1"  /* Insert of Hybrid Ext Fix into L Fibula, Perc Endo Approach */
"0QHL04Z"  =  "1"  /* Insertion of Int Fix into R Tarsal, Open Approach */
"0QHL05Z"  =  "1"  /* Insertion of Ext Fix into R Tarsal, Open Approach */
"0QHL34Z"  =  "1"  /* Insertion of Int Fix into R Tarsal, Perc Approach */
"0QHL35Z"  =  "1"  /* Insertion of Ext Fix into R Tarsal, Perc Approach */
"0QHL44Z"  =  "1"  /* Insertion of Int Fix into R Tarsal, Perc Endo Approach */
"0QHL45Z"  =  "1"  /* Insertion of Ext Fix into R Tarsal, Perc Endo Approach */
"0QHM04Z"  =  "1"  /* Insertion of Int Fix into L Tarsal, Open Approach */
"0QHM05Z"  =  "1"  /* Insertion of Ext Fix into L Tarsal, Open Approach */
"0QHM34Z"  =  "1"  /* Insertion of Int Fix into L Tarsal, Perc Approach */
"0QHM35Z"  =  "1"  /* Insertion of Ext Fix into L Tarsal, Perc Approach */
"0QHM44Z"  =  "1"  /* Insertion of Int Fix into L Tarsal, Perc Endo Approach */
"0QHM45Z"  =  "1"  /* Insertion of Ext Fix into L Tarsal, Perc Endo Approach */
"0QHN04Z"  =  "1"  /* Insertion of Int Fix into R Metatarsal, Open Approach */
"0QHN05Z"  =  "1"  /* Insertion of Ext Fix into R Metatarsal, Open Approach */
"0QHN34Z"  =  "1"  /* Insertion of Int Fix into R Metatarsal, Perc Approach */
"0QHN35Z"  =  "1"  /* Insertion of Ext Fix into R Metatarsal, Perc Approach */
"0QHN44Z"  =  "1"  /* Insertion of Int Fix into R Metatarsal, Perc Endo Approach */
"0QHN45Z"  =  "1"  /* Insertion of Ext Fix into R Metatarsal, Perc Endo Approach */
"0QHP04Z"  =  "1"  /* Insertion of Int Fix into L Metatarsal, Open Approach */
"0QHP05Z"  =  "1"  /* Insertion of Ext Fix into L Metatarsal, Open Approach */
"0QHP34Z"  =  "1"  /* Insertion of Int Fix into L Metatarsal, Perc Approach */
"0QHP35Z"  =  "1"  /* Insertion of Ext Fix into L Metatarsal, Perc Approach */
"0QHP44Z"  =  "1"  /* Insertion of Int Fix into L Metatarsal, Perc Endo Approach */
"0QHP45Z"  =  "1"  /* Insertion of Ext Fix into L Metatarsal, Perc Endo Approach */
"0QHQ04Z"  =  "1"  /* Insertion of Int Fix into R Toe Phalanx, Open Approach */
"0QHQ05Z"  =  "1"  /* Insertion of Ext Fix into R Toe Phalanx, Open Approach */
"0QHQ34Z"  =  "1"  /* Insertion of Int Fix into R Toe Phalanx, Perc Approach */
"0QHQ35Z"  =  "1"  /* Insertion of Ext Fix into R Toe Phalanx, Perc Approach */
"0QHQ44Z"  =  "1"  /* Insertion of Int Fix into R Toe Phalanx, Perc Endo Approach */
"0QHQ45Z"  =  "1"  /* Insertion of Ext Fix into R Toe Phalanx, Perc Endo Approach */
"0QHR04Z"  =  "1"  /* Insertion of Int Fix into L Toe Phalanx, Open Approach */
"0QHR05Z"  =  "1"  /* Insertion of Ext Fix into L Toe Phalanx, Open Approach */
"0QHR34Z"  =  "1"  /* Insertion of Int Fix into L Toe Phalanx, Perc Approach */
"0QHR35Z"  =  "1"  /* Insertion of Ext Fix into L Toe Phalanx, Perc Approach */
"0QHR44Z"  =  "1"  /* Insertion of Int Fix into L Toe Phalanx, Perc Endo Approach */
"0QHR45Z"  =  "1"  /* Insertion of Ext Fix into L Toe Phalanx, Perc Endo Approach */
"0QHS04Z"  =  "1"  /* Insertion of Int Fix into Coccyx, Open Approach */
"0QHS05Z"  =  "1"  /* Insertion of Ext Fix into Coccyx, Open Approach */
"0QHS34Z"  =  "1"  /* Insertion of Int Fix into Coccyx, Perc Approach */
"0QHS35Z"  =  "1"  /* Insertion of Ext Fix into Coccyx, Perc Approach */
"0QHS44Z"  =  "1"  /* Insertion of Int Fix into Coccyx, Perc Endo Approach */
"0QHS45Z"  =  "1"  /* Insertion of Ext Fix into Coccyx, Perc Endo Approach */
"0QHY0MZ"  =  "1"  /* Insertion of Bone Stim into Low Bone, Open Approach */
"0QHY3MZ"  =  "1"  /* Insertion of Bone Stim into Low Bone, Perc Approach */
"0QHY4MZ"  =  "1"  /* Insertion of Bone Stim into Low Bone, Perc Endo Approach */
"0QJY0ZZ"  =  "1"  /* Inspection of Lower Bone, Open Approach */
"0QJY3ZZ"  =  "1"  /* Inspection of Lower Bone, Percutaneous Approach */
"0QJY4ZZ"  =  "1"  /* Inspection of Lower Bone, Percutaneous Endoscopic Approach */
"0QN00ZZ"  =  "1"  /* Release Lumbar Vertebra, Open Approach */
"0QN03ZZ"  =  "1"  /* Release Lumbar Vertebra, Percutaneous Approach */
"0QN04ZZ"  =  "1"  /* Release Lumbar Vertebra, Percutaneous Endoscopic Approach */
"0QN10ZZ"  =  "1"  /* Release Sacrum, Open Approach */
"0QN13ZZ"  =  "1"  /* Release Sacrum, Percutaneous Approach */
"0QN14ZZ"  =  "1"  /* Release Sacrum, Percutaneous Endoscopic Approach */
"0QN20ZZ"  =  "1"  /* Release Right Pelvic Bone, Open Approach */
"0QN23ZZ"  =  "1"  /* Release Right Pelvic Bone, Percutaneous Approach */
"0QN24ZZ"  =  "1"  /* Release Right Pelvic Bone, Percutaneous Endoscopic Approach */
"0QN30ZZ"  =  "1"  /* Release Left Pelvic Bone, Open Approach */
"0QN33ZZ"  =  "1"  /* Release Left Pelvic Bone, Percutaneous Approach */
"0QN34ZZ"  =  "1"  /* Release Left Pelvic Bone, Percutaneous Endoscopic Approach */
"0QN40ZZ"  =  "1"  /* Release Right Acetabulum, Open Approach */
"0QN43ZZ"  =  "1"  /* Release Right Acetabulum, Percutaneous Approach */
"0QN44ZZ"  =  "1"  /* Release Right Acetabulum, Percutaneous Endoscopic Approach */
"0QN50ZZ"  =  "1"  /* Release Left Acetabulum, Open Approach */
"0QN53ZZ"  =  "1"  /* Release Left Acetabulum, Percutaneous Approach */
"0QN54ZZ"  =  "1"  /* Release Left Acetabulum, Percutaneous Endoscopic Approach */
"0QN60ZZ"  =  "1"  /* Release Right Upper Femur, Open Approach */
"0QN63ZZ"  =  "1"  /* Release Right Upper Femur, Percutaneous Approach */
"0QN64ZZ"  =  "1"  /* Release Right Upper Femur, Percutaneous Endoscopic Approach */
"0QN70ZZ"  =  "1"  /* Release Left Upper Femur, Open Approach */
"0QN73ZZ"  =  "1"  /* Release Left Upper Femur, Percutaneous Approach */
"0QN74ZZ"  =  "1"  /* Release Left Upper Femur, Percutaneous Endoscopic Approach */
"0QN80ZZ"  =  "1"  /* Release Right Femoral Shaft, Open Approach */
"0QN83ZZ"  =  "1"  /* Release Right Femoral Shaft, Percutaneous Approach */
"0QN84ZZ"  =  "1"  /* Release Right Femoral Shaft, Perc Endo Approach */
"0QN90ZZ"  =  "1"  /* Release Left Femoral Shaft, Open Approach */
"0QN93ZZ"  =  "1"  /* Release Left Femoral Shaft, Percutaneous Approach */
"0QN94ZZ"  =  "1"  /* Release Left Femoral Shaft, Percutaneous Endoscopic Approach */
"0QNB0ZZ"  =  "1"  /* Release Right Lower Femur, Open Approach */
"0QNB3ZZ"  =  "1"  /* Release Right Lower Femur, Percutaneous Approach */
"0QNB4ZZ"  =  "1"  /* Release Right Lower Femur, Percutaneous Endoscopic Approach */
"0QNC0ZZ"  =  "1"  /* Release Left Lower Femur, Open Approach */
"0QNC3ZZ"  =  "1"  /* Release Left Lower Femur, Percutaneous Approach */
"0QNC4ZZ"  =  "1"  /* Release Left Lower Femur, Percutaneous Endoscopic Approach */
"0QND0ZZ"  =  "1"  /* Release Right Patella, Open Approach */
"0QND3ZZ"  =  "1"  /* Release Right Patella, Percutaneous Approach */
"0QND4ZZ"  =  "1"  /* Release Right Patella, Percutaneous Endoscopic Approach */
"0QNF0ZZ"  =  "1"  /* Release Left Patella, Open Approach */
"0QNF3ZZ"  =  "1"  /* Release Left Patella, Percutaneous Approach */
"0QNF4ZZ"  =  "1"  /* Release Left Patella, Percutaneous Endoscopic Approach */
"0QNG0ZZ"  =  "1"  /* Release Right Tibia, Open Approach */
"0QNG3ZZ"  =  "1"  /* Release Right Tibia, Percutaneous Approach */
"0QNG4ZZ"  =  "1"  /* Release Right Tibia, Percutaneous Endoscopic Approach */
"0QNH0ZZ"  =  "1"  /* Release Left Tibia, Open Approach */
"0QNH3ZZ"  =  "1"  /* Release Left Tibia, Percutaneous Approach */
"0QNH4ZZ"  =  "1"  /* Release Left Tibia, Percutaneous Endoscopic Approach */
"0QNJ0ZZ"  =  "1"  /* Release Right Fibula, Open Approach */
"0QNJ3ZZ"  =  "1"  /* Release Right Fibula, Percutaneous Approach */
"0QNJ4ZZ"  =  "1"  /* Release Right Fibula, Percutaneous Endoscopic Approach */
"0QNK0ZZ"  =  "1"  /* Release Left Fibula, Open Approach */
"0QNK3ZZ"  =  "1"  /* Release Left Fibula, Percutaneous Approach */
"0QNK4ZZ"  =  "1"  /* Release Left Fibula, Percutaneous Endoscopic Approach */
"0QNL0ZZ"  =  "1"  /* Release Right Tarsal, Open Approach */
"0QNL3ZZ"  =  "1"  /* Release Right Tarsal, Percutaneous Approach */
"0QNL4ZZ"  =  "1"  /* Release Right Tarsal, Percutaneous Endoscopic Approach */
"0QNM0ZZ"  =  "1"  /* Release Left Tarsal, Open Approach */
"0QNM3ZZ"  =  "1"  /* Release Left Tarsal, Percutaneous Approach */
"0QNM4ZZ"  =  "1"  /* Release Left Tarsal, Percutaneous Endoscopic Approach */
"0QNN0ZZ"  =  "1"  /* Release Right Metatarsal, Open Approach */
"0QNN3ZZ"  =  "1"  /* Release Right Metatarsal, Percutaneous Approach */
"0QNN4ZZ"  =  "1"  /* Release Right Metatarsal, Percutaneous Endoscopic Approach */
"0QNP0ZZ"  =  "1"  /* Release Left Metatarsal, Open Approach */
"0QNP3ZZ"  =  "1"  /* Release Left Metatarsal, Percutaneous Approach */
"0QNP4ZZ"  =  "1"  /* Release Left Metatarsal, Percutaneous Endoscopic Approach */
"0QNQ0ZZ"  =  "1"  /* Release Right Toe Phalanx, Open Approach */
"0QNQ3ZZ"  =  "1"  /* Release Right Toe Phalanx, Percutaneous Approach */
"0QNQ4ZZ"  =  "1"  /* Release Right Toe Phalanx, Percutaneous Endoscopic Approach */
"0QNR0ZZ"  =  "1"  /* Release Left Toe Phalanx, Open Approach */
"0QNR3ZZ"  =  "1"  /* Release Left Toe Phalanx, Percutaneous Approach */
"0QNR4ZZ"  =  "1"  /* Release Left Toe Phalanx, Percutaneous Endoscopic Approach */
"0QNS0ZZ"  =  "1"  /* Release Coccyx, Open Approach */
"0QNS3ZZ"  =  "1"  /* Release Coccyx, Percutaneous Approach */
"0QNS4ZZ"  =  "1"  /* Release Coccyx, Percutaneous Endoscopic Approach */
"0QP004Z"  =  "1"  /* Removal of Int Fix from Lum Vertebra, Open Approach */
"0QP007Z"  =  "1"  /* Removal of Autol Sub from Lum Vertebra, Open Approach */
"0QP00JZ"  =  "1"  /* Removal of Synth Sub from Lum Vertebra, Open Approach */
"0QP00KZ"  =  "1"  /* Removal of Nonaut Sub from Lum Vertebra, Open Approach */
"0QP034Z"  =  "1"  /* Removal of Int Fix from Lum Vertebra, Perc Approach */
"0QP037Z"  =  "1"  /* Removal of Autol Sub from Lum Vertebra, Perc Approach */
"0QP03JZ"  =  "1"  /* Removal of Synth Sub from Lum Vertebra, Perc Approach */
"0QP03KZ"  =  "1"  /* Removal of Nonaut Sub from Lum Vertebra, Perc Approach */
"0QP044Z"  =  "1"  /* Removal of Int Fix from Lum Vertebra, Perc Endo Approach */
"0QP047Z"  =  "1"  /* Removal of Autol Sub from Lum Vertebra, Perc Endo Approach */
"0QP04JZ"  =  "1"  /* Removal of Synth Sub from Lum Vertebra, Perc Endo Approach */
"0QP04KZ"  =  "1"  /* Removal of Nonaut Sub from Lum Vertebra, Perc Endo Approach */
"0QP104Z"  =  "1"  /* Removal of Int Fix from Sacrum, Open Approach */
"0QP107Z"  =  "1"  /* Removal of Autol Sub from Sacrum, Open Approach */
"0QP10JZ"  =  "1"  /* Removal of Synthetic Substitute from Sacrum, Open Approach */
"0QP10KZ"  =  "1"  /* Removal of Nonaut Sub from Sacrum, Open Approach */
"0QP134Z"  =  "1"  /* Removal of Int Fix from Sacrum, Perc Approach */
"0QP137Z"  =  "1"  /* Removal of Autol Sub from Sacrum, Perc Approach */
"0QP13JZ"  =  "1"  /* Removal of Synthetic Substitute from Sacrum, Perc Approach */
"0QP13KZ"  =  "1"  /* Removal of Nonaut Sub from Sacrum, Perc Approach */
"0QP144Z"  =  "1"  /* Removal of Int Fix from Sacrum, Perc Endo Approach */
"0QP147Z"  =  "1"  /* Removal of Autol Sub from Sacrum, Perc Endo Approach */
"0QP14JZ"  =  "1"  /* Removal of Synth Sub from Sacrum, Perc Endo Approach */
"0QP14KZ"  =  "1"  /* Removal of Nonaut Sub from Sacrum, Perc Endo Approach */
"0QP204Z"  =  "1"  /* Removal of Int Fix from R Pelvic Bone, Open Approach */
"0QP205Z"  =  "1"  /* Removal of Ext Fix from R Pelvic Bone, Open Approach */
"0QP207Z"  =  "1"  /* Removal of Autol Sub from R Pelvic Bone, Open Approach */
"0QP20JZ"  =  "1"  /* Removal of Synth Sub from R Pelvic Bone, Open Approach */
"0QP20KZ"  =  "1"  /* Removal of Nonaut Sub from R Pelvic Bone, Open Approach */
"0QP234Z"  =  "1"  /* Removal of Int Fix from R Pelvic Bone, Perc Approach */
"0QP235Z"  =  "1"  /* Removal of Ext Fix from R Pelvic Bone, Perc Approach */
"0QP237Z"  =  "1"  /* Removal of Autol Sub from R Pelvic Bone, Perc Approach */
"0QP23JZ"  =  "1"  /* Removal of Synth Sub from R Pelvic Bone, Perc Approach */
"0QP23KZ"  =  "1"  /* Removal of Nonaut Sub from R Pelvic Bone, Perc Approach */
"0QP244Z"  =  "1"  /* Removal of Int Fix from R Pelvic Bone, Perc Endo Approach */
"0QP245Z"  =  "1"  /* Removal of Ext Fix from R Pelvic Bone, Perc Endo Approach */
"0QP247Z"  =  "1"  /* Removal of Autol Sub from R Pelvic Bone, Perc Endo Approach */
"0QP24JZ"  =  "1"  /* Removal of Synth Sub from R Pelvic Bone, Perc Endo Approach */
"0QP24KZ"  =  "1"  /* Removal of Nonaut Sub from R Pelvic Bone, Perc Endo Approach */
"0QP304Z"  =  "1"  /* Removal of Int Fix from L Pelvic Bone, Open Approach */
"0QP305Z"  =  "1"  /* Removal of Ext Fix from L Pelvic Bone, Open Approach */
"0QP307Z"  =  "1"  /* Removal of Autol Sub from L Pelvic Bone, Open Approach */
"0QP30JZ"  =  "1"  /* Removal of Synth Sub from L Pelvic Bone, Open Approach */
"0QP30KZ"  =  "1"  /* Removal of Nonaut Sub from L Pelvic Bone, Open Approach */
"0QP334Z"  =  "1"  /* Removal of Int Fix from L Pelvic Bone, Perc Approach */
"0QP335Z"  =  "1"  /* Removal of Ext Fix from L Pelvic Bone, Perc Approach */
"0QP337Z"  =  "1"  /* Removal of Autol Sub from L Pelvic Bone, Perc Approach */
"0QP33JZ"  =  "1"  /* Removal of Synth Sub from L Pelvic Bone, Perc Approach */
"0QP33KZ"  =  "1"  /* Removal of Nonaut Sub from L Pelvic Bone, Perc Approach */
"0QP344Z"  =  "1"  /* Removal of Int Fix from L Pelvic Bone, Perc Endo Approach */
"0QP345Z"  =  "1"  /* Removal of Ext Fix from L Pelvic Bone, Perc Endo Approach */
"0QP347Z"  =  "1"  /* Removal of Autol Sub from L Pelvic Bone, Perc Endo Approach */
"0QP34JZ"  =  "1"  /* Removal of Synth Sub from L Pelvic Bone, Perc Endo Approach */
"0QP34KZ"  =  "1"  /* Removal of Nonaut Sub from L Pelvic Bone, Perc Endo Approach */
"0QP404Z"  =  "1"  /* Removal of Int Fix from R Acetabulum, Open Approach */
"0QP407Z"  =  "1"  /* Removal of Autol Sub from R Acetabulum, Open Approach */
"0QP40JZ"  =  "1"  /* Removal of Synth Sub from R Acetabulum, Open Approach */
"0QP40KZ"  =  "1"  /* Removal of Nonaut Sub from R Acetabulum, Open Approach */
"0QP434Z"  =  "1"  /* Removal of Int Fix from R Acetabulum, Perc Approach */
"0QP437Z"  =  "1"  /* Removal of Autol Sub from R Acetabulum, Perc Approach */
"0QP43JZ"  =  "1"  /* Removal of Synth Sub from R Acetabulum, Perc Approach */
"0QP43KZ"  =  "1"  /* Removal of Nonaut Sub from R Acetabulum, Perc Approach */
"0QP444Z"  =  "1"  /* Removal of Int Fix from R Acetabulum, Perc Endo Approach */
"0QP447Z"  =  "1"  /* Removal of Autol Sub from R Acetabulum, Perc Endo Approach */
"0QP44JZ"  =  "1"  /* Removal of Synth Sub from R Acetabulum, Perc Endo Approach */
"0QP44KZ"  =  "1"  /* Removal of Nonaut Sub from R Acetabulum, Perc Endo Approach */
"0QP504Z"  =  "1"  /* Removal of Int Fix from L Acetabulum, Open Approach */
"0QP507Z"  =  "1"  /* Removal of Autol Sub from L Acetabulum, Open Approach */
"0QP50JZ"  =  "1"  /* Removal of Synth Sub from L Acetabulum, Open Approach */
"0QP50KZ"  =  "1"  /* Removal of Nonaut Sub from L Acetabulum, Open Approach */
"0QP534Z"  =  "1"  /* Removal of Int Fix from L Acetabulum, Perc Approach */
"0QP537Z"  =  "1"  /* Removal of Autol Sub from L Acetabulum, Perc Approach */
"0QP53JZ"  =  "1"  /* Removal of Synth Sub from L Acetabulum, Perc Approach */
"0QP53KZ"  =  "1"  /* Removal of Nonaut Sub from L Acetabulum, Perc Approach */
"0QP544Z"  =  "1"  /* Removal of Int Fix from L Acetabulum, Perc Endo Approach */
"0QP547Z"  =  "1"  /* Removal of Autol Sub from L Acetabulum, Perc Endo Approach */
"0QP54JZ"  =  "1"  /* Removal of Synth Sub from L Acetabulum, Perc Endo Approach */
"0QP54KZ"  =  "1"  /* Removal of Nonaut Sub from L Acetabulum, Perc Endo Approach */
"0QP604Z"  =  "1"  /* Removal of Int Fix from R Up Femur, Open Approach */
"0QP605Z"  =  "1"  /* Removal of Ext Fix from R Up Femur, Open Approach */
"0QP607Z"  =  "1"  /* Removal of Autol Sub from R Up Femur, Open Approach */
"0QP60JZ"  =  "1"  /* Removal of Synth Sub from R Up Femur, Open Approach */
"0QP60KZ"  =  "1"  /* Removal of Nonaut Sub from R Up Femur, Open Approach */
"0QP634Z"  =  "1"  /* Removal of Int Fix from R Up Femur, Perc Approach */
"0QP635Z"  =  "1"  /* Removal of Ext Fix from R Up Femur, Perc Approach */
"0QP637Z"  =  "1"  /* Removal of Autol Sub from R Up Femur, Perc Approach */
"0QP63JZ"  =  "1"  /* Removal of Synth Sub from R Up Femur, Perc Approach */
"0QP63KZ"  =  "1"  /* Removal of Nonaut Sub from R Up Femur, Perc Approach */
"0QP644Z"  =  "1"  /* Removal of Int Fix from R Up Femur, Perc Endo Approach */
"0QP645Z"  =  "1"  /* Removal of Ext Fix from R Up Femur, Perc Endo Approach */
"0QP647Z"  =  "1"  /* Removal of Autol Sub from R Up Femur, Perc Endo Approach */
"0QP64JZ"  =  "1"  /* Removal of Synth Sub from R Up Femur, Perc Endo Approach */
"0QP64KZ"  =  "1"  /* Removal of Nonaut Sub from R Up Femur, Perc Endo Approach */
"0QP704Z"  =  "1"  /* Removal of Int Fix from L Up Femur, Open Approach */
"0QP705Z"  =  "1"  /* Removal of Ext Fix from L Up Femur, Open Approach */
"0QP707Z"  =  "1"  /* Removal of Autol Sub from L Up Femur, Open Approach */
"0QP70JZ"  =  "1"  /* Removal of Synth Sub from L Up Femur, Open Approach */
"0QP70KZ"  =  "1"  /* Removal of Nonaut Sub from L Up Femur, Open Approach */
"0QP734Z"  =  "1"  /* Removal of Int Fix from L Up Femur, Perc Approach */
"0QP735Z"  =  "1"  /* Removal of Ext Fix from L Up Femur, Perc Approach */
"0QP737Z"  =  "1"  /* Removal of Autol Sub from L Up Femur, Perc Approach */
"0QP73JZ"  =  "1"  /* Removal of Synth Sub from L Up Femur, Perc Approach */
"0QP73KZ"  =  "1"  /* Removal of Nonaut Sub from L Up Femur, Perc Approach */
"0QP744Z"  =  "1"  /* Removal of Int Fix from L Up Femur, Perc Endo Approach */
"0QP745Z"  =  "1"  /* Removal of Ext Fix from L Up Femur, Perc Endo Approach */
"0QP747Z"  =  "1"  /* Removal of Autol Sub from L Up Femur, Perc Endo Approach */
"0QP74JZ"  =  "1"  /* Removal of Synth Sub from L Up Femur, Perc Endo Approach */
"0QP74KZ"  =  "1"  /* Removal of Nonaut Sub from L Up Femur, Perc Endo Approach */
"0QP804Z"  =  "1"  /* Removal of Int Fix from R Femur Shaft, Open Approach */
"0QP805Z"  =  "1"  /* Removal of Ext Fix from R Femur Shaft, Open Approach */
"0QP807Z"  =  "1"  /* Removal of Autol Sub from R Femur Shaft, Open Approach */
"0QP80JZ"  =  "1"  /* Removal of Synth Sub from R Femur Shaft, Open Approach */
"0QP80KZ"  =  "1"  /* Removal of Nonaut Sub from R Femur Shaft, Open Approach */
"0QP834Z"  =  "1"  /* Removal of Int Fix from R Femur Shaft, Perc Approach */
"0QP835Z"  =  "1"  /* Removal of Ext Fix from R Femur Shaft, Perc Approach */
"0QP837Z"  =  "1"  /* Removal of Autol Sub from R Femur Shaft, Perc Approach */
"0QP83JZ"  =  "1"  /* Removal of Synth Sub from R Femur Shaft, Perc Approach */
"0QP83KZ"  =  "1"  /* Removal of Nonaut Sub from R Femur Shaft, Perc Approach */
"0QP844Z"  =  "1"  /* Removal of Int Fix from R Femur Shaft, Perc Endo Approach */
"0QP845Z"  =  "1"  /* Removal of Ext Fix from R Femur Shaft, Perc Endo Approach */
"0QP847Z"  =  "1"  /* Removal of Autol Sub from R Femur Shaft, Perc Endo Approach */
"0QP84JZ"  =  "1"  /* Removal of Synth Sub from R Femur Shaft, Perc Endo Approach */
"0QP84KZ"  =  "1"  /* Removal of Nonaut Sub from R Femur Shaft, Perc Endo Approach */
"0QP904Z"  =  "1"  /* Removal of Int Fix from L Femur Shaft, Open Approach */
"0QP905Z"  =  "1"  /* Removal of Ext Fix from L Femur Shaft, Open Approach */
"0QP907Z"  =  "1"  /* Removal of Autol Sub from L Femur Shaft, Open Approach */
"0QP90JZ"  =  "1"  /* Removal of Synth Sub from L Femur Shaft, Open Approach */
"0QP90KZ"  =  "1"  /* Removal of Nonaut Sub from L Femur Shaft, Open Approach */
"0QP934Z"  =  "1"  /* Removal of Int Fix from L Femur Shaft, Perc Approach */
"0QP935Z"  =  "1"  /* Removal of Ext Fix from L Femur Shaft, Perc Approach */
"0QP937Z"  =  "1"  /* Removal of Autol Sub from L Femur Shaft, Perc Approach */
"0QP93JZ"  =  "1"  /* Removal of Synth Sub from L Femur Shaft, Perc Approach */
"0QP93KZ"  =  "1"  /* Removal of Nonaut Sub from L Femur Shaft, Perc Approach */
"0QP944Z"  =  "1"  /* Removal of Int Fix from L Femur Shaft, Perc Endo Approach */
"0QP945Z"  =  "1"  /* Removal of Ext Fix from L Femur Shaft, Perc Endo Approach */
"0QP947Z"  =  "1"  /* Removal of Autol Sub from L Femur Shaft, Perc Endo Approach */
"0QP94JZ"  =  "1"  /* Removal of Synth Sub from L Femur Shaft, Perc Endo Approach */
"0QP94KZ"  =  "1"  /* Removal of Nonaut Sub from L Femur Shaft, Perc Endo Approach */
"0QPB04Z"  =  "1"  /* Removal of Int Fix from R Low Femur, Open Approach */
"0QPB05Z"  =  "1"  /* Removal of Ext Fix from R Low Femur, Open Approach */
"0QPB07Z"  =  "1"  /* Removal of Autol Sub from R Low Femur, Open Approach */
"0QPB0JZ"  =  "1"  /* Removal of Synth Sub from R Low Femur, Open Approach */
"0QPB0KZ"  =  "1"  /* Removal of Nonaut Sub from R Low Femur, Open Approach */
"0QPB34Z"  =  "1"  /* Removal of Int Fix from R Low Femur, Perc Approach */
"0QPB35Z"  =  "1"  /* Removal of Ext Fix from R Low Femur, Perc Approach */
"0QPB37Z"  =  "1"  /* Removal of Autol Sub from R Low Femur, Perc Approach */
"0QPB3JZ"  =  "1"  /* Removal of Synth Sub from R Low Femur, Perc Approach */
"0QPB3KZ"  =  "1"  /* Removal of Nonaut Sub from R Low Femur, Perc Approach */
"0QPB44Z"  =  "1"  /* Removal of Int Fix from R Low Femur, Perc Endo Approach */
"0QPB45Z"  =  "1"  /* Removal of Ext Fix from R Low Femur, Perc Endo Approach */
"0QPB47Z"  =  "1"  /* Removal of Autol Sub from R Low Femur, Perc Endo Approach */
"0QPB4JZ"  =  "1"  /* Removal of Synth Sub from R Low Femur, Perc Endo Approach */
"0QPB4KZ"  =  "1"  /* Removal of Nonaut Sub from R Low Femur, Perc Endo Approach */
"0QPC04Z"  =  "1"  /* Removal of Int Fix from L Low Femur, Open Approach */
"0QPC05Z"  =  "1"  /* Removal of Ext Fix from L Low Femur, Open Approach */
"0QPC07Z"  =  "1"  /* Removal of Autol Sub from L Low Femur, Open Approach */
"0QPC0JZ"  =  "1"  /* Removal of Synth Sub from L Low Femur, Open Approach */
"0QPC0KZ"  =  "1"  /* Removal of Nonaut Sub from L Low Femur, Open Approach */
"0QPC34Z"  =  "1"  /* Removal of Int Fix from L Low Femur, Perc Approach */
"0QPC35Z"  =  "1"  /* Removal of Ext Fix from L Low Femur, Perc Approach */
"0QPC37Z"  =  "1"  /* Removal of Autol Sub from L Low Femur, Perc Approach */
"0QPC3JZ"  =  "1"  /* Removal of Synth Sub from L Low Femur, Perc Approach */
"0QPC3KZ"  =  "1"  /* Removal of Nonaut Sub from L Low Femur, Perc Approach */
"0QPC44Z"  =  "1"  /* Removal of Int Fix from L Low Femur, Perc Endo Approach */
"0QPC45Z"  =  "1"  /* Removal of Ext Fix from L Low Femur, Perc Endo Approach */
"0QPC47Z"  =  "1"  /* Removal of Autol Sub from L Low Femur, Perc Endo Approach */
"0QPC4JZ"  =  "1"  /* Removal of Synth Sub from L Low Femur, Perc Endo Approach */
"0QPC4KZ"  =  "1"  /* Removal of Nonaut Sub from L Low Femur, Perc Endo Approach */
"0QPD04Z"  =  "1"  /* Removal of Int Fix from R Patella, Open Approach */
"0QPD05Z"  =  "1"  /* Removal of Ext Fix from R Patella, Open Approach */
"0QPD07Z"  =  "1"  /* Removal of Autol Sub from R Patella, Open Approach */
"0QPD0JZ"  =  "1"  /* Removal of Synth Sub from R Patella, Open Approach */
"0QPD0KZ"  =  "1"  /* Removal of Nonaut Sub from R Patella, Open Approach */
"0QPD34Z"  =  "1"  /* Removal of Int Fix from R Patella, Perc Approach */
"0QPD35Z"  =  "1"  /* Removal of Ext Fix from R Patella, Perc Approach */
"0QPD37Z"  =  "1"  /* Removal of Autol Sub from R Patella, Perc Approach */
"0QPD3JZ"  =  "1"  /* Removal of Synth Sub from R Patella, Perc Approach */
"0QPD3KZ"  =  "1"  /* Removal of Nonaut Sub from R Patella, Perc Approach */
"0QPD44Z"  =  "1"  /* Removal of Int Fix from R Patella, Perc Endo Approach */
"0QPD45Z"  =  "1"  /* Removal of Ext Fix from R Patella, Perc Endo Approach */
"0QPD47Z"  =  "1"  /* Removal of Autol Sub from R Patella, Perc Endo Approach */
"0QPD4JZ"  =  "1"  /* Removal of Synth Sub from R Patella, Perc Endo Approach */
"0QPD4KZ"  =  "1"  /* Removal of Nonaut Sub from R Patella, Perc Endo Approach */
"0QPF04Z"  =  "1"  /* Removal of Int Fix from L Patella, Open Approach */
"0QPF05Z"  =  "1"  /* Removal of Ext Fix from L Patella, Open Approach */
"0QPF07Z"  =  "1"  /* Removal of Autol Sub from L Patella, Open Approach */
"0QPF0JZ"  =  "1"  /* Removal of Synth Sub from L Patella, Open Approach */
"0QPF0KZ"  =  "1"  /* Removal of Nonaut Sub from L Patella, Open Approach */
"0QPF34Z"  =  "1"  /* Removal of Int Fix from L Patella, Perc Approach */
"0QPF35Z"  =  "1"  /* Removal of Ext Fix from L Patella, Perc Approach */
"0QPF37Z"  =  "1"  /* Removal of Autol Sub from L Patella, Perc Approach */
"0QPF3JZ"  =  "1"  /* Removal of Synth Sub from L Patella, Perc Approach */
"0QPF3KZ"  =  "1"  /* Removal of Nonaut Sub from L Patella, Perc Approach */
"0QPF44Z"  =  "1"  /* Removal of Int Fix from L Patella, Perc Endo Approach */
"0QPF45Z"  =  "1"  /* Removal of Ext Fix from L Patella, Perc Endo Approach */
"0QPF47Z"  =  "1"  /* Removal of Autol Sub from L Patella, Perc Endo Approach */
"0QPF4JZ"  =  "1"  /* Removal of Synth Sub from L Patella, Perc Endo Approach */
"0QPF4KZ"  =  "1"  /* Removal of Nonaut Sub from L Patella, Perc Endo Approach */
"0QPG04Z"  =  "1"  /* Removal of Int Fix from R Tibia, Open Approach */
"0QPG05Z"  =  "1"  /* Removal of Ext Fix from R Tibia, Open Approach */
"0QPG07Z"  =  "1"  /* Removal of Autol Sub from R Tibia, Open Approach */
"0QPG0JZ"  =  "1"  /* Removal of Synthetic Substitute from R Tibia, Open Approach */
"0QPG0KZ"  =  "1"  /* Removal of Nonaut Sub from R Tibia, Open Approach */
"0QPG34Z"  =  "1"  /* Removal of Int Fix from R Tibia, Perc Approach */
"0QPG35Z"  =  "1"  /* Removal of Ext Fix from R Tibia, Perc Approach */
"0QPG37Z"  =  "1"  /* Removal of Autol Sub from R Tibia, Perc Approach */
"0QPG3JZ"  =  "1"  /* Removal of Synthetic Substitute from R Tibia, Perc Approach */
"0QPG3KZ"  =  "1"  /* Removal of Nonaut Sub from R Tibia, Perc Approach */
"0QPG44Z"  =  "1"  /* Removal of Int Fix from R Tibia, Perc Endo Approach */
"0QPG45Z"  =  "1"  /* Removal of Ext Fix from R Tibia, Perc Endo Approach */
"0QPG47Z"  =  "1"  /* Removal of Autol Sub from R Tibia, Perc Endo Approach */
"0QPG4JZ"  =  "1"  /* Removal of Synth Sub from R Tibia, Perc Endo Approach */
"0QPG4KZ"  =  "1"  /* Removal of Nonaut Sub from R Tibia, Perc Endo Approach */
"0QPH04Z"  =  "1"  /* Removal of Int Fix from L Tibia, Open Approach */
"0QPH05Z"  =  "1"  /* Removal of Ext Fix from L Tibia, Open Approach */
"0QPH07Z"  =  "1"  /* Removal of Autol Sub from L Tibia, Open Approach */
"0QPH0JZ"  =  "1"  /* Removal of Synthetic Substitute from L Tibia, Open Approach */
"0QPH0KZ"  =  "1"  /* Removal of Nonaut Sub from L Tibia, Open Approach */
"0QPH34Z"  =  "1"  /* Removal of Int Fix from L Tibia, Perc Approach */
"0QPH35Z"  =  "1"  /* Removal of Ext Fix from L Tibia, Perc Approach */
"0QPH37Z"  =  "1"  /* Removal of Autol Sub from L Tibia, Perc Approach */
"0QPH3JZ"  =  "1"  /* Removal of Synthetic Substitute from L Tibia, Perc Approach */
"0QPH3KZ"  =  "1"  /* Removal of Nonaut Sub from L Tibia, Perc Approach */
"0QPH44Z"  =  "1"  /* Removal of Int Fix from L Tibia, Perc Endo Approach */
"0QPH45Z"  =  "1"  /* Removal of Ext Fix from L Tibia, Perc Endo Approach */
"0QPH47Z"  =  "1"  /* Removal of Autol Sub from L Tibia, Perc Endo Approach */
"0QPH4JZ"  =  "1"  /* Removal of Synth Sub from L Tibia, Perc Endo Approach */
"0QPH4KZ"  =  "1"  /* Removal of Nonaut Sub from L Tibia, Perc Endo Approach */
"0QPJ04Z"  =  "1"  /* Removal of Int Fix from R Fibula, Open Approach */
"0QPJ05Z"  =  "1"  /* Removal of Ext Fix from R Fibula, Open Approach */
"0QPJ07Z"  =  "1"  /* Removal of Autol Sub from R Fibula, Open Approach */
"0QPJ0JZ"  =  "1"  /* Removal of Synthetic Substitute from R Fibula, Open Approach */
"0QPJ0KZ"  =  "1"  /* Removal of Nonaut Sub from R Fibula, Open Approach */
"0QPJ34Z"  =  "1"  /* Removal of Int Fix from R Fibula, Perc Approach */
"0QPJ35Z"  =  "1"  /* Removal of Ext Fix from R Fibula, Perc Approach */
"0QPJ37Z"  =  "1"  /* Removal of Autol Sub from R Fibula, Perc Approach */
"0QPJ3JZ"  =  "1"  /* Removal of Synthetic Substitute from R Fibula, Perc Approach */
"0QPJ3KZ"  =  "1"  /* Removal of Nonaut Sub from R Fibula, Perc Approach */
"0QPJ44Z"  =  "1"  /* Removal of Int Fix from R Fibula, Perc Endo Approach */
"0QPJ45Z"  =  "1"  /* Removal of Ext Fix from R Fibula, Perc Endo Approach */
"0QPJ47Z"  =  "1"  /* Removal of Autol Sub from R Fibula, Perc Endo Approach */
"0QPJ4JZ"  =  "1"  /* Removal of Synth Sub from R Fibula, Perc Endo Approach */
"0QPJ4KZ"  =  "1"  /* Removal of Nonaut Sub from R Fibula, Perc Endo Approach */
"0QPK04Z"  =  "1"  /* Removal of Int Fix from L Fibula, Open Approach */
"0QPK05Z"  =  "1"  /* Removal of Ext Fix from L Fibula, Open Approach */
"0QPK07Z"  =  "1"  /* Removal of Autol Sub from L Fibula, Open Approach */
"0QPK0JZ"  =  "1"  /* Removal of Synthetic Substitute from L Fibula, Open Approach */
"0QPK0KZ"  =  "1"  /* Removal of Nonaut Sub from L Fibula, Open Approach */
"0QPK34Z"  =  "1"  /* Removal of Int Fix from L Fibula, Perc Approach */
"0QPK35Z"  =  "1"  /* Removal of Ext Fix from L Fibula, Perc Approach */
"0QPK37Z"  =  "1"  /* Removal of Autol Sub from L Fibula, Perc Approach */
"0QPK3JZ"  =  "1"  /* Removal of Synthetic Substitute from L Fibula, Perc Approach */
"0QPK3KZ"  =  "1"  /* Removal of Nonaut Sub from L Fibula, Perc Approach */
"0QPK44Z"  =  "1"  /* Removal of Int Fix from L Fibula, Perc Endo Approach */
"0QPK45Z"  =  "1"  /* Removal of Ext Fix from L Fibula, Perc Endo Approach */
"0QPK47Z"  =  "1"  /* Removal of Autol Sub from L Fibula, Perc Endo Approach */
"0QPK4JZ"  =  "1"  /* Removal of Synth Sub from L Fibula, Perc Endo Approach */
"0QPK4KZ"  =  "1"  /* Removal of Nonaut Sub from L Fibula, Perc Endo Approach */
"0QPL04Z"  =  "1"  /* Removal of Int Fix from R Tarsal, Open Approach */
"0QPL05Z"  =  "1"  /* Removal of Ext Fix from R Tarsal, Open Approach */
"0QPL07Z"  =  "1"  /* Removal of Autol Sub from R Tarsal, Open Approach */
"0QPL0JZ"  =  "1"  /* Removal of Synthetic Substitute from R Tarsal, Open Approach */
"0QPL0KZ"  =  "1"  /* Removal of Nonaut Sub from R Tarsal, Open Approach */
"0QPL34Z"  =  "1"  /* Removal of Int Fix from R Tarsal, Perc Approach */
"0QPL35Z"  =  "1"  /* Removal of Ext Fix from R Tarsal, Perc Approach */
"0QPL37Z"  =  "1"  /* Removal of Autol Sub from R Tarsal, Perc Approach */
"0QPL3JZ"  =  "1"  /* Removal of Synthetic Substitute from R Tarsal, Perc Approach */
"0QPL3KZ"  =  "1"  /* Removal of Nonaut Sub from R Tarsal, Perc Approach */
"0QPL44Z"  =  "1"  /* Removal of Int Fix from R Tarsal, Perc Endo Approach */
"0QPL45Z"  =  "1"  /* Removal of Ext Fix from R Tarsal, Perc Endo Approach */
"0QPL47Z"  =  "1"  /* Removal of Autol Sub from R Tarsal, Perc Endo Approach */
"0QPL4JZ"  =  "1"  /* Removal of Synth Sub from R Tarsal, Perc Endo Approach */
"0QPL4KZ"  =  "1"  /* Removal of Nonaut Sub from R Tarsal, Perc Endo Approach */
"0QPM04Z"  =  "1"  /* Removal of Int Fix from L Tarsal, Open Approach */
"0QPM05Z"  =  "1"  /* Removal of Ext Fix from L Tarsal, Open Approach */
"0QPM07Z"  =  "1"  /* Removal of Autol Sub from L Tarsal, Open Approach */
"0QPM0JZ"  =  "1"  /* Removal of Synthetic Substitute from L Tarsal, Open Approach */
"0QPM0KZ"  =  "1"  /* Removal of Nonaut Sub from L Tarsal, Open Approach */
"0QPM34Z"  =  "1"  /* Removal of Int Fix from L Tarsal, Perc Approach */
"0QPM35Z"  =  "1"  /* Removal of Ext Fix from L Tarsal, Perc Approach */
"0QPM37Z"  =  "1"  /* Removal of Autol Sub from L Tarsal, Perc Approach */
"0QPM3JZ"  =  "1"  /* Removal of Synthetic Substitute from L Tarsal, Perc Approach */
"0QPM3KZ"  =  "1"  /* Removal of Nonaut Sub from L Tarsal, Perc Approach */
"0QPM44Z"  =  "1"  /* Removal of Int Fix from L Tarsal, Perc Endo Approach */
"0QPM45Z"  =  "1"  /* Removal of Ext Fix from L Tarsal, Perc Endo Approach */
"0QPM47Z"  =  "1"  /* Removal of Autol Sub from L Tarsal, Perc Endo Approach */
"0QPM4JZ"  =  "1"  /* Removal of Synth Sub from L Tarsal, Perc Endo Approach */
"0QPM4KZ"  =  "1"  /* Removal of Nonaut Sub from L Tarsal, Perc Endo Approach */
"0QPN04Z"  =  "1"  /* Removal of Int Fix from R Metatarsal, Open Approach */
"0QPN05Z"  =  "1"  /* Removal of Ext Fix from R Metatarsal, Open Approach */
"0QPN07Z"  =  "1"  /* Removal of Autol Sub from R Metatarsal, Open Approach */
"0QPN0JZ"  =  "1"  /* Removal of Synth Sub from R Metatarsal, Open Approach */
"0QPN0KZ"  =  "1"  /* Removal of Nonaut Sub from R Metatarsal, Open Approach */
"0QPN34Z"  =  "1"  /* Removal of Int Fix from R Metatarsal, Perc Approach */
"0QPN35Z"  =  "1"  /* Removal of Ext Fix from R Metatarsal, Perc Approach */
"0QPN37Z"  =  "1"  /* Removal of Autol Sub from R Metatarsal, Perc Approach */
"0QPN3JZ"  =  "1"  /* Removal of Synth Sub from R Metatarsal, Perc Approach */
"0QPN3KZ"  =  "1"  /* Removal of Nonaut Sub from R Metatarsal, Perc Approach */
"0QPN44Z"  =  "1"  /* Removal of Int Fix from R Metatarsal, Perc Endo Approach */
"0QPN45Z"  =  "1"  /* Removal of Ext Fix from R Metatarsal, Perc Endo Approach */
"0QPN47Z"  =  "1"  /* Removal of Autol Sub from R Metatarsal, Perc Endo Approach */
"0QPN4JZ"  =  "1"  /* Removal of Synth Sub from R Metatarsal, Perc Endo Approach */
"0QPN4KZ"  =  "1"  /* Removal of Nonaut Sub from R Metatarsal, Perc Endo Approach */
"0QPP04Z"  =  "1"  /* Removal of Int Fix from L Metatarsal, Open Approach */
"0QPP05Z"  =  "1"  /* Removal of Ext Fix from L Metatarsal, Open Approach */
"0QPP07Z"  =  "1"  /* Removal of Autol Sub from L Metatarsal, Open Approach */
"0QPP0JZ"  =  "1"  /* Removal of Synth Sub from L Metatarsal, Open Approach */
"0QPP0KZ"  =  "1"  /* Removal of Nonaut Sub from L Metatarsal, Open Approach */
"0QPP34Z"  =  "1"  /* Removal of Int Fix from L Metatarsal, Perc Approach */
"0QPP35Z"  =  "1"  /* Removal of Ext Fix from L Metatarsal, Perc Approach */
"0QPP37Z"  =  "1"  /* Removal of Autol Sub from L Metatarsal, Perc Approach */
"0QPP3JZ"  =  "1"  /* Removal of Synth Sub from L Metatarsal, Perc Approach */
"0QPP3KZ"  =  "1"  /* Removal of Nonaut Sub from L Metatarsal, Perc Approach */
"0QPP44Z"  =  "1"  /* Removal of Int Fix from L Metatarsal, Perc Endo Approach */
"0QPP45Z"  =  "1"  /* Removal of Ext Fix from L Metatarsal, Perc Endo Approach */
"0QPP47Z"  =  "1"  /* Removal of Autol Sub from L Metatarsal, Perc Endo Approach */
"0QPP4JZ"  =  "1"  /* Removal of Synth Sub from L Metatarsal, Perc Endo Approach */
"0QPP4KZ"  =  "1"  /* Removal of Nonaut Sub from L Metatarsal, Perc Endo Approach */
"0QPQ04Z"  =  "1"  /* Removal of Int Fix from R Toe Phalanx, Open Approach */
"0QPQ05Z"  =  "1"  /* Removal of Ext Fix from R Toe Phalanx, Open Approach */
"0QPQ07Z"  =  "1"  /* Removal of Autol Sub from R Toe Phalanx, Open Approach */
"0QPQ0JZ"  =  "1"  /* Removal of Synth Sub from R Toe Phalanx, Open Approach */
"0QPQ0KZ"  =  "1"  /* Removal of Nonaut Sub from R Toe Phalanx, Open Approach */
"0QPQ34Z"  =  "1"  /* Removal of Int Fix from R Toe Phalanx, Perc Approach */
"0QPQ35Z"  =  "1"  /* Removal of Ext Fix from R Toe Phalanx, Perc Approach */
"0QPQ37Z"  =  "1"  /* Removal of Autol Sub from R Toe Phalanx, Perc Approach */
"0QPQ3JZ"  =  "1"  /* Removal of Synth Sub from R Toe Phalanx, Perc Approach */
"0QPQ3KZ"  =  "1"  /* Removal of Nonaut Sub from R Toe Phalanx, Perc Approach */
"0QPQ44Z"  =  "1"  /* Removal of Int Fix from R Toe Phalanx, Perc Endo Approach */
"0QPQ45Z"  =  "1"  /* Removal of Ext Fix from R Toe Phalanx, Perc Endo Approach */
"0QPQ47Z"  =  "1"  /* Removal of Autol Sub from R Toe Phalanx, Perc Endo Approach */
"0QPQ4JZ"  =  "1"  /* Removal of Synth Sub from R Toe Phalanx, Perc Endo Approach */
"0QPQ4KZ"  =  "1"  /* Removal of Nonaut Sub from R Toe Phalanx, Perc Endo Approach */
"0QPR04Z"  =  "1"  /* Removal of Int Fix from L Toe Phalanx, Open Approach */
"0QPR05Z"  =  "1"  /* Removal of Ext Fix from L Toe Phalanx, Open Approach */
"0QPR07Z"  =  "1"  /* Removal of Autol Sub from L Toe Phalanx, Open Approach */
"0QPR0JZ"  =  "1"  /* Removal of Synth Sub from L Toe Phalanx, Open Approach */
"0QPR0KZ"  =  "1"  /* Removal of Nonaut Sub from L Toe Phalanx, Open Approach */
"0QPR34Z"  =  "1"  /* Removal of Int Fix from L Toe Phalanx, Perc Approach */
"0QPR35Z"  =  "1"  /* Removal of Ext Fix from L Toe Phalanx, Perc Approach */
"0QPR37Z"  =  "1"  /* Removal of Autol Sub from L Toe Phalanx, Perc Approach */
"0QPR3JZ"  =  "1"  /* Removal of Synth Sub from L Toe Phalanx, Perc Approach */
"0QPR3KZ"  =  "1"  /* Removal of Nonaut Sub from L Toe Phalanx, Perc Approach */
"0QPR44Z"  =  "1"  /* Removal of Int Fix from L Toe Phalanx, Perc Endo Approach */
"0QPR45Z"  =  "1"  /* Removal of Ext Fix from L Toe Phalanx, Perc Endo Approach */
"0QPR47Z"  =  "1"  /* Removal of Autol Sub from L Toe Phalanx, Perc Endo Approach */
"0QPR4JZ"  =  "1"  /* Removal of Synth Sub from L Toe Phalanx, Perc Endo Approach */
"0QPR4KZ"  =  "1"  /* Removal of Nonaut Sub from L Toe Phalanx, Perc Endo Approach */
"0QPS04Z"  =  "1"  /* Removal of Int Fix from Coccyx, Open Approach */
"0QPS07Z"  =  "1"  /* Removal of Autol Sub from Coccyx, Open Approach */
"0QPS0JZ"  =  "1"  /* Removal of Synthetic Substitute from Coccyx, Open Approach */
"0QPS0KZ"  =  "1"  /* Removal of Nonaut Sub from Coccyx, Open Approach */
"0QPS34Z"  =  "1"  /* Removal of Int Fix from Coccyx, Perc Approach */
"0QPS37Z"  =  "1"  /* Removal of Autol Sub from Coccyx, Perc Approach */
"0QPS3JZ"  =  "1"  /* Removal of Synthetic Substitute from Coccyx, Perc Approach */
"0QPS3KZ"  =  "1"  /* Removal of Nonaut Sub from Coccyx, Perc Approach */
"0QPS44Z"  =  "1"  /* Removal of Int Fix from Coccyx, Perc Endo Approach */
"0QPS47Z"  =  "1"  /* Removal of Autol Sub from Coccyx, Perc Endo Approach */
"0QPS4JZ"  =  "1"  /* Removal of Synth Sub from Coccyx, Perc Endo Approach */
"0QPS4KZ"  =  "1"  /* Removal of Nonaut Sub from Coccyx, Perc Endo Approach */
"0QPY00Z"  =  "1"  /* Removal of Drainage Device from Lower Bone, Open Approach */
"0QPY0MZ"  =  "1"  /* Removal of Bone Stim from Low Bone, Open Approach */
"0QPY30Z"  =  "1"  /* Removal of Drainage Device from Lower Bone, Perc Approach */
"0QPY3MZ"  =  "1"  /* Removal of Bone Stim from Low Bone, Perc Approach */
"0QPY40Z"  =  "1"  /* Removal of Drainage Device from Low Bone, Perc Endo Approach */
"0QPY4MZ"  =  "1"  /* Removal of Bone Stim from Low Bone, Perc Endo Approach */
"0QQ00ZZ"  =  "1"  /* Repair Lumbar Vertebra, Open Approach */
"0QQ03ZZ"  =  "1"  /* Repair Lumbar Vertebra, Percutaneous Approach */
"0QQ04ZZ"  =  "1"  /* Repair Lumbar Vertebra, Percutaneous Endoscopic Approach */
"0QQ0XZZ"  =  "1"  /* Repair Lumbar Vertebra, External Approach */
"0QQ10ZZ"  =  "1"  /* Repair Sacrum, Open Approach */
"0QQ13ZZ"  =  "1"  /* Repair Sacrum, Percutaneous Approach */
"0QQ14ZZ"  =  "1"  /* Repair Sacrum, Percutaneous Endoscopic Approach */
"0QQ1XZZ"  =  "1"  /* Repair Sacrum, External Approach */
"0QQ20ZZ"  =  "1"  /* Repair Right Pelvic Bone, Open Approach */
"0QQ23ZZ"  =  "1"  /* Repair Right Pelvic Bone, Percutaneous Approach */
"0QQ24ZZ"  =  "1"  /* Repair Right Pelvic Bone, Percutaneous Endoscopic Approach */
"0QQ2XZZ"  =  "1"  /* Repair Right Pelvic Bone, External Approach */
"0QQ30ZZ"  =  "1"  /* Repair Left Pelvic Bone, Open Approach */
"0QQ33ZZ"  =  "1"  /* Repair Left Pelvic Bone, Percutaneous Approach */
"0QQ34ZZ"  =  "1"  /* Repair Left Pelvic Bone, Percutaneous Endoscopic Approach */
"0QQ3XZZ"  =  "1"  /* Repair Left Pelvic Bone, External Approach */
"0QQ40ZZ"  =  "1"  /* Repair Right Acetabulum, Open Approach */
"0QQ43ZZ"  =  "1"  /* Repair Right Acetabulum, Percutaneous Approach */
"0QQ44ZZ"  =  "1"  /* Repair Right Acetabulum, Percutaneous Endoscopic Approach */
"0QQ4XZZ"  =  "1"  /* Repair Right Acetabulum, External Approach */
"0QQ50ZZ"  =  "1"  /* Repair Left Acetabulum, Open Approach */
"0QQ53ZZ"  =  "1"  /* Repair Left Acetabulum, Percutaneous Approach */
"0QQ54ZZ"  =  "1"  /* Repair Left Acetabulum, Percutaneous Endoscopic Approach */
"0QQ5XZZ"  =  "1"  /* Repair Left Acetabulum, External Approach */
"0QQ60ZZ"  =  "1"  /* Repair Right Upper Femur, Open Approach */
"0QQ63ZZ"  =  "1"  /* Repair Right Upper Femur, Percutaneous Approach */
"0QQ64ZZ"  =  "1"  /* Repair Right Upper Femur, Percutaneous Endoscopic Approach */
"0QQ6XZZ"  =  "1"  /* Repair Right Upper Femur, External Approach */
"0QQ70ZZ"  =  "1"  /* Repair Left Upper Femur, Open Approach */
"0QQ73ZZ"  =  "1"  /* Repair Left Upper Femur, Percutaneous Approach */
"0QQ74ZZ"  =  "1"  /* Repair Left Upper Femur, Percutaneous Endoscopic Approach */
"0QQ7XZZ"  =  "1"  /* Repair Left Upper Femur, External Approach */
"0QQ80ZZ"  =  "1"  /* Repair Right Femoral Shaft, Open Approach */
"0QQ83ZZ"  =  "1"  /* Repair Right Femoral Shaft, Percutaneous Approach */
"0QQ84ZZ"  =  "1"  /* Repair Right Femoral Shaft, Percutaneous Endoscopic Approach */
"0QQ8XZZ"  =  "1"  /* Repair Right Femoral Shaft, External Approach */
"0QQ90ZZ"  =  "1"  /* Repair Left Femoral Shaft, Open Approach */
"0QQ93ZZ"  =  "1"  /* Repair Left Femoral Shaft, Percutaneous Approach */
"0QQ94ZZ"  =  "1"  /* Repair Left Femoral Shaft, Percutaneous Endoscopic Approach */
"0QQ9XZZ"  =  "1"  /* Repair Left Femoral Shaft, External Approach */
"0QQB0ZZ"  =  "1"  /* Repair Right Lower Femur, Open Approach */
"0QQB3ZZ"  =  "1"  /* Repair Right Lower Femur, Percutaneous Approach */
"0QQB4ZZ"  =  "1"  /* Repair Right Lower Femur, Percutaneous Endoscopic Approach */
"0QQBXZZ"  =  "1"  /* Repair Right Lower Femur, External Approach */
"0QQC0ZZ"  =  "1"  /* Repair Left Lower Femur, Open Approach */
"0QQC3ZZ"  =  "1"  /* Repair Left Lower Femur, Percutaneous Approach */
"0QQC4ZZ"  =  "1"  /* Repair Left Lower Femur, Percutaneous Endoscopic Approach */
"0QQCXZZ"  =  "1"  /* Repair Left Lower Femur, External Approach */
"0QQD0ZZ"  =  "1"  /* Repair Right Patella, Open Approach */
"0QQD3ZZ"  =  "1"  /* Repair Right Patella, Percutaneous Approach */
"0QQD4ZZ"  =  "1"  /* Repair Right Patella, Percutaneous Endoscopic Approach */
"0QQDXZZ"  =  "1"  /* Repair Right Patella, External Approach */
"0QQF0ZZ"  =  "1"  /* Repair Left Patella, Open Approach */
"0QQF3ZZ"  =  "1"  /* Repair Left Patella, Percutaneous Approach */
"0QQF4ZZ"  =  "1"  /* Repair Left Patella, Percutaneous Endoscopic Approach */
"0QQFXZZ"  =  "1"  /* Repair Left Patella, External Approach */
"0QQG0ZZ"  =  "1"  /* Repair Right Tibia, Open Approach */
"0QQG3ZZ"  =  "1"  /* Repair Right Tibia, Percutaneous Approach */
"0QQG4ZZ"  =  "1"  /* Repair Right Tibia, Percutaneous Endoscopic Approach */
"0QQGXZZ"  =  "1"  /* Repair Right Tibia, External Approach */
"0QQH0ZZ"  =  "1"  /* Repair Left Tibia, Open Approach */
"0QQH3ZZ"  =  "1"  /* Repair Left Tibia, Percutaneous Approach */
"0QQH4ZZ"  =  "1"  /* Repair Left Tibia, Percutaneous Endoscopic Approach */
"0QQHXZZ"  =  "1"  /* Repair Left Tibia, External Approach */
"0QQJ0ZZ"  =  "1"  /* Repair Right Fibula, Open Approach */
"0QQJ3ZZ"  =  "1"  /* Repair Right Fibula, Percutaneous Approach */
"0QQJ4ZZ"  =  "1"  /* Repair Right Fibula, Percutaneous Endoscopic Approach */
"0QQJXZZ"  =  "1"  /* Repair Right Fibula, External Approach */
"0QQK0ZZ"  =  "1"  /* Repair Left Fibula, Open Approach */
"0QQK3ZZ"  =  "1"  /* Repair Left Fibula, Percutaneous Approach */
"0QQK4ZZ"  =  "1"  /* Repair Left Fibula, Percutaneous Endoscopic Approach */
"0QQKXZZ"  =  "1"  /* Repair Left Fibula, External Approach */
"0QQL0ZZ"  =  "1"  /* Repair Right Tarsal, Open Approach */
"0QQL3ZZ"  =  "1"  /* Repair Right Tarsal, Percutaneous Approach */
"0QQL4ZZ"  =  "1"  /* Repair Right Tarsal, Percutaneous Endoscopic Approach */
"0QQLXZZ"  =  "1"  /* Repair Right Tarsal, External Approach */
"0QQM0ZZ"  =  "1"  /* Repair Left Tarsal, Open Approach */
"0QQM3ZZ"  =  "1"  /* Repair Left Tarsal, Percutaneous Approach */
"0QQM4ZZ"  =  "1"  /* Repair Left Tarsal, Percutaneous Endoscopic Approach */
"0QQMXZZ"  =  "1"  /* Repair Left Tarsal, External Approach */
"0QQN0ZZ"  =  "1"  /* Repair Right Metatarsal, Open Approach */
"0QQN3ZZ"  =  "1"  /* Repair Right Metatarsal, Percutaneous Approach */
"0QQN4ZZ"  =  "1"  /* Repair Right Metatarsal, Percutaneous Endoscopic Approach */
"0QQNXZZ"  =  "1"  /* Repair Right Metatarsal, External Approach */
"0QQP0ZZ"  =  "1"  /* Repair Left Metatarsal, Open Approach */
"0QQP3ZZ"  =  "1"  /* Repair Left Metatarsal, Percutaneous Approach */
"0QQP4ZZ"  =  "1"  /* Repair Left Metatarsal, Percutaneous Endoscopic Approach */
"0QQPXZZ"  =  "1"  /* Repair Left Metatarsal, External Approach */
"0QQQ0ZZ"  =  "1"  /* Repair Right Toe Phalanx, Open Approach */
"0QQQ3ZZ"  =  "1"  /* Repair Right Toe Phalanx, Percutaneous Approach */
"0QQQ4ZZ"  =  "1"  /* Repair Right Toe Phalanx, Percutaneous Endoscopic Approach */
"0QQQXZZ"  =  "1"  /* Repair Right Toe Phalanx, External Approach */
"0QQR0ZZ"  =  "1"  /* Repair Left Toe Phalanx, Open Approach */
"0QQR3ZZ"  =  "1"  /* Repair Left Toe Phalanx, Percutaneous Approach */
"0QQR4ZZ"  =  "1"  /* Repair Left Toe Phalanx, Percutaneous Endoscopic Approach */
"0QQRXZZ"  =  "1"  /* Repair Left Toe Phalanx, External Approach */
"0QQS0ZZ"  =  "1"  /* Repair Coccyx, Open Approach */
"0QQS3ZZ"  =  "1"  /* Repair Coccyx, Percutaneous Approach */
"0QQS4ZZ"  =  "1"  /* Repair Coccyx, Percutaneous Endoscopic Approach */
"0QQSXZZ"  =  "1"  /* Repair Coccyx, External Approach */
"0QR007Z"  =  "1"  /* Replacement of Lumbar Vertebra with Autol Sub, Open Approach */
"0QR00JZ"  =  "1"  /* Replacement of Lumbar Vertebra with Synth Sub, Open Approach */
"0QR00KZ"  =  "1"  /* Replacement of Lum Vertebra with Nonaut Sub, Open Approach */
"0QR037Z"  =  "1"  /* Replacement of Lumbar Vertebra with Autol Sub, Perc Approach */
"0QR03JZ"  =  "1"  /* Replacement of Lumbar Vertebra with Synth Sub, Perc Approach */
"0QR03KZ"  =  "1"  /* Replacement of Lum Vertebra with Nonaut Sub, Perc Approach */
"0QR047Z"  =  "1"  /* Replace of Lum Vertebra with Autol Sub, Perc Endo Approach */
"0QR04JZ"  =  "1"  /* Replace of Lum Vertebra with Synth Sub, Perc Endo Approach */
"0QR04KZ"  =  "1"  /* Replace of Lum Vertebra with Nonaut Sub, Perc Endo Approach */
"0QR107Z"  =  "1"  /* Replacement of Sacrum with Autol Sub, Open Approach */
"0QR10JZ"  =  "1"  /* Replacement of Sacrum with Synth Sub, Open Approach */
"0QR10KZ"  =  "1"  /* Replacement of Sacrum with Nonaut Sub, Open Approach */
"0QR137Z"  =  "1"  /* Replacement of Sacrum with Autol Sub, Perc Approach */
"0QR13JZ"  =  "1"  /* Replacement of Sacrum with Synth Sub, Perc Approach */
"0QR13KZ"  =  "1"  /* Replacement of Sacrum with Nonaut Sub, Perc Approach */
"0QR147Z"  =  "1"  /* Replacement of Sacrum with Autol Sub, Perc Endo Approach */
"0QR14JZ"  =  "1"  /* Replacement of Sacrum with Synth Sub, Perc Endo Approach */
"0QR14KZ"  =  "1"  /* Replacement of Sacrum with Nonaut Sub, Perc Endo Approach */
"0QR207Z"  =  "1"  /* Replacement of R Pelvic Bone with Autol Sub, Open Approach */
"0QR20JZ"  =  "1"  /* Replacement of R Pelvic Bone with Synth Sub, Open Approach */
"0QR20KZ"  =  "1"  /* Replacement of R Pelvic Bone with Nonaut Sub, Open Approach */
"0QR237Z"  =  "1"  /* Replacement of R Pelvic Bone with Autol Sub, Perc Approach */
"0QR23JZ"  =  "1"  /* Replacement of R Pelvic Bone with Synth Sub, Perc Approach */
"0QR23KZ"  =  "1"  /* Replacement of R Pelvic Bone with Nonaut Sub, Perc Approach */
"0QR247Z"  =  "1"  /* Replace of R Pelvic Bone with Autol Sub, Perc Endo Approach */
"0QR24JZ"  =  "1"  /* Replace of R Pelvic Bone with Synth Sub, Perc Endo Approach */
"0QR24KZ"  =  "1"  /* Replace of R Pelvic Bone with Nonaut Sub, Perc Endo Approach */
"0QR307Z"  =  "1"  /* Replacement of L Pelvic Bone with Autol Sub, Open Approach */
"0QR30JZ"  =  "1"  /* Replacement of L Pelvic Bone with Synth Sub, Open Approach */
"0QR30KZ"  =  "1"  /* Replacement of L Pelvic Bone with Nonaut Sub, Open Approach */
"0QR337Z"  =  "1"  /* Replacement of L Pelvic Bone with Autol Sub, Perc Approach */
"0QR33JZ"  =  "1"  /* Replacement of L Pelvic Bone with Synth Sub, Perc Approach */
"0QR33KZ"  =  "1"  /* Replacement of L Pelvic Bone with Nonaut Sub, Perc Approach */
"0QR347Z"  =  "1"  /* Replace of L Pelvic Bone with Autol Sub, Perc Endo Approach */
"0QR34JZ"  =  "1"  /* Replace of L Pelvic Bone with Synth Sub, Perc Endo Approach */
"0QR34KZ"  =  "1"  /* Replace of L Pelvic Bone with Nonaut Sub, Perc Endo Approach */
"0QR407Z"  =  "1"  /* Replacement of R Acetabulum with Autol Sub, Open Approach */
"0QR40JZ"  =  "1"  /* Replacement of R Acetabulum with Synth Sub, Open Approach */
"0QR40KZ"  =  "1"  /* Replacement of R Acetabulum with Nonaut Sub, Open Approach */
"0QR437Z"  =  "1"  /* Replacement of R Acetabulum with Autol Sub, Perc Approach */
"0QR43JZ"  =  "1"  /* Replacement of R Acetabulum with Synth Sub, Perc Approach */
"0QR43KZ"  =  "1"  /* Replacement of R Acetabulum with Nonaut Sub, Perc Approach */
"0QR447Z"  =  "1"  /* Replace of R Acetabulum with Autol Sub, Perc Endo Approach */
"0QR44JZ"  =  "1"  /* Replace of R Acetabulum with Synth Sub, Perc Endo Approach */
"0QR44KZ"  =  "1"  /* Replace of R Acetabulum with Nonaut Sub, Perc Endo Approach */
"0QR507Z"  =  "1"  /* Replacement of Left Acetabulum with Autol Sub, Open Approach */
"0QR50JZ"  =  "1"  /* Replacement of Left Acetabulum with Synth Sub, Open Approach */
"0QR50KZ"  =  "1"  /* Replacement of L Acetabulum with Nonaut Sub, Open Approach */
"0QR537Z"  =  "1"  /* Replacement of Left Acetabulum with Autol Sub, Perc Approach */
"0QR53JZ"  =  "1"  /* Replacement of Left Acetabulum with Synth Sub, Perc Approach */
"0QR53KZ"  =  "1"  /* Replacement of L Acetabulum with Nonaut Sub, Perc Approach */
"0QR547Z"  =  "1"  /* Replace of L Acetabulum with Autol Sub, Perc Endo Approach */
"0QR54JZ"  =  "1"  /* Replace of L Acetabulum with Synth Sub, Perc Endo Approach */
"0QR54KZ"  =  "1"  /* Replace of L Acetabulum with Nonaut Sub, Perc Endo Approach */
"0QR607Z"  =  "1"  /* Replacement of R Up Femur with Autol Sub, Open Approach */
"0QR60JZ"  =  "1"  /* Replacement of R Up Femur with Synth Sub, Open Approach */
"0QR60KZ"  =  "1"  /* Replacement of R Up Femur with Nonaut Sub, Open Approach */
"0QR637Z"  =  "1"  /* Replacement of R Up Femur with Autol Sub, Perc Approach */
"0QR63JZ"  =  "1"  /* Replacement of R Up Femur with Synth Sub, Perc Approach */
"0QR63KZ"  =  "1"  /* Replacement of R Up Femur with Nonaut Sub, Perc Approach */
"0QR647Z"  =  "1"  /* Replacement of R Up Femur with Autol Sub, Perc Endo Approach */
"0QR64JZ"  =  "1"  /* Replacement of R Up Femur with Synth Sub, Perc Endo Approach */
"0QR64KZ"  =  "1"  /* Replace of R Up Femur with Nonaut Sub, Perc Endo Approach */
"0QR707Z"  =  "1"  /* Replacement of L Up Femur with Autol Sub, Open Approach */
"0QR70JZ"  =  "1"  /* Replacement of L Up Femur with Synth Sub, Open Approach */
"0QR70KZ"  =  "1"  /* Replacement of L Up Femur with Nonaut Sub, Open Approach */
"0QR737Z"  =  "1"  /* Replacement of L Up Femur with Autol Sub, Perc Approach */
"0QR73JZ"  =  "1"  /* Replacement of L Up Femur with Synth Sub, Perc Approach */
"0QR73KZ"  =  "1"  /* Replacement of L Up Femur with Nonaut Sub, Perc Approach */
"0QR747Z"  =  "1"  /* Replacement of L Up Femur with Autol Sub, Perc Endo Approach */
"0QR74JZ"  =  "1"  /* Replacement of L Up Femur with Synth Sub, Perc Endo Approach */
"0QR74KZ"  =  "1"  /* Replace of L Up Femur with Nonaut Sub, Perc Endo Approach */
"0QR807Z"  =  "1"  /* Replacement of R Femur Shaft with Autol Sub, Open Approach */
"0QR80JZ"  =  "1"  /* Replacement of R Femur Shaft with Synth Sub, Open Approach */
"0QR80KZ"  =  "1"  /* Replacement of R Femur Shaft with Nonaut Sub, Open Approach */
"0QR837Z"  =  "1"  /* Replacement of R Femur Shaft with Autol Sub, Perc Approach */
"0QR83JZ"  =  "1"  /* Replacement of R Femur Shaft with Synth Sub, Perc Approach */
"0QR83KZ"  =  "1"  /* Replacement of R Femur Shaft with Nonaut Sub, Perc Approach */
"0QR847Z"  =  "1"  /* Replace of R Femur Shaft with Autol Sub, Perc Endo Approach */
"0QR84JZ"  =  "1"  /* Replace of R Femur Shaft with Synth Sub, Perc Endo Approach */
"0QR84KZ"  =  "1"  /* Replace of R Femur Shaft with Nonaut Sub, Perc Endo Approach */
"0QR907Z"  =  "1"  /* Replacement of L Femur Shaft with Autol Sub, Open Approach */
"0QR90JZ"  =  "1"  /* Replacement of L Femur Shaft with Synth Sub, Open Approach */
"0QR90KZ"  =  "1"  /* Replacement of L Femur Shaft with Nonaut Sub, Open Approach */
"0QR937Z"  =  "1"  /* Replacement of L Femur Shaft with Autol Sub, Perc Approach */
"0QR93JZ"  =  "1"  /* Replacement of L Femur Shaft with Synth Sub, Perc Approach */
"0QR93KZ"  =  "1"  /* Replacement of L Femur Shaft with Nonaut Sub, Perc Approach */
"0QR947Z"  =  "1"  /* Replace of L Femur Shaft with Autol Sub, Perc Endo Approach */
"0QR94JZ"  =  "1"  /* Replace of L Femur Shaft with Synth Sub, Perc Endo Approach */
"0QR94KZ"  =  "1"  /* Replace of L Femur Shaft with Nonaut Sub, Perc Endo Approach */
"0QRB07Z"  =  "1"  /* Replacement of R Low Femur with Autol Sub, Open Approach */
"0QRB0JZ"  =  "1"  /* Replacement of R Low Femur with Synth Sub, Open Approach */
"0QRB0KZ"  =  "1"  /* Replacement of R Low Femur with Nonaut Sub, Open Approach */
"0QRB37Z"  =  "1"  /* Replacement of R Low Femur with Autol Sub, Perc Approach */
"0QRB3JZ"  =  "1"  /* Replacement of R Low Femur with Synth Sub, Perc Approach */
"0QRB3KZ"  =  "1"  /* Replacement of R Low Femur with Nonaut Sub, Perc Approach */
"0QRB47Z"  =  "1"  /* Replace of R Low Femur with Autol Sub, Perc Endo Approach */
"0QRB4JZ"  =  "1"  /* Replace of R Low Femur with Synth Sub, Perc Endo Approach */
"0QRB4KZ"  =  "1"  /* Replace of R Low Femur with Nonaut Sub, Perc Endo Approach */
"0QRC07Z"  =  "1"  /* Replacement of L Low Femur with Autol Sub, Open Approach */
"0QRC0JZ"  =  "1"  /* Replacement of L Low Femur with Synth Sub, Open Approach */
"0QRC0KZ"  =  "1"  /* Replacement of L Low Femur with Nonaut Sub, Open Approach */
"0QRC37Z"  =  "1"  /* Replacement of L Low Femur with Autol Sub, Perc Approach */
"0QRC3JZ"  =  "1"  /* Replacement of L Low Femur with Synth Sub, Perc Approach */
"0QRC3KZ"  =  "1"  /* Replacement of L Low Femur with Nonaut Sub, Perc Approach */
"0QRC47Z"  =  "1"  /* Replace of L Low Femur with Autol Sub, Perc Endo Approach */
"0QRC4JZ"  =  "1"  /* Replace of L Low Femur with Synth Sub, Perc Endo Approach */
"0QRC4KZ"  =  "1"  /* Replace of L Low Femur with Nonaut Sub, Perc Endo Approach */
"0QRD07Z"  =  "1"  /* Replacement of Right Patella with Autol Sub, Open Approach */
"0QRD0JZ"  =  "1"  /* Replacement of Right Patella with Synth Sub, Open Approach */
"0QRD0KZ"  =  "1"  /* Replacement of Right Patella with Nonaut Sub, Open Approach */
"0QRD37Z"  =  "1"  /* Replacement of Right Patella with Autol Sub, Perc Approach */
"0QRD3JZ"  =  "1"  /* Replacement of Right Patella with Synth Sub, Perc Approach */
"0QRD3KZ"  =  "1"  /* Replacement of Right Patella with Nonaut Sub, Perc Approach */
"0QRD47Z"  =  "1"  /* Replacement of R Patella with Autol Sub, Perc Endo Approach */
"0QRD4JZ"  =  "1"  /* Replacement of R Patella with Synth Sub, Perc Endo Approach */
"0QRD4KZ"  =  "1"  /* Replacement of R Patella with Nonaut Sub, Perc Endo Approach */
"0QRF07Z"  =  "1"  /* Replacement of Left Patella with Autol Sub, Open Approach */
"0QRF0JZ"  =  "1"  /* Replacement of Left Patella with Synth Sub, Open Approach */
"0QRF0KZ"  =  "1"  /* Replacement of Left Patella with Nonaut Sub, Open Approach */
"0QRF37Z"  =  "1"  /* Replacement of Left Patella with Autol Sub, Perc Approach */
"0QRF3JZ"  =  "1"  /* Replacement of Left Patella with Synth Sub, Perc Approach */
"0QRF3KZ"  =  "1"  /* Replacement of Left Patella with Nonaut Sub, Perc Approach */
"0QRF47Z"  =  "1"  /* Replacement of L Patella with Autol Sub, Perc Endo Approach */
"0QRF4JZ"  =  "1"  /* Replacement of L Patella with Synth Sub, Perc Endo Approach */
"0QRF4KZ"  =  "1"  /* Replacement of L Patella with Nonaut Sub, Perc Endo Approach */
"0QRG07Z"  =  "1"  /* Replacement of Right Tibia with Autol Sub, Open Approach */
"0QRG0JZ"  =  "1"  /* Replacement of Right Tibia with Synth Sub, Open Approach */
"0QRG0KZ"  =  "1"  /* Replacement of Right Tibia with Nonaut Sub, Open Approach */
"0QRG37Z"  =  "1"  /* Replacement of Right Tibia with Autol Sub, Perc Approach */
"0QRG3JZ"  =  "1"  /* Replacement of Right Tibia with Synth Sub, Perc Approach */
"0QRG3KZ"  =  "1"  /* Replacement of Right Tibia with Nonaut Sub, Perc Approach */
"0QRG47Z"  =  "1"  /* Replacement of R Tibia with Autol Sub, Perc Endo Approach */
"0QRG4JZ"  =  "1"  /* Replacement of R Tibia with Synth Sub, Perc Endo Approach */
"0QRG4KZ"  =  "1"  /* Replacement of R Tibia with Nonaut Sub, Perc Endo Approach */
"0QRH07Z"  =  "1"  /* Replacement of Left Tibia with Autol Sub, Open Approach */
"0QRH0JZ"  =  "1"  /* Replacement of Left Tibia with Synth Sub, Open Approach */
"0QRH0KZ"  =  "1"  /* Replacement of Left Tibia with Nonaut Sub, Open Approach */
"0QRH37Z"  =  "1"  /* Replacement of Left Tibia with Autol Sub, Perc Approach */
"0QRH3JZ"  =  "1"  /* Replacement of Left Tibia with Synth Sub, Perc Approach */
"0QRH3KZ"  =  "1"  /* Replacement of Left Tibia with Nonaut Sub, Perc Approach */
"0QRH47Z"  =  "1"  /* Replacement of Left Tibia with Autol Sub, Perc Endo Approach */
"0QRH4JZ"  =  "1"  /* Replacement of Left Tibia with Synth Sub, Perc Endo Approach */
"0QRH4KZ"  =  "1"  /* Replacement of L Tibia with Nonaut Sub, Perc Endo Approach */
"0QRJ07Z"  =  "1"  /* Replacement of Right Fibula with Autol Sub, Open Approach */
"0QRJ0JZ"  =  "1"  /* Replacement of Right Fibula with Synth Sub, Open Approach */
"0QRJ0KZ"  =  "1"  /* Replacement of Right Fibula with Nonaut Sub, Open Approach */
"0QRJ37Z"  =  "1"  /* Replacement of Right Fibula with Autol Sub, Perc Approach */
"0QRJ3JZ"  =  "1"  /* Replacement of Right Fibula with Synth Sub, Perc Approach */
"0QRJ3KZ"  =  "1"  /* Replacement of Right Fibula with Nonaut Sub, Perc Approach */
"0QRJ47Z"  =  "1"  /* Replacement of R Fibula with Autol Sub, Perc Endo Approach */
"0QRJ4JZ"  =  "1"  /* Replacement of R Fibula with Synth Sub, Perc Endo Approach */
"0QRJ4KZ"  =  "1"  /* Replacement of R Fibula with Nonaut Sub, Perc Endo Approach */
"0QRK07Z"  =  "1"  /* Replacement of Left Fibula with Autol Sub, Open Approach */
"0QRK0JZ"  =  "1"  /* Replacement of Left Fibula with Synth Sub, Open Approach */
"0QRK0KZ"  =  "1"  /* Replacement of Left Fibula with Nonaut Sub, Open Approach */
"0QRK37Z"  =  "1"  /* Replacement of Left Fibula with Autol Sub, Perc Approach */
"0QRK3JZ"  =  "1"  /* Replacement of Left Fibula with Synth Sub, Perc Approach */
"0QRK3KZ"  =  "1"  /* Replacement of Left Fibula with Nonaut Sub, Perc Approach */
"0QRK47Z"  =  "1"  /* Replacement of L Fibula with Autol Sub, Perc Endo Approach */
"0QRK4JZ"  =  "1"  /* Replacement of L Fibula with Synth Sub, Perc Endo Approach */
"0QRK4KZ"  =  "1"  /* Replacement of L Fibula with Nonaut Sub, Perc Endo Approach */
"0QRL07Z"  =  "1"  /* Replacement of Right Tarsal with Autol Sub, Open Approach */
"0QRL0JZ"  =  "1"  /* Replacement of Right Tarsal with Synth Sub, Open Approach */
"0QRL0KZ"  =  "1"  /* Replacement of Right Tarsal with Nonaut Sub, Open Approach */
"0QRL37Z"  =  "1"  /* Replacement of Right Tarsal with Autol Sub, Perc Approach */
"0QRL3JZ"  =  "1"  /* Replacement of Right Tarsal with Synth Sub, Perc Approach */
"0QRL3KZ"  =  "1"  /* Replacement of Right Tarsal with Nonaut Sub, Perc Approach */
"0QRL47Z"  =  "1"  /* Replacement of R Tarsal with Autol Sub, Perc Endo Approach */
"0QRL4JZ"  =  "1"  /* Replacement of R Tarsal with Synth Sub, Perc Endo Approach */
"0QRL4KZ"  =  "1"  /* Replacement of R Tarsal with Nonaut Sub, Perc Endo Approach */
"0QRM07Z"  =  "1"  /* Replacement of Left Tarsal with Autol Sub, Open Approach */
"0QRM0JZ"  =  "1"  /* Replacement of Left Tarsal with Synth Sub, Open Approach */
"0QRM0KZ"  =  "1"  /* Replacement of Left Tarsal with Nonaut Sub, Open Approach */
"0QRM37Z"  =  "1"  /* Replacement of Left Tarsal with Autol Sub, Perc Approach */
"0QRM3JZ"  =  "1"  /* Replacement of Left Tarsal with Synth Sub, Perc Approach */
"0QRM3KZ"  =  "1"  /* Replacement of Left Tarsal with Nonaut Sub, Perc Approach */
"0QRM47Z"  =  "1"  /* Replacement of L Tarsal with Autol Sub, Perc Endo Approach */
"0QRM4JZ"  =  "1"  /* Replacement of L Tarsal with Synth Sub, Perc Endo Approach */
"0QRM4KZ"  =  "1"  /* Replacement of L Tarsal with Nonaut Sub, Perc Endo Approach */
"0QRN07Z"  =  "1"  /* Replacement of R Metatarsal with Autol Sub, Open Approach */
"0QRN0JZ"  =  "1"  /* Replacement of R Metatarsal with Synth Sub, Open Approach */
"0QRN0KZ"  =  "1"  /* Replacement of R Metatarsal with Nonaut Sub, Open Approach */
"0QRN37Z"  =  "1"  /* Replacement of R Metatarsal with Autol Sub, Perc Approach */
"0QRN3JZ"  =  "1"  /* Replacement of R Metatarsal with Synth Sub, Perc Approach */
"0QRN3KZ"  =  "1"  /* Replacement of R Metatarsal with Nonaut Sub, Perc Approach */
"0QRN47Z"  =  "1"  /* Replace of R Metatarsal with Autol Sub, Perc Endo Approach */
"0QRN4JZ"  =  "1"  /* Replace of R Metatarsal with Synth Sub, Perc Endo Approach */
"0QRN4KZ"  =  "1"  /* Replace of R Metatarsal with Nonaut Sub, Perc Endo Approach */
"0QRP07Z"  =  "1"  /* Replacement of Left Metatarsal with Autol Sub, Open Approach */
"0QRP0JZ"  =  "1"  /* Replacement of Left Metatarsal with Synth Sub, Open Approach */
"0QRP0KZ"  =  "1"  /* Replacement of L Metatarsal with Nonaut Sub, Open Approach */
"0QRP37Z"  =  "1"  /* Replacement of Left Metatarsal with Autol Sub, Perc Approach */
"0QRP3JZ"  =  "1"  /* Replacement of Left Metatarsal with Synth Sub, Perc Approach */
"0QRP3KZ"  =  "1"  /* Replacement of L Metatarsal with Nonaut Sub, Perc Approach */
"0QRP47Z"  =  "1"  /* Replace of L Metatarsal with Autol Sub, Perc Endo Approach */
"0QRP4JZ"  =  "1"  /* Replace of L Metatarsal with Synth Sub, Perc Endo Approach */
"0QRP4KZ"  =  "1"  /* Replace of L Metatarsal with Nonaut Sub, Perc Endo Approach */
"0QRQ07Z"  =  "1"  /* Replacement of R Toe Phalanx with Autol Sub, Open Approach */
"0QRQ0JZ"  =  "1"  /* Replacement of R Toe Phalanx with Synth Sub, Open Approach */
"0QRQ0KZ"  =  "1"  /* Replacement of R Toe Phalanx with Nonaut Sub, Open Approach */
"0QRQ37Z"  =  "1"  /* Replacement of R Toe Phalanx with Autol Sub, Perc Approach */
"0QRQ3JZ"  =  "1"  /* Replacement of R Toe Phalanx with Synth Sub, Perc Approach */
"0QRQ3KZ"  =  "1"  /* Replacement of R Toe Phalanx with Nonaut Sub, Perc Approach */
"0QRQ47Z"  =  "1"  /* Replace of R Toe Phalanx with Autol Sub, Perc Endo Approach */
"0QRQ4JZ"  =  "1"  /* Replace of R Toe Phalanx with Synth Sub, Perc Endo Approach */
"0QRQ4KZ"  =  "1"  /* Replace of R Toe Phalanx with Nonaut Sub, Perc Endo Approach */
"0QRR07Z"  =  "1"  /* Replacement of L Toe Phalanx with Autol Sub, Open Approach */
"0QRR0JZ"  =  "1"  /* Replacement of L Toe Phalanx with Synth Sub, Open Approach */
"0QRR0KZ"  =  "1"  /* Replacement of L Toe Phalanx with Nonaut Sub, Open Approach */
"0QRR37Z"  =  "1"  /* Replacement of L Toe Phalanx with Autol Sub, Perc Approach */
"0QRR3JZ"  =  "1"  /* Replacement of L Toe Phalanx with Synth Sub, Perc Approach */
"0QRR3KZ"  =  "1"  /* Replacement of L Toe Phalanx with Nonaut Sub, Perc Approach */
"0QRR47Z"  =  "1"  /* Replace of L Toe Phalanx with Autol Sub, Perc Endo Approach */
"0QRR4JZ"  =  "1"  /* Replace of L Toe Phalanx with Synth Sub, Perc Endo Approach */
"0QRR4KZ"  =  "1"  /* Replace of L Toe Phalanx with Nonaut Sub, Perc Endo Approach */
"0QRS07Z"  =  "1"  /* Replacement of Coccyx with Autol Sub, Open Approach */
"0QRS0JZ"  =  "1"  /* Replacement of Coccyx with Synth Sub, Open Approach */
"0QRS0KZ"  =  "1"  /* Replacement of Coccyx with Nonaut Sub, Open Approach */
"0QRS37Z"  =  "1"  /* Replacement of Coccyx with Autol Sub, Perc Approach */
"0QRS3JZ"  =  "1"  /* Replacement of Coccyx with Synth Sub, Perc Approach */
"0QRS3KZ"  =  "1"  /* Replacement of Coccyx with Nonaut Sub, Perc Approach */
"0QRS47Z"  =  "1"  /* Replacement of Coccyx with Autol Sub, Perc Endo Approach */
"0QRS4JZ"  =  "1"  /* Replacement of Coccyx with Synth Sub, Perc Endo Approach */
"0QRS4KZ"  =  "1"  /* Replacement of Coccyx with Nonaut Sub, Perc Endo Approach */
"0QS004Z"  =  "1"  /* Reposition Lumbar Vertebra with Int Fix, Open Approach */
"0QS00ZZ"  =  "1"  /* Reposition Lumbar Vertebra, Open Approach */
"0QS034Z"  =  "1"  /* Reposition Lumbar Vertebra with Int Fix, Perc Approach */
"0QS03ZZ"  =  "1"  /* Reposition Lumbar Vertebra, Percutaneous Approach */
"0QS044Z"  =  "1"  /* Reposition Lumbar Vertebra with Int Fix, Perc Endo Approach */
"0QS04ZZ"  =  "1"  /* Reposition Lumbar Vertebra, Percutaneous Endoscopic Approach */
"0QS0XZZ"  =  "1"  /* Reposition Lumbar Vertebra, External Approach */
"0QS104Z"  =  "1"  /* Reposition Sacrum with Int Fix, Open Approach */
"0QS10ZZ"  =  "1"  /* Reposition Sacrum, Open Approach */
"0QS134Z"  =  "1"  /* Reposition Sacrum with Int Fix, Perc Approach */
"0QS13ZZ"  =  "1"  /* Reposition Sacrum, Percutaneous Approach */
"0QS144Z"  =  "1"  /* Reposition Sacrum with Int Fix, Perc Endo Approach */
"0QS14ZZ"  =  "1"  /* Reposition Sacrum, Percutaneous Endoscopic Approach */
"0QS1XZZ"  =  "1"  /* Reposition Sacrum, External Approach */
"0QS204Z"  =  "1"  /* Reposition Right Pelvic Bone with Int Fix, Open Approach */
"0QS205Z"  =  "1"  /* Reposition Right Pelvic Bone with Ext Fix, Open Approach */
"0QS20ZZ"  =  "1"  /* Reposition Right Pelvic Bone, Open Approach */
"0QS234Z"  =  "1"  /* Reposition Right Pelvic Bone with Int Fix, Perc Approach */
"0QS235Z"  =  "1"  /* Reposition Right Pelvic Bone with Ext Fix, Perc Approach */
"0QS244Z"  =  "1"  /* Reposition R Pelvic Bone with Int Fix, Perc Endo Approach */
"0QS245Z"  =  "1"  /* Reposition R Pelvic Bone with Ext Fix, Perc Endo Approach */
"0QS304Z"  =  "1"  /* Reposition Left Pelvic Bone with Int Fix, Open Approach */
"0QS305Z"  =  "1"  /* Reposition Left Pelvic Bone with Ext Fix, Open Approach */
"0QS30ZZ"  =  "1"  /* Reposition Left Pelvic Bone, Open Approach */
"0QS334Z"  =  "1"  /* Reposition Left Pelvic Bone with Int Fix, Perc Approach */
"0QS335Z"  =  "1"  /* Reposition Left Pelvic Bone with Ext Fix, Perc Approach */
"0QS344Z"  =  "1"  /* Reposition Left Pelvic Bone with Int Fix, Perc Endo Approach */
"0QS345Z"  =  "1"  /* Reposition Left Pelvic Bone with Ext Fix, Perc Endo Approach */
"0QS404Z"  =  "1"  /* Reposition Right Acetabulum with Int Fix, Open Approach */
"0QS40ZZ"  =  "1"  /* Reposition Right Acetabulum, Open Approach */
"0QS434Z"  =  "1"  /* Reposition Right Acetabulum with Int Fix, Perc Approach */
"0QS444Z"  =  "1"  /* Reposition Right Acetabulum with Int Fix, Perc Endo Approach */
"0QS504Z"  =  "1"  /* Reposition Left Acetabulum with Int Fix, Open Approach */
"0QS50ZZ"  =  "1"  /* Reposition Left Acetabulum, Open Approach */
"0QS534Z"  =  "1"  /* Reposition Left Acetabulum with Int Fix, Perc Approach */
"0QS544Z"  =  "1"  /* Reposition Left Acetabulum with Int Fix, Perc Endo Approach */
"0QS604Z"  =  "1"  /* Reposition Right Upper Femur with Int Fix, Open Approach */
"0QS605Z"  =  "1"  /* Reposition Right Upper Femur with Ext Fix, Open Approach */
"0QS606Z"  =  "1"  /* Reposition R Up Femur with Intramed Fix, Open Approach */
"0QS60BZ"  =  "1"  /* Reposition R Up Femur with Monopln Ext Fix, Open Approach */
"0QS60CZ"  =  "1"  /* Reposition R Up Femur with Ring Ext Fix, Open Approach */
"0QS60DZ"  =  "1"  /* Reposition R Up Femur with Hybrid Ext Fix, Open Approach */
"0QS60ZZ"  =  "1"  /* Reposition Right Upper Femur, Open Approach */
"0QS634Z"  =  "1"  /* Reposition Right Upper Femur with Int Fix, Perc Approach */
"0QS635Z"  =  "1"  /* Reposition Right Upper Femur with Ext Fix, Perc Approach */
"0QS636Z"  =  "1"  /* Reposition R Up Femur with Intramed Fix, Perc Approach */
"0QS63BZ"  =  "1"  /* Reposition R Up Femur with Monopln Ext Fix, Perc Approach */
"0QS63CZ"  =  "1"  /* Reposition R Up Femur with Ring Ext Fix, Perc Approach */
"0QS63DZ"  =  "1"  /* Reposition R Up Femur with Hybrid Ext Fix, Perc Approach */
"0QS644Z"  =  "1"  /* Reposition R Up Femur with Int Fix, Perc Endo Approach */
"0QS645Z"  =  "1"  /* Reposition R Up Femur with Ext Fix, Perc Endo Approach */
"0QS646Z"  =  "1"  /* Reposition R Up Femur with Intramed Fix, Perc Endo Approach */
"0QS64BZ"  =  "1"  /* Reposition R Up Femur w Monopln Ext Fix, Perc Endo */
"0QS64CZ"  =  "1"  /* Reposition R Up Femur with Ring Ext Fix, Perc Endo Approach */
"0QS64DZ"  =  "1"  /* Reposition R Up Femur w Hybrid Ext Fix, Perc Endo */
"0QS704Z"  =  "1"  /* Reposition Left Upper Femur with Int Fix, Open Approach */
"0QS705Z"  =  "1"  /* Reposition Left Upper Femur with Ext Fix, Open Approach */
"0QS706Z"  =  "1"  /* Reposition Left Upper Femur with Intramed Fix, Open Approach */
"0QS70BZ"  =  "1"  /* Reposition L Up Femur with Monopln Ext Fix, Open Approach */
"0QS70CZ"  =  "1"  /* Reposition Left Upper Femur with Ring Ext Fix, Open Approach */
"0QS70DZ"  =  "1"  /* Reposition L Up Femur with Hybrid Ext Fix, Open Approach */
"0QS70ZZ"  =  "1"  /* Reposition Left Upper Femur, Open Approach */
"0QS734Z"  =  "1"  /* Reposition Left Upper Femur with Int Fix, Perc Approach */
"0QS735Z"  =  "1"  /* Reposition Left Upper Femur with Ext Fix, Perc Approach */
"0QS736Z"  =  "1"  /* Reposition Left Upper Femur with Intramed Fix, Perc Approach */
"0QS73BZ"  =  "1"  /* Reposition L Up Femur with Monopln Ext Fix, Perc Approach */
"0QS73CZ"  =  "1"  /* Reposition Left Upper Femur with Ring Ext Fix, Perc Approach */
"0QS73DZ"  =  "1"  /* Reposition L Up Femur with Hybrid Ext Fix, Perc Approach */
"0QS744Z"  =  "1"  /* Reposition Left Upper Femur with Int Fix, Perc Endo Approach */
"0QS745Z"  =  "1"  /* Reposition Left Upper Femur with Ext Fix, Perc Endo Approach */
"0QS746Z"  =  "1"  /* Reposition L Up Femur with Intramed Fix, Perc Endo Approach */
"0QS74BZ"  =  "1"  /* Reposition L Up Femur w Monopln Ext Fix, Perc Endo */
"0QS74CZ"  =  "1"  /* Reposition L Up Femur with Ring Ext Fix, Perc Endo Approach */
"0QS74DZ"  =  "1"  /* Reposition L Up Femur w Hybrid Ext Fix, Perc Endo */
"0QS804Z"  =  "1"  /* Reposition Right Femoral Shaft with Int Fix, Open Approach */
"0QS805Z"  =  "1"  /* Reposition Right Femoral Shaft with Ext Fix, Open Approach */
"0QS806Z"  =  "1"  /* Reposition R Femur Shaft with Intramed Fix, Open Approach */
"0QS80BZ"  =  "1"  /* Reposition R Femur Shaft with Monopln Ext Fix, Open Approach */
"0QS80CZ"  =  "1"  /* Reposition R Femur Shaft with Ring Ext Fix, Open Approach */
"0QS80DZ"  =  "1"  /* Reposition R Femur Shaft with Hybrid Ext Fix, Open Approach */
"0QS80ZZ"  =  "1"  /* Reposition Right Femoral Shaft, Open Approach */
"0QS834Z"  =  "1"  /* Reposition Right Femoral Shaft with Int Fix, Perc Approach */
"0QS835Z"  =  "1"  /* Reposition Right Femoral Shaft with Ext Fix, Perc Approach */
"0QS836Z"  =  "1"  /* Reposition R Femur Shaft with Intramed Fix, Perc Approach */
"0QS83BZ"  =  "1"  /* Reposition R Femur Shaft with Monopln Ext Fix, Perc Approach */
"0QS83CZ"  =  "1"  /* Reposition R Femur Shaft with Ring Ext Fix, Perc Approach */
"0QS83DZ"  =  "1"  /* Reposition R Femur Shaft with Hybrid Ext Fix, Perc Approach */
"0QS844Z"  =  "1"  /* Reposition R Femur Shaft with Int Fix, Perc Endo Approach */
"0QS845Z"  =  "1"  /* Reposition R Femur Shaft with Ext Fix, Perc Endo Approach */
"0QS846Z"  =  "1"  /* Reposition R Femur Shaft w Intramed Fix, Perc Endo */
"0QS84BZ"  =  "1"  /* Reposition R Femur Shaft w Monopln Ext Fix, Perc Endo */
"0QS84CZ"  =  "1"  /* Reposition R Femur Shaft w Ring Ext Fix, Perc Endo */
"0QS84DZ"  =  "1"  /* Reposition R Femur Shaft w Hybrid Ext Fix, Perc Endo */
"0QS904Z"  =  "1"  /* Reposition Left Femoral Shaft with Int Fix, Open Approach */
"0QS905Z"  =  "1"  /* Reposition Left Femoral Shaft with Ext Fix, Open Approach */
"0QS906Z"  =  "1"  /* Reposition L Femur Shaft with Intramed Fix, Open Approach */
"0QS90BZ"  =  "1"  /* Reposition L Femur Shaft with Monopln Ext Fix, Open Approach */
"0QS90CZ"  =  "1"  /* Reposition L Femur Shaft with Ring Ext Fix, Open Approach */
"0QS90DZ"  =  "1"  /* Reposition L Femur Shaft with Hybrid Ext Fix, Open Approach */
"0QS90ZZ"  =  "1"  /* Reposition Left Femoral Shaft, Open Approach */
"0QS934Z"  =  "1"  /* Reposition Left Femoral Shaft with Int Fix, Perc Approach */
"0QS935Z"  =  "1"  /* Reposition Left Femoral Shaft with Ext Fix, Perc Approach */
"0QS936Z"  =  "1"  /* Reposition L Femur Shaft with Intramed Fix, Perc Approach */
"0QS93BZ"  =  "1"  /* Reposition L Femur Shaft with Monopln Ext Fix, Perc Approach */
"0QS93CZ"  =  "1"  /* Reposition L Femur Shaft with Ring Ext Fix, Perc Approach */
"0QS93DZ"  =  "1"  /* Reposition L Femur Shaft with Hybrid Ext Fix, Perc Approach */
"0QS944Z"  =  "1"  /* Reposition L Femur Shaft with Int Fix, Perc Endo Approach */
"0QS945Z"  =  "1"  /* Reposition L Femur Shaft with Ext Fix, Perc Endo Approach */
"0QS946Z"  =  "1"  /* Reposition L Femur Shaft w Intramed Fix, Perc Endo */
"0QS94BZ"  =  "1"  /* Reposition L Femur Shaft w Monopln Ext Fix, Perc Endo */
"0QS94CZ"  =  "1"  /* Reposition L Femur Shaft w Ring Ext Fix, Perc Endo */
"0QS94DZ"  =  "1"  /* Reposition L Femur Shaft w Hybrid Ext Fix, Perc Endo */
"0QSB04Z"  =  "1"  /* Reposition Right Lower Femur with Int Fix, Open Approach */
"0QSB05Z"  =  "1"  /* Reposition Right Lower Femur with Ext Fix, Open Approach */
"0QSB06Z"  =  "1"  /* Reposition R Low Femur with Intramed Fix, Open Approach */
"0QSB0BZ"  =  "1"  /* Reposition R Low Femur with Monopln Ext Fix, Open Approach */
"0QSB0CZ"  =  "1"  /* Reposition R Low Femur with Ring Ext Fix, Open Approach */
"0QSB0DZ"  =  "1"  /* Reposition R Low Femur with Hybrid Ext Fix, Open Approach */
"0QSB0ZZ"  =  "1"  /* Reposition Right Lower Femur, Open Approach */
"0QSB34Z"  =  "1"  /* Reposition Right Lower Femur with Int Fix, Perc Approach */
"0QSB35Z"  =  "1"  /* Reposition Right Lower Femur with Ext Fix, Perc Approach */
"0QSB36Z"  =  "1"  /* Reposition R Low Femur with Intramed Fix, Perc Approach */
"0QSB3BZ"  =  "1"  /* Reposition R Low Femur with Monopln Ext Fix, Perc Approach */
"0QSB3CZ"  =  "1"  /* Reposition R Low Femur with Ring Ext Fix, Perc Approach */
"0QSB3DZ"  =  "1"  /* Reposition R Low Femur with Hybrid Ext Fix, Perc Approach */
"0QSB44Z"  =  "1"  /* Reposition R Low Femur with Int Fix, Perc Endo Approach */
"0QSB45Z"  =  "1"  /* Reposition R Low Femur with Ext Fix, Perc Endo Approach */
"0QSB46Z"  =  "1"  /* Reposition R Low Femur with Intramed Fix, Perc Endo Approach */
"0QSB4BZ"  =  "1"  /* Reposition R Low Femur w Monopln Ext Fix, Perc Endo */
"0QSB4CZ"  =  "1"  /* Reposition R Low Femur with Ring Ext Fix, Perc Endo Approach */
"0QSB4DZ"  =  "1"  /* Reposition R Low Femur w Hybrid Ext Fix, Perc Endo */
"0QSC04Z"  =  "1"  /* Reposition Left Lower Femur with Int Fix, Open Approach */
"0QSC05Z"  =  "1"  /* Reposition Left Lower Femur with Ext Fix, Open Approach */
"0QSC06Z"  =  "1"  /* Reposition Left Lower Femur with Intramed Fix, Open Approach */
"0QSC0BZ"  =  "1"  /* Reposition L Low Femur with Monopln Ext Fix, Open Approach */
"0QSC0CZ"  =  "1"  /* Reposition Left Lower Femur with Ring Ext Fix, Open Approach */
"0QSC0DZ"  =  "1"  /* Reposition L Low Femur with Hybrid Ext Fix, Open Approach */
"0QSC0ZZ"  =  "1"  /* Reposition Left Lower Femur, Open Approach */
"0QSC34Z"  =  "1"  /* Reposition Left Lower Femur with Int Fix, Perc Approach */
"0QSC35Z"  =  "1"  /* Reposition Left Lower Femur with Ext Fix, Perc Approach */
"0QSC36Z"  =  "1"  /* Reposition Left Lower Femur with Intramed Fix, Perc Approach */
"0QSC3BZ"  =  "1"  /* Reposition L Low Femur with Monopln Ext Fix, Perc Approach */
"0QSC3CZ"  =  "1"  /* Reposition Left Lower Femur with Ring Ext Fix, Perc Approach */
"0QSC3DZ"  =  "1"  /* Reposition L Low Femur with Hybrid Ext Fix, Perc Approach */
"0QSC44Z"  =  "1"  /* Reposition Left Lower Femur with Int Fix, Perc Endo Approach */
"0QSC45Z"  =  "1"  /* Reposition Left Lower Femur with Ext Fix, Perc Endo Approach */
"0QSC46Z"  =  "1"  /* Reposition L Low Femur with Intramed Fix, Perc Endo Approach */
"0QSC4BZ"  =  "1"  /* Reposition L Low Femur w Monopln Ext Fix, Perc Endo */
"0QSC4CZ"  =  "1"  /* Reposition L Low Femur with Ring Ext Fix, Perc Endo Approach */
"0QSC4DZ"  =  "1"  /* Reposition L Low Femur w Hybrid Ext Fix, Perc Endo */
"0QSD04Z"  =  "1"  /* Reposition Right Patella with Int Fix, Open Approach */
"0QSD05Z"  =  "1"  /* Reposition Right Patella with Ext Fix, Open Approach */
"0QSD0ZZ"  =  "1"  /* Reposition Right Patella, Open Approach */
"0QSD34Z"  =  "1"  /* Reposition Right Patella with Int Fix, Perc Approach */
"0QSD35Z"  =  "1"  /* Reposition Right Patella with Ext Fix, Perc Approach */
"0QSD44Z"  =  "1"  /* Reposition Right Patella with Int Fix, Perc Endo Approach */
"0QSD45Z"  =  "1"  /* Reposition Right Patella with Ext Fix, Perc Endo Approach */
"0QSF04Z"  =  "1"  /* Reposition Left Patella with Int Fix, Open Approach */
"0QSF05Z"  =  "1"  /* Reposition Left Patella with Ext Fix, Open Approach */
"0QSF0ZZ"  =  "1"  /* Reposition Left Patella, Open Approach */
"0QSF34Z"  =  "1"  /* Reposition Left Patella with Int Fix, Perc Approach */
"0QSF35Z"  =  "1"  /* Reposition Left Patella with Ext Fix, Perc Approach */
"0QSF44Z"  =  "1"  /* Reposition Left Patella with Int Fix, Perc Endo Approach */
"0QSF45Z"  =  "1"  /* Reposition Left Patella with Ext Fix, Perc Endo Approach */
"0QSG04Z"  =  "1"  /* Reposition Right Tibia with Int Fix, Open Approach */
"0QSG05Z"  =  "1"  /* Reposition Right Tibia with Ext Fix, Open Approach */
"0QSG06Z"  =  "1"  /* Reposition Right Tibia with Intramed Fix, Open Approach */
"0QSG0BZ"  =  "1"  /* Reposition Right Tibia with Monopln Ext Fix, Open Approach */
"0QSG0CZ"  =  "1"  /* Reposition Right Tibia with Ring Ext Fix, Open Approach */
"0QSG0DZ"  =  "1"  /* Reposition Right Tibia with Hybrid Ext Fix, Open Approach */
"0QSG0ZZ"  =  "1"  /* Reposition Right Tibia, Open Approach */
"0QSG34Z"  =  "1"  /* Reposition Right Tibia with Int Fix, Perc Approach */
"0QSG35Z"  =  "1"  /* Reposition Right Tibia with Ext Fix, Perc Approach */
"0QSG36Z"  =  "1"  /* Reposition Right Tibia with Intramed Fix, Perc Approach */
"0QSG3BZ"  =  "1"  /* Reposition Right Tibia with Monopln Ext Fix, Perc Approach */
"0QSG3CZ"  =  "1"  /* Reposition Right Tibia with Ring Ext Fix, Perc Approach */
"0QSG3DZ"  =  "1"  /* Reposition Right Tibia with Hybrid Ext Fix, Perc Approach */
"0QSG44Z"  =  "1"  /* Reposition Right Tibia with Int Fix, Perc Endo Approach */
"0QSG45Z"  =  "1"  /* Reposition Right Tibia with Ext Fix, Perc Endo Approach */
"0QSG46Z"  =  "1"  /* Reposition Right Tibia with Intramed Fix, Perc Endo Approach */
"0QSG4BZ"  =  "1"  /* Reposition R Tibia with Monopln Ext Fix, Perc Endo Approach */
"0QSG4CZ"  =  "1"  /* Reposition Right Tibia with Ring Ext Fix, Perc Endo Approach */
"0QSG4DZ"  =  "1"  /* Reposition R Tibia with Hybrid Ext Fix, Perc Endo Approach */
"0QSH04Z"  =  "1"  /* Reposition Left Tibia with Int Fix, Open Approach */
"0QSH05Z"  =  "1"  /* Reposition Left Tibia with Ext Fix, Open Approach */
"0QSH06Z"  =  "1"  /* Reposition Left Tibia with Intramed Fix, Open Approach */
"0QSH0BZ"  =  "1"  /* Reposition Left Tibia with Monopln Ext Fix, Open Approach */
"0QSH0CZ"  =  "1"  /* Reposition Left Tibia with Ring Ext Fix, Open Approach */
"0QSH0DZ"  =  "1"  /* Reposition Left Tibia with Hybrid Ext Fix, Open Approach */
"0QSH0ZZ"  =  "1"  /* Reposition Left Tibia, Open Approach */
"0QSH34Z"  =  "1"  /* Reposition Left Tibia with Int Fix, Perc Approach */
"0QSH35Z"  =  "1"  /* Reposition Left Tibia with Ext Fix, Perc Approach */
"0QSH36Z"  =  "1"  /* Reposition Left Tibia with Intramed Fix, Perc Approach */
"0QSH3BZ"  =  "1"  /* Reposition Left Tibia with Monopln Ext Fix, Perc Approach */
"0QSH3CZ"  =  "1"  /* Reposition Left Tibia with Ring Ext Fix, Perc Approach */
"0QSH3DZ"  =  "1"  /* Reposition Left Tibia with Hybrid Ext Fix, Perc Approach */
"0QSH44Z"  =  "1"  /* Reposition Left Tibia with Int Fix, Perc Endo Approach */
"0QSH45Z"  =  "1"  /* Reposition Left Tibia with Ext Fix, Perc Endo Approach */
"0QSH46Z"  =  "1"  /* Reposition Left Tibia with Intramed Fix, Perc Endo Approach */
"0QSH4BZ"  =  "1"  /* Reposition L Tibia with Monopln Ext Fix, Perc Endo Approach */
"0QSH4CZ"  =  "1"  /* Reposition Left Tibia with Ring Ext Fix, Perc Endo Approach */
"0QSH4DZ"  =  "1"  /* Reposition L Tibia with Hybrid Ext Fix, Perc Endo Approach */
"0QSJ04Z"  =  "1"  /* Reposition Right Fibula with Int Fix, Open Approach */
"0QSJ05Z"  =  "1"  /* Reposition Right Fibula with Ext Fix, Open Approach */
"0QSJ06Z"  =  "1"  /* Reposition Right Fibula with Intramed Fix, Open Approach */
"0QSJ0BZ"  =  "1"  /* Reposition Right Fibula with Monopln Ext Fix, Open Approach */
"0QSJ0CZ"  =  "1"  /* Reposition Right Fibula with Ring Ext Fix, Open Approach */
"0QSJ0DZ"  =  "1"  /* Reposition Right Fibula with Hybrid Ext Fix, Open Approach */
"0QSJ0ZZ"  =  "1"  /* Reposition Right Fibula, Open Approach */
"0QSJ34Z"  =  "1"  /* Reposition Right Fibula with Int Fix, Perc Approach */
"0QSJ35Z"  =  "1"  /* Reposition Right Fibula with Ext Fix, Perc Approach */
"0QSJ36Z"  =  "1"  /* Reposition Right Fibula with Intramed Fix, Perc Approach */
"0QSJ3BZ"  =  "1"  /* Reposition Right Fibula with Monopln Ext Fix, Perc Approach */
"0QSJ3CZ"  =  "1"  /* Reposition Right Fibula with Ring Ext Fix, Perc Approach */
"0QSJ3DZ"  =  "1"  /* Reposition Right Fibula with Hybrid Ext Fix, Perc Approach */
"0QSJ44Z"  =  "1"  /* Reposition Right Fibula with Int Fix, Perc Endo Approach */
"0QSJ45Z"  =  "1"  /* Reposition Right Fibula with Ext Fix, Perc Endo Approach */
"0QSJ46Z"  =  "1"  /* Reposition R Fibula with Intramed Fix, Perc Endo Approach */
"0QSJ4BZ"  =  "1"  /* Reposition R Fibula with Monopln Ext Fix, Perc Endo Approach */
"0QSJ4CZ"  =  "1"  /* Reposition R Fibula with Ring Ext Fix, Perc Endo Approach */
"0QSJ4DZ"  =  "1"  /* Reposition R Fibula with Hybrid Ext Fix, Perc Endo Approach */
"0QSK04Z"  =  "1"  /* Reposition Left Fibula with Int Fix, Open Approach */
"0QSK05Z"  =  "1"  /* Reposition Left Fibula with Ext Fix, Open Approach */
"0QSK06Z"  =  "1"  /* Reposition Left Fibula with Intramed Fix, Open Approach */
"0QSK0BZ"  =  "1"  /* Reposition Left Fibula with Monopln Ext Fix, Open Approach */
"0QSK0CZ"  =  "1"  /* Reposition Left Fibula with Ring Ext Fix, Open Approach */
"0QSK0DZ"  =  "1"  /* Reposition Left Fibula with Hybrid Ext Fix, Open Approach */
"0QSK0ZZ"  =  "1"  /* Reposition Left Fibula, Open Approach */
"0QSK34Z"  =  "1"  /* Reposition Left Fibula with Int Fix, Perc Approach */
"0QSK35Z"  =  "1"  /* Reposition Left Fibula with Ext Fix, Perc Approach */
"0QSK36Z"  =  "1"  /* Reposition Left Fibula with Intramed Fix, Perc Approach */
"0QSK3BZ"  =  "1"  /* Reposition Left Fibula with Monopln Ext Fix, Perc Approach */
"0QSK3CZ"  =  "1"  /* Reposition Left Fibula with Ring Ext Fix, Perc Approach */
"0QSK3DZ"  =  "1"  /* Reposition Left Fibula with Hybrid Ext Fix, Perc Approach */
"0QSK44Z"  =  "1"  /* Reposition Left Fibula with Int Fix, Perc Endo Approach */
"0QSK45Z"  =  "1"  /* Reposition Left Fibula with Ext Fix, Perc Endo Approach */
"0QSK46Z"  =  "1"  /* Reposition Left Fibula with Intramed Fix, Perc Endo Approach */
"0QSK4BZ"  =  "1"  /* Reposition L Fibula with Monopln Ext Fix, Perc Endo Approach */
"0QSK4CZ"  =  "1"  /* Reposition Left Fibula with Ring Ext Fix, Perc Endo Approach */
"0QSK4DZ"  =  "1"  /* Reposition L Fibula with Hybrid Ext Fix, Perc Endo Approach */
"0QSL04Z"  =  "1"  /* Reposition Right Tarsal with Int Fix, Open Approach */
"0QSL05Z"  =  "1"  /* Reposition Right Tarsal with Ext Fix, Open Approach */
"0QSL0ZZ"  =  "1"  /* Reposition Right Tarsal, Open Approach */
"0QSL34Z"  =  "1"  /* Reposition Right Tarsal with Int Fix, Perc Approach */
"0QSL35Z"  =  "1"  /* Reposition Right Tarsal with Ext Fix, Perc Approach */
"0QSL44Z"  =  "1"  /* Reposition Right Tarsal with Int Fix, Perc Endo Approach */
"0QSL45Z"  =  "1"  /* Reposition Right Tarsal with Ext Fix, Perc Endo Approach */
"0QSM04Z"  =  "1"  /* Reposition Left Tarsal with Int Fix, Open Approach */
"0QSM05Z"  =  "1"  /* Reposition Left Tarsal with Ext Fix, Open Approach */
"0QSM0ZZ"  =  "1"  /* Reposition Left Tarsal, Open Approach */
"0QSM34Z"  =  "1"  /* Reposition Left Tarsal with Int Fix, Perc Approach */
"0QSM35Z"  =  "1"  /* Reposition Left Tarsal with Ext Fix, Perc Approach */
"0QSM44Z"  =  "1"  /* Reposition Left Tarsal with Int Fix, Perc Endo Approach */
"0QSM45Z"  =  "1"  /* Reposition Left Tarsal with Ext Fix, Perc Endo Approach */
"0QSN04Z"  =  "1"  /* Reposition Right Metatarsal with Int Fix, Open Approach */
"0QSN05Z"  =  "1"  /* Reposition Right Metatarsal with Ext Fix, Open Approach */
"0QSN0ZZ"  =  "1"  /* Reposition Right Metatarsal, Open Approach */
"0QSN34Z"  =  "1"  /* Reposition Right Metatarsal with Int Fix, Perc Approach */
"0QSN35Z"  =  "1"  /* Reposition Right Metatarsal with Ext Fix, Perc Approach */
"0QSN44Z"  =  "1"  /* Reposition Right Metatarsal with Int Fix, Perc Endo Approach */
"0QSN45Z"  =  "1"  /* Reposition Right Metatarsal with Ext Fix, Perc Endo Approach */
"0QSP04Z"  =  "1"  /* Reposition Left Metatarsal with Int Fix, Open Approach */
"0QSP05Z"  =  "1"  /* Reposition Left Metatarsal with Ext Fix, Open Approach */
"0QSP0ZZ"  =  "1"  /* Reposition Left Metatarsal, Open Approach */
"0QSP34Z"  =  "1"  /* Reposition Left Metatarsal with Int Fix, Perc Approach */
"0QSP35Z"  =  "1"  /* Reposition Left Metatarsal with Ext Fix, Perc Approach */
"0QSP44Z"  =  "1"  /* Reposition Left Metatarsal with Int Fix, Perc Endo Approach */
"0QSP45Z"  =  "1"  /* Reposition Left Metatarsal with Ext Fix, Perc Endo Approach */
"0QSQ04Z"  =  "1"  /* Reposition Right Toe Phalanx with Int Fix, Open Approach */
"0QSQ05Z"  =  "1"  /* Reposition Right Toe Phalanx with Ext Fix, Open Approach */
"0QSQ0ZZ"  =  "1"  /* Reposition Right Toe Phalanx, Open Approach */
"0QSQ34Z"  =  "1"  /* Reposition Right Toe Phalanx with Int Fix, Perc Approach */
"0QSQ35Z"  =  "1"  /* Reposition Right Toe Phalanx with Ext Fix, Perc Approach */
"0QSQ44Z"  =  "1"  /* Reposition R Toe Phalanx with Int Fix, Perc Endo Approach */
"0QSQ45Z"  =  "1"  /* Reposition R Toe Phalanx with Ext Fix, Perc Endo Approach */
"0QSR04Z"  =  "1"  /* Reposition Left Toe Phalanx with Int Fix, Open Approach */
"0QSR05Z"  =  "1"  /* Reposition Left Toe Phalanx with Ext Fix, Open Approach */
"0QSR0ZZ"  =  "1"  /* Reposition Left Toe Phalanx, Open Approach */
"0QSR34Z"  =  "1"  /* Reposition Left Toe Phalanx with Int Fix, Perc Approach */
"0QSR35Z"  =  "1"  /* Reposition Left Toe Phalanx with Ext Fix, Perc Approach */
"0QSR44Z"  =  "1"  /* Reposition Left Toe Phalanx with Int Fix, Perc Endo Approach */
"0QSR45Z"  =  "1"  /* Reposition Left Toe Phalanx with Ext Fix, Perc Endo Approach */
"0QSS04Z"  =  "1"  /* Reposition Coccyx with Int Fix, Open Approach */
"0QSS0ZZ"  =  "1"  /* Reposition Coccyx, Open Approach */
"0QSS34Z"  =  "1"  /* Reposition Coccyx with Int Fix, Perc Approach */
"0QSS3ZZ"  =  "1"  /* Reposition Coccyx, Percutaneous Approach */
"0QSS44Z"  =  "1"  /* Reposition Coccyx with Int Fix, Perc Endo Approach */
"0QSS4ZZ"  =  "1"  /* Reposition Coccyx, Percutaneous Endoscopic Approach */
"0QSSXZZ"  =  "1"  /* Reposition Coccyx, External Approach */
"0QT20ZZ"  =  "1"  /* Resection of Right Pelvic Bone, Open Approach */
"0QT30ZZ"  =  "1"  /* Resection of Left Pelvic Bone, Open Approach */
"0QT40ZZ"  =  "1"  /* Resection of Right Acetabulum, Open Approach */
"0QT50ZZ"  =  "1"  /* Resection of Left Acetabulum, Open Approach */
"0QT60ZZ"  =  "1"  /* Resection of Right Upper Femur, Open Approach */
"0QT70ZZ"  =  "1"  /* Resection of Left Upper Femur, Open Approach */
"0QT80ZZ"  =  "1"  /* Resection of Right Femoral Shaft, Open Approach */
"0QT90ZZ"  =  "1"  /* Resection of Left Femoral Shaft, Open Approach */
"0QTB0ZZ"  =  "1"  /* Resection of Right Lower Femur, Open Approach */
"0QTC0ZZ"  =  "1"  /* Resection of Left Lower Femur, Open Approach */
"0QTD0ZZ"  =  "1"  /* Resection of Right Patella, Open Approach */
"0QTF0ZZ"  =  "1"  /* Resection of Left Patella, Open Approach */
"0QTG0ZZ"  =  "1"  /* Resection of Right Tibia, Open Approach */
"0QTH0ZZ"  =  "1"  /* Resection of Left Tibia, Open Approach */
"0QTJ0ZZ"  =  "1"  /* Resection of Right Fibula, Open Approach */
"0QTK0ZZ"  =  "1"  /* Resection of Left Fibula, Open Approach */
"0QTL0ZZ"  =  "1"  /* Resection of Right Tarsal, Open Approach */
"0QTM0ZZ"  =  "1"  /* Resection of Left Tarsal, Open Approach */
"0QTN0ZZ"  =  "1"  /* Resection of Right Metatarsal, Open Approach */
"0QTP0ZZ"  =  "1"  /* Resection of Left Metatarsal, Open Approach */
"0QTQ0ZZ"  =  "1"  /* Resection of Right Toe Phalanx, Open Approach */
"0QTR0ZZ"  =  "1"  /* Resection of Left Toe Phalanx, Open Approach */
"0QTS0ZZ"  =  "1"  /* Resection of Coccyx, Open Approach */
"0QU007Z"  =  "1"  /* Supplement Lumbar Vertebra with Autol Sub, Open Approach */
"0QU00JZ"  =  "1"  /* Supplement Lumbar Vertebra with Synth Sub, Open Approach */
"0QU00KZ"  =  "1"  /* Supplement Lumbar Vertebra with Nonaut Sub, Open Approach */
"0QU037Z"  =  "1"  /* Supplement Lumbar Vertebra with Autol Sub, Perc Approach */
"0QU03JZ"  =  "1"  /* Supplement Lumbar Vertebra with Synth Sub, Perc Approach */
"0QU03KZ"  =  "1"  /* Supplement Lumbar Vertebra with Nonaut Sub, Perc Approach */
"0QU047Z"  =  "1"  /* Supplement Lum Vertebra with Autol Sub, Perc Endo Approach */
"0QU04JZ"  =  "1"  /* Supplement Lum Vertebra with Synth Sub, Perc Endo Approach */
"0QU04KZ"  =  "1"  /* Supplement Lum Vertebra with Nonaut Sub, Perc Endo Approach */
"0QU107Z"  =  "1"  /* Supplement Sacrum with Autol Sub, Open Approach */
"0QU10JZ"  =  "1"  /* Supplement Sacrum with Synthetic Substitute, Open Approach */
"0QU10KZ"  =  "1"  /* Supplement Sacrum with Nonaut Sub, Open Approach */
"0QU137Z"  =  "1"  /* Supplement Sacrum with Autol Sub, Perc Approach */
"0QU13JZ"  =  "1"  /* Supplement Sacrum with Synthetic Substitute, Perc Approach */
"0QU13KZ"  =  "1"  /* Supplement Sacrum with Nonaut Sub, Perc Approach */
"0QU147Z"  =  "1"  /* Supplement Sacrum with Autol Sub, Perc Endo Approach */
"0QU14JZ"  =  "1"  /* Supplement Sacrum with Synth Sub, Perc Endo Approach */
"0QU14KZ"  =  "1"  /* Supplement Sacrum with Nonaut Sub, Perc Endo Approach */
"0QU207Z"  =  "1"  /* Supplement Right Pelvic Bone with Autol Sub, Open Approach */
"0QU20JZ"  =  "1"  /* Supplement Right Pelvic Bone with Synth Sub, Open Approach */
"0QU20KZ"  =  "1"  /* Supplement Right Pelvic Bone with Nonaut Sub, Open Approach */
"0QU237Z"  =  "1"  /* Supplement Right Pelvic Bone with Autol Sub, Perc Approach */
"0QU23JZ"  =  "1"  /* Supplement Right Pelvic Bone with Synth Sub, Perc Approach */
"0QU23KZ"  =  "1"  /* Supplement Right Pelvic Bone with Nonaut Sub, Perc Approach */
"0QU247Z"  =  "1"  /* Supplement R Pelvic Bone with Autol Sub, Perc Endo Approach */
"0QU24JZ"  =  "1"  /* Supplement R Pelvic Bone with Synth Sub, Perc Endo Approach */
"0QU24KZ"  =  "1"  /* Supplement R Pelvic Bone with Nonaut Sub, Perc Endo Approach */
"0QU307Z"  =  "1"  /* Supplement Left Pelvic Bone with Autol Sub, Open Approach */
"0QU30JZ"  =  "1"  /* Supplement Left Pelvic Bone with Synth Sub, Open Approach */
"0QU30KZ"  =  "1"  /* Supplement Left Pelvic Bone with Nonaut Sub, Open Approach */
"0QU337Z"  =  "1"  /* Supplement Left Pelvic Bone with Autol Sub, Perc Approach */
"0QU33JZ"  =  "1"  /* Supplement Left Pelvic Bone with Synth Sub, Perc Approach */
"0QU33KZ"  =  "1"  /* Supplement Left Pelvic Bone with Nonaut Sub, Perc Approach */
"0QU347Z"  =  "1"  /* Supplement L Pelvic Bone with Autol Sub, Perc Endo Approach */
"0QU34JZ"  =  "1"  /* Supplement L Pelvic Bone with Synth Sub, Perc Endo Approach */
"0QU34KZ"  =  "1"  /* Supplement L Pelvic Bone with Nonaut Sub, Perc Endo Approach */
"0QU407Z"  =  "1"  /* Supplement Right Acetabulum with Autol Sub, Open Approach */
"0QU40JZ"  =  "1"  /* Supplement Right Acetabulum with Synth Sub, Open Approach */
"0QU40KZ"  =  "1"  /* Supplement Right Acetabulum with Nonaut Sub, Open Approach */
"0QU437Z"  =  "1"  /* Supplement Right Acetabulum with Autol Sub, Perc Approach */
"0QU43JZ"  =  "1"  /* Supplement Right Acetabulum with Synth Sub, Perc Approach */
"0QU43KZ"  =  "1"  /* Supplement Right Acetabulum with Nonaut Sub, Perc Approach */
"0QU447Z"  =  "1"  /* Supplement R Acetabulum with Autol Sub, Perc Endo Approach */
"0QU44JZ"  =  "1"  /* Supplement R Acetabulum with Synth Sub, Perc Endo Approach */
"0QU44KZ"  =  "1"  /* Supplement R Acetabulum with Nonaut Sub, Perc Endo Approach */
"0QU507Z"  =  "1"  /* Supplement Left Acetabulum with Autol Sub, Open Approach */
"0QU50JZ"  =  "1"  /* Supplement Left Acetabulum with Synth Sub, Open Approach */
"0QU50KZ"  =  "1"  /* Supplement Left Acetabulum with Nonaut Sub, Open Approach */
"0QU537Z"  =  "1"  /* Supplement Left Acetabulum with Autol Sub, Perc Approach */
"0QU53JZ"  =  "1"  /* Supplement Left Acetabulum with Synth Sub, Perc Approach */
"0QU53KZ"  =  "1"  /* Supplement Left Acetabulum with Nonaut Sub, Perc Approach */
"0QU547Z"  =  "1"  /* Supplement L Acetabulum with Autol Sub, Perc Endo Approach */
"0QU54JZ"  =  "1"  /* Supplement L Acetabulum with Synth Sub, Perc Endo Approach */
"0QU54KZ"  =  "1"  /* Supplement L Acetabulum with Nonaut Sub, Perc Endo Approach */
"0QU607Z"  =  "1"  /* Supplement Right Upper Femur with Autol Sub, Open Approach */
"0QU60JZ"  =  "1"  /* Supplement Right Upper Femur with Synth Sub, Open Approach */
"0QU60KZ"  =  "1"  /* Supplement Right Upper Femur with Nonaut Sub, Open Approach */
"0QU637Z"  =  "1"  /* Supplement Right Upper Femur with Autol Sub, Perc Approach */
"0QU63JZ"  =  "1"  /* Supplement Right Upper Femur with Synth Sub, Perc Approach */
"0QU63KZ"  =  "1"  /* Supplement Right Upper Femur with Nonaut Sub, Perc Approach */
"0QU647Z"  =  "1"  /* Supplement R Up Femur with Autol Sub, Perc Endo Approach */
"0QU64JZ"  =  "1"  /* Supplement R Up Femur with Synth Sub, Perc Endo Approach */
"0QU64KZ"  =  "1"  /* Supplement R Up Femur with Nonaut Sub, Perc Endo Approach */
"0QU707Z"  =  "1"  /* Supplement Left Upper Femur with Autol Sub, Open Approach */
"0QU70JZ"  =  "1"  /* Supplement Left Upper Femur with Synth Sub, Open Approach */
"0QU70KZ"  =  "1"  /* Supplement Left Upper Femur with Nonaut Sub, Open Approach */
"0QU737Z"  =  "1"  /* Supplement Left Upper Femur with Autol Sub, Perc Approach */
"0QU73JZ"  =  "1"  /* Supplement Left Upper Femur with Synth Sub, Perc Approach */
"0QU73KZ"  =  "1"  /* Supplement Left Upper Femur with Nonaut Sub, Perc Approach */
"0QU747Z"  =  "1"  /* Supplement L Up Femur with Autol Sub, Perc Endo Approach */
"0QU74JZ"  =  "1"  /* Supplement L Up Femur with Synth Sub, Perc Endo Approach */
"0QU74KZ"  =  "1"  /* Supplement L Up Femur with Nonaut Sub, Perc Endo Approach */
"0QU807Z"  =  "1"  /* Supplement Right Femoral Shaft with Autol Sub, Open Approach */
"0QU80JZ"  =  "1"  /* Supplement Right Femoral Shaft with Synth Sub, Open Approach */
"0QU80KZ"  =  "1"  /* Supplement R Femur Shaft with Nonaut Sub, Open Approach */
"0QU837Z"  =  "1"  /* Supplement Right Femoral Shaft with Autol Sub, Perc Approach */
"0QU83JZ"  =  "1"  /* Supplement Right Femoral Shaft with Synth Sub, Perc Approach */
"0QU83KZ"  =  "1"  /* Supplement R Femur Shaft with Nonaut Sub, Perc Approach */
"0QU847Z"  =  "1"  /* Supplement R Femur Shaft with Autol Sub, Perc Endo Approach */
"0QU84JZ"  =  "1"  /* Supplement R Femur Shaft with Synth Sub, Perc Endo Approach */
"0QU84KZ"  =  "1"  /* Supplement R Femur Shaft with Nonaut Sub, Perc Endo Approach */
"0QU907Z"  =  "1"  /* Supplement Left Femoral Shaft with Autol Sub, Open Approach */
"0QU90JZ"  =  "1"  /* Supplement Left Femoral Shaft with Synth Sub, Open Approach */
"0QU90KZ"  =  "1"  /* Supplement Left Femoral Shaft with Nonaut Sub, Open Approach */
"0QU937Z"  =  "1"  /* Supplement Left Femoral Shaft with Autol Sub, Perc Approach */
"0QU93JZ"  =  "1"  /* Supplement Left Femoral Shaft with Synth Sub, Perc Approach */
"0QU93KZ"  =  "1"  /* Supplement Left Femoral Shaft with Nonaut Sub, Perc Approach */
"0QU947Z"  =  "1"  /* Supplement L Femur Shaft with Autol Sub, Perc Endo Approach */
"0QU94JZ"  =  "1"  /* Supplement L Femur Shaft with Synth Sub, Perc Endo Approach */
"0QU94KZ"  =  "1"  /* Supplement L Femur Shaft with Nonaut Sub, Perc Endo Approach */
"0QUB07Z"  =  "1"  /* Supplement Right Lower Femur with Autol Sub, Open Approach */
"0QUB0JZ"  =  "1"  /* Supplement Right Lower Femur with Synth Sub, Open Approach */
"0QUB0KZ"  =  "1"  /* Supplement Right Lower Femur with Nonaut Sub, Open Approach */
"0QUB37Z"  =  "1"  /* Supplement Right Lower Femur with Autol Sub, Perc Approach */
"0QUB3JZ"  =  "1"  /* Supplement Right Lower Femur with Synth Sub, Perc Approach */
"0QUB3KZ"  =  "1"  /* Supplement Right Lower Femur with Nonaut Sub, Perc Approach */
"0QUB47Z"  =  "1"  /* Supplement R Low Femur with Autol Sub, Perc Endo Approach */
"0QUB4JZ"  =  "1"  /* Supplement R Low Femur with Synth Sub, Perc Endo Approach */
"0QUB4KZ"  =  "1"  /* Supplement R Low Femur with Nonaut Sub, Perc Endo Approach */
"0QUC07Z"  =  "1"  /* Supplement Left Lower Femur with Autol Sub, Open Approach */
"0QUC0JZ"  =  "1"  /* Supplement Left Lower Femur with Synth Sub, Open Approach */
"0QUC0KZ"  =  "1"  /* Supplement Left Lower Femur with Nonaut Sub, Open Approach */
"0QUC37Z"  =  "1"  /* Supplement Left Lower Femur with Autol Sub, Perc Approach */
"0QUC3JZ"  =  "1"  /* Supplement Left Lower Femur with Synth Sub, Perc Approach */
"0QUC3KZ"  =  "1"  /* Supplement Left Lower Femur with Nonaut Sub, Perc Approach */
"0QUC47Z"  =  "1"  /* Supplement L Low Femur with Autol Sub, Perc Endo Approach */
"0QUC4JZ"  =  "1"  /* Supplement L Low Femur with Synth Sub, Perc Endo Approach */
"0QUC4KZ"  =  "1"  /* Supplement L Low Femur with Nonaut Sub, Perc Endo Approach */
"0QUD07Z"  =  "1"  /* Supplement Right Patella with Autol Sub, Open Approach */
"0QUD0JZ"  =  "1"  /* Supplement Right Patella with Synth Sub, Open Approach */
"0QUD0KZ"  =  "1"  /* Supplement Right Patella with Nonaut Sub, Open Approach */
"0QUD37Z"  =  "1"  /* Supplement Right Patella with Autol Sub, Perc Approach */
"0QUD3JZ"  =  "1"  /* Supplement Right Patella with Synth Sub, Perc Approach */
"0QUD3KZ"  =  "1"  /* Supplement Right Patella with Nonaut Sub, Perc Approach */
"0QUD47Z"  =  "1"  /* Supplement Right Patella with Autol Sub, Perc Endo Approach */
"0QUD4JZ"  =  "1"  /* Supplement Right Patella with Synth Sub, Perc Endo Approach */
"0QUD4KZ"  =  "1"  /* Supplement Right Patella with Nonaut Sub, Perc Endo Approach */
"0QUF07Z"  =  "1"  /* Supplement Left Patella with Autol Sub, Open Approach */
"0QUF0JZ"  =  "1"  /* Supplement Left Patella with Synth Sub, Open Approach */
"0QUF0KZ"  =  "1"  /* Supplement Left Patella with Nonaut Sub, Open Approach */
"0QUF37Z"  =  "1"  /* Supplement Left Patella with Autol Sub, Perc Approach */
"0QUF3JZ"  =  "1"  /* Supplement Left Patella with Synth Sub, Perc Approach */
"0QUF3KZ"  =  "1"  /* Supplement Left Patella with Nonaut Sub, Perc Approach */
"0QUF47Z"  =  "1"  /* Supplement Left Patella with Autol Sub, Perc Endo Approach */
"0QUF4JZ"  =  "1"  /* Supplement Left Patella with Synth Sub, Perc Endo Approach */
"0QUF4KZ"  =  "1"  /* Supplement Left Patella with Nonaut Sub, Perc Endo Approach */
"0QUG07Z"  =  "1"  /* Supplement Right Tibia with Autol Sub, Open Approach */
"0QUG0JZ"  =  "1"  /* Supplement Right Tibia with Synth Sub, Open Approach */
"0QUG0KZ"  =  "1"  /* Supplement Right Tibia with Nonaut Sub, Open Approach */
"0QUG37Z"  =  "1"  /* Supplement Right Tibia with Autol Sub, Perc Approach */
"0QUG3JZ"  =  "1"  /* Supplement Right Tibia with Synth Sub, Perc Approach */
"0QUG3KZ"  =  "1"  /* Supplement Right Tibia with Nonaut Sub, Perc Approach */
"0QUG47Z"  =  "1"  /* Supplement Right Tibia with Autol Sub, Perc Endo Approach */
"0QUG4JZ"  =  "1"  /* Supplement Right Tibia with Synth Sub, Perc Endo Approach */
"0QUG4KZ"  =  "1"  /* Supplement Right Tibia with Nonaut Sub, Perc Endo Approach */
"0QUH07Z"  =  "1"  /* Supplement Left Tibia with Autol Sub, Open Approach */
"0QUH0JZ"  =  "1"  /* Supplement Left Tibia with Synth Sub, Open Approach */
"0QUH0KZ"  =  "1"  /* Supplement Left Tibia with Nonaut Sub, Open Approach */
"0QUH37Z"  =  "1"  /* Supplement Left Tibia with Autol Sub, Perc Approach */
"0QUH3JZ"  =  "1"  /* Supplement Left Tibia with Synth Sub, Perc Approach */
"0QUH3KZ"  =  "1"  /* Supplement Left Tibia with Nonaut Sub, Perc Approach */
"0QUH47Z"  =  "1"  /* Supplement Left Tibia with Autol Sub, Perc Endo Approach */
"0QUH4JZ"  =  "1"  /* Supplement Left Tibia with Synth Sub, Perc Endo Approach */
"0QUH4KZ"  =  "1"  /* Supplement Left Tibia with Nonaut Sub, Perc Endo Approach */
"0QUJ07Z"  =  "1"  /* Supplement Right Fibula with Autol Sub, Open Approach */
"0QUJ0JZ"  =  "1"  /* Supplement Right Fibula with Synth Sub, Open Approach */
"0QUJ0KZ"  =  "1"  /* Supplement Right Fibula with Nonaut Sub, Open Approach */
"0QUJ37Z"  =  "1"  /* Supplement Right Fibula with Autol Sub, Perc Approach */
"0QUJ3JZ"  =  "1"  /* Supplement Right Fibula with Synth Sub, Perc Approach */
"0QUJ3KZ"  =  "1"  /* Supplement Right Fibula with Nonaut Sub, Perc Approach */
"0QUJ47Z"  =  "1"  /* Supplement Right Fibula with Autol Sub, Perc Endo Approach */
"0QUJ4JZ"  =  "1"  /* Supplement Right Fibula with Synth Sub, Perc Endo Approach */
"0QUJ4KZ"  =  "1"  /* Supplement Right Fibula with Nonaut Sub, Perc Endo Approach */
"0QUK07Z"  =  "1"  /* Supplement Left Fibula with Autol Sub, Open Approach */
"0QUK0JZ"  =  "1"  /* Supplement Left Fibula with Synth Sub, Open Approach */
"0QUK0KZ"  =  "1"  /* Supplement Left Fibula with Nonaut Sub, Open Approach */
"0QUK37Z"  =  "1"  /* Supplement Left Fibula with Autol Sub, Perc Approach */
"0QUK3JZ"  =  "1"  /* Supplement Left Fibula with Synth Sub, Perc Approach */
"0QUK3KZ"  =  "1"  /* Supplement Left Fibula with Nonaut Sub, Perc Approach */
"0QUK47Z"  =  "1"  /* Supplement Left Fibula with Autol Sub, Perc Endo Approach */
"0QUK4JZ"  =  "1"  /* Supplement Left Fibula with Synth Sub, Perc Endo Approach */
"0QUK4KZ"  =  "1"  /* Supplement Left Fibula with Nonaut Sub, Perc Endo Approach */
"0QUL07Z"  =  "1"  /* Supplement Right Tarsal with Autol Sub, Open Approach */
"0QUL0JZ"  =  "1"  /* Supplement Right Tarsal with Synth Sub, Open Approach */
"0QUL0KZ"  =  "1"  /* Supplement Right Tarsal with Nonaut Sub, Open Approach */
"0QUL37Z"  =  "1"  /* Supplement Right Tarsal with Autol Sub, Perc Approach */
"0QUL3JZ"  =  "1"  /* Supplement Right Tarsal with Synth Sub, Perc Approach */
"0QUL3KZ"  =  "1"  /* Supplement Right Tarsal with Nonaut Sub, Perc Approach */
"0QUL47Z"  =  "1"  /* Supplement Right Tarsal with Autol Sub, Perc Endo Approach */
"0QUL4JZ"  =  "1"  /* Supplement Right Tarsal with Synth Sub, Perc Endo Approach */
"0QUL4KZ"  =  "1"  /* Supplement Right Tarsal with Nonaut Sub, Perc Endo Approach */
"0QUM07Z"  =  "1"  /* Supplement Left Tarsal with Autol Sub, Open Approach */
"0QUM0JZ"  =  "1"  /* Supplement Left Tarsal with Synth Sub, Open Approach */
"0QUM0KZ"  =  "1"  /* Supplement Left Tarsal with Nonaut Sub, Open Approach */
"0QUM37Z"  =  "1"  /* Supplement Left Tarsal with Autol Sub, Perc Approach */
"0QUM3JZ"  =  "1"  /* Supplement Left Tarsal with Synth Sub, Perc Approach */
"0QUM3KZ"  =  "1"  /* Supplement Left Tarsal with Nonaut Sub, Perc Approach */
"0QUM47Z"  =  "1"  /* Supplement Left Tarsal with Autol Sub, Perc Endo Approach */
"0QUM4JZ"  =  "1"  /* Supplement Left Tarsal with Synth Sub, Perc Endo Approach */
"0QUM4KZ"  =  "1"  /* Supplement Left Tarsal with Nonaut Sub, Perc Endo Approach */
"0QUN07Z"  =  "1"  /* Supplement Right Metatarsal with Autol Sub, Open Approach */
"0QUN0JZ"  =  "1"  /* Supplement Right Metatarsal with Synth Sub, Open Approach */
"0QUN0KZ"  =  "1"  /* Supplement Right Metatarsal with Nonaut Sub, Open Approach */
"0QUN37Z"  =  "1"  /* Supplement Right Metatarsal with Autol Sub, Perc Approach */
"0QUN3JZ"  =  "1"  /* Supplement Right Metatarsal with Synth Sub, Perc Approach */
"0QUN3KZ"  =  "1"  /* Supplement Right Metatarsal with Nonaut Sub, Perc Approach */
"0QUN47Z"  =  "1"  /* Supplement R Metatarsal with Autol Sub, Perc Endo Approach */
"0QUN4JZ"  =  "1"  /* Supplement R Metatarsal with Synth Sub, Perc Endo Approach */
"0QUN4KZ"  =  "1"  /* Supplement R Metatarsal with Nonaut Sub, Perc Endo Approach */
"0QUP07Z"  =  "1"  /* Supplement Left Metatarsal with Autol Sub, Open Approach */
"0QUP0JZ"  =  "1"  /* Supplement Left Metatarsal with Synth Sub, Open Approach */
"0QUP0KZ"  =  "1"  /* Supplement Left Metatarsal with Nonaut Sub, Open Approach */
"0QUP37Z"  =  "1"  /* Supplement Left Metatarsal with Autol Sub, Perc Approach */
"0QUP3JZ"  =  "1"  /* Supplement Left Metatarsal with Synth Sub, Perc Approach */
"0QUP3KZ"  =  "1"  /* Supplement Left Metatarsal with Nonaut Sub, Perc Approach */
"0QUP47Z"  =  "1"  /* Supplement L Metatarsal with Autol Sub, Perc Endo Approach */
"0QUP4JZ"  =  "1"  /* Supplement L Metatarsal with Synth Sub, Perc Endo Approach */
"0QUP4KZ"  =  "1"  /* Supplement L Metatarsal with Nonaut Sub, Perc Endo Approach */
"0QUQ07Z"  =  "1"  /* Supplement Right Toe Phalanx with Autol Sub, Open Approach */
"0QUQ0JZ"  =  "1"  /* Supplement Right Toe Phalanx with Synth Sub, Open Approach */
"0QUQ0KZ"  =  "1"  /* Supplement Right Toe Phalanx with Nonaut Sub, Open Approach */
"0QUQ37Z"  =  "1"  /* Supplement Right Toe Phalanx with Autol Sub, Perc Approach */
"0QUQ3JZ"  =  "1"  /* Supplement Right Toe Phalanx with Synth Sub, Perc Approach */
"0QUQ3KZ"  =  "1"  /* Supplement Right Toe Phalanx with Nonaut Sub, Perc Approach */
"0QUQ47Z"  =  "1"  /* Supplement R Toe Phalanx with Autol Sub, Perc Endo Approach */
"0QUQ4JZ"  =  "1"  /* Supplement R Toe Phalanx with Synth Sub, Perc Endo Approach */
"0QUQ4KZ"  =  "1"  /* Supplement R Toe Phalanx with Nonaut Sub, Perc Endo Approach */
"0QUR07Z"  =  "1"  /* Supplement Left Toe Phalanx with Autol Sub, Open Approach */
"0QUR0JZ"  =  "1"  /* Supplement Left Toe Phalanx with Synth Sub, Open Approach */
"0QUR0KZ"  =  "1"  /* Supplement Left Toe Phalanx with Nonaut Sub, Open Approach */
"0QUR37Z"  =  "1"  /* Supplement Left Toe Phalanx with Autol Sub, Perc Approach */
"0QUR3JZ"  =  "1"  /* Supplement Left Toe Phalanx with Synth Sub, Perc Approach */
"0QUR3KZ"  =  "1"  /* Supplement Left Toe Phalanx with Nonaut Sub, Perc Approach */
"0QUR47Z"  =  "1"  /* Supplement L Toe Phalanx with Autol Sub, Perc Endo Approach */
"0QUR4JZ"  =  "1"  /* Supplement L Toe Phalanx with Synth Sub, Perc Endo Approach */
"0QUR4KZ"  =  "1"  /* Supplement L Toe Phalanx with Nonaut Sub, Perc Endo Approach */
"0QUS07Z"  =  "1"  /* Supplement Coccyx with Autol Sub, Open Approach */
"0QUS0JZ"  =  "1"  /* Supplement Coccyx with Synthetic Substitute, Open Approach */
"0QUS0KZ"  =  "1"  /* Supplement Coccyx with Nonaut Sub, Open Approach */
"0QUS37Z"  =  "1"  /* Supplement Coccyx with Autol Sub, Perc Approach */
"0QUS3JZ"  =  "1"  /* Supplement Coccyx with Synthetic Substitute, Perc Approach */
"0QUS3KZ"  =  "1"  /* Supplement Coccyx with Nonaut Sub, Perc Approach */
"0QUS47Z"  =  "1"  /* Supplement Coccyx with Autol Sub, Perc Endo Approach */
"0QUS4JZ"  =  "1"  /* Supplement Coccyx with Synth Sub, Perc Endo Approach */
"0QUS4KZ"  =  "1"  /* Supplement Coccyx with Nonaut Sub, Perc Endo Approach */
"0QW004Z"  =  "1"  /* Revision of Int Fix in Lum Vertebra, Open Approach */
"0QW007Z"  =  "1"  /* Revision of Autol Sub in Lum Vertebra, Open Approach */
"0QW00JZ"  =  "1"  /* Revision of Synth Sub in Lum Vertebra, Open Approach */
"0QW00KZ"  =  "1"  /* Revision of Nonaut Sub in Lum Vertebra, Open Approach */
"0QW034Z"  =  "1"  /* Revision of Int Fix in Lum Vertebra, Perc Approach */
"0QW037Z"  =  "1"  /* Revision of Autol Sub in Lum Vertebra, Perc Approach */
"0QW03JZ"  =  "1"  /* Revision of Synth Sub in Lum Vertebra, Perc Approach */
"0QW03KZ"  =  "1"  /* Revision of Nonaut Sub in Lum Vertebra, Perc Approach */
"0QW044Z"  =  "1"  /* Revision of Int Fix in Lum Vertebra, Perc Endo Approach */
"0QW047Z"  =  "1"  /* Revision of Autol Sub in Lum Vertebra, Perc Endo Approach */
"0QW04JZ"  =  "1"  /* Revision of Synth Sub in Lum Vertebra, Perc Endo Approach */
"0QW04KZ"  =  "1"  /* Revision of Nonaut Sub in Lum Vertebra, Perc Endo Approach */
"0QW104Z"  =  "1"  /* Revision of Int Fix in Sacrum, Open Approach */
"0QW107Z"  =  "1"  /* Revision of Autol Sub in Sacrum, Open Approach */
"0QW10JZ"  =  "1"  /* Revision of Synthetic Substitute in Sacrum, Open Approach */
"0QW10KZ"  =  "1"  /* Revision of Nonaut Sub in Sacrum, Open Approach */
"0QW134Z"  =  "1"  /* Revision of Int Fix in Sacrum, Perc Approach */
"0QW137Z"  =  "1"  /* Revision of Autol Sub in Sacrum, Perc Approach */
"0QW13JZ"  =  "1"  /* Revision of Synthetic Substitute in Sacrum, Perc Approach */
"0QW13KZ"  =  "1"  /* Revision of Nonaut Sub in Sacrum, Perc Approach */
"0QW144Z"  =  "1"  /* Revision of Int Fix in Sacrum, Perc Endo Approach */
"0QW147Z"  =  "1"  /* Revision of Autol Sub in Sacrum, Perc Endo Approach */
"0QW14JZ"  =  "1"  /* Revision of Synth Sub in Sacrum, Perc Endo Approach */
"0QW14KZ"  =  "1"  /* Revision of Nonaut Sub in Sacrum, Perc Endo Approach */
"0QW204Z"  =  "1"  /* Revision of Int Fix in R Pelvic Bone, Open Approach */
"0QW205Z"  =  "1"  /* Revision of Ext Fix in R Pelvic Bone, Open Approach */
"0QW207Z"  =  "1"  /* Revision of Autol Sub in R Pelvic Bone, Open Approach */
"0QW20JZ"  =  "1"  /* Revision of Synth Sub in R Pelvic Bone, Open Approach */
"0QW20KZ"  =  "1"  /* Revision of Nonaut Sub in R Pelvic Bone, Open Approach */
"0QW234Z"  =  "1"  /* Revision of Int Fix in R Pelvic Bone, Perc Approach */
"0QW235Z"  =  "1"  /* Revision of Ext Fix in R Pelvic Bone, Perc Approach */
"0QW237Z"  =  "1"  /* Revision of Autol Sub in R Pelvic Bone, Perc Approach */
"0QW23JZ"  =  "1"  /* Revision of Synth Sub in R Pelvic Bone, Perc Approach */
"0QW23KZ"  =  "1"  /* Revision of Nonaut Sub in R Pelvic Bone, Perc Approach */
"0QW244Z"  =  "1"  /* Revision of Int Fix in R Pelvic Bone, Perc Endo Approach */
"0QW245Z"  =  "1"  /* Revision of Ext Fix in R Pelvic Bone, Perc Endo Approach */
"0QW247Z"  =  "1"  /* Revision of Autol Sub in R Pelvic Bone, Perc Endo Approach */
"0QW24JZ"  =  "1"  /* Revision of Synth Sub in R Pelvic Bone, Perc Endo Approach */
"0QW24KZ"  =  "1"  /* Revision of Nonaut Sub in R Pelvic Bone, Perc Endo Approach */
"0QW304Z"  =  "1"  /* Revision of Int Fix in L Pelvic Bone, Open Approach */
"0QW305Z"  =  "1"  /* Revision of Ext Fix in L Pelvic Bone, Open Approach */
"0QW307Z"  =  "1"  /* Revision of Autol Sub in L Pelvic Bone, Open Approach */
"0QW30JZ"  =  "1"  /* Revision of Synth Sub in L Pelvic Bone, Open Approach */
"0QW30KZ"  =  "1"  /* Revision of Nonaut Sub in L Pelvic Bone, Open Approach */
"0QW334Z"  =  "1"  /* Revision of Int Fix in L Pelvic Bone, Perc Approach */
"0QW335Z"  =  "1"  /* Revision of Ext Fix in L Pelvic Bone, Perc Approach */
"0QW337Z"  =  "1"  /* Revision of Autol Sub in L Pelvic Bone, Perc Approach */
"0QW33JZ"  =  "1"  /* Revision of Synth Sub in L Pelvic Bone, Perc Approach */
"0QW33KZ"  =  "1"  /* Revision of Nonaut Sub in L Pelvic Bone, Perc Approach */
"0QW344Z"  =  "1"  /* Revision of Int Fix in L Pelvic Bone, Perc Endo Approach */
"0QW345Z"  =  "1"  /* Revision of Ext Fix in L Pelvic Bone, Perc Endo Approach */
"0QW347Z"  =  "1"  /* Revision of Autol Sub in L Pelvic Bone, Perc Endo Approach */
"0QW34JZ"  =  "1"  /* Revision of Synth Sub in L Pelvic Bone, Perc Endo Approach */
"0QW34KZ"  =  "1"  /* Revision of Nonaut Sub in L Pelvic Bone, Perc Endo Approach */
"0QW404Z"  =  "1"  /* Revision of Int Fix in R Acetabulum, Open Approach */
"0QW407Z"  =  "1"  /* Revision of Autol Sub in R Acetabulum, Open Approach */
"0QW40JZ"  =  "1"  /* Revision of Synth Sub in R Acetabulum, Open Approach */
"0QW40KZ"  =  "1"  /* Revision of Nonaut Sub in R Acetabulum, Open Approach */
"0QW434Z"  =  "1"  /* Revision of Int Fix in R Acetabulum, Perc Approach */
"0QW437Z"  =  "1"  /* Revision of Autol Sub in R Acetabulum, Perc Approach */
"0QW43JZ"  =  "1"  /* Revision of Synth Sub in R Acetabulum, Perc Approach */
"0QW43KZ"  =  "1"  /* Revision of Nonaut Sub in R Acetabulum, Perc Approach */
"0QW444Z"  =  "1"  /* Revision of Int Fix in R Acetabulum, Perc Endo Approach */
"0QW447Z"  =  "1"  /* Revision of Autol Sub in R Acetabulum, Perc Endo Approach */
"0QW44JZ"  =  "1"  /* Revision of Synth Sub in R Acetabulum, Perc Endo Approach */
"0QW44KZ"  =  "1"  /* Revision of Nonaut Sub in R Acetabulum, Perc Endo Approach */
"0QW504Z"  =  "1"  /* Revision of Int Fix in L Acetabulum, Open Approach */
"0QW507Z"  =  "1"  /* Revision of Autol Sub in L Acetabulum, Open Approach */
"0QW50JZ"  =  "1"  /* Revision of Synth Sub in L Acetabulum, Open Approach */
"0QW50KZ"  =  "1"  /* Revision of Nonaut Sub in L Acetabulum, Open Approach */
"0QW534Z"  =  "1"  /* Revision of Int Fix in L Acetabulum, Perc Approach */
"0QW537Z"  =  "1"  /* Revision of Autol Sub in L Acetabulum, Perc Approach */
"0QW53JZ"  =  "1"  /* Revision of Synth Sub in L Acetabulum, Perc Approach */
"0QW53KZ"  =  "1"  /* Revision of Nonaut Sub in L Acetabulum, Perc Approach */
"0QW544Z"  =  "1"  /* Revision of Int Fix in L Acetabulum, Perc Endo Approach */
"0QW547Z"  =  "1"  /* Revision of Autol Sub in L Acetabulum, Perc Endo Approach */
"0QW54JZ"  =  "1"  /* Revision of Synth Sub in L Acetabulum, Perc Endo Approach */
"0QW54KZ"  =  "1"  /* Revision of Nonaut Sub in L Acetabulum, Perc Endo Approach */
"0QW604Z"  =  "1"  /* Revision of Int Fix in R Up Femur, Open Approach */
"0QW605Z"  =  "1"  /* Revision of Ext Fix in R Up Femur, Open Approach */
"0QW607Z"  =  "1"  /* Revision of Autol Sub in R Up Femur, Open Approach */
"0QW60JZ"  =  "1"  /* Revision of Synth Sub in R Up Femur, Open Approach */
"0QW60KZ"  =  "1"  /* Revision of Nonaut Sub in R Up Femur, Open Approach */
"0QW634Z"  =  "1"  /* Revision of Int Fix in R Up Femur, Perc Approach */
"0QW635Z"  =  "1"  /* Revision of Ext Fix in R Up Femur, Perc Approach */
"0QW637Z"  =  "1"  /* Revision of Autol Sub in R Up Femur, Perc Approach */
"0QW63JZ"  =  "1"  /* Revision of Synth Sub in R Up Femur, Perc Approach */
"0QW63KZ"  =  "1"  /* Revision of Nonaut Sub in R Up Femur, Perc Approach */
"0QW644Z"  =  "1"  /* Revision of Int Fix in R Up Femur, Perc Endo Approach */
"0QW645Z"  =  "1"  /* Revision of Ext Fix in R Up Femur, Perc Endo Approach */
"0QW647Z"  =  "1"  /* Revision of Autol Sub in R Up Femur, Perc Endo Approach */
"0QW64JZ"  =  "1"  /* Revision of Synth Sub in R Up Femur, Perc Endo Approach */
"0QW64KZ"  =  "1"  /* Revision of Nonaut Sub in R Up Femur, Perc Endo Approach */
"0QW704Z"  =  "1"  /* Revision of Int Fix in L Up Femur, Open Approach */
"0QW705Z"  =  "1"  /* Revision of Ext Fix in L Up Femur, Open Approach */
"0QW707Z"  =  "1"  /* Revision of Autol Sub in L Up Femur, Open Approach */
"0QW70JZ"  =  "1"  /* Revision of Synth Sub in L Up Femur, Open Approach */
"0QW70KZ"  =  "1"  /* Revision of Nonaut Sub in L Up Femur, Open Approach */
"0QW734Z"  =  "1"  /* Revision of Int Fix in L Up Femur, Perc Approach */
"0QW735Z"  =  "1"  /* Revision of Ext Fix in L Up Femur, Perc Approach */
"0QW737Z"  =  "1"  /* Revision of Autol Sub in L Up Femur, Perc Approach */
"0QW73JZ"  =  "1"  /* Revision of Synth Sub in L Up Femur, Perc Approach */
"0QW73KZ"  =  "1"  /* Revision of Nonaut Sub in L Up Femur, Perc Approach */
"0QW744Z"  =  "1"  /* Revision of Int Fix in L Up Femur, Perc Endo Approach */
"0QW745Z"  =  "1"  /* Revision of Ext Fix in L Up Femur, Perc Endo Approach */
"0QW747Z"  =  "1"  /* Revision of Autol Sub in L Up Femur, Perc Endo Approach */
"0QW74JZ"  =  "1"  /* Revision of Synth Sub in L Up Femur, Perc Endo Approach */
"0QW74KZ"  =  "1"  /* Revision of Nonaut Sub in L Up Femur, Perc Endo Approach */
"0QW804Z"  =  "1"  /* Revision of Int Fix in R Femur Shaft, Open Approach */
"0QW805Z"  =  "1"  /* Revision of Ext Fix in R Femur Shaft, Open Approach */
"0QW807Z"  =  "1"  /* Revision of Autol Sub in R Femur Shaft, Open Approach */
"0QW80JZ"  =  "1"  /* Revision of Synth Sub in R Femur Shaft, Open Approach */
"0QW80KZ"  =  "1"  /* Revision of Nonaut Sub in R Femur Shaft, Open Approach */
"0QW834Z"  =  "1"  /* Revision of Int Fix in R Femur Shaft, Perc Approach */
"0QW835Z"  =  "1"  /* Revision of Ext Fix in R Femur Shaft, Perc Approach */
"0QW837Z"  =  "1"  /* Revision of Autol Sub in R Femur Shaft, Perc Approach */
"0QW83JZ"  =  "1"  /* Revision of Synth Sub in R Femur Shaft, Perc Approach */
"0QW83KZ"  =  "1"  /* Revision of Nonaut Sub in R Femur Shaft, Perc Approach */
"0QW844Z"  =  "1"  /* Revision of Int Fix in R Femur Shaft, Perc Endo Approach */
"0QW845Z"  =  "1"  /* Revision of Ext Fix in R Femur Shaft, Perc Endo Approach */
"0QW847Z"  =  "1"  /* Revision of Autol Sub in R Femur Shaft, Perc Endo Approach */
"0QW84JZ"  =  "1"  /* Revision of Synth Sub in R Femur Shaft, Perc Endo Approach */
"0QW84KZ"  =  "1"  /* Revision of Nonaut Sub in R Femur Shaft, Perc Endo Approach */
"0QW904Z"  =  "1"  /* Revision of Int Fix in L Femur Shaft, Open Approach */
"0QW905Z"  =  "1"  /* Revision of Ext Fix in L Femur Shaft, Open Approach */
"0QW907Z"  =  "1"  /* Revision of Autol Sub in L Femur Shaft, Open Approach */
"0QW90JZ"  =  "1"  /* Revision of Synth Sub in L Femur Shaft, Open Approach */
"0QW90KZ"  =  "1"  /* Revision of Nonaut Sub in L Femur Shaft, Open Approach */
"0QW934Z"  =  "1"  /* Revision of Int Fix in L Femur Shaft, Perc Approach */
"0QW935Z"  =  "1"  /* Revision of Ext Fix in L Femur Shaft, Perc Approach */
"0QW937Z"  =  "1"  /* Revision of Autol Sub in L Femur Shaft, Perc Approach */
"0QW93JZ"  =  "1"  /* Revision of Synth Sub in L Femur Shaft, Perc Approach */
"0QW93KZ"  =  "1"  /* Revision of Nonaut Sub in L Femur Shaft, Perc Approach */
"0QW944Z"  =  "1"  /* Revision of Int Fix in L Femur Shaft, Perc Endo Approach */
"0QW945Z"  =  "1"  /* Revision of Ext Fix in L Femur Shaft, Perc Endo Approach */
"0QW947Z"  =  "1"  /* Revision of Autol Sub in L Femur Shaft, Perc Endo Approach */
"0QW94JZ"  =  "1"  /* Revision of Synth Sub in L Femur Shaft, Perc Endo Approach */
"0QW94KZ"  =  "1"  /* Revision of Nonaut Sub in L Femur Shaft, Perc Endo Approach */
"0QWB04Z"  =  "1"  /* Revision of Int Fix in R Low Femur, Open Approach */
"0QWB05Z"  =  "1"  /* Revision of Ext Fix in R Low Femur, Open Approach */
"0QWB07Z"  =  "1"  /* Revision of Autol Sub in R Low Femur, Open Approach */
"0QWB0JZ"  =  "1"  /* Revision of Synth Sub in R Low Femur, Open Approach */
"0QWB0KZ"  =  "1"  /* Revision of Nonaut Sub in R Low Femur, Open Approach */
"0QWB34Z"  =  "1"  /* Revision of Int Fix in R Low Femur, Perc Approach */
"0QWB35Z"  =  "1"  /* Revision of Ext Fix in R Low Femur, Perc Approach */
"0QWB37Z"  =  "1"  /* Revision of Autol Sub in R Low Femur, Perc Approach */
"0QWB3JZ"  =  "1"  /* Revision of Synth Sub in R Low Femur, Perc Approach */
"0QWB3KZ"  =  "1"  /* Revision of Nonaut Sub in R Low Femur, Perc Approach */
"0QWB44Z"  =  "1"  /* Revision of Int Fix in R Low Femur, Perc Endo Approach */
"0QWB45Z"  =  "1"  /* Revision of Ext Fix in R Low Femur, Perc Endo Approach */
"0QWB47Z"  =  "1"  /* Revision of Autol Sub in R Low Femur, Perc Endo Approach */
"0QWB4JZ"  =  "1"  /* Revision of Synth Sub in R Low Femur, Perc Endo Approach */
"0QWB4KZ"  =  "1"  /* Revision of Nonaut Sub in R Low Femur, Perc Endo Approach */
"0QWC04Z"  =  "1"  /* Revision of Int Fix in L Low Femur, Open Approach */
"0QWC05Z"  =  "1"  /* Revision of Ext Fix in L Low Femur, Open Approach */
"0QWC07Z"  =  "1"  /* Revision of Autol Sub in L Low Femur, Open Approach */
"0QWC0JZ"  =  "1"  /* Revision of Synth Sub in L Low Femur, Open Approach */
"0QWC0KZ"  =  "1"  /* Revision of Nonaut Sub in L Low Femur, Open Approach */
"0QWC34Z"  =  "1"  /* Revision of Int Fix in L Low Femur, Perc Approach */
"0QWC35Z"  =  "1"  /* Revision of Ext Fix in L Low Femur, Perc Approach */
"0QWC37Z"  =  "1"  /* Revision of Autol Sub in L Low Femur, Perc Approach */
"0QWC3JZ"  =  "1"  /* Revision of Synth Sub in L Low Femur, Perc Approach */
"0QWC3KZ"  =  "1"  /* Revision of Nonaut Sub in L Low Femur, Perc Approach */
"0QWC44Z"  =  "1"  /* Revision of Int Fix in L Low Femur, Perc Endo Approach */
"0QWC45Z"  =  "1"  /* Revision of Ext Fix in L Low Femur, Perc Endo Approach */
"0QWC47Z"  =  "1"  /* Revision of Autol Sub in L Low Femur, Perc Endo Approach */
"0QWC4JZ"  =  "1"  /* Revision of Synth Sub in L Low Femur, Perc Endo Approach */
"0QWC4KZ"  =  "1"  /* Revision of Nonaut Sub in L Low Femur, Perc Endo Approach */
"0QWD04Z"  =  "1"  /* Revision of Int Fix in R Patella, Open Approach */
"0QWD05Z"  =  "1"  /* Revision of Ext Fix in R Patella, Open Approach */
"0QWD07Z"  =  "1"  /* Revision of Autol Sub in R Patella, Open Approach */
"0QWD0JZ"  =  "1"  /* Revision of Synthetic Substitute in R Patella, Open Approach */
"0QWD0KZ"  =  "1"  /* Revision of Nonaut Sub in R Patella, Open Approach */
"0QWD34Z"  =  "1"  /* Revision of Int Fix in R Patella, Perc Approach */
"0QWD35Z"  =  "1"  /* Revision of Ext Fix in R Patella, Perc Approach */
"0QWD37Z"  =  "1"  /* Revision of Autol Sub in R Patella, Perc Approach */
"0QWD3JZ"  =  "1"  /* Revision of Synthetic Substitute in R Patella, Perc Approach */
"0QWD3KZ"  =  "1"  /* Revision of Nonaut Sub in R Patella, Perc Approach */
"0QWD44Z"  =  "1"  /* Revision of Int Fix in R Patella, Perc Endo Approach */
"0QWD45Z"  =  "1"  /* Revision of Ext Fix in R Patella, Perc Endo Approach */
"0QWD47Z"  =  "1"  /* Revision of Autol Sub in R Patella, Perc Endo Approach */
"0QWD4JZ"  =  "1"  /* Revision of Synth Sub in R Patella, Perc Endo Approach */
"0QWD4KZ"  =  "1"  /* Revision of Nonaut Sub in R Patella, Perc Endo Approach */
"0QWF04Z"  =  "1"  /* Revision of Int Fix in L Patella, Open Approach */
"0QWF05Z"  =  "1"  /* Revision of Ext Fix in L Patella, Open Approach */
"0QWF07Z"  =  "1"  /* Revision of Autol Sub in L Patella, Open Approach */
"0QWF0JZ"  =  "1"  /* Revision of Synthetic Substitute in L Patella, Open Approach */
"0QWF0KZ"  =  "1"  /* Revision of Nonaut Sub in L Patella, Open Approach */
"0QWF34Z"  =  "1"  /* Revision of Int Fix in L Patella, Perc Approach */
"0QWF35Z"  =  "1"  /* Revision of Ext Fix in L Patella, Perc Approach */
"0QWF37Z"  =  "1"  /* Revision of Autol Sub in L Patella, Perc Approach */
"0QWF3JZ"  =  "1"  /* Revision of Synthetic Substitute in L Patella, Perc Approach */
"0QWF3KZ"  =  "1"  /* Revision of Nonaut Sub in L Patella, Perc Approach */
"0QWF44Z"  =  "1"  /* Revision of Int Fix in L Patella, Perc Endo Approach */
"0QWF45Z"  =  "1"  /* Revision of Ext Fix in L Patella, Perc Endo Approach */
"0QWF47Z"  =  "1"  /* Revision of Autol Sub in L Patella, Perc Endo Approach */
"0QWF4JZ"  =  "1"  /* Revision of Synth Sub in L Patella, Perc Endo Approach */
"0QWF4KZ"  =  "1"  /* Revision of Nonaut Sub in L Patella, Perc Endo Approach */
"0QWG04Z"  =  "1"  /* Revision of Int Fix in R Tibia, Open Approach */
"0QWG05Z"  =  "1"  /* Revision of Ext Fix in R Tibia, Open Approach */
"0QWG07Z"  =  "1"  /* Revision of Autol Sub in R Tibia, Open Approach */
"0QWG0JZ"  =  "1"  /* Revision of Synthetic Substitute in R Tibia, Open Approach */
"0QWG0KZ"  =  "1"  /* Revision of Nonaut Sub in R Tibia, Open Approach */
"0QWG34Z"  =  "1"  /* Revision of Int Fix in R Tibia, Perc Approach */
"0QWG35Z"  =  "1"  /* Revision of Ext Fix in R Tibia, Perc Approach */
"0QWG37Z"  =  "1"  /* Revision of Autol Sub in R Tibia, Perc Approach */
"0QWG3JZ"  =  "1"  /* Revision of Synthetic Substitute in R Tibia, Perc Approach */
"0QWG3KZ"  =  "1"  /* Revision of Nonaut Sub in R Tibia, Perc Approach */
"0QWG44Z"  =  "1"  /* Revision of Int Fix in R Tibia, Perc Endo Approach */
"0QWG45Z"  =  "1"  /* Revision of Ext Fix in R Tibia, Perc Endo Approach */
"0QWG47Z"  =  "1"  /* Revision of Autol Sub in R Tibia, Perc Endo Approach */
"0QWG4JZ"  =  "1"  /* Revision of Synth Sub in R Tibia, Perc Endo Approach */
"0QWG4KZ"  =  "1"  /* Revision of Nonaut Sub in R Tibia, Perc Endo Approach */
"0QWH04Z"  =  "1"  /* Revision of Int Fix in L Tibia, Open Approach */
"0QWH05Z"  =  "1"  /* Revision of Ext Fix in L Tibia, Open Approach */
"0QWH07Z"  =  "1"  /* Revision of Autol Sub in L Tibia, Open Approach */
"0QWH0JZ"  =  "1"  /* Revision of Synthetic Substitute in L Tibia, Open Approach */
"0QWH0KZ"  =  "1"  /* Revision of Nonaut Sub in L Tibia, Open Approach */
"0QWH34Z"  =  "1"  /* Revision of Int Fix in L Tibia, Perc Approach */
"0QWH35Z"  =  "1"  /* Revision of Ext Fix in L Tibia, Perc Approach */
"0QWH37Z"  =  "1"  /* Revision of Autol Sub in L Tibia, Perc Approach */
"0QWH3JZ"  =  "1"  /* Revision of Synthetic Substitute in L Tibia, Perc Approach */
"0QWH3KZ"  =  "1"  /* Revision of Nonaut Sub in L Tibia, Perc Approach */
"0QWH44Z"  =  "1"  /* Revision of Int Fix in L Tibia, Perc Endo Approach */
"0QWH45Z"  =  "1"  /* Revision of Ext Fix in L Tibia, Perc Endo Approach */
"0QWH47Z"  =  "1"  /* Revision of Autol Sub in L Tibia, Perc Endo Approach */
"0QWH4JZ"  =  "1"  /* Revision of Synth Sub in L Tibia, Perc Endo Approach */
"0QWH4KZ"  =  "1"  /* Revision of Nonaut Sub in L Tibia, Perc Endo Approach */
"0QWJ04Z"  =  "1"  /* Revision of Int Fix in R Fibula, Open Approach */
"0QWJ05Z"  =  "1"  /* Revision of Ext Fix in R Fibula, Open Approach */
"0QWJ07Z"  =  "1"  /* Revision of Autol Sub in R Fibula, Open Approach */
"0QWJ0JZ"  =  "1"  /* Revision of Synthetic Substitute in R Fibula, Open Approach */
"0QWJ0KZ"  =  "1"  /* Revision of Nonaut Sub in R Fibula, Open Approach */
"0QWJ34Z"  =  "1"  /* Revision of Int Fix in R Fibula, Perc Approach */
"0QWJ35Z"  =  "1"  /* Revision of Ext Fix in R Fibula, Perc Approach */
"0QWJ37Z"  =  "1"  /* Revision of Autol Sub in R Fibula, Perc Approach */
"0QWJ3JZ"  =  "1"  /* Revision of Synthetic Substitute in R Fibula, Perc Approach */
"0QWJ3KZ"  =  "1"  /* Revision of Nonaut Sub in R Fibula, Perc Approach */
"0QWJ44Z"  =  "1"  /* Revision of Int Fix in R Fibula, Perc Endo Approach */
"0QWJ45Z"  =  "1"  /* Revision of Ext Fix in R Fibula, Perc Endo Approach */
"0QWJ47Z"  =  "1"  /* Revision of Autol Sub in R Fibula, Perc Endo Approach */
"0QWJ4JZ"  =  "1"  /* Revision of Synth Sub in R Fibula, Perc Endo Approach */
"0QWJ4KZ"  =  "1"  /* Revision of Nonaut Sub in R Fibula, Perc Endo Approach */
"0QWK04Z"  =  "1"  /* Revision of Int Fix in L Fibula, Open Approach */
"0QWK05Z"  =  "1"  /* Revision of Ext Fix in L Fibula, Open Approach */
"0QWK07Z"  =  "1"  /* Revision of Autol Sub in L Fibula, Open Approach */
"0QWK0JZ"  =  "1"  /* Revision of Synthetic Substitute in L Fibula, Open Approach */
"0QWK0KZ"  =  "1"  /* Revision of Nonaut Sub in L Fibula, Open Approach */
"0QWK34Z"  =  "1"  /* Revision of Int Fix in L Fibula, Perc Approach */
"0QWK35Z"  =  "1"  /* Revision of Ext Fix in L Fibula, Perc Approach */
"0QWK37Z"  =  "1"  /* Revision of Autol Sub in L Fibula, Perc Approach */
"0QWK3JZ"  =  "1"  /* Revision of Synthetic Substitute in L Fibula, Perc Approach */
"0QWK3KZ"  =  "1"  /* Revision of Nonaut Sub in L Fibula, Perc Approach */
"0QWK44Z"  =  "1"  /* Revision of Int Fix in L Fibula, Perc Endo Approach */
"0QWK45Z"  =  "1"  /* Revision of Ext Fix in L Fibula, Perc Endo Approach */
"0QWK47Z"  =  "1"  /* Revision of Autol Sub in L Fibula, Perc Endo Approach */
"0QWK4JZ"  =  "1"  /* Revision of Synth Sub in L Fibula, Perc Endo Approach */
"0QWK4KZ"  =  "1"  /* Revision of Nonaut Sub in L Fibula, Perc Endo Approach */
"0QWL04Z"  =  "1"  /* Revision of Int Fix in R Tarsal, Open Approach */
"0QWL05Z"  =  "1"  /* Revision of Ext Fix in R Tarsal, Open Approach */
"0QWL07Z"  =  "1"  /* Revision of Autol Sub in R Tarsal, Open Approach */
"0QWL0JZ"  =  "1"  /* Revision of Synthetic Substitute in R Tarsal, Open Approach */
"0QWL0KZ"  =  "1"  /* Revision of Nonaut Sub in R Tarsal, Open Approach */
"0QWL34Z"  =  "1"  /* Revision of Int Fix in R Tarsal, Perc Approach */
"0QWL35Z"  =  "1"  /* Revision of Ext Fix in R Tarsal, Perc Approach */
"0QWL37Z"  =  "1"  /* Revision of Autol Sub in R Tarsal, Perc Approach */
"0QWL3JZ"  =  "1"  /* Revision of Synthetic Substitute in R Tarsal, Perc Approach */
"0QWL3KZ"  =  "1"  /* Revision of Nonaut Sub in R Tarsal, Perc Approach */
"0QWL44Z"  =  "1"  /* Revision of Int Fix in R Tarsal, Perc Endo Approach */
"0QWL45Z"  =  "1"  /* Revision of Ext Fix in R Tarsal, Perc Endo Approach */
"0QWL47Z"  =  "1"  /* Revision of Autol Sub in R Tarsal, Perc Endo Approach */
"0QWL4JZ"  =  "1"  /* Revision of Synth Sub in R Tarsal, Perc Endo Approach */
"0QWL4KZ"  =  "1"  /* Revision of Nonaut Sub in R Tarsal, Perc Endo Approach */
"0QWM04Z"  =  "1"  /* Revision of Int Fix in L Tarsal, Open Approach */
"0QWM05Z"  =  "1"  /* Revision of Ext Fix in L Tarsal, Open Approach */
"0QWM07Z"  =  "1"  /* Revision of Autol Sub in L Tarsal, Open Approach */
"0QWM0JZ"  =  "1"  /* Revision of Synthetic Substitute in L Tarsal, Open Approach */
"0QWM0KZ"  =  "1"  /* Revision of Nonaut Sub in L Tarsal, Open Approach */
"0QWM34Z"  =  "1"  /* Revision of Int Fix in L Tarsal, Perc Approach */
"0QWM35Z"  =  "1"  /* Revision of Ext Fix in L Tarsal, Perc Approach */
"0QWM37Z"  =  "1"  /* Revision of Autol Sub in L Tarsal, Perc Approach */
"0QWM3JZ"  =  "1"  /* Revision of Synthetic Substitute in L Tarsal, Perc Approach */
"0QWM3KZ"  =  "1"  /* Revision of Nonaut Sub in L Tarsal, Perc Approach */
"0QWM44Z"  =  "1"  /* Revision of Int Fix in L Tarsal, Perc Endo Approach */
"0QWM45Z"  =  "1"  /* Revision of Ext Fix in L Tarsal, Perc Endo Approach */
"0QWM47Z"  =  "1"  /* Revision of Autol Sub in L Tarsal, Perc Endo Approach */
"0QWM4JZ"  =  "1"  /* Revision of Synth Sub in L Tarsal, Perc Endo Approach */
"0QWM4KZ"  =  "1"  /* Revision of Nonaut Sub in L Tarsal, Perc Endo Approach */
"0QWN04Z"  =  "1"  /* Revision of Int Fix in R Metatarsal, Open Approach */
"0QWN05Z"  =  "1"  /* Revision of Ext Fix in R Metatarsal, Open Approach */
"0QWN07Z"  =  "1"  /* Revision of Autol Sub in R Metatarsal, Open Approach */
"0QWN0JZ"  =  "1"  /* Revision of Synth Sub in R Metatarsal, Open Approach */
"0QWN0KZ"  =  "1"  /* Revision of Nonaut Sub in R Metatarsal, Open Approach */
"0QWN34Z"  =  "1"  /* Revision of Int Fix in R Metatarsal, Perc Approach */
"0QWN35Z"  =  "1"  /* Revision of Ext Fix in R Metatarsal, Perc Approach */
"0QWN37Z"  =  "1"  /* Revision of Autol Sub in R Metatarsal, Perc Approach */
"0QWN3JZ"  =  "1"  /* Revision of Synth Sub in R Metatarsal, Perc Approach */
"0QWN3KZ"  =  "1"  /* Revision of Nonaut Sub in R Metatarsal, Perc Approach */
"0QWN44Z"  =  "1"  /* Revision of Int Fix in R Metatarsal, Perc Endo Approach */
"0QWN45Z"  =  "1"  /* Revision of Ext Fix in R Metatarsal, Perc Endo Approach */
"0QWN47Z"  =  "1"  /* Revision of Autol Sub in R Metatarsal, Perc Endo Approach */
"0QWN4JZ"  =  "1"  /* Revision of Synth Sub in R Metatarsal, Perc Endo Approach */
"0QWN4KZ"  =  "1"  /* Revision of Nonaut Sub in R Metatarsal, Perc Endo Approach */
"0QWP04Z"  =  "1"  /* Revision of Int Fix in L Metatarsal, Open Approach */
"0QWP05Z"  =  "1"  /* Revision of Ext Fix in L Metatarsal, Open Approach */
"0QWP07Z"  =  "1"  /* Revision of Autol Sub in L Metatarsal, Open Approach */
"0QWP0JZ"  =  "1"  /* Revision of Synth Sub in L Metatarsal, Open Approach */
"0QWP0KZ"  =  "1"  /* Revision of Nonaut Sub in L Metatarsal, Open Approach */
"0QWP34Z"  =  "1"  /* Revision of Int Fix in L Metatarsal, Perc Approach */
"0QWP35Z"  =  "1"  /* Revision of Ext Fix in L Metatarsal, Perc Approach */
"0QWP37Z"  =  "1"  /* Revision of Autol Sub in L Metatarsal, Perc Approach */
"0QWP3JZ"  =  "1"  /* Revision of Synth Sub in L Metatarsal, Perc Approach */
"0QWP3KZ"  =  "1"  /* Revision of Nonaut Sub in L Metatarsal, Perc Approach */
"0QWP44Z"  =  "1"  /* Revision of Int Fix in L Metatarsal, Perc Endo Approach */
"0QWP45Z"  =  "1"  /* Revision of Ext Fix in L Metatarsal, Perc Endo Approach */
"0QWP47Z"  =  "1"  /* Revision of Autol Sub in L Metatarsal, Perc Endo Approach */
"0QWP4JZ"  =  "1"  /* Revision of Synth Sub in L Metatarsal, Perc Endo Approach */
"0QWP4KZ"  =  "1"  /* Revision of Nonaut Sub in L Metatarsal, Perc Endo Approach */
"0QWQ04Z"  =  "1"  /* Revision of Int Fix in R Toe Phalanx, Open Approach */
"0QWQ05Z"  =  "1"  /* Revision of Ext Fix in R Toe Phalanx, Open Approach */
"0QWQ07Z"  =  "1"  /* Revision of Autol Sub in R Toe Phalanx, Open Approach */
"0QWQ0JZ"  =  "1"  /* Revision of Synth Sub in R Toe Phalanx, Open Approach */
"0QWQ0KZ"  =  "1"  /* Revision of Nonaut Sub in R Toe Phalanx, Open Approach */
"0QWQ34Z"  =  "1"  /* Revision of Int Fix in R Toe Phalanx, Perc Approach */
"0QWQ35Z"  =  "1"  /* Revision of Ext Fix in R Toe Phalanx, Perc Approach */
"0QWQ37Z"  =  "1"  /* Revision of Autol Sub in R Toe Phalanx, Perc Approach */
"0QWQ3JZ"  =  "1"  /* Revision of Synth Sub in R Toe Phalanx, Perc Approach */
"0QWQ3KZ"  =  "1"  /* Revision of Nonaut Sub in R Toe Phalanx, Perc Approach */
"0QWQ44Z"  =  "1"  /* Revision of Int Fix in R Toe Phalanx, Perc Endo Approach */
"0QWQ45Z"  =  "1"  /* Revision of Ext Fix in R Toe Phalanx, Perc Endo Approach */
"0QWQ47Z"  =  "1"  /* Revision of Autol Sub in R Toe Phalanx, Perc Endo Approach */
"0QWQ4JZ"  =  "1"  /* Revision of Synth Sub in R Toe Phalanx, Perc Endo Approach */
"0QWQ4KZ"  =  "1"  /* Revision of Nonaut Sub in R Toe Phalanx, Perc Endo Approach */
"0QWR04Z"  =  "1"  /* Revision of Int Fix in L Toe Phalanx, Open Approach */
"0QWR05Z"  =  "1"  /* Revision of Ext Fix in L Toe Phalanx, Open Approach */
"0QWR07Z"  =  "1"  /* Revision of Autol Sub in L Toe Phalanx, Open Approach */
"0QWR0JZ"  =  "1"  /* Revision of Synth Sub in L Toe Phalanx, Open Approach */
"0QWR0KZ"  =  "1"  /* Revision of Nonaut Sub in L Toe Phalanx, Open Approach */
"0QWR34Z"  =  "1"  /* Revision of Int Fix in L Toe Phalanx, Perc Approach */
"0QWR35Z"  =  "1"  /* Revision of Ext Fix in L Toe Phalanx, Perc Approach */
"0QWR37Z"  =  "1"  /* Revision of Autol Sub in L Toe Phalanx, Perc Approach */
"0QWR3JZ"  =  "1"  /* Revision of Synth Sub in L Toe Phalanx, Perc Approach */
"0QWR3KZ"  =  "1"  /* Revision of Nonaut Sub in L Toe Phalanx, Perc Approach */
"0QWR44Z"  =  "1"  /* Revision of Int Fix in L Toe Phalanx, Perc Endo Approach */
"0QWR45Z"  =  "1"  /* Revision of Ext Fix in L Toe Phalanx, Perc Endo Approach */
"0QWR47Z"  =  "1"  /* Revision of Autol Sub in L Toe Phalanx, Perc Endo Approach */
"0QWR4JZ"  =  "1"  /* Revision of Synth Sub in L Toe Phalanx, Perc Endo Approach */
"0QWR4KZ"  =  "1"  /* Revision of Nonaut Sub in L Toe Phalanx, Perc Endo Approach */
"0QWS04Z"  =  "1"  /* Revision of Int Fix in Coccyx, Open Approach */
"0QWS07Z"  =  "1"  /* Revision of Autol Sub in Coccyx, Open Approach */
"0QWS0JZ"  =  "1"  /* Revision of Synthetic Substitute in Coccyx, Open Approach */
"0QWS0KZ"  =  "1"  /* Revision of Nonaut Sub in Coccyx, Open Approach */
"0QWS34Z"  =  "1"  /* Revision of Int Fix in Coccyx, Perc Approach */
"0QWS37Z"  =  "1"  /* Revision of Autol Sub in Coccyx, Perc Approach */
"0QWS3JZ"  =  "1"  /* Revision of Synthetic Substitute in Coccyx, Perc Approach */
"0QWS3KZ"  =  "1"  /* Revision of Nonaut Sub in Coccyx, Perc Approach */
"0QWS44Z"  =  "1"  /* Revision of Int Fix in Coccyx, Perc Endo Approach */
"0QWS47Z"  =  "1"  /* Revision of Autol Sub in Coccyx, Perc Endo Approach */
"0QWS4JZ"  =  "1"  /* Revision of Synth Sub in Coccyx, Perc Endo Approach */
"0QWS4KZ"  =  "1"  /* Revision of Nonaut Sub in Coccyx, Perc Endo Approach */
"0QWY00Z"  =  "1"  /* Revision of Drainage Device in Lower Bone, Open Approach */
"0QWY0MZ"  =  "1"  /* Revision of Bone Stim in Low Bone, Open Approach */
"0QWY30Z"  =  "1"  /* Revision of Drainage Device in Lower Bone, Perc Approach */
"0QWY3MZ"  =  "1"  /* Revision of Bone Stim in Low Bone, Perc Approach */
"0QWY40Z"  =  "1"  /* Revision of Drainage Device in Low Bone, Perc Endo Approach */
"0QWY4MZ"  =  "1"  /* Revision of Bone Stim in Low Bone, Perc Endo Approach */
"0R500ZZ"  =  "1"  /* Destruction of Occipital-cervical Joint, Open Approach */
"0R503ZZ"  =  "1"  /* Destruction of Occipital-cervical Joint, Perc Approach */
"0R504ZZ"  =  "1"  /* Destruction of Occipital-cervical Joint, Perc Endo Approach */
"0R510ZZ"  =  "1"  /* Destruction of Cervical Vertebral Joint, Open Approach */
"0R513ZZ"  =  "1"  /* Destruction of Cervical Vertebral Joint, Perc Approach */
"0R514ZZ"  =  "1"  /* Destruction of Cervical Vertebral Joint, Perc Endo Approach */
"0R530ZZ"  =  "1"  /* Destruction of Cervical Vertebral Disc, Open Approach */
"0R540ZZ"  =  "1"  /* Destruction of C-thor Jt, Open Approach */
"0R543ZZ"  =  "1"  /* Destruction of C-thor Jt, Perc Approach */
"0R544ZZ"  =  "1"  /* Destruction of C-thor Jt, Perc Endo Approach */
"0R550ZZ"  =  "1"  /* Destruction of Cervicothoracic Vertebral Disc, Open Approach */
"0R560ZZ"  =  "1"  /* Destruction of Thoracic Vertebral Joint, Open Approach */
"0R563ZZ"  =  "1"  /* Destruction of Thoracic Vertebral Joint, Perc Approach */
"0R564ZZ"  =  "1"  /* Destruction of Thoracic Vertebral Joint, Perc Endo Approach */
"0R590ZZ"  =  "1"  /* Destruction of Thoracic Vertebral Disc, Open Approach */
"0R5A0ZZ"  =  "1"  /* Destruction of Thoracolumbar Vertebral Joint, Open Approach */
"0R5A3ZZ"  =  "1"  /* Destruction of Thoracolumbar Vertebral Joint, Perc Approach */
"0R5A4ZZ"  =  "1"  /* Destruction of T-lum Jt, Perc Endo Approach */
"0R5B0ZZ"  =  "1"  /* Destruction of Thoracolumbar Vertebral Disc, Open Approach */
"0R5C0ZZ"  =  "1"  /* Destruction of Right Temporomandibular Joint, Open Approach */
"0R5C3ZZ"  =  "1"  /* Destruction of Right Temporomandibular Joint, Perc Approach */
"0R5C4ZZ"  =  "1"  /* Destruction of R Temporomandib Jt, Perc Endo Approach */
"0R5D0ZZ"  =  "1"  /* Destruction of Left Temporomandibular Joint, Open Approach */
"0R5D3ZZ"  =  "1"  /* Destruction of Left Temporomandibular Joint, Perc Approach */
"0R5D4ZZ"  =  "1"  /* Destruction of L Temporomandib Jt, Perc Endo Approach */
"0R5E0ZZ"  =  "1"  /* Destruction of Right Sternoclavicular Joint, Open Approach */
"0R5E3ZZ"  =  "1"  /* Destruction of Right Sternoclavicular Joint, Perc Approach */
"0R5E4ZZ"  =  "1"  /* Destruction of R Sternoclav Jt, Perc Endo Approach */
"0R5F0ZZ"  =  "1"  /* Destruction of Left Sternoclavicular Joint, Open Approach */
"0R5F3ZZ"  =  "1"  /* Destruction of Left Sternoclavicular Joint, Perc Approach */
"0R5F4ZZ"  =  "1"  /* Destruction of L Sternoclav Jt, Perc Endo Approach */
"0R5G0ZZ"  =  "1"  /* Destruction of Right Acromioclavicular Joint, Open Approach */
"0R5G3ZZ"  =  "1"  /* Destruction of Right Acromioclavicular Joint, Perc Approach */
"0R5G4ZZ"  =  "1"  /* Destruction of R Acromioclav Jt, Perc Endo Approach */
"0R5H0ZZ"  =  "1"  /* Destruction of Left Acromioclavicular Joint, Open Approach */
"0R5H3ZZ"  =  "1"  /* Destruction of Left Acromioclavicular Joint, Perc Approach */
"0R5H4ZZ"  =  "1"  /* Destruction of L Acromioclav Jt, Perc Endo Approach */
"0R5J0ZZ"  =  "1"  /* Destruction of Right Shoulder Joint, Open Approach */
"0R5J3ZZ"  =  "1"  /* Destruction of Right Shoulder Joint, Percutaneous Approach */
"0R5J4ZZ"  =  "1"  /* Destruction of Right Shoulder Joint, Perc Endo Approach */
"0R5K0ZZ"  =  "1"  /* Destruction of Left Shoulder Joint, Open Approach */
"0R5K3ZZ"  =  "1"  /* Destruction of Left Shoulder Joint, Percutaneous Approach */
"0R5K4ZZ"  =  "1"  /* Destruction of Left Shoulder Joint, Perc Endo Approach */
"0R5L0ZZ"  =  "1"  /* Destruction of Right Elbow Joint, Open Approach */
"0R5L3ZZ"  =  "1"  /* Destruction of Right Elbow Joint, Percutaneous Approach */
"0R5L4ZZ"  =  "1"  /* Destruction of Right Elbow Joint, Perc Endo Approach */
"0R5M0ZZ"  =  "1"  /* Destruction of Left Elbow Joint, Open Approach */
"0R5M3ZZ"  =  "1"  /* Destruction of Left Elbow Joint, Percutaneous Approach */
"0R5M4ZZ"  =  "1"  /* Destruction of Left Elbow Joint, Perc Endo Approach */
"0R5N0ZZ"  =  "1"  /* Destruction of Right Wrist Joint, Open Approach */
"0R5N3ZZ"  =  "1"  /* Destruction of Right Wrist Joint, Percutaneous Approach */
"0R5N4ZZ"  =  "1"  /* Destruction of Right Wrist Joint, Perc Endo Approach */
"0R5P0ZZ"  =  "1"  /* Destruction of Left Wrist Joint, Open Approach */
"0R5P3ZZ"  =  "1"  /* Destruction of Left Wrist Joint, Percutaneous Approach */
"0R5P4ZZ"  =  "1"  /* Destruction of Left Wrist Joint, Perc Endo Approach */
"0R5Q0ZZ"  =  "1"  /* Destruction of Right Carpal Joint, Open Approach */
"0R5Q3ZZ"  =  "1"  /* Destruction of Right Carpal Joint, Percutaneous Approach */
"0R5Q4ZZ"  =  "1"  /* Destruction of Right Carpal Joint, Perc Endo Approach */
"0R5R0ZZ"  =  "1"  /* Destruction of Left Carpal Joint, Open Approach */
"0R5R3ZZ"  =  "1"  /* Destruction of Left Carpal Joint, Percutaneous Approach */
"0R5R4ZZ"  =  "1"  /* Destruction of Left Carpal Joint, Perc Endo Approach */
"0R5S0ZZ"  =  "1"  /* Destruction of Right Metacarpocarpal Joint, Open Approach */
"0R5S3ZZ"  =  "1"  /* Destruction of Right Metacarpocarpal Joint, Perc Approach */
"0R5S4ZZ"  =  "1"  /* Destruction of R Metacarpocarp Jt, Perc Endo Approach */
"0R5T0ZZ"  =  "1"  /* Destruction of Left Metacarpocarpal Joint, Open Approach */
"0R5T3ZZ"  =  "1"  /* Destruction of Left Metacarpocarpal Joint, Perc Approach */
"0R5T4ZZ"  =  "1"  /* Destruction of L Metacarpocarp Jt, Perc Endo Approach */
"0R5U0ZZ"  =  "1"  /* Destruction of R Metacarpophal Jt, Open Approach */
"0R5U3ZZ"  =  "1"  /* Destruction of R Metacarpophal Jt, Perc Approach */
"0R5U4ZZ"  =  "1"  /* Destruction of R Metacarpophal Jt, Perc Endo Approach */
"0R5V0ZZ"  =  "1"  /* Destruction of Left Metacarpophalangeal Joint, Open Approach */
"0R5V3ZZ"  =  "1"  /* Destruction of Left Metacarpophalangeal Joint, Perc Approach */
"0R5V4ZZ"  =  "1"  /* Destruction of L Metacarpophal Jt, Perc Endo Approach */
"0R5W0ZZ"  =  "1"  /* Destruction of Right Finger Phalangeal Joint, Open Approach */
"0R5W3ZZ"  =  "1"  /* Destruction of Right Finger Phalangeal Joint, Perc Approach */
"0R5W4ZZ"  =  "1"  /* Destruction of R Finger Phalanx Jt, Perc Endo Approach */
"0R5X0ZZ"  =  "1"  /* Destruction of Left Finger Phalangeal Joint, Open Approach */
"0R5X3ZZ"  =  "1"  /* Destruction of Left Finger Phalangeal Joint, Perc Approach */
"0R5X4ZZ"  =  "1"  /* Destruction of L Finger Phalanx Jt, Perc Endo Approach */
"0R9000Z"  =  "1"  /* Drainage of Occip Jt with Drain Dev, Open Approach */
"0R900ZZ"  =  "1"  /* Drainage of Occipital-cervical Joint, Open Approach */
"0R9100Z"  =  "1"  /* Drainage of Cerv Jt with Drain Dev, Open Approach */
"0R910ZZ"  =  "1"  /* Drainage of Cervical Vertebral Joint, Open Approach */
"0R9300Z"  =  "1"  /* Drainage of Cerv Disc with Drain Dev, Open Approach */
"0R930ZZ"  =  "1"  /* Drainage of Cervical Vertebral Disc, Open Approach */
"0R9400Z"  =  "1"  /* Drainage of C-thor Jt with Drain Dev, Open Approach */
"0R940ZZ"  =  "1"  /* Drainage of Cervicothoracic Vertebral Joint, Open Approach */
"0R9500Z"  =  "1"  /* Drainage of C-thor Disc with Drain Dev, Open Approach */
"0R950ZZ"  =  "1"  /* Drainage of Cervicothoracic Vertebral Disc, Open Approach */
"0R9600Z"  =  "1"  /* Drainage of Thor Jt with Drain Dev, Open Approach */
"0R960ZZ"  =  "1"  /* Drainage of Thoracic Vertebral Joint, Open Approach */
"0R9900Z"  =  "1"  /* Drainage of Thor Disc with Drain Dev, Open Approach */
"0R990ZZ"  =  "1"  /* Drainage of Thoracic Vertebral Disc, Open Approach */
"0R9A00Z"  =  "1"  /* Drainage of T-lum Jt with Drain Dev, Open Approach */
"0R9A0ZZ"  =  "1"  /* Drainage of Thoracolumbar Vertebral Joint, Open Approach */
"0R9B00Z"  =  "1"  /* Drainage of T-lum Disc with Drain Dev, Open Approach */
"0R9B0ZZ"  =  "1"  /* Drainage of Thoracolumbar Vertebral Disc, Open Approach */
"0R9C00Z"  =  "1"  /* Drainage of R Temporomandib Jt with Drain Dev, Open Approach */
"0R9C0ZX"  =  "1"  /* Drainage of R Temporomandib Jt, Open Approach, Diagn */
"0R9C0ZZ"  =  "1"  /* Drainage of Right Temporomandibular Joint, Open Approach */
"0R9C30Z"  =  "1"  /* Drainage of R Temporomandib Jt with Drain Dev, Perc Approach */
"0R9C3ZX"  =  "1"  /* Drainage of R Temporomandib Jt, Perc Approach, Diagn */
"0R9C3ZZ"  =  "1"  /* Drainage of Right Temporomandibular Joint, Perc Approach */
"0R9C40Z"  =  "1"  /* Drain R Temporomandib Jt w Drain Dev, Perc Endo */
"0R9C4ZX"  =  "1"  /* Drainage of R Temporomandib Jt, Perc Endo Approach, Diagn */
"0R9C4ZZ"  =  "1"  /* Drainage of R Temporomandib Jt, Perc Endo Approach */
"0R9D00Z"  =  "1"  /* Drainage of L Temporomandib Jt with Drain Dev, Open Approach */
"0R9D0ZX"  =  "1"  /* Drainage of L Temporomandib Jt, Open Approach, Diagn */
"0R9D0ZZ"  =  "1"  /* Drainage of Left Temporomandibular Joint, Open Approach */
"0R9D30Z"  =  "1"  /* Drainage of L Temporomandib Jt with Drain Dev, Perc Approach */
"0R9D3ZX"  =  "1"  /* Drainage of L Temporomandib Jt, Perc Approach, Diagn */
"0R9D3ZZ"  =  "1"  /* Drainage of Left Temporomandibular Joint, Perc Approach */
"0R9D40Z"  =  "1"  /* Drain L Temporomandib Jt w Drain Dev, Perc Endo */
"0R9D4ZX"  =  "1"  /* Drainage of L Temporomandib Jt, Perc Endo Approach, Diagn */
"0R9D4ZZ"  =  "1"  /* Drainage of Left Temporomandibular Joint, Perc Endo Approach */
"0R9E00Z"  =  "1"  /* Drainage of R Sternoclav Jt with Drain Dev, Open Approach */
"0R9E0ZZ"  =  "1"  /* Drainage of Right Sternoclavicular Joint, Open Approach */
"0R9F00Z"  =  "1"  /* Drainage of L Sternoclav Jt with Drain Dev, Open Approach */
"0R9F0ZZ"  =  "1"  /* Drainage of Left Sternoclavicular Joint, Open Approach */
"0R9G00Z"  =  "1"  /* Drainage of R Acromioclav Jt with Drain Dev, Open Approach */
"0R9G0ZZ"  =  "1"  /* Drainage of Right Acromioclavicular Joint, Open Approach */
"0R9H00Z"  =  "1"  /* Drainage of L Acromioclav Jt with Drain Dev, Open Approach */
"0R9H0ZZ"  =  "1"  /* Drainage of Left Acromioclavicular Joint, Open Approach */
"0R9J00Z"  =  "1"  /* Drainage of R Shoulder Jt with Drain Dev, Open Approach */
"0R9J0ZZ"  =  "1"  /* Drainage of Right Shoulder Joint, Open Approach */
"0R9K00Z"  =  "1"  /* Drainage of L Shoulder Jt with Drain Dev, Open Approach */
"0R9K0ZZ"  =  "1"  /* Drainage of Left Shoulder Joint, Open Approach */
"0R9L00Z"  =  "1"  /* Drainage of Right Elbow Joint with Drain Dev, Open Approach */
"0R9L0ZZ"  =  "1"  /* Drainage of Right Elbow Joint, Open Approach */
"0R9M00Z"  =  "1"  /* Drainage of Left Elbow Joint with Drain Dev, Open Approach */
"0R9M0ZZ"  =  "1"  /* Drainage of Left Elbow Joint, Open Approach */
"0R9N00Z"  =  "1"  /* Drainage of Right Wrist Joint with Drain Dev, Open Approach */
"0R9N0ZZ"  =  "1"  /* Drainage of Right Wrist Joint, Open Approach */
"0R9P00Z"  =  "1"  /* Drainage of Left Wrist Joint with Drain Dev, Open Approach */
"0R9P0ZZ"  =  "1"  /* Drainage of Left Wrist Joint, Open Approach */
"0R9Q00Z"  =  "1"  /* Drainage of Right Carpal Joint with Drain Dev, Open Approach */
"0R9Q0ZZ"  =  "1"  /* Drainage of Right Carpal Joint, Open Approach */
"0R9R00Z"  =  "1"  /* Drainage of Left Carpal Joint with Drain Dev, Open Approach */
"0R9R0ZZ"  =  "1"  /* Drainage of Left Carpal Joint, Open Approach */
"0R9S00Z"  =  "1"  /* Drainage of R Metacarpocarp Jt with Drain Dev, Open Approach */
"0R9S0ZZ"  =  "1"  /* Drainage of Right Metacarpocarpal Joint, Open Approach */
"0R9T00Z"  =  "1"  /* Drainage of L Metacarpocarp Jt with Drain Dev, Open Approach */
"0R9T0ZZ"  =  "1"  /* Drainage of Left Metacarpocarpal Joint, Open Approach */
"0R9U00Z"  =  "1"  /* Drainage of R Metacarpophal Jt with Drain Dev, Open Approach */
"0R9U0ZZ"  =  "1"  /* Drainage of Right Metacarpophalangeal Joint, Open Approach */
"0R9V00Z"  =  "1"  /* Drainage of L Metacarpophal Jt with Drain Dev, Open Approach */
"0R9V0ZZ"  =  "1"  /* Drainage of Left Metacarpophalangeal Joint, Open Approach */
"0R9W00Z"  =  "1"  /* Drain of R Finger Phalanx Jt with Drain Dev, Open Approach */
"0R9W0ZZ"  =  "1"  /* Drainage of Right Finger Phalangeal Joint, Open Approach */
"0R9X00Z"  =  "1"  /* Drain of L Finger Phalanx Jt with Drain Dev, Open Approach */
"0R9X0ZZ"  =  "1"  /* Drainage of Left Finger Phalangeal Joint, Open Approach */
"0RB00ZZ"  =  "1"  /* Excision of Occipital-cervical Joint, Open Approach */
"0RB03ZZ"  =  "1"  /* Excision of Occipital-cervical Joint, Percutaneous Approach */
"0RB04ZZ"  =  "1"  /* Excision of Occipital-cervical Joint, Perc Endo Approach */
"0RB10ZZ"  =  "1"  /* Excision of Cervical Vertebral Joint, Open Approach */
"0RB13ZZ"  =  "1"  /* Excision of Cervical Vertebral Joint, Percutaneous Approach */
"0RB14ZZ"  =  "1"  /* Excision of Cervical Vertebral Joint, Perc Endo Approach */
"0RB30ZZ"  =  "1"  /* Excision of Cervical Vertebral Disc, Open Approach */
"0RB33ZZ"  =  "1"  /* Excision of Cervical Vertebral Disc, Percutaneous Approach */
"0RB34ZZ"  =  "1"  /* Excision of Cervical Vertebral Disc, Perc Endo Approach */
"0RB40ZZ"  =  "1"  /* Excision of Cervicothoracic Vertebral Joint, Open Approach */
"0RB43ZZ"  =  "1"  /* Excision of Cervicothoracic Vertebral Joint, Perc Approach */
"0RB44ZZ"  =  "1"  /* Excision of C-thor Jt, Perc Endo Approach */
"0RB50ZZ"  =  "1"  /* Excision of Cervicothoracic Vertebral Disc, Open Approach */
"0RB53ZZ"  =  "1"  /* Excision of Cervicothoracic Vertebral Disc, Perc Approach */
"0RB54ZZ"  =  "1"  /* Excision of C-thor Disc, Perc Endo Approach */
"0RB60ZZ"  =  "1"  /* Excision of Thoracic Vertebral Joint, Open Approach */
"0RB63ZZ"  =  "1"  /* Excision of Thoracic Vertebral Joint, Percutaneous Approach */
"0RB64ZZ"  =  "1"  /* Excision of Thoracic Vertebral Joint, Perc Endo Approach */
"0RB90ZZ"  =  "1"  /* Excision of Thoracic Vertebral Disc, Open Approach */
"0RB93ZZ"  =  "1"  /* Excision of Thoracic Vertebral Disc, Percutaneous Approach */
"0RB94ZZ"  =  "1"  /* Excision of Thoracic Vertebral Disc, Perc Endo Approach */
"0RBA0ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Joint, Open Approach */
"0RBA3ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Joint, Perc Approach */
"0RBA4ZZ"  =  "1"  /* Excision of T-lum Jt, Perc Endo Approach */
"0RBB0ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Disc, Open Approach */
"0RBB3ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Disc, Perc Approach */
"0RBB4ZZ"  =  "1"  /* Excision of Thoracolumbar Vertebral Disc, Perc Endo Approach */
"0RBC0ZX"  =  "1"  /* Excision of R Temporomandib Jt, Open Approach, Diagn */
"0RBC0ZZ"  =  "1"  /* Excision of Right Temporomandibular Joint, Open Approach */
"0RBC3ZX"  =  "1"  /* Excision of R Temporomandib Jt, Perc Approach, Diagn */
"0RBC3ZZ"  =  "1"  /* Excision of Right Temporomandibular Joint, Perc Approach */
"0RBC4ZX"  =  "1"  /* Excision of R Temporomandib Jt, Perc Endo Approach, Diagn */
"0RBC4ZZ"  =  "1"  /* Excision of R Temporomandib Jt, Perc Endo Approach */
"0RBD0ZX"  =  "1"  /* Excision of L Temporomandib Jt, Open Approach, Diagn */
"0RBD0ZZ"  =  "1"  /* Excision of Left Temporomandibular Joint, Open Approach */
"0RBD3ZX"  =  "1"  /* Excision of L Temporomandib Jt, Perc Approach, Diagn */
"0RBD3ZZ"  =  "1"  /* Excision of Left Temporomandibular Joint, Perc Approach */
"0RBD4ZX"  =  "1"  /* Excision of L Temporomandib Jt, Perc Endo Approach, Diagn */
"0RBD4ZZ"  =  "1"  /* Excision of Left Temporomandibular Joint, Perc Endo Approach */
"0RBE0ZZ"  =  "1"  /* Excision of Right Sternoclavicular Joint, Open Approach */
"0RBE3ZZ"  =  "1"  /* Excision of Right Sternoclavicular Joint, Perc Approach */
"0RBE4ZZ"  =  "1"  /* Excision of Right Sternoclavicular Joint, Perc Endo Approach */
"0RBF0ZZ"  =  "1"  /* Excision of Left Sternoclavicular Joint, Open Approach */
"0RBF3ZZ"  =  "1"  /* Excision of Left Sternoclavicular Joint, Perc Approach */
"0RBF4ZZ"  =  "1"  /* Excision of Left Sternoclavicular Joint, Perc Endo Approach */
"0RBG0ZZ"  =  "1"  /* Excision of Right Acromioclavicular Joint, Open Approach */
"0RBG3ZZ"  =  "1"  /* Excision of Right Acromioclavicular Joint, Perc Approach */
"0RBG4ZZ"  =  "1"  /* Excision of R Acromioclav Jt, Perc Endo Approach */
"0RBH0ZZ"  =  "1"  /* Excision of Left Acromioclavicular Joint, Open Approach */
"0RBH3ZZ"  =  "1"  /* Excision of Left Acromioclavicular Joint, Perc Approach */
"0RBH4ZZ"  =  "1"  /* Excision of Left Acromioclavicular Joint, Perc Endo Approach */
"0RBJ0ZZ"  =  "1"  /* Excision of Right Shoulder Joint, Open Approach */
"0RBJ3ZZ"  =  "1"  /* Excision of Right Shoulder Joint, Percutaneous Approach */
"0RBJ4ZZ"  =  "1"  /* Excision of Right Shoulder Joint, Perc Endo Approach */
"0RBK0ZZ"  =  "1"  /* Excision of Left Shoulder Joint, Open Approach */
"0RBK3ZZ"  =  "1"  /* Excision of Left Shoulder Joint, Percutaneous Approach */
"0RBK4ZZ"  =  "1"  /* Excision of Left Shoulder Joint, Perc Endo Approach */
"0RBL0ZZ"  =  "1"  /* Excision of Right Elbow Joint, Open Approach */
"0RBL3ZZ"  =  "1"  /* Excision of Right Elbow Joint, Percutaneous Approach */
"0RBL4ZZ"  =  "1"  /* Excision of Right Elbow Joint, Perc Endo Approach */
"0RBM0ZZ"  =  "1"  /* Excision of Left Elbow Joint, Open Approach */
"0RBM3ZZ"  =  "1"  /* Excision of Left Elbow Joint, Percutaneous Approach */
"0RBM4ZZ"  =  "1"  /* Excision of Left Elbow Joint, Perc Endo Approach */
"0RBN0ZZ"  =  "1"  /* Excision of Right Wrist Joint, Open Approach */
"0RBN3ZZ"  =  "1"  /* Excision of Right Wrist Joint, Percutaneous Approach */
"0RBN4ZZ"  =  "1"  /* Excision of Right Wrist Joint, Perc Endo Approach */
"0RBP0ZZ"  =  "1"  /* Excision of Left Wrist Joint, Open Approach */
"0RBP3ZZ"  =  "1"  /* Excision of Left Wrist Joint, Percutaneous Approach */
"0RBP4ZZ"  =  "1"  /* Excision of Left Wrist Joint, Perc Endo Approach */
"0RBQ0ZZ"  =  "1"  /* Excision of Right Carpal Joint, Open Approach */
"0RBQ3ZZ"  =  "1"  /* Excision of Right Carpal Joint, Percutaneous Approach */
"0RBQ4ZZ"  =  "1"  /* Excision of Right Carpal Joint, Perc Endo Approach */
"0RBR0ZZ"  =  "1"  /* Excision of Left Carpal Joint, Open Approach */
"0RBR3ZZ"  =  "1"  /* Excision of Left Carpal Joint, Percutaneous Approach */
"0RBR4ZZ"  =  "1"  /* Excision of Left Carpal Joint, Perc Endo Approach */
"0RBS0ZZ"  =  "1"  /* Excision of Right Metacarpocarpal Joint, Open Approach */
"0RBS3ZZ"  =  "1"  /* Excision of Right Metacarpocarpal Joint, Perc Approach */
"0RBS4ZZ"  =  "1"  /* Excision of Right Metacarpocarpal Joint, Perc Endo Approach */
"0RBT0ZZ"  =  "1"  /* Excision of Left Metacarpocarpal Joint, Open Approach */
"0RBT3ZZ"  =  "1"  /* Excision of Left Metacarpocarpal Joint, Perc Approach */
"0RBT4ZZ"  =  "1"  /* Excision of Left Metacarpocarpal Joint, Perc Endo Approach */
"0RBU0ZZ"  =  "1"  /* Excision of Right Metacarpophalangeal Joint, Open Approach */
"0RBU3ZZ"  =  "1"  /* Excision of Right Metacarpophalangeal Joint, Perc Approach */
"0RBU4ZZ"  =  "1"  /* Excision of R Metacarpophal Jt, Perc Endo Approach */
"0RBV0ZZ"  =  "1"  /* Excision of Left Metacarpophalangeal Joint, Open Approach */
"0RBV3ZZ"  =  "1"  /* Excision of Left Metacarpophalangeal Joint, Perc Approach */
"0RBV4ZZ"  =  "1"  /* Excision of L Metacarpophal Jt, Perc Endo Approach */
"0RBW0ZZ"  =  "1"  /* Excision of Right Finger Phalangeal Joint, Open Approach */
"0RBW3ZZ"  =  "1"  /* Excision of Right Finger Phalangeal Joint, Perc Approach */
"0RBW4ZZ"  =  "1"  /* Excision of R Finger Phalanx Jt, Perc Endo Approach */
"0RBX0ZZ"  =  "1"  /* Excision of Left Finger Phalangeal Joint, Open Approach */
"0RBX3ZZ"  =  "1"  /* Excision of Left Finger Phalangeal Joint, Perc Approach */
"0RBX4ZZ"  =  "1"  /* Excision of Left Finger Phalangeal Joint, Perc Endo Approach */
"0RC00ZZ"  =  "1"  /* Extirpation of Matter from Occip Jt, Open Approach */
"0RC03ZZ"  =  "1"  /* Extirpation of Matter from Occip Jt, Perc Approach */
"0RC04ZZ"  =  "1"  /* Extirpation of Matter from Occip Jt, Perc Endo Approach */
"0RC10ZZ"  =  "1"  /* Extirpation of Matter from Cerv Jt, Open Approach */
"0RC13ZZ"  =  "1"  /* Extirpation of Matter from Cerv Jt, Perc Approach */
"0RC14ZZ"  =  "1"  /* Extirpation of Matter from Cerv Jt, Perc Endo Approach */
"0RC30ZZ"  =  "1"  /* Extirpation of Matter from Cerv Disc, Open Approach */
"0RC33ZZ"  =  "1"  /* Extirpation of Matter from Cerv Disc, Perc Approach */
"0RC34ZZ"  =  "1"  /* Extirpation of Matter from Cerv Disc, Perc Endo Approach */
"0RC40ZZ"  =  "1"  /* Extirpation of Matter from C-thor Jt, Open Approach */
"0RC43ZZ"  =  "1"  /* Extirpation of Matter from C-thor Jt, Perc Approach */
"0RC44ZZ"  =  "1"  /* Extirpation of Matter from C-thor Jt, Perc Endo Approach */
"0RC50ZZ"  =  "1"  /* Extirpation of Matter from C-thor Disc, Open Approach */
"0RC53ZZ"  =  "1"  /* Extirpation of Matter from C-thor Disc, Perc Approach */
"0RC54ZZ"  =  "1"  /* Extirpation of Matter from C-thor Disc, Perc Endo Approach */
"0RC60ZZ"  =  "1"  /* Extirpation of Matter from Thor Jt, Open Approach */
"0RC63ZZ"  =  "1"  /* Extirpation of Matter from Thor Jt, Perc Approach */
"0RC64ZZ"  =  "1"  /* Extirpation of Matter from Thor Jt, Perc Endo Approach */
"0RC90ZZ"  =  "1"  /* Extirpation of Matter from Thor Disc, Open Approach */
"0RC93ZZ"  =  "1"  /* Extirpation of Matter from Thor Disc, Perc Approach */
"0RC94ZZ"  =  "1"  /* Extirpation of Matter from Thor Disc, Perc Endo Approach */
"0RCA0ZZ"  =  "1"  /* Extirpation of Matter from T-lum Jt, Open Approach */
"0RCA3ZZ"  =  "1"  /* Extirpation of Matter from T-lum Jt, Perc Approach */
"0RCA4ZZ"  =  "1"  /* Extirpation of Matter from T-lum Jt, Perc Endo Approach */
"0RCB0ZZ"  =  "1"  /* Extirpation of Matter from T-lum Disc, Open Approach */
"0RCB3ZZ"  =  "1"  /* Extirpation of Matter from T-lum Disc, Perc Approach */
"0RCB4ZZ"  =  "1"  /* Extirpation of Matter from T-lum Disc, Perc Endo Approach */
"0RCC0ZZ"  =  "1"  /* Extirpation of Matter from R Temporomandib Jt, Open Approach */
"0RCC3ZZ"  =  "1"  /* Extirpation of Matter from R Temporomandib Jt, Perc Approach */
"0RCC4ZZ"  =  "1"  /* Extirpate matter from R Temporomandib Jt, Perc Endo */
"0RCD0ZZ"  =  "1"  /* Extirpation of Matter from L Temporomandib Jt, Open Approach */
"0RCD3ZZ"  =  "1"  /* Extirpation of Matter from L Temporomandib Jt, Perc Approach */
"0RCD4ZZ"  =  "1"  /* Extirpate matter from L Temporomandib Jt, Perc Endo */
"0RCE0ZZ"  =  "1"  /* Extirpation of Matter from R Sternoclav Jt, Open Approach */
"0RCE3ZZ"  =  "1"  /* Extirpation of Matter from R Sternoclav Jt, Perc Approach */
"0RCE4ZZ"  =  "1"  /* Extirpate of Matter from R Sternoclav Jt, Perc Endo Approach */
"0RCF0ZZ"  =  "1"  /* Extirpation of Matter from L Sternoclav Jt, Open Approach */
"0RCF3ZZ"  =  "1"  /* Extirpation of Matter from L Sternoclav Jt, Perc Approach */
"0RCF4ZZ"  =  "1"  /* Extirpate of Matter from L Sternoclav Jt, Perc Endo Approach */
"0RCG0ZZ"  =  "1"  /* Extirpation of Matter from R Acromioclav Jt, Open Approach */
"0RCG3ZZ"  =  "1"  /* Extirpation of Matter from R Acromioclav Jt, Perc Approach */
"0RCG4ZZ"  =  "1"  /* Extirpate matter from R Acromioclav Jt, Perc Endo */
"0RCH0ZZ"  =  "1"  /* Extirpation of Matter from L Acromioclav Jt, Open Approach */
"0RCH3ZZ"  =  "1"  /* Extirpation of Matter from L Acromioclav Jt, Perc Approach */
"0RCH4ZZ"  =  "1"  /* Extirpate matter from L Acromioclav Jt, Perc Endo */
"0RCJ0ZZ"  =  "1"  /* Extirpation of Matter from R Shoulder Jt, Open Approach */
"0RCJ3ZZ"  =  "1"  /* Extirpation of Matter from R Shoulder Jt, Perc Approach */
"0RCJ4ZZ"  =  "1"  /* Extirpation of Matter from R Shoulder Jt, Perc Endo Approach */
"0RCK0ZZ"  =  "1"  /* Extirpation of Matter from L Shoulder Jt, Open Approach */
"0RCK3ZZ"  =  "1"  /* Extirpation of Matter from L Shoulder Jt, Perc Approach */
"0RCK4ZZ"  =  "1"  /* Extirpation of Matter from L Shoulder Jt, Perc Endo Approach */
"0RCL0ZZ"  =  "1"  /* Extirpation of Matter from Right Elbow Joint, Open Approach */
"0RCL3ZZ"  =  "1"  /* Extirpation of Matter from Right Elbow Joint, Perc Approach */
"0RCL4ZZ"  =  "1"  /* Extirpation of Matter from R Elbow Jt, Perc Endo Approach */
"0RCM0ZZ"  =  "1"  /* Extirpation of Matter from Left Elbow Joint, Open Approach */
"0RCM3ZZ"  =  "1"  /* Extirpation of Matter from Left Elbow Joint, Perc Approach */
"0RCM4ZZ"  =  "1"  /* Extirpation of Matter from L Elbow Jt, Perc Endo Approach */
"0RCN0ZZ"  =  "1"  /* Extirpation of Matter from Right Wrist Joint, Open Approach */
"0RCN3ZZ"  =  "1"  /* Extirpation of Matter from Right Wrist Joint, Perc Approach */
"0RCN4ZZ"  =  "1"  /* Extirpation of Matter from R Wrist Jt, Perc Endo Approach */
"0RCP0ZZ"  =  "1"  /* Extirpation of Matter from Left Wrist Joint, Open Approach */
"0RCP3ZZ"  =  "1"  /* Extirpation of Matter from Left Wrist Joint, Perc Approach */
"0RCP4ZZ"  =  "1"  /* Extirpation of Matter from L Wrist Jt, Perc Endo Approach */
"0RCQ0ZZ"  =  "1"  /* Extirpation of Matter from Right Carpal Joint, Open Approach */
"0RCQ3ZZ"  =  "1"  /* Extirpation of Matter from Right Carpal Joint, Perc Approach */
"0RCQ4ZZ"  =  "1"  /* Extirpation of Matter from R Carpal Jt, Perc Endo Approach */
"0RCR0ZZ"  =  "1"  /* Extirpation of Matter from Left Carpal Joint, Open Approach */
"0RCR3ZZ"  =  "1"  /* Extirpation of Matter from Left Carpal Joint, Perc Approach */
"0RCR4ZZ"  =  "1"  /* Extirpation of Matter from L Carpal Jt, Perc Endo Approach */
"0RCS0ZZ"  =  "1"  /* Extirpation of Matter from R Metacarpocarp Jt, Open Approach */
"0RCS3ZZ"  =  "1"  /* Extirpation of Matter from R Metacarpocarp Jt, Perc Approach */
"0RCS4ZZ"  =  "1"  /* Extirpate matter from R Metacarpocarp Jt, Perc Endo */
"0RCT0ZZ"  =  "1"  /* Extirpation of Matter from L Metacarpocarp Jt, Open Approach */
"0RCT3ZZ"  =  "1"  /* Extirpation of Matter from L Metacarpocarp Jt, Perc Approach */
"0RCT4ZZ"  =  "1"  /* Extirpate matter from L Metacarpocarp Jt, Perc Endo */
"0RCU0ZZ"  =  "1"  /* Extirpation of Matter from R Metacarpophal Jt, Open Approach */
"0RCU3ZZ"  =  "1"  /* Extirpation of Matter from R Metacarpophal Jt, Perc Approach */
"0RCU4ZZ"  =  "1"  /* Extirpate matter from R Metacarpophal Jt, Perc Endo */
"0RCV0ZZ"  =  "1"  /* Extirpation of Matter from L Metacarpophal Jt, Open Approach */
"0RCV3ZZ"  =  "1"  /* Extirpation of Matter from L Metacarpophal Jt, Perc Approach */
"0RCV4ZZ"  =  "1"  /* Extirpate matter from L Metacarpophal Jt, Perc Endo */
"0RCW0ZZ"  =  "1"  /* Extirpate of Matter from R Finger Phalanx Jt, Open Approach */
"0RCW3ZZ"  =  "1"  /* Extirpate of Matter from R Finger Phalanx Jt, Perc Approach */
"0RCW4ZZ"  =  "1"  /* Extirpate matter from R Finger Phalanx Jt, Perc Endo */
"0RCX0ZZ"  =  "1"  /* Extirpate of Matter from L Finger Phalanx Jt, Open Approach */
"0RCX3ZZ"  =  "1"  /* Extirpate of Matter from L Finger Phalanx Jt, Perc Approach */
"0RCX4ZZ"  =  "1"  /* Extirpate matter from L Finger Phalanx Jt, Perc Endo */
"0RG0070"  =  "1"  /* Fusion Occip Jt w Autol Sub, Ant Appr A Col, Open */
"0RG0071"  =  "1"  /* Fusion Occip Jt w Autol Sub, Post Appr P Col, Open */
"0RG007J"  =  "1"  /* Fusion Occip Jt w Autol Sub, Post Appr A Col, Open */
"0RG00A0"  =  "1"  /* Fusion Occip Jt w Intbd Fus Dev, Ant Appr A Col, Open */
"0RG00A1"  =  "1"  /* Fusion Occip Jt w Intbd Fus Dev, Post Appr P Col, Open */
"0RG00AJ"  =  "1"  /* Fusion Occip Jt w Intbd Fus Dev, Post Appr A Col, Open */
"0RG00J0"  =  "1"  /* Fusion Occip Jt w Synth Sub, Ant Appr A Col, Open */
"0RG00J1"  =  "1"  /* Fusion Occip Jt w Synth Sub, Post Appr P Col, Open */
"0RG00JJ"  =  "1"  /* Fusion Occip Jt w Synth Sub, Post Appr A Col, Open */
"0RG00K0"  =  "1"  /* Fusion Occip Jt w Nonaut Sub, Ant Appr A Col, Open */
"0RG00K1"  =  "1"  /* Fusion Occip Jt w Nonaut Sub, Post Appr P Col, Open */
"0RG00KJ"  =  "1"  /* Fusion Occip Jt w Nonaut Sub, Post Appr A Col, Open */
"0RG00Z0"  =  "1"  /* Fusion of Occip Jt, Ant Appr A Col, Open Approach */
"0RG00Z1"  =  "1"  /* Fusion of Occip Jt, Post Appr P Col, Open Approach */
"0RG00ZJ"  =  "1"  /* Fusion of Occip Jt, Post Appr A Col, Open Approach */
"0RG0370"  =  "1"  /* Fusion Occip Jt w Autol Sub, Ant Appr A Col, Perc */
"0RG0371"  =  "1"  /* Fusion Occip Jt w Autol Sub, Post Appr P Col, Perc */
"0RG037J"  =  "1"  /* Fusion Occip Jt w Autol Sub, Post Appr A Col, Perc */
"0RG03A0"  =  "1"  /* Fusion Occip Jt w Intbd Fus Dev, Ant Appr A Col, Perc */
"0RG03A1"  =  "1"  /* Fusion Occip Jt w Intbd Fus Dev, Post Appr P Col, Perc */
"0RG03AJ"  =  "1"  /* Fusion Occip Jt w Intbd Fus Dev, Post Appr A Col, Perc */
"0RG03J0"  =  "1"  /* Fusion Occip Jt w Synth Sub, Ant Appr A Col, Perc */
"0RG03J1"  =  "1"  /* Fusion Occip Jt w Synth Sub, Post Appr P Col, Perc */
"0RG03JJ"  =  "1"  /* Fusion Occip Jt w Synth Sub, Post Appr A Col, Perc */
"0RG03K0"  =  "1"  /* Fusion Occip Jt w Nonaut Sub, Ant Appr A Col, Perc */
"0RG03K1"  =  "1"  /* Fusion Occip Jt w Nonaut Sub, Post Appr P Col, Perc */
"0RG03KJ"  =  "1"  /* Fusion Occip Jt w Nonaut Sub, Post Appr A Col, Perc */
"0RG03Z0"  =  "1"  /* Fusion of Occip Jt, Ant Appr A Col, Perc Approach */
"0RG03Z1"  =  "1"  /* Fusion of Occip Jt, Post Appr P Col, Perc Approach */
"0RG03ZJ"  =  "1"  /* Fusion of Occip Jt, Post Appr A Col, Perc Approach */
"0RG0470"  =  "1"  /* Fusion Occip Jt w Autol Sub, Ant Appr A Col, Perc Endo */
"0RG0471"  =  "1"  /* Fusion Occip Jt w Autol Sub, Post Appr P Col, Perc Endo */
"0RG047J"  =  "1"  /* Fusion Occip Jt w Autol Sub, Post Appr A Col, Perc Endo */
"0RG04A0"  =  "1"  /* Fusion Occip Jt w Intbd Fus Dev, Ant Appr A Col, Perc Endo */
"0RG04A1"  =  "1"  /* Fusion Occip Jt w Intbd Fus Dev, Post Appr P Col, Perc Endo */
"0RG04AJ"  =  "1"  /* Fusion Occip Jt w Intbd Fus Dev, Post Appr A Col, Perc Endo */
"0RG04J0"  =  "1"  /* Fusion Occip Jt w Synth Sub, Ant Appr A Col, Perc Endo */
"0RG04J1"  =  "1"  /* Fusion Occip Jt w Synth Sub, Post Appr P Col, Perc Endo */
"0RG04JJ"  =  "1"  /* Fusion Occip Jt w Synth Sub, Post Appr A Col, Perc Endo */
"0RG04K0"  =  "1"  /* Fusion Occip Jt w Nonaut Sub, Ant Appr A Col, Perc Endo */
"0RG04K1"  =  "1"  /* Fusion Occip Jt w Nonaut Sub, Post Appr P Col, Perc Endo */
"0RG04KJ"  =  "1"  /* Fusion Occip Jt w Nonaut Sub, Post Appr A Col, Perc Endo */
"0RG04Z0"  =  "1"  /* Fusion of Occip Jt, Ant Appr A Col, Perc Endo Approach */
"0RG04Z1"  =  "1"  /* Fusion of Occip Jt, Post Appr P Col, Perc Endo Approach */
"0RG04ZJ"  =  "1"  /* Fusion of Occip Jt, Post Appr A Col, Perc Endo Approach */
"0RG1070"  =  "1"  /* Fusion Cerv Jt w Autol Sub, Ant Appr A Col, Open */
"0RG1071"  =  "1"  /* Fusion Cerv Jt w Autol Sub, Post Appr P Col, Open */
"0RG107J"  =  "1"  /* Fusion Cerv Jt w Autol Sub, Post Appr A Col, Open */
"0RG10A0"  =  "1"  /* Fusion Cerv Jt w Intbd Fus Dev, Ant Appr A Col, Open */
"0RG10A1"  =  "1"  /* Fusion Cerv Jt w Intbd Fus Dev, Post Appr P Col, Open */
"0RG10AJ"  =  "1"  /* Fusion Cerv Jt w Intbd Fus Dev, Post Appr A Col, Open */
"0RG10J0"  =  "1"  /* Fusion Cerv Jt w Synth Sub, Ant Appr A Col, Open */
"0RG10J1"  =  "1"  /* Fusion Cerv Jt w Synth Sub, Post Appr P Col, Open */
"0RG10JJ"  =  "1"  /* Fusion Cerv Jt w Synth Sub, Post Appr A Col, Open */
"0RG10K0"  =  "1"  /* Fusion Cerv Jt w Nonaut Sub, Ant Appr A Col, Open */
"0RG10K1"  =  "1"  /* Fusion Cerv Jt w Nonaut Sub, Post Appr P Col, Open */
"0RG10KJ"  =  "1"  /* Fusion Cerv Jt w Nonaut Sub, Post Appr A Col, Open */
"0RG10Z0"  =  "1"  /* Fusion of Cerv Jt, Ant Appr A Col, Open Approach */
"0RG10Z1"  =  "1"  /* Fusion of Cerv Jt, Post Appr P Col, Open Approach */
"0RG10ZJ"  =  "1"  /* Fusion of Cerv Jt, Post Appr A Col, Open Approach */
"0RG1370"  =  "1"  /* Fusion Cerv Jt w Autol Sub, Ant Appr A Col, Perc */
"0RG1371"  =  "1"  /* Fusion Cerv Jt w Autol Sub, Post Appr P Col, Perc */
"0RG137J"  =  "1"  /* Fusion Cerv Jt w Autol Sub, Post Appr A Col, Perc */
"0RG13A0"  =  "1"  /* Fusion Cerv Jt w Intbd Fus Dev, Ant Appr A Col, Perc */
"0RG13A1"  =  "1"  /* Fusion Cerv Jt w Intbd Fus Dev, Post Appr P Col, Perc */
"0RG13AJ"  =  "1"  /* Fusion Cerv Jt w Intbd Fus Dev, Post Appr A Col, Perc */
"0RG13J0"  =  "1"  /* Fusion Cerv Jt w Synth Sub, Ant Appr A Col, Perc */
"0RG13J1"  =  "1"  /* Fusion Cerv Jt w Synth Sub, Post Appr P Col, Perc */
"0RG13JJ"  =  "1"  /* Fusion Cerv Jt w Synth Sub, Post Appr A Col, Perc */
"0RG13K0"  =  "1"  /* Fusion Cerv Jt w Nonaut Sub, Ant Appr A Col, Perc */
"0RG13K1"  =  "1"  /* Fusion Cerv Jt w Nonaut Sub, Post Appr P Col, Perc */
"0RG13KJ"  =  "1"  /* Fusion Cerv Jt w Nonaut Sub, Post Appr A Col, Perc */
"0RG13Z0"  =  "1"  /* Fusion of Cerv Jt, Ant Appr A Col, Perc Approach */
"0RG13Z1"  =  "1"  /* Fusion of Cerv Jt, Post Appr P Col, Perc Approach */
"0RG13ZJ"  =  "1"  /* Fusion of Cerv Jt, Post Appr A Col, Perc Approach */
"0RG1470"  =  "1"  /* Fusion Cerv Jt w Autol Sub, Ant Appr A Col, Perc Endo */
"0RG1471"  =  "1"  /* Fusion Cerv Jt w Autol Sub, Post Appr P Col, Perc Endo */
"0RG147J"  =  "1"  /* Fusion Cerv Jt w Autol Sub, Post Appr A Col, Perc Endo */
"0RG14A0"  =  "1"  /* Fusion Cerv Jt w Intbd Fus Dev, Ant Appr A Col, Perc Endo */
"0RG14A1"  =  "1"  /* Fusion Cerv Jt w Intbd Fus Dev, Post Appr P Col, Perc Endo */
"0RG14AJ"  =  "1"  /* Fusion Cerv Jt w Intbd Fus Dev, Post Appr A Col, Perc Endo */
"0RG14J0"  =  "1"  /* Fusion Cerv Jt w Synth Sub, Ant Appr A Col, Perc Endo */
"0RG14J1"  =  "1"  /* Fusion Cerv Jt w Synth Sub, Post Appr P Col, Perc Endo */
"0RG14JJ"  =  "1"  /* Fusion Cerv Jt w Synth Sub, Post Appr A Col, Perc Endo */
"0RG14K0"  =  "1"  /* Fusion Cerv Jt w Nonaut Sub, Ant Appr A Col, Perc Endo */
"0RG14K1"  =  "1"  /* Fusion Cerv Jt w Nonaut Sub, Post Appr P Col, Perc Endo */
"0RG14KJ"  =  "1"  /* Fusion Cerv Jt w Nonaut Sub, Post Appr A Col, Perc Endo */
"0RG14Z0"  =  "1"  /* Fusion of Cerv Jt, Ant Appr A Col, Perc Endo Approach */
"0RG14Z1"  =  "1"  /* Fusion of Cerv Jt, Post Appr P Col, Perc Endo Approach */
"0RG14ZJ"  =  "1"  /* Fusion of Cerv Jt, Post Appr A Col, Perc Endo Approach */
"0RG2070"  =  "1"  /* Fusion 2-6 C Jt w Autol Sub, Ant Appr A Col, Open */
"0RG2071"  =  "1"  /* Fusion 2-6 C Jt w Autol Sub, Post Appr P Col, Open */
"0RG207J"  =  "1"  /* Fusion 2-6 C Jt w Autol Sub, Post Appr A Col, Open */
"0RG20A0"  =  "1"  /* Fusion 2-6 C Jt w Intbd Fus Dev, Ant Appr A Col, Open */
"0RG20A1"  =  "1"  /* Fusion 2-6 C Jt w Intbd Fus Dev, Post Appr P Col, Open */
"0RG20AJ"  =  "1"  /* Fusion 2-6 C Jt w Intbd Fus Dev, Post Appr A Col, Open */
"0RG20J0"  =  "1"  /* Fusion 2-6 C Jt w Synth Sub, Ant Appr A Col, Open */
"0RG20J1"  =  "1"  /* Fusion 2-6 C Jt w Synth Sub, Post Appr P Col, Open */
"0RG20JJ"  =  "1"  /* Fusion 2-6 C Jt w Synth Sub, Post Appr A Col, Open */
"0RG20K0"  =  "1"  /* Fusion 2-6 C Jt w Nonaut Sub, Ant Appr A Col, Open */
"0RG20K1"  =  "1"  /* Fusion 2-6 C Jt w Nonaut Sub, Post Appr P Col, Open */
"0RG20KJ"  =  "1"  /* Fusion 2-6 C Jt w Nonaut Sub, Post Appr A Col, Open */
"0RG20Z0"  =  "1"  /* Fusion of 2-6 C Jt, Ant Appr A Col, Open Approach */
"0RG20Z1"  =  "1"  /* Fusion of 2-6 C Jt, Post Appr P Col, Open Approach */
"0RG20ZJ"  =  "1"  /* Fusion of 2-6 C Jt, Post Appr A Col, Open Approach */
"0RG2370"  =  "1"  /* Fusion 2-6 C Jt w Autol Sub, Ant Appr A Col, Perc */
"0RG2371"  =  "1"  /* Fusion 2-6 C Jt w Autol Sub, Post Appr P Col, Perc */
"0RG237J"  =  "1"  /* Fusion 2-6 C Jt w Autol Sub, Post Appr A Col, Perc */
"0RG23A0"  =  "1"  /* Fusion 2-6 C Jt w Intbd Fus Dev, Ant Appr A Col, Perc */
"0RG23A1"  =  "1"  /* Fusion 2-6 C Jt w Intbd Fus Dev, Post Appr P Col, Perc */
"0RG23AJ"  =  "1"  /* Fusion 2-6 C Jt w Intbd Fus Dev, Post Appr A Col, Perc */
"0RG23J0"  =  "1"  /* Fusion 2-6 C Jt w Synth Sub, Ant Appr A Col, Perc */
"0RG23J1"  =  "1"  /* Fusion 2-6 C Jt w Synth Sub, Post Appr P Col, Perc */
"0RG23JJ"  =  "1"  /* Fusion 2-6 C Jt w Synth Sub, Post Appr A Col, Perc */
"0RG23K0"  =  "1"  /* Fusion 2-6 C Jt w Nonaut Sub, Ant Appr A Col, Perc */
"0RG23K1"  =  "1"  /* Fusion 2-6 C Jt w Nonaut Sub, Post Appr P Col, Perc */
"0RG23KJ"  =  "1"  /* Fusion 2-6 C Jt w Nonaut Sub, Post Appr A Col, Perc */
"0RG23Z0"  =  "1"  /* Fusion of 2-6 C Jt, Ant Appr A Col, Perc Approach */
"0RG23Z1"  =  "1"  /* Fusion of 2-6 C Jt, Post Appr P Col, Perc Approach */
"0RG23ZJ"  =  "1"  /* Fusion of 2-6 C Jt, Post Appr A Col, Perc Approach */
"0RG2470"  =  "1"  /* Fusion 2-6 C Jt w Autol Sub, Ant Appr A Col, Perc Endo */
"0RG2471"  =  "1"  /* Fusion 2-6 C Jt w Autol Sub, Post Appr P Col, Perc Endo */
"0RG247J"  =  "1"  /* Fusion 2-6 C Jt w Autol Sub, Post Appr A Col, Perc Endo */
"0RG24A0"  =  "1"  /* Fusion 2-6 C Jt w Intbd Fus Dev, Ant Appr A Col, Perc Endo */
"0RG24A1"  =  "1"  /* Fusion 2-6 C Jt w Intbd Fus Dev, Post Appr P Col, Perc Endo */
"0RG24AJ"  =  "1"  /* Fusion 2-6 C Jt w Intbd Fus Dev, Post Appr A Col, Perc Endo */
"0RG24J0"  =  "1"  /* Fusion 2-6 C Jt w Synth Sub, Ant Appr A Col, Perc Endo */
"0RG24J1"  =  "1"  /* Fusion 2-6 C Jt w Synth Sub, Post Appr P Col, Perc Endo */
"0RG24JJ"  =  "1"  /* Fusion 2-6 C Jt w Synth Sub, Post Appr A Col, Perc Endo */
"0RG24K0"  =  "1"  /* Fusion 2-6 C Jt w Nonaut Sub, Ant Appr A Col, Perc Endo */
"0RG24K1"  =  "1"  /* Fusion 2-6 C Jt w Nonaut Sub, Post Appr P Col, Perc Endo */
"0RG24KJ"  =  "1"  /* Fusion 2-6 C Jt w Nonaut Sub, Post Appr A Col, Perc Endo */
"0RG24Z0"  =  "1"  /* Fusion of 2-6 C Jt, Ant Appr A Col, Perc Endo Approach */
"0RG24Z1"  =  "1"  /* Fusion of 2-6 C Jt, Post Appr P Col, Perc Endo Approach */
"0RG24ZJ"  =  "1"  /* Fusion of 2-6 C Jt, Post Appr A Col, Perc Endo Approach */
"0RG4070"  =  "1"  /* Fusion C-thor Jt w Autol Sub, Ant Appr A Col, Open */
"0RG4071"  =  "1"  /* Fusion C-thor Jt w Autol Sub, Post Appr P Col, Open */
"0RG407J"  =  "1"  /* Fusion C-thor Jt w Autol Sub, Post Appr A Col, Open */
"0RG40A0"  =  "1"  /* Fusion C-thor Jt w Intbd Fus Dev, Ant Appr A Col, Open */
"0RG40A1"  =  "1"  /* Fusion C-thor Jt w Intbd Fus Dev, Post Appr P Col, Open */
"0RG40AJ"  =  "1"  /* Fusion C-thor Jt w Intbd Fus Dev, Post Appr A Col, Open */
"0RG40J0"  =  "1"  /* Fusion C-thor Jt w Synth Sub, Ant Appr A Col, Open */
"0RG40J1"  =  "1"  /* Fusion C-thor Jt w Synth Sub, Post Appr P Col, Open */
"0RG40JJ"  =  "1"  /* Fusion C-thor Jt w Synth Sub, Post Appr A Col, Open */
"0RG40K0"  =  "1"  /* Fusion C-thor Jt w Nonaut Sub, Ant Appr A Col, Open */
"0RG40K1"  =  "1"  /* Fusion C-thor Jt w Nonaut Sub, Post Appr P Col, Open */
"0RG40KJ"  =  "1"  /* Fusion C-thor Jt w Nonaut Sub, Post Appr A Col, Open */
"0RG40Z0"  =  "1"  /* Fusion of C-thor Jt, Ant Appr A Col, Open Approach */
"0RG40Z1"  =  "1"  /* Fusion of C-thor Jt, Post Appr P Col, Open Approach */
"0RG40ZJ"  =  "1"  /* Fusion of C-thor Jt, Post Appr A Col, Open Approach */
"0RG4370"  =  "1"  /* Fusion C-thor Jt w Autol Sub, Ant Appr A Col, Perc */
"0RG4371"  =  "1"  /* Fusion C-thor Jt w Autol Sub, Post Appr P Col, Perc */
"0RG437J"  =  "1"  /* Fusion C-thor Jt w Autol Sub, Post Appr A Col, Perc */
"0RG43A0"  =  "1"  /* Fusion C-thor Jt w Intbd Fus Dev, Ant Appr A Col, Perc */
"0RG43A1"  =  "1"  /* Fusion C-thor Jt w Intbd Fus Dev, Post Appr P Col, Perc */
"0RG43AJ"  =  "1"  /* Fusion C-thor Jt w Intbd Fus Dev, Post Appr A Col, Perc */
"0RG43J0"  =  "1"  /* Fusion C-thor Jt w Synth Sub, Ant Appr A Col, Perc */
"0RG43J1"  =  "1"  /* Fusion C-thor Jt w Synth Sub, Post Appr P Col, Perc */
"0RG43JJ"  =  "1"  /* Fusion C-thor Jt w Synth Sub, Post Appr A Col, Perc */
"0RG43K0"  =  "1"  /* Fusion C-thor Jt w Nonaut Sub, Ant Appr A Col, Perc */
"0RG43K1"  =  "1"  /* Fusion C-thor Jt w Nonaut Sub, Post Appr P Col, Perc */
"0RG43KJ"  =  "1"  /* Fusion C-thor Jt w Nonaut Sub, Post Appr A Col, Perc */
"0RG43Z0"  =  "1"  /* Fusion of C-thor Jt, Ant Appr A Col, Perc Approach */
"0RG43Z1"  =  "1"  /* Fusion of C-thor Jt, Post Appr P Col, Perc Approach */
"0RG43ZJ"  =  "1"  /* Fusion of C-thor Jt, Post Appr A Col, Perc Approach */
"0RG4470"  =  "1"  /* Fusion C-thor Jt w Autol Sub, Ant Appr A Col, Perc Endo */
"0RG4471"  =  "1"  /* Fusion C-thor Jt w Autol Sub, Post Appr P Col, Perc Endo */
"0RG447J"  =  "1"  /* Fusion C-thor Jt w Autol Sub, Post Appr A Col, Perc Endo */
"0RG44A0"  =  "1"  /* Fusion C-thor Jt w Intbd Fus Dev, Ant Appr A Col, Perc Endo */
"0RG44A1"  =  "1"  /* Fusion C-thor Jt w Intbd Fus Dev, Post Appr P Col, Perc Endo */
"0RG44AJ"  =  "1"  /* Fusion C-thor Jt w Intbd Fus Dev, Post Appr A Col, Perc Endo */
"0RG44J0"  =  "1"  /* Fusion C-thor Jt w Synth Sub, Ant Appr A Col, Perc Endo */
"0RG44J1"  =  "1"  /* Fusion C-thor Jt w Synth Sub, Post Appr P Col, Perc Endo */
"0RG44JJ"  =  "1"  /* Fusion C-thor Jt w Synth Sub, Post Appr A Col, Perc Endo */
"0RG44K0"  =  "1"  /* Fusion C-thor Jt w Nonaut Sub, Ant Appr A Col, Perc Endo */
"0RG44K1"  =  "1"  /* Fusion C-thor Jt w Nonaut Sub, Post Appr P Col, Perc Endo */
"0RG44KJ"  =  "1"  /* Fusion C-thor Jt w Nonaut Sub, Post Appr A Col, Perc Endo */
"0RG44Z0"  =  "1"  /* Fusion of C-thor Jt, Ant Appr A Col, Perc Endo Approach */
"0RG44Z1"  =  "1"  /* Fusion of C-thor Jt, Post Appr P Col, Perc Endo Approach */
"0RG44ZJ"  =  "1"  /* Fusion of C-thor Jt, Post Appr A Col, Perc Endo Approach */
"0RG6070"  =  "1"  /* Fusion Thor Jt w Autol Sub, Ant Appr A Col, Open */
"0RG6071"  =  "1"  /* Fusion Thor Jt w Autol Sub, Post Appr P Col, Open */
"0RG607J"  =  "1"  /* Fusion Thor Jt w Autol Sub, Post Appr A Col, Open */
"0RG60A0"  =  "1"  /* Fusion Thor Jt w Intbd Fus Dev, Ant Appr A Col, Open */
"0RG60A1"  =  "1"  /* Fusion Thor Jt w Intbd Fus Dev, Post Appr P Col, Open */
"0RG60AJ"  =  "1"  /* Fusion Thor Jt w Intbd Fus Dev, Post Appr A Col, Open */
"0RG60J0"  =  "1"  /* Fusion Thor Jt w Synth Sub, Ant Appr A Col, Open */
"0RG60J1"  =  "1"  /* Fusion Thor Jt w Synth Sub, Post Appr P Col, Open */
"0RG60JJ"  =  "1"  /* Fusion Thor Jt w Synth Sub, Post Appr A Col, Open */
"0RG60K0"  =  "1"  /* Fusion Thor Jt w Nonaut Sub, Ant Appr A Col, Open */
"0RG60K1"  =  "1"  /* Fusion Thor Jt w Nonaut Sub, Post Appr P Col, Open */
"0RG60KJ"  =  "1"  /* Fusion Thor Jt w Nonaut Sub, Post Appr A Col, Open */
"0RG60Z0"  =  "1"  /* Fusion of Thor Jt, Ant Appr A Col, Open Approach */
"0RG60Z1"  =  "1"  /* Fusion of Thor Jt, Post Appr P Col, Open Approach */
"0RG60ZJ"  =  "1"  /* Fusion of Thor Jt, Post Appr A Col, Open Approach */
"0RG6370"  =  "1"  /* Fusion Thor Jt w Autol Sub, Ant Appr A Col, Perc */
"0RG6371"  =  "1"  /* Fusion Thor Jt w Autol Sub, Post Appr P Col, Perc */
"0RG637J"  =  "1"  /* Fusion Thor Jt w Autol Sub, Post Appr A Col, Perc */
"0RG63A0"  =  "1"  /* Fusion Thor Jt w Intbd Fus Dev, Ant Appr A Col, Perc */
"0RG63A1"  =  "1"  /* Fusion Thor Jt w Intbd Fus Dev, Post Appr P Col, Perc */
"0RG63AJ"  =  "1"  /* Fusion Thor Jt w Intbd Fus Dev, Post Appr A Col, Perc */
"0RG63J0"  =  "1"  /* Fusion Thor Jt w Synth Sub, Ant Appr A Col, Perc */
"0RG63J1"  =  "1"  /* Fusion Thor Jt w Synth Sub, Post Appr P Col, Perc */
"0RG63JJ"  =  "1"  /* Fusion Thor Jt w Synth Sub, Post Appr A Col, Perc */
"0RG63K0"  =  "1"  /* Fusion Thor Jt w Nonaut Sub, Ant Appr A Col, Perc */
"0RG63K1"  =  "1"  /* Fusion Thor Jt w Nonaut Sub, Post Appr P Col, Perc */
"0RG63KJ"  =  "1"  /* Fusion Thor Jt w Nonaut Sub, Post Appr A Col, Perc */
"0RG63Z0"  =  "1"  /* Fusion of Thor Jt, Ant Appr A Col, Perc Approach */
"0RG63Z1"  =  "1"  /* Fusion of Thor Jt, Post Appr P Col, Perc Approach */
"0RG63ZJ"  =  "1"  /* Fusion of Thor Jt, Post Appr A Col, Perc Approach */
"0RG6470"  =  "1"  /* Fusion Thor Jt w Autol Sub, Ant Appr A Col, Perc Endo */
"0RG6471"  =  "1"  /* Fusion Thor Jt w Autol Sub, Post Appr P Col, Perc Endo */
"0RG647J"  =  "1"  /* Fusion Thor Jt w Autol Sub, Post Appr A Col, Perc Endo */
"0RG64A0"  =  "1"  /* Fusion Thor Jt w Intbd Fus Dev, Ant Appr A Col, Perc Endo */
"0RG64A1"  =  "1"  /* Fusion Thor Jt w Intbd Fus Dev, Post Appr P Col, Perc Endo */
"0RG64AJ"  =  "1"  /* Fusion Thor Jt w Intbd Fus Dev, Post Appr A Col, Perc Endo */
"0RG64J0"  =  "1"  /* Fusion Thor Jt w Synth Sub, Ant Appr A Col, Perc Endo */
"0RG64J1"  =  "1"  /* Fusion Thor Jt w Synth Sub, Post Appr P Col, Perc Endo */
"0RG64JJ"  =  "1"  /* Fusion Thor Jt w Synth Sub, Post Appr A Col, Perc Endo */
"0RG64K0"  =  "1"  /* Fusion Thor Jt w Nonaut Sub, Ant Appr A Col, Perc Endo */
"0RG64K1"  =  "1"  /* Fusion Thor Jt w Nonaut Sub, Post Appr P Col, Perc Endo */
"0RG64KJ"  =  "1"  /* Fusion Thor Jt w Nonaut Sub, Post Appr A Col, Perc Endo */
"0RG64Z0"  =  "1"  /* Fusion of Thor Jt, Ant Appr A Col, Perc Endo Approach */
"0RG64Z1"  =  "1"  /* Fusion of Thor Jt, Post Appr P Col, Perc Endo Approach */
"0RG64ZJ"  =  "1"  /* Fusion of Thor Jt, Post Appr A Col, Perc Endo Approach */
"0RG7070"  =  "1"  /* Fusion 2-7 T Jt w Autol Sub, Ant Appr A Col, Open */
"0RG7071"  =  "1"  /* Fusion 2-7 T Jt w Autol Sub, Post Appr P Col, Open */
"0RG707J"  =  "1"  /* Fusion 2-7 T Jt w Autol Sub, Post Appr A Col, Open */
"0RG70A0"  =  "1"  /* Fusion 2-7 T Jt w Intbd Fus Dev, Ant Appr A Col, Open */
"0RG70A1"  =  "1"  /* Fusion 2-7 T Jt w Intbd Fus Dev, Post Appr P Col, Open */
"0RG70AJ"  =  "1"  /* Fusion 2-7 T Jt w Intbd Fus Dev, Post Appr A Col, Open */
"0RG70J0"  =  "1"  /* Fusion 2-7 T Jt w Synth Sub, Ant Appr A Col, Open */
"0RG70J1"  =  "1"  /* Fusion 2-7 T Jt w Synth Sub, Post Appr P Col, Open */
"0RG70JJ"  =  "1"  /* Fusion 2-7 T Jt w Synth Sub, Post Appr A Col, Open */
"0RG70K0"  =  "1"  /* Fusion 2-7 T Jt w Nonaut Sub, Ant Appr A Col, Open */
"0RG70K1"  =  "1"  /* Fusion 2-7 T Jt w Nonaut Sub, Post Appr P Col, Open */
"0RG70KJ"  =  "1"  /* Fusion 2-7 T Jt w Nonaut Sub, Post Appr A Col, Open */
"0RG70Z0"  =  "1"  /* Fusion of 2-7 T Jt, Ant Appr A Col, Open Approach */
"0RG70Z1"  =  "1"  /* Fusion of 2-7 T Jt, Post Appr P Col, Open Approach */
"0RG70ZJ"  =  "1"  /* Fusion of 2-7 T Jt, Post Appr A Col, Open Approach */
"0RG7370"  =  "1"  /* Fusion 2-7 T Jt w Autol Sub, Ant Appr A Col, Perc */
"0RG7371"  =  "1"  /* Fusion 2-7 T Jt w Autol Sub, Post Appr P Col, Perc */
"0RG737J"  =  "1"  /* Fusion 2-7 T Jt w Autol Sub, Post Appr A Col, Perc */
"0RG73A0"  =  "1"  /* Fusion 2-7 T Jt w Intbd Fus Dev, Ant Appr A Col, Perc */
"0RG73A1"  =  "1"  /* Fusion 2-7 T Jt w Intbd Fus Dev, Post Appr P Col, Perc */
"0RG73AJ"  =  "1"  /* Fusion 2-7 T Jt w Intbd Fus Dev, Post Appr A Col, Perc */
"0RG73J0"  =  "1"  /* Fusion 2-7 T Jt w Synth Sub, Ant Appr A Col, Perc */
"0RG73J1"  =  "1"  /* Fusion 2-7 T Jt w Synth Sub, Post Appr P Col, Perc */
"0RG73JJ"  =  "1"  /* Fusion 2-7 T Jt w Synth Sub, Post Appr A Col, Perc */
"0RG73K0"  =  "1"  /* Fusion 2-7 T Jt w Nonaut Sub, Ant Appr A Col, Perc */
"0RG73K1"  =  "1"  /* Fusion 2-7 T Jt w Nonaut Sub, Post Appr P Col, Perc */
"0RG73KJ"  =  "1"  /* Fusion 2-7 T Jt w Nonaut Sub, Post Appr A Col, Perc */
"0RG73Z0"  =  "1"  /* Fusion of 2-7 T Jt, Ant Appr A Col, Perc Approach */
"0RG73Z1"  =  "1"  /* Fusion of 2-7 T Jt, Post Appr P Col, Perc Approach */
"0RG73ZJ"  =  "1"  /* Fusion of 2-7 T Jt, Post Appr A Col, Perc Approach */
"0RG7470"  =  "1"  /* Fusion 2-7 T Jt w Autol Sub, Ant Appr A Col, Perc Endo */
"0RG7471"  =  "1"  /* Fusion 2-7 T Jt w Autol Sub, Post Appr P Col, Perc Endo */
"0RG747J"  =  "1"  /* Fusion 2-7 T Jt w Autol Sub, Post Appr A Col, Perc Endo */
"0RG74A0"  =  "1"  /* Fusion 2-7 T Jt w Intbd Fus Dev, Ant Appr A Col, Perc Endo */
"0RG74A1"  =  "1"  /* Fusion 2-7 T Jt w Intbd Fus Dev, Post Appr P Col, Perc Endo */
"0RG74AJ"  =  "1"  /* Fusion 2-7 T Jt w Intbd Fus Dev, Post Appr A Col, Perc Endo */
"0RG74J0"  =  "1"  /* Fusion 2-7 T Jt w Synth Sub, Ant Appr A Col, Perc Endo */
"0RG74J1"  =  "1"  /* Fusion 2-7 T Jt w Synth Sub, Post Appr P Col, Perc Endo */
"0RG74JJ"  =  "1"  /* Fusion 2-7 T Jt w Synth Sub, Post Appr A Col, Perc Endo */
"0RG74K0"  =  "1"  /* Fusion 2-7 T Jt w Nonaut Sub, Ant Appr A Col, Perc Endo */
"0RG74K1"  =  "1"  /* Fusion 2-7 T Jt w Nonaut Sub, Post Appr P Col, Perc Endo */
"0RG74KJ"  =  "1"  /* Fusion 2-7 T Jt w Nonaut Sub, Post Appr A Col, Perc Endo */
"0RG74Z0"  =  "1"  /* Fusion of 2-7 T Jt, Ant Appr A Col, Perc Endo Approach */
"0RG74Z1"  =  "1"  /* Fusion of 2-7 T Jt, Post Appr P Col, Perc Endo Approach */
"0RG74ZJ"  =  "1"  /* Fusion of 2-7 T Jt, Post Appr A Col, Perc Endo Approach */
"0RG8070"  =  "1"  /* Fusion >7 T Jt w Autol Sub, Ant Appr A Col, Open */
"0RG8071"  =  "1"  /* Fusion >7 T Jt w Autol Sub, Post Appr P Col, Open */
"0RG807J"  =  "1"  /* Fusion >7 T Jt w Autol Sub, Post Appr A Col, Open */
"0RG80A0"  =  "1"  /* Fusion >7 T Jt w Intbd Fus Dev, Ant Appr A Col, Open */
"0RG80A1"  =  "1"  /* Fusion >7 T Jt w Intbd Fus Dev, Post Appr P Col, Open */
"0RG80AJ"  =  "1"  /* Fusion >7 T Jt w Intbd Fus Dev, Post Appr A Col, Open */
"0RG80J0"  =  "1"  /* Fusion >7 T Jt w Synth Sub, Ant Appr A Col, Open */
"0RG80J1"  =  "1"  /* Fusion >7 T Jt w Synth Sub, Post Appr P Col, Open */
"0RG80JJ"  =  "1"  /* Fusion >7 T Jt w Synth Sub, Post Appr A Col, Open */
"0RG80K0"  =  "1"  /* Fusion >7 T Jt w Nonaut Sub, Ant Appr A Col, Open */
"0RG80K1"  =  "1"  /* Fusion >7 T Jt w Nonaut Sub, Post Appr P Col, Open */
"0RG80KJ"  =  "1"  /* Fusion >7 T Jt w Nonaut Sub, Post Appr A Col, Open */
"0RG80Z0"  =  "1"  /* Fusion of >7 T Jt, Ant Appr A Col, Open Approach */
"0RG80Z1"  =  "1"  /* Fusion of >7 T Jt, Post Appr P Col, Open Approach */
"0RG80ZJ"  =  "1"  /* Fusion of >7 T Jt, Post Appr A Col, Open Approach */
"0RG8370"  =  "1"  /* Fusion >7 T Jt w Autol Sub, Ant Appr A Col, Perc */
"0RG8371"  =  "1"  /* Fusion >7 T Jt w Autol Sub, Post Appr P Col, Perc */
"0RG837J"  =  "1"  /* Fusion >7 T Jt w Autol Sub, Post Appr A Col, Perc */
"0RG83A0"  =  "1"  /* Fusion >7 T Jt w Intbd Fus Dev, Ant Appr A Col, Perc */
"0RG83A1"  =  "1"  /* Fusion >7 T Jt w Intbd Fus Dev, Post Appr P Col, Perc */
"0RG83AJ"  =  "1"  /* Fusion >7 T Jt w Intbd Fus Dev, Post Appr A Col, Perc */
"0RG83J0"  =  "1"  /* Fusion >7 T Jt w Synth Sub, Ant Appr A Col, Perc */
"0RG83J1"  =  "1"  /* Fusion >7 T Jt w Synth Sub, Post Appr P Col, Perc */
"0RG83JJ"  =  "1"  /* Fusion >7 T Jt w Synth Sub, Post Appr A Col, Perc */
"0RG83K0"  =  "1"  /* Fusion >7 T Jt w Nonaut Sub, Ant Appr A Col, Perc */
"0RG83K1"  =  "1"  /* Fusion >7 T Jt w Nonaut Sub, Post Appr P Col, Perc */
"0RG83KJ"  =  "1"  /* Fusion >7 T Jt w Nonaut Sub, Post Appr A Col, Perc */
"0RG83Z0"  =  "1"  /* Fusion of >7 T Jt, Ant Appr A Col, Perc Approach */
"0RG83Z1"  =  "1"  /* Fusion of >7 T Jt, Post Appr P Col, Perc Approach */
"0RG83ZJ"  =  "1"  /* Fusion of >7 T Jt, Post Appr A Col, Perc Approach */
"0RG8470"  =  "1"  /* Fusion >7 T Jt w Autol Sub, Ant Appr A Col, Perc Endo */
"0RG8471"  =  "1"  /* Fusion >7 T Jt w Autol Sub, Post Appr P Col, Perc Endo */
"0RG847J"  =  "1"  /* Fusion >7 T Jt w Autol Sub, Post Appr A Col, Perc Endo */
"0RG84A0"  =  "1"  /* Fusion >7 T Jt w Intbd Fus Dev, Ant Appr A Col, Perc Endo */
"0RG84A1"  =  "1"  /* Fusion >7 T Jt w Intbd Fus Dev, Post Appr P Col, Perc Endo */
"0RG84AJ"  =  "1"  /* Fusion >7 T Jt w Intbd Fus Dev, Post Appr A Col, Perc Endo */
"0RG84J0"  =  "1"  /* Fusion >7 T Jt w Synth Sub, Ant Appr A Col, Perc Endo */
"0RG84J1"  =  "1"  /* Fusion >7 T Jt w Synth Sub, Post Appr P Col, Perc Endo */
"0RG84JJ"  =  "1"  /* Fusion >7 T Jt w Synth Sub, Post Appr A Col, Perc Endo */
"0RG84K0"  =  "1"  /* Fusion >7 T Jt w Nonaut Sub, Ant Appr A Col, Perc Endo */
"0RG84K1"  =  "1"  /* Fusion >7 T Jt w Nonaut Sub, Post Appr P Col, Perc Endo */
"0RG84KJ"  =  "1"  /* Fusion >7 T Jt w Nonaut Sub, Post Appr A Col, Perc Endo */
"0RG84Z0"  =  "1"  /* Fusion of >7 T Jt, Ant Appr A Col, Perc Endo Approach */
"0RG84Z1"  =  "1"  /* Fusion of >7 T Jt, Post Appr P Col, Perc Endo Approach */
"0RG84ZJ"  =  "1"  /* Fusion of >7 T Jt, Post Appr A Col, Perc Endo Approach */
"0RGA070"  =  "1"  /* Fusion T-lum Jt w Autol Sub, Ant Appr A Col, Open */
"0RGA071"  =  "1"  /* Fusion T-lum Jt w Autol Sub, Post Appr P Col, Open */
"0RGA07J"  =  "1"  /* Fusion T-lum Jt w Autol Sub, Post Appr A Col, Open */
"0RGA0A0"  =  "1"  /* Fusion T-lum Jt w Intbd Fus Dev, Ant Appr A Col, Open */
"0RGA0A1"  =  "1"  /* Fusion T-lum Jt w Intbd Fus Dev, Post Appr P Col, Open */
"0RGA0AJ"  =  "1"  /* Fusion T-lum Jt w Intbd Fus Dev, Post Appr A Col, Open */
"0RGA0J0"  =  "1"  /* Fusion T-lum Jt w Synth Sub, Ant Appr A Col, Open */
"0RGA0J1"  =  "1"  /* Fusion T-lum Jt w Synth Sub, Post Appr P Col, Open */
"0RGA0JJ"  =  "1"  /* Fusion T-lum Jt w Synth Sub, Post Appr A Col, Open */
"0RGA0K0"  =  "1"  /* Fusion T-lum Jt w Nonaut Sub, Ant Appr A Col, Open */
"0RGA0K1"  =  "1"  /* Fusion T-lum Jt w Nonaut Sub, Post Appr P Col, Open */
"0RGA0KJ"  =  "1"  /* Fusion T-lum Jt w Nonaut Sub, Post Appr A Col, Open */
"0RGA0Z0"  =  "1"  /* Fusion of T-lum Jt, Ant Appr A Col, Open Approach */
"0RGA0Z1"  =  "1"  /* Fusion of T-lum Jt, Post Appr P Col, Open Approach */
"0RGA0ZJ"  =  "1"  /* Fusion of T-lum Jt, Post Appr A Col, Open Approach */
"0RGA370"  =  "1"  /* Fusion T-lum Jt w Autol Sub, Ant Appr A Col, Perc */
"0RGA371"  =  "1"  /* Fusion T-lum Jt w Autol Sub, Post Appr P Col, Perc */
"0RGA37J"  =  "1"  /* Fusion T-lum Jt w Autol Sub, Post Appr A Col, Perc */
"0RGA3A0"  =  "1"  /* Fusion T-lum Jt w Intbd Fus Dev, Ant Appr A Col, Perc */
"0RGA3A1"  =  "1"  /* Fusion T-lum Jt w Intbd Fus Dev, Post Appr P Col, Perc */
"0RGA3AJ"  =  "1"  /* Fusion T-lum Jt w Intbd Fus Dev, Post Appr A Col, Perc */
"0RGA3J0"  =  "1"  /* Fusion T-lum Jt w Synth Sub, Ant Appr A Col, Perc */
"0RGA3J1"  =  "1"  /* Fusion T-lum Jt w Synth Sub, Post Appr P Col, Perc */
"0RGA3JJ"  =  "1"  /* Fusion T-lum Jt w Synth Sub, Post Appr A Col, Perc */
"0RGA3K0"  =  "1"  /* Fusion T-lum Jt w Nonaut Sub, Ant Appr A Col, Perc */
"0RGA3K1"  =  "1"  /* Fusion T-lum Jt w Nonaut Sub, Post Appr P Col, Perc */
"0RGA3KJ"  =  "1"  /* Fusion T-lum Jt w Nonaut Sub, Post Appr A Col, Perc */
"0RGA3Z0"  =  "1"  /* Fusion of T-lum Jt, Ant Appr A Col, Perc Approach */
"0RGA3Z1"  =  "1"  /* Fusion of T-lum Jt, Post Appr P Col, Perc Approach */
"0RGA3ZJ"  =  "1"  /* Fusion of T-lum Jt, Post Appr A Col, Perc Approach */
"0RGA470"  =  "1"  /* Fusion T-lum Jt w Autol Sub, Ant Appr A Col, Perc Endo */
"0RGA471"  =  "1"  /* Fusion T-lum Jt w Autol Sub, Post Appr P Col, Perc Endo */
"0RGA47J"  =  "1"  /* Fusion T-lum Jt w Autol Sub, Post Appr A Col, Perc Endo */
"0RGA4A0"  =  "1"  /* Fusion T-lum Jt w Intbd Fus Dev, Ant Appr A Col, Perc Endo */
"0RGA4A1"  =  "1"  /* Fusion T-lum Jt w Intbd Fus Dev, Post Appr P Col, Perc Endo */
"0RGA4AJ"  =  "1"  /* Fusion T-lum Jt w Intbd Fus Dev, Post Appr A Col, Perc Endo */
"0RGA4J0"  =  "1"  /* Fusion T-lum Jt w Synth Sub, Ant Appr A Col, Perc Endo */
"0RGA4J1"  =  "1"  /* Fusion T-lum Jt w Synth Sub, Post Appr P Col, Perc Endo */
"0RGA4JJ"  =  "1"  /* Fusion T-lum Jt w Synth Sub, Post Appr A Col, Perc Endo */
"0RGA4K0"  =  "1"  /* Fusion T-lum Jt w Nonaut Sub, Ant Appr A Col, Perc Endo */
"0RGA4K1"  =  "1"  /* Fusion T-lum Jt w Nonaut Sub, Post Appr P Col, Perc Endo */
"0RGA4KJ"  =  "1"  /* Fusion T-lum Jt w Nonaut Sub, Post Appr A Col, Perc Endo */
"0RGA4Z0"  =  "1"  /* Fusion of T-lum Jt, Ant Appr A Col, Perc Endo Approach */
"0RGA4Z1"  =  "1"  /* Fusion of T-lum Jt, Post Appr P Col, Perc Endo Approach */
"0RGA4ZJ"  =  "1"  /* Fusion of T-lum Jt, Post Appr A Col, Perc Endo Approach */
"0RGC04Z"  =  "1"  /* Fusion of R Temporomandib Jt with Int Fix, Open Approach */
"0RGC07Z"  =  "1"  /* Fusion of R Temporomandib Jt with Autol Sub, Open Approach */
"0RGC0JZ"  =  "1"  /* Fusion of R Temporomandib Jt with Synth Sub, Open Approach */
"0RGC0KZ"  =  "1"  /* Fusion of R Temporomandib Jt with Nonaut Sub, Open Approach */
"0RGC0ZZ"  =  "1"  /* Fusion of Right Temporomandibular Joint, Open Approach */
"0RGC34Z"  =  "1"  /* Fusion of R Temporomandib Jt with Int Fix, Perc Approach */
"0RGC37Z"  =  "1"  /* Fusion of R Temporomandib Jt with Autol Sub, Perc Approach */
"0RGC3JZ"  =  "1"  /* Fusion of R Temporomandib Jt with Synth Sub, Perc Approach */
"0RGC3KZ"  =  "1"  /* Fusion of R Temporomandib Jt with Nonaut Sub, Perc Approach */
"0RGC3ZZ"  =  "1"  /* Fusion of Right Temporomandibular Joint, Perc Approach */
"0RGC44Z"  =  "1"  /* Fusion R Temporomandib Jt w Int Fix, Perc Endo */
"0RGC47Z"  =  "1"  /* Fusion R Temporomandib Jt w Autol Sub, Perc Endo */
"0RGC4JZ"  =  "1"  /* Fusion R Temporomandib Jt w Synth Sub, Perc Endo */
"0RGC4KZ"  =  "1"  /* Fusion R Temporomandib Jt w Nonaut Sub, Perc Endo */
"0RGC4ZZ"  =  "1"  /* Fusion of Right Temporomandibular Joint, Perc Endo Approach */
"0RGD04Z"  =  "1"  /* Fusion of L Temporomandib Jt with Int Fix, Open Approach */
"0RGD07Z"  =  "1"  /* Fusion of L Temporomandib Jt with Autol Sub, Open Approach */
"0RGD0JZ"  =  "1"  /* Fusion of L Temporomandib Jt with Synth Sub, Open Approach */
"0RGD0KZ"  =  "1"  /* Fusion of L Temporomandib Jt with Nonaut Sub, Open Approach */
"0RGD0ZZ"  =  "1"  /* Fusion of Left Temporomandibular Joint, Open Approach */
"0RGD34Z"  =  "1"  /* Fusion of L Temporomandib Jt with Int Fix, Perc Approach */
"0RGD37Z"  =  "1"  /* Fusion of L Temporomandib Jt with Autol Sub, Perc Approach */
"0RGD3JZ"  =  "1"  /* Fusion of L Temporomandib Jt with Synth Sub, Perc Approach */
"0RGD3KZ"  =  "1"  /* Fusion of L Temporomandib Jt with Nonaut Sub, Perc Approach */
"0RGD3ZZ"  =  "1"  /* Fusion of Left Temporomandibular Joint, Perc Approach */
"0RGD44Z"  =  "1"  /* Fusion L Temporomandib Jt w Int Fix, Perc Endo */
"0RGD47Z"  =  "1"  /* Fusion L Temporomandib Jt w Autol Sub, Perc Endo */
"0RGD4JZ"  =  "1"  /* Fusion L Temporomandib Jt w Synth Sub, Perc Endo */
"0RGD4KZ"  =  "1"  /* Fusion L Temporomandib Jt w Nonaut Sub, Perc Endo */
"0RGD4ZZ"  =  "1"  /* Fusion of Left Temporomandibular Joint, Perc Endo Approach */
"0RGE04Z"  =  "1"  /* Fusion of R Sternoclav Jt with Int Fix, Open Approach */
"0RGE07Z"  =  "1"  /* Fusion of R Sternoclav Jt with Autol Sub, Open Approach */
"0RGE0JZ"  =  "1"  /* Fusion of R Sternoclav Jt with Synth Sub, Open Approach */
"0RGE0KZ"  =  "1"  /* Fusion of R Sternoclav Jt with Nonaut Sub, Open Approach */
"0RGE0ZZ"  =  "1"  /* Fusion of Right Sternoclavicular Joint, Open Approach */
"0RGE34Z"  =  "1"  /* Fusion of R Sternoclav Jt with Int Fix, Perc Approach */
"0RGE37Z"  =  "1"  /* Fusion of R Sternoclav Jt with Autol Sub, Perc Approach */
"0RGE3JZ"  =  "1"  /* Fusion of R Sternoclav Jt with Synth Sub, Perc Approach */
"0RGE3KZ"  =  "1"  /* Fusion of R Sternoclav Jt with Nonaut Sub, Perc Approach */
"0RGE3ZZ"  =  "1"  /* Fusion of Right Sternoclavicular Joint, Perc Approach */
"0RGE44Z"  =  "1"  /* Fusion of R Sternoclav Jt with Int Fix, Perc Endo Approach */
"0RGE47Z"  =  "1"  /* Fusion of R Sternoclav Jt with Autol Sub, Perc Endo Approach */
"0RGE4JZ"  =  "1"  /* Fusion of R Sternoclav Jt with Synth Sub, Perc Endo Approach */
"0RGE4KZ"  =  "1"  /* Fusion R Sternoclav Jt w Nonaut Sub, Perc Endo */
"0RGE4ZZ"  =  "1"  /* Fusion of Right Sternoclavicular Joint, Perc Endo Approach */
"0RGF04Z"  =  "1"  /* Fusion of L Sternoclav Jt with Int Fix, Open Approach */
"0RGF07Z"  =  "1"  /* Fusion of L Sternoclav Jt with Autol Sub, Open Approach */
"0RGF0JZ"  =  "1"  /* Fusion of L Sternoclav Jt with Synth Sub, Open Approach */
"0RGF0KZ"  =  "1"  /* Fusion of L Sternoclav Jt with Nonaut Sub, Open Approach */
"0RGF0ZZ"  =  "1"  /* Fusion of Left Sternoclavicular Joint, Open Approach */
"0RGF34Z"  =  "1"  /* Fusion of L Sternoclav Jt with Int Fix, Perc Approach */
"0RGF37Z"  =  "1"  /* Fusion of L Sternoclav Jt with Autol Sub, Perc Approach */
"0RGF3JZ"  =  "1"  /* Fusion of L Sternoclav Jt with Synth Sub, Perc Approach */
"0RGF3KZ"  =  "1"  /* Fusion of L Sternoclav Jt with Nonaut Sub, Perc Approach */
"0RGF3ZZ"  =  "1"  /* Fusion of Left Sternoclavicular Joint, Percutaneous Approach */
"0RGF44Z"  =  "1"  /* Fusion of L Sternoclav Jt with Int Fix, Perc Endo Approach */
"0RGF47Z"  =  "1"  /* Fusion of L Sternoclav Jt with Autol Sub, Perc Endo Approach */
"0RGF4JZ"  =  "1"  /* Fusion of L Sternoclav Jt with Synth Sub, Perc Endo Approach */
"0RGF4KZ"  =  "1"  /* Fusion L Sternoclav Jt w Nonaut Sub, Perc Endo */
"0RGF4ZZ"  =  "1"  /* Fusion of Left Sternoclavicular Joint, Perc Endo Approach */
"0RGG04Z"  =  "1"  /* Fusion of R Acromioclav Jt with Int Fix, Open Approach */
"0RGG07Z"  =  "1"  /* Fusion of R Acromioclav Jt with Autol Sub, Open Approach */
"0RGG0JZ"  =  "1"  /* Fusion of R Acromioclav Jt with Synth Sub, Open Approach */
"0RGG0KZ"  =  "1"  /* Fusion of R Acromioclav Jt with Nonaut Sub, Open Approach */
"0RGG0ZZ"  =  "1"  /* Fusion of Right Acromioclavicular Joint, Open Approach */
"0RGG34Z"  =  "1"  /* Fusion of R Acromioclav Jt with Int Fix, Perc Approach */
"0RGG37Z"  =  "1"  /* Fusion of R Acromioclav Jt with Autol Sub, Perc Approach */
"0RGG3JZ"  =  "1"  /* Fusion of R Acromioclav Jt with Synth Sub, Perc Approach */
"0RGG3KZ"  =  "1"  /* Fusion of R Acromioclav Jt with Nonaut Sub, Perc Approach */
"0RGG3ZZ"  =  "1"  /* Fusion of Right Acromioclavicular Joint, Perc Approach */
"0RGG44Z"  =  "1"  /* Fusion of R Acromioclav Jt with Int Fix, Perc Endo Approach */
"0RGG47Z"  =  "1"  /* Fusion R Acromioclav Jt w Autol Sub, Perc Endo */
"0RGG4JZ"  =  "1"  /* Fusion R Acromioclav Jt w Synth Sub, Perc Endo */
"0RGG4KZ"  =  "1"  /* Fusion R Acromioclav Jt w Nonaut Sub, Perc Endo */
"0RGG4ZZ"  =  "1"  /* Fusion of Right Acromioclavicular Joint, Perc Endo Approach */
"0RGH04Z"  =  "1"  /* Fusion of L Acromioclav Jt with Int Fix, Open Approach */
"0RGH07Z"  =  "1"  /* Fusion of L Acromioclav Jt with Autol Sub, Open Approach */
"0RGH0JZ"  =  "1"  /* Fusion of L Acromioclav Jt with Synth Sub, Open Approach */
"0RGH0KZ"  =  "1"  /* Fusion of L Acromioclav Jt with Nonaut Sub, Open Approach */
"0RGH0ZZ"  =  "1"  /* Fusion of Left Acromioclavicular Joint, Open Approach */
"0RGH34Z"  =  "1"  /* Fusion of L Acromioclav Jt with Int Fix, Perc Approach */
"0RGH37Z"  =  "1"  /* Fusion of L Acromioclav Jt with Autol Sub, Perc Approach */
"0RGH3JZ"  =  "1"  /* Fusion of L Acromioclav Jt with Synth Sub, Perc Approach */
"0RGH3KZ"  =  "1"  /* Fusion of L Acromioclav Jt with Nonaut Sub, Perc Approach */
"0RGH3ZZ"  =  "1"  /* Fusion of Left Acromioclavicular Joint, Perc Approach */
"0RGH44Z"  =  "1"  /* Fusion of L Acromioclav Jt with Int Fix, Perc Endo Approach */
"0RGH47Z"  =  "1"  /* Fusion L Acromioclav Jt w Autol Sub, Perc Endo */
"0RGH4JZ"  =  "1"  /* Fusion L Acromioclav Jt w Synth Sub, Perc Endo */
"0RGH4KZ"  =  "1"  /* Fusion L Acromioclav Jt w Nonaut Sub, Perc Endo */
"0RGH4ZZ"  =  "1"  /* Fusion of Left Acromioclavicular Joint, Perc Endo Approach */
"0RGJ04Z"  =  "1"  /* Fusion of Right Shoulder Joint with Int Fix, Open Approach */
"0RGJ07Z"  =  "1"  /* Fusion of Right Shoulder Joint with Autol Sub, Open Approach */
"0RGJ0JZ"  =  "1"  /* Fusion of Right Shoulder Joint with Synth Sub, Open Approach */
"0RGJ0KZ"  =  "1"  /* Fusion of R Shoulder Jt with Nonaut Sub, Open Approach */
"0RGJ0ZZ"  =  "1"  /* Fusion of Right Shoulder Joint, Open Approach */
"0RGJ34Z"  =  "1"  /* Fusion of Right Shoulder Joint with Int Fix, Perc Approach */
"0RGJ37Z"  =  "1"  /* Fusion of Right Shoulder Joint with Autol Sub, Perc Approach */
"0RGJ3JZ"  =  "1"  /* Fusion of Right Shoulder Joint with Synth Sub, Perc Approach */
"0RGJ3KZ"  =  "1"  /* Fusion of R Shoulder Jt with Nonaut Sub, Perc Approach */
"0RGJ3ZZ"  =  "1"  /* Fusion of Right Shoulder Joint, Percutaneous Approach */
"0RGJ44Z"  =  "1"  /* Fusion of R Shoulder Jt with Int Fix, Perc Endo Approach */
"0RGJ47Z"  =  "1"  /* Fusion of R Shoulder Jt with Autol Sub, Perc Endo Approach */
"0RGJ4JZ"  =  "1"  /* Fusion of R Shoulder Jt with Synth Sub, Perc Endo Approach */
"0RGJ4KZ"  =  "1"  /* Fusion of R Shoulder Jt with Nonaut Sub, Perc Endo Approach */
"0RGJ4ZZ"  =  "1"  /* Fusion of Right Shoulder Joint, Perc Endo Approach */
"0RGK04Z"  =  "1"  /* Fusion of Left Shoulder Joint with Int Fix, Open Approach */
"0RGK07Z"  =  "1"  /* Fusion of Left Shoulder Joint with Autol Sub, Open Approach */
"0RGK0JZ"  =  "1"  /* Fusion of Left Shoulder Joint with Synth Sub, Open Approach */
"0RGK0KZ"  =  "1"  /* Fusion of Left Shoulder Joint with Nonaut Sub, Open Approach */
"0RGK0ZZ"  =  "1"  /* Fusion of Left Shoulder Joint, Open Approach */
"0RGK34Z"  =  "1"  /* Fusion of Left Shoulder Joint with Int Fix, Perc Approach */
"0RGK37Z"  =  "1"  /* Fusion of Left Shoulder Joint with Autol Sub, Perc Approach */
"0RGK3JZ"  =  "1"  /* Fusion of Left Shoulder Joint with Synth Sub, Perc Approach */
"0RGK3KZ"  =  "1"  /* Fusion of Left Shoulder Joint with Nonaut Sub, Perc Approach */
"0RGK3ZZ"  =  "1"  /* Fusion of Left Shoulder Joint, Percutaneous Approach */
"0RGK44Z"  =  "1"  /* Fusion of L Shoulder Jt with Int Fix, Perc Endo Approach */
"0RGK47Z"  =  "1"  /* Fusion of L Shoulder Jt with Autol Sub, Perc Endo Approach */
"0RGK4JZ"  =  "1"  /* Fusion of L Shoulder Jt with Synth Sub, Perc Endo Approach */
"0RGK4KZ"  =  "1"  /* Fusion of L Shoulder Jt with Nonaut Sub, Perc Endo Approach */
"0RGK4ZZ"  =  "1"  /* Fusion of Left Shoulder Joint, Perc Endo Approach */
"0RGL04Z"  =  "1"  /* Fusion of Right Elbow Joint with Int Fix, Open Approach */
"0RGL05Z"  =  "1"  /* Fusion of Right Elbow Joint with Ext Fix, Open Approach */
"0RGL07Z"  =  "1"  /* Fusion of Right Elbow Joint with Autol Sub, Open Approach */
"0RGL0JZ"  =  "1"  /* Fusion of Right Elbow Joint with Synth Sub, Open Approach */
"0RGL0KZ"  =  "1"  /* Fusion of Right Elbow Joint with Nonaut Sub, Open Approach */
"0RGL0ZZ"  =  "1"  /* Fusion of Right Elbow Joint, Open Approach */
"0RGL34Z"  =  "1"  /* Fusion of Right Elbow Joint with Int Fix, Perc Approach */
"0RGL35Z"  =  "1"  /* Fusion of Right Elbow Joint with Ext Fix, Perc Approach */
"0RGL37Z"  =  "1"  /* Fusion of Right Elbow Joint with Autol Sub, Perc Approach */
"0RGL3JZ"  =  "1"  /* Fusion of Right Elbow Joint with Synth Sub, Perc Approach */
"0RGL3KZ"  =  "1"  /* Fusion of Right Elbow Joint with Nonaut Sub, Perc Approach */
"0RGL3ZZ"  =  "1"  /* Fusion of Right Elbow Joint, Percutaneous Approach */
"0RGL44Z"  =  "1"  /* Fusion of Right Elbow Joint with Int Fix, Perc Endo Approach */
"0RGL45Z"  =  "1"  /* Fusion of Right Elbow Joint with Ext Fix, Perc Endo Approach */
"0RGL47Z"  =  "1"  /* Fusion of R Elbow Jt with Autol Sub, Perc Endo Approach */
"0RGL4JZ"  =  "1"  /* Fusion of R Elbow Jt with Synth Sub, Perc Endo Approach */
"0RGL4KZ"  =  "1"  /* Fusion of R Elbow Jt with Nonaut Sub, Perc Endo Approach */
"0RGL4ZZ"  =  "1"  /* Fusion of Right Elbow Joint, Perc Endo Approach */
"0RGM04Z"  =  "1"  /* Fusion of Left Elbow Joint with Int Fix, Open Approach */
"0RGM05Z"  =  "1"  /* Fusion of Left Elbow Joint with Ext Fix, Open Approach */
"0RGM07Z"  =  "1"  /* Fusion of Left Elbow Joint with Autol Sub, Open Approach */
"0RGM0JZ"  =  "1"  /* Fusion of Left Elbow Joint with Synth Sub, Open Approach */
"0RGM0KZ"  =  "1"  /* Fusion of Left Elbow Joint with Nonaut Sub, Open Approach */
"0RGM0ZZ"  =  "1"  /* Fusion of Left Elbow Joint, Open Approach */
"0RGM34Z"  =  "1"  /* Fusion of Left Elbow Joint with Int Fix, Perc Approach */
"0RGM35Z"  =  "1"  /* Fusion of Left Elbow Joint with Ext Fix, Perc Approach */
"0RGM37Z"  =  "1"  /* Fusion of Left Elbow Joint with Autol Sub, Perc Approach */
"0RGM3JZ"  =  "1"  /* Fusion of Left Elbow Joint with Synth Sub, Perc Approach */
"0RGM3KZ"  =  "1"  /* Fusion of Left Elbow Joint with Nonaut Sub, Perc Approach */
"0RGM3ZZ"  =  "1"  /* Fusion of Left Elbow Joint, Percutaneous Approach */
"0RGM44Z"  =  "1"  /* Fusion of Left Elbow Joint with Int Fix, Perc Endo Approach */
"0RGM45Z"  =  "1"  /* Fusion of Left Elbow Joint with Ext Fix, Perc Endo Approach */
"0RGM47Z"  =  "1"  /* Fusion of L Elbow Jt with Autol Sub, Perc Endo Approach */
"0RGM4JZ"  =  "1"  /* Fusion of L Elbow Jt with Synth Sub, Perc Endo Approach */
"0RGM4KZ"  =  "1"  /* Fusion of L Elbow Jt with Nonaut Sub, Perc Endo Approach */
"0RGM4ZZ"  =  "1"  /* Fusion of Left Elbow Joint, Percutaneous Endoscopic Approach */
"0RGN04Z"  =  "1"  /* Fusion of Right Wrist Joint with Int Fix, Open Approach */
"0RGN05Z"  =  "1"  /* Fusion of Right Wrist Joint with Ext Fix, Open Approach */
"0RGN07Z"  =  "1"  /* Fusion of Right Wrist Joint with Autol Sub, Open Approach */
"0RGN0JZ"  =  "1"  /* Fusion of Right Wrist Joint with Synth Sub, Open Approach */
"0RGN0KZ"  =  "1"  /* Fusion of Right Wrist Joint with Nonaut Sub, Open Approach */
"0RGN0ZZ"  =  "1"  /* Fusion of Right Wrist Joint, Open Approach */
"0RGN34Z"  =  "1"  /* Fusion of Right Wrist Joint with Int Fix, Perc Approach */
"0RGN35Z"  =  "1"  /* Fusion of Right Wrist Joint with Ext Fix, Perc Approach */
"0RGN37Z"  =  "1"  /* Fusion of Right Wrist Joint with Autol Sub, Perc Approach */
"0RGN3JZ"  =  "1"  /* Fusion of Right Wrist Joint with Synth Sub, Perc Approach */
"0RGN3KZ"  =  "1"  /* Fusion of Right Wrist Joint with Nonaut Sub, Perc Approach */
"0RGN3ZZ"  =  "1"  /* Fusion of Right Wrist Joint, Percutaneous Approach */
"0RGN44Z"  =  "1"  /* Fusion of Right Wrist Joint with Int Fix, Perc Endo Approach */
"0RGN45Z"  =  "1"  /* Fusion of Right Wrist Joint with Ext Fix, Perc Endo Approach */
"0RGN47Z"  =  "1"  /* Fusion of R Wrist Jt with Autol Sub, Perc Endo Approach */
"0RGN4JZ"  =  "1"  /* Fusion of R Wrist Jt with Synth Sub, Perc Endo Approach */
"0RGN4KZ"  =  "1"  /* Fusion of R Wrist Jt with Nonaut Sub, Perc Endo Approach */
"0RGN4ZZ"  =  "1"  /* Fusion of Right Wrist Joint, Perc Endo Approach */
"0RGP04Z"  =  "1"  /* Fusion of Left Wrist Joint with Int Fix, Open Approach */
"0RGP05Z"  =  "1"  /* Fusion of Left Wrist Joint with Ext Fix, Open Approach */
"0RGP07Z"  =  "1"  /* Fusion of Left Wrist Joint with Autol Sub, Open Approach */
"0RGP0JZ"  =  "1"  /* Fusion of Left Wrist Joint with Synth Sub, Open Approach */
"0RGP0KZ"  =  "1"  /* Fusion of Left Wrist Joint with Nonaut Sub, Open Approach */
"0RGP0ZZ"  =  "1"  /* Fusion of Left Wrist Joint, Open Approach */
"0RGP34Z"  =  "1"  /* Fusion of Left Wrist Joint with Int Fix, Perc Approach */
"0RGP35Z"  =  "1"  /* Fusion of Left Wrist Joint with Ext Fix, Perc Approach */
"0RGP37Z"  =  "1"  /* Fusion of Left Wrist Joint with Autol Sub, Perc Approach */
"0RGP3JZ"  =  "1"  /* Fusion of Left Wrist Joint with Synth Sub, Perc Approach */
"0RGP3KZ"  =  "1"  /* Fusion of Left Wrist Joint with Nonaut Sub, Perc Approach */
"0RGP3ZZ"  =  "1"  /* Fusion of Left Wrist Joint, Percutaneous Approach */
"0RGP44Z"  =  "1"  /* Fusion of Left Wrist Joint with Int Fix, Perc Endo Approach */
"0RGP45Z"  =  "1"  /* Fusion of Left Wrist Joint with Ext Fix, Perc Endo Approach */
"0RGP47Z"  =  "1"  /* Fusion of L Wrist Jt with Autol Sub, Perc Endo Approach */
"0RGP4JZ"  =  "1"  /* Fusion of L Wrist Jt with Synth Sub, Perc Endo Approach */
"0RGP4KZ"  =  "1"  /* Fusion of L Wrist Jt with Nonaut Sub, Perc Endo Approach */
"0RGP4ZZ"  =  "1"  /* Fusion of Left Wrist Joint, Percutaneous Endoscopic Approach */
"0RGQ04Z"  =  "1"  /* Fusion of Right Carpal Joint with Int Fix, Open Approach */
"0RGQ05Z"  =  "1"  /* Fusion of Right Carpal Joint with Ext Fix, Open Approach */
"0RGQ07Z"  =  "1"  /* Fusion of Right Carpal Joint with Autol Sub, Open Approach */
"0RGQ0JZ"  =  "1"  /* Fusion of Right Carpal Joint with Synth Sub, Open Approach */
"0RGQ0KZ"  =  "1"  /* Fusion of Right Carpal Joint with Nonaut Sub, Open Approach */
"0RGQ0ZZ"  =  "1"  /* Fusion of Right Carpal Joint, Open Approach */
"0RGQ34Z"  =  "1"  /* Fusion of Right Carpal Joint with Int Fix, Perc Approach */
"0RGQ35Z"  =  "1"  /* Fusion of Right Carpal Joint with Ext Fix, Perc Approach */
"0RGQ37Z"  =  "1"  /* Fusion of Right Carpal Joint with Autol Sub, Perc Approach */
"0RGQ3JZ"  =  "1"  /* Fusion of Right Carpal Joint with Synth Sub, Perc Approach */
"0RGQ3KZ"  =  "1"  /* Fusion of Right Carpal Joint with Nonaut Sub, Perc Approach */
"0RGQ3ZZ"  =  "1"  /* Fusion of Right Carpal Joint, Percutaneous Approach */
"0RGQ44Z"  =  "1"  /* Fusion of R Carpal Jt with Int Fix, Perc Endo Approach */
"0RGQ45Z"  =  "1"  /* Fusion of R Carpal Jt with Ext Fix, Perc Endo Approach */
"0RGQ47Z"  =  "1"  /* Fusion of R Carpal Jt with Autol Sub, Perc Endo Approach */
"0RGQ4JZ"  =  "1"  /* Fusion of R Carpal Jt with Synth Sub, Perc Endo Approach */
"0RGQ4KZ"  =  "1"  /* Fusion of R Carpal Jt with Nonaut Sub, Perc Endo Approach */
"0RGQ4ZZ"  =  "1"  /* Fusion of Right Carpal Joint, Perc Endo Approach */
"0RGR04Z"  =  "1"  /* Fusion of Left Carpal Joint with Int Fix, Open Approach */
"0RGR05Z"  =  "1"  /* Fusion of Left Carpal Joint with Ext Fix, Open Approach */
"0RGR07Z"  =  "1"  /* Fusion of Left Carpal Joint with Autol Sub, Open Approach */
"0RGR0JZ"  =  "1"  /* Fusion of Left Carpal Joint with Synth Sub, Open Approach */
"0RGR0KZ"  =  "1"  /* Fusion of Left Carpal Joint with Nonaut Sub, Open Approach */
"0RGR0ZZ"  =  "1"  /* Fusion of Left Carpal Joint, Open Approach */
"0RGR34Z"  =  "1"  /* Fusion of Left Carpal Joint with Int Fix, Perc Approach */
"0RGR35Z"  =  "1"  /* Fusion of Left Carpal Joint with Ext Fix, Perc Approach */
"0RGR37Z"  =  "1"  /* Fusion of Left Carpal Joint with Autol Sub, Perc Approach */
"0RGR3JZ"  =  "1"  /* Fusion of Left Carpal Joint with Synth Sub, Perc Approach */
"0RGR3KZ"  =  "1"  /* Fusion of Left Carpal Joint with Nonaut Sub, Perc Approach */
"0RGR3ZZ"  =  "1"  /* Fusion of Left Carpal Joint, Percutaneous Approach */
"0RGR44Z"  =  "1"  /* Fusion of Left Carpal Joint with Int Fix, Perc Endo Approach */
"0RGR45Z"  =  "1"  /* Fusion of Left Carpal Joint with Ext Fix, Perc Endo Approach */
"0RGR47Z"  =  "1"  /* Fusion of L Carpal Jt with Autol Sub, Perc Endo Approach */
"0RGR4JZ"  =  "1"  /* Fusion of L Carpal Jt with Synth Sub, Perc Endo Approach */
"0RGR4KZ"  =  "1"  /* Fusion of L Carpal Jt with Nonaut Sub, Perc Endo Approach */
"0RGR4ZZ"  =  "1"  /* Fusion of Left Carpal Joint, Perc Endo Approach */
"0RGS04Z"  =  "1"  /* Fusion of R Metacarpocarp Jt with Int Fix, Open Approach */
"0RGS05Z"  =  "1"  /* Fusion of R Metacarpocarp Jt with Ext Fix, Open Approach */
"0RGS07Z"  =  "1"  /* Fusion of R Metacarpocarp Jt with Autol Sub, Open Approach */
"0RGS0JZ"  =  "1"  /* Fusion of R Metacarpocarp Jt with Synth Sub, Open Approach */
"0RGS0KZ"  =  "1"  /* Fusion of R Metacarpocarp Jt with Nonaut Sub, Open Approach */
"0RGS0ZZ"  =  "1"  /* Fusion of Right Metacarpocarpal Joint, Open Approach */
"0RGS34Z"  =  "1"  /* Fusion of R Metacarpocarp Jt with Int Fix, Perc Approach */
"0RGS35Z"  =  "1"  /* Fusion of R Metacarpocarp Jt with Ext Fix, Perc Approach */
"0RGS37Z"  =  "1"  /* Fusion of R Metacarpocarp Jt with Autol Sub, Perc Approach */
"0RGS3JZ"  =  "1"  /* Fusion of R Metacarpocarp Jt with Synth Sub, Perc Approach */
"0RGS3KZ"  =  "1"  /* Fusion of R Metacarpocarp Jt with Nonaut Sub, Perc Approach */
"0RGS3ZZ"  =  "1"  /* Fusion of Right Metacarpocarpal Joint, Percutaneous Approach */
"0RGS44Z"  =  "1"  /* Fusion R Metacarpocarp Jt w Int Fix, Perc Endo */
"0RGS45Z"  =  "1"  /* Fusion R Metacarpocarp Jt w Ext Fix, Perc Endo */
"0RGS47Z"  =  "1"  /* Fusion R Metacarpocarp Jt w Autol Sub, Perc Endo */
"0RGS4JZ"  =  "1"  /* Fusion R Metacarpocarp Jt w Synth Sub, Perc Endo */
"0RGS4KZ"  =  "1"  /* Fusion R Metacarpocarp Jt w Nonaut Sub, Perc Endo */
"0RGS4ZZ"  =  "1"  /* Fusion of Right Metacarpocarpal Joint, Perc Endo Approach */
"0RGT04Z"  =  "1"  /* Fusion of L Metacarpocarp Jt with Int Fix, Open Approach */
"0RGT05Z"  =  "1"  /* Fusion of L Metacarpocarp Jt with Ext Fix, Open Approach */
"0RGT07Z"  =  "1"  /* Fusion of L Metacarpocarp Jt with Autol Sub, Open Approach */
"0RGT0JZ"  =  "1"  /* Fusion of L Metacarpocarp Jt with Synth Sub, Open Approach */
"0RGT0KZ"  =  "1"  /* Fusion of L Metacarpocarp Jt with Nonaut Sub, Open Approach */
"0RGT0ZZ"  =  "1"  /* Fusion of Left Metacarpocarpal Joint, Open Approach */
"0RGT34Z"  =  "1"  /* Fusion of L Metacarpocarp Jt with Int Fix, Perc Approach */
"0RGT35Z"  =  "1"  /* Fusion of L Metacarpocarp Jt with Ext Fix, Perc Approach */
"0RGT37Z"  =  "1"  /* Fusion of L Metacarpocarp Jt with Autol Sub, Perc Approach */
"0RGT3JZ"  =  "1"  /* Fusion of L Metacarpocarp Jt with Synth Sub, Perc Approach */
"0RGT3KZ"  =  "1"  /* Fusion of L Metacarpocarp Jt with Nonaut Sub, Perc Approach */
"0RGT3ZZ"  =  "1"  /* Fusion of Left Metacarpocarpal Joint, Percutaneous Approach */
"0RGT44Z"  =  "1"  /* Fusion L Metacarpocarp Jt w Int Fix, Perc Endo */
"0RGT45Z"  =  "1"  /* Fusion L Metacarpocarp Jt w Ext Fix, Perc Endo */
"0RGT47Z"  =  "1"  /* Fusion L Metacarpocarp Jt w Autol Sub, Perc Endo */
"0RGT4JZ"  =  "1"  /* Fusion L Metacarpocarp Jt w Synth Sub, Perc Endo */
"0RGT4KZ"  =  "1"  /* Fusion L Metacarpocarp Jt w Nonaut Sub, Perc Endo */
"0RGT4ZZ"  =  "1"  /* Fusion of Left Metacarpocarpal Joint, Perc Endo Approach */
"0RGU04Z"  =  "1"  /* Fusion of R Metacarpophal Jt with Int Fix, Open Approach */
"0RGU05Z"  =  "1"  /* Fusion of R Metacarpophal Jt with Ext Fix, Open Approach */
"0RGU07Z"  =  "1"  /* Fusion of R Metacarpophal Jt with Autol Sub, Open Approach */
"0RGU0JZ"  =  "1"  /* Fusion of R Metacarpophal Jt with Synth Sub, Open Approach */
"0RGU0KZ"  =  "1"  /* Fusion of R Metacarpophal Jt with Nonaut Sub, Open Approach */
"0RGU0ZZ"  =  "1"  /* Fusion of Right Metacarpophalangeal Joint, Open Approach */
"0RGU34Z"  =  "1"  /* Fusion of R Metacarpophal Jt with Int Fix, Perc Approach */
"0RGU35Z"  =  "1"  /* Fusion of R Metacarpophal Jt with Ext Fix, Perc Approach */
"0RGU37Z"  =  "1"  /* Fusion of R Metacarpophal Jt with Autol Sub, Perc Approach */
"0RGU3JZ"  =  "1"  /* Fusion of R Metacarpophal Jt with Synth Sub, Perc Approach */
"0RGU3KZ"  =  "1"  /* Fusion of R Metacarpophal Jt with Nonaut Sub, Perc Approach */
"0RGU3ZZ"  =  "1"  /* Fusion of Right Metacarpophalangeal Joint, Perc Approach */
"0RGU44Z"  =  "1"  /* Fusion R Metacarpophal Jt w Int Fix, Perc Endo */
"0RGU45Z"  =  "1"  /* Fusion R Metacarpophal Jt w Ext Fix, Perc Endo */
"0RGU47Z"  =  "1"  /* Fusion R Metacarpophal Jt w Autol Sub, Perc Endo */
"0RGU4JZ"  =  "1"  /* Fusion R Metacarpophal Jt w Synth Sub, Perc Endo */
"0RGU4KZ"  =  "1"  /* Fusion R Metacarpophal Jt w Nonaut Sub, Perc Endo */
"0RGU4ZZ"  =  "1"  /* Fusion of R Metacarpophal Jt, Perc Endo Approach */
"0RGV04Z"  =  "1"  /* Fusion of L Metacarpophal Jt with Int Fix, Open Approach */
"0RGV05Z"  =  "1"  /* Fusion of L Metacarpophal Jt with Ext Fix, Open Approach */
"0RGV07Z"  =  "1"  /* Fusion of L Metacarpophal Jt with Autol Sub, Open Approach */
"0RGV0JZ"  =  "1"  /* Fusion of L Metacarpophal Jt with Synth Sub, Open Approach */
"0RGV0KZ"  =  "1"  /* Fusion of L Metacarpophal Jt with Nonaut Sub, Open Approach */
"0RGV0ZZ"  =  "1"  /* Fusion of Left Metacarpophalangeal Joint, Open Approach */
"0RGV34Z"  =  "1"  /* Fusion of L Metacarpophal Jt with Int Fix, Perc Approach */
"0RGV35Z"  =  "1"  /* Fusion of L Metacarpophal Jt with Ext Fix, Perc Approach */
"0RGV37Z"  =  "1"  /* Fusion of L Metacarpophal Jt with Autol Sub, Perc Approach */
"0RGV3JZ"  =  "1"  /* Fusion of L Metacarpophal Jt with Synth Sub, Perc Approach */
"0RGV3KZ"  =  "1"  /* Fusion of L Metacarpophal Jt with Nonaut Sub, Perc Approach */
"0RGV3ZZ"  =  "1"  /* Fusion of Left Metacarpophalangeal Joint, Perc Approach */
"0RGV44Z"  =  "1"  /* Fusion L Metacarpophal Jt w Int Fix, Perc Endo */
"0RGV45Z"  =  "1"  /* Fusion L Metacarpophal Jt w Ext Fix, Perc Endo */
"0RGV47Z"  =  "1"  /* Fusion L Metacarpophal Jt w Autol Sub, Perc Endo */
"0RGV4JZ"  =  "1"  /* Fusion L Metacarpophal Jt w Synth Sub, Perc Endo */
"0RGV4KZ"  =  "1"  /* Fusion L Metacarpophal Jt w Nonaut Sub, Perc Endo */
"0RGV4ZZ"  =  "1"  /* Fusion of Left Metacarpophalangeal Joint, Perc Endo Approach */
"0RGW04Z"  =  "1"  /* Fusion of R Finger Phalanx Jt with Int Fix, Open Approach */
"0RGW05Z"  =  "1"  /* Fusion of R Finger Phalanx Jt with Ext Fix, Open Approach */
"0RGW07Z"  =  "1"  /* Fusion of R Finger Phalanx Jt with Autol Sub, Open Approach */
"0RGW0JZ"  =  "1"  /* Fusion of R Finger Phalanx Jt with Synth Sub, Open Approach */
"0RGW0KZ"  =  "1"  /* Fusion of R Finger Phalanx Jt with Nonaut Sub, Open Approach */
"0RGW0ZZ"  =  "1"  /* Fusion of Right Finger Phalangeal Joint, Open Approach */
"0RGW34Z"  =  "1"  /* Fusion of R Finger Phalanx Jt with Int Fix, Perc Approach */
"0RGW35Z"  =  "1"  /* Fusion of R Finger Phalanx Jt with Ext Fix, Perc Approach */
"0RGW37Z"  =  "1"  /* Fusion of R Finger Phalanx Jt with Autol Sub, Perc Approach */
"0RGW3JZ"  =  "1"  /* Fusion of R Finger Phalanx Jt with Synth Sub, Perc Approach */
"0RGW3KZ"  =  "1"  /* Fusion of R Finger Phalanx Jt with Nonaut Sub, Perc Approach */
"0RGW3ZZ"  =  "1"  /* Fusion of Right Finger Phalangeal Joint, Perc Approach */
"0RGW44Z"  =  "1"  /* Fusion R Finger Phalanx Jt w Int Fix, Perc Endo */
"0RGW45Z"  =  "1"  /* Fusion R Finger Phalanx Jt w Ext Fix, Perc Endo */
"0RGW47Z"  =  "1"  /* Fusion R Finger Phalanx Jt w Autol Sub, Perc Endo */
"0RGW4JZ"  =  "1"  /* Fusion R Finger Phalanx Jt w Synth Sub, Perc Endo */
"0RGW4KZ"  =  "1"  /* Fusion R Finger Phalanx Jt w Nonaut Sub, Perc Endo */
"0RGW4ZZ"  =  "1"  /* Fusion of Right Finger Phalangeal Joint, Perc Endo Approach */
"0RGX04Z"  =  "1"  /* Fusion of L Finger Phalanx Jt with Int Fix, Open Approach */
"0RGX05Z"  =  "1"  /* Fusion of L Finger Phalanx Jt with Ext Fix, Open Approach */
"0RGX07Z"  =  "1"  /* Fusion of L Finger Phalanx Jt with Autol Sub, Open Approach */
"0RGX0JZ"  =  "1"  /* Fusion of L Finger Phalanx Jt with Synth Sub, Open Approach */
"0RGX0KZ"  =  "1"  /* Fusion of L Finger Phalanx Jt with Nonaut Sub, Open Approach */
"0RGX0ZZ"  =  "1"  /* Fusion of Left Finger Phalangeal Joint, Open Approach */
"0RGX34Z"  =  "1"  /* Fusion of L Finger Phalanx Jt with Int Fix, Perc Approach */
"0RGX35Z"  =  "1"  /* Fusion of L Finger Phalanx Jt with Ext Fix, Perc Approach */
"0RGX37Z"  =  "1"  /* Fusion of L Finger Phalanx Jt with Autol Sub, Perc Approach */
"0RGX3JZ"  =  "1"  /* Fusion of L Finger Phalanx Jt with Synth Sub, Perc Approach */
"0RGX3KZ"  =  "1"  /* Fusion of L Finger Phalanx Jt with Nonaut Sub, Perc Approach */
"0RGX3ZZ"  =  "1"  /* Fusion of Left Finger Phalangeal Joint, Perc Approach */
"0RGX44Z"  =  "1"  /* Fusion L Finger Phalanx Jt w Int Fix, Perc Endo */
"0RGX45Z"  =  "1"  /* Fusion L Finger Phalanx Jt w Ext Fix, Perc Endo */
"0RGX47Z"  =  "1"  /* Fusion L Finger Phalanx Jt w Autol Sub, Perc Endo */
"0RGX4JZ"  =  "1"  /* Fusion L Finger Phalanx Jt w Synth Sub, Perc Endo */
"0RGX4KZ"  =  "1"  /* Fusion L Finger Phalanx Jt w Nonaut Sub, Perc Endo */
"0RGX4ZZ"  =  "1"  /* Fusion of Left Finger Phalangeal Joint, Perc Endo Approach */
"0RH004Z"  =  "1"  /* Insertion of Int Fix into Occip Jt, Open Approach */
"0RH00BZ"  =  "1"  /* Insert Intspin Prcs Stabl Dev in Occip Jt, Open */
"0RH00CZ"  =  "1"  /* Insertion of Pedcl Stabl Dev into Occip Jt, Open Approach */
"0RH00DZ"  =  "1"  /* Insertion of Facet Stabl Dev into Occip Jt, Open Approach */
"0RH034Z"  =  "1"  /* Insertion of Int Fix into Occip Jt, Perc Approach */
"0RH03BZ"  =  "1"  /* Insert Intspin Prcs Stabl Dev in Occip Jt, Perc */
"0RH03CZ"  =  "1"  /* Insertion of Pedcl Stabl Dev into Occip Jt, Perc Approach */
"0RH03DZ"  =  "1"  /* Insertion of Facet Stabl Dev into Occip Jt, Perc Approach */
"0RH044Z"  =  "1"  /* Insertion of Int Fix into Occip Jt, Perc Endo Approach */
"0RH04BZ"  =  "1"  /* Insert Intspin Prcs Stabl Dev in Occip Jt, Perc Endo */
"0RH04CZ"  =  "1"  /* Insert of Pedcl Stabl Dev into Occip Jt, Perc Endo Approach */
"0RH04DZ"  =  "1"  /* Insert of Facet Stabl Dev into Occip Jt, Perc Endo Approach */
"0RH104Z"  =  "1"  /* Insertion of Int Fix into Cerv Jt, Open Approach */
"0RH10BZ"  =  "1"  /* Insert of Intspin Prcs Stabl Dev into Cerv Jt, Open Approach */
"0RH10CZ"  =  "1"  /* Insertion of Pedcl Stabl Dev into Cerv Jt, Open Approach */
"0RH10DZ"  =  "1"  /* Insertion of Facet Stabl Dev into Cerv Jt, Open Approach */
"0RH134Z"  =  "1"  /* Insertion of Int Fix into Cerv Jt, Perc Approach */
"0RH13BZ"  =  "1"  /* Insert of Intspin Prcs Stabl Dev into Cerv Jt, Perc Approach */
"0RH13CZ"  =  "1"  /* Insertion of Pedcl Stabl Dev into Cerv Jt, Perc Approach */
"0RH13DZ"  =  "1"  /* Insertion of Facet Stabl Dev into Cerv Jt, Perc Approach */
"0RH144Z"  =  "1"  /* Insertion of Int Fix into Cerv Jt, Perc Endo Approach */
"0RH14BZ"  =  "1"  /* Insert Intspin Prcs Stabl Dev in Cerv Jt, Perc Endo */
"0RH14CZ"  =  "1"  /* Insert of Pedcl Stabl Dev into Cerv Jt, Perc Endo Approach */
"0RH14DZ"  =  "1"  /* Insert of Facet Stabl Dev into Cerv Jt, Perc Endo Approach */
"0RH404Z"  =  "1"  /* Insertion of Int Fix into C-thor Jt, Open Approach */
"0RH40BZ"  =  "1"  /* Insert Intspin Prcs Stabl Dev in C-thor Jt, Open */
"0RH40CZ"  =  "1"  /* Insertion of Pedcl Stabl Dev into C-thor Jt, Open Approach */
"0RH40DZ"  =  "1"  /* Insertion of Facet Stabl Dev into C-thor Jt, Open Approach */
"0RH434Z"  =  "1"  /* Insertion of Int Fix into C-thor Jt, Perc Approach */
"0RH43BZ"  =  "1"  /* Insert Intspin Prcs Stabl Dev in C-thor Jt, Perc */
"0RH43CZ"  =  "1"  /* Insertion of Pedcl Stabl Dev into C-thor Jt, Perc Approach */
"0RH43DZ"  =  "1"  /* Insertion of Facet Stabl Dev into C-thor Jt, Perc Approach */
"0RH444Z"  =  "1"  /* Insertion of Int Fix into C-thor Jt, Perc Endo Approach */
"0RH44BZ"  =  "1"  /* Insert Intspin Prcs Stabl Dev in C-thor Jt, Perc Endo */
"0RH44CZ"  =  "1"  /* Insert of Pedcl Stabl Dev into C-thor Jt, Perc Endo Approach */
"0RH44DZ"  =  "1"  /* Insert of Facet Stabl Dev into C-thor Jt, Perc Endo Approach */
"0RH604Z"  =  "1"  /* Insertion of Int Fix into Thor Jt, Open Approach */
"0RH60BZ"  =  "1"  /* Insert of Intspin Prcs Stabl Dev into Thor Jt, Open Approach */
"0RH60CZ"  =  "1"  /* Insertion of Pedcl Stabl Dev into Thor Jt, Open Approach */
"0RH60DZ"  =  "1"  /* Insertion of Facet Stabl Dev into Thor Jt, Open Approach */
"0RH634Z"  =  "1"  /* Insertion of Int Fix into Thor Jt, Perc Approach */
"0RH63BZ"  =  "1"  /* Insert of Intspin Prcs Stabl Dev into Thor Jt, Perc Approach */
"0RH63CZ"  =  "1"  /* Insertion of Pedcl Stabl Dev into Thor Jt, Perc Approach */
"0RH63DZ"  =  "1"  /* Insertion of Facet Stabl Dev into Thor Jt, Perc Approach */
"0RH644Z"  =  "1"  /* Insertion of Int Fix into Thor Jt, Perc Endo Approach */
"0RH64BZ"  =  "1"  /* Insert Intspin Prcs Stabl Dev in Thor Jt, Perc Endo */
"0RH64CZ"  =  "1"  /* Insert of Pedcl Stabl Dev into Thor Jt, Perc Endo Approach */
"0RH64DZ"  =  "1"  /* Insert of Facet Stabl Dev into Thor Jt, Perc Endo Approach */
"0RHA04Z"  =  "1"  /* Insertion of Int Fix into T-lum Jt, Open Approach */
"0RHA0BZ"  =  "1"  /* Insert Intspin Prcs Stabl Dev in T-lum Jt, Open */
"0RHA0CZ"  =  "1"  /* Insertion of Pedcl Stabl Dev into T-lum Jt, Open Approach */
"0RHA0DZ"  =  "1"  /* Insertion of Facet Stabl Dev into T-lum Jt, Open Approach */
"0RHA34Z"  =  "1"  /* Insertion of Int Fix into T-lum Jt, Perc Approach */
"0RHA3BZ"  =  "1"  /* Insert Intspin Prcs Stabl Dev in T-lum Jt, Perc */
"0RHA3CZ"  =  "1"  /* Insertion of Pedcl Stabl Dev into T-lum Jt, Perc Approach */
"0RHA3DZ"  =  "1"  /* Insertion of Facet Stabl Dev into T-lum Jt, Perc Approach */
"0RHA44Z"  =  "1"  /* Insertion of Int Fix into T-lum Jt, Perc Endo Approach */
"0RHA4BZ"  =  "1"  /* Insert Intspin Prcs Stabl Dev in T-lum Jt, Perc Endo */
"0RHA4CZ"  =  "1"  /* Insert of Pedcl Stabl Dev into T-lum Jt, Perc Endo Approach */
"0RHA4DZ"  =  "1"  /* Insert of Facet Stabl Dev into T-lum Jt, Perc Endo Approach */
"0RHC03Z"  =  "1"  /* Insert Infusion Dev in R Temporomandib Jt, Open */
"0RHC04Z"  =  "1"  /* Insertion of Int Fix into R Temporomandib Jt, Open Approach */
"0RHC33Z"  =  "1"  /* Insert Infusion Dev in R Temporomandib Jt, Perc */
"0RHC34Z"  =  "1"  /* Insertion of Int Fix into R Temporomandib Jt, Perc Approach */
"0RHC43Z"  =  "1"  /* Insert Infusion Dev in R Temporomandib Jt, Perc Endo */
"0RHC44Z"  =  "1"  /* Insert Int Fix in R Temporomandib Jt, Perc Endo */
"0RHD03Z"  =  "1"  /* Insert Infusion Dev in L Temporomandib Jt, Open */
"0RHD04Z"  =  "1"  /* Insertion of Int Fix into L Temporomandib Jt, Open Approach */
"0RHD33Z"  =  "1"  /* Insert Infusion Dev in L Temporomandib Jt, Perc */
"0RHD34Z"  =  "1"  /* Insertion of Int Fix into L Temporomandib Jt, Perc Approach */
"0RHD43Z"  =  "1"  /* Insert Infusion Dev in L Temporomandib Jt, Perc Endo */
"0RHD44Z"  =  "1"  /* Insert Int Fix in L Temporomandib Jt, Perc Endo */
"0RHE04Z"  =  "1"  /* Insertion of Int Fix into R Sternoclav Jt, Open Approach */
"0RHE34Z"  =  "1"  /* Insertion of Int Fix into R Sternoclav Jt, Perc Approach */
"0RHE44Z"  =  "1"  /* Insert of Int Fix into R Sternoclav Jt, Perc Endo Approach */
"0RHF04Z"  =  "1"  /* Insertion of Int Fix into L Sternoclav Jt, Open Approach */
"0RHF34Z"  =  "1"  /* Insertion of Int Fix into L Sternoclav Jt, Perc Approach */
"0RHF44Z"  =  "1"  /* Insert of Int Fix into L Sternoclav Jt, Perc Endo Approach */
"0RHG04Z"  =  "1"  /* Insertion of Int Fix into R Acromioclav Jt, Open Approach */
"0RHG34Z"  =  "1"  /* Insertion of Int Fix into R Acromioclav Jt, Perc Approach */
"0RHG44Z"  =  "1"  /* Insert of Int Fix into R Acromioclav Jt, Perc Endo Approach */
"0RHH04Z"  =  "1"  /* Insertion of Int Fix into L Acromioclav Jt, Open Approach */
"0RHH34Z"  =  "1"  /* Insertion of Int Fix into L Acromioclav Jt, Perc Approach */
"0RHH44Z"  =  "1"  /* Insert of Int Fix into L Acromioclav Jt, Perc Endo Approach */
"0RHJ04Z"  =  "1"  /* Insertion of Int Fix into R Shoulder Jt, Open Approach */
"0RHJ34Z"  =  "1"  /* Insertion of Int Fix into R Shoulder Jt, Perc Approach */
"0RHJ44Z"  =  "1"  /* Insertion of Int Fix into R Shoulder Jt, Perc Endo Approach */
"0RHK04Z"  =  "1"  /* Insertion of Int Fix into L Shoulder Jt, Open Approach */
"0RHK34Z"  =  "1"  /* Insertion of Int Fix into L Shoulder Jt, Perc Approach */
"0RHK44Z"  =  "1"  /* Insertion of Int Fix into L Shoulder Jt, Perc Endo Approach */
"0RHL04Z"  =  "1"  /* Insertion of Int Fix into R Elbow Jt, Open Approach */
"0RHL05Z"  =  "1"  /* Insertion of Ext Fix into R Elbow Jt, Open Approach */
"0RHL34Z"  =  "1"  /* Insertion of Int Fix into R Elbow Jt, Perc Approach */
"0RHL35Z"  =  "1"  /* Insertion of Ext Fix into R Elbow Jt, Perc Approach */
"0RHL44Z"  =  "1"  /* Insertion of Int Fix into R Elbow Jt, Perc Endo Approach */
"0RHL45Z"  =  "1"  /* Insertion of Ext Fix into R Elbow Jt, Perc Endo Approach */
"0RHM04Z"  =  "1"  /* Insertion of Int Fix into L Elbow Jt, Open Approach */
"0RHM05Z"  =  "1"  /* Insertion of Ext Fix into L Elbow Jt, Open Approach */
"0RHM34Z"  =  "1"  /* Insertion of Int Fix into L Elbow Jt, Perc Approach */
"0RHM35Z"  =  "1"  /* Insertion of Ext Fix into L Elbow Jt, Perc Approach */
"0RHM44Z"  =  "1"  /* Insertion of Int Fix into L Elbow Jt, Perc Endo Approach */
"0RHM45Z"  =  "1"  /* Insertion of Ext Fix into L Elbow Jt, Perc Endo Approach */
"0RHN04Z"  =  "1"  /* Insertion of Int Fix into R Wrist Jt, Open Approach */
"0RHN05Z"  =  "1"  /* Insertion of Ext Fix into R Wrist Jt, Open Approach */
"0RHN34Z"  =  "1"  /* Insertion of Int Fix into R Wrist Jt, Perc Approach */
"0RHN35Z"  =  "1"  /* Insertion of Ext Fix into R Wrist Jt, Perc Approach */
"0RHN44Z"  =  "1"  /* Insertion of Int Fix into R Wrist Jt, Perc Endo Approach */
"0RHN45Z"  =  "1"  /* Insertion of Ext Fix into R Wrist Jt, Perc Endo Approach */
"0RHP04Z"  =  "1"  /* Insertion of Int Fix into L Wrist Jt, Open Approach */
"0RHP05Z"  =  "1"  /* Insertion of Ext Fix into L Wrist Jt, Open Approach */
"0RHP34Z"  =  "1"  /* Insertion of Int Fix into L Wrist Jt, Perc Approach */
"0RHP35Z"  =  "1"  /* Insertion of Ext Fix into L Wrist Jt, Perc Approach */
"0RHP44Z"  =  "1"  /* Insertion of Int Fix into L Wrist Jt, Perc Endo Approach */
"0RHP45Z"  =  "1"  /* Insertion of Ext Fix into L Wrist Jt, Perc Endo Approach */
"0RHQ04Z"  =  "1"  /* Insertion of Int Fix into R Carpal Jt, Open Approach */
"0RHQ05Z"  =  "1"  /* Insertion of Ext Fix into R Carpal Jt, Open Approach */
"0RHQ34Z"  =  "1"  /* Insertion of Int Fix into R Carpal Jt, Perc Approach */
"0RHQ35Z"  =  "1"  /* Insertion of Ext Fix into R Carpal Jt, Perc Approach */
"0RHQ44Z"  =  "1"  /* Insertion of Int Fix into R Carpal Jt, Perc Endo Approach */
"0RHQ45Z"  =  "1"  /* Insertion of Ext Fix into R Carpal Jt, Perc Endo Approach */
"0RHR04Z"  =  "1"  /* Insertion of Int Fix into L Carpal Jt, Open Approach */
"0RHR05Z"  =  "1"  /* Insertion of Ext Fix into L Carpal Jt, Open Approach */
"0RHR34Z"  =  "1"  /* Insertion of Int Fix into L Carpal Jt, Perc Approach */
"0RHR35Z"  =  "1"  /* Insertion of Ext Fix into L Carpal Jt, Perc Approach */
"0RHR44Z"  =  "1"  /* Insertion of Int Fix into L Carpal Jt, Perc Endo Approach */
"0RHR45Z"  =  "1"  /* Insertion of Ext Fix into L Carpal Jt, Perc Endo Approach */
"0RHS04Z"  =  "1"  /* Insertion of Int Fix into R Metacarpocarp Jt, Open Approach */
"0RHS05Z"  =  "1"  /* Insertion of Ext Fix into R Metacarpocarp Jt, Open Approach */
"0RHS34Z"  =  "1"  /* Insertion of Int Fix into R Metacarpocarp Jt, Perc Approach */
"0RHS35Z"  =  "1"  /* Insertion of Ext Fix into R Metacarpocarp Jt, Perc Approach */
"0RHS44Z"  =  "1"  /* Insert Int Fix in R Metacarpocarp Jt, Perc Endo */
"0RHS45Z"  =  "1"  /* Insert Ext Fix in R Metacarpocarp Jt, Perc Endo */
"0RHT04Z"  =  "1"  /* Insertion of Int Fix into L Metacarpocarp Jt, Open Approach */
"0RHT05Z"  =  "1"  /* Insertion of Ext Fix into L Metacarpocarp Jt, Open Approach */
"0RHT34Z"  =  "1"  /* Insertion of Int Fix into L Metacarpocarp Jt, Perc Approach */
"0RHT35Z"  =  "1"  /* Insertion of Ext Fix into L Metacarpocarp Jt, Perc Approach */
"0RHT44Z"  =  "1"  /* Insert Int Fix in L Metacarpocarp Jt, Perc Endo */
"0RHT45Z"  =  "1"  /* Insert Ext Fix in L Metacarpocarp Jt, Perc Endo */
"0RHU04Z"  =  "1"  /* Insertion of Int Fix into R Metacarpophal Jt, Open Approach */
"0RHU05Z"  =  "1"  /* Insertion of Ext Fix into R Metacarpophal Jt, Open Approach */
"0RHU34Z"  =  "1"  /* Insertion of Int Fix into R Metacarpophal Jt, Perc Approach */
"0RHU35Z"  =  "1"  /* Insertion of Ext Fix into R Metacarpophal Jt, Perc Approach */
"0RHU44Z"  =  "1"  /* Insert Int Fix in R Metacarpophal Jt, Perc Endo */
"0RHU45Z"  =  "1"  /* Insert Ext Fix in R Metacarpophal Jt, Perc Endo */
"0RHV04Z"  =  "1"  /* Insertion of Int Fix into L Metacarpophal Jt, Open Approach */
"0RHV05Z"  =  "1"  /* Insertion of Ext Fix into L Metacarpophal Jt, Open Approach */
"0RHV34Z"  =  "1"  /* Insertion of Int Fix into L Metacarpophal Jt, Perc Approach */
"0RHV35Z"  =  "1"  /* Insertion of Ext Fix into L Metacarpophal Jt, Perc Approach */
"0RHV44Z"  =  "1"  /* Insert Int Fix in L Metacarpophal Jt, Perc Endo */
"0RHV45Z"  =  "1"  /* Insert Ext Fix in L Metacarpophal Jt, Perc Endo */
"0RHW04Z"  =  "1"  /* Insertion of Int Fix into R Finger Phalanx Jt, Open Approach */
"0RHW05Z"  =  "1"  /* Insertion of Ext Fix into R Finger Phalanx Jt, Open Approach */
"0RHW34Z"  =  "1"  /* Insertion of Int Fix into R Finger Phalanx Jt, Perc Approach */
"0RHW35Z"  =  "1"  /* Insertion of Ext Fix into R Finger Phalanx Jt, Perc Approach */
"0RHW44Z"  =  "1"  /* Insert Int Fix in R Finger Phalanx Jt, Perc Endo */
"0RHW45Z"  =  "1"  /* Insert Ext Fix in R Finger Phalanx Jt, Perc Endo */
"0RHX04Z"  =  "1"  /* Insertion of Int Fix into L Finger Phalanx Jt, Open Approach */
"0RHX05Z"  =  "1"  /* Insertion of Ext Fix into L Finger Phalanx Jt, Open Approach */
"0RHX34Z"  =  "1"  /* Insertion of Int Fix into L Finger Phalanx Jt, Perc Approach */
"0RHX35Z"  =  "1"  /* Insertion of Ext Fix into L Finger Phalanx Jt, Perc Approach */
"0RHX44Z"  =  "1"  /* Insert Int Fix in L Finger Phalanx Jt, Perc Endo */
"0RHX45Z"  =  "1"  /* Insert Ext Fix in L Finger Phalanx Jt, Perc Endo */
"0RJ00ZZ"  =  "1"  /* Inspection of Occipital-cervical Joint, Open Approach */
"0RJ03ZZ"  =  "1"  /* Inspection of Occipital-cervical Joint, Perc Approach */
"0RJ04ZZ"  =  "1"  /* Inspection of Occipital-cervical Joint, Perc Endo Approach */
"0RJ10ZZ"  =  "1"  /* Inspection of Cervical Vertebral Joint, Open Approach */
"0RJ13ZZ"  =  "1"  /* Inspection of Cervical Vertebral Joint, Perc Approach */
"0RJ14ZZ"  =  "1"  /* Inspection of Cervical Vertebral Joint, Perc Endo Approach */
"0RJ30ZZ"  =  "1"  /* Inspection of Cervical Vertebral Disc, Open Approach */
"0RJ33ZZ"  =  "1"  /* Inspection of Cervical Vertebral Disc, Percutaneous Approach */
"0RJ34ZZ"  =  "1"  /* Inspection of Cervical Vertebral Disc, Perc Endo Approach */
"0RJ40ZZ"  =  "1"  /* Inspection of Cervicothoracic Vertebral Joint, Open Approach */
"0RJ43ZZ"  =  "1"  /* Inspection of Cervicothoracic Vertebral Joint, Perc Approach */
"0RJ44ZZ"  =  "1"  /* Inspection of C-thor Jt, Perc Endo Approach */
"0RJ50ZZ"  =  "1"  /* Inspection of Cervicothoracic Vertebral Disc, Open Approach */
"0RJ53ZZ"  =  "1"  /* Inspection of Cervicothoracic Vertebral Disc, Perc Approach */
"0RJ54ZZ"  =  "1"  /* Inspection of C-thor Disc, Perc Endo Approach */
"0RJ60ZZ"  =  "1"  /* Inspection of Thoracic Vertebral Joint, Open Approach */
"0RJ63ZZ"  =  "1"  /* Inspection of Thoracic Vertebral Joint, Perc Approach */
"0RJ64ZZ"  =  "1"  /* Inspection of Thoracic Vertebral Joint, Perc Endo Approach */
"0RJ90ZZ"  =  "1"  /* Inspection of Thoracic Vertebral Disc, Open Approach */
"0RJ93ZZ"  =  "1"  /* Inspection of Thoracic Vertebral Disc, Percutaneous Approach */
"0RJ94ZZ"  =  "1"  /* Inspection of Thoracic Vertebral Disc, Perc Endo Approach */
"0RJA0ZZ"  =  "1"  /* Inspection of Thoracolumbar Vertebral Joint, Open Approach */
"0RJA3ZZ"  =  "1"  /* Inspection of Thoracolumbar Vertebral Joint, Perc Approach */
"0RJA4ZZ"  =  "1"  /* Inspection of T-lum Jt, Perc Endo Approach */
"0RJB0ZZ"  =  "1"  /* Inspection of Thoracolumbar Vertebral Disc, Open Approach */
"0RJB3ZZ"  =  "1"  /* Inspection of Thoracolumbar Vertebral Disc, Perc Approach */
"0RJB4ZZ"  =  "1"  /* Inspection of T-lum Disc, Perc Endo Approach */
"0RJC0ZZ"  =  "1"  /* Inspection of Right Temporomandibular Joint, Open Approach */
"0RJC3ZZ"  =  "1"  /* Inspection of Right Temporomandibular Joint, Perc Approach */
"0RJC4ZZ"  =  "1"  /* Inspection of R Temporomandib Jt, Perc Endo Approach */
"0RJD0ZZ"  =  "1"  /* Inspection of Left Temporomandibular Joint, Open Approach */
"0RJD3ZZ"  =  "1"  /* Inspection of Left Temporomandibular Joint, Perc Approach */
"0RJD4ZZ"  =  "1"  /* Inspection of L Temporomandib Jt, Perc Endo Approach */
"0RJE0ZZ"  =  "1"  /* Inspection of Right Sternoclavicular Joint, Open Approach */
"0RJE3ZZ"  =  "1"  /* Inspection of Right Sternoclavicular Joint, Perc Approach */
"0RJE4ZZ"  =  "1"  /* Inspection of R Sternoclav Jt, Perc Endo Approach */
"0RJF0ZZ"  =  "1"  /* Inspection of Left Sternoclavicular Joint, Open Approach */
"0RJF3ZZ"  =  "1"  /* Inspection of Left Sternoclavicular Joint, Perc Approach */
"0RJF4ZZ"  =  "1"  /* Inspection of L Sternoclav Jt, Perc Endo Approach */
"0RJG0ZZ"  =  "1"  /* Inspection of Right Acromioclavicular Joint, Open Approach */
"0RJG3ZZ"  =  "1"  /* Inspection of Right Acromioclavicular Joint, Perc Approach */
"0RJG4ZZ"  =  "1"  /* Inspection of R Acromioclav Jt, Perc Endo Approach */
"0RJH0ZZ"  =  "1"  /* Inspection of Left Acromioclavicular Joint, Open Approach */
"0RJH3ZZ"  =  "1"  /* Inspection of Left Acromioclavicular Joint, Perc Approach */
"0RJH4ZZ"  =  "1"  /* Inspection of L Acromioclav Jt, Perc Endo Approach */
"0RJJ0ZZ"  =  "1"  /* Inspection of Right Shoulder Joint, Open Approach */
"0RJJ3ZZ"  =  "1"  /* Inspection of Right Shoulder Joint, Percutaneous Approach */
"0RJJ4ZZ"  =  "1"  /* Inspection of Right Shoulder Joint, Perc Endo Approach */
"0RJK0ZZ"  =  "1"  /* Inspection of Left Shoulder Joint, Open Approach */
"0RJK3ZZ"  =  "1"  /* Inspection of Left Shoulder Joint, Percutaneous Approach */
"0RJK4ZZ"  =  "1"  /* Inspection of Left Shoulder Joint, Perc Endo Approach */
"0RJL0ZZ"  =  "1"  /* Inspection of Right Elbow Joint, Open Approach */
"0RJL3ZZ"  =  "1"  /* Inspection of Right Elbow Joint, Percutaneous Approach */
"0RJL4ZZ"  =  "1"  /* Inspection of Right Elbow Joint, Perc Endo Approach */
"0RJM0ZZ"  =  "1"  /* Inspection of Left Elbow Joint, Open Approach */
"0RJM3ZZ"  =  "1"  /* Inspection of Left Elbow Joint, Percutaneous Approach */
"0RJM4ZZ"  =  "1"  /* Inspection of Left Elbow Joint, Perc Endo Approach */
"0RJN0ZZ"  =  "1"  /* Inspection of Right Wrist Joint, Open Approach */
"0RJN3ZZ"  =  "1"  /* Inspection of Right Wrist Joint, Percutaneous Approach */
"0RJN4ZZ"  =  "1"  /* Inspection of Right Wrist Joint, Perc Endo Approach */
"0RJP0ZZ"  =  "1"  /* Inspection of Left Wrist Joint, Open Approach */
"0RJP3ZZ"  =  "1"  /* Inspection of Left Wrist Joint, Percutaneous Approach */
"0RJP4ZZ"  =  "1"  /* Inspection of Left Wrist Joint, Perc Endo Approach */
"0RJQ0ZZ"  =  "1"  /* Inspection of Right Carpal Joint, Open Approach */
"0RJQ3ZZ"  =  "1"  /* Inspection of Right Carpal Joint, Percutaneous Approach */
"0RJQ4ZZ"  =  "1"  /* Inspection of Right Carpal Joint, Perc Endo Approach */
"0RJR0ZZ"  =  "1"  /* Inspection of Left Carpal Joint, Open Approach */
"0RJR3ZZ"  =  "1"  /* Inspection of Left Carpal Joint, Percutaneous Approach */
"0RJR4ZZ"  =  "1"  /* Inspection of Left Carpal Joint, Perc Endo Approach */
"0RJS0ZZ"  =  "1"  /* Inspection of Right Metacarpocarpal Joint, Open Approach */
"0RJS3ZZ"  =  "1"  /* Inspection of Right Metacarpocarpal Joint, Perc Approach */
"0RJS4ZZ"  =  "1"  /* Inspection of R Metacarpocarp Jt, Perc Endo Approach */
"0RJT0ZZ"  =  "1"  /* Inspection of Left Metacarpocarpal Joint, Open Approach */
"0RJT3ZZ"  =  "1"  /* Inspection of Left Metacarpocarpal Joint, Perc Approach */
"0RJT4ZZ"  =  "1"  /* Inspection of Left Metacarpocarpal Joint, Perc Endo Approach */
"0RJU0ZZ"  =  "1"  /* Inspection of Right Metacarpophalangeal Joint, Open Approach */
"0RJU3ZZ"  =  "1"  /* Inspection of Right Metacarpophalangeal Joint, Perc Approach */
"0RJU4ZZ"  =  "1"  /* Inspection of R Metacarpophal Jt, Perc Endo Approach */
"0RJV0ZZ"  =  "1"  /* Inspection of Left Metacarpophalangeal Joint, Open Approach */
"0RJV3ZZ"  =  "1"  /* Inspection of Left Metacarpophalangeal Joint, Perc Approach */
"0RJV4ZZ"  =  "1"  /* Inspection of L Metacarpophal Jt, Perc Endo Approach */
"0RJW0ZZ"  =  "1"  /* Inspection of Right Finger Phalangeal Joint, Open Approach */
"0RJW3ZZ"  =  "1"  /* Inspection of Right Finger Phalangeal Joint, Perc Approach */
"0RJW4ZZ"  =  "1"  /* Inspection of R Finger Phalanx Jt, Perc Endo Approach */
"0RJX0ZZ"  =  "1"  /* Inspection of Left Finger Phalangeal Joint, Open Approach */
"0RJX3ZZ"  =  "1"  /* Inspection of Left Finger Phalangeal Joint, Perc Approach */
"0RJX4ZZ"  =  "1"  /* Inspection of L Finger Phalanx Jt, Perc Endo Approach */
"0RN00ZZ"  =  "1"  /* Release Occipital-cervical Joint, Open Approach */
"0RN03ZZ"  =  "1"  /* Release Occipital-cervical Joint, Percutaneous Approach */
"0RN04ZZ"  =  "1"  /* Release Occipital-cervical Joint, Perc Endo Approach */
"0RN10ZZ"  =  "1"  /* Release Cervical Vertebral Joint, Open Approach */
"0RN13ZZ"  =  "1"  /* Release Cervical Vertebral Joint, Percutaneous Approach */
"0RN14ZZ"  =  "1"  /* Release Cervical Vertebral Joint, Perc Endo Approach */
"0RN30ZZ"  =  "1"  /* Release Cervical Vertebral Disc, Open Approach */
"0RN33ZZ"  =  "1"  /* Release Cervical Vertebral Disc, Percutaneous Approach */
"0RN34ZZ"  =  "1"  /* Release Cervical Vertebral Disc, Perc Endo Approach */
"0RN40ZZ"  =  "1"  /* Release Cervicothoracic Vertebral Joint, Open Approach */
"0RN43ZZ"  =  "1"  /* Release Cervicothoracic Vertebral Joint, Perc Approach */
"0RN44ZZ"  =  "1"  /* Release Cervicothoracic Vertebral Joint, Perc Endo Approach */
"0RN50ZZ"  =  "1"  /* Release Cervicothoracic Vertebral Disc, Open Approach */
"0RN53ZZ"  =  "1"  /* Release Cervicothoracic Vertebral Disc, Perc Approach */
"0RN54ZZ"  =  "1"  /* Release Cervicothoracic Vertebral Disc, Perc Endo Approach */
"0RN60ZZ"  =  "1"  /* Release Thoracic Vertebral Joint, Open Approach */
"0RN63ZZ"  =  "1"  /* Release Thoracic Vertebral Joint, Percutaneous Approach */
"0RN64ZZ"  =  "1"  /* Release Thoracic Vertebral Joint, Perc Endo Approach */
"0RN90ZZ"  =  "1"  /* Release Thoracic Vertebral Disc, Open Approach */
"0RN93ZZ"  =  "1"  /* Release Thoracic Vertebral Disc, Percutaneous Approach */
"0RN94ZZ"  =  "1"  /* Release Thoracic Vertebral Disc, Perc Endo Approach */
"0RNA0ZZ"  =  "1"  /* Release Thoracolumbar Vertebral Joint, Open Approach */
"0RNA3ZZ"  =  "1"  /* Release Thoracolumbar Vertebral Joint, Percutaneous Approach */
"0RNA4ZZ"  =  "1"  /* Release Thoracolumbar Vertebral Joint, Perc Endo Approach */
"0RNB0ZZ"  =  "1"  /* Release Thoracolumbar Vertebral Disc, Open Approach */
"0RNB3ZZ"  =  "1"  /* Release Thoracolumbar Vertebral Disc, Percutaneous Approach */
"0RNB4ZZ"  =  "1"  /* Release Thoracolumbar Vertebral Disc, Perc Endo Approach */
"0RNC0ZZ"  =  "1"  /* Release Right Temporomandibular Joint, Open Approach */
"0RNC3ZZ"  =  "1"  /* Release Right Temporomandibular Joint, Percutaneous Approach */
"0RNC4ZZ"  =  "1"  /* Release Right Temporomandibular Joint, Perc Endo Approach */
"0RND0ZZ"  =  "1"  /* Release Left Temporomandibular Joint, Open Approach */
"0RND3ZZ"  =  "1"  /* Release Left Temporomandibular Joint, Percutaneous Approach */
"0RND4ZZ"  =  "1"  /* Release Left Temporomandibular Joint, Perc Endo Approach */
"0RNE0ZZ"  =  "1"  /* Release Right Sternoclavicular Joint, Open Approach */
"0RNE3ZZ"  =  "1"  /* Release Right Sternoclavicular Joint, Percutaneous Approach */
"0RNE4ZZ"  =  "1"  /* Release Right Sternoclavicular Joint, Perc Endo Approach */
"0RNF0ZZ"  =  "1"  /* Release Left Sternoclavicular Joint, Open Approach */
"0RNF3ZZ"  =  "1"  /* Release Left Sternoclavicular Joint, Percutaneous Approach */
"0RNF4ZZ"  =  "1"  /* Release Left Sternoclavicular Joint, Perc Endo Approach */
"0RNG0ZZ"  =  "1"  /* Release Right Acromioclavicular Joint, Open Approach */
"0RNG3ZZ"  =  "1"  /* Release Right Acromioclavicular Joint, Percutaneous Approach */
"0RNG4ZZ"  =  "1"  /* Release Right Acromioclavicular Joint, Perc Endo Approach */
"0RNH0ZZ"  =  "1"  /* Release Left Acromioclavicular Joint, Open Approach */
"0RNH3ZZ"  =  "1"  /* Release Left Acromioclavicular Joint, Percutaneous Approach */
"0RNH4ZZ"  =  "1"  /* Release Left Acromioclavicular Joint, Perc Endo Approach */
"0RNJ0ZZ"  =  "1"  /* Release Right Shoulder Joint, Open Approach */
"0RNJ3ZZ"  =  "1"  /* Release Right Shoulder Joint, Percutaneous Approach */
"0RNJ4ZZ"  =  "1"  /* Release Right Shoulder Joint, Perc Endo Approach */
"0RNK0ZZ"  =  "1"  /* Release Left Shoulder Joint, Open Approach */
"0RNK3ZZ"  =  "1"  /* Release Left Shoulder Joint, Percutaneous Approach */
"0RNK4ZZ"  =  "1"  /* Release Left Shoulder Joint, Perc Endo Approach */
"0RNL0ZZ"  =  "1"  /* Release Right Elbow Joint, Open Approach */
"0RNL3ZZ"  =  "1"  /* Release Right Elbow Joint, Percutaneous Approach */
"0RNL4ZZ"  =  "1"  /* Release Right Elbow Joint, Percutaneous Endoscopic Approach */
"0RNM0ZZ"  =  "1"  /* Release Left Elbow Joint, Open Approach */
"0RNM3ZZ"  =  "1"  /* Release Left Elbow Joint, Percutaneous Approach */
"0RNM4ZZ"  =  "1"  /* Release Left Elbow Joint, Percutaneous Endoscopic Approach */
"0RNN0ZZ"  =  "1"  /* Release Right Wrist Joint, Open Approach */
"0RNN3ZZ"  =  "1"  /* Release Right Wrist Joint, Percutaneous Approach */
"0RNN4ZZ"  =  "1"  /* Release Right Wrist Joint, Percutaneous Endoscopic Approach */
"0RNP0ZZ"  =  "1"  /* Release Left Wrist Joint, Open Approach */
"0RNP3ZZ"  =  "1"  /* Release Left Wrist Joint, Percutaneous Approach */
"0RNP4ZZ"  =  "1"  /* Release Left Wrist Joint, Percutaneous Endoscopic Approach */
"0RNQ0ZZ"  =  "1"  /* Release Right Carpal Joint, Open Approach */
"0RNQ3ZZ"  =  "1"  /* Release Right Carpal Joint, Percutaneous Approach */
"0RNQ4ZZ"  =  "1"  /* Release Right Carpal Joint, Percutaneous Endoscopic Approach */
"0RNR0ZZ"  =  "1"  /* Release Left Carpal Joint, Open Approach */
"0RNR3ZZ"  =  "1"  /* Release Left Carpal Joint, Percutaneous Approach */
"0RNR4ZZ"  =  "1"  /* Release Left Carpal Joint, Percutaneous Endoscopic Approach */
"0RNS0ZZ"  =  "1"  /* Release Right Metacarpocarpal Joint, Open Approach */
"0RNS3ZZ"  =  "1"  /* Release Right Metacarpocarpal Joint, Percutaneous Approach */
"0RNS4ZZ"  =  "1"  /* Release Right Metacarpocarpal Joint, Perc Endo Approach */
"0RNT0ZZ"  =  "1"  /* Release Left Metacarpocarpal Joint, Open Approach */
"0RNT3ZZ"  =  "1"  /* Release Left Metacarpocarpal Joint, Percutaneous Approach */
"0RNT4ZZ"  =  "1"  /* Release Left Metacarpocarpal Joint, Perc Endo Approach */
"0RNU0ZZ"  =  "1"  /* Release Right Metacarpophalangeal Joint, Open Approach */
"0RNU3ZZ"  =  "1"  /* Release Right Metacarpophalangeal Joint, Perc Approach */
"0RNU4ZZ"  =  "1"  /* Release Right Metacarpophalangeal Joint, Perc Endo Approach */
"0RNV0ZZ"  =  "1"  /* Release Left Metacarpophalangeal Joint, Open Approach */
"0RNV3ZZ"  =  "1"  /* Release Left Metacarpophalangeal Joint, Perc Approach */
"0RNV4ZZ"  =  "1"  /* Release Left Metacarpophalangeal Joint, Perc Endo Approach */
"0RNW0ZZ"  =  "1"  /* Release Right Finger Phalangeal Joint, Open Approach */
"0RNW3ZZ"  =  "1"  /* Release Right Finger Phalangeal Joint, Percutaneous Approach */
"0RNW4ZZ"  =  "1"  /* Release Right Finger Phalangeal Joint, Perc Endo Approach */
"0RNX0ZZ"  =  "1"  /* Release Left Finger Phalangeal Joint, Open Approach */
"0RNX3ZZ"  =  "1"  /* Release Left Finger Phalangeal Joint, Percutaneous Approach */
"0RNX4ZZ"  =  "1"  /* Release Left Finger Phalangeal Joint, Perc Endo Approach */
"0RP000Z"  =  "1"  /* Removal of Drainage Device from Occip Jt, Open Approach */
"0RP003Z"  =  "1"  /* Removal of Infusion Device from Occip Jt, Open Approach */
"0RP004Z"  =  "1"  /* Removal of Int Fix from Occip Jt, Open Approach */
"0RP007Z"  =  "1"  /* Removal of Autol Sub from Occip Jt, Open Approach */
"0RP00AZ"  =  "1"  /* Removal of Intbd Fus Dev from Occip Jt, Open Approach */
"0RP00JZ"  =  "1"  /* Removal of Synthetic Substitute from Occip Jt, Open Approach */
"0RP00KZ"  =  "1"  /* Removal of Nonaut Sub from Occip Jt, Open Approach */
"0RP030Z"  =  "1"  /* Removal of Drainage Device from Occip Jt, Perc Approach */
"0RP033Z"  =  "1"  /* Removal of Infusion Device from Occip Jt, Perc Approach */
"0RP034Z"  =  "1"  /* Removal of Int Fix from Occip Jt, Perc Approach */
"0RP037Z"  =  "1"  /* Removal of Autol Sub from Occip Jt, Perc Approach */
"0RP03AZ"  =  "1"  /* Removal of Intbd Fus Dev from Occip Jt, Perc Approach */
"0RP03JZ"  =  "1"  /* Removal of Synthetic Substitute from Occip Jt, Perc Approach */
"0RP03KZ"  =  "1"  /* Removal of Nonaut Sub from Occip Jt, Perc Approach */
"0RP040Z"  =  "1"  /* Removal of Drainage Device from Occip Jt, Perc Endo Approach */
"0RP043Z"  =  "1"  /* Removal of Infusion Device from Occip Jt, Perc Endo Approach */
"0RP044Z"  =  "1"  /* Removal of Int Fix from Occip Jt, Perc Endo Approach */
"0RP047Z"  =  "1"  /* Removal of Autol Sub from Occip Jt, Perc Endo Approach */
"0RP04AZ"  =  "1"  /* Removal of Intbd Fus Dev from Occip Jt, Perc Endo Approach */
"0RP04JZ"  =  "1"  /* Removal of Synth Sub from Occip Jt, Perc Endo Approach */
"0RP04KZ"  =  "1"  /* Removal of Nonaut Sub from Occip Jt, Perc Endo Approach */
"0RP100Z"  =  "1"  /* Removal of Drainage Device from Cerv Jt, Open Approach */
"0RP103Z"  =  "1"  /* Removal of Infusion Device from Cerv Jt, Open Approach */
"0RP104Z"  =  "1"  /* Removal of Int Fix from Cerv Jt, Open Approach */
"0RP107Z"  =  "1"  /* Removal of Autol Sub from Cerv Jt, Open Approach */
"0RP10AZ"  =  "1"  /* Removal of Intbd Fus Dev from Cerv Jt, Open Approach */
"0RP10JZ"  =  "1"  /* Removal of Synthetic Substitute from Cerv Jt, Open Approach */
"0RP10KZ"  =  "1"  /* Removal of Nonaut Sub from Cerv Jt, Open Approach */
"0RP130Z"  =  "1"  /* Removal of Drainage Device from Cerv Jt, Perc Approach */
"0RP133Z"  =  "1"  /* Removal of Infusion Device from Cerv Jt, Perc Approach */
"0RP134Z"  =  "1"  /* Removal of Int Fix from Cerv Jt, Perc Approach */
"0RP137Z"  =  "1"  /* Removal of Autol Sub from Cerv Jt, Perc Approach */
"0RP13AZ"  =  "1"  /* Removal of Intbd Fus Dev from Cerv Jt, Perc Approach */
"0RP13JZ"  =  "1"  /* Removal of Synthetic Substitute from Cerv Jt, Perc Approach */
"0RP13KZ"  =  "1"  /* Removal of Nonaut Sub from Cerv Jt, Perc Approach */
"0RP140Z"  =  "1"  /* Removal of Drainage Device from Cerv Jt, Perc Endo Approach */
"0RP143Z"  =  "1"  /* Removal of Infusion Device from Cerv Jt, Perc Endo Approach */
"0RP144Z"  =  "1"  /* Removal of Int Fix from Cerv Jt, Perc Endo Approach */
"0RP147Z"  =  "1"  /* Removal of Autol Sub from Cerv Jt, Perc Endo Approach */
"0RP14AZ"  =  "1"  /* Removal of Intbd Fus Dev from Cerv Jt, Perc Endo Approach */
"0RP14JZ"  =  "1"  /* Removal of Synth Sub from Cerv Jt, Perc Endo Approach */
"0RP14KZ"  =  "1"  /* Removal of Nonaut Sub from Cerv Jt, Perc Endo Approach */
"0RP300Z"  =  "1"  /* Removal of Drainage Device from Cerv Disc, Open Approach */
"0RP303Z"  =  "1"  /* Removal of Infusion Device from Cerv Disc, Open Approach */
"0RP307Z"  =  "1"  /* Removal of Autol Sub from Cerv Disc, Open Approach */
"0RP30JZ"  =  "1"  /* Removal of Synth Sub from Cerv Disc, Open Approach */
"0RP30KZ"  =  "1"  /* Removal of Nonaut Sub from Cerv Disc, Open Approach */
"0RP330Z"  =  "1"  /* Removal of Drainage Device from Cerv Disc, Perc Approach */
"0RP333Z"  =  "1"  /* Removal of Infusion Device from Cerv Disc, Perc Approach */
"0RP337Z"  =  "1"  /* Removal of Autol Sub from Cerv Disc, Perc Approach */
"0RP33JZ"  =  "1"  /* Removal of Synth Sub from Cerv Disc, Perc Approach */
"0RP33KZ"  =  "1"  /* Removal of Nonaut Sub from Cerv Disc, Perc Approach */
"0RP340Z"  =  "1"  /* Removal of Drain Dev from Cerv Disc, Perc Endo Approach */
"0RP343Z"  =  "1"  /* Removal of Infusion Dev from Cerv Disc, Perc Endo Approach */
"0RP347Z"  =  "1"  /* Removal of Autol Sub from Cerv Disc, Perc Endo Approach */
"0RP34JZ"  =  "1"  /* Removal of Synth Sub from Cerv Disc, Perc Endo Approach */
"0RP34KZ"  =  "1"  /* Removal of Nonaut Sub from Cerv Disc, Perc Endo Approach */
"0RP400Z"  =  "1"  /* Removal of Drainage Device from C-thor Jt, Open Approach */
"0RP403Z"  =  "1"  /* Removal of Infusion Device from C-thor Jt, Open Approach */
"0RP404Z"  =  "1"  /* Removal of Int Fix from C-thor Jt, Open Approach */
"0RP407Z"  =  "1"  /* Removal of Autol Sub from C-thor Jt, Open Approach */
"0RP40AZ"  =  "1"  /* Removal of Intbd Fus Dev from C-thor Jt, Open Approach */
"0RP40JZ"  =  "1"  /* Removal of Synth Sub from C-thor Jt, Open Approach */
"0RP40KZ"  =  "1"  /* Removal of Nonaut Sub from C-thor Jt, Open Approach */
"0RP430Z"  =  "1"  /* Removal of Drainage Device from C-thor Jt, Perc Approach */
"0RP433Z"  =  "1"  /* Removal of Infusion Device from C-thor Jt, Perc Approach */
"0RP434Z"  =  "1"  /* Removal of Int Fix from C-thor Jt, Perc Approach */
"0RP437Z"  =  "1"  /* Removal of Autol Sub from C-thor Jt, Perc Approach */
"0RP43AZ"  =  "1"  /* Removal of Intbd Fus Dev from C-thor Jt, Perc Approach */
"0RP43JZ"  =  "1"  /* Removal of Synth Sub from C-thor Jt, Perc Approach */
"0RP43KZ"  =  "1"  /* Removal of Nonaut Sub from C-thor Jt, Perc Approach */
"0RP440Z"  =  "1"  /* Removal of Drain Dev from C-thor Jt, Perc Endo Approach */
"0RP443Z"  =  "1"  /* Removal of Infusion Dev from C-thor Jt, Perc Endo Approach */
"0RP444Z"  =  "1"  /* Removal of Int Fix from C-thor Jt, Perc Endo Approach */
"0RP447Z"  =  "1"  /* Removal of Autol Sub from C-thor Jt, Perc Endo Approach */
"0RP44AZ"  =  "1"  /* Removal of Intbd Fus Dev from C-thor Jt, Perc Endo Approach */
"0RP44JZ"  =  "1"  /* Removal of Synth Sub from C-thor Jt, Perc Endo Approach */
"0RP44KZ"  =  "1"  /* Removal of Nonaut Sub from C-thor Jt, Perc Endo Approach */
"0RP500Z"  =  "1"  /* Removal of Drainage Device from C-thor Disc, Open Approach */
"0RP503Z"  =  "1"  /* Removal of Infusion Device from C-thor Disc, Open Approach */
"0RP507Z"  =  "1"  /* Removal of Autol Sub from C-thor Disc, Open Approach */
"0RP50JZ"  =  "1"  /* Removal of Synth Sub from C-thor Disc, Open Approach */
"0RP50KZ"  =  "1"  /* Removal of Nonaut Sub from C-thor Disc, Open Approach */
"0RP530Z"  =  "1"  /* Removal of Drainage Device from C-thor Disc, Perc Approach */
"0RP533Z"  =  "1"  /* Removal of Infusion Device from C-thor Disc, Perc Approach */
"0RP537Z"  =  "1"  /* Removal of Autol Sub from C-thor Disc, Perc Approach */
"0RP53JZ"  =  "1"  /* Removal of Synth Sub from C-thor Disc, Perc Approach */
"0RP53KZ"  =  "1"  /* Removal of Nonaut Sub from C-thor Disc, Perc Approach */
"0RP540Z"  =  "1"  /* Removal of Drain Dev from C-thor Disc, Perc Endo Approach */
"0RP543Z"  =  "1"  /* Removal of Infusion Dev from C-thor Disc, Perc Endo Approach */
"0RP547Z"  =  "1"  /* Removal of Autol Sub from C-thor Disc, Perc Endo Approach */
"0RP54JZ"  =  "1"  /* Removal of Synth Sub from C-thor Disc, Perc Endo Approach */
"0RP54KZ"  =  "1"  /* Removal of Nonaut Sub from C-thor Disc, Perc Endo Approach */
"0RP600Z"  =  "1"  /* Removal of Drainage Device from Thor Jt, Open Approach */
"0RP603Z"  =  "1"  /* Removal of Infusion Device from Thor Jt, Open Approach */
"0RP604Z"  =  "1"  /* Removal of Int Fix from Thor Jt, Open Approach */
"0RP607Z"  =  "1"  /* Removal of Autol Sub from Thor Jt, Open Approach */
"0RP60AZ"  =  "1"  /* Removal of Intbd Fus Dev from Thor Jt, Open Approach */
"0RP60JZ"  =  "1"  /* Removal of Synthetic Substitute from Thor Jt, Open Approach */
"0RP60KZ"  =  "1"  /* Removal of Nonaut Sub from Thor Jt, Open Approach */
"0RP630Z"  =  "1"  /* Removal of Drainage Device from Thor Jt, Perc Approach */
"0RP633Z"  =  "1"  /* Removal of Infusion Device from Thor Jt, Perc Approach */
"0RP634Z"  =  "1"  /* Removal of Int Fix from Thor Jt, Perc Approach */
"0RP637Z"  =  "1"  /* Removal of Autol Sub from Thor Jt, Perc Approach */
"0RP63AZ"  =  "1"  /* Removal of Intbd Fus Dev from Thor Jt, Perc Approach */
"0RP63JZ"  =  "1"  /* Removal of Synthetic Substitute from Thor Jt, Perc Approach */
"0RP63KZ"  =  "1"  /* Removal of Nonaut Sub from Thor Jt, Perc Approach */
"0RP640Z"  =  "1"  /* Removal of Drainage Device from Thor Jt, Perc Endo Approach */
"0RP643Z"  =  "1"  /* Removal of Infusion Device from Thor Jt, Perc Endo Approach */
"0RP644Z"  =  "1"  /* Removal of Int Fix from Thor Jt, Perc Endo Approach */
"0RP647Z"  =  "1"  /* Removal of Autol Sub from Thor Jt, Perc Endo Approach */
"0RP64AZ"  =  "1"  /* Removal of Intbd Fus Dev from Thor Jt, Perc Endo Approach */
"0RP64JZ"  =  "1"  /* Removal of Synth Sub from Thor Jt, Perc Endo Approach */
"0RP64KZ"  =  "1"  /* Removal of Nonaut Sub from Thor Jt, Perc Endo Approach */
"0RP900Z"  =  "1"  /* Removal of Drainage Device from Thor Disc, Open Approach */
"0RP903Z"  =  "1"  /* Removal of Infusion Device from Thor Disc, Open Approach */
"0RP907Z"  =  "1"  /* Removal of Autol Sub from Thor Disc, Open Approach */
"0RP90JZ"  =  "1"  /* Removal of Synth Sub from Thor Disc, Open Approach */
"0RP90KZ"  =  "1"  /* Removal of Nonaut Sub from Thor Disc, Open Approach */
"0RP930Z"  =  "1"  /* Removal of Drainage Device from Thor Disc, Perc Approach */
"0RP933Z"  =  "1"  /* Removal of Infusion Device from Thor Disc, Perc Approach */
"0RP937Z"  =  "1"  /* Removal of Autol Sub from Thor Disc, Perc Approach */
"0RP93JZ"  =  "1"  /* Removal of Synth Sub from Thor Disc, Perc Approach */
"0RP93KZ"  =  "1"  /* Removal of Nonaut Sub from Thor Disc, Perc Approach */
"0RP940Z"  =  "1"  /* Removal of Drain Dev from Thor Disc, Perc Endo Approach */
"0RP943Z"  =  "1"  /* Removal of Infusion Dev from Thor Disc, Perc Endo Approach */
"0RP947Z"  =  "1"  /* Removal of Autol Sub from Thor Disc, Perc Endo Approach */
"0RP94JZ"  =  "1"  /* Removal of Synth Sub from Thor Disc, Perc Endo Approach */
"0RP94KZ"  =  "1"  /* Removal of Nonaut Sub from Thor Disc, Perc Endo Approach */
"0RPA00Z"  =  "1"  /* Removal of Drainage Device from T-lum Jt, Open Approach */
"0RPA03Z"  =  "1"  /* Removal of Infusion Device from T-lum Jt, Open Approach */
"0RPA04Z"  =  "1"  /* Removal of Int Fix from T-lum Jt, Open Approach */
"0RPA07Z"  =  "1"  /* Removal of Autol Sub from T-lum Jt, Open Approach */
"0RPA0AZ"  =  "1"  /* Removal of Intbd Fus Dev from T-lum Jt, Open Approach */
"0RPA0JZ"  =  "1"  /* Removal of Synthetic Substitute from T-lum Jt, Open Approach */
"0RPA0KZ"  =  "1"  /* Removal of Nonaut Sub from T-lum Jt, Open Approach */
"0RPA30Z"  =  "1"  /* Removal of Drainage Device from T-lum Jt, Perc Approach */
"0RPA33Z"  =  "1"  /* Removal of Infusion Device from T-lum Jt, Perc Approach */
"0RPA34Z"  =  "1"  /* Removal of Int Fix from T-lum Jt, Perc Approach */
"0RPA37Z"  =  "1"  /* Removal of Autol Sub from T-lum Jt, Perc Approach */
"0RPA3AZ"  =  "1"  /* Removal of Intbd Fus Dev from T-lum Jt, Perc Approach */
"0RPA3JZ"  =  "1"  /* Removal of Synthetic Substitute from T-lum Jt, Perc Approach */
"0RPA3KZ"  =  "1"  /* Removal of Nonaut Sub from T-lum Jt, Perc Approach */
"0RPA40Z"  =  "1"  /* Removal of Drainage Device from T-lum Jt, Perc Endo Approach */
"0RPA43Z"  =  "1"  /* Removal of Infusion Device from T-lum Jt, Perc Endo Approach */
"0RPA44Z"  =  "1"  /* Removal of Int Fix from T-lum Jt, Perc Endo Approach */
"0RPA47Z"  =  "1"  /* Removal of Autol Sub from T-lum Jt, Perc Endo Approach */
"0RPA4AZ"  =  "1"  /* Removal of Intbd Fus Dev from T-lum Jt, Perc Endo Approach */
"0RPA4JZ"  =  "1"  /* Removal of Synth Sub from T-lum Jt, Perc Endo Approach */
"0RPA4KZ"  =  "1"  /* Removal of Nonaut Sub from T-lum Jt, Perc Endo Approach */
"0RPB00Z"  =  "1"  /* Removal of Drainage Device from T-lum Disc, Open Approach */
"0RPB03Z"  =  "1"  /* Removal of Infusion Device from T-lum Disc, Open Approach */
"0RPB07Z"  =  "1"  /* Removal of Autol Sub from T-lum Disc, Open Approach */
"0RPB0JZ"  =  "1"  /* Removal of Synth Sub from T-lum Disc, Open Approach */
"0RPB0KZ"  =  "1"  /* Removal of Nonaut Sub from T-lum Disc, Open Approach */
"0RPB30Z"  =  "1"  /* Removal of Drainage Device from T-lum Disc, Perc Approach */
"0RPB33Z"  =  "1"  /* Removal of Infusion Device from T-lum Disc, Perc Approach */
"0RPB37Z"  =  "1"  /* Removal of Autol Sub from T-lum Disc, Perc Approach */
"0RPB3JZ"  =  "1"  /* Removal of Synth Sub from T-lum Disc, Perc Approach */
"0RPB3KZ"  =  "1"  /* Removal of Nonaut Sub from T-lum Disc, Perc Approach */
"0RPB40Z"  =  "1"  /* Removal of Drain Dev from T-lum Disc, Perc Endo Approach */
"0RPB43Z"  =  "1"  /* Removal of Infusion Dev from T-lum Disc, Perc Endo Approach */
"0RPB47Z"  =  "1"  /* Removal of Autol Sub from T-lum Disc, Perc Endo Approach */
"0RPB4JZ"  =  "1"  /* Removal of Synth Sub from T-lum Disc, Perc Endo Approach */
"0RPB4KZ"  =  "1"  /* Removal of Nonaut Sub from T-lum Disc, Perc Endo Approach */
"0RPC00Z"  =  "1"  /* Removal of Drain Dev from R Temporomandib Jt, Open Approach */
"0RPC03Z"  =  "1"  /* Remove Infusion Dev from R Temporomandib Jt, Open */
"0RPC04Z"  =  "1"  /* Removal of Int Fix from R Temporomandib Jt, Open Approach */
"0RPC07Z"  =  "1"  /* Removal of Autol Sub from R Temporomandib Jt, Open Approach */
"0RPC0JZ"  =  "1"  /* Removal of Synth Sub from R Temporomandib Jt, Open Approach */
"0RPC0KZ"  =  "1"  /* Removal of Nonaut Sub from R Temporomandib Jt, Open Approach */
"0RPC30Z"  =  "1"  /* Removal of Drain Dev from R Temporomandib Jt, Perc Approach */
"0RPC33Z"  =  "1"  /* Remove Infusion Dev from R Temporomandib Jt, Perc */
"0RPC34Z"  =  "1"  /* Removal of Int Fix from R Temporomandib Jt, Perc Approach */
"0RPC37Z"  =  "1"  /* Removal of Autol Sub from R Temporomandib Jt, Perc Approach */
"0RPC3JZ"  =  "1"  /* Removal of Synth Sub from R Temporomandib Jt, Perc Approach */
"0RPC3KZ"  =  "1"  /* Removal of Nonaut Sub from R Temporomandib Jt, Perc Approach */
"0RPC40Z"  =  "1"  /* Remove Drain Dev from R Temporomandib Jt, Perc Endo */
"0RPC43Z"  =  "1"  /* Remove Infusion Dev from R Temporomandib Jt, Perc Endo */
"0RPC44Z"  =  "1"  /* Remove Int Fix from R Temporomandib Jt, Perc Endo */
"0RPC47Z"  =  "1"  /* Remove Autol Sub from R Temporomandib Jt, Perc Endo */
"0RPC4JZ"  =  "1"  /* Remove Synth Sub from R Temporomandib Jt, Perc Endo */
"0RPC4KZ"  =  "1"  /* Remove Nonaut Sub from R Temporomandib Jt, Perc Endo */
"0RPCX4Z"  =  "1"  /* Removal of Int Fix from R Temporomandib Jt, Extern Approach */
"0RPD00Z"  =  "1"  /* Removal of Drain Dev from L Temporomandib Jt, Open Approach */
"0RPD03Z"  =  "1"  /* Remove Infusion Dev from L Temporomandib Jt, Open */
"0RPD04Z"  =  "1"  /* Removal of Int Fix from L Temporomandib Jt, Open Approach */
"0RPD07Z"  =  "1"  /* Removal of Autol Sub from L Temporomandib Jt, Open Approach */
"0RPD0JZ"  =  "1"  /* Removal of Synth Sub from L Temporomandib Jt, Open Approach */
"0RPD0KZ"  =  "1"  /* Removal of Nonaut Sub from L Temporomandib Jt, Open Approach */
"0RPD30Z"  =  "1"  /* Removal of Drain Dev from L Temporomandib Jt, Perc Approach */
"0RPD33Z"  =  "1"  /* Remove Infusion Dev from L Temporomandib Jt, Perc */
"0RPD34Z"  =  "1"  /* Removal of Int Fix from L Temporomandib Jt, Perc Approach */
"0RPD37Z"  =  "1"  /* Removal of Autol Sub from L Temporomandib Jt, Perc Approach */
"0RPD3JZ"  =  "1"  /* Removal of Synth Sub from L Temporomandib Jt, Perc Approach */
"0RPD3KZ"  =  "1"  /* Removal of Nonaut Sub from L Temporomandib Jt, Perc Approach */
"0RPD40Z"  =  "1"  /* Remove Drain Dev from L Temporomandib Jt, Perc Endo */
"0RPD43Z"  =  "1"  /* Remove Infusion Dev from L Temporomandib Jt, Perc Endo */
"0RPD44Z"  =  "1"  /* Remove Int Fix from L Temporomandib Jt, Perc Endo */
"0RPD47Z"  =  "1"  /* Remove Autol Sub from L Temporomandib Jt, Perc Endo */
"0RPD4JZ"  =  "1"  /* Remove Synth Sub from L Temporomandib Jt, Perc Endo */
"0RPD4KZ"  =  "1"  /* Remove Nonaut Sub from L Temporomandib Jt, Perc Endo */
"0RPDX4Z"  =  "1"  /* Removal of Int Fix from L Temporomandib Jt, Extern Approach */
"0RPE00Z"  =  "1"  /* Removal of Drain Dev from R Sternoclav Jt, Open Approach */
"0RPE03Z"  =  "1"  /* Removal of Infusion Dev from R Sternoclav Jt, Open Approach */
"0RPE04Z"  =  "1"  /* Removal of Int Fix from R Sternoclav Jt, Open Approach */
"0RPE07Z"  =  "1"  /* Removal of Autol Sub from R Sternoclav Jt, Open Approach */
"0RPE0JZ"  =  "1"  /* Removal of Synth Sub from R Sternoclav Jt, Open Approach */
"0RPE0KZ"  =  "1"  /* Removal of Nonaut Sub from R Sternoclav Jt, Open Approach */
"0RPE30Z"  =  "1"  /* Removal of Drain Dev from R Sternoclav Jt, Perc Approach */
"0RPE33Z"  =  "1"  /* Removal of Infusion Dev from R Sternoclav Jt, Perc Approach */
"0RPE34Z"  =  "1"  /* Removal of Int Fix from R Sternoclav Jt, Perc Approach */
"0RPE37Z"  =  "1"  /* Removal of Autol Sub from R Sternoclav Jt, Perc Approach */
"0RPE3JZ"  =  "1"  /* Removal of Synth Sub from R Sternoclav Jt, Perc Approach */
"0RPE3KZ"  =  "1"  /* Removal of Nonaut Sub from R Sternoclav Jt, Perc Approach */
"0RPE40Z"  =  "1"  /* Remove of Drain Dev from R Sternoclav Jt, Perc Endo Approach */
"0RPE43Z"  =  "1"  /* Remove Infusion Dev from R Sternoclav Jt, Perc Endo */
"0RPE44Z"  =  "1"  /* Removal of Int Fix from R Sternoclav Jt, Perc Endo Approach */
"0RPE47Z"  =  "1"  /* Remove of Autol Sub from R Sternoclav Jt, Perc Endo Approach */
"0RPE4JZ"  =  "1"  /* Remove of Synth Sub from R Sternoclav Jt, Perc Endo Approach */
"0RPE4KZ"  =  "1"  /* Remove Nonaut Sub from R Sternoclav Jt, Perc Endo */
"0RPF00Z"  =  "1"  /* Removal of Drain Dev from L Sternoclav Jt, Open Approach */
"0RPF03Z"  =  "1"  /* Removal of Infusion Dev from L Sternoclav Jt, Open Approach */
"0RPF04Z"  =  "1"  /* Removal of Int Fix from L Sternoclav Jt, Open Approach */
"0RPF07Z"  =  "1"  /* Removal of Autol Sub from L Sternoclav Jt, Open Approach */
"0RPF0JZ"  =  "1"  /* Removal of Synth Sub from L Sternoclav Jt, Open Approach */
"0RPF0KZ"  =  "1"  /* Removal of Nonaut Sub from L Sternoclav Jt, Open Approach */
"0RPF30Z"  =  "1"  /* Removal of Drain Dev from L Sternoclav Jt, Perc Approach */
"0RPF33Z"  =  "1"  /* Removal of Infusion Dev from L Sternoclav Jt, Perc Approach */
"0RPF34Z"  =  "1"  /* Removal of Int Fix from L Sternoclav Jt, Perc Approach */
"0RPF37Z"  =  "1"  /* Removal of Autol Sub from L Sternoclav Jt, Perc Approach */
"0RPF3JZ"  =  "1"  /* Removal of Synth Sub from L Sternoclav Jt, Perc Approach */
"0RPF3KZ"  =  "1"  /* Removal of Nonaut Sub from L Sternoclav Jt, Perc Approach */
"0RPF40Z"  =  "1"  /* Remove of Drain Dev from L Sternoclav Jt, Perc Endo Approach */
"0RPF43Z"  =  "1"  /* Remove Infusion Dev from L Sternoclav Jt, Perc Endo */
"0RPF44Z"  =  "1"  /* Removal of Int Fix from L Sternoclav Jt, Perc Endo Approach */
"0RPF47Z"  =  "1"  /* Remove of Autol Sub from L Sternoclav Jt, Perc Endo Approach */
"0RPF4JZ"  =  "1"  /* Remove of Synth Sub from L Sternoclav Jt, Perc Endo Approach */
"0RPF4KZ"  =  "1"  /* Remove Nonaut Sub from L Sternoclav Jt, Perc Endo */
"0RPG00Z"  =  "1"  /* Removal of Drain Dev from R Acromioclav Jt, Open Approach */
"0RPG03Z"  =  "1"  /* Removal of Infusion Dev from R Acromioclav Jt, Open Approach */
"0RPG04Z"  =  "1"  /* Removal of Int Fix from R Acromioclav Jt, Open Approach */
"0RPG07Z"  =  "1"  /* Removal of Autol Sub from R Acromioclav Jt, Open Approach */
"0RPG0JZ"  =  "1"  /* Removal of Synth Sub from R Acromioclav Jt, Open Approach */
"0RPG0KZ"  =  "1"  /* Removal of Nonaut Sub from R Acromioclav Jt, Open Approach */
"0RPG30Z"  =  "1"  /* Removal of Drain Dev from R Acromioclav Jt, Perc Approach */
"0RPG33Z"  =  "1"  /* Removal of Infusion Dev from R Acromioclav Jt, Perc Approach */
"0RPG34Z"  =  "1"  /* Removal of Int Fix from R Acromioclav Jt, Perc Approach */
"0RPG37Z"  =  "1"  /* Removal of Autol Sub from R Acromioclav Jt, Perc Approach */
"0RPG3JZ"  =  "1"  /* Removal of Synth Sub from R Acromioclav Jt, Perc Approach */
"0RPG3KZ"  =  "1"  /* Removal of Nonaut Sub from R Acromioclav Jt, Perc Approach */
"0RPG40Z"  =  "1"  /* Remove Drain Dev from R Acromioclav Jt, Perc Endo */
"0RPG43Z"  =  "1"  /* Remove Infusion Dev from R Acromioclav Jt, Perc Endo */
"0RPG44Z"  =  "1"  /* Removal of Int Fix from R Acromioclav Jt, Perc Endo Approach */
"0RPG47Z"  =  "1"  /* Remove Autol Sub from R Acromioclav Jt, Perc Endo */
"0RPG4JZ"  =  "1"  /* Remove Synth Sub from R Acromioclav Jt, Perc Endo */
"0RPG4KZ"  =  "1"  /* Remove Nonaut Sub from R Acromioclav Jt, Perc Endo */
"0RPH00Z"  =  "1"  /* Removal of Drain Dev from L Acromioclav Jt, Open Approach */
"0RPH03Z"  =  "1"  /* Removal of Infusion Dev from L Acromioclav Jt, Open Approach */
"0RPH04Z"  =  "1"  /* Removal of Int Fix from L Acromioclav Jt, Open Approach */
"0RPH07Z"  =  "1"  /* Removal of Autol Sub from L Acromioclav Jt, Open Approach */
"0RPH0JZ"  =  "1"  /* Removal of Synth Sub from L Acromioclav Jt, Open Approach */
"0RPH0KZ"  =  "1"  /* Removal of Nonaut Sub from L Acromioclav Jt, Open Approach */
"0RPH30Z"  =  "1"  /* Removal of Drain Dev from L Acromioclav Jt, Perc Approach */
"0RPH33Z"  =  "1"  /* Removal of Infusion Dev from L Acromioclav Jt, Perc Approach */
"0RPH34Z"  =  "1"  /* Removal of Int Fix from L Acromioclav Jt, Perc Approach */
"0RPH37Z"  =  "1"  /* Removal of Autol Sub from L Acromioclav Jt, Perc Approach */
"0RPH3JZ"  =  "1"  /* Removal of Synth Sub from L Acromioclav Jt, Perc Approach */
"0RPH3KZ"  =  "1"  /* Removal of Nonaut Sub from L Acromioclav Jt, Perc Approach */
"0RPH40Z"  =  "1"  /* Remove Drain Dev from L Acromioclav Jt, Perc Endo */
"0RPH43Z"  =  "1"  /* Remove Infusion Dev from L Acromioclav Jt, Perc Endo */
"0RPH44Z"  =  "1"  /* Removal of Int Fix from L Acromioclav Jt, Perc Endo Approach */
"0RPH47Z"  =  "1"  /* Remove Autol Sub from L Acromioclav Jt, Perc Endo */
"0RPH4JZ"  =  "1"  /* Remove Synth Sub from L Acromioclav Jt, Perc Endo */
"0RPH4KZ"  =  "1"  /* Remove Nonaut Sub from L Acromioclav Jt, Perc Endo */
"0RPJ00Z"  =  "1"  /* Removal of Drainage Device from R Shoulder Jt, Open Approach */
"0RPJ03Z"  =  "1"  /* Removal of Infusion Device from R Shoulder Jt, Open Approach */
"0RPJ04Z"  =  "1"  /* Removal of Int Fix from R Shoulder Jt, Open Approach */
"0RPJ07Z"  =  "1"  /* Removal of Autol Sub from R Shoulder Jt, Open Approach */
"0RPJ0JZ"  =  "1"  /* Removal of Synth Sub from R Shoulder Jt, Open Approach */
"0RPJ0KZ"  =  "1"  /* Removal of Nonaut Sub from R Shoulder Jt, Open Approach */
"0RPJ30Z"  =  "1"  /* Removal of Drainage Device from R Shoulder Jt, Perc Approach */
"0RPJ33Z"  =  "1"  /* Removal of Infusion Device from R Shoulder Jt, Perc Approach */
"0RPJ34Z"  =  "1"  /* Removal of Int Fix from R Shoulder Jt, Perc Approach */
"0RPJ37Z"  =  "1"  /* Removal of Autol Sub from R Shoulder Jt, Perc Approach */
"0RPJ3JZ"  =  "1"  /* Removal of Synth Sub from R Shoulder Jt, Perc Approach */
"0RPJ3KZ"  =  "1"  /* Removal of Nonaut Sub from R Shoulder Jt, Perc Approach */
"0RPJ40Z"  =  "1"  /* Removal of Drain Dev from R Shoulder Jt, Perc Endo Approach */
"0RPJ43Z"  =  "1"  /* Remove Infusion Dev from R Shoulder Jt, Perc Endo */
"0RPJ44Z"  =  "1"  /* Removal of Int Fix from R Shoulder Jt, Perc Endo Approach */
"0RPJ47Z"  =  "1"  /* Removal of Autol Sub from R Shoulder Jt, Perc Endo Approach */
"0RPJ4JZ"  =  "1"  /* Removal of Synth Sub from R Shoulder Jt, Perc Endo Approach */
"0RPJ4KZ"  =  "1"  /* Removal of Nonaut Sub from R Shoulder Jt, Perc Endo Approach */
"0RPK00Z"  =  "1"  /* Removal of Drainage Device from L Shoulder Jt, Open Approach */
"0RPK03Z"  =  "1"  /* Removal of Infusion Device from L Shoulder Jt, Open Approach */
"0RPK04Z"  =  "1"  /* Removal of Int Fix from L Shoulder Jt, Open Approach */
"0RPK07Z"  =  "1"  /* Removal of Autol Sub from L Shoulder Jt, Open Approach */
"0RPK0JZ"  =  "1"  /* Removal of Synth Sub from L Shoulder Jt, Open Approach */
"0RPK0KZ"  =  "1"  /* Removal of Nonaut Sub from L Shoulder Jt, Open Approach */
"0RPK30Z"  =  "1"  /* Removal of Drainage Device from L Shoulder Jt, Perc Approach */
"0RPK33Z"  =  "1"  /* Removal of Infusion Device from L Shoulder Jt, Perc Approach */
"0RPK34Z"  =  "1"  /* Removal of Int Fix from L Shoulder Jt, Perc Approach */
"0RPK37Z"  =  "1"  /* Removal of Autol Sub from L Shoulder Jt, Perc Approach */
"0RPK3JZ"  =  "1"  /* Removal of Synth Sub from L Shoulder Jt, Perc Approach */
"0RPK3KZ"  =  "1"  /* Removal of Nonaut Sub from L Shoulder Jt, Perc Approach */
"0RPK40Z"  =  "1"  /* Removal of Drain Dev from L Shoulder Jt, Perc Endo Approach */
"0RPK43Z"  =  "1"  /* Remove Infusion Dev from L Shoulder Jt, Perc Endo */
"0RPK44Z"  =  "1"  /* Removal of Int Fix from L Shoulder Jt, Perc Endo Approach */
"0RPK47Z"  =  "1"  /* Removal of Autol Sub from L Shoulder Jt, Perc Endo Approach */
"0RPK4JZ"  =  "1"  /* Removal of Synth Sub from L Shoulder Jt, Perc Endo Approach */
"0RPK4KZ"  =  "1"  /* Removal of Nonaut Sub from L Shoulder Jt, Perc Endo Approach */
"0RPL00Z"  =  "1"  /* Removal of Drainage Device from R Elbow Jt, Open Approach */
"0RPL03Z"  =  "1"  /* Removal of Infusion Device from R Elbow Jt, Open Approach */
"0RPL04Z"  =  "1"  /* Removal of Int Fix from R Elbow Jt, Open Approach */
"0RPL05Z"  =  "1"  /* Removal of Ext Fix from R Elbow Jt, Open Approach */
"0RPL07Z"  =  "1"  /* Removal of Autol Sub from R Elbow Jt, Open Approach */
"0RPL0JZ"  =  "1"  /* Removal of Synth Sub from R Elbow Jt, Open Approach */
"0RPL0KZ"  =  "1"  /* Removal of Nonaut Sub from R Elbow Jt, Open Approach */
"0RPL30Z"  =  "1"  /* Removal of Drainage Device from R Elbow Jt, Perc Approach */
"0RPL33Z"  =  "1"  /* Removal of Infusion Device from R Elbow Jt, Perc Approach */
"0RPL34Z"  =  "1"  /* Removal of Int Fix from R Elbow Jt, Perc Approach */
"0RPL35Z"  =  "1"  /* Removal of Ext Fix from R Elbow Jt, Perc Approach */
"0RPL37Z"  =  "1"  /* Removal of Autol Sub from R Elbow Jt, Perc Approach */
"0RPL3JZ"  =  "1"  /* Removal of Synth Sub from R Elbow Jt, Perc Approach */
"0RPL3KZ"  =  "1"  /* Removal of Nonaut Sub from R Elbow Jt, Perc Approach */
"0RPL40Z"  =  "1"  /* Removal of Drain Dev from R Elbow Jt, Perc Endo Approach */
"0RPL43Z"  =  "1"  /* Removal of Infusion Dev from R Elbow Jt, Perc Endo Approach */
"0RPL44Z"  =  "1"  /* Removal of Int Fix from R Elbow Jt, Perc Endo Approach */
"0RPL45Z"  =  "1"  /* Removal of Ext Fix from R Elbow Jt, Perc Endo Approach */
"0RPL47Z"  =  "1"  /* Removal of Autol Sub from R Elbow Jt, Perc Endo Approach */
"0RPL4JZ"  =  "1"  /* Removal of Synth Sub from R Elbow Jt, Perc Endo Approach */
"0RPL4KZ"  =  "1"  /* Removal of Nonaut Sub from R Elbow Jt, Perc Endo Approach */
"0RPM00Z"  =  "1"  /* Removal of Drainage Device from L Elbow Jt, Open Approach */
"0RPM03Z"  =  "1"  /* Removal of Infusion Device from L Elbow Jt, Open Approach */
"0RPM04Z"  =  "1"  /* Removal of Int Fix from L Elbow Jt, Open Approach */
"0RPM05Z"  =  "1"  /* Removal of Ext Fix from L Elbow Jt, Open Approach */
"0RPM07Z"  =  "1"  /* Removal of Autol Sub from L Elbow Jt, Open Approach */
"0RPM0JZ"  =  "1"  /* Removal of Synth Sub from L Elbow Jt, Open Approach */
"0RPM0KZ"  =  "1"  /* Removal of Nonaut Sub from L Elbow Jt, Open Approach */
"0RPM30Z"  =  "1"  /* Removal of Drainage Device from L Elbow Jt, Perc Approach */
"0RPM33Z"  =  "1"  /* Removal of Infusion Device from L Elbow Jt, Perc Approach */
"0RPM34Z"  =  "1"  /* Removal of Int Fix from L Elbow Jt, Perc Approach */
"0RPM35Z"  =  "1"  /* Removal of Ext Fix from L Elbow Jt, Perc Approach */
"0RPM37Z"  =  "1"  /* Removal of Autol Sub from L Elbow Jt, Perc Approach */
"0RPM3JZ"  =  "1"  /* Removal of Synth Sub from L Elbow Jt, Perc Approach */
"0RPM3KZ"  =  "1"  /* Removal of Nonaut Sub from L Elbow Jt, Perc Approach */
"0RPM40Z"  =  "1"  /* Removal of Drain Dev from L Elbow Jt, Perc Endo Approach */
"0RPM43Z"  =  "1"  /* Removal of Infusion Dev from L Elbow Jt, Perc Endo Approach */
"0RPM44Z"  =  "1"  /* Removal of Int Fix from L Elbow Jt, Perc Endo Approach */
"0RPM45Z"  =  "1"  /* Removal of Ext Fix from L Elbow Jt, Perc Endo Approach */
"0RPM47Z"  =  "1"  /* Removal of Autol Sub from L Elbow Jt, Perc Endo Approach */
"0RPM4JZ"  =  "1"  /* Removal of Synth Sub from L Elbow Jt, Perc Endo Approach */
"0RPM4KZ"  =  "1"  /* Removal of Nonaut Sub from L Elbow Jt, Perc Endo Approach */
"0RPN00Z"  =  "1"  /* Removal of Drainage Device from R Wrist Jt, Open Approach */
"0RPN03Z"  =  "1"  /* Removal of Infusion Device from R Wrist Jt, Open Approach */
"0RPN04Z"  =  "1"  /* Removal of Int Fix from R Wrist Jt, Open Approach */
"0RPN05Z"  =  "1"  /* Removal of Ext Fix from R Wrist Jt, Open Approach */
"0RPN07Z"  =  "1"  /* Removal of Autol Sub from R Wrist Jt, Open Approach */
"0RPN0JZ"  =  "1"  /* Removal of Synth Sub from R Wrist Jt, Open Approach */
"0RPN0KZ"  =  "1"  /* Removal of Nonaut Sub from R Wrist Jt, Open Approach */
"0RPN30Z"  =  "1"  /* Removal of Drainage Device from R Wrist Jt, Perc Approach */
"0RPN33Z"  =  "1"  /* Removal of Infusion Device from R Wrist Jt, Perc Approach */
"0RPN34Z"  =  "1"  /* Removal of Int Fix from R Wrist Jt, Perc Approach */
"0RPN35Z"  =  "1"  /* Removal of Ext Fix from R Wrist Jt, Perc Approach */
"0RPN37Z"  =  "1"  /* Removal of Autol Sub from R Wrist Jt, Perc Approach */
"0RPN3JZ"  =  "1"  /* Removal of Synth Sub from R Wrist Jt, Perc Approach */
"0RPN3KZ"  =  "1"  /* Removal of Nonaut Sub from R Wrist Jt, Perc Approach */
"0RPN40Z"  =  "1"  /* Removal of Drain Dev from R Wrist Jt, Perc Endo Approach */
"0RPN43Z"  =  "1"  /* Removal of Infusion Dev from R Wrist Jt, Perc Endo Approach */
"0RPN44Z"  =  "1"  /* Removal of Int Fix from R Wrist Jt, Perc Endo Approach */
"0RPN45Z"  =  "1"  /* Removal of Ext Fix from R Wrist Jt, Perc Endo Approach */
"0RPN47Z"  =  "1"  /* Removal of Autol Sub from R Wrist Jt, Perc Endo Approach */
"0RPN4JZ"  =  "1"  /* Removal of Synth Sub from R Wrist Jt, Perc Endo Approach */
"0RPN4KZ"  =  "1"  /* Removal of Nonaut Sub from R Wrist Jt, Perc Endo Approach */
"0RPP00Z"  =  "1"  /* Removal of Drainage Device from L Wrist Jt, Open Approach */
"0RPP03Z"  =  "1"  /* Removal of Infusion Device from L Wrist Jt, Open Approach */
"0RPP04Z"  =  "1"  /* Removal of Int Fix from L Wrist Jt, Open Approach */
"0RPP05Z"  =  "1"  /* Removal of Ext Fix from L Wrist Jt, Open Approach */
"0RPP07Z"  =  "1"  /* Removal of Autol Sub from L Wrist Jt, Open Approach */
"0RPP0JZ"  =  "1"  /* Removal of Synth Sub from L Wrist Jt, Open Approach */
"0RPP0KZ"  =  "1"  /* Removal of Nonaut Sub from L Wrist Jt, Open Approach */
"0RPP30Z"  =  "1"  /* Removal of Drainage Device from L Wrist Jt, Perc Approach */
"0RPP33Z"  =  "1"  /* Removal of Infusion Device from L Wrist Jt, Perc Approach */
"0RPP34Z"  =  "1"  /* Removal of Int Fix from L Wrist Jt, Perc Approach */
"0RPP35Z"  =  "1"  /* Removal of Ext Fix from L Wrist Jt, Perc Approach */
"0RPP37Z"  =  "1"  /* Removal of Autol Sub from L Wrist Jt, Perc Approach */
"0RPP3JZ"  =  "1"  /* Removal of Synth Sub from L Wrist Jt, Perc Approach */
"0RPP3KZ"  =  "1"  /* Removal of Nonaut Sub from L Wrist Jt, Perc Approach */
"0RPP40Z"  =  "1"  /* Removal of Drain Dev from L Wrist Jt, Perc Endo Approach */
"0RPP43Z"  =  "1"  /* Removal of Infusion Dev from L Wrist Jt, Perc Endo Approach */
"0RPP44Z"  =  "1"  /* Removal of Int Fix from L Wrist Jt, Perc Endo Approach */
"0RPP45Z"  =  "1"  /* Removal of Ext Fix from L Wrist Jt, Perc Endo Approach */
"0RPP47Z"  =  "1"  /* Removal of Autol Sub from L Wrist Jt, Perc Endo Approach */
"0RPP4JZ"  =  "1"  /* Removal of Synth Sub from L Wrist Jt, Perc Endo Approach */
"0RPP4KZ"  =  "1"  /* Removal of Nonaut Sub from L Wrist Jt, Perc Endo Approach */
"0RPQ00Z"  =  "1"  /* Removal of Drainage Device from R Carpal Jt, Open Approach */
"0RPQ03Z"  =  "1"  /* Removal of Infusion Device from R Carpal Jt, Open Approach */
"0RPQ04Z"  =  "1"  /* Removal of Int Fix from R Carpal Jt, Open Approach */
"0RPQ05Z"  =  "1"  /* Removal of Ext Fix from R Carpal Jt, Open Approach */
"0RPQ07Z"  =  "1"  /* Removal of Autol Sub from R Carpal Jt, Open Approach */
"0RPQ0JZ"  =  "1"  /* Removal of Synth Sub from R Carpal Jt, Open Approach */
"0RPQ0KZ"  =  "1"  /* Removal of Nonaut Sub from R Carpal Jt, Open Approach */
"0RPQ30Z"  =  "1"  /* Removal of Drainage Device from R Carpal Jt, Perc Approach */
"0RPQ33Z"  =  "1"  /* Removal of Infusion Device from R Carpal Jt, Perc Approach */
"0RPQ34Z"  =  "1"  /* Removal of Int Fix from R Carpal Jt, Perc Approach */
"0RPQ35Z"  =  "1"  /* Removal of Ext Fix from R Carpal Jt, Perc Approach */
"0RPQ37Z"  =  "1"  /* Removal of Autol Sub from R Carpal Jt, Perc Approach */
"0RPQ3JZ"  =  "1"  /* Removal of Synth Sub from R Carpal Jt, Perc Approach */
"0RPQ3KZ"  =  "1"  /* Removal of Nonaut Sub from R Carpal Jt, Perc Approach */
"0RPQ40Z"  =  "1"  /* Removal of Drain Dev from R Carpal Jt, Perc Endo Approach */
"0RPQ43Z"  =  "1"  /* Removal of Infusion Dev from R Carpal Jt, Perc Endo Approach */
"0RPQ44Z"  =  "1"  /* Removal of Int Fix from R Carpal Jt, Perc Endo Approach */
"0RPQ45Z"  =  "1"  /* Removal of Ext Fix from R Carpal Jt, Perc Endo Approach */
"0RPQ47Z"  =  "1"  /* Removal of Autol Sub from R Carpal Jt, Perc Endo Approach */
"0RPQ4JZ"  =  "1"  /* Removal of Synth Sub from R Carpal Jt, Perc Endo Approach */
"0RPQ4KZ"  =  "1"  /* Removal of Nonaut Sub from R Carpal Jt, Perc Endo Approach */
"0RPR00Z"  =  "1"  /* Removal of Drainage Device from L Carpal Jt, Open Approach */
"0RPR03Z"  =  "1"  /* Removal of Infusion Device from L Carpal Jt, Open Approach */
"0RPR04Z"  =  "1"  /* Removal of Int Fix from L Carpal Jt, Open Approach */
"0RPR05Z"  =  "1"  /* Removal of Ext Fix from L Carpal Jt, Open Approach */
"0RPR07Z"  =  "1"  /* Removal of Autol Sub from L Carpal Jt, Open Approach */
"0RPR0JZ"  =  "1"  /* Removal of Synth Sub from L Carpal Jt, Open Approach */
"0RPR0KZ"  =  "1"  /* Removal of Nonaut Sub from L Carpal Jt, Open Approach */
"0RPR30Z"  =  "1"  /* Removal of Drainage Device from L Carpal Jt, Perc Approach */
"0RPR33Z"  =  "1"  /* Removal of Infusion Device from L Carpal Jt, Perc Approach */
"0RPR34Z"  =  "1"  /* Removal of Int Fix from L Carpal Jt, Perc Approach */
"0RPR35Z"  =  "1"  /* Removal of Ext Fix from L Carpal Jt, Perc Approach */
"0RPR37Z"  =  "1"  /* Removal of Autol Sub from L Carpal Jt, Perc Approach */
"0RPR3JZ"  =  "1"  /* Removal of Synth Sub from L Carpal Jt, Perc Approach */
"0RPR3KZ"  =  "1"  /* Removal of Nonaut Sub from L Carpal Jt, Perc Approach */
"0RPR40Z"  =  "1"  /* Removal of Drain Dev from L Carpal Jt, Perc Endo Approach */
"0RPR43Z"  =  "1"  /* Removal of Infusion Dev from L Carpal Jt, Perc Endo Approach */
"0RPR44Z"  =  "1"  /* Removal of Int Fix from L Carpal Jt, Perc Endo Approach */
"0RPR45Z"  =  "1"  /* Removal of Ext Fix from L Carpal Jt, Perc Endo Approach */
"0RPR47Z"  =  "1"  /* Removal of Autol Sub from L Carpal Jt, Perc Endo Approach */
"0RPR4JZ"  =  "1"  /* Removal of Synth Sub from L Carpal Jt, Perc Endo Approach */
"0RPR4KZ"  =  "1"  /* Removal of Nonaut Sub from L Carpal Jt, Perc Endo Approach */
"0RPS00Z"  =  "1"  /* Removal of Drain Dev from R Metacarpocarp Jt, Open Approach */
"0RPS03Z"  =  "1"  /* Remove Infusion Dev from R Metacarpocarp Jt, Open */
"0RPS04Z"  =  "1"  /* Removal of Int Fix from R Metacarpocarp Jt, Open Approach */
"0RPS05Z"  =  "1"  /* Removal of Ext Fix from R Metacarpocarp Jt, Open Approach */
"0RPS07Z"  =  "1"  /* Removal of Autol Sub from R Metacarpocarp Jt, Open Approach */
"0RPS0JZ"  =  "1"  /* Removal of Synth Sub from R Metacarpocarp Jt, Open Approach */
"0RPS0KZ"  =  "1"  /* Removal of Nonaut Sub from R Metacarpocarp Jt, Open Approach */
"0RPS30Z"  =  "1"  /* Removal of Drain Dev from R Metacarpocarp Jt, Perc Approach */
"0RPS33Z"  =  "1"  /* Remove Infusion Dev from R Metacarpocarp Jt, Perc */
"0RPS34Z"  =  "1"  /* Removal of Int Fix from R Metacarpocarp Jt, Perc Approach */
"0RPS35Z"  =  "1"  /* Removal of Ext Fix from R Metacarpocarp Jt, Perc Approach */
"0RPS37Z"  =  "1"  /* Removal of Autol Sub from R Metacarpocarp Jt, Perc Approach */
"0RPS3JZ"  =  "1"  /* Removal of Synth Sub from R Metacarpocarp Jt, Perc Approach */
"0RPS3KZ"  =  "1"  /* Removal of Nonaut Sub from R Metacarpocarp Jt, Perc Approach */
"0RPS40Z"  =  "1"  /* Remove Drain Dev from R Metacarpocarp Jt, Perc Endo */
"0RPS43Z"  =  "1"  /* Remove Infusion Dev from R Metacarpocarp Jt, Perc Endo */
"0RPS44Z"  =  "1"  /* Remove Int Fix from R Metacarpocarp Jt, Perc Endo */
"0RPS45Z"  =  "1"  /* Remove Ext Fix from R Metacarpocarp Jt, Perc Endo */
"0RPS47Z"  =  "1"  /* Remove Autol Sub from R Metacarpocarp Jt, Perc Endo */
"0RPS4JZ"  =  "1"  /* Remove Synth Sub from R Metacarpocarp Jt, Perc Endo */
"0RPS4KZ"  =  "1"  /* Remove Nonaut Sub from R Metacarpocarp Jt, Perc Endo */
"0RPT00Z"  =  "1"  /* Removal of Drain Dev from L Metacarpocarp Jt, Open Approach */
"0RPT03Z"  =  "1"  /* Remove Infusion Dev from L Metacarpocarp Jt, Open */
"0RPT04Z"  =  "1"  /* Removal of Int Fix from L Metacarpocarp Jt, Open Approach */
"0RPT05Z"  =  "1"  /* Removal of Ext Fix from L Metacarpocarp Jt, Open Approach */
"0RPT07Z"  =  "1"  /* Removal of Autol Sub from L Metacarpocarp Jt, Open Approach */
"0RPT0JZ"  =  "1"  /* Removal of Synth Sub from L Metacarpocarp Jt, Open Approach */
"0RPT0KZ"  =  "1"  /* Removal of Nonaut Sub from L Metacarpocarp Jt, Open Approach */
"0RPT30Z"  =  "1"  /* Removal of Drain Dev from L Metacarpocarp Jt, Perc Approach */
"0RPT33Z"  =  "1"  /* Remove Infusion Dev from L Metacarpocarp Jt, Perc */
"0RPT34Z"  =  "1"  /* Removal of Int Fix from L Metacarpocarp Jt, Perc Approach */
"0RPT35Z"  =  "1"  /* Removal of Ext Fix from L Metacarpocarp Jt, Perc Approach */
"0RPT37Z"  =  "1"  /* Removal of Autol Sub from L Metacarpocarp Jt, Perc Approach */
"0RPT3JZ"  =  "1"  /* Removal of Synth Sub from L Metacarpocarp Jt, Perc Approach */
"0RPT3KZ"  =  "1"  /* Removal of Nonaut Sub from L Metacarpocarp Jt, Perc Approach */
"0RPT40Z"  =  "1"  /* Remove Drain Dev from L Metacarpocarp Jt, Perc Endo */
"0RPT43Z"  =  "1"  /* Remove Infusion Dev from L Metacarpocarp Jt, Perc Endo */
"0RPT44Z"  =  "1"  /* Remove Int Fix from L Metacarpocarp Jt, Perc Endo */
"0RPT45Z"  =  "1"  /* Remove Ext Fix from L Metacarpocarp Jt, Perc Endo */
"0RPT47Z"  =  "1"  /* Remove Autol Sub from L Metacarpocarp Jt, Perc Endo */
"0RPT4JZ"  =  "1"  /* Remove Synth Sub from L Metacarpocarp Jt, Perc Endo */
"0RPT4KZ"  =  "1"  /* Remove Nonaut Sub from L Metacarpocarp Jt, Perc Endo */
"0RPU00Z"  =  "1"  /* Removal of Drain Dev from R Metacarpophal Jt, Open Approach */
"0RPU03Z"  =  "1"  /* Remove Infusion Dev from R Metacarpophal Jt, Open */
"0RPU04Z"  =  "1"  /* Removal of Int Fix from R Metacarpophal Jt, Open Approach */
"0RPU05Z"  =  "1"  /* Removal of Ext Fix from R Metacarpophal Jt, Open Approach */
"0RPU07Z"  =  "1"  /* Removal of Autol Sub from R Metacarpophal Jt, Open Approach */
"0RPU0JZ"  =  "1"  /* Removal of Synth Sub from R Metacarpophal Jt, Open Approach */
"0RPU0KZ"  =  "1"  /* Removal of Nonaut Sub from R Metacarpophal Jt, Open Approach */
"0RPU30Z"  =  "1"  /* Removal of Drain Dev from R Metacarpophal Jt, Perc Approach */
"0RPU33Z"  =  "1"  /* Remove Infusion Dev from R Metacarpophal Jt, Perc */
"0RPU34Z"  =  "1"  /* Removal of Int Fix from R Metacarpophal Jt, Perc Approach */
"0RPU35Z"  =  "1"  /* Removal of Ext Fix from R Metacarpophal Jt, Perc Approach */
"0RPU37Z"  =  "1"  /* Removal of Autol Sub from R Metacarpophal Jt, Perc Approach */
"0RPU3JZ"  =  "1"  /* Removal of Synth Sub from R Metacarpophal Jt, Perc Approach */
"0RPU3KZ"  =  "1"  /* Removal of Nonaut Sub from R Metacarpophal Jt, Perc Approach */
"0RPU40Z"  =  "1"  /* Remove Drain Dev from R Metacarpophal Jt, Perc Endo */
"0RPU43Z"  =  "1"  /* Remove Infusion Dev from R Metacarpophal Jt, Perc Endo */
"0RPU44Z"  =  "1"  /* Remove Int Fix from R Metacarpophal Jt, Perc Endo */
"0RPU45Z"  =  "1"  /* Remove Ext Fix from R Metacarpophal Jt, Perc Endo */
"0RPU47Z"  =  "1"  /* Remove Autol Sub from R Metacarpophal Jt, Perc Endo */
"0RPU4JZ"  =  "1"  /* Remove Synth Sub from R Metacarpophal Jt, Perc Endo */
"0RPU4KZ"  =  "1"  /* Remove Nonaut Sub from R Metacarpophal Jt, Perc Endo */
"0RPV00Z"  =  "1"  /* Removal of Drain Dev from L Metacarpophal Jt, Open Approach */
"0RPV03Z"  =  "1"  /* Remove Infusion Dev from L Metacarpophal Jt, Open */
"0RPV04Z"  =  "1"  /* Removal of Int Fix from L Metacarpophal Jt, Open Approach */
"0RPV05Z"  =  "1"  /* Removal of Ext Fix from L Metacarpophal Jt, Open Approach */
"0RPV07Z"  =  "1"  /* Removal of Autol Sub from L Metacarpophal Jt, Open Approach */
"0RPV0JZ"  =  "1"  /* Removal of Synth Sub from L Metacarpophal Jt, Open Approach */
"0RPV0KZ"  =  "1"  /* Removal of Nonaut Sub from L Metacarpophal Jt, Open Approach */
"0RPV30Z"  =  "1"  /* Removal of Drain Dev from L Metacarpophal Jt, Perc Approach */
"0RPV33Z"  =  "1"  /* Remove Infusion Dev from L Metacarpophal Jt, Perc */
"0RPV34Z"  =  "1"  /* Removal of Int Fix from L Metacarpophal Jt, Perc Approach */
"0RPV35Z"  =  "1"  /* Removal of Ext Fix from L Metacarpophal Jt, Perc Approach */
"0RPV37Z"  =  "1"  /* Removal of Autol Sub from L Metacarpophal Jt, Perc Approach */
"0RPV3JZ"  =  "1"  /* Removal of Synth Sub from L Metacarpophal Jt, Perc Approach */
"0RPV3KZ"  =  "1"  /* Removal of Nonaut Sub from L Metacarpophal Jt, Perc Approach */
"0RPV40Z"  =  "1"  /* Remove Drain Dev from L Metacarpophal Jt, Perc Endo */
"0RPV43Z"  =  "1"  /* Remove Infusion Dev from L Metacarpophal Jt, Perc Endo */
"0RPV44Z"  =  "1"  /* Remove Int Fix from L Metacarpophal Jt, Perc Endo */
"0RPV45Z"  =  "1"  /* Remove Ext Fix from L Metacarpophal Jt, Perc Endo */
"0RPV47Z"  =  "1"  /* Remove Autol Sub from L Metacarpophal Jt, Perc Endo */
"0RPV4JZ"  =  "1"  /* Remove Synth Sub from L Metacarpophal Jt, Perc Endo */
"0RPV4KZ"  =  "1"  /* Remove Nonaut Sub from L Metacarpophal Jt, Perc Endo */
"0RPW00Z"  =  "1"  /* Removal of Drain Dev from R Finger Phalanx Jt, Open Approach */
"0RPW03Z"  =  "1"  /* Remove Infusion Dev from R Finger Phalanx Jt, Open */
"0RPW04Z"  =  "1"  /* Removal of Int Fix from R Finger Phalanx Jt, Open Approach */
"0RPW05Z"  =  "1"  /* Removal of Ext Fix from R Finger Phalanx Jt, Open Approach */
"0RPW07Z"  =  "1"  /* Removal of Autol Sub from R Finger Phalanx Jt, Open Approach */
"0RPW0JZ"  =  "1"  /* Removal of Synth Sub from R Finger Phalanx Jt, Open Approach */
"0RPW0KZ"  =  "1"  /* Remove of Nonaut Sub from R Finger Phalanx Jt, Open Approach */
"0RPW30Z"  =  "1"  /* Removal of Drain Dev from R Finger Phalanx Jt, Perc Approach */
"0RPW33Z"  =  "1"  /* Remove Infusion Dev from R Finger Phalanx Jt, Perc */
"0RPW34Z"  =  "1"  /* Removal of Int Fix from R Finger Phalanx Jt, Perc Approach */
"0RPW35Z"  =  "1"  /* Removal of Ext Fix from R Finger Phalanx Jt, Perc Approach */
"0RPW37Z"  =  "1"  /* Removal of Autol Sub from R Finger Phalanx Jt, Perc Approach */
"0RPW3JZ"  =  "1"  /* Removal of Synth Sub from R Finger Phalanx Jt, Perc Approach */
"0RPW3KZ"  =  "1"  /* Remove of Nonaut Sub from R Finger Phalanx Jt, Perc Approach */
"0RPW40Z"  =  "1"  /* Remove Drain Dev from R Finger Phalanx Jt, Perc Endo */
"0RPW43Z"  =  "1"  /* Remove Infusion Dev from R Finger Phalanx Jt, Perc Endo */
"0RPW44Z"  =  "1"  /* Remove Int Fix from R Finger Phalanx Jt, Perc Endo */
"0RPW45Z"  =  "1"  /* Remove Ext Fix from R Finger Phalanx Jt, Perc Endo */
"0RPW47Z"  =  "1"  /* Remove Autol Sub from R Finger Phalanx Jt, Perc Endo */
"0RPW4JZ"  =  "1"  /* Remove Synth Sub from R Finger Phalanx Jt, Perc Endo */
"0RPW4KZ"  =  "1"  /* Remove Nonaut Sub from R Finger Phalanx Jt, Perc Endo */
"0RPX00Z"  =  "1"  /* Removal of Drain Dev from L Finger Phalanx Jt, Open Approach */
"0RPX03Z"  =  "1"  /* Remove Infusion Dev from L Finger Phalanx Jt, Open */
"0RPX04Z"  =  "1"  /* Removal of Int Fix from L Finger Phalanx Jt, Open Approach */
"0RPX05Z"  =  "1"  /* Removal of Ext Fix from L Finger Phalanx Jt, Open Approach */
"0RPX07Z"  =  "1"  /* Removal of Autol Sub from L Finger Phalanx Jt, Open Approach */
"0RPX0JZ"  =  "1"  /* Removal of Synth Sub from L Finger Phalanx Jt, Open Approach */
"0RPX0KZ"  =  "1"  /* Remove of Nonaut Sub from L Finger Phalanx Jt, Open Approach */
"0RPX30Z"  =  "1"  /* Removal of Drain Dev from L Finger Phalanx Jt, Perc Approach */
"0RPX33Z"  =  "1"  /* Remove Infusion Dev from L Finger Phalanx Jt, Perc */
"0RPX34Z"  =  "1"  /* Removal of Int Fix from L Finger Phalanx Jt, Perc Approach */
"0RPX35Z"  =  "1"  /* Removal of Ext Fix from L Finger Phalanx Jt, Perc Approach */
"0RPX37Z"  =  "1"  /* Removal of Autol Sub from L Finger Phalanx Jt, Perc Approach */
"0RPX3JZ"  =  "1"  /* Removal of Synth Sub from L Finger Phalanx Jt, Perc Approach */
"0RPX3KZ"  =  "1"  /* Remove of Nonaut Sub from L Finger Phalanx Jt, Perc Approach */
"0RPX40Z"  =  "1"  /* Remove Drain Dev from L Finger Phalanx Jt, Perc Endo */
"0RPX43Z"  =  "1"  /* Remove Infusion Dev from L Finger Phalanx Jt, Perc Endo */
"0RPX44Z"  =  "1"  /* Remove Int Fix from L Finger Phalanx Jt, Perc Endo */
"0RPX45Z"  =  "1"  /* Remove Ext Fix from L Finger Phalanx Jt, Perc Endo */
"0RPX47Z"  =  "1"  /* Remove Autol Sub from L Finger Phalanx Jt, Perc Endo */
"0RPX4JZ"  =  "1"  /* Remove Synth Sub from L Finger Phalanx Jt, Perc Endo */
"0RPX4KZ"  =  "1"  /* Remove Nonaut Sub from L Finger Phalanx Jt, Perc Endo */
"0RQ00ZZ"  =  "1"  /* Repair Occipital-cervical Joint, Open Approach */
"0RQ03ZZ"  =  "1"  /* Repair Occipital-cervical Joint, Percutaneous Approach */
"0RQ04ZZ"  =  "1"  /* Repair Occipital-cervical Joint, Perc Endo Approach */
"0RQ0XZZ"  =  "1"  /* Repair Occipital-cervical Joint, External Approach */
"0RQ10ZZ"  =  "1"  /* Repair Cervical Vertebral Joint, Open Approach */
"0RQ13ZZ"  =  "1"  /* Repair Cervical Vertebral Joint, Percutaneous Approach */
"0RQ14ZZ"  =  "1"  /* Repair Cervical Vertebral Joint, Perc Endo Approach */
"0RQ1XZZ"  =  "1"  /* Repair Cervical Vertebral Joint, External Approach */
"0RQ30ZZ"  =  "1"  /* Repair Cervical Vertebral Disc, Open Approach */
"0RQ33ZZ"  =  "1"  /* Repair Cervical Vertebral Disc, Percutaneous Approach */
"0RQ34ZZ"  =  "1"  /* Repair Cervical Vertebral Disc, Perc Endo Approach */
"0RQ3XZZ"  =  "1"  /* Repair Cervical Vertebral Disc, External Approach */
"0RQ40ZZ"  =  "1"  /* Repair Cervicothoracic Vertebral Joint, Open Approach */
"0RQ43ZZ"  =  "1"  /* Repair Cervicothoracic Vertebral Joint, Perc Approach */
"0RQ44ZZ"  =  "1"  /* Repair Cervicothoracic Vertebral Joint, Perc Endo Approach */
"0RQ4XZZ"  =  "1"  /* Repair Cervicothoracic Vertebral Joint, External Approach */
"0RQ50ZZ"  =  "1"  /* Repair Cervicothoracic Vertebral Disc, Open Approach */
"0RQ53ZZ"  =  "1"  /* Repair Cervicothoracic Vertebral Disc, Percutaneous Approach */
"0RQ54ZZ"  =  "1"  /* Repair Cervicothoracic Vertebral Disc, Perc Endo Approach */
"0RQ5XZZ"  =  "1"  /* Repair Cervicothoracic Vertebral Disc, External Approach */
"0RQ60ZZ"  =  "1"  /* Repair Thoracic Vertebral Joint, Open Approach */
"0RQ63ZZ"  =  "1"  /* Repair Thoracic Vertebral Joint, Percutaneous Approach */
"0RQ64ZZ"  =  "1"  /* Repair Thoracic Vertebral Joint, Perc Endo Approach */
"0RQ6XZZ"  =  "1"  /* Repair Thoracic Vertebral Joint, External Approach */
"0RQ90ZZ"  =  "1"  /* Repair Thoracic Vertebral Disc, Open Approach */
"0RQ93ZZ"  =  "1"  /* Repair Thoracic Vertebral Disc, Percutaneous Approach */
"0RQ94ZZ"  =  "1"  /* Repair Thoracic Vertebral Disc, Perc Endo Approach */
"0RQ9XZZ"  =  "1"  /* Repair Thoracic Vertebral Disc, External Approach */
"0RQA0ZZ"  =  "1"  /* Repair Thoracolumbar Vertebral Joint, Open Approach */
"0RQA3ZZ"  =  "1"  /* Repair Thoracolumbar Vertebral Joint, Percutaneous Approach */
"0RQA4ZZ"  =  "1"  /* Repair Thoracolumbar Vertebral Joint, Perc Endo Approach */
"0RQAXZZ"  =  "1"  /* Repair Thoracolumbar Vertebral Joint, External Approach */
"0RQB0ZZ"  =  "1"  /* Repair Thoracolumbar Vertebral Disc, Open Approach */
"0RQB3ZZ"  =  "1"  /* Repair Thoracolumbar Vertebral Disc, Percutaneous Approach */
"0RQB4ZZ"  =  "1"  /* Repair Thoracolumbar Vertebral Disc, Perc Endo Approach */
"0RQBXZZ"  =  "1"  /* Repair Thoracolumbar Vertebral Disc, External Approach */
"0RQC0ZZ"  =  "1"  /* Repair Right Temporomandibular Joint, Open Approach */
"0RQC3ZZ"  =  "1"  /* Repair Right Temporomandibular Joint, Percutaneous Approach */
"0RQC4ZZ"  =  "1"  /* Repair Right Temporomandibular Joint, Perc Endo Approach */
"0RQD0ZZ"  =  "1"  /* Repair Left Temporomandibular Joint, Open Approach */
"0RQD3ZZ"  =  "1"  /* Repair Left Temporomandibular Joint, Percutaneous Approach */
"0RQD4ZZ"  =  "1"  /* Repair Left Temporomandibular Joint, Perc Endo Approach */
"0RQE0ZZ"  =  "1"  /* Repair Right Sternoclavicular Joint, Open Approach */
"0RQE3ZZ"  =  "1"  /* Repair Right Sternoclavicular Joint, Percutaneous Approach */
"0RQE4ZZ"  =  "1"  /* Repair Right Sternoclavicular Joint, Perc Endo Approach */
"0RQEXZZ"  =  "1"  /* Repair Right Sternoclavicular Joint, External Approach */
"0RQF0ZZ"  =  "1"  /* Repair Left Sternoclavicular Joint, Open Approach */
"0RQF3ZZ"  =  "1"  /* Repair Left Sternoclavicular Joint, Percutaneous Approach */
"0RQF4ZZ"  =  "1"  /* Repair Left Sternoclavicular Joint, Perc Endo Approach */
"0RQFXZZ"  =  "1"  /* Repair Left Sternoclavicular Joint, External Approach */
"0RQG0ZZ"  =  "1"  /* Repair Right Acromioclavicular Joint, Open Approach */
"0RQG3ZZ"  =  "1"  /* Repair Right Acromioclavicular Joint, Percutaneous Approach */
"0RQG4ZZ"  =  "1"  /* Repair Right Acromioclavicular Joint, Perc Endo Approach */
"0RQGXZZ"  =  "1"  /* Repair Right Acromioclavicular Joint, External Approach */
"0RQH0ZZ"  =  "1"  /* Repair Left Acromioclavicular Joint, Open Approach */
"0RQH3ZZ"  =  "1"  /* Repair Left Acromioclavicular Joint, Percutaneous Approach */
"0RQH4ZZ"  =  "1"  /* Repair Left Acromioclavicular Joint, Perc Endo Approach */
"0RQHXZZ"  =  "1"  /* Repair Left Acromioclavicular Joint, External Approach */
"0RQJ0ZZ"  =  "1"  /* Repair Right Shoulder Joint, Open Approach */
"0RQJ3ZZ"  =  "1"  /* Repair Right Shoulder Joint, Percutaneous Approach */
"0RQJ4ZZ"  =  "1"  /* Repair Right Shoulder Joint, Perc Endo Approach */
"0RQJXZZ"  =  "1"  /* Repair Right Shoulder Joint, External Approach */
"0RQK0ZZ"  =  "1"  /* Repair Left Shoulder Joint, Open Approach */
"0RQK3ZZ"  =  "1"  /* Repair Left Shoulder Joint, Percutaneous Approach */
"0RQK4ZZ"  =  "1"  /* Repair Left Shoulder Joint, Percutaneous Endoscopic Approach */
"0RQKXZZ"  =  "1"  /* Repair Left Shoulder Joint, External Approach */
"0RQL0ZZ"  =  "1"  /* Repair Right Elbow Joint, Open Approach */
"0RQL3ZZ"  =  "1"  /* Repair Right Elbow Joint, Percutaneous Approach */
"0RQL4ZZ"  =  "1"  /* Repair Right Elbow Joint, Percutaneous Endoscopic Approach */
"0RQLXZZ"  =  "1"  /* Repair Right Elbow Joint, External Approach */
"0RQM0ZZ"  =  "1"  /* Repair Left Elbow Joint, Open Approach */
"0RQM3ZZ"  =  "1"  /* Repair Left Elbow Joint, Percutaneous Approach */
"0RQM4ZZ"  =  "1"  /* Repair Left Elbow Joint, Percutaneous Endoscopic Approach */
"0RQMXZZ"  =  "1"  /* Repair Left Elbow Joint, External Approach */
"0RQN0ZZ"  =  "1"  /* Repair Right Wrist Joint, Open Approach */
"0RQN3ZZ"  =  "1"  /* Repair Right Wrist Joint, Percutaneous Approach */
"0RQN4ZZ"  =  "1"  /* Repair Right Wrist Joint, Percutaneous Endoscopic Approach */
"0RQNXZZ"  =  "1"  /* Repair Right Wrist Joint, External Approach */
"0RQP0ZZ"  =  "1"  /* Repair Left Wrist Joint, Open Approach */
"0RQP3ZZ"  =  "1"  /* Repair Left Wrist Joint, Percutaneous Approach */
"0RQP4ZZ"  =  "1"  /* Repair Left Wrist Joint, Percutaneous Endoscopic Approach */
"0RQPXZZ"  =  "1"  /* Repair Left Wrist Joint, External Approach */
"0RQQ0ZZ"  =  "1"  /* Repair Right Carpal Joint, Open Approach */
"0RQQ3ZZ"  =  "1"  /* Repair Right Carpal Joint, Percutaneous Approach */
"0RQQ4ZZ"  =  "1"  /* Repair Right Carpal Joint, Percutaneous Endoscopic Approach */
"0RQQXZZ"  =  "1"  /* Repair Right Carpal Joint, External Approach */
"0RQR0ZZ"  =  "1"  /* Repair Left Carpal Joint, Open Approach */
"0RQR3ZZ"  =  "1"  /* Repair Left Carpal Joint, Percutaneous Approach */
"0RQR4ZZ"  =  "1"  /* Repair Left Carpal Joint, Percutaneous Endoscopic Approach */
"0RQRXZZ"  =  "1"  /* Repair Left Carpal Joint, External Approach */
"0RQS0ZZ"  =  "1"  /* Repair Right Metacarpocarpal Joint, Open Approach */
"0RQS3ZZ"  =  "1"  /* Repair Right Metacarpocarpal Joint, Percutaneous Approach */
"0RQS4ZZ"  =  "1"  /* Repair Right Metacarpocarpal Joint, Perc Endo Approach */
"0RQSXZZ"  =  "1"  /* Repair Right Metacarpocarpal Joint, External Approach */
"0RQT0ZZ"  =  "1"  /* Repair Left Metacarpocarpal Joint, Open Approach */
"0RQT3ZZ"  =  "1"  /* Repair Left Metacarpocarpal Joint, Percutaneous Approach */
"0RQT4ZZ"  =  "1"  /* Repair Left Metacarpocarpal Joint, Perc Endo Approach */
"0RQTXZZ"  =  "1"  /* Repair Left Metacarpocarpal Joint, External Approach */
"0RQU0ZZ"  =  "1"  /* Repair Right Metacarpophalangeal Joint, Open Approach */
"0RQU3ZZ"  =  "1"  /* Repair Right Metacarpophalangeal Joint, Perc Approach */
"0RQU4ZZ"  =  "1"  /* Repair Right Metacarpophalangeal Joint, Perc Endo Approach */
"0RQUXZZ"  =  "1"  /* Repair Right Metacarpophalangeal Joint, External Approach */
"0RQV0ZZ"  =  "1"  /* Repair Left Metacarpophalangeal Joint, Open Approach */
"0RQV3ZZ"  =  "1"  /* Repair Left Metacarpophalangeal Joint, Percutaneous Approach */
"0RQV4ZZ"  =  "1"  /* Repair Left Metacarpophalangeal Joint, Perc Endo Approach */
"0RQVXZZ"  =  "1"  /* Repair Left Metacarpophalangeal Joint, External Approach */
"0RQW0ZZ"  =  "1"  /* Repair Right Finger Phalangeal Joint, Open Approach */
"0RQW3ZZ"  =  "1"  /* Repair Right Finger Phalangeal Joint, Percutaneous Approach */
"0RQW4ZZ"  =  "1"  /* Repair Right Finger Phalangeal Joint, Perc Endo Approach */
"0RQWXZZ"  =  "1"  /* Repair Right Finger Phalangeal Joint, External Approach */
"0RQX0ZZ"  =  "1"  /* Repair Left Finger Phalangeal Joint, Open Approach */
"0RQX3ZZ"  =  "1"  /* Repair Left Finger Phalangeal Joint, Percutaneous Approach */
"0RQX4ZZ"  =  "1"  /* Repair Left Finger Phalangeal Joint, Perc Endo Approach */
"0RQXXZZ"  =  "1"  /* Repair Left Finger Phalangeal Joint, External Approach */
"0RR007Z"  =  "1"  /* Replacement of Occip Jt with Autol Sub, Open Approach */
"0RR00JZ"  =  "1"  /* Replacement of Occip Jt with Synth Sub, Open Approach */
"0RR00KZ"  =  "1"  /* Replacement of Occip Jt with Nonaut Sub, Open Approach */
"0RR107Z"  =  "1"  /* Replacement of Cerv Jt with Autol Sub, Open Approach */
"0RR10JZ"  =  "1"  /* Replacement of Cerv Jt with Synth Sub, Open Approach */
"0RR10KZ"  =  "1"  /* Replacement of Cerv Jt with Nonaut Sub, Open Approach */
"0RR307Z"  =  "1"  /* Replacement of Cerv Disc with Autol Sub, Open Approach */
"0RR30JZ"  =  "1"  /* Replacement of Cerv Disc with Synth Sub, Open Approach */
"0RR30KZ"  =  "1"  /* Replacement of Cerv Disc with Nonaut Sub, Open Approach */
"0RR407Z"  =  "1"  /* Replacement of C-thor Jt with Autol Sub, Open Approach */
"0RR40JZ"  =  "1"  /* Replacement of C-thor Jt with Synth Sub, Open Approach */
"0RR40KZ"  =  "1"  /* Replacement of C-thor Jt with Nonaut Sub, Open Approach */
"0RR507Z"  =  "1"  /* Replacement of C-thor Disc with Autol Sub, Open Approach */
"0RR50JZ"  =  "1"  /* Replacement of C-thor Disc with Synth Sub, Open Approach */
"0RR50KZ"  =  "1"  /* Replacement of C-thor Disc with Nonaut Sub, Open Approach */
"0RR607Z"  =  "1"  /* Replacement of Thor Jt with Autol Sub, Open Approach */
"0RR60JZ"  =  "1"  /* Replacement of Thor Jt with Synth Sub, Open Approach */
"0RR60KZ"  =  "1"  /* Replacement of Thor Jt with Nonaut Sub, Open Approach */
"0RR907Z"  =  "1"  /* Replacement of Thor Disc with Autol Sub, Open Approach */
"0RR90JZ"  =  "1"  /* Replacement of Thor Disc with Synth Sub, Open Approach */
"0RR90KZ"  =  "1"  /* Replacement of Thor Disc with Nonaut Sub, Open Approach */
"0RRA07Z"  =  "1"  /* Replacement of T-lum Jt with Autol Sub, Open Approach */
"0RRA0JZ"  =  "1"  /* Replacement of T-lum Jt with Synth Sub, Open Approach */
"0RRA0KZ"  =  "1"  /* Replacement of T-lum Jt with Nonaut Sub, Open Approach */
"0RRB07Z"  =  "1"  /* Replacement of T-lum Disc with Autol Sub, Open Approach */
"0RRB0JZ"  =  "1"  /* Replacement of T-lum Disc with Synth Sub, Open Approach */
"0RRB0KZ"  =  "1"  /* Replacement of T-lum Disc with Nonaut Sub, Open Approach */
"0RRC07Z"  =  "1"  /* Replace of R Temporomandib Jt with Autol Sub, Open Approach */
"0RRC0JZ"  =  "1"  /* Replace of R Temporomandib Jt with Synth Sub, Open Approach */
"0RRC0KZ"  =  "1"  /* Replace of R Temporomandib Jt with Nonaut Sub, Open Approach */
"0RRD07Z"  =  "1"  /* Replace of L Temporomandib Jt with Autol Sub, Open Approach */
"0RRD0JZ"  =  "1"  /* Replace of L Temporomandib Jt with Synth Sub, Open Approach */
"0RRD0KZ"  =  "1"  /* Replace of L Temporomandib Jt with Nonaut Sub, Open Approach */
"0RRE07Z"  =  "1"  /* Replacement of R Sternoclav Jt with Autol Sub, Open Approach */
"0RRE0JZ"  =  "1"  /* Replacement of R Sternoclav Jt with Synth Sub, Open Approach */
"0RRE0KZ"  =  "1"  /* Replace of R Sternoclav Jt with Nonaut Sub, Open Approach */
"0RRF07Z"  =  "1"  /* Replacement of L Sternoclav Jt with Autol Sub, Open Approach */
"0RRF0JZ"  =  "1"  /* Replacement of L Sternoclav Jt with Synth Sub, Open Approach */
"0RRF0KZ"  =  "1"  /* Replace of L Sternoclav Jt with Nonaut Sub, Open Approach */
"0RRG07Z"  =  "1"  /* Replace of R Acromioclav Jt with Autol Sub, Open Approach */
"0RRG0JZ"  =  "1"  /* Replace of R Acromioclav Jt with Synth Sub, Open Approach */
"0RRG0KZ"  =  "1"  /* Replace of R Acromioclav Jt with Nonaut Sub, Open Approach */
"0RRH07Z"  =  "1"  /* Replace of L Acromioclav Jt with Autol Sub, Open Approach */
"0RRH0JZ"  =  "1"  /* Replace of L Acromioclav Jt with Synth Sub, Open Approach */
"0RRH0KZ"  =  "1"  /* Replace of L Acromioclav Jt with Nonaut Sub, Open Approach */
"0RRJ00Z"  =  "1"  /* Replace of R Shoulder Jt with Rev Bl & Sockt, Open Approach */
"0RRJ07Z"  =  "1"  /* Replacement of R Shoulder Jt with Autol Sub, Open Approach */
"0RRJ0J6"  =  "1"  /* Replace R Shoulder Jt w Synth Sub, Humeral, Open */
"0RRJ0J7"  =  "1"  /* Replace R Shoulder Jt w Synth Sub, Glenoid, Open */
"0RRJ0JZ"  =  "1"  /* Replacement of R Shoulder Jt with Synth Sub, Open Approach */
"0RRJ0KZ"  =  "1"  /* Replacement of R Shoulder Jt with Nonaut Sub, Open Approach */
"0RRK00Z"  =  "1"  /* Replace of L Shoulder Jt with Rev Bl & Sockt, Open Approach */
"0RRK07Z"  =  "1"  /* Replacement of L Shoulder Jt with Autol Sub, Open Approach */
"0RRK0J6"  =  "1"  /* Replace L Shoulder Jt w Synth Sub, Humeral, Open */
"0RRK0J7"  =  "1"  /* Replace L Shoulder Jt w Synth Sub, Glenoid, Open */
"0RRK0JZ"  =  "1"  /* Replacement of L Shoulder Jt with Synth Sub, Open Approach */
"0RRK0KZ"  =  "1"  /* Replacement of L Shoulder Jt with Nonaut Sub, Open Approach */
"0RRL07Z"  =  "1"  /* Replacement of R Elbow Jt with Autol Sub, Open Approach */
"0RRL0JZ"  =  "1"  /* Replacement of R Elbow Jt with Synth Sub, Open Approach */
"0RRL0KZ"  =  "1"  /* Replacement of R Elbow Jt with Nonaut Sub, Open Approach */
"0RRM07Z"  =  "1"  /* Replacement of L Elbow Jt with Autol Sub, Open Approach */
"0RRM0JZ"  =  "1"  /* Replacement of L Elbow Jt with Synth Sub, Open Approach */
"0RRM0KZ"  =  "1"  /* Replacement of L Elbow Jt with Nonaut Sub, Open Approach */
"0RRN07Z"  =  "1"  /* Replacement of R Wrist Jt with Autol Sub, Open Approach */
"0RRN0JZ"  =  "1"  /* Replacement of R Wrist Jt with Synth Sub, Open Approach */
"0RRN0KZ"  =  "1"  /* Replacement of R Wrist Jt with Nonaut Sub, Open Approach */
"0RRP07Z"  =  "1"  /* Replacement of L Wrist Jt with Autol Sub, Open Approach */
"0RRP0JZ"  =  "1"  /* Replacement of L Wrist Jt with Synth Sub, Open Approach */
"0RRP0KZ"  =  "1"  /* Replacement of L Wrist Jt with Nonaut Sub, Open Approach */
"0RRQ07Z"  =  "1"  /* Replacement of R Carpal Jt with Autol Sub, Open Approach */
"0RRQ0JZ"  =  "1"  /* Replacement of R Carpal Jt with Synth Sub, Open Approach */
"0RRQ0KZ"  =  "1"  /* Replacement of R Carpal Jt with Nonaut Sub, Open Approach */
"0RRR07Z"  =  "1"  /* Replacement of L Carpal Jt with Autol Sub, Open Approach */
"0RRR0JZ"  =  "1"  /* Replacement of L Carpal Jt with Synth Sub, Open Approach */
"0RRR0KZ"  =  "1"  /* Replacement of L Carpal Jt with Nonaut Sub, Open Approach */
"0RRS07Z"  =  "1"  /* Replace of R Metacarpocarp Jt with Autol Sub, Open Approach */
"0RRS0JZ"  =  "1"  /* Replace of R Metacarpocarp Jt with Synth Sub, Open Approach */
"0RRS0KZ"  =  "1"  /* Replace of R Metacarpocarp Jt with Nonaut Sub, Open Approach */
"0RRT07Z"  =  "1"  /* Replace of L Metacarpocarp Jt with Autol Sub, Open Approach */
"0RRT0JZ"  =  "1"  /* Replace of L Metacarpocarp Jt with Synth Sub, Open Approach */
"0RRT0KZ"  =  "1"  /* Replace of L Metacarpocarp Jt with Nonaut Sub, Open Approach */
"0RRU07Z"  =  "1"  /* Replace of R Metacarpophal Jt with Autol Sub, Open Approach */
"0RRU0JZ"  =  "1"  /* Replace of R Metacarpophal Jt with Synth Sub, Open Approach */
"0RRU0KZ"  =  "1"  /* Replace of R Metacarpophal Jt with Nonaut Sub, Open Approach */
"0RRV07Z"  =  "1"  /* Replace of L Metacarpophal Jt with Autol Sub, Open Approach */
"0RRV0JZ"  =  "1"  /* Replace of L Metacarpophal Jt with Synth Sub, Open Approach */
"0RRV0KZ"  =  "1"  /* Replace of L Metacarpophal Jt with Nonaut Sub, Open Approach */
"0RRW07Z"  =  "1"  /* Replace of R Finger Phalanx Jt with Autol Sub, Open Approach */
"0RRW0JZ"  =  "1"  /* Replace of R Finger Phalanx Jt with Synth Sub, Open Approach */
"0RRW0KZ"  =  "1"  /* Replace R Finger Phalanx Jt w Nonaut Sub, Open */
"0RRX07Z"  =  "1"  /* Replace of L Finger Phalanx Jt with Autol Sub, Open Approach */
"0RRX0JZ"  =  "1"  /* Replace of L Finger Phalanx Jt with Synth Sub, Open Approach */
"0RRX0KZ"  =  "1"  /* Replace L Finger Phalanx Jt w Nonaut Sub, Open */
"0RS004Z"  =  "1"  /* Reposition Occip Jt with Int Fix, Open Approach */
"0RS00ZZ"  =  "1"  /* Reposition Occipital-cervical Joint, Open Approach */
"0RS104Z"  =  "1"  /* Reposition Cerv Jt with Int Fix, Open Approach */
"0RS10ZZ"  =  "1"  /* Reposition Cervical Vertebral Joint, Open Approach */
"0RS404Z"  =  "1"  /* Reposition C-thor Jt with Int Fix, Open Approach */
"0RS40ZZ"  =  "1"  /* Reposition Cervicothoracic Vertebral Joint, Open Approach */
"0RS604Z"  =  "1"  /* Reposition Thor Jt with Int Fix, Open Approach */
"0RS60ZZ"  =  "1"  /* Reposition Thoracic Vertebral Joint, Open Approach */
"0RSA04Z"  =  "1"  /* Reposition T-lum Jt with Int Fix, Open Approach */
"0RSA0ZZ"  =  "1"  /* Reposition Thoracolumbar Vertebral Joint, Open Approach */
"0RSC04Z"  =  "1"  /* Reposition R Temporomandib Jt with Int Fix, Open Approach */
"0RSC0ZZ"  =  "1"  /* Reposition Right Temporomandibular Joint, Open Approach */
"0RSD04Z"  =  "1"  /* Reposition L Temporomandib Jt with Int Fix, Open Approach */
"0RSD0ZZ"  =  "1"  /* Reposition Left Temporomandibular Joint, Open Approach */
"0RSE04Z"  =  "1"  /* Reposition R Sternoclav Jt with Int Fix, Open Approach */
"0RSE0ZZ"  =  "1"  /* Reposition Right Sternoclavicular Joint, Open Approach */
"0RSF04Z"  =  "1"  /* Reposition L Sternoclav Jt with Int Fix, Open Approach */
"0RSF0ZZ"  =  "1"  /* Reposition Left Sternoclavicular Joint, Open Approach */
"0RSG04Z"  =  "1"  /* Reposition R Acromioclav Jt with Int Fix, Open Approach */
"0RSG0ZZ"  =  "1"  /* Reposition Right Acromioclavicular Joint, Open Approach */
"0RSH04Z"  =  "1"  /* Reposition L Acromioclav Jt with Int Fix, Open Approach */
"0RSH0ZZ"  =  "1"  /* Reposition Left Acromioclavicular Joint, Open Approach */
"0RSJ04Z"  =  "1"  /* Reposition Right Shoulder Joint with Int Fix, Open Approach */
"0RSJ0ZZ"  =  "1"  /* Reposition Right Shoulder Joint, Open Approach */
"0RSK04Z"  =  "1"  /* Reposition Left Shoulder Joint with Int Fix, Open Approach */
"0RSK0ZZ"  =  "1"  /* Reposition Left Shoulder Joint, Open Approach */
"0RSL04Z"  =  "1"  /* Reposition Right Elbow Joint with Int Fix, Open Approach */
"0RSL05Z"  =  "1"  /* Reposition Right Elbow Joint with Ext Fix, Open Approach */
"0RSL0ZZ"  =  "1"  /* Reposition Right Elbow Joint, Open Approach */
"0RSM04Z"  =  "1"  /* Reposition Left Elbow Joint with Int Fix, Open Approach */
"0RSM05Z"  =  "1"  /* Reposition Left Elbow Joint with Ext Fix, Open Approach */
"0RSM0ZZ"  =  "1"  /* Reposition Left Elbow Joint, Open Approach */
"0RSN04Z"  =  "1"  /* Reposition Right Wrist Joint with Int Fix, Open Approach */
"0RSN05Z"  =  "1"  /* Reposition Right Wrist Joint with Ext Fix, Open Approach */
"0RSN0ZZ"  =  "1"  /* Reposition Right Wrist Joint, Open Approach */
"0RSP04Z"  =  "1"  /* Reposition Left Wrist Joint with Int Fix, Open Approach */
"0RSP05Z"  =  "1"  /* Reposition Left Wrist Joint with Ext Fix, Open Approach */
"0RSP0ZZ"  =  "1"  /* Reposition Left Wrist Joint, Open Approach */
"0RSQ04Z"  =  "1"  /* Reposition Right Carpal Joint with Int Fix, Open Approach */
"0RSQ05Z"  =  "1"  /* Reposition Right Carpal Joint with Ext Fix, Open Approach */
"0RSQ0ZZ"  =  "1"  /* Reposition Right Carpal Joint, Open Approach */
"0RSR04Z"  =  "1"  /* Reposition Left Carpal Joint with Int Fix, Open Approach */
"0RSR05Z"  =  "1"  /* Reposition Left Carpal Joint with Ext Fix, Open Approach */
"0RSR0ZZ"  =  "1"  /* Reposition Left Carpal Joint, Open Approach */
"0RSS04Z"  =  "1"  /* Reposition R Metacarpocarp Jt with Int Fix, Open Approach */
"0RSS05Z"  =  "1"  /* Reposition R Metacarpocarp Jt with Ext Fix, Open Approach */
"0RSS0ZZ"  =  "1"  /* Reposition Right Metacarpocarpal Joint, Open Approach */
"0RST04Z"  =  "1"  /* Reposition L Metacarpocarp Jt with Int Fix, Open Approach */
"0RST05Z"  =  "1"  /* Reposition L Metacarpocarp Jt with Ext Fix, Open Approach */
"0RST0ZZ"  =  "1"  /* Reposition Left Metacarpocarpal Joint, Open Approach */
"0RSU04Z"  =  "1"  /* Reposition R Metacarpophal Jt with Int Fix, Open Approach */
"0RSU05Z"  =  "1"  /* Reposition R Metacarpophal Jt with Ext Fix, Open Approach */
"0RSU0ZZ"  =  "1"  /* Reposition Right Metacarpophalangeal Joint, Open Approach */
"0RSV04Z"  =  "1"  /* Reposition L Metacarpophal Jt with Int Fix, Open Approach */
"0RSV05Z"  =  "1"  /* Reposition L Metacarpophal Jt with Ext Fix, Open Approach */
"0RSV0ZZ"  =  "1"  /* Reposition Left Metacarpophalangeal Joint, Open Approach */
"0RSW04Z"  =  "1"  /* Reposition R Finger Phalanx Jt with Int Fix, Open Approach */
"0RSW05Z"  =  "1"  /* Reposition R Finger Phalanx Jt with Ext Fix, Open Approach */
"0RSW0ZZ"  =  "1"  /* Reposition Right Finger Phalangeal Joint, Open Approach */
"0RSX04Z"  =  "1"  /* Reposition L Finger Phalanx Jt with Int Fix, Open Approach */
"0RSX05Z"  =  "1"  /* Reposition L Finger Phalanx Jt with Ext Fix, Open Approach */
"0RSX0ZZ"  =  "1"  /* Reposition Left Finger Phalangeal Joint, Open Approach */
"0RT30ZZ"  =  "1"  /* Resection of Cervical Vertebral Disc, Open Approach */
"0RT40ZZ"  =  "1"  /* Resection of Cervicothoracic Vertebral Joint, Open Approach */
"0RT50ZZ"  =  "1"  /* Resection of Cervicothoracic Vertebral Disc, Open Approach */
"0RT90ZZ"  =  "1"  /* Resection of Thoracic Vertebral Disc, Open Approach */
"0RTB0ZZ"  =  "1"  /* Resection of Thoracolumbar Vertebral Disc, Open Approach */
"0RTC0ZZ"  =  "1"  /* Resection of Right Temporomandibular Joint, Open Approach */
"0RTD0ZZ"  =  "1"  /* Resection of Left Temporomandibular Joint, Open Approach */
"0RTE0ZZ"  =  "1"  /* Resection of Right Sternoclavicular Joint, Open Approach */
"0RTF0ZZ"  =  "1"  /* Resection of Left Sternoclavicular Joint, Open Approach */
"0RTG0ZZ"  =  "1"  /* Resection of Right Acromioclavicular Joint, Open Approach */
"0RTH0ZZ"  =  "1"  /* Resection of Left Acromioclavicular Joint, Open Approach */
"0RTJ0ZZ"  =  "1"  /* Resection of Right Shoulder Joint, Open Approach */
"0RTK0ZZ"  =  "1"  /* Resection of Left Shoulder Joint, Open Approach */
"0RTL0ZZ"  =  "1"  /* Resection of Right Elbow Joint, Open Approach */
"0RTM0ZZ"  =  "1"  /* Resection of Left Elbow Joint, Open Approach */
"0RTN0ZZ"  =  "1"  /* Resection of Right Wrist Joint, Open Approach */
"0RTP0ZZ"  =  "1"  /* Resection of Left Wrist Joint, Open Approach */
"0RTQ0ZZ"  =  "1"  /* Resection of Right Carpal Joint, Open Approach */
"0RTR0ZZ"  =  "1"  /* Resection of Left Carpal Joint, Open Approach */
"0RTS0ZZ"  =  "1"  /* Resection of Right Metacarpocarpal Joint, Open Approach */
"0RTT0ZZ"  =  "1"  /* Resection of Left Metacarpocarpal Joint, Open Approach */
"0RTU0ZZ"  =  "1"  /* Resection of Right Metacarpophalangeal Joint, Open Approach */
"0RTV0ZZ"  =  "1"  /* Resection of Left Metacarpophalangeal Joint, Open Approach */
"0RTW0ZZ"  =  "1"  /* Resection of Right Finger Phalangeal Joint, Open Approach */
"0RTX0ZZ"  =  "1"  /* Resection of Left Finger Phalangeal Joint, Open Approach */
"0RU007Z"  =  "1"  /* Supplement Occip Jt with Autol Sub, Open Approach */
"0RU00JZ"  =  "1"  /* Supplement Occip Jt with Synth Sub, Open Approach */
"0RU00KZ"  =  "1"  /* Supplement Occip Jt with Nonaut Sub, Open Approach */
"0RU037Z"  =  "1"  /* Supplement Occip Jt with Autol Sub, Perc Approach */
"0RU03JZ"  =  "1"  /* Supplement Occip Jt with Synth Sub, Perc Approach */
"0RU03KZ"  =  "1"  /* Supplement Occip Jt with Nonaut Sub, Perc Approach */
"0RU047Z"  =  "1"  /* Supplement Occip Jt with Autol Sub, Perc Endo Approach */
"0RU04JZ"  =  "1"  /* Supplement Occip Jt with Synth Sub, Perc Endo Approach */
"0RU04KZ"  =  "1"  /* Supplement Occip Jt with Nonaut Sub, Perc Endo Approach */
"0RU107Z"  =  "1"  /* Supplement Cerv Jt with Autol Sub, Open Approach */
"0RU10JZ"  =  "1"  /* Supplement Cerv Jt with Synth Sub, Open Approach */
"0RU10KZ"  =  "1"  /* Supplement Cerv Jt with Nonaut Sub, Open Approach */
"0RU137Z"  =  "1"  /* Supplement Cerv Jt with Autol Sub, Perc Approach */
"0RU13JZ"  =  "1"  /* Supplement Cerv Jt with Synth Sub, Perc Approach */
"0RU13KZ"  =  "1"  /* Supplement Cerv Jt with Nonaut Sub, Perc Approach */
"0RU147Z"  =  "1"  /* Supplement Cerv Jt with Autol Sub, Perc Endo Approach */
"0RU14JZ"  =  "1"  /* Supplement Cerv Jt with Synth Sub, Perc Endo Approach */
"0RU14KZ"  =  "1"  /* Supplement Cerv Jt with Nonaut Sub, Perc Endo Approach */
"0RU307Z"  =  "1"  /* Supplement Cerv Disc with Autol Sub, Open Approach */
"0RU30JZ"  =  "1"  /* Supplement Cerv Disc with Synth Sub, Open Approach */
"0RU30KZ"  =  "1"  /* Supplement Cerv Disc with Nonaut Sub, Open Approach */
"0RU337Z"  =  "1"  /* Supplement Cerv Disc with Autol Sub, Perc Approach */
"0RU33JZ"  =  "1"  /* Supplement Cerv Disc with Synth Sub, Perc Approach */
"0RU33KZ"  =  "1"  /* Supplement Cerv Disc with Nonaut Sub, Perc Approach */
"0RU347Z"  =  "1"  /* Supplement Cerv Disc with Autol Sub, Perc Endo Approach */
"0RU34JZ"  =  "1"  /* Supplement Cerv Disc with Synth Sub, Perc Endo Approach */
"0RU34KZ"  =  "1"  /* Supplement Cerv Disc with Nonaut Sub, Perc Endo Approach */
"0RU407Z"  =  "1"  /* Supplement C-thor Jt with Autol Sub, Open Approach */
"0RU40JZ"  =  "1"  /* Supplement C-thor Jt with Synth Sub, Open Approach */
"0RU40KZ"  =  "1"  /* Supplement C-thor Jt with Nonaut Sub, Open Approach */
"0RU437Z"  =  "1"  /* Supplement C-thor Jt with Autol Sub, Perc Approach */
"0RU43JZ"  =  "1"  /* Supplement C-thor Jt with Synth Sub, Perc Approach */
"0RU43KZ"  =  "1"  /* Supplement C-thor Jt with Nonaut Sub, Perc Approach */
"0RU447Z"  =  "1"  /* Supplement C-thor Jt with Autol Sub, Perc Endo Approach */
"0RU44JZ"  =  "1"  /* Supplement C-thor Jt with Synth Sub, Perc Endo Approach */
"0RU44KZ"  =  "1"  /* Supplement C-thor Jt with Nonaut Sub, Perc Endo Approach */
"0RU507Z"  =  "1"  /* Supplement C-thor Disc with Autol Sub, Open Approach */
"0RU50JZ"  =  "1"  /* Supplement C-thor Disc with Synth Sub, Open Approach */
"0RU50KZ"  =  "1"  /* Supplement C-thor Disc with Nonaut Sub, Open Approach */
"0RU537Z"  =  "1"  /* Supplement C-thor Disc with Autol Sub, Perc Approach */
"0RU53JZ"  =  "1"  /* Supplement C-thor Disc with Synth Sub, Perc Approach */
"0RU53KZ"  =  "1"  /* Supplement C-thor Disc with Nonaut Sub, Perc Approach */
"0RU547Z"  =  "1"  /* Supplement C-thor Disc with Autol Sub, Perc Endo Approach */
"0RU54JZ"  =  "1"  /* Supplement C-thor Disc with Synth Sub, Perc Endo Approach */
"0RU54KZ"  =  "1"  /* Supplement C-thor Disc with Nonaut Sub, Perc Endo Approach */
"0RU607Z"  =  "1"  /* Supplement Thor Jt with Autol Sub, Open Approach */
"0RU60JZ"  =  "1"  /* Supplement Thor Jt with Synth Sub, Open Approach */
"0RU60KZ"  =  "1"  /* Supplement Thor Jt with Nonaut Sub, Open Approach */
"0RU637Z"  =  "1"  /* Supplement Thor Jt with Autol Sub, Perc Approach */
"0RU63JZ"  =  "1"  /* Supplement Thor Jt with Synth Sub, Perc Approach */
"0RU63KZ"  =  "1"  /* Supplement Thor Jt with Nonaut Sub, Perc Approach */
"0RU647Z"  =  "1"  /* Supplement Thor Jt with Autol Sub, Perc Endo Approach */
"0RU64JZ"  =  "1"  /* Supplement Thor Jt with Synth Sub, Perc Endo Approach */
"0RU64KZ"  =  "1"  /* Supplement Thor Jt with Nonaut Sub, Perc Endo Approach */
"0RU907Z"  =  "1"  /* Supplement Thor Disc with Autol Sub, Open Approach */
"0RU90JZ"  =  "1"  /* Supplement Thor Disc with Synth Sub, Open Approach */
"0RU90KZ"  =  "1"  /* Supplement Thor Disc with Nonaut Sub, Open Approach */
"0RU937Z"  =  "1"  /* Supplement Thor Disc with Autol Sub, Perc Approach */
"0RU93JZ"  =  "1"  /* Supplement Thor Disc with Synth Sub, Perc Approach */
"0RU93KZ"  =  "1"  /* Supplement Thor Disc with Nonaut Sub, Perc Approach */
"0RU947Z"  =  "1"  /* Supplement Thor Disc with Autol Sub, Perc Endo Approach */
"0RU94JZ"  =  "1"  /* Supplement Thor Disc with Synth Sub, Perc Endo Approach */
"0RU94KZ"  =  "1"  /* Supplement Thor Disc with Nonaut Sub, Perc Endo Approach */
"0RUA07Z"  =  "1"  /* Supplement T-lum Jt with Autol Sub, Open Approach */
"0RUA0JZ"  =  "1"  /* Supplement T-lum Jt with Synth Sub, Open Approach */
"0RUA0KZ"  =  "1"  /* Supplement T-lum Jt with Nonaut Sub, Open Approach */
"0RUA37Z"  =  "1"  /* Supplement T-lum Jt with Autol Sub, Perc Approach */
"0RUA3JZ"  =  "1"  /* Supplement T-lum Jt with Synth Sub, Perc Approach */
"0RUA3KZ"  =  "1"  /* Supplement T-lum Jt with Nonaut Sub, Perc Approach */
"0RUA47Z"  =  "1"  /* Supplement T-lum Jt with Autol Sub, Perc Endo Approach */
"0RUA4JZ"  =  "1"  /* Supplement T-lum Jt with Synth Sub, Perc Endo Approach */
"0RUA4KZ"  =  "1"  /* Supplement T-lum Jt with Nonaut Sub, Perc Endo Approach */
"0RUB07Z"  =  "1"  /* Supplement T-lum Disc with Autol Sub, Open Approach */
"0RUB0JZ"  =  "1"  /* Supplement T-lum Disc with Synth Sub, Open Approach */
"0RUB0KZ"  =  "1"  /* Supplement T-lum Disc with Nonaut Sub, Open Approach */
"0RUB37Z"  =  "1"  /* Supplement T-lum Disc with Autol Sub, Perc Approach */
"0RUB3JZ"  =  "1"  /* Supplement T-lum Disc with Synth Sub, Perc Approach */
"0RUB3KZ"  =  "1"  /* Supplement T-lum Disc with Nonaut Sub, Perc Approach */
"0RUB47Z"  =  "1"  /* Supplement T-lum Disc with Autol Sub, Perc Endo Approach */
"0RUB4JZ"  =  "1"  /* Supplement T-lum Disc with Synth Sub, Perc Endo Approach */
"0RUB4KZ"  =  "1"  /* Supplement T-lum Disc with Nonaut Sub, Perc Endo Approach */
"0RUC07Z"  =  "1"  /* Supplement R Temporomandib Jt with Autol Sub, Open Approach */
"0RUC0JZ"  =  "1"  /* Supplement R Temporomandib Jt with Synth Sub, Open Approach */
"0RUC0KZ"  =  "1"  /* Supplement R Temporomandib Jt with Nonaut Sub, Open Approach */
"0RUC37Z"  =  "1"  /* Supplement R Temporomandib Jt with Autol Sub, Perc Approach */
"0RUC3JZ"  =  "1"  /* Supplement R Temporomandib Jt with Synth Sub, Perc Approach */
"0RUC3KZ"  =  "1"  /* Supplement R Temporomandib Jt with Nonaut Sub, Perc Approach */
"0RUC47Z"  =  "1"  /* Supplement R Temporomandib Jt w Autol Sub, Perc Endo */
"0RUC4JZ"  =  "1"  /* Supplement R Temporomandib Jt w Synth Sub, Perc Endo */
"0RUC4KZ"  =  "1"  /* Supplement R Temporomandib Jt w Nonaut Sub, Perc Endo */
"0RUD07Z"  =  "1"  /* Supplement L Temporomandib Jt with Autol Sub, Open Approach */
"0RUD0JZ"  =  "1"  /* Supplement L Temporomandib Jt with Synth Sub, Open Approach */
"0RUD0KZ"  =  "1"  /* Supplement L Temporomandib Jt with Nonaut Sub, Open Approach */
"0RUD37Z"  =  "1"  /* Supplement L Temporomandib Jt with Autol Sub, Perc Approach */
"0RUD3JZ"  =  "1"  /* Supplement L Temporomandib Jt with Synth Sub, Perc Approach */
"0RUD3KZ"  =  "1"  /* Supplement L Temporomandib Jt with Nonaut Sub, Perc Approach */
"0RUD47Z"  =  "1"  /* Supplement L Temporomandib Jt w Autol Sub, Perc Endo */
"0RUD4JZ"  =  "1"  /* Supplement L Temporomandib Jt w Synth Sub, Perc Endo */
"0RUD4KZ"  =  "1"  /* Supplement L Temporomandib Jt w Nonaut Sub, Perc Endo */
"0RUE07Z"  =  "1"  /* Supplement R Sternoclav Jt with Autol Sub, Open Approach */
"0RUE0JZ"  =  "1"  /* Supplement R Sternoclav Jt with Synth Sub, Open Approach */
"0RUE0KZ"  =  "1"  /* Supplement R Sternoclav Jt with Nonaut Sub, Open Approach */
"0RUE37Z"  =  "1"  /* Supplement R Sternoclav Jt with Autol Sub, Perc Approach */
"0RUE3JZ"  =  "1"  /* Supplement R Sternoclav Jt with Synth Sub, Perc Approach */
"0RUE3KZ"  =  "1"  /* Supplement R Sternoclav Jt with Nonaut Sub, Perc Approach */
"0RUE47Z"  =  "1"  /* Supplement R Sternoclav Jt w Autol Sub, Perc Endo */
"0RUE4JZ"  =  "1"  /* Supplement R Sternoclav Jt w Synth Sub, Perc Endo */
"0RUE4KZ"  =  "1"  /* Supplement R Sternoclav Jt w Nonaut Sub, Perc Endo */
"0RUF07Z"  =  "1"  /* Supplement L Sternoclav Jt with Autol Sub, Open Approach */
"0RUF0JZ"  =  "1"  /* Supplement L Sternoclav Jt with Synth Sub, Open Approach */
"0RUF0KZ"  =  "1"  /* Supplement L Sternoclav Jt with Nonaut Sub, Open Approach */
"0RUF37Z"  =  "1"  /* Supplement L Sternoclav Jt with Autol Sub, Perc Approach */
"0RUF3JZ"  =  "1"  /* Supplement L Sternoclav Jt with Synth Sub, Perc Approach */
"0RUF3KZ"  =  "1"  /* Supplement L Sternoclav Jt with Nonaut Sub, Perc Approach */
"0RUF47Z"  =  "1"  /* Supplement L Sternoclav Jt w Autol Sub, Perc Endo */
"0RUF4JZ"  =  "1"  /* Supplement L Sternoclav Jt w Synth Sub, Perc Endo */
"0RUF4KZ"  =  "1"  /* Supplement L Sternoclav Jt w Nonaut Sub, Perc Endo */
"0RUG07Z"  =  "1"  /* Supplement R Acromioclav Jt with Autol Sub, Open Approach */
"0RUG0JZ"  =  "1"  /* Supplement R Acromioclav Jt with Synth Sub, Open Approach */
"0RUG0KZ"  =  "1"  /* Supplement R Acromioclav Jt with Nonaut Sub, Open Approach */
"0RUG37Z"  =  "1"  /* Supplement R Acromioclav Jt with Autol Sub, Perc Approach */
"0RUG3JZ"  =  "1"  /* Supplement R Acromioclav Jt with Synth Sub, Perc Approach */
"0RUG3KZ"  =  "1"  /* Supplement R Acromioclav Jt with Nonaut Sub, Perc Approach */
"0RUG47Z"  =  "1"  /* Supplement R Acromioclav Jt w Autol Sub, Perc Endo */
"0RUG4JZ"  =  "1"  /* Supplement R Acromioclav Jt w Synth Sub, Perc Endo */
"0RUG4KZ"  =  "1"  /* Supplement R Acromioclav Jt w Nonaut Sub, Perc Endo */
"0RUH07Z"  =  "1"  /* Supplement L Acromioclav Jt with Autol Sub, Open Approach */
"0RUH0JZ"  =  "1"  /* Supplement L Acromioclav Jt with Synth Sub, Open Approach */
"0RUH0KZ"  =  "1"  /* Supplement L Acromioclav Jt with Nonaut Sub, Open Approach */
"0RUH37Z"  =  "1"  /* Supplement L Acromioclav Jt with Autol Sub, Perc Approach */
"0RUH3JZ"  =  "1"  /* Supplement L Acromioclav Jt with Synth Sub, Perc Approach */
"0RUH3KZ"  =  "1"  /* Supplement L Acromioclav Jt with Nonaut Sub, Perc Approach */
"0RUH47Z"  =  "1"  /* Supplement L Acromioclav Jt w Autol Sub, Perc Endo */
"0RUH4JZ"  =  "1"  /* Supplement L Acromioclav Jt w Synth Sub, Perc Endo */
"0RUH4KZ"  =  "1"  /* Supplement L Acromioclav Jt w Nonaut Sub, Perc Endo */
"0RUJ07Z"  =  "1"  /* Supplement R Shoulder Jt with Autol Sub, Open Approach */
"0RUJ0JZ"  =  "1"  /* Supplement R Shoulder Jt with Synth Sub, Open Approach */
"0RUJ0KZ"  =  "1"  /* Supplement R Shoulder Jt with Nonaut Sub, Open Approach */
"0RUJ37Z"  =  "1"  /* Supplement R Shoulder Jt with Autol Sub, Perc Approach */
"0RUJ3JZ"  =  "1"  /* Supplement R Shoulder Jt with Synth Sub, Perc Approach */
"0RUJ3KZ"  =  "1"  /* Supplement R Shoulder Jt with Nonaut Sub, Perc Approach */
"0RUJ47Z"  =  "1"  /* Supplement R Shoulder Jt with Autol Sub, Perc Endo Approach */
"0RUJ4JZ"  =  "1"  /* Supplement R Shoulder Jt with Synth Sub, Perc Endo Approach */
"0RUJ4KZ"  =  "1"  /* Supplement R Shoulder Jt with Nonaut Sub, Perc Endo Approach */
"0RUK07Z"  =  "1"  /* Supplement Left Shoulder Joint with Autol Sub, Open Approach */
"0RUK0JZ"  =  "1"  /* Supplement Left Shoulder Joint with Synth Sub, Open Approach */
"0RUK0KZ"  =  "1"  /* Supplement L Shoulder Jt with Nonaut Sub, Open Approach */
"0RUK37Z"  =  "1"  /* Supplement Left Shoulder Joint with Autol Sub, Perc Approach */
"0RUK3JZ"  =  "1"  /* Supplement Left Shoulder Joint with Synth Sub, Perc Approach */
"0RUK3KZ"  =  "1"  /* Supplement L Shoulder Jt with Nonaut Sub, Perc Approach */
"0RUK47Z"  =  "1"  /* Supplement L Shoulder Jt with Autol Sub, Perc Endo Approach */
"0RUK4JZ"  =  "1"  /* Supplement L Shoulder Jt with Synth Sub, Perc Endo Approach */
"0RUK4KZ"  =  "1"  /* Supplement L Shoulder Jt with Nonaut Sub, Perc Endo Approach */
"0RUL07Z"  =  "1"  /* Supplement Right Elbow Joint with Autol Sub, Open Approach */
"0RUL0JZ"  =  "1"  /* Supplement Right Elbow Joint with Synth Sub, Open Approach */
"0RUL0KZ"  =  "1"  /* Supplement Right Elbow Joint with Nonaut Sub, Open Approach */
"0RUL37Z"  =  "1"  /* Supplement Right Elbow Joint with Autol Sub, Perc Approach */
"0RUL3JZ"  =  "1"  /* Supplement Right Elbow Joint with Synth Sub, Perc Approach */
"0RUL3KZ"  =  "1"  /* Supplement Right Elbow Joint with Nonaut Sub, Perc Approach */
"0RUL47Z"  =  "1"  /* Supplement R Elbow Jt with Autol Sub, Perc Endo Approach */
"0RUL4JZ"  =  "1"  /* Supplement R Elbow Jt with Synth Sub, Perc Endo Approach */
"0RUL4KZ"  =  "1"  /* Supplement R Elbow Jt with Nonaut Sub, Perc Endo Approach */
"0RUM07Z"  =  "1"  /* Supplement Left Elbow Joint with Autol Sub, Open Approach */
"0RUM0JZ"  =  "1"  /* Supplement Left Elbow Joint with Synth Sub, Open Approach */
"0RUM0KZ"  =  "1"  /* Supplement Left Elbow Joint with Nonaut Sub, Open Approach */
"0RUM37Z"  =  "1"  /* Supplement Left Elbow Joint with Autol Sub, Perc Approach */
"0RUM3JZ"  =  "1"  /* Supplement Left Elbow Joint with Synth Sub, Perc Approach */
"0RUM3KZ"  =  "1"  /* Supplement Left Elbow Joint with Nonaut Sub, Perc Approach */
"0RUM47Z"  =  "1"  /* Supplement L Elbow Jt with Autol Sub, Perc Endo Approach */
"0RUM4JZ"  =  "1"  /* Supplement L Elbow Jt with Synth Sub, Perc Endo Approach */
"0RUM4KZ"  =  "1"  /* Supplement L Elbow Jt with Nonaut Sub, Perc Endo Approach */
"0RUN07Z"  =  "1"  /* Supplement Right Wrist Joint with Autol Sub, Open Approach */
"0RUN0JZ"  =  "1"  /* Supplement Right Wrist Joint with Synth Sub, Open Approach */
"0RUN0KZ"  =  "1"  /* Supplement Right Wrist Joint with Nonaut Sub, Open Approach */
"0RUN37Z"  =  "1"  /* Supplement Right Wrist Joint with Autol Sub, Perc Approach */
"0RUN3JZ"  =  "1"  /* Supplement Right Wrist Joint with Synth Sub, Perc Approach */
"0RUN3KZ"  =  "1"  /* Supplement Right Wrist Joint with Nonaut Sub, Perc Approach */
"0RUN47Z"  =  "1"  /* Supplement R Wrist Jt with Autol Sub, Perc Endo Approach */
"0RUN4JZ"  =  "1"  /* Supplement R Wrist Jt with Synth Sub, Perc Endo Approach */
"0RUN4KZ"  =  "1"  /* Supplement R Wrist Jt with Nonaut Sub, Perc Endo Approach */
"0RUP07Z"  =  "1"  /* Supplement Left Wrist Joint with Autol Sub, Open Approach */
"0RUP0JZ"  =  "1"  /* Supplement Left Wrist Joint with Synth Sub, Open Approach */
"0RUP0KZ"  =  "1"  /* Supplement Left Wrist Joint with Nonaut Sub, Open Approach */
"0RUP37Z"  =  "1"  /* Supplement Left Wrist Joint with Autol Sub, Perc Approach */
"0RUP3JZ"  =  "1"  /* Supplement Left Wrist Joint with Synth Sub, Perc Approach */
"0RUP3KZ"  =  "1"  /* Supplement Left Wrist Joint with Nonaut Sub, Perc Approach */
"0RUP47Z"  =  "1"  /* Supplement L Wrist Jt with Autol Sub, Perc Endo Approach */
"0RUP4JZ"  =  "1"  /* Supplement L Wrist Jt with Synth Sub, Perc Endo Approach */
"0RUP4KZ"  =  "1"  /* Supplement L Wrist Jt with Nonaut Sub, Perc Endo Approach */
"0RUQ07Z"  =  "1"  /* Supplement Right Carpal Joint with Autol Sub, Open Approach */
"0RUQ0JZ"  =  "1"  /* Supplement Right Carpal Joint with Synth Sub, Open Approach */
"0RUQ0KZ"  =  "1"  /* Supplement Right Carpal Joint with Nonaut Sub, Open Approach */
"0RUQ37Z"  =  "1"  /* Supplement Right Carpal Joint with Autol Sub, Perc Approach */
"0RUQ3JZ"  =  "1"  /* Supplement Right Carpal Joint with Synth Sub, Perc Approach */
"0RUQ3KZ"  =  "1"  /* Supplement Right Carpal Joint with Nonaut Sub, Perc Approach */
"0RUQ47Z"  =  "1"  /* Supplement R Carpal Jt with Autol Sub, Perc Endo Approach */
"0RUQ4JZ"  =  "1"  /* Supplement R Carpal Jt with Synth Sub, Perc Endo Approach */
"0RUQ4KZ"  =  "1"  /* Supplement R Carpal Jt with Nonaut Sub, Perc Endo Approach */
"0RUR07Z"  =  "1"  /* Supplement Left Carpal Joint with Autol Sub, Open Approach */
"0RUR0JZ"  =  "1"  /* Supplement Left Carpal Joint with Synth Sub, Open Approach */
"0RUR0KZ"  =  "1"  /* Supplement Left Carpal Joint with Nonaut Sub, Open Approach */
"0RUR37Z"  =  "1"  /* Supplement Left Carpal Joint with Autol Sub, Perc Approach */
"0RUR3JZ"  =  "1"  /* Supplement Left Carpal Joint with Synth Sub, Perc Approach */
"0RUR3KZ"  =  "1"  /* Supplement Left Carpal Joint with Nonaut Sub, Perc Approach */
"0RUR47Z"  =  "1"  /* Supplement L Carpal Jt with Autol Sub, Perc Endo Approach */
"0RUR4JZ"  =  "1"  /* Supplement L Carpal Jt with Synth Sub, Perc Endo Approach */
"0RUR4KZ"  =  "1"  /* Supplement L Carpal Jt with Nonaut Sub, Perc Endo Approach */
"0RUS07Z"  =  "1"  /* Supplement R Metacarpocarp Jt with Autol Sub, Open Approach */
"0RUS0JZ"  =  "1"  /* Supplement R Metacarpocarp Jt with Synth Sub, Open Approach */
"0RUS0KZ"  =  "1"  /* Supplement R Metacarpocarp Jt with Nonaut Sub, Open Approach */
"0RUS37Z"  =  "1"  /* Supplement R Metacarpocarp Jt with Autol Sub, Perc Approach */
"0RUS3JZ"  =  "1"  /* Supplement R Metacarpocarp Jt with Synth Sub, Perc Approach */
"0RUS3KZ"  =  "1"  /* Supplement R Metacarpocarp Jt with Nonaut Sub, Perc Approach */
"0RUS47Z"  =  "1"  /* Supplement R Metacarpocarp Jt w Autol Sub, Perc Endo */
"0RUS4JZ"  =  "1"  /* Supplement R Metacarpocarp Jt w Synth Sub, Perc Endo */
"0RUS4KZ"  =  "1"  /* Supplement R Metacarpocarp Jt w Nonaut Sub, Perc Endo */
"0RUT07Z"  =  "1"  /* Supplement L Metacarpocarp Jt with Autol Sub, Open Approach */
"0RUT0JZ"  =  "1"  /* Supplement L Metacarpocarp Jt with Synth Sub, Open Approach */
"0RUT0KZ"  =  "1"  /* Supplement L Metacarpocarp Jt with Nonaut Sub, Open Approach */
"0RUT37Z"  =  "1"  /* Supplement L Metacarpocarp Jt with Autol Sub, Perc Approach */
"0RUT3JZ"  =  "1"  /* Supplement L Metacarpocarp Jt with Synth Sub, Perc Approach */
"0RUT3KZ"  =  "1"  /* Supplement L Metacarpocarp Jt with Nonaut Sub, Perc Approach */
"0RUT47Z"  =  "1"  /* Supplement L Metacarpocarp Jt w Autol Sub, Perc Endo */
"0RUT4JZ"  =  "1"  /* Supplement L Metacarpocarp Jt w Synth Sub, Perc Endo */
"0RUT4KZ"  =  "1"  /* Supplement L Metacarpocarp Jt w Nonaut Sub, Perc Endo */
"0RUU07Z"  =  "1"  /* Supplement R Metacarpophal Jt with Autol Sub, Open Approach */
"0RUU0JZ"  =  "1"  /* Supplement R Metacarpophal Jt with Synth Sub, Open Approach */
"0RUU0KZ"  =  "1"  /* Supplement R Metacarpophal Jt with Nonaut Sub, Open Approach */
"0RUU37Z"  =  "1"  /* Supplement R Metacarpophal Jt with Autol Sub, Perc Approach */
"0RUU3JZ"  =  "1"  /* Supplement R Metacarpophal Jt with Synth Sub, Perc Approach */
"0RUU3KZ"  =  "1"  /* Supplement R Metacarpophal Jt with Nonaut Sub, Perc Approach */
"0RUU47Z"  =  "1"  /* Supplement R Metacarpophal Jt w Autol Sub, Perc Endo */
"0RUU4JZ"  =  "1"  /* Supplement R Metacarpophal Jt w Synth Sub, Perc Endo */
"0RUU4KZ"  =  "1"  /* Supplement R Metacarpophal Jt w Nonaut Sub, Perc Endo */
"0RUV07Z"  =  "1"  /* Supplement L Metacarpophal Jt with Autol Sub, Open Approach */
"0RUV0JZ"  =  "1"  /* Supplement L Metacarpophal Jt with Synth Sub, Open Approach */
"0RUV0KZ"  =  "1"  /* Supplement L Metacarpophal Jt with Nonaut Sub, Open Approach */
"0RUV37Z"  =  "1"  /* Supplement L Metacarpophal Jt with Autol Sub, Perc Approach */
"0RUV3JZ"  =  "1"  /* Supplement L Metacarpophal Jt with Synth Sub, Perc Approach */
"0RUV3KZ"  =  "1"  /* Supplement L Metacarpophal Jt with Nonaut Sub, Perc Approach */
"0RUV47Z"  =  "1"  /* Supplement L Metacarpophal Jt w Autol Sub, Perc Endo */
"0RUV4JZ"  =  "1"  /* Supplement L Metacarpophal Jt w Synth Sub, Perc Endo */
"0RUV4KZ"  =  "1"  /* Supplement L Metacarpophal Jt w Nonaut Sub, Perc Endo */
"0RUW07Z"  =  "1"  /* Supplement R Finger Phalanx Jt with Autol Sub, Open Approach */
"0RUW0JZ"  =  "1"  /* Supplement R Finger Phalanx Jt with Synth Sub, Open Approach */
"0RUW0KZ"  =  "1"  /* Supplement R Finger Phalanx Jt w Nonaut Sub, Open */
"0RUW37Z"  =  "1"  /* Supplement R Finger Phalanx Jt with Autol Sub, Perc Approach */
"0RUW3JZ"  =  "1"  /* Supplement R Finger Phalanx Jt with Synth Sub, Perc Approach */
"0RUW3KZ"  =  "1"  /* Supplement R Finger Phalanx Jt w Nonaut Sub, Perc */
"0RUW47Z"  =  "1"  /* Supplement R Finger Phalanx Jt w Autol Sub, Perc Endo */
"0RUW4JZ"  =  "1"  /* Supplement R Finger Phalanx Jt w Synth Sub, Perc Endo */
"0RUW4KZ"  =  "1"  /* Supplement R Finger Phalanx Jt w Nonaut Sub, Perc Endo */
"0RUX07Z"  =  "1"  /* Supplement L Finger Phalanx Jt with Autol Sub, Open Approach */
"0RUX0JZ"  =  "1"  /* Supplement L Finger Phalanx Jt with Synth Sub, Open Approach */
"0RUX0KZ"  =  "1"  /* Supplement L Finger Phalanx Jt w Nonaut Sub, Open */
"0RUX37Z"  =  "1"  /* Supplement L Finger Phalanx Jt with Autol Sub, Perc Approach */
"0RUX3JZ"  =  "1"  /* Supplement L Finger Phalanx Jt with Synth Sub, Perc Approach */
"0RUX3KZ"  =  "1"  /* Supplement L Finger Phalanx Jt w Nonaut Sub, Perc */
"0RUX47Z"  =  "1"  /* Supplement L Finger Phalanx Jt w Autol Sub, Perc Endo */
"0RUX4JZ"  =  "1"  /* Supplement L Finger Phalanx Jt w Synth Sub, Perc Endo */
"0RUX4KZ"  =  "1"  /* Supplement L Finger Phalanx Jt w Nonaut Sub, Perc Endo */
"0RW000Z"  =  "1"  /* Revision of Drainage Device in Occip Jt, Open Approach */
"0RW003Z"  =  "1"  /* Revision of Infusion Device in Occip Jt, Open Approach */
"0RW004Z"  =  "1"  /* Revision of Int Fix in Occip Jt, Open Approach */
"0RW007Z"  =  "1"  /* Revision of Autol Sub in Occip Jt, Open Approach */
"0RW008Z"  =  "1"  /* Revision of Spacer in Occip Jt, Open Approach */
"0RW00AZ"  =  "1"  /* Revision of Intbd Fus Dev in Occip Jt, Open Approach */
"0RW00JZ"  =  "1"  /* Revision of Synthetic Substitute in Occip Jt, Open Approach */
"0RW00KZ"  =  "1"  /* Revision of Nonaut Sub in Occip Jt, Open Approach */
"0RW030Z"  =  "1"  /* Revision of Drainage Device in Occip Jt, Perc Approach */
"0RW033Z"  =  "1"  /* Revision of Infusion Device in Occip Jt, Perc Approach */
"0RW034Z"  =  "1"  /* Revision of Int Fix in Occip Jt, Perc Approach */
"0RW037Z"  =  "1"  /* Revision of Autol Sub in Occip Jt, Perc Approach */
"0RW038Z"  =  "1"  /* Revision of Spacer in Occip Jt, Perc Approach */
"0RW03AZ"  =  "1"  /* Revision of Intbd Fus Dev in Occip Jt, Perc Approach */
"0RW03JZ"  =  "1"  /* Revision of Synthetic Substitute in Occip Jt, Perc Approach */
"0RW03KZ"  =  "1"  /* Revision of Nonaut Sub in Occip Jt, Perc Approach */
"0RW040Z"  =  "1"  /* Revision of Drainage Device in Occip Jt, Perc Endo Approach */
"0RW043Z"  =  "1"  /* Revision of Infusion Device in Occip Jt, Perc Endo Approach */
"0RW044Z"  =  "1"  /* Revision of Int Fix in Occip Jt, Perc Endo Approach */
"0RW047Z"  =  "1"  /* Revision of Autol Sub in Occip Jt, Perc Endo Approach */
"0RW048Z"  =  "1"  /* Revision of Spacer in Occip Jt, Perc Endo Approach */
"0RW04AZ"  =  "1"  /* Revision of Intbd Fus Dev in Occip Jt, Perc Endo Approach */
"0RW04JZ"  =  "1"  /* Revision of Synth Sub in Occip Jt, Perc Endo Approach */
"0RW04KZ"  =  "1"  /* Revision of Nonaut Sub in Occip Jt, Perc Endo Approach */
"0RW100Z"  =  "1"  /* Revision of Drainage Device in Cerv Jt, Open Approach */
"0RW103Z"  =  "1"  /* Revision of Infusion Device in Cerv Jt, Open Approach */
"0RW104Z"  =  "1"  /* Revision of Int Fix in Cerv Jt, Open Approach */
"0RW107Z"  =  "1"  /* Revision of Autol Sub in Cerv Jt, Open Approach */
"0RW108Z"  =  "1"  /* Revision of Spacer in Cerv Jt, Open Approach */
"0RW10AZ"  =  "1"  /* Revision of Intbd Fus Dev in Cerv Jt, Open Approach */
"0RW10JZ"  =  "1"  /* Revision of Synthetic Substitute in Cerv Jt, Open Approach */
"0RW10KZ"  =  "1"  /* Revision of Nonaut Sub in Cerv Jt, Open Approach */
"0RW130Z"  =  "1"  /* Revision of Drainage Device in Cerv Jt, Perc Approach */
"0RW133Z"  =  "1"  /* Revision of Infusion Device in Cerv Jt, Perc Approach */
"0RW134Z"  =  "1"  /* Revision of Int Fix in Cerv Jt, Perc Approach */
"0RW137Z"  =  "1"  /* Revision of Autol Sub in Cerv Jt, Perc Approach */
"0RW138Z"  =  "1"  /* Revision of Spacer in Cerv Jt, Perc Approach */
"0RW13AZ"  =  "1"  /* Revision of Intbd Fus Dev in Cerv Jt, Perc Approach */
"0RW13JZ"  =  "1"  /* Revision of Synthetic Substitute in Cerv Jt, Perc Approach */
"0RW13KZ"  =  "1"  /* Revision of Nonaut Sub in Cerv Jt, Perc Approach */
"0RW140Z"  =  "1"  /* Revision of Drainage Device in Cerv Jt, Perc Endo Approach */
"0RW143Z"  =  "1"  /* Revision of Infusion Device in Cerv Jt, Perc Endo Approach */
"0RW144Z"  =  "1"  /* Revision of Int Fix in Cerv Jt, Perc Endo Approach */
"0RW147Z"  =  "1"  /* Revision of Autol Sub in Cerv Jt, Perc Endo Approach */
"0RW148Z"  =  "1"  /* Revision of Spacer in Cerv Jt, Perc Endo Approach */
"0RW14AZ"  =  "1"  /* Revision of Intbd Fus Dev in Cerv Jt, Perc Endo Approach */
"0RW14JZ"  =  "1"  /* Revision of Synth Sub in Cerv Jt, Perc Endo Approach */
"0RW14KZ"  =  "1"  /* Revision of Nonaut Sub in Cerv Jt, Perc Endo Approach */
"0RW300Z"  =  "1"  /* Revision of Drainage Device in Cerv Disc, Open Approach */
"0RW303Z"  =  "1"  /* Revision of Infusion Device in Cerv Disc, Open Approach */
"0RW307Z"  =  "1"  /* Revision of Autol Sub in Cerv Disc, Open Approach */
"0RW30JZ"  =  "1"  /* Revision of Synthetic Substitute in Cerv Disc, Open Approach */
"0RW30KZ"  =  "1"  /* Revision of Nonaut Sub in Cerv Disc, Open Approach */
"0RW330Z"  =  "1"  /* Revision of Drainage Device in Cerv Disc, Perc Approach */
"0RW333Z"  =  "1"  /* Revision of Infusion Device in Cerv Disc, Perc Approach */
"0RW337Z"  =  "1"  /* Revision of Autol Sub in Cerv Disc, Perc Approach */
"0RW33JZ"  =  "1"  /* Revision of Synthetic Substitute in Cerv Disc, Perc Approach */
"0RW33KZ"  =  "1"  /* Revision of Nonaut Sub in Cerv Disc, Perc Approach */
"0RW340Z"  =  "1"  /* Revision of Drainage Device in Cerv Disc, Perc Endo Approach */
"0RW343Z"  =  "1"  /* Revision of Infusion Device in Cerv Disc, Perc Endo Approach */
"0RW347Z"  =  "1"  /* Revision of Autol Sub in Cerv Disc, Perc Endo Approach */
"0RW34JZ"  =  "1"  /* Revision of Synth Sub in Cerv Disc, Perc Endo Approach */
"0RW34KZ"  =  "1"  /* Revision of Nonaut Sub in Cerv Disc, Perc Endo Approach */
"0RW400Z"  =  "1"  /* Revision of Drainage Device in C-thor Jt, Open Approach */
"0RW403Z"  =  "1"  /* Revision of Infusion Device in C-thor Jt, Open Approach */
"0RW404Z"  =  "1"  /* Revision of Int Fix in C-thor Jt, Open Approach */
"0RW407Z"  =  "1"  /* Revision of Autol Sub in C-thor Jt, Open Approach */
"0RW408Z"  =  "1"  /* Revision of Spacer in C-thor Jt, Open Approach */
"0RW40AZ"  =  "1"  /* Revision of Intbd Fus Dev in C-thor Jt, Open Approach */
"0RW40JZ"  =  "1"  /* Revision of Synthetic Substitute in C-thor Jt, Open Approach */
"0RW40KZ"  =  "1"  /* Revision of Nonaut Sub in C-thor Jt, Open Approach */
"0RW430Z"  =  "1"  /* Revision of Drainage Device in C-thor Jt, Perc Approach */
"0RW433Z"  =  "1"  /* Revision of Infusion Device in C-thor Jt, Perc Approach */
"0RW434Z"  =  "1"  /* Revision of Int Fix in C-thor Jt, Perc Approach */
"0RW437Z"  =  "1"  /* Revision of Autol Sub in C-thor Jt, Perc Approach */
"0RW438Z"  =  "1"  /* Revision of Spacer in C-thor Jt, Perc Approach */
"0RW43AZ"  =  "1"  /* Revision of Intbd Fus Dev in C-thor Jt, Perc Approach */
"0RW43JZ"  =  "1"  /* Revision of Synthetic Substitute in C-thor Jt, Perc Approach */
"0RW43KZ"  =  "1"  /* Revision of Nonaut Sub in C-thor Jt, Perc Approach */
"0RW440Z"  =  "1"  /* Revision of Drainage Device in C-thor Jt, Perc Endo Approach */
"0RW443Z"  =  "1"  /* Revision of Infusion Device in C-thor Jt, Perc Endo Approach */
"0RW444Z"  =  "1"  /* Revision of Int Fix in C-thor Jt, Perc Endo Approach */
"0RW447Z"  =  "1"  /* Revision of Autol Sub in C-thor Jt, Perc Endo Approach */
"0RW448Z"  =  "1"  /* Revision of Spacer in C-thor Jt, Perc Endo Approach */
"0RW44AZ"  =  "1"  /* Revision of Intbd Fus Dev in C-thor Jt, Perc Endo Approach */
"0RW44JZ"  =  "1"  /* Revision of Synth Sub in C-thor Jt, Perc Endo Approach */
"0RW44KZ"  =  "1"  /* Revision of Nonaut Sub in C-thor Jt, Perc Endo Approach */
"0RW500Z"  =  "1"  /* Revision of Drainage Device in C-thor Disc, Open Approach */
"0RW503Z"  =  "1"  /* Revision of Infusion Device in C-thor Disc, Open Approach */
"0RW507Z"  =  "1"  /* Revision of Autol Sub in C-thor Disc, Open Approach */
"0RW50JZ"  =  "1"  /* Revision of Synth Sub in C-thor Disc, Open Approach */
"0RW50KZ"  =  "1"  /* Revision of Nonaut Sub in C-thor Disc, Open Approach */
"0RW530Z"  =  "1"  /* Revision of Drainage Device in C-thor Disc, Perc Approach */
"0RW533Z"  =  "1"  /* Revision of Infusion Device in C-thor Disc, Perc Approach */
"0RW537Z"  =  "1"  /* Revision of Autol Sub in C-thor Disc, Perc Approach */
"0RW53JZ"  =  "1"  /* Revision of Synth Sub in C-thor Disc, Perc Approach */
"0RW53KZ"  =  "1"  /* Revision of Nonaut Sub in C-thor Disc, Perc Approach */
"0RW540Z"  =  "1"  /* Revision of Drain Dev in C-thor Disc, Perc Endo Approach */
"0RW543Z"  =  "1"  /* Revision of Infusion Dev in C-thor Disc, Perc Endo Approach */
"0RW547Z"  =  "1"  /* Revision of Autol Sub in C-thor Disc, Perc Endo Approach */
"0RW54JZ"  =  "1"  /* Revision of Synth Sub in C-thor Disc, Perc Endo Approach */
"0RW54KZ"  =  "1"  /* Revision of Nonaut Sub in C-thor Disc, Perc Endo Approach */
"0RW600Z"  =  "1"  /* Revision of Drainage Device in Thor Jt, Open Approach */
"0RW603Z"  =  "1"  /* Revision of Infusion Device in Thor Jt, Open Approach */
"0RW604Z"  =  "1"  /* Revision of Int Fix in Thor Jt, Open Approach */
"0RW607Z"  =  "1"  /* Revision of Autol Sub in Thor Jt, Open Approach */
"0RW608Z"  =  "1"  /* Revision of Spacer in Thor Jt, Open Approach */
"0RW60AZ"  =  "1"  /* Revision of Intbd Fus Dev in Thor Jt, Open Approach */
"0RW60JZ"  =  "1"  /* Revision of Synthetic Substitute in Thor Jt, Open Approach */
"0RW60KZ"  =  "1"  /* Revision of Nonaut Sub in Thor Jt, Open Approach */
"0RW630Z"  =  "1"  /* Revision of Drainage Device in Thor Jt, Perc Approach */
"0RW633Z"  =  "1"  /* Revision of Infusion Device in Thor Jt, Perc Approach */
"0RW634Z"  =  "1"  /* Revision of Int Fix in Thor Jt, Perc Approach */
"0RW637Z"  =  "1"  /* Revision of Autol Sub in Thor Jt, Perc Approach */
"0RW638Z"  =  "1"  /* Revision of Spacer in Thor Jt, Perc Approach */
"0RW63AZ"  =  "1"  /* Revision of Intbd Fus Dev in Thor Jt, Perc Approach */
"0RW63JZ"  =  "1"  /* Revision of Synthetic Substitute in Thor Jt, Perc Approach */
"0RW63KZ"  =  "1"  /* Revision of Nonaut Sub in Thor Jt, Perc Approach */
"0RW640Z"  =  "1"  /* Revision of Drainage Device in Thor Jt, Perc Endo Approach */
"0RW643Z"  =  "1"  /* Revision of Infusion Device in Thor Jt, Perc Endo Approach */
"0RW644Z"  =  "1"  /* Revision of Int Fix in Thor Jt, Perc Endo Approach */
"0RW647Z"  =  "1"  /* Revision of Autol Sub in Thor Jt, Perc Endo Approach */
"0RW648Z"  =  "1"  /* Revision of Spacer in Thor Jt, Perc Endo Approach */
"0RW64AZ"  =  "1"  /* Revision of Intbd Fus Dev in Thor Jt, Perc Endo Approach */
"0RW64JZ"  =  "1"  /* Revision of Synth Sub in Thor Jt, Perc Endo Approach */
"0RW64KZ"  =  "1"  /* Revision of Nonaut Sub in Thor Jt, Perc Endo Approach */
"0RW900Z"  =  "1"  /* Revision of Drainage Device in Thor Disc, Open Approach */
"0RW903Z"  =  "1"  /* Revision of Infusion Device in Thor Disc, Open Approach */
"0RW907Z"  =  "1"  /* Revision of Autol Sub in Thor Disc, Open Approach */
"0RW90JZ"  =  "1"  /* Revision of Synthetic Substitute in Thor Disc, Open Approach */
"0RW90KZ"  =  "1"  /* Revision of Nonaut Sub in Thor Disc, Open Approach */
"0RW930Z"  =  "1"  /* Revision of Drainage Device in Thor Disc, Perc Approach */
"0RW933Z"  =  "1"  /* Revision of Infusion Device in Thor Disc, Perc Approach */
"0RW937Z"  =  "1"  /* Revision of Autol Sub in Thor Disc, Perc Approach */
"0RW93JZ"  =  "1"  /* Revision of Synthetic Substitute in Thor Disc, Perc Approach */
"0RW93KZ"  =  "1"  /* Revision of Nonaut Sub in Thor Disc, Perc Approach */
"0RW940Z"  =  "1"  /* Revision of Drainage Device in Thor Disc, Perc Endo Approach */
"0RW943Z"  =  "1"  /* Revision of Infusion Device in Thor Disc, Perc Endo Approach */
"0RW947Z"  =  "1"  /* Revision of Autol Sub in Thor Disc, Perc Endo Approach */
"0RW94JZ"  =  "1"  /* Revision of Synth Sub in Thor Disc, Perc Endo Approach */
"0RW94KZ"  =  "1"  /* Revision of Nonaut Sub in Thor Disc, Perc Endo Approach */
"0RWA00Z"  =  "1"  /* Revision of Drainage Device in T-lum Jt, Open Approach */
"0RWA03Z"  =  "1"  /* Revision of Infusion Device in T-lum Jt, Open Approach */
"0RWA04Z"  =  "1"  /* Revision of Int Fix in T-lum Jt, Open Approach */
"0RWA07Z"  =  "1"  /* Revision of Autol Sub in T-lum Jt, Open Approach */
"0RWA08Z"  =  "1"  /* Revision of Spacer in T-lum Jt, Open Approach */
"0RWA0AZ"  =  "1"  /* Revision of Intbd Fus Dev in T-lum Jt, Open Approach */
"0RWA0JZ"  =  "1"  /* Revision of Synthetic Substitute in T-lum Jt, Open Approach */
"0RWA0KZ"  =  "1"  /* Revision of Nonaut Sub in T-lum Jt, Open Approach */
"0RWA30Z"  =  "1"  /* Revision of Drainage Device in T-lum Jt, Perc Approach */
"0RWA33Z"  =  "1"  /* Revision of Infusion Device in T-lum Jt, Perc Approach */
"0RWA34Z"  =  "1"  /* Revision of Int Fix in T-lum Jt, Perc Approach */
"0RWA37Z"  =  "1"  /* Revision of Autol Sub in T-lum Jt, Perc Approach */
"0RWA38Z"  =  "1"  /* Revision of Spacer in T-lum Jt, Perc Approach */
"0RWA3AZ"  =  "1"  /* Revision of Intbd Fus Dev in T-lum Jt, Perc Approach */
"0RWA3JZ"  =  "1"  /* Revision of Synthetic Substitute in T-lum Jt, Perc Approach */
"0RWA3KZ"  =  "1"  /* Revision of Nonaut Sub in T-lum Jt, Perc Approach */
"0RWA40Z"  =  "1"  /* Revision of Drainage Device in T-lum Jt, Perc Endo Approach */
"0RWA43Z"  =  "1"  /* Revision of Infusion Device in T-lum Jt, Perc Endo Approach */
"0RWA44Z"  =  "1"  /* Revision of Int Fix in T-lum Jt, Perc Endo Approach */
"0RWA47Z"  =  "1"  /* Revision of Autol Sub in T-lum Jt, Perc Endo Approach */
"0RWA48Z"  =  "1"  /* Revision of Spacer in T-lum Jt, Perc Endo Approach */
"0RWA4AZ"  =  "1"  /* Revision of Intbd Fus Dev in T-lum Jt, Perc Endo Approach */
"0RWA4JZ"  =  "1"  /* Revision of Synth Sub in T-lum Jt, Perc Endo Approach */
"0RWA4KZ"  =  "1"  /* Revision of Nonaut Sub in T-lum Jt, Perc Endo Approach */
"0RWB00Z"  =  "1"  /* Revision of Drainage Device in T-lum Disc, Open Approach */
"0RWB03Z"  =  "1"  /* Revision of Infusion Device in T-lum Disc, Open Approach */
"0RWB07Z"  =  "1"  /* Revision of Autol Sub in T-lum Disc, Open Approach */
"0RWB0JZ"  =  "1"  /* Revision of Synth Sub in T-lum Disc, Open Approach */
"0RWB0KZ"  =  "1"  /* Revision of Nonaut Sub in T-lum Disc, Open Approach */
"0RWB30Z"  =  "1"  /* Revision of Drainage Device in T-lum Disc, Perc Approach */
"0RWB33Z"  =  "1"  /* Revision of Infusion Device in T-lum Disc, Perc Approach */
"0RWB37Z"  =  "1"  /* Revision of Autol Sub in T-lum Disc, Perc Approach */
"0RWB3JZ"  =  "1"  /* Revision of Synth Sub in T-lum Disc, Perc Approach */
"0RWB3KZ"  =  "1"  /* Revision of Nonaut Sub in T-lum Disc, Perc Approach */
"0RWB40Z"  =  "1"  /* Revision of Drain Dev in T-lum Disc, Perc Endo Approach */
"0RWB43Z"  =  "1"  /* Revision of Infusion Dev in T-lum Disc, Perc Endo Approach */
"0RWB47Z"  =  "1"  /* Revision of Autol Sub in T-lum Disc, Perc Endo Approach */
"0RWB4JZ"  =  "1"  /* Revision of Synth Sub in T-lum Disc, Perc Endo Approach */
"0RWB4KZ"  =  "1"  /* Revision of Nonaut Sub in T-lum Disc, Perc Endo Approach */
"0RWC00Z"  =  "1"  /* Revision of Drain Dev in R Temporomandib Jt, Open Approach */
"0RWC03Z"  =  "1"  /* Revise of Infusion Dev in R Temporomandib Jt, Open Approach */
"0RWC04Z"  =  "1"  /* Revision of Int Fix in R Temporomandib Jt, Open Approach */
"0RWC07Z"  =  "1"  /* Revision of Autol Sub in R Temporomandib Jt, Open Approach */
"0RWC08Z"  =  "1"  /* Revision of Spacer in R Temporomandib Jt, Open Approach */
"0RWC0JZ"  =  "1"  /* Revision of Synth Sub in R Temporomandib Jt, Open Approach */
"0RWC0KZ"  =  "1"  /* Revision of Nonaut Sub in R Temporomandib Jt, Open Approach */
"0RWC30Z"  =  "1"  /* Revision of Drain Dev in R Temporomandib Jt, Perc Approach */
"0RWC33Z"  =  "1"  /* Revise of Infusion Dev in R Temporomandib Jt, Perc Approach */
"0RWC34Z"  =  "1"  /* Revision of Int Fix in R Temporomandib Jt, Perc Approach */
"0RWC37Z"  =  "1"  /* Revision of Autol Sub in R Temporomandib Jt, Perc Approach */
"0RWC38Z"  =  "1"  /* Revision of Spacer in R Temporomandib Jt, Perc Approach */
"0RWC3JZ"  =  "1"  /* Revision of Synth Sub in R Temporomandib Jt, Perc Approach */
"0RWC3KZ"  =  "1"  /* Revision of Nonaut Sub in R Temporomandib Jt, Perc Approach */
"0RWC40Z"  =  "1"  /* Revise Drain Dev in R Temporomandib Jt, Perc Endo */
"0RWC43Z"  =  "1"  /* Revise Infusion Dev in R Temporomandib Jt, Perc Endo */
"0RWC44Z"  =  "1"  /* Revise of Int Fix in R Temporomandib Jt, Perc Endo Approach */
"0RWC47Z"  =  "1"  /* Revise Autol Sub in R Temporomandib Jt, Perc Endo */
"0RWC48Z"  =  "1"  /* Revision of Spacer in R Temporomandib Jt, Perc Endo Approach */
"0RWC4JZ"  =  "1"  /* Revise Synth Sub in R Temporomandib Jt, Perc Endo */
"0RWC4KZ"  =  "1"  /* Revise Nonaut Sub in R Temporomandib Jt, Perc Endo */
"0RWD00Z"  =  "1"  /* Revision of Drain Dev in L Temporomandib Jt, Open Approach */
"0RWD03Z"  =  "1"  /* Revise of Infusion Dev in L Temporomandib Jt, Open Approach */
"0RWD04Z"  =  "1"  /* Revision of Int Fix in L Temporomandib Jt, Open Approach */
"0RWD07Z"  =  "1"  /* Revision of Autol Sub in L Temporomandib Jt, Open Approach */
"0RWD08Z"  =  "1"  /* Revision of Spacer in L Temporomandib Jt, Open Approach */
"0RWD0JZ"  =  "1"  /* Revision of Synth Sub in L Temporomandib Jt, Open Approach */
"0RWD0KZ"  =  "1"  /* Revision of Nonaut Sub in L Temporomandib Jt, Open Approach */
"0RWD30Z"  =  "1"  /* Revision of Drain Dev in L Temporomandib Jt, Perc Approach */
"0RWD33Z"  =  "1"  /* Revise of Infusion Dev in L Temporomandib Jt, Perc Approach */
"0RWD34Z"  =  "1"  /* Revision of Int Fix in L Temporomandib Jt, Perc Approach */
"0RWD37Z"  =  "1"  /* Revision of Autol Sub in L Temporomandib Jt, Perc Approach */
"0RWD38Z"  =  "1"  /* Revision of Spacer in L Temporomandib Jt, Perc Approach */
"0RWD3JZ"  =  "1"  /* Revision of Synth Sub in L Temporomandib Jt, Perc Approach */
"0RWD3KZ"  =  "1"  /* Revision of Nonaut Sub in L Temporomandib Jt, Perc Approach */
"0RWD40Z"  =  "1"  /* Revise Drain Dev in L Temporomandib Jt, Perc Endo */
"0RWD43Z"  =  "1"  /* Revise Infusion Dev in L Temporomandib Jt, Perc Endo */
"0RWD44Z"  =  "1"  /* Revise of Int Fix in L Temporomandib Jt, Perc Endo Approach */
"0RWD47Z"  =  "1"  /* Revise Autol Sub in L Temporomandib Jt, Perc Endo */
"0RWD48Z"  =  "1"  /* Revision of Spacer in L Temporomandib Jt, Perc Endo Approach */
"0RWD4JZ"  =  "1"  /* Revise Synth Sub in L Temporomandib Jt, Perc Endo */
"0RWD4KZ"  =  "1"  /* Revise Nonaut Sub in L Temporomandib Jt, Perc Endo */
"0RWE00Z"  =  "1"  /* Revision of Drain Dev in R Sternoclav Jt, Open Approach */
"0RWE03Z"  =  "1"  /* Revision of Infusion Dev in R Sternoclav Jt, Open Approach */
"0RWE04Z"  =  "1"  /* Revision of Int Fix in R Sternoclav Jt, Open Approach */
"0RWE07Z"  =  "1"  /* Revision of Autol Sub in R Sternoclav Jt, Open Approach */
"0RWE08Z"  =  "1"  /* Revision of Spacer in R Sternoclav Jt, Open Approach */
"0RWE0JZ"  =  "1"  /* Revision of Synth Sub in R Sternoclav Jt, Open Approach */
"0RWE0KZ"  =  "1"  /* Revision of Nonaut Sub in R Sternoclav Jt, Open Approach */
"0RWE30Z"  =  "1"  /* Revision of Drain Dev in R Sternoclav Jt, Perc Approach */
"0RWE33Z"  =  "1"  /* Revision of Infusion Dev in R Sternoclav Jt, Perc Approach */
"0RWE34Z"  =  "1"  /* Revision of Int Fix in R Sternoclav Jt, Perc Approach */
"0RWE37Z"  =  "1"  /* Revision of Autol Sub in R Sternoclav Jt, Perc Approach */
"0RWE38Z"  =  "1"  /* Revision of Spacer in R Sternoclav Jt, Perc Approach */
"0RWE3JZ"  =  "1"  /* Revision of Synth Sub in R Sternoclav Jt, Perc Approach */
"0RWE3KZ"  =  "1"  /* Revision of Nonaut Sub in R Sternoclav Jt, Perc Approach */
"0RWE40Z"  =  "1"  /* Revision of Drain Dev in R Sternoclav Jt, Perc Endo Approach */
"0RWE43Z"  =  "1"  /* Revise Infusion Dev in R Sternoclav Jt, Perc Endo */
"0RWE44Z"  =  "1"  /* Revision of Int Fix in R Sternoclav Jt, Perc Endo Approach */
"0RWE47Z"  =  "1"  /* Revision of Autol Sub in R Sternoclav Jt, Perc Endo Approach */
"0RWE48Z"  =  "1"  /* Revision of Spacer in R Sternoclav Jt, Perc Endo Approach */
"0RWE4JZ"  =  "1"  /* Revision of Synth Sub in R Sternoclav Jt, Perc Endo Approach */
"0RWE4KZ"  =  "1"  /* Revise of Nonaut Sub in R Sternoclav Jt, Perc Endo Approach */
"0RWF00Z"  =  "1"  /* Revision of Drain Dev in L Sternoclav Jt, Open Approach */
"0RWF03Z"  =  "1"  /* Revision of Infusion Dev in L Sternoclav Jt, Open Approach */
"0RWF04Z"  =  "1"  /* Revision of Int Fix in L Sternoclav Jt, Open Approach */
"0RWF07Z"  =  "1"  /* Revision of Autol Sub in L Sternoclav Jt, Open Approach */
"0RWF08Z"  =  "1"  /* Revision of Spacer in L Sternoclav Jt, Open Approach */
"0RWF0JZ"  =  "1"  /* Revision of Synth Sub in L Sternoclav Jt, Open Approach */
"0RWF0KZ"  =  "1"  /* Revision of Nonaut Sub in L Sternoclav Jt, Open Approach */
"0RWF30Z"  =  "1"  /* Revision of Drain Dev in L Sternoclav Jt, Perc Approach */
"0RWF33Z"  =  "1"  /* Revision of Infusion Dev in L Sternoclav Jt, Perc Approach */
"0RWF34Z"  =  "1"  /* Revision of Int Fix in L Sternoclav Jt, Perc Approach */
"0RWF37Z"  =  "1"  /* Revision of Autol Sub in L Sternoclav Jt, Perc Approach */
"0RWF38Z"  =  "1"  /* Revision of Spacer in L Sternoclav Jt, Perc Approach */
"0RWF3JZ"  =  "1"  /* Revision of Synth Sub in L Sternoclav Jt, Perc Approach */
"0RWF3KZ"  =  "1"  /* Revision of Nonaut Sub in L Sternoclav Jt, Perc Approach */
"0RWF40Z"  =  "1"  /* Revision of Drain Dev in L Sternoclav Jt, Perc Endo Approach */
"0RWF43Z"  =  "1"  /* Revise Infusion Dev in L Sternoclav Jt, Perc Endo */
"0RWF44Z"  =  "1"  /* Revision of Int Fix in L Sternoclav Jt, Perc Endo Approach */
"0RWF47Z"  =  "1"  /* Revision of Autol Sub in L Sternoclav Jt, Perc Endo Approach */
"0RWF48Z"  =  "1"  /* Revision of Spacer in L Sternoclav Jt, Perc Endo Approach */
"0RWF4JZ"  =  "1"  /* Revision of Synth Sub in L Sternoclav Jt, Perc Endo Approach */
"0RWF4KZ"  =  "1"  /* Revise of Nonaut Sub in L Sternoclav Jt, Perc Endo Approach */
"0RWG00Z"  =  "1"  /* Revision of Drain Dev in R Acromioclav Jt, Open Approach */
"0RWG03Z"  =  "1"  /* Revision of Infusion Dev in R Acromioclav Jt, Open Approach */
"0RWG04Z"  =  "1"  /* Revision of Int Fix in R Acromioclav Jt, Open Approach */
"0RWG07Z"  =  "1"  /* Revision of Autol Sub in R Acromioclav Jt, Open Approach */
"0RWG08Z"  =  "1"  /* Revision of Spacer in R Acromioclav Jt, Open Approach */
"0RWG0JZ"  =  "1"  /* Revision of Synth Sub in R Acromioclav Jt, Open Approach */
"0RWG0KZ"  =  "1"  /* Revision of Nonaut Sub in R Acromioclav Jt, Open Approach */
"0RWG30Z"  =  "1"  /* Revision of Drain Dev in R Acromioclav Jt, Perc Approach */
"0RWG33Z"  =  "1"  /* Revision of Infusion Dev in R Acromioclav Jt, Perc Approach */
"0RWG34Z"  =  "1"  /* Revision of Int Fix in R Acromioclav Jt, Perc Approach */
"0RWG37Z"  =  "1"  /* Revision of Autol Sub in R Acromioclav Jt, Perc Approach */
"0RWG38Z"  =  "1"  /* Revision of Spacer in R Acromioclav Jt, Perc Approach */
"0RWG3JZ"  =  "1"  /* Revision of Synth Sub in R Acromioclav Jt, Perc Approach */
"0RWG3KZ"  =  "1"  /* Revision of Nonaut Sub in R Acromioclav Jt, Perc Approach */
"0RWG40Z"  =  "1"  /* Revise of Drain Dev in R Acromioclav Jt, Perc Endo Approach */
"0RWG43Z"  =  "1"  /* Revise Infusion Dev in R Acromioclav Jt, Perc Endo */
"0RWG44Z"  =  "1"  /* Revision of Int Fix in R Acromioclav Jt, Perc Endo Approach */
"0RWG47Z"  =  "1"  /* Revise of Autol Sub in R Acromioclav Jt, Perc Endo Approach */
"0RWG48Z"  =  "1"  /* Revision of Spacer in R Acromioclav Jt, Perc Endo Approach */
"0RWG4JZ"  =  "1"  /* Revise of Synth Sub in R Acromioclav Jt, Perc Endo Approach */
"0RWG4KZ"  =  "1"  /* Revise of Nonaut Sub in R Acromioclav Jt, Perc Endo Approach */
"0RWH00Z"  =  "1"  /* Revision of Drain Dev in L Acromioclav Jt, Open Approach */
"0RWH03Z"  =  "1"  /* Revision of Infusion Dev in L Acromioclav Jt, Open Approach */
"0RWH04Z"  =  "1"  /* Revision of Int Fix in L Acromioclav Jt, Open Approach */
"0RWH07Z"  =  "1"  /* Revision of Autol Sub in L Acromioclav Jt, Open Approach */
"0RWH08Z"  =  "1"  /* Revision of Spacer in L Acromioclav Jt, Open Approach */
"0RWH0JZ"  =  "1"  /* Revision of Synth Sub in L Acromioclav Jt, Open Approach */
"0RWH0KZ"  =  "1"  /* Revision of Nonaut Sub in L Acromioclav Jt, Open Approach */
"0RWH30Z"  =  "1"  /* Revision of Drain Dev in L Acromioclav Jt, Perc Approach */
"0RWH33Z"  =  "1"  /* Revision of Infusion Dev in L Acromioclav Jt, Perc Approach */
"0RWH34Z"  =  "1"  /* Revision of Int Fix in L Acromioclav Jt, Perc Approach */
"0RWH37Z"  =  "1"  /* Revision of Autol Sub in L Acromioclav Jt, Perc Approach */
"0RWH38Z"  =  "1"  /* Revision of Spacer in L Acromioclav Jt, Perc Approach */
"0RWH3JZ"  =  "1"  /* Revision of Synth Sub in L Acromioclav Jt, Perc Approach */
"0RWH3KZ"  =  "1"  /* Revision of Nonaut Sub in L Acromioclav Jt, Perc Approach */
"0RWH40Z"  =  "1"  /* Revise of Drain Dev in L Acromioclav Jt, Perc Endo Approach */
"0RWH43Z"  =  "1"  /* Revise Infusion Dev in L Acromioclav Jt, Perc Endo */
"0RWH44Z"  =  "1"  /* Revision of Int Fix in L Acromioclav Jt, Perc Endo Approach */
"0RWH47Z"  =  "1"  /* Revise of Autol Sub in L Acromioclav Jt, Perc Endo Approach */
"0RWH48Z"  =  "1"  /* Revision of Spacer in L Acromioclav Jt, Perc Endo Approach */
"0RWH4JZ"  =  "1"  /* Revise of Synth Sub in L Acromioclav Jt, Perc Endo Approach */
"0RWH4KZ"  =  "1"  /* Revise of Nonaut Sub in L Acromioclav Jt, Perc Endo Approach */
"0RWJ00Z"  =  "1"  /* Revision of Drainage Device in R Shoulder Jt, Open Approach */
"0RWJ03Z"  =  "1"  /* Revision of Infusion Device in R Shoulder Jt, Open Approach */
"0RWJ04Z"  =  "1"  /* Revision of Int Fix in R Shoulder Jt, Open Approach */
"0RWJ07Z"  =  "1"  /* Revision of Autol Sub in R Shoulder Jt, Open Approach */
"0RWJ08Z"  =  "1"  /* Revision of Spacer in Right Shoulder Joint, Open Approach */
"0RWJ0JZ"  =  "1"  /* Revision of Synth Sub in R Shoulder Jt, Open Approach */
"0RWJ0KZ"  =  "1"  /* Revision of Nonaut Sub in R Shoulder Jt, Open Approach */
"0RWJ30Z"  =  "1"  /* Revision of Drainage Device in R Shoulder Jt, Perc Approach */
"0RWJ33Z"  =  "1"  /* Revision of Infusion Device in R Shoulder Jt, Perc Approach */
"0RWJ34Z"  =  "1"  /* Revision of Int Fix in R Shoulder Jt, Perc Approach */
"0RWJ37Z"  =  "1"  /* Revision of Autol Sub in R Shoulder Jt, Perc Approach */
"0RWJ38Z"  =  "1"  /* Revision of Spacer in Right Shoulder Joint, Perc Approach */
"0RWJ3JZ"  =  "1"  /* Revision of Synth Sub in R Shoulder Jt, Perc Approach */
"0RWJ3KZ"  =  "1"  /* Revision of Nonaut Sub in R Shoulder Jt, Perc Approach */
"0RWJ40Z"  =  "1"  /* Revision of Drain Dev in R Shoulder Jt, Perc Endo Approach */
"0RWJ43Z"  =  "1"  /* Revise of Infusion Dev in R Shoulder Jt, Perc Endo Approach */
"0RWJ44Z"  =  "1"  /* Revision of Int Fix in R Shoulder Jt, Perc Endo Approach */
"0RWJ47Z"  =  "1"  /* Revision of Autol Sub in R Shoulder Jt, Perc Endo Approach */
"0RWJ48Z"  =  "1"  /* Revision of Spacer in R Shoulder Jt, Perc Endo Approach */
"0RWJ4JZ"  =  "1"  /* Revision of Synth Sub in R Shoulder Jt, Perc Endo Approach */
"0RWJ4KZ"  =  "1"  /* Revision of Nonaut Sub in R Shoulder Jt, Perc Endo Approach */
"0RWK00Z"  =  "1"  /* Revision of Drainage Device in L Shoulder Jt, Open Approach */
"0RWK03Z"  =  "1"  /* Revision of Infusion Device in L Shoulder Jt, Open Approach */
"0RWK04Z"  =  "1"  /* Revision of Int Fix in L Shoulder Jt, Open Approach */
"0RWK07Z"  =  "1"  /* Revision of Autol Sub in L Shoulder Jt, Open Approach */
"0RWK08Z"  =  "1"  /* Revision of Spacer in Left Shoulder Joint, Open Approach */
"0RWK0JZ"  =  "1"  /* Revision of Synth Sub in L Shoulder Jt, Open Approach */
"0RWK0KZ"  =  "1"  /* Revision of Nonaut Sub in L Shoulder Jt, Open Approach */
"0RWK30Z"  =  "1"  /* Revision of Drainage Device in L Shoulder Jt, Perc Approach */
"0RWK33Z"  =  "1"  /* Revision of Infusion Device in L Shoulder Jt, Perc Approach */
"0RWK34Z"  =  "1"  /* Revision of Int Fix in L Shoulder Jt, Perc Approach */
"0RWK37Z"  =  "1"  /* Revision of Autol Sub in L Shoulder Jt, Perc Approach */
"0RWK38Z"  =  "1"  /* Revision of Spacer in Left Shoulder Joint, Perc Approach */
"0RWK3JZ"  =  "1"  /* Revision of Synth Sub in L Shoulder Jt, Perc Approach */
"0RWK3KZ"  =  "1"  /* Revision of Nonaut Sub in L Shoulder Jt, Perc Approach */
"0RWK40Z"  =  "1"  /* Revision of Drain Dev in L Shoulder Jt, Perc Endo Approach */
"0RWK43Z"  =  "1"  /* Revise of Infusion Dev in L Shoulder Jt, Perc Endo Approach */
"0RWK44Z"  =  "1"  /* Revision of Int Fix in L Shoulder Jt, Perc Endo Approach */
"0RWK47Z"  =  "1"  /* Revision of Autol Sub in L Shoulder Jt, Perc Endo Approach */
"0RWK48Z"  =  "1"  /* Revision of Spacer in L Shoulder Jt, Perc Endo Approach */
"0RWK4JZ"  =  "1"  /* Revision of Synth Sub in L Shoulder Jt, Perc Endo Approach */
"0RWK4KZ"  =  "1"  /* Revision of Nonaut Sub in L Shoulder Jt, Perc Endo Approach */
"0RWL00Z"  =  "1"  /* Revision of Drainage Device in R Elbow Jt, Open Approach */
"0RWL03Z"  =  "1"  /* Revision of Infusion Device in R Elbow Jt, Open Approach */
"0RWL04Z"  =  "1"  /* Revision of Int Fix in R Elbow Jt, Open Approach */
"0RWL05Z"  =  "1"  /* Revision of Ext Fix in R Elbow Jt, Open Approach */
"0RWL07Z"  =  "1"  /* Revision of Autol Sub in R Elbow Jt, Open Approach */
"0RWL08Z"  =  "1"  /* Revision of Spacer in Right Elbow Joint, Open Approach */
"0RWL0JZ"  =  "1"  /* Revision of Synth Sub in R Elbow Jt, Open Approach */
"0RWL0KZ"  =  "1"  /* Revision of Nonaut Sub in R Elbow Jt, Open Approach */
"0RWL30Z"  =  "1"  /* Revision of Drainage Device in R Elbow Jt, Perc Approach */
"0RWL33Z"  =  "1"  /* Revision of Infusion Device in R Elbow Jt, Perc Approach */
"0RWL34Z"  =  "1"  /* Revision of Int Fix in R Elbow Jt, Perc Approach */
"0RWL35Z"  =  "1"  /* Revision of Ext Fix in R Elbow Jt, Perc Approach */
"0RWL37Z"  =  "1"  /* Revision of Autol Sub in R Elbow Jt, Perc Approach */
"0RWL38Z"  =  "1"  /* Revision of Spacer in Right Elbow Joint, Perc Approach */
"0RWL3JZ"  =  "1"  /* Revision of Synth Sub in R Elbow Jt, Perc Approach */
"0RWL3KZ"  =  "1"  /* Revision of Nonaut Sub in R Elbow Jt, Perc Approach */
"0RWL40Z"  =  "1"  /* Revision of Drain Dev in R Elbow Jt, Perc Endo Approach */
"0RWL43Z"  =  "1"  /* Revision of Infusion Dev in R Elbow Jt, Perc Endo Approach */
"0RWL44Z"  =  "1"  /* Revision of Int Fix in R Elbow Jt, Perc Endo Approach */
"0RWL45Z"  =  "1"  /* Revision of Ext Fix in R Elbow Jt, Perc Endo Approach */
"0RWL47Z"  =  "1"  /* Revision of Autol Sub in R Elbow Jt, Perc Endo Approach */
"0RWL48Z"  =  "1"  /* Revision of Spacer in Right Elbow Joint, Perc Endo Approach */
"0RWL4JZ"  =  "1"  /* Revision of Synth Sub in R Elbow Jt, Perc Endo Approach */
"0RWL4KZ"  =  "1"  /* Revision of Nonaut Sub in R Elbow Jt, Perc Endo Approach */
"0RWM00Z"  =  "1"  /* Revision of Drainage Device in L Elbow Jt, Open Approach */
"0RWM03Z"  =  "1"  /* Revision of Infusion Device in L Elbow Jt, Open Approach */
"0RWM04Z"  =  "1"  /* Revision of Int Fix in L Elbow Jt, Open Approach */
"0RWM05Z"  =  "1"  /* Revision of Ext Fix in L Elbow Jt, Open Approach */
"0RWM07Z"  =  "1"  /* Revision of Autol Sub in L Elbow Jt, Open Approach */
"0RWM08Z"  =  "1"  /* Revision of Spacer in Left Elbow Joint, Open Approach */
"0RWM0JZ"  =  "1"  /* Revision of Synth Sub in L Elbow Jt, Open Approach */
"0RWM0KZ"  =  "1"  /* Revision of Nonaut Sub in L Elbow Jt, Open Approach */
"0RWM30Z"  =  "1"  /* Revision of Drainage Device in L Elbow Jt, Perc Approach */
"0RWM33Z"  =  "1"  /* Revision of Infusion Device in L Elbow Jt, Perc Approach */
"0RWM34Z"  =  "1"  /* Revision of Int Fix in L Elbow Jt, Perc Approach */
"0RWM35Z"  =  "1"  /* Revision of Ext Fix in L Elbow Jt, Perc Approach */
"0RWM37Z"  =  "1"  /* Revision of Autol Sub in L Elbow Jt, Perc Approach */
"0RWM38Z"  =  "1"  /* Revision of Spacer in Left Elbow Joint, Perc Approach */
"0RWM3JZ"  =  "1"  /* Revision of Synth Sub in L Elbow Jt, Perc Approach */
"0RWM3KZ"  =  "1"  /* Revision of Nonaut Sub in L Elbow Jt, Perc Approach */
"0RWM40Z"  =  "1"  /* Revision of Drain Dev in L Elbow Jt, Perc Endo Approach */
"0RWM43Z"  =  "1"  /* Revision of Infusion Dev in L Elbow Jt, Perc Endo Approach */
"0RWM44Z"  =  "1"  /* Revision of Int Fix in L Elbow Jt, Perc Endo Approach */
"0RWM45Z"  =  "1"  /* Revision of Ext Fix in L Elbow Jt, Perc Endo Approach */
"0RWM47Z"  =  "1"  /* Revision of Autol Sub in L Elbow Jt, Perc Endo Approach */
"0RWM48Z"  =  "1"  /* Revision of Spacer in Left Elbow Joint, Perc Endo Approach */
"0RWM4JZ"  =  "1"  /* Revision of Synth Sub in L Elbow Jt, Perc Endo Approach */
"0RWM4KZ"  =  "1"  /* Revision of Nonaut Sub in L Elbow Jt, Perc Endo Approach */
"0RWN00Z"  =  "1"  /* Revision of Drainage Device in R Wrist Jt, Open Approach */
"0RWN03Z"  =  "1"  /* Revision of Infusion Device in R Wrist Jt, Open Approach */
"0RWN04Z"  =  "1"  /* Revision of Int Fix in R Wrist Jt, Open Approach */
"0RWN05Z"  =  "1"  /* Revision of Ext Fix in R Wrist Jt, Open Approach */
"0RWN07Z"  =  "1"  /* Revision of Autol Sub in R Wrist Jt, Open Approach */
"0RWN08Z"  =  "1"  /* Revision of Spacer in Right Wrist Joint, Open Approach */
"0RWN0JZ"  =  "1"  /* Revision of Synth Sub in R Wrist Jt, Open Approach */
"0RWN0KZ"  =  "1"  /* Revision of Nonaut Sub in R Wrist Jt, Open Approach */
"0RWN30Z"  =  "1"  /* Revision of Drainage Device in R Wrist Jt, Perc Approach */
"0RWN33Z"  =  "1"  /* Revision of Infusion Device in R Wrist Jt, Perc Approach */
"0RWN34Z"  =  "1"  /* Revision of Int Fix in R Wrist Jt, Perc Approach */
"0RWN35Z"  =  "1"  /* Revision of Ext Fix in R Wrist Jt, Perc Approach */
"0RWN37Z"  =  "1"  /* Revision of Autol Sub in R Wrist Jt, Perc Approach */
"0RWN38Z"  =  "1"  /* Revision of Spacer in Right Wrist Joint, Perc Approach */
"0RWN3JZ"  =  "1"  /* Revision of Synth Sub in R Wrist Jt, Perc Approach */
"0RWN3KZ"  =  "1"  /* Revision of Nonaut Sub in R Wrist Jt, Perc Approach */
"0RWN40Z"  =  "1"  /* Revision of Drain Dev in R Wrist Jt, Perc Endo Approach */
"0RWN43Z"  =  "1"  /* Revision of Infusion Dev in R Wrist Jt, Perc Endo Approach */
"0RWN44Z"  =  "1"  /* Revision of Int Fix in R Wrist Jt, Perc Endo Approach */
"0RWN45Z"  =  "1"  /* Revision of Ext Fix in R Wrist Jt, Perc Endo Approach */
"0RWN47Z"  =  "1"  /* Revision of Autol Sub in R Wrist Jt, Perc Endo Approach */
"0RWN48Z"  =  "1"  /* Revision of Spacer in Right Wrist Joint, Perc Endo Approach */
"0RWN4JZ"  =  "1"  /* Revision of Synth Sub in R Wrist Jt, Perc Endo Approach */
"0RWN4KZ"  =  "1"  /* Revision of Nonaut Sub in R Wrist Jt, Perc Endo Approach */
"0RWP00Z"  =  "1"  /* Revision of Drainage Device in L Wrist Jt, Open Approach */
"0RWP03Z"  =  "1"  /* Revision of Infusion Device in L Wrist Jt, Open Approach */
"0RWP04Z"  =  "1"  /* Revision of Int Fix in L Wrist Jt, Open Approach */
"0RWP05Z"  =  "1"  /* Revision of Ext Fix in L Wrist Jt, Open Approach */
"0RWP07Z"  =  "1"  /* Revision of Autol Sub in L Wrist Jt, Open Approach */
"0RWP08Z"  =  "1"  /* Revision of Spacer in Left Wrist Joint, Open Approach */
"0RWP0JZ"  =  "1"  /* Revision of Synth Sub in L Wrist Jt, Open Approach */
"0RWP0KZ"  =  "1"  /* Revision of Nonaut Sub in L Wrist Jt, Open Approach */
"0RWP30Z"  =  "1"  /* Revision of Drainage Device in L Wrist Jt, Perc Approach */
"0RWP33Z"  =  "1"  /* Revision of Infusion Device in L Wrist Jt, Perc Approach */
"0RWP34Z"  =  "1"  /* Revision of Int Fix in L Wrist Jt, Perc Approach */
"0RWP35Z"  =  "1"  /* Revision of Ext Fix in L Wrist Jt, Perc Approach */
"0RWP37Z"  =  "1"  /* Revision of Autol Sub in L Wrist Jt, Perc Approach */
"0RWP38Z"  =  "1"  /* Revision of Spacer in Left Wrist Joint, Perc Approach */
"0RWP3JZ"  =  "1"  /* Revision of Synth Sub in L Wrist Jt, Perc Approach */
"0RWP3KZ"  =  "1"  /* Revision of Nonaut Sub in L Wrist Jt, Perc Approach */
"0RWP40Z"  =  "1"  /* Revision of Drain Dev in L Wrist Jt, Perc Endo Approach */
"0RWP43Z"  =  "1"  /* Revision of Infusion Dev in L Wrist Jt, Perc Endo Approach */
"0RWP44Z"  =  "1"  /* Revision of Int Fix in L Wrist Jt, Perc Endo Approach */
"0RWP45Z"  =  "1"  /* Revision of Ext Fix in L Wrist Jt, Perc Endo Approach */
"0RWP47Z"  =  "1"  /* Revision of Autol Sub in L Wrist Jt, Perc Endo Approach */
"0RWP48Z"  =  "1"  /* Revision of Spacer in Left Wrist Joint, Perc Endo Approach */
"0RWP4JZ"  =  "1"  /* Revision of Synth Sub in L Wrist Jt, Perc Endo Approach */
"0RWP4KZ"  =  "1"  /* Revision of Nonaut Sub in L Wrist Jt, Perc Endo Approach */
"0RWQ00Z"  =  "1"  /* Revision of Drainage Device in R Carpal Jt, Open Approach */
"0RWQ03Z"  =  "1"  /* Revision of Infusion Device in R Carpal Jt, Open Approach */
"0RWQ04Z"  =  "1"  /* Revision of Int Fix in R Carpal Jt, Open Approach */
"0RWQ05Z"  =  "1"  /* Revision of Ext Fix in R Carpal Jt, Open Approach */
"0RWQ07Z"  =  "1"  /* Revision of Autol Sub in R Carpal Jt, Open Approach */
"0RWQ08Z"  =  "1"  /* Revision of Spacer in Right Carpal Joint, Open Approach */
"0RWQ0JZ"  =  "1"  /* Revision of Synth Sub in R Carpal Jt, Open Approach */
"0RWQ0KZ"  =  "1"  /* Revision of Nonaut Sub in R Carpal Jt, Open Approach */
"0RWQ30Z"  =  "1"  /* Revision of Drainage Device in R Carpal Jt, Perc Approach */
"0RWQ33Z"  =  "1"  /* Revision of Infusion Device in R Carpal Jt, Perc Approach */
"0RWQ34Z"  =  "1"  /* Revision of Int Fix in R Carpal Jt, Perc Approach */
"0RWQ35Z"  =  "1"  /* Revision of Ext Fix in R Carpal Jt, Perc Approach */
"0RWQ37Z"  =  "1"  /* Revision of Autol Sub in R Carpal Jt, Perc Approach */
"0RWQ38Z"  =  "1"  /* Revision of Spacer in Right Carpal Joint, Perc Approach */
"0RWQ3JZ"  =  "1"  /* Revision of Synth Sub in R Carpal Jt, Perc Approach */
"0RWQ3KZ"  =  "1"  /* Revision of Nonaut Sub in R Carpal Jt, Perc Approach */
"0RWQ40Z"  =  "1"  /* Revision of Drain Dev in R Carpal Jt, Perc Endo Approach */
"0RWQ43Z"  =  "1"  /* Revision of Infusion Dev in R Carpal Jt, Perc Endo Approach */
"0RWQ44Z"  =  "1"  /* Revision of Int Fix in R Carpal Jt, Perc Endo Approach */
"0RWQ45Z"  =  "1"  /* Revision of Ext Fix in R Carpal Jt, Perc Endo Approach */
"0RWQ47Z"  =  "1"  /* Revision of Autol Sub in R Carpal Jt, Perc Endo Approach */
"0RWQ48Z"  =  "1"  /* Revision of Spacer in Right Carpal Joint, Perc Endo Approach */
"0RWQ4JZ"  =  "1"  /* Revision of Synth Sub in R Carpal Jt, Perc Endo Approach */
"0RWQ4KZ"  =  "1"  /* Revision of Nonaut Sub in R Carpal Jt, Perc Endo Approach */
"0RWR00Z"  =  "1"  /* Revision of Drainage Device in L Carpal Jt, Open Approach */
"0RWR03Z"  =  "1"  /* Revision of Infusion Device in L Carpal Jt, Open Approach */
"0RWR04Z"  =  "1"  /* Revision of Int Fix in L Carpal Jt, Open Approach */
"0RWR05Z"  =  "1"  /* Revision of Ext Fix in L Carpal Jt, Open Approach */
"0RWR07Z"  =  "1"  /* Revision of Autol Sub in L Carpal Jt, Open Approach */
"0RWR08Z"  =  "1"  /* Revision of Spacer in Left Carpal Joint, Open Approach */
"0RWR0JZ"  =  "1"  /* Revision of Synth Sub in L Carpal Jt, Open Approach */
"0RWR0KZ"  =  "1"  /* Revision of Nonaut Sub in L Carpal Jt, Open Approach */
"0RWR30Z"  =  "1"  /* Revision of Drainage Device in L Carpal Jt, Perc Approach */
"0RWR33Z"  =  "1"  /* Revision of Infusion Device in L Carpal Jt, Perc Approach */
"0RWR34Z"  =  "1"  /* Revision of Int Fix in L Carpal Jt, Perc Approach */
"0RWR35Z"  =  "1"  /* Revision of Ext Fix in L Carpal Jt, Perc Approach */
"0RWR37Z"  =  "1"  /* Revision of Autol Sub in L Carpal Jt, Perc Approach */
"0RWR38Z"  =  "1"  /* Revision of Spacer in Left Carpal Joint, Perc Approach */
"0RWR3JZ"  =  "1"  /* Revision of Synth Sub in L Carpal Jt, Perc Approach */
"0RWR3KZ"  =  "1"  /* Revision of Nonaut Sub in L Carpal Jt, Perc Approach */
"0RWR40Z"  =  "1"  /* Revision of Drain Dev in L Carpal Jt, Perc Endo Approach */
"0RWR43Z"  =  "1"  /* Revision of Infusion Dev in L Carpal Jt, Perc Endo Approach */
"0RWR44Z"  =  "1"  /* Revision of Int Fix in L Carpal Jt, Perc Endo Approach */
"0RWR45Z"  =  "1"  /* Revision of Ext Fix in L Carpal Jt, Perc Endo Approach */
"0RWR47Z"  =  "1"  /* Revision of Autol Sub in L Carpal Jt, Perc Endo Approach */
"0RWR48Z"  =  "1"  /* Revision of Spacer in Left Carpal Joint, Perc Endo Approach */
"0RWR4JZ"  =  "1"  /* Revision of Synth Sub in L Carpal Jt, Perc Endo Approach */
"0RWR4KZ"  =  "1"  /* Revision of Nonaut Sub in L Carpal Jt, Perc Endo Approach */
"0RWS00Z"  =  "1"  /* Revision of Drain Dev in R Metacarpocarp Jt, Open Approach */
"0RWS03Z"  =  "1"  /* Revise of Infusion Dev in R Metacarpocarp Jt, Open Approach */
"0RWS04Z"  =  "1"  /* Revision of Int Fix in R Metacarpocarp Jt, Open Approach */
"0RWS05Z"  =  "1"  /* Revision of Ext Fix in R Metacarpocarp Jt, Open Approach */
"0RWS07Z"  =  "1"  /* Revision of Autol Sub in R Metacarpocarp Jt, Open Approach */
"0RWS08Z"  =  "1"  /* Revision of Spacer in R Metacarpocarp Jt, Open Approach */
"0RWS0JZ"  =  "1"  /* Revision of Synth Sub in R Metacarpocarp Jt, Open Approach */
"0RWS0KZ"  =  "1"  /* Revision of Nonaut Sub in R Metacarpocarp Jt, Open Approach */
"0RWS30Z"  =  "1"  /* Revision of Drain Dev in R Metacarpocarp Jt, Perc Approach */
"0RWS33Z"  =  "1"  /* Revise of Infusion Dev in R Metacarpocarp Jt, Perc Approach */
"0RWS34Z"  =  "1"  /* Revision of Int Fix in R Metacarpocarp Jt, Perc Approach */
"0RWS35Z"  =  "1"  /* Revision of Ext Fix in R Metacarpocarp Jt, Perc Approach */
"0RWS37Z"  =  "1"  /* Revision of Autol Sub in R Metacarpocarp Jt, Perc Approach */
"0RWS38Z"  =  "1"  /* Revision of Spacer in R Metacarpocarp Jt, Perc Approach */
"0RWS3JZ"  =  "1"  /* Revision of Synth Sub in R Metacarpocarp Jt, Perc Approach */
"0RWS3KZ"  =  "1"  /* Revision of Nonaut Sub in R Metacarpocarp Jt, Perc Approach */
"0RWS40Z"  =  "1"  /* Revise Drain Dev in R Metacarpocarp Jt, Perc Endo */
"0RWS43Z"  =  "1"  /* Revise Infusion Dev in R Metacarpocarp Jt, Perc Endo */
"0RWS44Z"  =  "1"  /* Revise of Int Fix in R Metacarpocarp Jt, Perc Endo Approach */
"0RWS45Z"  =  "1"  /* Revise of Ext Fix in R Metacarpocarp Jt, Perc Endo Approach */
"0RWS47Z"  =  "1"  /* Revise Autol Sub in R Metacarpocarp Jt, Perc Endo */
"0RWS48Z"  =  "1"  /* Revision of Spacer in R Metacarpocarp Jt, Perc Endo Approach */
"0RWS4JZ"  =  "1"  /* Revise Synth Sub in R Metacarpocarp Jt, Perc Endo */
"0RWS4KZ"  =  "1"  /* Revise Nonaut Sub in R Metacarpocarp Jt, Perc Endo */
"0RWT00Z"  =  "1"  /* Revision of Drain Dev in L Metacarpocarp Jt, Open Approach */
"0RWT03Z"  =  "1"  /* Revise of Infusion Dev in L Metacarpocarp Jt, Open Approach */
"0RWT04Z"  =  "1"  /* Revision of Int Fix in L Metacarpocarp Jt, Open Approach */
"0RWT05Z"  =  "1"  /* Revision of Ext Fix in L Metacarpocarp Jt, Open Approach */
"0RWT07Z"  =  "1"  /* Revision of Autol Sub in L Metacarpocarp Jt, Open Approach */
"0RWT08Z"  =  "1"  /* Revision of Spacer in L Metacarpocarp Jt, Open Approach */
"0RWT0JZ"  =  "1"  /* Revision of Synth Sub in L Metacarpocarp Jt, Open Approach */
"0RWT0KZ"  =  "1"  /* Revision of Nonaut Sub in L Metacarpocarp Jt, Open Approach */
"0RWT30Z"  =  "1"  /* Revision of Drain Dev in L Metacarpocarp Jt, Perc Approach */
"0RWT33Z"  =  "1"  /* Revise of Infusion Dev in L Metacarpocarp Jt, Perc Approach */
"0RWT34Z"  =  "1"  /* Revision of Int Fix in L Metacarpocarp Jt, Perc Approach */
"0RWT35Z"  =  "1"  /* Revision of Ext Fix in L Metacarpocarp Jt, Perc Approach */
"0RWT37Z"  =  "1"  /* Revision of Autol Sub in L Metacarpocarp Jt, Perc Approach */
"0RWT38Z"  =  "1"  /* Revision of Spacer in L Metacarpocarp Jt, Perc Approach */
"0RWT3JZ"  =  "1"  /* Revision of Synth Sub in L Metacarpocarp Jt, Perc Approach */
"0RWT3KZ"  =  "1"  /* Revision of Nonaut Sub in L Metacarpocarp Jt, Perc Approach */
"0RWT40Z"  =  "1"  /* Revise Drain Dev in L Metacarpocarp Jt, Perc Endo */
"0RWT43Z"  =  "1"  /* Revise Infusion Dev in L Metacarpocarp Jt, Perc Endo */
"0RWT44Z"  =  "1"  /* Revise of Int Fix in L Metacarpocarp Jt, Perc Endo Approach */
"0RWT45Z"  =  "1"  /* Revise of Ext Fix in L Metacarpocarp Jt, Perc Endo Approach */
"0RWT47Z"  =  "1"  /* Revise Autol Sub in L Metacarpocarp Jt, Perc Endo */
"0RWT48Z"  =  "1"  /* Revision of Spacer in L Metacarpocarp Jt, Perc Endo Approach */
"0RWT4JZ"  =  "1"  /* Revise Synth Sub in L Metacarpocarp Jt, Perc Endo */
"0RWT4KZ"  =  "1"  /* Revise Nonaut Sub in L Metacarpocarp Jt, Perc Endo */
"0RWU00Z"  =  "1"  /* Revision of Drain Dev in R Metacarpophal Jt, Open Approach */
"0RWU03Z"  =  "1"  /* Revise of Infusion Dev in R Metacarpophal Jt, Open Approach */
"0RWU04Z"  =  "1"  /* Revision of Int Fix in R Metacarpophal Jt, Open Approach */
"0RWU05Z"  =  "1"  /* Revision of Ext Fix in R Metacarpophal Jt, Open Approach */
"0RWU07Z"  =  "1"  /* Revision of Autol Sub in R Metacarpophal Jt, Open Approach */
"0RWU08Z"  =  "1"  /* Revision of Spacer in R Metacarpophal Jt, Open Approach */
"0RWU0JZ"  =  "1"  /* Revision of Synth Sub in R Metacarpophal Jt, Open Approach */
"0RWU0KZ"  =  "1"  /* Revision of Nonaut Sub in R Metacarpophal Jt, Open Approach */
"0RWU30Z"  =  "1"  /* Revision of Drain Dev in R Metacarpophal Jt, Perc Approach */
"0RWU33Z"  =  "1"  /* Revise of Infusion Dev in R Metacarpophal Jt, Perc Approach */
"0RWU34Z"  =  "1"  /* Revision of Int Fix in R Metacarpophal Jt, Perc Approach */
"0RWU35Z"  =  "1"  /* Revision of Ext Fix in R Metacarpophal Jt, Perc Approach */
"0RWU37Z"  =  "1"  /* Revision of Autol Sub in R Metacarpophal Jt, Perc Approach */
"0RWU38Z"  =  "1"  /* Revision of Spacer in R Metacarpophal Jt, Perc Approach */
"0RWU3JZ"  =  "1"  /* Revision of Synth Sub in R Metacarpophal Jt, Perc Approach */
"0RWU3KZ"  =  "1"  /* Revision of Nonaut Sub in R Metacarpophal Jt, Perc Approach */
"0RWU40Z"  =  "1"  /* Revise Drain Dev in R Metacarpophal Jt, Perc Endo */
"0RWU43Z"  =  "1"  /* Revise Infusion Dev in R Metacarpophal Jt, Perc Endo */
"0RWU44Z"  =  "1"  /* Revise of Int Fix in R Metacarpophal Jt, Perc Endo Approach */
"0RWU45Z"  =  "1"  /* Revise of Ext Fix in R Metacarpophal Jt, Perc Endo Approach */
"0RWU47Z"  =  "1"  /* Revise Autol Sub in R Metacarpophal Jt, Perc Endo */
"0RWU48Z"  =  "1"  /* Revision of Spacer in R Metacarpophal Jt, Perc Endo Approach */
"0RWU4JZ"  =  "1"  /* Revise Synth Sub in R Metacarpophal Jt, Perc Endo */
"0RWU4KZ"  =  "1"  /* Revise Nonaut Sub in R Metacarpophal Jt, Perc Endo */
"0RWV00Z"  =  "1"  /* Revision of Drain Dev in L Metacarpophal Jt, Open Approach */
"0RWV03Z"  =  "1"  /* Revise of Infusion Dev in L Metacarpophal Jt, Open Approach */
"0RWV04Z"  =  "1"  /* Revision of Int Fix in L Metacarpophal Jt, Open Approach */
"0RWV05Z"  =  "1"  /* Revision of Ext Fix in L Metacarpophal Jt, Open Approach */
"0RWV07Z"  =  "1"  /* Revision of Autol Sub in L Metacarpophal Jt, Open Approach */
"0RWV08Z"  =  "1"  /* Revision of Spacer in L Metacarpophal Jt, Open Approach */
"0RWV0JZ"  =  "1"  /* Revision of Synth Sub in L Metacarpophal Jt, Open Approach */
"0RWV0KZ"  =  "1"  /* Revision of Nonaut Sub in L Metacarpophal Jt, Open Approach */
"0RWV30Z"  =  "1"  /* Revision of Drain Dev in L Metacarpophal Jt, Perc Approach */
"0RWV33Z"  =  "1"  /* Revise of Infusion Dev in L Metacarpophal Jt, Perc Approach */
"0RWV34Z"  =  "1"  /* Revision of Int Fix in L Metacarpophal Jt, Perc Approach */
"0RWV35Z"  =  "1"  /* Revision of Ext Fix in L Metacarpophal Jt, Perc Approach */
"0RWV37Z"  =  "1"  /* Revision of Autol Sub in L Metacarpophal Jt, Perc Approach */
"0RWV38Z"  =  "1"  /* Revision of Spacer in L Metacarpophal Jt, Perc Approach */
"0RWV3JZ"  =  "1"  /* Revision of Synth Sub in L Metacarpophal Jt, Perc Approach */
"0RWV3KZ"  =  "1"  /* Revision of Nonaut Sub in L Metacarpophal Jt, Perc Approach */
"0RWV40Z"  =  "1"  /* Revise Drain Dev in L Metacarpophal Jt, Perc Endo */
"0RWV43Z"  =  "1"  /* Revise Infusion Dev in L Metacarpophal Jt, Perc Endo */
"0RWV44Z"  =  "1"  /* Revise of Int Fix in L Metacarpophal Jt, Perc Endo Approach */
"0RWV45Z"  =  "1"  /* Revise of Ext Fix in L Metacarpophal Jt, Perc Endo Approach */
"0RWV47Z"  =  "1"  /* Revise Autol Sub in L Metacarpophal Jt, Perc Endo */
"0RWV48Z"  =  "1"  /* Revision of Spacer in L Metacarpophal Jt, Perc Endo Approach */
"0RWV4JZ"  =  "1"  /* Revise Synth Sub in L Metacarpophal Jt, Perc Endo */
"0RWV4KZ"  =  "1"  /* Revise Nonaut Sub in L Metacarpophal Jt, Perc Endo */
"0RWW00Z"  =  "1"  /* Revision of Drain Dev in R Finger Phalanx Jt, Open Approach */
"0RWW03Z"  =  "1"  /* Revise of Infusion Dev in R Finger Phalanx Jt, Open Approach */
"0RWW04Z"  =  "1"  /* Revision of Int Fix in R Finger Phalanx Jt, Open Approach */
"0RWW05Z"  =  "1"  /* Revision of Ext Fix in R Finger Phalanx Jt, Open Approach */
"0RWW07Z"  =  "1"  /* Revision of Autol Sub in R Finger Phalanx Jt, Open Approach */
"0RWW08Z"  =  "1"  /* Revision of Spacer in R Finger Phalanx Jt, Open Approach */
"0RWW0JZ"  =  "1"  /* Revision of Synth Sub in R Finger Phalanx Jt, Open Approach */
"0RWW0KZ"  =  "1"  /* Revision of Nonaut Sub in R Finger Phalanx Jt, Open Approach */
"0RWW30Z"  =  "1"  /* Revision of Drain Dev in R Finger Phalanx Jt, Perc Approach */
"0RWW33Z"  =  "1"  /* Revise of Infusion Dev in R Finger Phalanx Jt, Perc Approach */
"0RWW34Z"  =  "1"  /* Revision of Int Fix in R Finger Phalanx Jt, Perc Approach */
"0RWW35Z"  =  "1"  /* Revision of Ext Fix in R Finger Phalanx Jt, Perc Approach */
"0RWW37Z"  =  "1"  /* Revision of Autol Sub in R Finger Phalanx Jt, Perc Approach */
"0RWW38Z"  =  "1"  /* Revision of Spacer in R Finger Phalanx Jt, Perc Approach */
"0RWW3JZ"  =  "1"  /* Revision of Synth Sub in R Finger Phalanx Jt, Perc Approach */
"0RWW3KZ"  =  "1"  /* Revision of Nonaut Sub in R Finger Phalanx Jt, Perc Approach */
"0RWW40Z"  =  "1"  /* Revise Drain Dev in R Finger Phalanx Jt, Perc Endo */
"0RWW43Z"  =  "1"  /* Revise Infusion Dev in R Finger Phalanx Jt, Perc Endo */
"0RWW44Z"  =  "1"  /* Revise of Int Fix in R Finger Phalanx Jt, Perc Endo Approach */
"0RWW45Z"  =  "1"  /* Revise of Ext Fix in R Finger Phalanx Jt, Perc Endo Approach */
"0RWW47Z"  =  "1"  /* Revise Autol Sub in R Finger Phalanx Jt, Perc Endo */
"0RWW48Z"  =  "1"  /* Revise of Spacer in R Finger Phalanx Jt, Perc Endo Approach */
"0RWW4JZ"  =  "1"  /* Revise Synth Sub in R Finger Phalanx Jt, Perc Endo */
"0RWW4KZ"  =  "1"  /* Revise Nonaut Sub in R Finger Phalanx Jt, Perc Endo */
"0RWX00Z"  =  "1"  /* Revision of Drain Dev in L Finger Phalanx Jt, Open Approach */
"0RWX03Z"  =  "1"  /* Revise of Infusion Dev in L Finger Phalanx Jt, Open Approach */
"0RWX04Z"  =  "1"  /* Revision of Int Fix in L Finger Phalanx Jt, Open Approach */
"0RWX05Z"  =  "1"  /* Revision of Ext Fix in L Finger Phalanx Jt, Open Approach */
"0RWX07Z"  =  "1"  /* Revision of Autol Sub in L Finger Phalanx Jt, Open Approach */
"0RWX08Z"  =  "1"  /* Revision of Spacer in L Finger Phalanx Jt, Open Approach */
"0RWX0JZ"  =  "1"  /* Revision of Synth Sub in L Finger Phalanx Jt, Open Approach */
"0RWX0KZ"  =  "1"  /* Revision of Nonaut Sub in L Finger Phalanx Jt, Open Approach */
"0RWX30Z"  =  "1"  /* Revision of Drain Dev in L Finger Phalanx Jt, Perc Approach */
"0RWX33Z"  =  "1"  /* Revise of Infusion Dev in L Finger Phalanx Jt, Perc Approach */
"0RWX34Z"  =  "1"  /* Revision of Int Fix in L Finger Phalanx Jt, Perc Approach */
"0RWX35Z"  =  "1"  /* Revision of Ext Fix in L Finger Phalanx Jt, Perc Approach */
"0RWX37Z"  =  "1"  /* Revision of Autol Sub in L Finger Phalanx Jt, Perc Approach */
"0RWX38Z"  =  "1"  /* Revision of Spacer in L Finger Phalanx Jt, Perc Approach */
"0RWX3JZ"  =  "1"  /* Revision of Synth Sub in L Finger Phalanx Jt, Perc Approach */
"0RWX3KZ"  =  "1"  /* Revision of Nonaut Sub in L Finger Phalanx Jt, Perc Approach */
"0RWX40Z"  =  "1"  /* Revise Drain Dev in L Finger Phalanx Jt, Perc Endo */
"0RWX43Z"  =  "1"  /* Revise Infusion Dev in L Finger Phalanx Jt, Perc Endo */
"0RWX44Z"  =  "1"  /* Revise of Int Fix in L Finger Phalanx Jt, Perc Endo Approach */
"0RWX45Z"  =  "1"  /* Revise of Ext Fix in L Finger Phalanx Jt, Perc Endo Approach */
"0RWX47Z"  =  "1"  /* Revise Autol Sub in L Finger Phalanx Jt, Perc Endo */
"0RWX48Z"  =  "1"  /* Revise of Spacer in L Finger Phalanx Jt, Perc Endo Approach */
"0RWX4JZ"  =  "1"  /* Revise Synth Sub in L Finger Phalanx Jt, Perc Endo */
"0RWX4KZ"  =  "1"  /* Revise Nonaut Sub in L Finger Phalanx Jt, Perc Endo */
"0S500ZZ"  =  "1"  /* Destruction of Lumbar Vertebral Joint, Open Approach */
"0S503ZZ"  =  "1"  /* Destruction of Lumbar Vertebral Joint, Percutaneous Approach */
"0S504ZZ"  =  "1"  /* Destruction of Lumbar Vertebral Joint, Perc Endo Approach */
"0S520ZZ"  =  "1"  /* Destruction of Lumbar Vertebral Disc, Open Approach */
"0S523ZZ"  =  "1"  /* Destruction of Lumbar Vertebral Disc, Percutaneous Approach */
"0S524ZZ"  =  "1"  /* Destruction of Lumbar Vertebral Disc, Perc Endo Approach */
"0S530ZZ"  =  "1"  /* Destruction of Lumbosacral Joint, Open Approach */
"0S533ZZ"  =  "1"  /* Destruction of Lumbosacral Joint, Percutaneous Approach */
"0S534ZZ"  =  "1"  /* Destruction of Lumbosacral Joint, Perc Endo Approach */
"0S540ZZ"  =  "1"  /* Destruction of Lumbosacral Disc, Open Approach */
"0S543ZZ"  =  "1"  /* Destruction of Lumbosacral Disc, Percutaneous Approach */
"0S544ZZ"  =  "1"  /* Destruction of Lumbosacral Disc, Perc Endo Approach */
"0S550ZZ"  =  "1"  /* Destruction of Sacrococcygeal Joint, Open Approach */
"0S553ZZ"  =  "1"  /* Destruction of Sacrococcygeal Joint, Percutaneous Approach */
"0S554ZZ"  =  "1"  /* Destruction of Sacrococcygeal Joint, Perc Endo Approach */
"0S560ZZ"  =  "1"  /* Destruction of Coccygeal Joint, Open Approach */
"0S563ZZ"  =  "1"  /* Destruction of Coccygeal Joint, Percutaneous Approach */
"0S564ZZ"  =  "1"  /* Destruction of Coccygeal Joint, Perc Endo Approach */
"0S570ZZ"  =  "1"  /* Destruction of Right Sacroiliac Joint, Open Approach */
"0S573ZZ"  =  "1"  /* Destruction of Right Sacroiliac Joint, Percutaneous Approach */
"0S574ZZ"  =  "1"  /* Destruction of Right Sacroiliac Joint, Perc Endo Approach */
"0S580ZZ"  =  "1"  /* Destruction of Left Sacroiliac Joint, Open Approach */
"0S583ZZ"  =  "1"  /* Destruction of Left Sacroiliac Joint, Percutaneous Approach */
"0S584ZZ"  =  "1"  /* Destruction of Left Sacroiliac Joint, Perc Endo Approach */
"0S590ZZ"  =  "1"  /* Destruction of Right Hip Joint, Open Approach */
"0S593ZZ"  =  "1"  /* Destruction of Right Hip Joint, Percutaneous Approach */
"0S594ZZ"  =  "1"  /* Destruction of Right Hip Joint, Perc Endo Approach */
"0S5B0ZZ"  =  "1"  /* Destruction of Left Hip Joint, Open Approach */
"0S5B3ZZ"  =  "1"  /* Destruction of Left Hip Joint, Percutaneous Approach */
"0S5B4ZZ"  =  "1"  /* Destruction of Left Hip Joint, Perc Endo Approach */
"0S5C0ZZ"  =  "1"  /* Destruction of Right Knee Joint, Open Approach */
"0S5C3ZZ"  =  "1"  /* Destruction of Right Knee Joint, Percutaneous Approach */
"0S5C4ZZ"  =  "1"  /* Destruction of Right Knee Joint, Perc Endo Approach */
"0S5D0ZZ"  =  "1"  /* Destruction of Left Knee Joint, Open Approach */
"0S5D3ZZ"  =  "1"  /* Destruction of Left Knee Joint, Percutaneous Approach */
"0S5D4ZZ"  =  "1"  /* Destruction of Left Knee Joint, Perc Endo Approach */
"0S5F0ZZ"  =  "1"  /* Destruction of Right Ankle Joint, Open Approach */
"0S5F3ZZ"  =  "1"  /* Destruction of Right Ankle Joint, Percutaneous Approach */
"0S5F4ZZ"  =  "1"  /* Destruction of Right Ankle Joint, Perc Endo Approach */
"0S5G0ZZ"  =  "1"  /* Destruction of Left Ankle Joint, Open Approach */
"0S5G3ZZ"  =  "1"  /* Destruction of Left Ankle Joint, Percutaneous Approach */
"0S5G4ZZ"  =  "1"  /* Destruction of Left Ankle Joint, Perc Endo Approach */
"0S5H0ZZ"  =  "1"  /* Destruction of Right Tarsal Joint, Open Approach */
"0S5H3ZZ"  =  "1"  /* Destruction of Right Tarsal Joint, Percutaneous Approach */
"0S5H4ZZ"  =  "1"  /* Destruction of Right Tarsal Joint, Perc Endo Approach */
"0S5J0ZZ"  =  "1"  /* Destruction of Left Tarsal Joint, Open Approach */
"0S5J3ZZ"  =  "1"  /* Destruction of Left Tarsal Joint, Percutaneous Approach */
"0S5J4ZZ"  =  "1"  /* Destruction of Left Tarsal Joint, Perc Endo Approach */
"0S5K0ZZ"  =  "1"  /* Destruction of Right Metatarsal-Tarsal Joint, Open Approach */
"0S5K3ZZ"  =  "1"  /* Destruction of Right Metatarsal-Tarsal Joint, Perc Approach */
"0S5K4ZZ"  =  "1"  /* Destruction of R Metatarsotars Jt, Perc Endo Approach */
"0S5L0ZZ"  =  "1"  /* Destruction of Left Metatarsal-Tarsal Joint, Open Approach */
"0S5L3ZZ"  =  "1"  /* Destruction of Left Metatarsal-Tarsal Joint, Perc Approach */
"0S5L4ZZ"  =  "1"  /* Destruction of L Metatarsotars Jt, Perc Endo Approach */
"0S5M0ZZ"  =  "1"  /* Destruction of R Metatarsophal Jt, Open Approach */
"0S5M3ZZ"  =  "1"  /* Destruction of R Metatarsophal Jt, Perc Approach */
"0S5M4ZZ"  =  "1"  /* Destruction of R Metatarsophal Jt, Perc Endo Approach */
"0S5N0ZZ"  =  "1"  /* Destruction of L Metatarsophal Jt, Open Approach */
"0S5N3ZZ"  =  "1"  /* Destruction of L Metatarsophal Jt, Perc Approach */
"0S5N4ZZ"  =  "1"  /* Destruction of L Metatarsophal Jt, Perc Endo Approach */
"0S5P0ZZ"  =  "1"  /* Destruction of Right Toe Phalangeal Joint, Open Approach */
"0S5P3ZZ"  =  "1"  /* Destruction of Right Toe Phalangeal Joint, Perc Approach */
"0S5P4ZZ"  =  "1"  /* Destruction of R Toe Phalanx Jt, Perc Endo Approach */
"0S5Q0ZZ"  =  "1"  /* Destruction of Left Toe Phalangeal Joint, Open Approach */
"0S5Q3ZZ"  =  "1"  /* Destruction of Left Toe Phalangeal Joint, Perc Approach */
"0S5Q4ZZ"  =  "1"  /* Destruction of Left Toe Phalangeal Joint, Perc Endo Approach */
"0S9000Z"  =  "1"  /* Drainage of Lum Jt with Drain Dev, Open Approach */
"0S900ZZ"  =  "1"  /* Drainage of Lumbar Vertebral Joint, Open Approach */
"0S9200Z"  =  "1"  /* Drainage of Lum Disc with Drain Dev, Open Approach */
"0S920ZZ"  =  "1"  /* Drainage of Lumbar Vertebral Disc, Open Approach */
"0S9300Z"  =  "1"  /* Drainage of Lumbosacral Joint with Drain Dev, Open Approach */
"0S930ZZ"  =  "1"  /* Drainage of Lumbosacral Joint, Open Approach */
"0S9400Z"  =  "1"  /* Drainage of Lumbosacral Disc with Drain Dev, Open Approach */
"0S940ZZ"  =  "1"  /* Drainage of Lumbosacral Disc, Open Approach */
"0S9500Z"  =  "1"  /* Drainage of Sacrococcygeal Jt with Drain Dev, Open Approach */
"0S950ZZ"  =  "1"  /* Drainage of Sacrococcygeal Joint, Open Approach */
"0S9600Z"  =  "1"  /* Drainage of Coccygeal Joint with Drain Dev, Open Approach */
"0S960ZZ"  =  "1"  /* Drainage of Coccygeal Joint, Open Approach */
"0S9700Z"  =  "1"  /* Drainage of R Sacroiliac Jt with Drain Dev, Open Approach */
"0S970ZZ"  =  "1"  /* Drainage of Right Sacroiliac Joint, Open Approach */
"0S9800Z"  =  "1"  /* Drainage of L Sacroiliac Jt with Drain Dev, Open Approach */
"0S980ZZ"  =  "1"  /* Drainage of Left Sacroiliac Joint, Open Approach */
"0S9900Z"  =  "1"  /* Drainage of Right Hip Joint with Drain Dev, Open Approach */
"0S990ZZ"  =  "1"  /* Drainage of Right Hip Joint, Open Approach */
"0S9B00Z"  =  "1"  /* Drainage of Left Hip Joint with Drain Dev, Open Approach */
"0S9B0ZZ"  =  "1"  /* Drainage of Left Hip Joint, Open Approach */
"0S9C00Z"  =  "1"  /* Drainage of Right Knee Joint with Drain Dev, Open Approach */
"0S9C0ZZ"  =  "1"  /* Drainage of Right Knee Joint, Open Approach */
"0S9D00Z"  =  "1"  /* Drainage of Left Knee Joint with Drain Dev, Open Approach */
"0S9D0ZZ"  =  "1"  /* Drainage of Left Knee Joint, Open Approach */
"0S9F00Z"  =  "1"  /* Drainage of Right Ankle Joint with Drain Dev, Open Approach */
"0S9F0ZZ"  =  "1"  /* Drainage of Right Ankle Joint, Open Approach */
"0S9G00Z"  =  "1"  /* Drainage of Left Ankle Joint with Drain Dev, Open Approach */
"0S9G0ZZ"  =  "1"  /* Drainage of Left Ankle Joint, Open Approach */
"0S9H00Z"  =  "1"  /* Drainage of Right Tarsal Joint with Drain Dev, Open Approach */
"0S9H0ZZ"  =  "1"  /* Drainage of Right Tarsal Joint, Open Approach */
"0S9J00Z"  =  "1"  /* Drainage of Left Tarsal Joint with Drain Dev, Open Approach */
"0S9J0ZZ"  =  "1"  /* Drainage of Left Tarsal Joint, Open Approach */
"0S9K00Z"  =  "1"  /* Drainage of R Metatarsotars Jt with Drain Dev, Open Approach */
"0S9K0ZZ"  =  "1"  /* Drainage of Right Metatarsal-Tarsal Joint, Open Approach */
"0S9L00Z"  =  "1"  /* Drainage of L Metatarsotars Jt with Drain Dev, Open Approach */
"0S9L0ZZ"  =  "1"  /* Drainage of Left Metatarsal-Tarsal Joint, Open Approach */
"0S9M00Z"  =  "1"  /* Drainage of R Metatarsophal Jt with Drain Dev, Open Approach */
"0S9M0ZZ"  =  "1"  /* Drainage of Right Metatarsal-Phalangeal Joint, Open Approach */
"0S9N00Z"  =  "1"  /* Drainage of L Metatarsophal Jt with Drain Dev, Open Approach */
"0S9N0ZZ"  =  "1"  /* Drainage of Left Metatarsal-Phalangeal Joint, Open Approach */
"0S9P00Z"  =  "1"  /* Drainage of R Toe Phalanx Jt with Drain Dev, Open Approach */
"0S9P0ZZ"  =  "1"  /* Drainage of Right Toe Phalangeal Joint, Open Approach */
"0S9Q00Z"  =  "1"  /* Drainage of L Toe Phalanx Jt with Drain Dev, Open Approach */
"0S9Q0ZZ"  =  "1"  /* Drainage of Left Toe Phalangeal Joint, Open Approach */
"0SB00ZZ"  =  "1"  /* Excision of Lumbar Vertebral Joint, Open Approach */
"0SB03ZZ"  =  "1"  /* Excision of Lumbar Vertebral Joint, Percutaneous Approach */
"0SB04ZZ"  =  "1"  /* Excision of Lumbar Vertebral Joint, Perc Endo Approach */
"0SB20ZZ"  =  "1"  /* Excision of Lumbar Vertebral Disc, Open Approach */
"0SB23ZZ"  =  "1"  /* Excision of Lumbar Vertebral Disc, Percutaneous Approach */
"0SB24ZZ"  =  "1"  /* Excision of Lumbar Vertebral Disc, Perc Endo Approach */
"0SB30ZZ"  =  "1"  /* Excision of Lumbosacral Joint, Open Approach */
"0SB33ZZ"  =  "1"  /* Excision of Lumbosacral Joint, Percutaneous Approach */
"0SB34ZZ"  =  "1"  /* Excision of Lumbosacral Joint, Perc Endo Approach */
"0SB40ZZ"  =  "1"  /* Excision of Lumbosacral Disc, Open Approach */
"0SB43ZZ"  =  "1"  /* Excision of Lumbosacral Disc, Percutaneous Approach */
"0SB44ZZ"  =  "1"  /* Excision of Lumbosacral Disc, Perc Endo Approach */
"0SB50ZZ"  =  "1"  /* Excision of Sacrococcygeal Joint, Open Approach */
"0SB53ZZ"  =  "1"  /* Excision of Sacrococcygeal Joint, Percutaneous Approach */
"0SB54ZZ"  =  "1"  /* Excision of Sacrococcygeal Joint, Perc Endo Approach */
"0SB60ZZ"  =  "1"  /* Excision of Coccygeal Joint, Open Approach */
"0SB63ZZ"  =  "1"  /* Excision of Coccygeal Joint, Percutaneous Approach */
"0SB64ZZ"  =  "1"  /* Excision of Coccygeal Joint, Perc Endo Approach */
"0SB70ZZ"  =  "1"  /* Excision of Right Sacroiliac Joint, Open Approach */
"0SB73ZZ"  =  "1"  /* Excision of Right Sacroiliac Joint, Percutaneous Approach */
"0SB74ZZ"  =  "1"  /* Excision of Right Sacroiliac Joint, Perc Endo Approach */
"0SB80ZZ"  =  "1"  /* Excision of Left Sacroiliac Joint, Open Approach */
"0SB83ZZ"  =  "1"  /* Excision of Left Sacroiliac Joint, Percutaneous Approach */
"0SB84ZZ"  =  "1"  /* Excision of Left Sacroiliac Joint, Perc Endo Approach */
"0SB90ZZ"  =  "1"  /* Excision of Right Hip Joint, Open Approach */
"0SB93ZZ"  =  "1"  /* Excision of Right Hip Joint, Percutaneous Approach */
"0SB94ZZ"  =  "1"  /* Excision of Right Hip Joint, Perc Endo Approach */
"0SBB0ZZ"  =  "1"  /* Excision of Left Hip Joint, Open Approach */
"0SBB3ZZ"  =  "1"  /* Excision of Left Hip Joint, Percutaneous Approach */
"0SBB4ZZ"  =  "1"  /* Excision of Left Hip Joint, Percutaneous Endoscopic Approach */
"0SBC0ZZ"  =  "1"  /* Excision of Right Knee Joint, Open Approach */
"0SBC3ZZ"  =  "1"  /* Excision of Right Knee Joint, Percutaneous Approach */
"0SBC4ZZ"  =  "1"  /* Excision of Right Knee Joint, Perc Endo Approach */
"0SBD0ZZ"  =  "1"  /* Excision of Left Knee Joint, Open Approach */
"0SBD3ZZ"  =  "1"  /* Excision of Left Knee Joint, Percutaneous Approach */
"0SBD4ZZ"  =  "1"  /* Excision of Left Knee Joint, Perc Endo Approach */
"0SBF0ZZ"  =  "1"  /* Excision of Right Ankle Joint, Open Approach */
"0SBF3ZZ"  =  "1"  /* Excision of Right Ankle Joint, Percutaneous Approach */
"0SBF4ZZ"  =  "1"  /* Excision of Right Ankle Joint, Perc Endo Approach */
"0SBG0ZZ"  =  "1"  /* Excision of Left Ankle Joint, Open Approach */
"0SBG3ZZ"  =  "1"  /* Excision of Left Ankle Joint, Percutaneous Approach */
"0SBG4ZZ"  =  "1"  /* Excision of Left Ankle Joint, Perc Endo Approach */
"0SBH0ZZ"  =  "1"  /* Excision of Right Tarsal Joint, Open Approach */
"0SBH3ZZ"  =  "1"  /* Excision of Right Tarsal Joint, Percutaneous Approach */
"0SBH4ZZ"  =  "1"  /* Excision of Right Tarsal Joint, Perc Endo Approach */
"0SBJ0ZZ"  =  "1"  /* Excision of Left Tarsal Joint, Open Approach */
"0SBJ3ZZ"  =  "1"  /* Excision of Left Tarsal Joint, Percutaneous Approach */
"0SBJ4ZZ"  =  "1"  /* Excision of Left Tarsal Joint, Perc Endo Approach */
"0SBK0ZZ"  =  "1"  /* Excision of Right Metatarsal-Tarsal Joint, Open Approach */
"0SBK3ZZ"  =  "1"  /* Excision of Right Metatarsal-Tarsal Joint, Perc Approach */
"0SBK4ZZ"  =  "1"  /* Excision of R Metatarsotars Jt, Perc Endo Approach */
"0SBL0ZZ"  =  "1"  /* Excision of Left Metatarsal-Tarsal Joint, Open Approach */
"0SBL3ZZ"  =  "1"  /* Excision of Left Metatarsal-Tarsal Joint, Perc Approach */
"0SBL4ZZ"  =  "1"  /* Excision of Left Metatarsal-Tarsal Joint, Perc Endo Approach */
"0SBM0ZZ"  =  "1"  /* Excision of Right Metatarsal-Phalangeal Joint, Open Approach */
"0SBM3ZZ"  =  "1"  /* Excision of Right Metatarsal-Phalangeal Joint, Perc Approach */
"0SBM4ZZ"  =  "1"  /* Excision of R Metatarsophal Jt, Perc Endo Approach */
"0SBN0ZZ"  =  "1"  /* Excision of Left Metatarsal-Phalangeal Joint, Open Approach */
"0SBN3ZZ"  =  "1"  /* Excision of Left Metatarsal-Phalangeal Joint, Perc Approach */
"0SBN4ZZ"  =  "1"  /* Excision of L Metatarsophal Jt, Perc Endo Approach */
"0SBP0ZZ"  =  "1"  /* Excision of Right Toe Phalangeal Joint, Open Approach */
"0SBP3ZZ"  =  "1"  /* Excision of Right Toe Phalangeal Joint, Perc Approach */
"0SBP4ZZ"  =  "1"  /* Excision of Right Toe Phalangeal Joint, Perc Endo Approach */
"0SBQ0ZZ"  =  "1"  /* Excision of Left Toe Phalangeal Joint, Open Approach */
"0SBQ3ZZ"  =  "1"  /* Excision of Left Toe Phalangeal Joint, Percutaneous Approach */
"0SBQ4ZZ"  =  "1"  /* Excision of Left Toe Phalangeal Joint, Perc Endo Approach */
"0SC00ZZ"  =  "1"  /* Extirpation of Matter from Lum Jt, Open Approach */
"0SC03ZZ"  =  "1"  /* Extirpation of Matter from Lum Jt, Perc Approach */
"0SC04ZZ"  =  "1"  /* Extirpation of Matter from Lum Jt, Perc Endo Approach */
"0SC20ZZ"  =  "1"  /* Extirpation of Matter from Lum Disc, Open Approach */
"0SC23ZZ"  =  "1"  /* Extirpation of Matter from Lum Disc, Perc Approach */
"0SC24ZZ"  =  "1"  /* Extirpation of Matter from Lum Disc, Perc Endo Approach */
"0SC30ZZ"  =  "1"  /* Extirpation of Matter from Lumbosacral Joint, Open Approach */
"0SC33ZZ"  =  "1"  /* Extirpation of Matter from Lumbosacral Joint, Perc Approach */
"0SC34ZZ"  =  "1"  /* Extirpation of Matter from Lumsac Jt, Perc Endo Approach */
"0SC40ZZ"  =  "1"  /* Extirpation of Matter from Lumbosacral Disc, Open Approach */
"0SC43ZZ"  =  "1"  /* Extirpation of Matter from Lumbosacral Disc, Perc Approach */
"0SC44ZZ"  =  "1"  /* Extirpation of Matter from Lumsac Disc, Perc Endo Approach */
"0SC50ZZ"  =  "1"  /* Extirpation of Matter from Sacrococcygeal Jt, Open Approach */
"0SC53ZZ"  =  "1"  /* Extirpation of Matter from Sacrococcygeal Jt, Perc Approach */
"0SC54ZZ"  =  "1"  /* Extirpate matter from Sacrococcygeal Jt, Perc Endo */
"0SC60ZZ"  =  "1"  /* Extirpation of Matter from Coccygeal Joint, Open Approach */
"0SC63ZZ"  =  "1"  /* Extirpation of Matter from Coccygeal Joint, Perc Approach */
"0SC64ZZ"  =  "1"  /* Extirpation of Matter from Coccygeal Jt, Perc Endo Approach */
"0SC70ZZ"  =  "1"  /* Extirpation of Matter from R Sacroiliac Jt, Open Approach */
"0SC73ZZ"  =  "1"  /* Extirpation of Matter from R Sacroiliac Jt, Perc Approach */
"0SC74ZZ"  =  "1"  /* Extirpate of Matter from R Sacroiliac Jt, Perc Endo Approach */
"0SC80ZZ"  =  "1"  /* Extirpation of Matter from L Sacroiliac Jt, Open Approach */
"0SC83ZZ"  =  "1"  /* Extirpation of Matter from L Sacroiliac Jt, Perc Approach */
"0SC84ZZ"  =  "1"  /* Extirpate of Matter from L Sacroiliac Jt, Perc Endo Approach */
"0SC90ZZ"  =  "1"  /* Extirpation of Matter from Right Hip Joint, Open Approach */
"0SC93ZZ"  =  "1"  /* Extirpation of Matter from Right Hip Joint, Perc Approach */
"0SC94ZZ"  =  "1"  /* Extirpation of Matter from R Hip Jt, Perc Endo Approach */
"0SCB0ZZ"  =  "1"  /* Extirpation of Matter from Left Hip Joint, Open Approach */
"0SCB3ZZ"  =  "1"  /* Extirpation of Matter from Left Hip Joint, Perc Approach */
"0SCB4ZZ"  =  "1"  /* Extirpation of Matter from L Hip Jt, Perc Endo Approach */
"0SCC0ZZ"  =  "1"  /* Extirpation of Matter from Right Knee Joint, Open Approach */
"0SCC3ZZ"  =  "1"  /* Extirpation of Matter from Right Knee Joint, Perc Approach */
"0SCC4ZZ"  =  "1"  /* Extirpation of Matter from R Knee Jt, Perc Endo Approach */
"0SCD0ZZ"  =  "1"  /* Extirpation of Matter from Left Knee Joint, Open Approach */
"0SCD3ZZ"  =  "1"  /* Extirpation of Matter from Left Knee Joint, Perc Approach */
"0SCD4ZZ"  =  "1"  /* Extirpation of Matter from L Knee Jt, Perc Endo Approach */
"0SCF0ZZ"  =  "1"  /* Extirpation of Matter from Right Ankle Joint, Open Approach */
"0SCF3ZZ"  =  "1"  /* Extirpation of Matter from Right Ankle Joint, Perc Approach */
"0SCF4ZZ"  =  "1"  /* Extirpation of Matter from R Ankle Jt, Perc Endo Approach */
"0SCG0ZZ"  =  "1"  /* Extirpation of Matter from Left Ankle Joint, Open Approach */
"0SCG3ZZ"  =  "1"  /* Extirpation of Matter from Left Ankle Joint, Perc Approach */
"0SCG4ZZ"  =  "1"  /* Extirpation of Matter from L Ankle Jt, Perc Endo Approach */
"0SCH0ZZ"  =  "1"  /* Extirpation of Matter from Right Tarsal Joint, Open Approach */
"0SCH3ZZ"  =  "1"  /* Extirpation of Matter from Right Tarsal Joint, Perc Approach */
"0SCH4ZZ"  =  "1"  /* Extirpation of Matter from R Tarsal Jt, Perc Endo Approach */
"0SCJ0ZZ"  =  "1"  /* Extirpation of Matter from Left Tarsal Joint, Open Approach */
"0SCJ3ZZ"  =  "1"  /* Extirpation of Matter from Left Tarsal Joint, Perc Approach */
"0SCJ4ZZ"  =  "1"  /* Extirpation of Matter from L Tarsal Jt, Perc Endo Approach */
"0SCK0ZZ"  =  "1"  /* Extirpation of Matter from R Metatarsotars Jt, Open Approach */
"0SCK3ZZ"  =  "1"  /* Extirpation of Matter from R Metatarsotars Jt, Perc Approach */
"0SCK4ZZ"  =  "1"  /* Extirpate matter from R Metatarsotars Jt, Perc Endo */
"0SCL0ZZ"  =  "1"  /* Extirpation of Matter from L Metatarsotars Jt, Open Approach */
"0SCL3ZZ"  =  "1"  /* Extirpation of Matter from L Metatarsotars Jt, Perc Approach */
"0SCL4ZZ"  =  "1"  /* Extirpate matter from L Metatarsotars Jt, Perc Endo */
"0SCM0ZZ"  =  "1"  /* Extirpation of Matter from R Metatarsophal Jt, Open Approach */
"0SCM3ZZ"  =  "1"  /* Extirpation of Matter from R Metatarsophal Jt, Perc Approach */
"0SCM4ZZ"  =  "1"  /* Extirpate matter from R Metatarsophal Jt, Perc Endo */
"0SCN0ZZ"  =  "1"  /* Extirpation of Matter from L Metatarsophal Jt, Open Approach */
"0SCN3ZZ"  =  "1"  /* Extirpation of Matter from L Metatarsophal Jt, Perc Approach */
"0SCN4ZZ"  =  "1"  /* Extirpate matter from L Metatarsophal Jt, Perc Endo */
"0SCP0ZZ"  =  "1"  /* Extirpation of Matter from R Toe Phalanx Jt, Open Approach */
"0SCP3ZZ"  =  "1"  /* Extirpation of Matter from R Toe Phalanx Jt, Perc Approach */
"0SCP4ZZ"  =  "1"  /* Extirpate matter from R Toe Phalanx Jt, Perc Endo */
"0SCQ0ZZ"  =  "1"  /* Extirpation of Matter from L Toe Phalanx Jt, Open Approach */
"0SCQ3ZZ"  =  "1"  /* Extirpation of Matter from L Toe Phalanx Jt, Perc Approach */
"0SCQ4ZZ"  =  "1"  /* Extirpate matter from L Toe Phalanx Jt, Perc Endo */
"0SG0070"  =  "1"  /* Fusion Lum Jt w Autol Sub, Ant Appr A Col, Open */
"0SG0071"  =  "1"  /* Fusion Lum Jt w Autol Sub, Post Appr P Col, Open */
"0SG007J"  =  "1"  /* Fusion Lum Jt w Autol Sub, Post Appr A Col, Open */
"0SG00A0"  =  "1"  /* Fusion Lum Jt w Intbd Fus Dev, Ant Appr A Col, Open */
"0SG00A1"  =  "1"  /* Fusion Lum Jt w Intbd Fus Dev, Post Appr P Col, Open */
"0SG00AJ"  =  "1"  /* Fusion Lum Jt w Intbd Fus Dev, Post Appr A Col, Open */
"0SG00J0"  =  "1"  /* Fusion Lum Jt w Synth Sub, Ant Appr A Col, Open */
"0SG00J1"  =  "1"  /* Fusion Lum Jt w Synth Sub, Post Appr P Col, Open */
"0SG00JJ"  =  "1"  /* Fusion Lum Jt w Synth Sub, Post Appr A Col, Open */
"0SG00K0"  =  "1"  /* Fusion Lum Jt w Nonaut Sub, Ant Appr A Col, Open */
"0SG00K1"  =  "1"  /* Fusion Lum Jt w Nonaut Sub, Post Appr P Col, Open */
"0SG00KJ"  =  "1"  /* Fusion Lum Jt w Nonaut Sub, Post Appr A Col, Open */
"0SG00Z0"  =  "1"  /* Fusion of Lum Jt, Ant Appr A Col, Open Approach */
"0SG00Z1"  =  "1"  /* Fusion of Lum Jt, Post Appr P Col, Open Approach */
"0SG00ZJ"  =  "1"  /* Fusion of Lum Jt, Post Appr A Col, Open Approach */
"0SG0370"  =  "1"  /* Fusion Lum Jt w Autol Sub, Ant Appr A Col, Perc */
"0SG0371"  =  "1"  /* Fusion Lum Jt w Autol Sub, Post Appr P Col, Perc */
"0SG037J"  =  "1"  /* Fusion Lum Jt w Autol Sub, Post Appr A Col, Perc */
"0SG03A0"  =  "1"  /* Fusion Lum Jt w Intbd Fus Dev, Ant Appr A Col, Perc */
"0SG03A1"  =  "1"  /* Fusion Lum Jt w Intbd Fus Dev, Post Appr P Col, Perc */
"0SG03AJ"  =  "1"  /* Fusion Lum Jt w Intbd Fus Dev, Post Appr A Col, Perc */
"0SG03J0"  =  "1"  /* Fusion Lum Jt w Synth Sub, Ant Appr A Col, Perc */
"0SG03J1"  =  "1"  /* Fusion Lum Jt w Synth Sub, Post Appr P Col, Perc */
"0SG03JJ"  =  "1"  /* Fusion Lum Jt w Synth Sub, Post Appr A Col, Perc */
"0SG03K0"  =  "1"  /* Fusion Lum Jt w Nonaut Sub, Ant Appr A Col, Perc */
"0SG03K1"  =  "1"  /* Fusion Lum Jt w Nonaut Sub, Post Appr P Col, Perc */
"0SG03KJ"  =  "1"  /* Fusion Lum Jt w Nonaut Sub, Post Appr A Col, Perc */
"0SG03Z0"  =  "1"  /* Fusion of Lum Jt, Ant Appr A Col, Perc Approach */
"0SG03Z1"  =  "1"  /* Fusion of Lum Jt, Post Appr P Col, Perc Approach */
"0SG03ZJ"  =  "1"  /* Fusion of Lum Jt, Post Appr A Col, Perc Approach */
"0SG0470"  =  "1"  /* Fusion Lum Jt w Autol Sub, Ant Appr A Col, Perc Endo */
"0SG0471"  =  "1"  /* Fusion Lum Jt w Autol Sub, Post Appr P Col, Perc Endo */
"0SG047J"  =  "1"  /* Fusion Lum Jt w Autol Sub, Post Appr A Col, Perc Endo */
"0SG04A0"  =  "1"  /* Fusion Lum Jt w Intbd Fus Dev, Ant Appr A Col, Perc Endo */
"0SG04A1"  =  "1"  /* Fusion Lum Jt w Intbd Fus Dev, Post Appr P Col, Perc Endo */
"0SG04AJ"  =  "1"  /* Fusion Lum Jt w Intbd Fus Dev, Post Appr A Col, Perc Endo */
"0SG04J0"  =  "1"  /* Fusion Lum Jt w Synth Sub, Ant Appr A Col, Perc Endo */
"0SG04J1"  =  "1"  /* Fusion Lum Jt w Synth Sub, Post Appr P Col, Perc Endo */
"0SG04JJ"  =  "1"  /* Fusion Lum Jt w Synth Sub, Post Appr A Col, Perc Endo */
"0SG04K0"  =  "1"  /* Fusion Lum Jt w Nonaut Sub, Ant Appr A Col, Perc Endo */
"0SG04K1"  =  "1"  /* Fusion Lum Jt w Nonaut Sub, Post Appr P Col, Perc Endo */
"0SG04KJ"  =  "1"  /* Fusion Lum Jt w Nonaut Sub, Post Appr A Col, Perc Endo */
"0SG04Z0"  =  "1"  /* Fusion of Lum Jt, Ant Appr A Col, Perc Endo Approach */
"0SG04Z1"  =  "1"  /* Fusion of Lum Jt, Post Appr P Col, Perc Endo Approach */
"0SG04ZJ"  =  "1"  /* Fusion of Lum Jt, Post Appr A Col, Perc Endo Approach */
"0SG1070"  =  "1"  /* Fusion 2-4 L Jt w Autol Sub, Ant Appr A Col, Open */
"0SG1071"  =  "1"  /* Fusion 2-4 L Jt w Autol Sub, Post Appr P Col, Open */
"0SG107J"  =  "1"  /* Fusion 2-4 L Jt w Autol Sub, Post Appr A Col, Open */
"0SG10A0"  =  "1"  /* Fusion 2-4 L Jt w Intbd Fus Dev, Ant Appr A Col, Open */
"0SG10A1"  =  "1"  /* Fusion 2-4 L Jt w Intbd Fus Dev, Post Appr P Col, Open */
"0SG10AJ"  =  "1"  /* Fusion 2-4 L Jt w Intbd Fus Dev, Post Appr A Col, Open */
"0SG10J0"  =  "1"  /* Fusion 2-4 L Jt w Synth Sub, Ant Appr A Col, Open */
"0SG10J1"  =  "1"  /* Fusion 2-4 L Jt w Synth Sub, Post Appr P Col, Open */
"0SG10JJ"  =  "1"  /* Fusion 2-4 L Jt w Synth Sub, Post Appr A Col, Open */
"0SG10K0"  =  "1"  /* Fusion 2-4 L Jt w Nonaut Sub, Ant Appr A Col, Open */
"0SG10K1"  =  "1"  /* Fusion 2-4 L Jt w Nonaut Sub, Post Appr P Col, Open */
"0SG10KJ"  =  "1"  /* Fusion 2-4 L Jt w Nonaut Sub, Post Appr A Col, Open */
"0SG10Z0"  =  "1"  /* Fusion of 2-4 L Jt, Ant Appr A Col, Open Approach */
"0SG10Z1"  =  "1"  /* Fusion of 2-4 L Jt, Post Appr P Col, Open Approach */
"0SG10ZJ"  =  "1"  /* Fusion of 2-4 L Jt, Post Appr A Col, Open Approach */
"0SG1370"  =  "1"  /* Fusion 2-4 L Jt w Autol Sub, Ant Appr A Col, Perc */
"0SG1371"  =  "1"  /* Fusion 2-4 L Jt w Autol Sub, Post Appr P Col, Perc */
"0SG137J"  =  "1"  /* Fusion 2-4 L Jt w Autol Sub, Post Appr A Col, Perc */
"0SG13A0"  =  "1"  /* Fusion 2-4 L Jt w Intbd Fus Dev, Ant Appr A Col, Perc */
"0SG13A1"  =  "1"  /* Fusion 2-4 L Jt w Intbd Fus Dev, Post Appr P Col, Perc */
"0SG13AJ"  =  "1"  /* Fusion 2-4 L Jt w Intbd Fus Dev, Post Appr A Col, Perc */
"0SG13J0"  =  "1"  /* Fusion 2-4 L Jt w Synth Sub, Ant Appr A Col, Perc */
"0SG13J1"  =  "1"  /* Fusion 2-4 L Jt w Synth Sub, Post Appr P Col, Perc */
"0SG13JJ"  =  "1"  /* Fusion 2-4 L Jt w Synth Sub, Post Appr A Col, Perc */
"0SG13K0"  =  "1"  /* Fusion 2-4 L Jt w Nonaut Sub, Ant Appr A Col, Perc */
"0SG13K1"  =  "1"  /* Fusion 2-4 L Jt w Nonaut Sub, Post Appr P Col, Perc */
"0SG13KJ"  =  "1"  /* Fusion 2-4 L Jt w Nonaut Sub, Post Appr A Col, Perc */
"0SG13Z0"  =  "1"  /* Fusion of 2-4 L Jt, Ant Appr A Col, Perc Approach */
"0SG13Z1"  =  "1"  /* Fusion of 2-4 L Jt, Post Appr P Col, Perc Approach */
"0SG13ZJ"  =  "1"  /* Fusion of 2-4 L Jt, Post Appr A Col, Perc Approach */
"0SG1470"  =  "1"  /* Fusion 2-4 L Jt w Autol Sub, Ant Appr A Col, Perc Endo */
"0SG1471"  =  "1"  /* Fusion 2-4 L Jt w Autol Sub, Post Appr P Col, Perc Endo */
"0SG147J"  =  "1"  /* Fusion 2-4 L Jt w Autol Sub, Post Appr A Col, Perc Endo */
"0SG14A0"  =  "1"  /* Fusion 2-4 L Jt w Intbd Fus Dev, Ant Appr A Col, Perc Endo */
"0SG14A1"  =  "1"  /* Fusion 2-4 L Jt w Intbd Fus Dev, Post Appr P Col, Perc Endo */
"0SG14AJ"  =  "1"  /* Fusion 2-4 L Jt w Intbd Fus Dev, Post Appr A Col, Perc Endo */
"0SG14J0"  =  "1"  /* Fusion 2-4 L Jt w Synth Sub, Ant Appr A Col, Perc Endo */
"0SG14J1"  =  "1"  /* Fusion 2-4 L Jt w Synth Sub, Post Appr P Col, Perc Endo */
"0SG14JJ"  =  "1"  /* Fusion 2-4 L Jt w Synth Sub, Post Appr A Col, Perc Endo */
"0SG14K0"  =  "1"  /* Fusion 2-4 L Jt w Nonaut Sub, Ant Appr A Col, Perc Endo */
"0SG14K1"  =  "1"  /* Fusion 2-4 L Jt w Nonaut Sub, Post Appr P Col, Perc Endo */
"0SG14KJ"  =  "1"  /* Fusion 2-4 L Jt w Nonaut Sub, Post Appr A Col, Perc Endo */
"0SG14Z0"  =  "1"  /* Fusion of 2-4 L Jt, Ant Appr A Col, Perc Endo Approach */
"0SG14Z1"  =  "1"  /* Fusion of 2-4 L Jt, Post Appr P Col, Perc Endo Approach */
"0SG14ZJ"  =  "1"  /* Fusion of 2-4 L Jt, Post Appr A Col, Perc Endo Approach */
"0SG3070"  =  "1"  /* Fusion Lumsac Jt w Autol Sub, Ant Appr A Col, Open */
"0SG3071"  =  "1"  /* Fusion Lumsac Jt w Autol Sub, Post Appr P Col, Open */
"0SG307J"  =  "1"  /* Fusion Lumsac Jt w Autol Sub, Post Appr A Col, Open */
"0SG30A0"  =  "1"  /* Fusion Lumsac Jt w Intbd Fus Dev, Ant Appr A Col, Open */
"0SG30A1"  =  "1"  /* Fusion Lumsac Jt w Intbd Fus Dev, Post Appr P Col, Open */
"0SG30AJ"  =  "1"  /* Fusion Lumsac Jt w Intbd Fus Dev, Post Appr A Col, Open */
"0SG30J0"  =  "1"  /* Fusion Lumsac Jt w Synth Sub, Ant Appr A Col, Open */
"0SG30J1"  =  "1"  /* Fusion Lumsac Jt w Synth Sub, Post Appr P Col, Open */
"0SG30JJ"  =  "1"  /* Fusion Lumsac Jt w Synth Sub, Post Appr A Col, Open */
"0SG30K0"  =  "1"  /* Fusion Lumsac Jt w Nonaut Sub, Ant Appr A Col, Open */
"0SG30K1"  =  "1"  /* Fusion Lumsac Jt w Nonaut Sub, Post Appr P Col, Open */
"0SG30KJ"  =  "1"  /* Fusion Lumsac Jt w Nonaut Sub, Post Appr A Col, Open */
"0SG30Z0"  =  "1"  /* Fusion of Lumbosacral Joint, Ant Appr A Col, Open Approach */
"0SG30Z1"  =  "1"  /* Fusion of Lumbosacral Joint, Post Appr P Col, Open Approach */
"0SG30ZJ"  =  "1"  /* Fusion of Lumbosacral Joint, Post Appr A Col, Open Approach */
"0SG3370"  =  "1"  /* Fusion Lumsac Jt w Autol Sub, Ant Appr A Col, Perc */
"0SG3371"  =  "1"  /* Fusion Lumsac Jt w Autol Sub, Post Appr P Col, Perc */
"0SG337J"  =  "1"  /* Fusion Lumsac Jt w Autol Sub, Post Appr A Col, Perc */
"0SG33A0"  =  "1"  /* Fusion Lumsac Jt w Intbd Fus Dev, Ant Appr A Col, Perc */
"0SG33A1"  =  "1"  /* Fusion Lumsac Jt w Intbd Fus Dev, Post Appr P Col, Perc */
"0SG33AJ"  =  "1"  /* Fusion Lumsac Jt w Intbd Fus Dev, Post Appr A Col, Perc */
"0SG33J0"  =  "1"  /* Fusion Lumsac Jt w Synth Sub, Ant Appr A Col, Perc */
"0SG33J1"  =  "1"  /* Fusion Lumsac Jt w Synth Sub, Post Appr P Col, Perc */
"0SG33JJ"  =  "1"  /* Fusion Lumsac Jt w Synth Sub, Post Appr A Col, Perc */
"0SG33K0"  =  "1"  /* Fusion Lumsac Jt w Nonaut Sub, Ant Appr A Col, Perc */
"0SG33K1"  =  "1"  /* Fusion Lumsac Jt w Nonaut Sub, Post Appr P Col, Perc */
"0SG33KJ"  =  "1"  /* Fusion Lumsac Jt w Nonaut Sub, Post Appr A Col, Perc */
"0SG33Z0"  =  "1"  /* Fusion of Lumbosacral Joint, Ant Appr A Col, Perc Approach */
"0SG33Z1"  =  "1"  /* Fusion of Lumbosacral Joint, Post Appr P Col, Perc Approach */
"0SG33ZJ"  =  "1"  /* Fusion of Lumbosacral Joint, Post Appr A Col, Perc Approach */
"0SG3470"  =  "1"  /* Fusion Lumsac Jt w Autol Sub, Ant Appr A Col, Perc Endo */
"0SG3471"  =  "1"  /* Fusion Lumsac Jt w Autol Sub, Post Appr P Col, Perc Endo */
"0SG347J"  =  "1"  /* Fusion Lumsac Jt w Autol Sub, Post Appr A Col, Perc Endo */
"0SG34A0"  =  "1"  /* Fusion Lumsac Jt w Intbd Fus Dev, Ant Appr A Col, Perc Endo */
"0SG34A1"  =  "1"  /* Fusion Lumsac Jt w Intbd Fus Dev, Post Appr P Col, Perc Endo */
"0SG34AJ"  =  "1"  /* Fusion Lumsac Jt w Intbd Fus Dev, Post Appr A Col, Perc Endo */
"0SG34J0"  =  "1"  /* Fusion Lumsac Jt w Synth Sub, Ant Appr A Col, Perc Endo */
"0SG34J1"  =  "1"  /* Fusion Lumsac Jt w Synth Sub, Post Appr P Col, Perc Endo */
"0SG34JJ"  =  "1"  /* Fusion Lumsac Jt w Synth Sub, Post Appr A Col, Perc Endo */
"0SG34K0"  =  "1"  /* Fusion Lumsac Jt w Nonaut Sub, Ant Appr A Col, Perc Endo */
"0SG34K1"  =  "1"  /* Fusion Lumsac Jt w Nonaut Sub, Post Appr P Col, Perc Endo */
"0SG34KJ"  =  "1"  /* Fusion Lumsac Jt w Nonaut Sub, Post Appr A Col, Perc Endo */
"0SG34Z0"  =  "1"  /* Fusion of Lumsac Jt, Ant Appr A Col, Perc Endo Approach */
"0SG34Z1"  =  "1"  /* Fusion of Lumsac Jt, Post Appr P Col, Perc Endo Approach */
"0SG34ZJ"  =  "1"  /* Fusion of Lumsac Jt, Post Appr A Col, Perc Endo Approach */
"0SG504Z"  =  "1"  /* Fusion of Sacrococcygeal Joint with Int Fix, Open Approach */
"0SG507Z"  =  "1"  /* Fusion of Sacrococcygeal Joint with Autol Sub, Open Approach */
"0SG50JZ"  =  "1"  /* Fusion of Sacrococcygeal Joint with Synth Sub, Open Approach */
"0SG50KZ"  =  "1"  /* Fusion of Sacrococcygeal Jt with Nonaut Sub, Open Approach */
"0SG50ZZ"  =  "1"  /* Fusion of Sacrococcygeal Joint, Open Approach */
"0SG534Z"  =  "1"  /* Fusion of Sacrococcygeal Joint with Int Fix, Perc Approach */
"0SG537Z"  =  "1"  /* Fusion of Sacrococcygeal Joint with Autol Sub, Perc Approach */
"0SG53JZ"  =  "1"  /* Fusion of Sacrococcygeal Joint with Synth Sub, Perc Approach */
"0SG53KZ"  =  "1"  /* Fusion of Sacrococcygeal Jt with Nonaut Sub, Perc Approach */
"0SG53ZZ"  =  "1"  /* Fusion of Sacrococcygeal Joint, Percutaneous Approach */
"0SG544Z"  =  "1"  /* Fusion of Sacrococcygeal Jt with Int Fix, Perc Endo Approach */
"0SG547Z"  =  "1"  /* Fusion Sacrococcygeal Jt w Autol Sub, Perc Endo */
"0SG54JZ"  =  "1"  /* Fusion Sacrococcygeal Jt w Synth Sub, Perc Endo */
"0SG54KZ"  =  "1"  /* Fusion Sacrococcygeal Jt w Nonaut Sub, Perc Endo */
"0SG54ZZ"  =  "1"  /* Fusion of Sacrococcygeal Joint, Perc Endo Approach */
"0SG604Z"  =  "1"  /* Fusion of Coccygeal Joint with Int Fix, Open Approach */
"0SG607Z"  =  "1"  /* Fusion of Coccygeal Joint with Autol Sub, Open Approach */
"0SG60JZ"  =  "1"  /* Fusion of Coccygeal Joint with Synth Sub, Open Approach */
"0SG60KZ"  =  "1"  /* Fusion of Coccygeal Joint with Nonaut Sub, Open Approach */
"0SG60ZZ"  =  "1"  /* Fusion of Coccygeal Joint, Open Approach */
"0SG634Z"  =  "1"  /* Fusion of Coccygeal Joint with Int Fix, Perc Approach */
"0SG637Z"  =  "1"  /* Fusion of Coccygeal Joint with Autol Sub, Perc Approach */
"0SG63JZ"  =  "1"  /* Fusion of Coccygeal Joint with Synth Sub, Perc Approach */
"0SG63KZ"  =  "1"  /* Fusion of Coccygeal Joint with Nonaut Sub, Perc Approach */
"0SG63ZZ"  =  "1"  /* Fusion of Coccygeal Joint, Percutaneous Approach */
"0SG644Z"  =  "1"  /* Fusion of Coccygeal Joint with Int Fix, Perc Endo Approach */
"0SG647Z"  =  "1"  /* Fusion of Coccygeal Joint with Autol Sub, Perc Endo Approach */
"0SG64JZ"  =  "1"  /* Fusion of Coccygeal Joint with Synth Sub, Perc Endo Approach */
"0SG64KZ"  =  "1"  /* Fusion of Coccygeal Jt with Nonaut Sub, Perc Endo Approach */
"0SG64ZZ"  =  "1"  /* Fusion of Coccygeal Joint, Percutaneous Endoscopic Approach */
"0SG704Z"  =  "1"  /* Fusion of Right Sacroiliac Joint with Int Fix, Open Approach */
"0SG707Z"  =  "1"  /* Fusion of R Sacroiliac Jt with Autol Sub, Open Approach */
"0SG70JZ"  =  "1"  /* Fusion of R Sacroiliac Jt with Synth Sub, Open Approach */
"0SG70KZ"  =  "1"  /* Fusion of R Sacroiliac Jt with Nonaut Sub, Open Approach */
"0SG70ZZ"  =  "1"  /* Fusion of Right Sacroiliac Joint, Open Approach */
"0SG734Z"  =  "1"  /* Fusion of Right Sacroiliac Joint with Int Fix, Perc Approach */
"0SG737Z"  =  "1"  /* Fusion of R Sacroiliac Jt with Autol Sub, Perc Approach */
"0SG73JZ"  =  "1"  /* Fusion of R Sacroiliac Jt with Synth Sub, Perc Approach */
"0SG73KZ"  =  "1"  /* Fusion of R Sacroiliac Jt with Nonaut Sub, Perc Approach */
"0SG73ZZ"  =  "1"  /* Fusion of Right Sacroiliac Joint, Percutaneous Approach */
"0SG744Z"  =  "1"  /* Fusion of R Sacroiliac Jt with Int Fix, Perc Endo Approach */
"0SG747Z"  =  "1"  /* Fusion of R Sacroiliac Jt with Autol Sub, Perc Endo Approach */
"0SG74JZ"  =  "1"  /* Fusion of R Sacroiliac Jt with Synth Sub, Perc Endo Approach */
"0SG74KZ"  =  "1"  /* Fusion R Sacroiliac Jt w Nonaut Sub, Perc Endo */
"0SG74ZZ"  =  "1"  /* Fusion of Right Sacroiliac Joint, Perc Endo Approach */
"0SG804Z"  =  "1"  /* Fusion of Left Sacroiliac Joint with Int Fix, Open Approach */
"0SG807Z"  =  "1"  /* Fusion of L Sacroiliac Jt with Autol Sub, Open Approach */
"0SG80JZ"  =  "1"  /* Fusion of L Sacroiliac Jt with Synth Sub, Open Approach */
"0SG80KZ"  =  "1"  /* Fusion of L Sacroiliac Jt with Nonaut Sub, Open Approach */
"0SG80ZZ"  =  "1"  /* Fusion of Left Sacroiliac Joint, Open Approach */
"0SG834Z"  =  "1"  /* Fusion of Left Sacroiliac Joint with Int Fix, Perc Approach */
"0SG837Z"  =  "1"  /* Fusion of L Sacroiliac Jt with Autol Sub, Perc Approach */
"0SG83JZ"  =  "1"  /* Fusion of L Sacroiliac Jt with Synth Sub, Perc Approach */
"0SG83KZ"  =  "1"  /* Fusion of L Sacroiliac Jt with Nonaut Sub, Perc Approach */
"0SG83ZZ"  =  "1"  /* Fusion of Left Sacroiliac Joint, Percutaneous Approach */
"0SG844Z"  =  "1"  /* Fusion of L Sacroiliac Jt with Int Fix, Perc Endo Approach */
"0SG847Z"  =  "1"  /* Fusion of L Sacroiliac Jt with Autol Sub, Perc Endo Approach */
"0SG84JZ"  =  "1"  /* Fusion of L Sacroiliac Jt with Synth Sub, Perc Endo Approach */
"0SG84KZ"  =  "1"  /* Fusion L Sacroiliac Jt w Nonaut Sub, Perc Endo */
"0SG84ZZ"  =  "1"  /* Fusion of Left Sacroiliac Joint, Perc Endo Approach */
"0SG904Z"  =  "1"  /* Fusion of Right Hip Joint with Int Fix, Open Approach */
"0SG905Z"  =  "1"  /* Fusion of Right Hip Joint with Ext Fix, Open Approach */
"0SG907Z"  =  "1"  /* Fusion of Right Hip Joint with Autol Sub, Open Approach */
"0SG90JZ"  =  "1"  /* Fusion of Right Hip Joint with Synth Sub, Open Approach */
"0SG90KZ"  =  "1"  /* Fusion of Right Hip Joint with Nonaut Sub, Open Approach */
"0SG90ZZ"  =  "1"  /* Fusion of Right Hip Joint, Open Approach */
"0SG934Z"  =  "1"  /* Fusion of Right Hip Joint with Int Fix, Perc Approach */
"0SG935Z"  =  "1"  /* Fusion of Right Hip Joint with Ext Fix, Perc Approach */
"0SG937Z"  =  "1"  /* Fusion of Right Hip Joint with Autol Sub, Perc Approach */
"0SG93JZ"  =  "1"  /* Fusion of Right Hip Joint with Synth Sub, Perc Approach */
"0SG93KZ"  =  "1"  /* Fusion of Right Hip Joint with Nonaut Sub, Perc Approach */
"0SG93ZZ"  =  "1"  /* Fusion of Right Hip Joint, Percutaneous Approach */
"0SG944Z"  =  "1"  /* Fusion of Right Hip Joint with Int Fix, Perc Endo Approach */
"0SG945Z"  =  "1"  /* Fusion of Right Hip Joint with Ext Fix, Perc Endo Approach */
"0SG947Z"  =  "1"  /* Fusion of Right Hip Joint with Autol Sub, Perc Endo Approach */
"0SG94JZ"  =  "1"  /* Fusion of Right Hip Joint with Synth Sub, Perc Endo Approach */
"0SG94KZ"  =  "1"  /* Fusion of R Hip Jt with Nonaut Sub, Perc Endo Approach */
"0SG94ZZ"  =  "1"  /* Fusion of Right Hip Joint, Percutaneous Endoscopic Approach */
"0SGB04Z"  =  "1"  /* Fusion of Left Hip Joint with Int Fix, Open Approach */
"0SGB05Z"  =  "1"  /* Fusion of Left Hip Joint with Ext Fix, Open Approach */
"0SGB07Z"  =  "1"  /* Fusion of Left Hip Joint with Autol Sub, Open Approach */
"0SGB0JZ"  =  "1"  /* Fusion of Left Hip Joint with Synth Sub, Open Approach */
"0SGB0KZ"  =  "1"  /* Fusion of Left Hip Joint with Nonaut Sub, Open Approach */
"0SGB0ZZ"  =  "1"  /* Fusion of Left Hip Joint, Open Approach */
"0SGB34Z"  =  "1"  /* Fusion of Left Hip Joint with Int Fix, Perc Approach */
"0SGB35Z"  =  "1"  /* Fusion of Left Hip Joint with Ext Fix, Perc Approach */
"0SGB37Z"  =  "1"  /* Fusion of Left Hip Joint with Autol Sub, Perc Approach */
"0SGB3JZ"  =  "1"  /* Fusion of Left Hip Joint with Synth Sub, Perc Approach */
"0SGB3KZ"  =  "1"  /* Fusion of Left Hip Joint with Nonaut Sub, Perc Approach */
"0SGB3ZZ"  =  "1"  /* Fusion of Left Hip Joint, Percutaneous Approach */
"0SGB44Z"  =  "1"  /* Fusion of Left Hip Joint with Int Fix, Perc Endo Approach */
"0SGB45Z"  =  "1"  /* Fusion of Left Hip Joint with Ext Fix, Perc Endo Approach */
"0SGB47Z"  =  "1"  /* Fusion of Left Hip Joint with Autol Sub, Perc Endo Approach */
"0SGB4JZ"  =  "1"  /* Fusion of Left Hip Joint with Synth Sub, Perc Endo Approach */
"0SGB4KZ"  =  "1"  /* Fusion of Left Hip Joint with Nonaut Sub, Perc Endo Approach */
"0SGB4ZZ"  =  "1"  /* Fusion of Left Hip Joint, Percutaneous Endoscopic Approach */
"0SGC04Z"  =  "1"  /* Fusion of Right Knee Joint with Int Fix, Open Approach */
"0SGC05Z"  =  "1"  /* Fusion of Right Knee Joint with Ext Fix, Open Approach */
"0SGC07Z"  =  "1"  /* Fusion of Right Knee Joint with Autol Sub, Open Approach */
"0SGC0JZ"  =  "1"  /* Fusion of Right Knee Joint with Synth Sub, Open Approach */
"0SGC0KZ"  =  "1"  /* Fusion of Right Knee Joint with Nonaut Sub, Open Approach */
"0SGC0ZZ"  =  "1"  /* Fusion of Right Knee Joint, Open Approach */
"0SGC34Z"  =  "1"  /* Fusion of Right Knee Joint with Int Fix, Perc Approach */
"0SGC35Z"  =  "1"  /* Fusion of Right Knee Joint with Ext Fix, Perc Approach */
"0SGC37Z"  =  "1"  /* Fusion of Right Knee Joint with Autol Sub, Perc Approach */
"0SGC3JZ"  =  "1"  /* Fusion of Right Knee Joint with Synth Sub, Perc Approach */
"0SGC3KZ"  =  "1"  /* Fusion of Right Knee Joint with Nonaut Sub, Perc Approach */
"0SGC3ZZ"  =  "1"  /* Fusion of Right Knee Joint, Percutaneous Approach */
"0SGC44Z"  =  "1"  /* Fusion of Right Knee Joint with Int Fix, Perc Endo Approach */
"0SGC45Z"  =  "1"  /* Fusion of Right Knee Joint with Ext Fix, Perc Endo Approach */
"0SGC47Z"  =  "1"  /* Fusion of R Knee Jt with Autol Sub, Perc Endo Approach */
"0SGC4JZ"  =  "1"  /* Fusion of R Knee Jt with Synth Sub, Perc Endo Approach */
"0SGC4KZ"  =  "1"  /* Fusion of R Knee Jt with Nonaut Sub, Perc Endo Approach */
"0SGC4ZZ"  =  "1"  /* Fusion of Right Knee Joint, Percutaneous Endoscopic Approach */
"0SGD04Z"  =  "1"  /* Fusion of Left Knee Joint with Int Fix, Open Approach */
"0SGD05Z"  =  "1"  /* Fusion of Left Knee Joint with Ext Fix, Open Approach */
"0SGD07Z"  =  "1"  /* Fusion of Left Knee Joint with Autol Sub, Open Approach */
"0SGD0JZ"  =  "1"  /* Fusion of Left Knee Joint with Synth Sub, Open Approach */
"0SGD0KZ"  =  "1"  /* Fusion of Left Knee Joint with Nonaut Sub, Open Approach */
"0SGD0ZZ"  =  "1"  /* Fusion of Left Knee Joint, Open Approach */
"0SGD34Z"  =  "1"  /* Fusion of Left Knee Joint with Int Fix, Perc Approach */
"0SGD35Z"  =  "1"  /* Fusion of Left Knee Joint with Ext Fix, Perc Approach */
"0SGD37Z"  =  "1"  /* Fusion of Left Knee Joint with Autol Sub, Perc Approach */
"0SGD3JZ"  =  "1"  /* Fusion of Left Knee Joint with Synth Sub, Perc Approach */
"0SGD3KZ"  =  "1"  /* Fusion of Left Knee Joint with Nonaut Sub, Perc Approach */
"0SGD3ZZ"  =  "1"  /* Fusion of Left Knee Joint, Percutaneous Approach */
"0SGD44Z"  =  "1"  /* Fusion of Left Knee Joint with Int Fix, Perc Endo Approach */
"0SGD45Z"  =  "1"  /* Fusion of Left Knee Joint with Ext Fix, Perc Endo Approach */
"0SGD47Z"  =  "1"  /* Fusion of Left Knee Joint with Autol Sub, Perc Endo Approach */
"0SGD4JZ"  =  "1"  /* Fusion of Left Knee Joint with Synth Sub, Perc Endo Approach */
"0SGD4KZ"  =  "1"  /* Fusion of L Knee Jt with Nonaut Sub, Perc Endo Approach */
"0SGD4ZZ"  =  "1"  /* Fusion of Left Knee Joint, Percutaneous Endoscopic Approach */
"0SGF04Z"  =  "1"  /* Fusion of Right Ankle Joint with Int Fix, Open Approach */
"0SGF05Z"  =  "1"  /* Fusion of Right Ankle Joint with Ext Fix, Open Approach */
"0SGF07Z"  =  "1"  /* Fusion of Right Ankle Joint with Autol Sub, Open Approach */
"0SGF0JZ"  =  "1"  /* Fusion of Right Ankle Joint with Synth Sub, Open Approach */
"0SGF0KZ"  =  "1"  /* Fusion of Right Ankle Joint with Nonaut Sub, Open Approach */
"0SGF0ZZ"  =  "1"  /* Fusion of Right Ankle Joint, Open Approach */
"0SGF34Z"  =  "1"  /* Fusion of Right Ankle Joint with Int Fix, Perc Approach */
"0SGF35Z"  =  "1"  /* Fusion of Right Ankle Joint with Ext Fix, Perc Approach */
"0SGF37Z"  =  "1"  /* Fusion of Right Ankle Joint with Autol Sub, Perc Approach */
"0SGF3JZ"  =  "1"  /* Fusion of Right Ankle Joint with Synth Sub, Perc Approach */
"0SGF3KZ"  =  "1"  /* Fusion of Right Ankle Joint with Nonaut Sub, Perc Approach */
"0SGF3ZZ"  =  "1"  /* Fusion of Right Ankle Joint, Percutaneous Approach */
"0SGF44Z"  =  "1"  /* Fusion of Right Ankle Joint with Int Fix, Perc Endo Approach */
"0SGF45Z"  =  "1"  /* Fusion of Right Ankle Joint with Ext Fix, Perc Endo Approach */
"0SGF47Z"  =  "1"  /* Fusion of R Ankle Jt with Autol Sub, Perc Endo Approach */
"0SGF4JZ"  =  "1"  /* Fusion of R Ankle Jt with Synth Sub, Perc Endo Approach */
"0SGF4KZ"  =  "1"  /* Fusion of R Ankle Jt with Nonaut Sub, Perc Endo Approach */
"0SGF4ZZ"  =  "1"  /* Fusion of Right Ankle Joint, Perc Endo Approach */
"0SGG04Z"  =  "1"  /* Fusion of Left Ankle Joint with Int Fix, Open Approach */
"0SGG05Z"  =  "1"  /* Fusion of Left Ankle Joint with Ext Fix, Open Approach */
"0SGG07Z"  =  "1"  /* Fusion of Left Ankle Joint with Autol Sub, Open Approach */
"0SGG0JZ"  =  "1"  /* Fusion of Left Ankle Joint with Synth Sub, Open Approach */
"0SGG0KZ"  =  "1"  /* Fusion of Left Ankle Joint with Nonaut Sub, Open Approach */
"0SGG0ZZ"  =  "1"  /* Fusion of Left Ankle Joint, Open Approach */
"0SGG34Z"  =  "1"  /* Fusion of Left Ankle Joint with Int Fix, Perc Approach */
"0SGG35Z"  =  "1"  /* Fusion of Left Ankle Joint with Ext Fix, Perc Approach */
"0SGG37Z"  =  "1"  /* Fusion of Left Ankle Joint with Autol Sub, Perc Approach */
"0SGG3JZ"  =  "1"  /* Fusion of Left Ankle Joint with Synth Sub, Perc Approach */
"0SGG3KZ"  =  "1"  /* Fusion of Left Ankle Joint with Nonaut Sub, Perc Approach */
"0SGG3ZZ"  =  "1"  /* Fusion of Left Ankle Joint, Percutaneous Approach */
"0SGG44Z"  =  "1"  /* Fusion of Left Ankle Joint with Int Fix, Perc Endo Approach */
"0SGG45Z"  =  "1"  /* Fusion of Left Ankle Joint with Ext Fix, Perc Endo Approach */
"0SGG47Z"  =  "1"  /* Fusion of L Ankle Jt with Autol Sub, Perc Endo Approach */
"0SGG4JZ"  =  "1"  /* Fusion of L Ankle Jt with Synth Sub, Perc Endo Approach */
"0SGG4KZ"  =  "1"  /* Fusion of L Ankle Jt with Nonaut Sub, Perc Endo Approach */
"0SGG4ZZ"  =  "1"  /* Fusion of Left Ankle Joint, Percutaneous Endoscopic Approach */
"0SGH04Z"  =  "1"  /* Fusion of Right Tarsal Joint with Int Fix, Open Approach */
"0SGH05Z"  =  "1"  /* Fusion of Right Tarsal Joint with Ext Fix, Open Approach */
"0SGH07Z"  =  "1"  /* Fusion of Right Tarsal Joint with Autol Sub, Open Approach */
"0SGH0JZ"  =  "1"  /* Fusion of Right Tarsal Joint with Synth Sub, Open Approach */
"0SGH0KZ"  =  "1"  /* Fusion of Right Tarsal Joint with Nonaut Sub, Open Approach */
"0SGH0ZZ"  =  "1"  /* Fusion of Right Tarsal Joint, Open Approach */
"0SGH34Z"  =  "1"  /* Fusion of Right Tarsal Joint with Int Fix, Perc Approach */
"0SGH35Z"  =  "1"  /* Fusion of Right Tarsal Joint with Ext Fix, Perc Approach */
"0SGH37Z"  =  "1"  /* Fusion of Right Tarsal Joint with Autol Sub, Perc Approach */
"0SGH3JZ"  =  "1"  /* Fusion of Right Tarsal Joint with Synth Sub, Perc Approach */
"0SGH3KZ"  =  "1"  /* Fusion of Right Tarsal Joint with Nonaut Sub, Perc Approach */
"0SGH3ZZ"  =  "1"  /* Fusion of Right Tarsal Joint, Percutaneous Approach */
"0SGH44Z"  =  "1"  /* Fusion of R Tarsal Jt with Int Fix, Perc Endo Approach */
"0SGH45Z"  =  "1"  /* Fusion of R Tarsal Jt with Ext Fix, Perc Endo Approach */
"0SGH47Z"  =  "1"  /* Fusion of R Tarsal Jt with Autol Sub, Perc Endo Approach */
"0SGH4JZ"  =  "1"  /* Fusion of R Tarsal Jt with Synth Sub, Perc Endo Approach */
"0SGH4KZ"  =  "1"  /* Fusion of R Tarsal Jt with Nonaut Sub, Perc Endo Approach */
"0SGH4ZZ"  =  "1"  /* Fusion of Right Tarsal Joint, Perc Endo Approach */
"0SGJ04Z"  =  "1"  /* Fusion of Left Tarsal Joint with Int Fix, Open Approach */
"0SGJ05Z"  =  "1"  /* Fusion of Left Tarsal Joint with Ext Fix, Open Approach */
"0SGJ07Z"  =  "1"  /* Fusion of Left Tarsal Joint with Autol Sub, Open Approach */
"0SGJ0JZ"  =  "1"  /* Fusion of Left Tarsal Joint with Synth Sub, Open Approach */
"0SGJ0KZ"  =  "1"  /* Fusion of Left Tarsal Joint with Nonaut Sub, Open Approach */
"0SGJ0ZZ"  =  "1"  /* Fusion of Left Tarsal Joint, Open Approach */
"0SGJ34Z"  =  "1"  /* Fusion of Left Tarsal Joint with Int Fix, Perc Approach */
"0SGJ35Z"  =  "1"  /* Fusion of Left Tarsal Joint with Ext Fix, Perc Approach */
"0SGJ37Z"  =  "1"  /* Fusion of Left Tarsal Joint with Autol Sub, Perc Approach */
"0SGJ3JZ"  =  "1"  /* Fusion of Left Tarsal Joint with Synth Sub, Perc Approach */
"0SGJ3KZ"  =  "1"  /* Fusion of Left Tarsal Joint with Nonaut Sub, Perc Approach */
"0SGJ3ZZ"  =  "1"  /* Fusion of Left Tarsal Joint, Percutaneous Approach */
"0SGJ44Z"  =  "1"  /* Fusion of Left Tarsal Joint with Int Fix, Perc Endo Approach */
"0SGJ45Z"  =  "1"  /* Fusion of Left Tarsal Joint with Ext Fix, Perc Endo Approach */
"0SGJ47Z"  =  "1"  /* Fusion of L Tarsal Jt with Autol Sub, Perc Endo Approach */
"0SGJ4JZ"  =  "1"  /* Fusion of L Tarsal Jt with Synth Sub, Perc Endo Approach */
"0SGJ4KZ"  =  "1"  /* Fusion of L Tarsal Jt with Nonaut Sub, Perc Endo Approach */
"0SGJ4ZZ"  =  "1"  /* Fusion of Left Tarsal Joint, Perc Endo Approach */
"0SGK04Z"  =  "1"  /* Fusion of R Metatarsotars Jt with Int Fix, Open Approach */
"0SGK05Z"  =  "1"  /* Fusion of R Metatarsotars Jt with Ext Fix, Open Approach */
"0SGK07Z"  =  "1"  /* Fusion of R Metatarsotars Jt with Autol Sub, Open Approach */
"0SGK0JZ"  =  "1"  /* Fusion of R Metatarsotars Jt with Synth Sub, Open Approach */
"0SGK0KZ"  =  "1"  /* Fusion of R Metatarsotars Jt with Nonaut Sub, Open Approach */
"0SGK0ZZ"  =  "1"  /* Fusion of Right Metatarsal-Tarsal Joint, Open Approach */
"0SGK34Z"  =  "1"  /* Fusion of R Metatarsotars Jt with Int Fix, Perc Approach */
"0SGK35Z"  =  "1"  /* Fusion of R Metatarsotars Jt with Ext Fix, Perc Approach */
"0SGK37Z"  =  "1"  /* Fusion of R Metatarsotars Jt with Autol Sub, Perc Approach */
"0SGK3JZ"  =  "1"  /* Fusion of R Metatarsotars Jt with Synth Sub, Perc Approach */
"0SGK3KZ"  =  "1"  /* Fusion of R Metatarsotars Jt with Nonaut Sub, Perc Approach */
"0SGK3ZZ"  =  "1"  /* Fusion of Right Metatarsal-Tarsal Joint, Perc Approach */
"0SGK44Z"  =  "1"  /* Fusion R Metatarsotars Jt w Int Fix, Perc Endo */
"0SGK45Z"  =  "1"  /* Fusion R Metatarsotars Jt w Ext Fix, Perc Endo */
"0SGK47Z"  =  "1"  /* Fusion R Metatarsotars Jt w Autol Sub, Perc Endo */
"0SGK4JZ"  =  "1"  /* Fusion R Metatarsotars Jt w Synth Sub, Perc Endo */
"0SGK4KZ"  =  "1"  /* Fusion R Metatarsotars Jt w Nonaut Sub, Perc Endo */
"0SGK4ZZ"  =  "1"  /* Fusion of Right Metatarsal-Tarsal Joint, Perc Endo Approach */
"0SGL04Z"  =  "1"  /* Fusion of L Metatarsotars Jt with Int Fix, Open Approach */
"0SGL05Z"  =  "1"  /* Fusion of L Metatarsotars Jt with Ext Fix, Open Approach */
"0SGL07Z"  =  "1"  /* Fusion of L Metatarsotars Jt with Autol Sub, Open Approach */
"0SGL0JZ"  =  "1"  /* Fusion of L Metatarsotars Jt with Synth Sub, Open Approach */
"0SGL0KZ"  =  "1"  /* Fusion of L Metatarsotars Jt with Nonaut Sub, Open Approach */
"0SGL0ZZ"  =  "1"  /* Fusion of Left Metatarsal-Tarsal Joint, Open Approach */
"0SGL34Z"  =  "1"  /* Fusion of L Metatarsotars Jt with Int Fix, Perc Approach */
"0SGL35Z"  =  "1"  /* Fusion of L Metatarsotars Jt with Ext Fix, Perc Approach */
"0SGL37Z"  =  "1"  /* Fusion of L Metatarsotars Jt with Autol Sub, Perc Approach */
"0SGL3JZ"  =  "1"  /* Fusion of L Metatarsotars Jt with Synth Sub, Perc Approach */
"0SGL3KZ"  =  "1"  /* Fusion of L Metatarsotars Jt with Nonaut Sub, Perc Approach */
"0SGL3ZZ"  =  "1"  /* Fusion of Left Metatarsal-Tarsal Joint, Perc Approach */
"0SGL44Z"  =  "1"  /* Fusion L Metatarsotars Jt w Int Fix, Perc Endo */
"0SGL45Z"  =  "1"  /* Fusion L Metatarsotars Jt w Ext Fix, Perc Endo */
"0SGL47Z"  =  "1"  /* Fusion L Metatarsotars Jt w Autol Sub, Perc Endo */
"0SGL4JZ"  =  "1"  /* Fusion L Metatarsotars Jt w Synth Sub, Perc Endo */
"0SGL4KZ"  =  "1"  /* Fusion L Metatarsotars Jt w Nonaut Sub, Perc Endo */
"0SGL4ZZ"  =  "1"  /* Fusion of Left Metatarsal-Tarsal Joint, Perc Endo Approach */
"0SGM04Z"  =  "1"  /* Fusion of R Metatarsophal Jt with Int Fix, Open Approach */
"0SGM05Z"  =  "1"  /* Fusion of R Metatarsophal Jt with Ext Fix, Open Approach */
"0SGM07Z"  =  "1"  /* Fusion of R Metatarsophal Jt with Autol Sub, Open Approach */
"0SGM0JZ"  =  "1"  /* Fusion of R Metatarsophal Jt with Synth Sub, Open Approach */
"0SGM0KZ"  =  "1"  /* Fusion of R Metatarsophal Jt with Nonaut Sub, Open Approach */
"0SGM0ZZ"  =  "1"  /* Fusion of Right Metatarsal-Phalangeal Joint, Open Approach */
"0SGM34Z"  =  "1"  /* Fusion of R Metatarsophal Jt with Int Fix, Perc Approach */
"0SGM35Z"  =  "1"  /* Fusion of R Metatarsophal Jt with Ext Fix, Perc Approach */
"0SGM37Z"  =  "1"  /* Fusion of R Metatarsophal Jt with Autol Sub, Perc Approach */
"0SGM3JZ"  =  "1"  /* Fusion of R Metatarsophal Jt with Synth Sub, Perc Approach */
"0SGM3KZ"  =  "1"  /* Fusion of R Metatarsophal Jt with Nonaut Sub, Perc Approach */
"0SGM3ZZ"  =  "1"  /* Fusion of Right Metatarsal-Phalangeal Joint, Perc Approach */
"0SGM44Z"  =  "1"  /* Fusion R Metatarsophal Jt w Int Fix, Perc Endo */
"0SGM45Z"  =  "1"  /* Fusion R Metatarsophal Jt w Ext Fix, Perc Endo */
"0SGM47Z"  =  "1"  /* Fusion R Metatarsophal Jt w Autol Sub, Perc Endo */
"0SGM4JZ"  =  "1"  /* Fusion R Metatarsophal Jt w Synth Sub, Perc Endo */
"0SGM4KZ"  =  "1"  /* Fusion R Metatarsophal Jt w Nonaut Sub, Perc Endo */
"0SGM4ZZ"  =  "1"  /* Fusion of R Metatarsophal Jt, Perc Endo Approach */
"0SGN04Z"  =  "1"  /* Fusion of L Metatarsophal Jt with Int Fix, Open Approach */
"0SGN05Z"  =  "1"  /* Fusion of L Metatarsophal Jt with Ext Fix, Open Approach */
"0SGN07Z"  =  "1"  /* Fusion of L Metatarsophal Jt with Autol Sub, Open Approach */
"0SGN0JZ"  =  "1"  /* Fusion of L Metatarsophal Jt with Synth Sub, Open Approach */
"0SGN0KZ"  =  "1"  /* Fusion of L Metatarsophal Jt with Nonaut Sub, Open Approach */
"0SGN0ZZ"  =  "1"  /* Fusion of Left Metatarsal-Phalangeal Joint, Open Approach */
"0SGN34Z"  =  "1"  /* Fusion of L Metatarsophal Jt with Int Fix, Perc Approach */
"0SGN35Z"  =  "1"  /* Fusion of L Metatarsophal Jt with Ext Fix, Perc Approach */
"0SGN37Z"  =  "1"  /* Fusion of L Metatarsophal Jt with Autol Sub, Perc Approach */
"0SGN3JZ"  =  "1"  /* Fusion of L Metatarsophal Jt with Synth Sub, Perc Approach */
"0SGN3KZ"  =  "1"  /* Fusion of L Metatarsophal Jt with Nonaut Sub, Perc Approach */
"0SGN3ZZ"  =  "1"  /* Fusion of Left Metatarsal-Phalangeal Joint, Perc Approach */
"0SGN44Z"  =  "1"  /* Fusion L Metatarsophal Jt w Int Fix, Perc Endo */
"0SGN45Z"  =  "1"  /* Fusion L Metatarsophal Jt w Ext Fix, Perc Endo */
"0SGN47Z"  =  "1"  /* Fusion L Metatarsophal Jt w Autol Sub, Perc Endo */
"0SGN4JZ"  =  "1"  /* Fusion L Metatarsophal Jt w Synth Sub, Perc Endo */
"0SGN4KZ"  =  "1"  /* Fusion L Metatarsophal Jt w Nonaut Sub, Perc Endo */
"0SGN4ZZ"  =  "1"  /* Fusion of L Metatarsophal Jt, Perc Endo Approach */
"0SGP04Z"  =  "1"  /* Fusion of R Toe Phalanx Jt with Int Fix, Open Approach */
"0SGP05Z"  =  "1"  /* Fusion of R Toe Phalanx Jt with Ext Fix, Open Approach */
"0SGP07Z"  =  "1"  /* Fusion of R Toe Phalanx Jt with Autol Sub, Open Approach */
"0SGP0JZ"  =  "1"  /* Fusion of R Toe Phalanx Jt with Synth Sub, Open Approach */
"0SGP0KZ"  =  "1"  /* Fusion of R Toe Phalanx Jt with Nonaut Sub, Open Approach */
"0SGP0ZZ"  =  "1"  /* Fusion of Right Toe Phalangeal Joint, Open Approach */
"0SGP34Z"  =  "1"  /* Fusion of R Toe Phalanx Jt with Int Fix, Perc Approach */
"0SGP35Z"  =  "1"  /* Fusion of R Toe Phalanx Jt with Ext Fix, Perc Approach */
"0SGP37Z"  =  "1"  /* Fusion of R Toe Phalanx Jt with Autol Sub, Perc Approach */
"0SGP3JZ"  =  "1"  /* Fusion of R Toe Phalanx Jt with Synth Sub, Perc Approach */
"0SGP3KZ"  =  "1"  /* Fusion of R Toe Phalanx Jt with Nonaut Sub, Perc Approach */
"0SGP3ZZ"  =  "1"  /* Fusion of Right Toe Phalangeal Joint, Percutaneous Approach */
"0SGP44Z"  =  "1"  /* Fusion of R Toe Phalanx Jt with Int Fix, Perc Endo Approach */
"0SGP45Z"  =  "1"  /* Fusion of R Toe Phalanx Jt with Ext Fix, Perc Endo Approach */
"0SGP47Z"  =  "1"  /* Fusion R Toe Phalanx Jt w Autol Sub, Perc Endo */
"0SGP4JZ"  =  "1"  /* Fusion R Toe Phalanx Jt w Synth Sub, Perc Endo */
"0SGP4KZ"  =  "1"  /* Fusion R Toe Phalanx Jt w Nonaut Sub, Perc Endo */
"0SGP4ZZ"  =  "1"  /* Fusion of Right Toe Phalangeal Joint, Perc Endo Approach */
"0SGQ04Z"  =  "1"  /* Fusion of L Toe Phalanx Jt with Int Fix, Open Approach */
"0SGQ05Z"  =  "1"  /* Fusion of L Toe Phalanx Jt with Ext Fix, Open Approach */
"0SGQ07Z"  =  "1"  /* Fusion of L Toe Phalanx Jt with Autol Sub, Open Approach */
"0SGQ0JZ"  =  "1"  /* Fusion of L Toe Phalanx Jt with Synth Sub, Open Approach */
"0SGQ0KZ"  =  "1"  /* Fusion of L Toe Phalanx Jt with Nonaut Sub, Open Approach */
"0SGQ0ZZ"  =  "1"  /* Fusion of Left Toe Phalangeal Joint, Open Approach */
"0SGQ34Z"  =  "1"  /* Fusion of L Toe Phalanx Jt with Int Fix, Perc Approach */
"0SGQ35Z"  =  "1"  /* Fusion of L Toe Phalanx Jt with Ext Fix, Perc Approach */
"0SGQ37Z"  =  "1"  /* Fusion of L Toe Phalanx Jt with Autol Sub, Perc Approach */
"0SGQ3JZ"  =  "1"  /* Fusion of L Toe Phalanx Jt with Synth Sub, Perc Approach */
"0SGQ3KZ"  =  "1"  /* Fusion of L Toe Phalanx Jt with Nonaut Sub, Perc Approach */
"0SGQ3ZZ"  =  "1"  /* Fusion of Left Toe Phalangeal Joint, Percutaneous Approach */
"0SGQ44Z"  =  "1"  /* Fusion of L Toe Phalanx Jt with Int Fix, Perc Endo Approach */
"0SGQ45Z"  =  "1"  /* Fusion of L Toe Phalanx Jt with Ext Fix, Perc Endo Approach */
"0SGQ47Z"  =  "1"  /* Fusion L Toe Phalanx Jt w Autol Sub, Perc Endo */
"0SGQ4JZ"  =  "1"  /* Fusion L Toe Phalanx Jt w Synth Sub, Perc Endo */
"0SGQ4KZ"  =  "1"  /* Fusion L Toe Phalanx Jt w Nonaut Sub, Perc Endo */
"0SGQ4ZZ"  =  "1"  /* Fusion of Left Toe Phalangeal Joint, Perc Endo Approach */
"0SH004Z"  =  "1"  /* Insertion of Int Fix into Lum Jt, Open Approach */
"0SH00BZ"  =  "1"  /* Insert of Intspin Prcs Stabl Dev into Lum Jt, Open Approach */
"0SH00CZ"  =  "1"  /* Insertion of Pedcl Stabl Dev into Lum Jt, Open Approach */
"0SH00DZ"  =  "1"  /* Insertion of Facet Stabl Dev into Lum Jt, Open Approach */
"0SH034Z"  =  "1"  /* Insertion of Int Fix into Lum Jt, Perc Approach */
"0SH03BZ"  =  "1"  /* Insert of Intspin Prcs Stabl Dev into Lum Jt, Perc Approach */
"0SH03CZ"  =  "1"  /* Insertion of Pedcl Stabl Dev into Lum Jt, Perc Approach */
"0SH03DZ"  =  "1"  /* Insertion of Facet Stabl Dev into Lum Jt, Perc Approach */
"0SH044Z"  =  "1"  /* Insertion of Int Fix into Lum Jt, Perc Endo Approach */
"0SH04BZ"  =  "1"  /* Insert Intspin Prcs Stabl Dev in Lum Jt, Perc Endo */
"0SH04CZ"  =  "1"  /* Insertion of Pedcl Stabl Dev into Lum Jt, Perc Endo Approach */
"0SH04DZ"  =  "1"  /* Insertion of Facet Stabl Dev into Lum Jt, Perc Endo Approach */
"0SH304Z"  =  "1"  /* Insertion of Int Fix into Lumsac Jt, Open Approach */
"0SH30BZ"  =  "1"  /* Insert Intspin Prcs Stabl Dev in Lumsac Jt, Open */
"0SH30CZ"  =  "1"  /* Insertion of Pedcl Stabl Dev into Lumsac Jt, Open Approach */
"0SH30DZ"  =  "1"  /* Insertion of Facet Stabl Dev into Lumsac Jt, Open Approach */
"0SH334Z"  =  "1"  /* Insertion of Int Fix into Lumsac Jt, Perc Approach */
"0SH33BZ"  =  "1"  /* Insert Intspin Prcs Stabl Dev in Lumsac Jt, Perc */
"0SH33CZ"  =  "1"  /* Insertion of Pedcl Stabl Dev into Lumsac Jt, Perc Approach */
"0SH33DZ"  =  "1"  /* Insertion of Facet Stabl Dev into Lumsac Jt, Perc Approach */
"0SH344Z"  =  "1"  /* Insertion of Int Fix into Lumsac Jt, Perc Endo Approach */
"0SH34BZ"  =  "1"  /* Insert Intspin Prcs Stabl Dev in Lumsac Jt, Perc Endo */
"0SH34CZ"  =  "1"  /* Insert of Pedcl Stabl Dev into Lumsac Jt, Perc Endo Approach */
"0SH34DZ"  =  "1"  /* Insert of Facet Stabl Dev into Lumsac Jt, Perc Endo Approach */
"0SH504Z"  =  "1"  /* Insertion of Int Fix into Sacrococcygeal Jt, Open Approach */
"0SH534Z"  =  "1"  /* Insertion of Int Fix into Sacrococcygeal Jt, Perc Approach */
"0SH544Z"  =  "1"  /* Insert of Int Fix into Sacrococcygeal Jt, Perc Endo Approach */
"0SH604Z"  =  "1"  /* Insertion of Int Fix into Coccygeal Jt, Open Approach */
"0SH634Z"  =  "1"  /* Insertion of Int Fix into Coccygeal Jt, Perc Approach */
"0SH644Z"  =  "1"  /* Insertion of Int Fix into Coccygeal Jt, Perc Endo Approach */
"0SH704Z"  =  "1"  /* Insertion of Int Fix into R Sacroiliac Jt, Open Approach */
"0SH734Z"  =  "1"  /* Insertion of Int Fix into R Sacroiliac Jt, Perc Approach */
"0SH744Z"  =  "1"  /* Insert of Int Fix into R Sacroiliac Jt, Perc Endo Approach */
"0SH804Z"  =  "1"  /* Insertion of Int Fix into L Sacroiliac Jt, Open Approach */
"0SH834Z"  =  "1"  /* Insertion of Int Fix into L Sacroiliac Jt, Perc Approach */
"0SH844Z"  =  "1"  /* Insert of Int Fix into L Sacroiliac Jt, Perc Endo Approach */
"0SH904Z"  =  "1"  /* Insertion of Int Fix into R Hip Jt, Open Approach */
"0SH905Z"  =  "1"  /* Insertion of Ext Fix into R Hip Jt, Open Approach */
"0SH934Z"  =  "1"  /* Insertion of Int Fix into R Hip Jt, Perc Approach */
"0SH935Z"  =  "1"  /* Insertion of Ext Fix into R Hip Jt, Perc Approach */
"0SH944Z"  =  "1"  /* Insertion of Int Fix into R Hip Jt, Perc Endo Approach */
"0SH945Z"  =  "1"  /* Insertion of Ext Fix into R Hip Jt, Perc Endo Approach */
"0SHB04Z"  =  "1"  /* Insertion of Int Fix into L Hip Jt, Open Approach */
"0SHB05Z"  =  "1"  /* Insertion of Ext Fix into L Hip Jt, Open Approach */
"0SHB34Z"  =  "1"  /* Insertion of Int Fix into L Hip Jt, Perc Approach */
"0SHB35Z"  =  "1"  /* Insertion of Ext Fix into L Hip Jt, Perc Approach */
"0SHB44Z"  =  "1"  /* Insertion of Int Fix into L Hip Jt, Perc Endo Approach */
"0SHB45Z"  =  "1"  /* Insertion of Ext Fix into L Hip Jt, Perc Endo Approach */
"0SHC04Z"  =  "1"  /* Insertion of Int Fix into R Knee Jt, Open Approach */
"0SHC05Z"  =  "1"  /* Insertion of Ext Fix into R Knee Jt, Open Approach */
"0SHC34Z"  =  "1"  /* Insertion of Int Fix into R Knee Jt, Perc Approach */
"0SHC35Z"  =  "1"  /* Insertion of Ext Fix into R Knee Jt, Perc Approach */
"0SHC44Z"  =  "1"  /* Insertion of Int Fix into R Knee Jt, Perc Endo Approach */
"0SHC45Z"  =  "1"  /* Insertion of Ext Fix into R Knee Jt, Perc Endo Approach */
"0SHD04Z"  =  "1"  /* Insertion of Int Fix into L Knee Jt, Open Approach */
"0SHD05Z"  =  "1"  /* Insertion of Ext Fix into L Knee Jt, Open Approach */
"0SHD34Z"  =  "1"  /* Insertion of Int Fix into L Knee Jt, Perc Approach */
"0SHD35Z"  =  "1"  /* Insertion of Ext Fix into L Knee Jt, Perc Approach */
"0SHD44Z"  =  "1"  /* Insertion of Int Fix into L Knee Jt, Perc Endo Approach */
"0SHD45Z"  =  "1"  /* Insertion of Ext Fix into L Knee Jt, Perc Endo Approach */
"0SHF04Z"  =  "1"  /* Insertion of Int Fix into R Ankle Jt, Open Approach */
"0SHF05Z"  =  "1"  /* Insertion of Ext Fix into R Ankle Jt, Open Approach */
"0SHF34Z"  =  "1"  /* Insertion of Int Fix into R Ankle Jt, Perc Approach */
"0SHF35Z"  =  "1"  /* Insertion of Ext Fix into R Ankle Jt, Perc Approach */
"0SHF44Z"  =  "1"  /* Insertion of Int Fix into R Ankle Jt, Perc Endo Approach */
"0SHF45Z"  =  "1"  /* Insertion of Ext Fix into R Ankle Jt, Perc Endo Approach */
"0SHG04Z"  =  "1"  /* Insertion of Int Fix into L Ankle Jt, Open Approach */
"0SHG05Z"  =  "1"  /* Insertion of Ext Fix into L Ankle Jt, Open Approach */
"0SHG34Z"  =  "1"  /* Insertion of Int Fix into L Ankle Jt, Perc Approach */
"0SHG35Z"  =  "1"  /* Insertion of Ext Fix into L Ankle Jt, Perc Approach */
"0SHG44Z"  =  "1"  /* Insertion of Int Fix into L Ankle Jt, Perc Endo Approach */
"0SHG45Z"  =  "1"  /* Insertion of Ext Fix into L Ankle Jt, Perc Endo Approach */
"0SHH04Z"  =  "1"  /* Insertion of Int Fix into R Tarsal Jt, Open Approach */
"0SHH05Z"  =  "1"  /* Insertion of Ext Fix into R Tarsal Jt, Open Approach */
"0SHH34Z"  =  "1"  /* Insertion of Int Fix into R Tarsal Jt, Perc Approach */
"0SHH35Z"  =  "1"  /* Insertion of Ext Fix into R Tarsal Jt, Perc Approach */
"0SHH44Z"  =  "1"  /* Insertion of Int Fix into R Tarsal Jt, Perc Endo Approach */
"0SHH45Z"  =  "1"  /* Insertion of Ext Fix into R Tarsal Jt, Perc Endo Approach */
"0SHJ04Z"  =  "1"  /* Insertion of Int Fix into L Tarsal Jt, Open Approach */
"0SHJ05Z"  =  "1"  /* Insertion of Ext Fix into L Tarsal Jt, Open Approach */
"0SHJ34Z"  =  "1"  /* Insertion of Int Fix into L Tarsal Jt, Perc Approach */
"0SHJ35Z"  =  "1"  /* Insertion of Ext Fix into L Tarsal Jt, Perc Approach */
"0SHJ44Z"  =  "1"  /* Insertion of Int Fix into L Tarsal Jt, Perc Endo Approach */
"0SHJ45Z"  =  "1"  /* Insertion of Ext Fix into L Tarsal Jt, Perc Endo Approach */
"0SHK04Z"  =  "1"  /* Insertion of Int Fix into R Metatarsotars Jt, Open Approach */
"0SHK05Z"  =  "1"  /* Insertion of Ext Fix into R Metatarsotars Jt, Open Approach */
"0SHK34Z"  =  "1"  /* Insertion of Int Fix into R Metatarsotars Jt, Perc Approach */
"0SHK35Z"  =  "1"  /* Insertion of Ext Fix into R Metatarsotars Jt, Perc Approach */
"0SHK44Z"  =  "1"  /* Insert Int Fix in R Metatarsotars Jt, Perc Endo */
"0SHK45Z"  =  "1"  /* Insert Ext Fix in R Metatarsotars Jt, Perc Endo */
"0SHL04Z"  =  "1"  /* Insertion of Int Fix into L Metatarsotars Jt, Open Approach */
"0SHL05Z"  =  "1"  /* Insertion of Ext Fix into L Metatarsotars Jt, Open Approach */
"0SHL34Z"  =  "1"  /* Insertion of Int Fix into L Metatarsotars Jt, Perc Approach */
"0SHL35Z"  =  "1"  /* Insertion of Ext Fix into L Metatarsotars Jt, Perc Approach */
"0SHL44Z"  =  "1"  /* Insert Int Fix in L Metatarsotars Jt, Perc Endo */
"0SHL45Z"  =  "1"  /* Insert Ext Fix in L Metatarsotars Jt, Perc Endo */
"0SHM04Z"  =  "1"  /* Insertion of Int Fix into R Metatarsophal Jt, Open Approach */
"0SHM05Z"  =  "1"  /* Insertion of Ext Fix into R Metatarsophal Jt, Open Approach */
"0SHM34Z"  =  "1"  /* Insertion of Int Fix into R Metatarsophal Jt, Perc Approach */
"0SHM35Z"  =  "1"  /* Insertion of Ext Fix into R Metatarsophal Jt, Perc Approach */
"0SHM44Z"  =  "1"  /* Insert Int Fix in R Metatarsophal Jt, Perc Endo */
"0SHM45Z"  =  "1"  /* Insert Ext Fix in R Metatarsophal Jt, Perc Endo */
"0SHN04Z"  =  "1"  /* Insertion of Int Fix into L Metatarsophal Jt, Open Approach */
"0SHN05Z"  =  "1"  /* Insertion of Ext Fix into L Metatarsophal Jt, Open Approach */
"0SHN34Z"  =  "1"  /* Insertion of Int Fix into L Metatarsophal Jt, Perc Approach */
"0SHN35Z"  =  "1"  /* Insertion of Ext Fix into L Metatarsophal Jt, Perc Approach */
"0SHN44Z"  =  "1"  /* Insert Int Fix in L Metatarsophal Jt, Perc Endo */
"0SHN45Z"  =  "1"  /* Insert Ext Fix in L Metatarsophal Jt, Perc Endo */
"0SHP04Z"  =  "1"  /* Insertion of Int Fix into R Toe Phalanx Jt, Open Approach */
"0SHP05Z"  =  "1"  /* Insertion of Ext Fix into R Toe Phalanx Jt, Open Approach */
"0SHP34Z"  =  "1"  /* Insertion of Int Fix into R Toe Phalanx Jt, Perc Approach */
"0SHP35Z"  =  "1"  /* Insertion of Ext Fix into R Toe Phalanx Jt, Perc Approach */
"0SHP44Z"  =  "1"  /* Insert of Int Fix into R Toe Phalanx Jt, Perc Endo Approach */
"0SHP45Z"  =  "1"  /* Insert of Ext Fix into R Toe Phalanx Jt, Perc Endo Approach */
"0SHQ04Z"  =  "1"  /* Insertion of Int Fix into L Toe Phalanx Jt, Open Approach */
"0SHQ05Z"  =  "1"  /* Insertion of Ext Fix into L Toe Phalanx Jt, Open Approach */
"0SHQ34Z"  =  "1"  /* Insertion of Int Fix into L Toe Phalanx Jt, Perc Approach */
"0SHQ35Z"  =  "1"  /* Insertion of Ext Fix into L Toe Phalanx Jt, Perc Approach */
"0SHQ44Z"  =  "1"  /* Insert of Int Fix into L Toe Phalanx Jt, Perc Endo Approach */
"0SHQ45Z"  =  "1"  /* Insert of Ext Fix into L Toe Phalanx Jt, Perc Endo Approach */
"0SJ00ZZ"  =  "1"  /* Inspection of Lumbar Vertebral Joint, Open Approach */
"0SJ03ZZ"  =  "1"  /* Inspection of Lumbar Vertebral Joint, Percutaneous Approach */
"0SJ04ZZ"  =  "1"  /* Inspection of Lumbar Vertebral Joint, Perc Endo Approach */
"0SJ20ZZ"  =  "1"  /* Inspection of Lumbar Vertebral Disc, Open Approach */
"0SJ23ZZ"  =  "1"  /* Inspection of Lumbar Vertebral Disc, Percutaneous Approach */
"0SJ24ZZ"  =  "1"  /* Inspection of Lumbar Vertebral Disc, Perc Endo Approach */
"0SJ30ZZ"  =  "1"  /* Inspection of Lumbosacral Joint, Open Approach */
"0SJ33ZZ"  =  "1"  /* Inspection of Lumbosacral Joint, Percutaneous Approach */
"0SJ34ZZ"  =  "1"  /* Inspection of Lumbosacral Joint, Perc Endo Approach */
"0SJ40ZZ"  =  "1"  /* Inspection of Lumbosacral Disc, Open Approach */
"0SJ43ZZ"  =  "1"  /* Inspection of Lumbosacral Disc, Percutaneous Approach */
"0SJ44ZZ"  =  "1"  /* Inspection of Lumbosacral Disc, Perc Endo Approach */
"0SJ50ZZ"  =  "1"  /* Inspection of Sacrococcygeal Joint, Open Approach */
"0SJ53ZZ"  =  "1"  /* Inspection of Sacrococcygeal Joint, Percutaneous Approach */
"0SJ54ZZ"  =  "1"  /* Inspection of Sacrococcygeal Joint, Perc Endo Approach */
"0SJ60ZZ"  =  "1"  /* Inspection of Coccygeal Joint, Open Approach */
"0SJ63ZZ"  =  "1"  /* Inspection of Coccygeal Joint, Percutaneous Approach */
"0SJ64ZZ"  =  "1"  /* Inspection of Coccygeal Joint, Perc Endo Approach */
"0SJ70ZZ"  =  "1"  /* Inspection of Right Sacroiliac Joint, Open Approach */
"0SJ73ZZ"  =  "1"  /* Inspection of Right Sacroiliac Joint, Percutaneous Approach */
"0SJ74ZZ"  =  "1"  /* Inspection of Right Sacroiliac Joint, Perc Endo Approach */
"0SJ80ZZ"  =  "1"  /* Inspection of Left Sacroiliac Joint, Open Approach */
"0SJ83ZZ"  =  "1"  /* Inspection of Left Sacroiliac Joint, Percutaneous Approach */
"0SJ84ZZ"  =  "1"  /* Inspection of Left Sacroiliac Joint, Perc Endo Approach */
"0SJ90ZZ"  =  "1"  /* Inspection of Right Hip Joint, Open Approach */
"0SJ93ZZ"  =  "1"  /* Inspection of Right Hip Joint, Percutaneous Approach */
"0SJ94ZZ"  =  "1"  /* Inspection of Right Hip Joint, Perc Endo Approach */
"0SJB0ZZ"  =  "1"  /* Inspection of Left Hip Joint, Open Approach */
"0SJB3ZZ"  =  "1"  /* Inspection of Left Hip Joint, Percutaneous Approach */
"0SJB4ZZ"  =  "1"  /* Inspection of Left Hip Joint, Perc Endo Approach */
"0SJC0ZZ"  =  "1"  /* Inspection of Right Knee Joint, Open Approach */
"0SJC3ZZ"  =  "1"  /* Inspection of Right Knee Joint, Percutaneous Approach */
"0SJC4ZZ"  =  "1"  /* Inspection of Right Knee Joint, Perc Endo Approach */
"0SJD0ZZ"  =  "1"  /* Inspection of Left Knee Joint, Open Approach */
"0SJD3ZZ"  =  "1"  /* Inspection of Left Knee Joint, Percutaneous Approach */
"0SJD4ZZ"  =  "1"  /* Inspection of Left Knee Joint, Perc Endo Approach */
"0SJF0ZZ"  =  "1"  /* Inspection of Right Ankle Joint, Open Approach */
"0SJF3ZZ"  =  "1"  /* Inspection of Right Ankle Joint, Percutaneous Approach */
"0SJF4ZZ"  =  "1"  /* Inspection of Right Ankle Joint, Perc Endo Approach */
"0SJG0ZZ"  =  "1"  /* Inspection of Left Ankle Joint, Open Approach */
"0SJG3ZZ"  =  "1"  /* Inspection of Left Ankle Joint, Percutaneous Approach */
"0SJG4ZZ"  =  "1"  /* Inspection of Left Ankle Joint, Perc Endo Approach */
"0SJH0ZZ"  =  "1"  /* Inspection of Right Tarsal Joint, Open Approach */
"0SJH3ZZ"  =  "1"  /* Inspection of Right Tarsal Joint, Percutaneous Approach */
"0SJH4ZZ"  =  "1"  /* Inspection of Right Tarsal Joint, Perc Endo Approach */
"0SJJ0ZZ"  =  "1"  /* Inspection of Left Tarsal Joint, Open Approach */
"0SJJ3ZZ"  =  "1"  /* Inspection of Left Tarsal Joint, Percutaneous Approach */
"0SJJ4ZZ"  =  "1"  /* Inspection of Left Tarsal Joint, Perc Endo Approach */
"0SJK0ZZ"  =  "1"  /* Inspection of Right Metatarsal-Tarsal Joint, Open Approach */
"0SJK3ZZ"  =  "1"  /* Inspection of Right Metatarsal-Tarsal Joint, Perc Approach */
"0SJK4ZZ"  =  "1"  /* Inspection of R Metatarsotars Jt, Perc Endo Approach */
"0SJL0ZZ"  =  "1"  /* Inspection of Left Metatarsal-Tarsal Joint, Open Approach */
"0SJL3ZZ"  =  "1"  /* Inspection of Left Metatarsal-Tarsal Joint, Perc Approach */
"0SJL4ZZ"  =  "1"  /* Inspection of L Metatarsotars Jt, Perc Endo Approach */
"0SJM0ZZ"  =  "1"  /* Inspection of R Metatarsophal Jt, Open Approach */
"0SJM3ZZ"  =  "1"  /* Inspection of R Metatarsophal Jt, Perc Approach */
"0SJM4ZZ"  =  "1"  /* Inspection of R Metatarsophal Jt, Perc Endo Approach */
"0SJN0ZZ"  =  "1"  /* Inspection of L Metatarsophal Jt, Open Approach */
"0SJN3ZZ"  =  "1"  /* Inspection of L Metatarsophal Jt, Perc Approach */
"0SJN4ZZ"  =  "1"  /* Inspection of L Metatarsophal Jt, Perc Endo Approach */
"0SJP0ZZ"  =  "1"  /* Inspection of Right Toe Phalangeal Joint, Open Approach */
"0SJP3ZZ"  =  "1"  /* Inspection of Right Toe Phalangeal Joint, Perc Approach */
"0SJP4ZZ"  =  "1"  /* Inspection of Right Toe Phalangeal Joint, Perc Endo Approach */
"0SJQ0ZZ"  =  "1"  /* Inspection of Left Toe Phalangeal Joint, Open Approach */
"0SJQ3ZZ"  =  "1"  /* Inspection of Left Toe Phalangeal Joint, Perc Approach */
"0SJQ4ZZ"  =  "1"  /* Inspection of Left Toe Phalangeal Joint, Perc Endo Approach */
"0SN00ZZ"  =  "1"  /* Release Lumbar Vertebral Joint, Open Approach */
"0SN03ZZ"  =  "1"  /* Release Lumbar Vertebral Joint, Percutaneous Approach */
"0SN04ZZ"  =  "1"  /* Release Lumbar Vertebral Joint, Perc Endo Approach */
"0SN20ZZ"  =  "1"  /* Release Lumbar Vertebral Disc, Open Approach */
"0SN23ZZ"  =  "1"  /* Release Lumbar Vertebral Disc, Percutaneous Approach */
"0SN24ZZ"  =  "1"  /* Release Lumbar Vertebral Disc, Perc Endo Approach */
"0SN30ZZ"  =  "1"  /* Release Lumbosacral Joint, Open Approach */
"0SN33ZZ"  =  "1"  /* Release Lumbosacral Joint, Percutaneous Approach */
"0SN34ZZ"  =  "1"  /* Release Lumbosacral Joint, Percutaneous Endoscopic Approach */
"0SN40ZZ"  =  "1"  /* Release Lumbosacral Disc, Open Approach */
"0SN43ZZ"  =  "1"  /* Release Lumbosacral Disc, Percutaneous Approach */
"0SN44ZZ"  =  "1"  /* Release Lumbosacral Disc, Percutaneous Endoscopic Approach */
"0SN50ZZ"  =  "1"  /* Release Sacrococcygeal Joint, Open Approach */
"0SN53ZZ"  =  "1"  /* Release Sacrococcygeal Joint, Percutaneous Approach */
"0SN54ZZ"  =  "1"  /* Release Sacrococcygeal Joint, Perc Endo Approach */
"0SN60ZZ"  =  "1"  /* Release Coccygeal Joint, Open Approach */
"0SN63ZZ"  =  "1"  /* Release Coccygeal Joint, Percutaneous Approach */
"0SN64ZZ"  =  "1"  /* Release Coccygeal Joint, Percutaneous Endoscopic Approach */
"0SN70ZZ"  =  "1"  /* Release Right Sacroiliac Joint, Open Approach */
"0SN73ZZ"  =  "1"  /* Release Right Sacroiliac Joint, Percutaneous Approach */
"0SN74ZZ"  =  "1"  /* Release Right Sacroiliac Joint, Perc Endo Approach */
"0SN80ZZ"  =  "1"  /* Release Left Sacroiliac Joint, Open Approach */
"0SN83ZZ"  =  "1"  /* Release Left Sacroiliac Joint, Percutaneous Approach */
"0SN84ZZ"  =  "1"  /* Release Left Sacroiliac Joint, Perc Endo Approach */
"0SN90ZZ"  =  "1"  /* Release Right Hip Joint, Open Approach */
"0SN93ZZ"  =  "1"  /* Release Right Hip Joint, Percutaneous Approach */
"0SN94ZZ"  =  "1"  /* Release Right Hip Joint, Percutaneous Endoscopic Approach */
"0SNB0ZZ"  =  "1"  /* Release Left Hip Joint, Open Approach */
"0SNB3ZZ"  =  "1"  /* Release Left Hip Joint, Percutaneous Approach */
"0SNB4ZZ"  =  "1"  /* Release Left Hip Joint, Percutaneous Endoscopic Approach */
"0SNC0ZZ"  =  "1"  /* Release Right Knee Joint, Open Approach */
"0SNC3ZZ"  =  "1"  /* Release Right Knee Joint, Percutaneous Approach */
"0SNC4ZZ"  =  "1"  /* Release Right Knee Joint, Percutaneous Endoscopic Approach */
"0SND0ZZ"  =  "1"  /* Release Left Knee Joint, Open Approach */
"0SND3ZZ"  =  "1"  /* Release Left Knee Joint, Percutaneous Approach */
"0SND4ZZ"  =  "1"  /* Release Left Knee Joint, Percutaneous Endoscopic Approach */
"0SNF0ZZ"  =  "1"  /* Release Right Ankle Joint, Open Approach */
"0SNF3ZZ"  =  "1"  /* Release Right Ankle Joint, Percutaneous Approach */
"0SNF4ZZ"  =  "1"  /* Release Right Ankle Joint, Percutaneous Endoscopic Approach */
"0SNG0ZZ"  =  "1"  /* Release Left Ankle Joint, Open Approach */
"0SNG3ZZ"  =  "1"  /* Release Left Ankle Joint, Percutaneous Approach */
"0SNG4ZZ"  =  "1"  /* Release Left Ankle Joint, Percutaneous Endoscopic Approach */
"0SNH0ZZ"  =  "1"  /* Release Right Tarsal Joint, Open Approach */
"0SNH3ZZ"  =  "1"  /* Release Right Tarsal Joint, Percutaneous Approach */
"0SNH4ZZ"  =  "1"  /* Release Right Tarsal Joint, Percutaneous Endoscopic Approach */
"0SNJ0ZZ"  =  "1"  /* Release Left Tarsal Joint, Open Approach */
"0SNJ3ZZ"  =  "1"  /* Release Left Tarsal Joint, Percutaneous Approach */
"0SNJ4ZZ"  =  "1"  /* Release Left Tarsal Joint, Percutaneous Endoscopic Approach */
"0SNK0ZZ"  =  "1"  /* Release Right Metatarsal-Tarsal Joint, Open Approach */
"0SNK3ZZ"  =  "1"  /* Release Right Metatarsal-Tarsal Joint, Percutaneous Approach */
"0SNK4ZZ"  =  "1"  /* Release Right Metatarsal-Tarsal Joint, Perc Endo Approach */
"0SNL0ZZ"  =  "1"  /* Release Left Metatarsal-Tarsal Joint, Open Approach */
"0SNL3ZZ"  =  "1"  /* Release Left Metatarsal-Tarsal Joint, Percutaneous Approach */
"0SNL4ZZ"  =  "1"  /* Release Left Metatarsal-Tarsal Joint, Perc Endo Approach */
"0SNM0ZZ"  =  "1"  /* Release Right Metatarsal-Phalangeal Joint, Open Approach */
"0SNM3ZZ"  =  "1"  /* Release Right Metatarsal-Phalangeal Joint, Perc Approach */
"0SNM4ZZ"  =  "1"  /* Release R Metatarsophal Jt, Perc Endo Approach */
"0SNN0ZZ"  =  "1"  /* Release Left Metatarsal-Phalangeal Joint, Open Approach */
"0SNN3ZZ"  =  "1"  /* Release Left Metatarsal-Phalangeal Joint, Perc Approach */
"0SNN4ZZ"  =  "1"  /* Release Left Metatarsal-Phalangeal Joint, Perc Endo Approach */
"0SNP0ZZ"  =  "1"  /* Release Right Toe Phalangeal Joint, Open Approach */
"0SNP3ZZ"  =  "1"  /* Release Right Toe Phalangeal Joint, Percutaneous Approach */
"0SNP4ZZ"  =  "1"  /* Release Right Toe Phalangeal Joint, Perc Endo Approach */
"0SNQ0ZZ"  =  "1"  /* Release Left Toe Phalangeal Joint, Open Approach */
"0SNQ3ZZ"  =  "1"  /* Release Left Toe Phalangeal Joint, Percutaneous Approach */
"0SNQ4ZZ"  =  "1"  /* Release Left Toe Phalangeal Joint, Perc Endo Approach */
"0SP000Z"  =  "1"  /* Removal of Drainage Device from Lum Jt, Open Approach */
"0SP003Z"  =  "1"  /* Removal of Infusion Device from Lum Jt, Open Approach */
"0SP004Z"  =  "1"  /* Removal of Int Fix from Lum Jt, Open Approach */
"0SP007Z"  =  "1"  /* Removal of Autol Sub from Lum Jt, Open Approach */
"0SP00AZ"  =  "1"  /* Removal of Intbd Fus Dev from Lum Jt, Open Approach */
"0SP00JZ"  =  "1"  /* Removal of Synthetic Substitute from Lum Jt, Open Approach */
"0SP00KZ"  =  "1"  /* Removal of Nonaut Sub from Lum Jt, Open Approach */
"0SP030Z"  =  "1"  /* Removal of Drainage Device from Lum Jt, Perc Approach */
"0SP033Z"  =  "1"  /* Removal of Infusion Device from Lum Jt, Perc Approach */
"0SP034Z"  =  "1"  /* Removal of Int Fix from Lum Jt, Perc Approach */
"0SP037Z"  =  "1"  /* Removal of Autol Sub from Lum Jt, Perc Approach */
"0SP03AZ"  =  "1"  /* Removal of Intbd Fus Dev from Lum Jt, Perc Approach */
"0SP03JZ"  =  "1"  /* Removal of Synthetic Substitute from Lum Jt, Perc Approach */
"0SP03KZ"  =  "1"  /* Removal of Nonaut Sub from Lum Jt, Perc Approach */
"0SP040Z"  =  "1"  /* Removal of Drainage Device from Lum Jt, Perc Endo Approach */
"0SP043Z"  =  "1"  /* Removal of Infusion Device from Lum Jt, Perc Endo Approach */
"0SP044Z"  =  "1"  /* Removal of Int Fix from Lum Jt, Perc Endo Approach */
"0SP047Z"  =  "1"  /* Removal of Autol Sub from Lum Jt, Perc Endo Approach */
"0SP04AZ"  =  "1"  /* Removal of Intbd Fus Dev from Lum Jt, Perc Endo Approach */
"0SP04JZ"  =  "1"  /* Removal of Synth Sub from Lum Jt, Perc Endo Approach */
"0SP04KZ"  =  "1"  /* Removal of Nonaut Sub from Lum Jt, Perc Endo Approach */
"0SP200Z"  =  "1"  /* Removal of Drainage Device from Lum Disc, Open Approach */
"0SP203Z"  =  "1"  /* Removal of Infusion Device from Lum Disc, Open Approach */
"0SP207Z"  =  "1"  /* Removal of Autol Sub from Lum Disc, Open Approach */
"0SP20JZ"  =  "1"  /* Removal of Synthetic Substitute from Lum Disc, Open Approach */
"0SP20KZ"  =  "1"  /* Removal of Nonaut Sub from Lum Disc, Open Approach */
"0SP230Z"  =  "1"  /* Removal of Drainage Device from Lum Disc, Perc Approach */
"0SP233Z"  =  "1"  /* Removal of Infusion Device from Lum Disc, Perc Approach */
"0SP237Z"  =  "1"  /* Removal of Autol Sub from Lum Disc, Perc Approach */
"0SP23JZ"  =  "1"  /* Removal of Synthetic Substitute from Lum Disc, Perc Approach */
"0SP23KZ"  =  "1"  /* Removal of Nonaut Sub from Lum Disc, Perc Approach */
"0SP240Z"  =  "1"  /* Removal of Drainage Device from Lum Disc, Perc Endo Approach */
"0SP243Z"  =  "1"  /* Removal of Infusion Device from Lum Disc, Perc Endo Approach */
"0SP247Z"  =  "1"  /* Removal of Autol Sub from Lum Disc, Perc Endo Approach */
"0SP24JZ"  =  "1"  /* Removal of Synth Sub from Lum Disc, Perc Endo Approach */
"0SP24KZ"  =  "1"  /* Removal of Nonaut Sub from Lum Disc, Perc Endo Approach */
"0SP300Z"  =  "1"  /* Removal of Drainage Device from Lumsac Jt, Open Approach */
"0SP303Z"  =  "1"  /* Removal of Infusion Device from Lumsac Jt, Open Approach */
"0SP304Z"  =  "1"  /* Removal of Int Fix from Lumsac Jt, Open Approach */
"0SP307Z"  =  "1"  /* Removal of Autol Sub from Lumsac Jt, Open Approach */
"0SP30AZ"  =  "1"  /* Removal of Intbd Fus Dev from Lumsac Jt, Open Approach */
"0SP30JZ"  =  "1"  /* Removal of Synth Sub from Lumsac Jt, Open Approach */
"0SP30KZ"  =  "1"  /* Removal of Nonaut Sub from Lumsac Jt, Open Approach */
"0SP330Z"  =  "1"  /* Removal of Drainage Device from Lumsac Jt, Perc Approach */
"0SP333Z"  =  "1"  /* Removal of Infusion Device from Lumsac Jt, Perc Approach */
"0SP334Z"  =  "1"  /* Removal of Int Fix from Lumsac Jt, Perc Approach */
"0SP337Z"  =  "1"  /* Removal of Autol Sub from Lumsac Jt, Perc Approach */
"0SP33AZ"  =  "1"  /* Removal of Intbd Fus Dev from Lumsac Jt, Perc Approach */
"0SP33JZ"  =  "1"  /* Removal of Synth Sub from Lumsac Jt, Perc Approach */
"0SP33KZ"  =  "1"  /* Removal of Nonaut Sub from Lumsac Jt, Perc Approach */
"0SP340Z"  =  "1"  /* Removal of Drain Dev from Lumsac Jt, Perc Endo Approach */
"0SP343Z"  =  "1"  /* Removal of Infusion Dev from Lumsac Jt, Perc Endo Approach */
"0SP344Z"  =  "1"  /* Removal of Int Fix from Lumsac Jt, Perc Endo Approach */
"0SP347Z"  =  "1"  /* Removal of Autol Sub from Lumsac Jt, Perc Endo Approach */
"0SP34AZ"  =  "1"  /* Removal of Intbd Fus Dev from Lumsac Jt, Perc Endo Approach */
"0SP34JZ"  =  "1"  /* Removal of Synth Sub from Lumsac Jt, Perc Endo Approach */
"0SP34KZ"  =  "1"  /* Removal of Nonaut Sub from Lumsac Jt, Perc Endo Approach */
"0SP400Z"  =  "1"  /* Removal of Drainage Device from Lumsac Disc, Open Approach */
"0SP403Z"  =  "1"  /* Removal of Infusion Device from Lumsac Disc, Open Approach */
"0SP407Z"  =  "1"  /* Removal of Autol Sub from Lumsac Disc, Open Approach */
"0SP40JZ"  =  "1"  /* Removal of Synth Sub from Lumsac Disc, Open Approach */
"0SP40KZ"  =  "1"  /* Removal of Nonaut Sub from Lumsac Disc, Open Approach */
"0SP430Z"  =  "1"  /* Removal of Drainage Device from Lumsac Disc, Perc Approach */
"0SP433Z"  =  "1"  /* Removal of Infusion Device from Lumsac Disc, Perc Approach */
"0SP437Z"  =  "1"  /* Removal of Autol Sub from Lumsac Disc, Perc Approach */
"0SP43JZ"  =  "1"  /* Removal of Synth Sub from Lumsac Disc, Perc Approach */
"0SP43KZ"  =  "1"  /* Removal of Nonaut Sub from Lumsac Disc, Perc Approach */
"0SP440Z"  =  "1"  /* Removal of Drain Dev from Lumsac Disc, Perc Endo Approach */
"0SP443Z"  =  "1"  /* Removal of Infusion Dev from Lumsac Disc, Perc Endo Approach */
"0SP447Z"  =  "1"  /* Removal of Autol Sub from Lumsac Disc, Perc Endo Approach */
"0SP44JZ"  =  "1"  /* Removal of Synth Sub from Lumsac Disc, Perc Endo Approach */
"0SP44KZ"  =  "1"  /* Removal of Nonaut Sub from Lumsac Disc, Perc Endo Approach */
"0SP500Z"  =  "1"  /* Removal of Drain Dev from Sacrococcygeal Jt, Open Approach */
"0SP503Z"  =  "1"  /* Remove of Infusion Dev from Sacrococcygeal Jt, Open Approach */
"0SP504Z"  =  "1"  /* Removal of Int Fix from Sacrococcygeal Jt, Open Approach */
"0SP507Z"  =  "1"  /* Removal of Autol Sub from Sacrococcygeal Jt, Open Approach */
"0SP50JZ"  =  "1"  /* Removal of Synth Sub from Sacrococcygeal Jt, Open Approach */
"0SP50KZ"  =  "1"  /* Removal of Nonaut Sub from Sacrococcygeal Jt, Open Approach */
"0SP530Z"  =  "1"  /* Removal of Drain Dev from Sacrococcygeal Jt, Perc Approach */
"0SP533Z"  =  "1"  /* Remove of Infusion Dev from Sacrococcygeal Jt, Perc Approach */
"0SP534Z"  =  "1"  /* Removal of Int Fix from Sacrococcygeal Jt, Perc Approach */
"0SP537Z"  =  "1"  /* Removal of Autol Sub from Sacrococcygeal Jt, Perc Approach */
"0SP53JZ"  =  "1"  /* Removal of Synth Sub from Sacrococcygeal Jt, Perc Approach */
"0SP53KZ"  =  "1"  /* Removal of Nonaut Sub from Sacrococcygeal Jt, Perc Approach */
"0SP540Z"  =  "1"  /* Remove Drain Dev from Sacrococcygeal Jt, Perc Endo */
"0SP543Z"  =  "1"  /* Remove Infusion Dev from Sacrococcygeal Jt, Perc Endo */
"0SP544Z"  =  "1"  /* Remove of Int Fix from Sacrococcygeal Jt, Perc Endo Approach */
"0SP547Z"  =  "1"  /* Remove Autol Sub from Sacrococcygeal Jt, Perc Endo */
"0SP54JZ"  =  "1"  /* Remove Synth Sub from Sacrococcygeal Jt, Perc Endo */
"0SP54KZ"  =  "1"  /* Remove Nonaut Sub from Sacrococcygeal Jt, Perc Endo */
"0SP600Z"  =  "1"  /* Removal of Drainage Device from Coccygeal Jt, Open Approach */
"0SP603Z"  =  "1"  /* Removal of Infusion Device from Coccygeal Jt, Open Approach */
"0SP604Z"  =  "1"  /* Removal of Int Fix from Coccygeal Jt, Open Approach */
"0SP607Z"  =  "1"  /* Removal of Autol Sub from Coccygeal Jt, Open Approach */
"0SP60JZ"  =  "1"  /* Removal of Synth Sub from Coccygeal Jt, Open Approach */
"0SP60KZ"  =  "1"  /* Removal of Nonaut Sub from Coccygeal Jt, Open Approach */
"0SP630Z"  =  "1"  /* Removal of Drainage Device from Coccygeal Jt, Perc Approach */
"0SP633Z"  =  "1"  /* Removal of Infusion Device from Coccygeal Jt, Perc Approach */
"0SP634Z"  =  "1"  /* Removal of Int Fix from Coccygeal Jt, Perc Approach */
"0SP637Z"  =  "1"  /* Removal of Autol Sub from Coccygeal Jt, Perc Approach */
"0SP63JZ"  =  "1"  /* Removal of Synth Sub from Coccygeal Jt, Perc Approach */
"0SP63KZ"  =  "1"  /* Removal of Nonaut Sub from Coccygeal Jt, Perc Approach */
"0SP640Z"  =  "1"  /* Removal of Drain Dev from Coccygeal Jt, Perc Endo Approach */
"0SP643Z"  =  "1"  /* Remove of Infusion Dev from Coccygeal Jt, Perc Endo Approach */
"0SP644Z"  =  "1"  /* Removal of Int Fix from Coccygeal Jt, Perc Endo Approach */
"0SP647Z"  =  "1"  /* Removal of Autol Sub from Coccygeal Jt, Perc Endo Approach */
"0SP64JZ"  =  "1"  /* Removal of Synth Sub from Coccygeal Jt, Perc Endo Approach */
"0SP64KZ"  =  "1"  /* Removal of Nonaut Sub from Coccygeal Jt, Perc Endo Approach */
"0SP700Z"  =  "1"  /* Removal of Drain Dev from R Sacroiliac Jt, Open Approach */
"0SP703Z"  =  "1"  /* Removal of Infusion Dev from R Sacroiliac Jt, Open Approach */
"0SP704Z"  =  "1"  /* Removal of Int Fix from R Sacroiliac Jt, Open Approach */
"0SP707Z"  =  "1"  /* Removal of Autol Sub from R Sacroiliac Jt, Open Approach */
"0SP70JZ"  =  "1"  /* Removal of Synth Sub from R Sacroiliac Jt, Open Approach */
"0SP70KZ"  =  "1"  /* Removal of Nonaut Sub from R Sacroiliac Jt, Open Approach */
"0SP730Z"  =  "1"  /* Removal of Drain Dev from R Sacroiliac Jt, Perc Approach */
"0SP733Z"  =  "1"  /* Removal of Infusion Dev from R Sacroiliac Jt, Perc Approach */
"0SP734Z"  =  "1"  /* Removal of Int Fix from R Sacroiliac Jt, Perc Approach */
"0SP737Z"  =  "1"  /* Removal of Autol Sub from R Sacroiliac Jt, Perc Approach */
"0SP73JZ"  =  "1"  /* Removal of Synth Sub from R Sacroiliac Jt, Perc Approach */
"0SP73KZ"  =  "1"  /* Removal of Nonaut Sub from R Sacroiliac Jt, Perc Approach */
"0SP740Z"  =  "1"  /* Remove of Drain Dev from R Sacroiliac Jt, Perc Endo Approach */
"0SP743Z"  =  "1"  /* Remove Infusion Dev from R Sacroiliac Jt, Perc Endo */
"0SP744Z"  =  "1"  /* Removal of Int Fix from R Sacroiliac Jt, Perc Endo Approach */
"0SP747Z"  =  "1"  /* Remove of Autol Sub from R Sacroiliac Jt, Perc Endo Approach */
"0SP74JZ"  =  "1"  /* Remove of Synth Sub from R Sacroiliac Jt, Perc Endo Approach */
"0SP74KZ"  =  "1"  /* Remove Nonaut Sub from R Sacroiliac Jt, Perc Endo */
"0SP800Z"  =  "1"  /* Removal of Drain Dev from L Sacroiliac Jt, Open Approach */
"0SP803Z"  =  "1"  /* Removal of Infusion Dev from L Sacroiliac Jt, Open Approach */
"0SP804Z"  =  "1"  /* Removal of Int Fix from L Sacroiliac Jt, Open Approach */
"0SP807Z"  =  "1"  /* Removal of Autol Sub from L Sacroiliac Jt, Open Approach */
"0SP80JZ"  =  "1"  /* Removal of Synth Sub from L Sacroiliac Jt, Open Approach */
"0SP80KZ"  =  "1"  /* Removal of Nonaut Sub from L Sacroiliac Jt, Open Approach */
"0SP830Z"  =  "1"  /* Removal of Drain Dev from L Sacroiliac Jt, Perc Approach */
"0SP833Z"  =  "1"  /* Removal of Infusion Dev from L Sacroiliac Jt, Perc Approach */
"0SP834Z"  =  "1"  /* Removal of Int Fix from L Sacroiliac Jt, Perc Approach */
"0SP837Z"  =  "1"  /* Removal of Autol Sub from L Sacroiliac Jt, Perc Approach */
"0SP83JZ"  =  "1"  /* Removal of Synth Sub from L Sacroiliac Jt, Perc Approach */
"0SP83KZ"  =  "1"  /* Removal of Nonaut Sub from L Sacroiliac Jt, Perc Approach */
"0SP840Z"  =  "1"  /* Remove of Drain Dev from L Sacroiliac Jt, Perc Endo Approach */
"0SP843Z"  =  "1"  /* Remove Infusion Dev from L Sacroiliac Jt, Perc Endo */
"0SP844Z"  =  "1"  /* Removal of Int Fix from L Sacroiliac Jt, Perc Endo Approach */
"0SP847Z"  =  "1"  /* Remove of Autol Sub from L Sacroiliac Jt, Perc Endo Approach */
"0SP84JZ"  =  "1"  /* Remove of Synth Sub from L Sacroiliac Jt, Perc Endo Approach */
"0SP84KZ"  =  "1"  /* Remove Nonaut Sub from L Sacroiliac Jt, Perc Endo */
"0SP900Z"  =  "1"  /* Removal of Drainage Device from R Hip Jt, Open Approach */
"0SP903Z"  =  "1"  /* Removal of Infusion Device from R Hip Jt, Open Approach */
"0SP904Z"  =  "1"  /* Removal of Int Fix from R Hip Jt, Open Approach */
"0SP905Z"  =  "1"  /* Removal of Ext Fix from R Hip Jt, Open Approach */
"0SP907Z"  =  "1"  /* Removal of Autol Sub from R Hip Jt, Open Approach */
"0SP909Z"  =  "1"  /* Removal of Liner from Right Hip Joint, Open Approach */
"0SP90BZ"  =  "1"  /* Removal of Resurfacing Device from R Hip Jt, Open Approach */
"0SP90JZ"  =  "1"  /* Removal of Synthetic Substitute from R Hip Jt, Open Approach */
"0SP90KZ"  =  "1"  /* Removal of Nonaut Sub from R Hip Jt, Open Approach */
"0SP930Z"  =  "1"  /* Removal of Drainage Device from R Hip Jt, Perc Approach */
"0SP933Z"  =  "1"  /* Removal of Infusion Device from R Hip Jt, Perc Approach */
"0SP934Z"  =  "1"  /* Removal of Int Fix from R Hip Jt, Perc Approach */
"0SP935Z"  =  "1"  /* Removal of Ext Fix from R Hip Jt, Perc Approach */
"0SP937Z"  =  "1"  /* Removal of Autol Sub from R Hip Jt, Perc Approach */
"0SP93JZ"  =  "1"  /* Removal of Synthetic Substitute from R Hip Jt, Perc Approach */
"0SP93KZ"  =  "1"  /* Removal of Nonaut Sub from R Hip Jt, Perc Approach */
"0SP940Z"  =  "1"  /* Removal of Drainage Device from R Hip Jt, Perc Endo Approach */
"0SP943Z"  =  "1"  /* Removal of Infusion Device from R Hip Jt, Perc Endo Approach */
"0SP944Z"  =  "1"  /* Removal of Int Fix from R Hip Jt, Perc Endo Approach */
"0SP945Z"  =  "1"  /* Removal of Ext Fix from R Hip Jt, Perc Endo Approach */
"0SP947Z"  =  "1"  /* Removal of Autol Sub from R Hip Jt, Perc Endo Approach */
"0SP94JZ"  =  "1"  /* Removal of Synth Sub from R Hip Jt, Perc Endo Approach */
"0SP94KZ"  =  "1"  /* Removal of Nonaut Sub from R Hip Jt, Perc Endo Approach */
"0SPB00Z"  =  "1"  /* Removal of Drainage Device from L Hip Jt, Open Approach */
"0SPB03Z"  =  "1"  /* Removal of Infusion Device from L Hip Jt, Open Approach */
"0SPB04Z"  =  "1"  /* Removal of Int Fix from L Hip Jt, Open Approach */
"0SPB05Z"  =  "1"  /* Removal of Ext Fix from L Hip Jt, Open Approach */
"0SPB07Z"  =  "1"  /* Removal of Autol Sub from L Hip Jt, Open Approach */
"0SPB09Z"  =  "1"  /* Removal of Liner from Left Hip Joint, Open Approach */
"0SPB0BZ"  =  "1"  /* Removal of Resurfacing Device from L Hip Jt, Open Approach */
"0SPB0JZ"  =  "1"  /* Removal of Synthetic Substitute from L Hip Jt, Open Approach */
"0SPB0KZ"  =  "1"  /* Removal of Nonaut Sub from L Hip Jt, Open Approach */
"0SPB30Z"  =  "1"  /* Removal of Drainage Device from L Hip Jt, Perc Approach */
"0SPB33Z"  =  "1"  /* Removal of Infusion Device from L Hip Jt, Perc Approach */
"0SPB34Z"  =  "1"  /* Removal of Int Fix from L Hip Jt, Perc Approach */
"0SPB35Z"  =  "1"  /* Removal of Ext Fix from L Hip Jt, Perc Approach */
"0SPB37Z"  =  "1"  /* Removal of Autol Sub from L Hip Jt, Perc Approach */
"0SPB3JZ"  =  "1"  /* Removal of Synthetic Substitute from L Hip Jt, Perc Approach */
"0SPB3KZ"  =  "1"  /* Removal of Nonaut Sub from L Hip Jt, Perc Approach */
"0SPB40Z"  =  "1"  /* Removal of Drainage Device from L Hip Jt, Perc Endo Approach */
"0SPB43Z"  =  "1"  /* Removal of Infusion Device from L Hip Jt, Perc Endo Approach */
"0SPB44Z"  =  "1"  /* Removal of Int Fix from L Hip Jt, Perc Endo Approach */
"0SPB45Z"  =  "1"  /* Removal of Ext Fix from L Hip Jt, Perc Endo Approach */
"0SPB47Z"  =  "1"  /* Removal of Autol Sub from L Hip Jt, Perc Endo Approach */
"0SPB4JZ"  =  "1"  /* Removal of Synth Sub from L Hip Jt, Perc Endo Approach */
"0SPB4KZ"  =  "1"  /* Removal of Nonaut Sub from L Hip Jt, Perc Endo Approach */
"0SPC00Z"  =  "1"  /* Removal of Drainage Device from R Knee Jt, Open Approach */
"0SPC03Z"  =  "1"  /* Removal of Infusion Device from R Knee Jt, Open Approach */
"0SPC04Z"  =  "1"  /* Removal of Int Fix from R Knee Jt, Open Approach */
"0SPC05Z"  =  "1"  /* Removal of Ext Fix from R Knee Jt, Open Approach */
"0SPC07Z"  =  "1"  /* Removal of Autol Sub from R Knee Jt, Open Approach */
"0SPC09Z"  =  "1"  /* Removal of Liner from Right Knee Joint, Open Approach */
"0SPC0JZ"  =  "1"  /* Removal of Synth Sub from R Knee Jt, Open Approach */
"0SPC0KZ"  =  "1"  /* Removal of Nonaut Sub from R Knee Jt, Open Approach */
"0SPC30Z"  =  "1"  /* Removal of Drainage Device from R Knee Jt, Perc Approach */
"0SPC33Z"  =  "1"  /* Removal of Infusion Device from R Knee Jt, Perc Approach */
"0SPC34Z"  =  "1"  /* Removal of Int Fix from R Knee Jt, Perc Approach */
"0SPC35Z"  =  "1"  /* Removal of Ext Fix from R Knee Jt, Perc Approach */
"0SPC37Z"  =  "1"  /* Removal of Autol Sub from R Knee Jt, Perc Approach */
"0SPC3JZ"  =  "1"  /* Removal of Synth Sub from R Knee Jt, Perc Approach */
"0SPC3KZ"  =  "1"  /* Removal of Nonaut Sub from R Knee Jt, Perc Approach */
"0SPC40Z"  =  "1"  /* Removal of Drain Dev from R Knee Jt, Perc Endo Approach */
"0SPC43Z"  =  "1"  /* Removal of Infusion Dev from R Knee Jt, Perc Endo Approach */
"0SPC44Z"  =  "1"  /* Removal of Int Fix from R Knee Jt, Perc Endo Approach */
"0SPC45Z"  =  "1"  /* Removal of Ext Fix from R Knee Jt, Perc Endo Approach */
"0SPC47Z"  =  "1"  /* Removal of Autol Sub from R Knee Jt, Perc Endo Approach */
"0SPC4JZ"  =  "1"  /* Removal of Synth Sub from R Knee Jt, Perc Endo Approach */
"0SPC4KZ"  =  "1"  /* Removal of Nonaut Sub from R Knee Jt, Perc Endo Approach */
"0SPD00Z"  =  "1"  /* Removal of Drainage Device from L Knee Jt, Open Approach */
"0SPD03Z"  =  "1"  /* Removal of Infusion Device from L Knee Jt, Open Approach */
"0SPD04Z"  =  "1"  /* Removal of Int Fix from L Knee Jt, Open Approach */
"0SPD05Z"  =  "1"  /* Removal of Ext Fix from L Knee Jt, Open Approach */
"0SPD07Z"  =  "1"  /* Removal of Autol Sub from L Knee Jt, Open Approach */
"0SPD09Z"  =  "1"  /* Removal of Liner from Left Knee Joint, Open Approach */
"0SPD0JZ"  =  "1"  /* Removal of Synth Sub from L Knee Jt, Open Approach */
"0SPD0KZ"  =  "1"  /* Removal of Nonaut Sub from L Knee Jt, Open Approach */
"0SPD30Z"  =  "1"  /* Removal of Drainage Device from L Knee Jt, Perc Approach */
"0SPD33Z"  =  "1"  /* Removal of Infusion Device from L Knee Jt, Perc Approach */
"0SPD34Z"  =  "1"  /* Removal of Int Fix from L Knee Jt, Perc Approach */
"0SPD35Z"  =  "1"  /* Removal of Ext Fix from L Knee Jt, Perc Approach */
"0SPD37Z"  =  "1"  /* Removal of Autol Sub from L Knee Jt, Perc Approach */
"0SPD3JZ"  =  "1"  /* Removal of Synth Sub from L Knee Jt, Perc Approach */
"0SPD3KZ"  =  "1"  /* Removal of Nonaut Sub from L Knee Jt, Perc Approach */
"0SPD40Z"  =  "1"  /* Removal of Drain Dev from L Knee Jt, Perc Endo Approach */
"0SPD43Z"  =  "1"  /* Removal of Infusion Dev from L Knee Jt, Perc Endo Approach */
"0SPD44Z"  =  "1"  /* Removal of Int Fix from L Knee Jt, Perc Endo Approach */
"0SPD45Z"  =  "1"  /* Removal of Ext Fix from L Knee Jt, Perc Endo Approach */
"0SPD47Z"  =  "1"  /* Removal of Autol Sub from L Knee Jt, Perc Endo Approach */
"0SPD4JZ"  =  "1"  /* Removal of Synth Sub from L Knee Jt, Perc Endo Approach */
"0SPD4KZ"  =  "1"  /* Removal of Nonaut Sub from L Knee Jt, Perc Endo Approach */
"0SPF00Z"  =  "1"  /* Removal of Drainage Device from R Ankle Jt, Open Approach */
"0SPF03Z"  =  "1"  /* Removal of Infusion Device from R Ankle Jt, Open Approach */
"0SPF04Z"  =  "1"  /* Removal of Int Fix from R Ankle Jt, Open Approach */
"0SPF05Z"  =  "1"  /* Removal of Ext Fix from R Ankle Jt, Open Approach */
"0SPF07Z"  =  "1"  /* Removal of Autol Sub from R Ankle Jt, Open Approach */
"0SPF0JZ"  =  "1"  /* Removal of Synth Sub from R Ankle Jt, Open Approach */
"0SPF0KZ"  =  "1"  /* Removal of Nonaut Sub from R Ankle Jt, Open Approach */
"0SPF30Z"  =  "1"  /* Removal of Drainage Device from R Ankle Jt, Perc Approach */
"0SPF33Z"  =  "1"  /* Removal of Infusion Device from R Ankle Jt, Perc Approach */
"0SPF34Z"  =  "1"  /* Removal of Int Fix from R Ankle Jt, Perc Approach */
"0SPF35Z"  =  "1"  /* Removal of Ext Fix from R Ankle Jt, Perc Approach */
"0SPF37Z"  =  "1"  /* Removal of Autol Sub from R Ankle Jt, Perc Approach */
"0SPF3JZ"  =  "1"  /* Removal of Synth Sub from R Ankle Jt, Perc Approach */
"0SPF3KZ"  =  "1"  /* Removal of Nonaut Sub from R Ankle Jt, Perc Approach */
"0SPF40Z"  =  "1"  /* Removal of Drain Dev from R Ankle Jt, Perc Endo Approach */
"0SPF43Z"  =  "1"  /* Removal of Infusion Dev from R Ankle Jt, Perc Endo Approach */
"0SPF44Z"  =  "1"  /* Removal of Int Fix from R Ankle Jt, Perc Endo Approach */
"0SPF45Z"  =  "1"  /* Removal of Ext Fix from R Ankle Jt, Perc Endo Approach */
"0SPF47Z"  =  "1"  /* Removal of Autol Sub from R Ankle Jt, Perc Endo Approach */
"0SPF4JZ"  =  "1"  /* Removal of Synth Sub from R Ankle Jt, Perc Endo Approach */
"0SPF4KZ"  =  "1"  /* Removal of Nonaut Sub from R Ankle Jt, Perc Endo Approach */
"0SPG00Z"  =  "1"  /* Removal of Drainage Device from L Ankle Jt, Open Approach */
"0SPG03Z"  =  "1"  /* Removal of Infusion Device from L Ankle Jt, Open Approach */
"0SPG04Z"  =  "1"  /* Removal of Int Fix from L Ankle Jt, Open Approach */
"0SPG05Z"  =  "1"  /* Removal of Ext Fix from L Ankle Jt, Open Approach */
"0SPG07Z"  =  "1"  /* Removal of Autol Sub from L Ankle Jt, Open Approach */
"0SPG0JZ"  =  "1"  /* Removal of Synth Sub from L Ankle Jt, Open Approach */
"0SPG0KZ"  =  "1"  /* Removal of Nonaut Sub from L Ankle Jt, Open Approach */
"0SPG30Z"  =  "1"  /* Removal of Drainage Device from L Ankle Jt, Perc Approach */
"0SPG33Z"  =  "1"  /* Removal of Infusion Device from L Ankle Jt, Perc Approach */
"0SPG34Z"  =  "1"  /* Removal of Int Fix from L Ankle Jt, Perc Approach */
"0SPG35Z"  =  "1"  /* Removal of Ext Fix from L Ankle Jt, Perc Approach */
"0SPG37Z"  =  "1"  /* Removal of Autol Sub from L Ankle Jt, Perc Approach */
"0SPG3JZ"  =  "1"  /* Removal of Synth Sub from L Ankle Jt, Perc Approach */
"0SPG3KZ"  =  "1"  /* Removal of Nonaut Sub from L Ankle Jt, Perc Approach */
"0SPG40Z"  =  "1"  /* Removal of Drain Dev from L Ankle Jt, Perc Endo Approach */
"0SPG43Z"  =  "1"  /* Removal of Infusion Dev from L Ankle Jt, Perc Endo Approach */
"0SPG44Z"  =  "1"  /* Removal of Int Fix from L Ankle Jt, Perc Endo Approach */
"0SPG45Z"  =  "1"  /* Removal of Ext Fix from L Ankle Jt, Perc Endo Approach */
"0SPG47Z"  =  "1"  /* Removal of Autol Sub from L Ankle Jt, Perc Endo Approach */
"0SPG4JZ"  =  "1"  /* Removal of Synth Sub from L Ankle Jt, Perc Endo Approach */
"0SPG4KZ"  =  "1"  /* Removal of Nonaut Sub from L Ankle Jt, Perc Endo Approach */
"0SPH00Z"  =  "1"  /* Removal of Drainage Device from R Tarsal Jt, Open Approach */
"0SPH03Z"  =  "1"  /* Removal of Infusion Device from R Tarsal Jt, Open Approach */
"0SPH04Z"  =  "1"  /* Removal of Int Fix from R Tarsal Jt, Open Approach */
"0SPH05Z"  =  "1"  /* Removal of Ext Fix from R Tarsal Jt, Open Approach */
"0SPH07Z"  =  "1"  /* Removal of Autol Sub from R Tarsal Jt, Open Approach */
"0SPH0JZ"  =  "1"  /* Removal of Synth Sub from R Tarsal Jt, Open Approach */
"0SPH0KZ"  =  "1"  /* Removal of Nonaut Sub from R Tarsal Jt, Open Approach */
"0SPH30Z"  =  "1"  /* Removal of Drainage Device from R Tarsal Jt, Perc Approach */
"0SPH33Z"  =  "1"  /* Removal of Infusion Device from R Tarsal Jt, Perc Approach */
"0SPH34Z"  =  "1"  /* Removal of Int Fix from R Tarsal Jt, Perc Approach */
"0SPH35Z"  =  "1"  /* Removal of Ext Fix from R Tarsal Jt, Perc Approach */
"0SPH37Z"  =  "1"  /* Removal of Autol Sub from R Tarsal Jt, Perc Approach */
"0SPH3JZ"  =  "1"  /* Removal of Synth Sub from R Tarsal Jt, Perc Approach */
"0SPH3KZ"  =  "1"  /* Removal of Nonaut Sub from R Tarsal Jt, Perc Approach */
"0SPH40Z"  =  "1"  /* Removal of Drain Dev from R Tarsal Jt, Perc Endo Approach */
"0SPH43Z"  =  "1"  /* Removal of Infusion Dev from R Tarsal Jt, Perc Endo Approach */
"0SPH44Z"  =  "1"  /* Removal of Int Fix from R Tarsal Jt, Perc Endo Approach */
"0SPH45Z"  =  "1"  /* Removal of Ext Fix from R Tarsal Jt, Perc Endo Approach */
"0SPH47Z"  =  "1"  /* Removal of Autol Sub from R Tarsal Jt, Perc Endo Approach */
"0SPH4JZ"  =  "1"  /* Removal of Synth Sub from R Tarsal Jt, Perc Endo Approach */
"0SPH4KZ"  =  "1"  /* Removal of Nonaut Sub from R Tarsal Jt, Perc Endo Approach */
"0SPJ00Z"  =  "1"  /* Removal of Drainage Device from L Tarsal Jt, Open Approach */
"0SPJ03Z"  =  "1"  /* Removal of Infusion Device from L Tarsal Jt, Open Approach */
"0SPJ04Z"  =  "1"  /* Removal of Int Fix from L Tarsal Jt, Open Approach */
"0SPJ05Z"  =  "1"  /* Removal of Ext Fix from L Tarsal Jt, Open Approach */
"0SPJ07Z"  =  "1"  /* Removal of Autol Sub from L Tarsal Jt, Open Approach */
"0SPJ0JZ"  =  "1"  /* Removal of Synth Sub from L Tarsal Jt, Open Approach */
"0SPJ0KZ"  =  "1"  /* Removal of Nonaut Sub from L Tarsal Jt, Open Approach */
"0SPJ30Z"  =  "1"  /* Removal of Drainage Device from L Tarsal Jt, Perc Approach */
"0SPJ33Z"  =  "1"  /* Removal of Infusion Device from L Tarsal Jt, Perc Approach */
"0SPJ34Z"  =  "1"  /* Removal of Int Fix from L Tarsal Jt, Perc Approach */
"0SPJ35Z"  =  "1"  /* Removal of Ext Fix from L Tarsal Jt, Perc Approach */
"0SPJ37Z"  =  "1"  /* Removal of Autol Sub from L Tarsal Jt, Perc Approach */
"0SPJ3JZ"  =  "1"  /* Removal of Synth Sub from L Tarsal Jt, Perc Approach */
"0SPJ3KZ"  =  "1"  /* Removal of Nonaut Sub from L Tarsal Jt, Perc Approach */
"0SPJ40Z"  =  "1"  /* Removal of Drain Dev from L Tarsal Jt, Perc Endo Approach */
"0SPJ43Z"  =  "1"  /* Removal of Infusion Dev from L Tarsal Jt, Perc Endo Approach */
"0SPJ44Z"  =  "1"  /* Removal of Int Fix from L Tarsal Jt, Perc Endo Approach */
"0SPJ45Z"  =  "1"  /* Removal of Ext Fix from L Tarsal Jt, Perc Endo Approach */
"0SPJ47Z"  =  "1"  /* Removal of Autol Sub from L Tarsal Jt, Perc Endo Approach */
"0SPJ4JZ"  =  "1"  /* Removal of Synth Sub from L Tarsal Jt, Perc Endo Approach */
"0SPJ4KZ"  =  "1"  /* Removal of Nonaut Sub from L Tarsal Jt, Perc Endo Approach */
"0SPK00Z"  =  "1"  /* Removal of Drain Dev from R Metatarsotars Jt, Open Approach */
"0SPK03Z"  =  "1"  /* Remove Infusion Dev from R Metatarsotars Jt, Open */
"0SPK04Z"  =  "1"  /* Removal of Int Fix from R Metatarsotars Jt, Open Approach */
"0SPK05Z"  =  "1"  /* Removal of Ext Fix from R Metatarsotars Jt, Open Approach */
"0SPK07Z"  =  "1"  /* Removal of Autol Sub from R Metatarsotars Jt, Open Approach */
"0SPK0JZ"  =  "1"  /* Removal of Synth Sub from R Metatarsotars Jt, Open Approach */
"0SPK0KZ"  =  "1"  /* Removal of Nonaut Sub from R Metatarsotars Jt, Open Approach */
"0SPK30Z"  =  "1"  /* Removal of Drain Dev from R Metatarsotars Jt, Perc Approach */
"0SPK33Z"  =  "1"  /* Remove Infusion Dev from R Metatarsotars Jt, Perc */
"0SPK34Z"  =  "1"  /* Removal of Int Fix from R Metatarsotars Jt, Perc Approach */
"0SPK35Z"  =  "1"  /* Removal of Ext Fix from R Metatarsotars Jt, Perc Approach */
"0SPK37Z"  =  "1"  /* Removal of Autol Sub from R Metatarsotars Jt, Perc Approach */
"0SPK3JZ"  =  "1"  /* Removal of Synth Sub from R Metatarsotars Jt, Perc Approach */
"0SPK3KZ"  =  "1"  /* Removal of Nonaut Sub from R Metatarsotars Jt, Perc Approach */
"0SPK40Z"  =  "1"  /* Remove Drain Dev from R Metatarsotars Jt, Perc Endo */
"0SPK43Z"  =  "1"  /* Remove Infusion Dev from R Metatarsotars Jt, Perc Endo */
"0SPK44Z"  =  "1"  /* Remove Int Fix from R Metatarsotars Jt, Perc Endo */
"0SPK45Z"  =  "1"  /* Remove Ext Fix from R Metatarsotars Jt, Perc Endo */
"0SPK47Z"  =  "1"  /* Remove Autol Sub from R Metatarsotars Jt, Perc Endo */
"0SPK4JZ"  =  "1"  /* Remove Synth Sub from R Metatarsotars Jt, Perc Endo */
"0SPK4KZ"  =  "1"  /* Remove Nonaut Sub from R Metatarsotars Jt, Perc Endo */
"0SPL00Z"  =  "1"  /* Removal of Drain Dev from L Metatarsotars Jt, Open Approach */
"0SPL03Z"  =  "1"  /* Remove Infusion Dev from L Metatarsotars Jt, Open */
"0SPL04Z"  =  "1"  /* Removal of Int Fix from L Metatarsotars Jt, Open Approach */
"0SPL05Z"  =  "1"  /* Removal of Ext Fix from L Metatarsotars Jt, Open Approach */
"0SPL07Z"  =  "1"  /* Removal of Autol Sub from L Metatarsotars Jt, Open Approach */
"0SPL0JZ"  =  "1"  /* Removal of Synth Sub from L Metatarsotars Jt, Open Approach */
"0SPL0KZ"  =  "1"  /* Removal of Nonaut Sub from L Metatarsotars Jt, Open Approach */
"0SPL30Z"  =  "1"  /* Removal of Drain Dev from L Metatarsotars Jt, Perc Approach */
"0SPL33Z"  =  "1"  /* Remove Infusion Dev from L Metatarsotars Jt, Perc */
"0SPL34Z"  =  "1"  /* Removal of Int Fix from L Metatarsotars Jt, Perc Approach */
"0SPL35Z"  =  "1"  /* Removal of Ext Fix from L Metatarsotars Jt, Perc Approach */
"0SPL37Z"  =  "1"  /* Removal of Autol Sub from L Metatarsotars Jt, Perc Approach */
"0SPL3JZ"  =  "1"  /* Removal of Synth Sub from L Metatarsotars Jt, Perc Approach */
"0SPL3KZ"  =  "1"  /* Removal of Nonaut Sub from L Metatarsotars Jt, Perc Approach */
"0SPL40Z"  =  "1"  /* Remove Drain Dev from L Metatarsotars Jt, Perc Endo */
"0SPL43Z"  =  "1"  /* Remove Infusion Dev from L Metatarsotars Jt, Perc Endo */
"0SPL44Z"  =  "1"  /* Remove Int Fix from L Metatarsotars Jt, Perc Endo */
"0SPL45Z"  =  "1"  /* Remove Ext Fix from L Metatarsotars Jt, Perc Endo */
"0SPL47Z"  =  "1"  /* Remove Autol Sub from L Metatarsotars Jt, Perc Endo */
"0SPL4JZ"  =  "1"  /* Remove Synth Sub from L Metatarsotars Jt, Perc Endo */
"0SPL4KZ"  =  "1"  /* Remove Nonaut Sub from L Metatarsotars Jt, Perc Endo */
"0SPM00Z"  =  "1"  /* Removal of Drain Dev from R Metatarsophal Jt, Open Approach */
"0SPM03Z"  =  "1"  /* Remove Infusion Dev from R Metatarsophal Jt, Open */
"0SPM04Z"  =  "1"  /* Removal of Int Fix from R Metatarsophal Jt, Open Approach */
"0SPM05Z"  =  "1"  /* Removal of Ext Fix from R Metatarsophal Jt, Open Approach */
"0SPM07Z"  =  "1"  /* Removal of Autol Sub from R Metatarsophal Jt, Open Approach */
"0SPM0JZ"  =  "1"  /* Removal of Synth Sub from R Metatarsophal Jt, Open Approach */
"0SPM0KZ"  =  "1"  /* Removal of Nonaut Sub from R Metatarsophal Jt, Open Approach */
"0SPM30Z"  =  "1"  /* Removal of Drain Dev from R Metatarsophal Jt, Perc Approach */
"0SPM33Z"  =  "1"  /* Remove Infusion Dev from R Metatarsophal Jt, Perc */
"0SPM34Z"  =  "1"  /* Removal of Int Fix from R Metatarsophal Jt, Perc Approach */
"0SPM35Z"  =  "1"  /* Removal of Ext Fix from R Metatarsophal Jt, Perc Approach */
"0SPM37Z"  =  "1"  /* Removal of Autol Sub from R Metatarsophal Jt, Perc Approach */
"0SPM3JZ"  =  "1"  /* Removal of Synth Sub from R Metatarsophal Jt, Perc Approach */
"0SPM3KZ"  =  "1"  /* Removal of Nonaut Sub from R Metatarsophal Jt, Perc Approach */
"0SPM40Z"  =  "1"  /* Remove Drain Dev from R Metatarsophal Jt, Perc Endo */
"0SPM43Z"  =  "1"  /* Remove Infusion Dev from R Metatarsophal Jt, Perc Endo */
"0SPM44Z"  =  "1"  /* Remove Int Fix from R Metatarsophal Jt, Perc Endo */
"0SPM45Z"  =  "1"  /* Remove Ext Fix from R Metatarsophal Jt, Perc Endo */
"0SPM47Z"  =  "1"  /* Remove Autol Sub from R Metatarsophal Jt, Perc Endo */
"0SPM4JZ"  =  "1"  /* Remove Synth Sub from R Metatarsophal Jt, Perc Endo */
"0SPM4KZ"  =  "1"  /* Remove Nonaut Sub from R Metatarsophal Jt, Perc Endo */
"0SPN00Z"  =  "1"  /* Removal of Drain Dev from L Metatarsophal Jt, Open Approach */
"0SPN03Z"  =  "1"  /* Remove Infusion Dev from L Metatarsophal Jt, Open */
"0SPN04Z"  =  "1"  /* Removal of Int Fix from L Metatarsophal Jt, Open Approach */
"0SPN05Z"  =  "1"  /* Removal of Ext Fix from L Metatarsophal Jt, Open Approach */
"0SPN07Z"  =  "1"  /* Removal of Autol Sub from L Metatarsophal Jt, Open Approach */
"0SPN0JZ"  =  "1"  /* Removal of Synth Sub from L Metatarsophal Jt, Open Approach */
"0SPN0KZ"  =  "1"  /* Removal of Nonaut Sub from L Metatarsophal Jt, Open Approach */
"0SPN30Z"  =  "1"  /* Removal of Drain Dev from L Metatarsophal Jt, Perc Approach */
"0SPN33Z"  =  "1"  /* Remove Infusion Dev from L Metatarsophal Jt, Perc */
"0SPN34Z"  =  "1"  /* Removal of Int Fix from L Metatarsophal Jt, Perc Approach */
"0SPN35Z"  =  "1"  /* Removal of Ext Fix from L Metatarsophal Jt, Perc Approach */
"0SPN37Z"  =  "1"  /* Removal of Autol Sub from L Metatarsophal Jt, Perc Approach */
"0SPN3JZ"  =  "1"  /* Removal of Synth Sub from L Metatarsophal Jt, Perc Approach */
"0SPN3KZ"  =  "1"  /* Removal of Nonaut Sub from L Metatarsophal Jt, Perc Approach */
"0SPN40Z"  =  "1"  /* Remove Drain Dev from L Metatarsophal Jt, Perc Endo */
"0SPN43Z"  =  "1"  /* Remove Infusion Dev from L Metatarsophal Jt, Perc Endo */
"0SPN44Z"  =  "1"  /* Remove Int Fix from L Metatarsophal Jt, Perc Endo */
"0SPN45Z"  =  "1"  /* Remove Ext Fix from L Metatarsophal Jt, Perc Endo */
"0SPN47Z"  =  "1"  /* Remove Autol Sub from L Metatarsophal Jt, Perc Endo */
"0SPN4JZ"  =  "1"  /* Remove Synth Sub from L Metatarsophal Jt, Perc Endo */
"0SPN4KZ"  =  "1"  /* Remove Nonaut Sub from L Metatarsophal Jt, Perc Endo */
"0SPP00Z"  =  "1"  /* Removal of Drain Dev from R Toe Phalanx Jt, Open Approach */
"0SPP03Z"  =  "1"  /* Removal of Infusion Dev from R Toe Phalanx Jt, Open Approach */
"0SPP04Z"  =  "1"  /* Removal of Int Fix from R Toe Phalanx Jt, Open Approach */
"0SPP05Z"  =  "1"  /* Removal of Ext Fix from R Toe Phalanx Jt, Open Approach */
"0SPP07Z"  =  "1"  /* Removal of Autol Sub from R Toe Phalanx Jt, Open Approach */
"0SPP0JZ"  =  "1"  /* Removal of Synth Sub from R Toe Phalanx Jt, Open Approach */
"0SPP0KZ"  =  "1"  /* Removal of Nonaut Sub from R Toe Phalanx Jt, Open Approach */
"0SPP30Z"  =  "1"  /* Removal of Drain Dev from R Toe Phalanx Jt, Perc Approach */
"0SPP33Z"  =  "1"  /* Removal of Infusion Dev from R Toe Phalanx Jt, Perc Approach */
"0SPP34Z"  =  "1"  /* Removal of Int Fix from R Toe Phalanx Jt, Perc Approach */
"0SPP35Z"  =  "1"  /* Removal of Ext Fix from R Toe Phalanx Jt, Perc Approach */
"0SPP37Z"  =  "1"  /* Removal of Autol Sub from R Toe Phalanx Jt, Perc Approach */
"0SPP3JZ"  =  "1"  /* Removal of Synth Sub from R Toe Phalanx Jt, Perc Approach */
"0SPP3KZ"  =  "1"  /* Removal of Nonaut Sub from R Toe Phalanx Jt, Perc Approach */
"0SPP40Z"  =  "1"  /* Remove Drain Dev from R Toe Phalanx Jt, Perc Endo */
"0SPP43Z"  =  "1"  /* Remove Infusion Dev from R Toe Phalanx Jt, Perc Endo */
"0SPP44Z"  =  "1"  /* Removal of Int Fix from R Toe Phalanx Jt, Perc Endo Approach */
"0SPP45Z"  =  "1"  /* Removal of Ext Fix from R Toe Phalanx Jt, Perc Endo Approach */
"0SPP47Z"  =  "1"  /* Remove Autol Sub from R Toe Phalanx Jt, Perc Endo */
"0SPP4JZ"  =  "1"  /* Remove Synth Sub from R Toe Phalanx Jt, Perc Endo */
"0SPP4KZ"  =  "1"  /* Remove Nonaut Sub from R Toe Phalanx Jt, Perc Endo */
"0SPQ00Z"  =  "1"  /* Removal of Drain Dev from L Toe Phalanx Jt, Open Approach */
"0SPQ03Z"  =  "1"  /* Removal of Infusion Dev from L Toe Phalanx Jt, Open Approach */
"0SPQ04Z"  =  "1"  /* Removal of Int Fix from L Toe Phalanx Jt, Open Approach */
"0SPQ05Z"  =  "1"  /* Removal of Ext Fix from L Toe Phalanx Jt, Open Approach */
"0SPQ07Z"  =  "1"  /* Removal of Autol Sub from L Toe Phalanx Jt, Open Approach */
"0SPQ0JZ"  =  "1"  /* Removal of Synth Sub from L Toe Phalanx Jt, Open Approach */
"0SPQ0KZ"  =  "1"  /* Removal of Nonaut Sub from L Toe Phalanx Jt, Open Approach */
"0SPQ30Z"  =  "1"  /* Removal of Drain Dev from L Toe Phalanx Jt, Perc Approach */
"0SPQ33Z"  =  "1"  /* Removal of Infusion Dev from L Toe Phalanx Jt, Perc Approach */
"0SPQ34Z"  =  "1"  /* Removal of Int Fix from L Toe Phalanx Jt, Perc Approach */
"0SPQ35Z"  =  "1"  /* Removal of Ext Fix from L Toe Phalanx Jt, Perc Approach */
"0SPQ37Z"  =  "1"  /* Removal of Autol Sub from L Toe Phalanx Jt, Perc Approach */
"0SPQ3JZ"  =  "1"  /* Removal of Synth Sub from L Toe Phalanx Jt, Perc Approach */
"0SPQ3KZ"  =  "1"  /* Removal of Nonaut Sub from L Toe Phalanx Jt, Perc Approach */
"0SPQ40Z"  =  "1"  /* Remove Drain Dev from L Toe Phalanx Jt, Perc Endo */
"0SPQ43Z"  =  "1"  /* Remove Infusion Dev from L Toe Phalanx Jt, Perc Endo */
"0SPQ44Z"  =  "1"  /* Removal of Int Fix from L Toe Phalanx Jt, Perc Endo Approach */
"0SPQ45Z"  =  "1"  /* Removal of Ext Fix from L Toe Phalanx Jt, Perc Endo Approach */
"0SPQ47Z"  =  "1"  /* Remove Autol Sub from L Toe Phalanx Jt, Perc Endo */
"0SPQ4JZ"  =  "1"  /* Remove Synth Sub from L Toe Phalanx Jt, Perc Endo */
"0SPQ4KZ"  =  "1"  /* Remove Nonaut Sub from L Toe Phalanx Jt, Perc Endo */
"0SQ00ZZ"  =  "1"  /* Repair Lumbar Vertebral Joint, Open Approach */
"0SQ03ZZ"  =  "1"  /* Repair Lumbar Vertebral Joint, Percutaneous Approach */
"0SQ04ZZ"  =  "1"  /* Repair Lumbar Vertebral Joint, Perc Endo Approach */
"0SQ0XZZ"  =  "1"  /* Repair Lumbar Vertebral Joint, External Approach */
"0SQ20ZZ"  =  "1"  /* Repair Lumbar Vertebral Disc, Open Approach */
"0SQ23ZZ"  =  "1"  /* Repair Lumbar Vertebral Disc, Percutaneous Approach */
"0SQ24ZZ"  =  "1"  /* Repair Lumbar Vertebral Disc, Perc Endo Approach */
"0SQ2XZZ"  =  "1"  /* Repair Lumbar Vertebral Disc, External Approach */
"0SQ30ZZ"  =  "1"  /* Repair Lumbosacral Joint, Open Approach */
"0SQ33ZZ"  =  "1"  /* Repair Lumbosacral Joint, Percutaneous Approach */
"0SQ34ZZ"  =  "1"  /* Repair Lumbosacral Joint, Percutaneous Endoscopic Approach */
"0SQ3XZZ"  =  "1"  /* Repair Lumbosacral Joint, External Approach */
"0SQ40ZZ"  =  "1"  /* Repair Lumbosacral Disc, Open Approach */
"0SQ43ZZ"  =  "1"  /* Repair Lumbosacral Disc, Percutaneous Approach */
"0SQ44ZZ"  =  "1"  /* Repair Lumbosacral Disc, Percutaneous Endoscopic Approach */
"0SQ4XZZ"  =  "1"  /* Repair Lumbosacral Disc, External Approach */
"0SQ50ZZ"  =  "1"  /* Repair Sacrococcygeal Joint, Open Approach */
"0SQ53ZZ"  =  "1"  /* Repair Sacrococcygeal Joint, Percutaneous Approach */
"0SQ54ZZ"  =  "1"  /* Repair Sacrococcygeal Joint, Perc Endo Approach */
"0SQ5XZZ"  =  "1"  /* Repair Sacrococcygeal Joint, External Approach */
"0SQ60ZZ"  =  "1"  /* Repair Coccygeal Joint, Open Approach */
"0SQ63ZZ"  =  "1"  /* Repair Coccygeal Joint, Percutaneous Approach */
"0SQ64ZZ"  =  "1"  /* Repair Coccygeal Joint, Percutaneous Endoscopic Approach */
"0SQ6XZZ"  =  "1"  /* Repair Coccygeal Joint, External Approach */
"0SQ70ZZ"  =  "1"  /* Repair Right Sacroiliac Joint, Open Approach */
"0SQ73ZZ"  =  "1"  /* Repair Right Sacroiliac Joint, Percutaneous Approach */
"0SQ74ZZ"  =  "1"  /* Repair Right Sacroiliac Joint, Perc Endo Approach */
"0SQ7XZZ"  =  "1"  /* Repair Right Sacroiliac Joint, External Approach */
"0SQ80ZZ"  =  "1"  /* Repair Left Sacroiliac Joint, Open Approach */
"0SQ83ZZ"  =  "1"  /* Repair Left Sacroiliac Joint, Percutaneous Approach */
"0SQ84ZZ"  =  "1"  /* Repair Left Sacroiliac Joint, Perc Endo Approach */
"0SQ8XZZ"  =  "1"  /* Repair Left Sacroiliac Joint, External Approach */
"0SQ90ZZ"  =  "1"  /* Repair Right Hip Joint, Open Approach */
"0SQ93ZZ"  =  "1"  /* Repair Right Hip Joint, Percutaneous Approach */
"0SQ94ZZ"  =  "1"  /* Repair Right Hip Joint, Percutaneous Endoscopic Approach */
"0SQ9XZZ"  =  "1"  /* Repair Right Hip Joint, External Approach */
"0SQB0ZZ"  =  "1"  /* Repair Left Hip Joint, Open Approach */
"0SQB3ZZ"  =  "1"  /* Repair Left Hip Joint, Percutaneous Approach */
"0SQB4ZZ"  =  "1"  /* Repair Left Hip Joint, Percutaneous Endoscopic Approach */
"0SQBXZZ"  =  "1"  /* Repair Left Hip Joint, External Approach */
"0SQC0ZZ"  =  "1"  /* Repair Right Knee Joint, Open Approach */
"0SQC3ZZ"  =  "1"  /* Repair Right Knee Joint, Percutaneous Approach */
"0SQC4ZZ"  =  "1"  /* Repair Right Knee Joint, Percutaneous Endoscopic Approach */
"0SQCXZZ"  =  "1"  /* Repair Right Knee Joint, External Approach */
"0SQD0ZZ"  =  "1"  /* Repair Left Knee Joint, Open Approach */
"0SQD3ZZ"  =  "1"  /* Repair Left Knee Joint, Percutaneous Approach */
"0SQD4ZZ"  =  "1"  /* Repair Left Knee Joint, Percutaneous Endoscopic Approach */
"0SQDXZZ"  =  "1"  /* Repair Left Knee Joint, External Approach */
"0SQF0ZZ"  =  "1"  /* Repair Right Ankle Joint, Open Approach */
"0SQF3ZZ"  =  "1"  /* Repair Right Ankle Joint, Percutaneous Approach */
"0SQF4ZZ"  =  "1"  /* Repair Right Ankle Joint, Percutaneous Endoscopic Approach */
"0SQFXZZ"  =  "1"  /* Repair Right Ankle Joint, External Approach */
"0SQG0ZZ"  =  "1"  /* Repair Left Ankle Joint, Open Approach */
"0SQG3ZZ"  =  "1"  /* Repair Left Ankle Joint, Percutaneous Approach */
"0SQG4ZZ"  =  "1"  /* Repair Left Ankle Joint, Percutaneous Endoscopic Approach */
"0SQGXZZ"  =  "1"  /* Repair Left Ankle Joint, External Approach */
"0SQH0ZZ"  =  "1"  /* Repair Right Tarsal Joint, Open Approach */
"0SQH3ZZ"  =  "1"  /* Repair Right Tarsal Joint, Percutaneous Approach */
"0SQH4ZZ"  =  "1"  /* Repair Right Tarsal Joint, Percutaneous Endoscopic Approach */
"0SQHXZZ"  =  "1"  /* Repair Right Tarsal Joint, External Approach */
"0SQJ0ZZ"  =  "1"  /* Repair Left Tarsal Joint, Open Approach */
"0SQJ3ZZ"  =  "1"  /* Repair Left Tarsal Joint, Percutaneous Approach */
"0SQJ4ZZ"  =  "1"  /* Repair Left Tarsal Joint, Percutaneous Endoscopic Approach */
"0SQJXZZ"  =  "1"  /* Repair Left Tarsal Joint, External Approach */
"0SQK0ZZ"  =  "1"  /* Repair Right Metatarsal-Tarsal Joint, Open Approach */
"0SQK3ZZ"  =  "1"  /* Repair Right Metatarsal-Tarsal Joint, Percutaneous Approach */
"0SQK4ZZ"  =  "1"  /* Repair Right Metatarsal-Tarsal Joint, Perc Endo Approach */
"0SQKXZZ"  =  "1"  /* Repair Right Metatarsal-Tarsal Joint, External Approach */
"0SQL0ZZ"  =  "1"  /* Repair Left Metatarsal-Tarsal Joint, Open Approach */
"0SQL3ZZ"  =  "1"  /* Repair Left Metatarsal-Tarsal Joint, Percutaneous Approach */
"0SQL4ZZ"  =  "1"  /* Repair Left Metatarsal-Tarsal Joint, Perc Endo Approach */
"0SQLXZZ"  =  "1"  /* Repair Left Metatarsal-Tarsal Joint, External Approach */
"0SQM0ZZ"  =  "1"  /* Repair Right Metatarsal-Phalangeal Joint, Open Approach */
"0SQM3ZZ"  =  "1"  /* Repair Right Metatarsal-Phalangeal Joint, Perc Approach */
"0SQM4ZZ"  =  "1"  /* Repair Right Metatarsal-Phalangeal Joint, Perc Endo Approach */
"0SQMXZZ"  =  "1"  /* Repair Right Metatarsal-Phalangeal Joint, External Approach */
"0SQN0ZZ"  =  "1"  /* Repair Left Metatarsal-Phalangeal Joint, Open Approach */
"0SQN3ZZ"  =  "1"  /* Repair Left Metatarsal-Phalangeal Joint, Perc Approach */
"0SQN4ZZ"  =  "1"  /* Repair Left Metatarsal-Phalangeal Joint, Perc Endo Approach */
"0SQNXZZ"  =  "1"  /* Repair Left Metatarsal-Phalangeal Joint, External Approach */
"0SQP0ZZ"  =  "1"  /* Repair Right Toe Phalangeal Joint, Open Approach */
"0SQP3ZZ"  =  "1"  /* Repair Right Toe Phalangeal Joint, Percutaneous Approach */
"0SQP4ZZ"  =  "1"  /* Repair Right Toe Phalangeal Joint, Perc Endo Approach */
"0SQPXZZ"  =  "1"  /* Repair Right Toe Phalangeal Joint, External Approach */
"0SQQ0ZZ"  =  "1"  /* Repair Left Toe Phalangeal Joint, Open Approach */
"0SQQ3ZZ"  =  "1"  /* Repair Left Toe Phalangeal Joint, Percutaneous Approach */
"0SQQ4ZZ"  =  "1"  /* Repair Left Toe Phalangeal Joint, Perc Endo Approach */
"0SQQXZZ"  =  "1"  /* Repair Left Toe Phalangeal Joint, External Approach */
"0SR007Z"  =  "1"  /* Replacement of Lum Jt with Autol Sub, Open Approach */
"0SR00JZ"  =  "1"  /* Replacement of Lum Jt with Synth Sub, Open Approach */
"0SR00KZ"  =  "1"  /* Replacement of Lum Jt with Nonaut Sub, Open Approach */
"0SR207Z"  =  "1"  /* Replacement of Lum Disc with Autol Sub, Open Approach */
"0SR20JZ"  =  "1"  /* Replacement of Lum Disc with Synth Sub, Open Approach */
"0SR20KZ"  =  "1"  /* Replacement of Lum Disc with Nonaut Sub, Open Approach */
"0SR307Z"  =  "1"  /* Replacement of Lumsac Jt with Autol Sub, Open Approach */
"0SR30JZ"  =  "1"  /* Replacement of Lumsac Jt with Synth Sub, Open Approach */
"0SR30KZ"  =  "1"  /* Replacement of Lumsac Jt with Nonaut Sub, Open Approach */
"0SR407Z"  =  "1"  /* Replacement of Lumsac Disc with Autol Sub, Open Approach */
"0SR40JZ"  =  "1"  /* Replacement of Lumsac Disc with Synth Sub, Open Approach */
"0SR40KZ"  =  "1"  /* Replacement of Lumsac Disc with Nonaut Sub, Open Approach */
"0SR507Z"  =  "1"  /* Replace of Sacrococcygeal Jt with Autol Sub, Open Approach */
"0SR50JZ"  =  "1"  /* Replace of Sacrococcygeal Jt with Synth Sub, Open Approach */
"0SR50KZ"  =  "1"  /* Replace of Sacrococcygeal Jt with Nonaut Sub, Open Approach */
"0SR607Z"  =  "1"  /* Replacement of Coccygeal Joint with Autol Sub, Open Approach */
"0SR60JZ"  =  "1"  /* Replacement of Coccygeal Joint with Synth Sub, Open Approach */
"0SR60KZ"  =  "1"  /* Replacement of Coccygeal Jt with Nonaut Sub, Open Approach */
"0SR707Z"  =  "1"  /* Replacement of R Sacroiliac Jt with Autol Sub, Open Approach */
"0SR70JZ"  =  "1"  /* Replacement of R Sacroiliac Jt with Synth Sub, Open Approach */
"0SR70KZ"  =  "1"  /* Replace of R Sacroiliac Jt with Nonaut Sub, Open Approach */
"0SR807Z"  =  "1"  /* Replacement of L Sacroiliac Jt with Autol Sub, Open Approach */
"0SR80JZ"  =  "1"  /* Replacement of L Sacroiliac Jt with Synth Sub, Open Approach */
"0SR80KZ"  =  "1"  /* Replace of L Sacroiliac Jt with Nonaut Sub, Open Approach */
"0SR9019"  =  "1"  /* Replacement of R Hip Jt with Metal, Cement, Open Approach */
"0SR901A"  =  "1"  /* Replacement of R Hip Jt with Metal, Uncement, Open Approach */
"0SR901Z"  =  "1"  /* Replacement of Right Hip Joint with Metal, Open Approach */
"0SR9029"  =  "1"  /* Replace R Hip Jt w Metal on Poly, Cement, Open */
"0SR902A"  =  "1"  /* Replace R Hip Jt w Metal on Poly, Uncement, Open */
"0SR902Z"  =  "1"  /* Replacement of R Hip Jt with Metal on Poly, Open Approach */
"0SR9039"  =  "1"  /* Replacement of R Hip Jt with Ceramic, Cement, Open Approach */
"0SR903A"  =  "1"  /* Replace of R Hip Jt with Ceramic, Uncement, Open Approach */
"0SR903Z"  =  "1"  /* Replacement of Right Hip Joint with Ceramic, Open Approach */
"0SR9049"  =  "1"  /* Replace R Hip Jt w Ceramic on Poly, Cement, Open */
"0SR904A"  =  "1"  /* Replace R Hip Jt w Ceramic on Poly, Uncement, Open */
"0SR904Z"  =  "1"  /* Replacement of R Hip Jt with Ceramic on Poly, Open Approach */
"0SR907Z"  =  "1"  /* Replacement of Right Hip Joint with Autol Sub, Open Approach */
"0SR90J9"  =  "1"  /* Replace of R Hip Jt with Synth Sub, Cement, Open Approach */
"0SR90JA"  =  "1"  /* Replace of R Hip Jt with Synth Sub, Uncement, Open Approach */
"0SR90JZ"  =  "1"  /* Replacement of Right Hip Joint with Synth Sub, Open Approach */
"0SR90KZ"  =  "1"  /* Replacement of R Hip Jt with Nonaut Sub, Open Approach */
"0SRA009"  =  "1"  /* Replace R Hip Jt, Acetab w Polyeth, Cement, Open */
"0SRA00A"  =  "1"  /* Replace R Hip Jt, Acetab w Polyeth, Uncement, Open */
"0SRA00Z"  =  "1"  /* Replacement of R Hip Jt, Acetab with Polyeth, Open Approach */
"0SRA019"  =  "1"  /* Replace R Hip Jt, Acetab w Metal, Cement, Open */
"0SRA01A"  =  "1"  /* Replace R Hip Jt, Acetab w Metal, Uncement, Open */
"0SRA01Z"  =  "1"  /* Replacement of R Hip Jt, Acetab with Metal, Open Approach */
"0SRA039"  =  "1"  /* Replace R Hip Jt, Acetab w Ceramic, Cement, Open */
"0SRA03A"  =  "1"  /* Replace R Hip Jt, Acetab w Ceramic, Uncement, Open */
"0SRA03Z"  =  "1"  /* Replacement of R Hip Jt, Acetab with Ceramic, Open Approach */
"0SRA07Z"  =  "1"  /* Replace of R Hip Jt, Acetab with Autol Sub, Open Approach */
"0SRA0J9"  =  "1"  /* Replace R Hip Jt, Acetab w Synth Sub, Cement, Open */
"0SRA0JA"  =  "1"  /* Replace R Hip Jt, Acetab w Synth Sub, Uncement, Open */
"0SRA0JZ"  =  "1"  /* Replace of R Hip Jt, Acetab with Synth Sub, Open Approach */
"0SRA0KZ"  =  "1"  /* Replace of R Hip Jt, Acetab with Nonaut Sub, Open Approach */
"0SRB019"  =  "1"  /* Replacement of L Hip Jt with Metal, Cement, Open Approach */
"0SRB01A"  =  "1"  /* Replacement of L Hip Jt with Metal, Uncement, Open Approach */
"0SRB01Z"  =  "1"  /* Replacement of Left Hip Joint with Metal, Open Approach */
"0SRB029"  =  "1"  /* Replace L Hip Jt w Metal on Poly, Cement, Open */
"0SRB02A"  =  "1"  /* Replace L Hip Jt w Metal on Poly, Uncement, Open */
"0SRB02Z"  =  "1"  /* Replacement of L Hip Jt with Metal on Poly, Open Approach */
"0SRB039"  =  "1"  /* Replacement of L Hip Jt with Ceramic, Cement, Open Approach */
"0SRB03A"  =  "1"  /* Replace of L Hip Jt with Ceramic, Uncement, Open Approach */
"0SRB03Z"  =  "1"  /* Replacement of Left Hip Joint with Ceramic, Open Approach */
"0SRB049"  =  "1"  /* Replace L Hip Jt w Ceramic on Poly, Cement, Open */
"0SRB04A"  =  "1"  /* Replace L Hip Jt w Ceramic on Poly, Uncement, Open */
"0SRB04Z"  =  "1"  /* Replacement of L Hip Jt with Ceramic on Poly, Open Approach */
"0SRB07Z"  =  "1"  /* Replacement of Left Hip Joint with Autol Sub, Open Approach */
"0SRB0J9"  =  "1"  /* Replace of L Hip Jt with Synth Sub, Cement, Open Approach */
"0SRB0JA"  =  "1"  /* Replace of L Hip Jt with Synth Sub, Uncement, Open Approach */
"0SRB0JZ"  =  "1"  /* Replacement of Left Hip Joint with Synth Sub, Open Approach */
"0SRB0KZ"  =  "1"  /* Replacement of Left Hip Joint with Nonaut Sub, Open Approach */
"0SRC07Z"  =  "1"  /* Replacement of R Knee Jt with Autol Sub, Open Approach */
"0SRC0J9"  =  "1"  /* Replace of R Knee Jt with Synth Sub, Cement, Open Approach */
"0SRC0JA"  =  "1"  /* Replace of R Knee Jt with Synth Sub, Uncement, Open Approach */
"0SRC0JZ"  =  "1"  /* Replacement of R Knee Jt with Synth Sub, Open Approach */
"0SRC0KZ"  =  "1"  /* Replacement of R Knee Jt with Nonaut Sub, Open Approach */
"0SRD07Z"  =  "1"  /* Replacement of Left Knee Joint with Autol Sub, Open Approach */
"0SRD0J9"  =  "1"  /* Replace of L Knee Jt with Synth Sub, Cement, Open Approach */
"0SRD0JA"  =  "1"  /* Replace of L Knee Jt with Synth Sub, Uncement, Open Approach */
"0SRD0JZ"  =  "1"  /* Replacement of Left Knee Joint with Synth Sub, Open Approach */
"0SRD0KZ"  =  "1"  /* Replacement of L Knee Jt with Nonaut Sub, Open Approach */
"0SRE009"  =  "1"  /* Replace L Hip Jt, Acetab w Polyeth, Cement, Open */
"0SRE00A"  =  "1"  /* Replace L Hip Jt, Acetab w Polyeth, Uncement, Open */
"0SRE00Z"  =  "1"  /* Replacement of L Hip Jt, Acetab with Polyeth, Open Approach */
"0SRE019"  =  "1"  /* Replace L Hip Jt, Acetab w Metal, Cement, Open */
"0SRE01A"  =  "1"  /* Replace L Hip Jt, Acetab w Metal, Uncement, Open */
"0SRE01Z"  =  "1"  /* Replacement of L Hip Jt, Acetab with Metal, Open Approach */
"0SRE039"  =  "1"  /* Replace L Hip Jt, Acetab w Ceramic, Cement, Open */
"0SRE03A"  =  "1"  /* Replace L Hip Jt, Acetab w Ceramic, Uncement, Open */
"0SRE03Z"  =  "1"  /* Replacement of L Hip Jt, Acetab with Ceramic, Open Approach */
"0SRE07Z"  =  "1"  /* Replace of L Hip Jt, Acetab with Autol Sub, Open Approach */
"0SRE0J9"  =  "1"  /* Replace L Hip Jt, Acetab w Synth Sub, Cement, Open */
"0SRE0JA"  =  "1"  /* Replace L Hip Jt, Acetab w Synth Sub, Uncement, Open */
"0SRE0JZ"  =  "1"  /* Replace of L Hip Jt, Acetab with Synth Sub, Open Approach */
"0SRE0KZ"  =  "1"  /* Replace of L Hip Jt, Acetab with Nonaut Sub, Open Approach */
"0SRF07Z"  =  "1"  /* Replacement of R Ankle Jt with Autol Sub, Open Approach */
"0SRF0J9"  =  "1"  /* Replace of R Ankle Jt with Synth Sub, Cement, Open Approach */
"0SRF0JA"  =  "1"  /* Replace R Ankle Jt w Synth Sub, Uncement, Open */
"0SRF0JZ"  =  "1"  /* Replacement of R Ankle Jt with Synth Sub, Open Approach */
"0SRF0KZ"  =  "1"  /* Replacement of R Ankle Jt with Nonaut Sub, Open Approach */
"0SRG07Z"  =  "1"  /* Replacement of L Ankle Jt with Autol Sub, Open Approach */
"0SRG0J9"  =  "1"  /* Replace of L Ankle Jt with Synth Sub, Cement, Open Approach */
"0SRG0JA"  =  "1"  /* Replace L Ankle Jt w Synth Sub, Uncement, Open */
"0SRG0JZ"  =  "1"  /* Replacement of L Ankle Jt with Synth Sub, Open Approach */
"0SRG0KZ"  =  "1"  /* Replacement of L Ankle Jt with Nonaut Sub, Open Approach */
"0SRH07Z"  =  "1"  /* Replacement of R Tarsal Jt with Autol Sub, Open Approach */
"0SRH0JZ"  =  "1"  /* Replacement of R Tarsal Jt with Synth Sub, Open Approach */
"0SRH0KZ"  =  "1"  /* Replacement of R Tarsal Jt with Nonaut Sub, Open Approach */
"0SRJ07Z"  =  "1"  /* Replacement of L Tarsal Jt with Autol Sub, Open Approach */
"0SRJ0JZ"  =  "1"  /* Replacement of L Tarsal Jt with Synth Sub, Open Approach */
"0SRJ0KZ"  =  "1"  /* Replacement of L Tarsal Jt with Nonaut Sub, Open Approach */
"0SRK07Z"  =  "1"  /* Replace of R Metatarsotars Jt with Autol Sub, Open Approach */
"0SRK0JZ"  =  "1"  /* Replace of R Metatarsotars Jt with Synth Sub, Open Approach */
"0SRK0KZ"  =  "1"  /* Replace of R Metatarsotars Jt with Nonaut Sub, Open Approach */
"0SRL07Z"  =  "1"  /* Replace of L Metatarsotars Jt with Autol Sub, Open Approach */
"0SRL0JZ"  =  "1"  /* Replace of L Metatarsotars Jt with Synth Sub, Open Approach */
"0SRL0KZ"  =  "1"  /* Replace of L Metatarsotars Jt with Nonaut Sub, Open Approach */
"0SRM07Z"  =  "1"  /* Replace of R Metatarsophal Jt with Autol Sub, Open Approach */
"0SRM0JZ"  =  "1"  /* Replace of R Metatarsophal Jt with Synth Sub, Open Approach */
"0SRM0KZ"  =  "1"  /* Replace of R Metatarsophal Jt with Nonaut Sub, Open Approach */
"0SRN07Z"  =  "1"  /* Replace of L Metatarsophal Jt with Autol Sub, Open Approach */
"0SRN0JZ"  =  "1"  /* Replace of L Metatarsophal Jt with Synth Sub, Open Approach */
"0SRN0KZ"  =  "1"  /* Replace of L Metatarsophal Jt with Nonaut Sub, Open Approach */
"0SRP07Z"  =  "1"  /* Replace of R Toe Phalanx Jt with Autol Sub, Open Approach */
"0SRP0JZ"  =  "1"  /* Replace of R Toe Phalanx Jt with Synth Sub, Open Approach */
"0SRP0KZ"  =  "1"  /* Replace of R Toe Phalanx Jt with Nonaut Sub, Open Approach */
"0SRQ07Z"  =  "1"  /* Replace of L Toe Phalanx Jt with Autol Sub, Open Approach */
"0SRQ0JZ"  =  "1"  /* Replace of L Toe Phalanx Jt with Synth Sub, Open Approach */
"0SRQ0KZ"  =  "1"  /* Replace of L Toe Phalanx Jt with Nonaut Sub, Open Approach */
"0SRR019"  =  "1"  /* Replace R Hip Jt, Femoral w Metal, Cement, Open */
"0SRR01A"  =  "1"  /* Replace R Hip Jt, Femoral w Metal, Uncement, Open */
"0SRR01Z"  =  "1"  /* Replacement of R Hip Jt, Femoral with Metal, Open Approach */
"0SRR039"  =  "1"  /* Replace R Hip Jt, Femoral w Ceramic, Cement, Open */
"0SRR03A"  =  "1"  /* Replace R Hip Jt, Femoral w Ceramic, Uncement, Open */
"0SRR03Z"  =  "1"  /* Replacement of R Hip Jt, Femoral with Ceramic, Open Approach */
"0SRR07Z"  =  "1"  /* Replace of R Hip Jt, Femoral with Autol Sub, Open Approach */
"0SRR0J9"  =  "1"  /* Replace R Hip Jt, Femoral w Synth Sub, Cement, Open */
"0SRR0JA"  =  "1"  /* Replace R Hip Jt, Femoral w Synth Sub, Uncement, Open */
"0SRR0JZ"  =  "1"  /* Replace of R Hip Jt, Femoral with Synth Sub, Open Approach */
"0SRR0KZ"  =  "1"  /* Replace of R Hip Jt, Femoral with Nonaut Sub, Open Approach */
"0SRS019"  =  "1"  /* Replace L Hip Jt, Femoral w Metal, Cement, Open */
"0SRS01A"  =  "1"  /* Replace L Hip Jt, Femoral w Metal, Uncement, Open */
"0SRS01Z"  =  "1"  /* Replacement of L Hip Jt, Femoral with Metal, Open Approach */
"0SRS039"  =  "1"  /* Replace L Hip Jt, Femoral w Ceramic, Cement, Open */
"0SRS03A"  =  "1"  /* Replace L Hip Jt, Femoral w Ceramic, Uncement, Open */
"0SRS03Z"  =  "1"  /* Replacement of L Hip Jt, Femoral with Ceramic, Open Approach */
"0SRS07Z"  =  "1"  /* Replace of L Hip Jt, Femoral with Autol Sub, Open Approach */
"0SRS0J9"  =  "1"  /* Replace L Hip Jt, Femoral w Synth Sub, Cement, Open */
"0SRS0JA"  =  "1"  /* Replace L Hip Jt, Femoral w Synth Sub, Uncement, Open */
"0SRS0JZ"  =  "1"  /* Replace of L Hip Jt, Femoral with Synth Sub, Open Approach */
"0SRS0KZ"  =  "1"  /* Replace of L Hip Jt, Femoral with Nonaut Sub, Open Approach */
"0SRT07Z"  =  "1"  /* Replace of R Knee Jt, Femoral with Autol Sub, Open Approach */
"0SRT0J9"  =  "1"  /* Replace R Knee Jt, Femoral w Synth Sub, Cement, Open */
"0SRT0JA"  =  "1"  /* Replace R Knee Jt, Femoral w Synth Sub, Uncement, Open */
"0SRT0JZ"  =  "1"  /* Replace of R Knee Jt, Femoral with Synth Sub, Open Approach */
"0SRT0KZ"  =  "1"  /* Replace of R Knee Jt, Femoral with Nonaut Sub, Open Approach */
"0SRU07Z"  =  "1"  /* Replace of L Knee Jt, Femoral with Autol Sub, Open Approach */
"0SRU0J9"  =  "1"  /* Replace L Knee Jt, Femoral w Synth Sub, Cement, Open */
"0SRU0JA"  =  "1"  /* Replace L Knee Jt, Femoral w Synth Sub, Uncement, Open */
"0SRU0JZ"  =  "1"  /* Replace of L Knee Jt, Femoral with Synth Sub, Open Approach */
"0SRU0KZ"  =  "1"  /* Replace of L Knee Jt, Femoral with Nonaut Sub, Open Approach */
"0SRV07Z"  =  "1"  /* Replace of R Knee Jt, Tibial with Autol Sub, Open Approach */
"0SRV0J9"  =  "1"  /* Replace R Knee Jt, Tibial w Synth Sub, Cement, Open */
"0SRV0JA"  =  "1"  /* Replace R Knee Jt, Tibial w Synth Sub, Uncement, Open */
"0SRV0JZ"  =  "1"  /* Replace of R Knee Jt, Tibial with Synth Sub, Open Approach */
"0SRV0KZ"  =  "1"  /* Replace of R Knee Jt, Tibial with Nonaut Sub, Open Approach */
"0SRW07Z"  =  "1"  /* Replace of L Knee Jt, Tibial with Autol Sub, Open Approach */
"0SRW0J9"  =  "1"  /* Replace L Knee Jt, Tibial w Synth Sub, Cement, Open */
"0SRW0JA"  =  "1"  /* Replace L Knee Jt, Tibial w Synth Sub, Uncement, Open */
"0SRW0JZ"  =  "1"  /* Replace of L Knee Jt, Tibial with Synth Sub, Open Approach */
"0SRW0KZ"  =  "1"  /* Replace of L Knee Jt, Tibial with Nonaut Sub, Open Approach */
"0SS004Z"  =  "1"  /* Reposition Lum Jt with Int Fix, Open Approach */
"0SS00ZZ"  =  "1"  /* Reposition Lumbar Vertebral Joint, Open Approach */
"0SS304Z"  =  "1"  /* Reposition Lumbosacral Joint with Int Fix, Open Approach */
"0SS30ZZ"  =  "1"  /* Reposition Lumbosacral Joint, Open Approach */
"0SS504Z"  =  "1"  /* Reposition Sacrococcygeal Joint with Int Fix, Open Approach */
"0SS50ZZ"  =  "1"  /* Reposition Sacrococcygeal Joint, Open Approach */
"0SS604Z"  =  "1"  /* Reposition Coccygeal Joint with Int Fix, Open Approach */
"0SS60ZZ"  =  "1"  /* Reposition Coccygeal Joint, Open Approach */
"0SS704Z"  =  "1"  /* Reposition R Sacroiliac Jt with Int Fix, Open Approach */
"0SS70ZZ"  =  "1"  /* Reposition Right Sacroiliac Joint, Open Approach */
"0SS804Z"  =  "1"  /* Reposition Left Sacroiliac Joint with Int Fix, Open Approach */
"0SS80ZZ"  =  "1"  /* Reposition Left Sacroiliac Joint, Open Approach */
"0SS904Z"  =  "1"  /* Reposition Right Hip Joint with Int Fix, Open Approach */
"0SS905Z"  =  "1"  /* Reposition Right Hip Joint with Ext Fix, Open Approach */
"0SS90ZZ"  =  "1"  /* Reposition Right Hip Joint, Open Approach */
"0SSB04Z"  =  "1"  /* Reposition Left Hip Joint with Int Fix, Open Approach */
"0SSB05Z"  =  "1"  /* Reposition Left Hip Joint with Ext Fix, Open Approach */
"0SSB0ZZ"  =  "1"  /* Reposition Left Hip Joint, Open Approach */
"0SSC04Z"  =  "1"  /* Reposition Right Knee Joint with Int Fix, Open Approach */
"0SSC05Z"  =  "1"  /* Reposition Right Knee Joint with Ext Fix, Open Approach */
"0SSC0ZZ"  =  "1"  /* Reposition Right Knee Joint, Open Approach */
"0SSD04Z"  =  "1"  /* Reposition Left Knee Joint with Int Fix, Open Approach */
"0SSD05Z"  =  "1"  /* Reposition Left Knee Joint with Ext Fix, Open Approach */
"0SSD0ZZ"  =  "1"  /* Reposition Left Knee Joint, Open Approach */
"0SSF04Z"  =  "1"  /* Reposition Right Ankle Joint with Int Fix, Open Approach */
"0SSF05Z"  =  "1"  /* Reposition Right Ankle Joint with Ext Fix, Open Approach */
"0SSF0ZZ"  =  "1"  /* Reposition Right Ankle Joint, Open Approach */
"0SSG04Z"  =  "1"  /* Reposition Left Ankle Joint with Int Fix, Open Approach */
"0SSG05Z"  =  "1"  /* Reposition Left Ankle Joint with Ext Fix, Open Approach */
"0SSG0ZZ"  =  "1"  /* Reposition Left Ankle Joint, Open Approach */
"0SSH04Z"  =  "1"  /* Reposition Right Tarsal Joint with Int Fix, Open Approach */
"0SSH05Z"  =  "1"  /* Reposition Right Tarsal Joint with Ext Fix, Open Approach */
"0SSH0ZZ"  =  "1"  /* Reposition Right Tarsal Joint, Open Approach */
"0SSJ04Z"  =  "1"  /* Reposition Left Tarsal Joint with Int Fix, Open Approach */
"0SSJ05Z"  =  "1"  /* Reposition Left Tarsal Joint with Ext Fix, Open Approach */
"0SSJ0ZZ"  =  "1"  /* Reposition Left Tarsal Joint, Open Approach */
"0SSK04Z"  =  "1"  /* Reposition R Metatarsotars Jt with Int Fix, Open Approach */
"0SSK05Z"  =  "1"  /* Reposition R Metatarsotars Jt with Ext Fix, Open Approach */
"0SSK0ZZ"  =  "1"  /* Reposition Right Metatarsal-Tarsal Joint, Open Approach */
"0SSL04Z"  =  "1"  /* Reposition L Metatarsotars Jt with Int Fix, Open Approach */
"0SSL05Z"  =  "1"  /* Reposition L Metatarsotars Jt with Ext Fix, Open Approach */
"0SSL0ZZ"  =  "1"  /* Reposition Left Metatarsal-Tarsal Joint, Open Approach */
"0SSM04Z"  =  "1"  /* Reposition R Metatarsophal Jt with Int Fix, Open Approach */
"0SSM05Z"  =  "1"  /* Reposition R Metatarsophal Jt with Ext Fix, Open Approach */
"0SSM0ZZ"  =  "1"  /* Reposition Right Metatarsal-Phalangeal Joint, Open Approach */
"0SSN04Z"  =  "1"  /* Reposition L Metatarsophal Jt with Int Fix, Open Approach */
"0SSN05Z"  =  "1"  /* Reposition L Metatarsophal Jt with Ext Fix, Open Approach */
"0SSN0ZZ"  =  "1"  /* Reposition Left Metatarsal-Phalangeal Joint, Open Approach */
"0SSP04Z"  =  "1"  /* Reposition R Toe Phalanx Jt with Int Fix, Open Approach */
"0SSP05Z"  =  "1"  /* Reposition R Toe Phalanx Jt with Ext Fix, Open Approach */
"0SSP0ZZ"  =  "1"  /* Reposition Right Toe Phalangeal Joint, Open Approach */
"0SSQ04Z"  =  "1"  /* Reposition L Toe Phalanx Jt with Int Fix, Open Approach */
"0SSQ05Z"  =  "1"  /* Reposition L Toe Phalanx Jt with Ext Fix, Open Approach */
"0SSQ0ZZ"  =  "1"  /* Reposition Left Toe Phalangeal Joint, Open Approach */
"0ST20ZZ"  =  "1"  /* Resection of Lumbar Vertebral Disc, Open Approach */
"0ST40ZZ"  =  "1"  /* Resection of Lumbosacral Disc, Open Approach */
"0ST50ZZ"  =  "1"  /* Resection of Sacrococcygeal Joint, Open Approach */
"0ST60ZZ"  =  "1"  /* Resection of Coccygeal Joint, Open Approach */
"0ST70ZZ"  =  "1"  /* Resection of Right Sacroiliac Joint, Open Approach */
"0ST80ZZ"  =  "1"  /* Resection of Left Sacroiliac Joint, Open Approach */
"0ST90ZZ"  =  "1"  /* Resection of Right Hip Joint, Open Approach */
"0STB0ZZ"  =  "1"  /* Resection of Left Hip Joint, Open Approach */
"0STC0ZZ"  =  "1"  /* Resection of Right Knee Joint, Open Approach */
"0STD0ZZ"  =  "1"  /* Resection of Left Knee Joint, Open Approach */
"0STF0ZZ"  =  "1"  /* Resection of Right Ankle Joint, Open Approach */
"0STG0ZZ"  =  "1"  /* Resection of Left Ankle Joint, Open Approach */
"0STH0ZZ"  =  "1"  /* Resection of Right Tarsal Joint, Open Approach */
"0STJ0ZZ"  =  "1"  /* Resection of Left Tarsal Joint, Open Approach */
"0STK0ZZ"  =  "1"  /* Resection of Right Metatarsal-Tarsal Joint, Open Approach */
"0STL0ZZ"  =  "1"  /* Resection of Left Metatarsal-Tarsal Joint, Open Approach */
"0STM0ZZ"  =  "1"  /* Resection of R Metatarsophal Jt, Open Approach */
"0STN0ZZ"  =  "1"  /* Resection of Left Metatarsal-Phalangeal Joint, Open Approach */
"0STP0ZZ"  =  "1"  /* Resection of Right Toe Phalangeal Joint, Open Approach */
"0STQ0ZZ"  =  "1"  /* Resection of Left Toe Phalangeal Joint, Open Approach */
"0SU007Z"  =  "1"  /* Supplement Lum Jt with Autol Sub, Open Approach */
"0SU00JZ"  =  "1"  /* Supplement Lum Jt with Synth Sub, Open Approach */
"0SU00KZ"  =  "1"  /* Supplement Lum Jt with Nonaut Sub, Open Approach */
"0SU037Z"  =  "1"  /* Supplement Lum Jt with Autol Sub, Perc Approach */
"0SU03JZ"  =  "1"  /* Supplement Lum Jt with Synth Sub, Perc Approach */
"0SU03KZ"  =  "1"  /* Supplement Lum Jt with Nonaut Sub, Perc Approach */
"0SU047Z"  =  "1"  /* Supplement Lum Jt with Autol Sub, Perc Endo Approach */
"0SU04JZ"  =  "1"  /* Supplement Lum Jt with Synth Sub, Perc Endo Approach */
"0SU04KZ"  =  "1"  /* Supplement Lum Jt with Nonaut Sub, Perc Endo Approach */
"0SU207Z"  =  "1"  /* Supplement Lum Disc with Autol Sub, Open Approach */
"0SU20JZ"  =  "1"  /* Supplement Lum Disc with Synth Sub, Open Approach */
"0SU20KZ"  =  "1"  /* Supplement Lum Disc with Nonaut Sub, Open Approach */
"0SU237Z"  =  "1"  /* Supplement Lum Disc with Autol Sub, Perc Approach */
"0SU23JZ"  =  "1"  /* Supplement Lum Disc with Synth Sub, Perc Approach */
"0SU23KZ"  =  "1"  /* Supplement Lum Disc with Nonaut Sub, Perc Approach */
"0SU247Z"  =  "1"  /* Supplement Lum Disc with Autol Sub, Perc Endo Approach */
"0SU24JZ"  =  "1"  /* Supplement Lum Disc with Synth Sub, Perc Endo Approach */
"0SU24KZ"  =  "1"  /* Supplement Lum Disc with Nonaut Sub, Perc Endo Approach */
"0SU307Z"  =  "1"  /* Supplement Lumbosacral Joint with Autol Sub, Open Approach */
"0SU30JZ"  =  "1"  /* Supplement Lumbosacral Joint with Synth Sub, Open Approach */
"0SU30KZ"  =  "1"  /* Supplement Lumbosacral Joint with Nonaut Sub, Open Approach */
"0SU337Z"  =  "1"  /* Supplement Lumbosacral Joint with Autol Sub, Perc Approach */
"0SU33JZ"  =  "1"  /* Supplement Lumbosacral Joint with Synth Sub, Perc Approach */
"0SU33KZ"  =  "1"  /* Supplement Lumbosacral Joint with Nonaut Sub, Perc Approach */
"0SU347Z"  =  "1"  /* Supplement Lumsac Jt with Autol Sub, Perc Endo Approach */
"0SU34JZ"  =  "1"  /* Supplement Lumsac Jt with Synth Sub, Perc Endo Approach */
"0SU34KZ"  =  "1"  /* Supplement Lumsac Jt with Nonaut Sub, Perc Endo Approach */
"0SU407Z"  =  "1"  /* Supplement Lumbosacral Disc with Autol Sub, Open Approach */
"0SU40JZ"  =  "1"  /* Supplement Lumbosacral Disc with Synth Sub, Open Approach */
"0SU40KZ"  =  "1"  /* Supplement Lumbosacral Disc with Nonaut Sub, Open Approach */
"0SU437Z"  =  "1"  /* Supplement Lumbosacral Disc with Autol Sub, Perc Approach */
"0SU43JZ"  =  "1"  /* Supplement Lumbosacral Disc with Synth Sub, Perc Approach */
"0SU43KZ"  =  "1"  /* Supplement Lumbosacral Disc with Nonaut Sub, Perc Approach */
"0SU447Z"  =  "1"  /* Supplement Lumsac Disc with Autol Sub, Perc Endo Approach */
"0SU44JZ"  =  "1"  /* Supplement Lumsac Disc with Synth Sub, Perc Endo Approach */
"0SU44KZ"  =  "1"  /* Supplement Lumsac Disc with Nonaut Sub, Perc Endo Approach */
"0SU507Z"  =  "1"  /* Supplement Sacrococcygeal Jt with Autol Sub, Open Approach */
"0SU50JZ"  =  "1"  /* Supplement Sacrococcygeal Jt with Synth Sub, Open Approach */
"0SU50KZ"  =  "1"  /* Supplement Sacrococcygeal Jt with Nonaut Sub, Open Approach */
"0SU537Z"  =  "1"  /* Supplement Sacrococcygeal Jt with Autol Sub, Perc Approach */
"0SU53JZ"  =  "1"  /* Supplement Sacrococcygeal Jt with Synth Sub, Perc Approach */
"0SU53KZ"  =  "1"  /* Supplement Sacrococcygeal Jt with Nonaut Sub, Perc Approach */
"0SU547Z"  =  "1"  /* Supplement Sacrococcygeal Jt w Autol Sub, Perc Endo */
"0SU54JZ"  =  "1"  /* Supplement Sacrococcygeal Jt w Synth Sub, Perc Endo */
"0SU54KZ"  =  "1"  /* Supplement Sacrococcygeal Jt w Nonaut Sub, Perc Endo */
"0SU607Z"  =  "1"  /* Supplement Coccygeal Joint with Autol Sub, Open Approach */
"0SU60JZ"  =  "1"  /* Supplement Coccygeal Joint with Synth Sub, Open Approach */
"0SU60KZ"  =  "1"  /* Supplement Coccygeal Joint with Nonaut Sub, Open Approach */
"0SU637Z"  =  "1"  /* Supplement Coccygeal Joint with Autol Sub, Perc Approach */
"0SU63JZ"  =  "1"  /* Supplement Coccygeal Joint with Synth Sub, Perc Approach */
"0SU63KZ"  =  "1"  /* Supplement Coccygeal Joint with Nonaut Sub, Perc Approach */
"0SU647Z"  =  "1"  /* Supplement Coccygeal Jt with Autol Sub, Perc Endo Approach */
"0SU64JZ"  =  "1"  /* Supplement Coccygeal Jt with Synth Sub, Perc Endo Approach */
"0SU64KZ"  =  "1"  /* Supplement Coccygeal Jt with Nonaut Sub, Perc Endo Approach */
"0SU707Z"  =  "1"  /* Supplement R Sacroiliac Jt with Autol Sub, Open Approach */
"0SU70JZ"  =  "1"  /* Supplement R Sacroiliac Jt with Synth Sub, Open Approach */
"0SU70KZ"  =  "1"  /* Supplement R Sacroiliac Jt with Nonaut Sub, Open Approach */
"0SU737Z"  =  "1"  /* Supplement R Sacroiliac Jt with Autol Sub, Perc Approach */
"0SU73JZ"  =  "1"  /* Supplement R Sacroiliac Jt with Synth Sub, Perc Approach */
"0SU73KZ"  =  "1"  /* Supplement R Sacroiliac Jt with Nonaut Sub, Perc Approach */
"0SU747Z"  =  "1"  /* Supplement R Sacroiliac Jt w Autol Sub, Perc Endo */
"0SU74JZ"  =  "1"  /* Supplement R Sacroiliac Jt w Synth Sub, Perc Endo */
"0SU74KZ"  =  "1"  /* Supplement R Sacroiliac Jt w Nonaut Sub, Perc Endo */
"0SU807Z"  =  "1"  /* Supplement L Sacroiliac Jt with Autol Sub, Open Approach */
"0SU80JZ"  =  "1"  /* Supplement L Sacroiliac Jt with Synth Sub, Open Approach */
"0SU80KZ"  =  "1"  /* Supplement L Sacroiliac Jt with Nonaut Sub, Open Approach */
"0SU837Z"  =  "1"  /* Supplement L Sacroiliac Jt with Autol Sub, Perc Approach */
"0SU83JZ"  =  "1"  /* Supplement L Sacroiliac Jt with Synth Sub, Perc Approach */
"0SU83KZ"  =  "1"  /* Supplement L Sacroiliac Jt with Nonaut Sub, Perc Approach */
"0SU847Z"  =  "1"  /* Supplement L Sacroiliac Jt w Autol Sub, Perc Endo */
"0SU84JZ"  =  "1"  /* Supplement L Sacroiliac Jt w Synth Sub, Perc Endo */
"0SU84KZ"  =  "1"  /* Supplement L Sacroiliac Jt w Nonaut Sub, Perc Endo */
"0SU907Z"  =  "1"  /* Supplement Right Hip Joint with Autol Sub, Open Approach */
"0SU909Z"  =  "1"  /* Supplement Right Hip Joint with Liner, Open Approach */
"0SU90BZ"  =  "1"  /* Supplement Right Hip Joint with Resurf Dev, Open Approach */
"0SU90JZ"  =  "1"  /* Supplement Right Hip Joint with Synth Sub, Open Approach */
"0SU90KZ"  =  "1"  /* Supplement Right Hip Joint with Nonaut Sub, Open Approach */
"0SU937Z"  =  "1"  /* Supplement Right Hip Joint with Autol Sub, Perc Approach */
"0SU93JZ"  =  "1"  /* Supplement Right Hip Joint with Synth Sub, Perc Approach */
"0SU93KZ"  =  "1"  /* Supplement Right Hip Joint with Nonaut Sub, Perc Approach */
"0SU947Z"  =  "1"  /* Supplement R Hip Jt with Autol Sub, Perc Endo Approach */
"0SU94JZ"  =  "1"  /* Supplement R Hip Jt with Synth Sub, Perc Endo Approach */
"0SU94KZ"  =  "1"  /* Supplement R Hip Jt with Nonaut Sub, Perc Endo Approach */
"0SUA09Z"  =  "1"  /* Supplement R Hip Jt, Acetab with Liner, Open Approach */
"0SUA0BZ"  =  "1"  /* Supplement R Hip Jt, Acetab with Resurf Dev, Open Approach */
"0SUB07Z"  =  "1"  /* Supplement Left Hip Joint with Autol Sub, Open Approach */
"0SUB09Z"  =  "1"  /* Supplement Left Hip Joint with Liner, Open Approach */
"0SUB0BZ"  =  "1"  /* Supplement Left Hip Joint with Resurf Dev, Open Approach */
"0SUB0JZ"  =  "1"  /* Supplement Left Hip Joint with Synth Sub, Open Approach */
"0SUB0KZ"  =  "1"  /* Supplement Left Hip Joint with Nonaut Sub, Open Approach */
"0SUB37Z"  =  "1"  /* Supplement Left Hip Joint with Autol Sub, Perc Approach */
"0SUB3JZ"  =  "1"  /* Supplement Left Hip Joint with Synth Sub, Perc Approach */
"0SUB3KZ"  =  "1"  /* Supplement Left Hip Joint with Nonaut Sub, Perc Approach */
"0SUB47Z"  =  "1"  /* Supplement Left Hip Joint with Autol Sub, Perc Endo Approach */
"0SUB4JZ"  =  "1"  /* Supplement Left Hip Joint with Synth Sub, Perc Endo Approach */
"0SUB4KZ"  =  "1"  /* Supplement L Hip Jt with Nonaut Sub, Perc Endo Approach */
"0SUC07Z"  =  "1"  /* Supplement Right Knee Joint with Autol Sub, Open Approach */
"0SUC09C"  =  "1"  /* Supplement R Knee Jt with Liner, Patella, Open Approach */
"0SUC09Z"  =  "1"  /* Supplement Right Knee Joint with Liner, Open Approach */
"0SUC0JZ"  =  "1"  /* Supplement Right Knee Joint with Synth Sub, Open Approach */
"0SUC0KZ"  =  "1"  /* Supplement Right Knee Joint with Nonaut Sub, Open Approach */
"0SUC37Z"  =  "1"  /* Supplement Right Knee Joint with Autol Sub, Perc Approach */
"0SUC3JZ"  =  "1"  /* Supplement Right Knee Joint with Synth Sub, Perc Approach */
"0SUC3KZ"  =  "1"  /* Supplement Right Knee Joint with Nonaut Sub, Perc Approach */
"0SUC47Z"  =  "1"  /* Supplement R Knee Jt with Autol Sub, Perc Endo Approach */
"0SUC4JZ"  =  "1"  /* Supplement R Knee Jt with Synth Sub, Perc Endo Approach */
"0SUC4KZ"  =  "1"  /* Supplement R Knee Jt with Nonaut Sub, Perc Endo Approach */
"0SUD07Z"  =  "1"  /* Supplement Left Knee Joint with Autol Sub, Open Approach */
"0SUD09C"  =  "1"  /* Supplement L Knee Jt with Liner, Patella, Open Approach */
"0SUD09Z"  =  "1"  /* Supplement Left Knee Joint with Liner, Open Approach */
"0SUD0JZ"  =  "1"  /* Supplement Left Knee Joint with Synth Sub, Open Approach */
"0SUD0KZ"  =  "1"  /* Supplement Left Knee Joint with Nonaut Sub, Open Approach */
"0SUD37Z"  =  "1"  /* Supplement Left Knee Joint with Autol Sub, Perc Approach */
"0SUD3JZ"  =  "1"  /* Supplement Left Knee Joint with Synth Sub, Perc Approach */
"0SUD3KZ"  =  "1"  /* Supplement Left Knee Joint with Nonaut Sub, Perc Approach */
"0SUD47Z"  =  "1"  /* Supplement L Knee Jt with Autol Sub, Perc Endo Approach */
"0SUD4JZ"  =  "1"  /* Supplement L Knee Jt with Synth Sub, Perc Endo Approach */
"0SUD4KZ"  =  "1"  /* Supplement L Knee Jt with Nonaut Sub, Perc Endo Approach */
"0SUE09Z"  =  "1"  /* Supplement L Hip Jt, Acetab with Liner, Open Approach */
"0SUE0BZ"  =  "1"  /* Supplement L Hip Jt, Acetab with Resurf Dev, Open Approach */
"0SUF07Z"  =  "1"  /* Supplement Right Ankle Joint with Autol Sub, Open Approach */
"0SUF0JZ"  =  "1"  /* Supplement Right Ankle Joint with Synth Sub, Open Approach */
"0SUF0KZ"  =  "1"  /* Supplement Right Ankle Joint with Nonaut Sub, Open Approach */
"0SUF37Z"  =  "1"  /* Supplement Right Ankle Joint with Autol Sub, Perc Approach */
"0SUF3JZ"  =  "1"  /* Supplement Right Ankle Joint with Synth Sub, Perc Approach */
"0SUF3KZ"  =  "1"  /* Supplement Right Ankle Joint with Nonaut Sub, Perc Approach */
"0SUF47Z"  =  "1"  /* Supplement R Ankle Jt with Autol Sub, Perc Endo Approach */
"0SUF4JZ"  =  "1"  /* Supplement R Ankle Jt with Synth Sub, Perc Endo Approach */
"0SUF4KZ"  =  "1"  /* Supplement R Ankle Jt with Nonaut Sub, Perc Endo Approach */
"0SUG07Z"  =  "1"  /* Supplement Left Ankle Joint with Autol Sub, Open Approach */
"0SUG0JZ"  =  "1"  /* Supplement Left Ankle Joint with Synth Sub, Open Approach */
"0SUG0KZ"  =  "1"  /* Supplement Left Ankle Joint with Nonaut Sub, Open Approach */
"0SUG37Z"  =  "1"  /* Supplement Left Ankle Joint with Autol Sub, Perc Approach */
"0SUG3JZ"  =  "1"  /* Supplement Left Ankle Joint with Synth Sub, Perc Approach */
"0SUG3KZ"  =  "1"  /* Supplement Left Ankle Joint with Nonaut Sub, Perc Approach */
"0SUG47Z"  =  "1"  /* Supplement L Ankle Jt with Autol Sub, Perc Endo Approach */
"0SUG4JZ"  =  "1"  /* Supplement L Ankle Jt with Synth Sub, Perc Endo Approach */
"0SUG4KZ"  =  "1"  /* Supplement L Ankle Jt with Nonaut Sub, Perc Endo Approach */
"0SUH07Z"  =  "1"  /* Supplement Right Tarsal Joint with Autol Sub, Open Approach */
"0SUH0JZ"  =  "1"  /* Supplement Right Tarsal Joint with Synth Sub, Open Approach */
"0SUH0KZ"  =  "1"  /* Supplement Right Tarsal Joint with Nonaut Sub, Open Approach */
"0SUH37Z"  =  "1"  /* Supplement Right Tarsal Joint with Autol Sub, Perc Approach */
"0SUH3JZ"  =  "1"  /* Supplement Right Tarsal Joint with Synth Sub, Perc Approach */
"0SUH3KZ"  =  "1"  /* Supplement Right Tarsal Joint with Nonaut Sub, Perc Approach */
"0SUH47Z"  =  "1"  /* Supplement R Tarsal Jt with Autol Sub, Perc Endo Approach */
"0SUH4JZ"  =  "1"  /* Supplement R Tarsal Jt with Synth Sub, Perc Endo Approach */
"0SUH4KZ"  =  "1"  /* Supplement R Tarsal Jt with Nonaut Sub, Perc Endo Approach */
"0SUJ07Z"  =  "1"  /* Supplement Left Tarsal Joint with Autol Sub, Open Approach */
"0SUJ0JZ"  =  "1"  /* Supplement Left Tarsal Joint with Synth Sub, Open Approach */
"0SUJ0KZ"  =  "1"  /* Supplement Left Tarsal Joint with Nonaut Sub, Open Approach */
"0SUJ37Z"  =  "1"  /* Supplement Left Tarsal Joint with Autol Sub, Perc Approach */
"0SUJ3JZ"  =  "1"  /* Supplement Left Tarsal Joint with Synth Sub, Perc Approach */
"0SUJ3KZ"  =  "1"  /* Supplement Left Tarsal Joint with Nonaut Sub, Perc Approach */
"0SUJ47Z"  =  "1"  /* Supplement L Tarsal Jt with Autol Sub, Perc Endo Approach */
"0SUJ4JZ"  =  "1"  /* Supplement L Tarsal Jt with Synth Sub, Perc Endo Approach */
"0SUJ4KZ"  =  "1"  /* Supplement L Tarsal Jt with Nonaut Sub, Perc Endo Approach */
"0SUK07Z"  =  "1"  /* Supplement R Metatarsotars Jt with Autol Sub, Open Approach */
"0SUK0JZ"  =  "1"  /* Supplement R Metatarsotars Jt with Synth Sub, Open Approach */
"0SUK0KZ"  =  "1"  /* Supplement R Metatarsotars Jt with Nonaut Sub, Open Approach */
"0SUK37Z"  =  "1"  /* Supplement R Metatarsotars Jt with Autol Sub, Perc Approach */
"0SUK3JZ"  =  "1"  /* Supplement R Metatarsotars Jt with Synth Sub, Perc Approach */
"0SUK3KZ"  =  "1"  /* Supplement R Metatarsotars Jt with Nonaut Sub, Perc Approach */
"0SUK47Z"  =  "1"  /* Supplement R Metatarsotars Jt w Autol Sub, Perc Endo */
"0SUK4JZ"  =  "1"  /* Supplement R Metatarsotars Jt w Synth Sub, Perc Endo */
"0SUK4KZ"  =  "1"  /* Supplement R Metatarsotars Jt w Nonaut Sub, Perc Endo */
"0SUL07Z"  =  "1"  /* Supplement L Metatarsotars Jt with Autol Sub, Open Approach */
"0SUL0JZ"  =  "1"  /* Supplement L Metatarsotars Jt with Synth Sub, Open Approach */
"0SUL0KZ"  =  "1"  /* Supplement L Metatarsotars Jt with Nonaut Sub, Open Approach */
"0SUL37Z"  =  "1"  /* Supplement L Metatarsotars Jt with Autol Sub, Perc Approach */
"0SUL3JZ"  =  "1"  /* Supplement L Metatarsotars Jt with Synth Sub, Perc Approach */
"0SUL3KZ"  =  "1"  /* Supplement L Metatarsotars Jt with Nonaut Sub, Perc Approach */
"0SUL47Z"  =  "1"  /* Supplement L Metatarsotars Jt w Autol Sub, Perc Endo */
"0SUL4JZ"  =  "1"  /* Supplement L Metatarsotars Jt w Synth Sub, Perc Endo */
"0SUL4KZ"  =  "1"  /* Supplement L Metatarsotars Jt w Nonaut Sub, Perc Endo */
"0SUM07Z"  =  "1"  /* Supplement R Metatarsophal Jt with Autol Sub, Open Approach */
"0SUM0JZ"  =  "1"  /* Supplement R Metatarsophal Jt with Synth Sub, Open Approach */
"0SUM0KZ"  =  "1"  /* Supplement R Metatarsophal Jt with Nonaut Sub, Open Approach */
"0SUM37Z"  =  "1"  /* Supplement R Metatarsophal Jt with Autol Sub, Perc Approach */
"0SUM3JZ"  =  "1"  /* Supplement R Metatarsophal Jt with Synth Sub, Perc Approach */
"0SUM3KZ"  =  "1"  /* Supplement R Metatarsophal Jt with Nonaut Sub, Perc Approach */
"0SUM47Z"  =  "1"  /* Supplement R Metatarsophal Jt w Autol Sub, Perc Endo */
"0SUM4JZ"  =  "1"  /* Supplement R Metatarsophal Jt w Synth Sub, Perc Endo */
"0SUM4KZ"  =  "1"  /* Supplement R Metatarsophal Jt w Nonaut Sub, Perc Endo */
"0SUN07Z"  =  "1"  /* Supplement L Metatarsophal Jt with Autol Sub, Open Approach */
"0SUN0JZ"  =  "1"  /* Supplement L Metatarsophal Jt with Synth Sub, Open Approach */
"0SUN0KZ"  =  "1"  /* Supplement L Metatarsophal Jt with Nonaut Sub, Open Approach */
"0SUN37Z"  =  "1"  /* Supplement L Metatarsophal Jt with Autol Sub, Perc Approach */
"0SUN3JZ"  =  "1"  /* Supplement L Metatarsophal Jt with Synth Sub, Perc Approach */
"0SUN3KZ"  =  "1"  /* Supplement L Metatarsophal Jt with Nonaut Sub, Perc Approach */
"0SUN47Z"  =  "1"  /* Supplement L Metatarsophal Jt w Autol Sub, Perc Endo */
"0SUN4JZ"  =  "1"  /* Supplement L Metatarsophal Jt w Synth Sub, Perc Endo */
"0SUN4KZ"  =  "1"  /* Supplement L Metatarsophal Jt w Nonaut Sub, Perc Endo */
"0SUP07Z"  =  "1"  /* Supplement R Toe Phalanx Jt with Autol Sub, Open Approach */
"0SUP0JZ"  =  "1"  /* Supplement R Toe Phalanx Jt with Synth Sub, Open Approach */
"0SUP0KZ"  =  "1"  /* Supplement R Toe Phalanx Jt with Nonaut Sub, Open Approach */
"0SUP37Z"  =  "1"  /* Supplement R Toe Phalanx Jt with Autol Sub, Perc Approach */
"0SUP3JZ"  =  "1"  /* Supplement R Toe Phalanx Jt with Synth Sub, Perc Approach */
"0SUP3KZ"  =  "1"  /* Supplement R Toe Phalanx Jt with Nonaut Sub, Perc Approach */
"0SUP47Z"  =  "1"  /* Supplement R Toe Phalanx Jt w Autol Sub, Perc Endo */
"0SUP4JZ"  =  "1"  /* Supplement R Toe Phalanx Jt w Synth Sub, Perc Endo */
"0SUP4KZ"  =  "1"  /* Supplement R Toe Phalanx Jt w Nonaut Sub, Perc Endo */
"0SUQ07Z"  =  "1"  /* Supplement L Toe Phalanx Jt with Autol Sub, Open Approach */
"0SUQ0JZ"  =  "1"  /* Supplement L Toe Phalanx Jt with Synth Sub, Open Approach */
"0SUQ0KZ"  =  "1"  /* Supplement L Toe Phalanx Jt with Nonaut Sub, Open Approach */
"0SUQ37Z"  =  "1"  /* Supplement L Toe Phalanx Jt with Autol Sub, Perc Approach */
"0SUQ3JZ"  =  "1"  /* Supplement L Toe Phalanx Jt with Synth Sub, Perc Approach */
"0SUQ3KZ"  =  "1"  /* Supplement L Toe Phalanx Jt with Nonaut Sub, Perc Approach */
"0SUQ47Z"  =  "1"  /* Supplement L Toe Phalanx Jt w Autol Sub, Perc Endo */
"0SUQ4JZ"  =  "1"  /* Supplement L Toe Phalanx Jt w Synth Sub, Perc Endo */
"0SUQ4KZ"  =  "1"  /* Supplement L Toe Phalanx Jt w Nonaut Sub, Perc Endo */
"0SUR09Z"  =  "1"  /* Supplement R Hip Jt, Femoral with Liner, Open Approach */
"0SUR0BZ"  =  "1"  /* Supplement R Hip Jt, Femoral with Resurf Dev, Open Approach */
"0SUS09Z"  =  "1"  /* Supplement L Hip Jt, Femoral with Liner, Open Approach */
"0SUS0BZ"  =  "1"  /* Supplement L Hip Jt, Femoral with Resurf Dev, Open Approach */
"0SUT09Z"  =  "1"  /* Supplement R Knee Jt, Femoral with Liner, Open Approach */
"0SUU09Z"  =  "1"  /* Supplement L Knee Jt, Femoral with Liner, Open Approach */
"0SUV09Z"  =  "1"  /* Supplement R Knee Jt, Tibial with Liner, Open Approach */
"0SUW09Z"  =  "1"  /* Supplement L Knee Jt, Tibial with Liner, Open Approach */
"0SW000Z"  =  "1"  /* Revision of Drainage Device in Lum Jt, Open Approach */
"0SW003Z"  =  "1"  /* Revision of Infusion Device in Lum Jt, Open Approach */
"0SW004Z"  =  "1"  /* Revision of Int Fix in Lum Jt, Open Approach */
"0SW007Z"  =  "1"  /* Revision of Autol Sub in Lum Jt, Open Approach */
"0SW008Z"  =  "1"  /* Revision of Spacer in Lumbar Vertebral Joint, Open Approach */
"0SW00AZ"  =  "1"  /* Revision of Interbody Fusion Device in Lum Jt, Open Approach */
"0SW00JZ"  =  "1"  /* Revision of Synthetic Substitute in Lum Jt, Open Approach */
"0SW00KZ"  =  "1"  /* Revision of Nonaut Sub in Lum Jt, Open Approach */
"0SW030Z"  =  "1"  /* Revision of Drainage Device in Lum Jt, Perc Approach */
"0SW033Z"  =  "1"  /* Revision of Infusion Device in Lum Jt, Perc Approach */
"0SW034Z"  =  "1"  /* Revision of Int Fix in Lum Jt, Perc Approach */
"0SW037Z"  =  "1"  /* Revision of Autol Sub in Lum Jt, Perc Approach */
"0SW038Z"  =  "1"  /* Revision of Spacer in Lumbar Vertebral Joint, Perc Approach */
"0SW03AZ"  =  "1"  /* Revision of Interbody Fusion Device in Lum Jt, Perc Approach */
"0SW03JZ"  =  "1"  /* Revision of Synthetic Substitute in Lum Jt, Perc Approach */
"0SW03KZ"  =  "1"  /* Revision of Nonaut Sub in Lum Jt, Perc Approach */
"0SW040Z"  =  "1"  /* Revision of Drainage Device in Lum Jt, Perc Endo Approach */
"0SW043Z"  =  "1"  /* Revision of Infusion Device in Lum Jt, Perc Endo Approach */
"0SW044Z"  =  "1"  /* Revision of Int Fix in Lum Jt, Perc Endo Approach */
"0SW047Z"  =  "1"  /* Revision of Autol Sub in Lum Jt, Perc Endo Approach */
"0SW048Z"  =  "1"  /* Revision of Spacer in Lum Jt, Perc Endo Approach */
"0SW04AZ"  =  "1"  /* Revision of Intbd Fus Dev in Lum Jt, Perc Endo Approach */
"0SW04JZ"  =  "1"  /* Revision of Synth Sub in Lum Jt, Perc Endo Approach */
"0SW04KZ"  =  "1"  /* Revision of Nonaut Sub in Lum Jt, Perc Endo Approach */
"0SW200Z"  =  "1"  /* Revision of Drainage Device in Lum Disc, Open Approach */
"0SW203Z"  =  "1"  /* Revision of Infusion Device in Lum Disc, Open Approach */
"0SW207Z"  =  "1"  /* Revision of Autol Sub in Lum Disc, Open Approach */
"0SW20JZ"  =  "1"  /* Revision of Synthetic Substitute in Lum Disc, Open Approach */
"0SW20KZ"  =  "1"  /* Revision of Nonaut Sub in Lum Disc, Open Approach */
"0SW230Z"  =  "1"  /* Revision of Drainage Device in Lum Disc, Perc Approach */
"0SW233Z"  =  "1"  /* Revision of Infusion Device in Lum Disc, Perc Approach */
"0SW237Z"  =  "1"  /* Revision of Autol Sub in Lum Disc, Perc Approach */
"0SW23JZ"  =  "1"  /* Revision of Synthetic Substitute in Lum Disc, Perc Approach */
"0SW23KZ"  =  "1"  /* Revision of Nonaut Sub in Lum Disc, Perc Approach */
"0SW240Z"  =  "1"  /* Revision of Drainage Device in Lum Disc, Perc Endo Approach */
"0SW243Z"  =  "1"  /* Revision of Infusion Device in Lum Disc, Perc Endo Approach */
"0SW247Z"  =  "1"  /* Revision of Autol Sub in Lum Disc, Perc Endo Approach */
"0SW24JZ"  =  "1"  /* Revision of Synth Sub in Lum Disc, Perc Endo Approach */
"0SW24KZ"  =  "1"  /* Revision of Nonaut Sub in Lum Disc, Perc Endo Approach */
"0SW300Z"  =  "1"  /* Revision of Drainage Device in Lumsac Jt, Open Approach */
"0SW303Z"  =  "1"  /* Revision of Infusion Device in Lumsac Jt, Open Approach */
"0SW304Z"  =  "1"  /* Revision of Int Fix in Lumsac Jt, Open Approach */
"0SW307Z"  =  "1"  /* Revision of Autol Sub in Lumsac Jt, Open Approach */
"0SW308Z"  =  "1"  /* Revision of Spacer in Lumbosacral Joint, Open Approach */
"0SW30AZ"  =  "1"  /* Revision of Intbd Fus Dev in Lumsac Jt, Open Approach */
"0SW30JZ"  =  "1"  /* Revision of Synthetic Substitute in Lumsac Jt, Open Approach */
"0SW30KZ"  =  "1"  /* Revision of Nonaut Sub in Lumsac Jt, Open Approach */
"0SW330Z"  =  "1"  /* Revision of Drainage Device in Lumsac Jt, Perc Approach */
"0SW333Z"  =  "1"  /* Revision of Infusion Device in Lumsac Jt, Perc Approach */
"0SW334Z"  =  "1"  /* Revision of Int Fix in Lumsac Jt, Perc Approach */
"0SW337Z"  =  "1"  /* Revision of Autol Sub in Lumsac Jt, Perc Approach */
"0SW338Z"  =  "1"  /* Revision of Spacer in Lumbosacral Joint, Perc Approach */
"0SW33AZ"  =  "1"  /* Revision of Intbd Fus Dev in Lumsac Jt, Perc Approach */
"0SW33JZ"  =  "1"  /* Revision of Synthetic Substitute in Lumsac Jt, Perc Approach */
"0SW33KZ"  =  "1"  /* Revision of Nonaut Sub in Lumsac Jt, Perc Approach */
"0SW340Z"  =  "1"  /* Revision of Drainage Device in Lumsac Jt, Perc Endo Approach */
"0SW343Z"  =  "1"  /* Revision of Infusion Device in Lumsac Jt, Perc Endo Approach */
"0SW344Z"  =  "1"  /* Revision of Int Fix in Lumsac Jt, Perc Endo Approach */
"0SW347Z"  =  "1"  /* Revision of Autol Sub in Lumsac Jt, Perc Endo Approach */
"0SW348Z"  =  "1"  /* Revision of Spacer in Lumbosacral Joint, Perc Endo Approach */
"0SW34AZ"  =  "1"  /* Revision of Intbd Fus Dev in Lumsac Jt, Perc Endo Approach */
"0SW34JZ"  =  "1"  /* Revision of Synth Sub in Lumsac Jt, Perc Endo Approach */
"0SW34KZ"  =  "1"  /* Revision of Nonaut Sub in Lumsac Jt, Perc Endo Approach */
"0SW400Z"  =  "1"  /* Revision of Drainage Device in Lumsac Disc, Open Approach */
"0SW403Z"  =  "1"  /* Revision of Infusion Device in Lumsac Disc, Open Approach */
"0SW407Z"  =  "1"  /* Revision of Autol Sub in Lumsac Disc, Open Approach */
"0SW40JZ"  =  "1"  /* Revision of Synth Sub in Lumsac Disc, Open Approach */
"0SW40KZ"  =  "1"  /* Revision of Nonaut Sub in Lumsac Disc, Open Approach */
"0SW430Z"  =  "1"  /* Revision of Drainage Device in Lumsac Disc, Perc Approach */
"0SW433Z"  =  "1"  /* Revision of Infusion Device in Lumsac Disc, Perc Approach */
"0SW437Z"  =  "1"  /* Revision of Autol Sub in Lumsac Disc, Perc Approach */
"0SW43JZ"  =  "1"  /* Revision of Synth Sub in Lumsac Disc, Perc Approach */
"0SW43KZ"  =  "1"  /* Revision of Nonaut Sub in Lumsac Disc, Perc Approach */
"0SW440Z"  =  "1"  /* Revision of Drain Dev in Lumsac Disc, Perc Endo Approach */
"0SW443Z"  =  "1"  /* Revision of Infusion Dev in Lumsac Disc, Perc Endo Approach */
"0SW447Z"  =  "1"  /* Revision of Autol Sub in Lumsac Disc, Perc Endo Approach */
"0SW44JZ"  =  "1"  /* Revision of Synth Sub in Lumsac Disc, Perc Endo Approach */
"0SW44KZ"  =  "1"  /* Revision of Nonaut Sub in Lumsac Disc, Perc Endo Approach */
"0SW500Z"  =  "1"  /* Revision of Drain Dev in Sacrococcygeal Jt, Open Approach */
"0SW503Z"  =  "1"  /* Revision of Infusion Dev in Sacrococcygeal Jt, Open Approach */
"0SW504Z"  =  "1"  /* Revision of Int Fix in Sacrococcygeal Jt, Open Approach */
"0SW507Z"  =  "1"  /* Revision of Autol Sub in Sacrococcygeal Jt, Open Approach */
"0SW508Z"  =  "1"  /* Revision of Spacer in Sacrococcygeal Joint, Open Approach */
"0SW50JZ"  =  "1"  /* Revision of Synth Sub in Sacrococcygeal Jt, Open Approach */
"0SW50KZ"  =  "1"  /* Revision of Nonaut Sub in Sacrococcygeal Jt, Open Approach */
"0SW530Z"  =  "1"  /* Revision of Drain Dev in Sacrococcygeal Jt, Perc Approach */
"0SW533Z"  =  "1"  /* Revision of Infusion Dev in Sacrococcygeal Jt, Perc Approach */
"0SW534Z"  =  "1"  /* Revision of Int Fix in Sacrococcygeal Jt, Perc Approach */
"0SW537Z"  =  "1"  /* Revision of Autol Sub in Sacrococcygeal Jt, Perc Approach */
"0SW538Z"  =  "1"  /* Revision of Spacer in Sacrococcygeal Joint, Perc Approach */
"0SW53JZ"  =  "1"  /* Revision of Synth Sub in Sacrococcygeal Jt, Perc Approach */
"0SW53KZ"  =  "1"  /* Revision of Nonaut Sub in Sacrococcygeal Jt, Perc Approach */
"0SW540Z"  =  "1"  /* Revise of Drain Dev in Sacrococcygeal Jt, Perc Endo Approach */
"0SW543Z"  =  "1"  /* Revise Infusion Dev in Sacrococcygeal Jt, Perc Endo */
"0SW544Z"  =  "1"  /* Revision of Int Fix in Sacrococcygeal Jt, Perc Endo Approach */
"0SW547Z"  =  "1"  /* Revise of Autol Sub in Sacrococcygeal Jt, Perc Endo Approach */
"0SW548Z"  =  "1"  /* Revision of Spacer in Sacrococcygeal Jt, Perc Endo Approach */
"0SW54JZ"  =  "1"  /* Revise of Synth Sub in Sacrococcygeal Jt, Perc Endo Approach */
"0SW54KZ"  =  "1"  /* Revise Nonaut Sub in Sacrococcygeal Jt, Perc Endo */
"0SW600Z"  =  "1"  /* Revision of Drainage Device in Coccygeal Jt, Open Approach */
"0SW603Z"  =  "1"  /* Revision of Infusion Device in Coccygeal Jt, Open Approach */
"0SW604Z"  =  "1"  /* Revision of Int Fix in Coccygeal Jt, Open Approach */
"0SW607Z"  =  "1"  /* Revision of Autol Sub in Coccygeal Jt, Open Approach */
"0SW608Z"  =  "1"  /* Revision of Spacer in Coccygeal Joint, Open Approach */
"0SW60JZ"  =  "1"  /* Revision of Synth Sub in Coccygeal Jt, Open Approach */
"0SW60KZ"  =  "1"  /* Revision of Nonaut Sub in Coccygeal Jt, Open Approach */
"0SW630Z"  =  "1"  /* Revision of Drainage Device in Coccygeal Jt, Perc Approach */
"0SW633Z"  =  "1"  /* Revision of Infusion Device in Coccygeal Jt, Perc Approach */
"0SW634Z"  =  "1"  /* Revision of Int Fix in Coccygeal Jt, Perc Approach */
"0SW637Z"  =  "1"  /* Revision of Autol Sub in Coccygeal Jt, Perc Approach */
"0SW638Z"  =  "1"  /* Revision of Spacer in Coccygeal Joint, Percutaneous Approach */
"0SW63JZ"  =  "1"  /* Revision of Synth Sub in Coccygeal Jt, Perc Approach */
"0SW63KZ"  =  "1"  /* Revision of Nonaut Sub in Coccygeal Jt, Perc Approach */
"0SW640Z"  =  "1"  /* Revision of Drain Dev in Coccygeal Jt, Perc Endo Approach */
"0SW643Z"  =  "1"  /* Revision of Infusion Dev in Coccygeal Jt, Perc Endo Approach */
"0SW644Z"  =  "1"  /* Revision of Int Fix in Coccygeal Jt, Perc Endo Approach */
"0SW647Z"  =  "1"  /* Revision of Autol Sub in Coccygeal Jt, Perc Endo Approach */
"0SW648Z"  =  "1"  /* Revision of Spacer in Coccygeal Joint, Perc Endo Approach */
"0SW64JZ"  =  "1"  /* Revision of Synth Sub in Coccygeal Jt, Perc Endo Approach */
"0SW64KZ"  =  "1"  /* Revision of Nonaut Sub in Coccygeal Jt, Perc Endo Approach */
"0SW700Z"  =  "1"  /* Revision of Drain Dev in R Sacroiliac Jt, Open Approach */
"0SW703Z"  =  "1"  /* Revision of Infusion Dev in R Sacroiliac Jt, Open Approach */
"0SW704Z"  =  "1"  /* Revision of Int Fix in R Sacroiliac Jt, Open Approach */
"0SW707Z"  =  "1"  /* Revision of Autol Sub in R Sacroiliac Jt, Open Approach */
"0SW708Z"  =  "1"  /* Revision of Spacer in Right Sacroiliac Joint, Open Approach */
"0SW70JZ"  =  "1"  /* Revision of Synth Sub in R Sacroiliac Jt, Open Approach */
"0SW70KZ"  =  "1"  /* Revision of Nonaut Sub in R Sacroiliac Jt, Open Approach */
"0SW730Z"  =  "1"  /* Revision of Drain Dev in R Sacroiliac Jt, Perc Approach */
"0SW733Z"  =  "1"  /* Revision of Infusion Dev in R Sacroiliac Jt, Perc Approach */
"0SW734Z"  =  "1"  /* Revision of Int Fix in R Sacroiliac Jt, Perc Approach */
"0SW737Z"  =  "1"  /* Revision of Autol Sub in R Sacroiliac Jt, Perc Approach */
"0SW738Z"  =  "1"  /* Revision of Spacer in Right Sacroiliac Joint, Perc Approach */
"0SW73JZ"  =  "1"  /* Revision of Synth Sub in R Sacroiliac Jt, Perc Approach */
"0SW73KZ"  =  "1"  /* Revision of Nonaut Sub in R Sacroiliac Jt, Perc Approach */
"0SW740Z"  =  "1"  /* Revision of Drain Dev in R Sacroiliac Jt, Perc Endo Approach */
"0SW743Z"  =  "1"  /* Revise Infusion Dev in R Sacroiliac Jt, Perc Endo */
"0SW744Z"  =  "1"  /* Revision of Int Fix in R Sacroiliac Jt, Perc Endo Approach */
"0SW747Z"  =  "1"  /* Revision of Autol Sub in R Sacroiliac Jt, Perc Endo Approach */
"0SW748Z"  =  "1"  /* Revision of Spacer in R Sacroiliac Jt, Perc Endo Approach */
"0SW74JZ"  =  "1"  /* Revision of Synth Sub in R Sacroiliac Jt, Perc Endo Approach */
"0SW74KZ"  =  "1"  /* Revise of Nonaut Sub in R Sacroiliac Jt, Perc Endo Approach */
"0SW800Z"  =  "1"  /* Revision of Drain Dev in L Sacroiliac Jt, Open Approach */
"0SW803Z"  =  "1"  /* Revision of Infusion Dev in L Sacroiliac Jt, Open Approach */
"0SW804Z"  =  "1"  /* Revision of Int Fix in L Sacroiliac Jt, Open Approach */
"0SW807Z"  =  "1"  /* Revision of Autol Sub in L Sacroiliac Jt, Open Approach */
"0SW808Z"  =  "1"  /* Revision of Spacer in Left Sacroiliac Joint, Open Approach */
"0SW80JZ"  =  "1"  /* Revision of Synth Sub in L Sacroiliac Jt, Open Approach */
"0SW80KZ"  =  "1"  /* Revision of Nonaut Sub in L Sacroiliac Jt, Open Approach */
"0SW830Z"  =  "1"  /* Revision of Drain Dev in L Sacroiliac Jt, Perc Approach */
"0SW833Z"  =  "1"  /* Revision of Infusion Dev in L Sacroiliac Jt, Perc Approach */
"0SW834Z"  =  "1"  /* Revision of Int Fix in L Sacroiliac Jt, Perc Approach */
"0SW837Z"  =  "1"  /* Revision of Autol Sub in L Sacroiliac Jt, Perc Approach */
"0SW838Z"  =  "1"  /* Revision of Spacer in Left Sacroiliac Joint, Perc Approach */
"0SW83JZ"  =  "1"  /* Revision of Synth Sub in L Sacroiliac Jt, Perc Approach */
"0SW83KZ"  =  "1"  /* Revision of Nonaut Sub in L Sacroiliac Jt, Perc Approach */
"0SW840Z"  =  "1"  /* Revision of Drain Dev in L Sacroiliac Jt, Perc Endo Approach */
"0SW843Z"  =  "1"  /* Revise Infusion Dev in L Sacroiliac Jt, Perc Endo */
"0SW844Z"  =  "1"  /* Revision of Int Fix in L Sacroiliac Jt, Perc Endo Approach */
"0SW847Z"  =  "1"  /* Revision of Autol Sub in L Sacroiliac Jt, Perc Endo Approach */
"0SW848Z"  =  "1"  /* Revision of Spacer in L Sacroiliac Jt, Perc Endo Approach */
"0SW84JZ"  =  "1"  /* Revision of Synth Sub in L Sacroiliac Jt, Perc Endo Approach */
"0SW84KZ"  =  "1"  /* Revise of Nonaut Sub in L Sacroiliac Jt, Perc Endo Approach */
"0SW900Z"  =  "1"  /* Revision of Drainage Device in R Hip Jt, Open Approach */
"0SW903Z"  =  "1"  /* Revision of Infusion Device in R Hip Jt, Open Approach */
"0SW904Z"  =  "1"  /* Revision of Int Fix in R Hip Jt, Open Approach */
"0SW905Z"  =  "1"  /* Revision of Ext Fix in R Hip Jt, Open Approach */
"0SW907Z"  =  "1"  /* Revision of Autol Sub in R Hip Jt, Open Approach */
"0SW908Z"  =  "1"  /* Revision of Spacer in Right Hip Joint, Open Approach */
"0SW909Z"  =  "1"  /* Revision of Liner in Right Hip Joint, Open Approach */
"0SW90BZ"  =  "1"  /* Revision of Resurfacing Device in R Hip Jt, Open Approach */
"0SW90JZ"  =  "1"  /* Revision of Synthetic Substitute in R Hip Jt, Open Approach */
"0SW90KZ"  =  "1"  /* Revision of Nonaut Sub in R Hip Jt, Open Approach */
"0SW930Z"  =  "1"  /* Revision of Drainage Device in R Hip Jt, Perc Approach */
"0SW933Z"  =  "1"  /* Revision of Infusion Device in R Hip Jt, Perc Approach */
"0SW934Z"  =  "1"  /* Revision of Int Fix in R Hip Jt, Perc Approach */
"0SW935Z"  =  "1"  /* Revision of Ext Fix in R Hip Jt, Perc Approach */
"0SW937Z"  =  "1"  /* Revision of Autol Sub in R Hip Jt, Perc Approach */
"0SW938Z"  =  "1"  /* Revision of Spacer in Right Hip Joint, Percutaneous Approach */
"0SW93JZ"  =  "1"  /* Revision of Synthetic Substitute in R Hip Jt, Perc Approach */
"0SW93KZ"  =  "1"  /* Revision of Nonaut Sub in R Hip Jt, Perc Approach */
"0SW940Z"  =  "1"  /* Revision of Drainage Device in R Hip Jt, Perc Endo Approach */
"0SW943Z"  =  "1"  /* Revision of Infusion Device in R Hip Jt, Perc Endo Approach */
"0SW944Z"  =  "1"  /* Revision of Int Fix in R Hip Jt, Perc Endo Approach */
"0SW945Z"  =  "1"  /* Revision of Ext Fix in R Hip Jt, Perc Endo Approach */
"0SW947Z"  =  "1"  /* Revision of Autol Sub in R Hip Jt, Perc Endo Approach */
"0SW948Z"  =  "1"  /* Revision of Spacer in Right Hip Joint, Perc Endo Approach */
"0SW94JZ"  =  "1"  /* Revision of Synth Sub in R Hip Jt, Perc Endo Approach */
"0SW94KZ"  =  "1"  /* Revision of Nonaut Sub in R Hip Jt, Perc Endo Approach */
"0SWB00Z"  =  "1"  /* Revision of Drainage Device in Left Hip Joint, Open Approach */
"0SWB03Z"  =  "1"  /* Revision of Infusion Device in Left Hip Joint, Open Approach */
"0SWB04Z"  =  "1"  /* Revision of Int Fix in L Hip Jt, Open Approach */
"0SWB05Z"  =  "1"  /* Revision of Ext Fix in L Hip Jt, Open Approach */
"0SWB07Z"  =  "1"  /* Revision of Autol Sub in L Hip Jt, Open Approach */
"0SWB08Z"  =  "1"  /* Revision of Spacer in Left Hip Joint, Open Approach */
"0SWB09Z"  =  "1"  /* Revision of Liner in Left Hip Joint, Open Approach */
"0SWB0BZ"  =  "1"  /* Revision of Resurfacing Device in L Hip Jt, Open Approach */
"0SWB0JZ"  =  "1"  /* Revision of Synthetic Substitute in L Hip Jt, Open Approach */
"0SWB0KZ"  =  "1"  /* Revision of Nonaut Sub in L Hip Jt, Open Approach */
"0SWB30Z"  =  "1"  /* Revision of Drainage Device in Left Hip Joint, Perc Approach */
"0SWB33Z"  =  "1"  /* Revision of Infusion Device in Left Hip Joint, Perc Approach */
"0SWB34Z"  =  "1"  /* Revision of Int Fix in L Hip Jt, Perc Approach */
"0SWB35Z"  =  "1"  /* Revision of Ext Fix in L Hip Jt, Perc Approach */
"0SWB37Z"  =  "1"  /* Revision of Autol Sub in L Hip Jt, Perc Approach */
"0SWB38Z"  =  "1"  /* Revision of Spacer in Left Hip Joint, Percutaneous Approach */
"0SWB3JZ"  =  "1"  /* Revision of Synthetic Substitute in L Hip Jt, Perc Approach */
"0SWB3KZ"  =  "1"  /* Revision of Nonaut Sub in L Hip Jt, Perc Approach */
"0SWB40Z"  =  "1"  /* Revision of Drainage Device in L Hip Jt, Perc Endo Approach */
"0SWB43Z"  =  "1"  /* Revision of Infusion Device in L Hip Jt, Perc Endo Approach */
"0SWB44Z"  =  "1"  /* Revision of Int Fix in L Hip Jt, Perc Endo Approach */
"0SWB45Z"  =  "1"  /* Revision of Ext Fix in L Hip Jt, Perc Endo Approach */
"0SWB47Z"  =  "1"  /* Revision of Autol Sub in L Hip Jt, Perc Endo Approach */
"0SWB48Z"  =  "1"  /* Revision of Spacer in Left Hip Joint, Perc Endo Approach */
"0SWB4JZ"  =  "1"  /* Revision of Synth Sub in L Hip Jt, Perc Endo Approach */
"0SWB4KZ"  =  "1"  /* Revision of Nonaut Sub in L Hip Jt, Perc Endo Approach */
"0SWC00Z"  =  "1"  /* Revision of Drainage Device in R Knee Jt, Open Approach */
"0SWC03Z"  =  "1"  /* Revision of Infusion Device in R Knee Jt, Open Approach */
"0SWC04Z"  =  "1"  /* Revision of Int Fix in R Knee Jt, Open Approach */
"0SWC05Z"  =  "1"  /* Revision of Ext Fix in R Knee Jt, Open Approach */
"0SWC07Z"  =  "1"  /* Revision of Autol Sub in R Knee Jt, Open Approach */
"0SWC08Z"  =  "1"  /* Revision of Spacer in Right Knee Joint, Open Approach */
"0SWC09Z"  =  "1"  /* Revision of Liner in Right Knee Joint, Open Approach */
"0SWC0JZ"  =  "1"  /* Revision of Synthetic Substitute in R Knee Jt, Open Approach */
"0SWC0KZ"  =  "1"  /* Revision of Nonaut Sub in R Knee Jt, Open Approach */
"0SWC30Z"  =  "1"  /* Revision of Drainage Device in R Knee Jt, Perc Approach */
"0SWC33Z"  =  "1"  /* Revision of Infusion Device in R Knee Jt, Perc Approach */
"0SWC34Z"  =  "1"  /* Revision of Int Fix in R Knee Jt, Perc Approach */
"0SWC35Z"  =  "1"  /* Revision of Ext Fix in R Knee Jt, Perc Approach */
"0SWC37Z"  =  "1"  /* Revision of Autol Sub in R Knee Jt, Perc Approach */
"0SWC38Z"  =  "1"  /* Revision of Spacer in Right Knee Joint, Perc Approach */
"0SWC3JZ"  =  "1"  /* Revision of Synthetic Substitute in R Knee Jt, Perc Approach */
"0SWC3KZ"  =  "1"  /* Revision of Nonaut Sub in R Knee Jt, Perc Approach */
"0SWC40Z"  =  "1"  /* Revision of Drainage Device in R Knee Jt, Perc Endo Approach */
"0SWC43Z"  =  "1"  /* Revision of Infusion Device in R Knee Jt, Perc Endo Approach */
"0SWC44Z"  =  "1"  /* Revision of Int Fix in R Knee Jt, Perc Endo Approach */
"0SWC45Z"  =  "1"  /* Revision of Ext Fix in R Knee Jt, Perc Endo Approach */
"0SWC47Z"  =  "1"  /* Revision of Autol Sub in R Knee Jt, Perc Endo Approach */
"0SWC48Z"  =  "1"  /* Revision of Spacer in Right Knee Joint, Perc Endo Approach */
"0SWC4JZ"  =  "1"  /* Revision of Synth Sub in R Knee Jt, Perc Endo Approach */
"0SWC4KZ"  =  "1"  /* Revision of Nonaut Sub in R Knee Jt, Perc Endo Approach */
"0SWD00Z"  =  "1"  /* Revision of Drainage Device in L Knee Jt, Open Approach */
"0SWD03Z"  =  "1"  /* Revision of Infusion Device in L Knee Jt, Open Approach */
"0SWD04Z"  =  "1"  /* Revision of Int Fix in L Knee Jt, Open Approach */
"0SWD05Z"  =  "1"  /* Revision of Ext Fix in L Knee Jt, Open Approach */
"0SWD07Z"  =  "1"  /* Revision of Autol Sub in L Knee Jt, Open Approach */
"0SWD08Z"  =  "1"  /* Revision of Spacer in Left Knee Joint, Open Approach */
"0SWD09Z"  =  "1"  /* Revision of Liner in Left Knee Joint, Open Approach */
"0SWD0JZ"  =  "1"  /* Revision of Synthetic Substitute in L Knee Jt, Open Approach */
"0SWD0KZ"  =  "1"  /* Revision of Nonaut Sub in L Knee Jt, Open Approach */
"0SWD30Z"  =  "1"  /* Revision of Drainage Device in L Knee Jt, Perc Approach */
"0SWD33Z"  =  "1"  /* Revision of Infusion Device in L Knee Jt, Perc Approach */
"0SWD34Z"  =  "1"  /* Revision of Int Fix in L Knee Jt, Perc Approach */
"0SWD35Z"  =  "1"  /* Revision of Ext Fix in L Knee Jt, Perc Approach */
"0SWD37Z"  =  "1"  /* Revision of Autol Sub in L Knee Jt, Perc Approach */
"0SWD38Z"  =  "1"  /* Revision of Spacer in Left Knee Joint, Percutaneous Approach */
"0SWD3JZ"  =  "1"  /* Revision of Synthetic Substitute in L Knee Jt, Perc Approach */
"0SWD3KZ"  =  "1"  /* Revision of Nonaut Sub in L Knee Jt, Perc Approach */
"0SWD40Z"  =  "1"  /* Revision of Drainage Device in L Knee Jt, Perc Endo Approach */
"0SWD43Z"  =  "1"  /* Revision of Infusion Device in L Knee Jt, Perc Endo Approach */
"0SWD44Z"  =  "1"  /* Revision of Int Fix in L Knee Jt, Perc Endo Approach */
"0SWD45Z"  =  "1"  /* Revision of Ext Fix in L Knee Jt, Perc Endo Approach */
"0SWD47Z"  =  "1"  /* Revision of Autol Sub in L Knee Jt, Perc Endo Approach */
"0SWD48Z"  =  "1"  /* Revision of Spacer in Left Knee Joint, Perc Endo Approach */
"0SWD4JZ"  =  "1"  /* Revision of Synth Sub in L Knee Jt, Perc Endo Approach */
"0SWD4KZ"  =  "1"  /* Revision of Nonaut Sub in L Knee Jt, Perc Endo Approach */
"0SWF00Z"  =  "1"  /* Revision of Drainage Device in R Ankle Jt, Open Approach */
"0SWF03Z"  =  "1"  /* Revision of Infusion Device in R Ankle Jt, Open Approach */
"0SWF04Z"  =  "1"  /* Revision of Int Fix in R Ankle Jt, Open Approach */
"0SWF05Z"  =  "1"  /* Revision of Ext Fix in R Ankle Jt, Open Approach */
"0SWF07Z"  =  "1"  /* Revision of Autol Sub in R Ankle Jt, Open Approach */
"0SWF08Z"  =  "1"  /* Revision of Spacer in Right Ankle Joint, Open Approach */
"0SWF0JZ"  =  "1"  /* Revision of Synth Sub in R Ankle Jt, Open Approach */
"0SWF0KZ"  =  "1"  /* Revision of Nonaut Sub in R Ankle Jt, Open Approach */
"0SWF30Z"  =  "1"  /* Revision of Drainage Device in R Ankle Jt, Perc Approach */
"0SWF33Z"  =  "1"  /* Revision of Infusion Device in R Ankle Jt, Perc Approach */
"0SWF34Z"  =  "1"  /* Revision of Int Fix in R Ankle Jt, Perc Approach */
"0SWF35Z"  =  "1"  /* Revision of Ext Fix in R Ankle Jt, Perc Approach */
"0SWF37Z"  =  "1"  /* Revision of Autol Sub in R Ankle Jt, Perc Approach */
"0SWF38Z"  =  "1"  /* Revision of Spacer in Right Ankle Joint, Perc Approach */
"0SWF3JZ"  =  "1"  /* Revision of Synth Sub in R Ankle Jt, Perc Approach */
"0SWF3KZ"  =  "1"  /* Revision of Nonaut Sub in R Ankle Jt, Perc Approach */
"0SWF40Z"  =  "1"  /* Revision of Drain Dev in R Ankle Jt, Perc Endo Approach */
"0SWF43Z"  =  "1"  /* Revision of Infusion Dev in R Ankle Jt, Perc Endo Approach */
"0SWF44Z"  =  "1"  /* Revision of Int Fix in R Ankle Jt, Perc Endo Approach */
"0SWF45Z"  =  "1"  /* Revision of Ext Fix in R Ankle Jt, Perc Endo Approach */
"0SWF47Z"  =  "1"  /* Revision of Autol Sub in R Ankle Jt, Perc Endo Approach */
"0SWF48Z"  =  "1"  /* Revision of Spacer in Right Ankle Joint, Perc Endo Approach */
"0SWF4JZ"  =  "1"  /* Revision of Synth Sub in R Ankle Jt, Perc Endo Approach */
"0SWF4KZ"  =  "1"  /* Revision of Nonaut Sub in R Ankle Jt, Perc Endo Approach */
"0SWG00Z"  =  "1"  /* Revision of Drainage Device in L Ankle Jt, Open Approach */
"0SWG03Z"  =  "1"  /* Revision of Infusion Device in L Ankle Jt, Open Approach */
"0SWG04Z"  =  "1"  /* Revision of Int Fix in L Ankle Jt, Open Approach */
"0SWG05Z"  =  "1"  /* Revision of Ext Fix in L Ankle Jt, Open Approach */
"0SWG07Z"  =  "1"  /* Revision of Autol Sub in L Ankle Jt, Open Approach */
"0SWG08Z"  =  "1"  /* Revision of Spacer in Left Ankle Joint, Open Approach */
"0SWG0JZ"  =  "1"  /* Revision of Synth Sub in L Ankle Jt, Open Approach */
"0SWG0KZ"  =  "1"  /* Revision of Nonaut Sub in L Ankle Jt, Open Approach */
"0SWG30Z"  =  "1"  /* Revision of Drainage Device in L Ankle Jt, Perc Approach */
"0SWG33Z"  =  "1"  /* Revision of Infusion Device in L Ankle Jt, Perc Approach */
"0SWG34Z"  =  "1"  /* Revision of Int Fix in L Ankle Jt, Perc Approach */
"0SWG35Z"  =  "1"  /* Revision of Ext Fix in L Ankle Jt, Perc Approach */
"0SWG37Z"  =  "1"  /* Revision of Autol Sub in L Ankle Jt, Perc Approach */
"0SWG38Z"  =  "1"  /* Revision of Spacer in Left Ankle Joint, Perc Approach */
"0SWG3JZ"  =  "1"  /* Revision of Synth Sub in L Ankle Jt, Perc Approach */
"0SWG3KZ"  =  "1"  /* Revision of Nonaut Sub in L Ankle Jt, Perc Approach */
"0SWG40Z"  =  "1"  /* Revision of Drain Dev in L Ankle Jt, Perc Endo Approach */
"0SWG43Z"  =  "1"  /* Revision of Infusion Dev in L Ankle Jt, Perc Endo Approach */
"0SWG44Z"  =  "1"  /* Revision of Int Fix in L Ankle Jt, Perc Endo Approach */
"0SWG45Z"  =  "1"  /* Revision of Ext Fix in L Ankle Jt, Perc Endo Approach */
"0SWG47Z"  =  "1"  /* Revision of Autol Sub in L Ankle Jt, Perc Endo Approach */
"0SWG48Z"  =  "1"  /* Revision of Spacer in Left Ankle Joint, Perc Endo Approach */
"0SWG4JZ"  =  "1"  /* Revision of Synth Sub in L Ankle Jt, Perc Endo Approach */
"0SWG4KZ"  =  "1"  /* Revision of Nonaut Sub in L Ankle Jt, Perc Endo Approach */
"0SWH00Z"  =  "1"  /* Revision of Drainage Device in R Tarsal Jt, Open Approach */
"0SWH03Z"  =  "1"  /* Revision of Infusion Device in R Tarsal Jt, Open Approach */
"0SWH04Z"  =  "1"  /* Revision of Int Fix in R Tarsal Jt, Open Approach */
"0SWH05Z"  =  "1"  /* Revision of Ext Fix in R Tarsal Jt, Open Approach */
"0SWH07Z"  =  "1"  /* Revision of Autol Sub in R Tarsal Jt, Open Approach */
"0SWH08Z"  =  "1"  /* Revision of Spacer in Right Tarsal Joint, Open Approach */
"0SWH0JZ"  =  "1"  /* Revision of Synth Sub in R Tarsal Jt, Open Approach */
"0SWH0KZ"  =  "1"  /* Revision of Nonaut Sub in R Tarsal Jt, Open Approach */
"0SWH30Z"  =  "1"  /* Revision of Drainage Device in R Tarsal Jt, Perc Approach */
"0SWH33Z"  =  "1"  /* Revision of Infusion Device in R Tarsal Jt, Perc Approach */
"0SWH34Z"  =  "1"  /* Revision of Int Fix in R Tarsal Jt, Perc Approach */
"0SWH35Z"  =  "1"  /* Revision of Ext Fix in R Tarsal Jt, Perc Approach */
"0SWH37Z"  =  "1"  /* Revision of Autol Sub in R Tarsal Jt, Perc Approach */
"0SWH38Z"  =  "1"  /* Revision of Spacer in Right Tarsal Joint, Perc Approach */
"0SWH3JZ"  =  "1"  /* Revision of Synth Sub in R Tarsal Jt, Perc Approach */
"0SWH3KZ"  =  "1"  /* Revision of Nonaut Sub in R Tarsal Jt, Perc Approach */
"0SWH40Z"  =  "1"  /* Revision of Drain Dev in R Tarsal Jt, Perc Endo Approach */
"0SWH43Z"  =  "1"  /* Revision of Infusion Dev in R Tarsal Jt, Perc Endo Approach */
"0SWH44Z"  =  "1"  /* Revision of Int Fix in R Tarsal Jt, Perc Endo Approach */
"0SWH45Z"  =  "1"  /* Revision of Ext Fix in R Tarsal Jt, Perc Endo Approach */
"0SWH47Z"  =  "1"  /* Revision of Autol Sub in R Tarsal Jt, Perc Endo Approach */
"0SWH48Z"  =  "1"  /* Revision of Spacer in Right Tarsal Joint, Perc Endo Approach */
"0SWH4JZ"  =  "1"  /* Revision of Synth Sub in R Tarsal Jt, Perc Endo Approach */
"0SWH4KZ"  =  "1"  /* Revision of Nonaut Sub in R Tarsal Jt, Perc Endo Approach */
"0SWJ00Z"  =  "1"  /* Revision of Drainage Device in L Tarsal Jt, Open Approach */
"0SWJ03Z"  =  "1"  /* Revision of Infusion Device in L Tarsal Jt, Open Approach */
"0SWJ04Z"  =  "1"  /* Revision of Int Fix in L Tarsal Jt, Open Approach */
"0SWJ05Z"  =  "1"  /* Revision of Ext Fix in L Tarsal Jt, Open Approach */
"0SWJ07Z"  =  "1"  /* Revision of Autol Sub in L Tarsal Jt, Open Approach */
"0SWJ08Z"  =  "1"  /* Revision of Spacer in Left Tarsal Joint, Open Approach */
"0SWJ0JZ"  =  "1"  /* Revision of Synth Sub in L Tarsal Jt, Open Approach */
"0SWJ0KZ"  =  "1"  /* Revision of Nonaut Sub in L Tarsal Jt, Open Approach */
"0SWJ30Z"  =  "1"  /* Revision of Drainage Device in L Tarsal Jt, Perc Approach */
"0SWJ33Z"  =  "1"  /* Revision of Infusion Device in L Tarsal Jt, Perc Approach */
"0SWJ34Z"  =  "1"  /* Revision of Int Fix in L Tarsal Jt, Perc Approach */
"0SWJ35Z"  =  "1"  /* Revision of Ext Fix in L Tarsal Jt, Perc Approach */
"0SWJ37Z"  =  "1"  /* Revision of Autol Sub in L Tarsal Jt, Perc Approach */
"0SWJ38Z"  =  "1"  /* Revision of Spacer in Left Tarsal Joint, Perc Approach */
"0SWJ3JZ"  =  "1"  /* Revision of Synth Sub in L Tarsal Jt, Perc Approach */
"0SWJ3KZ"  =  "1"  /* Revision of Nonaut Sub in L Tarsal Jt, Perc Approach */
"0SWJ40Z"  =  "1"  /* Revision of Drain Dev in L Tarsal Jt, Perc Endo Approach */
"0SWJ43Z"  =  "1"  /* Revision of Infusion Dev in L Tarsal Jt, Perc Endo Approach */
"0SWJ44Z"  =  "1"  /* Revision of Int Fix in L Tarsal Jt, Perc Endo Approach */
"0SWJ45Z"  =  "1"  /* Revision of Ext Fix in L Tarsal Jt, Perc Endo Approach */
"0SWJ47Z"  =  "1"  /* Revision of Autol Sub in L Tarsal Jt, Perc Endo Approach */
"0SWJ48Z"  =  "1"  /* Revision of Spacer in Left Tarsal Joint, Perc Endo Approach */
"0SWJ4JZ"  =  "1"  /* Revision of Synth Sub in L Tarsal Jt, Perc Endo Approach */
"0SWJ4KZ"  =  "1"  /* Revision of Nonaut Sub in L Tarsal Jt, Perc Endo Approach */
"0SWK00Z"  =  "1"  /* Revision of Drain Dev in R Metatarsotars Jt, Open Approach */
"0SWK03Z"  =  "1"  /* Revise of Infusion Dev in R Metatarsotars Jt, Open Approach */
"0SWK04Z"  =  "1"  /* Revision of Int Fix in R Metatarsotars Jt, Open Approach */
"0SWK05Z"  =  "1"  /* Revision of Ext Fix in R Metatarsotars Jt, Open Approach */
"0SWK07Z"  =  "1"  /* Revision of Autol Sub in R Metatarsotars Jt, Open Approach */
"0SWK08Z"  =  "1"  /* Revision of Spacer in R Metatarsotars Jt, Open Approach */
"0SWK0JZ"  =  "1"  /* Revision of Synth Sub in R Metatarsotars Jt, Open Approach */
"0SWK0KZ"  =  "1"  /* Revision of Nonaut Sub in R Metatarsotars Jt, Open Approach */
"0SWK30Z"  =  "1"  /* Revision of Drain Dev in R Metatarsotars Jt, Perc Approach */
"0SWK33Z"  =  "1"  /* Revise of Infusion Dev in R Metatarsotars Jt, Perc Approach */
"0SWK34Z"  =  "1"  /* Revision of Int Fix in R Metatarsotars Jt, Perc Approach */
"0SWK35Z"  =  "1"  /* Revision of Ext Fix in R Metatarsotars Jt, Perc Approach */
"0SWK37Z"  =  "1"  /* Revision of Autol Sub in R Metatarsotars Jt, Perc Approach */
"0SWK38Z"  =  "1"  /* Revision of Spacer in R Metatarsotars Jt, Perc Approach */
"0SWK3JZ"  =  "1"  /* Revision of Synth Sub in R Metatarsotars Jt, Perc Approach */
"0SWK3KZ"  =  "1"  /* Revision of Nonaut Sub in R Metatarsotars Jt, Perc Approach */
"0SWK40Z"  =  "1"  /* Revise Drain Dev in R Metatarsotars Jt, Perc Endo */
"0SWK43Z"  =  "1"  /* Revise Infusion Dev in R Metatarsotars Jt, Perc Endo */
"0SWK44Z"  =  "1"  /* Revise of Int Fix in R Metatarsotars Jt, Perc Endo Approach */
"0SWK45Z"  =  "1"  /* Revise of Ext Fix in R Metatarsotars Jt, Perc Endo Approach */
"0SWK47Z"  =  "1"  /* Revise Autol Sub in R Metatarsotars Jt, Perc Endo */
"0SWK48Z"  =  "1"  /* Revision of Spacer in R Metatarsotars Jt, Perc Endo Approach */
"0SWK4JZ"  =  "1"  /* Revise Synth Sub in R Metatarsotars Jt, Perc Endo */
"0SWK4KZ"  =  "1"  /* Revise Nonaut Sub in R Metatarsotars Jt, Perc Endo */
"0SWL00Z"  =  "1"  /* Revision of Drain Dev in L Metatarsotars Jt, Open Approach */
"0SWL03Z"  =  "1"  /* Revise of Infusion Dev in L Metatarsotars Jt, Open Approach */
"0SWL04Z"  =  "1"  /* Revision of Int Fix in L Metatarsotars Jt, Open Approach */
"0SWL05Z"  =  "1"  /* Revision of Ext Fix in L Metatarsotars Jt, Open Approach */
"0SWL07Z"  =  "1"  /* Revision of Autol Sub in L Metatarsotars Jt, Open Approach */
"0SWL08Z"  =  "1"  /* Revision of Spacer in L Metatarsotars Jt, Open Approach */
"0SWL0JZ"  =  "1"  /* Revision of Synth Sub in L Metatarsotars Jt, Open Approach */
"0SWL0KZ"  =  "1"  /* Revision of Nonaut Sub in L Metatarsotars Jt, Open Approach */
"0SWL30Z"  =  "1"  /* Revision of Drain Dev in L Metatarsotars Jt, Perc Approach */
"0SWL33Z"  =  "1"  /* Revise of Infusion Dev in L Metatarsotars Jt, Perc Approach */
"0SWL34Z"  =  "1"  /* Revision of Int Fix in L Metatarsotars Jt, Perc Approach */
"0SWL35Z"  =  "1"  /* Revision of Ext Fix in L Metatarsotars Jt, Perc Approach */
"0SWL37Z"  =  "1"  /* Revision of Autol Sub in L Metatarsotars Jt, Perc Approach */
"0SWL38Z"  =  "1"  /* Revision of Spacer in L Metatarsotars Jt, Perc Approach */
"0SWL3JZ"  =  "1"  /* Revision of Synth Sub in L Metatarsotars Jt, Perc Approach */
"0SWL3KZ"  =  "1"  /* Revision of Nonaut Sub in L Metatarsotars Jt, Perc Approach */
"0SWL40Z"  =  "1"  /* Revise Drain Dev in L Metatarsotars Jt, Perc Endo */
"0SWL43Z"  =  "1"  /* Revise Infusion Dev in L Metatarsotars Jt, Perc Endo */
"0SWL44Z"  =  "1"  /* Revise of Int Fix in L Metatarsotars Jt, Perc Endo Approach */
"0SWL45Z"  =  "1"  /* Revise of Ext Fix in L Metatarsotars Jt, Perc Endo Approach */
"0SWL47Z"  =  "1"  /* Revise Autol Sub in L Metatarsotars Jt, Perc Endo */
"0SWL48Z"  =  "1"  /* Revision of Spacer in L Metatarsotars Jt, Perc Endo Approach */
"0SWL4JZ"  =  "1"  /* Revise Synth Sub in L Metatarsotars Jt, Perc Endo */
"0SWL4KZ"  =  "1"  /* Revise Nonaut Sub in L Metatarsotars Jt, Perc Endo */
"0SWM00Z"  =  "1"  /* Revision of Drain Dev in R Metatarsophal Jt, Open Approach */
"0SWM03Z"  =  "1"  /* Revise of Infusion Dev in R Metatarsophal Jt, Open Approach */
"0SWM04Z"  =  "1"  /* Revision of Int Fix in R Metatarsophal Jt, Open Approach */
"0SWM05Z"  =  "1"  /* Revision of Ext Fix in R Metatarsophal Jt, Open Approach */
"0SWM07Z"  =  "1"  /* Revision of Autol Sub in R Metatarsophal Jt, Open Approach */
"0SWM08Z"  =  "1"  /* Revision of Spacer in R Metatarsophal Jt, Open Approach */
"0SWM0JZ"  =  "1"  /* Revision of Synth Sub in R Metatarsophal Jt, Open Approach */
"0SWM0KZ"  =  "1"  /* Revision of Nonaut Sub in R Metatarsophal Jt, Open Approach */
"0SWM30Z"  =  "1"  /* Revision of Drain Dev in R Metatarsophal Jt, Perc Approach */
"0SWM33Z"  =  "1"  /* Revise of Infusion Dev in R Metatarsophal Jt, Perc Approach */
"0SWM34Z"  =  "1"  /* Revision of Int Fix in R Metatarsophal Jt, Perc Approach */
"0SWM35Z"  =  "1"  /* Revision of Ext Fix in R Metatarsophal Jt, Perc Approach */
"0SWM37Z"  =  "1"  /* Revision of Autol Sub in R Metatarsophal Jt, Perc Approach */
"0SWM38Z"  =  "1"  /* Revision of Spacer in R Metatarsophal Jt, Perc Approach */
"0SWM3JZ"  =  "1"  /* Revision of Synth Sub in R Metatarsophal Jt, Perc Approach */
"0SWM3KZ"  =  "1"  /* Revision of Nonaut Sub in R Metatarsophal Jt, Perc Approach */
"0SWM40Z"  =  "1"  /* Revise Drain Dev in R Metatarsophal Jt, Perc Endo */
"0SWM43Z"  =  "1"  /* Revise Infusion Dev in R Metatarsophal Jt, Perc Endo */
"0SWM44Z"  =  "1"  /* Revise of Int Fix in R Metatarsophal Jt, Perc Endo Approach */
"0SWM45Z"  =  "1"  /* Revise of Ext Fix in R Metatarsophal Jt, Perc Endo Approach */
"0SWM47Z"  =  "1"  /* Revise Autol Sub in R Metatarsophal Jt, Perc Endo */
"0SWM48Z"  =  "1"  /* Revision of Spacer in R Metatarsophal Jt, Perc Endo Approach */
"0SWM4JZ"  =  "1"  /* Revise Synth Sub in R Metatarsophal Jt, Perc Endo */
"0SWM4KZ"  =  "1"  /* Revise Nonaut Sub in R Metatarsophal Jt, Perc Endo */
"0SWN00Z"  =  "1"  /* Revision of Drain Dev in L Metatarsophal Jt, Open Approach */
"0SWN03Z"  =  "1"  /* Revise of Infusion Dev in L Metatarsophal Jt, Open Approach */
"0SWN04Z"  =  "1"  /* Revision of Int Fix in L Metatarsophal Jt, Open Approach */
"0SWN05Z"  =  "1"  /* Revision of Ext Fix in L Metatarsophal Jt, Open Approach */
"0SWN07Z"  =  "1"  /* Revision of Autol Sub in L Metatarsophal Jt, Open Approach */
"0SWN08Z"  =  "1"  /* Revision of Spacer in L Metatarsophal Jt, Open Approach */
"0SWN0JZ"  =  "1"  /* Revision of Synth Sub in L Metatarsophal Jt, Open Approach */
"0SWN0KZ"  =  "1"  /* Revision of Nonaut Sub in L Metatarsophal Jt, Open Approach */
"0SWN30Z"  =  "1"  /* Revision of Drain Dev in L Metatarsophal Jt, Perc Approach */
"0SWN33Z"  =  "1"  /* Revise of Infusion Dev in L Metatarsophal Jt, Perc Approach */
"0SWN34Z"  =  "1"  /* Revision of Int Fix in L Metatarsophal Jt, Perc Approach */
"0SWN35Z"  =  "1"  /* Revision of Ext Fix in L Metatarsophal Jt, Perc Approach */
"0SWN37Z"  =  "1"  /* Revision of Autol Sub in L Metatarsophal Jt, Perc Approach */
"0SWN38Z"  =  "1"  /* Revision of Spacer in L Metatarsophal Jt, Perc Approach */
"0SWN3JZ"  =  "1"  /* Revision of Synth Sub in L Metatarsophal Jt, Perc Approach */
"0SWN3KZ"  =  "1"  /* Revision of Nonaut Sub in L Metatarsophal Jt, Perc Approach */
"0SWN40Z"  =  "1"  /* Revise Drain Dev in L Metatarsophal Jt, Perc Endo */
"0SWN43Z"  =  "1"  /* Revise Infusion Dev in L Metatarsophal Jt, Perc Endo */
"0SWN44Z"  =  "1"  /* Revise of Int Fix in L Metatarsophal Jt, Perc Endo Approach */
"0SWN45Z"  =  "1"  /* Revise of Ext Fix in L Metatarsophal Jt, Perc Endo Approach */
"0SWN47Z"  =  "1"  /* Revise Autol Sub in L Metatarsophal Jt, Perc Endo */
"0SWN48Z"  =  "1"  /* Revision of Spacer in L Metatarsophal Jt, Perc Endo Approach */
"0SWN4JZ"  =  "1"  /* Revise Synth Sub in L Metatarsophal Jt, Perc Endo */
"0SWN4KZ"  =  "1"  /* Revise Nonaut Sub in L Metatarsophal Jt, Perc Endo */
"0SWP00Z"  =  "1"  /* Revision of Drain Dev in R Toe Phalanx Jt, Open Approach */
"0SWP03Z"  =  "1"  /* Revision of Infusion Dev in R Toe Phalanx Jt, Open Approach */
"0SWP04Z"  =  "1"  /* Revision of Int Fix in R Toe Phalanx Jt, Open Approach */
"0SWP05Z"  =  "1"  /* Revision of Ext Fix in R Toe Phalanx Jt, Open Approach */
"0SWP07Z"  =  "1"  /* Revision of Autol Sub in R Toe Phalanx Jt, Open Approach */
"0SWP08Z"  =  "1"  /* Revision of Spacer in R Toe Phalanx Jt, Open Approach */
"0SWP0JZ"  =  "1"  /* Revision of Synth Sub in R Toe Phalanx Jt, Open Approach */
"0SWP0KZ"  =  "1"  /* Revision of Nonaut Sub in R Toe Phalanx Jt, Open Approach */
"0SWP30Z"  =  "1"  /* Revision of Drain Dev in R Toe Phalanx Jt, Perc Approach */
"0SWP33Z"  =  "1"  /* Revision of Infusion Dev in R Toe Phalanx Jt, Perc Approach */
"0SWP34Z"  =  "1"  /* Revision of Int Fix in R Toe Phalanx Jt, Perc Approach */
"0SWP35Z"  =  "1"  /* Revision of Ext Fix in R Toe Phalanx Jt, Perc Approach */
"0SWP37Z"  =  "1"  /* Revision of Autol Sub in R Toe Phalanx Jt, Perc Approach */
"0SWP38Z"  =  "1"  /* Revision of Spacer in R Toe Phalanx Jt, Perc Approach */
"0SWP3JZ"  =  "1"  /* Revision of Synth Sub in R Toe Phalanx Jt, Perc Approach */
"0SWP3KZ"  =  "1"  /* Revision of Nonaut Sub in R Toe Phalanx Jt, Perc Approach */
"0SWP40Z"  =  "1"  /* Revise of Drain Dev in R Toe Phalanx Jt, Perc Endo Approach */
"0SWP43Z"  =  "1"  /* Revise Infusion Dev in R Toe Phalanx Jt, Perc Endo */
"0SWP44Z"  =  "1"  /* Revision of Int Fix in R Toe Phalanx Jt, Perc Endo Approach */
"0SWP45Z"  =  "1"  /* Revision of Ext Fix in R Toe Phalanx Jt, Perc Endo Approach */
"0SWP47Z"  =  "1"  /* Revise of Autol Sub in R Toe Phalanx Jt, Perc Endo Approach */
"0SWP48Z"  =  "1"  /* Revision of Spacer in R Toe Phalanx Jt, Perc Endo Approach */
"0SWP4JZ"  =  "1"  /* Revise of Synth Sub in R Toe Phalanx Jt, Perc Endo Approach */
"0SWP4KZ"  =  "1"  /* Revise of Nonaut Sub in R Toe Phalanx Jt, Perc Endo Approach */
"0SWQ00Z"  =  "1"  /* Revision of Drain Dev in L Toe Phalanx Jt, Open Approach */
"0SWQ03Z"  =  "1"  /* Revision of Infusion Dev in L Toe Phalanx Jt, Open Approach */
"0SWQ04Z"  =  "1"  /* Revision of Int Fix in L Toe Phalanx Jt, Open Approach */
"0SWQ05Z"  =  "1"  /* Revision of Ext Fix in L Toe Phalanx Jt, Open Approach */
"0SWQ07Z"  =  "1"  /* Revision of Autol Sub in L Toe Phalanx Jt, Open Approach */
"0SWQ08Z"  =  "1"  /* Revision of Spacer in L Toe Phalanx Jt, Open Approach */
"0SWQ0JZ"  =  "1"  /* Revision of Synth Sub in L Toe Phalanx Jt, Open Approach */
"0SWQ0KZ"  =  "1"  /* Revision of Nonaut Sub in L Toe Phalanx Jt, Open Approach */
"0SWQ30Z"  =  "1"  /* Revision of Drain Dev in L Toe Phalanx Jt, Perc Approach */
"0SWQ33Z"  =  "1"  /* Revision of Infusion Dev in L Toe Phalanx Jt, Perc Approach */
"0SWQ34Z"  =  "1"  /* Revision of Int Fix in L Toe Phalanx Jt, Perc Approach */
"0SWQ35Z"  =  "1"  /* Revision of Ext Fix in L Toe Phalanx Jt, Perc Approach */
"0SWQ37Z"  =  "1"  /* Revision of Autol Sub in L Toe Phalanx Jt, Perc Approach */
"0SWQ38Z"  =  "1"  /* Revision of Spacer in L Toe Phalanx Jt, Perc Approach */
"0SWQ3JZ"  =  "1"  /* Revision of Synth Sub in L Toe Phalanx Jt, Perc Approach */
"0SWQ3KZ"  =  "1"  /* Revision of Nonaut Sub in L Toe Phalanx Jt, Perc Approach */
"0SWQ40Z"  =  "1"  /* Revise of Drain Dev in L Toe Phalanx Jt, Perc Endo Approach */
"0SWQ43Z"  =  "1"  /* Revise Infusion Dev in L Toe Phalanx Jt, Perc Endo */
"0SWQ44Z"  =  "1"  /* Revision of Int Fix in L Toe Phalanx Jt, Perc Endo Approach */
"0SWQ45Z"  =  "1"  /* Revision of Ext Fix in L Toe Phalanx Jt, Perc Endo Approach */
"0SWQ47Z"  =  "1"  /* Revise of Autol Sub in L Toe Phalanx Jt, Perc Endo Approach */
"0SWQ48Z"  =  "1"  /* Revision of Spacer in L Toe Phalanx Jt, Perc Endo Approach */
"0SWQ4JZ"  =  "1"  /* Revise of Synth Sub in L Toe Phalanx Jt, Perc Endo Approach */
"0SWQ4KZ"  =  "1"  /* Revise of Nonaut Sub in L Toe Phalanx Jt, Perc Endo Approach */
"0T13073"  =  "1"  /* Bypass R Kidney Pelvis to R Kid Pelv w Autol Sub, Open */
"0T13074"  =  "1"  /* Bypass R Kidney Pelvis to L Kid Pelv w Autol Sub, Open */
"0T13076"  =  "1"  /* Bypass R Kidney Pelvis to R Ureter w Autol Sub, Open */
"0T13077"  =  "1"  /* Bypass R Kidney Pelvis to L Ureter w Autol Sub, Open */
"0T13078"  =  "1"  /* Bypass R Kidney Pelvis to Colon w Autol Sub, Open */
"0T13079"  =  "1"  /* Bypass R Kidney Pelvis to Colocutan w Autol Sub, Open */
"0T1307A"  =  "1"  /* Bypass R Kidney Pelvis to Ileum w Autol Sub, Open */
"0T1307B"  =  "1"  /* Bypass R Kidney Pelvis to Bladder w Autol Sub, Open */
"0T1307C"  =  "1"  /* Bypass R Kidney Pelvis to Ileocutan w Autol Sub, Open */
"0T1307D"  =  "1"  /* Bypass R Kidney Pelvis to Cutan w Autol Sub, Open */
"0T130J3"  =  "1"  /* Bypass R Kidney Pelvis to R Kid Pelv w Synth Sub, Open */
"0T130J4"  =  "1"  /* Bypass R Kidney Pelvis to L Kid Pelv w Synth Sub, Open */
"0T130J6"  =  "1"  /* Bypass R Kidney Pelvis to R Ureter w Synth Sub, Open */
"0T130J7"  =  "1"  /* Bypass R Kidney Pelvis to L Ureter w Synth Sub, Open */
"0T130J8"  =  "1"  /* Bypass R Kidney Pelvis to Colon w Synth Sub, Open */
"0T130J9"  =  "1"  /* Bypass R Kidney Pelvis to Colocutan w Synth Sub, Open */
"0T130JA"  =  "1"  /* Bypass R Kidney Pelvis to Ileum w Synth Sub, Open */
"0T130JB"  =  "1"  /* Bypass R Kidney Pelvis to Bladder w Synth Sub, Open */
"0T130JC"  =  "1"  /* Bypass R Kidney Pelvis to Ileocutan w Synth Sub, Open */
"0T130JD"  =  "1"  /* Bypass R Kidney Pelvis to Cutan w Synth Sub, Open */
"0T130K3"  =  "1"  /* Bypass R Kidney Pelvis to R Kid Pelv w Nonaut Sub, Open */
"0T130K4"  =  "1"  /* Bypass R Kidney Pelvis to L Kid Pelv w Nonaut Sub, Open */
"0T130K6"  =  "1"  /* Bypass R Kidney Pelvis to R Ureter w Nonaut Sub, Open */
"0T130K7"  =  "1"  /* Bypass R Kidney Pelvis to L Ureter w Nonaut Sub, Open */
"0T130K8"  =  "1"  /* Bypass R Kidney Pelvis to Colon w Nonaut Sub, Open */
"0T130K9"  =  "1"  /* Bypass R Kidney Pelvis to Colocutan w Nonaut Sub, Open */
"0T130KA"  =  "1"  /* Bypass R Kidney Pelvis to Ileum w Nonaut Sub, Open */
"0T130KB"  =  "1"  /* Bypass R Kidney Pelvis to Bladder w Nonaut Sub, Open */
"0T130KC"  =  "1"  /* Bypass R Kidney Pelvis to Ileocutan w Nonaut Sub, Open */
"0T130KD"  =  "1"  /* Bypass R Kidney Pelvis to Cutan w Nonaut Sub, Open */
"0T130Z3"  =  "1"  /* Bypass Right Kidney Pelvis to R Kid Pelv, Open Approach */
"0T130Z4"  =  "1"  /* Bypass Right Kidney Pelvis to L Kid Pelv, Open Approach */
"0T130Z6"  =  "1"  /* Bypass Right Kidney Pelvis to Right Ureter, Open Approach */
"0T130Z7"  =  "1"  /* Bypass Right Kidney Pelvis to Left Ureter, Open Approach */
"0T130Z8"  =  "1"  /* Bypass Right Kidney Pelvis to Colon, Open Approach */
"0T130Z9"  =  "1"  /* Bypass Right Kidney Pelvis to Colocutaneous, Open Approach */
"0T130ZA"  =  "1"  /* Bypass Right Kidney Pelvis to Ileum, Open Approach */
"0T130ZB"  =  "1"  /* Bypass Right Kidney Pelvis to Bladder, Open Approach */
"0T130ZC"  =  "1"  /* Bypass Right Kidney Pelvis to Ileocutaneous, Open Approach */
"0T130ZD"  =  "1"  /* Bypass Right Kidney Pelvis to Cutaneous, Open Approach */
"0T133JD"  =  "1"  /* Bypass R Kidney Pelvis to Cutan w Synth Sub, Perc */
"0T13473"  =  "1"  /* Bypass R Kidney Pelvis to R Kid Pelv w Autol Sub, Perc Endo */
"0T13474"  =  "1"  /* Bypass R Kidney Pelvis to L Kid Pelv w Autol Sub, Perc Endo */
"0T13476"  =  "1"  /* Bypass R Kidney Pelvis to R Ureter w Autol Sub, Perc Endo */
"0T13477"  =  "1"  /* Bypass R Kidney Pelvis to L Ureter w Autol Sub, Perc Endo */
"0T13478"  =  "1"  /* Bypass R Kidney Pelvis to Colon w Autol Sub, Perc Endo */
"0T13479"  =  "1"  /* Bypass R Kidney Pelvis to Colocutan w Autol Sub, Perc Endo */
"0T1347A"  =  "1"  /* Bypass R Kidney Pelvis to Ileum w Autol Sub, Perc Endo */
"0T1347B"  =  "1"  /* Bypass R Kidney Pelvis to Bladder w Autol Sub, Perc Endo */
"0T1347C"  =  "1"  /* Bypass R Kidney Pelvis to Ileocutan w Autol Sub, Perc Endo */
"0T1347D"  =  "1"  /* Bypass R Kidney Pelvis to Cutan w Autol Sub, Perc Endo */
"0T134J3"  =  "1"  /* Bypass R Kidney Pelvis to R Kid Pelv w Synth Sub, Perc Endo */
"0T134J4"  =  "1"  /* Bypass R Kidney Pelvis to L Kid Pelv w Synth Sub, Perc Endo */
"0T134J6"  =  "1"  /* Bypass R Kidney Pelvis to R Ureter w Synth Sub, Perc Endo */
"0T134J7"  =  "1"  /* Bypass R Kidney Pelvis to L Ureter w Synth Sub, Perc Endo */
"0T134J8"  =  "1"  /* Bypass R Kidney Pelvis to Colon w Synth Sub, Perc Endo */
"0T134J9"  =  "1"  /* Bypass R Kidney Pelvis to Colocutan w Synth Sub, Perc Endo */
"0T134JA"  =  "1"  /* Bypass R Kidney Pelvis to Ileum w Synth Sub, Perc Endo */
"0T134JB"  =  "1"  /* Bypass R Kidney Pelvis to Bladder w Synth Sub, Perc Endo */
"0T134JC"  =  "1"  /* Bypass R Kidney Pelvis to Ileocutan w Synth Sub, Perc Endo */
"0T134JD"  =  "1"  /* Bypass R Kidney Pelvis to Cutan w Synth Sub, Perc Endo */
"0T134K3"  =  "1"  /* Bypass R Kidney Pelvis to R Kid Pelv w Nonaut Sub, Perc Endo */
"0T134K4"  =  "1"  /* Bypass R Kidney Pelvis to L Kid Pelv w Nonaut Sub, Perc Endo */
"0T134K6"  =  "1"  /* Bypass R Kidney Pelvis to R Ureter w Nonaut Sub, Perc Endo */
"0T134K7"  =  "1"  /* Bypass R Kidney Pelvis to L Ureter w Nonaut Sub, Perc Endo */
"0T134K8"  =  "1"  /* Bypass R Kidney Pelvis to Colon w Nonaut Sub, Perc Endo */
"0T134K9"  =  "1"  /* Bypass R Kidney Pelvis to Colocutan w Nonaut Sub, Perc Endo */
"0T134KA"  =  "1"  /* Bypass R Kidney Pelvis to Ileum w Nonaut Sub, Perc Endo */
"0T134KB"  =  "1"  /* Bypass R Kidney Pelvis to Bladder w Nonaut Sub, Perc Endo */
"0T134KC"  =  "1"  /* Bypass R Kidney Pelvis to Ileocutan w Nonaut Sub, Perc Endo */
"0T134KD"  =  "1"  /* Bypass R Kidney Pelvis to Cutan w Nonaut Sub, Perc Endo */
"0T134Z3"  =  "1"  /* Bypass Right Kidney Pelvis to R Kid Pelv, Perc Endo Approach */
"0T134Z4"  =  "1"  /* Bypass Right Kidney Pelvis to L Kid Pelv, Perc Endo Approach */
"0T134Z6"  =  "1"  /* Bypass Right Kidney Pelvis to R Ureter, Perc Endo Approach */
"0T134Z7"  =  "1"  /* Bypass Right Kidney Pelvis to L Ureter, Perc Endo Approach */
"0T134Z8"  =  "1"  /* Bypass Right Kidney Pelvis to Colon, Perc Endo Approach */
"0T134Z9"  =  "1"  /* Bypass Right Kidney Pelvis to Colocutan, Perc Endo Approach */
"0T134ZA"  =  "1"  /* Bypass Right Kidney Pelvis to Ileum, Perc Endo Approach */
"0T134ZB"  =  "1"  /* Bypass Right Kidney Pelvis to Bladder, Perc Endo Approach */
"0T134ZC"  =  "1"  /* Bypass Right Kidney Pelvis to Ileocutan, Perc Endo Approach */
"0T134ZD"  =  "1"  /* Bypass Right Kidney Pelvis to Cutaneous, Perc Endo Approach */
"0T14073"  =  "1"  /* Bypass L Kidney Pelvis to R Kid Pelv w Autol Sub, Open */
"0T14074"  =  "1"  /* Bypass L Kidney Pelvis to L Kid Pelv w Autol Sub, Open */
"0T14076"  =  "1"  /* Bypass L Kidney Pelvis to R Ureter w Autol Sub, Open */
"0T14077"  =  "1"  /* Bypass L Kidney Pelvis to L Ureter w Autol Sub, Open */
"0T14078"  =  "1"  /* Bypass L Kidney Pelvis to Colon w Autol Sub, Open */
"0T14079"  =  "1"  /* Bypass L Kidney Pelvis to Colocutan w Autol Sub, Open */
"0T1407A"  =  "1"  /* Bypass L Kidney Pelvis to Ileum w Autol Sub, Open */
"0T1407B"  =  "1"  /* Bypass L Kidney Pelvis to Bladder w Autol Sub, Open */
"0T1407C"  =  "1"  /* Bypass L Kidney Pelvis to Ileocutan w Autol Sub, Open */
"0T1407D"  =  "1"  /* Bypass L Kidney Pelvis to Cutan w Autol Sub, Open */
"0T140J3"  =  "1"  /* Bypass L Kidney Pelvis to R Kid Pelv w Synth Sub, Open */
"0T140J4"  =  "1"  /* Bypass L Kidney Pelvis to L Kid Pelv w Synth Sub, Open */
"0T140J6"  =  "1"  /* Bypass L Kidney Pelvis to R Ureter w Synth Sub, Open */
"0T140J7"  =  "1"  /* Bypass L Kidney Pelvis to L Ureter w Synth Sub, Open */
"0T140J8"  =  "1"  /* Bypass L Kidney Pelvis to Colon w Synth Sub, Open */
"0T140J9"  =  "1"  /* Bypass L Kidney Pelvis to Colocutan w Synth Sub, Open */
"0T140JA"  =  "1"  /* Bypass L Kidney Pelvis to Ileum w Synth Sub, Open */
"0T140JB"  =  "1"  /* Bypass L Kidney Pelvis to Bladder w Synth Sub, Open */
"0T140JC"  =  "1"  /* Bypass L Kidney Pelvis to Ileocutan w Synth Sub, Open */
"0T140JD"  =  "1"  /* Bypass L Kidney Pelvis to Cutan w Synth Sub, Open */
"0T140K3"  =  "1"  /* Bypass L Kidney Pelvis to R Kid Pelv w Nonaut Sub, Open */
"0T140K4"  =  "1"  /* Bypass L Kidney Pelvis to L Kid Pelv w Nonaut Sub, Open */
"0T140K6"  =  "1"  /* Bypass L Kidney Pelvis to R Ureter w Nonaut Sub, Open */
"0T140K7"  =  "1"  /* Bypass L Kidney Pelvis to L Ureter w Nonaut Sub, Open */
"0T140K8"  =  "1"  /* Bypass L Kidney Pelvis to Colon w Nonaut Sub, Open */
"0T140K9"  =  "1"  /* Bypass L Kidney Pelvis to Colocutan w Nonaut Sub, Open */
"0T140KA"  =  "1"  /* Bypass L Kidney Pelvis to Ileum w Nonaut Sub, Open */
"0T140KB"  =  "1"  /* Bypass L Kidney Pelvis to Bladder w Nonaut Sub, Open */
"0T140KC"  =  "1"  /* Bypass L Kidney Pelvis to Ileocutan w Nonaut Sub, Open */
"0T140KD"  =  "1"  /* Bypass L Kidney Pelvis to Cutan w Nonaut Sub, Open */
"0T140Z3"  =  "1"  /* Bypass Left Kidney Pelvis to R Kid Pelv, Open Approach */
"0T140Z4"  =  "1"  /* Bypass Left Kidney Pelvis to L Kid Pelv, Open Approach */
"0T140Z6"  =  "1"  /* Bypass Left Kidney Pelvis to Right Ureter, Open Approach */
"0T140Z7"  =  "1"  /* Bypass Left Kidney Pelvis to Left Ureter, Open Approach */
"0T140Z8"  =  "1"  /* Bypass Left Kidney Pelvis to Colon, Open Approach */
"0T140Z9"  =  "1"  /* Bypass Left Kidney Pelvis to Colocutaneous, Open Approach */
"0T140ZA"  =  "1"  /* Bypass Left Kidney Pelvis to Ileum, Open Approach */
"0T140ZB"  =  "1"  /* Bypass Left Kidney Pelvis to Bladder, Open Approach */
"0T140ZC"  =  "1"  /* Bypass Left Kidney Pelvis to Ileocutaneous, Open Approach */
"0T140ZD"  =  "1"  /* Bypass Left Kidney Pelvis to Cutaneous, Open Approach */
"0T143JD"  =  "1"  /* Bypass L Kidney Pelvis to Cutan w Synth Sub, Perc */
"0T14473"  =  "1"  /* Bypass L Kidney Pelvis to R Kid Pelv w Autol Sub, Perc Endo */
"0T14474"  =  "1"  /* Bypass L Kidney Pelvis to L Kid Pelv w Autol Sub, Perc Endo */
"0T14476"  =  "1"  /* Bypass L Kidney Pelvis to R Ureter w Autol Sub, Perc Endo */
"0T14477"  =  "1"  /* Bypass L Kidney Pelvis to L Ureter w Autol Sub, Perc Endo */
"0T14478"  =  "1"  /* Bypass L Kidney Pelvis to Colon w Autol Sub, Perc Endo */
"0T14479"  =  "1"  /* Bypass L Kidney Pelvis to Colocutan w Autol Sub, Perc Endo */
"0T1447A"  =  "1"  /* Bypass L Kidney Pelvis to Ileum w Autol Sub, Perc Endo */
"0T1447B"  =  "1"  /* Bypass L Kidney Pelvis to Bladder w Autol Sub, Perc Endo */
"0T1447C"  =  "1"  /* Bypass L Kidney Pelvis to Ileocutan w Autol Sub, Perc Endo */
"0T1447D"  =  "1"  /* Bypass L Kidney Pelvis to Cutan w Autol Sub, Perc Endo */
"0T144J3"  =  "1"  /* Bypass L Kidney Pelvis to R Kid Pelv w Synth Sub, Perc Endo */
"0T144J4"  =  "1"  /* Bypass L Kidney Pelvis to L Kid Pelv w Synth Sub, Perc Endo */
"0T144J6"  =  "1"  /* Bypass L Kidney Pelvis to R Ureter w Synth Sub, Perc Endo */
"0T144J7"  =  "1"  /* Bypass L Kidney Pelvis to L Ureter w Synth Sub, Perc Endo */
"0T144J8"  =  "1"  /* Bypass L Kidney Pelvis to Colon w Synth Sub, Perc Endo */
"0T144J9"  =  "1"  /* Bypass L Kidney Pelvis to Colocutan w Synth Sub, Perc Endo */
"0T144JA"  =  "1"  /* Bypass L Kidney Pelvis to Ileum w Synth Sub, Perc Endo */
"0T144JB"  =  "1"  /* Bypass L Kidney Pelvis to Bladder w Synth Sub, Perc Endo */
"0T144JC"  =  "1"  /* Bypass L Kidney Pelvis to Ileocutan w Synth Sub, Perc Endo */
"0T144JD"  =  "1"  /* Bypass L Kidney Pelvis to Cutan w Synth Sub, Perc Endo */
"0T144K3"  =  "1"  /* Bypass L Kidney Pelvis to R Kid Pelv w Nonaut Sub, Perc Endo */
"0T144K4"  =  "1"  /* Bypass L Kidney Pelvis to L Kid Pelv w Nonaut Sub, Perc Endo */
"0T144K6"  =  "1"  /* Bypass L Kidney Pelvis to R Ureter w Nonaut Sub, Perc Endo */
"0T144K7"  =  "1"  /* Bypass L Kidney Pelvis to L Ureter w Nonaut Sub, Perc Endo */
"0T144K8"  =  "1"  /* Bypass L Kidney Pelvis to Colon w Nonaut Sub, Perc Endo */
"0T144K9"  =  "1"  /* Bypass L Kidney Pelvis to Colocutan w Nonaut Sub, Perc Endo */
"0T144KA"  =  "1"  /* Bypass L Kidney Pelvis to Ileum w Nonaut Sub, Perc Endo */
"0T144KB"  =  "1"  /* Bypass L Kidney Pelvis to Bladder w Nonaut Sub, Perc Endo */
"0T144KC"  =  "1"  /* Bypass L Kidney Pelvis to Ileocutan w Nonaut Sub, Perc Endo */
"0T144KD"  =  "1"  /* Bypass L Kidney Pelvis to Cutan w Nonaut Sub, Perc Endo */
"0T144Z3"  =  "1"  /* Bypass Left Kidney Pelvis to R Kid Pelv, Perc Endo Approach */
"0T144Z4"  =  "1"  /* Bypass Left Kidney Pelvis to L Kid Pelv, Perc Endo Approach */
"0T144Z6"  =  "1"  /* Bypass Left Kidney Pelvis to R Ureter, Perc Endo Approach */
"0T144Z7"  =  "1"  /* Bypass Left Kidney Pelvis to Left Ureter, Perc Endo Approach */
"0T144Z8"  =  "1"  /* Bypass Left Kidney Pelvis to Colon, Perc Endo Approach */
"0T144Z9"  =  "1"  /* Bypass Left Kidney Pelvis to Colocutan, Perc Endo Approach */
"0T144ZA"  =  "1"  /* Bypass Left Kidney Pelvis to Ileum, Perc Endo Approach */
"0T144ZB"  =  "1"  /* Bypass Left Kidney Pelvis to Bladder, Perc Endo Approach */
"0T144ZC"  =  "1"  /* Bypass Left Kidney Pelvis to Ileocutan, Perc Endo Approach */
"0T144ZD"  =  "1"  /* Bypass Left Kidney Pelvis to Cutaneous, Perc Endo Approach */
"0T16076"  =  "1"  /* Bypass R Ureter to R Ureter with Autol Sub, Open Approach */
"0T16077"  =  "1"  /* Bypass R Ureter to L Ureter with Autol Sub, Open Approach */
"0T16078"  =  "1"  /* Bypass Right Ureter to Colon with Autol Sub, Open Approach */
"0T16079"  =  "1"  /* Bypass R Ureter to Colocutan with Autol Sub, Open Approach */
"0T1607A"  =  "1"  /* Bypass Right Ureter to Ileum with Autol Sub, Open Approach */
"0T1607B"  =  "1"  /* Bypass Right Ureter to Bladder with Autol Sub, Open Approach */
"0T1607C"  =  "1"  /* Bypass R Ureter to Ileocutan with Autol Sub, Open Approach */
"0T1607D"  =  "1"  /* Bypass Right Ureter to Cutan with Autol Sub, Open Approach */
"0T160J6"  =  "1"  /* Bypass R Ureter to R Ureter with Synth Sub, Open Approach */
"0T160J7"  =  "1"  /* Bypass R Ureter to L Ureter with Synth Sub, Open Approach */
"0T160J8"  =  "1"  /* Bypass Right Ureter to Colon with Synth Sub, Open Approach */
"0T160J9"  =  "1"  /* Bypass R Ureter to Colocutan with Synth Sub, Open Approach */
"0T160JA"  =  "1"  /* Bypass Right Ureter to Ileum with Synth Sub, Open Approach */
"0T160JB"  =  "1"  /* Bypass Right Ureter to Bladder with Synth Sub, Open Approach */
"0T160JC"  =  "1"  /* Bypass R Ureter to Ileocutan with Synth Sub, Open Approach */
"0T160JD"  =  "1"  /* Bypass Right Ureter to Cutan with Synth Sub, Open Approach */
"0T160K6"  =  "1"  /* Bypass R Ureter to R Ureter with Nonaut Sub, Open Approach */
"0T160K7"  =  "1"  /* Bypass R Ureter to L Ureter with Nonaut Sub, Open Approach */
"0T160K8"  =  "1"  /* Bypass Right Ureter to Colon with Nonaut Sub, Open Approach */
"0T160K9"  =  "1"  /* Bypass R Ureter to Colocutan with Nonaut Sub, Open Approach */
"0T160KA"  =  "1"  /* Bypass Right Ureter to Ileum with Nonaut Sub, Open Approach */
"0T160KB"  =  "1"  /* Bypass R Ureter to Bladder with Nonaut Sub, Open Approach */
"0T160KC"  =  "1"  /* Bypass R Ureter to Ileocutan with Nonaut Sub, Open Approach */
"0T160KD"  =  "1"  /* Bypass Right Ureter to Cutan with Nonaut Sub, Open Approach */
"0T160Z6"  =  "1"  /* Bypass Right Ureter to Right Ureter, Open Approach */
"0T160Z7"  =  "1"  /* Bypass Right Ureter to Left Ureter, Open Approach */
"0T160Z8"  =  "1"  /* Bypass Right Ureter to Colon, Open Approach */
"0T160Z9"  =  "1"  /* Bypass Right Ureter to Colocutaneous, Open Approach */
"0T160ZA"  =  "1"  /* Bypass Right Ureter to Ileum, Open Approach */
"0T160ZB"  =  "1"  /* Bypass Right Ureter to Bladder, Open Approach */
"0T160ZC"  =  "1"  /* Bypass Right Ureter to Ileocutaneous, Open Approach */
"0T160ZD"  =  "1"  /* Bypass Right Ureter to Cutaneous, Open Approach */
"0T163JD"  =  "1"  /* Bypass Right Ureter to Cutan with Synth Sub, Perc Approach */
"0T16476"  =  "1"  /* Bypass R Ureter to R Ureter w Autol Sub, Perc Endo */
"0T16477"  =  "1"  /* Bypass R Ureter to L Ureter w Autol Sub, Perc Endo */
"0T16478"  =  "1"  /* Bypass R Ureter to Colon with Autol Sub, Perc Endo Approach */
"0T16479"  =  "1"  /* Bypass R Ureter to Colocutan w Autol Sub, Perc Endo */
"0T1647A"  =  "1"  /* Bypass R Ureter to Ileum with Autol Sub, Perc Endo Approach */
"0T1647B"  =  "1"  /* Bypass R Ureter to Bladder w Autol Sub, Perc Endo */
"0T1647C"  =  "1"  /* Bypass R Ureter to Ileocutan w Autol Sub, Perc Endo */
"0T1647D"  =  "1"  /* Bypass R Ureter to Cutan with Autol Sub, Perc Endo Approach */
"0T164J6"  =  "1"  /* Bypass R Ureter to R Ureter w Synth Sub, Perc Endo */
"0T164J7"  =  "1"  /* Bypass R Ureter to L Ureter w Synth Sub, Perc Endo */
"0T164J8"  =  "1"  /* Bypass R Ureter to Colon with Synth Sub, Perc Endo Approach */
"0T164J9"  =  "1"  /* Bypass R Ureter to Colocutan w Synth Sub, Perc Endo */
"0T164JA"  =  "1"  /* Bypass R Ureter to Ileum with Synth Sub, Perc Endo Approach */
"0T164JB"  =  "1"  /* Bypass R Ureter to Bladder w Synth Sub, Perc Endo */
"0T164JC"  =  "1"  /* Bypass R Ureter to Ileocutan w Synth Sub, Perc Endo */
"0T164JD"  =  "1"  /* Bypass R Ureter to Cutan with Synth Sub, Perc Endo Approach */
"0T164K6"  =  "1"  /* Bypass R Ureter to R Ureter w Nonaut Sub, Perc Endo */
"0T164K7"  =  "1"  /* Bypass R Ureter to L Ureter w Nonaut Sub, Perc Endo */
"0T164K8"  =  "1"  /* Bypass R Ureter to Colon with Nonaut Sub, Perc Endo Approach */
"0T164K9"  =  "1"  /* Bypass R Ureter to Colocutan w Nonaut Sub, Perc Endo */
"0T164KA"  =  "1"  /* Bypass R Ureter to Ileum with Nonaut Sub, Perc Endo Approach */
"0T164KB"  =  "1"  /* Bypass R Ureter to Bladder w Nonaut Sub, Perc Endo */
"0T164KC"  =  "1"  /* Bypass R Ureter to Ileocutan w Nonaut Sub, Perc Endo */
"0T164KD"  =  "1"  /* Bypass R Ureter to Cutan with Nonaut Sub, Perc Endo Approach */
"0T164Z6"  =  "1"  /* Bypass Right Ureter to Right Ureter, Perc Endo Approach */
"0T164Z7"  =  "1"  /* Bypass Right Ureter to Left Ureter, Perc Endo Approach */
"0T164Z8"  =  "1"  /* Bypass Right Ureter to Colon, Perc Endo Approach */
"0T164Z9"  =  "1"  /* Bypass Right Ureter to Colocutaneous, Perc Endo Approach */
"0T164ZA"  =  "1"  /* Bypass Right Ureter to Ileum, Perc Endo Approach */
"0T164ZB"  =  "1"  /* Bypass Right Ureter to Bladder, Perc Endo Approach */
"0T164ZC"  =  "1"  /* Bypass Right Ureter to Ileocutaneous, Perc Endo Approach */
"0T164ZD"  =  "1"  /* Bypass Right Ureter to Cutaneous, Perc Endo Approach */
"0T17076"  =  "1"  /* Bypass Left Ureter to R Ureter with Autol Sub, Open Approach */
"0T17077"  =  "1"  /* Bypass Left Ureter to L Ureter with Autol Sub, Open Approach */
"0T17078"  =  "1"  /* Bypass Left Ureter to Colon with Autol Sub, Open Approach */
"0T17079"  =  "1"  /* Bypass L Ureter to Colocutan with Autol Sub, Open Approach */
"0T1707A"  =  "1"  /* Bypass Left Ureter to Ileum with Autol Sub, Open Approach */
"0T1707B"  =  "1"  /* Bypass Left Ureter to Bladder with Autol Sub, Open Approach */
"0T1707C"  =  "1"  /* Bypass L Ureter to Ileocutan with Autol Sub, Open Approach */
"0T1707D"  =  "1"  /* Bypass Left Ureter to Cutan with Autol Sub, Open Approach */
"0T170J6"  =  "1"  /* Bypass Left Ureter to R Ureter with Synth Sub, Open Approach */
"0T170J7"  =  "1"  /* Bypass Left Ureter to L Ureter with Synth Sub, Open Approach */
"0T170J8"  =  "1"  /* Bypass Left Ureter to Colon with Synth Sub, Open Approach */
"0T170J9"  =  "1"  /* Bypass L Ureter to Colocutan with Synth Sub, Open Approach */
"0T170JA"  =  "1"  /* Bypass Left Ureter to Ileum with Synth Sub, Open Approach */
"0T170JB"  =  "1"  /* Bypass Left Ureter to Bladder with Synth Sub, Open Approach */
"0T170JC"  =  "1"  /* Bypass L Ureter to Ileocutan with Synth Sub, Open Approach */
"0T170JD"  =  "1"  /* Bypass Left Ureter to Cutan with Synth Sub, Open Approach */
"0T170K6"  =  "1"  /* Bypass L Ureter to R Ureter with Nonaut Sub, Open Approach */
"0T170K7"  =  "1"  /* Bypass L Ureter to L Ureter with Nonaut Sub, Open Approach */
"0T170K8"  =  "1"  /* Bypass Left Ureter to Colon with Nonaut Sub, Open Approach */
"0T170K9"  =  "1"  /* Bypass L Ureter to Colocutan with Nonaut Sub, Open Approach */
"0T170KA"  =  "1"  /* Bypass Left Ureter to Ileum with Nonaut Sub, Open Approach */
"0T170KB"  =  "1"  /* Bypass Left Ureter to Bladder with Nonaut Sub, Open Approach */
"0T170KC"  =  "1"  /* Bypass L Ureter to Ileocutan with Nonaut Sub, Open Approach */
"0T170KD"  =  "1"  /* Bypass Left Ureter to Cutan with Nonaut Sub, Open Approach */
"0T170Z6"  =  "1"  /* Bypass Left Ureter to Right Ureter, Open Approach */
"0T170Z7"  =  "1"  /* Bypass Left Ureter to Left Ureter, Open Approach */
"0T170Z8"  =  "1"  /* Bypass Left Ureter to Colon, Open Approach */
"0T170Z9"  =  "1"  /* Bypass Left Ureter to Colocutaneous, Open Approach */
"0T170ZA"  =  "1"  /* Bypass Left Ureter to Ileum, Open Approach */
"0T170ZB"  =  "1"  /* Bypass Left Ureter to Bladder, Open Approach */
"0T170ZC"  =  "1"  /* Bypass Left Ureter to Ileocutaneous, Open Approach */
"0T170ZD"  =  "1"  /* Bypass Left Ureter to Cutaneous, Open Approach */
"0T173JD"  =  "1"  /* Bypass Left Ureter to Cutan with Synth Sub, Perc Approach */
"0T17476"  =  "1"  /* Bypass L Ureter to R Ureter w Autol Sub, Perc Endo */
"0T17477"  =  "1"  /* Bypass L Ureter to L Ureter w Autol Sub, Perc Endo */
"0T17478"  =  "1"  /* Bypass L Ureter to Colon with Autol Sub, Perc Endo Approach */
"0T17479"  =  "1"  /* Bypass L Ureter to Colocutan w Autol Sub, Perc Endo */
"0T1747A"  =  "1"  /* Bypass L Ureter to Ileum with Autol Sub, Perc Endo Approach */
"0T1747B"  =  "1"  /* Bypass L Ureter to Bladder w Autol Sub, Perc Endo */
"0T1747C"  =  "1"  /* Bypass L Ureter to Ileocutan w Autol Sub, Perc Endo */
"0T1747D"  =  "1"  /* Bypass L Ureter to Cutan with Autol Sub, Perc Endo Approach */
"0T174J6"  =  "1"  /* Bypass L Ureter to R Ureter w Synth Sub, Perc Endo */
"0T174J7"  =  "1"  /* Bypass L Ureter to L Ureter w Synth Sub, Perc Endo */
"0T174J8"  =  "1"  /* Bypass L Ureter to Colon with Synth Sub, Perc Endo Approach */
"0T174J9"  =  "1"  /* Bypass L Ureter to Colocutan w Synth Sub, Perc Endo */
"0T174JA"  =  "1"  /* Bypass L Ureter to Ileum with Synth Sub, Perc Endo Approach */
"0T174JB"  =  "1"  /* Bypass L Ureter to Bladder w Synth Sub, Perc Endo */
"0T174JC"  =  "1"  /* Bypass L Ureter to Ileocutan w Synth Sub, Perc Endo */
"0T174JD"  =  "1"  /* Bypass L Ureter to Cutan with Synth Sub, Perc Endo Approach */
"0T174K6"  =  "1"  /* Bypass L Ureter to R Ureter w Nonaut Sub, Perc Endo */
"0T174K7"  =  "1"  /* Bypass L Ureter to L Ureter w Nonaut Sub, Perc Endo */
"0T174K8"  =  "1"  /* Bypass L Ureter to Colon with Nonaut Sub, Perc Endo Approach */
"0T174K9"  =  "1"  /* Bypass L Ureter to Colocutan w Nonaut Sub, Perc Endo */
"0T174KA"  =  "1"  /* Bypass L Ureter to Ileum with Nonaut Sub, Perc Endo Approach */
"0T174KB"  =  "1"  /* Bypass L Ureter to Bladder w Nonaut Sub, Perc Endo */
"0T174KC"  =  "1"  /* Bypass L Ureter to Ileocutan w Nonaut Sub, Perc Endo */
"0T174KD"  =  "1"  /* Bypass L Ureter to Cutan with Nonaut Sub, Perc Endo Approach */
"0T174Z6"  =  "1"  /* Bypass Left Ureter to Right Ureter, Perc Endo Approach */
"0T174Z7"  =  "1"  /* Bypass Left Ureter to Left Ureter, Perc Endo Approach */
"0T174Z8"  =  "1"  /* Bypass Left Ureter to Colon, Perc Endo Approach */
"0T174Z9"  =  "1"  /* Bypass Left Ureter to Colocutaneous, Perc Endo Approach */
"0T174ZA"  =  "1"  /* Bypass Left Ureter to Ileum, Perc Endo Approach */
"0T174ZB"  =  "1"  /* Bypass Left Ureter to Bladder, Perc Endo Approach */
"0T174ZC"  =  "1"  /* Bypass Left Ureter to Ileocutaneous, Perc Endo Approach */
"0T174ZD"  =  "1"  /* Bypass Left Ureter to Cutaneous, Perc Endo Approach */
"0T18076"  =  "1"  /* Bypass Bi Ureter to R Ureter with Autol Sub, Open Approach */
"0T18077"  =  "1"  /* Bypass Bi Ureter to L Ureter with Autol Sub, Open Approach */
"0T18078"  =  "1"  /* Bypass Bi Ureter to Colon with Autol Sub, Open Approach */
"0T18079"  =  "1"  /* Bypass Bi Ureter to Colocutan with Autol Sub, Open Approach */
"0T1807A"  =  "1"  /* Bypass Bi Ureter to Ileum with Autol Sub, Open Approach */
"0T1807B"  =  "1"  /* Bypass Bi Ureter to Bladder with Autol Sub, Open Approach */
"0T1807C"  =  "1"  /* Bypass Bi Ureter to Ileocutan with Autol Sub, Open Approach */
"0T1807D"  =  "1"  /* Bypass Bi Ureter to Cutan with Autol Sub, Open Approach */
"0T180J6"  =  "1"  /* Bypass Bi Ureter to R Ureter with Synth Sub, Open Approach */
"0T180J7"  =  "1"  /* Bypass Bi Ureter to L Ureter with Synth Sub, Open Approach */
"0T180J8"  =  "1"  /* Bypass Bi Ureter to Colon with Synth Sub, Open Approach */
"0T180J9"  =  "1"  /* Bypass Bi Ureter to Colocutan with Synth Sub, Open Approach */
"0T180JA"  =  "1"  /* Bypass Bi Ureter to Ileum with Synth Sub, Open Approach */
"0T180JB"  =  "1"  /* Bypass Bi Ureter to Bladder with Synth Sub, Open Approach */
"0T180JC"  =  "1"  /* Bypass Bi Ureter to Ileocutan with Synth Sub, Open Approach */
"0T180JD"  =  "1"  /* Bypass Bi Ureter to Cutan with Synth Sub, Open Approach */
"0T180K6"  =  "1"  /* Bypass Bi Ureter to R Ureter with Nonaut Sub, Open Approach */
"0T180K7"  =  "1"  /* Bypass Bi Ureter to L Ureter with Nonaut Sub, Open Approach */
"0T180K8"  =  "1"  /* Bypass Bi Ureter to Colon with Nonaut Sub, Open Approach */
"0T180K9"  =  "1"  /* Bypass Bi Ureter to Colocutan with Nonaut Sub, Open Approach */
"0T180KA"  =  "1"  /* Bypass Bi Ureter to Ileum with Nonaut Sub, Open Approach */
"0T180KB"  =  "1"  /* Bypass Bi Ureter to Bladder with Nonaut Sub, Open Approach */
"0T180KC"  =  "1"  /* Bypass Bi Ureter to Ileocutan with Nonaut Sub, Open Approach */
"0T180KD"  =  "1"  /* Bypass Bi Ureter to Cutan with Nonaut Sub, Open Approach */
"0T180Z6"  =  "1"  /* Bypass Bilateral Ureters to Right Ureter, Open Approach */
"0T180Z7"  =  "1"  /* Bypass Bilateral Ureters to Left Ureter, Open Approach */
"0T180Z8"  =  "1"  /* Bypass Bilateral Ureters to Colon, Open Approach */
"0T180Z9"  =  "1"  /* Bypass Bilateral Ureters to Colocutaneous, Open Approach */
"0T180ZA"  =  "1"  /* Bypass Bilateral Ureters to Ileum, Open Approach */
"0T180ZB"  =  "1"  /* Bypass Bilateral Ureters to Bladder, Open Approach */
"0T180ZC"  =  "1"  /* Bypass Bilateral Ureters to Ileocutaneous, Open Approach */
"0T180ZD"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous, Open Approach */
"0T183JD"  =  "1"  /* Bypass Bi Ureter to Cutan with Synth Sub, Perc Approach */
"0T18476"  =  "1"  /* Bypass Bi Ureter to R Ureter w Autol Sub, Perc Endo */
"0T18477"  =  "1"  /* Bypass Bi Ureter to L Ureter w Autol Sub, Perc Endo */
"0T18478"  =  "1"  /* Bypass Bi Ureter to Colon with Autol Sub, Perc Endo Approach */
"0T18479"  =  "1"  /* Bypass Bi Ureter to Colocutan w Autol Sub, Perc Endo */
"0T1847A"  =  "1"  /* Bypass Bi Ureter to Ileum with Autol Sub, Perc Endo Approach */
"0T1847B"  =  "1"  /* Bypass Bi Ureter to Bladder w Autol Sub, Perc Endo */
"0T1847C"  =  "1"  /* Bypass Bi Ureter to Ileocutan w Autol Sub, Perc Endo */
"0T1847D"  =  "1"  /* Bypass Bi Ureter to Cutan with Autol Sub, Perc Endo Approach */
"0T184J6"  =  "1"  /* Bypass Bi Ureter to R Ureter w Synth Sub, Perc Endo */
"0T184J7"  =  "1"  /* Bypass Bi Ureter to L Ureter w Synth Sub, Perc Endo */
"0T184J8"  =  "1"  /* Bypass Bi Ureter to Colon with Synth Sub, Perc Endo Approach */
"0T184J9"  =  "1"  /* Bypass Bi Ureter to Colocutan w Synth Sub, Perc Endo */
"0T184JA"  =  "1"  /* Bypass Bi Ureter to Ileum with Synth Sub, Perc Endo Approach */
"0T184JB"  =  "1"  /* Bypass Bi Ureter to Bladder w Synth Sub, Perc Endo */
"0T184JC"  =  "1"  /* Bypass Bi Ureter to Ileocutan w Synth Sub, Perc Endo */
"0T184JD"  =  "1"  /* Bypass Bi Ureter to Cutan with Synth Sub, Perc Endo Approach */
"0T184K6"  =  "1"  /* Bypass Bi Ureter to R Ureter w Nonaut Sub, Perc Endo */
"0T184K7"  =  "1"  /* Bypass Bi Ureter to L Ureter w Nonaut Sub, Perc Endo */
"0T184K8"  =  "1"  /* Bypass Bi Ureter to Colon w Nonaut Sub, Perc Endo */
"0T184K9"  =  "1"  /* Bypass Bi Ureter to Colocutan w Nonaut Sub, Perc Endo */
"0T184KA"  =  "1"  /* Bypass Bi Ureter to Ileum w Nonaut Sub, Perc Endo */
"0T184KB"  =  "1"  /* Bypass Bi Ureter to Bladder w Nonaut Sub, Perc Endo */
"0T184KC"  =  "1"  /* Bypass Bi Ureter to Ileocutan w Nonaut Sub, Perc Endo */
"0T184KD"  =  "1"  /* Bypass Bi Ureter to Cutan w Nonaut Sub, Perc Endo */
"0T184Z6"  =  "1"  /* Bypass Bilateral Ureters to Right Ureter, Perc Endo Approach */
"0T184Z7"  =  "1"  /* Bypass Bilateral Ureters to Left Ureter, Perc Endo Approach */
"0T184Z8"  =  "1"  /* Bypass Bilateral Ureters to Colon, Perc Endo Approach */
"0T184Z9"  =  "1"  /* Bypass Bilateral Ureters to Colocutan, Perc Endo Approach */
"0T184ZA"  =  "1"  /* Bypass Bilateral Ureters to Ileum, Perc Endo Approach */
"0T184ZB"  =  "1"  /* Bypass Bilateral Ureters to Bladder, Perc Endo Approach */
"0T184ZC"  =  "1"  /* Bypass Bilateral Ureters to Ileocutan, Perc Endo Approach */
"0T184ZD"  =  "1"  /* Bypass Bilateral Ureters to Cutaneous, Perc Endo Approach */
"0T1B079"  =  "1"  /* Bypass Bladder to Colocutan with Autol Sub, Open Approach */
"0T1B07C"  =  "1"  /* Bypass Bladder to Ileocutan with Autol Sub, Open Approach */
"0T1B07D"  =  "1"  /* Bypass Bladder to Cutaneous with Autol Sub, Open Approach */
"0T1B0J9"  =  "1"  /* Bypass Bladder to Colocutan with Synth Sub, Open Approach */
"0T1B0JC"  =  "1"  /* Bypass Bladder to Ileocutan with Synth Sub, Open Approach */
"0T1B0JD"  =  "1"  /* Bypass Bladder to Cutaneous with Synth Sub, Open Approach */
"0T1B0K9"  =  "1"  /* Bypass Bladder to Colocutan with Nonaut Sub, Open Approach */
"0T1B0KC"  =  "1"  /* Bypass Bladder to Ileocutan with Nonaut Sub, Open Approach */
"0T1B0KD"  =  "1"  /* Bypass Bladder to Cutaneous with Nonaut Sub, Open Approach */
"0T1B0Z9"  =  "1"  /* Bypass Bladder to Colocutaneous, Open Approach */
"0T1B0ZC"  =  "1"  /* Bypass Bladder to Ileocutaneous, Open Approach */
"0T1B0ZD"  =  "1"  /* Bypass Bladder to Cutaneous, Open Approach */
"0T1B3JD"  =  "1"  /* Bypass Bladder to Cutaneous with Synth Sub, Perc Approach */
"0T1B479"  =  "1"  /* Bypass Bladder to Colocutan w Autol Sub, Perc Endo */
"0T1B47C"  =  "1"  /* Bypass Bladder to Ileocutan w Autol Sub, Perc Endo */
"0T1B47D"  =  "1"  /* Bypass Bladder to Cutan with Autol Sub, Perc Endo Approach */
"0T1B4J9"  =  "1"  /* Bypass Bladder to Colocutan w Synth Sub, Perc Endo */
"0T1B4JC"  =  "1"  /* Bypass Bladder to Ileocutan w Synth Sub, Perc Endo */
"0T1B4JD"  =  "1"  /* Bypass Bladder to Cutan with Synth Sub, Perc Endo Approach */
"0T1B4K9"  =  "1"  /* Bypass Bladder to Colocutan w Nonaut Sub, Perc Endo */
"0T1B4KC"  =  "1"  /* Bypass Bladder to Ileocutan w Nonaut Sub, Perc Endo */
"0T1B4KD"  =  "1"  /* Bypass Bladder to Cutan with Nonaut Sub, Perc Endo Approach */
"0T1B4Z9"  =  "1"  /* Bypass Bladder to Colocutaneous, Perc Endo Approach */
"0T1B4ZC"  =  "1"  /* Bypass Bladder to Ileocutaneous, Perc Endo Approach */
"0T1B4ZD"  =  "1"  /* Bypass Bladder to Cutaneous, Perc Endo Approach */
"0T500ZZ"  =  "1"  /* Destruction of Right Kidney, Open Approach */
"0T503ZZ"  =  "1"  /* Destruction of Right Kidney, Percutaneous Approach */
"0T504ZZ"  =  "1"  /* Destruction of Right Kidney, Perc Endo Approach */
"0T507ZZ"  =  "1"  /* Destruction of Right Kidney, Via Opening */
"0T508ZZ"  =  "1"  /* Destruction of Right Kidney, Endo */
"0T510ZZ"  =  "1"  /* Destruction of Left Kidney, Open Approach */
"0T513ZZ"  =  "1"  /* Destruction of Left Kidney, Percutaneous Approach */
"0T514ZZ"  =  "1"  /* Destruction of Left Kidney, Percutaneous Endoscopic Approach */
"0T517ZZ"  =  "1"  /* Destruction of Left Kidney, Via Opening */
"0T518ZZ"  =  "1"  /* Destruction of Left Kidney, Endo */
"0T530ZZ"  =  "1"  /* Destruction of Right Kidney Pelvis, Open Approach */
"0T533ZZ"  =  "1"  /* Destruction of Right Kidney Pelvis, Percutaneous Approach */
"0T534ZZ"  =  "1"  /* Destruction of Right Kidney Pelvis, Perc Endo Approach */
"0T537ZZ"  =  "1"  /* Destruction of Right Kidney Pelvis, Via Opening */
"0T538ZZ"  =  "1"  /* Destruction of Right Kidney Pelvis, Endo */
"0T540ZZ"  =  "1"  /* Destruction of Left Kidney Pelvis, Open Approach */
"0T543ZZ"  =  "1"  /* Destruction of Left Kidney Pelvis, Percutaneous Approach */
"0T544ZZ"  =  "1"  /* Destruction of Left Kidney Pelvis, Perc Endo Approach */
"0T547ZZ"  =  "1"  /* Destruction of Left Kidney Pelvis, Via Opening */
"0T548ZZ"  =  "1"  /* Destruction of Left Kidney Pelvis, Endo */
"0T560ZZ"  =  "1"  /* Destruction of Right Ureter, Open Approach */
"0T563ZZ"  =  "1"  /* Destruction of Right Ureter, Percutaneous Approach */
"0T564ZZ"  =  "1"  /* Destruction of Right Ureter, Perc Endo Approach */
"0T567ZZ"  =  "1"  /* Destruction of Right Ureter, Via Opening */
"0T568ZZ"  =  "1"  /* Destruction of Right Ureter, Endo */
"0T570ZZ"  =  "1"  /* Destruction of Left Ureter, Open Approach */
"0T573ZZ"  =  "1"  /* Destruction of Left Ureter, Percutaneous Approach */
"0T574ZZ"  =  "1"  /* Destruction of Left Ureter, Percutaneous Endoscopic Approach */
"0T577ZZ"  =  "1"  /* Destruction of Left Ureter, Via Opening */
"0T578ZZ"  =  "1"  /* Destruction of Left Ureter, Endo */
"0T5B0ZZ"  =  "1"  /* Destruction of Bladder, Open Approach */
"0T5B3ZZ"  =  "1"  /* Destruction of Bladder, Percutaneous Approach */
"0T5B4ZZ"  =  "1"  /* Destruction of Bladder, Percutaneous Endoscopic Approach */
"0T5B7ZZ"  =  "1"  /* Destruction of Bladder, Via Natural or Artificial Opening */
"0T5B8ZZ"  =  "1"  /* Destruction of Bladder, Endo */
"0T5C0ZZ"  =  "1"  /* Destruction of Bladder Neck, Open Approach */
"0T5C3ZZ"  =  "1"  /* Destruction of Bladder Neck, Percutaneous Approach */
"0T5C4ZZ"  =  "1"  /* Destruction of Bladder Neck, Perc Endo Approach */
"0T5C7ZZ"  =  "1"  /* Destruction of Bladder Neck, Via Opening */
"0T5C8ZZ"  =  "1"  /* Destruction of Bladder Neck, Endo */
"0T730DZ"  =  "1"  /* Dilation of R Kidney Pelvis with Intralum Dev, Open Approach */
"0T730ZZ"  =  "1"  /* Dilation of Right Kidney Pelvis, Open Approach */
"0T733DZ"  =  "1"  /* Dilation of R Kidney Pelvis with Intralum Dev, Perc Approach */
"0T733ZZ"  =  "1"  /* Dilation of Right Kidney Pelvis, Percutaneous Approach */
"0T734DZ"  =  "1"  /* Dilate R Kidney Pelvis w Intralum Dev, Perc Endo */
"0T734ZZ"  =  "1"  /* Dilation of Right Kidney Pelvis, Perc Endo Approach */
"0T737DZ"  =  "1"  /* Dilation of R Kidney Pelvis with Intralum Dev, Via Opening */
"0T737ZZ"  =  "1"  /* Dilation of Right Kidney Pelvis, Via Opening */
"0T738DZ"  =  "1"  /* Dilation of Right Kidney Pelvis with Intralum Dev, Endo */
"0T738ZZ"  =  "1"  /* Dilation of Right Kidney Pelvis, Endo */
"0T740DZ"  =  "1"  /* Dilation of L Kidney Pelvis with Intralum Dev, Open Approach */
"0T740ZZ"  =  "1"  /* Dilation of Left Kidney Pelvis, Open Approach */
"0T743DZ"  =  "1"  /* Dilation of L Kidney Pelvis with Intralum Dev, Perc Approach */
"0T743ZZ"  =  "1"  /* Dilation of Left Kidney Pelvis, Percutaneous Approach */
"0T744DZ"  =  "1"  /* Dilate L Kidney Pelvis w Intralum Dev, Perc Endo */
"0T744ZZ"  =  "1"  /* Dilation of Left Kidney Pelvis, Perc Endo Approach */
"0T747DZ"  =  "1"  /* Dilation of L Kidney Pelvis with Intralum Dev, Via Opening */
"0T747ZZ"  =  "1"  /* Dilation of Left Kidney Pelvis, Via Opening */
"0T748DZ"  =  "1"  /* Dilation of Left Kidney Pelvis with Intralum Dev, Endo */
"0T748ZZ"  =  "1"  /* Dilation of Left Kidney Pelvis, Endo */
"0T760ZZ"  =  "1"  /* Dilation of Right Ureter, Open Approach */
"0T763ZZ"  =  "1"  /* Dilation of Right Ureter, Percutaneous Approach */
"0T764ZZ"  =  "1"  /* Dilation of Right Ureter, Percutaneous Endoscopic Approach */
"0T767ZZ"  =  "1"  /* Dilation of Right Ureter, Via Natural or Artificial Opening */
"0T768ZZ"  =  "1"  /* Dilation of Right Ureter, Endo */
"0T770ZZ"  =  "1"  /* Dilation of Left Ureter, Open Approach */
"0T773ZZ"  =  "1"  /* Dilation of Left Ureter, Percutaneous Approach */
"0T774ZZ"  =  "1"  /* Dilation of Left Ureter, Percutaneous Endoscopic Approach */
"0T777ZZ"  =  "1"  /* Dilation of Left Ureter, Via Natural or Artificial Opening */
"0T778ZZ"  =  "1"  /* Dilation of Left Ureter, Endo */
"0T780ZZ"  =  "1"  /* Dilation of Bilateral Ureters, Open Approach */
"0T783ZZ"  =  "1"  /* Dilation of Bilateral Ureters, Percutaneous Approach */
"0T784ZZ"  =  "1"  /* Dilation of Bilateral Ureters, Perc Endo Approach */
"0T7B0DZ"  =  "1"  /* Dilation of Bladder with Intraluminal Device, Open Approach */
"0T7B0ZZ"  =  "1"  /* Dilation of Bladder, Open Approach */
"0T7B3DZ"  =  "1"  /* Dilation of Bladder with Intraluminal Device, Perc Approach */
"0T7B3ZZ"  =  "1"  /* Dilation of Bladder, Percutaneous Approach */
"0T7B4DZ"  =  "1"  /* Dilation of Bladder with Intralum Dev, Perc Endo Approach */
"0T7B4ZZ"  =  "1"  /* Dilation of Bladder, Percutaneous Endoscopic Approach */
"0T7B7DZ"  =  "1"  /* Dilation of Bladder with Intraluminal Device, Via Opening */
"0T7B7ZZ"  =  "1"  /* Dilation of Bladder, Via Natural or Artificial Opening */
"0T7B8DZ"  =  "1"  /* Dilation of Bladder with Intraluminal Device, Endo */
"0T7B8ZZ"  =  "1"  /* Dilation of Bladder, Endo */
"0T7D0ZZ"  =  "1"  /* Dilation of Urethra, Open Approach */
"0T7D3ZZ"  =  "1"  /* Dilation of Urethra, Percutaneous Approach */
"0T7D4ZZ"  =  "1"  /* Dilation of Urethra, Percutaneous Endoscopic Approach */
"0T820ZZ"  =  "1"  /* Division of Bilateral Kidneys, Open Approach */
"0T823ZZ"  =  "1"  /* Division of Bilateral Kidneys, Percutaneous Approach */
"0T824ZZ"  =  "1"  /* Division of Bilateral Kidneys, Perc Endo Approach */
"0T8C0ZZ"  =  "1"  /* Division of Bladder Neck, Open Approach */
"0T8C3ZZ"  =  "1"  /* Division of Bladder Neck, Percutaneous Approach */
"0T8C4ZZ"  =  "1"  /* Division of Bladder Neck, Percutaneous Endoscopic Approach */
"0T9000Z"  =  "1"  /* Drainage of Right Kidney with Drainage Device, Open Approach */
"0T900ZX"  =  "1"  /* Drainage of Right Kidney, Open Approach, Diagnostic */
"0T900ZZ"  =  "1"  /* Drainage of Right Kidney, Open Approach */
"0T9030Z"  =  "1"  /* Drainage of Right Kidney with Drainage Device, Perc Approach */
"0T9040Z"  =  "1"  /* Drainage of Right Kidney with Drain Dev, Perc Endo Approach */
"0T9070Z"  =  "1"  /* Drainage of Right Kidney with Drainage Device, Via Opening */
"0T907ZZ"  =  "1"  /* Drainage of Right Kidney, Via Natural or Artificial Opening */
"0T9080Z"  =  "1"  /* Drainage of Right Kidney with Drainage Device, Endo */
"0T908ZZ"  =  "1"  /* Drainage of Right Kidney, Endo */
"0T9100Z"  =  "1"  /* Drainage of Left Kidney with Drainage Device, Open Approach */
"0T910ZX"  =  "1"  /* Drainage of Left Kidney, Open Approach, Diagnostic */
"0T910ZZ"  =  "1"  /* Drainage of Left Kidney, Open Approach */
"0T9130Z"  =  "1"  /* Drainage of Left Kidney with Drainage Device, Perc Approach */
"0T9140Z"  =  "1"  /* Drainage of Left Kidney with Drain Dev, Perc Endo Approach */
"0T9170Z"  =  "1"  /* Drainage of Left Kidney with Drainage Device, Via Opening */
"0T917ZZ"  =  "1"  /* Drainage of Left Kidney, Via Natural or Artificial Opening */
"0T9180Z"  =  "1"  /* Drainage of Left Kidney with Drainage Device, Endo */
"0T918ZZ"  =  "1"  /* Drainage of Left Kidney, Endo */
"0T9300Z"  =  "1"  /* Drainage of R Kidney Pelvis with Drain Dev, Open Approach */
"0T930ZX"  =  "1"  /* Drainage of Right Kidney Pelvis, Open Approach, Diagnostic */
"0T930ZZ"  =  "1"  /* Drainage of Right Kidney Pelvis, Open Approach */
"0T9330Z"  =  "1"  /* Drainage of R Kidney Pelvis with Drain Dev, Perc Approach */
"0T9340Z"  =  "1"  /* Drain of R Kidney Pelvis with Drain Dev, Perc Endo Approach */
"0T9370Z"  =  "1"  /* Drainage of Right Kidney Pelvis with Drain Dev, Via Opening */
"0T937ZZ"  =  "1"  /* Drainage of Right Kidney Pelvis, Via Opening */
"0T9380Z"  =  "1"  /* Drainage of Right Kidney Pelvis with Drainage Device, Endo */
"0T938ZZ"  =  "1"  /* Drainage of Right Kidney Pelvis, Endo */
"0T9400Z"  =  "1"  /* Drainage of Left Kidney Pelvis with Drain Dev, Open Approach */
"0T940ZX"  =  "1"  /* Drainage of Left Kidney Pelvis, Open Approach, Diagnostic */
"0T940ZZ"  =  "1"  /* Drainage of Left Kidney Pelvis, Open Approach */
"0T9430Z"  =  "1"  /* Drainage of Left Kidney Pelvis with Drain Dev, Perc Approach */
"0T9440Z"  =  "1"  /* Drain of L Kidney Pelvis with Drain Dev, Perc Endo Approach */
"0T9470Z"  =  "1"  /* Drainage of Left Kidney Pelvis with Drain Dev, Via Opening */
"0T947ZZ"  =  "1"  /* Drainage of Left Kidney Pelvis, Via Opening */
"0T9480Z"  =  "1"  /* Drainage of Left Kidney Pelvis with Drainage Device, Endo */
"0T948ZZ"  =  "1"  /* Drainage of Left Kidney Pelvis, Endo */
"0T960ZX"  =  "1"  /* Drainage of Right Ureter, Open Approach, Diagnostic */
"0T960ZZ"  =  "1"  /* Drainage of Right Ureter, Open Approach */
"0T963ZZ"  =  "1"  /* Drainage of Right Ureter, Percutaneous Approach */
"0T964ZZ"  =  "1"  /* Drainage of Right Ureter, Percutaneous Endoscopic Approach */
"0T967ZZ"  =  "1"  /* Drainage of Right Ureter, Via Natural or Artificial Opening */
"0T968ZZ"  =  "1"  /* Drainage of Right Ureter, Endo */
"0T970ZX"  =  "1"  /* Drainage of Left Ureter, Open Approach, Diagnostic */
"0T970ZZ"  =  "1"  /* Drainage of Left Ureter, Open Approach */
"0T973ZZ"  =  "1"  /* Drainage of Left Ureter, Percutaneous Approach */
"0T974ZZ"  =  "1"  /* Drainage of Left Ureter, Percutaneous Endoscopic Approach */
"0T977ZZ"  =  "1"  /* Drainage of Left Ureter, Via Natural or Artificial Opening */
"0T978ZZ"  =  "1"  /* Drainage of Left Ureter, Endo */
"0T980ZX"  =  "1"  /* Drainage of Bilateral Ureters, Open Approach, Diagnostic */
"0T980ZZ"  =  "1"  /* Drainage of Bilateral Ureters, Open Approach */
"0T983ZZ"  =  "1"  /* Drainage of Bilateral Ureters, Percutaneous Approach */
"0T984ZZ"  =  "1"  /* Drainage of Bilateral Ureters, Perc Endo Approach */
"0T987ZZ"  =  "1"  /* Drainage of Bilateral Ureters, Via Opening */
"0T988ZZ"  =  "1"  /* Drainage of Bilateral Ureters, Endo */
"0T9B00Z"  =  "1"  /* Drainage of Bladder with Drainage Device, Open Approach */
"0T9B0ZX"  =  "1"  /* Drainage of Bladder, Open Approach, Diagnostic */
"0T9B0ZZ"  =  "1"  /* Drainage of Bladder, Open Approach */
"0T9B3ZX"  =  "1"  /* Drainage of Bladder, Percutaneous Approach, Diagnostic */
"0T9B4ZX"  =  "1"  /* Drainage of Bladder, Percutaneous Endoscopic Approach, Diagn */
"0T9B7ZX"  =  "1"  /* Drainage of Bladder, Via Opening, Diagn */
"0T9B8ZX"  =  "1"  /* Drainage of Bladder, Endo, Diagn */
"0T9C00Z"  =  "1"  /* Drainage of Bladder Neck with Drainage Device, Open Approach */
"0T9C0ZX"  =  "1"  /* Drainage of Bladder Neck, Open Approach, Diagnostic */
"0T9C0ZZ"  =  "1"  /* Drainage of Bladder Neck, Open Approach */
"0T9C3ZX"  =  "1"  /* Drainage of Bladder Neck, Percutaneous Approach, Diagnostic */
"0T9C4ZX"  =  "1"  /* Drainage of Bladder Neck, Perc Endo Approach, Diagn */
"0T9C7ZX"  =  "1"  /* Drainage of Bladder Neck, Via Opening, Diagn */
"0T9C8ZX"  =  "1"  /* Drainage of Bladder Neck, Endo, Diagn */
"0T9D00Z"  =  "1"  /* Drainage of Urethra with Drainage Device, Open Approach */
"0T9D0ZZ"  =  "1"  /* Drainage of Urethra, Open Approach */
"0T9D30Z"  =  "1"  /* Drainage of Urethra with Drainage Device, Perc Approach */
"0T9D3ZZ"  =  "1"  /* Drainage of Urethra, Percutaneous Approach */
"0T9D40Z"  =  "1"  /* Drainage of Urethra with Drainage Device, Perc Endo Approach */
"0T9D4ZZ"  =  "1"  /* Drainage of Urethra, Percutaneous Endoscopic Approach */
"0T9D70Z"  =  "1"  /* Drainage of Urethra with Drainage Device, Via Opening */
"0T9D7ZZ"  =  "1"  /* Drainage of Urethra, Via Natural or Artificial Opening */
"0T9D80Z"  =  "1"  /* Drainage of Urethra with Drainage Device, Endo */
"0T9D8ZZ"  =  "1"  /* Drainage of Urethra, Endo */
"0T9DX0Z"  =  "1"  /* Drainage of Urethra with Drainage Device, External Approach */
"0T9DXZZ"  =  "1"  /* Drainage of Urethra, External Approach */
"0TB00ZX"  =  "1"  /* Excision of Right Kidney, Open Approach, Diagnostic */
"0TB00ZZ"  =  "1"  /* Excision of Right Kidney, Open Approach */
"0TB03ZZ"  =  "1"  /* Excision of Right Kidney, Percutaneous Approach */
"0TB04ZZ"  =  "1"  /* Excision of Right Kidney, Percutaneous Endoscopic Approach */
"0TB07ZZ"  =  "1"  /* Excision of Right Kidney, Via Natural or Artificial Opening */
"0TB08ZZ"  =  "1"  /* Excision of Right Kidney, Endo */
"0TB10ZX"  =  "1"  /* Excision of Left Kidney, Open Approach, Diagnostic */
"0TB10ZZ"  =  "1"  /* Excision of Left Kidney, Open Approach */
"0TB13ZZ"  =  "1"  /* Excision of Left Kidney, Percutaneous Approach */
"0TB14ZZ"  =  "1"  /* Excision of Left Kidney, Percutaneous Endoscopic Approach */
"0TB17ZZ"  =  "1"  /* Excision of Left Kidney, Via Natural or Artificial Opening */
"0TB18ZZ"  =  "1"  /* Excision of Left Kidney, Endo */
"0TB30ZX"  =  "1"  /* Excision of Right Kidney Pelvis, Open Approach, Diagnostic */
"0TB30ZZ"  =  "1"  /* Excision of Right Kidney Pelvis, Open Approach */
"0TB33ZZ"  =  "1"  /* Excision of Right Kidney Pelvis, Percutaneous Approach */
"0TB34ZZ"  =  "1"  /* Excision of Right Kidney Pelvis, Perc Endo Approach */
"0TB37ZZ"  =  "1"  /* Excision of Right Kidney Pelvis, Via Opening */
"0TB38ZZ"  =  "1"  /* Excision of Right Kidney Pelvis, Endo */
"0TB40ZX"  =  "1"  /* Excision of Left Kidney Pelvis, Open Approach, Diagnostic */
"0TB40ZZ"  =  "1"  /* Excision of Left Kidney Pelvis, Open Approach */
"0TB43ZZ"  =  "1"  /* Excision of Left Kidney Pelvis, Percutaneous Approach */
"0TB44ZZ"  =  "1"  /* Excision of Left Kidney Pelvis, Perc Endo Approach */
"0TB47ZZ"  =  "1"  /* Excision of Left Kidney Pelvis, Via Opening */
"0TB48ZZ"  =  "1"  /* Excision of Left Kidney Pelvis, Endo */
"0TB60ZX"  =  "1"  /* Excision of Right Ureter, Open Approach, Diagnostic */
"0TB60ZZ"  =  "1"  /* Excision of Right Ureter, Open Approach */
"0TB63ZZ"  =  "1"  /* Excision of Right Ureter, Percutaneous Approach */
"0TB64ZZ"  =  "1"  /* Excision of Right Ureter, Percutaneous Endoscopic Approach */
"0TB67ZZ"  =  "1"  /* Excision of Right Ureter, Via Natural or Artificial Opening */
"0TB68ZZ"  =  "1"  /* Excision of Right Ureter, Endo */
"0TB70ZX"  =  "1"  /* Excision of Left Ureter, Open Approach, Diagnostic */
"0TB70ZZ"  =  "1"  /* Excision of Left Ureter, Open Approach */
"0TB73ZZ"  =  "1"  /* Excision of Left Ureter, Percutaneous Approach */
"0TB74ZZ"  =  "1"  /* Excision of Left Ureter, Percutaneous Endoscopic Approach */
"0TB77ZZ"  =  "1"  /* Excision of Left Ureter, Via Natural or Artificial Opening */
"0TB78ZZ"  =  "1"  /* Excision of Left Ureter, Endo */
"0TBB0ZX"  =  "1"  /* Excision of Bladder, Open Approach, Diagnostic */
"0TBB0ZZ"  =  "1"  /* Excision of Bladder, Open Approach */
"0TBB3ZX"  =  "1"  /* Excision of Bladder, Percutaneous Approach, Diagnostic */
"0TBB3ZZ"  =  "1"  /* Excision of Bladder, Percutaneous Approach */
"0TBB4ZX"  =  "1"  /* Excision of Bladder, Percutaneous Endoscopic Approach, Diagn */
"0TBB4ZZ"  =  "1"  /* Excision of Bladder, Percutaneous Endoscopic Approach */
"0TBB7ZX"  =  "1"  /* Excision of Bladder, Via Opening, Diagn */
"0TBB7ZZ"  =  "1"  /* Excision of Bladder, Via Natural or Artificial Opening */
"0TBB8ZX"  =  "1"  /* Excision of Bladder, Endo, Diagn */
"0TBB8ZZ"  =  "1"  /* Excision of Bladder, Endo */
"0TBC0ZX"  =  "1"  /* Excision of Bladder Neck, Open Approach, Diagnostic */
"0TBC0ZZ"  =  "1"  /* Excision of Bladder Neck, Open Approach */
"0TBC3ZX"  =  "1"  /* Excision of Bladder Neck, Percutaneous Approach, Diagnostic */
"0TBC3ZZ"  =  "1"  /* Excision of Bladder Neck, Percutaneous Approach */
"0TBC4ZX"  =  "1"  /* Excision of Bladder Neck, Perc Endo Approach, Diagn */
"0TBC4ZZ"  =  "1"  /* Excision of Bladder Neck, Percutaneous Endoscopic Approach */
"0TBC7ZX"  =  "1"  /* Excision of Bladder Neck, Via Opening, Diagn */
"0TBC7ZZ"  =  "1"  /* Excision of Bladder Neck, Via Natural or Artificial Opening */
"0TBC8ZX"  =  "1"  /* Excision of Bladder Neck, Endo, Diagn */
"0TBC8ZZ"  =  "1"  /* Excision of Bladder Neck, Endo */
"0TBD0ZZ"  =  "1"  /* Excision of Urethra, Open Approach */
"0TBD3ZZ"  =  "1"  /* Excision of Urethra, Percutaneous Approach */
"0TBD4ZZ"  =  "1"  /* Excision of Urethra, Percutaneous Endoscopic Approach */
"0TBD7ZZ"  =  "1"  /* Excision of Urethra, Via Natural or Artificial Opening */
"0TBD8ZZ"  =  "1"  /* Excision of Urethra, Endo */
"0TBDXZZ"  =  "1"  /* Excision of Urethra, External Approach */
"0TC00ZZ"  =  "1"  /* Extirpation of Matter from Right Kidney, Open Approach */
"0TC03ZZ"  =  "1"  /* Extirpation of Matter from Right Kidney, Perc Approach */
"0TC04ZZ"  =  "1"  /* Extirpation of Matter from Right Kidney, Perc Endo Approach */
"0TC07ZZ"  =  "1"  /* Extirpation of Matter from Right Kidney, Via Opening */
"0TC08ZZ"  =  "1"  /* Extirpation of Matter from Right Kidney, Endo */
"0TC10ZZ"  =  "1"  /* Extirpation of Matter from Left Kidney, Open Approach */
"0TC13ZZ"  =  "1"  /* Extirpation of Matter from Left Kidney, Perc Approach */
"0TC14ZZ"  =  "1"  /* Extirpation of Matter from Left Kidney, Perc Endo Approach */
"0TC17ZZ"  =  "1"  /* Extirpation of Matter from Left Kidney, Via Opening */
"0TC18ZZ"  =  "1"  /* Extirpation of Matter from Left Kidney, Endo */
"0TC30ZZ"  =  "1"  /* Extirpation of Matter from R Kidney Pelvis, Open Approach */
"0TC33ZZ"  =  "1"  /* Extirpation of Matter from R Kidney Pelvis, Perc Approach */
"0TC34ZZ"  =  "1"  /* Extirpate of Matter from R Kidney Pelvis, Perc Endo Approach */
"0TC37ZZ"  =  "1"  /* Extirpation of Matter from Right Kidney Pelvis, Via Opening */
"0TC38ZZ"  =  "1"  /* Extirpation of Matter from Right Kidney Pelvis, Endo */
"0TC40ZZ"  =  "1"  /* Extirpation of Matter from Left Kidney Pelvis, Open Approach */
"0TC43ZZ"  =  "1"  /* Extirpation of Matter from Left Kidney Pelvis, Perc Approach */
"0TC44ZZ"  =  "1"  /* Extirpate of Matter from L Kidney Pelvis, Perc Endo Approach */
"0TC47ZZ"  =  "1"  /* Extirpation of Matter from Left Kidney Pelvis, Via Opening */
"0TC48ZZ"  =  "1"  /* Extirpation of Matter from Left Kidney Pelvis, Endo */
"0TC60ZZ"  =  "1"  /* Extirpation of Matter from Right Ureter, Open Approach */
"0TC63ZZ"  =  "1"  /* Extirpation of Matter from Right Ureter, Perc Approach */
"0TC64ZZ"  =  "1"  /* Extirpation of Matter from Right Ureter, Perc Endo Approach */
"0TC67ZZ"  =  "1"  /* Extirpation of Matter from Right Ureter, Via Opening */
"0TC68ZZ"  =  "1"  /* Extirpation of Matter from Right Ureter, Endo */
"0TC70ZZ"  =  "1"  /* Extirpation of Matter from Left Ureter, Open Approach */
"0TC73ZZ"  =  "1"  /* Extirpation of Matter from Left Ureter, Perc Approach */
"0TC74ZZ"  =  "1"  /* Extirpation of Matter from Left Ureter, Perc Endo Approach */
"0TC77ZZ"  =  "1"  /* Extirpation of Matter from Left Ureter, Via Opening */
"0TC78ZZ"  =  "1"  /* Extirpation of Matter from Left Ureter, Endo */
"0TCB0ZZ"  =  "1"  /* Extirpation of Matter from Bladder, Open Approach */
"0TCB3ZZ"  =  "1"  /* Extirpation of Matter from Bladder, Percutaneous Approach */
"0TCB4ZZ"  =  "1"  /* Extirpation of Matter from Bladder, Perc Endo Approach */
"0TCC0ZZ"  =  "1"  /* Extirpation of Matter from Bladder Neck, Open Approach */
"0TCC3ZZ"  =  "1"  /* Extirpation of Matter from Bladder Neck, Perc Approach */
"0TCC4ZZ"  =  "1"  /* Extirpation of Matter from Bladder Neck, Perc Endo Approach */
"0TCD0ZZ"  =  "1"  /* Extirpation of Matter from Urethra, Open Approach */
"0TCD3ZZ"  =  "1"  /* Extirpation of Matter from Urethra, Percutaneous Approach */
"0TCD4ZZ"  =  "1"  /* Extirpation of Matter from Urethra, Perc Endo Approach */
"0TD00ZZ"  =  "1"  /* Extraction of Right Kidney, Open Approach */
"0TD03ZZ"  =  "1"  /* Extraction of Right Kidney, Percutaneous Approach */
"0TD04ZZ"  =  "1"  /* Extraction of Right Kidney, Percutaneous Endoscopic Approach */
"0TD10ZZ"  =  "1"  /* Extraction of Left Kidney, Open Approach */
"0TD13ZZ"  =  "1"  /* Extraction of Left Kidney, Percutaneous Approach */
"0TD14ZZ"  =  "1"  /* Extraction of Left Kidney, Percutaneous Endoscopic Approach */
"0TF33ZZ"  =  "1"  /* Fragmentation in Right Kidney Pelvis, Percutaneous Approach */
"0TF34ZZ"  =  "1"  /* Fragmentation in Right Kidney Pelvis, Perc Endo Approach */
"0TF43ZZ"  =  "1"  /* Fragmentation in Left Kidney Pelvis, Percutaneous Approach */
"0TF44ZZ"  =  "1"  /* Fragmentation in Left Kidney Pelvis, Perc Endo Approach */
"0TH502Z"  =  "1"  /* Insertion of Monitoring Device into Kidney, Open Approach */
"0TH532Z"  =  "1"  /* Insertion of Monitoring Device into Kidney, Perc Approach */
"0TH542Z"  =  "1"  /* Insertion of Monitor Dev into Kidney, Perc Endo Approach */
"0TH572Z"  =  "1"  /* Insertion of Monitoring Device into Kidney, Via Opening */
"0TH582Z"  =  "1"  /* Insertion of Monitoring Device into Kidney, Endo */
"0TH902Z"  =  "1"  /* Insertion of Monitoring Device into Ureter, Open Approach */
"0TH90MZ"  =  "1"  /* Insertion of Stimulator Lead into Ureter, Open Approach */
"0TH932Z"  =  "1"  /* Insertion of Monitoring Device into Ureter, Perc Approach */
"0TH93MZ"  =  "1"  /* Insertion of Stimulator Lead into Ureter, Perc Approach */
"0TH942Z"  =  "1"  /* Insertion of Monitor Dev into Ureter, Perc Endo Approach */
"0TH94MZ"  =  "1"  /* Insertion of Stimulator Lead into Ureter, Perc Endo Approach */
"0TH972Z"  =  "1"  /* Insertion of Monitoring Device into Ureter, Via Opening */
"0TH97MZ"  =  "1"  /* Insertion of Stimulator Lead into Ureter, Via Opening */
"0TH982Z"  =  "1"  /* Insertion of Monitoring Device into Ureter, Endo */
"0TH98MZ"  =  "1"  /* Insertion of Stimulator Lead into Ureter, Endo */
"0THB02Z"  =  "1"  /* Insertion of Monitoring Device into Bladder, Open Approach */
"0THB0LZ"  =  "1"  /* Insertion of Artif Sphincter into Bladder, Open Approach */
"0THB0MZ"  =  "1"  /* Insertion of Stimulator Lead into Bladder, Open Approach */
"0THB32Z"  =  "1"  /* Insertion of Monitoring Device into Bladder, Perc Approach */
"0THB3LZ"  =  "1"  /* Insertion of Artif Sphincter into Bladder, Perc Approach */
"0THB3MZ"  =  "1"  /* Insertion of Stimulator Lead into Bladder, Perc Approach */
"0THB42Z"  =  "1"  /* Insertion of Monitor Dev into Bladder, Perc Endo Approach */
"0THB4LZ"  =  "1"  /* Insert of Artif Sphincter into Bladder, Perc Endo Approach */
"0THB4MZ"  =  "1"  /* Insertion of Stim Lead into Bladder, Perc Endo Approach */
"0THB72Z"  =  "1"  /* Insertion of Monitoring Device into Bladder, Via Opening */
"0THB7LZ"  =  "1"  /* Insertion of Artificial Sphincter into Bladder, Via Opening */
"0THB7MZ"  =  "1"  /* Insertion of Stimulator Lead into Bladder, Via Opening */
"0THB82Z"  =  "1"  /* Insertion of Monitoring Device into Bladder, Endo */
"0THB8LZ"  =  "1"  /* Insertion of Artificial Sphincter into Bladder, Endo */
"0THB8MZ"  =  "1"  /* Insertion of Stimulator Lead into Bladder, Endo */
"0THC0LZ"  =  "1"  /* Insert of Artif Sphincter into Bladder Neck, Open Approach */
"0THC3LZ"  =  "1"  /* Insert of Artif Sphincter into Bladder Neck, Perc Approach */
"0THC4LZ"  =  "1"  /* Insert Artif Sphincter in Bladder Neck, Perc Endo */
"0THC7LZ"  =  "1"  /* Insertion of Artif Sphincter into Bladder Neck, Via Opening */
"0THC8LZ"  =  "1"  /* Insertion of Artificial Sphincter into Bladder Neck, Endo */
"0THD02Z"  =  "1"  /* Insertion of Monitoring Device into Urethra, Open Approach */
"0THD0LZ"  =  "1"  /* Insertion of Artif Sphincter into Urethra, Open Approach */
"0THD32Z"  =  "1"  /* Insertion of Monitoring Device into Urethra, Perc Approach */
"0THD3LZ"  =  "1"  /* Insertion of Artif Sphincter into Urethra, Perc Approach */
"0THD42Z"  =  "1"  /* Insertion of Monitor Dev into Urethra, Perc Endo Approach */
"0THD4LZ"  =  "1"  /* Insert of Artif Sphincter into Urethra, Perc Endo Approach */
"0THD72Z"  =  "1"  /* Insertion of Monitoring Device into Urethra, Via Opening */
"0THD7LZ"  =  "1"  /* Insertion of Artificial Sphincter into Urethra, Via Opening */
"0THD82Z"  =  "1"  /* Insertion of Monitoring Device into Urethra, Endo */
"0THD8LZ"  =  "1"  /* Insertion of Artificial Sphincter into Urethra, Endo */
"0THDX2Z"  =  "1"  /* Insertion of Monitoring Device into Urethra, Extern Approach */
"0THDXLZ"  =  "1"  /* Insertion of Artif Sphincter into Urethra, Extern Approach */
"0TJ50ZZ"  =  "1"  /* Inspection of Kidney, Open Approach */
"0TJ53ZZ"  =  "1"  /* Inspection of Kidney, Percutaneous Approach */
"0TJ57ZZ"  =  "1"  /* Inspection of Kidney, Via Natural or Artificial Opening */
"0TJ90ZZ"  =  "1"  /* Inspection of Ureter, Open Approach */
"0TJ93ZZ"  =  "1"  /* Inspection of Ureter, Percutaneous Approach */
"0TJ97ZZ"  =  "1"  /* Inspection of Ureter, Via Natural or Artificial Opening */
"0TJB0ZZ"  =  "1"  /* Inspection of Bladder, Open Approach */
"0TJB3ZZ"  =  "1"  /* Inspection of Bladder, Percutaneous Approach */
"0TJB4ZZ"  =  "1"  /* Inspection of Bladder, Percutaneous Endoscopic Approach */
"0TJB7ZZ"  =  "1"  /* Inspection of Bladder, Via Natural or Artificial Opening */
"0TJD0ZZ"  =  "1"  /* Inspection of Urethra, Open Approach */
"0TL30CZ"  =  "1"  /* Occlusion R Kidney Pelvis w Extralum Dev, Open */
"0TL30DZ"  =  "1"  /* Occlusion R Kidney Pelvis w Intralum Dev, Open */
"0TL30ZZ"  =  "1"  /* Occlusion of Right Kidney Pelvis, Open Approach */
"0TL33CZ"  =  "1"  /* Occlusion R Kidney Pelvis w Extralum Dev, Perc */
"0TL33DZ"  =  "1"  /* Occlusion R Kidney Pelvis w Intralum Dev, Perc */
"0TL33ZZ"  =  "1"  /* Occlusion of Right Kidney Pelvis, Percutaneous Approach */
"0TL34CZ"  =  "1"  /* Occlusion R Kidney Pelvis w Extralum Dev, Perc Endo */
"0TL34DZ"  =  "1"  /* Occlusion R Kidney Pelvis w Intralum Dev, Perc Endo */
"0TL34ZZ"  =  "1"  /* Occlusion of Right Kidney Pelvis, Perc Endo Approach */
"0TL37DZ"  =  "1"  /* Occlusion of R Kidney Pelvis with Intralum Dev, Via Opening */
"0TL37ZZ"  =  "1"  /* Occlusion of Right Kidney Pelvis, Via Opening */
"0TL38DZ"  =  "1"  /* Occlusion of Right Kidney Pelvis with Intralum Dev, Endo */
"0TL38ZZ"  =  "1"  /* Occlusion of Right Kidney Pelvis, Endo */
"0TL40CZ"  =  "1"  /* Occlusion L Kidney Pelvis w Extralum Dev, Open */
"0TL40DZ"  =  "1"  /* Occlusion L Kidney Pelvis w Intralum Dev, Open */
"0TL40ZZ"  =  "1"  /* Occlusion of Left Kidney Pelvis, Open Approach */
"0TL43CZ"  =  "1"  /* Occlusion L Kidney Pelvis w Extralum Dev, Perc */
"0TL43DZ"  =  "1"  /* Occlusion L Kidney Pelvis w Intralum Dev, Perc */
"0TL43ZZ"  =  "1"  /* Occlusion of Left Kidney Pelvis, Percutaneous Approach */
"0TL44CZ"  =  "1"  /* Occlusion L Kidney Pelvis w Extralum Dev, Perc Endo */
"0TL44DZ"  =  "1"  /* Occlusion L Kidney Pelvis w Intralum Dev, Perc Endo */
"0TL44ZZ"  =  "1"  /* Occlusion of Left Kidney Pelvis, Perc Endo Approach */
"0TL47DZ"  =  "1"  /* Occlusion of L Kidney Pelvis with Intralum Dev, Via Opening */
"0TL47ZZ"  =  "1"  /* Occlusion of Left Kidney Pelvis, Via Opening */
"0TL48DZ"  =  "1"  /* Occlusion of Left Kidney Pelvis with Intralum Dev, Endo */
"0TL48ZZ"  =  "1"  /* Occlusion of Left Kidney Pelvis, Endo */
"0TL60CZ"  =  "1"  /* Occlusion of Right Ureter with Extralum Dev, Open Approach */
"0TL60DZ"  =  "1"  /* Occlusion of Right Ureter with Intralum Dev, Open Approach */
"0TL60ZZ"  =  "1"  /* Occlusion of Right Ureter, Open Approach */
"0TL63CZ"  =  "1"  /* Occlusion of Right Ureter with Extralum Dev, Perc Approach */
"0TL63DZ"  =  "1"  /* Occlusion of Right Ureter with Intralum Dev, Perc Approach */
"0TL63ZZ"  =  "1"  /* Occlusion of Right Ureter, Percutaneous Approach */
"0TL64CZ"  =  "1"  /* Occlusion of R Ureter with Extralum Dev, Perc Endo Approach */
"0TL64DZ"  =  "1"  /* Occlusion of R Ureter with Intralum Dev, Perc Endo Approach */
"0TL64ZZ"  =  "1"  /* Occlusion of Right Ureter, Percutaneous Endoscopic Approach */
"0TL67DZ"  =  "1"  /* Occlusion of Right Ureter with Intralum Dev, Via Opening */
"0TL67ZZ"  =  "1"  /* Occlusion of Right Ureter, Via Natural or Artificial Opening */
"0TL68DZ"  =  "1"  /* Occlusion of Right Ureter with Intraluminal Device, Endo */
"0TL68ZZ"  =  "1"  /* Occlusion of Right Ureter, Endo */
"0TL70CZ"  =  "1"  /* Occlusion of Left Ureter with Extralum Dev, Open Approach */
"0TL70DZ"  =  "1"  /* Occlusion of Left Ureter with Intralum Dev, Open Approach */
"0TL70ZZ"  =  "1"  /* Occlusion of Left Ureter, Open Approach */
"0TL73CZ"  =  "1"  /* Occlusion of Left Ureter with Extralum Dev, Perc Approach */
"0TL73DZ"  =  "1"  /* Occlusion of Left Ureter with Intralum Dev, Perc Approach */
"0TL73ZZ"  =  "1"  /* Occlusion of Left Ureter, Percutaneous Approach */
"0TL74CZ"  =  "1"  /* Occlusion of L Ureter with Extralum Dev, Perc Endo Approach */
"0TL74DZ"  =  "1"  /* Occlusion of L Ureter with Intralum Dev, Perc Endo Approach */
"0TL74ZZ"  =  "1"  /* Occlusion of Left Ureter, Percutaneous Endoscopic Approach */
"0TL77DZ"  =  "1"  /* Occlusion of Left Ureter with Intralum Dev, Via Opening */
"0TL77ZZ"  =  "1"  /* Occlusion of Left Ureter, Via Natural or Artificial Opening */
"0TL78DZ"  =  "1"  /* Occlusion of Left Ureter with Intraluminal Device, Endo */
"0TL78ZZ"  =  "1"  /* Occlusion of Left Ureter, Endo */
"0TLB0CZ"  =  "1"  /* Occlusion of Bladder with Extraluminal Device, Open Approach */
"0TLB0DZ"  =  "1"  /* Occlusion of Bladder with Intraluminal Device, Open Approach */
"0TLB0ZZ"  =  "1"  /* Occlusion of Bladder, Open Approach */
"0TLB3CZ"  =  "1"  /* Occlusion of Bladder with Extraluminal Device, Perc Approach */
"0TLB3DZ"  =  "1"  /* Occlusion of Bladder with Intraluminal Device, Perc Approach */
"0TLB3ZZ"  =  "1"  /* Occlusion of Bladder, Percutaneous Approach */
"0TLB4CZ"  =  "1"  /* Occlusion of Bladder with Extralum Dev, Perc Endo Approach */
"0TLB4DZ"  =  "1"  /* Occlusion of Bladder with Intralum Dev, Perc Endo Approach */
"0TLB4ZZ"  =  "1"  /* Occlusion of Bladder, Percutaneous Endoscopic Approach */
"0TLB7DZ"  =  "1"  /* Occlusion of Bladder with Intraluminal Device, Via Opening */
"0TLB7ZZ"  =  "1"  /* Occlusion of Bladder, Via Natural or Artificial Opening */
"0TLB8DZ"  =  "1"  /* Occlusion of Bladder with Intraluminal Device, Endo */
"0TLB8ZZ"  =  "1"  /* Occlusion of Bladder, Endo */
"0TLC0CZ"  =  "1"  /* Occlusion of Bladder Neck with Extralum Dev, Open Approach */
"0TLC0DZ"  =  "1"  /* Occlusion of Bladder Neck with Intralum Dev, Open Approach */
"0TLC0ZZ"  =  "1"  /* Occlusion of Bladder Neck, Open Approach */
"0TLC3CZ"  =  "1"  /* Occlusion of Bladder Neck with Extralum Dev, Perc Approach */
"0TLC3DZ"  =  "1"  /* Occlusion of Bladder Neck with Intralum Dev, Perc Approach */
"0TLC3ZZ"  =  "1"  /* Occlusion of Bladder Neck, Percutaneous Approach */
"0TLC4CZ"  =  "1"  /* Occlusion Bladder Neck w Extralum Dev, Perc Endo */
"0TLC4DZ"  =  "1"  /* Occlusion Bladder Neck w Intralum Dev, Perc Endo */
"0TLC4ZZ"  =  "1"  /* Occlusion of Bladder Neck, Percutaneous Endoscopic Approach */
"0TLC7DZ"  =  "1"  /* Occlusion of Bladder Neck with Intralum Dev, Via Opening */
"0TLC7ZZ"  =  "1"  /* Occlusion of Bladder Neck, Via Natural or Artificial Opening */
"0TLC8DZ"  =  "1"  /* Occlusion of Bladder Neck with Intraluminal Device, Endo */
"0TLC8ZZ"  =  "1"  /* Occlusion of Bladder Neck, Endo */
"0TLD0CZ"  =  "1"  /* Occlusion of Urethra with Extraluminal Device, Open Approach */
"0TLD0DZ"  =  "1"  /* Occlusion of Urethra with Intraluminal Device, Open Approach */
"0TLD0ZZ"  =  "1"  /* Occlusion of Urethra, Open Approach */
"0TLD3CZ"  =  "1"  /* Occlusion of Urethra with Extraluminal Device, Perc Approach */
"0TLD3DZ"  =  "1"  /* Occlusion of Urethra with Intraluminal Device, Perc Approach */
"0TLD3ZZ"  =  "1"  /* Occlusion of Urethra, Percutaneous Approach */
"0TLD4CZ"  =  "1"  /* Occlusion of Urethra with Extralum Dev, Perc Endo Approach */
"0TLD4DZ"  =  "1"  /* Occlusion of Urethra with Intralum Dev, Perc Endo Approach */
"0TLD4ZZ"  =  "1"  /* Occlusion of Urethra, Percutaneous Endoscopic Approach */
"0TLD7DZ"  =  "1"  /* Occlusion of Urethra with Intraluminal Device, Via Opening */
"0TLD7ZZ"  =  "1"  /* Occlusion of Urethra, Via Natural or Artificial Opening */
"0TLD8DZ"  =  "1"  /* Occlusion of Urethra with Intraluminal Device, Endo */
"0TLD8ZZ"  =  "1"  /* Occlusion of Urethra, Endo */
"0TLDXCZ"  =  "1"  /* Occlusion of Urethra with Extralum Dev, Extern Approach */
"0TLDXDZ"  =  "1"  /* Occlusion of Urethra with Intralum Dev, Extern Approach */
"0TLDXZZ"  =  "1"  /* Occlusion of Urethra, External Approach */
"0TM00ZZ"  =  "1"  /* Reattachment of Right Kidney, Open Approach */
"0TM04ZZ"  =  "1"  /* Reattachment of Right Kidney, Perc Endo Approach */
"0TM10ZZ"  =  "1"  /* Reattachment of Left Kidney, Open Approach */
"0TM14ZZ"  =  "1"  /* Reattachment of Left Kidney, Perc Endo Approach */
"0TM20ZZ"  =  "1"  /* Reattachment of Bilateral Kidneys, Open Approach */
"0TM24ZZ"  =  "1"  /* Reattachment of Bilateral Kidneys, Perc Endo Approach */
"0TM30ZZ"  =  "1"  /* Reattachment of Right Kidney Pelvis, Open Approach */
"0TM34ZZ"  =  "1"  /* Reattachment of Right Kidney Pelvis, Perc Endo Approach */
"0TM40ZZ"  =  "1"  /* Reattachment of Left Kidney Pelvis, Open Approach */
"0TM44ZZ"  =  "1"  /* Reattachment of Left Kidney Pelvis, Perc Endo Approach */
"0TM60ZZ"  =  "1"  /* Reattachment of Right Ureter, Open Approach */
"0TM64ZZ"  =  "1"  /* Reattachment of Right Ureter, Perc Endo Approach */
"0TM70ZZ"  =  "1"  /* Reattachment of Left Ureter, Open Approach */
"0TM74ZZ"  =  "1"  /* Reattachment of Left Ureter, Perc Endo Approach */
"0TM80ZZ"  =  "1"  /* Reattachment of Bilateral Ureters, Open Approach */
"0TM84ZZ"  =  "1"  /* Reattachment of Bilateral Ureters, Perc Endo Approach */
"0TMB0ZZ"  =  "1"  /* Reattachment of Bladder, Open Approach */
"0TMB4ZZ"  =  "1"  /* Reattachment of Bladder, Percutaneous Endoscopic Approach */
"0TMC0ZZ"  =  "1"  /* Reattachment of Bladder Neck, Open Approach */
"0TMC4ZZ"  =  "1"  /* Reattachment of Bladder Neck, Perc Endo Approach */
"0TMD0ZZ"  =  "1"  /* Reattachment of Urethra, Open Approach */
"0TMD4ZZ"  =  "1"  /* Reattachment of Urethra, Percutaneous Endoscopic Approach */
"0TN00ZZ"  =  "1"  /* Release Right Kidney, Open Approach */
"0TN03ZZ"  =  "1"  /* Release Right Kidney, Percutaneous Approach */
"0TN04ZZ"  =  "1"  /* Release Right Kidney, Percutaneous Endoscopic Approach */
"0TN07ZZ"  =  "1"  /* Release Right Kidney, Via Natural or Artificial Opening */
"0TN08ZZ"  =  "1"  /* Release Right Kidney, Endo */
"0TN10ZZ"  =  "1"  /* Release Left Kidney, Open Approach */
"0TN13ZZ"  =  "1"  /* Release Left Kidney, Percutaneous Approach */
"0TN14ZZ"  =  "1"  /* Release Left Kidney, Percutaneous Endoscopic Approach */
"0TN17ZZ"  =  "1"  /* Release Left Kidney, Via Natural or Artificial Opening */
"0TN18ZZ"  =  "1"  /* Release Left Kidney, Endo */
"0TN30ZZ"  =  "1"  /* Release Right Kidney Pelvis, Open Approach */
"0TN33ZZ"  =  "1"  /* Release Right Kidney Pelvis, Percutaneous Approach */
"0TN34ZZ"  =  "1"  /* Release Right Kidney Pelvis, Perc Endo Approach */
"0TN37ZZ"  =  "1"  /* Release Right Kidney Pelvis, Via Opening */
"0TN38ZZ"  =  "1"  /* Release Right Kidney Pelvis, Endo */
"0TN40ZZ"  =  "1"  /* Release Left Kidney Pelvis, Open Approach */
"0TN43ZZ"  =  "1"  /* Release Left Kidney Pelvis, Percutaneous Approach */
"0TN44ZZ"  =  "1"  /* Release Left Kidney Pelvis, Percutaneous Endoscopic Approach */
"0TN47ZZ"  =  "1"  /* Release Left Kidney Pelvis, Via Opening */
"0TN48ZZ"  =  "1"  /* Release Left Kidney Pelvis, Endo */
"0TN60ZZ"  =  "1"  /* Release Right Ureter, Open Approach */
"0TN63ZZ"  =  "1"  /* Release Right Ureter, Percutaneous Approach */
"0TN64ZZ"  =  "1"  /* Release Right Ureter, Percutaneous Endoscopic Approach */
"0TN67ZZ"  =  "1"  /* Release Right Ureter, Via Natural or Artificial Opening */
"0TN68ZZ"  =  "1"  /* Release Right Ureter, Endo */
"0TN70ZZ"  =  "1"  /* Release Left Ureter, Open Approach */
"0TN73ZZ"  =  "1"  /* Release Left Ureter, Percutaneous Approach */
"0TN74ZZ"  =  "1"  /* Release Left Ureter, Percutaneous Endoscopic Approach */
"0TN77ZZ"  =  "1"  /* Release Left Ureter, Via Natural or Artificial Opening */
"0TN78ZZ"  =  "1"  /* Release Left Ureter, Endo */
"0TNB0ZZ"  =  "1"  /* Release Bladder, Open Approach */
"0TNB3ZZ"  =  "1"  /* Release Bladder, Percutaneous Approach */
"0TNB4ZZ"  =  "1"  /* Release Bladder, Percutaneous Endoscopic Approach */
"0TNB7ZZ"  =  "1"  /* Release Bladder, Via Natural or Artificial Opening */
"0TNB8ZZ"  =  "1"  /* Release Bladder, Endo */
"0TNC0ZZ"  =  "1"  /* Release Bladder Neck, Open Approach */
"0TNC3ZZ"  =  "1"  /* Release Bladder Neck, Percutaneous Approach */
"0TNC4ZZ"  =  "1"  /* Release Bladder Neck, Percutaneous Endoscopic Approach */
"0TNC7ZZ"  =  "1"  /* Release Bladder Neck, Via Natural or Artificial Opening */
"0TNC8ZZ"  =  "1"  /* Release Bladder Neck, Endo */
"0TND0ZZ"  =  "1"  /* Release Urethra, Open Approach */
"0TND3ZZ"  =  "1"  /* Release Urethra, Percutaneous Approach */
"0TND4ZZ"  =  "1"  /* Release Urethra, Percutaneous Endoscopic Approach */
"0TND7ZZ"  =  "1"  /* Release Urethra, Via Natural or Artificial Opening */
"0TND8ZZ"  =  "1"  /* Release Urethra, Endo */
"0TNDXZZ"  =  "1"  /* Release Urethra, External Approach */
"0TP500Z"  =  "1"  /* Removal of Drainage Device from Kidney, Open Approach */
"0TP502Z"  =  "1"  /* Removal of Monitoring Device from Kidney, Open Approach */
"0TP503Z"  =  "1"  /* Removal of Infusion Device from Kidney, Open Approach */
"0TP507Z"  =  "1"  /* Removal of Autol Sub from Kidney, Open Approach */
"0TP50CZ"  =  "1"  /* Removal of Extraluminal Device from Kidney, Open Approach */
"0TP50DZ"  =  "1"  /* Removal of Intraluminal Device from Kidney, Open Approach */
"0TP50JZ"  =  "1"  /* Removal of Synthetic Substitute from Kidney, Open Approach */
"0TP50KZ"  =  "1"  /* Removal of Nonaut Sub from Kidney, Open Approach */
"0TP530Z"  =  "1"  /* Removal of Drainage Device from Kidney, Perc Approach */
"0TP532Z"  =  "1"  /* Removal of Monitoring Device from Kidney, Perc Approach */
"0TP533Z"  =  "1"  /* Removal of Infusion Device from Kidney, Perc Approach */
"0TP537Z"  =  "1"  /* Removal of Autol Sub from Kidney, Perc Approach */
"0TP53CZ"  =  "1"  /* Removal of Extraluminal Device from Kidney, Perc Approach */
"0TP53DZ"  =  "1"  /* Removal of Intraluminal Device from Kidney, Perc Approach */
"0TP53JZ"  =  "1"  /* Removal of Synthetic Substitute from Kidney, Perc Approach */
"0TP53KZ"  =  "1"  /* Removal of Nonaut Sub from Kidney, Perc Approach */
"0TP540Z"  =  "1"  /* Removal of Drainage Device from Kidney, Perc Endo Approach */
"0TP542Z"  =  "1"  /* Removal of Monitoring Device from Kidney, Perc Endo Approach */
"0TP543Z"  =  "1"  /* Removal of Infusion Device from Kidney, Perc Endo Approach */
"0TP547Z"  =  "1"  /* Removal of Autol Sub from Kidney, Perc Endo Approach */
"0TP54CZ"  =  "1"  /* Removal of Extralum Dev from Kidney, Perc Endo Approach */
"0TP54DZ"  =  "1"  /* Removal of Intralum Dev from Kidney, Perc Endo Approach */
"0TP54JZ"  =  "1"  /* Removal of Synth Sub from Kidney, Perc Endo Approach */
"0TP54KZ"  =  "1"  /* Removal of Nonaut Sub from Kidney, Perc Endo Approach */
"0TP570Z"  =  "1"  /* Removal of Drainage Device from Kidney, Via Opening */
"0TP572Z"  =  "1"  /* Removal of Monitoring Device from Kidney, Via Opening */
"0TP573Z"  =  "1"  /* Removal of Infusion Device from Kidney, Via Opening */
"0TP577Z"  =  "1"  /* Removal of Autol Sub from Kidney, Via Opening */
"0TP57CZ"  =  "1"  /* Removal of Extraluminal Device from Kidney, Via Opening */
"0TP57DZ"  =  "1"  /* Removal of Intraluminal Device from Kidney, Via Opening */
"0TP57JZ"  =  "1"  /* Removal of Synthetic Substitute from Kidney, Via Opening */
"0TP57KZ"  =  "1"  /* Removal of Nonaut Sub from Kidney, Via Opening */
"0TP580Z"  =  "1"  /* Removal of Drainage Device from Kidney, Endo */
"0TP582Z"  =  "1"  /* Removal of Monitoring Device from Kidney, Endo */
"0TP583Z"  =  "1"  /* Removal of Infusion Device from Kidney, Endo */
"0TP587Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Kidney, Endo */
"0TP58CZ"  =  "1"  /* Removal of Extraluminal Device from Kidney, Endo */
"0TP58DZ"  =  "1"  /* Removal of Intraluminal Device from Kidney, Endo */
"0TP58JZ"  =  "1"  /* Removal of Synthetic Substitute from Kidney, Endo */
"0TP58KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Kidney, Endo */
"0TP900Z"  =  "1"  /* Removal of Drainage Device from Ureter, Open Approach */
"0TP902Z"  =  "1"  /* Removal of Monitoring Device from Ureter, Open Approach */
"0TP903Z"  =  "1"  /* Removal of Infusion Device from Ureter, Open Approach */
"0TP907Z"  =  "1"  /* Removal of Autol Sub from Ureter, Open Approach */
"0TP90CZ"  =  "1"  /* Removal of Extraluminal Device from Ureter, Open Approach */
"0TP90DZ"  =  "1"  /* Removal of Intraluminal Device from Ureter, Open Approach */
"0TP90JZ"  =  "1"  /* Removal of Synthetic Substitute from Ureter, Open Approach */
"0TP90KZ"  =  "1"  /* Removal of Nonaut Sub from Ureter, Open Approach */
"0TP90MZ"  =  "1"  /* Removal of Stimulator Lead from Ureter, Open Approach */
"0TP930Z"  =  "1"  /* Removal of Drainage Device from Ureter, Perc Approach */
"0TP932Z"  =  "1"  /* Removal of Monitoring Device from Ureter, Perc Approach */
"0TP933Z"  =  "1"  /* Removal of Infusion Device from Ureter, Perc Approach */
"0TP937Z"  =  "1"  /* Removal of Autol Sub from Ureter, Perc Approach */
"0TP93CZ"  =  "1"  /* Removal of Extraluminal Device from Ureter, Perc Approach */
"0TP93DZ"  =  "1"  /* Removal of Intraluminal Device from Ureter, Perc Approach */
"0TP93JZ"  =  "1"  /* Removal of Synthetic Substitute from Ureter, Perc Approach */
"0TP93KZ"  =  "1"  /* Removal of Nonaut Sub from Ureter, Perc Approach */
"0TP93MZ"  =  "1"  /* Removal of Stimulator Lead from Ureter, Perc Approach */
"0TP940Z"  =  "1"  /* Removal of Drainage Device from Ureter, Perc Endo Approach */
"0TP942Z"  =  "1"  /* Removal of Monitoring Device from Ureter, Perc Endo Approach */
"0TP943Z"  =  "1"  /* Removal of Infusion Device from Ureter, Perc Endo Approach */
"0TP947Z"  =  "1"  /* Removal of Autol Sub from Ureter, Perc Endo Approach */
"0TP94CZ"  =  "1"  /* Removal of Extralum Dev from Ureter, Perc Endo Approach */
"0TP94DZ"  =  "1"  /* Removal of Intralum Dev from Ureter, Perc Endo Approach */
"0TP94JZ"  =  "1"  /* Removal of Synth Sub from Ureter, Perc Endo Approach */
"0TP94KZ"  =  "1"  /* Removal of Nonaut Sub from Ureter, Perc Endo Approach */
"0TP94MZ"  =  "1"  /* Removal of Stimulator Lead from Ureter, Perc Endo Approach */
"0TP970Z"  =  "1"  /* Removal of Drainage Device from Ureter, Via Opening */
"0TP972Z"  =  "1"  /* Removal of Monitoring Device from Ureter, Via Opening */
"0TP973Z"  =  "1"  /* Removal of Infusion Device from Ureter, Via Opening */
"0TP977Z"  =  "1"  /* Removal of Autol Sub from Ureter, Via Opening */
"0TP97CZ"  =  "1"  /* Removal of Extraluminal Device from Ureter, Via Opening */
"0TP97DZ"  =  "1"  /* Removal of Intraluminal Device from Ureter, Via Opening */
"0TP97JZ"  =  "1"  /* Removal of Synthetic Substitute from Ureter, Via Opening */
"0TP97KZ"  =  "1"  /* Removal of Nonaut Sub from Ureter, Via Opening */
"0TP97MZ"  =  "1"  /* Removal of Stimulator Lead from Ureter, Via Opening */
"0TP980Z"  =  "1"  /* Removal of Drainage Device from Ureter, Endo */
"0TP982Z"  =  "1"  /* Removal of Monitoring Device from Ureter, Endo */
"0TP983Z"  =  "1"  /* Removal of Infusion Device from Ureter, Endo */
"0TP987Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Ureter, Endo */
"0TP98CZ"  =  "1"  /* Removal of Extraluminal Device from Ureter, Endo */
"0TP98DZ"  =  "1"  /* Removal of Intraluminal Device from Ureter, Endo */
"0TP98JZ"  =  "1"  /* Removal of Synthetic Substitute from Ureter, Endo */
"0TP98KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Ureter, Endo */
"0TP98MZ"  =  "1"  /* Removal of Stimulator Lead from Ureter, Endo */
"0TP9XMZ"  =  "1"  /* Removal of Stimulator Lead from Ureter, External Approach */
"0TPB00Z"  =  "1"  /* Removal of Drainage Device from Bladder, Open Approach */
"0TPB02Z"  =  "1"  /* Removal of Monitoring Device from Bladder, Open Approach */
"0TPB03Z"  =  "1"  /* Removal of Infusion Device from Bladder, Open Approach */
"0TPB07Z"  =  "1"  /* Removal of Autol Sub from Bladder, Open Approach */
"0TPB0CZ"  =  "1"  /* Removal of Extraluminal Device from Bladder, Open Approach */
"0TPB0DZ"  =  "1"  /* Removal of Intraluminal Device from Bladder, Open Approach */
"0TPB0JZ"  =  "1"  /* Removal of Synthetic Substitute from Bladder, Open Approach */
"0TPB0KZ"  =  "1"  /* Removal of Nonaut Sub from Bladder, Open Approach */
"0TPB0LZ"  =  "1"  /* Removal of Artificial Sphincter from Bladder, Open Approach */
"0TPB0MZ"  =  "1"  /* Removal of Stimulator Lead from Bladder, Open Approach */
"0TPB30Z"  =  "1"  /* Removal of Drainage Device from Bladder, Perc Approach */
"0TPB32Z"  =  "1"  /* Removal of Monitoring Device from Bladder, Perc Approach */
"0TPB33Z"  =  "1"  /* Removal of Infusion Device from Bladder, Perc Approach */
"0TPB37Z"  =  "1"  /* Removal of Autol Sub from Bladder, Perc Approach */
"0TPB3CZ"  =  "1"  /* Removal of Extraluminal Device from Bladder, Perc Approach */
"0TPB3DZ"  =  "1"  /* Removal of Intraluminal Device from Bladder, Perc Approach */
"0TPB3JZ"  =  "1"  /* Removal of Synthetic Substitute from Bladder, Perc Approach */
"0TPB3KZ"  =  "1"  /* Removal of Nonaut Sub from Bladder, Perc Approach */
"0TPB3LZ"  =  "1"  /* Removal of Artificial Sphincter from Bladder, Perc Approach */
"0TPB3MZ"  =  "1"  /* Removal of Stimulator Lead from Bladder, Perc Approach */
"0TPB40Z"  =  "1"  /* Removal of Drainage Device from Bladder, Perc Endo Approach */
"0TPB42Z"  =  "1"  /* Removal of Monitor Dev from Bladder, Perc Endo Approach */
"0TPB43Z"  =  "1"  /* Removal of Infusion Device from Bladder, Perc Endo Approach */
"0TPB47Z"  =  "1"  /* Removal of Autol Sub from Bladder, Perc Endo Approach */
"0TPB4CZ"  =  "1"  /* Removal of Extralum Dev from Bladder, Perc Endo Approach */
"0TPB4DZ"  =  "1"  /* Removal of Intralum Dev from Bladder, Perc Endo Approach */
"0TPB4JZ"  =  "1"  /* Removal of Synth Sub from Bladder, Perc Endo Approach */
"0TPB4KZ"  =  "1"  /* Removal of Nonaut Sub from Bladder, Perc Endo Approach */
"0TPB4LZ"  =  "1"  /* Removal of Artif Sphincter from Bladder, Perc Endo Approach */
"0TPB4MZ"  =  "1"  /* Removal of Stimulator Lead from Bladder, Perc Endo Approach */
"0TPB70Z"  =  "1"  /* Removal of Drainage Device from Bladder, Via Opening */
"0TPB72Z"  =  "1"  /* Removal of Monitoring Device from Bladder, Via Opening */
"0TPB73Z"  =  "1"  /* Removal of Infusion Device from Bladder, Via Opening */
"0TPB77Z"  =  "1"  /* Removal of Autol Sub from Bladder, Via Opening */
"0TPB7CZ"  =  "1"  /* Removal of Extraluminal Device from Bladder, Via Opening */
"0TPB7DZ"  =  "1"  /* Removal of Intraluminal Device from Bladder, Via Opening */
"0TPB7JZ"  =  "1"  /* Removal of Synthetic Substitute from Bladder, Via Opening */
"0TPB7KZ"  =  "1"  /* Removal of Nonaut Sub from Bladder, Via Opening */
"0TPB7LZ"  =  "1"  /* Removal of Artificial Sphincter from Bladder, Via Opening */
"0TPB7MZ"  =  "1"  /* Removal of Stimulator Lead from Bladder, Via Opening */
"0TPB80Z"  =  "1"  /* Removal of Drainage Device from Bladder, Endo */
"0TPB82Z"  =  "1"  /* Removal of Monitoring Device from Bladder, Endo */
"0TPB83Z"  =  "1"  /* Removal of Infusion Device from Bladder, Endo */
"0TPB87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Bladder, Endo */
"0TPB8CZ"  =  "1"  /* Removal of Extraluminal Device from Bladder, Endo */
"0TPB8DZ"  =  "1"  /* Removal of Intraluminal Device from Bladder, Endo */
"0TPB8JZ"  =  "1"  /* Removal of Synthetic Substitute from Bladder, Endo */
"0TPB8KZ"  =  "1"  /* Removal of Nonaut Sub from Bladder, Endo */
"0TPB8LZ"  =  "1"  /* Removal of Artificial Sphincter from Bladder, Endo */
"0TPB8MZ"  =  "1"  /* Removal of Stimulator Lead from Bladder, Endo */
"0TPBXMZ"  =  "1"  /* Removal of Stimulator Lead from Bladder, External Approach */
"0TPD00Z"  =  "1"  /* Removal of Drainage Device from Urethra, Open Approach */
"0TPD02Z"  =  "1"  /* Removal of Monitoring Device from Urethra, Open Approach */
"0TPD03Z"  =  "1"  /* Removal of Infusion Device from Urethra, Open Approach */
"0TPD07Z"  =  "1"  /* Removal of Autol Sub from Urethra, Open Approach */
"0TPD0CZ"  =  "1"  /* Removal of Extraluminal Device from Urethra, Open Approach */
"0TPD0DZ"  =  "1"  /* Removal of Intraluminal Device from Urethra, Open Approach */
"0TPD0JZ"  =  "1"  /* Removal of Synthetic Substitute from Urethra, Open Approach */
"0TPD0KZ"  =  "1"  /* Removal of Nonaut Sub from Urethra, Open Approach */
"0TPD0LZ"  =  "1"  /* Removal of Artificial Sphincter from Urethra, Open Approach */
"0TPD30Z"  =  "1"  /* Removal of Drainage Device from Urethra, Perc Approach */
"0TPD32Z"  =  "1"  /* Removal of Monitoring Device from Urethra, Perc Approach */
"0TPD33Z"  =  "1"  /* Removal of Infusion Device from Urethra, Perc Approach */
"0TPD37Z"  =  "1"  /* Removal of Autol Sub from Urethra, Perc Approach */
"0TPD3CZ"  =  "1"  /* Removal of Extraluminal Device from Urethra, Perc Approach */
"0TPD3DZ"  =  "1"  /* Removal of Intraluminal Device from Urethra, Perc Approach */
"0TPD3JZ"  =  "1"  /* Removal of Synthetic Substitute from Urethra, Perc Approach */
"0TPD3KZ"  =  "1"  /* Removal of Nonaut Sub from Urethra, Perc Approach */
"0TPD3LZ"  =  "1"  /* Removal of Artificial Sphincter from Urethra, Perc Approach */
"0TPD40Z"  =  "1"  /* Removal of Drainage Device from Urethra, Perc Endo Approach */
"0TPD42Z"  =  "1"  /* Removal of Monitor Dev from Urethra, Perc Endo Approach */
"0TPD43Z"  =  "1"  /* Removal of Infusion Device from Urethra, Perc Endo Approach */
"0TPD47Z"  =  "1"  /* Removal of Autol Sub from Urethra, Perc Endo Approach */
"0TPD4CZ"  =  "1"  /* Removal of Extralum Dev from Urethra, Perc Endo Approach */
"0TPD4DZ"  =  "1"  /* Removal of Intralum Dev from Urethra, Perc Endo Approach */
"0TPD4JZ"  =  "1"  /* Removal of Synth Sub from Urethra, Perc Endo Approach */
"0TPD4KZ"  =  "1"  /* Removal of Nonaut Sub from Urethra, Perc Endo Approach */
"0TPD4LZ"  =  "1"  /* Removal of Artif Sphincter from Urethra, Perc Endo Approach */
"0TPD70Z"  =  "1"  /* Removal of Drainage Device from Urethra, Via Opening */
"0TPD72Z"  =  "1"  /* Removal of Monitoring Device from Urethra, Via Opening */
"0TPD73Z"  =  "1"  /* Removal of Infusion Device from Urethra, Via Opening */
"0TPD77Z"  =  "1"  /* Removal of Autol Sub from Urethra, Via Opening */
"0TPD7CZ"  =  "1"  /* Removal of Extraluminal Device from Urethra, Via Opening */
"0TPD7DZ"  =  "1"  /* Removal of Intraluminal Device from Urethra, Via Opening */
"0TPD7JZ"  =  "1"  /* Removal of Synthetic Substitute from Urethra, Via Opening */
"0TPD7KZ"  =  "1"  /* Removal of Nonaut Sub from Urethra, Via Opening */
"0TPD7LZ"  =  "1"  /* Removal of Artificial Sphincter from Urethra, Via Opening */
"0TPD80Z"  =  "1"  /* Removal of Drainage Device from Urethra, Endo */
"0TPD82Z"  =  "1"  /* Removal of Monitoring Device from Urethra, Endo */
"0TPD83Z"  =  "1"  /* Removal of Infusion Device from Urethra, Endo */
"0TPD87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Urethra, Endo */
"0TPD8CZ"  =  "1"  /* Removal of Extraluminal Device from Urethra, Endo */
"0TPD8DZ"  =  "1"  /* Removal of Intraluminal Device from Urethra, Endo */
"0TPD8JZ"  =  "1"  /* Removal of Synthetic Substitute from Urethra, Endo */
"0TPD8KZ"  =  "1"  /* Removal of Nonaut Sub from Urethra, Endo */
"0TPD8LZ"  =  "1"  /* Removal of Artificial Sphincter from Urethra, Endo */
"0TPDXLZ"  =  "1"  /* Removal of Artif Sphincter from Urethra, Extern Approach */
"0TQ00ZZ"  =  "1"  /* Repair Right Kidney, Open Approach */
"0TQ03ZZ"  =  "1"  /* Repair Right Kidney, Percutaneous Approach */
"0TQ04ZZ"  =  "1"  /* Repair Right Kidney, Percutaneous Endoscopic Approach */
"0TQ07ZZ"  =  "1"  /* Repair Right Kidney, Via Natural or Artificial Opening */
"0TQ08ZZ"  =  "1"  /* Repair Right Kidney, Endo */
"0TQ10ZZ"  =  "1"  /* Repair Left Kidney, Open Approach */
"0TQ13ZZ"  =  "1"  /* Repair Left Kidney, Percutaneous Approach */
"0TQ14ZZ"  =  "1"  /* Repair Left Kidney, Percutaneous Endoscopic Approach */
"0TQ17ZZ"  =  "1"  /* Repair Left Kidney, Via Natural or Artificial Opening */
"0TQ18ZZ"  =  "1"  /* Repair Left Kidney, Endo */
"0TQ30ZZ"  =  "1"  /* Repair Right Kidney Pelvis, Open Approach */
"0TQ33ZZ"  =  "1"  /* Repair Right Kidney Pelvis, Percutaneous Approach */
"0TQ34ZZ"  =  "1"  /* Repair Right Kidney Pelvis, Percutaneous Endoscopic Approach */
"0TQ37ZZ"  =  "1"  /* Repair Right Kidney Pelvis, Via Opening */
"0TQ38ZZ"  =  "1"  /* Repair Right Kidney Pelvis, Endo */
"0TQ40ZZ"  =  "1"  /* Repair Left Kidney Pelvis, Open Approach */
"0TQ43ZZ"  =  "1"  /* Repair Left Kidney Pelvis, Percutaneous Approach */
"0TQ44ZZ"  =  "1"  /* Repair Left Kidney Pelvis, Percutaneous Endoscopic Approach */
"0TQ47ZZ"  =  "1"  /* Repair Left Kidney Pelvis, Via Natural or Artificial Opening */
"0TQ48ZZ"  =  "1"  /* Repair Left Kidney Pelvis, Endo */
"0TQ60ZZ"  =  "1"  /* Repair Right Ureter, Open Approach */
"0TQ63ZZ"  =  "1"  /* Repair Right Ureter, Percutaneous Approach */
"0TQ64ZZ"  =  "1"  /* Repair Right Ureter, Percutaneous Endoscopic Approach */
"0TQ67ZZ"  =  "1"  /* Repair Right Ureter, Via Natural or Artificial Opening */
"0TQ68ZZ"  =  "1"  /* Repair Right Ureter, Endo */
"0TQ70ZZ"  =  "1"  /* Repair Left Ureter, Open Approach */
"0TQ73ZZ"  =  "1"  /* Repair Left Ureter, Percutaneous Approach */
"0TQ74ZZ"  =  "1"  /* Repair Left Ureter, Percutaneous Endoscopic Approach */
"0TQ77ZZ"  =  "1"  /* Repair Left Ureter, Via Natural or Artificial Opening */
"0TQ78ZZ"  =  "1"  /* Repair Left Ureter, Endo */
"0TQB0ZZ"  =  "1"  /* Repair Bladder, Open Approach */
"0TQB3ZZ"  =  "1"  /* Repair Bladder, Percutaneous Approach */
"0TQB4ZZ"  =  "1"  /* Repair Bladder, Percutaneous Endoscopic Approach */
"0TQB7ZZ"  =  "1"  /* Repair Bladder, Via Natural or Artificial Opening */
"0TQB8ZZ"  =  "1"  /* Repair Bladder, Via Natural or Artificial Opening Endoscopic */
"0TQD0ZZ"  =  "1"  /* Repair Urethra, Open Approach */
"0TQD3ZZ"  =  "1"  /* Repair Urethra, Percutaneous Approach */
"0TQD4ZZ"  =  "1"  /* Repair Urethra, Percutaneous Endoscopic Approach */
"0TQD7ZZ"  =  "1"  /* Repair Urethra, Via Natural or Artificial Opening */
"0TQD8ZZ"  =  "1"  /* Repair Urethra, Via Natural or Artificial Opening Endoscopic */
"0TQDXZZ"  =  "1"  /* Repair Urethra, External Approach */
"0TR307Z"  =  "1"  /* Replacement of R Kidney Pelvis with Autol Sub, Open Approach */
"0TR30JZ"  =  "1"  /* Replacement of R Kidney Pelvis with Synth Sub, Open Approach */
"0TR30KZ"  =  "1"  /* Replace of R Kidney Pelvis with Nonaut Sub, Open Approach */
"0TR347Z"  =  "1"  /* Replace R Kidney Pelvis w Autol Sub, Perc Endo */
"0TR34JZ"  =  "1"  /* Replace R Kidney Pelvis w Synth Sub, Perc Endo */
"0TR34KZ"  =  "1"  /* Replace R Kidney Pelvis w Nonaut Sub, Perc Endo */
"0TR377Z"  =  "1"  /* Replacement of R Kidney Pelvis with Autol Sub, Via Opening */
"0TR37JZ"  =  "1"  /* Replacement of R Kidney Pelvis with Synth Sub, Via Opening */
"0TR37KZ"  =  "1"  /* Replacement of R Kidney Pelvis with Nonaut Sub, Via Opening */
"0TR387Z"  =  "1"  /* Replacement of Right Kidney Pelvis with Autol Sub, Endo */
"0TR38JZ"  =  "1"  /* Replacement of Right Kidney Pelvis with Synth Sub, Endo */
"0TR38KZ"  =  "1"  /* Replacement of Right Kidney Pelvis with Nonaut Sub, Endo */
"0TR407Z"  =  "1"  /* Replacement of L Kidney Pelvis with Autol Sub, Open Approach */
"0TR40JZ"  =  "1"  /* Replacement of L Kidney Pelvis with Synth Sub, Open Approach */
"0TR40KZ"  =  "1"  /* Replace of L Kidney Pelvis with Nonaut Sub, Open Approach */
"0TR447Z"  =  "1"  /* Replace L Kidney Pelvis w Autol Sub, Perc Endo */
"0TR44JZ"  =  "1"  /* Replace L Kidney Pelvis w Synth Sub, Perc Endo */
"0TR44KZ"  =  "1"  /* Replace L Kidney Pelvis w Nonaut Sub, Perc Endo */
"0TR477Z"  =  "1"  /* Replacement of L Kidney Pelvis with Autol Sub, Via Opening */
"0TR47JZ"  =  "1"  /* Replacement of L Kidney Pelvis with Synth Sub, Via Opening */
"0TR47KZ"  =  "1"  /* Replacement of L Kidney Pelvis with Nonaut Sub, Via Opening */
"0TR487Z"  =  "1"  /* Replacement of Left Kidney Pelvis with Autol Sub, Endo */
"0TR48JZ"  =  "1"  /* Replacement of Left Kidney Pelvis with Synth Sub, Endo */
"0TR48KZ"  =  "1"  /* Replacement of Left Kidney Pelvis with Nonaut Sub, Endo */
"0TR607Z"  =  "1"  /* Replacement of Right Ureter with Autol Sub, Open Approach */
"0TR60JZ"  =  "1"  /* Replacement of Right Ureter with Synth Sub, Open Approach */
"0TR60KZ"  =  "1"  /* Replacement of Right Ureter with Nonaut Sub, Open Approach */
"0TR647Z"  =  "1"  /* Replacement of R Ureter with Autol Sub, Perc Endo Approach */
"0TR64JZ"  =  "1"  /* Replacement of R Ureter with Synth Sub, Perc Endo Approach */
"0TR64KZ"  =  "1"  /* Replacement of R Ureter with Nonaut Sub, Perc Endo Approach */
"0TR677Z"  =  "1"  /* Replacement of Right Ureter with Autol Sub, Via Opening */
"0TR67JZ"  =  "1"  /* Replacement of Right Ureter with Synth Sub, Via Opening */
"0TR67KZ"  =  "1"  /* Replacement of Right Ureter with Nonaut Sub, Via Opening */
"0TR687Z"  =  "1"  /* Replacement of Right Ureter with Autol Sub, Endo */
"0TR68JZ"  =  "1"  /* Replacement of Right Ureter with Synthetic Substitute, Endo */
"0TR68KZ"  =  "1"  /* Replacement of Right Ureter with Nonaut Sub, Endo */
"0TR707Z"  =  "1"  /* Replacement of Left Ureter with Autol Sub, Open Approach */
"0TR70JZ"  =  "1"  /* Replacement of Left Ureter with Synth Sub, Open Approach */
"0TR70KZ"  =  "1"  /* Replacement of Left Ureter with Nonaut Sub, Open Approach */
"0TR747Z"  =  "1"  /* Replacement of L Ureter with Autol Sub, Perc Endo Approach */
"0TR74JZ"  =  "1"  /* Replacement of L Ureter with Synth Sub, Perc Endo Approach */
"0TR74KZ"  =  "1"  /* Replacement of L Ureter with Nonaut Sub, Perc Endo Approach */
"0TR777Z"  =  "1"  /* Replacement of Left Ureter with Autol Sub, Via Opening */
"0TR77JZ"  =  "1"  /* Replacement of Left Ureter with Synth Sub, Via Opening */
"0TR77KZ"  =  "1"  /* Replacement of Left Ureter with Nonaut Sub, Via Opening */
"0TR787Z"  =  "1"  /* Replacement of Left Ureter with Autol Sub, Endo */
"0TR78JZ"  =  "1"  /* Replacement of Left Ureter with Synthetic Substitute, Endo */
"0TR78KZ"  =  "1"  /* Replacement of Left Ureter with Nonaut Sub, Endo */
"0TRB07Z"  =  "1"  /* Replacement of Bladder with Autol Sub, Open Approach */
"0TRB0JZ"  =  "1"  /* Replacement of Bladder with Synth Sub, Open Approach */
"0TRB0KZ"  =  "1"  /* Replacement of Bladder with Nonaut Sub, Open Approach */
"0TRB47Z"  =  "1"  /* Replacement of Bladder with Autol Sub, Perc Endo Approach */
"0TRB4JZ"  =  "1"  /* Replacement of Bladder with Synth Sub, Perc Endo Approach */
"0TRB4KZ"  =  "1"  /* Replacement of Bladder with Nonaut Sub, Perc Endo Approach */
"0TRB77Z"  =  "1"  /* Replacement of Bladder with Autol Sub, Via Opening */
"0TRB7JZ"  =  "1"  /* Replacement of Bladder with Synth Sub, Via Opening */
"0TRB7KZ"  =  "1"  /* Replacement of Bladder with Nonaut Sub, Via Opening */
"0TRB87Z"  =  "1"  /* Replacement of Bladder with Autol Sub, Endo */
"0TRB8JZ"  =  "1"  /* Replacement of Bladder with Synthetic Substitute, Endo */
"0TRB8KZ"  =  "1"  /* Replacement of Bladder with Nonaut Sub, Endo */
"0TRC07Z"  =  "1"  /* Replacement of Bladder Neck with Autol Sub, Open Approach */
"0TRC0JZ"  =  "1"  /* Replacement of Bladder Neck with Synth Sub, Open Approach */
"0TRC0KZ"  =  "1"  /* Replacement of Bladder Neck with Nonaut Sub, Open Approach */
"0TRC47Z"  =  "1"  /* Replace of Bladder Neck with Autol Sub, Perc Endo Approach */
"0TRC4JZ"  =  "1"  /* Replace of Bladder Neck with Synth Sub, Perc Endo Approach */
"0TRC4KZ"  =  "1"  /* Replace of Bladder Neck with Nonaut Sub, Perc Endo Approach */
"0TRC77Z"  =  "1"  /* Replacement of Bladder Neck with Autol Sub, Via Opening */
"0TRC7JZ"  =  "1"  /* Replacement of Bladder Neck with Synth Sub, Via Opening */
"0TRC7KZ"  =  "1"  /* Replacement of Bladder Neck with Nonaut Sub, Via Opening */
"0TRC87Z"  =  "1"  /* Replacement of Bladder Neck with Autol Sub, Endo */
"0TRC8JZ"  =  "1"  /* Replacement of Bladder Neck with Synthetic Substitute, Endo */
"0TRC8KZ"  =  "1"  /* Replacement of Bladder Neck with Nonaut Sub, Endo */
"0TRD07Z"  =  "1"  /* Replacement of Urethra with Autol Sub, Open Approach */
"0TRD0JZ"  =  "1"  /* Replacement of Urethra with Synth Sub, Open Approach */
"0TRD0KZ"  =  "1"  /* Replacement of Urethra with Nonaut Sub, Open Approach */
"0TRD47Z"  =  "1"  /* Replacement of Urethra with Autol Sub, Perc Endo Approach */
"0TRD4JZ"  =  "1"  /* Replacement of Urethra with Synth Sub, Perc Endo Approach */
"0TRD4KZ"  =  "1"  /* Replacement of Urethra with Nonaut Sub, Perc Endo Approach */
"0TRD77Z"  =  "1"  /* Replacement of Urethra with Autol Sub, Via Opening */
"0TRD7JZ"  =  "1"  /* Replacement of Urethra with Synth Sub, Via Opening */
"0TRD7KZ"  =  "1"  /* Replacement of Urethra with Nonaut Sub, Via Opening */
"0TRD87Z"  =  "1"  /* Replacement of Urethra with Autol Sub, Endo */
"0TRD8JZ"  =  "1"  /* Replacement of Urethra with Synthetic Substitute, Endo */
"0TRD8KZ"  =  "1"  /* Replacement of Urethra with Nonaut Sub, Endo */
"0TRDX7Z"  =  "1"  /* Replacement of Urethra with Autol Sub, Extern Approach */
"0TRDXJZ"  =  "1"  /* Replacement of Urethra with Synth Sub, Extern Approach */
"0TRDXKZ"  =  "1"  /* Replacement of Urethra with Nonaut Sub, Extern Approach */
"0TS00ZZ"  =  "1"  /* Reposition Right Kidney, Open Approach */
"0TS04ZZ"  =  "1"  /* Reposition Right Kidney, Percutaneous Endoscopic Approach */
"0TS10ZZ"  =  "1"  /* Reposition Left Kidney, Open Approach */
"0TS14ZZ"  =  "1"  /* Reposition Left Kidney, Percutaneous Endoscopic Approach */
"0TS20ZZ"  =  "1"  /* Reposition Bilateral Kidneys, Open Approach */
"0TS24ZZ"  =  "1"  /* Reposition Bilateral Kidneys, Perc Endo Approach */
"0TS30ZZ"  =  "1"  /* Reposition Right Kidney Pelvis, Open Approach */
"0TS34ZZ"  =  "1"  /* Reposition Right Kidney Pelvis, Perc Endo Approach */
"0TS40ZZ"  =  "1"  /* Reposition Left Kidney Pelvis, Open Approach */
"0TS44ZZ"  =  "1"  /* Reposition Left Kidney Pelvis, Perc Endo Approach */
"0TS60ZZ"  =  "1"  /* Reposition Right Ureter, Open Approach */
"0TS64ZZ"  =  "1"  /* Reposition Right Ureter, Percutaneous Endoscopic Approach */
"0TS70ZZ"  =  "1"  /* Reposition Left Ureter, Open Approach */
"0TS74ZZ"  =  "1"  /* Reposition Left Ureter, Percutaneous Endoscopic Approach */
"0TS80ZZ"  =  "1"  /* Reposition Bilateral Ureters, Open Approach */
"0TS84ZZ"  =  "1"  /* Reposition Bilateral Ureters, Perc Endo Approach */
"0TSB0ZZ"  =  "1"  /* Reposition Bladder, Open Approach */
"0TSB4ZZ"  =  "1"  /* Reposition Bladder, Percutaneous Endoscopic Approach */
"0TSC0ZZ"  =  "1"  /* Reposition Bladder Neck, Open Approach */
"0TSC4ZZ"  =  "1"  /* Reposition Bladder Neck, Percutaneous Endoscopic Approach */
"0TSD0ZZ"  =  "1"  /* Reposition Urethra, Open Approach */
"0TSD4ZZ"  =  "1"  /* Reposition Urethra, Percutaneous Endoscopic Approach */
"0TT00ZZ"  =  "1"  /* Resection of Right Kidney, Open Approach */
"0TT04ZZ"  =  "1"  /* Resection of Right Kidney, Percutaneous Endoscopic Approach */
"0TT10ZZ"  =  "1"  /* Resection of Left Kidney, Open Approach */
"0TT14ZZ"  =  "1"  /* Resection of Left Kidney, Percutaneous Endoscopic Approach */
"0TT20ZZ"  =  "1"  /* Resection of Bilateral Kidneys, Open Approach */
"0TT24ZZ"  =  "1"  /* Resection of Bilateral Kidneys, Perc Endo Approach */
"0TT30ZZ"  =  "1"  /* Resection of Right Kidney Pelvis, Open Approach */
"0TT34ZZ"  =  "1"  /* Resection of Right Kidney Pelvis, Perc Endo Approach */
"0TT37ZZ"  =  "1"  /* Resection of Right Kidney Pelvis, Via Opening */
"0TT38ZZ"  =  "1"  /* Resection of Right Kidney Pelvis, Endo */
"0TT40ZZ"  =  "1"  /* Resection of Left Kidney Pelvis, Open Approach */
"0TT44ZZ"  =  "1"  /* Resection of Left Kidney Pelvis, Perc Endo Approach */
"0TT47ZZ"  =  "1"  /* Resection of Left Kidney Pelvis, Via Opening */
"0TT48ZZ"  =  "1"  /* Resection of Left Kidney Pelvis, Endo */
"0TT60ZZ"  =  "1"  /* Resection of Right Ureter, Open Approach */
"0TT64ZZ"  =  "1"  /* Resection of Right Ureter, Percutaneous Endoscopic Approach */
"0TT67ZZ"  =  "1"  /* Resection of Right Ureter, Via Natural or Artificial Opening */
"0TT68ZZ"  =  "1"  /* Resection of Right Ureter, Endo */
"0TT70ZZ"  =  "1"  /* Resection of Left Ureter, Open Approach */
"0TT74ZZ"  =  "1"  /* Resection of Left Ureter, Percutaneous Endoscopic Approach */
"0TT77ZZ"  =  "1"  /* Resection of Left Ureter, Via Natural or Artificial Opening */
"0TT78ZZ"  =  "1"  /* Resection of Left Ureter, Endo */
"0TTB0ZZ"  =  "1"  /* Resection of Bladder, Open Approach */
"0TTB4ZZ"  =  "1"  /* Resection of Bladder, Percutaneous Endoscopic Approach */
"0TTB7ZZ"  =  "1"  /* Resection of Bladder, Via Natural or Artificial Opening */
"0TTB8ZZ"  =  "1"  /* Resection of Bladder, Endo */
"0TTC0ZZ"  =  "1"  /* Resection of Bladder Neck, Open Approach */
"0TTC4ZZ"  =  "1"  /* Resection of Bladder Neck, Percutaneous Endoscopic Approach */
"0TTC7ZZ"  =  "1"  /* Resection of Bladder Neck, Via Natural or Artificial Opening */
"0TTC8ZZ"  =  "1"  /* Resection of Bladder Neck, Endo */
"0TU307Z"  =  "1"  /* Supplement Right Kidney Pelvis with Autol Sub, Open Approach */
"0TU30JZ"  =  "1"  /* Supplement Right Kidney Pelvis with Synth Sub, Open Approach */
"0TU30KZ"  =  "1"  /* Supplement R Kidney Pelvis with Nonaut Sub, Open Approach */
"0TU347Z"  =  "1"  /* Supplement R Kidney Pelvis w Autol Sub, Perc Endo */
"0TU34JZ"  =  "1"  /* Supplement R Kidney Pelvis w Synth Sub, Perc Endo */
"0TU34KZ"  =  "1"  /* Supplement R Kidney Pelvis w Nonaut Sub, Perc Endo */
"0TU377Z"  =  "1"  /* Supplement Right Kidney Pelvis with Autol Sub, Via Opening */
"0TU37JZ"  =  "1"  /* Supplement Right Kidney Pelvis with Synth Sub, Via Opening */
"0TU37KZ"  =  "1"  /* Supplement Right Kidney Pelvis with Nonaut Sub, Via Opening */
"0TU387Z"  =  "1"  /* Supplement Right Kidney Pelvis with Autol Sub, Endo */
"0TU38JZ"  =  "1"  /* Supplement Right Kidney Pelvis with Synth Sub, Endo */
"0TU38KZ"  =  "1"  /* Supplement Right Kidney Pelvis with Nonaut Sub, Endo */
"0TU407Z"  =  "1"  /* Supplement Left Kidney Pelvis with Autol Sub, Open Approach */
"0TU40JZ"  =  "1"  /* Supplement Left Kidney Pelvis with Synth Sub, Open Approach */
"0TU40KZ"  =  "1"  /* Supplement Left Kidney Pelvis with Nonaut Sub, Open Approach */
"0TU447Z"  =  "1"  /* Supplement L Kidney Pelvis w Autol Sub, Perc Endo */
"0TU44JZ"  =  "1"  /* Supplement L Kidney Pelvis w Synth Sub, Perc Endo */
"0TU44KZ"  =  "1"  /* Supplement L Kidney Pelvis w Nonaut Sub, Perc Endo */
"0TU477Z"  =  "1"  /* Supplement Left Kidney Pelvis with Autol Sub, Via Opening */
"0TU47JZ"  =  "1"  /* Supplement Left Kidney Pelvis with Synth Sub, Via Opening */
"0TU47KZ"  =  "1"  /* Supplement Left Kidney Pelvis with Nonaut Sub, Via Opening */
"0TU487Z"  =  "1"  /* Supplement Left Kidney Pelvis with Autol Sub, Endo */
"0TU48JZ"  =  "1"  /* Supplement Left Kidney Pelvis with Synth Sub, Endo */
"0TU48KZ"  =  "1"  /* Supplement Left Kidney Pelvis with Nonaut Sub, Endo */
"0TU607Z"  =  "1"  /* Supplement Right Ureter with Autol Sub, Open Approach */
"0TU60JZ"  =  "1"  /* Supplement Right Ureter with Synth Sub, Open Approach */
"0TU60KZ"  =  "1"  /* Supplement Right Ureter with Nonaut Sub, Open Approach */
"0TU647Z"  =  "1"  /* Supplement Right Ureter with Autol Sub, Perc Endo Approach */
"0TU64JZ"  =  "1"  /* Supplement Right Ureter with Synth Sub, Perc Endo Approach */
"0TU64KZ"  =  "1"  /* Supplement Right Ureter with Nonaut Sub, Perc Endo Approach */
"0TU677Z"  =  "1"  /* Supplement Right Ureter with Autol Sub, Via Opening */
"0TU67JZ"  =  "1"  /* Supplement Right Ureter with Synth Sub, Via Opening */
"0TU67KZ"  =  "1"  /* Supplement Right Ureter with Nonaut Sub, Via Opening */
"0TU687Z"  =  "1"  /* Supplement Right Ureter with Autol Sub, Endo */
"0TU68JZ"  =  "1"  /* Supplement Right Ureter with Synthetic Substitute, Endo */
"0TU68KZ"  =  "1"  /* Supplement Right Ureter with Nonaut Sub, Endo */
"0TU707Z"  =  "1"  /* Supplement Left Ureter with Autol Sub, Open Approach */
"0TU70JZ"  =  "1"  /* Supplement Left Ureter with Synth Sub, Open Approach */
"0TU70KZ"  =  "1"  /* Supplement Left Ureter with Nonaut Sub, Open Approach */
"0TU747Z"  =  "1"  /* Supplement Left Ureter with Autol Sub, Perc Endo Approach */
"0TU74JZ"  =  "1"  /* Supplement Left Ureter with Synth Sub, Perc Endo Approach */
"0TU74KZ"  =  "1"  /* Supplement Left Ureter with Nonaut Sub, Perc Endo Approach */
"0TU777Z"  =  "1"  /* Supplement Left Ureter with Autol Sub, Via Opening */
"0TU77JZ"  =  "1"  /* Supplement Left Ureter with Synth Sub, Via Opening */
"0TU77KZ"  =  "1"  /* Supplement Left Ureter with Nonaut Sub, Via Opening */
"0TU787Z"  =  "1"  /* Supplement Left Ureter with Autol Sub, Endo */
"0TU78JZ"  =  "1"  /* Supplement Left Ureter with Synthetic Substitute, Endo */
"0TU78KZ"  =  "1"  /* Supplement Left Ureter with Nonaut Sub, Endo */
"0TUB07Z"  =  "1"  /* Supplement Bladder with Autol Sub, Open Approach */
"0TUB0JZ"  =  "1"  /* Supplement Bladder with Synthetic Substitute, Open Approach */
"0TUB0KZ"  =  "1"  /* Supplement Bladder with Nonaut Sub, Open Approach */
"0TUB47Z"  =  "1"  /* Supplement Bladder with Autol Sub, Perc Endo Approach */
"0TUB4JZ"  =  "1"  /* Supplement Bladder with Synth Sub, Perc Endo Approach */
"0TUB4KZ"  =  "1"  /* Supplement Bladder with Nonaut Sub, Perc Endo Approach */
"0TUB77Z"  =  "1"  /* Supplement Bladder with Autol Sub, Via Opening */
"0TUB7JZ"  =  "1"  /* Supplement Bladder with Synthetic Substitute, Via Opening */
"0TUB7KZ"  =  "1"  /* Supplement Bladder with Nonaut Sub, Via Opening */
"0TUB87Z"  =  "1"  /* Supplement Bladder with Autologous Tissue Substitute, Endo */
"0TUB8JZ"  =  "1"  /* Supplement Bladder with Synthetic Substitute, Endo */
"0TUB8KZ"  =  "1"  /* Supplement Bladder with Nonaut Sub, Endo */
"0TUC07Z"  =  "1"  /* Supplement Bladder Neck with Autol Sub, Open Approach */
"0TUC0JZ"  =  "1"  /* Supplement Bladder Neck with Synth Sub, Open Approach */
"0TUC0KZ"  =  "1"  /* Supplement Bladder Neck with Nonaut Sub, Open Approach */
"0TUC47Z"  =  "1"  /* Supplement Bladder Neck with Autol Sub, Perc Endo Approach */
"0TUC4JZ"  =  "1"  /* Supplement Bladder Neck with Synth Sub, Perc Endo Approach */
"0TUC4KZ"  =  "1"  /* Supplement Bladder Neck with Nonaut Sub, Perc Endo Approach */
"0TUC77Z"  =  "1"  /* Supplement Bladder Neck with Autol Sub, Via Opening */
"0TUC7JZ"  =  "1"  /* Supplement Bladder Neck with Synth Sub, Via Opening */
"0TUC7KZ"  =  "1"  /* Supplement Bladder Neck with Nonaut Sub, Via Opening */
"0TUC87Z"  =  "1"  /* Supplement Bladder Neck with Autol Sub, Endo */
"0TUC8JZ"  =  "1"  /* Supplement Bladder Neck with Synthetic Substitute, Endo */
"0TUC8KZ"  =  "1"  /* Supplement Bladder Neck with Nonaut Sub, Endo */
"0TUD07Z"  =  "1"  /* Supplement Urethra with Autol Sub, Open Approach */
"0TUD0JZ"  =  "1"  /* Supplement Urethra with Synthetic Substitute, Open Approach */
"0TUD0KZ"  =  "1"  /* Supplement Urethra with Nonaut Sub, Open Approach */
"0TUD47Z"  =  "1"  /* Supplement Urethra with Autol Sub, Perc Endo Approach */
"0TUD4JZ"  =  "1"  /* Supplement Urethra with Synth Sub, Perc Endo Approach */
"0TUD4KZ"  =  "1"  /* Supplement Urethra with Nonaut Sub, Perc Endo Approach */
"0TUD77Z"  =  "1"  /* Supplement Urethra with Autol Sub, Via Opening */
"0TUD7JZ"  =  "1"  /* Supplement Urethra with Synthetic Substitute, Via Opening */
"0TUD7KZ"  =  "1"  /* Supplement Urethra with Nonaut Sub, Via Opening */
"0TUD87Z"  =  "1"  /* Supplement Urethra with Autologous Tissue Substitute, Endo */
"0TUD8JZ"  =  "1"  /* Supplement Urethra with Synthetic Substitute, Endo */
"0TUD8KZ"  =  "1"  /* Supplement Urethra with Nonaut Sub, Endo */
"0TUDX7Z"  =  "1"  /* Supplement Urethra with Autol Sub, Extern Approach */
"0TUDXJZ"  =  "1"  /* Supplement Urethra with Synth Sub, Extern Approach */
"0TUDXKZ"  =  "1"  /* Supplement Urethra with Nonaut Sub, Extern Approach */
"0TV30CZ"  =  "1"  /* Restrict of R Kidney Pelvis with Extralum Dev, Open Approach */
"0TV30DZ"  =  "1"  /* Restrict of R Kidney Pelvis with Intralum Dev, Open Approach */
"0TV30ZZ"  =  "1"  /* Restriction of Right Kidney Pelvis, Open Approach */
"0TV33CZ"  =  "1"  /* Restrict of R Kidney Pelvis with Extralum Dev, Perc Approach */
"0TV33DZ"  =  "1"  /* Restrict of R Kidney Pelvis with Intralum Dev, Perc Approach */
"0TV33ZZ"  =  "1"  /* Restriction of Right Kidney Pelvis, Percutaneous Approach */
"0TV34CZ"  =  "1"  /* Restrict R Kidney Pelvis w Extralum Dev, Perc Endo */
"0TV34DZ"  =  "1"  /* Restrict R Kidney Pelvis w Intralum Dev, Perc Endo */
"0TV34ZZ"  =  "1"  /* Restriction of Right Kidney Pelvis, Perc Endo Approach */
"0TV37DZ"  =  "1"  /* Restrict of R Kidney Pelvis with Intralum Dev, Via Opening */
"0TV37ZZ"  =  "1"  /* Restriction of Right Kidney Pelvis, Via Opening */
"0TV38DZ"  =  "1"  /* Restriction of Right Kidney Pelvis with Intralum Dev, Endo */
"0TV38ZZ"  =  "1"  /* Restriction of Right Kidney Pelvis, Endo */
"0TV40CZ"  =  "1"  /* Restrict of L Kidney Pelvis with Extralum Dev, Open Approach */
"0TV40DZ"  =  "1"  /* Restrict of L Kidney Pelvis with Intralum Dev, Open Approach */
"0TV40ZZ"  =  "1"  /* Restriction of Left Kidney Pelvis, Open Approach */
"0TV43CZ"  =  "1"  /* Restrict of L Kidney Pelvis with Extralum Dev, Perc Approach */
"0TV43DZ"  =  "1"  /* Restrict of L Kidney Pelvis with Intralum Dev, Perc Approach */
"0TV43ZZ"  =  "1"  /* Restriction of Left Kidney Pelvis, Percutaneous Approach */
"0TV44CZ"  =  "1"  /* Restrict L Kidney Pelvis w Extralum Dev, Perc Endo */
"0TV44DZ"  =  "1"  /* Restrict L Kidney Pelvis w Intralum Dev, Perc Endo */
"0TV44ZZ"  =  "1"  /* Restriction of Left Kidney Pelvis, Perc Endo Approach */
"0TV47DZ"  =  "1"  /* Restrict of L Kidney Pelvis with Intralum Dev, Via Opening */
"0TV47ZZ"  =  "1"  /* Restriction of Left Kidney Pelvis, Via Opening */
"0TV48DZ"  =  "1"  /* Restriction of Left Kidney Pelvis with Intralum Dev, Endo */
"0TV48ZZ"  =  "1"  /* Restriction of Left Kidney Pelvis, Endo */
"0TV60CZ"  =  "1"  /* Restriction of Right Ureter with Extralum Dev, Open Approach */
"0TV60DZ"  =  "1"  /* Restriction of Right Ureter with Intralum Dev, Open Approach */
"0TV60ZZ"  =  "1"  /* Restriction of Right Ureter, Open Approach */
"0TV63CZ"  =  "1"  /* Restriction of Right Ureter with Extralum Dev, Perc Approach */
"0TV63DZ"  =  "1"  /* Restriction of Right Ureter with Intralum Dev, Perc Approach */
"0TV63ZZ"  =  "1"  /* Restriction of Right Ureter, Percutaneous Approach */
"0TV64CZ"  =  "1"  /* Restrict of R Ureter with Extralum Dev, Perc Endo Approach */
"0TV64DZ"  =  "1"  /* Restrict of R Ureter with Intralum Dev, Perc Endo Approach */
"0TV64ZZ"  =  "1"  /* Restriction of Right Ureter, Perc Endo Approach */
"0TV67DZ"  =  "1"  /* Restriction of Right Ureter with Intralum Dev, Via Opening */
"0TV67ZZ"  =  "1"  /* Restriction of Right Ureter, Via Opening */
"0TV68DZ"  =  "1"  /* Restriction of Right Ureter with Intraluminal Device, Endo */
"0TV68ZZ"  =  "1"  /* Restriction of Right Ureter, Endo */
"0TV70CZ"  =  "1"  /* Restriction of Left Ureter with Extralum Dev, Open Approach */
"0TV70DZ"  =  "1"  /* Restriction of Left Ureter with Intralum Dev, Open Approach */
"0TV70ZZ"  =  "1"  /* Restriction of Left Ureter, Open Approach */
"0TV73CZ"  =  "1"  /* Restriction of Left Ureter with Extralum Dev, Perc Approach */
"0TV73DZ"  =  "1"  /* Restriction of Left Ureter with Intralum Dev, Perc Approach */
"0TV73ZZ"  =  "1"  /* Restriction of Left Ureter, Percutaneous Approach */
"0TV74CZ"  =  "1"  /* Restrict of L Ureter with Extralum Dev, Perc Endo Approach */
"0TV74DZ"  =  "1"  /* Restrict of L Ureter with Intralum Dev, Perc Endo Approach */
"0TV74ZZ"  =  "1"  /* Restriction of Left Ureter, Percutaneous Endoscopic Approach */
"0TV77DZ"  =  "1"  /* Restriction of Left Ureter with Intralum Dev, Via Opening */
"0TV77ZZ"  =  "1"  /* Restriction of Left Ureter, Via Opening */
"0TV78DZ"  =  "1"  /* Restriction of Left Ureter with Intraluminal Device, Endo */
"0TV78ZZ"  =  "1"  /* Restriction of Left Ureter, Endo */
"0TVB0CZ"  =  "1"  /* Restriction of Bladder with Extralum Dev, Open Approach */
"0TVB0DZ"  =  "1"  /* Restriction of Bladder with Intralum Dev, Open Approach */
"0TVB0ZZ"  =  "1"  /* Restriction of Bladder, Open Approach */
"0TVB3CZ"  =  "1"  /* Restriction of Bladder with Extralum Dev, Perc Approach */
"0TVB3DZ"  =  "1"  /* Restriction of Bladder with Intralum Dev, Perc Approach */
"0TVB3ZZ"  =  "1"  /* Restriction of Bladder, Percutaneous Approach */
"0TVB4CZ"  =  "1"  /* Restriction of Bladder with Extralum Dev, Perc Endo Approach */
"0TVB4DZ"  =  "1"  /* Restriction of Bladder with Intralum Dev, Perc Endo Approach */
"0TVB4ZZ"  =  "1"  /* Restriction of Bladder, Percutaneous Endoscopic Approach */
"0TVB7DZ"  =  "1"  /* Restriction of Bladder with Intraluminal Device, Via Opening */
"0TVB7ZZ"  =  "1"  /* Restriction of Bladder, Via Natural or Artificial Opening */
"0TVB8DZ"  =  "1"  /* Restriction of Bladder with Intraluminal Device, Endo */
"0TVB8ZZ"  =  "1"  /* Restriction of Bladder, Endo */
"0TVC0CZ"  =  "1"  /* Restriction of Bladder Neck with Extralum Dev, Open Approach */
"0TVC0DZ"  =  "1"  /* Restriction of Bladder Neck with Intralum Dev, Open Approach */
"0TVC0ZZ"  =  "1"  /* Restriction of Bladder Neck, Open Approach */
"0TVC3CZ"  =  "1"  /* Restriction of Bladder Neck with Extralum Dev, Perc Approach */
"0TVC3DZ"  =  "1"  /* Restriction of Bladder Neck with Intralum Dev, Perc Approach */
"0TVC3ZZ"  =  "1"  /* Restriction of Bladder Neck, Percutaneous Approach */
"0TVC4CZ"  =  "1"  /* Restrict Bladder Neck w Extralum Dev, Perc Endo */
"0TVC4DZ"  =  "1"  /* Restrict Bladder Neck w Intralum Dev, Perc Endo */
"0TVC4ZZ"  =  "1"  /* Restriction of Bladder Neck, Perc Endo Approach */
"0TVC7DZ"  =  "1"  /* Restriction of Bladder Neck with Intralum Dev, Via Opening */
"0TVC7ZZ"  =  "1"  /* Restriction of Bladder Neck, Via Opening */
"0TVC8DZ"  =  "1"  /* Restriction of Bladder Neck with Intraluminal Device, Endo */
"0TVC8ZZ"  =  "1"  /* Restriction of Bladder Neck, Endo */
"0TVD0CZ"  =  "1"  /* Restriction of Urethra with Extralum Dev, Open Approach */
"0TVD0DZ"  =  "1"  /* Restriction of Urethra with Intralum Dev, Open Approach */
"0TVD0ZZ"  =  "1"  /* Restriction of Urethra, Open Approach */
"0TVD3CZ"  =  "1"  /* Restriction of Urethra with Extralum Dev, Perc Approach */
"0TVD3DZ"  =  "1"  /* Restriction of Urethra with Intralum Dev, Perc Approach */
"0TVD3ZZ"  =  "1"  /* Restriction of Urethra, Percutaneous Approach */
"0TVD4CZ"  =  "1"  /* Restriction of Urethra with Extralum Dev, Perc Endo Approach */
"0TVD4DZ"  =  "1"  /* Restriction of Urethra with Intralum Dev, Perc Endo Approach */
"0TVD4ZZ"  =  "1"  /* Restriction of Urethra, Percutaneous Endoscopic Approach */
"0TVD7DZ"  =  "1"  /* Restriction of Urethra with Intraluminal Device, Via Opening */
"0TVD7ZZ"  =  "1"  /* Restriction of Urethra, Via Natural or Artificial Opening */
"0TVD8DZ"  =  "1"  /* Restriction of Urethra with Intraluminal Device, Endo */
"0TVD8ZZ"  =  "1"  /* Restriction of Urethra, Endo */
"0TVDXZZ"  =  "1"  /* Restriction of Urethra, External Approach */
"0TW500Z"  =  "1"  /* Revision of Drainage Device in Kidney, Open Approach */
"0TW502Z"  =  "1"  /* Revision of Monitoring Device in Kidney, Open Approach */
"0TW503Z"  =  "1"  /* Revision of Infusion Device in Kidney, Open Approach */
"0TW507Z"  =  "1"  /* Revision of Autol Sub in Kidney, Open Approach */
"0TW50CZ"  =  "1"  /* Revision of Extraluminal Device in Kidney, Open Approach */
"0TW50DZ"  =  "1"  /* Revision of Intraluminal Device in Kidney, Open Approach */
"0TW50JZ"  =  "1"  /* Revision of Synthetic Substitute in Kidney, Open Approach */
"0TW50KZ"  =  "1"  /* Revision of Nonaut Sub in Kidney, Open Approach */
"0TW530Z"  =  "1"  /* Revision of Drainage Device in Kidney, Percutaneous Approach */
"0TW532Z"  =  "1"  /* Revision of Monitoring Device in Kidney, Perc Approach */
"0TW533Z"  =  "1"  /* Revision of Infusion Device in Kidney, Percutaneous Approach */
"0TW537Z"  =  "1"  /* Revision of Autol Sub in Kidney, Perc Approach */
"0TW53CZ"  =  "1"  /* Revision of Extraluminal Device in Kidney, Perc Approach */
"0TW53DZ"  =  "1"  /* Revision of Intraluminal Device in Kidney, Perc Approach */
"0TW53JZ"  =  "1"  /* Revision of Synthetic Substitute in Kidney, Perc Approach */
"0TW53KZ"  =  "1"  /* Revision of Nonaut Sub in Kidney, Perc Approach */
"0TW540Z"  =  "1"  /* Revision of Drainage Device in Kidney, Perc Endo Approach */
"0TW542Z"  =  "1"  /* Revision of Monitoring Device in Kidney, Perc Endo Approach */
"0TW543Z"  =  "1"  /* Revision of Infusion Device in Kidney, Perc Endo Approach */
"0TW547Z"  =  "1"  /* Revision of Autol Sub in Kidney, Perc Endo Approach */
"0TW54CZ"  =  "1"  /* Revision of Extralum Dev in Kidney, Perc Endo Approach */
"0TW54DZ"  =  "1"  /* Revision of Intralum Dev in Kidney, Perc Endo Approach */
"0TW54JZ"  =  "1"  /* Revision of Synth Sub in Kidney, Perc Endo Approach */
"0TW54KZ"  =  "1"  /* Revision of Nonaut Sub in Kidney, Perc Endo Approach */
"0TW570Z"  =  "1"  /* Revision of Drainage Device in Kidney, Via Opening */
"0TW572Z"  =  "1"  /* Revision of Monitoring Device in Kidney, Via Opening */
"0TW573Z"  =  "1"  /* Revision of Infusion Device in Kidney, Via Opening */
"0TW577Z"  =  "1"  /* Revision of Autol Sub in Kidney, Via Opening */
"0TW57CZ"  =  "1"  /* Revision of Extraluminal Device in Kidney, Via Opening */
"0TW57DZ"  =  "1"  /* Revision of Intraluminal Device in Kidney, Via Opening */
"0TW57JZ"  =  "1"  /* Revision of Synthetic Substitute in Kidney, Via Opening */
"0TW57KZ"  =  "1"  /* Revision of Nonaut Sub in Kidney, Via Opening */
"0TW580Z"  =  "1"  /* Revision of Drainage Device in Kidney, Endo */
"0TW582Z"  =  "1"  /* Revision of Monitoring Device in Kidney, Endo */
"0TW583Z"  =  "1"  /* Revision of Infusion Device in Kidney, Endo */
"0TW587Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Kidney, Endo */
"0TW58CZ"  =  "1"  /* Revision of Extraluminal Device in Kidney, Endo */
"0TW58DZ"  =  "1"  /* Revision of Intraluminal Device in Kidney, Endo */
"0TW58JZ"  =  "1"  /* Revision of Synthetic Substitute in Kidney, Endo */
"0TW58KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Kidney, Endo */
"0TW900Z"  =  "1"  /* Revision of Drainage Device in Ureter, Open Approach */
"0TW902Z"  =  "1"  /* Revision of Monitoring Device in Ureter, Open Approach */
"0TW903Z"  =  "1"  /* Revision of Infusion Device in Ureter, Open Approach */
"0TW907Z"  =  "1"  /* Revision of Autol Sub in Ureter, Open Approach */
"0TW90CZ"  =  "1"  /* Revision of Extraluminal Device in Ureter, Open Approach */
"0TW90DZ"  =  "1"  /* Revision of Intraluminal Device in Ureter, Open Approach */
"0TW90JZ"  =  "1"  /* Revision of Synthetic Substitute in Ureter, Open Approach */
"0TW90KZ"  =  "1"  /* Revision of Nonaut Sub in Ureter, Open Approach */
"0TW90MZ"  =  "1"  /* Revision of Stimulator Lead in Ureter, Open Approach */
"0TW930Z"  =  "1"  /* Revision of Drainage Device in Ureter, Percutaneous Approach */
"0TW932Z"  =  "1"  /* Revision of Monitoring Device in Ureter, Perc Approach */
"0TW933Z"  =  "1"  /* Revision of Infusion Device in Ureter, Percutaneous Approach */
"0TW937Z"  =  "1"  /* Revision of Autol Sub in Ureter, Perc Approach */
"0TW93CZ"  =  "1"  /* Revision of Extraluminal Device in Ureter, Perc Approach */
"0TW93DZ"  =  "1"  /* Revision of Intraluminal Device in Ureter, Perc Approach */
"0TW93JZ"  =  "1"  /* Revision of Synthetic Substitute in Ureter, Perc Approach */
"0TW93KZ"  =  "1"  /* Revision of Nonaut Sub in Ureter, Perc Approach */
"0TW93MZ"  =  "1"  /* Revision of Stimulator Lead in Ureter, Percutaneous Approach */
"0TW940Z"  =  "1"  /* Revision of Drainage Device in Ureter, Perc Endo Approach */
"0TW942Z"  =  "1"  /* Revision of Monitoring Device in Ureter, Perc Endo Approach */
"0TW943Z"  =  "1"  /* Revision of Infusion Device in Ureter, Perc Endo Approach */
"0TW947Z"  =  "1"  /* Revision of Autol Sub in Ureter, Perc Endo Approach */
"0TW94CZ"  =  "1"  /* Revision of Extralum Dev in Ureter, Perc Endo Approach */
"0TW94DZ"  =  "1"  /* Revision of Intralum Dev in Ureter, Perc Endo Approach */
"0TW94JZ"  =  "1"  /* Revision of Synth Sub in Ureter, Perc Endo Approach */
"0TW94KZ"  =  "1"  /* Revision of Nonaut Sub in Ureter, Perc Endo Approach */
"0TW94MZ"  =  "1"  /* Revision of Stimulator Lead in Ureter, Perc Endo Approach */
"0TW970Z"  =  "1"  /* Revision of Drainage Device in Ureter, Via Opening */
"0TW972Z"  =  "1"  /* Revision of Monitoring Device in Ureter, Via Opening */
"0TW973Z"  =  "1"  /* Revision of Infusion Device in Ureter, Via Opening */
"0TW977Z"  =  "1"  /* Revision of Autol Sub in Ureter, Via Opening */
"0TW97CZ"  =  "1"  /* Revision of Extraluminal Device in Ureter, Via Opening */
"0TW97DZ"  =  "1"  /* Revision of Intraluminal Device in Ureter, Via Opening */
"0TW97JZ"  =  "1"  /* Revision of Synthetic Substitute in Ureter, Via Opening */
"0TW97KZ"  =  "1"  /* Revision of Nonaut Sub in Ureter, Via Opening */
"0TW97MZ"  =  "1"  /* Revision of Stimulator Lead in Ureter, Via Opening */
"0TW980Z"  =  "1"  /* Revision of Drainage Device in Ureter, Endo */
"0TW982Z"  =  "1"  /* Revision of Monitoring Device in Ureter, Endo */
"0TW983Z"  =  "1"  /* Revision of Infusion Device in Ureter, Endo */
"0TW987Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Ureter, Endo */
"0TW98CZ"  =  "1"  /* Revision of Extraluminal Device in Ureter, Endo */
"0TW98DZ"  =  "1"  /* Revision of Intraluminal Device in Ureter, Endo */
"0TW98JZ"  =  "1"  /* Revision of Synthetic Substitute in Ureter, Endo */
"0TW98KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Ureter, Endo */
"0TW98MZ"  =  "1"  /* Revision of Stimulator Lead in Ureter, Endo */
"0TWB00Z"  =  "1"  /* Revision of Drainage Device in Bladder, Open Approach */
"0TWB02Z"  =  "1"  /* Revision of Monitoring Device in Bladder, Open Approach */
"0TWB03Z"  =  "1"  /* Revision of Infusion Device in Bladder, Open Approach */
"0TWB07Z"  =  "1"  /* Revision of Autol Sub in Bladder, Open Approach */
"0TWB0CZ"  =  "1"  /* Revision of Extraluminal Device in Bladder, Open Approach */
"0TWB0DZ"  =  "1"  /* Revision of Intraluminal Device in Bladder, Open Approach */
"0TWB0JZ"  =  "1"  /* Revision of Synthetic Substitute in Bladder, Open Approach */
"0TWB0KZ"  =  "1"  /* Revision of Nonaut Sub in Bladder, Open Approach */
"0TWB0LZ"  =  "1"  /* Revision of Artificial Sphincter in Bladder, Open Approach */
"0TWB0MZ"  =  "1"  /* Revision of Stimulator Lead in Bladder, Open Approach */
"0TWB30Z"  =  "1"  /* Revision of Drainage Device in Bladder, Perc Approach */
"0TWB32Z"  =  "1"  /* Revision of Monitoring Device in Bladder, Perc Approach */
"0TWB33Z"  =  "1"  /* Revision of Infusion Device in Bladder, Perc Approach */
"0TWB37Z"  =  "1"  /* Revision of Autol Sub in Bladder, Perc Approach */
"0TWB3CZ"  =  "1"  /* Revision of Extraluminal Device in Bladder, Perc Approach */
"0TWB3DZ"  =  "1"  /* Revision of Intraluminal Device in Bladder, Perc Approach */
"0TWB3JZ"  =  "1"  /* Revision of Synthetic Substitute in Bladder, Perc Approach */
"0TWB3KZ"  =  "1"  /* Revision of Nonaut Sub in Bladder, Perc Approach */
"0TWB3LZ"  =  "1"  /* Revision of Artificial Sphincter in Bladder, Perc Approach */
"0TWB3MZ"  =  "1"  /* Revision of Stimulator Lead in Bladder, Perc Approach */
"0TWB40Z"  =  "1"  /* Revision of Drainage Device in Bladder, Perc Endo Approach */
"0TWB42Z"  =  "1"  /* Revision of Monitoring Device in Bladder, Perc Endo Approach */
"0TWB43Z"  =  "1"  /* Revision of Infusion Device in Bladder, Perc Endo Approach */
"0TWB47Z"  =  "1"  /* Revision of Autol Sub in Bladder, Perc Endo Approach */
"0TWB4CZ"  =  "1"  /* Revision of Extralum Dev in Bladder, Perc Endo Approach */
"0TWB4DZ"  =  "1"  /* Revision of Intralum Dev in Bladder, Perc Endo Approach */
"0TWB4JZ"  =  "1"  /* Revision of Synth Sub in Bladder, Perc Endo Approach */
"0TWB4KZ"  =  "1"  /* Revision of Nonaut Sub in Bladder, Perc Endo Approach */
"0TWB4LZ"  =  "1"  /* Revision of Artif Sphincter in Bladder, Perc Endo Approach */
"0TWB4MZ"  =  "1"  /* Revision of Stimulator Lead in Bladder, Perc Endo Approach */
"0TWB70Z"  =  "1"  /* Revision of Drainage Device in Bladder, Via Opening */
"0TWB72Z"  =  "1"  /* Revision of Monitoring Device in Bladder, Via Opening */
"0TWB73Z"  =  "1"  /* Revision of Infusion Device in Bladder, Via Opening */
"0TWB77Z"  =  "1"  /* Revision of Autol Sub in Bladder, Via Opening */
"0TWB7CZ"  =  "1"  /* Revision of Extraluminal Device in Bladder, Via Opening */
"0TWB7DZ"  =  "1"  /* Revision of Intraluminal Device in Bladder, Via Opening */
"0TWB7JZ"  =  "1"  /* Revision of Synthetic Substitute in Bladder, Via Opening */
"0TWB7KZ"  =  "1"  /* Revision of Nonaut Sub in Bladder, Via Opening */
"0TWB7LZ"  =  "1"  /* Revision of Artificial Sphincter in Bladder, Via Opening */
"0TWB7MZ"  =  "1"  /* Revision of Stimulator Lead in Bladder, Via Opening */
"0TWB80Z"  =  "1"  /* Revision of Drainage Device in Bladder, Endo */
"0TWB82Z"  =  "1"  /* Revision of Monitoring Device in Bladder, Endo */
"0TWB83Z"  =  "1"  /* Revision of Infusion Device in Bladder, Endo */
"0TWB87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Bladder, Endo */
"0TWB8CZ"  =  "1"  /* Revision of Extraluminal Device in Bladder, Endo */
"0TWB8DZ"  =  "1"  /* Revision of Intraluminal Device in Bladder, Endo */
"0TWB8JZ"  =  "1"  /* Revision of Synthetic Substitute in Bladder, Endo */
"0TWB8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Bladder, Endo */
"0TWB8LZ"  =  "1"  /* Revision of Artificial Sphincter in Bladder, Endo */
"0TWB8MZ"  =  "1"  /* Revision of Stimulator Lead in Bladder, Endo */
"0TWD00Z"  =  "1"  /* Revision of Drainage Device in Urethra, Open Approach */
"0TWD02Z"  =  "1"  /* Revision of Monitoring Device in Urethra, Open Approach */
"0TWD03Z"  =  "1"  /* Revision of Infusion Device in Urethra, Open Approach */
"0TWD07Z"  =  "1"  /* Revision of Autol Sub in Urethra, Open Approach */
"0TWD0CZ"  =  "1"  /* Revision of Extraluminal Device in Urethra, Open Approach */
"0TWD0DZ"  =  "1"  /* Revision of Intraluminal Device in Urethra, Open Approach */
"0TWD0JZ"  =  "1"  /* Revision of Synthetic Substitute in Urethra, Open Approach */
"0TWD0KZ"  =  "1"  /* Revision of Nonaut Sub in Urethra, Open Approach */
"0TWD0LZ"  =  "1"  /* Revision of Artificial Sphincter in Urethra, Open Approach */
"0TWD30Z"  =  "1"  /* Revision of Drainage Device in Urethra, Perc Approach */
"0TWD32Z"  =  "1"  /* Revision of Monitoring Device in Urethra, Perc Approach */
"0TWD33Z"  =  "1"  /* Revision of Infusion Device in Urethra, Perc Approach */
"0TWD37Z"  =  "1"  /* Revision of Autol Sub in Urethra, Perc Approach */
"0TWD3CZ"  =  "1"  /* Revision of Extraluminal Device in Urethra, Perc Approach */
"0TWD3DZ"  =  "1"  /* Revision of Intraluminal Device in Urethra, Perc Approach */
"0TWD3JZ"  =  "1"  /* Revision of Synthetic Substitute in Urethra, Perc Approach */
"0TWD3KZ"  =  "1"  /* Revision of Nonaut Sub in Urethra, Perc Approach */
"0TWD3LZ"  =  "1"  /* Revision of Artificial Sphincter in Urethra, Perc Approach */
"0TWD40Z"  =  "1"  /* Revision of Drainage Device in Urethra, Perc Endo Approach */
"0TWD42Z"  =  "1"  /* Revision of Monitoring Device in Urethra, Perc Endo Approach */
"0TWD43Z"  =  "1"  /* Revision of Infusion Device in Urethra, Perc Endo Approach */
"0TWD47Z"  =  "1"  /* Revision of Autol Sub in Urethra, Perc Endo Approach */
"0TWD4CZ"  =  "1"  /* Revision of Extralum Dev in Urethra, Perc Endo Approach */
"0TWD4DZ"  =  "1"  /* Revision of Intralum Dev in Urethra, Perc Endo Approach */
"0TWD4JZ"  =  "1"  /* Revision of Synth Sub in Urethra, Perc Endo Approach */
"0TWD4KZ"  =  "1"  /* Revision of Nonaut Sub in Urethra, Perc Endo Approach */
"0TWD4LZ"  =  "1"  /* Revision of Artif Sphincter in Urethra, Perc Endo Approach */
"0TWD70Z"  =  "1"  /* Revision of Drainage Device in Urethra, Via Opening */
"0TWD72Z"  =  "1"  /* Revision of Monitoring Device in Urethra, Via Opening */
"0TWD73Z"  =  "1"  /* Revision of Infusion Device in Urethra, Via Opening */
"0TWD77Z"  =  "1"  /* Revision of Autol Sub in Urethra, Via Opening */
"0TWD7CZ"  =  "1"  /* Revision of Extraluminal Device in Urethra, Via Opening */
"0TWD7DZ"  =  "1"  /* Revision of Intraluminal Device in Urethra, Via Opening */
"0TWD7JZ"  =  "1"  /* Revision of Synthetic Substitute in Urethra, Via Opening */
"0TWD7KZ"  =  "1"  /* Revision of Nonaut Sub in Urethra, Via Opening */
"0TWD7LZ"  =  "1"  /* Revision of Artificial Sphincter in Urethra, Via Opening */
"0TWD80Z"  =  "1"  /* Revision of Drainage Device in Urethra, Endo */
"0TWD82Z"  =  "1"  /* Revision of Monitoring Device in Urethra, Endo */
"0TWD83Z"  =  "1"  /* Revision of Infusion Device in Urethra, Endo */
"0TWD87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Urethra, Endo */
"0TWD8CZ"  =  "1"  /* Revision of Extraluminal Device in Urethra, Endo */
"0TWD8DZ"  =  "1"  /* Revision of Intraluminal Device in Urethra, Endo */
"0TWD8JZ"  =  "1"  /* Revision of Synthetic Substitute in Urethra, Endo */
"0TWD8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Urethra, Endo */
"0TWD8LZ"  =  "1"  /* Revision of Artificial Sphincter in Urethra, Endo */
"0TY00Z0"  =  "1"  /* Transplantation of Right Kidney, Allogeneic, Open Approach */
"0TY00Z1"  =  "1"  /* Transplantation of Right Kidney, Syngeneic, Open Approach */
"0TY00Z2"  =  "1"  /* Transplantation of Right Kidney, Zooplastic, Open Approach */
"0TY10Z0"  =  "1"  /* Transplantation of Left Kidney, Allogeneic, Open Approach */
"0TY10Z1"  =  "1"  /* Transplantation of Left Kidney, Syngeneic, Open Approach */
"0TY10Z2"  =  "1"  /* Transplantation of Left Kidney, Zooplastic, Open Approach */
"0U15075"  =  "1"  /* Bypass R Fallopian Tube to R Fallop w Autol Sub, Open */
"0U15076"  =  "1"  /* Bypass R Fallopian Tube to L Fallop w Autol Sub, Open */
"0U15079"  =  "1"  /* Bypass R Fallopian Tube to Uterus w Autol Sub, Open */
"0U150J5"  =  "1"  /* Bypass R Fallopian Tube to R Fallop w Synth Sub, Open */
"0U150J6"  =  "1"  /* Bypass R Fallopian Tube to L Fallop w Synth Sub, Open */
"0U150J9"  =  "1"  /* Bypass R Fallopian Tube to Uterus w Synth Sub, Open */
"0U150K5"  =  "1"  /* Bypass R Fallopian Tube to R Fallop w Nonaut Sub, Open */
"0U150K6"  =  "1"  /* Bypass R Fallopian Tube to L Fallop w Nonaut Sub, Open */
"0U150K9"  =  "1"  /* Bypass R Fallopian Tube to Uterus w Nonaut Sub, Open */
"0U150Z5"  =  "1"  /* Bypass Right Fallopian Tube to R Fallop, Open Approach */
"0U150Z6"  =  "1"  /* Bypass Right Fallopian Tube to L Fallop, Open Approach */
"0U150Z9"  =  "1"  /* Bypass Right Fallopian Tube to Uterus, Open Approach */
"0U15475"  =  "1"  /* Bypass R Fallopian Tube to R Fallop w Autol Sub, Perc Endo */
"0U15476"  =  "1"  /* Bypass R Fallopian Tube to L Fallop w Autol Sub, Perc Endo */
"0U15479"  =  "1"  /* Bypass R Fallopian Tube to Uterus w Autol Sub, Perc Endo */
"0U154J5"  =  "1"  /* Bypass R Fallopian Tube to R Fallop w Synth Sub, Perc Endo */
"0U154J6"  =  "1"  /* Bypass R Fallopian Tube to L Fallop w Synth Sub, Perc Endo */
"0U154J9"  =  "1"  /* Bypass R Fallopian Tube to Uterus w Synth Sub, Perc Endo */
"0U154K5"  =  "1"  /* Bypass R Fallopian Tube to R Fallop w Nonaut Sub, Perc Endo */
"0U154K6"  =  "1"  /* Bypass R Fallopian Tube to L Fallop w Nonaut Sub, Perc Endo */
"0U154K9"  =  "1"  /* Bypass R Fallopian Tube to Uterus w Nonaut Sub, Perc Endo */
"0U154Z5"  =  "1"  /* Bypass Right Fallopian Tube to R Fallop, Perc Endo Approach */
"0U154Z6"  =  "1"  /* Bypass Right Fallopian Tube to L Fallop, Perc Endo Approach */
"0U154Z9"  =  "1"  /* Bypass Right Fallopian Tube to Uterus, Perc Endo Approach */
"0U16075"  =  "1"  /* Bypass L Fallopian Tube to R Fallop w Autol Sub, Open */
"0U16076"  =  "1"  /* Bypass L Fallopian Tube to L Fallop w Autol Sub, Open */
"0U16079"  =  "1"  /* Bypass L Fallopian Tube to Uterus w Autol Sub, Open */
"0U160J5"  =  "1"  /* Bypass L Fallopian Tube to R Fallop w Synth Sub, Open */
"0U160J6"  =  "1"  /* Bypass L Fallopian Tube to L Fallop w Synth Sub, Open */
"0U160J9"  =  "1"  /* Bypass L Fallopian Tube to Uterus w Synth Sub, Open */
"0U160K5"  =  "1"  /* Bypass L Fallopian Tube to R Fallop w Nonaut Sub, Open */
"0U160K6"  =  "1"  /* Bypass L Fallopian Tube to L Fallop w Nonaut Sub, Open */
"0U160K9"  =  "1"  /* Bypass L Fallopian Tube to Uterus w Nonaut Sub, Open */
"0U160Z5"  =  "1"  /* Bypass Left Fallopian Tube to R Fallop, Open Approach */
"0U160Z6"  =  "1"  /* Bypass Left Fallopian Tube to L Fallop, Open Approach */
"0U160Z9"  =  "1"  /* Bypass Left Fallopian Tube to Uterus, Open Approach */
"0U16475"  =  "1"  /* Bypass L Fallopian Tube to R Fallop w Autol Sub, Perc Endo */
"0U16476"  =  "1"  /* Bypass L Fallopian Tube to L Fallop w Autol Sub, Perc Endo */
"0U16479"  =  "1"  /* Bypass L Fallopian Tube to Uterus w Autol Sub, Perc Endo */
"0U164J5"  =  "1"  /* Bypass L Fallopian Tube to R Fallop w Synth Sub, Perc Endo */
"0U164J6"  =  "1"  /* Bypass L Fallopian Tube to L Fallop w Synth Sub, Perc Endo */
"0U164J9"  =  "1"  /* Bypass L Fallopian Tube to Uterus w Synth Sub, Perc Endo */
"0U164K5"  =  "1"  /* Bypass L Fallopian Tube to R Fallop w Nonaut Sub, Perc Endo */
"0U164K6"  =  "1"  /* Bypass L Fallopian Tube to L Fallop w Nonaut Sub, Perc Endo */
"0U164K9"  =  "1"  /* Bypass L Fallopian Tube to Uterus w Nonaut Sub, Perc Endo */
"0U164Z5"  =  "1"  /* Bypass Left Fallopian Tube to R Fallop, Perc Endo Approach */
"0U164Z6"  =  "1"  /* Bypass Left Fallopian Tube to L Fallop, Perc Endo Approach */
"0U164Z9"  =  "1"  /* Bypass Left Fallopian Tube to Uterus, Perc Endo Approach */
"0U500ZZ"  =  "1"  /* Destruction of Right Ovary, Open Approach */
"0U503ZZ"  =  "1"  /* Destruction of Right Ovary, Percutaneous Approach */
"0U504ZZ"  =  "1"  /* Destruction of Right Ovary, Percutaneous Endoscopic Approach */
"0U510ZZ"  =  "1"  /* Destruction of Left Ovary, Open Approach */
"0U513ZZ"  =  "1"  /* Destruction of Left Ovary, Percutaneous Approach */
"0U514ZZ"  =  "1"  /* Destruction of Left Ovary, Percutaneous Endoscopic Approach */
"0U520ZZ"  =  "1"  /* Destruction of Bilateral Ovaries, Open Approach */
"0U523ZZ"  =  "1"  /* Destruction of Bilateral Ovaries, Percutaneous Approach */
"0U524ZZ"  =  "1"  /* Destruction of Bilateral Ovaries, Perc Endo Approach */
"0U540ZZ"  =  "1"  /* Destruction of Uterine Supporting Structure, Open Approach */
"0U543ZZ"  =  "1"  /* Destruction of Uterine Supporting Structure, Perc Approach */
"0U544ZZ"  =  "1"  /* Destruction of Uterine Support Struct, Perc Endo Approach */
"0U550ZZ"  =  "1"  /* Destruction of Right Fallopian Tube, Open Approach */
"0U553ZZ"  =  "1"  /* Destruction of Right Fallopian Tube, Percutaneous Approach */
"0U554ZZ"  =  "1"  /* Destruction of Right Fallopian Tube, Perc Endo Approach */
"0U557ZZ"  =  "1"  /* Destruction of Right Fallopian Tube, Via Opening */
"0U558ZZ"  =  "1"  /* Destruction of Right Fallopian Tube, Endo */
"0U560ZZ"  =  "1"  /* Destruction of Left Fallopian Tube, Open Approach */
"0U563ZZ"  =  "1"  /* Destruction of Left Fallopian Tube, Percutaneous Approach */
"0U564ZZ"  =  "1"  /* Destruction of Left Fallopian Tube, Perc Endo Approach */
"0U567ZZ"  =  "1"  /* Destruction of Left Fallopian Tube, Via Opening */
"0U568ZZ"  =  "1"  /* Destruction of Left Fallopian Tube, Endo */
"0U570ZZ"  =  "1"  /* Destruction of Bilateral Fallopian Tubes, Open Approach */
"0U573ZZ"  =  "1"  /* Destruction of Bilateral Fallopian Tubes, Perc Approach */
"0U574ZZ"  =  "1"  /* Destruction of Bilateral Fallopian Tubes, Perc Endo Approach */
"0U577ZZ"  =  "1"  /* Destruction of Bilateral Fallopian Tubes, Via Opening */
"0U578ZZ"  =  "1"  /* Destruction of Bilateral Fallopian Tubes, Endo */
"0U590ZZ"  =  "1"  /* Destruction of Uterus, Open Approach */
"0U593ZZ"  =  "1"  /* Destruction of Uterus, Percutaneous Approach */
"0U594ZZ"  =  "1"  /* Destruction of Uterus, Percutaneous Endoscopic Approach */
"0U597ZZ"  =  "1"  /* Destruction of Uterus, Via Natural or Artificial Opening */
"0U598ZZ"  =  "1"  /* Destruction of Uterus, Endo */
"0U5B0ZZ"  =  "1"  /* Destruction of Endometrium, Open Approach */
"0U5B3ZZ"  =  "1"  /* Destruction of Endometrium, Percutaneous Approach */
"0U5B4ZZ"  =  "1"  /* Destruction of Endometrium, Percutaneous Endoscopic Approach */
"0U5B7ZZ"  =  "1"  /* Destruction of Endometrium, Via Opening */
"0U5B8ZZ"  =  "1"  /* Destruction of Endometrium, Endo */
"0U5C0ZZ"  =  "1"  /* Destruction of Cervix, Open Approach */
"0U5C3ZZ"  =  "1"  /* Destruction of Cervix, Percutaneous Approach */
"0U5C4ZZ"  =  "1"  /* Destruction of Cervix, Percutaneous Endoscopic Approach */
"0U5C7ZZ"  =  "1"  /* Destruction of Cervix, Via Natural or Artificial Opening */
"0U5C8ZZ"  =  "1"  /* Destruction of Cervix, Endo */
"0U5F0ZZ"  =  "1"  /* Destruction of Cul-de-sac, Open Approach */
"0U5F3ZZ"  =  "1"  /* Destruction of Cul-de-sac, Percutaneous Approach */
"0U5F4ZZ"  =  "1"  /* Destruction of Cul-de-sac, Percutaneous Endoscopic Approach */
"0U5F7ZZ"  =  "1"  /* Destruction of Cul-de-sac, Via Natural or Artificial Opening */
"0U5F8ZZ"  =  "1"  /* Destruction of Cul-de-sac, Endo */
"0U5G0ZZ"  =  "1"  /* Destruction of Vagina, Open Approach */
"0U5G3ZZ"  =  "1"  /* Destruction of Vagina, Percutaneous Approach */
"0U5G4ZZ"  =  "1"  /* Destruction of Vagina, Percutaneous Endoscopic Approach */
"0U5G7ZZ"  =  "1"  /* Destruction of Vagina, Via Natural or Artificial Opening */
"0U5G8ZZ"  =  "1"  /* Destruction of Vagina, Endo */
"0U5GXZZ"  =  "1"  /* Destruction of Vagina, External Approach */
"0U5J0ZZ"  =  "1"  /* Destruction of Clitoris, Open Approach */
"0U5JXZZ"  =  "1"  /* Destruction of Clitoris, External Approach */
"0U5K0ZZ"  =  "1"  /* Destruction of Hymen, Open Approach */
"0U5K3ZZ"  =  "1"  /* Destruction of Hymen, Percutaneous Approach */
"0U5K4ZZ"  =  "1"  /* Destruction of Hymen, Percutaneous Endoscopic Approach */
"0U5K7ZZ"  =  "1"  /* Destruction of Hymen, Via Natural or Artificial Opening */
"0U5K8ZZ"  =  "1"  /* Destruction of Hymen, Endo */
"0U5KXZZ"  =  "1"  /* Destruction of Hymen, External Approach */
"0U5L0ZZ"  =  "1"  /* Destruction of Vestibular Gland, Open Approach */
"0U5LXZZ"  =  "1"  /* Destruction of Vestibular Gland, External Approach */
"0U5M0ZZ"  =  "1"  /* Destruction of Vulva, Open Approach */
"0U5MXZZ"  =  "1"  /* Destruction of Vulva, External Approach */
"0U750DZ"  =  "1"  /* Dilate of R Fallopian Tube with Intralum Dev, Open Approach */
"0U750ZZ"  =  "1"  /* Dilation of Right Fallopian Tube, Open Approach */
"0U753DZ"  =  "1"  /* Dilate of R Fallopian Tube with Intralum Dev, Perc Approach */
"0U753ZZ"  =  "1"  /* Dilation of Right Fallopian Tube, Percutaneous Approach */
"0U754DZ"  =  "1"  /* Dilate R Fallopian Tube w Intralum Dev, Perc Endo */
"0U754ZZ"  =  "1"  /* Dilation of Right Fallopian Tube, Perc Endo Approach */
"0U757DZ"  =  "1"  /* Dilation of R Fallopian Tube with Intralum Dev, Via Opening */
"0U757ZZ"  =  "1"  /* Dilation of Right Fallopian Tube, Via Opening */
"0U758DZ"  =  "1"  /* Dilation of Right Fallopian Tube with Intralum Dev, Endo */
"0U758ZZ"  =  "1"  /* Dilation of Right Fallopian Tube, Endo */
"0U760DZ"  =  "1"  /* Dilate of L Fallopian Tube with Intralum Dev, Open Approach */
"0U760ZZ"  =  "1"  /* Dilation of Left Fallopian Tube, Open Approach */
"0U763DZ"  =  "1"  /* Dilate of L Fallopian Tube with Intralum Dev, Perc Approach */
"0U763ZZ"  =  "1"  /* Dilation of Left Fallopian Tube, Percutaneous Approach */
"0U764DZ"  =  "1"  /* Dilate L Fallopian Tube w Intralum Dev, Perc Endo */
"0U764ZZ"  =  "1"  /* Dilation of Left Fallopian Tube, Perc Endo Approach */
"0U767DZ"  =  "1"  /* Dilation of L Fallopian Tube with Intralum Dev, Via Opening */
"0U767ZZ"  =  "1"  /* Dilation of Left Fallopian Tube, Via Opening */
"0U768DZ"  =  "1"  /* Dilation of Left Fallopian Tube with Intralum Dev, Endo */
"0U768ZZ"  =  "1"  /* Dilation of Left Fallopian Tube, Endo */
"0U770DZ"  =  "1"  /* Dilate of Bi Fallopian Tube with Intralum Dev, Open Approach */
"0U770ZZ"  =  "1"  /* Dilation of Bilateral Fallopian Tubes, Open Approach */
"0U773DZ"  =  "1"  /* Dilate of Bi Fallopian Tube with Intralum Dev, Perc Approach */
"0U773ZZ"  =  "1"  /* Dilation of Bilateral Fallopian Tubes, Percutaneous Approach */
"0U774DZ"  =  "1"  /* Dilate Bi Fallopian Tube w Intralum Dev, Perc Endo */
"0U774ZZ"  =  "1"  /* Dilation of Bilateral Fallopian Tubes, Perc Endo Approach */
"0U777DZ"  =  "1"  /* Dilation of Bi Fallopian Tube with Intralum Dev, Via Opening */
"0U777ZZ"  =  "1"  /* Dilation of Bilateral Fallopian Tubes, Via Opening */
"0U778DZ"  =  "1"  /* Dilation of Bi Fallopian Tube with Intralum Dev, Endo */
"0U778ZZ"  =  "1"  /* Dilation of Bilateral Fallopian Tubes, Endo */
"0U790DZ"  =  "1"  /* Dilation of Uterus with Intraluminal Device, Open Approach */
"0U790ZZ"  =  "1"  /* Dilation of Uterus, Open Approach */
"0U793DZ"  =  "1"  /* Dilation of Uterus with Intraluminal Device, Perc Approach */
"0U793ZZ"  =  "1"  /* Dilation of Uterus, Percutaneous Approach */
"0U794DZ"  =  "1"  /* Dilation of Uterus with Intralum Dev, Perc Endo Approach */
"0U794ZZ"  =  "1"  /* Dilation of Uterus, Percutaneous Endoscopic Approach */
"0U797DZ"  =  "1"  /* Dilation of Uterus with Intraluminal Device, Via Opening */
"0U797ZZ"  =  "1"  /* Dilation of Uterus, Via Natural or Artificial Opening */
"0U798DZ"  =  "1"  /* Dilation of Uterus with Intraluminal Device, Endo */
"0U798ZZ"  =  "1"  /* Dilation of Uterus, Endo */
"0U7G0DZ"  =  "1"  /* Dilation of Vagina with Intraluminal Device, Open Approach */
"0U7G0ZZ"  =  "1"  /* Dilation of Vagina, Open Approach */
"0U7G3DZ"  =  "1"  /* Dilation of Vagina with Intraluminal Device, Perc Approach */
"0U7G3ZZ"  =  "1"  /* Dilation of Vagina, Percutaneous Approach */
"0U7G4DZ"  =  "1"  /* Dilation of Vagina with Intralum Dev, Perc Endo Approach */
"0U7G4ZZ"  =  "1"  /* Dilation of Vagina, Percutaneous Endoscopic Approach */
"0U7K0DZ"  =  "1"  /* Dilation of Hymen with Intraluminal Device, Open Approach */
"0U7K0ZZ"  =  "1"  /* Dilation of Hymen, Open Approach */
"0U7K3DZ"  =  "1"  /* Dilation of Hymen with Intraluminal Device, Perc Approach */
"0U7K3ZZ"  =  "1"  /* Dilation of Hymen, Percutaneous Approach */
"0U7K4DZ"  =  "1"  /* Dilation of Hymen with Intralum Dev, Perc Endo Approach */
"0U7K4ZZ"  =  "1"  /* Dilation of Hymen, Percutaneous Endoscopic Approach */
"0U7K7DZ"  =  "1"  /* Dilation of Hymen with Intraluminal Device, Via Opening */
"0U7K7ZZ"  =  "1"  /* Dilation of Hymen, Via Natural or Artificial Opening */
"0U7K8DZ"  =  "1"  /* Dilation of Hymen with Intraluminal Device, Endo */
"0U7K8ZZ"  =  "1"  /* Dilation of Hymen, Endo */
"0U7KXDZ"  =  "1"  /* Dilation of Hymen with Intraluminal Device, Extern Approach */
"0U7KXZZ"  =  "1"  /* Dilation of Hymen, External Approach */
"0U800ZZ"  =  "1"  /* Division of Right Ovary, Open Approach */
"0U803ZZ"  =  "1"  /* Division of Right Ovary, Percutaneous Approach */
"0U804ZZ"  =  "1"  /* Division of Right Ovary, Percutaneous Endoscopic Approach */
"0U810ZZ"  =  "1"  /* Division of Left Ovary, Open Approach */
"0U813ZZ"  =  "1"  /* Division of Left Ovary, Percutaneous Approach */
"0U814ZZ"  =  "1"  /* Division of Left Ovary, Percutaneous Endoscopic Approach */
"0U820ZZ"  =  "1"  /* Division of Bilateral Ovaries, Open Approach */
"0U823ZZ"  =  "1"  /* Division of Bilateral Ovaries, Percutaneous Approach */
"0U824ZZ"  =  "1"  /* Division of Bilateral Ovaries, Perc Endo Approach */
"0U840ZZ"  =  "1"  /* Division of Uterine Supporting Structure, Open Approach */
"0U843ZZ"  =  "1"  /* Division of Uterine Supporting Structure, Perc Approach */
"0U844ZZ"  =  "1"  /* Division of Uterine Supporting Structure, Perc Endo Approach */
"0U9000Z"  =  "1"  /* Drainage of Right Ovary with Drainage Device, Open Approach */
"0U900ZX"  =  "1"  /* Drainage of Right Ovary, Open Approach, Diagnostic */
"0U900ZZ"  =  "1"  /* Drainage of Right Ovary, Open Approach */
"0U9030Z"  =  "1"  /* Drainage of Right Ovary with Drainage Device, Perc Approach */
"0U903ZX"  =  "1"  /* Drainage of Right Ovary, Percutaneous Approach, Diagnostic */
"0U903ZZ"  =  "1"  /* Drainage of Right Ovary, Percutaneous Approach */
"0U9040Z"  =  "1"  /* Drainage of Right Ovary with Drain Dev, Perc Endo Approach */
"0U904ZX"  =  "1"  /* Drainage of Right Ovary, Perc Endo Approach, Diagn */
"0U904ZZ"  =  "1"  /* Drainage of Right Ovary, Percutaneous Endoscopic Approach */
"0U90XZZ"  =  "1"  /* Drainage of Right Ovary, External Approach */
"0U9100Z"  =  "1"  /* Drainage of Left Ovary with Drainage Device, Open Approach */
"0U910ZX"  =  "1"  /* Drainage of Left Ovary, Open Approach, Diagnostic */
"0U910ZZ"  =  "1"  /* Drainage of Left Ovary, Open Approach */
"0U9130Z"  =  "1"  /* Drainage of Left Ovary with Drainage Device, Perc Approach */
"0U913ZX"  =  "1"  /* Drainage of Left Ovary, Percutaneous Approach, Diagnostic */
"0U913ZZ"  =  "1"  /* Drainage of Left Ovary, Percutaneous Approach */
"0U9140Z"  =  "1"  /* Drainage of Left Ovary with Drain Dev, Perc Endo Approach */
"0U914ZX"  =  "1"  /* Drainage of Left Ovary, Perc Endo Approach, Diagn */
"0U914ZZ"  =  "1"  /* Drainage of Left Ovary, Percutaneous Endoscopic Approach */
"0U91XZZ"  =  "1"  /* Drainage of Left Ovary, External Approach */
"0U9200Z"  =  "1"  /* Drainage of Bilateral Ovaries with Drain Dev, Open Approach */
"0U920ZX"  =  "1"  /* Drainage of Bilateral Ovaries, Open Approach, Diagnostic */
"0U920ZZ"  =  "1"  /* Drainage of Bilateral Ovaries, Open Approach */
"0U9230Z"  =  "1"  /* Drainage of Bilateral Ovaries with Drain Dev, Perc Approach */
"0U923ZX"  =  "1"  /* Drainage of Bilateral Ovaries, Percutaneous Approach, Diagn */
"0U923ZZ"  =  "1"  /* Drainage of Bilateral Ovaries, Percutaneous Approach */
"0U9240Z"  =  "1"  /* Drainage of Bi Ovary with Drain Dev, Perc Endo Approach */
"0U924ZX"  =  "1"  /* Drainage of Bilateral Ovaries, Perc Endo Approach, Diagn */
"0U924ZZ"  =  "1"  /* Drainage of Bilateral Ovaries, Perc Endo Approach */
"0U92XZZ"  =  "1"  /* Drainage of Bilateral Ovaries, External Approach */
"0U9400Z"  =  "1"  /* Drain Uterine Support Struct w Drain Dev, Open */
"0U940ZX"  =  "1"  /* Drainage of Uterine Support Struct, Open Approach, Diagn */
"0U940ZZ"  =  "1"  /* Drainage of Uterine Supporting Structure, Open Approach */
"0U9430Z"  =  "1"  /* Drain Uterine Support Struct w Drain Dev, Perc */
"0U943ZX"  =  "1"  /* Drainage of Uterine Support Struct, Perc Approach, Diagn */
"0U943ZZ"  =  "1"  /* Drainage of Uterine Supporting Structure, Perc Approach */
"0U9440Z"  =  "1"  /* Drain Uterine Support Struct w Drain Dev, Perc Endo */
"0U944ZX"  =  "1"  /* Drain of Uterine Support Struct, Perc Endo Approach, Diagn */
"0U944ZZ"  =  "1"  /* Drainage of Uterine Supporting Structure, Perc Endo Approach */
"0U9500Z"  =  "1"  /* Drainage of R Fallopian Tube with Drain Dev, Open Approach */
"0U950ZX"  =  "1"  /* Drainage of Right Fallopian Tube, Open Approach, Diagnostic */
"0U950ZZ"  =  "1"  /* Drainage of Right Fallopian Tube, Open Approach */
"0U9530Z"  =  "1"  /* Drainage of R Fallopian Tube with Drain Dev, Perc Approach */
"0U953ZX"  =  "1"  /* Drainage of Right Fallopian Tube, Perc Approach, Diagn */
"0U9540Z"  =  "1"  /* Drain of R Fallopian Tube with Drain Dev, Perc Endo Approach */
"0U954ZX"  =  "1"  /* Drainage of Right Fallopian Tube, Perc Endo Approach, Diagn */
"0U9570Z"  =  "1"  /* Drainage of Right Fallopian Tube with Drain Dev, Via Opening */
"0U957ZX"  =  "1"  /* Drainage of Right Fallopian Tube, Via Opening, Diagn */
"0U9580Z"  =  "1"  /* Drainage of Right Fallopian Tube with Drainage Device, Endo */
"0U958ZX"  =  "1"  /* Drainage of Right Fallopian Tube, Endo, Diagn */
"0U9600Z"  =  "1"  /* Drainage of L Fallopian Tube with Drain Dev, Open Approach */
"0U960ZX"  =  "1"  /* Drainage of Left Fallopian Tube, Open Approach, Diagnostic */
"0U960ZZ"  =  "1"  /* Drainage of Left Fallopian Tube, Open Approach */
"0U9630Z"  =  "1"  /* Drainage of L Fallopian Tube with Drain Dev, Perc Approach */
"0U963ZX"  =  "1"  /* Drainage of Left Fallopian Tube, Perc Approach, Diagn */
"0U9640Z"  =  "1"  /* Drain of L Fallopian Tube with Drain Dev, Perc Endo Approach */
"0U964ZX"  =  "1"  /* Drainage of Left Fallopian Tube, Perc Endo Approach, Diagn */
"0U9670Z"  =  "1"  /* Drainage of Left Fallopian Tube with Drain Dev, Via Opening */
"0U967ZX"  =  "1"  /* Drainage of Left Fallopian Tube, Via Opening, Diagn */
"0U9680Z"  =  "1"  /* Drainage of Left Fallopian Tube with Drainage Device, Endo */
"0U968ZX"  =  "1"  /* Drainage of Left Fallopian Tube, Endo, Diagn */
"0U9700Z"  =  "1"  /* Drainage of Bi Fallopian Tube with Drain Dev, Open Approach */
"0U970ZX"  =  "1"  /* Drainage of Bilateral Fallopian Tubes, Open Approach, Diagn */
"0U970ZZ"  =  "1"  /* Drainage of Bilateral Fallopian Tubes, Open Approach */
"0U9730Z"  =  "1"  /* Drainage of Bi Fallopian Tube with Drain Dev, Perc Approach */
"0U973ZX"  =  "1"  /* Drainage of Bilateral Fallopian Tubes, Perc Approach, Diagn */
"0U9740Z"  =  "1"  /* Drain Bi Fallopian Tube w Drain Dev, Perc Endo */
"0U974ZX"  =  "1"  /* Drainage of Bi Fallopian Tube, Perc Endo Approach, Diagn */
"0U9770Z"  =  "1"  /* Drainage of Bi Fallopian Tube with Drain Dev, Via Opening */
"0U977ZX"  =  "1"  /* Drainage of Bilateral Fallopian Tubes, Via Opening, Diagn */
"0U9780Z"  =  "1"  /* Drainage of Bilateral Fallopian Tubes with Drain Dev, Endo */
"0U978ZX"  =  "1"  /* Drainage of Bilateral Fallopian Tubes, Endo, Diagn */
"0U9900Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Open Approach */
"0U990ZX"  =  "1"  /* Drainage of Uterus, Open Approach, Diagnostic */
"0U990ZZ"  =  "1"  /* Drainage of Uterus, Open Approach */
"0U9930Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Perc Approach */
"0U993ZX"  =  "1"  /* Drainage of Uterus, Percutaneous Approach, Diagnostic */
"0U993ZZ"  =  "1"  /* Drainage of Uterus, Percutaneous Approach */
"0U9940Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Perc Endo Approach */
"0U994ZX"  =  "1"  /* Drainage of Uterus, Percutaneous Endoscopic Approach, Diagn */
"0U994ZZ"  =  "1"  /* Drainage of Uterus, Percutaneous Endoscopic Approach */
"0U9970Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Via Opening */
"0U997ZX"  =  "1"  /* Drainage of Uterus, Via Natural or Artificial Opening, Diagn */
"0U997ZZ"  =  "1"  /* Drainage of Uterus, Via Natural or Artificial Opening */
"0U9980Z"  =  "1"  /* Drainage of Uterus with Drainage Device, Endo */
"0U998ZX"  =  "1"  /* Drainage of Uterus, Endo, Diagn */
"0U998ZZ"  =  "1"  /* Drainage of Uterus, Endo */
"0U9C00Z"  =  "1"  /* Drainage of Cervix with Drainage Device, Open Approach */
"0U9C0ZX"  =  "1"  /* Drainage of Cervix, Open Approach, Diagnostic */
"0U9C0ZZ"  =  "1"  /* Drainage of Cervix, Open Approach */
"0U9C30Z"  =  "1"  /* Drainage of Cervix with Drainage Device, Perc Approach */
"0U9C3ZX"  =  "1"  /* Drainage of Cervix, Percutaneous Approach, Diagnostic */
"0U9C3ZZ"  =  "1"  /* Drainage of Cervix, Percutaneous Approach */
"0U9C40Z"  =  "1"  /* Drainage of Cervix with Drainage Device, Perc Endo Approach */
"0U9C4ZX"  =  "1"  /* Drainage of Cervix, Percutaneous Endoscopic Approach, Diagn */
"0U9C4ZZ"  =  "1"  /* Drainage of Cervix, Percutaneous Endoscopic Approach */
"0U9C70Z"  =  "1"  /* Drainage of Cervix with Drainage Device, Via Opening */
"0U9C7ZX"  =  "1"  /* Drainage of Cervix, Via Natural or Artificial Opening, Diagn */
"0U9C7ZZ"  =  "1"  /* Drainage of Cervix, Via Natural or Artificial Opening */
"0U9C80Z"  =  "1"  /* Drainage of Cervix with Drainage Device, Endo */
"0U9C8ZX"  =  "1"  /* Drainage of Cervix, Endo, Diagn */
"0U9C8ZZ"  =  "1"  /* Drainage of Cervix, Endo */
"0U9F00Z"  =  "1"  /* Drainage of Cul-de-sac with Drainage Device, Open Approach */
"0U9F0ZX"  =  "1"  /* Drainage of Cul-de-sac, Open Approach, Diagnostic */
"0U9F0ZZ"  =  "1"  /* Drainage of Cul-de-sac, Open Approach */
"0U9F3ZX"  =  "1"  /* Drainage of Cul-de-sac, Percutaneous Approach, Diagnostic */
"0U9F4ZX"  =  "1"  /* Drainage of Cul-de-sac, Perc Endo Approach, Diagn */
"0U9F70Z"  =  "1"  /* Drainage of Cul-de-sac with Drainage Device, Via Opening */
"0U9F7ZX"  =  "1"  /* Drainage of Cul-de-sac, Via Opening, Diagn */
"0U9F7ZZ"  =  "1"  /* Drainage of Cul-de-sac, Via Natural or Artificial Opening */
"0U9F80Z"  =  "1"  /* Drainage of Cul-de-sac with Drainage Device, Endo */
"0U9F8ZX"  =  "1"  /* Drainage of Cul-de-sac, Endo, Diagn */
"0U9F8ZZ"  =  "1"  /* Drainage of Cul-de-sac, Endo */
"0U9G00Z"  =  "1"  /* Drainage of Vagina with Drainage Device, Open Approach */
"0U9G0ZX"  =  "1"  /* Drainage of Vagina, Open Approach, Diagnostic */
"0U9G0ZZ"  =  "1"  /* Drainage of Vagina, Open Approach */
"0U9G30Z"  =  "1"  /* Drainage of Vagina with Drainage Device, Perc Approach */
"0U9G3ZX"  =  "1"  /* Drainage of Vagina, Percutaneous Approach, Diagnostic */
"0U9G3ZZ"  =  "1"  /* Drainage of Vagina, Percutaneous Approach */
"0U9G40Z"  =  "1"  /* Drainage of Vagina with Drainage Device, Perc Endo Approach */
"0U9G4ZX"  =  "1"  /* Drainage of Vagina, Percutaneous Endoscopic Approach, Diagn */
"0U9G4ZZ"  =  "1"  /* Drainage of Vagina, Percutaneous Endoscopic Approach */
"0U9G70Z"  =  "1"  /* Drainage of Vagina with Drainage Device, Via Opening */
"0U9G7ZX"  =  "1"  /* Drainage of Vagina, Via Natural or Artificial Opening, Diagn */
"0U9G7ZZ"  =  "1"  /* Drainage of Vagina, Via Natural or Artificial Opening */
"0U9G80Z"  =  "1"  /* Drainage of Vagina with Drainage Device, Endo */
"0U9G8ZX"  =  "1"  /* Drainage of Vagina, Endo, Diagn */
"0U9G8ZZ"  =  "1"  /* Drainage of Vagina, Endo */
"0U9GX0Z"  =  "1"  /* Drainage of Vagina with Drainage Device, External Approach */
"0U9GXZX"  =  "1"  /* Drainage of Vagina, External Approach, Diagnostic */
"0U9GXZZ"  =  "1"  /* Drainage of Vagina, External Approach */
"0U9J00Z"  =  "1"  /* Drainage of Clitoris with Drainage Device, Open Approach */
"0U9J0ZX"  =  "1"  /* Drainage of Clitoris, Open Approach, Diagnostic */
"0U9J0ZZ"  =  "1"  /* Drainage of Clitoris, Open Approach */
"0U9JX0Z"  =  "1"  /* Drainage of Clitoris with Drainage Device, External Approach */
"0U9JXZX"  =  "1"  /* Drainage of Clitoris, External Approach, Diagnostic */
"0U9JXZZ"  =  "1"  /* Drainage of Clitoris, External Approach */
"0U9K0ZX"  =  "1"  /* Drainage of Hymen, Open Approach, Diagnostic */
"0U9K3ZX"  =  "1"  /* Drainage of Hymen, Percutaneous Approach, Diagnostic */
"0U9K4ZX"  =  "1"  /* Drainage of Hymen, Percutaneous Endoscopic Approach, Diagn */
"0U9K7ZX"  =  "1"  /* Drainage of Hymen, Via Natural or Artificial Opening, Diagn */
"0U9K8ZX"  =  "1"  /* Drainage of Hymen, Endo, Diagn */
"0U9KXZX"  =  "1"  /* Drainage of Hymen, External Approach, Diagnostic */
"0U9L0ZX"  =  "1"  /* Drainage of Vestibular Gland, Open Approach, Diagnostic */
"0U9LXZX"  =  "1"  /* Drainage of Vestibular Gland, External Approach, Diagnostic */
"0U9M00Z"  =  "1"  /* Drainage of Vulva with Drainage Device, Open Approach */
"0U9M0ZX"  =  "1"  /* Drainage of Vulva, Open Approach, Diagnostic */
"0U9M0ZZ"  =  "1"  /* Drainage of Vulva, Open Approach */
"0U9MX0Z"  =  "1"  /* Drainage of Vulva with Drainage Device, External Approach */
"0U9MXZX"  =  "1"  /* Drainage of Vulva, External Approach, Diagnostic */
"0U9MXZZ"  =  "1"  /* Drainage of Vulva, External Approach */
"0UB00ZX"  =  "1"  /* Excision of Right Ovary, Open Approach, Diagnostic */
"0UB00ZZ"  =  "1"  /* Excision of Right Ovary, Open Approach */
"0UB03ZX"  =  "1"  /* Excision of Right Ovary, Percutaneous Approach, Diagnostic */
"0UB03ZZ"  =  "1"  /* Excision of Right Ovary, Percutaneous Approach */
"0UB04ZX"  =  "1"  /* Excision of Right Ovary, Perc Endo Approach, Diagn */
"0UB04ZZ"  =  "1"  /* Excision of Right Ovary, Percutaneous Endoscopic Approach */
"0UB07ZX"  =  "1"  /* Excision of Right Ovary, Via Opening, Diagn */
"0UB07ZZ"  =  "1"  /* Excision of Right Ovary, Via Natural or Artificial Opening */
"0UB08ZX"  =  "1"  /* Excision of Right Ovary, Endo, Diagn */
"0UB08ZZ"  =  "1"  /* Excision of Right Ovary, Endo */
"0UB10ZX"  =  "1"  /* Excision of Left Ovary, Open Approach, Diagnostic */
"0UB10ZZ"  =  "1"  /* Excision of Left Ovary, Open Approach */
"0UB13ZX"  =  "1"  /* Excision of Left Ovary, Percutaneous Approach, Diagnostic */
"0UB13ZZ"  =  "1"  /* Excision of Left Ovary, Percutaneous Approach */
"0UB14ZX"  =  "1"  /* Excision of Left Ovary, Perc Endo Approach, Diagn */
"0UB14ZZ"  =  "1"  /* Excision of Left Ovary, Percutaneous Endoscopic Approach */
"0UB17ZX"  =  "1"  /* Excision of Left Ovary, Via Opening, Diagn */
"0UB17ZZ"  =  "1"  /* Excision of Left Ovary, Via Natural or Artificial Opening */
"0UB18ZX"  =  "1"  /* Excision of Left Ovary, Endo, Diagn */
"0UB18ZZ"  =  "1"  /* Excision of Left Ovary, Endo */
"0UB20ZX"  =  "1"  /* Excision of Bilateral Ovaries, Open Approach, Diagnostic */
"0UB20ZZ"  =  "1"  /* Excision of Bilateral Ovaries, Open Approach */
"0UB23ZX"  =  "1"  /* Excision of Bilateral Ovaries, Percutaneous Approach, Diagn */
"0UB23ZZ"  =  "1"  /* Excision of Bilateral Ovaries, Percutaneous Approach */
"0UB24ZX"  =  "1"  /* Excision of Bilateral Ovaries, Perc Endo Approach, Diagn */
"0UB24ZZ"  =  "1"  /* Excision of Bilateral Ovaries, Perc Endo Approach */
"0UB27ZX"  =  "1"  /* Excision of Bilateral Ovaries, Via Opening, Diagn */
"0UB27ZZ"  =  "1"  /* Excision of Bilateral Ovaries, Via Opening */
"0UB28ZX"  =  "1"  /* Excision of Bilateral Ovaries, Endo, Diagn */
"0UB28ZZ"  =  "1"  /* Excision of Bilateral Ovaries, Endo */
"0UB40ZX"  =  "1"  /* Excision of Uterine Support Struct, Open Approach, Diagn */
"0UB40ZZ"  =  "1"  /* Excision of Uterine Supporting Structure, Open Approach */
"0UB43ZX"  =  "1"  /* Excision of Uterine Support Struct, Perc Approach, Diagn */
"0UB43ZZ"  =  "1"  /* Excision of Uterine Supporting Structure, Perc Approach */
"0UB44ZX"  =  "1"  /* Excise of Uterine Support Struct, Perc Endo Approach, Diagn */
"0UB44ZZ"  =  "1"  /* Excision of Uterine Supporting Structure, Perc Endo Approach */
"0UB47ZX"  =  "1"  /* Excision of Uterine Supporting Structure, Via Opening, Diagn */
"0UB47ZZ"  =  "1"  /* Excision of Uterine Supporting Structure, Via Opening */
"0UB48ZX"  =  "1"  /* Excision of Uterine Supporting Structure, Endo, Diagn */
"0UB48ZZ"  =  "1"  /* Excision of Uterine Supporting Structure, Endo */
"0UB50ZX"  =  "1"  /* Excision of Right Fallopian Tube, Open Approach, Diagnostic */
"0UB50ZZ"  =  "1"  /* Excision of Right Fallopian Tube, Open Approach */
"0UB53ZX"  =  "1"  /* Excision of Right Fallopian Tube, Perc Approach, Diagn */
"0UB53ZZ"  =  "1"  /* Excision of Right Fallopian Tube, Percutaneous Approach */
"0UB54ZX"  =  "1"  /* Excision of Right Fallopian Tube, Perc Endo Approach, Diagn */
"0UB54ZZ"  =  "1"  /* Excision of Right Fallopian Tube, Perc Endo Approach */
"0UB57ZX"  =  "1"  /* Excision of Right Fallopian Tube, Via Opening, Diagn */
"0UB57ZZ"  =  "1"  /* Excision of Right Fallopian Tube, Via Opening */
"0UB58ZX"  =  "1"  /* Excision of Right Fallopian Tube, Endo, Diagn */
"0UB58ZZ"  =  "1"  /* Excision of Right Fallopian Tube, Endo */
"0UB60ZX"  =  "1"  /* Excision of Left Fallopian Tube, Open Approach, Diagnostic */
"0UB60ZZ"  =  "1"  /* Excision of Left Fallopian Tube, Open Approach */
"0UB63ZX"  =  "1"  /* Excision of Left Fallopian Tube, Perc Approach, Diagn */
"0UB63ZZ"  =  "1"  /* Excision of Left Fallopian Tube, Percutaneous Approach */
"0UB64ZX"  =  "1"  /* Excision of Left Fallopian Tube, Perc Endo Approach, Diagn */
"0UB64ZZ"  =  "1"  /* Excision of Left Fallopian Tube, Perc Endo Approach */
"0UB67ZX"  =  "1"  /* Excision of Left Fallopian Tube, Via Opening, Diagn */
"0UB67ZZ"  =  "1"  /* Excision of Left Fallopian Tube, Via Opening */
"0UB68ZX"  =  "1"  /* Excision of Left Fallopian Tube, Endo, Diagn */
"0UB68ZZ"  =  "1"  /* Excision of Left Fallopian Tube, Endo */
"0UB70ZX"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Open Approach, Diagn */
"0UB70ZZ"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Open Approach */
"0UB73ZX"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Perc Approach, Diagn */
"0UB73ZZ"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Percutaneous Approach */
"0UB74ZX"  =  "1"  /* Excision of Bi Fallopian Tube, Perc Endo Approach, Diagn */
"0UB74ZZ"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Perc Endo Approach */
"0UB77ZX"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Via Opening, Diagn */
"0UB77ZZ"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Via Opening */
"0UB78ZX"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Endo, Diagn */
"0UB78ZZ"  =  "1"  /* Excision of Bilateral Fallopian Tubes, Endo */
"0UB90ZX"  =  "1"  /* Excision of Uterus, Open Approach, Diagnostic */
"0UB90ZZ"  =  "1"  /* Excision of Uterus, Open Approach */
"0UB93ZX"  =  "1"  /* Excision of Uterus, Percutaneous Approach, Diagnostic */
"0UB93ZZ"  =  "1"  /* Excision of Uterus, Percutaneous Approach */
"0UB94ZX"  =  "1"  /* Excision of Uterus, Percutaneous Endoscopic Approach, Diagn */
"0UB94ZZ"  =  "1"  /* Excision of Uterus, Percutaneous Endoscopic Approach */
"0UB97ZX"  =  "1"  /* Excision of Uterus, Via Natural or Artificial Opening, Diagn */
"0UB97ZZ"  =  "1"  /* Excision of Uterus, Via Natural or Artificial Opening */
"0UB98ZX"  =  "1"  /* Excision of Uterus, Endo, Diagn */
"0UB98ZZ"  =  "1"  /* Excision of Uterus, Endo */
"0UBC0ZX"  =  "1"  /* Excision of Cervix, Open Approach, Diagnostic */
"0UBC0ZZ"  =  "1"  /* Excision of Cervix, Open Approach */
"0UBC3ZX"  =  "1"  /* Excision of Cervix, Percutaneous Approach, Diagnostic */
"0UBC3ZZ"  =  "1"  /* Excision of Cervix, Percutaneous Approach */
"0UBC4ZX"  =  "1"  /* Excision of Cervix, Percutaneous Endoscopic Approach, Diagn */
"0UBC4ZZ"  =  "1"  /* Excision of Cervix, Percutaneous Endoscopic Approach */
"0UBC7ZX"  =  "1"  /* Excision of Cervix, Via Natural or Artificial Opening, Diagn */
"0UBC7ZZ"  =  "1"  /* Excision of Cervix, Via Natural or Artificial Opening */
"0UBC8ZX"  =  "1"  /* Excision of Cervix, Endo, Diagn */
"0UBC8ZZ"  =  "1"  /* Excision of Cervix, Endo */
"0UBF0ZX"  =  "1"  /* Excision of Cul-de-sac, Open Approach, Diagnostic */
"0UBF0ZZ"  =  "1"  /* Excision of Cul-de-sac, Open Approach */
"0UBF3ZX"  =  "1"  /* Excision of Cul-de-sac, Percutaneous Approach, Diagnostic */
"0UBF3ZZ"  =  "1"  /* Excision of Cul-de-sac, Percutaneous Approach */
"0UBF4ZX"  =  "1"  /* Excision of Cul-de-sac, Perc Endo Approach, Diagn */
"0UBF4ZZ"  =  "1"  /* Excision of Cul-de-sac, Percutaneous Endoscopic Approach */
"0UBF7ZX"  =  "1"  /* Excision of Cul-de-sac, Via Opening, Diagn */
"0UBF7ZZ"  =  "1"  /* Excision of Cul-de-sac, Via Natural or Artificial Opening */
"0UBF8ZX"  =  "1"  /* Excision of Cul-de-sac, Endo, Diagn */
"0UBF8ZZ"  =  "1"  /* Excision of Cul-de-sac, Endo */
"0UBG0ZX"  =  "1"  /* Excision of Vagina, Open Approach, Diagnostic */
"0UBG0ZZ"  =  "1"  /* Excision of Vagina, Open Approach */
"0UBG3ZX"  =  "1"  /* Excision of Vagina, Percutaneous Approach, Diagnostic */
"0UBG3ZZ"  =  "1"  /* Excision of Vagina, Percutaneous Approach */
"0UBG4ZX"  =  "1"  /* Excision of Vagina, Percutaneous Endoscopic Approach, Diagn */
"0UBG4ZZ"  =  "1"  /* Excision of Vagina, Percutaneous Endoscopic Approach */
"0UBG7ZX"  =  "1"  /* Excision of Vagina, Via Natural or Artificial Opening, Diagn */
"0UBG7ZZ"  =  "1"  /* Excision of Vagina, Via Natural or Artificial Opening */
"0UBG8ZX"  =  "1"  /* Excision of Vagina, Endo, Diagn */
"0UBG8ZZ"  =  "1"  /* Excision of Vagina, Endo */
"0UBGXZX"  =  "1"  /* Excision of Vagina, External Approach, Diagnostic */
"0UBGXZZ"  =  "1"  /* Excision of Vagina, External Approach */
"0UBJ0ZX"  =  "1"  /* Excision of Clitoris, Open Approach, Diagnostic */
"0UBJ0ZZ"  =  "1"  /* Excision of Clitoris, Open Approach */
"0UBJXZX"  =  "1"  /* Excision of Clitoris, External Approach, Diagnostic */
"0UBJXZZ"  =  "1"  /* Excision of Clitoris, External Approach */
"0UBK0ZX"  =  "1"  /* Excision of Hymen, Open Approach, Diagnostic */
"0UBK0ZZ"  =  "1"  /* Excision of Hymen, Open Approach */
"0UBK3ZX"  =  "1"  /* Excision of Hymen, Percutaneous Approach, Diagnostic */
"0UBK3ZZ"  =  "1"  /* Excision of Hymen, Percutaneous Approach */
"0UBK4ZX"  =  "1"  /* Excision of Hymen, Percutaneous Endoscopic Approach, Diagn */
"0UBK4ZZ"  =  "1"  /* Excision of Hymen, Percutaneous Endoscopic Approach */
"0UBK7ZX"  =  "1"  /* Excision of Hymen, Via Natural or Artificial Opening, Diagn */
"0UBK7ZZ"  =  "1"  /* Excision of Hymen, Via Natural or Artificial Opening */
"0UBK8ZX"  =  "1"  /* Excision of Hymen, Endo, Diagn */
"0UBK8ZZ"  =  "1"  /* Excision of Hymen, Endo */
"0UBKXZX"  =  "1"  /* Excision of Hymen, External Approach, Diagnostic */
"0UBKXZZ"  =  "1"  /* Excision of Hymen, External Approach */
"0UBL0ZX"  =  "1"  /* Excision of Vestibular Gland, Open Approach, Diagnostic */
"0UBL0ZZ"  =  "1"  /* Excision of Vestibular Gland, Open Approach */
"0UBLXZX"  =  "1"  /* Excision of Vestibular Gland, External Approach, Diagnostic */
"0UBLXZZ"  =  "1"  /* Excision of Vestibular Gland, External Approach */
"0UBM0ZX"  =  "1"  /* Excision of Vulva, Open Approach, Diagnostic */
"0UBM0ZZ"  =  "1"  /* Excision of Vulva, Open Approach */
"0UBMXZX"  =  "1"  /* Excision of Vulva, External Approach, Diagnostic */
"0UBMXZZ"  =  "1"  /* Excision of Vulva, External Approach */
"0UC00ZZ"  =  "1"  /* Extirpation of Matter from Right Ovary, Open Approach */
"0UC03ZZ"  =  "1"  /* Extirpation of Matter from Right Ovary, Perc Approach */
"0UC04ZZ"  =  "1"  /* Extirpation of Matter from Right Ovary, Perc Endo Approach */
"0UC10ZZ"  =  "1"  /* Extirpation of Matter from Left Ovary, Open Approach */
"0UC13ZZ"  =  "1"  /* Extirpation of Matter from Left Ovary, Percutaneous Approach */
"0UC14ZZ"  =  "1"  /* Extirpation of Matter from Left Ovary, Perc Endo Approach */
"0UC20ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Ovaries, Open Approach */
"0UC23ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Ovaries, Perc Approach */
"0UC24ZZ"  =  "1"  /* Extirpation of Matter from Bi Ovary, Perc Endo Approach */
"0UC40ZZ"  =  "1"  /* Extirpate matter from Uterine Support Struct, Open */
"0UC43ZZ"  =  "1"  /* Extirpate matter from Uterine Support Struct, Perc */
"0UC44ZZ"  =  "1"  /* Extirpate matter from Uterine Support Struct, Perc Endo */
"0UC50ZZ"  =  "1"  /* Extirpation of Matter from R Fallopian Tube, Open Approach */
"0UC53ZZ"  =  "1"  /* Extirpation of Matter from R Fallopian Tube, Perc Approach */
"0UC54ZZ"  =  "1"  /* Extirpate matter from R Fallopian Tube, Perc Endo */
"0UC57ZZ"  =  "1"  /* Extirpation of Matter from Right Fallopian Tube, Via Opening */
"0UC58ZZ"  =  "1"  /* Extirpation of Matter from Right Fallopian Tube, Endo */
"0UC60ZZ"  =  "1"  /* Extirpation of Matter from L Fallopian Tube, Open Approach */
"0UC63ZZ"  =  "1"  /* Extirpation of Matter from L Fallopian Tube, Perc Approach */
"0UC64ZZ"  =  "1"  /* Extirpate matter from L Fallopian Tube, Perc Endo */
"0UC67ZZ"  =  "1"  /* Extirpation of Matter from Left Fallopian Tube, Via Opening */
"0UC68ZZ"  =  "1"  /* Extirpation of Matter from Left Fallopian Tube, Endo */
"0UC70ZZ"  =  "1"  /* Extirpation of Matter from Bi Fallopian Tube, Open Approach */
"0UC73ZZ"  =  "1"  /* Extirpation of Matter from Bi Fallopian Tube, Perc Approach */
"0UC74ZZ"  =  "1"  /* Extirpate matter from Bi Fallopian Tube, Perc Endo */
"0UC77ZZ"  =  "1"  /* Extirpation of Matter from Bi Fallopian Tube, Via Opening */
"0UC78ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Fallopian Tubes, Endo */
"0UC90ZZ"  =  "1"  /* Extirpation of Matter from Uterus, Open Approach */
"0UC93ZZ"  =  "1"  /* Extirpation of Matter from Uterus, Percutaneous Approach */
"0UC94ZZ"  =  "1"  /* Extirpation of Matter from Uterus, Perc Endo Approach */
"0UCB0ZZ"  =  "1"  /* Extirpation of Matter from Endometrium, Open Approach */
"0UCB3ZZ"  =  "1"  /* Extirpation of Matter from Endometrium, Perc Approach */
"0UCB4ZZ"  =  "1"  /* Extirpation of Matter from Endometrium, Perc Endo Approach */
"0UCB7ZZ"  =  "1"  /* Extirpation of Matter from Endometrium, Via Opening */
"0UCB8ZZ"  =  "1"  /* Extirpation of Matter from Endometrium, Endo */
"0UCC0ZZ"  =  "1"  /* Extirpation of Matter from Cervix, Open Approach */
"0UCC3ZZ"  =  "1"  /* Extirpation of Matter from Cervix, Percutaneous Approach */
"0UCC4ZZ"  =  "1"  /* Extirpation of Matter from Cervix, Perc Endo Approach */
"0UCC7ZZ"  =  "1"  /* Extirpation of Matter from Cervix, Via Opening */
"0UCC8ZZ"  =  "1"  /* Extirpation of Matter from Cervix, Endo */
"0UCF0ZZ"  =  "1"  /* Extirpation of Matter from Cul-de-sac, Open Approach */
"0UCF3ZZ"  =  "1"  /* Extirpation of Matter from Cul-de-sac, Percutaneous Approach */
"0UCF4ZZ"  =  "1"  /* Extirpation of Matter from Cul-de-sac, Perc Endo Approach */
"0UCF7ZZ"  =  "1"  /* Extirpation of Matter from Cul-de-sac, Via Opening */
"0UCF8ZZ"  =  "1"  /* Extirpation of Matter from Cul-de-sac, Endo */
"0UCG0ZZ"  =  "1"  /* Extirpation of Matter from Vagina, Open Approach */
"0UCG3ZZ"  =  "1"  /* Extirpation of Matter from Vagina, Percutaneous Approach */
"0UCG4ZZ"  =  "1"  /* Extirpation of Matter from Vagina, Perc Endo Approach */
"0UCJ0ZZ"  =  "1"  /* Extirpation of Matter from Clitoris, Open Approach */
"0UCJXZZ"  =  "1"  /* Extirpation of Matter from Clitoris, External Approach */
"0UCL0ZZ"  =  "1"  /* Extirpation of Matter from Vestibular Gland, Open Approach */
"0UCLXZZ"  =  "1"  /* Extirpation of Matter from Vestibular Gland, Extern Approach */
"0UCM0ZZ"  =  "1"  /* Extirpation of Matter from Vulva, Open Approach */
"0UDB7ZX"  =  "1"  /* Extraction of Endometrium, Via Opening, Diagn */
"0UDB7ZZ"  =  "1"  /* Extraction of Endometrium, Via Natural or Artificial Opening */
"0UDB8ZX"  =  "1"  /* Extraction of Endometrium, Endo, Diagn */
"0UDB8ZZ"  =  "1"  /* Extraction of Endometrium, Endo */
"0UDN0ZZ"  =  "1"  /* Extraction of Ova, Open */
"0UDN3ZZ"  =  "1"  /* Extraction of Ova, Percutaneous */
"0UDN4ZZ"  =  "1"  /* Extraction of Ova, Percutaneous Endoscopic */
"0UF50ZZ"  =  "1"  /* Fragmentation in Right Fallopian Tube, Open Approach */
"0UF53ZZ"  =  "1"  /* Fragmentation in Right Fallopian Tube, Percutaneous Approach */
"0UF54ZZ"  =  "1"  /* Fragmentation in Right Fallopian Tube, Perc Endo Approach */
"0UF57ZZ"  =  "1"  /* Fragmentation in Right Fallopian Tube, Via Opening */
"0UF58ZZ"  =  "1"  /* Fragmentation in Right Fallopian Tube, Endo */
"0UF60ZZ"  =  "1"  /* Fragmentation in Left Fallopian Tube, Open Approach */
"0UF63ZZ"  =  "1"  /* Fragmentation in Left Fallopian Tube, Percutaneous Approach */
"0UF64ZZ"  =  "1"  /* Fragmentation in Left Fallopian Tube, Perc Endo Approach */
"0UF67ZZ"  =  "1"  /* Fragmentation in Left Fallopian Tube, Via Opening */
"0UF68ZZ"  =  "1"  /* Fragmentation in Left Fallopian Tube, Endo */
"0UF70ZZ"  =  "1"  /* Fragmentation in Bilateral Fallopian Tubes, Open Approach */
"0UF73ZZ"  =  "1"  /* Fragmentation in Bilateral Fallopian Tubes, Perc Approach */
"0UF74ZZ"  =  "1"  /* Fragmentation in Bi Fallopian Tube, Perc Endo Approach */
"0UF77ZZ"  =  "1"  /* Fragmentation in Bilateral Fallopian Tubes, Via Opening */
"0UF78ZZ"  =  "1"  /* Fragmentation in Bilateral Fallopian Tubes, Endo */
"0UF90ZZ"  =  "1"  /* Fragmentation in Uterus, Open Approach */
"0UF93ZZ"  =  "1"  /* Fragmentation in Uterus, Percutaneous Approach */
"0UF94ZZ"  =  "1"  /* Fragmentation in Uterus, Percutaneous Endoscopic Approach */
"0UF97ZZ"  =  "1"  /* Fragmentation in Uterus, Via Natural or Artificial Opening */
"0UF98ZZ"  =  "1"  /* Fragmentation in Uterus, Endo */
"0UHC01Z"  =  "1"  /* Insertion of Radioactive Element into Cervix, Open Approach */
"0UHC31Z"  =  "1"  /* Insertion of Radioactive Element into Cervix, Perc Approach */
"0UHC41Z"  =  "1"  /* Insertion of Radioact Elem into Cervix, Perc Endo Approach */
"0UHC71Z"  =  "1"  /* Insertion of Radioactive Element into Cervix, Via Opening */
"0UHC81Z"  =  "1"  /* Insertion of Radioactive Element into Cervix, Endo */
"0UHG01Z"  =  "1"  /* Insertion of Radioactive Element into Vagina, Open Approach */
"0UHG31Z"  =  "1"  /* Insertion of Radioactive Element into Vagina, Perc Approach */
"0UHG41Z"  =  "1"  /* Insertion of Radioact Elem into Vagina, Perc Endo Approach */
"0UHG71Z"  =  "1"  /* Insertion of Radioactive Element into Vagina, Via Opening */
"0UHG81Z"  =  "1"  /* Insertion of Radioactive Element into Vagina, Endo */
"0UHGX1Z"  =  "1"  /* Insertion of Radioact Elem into Vagina, Extern Approach */
"0UHH03Z"  =  "1"  /* Insert Infusion Dev in Vagina & Cul-de-sac, Open */
"0UHH33Z"  =  "1"  /* Insert Infusion Dev in Vagina & Cul-de-sac, Perc */
"0UHH43Z"  =  "1"  /* Insert Infusion Dev in Vagina & Cul-de-sac, Perc Endo */
"0UHH73Z"  =  "1"  /* Insert of Infusion Dev into Vagina & Cul-de-sac, Via Opening */
"0UHH83Z"  =  "1"  /* Insertion of Infusion Device into Vagina & Cul-de-sac, Endo */
"0UJ30ZZ"  =  "1"  /* Inspection of Ovary, Open Approach */
"0UJ33ZZ"  =  "1"  /* Inspection of Ovary, Percutaneous Approach */
"0UJ34ZZ"  =  "1"  /* Inspection of Ovary, Percutaneous Endoscopic Approach */
"0UJ80ZZ"  =  "1"  /* Inspection of Fallopian Tube, Open Approach */
"0UJ83ZZ"  =  "1"  /* Inspection of Fallopian Tube, Percutaneous Approach */
"0UJ84ZZ"  =  "1"  /* Inspection of Fallopian Tube, Perc Endo Approach */
"0UJ87ZZ"  =  "1"  /* Inspection of Fallopian Tube, Via Opening */
"0UJ88ZZ"  =  "1"  /* Inspection of Fallopian Tube, Endo */
"0UJD0ZZ"  =  "1"  /* Inspection of Uterus and Cervix, Open Approach */
"0UJD3ZZ"  =  "1"  /* Inspection of Uterus and Cervix, Percutaneous Approach */
"0UJD4ZZ"  =  "1"  /* Inspection of Uterus and Cervix, Perc Endo Approach */
"0UJH0ZZ"  =  "1"  /* Inspection of Vagina and Cul-de-sac, Open Approach */
"0UJH3ZZ"  =  "1"  /* Inspection of Vagina and Cul-de-sac, Percutaneous Approach */
"0UJH4ZZ"  =  "1"  /* Inspection of Vagina and Cul-de-sac, Perc Endo Approach */
"0UJH7ZZ"  =  "1"  /* Inspection of Vagina and Cul-de-sac, Via Opening */
"0UJM0ZZ"  =  "1"  /* Inspection of Vulva, Open Approach */
"0UL50CZ"  =  "1"  /* Occlusion R Fallopian Tube w Extralum Dev, Open */
"0UL50DZ"  =  "1"  /* Occlusion R Fallopian Tube w Intralum Dev, Open */
"0UL50ZZ"  =  "1"  /* Occlusion of Right Fallopian Tube, Open Approach */
"0UL53CZ"  =  "1"  /* Occlusion R Fallopian Tube w Extralum Dev, Perc */
"0UL53DZ"  =  "1"  /* Occlusion R Fallopian Tube w Intralum Dev, Perc */
"0UL53ZZ"  =  "1"  /* Occlusion of Right Fallopian Tube, Percutaneous Approach */
"0UL54CZ"  =  "1"  /* Occlusion R Fallopian Tube w Extralum Dev, Perc Endo */
"0UL54DZ"  =  "1"  /* Occlusion R Fallopian Tube w Intralum Dev, Perc Endo */
"0UL54ZZ"  =  "1"  /* Occlusion of Right Fallopian Tube, Perc Endo Approach */
"0UL57DZ"  =  "1"  /* Occlusion of R Fallopian Tube with Intralum Dev, Via Opening */
"0UL57ZZ"  =  "1"  /* Occlusion of Right Fallopian Tube, Via Opening */
"0UL58DZ"  =  "1"  /* Occlusion of Right Fallopian Tube with Intralum Dev, Endo */
"0UL58ZZ"  =  "1"  /* Occlusion of Right Fallopian Tube, Endo */
"0UL60CZ"  =  "1"  /* Occlusion L Fallopian Tube w Extralum Dev, Open */
"0UL60DZ"  =  "1"  /* Occlusion L Fallopian Tube w Intralum Dev, Open */
"0UL60ZZ"  =  "1"  /* Occlusion of Left Fallopian Tube, Open Approach */
"0UL63CZ"  =  "1"  /* Occlusion L Fallopian Tube w Extralum Dev, Perc */
"0UL63DZ"  =  "1"  /* Occlusion L Fallopian Tube w Intralum Dev, Perc */
"0UL63ZZ"  =  "1"  /* Occlusion of Left Fallopian Tube, Percutaneous Approach */
"0UL64CZ"  =  "1"  /* Occlusion L Fallopian Tube w Extralum Dev, Perc Endo */
"0UL64DZ"  =  "1"  /* Occlusion L Fallopian Tube w Intralum Dev, Perc Endo */
"0UL64ZZ"  =  "1"  /* Occlusion of Left Fallopian Tube, Perc Endo Approach */
"0UL67DZ"  =  "1"  /* Occlusion of L Fallopian Tube with Intralum Dev, Via Opening */
"0UL67ZZ"  =  "1"  /* Occlusion of Left Fallopian Tube, Via Opening */
"0UL68DZ"  =  "1"  /* Occlusion of Left Fallopian Tube with Intralum Dev, Endo */
"0UL68ZZ"  =  "1"  /* Occlusion of Left Fallopian Tube, Endo */
"0UL70CZ"  =  "1"  /* Occlusion Bi Fallopian Tube w Extralum Dev, Open */
"0UL70DZ"  =  "1"  /* Occlusion Bi Fallopian Tube w Intralum Dev, Open */
"0UL70ZZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes, Open Approach */
"0UL73CZ"  =  "1"  /* Occlusion Bi Fallopian Tube w Extralum Dev, Perc */
"0UL73DZ"  =  "1"  /* Occlusion Bi Fallopian Tube w Intralum Dev, Perc */
"0UL73ZZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes, Perc Approach */
"0UL74CZ"  =  "1"  /* Occlusion Bi Fallopian Tube w Extralum Dev, Perc Endo */
"0UL74DZ"  =  "1"  /* Occlusion Bi Fallopian Tube w Intralum Dev, Perc Endo */
"0UL74ZZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes, Perc Endo Approach */
"0UL77DZ"  =  "1"  /* Occlusion Bi Fallopian Tube w Intralum Dev, Via Opening */
"0UL77ZZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes, Via Opening */
"0UL78DZ"  =  "1"  /* Occlusion of Bi Fallopian Tube with Intralum Dev, Endo */
"0UL78ZZ"  =  "1"  /* Occlusion of Bilateral Fallopian Tubes, Endo */
"0ULF7DZ"  =  "1"  /* Occlusion of Cul-de-sac with Intralum Dev, Via Opening */
"0ULF7ZZ"  =  "1"  /* Occlusion of Cul-de-sac, Via Natural or Artificial Opening */
"0ULF8DZ"  =  "1"  /* Occlusion of Cul-de-sac with Intraluminal Device, Endo */
"0ULF8ZZ"  =  "1"  /* Occlusion of Cul-de-sac, Endo */
"0ULG7DZ"  =  "1"  /* Occlusion of Vagina with Intraluminal Device, Via Opening */
"0ULG7ZZ"  =  "1"  /* Occlusion of Vagina, Via Natural or Artificial Opening */
"0ULG8DZ"  =  "1"  /* Occlusion of Vagina with Intraluminal Device, Endo */
"0ULG8ZZ"  =  "1"  /* Occlusion of Vagina, Endo */
"0UM00ZZ"  =  "1"  /* Reattachment of Right Ovary, Open Approach */
"0UM04ZZ"  =  "1"  /* Reattachment of Right Ovary, Perc Endo Approach */
"0UM10ZZ"  =  "1"  /* Reattachment of Left Ovary, Open Approach */
"0UM14ZZ"  =  "1"  /* Reattachment of Left Ovary, Percutaneous Endoscopic Approach */
"0UM20ZZ"  =  "1"  /* Reattachment of Bilateral Ovaries, Open Approach */
"0UM24ZZ"  =  "1"  /* Reattachment of Bilateral Ovaries, Perc Endo Approach */
"0UM40ZZ"  =  "1"  /* Reattachment of Uterine Supporting Structure, Open Approach */
"0UM44ZZ"  =  "1"  /* Reattachment of Uterine Support Struct, Perc Endo Approach */
"0UM50ZZ"  =  "1"  /* Reattachment of Right Fallopian Tube, Open Approach */
"0UM54ZZ"  =  "1"  /* Reattachment of Right Fallopian Tube, Perc Endo Approach */
"0UM60ZZ"  =  "1"  /* Reattachment of Left Fallopian Tube, Open Approach */
"0UM64ZZ"  =  "1"  /* Reattachment of Left Fallopian Tube, Perc Endo Approach */
"0UM70ZZ"  =  "1"  /* Reattachment of Bilateral Fallopian Tubes, Open Approach */
"0UM74ZZ"  =  "1"  /* Reattachment of Bi Fallopian Tube, Perc Endo Approach */
"0UM90ZZ"  =  "1"  /* Reattachment of Uterus, Open Approach */
"0UM94ZZ"  =  "1"  /* Reattachment of Uterus, Percutaneous Endoscopic Approach */
"0UMC0ZZ"  =  "1"  /* Reattachment of Cervix, Open Approach */
"0UMC4ZZ"  =  "1"  /* Reattachment of Cervix, Percutaneous Endoscopic Approach */
"0UMF0ZZ"  =  "1"  /* Reattachment of Cul-de-sac, Open Approach */
"0UMF4ZZ"  =  "1"  /* Reattachment of Cul-de-sac, Percutaneous Endoscopic Approach */
"0UMG0ZZ"  =  "1"  /* Reattachment of Vagina, Open Approach */
"0UMG4ZZ"  =  "1"  /* Reattachment of Vagina, Percutaneous Endoscopic Approach */
"0UMJXZZ"  =  "1"  /* Reattachment of Clitoris, External Approach */
"0UMK0ZZ"  =  "1"  /* Reattachment of Hymen, Open Approach */
"0UMK4ZZ"  =  "1"  /* Reattachment of Hymen, Percutaneous Endoscopic Approach */
"0UMKXZZ"  =  "1"  /* Reattachment of Hymen, External Approach */
"0UMMXZZ"  =  "1"  /* Reattachment of Vulva, External Approach */
"0UN00ZZ"  =  "1"  /* Release Right Ovary, Open Approach */
"0UN03ZZ"  =  "1"  /* Release Right Ovary, Percutaneous Approach */
"0UN04ZZ"  =  "1"  /* Release Right Ovary, Percutaneous Endoscopic Approach */
"0UN10ZZ"  =  "1"  /* Release Left Ovary, Open Approach */
"0UN13ZZ"  =  "1"  /* Release Left Ovary, Percutaneous Approach */
"0UN14ZZ"  =  "1"  /* Release Left Ovary, Percutaneous Endoscopic Approach */
"0UN20ZZ"  =  "1"  /* Release Bilateral Ovaries, Open Approach */
"0UN23ZZ"  =  "1"  /* Release Bilateral Ovaries, Percutaneous Approach */
"0UN24ZZ"  =  "1"  /* Release Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0UN40ZZ"  =  "1"  /* Release Uterine Supporting Structure, Open Approach */
"0UN43ZZ"  =  "1"  /* Release Uterine Supporting Structure, Percutaneous Approach */
"0UN44ZZ"  =  "1"  /* Release Uterine Supporting Structure, Perc Endo Approach */
"0UN50ZZ"  =  "1"  /* Release Right Fallopian Tube, Open Approach */
"0UN53ZZ"  =  "1"  /* Release Right Fallopian Tube, Percutaneous Approach */
"0UN54ZZ"  =  "1"  /* Release Right Fallopian Tube, Perc Endo Approach */
"0UN57ZZ"  =  "1"  /* Release Right Fallopian Tube, Via Opening */
"0UN58ZZ"  =  "1"  /* Release Right Fallopian Tube, Endo */
"0UN60ZZ"  =  "1"  /* Release Left Fallopian Tube, Open Approach */
"0UN63ZZ"  =  "1"  /* Release Left Fallopian Tube, Percutaneous Approach */
"0UN64ZZ"  =  "1"  /* Release Left Fallopian Tube, Perc Endo Approach */
"0UN67ZZ"  =  "1"  /* Release Left Fallopian Tube, Via Opening */
"0UN68ZZ"  =  "1"  /* Release Left Fallopian Tube, Endo */
"0UN70ZZ"  =  "1"  /* Release Bilateral Fallopian Tubes, Open Approach */
"0UN73ZZ"  =  "1"  /* Release Bilateral Fallopian Tubes, Percutaneous Approach */
"0UN74ZZ"  =  "1"  /* Release Bilateral Fallopian Tubes, Perc Endo Approach */
"0UN77ZZ"  =  "1"  /* Release Bilateral Fallopian Tubes, Via Opening */
"0UN78ZZ"  =  "1"  /* Release Bilateral Fallopian Tubes, Endo */
"0UN90ZZ"  =  "1"  /* Release Uterus, Open Approach */
"0UN93ZZ"  =  "1"  /* Release Uterus, Percutaneous Approach */
"0UN94ZZ"  =  "1"  /* Release Uterus, Percutaneous Endoscopic Approach */
"0UN97ZZ"  =  "1"  /* Release Uterus, Via Natural or Artificial Opening */
"0UN98ZZ"  =  "1"  /* Release Uterus, Via Natural or Artificial Opening Endoscopic */
"0UNC0ZZ"  =  "1"  /* Release Cervix, Open Approach */
"0UNC3ZZ"  =  "1"  /* Release Cervix, Percutaneous Approach */
"0UNC4ZZ"  =  "1"  /* Release Cervix, Percutaneous Endoscopic Approach */
"0UNC7ZZ"  =  "1"  /* Release Cervix, Via Natural or Artificial Opening */
"0UNC8ZZ"  =  "1"  /* Release Cervix, Via Natural or Artificial Opening Endoscopic */
"0UNF0ZZ"  =  "1"  /* Release Cul-de-sac, Open Approach */
"0UNF3ZZ"  =  "1"  /* Release Cul-de-sac, Percutaneous Approach */
"0UNF4ZZ"  =  "1"  /* Release Cul-de-sac, Percutaneous Endoscopic Approach */
"0UNF7ZZ"  =  "1"  /* Release Cul-de-sac, Via Natural or Artificial Opening */
"0UNF8ZZ"  =  "1"  /* Release Cul-de-sac, Endo */
"0UNG0ZZ"  =  "1"  /* Release Vagina, Open Approach */
"0UNG3ZZ"  =  "1"  /* Release Vagina, Percutaneous Approach */
"0UNG4ZZ"  =  "1"  /* Release Vagina, Percutaneous Endoscopic Approach */
"0UNG7ZZ"  =  "1"  /* Release Vagina, Via Natural or Artificial Opening */
"0UNG8ZZ"  =  "1"  /* Release Vagina, Via Natural or Artificial Opening Endoscopic */
"0UNGXZZ"  =  "1"  /* Release Vagina, External Approach */
"0UNJ0ZZ"  =  "1"  /* Release Clitoris, Open Approach */
"0UNJXZZ"  =  "1"  /* Release Clitoris, External Approach */
"0UNK0ZZ"  =  "1"  /* Release Hymen, Open Approach */
"0UNK3ZZ"  =  "1"  /* Release Hymen, Percutaneous Approach */
"0UNK4ZZ"  =  "1"  /* Release Hymen, Percutaneous Endoscopic Approach */
"0UNK7ZZ"  =  "1"  /* Release Hymen, Via Natural or Artificial Opening */
"0UNK8ZZ"  =  "1"  /* Release Hymen, Via Natural or Artificial Opening Endoscopic */
"0UNKXZZ"  =  "1"  /* Release Hymen, External Approach */
"0UNL0ZZ"  =  "1"  /* Release Vestibular Gland, Open Approach */
"0UNLXZZ"  =  "1"  /* Release Vestibular Gland, External Approach */
"0UNM0ZZ"  =  "1"  /* Release Vulva, Open Approach */
"0UNMXZZ"  =  "1"  /* Release Vulva, External Approach */
"0UP300Z"  =  "1"  /* Removal of Drainage Device from Ovary, Open Approach */
"0UP303Z"  =  "1"  /* Removal of Infusion Device from Ovary, Open Approach */
"0UP330Z"  =  "1"  /* Removal of Drainage Device from Ovary, Percutaneous Approach */
"0UP333Z"  =  "1"  /* Removal of Infusion Device from Ovary, Percutaneous Approach */
"0UP340Z"  =  "1"  /* Removal of Drainage Device from Ovary, Perc Endo Approach */
"0UP343Z"  =  "1"  /* Removal of Infusion Device from Ovary, Perc Endo Approach */
"0UP800Z"  =  "1"  /* Removal of Drain Dev from Fallopian Tube, Open Approach */
"0UP803Z"  =  "1"  /* Removal of Infusion Dev from Fallopian Tube, Open Approach */
"0UP807Z"  =  "1"  /* Removal of Autol Sub from Fallopian Tube, Open Approach */
"0UP80CZ"  =  "1"  /* Removal of Extralum Dev from Fallopian Tube, Open Approach */
"0UP80DZ"  =  "1"  /* Removal of Intralum Dev from Fallopian Tube, Open Approach */
"0UP80JZ"  =  "1"  /* Removal of Synth Sub from Fallopian Tube, Open Approach */
"0UP80KZ"  =  "1"  /* Removal of Nonaut Sub from Fallopian Tube, Open Approach */
"0UP830Z"  =  "1"  /* Removal of Drain Dev from Fallopian Tube, Perc Approach */
"0UP833Z"  =  "1"  /* Removal of Infusion Dev from Fallopian Tube, Perc Approach */
"0UP837Z"  =  "1"  /* Removal of Autol Sub from Fallopian Tube, Perc Approach */
"0UP83CZ"  =  "1"  /* Removal of Extralum Dev from Fallopian Tube, Perc Approach */
"0UP83DZ"  =  "1"  /* Removal of Intralum Dev from Fallopian Tube, Perc Approach */
"0UP83JZ"  =  "1"  /* Removal of Synth Sub from Fallopian Tube, Perc Approach */
"0UP83KZ"  =  "1"  /* Removal of Nonaut Sub from Fallopian Tube, Perc Approach */
"0UP840Z"  =  "1"  /* Removal of Drain Dev from Fallopian Tube, Perc Endo Approach */
"0UP843Z"  =  "1"  /* Remove Infusion Dev from Fallopian Tube, Perc Endo */
"0UP847Z"  =  "1"  /* Removal of Autol Sub from Fallopian Tube, Perc Endo Approach */
"0UP84CZ"  =  "1"  /* Remove Extralum Dev from Fallopian Tube, Perc Endo */
"0UP84DZ"  =  "1"  /* Remove Intralum Dev from Fallopian Tube, Perc Endo */
"0UP84JZ"  =  "1"  /* Removal of Synth Sub from Fallopian Tube, Perc Endo Approach */
"0UP84KZ"  =  "1"  /* Remove of Nonaut Sub from Fallopian Tube, Perc Endo Approach */
"0UP870Z"  =  "1"  /* Removal of Drainage Device from Fallopian Tube, Via Opening */
"0UP873Z"  =  "1"  /* Removal of Infusion Device from Fallopian Tube, Via Opening */
"0UP877Z"  =  "1"  /* Removal of Autol Sub from Fallopian Tube, Via Opening */
"0UP87CZ"  =  "1"  /* Removal of Extralum Dev from Fallopian Tube, Via Opening */
"0UP87DZ"  =  "1"  /* Removal of Intralum Dev from Fallopian Tube, Via Opening */
"0UP87JZ"  =  "1"  /* Removal of Synth Sub from Fallopian Tube, Via Opening */
"0UP87KZ"  =  "1"  /* Removal of Nonaut Sub from Fallopian Tube, Via Opening */
"0UP880Z"  =  "1"  /* Removal of Drainage Device from Fallopian Tube, Endo */
"0UP883Z"  =  "1"  /* Removal of Infusion Device from Fallopian Tube, Endo */
"0UP887Z"  =  "1"  /* Removal of Autol Sub from Fallopian Tube, Endo */
"0UP88CZ"  =  "1"  /* Removal of Extraluminal Device from Fallopian Tube, Endo */
"0UP88DZ"  =  "1"  /* Removal of Intraluminal Device from Fallopian Tube, Endo */
"0UP88JZ"  =  "1"  /* Removal of Synthetic Substitute from Fallopian Tube, Endo */
"0UP88KZ"  =  "1"  /* Removal of Nonaut Sub from Fallopian Tube, Endo */
"0UPD00Z"  =  "1"  /* Removal of Drain Dev from Uterus & Cervix, Open Approach */
"0UPD01Z"  =  "1"  /* Removal of Radioact Elem from Uterus & Cervix, Open Approach */
"0UPD03Z"  =  "1"  /* Removal of Infusion Dev from Uterus & Cervix, Open Approach */
"0UPD07Z"  =  "1"  /* Removal of Autol Sub from Uterus & Cervix, Open Approach */
"0UPD0CZ"  =  "1"  /* Removal of Extralum Dev from Uterus & Cervix, Open Approach */
"0UPD0DZ"  =  "1"  /* Removal of Intralum Dev from Uterus & Cervix, Open Approach */
"0UPD0HZ"  =  "1"  /* Removal of Contracep Dev from Uterus & Cervix, Open Approach */
"0UPD0JZ"  =  "1"  /* Removal of Synth Sub from Uterus & Cervix, Open Approach */
"0UPD0KZ"  =  "1"  /* Removal of Nonaut Sub from Uterus & Cervix, Open Approach */
"0UPD30Z"  =  "1"  /* Removal of Drain Dev from Uterus & Cervix, Perc Approach */
"0UPD31Z"  =  "1"  /* Removal of Radioact Elem from Uterus & Cervix, Perc Approach */
"0UPD33Z"  =  "1"  /* Removal of Infusion Dev from Uterus & Cervix, Perc Approach */
"0UPD37Z"  =  "1"  /* Removal of Autol Sub from Uterus & Cervix, Perc Approach */
"0UPD3DZ"  =  "1"  /* Removal of Intralum Dev from Uterus & Cervix, Perc Approach */
"0UPD3HZ"  =  "1"  /* Removal of Contracep Dev from Uterus & Cervix, Perc Approach */
"0UPD3JZ"  =  "1"  /* Removal of Synth Sub from Uterus & Cervix, Perc Approach */
"0UPD3KZ"  =  "1"  /* Removal of Nonaut Sub from Uterus & Cervix, Perc Approach */
"0UPD40Z"  =  "1"  /* Remove of Drain Dev from Uterus & Cervix, Perc Endo Approach */
"0UPD41Z"  =  "1"  /* Remove Radioact Elem from Uterus & Cervix, Perc Endo */
"0UPD43Z"  =  "1"  /* Remove Infusion Dev from Uterus & Cervix, Perc Endo */
"0UPD47Z"  =  "1"  /* Remove of Autol Sub from Uterus & Cervix, Perc Endo Approach */
"0UPD4DZ"  =  "1"  /* Remove Intralum Dev from Uterus & Cervix, Perc Endo */
"0UPD4HZ"  =  "1"  /* Remove Contracep Dev from Uterus & Cervix, Perc Endo */
"0UPD4JZ"  =  "1"  /* Remove of Synth Sub from Uterus & Cervix, Perc Endo Approach */
"0UPD4KZ"  =  "1"  /* Remove Nonaut Sub from Uterus & Cervix, Perc Endo */
"0UPD70Z"  =  "1"  /* Removal of Drainage Device from Uterus & Cervix, Via Opening */
"0UPD71Z"  =  "1"  /* Removal of Radioact Elem from Uterus & Cervix, Via Opening */
"0UPD73Z"  =  "1"  /* Removal of Infusion Device from Uterus & Cervix, Via Opening */
"0UPD77Z"  =  "1"  /* Removal of Autol Sub from Uterus & Cervix, Via Opening */
"0UPD7DZ"  =  "1"  /* Removal of Intralum Dev from Uterus & Cervix, Via Opening */
"0UPD7JZ"  =  "1"  /* Removal of Synth Sub from Uterus & Cervix, Via Opening */
"0UPD7KZ"  =  "1"  /* Removal of Nonaut Sub from Uterus & Cervix, Via Opening */
"0UPD80Z"  =  "1"  /* Removal of Drainage Device from Uterus and Cervix, Endo */
"0UPD81Z"  =  "1"  /* Removal of Radioactive Element from Uterus and Cervix, Endo */
"0UPD83Z"  =  "1"  /* Removal of Infusion Device from Uterus and Cervix, Endo */
"0UPD87Z"  =  "1"  /* Removal of Autol Sub from Uterus & Cervix, Endo */
"0UPD8DZ"  =  "1"  /* Removal of Intraluminal Device from Uterus and Cervix, Endo */
"0UPD8JZ"  =  "1"  /* Removal of Synthetic Substitute from Uterus and Cervix, Endo */
"0UPD8KZ"  =  "1"  /* Removal of Nonaut Sub from Uterus & Cervix, Endo */
"0UPH00Z"  =  "1"  /* Removal of Drain Dev from Vagina & Cul-de-sac, Open Approach */
"0UPH01Z"  =  "1"  /* Remove Radioact Elem from Vagina & Cul-de-sac, Open */
"0UPH03Z"  =  "1"  /* Remove Infusion Dev from Vagina & Cul-de-sac, Open */
"0UPH07Z"  =  "1"  /* Removal of Autol Sub from Vagina & Cul-de-sac, Open Approach */
"0UPH0DZ"  =  "1"  /* Remove Intralum Dev from Vagina & Cul-de-sac, Open */
"0UPH0JZ"  =  "1"  /* Removal of Synth Sub from Vagina & Cul-de-sac, Open Approach */
"0UPH0KZ"  =  "1"  /* Remove of Nonaut Sub from Vagina & Cul-de-sac, Open Approach */
"0UPH30Z"  =  "1"  /* Removal of Drain Dev from Vagina & Cul-de-sac, Perc Approach */
"0UPH31Z"  =  "1"  /* Remove Radioact Elem from Vagina & Cul-de-sac, Perc */
"0UPH33Z"  =  "1"  /* Remove Infusion Dev from Vagina & Cul-de-sac, Perc */
"0UPH37Z"  =  "1"  /* Removal of Autol Sub from Vagina & Cul-de-sac, Perc Approach */
"0UPH3DZ"  =  "1"  /* Remove Intralum Dev from Vagina & Cul-de-sac, Perc */
"0UPH3JZ"  =  "1"  /* Removal of Synth Sub from Vagina & Cul-de-sac, Perc Approach */
"0UPH3KZ"  =  "1"  /* Remove of Nonaut Sub from Vagina & Cul-de-sac, Perc Approach */
"0UPH40Z"  =  "1"  /* Remove Drain Dev from Vagina & Cul-de-sac, Perc Endo */
"0UPH41Z"  =  "1"  /* Remove Radioact Elem from Vagina & Cul-de-sac, Perc Endo */
"0UPH43Z"  =  "1"  /* Remove Infusion Dev from Vagina & Cul-de-sac, Perc Endo */
"0UPH47Z"  =  "1"  /* Remove Autol Sub from Vagina & Cul-de-sac, Perc Endo */
"0UPH4DZ"  =  "1"  /* Remove Intralum Dev from Vagina & Cul-de-sac, Perc Endo */
"0UPH4JZ"  =  "1"  /* Remove Synth Sub from Vagina & Cul-de-sac, Perc Endo */
"0UPH4KZ"  =  "1"  /* Remove Nonaut Sub from Vagina & Cul-de-sac, Perc Endo */
"0UPH70Z"  =  "1"  /* Removal of Drain Dev from Vagina & Cul-de-sac, Via Opening */
"0UPH71Z"  =  "1"  /* Remove Radioact Elem from Vagina & Cul-de-sac, Via Opening */
"0UPH73Z"  =  "1"  /* Remove of Infusion Dev from Vagina & Cul-de-sac, Via Opening */
"0UPH77Z"  =  "1"  /* Removal of Autol Sub from Vagina & Cul-de-sac, Via Opening */
"0UPH7DZ"  =  "1"  /* Remove of Intralum Dev from Vagina & Cul-de-sac, Via Opening */
"0UPH7JZ"  =  "1"  /* Removal of Synth Sub from Vagina & Cul-de-sac, Via Opening */
"0UPH7KZ"  =  "1"  /* Removal of Nonaut Sub from Vagina & Cul-de-sac, Via Opening */
"0UPH80Z"  =  "1"  /* Removal of Drainage Device from Vagina and Cul-de-sac, Endo */
"0UPH81Z"  =  "1"  /* Removal of Radioact Elem from Vagina & Cul-de-sac, Endo */
"0UPH83Z"  =  "1"  /* Removal of Infusion Device from Vagina and Cul-de-sac, Endo */
"0UPH87Z"  =  "1"  /* Removal of Autol Sub from Vagina & Cul-de-sac, Endo */
"0UPH8DZ"  =  "1"  /* Removal of Intralum Dev from Vagina & Cul-de-sac, Endo */
"0UPH8JZ"  =  "1"  /* Removal of Synth Sub from Vagina & Cul-de-sac, Endo */
"0UPH8KZ"  =  "1"  /* Removal of Nonaut Sub from Vagina & Cul-de-sac, Endo */
"0UPM00Z"  =  "1"  /* Removal of Drainage Device from Vulva, Open Approach */
"0UPM07Z"  =  "1"  /* Removal of Autol Sub from Vulva, Open Approach */
"0UPM0JZ"  =  "1"  /* Removal of Synthetic Substitute from Vulva, Open Approach */
"0UPM0KZ"  =  "1"  /* Removal of Nonaut Sub from Vulva, Open Approach */
"0UQ00ZZ"  =  "1"  /* Repair Right Ovary, Open Approach */
"0UQ03ZZ"  =  "1"  /* Repair Right Ovary, Percutaneous Approach */
"0UQ04ZZ"  =  "1"  /* Repair Right Ovary, Percutaneous Endoscopic Approach */
"0UQ10ZZ"  =  "1"  /* Repair Left Ovary, Open Approach */
"0UQ13ZZ"  =  "1"  /* Repair Left Ovary, Percutaneous Approach */
"0UQ14ZZ"  =  "1"  /* Repair Left Ovary, Percutaneous Endoscopic Approach */
"0UQ20ZZ"  =  "1"  /* Repair Bilateral Ovaries, Open Approach */
"0UQ23ZZ"  =  "1"  /* Repair Bilateral Ovaries, Percutaneous Approach */
"0UQ24ZZ"  =  "1"  /* Repair Bilateral Ovaries, Percutaneous Endoscopic Approach */
"0UQ40ZZ"  =  "1"  /* Repair Uterine Supporting Structure, Open Approach */
"0UQ43ZZ"  =  "1"  /* Repair Uterine Supporting Structure, Percutaneous Approach */
"0UQ44ZZ"  =  "1"  /* Repair Uterine Supporting Structure, Perc Endo Approach */
"0UQ50ZZ"  =  "1"  /* Repair Right Fallopian Tube, Open Approach */
"0UQ53ZZ"  =  "1"  /* Repair Right Fallopian Tube, Percutaneous Approach */
"0UQ54ZZ"  =  "1"  /* Repair Right Fallopian Tube, Perc Endo Approach */
"0UQ57ZZ"  =  "1"  /* Repair Right Fallopian Tube, Via Opening */
"0UQ58ZZ"  =  "1"  /* Repair Right Fallopian Tube, Endo */
"0UQ60ZZ"  =  "1"  /* Repair Left Fallopian Tube, Open Approach */
"0UQ63ZZ"  =  "1"  /* Repair Left Fallopian Tube, Percutaneous Approach */
"0UQ64ZZ"  =  "1"  /* Repair Left Fallopian Tube, Percutaneous Endoscopic Approach */
"0UQ67ZZ"  =  "1"  /* Repair Left Fallopian Tube, Via Opening */
"0UQ68ZZ"  =  "1"  /* Repair Left Fallopian Tube, Endo */
"0UQ70ZZ"  =  "1"  /* Repair Bilateral Fallopian Tubes, Open Approach */
"0UQ73ZZ"  =  "1"  /* Repair Bilateral Fallopian Tubes, Percutaneous Approach */
"0UQ74ZZ"  =  "1"  /* Repair Bilateral Fallopian Tubes, Perc Endo Approach */
"0UQ77ZZ"  =  "1"  /* Repair Bilateral Fallopian Tubes, Via Opening */
"0UQ78ZZ"  =  "1"  /* Repair Bilateral Fallopian Tubes, Endo */
"0UQ90ZZ"  =  "1"  /* Repair Uterus, Open Approach */
"0UQ93ZZ"  =  "1"  /* Repair Uterus, Percutaneous Approach */
"0UQ94ZZ"  =  "1"  /* Repair Uterus, Percutaneous Endoscopic Approach */
"0UQ97ZZ"  =  "1"  /* Repair Uterus, Via Natural or Artificial Opening */
"0UQ98ZZ"  =  "1"  /* Repair Uterus, Via Natural or Artificial Opening Endoscopic */
"0UQC0ZZ"  =  "1"  /* Repair Cervix, Open Approach */
"0UQC3ZZ"  =  "1"  /* Repair Cervix, Percutaneous Approach */
"0UQC4ZZ"  =  "1"  /* Repair Cervix, Percutaneous Endoscopic Approach */
"0UQC7ZZ"  =  "1"  /* Repair Cervix, Via Natural or Artificial Opening */
"0UQC8ZZ"  =  "1"  /* Repair Cervix, Via Natural or Artificial Opening Endoscopic */
"0UQF0ZZ"  =  "1"  /* Repair Cul-de-sac, Open Approach */
"0UQF3ZZ"  =  "1"  /* Repair Cul-de-sac, Percutaneous Approach */
"0UQF4ZZ"  =  "1"  /* Repair Cul-de-sac, Percutaneous Endoscopic Approach */
"0UQF7ZZ"  =  "1"  /* Repair Cul-de-sac, Via Natural or Artificial Opening */
"0UQF8ZZ"  =  "1"  /* Repair Cul-de-sac, Endo */
"0UQG0ZZ"  =  "1"  /* Repair Vagina, Open Approach */
"0UQG3ZZ"  =  "1"  /* Repair Vagina, Percutaneous Approach */
"0UQG4ZZ"  =  "1"  /* Repair Vagina, Percutaneous Endoscopic Approach */
"0UQJ0ZZ"  =  "1"  /* Repair Clitoris, Open Approach */
"0UQJXZZ"  =  "1"  /* Repair Clitoris, External Approach */
"0UQK0ZZ"  =  "1"  /* Repair Hymen, Open Approach */
"0UQK3ZZ"  =  "1"  /* Repair Hymen, Percutaneous Approach */
"0UQK4ZZ"  =  "1"  /* Repair Hymen, Percutaneous Endoscopic Approach */
"0UQK7ZZ"  =  "1"  /* Repair Hymen, Via Natural or Artificial Opening */
"0UQK8ZZ"  =  "1"  /* Repair Hymen, Via Natural or Artificial Opening Endoscopic */
"0UQKXZZ"  =  "1"  /* Repair Hymen, External Approach */
"0UQL0ZZ"  =  "1"  /* Repair Vestibular Gland, Open Approach */
"0UQLXZZ"  =  "1"  /* Repair Vestibular Gland, External Approach */
"0US00ZZ"  =  "1"  /* Reposition Right Ovary, Open Approach */
"0US04ZZ"  =  "1"  /* Reposition Right Ovary, Percutaneous Endoscopic Approach */
"0US10ZZ"  =  "1"  /* Reposition Left Ovary, Open Approach */
"0US14ZZ"  =  "1"  /* Reposition Left Ovary, Percutaneous Endoscopic Approach */
"0US20ZZ"  =  "1"  /* Reposition Bilateral Ovaries, Open Approach */
"0US24ZZ"  =  "1"  /* Reposition Bilateral Ovaries, Perc Endo Approach */
"0US40ZZ"  =  "1"  /* Reposition Uterine Supporting Structure, Open Approach */
"0US44ZZ"  =  "1"  /* Reposition Uterine Supporting Structure, Perc Endo Approach */
"0US50ZZ"  =  "1"  /* Reposition Right Fallopian Tube, Open Approach */
"0US54ZZ"  =  "1"  /* Reposition Right Fallopian Tube, Perc Endo Approach */
"0US60ZZ"  =  "1"  /* Reposition Left Fallopian Tube, Open Approach */
"0US64ZZ"  =  "1"  /* Reposition Left Fallopian Tube, Perc Endo Approach */
"0US70ZZ"  =  "1"  /* Reposition Bilateral Fallopian Tubes, Open Approach */
"0US74ZZ"  =  "1"  /* Reposition Bilateral Fallopian Tubes, Perc Endo Approach */
"0US90ZZ"  =  "1"  /* Reposition Uterus, Open Approach */
"0US94ZZ"  =  "1"  /* Reposition Uterus, Percutaneous Endoscopic Approach */
"0USC0ZZ"  =  "1"  /* Reposition Cervix, Open Approach */
"0USC4ZZ"  =  "1"  /* Reposition Cervix, Percutaneous Endoscopic Approach */
"0USF0ZZ"  =  "1"  /* Reposition Cul-de-sac, Open Approach */
"0USF4ZZ"  =  "1"  /* Reposition Cul-de-sac, Percutaneous Endoscopic Approach */
"0USG0ZZ"  =  "1"  /* Reposition Vagina, Open Approach */
"0USG4ZZ"  =  "1"  /* Reposition Vagina, Percutaneous Endoscopic Approach */
"0USGXZZ"  =  "1"  /* Reposition Vagina, External Approach */
"0UT00ZZ"  =  "1"  /* Resection of Right Ovary, Open Approach */
"0UT04ZZ"  =  "1"  /* Resection of Right Ovary, Percutaneous Endoscopic Approach */
"0UT07ZZ"  =  "1"  /* Resection of Right Ovary, Via Natural or Artificial Opening */
"0UT08ZZ"  =  "1"  /* Resection of Right Ovary, Endo */
"0UT0FZZ"  =  "1"  /* Resection of Right Ovary, Via Opening w Perc Endo */
"0UT10ZZ"  =  "1"  /* Resection of Left Ovary, Open Approach */
"0UT14ZZ"  =  "1"  /* Resection of Left Ovary, Percutaneous Endoscopic Approach */
"0UT17ZZ"  =  "1"  /* Resection of Left Ovary, Via Natural or Artificial Opening */
"0UT18ZZ"  =  "1"  /* Resection of Left Ovary, Endo */
"0UT1FZZ"  =  "1"  /* Resection of Left Ovary, Via Opening w Perc Endo */
"0UT20ZZ"  =  "1"  /* Resection of Bilateral Ovaries, Open Approach */
"0UT24ZZ"  =  "1"  /* Resection of Bilateral Ovaries, Perc Endo Approach */
"0UT27ZZ"  =  "1"  /* Resection of Bilateral Ovaries, Via Opening */
"0UT28ZZ"  =  "1"  /* Resection of Bilateral Ovaries, Endo */
"0UT2FZZ"  =  "1"  /* Resection of Bilateral Ovaries, Via Opening w Perc Endo */
"0UT40ZZ"  =  "1"  /* Resection of Uterine Supporting Structure, Open Approach */
"0UT44ZZ"  =  "1"  /* Resection of Uterine Support Struct, Perc Endo Approach */
"0UT47ZZ"  =  "1"  /* Resection of Uterine Supporting Structure, Via Opening */
"0UT48ZZ"  =  "1"  /* Resection of Uterine Supporting Structure, Endo */
"0UT50ZZ"  =  "1"  /* Resection of Right Fallopian Tube, Open Approach */
"0UT54ZZ"  =  "1"  /* Resection of Right Fallopian Tube, Perc Endo Approach */
"0UT57ZZ"  =  "1"  /* Resection of Right Fallopian Tube, Via Opening */
"0UT58ZZ"  =  "1"  /* Resection of Right Fallopian Tube, Endo */
"0UT5FZZ"  =  "1"  /* Resection of Right Fallopian Tube, Via Opening w Perc Endo */
"0UT60ZZ"  =  "1"  /* Resection of Left Fallopian Tube, Open Approach */
"0UT64ZZ"  =  "1"  /* Resection of Left Fallopian Tube, Perc Endo Approach */
"0UT67ZZ"  =  "1"  /* Resection of Left Fallopian Tube, Via Opening */
"0UT68ZZ"  =  "1"  /* Resection of Left Fallopian Tube, Endo */
"0UT6FZZ"  =  "1"  /* Resection of Left Fallopian Tube, Via Opening w Perc Endo */
"0UT70ZZ"  =  "1"  /* Resection of Bilateral Fallopian Tubes, Open Approach */
"0UT74ZZ"  =  "1"  /* Resection of Bilateral Fallopian Tubes, Perc Endo Approach */
"0UT77ZZ"  =  "1"  /* Resection of Bilateral Fallopian Tubes, Via Opening */
"0UT78ZZ"  =  "1"  /* Resection of Bilateral Fallopian Tubes, Endo */
"0UT7FZZ"  =  "1"  /* Resection of Bi Fallopian Tube, Via Opening w Perc Endo */
"0UT90ZZ"  =  "1"  /* Resection of Uterus, Open Approach */
"0UT94ZZ"  =  "1"  /* Resection of Uterus, Percutaneous Endoscopic Approach */
"0UT97ZZ"  =  "1"  /* Resection of Uterus, Via Natural or Artificial Opening */
"0UT98ZZ"  =  "1"  /* Resection of Uterus, Endo */
"0UT9FZZ"  =  "1"  /* Resection of Uterus, Via Opening w Perc Endo */
"0UTC0ZZ"  =  "1"  /* Resection of Cervix, Open Approach */
"0UTC4ZZ"  =  "1"  /* Resection of Cervix, Percutaneous Endoscopic Approach */
"0UTC7ZZ"  =  "1"  /* Resection of Cervix, Via Natural or Artificial Opening */
"0UTC8ZZ"  =  "1"  /* Resection of Cervix, Endo */
"0UTF0ZZ"  =  "1"  /* Resection of Cul-de-sac, Open Approach */
"0UTF4ZZ"  =  "1"  /* Resection of Cul-de-sac, Percutaneous Endoscopic Approach */
"0UTF7ZZ"  =  "1"  /* Resection of Cul-de-sac, Via Natural or Artificial Opening */
"0UTF8ZZ"  =  "1"  /* Resection of Cul-de-sac, Endo */
"0UTG0ZZ"  =  "1"  /* Resection of Vagina, Open Approach */
"0UTG4ZZ"  =  "1"  /* Resection of Vagina, Percutaneous Endoscopic Approach */
"0UTG7ZZ"  =  "1"  /* Resection of Vagina, Via Natural or Artificial Opening */
"0UTG8ZZ"  =  "1"  /* Resection of Vagina, Endo */
"0UTJ0ZZ"  =  "1"  /* Resection of Clitoris, Open Approach */
"0UTJXZZ"  =  "1"  /* Resection of Clitoris, External Approach */
"0UTK0ZZ"  =  "1"  /* Resection of Hymen, Open Approach */
"0UTK4ZZ"  =  "1"  /* Resection of Hymen, Percutaneous Endoscopic Approach */
"0UTK7ZZ"  =  "1"  /* Resection of Hymen, Via Natural or Artificial Opening */
"0UTK8ZZ"  =  "1"  /* Resection of Hymen, Endo */
"0UTKXZZ"  =  "1"  /* Resection of Hymen, External Approach */
"0UTL0ZZ"  =  "1"  /* Resection of Vestibular Gland, Open Approach */
"0UTLXZZ"  =  "1"  /* Resection of Vestibular Gland, External Approach */
"0UTM0ZZ"  =  "1"  /* Resection of Vulva, Open Approach */
"0UTMXZZ"  =  "1"  /* Resection of Vulva, External Approach */
"0UU407Z"  =  "1"  /* Supplement Uterine Support Struct w Autol Sub, Open */
"0UU40JZ"  =  "1"  /* Supplement Uterine Support Struct w Synth Sub, Open */
"0UU40KZ"  =  "1"  /* Supplement Uterine Support Struct w Nonaut Sub, Open */
"0UU447Z"  =  "1"  /* Supplement Uterine Support Struct w Autol Sub, Perc Endo */
"0UU44JZ"  =  "1"  /* Supplement Uterine Support Struct w Synth Sub, Perc Endo */
"0UU44KZ"  =  "1"  /* Supplement Uterine Support Struct w Nonaut Sub, Perc Endo */
"0UU507Z"  =  "1"  /* Supplement R Fallopian Tube with Autol Sub, Open Approach */
"0UU50JZ"  =  "1"  /* Supplement R Fallopian Tube with Synth Sub, Open Approach */
"0UU50KZ"  =  "1"  /* Supplement R Fallopian Tube with Nonaut Sub, Open Approach */
"0UU547Z"  =  "1"  /* Supplement R Fallopian Tube w Autol Sub, Perc Endo */
"0UU54JZ"  =  "1"  /* Supplement R Fallopian Tube w Synth Sub, Perc Endo */
"0UU54KZ"  =  "1"  /* Supplement R Fallopian Tube w Nonaut Sub, Perc Endo */
"0UU577Z"  =  "1"  /* Supplement Right Fallopian Tube with Autol Sub, Via Opening */
"0UU57JZ"  =  "1"  /* Supplement Right Fallopian Tube with Synth Sub, Via Opening */
"0UU57KZ"  =  "1"  /* Supplement Right Fallopian Tube with Nonaut Sub, Via Opening */
"0UU587Z"  =  "1"  /* Supplement Right Fallopian Tube with Autol Sub, Endo */
"0UU58JZ"  =  "1"  /* Supplement Right Fallopian Tube with Synth Sub, Endo */
"0UU58KZ"  =  "1"  /* Supplement Right Fallopian Tube with Nonaut Sub, Endo */
"0UU607Z"  =  "1"  /* Supplement Left Fallopian Tube with Autol Sub, Open Approach */
"0UU60JZ"  =  "1"  /* Supplement Left Fallopian Tube with Synth Sub, Open Approach */
"0UU60KZ"  =  "1"  /* Supplement L Fallopian Tube with Nonaut Sub, Open Approach */
"0UU647Z"  =  "1"  /* Supplement L Fallopian Tube w Autol Sub, Perc Endo */
"0UU64JZ"  =  "1"  /* Supplement L Fallopian Tube w Synth Sub, Perc Endo */
"0UU64KZ"  =  "1"  /* Supplement L Fallopian Tube w Nonaut Sub, Perc Endo */
"0UU677Z"  =  "1"  /* Supplement Left Fallopian Tube with Autol Sub, Via Opening */
"0UU67JZ"  =  "1"  /* Supplement Left Fallopian Tube with Synth Sub, Via Opening */
"0UU67KZ"  =  "1"  /* Supplement Left Fallopian Tube with Nonaut Sub, Via Opening */
"0UU687Z"  =  "1"  /* Supplement Left Fallopian Tube with Autol Sub, Endo */
"0UU68JZ"  =  "1"  /* Supplement Left Fallopian Tube with Synth Sub, Endo */
"0UU68KZ"  =  "1"  /* Supplement Left Fallopian Tube with Nonaut Sub, Endo */
"0UU707Z"  =  "1"  /* Supplement Bi Fallopian Tube with Autol Sub, Open Approach */
"0UU70JZ"  =  "1"  /* Supplement Bi Fallopian Tube with Synth Sub, Open Approach */
"0UU70KZ"  =  "1"  /* Supplement Bi Fallopian Tube with Nonaut Sub, Open Approach */
"0UU747Z"  =  "1"  /* Supplement Bi Fallopian Tube w Autol Sub, Perc Endo */
"0UU74JZ"  =  "1"  /* Supplement Bi Fallopian Tube w Synth Sub, Perc Endo */
"0UU74KZ"  =  "1"  /* Supplement Bi Fallopian Tube w Nonaut Sub, Perc Endo */
"0UU777Z"  =  "1"  /* Supplement Bi Fallopian Tube with Autol Sub, Via Opening */
"0UU77JZ"  =  "1"  /* Supplement Bi Fallopian Tube with Synth Sub, Via Opening */
"0UU77KZ"  =  "1"  /* Supplement Bi Fallopian Tube with Nonaut Sub, Via Opening */
"0UU787Z"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Autol Sub, Endo */
"0UU78JZ"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Synth Sub, Endo */
"0UU78KZ"  =  "1"  /* Supplement Bilateral Fallopian Tubes with Nonaut Sub, Endo */
"0UUF07Z"  =  "1"  /* Supplement Cul-de-sac with Autol Sub, Open Approach */
"0UUF0JZ"  =  "1"  /* Supplement Cul-de-sac with Synth Sub, Open Approach */
"0UUF0KZ"  =  "1"  /* Supplement Cul-de-sac with Nonaut Sub, Open Approach */
"0UUF47Z"  =  "1"  /* Supplement Cul-de-sac with Autol Sub, Perc Endo Approach */
"0UUF4JZ"  =  "1"  /* Supplement Cul-de-sac with Synth Sub, Perc Endo Approach */
"0UUF4KZ"  =  "1"  /* Supplement Cul-de-sac with Nonaut Sub, Perc Endo Approach */
"0UUF77Z"  =  "1"  /* Supplement Cul-de-sac with Autol Sub, Via Opening */
"0UUF7JZ"  =  "1"  /* Supplement Cul-de-sac with Synthetic Substitute, Via Opening */
"0UUF7KZ"  =  "1"  /* Supplement Cul-de-sac with Nonaut Sub, Via Opening */
"0UUF87Z"  =  "1"  /* Supplement Cul-de-sac with Autol Sub, Endo */
"0UUF8JZ"  =  "1"  /* Supplement Cul-de-sac with Synthetic Substitute, Endo */
"0UUF8KZ"  =  "1"  /* Supplement Cul-de-sac with Nonaut Sub, Endo */
"0UUG07Z"  =  "1"  /* Supplement Vagina with Autol Sub, Open Approach */
"0UUG0JZ"  =  "1"  /* Supplement Vagina with Synthetic Substitute, Open Approach */
"0UUG0KZ"  =  "1"  /* Supplement Vagina with Nonaut Sub, Open Approach */
"0UUG47Z"  =  "1"  /* Supplement Vagina with Autol Sub, Perc Endo Approach */
"0UUG4JZ"  =  "1"  /* Supplement Vagina with Synth Sub, Perc Endo Approach */
"0UUG4KZ"  =  "1"  /* Supplement Vagina with Nonaut Sub, Perc Endo Approach */
"0UUG77Z"  =  "1"  /* Supplement Vagina with Autol Sub, Via Opening */
"0UUG7JZ"  =  "1"  /* Supplement Vagina with Synthetic Substitute, Via Opening */
"0UUG7KZ"  =  "1"  /* Supplement Vagina with Nonaut Sub, Via Opening */
"0UUG87Z"  =  "1"  /* Supplement Vagina with Autologous Tissue Substitute, Endo */
"0UUG8JZ"  =  "1"  /* Supplement Vagina with Synthetic Substitute, Endo */
"0UUG8KZ"  =  "1"  /* Supplement Vagina with Nonautologous Tissue Substitute, Endo */
"0UUGX7Z"  =  "1"  /* Supplement Vagina with Autol Sub, Extern Approach */
"0UUGXJZ"  =  "1"  /* Supplement Vagina with Synthetic Substitute, Extern Approach */
"0UUGXKZ"  =  "1"  /* Supplement Vagina with Nonaut Sub, Extern Approach */
"0UUJ07Z"  =  "1"  /* Supplement Clitoris with Autol Sub, Open Approach */
"0UUJ0JZ"  =  "1"  /* Supplement Clitoris with Synthetic Substitute, Open Approach */
"0UUJ0KZ"  =  "1"  /* Supplement Clitoris with Nonaut Sub, Open Approach */
"0UUJX7Z"  =  "1"  /* Supplement Clitoris with Autol Sub, Extern Approach */
"0UUJXJZ"  =  "1"  /* Supplement Clitoris with Synth Sub, Extern Approach */
"0UUJXKZ"  =  "1"  /* Supplement Clitoris with Nonaut Sub, Extern Approach */
"0UUK07Z"  =  "1"  /* Supplement Hymen with Autol Sub, Open Approach */
"0UUK0JZ"  =  "1"  /* Supplement Hymen with Synthetic Substitute, Open Approach */
"0UUK0KZ"  =  "1"  /* Supplement Hymen with Nonaut Sub, Open Approach */
"0UUK47Z"  =  "1"  /* Supplement Hymen with Autol Sub, Perc Endo Approach */
"0UUK4JZ"  =  "1"  /* Supplement Hymen with Synth Sub, Perc Endo Approach */
"0UUK4KZ"  =  "1"  /* Supplement Hymen with Nonaut Sub, Perc Endo Approach */
"0UUK77Z"  =  "1"  /* Supplement Hymen with Autol Sub, Via Opening */
"0UUK7JZ"  =  "1"  /* Supplement Hymen with Synthetic Substitute, Via Opening */
"0UUK7KZ"  =  "1"  /* Supplement Hymen with Nonaut Sub, Via Opening */
"0UUK87Z"  =  "1"  /* Supplement Hymen with Autologous Tissue Substitute, Endo */
"0UUK8JZ"  =  "1"  /* Supplement Hymen with Synthetic Substitute, Endo */
"0UUK8KZ"  =  "1"  /* Supplement Hymen with Nonautologous Tissue Substitute, Endo */
"0UUKX7Z"  =  "1"  /* Supplement Hymen with Autol Sub, Extern Approach */
"0UUKXJZ"  =  "1"  /* Supplement Hymen with Synthetic Substitute, Extern Approach */
"0UUKXKZ"  =  "1"  /* Supplement Hymen with Nonaut Sub, Extern Approach */
"0UUM07Z"  =  "1"  /* Supplement Vulva with Autol Sub, Open Approach */
"0UUM0JZ"  =  "1"  /* Supplement Vulva with Synthetic Substitute, Open Approach */
"0UUM0KZ"  =  "1"  /* Supplement Vulva with Nonaut Sub, Open Approach */
"0UUMX7Z"  =  "1"  /* Supplement Vulva with Autol Sub, Extern Approach */
"0UUMXJZ"  =  "1"  /* Supplement Vulva with Synthetic Substitute, Extern Approach */
"0UUMXKZ"  =  "1"  /* Supplement Vulva with Nonaut Sub, Extern Approach */
"0UVC0CZ"  =  "1"  /* Restriction of Cervix with Extralum Dev, Open Approach */
"0UVC0DZ"  =  "1"  /* Restriction of Cervix with Intralum Dev, Open Approach */
"0UVC0ZZ"  =  "1"  /* Restriction of Cervix, Open Approach */
"0UVC3CZ"  =  "1"  /* Restriction of Cervix with Extralum Dev, Perc Approach */
"0UVC3DZ"  =  "1"  /* Restriction of Cervix with Intralum Dev, Perc Approach */
"0UVC3ZZ"  =  "1"  /* Restriction of Cervix, Percutaneous Approach */
"0UVC4CZ"  =  "1"  /* Restriction of Cervix with Extralum Dev, Perc Endo Approach */
"0UVC4DZ"  =  "1"  /* Restriction of Cervix with Intralum Dev, Perc Endo Approach */
"0UVC4ZZ"  =  "1"  /* Restriction of Cervix, Percutaneous Endoscopic Approach */
"0UVC7DZ"  =  "1"  /* Restriction of Cervix with Intraluminal Device, Via Opening */
"0UVC7ZZ"  =  "1"  /* Restriction of Cervix, Via Natural or Artificial Opening */
"0UVC8DZ"  =  "1"  /* Restriction of Cervix with Intraluminal Device, Endo */
"0UVC8ZZ"  =  "1"  /* Restriction of Cervix, Endo */
"0UW300Z"  =  "1"  /* Revision of Drainage Device in Ovary, Open Approach */
"0UW303Z"  =  "1"  /* Revision of Infusion Device in Ovary, Open Approach */
"0UW330Z"  =  "1"  /* Revision of Drainage Device in Ovary, Percutaneous Approach */
"0UW333Z"  =  "1"  /* Revision of Infusion Device in Ovary, Percutaneous Approach */
"0UW340Z"  =  "1"  /* Revision of Drainage Device in Ovary, Perc Endo Approach */
"0UW343Z"  =  "1"  /* Revision of Infusion Device in Ovary, Perc Endo Approach */
"0UW800Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Open Approach */
"0UW803Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Open Approach */
"0UW807Z"  =  "1"  /* Revision of Autol Sub in Fallopian Tube, Open Approach */
"0UW80CZ"  =  "1"  /* Revision of Extralum Dev in Fallopian Tube, Open Approach */
"0UW80DZ"  =  "1"  /* Revision of Intralum Dev in Fallopian Tube, Open Approach */
"0UW80JZ"  =  "1"  /* Revision of Synth Sub in Fallopian Tube, Open Approach */
"0UW80KZ"  =  "1"  /* Revision of Nonaut Sub in Fallopian Tube, Open Approach */
"0UW830Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Perc Approach */
"0UW833Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Perc Approach */
"0UW837Z"  =  "1"  /* Revision of Autol Sub in Fallopian Tube, Perc Approach */
"0UW83CZ"  =  "1"  /* Revision of Extralum Dev in Fallopian Tube, Perc Approach */
"0UW83DZ"  =  "1"  /* Revision of Intralum Dev in Fallopian Tube, Perc Approach */
"0UW83JZ"  =  "1"  /* Revision of Synth Sub in Fallopian Tube, Perc Approach */
"0UW83KZ"  =  "1"  /* Revision of Nonaut Sub in Fallopian Tube, Perc Approach */
"0UW840Z"  =  "1"  /* Revision of Drain Dev in Fallopian Tube, Perc Endo Approach */
"0UW843Z"  =  "1"  /* Revise of Infusion Dev in Fallopian Tube, Perc Endo Approach */
"0UW847Z"  =  "1"  /* Revision of Autol Sub in Fallopian Tube, Perc Endo Approach */
"0UW84CZ"  =  "1"  /* Revise of Extralum Dev in Fallopian Tube, Perc Endo Approach */
"0UW84DZ"  =  "1"  /* Revise of Intralum Dev in Fallopian Tube, Perc Endo Approach */
"0UW84JZ"  =  "1"  /* Revision of Synth Sub in Fallopian Tube, Perc Endo Approach */
"0UW84KZ"  =  "1"  /* Revision of Nonaut Sub in Fallopian Tube, Perc Endo Approach */
"0UW870Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Via Opening */
"0UW873Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Via Opening */
"0UW877Z"  =  "1"  /* Revision of Autol Sub in Fallopian Tube, Via Opening */
"0UW87CZ"  =  "1"  /* Revision of Extralum Dev in Fallopian Tube, Via Opening */
"0UW87DZ"  =  "1"  /* Revision of Intralum Dev in Fallopian Tube, Via Opening */
"0UW87JZ"  =  "1"  /* Revision of Synth Sub in Fallopian Tube, Via Opening */
"0UW87KZ"  =  "1"  /* Revision of Nonaut Sub in Fallopian Tube, Via Opening */
"0UW880Z"  =  "1"  /* Revision of Drainage Device in Fallopian Tube, Endo */
"0UW883Z"  =  "1"  /* Revision of Infusion Device in Fallopian Tube, Endo */
"0UW887Z"  =  "1"  /* Revision of Autol Sub in Fallopian Tube, Endo */
"0UW88CZ"  =  "1"  /* Revision of Extraluminal Device in Fallopian Tube, Endo */
"0UW88DZ"  =  "1"  /* Revision of Intraluminal Device in Fallopian Tube, Endo */
"0UW88JZ"  =  "1"  /* Revision of Synthetic Substitute in Fallopian Tube, Endo */
"0UW88KZ"  =  "1"  /* Revision of Nonaut Sub in Fallopian Tube, Endo */
"0UWD00Z"  =  "1"  /* Revision of Drain Dev in Uterus & Cervix, Open Approach */
"0UWD01Z"  =  "1"  /* Revision of Radioact Elem in Uterus & Cervix, Open Approach */
"0UWD03Z"  =  "1"  /* Revision of Infusion Dev in Uterus & Cervix, Open Approach */
"0UWD07Z"  =  "1"  /* Revision of Autol Sub in Uterus & Cervix, Open Approach */
"0UWD0CZ"  =  "1"  /* Revision of Extralum Dev in Uterus & Cervix, Open Approach */
"0UWD0DZ"  =  "1"  /* Revision of Intralum Dev in Uterus & Cervix, Open Approach */
"0UWD0HZ"  =  "1"  /* Revision of Contracep Dev in Uterus & Cervix, Open Approach */
"0UWD0JZ"  =  "1"  /* Revision of Synth Sub in Uterus & Cervix, Open Approach */
"0UWD0KZ"  =  "1"  /* Revision of Nonaut Sub in Uterus & Cervix, Open Approach */
"0UWD30Z"  =  "1"  /* Revision of Drain Dev in Uterus & Cervix, Perc Approach */
"0UWD31Z"  =  "1"  /* Revision of Radioact Elem in Uterus & Cervix, Perc Approach */
"0UWD33Z"  =  "1"  /* Revision of Infusion Dev in Uterus & Cervix, Perc Approach */
"0UWD37Z"  =  "1"  /* Revision of Autol Sub in Uterus & Cervix, Perc Approach */
"0UWD3CZ"  =  "1"  /* Revision of Extralum Dev in Uterus & Cervix, Perc Approach */
"0UWD3DZ"  =  "1"  /* Revision of Intralum Dev in Uterus & Cervix, Perc Approach */
"0UWD3HZ"  =  "1"  /* Revision of Contracep Dev in Uterus & Cervix, Perc Approach */
"0UWD3JZ"  =  "1"  /* Revision of Synth Sub in Uterus & Cervix, Perc Approach */
"0UWD3KZ"  =  "1"  /* Revision of Nonaut Sub in Uterus & Cervix, Perc Approach */
"0UWD40Z"  =  "1"  /* Revision of Drain Dev in Uterus & Cervix, Perc Endo Approach */
"0UWD41Z"  =  "1"  /* Revise Radioact Elem in Uterus & Cervix, Perc Endo */
"0UWD43Z"  =  "1"  /* Revise Infusion Dev in Uterus & Cervix, Perc Endo */
"0UWD47Z"  =  "1"  /* Revision of Autol Sub in Uterus & Cervix, Perc Endo Approach */
"0UWD4CZ"  =  "1"  /* Revise Extralum Dev in Uterus & Cervix, Perc Endo */
"0UWD4DZ"  =  "1"  /* Revise Intralum Dev in Uterus & Cervix, Perc Endo */
"0UWD4HZ"  =  "1"  /* Revise Contracep Dev in Uterus & Cervix, Perc Endo */
"0UWD4JZ"  =  "1"  /* Revision of Synth Sub in Uterus & Cervix, Perc Endo Approach */
"0UWD4KZ"  =  "1"  /* Revise of Nonaut Sub in Uterus & Cervix, Perc Endo Approach */
"0UWD70Z"  =  "1"  /* Revision of Drainage Device in Uterus & Cervix, Via Opening */
"0UWD71Z"  =  "1"  /* Revision of Radioact Elem in Uterus & Cervix, Via Opening */
"0UWD73Z"  =  "1"  /* Revision of Infusion Device in Uterus & Cervix, Via Opening */
"0UWD77Z"  =  "1"  /* Revision of Autol Sub in Uterus & Cervix, Via Opening */
"0UWD7CZ"  =  "1"  /* Revision of Extralum Dev in Uterus & Cervix, Via Opening */
"0UWD7DZ"  =  "1"  /* Revision of Intralum Dev in Uterus & Cervix, Via Opening */
"0UWD7HZ"  =  "1"  /* Revision of Contracep Dev in Uterus & Cervix, Via Opening */
"0UWD7JZ"  =  "1"  /* Revision of Synth Sub in Uterus & Cervix, Via Opening */
"0UWD7KZ"  =  "1"  /* Revision of Nonaut Sub in Uterus & Cervix, Via Opening */
"0UWD80Z"  =  "1"  /* Revision of Drainage Device in Uterus and Cervix, Endo */
"0UWD81Z"  =  "1"  /* Revision of Radioactive Element in Uterus and Cervix, Endo */
"0UWD83Z"  =  "1"  /* Revision of Infusion Device in Uterus and Cervix, Endo */
"0UWD87Z"  =  "1"  /* Revision of Autol Sub in Uterus & Cervix, Endo */
"0UWD8CZ"  =  "1"  /* Revision of Extraluminal Device in Uterus and Cervix, Endo */
"0UWD8DZ"  =  "1"  /* Revision of Intraluminal Device in Uterus and Cervix, Endo */
"0UWD8HZ"  =  "1"  /* Revision of Contraceptive Device in Uterus and Cervix, Endo */
"0UWD8JZ"  =  "1"  /* Revision of Synthetic Substitute in Uterus and Cervix, Endo */
"0UWD8KZ"  =  "1"  /* Revision of Nonaut Sub in Uterus & Cervix, Endo */
"0UWH00Z"  =  "1"  /* Revision of Drain Dev in Vagina & Cul-de-sac, Open Approach */
"0UWH01Z"  =  "1"  /* Revise Radioact Elem in Vagina & Cul-de-sac, Open */
"0UWH03Z"  =  "1"  /* Revise of Infusion Dev in Vagina & Cul-de-sac, Open Approach */
"0UWH07Z"  =  "1"  /* Revision of Autol Sub in Vagina & Cul-de-sac, Open Approach */
"0UWH0DZ"  =  "1"  /* Revise of Intralum Dev in Vagina & Cul-de-sac, Open Approach */
"0UWH0JZ"  =  "1"  /* Revision of Synth Sub in Vagina & Cul-de-sac, Open Approach */
"0UWH0KZ"  =  "1"  /* Revision of Nonaut Sub in Vagina & Cul-de-sac, Open Approach */
"0UWH30Z"  =  "1"  /* Revision of Drain Dev in Vagina & Cul-de-sac, Perc Approach */
"0UWH31Z"  =  "1"  /* Revise Radioact Elem in Vagina & Cul-de-sac, Perc */
"0UWH33Z"  =  "1"  /* Revise of Infusion Dev in Vagina & Cul-de-sac, Perc Approach */
"0UWH37Z"  =  "1"  /* Revision of Autol Sub in Vagina & Cul-de-sac, Perc Approach */
"0UWH3DZ"  =  "1"  /* Revise of Intralum Dev in Vagina & Cul-de-sac, Perc Approach */
"0UWH3JZ"  =  "1"  /* Revision of Synth Sub in Vagina & Cul-de-sac, Perc Approach */
"0UWH3KZ"  =  "1"  /* Revision of Nonaut Sub in Vagina & Cul-de-sac, Perc Approach */
"0UWH40Z"  =  "1"  /* Revise Drain Dev in Vagina & Cul-de-sac, Perc Endo */
"0UWH41Z"  =  "1"  /* Revise Radioact Elem in Vagina & Cul-de-sac, Perc Endo */
"0UWH43Z"  =  "1"  /* Revise Infusion Dev in Vagina & Cul-de-sac, Perc Endo */
"0UWH47Z"  =  "1"  /* Revise Autol Sub in Vagina & Cul-de-sac, Perc Endo */
"0UWH4DZ"  =  "1"  /* Revise Intralum Dev in Vagina & Cul-de-sac, Perc Endo */
"0UWH4JZ"  =  "1"  /* Revise Synth Sub in Vagina & Cul-de-sac, Perc Endo */
"0UWH4KZ"  =  "1"  /* Revise Nonaut Sub in Vagina & Cul-de-sac, Perc Endo */
"0UWH70Z"  =  "1"  /* Revision of Drain Dev in Vagina & Cul-de-sac, Via Opening */
"0UWH71Z"  =  "1"  /* Revise of Radioact Elem in Vagina & Cul-de-sac, Via Opening */
"0UWH73Z"  =  "1"  /* Revision of Infusion Dev in Vagina & Cul-de-sac, Via Opening */
"0UWH77Z"  =  "1"  /* Revision of Autol Sub in Vagina & Cul-de-sac, Via Opening */
"0UWH7DZ"  =  "1"  /* Revision of Intralum Dev in Vagina & Cul-de-sac, Via Opening */
"0UWH7JZ"  =  "1"  /* Revision of Synth Sub in Vagina & Cul-de-sac, Via Opening */
"0UWH7KZ"  =  "1"  /* Revision of Nonaut Sub in Vagina & Cul-de-sac, Via Opening */
"0UWH80Z"  =  "1"  /* Revision of Drainage Device in Vagina and Cul-de-sac, Endo */
"0UWH81Z"  =  "1"  /* Revision of Radioactive Element in Vagina & Cul-de-sac, Endo */
"0UWH83Z"  =  "1"  /* Revision of Infusion Device in Vagina and Cul-de-sac, Endo */
"0UWH87Z"  =  "1"  /* Revision of Autol Sub in Vagina & Cul-de-sac, Endo */
"0UWH8DZ"  =  "1"  /* Revision of Intraluminal Device in Vagina & Cul-de-sac, Endo */
"0UWH8JZ"  =  "1"  /* Revision of Synth Sub in Vagina & Cul-de-sac, Endo */
"0UWH8KZ"  =  "1"  /* Revision of Nonaut Sub in Vagina & Cul-de-sac, Endo */
"0UWM00Z"  =  "1"  /* Revision of Drainage Device in Vulva, Open Approach */
"0UWM07Z"  =  "1"  /* Revision of Autol Sub in Vulva, Open Approach */
"0UWM0JZ"  =  "1"  /* Revision of Synthetic Substitute in Vulva, Open Approach */
"0UWM0KZ"  =  "1"  /* Revision of Nonaut Sub in Vulva, Open Approach */
"0UY00Z0"  =  "1"  /* Transplantation of Right Ovary, Allogeneic, Open Approach */
"0UY00Z1"  =  "1"  /* Transplantation of Right Ovary, Syngeneic, Open Approach */
"0UY00Z2"  =  "1"  /* Transplantation of Right Ovary, Zooplastic, Open Approach */
"0UY10Z0"  =  "1"  /* Transplantation of Left Ovary, Allogeneic, Open Approach */
"0UY10Z1"  =  "1"  /* Transplantation of Left Ovary, Syngeneic, Open Approach */
"0UY10Z2"  =  "1"  /* Transplantation of Left Ovary, Zooplastic, Open Approach */
"0V1N07J"  =  "1"  /* Bypass R Vas Deferens to R Epidid w Autol Sub, Open */
"0V1N07K"  =  "1"  /* Bypass R Vas Deferens to L Epidid w Autol Sub, Open */
"0V1N07N"  =  "1"  /* Bypass R Vas Deferens to R Vas Def w Autol Sub, Open */
"0V1N07P"  =  "1"  /* Bypass R Vas Deferens to L Vas Def w Autol Sub, Open */
"0V1N0JJ"  =  "1"  /* Bypass R Vas Deferens to R Epidid w Synth Sub, Open */
"0V1N0JK"  =  "1"  /* Bypass R Vas Deferens to L Epidid w Synth Sub, Open */
"0V1N0JN"  =  "1"  /* Bypass R Vas Deferens to R Vas Def w Synth Sub, Open */
"0V1N0JP"  =  "1"  /* Bypass R Vas Deferens to L Vas Def w Synth Sub, Open */
"0V1N0KJ"  =  "1"  /* Bypass R Vas Deferens to R Epidid w Nonaut Sub, Open */
"0V1N0KK"  =  "1"  /* Bypass R Vas Deferens to L Epidid w Nonaut Sub, Open */
"0V1N0KN"  =  "1"  /* Bypass R Vas Deferens to R Vas Def w Nonaut Sub, Open */
"0V1N0KP"  =  "1"  /* Bypass R Vas Deferens to L Vas Def w Nonaut Sub, Open */
"0V1N0ZJ"  =  "1"  /* Bypass Right Vas Deferens to Right Epididymis, Open Approach */
"0V1N0ZK"  =  "1"  /* Bypass Right Vas Deferens to Left Epididymis, Open Approach */
"0V1N0ZN"  =  "1"  /* Bypass Right Vas Deferens to R Vas Def, Open Approach */
"0V1N0ZP"  =  "1"  /* Bypass Right Vas Deferens to L Vas Def, Open Approach */
"0V1N47J"  =  "1"  /* Bypass R Vas Deferens to R Epidid w Autol Sub, Perc Endo */
"0V1N47K"  =  "1"  /* Bypass R Vas Deferens to L Epidid w Autol Sub, Perc Endo */
"0V1N47N"  =  "1"  /* Bypass R Vas Deferens to R Vas Def w Autol Sub, Perc Endo */
"0V1N47P"  =  "1"  /* Bypass R Vas Deferens to L Vas Def w Autol Sub, Perc Endo */
"0V1N4JJ"  =  "1"  /* Bypass R Vas Deferens to R Epidid w Synth Sub, Perc Endo */
"0V1N4JK"  =  "1"  /* Bypass R Vas Deferens to L Epidid w Synth Sub, Perc Endo */
"0V1N4JN"  =  "1"  /* Bypass R Vas Deferens to R Vas Def w Synth Sub, Perc Endo */
"0V1N4JP"  =  "1"  /* Bypass R Vas Deferens to L Vas Def w Synth Sub, Perc Endo */
"0V1N4KJ"  =  "1"  /* Bypass R Vas Deferens to R Epidid w Nonaut Sub, Perc Endo */
"0V1N4KK"  =  "1"  /* Bypass R Vas Deferens to L Epidid w Nonaut Sub, Perc Endo */
"0V1N4KN"  =  "1"  /* Bypass R Vas Deferens to R Vas Def w Nonaut Sub, Perc Endo */
"0V1N4KP"  =  "1"  /* Bypass R Vas Deferens to L Vas Def w Nonaut Sub, Perc Endo */
"0V1N4ZJ"  =  "1"  /* Bypass Right Vas Deferens to R Epidid, Perc Endo Approach */
"0V1N4ZK"  =  "1"  /* Bypass Right Vas Deferens to L Epidid, Perc Endo Approach */
"0V1N4ZN"  =  "1"  /* Bypass Right Vas Deferens to R Vas Def, Perc Endo Approach */
"0V1N4ZP"  =  "1"  /* Bypass Right Vas Deferens to L Vas Def, Perc Endo Approach */
"0V1P07J"  =  "1"  /* Bypass L Vas Deferens to R Epidid w Autol Sub, Open */
"0V1P07K"  =  "1"  /* Bypass L Vas Deferens to L Epidid w Autol Sub, Open */
"0V1P07N"  =  "1"  /* Bypass L Vas Deferens to R Vas Def w Autol Sub, Open */
"0V1P07P"  =  "1"  /* Bypass L Vas Deferens to L Vas Def w Autol Sub, Open */
"0V1P0JJ"  =  "1"  /* Bypass L Vas Deferens to R Epidid w Synth Sub, Open */
"0V1P0JK"  =  "1"  /* Bypass L Vas Deferens to L Epidid w Synth Sub, Open */
"0V1P0JN"  =  "1"  /* Bypass L Vas Deferens to R Vas Def w Synth Sub, Open */
"0V1P0JP"  =  "1"  /* Bypass L Vas Deferens to L Vas Def w Synth Sub, Open */
"0V1P0KJ"  =  "1"  /* Bypass L Vas Deferens to R Epidid w Nonaut Sub, Open */
"0V1P0KK"  =  "1"  /* Bypass L Vas Deferens to L Epidid w Nonaut Sub, Open */
"0V1P0KN"  =  "1"  /* Bypass L Vas Deferens to R Vas Def w Nonaut Sub, Open */
"0V1P0KP"  =  "1"  /* Bypass L Vas Deferens to L Vas Def w Nonaut Sub, Open */
"0V1P0ZJ"  =  "1"  /* Bypass Left Vas Deferens to Right Epididymis, Open Approach */
"0V1P0ZK"  =  "1"  /* Bypass Left Vas Deferens to Left Epididymis, Open Approach */
"0V1P0ZN"  =  "1"  /* Bypass Left Vas Deferens to R Vas Def, Open Approach */
"0V1P0ZP"  =  "1"  /* Bypass Left Vas Deferens to Left Vas Deferens, Open Approach */
"0V1P47J"  =  "1"  /* Bypass L Vas Deferens to R Epidid w Autol Sub, Perc Endo */
"0V1P47K"  =  "1"  /* Bypass L Vas Deferens to L Epidid w Autol Sub, Perc Endo */
"0V1P47N"  =  "1"  /* Bypass L Vas Deferens to R Vas Def w Autol Sub, Perc Endo */
"0V1P47P"  =  "1"  /* Bypass L Vas Deferens to L Vas Def w Autol Sub, Perc Endo */
"0V1P4JJ"  =  "1"  /* Bypass L Vas Deferens to R Epidid w Synth Sub, Perc Endo */
"0V1P4JK"  =  "1"  /* Bypass L Vas Deferens to L Epidid w Synth Sub, Perc Endo */
"0V1P4JN"  =  "1"  /* Bypass L Vas Deferens to R Vas Def w Synth Sub, Perc Endo */
"0V1P4JP"  =  "1"  /* Bypass L Vas Deferens to L Vas Def w Synth Sub, Perc Endo */
"0V1P4KJ"  =  "1"  /* Bypass L Vas Deferens to R Epidid w Nonaut Sub, Perc Endo */
"0V1P4KK"  =  "1"  /* Bypass L Vas Deferens to L Epidid w Nonaut Sub, Perc Endo */
"0V1P4KN"  =  "1"  /* Bypass L Vas Deferens to R Vas Def w Nonaut Sub, Perc Endo */
"0V1P4KP"  =  "1"  /* Bypass L Vas Deferens to L Vas Def w Nonaut Sub, Perc Endo */
"0V1P4ZJ"  =  "1"  /* Bypass Left Vas Deferens to R Epidid, Perc Endo Approach */
"0V1P4ZK"  =  "1"  /* Bypass Left Vas Deferens to L Epidid, Perc Endo Approach */
"0V1P4ZN"  =  "1"  /* Bypass Left Vas Deferens to R Vas Def, Perc Endo Approach */
"0V1P4ZP"  =  "1"  /* Bypass Left Vas Deferens to L Vas Def, Perc Endo Approach */
"0V1Q07J"  =  "1"  /* Bypass Bi Vas Deferens to R Epidid w Autol Sub, Open */
"0V1Q07K"  =  "1"  /* Bypass Bi Vas Deferens to L Epidid w Autol Sub, Open */
"0V1Q07N"  =  "1"  /* Bypass Bi Vas Deferens to R Vas Def w Autol Sub, Open */
"0V1Q07P"  =  "1"  /* Bypass Bi Vas Deferens to L Vas Def w Autol Sub, Open */
"0V1Q0JJ"  =  "1"  /* Bypass Bi Vas Deferens to R Epidid w Synth Sub, Open */
"0V1Q0JK"  =  "1"  /* Bypass Bi Vas Deferens to L Epidid w Synth Sub, Open */
"0V1Q0JN"  =  "1"  /* Bypass Bi Vas Deferens to R Vas Def w Synth Sub, Open */
"0V1Q0JP"  =  "1"  /* Bypass Bi Vas Deferens to L Vas Def w Synth Sub, Open */
"0V1Q0KJ"  =  "1"  /* Bypass Bi Vas Deferens to R Epidid w Nonaut Sub, Open */
"0V1Q0KK"  =  "1"  /* Bypass Bi Vas Deferens to L Epidid w Nonaut Sub, Open */
"0V1Q0KN"  =  "1"  /* Bypass Bi Vas Deferens to R Vas Def w Nonaut Sub, Open */
"0V1Q0KP"  =  "1"  /* Bypass Bi Vas Deferens to L Vas Def w Nonaut Sub, Open */
"0V1Q0ZJ"  =  "1"  /* Bypass Bilateral Vas Deferens to R Epidid, Open Approach */
"0V1Q0ZK"  =  "1"  /* Bypass Bilateral Vas Deferens to L Epidid, Open Approach */
"0V1Q0ZN"  =  "1"  /* Bypass Bilateral Vas Deferens to R Vas Def, Open Approach */
"0V1Q0ZP"  =  "1"  /* Bypass Bilateral Vas Deferens to L Vas Def, Open Approach */
"0V1Q47J"  =  "1"  /* Bypass Bi Vas Deferens to R Epidid w Autol Sub, Perc Endo */
"0V1Q47K"  =  "1"  /* Bypass Bi Vas Deferens to L Epidid w Autol Sub, Perc Endo */
"0V1Q47N"  =  "1"  /* Bypass Bi Vas Deferens to R Vas Def w Autol Sub, Perc Endo */
"0V1Q47P"  =  "1"  /* Bypass Bi Vas Deferens to L Vas Def w Autol Sub, Perc Endo */
"0V1Q4JJ"  =  "1"  /* Bypass Bi Vas Deferens to R Epidid w Synth Sub, Perc Endo */
"0V1Q4JK"  =  "1"  /* Bypass Bi Vas Deferens to L Epidid w Synth Sub, Perc Endo */
"0V1Q4JN"  =  "1"  /* Bypass Bi Vas Deferens to R Vas Def w Synth Sub, Perc Endo */
"0V1Q4JP"  =  "1"  /* Bypass Bi Vas Deferens to L Vas Def w Synth Sub, Perc Endo */
"0V1Q4KJ"  =  "1"  /* Bypass Bi Vas Deferens to R Epidid w Nonaut Sub, Perc Endo */
"0V1Q4KK"  =  "1"  /* Bypass Bi Vas Deferens to L Epidid w Nonaut Sub, Perc Endo */
"0V1Q4KN"  =  "1"  /* Bypass Bi Vas Deferens to R Vas Def w Nonaut Sub, Perc Endo */
"0V1Q4KP"  =  "1"  /* Bypass Bi Vas Deferens to L Vas Def w Nonaut Sub, Perc Endo */
"0V1Q4ZJ"  =  "1"  /* Bypass Bi Vas Deferens to R Epidid, Perc Endo Approach */
"0V1Q4ZK"  =  "1"  /* Bypass Bi Vas Deferens to L Epidid, Perc Endo Approach */
"0V1Q4ZN"  =  "1"  /* Bypass Bi Vas Deferens to R Vas Def, Perc Endo Approach */
"0V1Q4ZP"  =  "1"  /* Bypass Bi Vas Deferens to L Vas Def, Perc Endo Approach */
"0V500ZZ"  =  "1"  /* Destruction of Prostate, Open Approach */
"0V503ZZ"  =  "1"  /* Destruction of Prostate, Percutaneous Approach */
"0V504ZZ"  =  "1"  /* Destruction of Prostate, Percutaneous Endoscopic Approach */
"0V507ZZ"  =  "1"  /* Destruction of Prostate, Via Natural or Artificial Opening */
"0V508ZZ"  =  "1"  /* Destruction of Prostate, Endo */
"0V510ZZ"  =  "1"  /* Destruction of Right Seminal Vesicle, Open Approach */
"0V513ZZ"  =  "1"  /* Destruction of Right Seminal Vesicle, Percutaneous Approach */
"0V514ZZ"  =  "1"  /* Destruction of Right Seminal Vesicle, Perc Endo Approach */
"0V520ZZ"  =  "1"  /* Destruction of Left Seminal Vesicle, Open Approach */
"0V523ZZ"  =  "1"  /* Destruction of Left Seminal Vesicle, Percutaneous Approach */
"0V524ZZ"  =  "1"  /* Destruction of Left Seminal Vesicle, Perc Endo Approach */
"0V530ZZ"  =  "1"  /* Destruction of Bilateral Seminal Vesicles, Open Approach */
"0V533ZZ"  =  "1"  /* Destruction of Bilateral Seminal Vesicles, Perc Approach */
"0V534ZZ"  =  "1"  /* Destruction of Bi Seminal Vesicle, Perc Endo Approach */
"0V560ZZ"  =  "1"  /* Destruction of Right Tunica Vaginalis, Open Approach */
"0V563ZZ"  =  "1"  /* Destruction of Right Tunica Vaginalis, Percutaneous Approach */
"0V564ZZ"  =  "1"  /* Destruction of Right Tunica Vaginalis, Perc Endo Approach */
"0V570ZZ"  =  "1"  /* Destruction of Left Tunica Vaginalis, Open Approach */
"0V573ZZ"  =  "1"  /* Destruction of Left Tunica Vaginalis, Percutaneous Approach */
"0V574ZZ"  =  "1"  /* Destruction of Left Tunica Vaginalis, Perc Endo Approach */
"0V590ZZ"  =  "1"  /* Destruction of Right Testis, Open Approach */
"0V593ZZ"  =  "1"  /* Destruction of Right Testis, Percutaneous Approach */
"0V594ZZ"  =  "1"  /* Destruction of Right Testis, Perc Endo Approach */
"0V5B0ZZ"  =  "1"  /* Destruction of Left Testis, Open Approach */
"0V5B3ZZ"  =  "1"  /* Destruction of Left Testis, Percutaneous Approach */
"0V5B4ZZ"  =  "1"  /* Destruction of Left Testis, Percutaneous Endoscopic Approach */
"0V5C0ZZ"  =  "1"  /* Destruction of Bilateral Testes, Open Approach */
"0V5C3ZZ"  =  "1"  /* Destruction of Bilateral Testes, Percutaneous Approach */
"0V5C4ZZ"  =  "1"  /* Destruction of Bilateral Testes, Perc Endo Approach */
"0V5F0ZZ"  =  "1"  /* Destruction of Right Spermatic Cord, Open Approach */
"0V5F3ZZ"  =  "1"  /* Destruction of Right Spermatic Cord, Percutaneous Approach */
"0V5F4ZZ"  =  "1"  /* Destruction of Right Spermatic Cord, Perc Endo Approach */
"0V5G0ZZ"  =  "1"  /* Destruction of Left Spermatic Cord, Open Approach */
"0V5G3ZZ"  =  "1"  /* Destruction of Left Spermatic Cord, Percutaneous Approach */
"0V5G4ZZ"  =  "1"  /* Destruction of Left Spermatic Cord, Perc Endo Approach */
"0V5H0ZZ"  =  "1"  /* Destruction of Bilateral Spermatic Cords, Open Approach */
"0V5H3ZZ"  =  "1"  /* Destruction of Bilateral Spermatic Cords, Perc Approach */
"0V5H4ZZ"  =  "1"  /* Destruction of Bilateral Spermatic Cords, Perc Endo Approach */
"0V5J0ZZ"  =  "1"  /* Destruction of Right Epididymis, Open Approach */
"0V5J3ZZ"  =  "1"  /* Destruction of Right Epididymis, Percutaneous Approach */
"0V5J4ZZ"  =  "1"  /* Destruction of Right Epididymis, Perc Endo Approach */
"0V5K0ZZ"  =  "1"  /* Destruction of Left Epididymis, Open Approach */
"0V5K3ZZ"  =  "1"  /* Destruction of Left Epididymis, Percutaneous Approach */
"0V5K4ZZ"  =  "1"  /* Destruction of Left Epididymis, Perc Endo Approach */
"0V5L0ZZ"  =  "1"  /* Destruction of Bilateral Epididymis, Open Approach */
"0V5L3ZZ"  =  "1"  /* Destruction of Bilateral Epididymis, Percutaneous Approach */
"0V5L4ZZ"  =  "1"  /* Destruction of Bilateral Epididymis, Perc Endo Approach */
"0V5S0ZZ"  =  "1"  /* Destruction of Penis, Open Approach */
"0V5S3ZZ"  =  "1"  /* Destruction of Penis, Percutaneous Approach */
"0V5S4ZZ"  =  "1"  /* Destruction of Penis, Percutaneous Endoscopic Approach */
"0V5SXZZ"  =  "1"  /* Destruction of Penis, External Approach */
"0V5T0ZZ"  =  "1"  /* Destruction of Prepuce, Open Approach */
"0V5T3ZZ"  =  "1"  /* Destruction of Prepuce, Percutaneous Approach */
"0V5T4ZZ"  =  "1"  /* Destruction of Prepuce, Percutaneous Endoscopic Approach */
"0V5TXZZ"  =  "1"  /* Destruction of Prepuce, External Approach */
"0V7N0DZ"  =  "1"  /* Dilation of R Vas Deferens with Intralum Dev, Open Approach */
"0V7N0ZZ"  =  "1"  /* Dilation of Right Vas Deferens, Open Approach */
"0V7N3DZ"  =  "1"  /* Dilation of R Vas Deferens with Intralum Dev, Perc Approach */
"0V7N3ZZ"  =  "1"  /* Dilation of Right Vas Deferens, Percutaneous Approach */
"0V7N4DZ"  =  "1"  /* Dilate R Vas Deferens w Intralum Dev, Perc Endo */
"0V7N4ZZ"  =  "1"  /* Dilation of Right Vas Deferens, Perc Endo Approach */
"0V7P0DZ"  =  "1"  /* Dilation of L Vas Deferens with Intralum Dev, Open Approach */
"0V7P0ZZ"  =  "1"  /* Dilation of Left Vas Deferens, Open Approach */
"0V7P3DZ"  =  "1"  /* Dilation of L Vas Deferens with Intralum Dev, Perc Approach */
"0V7P3ZZ"  =  "1"  /* Dilation of Left Vas Deferens, Percutaneous Approach */
"0V7P4DZ"  =  "1"  /* Dilate L Vas Deferens w Intralum Dev, Perc Endo */
"0V7P4ZZ"  =  "1"  /* Dilation of Left Vas Deferens, Perc Endo Approach */
"0V7Q0DZ"  =  "1"  /* Dilation of Bi Vas Deferens with Intralum Dev, Open Approach */
"0V7Q0ZZ"  =  "1"  /* Dilation of Bilateral Vas Deferens, Open Approach */
"0V7Q3DZ"  =  "1"  /* Dilation of Bi Vas Deferens with Intralum Dev, Perc Approach */
"0V7Q3ZZ"  =  "1"  /* Dilation of Bilateral Vas Deferens, Percutaneous Approach */
"0V7Q4DZ"  =  "1"  /* Dilate Bi Vas Deferens w Intralum Dev, Perc Endo */
"0V7Q4ZZ"  =  "1"  /* Dilation of Bilateral Vas Deferens, Perc Endo Approach */
"0V9000Z"  =  "1"  /* Drainage of Prostate with Drainage Device, Open Approach */
"0V900ZX"  =  "1"  /* Drainage of Prostate, Open Approach, Diagnostic */
"0V900ZZ"  =  "1"  /* Drainage of Prostate, Open Approach */
"0V9070Z"  =  "1"  /* Drainage of Prostate with Drainage Device, Via Opening */
"0V907ZZ"  =  "1"  /* Drainage of Prostate, Via Natural or Artificial Opening */
"0V9080Z"  =  "1"  /* Drainage of Prostate with Drainage Device, Endo */
"0V908ZZ"  =  "1"  /* Drainage of Prostate, Endo */
"0V9100Z"  =  "1"  /* Drainage of R Seminal Vesicle with Drain Dev, Open Approach */
"0V910ZX"  =  "1"  /* Drainage of Right Seminal Vesicle, Open Approach, Diagnostic */
"0V910ZZ"  =  "1"  /* Drainage of Right Seminal Vesicle, Open Approach */
"0V9200Z"  =  "1"  /* Drainage of L Seminal Vesicle with Drain Dev, Open Approach */
"0V920ZX"  =  "1"  /* Drainage of Left Seminal Vesicle, Open Approach, Diagnostic */
"0V920ZZ"  =  "1"  /* Drainage of Left Seminal Vesicle, Open Approach */
"0V9300Z"  =  "1"  /* Drainage of Bi Seminal Vesicle with Drain Dev, Open Approach */
"0V930ZX"  =  "1"  /* Drainage of Bilateral Seminal Vesicles, Open Approach, Diagn */
"0V930ZZ"  =  "1"  /* Drainage of Bilateral Seminal Vesicles, Open Approach */
"0V9900Z"  =  "1"  /* Drainage of Right Testis with Drainage Device, Open Approach */
"0V990ZX"  =  "1"  /* Drainage of Right Testis, Open Approach, Diagnostic */
"0V990ZZ"  =  "1"  /* Drainage of Right Testis, Open Approach */
"0V9B00Z"  =  "1"  /* Drainage of Left Testis with Drainage Device, Open Approach */
"0V9B0ZX"  =  "1"  /* Drainage of Left Testis, Open Approach, Diagnostic */
"0V9B0ZZ"  =  "1"  /* Drainage of Left Testis, Open Approach */
"0V9C00Z"  =  "1"  /* Drainage of Bilateral Testes with Drain Dev, Open Approach */
"0V9C0ZX"  =  "1"  /* Drainage of Bilateral Testes, Open Approach, Diagnostic */
"0V9C0ZZ"  =  "1"  /* Drainage of Bilateral Testes, Open Approach */
"0V9J00Z"  =  "1"  /* Drainage of Right Epididymis with Drain Dev, Open Approach */
"0V9J0ZZ"  =  "1"  /* Drainage of Right Epididymis, Open Approach */
"0V9J30Z"  =  "1"  /* Drainage of Right Epididymis with Drain Dev, Perc Approach */
"0V9J3ZZ"  =  "1"  /* Drainage of Right Epididymis, Percutaneous Approach */
"0V9J40Z"  =  "1"  /* Drainage of R Epididymis with Drain Dev, Perc Endo Approach */
"0V9J4ZZ"  =  "1"  /* Drainage of Right Epididymis, Perc Endo Approach */
"0V9K00Z"  =  "1"  /* Drainage of Left Epididymis with Drain Dev, Open Approach */
"0V9K0ZZ"  =  "1"  /* Drainage of Left Epididymis, Open Approach */
"0V9K30Z"  =  "1"  /* Drainage of Left Epididymis with Drain Dev, Perc Approach */
"0V9K3ZZ"  =  "1"  /* Drainage of Left Epididymis, Percutaneous Approach */
"0V9K40Z"  =  "1"  /* Drainage of L Epididymis with Drain Dev, Perc Endo Approach */
"0V9K4ZZ"  =  "1"  /* Drainage of Left Epididymis, Perc Endo Approach */
"0V9L00Z"  =  "1"  /* Drainage of B Epididymis with Drain Dev, Open Approach */
"0V9L0ZZ"  =  "1"  /* Drainage of Bilateral Epididymis, Open Approach */
"0V9L30Z"  =  "1"  /* Drainage of B Epididymis with Drain Dev, Perc Approach */
"0V9L3ZZ"  =  "1"  /* Drainage of Bilateral Epididymis, Percutaneous Approach */
"0V9L40Z"  =  "1"  /* Drainage of B Epididymis with Drain Dev, Perc Endo Approach */
"0V9L4ZZ"  =  "1"  /* Drainage of Bilateral Epididymis, Perc Endo Approach */
"0V9S00Z"  =  "1"  /* Drainage of Penis with Drainage Device, Open Approach */
"0V9S0ZX"  =  "1"  /* Drainage of Penis, Open Approach, Diagnostic */
"0V9S0ZZ"  =  "1"  /* Drainage of Penis, Open Approach */
"0V9S30Z"  =  "1"  /* Drainage of Penis with Drainage Device, Perc Approach */
"0V9S3ZX"  =  "1"  /* Drainage of Penis, Percutaneous Approach, Diagnostic */
"0V9S3ZZ"  =  "1"  /* Drainage of Penis, Percutaneous Approach */
"0V9S40Z"  =  "1"  /* Drainage of Penis with Drainage Device, Perc Endo Approach */
"0V9S4ZX"  =  "1"  /* Drainage of Penis, Percutaneous Endoscopic Approach, Diagn */
"0V9S4ZZ"  =  "1"  /* Drainage of Penis, Percutaneous Endoscopic Approach */
"0V9SX0Z"  =  "1"  /* Drainage of Penis with Drainage Device, External Approach */
"0V9SXZX"  =  "1"  /* Drainage of Penis, External Approach, Diagnostic */
"0V9SXZZ"  =  "1"  /* Drainage of Penis, External Approach */
"0V9T00Z"  =  "1"  /* Drainage of Prepuce with Drainage Device, Open Approach */
"0V9T0ZX"  =  "1"  /* Drainage of Prepuce, Open Approach, Diagnostic */
"0V9T0ZZ"  =  "1"  /* Drainage of Prepuce, Open Approach */
"0V9T30Z"  =  "1"  /* Drainage of Prepuce with Drainage Device, Perc Approach */
"0V9T3ZX"  =  "1"  /* Drainage of Prepuce, Percutaneous Approach, Diagnostic */
"0V9T3ZZ"  =  "1"  /* Drainage of Prepuce, Percutaneous Approach */
"0V9T40Z"  =  "1"  /* Drainage of Prepuce with Drainage Device, Perc Endo Approach */
"0V9T4ZX"  =  "1"  /* Drainage of Prepuce, Percutaneous Endoscopic Approach, Diagn */
"0V9T4ZZ"  =  "1"  /* Drainage of Prepuce, Percutaneous Endoscopic Approach */
"0V9TX0Z"  =  "1"  /* Drainage of Prepuce with Drainage Device, External Approach */
"0V9TXZX"  =  "1"  /* Drainage of Prepuce, External Approach, Diagnostic */
"0V9TXZZ"  =  "1"  /* Drainage of Prepuce, External Approach */
"0VB00ZX"  =  "1"  /* Excision of Prostate, Open Approach, Diagnostic */
"0VB00ZZ"  =  "1"  /* Excision of Prostate, Open Approach */
"0VB03ZZ"  =  "1"  /* Excision of Prostate, Percutaneous Approach */
"0VB04ZZ"  =  "1"  /* Excision of Prostate, Percutaneous Endoscopic Approach */
"0VB07ZZ"  =  "1"  /* Excision of Prostate, Via Natural or Artificial Opening */
"0VB08ZZ"  =  "1"  /* Excision of Prostate, Endo */
"0VB10ZX"  =  "1"  /* Excision of Right Seminal Vesicle, Open Approach, Diagnostic */
"0VB10ZZ"  =  "1"  /* Excision of Right Seminal Vesicle, Open Approach */
"0VB13ZZ"  =  "1"  /* Excision of Right Seminal Vesicle, Percutaneous Approach */
"0VB14ZZ"  =  "1"  /* Excision of Right Seminal Vesicle, Perc Endo Approach */
"0VB20ZX"  =  "1"  /* Excision of Left Seminal Vesicle, Open Approach, Diagnostic */
"0VB20ZZ"  =  "1"  /* Excision of Left Seminal Vesicle, Open Approach */
"0VB23ZZ"  =  "1"  /* Excision of Left Seminal Vesicle, Percutaneous Approach */
"0VB24ZZ"  =  "1"  /* Excision of Left Seminal Vesicle, Perc Endo Approach */
"0VB30ZX"  =  "1"  /* Excision of Bilateral Seminal Vesicles, Open Approach, Diagn */
"0VB30ZZ"  =  "1"  /* Excision of Bilateral Seminal Vesicles, Open Approach */
"0VB33ZZ"  =  "1"  /* Excision of Bilateral Seminal Vesicles, Perc Approach */
"0VB34ZZ"  =  "1"  /* Excision of Bilateral Seminal Vesicles, Perc Endo Approach */
"0VB60ZZ"  =  "1"  /* Excision of Right Tunica Vaginalis, Open Approach */
"0VB63ZZ"  =  "1"  /* Excision of Right Tunica Vaginalis, Percutaneous Approach */
"0VB64ZZ"  =  "1"  /* Excision of Right Tunica Vaginalis, Perc Endo Approach */
"0VB70ZZ"  =  "1"  /* Excision of Left Tunica Vaginalis, Open Approach */
"0VB73ZZ"  =  "1"  /* Excision of Left Tunica Vaginalis, Percutaneous Approach */
"0VB74ZZ"  =  "1"  /* Excision of Left Tunica Vaginalis, Perc Endo Approach */
"0VB90ZX"  =  "1"  /* Excision of Right Testis, Open Approach, Diagnostic */
"0VB90ZZ"  =  "1"  /* Excision of Right Testis, Open Approach */
"0VB93ZZ"  =  "1"  /* Excision of Right Testis, Percutaneous Approach */
"0VB94ZZ"  =  "1"  /* Excision of Right Testis, Percutaneous Endoscopic Approach */
"0VBB0ZX"  =  "1"  /* Excision of Left Testis, Open Approach, Diagnostic */
"0VBB0ZZ"  =  "1"  /* Excision of Left Testis, Open Approach */
"0VBB3ZZ"  =  "1"  /* Excision of Left Testis, Percutaneous Approach */
"0VBB4ZZ"  =  "1"  /* Excision of Left Testis, Percutaneous Endoscopic Approach */
"0VBC0ZX"  =  "1"  /* Excision of Bilateral Testes, Open Approach, Diagnostic */
"0VBC0ZZ"  =  "1"  /* Excision of Bilateral Testes, Open Approach */
"0VBC3ZZ"  =  "1"  /* Excision of Bilateral Testes, Percutaneous Approach */
"0VBC4ZZ"  =  "1"  /* Excision of Bilateral Testes, Perc Endo Approach */
"0VBF0ZZ"  =  "1"  /* Excision of Right Spermatic Cord, Open Approach */
"0VBF3ZZ"  =  "1"  /* Excision of Right Spermatic Cord, Percutaneous Approach */
"0VBF4ZZ"  =  "1"  /* Excision of Right Spermatic Cord, Perc Endo Approach */
"0VBG0ZZ"  =  "1"  /* Excision of Left Spermatic Cord, Open Approach */
"0VBG3ZZ"  =  "1"  /* Excision of Left Spermatic Cord, Percutaneous Approach */
"0VBG4ZZ"  =  "1"  /* Excision of Left Spermatic Cord, Perc Endo Approach */
"0VBH0ZZ"  =  "1"  /* Excision of Bilateral Spermatic Cords, Open Approach */
"0VBH3ZZ"  =  "1"  /* Excision of Bilateral Spermatic Cords, Percutaneous Approach */
"0VBH4ZZ"  =  "1"  /* Excision of Bilateral Spermatic Cords, Perc Endo Approach */
"0VBJ0ZZ"  =  "1"  /* Excision of Right Epididymis, Open Approach */
"0VBJ3ZZ"  =  "1"  /* Excision of Right Epididymis, Percutaneous Approach */
"0VBJ4ZZ"  =  "1"  /* Excision of Right Epididymis, Perc Endo Approach */
"0VBK0ZZ"  =  "1"  /* Excision of Left Epididymis, Open Approach */
"0VBK3ZZ"  =  "1"  /* Excision of Left Epididymis, Percutaneous Approach */
"0VBK4ZZ"  =  "1"  /* Excision of Left Epididymis, Perc Endo Approach */
"0VBL0ZZ"  =  "1"  /* Excision of Bilateral Epididymis, Open Approach */
"0VBL3ZZ"  =  "1"  /* Excision of Bilateral Epididymis, Percutaneous Approach */
"0VBL4ZZ"  =  "1"  /* Excision of Bilateral Epididymis, Perc Endo Approach */
"0VBS0ZX"  =  "1"  /* Excision of Penis, Open Approach, Diagnostic */
"0VBS0ZZ"  =  "1"  /* Excision of Penis, Open Approach */
"0VBS3ZX"  =  "1"  /* Excision of Penis, Percutaneous Approach, Diagnostic */
"0VBS3ZZ"  =  "1"  /* Excision of Penis, Percutaneous Approach */
"0VBS4ZX"  =  "1"  /* Excision of Penis, Percutaneous Endoscopic Approach, Diagn */
"0VBS4ZZ"  =  "1"  /* Excision of Penis, Percutaneous Endoscopic Approach */
"0VBSXZX"  =  "1"  /* Excision of Penis, External Approach, Diagnostic */
"0VBSXZZ"  =  "1"  /* Excision of Penis, External Approach */
"0VBT0ZX"  =  "1"  /* Excision of Prepuce, Open Approach, Diagnostic */
"0VBT0ZZ"  =  "1"  /* Excision of Prepuce, Open Approach */
"0VBT3ZX"  =  "1"  /* Excision of Prepuce, Percutaneous Approach, Diagnostic */
"0VBT3ZZ"  =  "1"  /* Excision of Prepuce, Percutaneous Approach */
"0VBT4ZX"  =  "1"  /* Excision of Prepuce, Percutaneous Endoscopic Approach, Diagn */
"0VBT4ZZ"  =  "1"  /* Excision of Prepuce, Percutaneous Endoscopic Approach */
"0VBTXZX"  =  "1"  /* Excision of Prepuce, External Approach, Diagnostic */
"0VBTXZZ"  =  "1"  /* Excision of Prepuce, External Approach */
"0VC00ZZ"  =  "1"  /* Extirpation of Matter from Prostate, Open Approach */
"0VC03ZZ"  =  "1"  /* Extirpation of Matter from Prostate, Percutaneous Approach */
"0VC04ZZ"  =  "1"  /* Extirpation of Matter from Prostate, Perc Endo Approach */
"0VC07ZZ"  =  "1"  /* Extirpation of Matter from Prostate, Via Opening */
"0VC08ZZ"  =  "1"  /* Extirpation of Matter from Prostate, Endo */
"0VC10ZZ"  =  "1"  /* Extirpation of Matter from R Seminal Vesicle, Open Approach */
"0VC13ZZ"  =  "1"  /* Extirpation of Matter from R Seminal Vesicle, Perc Approach */
"0VC14ZZ"  =  "1"  /* Extirpate matter from R Seminal Vesicle, Perc Endo */
"0VC20ZZ"  =  "1"  /* Extirpation of Matter from L Seminal Vesicle, Open Approach */
"0VC23ZZ"  =  "1"  /* Extirpation of Matter from L Seminal Vesicle, Perc Approach */
"0VC24ZZ"  =  "1"  /* Extirpate matter from L Seminal Vesicle, Perc Endo */
"0VC30ZZ"  =  "1"  /* Extirpation of Matter from Bi Seminal Vesicle, Open Approach */
"0VC33ZZ"  =  "1"  /* Extirpation of Matter from Bi Seminal Vesicle, Perc Approach */
"0VC34ZZ"  =  "1"  /* Extirpate matter from Bi Seminal Vesicle, Perc Endo */
"0VC90ZZ"  =  "1"  /* Extirpation of Matter from Right Testis, Open Approach */
"0VC93ZZ"  =  "1"  /* Extirpation of Matter from Right Testis, Perc Approach */
"0VC94ZZ"  =  "1"  /* Extirpation of Matter from Right Testis, Perc Endo Approach */
"0VCB0ZZ"  =  "1"  /* Extirpation of Matter from Left Testis, Open Approach */
"0VCB3ZZ"  =  "1"  /* Extirpation of Matter from Left Testis, Perc Approach */
"0VCB4ZZ"  =  "1"  /* Extirpation of Matter from Left Testis, Perc Endo Approach */
"0VCC0ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Testes, Open Approach */
"0VCC3ZZ"  =  "1"  /* Extirpation of Matter from Bilateral Testes, Perc Approach */
"0VCC4ZZ"  =  "1"  /* Extirpation of Matter from Bi Testis, Perc Endo Approach */
"0VCF0ZZ"  =  "1"  /* Extirpation of Matter from R Sperm Cord, Open Approach */
"0VCF3ZZ"  =  "1"  /* Extirpation of Matter from R Sperm Cord, Perc Approach */
"0VCF4ZZ"  =  "1"  /* Extirpation of Matter from R Sperm Cord, Perc Endo Approach */
"0VCG0ZZ"  =  "1"  /* Extirpation of Matter from L Sperm Cord, Open Approach */
"0VCG3ZZ"  =  "1"  /* Extirpation of Matter from L Sperm Cord, Perc Approach */
"0VCG4ZZ"  =  "1"  /* Extirpation of Matter from L Sperm Cord, Perc Endo Approach */
"0VCH0ZZ"  =  "1"  /* Extirpation of Matter from Bi Sperm Cord, Open Approach */
"0VCH3ZZ"  =  "1"  /* Extirpation of Matter from Bi Sperm Cord, Perc Approach */
"0VCH4ZZ"  =  "1"  /* Extirpation of Matter from Bi Sperm Cord, Perc Endo Approach */
"0VCJ0ZZ"  =  "1"  /* Extirpation of Matter from Right Epididymis, Open Approach */
"0VCJ3ZZ"  =  "1"  /* Extirpation of Matter from Right Epididymis, Perc Approach */
"0VCJ4ZZ"  =  "1"  /* Extirpation of Matter from R Epididymis, Perc Endo Approach */
"0VCK0ZZ"  =  "1"  /* Extirpation of Matter from Left Epididymis, Open Approach */
"0VCK3ZZ"  =  "1"  /* Extirpation of Matter from Left Epididymis, Perc Approach */
"0VCK4ZZ"  =  "1"  /* Extirpation of Matter from L Epididymis, Perc Endo Approach */
"0VCL0ZZ"  =  "1"  /* Extirpation of Matter from B Epididymis, Open Approach */
"0VCL3ZZ"  =  "1"  /* Extirpation of Matter from B Epididymis, Perc Approach */
"0VCL4ZZ"  =  "1"  /* Extirpation of Matter from B Epididymis, Perc Endo Approach */
"0VCS0ZZ"  =  "1"  /* Extirpation of Matter from Penis, Open Approach */
"0VCS3ZZ"  =  "1"  /* Extirpation of Matter from Penis, Percutaneous Approach */
"0VCS4ZZ"  =  "1"  /* Extirpation of Matter from Penis, Perc Endo Approach */
"0VCT0ZZ"  =  "1"  /* Extirpation of Matter from Prepuce, Open Approach */
"0VCT3ZZ"  =  "1"  /* Extirpation of Matter from Prepuce, Percutaneous Approach */
"0VCT4ZZ"  =  "1"  /* Extirpation of Matter from Prepuce, Perc Endo Approach */
"0VCTXZZ"  =  "1"  /* Extirpation of Matter from Prepuce, External Approach */
"0VH001Z"  =  "1"  /* Insertion of Radioact Elem into Prostate, Open Approach */
"0VH031Z"  =  "1"  /* Insertion of Radioact Elem into Prostate, Perc Approach */
"0VH041Z"  =  "1"  /* Insertion of Radioact Elem into Prostate, Perc Endo Approach */
"0VH071Z"  =  "1"  /* Insertion of Radioactive Element into Prostate, Via Opening */
"0VH081Z"  =  "1"  /* Insertion of Radioactive Element into Prostate, Endo */
"0VJ40ZZ"  =  "1"  /* Inspection of Prostate and Seminal Vesicles, Open Approach */
"0VJ43ZZ"  =  "1"  /* Inspection of Prostate and Seminal Vesicles, Perc Approach */
"0VJ44ZZ"  =  "1"  /* Inspection of Prostate/Seminal Ves, Perc Endo Approach */
"0VJD0ZZ"  =  "1"  /* Inspection of Testis, Open Approach */
"0VJD3ZZ"  =  "1"  /* Inspection of Testis, Percutaneous Approach */
"0VJD4ZZ"  =  "1"  /* Inspection of Testis, Percutaneous Endoscopic Approach */
"0VJM0ZZ"  =  "1"  /* Inspection of Epididymis and Spermatic Cord, Open Approach */
"0VJM3ZZ"  =  "1"  /* Inspection of Epididymis and Spermatic Cord, Perc Approach */
"0VJM4ZZ"  =  "1"  /* Inspection of Epididymis/Sperm Cord, Perc Endo Approach */
"0VJR0ZZ"  =  "1"  /* Inspection of Vas Deferens, Open Approach */
"0VJR3ZZ"  =  "1"  /* Inspection of Vas Deferens, Percutaneous Approach */
"0VJR4ZZ"  =  "1"  /* Inspection of Vas Deferens, Percutaneous Endoscopic Approach */
"0VLN0DZ"  =  "1"  /* Occlusion of R Vas Deferens with Intralum Dev, Open Approach */
"0VLN3DZ"  =  "1"  /* Occlusion of R Vas Deferens with Intralum Dev, Perc Approach */
"0VLN4DZ"  =  "1"  /* Occlusion R Vas Deferens w Intralum Dev, Perc Endo */
"0VLP0DZ"  =  "1"  /* Occlusion of L Vas Deferens with Intralum Dev, Open Approach */
"0VLP3DZ"  =  "1"  /* Occlusion of L Vas Deferens with Intralum Dev, Perc Approach */
"0VLP4DZ"  =  "1"  /* Occlusion L Vas Deferens w Intralum Dev, Perc Endo */
"0VLQ0DZ"  =  "1"  /* Occlusion Bi Vas Deferens w Intralum Dev, Open */
"0VLQ3DZ"  =  "1"  /* Occlusion Bi Vas Deferens w Intralum Dev, Perc */
"0VLQ4DZ"  =  "1"  /* Occlusion Bi Vas Deferens w Intralum Dev, Perc Endo */
"0VM5XZZ"  =  "1"  /* Reattachment of Scrotum, External Approach */
"0VM60ZZ"  =  "1"  /* Reattachment of Right Tunica Vaginalis, Open Approach */
"0VM64ZZ"  =  "1"  /* Reattachment of Right Tunica Vaginalis, Perc Endo Approach */
"0VM70ZZ"  =  "1"  /* Reattachment of Left Tunica Vaginalis, Open Approach */
"0VM74ZZ"  =  "1"  /* Reattachment of Left Tunica Vaginalis, Perc Endo Approach */
"0VM90ZZ"  =  "1"  /* Reattachment of Right Testis, Open Approach */
"0VM94ZZ"  =  "1"  /* Reattachment of Right Testis, Perc Endo Approach */
"0VMB0ZZ"  =  "1"  /* Reattachment of Left Testis, Open Approach */
"0VMB4ZZ"  =  "1"  /* Reattachment of Left Testis, Perc Endo Approach */
"0VMC0ZZ"  =  "1"  /* Reattachment of Bilateral Testes, Open Approach */
"0VMC4ZZ"  =  "1"  /* Reattachment of Bilateral Testes, Perc Endo Approach */
"0VMF0ZZ"  =  "1"  /* Reattachment of Right Spermatic Cord, Open Approach */
"0VMF4ZZ"  =  "1"  /* Reattachment of Right Spermatic Cord, Perc Endo Approach */
"0VMG0ZZ"  =  "1"  /* Reattachment of Left Spermatic Cord, Open Approach */
"0VMG4ZZ"  =  "1"  /* Reattachment of Left Spermatic Cord, Perc Endo Approach */
"0VMH0ZZ"  =  "1"  /* Reattachment of Bilateral Spermatic Cords, Open Approach */
"0VMH4ZZ"  =  "1"  /* Reattachment of Bi Sperm Cord, Perc Endo Approach */
"0VMSXZZ"  =  "1"  /* Reattachment of Penis, External Approach */
"0VN00ZZ"  =  "1"  /* Release Prostate, Open Approach */
"0VN03ZZ"  =  "1"  /* Release Prostate, Percutaneous Approach */
"0VN04ZZ"  =  "1"  /* Release Prostate, Percutaneous Endoscopic Approach */
"0VN07ZZ"  =  "1"  /* Release Prostate, Via Natural or Artificial Opening */
"0VN08ZZ"  =  "1"  /* Release Prostate, Endo */
"0VN10ZZ"  =  "1"  /* Release Right Seminal Vesicle, Open Approach */
"0VN13ZZ"  =  "1"  /* Release Right Seminal Vesicle, Percutaneous Approach */
"0VN14ZZ"  =  "1"  /* Release Right Seminal Vesicle, Perc Endo Approach */
"0VN20ZZ"  =  "1"  /* Release Left Seminal Vesicle, Open Approach */
"0VN23ZZ"  =  "1"  /* Release Left Seminal Vesicle, Percutaneous Approach */
"0VN24ZZ"  =  "1"  /* Release Left Seminal Vesicle, Perc Endo Approach */
"0VN30ZZ"  =  "1"  /* Release Bilateral Seminal Vesicles, Open Approach */
"0VN33ZZ"  =  "1"  /* Release Bilateral Seminal Vesicles, Percutaneous Approach */
"0VN34ZZ"  =  "1"  /* Release Bilateral Seminal Vesicles, Perc Endo Approach */
"0VN50ZZ"  =  "1"  /* Release Scrotum, Open Approach */
"0VN53ZZ"  =  "1"  /* Release Scrotum, Percutaneous Approach */
"0VN54ZZ"  =  "1"  /* Release Scrotum, Percutaneous Endoscopic Approach */
"0VN5XZZ"  =  "1"  /* Release Scrotum, External Approach */
"0VN60ZZ"  =  "1"  /* Release Right Tunica Vaginalis, Open Approach */
"0VN63ZZ"  =  "1"  /* Release Right Tunica Vaginalis, Percutaneous Approach */
"0VN64ZZ"  =  "1"  /* Release Right Tunica Vaginalis, Perc Endo Approach */
"0VN70ZZ"  =  "1"  /* Release Left Tunica Vaginalis, Open Approach */
"0VN73ZZ"  =  "1"  /* Release Left Tunica Vaginalis, Percutaneous Approach */
"0VN74ZZ"  =  "1"  /* Release Left Tunica Vaginalis, Perc Endo Approach */
"0VNF0ZZ"  =  "1"  /* Release Right Spermatic Cord, Open Approach */
"0VNF3ZZ"  =  "1"  /* Release Right Spermatic Cord, Percutaneous Approach */
"0VNF4ZZ"  =  "1"  /* Release Right Spermatic Cord, Perc Endo Approach */
"0VNG0ZZ"  =  "1"  /* Release Left Spermatic Cord, Open Approach */
"0VNG3ZZ"  =  "1"  /* Release Left Spermatic Cord, Percutaneous Approach */
"0VNG4ZZ"  =  "1"  /* Release Left Spermatic Cord, Perc Endo Approach */
"0VNH0ZZ"  =  "1"  /* Release Bilateral Spermatic Cords, Open Approach */
"0VNH3ZZ"  =  "1"  /* Release Bilateral Spermatic Cords, Percutaneous Approach */
"0VNH4ZZ"  =  "1"  /* Release Bilateral Spermatic Cords, Perc Endo Approach */
"0VNJ0ZZ"  =  "1"  /* Release Right Epididymis, Open Approach */
"0VNJ3ZZ"  =  "1"  /* Release Right Epididymis, Percutaneous Approach */
"0VNJ4ZZ"  =  "1"  /* Release Right Epididymis, Percutaneous Endoscopic Approach */
"0VNK0ZZ"  =  "1"  /* Release Left Epididymis, Open Approach */
"0VNK3ZZ"  =  "1"  /* Release Left Epididymis, Percutaneous Approach */
"0VNK4ZZ"  =  "1"  /* Release Left Epididymis, Percutaneous Endoscopic Approach */
"0VNL0ZZ"  =  "1"  /* Release Bilateral Epididymis, Open Approach */
"0VNL3ZZ"  =  "1"  /* Release Bilateral Epididymis, Percutaneous Approach */
"0VNL4ZZ"  =  "1"  /* Release Bilateral Epididymis, Perc Endo Approach */
"0VNN0ZZ"  =  "1"  /* Release Right Vas Deferens, Open Approach */
"0VNN3ZZ"  =  "1"  /* Release Right Vas Deferens, Percutaneous Approach */
"0VNN4ZZ"  =  "1"  /* Release Right Vas Deferens, Percutaneous Endoscopic Approach */
"0VNP0ZZ"  =  "1"  /* Release Left Vas Deferens, Open Approach */
"0VNP3ZZ"  =  "1"  /* Release Left Vas Deferens, Percutaneous Approach */
"0VNP4ZZ"  =  "1"  /* Release Left Vas Deferens, Percutaneous Endoscopic Approach */
"0VNQ0ZZ"  =  "1"  /* Release Bilateral Vas Deferens, Open Approach */
"0VNQ3ZZ"  =  "1"  /* Release Bilateral Vas Deferens, Percutaneous Approach */
"0VNQ4ZZ"  =  "1"  /* Release Bilateral Vas Deferens, Perc Endo Approach */
"0VNS0ZZ"  =  "1"  /* Release Penis, Open Approach */
"0VNS3ZZ"  =  "1"  /* Release Penis, Percutaneous Approach */
"0VNS4ZZ"  =  "1"  /* Release Penis, Percutaneous Endoscopic Approach */
"0VNSXZZ"  =  "1"  /* Release Penis, External Approach */
"0VP400Z"  =  "1"  /* Remove of Drain Dev from Prostate/Seminal Ves, Open Approach */
"0VP401Z"  =  "1"  /* Remove Radioact Elem from Prostate/Seminal Ves, Open */
"0VP403Z"  =  "1"  /* Remove Infusion Dev from Prostate/Seminal Ves, Open */
"0VP407Z"  =  "1"  /* Remove of Autol Sub from Prostate/Seminal Ves, Open Approach */
"0VP40JZ"  =  "1"  /* Remove of Synth Sub from Prostate/Seminal Ves, Open Approach */
"0VP40KZ"  =  "1"  /* Remove Nonaut Sub from Prostate/Seminal Ves, Open */
"0VP430Z"  =  "1"  /* Remove of Drain Dev from Prostate/Seminal Ves, Perc Approach */
"0VP431Z"  =  "1"  /* Remove Radioact Elem from Prostate/Seminal Ves, Perc */
"0VP433Z"  =  "1"  /* Remove Infusion Dev from Prostate/Seminal Ves, Perc */
"0VP437Z"  =  "1"  /* Remove of Autol Sub from Prostate/Seminal Ves, Perc Approach */
"0VP43JZ"  =  "1"  /* Remove of Synth Sub from Prostate/Seminal Ves, Perc Approach */
"0VP43KZ"  =  "1"  /* Remove Nonaut Sub from Prostate/Seminal Ves, Perc */
"0VP440Z"  =  "1"  /* Remove Drain Dev from Prostate/Seminal Ves, Perc Endo */
"0VP441Z"  =  "1"  /* Remove Radioact Elem from Prostate/Seminal Ves, Perc Endo */
"0VP443Z"  =  "1"  /* Remove Infusion Dev from Prostate/Seminal Ves, Perc Endo */
"0VP447Z"  =  "1"  /* Remove Autol Sub from Prostate/Seminal Ves, Perc Endo */
"0VP44JZ"  =  "1"  /* Remove Synth Sub from Prostate/Seminal Ves, Perc Endo */
"0VP44KZ"  =  "1"  /* Remove Nonaut Sub from Prostate/Seminal Ves, Perc Endo */
"0VP470Z"  =  "1"  /* Removal of Drain Dev from Prostate/Seminal Ves, Via Opening */
"0VP471Z"  =  "1"  /* Remove Radioact Elem from Prostate/Seminal Ves, Via Opening */
"0VP473Z"  =  "1"  /* Remove Infusion Dev from Prostate/Seminal Ves, Via Opening */
"0VP477Z"  =  "1"  /* Removal of Autol Sub from Prostate/Seminal Ves, Via Opening */
"0VP47JZ"  =  "1"  /* Removal of Synth Sub from Prostate/Seminal Ves, Via Opening */
"0VP47KZ"  =  "1"  /* Removal of Nonaut Sub from Prostate/Seminal Ves, Via Opening */
"0VP480Z"  =  "1"  /* Removal of Drainage Device from Prostate/Seminal Ves, Endo */
"0VP481Z"  =  "1"  /* Removal of Radioact Elem from Prostate/Seminal Ves, Endo */
"0VP483Z"  =  "1"  /* Removal of Infusion Device from Prostate/Seminal Ves, Endo */
"0VP487Z"  =  "1"  /* Removal of Autol Sub from Prostate/Seminal Ves, Endo */
"0VP48JZ"  =  "1"  /* Removal of Synth Sub from Prostate/Seminal Ves, Endo */
"0VP48KZ"  =  "1"  /* Removal of Nonaut Sub from Prostate/Seminal Ves, Endo */
"0VPD00Z"  =  "1"  /* Removal of Drainage Device from Testis, Open Approach */
"0VPD03Z"  =  "1"  /* Removal of Infusion Device from Testis, Open Approach */
"0VPD07Z"  =  "1"  /* Removal of Autol Sub from Testis, Open Approach */
"0VPD0JZ"  =  "1"  /* Removal of Synthetic Substitute from Testis, Open Approach */
"0VPD0KZ"  =  "1"  /* Removal of Nonaut Sub from Testis, Open Approach */
"0VPD30Z"  =  "1"  /* Removal of Drainage Device from Testis, Perc Approach */
"0VPD33Z"  =  "1"  /* Removal of Infusion Device from Testis, Perc Approach */
"0VPD37Z"  =  "1"  /* Removal of Autol Sub from Testis, Perc Approach */
"0VPD3JZ"  =  "1"  /* Removal of Synthetic Substitute from Testis, Perc Approach */
"0VPD3KZ"  =  "1"  /* Removal of Nonaut Sub from Testis, Perc Approach */
"0VPD40Z"  =  "1"  /* Removal of Drainage Device from Testis, Perc Endo Approach */
"0VPD43Z"  =  "1"  /* Removal of Infusion Device from Testis, Perc Endo Approach */
"0VPD47Z"  =  "1"  /* Removal of Autol Sub from Testis, Perc Endo Approach */
"0VPD4JZ"  =  "1"  /* Removal of Synth Sub from Testis, Perc Endo Approach */
"0VPD4KZ"  =  "1"  /* Removal of Nonaut Sub from Testis, Perc Endo Approach */
"0VPD70Z"  =  "1"  /* Removal of Drainage Device from Testis, Via Opening */
"0VPD73Z"  =  "1"  /* Removal of Infusion Device from Testis, Via Opening */
"0VPD77Z"  =  "1"  /* Removal of Autol Sub from Testis, Via Opening */
"0VPD7JZ"  =  "1"  /* Removal of Synthetic Substitute from Testis, Via Opening */
"0VPD7KZ"  =  "1"  /* Removal of Nonaut Sub from Testis, Via Opening */
"0VPD80Z"  =  "1"  /* Removal of Drainage Device from Testis, Endo */
"0VPD83Z"  =  "1"  /* Removal of Infusion Device from Testis, Endo */
"0VPD87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Testis, Endo */
"0VPD8JZ"  =  "1"  /* Removal of Synthetic Substitute from Testis, Endo */
"0VPD8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Testis, Endo */
"0VPM00Z"  =  "1"  /* Remove Drain Dev from Epididymis/Sperm Cord, Open */
"0VPM03Z"  =  "1"  /* Remove Infusion Dev from Epididymis/Sperm Cord, Open */
"0VPM07Z"  =  "1"  /* Remove Autol Sub from Epididymis/Sperm Cord, Open */
"0VPM0CZ"  =  "1"  /* Remove Extralum Dev from Epididymis/Sperm Cord, Open */
"0VPM0JZ"  =  "1"  /* Remove Synth Sub from Epididymis/Sperm Cord, Open */
"0VPM0KZ"  =  "1"  /* Remove Nonaut Sub from Epididymis/Sperm Cord, Open */
"0VPM30Z"  =  "1"  /* Remove Drain Dev from Epididymis/Sperm Cord, Perc */
"0VPM33Z"  =  "1"  /* Remove Infusion Dev from Epididymis/Sperm Cord, Perc */
"0VPM37Z"  =  "1"  /* Remove Autol Sub from Epididymis/Sperm Cord, Perc */
"0VPM3CZ"  =  "1"  /* Remove Extralum Dev from Epididymis/Sperm Cord, Perc */
"0VPM3JZ"  =  "1"  /* Remove Synth Sub from Epididymis/Sperm Cord, Perc */
"0VPM3KZ"  =  "1"  /* Remove Nonaut Sub from Epididymis/Sperm Cord, Perc */
"0VPM40Z"  =  "1"  /* Remove Drain Dev from Epididymis/Sperm Cord, Perc Endo */
"0VPM43Z"  =  "1"  /* Remove Infusion Dev from Epididymis/Sperm Cord, Perc Endo */
"0VPM47Z"  =  "1"  /* Remove Autol Sub from Epididymis/Sperm Cord, Perc Endo */
"0VPM4CZ"  =  "1"  /* Remove Extralum Dev from Epididymis/Sperm Cord, Perc Endo */
"0VPM4JZ"  =  "1"  /* Remove Synth Sub from Epididymis/Sperm Cord, Perc Endo */
"0VPM4KZ"  =  "1"  /* Remove Nonaut Sub from Epididymis/Sperm Cord, Perc Endo */
"0VPM70Z"  =  "1"  /* Removal of Drain Dev from Epididymis/Sperm Cord, Via Opening */
"0VPM73Z"  =  "1"  /* Remove Infusion Dev from Epididymis/Sperm Cord, Via Opening */
"0VPM77Z"  =  "1"  /* Removal of Autol Sub from Epididymis/Sperm Cord, Via Opening */
"0VPM7CZ"  =  "1"  /* Remove Extralum Dev from Epididymis/Sperm Cord, Via Opening */
"0VPM7JZ"  =  "1"  /* Removal of Synth Sub from Epididymis/Sperm Cord, Via Opening */
"0VPM7KZ"  =  "1"  /* Remove of Nonaut Sub from Epididymis/Sperm Cord, Via Opening */
"0VPM80Z"  =  "1"  /* Removal of Drainage Device from Epididymis/Sperm Cord, Endo */
"0VPM83Z"  =  "1"  /* Removal of Infusion Device from Epididymis/Sperm Cord, Endo */
"0VPM87Z"  =  "1"  /* Removal of Autol Sub from Epididymis/Sperm Cord, Endo */
"0VPM8CZ"  =  "1"  /* Removal of Extralum Dev from Epididymis/Sperm Cord, Endo */
"0VPM8JZ"  =  "1"  /* Removal of Synth Sub from Epididymis/Sperm Cord, Endo */
"0VPM8KZ"  =  "1"  /* Removal of Nonaut Sub from Epididymis/Sperm Cord, Endo */
"0VPR0DZ"  =  "1"  /* Removal of Intralum Dev from Vas Deferens, Open Approach */
"0VPR3DZ"  =  "1"  /* Removal of Intralum Dev from Vas Deferens, Perc Approach */
"0VPR4DZ"  =  "1"  /* Remove of Intralum Dev from Vas Deferens, Perc Endo Approach */
"0VPR7DZ"  =  "1"  /* Removal of Intralum Dev from Vas Deferens, Via Opening */
"0VPR8DZ"  =  "1"  /* Removal of Intraluminal Device from Vas Deferens, Endo */
"0VPS00Z"  =  "1"  /* Removal of Drainage Device from Penis, Open Approach */
"0VPS03Z"  =  "1"  /* Removal of Infusion Device from Penis, Open Approach */
"0VPS07Z"  =  "1"  /* Removal of Autol Sub from Penis, Open Approach */
"0VPS0JZ"  =  "1"  /* Removal of Synthetic Substitute from Penis, Open Approach */
"0VPS0KZ"  =  "1"  /* Removal of Nonaut Sub from Penis, Open Approach */
"0VPS30Z"  =  "1"  /* Removal of Drainage Device from Penis, Percutaneous Approach */
"0VPS33Z"  =  "1"  /* Removal of Infusion Device from Penis, Percutaneous Approach */
"0VPS37Z"  =  "1"  /* Removal of Autol Sub from Penis, Perc Approach */
"0VPS3JZ"  =  "1"  /* Removal of Synthetic Substitute from Penis, Perc Approach */
"0VPS3KZ"  =  "1"  /* Removal of Nonaut Sub from Penis, Perc Approach */
"0VPS40Z"  =  "1"  /* Removal of Drainage Device from Penis, Perc Endo Approach */
"0VPS43Z"  =  "1"  /* Removal of Infusion Device from Penis, Perc Endo Approach */
"0VPS47Z"  =  "1"  /* Removal of Autol Sub from Penis, Perc Endo Approach */
"0VPS4JZ"  =  "1"  /* Removal of Synth Sub from Penis, Perc Endo Approach */
"0VPS4KZ"  =  "1"  /* Removal of Nonaut Sub from Penis, Perc Endo Approach */
"0VPS70Z"  =  "1"  /* Removal of Drainage Device from Penis, Via Opening */
"0VPS73Z"  =  "1"  /* Removal of Infusion Device from Penis, Via Opening */
"0VPS77Z"  =  "1"  /* Removal of Autol Sub from Penis, Via Opening */
"0VPS7JZ"  =  "1"  /* Removal of Synthetic Substitute from Penis, Via Opening */
"0VPS7KZ"  =  "1"  /* Removal of Nonaut Sub from Penis, Via Opening */
"0VPS80Z"  =  "1"  /* Removal of Drainage Device from Penis, Endo */
"0VPS83Z"  =  "1"  /* Removal of Infusion Device from Penis, Endo */
"0VPS87Z"  =  "1"  /* Removal of Autologous Tissue Substitute from Penis, Endo */
"0VPS8JZ"  =  "1"  /* Removal of Synthetic Substitute from Penis, Endo */
"0VPS8KZ"  =  "1"  /* Removal of Nonautologous Tissue Substitute from Penis, Endo */
"0VQ00ZZ"  =  "1"  /* Repair Prostate, Open Approach */
"0VQ03ZZ"  =  "1"  /* Repair Prostate, Percutaneous Approach */
"0VQ04ZZ"  =  "1"  /* Repair Prostate, Percutaneous Endoscopic Approach */
"0VQ07ZZ"  =  "1"  /* Repair Prostate, Via Natural or Artificial Opening */
"0VQ08ZZ"  =  "1"  /* Repair Prostate, Endo */
"0VQ10ZZ"  =  "1"  /* Repair Right Seminal Vesicle, Open Approach */
"0VQ13ZZ"  =  "1"  /* Repair Right Seminal Vesicle, Percutaneous Approach */
"0VQ14ZZ"  =  "1"  /* Repair Right Seminal Vesicle, Perc Endo Approach */
"0VQ20ZZ"  =  "1"  /* Repair Left Seminal Vesicle, Open Approach */
"0VQ23ZZ"  =  "1"  /* Repair Left Seminal Vesicle, Percutaneous Approach */
"0VQ24ZZ"  =  "1"  /* Repair Left Seminal Vesicle, Perc Endo Approach */
"0VQ30ZZ"  =  "1"  /* Repair Bilateral Seminal Vesicles, Open Approach */
"0VQ33ZZ"  =  "1"  /* Repair Bilateral Seminal Vesicles, Percutaneous Approach */
"0VQ34ZZ"  =  "1"  /* Repair Bilateral Seminal Vesicles, Perc Endo Approach */
"0VQ90ZZ"  =  "1"  /* Repair Right Testis, Open Approach */
"0VQ93ZZ"  =  "1"  /* Repair Right Testis, Percutaneous Approach */
"0VQ94ZZ"  =  "1"  /* Repair Right Testis, Percutaneous Endoscopic Approach */
"0VQB0ZZ"  =  "1"  /* Repair Left Testis, Open Approach */
"0VQB3ZZ"  =  "1"  /* Repair Left Testis, Percutaneous Approach */
"0VQB4ZZ"  =  "1"  /* Repair Left Testis, Percutaneous Endoscopic Approach */
"0VQC0ZZ"  =  "1"  /* Repair Bilateral Testes, Open Approach */
"0VQC3ZZ"  =  "1"  /* Repair Bilateral Testes, Percutaneous Approach */
"0VQC4ZZ"  =  "1"  /* Repair Bilateral Testes, Percutaneous Endoscopic Approach */
"0VQF0ZZ"  =  "1"  /* Repair Right Spermatic Cord, Open Approach */
"0VQF3ZZ"  =  "1"  /* Repair Right Spermatic Cord, Percutaneous Approach */
"0VQF4ZZ"  =  "1"  /* Repair Right Spermatic Cord, Perc Endo Approach */
"0VQG0ZZ"  =  "1"  /* Repair Left Spermatic Cord, Open Approach */
"0VQG3ZZ"  =  "1"  /* Repair Left Spermatic Cord, Percutaneous Approach */
"0VQG4ZZ"  =  "1"  /* Repair Left Spermatic Cord, Percutaneous Endoscopic Approach */
"0VQH0ZZ"  =  "1"  /* Repair Bilateral Spermatic Cords, Open Approach */
"0VQH3ZZ"  =  "1"  /* Repair Bilateral Spermatic Cords, Percutaneous Approach */
"0VQH4ZZ"  =  "1"  /* Repair Bilateral Spermatic Cords, Perc Endo Approach */
"0VQJ0ZZ"  =  "1"  /* Repair Right Epididymis, Open Approach */
"0VQJ3ZZ"  =  "1"  /* Repair Right Epididymis, Percutaneous Approach */
"0VQJ4ZZ"  =  "1"  /* Repair Right Epididymis, Percutaneous Endoscopic Approach */
"0VQK0ZZ"  =  "1"  /* Repair Left Epididymis, Open Approach */
"0VQK3ZZ"  =  "1"  /* Repair Left Epididymis, Percutaneous Approach */
"0VQK4ZZ"  =  "1"  /* Repair Left Epididymis, Percutaneous Endoscopic Approach */
"0VQL0ZZ"  =  "1"  /* Repair Bilateral Epididymis, Open Approach */
"0VQL3ZZ"  =  "1"  /* Repair Bilateral Epididymis, Percutaneous Approach */
"0VQL4ZZ"  =  "1"  /* Repair Bilateral Epididymis, Perc Endo Approach */
"0VQN0ZZ"  =  "1"  /* Repair Right Vas Deferens, Open Approach */
"0VQN3ZZ"  =  "1"  /* Repair Right Vas Deferens, Percutaneous Approach */
"0VQN4ZZ"  =  "1"  /* Repair Right Vas Deferens, Percutaneous Endoscopic Approach */
"0VQP0ZZ"  =  "1"  /* Repair Left Vas Deferens, Open Approach */
"0VQP3ZZ"  =  "1"  /* Repair Left Vas Deferens, Percutaneous Approach */
"0VQP4ZZ"  =  "1"  /* Repair Left Vas Deferens, Percutaneous Endoscopic Approach */
"0VQQ0ZZ"  =  "1"  /* Repair Bilateral Vas Deferens, Open Approach */
"0VQQ3ZZ"  =  "1"  /* Repair Bilateral Vas Deferens, Percutaneous Approach */
"0VQQ4ZZ"  =  "1"  /* Repair Bilateral Vas Deferens, Perc Endo Approach */
"0VQS0ZZ"  =  "1"  /* Repair Penis, Open Approach */
"0VQS3ZZ"  =  "1"  /* Repair Penis, Percutaneous Approach */
"0VQS4ZZ"  =  "1"  /* Repair Penis, Percutaneous Endoscopic Approach */
"0VQSXZZ"  =  "1"  /* Repair Penis, External Approach */
"0VQT0ZZ"  =  "1"  /* Repair Prepuce, Open Approach */
"0VQT3ZZ"  =  "1"  /* Repair Prepuce, Percutaneous Approach */
"0VQT4ZZ"  =  "1"  /* Repair Prepuce, Percutaneous Endoscopic Approach */
"0VQTXZZ"  =  "1"  /* Repair Prepuce, External Approach */
"0VR90JZ"  =  "1"  /* Replacement of Right Testis with Synth Sub, Open Approach */
"0VRB0JZ"  =  "1"  /* Replacement of Left Testis with Synth Sub, Open Approach */
"0VRC0JZ"  =  "1"  /* Replacement of Bi Testis with Synth Sub, Open Approach */
"0VS90ZZ"  =  "1"  /* Reposition Right Testis, Open Approach */
"0VS93ZZ"  =  "1"  /* Reposition Right Testis, Percutaneous Approach */
"0VS94ZZ"  =  "1"  /* Reposition Right Testis, Percutaneous Endoscopic Approach */
"0VSB0ZZ"  =  "1"  /* Reposition Left Testis, Open Approach */
"0VSB3ZZ"  =  "1"  /* Reposition Left Testis, Percutaneous Approach */
"0VSB4ZZ"  =  "1"  /* Reposition Left Testis, Percutaneous Endoscopic Approach */
"0VSC0ZZ"  =  "1"  /* Reposition Bilateral Testes, Open Approach */
"0VSC3ZZ"  =  "1"  /* Reposition Bilateral Testes, Percutaneous Approach */
"0VSC4ZZ"  =  "1"  /* Reposition Bilateral Testes, Perc Endo Approach */
"0VSF0ZZ"  =  "1"  /* Reposition Right Spermatic Cord, Open Approach */
"0VSF3ZZ"  =  "1"  /* Reposition Right Spermatic Cord, Percutaneous Approach */
"0VSF4ZZ"  =  "1"  /* Reposition Right Spermatic Cord, Perc Endo Approach */
"0VSG0ZZ"  =  "1"  /* Reposition Left Spermatic Cord, Open Approach */
"0VSG3ZZ"  =  "1"  /* Reposition Left Spermatic Cord, Percutaneous Approach */
"0VSG4ZZ"  =  "1"  /* Reposition Left Spermatic Cord, Perc Endo Approach */
"0VSH0ZZ"  =  "1"  /* Reposition Bilateral Spermatic Cords, Open Approach */
"0VSH3ZZ"  =  "1"  /* Reposition Bilateral Spermatic Cords, Percutaneous Approach */
"0VSH4ZZ"  =  "1"  /* Reposition Bilateral Spermatic Cords, Perc Endo Approach */
"0VT00ZZ"  =  "1"  /* Resection of Prostate, Open Approach */
"0VT04ZZ"  =  "1"  /* Resection of Prostate, Percutaneous Endoscopic Approach */
"0VT07ZZ"  =  "1"  /* Resection of Prostate, Via Natural or Artificial Opening */
"0VT08ZZ"  =  "1"  /* Resection of Prostate, Endo */
"0VT10ZZ"  =  "1"  /* Resection of Right Seminal Vesicle, Open Approach */
"0VT14ZZ"  =  "1"  /* Resection of Right Seminal Vesicle, Perc Endo Approach */
"0VT20ZZ"  =  "1"  /* Resection of Left Seminal Vesicle, Open Approach */
"0VT24ZZ"  =  "1"  /* Resection of Left Seminal Vesicle, Perc Endo Approach */
"0VT30ZZ"  =  "1"  /* Resection of Bilateral Seminal Vesicles, Open Approach */
"0VT34ZZ"  =  "1"  /* Resection of Bilateral Seminal Vesicles, Perc Endo Approach */
"0VT60ZZ"  =  "1"  /* Resection of Right Tunica Vaginalis, Open Approach */
"0VT64ZZ"  =  "1"  /* Resection of Right Tunica Vaginalis, Perc Endo Approach */
"0VT70ZZ"  =  "1"  /* Resection of Left Tunica Vaginalis, Open Approach */
"0VT74ZZ"  =  "1"  /* Resection of Left Tunica Vaginalis, Perc Endo Approach */
"0VT90ZZ"  =  "1"  /* Resection of Right Testis, Open Approach */
"0VT94ZZ"  =  "1"  /* Resection of Right Testis, Percutaneous Endoscopic Approach */
"0VTB0ZZ"  =  "1"  /* Resection of Left Testis, Open Approach */
"0VTB4ZZ"  =  "1"  /* Resection of Left Testis, Percutaneous Endoscopic Approach */
"0VTC0ZZ"  =  "1"  /* Resection of Bilateral Testes, Open Approach */
"0VTC4ZZ"  =  "1"  /* Resection of Bilateral Testes, Perc Endo Approach */
"0VTF0ZZ"  =  "1"  /* Resection of Right Spermatic Cord, Open Approach */
"0VTF4ZZ"  =  "1"  /* Resection of Right Spermatic Cord, Perc Endo Approach */
"0VTG0ZZ"  =  "1"  /* Resection of Left Spermatic Cord, Open Approach */
"0VTG4ZZ"  =  "1"  /* Resection of Left Spermatic Cord, Perc Endo Approach */
"0VTH0ZZ"  =  "1"  /* Resection of Bilateral Spermatic Cords, Open Approach */
"0VTH4ZZ"  =  "1"  /* Resection of Bilateral Spermatic Cords, Perc Endo Approach */
"0VTJ0ZZ"  =  "1"  /* Resection of Right Epididymis, Open Approach */
"0VTJ4ZZ"  =  "1"  /* Resection of Right Epididymis, Perc Endo Approach */
"0VTK0ZZ"  =  "1"  /* Resection of Left Epididymis, Open Approach */
"0VTK4ZZ"  =  "1"  /* Resection of Left Epididymis, Perc Endo Approach */
"0VTL0ZZ"  =  "1"  /* Resection of Bilateral Epididymis, Open Approach */
"0VTL4ZZ"  =  "1"  /* Resection of Bilateral Epididymis, Perc Endo Approach */
"0VTS0ZZ"  =  "1"  /* Resection of Penis, Open Approach */
"0VTS4ZZ"  =  "1"  /* Resection of Penis, Percutaneous Endoscopic Approach */
"0VTSXZZ"  =  "1"  /* Resection of Penis, External Approach */
"0VU107Z"  =  "1"  /* Supplement R Seminal Vesicle with Autol Sub, Open Approach */
"0VU10JZ"  =  "1"  /* Supplement R Seminal Vesicle with Synth Sub, Open Approach */
"0VU10KZ"  =  "1"  /* Supplement R Seminal Vesicle with Nonaut Sub, Open Approach */
"0VU147Z"  =  "1"  /* Supplement R Seminal Vesicle w Autol Sub, Perc Endo */
"0VU14JZ"  =  "1"  /* Supplement R Seminal Vesicle w Synth Sub, Perc Endo */
"0VU14KZ"  =  "1"  /* Supplement R Seminal Vesicle w Nonaut Sub, Perc Endo */
"0VU207Z"  =  "1"  /* Supplement L Seminal Vesicle with Autol Sub, Open Approach */
"0VU20JZ"  =  "1"  /* Supplement L Seminal Vesicle with Synth Sub, Open Approach */
"0VU20KZ"  =  "1"  /* Supplement L Seminal Vesicle with Nonaut Sub, Open Approach */
"0VU247Z"  =  "1"  /* Supplement L Seminal Vesicle w Autol Sub, Perc Endo */
"0VU24JZ"  =  "1"  /* Supplement L Seminal Vesicle w Synth Sub, Perc Endo */
"0VU24KZ"  =  "1"  /* Supplement L Seminal Vesicle w Nonaut Sub, Perc Endo */
"0VU307Z"  =  "1"  /* Supplement Bi Seminal Vesicle with Autol Sub, Open Approach */
"0VU30JZ"  =  "1"  /* Supplement Bi Seminal Vesicle with Synth Sub, Open Approach */
"0VU30KZ"  =  "1"  /* Supplement Bi Seminal Vesicle with Nonaut Sub, Open Approach */
"0VU347Z"  =  "1"  /* Supplement Bi Seminal Vesicle w Autol Sub, Perc Endo */
"0VU34JZ"  =  "1"  /* Supplement Bi Seminal Vesicle w Synth Sub, Perc Endo */
"0VU34KZ"  =  "1"  /* Supplement Bi Seminal Vesicle w Nonaut Sub, Perc Endo */
"0VU507Z"  =  "1"  /* Supplement Scrotum with Autol Sub, Open Approach */
"0VU50JZ"  =  "1"  /* Supplement Scrotum with Synthetic Substitute, Open Approach */
"0VU50KZ"  =  "1"  /* Supplement Scrotum with Nonaut Sub, Open Approach */
"0VU547Z"  =  "1"  /* Supplement Scrotum with Autol Sub, Perc Endo Approach */
"0VU54JZ"  =  "1"  /* Supplement Scrotum with Synth Sub, Perc Endo Approach */
"0VU54KZ"  =  "1"  /* Supplement Scrotum with Nonaut Sub, Perc Endo Approach */
"0VU5X7Z"  =  "1"  /* Supplement Scrotum with Autol Sub, Extern Approach */
"0VU5XJZ"  =  "1"  /* Supplement Scrotum with Synth Sub, Extern Approach */
"0VU5XKZ"  =  "1"  /* Supplement Scrotum with Nonaut Sub, Extern Approach */
"0VU607Z"  =  "1"  /* Supplement R Tunica Vaginalis with Autol Sub, Open Approach */
"0VU60JZ"  =  "1"  /* Supplement R Tunica Vaginalis with Synth Sub, Open Approach */
"0VU60KZ"  =  "1"  /* Supplement R Tunica Vaginalis with Nonaut Sub, Open Approach */
"0VU647Z"  =  "1"  /* Supplement R Tunica Vaginalis w Autol Sub, Perc Endo */
"0VU64JZ"  =  "1"  /* Supplement R Tunica Vaginalis w Synth Sub, Perc Endo */
"0VU64KZ"  =  "1"  /* Supplement R Tunica Vaginalis w Nonaut Sub, Perc Endo */
"0VU707Z"  =  "1"  /* Supplement L Tunica Vaginalis with Autol Sub, Open Approach */
"0VU70JZ"  =  "1"  /* Supplement L Tunica Vaginalis with Synth Sub, Open Approach */
"0VU70KZ"  =  "1"  /* Supplement L Tunica Vaginalis with Nonaut Sub, Open Approach */
"0VU747Z"  =  "1"  /* Supplement L Tunica Vaginalis w Autol Sub, Perc Endo */
"0VU74JZ"  =  "1"  /* Supplement L Tunica Vaginalis w Synth Sub, Perc Endo */
"0VU74KZ"  =  "1"  /* Supplement L Tunica Vaginalis w Nonaut Sub, Perc Endo */
"0VU907Z"  =  "1"  /* Supplement Right Testis with Autol Sub, Open Approach */
"0VU90JZ"  =  "1"  /* Supplement Right Testis with Synth Sub, Open Approach */
"0VU90KZ"  =  "1"  /* Supplement Right Testis with Nonaut Sub, Open Approach */
"0VUB07Z"  =  "1"  /* Supplement Left Testis with Autol Sub, Open Approach */
"0VUB0JZ"  =  "1"  /* Supplement Left Testis with Synth Sub, Open Approach */
"0VUB0KZ"  =  "1"  /* Supplement Left Testis with Nonaut Sub, Open Approach */
"0VUC07Z"  =  "1"  /* Supplement Bilateral Testes with Autol Sub, Open Approach */
"0VUC0JZ"  =  "1"  /* Supplement Bilateral Testes with Synth Sub, Open Approach */
"0VUC0KZ"  =  "1"  /* Supplement Bilateral Testes with Nonaut Sub, Open Approach */
"0VUF07Z"  =  "1"  /* Supplement R Sperm Cord with Autol Sub, Open Approach */
"0VUF0JZ"  =  "1"  /* Supplement R Sperm Cord with Synth Sub, Open Approach */
"0VUF0KZ"  =  "1"  /* Supplement R Sperm Cord with Nonaut Sub, Open Approach */
"0VUF47Z"  =  "1"  /* Supplement R Sperm Cord with Autol Sub, Perc Endo Approach */
"0VUF4JZ"  =  "1"  /* Supplement R Sperm Cord with Synth Sub, Perc Endo Approach */
"0VUF4KZ"  =  "1"  /* Supplement R Sperm Cord with Nonaut Sub, Perc Endo Approach */
"0VUG07Z"  =  "1"  /* Supplement Left Spermatic Cord with Autol Sub, Open Approach */
"0VUG0JZ"  =  "1"  /* Supplement Left Spermatic Cord with Synth Sub, Open Approach */
"0VUG0KZ"  =  "1"  /* Supplement L Sperm Cord with Nonaut Sub, Open Approach */
"0VUG47Z"  =  "1"  /* Supplement L Sperm Cord with Autol Sub, Perc Endo Approach */
"0VUG4JZ"  =  "1"  /* Supplement L Sperm Cord with Synth Sub, Perc Endo Approach */
"0VUG4KZ"  =  "1"  /* Supplement L Sperm Cord with Nonaut Sub, Perc Endo Approach */
"0VUH07Z"  =  "1"  /* Supplement Bi Sperm Cord with Autol Sub, Open Approach */
"0VUH0JZ"  =  "1"  /* Supplement Bi Sperm Cord with Synth Sub, Open Approach */
"0VUH0KZ"  =  "1"  /* Supplement Bi Sperm Cord with Nonaut Sub, Open Approach */
"0VUH47Z"  =  "1"  /* Supplement Bi Sperm Cord with Autol Sub, Perc Endo Approach */
"0VUH4JZ"  =  "1"  /* Supplement Bi Sperm Cord with Synth Sub, Perc Endo Approach */
"0VUH4KZ"  =  "1"  /* Supplement Bi Sperm Cord with Nonaut Sub, Perc Endo Approach */
"0VUJ07Z"  =  "1"  /* Supplement Right Epididymis with Autol Sub, Open Approach */
"0VUJ0JZ"  =  "1"  /* Supplement Right Epididymis with Synth Sub, Open Approach */
"0VUJ0KZ"  =  "1"  /* Supplement Right Epididymis with Nonaut Sub, Open Approach */
"0VUJ47Z"  =  "1"  /* Supplement R Epididymis with Autol Sub, Perc Endo Approach */
"0VUJ4JZ"  =  "1"  /* Supplement R Epididymis with Synth Sub, Perc Endo Approach */
"0VUJ4KZ"  =  "1"  /* Supplement R Epididymis with Nonaut Sub, Perc Endo Approach */
"0VUK07Z"  =  "1"  /* Supplement Left Epididymis with Autol Sub, Open Approach */
"0VUK0JZ"  =  "1"  /* Supplement Left Epididymis with Synth Sub, Open Approach */
"0VUK0KZ"  =  "1"  /* Supplement Left Epididymis with Nonaut Sub, Open Approach */
"0VUK47Z"  =  "1"  /* Supplement L Epididymis with Autol Sub, Perc Endo Approach */
"0VUK4JZ"  =  "1"  /* Supplement L Epididymis with Synth Sub, Perc Endo Approach */
"0VUK4KZ"  =  "1"  /* Supplement L Epididymis with Nonaut Sub, Perc Endo Approach */
"0VUL07Z"  =  "1"  /* Supplement B Epididymis with Autol Sub, Open Approach */
"0VUL0JZ"  =  "1"  /* Supplement B Epididymis with Synth Sub, Open Approach */
"0VUL0KZ"  =  "1"  /* Supplement B Epididymis with Nonaut Sub, Open Approach */
"0VUL47Z"  =  "1"  /* Supplement B Epididymis with Autol Sub, Perc Endo Approach */
"0VUL4JZ"  =  "1"  /* Supplement B Epididymis with Synth Sub, Perc Endo Approach */
"0VUL4KZ"  =  "1"  /* Supplement B Epididymis with Nonaut Sub, Perc Endo Approach */
"0VUN07Z"  =  "1"  /* Supplement Right Vas Deferens with Autol Sub, Open Approach */
"0VUN0JZ"  =  "1"  /* Supplement Right Vas Deferens with Synth Sub, Open Approach */
"0VUN0KZ"  =  "1"  /* Supplement Right Vas Deferens with Nonaut Sub, Open Approach */
"0VUN47Z"  =  "1"  /* Supplement R Vas Deferens with Autol Sub, Perc Endo Approach */
"0VUN4JZ"  =  "1"  /* Supplement R Vas Deferens with Synth Sub, Perc Endo Approach */
"0VUN4KZ"  =  "1"  /* Supplement R Vas Deferens w Nonaut Sub, Perc Endo */
"0VUP07Z"  =  "1"  /* Supplement Left Vas Deferens with Autol Sub, Open Approach */
"0VUP0JZ"  =  "1"  /* Supplement Left Vas Deferens with Synth Sub, Open Approach */
"0VUP0KZ"  =  "1"  /* Supplement Left Vas Deferens with Nonaut Sub, Open Approach */
"0VUP47Z"  =  "1"  /* Supplement L Vas Deferens with Autol Sub, Perc Endo Approach */
"0VUP4JZ"  =  "1"  /* Supplement L Vas Deferens with Synth Sub, Perc Endo Approach */
"0VUP4KZ"  =  "1"  /* Supplement L Vas Deferens w Nonaut Sub, Perc Endo */
"0VUQ07Z"  =  "1"  /* Supplement Bi Vas Deferens with Autol Sub, Open Approach */
"0VUQ0JZ"  =  "1"  /* Supplement Bi Vas Deferens with Synth Sub, Open Approach */
"0VUQ0KZ"  =  "1"  /* Supplement Bi Vas Deferens with Nonaut Sub, Open Approach */
"0VUQ47Z"  =  "1"  /* Supplement Bi Vas Deferens w Autol Sub, Perc Endo */
"0VUQ4JZ"  =  "1"  /* Supplement Bi Vas Deferens w Synth Sub, Perc Endo */
"0VUQ4KZ"  =  "1"  /* Supplement Bi Vas Deferens w Nonaut Sub, Perc Endo */
"0VUS07Z"  =  "1"  /* Supplement Penis with Autol Sub, Open Approach */
"0VUS0JZ"  =  "1"  /* Supplement Penis with Synthetic Substitute, Open Approach */
"0VUS0KZ"  =  "1"  /* Supplement Penis with Nonaut Sub, Open Approach */
"0VUS47Z"  =  "1"  /* Supplement Penis with Autol Sub, Perc Endo Approach */
"0VUS4JZ"  =  "1"  /* Supplement Penis with Synth Sub, Perc Endo Approach */
"0VUS4KZ"  =  "1"  /* Supplement Penis with Nonaut Sub, Perc Endo Approach */
"0VUT07Z"  =  "1"  /* Supplement Prepuce with Autol Sub, Open Approach */
"0VUT0JZ"  =  "1"  /* Supplement Prepuce with Synthetic Substitute, Open Approach */
"0VUT0KZ"  =  "1"  /* Supplement Prepuce with Nonaut Sub, Open Approach */
"0VUT47Z"  =  "1"  /* Supplement Prepuce with Autol Sub, Perc Endo Approach */
"0VUT4JZ"  =  "1"  /* Supplement Prepuce with Synth Sub, Perc Endo Approach */
"0VUT4KZ"  =  "1"  /* Supplement Prepuce with Nonaut Sub, Perc Endo Approach */
"0VUTX7Z"  =  "1"  /* Supplement Prepuce with Autol Sub, Extern Approach */
"0VUTXJZ"  =  "1"  /* Supplement Prepuce with Synth Sub, Extern Approach */
"0VUTXKZ"  =  "1"  /* Supplement Prepuce with Nonaut Sub, Extern Approach */
"0VW400Z"  =  "1"  /* Revision of Drain Dev in Prostate/Seminal Ves, Open Approach */
"0VW403Z"  =  "1"  /* Revise Infusion Dev in Prostate/Seminal Ves, Open */
"0VW407Z"  =  "1"  /* Revision of Autol Sub in Prostate/Seminal Ves, Open Approach */
"0VW40JZ"  =  "1"  /* Revision of Synth Sub in Prostate/Seminal Ves, Open Approach */
"0VW40KZ"  =  "1"  /* Revise of Nonaut Sub in Prostate/Seminal Ves, Open Approach */
"0VW430Z"  =  "1"  /* Revision of Drain Dev in Prostate/Seminal Ves, Perc Approach */
"0VW433Z"  =  "1"  /* Revise Infusion Dev in Prostate/Seminal Ves, Perc */
"0VW437Z"  =  "1"  /* Revision of Autol Sub in Prostate/Seminal Ves, Perc Approach */
"0VW43JZ"  =  "1"  /* Revision of Synth Sub in Prostate/Seminal Ves, Perc Approach */
"0VW43KZ"  =  "1"  /* Revise of Nonaut Sub in Prostate/Seminal Ves, Perc Approach */
"0VW440Z"  =  "1"  /* Revise Drain Dev in Prostate/Seminal Ves, Perc Endo */
"0VW443Z"  =  "1"  /* Revise Infusion Dev in Prostate/Seminal Ves, Perc Endo */
"0VW447Z"  =  "1"  /* Revise Autol Sub in Prostate/Seminal Ves, Perc Endo */
"0VW44JZ"  =  "1"  /* Revise Synth Sub in Prostate/Seminal Ves, Perc Endo */
"0VW44KZ"  =  "1"  /* Revise Nonaut Sub in Prostate/Seminal Ves, Perc Endo */
"0VW470Z"  =  "1"  /* Revision of Drain Dev in Prostate/Seminal Ves, Via Opening */
"0VW473Z"  =  "1"  /* Revise of Infusion Dev in Prostate/Seminal Ves, Via Opening */
"0VW477Z"  =  "1"  /* Revision of Autol Sub in Prostate/Seminal Ves, Via Opening */
"0VW47JZ"  =  "1"  /* Revision of Synth Sub in Prostate/Seminal Ves, Via Opening */
"0VW47KZ"  =  "1"  /* Revision of Nonaut Sub in Prostate/Seminal Ves, Via Opening */
"0VW480Z"  =  "1"  /* Revision of Drainage Device in Prostate/Seminal Ves, Endo */
"0VW483Z"  =  "1"  /* Revision of Infusion Device in Prostate/Seminal Ves, Endo */
"0VW487Z"  =  "1"  /* Revision of Autol Sub in Prostate/Seminal Ves, Endo */
"0VW48JZ"  =  "1"  /* Revision of Synth Sub in Prostate/Seminal Ves, Endo */
"0VW48KZ"  =  "1"  /* Revision of Nonaut Sub in Prostate/Seminal Ves, Endo */
"0VWD00Z"  =  "1"  /* Revision of Drainage Device in Testis, Open Approach */
"0VWD03Z"  =  "1"  /* Revision of Infusion Device in Testis, Open Approach */
"0VWD07Z"  =  "1"  /* Revision of Autol Sub in Testis, Open Approach */
"0VWD0JZ"  =  "1"  /* Revision of Synthetic Substitute in Testis, Open Approach */
"0VWD0KZ"  =  "1"  /* Revision of Nonaut Sub in Testis, Open Approach */
"0VWD30Z"  =  "1"  /* Revision of Drainage Device in Testis, Percutaneous Approach */
"0VWD33Z"  =  "1"  /* Revision of Infusion Device in Testis, Percutaneous Approach */
"0VWD37Z"  =  "1"  /* Revision of Autol Sub in Testis, Perc Approach */
"0VWD3JZ"  =  "1"  /* Revision of Synthetic Substitute in Testis, Perc Approach */
"0VWD3KZ"  =  "1"  /* Revision of Nonaut Sub in Testis, Perc Approach */
"0VWD40Z"  =  "1"  /* Revision of Drainage Device in Testis, Perc Endo Approach */
"0VWD43Z"  =  "1"  /* Revision of Infusion Device in Testis, Perc Endo Approach */
"0VWD47Z"  =  "1"  /* Revision of Autol Sub in Testis, Perc Endo Approach */
"0VWD4JZ"  =  "1"  /* Revision of Synth Sub in Testis, Perc Endo Approach */
"0VWD4KZ"  =  "1"  /* Revision of Nonaut Sub in Testis, Perc Endo Approach */
"0VWD70Z"  =  "1"  /* Revision of Drainage Device in Testis, Via Opening */
"0VWD73Z"  =  "1"  /* Revision of Infusion Device in Testis, Via Opening */
"0VWD77Z"  =  "1"  /* Revision of Autol Sub in Testis, Via Opening */
"0VWD7JZ"  =  "1"  /* Revision of Synthetic Substitute in Testis, Via Opening */
"0VWD7KZ"  =  "1"  /* Revision of Nonaut Sub in Testis, Via Opening */
"0VWD80Z"  =  "1"  /* Revision of Drainage Device in Testis, Endo */
"0VWD83Z"  =  "1"  /* Revision of Infusion Device in Testis, Endo */
"0VWD87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Testis, Endo */
"0VWD8JZ"  =  "1"  /* Revision of Synthetic Substitute in Testis, Endo */
"0VWD8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Testis, Endo */
"0VWM00Z"  =  "1"  /* Revise of Drain Dev in Epididymis/Sperm Cord, Open Approach */
"0VWM03Z"  =  "1"  /* Revise Infusion Dev in Epididymis/Sperm Cord, Open */
"0VWM07Z"  =  "1"  /* Revise of Autol Sub in Epididymis/Sperm Cord, Open Approach */
"0VWM0CZ"  =  "1"  /* Revise Extralum Dev in Epididymis/Sperm Cord, Open */
"0VWM0JZ"  =  "1"  /* Revise of Synth Sub in Epididymis/Sperm Cord, Open Approach */
"0VWM0KZ"  =  "1"  /* Revise of Nonaut Sub in Epididymis/Sperm Cord, Open Approach */
"0VWM30Z"  =  "1"  /* Revise of Drain Dev in Epididymis/Sperm Cord, Perc Approach */
"0VWM33Z"  =  "1"  /* Revise Infusion Dev in Epididymis/Sperm Cord, Perc */
"0VWM37Z"  =  "1"  /* Revise of Autol Sub in Epididymis/Sperm Cord, Perc Approach */
"0VWM3CZ"  =  "1"  /* Revise Extralum Dev in Epididymis/Sperm Cord, Perc */
"0VWM3JZ"  =  "1"  /* Revise of Synth Sub in Epididymis/Sperm Cord, Perc Approach */
"0VWM3KZ"  =  "1"  /* Revise of Nonaut Sub in Epididymis/Sperm Cord, Perc Approach */
"0VWM40Z"  =  "1"  /* Revise Drain Dev in Epididymis/Sperm Cord, Perc Endo */
"0VWM43Z"  =  "1"  /* Revise Infusion Dev in Epididymis/Sperm Cord, Perc Endo */
"0VWM47Z"  =  "1"  /* Revise Autol Sub in Epididymis/Sperm Cord, Perc Endo */
"0VWM4CZ"  =  "1"  /* Revise Extralum Dev in Epididymis/Sperm Cord, Perc Endo */
"0VWM4JZ"  =  "1"  /* Revise Synth Sub in Epididymis/Sperm Cord, Perc Endo */
"0VWM4KZ"  =  "1"  /* Revise Nonaut Sub in Epididymis/Sperm Cord, Perc Endo */
"0VWM70Z"  =  "1"  /* Revision of Drain Dev in Epididymis/Sperm Cord, Via Opening */
"0VWM73Z"  =  "1"  /* Revise of Infusion Dev in Epididymis/Sperm Cord, Via Opening */
"0VWM77Z"  =  "1"  /* Revision of Autol Sub in Epididymis/Sperm Cord, Via Opening */
"0VWM7CZ"  =  "1"  /* Revise of Extralum Dev in Epididymis/Sperm Cord, Via Opening */
"0VWM7JZ"  =  "1"  /* Revision of Synth Sub in Epididymis/Sperm Cord, Via Opening */
"0VWM7KZ"  =  "1"  /* Revision of Nonaut Sub in Epididymis/Sperm Cord, Via Opening */
"0VWM80Z"  =  "1"  /* Revision of Drainage Device in Epididymis/Sperm Cord, Endo */
"0VWM83Z"  =  "1"  /* Revision of Infusion Device in Epididymis/Sperm Cord, Endo */
"0VWM87Z"  =  "1"  /* Revision of Autol Sub in Epididymis/Sperm Cord, Endo */
"0VWM8CZ"  =  "1"  /* Revision of Extralum Dev in Epididymis/Sperm Cord, Endo */
"0VWM8JZ"  =  "1"  /* Revision of Synth Sub in Epididymis/Sperm Cord, Endo */
"0VWM8KZ"  =  "1"  /* Revision of Nonaut Sub in Epididymis/Sperm Cord, Endo */
"0VWS00Z"  =  "1"  /* Revision of Drainage Device in Penis, Open Approach */
"0VWS03Z"  =  "1"  /* Revision of Infusion Device in Penis, Open Approach */
"0VWS07Z"  =  "1"  /* Revision of Autol Sub in Penis, Open Approach */
"0VWS0JZ"  =  "1"  /* Revision of Synthetic Substitute in Penis, Open Approach */
"0VWS0KZ"  =  "1"  /* Revision of Nonaut Sub in Penis, Open Approach */
"0VWS30Z"  =  "1"  /* Revision of Drainage Device in Penis, Percutaneous Approach */
"0VWS33Z"  =  "1"  /* Revision of Infusion Device in Penis, Percutaneous Approach */
"0VWS37Z"  =  "1"  /* Revision of Autol Sub in Penis, Perc Approach */
"0VWS3JZ"  =  "1"  /* Revision of Synthetic Substitute in Penis, Perc Approach */
"0VWS3KZ"  =  "1"  /* Revision of Nonaut Sub in Penis, Perc Approach */
"0VWS40Z"  =  "1"  /* Revision of Drainage Device in Penis, Perc Endo Approach */
"0VWS43Z"  =  "1"  /* Revision of Infusion Device in Penis, Perc Endo Approach */
"0VWS47Z"  =  "1"  /* Revision of Autol Sub in Penis, Perc Endo Approach */
"0VWS4JZ"  =  "1"  /* Revision of Synth Sub in Penis, Perc Endo Approach */
"0VWS4KZ"  =  "1"  /* Revision of Nonaut Sub in Penis, Perc Endo Approach */
"0VWS70Z"  =  "1"  /* Revision of Drainage Device in Penis, Via Opening */
"0VWS73Z"  =  "1"  /* Revision of Infusion Device in Penis, Via Opening */
"0VWS77Z"  =  "1"  /* Revision of Autol Sub in Penis, Via Opening */
"0VWS7JZ"  =  "1"  /* Revision of Synthetic Substitute in Penis, Via Opening */
"0VWS7KZ"  =  "1"  /* Revision of Nonaut Sub in Penis, Via Opening */
"0VWS80Z"  =  "1"  /* Revision of Drainage Device in Penis, Endo */
"0VWS83Z"  =  "1"  /* Revision of Infusion Device in Penis, Endo */
"0VWS87Z"  =  "1"  /* Revision of Autologous Tissue Substitute in Penis, Endo */
"0VWS8JZ"  =  "1"  /* Revision of Synthetic Substitute in Penis, Endo */
"0VWS8KZ"  =  "1"  /* Revision of Nonautologous Tissue Substitute in Penis, Endo */
"0W0007Z"  =  "1"  /* Alteration of Head with Autol Sub, Open Approach */
"0W000JZ"  =  "1"  /* Alteration of Head with Synthetic Substitute, Open Approach */
"0W000KZ"  =  "1"  /* Alteration of Head with Nonaut Sub, Open Approach */
"0W000ZZ"  =  "1"  /* Alteration of Head, Open Approach */
"0W0037Z"  =  "1"  /* Alteration of Head with Autol Sub, Perc Approach */
"0W003JZ"  =  "1"  /* Alteration of Head with Synthetic Substitute, Perc Approach */
"0W003KZ"  =  "1"  /* Alteration of Head with Nonaut Sub, Perc Approach */
"0W003ZZ"  =  "1"  /* Alteration of Head, Percutaneous Approach */
"0W0047Z"  =  "1"  /* Alteration of Head with Autol Sub, Perc Endo Approach */
"0W004JZ"  =  "1"  /* Alteration of Head with Synth Sub, Perc Endo Approach */
"0W004KZ"  =  "1"  /* Alteration of Head with Nonaut Sub, Perc Endo Approach */
"0W004ZZ"  =  "1"  /* Alteration of Head, Percutaneous Endoscopic Approach */
"0W0207Z"  =  "1"  /* Alteration of Face with Autol Sub, Open Approach */
"0W020JZ"  =  "1"  /* Alteration of Face with Synthetic Substitute, Open Approach */
"0W020KZ"  =  "1"  /* Alteration of Face with Nonaut Sub, Open Approach */
"0W020ZZ"  =  "1"  /* Alteration of Face, Open Approach */
"0W0237Z"  =  "1"  /* Alteration of Face with Autol Sub, Perc Approach */
"0W023JZ"  =  "1"  /* Alteration of Face with Synthetic Substitute, Perc Approach */
"0W023KZ"  =  "1"  /* Alteration of Face with Nonaut Sub, Perc Approach */
"0W023ZZ"  =  "1"  /* Alteration of Face, Percutaneous Approach */
"0W0247Z"  =  "1"  /* Alteration of Face with Autol Sub, Perc Endo Approach */
"0W024JZ"  =  "1"  /* Alteration of Face with Synth Sub, Perc Endo Approach */
"0W024KZ"  =  "1"  /* Alteration of Face with Nonaut Sub, Perc Endo Approach */
"0W024ZZ"  =  "1"  /* Alteration of Face, Percutaneous Endoscopic Approach */
"0W0407Z"  =  "1"  /* Alteration of Upper Jaw with Autol Sub, Open Approach */
"0W040JZ"  =  "1"  /* Alteration of Upper Jaw with Synth Sub, Open Approach */
"0W040KZ"  =  "1"  /* Alteration of Upper Jaw with Nonaut Sub, Open Approach */
"0W040ZZ"  =  "1"  /* Alteration of Upper Jaw, Open Approach */
"0W0437Z"  =  "1"  /* Alteration of Upper Jaw with Autol Sub, Perc Approach */
"0W043JZ"  =  "1"  /* Alteration of Upper Jaw with Synth Sub, Perc Approach */
"0W043KZ"  =  "1"  /* Alteration of Upper Jaw with Nonaut Sub, Perc Approach */
"0W043ZZ"  =  "1"  /* Alteration of Upper Jaw, Percutaneous Approach */
"0W0447Z"  =  "1"  /* Alteration of Upper Jaw with Autol Sub, Perc Endo Approach */
"0W044JZ"  =  "1"  /* Alteration of Upper Jaw with Synth Sub, Perc Endo Approach */
"0W044KZ"  =  "1"  /* Alteration of Upper Jaw with Nonaut Sub, Perc Endo Approach */
"0W044ZZ"  =  "1"  /* Alteration of Upper Jaw, Percutaneous Endoscopic Approach */
"0W0507Z"  =  "1"  /* Alteration of Lower Jaw with Autol Sub, Open Approach */
"0W050JZ"  =  "1"  /* Alteration of Lower Jaw with Synth Sub, Open Approach */
"0W050KZ"  =  "1"  /* Alteration of Lower Jaw with Nonaut Sub, Open Approach */
"0W050ZZ"  =  "1"  /* Alteration of Lower Jaw, Open Approach */
"0W0537Z"  =  "1"  /* Alteration of Lower Jaw with Autol Sub, Perc Approach */
"0W053JZ"  =  "1"  /* Alteration of Lower Jaw with Synth Sub, Perc Approach */
"0W053KZ"  =  "1"  /* Alteration of Lower Jaw with Nonaut Sub, Perc Approach */
"0W053ZZ"  =  "1"  /* Alteration of Lower Jaw, Percutaneous Approach */
"0W0547Z"  =  "1"  /* Alteration of Lower Jaw with Autol Sub, Perc Endo Approach */
"0W054JZ"  =  "1"  /* Alteration of Lower Jaw with Synth Sub, Perc Endo Approach */
"0W054KZ"  =  "1"  /* Alteration of Lower Jaw with Nonaut Sub, Perc Endo Approach */
"0W054ZZ"  =  "1"  /* Alteration of Lower Jaw, Percutaneous Endoscopic Approach */
"0W0607Z"  =  "1"  /* Alteration of Neck with Autol Sub, Open Approach */
"0W060JZ"  =  "1"  /* Alteration of Neck with Synthetic Substitute, Open Approach */
"0W060KZ"  =  "1"  /* Alteration of Neck with Nonaut Sub, Open Approach */
"0W060ZZ"  =  "1"  /* Alteration of Neck, Open Approach */
"0W0637Z"  =  "1"  /* Alteration of Neck with Autol Sub, Perc Approach */
"0W063JZ"  =  "1"  /* Alteration of Neck with Synthetic Substitute, Perc Approach */
"0W063KZ"  =  "1"  /* Alteration of Neck with Nonaut Sub, Perc Approach */
"0W063ZZ"  =  "1"  /* Alteration of Neck, Percutaneous Approach */
"0W0647Z"  =  "1"  /* Alteration of Neck with Autol Sub, Perc Endo Approach */
"0W064JZ"  =  "1"  /* Alteration of Neck with Synth Sub, Perc Endo Approach */
"0W064KZ"  =  "1"  /* Alteration of Neck with Nonaut Sub, Perc Endo Approach */
"0W064ZZ"  =  "1"  /* Alteration of Neck, Percutaneous Endoscopic Approach */
"0W0807Z"  =  "1"  /* Alteration of Chest Wall with Autol Sub, Open Approach */
"0W080JZ"  =  "1"  /* Alteration of Chest Wall with Synth Sub, Open Approach */
"0W080KZ"  =  "1"  /* Alteration of Chest Wall with Nonaut Sub, Open Approach */
"0W080ZZ"  =  "1"  /* Alteration of Chest Wall, Open Approach */
"0W0837Z"  =  "1"  /* Alteration of Chest Wall with Autol Sub, Perc Approach */
"0W083JZ"  =  "1"  /* Alteration of Chest Wall with Synth Sub, Perc Approach */
"0W083KZ"  =  "1"  /* Alteration of Chest Wall with Nonaut Sub, Perc Approach */
"0W083ZZ"  =  "1"  /* Alteration of Chest Wall, Percutaneous Approach */
"0W0847Z"  =  "1"  /* Alteration of Chest Wall with Autol Sub, Perc Endo Approach */
"0W084JZ"  =  "1"  /* Alteration of Chest Wall with Synth Sub, Perc Endo Approach */
"0W084KZ"  =  "1"  /* Alteration of Chest Wall with Nonaut Sub, Perc Endo Approach */
"0W084ZZ"  =  "1"  /* Alteration of Chest Wall, Percutaneous Endoscopic Approach */
"0W0F07Z"  =  "1"  /* Alteration of Abdominal Wall with Autol Sub, Open Approach */
"0W0F0JZ"  =  "1"  /* Alteration of Abdominal Wall with Synth Sub, Open Approach */
"0W0F0KZ"  =  "1"  /* Alteration of Abdominal Wall with Nonaut Sub, Open Approach */
"0W0F0ZZ"  =  "1"  /* Alteration of Abdominal Wall, Open Approach */
"0W0F37Z"  =  "1"  /* Alteration of Abdominal Wall with Autol Sub, Perc Approach */
"0W0F3JZ"  =  "1"  /* Alteration of Abdominal Wall with Synth Sub, Perc Approach */
"0W0F3KZ"  =  "1"  /* Alteration of Abdominal Wall with Nonaut Sub, Perc Approach */
"0W0F3ZZ"  =  "1"  /* Alteration of Abdominal Wall, Percutaneous Approach */
"0W0F47Z"  =  "1"  /* Alteration of Abd Wall with Autol Sub, Perc Endo Approach */
"0W0F4JZ"  =  "1"  /* Alteration of Abd Wall with Synth Sub, Perc Endo Approach */
"0W0F4KZ"  =  "1"  /* Alteration of Abd Wall with Nonaut Sub, Perc Endo Approach */
"0W0F4ZZ"  =  "1"  /* Alteration of Abdominal Wall, Perc Endo Approach */
"0W0K07Z"  =  "1"  /* Alteration of Upper Back with Autol Sub, Open Approach */
"0W0K0JZ"  =  "1"  /* Alteration of Upper Back with Synth Sub, Open Approach */
"0W0K0KZ"  =  "1"  /* Alteration of Upper Back with Nonaut Sub, Open Approach */
"0W0K0ZZ"  =  "1"  /* Alteration of Upper Back, Open Approach */
"0W0K37Z"  =  "1"  /* Alteration of Upper Back with Autol Sub, Perc Approach */
"0W0K3JZ"  =  "1"  /* Alteration of Upper Back with Synth Sub, Perc Approach */
"0W0K3KZ"  =  "1"  /* Alteration of Upper Back with Nonaut Sub, Perc Approach */
"0W0K3ZZ"  =  "1"  /* Alteration of Upper Back, Percutaneous Approach */
"0W0K47Z"  =  "1"  /* Alteration of Upper Back with Autol Sub, Perc Endo Approach */
"0W0K4JZ"  =  "1"  /* Alteration of Upper Back with Synth Sub, Perc Endo Approach */
"0W0K4KZ"  =  "1"  /* Alteration of Upper Back with Nonaut Sub, Perc Endo Approach */
"0W0K4ZZ"  =  "1"  /* Alteration of Upper Back, Percutaneous Endoscopic Approach */
"0W0L07Z"  =  "1"  /* Alteration of Lower Back with Autol Sub, Open Approach */
"0W0L0JZ"  =  "1"  /* Alteration of Lower Back with Synth Sub, Open Approach */
"0W0L0KZ"  =  "1"  /* Alteration of Lower Back with Nonaut Sub, Open Approach */
"0W0L0ZZ"  =  "1"  /* Alteration of Lower Back, Open Approach */
"0W0L37Z"  =  "1"  /* Alteration of Lower Back with Autol Sub, Perc Approach */
"0W0L3JZ"  =  "1"  /* Alteration of Lower Back with Synth Sub, Perc Approach */
"0W0L3KZ"  =  "1"  /* Alteration of Lower Back with Nonaut Sub, Perc Approach */
"0W0L3ZZ"  =  "1"  /* Alteration of Lower Back, Percutaneous Approach */
"0W0L47Z"  =  "1"  /* Alteration of Lower Back with Autol Sub, Perc Endo Approach */
"0W0L4JZ"  =  "1"  /* Alteration of Lower Back with Synth Sub, Perc Endo Approach */
"0W0L4KZ"  =  "1"  /* Alteration of Lower Back with Nonaut Sub, Perc Endo Approach */
"0W0L4ZZ"  =  "1"  /* Alteration of Lower Back, Percutaneous Endoscopic Approach */
"0W0M07Z"  =  "1"  /* Alteration of Male Perineum with Autol Sub, Open Approach */
"0W0M0JZ"  =  "1"  /* Alteration of Male Perineum with Synth Sub, Open Approach */
"0W0M0KZ"  =  "1"  /* Alteration of Male Perineum with Nonaut Sub, Open Approach */
"0W0M0ZZ"  =  "1"  /* Alteration of Male Perineum, Open Approach */
"0W0M37Z"  =  "1"  /* Alteration of Male Perineum with Autol Sub, Perc Approach */
"0W0M3JZ"  =  "1"  /* Alteration of Male Perineum with Synth Sub, Perc Approach */
"0W0M3KZ"  =  "1"  /* Alteration of Male Perineum with Nonaut Sub, Perc Approach */
"0W0M3ZZ"  =  "1"  /* Alteration of Male Perineum, Percutaneous Approach */
"0W0M47Z"  =  "1"  /* Alter of Male Perineum with Autol Sub, Perc Endo Approach */
"0W0M4JZ"  =  "1"  /* Alter of Male Perineum with Synth Sub, Perc Endo Approach */
"0W0M4KZ"  =  "1"  /* Alter of Male Perineum with Nonaut Sub, Perc Endo Approach */
"0W0M4ZZ"  =  "1"  /* Alteration of Male Perineum, Perc Endo Approach */
"0W0N07Z"  =  "1"  /* Alteration of Female Perineum with Autol Sub, Open Approach */
"0W0N0JZ"  =  "1"  /* Alteration of Female Perineum with Synth Sub, Open Approach */
"0W0N0KZ"  =  "1"  /* Alteration of Female Perineum with Nonaut Sub, Open Approach */
"0W0N0ZZ"  =  "1"  /* Alteration of Female Perineum, Open Approach */
"0W0N37Z"  =  "1"  /* Alteration of Female Perineum with Autol Sub, Perc Approach */
"0W0N3JZ"  =  "1"  /* Alteration of Female Perineum with Synth Sub, Perc Approach */
"0W0N3KZ"  =  "1"  /* Alteration of Female Perineum with Nonaut Sub, Perc Approach */
"0W0N3ZZ"  =  "1"  /* Alteration of Female Perineum, Percutaneous Approach */
"0W0N47Z"  =  "1"  /* Alter of Fem Perineum with Autol Sub, Perc Endo Approach */
"0W0N4JZ"  =  "1"  /* Alter of Fem Perineum with Synth Sub, Perc Endo Approach */
"0W0N4KZ"  =  "1"  /* Alter of Fem Perineum with Nonaut Sub, Perc Endo Approach */
"0W0N4ZZ"  =  "1"  /* Alteration of Female Perineum, Perc Endo Approach */
"0W110J9"  =  "1"  /* Bypass Cranial Cav to R Pleural Cav w Synth Sub, Open */
"0W110JB"  =  "1"  /* Bypass Cranial Cav to L Pleural Cav w Synth Sub, Open */
"0W110JG"  =  "1"  /* Bypass Cranial Cav to Periton Cav w Synth Sub, Open */
"0W110JJ"  =  "1"  /* Bypass Cranial Cav to Pelvic Cav w Synth Sub, Open */
"0W190J9"  =  "1"  /* Bypass R Pleural Cav to R Pleural Cav w Synth Sub, Open */
"0W190JB"  =  "1"  /* Bypass R Pleural Cav to L Pleural Cav w Synth Sub, Open */
"0W190JJ"  =  "1"  /* Bypass R Pleural Cav to Pelvic Cav w Synth Sub, Open */
"0W194J9"  =  "1"  /* Bypass R Pleural Cav to R Pleural Cav w Synth Sub, Perc Endo */
"0W194JB"  =  "1"  /* Bypass R Pleural Cav to L Pleural Cav w Synth Sub, Perc Endo */
"0W194JJ"  =  "1"  /* Bypass R Pleural Cav to Pelvic Cav w Synth Sub, Perc Endo */
"0W1B0J9"  =  "1"  /* Bypass L Pleural Cav to R Pleural Cav w Synth Sub, Open */
"0W1B0JB"  =  "1"  /* Bypass L Pleural Cav to L Pleural Cav w Synth Sub, Open */
"0W1B0JJ"  =  "1"  /* Bypass L Pleural Cav to Pelvic Cav w Synth Sub, Open */
"0W1B4J9"  =  "1"  /* Bypass L Pleural Cav to R Pleural Cav w Synth Sub, Perc Endo */
"0W1B4JB"  =  "1"  /* Bypass L Pleural Cav to L Pleural Cav w Synth Sub, Perc Endo */
"0W1B4JJ"  =  "1"  /* Bypass L Pleural Cav to Pelvic Cav w Synth Sub, Perc Endo */
"0W1G0J4"  =  "1"  /* Bypass Periton Cav to Cutan with Synth Sub, Open Approach */
"0W1G0JY"  =  "1"  /* Bypass Periton Cav to Low Vein with Synth Sub, Open Approach */
"0W1G3J4"  =  "1"  /* Bypass Periton Cav to Cutan with Synth Sub, Perc Approach */
"0W1G4J4"  =  "1"  /* Bypass Periton Cav to Cutan w Synth Sub, Perc Endo */
"0W1G4JY"  =  "1"  /* Bypass Periton Cav to Low Vein w Synth Sub, Perc Endo */
"0W1J0J9"  =  "1"  /* Bypass Pelvic Cav to R Pleural Cav w Synth Sub, Open */
"0W1J0JB"  =  "1"  /* Bypass Pelvic Cav to L Pleural Cav w Synth Sub, Open */
"0W1J0JG"  =  "1"  /* Bypass Pelvic Cav to Periton Cav w Synth Sub, Open */
"0W1J0JJ"  =  "1"  /* Bypass Pelvic Cav to Pelvic Cav w Synth Sub, Open */
"0W1J4J9"  =  "1"  /* Bypass Pelvic Cav to R Pleural Cav w Synth Sub, Perc Endo */
"0W1J4JB"  =  "1"  /* Bypass Pelvic Cav to L Pleural Cav w Synth Sub, Perc Endo */
"0W1J4JG"  =  "1"  /* Bypass Pelvic Cav to Periton Cav w Synth Sub, Perc Endo */
"0W1J4JJ"  =  "1"  /* Bypass Pelvic Cav to Pelvic Cav w Synth Sub, Perc Endo */
"0W300ZZ"  =  "1"  /* Control Bleeding in Head, Open Approach */
"0W303ZZ"  =  "1"  /* Control Bleeding in Head, Percutaneous Approach */
"0W304ZZ"  =  "1"  /* Control Bleeding in Head, Percutaneous Endoscopic Approach */
"0W310ZZ"  =  "1"  /* Control Bleeding in Cranial Cavity, Open Approach */
"0W313ZZ"  =  "1"  /* Control Bleeding in Cranial Cavity, Percutaneous Approach */
"0W314ZZ"  =  "1"  /* Control Bleeding in Cranial Cavity, Perc Endo Approach */
"0W320ZZ"  =  "1"  /* Control Bleeding in Face, Open Approach */
"0W323ZZ"  =  "1"  /* Control Bleeding in Face, Percutaneous Approach */
"0W324ZZ"  =  "1"  /* Control Bleeding in Face, Percutaneous Endoscopic Approach */
"0W330ZZ"  =  "1"  /* Control Bleeding in Oral Cavity and Throat, Open Approach */
"0W333ZZ"  =  "1"  /* Control Bleeding in Oral Cavity and Throat, Perc Approach */
"0W334ZZ"  =  "1"  /* Control Bleeding in Oral Cav & Throat, Perc Endo Approach */
"0W337ZZ"  =  "1"  /* Control Bleeding in Oral Cavity and Throat, Via Opening */
"0W338ZZ"  =  "1"  /* Control Bleeding in Oral Cavity and Throat, Endo */
"0W33XZZ"  =  "1"  /* Control Bleeding in Oral Cavity and Throat, Extern Approach */
"0W340ZZ"  =  "1"  /* Control Bleeding in Upper Jaw, Open Approach */
"0W343ZZ"  =  "1"  /* Control Bleeding in Upper Jaw, Percutaneous Approach */
"0W344ZZ"  =  "1"  /* Control Bleeding in Upper Jaw, Perc Endo Approach */
"0W350ZZ"  =  "1"  /* Control Bleeding in Lower Jaw, Open Approach */
"0W353ZZ"  =  "1"  /* Control Bleeding in Lower Jaw, Percutaneous Approach */
"0W354ZZ"  =  "1"  /* Control Bleeding in Lower Jaw, Perc Endo Approach */
"0W360ZZ"  =  "1"  /* Control Bleeding in Neck, Open Approach */
"0W363ZZ"  =  "1"  /* Control Bleeding in Neck, Percutaneous Approach */
"0W364ZZ"  =  "1"  /* Control Bleeding in Neck, Percutaneous Endoscopic Approach */
"0W380ZZ"  =  "1"  /* Control Bleeding in Chest Wall, Open Approach */
"0W383ZZ"  =  "1"  /* Control Bleeding in Chest Wall, Percutaneous Approach */
"0W384ZZ"  =  "1"  /* Control Bleeding in Chest Wall, Perc Endo Approach */
"0W390ZZ"  =  "1"  /* Control Bleeding in Right Pleural Cavity, Open Approach */
"0W393ZZ"  =  "1"  /* Control Bleeding in Right Pleural Cavity, Perc Approach */
"0W394ZZ"  =  "1"  /* Control Bleeding in Right Pleural Cavity, Perc Endo Approach */
"0W3B0ZZ"  =  "1"  /* Control Bleeding in Left Pleural Cavity, Open Approach */
"0W3B3ZZ"  =  "1"  /* Control Bleeding in Left Pleural Cavity, Perc Approach */
"0W3B4ZZ"  =  "1"  /* Control Bleeding in Left Pleural Cavity, Perc Endo Approach */
"0W3C0ZZ"  =  "1"  /* Control Bleeding in Mediastinum, Open Approach */
"0W3C3ZZ"  =  "1"  /* Control Bleeding in Mediastinum, Percutaneous Approach */
"0W3C4ZZ"  =  "1"  /* Control Bleeding in Mediastinum, Perc Endo Approach */
"0W3D0ZZ"  =  "1"  /* Control Bleeding in Pericardial Cavity, Open Approach */
"0W3D3ZZ"  =  "1"  /* Control Bleeding in Pericardial Cavity, Perc Approach */
"0W3D4ZZ"  =  "1"  /* Control Bleeding in Pericardial Cavity, Perc Endo Approach */
"0W3F0ZZ"  =  "1"  /* Control Bleeding in Abdominal Wall, Open Approach */
"0W3F3ZZ"  =  "1"  /* Control Bleeding in Abdominal Wall, Percutaneous Approach */
"0W3F4ZZ"  =  "1"  /* Control Bleeding in Abdominal Wall, Perc Endo Approach */
"0W3G3ZZ"  =  "1"  /* Control Bleeding in Peritoneal Cavity, Percutaneous Approach */
"0W3G4ZZ"  =  "1"  /* Control Bleeding in Peritoneal Cavity, Perc Endo Approach */
"0W3H0ZZ"  =  "1"  /* Control Bleeding in Retroperitoneum, Open Approach */
"0W3H3ZZ"  =  "1"  /* Control Bleeding in Retroperitoneum, Percutaneous Approach */
"0W3H4ZZ"  =  "1"  /* Control Bleeding in Retroperitoneum, Perc Endo Approach */
"0W3J0ZZ"  =  "1"  /* Control Bleeding in Pelvic Cavity, Open Approach */
"0W3J3ZZ"  =  "1"  /* Control Bleeding in Pelvic Cavity, Percutaneous Approach */
"0W3J4ZZ"  =  "1"  /* Control Bleeding in Pelvic Cavity, Perc Endo Approach */
"0W3K0ZZ"  =  "1"  /* Control Bleeding in Upper Back, Open Approach */
"0W3K3ZZ"  =  "1"  /* Control Bleeding in Upper Back, Percutaneous Approach */
"0W3K4ZZ"  =  "1"  /* Control Bleeding in Upper Back, Perc Endo Approach */
"0W3L0ZZ"  =  "1"  /* Control Bleeding in Lower Back, Open Approach */
"0W3L3ZZ"  =  "1"  /* Control Bleeding in Lower Back, Percutaneous Approach */
"0W3L4ZZ"  =  "1"  /* Control Bleeding in Lower Back, Perc Endo Approach */
"0W3M0ZZ"  =  "1"  /* Control Bleeding in Male Perineum, Open Approach */
"0W3M3ZZ"  =  "1"  /* Control Bleeding in Male Perineum, Percutaneous Approach */
"0W3M4ZZ"  =  "1"  /* Control Bleeding in Male Perineum, Perc Endo Approach */
"0W3N0ZZ"  =  "1"  /* Control Bleeding in Female Perineum, Open Approach */
"0W3N3ZZ"  =  "1"  /* Control Bleeding in Female Perineum, Percutaneous Approach */
"0W3N4ZZ"  =  "1"  /* Control Bleeding in Female Perineum, Perc Endo Approach */
"0W3P0ZZ"  =  "1"  /* Control Bleeding in Gastrointestinal Tract, Open Approach */
"0W3P3ZZ"  =  "1"  /* Control Bleeding in Gastrointestinal Tract, Perc Approach */
"0W3P4ZZ"  =  "1"  /* Control Bleeding in GI Tract, Perc Endo Approach */
"0W3P7ZZ"  =  "1"  /* Control Bleeding in Gastrointestinal Tract, Via Opening */
"0W3Q0ZZ"  =  "1"  /* Control Bleeding in Respiratory Tract, Open Approach */
"0W3Q3ZZ"  =  "1"  /* Control Bleeding in Respiratory Tract, Percutaneous Approach */
"0W3Q4ZZ"  =  "1"  /* Control Bleeding in Respiratory Tract, Perc Endo Approach */
"0W3Q7ZZ"  =  "1"  /* Control Bleeding in Respiratory Tract, Via Opening */
"0W3Q8ZZ"  =  "1"  /* Control Bleeding in Respiratory Tract, Endo */
"0W3R0ZZ"  =  "1"  /* Control Bleeding in Genitourinary Tract, Open Approach */
"0W3R3ZZ"  =  "1"  /* Control Bleeding in Genitourinary Tract, Perc Approach */
"0W3R4ZZ"  =  "1"  /* Control Bleeding in Genitourinary Tract, Perc Endo Approach */
"0W3R7ZZ"  =  "1"  /* Control Bleeding in Genitourinary Tract, Via Opening */
"0W3R8ZZ"  =  "1"  /* Control Bleeding in Genitourinary Tract, Endo */
"0W4M070"  =  "1"  /* Create Vagina in Male Perineum w Autol Sub, Open */
"0W4M0J0"  =  "1"  /* Create Vagina in Male Perineum w Synth Sub, Open */
"0W4M0K0"  =  "1"  /* Create Vagina in Male Perineum w Nonaut Sub, Open */
"0W4M0Z0"  =  "1"  /* Creation of Vagina in Male Perineum, Open Approach */
"0W4N071"  =  "1"  /* Create Penis in Fem Perineum w Autol Sub, Open */
"0W4N0J1"  =  "1"  /* Create Penis in Fem Perineum w Synth Sub, Open */
"0W4N0K1"  =  "1"  /* Create Penis in Fem Perineum w Nonaut Sub, Open */
"0W4N0Z1"  =  "1"  /* Creation of Penis in Female Perineum, Open Approach */
"0W9100Z"  =  "1"  /* Drainage of Cranial Cavity with Drain Dev, Open Approach */
"0W910ZX"  =  "1"  /* Drainage of Cranial Cavity, Open Approach, Diagnostic */
"0W910ZZ"  =  "1"  /* Drainage of Cranial Cavity, Open Approach */
"0W9200Z"  =  "1"  /* Drainage of Face with Drainage Device, Open Approach */
"0W920ZZ"  =  "1"  /* Drainage of Face, Open Approach */
"0W9230Z"  =  "1"  /* Drainage of Face with Drainage Device, Percutaneous Approach */
"0W923ZZ"  =  "1"  /* Drainage of Face, Percutaneous Approach */
"0W9240Z"  =  "1"  /* Drainage of Face with Drainage Device, Perc Endo Approach */
"0W924ZZ"  =  "1"  /* Drainage of Face, Percutaneous Endoscopic Approach */
"0W9300Z"  =  "1"  /* Drainage of Oral Cav & Throat with Drain Dev, Open Approach */
"0W930ZZ"  =  "1"  /* Drainage of Oral Cavity and Throat, Open Approach */
"0W9330Z"  =  "1"  /* Drainage of Oral Cav & Throat with Drain Dev, Perc Approach */
"0W933ZZ"  =  "1"  /* Drainage of Oral Cavity and Throat, Percutaneous Approach */
"0W9340Z"  =  "1"  /* Drain Oral Cav & Throat w Drain Dev, Perc Endo */
"0W934ZZ"  =  "1"  /* Drainage of Oral Cavity and Throat, Perc Endo Approach */
"0W9400Z"  =  "1"  /* Drainage of Upper Jaw with Drainage Device, Open Approach */
"0W940ZZ"  =  "1"  /* Drainage of Upper Jaw, Open Approach */
"0W9430Z"  =  "1"  /* Drainage of Upper Jaw with Drainage Device, Perc Approach */
"0W943ZZ"  =  "1"  /* Drainage of Upper Jaw, Percutaneous Approach */
"0W9440Z"  =  "1"  /* Drainage of Upper Jaw with Drain Dev, Perc Endo Approach */
"0W944ZZ"  =  "1"  /* Drainage of Upper Jaw, Percutaneous Endoscopic Approach */
"0W9500Z"  =  "1"  /* Drainage of Lower Jaw with Drainage Device, Open Approach */
"0W950ZZ"  =  "1"  /* Drainage of Lower Jaw, Open Approach */
"0W9530Z"  =  "1"  /* Drainage of Lower Jaw with Drainage Device, Perc Approach */
"0W953ZZ"  =  "1"  /* Drainage of Lower Jaw, Percutaneous Approach */
"0W9540Z"  =  "1"  /* Drainage of Lower Jaw with Drain Dev, Perc Endo Approach */
"0W954ZZ"  =  "1"  /* Drainage of Lower Jaw, Percutaneous Endoscopic Approach */
"0W9600Z"  =  "1"  /* Drainage of Neck with Drainage Device, Open Approach */
"0W960ZZ"  =  "1"  /* Drainage of Neck, Open Approach */
"0W9630Z"  =  "1"  /* Drainage of Neck with Drainage Device, Percutaneous Approach */
"0W963ZZ"  =  "1"  /* Drainage of Neck, Percutaneous Approach */
"0W9640Z"  =  "1"  /* Drainage of Neck with Drainage Device, Perc Endo Approach */
"0W964ZZ"  =  "1"  /* Drainage of Neck, Percutaneous Endoscopic Approach */
"0W9C00Z"  =  "1"  /* Drainage of Mediastinum with Drainage Device, Open Approach */
"0W9C0ZX"  =  "1"  /* Drainage of Mediastinum, Open Approach, Diagnostic */
"0W9C0ZZ"  =  "1"  /* Drainage of Mediastinum, Open Approach */
"0W9C30Z"  =  "1"  /* Drainage of Mediastinum with Drainage Device, Perc Approach */
"0W9C3ZZ"  =  "1"  /* Drainage of Mediastinum, Percutaneous Approach */
"0W9C40Z"  =  "1"  /* Drainage of Mediastinum with Drain Dev, Perc Endo Approach */
"0W9C4ZZ"  =  "1"  /* Drainage of Mediastinum, Percutaneous Endoscopic Approach */
"0W9D00Z"  =  "1"  /* Drainage of Pericardial Cavity with Drain Dev, Open Approach */
"0W9D0ZX"  =  "1"  /* Drainage of Pericardial Cavity, Open Approach, Diagnostic */
"0W9D0ZZ"  =  "1"  /* Drainage of Pericardial Cavity, Open Approach */
"0W9F00Z"  =  "1"  /* Drainage of Abdominal Wall with Drain Dev, Open Approach */
"0W9F0ZX"  =  "1"  /* Drainage of Abdominal Wall, Open Approach, Diagnostic */
"0W9F0ZZ"  =  "1"  /* Drainage of Abdominal Wall, Open Approach */
"0W9F3ZX"  =  "1"  /* Drainage of Abdominal Wall, Percutaneous Approach, Diagn */
"0W9F4ZX"  =  "1"  /* Drainage of Abdominal Wall, Perc Endo Approach, Diagn */
"0W9G00Z"  =  "1"  /* Drainage of Peritoneal Cavity with Drain Dev, Open Approach */
"0W9G0ZX"  =  "1"  /* Drainage of Peritoneal Cavity, Open Approach, Diagnostic */
"0W9G0ZZ"  =  "1"  /* Drainage of Peritoneal Cavity, Open Approach */
"0W9G3ZX"  =  "1"  /* Drainage of Peritoneal Cavity, Percutaneous Approach, Diagn */
"0W9G4ZX"  =  "1"  /* Drainage of Peritoneal Cavity, Perc Endo Approach, Diagn */
"0W9H00Z"  =  "1"  /* Drainage of Retroperitoneum with Drain Dev, Open Approach */
"0W9H0ZX"  =  "1"  /* Drainage of Retroperitoneum, Open Approach, Diagnostic */
"0W9H0ZZ"  =  "1"  /* Drainage of Retroperitoneum, Open Approach */
"0W9H30Z"  =  "1"  /* Drainage of Retroperitoneum with Drain Dev, Perc Approach */
"0W9H3ZX"  =  "1"  /* Drainage of Retroperitoneum, Percutaneous Approach, Diagn */
"0W9H3ZZ"  =  "1"  /* Drainage of Retroperitoneum, Percutaneous Approach */
"0W9H40Z"  =  "1"  /* Drain of Retroperitoneum with Drain Dev, Perc Endo Approach */
"0W9H4ZX"  =  "1"  /* Drainage of Retroperitoneum, Perc Endo Approach, Diagn */
"0W9H4ZZ"  =  "1"  /* Drainage of Retroperitoneum, Perc Endo Approach */
"0W9J00Z"  =  "1"  /* Drainage of Pelvic Cavity with Drain Dev, Open Approach */
"0W9J0ZX"  =  "1"  /* Drainage of Pelvic Cavity, Open Approach, Diagnostic */
"0W9J0ZZ"  =  "1"  /* Drainage of Pelvic Cavity, Open Approach */
"0W9J3ZX"  =  "1"  /* Drainage of Pelvic Cavity, Percutaneous Approach, Diagnostic */
"0W9J40Z"  =  "1"  /* Drainage of Pelvic Cavity with Drain Dev, Perc Endo Approach */
"0W9J4ZX"  =  "1"  /* Drainage of Pelvic Cavity, Perc Endo Approach, Diagn */
"0W9J4ZZ"  =  "1"  /* Drainage of Pelvic Cavity, Percutaneous Endoscopic Approach */
"0W9N00Z"  =  "1"  /* Drainage of Female Perineum with Drain Dev, Open Approach */
"0W9N0ZZ"  =  "1"  /* Drainage of Female Perineum, Open Approach */
"0W9N30Z"  =  "1"  /* Drainage of Female Perineum with Drain Dev, Perc Approach */
"0W9N3ZZ"  =  "1"  /* Drainage of Female Perineum, Percutaneous Approach */
"0W9N40Z"  =  "1"  /* Drainage of Fem Perineum with Drain Dev, Perc Endo Approach */
"0W9N4ZZ"  =  "1"  /* Drainage of Female Perineum, Perc Endo Approach */
"0WB00ZZ"  =  "1"  /* Excision of Head, Open Approach */
"0WB03ZZ"  =  "1"  /* Excision of Head, Percutaneous Approach */
"0WB04ZZ"  =  "1"  /* Excision of Head, Percutaneous Endoscopic Approach */
"0WB0XZZ"  =  "1"  /* Excision of Head, External Approach */
"0WB20ZZ"  =  "1"  /* Excision of Face, Open Approach */
"0WB23ZZ"  =  "1"  /* Excision of Face, Percutaneous Approach */
"0WB24ZZ"  =  "1"  /* Excision of Face, Percutaneous Endoscopic Approach */
"0WB2XZZ"  =  "1"  /* Excision of Face, External Approach */
"0WB40ZZ"  =  "1"  /* Excision of Upper Jaw, Open Approach */
"0WB43ZZ"  =  "1"  /* Excision of Upper Jaw, Percutaneous Approach */
"0WB44ZZ"  =  "1"  /* Excision of Upper Jaw, Percutaneous Endoscopic Approach */
"0WB4XZZ"  =  "1"  /* Excision of Upper Jaw, External Approach */
"0WB50ZZ"  =  "1"  /* Excision of Lower Jaw, Open Approach */
"0WB53ZZ"  =  "1"  /* Excision of Lower Jaw, Percutaneous Approach */
"0WB54ZZ"  =  "1"  /* Excision of Lower Jaw, Percutaneous Endoscopic Approach */
"0WB5XZZ"  =  "1"  /* Excision of Lower Jaw, External Approach */
"0WB60ZZ"  =  "1"  /* Excision of Neck, Open Approach */
"0WB63ZZ"  =  "1"  /* Excision of Neck, Percutaneous Approach */
"0WB64ZZ"  =  "1"  /* Excision of Neck, Percutaneous Endoscopic Approach */
"0WB6XZ2"  =  "1"  /* Excision of Neck, Stoma, External Approach */
"0WB6XZZ"  =  "1"  /* Excision of Neck, External Approach */
"0WB80ZZ"  =  "1"  /* Excision of Chest Wall, Open Approach */
"0WB83ZZ"  =  "1"  /* Excision of Chest Wall, Percutaneous Approach */
"0WB84ZZ"  =  "1"  /* Excision of Chest Wall, Percutaneous Endoscopic Approach */
"0WB8XZZ"  =  "1"  /* Excision of Chest Wall, External Approach */
"0WBC0ZX"  =  "1"  /* Excision of Mediastinum, Open Approach, Diagnostic */
"0WBC0ZZ"  =  "1"  /* Excision of Mediastinum, Open Approach */
"0WBC3ZZ"  =  "1"  /* Excision of Mediastinum, Percutaneous Approach */
"0WBC4ZZ"  =  "1"  /* Excision of Mediastinum, Percutaneous Endoscopic Approach */
"0WBF0ZX"  =  "1"  /* Excision of Abdominal Wall, Open Approach, Diagnostic */
"0WBF0ZZ"  =  "1"  /* Excision of Abdominal Wall, Open Approach */
"0WBF3ZX"  =  "1"  /* Excision of Abdominal Wall, Percutaneous Approach, Diagn */
"0WBF3ZZ"  =  "1"  /* Excision of Abdominal Wall, Percutaneous Approach */
"0WBF4ZX"  =  "1"  /* Excision of Abdominal Wall, Perc Endo Approach, Diagn */
"0WBF4ZZ"  =  "1"  /* Excision of Abdominal Wall, Percutaneous Endoscopic Approach */
"0WBFXZ2"  =  "1"  /* Excision of Abdominal Wall, Stoma, External Approach */
"0WBFXZX"  =  "1"  /* Excision of Abdominal Wall, External Approach, Diagnostic */
"0WBFXZZ"  =  "1"  /* Excision of Abdominal Wall, External Approach */
"0WBH0ZX"  =  "1"  /* Excision of Retroperitoneum, Open Approach, Diagnostic */
"0WBH0ZZ"  =  "1"  /* Excision of Retroperitoneum, Open Approach */
"0WBH3ZZ"  =  "1"  /* Excision of Retroperitoneum, Percutaneous Approach */
"0WBH4ZZ"  =  "1"  /* Excision of Retroperitoneum, Perc Endo Approach */
"0WBK0ZZ"  =  "1"  /* Excision of Upper Back, Open Approach */
"0WBK3ZZ"  =  "1"  /* Excision of Upper Back, Percutaneous Approach */
"0WBK4ZZ"  =  "1"  /* Excision of Upper Back, Percutaneous Endoscopic Approach */
"0WBKXZZ"  =  "1"  /* Excision of Upper Back, External Approach */
"0WBL0ZZ"  =  "1"  /* Excision of Lower Back, Open Approach */
"0WBL3ZZ"  =  "1"  /* Excision of Lower Back, Percutaneous Approach */
"0WBL4ZZ"  =  "1"  /* Excision of Lower Back, Percutaneous Endoscopic Approach */
"0WBLXZZ"  =  "1"  /* Excision of Lower Back, External Approach */
"0WBM0ZZ"  =  "1"  /* Excision of Male Perineum, Open Approach */
"0WBM3ZZ"  =  "1"  /* Excision of Male Perineum, Percutaneous Approach */
"0WBM4ZZ"  =  "1"  /* Excision of Male Perineum, Percutaneous Endoscopic Approach */
"0WBMXZZ"  =  "1"  /* Excision of Male Perineum, External Approach */
"0WBN0ZX"  =  "1"  /* Excision of Female Perineum, Open Approach, Diagnostic */
"0WBN0ZZ"  =  "1"  /* Excision of Female Perineum, Open Approach */
"0WBN3ZX"  =  "1"  /* Excision of Female Perineum, Percutaneous Approach, Diagn */
"0WBN3ZZ"  =  "1"  /* Excision of Female Perineum, Percutaneous Approach */
"0WBN4ZX"  =  "1"  /* Excision of Female Perineum, Perc Endo Approach, Diagn */
"0WBN4ZZ"  =  "1"  /* Excision of Female Perineum, Perc Endo Approach */
"0WBNXZX"  =  "1"  /* Excision of Female Perineum, External Approach, Diagnostic */
"0WBNXZZ"  =  "1"  /* Excision of Female Perineum, External Approach */
"0WC10ZZ"  =  "1"  /* Extirpation of Matter from Cranial Cavity, Open Approach */
"0WC13ZZ"  =  "1"  /* Extirpation of Matter from Cranial Cavity, Perc Approach */
"0WC14ZZ"  =  "1"  /* Extirpation of Matter from Cranial Cav, Perc Endo Approach */
"0WC30ZZ"  =  "1"  /* Extirpation of Matter from Oral Cav & Throat, Open Approach */
"0WC33ZZ"  =  "1"  /* Extirpation of Matter from Oral Cav & Throat, Perc Approach */
"0WC34ZZ"  =  "1"  /* Extirpate matter from Oral Cav & Throat, Perc Endo */
"0WCC0ZZ"  =  "1"  /* Extirpation of Matter from Mediastinum, Open Approach */
"0WCC3ZZ"  =  "1"  /* Extirpation of Matter from Mediastinum, Perc Approach */
"0WCC4ZZ"  =  "1"  /* Extirpation of Matter from Mediastinum, Perc Endo Approach */
"0WCD0ZZ"  =  "1"  /* Extirpation of Matter from Pericardial Cavity, Open Approach */
"0WCD3ZZ"  =  "1"  /* Extirpation of Matter from Pericardial Cavity, Perc Approach */
"0WCD4ZZ"  =  "1"  /* Extirpation of Matter from Pericard Cav, Perc Endo Approach */
"0WCG0ZZ"  =  "1"  /* Extirpation of Matter from Peritoneal Cavity, Open Approach */
"0WCG3ZZ"  =  "1"  /* Extirpation of Matter from Peritoneal Cavity, Perc Approach */
"0WCG4ZZ"  =  "1"  /* Extirpation of Matter from Periton Cav, Perc Endo Approach */
"0WCJ0ZZ"  =  "1"  /* Extirpation of Matter from Pelvic Cavity, Open Approach */
"0WCJ3ZZ"  =  "1"  /* Extirpation of Matter from Pelvic Cavity, Perc Approach */
"0WCJ4ZZ"  =  "1"  /* Extirpation of Matter from Pelvic Cavity, Perc Endo Approach */
"0WCP0ZZ"  =  "1"  /* Extirpation of Matter from GI Tract, Open Approach */
"0WCP3ZZ"  =  "1"  /* Extirpation of Matter from GI Tract, Perc Approach */
"0WCP4ZZ"  =  "1"  /* Extirpation of Matter from GI Tract, Perc Endo Approach */
"0WCQ7ZZ"  =  "1"  /* Extirpation of Matter from Respiratory Tract, Via Opening */
"0WCQ8ZZ"  =  "1"  /* Extirpation of Matter from Respiratory Tract, Endo */
"0WCR0ZZ"  =  "1"  /* Extirpation of Matter from GU Tract, Open Approach */
"0WCR3ZZ"  =  "1"  /* Extirpation of Matter from GU Tract, Perc Approach */
"0WCR4ZZ"  =  "1"  /* Extirpation of Matter from GU Tract, Perc Endo Approach */
"0WF10ZZ"  =  "1"  /* Fragmentation in Cranial Cavity, Open Approach */
"0WF13ZZ"  =  "1"  /* Fragmentation in Cranial Cavity, Percutaneous Approach */
"0WF14ZZ"  =  "1"  /* Fragmentation in Cranial Cavity, Perc Endo Approach */
"0WF30ZZ"  =  "1"  /* Fragmentation in Oral Cavity and Throat, Open Approach */
"0WF33ZZ"  =  "1"  /* Fragmentation in Oral Cavity and Throat, Perc Approach */
"0WF34ZZ"  =  "1"  /* Fragmentation in Oral Cavity and Throat, Perc Endo Approach */
"0WF90ZZ"  =  "1"  /* Fragmentation in Right Pleural Cavity, Open Approach */
"0WF93ZZ"  =  "1"  /* Fragmentation in Right Pleural Cavity, Percutaneous Approach */
"0WF94ZZ"  =  "1"  /* Fragmentation in Right Pleural Cavity, Perc Endo Approach */
"0WFB0ZZ"  =  "1"  /* Fragmentation in Left Pleural Cavity, Open Approach */
"0WFB3ZZ"  =  "1"  /* Fragmentation in Left Pleural Cavity, Percutaneous Approach */
"0WFB4ZZ"  =  "1"  /* Fragmentation in Left Pleural Cavity, Perc Endo Approach */
"0WFC0ZZ"  =  "1"  /* Fragmentation in Mediastinum, Open Approach */
"0WFC3ZZ"  =  "1"  /* Fragmentation in Mediastinum, Percutaneous Approach */
"0WFC4ZZ"  =  "1"  /* Fragmentation in Mediastinum, Perc Endo Approach */
"0WFD0ZZ"  =  "1"  /* Fragmentation in Pericardial Cavity, Open Approach */
"0WFD3ZZ"  =  "1"  /* Fragmentation in Pericardial Cavity, Percutaneous Approach */
"0WFD4ZZ"  =  "1"  /* Fragmentation in Pericardial Cavity, Perc Endo Approach */
"0WFDXZZ"  =  "1"  /* Fragmentation in Pericardial Cavity, External Approach */
"0WFG0ZZ"  =  "1"  /* Fragmentation in Peritoneal Cavity, Open Approach */
"0WFG3ZZ"  =  "1"  /* Fragmentation in Peritoneal Cavity, Percutaneous Approach */
"0WFG4ZZ"  =  "1"  /* Fragmentation in Peritoneal Cavity, Perc Endo Approach */
"0WFQ0ZZ"  =  "1"  /* Fragmentation in Respiratory Tract, Open Approach */
"0WFQ3ZZ"  =  "1"  /* Fragmentation in Respiratory Tract, Percutaneous Approach */
"0WFQ4ZZ"  =  "1"  /* Fragmentation in Respiratory Tract, Perc Endo Approach */
"0WFQ7ZZ"  =  "1"  /* Fragmentation in Respiratory Tract, Via Opening */
"0WFQ8ZZ"  =  "1"  /* Fragmentation in Respiratory Tract, Endo */
"0WH001Z"  =  "1"  /* Insertion of Radioactive Element into Head, Open Approach */
"0WH031Z"  =  "1"  /* Insertion of Radioactive Element into Head, Perc Approach */
"0WH041Z"  =  "1"  /* Insertion of Radioact Elem into Head, Perc Endo Approach */
"0WH101Z"  =  "1"  /* Insertion of Radioact Elem into Cranial Cav, Open Approach */
"0WH10YZ"  =  "1"  /* Insertion of Other Device into Cranial Cavity, Open Approach */
"0WH131Z"  =  "1"  /* Insertion of Radioact Elem into Cranial Cav, Perc Approach */
"0WH13YZ"  =  "1"  /* Insertion of Other Device into Cranial Cavity, Perc Approach */
"0WH141Z"  =  "1"  /* Insert of Radioact Elem into Cranial Cav, Perc Endo Approach */
"0WH14YZ"  =  "1"  /* Insertion of Oth Dev into Cranial Cav, Perc Endo Approach */
"0WH201Z"  =  "1"  /* Insertion of Radioactive Element into Face, Open Approach */
"0WH231Z"  =  "1"  /* Insertion of Radioactive Element into Face, Perc Approach */
"0WH241Z"  =  "1"  /* Insertion of Radioact Elem into Face, Perc Endo Approach */
"0WH301Z"  =  "1"  /* Insert Radioact Elem in Oral Cav & Throat, Open */
"0WH303Z"  =  "1"  /* Insert of Infusion Dev into Oral Cav & Throat, Open Approach */
"0WH30YZ"  =  "1"  /* Insertion of Oth Dev into Oral Cav & Throat, Open Approach */
"0WH331Z"  =  "1"  /* Insert Radioact Elem in Oral Cav & Throat, Perc */
"0WH333Z"  =  "1"  /* Insert of Infusion Dev into Oral Cav & Throat, Perc Approach */
"0WH33YZ"  =  "1"  /* Insertion of Oth Dev into Oral Cav & Throat, Perc Approach */
"0WH341Z"  =  "1"  /* Insert Radioact Elem in Oral Cav & Throat, Perc Endo */
"0WH343Z"  =  "1"  /* Insert Infusion Dev in Oral Cav & Throat, Perc Endo */
"0WH34YZ"  =  "1"  /* Insert of Oth Dev into Oral Cav & Throat, Perc Endo Approach */
"0WH401Z"  =  "1"  /* Insertion of Radioactive Element into Up Jaw, Open Approach */
"0WH431Z"  =  "1"  /* Insertion of Radioactive Element into Up Jaw, Perc Approach */
"0WH441Z"  =  "1"  /* Insertion of Radioact Elem into Up Jaw, Perc Endo Approach */
"0WH501Z"  =  "1"  /* Insertion of Radioactive Element into Low Jaw, Open Approach */
"0WH531Z"  =  "1"  /* Insertion of Radioactive Element into Low Jaw, Perc Approach */
"0WH541Z"  =  "1"  /* Insertion of Radioact Elem into Low Jaw, Perc Endo Approach */
"0WH601Z"  =  "1"  /* Insertion of Radioactive Element into Neck, Open Approach */
"0WH631Z"  =  "1"  /* Insertion of Radioactive Element into Neck, Perc Approach */
"0WH641Z"  =  "1"  /* Insertion of Radioact Elem into Neck, Perc Endo Approach */
"0WH801Z"  =  "1"  /* Insertion of Radioact Elem into Chest Wall, Open Approach */
"0WH831Z"  =  "1"  /* Insertion of Radioact Elem into Chest Wall, Perc Approach */
"0WH841Z"  =  "1"  /* Insert of Radioact Elem into Chest Wall, Perc Endo Approach */
"0WH901Z"  =  "1"  /* Insertion of Radioact Elem into R Pleural Cav, Open Approach */
"0WH931Z"  =  "1"  /* Insertion of Radioact Elem into R Pleural Cav, Perc Approach */
"0WH941Z"  =  "1"  /* Insert Radioact Elem in R Pleural Cav, Perc Endo */
"0WHB01Z"  =  "1"  /* Insertion of Radioact Elem into L Pleural Cav, Open Approach */
"0WHB31Z"  =  "1"  /* Insertion of Radioact Elem into L Pleural Cav, Perc Approach */
"0WHB41Z"  =  "1"  /* Insert Radioact Elem in L Pleural Cav, Perc Endo */
"0WHC01Z"  =  "1"  /* Insertion of Radioact Elem into Mediastinum, Open Approach */
"0WHC03Z"  =  "1"  /* Insertion of Infusion Device into Mediastinum, Open Approach */
"0WHC0YZ"  =  "1"  /* Insertion of Other Device into Mediastinum, Open Approach */
"0WHC31Z"  =  "1"  /* Insertion of Radioact Elem into Mediastinum, Perc Approach */
"0WHC33Z"  =  "1"  /* Insertion of Infusion Device into Mediastinum, Perc Approach */
"0WHC3YZ"  =  "1"  /* Insertion of Other Device into Mediastinum, Perc Approach */
"0WHC41Z"  =  "1"  /* Insert of Radioact Elem into Mediastinum, Perc Endo Approach */
"0WHC43Z"  =  "1"  /* Insert of Infusion Dev into Mediastinum, Perc Endo Approach */
"0WHC4YZ"  =  "1"  /* Insertion of Oth Dev into Mediastinum, Perc Endo Approach */
"0WHD01Z"  =  "1"  /* Insertion of Radioact Elem into Pericard Cav, Open Approach */
"0WHD03Z"  =  "1"  /* Insertion of Infusion Dev into Pericard Cav, Open Approach */
"0WHD0YZ"  =  "1"  /* Insertion of Other Device into Pericard Cav, Open Approach */
"0WHD31Z"  =  "1"  /* Insertion of Radioact Elem into Pericard Cav, Perc Approach */
"0WHD33Z"  =  "1"  /* Insertion of Infusion Dev into Pericard Cav, Perc Approach */
"0WHD3YZ"  =  "1"  /* Insertion of Other Device into Pericard Cav, Perc Approach */
"0WHD41Z"  =  "1"  /* Insert Radioact Elem in Pericard Cav, Perc Endo */
"0WHD43Z"  =  "1"  /* Insert of Infusion Dev into Pericard Cav, Perc Endo Approach */
"0WHD4YZ"  =  "1"  /* Insertion of Oth Dev into Pericard Cav, Perc Endo Approach */
"0WHF01Z"  =  "1"  /* Insertion of Radioact Elem into Abd Wall, Open Approach */
"0WHF03Z"  =  "1"  /* Insertion of Infusion Device into Abd Wall, Open Approach */
"0WHF0YZ"  =  "1"  /* Insertion of Other Device into Abdominal Wall, Open Approach */
"0WHF31Z"  =  "1"  /* Insertion of Radioact Elem into Abd Wall, Perc Approach */
"0WHF33Z"  =  "1"  /* Insertion of Infusion Device into Abd Wall, Perc Approach */
"0WHF3YZ"  =  "1"  /* Insertion of Other Device into Abdominal Wall, Perc Approach */
"0WHF41Z"  =  "1"  /* Insertion of Radioact Elem into Abd Wall, Perc Endo Approach */
"0WHF43Z"  =  "1"  /* Insertion of Infusion Dev into Abd Wall, Perc Endo Approach */
"0WHF4YZ"  =  "1"  /* Insertion of Other Device into Abd Wall, Perc Endo Approach */
"0WHG01Z"  =  "1"  /* Insertion of Radioact Elem into Periton Cav, Open Approach */
"0WHG03Z"  =  "1"  /* Insertion of Infusion Device into Periton Cav, Open Approach */
"0WHG0YZ"  =  "1"  /* Insertion of Other Device into Periton Cav, Open Approach */
"0WHG31Z"  =  "1"  /* Insertion of Radioact Elem into Periton Cav, Perc Approach */
"0WHG33Z"  =  "1"  /* Insertion of Infusion Device into Periton Cav, Perc Approach */
"0WHG3YZ"  =  "1"  /* Insertion of Other Device into Periton Cav, Perc Approach */
"0WHG41Z"  =  "1"  /* Insert of Radioact Elem into Periton Cav, Perc Endo Approach */
"0WHG43Z"  =  "1"  /* Insert of Infusion Dev into Periton Cav, Perc Endo Approach */
"0WHG4YZ"  =  "1"  /* Insertion of Oth Dev into Periton Cav, Perc Endo Approach */
"0WHH01Z"  =  "1"  /* Insert of Radioact Elem into Retroperitoneum, Open Approach */
"0WHH03Z"  =  "1"  /* Insert of Infusion Dev into Retroperitoneum, Open Approach */
"0WHH0YZ"  =  "1"  /* Insertion of Oth Dev into Retroperitoneum, Open Approach */
"0WHH31Z"  =  "1"  /* Insert of Radioact Elem into Retroperitoneum, Perc Approach */
"0WHH33Z"  =  "1"  /* Insert of Infusion Dev into Retroperitoneum, Perc Approach */
"0WHH3YZ"  =  "1"  /* Insertion of Oth Dev into Retroperitoneum, Perc Approach */
"0WHH41Z"  =  "1"  /* Insert Radioact Elem in Retroperitoneum, Perc Endo */
"0WHH43Z"  =  "1"  /* Insert Infusion Dev in Retroperitoneum, Perc Endo */
"0WHH4YZ"  =  "1"  /* Insert of Oth Dev into Retroperitoneum, Perc Endo Approach */
"0WHJ01Z"  =  "1"  /* Insertion of Radioact Elem into Pelvic Cav, Open Approach */
"0WHJ03Z"  =  "1"  /* Insertion of Infusion Device into Pelvic Cav, Open Approach */
"0WHJ0YZ"  =  "1"  /* Insertion of Other Device into Pelvic Cavity, Open Approach */
"0WHJ31Z"  =  "1"  /* Insertion of Radioact Elem into Pelvic Cav, Perc Approach */
"0WHJ33Z"  =  "1"  /* Insertion of Infusion Device into Pelvic Cav, Perc Approach */
"0WHJ3YZ"  =  "1"  /* Insertion of Other Device into Pelvic Cavity, Perc Approach */
"0WHJ41Z"  =  "1"  /* Insert of Radioact Elem into Pelvic Cav, Perc Endo Approach */
"0WHJ43Z"  =  "1"  /* Insert of Infusion Dev into Pelvic Cav, Perc Endo Approach */
"0WHJ4YZ"  =  "1"  /* Insertion of Oth Dev into Pelvic Cav, Perc Endo Approach */
"0WHK01Z"  =  "1"  /* Insertion of Radioactive Element into Up Back, Open Approach */
"0WHK31Z"  =  "1"  /* Insertion of Radioactive Element into Up Back, Perc Approach */
"0WHK41Z"  =  "1"  /* Insertion of Radioact Elem into Up Back, Perc Endo Approach */
"0WHL01Z"  =  "1"  /* Insertion of Radioact Elem into Low Back, Open Approach */
"0WHL31Z"  =  "1"  /* Insertion of Radioact Elem into Low Back, Perc Approach */
"0WHL41Z"  =  "1"  /* Insertion of Radioact Elem into Low Back, Perc Endo Approach */
"0WHM01Z"  =  "1"  /* Insertion of Radioact Elem into Male Perineum, Open Approach */
"0WHM31Z"  =  "1"  /* Insertion of Radioact Elem into Male Perineum, Perc Approach */
"0WHM41Z"  =  "1"  /* Insert Radioact Elem in Male Perineum, Perc Endo */
"0WHN01Z"  =  "1"  /* Insertion of Radioact Elem into Fem Perineum, Open Approach */
"0WHN03Z"  =  "1"  /* Insertion of Infusion Dev into Fem Perineum, Open Approach */
"0WHN0YZ"  =  "1"  /* Insertion of Other Device into Fem Perineum, Open Approach */
"0WHN31Z"  =  "1"  /* Insertion of Radioact Elem into Fem Perineum, Perc Approach */
"0WHN33Z"  =  "1"  /* Insertion of Infusion Dev into Fem Perineum, Perc Approach */
"0WHN3YZ"  =  "1"  /* Insertion of Other Device into Fem Perineum, Perc Approach */
"0WHN41Z"  =  "1"  /* Insert Radioact Elem in Fem Perineum, Perc Endo */
"0WHN43Z"  =  "1"  /* Insert of Infusion Dev into Fem Perineum, Perc Endo Approach */
"0WHN4YZ"  =  "1"  /* Insertion of Oth Dev into Fem Perineum, Perc Endo Approach */
"0WHP01Z"  =  "1"  /* Insertion of Radioact Elem into GI Tract, Open Approach */
"0WHP03Z"  =  "1"  /* Insertion of Infusion Device into GI Tract, Open Approach */
"0WHP31Z"  =  "1"  /* Insertion of Radioact Elem into GI Tract, Perc Approach */
"0WHP41Z"  =  "1"  /* Insertion of Radioact Elem into GI Tract, Perc Endo Approach */
"0WHP71Z"  =  "1"  /* Insertion of Radioactive Element into GI Tract, Via Opening */
"0WHP81Z"  =  "1"  /* Insertion of Radioactive Element into GI Tract, Endo */
"0WHQ01Z"  =  "1"  /* Insertion of Radioact Elem into Resp Tract, Open Approach */
"0WHQ31Z"  =  "1"  /* Insertion of Radioact Elem into Resp Tract, Perc Approach */
"0WHQ33Z"  =  "1"  /* Insertion of Infusion Device into Resp Tract, Perc Approach */
"0WHQ3YZ"  =  "1"  /* Insertion of Other Device into Resp Tract, Perc Approach */
"0WHQ41Z"  =  "1"  /* Insert of Radioact Elem into Resp Tract, Perc Endo Approach */
"0WHQ43Z"  =  "1"  /* Insert of Infusion Dev into Resp Tract, Perc Endo Approach */
"0WHQ4YZ"  =  "1"  /* Insertion of Oth Dev into Resp Tract, Perc Endo Approach */
"0WHQ71Z"  =  "1"  /* Insertion of Radioact Elem into Resp Tract, Via Opening */
"0WHQ81Z"  =  "1"  /* Insertion of Radioactive Element into Resp Tract, Endo */
"0WHR01Z"  =  "1"  /* Insertion of Radioact Elem into GU Tract, Open Approach */
"0WHR31Z"  =  "1"  /* Insertion of Radioact Elem into GU Tract, Perc Approach */
"0WHR41Z"  =  "1"  /* Insertion of Radioact Elem into GU Tract, Perc Endo Approach */
"0WHR71Z"  =  "1"  /* Insertion of Radioactive Element into GU Tract, Via Opening */
"0WHR81Z"  =  "1"  /* Insertion of Radioactive Element into GU Tract, Endo */
"0WJ10ZZ"  =  "1"  /* Inspection of Cranial Cavity, Open Approach */
"0WJ13ZZ"  =  "1"  /* Inspection of Cranial Cavity, Percutaneous Approach */
"0WJ14ZZ"  =  "1"  /* Inspection of Cranial Cavity, Perc Endo Approach */
"0WJ60ZZ"  =  "1"  /* Inspection of Neck, Open Approach */
"0WJ63ZZ"  =  "1"  /* Inspection of Neck, Percutaneous Approach */
"0WJ64ZZ"  =  "1"  /* Inspection of Neck, Percutaneous Endoscopic Approach */
"0WJ80ZZ"  =  "1"  /* Inspection of Chest Wall, Open Approach */
"0WJ83ZZ"  =  "1"  /* Inspection of Chest Wall, Percutaneous Approach */
"0WJ84ZZ"  =  "1"  /* Inspection of Chest Wall, Percutaneous Endoscopic Approach */
"0WJ90ZZ"  =  "1"  /* Inspection of Right Pleural Cavity, Open Approach */
"0WJ93ZZ"  =  "1"  /* Inspection of Right Pleural Cavity, Percutaneous Approach */
"0WJ94ZZ"  =  "1"  /* Inspection of Right Pleural Cavity, Perc Endo Approach */
"0WJB0ZZ"  =  "1"  /* Inspection of Left Pleural Cavity, Open Approach */
"0WJB3ZZ"  =  "1"  /* Inspection of Left Pleural Cavity, Percutaneous Approach */
"0WJB4ZZ"  =  "1"  /* Inspection of Left Pleural Cavity, Perc Endo Approach */
"0WJC0ZZ"  =  "1"  /* Inspection of Mediastinum, Open Approach */
"0WJC3ZZ"  =  "1"  /* Inspection of Mediastinum, Percutaneous Approach */
"0WJC4ZZ"  =  "1"  /* Inspection of Mediastinum, Percutaneous Endoscopic Approach */
"0WJD4ZZ"  =  "1"  /* Inspection of Pericardial Cavity, Perc Endo Approach */
"0WJF0ZZ"  =  "1"  /* Inspection of Abdominal Wall, Open Approach */
"0WJF3ZZ"  =  "1"  /* Inspection of Abdominal Wall, Percutaneous Approach */
"0WJF4ZZ"  =  "1"  /* Inspection of Abdominal Wall, Perc Endo Approach */
"0WJG0ZZ"  =  "1"  /* Inspection of Peritoneal Cavity, Open Approach */
"0WJG3ZZ"  =  "1"  /* Inspection of Peritoneal Cavity, Percutaneous Approach */
"0WJG4ZZ"  =  "1"  /* Inspection of Peritoneal Cavity, Perc Endo Approach */
"0WJH0ZZ"  =  "1"  /* Inspection of Retroperitoneum, Open Approach */
"0WJH3ZZ"  =  "1"  /* Inspection of Retroperitoneum, Percutaneous Approach */
"0WJH4ZZ"  =  "1"  /* Inspection of Retroperitoneum, Perc Endo Approach */
"0WJJ0ZZ"  =  "1"  /* Inspection of Pelvic Cavity, Open Approach */
"0WJJ3ZZ"  =  "1"  /* Inspection of Pelvic Cavity, Percutaneous Approach */
"0WJJ4ZZ"  =  "1"  /* Inspection of Pelvic Cavity, Perc Endo Approach */
"0WJN0ZZ"  =  "1"  /* Inspection of Female Perineum, Open Approach */
"0WJN3ZZ"  =  "1"  /* Inspection of Female Perineum, Percutaneous Approach */
"0WJN4ZZ"  =  "1"  /* Inspection of Female Perineum, Perc Endo Approach */
"0WJP0ZZ"  =  "1"  /* Inspection of Gastrointestinal Tract, Open Approach */
"0WJP3ZZ"  =  "1"  /* Inspection of Gastrointestinal Tract, Percutaneous Approach */
"0WJP4ZZ"  =  "1"  /* Inspection of Gastrointestinal Tract, Perc Endo Approach */
"0WJP7ZZ"  =  "1"  /* Inspection of Gastrointestinal Tract, Via Opening Approach */
"0WJP8ZZ"  =  "1"  /* Inspection of Gastrointestinal Tract, Endo Approach */
"0WJQ0ZZ"  =  "1"  /* Inspection of Respiratory Tract, Open Approach */
"0WJQ3ZZ"  =  "1"  /* Inspection of Respiratory Tract, Percutaneous Approach */
"0WJQ4ZZ"  =  "1"  /* Inspection of Respiratory Tract, Perc Endo Approach */
"0WJQ7ZZ"  =  "1"  /* Inspection of Respiratory Tract, Via Opening Approach */
"0WJQ8ZZ"  =  "1"  /* Inspection of Respiratory Tract, Endo Approach */
"0WJR0ZZ"  =  "1"  /* Inspection of Genitourinary Tract, Open Approach */
"0WJR3ZZ"  =  "1"  /* Inspection of Genitourinary Tract, Percutaneous Approach */
"0WJR4ZZ"  =  "1"  /* Inspection of Genitourinary Tract, Perc Endo Approach */
"0WJR7ZZ"  =  "1"  /* Inspection of Genitourinary Tract, Via Opening Approach */
"0WJR8ZZ"  =  "1"  /* Inspection of Genitourinary Tract, Endo Approach */
"0WM20ZZ"  =  "1"  /* Reattachment of Face, Open Approach */
"0WM40ZZ"  =  "1"  /* Reattachment of Upper Jaw, Open Approach */
"0WM50ZZ"  =  "1"  /* Reattachment of Lower Jaw, Open Approach */
"0WM60ZZ"  =  "1"  /* Reattachment of Neck, Open Approach */
"0WM80ZZ"  =  "1"  /* Reattachment of Chest Wall, Open Approach */
"0WMF0ZZ"  =  "1"  /* Reattachment of Abdominal Wall, Open Approach */
"0WMK0ZZ"  =  "1"  /* Reattachment of Upper Back, Open Approach */
"0WML0ZZ"  =  "1"  /* Reattachment of Lower Back, Open Approach */
"0WMM0ZZ"  =  "1"  /* Reattachment of Male Perineum, Open Approach */
"0WMN0ZZ"  =  "1"  /* Reattachment of Female Perineum, Open Approach */
"0WP100Z"  =  "1"  /* Removal of Drainage Device from Cranial Cav, Open Approach */
"0WP101Z"  =  "1"  /* Removal of Radioact Elem from Cranial Cav, Open Approach */
"0WP10JZ"  =  "1"  /* Removal of Synth Sub from Cranial Cav, Open Approach */
"0WP10YZ"  =  "1"  /* Removal of Other Device from Cranial Cavity, Open Approach */
"0WP130Z"  =  "1"  /* Removal of Drainage Device from Cranial Cav, Perc Approach */
"0WP131Z"  =  "1"  /* Removal of Radioact Elem from Cranial Cav, Perc Approach */
"0WP13JZ"  =  "1"  /* Removal of Synth Sub from Cranial Cav, Perc Approach */
"0WP13YZ"  =  "1"  /* Removal of Other Device from Cranial Cavity, Perc Approach */
"0WP140Z"  =  "1"  /* Removal of Drain Dev from Cranial Cav, Perc Endo Approach */
"0WP141Z"  =  "1"  /* Remove of Radioact Elem from Cranial Cav, Perc Endo Approach */
"0WP14JZ"  =  "1"  /* Removal of Synth Sub from Cranial Cav, Perc Endo Approach */
"0WP14YZ"  =  "1"  /* Removal of Other Device from Cranial Cav, Perc Endo Approach */
"0WPC00Z"  =  "1"  /* Removal of Drainage Device from Mediastinum, Open Approach */
"0WPC01Z"  =  "1"  /* Removal of Radioact Elem from Mediastinum, Open Approach */
"0WPC03Z"  =  "1"  /* Removal of Infusion Device from Mediastinum, Open Approach */
"0WPC07Z"  =  "1"  /* Removal of Autol Sub from Mediastinum, Open Approach */
"0WPC0JZ"  =  "1"  /* Removal of Synth Sub from Mediastinum, Open Approach */
"0WPC0KZ"  =  "1"  /* Removal of Nonaut Sub from Mediastinum, Open Approach */
"0WPC0YZ"  =  "1"  /* Removal of Other Device from Mediastinum, Open Approach */
"0WPC30Z"  =  "1"  /* Removal of Drainage Device from Mediastinum, Perc Approach */
"0WPC31Z"  =  "1"  /* Removal of Radioact Elem from Mediastinum, Perc Approach */
"0WPC33Z"  =  "1"  /* Removal of Infusion Device from Mediastinum, Perc Approach */
"0WPC37Z"  =  "1"  /* Removal of Autol Sub from Mediastinum, Perc Approach */
"0WPC3JZ"  =  "1"  /* Removal of Synth Sub from Mediastinum, Perc Approach */
"0WPC3KZ"  =  "1"  /* Removal of Nonaut Sub from Mediastinum, Perc Approach */
"0WPC3YZ"  =  "1"  /* Removal of Other Device from Mediastinum, Perc Approach */
"0WPC40Z"  =  "1"  /* Removal of Drain Dev from Mediastinum, Perc Endo Approach */
"0WPC41Z"  =  "1"  /* Remove of Radioact Elem from Mediastinum, Perc Endo Approach */
"0WPC43Z"  =  "1"  /* Removal of Infusion Dev from Mediastinum, Perc Endo Approach */
"0WPC47Z"  =  "1"  /* Removal of Autol Sub from Mediastinum, Perc Endo Approach */
"0WPC4JZ"  =  "1"  /* Removal of Synth Sub from Mediastinum, Perc Endo Approach */
"0WPC4KZ"  =  "1"  /* Removal of Nonaut Sub from Mediastinum, Perc Endo Approach */
"0WPC4YZ"  =  "1"  /* Removal of Other Device from Mediastinum, Perc Endo Approach */
"0WPD00Z"  =  "1"  /* Removal of Drainage Device from Pericard Cav, Open Approach */
"0WPD01Z"  =  "1"  /* Removal of Radioact Elem from Pericard Cav, Open Approach */
"0WPD03Z"  =  "1"  /* Removal of Infusion Device from Pericard Cav, Open Approach */
"0WPD0YZ"  =  "1"  /* Removal of Other Device from Pericard Cav, Open Approach */
"0WPD30Z"  =  "1"  /* Removal of Drainage Device from Pericard Cav, Perc Approach */
"0WPD31Z"  =  "1"  /* Removal of Radioact Elem from Pericard Cav, Perc Approach */
"0WPD33Z"  =  "1"  /* Removal of Infusion Device from Pericard Cav, Perc Approach */
"0WPD3YZ"  =  "1"  /* Removal of Other Device from Pericard Cav, Perc Approach */
"0WPD40Z"  =  "1"  /* Removal of Drain Dev from Pericard Cav, Perc Endo Approach */
"0WPD41Z"  =  "1"  /* Remove Radioact Elem from Pericard Cav, Perc Endo */
"0WPD43Z"  =  "1"  /* Remove of Infusion Dev from Pericard Cav, Perc Endo Approach */
"0WPD4YZ"  =  "1"  /* Removal of Oth Dev from Pericard Cav, Perc Endo Approach */
"0WPF00Z"  =  "1"  /* Removal of Drainage Device from Abd Wall, Open Approach */
"0WPF01Z"  =  "1"  /* Removal of Radioactive Element from Abd Wall, Open Approach */
"0WPF03Z"  =  "1"  /* Removal of Infusion Device from Abd Wall, Open Approach */
"0WPF07Z"  =  "1"  /* Removal of Autol Sub from Abd Wall, Open Approach */
"0WPF0JZ"  =  "1"  /* Removal of Synthetic Substitute from Abd Wall, Open Approach */
"0WPF0KZ"  =  "1"  /* Removal of Nonaut Sub from Abd Wall, Open Approach */
"0WPF0YZ"  =  "1"  /* Removal of Other Device from Abdominal Wall, Open Approach */
"0WPF30Z"  =  "1"  /* Removal of Drainage Device from Abd Wall, Perc Approach */
"0WPF31Z"  =  "1"  /* Removal of Radioactive Element from Abd Wall, Perc Approach */
"0WPF33Z"  =  "1"  /* Removal of Infusion Device from Abd Wall, Perc Approach */
"0WPF37Z"  =  "1"  /* Removal of Autol Sub from Abd Wall, Perc Approach */
"0WPF3JZ"  =  "1"  /* Removal of Synthetic Substitute from Abd Wall, Perc Approach */
"0WPF3KZ"  =  "1"  /* Removal of Nonaut Sub from Abd Wall, Perc Approach */
"0WPF3YZ"  =  "1"  /* Removal of Other Device from Abdominal Wall, Perc Approach */
"0WPF40Z"  =  "1"  /* Removal of Drainage Device from Abd Wall, Perc Endo Approach */
"0WPF41Z"  =  "1"  /* Removal of Radioact Elem from Abd Wall, Perc Endo Approach */
"0WPF43Z"  =  "1"  /* Removal of Infusion Device from Abd Wall, Perc Endo Approach */
"0WPF47Z"  =  "1"  /* Removal of Autol Sub from Abd Wall, Perc Endo Approach */
"0WPF4JZ"  =  "1"  /* Removal of Synth Sub from Abd Wall, Perc Endo Approach */
"0WPF4KZ"  =  "1"  /* Removal of Nonaut Sub from Abd Wall, Perc Endo Approach */
"0WPF4YZ"  =  "1"  /* Removal of Other Device from Abd Wall, Perc Endo Approach */
"0WPG00Z"  =  "1"  /* Removal of Drainage Device from Periton Cav, Open Approach */
"0WPG01Z"  =  "1"  /* Removal of Radioact Elem from Periton Cav, Open Approach */
"0WPG03Z"  =  "1"  /* Removal of Infusion Device from Periton Cav, Open Approach */
"0WPG0JZ"  =  "1"  /* Removal of Synth Sub from Periton Cav, Open Approach */
"0WPG0YZ"  =  "1"  /* Removal of Other Device from Periton Cav, Open Approach */
"0WPG30Z"  =  "1"  /* Removal of Drainage Device from Periton Cav, Perc Approach */
"0WPG31Z"  =  "1"  /* Removal of Radioact Elem from Periton Cav, Perc Approach */
"0WPG33Z"  =  "1"  /* Removal of Infusion Device from Periton Cav, Perc Approach */
"0WPG3JZ"  =  "1"  /* Removal of Synth Sub from Periton Cav, Perc Approach */
"0WPG3YZ"  =  "1"  /* Removal of Other Device from Periton Cav, Perc Approach */
"0WPG40Z"  =  "1"  /* Removal of Drain Dev from Periton Cav, Perc Endo Approach */
"0WPG41Z"  =  "1"  /* Remove of Radioact Elem from Periton Cav, Perc Endo Approach */
"0WPG43Z"  =  "1"  /* Removal of Infusion Dev from Periton Cav, Perc Endo Approach */
"0WPG4JZ"  =  "1"  /* Removal of Synth Sub from Periton Cav, Perc Endo Approach */
"0WPG4YZ"  =  "1"  /* Removal of Other Device from Periton Cav, Perc Endo Approach */
"0WPH00Z"  =  "1"  /* Removal of Drain Dev from Retroperitoneum, Open Approach */
"0WPH01Z"  =  "1"  /* Removal of Radioact Elem from Retroperitoneum, Open Approach */
"0WPH03Z"  =  "1"  /* Removal of Infusion Dev from Retroperitoneum, Open Approach */
"0WPH0YZ"  =  "1"  /* Removal of Other Device from Retroperitoneum, Open Approach */
"0WPH30Z"  =  "1"  /* Removal of Drain Dev from Retroperitoneum, Perc Approach */
"0WPH31Z"  =  "1"  /* Removal of Radioact Elem from Retroperitoneum, Perc Approach */
"0WPH33Z"  =  "1"  /* Removal of Infusion Dev from Retroperitoneum, Perc Approach */
"0WPH3YZ"  =  "1"  /* Removal of Other Device from Retroperitoneum, Perc Approach */
"0WPH40Z"  =  "1"  /* Remove of Drain Dev from Retroperitoneum, Perc Endo Approach */
"0WPH41Z"  =  "1"  /* Remove Radioact Elem from Retroperitoneum, Perc Endo */
"0WPH43Z"  =  "1"  /* Remove Infusion Dev from Retroperitoneum, Perc Endo */
"0WPH4YZ"  =  "1"  /* Removal of Oth Dev from Retroperitoneum, Perc Endo Approach */
"0WPM07Z"  =  "1"  /* Removal of Autol Sub from Male Perineum, Open Approach */
"0WPM0KZ"  =  "1"  /* Removal of Nonaut Sub from Male Perineum, Open Approach */
"0WPM37Z"  =  "1"  /* Removal of Autol Sub from Male Perineum, Perc Approach */
"0WPM3KZ"  =  "1"  /* Removal of Nonaut Sub from Male Perineum, Perc Approach */
"0WPM47Z"  =  "1"  /* Removal of Autol Sub from Male Perineum, Perc Endo Approach */
"0WPM4KZ"  =  "1"  /* Removal of Nonaut Sub from Male Perineum, Perc Endo Approach */
"0WPMX7Z"  =  "1"  /* Removal of Autol Sub from Male Perineum, Extern Approach */
"0WPMXJZ"  =  "1"  /* Removal of Synth Sub from Male Perineum, Extern Approach */
"0WPMXKZ"  =  "1"  /* Removal of Nonaut Sub from Male Perineum, Extern Approach */
"0WPN00Z"  =  "1"  /* Removal of Drainage Device from Fem Perineum, Open Approach */
"0WPN01Z"  =  "1"  /* Removal of Radioact Elem from Fem Perineum, Open Approach */
"0WPN03Z"  =  "1"  /* Removal of Infusion Device from Fem Perineum, Open Approach */
"0WPN07Z"  =  "1"  /* Removal of Autol Sub from Fem Perineum, Open Approach */
"0WPN0JZ"  =  "1"  /* Removal of Synth Sub from Fem Perineum, Open Approach */
"0WPN0KZ"  =  "1"  /* Removal of Nonaut Sub from Fem Perineum, Open Approach */
"0WPN0YZ"  =  "1"  /* Removal of Other Device from Female Perineum, Open Approach */
"0WPN30Z"  =  "1"  /* Removal of Drainage Device from Fem Perineum, Perc Approach */
"0WPN31Z"  =  "1"  /* Removal of Radioact Elem from Fem Perineum, Perc Approach */
"0WPN33Z"  =  "1"  /* Removal of Infusion Device from Fem Perineum, Perc Approach */
"0WPN37Z"  =  "1"  /* Removal of Autol Sub from Fem Perineum, Perc Approach */
"0WPN3JZ"  =  "1"  /* Removal of Synth Sub from Fem Perineum, Perc Approach */
"0WPN3KZ"  =  "1"  /* Removal of Nonaut Sub from Fem Perineum, Perc Approach */
"0WPN3YZ"  =  "1"  /* Removal of Other Device from Female Perineum, Perc Approach */
"0WPN40Z"  =  "1"  /* Removal of Drain Dev from Fem Perineum, Perc Endo Approach */
"0WPN41Z"  =  "1"  /* Remove Radioact Elem from Fem Perineum, Perc Endo */
"0WPN43Z"  =  "1"  /* Remove of Infusion Dev from Fem Perineum, Perc Endo Approach */
"0WPN47Z"  =  "1"  /* Removal of Autol Sub from Fem Perineum, Perc Endo Approach */
"0WPN4JZ"  =  "1"  /* Removal of Synth Sub from Fem Perineum, Perc Endo Approach */
"0WPN4KZ"  =  "1"  /* Removal of Nonaut Sub from Fem Perineum, Perc Endo Approach */
"0WPN4YZ"  =  "1"  /* Removal of Oth Dev from Fem Perineum, Perc Endo Approach */
"0WPP01Z"  =  "1"  /* Removal of Radioactive Element from GI Tract, Open Approach */
"0WPP03Z"  =  "1"  /* Removal of Infusion Device from GI Tract, Open Approach */
"0WPP0YZ"  =  "1"  /* Removal of Other Device from GI Tract, Open Approach */
"0WPQ31Z"  =  "1"  /* Removal of Radioact Elem from Resp Tract, Perc Approach */
"0WPQ33Z"  =  "1"  /* Removal of Infusion Device from Resp Tract, Perc Approach */
"0WPQ3YZ"  =  "1"  /* Removal of Other Device from Resp Tract, Perc Approach */
"0WPQ41Z"  =  "1"  /* Removal of Radioact Elem from Resp Tract, Perc Endo Approach */
"0WPQ43Z"  =  "1"  /* Removal of Infusion Dev from Resp Tract, Perc Endo Approach */
"0WPQ4YZ"  =  "1"  /* Removal of Other Device from Resp Tract, Perc Endo Approach */
"0WPQ71Z"  =  "1"  /* Removal of Radioactive Element from Resp Tract, Via Opening */
"0WPQ73Z"  =  "1"  /* Removal of Infusion Device from Resp Tract, Via Opening */
"0WPQ7YZ"  =  "1"  /* Removal of Other Device from Respiratory Tract, Via Opening */
"0WPQ81Z"  =  "1"  /* Removal of Radioactive Element from Respiratory Tract, Endo */
"0WQ00ZZ"  =  "1"  /* Repair Head, Open Approach */
"0WQ03ZZ"  =  "1"  /* Repair Head, Percutaneous Approach */
"0WQ04ZZ"  =  "1"  /* Repair Head, Percutaneous Endoscopic Approach */
"0WQ0XZZ"  =  "1"  /* Repair Head, External Approach */
"0WQ20ZZ"  =  "1"  /* Repair Face, Open Approach */
"0WQ23ZZ"  =  "1"  /* Repair Face, Percutaneous Approach */
"0WQ24ZZ"  =  "1"  /* Repair Face, Percutaneous Endoscopic Approach */
"0WQ2XZZ"  =  "1"  /* Repair Face, External Approach */
"0WQ40ZZ"  =  "1"  /* Repair Upper Jaw, Open Approach */
"0WQ43ZZ"  =  "1"  /* Repair Upper Jaw, Percutaneous Approach */
"0WQ44ZZ"  =  "1"  /* Repair Upper Jaw, Percutaneous Endoscopic Approach */
"0WQ4XZZ"  =  "1"  /* Repair Upper Jaw, External Approach */
"0WQ50ZZ"  =  "1"  /* Repair Lower Jaw, Open Approach */
"0WQ53ZZ"  =  "1"  /* Repair Lower Jaw, Percutaneous Approach */
"0WQ54ZZ"  =  "1"  /* Repair Lower Jaw, Percutaneous Endoscopic Approach */
"0WQ5XZZ"  =  "1"  /* Repair Lower Jaw, External Approach */
"0WQ60ZZ"  =  "1"  /* Repair Neck, Open Approach */
"0WQ63ZZ"  =  "1"  /* Repair Neck, Percutaneous Approach */
"0WQ64ZZ"  =  "1"  /* Repair Neck, Percutaneous Endoscopic Approach */
"0WQ6XZ2"  =  "1"  /* Repair Neck, Stoma, External Approach */
"0WQ6XZZ"  =  "1"  /* Repair Neck, External Approach */
"0WQ80ZZ"  =  "1"  /* Repair Chest Wall, Open Approach */
"0WQ83ZZ"  =  "1"  /* Repair Chest Wall, Percutaneous Approach */
"0WQ84ZZ"  =  "1"  /* Repair Chest Wall, Percutaneous Endoscopic Approach */
"0WQ8XZZ"  =  "1"  /* Repair Chest Wall, External Approach */
"0WQC0ZZ"  =  "1"  /* Repair Mediastinum, Open Approach */
"0WQC3ZZ"  =  "1"  /* Repair Mediastinum, Percutaneous Approach */
"0WQC4ZZ"  =  "1"  /* Repair Mediastinum, Percutaneous Endoscopic Approach */
"0WQF0ZZ"  =  "1"  /* Repair Abdominal Wall, Open Approach */
"0WQF3ZZ"  =  "1"  /* Repair Abdominal Wall, Percutaneous Approach */
"0WQF4ZZ"  =  "1"  /* Repair Abdominal Wall, Percutaneous Endoscopic Approach */
"0WQFXZ2"  =  "1"  /* Repair Abdominal Wall, Stoma, External Approach */
"0WQFXZZ"  =  "1"  /* Repair Abdominal Wall, External Approach */
"0WQK0ZZ"  =  "1"  /* Repair Upper Back, Open Approach */
"0WQK3ZZ"  =  "1"  /* Repair Upper Back, Percutaneous Approach */
"0WQK4ZZ"  =  "1"  /* Repair Upper Back, Percutaneous Endoscopic Approach */
"0WQKXZZ"  =  "1"  /* Repair Upper Back, External Approach */
"0WQL0ZZ"  =  "1"  /* Repair Lower Back, Open Approach */
"0WQL3ZZ"  =  "1"  /* Repair Lower Back, Percutaneous Approach */
"0WQL4ZZ"  =  "1"  /* Repair Lower Back, Percutaneous Endoscopic Approach */
"0WQLXZZ"  =  "1"  /* Repair Lower Back, External Approach */
"0WQM0ZZ"  =  "1"  /* Repair Male Perineum, Open Approach */
"0WQM3ZZ"  =  "1"  /* Repair Male Perineum, Percutaneous Approach */
"0WQM4ZZ"  =  "1"  /* Repair Male Perineum, Percutaneous Endoscopic Approach */
"0WQMXZZ"  =  "1"  /* Repair Male Perineum, External Approach */
"0WQN0ZZ"  =  "1"  /* Repair Female Perineum, Open Approach */
"0WQN3ZZ"  =  "1"  /* Repair Female Perineum, Percutaneous Approach */
"0WQN4ZZ"  =  "1"  /* Repair Female Perineum, Percutaneous Endoscopic Approach */
"0WU007Z"  =  "1"  /* Supplement Head with Autol Sub, Open Approach */
"0WU00JZ"  =  "1"  /* Supplement Head with Synthetic Substitute, Open Approach */
"0WU00KZ"  =  "1"  /* Supplement Head with Nonaut Sub, Open Approach */
"0WU047Z"  =  "1"  /* Supplement Head with Autol Sub, Perc Endo Approach */
"0WU04JZ"  =  "1"  /* Supplement Head with Synth Sub, Perc Endo Approach */
"0WU04KZ"  =  "1"  /* Supplement Head with Nonaut Sub, Perc Endo Approach */
"0WU207Z"  =  "1"  /* Supplement Face with Autol Sub, Open Approach */
"0WU20JZ"  =  "1"  /* Supplement Face with Synthetic Substitute, Open Approach */
"0WU20KZ"  =  "1"  /* Supplement Face with Nonaut Sub, Open Approach */
"0WU247Z"  =  "1"  /* Supplement Face with Autol Sub, Perc Endo Approach */
"0WU24JZ"  =  "1"  /* Supplement Face with Synth Sub, Perc Endo Approach */
"0WU24KZ"  =  "1"  /* Supplement Face with Nonaut Sub, Perc Endo Approach */
"0WU407Z"  =  "1"  /* Supplement Upper Jaw with Autol Sub, Open Approach */
"0WU40JZ"  =  "1"  /* Supplement Upper Jaw with Synth Sub, Open Approach */
"0WU40KZ"  =  "1"  /* Supplement Upper Jaw with Nonaut Sub, Open Approach */
"0WU447Z"  =  "1"  /* Supplement Upper Jaw with Autol Sub, Perc Endo Approach */
"0WU44JZ"  =  "1"  /* Supplement Upper Jaw with Synth Sub, Perc Endo Approach */
"0WU44KZ"  =  "1"  /* Supplement Upper Jaw with Nonaut Sub, Perc Endo Approach */
"0WU507Z"  =  "1"  /* Supplement Lower Jaw with Autol Sub, Open Approach */
"0WU50JZ"  =  "1"  /* Supplement Lower Jaw with Synth Sub, Open Approach */
"0WU50KZ"  =  "1"  /* Supplement Lower Jaw with Nonaut Sub, Open Approach */
"0WU547Z"  =  "1"  /* Supplement Lower Jaw with Autol Sub, Perc Endo Approach */
"0WU54JZ"  =  "1"  /* Supplement Lower Jaw with Synth Sub, Perc Endo Approach */
"0WU54KZ"  =  "1"  /* Supplement Lower Jaw with Nonaut Sub, Perc Endo Approach */
"0WU607Z"  =  "1"  /* Supplement Neck with Autol Sub, Open Approach */
"0WU60JZ"  =  "1"  /* Supplement Neck with Synthetic Substitute, Open Approach */
"0WU60KZ"  =  "1"  /* Supplement Neck with Nonaut Sub, Open Approach */
"0WU647Z"  =  "1"  /* Supplement Neck with Autol Sub, Perc Endo Approach */
"0WU64JZ"  =  "1"  /* Supplement Neck with Synth Sub, Perc Endo Approach */
"0WU64KZ"  =  "1"  /* Supplement Neck with Nonaut Sub, Perc Endo Approach */
"0WU807Z"  =  "1"  /* Supplement Chest Wall with Autol Sub, Open Approach */
"0WU80JZ"  =  "1"  /* Supplement Chest Wall with Synth Sub, Open Approach */
"0WU80KZ"  =  "1"  /* Supplement Chest Wall with Nonaut Sub, Open Approach */
"0WU847Z"  =  "1"  /* Supplement Chest Wall with Autol Sub, Perc Endo Approach */
"0WU84JZ"  =  "1"  /* Supplement Chest Wall with Synth Sub, Perc Endo Approach */
"0WU84KZ"  =  "1"  /* Supplement Chest Wall with Nonaut Sub, Perc Endo Approach */
"0WUC07Z"  =  "1"  /* Supplement Mediastinum with Autol Sub, Open Approach */
"0WUC0JZ"  =  "1"  /* Supplement Mediastinum with Synth Sub, Open Approach */
"0WUC0KZ"  =  "1"  /* Supplement Mediastinum with Nonaut Sub, Open Approach */
"0WUC47Z"  =  "1"  /* Supplement Mediastinum with Autol Sub, Perc Endo Approach */
"0WUC4JZ"  =  "1"  /* Supplement Mediastinum with Synth Sub, Perc Endo Approach */
"0WUC4KZ"  =  "1"  /* Supplement Mediastinum with Nonaut Sub, Perc Endo Approach */
"0WUF07Z"  =  "1"  /* Supplement Abdominal Wall with Autol Sub, Open Approach */
"0WUF0JZ"  =  "1"  /* Supplement Abdominal Wall with Synth Sub, Open Approach */
"0WUF0KZ"  =  "1"  /* Supplement Abdominal Wall with Nonaut Sub, Open Approach */
"0WUF47Z"  =  "1"  /* Supplement Abdominal Wall with Autol Sub, Perc Endo Approach */
"0WUF4JZ"  =  "1"  /* Supplement Abdominal Wall with Synth Sub, Perc Endo Approach */
"0WUF4KZ"  =  "1"  /* Supplement Abd Wall with Nonaut Sub, Perc Endo Approach */
"0WUK07Z"  =  "1"  /* Supplement Upper Back with Autol Sub, Open Approach */
"0WUK0JZ"  =  "1"  /* Supplement Upper Back with Synth Sub, Open Approach */
"0WUK0KZ"  =  "1"  /* Supplement Upper Back with Nonaut Sub, Open Approach */
"0WUK47Z"  =  "1"  /* Supplement Upper Back with Autol Sub, Perc Endo Approach */
"0WUK4JZ"  =  "1"  /* Supplement Upper Back with Synth Sub, Perc Endo Approach */
"0WUK4KZ"  =  "1"  /* Supplement Upper Back with Nonaut Sub, Perc Endo Approach */
"0WUL07Z"  =  "1"  /* Supplement Lower Back with Autol Sub, Open Approach */
"0WUL0JZ"  =  "1"  /* Supplement Lower Back with Synth Sub, Open Approach */
"0WUL0KZ"  =  "1"  /* Supplement Lower Back with Nonaut Sub, Open Approach */
"0WUL47Z"  =  "1"  /* Supplement Lower Back with Autol Sub, Perc Endo Approach */
"0WUL4JZ"  =  "1"  /* Supplement Lower Back with Synth Sub, Perc Endo Approach */
"0WUL4KZ"  =  "1"  /* Supplement Lower Back with Nonaut Sub, Perc Endo Approach */
"0WUM07Z"  =  "1"  /* Supplement Male Perineum with Autol Sub, Open Approach */
"0WUM0JZ"  =  "1"  /* Supplement Male Perineum with Synth Sub, Open Approach */
"0WUM0KZ"  =  "1"  /* Supplement Male Perineum with Nonaut Sub, Open Approach */
"0WUM47Z"  =  "1"  /* Supplement Male Perineum with Autol Sub, Perc Endo Approach */
"0WUM4JZ"  =  "1"  /* Supplement Male Perineum with Synth Sub, Perc Endo Approach */
"0WUM4KZ"  =  "1"  /* Supplement Male Perineum with Nonaut Sub, Perc Endo Approach */
"0WUN07Z"  =  "1"  /* Supplement Female Perineum with Autol Sub, Open Approach */
"0WUN0JZ"  =  "1"  /* Supplement Female Perineum with Synth Sub, Open Approach */
"0WUN0KZ"  =  "1"  /* Supplement Female Perineum with Nonaut Sub, Open Approach */
"0WUN47Z"  =  "1"  /* Supplement Fem Perineum with Autol Sub, Perc Endo Approach */
"0WUN4JZ"  =  "1"  /* Supplement Fem Perineum with Synth Sub, Perc Endo Approach */
"0WUN4KZ"  =  "1"  /* Supplement Fem Perineum with Nonaut Sub, Perc Endo Approach */
"0WW100Z"  =  "1"  /* Revision of Drainage Device in Cranial Cavity, Open Approach */
"0WW101Z"  =  "1"  /* Revision of Radioact Elem in Cranial Cav, Open Approach */
"0WW103Z"  =  "1"  /* Revision of Infusion Device in Cranial Cavity, Open Approach */
"0WW10JZ"  =  "1"  /* Revision of Synth Sub in Cranial Cav, Open Approach */
"0WW10YZ"  =  "1"  /* Revision of Other Device in Cranial Cavity, Open Approach */
"0WW130Z"  =  "1"  /* Revision of Drainage Device in Cranial Cavity, Perc Approach */
"0WW131Z"  =  "1"  /* Revision of Radioact Elem in Cranial Cav, Perc Approach */
"0WW133Z"  =  "1"  /* Revision of Infusion Device in Cranial Cavity, Perc Approach */
"0WW13JZ"  =  "1"  /* Revision of Synth Sub in Cranial Cav, Perc Approach */
"0WW13YZ"  =  "1"  /* Revision of Other Device in Cranial Cavity, Perc Approach */
"0WW140Z"  =  "1"  /* Revision of Drain Dev in Cranial Cav, Perc Endo Approach */
"0WW141Z"  =  "1"  /* Revision of Radioact Elem in Cranial Cav, Perc Endo Approach */
"0WW143Z"  =  "1"  /* Revision of Infusion Dev in Cranial Cav, Perc Endo Approach */
"0WW14JZ"  =  "1"  /* Revision of Synth Sub in Cranial Cav, Perc Endo Approach */
"0WW14YZ"  =  "1"  /* Revision of Other Device in Cranial Cav, Perc Endo Approach */
"0WWC00Z"  =  "1"  /* Revision of Drainage Device in Mediastinum, Open Approach */
"0WWC01Z"  =  "1"  /* Revision of Radioact Elem in Mediastinum, Open Approach */
"0WWC03Z"  =  "1"  /* Revision of Infusion Device in Mediastinum, Open Approach */
"0WWC07Z"  =  "1"  /* Revision of Autol Sub in Mediastinum, Open Approach */
"0WWC0JZ"  =  "1"  /* Revision of Synth Sub in Mediastinum, Open Approach */
"0WWC0KZ"  =  "1"  /* Revision of Nonaut Sub in Mediastinum, Open Approach */
"0WWC0YZ"  =  "1"  /* Revision of Other Device in Mediastinum, Open Approach */
"0WWC30Z"  =  "1"  /* Revision of Drainage Device in Mediastinum, Perc Approach */
"0WWC31Z"  =  "1"  /* Revision of Radioact Elem in Mediastinum, Perc Approach */
"0WWC33Z"  =  "1"  /* Revision of Infusion Device in Mediastinum, Perc Approach */
"0WWC37Z"  =  "1"  /* Revision of Autol Sub in Mediastinum, Perc Approach */
"0WWC3JZ"  =  "1"  /* Revision of Synth Sub in Mediastinum, Perc Approach */
"0WWC3KZ"  =  "1"  /* Revision of Nonaut Sub in Mediastinum, Perc Approach */
"0WWC3YZ"  =  "1"  /* Revision of Other Device in Mediastinum, Perc Approach */
"0WWC40Z"  =  "1"  /* Revision of Drain Dev in Mediastinum, Perc Endo Approach */
"0WWC41Z"  =  "1"  /* Revision of Radioact Elem in Mediastinum, Perc Endo Approach */
"0WWC43Z"  =  "1"  /* Revision of Infusion Dev in Mediastinum, Perc Endo Approach */
"0WWC47Z"  =  "1"  /* Revision of Autol Sub in Mediastinum, Perc Endo Approach */
"0WWC4JZ"  =  "1"  /* Revision of Synth Sub in Mediastinum, Perc Endo Approach */
"0WWC4KZ"  =  "1"  /* Revision of Nonaut Sub in Mediastinum, Perc Endo Approach */
"0WWC4YZ"  =  "1"  /* Revision of Other Device in Mediastinum, Perc Endo Approach */
"0WWD00Z"  =  "1"  /* Revision of Drainage Device in Pericard Cav, Open Approach */
"0WWD01Z"  =  "1"  /* Revision of Radioact Elem in Pericard Cav, Open Approach */
"0WWD03Z"  =  "1"  /* Revision of Infusion Device in Pericard Cav, Open Approach */
"0WWD0YZ"  =  "1"  /* Revision of Other Device in Pericard Cav, Open Approach */
"0WWD30Z"  =  "1"  /* Revision of Drainage Device in Pericard Cav, Perc Approach */
"0WWD31Z"  =  "1"  /* Revision of Radioact Elem in Pericard Cav, Perc Approach */
"0WWD33Z"  =  "1"  /* Revision of Infusion Device in Pericard Cav, Perc Approach */
"0WWD3YZ"  =  "1"  /* Revision of Other Device in Pericard Cav, Perc Approach */
"0WWD40Z"  =  "1"  /* Revision of Drain Dev in Pericard Cav, Perc Endo Approach */
"0WWD41Z"  =  "1"  /* Revise of Radioact Elem in Pericard Cav, Perc Endo Approach */
"0WWD43Z"  =  "1"  /* Revision of Infusion Dev in Pericard Cav, Perc Endo Approach */
"0WWD4YZ"  =  "1"  /* Revision of Other Device in Pericard Cav, Perc Endo Approach */
"0WWF00Z"  =  "1"  /* Revision of Drainage Device in Abdominal Wall, Open Approach */
"0WWF01Z"  =  "1"  /* Revision of Radioactive Element in Abd Wall, Open Approach */
"0WWF03Z"  =  "1"  /* Revision of Infusion Device in Abdominal Wall, Open Approach */
"0WWF07Z"  =  "1"  /* Revision of Autol Sub in Abd Wall, Open Approach */
"0WWF0JZ"  =  "1"  /* Revision of Synthetic Substitute in Abd Wall, Open Approach */
"0WWF0KZ"  =  "1"  /* Revision of Nonaut Sub in Abd Wall, Open Approach */
"0WWF0YZ"  =  "1"  /* Revision of Other Device in Abdominal Wall, Open Approach */
"0WWF30Z"  =  "1"  /* Revision of Drainage Device in Abdominal Wall, Perc Approach */
"0WWF31Z"  =  "1"  /* Revision of Radioactive Element in Abd Wall, Perc Approach */
"0WWF33Z"  =  "1"  /* Revision of Infusion Device in Abdominal Wall, Perc Approach */
"0WWF37Z"  =  "1"  /* Revision of Autol Sub in Abd Wall, Perc Approach */
"0WWF3JZ"  =  "1"  /* Revision of Synthetic Substitute in Abd Wall, Perc Approach */
"0WWF3KZ"  =  "1"  /* Revision of Nonaut Sub in Abd Wall, Perc Approach */
"0WWF3YZ"  =  "1"  /* Revision of Other Device in Abdominal Wall, Perc Approach */
"0WWF40Z"  =  "1"  /* Revision of Drainage Device in Abd Wall, Perc Endo Approach */
"0WWF41Z"  =  "1"  /* Revision of Radioact Elem in Abd Wall, Perc Endo Approach */
"0WWF43Z"  =  "1"  /* Revision of Infusion Device in Abd Wall, Perc Endo Approach */
"0WWF47Z"  =  "1"  /* Revision of Autol Sub in Abd Wall, Perc Endo Approach */
"0WWF4JZ"  =  "1"  /* Revision of Synth Sub in Abd Wall, Perc Endo Approach */
"0WWF4KZ"  =  "1"  /* Revision of Nonaut Sub in Abd Wall, Perc Endo Approach */
"0WWF4YZ"  =  "1"  /* Revision of Other Device in Abd Wall, Perc Endo Approach */
"0WWG00Z"  =  "1"  /* Revision of Drainage Device in Periton Cav, Open Approach */
"0WWG01Z"  =  "1"  /* Revision of Radioact Elem in Periton Cav, Open Approach */
"0WWG03Z"  =  "1"  /* Revision of Infusion Device in Periton Cav, Open Approach */
"0WWG0JZ"  =  "1"  /* Revision of Synth Sub in Periton Cav, Open Approach */
"0WWG0YZ"  =  "1"  /* Revision of Other Device in Peritoneal Cavity, Open Approach */
"0WWG30Z"  =  "1"  /* Revision of Drainage Device in Periton Cav, Perc Approach */
"0WWG31Z"  =  "1"  /* Revision of Radioact Elem in Periton Cav, Perc Approach */
"0WWG33Z"  =  "1"  /* Revision of Infusion Device in Periton Cav, Perc Approach */
"0WWG3JZ"  =  "1"  /* Revision of Synth Sub in Periton Cav, Perc Approach */
"0WWG3YZ"  =  "1"  /* Revision of Other Device in Peritoneal Cavity, Perc Approach */
"0WWG40Z"  =  "1"  /* Revision of Drain Dev in Periton Cav, Perc Endo Approach */
"0WWG41Z"  =  "1"  /* Revision of Radioact Elem in Periton Cav, Perc Endo Approach */
"0WWG43Z"  =  "1"  /* Revision of Infusion Dev in Periton Cav, Perc Endo Approach */
"0WWG4JZ"  =  "1"  /* Revision of Synth Sub in Periton Cav, Perc Endo Approach */
"0WWG4YZ"  =  "1"  /* Revision of Other Device in Periton Cav, Perc Endo Approach */
"0WWH00Z"  =  "1"  /* Revision of Drain Dev in Retroperitoneum, Open Approach */
"0WWH01Z"  =  "1"  /* Revision of Radioact Elem in Retroperitoneum, Open Approach */
"0WWH03Z"  =  "1"  /* Revision of Infusion Dev in Retroperitoneum, Open Approach */
"0WWH0YZ"  =  "1"  /* Revision of Other Device in Retroperitoneum, Open Approach */
"0WWH30Z"  =  "1"  /* Revision of Drain Dev in Retroperitoneum, Perc Approach */
"0WWH31Z"  =  "1"  /* Revision of Radioact Elem in Retroperitoneum, Perc Approach */
"0WWH33Z"  =  "1"  /* Revision of Infusion Dev in Retroperitoneum, Perc Approach */
"0WWH3YZ"  =  "1"  /* Revision of Other Device in Retroperitoneum, Perc Approach */
"0WWH40Z"  =  "1"  /* Revision of Drain Dev in Retroperitoneum, Perc Endo Approach */
"0WWH41Z"  =  "1"  /* Revise Radioact Elem in Retroperitoneum, Perc Endo */
"0WWH43Z"  =  "1"  /* Revise Infusion Dev in Retroperitoneum, Perc Endo */
"0WWH4YZ"  =  "1"  /* Revision of Oth Dev in Retroperitoneum, Perc Endo Approach */
"0WWJ00Z"  =  "1"  /* Revision of Drainage Device in Pelvic Cavity, Open Approach */
"0WWJ01Z"  =  "1"  /* Revision of Radioactive Element in Pelvic Cav, Open Approach */
"0WWJ03Z"  =  "1"  /* Revision of Infusion Device in Pelvic Cavity, Open Approach */
"0WWJ0JZ"  =  "1"  /* Revision of Synth Sub in Pelvic Cav, Open Approach */
"0WWJ0YZ"  =  "1"  /* Revision of Other Device in Pelvic Cavity, Open Approach */
"0WWJ30Z"  =  "1"  /* Revision of Drainage Device in Pelvic Cavity, Perc Approach */
"0WWJ31Z"  =  "1"  /* Revision of Radioactive Element in Pelvic Cav, Perc Approach */
"0WWJ33Z"  =  "1"  /* Revision of Infusion Device in Pelvic Cavity, Perc Approach */
"0WWJ3JZ"  =  "1"  /* Revision of Synth Sub in Pelvic Cav, Perc Approach */
"0WWJ3YZ"  =  "1"  /* Revision of Other Device in Pelvic Cavity, Perc Approach */
"0WWJ40Z"  =  "1"  /* Revision of Drain Dev in Pelvic Cav, Perc Endo Approach */
"0WWJ41Z"  =  "1"  /* Revision of Radioact Elem in Pelvic Cav, Perc Endo Approach */
"0WWJ43Z"  =  "1"  /* Revision of Infusion Dev in Pelvic Cav, Perc Endo Approach */
"0WWJ4JZ"  =  "1"  /* Revision of Synth Sub in Pelvic Cav, Perc Endo Approach */
"0WWJ4YZ"  =  "1"  /* Revision of Other Device in Pelvic Cav, Perc Endo Approach */
"0WWM07Z"  =  "1"  /* Revision of Autol Sub in Male Perineum, Open Approach */
"0WWM0KZ"  =  "1"  /* Revision of Nonaut Sub in Male Perineum, Open Approach */
"0WWM37Z"  =  "1"  /* Revision of Autol Sub in Male Perineum, Perc Approach */
"0WWM3KZ"  =  "1"  /* Revision of Nonaut Sub in Male Perineum, Perc Approach */
"0WWM47Z"  =  "1"  /* Revision of Autol Sub in Male Perineum, Perc Endo Approach */
"0WWM4KZ"  =  "1"  /* Revision of Nonaut Sub in Male Perineum, Perc Endo Approach */
"0WWN00Z"  =  "1"  /* Revision of Drainage Device in Fem Perineum, Open Approach */
"0WWN01Z"  =  "1"  /* Revision of Radioact Elem in Fem Perineum, Open Approach */
"0WWN03Z"  =  "1"  /* Revision of Infusion Device in Fem Perineum, Open Approach */
"0WWN07Z"  =  "1"  /* Revision of Autol Sub in Fem Perineum, Open Approach */
"0WWN0JZ"  =  "1"  /* Revision of Synth Sub in Fem Perineum, Open Approach */
"0WWN0KZ"  =  "1"  /* Revision of Nonaut Sub in Fem Perineum, Open Approach */
"0WWN0YZ"  =  "1"  /* Revision of Other Device in Female Perineum, Open Approach */
"0WWN30Z"  =  "1"  /* Revision of Drainage Device in Fem Perineum, Perc Approach */
"0WWN31Z"  =  "1"  /* Revision of Radioact Elem in Fem Perineum, Perc Approach */
"0WWN33Z"  =  "1"  /* Revision of Infusion Device in Fem Perineum, Perc Approach */
"0WWN37Z"  =  "1"  /* Revision of Autol Sub in Fem Perineum, Perc Approach */
"0WWN3JZ"  =  "1"  /* Revision of Synth Sub in Fem Perineum, Perc Approach */
"0WWN3KZ"  =  "1"  /* Revision of Nonaut Sub in Fem Perineum, Perc Approach */
"0WWN3YZ"  =  "1"  /* Revision of Other Device in Female Perineum, Perc Approach */
"0WWN40Z"  =  "1"  /* Revision of Drain Dev in Fem Perineum, Perc Endo Approach */
"0WWN41Z"  =  "1"  /* Revise of Radioact Elem in Fem Perineum, Perc Endo Approach */
"0WWN43Z"  =  "1"  /* Revision of Infusion Dev in Fem Perineum, Perc Endo Approach */
"0WWN47Z"  =  "1"  /* Revision of Autol Sub in Fem Perineum, Perc Endo Approach */
"0WWN4JZ"  =  "1"  /* Revision of Synth Sub in Fem Perineum, Perc Endo Approach */
"0WWN4KZ"  =  "1"  /* Revision of Nonaut Sub in Fem Perineum, Perc Endo Approach */
"0WWN4YZ"  =  "1"  /* Revision of Other Device in Fem Perineum, Perc Endo Approach */
"0WWP01Z"  =  "1"  /* Revision of Radioactive Element in GI Tract, Open Approach */
"0WWP03Z"  =  "1"  /* Revision of Infusion Device in GI Tract, Open Approach */
"0WWP0YZ"  =  "1"  /* Revision of Other Device in GI Tract, Open Approach */
"0WWQ31Z"  =  "1"  /* Revision of Radioactive Element in Resp Tract, Perc Approach */
"0WWQ33Z"  =  "1"  /* Revision of Infusion Device in Resp Tract, Perc Approach */
"0WWQ3YZ"  =  "1"  /* Revision of Other Device in Respiratory Tract, Perc Approach */
"0WWQ41Z"  =  "1"  /* Revision of Radioact Elem in Resp Tract, Perc Endo Approach */
"0WWQ43Z"  =  "1"  /* Revision of Infusion Dev in Resp Tract, Perc Endo Approach */
"0WWQ4YZ"  =  "1"  /* Revision of Other Device in Resp Tract, Perc Endo Approach */
"0WWQ71Z"  =  "1"  /* Revision of Radioactive Element in Resp Tract, Via Opening */
"0WWQ73Z"  =  "1"  /* Revision of Infusion Device in Resp Tract, Via Opening */
"0WWQ7YZ"  =  "1"  /* Revision of Other Device in Respiratory Tract, Via Opening */
"0WWQ81Z"  =  "1"  /* Revision of Radioactive Element in Respiratory Tract, Endo */
"0WWQ83Z"  =  "1"  /* Revision of Infusion Device in Respiratory Tract, Endo */
"0WWQ8YZ"  =  "1"  /* Revision of Other Device in Respiratory Tract, Endo */
"0X0207Z"  =  "1"  /* Alteration of R Shoulder with Autol Sub, Open Approach */
"0X020JZ"  =  "1"  /* Alteration of R Shoulder with Synth Sub, Open Approach */
"0X020KZ"  =  "1"  /* Alteration of R Shoulder with Nonaut Sub, Open Approach */
"0X020ZZ"  =  "1"  /* Alteration of Right Shoulder Region, Open Approach */
"0X0237Z"  =  "1"  /* Alteration of R Shoulder with Autol Sub, Perc Approach */
"0X023JZ"  =  "1"  /* Alteration of R Shoulder with Synth Sub, Perc Approach */
"0X023KZ"  =  "1"  /* Alteration of R Shoulder with Nonaut Sub, Perc Approach */
"0X023ZZ"  =  "1"  /* Alteration of Right Shoulder Region, Percutaneous Approach */
"0X0247Z"  =  "1"  /* Alteration of R Shoulder with Autol Sub, Perc Endo Approach */
"0X024JZ"  =  "1"  /* Alteration of R Shoulder with Synth Sub, Perc Endo Approach */
"0X024KZ"  =  "1"  /* Alteration of R Shoulder with Nonaut Sub, Perc Endo Approach */
"0X024ZZ"  =  "1"  /* Alteration of Right Shoulder Region, Perc Endo Approach */
"0X0307Z"  =  "1"  /* Alteration of L Shoulder with Autol Sub, Open Approach */
"0X030JZ"  =  "1"  /* Alteration of L Shoulder with Synth Sub, Open Approach */
"0X030KZ"  =  "1"  /* Alteration of L Shoulder with Nonaut Sub, Open Approach */
"0X030ZZ"  =  "1"  /* Alteration of Left Shoulder Region, Open Approach */
"0X0337Z"  =  "1"  /* Alteration of L Shoulder with Autol Sub, Perc Approach */
"0X033JZ"  =  "1"  /* Alteration of L Shoulder with Synth Sub, Perc Approach */
"0X033KZ"  =  "1"  /* Alteration of L Shoulder with Nonaut Sub, Perc Approach */
"0X033ZZ"  =  "1"  /* Alteration of Left Shoulder Region, Percutaneous Approach */
"0X0347Z"  =  "1"  /* Alteration of L Shoulder with Autol Sub, Perc Endo Approach */
"0X034JZ"  =  "1"  /* Alteration of L Shoulder with Synth Sub, Perc Endo Approach */
"0X034KZ"  =  "1"  /* Alteration of L Shoulder with Nonaut Sub, Perc Endo Approach */
"0X034ZZ"  =  "1"  /* Alteration of Left Shoulder Region, Perc Endo Approach */
"0X0407Z"  =  "1"  /* Alteration of Right Axilla with Autol Sub, Open Approach */
"0X040JZ"  =  "1"  /* Alteration of Right Axilla with Synth Sub, Open Approach */
"0X040KZ"  =  "1"  /* Alteration of Right Axilla with Nonaut Sub, Open Approach */
"0X040ZZ"  =  "1"  /* Alteration of Right Axilla, Open Approach */
"0X0437Z"  =  "1"  /* Alteration of Right Axilla with Autol Sub, Perc Approach */
"0X043JZ"  =  "1"  /* Alteration of Right Axilla with Synth Sub, Perc Approach */
"0X043KZ"  =  "1"  /* Alteration of Right Axilla with Nonaut Sub, Perc Approach */
"0X043ZZ"  =  "1"  /* Alteration of Right Axilla, Percutaneous Approach */
"0X0447Z"  =  "1"  /* Alteration of R Axilla with Autol Sub, Perc Endo Approach */
"0X044JZ"  =  "1"  /* Alteration of R Axilla with Synth Sub, Perc Endo Approach */
"0X044KZ"  =  "1"  /* Alteration of R Axilla with Nonaut Sub, Perc Endo Approach */
"0X044ZZ"  =  "1"  /* Alteration of Right Axilla, Percutaneous Endoscopic Approach */
"0X0507Z"  =  "1"  /* Alteration of Left Axilla with Autol Sub, Open Approach */
"0X050JZ"  =  "1"  /* Alteration of Left Axilla with Synth Sub, Open Approach */
"0X050KZ"  =  "1"  /* Alteration of Left Axilla with Nonaut Sub, Open Approach */
"0X050ZZ"  =  "1"  /* Alteration of Left Axilla, Open Approach */
"0X0537Z"  =  "1"  /* Alteration of Left Axilla with Autol Sub, Perc Approach */
"0X053JZ"  =  "1"  /* Alteration of Left Axilla with Synth Sub, Perc Approach */
"0X053KZ"  =  "1"  /* Alteration of Left Axilla with Nonaut Sub, Perc Approach */
"0X053ZZ"  =  "1"  /* Alteration of Left Axilla, Percutaneous Approach */
"0X0547Z"  =  "1"  /* Alteration of Left Axilla with Autol Sub, Perc Endo Approach */
"0X054JZ"  =  "1"  /* Alteration of Left Axilla with Synth Sub, Perc Endo Approach */
"0X054KZ"  =  "1"  /* Alteration of L Axilla with Nonaut Sub, Perc Endo Approach */
"0X054ZZ"  =  "1"  /* Alteration of Left Axilla, Percutaneous Endoscopic Approach */
"0X0607Z"  =  "1"  /* Alteration of R Up Extrem with Autol Sub, Open Approach */
"0X060JZ"  =  "1"  /* Alteration of R Up Extrem with Synth Sub, Open Approach */
"0X060KZ"  =  "1"  /* Alteration of R Up Extrem with Nonaut Sub, Open Approach */
"0X060ZZ"  =  "1"  /* Alteration of Right Upper Extremity, Open Approach */
"0X0637Z"  =  "1"  /* Alteration of R Up Extrem with Autol Sub, Perc Approach */
"0X063JZ"  =  "1"  /* Alteration of R Up Extrem with Synth Sub, Perc Approach */
"0X063KZ"  =  "1"  /* Alteration of R Up Extrem with Nonaut Sub, Perc Approach */
"0X063ZZ"  =  "1"  /* Alteration of Right Upper Extremity, Percutaneous Approach */
"0X0647Z"  =  "1"  /* Alteration of R Up Extrem with Autol Sub, Perc Endo Approach */
"0X064JZ"  =  "1"  /* Alteration of R Up Extrem with Synth Sub, Perc Endo Approach */
"0X064KZ"  =  "1"  /* Alter of R Up Extrem with Nonaut Sub, Perc Endo Approach */
"0X064ZZ"  =  "1"  /* Alteration of Right Upper Extremity, Perc Endo Approach */
"0X0707Z"  =  "1"  /* Alteration of L Up Extrem with Autol Sub, Open Approach */
"0X070JZ"  =  "1"  /* Alteration of L Up Extrem with Synth Sub, Open Approach */
"0X070KZ"  =  "1"  /* Alteration of L Up Extrem with Nonaut Sub, Open Approach */
"0X070ZZ"  =  "1"  /* Alteration of Left Upper Extremity, Open Approach */
"0X0737Z"  =  "1"  /* Alteration of L Up Extrem with Autol Sub, Perc Approach */
"0X073JZ"  =  "1"  /* Alteration of L Up Extrem with Synth Sub, Perc Approach */
"0X073KZ"  =  "1"  /* Alteration of L Up Extrem with Nonaut Sub, Perc Approach */
"0X073ZZ"  =  "1"  /* Alteration of Left Upper Extremity, Percutaneous Approach */
"0X0747Z"  =  "1"  /* Alteration of L Up Extrem with Autol Sub, Perc Endo Approach */
"0X074JZ"  =  "1"  /* Alteration of L Up Extrem with Synth Sub, Perc Endo Approach */
"0X074KZ"  =  "1"  /* Alter of L Up Extrem with Nonaut Sub, Perc Endo Approach */
"0X074ZZ"  =  "1"  /* Alteration of Left Upper Extremity, Perc Endo Approach */
"0X0807Z"  =  "1"  /* Alteration of Right Upper Arm with Autol Sub, Open Approach */
"0X080JZ"  =  "1"  /* Alteration of Right Upper Arm with Synth Sub, Open Approach */
"0X080KZ"  =  "1"  /* Alteration of Right Upper Arm with Nonaut Sub, Open Approach */
"0X080ZZ"  =  "1"  /* Alteration of Right Upper Arm, Open Approach */
"0X0837Z"  =  "1"  /* Alteration of Right Upper Arm with Autol Sub, Perc Approach */
"0X083JZ"  =  "1"  /* Alteration of Right Upper Arm with Synth Sub, Perc Approach */
"0X083KZ"  =  "1"  /* Alteration of Right Upper Arm with Nonaut Sub, Perc Approach */
"0X083ZZ"  =  "1"  /* Alteration of Right Upper Arm, Percutaneous Approach */
"0X0847Z"  =  "1"  /* Alteration of R Up Arm with Autol Sub, Perc Endo Approach */
"0X084JZ"  =  "1"  /* Alteration of R Up Arm with Synth Sub, Perc Endo Approach */
"0X084KZ"  =  "1"  /* Alteration of R Up Arm with Nonaut Sub, Perc Endo Approach */
"0X084ZZ"  =  "1"  /* Alteration of Right Upper Arm, Perc Endo Approach */
"0X0907Z"  =  "1"  /* Alteration of Left Upper Arm with Autol Sub, Open Approach */
"0X090JZ"  =  "1"  /* Alteration of Left Upper Arm with Synth Sub, Open Approach */
"0X090KZ"  =  "1"  /* Alteration of Left Upper Arm with Nonaut Sub, Open Approach */
"0X090ZZ"  =  "1"  /* Alteration of Left Upper Arm, Open Approach */
"0X0937Z"  =  "1"  /* Alteration of Left Upper Arm with Autol Sub, Perc Approach */
"0X093JZ"  =  "1"  /* Alteration of Left Upper Arm with Synth Sub, Perc Approach */
"0X093KZ"  =  "1"  /* Alteration of Left Upper Arm with Nonaut Sub, Perc Approach */
"0X093ZZ"  =  "1"  /* Alteration of Left Upper Arm, Percutaneous Approach */
"0X0947Z"  =  "1"  /* Alteration of L Up Arm with Autol Sub, Perc Endo Approach */
"0X094JZ"  =  "1"  /* Alteration of L Up Arm with Synth Sub, Perc Endo Approach */
"0X094KZ"  =  "1"  /* Alteration of L Up Arm with Nonaut Sub, Perc Endo Approach */
"0X094ZZ"  =  "1"  /* Alteration of Left Upper Arm, Perc Endo Approach */
"0X0B07Z"  =  "1"  /* Alteration of R Elbow with Autol Sub, Open Approach */
"0X0B0JZ"  =  "1"  /* Alteration of R Elbow with Synth Sub, Open Approach */
"0X0B0KZ"  =  "1"  /* Alteration of R Elbow with Nonaut Sub, Open Approach */
"0X0B0ZZ"  =  "1"  /* Alteration of Right Elbow Region, Open Approach */
"0X0B37Z"  =  "1"  /* Alteration of R Elbow with Autol Sub, Perc Approach */
"0X0B3JZ"  =  "1"  /* Alteration of R Elbow with Synth Sub, Perc Approach */
"0X0B3KZ"  =  "1"  /* Alteration of R Elbow with Nonaut Sub, Perc Approach */
"0X0B3ZZ"  =  "1"  /* Alteration of Right Elbow Region, Percutaneous Approach */
"0X0B47Z"  =  "1"  /* Alteration of R Elbow with Autol Sub, Perc Endo Approach */
"0X0B4JZ"  =  "1"  /* Alteration of R Elbow with Synth Sub, Perc Endo Approach */
"0X0B4KZ"  =  "1"  /* Alteration of R Elbow with Nonaut Sub, Perc Endo Approach */
"0X0B4ZZ"  =  "1"  /* Alteration of Right Elbow Region, Perc Endo Approach */
"0X0C07Z"  =  "1"  /* Alteration of L Elbow with Autol Sub, Open Approach */
"0X0C0JZ"  =  "1"  /* Alteration of L Elbow with Synth Sub, Open Approach */
"0X0C0KZ"  =  "1"  /* Alteration of L Elbow with Nonaut Sub, Open Approach */
"0X0C0ZZ"  =  "1"  /* Alteration of Left Elbow Region, Open Approach */
"0X0C37Z"  =  "1"  /* Alteration of L Elbow with Autol Sub, Perc Approach */
"0X0C3JZ"  =  "1"  /* Alteration of L Elbow with Synth Sub, Perc Approach */
"0X0C3KZ"  =  "1"  /* Alteration of L Elbow with Nonaut Sub, Perc Approach */
"0X0C3ZZ"  =  "1"  /* Alteration of Left Elbow Region, Percutaneous Approach */
"0X0C47Z"  =  "1"  /* Alteration of L Elbow with Autol Sub, Perc Endo Approach */
"0X0C4JZ"  =  "1"  /* Alteration of L Elbow with Synth Sub, Perc Endo Approach */
"0X0C4KZ"  =  "1"  /* Alteration of L Elbow with Nonaut Sub, Perc Endo Approach */
"0X0C4ZZ"  =  "1"  /* Alteration of Left Elbow Region, Perc Endo Approach */
"0X0D07Z"  =  "1"  /* Alteration of Right Lower Arm with Autol Sub, Open Approach */
"0X0D0JZ"  =  "1"  /* Alteration of Right Lower Arm with Synth Sub, Open Approach */
"0X0D0KZ"  =  "1"  /* Alteration of Right Lower Arm with Nonaut Sub, Open Approach */
"0X0D0ZZ"  =  "1"  /* Alteration of Right Lower Arm, Open Approach */
"0X0D37Z"  =  "1"  /* Alteration of Right Lower Arm with Autol Sub, Perc Approach */
"0X0D3JZ"  =  "1"  /* Alteration of Right Lower Arm with Synth Sub, Perc Approach */
"0X0D3KZ"  =  "1"  /* Alteration of Right Lower Arm with Nonaut Sub, Perc Approach */
"0X0D3ZZ"  =  "1"  /* Alteration of Right Lower Arm, Percutaneous Approach */
"0X0D47Z"  =  "1"  /* Alteration of R Low Arm with Autol Sub, Perc Endo Approach */
"0X0D4JZ"  =  "1"  /* Alteration of R Low Arm with Synth Sub, Perc Endo Approach */
"0X0D4KZ"  =  "1"  /* Alteration of R Low Arm with Nonaut Sub, Perc Endo Approach */
"0X0D4ZZ"  =  "1"  /* Alteration of Right Lower Arm, Perc Endo Approach */
"0X0F07Z"  =  "1"  /* Alteration of Left Lower Arm with Autol Sub, Open Approach */
"0X0F0JZ"  =  "1"  /* Alteration of Left Lower Arm with Synth Sub, Open Approach */
"0X0F0KZ"  =  "1"  /* Alteration of Left Lower Arm with Nonaut Sub, Open Approach */
"0X0F0ZZ"  =  "1"  /* Alteration of Left Lower Arm, Open Approach */
"0X0F37Z"  =  "1"  /* Alteration of Left Lower Arm with Autol Sub, Perc Approach */
"0X0F3JZ"  =  "1"  /* Alteration of Left Lower Arm with Synth Sub, Perc Approach */
"0X0F3KZ"  =  "1"  /* Alteration of Left Lower Arm with Nonaut Sub, Perc Approach */
"0X0F3ZZ"  =  "1"  /* Alteration of Left Lower Arm, Percutaneous Approach */
"0X0F47Z"  =  "1"  /* Alteration of L Low Arm with Autol Sub, Perc Endo Approach */
"0X0F4JZ"  =  "1"  /* Alteration of L Low Arm with Synth Sub, Perc Endo Approach */
"0X0F4KZ"  =  "1"  /* Alteration of L Low Arm with Nonaut Sub, Perc Endo Approach */
"0X0F4ZZ"  =  "1"  /* Alteration of Left Lower Arm, Perc Endo Approach */
"0X0G07Z"  =  "1"  /* Alteration of R Wrist with Autol Sub, Open Approach */
"0X0G0JZ"  =  "1"  /* Alteration of R Wrist with Synth Sub, Open Approach */
"0X0G0KZ"  =  "1"  /* Alteration of R Wrist with Nonaut Sub, Open Approach */
"0X0G0ZZ"  =  "1"  /* Alteration of Right Wrist Region, Open Approach */
"0X0G37Z"  =  "1"  /* Alteration of R Wrist with Autol Sub, Perc Approach */
"0X0G3JZ"  =  "1"  /* Alteration of R Wrist with Synth Sub, Perc Approach */
"0X0G3KZ"  =  "1"  /* Alteration of R Wrist with Nonaut Sub, Perc Approach */
"0X0G3ZZ"  =  "1"  /* Alteration of Right Wrist Region, Percutaneous Approach */
"0X0G47Z"  =  "1"  /* Alteration of R Wrist with Autol Sub, Perc Endo Approach */
"0X0G4JZ"  =  "1"  /* Alteration of R Wrist with Synth Sub, Perc Endo Approach */
"0X0G4KZ"  =  "1"  /* Alteration of R Wrist with Nonaut Sub, Perc Endo Approach */
"0X0G4ZZ"  =  "1"  /* Alteration of Right Wrist Region, Perc Endo Approach */
"0X0H07Z"  =  "1"  /* Alteration of L Wrist with Autol Sub, Open Approach */
"0X0H0JZ"  =  "1"  /* Alteration of L Wrist with Synth Sub, Open Approach */
"0X0H0KZ"  =  "1"  /* Alteration of L Wrist with Nonaut Sub, Open Approach */
"0X0H0ZZ"  =  "1"  /* Alteration of Left Wrist Region, Open Approach */
"0X0H37Z"  =  "1"  /* Alteration of L Wrist with Autol Sub, Perc Approach */
"0X0H3JZ"  =  "1"  /* Alteration of L Wrist with Synth Sub, Perc Approach */
"0X0H3KZ"  =  "1"  /* Alteration of L Wrist with Nonaut Sub, Perc Approach */
"0X0H3ZZ"  =  "1"  /* Alteration of Left Wrist Region, Percutaneous Approach */
"0X0H47Z"  =  "1"  /* Alteration of L Wrist with Autol Sub, Perc Endo Approach */
"0X0H4JZ"  =  "1"  /* Alteration of L Wrist with Synth Sub, Perc Endo Approach */
"0X0H4KZ"  =  "1"  /* Alteration of L Wrist with Nonaut Sub, Perc Endo Approach */
"0X0H4ZZ"  =  "1"  /* Alteration of Left Wrist Region, Perc Endo Approach */
"0X320ZZ"  =  "1"  /* Control Bleeding in Right Shoulder Region, Open Approach */
"0X323ZZ"  =  "1"  /* Control Bleeding in Right Shoulder Region, Perc Approach */
"0X324ZZ"  =  "1"  /* Control Bleeding in R Shoulder, Perc Endo Approach */
"0X330ZZ"  =  "1"  /* Control Bleeding in Left Shoulder Region, Open Approach */
"0X333ZZ"  =  "1"  /* Control Bleeding in Left Shoulder Region, Perc Approach */
"0X334ZZ"  =  "1"  /* Control Bleeding in Left Shoulder Region, Perc Endo Approach */
"0X340ZZ"  =  "1"  /* Control Bleeding in Right Axilla, Open Approach */
"0X343ZZ"  =  "1"  /* Control Bleeding in Right Axilla, Percutaneous Approach */
"0X344ZZ"  =  "1"  /* Control Bleeding in Right Axilla, Perc Endo Approach */
"0X350ZZ"  =  "1"  /* Control Bleeding in Left Axilla, Open Approach */
"0X353ZZ"  =  "1"  /* Control Bleeding in Left Axilla, Percutaneous Approach */
"0X354ZZ"  =  "1"  /* Control Bleeding in Left Axilla, Perc Endo Approach */
"0X360ZZ"  =  "1"  /* Control Bleeding in Right Upper Extremity, Open Approach */
"0X363ZZ"  =  "1"  /* Control Bleeding in Right Upper Extremity, Perc Approach */
"0X364ZZ"  =  "1"  /* Control Bleeding in R Up Extrem, Perc Endo Approach */
"0X370ZZ"  =  "1"  /* Control Bleeding in Left Upper Extremity, Open Approach */
"0X373ZZ"  =  "1"  /* Control Bleeding in Left Upper Extremity, Perc Approach */
"0X374ZZ"  =  "1"  /* Control Bleeding in Left Upper Extremity, Perc Endo Approach */
"0X380ZZ"  =  "1"  /* Control Bleeding in Right Upper Arm, Open Approach */
"0X383ZZ"  =  "1"  /* Control Bleeding in Right Upper Arm, Percutaneous Approach */
"0X384ZZ"  =  "1"  /* Control Bleeding in Right Upper Arm, Perc Endo Approach */
"0X390ZZ"  =  "1"  /* Control Bleeding in Left Upper Arm, Open Approach */
"0X393ZZ"  =  "1"  /* Control Bleeding in Left Upper Arm, Percutaneous Approach */
"0X394ZZ"  =  "1"  /* Control Bleeding in Left Upper Arm, Perc Endo Approach */
"0X3B0ZZ"  =  "1"  /* Control Bleeding in Right Elbow Region, Open Approach */
"0X3B3ZZ"  =  "1"  /* Control Bleeding in Right Elbow Region, Perc Approach */
"0X3B4ZZ"  =  "1"  /* Control Bleeding in Right Elbow Region, Perc Endo Approach */
"0X3C0ZZ"  =  "1"  /* Control Bleeding in Left Elbow Region, Open Approach */
"0X3C3ZZ"  =  "1"  /* Control Bleeding in Left Elbow Region, Percutaneous Approach */
"0X3C4ZZ"  =  "1"  /* Control Bleeding in Left Elbow Region, Perc Endo Approach */
"0X3D0ZZ"  =  "1"  /* Control Bleeding in Right Lower Arm, Open Approach */
"0X3D3ZZ"  =  "1"  /* Control Bleeding in Right Lower Arm, Percutaneous Approach */
"0X3D4ZZ"  =  "1"  /* Control Bleeding in Right Lower Arm, Perc Endo Approach */
"0X3F0ZZ"  =  "1"  /* Control Bleeding in Left Lower Arm, Open Approach */
"0X3F3ZZ"  =  "1"  /* Control Bleeding in Left Lower Arm, Percutaneous Approach */
"0X3F4ZZ"  =  "1"  /* Control Bleeding in Left Lower Arm, Perc Endo Approach */
"0X3G0ZZ"  =  "1"  /* Control Bleeding in Right Wrist Region, Open Approach */
"0X3G3ZZ"  =  "1"  /* Control Bleeding in Right Wrist Region, Perc Approach */
"0X3G4ZZ"  =  "1"  /* Control Bleeding in Right Wrist Region, Perc Endo Approach */
"0X3H0ZZ"  =  "1"  /* Control Bleeding in Left Wrist Region, Open Approach */
"0X3H3ZZ"  =  "1"  /* Control Bleeding in Left Wrist Region, Percutaneous Approach */
"0X3H4ZZ"  =  "1"  /* Control Bleeding in Left Wrist Region, Perc Endo Approach */
"0X3J0ZZ"  =  "1"  /* Control Bleeding in Right Hand, Open Approach */
"0X3J3ZZ"  =  "1"  /* Control Bleeding in Right Hand, Percutaneous Approach */
"0X3J4ZZ"  =  "1"  /* Control Bleeding in Right Hand, Perc Endo Approach */
"0X3K0ZZ"  =  "1"  /* Control Bleeding in Left Hand, Open Approach */
"0X3K3ZZ"  =  "1"  /* Control Bleeding in Left Hand, Percutaneous Approach */
"0X3K4ZZ"  =  "1"  /* Control Bleeding in Left Hand, Perc Endo Approach */
"0X600ZZ"  =  "1"  /* Detachment at Right Forequarter, Open Approach */
"0X610ZZ"  =  "1"  /* Detachment at Left Forequarter, Open Approach */
"0X620ZZ"  =  "1"  /* Detachment at Right Shoulder Region, Open Approach */
"0X630ZZ"  =  "1"  /* Detachment at Left Shoulder Region, Open Approach */
"0X680Z1"  =  "1"  /* Detachment at Right Upper Arm, High, Open Approach */
"0X680Z2"  =  "1"  /* Detachment at Right Upper Arm, Mid, Open Approach */
"0X680Z3"  =  "1"  /* Detachment at Right Upper Arm, Low, Open Approach */
"0X690Z1"  =  "1"  /* Detachment at Left Upper Arm, High, Open Approach */
"0X690Z2"  =  "1"  /* Detachment at Left Upper Arm, Mid, Open Approach */
"0X690Z3"  =  "1"  /* Detachment at Left Upper Arm, Low, Open Approach */
"0X6B0ZZ"  =  "1"  /* Detachment at Right Elbow Region, Open Approach */
"0X6C0ZZ"  =  "1"  /* Detachment at Left Elbow Region, Open Approach */
"0X6D0Z1"  =  "1"  /* Detachment at Right Lower Arm, High, Open Approach */
"0X6D0Z2"  =  "1"  /* Detachment at Right Lower Arm, Mid, Open Approach */
"0X6D0Z3"  =  "1"  /* Detachment at Right Lower Arm, Low, Open Approach */
"0X6F0Z1"  =  "1"  /* Detachment at Left Lower Arm, High, Open Approach */
"0X6F0Z2"  =  "1"  /* Detachment at Left Lower Arm, Mid, Open Approach */
"0X6F0Z3"  =  "1"  /* Detachment at Left Lower Arm, Low, Open Approach */
"0X6J0Z0"  =  "1"  /* Detachment at Right Hand, Complete, Open Approach */
"0X6J0Z4"  =  "1"  /* Detachment at Right Hand, Complete 1st Ray, Open Approach */
"0X6J0Z5"  =  "1"  /* Detachment at Right Hand, Complete 2nd Ray, Open Approach */
"0X6J0Z6"  =  "1"  /* Detachment at Right Hand, Complete 3rd Ray, Open Approach */
"0X6J0Z7"  =  "1"  /* Detachment at Right Hand, Complete 4th Ray, Open Approach */
"0X6J0Z8"  =  "1"  /* Detachment at Right Hand, Complete 5th Ray, Open Approach */
"0X6J0Z9"  =  "1"  /* Detachment at Right Hand, Partial 1st Ray, Open Approach */
"0X6J0ZB"  =  "1"  /* Detachment at Right Hand, Partial 2nd Ray, Open Approach */
"0X6J0ZC"  =  "1"  /* Detachment at Right Hand, Partial 3rd Ray, Open Approach */
"0X6J0ZD"  =  "1"  /* Detachment at Right Hand, Partial 4th Ray, Open Approach */
"0X6J0ZF"  =  "1"  /* Detachment at Right Hand, Partial 5th Ray, Open Approach */
"0X6K0Z0"  =  "1"  /* Detachment at Left Hand, Complete, Open Approach */
"0X6K0Z4"  =  "1"  /* Detachment at Left Hand, Complete 1st Ray, Open Approach */
"0X6K0Z5"  =  "1"  /* Detachment at Left Hand, Complete 2nd Ray, Open Approach */
"0X6K0Z6"  =  "1"  /* Detachment at Left Hand, Complete 3rd Ray, Open Approach */
"0X6K0Z7"  =  "1"  /* Detachment at Left Hand, Complete 4th Ray, Open Approach */
"0X6K0Z8"  =  "1"  /* Detachment at Left Hand, Complete 5th Ray, Open Approach */
"0X6K0Z9"  =  "1"  /* Detachment at Left Hand, Partial 1st Ray, Open Approach */
"0X6K0ZB"  =  "1"  /* Detachment at Left Hand, Partial 2nd Ray, Open Approach */
"0X6K0ZC"  =  "1"  /* Detachment at Left Hand, Partial 3rd Ray, Open Approach */
"0X6K0ZD"  =  "1"  /* Detachment at Left Hand, Partial 4th Ray, Open Approach */
"0X6K0ZF"  =  "1"  /* Detachment at Left Hand, Partial 5th Ray, Open Approach */
"0X6L0Z0"  =  "1"  /* Detachment at Right Thumb, Complete, Open Approach */
"0X6L0Z1"  =  "1"  /* Detachment at Right Thumb, High, Open Approach */
"0X6L0Z2"  =  "1"  /* Detachment at Right Thumb, Mid, Open Approach */
"0X6L0Z3"  =  "1"  /* Detachment at Right Thumb, Low, Open Approach */
"0X6M0Z0"  =  "1"  /* Detachment at Left Thumb, Complete, Open Approach */
"0X6M0Z1"  =  "1"  /* Detachment at Left Thumb, High, Open Approach */
"0X6M0Z2"  =  "1"  /* Detachment at Left Thumb, Mid, Open Approach */
"0X6M0Z3"  =  "1"  /* Detachment at Left Thumb, Low, Open Approach */
"0X6N0Z0"  =  "1"  /* Detachment at Right Index Finger, Complete, Open Approach */
"0X6N0Z1"  =  "1"  /* Detachment at Right Index Finger, High, Open Approach */
"0X6N0Z2"  =  "1"  /* Detachment at Right Index Finger, Mid, Open Approach */
"0X6N0Z3"  =  "1"  /* Detachment at Right Index Finger, Low, Open Approach */
"0X6P0Z0"  =  "1"  /* Detachment at Left Index Finger, Complete, Open Approach */
"0X6P0Z1"  =  "1"  /* Detachment at Left Index Finger, High, Open Approach */
"0X6P0Z2"  =  "1"  /* Detachment at Left Index Finger, Mid, Open Approach */
"0X6P0Z3"  =  "1"  /* Detachment at Left Index Finger, Low, Open Approach */
"0X6Q0Z0"  =  "1"  /* Detachment at Right Middle Finger, Complete, Open Approach */
"0X6Q0Z1"  =  "1"  /* Detachment at Right Middle Finger, High, Open Approach */
"0X6Q0Z2"  =  "1"  /* Detachment at Right Middle Finger, Mid, Open Approach */
"0X6Q0Z3"  =  "1"  /* Detachment at Right Middle Finger, Low, Open Approach */
"0X6R0Z0"  =  "1"  /* Detachment at Left Middle Finger, Complete, Open Approach */
"0X6R0Z1"  =  "1"  /* Detachment at Left Middle Finger, High, Open Approach */
"0X6R0Z2"  =  "1"  /* Detachment at Left Middle Finger, Mid, Open Approach */
"0X6R0Z3"  =  "1"  /* Detachment at Left Middle Finger, Low, Open Approach */
"0X6S0Z0"  =  "1"  /* Detachment at Right Ring Finger, Complete, Open Approach */
"0X6S0Z1"  =  "1"  /* Detachment at Right Ring Finger, High, Open Approach */
"0X6S0Z2"  =  "1"  /* Detachment at Right Ring Finger, Mid, Open Approach */
"0X6S0Z3"  =  "1"  /* Detachment at Right Ring Finger, Low, Open Approach */
"0X6T0Z0"  =  "1"  /* Detachment at Left Ring Finger, Complete, Open Approach */
"0X6T0Z1"  =  "1"  /* Detachment at Left Ring Finger, High, Open Approach */
"0X6T0Z2"  =  "1"  /* Detachment at Left Ring Finger, Mid, Open Approach */
"0X6T0Z3"  =  "1"  /* Detachment at Left Ring Finger, Low, Open Approach */
"0X6V0Z0"  =  "1"  /* Detachment at Right Little Finger, Complete, Open Approach */
"0X6V0Z1"  =  "1"  /* Detachment at Right Little Finger, High, Open Approach */
"0X6V0Z2"  =  "1"  /* Detachment at Right Little Finger, Mid, Open Approach */
"0X6V0Z3"  =  "1"  /* Detachment at Right Little Finger, Low, Open Approach */
"0X6W0Z0"  =  "1"  /* Detachment at Left Little Finger, Complete, Open Approach */
"0X6W0Z1"  =  "1"  /* Detachment at Left Little Finger, High, Open Approach */
"0X6W0Z2"  =  "1"  /* Detachment at Left Little Finger, Mid, Open Approach */
"0X6W0Z3"  =  "1"  /* Detachment at Left Little Finger, Low, Open Approach */
"0XB20ZZ"  =  "1"  /* Excision of Right Shoulder Region, Open Approach */
"0XB23ZZ"  =  "1"  /* Excision of Right Shoulder Region, Percutaneous Approach */
"0XB24ZZ"  =  "1"  /* Excision of Right Shoulder Region, Perc Endo Approach */
"0XB30ZZ"  =  "1"  /* Excision of Left Shoulder Region, Open Approach */
"0XB33ZZ"  =  "1"  /* Excision of Left Shoulder Region, Percutaneous Approach */
"0XB34ZZ"  =  "1"  /* Excision of Left Shoulder Region, Perc Endo Approach */
"0XB40ZZ"  =  "1"  /* Excision of Right Axilla, Open Approach */
"0XB43ZZ"  =  "1"  /* Excision of Right Axilla, Percutaneous Approach */
"0XB44ZZ"  =  "1"  /* Excision of Right Axilla, Percutaneous Endoscopic Approach */
"0XB50ZZ"  =  "1"  /* Excision of Left Axilla, Open Approach */
"0XB53ZZ"  =  "1"  /* Excision of Left Axilla, Percutaneous Approach */
"0XB54ZZ"  =  "1"  /* Excision of Left Axilla, Percutaneous Endoscopic Approach */
"0XB60ZZ"  =  "1"  /* Excision of Right Upper Extremity, Open Approach */
"0XB63ZZ"  =  "1"  /* Excision of Right Upper Extremity, Percutaneous Approach */
"0XB64ZZ"  =  "1"  /* Excision of Right Upper Extremity, Perc Endo Approach */
"0XB70ZZ"  =  "1"  /* Excision of Left Upper Extremity, Open Approach */
"0XB73ZZ"  =  "1"  /* Excision of Left Upper Extremity, Percutaneous Approach */
"0XB74ZZ"  =  "1"  /* Excision of Left Upper Extremity, Perc Endo Approach */
"0XB80ZZ"  =  "1"  /* Excision of Right Upper Arm, Open Approach */
"0XB83ZZ"  =  "1"  /* Excision of Right Upper Arm, Percutaneous Approach */
"0XB84ZZ"  =  "1"  /* Excision of Right Upper Arm, Perc Endo Approach */
"0XB90ZZ"  =  "1"  /* Excision of Left Upper Arm, Open Approach */
"0XB93ZZ"  =  "1"  /* Excision of Left Upper Arm, Percutaneous Approach */
"0XB94ZZ"  =  "1"  /* Excision of Left Upper Arm, Percutaneous Endoscopic Approach */
"0XBB0ZZ"  =  "1"  /* Excision of Right Elbow Region, Open Approach */
"0XBB3ZZ"  =  "1"  /* Excision of Right Elbow Region, Percutaneous Approach */
"0XBB4ZZ"  =  "1"  /* Excision of Right Elbow Region, Perc Endo Approach */
"0XBC0ZZ"  =  "1"  /* Excision of Left Elbow Region, Open Approach */
"0XBC3ZZ"  =  "1"  /* Excision of Left Elbow Region, Percutaneous Approach */
"0XBC4ZZ"  =  "1"  /* Excision of Left Elbow Region, Perc Endo Approach */
"0XBD0ZZ"  =  "1"  /* Excision of Right Lower Arm, Open Approach */
"0XBD3ZZ"  =  "1"  /* Excision of Right Lower Arm, Percutaneous Approach */
"0XBD4ZZ"  =  "1"  /* Excision of Right Lower Arm, Perc Endo Approach */
"0XBF0ZZ"  =  "1"  /* Excision of Left Lower Arm, Open Approach */
"0XBF3ZZ"  =  "1"  /* Excision of Left Lower Arm, Percutaneous Approach */
"0XBF4ZZ"  =  "1"  /* Excision of Left Lower Arm, Percutaneous Endoscopic Approach */
"0XBG0ZZ"  =  "1"  /* Excision of Right Wrist Region, Open Approach */
"0XBG3ZZ"  =  "1"  /* Excision of Right Wrist Region, Percutaneous Approach */
"0XBG4ZZ"  =  "1"  /* Excision of Right Wrist Region, Perc Endo Approach */
"0XBH0ZZ"  =  "1"  /* Excision of Left Wrist Region, Open Approach */
"0XBH3ZZ"  =  "1"  /* Excision of Left Wrist Region, Percutaneous Approach */
"0XBH4ZZ"  =  "1"  /* Excision of Left Wrist Region, Perc Endo Approach */
"0XBJ0ZZ"  =  "1"  /* Excision of Right Hand, Open Approach */
"0XBJ3ZZ"  =  "1"  /* Excision of Right Hand, Percutaneous Approach */
"0XBJ4ZZ"  =  "1"  /* Excision of Right Hand, Percutaneous Endoscopic Approach */
"0XBK0ZZ"  =  "1"  /* Excision of Left Hand, Open Approach */
"0XBK3ZZ"  =  "1"  /* Excision of Left Hand, Percutaneous Approach */
"0XBK4ZZ"  =  "1"  /* Excision of Left Hand, Percutaneous Endoscopic Approach */
"0XH201Z"  =  "1"  /* Insertion of Radioact Elem into R Shoulder, Open Approach */
"0XH231Z"  =  "1"  /* Insertion of Radioact Elem into R Shoulder, Perc Approach */
"0XH241Z"  =  "1"  /* Insert of Radioact Elem into R Shoulder, Perc Endo Approach */
"0XH301Z"  =  "1"  /* Insertion of Radioact Elem into L Shoulder, Open Approach */
"0XH331Z"  =  "1"  /* Insertion of Radioact Elem into L Shoulder, Perc Approach */
"0XH341Z"  =  "1"  /* Insert of Radioact Elem into L Shoulder, Perc Endo Approach */
"0XH401Z"  =  "1"  /* Insertion of Radioact Elem into R Axilla, Open Approach */
"0XH431Z"  =  "1"  /* Insertion of Radioact Elem into R Axilla, Perc Approach */
"0XH441Z"  =  "1"  /* Insertion of Radioact Elem into R Axilla, Perc Endo Approach */
"0XH501Z"  =  "1"  /* Insertion of Radioact Elem into L Axilla, Open Approach */
"0XH531Z"  =  "1"  /* Insertion of Radioact Elem into L Axilla, Perc Approach */
"0XH541Z"  =  "1"  /* Insertion of Radioact Elem into L Axilla, Perc Endo Approach */
"0XH601Z"  =  "1"  /* Insertion of Radioact Elem into R Up Extrem, Open Approach */
"0XH631Z"  =  "1"  /* Insertion of Radioact Elem into R Up Extrem, Perc Approach */
"0XH641Z"  =  "1"  /* Insert of Radioact Elem into R Up Extrem, Perc Endo Approach */
"0XH701Z"  =  "1"  /* Insertion of Radioact Elem into L Up Extrem, Open Approach */
"0XH731Z"  =  "1"  /* Insertion of Radioact Elem into L Up Extrem, Perc Approach */
"0XH741Z"  =  "1"  /* Insert of Radioact Elem into L Up Extrem, Perc Endo Approach */
"0XH801Z"  =  "1"  /* Insertion of Radioact Elem into R Up Arm, Open Approach */
"0XH831Z"  =  "1"  /* Insertion of Radioact Elem into R Up Arm, Perc Approach */
"0XH841Z"  =  "1"  /* Insertion of Radioact Elem into R Up Arm, Perc Endo Approach */
"0XH901Z"  =  "1"  /* Insertion of Radioact Elem into L Up Arm, Open Approach */
"0XH931Z"  =  "1"  /* Insertion of Radioact Elem into L Up Arm, Perc Approach */
"0XH941Z"  =  "1"  /* Insertion of Radioact Elem into L Up Arm, Perc Endo Approach */
"0XHB01Z"  =  "1"  /* Insertion of Radioactive Element into R Elbow, Open Approach */
"0XHB31Z"  =  "1"  /* Insertion of Radioactive Element into R Elbow, Perc Approach */
"0XHB41Z"  =  "1"  /* Insertion of Radioact Elem into R Elbow, Perc Endo Approach */
"0XHC01Z"  =  "1"  /* Insertion of Radioactive Element into L Elbow, Open Approach */
"0XHC31Z"  =  "1"  /* Insertion of Radioactive Element into L Elbow, Perc Approach */
"0XHC41Z"  =  "1"  /* Insertion of Radioact Elem into L Elbow, Perc Endo Approach */
"0XHD01Z"  =  "1"  /* Insertion of Radioact Elem into R Low Arm, Open Approach */
"0XHD31Z"  =  "1"  /* Insertion of Radioact Elem into R Low Arm, Perc Approach */
"0XHD41Z"  =  "1"  /* Insert of Radioact Elem into R Low Arm, Perc Endo Approach */
"0XHF01Z"  =  "1"  /* Insertion of Radioact Elem into L Low Arm, Open Approach */
"0XHF31Z"  =  "1"  /* Insertion of Radioact Elem into L Low Arm, Perc Approach */
"0XHF41Z"  =  "1"  /* Insert of Radioact Elem into L Low Arm, Perc Endo Approach */
"0XHG01Z"  =  "1"  /* Insertion of Radioactive Element into R Wrist, Open Approach */
"0XHG31Z"  =  "1"  /* Insertion of Radioactive Element into R Wrist, Perc Approach */
"0XHG41Z"  =  "1"  /* Insertion of Radioact Elem into R Wrist, Perc Endo Approach */
"0XHH01Z"  =  "1"  /* Insertion of Radioactive Element into L Wrist, Open Approach */
"0XHH31Z"  =  "1"  /* Insertion of Radioactive Element into L Wrist, Perc Approach */
"0XHH41Z"  =  "1"  /* Insertion of Radioact Elem into L Wrist, Perc Endo Approach */
"0XHJ01Z"  =  "1"  /* Insertion of Radioactive Element into R Hand, Open Approach */
"0XHJ31Z"  =  "1"  /* Insertion of Radioactive Element into R Hand, Perc Approach */
"0XHJ41Z"  =  "1"  /* Insertion of Radioact Elem into R Hand, Perc Endo Approach */
"0XHK01Z"  =  "1"  /* Insertion of Radioactive Element into L Hand, Open Approach */
"0XHK31Z"  =  "1"  /* Insertion of Radioactive Element into L Hand, Perc Approach */
"0XHK41Z"  =  "1"  /* Insertion of Radioact Elem into L Hand, Perc Endo Approach */
"0XJJ3ZZ"  =  "1"  /* Inspection of Right Hand, Percutaneous Approach */
"0XJJ4ZZ"  =  "1"  /* Inspection of Right Hand, Percutaneous Endoscopic Approach */
"0XJK3ZZ"  =  "1"  /* Inspection of Left Hand, Percutaneous Approach */
"0XJK4ZZ"  =  "1"  /* Inspection of Left Hand, Percutaneous Endoscopic Approach */
"0XM00ZZ"  =  "1"  /* Reattachment of Right Forequarter, Open Approach */
"0XM10ZZ"  =  "1"  /* Reattachment of Left Forequarter, Open Approach */
"0XM20ZZ"  =  "1"  /* Reattachment of Right Shoulder Region, Open Approach */
"0XM30ZZ"  =  "1"  /* Reattachment of Left Shoulder Region, Open Approach */
"0XM40ZZ"  =  "1"  /* Reattachment of Right Axilla, Open Approach */
"0XM50ZZ"  =  "1"  /* Reattachment of Left Axilla, Open Approach */
"0XM60ZZ"  =  "1"  /* Reattachment of Right Upper Extremity, Open Approach */
"0XM70ZZ"  =  "1"  /* Reattachment of Left Upper Extremity, Open Approach */
"0XM80ZZ"  =  "1"  /* Reattachment of Right Upper Arm, Open Approach */
"0XM90ZZ"  =  "1"  /* Reattachment of Left Upper Arm, Open Approach */
"0XMB0ZZ"  =  "1"  /* Reattachment of Right Elbow Region, Open Approach */
"0XMC0ZZ"  =  "1"  /* Reattachment of Left Elbow Region, Open Approach */
"0XMD0ZZ"  =  "1"  /* Reattachment of Right Lower Arm, Open Approach */
"0XMF0ZZ"  =  "1"  /* Reattachment of Left Lower Arm, Open Approach */
"0XMG0ZZ"  =  "1"  /* Reattachment of Right Wrist Region, Open Approach */
"0XMH0ZZ"  =  "1"  /* Reattachment of Left Wrist Region, Open Approach */
"0XMJ0ZZ"  =  "1"  /* Reattachment of Right Hand, Open Approach */
"0XMK0ZZ"  =  "1"  /* Reattachment of Left Hand, Open Approach */
"0XML0ZZ"  =  "1"  /* Reattachment of Right Thumb, Open Approach */
"0XMM0ZZ"  =  "1"  /* Reattachment of Left Thumb, Open Approach */
"0XMN0ZZ"  =  "1"  /* Reattachment of Right Index Finger, Open Approach */
"0XMP0ZZ"  =  "1"  /* Reattachment of Left Index Finger, Open Approach */
"0XMQ0ZZ"  =  "1"  /* Reattachment of Right Middle Finger, Open Approach */
"0XMR0ZZ"  =  "1"  /* Reattachment of Left Middle Finger, Open Approach */
"0XMS0ZZ"  =  "1"  /* Reattachment of Right Ring Finger, Open Approach */
"0XMT0ZZ"  =  "1"  /* Reattachment of Left Ring Finger, Open Approach */
"0XMV0ZZ"  =  "1"  /* Reattachment of Right Little Finger, Open Approach */
"0XMW0ZZ"  =  "1"  /* Reattachment of Left Little Finger, Open Approach */
"0XQ20ZZ"  =  "1"  /* Repair Right Shoulder Region, Open Approach */
"0XQ23ZZ"  =  "1"  /* Repair Right Shoulder Region, Percutaneous Approach */
"0XQ24ZZ"  =  "1"  /* Repair Right Shoulder Region, Perc Endo Approach */
"0XQ2XZZ"  =  "1"  /* Repair Right Shoulder Region, External Approach */
"0XQ30ZZ"  =  "1"  /* Repair Left Shoulder Region, Open Approach */
"0XQ33ZZ"  =  "1"  /* Repair Left Shoulder Region, Percutaneous Approach */
"0XQ34ZZ"  =  "1"  /* Repair Left Shoulder Region, Perc Endo Approach */
"0XQ3XZZ"  =  "1"  /* Repair Left Shoulder Region, External Approach */
"0XQ40ZZ"  =  "1"  /* Repair Right Axilla, Open Approach */
"0XQ43ZZ"  =  "1"  /* Repair Right Axilla, Percutaneous Approach */
"0XQ44ZZ"  =  "1"  /* Repair Right Axilla, Percutaneous Endoscopic Approach */
"0XQ4XZZ"  =  "1"  /* Repair Right Axilla, External Approach */
"0XQ50ZZ"  =  "1"  /* Repair Left Axilla, Open Approach */
"0XQ53ZZ"  =  "1"  /* Repair Left Axilla, Percutaneous Approach */
"0XQ54ZZ"  =  "1"  /* Repair Left Axilla, Percutaneous Endoscopic Approach */
"0XQ5XZZ"  =  "1"  /* Repair Left Axilla, External Approach */
"0XQ60ZZ"  =  "1"  /* Repair Right Upper Extremity, Open Approach */
"0XQ63ZZ"  =  "1"  /* Repair Right Upper Extremity, Percutaneous Approach */
"0XQ64ZZ"  =  "1"  /* Repair Right Upper Extremity, Perc Endo Approach */
"0XQ6XZZ"  =  "1"  /* Repair Right Upper Extremity, External Approach */
"0XQ70ZZ"  =  "1"  /* Repair Left Upper Extremity, Open Approach */
"0XQ73ZZ"  =  "1"  /* Repair Left Upper Extremity, Percutaneous Approach */
"0XQ74ZZ"  =  "1"  /* Repair Left Upper Extremity, Perc Endo Approach */
"0XQ7XZZ"  =  "1"  /* Repair Left Upper Extremity, External Approach */
"0XQ80ZZ"  =  "1"  /* Repair Right Upper Arm, Open Approach */
"0XQ83ZZ"  =  "1"  /* Repair Right Upper Arm, Percutaneous Approach */
"0XQ84ZZ"  =  "1"  /* Repair Right Upper Arm, Percutaneous Endoscopic Approach */
"0XQ8XZZ"  =  "1"  /* Repair Right Upper Arm, External Approach */
"0XQ90ZZ"  =  "1"  /* Repair Left Upper Arm, Open Approach */
"0XQ93ZZ"  =  "1"  /* Repair Left Upper Arm, Percutaneous Approach */
"0XQ94ZZ"  =  "1"  /* Repair Left Upper Arm, Percutaneous Endoscopic Approach */
"0XQ9XZZ"  =  "1"  /* Repair Left Upper Arm, External Approach */
"0XQB0ZZ"  =  "1"  /* Repair Right Elbow Region, Open Approach */
"0XQB3ZZ"  =  "1"  /* Repair Right Elbow Region, Percutaneous Approach */
"0XQB4ZZ"  =  "1"  /* Repair Right Elbow Region, Percutaneous Endoscopic Approach */
"0XQBXZZ"  =  "1"  /* Repair Right Elbow Region, External Approach */
"0XQC0ZZ"  =  "1"  /* Repair Left Elbow Region, Open Approach */
"0XQC3ZZ"  =  "1"  /* Repair Left Elbow Region, Percutaneous Approach */
"0XQC4ZZ"  =  "1"  /* Repair Left Elbow Region, Percutaneous Endoscopic Approach */
"0XQCXZZ"  =  "1"  /* Repair Left Elbow Region, External Approach */
"0XQD0ZZ"  =  "1"  /* Repair Right Lower Arm, Open Approach */
"0XQD3ZZ"  =  "1"  /* Repair Right Lower Arm, Percutaneous Approach */
"0XQD4ZZ"  =  "1"  /* Repair Right Lower Arm, Percutaneous Endoscopic Approach */
"0XQDXZZ"  =  "1"  /* Repair Right Lower Arm, External Approach */
"0XQF0ZZ"  =  "1"  /* Repair Left Lower Arm, Open Approach */
"0XQF3ZZ"  =  "1"  /* Repair Left Lower Arm, Percutaneous Approach */
"0XQF4ZZ"  =  "1"  /* Repair Left Lower Arm, Percutaneous Endoscopic Approach */
"0XQFXZZ"  =  "1"  /* Repair Left Lower Arm, External Approach */
"0XQG0ZZ"  =  "1"  /* Repair Right Wrist Region, Open Approach */
"0XQG3ZZ"  =  "1"  /* Repair Right Wrist Region, Percutaneous Approach */
"0XQG4ZZ"  =  "1"  /* Repair Right Wrist Region, Percutaneous Endoscopic Approach */
"0XQGXZZ"  =  "1"  /* Repair Right Wrist Region, External Approach */
"0XQH0ZZ"  =  "1"  /* Repair Left Wrist Region, Open Approach */
"0XQH3ZZ"  =  "1"  /* Repair Left Wrist Region, Percutaneous Approach */
"0XQH4ZZ"  =  "1"  /* Repair Left Wrist Region, Percutaneous Endoscopic Approach */
"0XQHXZZ"  =  "1"  /* Repair Left Wrist Region, External Approach */
"0XQJ0ZZ"  =  "1"  /* Repair Right Hand, Open Approach */
"0XQJ3ZZ"  =  "1"  /* Repair Right Hand, Percutaneous Approach */
"0XQJ4ZZ"  =  "1"  /* Repair Right Hand, Percutaneous Endoscopic Approach */
"0XQJXZZ"  =  "1"  /* Repair Right Hand, External Approach */
"0XQK0ZZ"  =  "1"  /* Repair Left Hand, Open Approach */
"0XQK3ZZ"  =  "1"  /* Repair Left Hand, Percutaneous Approach */
"0XQK4ZZ"  =  "1"  /* Repair Left Hand, Percutaneous Endoscopic Approach */
"0XQKXZZ"  =  "1"  /* Repair Left Hand, External Approach */
"0XQL0ZZ"  =  "1"  /* Repair Right Thumb, Open Approach */
"0XQL3ZZ"  =  "1"  /* Repair Right Thumb, Percutaneous Approach */
"0XQL4ZZ"  =  "1"  /* Repair Right Thumb, Percutaneous Endoscopic Approach */
"0XQLXZZ"  =  "1"  /* Repair Right Thumb, External Approach */
"0XQM0ZZ"  =  "1"  /* Repair Left Thumb, Open Approach */
"0XQM3ZZ"  =  "1"  /* Repair Left Thumb, Percutaneous Approach */
"0XQM4ZZ"  =  "1"  /* Repair Left Thumb, Percutaneous Endoscopic Approach */
"0XQMXZZ"  =  "1"  /* Repair Left Thumb, External Approach */
"0XQN0ZZ"  =  "1"  /* Repair Right Index Finger, Open Approach */
"0XQN3ZZ"  =  "1"  /* Repair Right Index Finger, Percutaneous Approach */
"0XQN4ZZ"  =  "1"  /* Repair Right Index Finger, Percutaneous Endoscopic Approach */
"0XQNXZZ"  =  "1"  /* Repair Right Index Finger, External Approach */
"0XQP0ZZ"  =  "1"  /* Repair Left Index Finger, Open Approach */
"0XQP3ZZ"  =  "1"  /* Repair Left Index Finger, Percutaneous Approach */
"0XQP4ZZ"  =  "1"  /* Repair Left Index Finger, Percutaneous Endoscopic Approach */
"0XQPXZZ"  =  "1"  /* Repair Left Index Finger, External Approach */
"0XQQ0ZZ"  =  "1"  /* Repair Right Middle Finger, Open Approach */
"0XQQ3ZZ"  =  "1"  /* Repair Right Middle Finger, Percutaneous Approach */
"0XQQ4ZZ"  =  "1"  /* Repair Right Middle Finger, Percutaneous Endoscopic Approach */
"0XQQXZZ"  =  "1"  /* Repair Right Middle Finger, External Approach */
"0XQR0ZZ"  =  "1"  /* Repair Left Middle Finger, Open Approach */
"0XQR3ZZ"  =  "1"  /* Repair Left Middle Finger, Percutaneous Approach */
"0XQR4ZZ"  =  "1"  /* Repair Left Middle Finger, Percutaneous Endoscopic Approach */
"0XQRXZZ"  =  "1"  /* Repair Left Middle Finger, External Approach */
"0XQS0ZZ"  =  "1"  /* Repair Right Ring Finger, Open Approach */
"0XQS3ZZ"  =  "1"  /* Repair Right Ring Finger, Percutaneous Approach */
"0XQS4ZZ"  =  "1"  /* Repair Right Ring Finger, Percutaneous Endoscopic Approach */
"0XQSXZZ"  =  "1"  /* Repair Right Ring Finger, External Approach */
"0XQT0ZZ"  =  "1"  /* Repair Left Ring Finger, Open Approach */
"0XQT3ZZ"  =  "1"  /* Repair Left Ring Finger, Percutaneous Approach */
"0XQT4ZZ"  =  "1"  /* Repair Left Ring Finger, Percutaneous Endoscopic Approach */
"0XQTXZZ"  =  "1"  /* Repair Left Ring Finger, External Approach */
"0XQV0ZZ"  =  "1"  /* Repair Right Little Finger, Open Approach */
"0XQV3ZZ"  =  "1"  /* Repair Right Little Finger, Percutaneous Approach */
"0XQV4ZZ"  =  "1"  /* Repair Right Little Finger, Percutaneous Endoscopic Approach */
"0XQVXZZ"  =  "1"  /* Repair Right Little Finger, External Approach */
"0XQW0ZZ"  =  "1"  /* Repair Left Little Finger, Open Approach */
"0XQW3ZZ"  =  "1"  /* Repair Left Little Finger, Percutaneous Approach */
"0XQW4ZZ"  =  "1"  /* Repair Left Little Finger, Percutaneous Endoscopic Approach */
"0XQWXZZ"  =  "1"  /* Repair Left Little Finger, External Approach */
"0XRL07N"  =  "1"  /* Replacement of R Thumb with R Toe, Autol Sub, Open Approach */
"0XRL07P"  =  "1"  /* Replacement of R Thumb with L Toe, Autol Sub, Open Approach */
"0XRL47N"  =  "1"  /* Replace of R Thumb with R Toe, Autol Sub, Perc Endo Approach */
"0XRL47P"  =  "1"  /* Replace of R Thumb with L Toe, Autol Sub, Perc Endo Approach */
"0XRM07N"  =  "1"  /* Replacement of L Thumb with R Toe, Autol Sub, Open Approach */
"0XRM07P"  =  "1"  /* Replacement of L Thumb with L Toe, Autol Sub, Open Approach */
"0XRM47N"  =  "1"  /* Replace of L Thumb with R Toe, Autol Sub, Perc Endo Approach */
"0XRM47P"  =  "1"  /* Replace of L Thumb with L Toe, Autol Sub, Perc Endo Approach */
"0XU207Z"  =  "1"  /* Supplement R Shoulder with Autol Sub, Open Approach */
"0XU20JZ"  =  "1"  /* Supplement R Shoulder with Synth Sub, Open Approach */
"0XU20KZ"  =  "1"  /* Supplement R Shoulder with Nonaut Sub, Open Approach */
"0XU247Z"  =  "1"  /* Supplement R Shoulder with Autol Sub, Perc Endo Approach */
"0XU24JZ"  =  "1"  /* Supplement R Shoulder with Synth Sub, Perc Endo Approach */
"0XU24KZ"  =  "1"  /* Supplement R Shoulder with Nonaut Sub, Perc Endo Approach */
"0XU307Z"  =  "1"  /* Supplement L Shoulder with Autol Sub, Open Approach */
"0XU30JZ"  =  "1"  /* Supplement L Shoulder with Synth Sub, Open Approach */
"0XU30KZ"  =  "1"  /* Supplement L Shoulder with Nonaut Sub, Open Approach */
"0XU347Z"  =  "1"  /* Supplement L Shoulder with Autol Sub, Perc Endo Approach */
"0XU34JZ"  =  "1"  /* Supplement L Shoulder with Synth Sub, Perc Endo Approach */
"0XU34KZ"  =  "1"  /* Supplement L Shoulder with Nonaut Sub, Perc Endo Approach */
"0XU407Z"  =  "1"  /* Supplement Right Axilla with Autol Sub, Open Approach */
"0XU40JZ"  =  "1"  /* Supplement Right Axilla with Synth Sub, Open Approach */
"0XU40KZ"  =  "1"  /* Supplement Right Axilla with Nonaut Sub, Open Approach */
"0XU447Z"  =  "1"  /* Supplement Right Axilla with Autol Sub, Perc Endo Approach */
"0XU44JZ"  =  "1"  /* Supplement Right Axilla with Synth Sub, Perc Endo Approach */
"0XU44KZ"  =  "1"  /* Supplement Right Axilla with Nonaut Sub, Perc Endo Approach */
"0XU507Z"  =  "1"  /* Supplement Left Axilla with Autol Sub, Open Approach */
"0XU50JZ"  =  "1"  /* Supplement Left Axilla with Synth Sub, Open Approach */
"0XU50KZ"  =  "1"  /* Supplement Left Axilla with Nonaut Sub, Open Approach */
"0XU547Z"  =  "1"  /* Supplement Left Axilla with Autol Sub, Perc Endo Approach */
"0XU54JZ"  =  "1"  /* Supplement Left Axilla with Synth Sub, Perc Endo Approach */
"0XU54KZ"  =  "1"  /* Supplement Left Axilla with Nonaut Sub, Perc Endo Approach */
"0XU607Z"  =  "1"  /* Supplement R Up Extrem with Autol Sub, Open Approach */
"0XU60JZ"  =  "1"  /* Supplement R Up Extrem with Synth Sub, Open Approach */
"0XU60KZ"  =  "1"  /* Supplement R Up Extrem with Nonaut Sub, Open Approach */
"0XU647Z"  =  "1"  /* Supplement R Up Extrem with Autol Sub, Perc Endo Approach */
"0XU64JZ"  =  "1"  /* Supplement R Up Extrem with Synth Sub, Perc Endo Approach */
"0XU64KZ"  =  "1"  /* Supplement R Up Extrem with Nonaut Sub, Perc Endo Approach */
"0XU707Z"  =  "1"  /* Supplement L Up Extrem with Autol Sub, Open Approach */
"0XU70JZ"  =  "1"  /* Supplement L Up Extrem with Synth Sub, Open Approach */
"0XU70KZ"  =  "1"  /* Supplement L Up Extrem with Nonaut Sub, Open Approach */
"0XU747Z"  =  "1"  /* Supplement L Up Extrem with Autol Sub, Perc Endo Approach */
"0XU74JZ"  =  "1"  /* Supplement L Up Extrem with Synth Sub, Perc Endo Approach */
"0XU74KZ"  =  "1"  /* Supplement L Up Extrem with Nonaut Sub, Perc Endo Approach */
"0XU807Z"  =  "1"  /* Supplement Right Upper Arm with Autol Sub, Open Approach */
"0XU80JZ"  =  "1"  /* Supplement Right Upper Arm with Synth Sub, Open Approach */
"0XU80KZ"  =  "1"  /* Supplement Right Upper Arm with Nonaut Sub, Open Approach */
"0XU847Z"  =  "1"  /* Supplement R Up Arm with Autol Sub, Perc Endo Approach */
"0XU84JZ"  =  "1"  /* Supplement R Up Arm with Synth Sub, Perc Endo Approach */
"0XU84KZ"  =  "1"  /* Supplement R Up Arm with Nonaut Sub, Perc Endo Approach */
"0XU907Z"  =  "1"  /* Supplement Left Upper Arm with Autol Sub, Open Approach */
"0XU90JZ"  =  "1"  /* Supplement Left Upper Arm with Synth Sub, Open Approach */
"0XU90KZ"  =  "1"  /* Supplement Left Upper Arm with Nonaut Sub, Open Approach */
"0XU947Z"  =  "1"  /* Supplement Left Upper Arm with Autol Sub, Perc Endo Approach */
"0XU94JZ"  =  "1"  /* Supplement Left Upper Arm with Synth Sub, Perc Endo Approach */
"0XU94KZ"  =  "1"  /* Supplement L Up Arm with Nonaut Sub, Perc Endo Approach */
"0XUB07Z"  =  "1"  /* Supplement Right Elbow Region with Autol Sub, Open Approach */
"0XUB0JZ"  =  "1"  /* Supplement Right Elbow Region with Synth Sub, Open Approach */
"0XUB0KZ"  =  "1"  /* Supplement Right Elbow Region with Nonaut Sub, Open Approach */
"0XUB47Z"  =  "1"  /* Supplement R Elbow with Autol Sub, Perc Endo Approach */
"0XUB4JZ"  =  "1"  /* Supplement R Elbow with Synth Sub, Perc Endo Approach */
"0XUB4KZ"  =  "1"  /* Supplement R Elbow with Nonaut Sub, Perc Endo Approach */
"0XUC07Z"  =  "1"  /* Supplement Left Elbow Region with Autol Sub, Open Approach */
"0XUC0JZ"  =  "1"  /* Supplement Left Elbow Region with Synth Sub, Open Approach */
"0XUC0KZ"  =  "1"  /* Supplement Left Elbow Region with Nonaut Sub, Open Approach */
"0XUC47Z"  =  "1"  /* Supplement L Elbow with Autol Sub, Perc Endo Approach */
"0XUC4JZ"  =  "1"  /* Supplement L Elbow with Synth Sub, Perc Endo Approach */
"0XUC4KZ"  =  "1"  /* Supplement L Elbow with Nonaut Sub, Perc Endo Approach */
"0XUD07Z"  =  "1"  /* Supplement Right Lower Arm with Autol Sub, Open Approach */
"0XUD0JZ"  =  "1"  /* Supplement Right Lower Arm with Synth Sub, Open Approach */
"0XUD0KZ"  =  "1"  /* Supplement Right Lower Arm with Nonaut Sub, Open Approach */
"0XUD47Z"  =  "1"  /* Supplement R Low Arm with Autol Sub, Perc Endo Approach */
"0XUD4JZ"  =  "1"  /* Supplement R Low Arm with Synth Sub, Perc Endo Approach */
"0XUD4KZ"  =  "1"  /* Supplement R Low Arm with Nonaut Sub, Perc Endo Approach */
"0XUF07Z"  =  "1"  /* Supplement Left Lower Arm with Autol Sub, Open Approach */
"0XUF0JZ"  =  "1"  /* Supplement Left Lower Arm with Synth Sub, Open Approach */
"0XUF0KZ"  =  "1"  /* Supplement Left Lower Arm with Nonaut Sub, Open Approach */
"0XUF47Z"  =  "1"  /* Supplement Left Lower Arm with Autol Sub, Perc Endo Approach */
"0XUF4JZ"  =  "1"  /* Supplement Left Lower Arm with Synth Sub, Perc Endo Approach */
"0XUF4KZ"  =  "1"  /* Supplement L Low Arm with Nonaut Sub, Perc Endo Approach */
"0XUG07Z"  =  "1"  /* Supplement Right Wrist Region with Autol Sub, Open Approach */
"0XUG0JZ"  =  "1"  /* Supplement Right Wrist Region with Synth Sub, Open Approach */
"0XUG0KZ"  =  "1"  /* Supplement Right Wrist Region with Nonaut Sub, Open Approach */
"0XUG47Z"  =  "1"  /* Supplement R Wrist with Autol Sub, Perc Endo Approach */
"0XUG4JZ"  =  "1"  /* Supplement R Wrist with Synth Sub, Perc Endo Approach */
"0XUG4KZ"  =  "1"  /* Supplement R Wrist with Nonaut Sub, Perc Endo Approach */
"0XUH07Z"  =  "1"  /* Supplement Left Wrist Region with Autol Sub, Open Approach */
"0XUH0JZ"  =  "1"  /* Supplement Left Wrist Region with Synth Sub, Open Approach */
"0XUH0KZ"  =  "1"  /* Supplement Left Wrist Region with Nonaut Sub, Open Approach */
"0XUH47Z"  =  "1"  /* Supplement L Wrist with Autol Sub, Perc Endo Approach */
"0XUH4JZ"  =  "1"  /* Supplement L Wrist with Synth Sub, Perc Endo Approach */
"0XUH4KZ"  =  "1"  /* Supplement L Wrist with Nonaut Sub, Perc Endo Approach */
"0XUJ07Z"  =  "1"  /* Supplement Right Hand with Autol Sub, Open Approach */
"0XUJ0JZ"  =  "1"  /* Supplement Right Hand with Synth Sub, Open Approach */
"0XUJ0KZ"  =  "1"  /* Supplement Right Hand with Nonaut Sub, Open Approach */
"0XUJ47Z"  =  "1"  /* Supplement Right Hand with Autol Sub, Perc Endo Approach */
"0XUJ4JZ"  =  "1"  /* Supplement Right Hand with Synth Sub, Perc Endo Approach */
"0XUJ4KZ"  =  "1"  /* Supplement Right Hand with Nonaut Sub, Perc Endo Approach */
"0XUK07Z"  =  "1"  /* Supplement Left Hand with Autol Sub, Open Approach */
"0XUK0JZ"  =  "1"  /* Supplement Left Hand with Synth Sub, Open Approach */
"0XUK0KZ"  =  "1"  /* Supplement Left Hand with Nonaut Sub, Open Approach */
"0XUK47Z"  =  "1"  /* Supplement Left Hand with Autol Sub, Perc Endo Approach */
"0XUK4JZ"  =  "1"  /* Supplement Left Hand with Synth Sub, Perc Endo Approach */
"0XUK4KZ"  =  "1"  /* Supplement Left Hand with Nonaut Sub, Perc Endo Approach */
"0XUL07Z"  =  "1"  /* Supplement Right Thumb with Autol Sub, Open Approach */
"0XUL0JZ"  =  "1"  /* Supplement Right Thumb with Synth Sub, Open Approach */
"0XUL0KZ"  =  "1"  /* Supplement Right Thumb with Nonaut Sub, Open Approach */
"0XUL47Z"  =  "1"  /* Supplement Right Thumb with Autol Sub, Perc Endo Approach */
"0XUL4JZ"  =  "1"  /* Supplement Right Thumb with Synth Sub, Perc Endo Approach */
"0XUL4KZ"  =  "1"  /* Supplement Right Thumb with Nonaut Sub, Perc Endo Approach */
"0XUM07Z"  =  "1"  /* Supplement Left Thumb with Autol Sub, Open Approach */
"0XUM0JZ"  =  "1"  /* Supplement Left Thumb with Synth Sub, Open Approach */
"0XUM0KZ"  =  "1"  /* Supplement Left Thumb with Nonaut Sub, Open Approach */
"0XUM47Z"  =  "1"  /* Supplement Left Thumb with Autol Sub, Perc Endo Approach */
"0XUM4JZ"  =  "1"  /* Supplement Left Thumb with Synth Sub, Perc Endo Approach */
"0XUM4KZ"  =  "1"  /* Supplement Left Thumb with Nonaut Sub, Perc Endo Approach */
"0XUN07Z"  =  "1"  /* Supplement Right Index Finger with Autol Sub, Open Approach */
"0XUN0JZ"  =  "1"  /* Supplement Right Index Finger with Synth Sub, Open Approach */
"0XUN0KZ"  =  "1"  /* Supplement Right Index Finger with Nonaut Sub, Open Approach */
"0XUN47Z"  =  "1"  /* Supplement R Index Finger with Autol Sub, Perc Endo Approach */
"0XUN4JZ"  =  "1"  /* Supplement R Index Finger with Synth Sub, Perc Endo Approach */
"0XUN4KZ"  =  "1"  /* Supplement R Index Finger w Nonaut Sub, Perc Endo */
"0XUP07Z"  =  "1"  /* Supplement Left Index Finger with Autol Sub, Open Approach */
"0XUP0JZ"  =  "1"  /* Supplement Left Index Finger with Synth Sub, Open Approach */
"0XUP0KZ"  =  "1"  /* Supplement Left Index Finger with Nonaut Sub, Open Approach */
"0XUP47Z"  =  "1"  /* Supplement L Index Finger with Autol Sub, Perc Endo Approach */
"0XUP4JZ"  =  "1"  /* Supplement L Index Finger with Synth Sub, Perc Endo Approach */
"0XUP4KZ"  =  "1"  /* Supplement L Index Finger w Nonaut Sub, Perc Endo */
"0XUQ07Z"  =  "1"  /* Supplement Right Middle Finger with Autol Sub, Open Approach */
"0XUQ0JZ"  =  "1"  /* Supplement Right Middle Finger with Synth Sub, Open Approach */
"0XUQ0KZ"  =  "1"  /* Supplement R Mid Finger with Nonaut Sub, Open Approach */
"0XUQ47Z"  =  "1"  /* Supplement R Mid Finger with Autol Sub, Perc Endo Approach */
"0XUQ4JZ"  =  "1"  /* Supplement R Mid Finger with Synth Sub, Perc Endo Approach */
"0XUQ4KZ"  =  "1"  /* Supplement R Mid Finger with Nonaut Sub, Perc Endo Approach */
"0XUR07Z"  =  "1"  /* Supplement Left Middle Finger with Autol Sub, Open Approach */
"0XUR0JZ"  =  "1"  /* Supplement Left Middle Finger with Synth Sub, Open Approach */
"0XUR0KZ"  =  "1"  /* Supplement Left Middle Finger with Nonaut Sub, Open Approach */
"0XUR47Z"  =  "1"  /* Supplement L Mid Finger with Autol Sub, Perc Endo Approach */
"0XUR4JZ"  =  "1"  /* Supplement L Mid Finger with Synth Sub, Perc Endo Approach */
"0XUR4KZ"  =  "1"  /* Supplement L Mid Finger with Nonaut Sub, Perc Endo Approach */
"0XUS07Z"  =  "1"  /* Supplement Right Ring Finger with Autol Sub, Open Approach */
"0XUS0JZ"  =  "1"  /* Supplement Right Ring Finger with Synth Sub, Open Approach */
"0XUS0KZ"  =  "1"  /* Supplement Right Ring Finger with Nonaut Sub, Open Approach */
"0XUS47Z"  =  "1"  /* Supplement R Ring Finger with Autol Sub, Perc Endo Approach */
"0XUS4JZ"  =  "1"  /* Supplement R Ring Finger with Synth Sub, Perc Endo Approach */
"0XUS4KZ"  =  "1"  /* Supplement R Ring Finger with Nonaut Sub, Perc Endo Approach */
"0XUT07Z"  =  "1"  /* Supplement Left Ring Finger with Autol Sub, Open Approach */
"0XUT0JZ"  =  "1"  /* Supplement Left Ring Finger with Synth Sub, Open Approach */
"0XUT0KZ"  =  "1"  /* Supplement Left Ring Finger with Nonaut Sub, Open Approach */
"0XUT47Z"  =  "1"  /* Supplement L Ring Finger with Autol Sub, Perc Endo Approach */
"0XUT4JZ"  =  "1"  /* Supplement L Ring Finger with Synth Sub, Perc Endo Approach */
"0XUT4KZ"  =  "1"  /* Supplement L Ring Finger with Nonaut Sub, Perc Endo Approach */
"0XUV07Z"  =  "1"  /* Supplement Right Little Finger with Autol Sub, Open Approach */
"0XUV0JZ"  =  "1"  /* Supplement Right Little Finger with Synth Sub, Open Approach */
"0XUV0KZ"  =  "1"  /* Supplement R Little Finger with Nonaut Sub, Open Approach */
"0XUV47Z"  =  "1"  /* Supplement R Little Finger w Autol Sub, Perc Endo */
"0XUV4JZ"  =  "1"  /* Supplement R Little Finger w Synth Sub, Perc Endo */
"0XUV4KZ"  =  "1"  /* Supplement R Little Finger w Nonaut Sub, Perc Endo */
"0XUW07Z"  =  "1"  /* Supplement Left Little Finger with Autol Sub, Open Approach */
"0XUW0JZ"  =  "1"  /* Supplement Left Little Finger with Synth Sub, Open Approach */
"0XUW0KZ"  =  "1"  /* Supplement Left Little Finger with Nonaut Sub, Open Approach */
"0XUW47Z"  =  "1"  /* Supplement L Little Finger w Autol Sub, Perc Endo */
"0XUW4JZ"  =  "1"  /* Supplement L Little Finger w Synth Sub, Perc Endo */
"0XUW4KZ"  =  "1"  /* Supplement L Little Finger w Nonaut Sub, Perc Endo */
"0XXN0ZL"  =  "1"  /* Transfer Right Index Finger to Right Thumb, Open Approach */
"0XXP0ZM"  =  "1"  /* Transfer Left Index Finger to Left Thumb, Open Approach */
"0Y0007Z"  =  "1"  /* Alteration of Right Buttock with Autol Sub, Open Approach */
"0Y000JZ"  =  "1"  /* Alteration of Right Buttock with Synth Sub, Open Approach */
"0Y000KZ"  =  "1"  /* Alteration of Right Buttock with Nonaut Sub, Open Approach */
"0Y000ZZ"  =  "1"  /* Alteration of Right Buttock, Open Approach */
"0Y0037Z"  =  "1"  /* Alteration of Right Buttock with Autol Sub, Perc Approach */
"0Y003JZ"  =  "1"  /* Alteration of Right Buttock with Synth Sub, Perc Approach */
"0Y003KZ"  =  "1"  /* Alteration of Right Buttock with Nonaut Sub, Perc Approach */
"0Y003ZZ"  =  "1"  /* Alteration of Right Buttock, Percutaneous Approach */
"0Y0047Z"  =  "1"  /* Alteration of R Buttock with Autol Sub, Perc Endo Approach */
"0Y004JZ"  =  "1"  /* Alteration of R Buttock with Synth Sub, Perc Endo Approach */
"0Y004KZ"  =  "1"  /* Alteration of R Buttock with Nonaut Sub, Perc Endo Approach */
"0Y004ZZ"  =  "1"  /* Alteration of Right Buttock, Perc Endo Approach */
"0Y0107Z"  =  "1"  /* Alteration of Left Buttock with Autol Sub, Open Approach */
"0Y010JZ"  =  "1"  /* Alteration of Left Buttock with Synth Sub, Open Approach */
"0Y010KZ"  =  "1"  /* Alteration of Left Buttock with Nonaut Sub, Open Approach */
"0Y010ZZ"  =  "1"  /* Alteration of Left Buttock, Open Approach */
"0Y0137Z"  =  "1"  /* Alteration of Left Buttock with Autol Sub, Perc Approach */
"0Y013JZ"  =  "1"  /* Alteration of Left Buttock with Synth Sub, Perc Approach */
"0Y013KZ"  =  "1"  /* Alteration of Left Buttock with Nonaut Sub, Perc Approach */
"0Y013ZZ"  =  "1"  /* Alteration of Left Buttock, Percutaneous Approach */
"0Y0147Z"  =  "1"  /* Alteration of L Buttock with Autol Sub, Perc Endo Approach */
"0Y014JZ"  =  "1"  /* Alteration of L Buttock with Synth Sub, Perc Endo Approach */
"0Y014KZ"  =  "1"  /* Alteration of L Buttock with Nonaut Sub, Perc Endo Approach */
"0Y014ZZ"  =  "1"  /* Alteration of Left Buttock, Percutaneous Endoscopic Approach */
"0Y0907Z"  =  "1"  /* Alteration of R Low Extrem with Autol Sub, Open Approach */
"0Y090JZ"  =  "1"  /* Alteration of R Low Extrem with Synth Sub, Open Approach */
"0Y090KZ"  =  "1"  /* Alteration of R Low Extrem with Nonaut Sub, Open Approach */
"0Y090ZZ"  =  "1"  /* Alteration of Right Lower Extremity, Open Approach */
"0Y0937Z"  =  "1"  /* Alteration of R Low Extrem with Autol Sub, Perc Approach */
"0Y093JZ"  =  "1"  /* Alteration of R Low Extrem with Synth Sub, Perc Approach */
"0Y093KZ"  =  "1"  /* Alteration of R Low Extrem with Nonaut Sub, Perc Approach */
"0Y093ZZ"  =  "1"  /* Alteration of Right Lower Extremity, Percutaneous Approach */
"0Y0947Z"  =  "1"  /* Alter of R Low Extrem with Autol Sub, Perc Endo Approach */
"0Y094JZ"  =  "1"  /* Alter of R Low Extrem with Synth Sub, Perc Endo Approach */
"0Y094KZ"  =  "1"  /* Alter of R Low Extrem with Nonaut Sub, Perc Endo Approach */
"0Y094ZZ"  =  "1"  /* Alteration of Right Lower Extremity, Perc Endo Approach */
"0Y0B07Z"  =  "1"  /* Alteration of L Low Extrem with Autol Sub, Open Approach */
"0Y0B0JZ"  =  "1"  /* Alteration of L Low Extrem with Synth Sub, Open Approach */
"0Y0B0KZ"  =  "1"  /* Alteration of L Low Extrem with Nonaut Sub, Open Approach */
"0Y0B0ZZ"  =  "1"  /* Alteration of Left Lower Extremity, Open Approach */
"0Y0B37Z"  =  "1"  /* Alteration of L Low Extrem with Autol Sub, Perc Approach */
"0Y0B3JZ"  =  "1"  /* Alteration of L Low Extrem with Synth Sub, Perc Approach */
"0Y0B3KZ"  =  "1"  /* Alteration of L Low Extrem with Nonaut Sub, Perc Approach */
"0Y0B3ZZ"  =  "1"  /* Alteration of Left Lower Extremity, Percutaneous Approach */
"0Y0B47Z"  =  "1"  /* Alter of L Low Extrem with Autol Sub, Perc Endo Approach */
"0Y0B4JZ"  =  "1"  /* Alter of L Low Extrem with Synth Sub, Perc Endo Approach */
"0Y0B4KZ"  =  "1"  /* Alter of L Low Extrem with Nonaut Sub, Perc Endo Approach */
"0Y0B4ZZ"  =  "1"  /* Alteration of Left Lower Extremity, Perc Endo Approach */
"0Y0C07Z"  =  "1"  /* Alteration of Right Upper Leg with Autol Sub, Open Approach */
"0Y0C0JZ"  =  "1"  /* Alteration of Right Upper Leg with Synth Sub, Open Approach */
"0Y0C0KZ"  =  "1"  /* Alteration of Right Upper Leg with Nonaut Sub, Open Approach */
"0Y0C0ZZ"  =  "1"  /* Alteration of Right Upper Leg, Open Approach */
"0Y0C37Z"  =  "1"  /* Alteration of Right Upper Leg with Autol Sub, Perc Approach */
"0Y0C3JZ"  =  "1"  /* Alteration of Right Upper Leg with Synth Sub, Perc Approach */
"0Y0C3KZ"  =  "1"  /* Alteration of Right Upper Leg with Nonaut Sub, Perc Approach */
"0Y0C3ZZ"  =  "1"  /* Alteration of Right Upper Leg, Percutaneous Approach */
"0Y0C47Z"  =  "1"  /* Alteration of R Up Leg with Autol Sub, Perc Endo Approach */
"0Y0C4JZ"  =  "1"  /* Alteration of R Up Leg with Synth Sub, Perc Endo Approach */
"0Y0C4KZ"  =  "1"  /* Alteration of R Up Leg with Nonaut Sub, Perc Endo Approach */
"0Y0C4ZZ"  =  "1"  /* Alteration of Right Upper Leg, Perc Endo Approach */
"0Y0D07Z"  =  "1"  /* Alteration of Left Upper Leg with Autol Sub, Open Approach */
"0Y0D0JZ"  =  "1"  /* Alteration of Left Upper Leg with Synth Sub, Open Approach */
"0Y0D0KZ"  =  "1"  /* Alteration of Left Upper Leg with Nonaut Sub, Open Approach */
"0Y0D0ZZ"  =  "1"  /* Alteration of Left Upper Leg, Open Approach */
"0Y0D37Z"  =  "1"  /* Alteration of Left Upper Leg with Autol Sub, Perc Approach */
"0Y0D3JZ"  =  "1"  /* Alteration of Left Upper Leg with Synth Sub, Perc Approach */
"0Y0D3KZ"  =  "1"  /* Alteration of Left Upper Leg with Nonaut Sub, Perc Approach */
"0Y0D3ZZ"  =  "1"  /* Alteration of Left Upper Leg, Percutaneous Approach */
"0Y0D47Z"  =  "1"  /* Alteration of L Up Leg with Autol Sub, Perc Endo Approach */
"0Y0D4JZ"  =  "1"  /* Alteration of L Up Leg with Synth Sub, Perc Endo Approach */
"0Y0D4KZ"  =  "1"  /* Alteration of L Up Leg with Nonaut Sub, Perc Endo Approach */
"0Y0D4ZZ"  =  "1"  /* Alteration of Left Upper Leg, Perc Endo Approach */
"0Y0F07Z"  =  "1"  /* Alteration of R Knee with Autol Sub, Open Approach */
"0Y0F0JZ"  =  "1"  /* Alteration of R Knee with Synth Sub, Open Approach */
"0Y0F0KZ"  =  "1"  /* Alteration of R Knee with Nonaut Sub, Open Approach */
"0Y0F0ZZ"  =  "1"  /* Alteration of Right Knee Region, Open Approach */
"0Y0F37Z"  =  "1"  /* Alteration of R Knee with Autol Sub, Perc Approach */
"0Y0F3JZ"  =  "1"  /* Alteration of R Knee with Synth Sub, Perc Approach */
"0Y0F3KZ"  =  "1"  /* Alteration of R Knee with Nonaut Sub, Perc Approach */
"0Y0F3ZZ"  =  "1"  /* Alteration of Right Knee Region, Percutaneous Approach */
"0Y0F47Z"  =  "1"  /* Alteration of R Knee with Autol Sub, Perc Endo Approach */
"0Y0F4JZ"  =  "1"  /* Alteration of R Knee with Synth Sub, Perc Endo Approach */
"0Y0F4KZ"  =  "1"  /* Alteration of R Knee with Nonaut Sub, Perc Endo Approach */
"0Y0F4ZZ"  =  "1"  /* Alteration of Right Knee Region, Perc Endo Approach */
"0Y0G07Z"  =  "1"  /* Alteration of Left Knee Region with Autol Sub, Open Approach */
"0Y0G0JZ"  =  "1"  /* Alteration of Left Knee Region with Synth Sub, Open Approach */
"0Y0G0KZ"  =  "1"  /* Alteration of L Knee with Nonaut Sub, Open Approach */
"0Y0G0ZZ"  =  "1"  /* Alteration of Left Knee Region, Open Approach */
"0Y0G37Z"  =  "1"  /* Alteration of Left Knee Region with Autol Sub, Perc Approach */
"0Y0G3JZ"  =  "1"  /* Alteration of Left Knee Region with Synth Sub, Perc Approach */
"0Y0G3KZ"  =  "1"  /* Alteration of L Knee with Nonaut Sub, Perc Approach */
"0Y0G3ZZ"  =  "1"  /* Alteration of Left Knee Region, Percutaneous Approach */
"0Y0G47Z"  =  "1"  /* Alteration of L Knee with Autol Sub, Perc Endo Approach */
"0Y0G4JZ"  =  "1"  /* Alteration of L Knee with Synth Sub, Perc Endo Approach */
"0Y0G4KZ"  =  "1"  /* Alteration of L Knee with Nonaut Sub, Perc Endo Approach */
"0Y0G4ZZ"  =  "1"  /* Alteration of Left Knee Region, Perc Endo Approach */
"0Y0H07Z"  =  "1"  /* Alteration of Right Lower Leg with Autol Sub, Open Approach */
"0Y0H0JZ"  =  "1"  /* Alteration of Right Lower Leg with Synth Sub, Open Approach */
"0Y0H0KZ"  =  "1"  /* Alteration of Right Lower Leg with Nonaut Sub, Open Approach */
"0Y0H0ZZ"  =  "1"  /* Alteration of Right Lower Leg, Open Approach */
"0Y0H37Z"  =  "1"  /* Alteration of Right Lower Leg with Autol Sub, Perc Approach */
"0Y0H3JZ"  =  "1"  /* Alteration of Right Lower Leg with Synth Sub, Perc Approach */
"0Y0H3KZ"  =  "1"  /* Alteration of Right Lower Leg with Nonaut Sub, Perc Approach */
"0Y0H3ZZ"  =  "1"  /* Alteration of Right Lower Leg, Percutaneous Approach */
"0Y0H47Z"  =  "1"  /* Alteration of R Low Leg with Autol Sub, Perc Endo Approach */
"0Y0H4JZ"  =  "1"  /* Alteration of R Low Leg with Synth Sub, Perc Endo Approach */
"0Y0H4KZ"  =  "1"  /* Alteration of R Low Leg with Nonaut Sub, Perc Endo Approach */
"0Y0H4ZZ"  =  "1"  /* Alteration of Right Lower Leg, Perc Endo Approach */
"0Y0J07Z"  =  "1"  /* Alteration of Left Lower Leg with Autol Sub, Open Approach */
"0Y0J0JZ"  =  "1"  /* Alteration of Left Lower Leg with Synth Sub, Open Approach */
"0Y0J0KZ"  =  "1"  /* Alteration of Left Lower Leg with Nonaut Sub, Open Approach */
"0Y0J0ZZ"  =  "1"  /* Alteration of Left Lower Leg, Open Approach */
"0Y0J37Z"  =  "1"  /* Alteration of Left Lower Leg with Autol Sub, Perc Approach */
"0Y0J3JZ"  =  "1"  /* Alteration of Left Lower Leg with Synth Sub, Perc Approach */
"0Y0J3KZ"  =  "1"  /* Alteration of Left Lower Leg with Nonaut Sub, Perc Approach */
"0Y0J3ZZ"  =  "1"  /* Alteration of Left Lower Leg, Percutaneous Approach */
"0Y0J47Z"  =  "1"  /* Alteration of L Low Leg with Autol Sub, Perc Endo Approach */
"0Y0J4JZ"  =  "1"  /* Alteration of L Low Leg with Synth Sub, Perc Endo Approach */
"0Y0J4KZ"  =  "1"  /* Alteration of L Low Leg with Nonaut Sub, Perc Endo Approach */
"0Y0J4ZZ"  =  "1"  /* Alteration of Left Lower Leg, Perc Endo Approach */
"0Y0K07Z"  =  "1"  /* Alteration of R Ankle with Autol Sub, Open Approach */
"0Y0K0JZ"  =  "1"  /* Alteration of R Ankle with Synth Sub, Open Approach */
"0Y0K0KZ"  =  "1"  /* Alteration of R Ankle with Nonaut Sub, Open Approach */
"0Y0K0ZZ"  =  "1"  /* Alteration of Right Ankle Region, Open Approach */
"0Y0K37Z"  =  "1"  /* Alteration of R Ankle with Autol Sub, Perc Approach */
"0Y0K3JZ"  =  "1"  /* Alteration of R Ankle with Synth Sub, Perc Approach */
"0Y0K3KZ"  =  "1"  /* Alteration of R Ankle with Nonaut Sub, Perc Approach */
"0Y0K3ZZ"  =  "1"  /* Alteration of Right Ankle Region, Percutaneous Approach */
"0Y0K47Z"  =  "1"  /* Alteration of R Ankle with Autol Sub, Perc Endo Approach */
"0Y0K4JZ"  =  "1"  /* Alteration of R Ankle with Synth Sub, Perc Endo Approach */
"0Y0K4KZ"  =  "1"  /* Alteration of R Ankle with Nonaut Sub, Perc Endo Approach */
"0Y0K4ZZ"  =  "1"  /* Alteration of Right Ankle Region, Perc Endo Approach */
"0Y0L07Z"  =  "1"  /* Alteration of L Ankle with Autol Sub, Open Approach */
"0Y0L0JZ"  =  "1"  /* Alteration of L Ankle with Synth Sub, Open Approach */
"0Y0L0KZ"  =  "1"  /* Alteration of L Ankle with Nonaut Sub, Open Approach */
"0Y0L0ZZ"  =  "1"  /* Alteration of Left Ankle Region, Open Approach */
"0Y0L37Z"  =  "1"  /* Alteration of L Ankle with Autol Sub, Perc Approach */
"0Y0L3JZ"  =  "1"  /* Alteration of L Ankle with Synth Sub, Perc Approach */
"0Y0L3KZ"  =  "1"  /* Alteration of L Ankle with Nonaut Sub, Perc Approach */
"0Y0L3ZZ"  =  "1"  /* Alteration of Left Ankle Region, Percutaneous Approach */
"0Y0L47Z"  =  "1"  /* Alteration of L Ankle with Autol Sub, Perc Endo Approach */
"0Y0L4JZ"  =  "1"  /* Alteration of L Ankle with Synth Sub, Perc Endo Approach */
"0Y0L4KZ"  =  "1"  /* Alteration of L Ankle with Nonaut Sub, Perc Endo Approach */
"0Y0L4ZZ"  =  "1"  /* Alteration of Left Ankle Region, Perc Endo Approach */
"0Y300ZZ"  =  "1"  /* Control Bleeding in Right Buttock, Open Approach */
"0Y303ZZ"  =  "1"  /* Control Bleeding in Right Buttock, Percutaneous Approach */
"0Y304ZZ"  =  "1"  /* Control Bleeding in Right Buttock, Perc Endo Approach */
"0Y310ZZ"  =  "1"  /* Control Bleeding in Left Buttock, Open Approach */
"0Y313ZZ"  =  "1"  /* Control Bleeding in Left Buttock, Percutaneous Approach */
"0Y314ZZ"  =  "1"  /* Control Bleeding in Left Buttock, Perc Endo Approach */
"0Y350ZZ"  =  "1"  /* Control Bleeding in Right Inguinal Region, Open Approach */
"0Y353ZZ"  =  "1"  /* Control Bleeding in Right Inguinal Region, Perc Approach */
"0Y354ZZ"  =  "1"  /* Control Bleeding in R Inguinal Region, Perc Endo Approach */
"0Y360ZZ"  =  "1"  /* Control Bleeding in Left Inguinal Region, Open Approach */
"0Y363ZZ"  =  "1"  /* Control Bleeding in Left Inguinal Region, Perc Approach */
"0Y364ZZ"  =  "1"  /* Control Bleeding in Left Inguinal Region, Perc Endo Approach */
"0Y370ZZ"  =  "1"  /* Control Bleeding in Right Femoral Region, Open Approach */
"0Y373ZZ"  =  "1"  /* Control Bleeding in Right Femoral Region, Perc Approach */
"0Y374ZZ"  =  "1"  /* Control Bleeding in Right Femoral Region, Perc Endo Approach */
"0Y380ZZ"  =  "1"  /* Control Bleeding in Left Femoral Region, Open Approach */
"0Y383ZZ"  =  "1"  /* Control Bleeding in Left Femoral Region, Perc Approach */
"0Y384ZZ"  =  "1"  /* Control Bleeding in Left Femoral Region, Perc Endo Approach */
"0Y390ZZ"  =  "1"  /* Control Bleeding in Right Lower Extremity, Open Approach */
"0Y393ZZ"  =  "1"  /* Control Bleeding in Right Lower Extremity, Perc Approach */
"0Y394ZZ"  =  "1"  /* Control Bleeding in R Low Extrem, Perc Endo Approach */
"0Y3B0ZZ"  =  "1"  /* Control Bleeding in Left Lower Extremity, Open Approach */
"0Y3B3ZZ"  =  "1"  /* Control Bleeding in Left Lower Extremity, Perc Approach */
"0Y3B4ZZ"  =  "1"  /* Control Bleeding in Left Lower Extremity, Perc Endo Approach */
"0Y3C0ZZ"  =  "1"  /* Control Bleeding in Right Upper Leg, Open Approach */
"0Y3C3ZZ"  =  "1"  /* Control Bleeding in Right Upper Leg, Percutaneous Approach */
"0Y3C4ZZ"  =  "1"  /* Control Bleeding in Right Upper Leg, Perc Endo Approach */
"0Y3D0ZZ"  =  "1"  /* Control Bleeding in Left Upper Leg, Open Approach */
"0Y3D3ZZ"  =  "1"  /* Control Bleeding in Left Upper Leg, Percutaneous Approach */
"0Y3D4ZZ"  =  "1"  /* Control Bleeding in Left Upper Leg, Perc Endo Approach */
"0Y3F0ZZ"  =  "1"  /* Control Bleeding in Right Knee Region, Open Approach */
"0Y3F3ZZ"  =  "1"  /* Control Bleeding in Right Knee Region, Percutaneous Approach */
"0Y3F4ZZ"  =  "1"  /* Control Bleeding in Right Knee Region, Perc Endo Approach */
"0Y3G0ZZ"  =  "1"  /* Control Bleeding in Left Knee Region, Open Approach */
"0Y3G3ZZ"  =  "1"  /* Control Bleeding in Left Knee Region, Percutaneous Approach */
"0Y3G4ZZ"  =  "1"  /* Control Bleeding in Left Knee Region, Perc Endo Approach */
"0Y3H0ZZ"  =  "1"  /* Control Bleeding in Right Lower Leg, Open Approach */
"0Y3H3ZZ"  =  "1"  /* Control Bleeding in Right Lower Leg, Percutaneous Approach */
"0Y3H4ZZ"  =  "1"  /* Control Bleeding in Right Lower Leg, Perc Endo Approach */
"0Y3J0ZZ"  =  "1"  /* Control Bleeding in Left Lower Leg, Open Approach */
"0Y3J3ZZ"  =  "1"  /* Control Bleeding in Left Lower Leg, Percutaneous Approach */
"0Y3J4ZZ"  =  "1"  /* Control Bleeding in Left Lower Leg, Perc Endo Approach */
"0Y3K0ZZ"  =  "1"  /* Control Bleeding in Right Ankle Region, Open Approach */
"0Y3K3ZZ"  =  "1"  /* Control Bleeding in Right Ankle Region, Perc Approach */
"0Y3K4ZZ"  =  "1"  /* Control Bleeding in Right Ankle Region, Perc Endo Approach */
"0Y3L0ZZ"  =  "1"  /* Control Bleeding in Left Ankle Region, Open Approach */
"0Y3L3ZZ"  =  "1"  /* Control Bleeding in Left Ankle Region, Percutaneous Approach */
"0Y3L4ZZ"  =  "1"  /* Control Bleeding in Left Ankle Region, Perc Endo Approach */
"0Y3M0ZZ"  =  "1"  /* Control Bleeding in Right Foot, Open Approach */
"0Y3M3ZZ"  =  "1"  /* Control Bleeding in Right Foot, Percutaneous Approach */
"0Y3M4ZZ"  =  "1"  /* Control Bleeding in Right Foot, Perc Endo Approach */
"0Y3N0ZZ"  =  "1"  /* Control Bleeding in Left Foot, Open Approach */
"0Y3N3ZZ"  =  "1"  /* Control Bleeding in Left Foot, Percutaneous Approach */
"0Y3N4ZZ"  =  "1"  /* Control Bleeding in Left Foot, Perc Endo Approach */
"0Y620ZZ"  =  "1"  /* Detachment at Right Hindquarter, Open Approach */
"0Y630ZZ"  =  "1"  /* Detachment at Left Hindquarter, Open Approach */
"0Y640ZZ"  =  "1"  /* Detachment at Bilateral Hindquarter, Open Approach */
"0Y670ZZ"  =  "1"  /* Detachment at Right Femoral Region, Open Approach */
"0Y680ZZ"  =  "1"  /* Detachment at Left Femoral Region, Open Approach */
"0Y6C0Z1"  =  "1"  /* Detachment at Right Upper Leg, High, Open Approach */
"0Y6C0Z2"  =  "1"  /* Detachment at Right Upper Leg, Mid, Open Approach */
"0Y6C0Z3"  =  "1"  /* Detachment at Right Upper Leg, Low, Open Approach */
"0Y6D0Z1"  =  "1"  /* Detachment at Left Upper Leg, High, Open Approach */
"0Y6D0Z2"  =  "1"  /* Detachment at Left Upper Leg, Mid, Open Approach */
"0Y6D0Z3"  =  "1"  /* Detachment at Left Upper Leg, Low, Open Approach */
"0Y6F0ZZ"  =  "1"  /* Detachment at Right Knee Region, Open Approach */
"0Y6G0ZZ"  =  "1"  /* Detachment at Left Knee Region, Open Approach */
"0Y6H0Z1"  =  "1"  /* Detachment at Right Lower Leg, High, Open Approach */
"0Y6H0Z2"  =  "1"  /* Detachment at Right Lower Leg, Mid, Open Approach */
"0Y6H0Z3"  =  "1"  /* Detachment at Right Lower Leg, Low, Open Approach */
"0Y6J0Z1"  =  "1"  /* Detachment at Left Lower Leg, High, Open Approach */
"0Y6J0Z2"  =  "1"  /* Detachment at Left Lower Leg, Mid, Open Approach */
"0Y6J0Z3"  =  "1"  /* Detachment at Left Lower Leg, Low, Open Approach */
"0Y6M0Z0"  =  "1"  /* Detachment at Right Foot, Complete, Open Approach */
"0Y6M0Z4"  =  "1"  /* Detachment at Right Foot, Complete 1st Ray, Open Approach */
"0Y6M0Z5"  =  "1"  /* Detachment at Right Foot, Complete 2nd Ray, Open Approach */
"0Y6M0Z6"  =  "1"  /* Detachment at Right Foot, Complete 3rd Ray, Open Approach */
"0Y6M0Z7"  =  "1"  /* Detachment at Right Foot, Complete 4th Ray, Open Approach */
"0Y6M0Z8"  =  "1"  /* Detachment at Right Foot, Complete 5th Ray, Open Approach */
"0Y6M0Z9"  =  "1"  /* Detachment at Right Foot, Partial 1st Ray, Open Approach */
"0Y6M0ZB"  =  "1"  /* Detachment at Right Foot, Partial 2nd Ray, Open Approach */
"0Y6M0ZC"  =  "1"  /* Detachment at Right Foot, Partial 3rd Ray, Open Approach */
"0Y6M0ZD"  =  "1"  /* Detachment at Right Foot, Partial 4th Ray, Open Approach */
"0Y6M0ZF"  =  "1"  /* Detachment at Right Foot, Partial 5th Ray, Open Approach */
"0Y6N0Z0"  =  "1"  /* Detachment at Left Foot, Complete, Open Approach */
"0Y6N0Z4"  =  "1"  /* Detachment at Left Foot, Complete 1st Ray, Open Approach */
"0Y6N0Z5"  =  "1"  /* Detachment at Left Foot, Complete 2nd Ray, Open Approach */
"0Y6N0Z6"  =  "1"  /* Detachment at Left Foot, Complete 3rd Ray, Open Approach */
"0Y6N0Z7"  =  "1"  /* Detachment at Left Foot, Complete 4th Ray, Open Approach */
"0Y6N0Z8"  =  "1"  /* Detachment at Left Foot, Complete 5th Ray, Open Approach */
"0Y6N0Z9"  =  "1"  /* Detachment at Left Foot, Partial 1st Ray, Open Approach */
"0Y6N0ZB"  =  "1"  /* Detachment at Left Foot, Partial 2nd Ray, Open Approach */
"0Y6N0ZC"  =  "1"  /* Detachment at Left Foot, Partial 3rd Ray, Open Approach */
"0Y6N0ZD"  =  "1"  /* Detachment at Left Foot, Partial 4th Ray, Open Approach */
"0Y6N0ZF"  =  "1"  /* Detachment at Left Foot, Partial 5th Ray, Open Approach */
"0Y6P0Z0"  =  "1"  /* Detachment at Right 1st Toe, Complete, Open Approach */
"0Y6P0Z1"  =  "1"  /* Detachment at Right 1st Toe, High, Open Approach */
"0Y6P0Z2"  =  "1"  /* Detachment at Right 1st Toe, Mid, Open Approach */
"0Y6P0Z3"  =  "1"  /* Detachment at Right 1st Toe, Low, Open Approach */
"0Y6Q0Z0"  =  "1"  /* Detachment at Left 1st Toe, Complete, Open Approach */
"0Y6Q0Z1"  =  "1"  /* Detachment at Left 1st Toe, High, Open Approach */
"0Y6Q0Z2"  =  "1"  /* Detachment at Left 1st Toe, Mid, Open Approach */
"0Y6Q0Z3"  =  "1"  /* Detachment at Left 1st Toe, Low, Open Approach */
"0Y6R0Z0"  =  "1"  /* Detachment at Right 2nd Toe, Complete, Open Approach */
"0Y6R0Z1"  =  "1"  /* Detachment at Right 2nd Toe, High, Open Approach */
"0Y6R0Z2"  =  "1"  /* Detachment at Right 2nd Toe, Mid, Open Approach */
"0Y6R0Z3"  =  "1"  /* Detachment at Right 2nd Toe, Low, Open Approach */
"0Y6S0Z0"  =  "1"  /* Detachment at Left 2nd Toe, Complete, Open Approach */
"0Y6S0Z1"  =  "1"  /* Detachment at Left 2nd Toe, High, Open Approach */
"0Y6S0Z2"  =  "1"  /* Detachment at Left 2nd Toe, Mid, Open Approach */
"0Y6S0Z3"  =  "1"  /* Detachment at Left 2nd Toe, Low, Open Approach */
"0Y6T0Z0"  =  "1"  /* Detachment at Right 3rd Toe, Complete, Open Approach */
"0Y6T0Z1"  =  "1"  /* Detachment at Right 3rd Toe, High, Open Approach */
"0Y6T0Z2"  =  "1"  /* Detachment at Right 3rd Toe, Mid, Open Approach */
"0Y6T0Z3"  =  "1"  /* Detachment at Right 3rd Toe, Low, Open Approach */
"0Y6U0Z0"  =  "1"  /* Detachment at Left 3rd Toe, Complete, Open Approach */
"0Y6U0Z1"  =  "1"  /* Detachment at Left 3rd Toe, High, Open Approach */
"0Y6U0Z2"  =  "1"  /* Detachment at Left 3rd Toe, Mid, Open Approach */
"0Y6U0Z3"  =  "1"  /* Detachment at Left 3rd Toe, Low, Open Approach */
"0Y6V0Z0"  =  "1"  /* Detachment at Right 4th Toe, Complete, Open Approach */
"0Y6V0Z1"  =  "1"  /* Detachment at Right 4th Toe, High, Open Approach */
"0Y6V0Z2"  =  "1"  /* Detachment at Right 4th Toe, Mid, Open Approach */
"0Y6V0Z3"  =  "1"  /* Detachment at Right 4th Toe, Low, Open Approach */
"0Y6W0Z0"  =  "1"  /* Detachment at Left 4th Toe, Complete, Open Approach */
"0Y6W0Z1"  =  "1"  /* Detachment at Left 4th Toe, High, Open Approach */
"0Y6W0Z2"  =  "1"  /* Detachment at Left 4th Toe, Mid, Open Approach */
"0Y6W0Z3"  =  "1"  /* Detachment at Left 4th Toe, Low, Open Approach */
"0Y6X0Z0"  =  "1"  /* Detachment at Right 5th Toe, Complete, Open Approach */
"0Y6X0Z1"  =  "1"  /* Detachment at Right 5th Toe, High, Open Approach */
"0Y6X0Z2"  =  "1"  /* Detachment at Right 5th Toe, Mid, Open Approach */
"0Y6X0Z3"  =  "1"  /* Detachment at Right 5th Toe, Low, Open Approach */
"0Y6Y0Z0"  =  "1"  /* Detachment at Left 5th Toe, Complete, Open Approach */
"0Y6Y0Z1"  =  "1"  /* Detachment at Left 5th Toe, High, Open Approach */
"0Y6Y0Z2"  =  "1"  /* Detachment at Left 5th Toe, Mid, Open Approach */
"0Y6Y0Z3"  =  "1"  /* Detachment at Left 5th Toe, Low, Open Approach */
"0Y9500Z"  =  "1"  /* Drainage of R Inguinal Region with Drain Dev, Open Approach */
"0Y950ZX"  =  "1"  /* Drainage of Right Inguinal Region, Open Approach, Diagnostic */
"0Y950ZZ"  =  "1"  /* Drainage of Right Inguinal Region, Open Approach */
"0Y9530Z"  =  "1"  /* Drainage of R Inguinal Region with Drain Dev, Perc Approach */
"0Y953ZX"  =  "1"  /* Drainage of Right Inguinal Region, Perc Approach, Diagn */
"0Y953ZZ"  =  "1"  /* Drainage of Right Inguinal Region, Percutaneous Approach */
"0Y9540Z"  =  "1"  /* Drain R Inguinal Region w Drain Dev, Perc Endo */
"0Y954ZX"  =  "1"  /* Drainage of Right Inguinal Region, Perc Endo Approach, Diagn */
"0Y954ZZ"  =  "1"  /* Drainage of Right Inguinal Region, Perc Endo Approach */
"0Y9600Z"  =  "1"  /* Drainage of L Inguinal Region with Drain Dev, Open Approach */
"0Y960ZX"  =  "1"  /* Drainage of Left Inguinal Region, Open Approach, Diagnostic */
"0Y960ZZ"  =  "1"  /* Drainage of Left Inguinal Region, Open Approach */
"0Y9630Z"  =  "1"  /* Drainage of L Inguinal Region with Drain Dev, Perc Approach */
"0Y963ZX"  =  "1"  /* Drainage of Left Inguinal Region, Perc Approach, Diagn */
"0Y963ZZ"  =  "1"  /* Drainage of Left Inguinal Region, Percutaneous Approach */
"0Y9640Z"  =  "1"  /* Drain L Inguinal Region w Drain Dev, Perc Endo */
"0Y964ZX"  =  "1"  /* Drainage of Left Inguinal Region, Perc Endo Approach, Diagn */
"0Y964ZZ"  =  "1"  /* Drainage of Left Inguinal Region, Perc Endo Approach */
"0YB00ZZ"  =  "1"  /* Excision of Right Buttock, Open Approach */
"0YB03ZZ"  =  "1"  /* Excision of Right Buttock, Percutaneous Approach */
"0YB04ZZ"  =  "1"  /* Excision of Right Buttock, Percutaneous Endoscopic Approach */
"0YB10ZZ"  =  "1"  /* Excision of Left Buttock, Open Approach */
"0YB13ZZ"  =  "1"  /* Excision of Left Buttock, Percutaneous Approach */
"0YB14ZZ"  =  "1"  /* Excision of Left Buttock, Percutaneous Endoscopic Approach */
"0YB50ZX"  =  "1"  /* Excision of Right Inguinal Region, Open Approach, Diagnostic */
"0YB50ZZ"  =  "1"  /* Excision of Right Inguinal Region, Open Approach */
"0YB53ZX"  =  "1"  /* Excision of Right Inguinal Region, Perc Approach, Diagn */
"0YB53ZZ"  =  "1"  /* Excision of Right Inguinal Region, Percutaneous Approach */
"0YB54ZX"  =  "1"  /* Excision of Right Inguinal Region, Perc Endo Approach, Diagn */
"0YB54ZZ"  =  "1"  /* Excision of Right Inguinal Region, Perc Endo Approach */
"0YB60ZX"  =  "1"  /* Excision of Left Inguinal Region, Open Approach, Diagnostic */
"0YB60ZZ"  =  "1"  /* Excision of Left Inguinal Region, Open Approach */
"0YB63ZX"  =  "1"  /* Excision of Left Inguinal Region, Perc Approach, Diagn */
"0YB63ZZ"  =  "1"  /* Excision of Left Inguinal Region, Percutaneous Approach */
"0YB64ZX"  =  "1"  /* Excision of Left Inguinal Region, Perc Endo Approach, Diagn */
"0YB64ZZ"  =  "1"  /* Excision of Left Inguinal Region, Perc Endo Approach */
"0YB70ZX"  =  "1"  /* Excision of Right Femoral Region, Open Approach, Diagnostic */
"0YB70ZZ"  =  "1"  /* Excision of Right Femoral Region, Open Approach */
"0YB73ZX"  =  "1"  /* Excision of Right Femoral Region, Perc Approach, Diagn */
"0YB73ZZ"  =  "1"  /* Excision of Right Femoral Region, Percutaneous Approach */
"0YB74ZX"  =  "1"  /* Excision of Right Femoral Region, Perc Endo Approach, Diagn */
"0YB74ZZ"  =  "1"  /* Excision of Right Femoral Region, Perc Endo Approach */
"0YB80ZX"  =  "1"  /* Excision of Left Femoral Region, Open Approach, Diagnostic */
"0YB80ZZ"  =  "1"  /* Excision of Left Femoral Region, Open Approach */
"0YB83ZX"  =  "1"  /* Excision of Left Femoral Region, Perc Approach, Diagn */
"0YB83ZZ"  =  "1"  /* Excision of Left Femoral Region, Percutaneous Approach */
"0YB84ZX"  =  "1"  /* Excision of Left Femoral Region, Perc Endo Approach, Diagn */
"0YB84ZZ"  =  "1"  /* Excision of Left Femoral Region, Perc Endo Approach */
"0YB90ZZ"  =  "1"  /* Excision of Right Lower Extremity, Open Approach */
"0YB93ZZ"  =  "1"  /* Excision of Right Lower Extremity, Percutaneous Approach */
"0YB94ZZ"  =  "1"  /* Excision of Right Lower Extremity, Perc Endo Approach */
"0YBB0ZZ"  =  "1"  /* Excision of Left Lower Extremity, Open Approach */
"0YBB3ZZ"  =  "1"  /* Excision of Left Lower Extremity, Percutaneous Approach */
"0YBB4ZZ"  =  "1"  /* Excision of Left Lower Extremity, Perc Endo Approach */
"0YBC0ZZ"  =  "1"  /* Excision of Right Upper Leg, Open Approach */
"0YBC3ZZ"  =  "1"  /* Excision of Right Upper Leg, Percutaneous Approach */
"0YBC4ZZ"  =  "1"  /* Excision of Right Upper Leg, Perc Endo Approach */
"0YBD0ZZ"  =  "1"  /* Excision of Left Upper Leg, Open Approach */
"0YBD3ZZ"  =  "1"  /* Excision of Left Upper Leg, Percutaneous Approach */
"0YBD4ZZ"  =  "1"  /* Excision of Left Upper Leg, Percutaneous Endoscopic Approach */
"0YBF0ZZ"  =  "1"  /* Excision of Right Knee Region, Open Approach */
"0YBF3ZZ"  =  "1"  /* Excision of Right Knee Region, Percutaneous Approach */
"0YBF4ZZ"  =  "1"  /* Excision of Right Knee Region, Perc Endo Approach */
"0YBG0ZZ"  =  "1"  /* Excision of Left Knee Region, Open Approach */
"0YBG3ZZ"  =  "1"  /* Excision of Left Knee Region, Percutaneous Approach */
"0YBG4ZZ"  =  "1"  /* Excision of Left Knee Region, Perc Endo Approach */
"0YBH0ZZ"  =  "1"  /* Excision of Right Lower Leg, Open Approach */
"0YBH3ZZ"  =  "1"  /* Excision of Right Lower Leg, Percutaneous Approach */
"0YBH4ZZ"  =  "1"  /* Excision of Right Lower Leg, Perc Endo Approach */
"0YBJ0ZZ"  =  "1"  /* Excision of Left Lower Leg, Open Approach */
"0YBJ3ZZ"  =  "1"  /* Excision of Left Lower Leg, Percutaneous Approach */
"0YBJ4ZZ"  =  "1"  /* Excision of Left Lower Leg, Percutaneous Endoscopic Approach */
"0YBK0ZZ"  =  "1"  /* Excision of Right Ankle Region, Open Approach */
"0YBK3ZZ"  =  "1"  /* Excision of Right Ankle Region, Percutaneous Approach */
"0YBK4ZZ"  =  "1"  /* Excision of Right Ankle Region, Perc Endo Approach */
"0YBL0ZZ"  =  "1"  /* Excision of Left Ankle Region, Open Approach */
"0YBL3ZZ"  =  "1"  /* Excision of Left Ankle Region, Percutaneous Approach */
"0YBL4ZZ"  =  "1"  /* Excision of Left Ankle Region, Perc Endo Approach */
"0YBM0ZZ"  =  "1"  /* Excision of Right Foot, Open Approach */
"0YBM3ZZ"  =  "1"  /* Excision of Right Foot, Percutaneous Approach */
"0YBM4ZZ"  =  "1"  /* Excision of Right Foot, Percutaneous Endoscopic Approach */
"0YBN0ZZ"  =  "1"  /* Excision of Left Foot, Open Approach */
"0YBN3ZZ"  =  "1"  /* Excision of Left Foot, Percutaneous Approach */
"0YBN4ZZ"  =  "1"  /* Excision of Left Foot, Percutaneous Endoscopic Approach */
"0YH001Z"  =  "1"  /* Insertion of Radioact Elem into R Buttock, Open Approach */
"0YH031Z"  =  "1"  /* Insertion of Radioact Elem into R Buttock, Perc Approach */
"0YH041Z"  =  "1"  /* Insert of Radioact Elem into R Buttock, Perc Endo Approach */
"0YH101Z"  =  "1"  /* Insertion of Radioact Elem into L Buttock, Open Approach */
"0YH131Z"  =  "1"  /* Insertion of Radioact Elem into L Buttock, Perc Approach */
"0YH141Z"  =  "1"  /* Insert of Radioact Elem into L Buttock, Perc Endo Approach */
"0YH501Z"  =  "1"  /* Insert Radioact Elem in R Inguinal Region, Open */
"0YH531Z"  =  "1"  /* Insert Radioact Elem in R Inguinal Region, Perc */
"0YH541Z"  =  "1"  /* Insert Radioact Elem in R Inguinal Region, Perc Endo */
"0YH601Z"  =  "1"  /* Insert Radioact Elem in L Inguinal Region, Open */
"0YH631Z"  =  "1"  /* Insert Radioact Elem in L Inguinal Region, Perc */
"0YH641Z"  =  "1"  /* Insert Radioact Elem in L Inguinal Region, Perc Endo */
"0YH701Z"  =  "1"  /* Insert of Radioact Elem into R Femoral Region, Open Approach */
"0YH731Z"  =  "1"  /* Insert of Radioact Elem into R Femoral Region, Perc Approach */
"0YH741Z"  =  "1"  /* Insert Radioact Elem in R Femoral Region, Perc Endo */
"0YH801Z"  =  "1"  /* Insert of Radioact Elem into L Femoral Region, Open Approach */
"0YH831Z"  =  "1"  /* Insert of Radioact Elem into L Femoral Region, Perc Approach */
"0YH841Z"  =  "1"  /* Insert Radioact Elem in L Femoral Region, Perc Endo */
"0YH901Z"  =  "1"  /* Insertion of Radioact Elem into R Low Extrem, Open Approach */
"0YH931Z"  =  "1"  /* Insertion of Radioact Elem into R Low Extrem, Perc Approach */
"0YH941Z"  =  "1"  /* Insert Radioact Elem in R Low Extrem, Perc Endo */
"0YHB01Z"  =  "1"  /* Insertion of Radioact Elem into L Low Extrem, Open Approach */
"0YHB31Z"  =  "1"  /* Insertion of Radioact Elem into L Low Extrem, Perc Approach */
"0YHB41Z"  =  "1"  /* Insert Radioact Elem in L Low Extrem, Perc Endo */
"0YHC01Z"  =  "1"  /* Insertion of Radioact Elem into R Up Leg, Open Approach */
"0YHC31Z"  =  "1"  /* Insertion of Radioact Elem into R Up Leg, Perc Approach */
"0YHC41Z"  =  "1"  /* Insertion of Radioact Elem into R Up Leg, Perc Endo Approach */
"0YHD01Z"  =  "1"  /* Insertion of Radioact Elem into L Up Leg, Open Approach */
"0YHD31Z"  =  "1"  /* Insertion of Radioact Elem into L Up Leg, Perc Approach */
"0YHD41Z"  =  "1"  /* Insertion of Radioact Elem into L Up Leg, Perc Endo Approach */
"0YHF01Z"  =  "1"  /* Insertion of Radioactive Element into R Knee, Open Approach */
"0YHF31Z"  =  "1"  /* Insertion of Radioactive Element into R Knee, Perc Approach */
"0YHF41Z"  =  "1"  /* Insertion of Radioact Elem into R Knee, Perc Endo Approach */
"0YHG01Z"  =  "1"  /* Insertion of Radioactive Element into L Knee, Open Approach */
"0YHG31Z"  =  "1"  /* Insertion of Radioactive Element into L Knee, Perc Approach */
"0YHG41Z"  =  "1"  /* Insertion of Radioact Elem into L Knee, Perc Endo Approach */
"0YHH01Z"  =  "1"  /* Insertion of Radioact Elem into R Low Leg, Open Approach */
"0YHH31Z"  =  "1"  /* Insertion of Radioact Elem into R Low Leg, Perc Approach */
"0YHH41Z"  =  "1"  /* Insert of Radioact Elem into R Low Leg, Perc Endo Approach */
"0YHJ01Z"  =  "1"  /* Insertion of Radioact Elem into L Low Leg, Open Approach */
"0YHJ31Z"  =  "1"  /* Insertion of Radioact Elem into L Low Leg, Perc Approach */
"0YHJ41Z"  =  "1"  /* Insert of Radioact Elem into L Low Leg, Perc Endo Approach */
"0YHK01Z"  =  "1"  /* Insertion of Radioactive Element into R Ankle, Open Approach */
"0YHK31Z"  =  "1"  /* Insertion of Radioactive Element into R Ankle, Perc Approach */
"0YHK41Z"  =  "1"  /* Insertion of Radioact Elem into R Ankle, Perc Endo Approach */
"0YHL01Z"  =  "1"  /* Insertion of Radioactive Element into L Ankle, Open Approach */
"0YHL31Z"  =  "1"  /* Insertion of Radioactive Element into L Ankle, Perc Approach */
"0YHL41Z"  =  "1"  /* Insertion of Radioact Elem into L Ankle, Perc Endo Approach */
"0YHM01Z"  =  "1"  /* Insertion of Radioactive Element into R Foot, Open Approach */
"0YHM31Z"  =  "1"  /* Insertion of Radioactive Element into R Foot, Perc Approach */
"0YHM41Z"  =  "1"  /* Insertion of Radioact Elem into R Foot, Perc Endo Approach */
"0YHN01Z"  =  "1"  /* Insertion of Radioactive Element into L Foot, Open Approach */
"0YHN31Z"  =  "1"  /* Insertion of Radioactive Element into L Foot, Perc Approach */
"0YHN41Z"  =  "1"  /* Insertion of Radioact Elem into L Foot, Perc Endo Approach */
"0YJ50ZZ"  =  "1"  /* Inspection of Right Inguinal Region, Open Approach */
"0YJ53ZZ"  =  "1"  /* Inspection of Right Inguinal Region, Percutaneous Approach */
"0YJ54ZZ"  =  "1"  /* Inspection of Right Inguinal Region, Perc Endo Approach */
"0YJ60ZZ"  =  "1"  /* Inspection of Left Inguinal Region, Open Approach */
"0YJ63ZZ"  =  "1"  /* Inspection of Left Inguinal Region, Percutaneous Approach */
"0YJ64ZZ"  =  "1"  /* Inspection of Left Inguinal Region, Perc Endo Approach */
"0YJ70ZZ"  =  "1"  /* Inspection of Right Femoral Region, Open Approach */
"0YJ73ZZ"  =  "1"  /* Inspection of Right Femoral Region, Percutaneous Approach */
"0YJ74ZZ"  =  "1"  /* Inspection of Right Femoral Region, Perc Endo Approach */
"0YJ83ZZ"  =  "1"  /* Inspection of Left Femoral Region, Percutaneous Approach */
"0YJ84ZZ"  =  "1"  /* Inspection of Left Femoral Region, Perc Endo Approach */
"0YJA0ZZ"  =  "1"  /* Inspection of Bilateral Inguinal Region, Open Approach */
"0YJA3ZZ"  =  "1"  /* Inspection of Bilateral Inguinal Region, Perc Approach */
"0YJA4ZZ"  =  "1"  /* Inspection of Bilateral Inguinal Region, Perc Endo Approach */
"0YJE3ZZ"  =  "1"  /* Inspection of Bilateral Femoral Region, Perc Approach */
"0YJE4ZZ"  =  "1"  /* Inspection of Bilateral Femoral Region, Perc Endo Approach */
"0YM00ZZ"  =  "1"  /* Reattachment of Right Buttock, Open Approach */
"0YM10ZZ"  =  "1"  /* Reattachment of Left Buttock, Open Approach */
"0YM20ZZ"  =  "1"  /* Reattachment of Right Hindquarter, Open Approach */
"0YM30ZZ"  =  "1"  /* Reattachment of Left Hindquarter, Open Approach */
"0YM40ZZ"  =  "1"  /* Reattachment of Bilateral Hindquarter, Open Approach */
"0YM50ZZ"  =  "1"  /* Reattachment of Right Inguinal Region, Open Approach */
"0YM60ZZ"  =  "1"  /* Reattachment of Left Inguinal Region, Open Approach */
"0YM70ZZ"  =  "1"  /* Reattachment of Right Femoral Region, Open Approach */
"0YM80ZZ"  =  "1"  /* Reattachment of Left Femoral Region, Open Approach */
"0YM90ZZ"  =  "1"  /* Reattachment of Right Lower Extremity, Open Approach */
"0YMB0ZZ"  =  "1"  /* Reattachment of Left Lower Extremity, Open Approach */
"0YMC0ZZ"  =  "1"  /* Reattachment of Right Upper Leg, Open Approach */
"0YMD0ZZ"  =  "1"  /* Reattachment of Left Upper Leg, Open Approach */
"0YMF0ZZ"  =  "1"  /* Reattachment of Right Knee Region, Open Approach */
"0YMG0ZZ"  =  "1"  /* Reattachment of Left Knee Region, Open Approach */
"0YMH0ZZ"  =  "1"  /* Reattachment of Right Lower Leg, Open Approach */
"0YMJ0ZZ"  =  "1"  /* Reattachment of Left Lower Leg, Open Approach */
"0YMK0ZZ"  =  "1"  /* Reattachment of Right Ankle Region, Open Approach */
"0YML0ZZ"  =  "1"  /* Reattachment of Left Ankle Region, Open Approach */
"0YMM0ZZ"  =  "1"  /* Reattachment of Right Foot, Open Approach */
"0YMN0ZZ"  =  "1"  /* Reattachment of Left Foot, Open Approach */
"0YMP0ZZ"  =  "1"  /* Reattachment of Right 1st Toe, Open Approach */
"0YMQ0ZZ"  =  "1"  /* Reattachment of Left 1st Toe, Open Approach */
"0YMR0ZZ"  =  "1"  /* Reattachment of Right 2nd Toe, Open Approach */
"0YMS0ZZ"  =  "1"  /* Reattachment of Left 2nd Toe, Open Approach */
"0YMT0ZZ"  =  "1"  /* Reattachment of Right 3rd Toe, Open Approach */
"0YMU0ZZ"  =  "1"  /* Reattachment of Left 3rd Toe, Open Approach */
"0YMV0ZZ"  =  "1"  /* Reattachment of Right 4th Toe, Open Approach */
"0YMW0ZZ"  =  "1"  /* Reattachment of Left 4th Toe, Open Approach */
"0YMX0ZZ"  =  "1"  /* Reattachment of Right 5th Toe, Open Approach */
"0YMY0ZZ"  =  "1"  /* Reattachment of Left 5th Toe, Open Approach */
"0YQ00ZZ"  =  "1"  /* Repair Right Buttock, Open Approach */
"0YQ03ZZ"  =  "1"  /* Repair Right Buttock, Percutaneous Approach */
"0YQ04ZZ"  =  "1"  /* Repair Right Buttock, Percutaneous Endoscopic Approach */
"0YQ0XZZ"  =  "1"  /* Repair Right Buttock, External Approach */
"0YQ10ZZ"  =  "1"  /* Repair Left Buttock, Open Approach */
"0YQ13ZZ"  =  "1"  /* Repair Left Buttock, Percutaneous Approach */
"0YQ14ZZ"  =  "1"  /* Repair Left Buttock, Percutaneous Endoscopic Approach */
"0YQ1XZZ"  =  "1"  /* Repair Left Buttock, External Approach */
"0YQ50ZZ"  =  "1"  /* Repair Right Inguinal Region, Open Approach */
"0YQ53ZZ"  =  "1"  /* Repair Right Inguinal Region, Percutaneous Approach */
"0YQ54ZZ"  =  "1"  /* Repair Right Inguinal Region, Perc Endo Approach */
"0YQ60ZZ"  =  "1"  /* Repair Left Inguinal Region, Open Approach */
"0YQ63ZZ"  =  "1"  /* Repair Left Inguinal Region, Percutaneous Approach */
"0YQ64ZZ"  =  "1"  /* Repair Left Inguinal Region, Perc Endo Approach */
"0YQ70ZZ"  =  "1"  /* Repair Right Femoral Region, Open Approach */
"0YQ73ZZ"  =  "1"  /* Repair Right Femoral Region, Percutaneous Approach */
"0YQ74ZZ"  =  "1"  /* Repair Right Femoral Region, Perc Endo Approach */
"0YQ80ZZ"  =  "1"  /* Repair Left Femoral Region, Open Approach */
"0YQ83ZZ"  =  "1"  /* Repair Left Femoral Region, Percutaneous Approach */
"0YQ84ZZ"  =  "1"  /* Repair Left Femoral Region, Percutaneous Endoscopic Approach */
"0YQ90ZZ"  =  "1"  /* Repair Right Lower Extremity, Open Approach */
"0YQ93ZZ"  =  "1"  /* Repair Right Lower Extremity, Percutaneous Approach */
"0YQ94ZZ"  =  "1"  /* Repair Right Lower Extremity, Perc Endo Approach */
"0YQ9XZZ"  =  "1"  /* Repair Right Lower Extremity, External Approach */
"0YQA0ZZ"  =  "1"  /* Repair Bilateral Inguinal Region, Open Approach */
"0YQA3ZZ"  =  "1"  /* Repair Bilateral Inguinal Region, Percutaneous Approach */
"0YQA4ZZ"  =  "1"  /* Repair Bilateral Inguinal Region, Perc Endo Approach */
"0YQB0ZZ"  =  "1"  /* Repair Left Lower Extremity, Open Approach */
"0YQB3ZZ"  =  "1"  /* Repair Left Lower Extremity, Percutaneous Approach */
"0YQB4ZZ"  =  "1"  /* Repair Left Lower Extremity, Perc Endo Approach */
"0YQBXZZ"  =  "1"  /* Repair Left Lower Extremity, External Approach */
"0YQC0ZZ"  =  "1"  /* Repair Right Upper Leg, Open Approach */
"0YQC3ZZ"  =  "1"  /* Repair Right Upper Leg, Percutaneous Approach */
"0YQC4ZZ"  =  "1"  /* Repair Right Upper Leg, Percutaneous Endoscopic Approach */
"0YQCXZZ"  =  "1"  /* Repair Right Upper Leg, External Approach */
"0YQD0ZZ"  =  "1"  /* Repair Left Upper Leg, Open Approach */
"0YQD3ZZ"  =  "1"  /* Repair Left Upper Leg, Percutaneous Approach */
"0YQD4ZZ"  =  "1"  /* Repair Left Upper Leg, Percutaneous Endoscopic Approach */
"0YQDXZZ"  =  "1"  /* Repair Left Upper Leg, External Approach */
"0YQE0ZZ"  =  "1"  /* Repair Bilateral Femoral Region, Open Approach */
"0YQE3ZZ"  =  "1"  /* Repair Bilateral Femoral Region, Percutaneous Approach */
"0YQE4ZZ"  =  "1"  /* Repair Bilateral Femoral Region, Perc Endo Approach */
"0YQF0ZZ"  =  "1"  /* Repair Right Knee Region, Open Approach */
"0YQF3ZZ"  =  "1"  /* Repair Right Knee Region, Percutaneous Approach */
"0YQF4ZZ"  =  "1"  /* Repair Right Knee Region, Percutaneous Endoscopic Approach */
"0YQFXZZ"  =  "1"  /* Repair Right Knee Region, External Approach */
"0YQG0ZZ"  =  "1"  /* Repair Left Knee Region, Open Approach */
"0YQG3ZZ"  =  "1"  /* Repair Left Knee Region, Percutaneous Approach */
"0YQG4ZZ"  =  "1"  /* Repair Left Knee Region, Percutaneous Endoscopic Approach */
"0YQGXZZ"  =  "1"  /* Repair Left Knee Region, External Approach */
"0YQH0ZZ"  =  "1"  /* Repair Right Lower Leg, Open Approach */
"0YQH3ZZ"  =  "1"  /* Repair Right Lower Leg, Percutaneous Approach */
"0YQH4ZZ"  =  "1"  /* Repair Right Lower Leg, Percutaneous Endoscopic Approach */
"0YQHXZZ"  =  "1"  /* Repair Right Lower Leg, External Approach */
"0YQJ0ZZ"  =  "1"  /* Repair Left Lower Leg, Open Approach */
"0YQJ3ZZ"  =  "1"  /* Repair Left Lower Leg, Percutaneous Approach */
"0YQJ4ZZ"  =  "1"  /* Repair Left Lower Leg, Percutaneous Endoscopic Approach */
"0YQJXZZ"  =  "1"  /* Repair Left Lower Leg, External Approach */
"0YQK0ZZ"  =  "1"  /* Repair Right Ankle Region, Open Approach */
"0YQK3ZZ"  =  "1"  /* Repair Right Ankle Region, Percutaneous Approach */
"0YQK4ZZ"  =  "1"  /* Repair Right Ankle Region, Percutaneous Endoscopic Approach */
"0YQKXZZ"  =  "1"  /* Repair Right Ankle Region, External Approach */
"0YQL0ZZ"  =  "1"  /* Repair Left Ankle Region, Open Approach */
"0YQL3ZZ"  =  "1"  /* Repair Left Ankle Region, Percutaneous Approach */
"0YQL4ZZ"  =  "1"  /* Repair Left Ankle Region, Percutaneous Endoscopic Approach */
"0YQLXZZ"  =  "1"  /* Repair Left Ankle Region, External Approach */
"0YQM0ZZ"  =  "1"  /* Repair Right Foot, Open Approach */
"0YQM3ZZ"  =  "1"  /* Repair Right Foot, Percutaneous Approach */
"0YQM4ZZ"  =  "1"  /* Repair Right Foot, Percutaneous Endoscopic Approach */
"0YQMXZZ"  =  "1"  /* Repair Right Foot, External Approach */
"0YQN0ZZ"  =  "1"  /* Repair Left Foot, Open Approach */
"0YQN3ZZ"  =  "1"  /* Repair Left Foot, Percutaneous Approach */
"0YQN4ZZ"  =  "1"  /* Repair Left Foot, Percutaneous Endoscopic Approach */
"0YQNXZZ"  =  "1"  /* Repair Left Foot, External Approach */
"0YQP0ZZ"  =  "1"  /* Repair Right 1st Toe, Open Approach */
"0YQP3ZZ"  =  "1"  /* Repair Right 1st Toe, Percutaneous Approach */
"0YQP4ZZ"  =  "1"  /* Repair Right 1st Toe, Percutaneous Endoscopic Approach */
"0YQPXZZ"  =  "1"  /* Repair Right 1st Toe, External Approach */
"0YQQ0ZZ"  =  "1"  /* Repair Left 1st Toe, Open Approach */
"0YQQ3ZZ"  =  "1"  /* Repair Left 1st Toe, Percutaneous Approach */
"0YQQ4ZZ"  =  "1"  /* Repair Left 1st Toe, Percutaneous Endoscopic Approach */
"0YQQXZZ"  =  "1"  /* Repair Left 1st Toe, External Approach */
"0YQR0ZZ"  =  "1"  /* Repair Right 2nd Toe, Open Approach */
"0YQR3ZZ"  =  "1"  /* Repair Right 2nd Toe, Percutaneous Approach */
"0YQR4ZZ"  =  "1"  /* Repair Right 2nd Toe, Percutaneous Endoscopic Approach */
"0YQRXZZ"  =  "1"  /* Repair Right 2nd Toe, External Approach */
"0YQS0ZZ"  =  "1"  /* Repair Left 2nd Toe, Open Approach */
"0YQS3ZZ"  =  "1"  /* Repair Left 2nd Toe, Percutaneous Approach */
"0YQS4ZZ"  =  "1"  /* Repair Left 2nd Toe, Percutaneous Endoscopic Approach */
"0YQSXZZ"  =  "1"  /* Repair Left 2nd Toe, External Approach */
"0YQT0ZZ"  =  "1"  /* Repair Right 3rd Toe, Open Approach */
"0YQT3ZZ"  =  "1"  /* Repair Right 3rd Toe, Percutaneous Approach */
"0YQT4ZZ"  =  "1"  /* Repair Right 3rd Toe, Percutaneous Endoscopic Approach */
"0YQTXZZ"  =  "1"  /* Repair Right 3rd Toe, External Approach */
"0YQU0ZZ"  =  "1"  /* Repair Left 3rd Toe, Open Approach */
"0YQU3ZZ"  =  "1"  /* Repair Left 3rd Toe, Percutaneous Approach */
"0YQU4ZZ"  =  "1"  /* Repair Left 3rd Toe, Percutaneous Endoscopic Approach */
"0YQUXZZ"  =  "1"  /* Repair Left 3rd Toe, External Approach */
"0YQV0ZZ"  =  "1"  /* Repair Right 4th Toe, Open Approach */
"0YQV3ZZ"  =  "1"  /* Repair Right 4th Toe, Percutaneous Approach */
"0YQV4ZZ"  =  "1"  /* Repair Right 4th Toe, Percutaneous Endoscopic Approach */
"0YQVXZZ"  =  "1"  /* Repair Right 4th Toe, External Approach */
"0YQW0ZZ"  =  "1"  /* Repair Left 4th Toe, Open Approach */
"0YQW3ZZ"  =  "1"  /* Repair Left 4th Toe, Percutaneous Approach */
"0YQW4ZZ"  =  "1"  /* Repair Left 4th Toe, Percutaneous Endoscopic Approach */
"0YQWXZZ"  =  "1"  /* Repair Left 4th Toe, External Approach */
"0YQX0ZZ"  =  "1"  /* Repair Right 5th Toe, Open Approach */
"0YQX3ZZ"  =  "1"  /* Repair Right 5th Toe, Percutaneous Approach */
"0YQX4ZZ"  =  "1"  /* Repair Right 5th Toe, Percutaneous Endoscopic Approach */
"0YQXXZZ"  =  "1"  /* Repair Right 5th Toe, External Approach */
"0YQY0ZZ"  =  "1"  /* Repair Left 5th Toe, Open Approach */
"0YQY3ZZ"  =  "1"  /* Repair Left 5th Toe, Percutaneous Approach */
"0YQY4ZZ"  =  "1"  /* Repair Left 5th Toe, Percutaneous Endoscopic Approach */
"0YQYXZZ"  =  "1"  /* Repair Left 5th Toe, External Approach */
"0YU007Z"  =  "1"  /* Supplement Right Buttock with Autol Sub, Open Approach */
"0YU00JZ"  =  "1"  /* Supplement Right Buttock with Synth Sub, Open Approach */
"0YU00KZ"  =  "1"  /* Supplement Right Buttock with Nonaut Sub, Open Approach */
"0YU047Z"  =  "1"  /* Supplement Right Buttock with Autol Sub, Perc Endo Approach */
"0YU04JZ"  =  "1"  /* Supplement Right Buttock with Synth Sub, Perc Endo Approach */
"0YU04KZ"  =  "1"  /* Supplement Right Buttock with Nonaut Sub, Perc Endo Approach */
"0YU107Z"  =  "1"  /* Supplement Left Buttock with Autol Sub, Open Approach */
"0YU10JZ"  =  "1"  /* Supplement Left Buttock with Synth Sub, Open Approach */
"0YU10KZ"  =  "1"  /* Supplement Left Buttock with Nonaut Sub, Open Approach */
"0YU147Z"  =  "1"  /* Supplement Left Buttock with Autol Sub, Perc Endo Approach */
"0YU14JZ"  =  "1"  /* Supplement Left Buttock with Synth Sub, Perc Endo Approach */
"0YU14KZ"  =  "1"  /* Supplement Left Buttock with Nonaut Sub, Perc Endo Approach */
"0YU507Z"  =  "1"  /* Supplement R Inguinal Region with Autol Sub, Open Approach */
"0YU50JZ"  =  "1"  /* Supplement R Inguinal Region with Synth Sub, Open Approach */
"0YU50KZ"  =  "1"  /* Supplement R Inguinal Region with Nonaut Sub, Open Approach */
"0YU547Z"  =  "1"  /* Supplement R Inguinal Region w Autol Sub, Perc Endo */
"0YU54JZ"  =  "1"  /* Supplement R Inguinal Region w Synth Sub, Perc Endo */
"0YU54KZ"  =  "1"  /* Supplement R Inguinal Region w Nonaut Sub, Perc Endo */
"0YU607Z"  =  "1"  /* Supplement L Inguinal Region with Autol Sub, Open Approach */
"0YU60JZ"  =  "1"  /* Supplement L Inguinal Region with Synth Sub, Open Approach */
"0YU60KZ"  =  "1"  /* Supplement L Inguinal Region with Nonaut Sub, Open Approach */
"0YU647Z"  =  "1"  /* Supplement L Inguinal Region w Autol Sub, Perc Endo */
"0YU64JZ"  =  "1"  /* Supplement L Inguinal Region w Synth Sub, Perc Endo */
"0YU64KZ"  =  "1"  /* Supplement L Inguinal Region w Nonaut Sub, Perc Endo */
"0YU707Z"  =  "1"  /* Supplement R Femoral Region with Autol Sub, Open Approach */
"0YU70JZ"  =  "1"  /* Supplement R Femoral Region with Synth Sub, Open Approach */
"0YU70KZ"  =  "1"  /* Supplement R Femoral Region with Nonaut Sub, Open Approach */
"0YU747Z"  =  "1"  /* Supplement R Femoral Region w Autol Sub, Perc Endo */
"0YU74JZ"  =  "1"  /* Supplement R Femoral Region w Synth Sub, Perc Endo */
"0YU74KZ"  =  "1"  /* Supplement R Femoral Region w Nonaut Sub, Perc Endo */
"0YU807Z"  =  "1"  /* Supplement Left Femoral Region with Autol Sub, Open Approach */
"0YU80JZ"  =  "1"  /* Supplement Left Femoral Region with Synth Sub, Open Approach */
"0YU80KZ"  =  "1"  /* Supplement L Femoral Region with Nonaut Sub, Open Approach */
"0YU847Z"  =  "1"  /* Supplement L Femoral Region w Autol Sub, Perc Endo */
"0YU84JZ"  =  "1"  /* Supplement L Femoral Region w Synth Sub, Perc Endo */
"0YU84KZ"  =  "1"  /* Supplement L Femoral Region w Nonaut Sub, Perc Endo */
"0YU907Z"  =  "1"  /* Supplement R Low Extrem with Autol Sub, Open Approach */
"0YU90JZ"  =  "1"  /* Supplement R Low Extrem with Synth Sub, Open Approach */
"0YU90KZ"  =  "1"  /* Supplement R Low Extrem with Nonaut Sub, Open Approach */
"0YU947Z"  =  "1"  /* Supplement R Low Extrem with Autol Sub, Perc Endo Approach */
"0YU94JZ"  =  "1"  /* Supplement R Low Extrem with Synth Sub, Perc Endo Approach */
"0YU94KZ"  =  "1"  /* Supplement R Low Extrem with Nonaut Sub, Perc Endo Approach */
"0YUA07Z"  =  "1"  /* Supplement Bi Inguinal Region with Autol Sub, Open Approach */
"0YUA0JZ"  =  "1"  /* Supplement Bi Inguinal Region with Synth Sub, Open Approach */
"0YUA0KZ"  =  "1"  /* Supplement Bi Inguinal Region with Nonaut Sub, Open Approach */
"0YUA47Z"  =  "1"  /* Supplement Bi Inguinal Region w Autol Sub, Perc Endo */
"0YUA4JZ"  =  "1"  /* Supplement Bi Inguinal Region w Synth Sub, Perc Endo */
"0YUA4KZ"  =  "1"  /* Supplement Bi Inguinal Region w Nonaut Sub, Perc Endo */
"0YUB07Z"  =  "1"  /* Supplement L Low Extrem with Autol Sub, Open Approach */
"0YUB0JZ"  =  "1"  /* Supplement L Low Extrem with Synth Sub, Open Approach */
"0YUB0KZ"  =  "1"  /* Supplement L Low Extrem with Nonaut Sub, Open Approach */
"0YUB47Z"  =  "1"  /* Supplement L Low Extrem with Autol Sub, Perc Endo Approach */
"0YUB4JZ"  =  "1"  /* Supplement L Low Extrem with Synth Sub, Perc Endo Approach */
"0YUB4KZ"  =  "1"  /* Supplement L Low Extrem with Nonaut Sub, Perc Endo Approach */
"0YUC07Z"  =  "1"  /* Supplement Right Upper Leg with Autol Sub, Open Approach */
"0YUC0JZ"  =  "1"  /* Supplement Right Upper Leg with Synth Sub, Open Approach */
"0YUC0KZ"  =  "1"  /* Supplement Right Upper Leg with Nonaut Sub, Open Approach */
"0YUC47Z"  =  "1"  /* Supplement R Up Leg with Autol Sub, Perc Endo Approach */
"0YUC4JZ"  =  "1"  /* Supplement R Up Leg with Synth Sub, Perc Endo Approach */
"0YUC4KZ"  =  "1"  /* Supplement R Up Leg with Nonaut Sub, Perc Endo Approach */
"0YUD07Z"  =  "1"  /* Supplement Left Upper Leg with Autol Sub, Open Approach */
"0YUD0JZ"  =  "1"  /* Supplement Left Upper Leg with Synth Sub, Open Approach */
"0YUD0KZ"  =  "1"  /* Supplement Left Upper Leg with Nonaut Sub, Open Approach */
"0YUD47Z"  =  "1"  /* Supplement Left Upper Leg with Autol Sub, Perc Endo Approach */
"0YUD4JZ"  =  "1"  /* Supplement Left Upper Leg with Synth Sub, Perc Endo Approach */
"0YUD4KZ"  =  "1"  /* Supplement L Up Leg with Nonaut Sub, Perc Endo Approach */
"0YUE07Z"  =  "1"  /* Supplement Bi Femoral Region with Autol Sub, Open Approach */
"0YUE0JZ"  =  "1"  /* Supplement Bi Femoral Region with Synth Sub, Open Approach */
"0YUE0KZ"  =  "1"  /* Supplement Bi Femoral Region with Nonaut Sub, Open Approach */
"0YUE47Z"  =  "1"  /* Supplement Bi Femoral Region w Autol Sub, Perc Endo */
"0YUE4JZ"  =  "1"  /* Supplement Bi Femoral Region w Synth Sub, Perc Endo */
"0YUE4KZ"  =  "1"  /* Supplement Bi Femoral Region w Nonaut Sub, Perc Endo */
"0YUF07Z"  =  "1"  /* Supplement Right Knee Region with Autol Sub, Open Approach */
"0YUF0JZ"  =  "1"  /* Supplement Right Knee Region with Synth Sub, Open Approach */
"0YUF0KZ"  =  "1"  /* Supplement Right Knee Region with Nonaut Sub, Open Approach */
"0YUF47Z"  =  "1"  /* Supplement R Knee with Autol Sub, Perc Endo Approach */
"0YUF4JZ"  =  "1"  /* Supplement R Knee with Synth Sub, Perc Endo Approach */
"0YUF4KZ"  =  "1"  /* Supplement R Knee with Nonaut Sub, Perc Endo Approach */
"0YUG07Z"  =  "1"  /* Supplement Left Knee Region with Autol Sub, Open Approach */
"0YUG0JZ"  =  "1"  /* Supplement Left Knee Region with Synth Sub, Open Approach */
"0YUG0KZ"  =  "1"  /* Supplement Left Knee Region with Nonaut Sub, Open Approach */
"0YUG47Z"  =  "1"  /* Supplement L Knee with Autol Sub, Perc Endo Approach */
"0YUG4JZ"  =  "1"  /* Supplement L Knee with Synth Sub, Perc Endo Approach */
"0YUG4KZ"  =  "1"  /* Supplement L Knee with Nonaut Sub, Perc Endo Approach */
"0YUH07Z"  =  "1"  /* Supplement Right Lower Leg with Autol Sub, Open Approach */
"0YUH0JZ"  =  "1"  /* Supplement Right Lower Leg with Synth Sub, Open Approach */
"0YUH0KZ"  =  "1"  /* Supplement Right Lower Leg with Nonaut Sub, Open Approach */
"0YUH47Z"  =  "1"  /* Supplement R Low Leg with Autol Sub, Perc Endo Approach */
"0YUH4JZ"  =  "1"  /* Supplement R Low Leg with Synth Sub, Perc Endo Approach */
"0YUH4KZ"  =  "1"  /* Supplement R Low Leg with Nonaut Sub, Perc Endo Approach */
"0YUJ07Z"  =  "1"  /* Supplement Left Lower Leg with Autol Sub, Open Approach */
"0YUJ0JZ"  =  "1"  /* Supplement Left Lower Leg with Synth Sub, Open Approach */
"0YUJ0KZ"  =  "1"  /* Supplement Left Lower Leg with Nonaut Sub, Open Approach */
"0YUJ47Z"  =  "1"  /* Supplement Left Lower Leg with Autol Sub, Perc Endo Approach */
"0YUJ4JZ"  =  "1"  /* Supplement Left Lower Leg with Synth Sub, Perc Endo Approach */
"0YUJ4KZ"  =  "1"  /* Supplement L Low Leg with Nonaut Sub, Perc Endo Approach */
"0YUK07Z"  =  "1"  /* Supplement Right Ankle Region with Autol Sub, Open Approach */
"0YUK0JZ"  =  "1"  /* Supplement Right Ankle Region with Synth Sub, Open Approach */
"0YUK0KZ"  =  "1"  /* Supplement Right Ankle Region with Nonaut Sub, Open Approach */
"0YUK47Z"  =  "1"  /* Supplement R Ankle with Autol Sub, Perc Endo Approach */
"0YUK4JZ"  =  "1"  /* Supplement R Ankle with Synth Sub, Perc Endo Approach */
"0YUK4KZ"  =  "1"  /* Supplement R Ankle with Nonaut Sub, Perc Endo Approach */
"0YUL07Z"  =  "1"  /* Supplement Left Ankle Region with Autol Sub, Open Approach */
"0YUL0JZ"  =  "1"  /* Supplement Left Ankle Region with Synth Sub, Open Approach */
"0YUL0KZ"  =  "1"  /* Supplement Left Ankle Region with Nonaut Sub, Open Approach */
"0YUL47Z"  =  "1"  /* Supplement L Ankle with Autol Sub, Perc Endo Approach */
"0YUL4JZ"  =  "1"  /* Supplement L Ankle with Synth Sub, Perc Endo Approach */
"0YUL4KZ"  =  "1"  /* Supplement L Ankle with Nonaut Sub, Perc Endo Approach */
"0YUM07Z"  =  "1"  /* Supplement Right Foot with Autol Sub, Open Approach */
"0YUM0JZ"  =  "1"  /* Supplement Right Foot with Synth Sub, Open Approach */
"0YUM0KZ"  =  "1"  /* Supplement Right Foot with Nonaut Sub, Open Approach */
"0YUM47Z"  =  "1"  /* Supplement Right Foot with Autol Sub, Perc Endo Approach */
"0YUM4JZ"  =  "1"  /* Supplement Right Foot with Synth Sub, Perc Endo Approach */
"0YUM4KZ"  =  "1"  /* Supplement Right Foot with Nonaut Sub, Perc Endo Approach */
"0YUN07Z"  =  "1"  /* Supplement Left Foot with Autol Sub, Open Approach */
"0YUN0JZ"  =  "1"  /* Supplement Left Foot with Synth Sub, Open Approach */
"0YUN0KZ"  =  "1"  /* Supplement Left Foot with Nonaut Sub, Open Approach */
"0YUN47Z"  =  "1"  /* Supplement Left Foot with Autol Sub, Perc Endo Approach */
"0YUN4JZ"  =  "1"  /* Supplement Left Foot with Synth Sub, Perc Endo Approach */
"0YUN4KZ"  =  "1"  /* Supplement Left Foot with Nonaut Sub, Perc Endo Approach */
"0YUP07Z"  =  "1"  /* Supplement Right 1st Toe with Autol Sub, Open Approach */
"0YUP0JZ"  =  "1"  /* Supplement Right 1st Toe with Synth Sub, Open Approach */
"0YUP0KZ"  =  "1"  /* Supplement Right 1st Toe with Nonaut Sub, Open Approach */
"0YUP47Z"  =  "1"  /* Supplement Right 1st Toe with Autol Sub, Perc Endo Approach */
"0YUP4JZ"  =  "1"  /* Supplement Right 1st Toe with Synth Sub, Perc Endo Approach */
"0YUP4KZ"  =  "1"  /* Supplement Right 1st Toe with Nonaut Sub, Perc Endo Approach */
"0YUQ07Z"  =  "1"  /* Supplement Left 1st Toe with Autol Sub, Open Approach */
"0YUQ0JZ"  =  "1"  /* Supplement Left 1st Toe with Synth Sub, Open Approach */
"0YUQ0KZ"  =  "1"  /* Supplement Left 1st Toe with Nonaut Sub, Open Approach */
"0YUQ47Z"  =  "1"  /* Supplement Left 1st Toe with Autol Sub, Perc Endo Approach */
"0YUQ4JZ"  =  "1"  /* Supplement Left 1st Toe with Synth Sub, Perc Endo Approach */
"0YUQ4KZ"  =  "1"  /* Supplement Left 1st Toe with Nonaut Sub, Perc Endo Approach */
"0YUR07Z"  =  "1"  /* Supplement Right 2nd Toe with Autol Sub, Open Approach */
"0YUR0JZ"  =  "1"  /* Supplement Right 2nd Toe with Synth Sub, Open Approach */
"0YUR0KZ"  =  "1"  /* Supplement Right 2nd Toe with Nonaut Sub, Open Approach */
"0YUR47Z"  =  "1"  /* Supplement Right 2nd Toe with Autol Sub, Perc Endo Approach */
"0YUR4JZ"  =  "1"  /* Supplement Right 2nd Toe with Synth Sub, Perc Endo Approach */
"0YUR4KZ"  =  "1"  /* Supplement Right 2nd Toe with Nonaut Sub, Perc Endo Approach */
"0YUS07Z"  =  "1"  /* Supplement Left 2nd Toe with Autol Sub, Open Approach */
"0YUS0JZ"  =  "1"  /* Supplement Left 2nd Toe with Synth Sub, Open Approach */
"0YUS0KZ"  =  "1"  /* Supplement Left 2nd Toe with Nonaut Sub, Open Approach */
"0YUS47Z"  =  "1"  /* Supplement Left 2nd Toe with Autol Sub, Perc Endo Approach */
"0YUS4JZ"  =  "1"  /* Supplement Left 2nd Toe with Synth Sub, Perc Endo Approach */
"0YUS4KZ"  =  "1"  /* Supplement Left 2nd Toe with Nonaut Sub, Perc Endo Approach */
"0YUT07Z"  =  "1"  /* Supplement Right 3rd Toe with Autol Sub, Open Approach */
"0YUT0JZ"  =  "1"  /* Supplement Right 3rd Toe with Synth Sub, Open Approach */
"0YUT0KZ"  =  "1"  /* Supplement Right 3rd Toe with Nonaut Sub, Open Approach */
"0YUT47Z"  =  "1"  /* Supplement Right 3rd Toe with Autol Sub, Perc Endo Approach */
"0YUT4JZ"  =  "1"  /* Supplement Right 3rd Toe with Synth Sub, Perc Endo Approach */
"0YUT4KZ"  =  "1"  /* Supplement Right 3rd Toe with Nonaut Sub, Perc Endo Approach */
"0YUU07Z"  =  "1"  /* Supplement Left 3rd Toe with Autol Sub, Open Approach */
"0YUU0JZ"  =  "1"  /* Supplement Left 3rd Toe with Synth Sub, Open Approach */
"0YUU0KZ"  =  "1"  /* Supplement Left 3rd Toe with Nonaut Sub, Open Approach */
"0YUU47Z"  =  "1"  /* Supplement Left 3rd Toe with Autol Sub, Perc Endo Approach */
"0YUU4JZ"  =  "1"  /* Supplement Left 3rd Toe with Synth Sub, Perc Endo Approach */
"0YUU4KZ"  =  "1"  /* Supplement Left 3rd Toe with Nonaut Sub, Perc Endo Approach */
"0YUV07Z"  =  "1"  /* Supplement Right 4th Toe with Autol Sub, Open Approach */
"0YUV0JZ"  =  "1"  /* Supplement Right 4th Toe with Synth Sub, Open Approach */
"0YUV0KZ"  =  "1"  /* Supplement Right 4th Toe with Nonaut Sub, Open Approach */
"0YUV47Z"  =  "1"  /* Supplement Right 4th Toe with Autol Sub, Perc Endo Approach */
"0YUV4JZ"  =  "1"  /* Supplement Right 4th Toe with Synth Sub, Perc Endo Approach */
"0YUV4KZ"  =  "1"  /* Supplement Right 4th Toe with Nonaut Sub, Perc Endo Approach */
"0YUW07Z"  =  "1"  /* Supplement Left 4th Toe with Autol Sub, Open Approach */
"0YUW0JZ"  =  "1"  /* Supplement Left 4th Toe with Synth Sub, Open Approach */
"0YUW0KZ"  =  "1"  /* Supplement Left 4th Toe with Nonaut Sub, Open Approach */
"0YUW47Z"  =  "1"  /* Supplement Left 4th Toe with Autol Sub, Perc Endo Approach */
"0YUW4JZ"  =  "1"  /* Supplement Left 4th Toe with Synth Sub, Perc Endo Approach */
"0YUW4KZ"  =  "1"  /* Supplement Left 4th Toe with Nonaut Sub, Perc Endo Approach */
"0YUX07Z"  =  "1"  /* Supplement Right 5th Toe with Autol Sub, Open Approach */
"0YUX0JZ"  =  "1"  /* Supplement Right 5th Toe with Synth Sub, Open Approach */
"0YUX0KZ"  =  "1"  /* Supplement Right 5th Toe with Nonaut Sub, Open Approach */
"0YUX47Z"  =  "1"  /* Supplement Right 5th Toe with Autol Sub, Perc Endo Approach */
"0YUX4JZ"  =  "1"  /* Supplement Right 5th Toe with Synth Sub, Perc Endo Approach */
"0YUX4KZ"  =  "1"  /* Supplement Right 5th Toe with Nonaut Sub, Perc Endo Approach */
"0YUY07Z"  =  "1"  /* Supplement Left 5th Toe with Autol Sub, Open Approach */
"0YUY0JZ"  =  "1"  /* Supplement Left 5th Toe with Synth Sub, Open Approach */
"0YUY0KZ"  =  "1"  /* Supplement Left 5th Toe with Nonaut Sub, Open Approach */
"0YUY47Z"  =  "1"  /* Supplement Left 5th Toe with Autol Sub, Perc Endo Approach */
"0YUY4JZ"  =  "1"  /* Supplement Left 5th Toe with Synth Sub, Perc Endo Approach */
"0YUY4KZ"  =  "1"  /* Supplement Left 5th Toe with Nonaut Sub, Perc Endo Approach */
"10A00ZZ"  =  "1"  /* Abortion of Products of Conception, Open Approach */
"10A03ZZ"  =  "1"  /* Abortion of Products of Conception, Percutaneous Approach */
"10A04ZZ"  =  "1"  /* Abortion of Products of Conception, Perc Endo Approach */
"10A07ZZ"  =  "1"  /* Abortion of Products of Conception, Via Opening */
"10A08ZZ"  =  "1"  /* Abortion of Products of Conception, Endo */
"10D00Z0"  =  "1"  /* Extraction of POC, Classical, Open Approach */
"10D00Z1"  =  "1"  /* Extraction of POC, Low Cervical, Open Approach */
"10D00Z2"  =  "1"  /* Extraction of POC, Extraperitoneal, Open Approach */
"10D17ZZ"  =  "1"  /* Extraction of Products of Conception, Retained, Via Opening */
"10D18ZZ"  =  "1"  /* Extraction of Products of Conception, Retained, Endo */
"10D27ZZ"  =  "1"  /* Extraction of Products of Conception, Ectopic, Via Opening */
"10D28ZZ"  =  "1"  /* Extraction of Products of Conception, Ectopic, Endo */
"10H003Z"  =  "1"  /* Insertion of Monitoring Electrode into POC, Open Approach */
"10H00YZ"  =  "1"  /* Insertion of Other Device into POC, Open Approach */
"10P003Z"  =  "1"  /* Removal of Monitoring Electrode from POC, Open Approach */
"10P00YZ"  =  "1"  /* Removal of Other Device from POC, Open Approach */
"10P073Z"  =  "1"  /* Removal of Monitoring Electrode from POC, Via Opening */
"10P07YZ"  =  "1"  /* Removal of Other Device from POC, Via Opening */
"10S20ZZ"  =  "1"  /* Reposition Products of Conception, Ectopic, Open Approach */
"10S23ZZ"  =  "1"  /* Reposition Products of Conception, Ectopic, Perc Approach */
"10S24ZZ"  =  "1"  /* Reposition Ectopic POC, Perc Endo Approach */
"10S27ZZ"  =  "1"  /* Reposition Products of Conception, Ectopic, Via Opening */
"10S28ZZ"  =  "1"  /* Reposition Products of Conception, Ectopic, Endo */
"10T20ZZ"  =  "1"  /* Resection of Products of Conception, Ectopic, Open Approach */
"10T23ZZ"  =  "1"  /* Resection of Products of Conception, Ectopic, Perc Approach */
"10T24ZZ"  =  "1"  /* Resection of Ectopic POC, Perc Endo Approach */
"10T27ZZ"  =  "1"  /* Resection of Products of Conception, Ectopic, Via Opening */
"10T28ZZ"  =  "1"  /* Resection of Products of Conception, Ectopic, Endo */
"30230AZ"  =  "1"  /* Transfuse of Embr Stem Cell into Periph Vein, Open Approach */
"30230G0"  =  "1"  /* Transfuse Autol Bone Marrow in Periph Vein, Open */
"30230G1"  =  "1"  /* Transfuse Nonaut Bone Marrow in Periph Vein, Open */
"30230X0"  =  "1"  /* Transfuse Autol Cord Bld Stem Cell in Periph Vein, Open */
"30230X1"  =  "1"  /* Transfuse Nonaut Cord Bld Stem Cell in Periph Vein, Open */
"30230Y0"  =  "1"  /* Transfuse Autol Hemat Stem Cell in Periph Vein, Open */
"30230Y1"  =  "1"  /* Transfuse Nonaut Hemat Stem Cell in Periph Vein, Open */
"30233AZ"  =  "1"  /* Transfuse of Embr Stem Cell into Periph Vein, Perc Approach */
"30233G0"  =  "1"  /* Transfuse Autol Bone Marrow in Periph Vein, Perc */
"30233G1"  =  "1"  /* Transfuse Nonaut Bone Marrow in Periph Vein, Perc */
"30233X0"  =  "1"  /* Transfuse Autol Cord Bld Stem Cell in Periph Vein, Perc */
"30233X1"  =  "1"  /* Transfuse Nonaut Cord Bld Stem Cell in Periph Vein, Perc */
"30233Y0"  =  "1"  /* Transfuse Autol Hemat Stem Cell in Periph Vein, Perc */
"30233Y1"  =  "1"  /* Transfuse Nonaut Hemat Stem Cell in Periph Vein, Perc */
"30240AZ"  =  "1"  /* Transfuse of Embr Stem Cell into Central Vein, Open Approach */
"30240G0"  =  "1"  /* Transfuse Autol Bone Marrow in Central Vein, Open */
"30240G1"  =  "1"  /* Transfuse Nonaut Bone Marrow in Central Vein, Open */
"30240X0"  =  "1"  /* Transfuse Autol Cord Bld Stem Cell in Central Vein, Open */
"30240X1"  =  "1"  /* Transfuse Nonaut Cord Bld Stem Cell in Central Vein, Open */
"30240Y0"  =  "1"  /* Transfuse Autol Hemat Stem Cell in Central Vein, Open */
"30240Y1"  =  "1"  /* Transfuse Nonaut Hemat Stem Cell in Central Vein, Open */
"30243AZ"  =  "1"  /* Transfuse of Embr Stem Cell into Central Vein, Perc Approach */
"30243G0"  =  "1"  /* Transfuse Autol Bone Marrow in Central Vein, Perc */
"30243G1"  =  "1"  /* Transfuse Nonaut Bone Marrow in Central Vein, Perc */
"30243X0"  =  "1"  /* Transfuse Autol Cord Bld Stem Cell in Central Vein, Perc */
"30243X1"  =  "1"  /* Transfuse Nonaut Cord Bld Stem Cell in Central Vein, Perc */
"30243Y0"  =  "1"  /* Transfuse Autol Hemat Stem Cell in Central Vein, Perc */
"30243Y1"  =  "1"  /* Transfuse Nonaut Hemat Stem Cell in Central Vein, Perc */
"30250G0"  =  "1"  /* Transfuse Autol Bone Marrow in Periph Art, Open */
"30250G1"  =  "1"  /* Transfuse Nonaut Bone Marrow in Periph Art, Open */
"30250X0"  =  "1"  /* Transfuse Autol Cord Bld Stem Cell in Periph Art, Open */
"30250X1"  =  "1"  /* Transfuse Nonaut Cord Bld Stem Cell in Periph Art, Open */
"30250Y0"  =  "1"  /* Transfuse Autol Hemat Stem Cell in Periph Art, Open */
"30250Y1"  =  "1"  /* Transfuse Nonaut Hemat Stem Cell in Periph Art, Open */
"30253G0"  =  "1"  /* Transfuse Autol Bone Marrow in Periph Art, Perc */
"30253G1"  =  "1"  /* Transfuse Nonaut Bone Marrow in Periph Art, Perc */
"30253X0"  =  "1"  /* Transfuse Autol Cord Bld Stem Cell in Periph Art, Perc */
"30253X1"  =  "1"  /* Transfuse Nonaut Cord Bld Stem Cell in Periph Art, Perc */
"30253Y0"  =  "1"  /* Transfuse Autol Hemat Stem Cell in Periph Art, Perc */
"30253Y1"  =  "1"  /* Transfuse Nonaut Hemat Stem Cell in Periph Art, Perc */
"30260G0"  =  "1"  /* Transfuse Autol Bone Marrow in Central Art, Open */
"30260G1"  =  "1"  /* Transfuse Nonaut Bone Marrow in Central Art, Open */
"30260X0"  =  "1"  /* Transfuse Autol Cord Bld Stem Cell in Central Art, Open */
"30260X1"  =  "1"  /* Transfuse Nonaut Cord Bld Stem Cell in Central Art, Open */
"30260Y0"  =  "1"  /* Transfuse Autol Hemat Stem Cell in Central Art, Open */
"30260Y1"  =  "1"  /* Transfuse Nonaut Hemat Stem Cell in Central Art, Open */
"30263G0"  =  "1"  /* Transfuse Autol Bone Marrow in Central Art, Perc */
"30263G1"  =  "1"  /* Transfuse Nonaut Bone Marrow in Central Art, Perc */
"30263X0"  =  "1"  /* Transfuse Autol Cord Bld Stem Cell in Central Art, Perc */
"30263X1"  =  "1"  /* Transfuse Nonaut Cord Bld Stem Cell in Central Art, Perc */
"30263Y0"  =  "1"  /* Transfuse Autol Hemat Stem Cell in Central Art, Perc */
"30263Y1"  =  "1"  /* Transfuse Nonaut Hemat Stem Cell in Central Art, Perc */
"3E030TZ"  =  "1"  /* Introduction of Destr Agent into Periph Vein, Open Approach */
"3E033TZ"  =  "1"  /* Introduction of Destr Agent into Periph Vein, Perc Approach */
"3E040TZ"  =  "1"  /* Introduction of Destr Agent into Central Vein, Open Approach */
"3E043TZ"  =  "1"  /* Introduction of Destr Agent into Central Vein, Perc Approach */
"3E0B329"  =  "1"  /* Introduction of Other Anti-infective into Ear, Perc Approach */
"3E0B33Z"  =  "1"  /* Introduction of Anti-inflammatory into Ear, Perc Approach */
"3E0B3BZ"  =  "1"  /* Introduction of Local Anesthetic into Ear, Perc Approach */
"3E0B3GC"  =  "1"  /* Introduction of Oth Therap Subst into Ear, Perc Approach */
"3E0B3HZ"  =  "1"  /* Introduction of Radioact Subst into Ear, Perc Approach */
"3E0B3KZ"  =  "1"  /* Introduction of Oth Diagn Subst into Ear, Perc Approach */
"3E0B3TZ"  =  "1"  /* Introduction of Destructive Agent into Ear, Perc Approach */
"3E0B729"  =  "1"  /* Introduction of Other Anti-infective into Ear, Via Opening */
"3E0B73Z"  =  "1"  /* Introduction of Anti-inflammatory into Ear, Via Opening */
"3E0B7BZ"  =  "1"  /* Introduction of Local Anesthetic into Ear, Via Opening */
"3E0B7GC"  =  "1"  /* Introduction of Oth Therap Subst into Ear, Via Opening */
"3E0B7HZ"  =  "1"  /* Introduction of Radioactive Substance into Ear, Via Opening */
"3E0B7KZ"  =  "1"  /* Introduction of Oth Diagn Subst into Ear, Via Opening */
"3E0B7TZ"  =  "1"  /* Introduction of Destructive Agent into Ear, Via Opening */
"3E0BX29"  =  "1"  /* Introduction of Oth Anti-infect into Ear, Extern Approach */
"3E0BX3Z"  =  "1"  /* Introduction of Anti-inflammatory into Ear, Extern Approach */
"3E0BXBZ"  =  "1"  /* Introduction of Local Anesthetic into Ear, External Approach */
"3E0BXGC"  =  "1"  /* Introduction of Oth Therap Subst into Ear, Extern Approach */
"3E0BXHZ"  =  "1"  /* Introduction of Radioact Subst into Ear, Extern Approach */
"3E0BXKZ"  =  "1"  /* Introduction of Oth Diagn Subst into Ear, Extern Approach */
"3E0BXTZ"  =  "1"  /* Introduction of Destructive Agent into Ear, Extern Approach */
"3E0C33Z"  =  "1"  /* Introduction of Anti-inflammatory into Eye, Perc Approach */
"3E0C3BZ"  =  "1"  /* Introduction of Local Anesthetic into Eye, Perc Approach */
"3E0C3GC"  =  "1"  /* Introduction of Oth Therap Subst into Eye, Perc Approach */
"3E0C3HZ"  =  "1"  /* Introduction of Radioact Subst into Eye, Perc Approach */
"3E0C3KZ"  =  "1"  /* Introduction of Oth Diagn Subst into Eye, Perc Approach */
"3E0C3MZ"  =  "1"  /* Introduction of Pigment into Eye, Percutaneous Approach */
"3E0C3SF"  =  "1"  /* Introduction of Other Gas into Eye, Percutaneous Approach */
"3E0C3TZ"  =  "1"  /* Introduction of Destructive Agent into Eye, Perc Approach */
"3E0C729"  =  "1"  /* Introduction of Other Anti-infective into Eye, Via Opening */
"3E0C73Z"  =  "1"  /* Introduction of Anti-inflammatory into Eye, Via Opening */
"3E0C7BZ"  =  "1"  /* Introduction of Local Anesthetic into Eye, Via Opening */
"3E0C7GC"  =  "1"  /* Introduction of Oth Therap Subst into Eye, Via Opening */
"3E0C7HZ"  =  "1"  /* Introduction of Radioactive Substance into Eye, Via Opening */
"3E0C7KZ"  =  "1"  /* Introduction of Oth Diagn Subst into Eye, Via Opening */
"3E0C7MZ"  =  "1"  /* Introduction of Pigment into Eye, Via Opening */
"3E0C7SF"  =  "1"  /* Introduction of Other Gas into Eye, Via Opening */
"3E0C7TZ"  =  "1"  /* Introduction of Destructive Agent into Eye, Via Opening */
"3E0CX29"  =  "1"  /* Introduction of Oth Anti-infect into Eye, Extern Approach */
"3E0CX3Z"  =  "1"  /* Introduction of Anti-inflammatory into Eye, Extern Approach */
"3E0CXBZ"  =  "1"  /* Introduction of Local Anesthetic into Eye, External Approach */
"3E0CXGC"  =  "1"  /* Introduction of Oth Therap Subst into Eye, Extern Approach */
"3E0CXHZ"  =  "1"  /* Introduction of Radioact Subst into Eye, Extern Approach */
"3E0CXKZ"  =  "1"  /* Introduction of Oth Diagn Subst into Eye, Extern Approach */
"3E0CXMZ"  =  "1"  /* Introduction of Pigment into Eye, External Approach */
"3E0CXSF"  =  "1"  /* Introduction of Other Gas into Eye, External Approach */
"3E0CXTZ"  =  "1"  /* Introduction of Destructive Agent into Eye, Extern Approach */
"3E0G3GC"  =  "1"  /* Introduction of Oth Therap Subst into Up GI, Perc Approach */
"3E0P3Q0"  =  "1"  /* Introduce Autol Fertilized Ovum in Fem Reprod, Perc */
"3E0P3Q1"  =  "1"  /* Introduce Nonaut Fertilized Ovum in Fem Reprod, Perc */
"3E0P73Z"  =  "1"  /* Introduction of Anti-inflam into Fem Reprod, Via Opening */
"3E0P76Z"  =  "1"  /* Introduction of Nutritional into Fem Reprod, Via Opening */
"3E0P77Z"  =  "1"  /* Introduce of Electrol/Water Bal into Fem Reprod, Via Opening */
"3E0P7GC"  =  "1"  /* Introduce of Oth Therap Subst into Fem Reprod, Via Opening */
"3E0P7Q0"  =  "1"  /* Introduce Autol Fertilized Ovum in Fem Reprod, Via Opening */
"3E0P7Q1"  =  "1"  /* Introduce Nonaut Fertilized Ovum in Fem Reprod, Via Opening */
"3E0P7SF"  =  "1"  /* Introduction of Other Gas into Fem Reprod, Via Opening */
"3E0P83Z"  =  "1"  /* Introduction of Anti-inflammatory into Fem Reprod, Endo */
"3E0P86Z"  =  "1"  /* Introduction of Nutritional Substance into Fem Reprod, Endo */
"3E0P87Z"  =  "1"  /* Introduction of Electrol/Water Bal into Fem Reprod, Endo */
"3E0P8GC"  =  "1"  /* Introduction of Oth Therap Subst into Fem Reprod, Endo */
"3E0P8SF"  =  "1"  /* Introduction of Other Gas into Female Reproductive, Endo */
"3E1N38X"  =  "1"  /* Irrigate of Male Reprod using Irrigat, Perc Approach, Diagn */
"3E1N38Z"  =  "1"  /* Irrigation of Male Reproductive using Irrigat, Perc Approach */
"3E1N78X"  =  "1"  /* Irrigation of Male Reprod using Irrigat, Via Opening, Diagn */
"3E1N78Z"  =  "1"  /* Irrigation of Male Reproductive using Irrigat, Via Opening */
"3E1N88X"  =  "1"  /* Irrigation of Male Reproductive using Irrigat, Endo, Diagn */
"3E1N88Z"  =  "1"  /* Irrigation of Male Reproductive using Irrigat, Endo */
"4A0605Z"  =  "1"  /* Measurement of Lymphatic Flow, Open Approach */
"4A060BZ"  =  "1"  /* Measurement of Lymphatic Pressure, Open Approach */
"4A0635Z"  =  "1"  /* Measurement of Lymphatic Flow, Percutaneous Approach */
"4A063BZ"  =  "1"  /* Measurement of Lymphatic Pressure, Percutaneous Approach */
"4A0C35Z"  =  "1"  /* Measurement of Biliary Flow, Percutaneous Approach */
"4A0C3BZ"  =  "1"  /* Measurement of Biliary Pressure, Percutaneous Approach */
"4A0C45Z"  =  "1"  /* Measurement of Biliary Flow, Perc Endo Approach */
"4A0C4BZ"  =  "1"  /* Measurement of Biliary Pressure, Perc Endo Approach */
"4A0C75Z"  =  "1"  /* Measurement of Biliary Flow, Via Opening */
"4A0C7BZ"  =  "1"  /* Measurement of Biliary Pressure, Via Opening */
"4A0C85Z"  =  "1"  /* Measurement of Biliary Flow, Endo */
"4A1605Z"  =  "1"  /* Monitoring of Lymphatic Flow, Open Approach */
"4A160BZ"  =  "1"  /* Monitoring of Lymphatic Pressure, Open Approach */
"4A1635Z"  =  "1"  /* Monitoring of Lymphatic Flow, Percutaneous Approach */
"4A163BZ"  =  "1"  /* Monitoring of Lymphatic Pressure, Percutaneous Approach */
"5A02110"  =  "1"  /* Assist with Cardiac Output using Balloon Pump, Intermittent */
"5A02116"  =  "1"  /* Assist with Cardiac Output using Other Pump, Intermittent */
"5A0211D"  =  "1"  /* Assist with Cardiac Output using Impeller Pump, Intermittent */
"5A02210"  =  "1"  /* Assist with Cardiac Output using Balloon Pump, Continuous */
"5A02216"  =  "1"  /* Assistance with Cardiac Output using Other Pump, Continuous */
"5A0221D"  =  "1"  /* Assist with Cardiac Output using Impeller Pump, Continuous */
"5A15223"  =  "1"  /* Extracorporeal Membrane Oxygenation, Continuous */
"BF030ZZ"  =  "1"  /* Radiography of Gallbladder & Bile Duct using H Osm Contrast */
"BF031ZZ"  =  "1"  /* Radiography of Gallbladder & Bile Duct using L Osm Contrast */
"BF03YZZ"  =  "1"  /* Radiography of Gallbladder & Bile Duct using Oth Contrast */
"BF0C0ZZ"  =  "1"  /* Plain Radiography of Hepatobil Sys, All using H Osm Contrast */
"BF0C1ZZ"  =  "1"  /* Plain Radiography of Hepatobil Sys, All using L Osm Contrast */
"BF0CYZZ"  =  "1"  /* Plain Radiography of Hepatobil Sys, All using Oth Contrast */
"D0Y0KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Brain */
"D0Y1KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Brain Stem */
"D0Y6KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Spinal Cord */
"D0Y7KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Peripheral Nerve */
"DBY0KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Trachea */
"DBY1KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Bronchus */
"DBY2KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Lung */
"DBY5KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Pleura */
"DBY6KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Mediastinum */
"DBY7KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Chest Wall */
"DBY8KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Diaphragm */
"DDY0KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Esophagus */
"DDY1KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Stomach */
"DDY2KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Duodenum */
"DDY3KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Jejunum */
"DDY4KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Ileum */
"DDY5KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Colon */
"DDY7KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Rectum */
"DDY8KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Anus */
"DFY0KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Liver */
"DFY1KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Gallbladder */
"DFY2KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Bile Ducts */
"DFY3KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Pancreas */
"DGY0KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Pituitary Gland */
"DGY1KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Pineal Body */
"DGY2KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Adrenal Glands */
"DGY4KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Parathyroid Glands */
"DGY5KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Thyroid */
"DMY0KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Left Breast */
"DMY1KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Right Breast */
"DVY0KZZ"  =  "1"  /* Laser Interstitial Thermal Therapy of Prostate */
"F0DZ8ZZ"  =  "1"  /* Prosthesis Device Fitting */
"F0DZ9EZ"  =  "1"  /* Assist Device Fit using Orthosis */
"F0DZ9FZ"  =  "1"  /* Assist Device Fit using Assist Equipment */
"F0DZ9UZ"  =  "1"  /* Assist Device Fit using Prosthesis */
"F0DZ9ZZ"  =  "1"  /* Assist Device Fit */
OTHER  = "0" ;

VALUE $PRCLASS
'0001' = 2
'0002' = 2
'0003' = 2
'0009' = 2
'0010' = 2
'0011' = 2
'0012' = 2
'0013' = 2
'0014' = 2
'0015' = 2
'0016' = 2
'0017' = 2
'0018' = 2
'0021' = 1
'0022' = 1
'0023' = 1
'0024' = 1
'0025' = 1
'0028' = 1
'0029' = 1
'0031' = 1
'0032' = 1
'0033' = 1
'0034' = 1
'0035' = 1
'0039' = 1
'0040' = 2
'0041' = 2
'0042' = 2
'0043' = 2
'0044' = 2  
'0045' = 2
'0046' = 2
'0047' = 2
'0048' = 2
'0049' = 2
'0050' = 4
'0051' = 4
'0052' = 4
'0053' = 4
'0054' = 4
'0055' = 2
'0056' = 4
'0057' = 4
'0058' = 3
'0059' = 3
'0060' = 2
'0061' = 4
'0062' = 4
'0063' = 2
'0064' = 2
'0065' = 2
'0066' = 4
'0067' = 3
'0068' = 3
'0069' = 3
'0070' = 4
'0071' = 4
'0072' = 4
'0073' = 4
'0074' = 2 
'0075' = 2 
'0076' = 2 
'0077' = 2 
'0080' = 4
'0081' = 4
'0082' = 4
'0083' = 4
'0084' = 4
'0085' = 4
'0086' = 4
'0087' = 4
'0091' = 2
'0092' = 2
'0093' = 2
'0094' = 1 
'0095' = 2  /** ADDED FOR V4.5 **/
'0101' = 2
'0102' = 2
'0109' = 2
'0110' = 1 
'0111' = 1
'0112' = 3
'0113' = 1
'0114' = 3
'0115' = 3
'0116' = 1 
'0117' = 1 
'0118' = 3
'0119' = 3
'0120' = 4
'0121' = 4
'0122' = 4
'0123' = 4
'0124' = 4
'0125' = 4
'0126' = 2
'0127' = 2 
'0128' = 4
'0129' = 4
'0131' = 4
'0132' = 4
'0139' = 4
'0141' = 4
'0142' = 4
'0151' = 4
'0152' = 4
'0153' = 4
'0159' = 4
'016' = 4
'0201' = 4
'0202' = 4
'0203' = 4
'0204' = 4
'0205' = 4
'0206' = 4
'0207' = 4
'0211' = 4
'0212' = 4
'0213' = 4
'0214' = 4
'022'  = 4
'0221' = 4  
'0222' = 4 
'0231' = 4
'0232' = 4
'0233' = 4
'0234' = 4
'0235' = 4
'0239' = 4
'0241' = 2
'0242' = 4
'0243' = 4
'0291' = 4
'0292' = 4
'0293' = 4
'0294' = 4
'0295' = 2
'0296' = 2
'0299' = 4
'0301' = 4
'0302' = 4
'0309' = 4
'031' = 4
'0321' = 4
'0329' = 4
'0331' = 1
'0332' = 3
'0339' = 3
'034' = 4
'0351' = 4
'0352' = 4
'0353' = 4
'0359' = 4
'036' = 4
'0371' = 4
'0372' = 4
'0379' = 4
'038' = 2
'0390' = 2
'0391' = 2
'0392' = 2
'0393' = 4
'0394' = 4
'0395' = 2
'0396' = 2
'0397' = 4
'0398' = 4
'0399' = 4
'0401' = 4
'0402' = 4
'0403' = 4
'0404' = 4
'0405' = 4
'0406' = 4
'0407' = 4
'0411' = 1
'0412' = 3
'0419' = 3
'042' = 2
'043' = 4
'0441' = 4
'0442' = 4
'0443' = 4
'0444' = 4
'0449' = 4
'045' = 4
'046' = 4
'0471' = 4
'0472' = 4
'0473' = 4
'0474' = 4
'0475' = 4
'0476' = 4
'0479' = 4
'0480' = 2
'0481' = 2
'0489' = 2
'0491' = 4
'0492' = 4
'0493' = 4
'0499' = 4
'050' = 4
'0511' = 3
'0519' = 3
'0521' = 4
'0522' = 4
'0523' = 4
'0524' = 4
'0525' = 4
'0529' = 4
'0531' = 2
'0532' = 2
'0539' = 2
'0581' = 4
'0589' = 4
'059' = 4
'0601' = 2
'0602' = 4
'0609' = 4
'0611' = 1
'0612' = 3
'0613' = 3
'0619' = 3
'062' = 4
'0631' = 4
'0639' = 4
'064' = 4
'0650' = 4
'0651' = 4
'0652' = 4
'066' = 4
'067' = 4
'0681' = 4
'0689' = 4
'0691' = 4
'0692' = 4
'0693' = 4
'0694' = 4
'0695' = 4
'0698' = 4
'0699' = 4
'0700' = 4
'0701' = 4
'0702' = 4
'0711' = 1
'0712' = 3
'0713' = 3
'0714' = 3
'0715' = 3
'0716' = 3
'0717' = 3
'0719' = 3
'0721' = 4
'0722' = 4
'0729' = 4
'073' = 4
'0741' = 4
'0742' = 4
'0743' = 4
'0744' = 4
'0745' = 4
'0749' = 4
'0751' = 4
'0752' = 4
'0753' = 4
'0754' = 4
'0759' = 4
'0761' = 4
'0762' = 4
'0763' = 4
'0764' = 4
'0765' = 4
'0768' = 4
'0769' = 4
'0771' = 4
'0772' = 4
'0779' = 4
'0780' = 4
'0781' = 4
'0782' = 4
'0791' = 4
'0792' = 4
'0793' = 4
'0794' = 4
'0799' = 4
'0801' = 2
'0802' = 2
'0809' = 2
'0811' = 3
'0819' = 1
'0820' = 4
'0821' = 4
'0822' = 4
'0823' = 4
'0824' = 4
'0825' = 4
'0831' = 4
'0832' = 4
'0833' = 4
'0834' = 4
'0835' = 4
'0836' = 4
'0837' = 4
'0838' = 4
'0841' = 4
'0842' = 4
'0843' = 4
'0844' = 4
'0849' = 4
'0851' = 4
'0852' = 4
'0859' = 4
'0861' = 4
'0862' = 4
'0863' = 4
'0864' = 4
'0869' = 4
'0870' = 4
'0871' = 4
'0872' = 4
'0873' = 4
'0874' = 4
'0881' = 2
'0882' = 2
'0883' = 2
'0884' = 2
'0885' = 2
'0886' = 2
'0887' = 2
'0889' = 2
'0891' = 4
'0892' = 4
'0893' = 4
'0899' = 4
'090' = 4
'0911' = 3
'0912' = 3
'0919' = 3
'0920' = 4
'0921' = 4
'0922' = 4
'0923' = 4
'093' = 4
'0941' = 4
'0942' = 4
'0943' = 4
'0944' = 4
'0949' = 4
'0951' = 4
'0952' = 4
'0953' = 4
'0959' = 4
'096' = 4
'0971' = 4
'0972' = 4
'0973' = 4
'0981' = 4
'0982' = 4
'0983' = 4
'0991' = 4
'0999' = 4
'100' = 4
'101' = 4
'1021' = 3
'1029' = 3
'1031' = 4
'1032' = 4
'1033' = 4
'1041' = 4
'1042' = 4
'1043' = 4
'1044' = 4
'1049' = 4
'105' = 4
'106' = 4
'1091' = 4
'1099' = 4
'110' = 4
'111' = 4
'1121' = 3
'1122' = 3
'1129' = 3
'1131' = 4
'1132' = 4
'1139' = 4
'1141' = 4
'1142' = 4
'1143' = 4
'1149' = 4
'1151' = 4
'1152' = 4
'1153' = 4
'1159' = 4
'1160' = 4
'1161' = 4
'1162' = 4
'1163' = 4
'1164' = 4
'1169' = 4
'1171' = 4
'1172' = 4
'1173' = 4
'1174' = 4
'1175' = 4
'1176' = 4
'1179' = 4
'1191' = 4
'1192' = 4
'1199' = 4
'1200' = 4
'1201' = 4
'1202' = 4
'1211' = 4
'1212' = 4
'1213' = 4
'1214' = 4
'1221' = 3
'1222' = 3
'1229' = 3
'1231' = 4
'1232' = 4
'1233' = 4
'1234' = 4
'1235' = 4
'1239' = 4
'1240' = 4
'1241' = 4
'1242' = 4
'1243' = 4
'1244' = 4
'1251' = 4
'1252' = 4
'1253' = 4
'1254' = 4
'1255' = 4
'1259' = 4
'1261' = 4
'1262' = 4
'1263' = 4
'1264' = 4
'1265' = 4
'1266' = 4
'1267' = 4   /** Added new in V4.4 (Nov 2011)  **/
'1269' = 4
'1271' = 4
'1272' = 4
'1273' = 4
'1274' = 4
'1279' = 4
'1281' = 4
'1282' = 4
'1283' = 4
'1284' = 4
'1285' = 4
'1286' = 4
'1287' = 4
'1288' = 4
'1289' = 4
'1291' = 4
'1292' = 4
'1293' = 4
'1297' = 4
'1298' = 4
'1299' = 4
'1300' = 4
'1301' = 4
'1302' = 4
'1311' = 4
'1319' = 4
'132' = 4
'133' = 4
'1341' = 4
'1342' = 4
'1343' = 4
'1351' = 4
'1359' = 4
'1361' = 4
'1362' = 4
'1363' = 4
'1364' = 4
'1365' = 4
'1366' = 4
'1369' = 4
'1370' = 4
'1371' = 4
'1372' = 4
'138' = 4
'139' = 4
'1390' = 4
'1391' = 4
'1400' = 4
'1401' = 4
'1402' = 4
'1411' = 3
'1419' = 3
'1421' = 4
'1422' = 4
'1423' = 2
'1424' = 2
'1425' = 2
'1426' = 4
'1427' = 4
'1429' = 4
'1431' = 4
'1432' = 4
'1433' = 2
'1434' = 2
'1435' = 2
'1439' = 4
'1441' = 4
'1449' = 4
'1451' = 4
'1452' = 4
'1453' = 4
'1454' = 4
'1455' = 4
'1459' = 4
'146' = 4
'1471' = 4
'1472' = 4
'1473' = 4
'1474' = 4
'1475' = 4
'1479' = 4
'149' = 4
'1501' = 3
'1509' = 3
'1511' = 4
'1512' = 4
'1513' = 4
'1519' = 4
'1521' = 4
'1522' = 4
'1529' = 4
'153' = 4
'154' = 4
'155' = 4
'156' = 4
'157' = 4
'159' = 4
'1601' = 4
'1602' = 4
'1609' = 4
'161' = 4
'1621' = 1
'1622' = 3
'1623' = 3
'1629' = 3
'1631' = 4
'1639' = 4
'1641' = 4
'1642' = 4
'1649' = 4
'1651' = 4
'1652' = 4
'1659' = 4
'1661' = 4
'1662' = 4
'1663' = 4
'1664' = 4
'1665' = 4
'1666' = 4
'1669' = 4
'1671' = 4
'1672' = 4
'1681' = 4
'1682' = 4
'1689' = 4
'1691' = 2
'1692' = 4
'1693' = 4
'1698' = 4
'1699' = 4
'1741' = 2 
'1742' = 2 
'1743' = 2 
'1744' = 2 
'1745' = 2 
'1749' = 2 
'1751' = 4
'1752' = 4
'1753' = 4   /** Added new in V4.4 (Nov 2011)  **/
'1754' = 4   /** Added new in V4.4 (Nov 2011)  **/
'1755' = 4   /** Added new in V4.4 (Nov 2011)  **/
'1756' = 4   /** Added new in V4.4 (Nov 2011)  **/
'1761' = 4
'1762' = 4
'1763' = 4
'1769' = 4
'1770' = 2
'1771' = 1
'1781' = 2   /** Added new in V4.4 (Nov 2011)  **/
'1801' = 2
'1802' = 2
'1809' = 2
'1811' = 1
'1812' = 1
'1819' = 1
'1821' = 4
'1829' = 2
'1831' = 4
'1839' = 4
'184'  = 2
'185'  = 4
'186'  = 4
'1871' = 4
'1872' = 4
'1879' = 4
'189' = 4
'190' = 4
'1911' = 4
'1919' = 4
'1921' = 4
'1929' = 4
'193' = 4
'194' = 4
'1952' = 4
'1953' = 4
'1954' = 4
'1955' = 4
'196' = 4
'199' = 4
'2001' = 4
'2009' = 2
'201' = 2
'2021' = 4
'2022' = 4
'2023' = 4
'2031' = 1
'2032' = 3
'2039' = 3
'2041' = 4
'2042' = 4
'2049' = 4
'2051' = 4
'2059' = 4
'2061' = 4
'2062' = 4
'2071' = 4
'2072' = 4
'2079' = 4
'208' = 2
'2091' = 4
'2092' = 4
'2093' = 4
'2094' = 2
'2095' = 4
'2096' = 4
'2097' = 4
'2098' = 4
'2099' = 4
'2100' = 2
'2101' = 2
'2102' = 2
'2103' = 2
'2104' = 4
'2105' = 4
'2106' = 4
'2107' = 4
'2109' = 4
'211' = 2
'2121' = 1
'2122' = 1
'2129' = 1
'2130' = 2
'2131' = 2
'2132' = 2
'214' = 4
'215' = 4
'2161' = 4
'2162' = 4
'2169' = 4
'2171' = 2
'2172' = 4
'2181' = 2
'2182' = 4
'2183' = 4
'2184' = 4
'2185' = 4
'2186' = 4
'2187' = 4
'2188' = 4
'2189' = 4
'2191' = 2
'2199' = 4
'2200' = 2
'2201' = 2
'2202' = 2
'2211' = 1
'2212' = 3
'2219' = 1
'222' = 2
'2231' = 4
'2239' = 4
'2241' = 4
'2242' = 4
'2250' = 4
'2251' = 4
'2252' = 4
'2253' = 4
'2260' = 4
'2261' = 4
'2262' = 4
'2263' = 4
'2264' = 4
'2271' = 4
'2279' = 4
'229' = 4
'2301' = 2
'2309' = 2
'2311' = 2
'2319' = 2
'232' = 2
'233' = 2
'2341' = 2
'2342' = 2
'2343' = 2
'2349' = 2
'235' = 2
'236' = 2
'2370' = 2
'2371' = 2
'2372' = 2
'2373' = 2
'240' = 2
'2411' = 1
'2412' = 1
'2419' = 1
'242' = 4
'2431' = 2
'2432' = 2
'2439' = 2
'244' = 4
'245' = 4
'246' = 2
'247' = 2
'248' = 2
'2491' = 2
'2499' = 2
'2501' = 1
'2502' = 3
'2509' = 1
'251' = 4
'252' = 4
'253' = 4
'254' = 4
'2551' = 2
'2559' = 4
'2591' = 2
'2592' = 2
'2593' = 2
'2594' = 4
'2599' = 4
'260' = 2
'2611' = 1
'2612' = 3
'2619' = 1
'2621' = 4
'2629' = 4
'2630' = 4
'2631' = 4
'2632' = 4
'2641' = 4
'2642' = 4
'2649' = 4
'2691' = 2
'2699' = 4
'270' = 4
'271' = 4
'2721' = 3
'2722' = 3
'2723' = 1
'2724' = 1
'2729' = 1
'2731' = 4
'2732' = 4
'2741' = 2
'2742' = 4
'2743' = 4
'2749' = 4
'2751' = 2
'2752' = 2
'2753' = 4
'2754' = 4
'2755' = 4
'2756' = 4
'2757' = 4
'2759' = 4
'2761' = 4
'2762' = 4
'2763' = 4
'2764' = 2
'2769' = 4
'2771' = 4
'2772' = 4
'2773' = 4
'2779' = 4
'2791' = 2
'2792' = 4
'2799' = 4
'280' = 4
'2811' = 3
'2819' = 3
'282' = 4
'283' = 4
'284' = 4
'285' = 4
'286' = 4
'287' = 4
'2891' = 4
'2892' = 4
'2899' = 4
'290' = 4
'2911' = 1
'2912' = 1
'2919' = 1
'292' = 4
'293' = 4
'2931' = 4
'2932' = 4
'2933' = 4
'2939' = 4
'294' = 4
'2951' = 4
'2952' = 4
'2953' = 4
'2954' = 4
'2959' = 4
'2991' = 2
'2992' = 4
'2999' = 4
'3001' = 4
'3009' = 4
'301' = 4
'3021' = 4
'3022' = 4
'3029' = 4
'303' = 4
'304' = 4
'310' = 2
'311' = 2
'3121' = 4
'3129' = 4
'313' = 4
'3141' = 1
'3142' = 1
'3143' = 1
'3144' = 1
'3145' = 3
'3148' = 1
'3149' = 1
'315' = 4
'3161' = 4
'3162' = 4
'3163' = 4
'3164' = 4
'3169' = 4
'3171' = 4
'3172' = 4
'3173' = 4
'3174' = 4
'3175' = 4
'3179' = 4
'3191' = 4
'3192' = 4
'3193' = 2
'3194' = 2
'3195' = 2
'3198' = 4
'3199' = 4
'320' = 4
'3201' = 2
'3209' = 4
'321' = 4
'3221' = 4
'3222' = 4
'3223' = 4
'3224' = 4
'3225' = 4
'3226' = 4
'3227' = 4
'3228' = 2
'3229' = 4
'323' = 4
'324' = 4
'325' = 4
'326' = 4
'329' = 4
'330' = 4
'331' = 4
'3321' = 1
'3322' = 1
'3323' = 1
'3324' = 1
'3325' = 3
'3326' = 1
'3327' = 3
'3328' = 3
'3329' = 3
'3331' = 2
'3332' = 2
'3333' = 2
'3334' = 4
'3339' = 4
'3341' = 4
'3342' = 4
'3343' = 4
'3348' = 4
'3349' = 4
'335' = 4
'3350' = 4
'3351' = 4
'3352' = 4
'336' = 4
'3371' = 2
'3372' = 2 
'3373' = 2
'3378' = 2
'3379' = 2
'3391' = 2
'3392' = 4
'3393' = 4
'3398' = 4
'3399' = 4
'3401' = 2
'3402' = 4
'3403' = 4
'3404' = 2
'3405' = 2
'3409' = 2
'341' = 4
'3421' = 3
'3422' = 3
'3423' = 1
'3424' = 1
'3425' = 1
'3426' = 3
'3427' = 3
'3428' = 3
'3429' = 3
'343' = 4
'344' = 4
'3451' = 4
'3459' = 4
'346' = 4
'3471' = 2
'3472' = 2
'3473' = 4
'3474' = 4
'3479' = 4
'3481' = 4
'3482' = 4
'3483' = 4
'3484' = 4
'3485' = 4
'3489' = 4
'3491' = 2
'3492' = 2
'3493' = 4
'3499' = 4
'3500' = 4
'3501' = 4
'3502' = 4
'3503' = 4
'3504' = 4
'3505' = 4   /** Added new in V4.4 (Nov 2011)  **/
'3506' = 4   /** Added new in V4.4 (Nov 2011)  **/
'3507' = 4   /** Added new in V4.4 (Nov 2011)  **/
'3508' = 4   /** Added new in V4.4 (Nov 2011)  **/
'3509' = 4   /** Added new in V4.4 (Nov 2011)  **/
'3510' = 4
'3511' = 4
'3512' = 4
'3513' = 4
'3514' = 4
'3520' = 4
'3521' = 4
'3522' = 4
'3523' = 4
'3524' = 4
'3525' = 4
'3526' = 4
'3527' = 4
'3528' = 4
'3531' = 4
'3532' = 4
'3533' = 4
'3534' = 4
'3535' = 4
'3539' = 4
'3541' = 2
'3542' = 4
'3550' = 4
'3551' = 4
'3552' = 4
'3553' = 4
'3554' = 4
'3555' = 4
'3560' = 4
'3561' = 4
'3562' = 4
'3563' = 4
'3570' = 4
'3571' = 4
'3572' = 4
'3573' = 4
'3581' = 4
'3582' = 4
'3583' = 4
'3584' = 4
'3591' = 4
'3592' = 4
'3593' = 4
'3594' = 4
'3595' = 4
'3596' = 4
'3597' = 4
'3598' = 4
'3599' = 4
'3600' = 4
'3601' = 4
'3602' = 4
'3603' = 4
'3604' = 2
'3605' = 4
'3606' = 2
'3607' = 2
'3609' = 4
'3610' = 4
'3611' = 4
'3612' = 4
'3613' = 4
'3614' = 4
'3615' = 4
'3616' = 4
'3617' = 4
'3619' = 4
'362' = 4
'363' = 4
'3631' = 4
'3632' = 4
'3633' = 4
'3634' = 4
'3639' = 4
'3691' = 4
'3699' = 4
'370' = 2
'3710' = 4
'3711' = 4
'3712' = 4
'3720' = 1
'3721' = 1
'3722' = 1
'3723' = 1
'3724' = 3
'3725' = 1
'3726' = 1
'3727' = 1
'3728' = 1
'3729' = 1
'3731' = 4
'3732' = 4
'3733' = 4
'3734' = 4
'3735' = 4
'3736' = 2 
'3737' = 4
'374' = 4
'3741' = 4
'3749' = 4
'375' = 4
'3751' = 4
'3752' = 4
'3753' = 4
'3754' = 4
'3760' = 4
'3761' = 4
'3762' = 4
'3763' = 4
'3764' = 4
'3765' = 4
'3766' = 4
'3767' = 4
'3768' = 4 
'3770' = 2
'3771' = 2
'3772' = 2
'3773' = 2
'3774' = 4
'3775' = 4
'3776' = 4
'3777' = 4
'3778' = 2
'3779' = 4
'3780' = 4
'3781' = 2
'3782' = 2
'3783' = 2
'3785' = 4
'3786' = 4
'3787' = 4
'3789' = 4
'3790' = 2 
'3791' = 4
'3792' = 2
'3793' = 2
'3794' = 4
'3795' = 4
'3796' = 4
'3797' = 4
'3798' = 4
'3799' = 4
'3800' = 4
'3801' = 4
'3802' = 4
'3803' = 4
'3804' = 4
'3805' = 4
'3806' = 4
'3807' = 4
'3808' = 4
'3809' = 4
'3810' = 4
'3811' = 4
'3812' = 4
'3813' = 4
'3814' = 4
'3815' = 4
'3816' = 4
'3818' = 4
'3821' = 3
'3822' = 1
'3823' = 1 
'3824' = 1
'3825' = 1
'3826' = 4   /** Added new in V4.4 (Nov 2011)  **/
'3829' = 3
'3830' = 4
'3831' = 4
'3832' = 4
'3833' = 4
'3834' = 4
'3835' = 4
'3836' = 4
'3837' = 4
'3838' = 4
'3839' = 4
'3840' = 4
'3841' = 4
'3842' = 4
'3843' = 4
'3844' = 4
'3845' = 4
'3846' = 4
'3847' = 4
'3848' = 4
'3849' = 4
'3850' = 4
'3851' = 4
'3852' = 4
'3853' = 4
'3855' = 4
'3857' = 4
'3859' = 4
'3860' = 4
'3861' = 4
'3862' = 4
'3863' = 4
'3864' = 4
'3865' = 4
'3866' = 4
'3867' = 4
'3868' = 4
'3869' = 4
'387' = 4
'3880' = 4
'3881' = 4
'3882' = 4
'3883' = 4
'3884' = 4
'3885' = 4
'3886' = 4
'3887' = 4
'3888' = 4
'3889' = 4
'3891' = 2
'3892' = 2
'3893' = 2
'3894' = 2
'3895' = 2
'3897' = 2
'3898' = 2
'3899' = 2
'390' = 4
'391'  = 4
'3921' = 4
'3922' = 4
'3923' = 4
'3924' = 4
'3925' = 4
'3926' = 4
'3927' = 4
'3928' = 4
'3929' = 4
'3930' = 4
'3931' = 4
'3932' = 4
'3941' = 4
'3942' = 4
'3943' = 4
'3949' = 4
'3950' = 4
'3951' = 4
'3952' = 4
'3953' = 4
'3954' = 4
'3955' = 4
'3956' = 4
'3957' = 4
'3958' = 4
'3959' = 4
'3961' = 2
'3962' = 2
'3963' = 2
'3964' = 2
'3965' = 4  
'3966' = 2
'397'  = 4
'3971' = 4
'3972' = 4
'3973' = 4
'3974' = 4
'3975' = 4
'3976' = 4
'3977' = 4   
'3978' = 4   
'3979' = 4
'398'  = 4
'3981' = 4
'3982' = 4
'3983' = 4
'3984' = 4
'3985' = 4
'3986' = 4
'3987' = 4
'3988' = 4
'3989' = 4
'3990' = 2
'3991' = 4
'3992' = 4
'3993' = 4
'3994' = 4
'3995' = 2
'3996' = 2
'3997' = 2
'3998' = 4
'3999' = 4
'400'  = 4
'4011' = 3
'4019' = 3
'4021' = 4
'4022' = 4
'4023' = 4
'4024' = 4
'4029' = 4
'403' = 4
'4040' = 4
'4041' = 4
'4042' = 4
'4050' = 4
'4051' = 4
'4052' = 4
'4053' = 4
'4054' = 4
'4059' = 4
'4061' = 4
'4062' = 4
'4063' = 4
'4064' = 4
'4069' = 4
'409' = 4
'410' = 2  
'4100' = 2 
'4101' = 2
'4102' = 2 
'4103' = 2 
'4104' = 2 
'4105' = 2 
'4106' = 2 
'4107' = 2 
'4108' = 2 
'4109' = 2 
'411' = 2
'412' = 4
'4131' = 1
'4132' = 1
'4133' = 3
'4138' = 1
'4139' = 1
'4141' = 4
'4142' = 4
'4143' = 4
'415' = 4
'4191' = 2
'4192' = 2
'4193' = 4
'4194' = 4
'4195' = 4
'4198' = 2
'4199' = 4
'4201' = 4
'4209' = 4
'4210' = 4
'4211' = 4
'4212' = 4
'4219' = 4
'4221' = 3
'4222' = 1
'4223' = 1
'4224' = 1
'4225' = 3
'4229' = 1
'4231' = 4
'4232' = 4
'4233' = 2
'4239' = 4
'4240' = 4
'4241' = 4
'4242' = 4
'4251' = 4
'4252' = 4
'4253' = 4
'4254' = 4
'4255' = 4
'4256' = 4
'4258' = 4
'4259' = 4
'4261' = 4
'4262' = 4
'4263' = 4
'4264' = 4
'4265' = 4
'4266' = 4
'4268' = 4
'4269' = 4
'427' = 4
'4281' = 2
'4282' = 4
'4283' = 4
'4284' = 4
'4285' = 4
'4286' = 4
'4287' = 4
'4289' = 4
'4291' = 4
'4292' = 2
'4299' = 2
'430' = 4
'431' = 4
'4311' = 2
'4319' = 2
'432' = 4
'433' = 4
'4341' = 2
'4342' = 4
'4349' = 4
'435' = 4
'436' = 4
'437' = 4
'4381' = 4
'4382' = 4   /** Added new in V4.4 (Nov 2011)  **/
'4389' = 4
'4391' = 4
'4399' = 4
'4400' = 4
'4401' = 4
'4402' = 4
'4403' = 4
'4411' = 3
'4412' = 1
'4413' = 1
'4414' = 1
'4415' = 3
'4419' = 1
'442' = 4
'4421' = 4
'4422' = 2
'4429' = 4
'4431' = 4
'4432' = 4
'4438' = 4
'4439' = 4
'4440' = 4
'4441' = 4
'4442' = 4
'4443' = 2
'4444' = 2
'4449' = 2
'445' = 4
'4461' = 4
'4462' = 2
'4463' = 4
'4464' = 4
'4465' = 4
'4466' = 4
'4467' = 4
'4468' = 4
'4469' = 4
'4491' = 4
'4492' = 4
'4493' = 2
'4494' = 2
'4495' = 4
'4496' = 4
'4497' = 4
'4498' = 4
'4499' = 4
'4500' = 4
'4501' = 4
'4502' = 4
'4503' = 4
'4511' = 3
'4512' = 1
'4513' = 1
'4514' = 1
'4515' = 3
'4516' = 1
'4519' = 1
'4521' = 3
'4522' = 1
'4523' = 1
'4524' = 1
'4525' = 1
'4526' = 3
'4527' = 1
'4528' = 1
'4529' = 1
'4530' = 2
'4531' = 4
'4532' = 4
'4533' = 4
'4534' = 4
'4541' = 4
'4542' = 2
'4543' = 2
'4549' = 4
'4550' = 4
'4551' = 4
'4552' = 4
'4561' = 4
'4562' = 4
'4563' = 4
'4571' = 4
'4572' = 4
'4573' = 4
'4574' = 4
'4575' = 4
'4576' = 4
'4579' = 4
'458' = 4
'4590' = 4
'4591' = 4
'4592' = 4
'4593' = 4
'4594' = 4
'4595' = 4
'4601' = 4
'4602' = 4
'4603' = 4
'4604' = 4
'4610' = 4
'4611' = 4
'4612' = 4
'4613' = 4
'4614' = 2
'4620' = 4
'4621' = 4
'4622' = 4
'4623' = 4
'4624' = 2
'4631' = 2
'4632' = 2
'4639' = 2
'4640' = 4
'4641' = 4
'4642' = 4
'4643' = 4
'4650' = 4
'4651' = 4
'4652' = 4
'4660' = 4
'4661' = 4
'4662' = 4
'4663' = 4
'4664' = 4
'4671' = 4
'4672' = 4
'4673' = 4
'4674' = 4
'4675' = 4
'4676' = 4
'4679' = 4
'4680' = 4
'4681' = 4
'4682' = 4
'4685' = 2
'4686' = 2
'4687' = 2
'4691' = 4
'4692' = 4
'4693' = 4
'4694' = 4
'4695' = 2
'4696' = 2
'4697' = 4
'4699' = 4
'470' = 4
'4701' = 4
'4709' = 4
'471' = 4
'4711' = 4
'4719' = 4
'472' = 4
'4791' = 4
'4792' = 4
'4799' = 4
'480' = 4
'481' = 4
'4821' = 3
'4822' = 1
'4823' = 1
'4824' = 1
'4825' = 3
'4826' = 1
'4829' = 1
'4831' = 2
'4832' = 2
'4833' = 2
'4834' = 2
'4835' = 4
'4836' = 2
'4840' = 4
'4841' = 4
'4849' = 4
'485' = 4
'4850' = 4
'4861' = 4
'4862' = 4
'4863' = 4
'4864' = 4
'4865' = 4
'4866' = 4
'4869' = 4
'4871' = 4
'4872' = 4
'4873' = 4
'4874' = 4
'4875' = 4
'4876' = 4
'4879' = 4
'4881' = 4
'4882' = 4
'4891' = 4
'4892' = 4
'4893' = 4
'4899' = 4
'4901' = 4
'4902' = 4
'4903' = 2
'4904' = 4
'4911' = 4
'4912' = 4
'4921' = 1
'4922' = 1
'4923' = 1
'4929' = 1
'493' = 4
'4931' = 2
'4939' = 4
'4941' = 2
'4942' = 2
'4943' = 2
'4944' = 4
'4945' = 4
'4946' = 4
'4947' = 2
'4949' = 4
'4951' = 4
'4952' = 4
'4959' = 4
'496' = 4
'4971' = 4
'4972' = 4
'4973' = 4
'4974' = 4
'4975' = 4
'4976' = 4
'4979' = 4
'4991' = 4
'4992' = 4
'4993' = 4
'4994' = 4
'4995' = 4
'4999' = 4
'500' = 4
'5011' = 1
'5012' = 3
'5013' = 1 
'5019' = 3
'5021' = 4
'5022' = 4
'5023' = 4
'5024' = 4
'5025' = 4
'5026' = 4
'5029' = 4
'503' = 4
'504' = 4
'5051' = 4
'5059' = 4
'5061' = 4
'5069' = 4
'5091' = 2
'5092' = 2
'5093' = 2
'5094' = 2
'5099' = 2
'5101' = 2
'5102' = 4
'5103' = 4
'5104' = 4
'5110' = 1
'5111' = 1
'5112' = 1
'5113' = 3
'5114' = 1
'5115' = 1
'5119' = 3
'5121' = 4
'5122' = 4
'5123' = 4
'5124' = 4
'5131' = 4
'5132' = 4
'5133' = 4
'5134' = 4
'5135' = 4
'5136' = 4
'5137' = 4
'5139' = 4
'5141' = 4
'5142' = 4
'5143' = 4
'5149' = 4
'5151' = 4
'5159' = 4
'5161' = 4
'5162' = 4
'5163' = 4
'5164' = 2
'5169' = 4
'5171' = 4
'5172' = 4
'5179' = 4
'5181' = 4
'5182' = 4
'5183' = 4
'5184' = 2
'5185' = 2
'5186' = 2
'5187' = 2
'5188' = 2
'5189' = 4
'5191' = 4
'5192' = 4
'5193' = 4
'5194' = 4
'5195' = 4
'5196' = 2
'5197' = 2
'5198' = 2
'5199' = 4
'5201' = 4
'5209' = 4
'5211' = 1
'5212' = 3
'5213' = 1
'5214' = 1
'5219' = 3
'522' = 4
'5221' = 2
'5222' = 4
'523' = 4
'524' = 4
'5251' = 4
'5252' = 4
'5253' = 4
'5259' = 4
'526' = 4
'527' = 4
'5280' = 4
'5281' = 4
'5282' = 4
'5283' = 4
'5284' = 2
'5285' = 2
'5286' = 2
'5291' = 4
'5292' = 4
'5293' = 2
'5294' = 2
'5295' = 4
'5296' = 4
'5297' = 2
'5298' = 2
'5299' = 4
'5300' = 4
'5301' = 4
'5302' = 4
'5303' = 4
'5304' = 4
'5305' = 4
'5310' = 4
'5311' = 4
'5312' = 4
'5313' = 4
'5314' = 4
'5315' = 4
'5316' = 4
'5317' = 4
'5321' = 4
'5329' = 4
'5331' = 4
'5339' = 4
'5341' = 4
'5349' = 4
'5351' = 4
'5359' = 4
'5361' = 4
'5369' = 4
'537' = 4
'5380' = 4
'5381' = 4
'5382' = 4
'539' = 4
'540' = 4
'5411' = 4
'5412' = 4
'5419' = 4
'5421' = 3
'5422' = 3
'5423' = 3
'5424' = 1
'5425' = 1
'5429' = 3
'543' = 4
'544' = 4
'545' = 4
'5451' = 4
'5459' = 4
'5461' = 4
'5462' = 4
'5463' = 4
'5464' = 4
'5471' = 4
'5472' = 4
'5473' = 4
'5474' = 4
'5475' = 4
'5491' = 2
'5492' = 4
'5493' = 4
'5494' = 4
'5495' = 4
'5496' = 2
'5497' = 2
'5498' = 2
'5499' = 2
'5501' = 4
'5502' = 4
'5503' = 4
'5504' = 4
'5511' = 4
'5512' = 4
'5521' = 1
'5522' = 1
'5523' = 1
'5524' = 3
'5529' = 3
'5531' = 4
'5532' = 4
'5533' = 4
'5534' = 4
'5535' = 4
'5539' = 4
'554' = 4
'5551' = 4
'5552' = 4
'5553' = 4
'5554' = 4
'5561' = 4
'5569' = 4
'557' = 4
'5581' = 4
'5582' = 4
'5583' = 4
'5584' = 4
'5585' = 4
'5586' = 4
'5587' = 4
'5589' = 4
'5591' = 4
'5592' = 2
'5593' = 2
'5594' = 2
'5595' = 2
'5596' = 2
'5597' = 4
'5598' = 4
'5599' = 4
'560' = 4
'561' = 4
'562' = 4
'5631' = 1
'5632' = 1
'5633' = 1
'5634' = 3
'5635' = 1
'5639' = 3
'5640' = 4
'5641' = 4
'5642' = 4
'5651' = 4
'5652' = 4
'5661' = 4
'5662' = 4
'5671' = 4
'5672' = 4
'5673' = 4
'5674' = 4
'5675' = 4
'5679' = 4
'5681' = 4
'5682' = 4
'5683' = 4
'5684' = 4
'5685' = 4
'5686' = 4
'5689' = 4
'5691' = 2
'5692' = 4
'5693' = 4
'5694' = 4
'5695' = 4
'5699' = 4
'570' = 2
'5711' = 2
'5712' = 4
'5717' = 2
'5718' = 4
'5719' = 4
'5721' = 4
'5722' = 4
'5731' = 1
'5732' = 1
'5733' = 3
'5734' = 3
'5739' = 3
'5741' = 4
'5749' = 4
'5751' = 4
'5759' = 4
'576' = 4
'5771' = 4
'5779' = 4
'5781' = 4
'5782' = 4
'5783' = 4
'5784' = 4
'5785' = 4
'5786' = 4
'5787' = 4
'5788' = 4
'5789' = 4
'5791' = 4
'5792' = 2
'5793' = 4
'5794' = 2
'5795' = 2
'5796' = 4
'5797' = 4
'5798' = 4
'5799' = 4
'580' = 4
'581' = 4
'5821' = 1
'5822' = 1
'5823' = 1
'5824' = 1
'5829' = 1
'583' = 2
'5831' = 2
'5839' = 2
'5841' = 4
'5842' = 4
'5843' = 4
'5844' = 4
'5845' = 4
'5846' = 4
'5847' = 4
'5849' = 4
'585' = 4
'586' = 2
'5891' = 4
'5892' = 4
'5893' = 4
'5899' = 4
'5900' = 4
'5901' = 4
'5902' = 4
'5903' = 4
'5909' = 4
'5911' = 4
'5912' = 4
'5919' = 4
'5921' = 3
'5929' = 3
'593' = 4
'594' = 4
'595' = 4
'596' = 4
'5971' = 4
'5972' = 2
'5979' = 4
'598' = 2
'5991' = 4
'5992' = 4
'5993' = 2
'5994' = 2
'5995' = 2
'5996' = 2
'5999' = 2
'600' = 4
'6011' = 1
'6012' = 3
'6013' = 1
'6014' = 3
'6015' = 3
'6018' = 3
'6019' = 3
'602' = 4
'6021' = 4
'6029' = 4
'603' = 4
'604' = 4
'605' = 4
'6061' = 4
'6062' = 4
'6069' = 4
'6071' = 2
'6072' = 4
'6073' = 4
'6079' = 4
'6081' = 4
'6082' = 4
'6091' = 2
'6092' = 2
'6093' = 4
'6094' = 4
'6095' = 4
'6096' = 4
'6097' = 4
'6099' = 4
'610' = 2
'6111' = 1
'6119' = 1
'612' = 4
'613' = 2
'6141' = 2
'6142' = 4
'6149' = 4
'6191' = 2
'6192' = 4
'6199' = 4
'620' = 4
'6211' = 1
'6212' = 3
'6219' = 3
'622' = 4
'623' = 4
'6241' = 4
'6242' = 4
'625' = 4
'6261' = 4
'6269' = 4
'627' = 4
'6291' = 2
'6292' = 2
'6299' = 4
'6301' = 1
'6309' = 3
'631' = 4
'632' = 4
'633' = 4
'634' = 4
'6351' = 4
'6352' = 2
'6353' = 4
'6359' = 4
'636' = 2
'6370' = 2
'6371' = 2
'6372' = 2
'6373' = 2
'6381' = 4
'6382' = 4
'6383' = 4
'6384' = 2
'6385' = 4
'6389' = 4
'6391' = 2
'6392' = 4
'6393' = 4
'6394' = 4
'6395' = 4
'6399' = 4
'640' = 4
'6411' = 3
'6419' = 1
'642' = 4
'643' = 4
'6441' = 4
'6442' = 4
'6443' = 4
'6444' = 4
'6445' = 4
'6449' = 4
'645' = 4
'6491' = 2
'6492' = 4
'6493' = 4
'6494' = 2
'6495' = 4
'6496' = 4
'6497' = 4
'6498' = 4
'6499' = 4
'650' = 4
'6501' = 4
'6509' = 4
'6511' = 3
'6512' = 3
'6513' = 3
'6514' = 3
'6519' = 3
'6521' = 4
'6522' = 4
'6523' = 4
'6524' = 4
'6525' = 4
'6529' = 4
'653' = 4
'6531' = 4
'6539' = 4
'654' = 4
'6541' = 4
'6549' = 4
'6551' = 4
'6552' = 4
'6553' = 4
'6554' = 4
'6561' = 4
'6562' = 4
'6563' = 4
'6564' = 4
'6571' = 4
'6572' = 4
'6573' = 4
'6574' = 4
'6575' = 4
'6576' = 4
'6579' = 4
'658' = 4
'6581' = 4
'6589' = 4
'6591' = 4
'6592' = 4
'6593' = 4
'6594' = 4
'6595' = 4
'6599' = 4
'660' = 4
'6601' = 4
'6602' = 4
'6611' = 3
'6619' = 3
'6621' = 4
'6622' = 4
'6629' = 4
'6631' = 4
'6632' = 4
'6639' = 4
'664' = 4
'6651' = 4
'6652' = 4
'6661' = 4
'6662' = 4
'6663' = 4
'6669' = 4
'6671' = 4
'6672' = 4
'6673' = 4
'6674' = 4
'6679' = 4
'668' = 2
'6691' = 2
'6692' = 4
'6693' = 4
'6694' = 4
'6695' = 4
'6696' = 4
'6697' = 4
'6699' = 4
'670' = 2
'6711' = 3
'6712' = 3
'6719' = 3
'672' = 4
'6731' = 4
'6732' = 4
'6733' = 4
'6739' = 4
'674' = 4
'675' = 4
'6751' = 4
'6759' = 4
'6761' = 4
'6762' = 4
'6769' = 4
'680' = 4
'6811' = 1
'6812' = 1
'6813' = 3
'6814' = 3
'6815' = 3
'6816' = 3
'6819' = 3
'6821' = 4
'6822' = 4
'6823' = 4
'6824' = 4   /** Added new in V4.4 (Nov 2011)  **/
'6825' = 4   /** Added new in V4.4 (Nov 2011)  **/
'6829' = 4
'683' = 4
'6831' = 4
'6839' = 4
'684' = 4
'6841' = 4
'6849' = 4
'685' = 4
'6851' = 4
'6859' = 4
'686' = 4
'6861' = 4
'6869' = 4
'687' = 4
'6871' = 4
'6879' = 4
'688' = 4
'689' = 4
'6901' = 4
'6902' = 4
'6909' = 3
'6911' = 4
'6919' = 4
'6921' = 4
'6922' = 4
'6923' = 4
'6929' = 4
'693' = 4
'6941' = 4
'6942' = 4
'6949' = 4
'6951' = 4
'6952' = 4
'6959' = 2
'696' = 2
'697' = 2
'6991' = 2
'6992' = 2
'6993' = 2
'6994' = 2
'6995' = 4
'6996' = 2
'6997' = 4
'6998' = 4
'6999' = 4
'700' = 2
'7011' = 2
'7012' = 4
'7013' = 4
'7014' = 4
'7021' = 1
'7022' = 1
'7023' = 3
'7024' = 3
'7029' = 3
'7031' = 4
'7032' = 4
'7033' = 4
'704' = 4
'7050' = 4
'7051' = 4
'7052' = 4
'7061' = 4
'7062' = 4
'7071' = 4
'7072' = 4
'7073' = 4
'7074' = 4
'7075' = 4
'7076' = 4
'7077' = 4
'7079' = 4
'708' = 4
'7091' = 4
'7092' = 4
'7094' = 2 
'7095' = 2 
'7101' = 4
'7109' = 4
'7111' = 3
'7119' = 3
'7121' = 2
'7122' = 4
'7123' = 4
'7124' = 4
'7129' = 4
'713' = 4
'714' = 4
'715' = 4
'7161' = 4
'7162' = 4
'7171' = 4
'7172' = 4
'7179' = 4
'718' = 4
'719' = 4
'720' = 2
'721' = 2
'7221' = 2
'7229' = 2
'7231' = 2
'7239' = 2
'724' = 2
'7251' = 2
'7252' = 2
'7253' = 2
'7254' = 2
'726' = 2
'7271' = 2
'7279' = 2
'728' = 2
'729' = 2
'7301' = 2
'7309' = 2
'731' = 2
'7321' = 2
'7322' = 2
'733' = 2
'734' = 2
'7351' = 2
'7359' = 2
'736' = 2
'738' = 2
'7391' = 2
'7392' = 2
'7393' = 2
'7394' = 4
'7399' = 4
'740' = 4
'741' = 4
'742' = 4
'743' = 4
'744' = 4
'7491' = 4
'7499' = 4
'750' = 2
'751' = 1
'752' = 2
'7531' = 1
'7532' = 1
'7533' = 1
'7534' = 1
'7535' = 1
'7536' = 4
'7537' = 2
'7538' = 1
'754' = 2
'7550' = 4
'7551' = 4
'7552' = 4
'7561' = 4
'7562' = 2
'7569' = 2
'757' = 2
'758' = 2
'7591' = 2
'7592' = 2
'7593' = 4
'7594' = 2
'7599' = 4
'7601' = 4
'7609' = 4
'7611' = 3
'7619' = 3
'762' = 4
'7631' = 4
'7639' = 4
'7641' = 4
'7642' = 4
'7643' = 4
'7644' = 4
'7645' = 4
'7646' = 4
'765' = 4
'7661' = 4
'7662' = 4
'7663' = 4
'7664' = 4
'7665' = 4
'7666' = 4
'7667' = 4
'7668' = 4
'7669' = 4
'7670' = 4
'7671' = 2
'7672' = 4
'7673' = 2
'7674' = 4
'7675' = 2
'7676' = 4
'7677' = 4
'7678' = 2
'7679' = 4
'7691' = 4
'7692' = 4
'7693' = 2
'7694' = 4
'7695' = 2
'7696' = 2
'7697' = 4
'7699' = 4
'7700' = 4
'7701' = 4
'7702' = 4
'7703' = 4
'7704' = 4
'7705' = 4
'7706' = 4
'7707' = 4
'7708' = 4
'7709' = 4
'7710' = 4
'7711' = 4
'7712' = 4
'7713' = 4
'7714' = 4
'7715' = 4
'7716' = 4
'7717' = 4
'7718' = 4
'7719' = 4
'7720' = 4
'7721' = 4
'7722' = 4
'7723' = 4
'7724' = 4
'7725' = 4
'7726' = 4
'7727' = 4
'7728' = 4
'7729' = 4
'7730' = 4
'7731' = 4
'7732' = 4
'7733' = 4
'7734' = 4
'7735' = 4
'7736' = 4
'7737' = 4
'7738' = 4
'7739' = 4
'7740' = 3
'7741' = 3
'7742' = 3
'7743' = 3
'7744' = 3
'7745' = 3
'7746' = 3
'7747' = 3
'7748' = 3
'7749' = 3
'7751' = 4
'7752' = 4
'7753' = 4
'7754' = 4
'7756' = 4
'7757' = 4
'7758' = 4
'7759' = 4
'7760' = 4
'7761' = 4
'7762' = 4
'7763' = 4
'7764' = 4
'7765' = 4
'7766' = 4
'7767' = 4
'7768' = 4
'7769' = 4
'7770' = 4
'7771' = 4
'7772' = 4
'7773' = 4
'7774' = 4
'7775' = 4
'7776' = 4
'7777' = 4
'7778' = 4
'7779' = 4
'7780' = 4
'7781' = 4
'7782' = 4
'7783' = 4
'7784' = 4
'7785' = 4
'7786' = 4
'7787' = 4
'7788' = 4
'7789' = 4
'7790' = 4
'7791' = 4
'7792' = 4
'7793' = 4
'7794' = 4
'7795' = 4
'7796' = 4
'7797' = 4
'7798' = 4
'7799' = 4
'7800' = 4
'7801' = 4
'7802' = 4
'7803' = 4
'7804' = 4
'7805' = 4
'7806' = 4
'7807' = 4
'7808' = 4
'7809' = 4
'7810' = 4
'7811' = 4
'7812' = 4
'7813' = 4
'7814' = 4
'7815' = 4
'7816' = 4
'7817' = 4
'7818' = 4
'7819' = 4
'7820' = 4
'7822' = 4
'7823' = 4
'7824' = 4
'7825' = 4
'7827' = 4
'7828' = 4
'7829' = 4
'7830' = 4
'7831' = 4
'7832' = 4
'7833' = 4
'7834' = 4
'7835' = 4
'7837' = 4
'7838' = 4
'7839' = 4
'7840' = 4
'7841' = 4
'7842' = 4
'7843' = 4
'7844' = 4
'7845' = 4
'7846' = 4
'7847' = 4
'7848' = 4
'7849' = 4
'7850' = 4
'7851' = 4
'7852' = 4
'7853' = 4
'7854' = 4
'7855' = 4
'7856' = 4
'7857' = 4
'7858' = 4
'7859' = 4
'7860' = 4
'7861' = 4
'7862' = 4
'7863' = 4
'7864' = 4
'7865' = 4
'7866' = 4
'7867' = 4
'7868' = 4
'7869' = 4
'7870' = 4
'7871' = 4
'7872' = 4
'7873' = 4
'7874' = 4
'7875' = 4
'7876' = 4
'7877' = 4
'7878' = 4
'7879' = 4
'7880' = 3
'7881' = 3
'7882' = 3
'7883' = 3
'7884' = 3
'7885' = 3
'7886' = 3
'7887' = 3
'7888' = 3
'7889' = 3
'7890' = 4
'7891' = 4
'7892' = 4
'7893' = 4
'7894' = 4
'7895' = 4
'7896' = 4
'7897' = 4
'7898' = 4
'7899' = 4
'7900' = 2
'7901' = 2
'7902' = 2
'7903' = 2
'7904' = 2
'7905' = 2
'7906' = 2
'7907' = 2
'7908' = 2
'7909' = 2
'7910' = 4
'7911' = 4
'7912' = 4
'7913' = 4
'7914' = 4
'7915' = 4
'7916' = 4
'7917' = 4
'7918' = 4
'7919' = 4
'7920' = 4
'7921' = 4
'7922' = 4
'7923' = 4
'7924' = 4
'7925' = 4
'7926' = 4
'7927' = 4
'7928' = 4
'7929' = 4
'7930' = 4
'7931' = 4
'7932' = 4
'7933' = 4
'7934' = 4
'7935' = 4
'7936' = 4
'7937' = 4
'7938' = 4
'7939' = 4
'7940' = 4
'7941' = 4
'7942' = 4
'7945' = 4
'7946' = 4
'7949' = 4
'7950' = 4
'7951' = 4
'7952' = 4
'7955' = 4
'7956' = 4
'7959' = 4
'7960' = 4
'7961' = 4
'7962' = 4
'7963' = 4
'7964' = 4
'7965' = 4
'7966' = 4
'7967' = 4
'7968' = 4
'7969' = 4
'7970' = 2
'7971' = 2
'7972' = 2
'7973' = 2
'7974' = 2
'7975' = 2
'7976' = 2
'7977' = 2
'7978' = 2
'7979' = 2
'7980' = 4
'7981' = 4
'7982' = 4
'7983' = 4
'7984' = 4
'7985' = 4
'7986' = 4
'7987' = 4
'7988' = 4
'7989' = 4
'7990' = 4
'7991' = 4
'7992' = 4
'7993' = 4
'7994' = 4
'7995' = 4
'7996' = 4
'7997' = 4
'7998' = 4
'7999' = 4
'8000' = 4
'8001' = 4
'8002' = 4
'8003' = 4
'8004' = 4
'8005' = 4
'8006' = 4
'8007' = 4
'8008' = 4
'8009' = 4
'8010' = 4
'8011' = 4
'8012' = 4
'8013' = 4
'8014' = 4
'8015' = 4
'8016' = 4
'8017' = 4
'8018' = 4
'8019' = 4
'8020' = 3
'8021' = 3
'8022' = 3
'8023' = 3
'8024' = 3
'8025' = 3
'8026' = 3
'8027' = 3
'8028' = 3
'8029' = 3
'8030' = 1
'8031' = 1
'8032' = 1
'8033' = 1
'8034' = 1
'8035' = 1
'8036' = 1
'8037' = 1
'8038' = 1
'8039' = 1
'8040' = 4
'8041' = 4
'8042' = 4
'8043' = 4
'8044' = 4
'8045' = 4
'8046' = 4
'8047' = 4
'8048' = 4
'8049' = 4
'805' = 4
'8050' = 4
'8051' = 4
'8052' = 2
'8059' = 4
'806' = 4
'8070' = 4
'8071' = 4
'8072' = 4
'8073' = 4
'8074' = 4
'8075' = 4
'8076' = 4
'8077' = 4
'8078' = 4
'8079' = 4
'8080' = 4
'8081' = 4
'8082' = 4
'8083' = 4
'8084' = 4
'8085' = 4
'8086' = 4
'8087' = 4
'8088' = 4
'8089' = 4
'8090' = 4
'8091' = 4
'8092' = 4
'8093' = 4
'8094' = 4
'8095' = 4
'8096' = 4
'8097' = 4
'8098' = 4
'8099' = 4
'8100' = 4
'8101' = 4
'8102' = 4
'8103' = 4
'8104' = 4
'8105' = 4
'8106' = 4
'8107' = 4
'8108' = 4
'8109' = 4
'8111' = 4
'8112' = 4
'8113' = 4
'8114' = 4
'8115' = 4
'8116' = 4
'8117' = 4
'8118' = 4
'8120' = 4
'8121' = 4
'8122' = 4
'8123' = 4
'8124' = 4
'8125' = 4
'8126' = 4
'8127' = 4
'8128' = 4
'8129' = 4
'8130' = 4
'8131' = 4
'8132' = 4
'8133' = 4
'8134' = 4
'8135' = 4
'8136' = 4
'8137' = 4
'8138' = 4
'8139' = 4
'8140' = 4
'8141' = 4
'8142' = 4
'8143' = 4
'8144' = 4
'8145' = 4
'8146' = 4
'8147' = 4
'8148' = 4
'8149' = 4
'8151' = 4
'8152' = 4
'8153' = 4
'8154' = 4
'8155' = 4
'8156' = 4
'8157' = 4
'8159' = 4
'8161' = 4
'8162' = 4
'8163' = 4
'8164' = 4
'8165' = 4
'8166' = 4
'8169' = 4
'8171' = 4
'8172' = 4
'8173' = 4
'8174' = 4
'8175' = 4
'8179' = 4
'8180' = 4
'8181' = 4
'8182' = 4
'8183' = 4
'8184' = 4
'8185' = 4
'8186' = 4
'8187' = 4
'8188' = 4
'8191' = 2
'8192' = 2
'8193' = 4
'8194' = 4
'8195' = 4
'8196' = 4
'8197' = 4
'8198' = 3
'8199' = 4
'8201' = 4
'8202' = 4
'8203' = 4
'8204' = 2
'8209' = 4
'8211' = 4
'8212' = 4
'8219' = 4
'8221' = 4
'8222' = 4
'8229' = 4
'8231' = 4
'8232' = 4
'8233' = 4
'8234' = 4
'8235' = 4
'8236' = 4
'8239' = 4
'8241' = 4
'8242' = 4
'8243' = 4
'8244' = 4
'8245' = 4
'8246' = 4
'8251' = 4
'8252' = 4
'8253' = 4
'8254' = 4
'8255' = 4
'8256' = 4
'8257' = 4
'8258' = 4
'8259' = 4
'8261' = 4
'8269' = 4
'8271' = 4
'8272' = 4
'8279' = 4
'8281' = 4
'8282' = 4
'8283' = 4
'8284' = 4
'8285' = 4
'8286' = 4
'8289' = 4
'8291' = 4
'8292' = 2
'8293' = 2
'8294' = 2
'8295' = 2
'8296' = 2
'8299' = 4
'8301' = 4
'8302' = 4
'8303' = 4
'8309' = 4
'8311' = 4
'8312' = 4
'8313' = 4
'8314' = 4
'8319' = 4
'8321' = 3
'8329' = 3
'8331' = 4
'8332' = 4
'8339' = 4
'8341' = 4
'8342' = 4
'8343' = 4
'8344' = 4
'8345' = 4
'8349' = 4
'835' = 4
'8361' = 4
'8362' = 4
'8363' = 4
'8364' = 4
'8365' = 4
'8371' = 4
'8372' = 4
'8373' = 4
'8374' = 4
'8375' = 4
'8376' = 4
'8377' = 4
'8379' = 4
'8381' = 4
'8382' = 4
'8383' = 4
'8384' = 4
'8385' = 4
'8386' = 4
'8387' = 4
'8388' = 4
'8389' = 4
'8391' = 4
'8392' = 4
'8393' = 4
'8394' = 2
'8395' = 2
'8396' = 2
'8397' = 2
'8398' = 2
'8399' = 4
'8400' = 4
'8401' = 4
'8402' = 4
'8403' = 4
'8404' = 4
'8405' = 4
'8406' = 4
'8407' = 4
'8408' = 4
'8409' = 4
'8410' = 4
'8411' = 4
'8412' = 4
'8413' = 4
'8414' = 4
'8415' = 4
'8416' = 4
'8417' = 4
'8418' = 4
'8419' = 4
'8421' = 4
'8422' = 4
'8423' = 4
'8424' = 4
'8425' = 4
'8426' = 4
'8427' = 4
'8428' = 4
'8429' = 4
'843' = 4
'8440' = 4
'8441' = 2
'8442' = 2
'8443' = 2
'8444' = 4
'8445' = 2
'8446' = 2
'8447' = 2
'8448' = 4
'8451' = 2
'8452' = 2
'8453' = 2
'8454' = 2
'8455' = 2
'8456' = 2
'8457' = 2
'8458' = 4
'8459' = 4
'8460' = 4
'8461' = 4
'8462' = 4
'8463' = 4
'8464' = 4
'8465' = 4
'8466' = 4
'8467' = 4
'8468' = 4
'8469' = 4
'8471' = 2
'8472' = 2 
'8473' = 2 
'8491' = 4
'8492' = 4
'8493' = 4
'8494' = 4
'8499' = 4
'850' = 2
'8511' = 1
'8512' = 3
'8519' = 1
'8520' = 4
'8521' = 4
'8522' = 4
'8523' = 4
'8524' = 4
'8525' = 4
'8531' = 4
'8532' = 4
'8533' = 4
'8534' = 4
'8535' = 4
'8536' = 4
'8541' = 4
'8542' = 4
'8543' = 4
'8544' = 4
'8545' = 4
'8546' = 4
'8547' = 4
'8548' = 4
'8550' = 4
'8551' = 2
'8552' = 2
'8553' = 4
'8554' = 4
'8555' = 4
'856'  = 4
'857'  = 4
'8570' = 4
'8581' = 2
'8582' = 4
'8583' = 4
'8584' = 4
'8585' = 4
'8586' = 4
'8587' = 4
'8589' = 4
'8591' = 2
'8592' = 2
'8593' = 4
'8594' = 4
'8595' = 4
'8596' = 4
'8599' = 4
'8601' = 2
'8602' = 2
'8603' = 2
'8604' = 2
'8605' = 2
'8606' = 4
'8607' = 2
'8609' = 2
'8611' = 1
'8619' = 1
'8621' = 4
'8622' = 4
'8623' = 2
'8624' = 2
'8625' = 4
'8626' = 2
'8627' = 2
'8628' = 2
'863' = 2
'864' = 4
'8651' = 2
'8659' = 2
'8660' = 4
'8661' = 4
'8662' = 4
'8663' = 4
'8664' = 2
'8665' = 4
'8666' = 4
'8667' = 4
'8669' = 4
'8670' = 4
'8671' = 4
'8672' = 4
'8673' = 4
'8674' = 4
'8675' = 4
'8681' = 4
'8682' = 4
'8683' = 4
'8684' = 4
'8685' = 4
'8686' = 4
'8687' = 4
'8689' = 4
'8690' = 4
'8691' = 4
'8692' = 2
'8693' = 4
'8694' = 4
'8695' = 4
'8696' = 4
'8697' = 4
'8698' = 4
'8699' = 2
'8701' = 1
'8702' = 1
'8703' = 1
'8704' = 1
'8705' = 1
'8706' = 1
'8707' = 1
'8708' = 1
'8709' = 1
'8711' = 1
'8712' = 1
'8713' = 1
'8714' = 1
'8715' = 1
'8716' = 1
'8717' = 1
'8721' = 1
'8722' = 1
'8723' = 1
'8724' = 1
'8729' = 1
'8731' = 1
'8732' = 1
'8733' = 1
'8734' = 1
'8735' = 1
'8736' = 1
'8737' = 1
'8738' = 1
'8739' = 1
'8741' = 1
'8742' = 1
'8743' = 1
'8744' = 1
'8749' = 1
'8751' = 1
'8752' = 1
'8753' = 3
'8754' = 1
'8759' = 1
'8761' = 1
'8762' = 1
'8763' = 1
'8764' = 1
'8765' = 1
'8766' = 1
'8769' = 1
'8771' = 1
'8772' = 1
'8773' = 1
'8774' = 1
'8775' = 1
'8776' = 1
'8777' = 1
'8778' = 1
'8779' = 1
'8781' = 1
'8782' = 1
'8783' = 1
'8784' = 1
'8785' = 1
'8789' = 1
'8791' = 1
'8792' = 1
'8793' = 1
'8794' = 1
'8795' = 1
'8799' = 1
'8801' = 1
'8802' = 1
'8803' = 1
'8804' = 1
'8809' = 1
'8811' = 1
'8812' = 1
'8813' = 1
'8814' = 1
'8815' = 1
'8816' = 1
'8819' = 1
'8821' = 1
'8822' = 1
'8823' = 1
'8824' = 1
'8825' = 1
'8826' = 1
'8827' = 1
'8828' = 1
'8829' = 1
'8831' = 1
'8832' = 1
'8833' = 1
'8834' = 1
'8835' = 1
'8836' = 1
'8837' = 1
'8838' = 1
'8839' = 1
'8840' = 1
'8841' = 1
'8842' = 1
'8843' = 1
'8844' = 1
'8845' = 1
'8846' = 1
'8847' = 1
'8848' = 1
'8849' = 1
'8850' = 1
'8851' = 1
'8852' = 1
'8853' = 1
'8854' = 1
'8855' = 1
'8856' = 1
'8857' = 1
'8858' = 1
'8860' = 1
'8861' = 1
'8862' = 1
'8863' = 1
'8864' = 1
'8865' = 1
'8866' = 1
'8867' = 1
'8868' = 1
'8871' = 1
'8872' = 1
'8873' = 1
'8874' = 1
'8875' = 1
'8876' = 1
'8877' = 1
'8878' = 1
'8879' = 1
'8881' = 1
'8882' = 1
'8883' = 1
'8884' = 1
'8885' = 1
'8886' = 1
'8889' = 1
'8890' = 1
'8891' = 1
'8892' = 1
'8893' = 1
'8894' = 1
'8895' = 1
'8896' = 1
'8897' = 1
'8898' = 1
'8899' = 1
'8901' = 1
'8902' = 1
'8903' = 1
'8904' = 1
'8905' = 1
'8906' = 1
'8907' = 1
'8908' = 1
'8909' = 1
'8910' = 1
'8911' = 1
'8912' = 1
'8913' = 1
'8914' = 1
'8915' = 1
'8916' = 1
'8917' = 1
'8918' = 1
'8919' = 1
'8921' = 1
'8922' = 1
'8923' = 1
'8924' = 1
'8925' = 1
'8926' = 1
'8929' = 1
'8931' = 1
'8932' = 1
'8933' = 1
'8934' = 1
'8935' = 1
'8936' = 1
'8937' = 1
'8938' = 1
'8939' = 1
'8941' = 1
'8942' = 1
'8943' = 1
'8944' = 1
'8945' = 1
'8946' = 1
'8947' = 1
'8948' = 1
'8949' = 1
'8950' = 1
'8951' = 1
'8952' = 1
'8953' = 1
'8954' = 1
'8955' = 1
'8956' = 1
'8957' = 1
'8958' = 1
'8959' = 1
'8960' = 1
'8961' = 1
'8962' = 1
'8963' = 1
'8964' = 1
'8965' = 1
'8966' = 1
'8967' = 1
'8968' = 1
'8969' = 1
'897' = 1
'898' = 1
'9001' = 1
'9002' = 1
'9003' = 1
'9004' = 1
'9005' = 1
'9006' = 1
'9009' = 1
'9011' = 1
'9012' = 1
'9013' = 1
'9014' = 1
'9015' = 1
'9016' = 1
'9019' = 1
'9021' = 1
'9022' = 1
'9023' = 1
'9024' = 1
'9025' = 1
'9026' = 1
'9029' = 1
'9031' = 1
'9032' = 1
'9033' = 1
'9034' = 1
'9035' = 1
'9036' = 1
'9039' = 1
'9041' = 1
'9042' = 1
'9043' = 1
'9044' = 1
'9045' = 1
'9046' = 1
'9049' = 1
'9051' = 1
'9052' = 1
'9053' = 1
'9054' = 1
'9055' = 1
'9056' = 1
'9059' = 1
'9061' = 1
'9062' = 1
'9063' = 1
'9064' = 1
'9065' = 1
'9066' = 1
'9069' = 1
'9071' = 1
'9072' = 1
'9073' = 1
'9074' = 1
'9075' = 1
'9076' = 1
'9079' = 1
'9081' = 1
'9082' = 1
'9083' = 1
'9084' = 1
'9085' = 1
'9086' = 1
'9089' = 1
'9091' = 1
'9092' = 1
'9093' = 1
'9094' = 1
'9095' = 1
'9096' = 1
'9099' = 1
'9101' = 1
'9102' = 1
'9103' = 1
'9104' = 1
'9105' = 1
'9106' = 1
'9109' = 1
'9111' = 1
'9112' = 1
'9113' = 1
'9114' = 1
'9115' = 1
'9116' = 1
'9119' = 1
'9121' = 1
'9122' = 1
'9123' = 1
'9124' = 1
'9125' = 1
'9126' = 1
'9129' = 1
'9131' = 1
'9132' = 1
'9133' = 1
'9134' = 1
'9135' = 1
'9136' = 1
'9139' = 1
'9141' = 1
'9142' = 1
'9143' = 1
'9144' = 1
'9145' = 1
'9146' = 1
'9149' = 1
'9151' = 1
'9152' = 1
'9153' = 1
'9154' = 1
'9155' = 1
'9156' = 1
'9159' = 1
'9161' = 1
'9162' = 1
'9163' = 1
'9164' = 1
'9165' = 1
'9166' = 1
'9169' = 1
'9171' = 1
'9172' = 1
'9173' = 1
'9174' = 1
'9175' = 1
'9176' = 1
'9179' = 1
'9181' = 1
'9182' = 1
'9183' = 1
'9184' = 1
'9185' = 1
'9186' = 1
'9189' = 1
'9191' = 1
'9192' = 1
'9193' = 1
'9194' = 1
'9195' = 1
'9196' = 1
'9199' = 1
'9201' = 1
'9202' = 1
'9203' = 1
'9204' = 1
'9205' = 1
'9209' = 1
'9211' = 1
'9212' = 1
'9213' = 1
'9214' = 1
'9215' = 1
'9216' = 1
'9217' = 1
'9218' = 1
'9219' = 1
'9220' = 2
'9221' = 2
'9222' = 2
'9223' = 2
'9224' = 2
'9225' = 2
'9226' = 2
'9227' = 4 
'9228' = 2
'9229' = 2
'923' = 2
'9230' = 2
'9231' = 2
'9232' = 2
'9233' = 2
'9239' = 2
'9301' = 1
'9302' = 1
'9303' = 1
'9304' = 1
'9305' = 1
'9306' = 1
'9307' = 1
'9308' = 1
'9309' = 1
'9311' = 2
'9312' = 2
'9313' = 2
'9314' = 2
'9315' = 2
'9316' = 2
'9317' = 2
'9318' = 2
'9319' = 2
'9321' = 2
'9322' = 2
'9323' = 2
'9324' = 2
'9325' = 2
'9326' = 2
'9327' = 2
'9328' = 2
'9329' = 2
'9331' = 2
'9332' = 2
'9333' = 2
'9334' = 2
'9335' = 2
'9336' = 2
'9337' = 2
'9338' = 2
'9339' = 2
'9341' = 2
'9342' = 2
'9343' = 2
'9344' = 2
'9345' = 2
'9346' = 2
'9351' = 2
'9352' = 2
'9353' = 2
'9354' = 2
'9355' = 2
'9356' = 2
'9357' = 2
'9358' = 2
'9359' = 2
'9361' = 2
'9362' = 2
'9363' = 2
'9364' = 2
'9365' = 2
'9366' = 2
'9367' = 2
'9371' = 2
'9372' = 2
'9373' = 2
'9374' = 2
'9375' = 2
'9376' = 2
'9377' = 2
'9378' = 2
'9381' = 2
'9382' = 2
'9383' = 2
'9384' = 2
'9385' = 2
'9389' = 2
'9390' = 2
'9391' = 2
'9392' = 2
'9393' = 2
'9394' = 2
'9395' = 2
'9396' = 2
'9397' = 2
'9398' = 2
'9399' = 2
'9401' = 1
'9402' = 1
'9403' = 1
'9408' = 1
'9409' = 1
'9411' = 1
'9412' = 1
'9413' = 1
'9419' = 1
'9421' = 2
'9422' = 2
'9423' = 2
'9424' = 2
'9425' = 2
'9426' = 2
'9427' = 2
'9429' = 2
'9431' = 2
'9432' = 2
'9433' = 2
'9434' = 2
'9435' = 2
'9436' = 2
'9437' = 2
'9438' = 2
'9439' = 2
'9441' = 2
'9442' = 2
'9443' = 2
'9444' = 2
'9445' = 2
'9446' = 2
'9449' = 2
'9451' = 2
'9452' = 2
'9453' = 2
'9454' = 2
'9455' = 2
'9459' = 2
'9461' = 2
'9462' = 2
'9463' = 2
'9464' = 2
'9465' = 2
'9466' = 2
'9467' = 2
'9468' = 2
'9469' = 2
'9501' = 1
'9502' = 1
'9503' = 1
'9504' = 3
'9505' = 1
'9506' = 1
'9507' = 1
'9509' = 1
'9511' = 1
'9512' = 1
'9513' = 1
'9514' = 1
'9515' = 1
'9516' = 1
'9521' = 1
'9522' = 1
'9523' = 1
'9524' = 1
'9525' = 1
'9526' = 1
'9531' = 2
'9532' = 2
'9533' = 2
'9534' = 2
'9535' = 2
'9536' = 2
'9541' = 1
'9542' = 1
'9543' = 1
'9544' = 1
'9545' = 1
'9546' = 1
'9547' = 1
'9548' = 2
'9549' = 2
'9601' = 2
'9602' = 2
'9603' = 2
'9604' = 2
'9605' = 2
'9606' = 2
'9607' = 2
'9608' = 2
'9609' = 2
'9611' = 2
'9614' = 2
'9615' = 2
'9616' = 2
'9617' = 2
'9618' = 2
'9619' = 2
'9621' = 2
'9622' = 2
'9623' = 2
'9624' = 2
'9625' = 2
'9626' = 2
'9627' = 2
'9628' = 2
'9629' = 2
'9631' = 2
'9632' = 2
'9633' = 2
'9634' = 2
'9635' = 2
'9636' = 2
'9637' = 2
'9638' = 2
'9639' = 2
'9641' = 2
'9642' = 2
'9643' = 2
'9644' = 2
'9645' = 2
'9646' = 2
'9647' = 2
'9648' = 2
'9649' = 2
'9651' = 2
'9652' = 2
'9653' = 2
'9654' = 2
'9655' = 2
'9656' = 2
'9657' = 2
'9658' = 2
'9659' = 2
'966' = 2
'9670' = 2
'9671' = 2
'9672' = 2
'9701' = 2
'9702' = 2
'9703' = 2
'9704' = 2
'9705' = 2
'9711' = 2
'9712' = 2
'9713' = 2
'9714' = 2
'9715' = 2
'9716' = 2
'9721' = 2
'9722' = 2
'9723' = 2
'9724' = 2
'9725' = 2
'9726' = 2
'9729' = 2
'9731' = 2
'9732' = 2
'9733' = 2
'9734' = 2
'9735' = 2
'9736' = 2
'9737' = 2
'9738' = 2
'9739' = 2
'9741' = 2
'9742' = 2
'9743' = 2
'9744' = 2
'9749' = 2
'9751' = 2
'9752' = 2
'9753' = 2
'9754' = 2
'9755' = 2
'9756' = 2
'9759' = 2
'9761' = 2
'9762' = 2
'9763' = 2
'9764' = 2
'9765' = 2
'9769' = 2
'9771' = 2
'9772' = 2
'9773' = 2
'9774' = 2
'9775' = 2
'9779' = 2
'9781' = 2
'9782' = 2
'9783' = 2
'9784' = 2
'9785' = 2
'9786' = 2
'9787' = 2
'9788' = 2
'9789' = 2
'9801' = 2
'9802' = 2
'9803' = 2
'9804' = 2
'9805' = 2
'9811' = 2
'9812' = 2
'9813' = 2
'9814' = 2
'9815' = 2
'9816' = 2
'9817' = 2
'9818' = 2
'9819' = 2
'9820' = 2
'9821' = 2
'9822' = 2
'9823' = 2
'9824' = 2
'9825' = 2
'9826' = 2
'9827' = 2
'9828' = 2
'9829' = 2
'9851' = 2
'9852' = 2
'9859' = 2
'9900' = 2
'9901' = 2
'9902' = 2
'9903' = 2
'9904' = 2
'9905' = 2
'9906' = 2
'9907' = 2
'9908' = 2
'9909' = 2
'9910' = 2
'9911' = 2
'9912' = 2
'9913' = 2
'9914' = 2
'9915' = 2
'9916' = 2
'9917' = 2
'9918' = 2
'9919' = 2
'9920' = 2
'9921' = 2
'9922' = 2
'9923' = 2
'9924' = 2
'9925' = 2
'9926' = 2
'9927' = 2
'9928' = 2
'9929' = 2
'9931' = 2
'9932' = 2
'9933' = 2
'9934' = 2
'9935' = 2
'9936' = 2
'9937' = 2
'9938' = 2
'9939' = 2
'9941' = 2
'9942' = 2
'9943' = 2
'9944' = 2
'9945' = 2
'9946' = 2
'9947' = 2
'9948' = 2
'9951' = 2
'9952' = 2
'9953' = 2
'9954' = 2
'9955' = 2
'9956' = 2
'9957' = 2
'9958' = 2
'9959' = 2
'9960' = 2
'9961' = 2
'9962' = 2
'9963' = 2
'9964' = 2
'9969' = 2
'9971' = 2
'9972' = 2
'9973' = 2
'9974' = 2
'9975' = 2
'9976' = 2
'9977' = 2
'9978' = 2
'9979' = 2
'9981' = 2
'9982' = 2
'9983' = 2
'9984' = 2
'9985' = 2
'9986' = 2
'9988' = 2
'9991' = 2
'9992' = 2
'9993' = 2
'9994' = 2
'9995' = 2
'9996' = 2
'9997' = 2
'9998' = 2
'9999' = 2
OTHER  = 0;

 /* -------------------------------------------------------------- */
 /* ---   FORMAT FOR AGE STRATIFIERS                           --- */
 /* ---   MODULE 3. ASSIGNS ARRAY INDEX FOR AGE CATEGORIES     --- */
 /* -------------------------------------------------------------- */

VALUE AGEFMT
low - 4 = 1
 5 -  9 = 2
10 - 14 = 3
15 - 17 = 4
18 - 24 = 5
25 - 29 = 6
30 - 34 = 7
35 - 39 = 8
40 - 44 = 9
45 - 49 = 10
50 - 54 = 11
55 - 59 = 12
60 - 64 = 13
65 - 69 = 14
70 - 74 = 15
75 - 79 = 16
80 - 84 = 17
85 - high = 18;


VALUE $CCS
'B350' = '4' /* Tinea barbae and tinea capitis - Mycoses */
'B351' = '4' /* Tinea unguium - Mycoses */
'B352' = '4' /* Tinea manuum - Mycoses */
'B353' = '4' /* Tinea pedis - Mycoses */
'B354' = '4' /* Tinea corporis - Mycoses */
'B355' = '4' /* Tinea imbricata - Mycoses */
'B356' = '4' /* Tinea cruris - Mycoses */
'B358' = '4' /* Other dermatophytoses - Mycoses */
'B359' = '4' /* Dermatophytosis, unspecified - Mycoses */
'B360' = '4' /* Pityriasis versicolor - Mycoses */
'B361' = '4' /* Tinea nigra - Mycoses */
'B362' = '4' /* White piedra - Mycoses */
'B363' = '4' /* Black piedra - Mycoses */
'B368' = '4' /* Other specified superficial mycoses - Mycoses */
'B369' = '4' /* Superficial mycosis, unspecified - Mycoses */
'B370' = '4' /* Candidal stomatitis - Mycoses */
'B372' = '4' /* Candidiasis of skin and nail - Mycoses */
'B373' = '4' /* Candidiasis of vulva and vagina - Mycoses */
'B3741' = '4' /* Candidal cystitis and urethritis - Mycoses */
'B3742' = '4' /* Candidal balanitis - Mycoses */
'B3749' = '4' /* Other urogenital candidiasis - Mycoses */
'B3781' = '4' /* Candidal esophagitis - Mycoses */
'B3782' = '4' /* Candidal enteritis - Mycoses */
'B3783' = '4' /* Candidal cheilitis - Mycoses */
'B3784' = '4' /* Candidal otitis externa - Mycoses */
'B3789' = '4' /* Other sites of candidiasis - Mycoses */
'B379' = '4' /* Candidiasis, unspecified - Mycoses */
'B383' = '4' /* Cutaneous coccidioidomycosis - Mycoses */
'B387' = '4' /* Disseminated coccidioidomycosis - Mycoses */
'B3881' = '4' /* Prostatic coccidioidomycosis - Mycoses */
'B3889' = '4' /* Other forms of coccidioidomycosis - Mycoses */
'B389' = '4' /* Coccidioidomycosis, unspecified - Mycoses */
'B393' = '4' /* Disseminated histoplasmosis capsulati - Mycoses */
'B394' = '4' /* Histoplasmosis capsulati, unspecified - Mycoses */
'B395' = '4' /* Histoplasmosis duboisii - Mycoses */
'B399' = '4' /* Histoplasmosis, unspecified - Mycoses */
'B400' = '4' /* Acute pulmonary blastomycosis - Mycoses */
'B401' = '4' /* Chronic pulmonary blastomycosis - Mycoses */
'B402' = '4' /* Pulmonary blastomycosis, unspecified - Mycoses */
'B403' = '4' /* Cutaneous blastomycosis - Mycoses */
'B407' = '4' /* Disseminated blastomycosis - Mycoses */
'B4089' = '4' /* Other forms of blastomycosis - Mycoses */
'B409' = '4' /* Blastomycosis, unspecified - Mycoses */
'B410' = '4' /* Pulmonary paracoccidioidomycosis - Mycoses */
'B417' = '4' /* Disseminated paracoccidioidomycosis - Mycoses */
'B418' = '4' /* Other forms of paracoccidioidomycosis - Mycoses */
'B419' = '4' /* Paracoccidioidomycosis, unspecified - Mycoses */
'B420' = '4' /* Pulmonary sporotrichosis - Mycoses */
'B421' = '4' /* Lymphocutaneous sporotrichosis - Mycoses */
'B427' = '4' /* Disseminated sporotrichosis - Mycoses */
'B4281' = '4' /* Cerebral sporotrichosis - Mycoses */
'B4282' = '4' /* Sporotrichosis arthritis - Mycoses */
'B4289' = '4' /* Other forms of sporotrichosis - Mycoses */
'B429' = '4' /* Sporotrichosis, unspecified - Mycoses */
'B430' = '4' /* Cutaneous chromomycosis - Mycoses */
'B431' = '4' /* Pheomycotic brain abscess - Mycoses */
'B432' = '4' /* Subcutaneous pheomycotic abscess and cyst - Mycoses */
'B438' = '4' /* Other forms of chromomycosis - Mycoses */
'B439' = '4' /* Chromomycosis, unspecified - Mycoses */
'B440' = '4' /* Invasive pulmonary aspergillosis - Mycoses */
'B441' = '4' /* Other pulmonary aspergillosis - Mycoses */
'B442' = '4' /* Tonsillar aspergillosis - Mycoses */
'B447' = '4' /* Disseminated aspergillosis - Mycoses */
'B4489' = '4' /* Other forms of aspergillosis - Mycoses */
'B449' = '4' /* Aspergillosis, unspecified - Mycoses */
'B450' = '4' /* Pulmonary cryptococcosis - Mycoses */
'B452' = '4' /* Cutaneous cryptococcosis - Mycoses */
'B453' = '4' /* Osseous cryptococcosis - Mycoses */
'B457' = '4' /* Disseminated cryptococcosis - Mycoses */
'B458' = '4' /* Other forms of cryptococcosis - Mycoses */
'B459' = '4' /* Cryptococcosis, unspecified - Mycoses */
'B460' = '4' /* Pulmonary mucormycosis - Mycoses */
'B461' = '4' /* Rhinocerebral mucormycosis - Mycoses */
'B462' = '4' /* Gastrointestinal mucormycosis - Mycoses */
'B463' = '4' /* Cutaneous mucormycosis - Mycoses */
'B464' = '4' /* Disseminated mucormycosis - Mycoses */
'B465' = '4' /* Mucormycosis, unspecified - Mycoses */
'B468' = '4' /* Other zygomycoses - Mycoses */
'B469' = '4' /* Zygomycosis, unspecified - Mycoses */
'B470' = '4' /* Eumycetoma - Mycoses */
'B480' = '4' /* Lobomycosis - Mycoses */
'B481' = '4' /* Rhinosporidiosis - Mycoses */
'B482' = '4' /* Allescheriasis - Mycoses */
'B483' = '4' /* Geotrichosis - Mycoses */
'B484' = '4' /* Penicillosis - Mycoses */
'B488' = '4' /* Other specified mycoses - Mycoses */
'B49' = '4' /* Unspecified mycosis - Mycoses */
'B150' = '6' /* Hepatitis A with hepatic coma - Hepatitis */
'B159' = '6' /* Hepatitis A without hepatic coma - Hepatitis */
'B160' = '6' /* Acute hepatitis B with delta-agent with hepatic coma - Hepatitis */
'B161' = '6' /* Acute hepatitis B with delta-agent without hepatic coma - Hepatitis */
'B162' = '6' /* Acute hepatitis B without delta-agent with hepatic coma - Hepatitis */
'B169' = '6' /* Acute hepatitis B w/o delta-agent and without hepatic coma - Hepatitis */
'B170' = '6' /* Acute delta-(super) infection of hepatitis B carrier - Hepatitis */
'B1710' = '6' /* Acute hepatitis C without hepatic coma - Hepatitis */
'B1711' = '6' /* Acute hepatitis C with hepatic coma - Hepatitis */
'B172' = '6' /* Acute hepatitis E - Hepatitis */
'B178' = '6' /* Other specified acute viral hepatitis - Hepatitis */
'B179' = '6' /* Acute viral hepatitis, unspecified - Hepatitis */
'B180' = '6' /* Chronic viral hepatitis B with delta-agent - Hepatitis */
'B181' = '6' /* Chronic viral hepatitis B without delta-agent - Hepatitis */
'B182' = '6' /* Chronic viral hepatitis C - Hepatitis */
'B188' = '6' /* Other chronic viral hepatitis - Hepatitis */
'B189' = '6' /* Chronic viral hepatitis, unspecified - Hepatitis */
'B190' = '6' /* Unspecified viral hepatitis with hepatic coma - Hepatitis */
'B1910' = '6' /* Unspecified viral hepatitis B without hepatic coma - Hepatitis */
'B1911' = '6' /* Unspecified viral hepatitis B with hepatic coma - Hepatitis */
'B1920' = '6' /* Unspecified viral hepatitis C without hepatic coma - Hepatitis */
'B1921' = '6' /* Unspecified viral hepatitis C with hepatic coma - Hepatitis */
'B199' = '6' /* Unspecified viral hepatitis without hepatic coma - Hepatitis */
'B251' = '6' /* Cytomegaloviral hepatitis - Hepatitis */
'B2681' = '6' /* Mumps hepatitis - Hepatitis */
'B581' = '6' /* Toxoplasma hepatitis - Hepatitis */
'B942' = '6' /* Sequelae of viral hepatitis - Hepatitis */
'K710' = '6' /* Toxic liver disease with cholestasis - Hepatitis */
'K712' = '6' /* Toxic liver disease with acute hepatitis - Hepatitis */
'K713' = '6' /* Toxic liver disease with chronic persistent hepatitis - Hepatitis */
'K714' = '6' /* Toxic liver disease with chronic lobular hepatitis - Hepatitis */
'K7150' = '6' /* Toxic liver disease w chronic active hepatitis w/o ascites - Hepatitis */
'K7151' = '6' /* Toxic liver disease w chronic active hepatitis with ascites - Hepatitis */
'K716' = '6' /* Toxic liver disease with hepatitis, not elsewhere classified - Hepatitis */
'K717' = '6' /* Toxic liver disease with fibrosis and cirrhosis of liver - Hepatitis */
'K730' = '6' /* Chronic persistent hepatitis, not elsewhere classified - Hepatitis */
'K731' = '6' /* Chronic lobular hepatitis, not elsewhere classified - Hepatitis */
'K732' = '6' /* Chronic active hepatitis, not elsewhere classified - Hepatitis */
'K738' = '6' /* Other chronic hepatitis, not elsewhere classified - Hepatitis */
'K739' = '6' /* Chronic hepatitis, unspecified - Hepatitis */
'K752' = '6' /* Nonspecific reactive hepatitis - Hepatitis */
'K753' = '6' /* Granulomatous hepatitis, not elsewhere classified - Hepatitis */
'K754' = '6' /* Autoimmune hepatitis - Hepatitis */
'K7581' = '6' /* Nonalcoholic steatohepatitis (NASH) - Hepatitis */
'K764' = '6' /* Peliosis hepatis - Hepatitis */
'C4000' = '21' /* Malig neoplasm of scapula and long bones of unsp upper limb - Cancer of bone and connective tissue */
'C4001' = '21' /* Malig neoplasm of scapula and long bones of right upper limb - Cancer of bone and connective tissue */
'C4002' = '21' /* Malig neoplasm of scapula and long bones of left upper limb - Cancer of bone and connective tissue */
'C4010' = '21' /* Malignant neoplasm of short bones of unspecified upper limb - Cancer of bone and connective tissue */
'C4011' = '21' /* Malignant neoplasm of short bones of right upper limb - Cancer of bone and connective tissue */
'C4012' = '21' /* Malignant neoplasm of short bones of left upper limb - Cancer of bone and connective tissue */
'C4020' = '21' /* Malignant neoplasm of long bones of unspecified lower limb - Cancer of bone and connective tissue */
'C4021' = '21' /* Malignant neoplasm of long bones of right lower limb - Cancer of bone and connective tissue */
'C4022' = '21' /* Malignant neoplasm of long bones of left lower limb - Cancer of bone and connective tissue */
'C4030' = '21' /* Malignant neoplasm of short bones of unspecified lower limb - Cancer of bone and connective tissue */
'C4031' = '21' /* Malignant neoplasm of short bones of right lower limb - Cancer of bone and connective tissue */
'C4032' = '21' /* Malignant neoplasm of short bones of left lower limb - Cancer of bone and connective tissue */
'C4080' = '21' /* Malig neoplm of ovrlp sites of bone/artic cartl of unsp limb - Cancer of bone and connective tissue */
'C4081' = '21' /* Malig neoplm of ovrlp sites of bone/artic cartl of r limb - Cancer of bone and connective tissue */
'C4082' = '21' /* Malig neoplm of ovrlp sites of bone/artic cartl of left limb - Cancer of bone and connective tissue */
'C4090' = '21' /* Malig neoplasm of unsp bones and artic cartlg of unsp limb - Cancer of bone and connective tissue */
'C4091' = '21' /* Malig neoplasm of unsp bones and artic cartlg of right limb - Cancer of bone and connective tissue */
'C4092' = '21' /* Malig neoplasm of unsp bones and artic cartlg of left limb - Cancer of bone and connective tissue */
'C410' = '21' /* Malignant neoplasm of bones of skull and face - Cancer of bone and connective tissue */
'C411' = '21' /* Malignant neoplasm of mandible - Cancer of bone and connective tissue */
'C412' = '21' /* Malignant neoplasm of vertebral column - Cancer of bone and connective tissue */
'C413' = '21' /* Malignant neoplasm of ribs, sternum and clavicle - Cancer of bone and connective tissue */
'C414' = '21' /* Malignant neoplasm of pelvic bones, sacrum and coccyx - Cancer of bone and connective tissue */
'C419' = '21' /* Malignant neoplasm of bone and articular cartilage, unsp - Cancer of bone and connective tissue */
'C490' = '21' /* Malig neoplm of conn and soft tissue of head, face and neck - Cancer of bone and connective tissue */
'C4910' = '21' /* Malig neoplm of conn & soft tiss of unsp upr lmb, inc shldr - Cancer of bone and connective tissue */
'C4911' = '21' /* Malig neoplm of conn and soft tiss of r upr limb, inc shldr - Cancer of bone and connective tissue */
'C4912' = '21' /* Malig neoplm of conn and soft tiss of l upr limb, inc shldr - Cancer of bone and connective tissue */
'C4920' = '21' /* Malig neoplm of conn and soft tiss of unsp low limb, inc hip - Cancer of bone and connective tissue */
'C4921' = '21' /* Malig neoplm of conn and soft tiss of r low limb, inc hip - Cancer of bone and connective tissue */
'C4922' = '21' /* Malig neoplm of conn and soft tiss of left low limb, inc hip - Cancer of bone and connective tissue */
'C493' = '21' /* Malignant neoplasm of connective and soft tissue of thorax - Cancer of bone and connective tissue */
'C494' = '21' /* Malignant neoplasm of connective and soft tissue of abdomen - Cancer of bone and connective tissue */
'C495' = '21' /* Malignant neoplasm of connective and soft tissue of pelvis - Cancer of bone and connective tissue */
'C496' = '21' /* Malignant neoplasm of conn and soft tissue of trunk, unsp - Cancer of bone and connective tissue */
'C498' = '21' /* Malignant neoplasm of ovrlp sites of conn and soft tissue - Cancer of bone and connective tissue */
'C499' = '21' /* Malignant neoplasm of connective and soft tissue, unsp - Cancer of bone and connective tissue */
'C641' = '33' /* Malignant neoplasm of right kidney, except renal pelvis - Cancer of kidney and renal pelvis */
'C642' = '33' /* Malignant neoplasm of left kidney, except renal pelvis - Cancer of kidney and renal pelvis */
'C649' = '33' /* Malignant neoplasm of unsp kidney, except renal pelvis - Cancer of kidney and renal pelvis */
'C651' = '33' /* Malignant neoplasm of right renal pelvis - Cancer of kidney and renal pelvis */
'C652' = '33' /* Malignant neoplasm of left renal pelvis - Cancer of kidney and renal pelvis */
'C659' = '33' /* Malignant neoplasm of unspecified renal pelvis - Cancer of kidney and renal pelvis */
'C7A093' = '33' /* Malignant carcinoid tumor of the kidney - Cancer of kidney and renal pelvis */
'Z85528' = '33' /* Personal history of other malignant neoplasm of kidney - Cancer of kidney and renal pelvis */
'Z8553' = '33' /* Personal history of malignant neoplasm of renal pelvis - Cancer of kidney and renal pelvis */
'C470' = '35' /* Malignant neoplasm of prph nerves of head, face and neck - Cancer of brain and nervous system */
'C4710' = '35' /* Malig neoplm of prph nerves of unsp upper limb, inc shoulder - Cancer of brain and nervous system */
'C4711' = '35' /* Malig neoplm of prph nerves of right upper limb, inc shldr - Cancer of brain and nervous system */
'C4712' = '35' /* Malig neoplm of prph nerves of left upper limb, inc shoulder - Cancer of brain and nervous system */
'C4720' = '35' /* Malig neoplasm of prph nerves of unsp lower limb, inc hip - Cancer of brain and nervous system */
'C4721' = '35' /* Malig neoplasm of prph nerves of right lower limb, inc hip - Cancer of brain and nervous system */
'C4722' = '35' /* Malig neoplasm of prph nerves of left lower limb, inc hip - Cancer of brain and nervous system */
'C473' = '35' /* Malignant neoplasm of peripheral nerves of thorax - Cancer of brain and nervous system */
'C474' = '35' /* Malignant neoplasm of peripheral nerves of abdomen - Cancer of brain and nervous system */
'C475' = '35' /* Malignant neoplasm of peripheral nerves of pelvis - Cancer of brain and nervous system */
'C476' = '35' /* Malignant neoplasm of peripheral nerves of trunk, unsp - Cancer of brain and nervous system */
'C478' = '35' /* Malig neoplm of ovrlp sites of prph nrv and autonm nrv sys - Cancer of brain and nervous system */
'C479' = '35' /* Malig neoplasm of prph nerves and autonm nervous sys, unsp - Cancer of brain and nervous system */
'C700' = '35' /* Malignant neoplasm of cerebral meninges - Cancer of brain and nervous system */
'C701' = '35' /* Malignant neoplasm of spinal meninges - Cancer of brain and nervous system */
'C709' = '35' /* Malignant neoplasm of meninges, unspecified - Cancer of brain and nervous system */
'C710' = '35' /* Malignant neoplasm of cerebrum, except lobes and ventricles - Cancer of brain and nervous system */
'C711' = '35' /* Malignant neoplasm of frontal lobe - Cancer of brain and nervous system */
'C712' = '35' /* Malignant neoplasm of temporal lobe - Cancer of brain and nervous system */
'C713' = '35' /* Malignant neoplasm of parietal lobe - Cancer of brain and nervous system */
'C714' = '35' /* Malignant neoplasm of occipital lobe - Cancer of brain and nervous system */
'C715' = '35' /* Malignant neoplasm of cerebral ventricle - Cancer of brain and nervous system */
'C716' = '35' /* Malignant neoplasm of cerebellum - Cancer of brain and nervous system */
'C717' = '35' /* Malignant neoplasm of brain stem - Cancer of brain and nervous system */
'C718' = '35' /* Malignant neoplasm of overlapping sites of brain - Cancer of brain and nervous system */
'C719' = '35' /* Malignant neoplasm of brain, unspecified - Cancer of brain and nervous system */
'C720' = '35' /* Malignant neoplasm of spinal cord - Cancer of brain and nervous system */
'C721' = '35' /* Malignant neoplasm of cauda equina - Cancer of brain and nervous system */
'C7220' = '35' /* Malignant neoplasm of unspecified olfactory nerve - Cancer of brain and nervous system */
'C7221' = '35' /* Malignant neoplasm of right olfactory nerve - Cancer of brain and nervous system */
'C7222' = '35' /* Malignant neoplasm of left olfactory nerve - Cancer of brain and nervous system */
'C7230' = '35' /* Malignant neoplasm of unspecified optic nerve - Cancer of brain and nervous system */
'C7231' = '35' /* Malignant neoplasm of right optic nerve - Cancer of brain and nervous system */
'C7232' = '35' /* Malignant neoplasm of left optic nerve - Cancer of brain and nervous system */
'C7240' = '35' /* Malignant neoplasm of unspecified acoustic nerve - Cancer of brain and nervous system */
'C7241' = '35' /* Malignant neoplasm of right acoustic nerve - Cancer of brain and nervous system */
'C7242' = '35' /* Malignant neoplasm of left acoustic nerve - Cancer of brain and nervous system */
'C7250' = '35' /* Malignant neoplasm of unspecified cranial nerve - Cancer of brain and nervous system */
'C7259' = '35' /* Malignant neoplasm of other cranial nerves - Cancer of brain and nervous system */
'C729' = '35' /* Malignant neoplasm of central nervous system, unspecified - Cancer of brain and nervous system */
'Z85841' = '35' /* Personal history of malignant neoplasm of brain - Cancer of brain and nervous system */
'Z85848' = '35' /* Personal history of malignant neoplasm of prt nervous tissue - Cancer of brain and nervous system */
'C8200' = '38' /* Follicular lymphoma grade I, unspecified site - Non-Hodgkin`s lymphoma */
'C8201' = '38' /* Follicular lymphoma grade I, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8202' = '38' /* Follicular lymphoma grade I, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8203' = '38' /* Follicular lymphoma grade I, intra-abdominal lymph nodes - Non-Hodgkin`s lymphoma */
'C8204' = '38' /* Follicular lymphoma grade I, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8205' = '38' /* Foliclar lymph grade I, nodes of ing region and lower limb - Non-Hodgkin`s lymphoma */
'C8206' = '38' /* Follicular lymphoma grade I, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8207' = '38' /* Follicular lymphoma grade I, spleen - Non-Hodgkin`s lymphoma */
'C8208' = '38' /* Follicular lymphoma grade I, lymph nodes of multiple sites - Non-Hodgkin`s lymphoma */
'C8209' = '38' /* Follicular lymphoma grade I, extrnod and solid organ sites - Non-Hodgkin`s lymphoma */
'C8210' = '38' /* Follicular lymphoma grade II, unspecified site - Non-Hodgkin`s lymphoma */
'C8211' = '38' /* Follicular lymphoma grade II, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8212' = '38' /* Follicular lymphoma grade II, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8213' = '38' /* Follicular lymphoma grade II, intra-abdominal lymph nodes - Non-Hodgkin`s lymphoma */
'C8214' = '38' /* Follicular lymphoma grade II, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8215' = '38' /* Foliclar lymph grade II, nodes of ing region and lower limb - Non-Hodgkin`s lymphoma */
'C8216' = '38' /* Follicular lymphoma grade II, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8217' = '38' /* Follicular lymphoma grade II, spleen - Non-Hodgkin`s lymphoma */
'C8218' = '38' /* Follicular lymphoma grade II, lymph nodes of multiple sites - Non-Hodgkin`s lymphoma */
'C8219' = '38' /* Follicular lymphoma grade II, extrnod and solid organ sites - Non-Hodgkin`s lymphoma */
'C8220' = '38' /* Follicular lymphoma grade III, unspecified, unspecified site - Non-Hodgkin`s lymphoma */
'C8221' = '38' /* Foliclar lymph grade III, unsp, nodes of head, face, and nk - Non-Hodgkin`s lymphoma */
'C8222' = '38' /* Follicular lymphoma grade III, unsp, intrathorac lymph nodes - Non-Hodgkin`s lymphoma */
'C8223' = '38' /* Follicular lymphoma grade III, unsp, intra-abd lymph nodes - Non-Hodgkin`s lymphoma */
'C8224' = '38' /* Foliclar lymph grade III, unsp, nodes of axla and upper limb - Non-Hodgkin`s lymphoma */
'C8225' = '38' /* Foliclar lymph grade III, unsp, nodes of ing rgn and low lmb - Non-Hodgkin`s lymphoma */
'C8226' = '38' /* Follicular lymphoma grade III, unsp, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8227' = '38' /* Follicular lymphoma grade III, unspecified, spleen - Non-Hodgkin`s lymphoma */
'C8228' = '38' /* Follicular lymphoma grade III, unsp, lymph nodes mult site - Non-Hodgkin`s lymphoma */
'C8229' = '38' /* Foliclar lymph grade III, unsp, extrnod and solid org sites - Non-Hodgkin`s lymphoma */
'C8230' = '38' /* Follicular lymphoma grade IIIa, unspecified site - Non-Hodgkin`s lymphoma */
'C8231' = '38' /* Foliclar lymphoma grade IIIa, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8232' = '38' /* Follicular lymphoma grade IIIa, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8233' = '38' /* Follicular lymphoma grade IIIa, intra-abdominal lymph nodes - Non-Hodgkin`s lymphoma */
'C8234' = '38' /* Foliclar lymphoma grade IIIa, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8235' = '38' /* Foliclar lymph grade IIIa, nodes of ing rgn and lower limb - Non-Hodgkin`s lymphoma */
'C8236' = '38' /* Follicular lymphoma grade IIIa, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8237' = '38' /* Follicular lymphoma grade IIIa, spleen - Non-Hodgkin`s lymphoma */
'C8238' = '38' /* Follicular lymphoma grade IIIa, lymph nodes mult site - Non-Hodgkin`s lymphoma */
'C8239' = '38' /* Foliclar lymphoma grade IIIa, extrnod and solid organ sites - Non-Hodgkin`s lymphoma */
'C8240' = '38' /* Follicular lymphoma grade IIIb, unspecified site - Non-Hodgkin`s lymphoma */
'C8241' = '38' /* Foliclar lymphoma grade IIIb, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8242' = '38' /* Follicular lymphoma grade IIIb, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8243' = '38' /* Follicular lymphoma grade IIIb, intra-abdominal lymph nodes - Non-Hodgkin`s lymphoma */
'C8244' = '38' /* Foliclar lymphoma grade IIIb, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8245' = '38' /* Foliclar lymph grade IIIb, nodes of ing rgn and lower limb - Non-Hodgkin`s lymphoma */
'C8246' = '38' /* Follicular lymphoma grade IIIb, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8247' = '38' /* Follicular lymphoma grade IIIb, spleen - Non-Hodgkin`s lymphoma */
'C8248' = '38' /* Follicular lymphoma grade IIIb, lymph nodes mult site - Non-Hodgkin`s lymphoma */
'C8249' = '38' /* Foliclar lymphoma grade IIIb, extrnod and solid organ sites - Non-Hodgkin`s lymphoma */
'C8250' = '38' /* Diffuse follicle center lymphoma, unspecified site - Non-Hodgkin`s lymphoma */
'C8251' = '38' /* Diffuse folicl center lymph, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8252' = '38' /* Diffuse follicle center lymphoma, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8253' = '38' /* Diffuse follicle center lymphoma, intra-abd lymph nodes - Non-Hodgkin`s lymphoma */
'C8254' = '38' /* Diffuse folicl center lymph, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8255' = '38' /* Diffus folicl cntr lymph, nodes of ing region and lower limb - Non-Hodgkin`s lymphoma */
'C8256' = '38' /* Diffuse follicle center lymphoma, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8257' = '38' /* Diffuse follicle center lymphoma, spleen - Non-Hodgkin`s lymphoma */
'C8258' = '38' /* Diffuse follicle center lymphoma, lymph nodes mult site - Non-Hodgkin`s lymphoma */
'C8259' = '38' /* Diffuse folicl center lymph, extrnod and solid organ sites - Non-Hodgkin`s lymphoma */
'C8260' = '38' /* Cutaneous follicle center lymphoma, unspecified site - Non-Hodgkin`s lymphoma */
'C8261' = '38' /* Cutan folicl center lymphoma, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8262' = '38' /* Cutaneous follicle center lymphoma, intrathorac lymph nodes - Non-Hodgkin`s lymphoma */
'C8263' = '38' /* Cutaneous follicle center lymphoma, intra-abd lymph nodes - Non-Hodgkin`s lymphoma */
'C8264' = '38' /* Cutan folicl center lymphoma, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8265' = '38' /* Cutan folicl cntr lymph, nodes of ing region and lower limb - Non-Hodgkin`s lymphoma */
'C8266' = '38' /* Cutaneous follicle center lymphoma, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8267' = '38' /* Cutaneous follicle center lymphoma, spleen - Non-Hodgkin`s lymphoma */
'C8268' = '38' /* Cutaneous follicle center lymphoma, lymph nodes mult site - Non-Hodgkin`s lymphoma */
'C8269' = '38' /* Cutan folicl center lymphoma, extrnod and solid organ sites - Non-Hodgkin`s lymphoma */
'C8280' = '38' /* Other types of follicular lymphoma, unspecified site - Non-Hodgkin`s lymphoma */
'C8281' = '38' /* Oth types of foliclar lymph, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8282' = '38' /* Oth types of follicular lymphoma, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8283' = '38' /* Oth types of follicular lymphoma, intra-abd lymph nodes - Non-Hodgkin`s lymphoma */
'C8284' = '38' /* Oth types of foliclar lymph, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8285' = '38' /* Oth types of foliclar lymph, nodes of ing rgn and lower limb - Non-Hodgkin`s lymphoma */
'C8286' = '38' /* Other types of follicular lymphoma, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8287' = '38' /* Other types of follicular lymphoma, spleen - Non-Hodgkin`s lymphoma */
'C8288' = '38' /* Oth types of follicular lymphoma, lymph nodes mult site - Non-Hodgkin`s lymphoma */
'C8289' = '38' /* Oth types of foliclar lymph, extrnod and solid organ sites - Non-Hodgkin`s lymphoma */
'C8290' = '38' /* Follicular lymphoma, unspecified, unspecified site - Non-Hodgkin`s lymphoma */
'C8291' = '38' /* Follicular lymphoma, unsp, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8292' = '38' /* Follicular lymphoma, unspecified, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8293' = '38' /* Follicular lymphoma, unsp, intra-abdominal lymph nodes - Non-Hodgkin`s lymphoma */
'C8294' = '38' /* Follicular lymphoma, unsp, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8295' = '38' /* Foliclar lymphoma, unsp, nodes of ing region and lower limb - Non-Hodgkin`s lymphoma */
'C8296' = '38' /* Follicular lymphoma, unspecified, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8297' = '38' /* Follicular lymphoma, unspecified, spleen - Non-Hodgkin`s lymphoma */
'C8298' = '38' /* Follicular lymphoma, unsp, lymph nodes of multiple sites - Non-Hodgkin`s lymphoma */
'C8299' = '38' /* Follicular lymphoma, unsp, extranodal and solid organ sites - Non-Hodgkin`s lymphoma */
'C8300' = '38' /* Small cell B-cell lymphoma, unspecified site - Non-Hodgkin`s lymphoma */
'C8301' = '38' /* Small cell B-cell lymphoma, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8302' = '38' /* Small cell B-cell lymphoma, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8303' = '38' /* Small cell B-cell lymphoma, intra-abdominal lymph nodes - Non-Hodgkin`s lymphoma */
'C8304' = '38' /* Small cell B-cell lymphoma, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8305' = '38' /* Small cell B-cell lymph, nodes of ing region and lower limb - Non-Hodgkin`s lymphoma */
'C8306' = '38' /* Small cell B-cell lymphoma, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8307' = '38' /* Small cell B-cell lymphoma, spleen - Non-Hodgkin`s lymphoma */
'C8308' = '38' /* Small cell B-cell lymphoma, lymph nodes of multiple sites - Non-Hodgkin`s lymphoma */
'C8309' = '38' /* Small cell B-cell lymphoma, extranodal and solid organ sites - Non-Hodgkin`s lymphoma */
'C8310' = '38' /* Mantle cell lymphoma, unspecified site - Non-Hodgkin`s lymphoma */
'C8311' = '38' /* Mantle cell lymphoma, lymph nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8312' = '38' /* Mantle cell lymphoma, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8313' = '38' /* Mantle cell lymphoma, intra-abdominal lymph nodes - Non-Hodgkin`s lymphoma */
'C8314' = '38' /* Mantle cell lymphoma, lymph nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8315' = '38' /* Mantle cell lymphoma, nodes of ing region and lower limb - Non-Hodgkin`s lymphoma */
'C8316' = '38' /* Mantle cell lymphoma, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8317' = '38' /* Mantle cell lymphoma, spleen - Non-Hodgkin`s lymphoma */
'C8318' = '38' /* Mantle cell lymphoma, lymph nodes of multiple sites - Non-Hodgkin`s lymphoma */
'C8319' = '38' /* Mantle cell lymphoma, extranodal and solid organ sites - Non-Hodgkin`s lymphoma */
'C8330' = '38' /* Diffuse large B-cell lymphoma, unspecified site - Non-Hodgkin`s lymphoma */
'C8331' = '38' /* Diffuse large B-cell lymphoma, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8332' = '38' /* Diffuse large B-cell lymphoma, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8333' = '38' /* Diffuse large B-cell lymphoma, intra-abdominal lymph nodes - Non-Hodgkin`s lymphoma */
'C8334' = '38' /* Diffuse large B-cell lymph, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8335' = '38' /* Diffus large B-cell lymph, nodes of ing rgn and lower limb - Non-Hodgkin`s lymphoma */
'C8336' = '38' /* Diffuse large B-cell lymphoma, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8337' = '38' /* Diffuse large B-cell lymphoma, spleen - Non-Hodgkin`s lymphoma */
'C8338' = '38' /* Diffuse large B-cell lymphoma, lymph nodes of multiple sites - Non-Hodgkin`s lymphoma */
'C8339' = '38' /* Diffuse large B-cell lymphoma, extrnod and solid organ sites - Non-Hodgkin`s lymphoma */
'C8350' = '38' /* Lymphoblastic (diffuse) lymphoma, unspecified site - Non-Hodgkin`s lymphoma */
'C8351' = '38' /* Lymphoblastic lymphoma, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8352' = '38' /* Lymphoblastic (diffuse) lymphoma, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8353' = '38' /* Lymphoblastic (diffuse) lymphoma, intra-abd lymph nodes - Non-Hodgkin`s lymphoma */
'C8354' = '38' /* Lymphoblastic lymphoma, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8355' = '38' /* Lymphoblastic lymphoma, nodes of ing region and lower limb - Non-Hodgkin`s lymphoma */
'C8356' = '38' /* Lymphoblastic (diffuse) lymphoma, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8357' = '38' /* Lymphoblastic (diffuse) lymphoma, spleen - Non-Hodgkin`s lymphoma */
'C8358' = '38' /* Lymphoblastic (diffuse) lymphoma, lymph nodes mult site - Non-Hodgkin`s lymphoma */
'C8359' = '38' /* Lymphoblastic lymphoma, extrnod and solid organ sites - Non-Hodgkin`s lymphoma */
'C8370' = '38' /* Burkitt lymphoma, unspecified site - Non-Hodgkin`s lymphoma */
'C8371' = '38' /* Burkitt lymphoma, lymph nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8372' = '38' /* Burkitt lymphoma, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8373' = '38' /* Burkitt lymphoma, intra-abdominal lymph nodes - Non-Hodgkin`s lymphoma */
'C8374' = '38' /* Burkitt lymphoma, lymph nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8375' = '38' /* Burkitt lymphoma, nodes of inguinal region and lower limb - Non-Hodgkin`s lymphoma */
'C8376' = '38' /* Burkitt lymphoma, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8377' = '38' /* Burkitt lymphoma, spleen - Non-Hodgkin`s lymphoma */
'C8378' = '38' /* Burkitt lymphoma, lymph nodes of multiple sites - Non-Hodgkin`s lymphoma */
'C8379' = '38' /* Burkitt lymphoma, extranodal and solid organ sites - Non-Hodgkin`s lymphoma */
'C8380' = '38' /* Other non-follicular lymphoma, unspecified site - Non-Hodgkin`s lymphoma */
'C8381' = '38' /* Oth non-follic lymphoma, lymph nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8382' = '38' /* Other non-follicular lymphoma, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8383' = '38' /* Other non-follicular lymphoma, intra-abdominal lymph nodes - Non-Hodgkin`s lymphoma */
'C8384' = '38' /* Oth non-follic lymphoma, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8385' = '38' /* Oth non-follic lymphoma, nodes of ing region and lower limb - Non-Hodgkin`s lymphoma */
'C8386' = '38' /* Other non-follicular lymphoma, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8387' = '38' /* Other non-follicular lymphoma, spleen - Non-Hodgkin`s lymphoma */
'C8388' = '38' /* Other non-follicular lymphoma, lymph nodes of multiple sites - Non-Hodgkin`s lymphoma */
'C8389' = '38' /* Oth non-follic lymphoma, extranodal and solid organ sites - Non-Hodgkin`s lymphoma */
'C8390' = '38' /* Non-follicular (diffuse) lymphoma, unsp, unspecified site - Non-Hodgkin`s lymphoma */
'C8391' = '38' /* Non-follic lymphoma, unsp, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8392' = '38' /* Non-follic (diffuse) lymphoma, unsp, intrathorac lymph nodes - Non-Hodgkin`s lymphoma */
'C8393' = '38' /* Non-follic (diffuse) lymphoma, unsp, intra-abd lymph nodes - Non-Hodgkin`s lymphoma */
'C8394' = '38' /* Non-follic lymphoma, unsp, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8395' = '38' /* Non-follic lymph, unsp, nodes of ing region and lower limb - Non-Hodgkin`s lymphoma */
'C8396' = '38' /* Non-follic (diffuse) lymphoma, unsp, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8397' = '38' /* Non-follicular (diffuse) lymphoma, unspecified, spleen - Non-Hodgkin`s lymphoma */
'C8398' = '38' /* Non-follic (diffuse) lymphoma, unsp, lymph nodes mult site - Non-Hodgkin`s lymphoma */
'C8399' = '38' /* Non-follic lymphoma, unsp, extrnod and solid organ sites - Non-Hodgkin`s lymphoma */
'C8400' = '38' /* Mycosis fungoides, unspecified site - Non-Hodgkin`s lymphoma */
'C8401' = '38' /* Mycosis fungoides, lymph nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8402' = '38' /* Mycosis fungoides, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8403' = '38' /* Mycosis fungoides, intra-abdominal lymph nodes - Non-Hodgkin`s lymphoma */
'C8404' = '38' /* Mycosis fungoides, lymph nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8405' = '38' /* Mycosis fungoides, nodes of inguinal region and lower limb - Non-Hodgkin`s lymphoma */
'C8406' = '38' /* Mycosis fungoides, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8407' = '38' /* Mycosis fungoides, spleen - Non-Hodgkin`s lymphoma */
'C8408' = '38' /* Mycosis fungoides, lymph nodes of multiple sites - Non-Hodgkin`s lymphoma */
'C8409' = '38' /* Mycosis fungoides, extranodal and solid organ sites - Non-Hodgkin`s lymphoma */
'C8410' = '38' /* Sezary disease, unspecified site - Non-Hodgkin`s lymphoma */
'C8411' = '38' /* Sezary disease, lymph nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8412' = '38' /* Sezary disease, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8413' = '38' /* Sezary disease, intra-abdominal lymph nodes - Non-Hodgkin`s lymphoma */
'C8414' = '38' /* Sezary disease, lymph nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8415' = '38' /* Sezary disease, nodes of inguinal region and lower limb - Non-Hodgkin`s lymphoma */
'C8416' = '38' /* Sezary disease, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8417' = '38' /* Sezary disease, spleen - Non-Hodgkin`s lymphoma */
'C8418' = '38' /* Sezary disease, lymph nodes of multiple sites - Non-Hodgkin`s lymphoma */
'C8419' = '38' /* Sezary disease, extranodal and solid organ sites - Non-Hodgkin`s lymphoma */
'C8440' = '38' /* Peripheral T-cell lymphoma, not classified, unspecified site - Non-Hodgkin`s lymphoma */
'C8441' = '38' /* Prph T-cell lymph, not class, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8442' = '38' /* Peripheral T-cell lymphoma, not class, intrathorac nodes - Non-Hodgkin`s lymphoma */
'C8443' = '38' /* Peripheral T-cell lymphoma, not classified, intra-abd nodes - Non-Hodgkin`s lymphoma */
'C8444' = '38' /* Prph T-cell lymph, not class, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8445' = '38' /* Prph T-cell lymph, not class, nodes of ing rgn and low limb - Non-Hodgkin`s lymphoma */
'C8446' = '38' /* Peripheral T-cell lymphoma, not classified, intrapelv nodes - Non-Hodgkin`s lymphoma */
'C8447' = '38' /* Peripheral T-cell lymphoma, not classified, spleen - Non-Hodgkin`s lymphoma */
'C8448' = '38' /* Peripheral T-cell lymphoma, not classified, nodes mult site - Non-Hodgkin`s lymphoma */
'C8449' = '38' /* Prph T-cell lymph, not class, extrnod and solid organ sites - Non-Hodgkin`s lymphoma */
'C8460' = '38' /* Anaplastic large cell lymphoma, ALK-positive, unsp site - Non-Hodgkin`s lymphoma */
'C8461' = '38' /* Anaplstc lg cell lymph, ALK-pos, nodes of head, face, and nk - Non-Hodgkin`s lymphoma */
'C8462' = '38' /* Anaplastic large cell lymphoma, ALK-pos, intrathorac nodes - Non-Hodgkin`s lymphoma */
'C8463' = '38' /* Anaplastic large cell lymphoma, ALK-pos, intra-abd nodes - Non-Hodgkin`s lymphoma */
'C8464' = '38' /* Anaplstc lg cell lymph, ALK-pos, nodes of axla and upr limb - Non-Hodgkin`s lymphoma */
'C8465' = '38' /* Anaplstc lg cell lymph, ALK-pos, nodes of ing rgn & low lmb - Non-Hodgkin`s lymphoma */
'C8466' = '38' /* Anaplastic large cell lymphoma, ALK-pos, intrapelv nodes - Non-Hodgkin`s lymphoma */
'C8467' = '38' /* Anaplastic large cell lymphoma, ALK-positive, spleen - Non-Hodgkin`s lymphoma */
'C8468' = '38' /* Anaplastic large cell lymphoma, ALK-pos, nodes mult site - Non-Hodgkin`s lymphoma */
'C8469' = '38' /* Anaplstc lg cell lymph, ALK-pos, extrnod and solid org sites - Non-Hodgkin`s lymphoma */
'C8470' = '38' /* Anaplastic large cell lymphoma, ALK-negative, unsp site - Non-Hodgkin`s lymphoma */
'C8471' = '38' /* Anaplstc lg cell lymph, ALK-neg, nodes of head, face, and nk - Non-Hodgkin`s lymphoma */
'C8472' = '38' /* Anaplastic large cell lymphoma, ALK-neg, intrathorac nodes - Non-Hodgkin`s lymphoma */
'C8473' = '38' /* Anaplastic large cell lymphoma, ALK-neg, intra-abd nodes - Non-Hodgkin`s lymphoma */
'C8474' = '38' /* Anaplstc lg cell lymph, ALK-neg, nodes of axla and upr limb - Non-Hodgkin`s lymphoma */
'C8475' = '38' /* Anaplstc lg cell lymph, ALK-neg, nodes of ing rgn & low lmb - Non-Hodgkin`s lymphoma */
'C8476' = '38' /* Anaplastic large cell lymphoma, ALK-neg, intrapelv nodes - Non-Hodgkin`s lymphoma */
'C8477' = '38' /* Anaplastic large cell lymphoma, ALK-negative, spleen - Non-Hodgkin`s lymphoma */
'C8478' = '38' /* Anaplastic large cell lymphoma, ALK-neg, nodes mult site - Non-Hodgkin`s lymphoma */
'C8479' = '38' /* Anaplstc lg cell lymph, ALK-neg, extrnod and solid org sites - Non-Hodgkin`s lymphoma */
'C8490' = '38' /* Mature T/NK-cell lymphomas, unspecified, unspecified site - Non-Hodgkin`s lymphoma */
'C8491' = '38' /* Mature T/NK-cell lymph, unsp, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8492' = '38' /* Mature T/NK-cell lymphomas, unsp, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8493' = '38' /* Mature T/NK-cell lymphomas, unsp, intra-abd lymph nodes - Non-Hodgkin`s lymphoma */
'C8494' = '38' /* Mature T/NK-cell lymph, unsp, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8495' = '38' /* Mature T/NK-cell lymph, unsp, nodes of ing rgn and low limb - Non-Hodgkin`s lymphoma */
'C8496' = '38' /* Mature T/NK-cell lymphomas, unsp, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8497' = '38' /* Mature T/NK-cell lymphomas, unspecified, spleen - Non-Hodgkin`s lymphoma */
'C8498' = '38' /* Mature T/NK-cell lymphomas, unsp, lymph nodes mult site - Non-Hodgkin`s lymphoma */
'C8499' = '38' /* Mature T/NK-cell lymph, unsp, extrnod and solid organ sites - Non-Hodgkin`s lymphoma */
'C84A0' = '38' /* Cutaneous T-cell lymphoma, unspecified, unspecified site - Non-Hodgkin`s lymphoma */
'C84A1' = '38' /* Cutan T-cell lymphoma, unsp nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C84A2' = '38' /* Cutaneous T-cell lymphoma, unsp, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C84A3' = '38' /* Cutaneous T-cell lymphoma, unsp, intra-abdominal lymph nodes - Non-Hodgkin`s lymphoma */
'C84A4' = '38' /* Cutan T-cell lymphoma, unsp, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C84A5' = '38' /* Cutan T-cell lymph, unsp, nodes of ing region and lower limb - Non-Hodgkin`s lymphoma */
'C84A6' = '38' /* Cutaneous T-cell lymphoma, unsp, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C84A7' = '38' /* Cutaneous T-cell lymphoma, unspecified, spleen - Non-Hodgkin`s lymphoma */
'C84A8' = '38' /* Cutaneous T-cell lymphoma, unsp, lymph nodes mult site - Non-Hodgkin`s lymphoma */
'C84A9' = '38' /* Cutan T-cell lymphoma, unsp, extrnod and solid organ sites - Non-Hodgkin`s lymphoma */
'C84Z0' = '38' /* Other mature T/NK-cell lymphomas, unspecified site - Non-Hodgkin`s lymphoma */
'C84Z1' = '38' /* Oth mature T/NK-cell lymph, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C84Z2' = '38' /* Other mature T/NK-cell lymphomas, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C84Z3' = '38' /* Oth mature T/NK-cell lymphomas, intra-abdominal lymph nodes - Non-Hodgkin`s lymphoma */
'C84Z4' = '38' /* Oth mature T/NK-cell lymph, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C84Z5' = '38' /* Oth mature T/NK-cell lymph, nodes of ing rgn and lower limb - Non-Hodgkin`s lymphoma */
'C84Z6' = '38' /* Other mature T/NK-cell lymphomas, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C84Z7' = '38' /* Other mature T/NK-cell lymphomas, spleen - Non-Hodgkin`s lymphoma */
'C84Z8' = '38' /* Oth mature T/NK-cell lymphomas, lymph nodes mult site - Non-Hodgkin`s lymphoma */
'C84Z9' = '38' /* Oth mature T/NK-cell lymph, extrnod and solid organ sites - Non-Hodgkin`s lymphoma */
'C8510' = '38' /* Unspecified B-cell lymphoma, unspecified site - Non-Hodgkin`s lymphoma */
'C8511' = '38' /* Unsp B-cell lymphoma, lymph nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8512' = '38' /* Unspecified B-cell lymphoma, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8513' = '38' /* Unspecified B-cell lymphoma, intra-abdominal lymph nodes - Non-Hodgkin`s lymphoma */
'C8514' = '38' /* Unsp B-cell lymphoma, lymph nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8515' = '38' /* Unsp B-cell lymphoma, nodes of ing region and lower limb - Non-Hodgkin`s lymphoma */
'C8516' = '38' /* Unspecified B-cell lymphoma, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8517' = '38' /* Unspecified B-cell lymphoma, spleen - Non-Hodgkin`s lymphoma */
'C8518' = '38' /* Unspecified B-cell lymphoma, lymph nodes of multiple sites - Non-Hodgkin`s lymphoma */
'C8519' = '38' /* Unsp B-cell lymphoma, extranodal and solid organ sites - Non-Hodgkin`s lymphoma */
'C8520' = '38' /* Mediastinal (thymic) large B-cell lymphoma, unspecified site - Non-Hodgkin`s lymphoma */
'C8521' = '38' /* Mediastnl large B-cell lymph, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8522' = '38' /* Mediastnl (thymic) large B-cell lymphoma, intrathorac nodes - Non-Hodgkin`s lymphoma */
'C8523' = '38' /* Mediastinal (thymic) large B-cell lymphoma, intra-abd nodes - Non-Hodgkin`s lymphoma */
'C8524' = '38' /* Mediastnl large B-cell lymph, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8525' = '38' /* Mediastnl lg B-cell lymph, nodes of ing rgn and lower limb - Non-Hodgkin`s lymphoma */
'C8526' = '38' /* Mediastinal (thymic) large B-cell lymphoma, intrapelv nodes - Non-Hodgkin`s lymphoma */
'C8527' = '38' /* Mediastinal (thymic) large B-cell lymphoma, spleen - Non-Hodgkin`s lymphoma */
'C8528' = '38' /* Mediastinal (thymic) large B-cell lymphoma, nodes mult site - Non-Hodgkin`s lymphoma */
'C8529' = '38' /* Mediastnl large B-cell lymph, extrnod and solid organ sites - Non-Hodgkin`s lymphoma */
'C8580' = '38' /* Oth types of non-Hodgkin lymphoma, unspecified site - Non-Hodgkin`s lymphoma */
'C8581' = '38' /* Oth types of non-hodg lymph, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8582' = '38' /* Oth types of non-Hodgkin lymphoma, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8583' = '38' /* Oth types of non-Hodgkin lymphoma, intra-abd lymph nodes - Non-Hodgkin`s lymphoma */
'C8584' = '38' /* Oth types of non-hodg lymph, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8585' = '38' /* Oth types of non-hodg lymph, nodes of ing rgn and lower limb - Non-Hodgkin`s lymphoma */
'C8586' = '38' /* Oth types of non-Hodgkin lymphoma, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8587' = '38' /* Other specified types of non-Hodgkin lymphoma, spleen - Non-Hodgkin`s lymphoma */
'C8588' = '38' /* Oth types of non-Hodgkin lymphoma, lymph nodes mult site - Non-Hodgkin`s lymphoma */
'C8589' = '38' /* Oth types of non-hodg lymph, extrnod and solid organ sites - Non-Hodgkin`s lymphoma */
'C8590' = '38' /* Non-Hodgkin lymphoma, unspecified, unspecified site - Non-Hodgkin`s lymphoma */
'C8591' = '38' /* Non-Hodgkin lymphoma, unsp, nodes of head, face, and neck - Non-Hodgkin`s lymphoma */
'C8592' = '38' /* Non-Hodgkin lymphoma, unspecified, intrathoracic lymph nodes - Non-Hodgkin`s lymphoma */
'C8593' = '38' /* Non-Hodgkin lymphoma, unsp, intra-abdominal lymph nodes - Non-Hodgkin`s lymphoma */
'C8594' = '38' /* Non-Hodgkin lymphoma, unsp, nodes of axilla and upper limb - Non-Hodgkin`s lymphoma */
'C8595' = '38' /* Non-hodg lymphoma, unsp, nodes of ing region and lower limb - Non-Hodgkin`s lymphoma */
'C8596' = '38' /* Non-Hodgkin lymphoma, unspecified, intrapelvic lymph nodes - Non-Hodgkin`s lymphoma */
'C8597' = '38' /* Non-Hodgkin lymphoma, unspecified, spleen - Non-Hodgkin`s lymphoma */
'C8598' = '38' /* Non-Hodgkin lymphoma, unsp, lymph nodes of multiple sites - Non-Hodgkin`s lymphoma */
'C8599' = '38' /* Non-Hodgkin lymphoma, unsp, extranodal and solid organ sites - Non-Hodgkin`s lymphoma */
'C860' = '38' /* Extranodal NK/T-cell lymphoma, nasal type - Non-Hodgkin`s lymphoma */
'C861' = '38' /* Hepatosplenic T-cell lymphoma - Non-Hodgkin`s lymphoma */
'C862' = '38' /* Enteropathy-type (intestinal) T-cell lymphoma - Non-Hodgkin`s lymphoma */
'C863' = '38' /* Subcutaneous panniculitis-like T-cell lymphoma - Non-Hodgkin`s lymphoma */
'C864' = '38' /* Blastic NK-cell lymphoma - Non-Hodgkin`s lymphoma */
'C865' = '38' /* Angioimmunoblastic T-cell lymphoma - Non-Hodgkin`s lymphoma */
'C866' = '38' /* Primary cutaneous CD30-positive T-cell proliferations - Non-Hodgkin`s lymphoma */
'C880' = '38' /* Waldenstrom macroglobulinemia - Non-Hodgkin`s lymphoma */
'C882' = '38' /* Heavy chain disease - Non-Hodgkin`s lymphoma */
'C883' = '38' /* Immunoproliferative small intestinal disease - Non-Hodgkin`s lymphoma */
'C884' = '38' /* Extrnod mrgnl zn B-cell lymph of mucosa-assoc lymphoid tiss - Non-Hodgkin`s lymphoma */
'C888' = '38' /* Other malignant immunoproliferative diseases - Non-Hodgkin`s lymphoma */
'C889' = '38' /* Malignant immunoproliferative disease, unspecified - Non-Hodgkin`s lymphoma */
'C964' = '38' /* Sarcoma of dendritic cells (accessory cells) - Non-Hodgkin`s lymphoma */
'C969' = '38' /* Malig neoplm of lymphoid, hematpoetc and rel tissue, unsp - Non-Hodgkin`s lymphoma */
'C96Z' = '38' /* Oth malig neoplm of lymphoid, hematpoetc and related tissue - Non-Hodgkin`s lymphoma */
'Z8572' = '38' /* Personal history of non-Hodgkin lymphomas - Non-Hodgkin`s lymphoma */
'Z8579' = '38' /* Prsnl hx of malig neoplm of lymphoid, hematpoetc & rel tiss - Non-Hodgkin`s lymphoma */
'C9010' = '39' /* Plasma cell leukemia not having achieved remission - Leukemias */
'C9011' = '39' /* Plasma cell leukemia in remission - Leukemias */
'C9012' = '39' /* Plasma cell leukemia in relapse - Leukemias */
'C9100' = '39' /* Acute lymphoblastic leukemia not having achieved remission - Leukemias */
'C9101' = '39' /* Acute lymphoblastic leukemia, in remission - Leukemias */
'C9102' = '39' /* Acute lymphoblastic leukemia, in relapse - Leukemias */
'C9110' = '39' /* Chronic lymphocytic leuk of B-cell type not achieve remis - Leukemias */
'C9111' = '39' /* Chronic lymphocytic leukemia of B-cell type in remission - Leukemias */
'C9112' = '39' /* Chronic lymphocytic leukemia of B-cell type in relapse - Leukemias */
'C9130' = '39' /* Prolymphocytic leukemia of B-cell type not achieve remission - Leukemias */
'C9131' = '39' /* Prolymphocytic leukemia of B-cell type, in remission - Leukemias */
'C9132' = '39' /* Prolymphocytic leukemia of B-cell type, in relapse - Leukemias */
'C9140' = '39' /* Hairy cell leukemia not having achieved remission - Leukemias */
'C9141' = '39' /* Hairy cell leukemia, in remission - Leukemias */
'C9142' = '39' /* Hairy cell leukemia, in relapse - Leukemias */
'C9150' = '39' /* Adult T-cell lymph/leuk (HTLV-1-assoc) not achieve remission - Leukemias */
'C9151' = '39' /* Adult T-cell lymphoma/leukemia (HTLV-1-assoc), in remission - Leukemias */
'C9152' = '39' /* Adult T-cell lymphoma/leukemia (HTLV-1-assoc), in relapse - Leukemias */
'C9160' = '39' /* Prolymphocytic leukemia of T-cell type not achieve remission - Leukemias */
'C9161' = '39' /* Prolymphocytic leukemia of T-cell type, in remission - Leukemias */
'C9162' = '39' /* Prolymphocytic leukemia of T-cell type, in relapse - Leukemias */
'C9190' = '39' /* Lymphoid leukemia, unspecified not having achieved remission - Leukemias */
'C9191' = '39' /* Lymphoid leukemia, unspecified, in remission - Leukemias */
'C9192' = '39' /* Lymphoid leukemia, unspecified, in relapse - Leukemias */
'C91A0' = '39' /* Mature B-cell leukemia Burkitt-type not achieve remission - Leukemias */
'C91A1' = '39' /* Mature B-cell leukemia Burkitt-type, in remission - Leukemias */
'C91A2' = '39' /* Mature B-cell leukemia Burkitt-type, in relapse - Leukemias */
'C91Z0' = '39' /* Other lymphoid leukemia not having achieved remission - Leukemias */
'C91Z1' = '39' /* Other lymphoid leukemia, in remission - Leukemias */
'C91Z2' = '39' /* Other lymphoid leukemia, in relapse - Leukemias */
'C9200' = '39' /* Acute myeloblastic leukemia, not having achieved remission - Leukemias */
'C9201' = '39' /* Acute myeloblastic leukemia, in remission - Leukemias */
'C9202' = '39' /* Acute myeloblastic leukemia, in relapse - Leukemias */
'C9210' = '39' /* Chronic myeloid leuk, BCR/ABL-positive, not achieve remis - Leukemias */
'C9211' = '39' /* Chronic myeloid leukemia, BCR/ABL-positive, in remission - Leukemias */
'C9212' = '39' /* Chronic myeloid leukemia, BCR/ABL-positive, in relapse - Leukemias */
'C9220' = '39' /* Atyp chronic myeloid leuk, BCR/ABL-neg, not achieve remis - Leukemias */
'C9221' = '39' /* Atypical chronic myeloid leukemia, BCR/ABL-neg, in remission - Leukemias */
'C9222' = '39' /* Atypical chronic myeloid leukemia, BCR/ABL-neg, in relapse - Leukemias */
'C9230' = '39' /* Myeloid sarcoma, not having achieved remission - Leukemias */
'C9231' = '39' /* Myeloid sarcoma, in remission - Leukemias */
'C9232' = '39' /* Myeloid sarcoma, in relapse - Leukemias */
'C9240' = '39' /* Acute promyelocytic leukemia, not having achieved remission - Leukemias */
'C9241' = '39' /* Acute promyelocytic leukemia, in remission - Leukemias */
'C9242' = '39' /* Acute promyelocytic leukemia, in relapse - Leukemias */
'C9250' = '39' /* Acute myelomonocytic leukemia, not having achieved remission - Leukemias */
'C9251' = '39' /* Acute myelomonocytic leukemia, in remission - Leukemias */
'C9252' = '39' /* Acute myelomonocytic leukemia, in relapse - Leukemias */
'C9260' = '39' /* Acute myeloid leukemia w 11q23-abnormality not achieve remis - Leukemias */
'C9261' = '39' /* Acute myeloid leukemia with 11q23-abnormality in remission - Leukemias */
'C9262' = '39' /* Acute myeloid leukemia with 11q23-abnormality in relapse - Leukemias */
'C9290' = '39' /* Myeloid leukemia, unspecified, not having achieved remission - Leukemias */
'C9291' = '39' /* Myeloid leukemia, unspecified in remission - Leukemias */
'C9292' = '39' /* Myeloid leukemia, unspecified in relapse - Leukemias */
'C92A0' = '39' /* Acute myeloid leuk w multilin dysplasia, not achieve remis - Leukemias */
'C92A1' = '39' /* Acute myeloid leukemia w multilin dysplasia, in remission - Leukemias */
'C92A2' = '39' /* Acute myeloid leukemia w multilineage dysplasia, in relapse - Leukemias */
'C92Z0' = '39' /* Other myeloid leukemia not having achieved remission - Leukemias */
'C92Z1' = '39' /* Other myeloid leukemia, in remission - Leukemias */
'C92Z2' = '39' /* Other myeloid leukemia, in relapse - Leukemias */
'C9300' = '39' /* Acute monoblastic/monocytic leukemia, not achieve remission - Leukemias */
'C9301' = '39' /* Acute monoblastic/monocytic leukemia, in remission - Leukemias */
'C9302' = '39' /* Acute monoblastic/monocytic leukemia, in relapse - Leukemias */
'C9310' = '39' /* Chronic myelomonocytic leukemia not achieve remission - Leukemias */
'C9311' = '39' /* Chronic myelomonocytic leukemia, in remission - Leukemias */
'C9312' = '39' /* Chronic myelomonocytic leukemia, in relapse - Leukemias */
'C9330' = '39' /* Juvenile myelomonocytic leukemia, not achieve remission - Leukemias */
'C9331' = '39' /* Juvenile myelomonocytic leukemia, in remission - Leukemias */
'C9332' = '39' /* Juvenile myelomonocytic leukemia, in relapse - Leukemias */
'C9390' = '39' /* Monocytic leukemia, unsp, not having achieved remission - Leukemias */
'C9391' = '39' /* Monocytic leukemia, unspecified in remission - Leukemias */
'C9392' = '39' /* Monocytic leukemia, unspecified in relapse - Leukemias */
'C93Z0' = '39' /* Other monocytic leukemia, not having achieved remission - Leukemias */
'C93Z1' = '39' /* Other monocytic leukemia, in remission - Leukemias */
'C93Z2' = '39' /* Other monocytic leukemia, in relapse - Leukemias */
'C9400' = '39' /* Acute erythroid leukemia, not having achieved remission - Leukemias */
'C9401' = '39' /* Acute erythroid leukemia, in remission - Leukemias */
'C9402' = '39' /* Acute erythroid leukemia, in relapse - Leukemias */
'C9420' = '39' /* Acute megakaryoblastic leukemia not achieve remission - Leukemias */
'C9421' = '39' /* Acute megakaryoblastic leukemia, in remission - Leukemias */
'C9422' = '39' /* Acute megakaryoblastic leukemia, in relapse - Leukemias */
'C9430' = '39' /* Mast cell leukemia not having achieved remission - Leukemias */
'C9431' = '39' /* Mast cell leukemia, in remission - Leukemias */
'C9432' = '39' /* Mast cell leukemia, in relapse - Leukemias */
'C9440' = '39' /* Acute panmyelosis w myelofibrosis not achieve remission - Leukemias */
'C9441' = '39' /* Acute panmyelosis with myelofibrosis, in remission - Leukemias */
'C9442' = '39' /* Acute panmyelosis with myelofibrosis, in relapse - Leukemias */
'C946' = '39' /* Myelodysplastic disease, not classified - Leukemias */
'C9480' = '39' /* Other specified leukemias not having achieved remission - Leukemias */
'C9481' = '39' /* Other specified leukemias, in remission - Leukemias */
'C9482' = '39' /* Other specified leukemias, in relapse - Leukemias */
'C9500' = '39' /* Acute leukemia of unsp cell type not achieve remission - Leukemias */
'C9501' = '39' /* Acute leukemia of unspecified cell type, in remission - Leukemias */
'C9502' = '39' /* Acute leukemia of unspecified cell type, in relapse - Leukemias */
'C9510' = '39' /* Chronic leukemia of unsp cell type not achieve remission - Leukemias */
'C9511' = '39' /* Chronic leukemia of unspecified cell type, in remission - Leukemias */
'C9512' = '39' /* Chronic leukemia of unspecified cell type, in relapse - Leukemias */
'C9590' = '39' /* Leukemia, unspecified not having achieved remission - Leukemias */
'C9591' = '39' /* Leukemia, unspecified, in remission - Leukemias */
'C9592' = '39' /* Leukemia, unspecified, in relapse - Leukemias */
'Z856' = '39' /* Personal history of leukemia - Leukemias */
'C37' = '41' /* Malignant neoplasm of thymus - Cancer; other and unspecified primary */
'C380' = '41' /* Malignant neoplasm of heart - Cancer; other and unspecified primary */
'C381' = '41' /* Malignant neoplasm of anterior mediastinum - Cancer; other and unspecified primary */
'C382' = '41' /* Malignant neoplasm of posterior mediastinum - Cancer; other and unspecified primary */
'C383' = '41' /* Malignant neoplasm of mediastinum, part unspecified - Cancer; other and unspecified primary */
'C388' = '41' /* Malig neoplm of ovrlp sites of heart, mediastinum and pleura - Cancer; other and unspecified primary */
'C452' = '41' /* Mesothelioma of pericardium - Cancer; other and unspecified primary */
'C457' = '41' /* Mesothelioma of other sites - Cancer; other and unspecified primary */
'C460' = '41' /* Kaposi's sarcoma of skin - Cancer; other and unspecified primary */
'C461' = '41' /* Kaposi's sarcoma of soft tissue - Cancer; other and unspecified primary */
'C462' = '41' /* Kaposi's sarcoma of palate - Cancer; other and unspecified primary */
'C463' = '41' /* Kaposi's sarcoma of lymph nodes - Cancer; other and unspecified primary */
'C464' = '41' /* Kaposi's sarcoma of gastrointestinal sites - Cancer; other and unspecified primary */
'C4650' = '41' /* Kaposi's sarcoma of unspecified lung - Cancer; other and unspecified primary */
'C4651' = '41' /* Kaposi's sarcoma of right lung - Cancer; other and unspecified primary */
'C4652' = '41' /* Kaposi's sarcoma of left lung - Cancer; other and unspecified primary */
'C467' = '41' /* Kaposi's sarcoma of other sites - Cancer; other and unspecified primary */
'C469' = '41' /* Kaposi's sarcoma, unspecified - Cancer; other and unspecified primary */
'C6900' = '41' /* Malignant neoplasm of unspecified conjunctiva - Cancer; other and unspecified primary */
'C6901' = '41' /* Malignant neoplasm of right conjunctiva - Cancer; other and unspecified primary */
'C6902' = '41' /* Malignant neoplasm of left conjunctiva - Cancer; other and unspecified primary */
'C6910' = '41' /* Malignant neoplasm of unspecified cornea - Cancer; other and unspecified primary */
'C6911' = '41' /* Malignant neoplasm of right cornea - Cancer; other and unspecified primary */
'C6912' = '41' /* Malignant neoplasm of left cornea - Cancer; other and unspecified primary */
'C6920' = '41' /* Malignant neoplasm of unspecified retina - Cancer; other and unspecified primary */
'C6921' = '41' /* Malignant neoplasm of right retina - Cancer; other and unspecified primary */
'C6922' = '41' /* Malignant neoplasm of left retina - Cancer; other and unspecified primary */
'C6930' = '41' /* Malignant neoplasm of unspecified choroid - Cancer; other and unspecified primary */
'C6931' = '41' /* Malignant neoplasm of right choroid - Cancer; other and unspecified primary */
'C6932' = '41' /* Malignant neoplasm of left choroid - Cancer; other and unspecified primary */
'C6940' = '41' /* Malignant neoplasm of unspecified ciliary body - Cancer; other and unspecified primary */
'C6941' = '41' /* Malignant neoplasm of right ciliary body - Cancer; other and unspecified primary */
'C6942' = '41' /* Malignant neoplasm of left ciliary body - Cancer; other and unspecified primary */
'C6950' = '41' /* Malignant neoplasm of unspecified lacrimal gland and duct - Cancer; other and unspecified primary */
'C6951' = '41' /* Malignant neoplasm of right lacrimal gland and duct - Cancer; other and unspecified primary */
'C6952' = '41' /* Malignant neoplasm of left lacrimal gland and duct - Cancer; other and unspecified primary */
'C6960' = '41' /* Malignant neoplasm of unspecified orbit - Cancer; other and unspecified primary */
'C6961' = '41' /* Malignant neoplasm of right orbit - Cancer; other and unspecified primary */
'C6962' = '41' /* Malignant neoplasm of left orbit - Cancer; other and unspecified primary */
'C6980' = '41' /* Malignant neoplasm of ovrlp sites of unsp eye and adnexa - Cancer; other and unspecified primary */
'C6981' = '41' /* Malignant neoplasm of ovrlp sites of right eye and adnexa - Cancer; other and unspecified primary */
'C6982' = '41' /* Malignant neoplasm of ovrlp sites of left eye and adnexa - Cancer; other and unspecified primary */
'C6990' = '41' /* Malignant neoplasm of unspecified site of unspecified eye - Cancer; other and unspecified primary */
'C6991' = '41' /* Malignant neoplasm of unspecified site of right eye - Cancer; other and unspecified primary */
'C6992' = '41' /* Malignant neoplasm of unspecified site of left eye - Cancer; other and unspecified primary */
'C7400' = '41' /* Malignant neoplasm of cortex of unspecified adrenal gland - Cancer; other and unspecified primary */
'C7401' = '41' /* Malignant neoplasm of cortex of right adrenal gland - Cancer; other and unspecified primary */
'C7402' = '41' /* Malignant neoplasm of cortex of left adrenal gland - Cancer; other and unspecified primary */
'C7410' = '41' /* Malignant neoplasm of medulla of unspecified adrenal gland - Cancer; other and unspecified primary */
'C7411' = '41' /* Malignant neoplasm of medulla of right adrenal gland - Cancer; other and unspecified primary */
'C7412' = '41' /* Malignant neoplasm of medulla of left adrenal gland - Cancer; other and unspecified primary */
'C7490' = '41' /* Malignant neoplasm of unsp part of unspecified adrenal gland - Cancer; other and unspecified primary */
'C7491' = '41' /* Malignant neoplasm of unsp part of right adrenal gland - Cancer; other and unspecified primary */
'C7492' = '41' /* Malignant neoplasm of unspecified part of left adrenal gland - Cancer; other and unspecified primary */
'C750' = '41' /* Malignant neoplasm of parathyroid gland - Cancer; other and unspecified primary */
'C751' = '41' /* Malignant neoplasm of pituitary gland - Cancer; other and unspecified primary */
'C752' = '41' /* Malignant neoplasm of craniopharyngeal duct - Cancer; other and unspecified primary */
'C753' = '41' /* Malignant neoplasm of pineal gland - Cancer; other and unspecified primary */
'C754' = '41' /* Malignant neoplasm of carotid body - Cancer; other and unspecified primary */
'C755' = '41' /* Malignant neoplasm of aortic body and other paraganglia - Cancer; other and unspecified primary */
'C758' = '41' /* Malignant neoplasm with pluriglandular involvement, unsp - Cancer; other and unspecified primary */
'C759' = '41' /* Malignant neoplasm of endocrine gland, unspecified - Cancer; other and unspecified primary */
'C761' = '41' /* Malignant neoplasm of thorax - Cancer; other and unspecified primary */
'C762' = '41' /* Malignant neoplasm of abdomen - Cancer; other and unspecified primary */
'C763' = '41' /* Malignant neoplasm of pelvis - Cancer; other and unspecified primary */
'C7640' = '41' /* Malignant neoplasm of unspecified upper limb - Cancer; other and unspecified primary */
'C7641' = '41' /* Malignant neoplasm of right upper limb - Cancer; other and unspecified primary */
'C7642' = '41' /* Malignant neoplasm of left upper limb - Cancer; other and unspecified primary */
'C7650' = '41' /* Malignant neoplasm of unspecified lower limb - Cancer; other and unspecified primary */
'C7651' = '41' /* Malignant neoplasm of right lower limb - Cancer; other and unspecified primary */
'C7652' = '41' /* Malignant neoplasm of left lower limb - Cancer; other and unspecified primary */
'C768' = '41' /* Malignant neoplasm of other specified ill-defined sites - Cancer; other and unspecified primary */
'C7A091' = '41' /* Malignant carcinoid tumor of the thymus - Cancer; other and unspecified primary */
'C7A094' = '41' /* Malignant carcinoid tumor of the foregut NOS - Cancer; other and unspecified primary */
'C7A095' = '41' /* Malignant carcinoid tumor of the midgut NOS - Cancer; other and unspecified primary */
'C7A096' = '41' /* Malignant carcinoid tumor of the hindgut NOS - Cancer; other and unspecified primary */
'C960' = '41' /* Multifocal and multisystemic Langerhans-cell histiocytosis - Cancer; other and unspecified primary */
'C962' = '41' /* Malignant mast cell tumor - Cancer; other and unspecified primary */
'C965' = '41' /* Multifocal and unisystemic Langerhans-cell histiocytosis - Cancer; other and unspecified primary */
'C966' = '41' /* Unifocal Langerhans-cell histiocytosis - Cancer; other and unspecified primary */
'C96A' = '41' /* Histiocytic sarcoma - Cancer; other and unspecified primary */
'D0920' = '41' /* Carcinoma in situ of unspecified eye - Cancer; other and unspecified primary */
'D0921' = '41' /* Carcinoma in situ of right eye - Cancer; other and unspecified primary */
'D0922' = '41' /* Carcinoma in situ of left eye - Cancer; other and unspecified primary */
'D098' = '41' /* Carcinoma in situ of other specified sites - Cancer; other and unspecified primary */
'D099' = '41' /* Carcinoma in situ, unspecified - Cancer; other and unspecified primary */
'Z85020' = '41' /* Personal history of malignant carcinoid tumor of stomach - Cancer; other and unspecified primary */
'Z85030' = '41' /* Personal history of malignant carcinoid tumor of lg int - Cancer; other and unspecified primary */
'Z85040' = '41' /* Personal history of malignant carcinoid tumor of rectum - Cancer; other and unspecified primary */
'Z85060' = '41' /* Personal history of malignant carcinoid tumor of sm int - Cancer; other and unspecified primary */
'Z85110' = '41' /* Personal history of malig carcinoid tumor of bronc and lung - Cancer; other and unspecified primary */
'Z85230' = '41' /* Personal history of malignant carcinoid tumor of thymus - Cancer; other and unspecified primary */
'Z85238' = '41' /* Personal history of other malignant neoplasm of thymus - Cancer; other and unspecified primary */
'Z8529' = '41' /* Prsnl history of malig neoplm of resp and intrathorac organs - Cancer; other and unspecified primary */
'Z85520' = '41' /* Personal history of malignant carcinoid tumor of kidney - Cancer; other and unspecified primary */
'Z85821' = '41' /* Personal history of Merkel cell carcinoma - Cancer; other and unspecified primary */
'Z85830' = '41' /* Personal history of malignant neoplasm of bone - Cancer; other and unspecified primary */
'Z85831' = '41' /* Personal history of malignant neoplasm of soft tissue - Cancer; other and unspecified primary */
'Z85840' = '41' /* Personal history of malignant neoplasm of eye - Cancer; other and unspecified primary */
'Z85858' = '41' /* Personal history of malignant neoplasm of endocrine glands - Cancer; other and unspecified primary */
'Z8589' = '41' /* Personal history of malignant neoplasm of organs and systems - Cancer; other and unspecified primary */
'Z859' = '41' /* Personal history of malignant neoplasm, unspecified - Cancer; other and unspecified primary */
'Z86008' = '41' /* Personal history of in-situ neoplasm of other site - Cancer; other and unspecified primary */
'Z86012' = '41' /* Personal history of benign carcinoid tumor - Cancer; other and unspecified primary */
'Z86018' = '41' /* Personal history of other benign neoplasm - Cancer; other and unspecified primary */
'Z8603' = '41' /* Personal history of neoplasm of uncertain behavior - Cancer; other and unspecified primary */
'C770' = '42' /* Sec and unsp malig neoplasm of nodes of head, face and neck - Secondary malignancies */
'C771' = '42' /* Secondary and unsp malignant neoplasm of intrathorac nodes - Secondary malignancies */
'C772' = '42' /* Secondary and unsp malignant neoplasm of intra-abd nodes - Secondary malignancies */
'C773' = '42' /* Sec and unsp malig neoplasm of axilla and upper limb nodes - Secondary malignancies */
'C774' = '42' /* Sec and unsp malig neoplasm of inguinal and lower limb nodes - Secondary malignancies */
'C775' = '42' /* Secondary and unsp malignant neoplasm of intrapelv nodes - Secondary malignancies */
'C778' = '42' /* Sec and unsp malig neoplasm of nodes of multiple regions - Secondary malignancies */
'C779' = '42' /* Secondary and unsp malignant neoplasm of lymph node, unsp - Secondary malignancies */
'C7800' = '42' /* Secondary malignant neoplasm of unspecified lung - Secondary malignancies */
'C7801' = '42' /* Secondary malignant neoplasm of right lung - Secondary malignancies */
'C7802' = '42' /* Secondary malignant neoplasm of left lung - Secondary malignancies */
'C781' = '42' /* Secondary malignant neoplasm of mediastinum - Secondary malignancies */
'C782' = '42' /* Secondary malignant neoplasm of pleura - Secondary malignancies */
'C7830' = '42' /* Secondary malignant neoplasm of unsp respiratory organ - Secondary malignancies */
'C7839' = '42' /* Secondary malignant neoplasm of other respiratory organs - Secondary malignancies */
'C784' = '42' /* Secondary malignant neoplasm of small intestine - Secondary malignancies */
'C785' = '42' /* Secondary malignant neoplasm of large intestine and rectum - Secondary malignancies */
'C786' = '42' /* Secondary malignant neoplasm of retroperiton and peritoneum - Secondary malignancies */
'C787' = '42' /* Secondary malig neoplasm of liver and intrahepatic bile duct - Secondary malignancies */
'C7880' = '42' /* Secondary malignant neoplasm of unspecified digestive organ - Secondary malignancies */
'C7889' = '42' /* Secondary malignant neoplasm of other digestive organs - Secondary malignancies */
'C7900' = '42' /* Secondary malignant neoplasm of unsp kidney and renal pelvis - Secondary malignancies */
'C7901' = '42' /* Secondary malignant neoplasm of r kidney and renal pelvis - Secondary malignancies */
'C7902' = '42' /* Secondary malignant neoplasm of left kidney and renal pelvis - Secondary malignancies */
'C7910' = '42' /* Secondary malignant neoplasm of unspecified urinary organs - Secondary malignancies */
'C7911' = '42' /* Secondary malignant neoplasm of bladder - Secondary malignancies */
'C7919' = '42' /* Secondary malignant neoplasm of other urinary organs - Secondary malignancies */
'C792' = '42' /* Secondary malignant neoplasm of skin - Secondary malignancies */
'C7931' = '42' /* Secondary malignant neoplasm of brain - Secondary malignancies */
'C7932' = '42' /* Secondary malignant neoplasm of cerebral meninges - Secondary malignancies */
'C7940' = '42' /* Secondary malignant neoplasm of unsp part of nervous system - Secondary malignancies */
'C7949' = '42' /* Secondary malignant neoplasm of oth parts of nervous system - Secondary malignancies */
'C7951' = '42' /* Secondary malignant neoplasm of bone - Secondary malignancies */
'C7952' = '42' /* Secondary malignant neoplasm of bone marrow - Secondary malignancies */
'C7960' = '42' /* Secondary malignant neoplasm of unspecified ovary - Secondary malignancies */
'C7961' = '42' /* Secondary malignant neoplasm of right ovary - Secondary malignancies */
'C7962' = '42' /* Secondary malignant neoplasm of left ovary - Secondary malignancies */
'C7970' = '42' /* Secondary malignant neoplasm of unspecified adrenal gland - Secondary malignancies */
'C7971' = '42' /* Secondary malignant neoplasm of right adrenal gland - Secondary malignancies */
'C7972' = '42' /* Secondary malignant neoplasm of left adrenal gland - Secondary malignancies */
'C7981' = '42' /* Secondary malignant neoplasm of breast - Secondary malignancies */
'C7982' = '42' /* Secondary malignant neoplasm of genital organs - Secondary malignancies */
'C7989' = '42' /* Secondary malignant neoplasm of other specified sites - Secondary malignancies */
'C799' = '42' /* Secondary malignant neoplasm of unspecified site - Secondary malignancies */
'C7B01' = '42' /* Secondary carcinoid tumors of distant lymph nodes - Secondary malignancies */
'C7B02' = '42' /* Secondary carcinoid tumors of liver - Secondary malignancies */
'C7B03' = '42' /* Secondary carcinoid tumors of bone - Secondary malignancies */
'C7B04' = '42' /* Secondary carcinoid tumors of peritoneum - Secondary malignancies */
'J910' = '42' /* Malignant pleural effusion - Secondary malignancies */
'R180' = '42' /* Malignant ascites - Secondary malignancies */
'E000' = '48' /* Congenital iodine-deficiency syndrome, neurological type - Thyroid disorders */
'E001' = '48' /* Congenital iodine-deficiency syndrome, myxedematous type - Thyroid disorders */
'E002' = '48' /* Congenital iodine-deficiency syndrome, mixed type - Thyroid disorders */
'E009' = '48' /* Congenital iodine-deficiency syndrome, unspecified - Thyroid disorders */
'E010' = '48' /* Iodine-deficiency related diffuse (endemic) goiter - Thyroid disorders */
'E011' = '48' /* Iodine-deficiency related multinodular (endemic) goiter - Thyroid disorders */
'E012' = '48' /* Iodine-deficiency related (endemic) goiter, unspecified - Thyroid disorders */
'E018' = '48' /* Oth iodine-deficiency related thyroid disord and allied cond - Thyroid disorders */
'E02' = '48' /* Subclinical iodine-deficiency hypothyroidism - Thyroid disorders */
'E030' = '48' /* Congenital hypothyroidism with diffuse goiter - Thyroid disorders */
'E031' = '48' /* Congenital hypothyroidism without goiter - Thyroid disorders */
'E032' = '48' /* Hypothyroidism due to meds and oth exogenous substances - Thyroid disorders */
'E033' = '48' /* Postinfectious hypothyroidism - Thyroid disorders */
'E034' = '48' /* Atrophy of thyroid (acquired) - Thyroid disorders */
'E035' = '48' /* Myxedema coma - Thyroid disorders */
'E038' = '48' /* Other specified hypothyroidism - Thyroid disorders */
'E039' = '48' /* Hypothyroidism, unspecified - Thyroid disorders */
'E040' = '48' /* Nontoxic diffuse goiter - Thyroid disorders */
'E041' = '48' /* Nontoxic single thyroid nodule - Thyroid disorders */
'E042' = '48' /* Nontoxic multinodular goiter - Thyroid disorders */
'E048' = '48' /* Other specified nontoxic goiter - Thyroid disorders */
'E049' = '48' /* Nontoxic goiter, unspecified - Thyroid disorders */
'E0500' = '48' /* Thyrotoxicosis w diffuse goiter w/o thyrotoxic crisis - Thyroid disorders */
'E0501' = '48' /* Thyrotoxicosis w diffuse goiter w thyrotoxic crisis or storm - Thyroid disorders */
'E0510' = '48' /* Thyrotxcosis w toxic sing thyroid nodule w/o thyrotxc crisis - Thyroid disorders */
'E0511' = '48' /* Thyrotxcosis w toxic single thyroid nodule w thyrotxc crisis - Thyroid disorders */
'E0520' = '48' /* Thyrotxcosis w toxic multinod goiter w/o thyrotoxic crisis - Thyroid disorders */
'E0521' = '48' /* Thyrotxcosis w toxic multinodular goiter w thyrotoxic crisis - Thyroid disorders */
'E0530' = '48' /* Thyrotxcosis from ectopic thyroid tissue w/o thyrotxc crisis - Thyroid disorders */
'E0531' = '48' /* Thyrotxcosis from ectopic thyroid tissue w thyrotoxic crisis - Thyroid disorders */
'E0540' = '48' /* Thyrotoxicosis factitia without thyrotoxic crisis or storm - Thyroid disorders */
'E0541' = '48' /* Thyrotoxicosis factitia with thyrotoxic crisis or storm - Thyroid disorders */
'E0580' = '48' /* Other thyrotoxicosis without thyrotoxic crisis or storm - Thyroid disorders */
'E0581' = '48' /* Other thyrotoxicosis with thyrotoxic crisis or storm - Thyroid disorders */
'E0590' = '48' /* Thyrotoxicosis, unsp without thyrotoxic crisis or storm - Thyroid disorders */
'E0591' = '48' /* Thyrotoxicosis, unspecified with thyrotoxic crisis or storm - Thyroid disorders */
'E060' = '48' /* Acute thyroiditis - Thyroid disorders */
'E061' = '48' /* Subacute thyroiditis - Thyroid disorders */
'E062' = '48' /* Chronic thyroiditis with transient thyrotoxicosis - Thyroid disorders */
'E063' = '48' /* Autoimmune thyroiditis - Thyroid disorders */
'E064' = '48' /* Drug-induced thyroiditis - Thyroid disorders */
'E065' = '48' /* Other chronic thyroiditis - Thyroid disorders */
'E069' = '48' /* Thyroiditis, unspecified - Thyroid disorders */
'E070' = '48' /* Hypersecretion of calcitonin - Thyroid disorders */
'E071' = '48' /* Dyshormogenetic goiter - Thyroid disorders */
'E0781' = '48' /* Sick-euthyroid syndrome - Thyroid disorders */
'E0789' = '48' /* Other specified disorders of thyroid - Thyroid disorders */
'E079' = '48' /* Disorder of thyroid, unspecified - Thyroid disorders */
'E15' = '51' /* Nondiabetic hypoglycemic coma - Other endocrine disorders */
'E160' = '51' /* Drug-induced hypoglycemia without coma - Other endocrine disorders */
'E161' = '51' /* Other hypoglycemia - Other endocrine disorders */
'E162' = '51' /* Hypoglycemia, unspecified - Other endocrine disorders */
'E163' = '51' /* Increased secretion of glucagon - Other endocrine disorders */
'E164' = '51' /* Increased secretion of gastrin - Other endocrine disorders */
'E168' = '51' /* Other specified disorders of pancreatic internal secretion - Other endocrine disorders */
'E169' = '51' /* Disorder of pancreatic internal secretion, unspecified - Other endocrine disorders */
'E200' = '51' /* Idiopathic hypoparathyroidism - Other endocrine disorders */
'E208' = '51' /* Other hypoparathyroidism - Other endocrine disorders */
'E209' = '51' /* Hypoparathyroidism, unspecified - Other endocrine disorders */
'E210' = '51' /* Primary hyperparathyroidism - Other endocrine disorders */
'E211' = '51' /* Secondary hyperparathyroidism, not elsewhere classified - Other endocrine disorders */
'E212' = '51' /* Other hyperparathyroidism - Other endocrine disorders */
'E213' = '51' /* Hyperparathyroidism, unspecified - Other endocrine disorders */
'E214' = '51' /* Other specified disorders of parathyroid gland - Other endocrine disorders */
'E215' = '51' /* Disorder of parathyroid gland, unspecified - Other endocrine disorders */
'E220' = '51' /* Acromegaly and pituitary gigantism - Other endocrine disorders */
'E221' = '51' /* Hyperprolactinemia - Other endocrine disorders */
'E222' = '51' /* Syndrome of inappropriate secretion of antidiuretic hormone - Other endocrine disorders */
'E228' = '51' /* Other hyperfunction of pituitary gland - Other endocrine disorders */
'E229' = '51' /* Hyperfunction of pituitary gland, unspecified - Other endocrine disorders */
'E230' = '51' /* Hypopituitarism - Other endocrine disorders */
'E231' = '51' /* Drug-induced hypopituitarism - Other endocrine disorders */
'E232' = '51' /* Diabetes insipidus - Other endocrine disorders */
'E233' = '51' /* Hypothalamic dysfunction, not elsewhere classified - Other endocrine disorders */
'E236' = '51' /* Other disorders of pituitary gland - Other endocrine disorders */
'E237' = '51' /* Disorder of pituitary gland, unspecified - Other endocrine disorders */
'E240' = '51' /* Pituitary-dependent Cushing's disease - Other endocrine disorders */
'E241' = '51' /* Nelson's syndrome - Other endocrine disorders */
'E242' = '51' /* Drug-induced Cushing's syndrome - Other endocrine disorders */
'E243' = '51' /* Ectopic ACTH syndrome - Other endocrine disorders */
'E244' = '51' /* Alcohol-induced pseudo-Cushing's syndrome - Other endocrine disorders */
'E248' = '51' /* Other Cushing's syndrome - Other endocrine disorders */
'E249' = '51' /* Cushing's syndrome, unspecified - Other endocrine disorders */
'E250' = '51' /* Congenital adrenogenital disorders assoc w enzyme deficiency - Other endocrine disorders */
'E258' = '51' /* Other adrenogenital disorders - Other endocrine disorders */
'E259' = '51' /* Adrenogenital disorder, unspecified - Other endocrine disorders */
'E2601' = '51' /* Conn's syndrome - Other endocrine disorders */
'E2602' = '51' /* Glucocorticoid-remediable aldosteronism - Other endocrine disorders */
'E2609' = '51' /* Other primary hyperaldosteronism - Other endocrine disorders */
'E261' = '51' /* Secondary hyperaldosteronism - Other endocrine disorders */
'E2681' = '51' /* Bartter's syndrome - Other endocrine disorders */
'E2689' = '51' /* Other hyperaldosteronism - Other endocrine disorders */
'E269' = '51' /* Hyperaldosteronism, unspecified - Other endocrine disorders */
'E270' = '51' /* Other adrenocortical overactivity - Other endocrine disorders */
'E271' = '51' /* Primary adrenocortical insufficiency - Other endocrine disorders */
'E272' = '51' /* Addisonian crisis - Other endocrine disorders */
'E273' = '51' /* Drug-induced adrenocortical insufficiency - Other endocrine disorders */
'E2740' = '51' /* Unspecified adrenocortical insufficiency - Other endocrine disorders */
'E2749' = '51' /* Other adrenocortical insufficiency - Other endocrine disorders */
'E275' = '51' /* Adrenomedullary hyperfunction - Other endocrine disorders */
'E278' = '51' /* Other specified disorders of adrenal gland - Other endocrine disorders */
'E279' = '51' /* Disorder of adrenal gland, unspecified - Other endocrine disorders */
'E280' = '51' /* Estrogen excess - Other endocrine disorders */
'E281' = '51' /* Androgen excess - Other endocrine disorders */
'E282' = '51' /* Polycystic ovarian syndrome - Other endocrine disorders */
'E288' = '51' /* Other ovarian dysfunction - Other endocrine disorders */
'E289' = '51' /* Ovarian dysfunction, unspecified - Other endocrine disorders */
'E290' = '51' /* Testicular hyperfunction - Other endocrine disorders */
'E291' = '51' /* Testicular hypofunction - Other endocrine disorders */
'E298' = '51' /* Other testicular dysfunction - Other endocrine disorders */
'E299' = '51' /* Testicular dysfunction, unspecified - Other endocrine disorders */
'E300' = '51' /* Delayed puberty - Other endocrine disorders */
'E301' = '51' /* Precocious puberty - Other endocrine disorders */
'E308' = '51' /* Other disorders of puberty - Other endocrine disorders */
'E309' = '51' /* Disorder of puberty, unspecified - Other endocrine disorders */
'E310' = '51' /* Autoimmune polyglandular failure - Other endocrine disorders */
'E311' = '51' /* Polyglandular hyperfunction - Other endocrine disorders */
'E3120' = '51' /* Multiple endocrine neoplasia [MEN] syndrome, unspecified - Other endocrine disorders */
'E3121' = '51' /* Multiple endocrine neoplasia [MEN] type I - Other endocrine disorders */
'E318' = '51' /* Other polyglandular dysfunction - Other endocrine disorders */
'E319' = '51' /* Polyglandular dysfunction, unspecified - Other endocrine disorders */
'E320' = '51' /* Persistent hyperplasia of thymus - Other endocrine disorders */
'E321' = '51' /* Abscess of thymus - Other endocrine disorders */
'E328' = '51' /* Other diseases of thymus - Other endocrine disorders */
'E329' = '51' /* Disease of thymus, unspecified - Other endocrine disorders */
'E340' = '51' /* Carcinoid syndrome - Other endocrine disorders */
'E341' = '51' /* Other hypersecretion of intestinal hormones - Other endocrine disorders */
'E342' = '51' /* Ectopic hormone secretion, not elsewhere classified - Other endocrine disorders */
'E343' = '51' /* Short stature due to endocrine disorder - Other endocrine disorders */
'E344' = '51' /* Constitutional tall stature - Other endocrine disorders */
'E3450' = '51' /* Androgen insensitivity syndrome, unspecified - Other endocrine disorders */
'E3451' = '51' /* Complete androgen insensitivity syndrome - Other endocrine disorders */
'E3452' = '51' /* Partial androgen insensitivity syndrome - Other endocrine disorders */
'E348' = '51' /* Other specified endocrine disorders - Other endocrine disorders */
'E349' = '51' /* Endocrine disorder, unspecified - Other endocrine disorders */
'E35' = '51' /* Disorders of endocrine glands in diseases classd elswhr - Other endocrine disorders */
'Z9483' = '51' /* Pancreas transplant status - Other endocrine disorders */
'Z9649' = '51' /* Presence of other endocrine implants - Other endocrine disorders */
'D81818' = '52' /* Other biotin-dependent carboxylase deficiency - Nutritional deficiencies */
'D81819' = '52' /* Biotin-dependent carboxylase deficiency, unspecified - Nutritional deficiencies */
'E40' = '52' /* Kwashiorkor - Nutritional deficiencies */
'E41' = '52' /* Nutritional marasmus - Nutritional deficiencies */
'E42' = '52' /* Marasmic kwashiorkor - Nutritional deficiencies */
'E43' = '52' /* Unspecified severe protein-calorie malnutrition - Nutritional deficiencies */
'E440' = '52' /* Moderate protein-calorie malnutrition - Nutritional deficiencies */
'E441' = '52' /* Mild protein-calorie malnutrition - Nutritional deficiencies */
'E45' = '52' /* Retarded development following protein-calorie malnutrition - Nutritional deficiencies */
'E46' = '52' /* Unspecified protein-calorie malnutrition - Nutritional deficiencies */
'E500' = '52' /* Vitamin A deficiency with conjunctival xerosis - Nutritional deficiencies */
'E501' = '52' /* Vitamin A deficiency w Bitot's spot and conjunctival xerosis - Nutritional deficiencies */
'E502' = '52' /* Vitamin A deficiency with corneal xerosis - Nutritional deficiencies */
'E503' = '52' /* Vitamin A deficiency with corneal ulceration and xerosis - Nutritional deficiencies */
'E504' = '52' /* Vitamin A deficiency with keratomalacia - Nutritional deficiencies */
'E505' = '52' /* Vitamin A deficiency with night blindness - Nutritional deficiencies */
'E506' = '52' /* Vitamin A deficiency with xerophthalmic scars of cornea - Nutritional deficiencies */
'E507' = '52' /* Other ocular manifestations of vitamin A deficiency - Nutritional deficiencies */
'E508' = '52' /* Other manifestations of vitamin A deficiency - Nutritional deficiencies */
'E509' = '52' /* Vitamin A deficiency, unspecified - Nutritional deficiencies */
'E5111' = '52' /* Dry beriberi - Nutritional deficiencies */
'E5112' = '52' /* Wet beriberi - Nutritional deficiencies */
'E512' = '52' /* Wernicke's encephalopathy - Nutritional deficiencies */
'E518' = '52' /* Other manifestations of thiamine deficiency - Nutritional deficiencies */
'E519' = '52' /* Thiamine deficiency, unspecified - Nutritional deficiencies */
'E52' = '52' /* Niacin deficiency [pellagra] - Nutritional deficiencies */
'E530' = '52' /* Riboflavin deficiency - Nutritional deficiencies */
'E531' = '52' /* Pyridoxine deficiency - Nutritional deficiencies */
'E538' = '52' /* Deficiency of other specified B group vitamins - Nutritional deficiencies */
'E539' = '52' /* Vitamin B deficiency, unspecified - Nutritional deficiencies */
'E54' = '52' /* Ascorbic acid deficiency - Nutritional deficiencies */
'E550' = '52' /* Rickets, active - Nutritional deficiencies */
'E559' = '52' /* Vitamin D deficiency, unspecified - Nutritional deficiencies */
'E560' = '52' /* Deficiency of vitamin E - Nutritional deficiencies */
'E561' = '52' /* Deficiency of vitamin K - Nutritional deficiencies */
'E568' = '52' /* Deficiency of other vitamins - Nutritional deficiencies */
'E569' = '52' /* Vitamin deficiency, unspecified - Nutritional deficiencies */
'E58' = '52' /* Dietary calcium deficiency - Nutritional deficiencies */
'E59' = '52' /* Dietary selenium deficiency - Nutritional deficiencies */
'E60' = '52' /* Dietary zinc deficiency - Nutritional deficiencies */
'E610' = '52' /* Copper deficiency - Nutritional deficiencies */
'E611' = '52' /* Iron deficiency - Nutritional deficiencies */
'E612' = '52' /* Magnesium deficiency - Nutritional deficiencies */
'E613' = '52' /* Manganese deficiency - Nutritional deficiencies */
'E614' = '52' /* Chromium deficiency - Nutritional deficiencies */
'E615' = '52' /* Molybdenum deficiency - Nutritional deficiencies */
'E616' = '52' /* Vanadium deficiency - Nutritional deficiencies */
'E617' = '52' /* Deficiency of multiple nutrient elements - Nutritional deficiencies */
'E618' = '52' /* Deficiency of other specified nutrient elements - Nutritional deficiencies */
'E619' = '52' /* Deficiency of nutrient element, unspecified - Nutritional deficiencies */
'E630' = '52' /* Essential fatty acid [EFA] deficiency - Nutritional deficiencies */
'E631' = '52' /* Imbalance of constituents of food intake - Nutritional deficiencies */
'E638' = '52' /* Other specified nutritional deficiencies - Nutritional deficiencies */
'E639' = '52' /* Nutritional deficiency, unspecified - Nutritional deficiencies */
'E640' = '52' /* Sequelae of protein-calorie malnutrition - Nutritional deficiencies */
'E641' = '52' /* Sequelae of vitamin A deficiency - Nutritional deficiencies */
'E642' = '52' /* Sequelae of vitamin C deficiency - Nutritional deficiencies */
'E643' = '52' /* Sequelae of rickets - Nutritional deficiencies */
'E648' = '52' /* Sequelae of other nutritional deficiencies - Nutritional deficiencies */
'E649' = '52' /* Sequelae of unspecified nutritional deficiency - Nutritional deficiencies */
'M830' = '52' /* Puerperal osteomalacia - Nutritional deficiencies */
'M831' = '52' /* Senile osteomalacia - Nutritional deficiencies */
'M832' = '52' /* Adult osteomalacia due to malabsorption - Nutritional deficiencies */
'M833' = '52' /* Adult osteomalacia due to malnutrition - Nutritional deficiencies */
'M834' = '52' /* Aluminum bone disease - Nutritional deficiencies */
'M835' = '52' /* Other drug-induced osteomalacia in adults - Nutritional deficiencies */
'M838' = '52' /* Other adult osteomalacia - Nutritional deficiencies */
'M839' = '52' /* Adult osteomalacia, unspecified - Nutritional deficiencies */
'R64' = '52' /* Cachexia - Nutritional deficiencies */
'E840' = '56' /* Cystic fibrosis with pulmonary manifestations - Cystic fibrosis */
'E8411' = '56' /* Meconium ileus in cystic fibrosis - Cystic fibrosis */
'E8419' = '56' /* Cystic fibrosis with other intestinal manifestations - Cystic fibrosis */
'E848' = '56' /* Cystic fibrosis with other manifestations - Cystic fibrosis */
'E849' = '56' /* Cystic fibrosis, unspecified - Cystic fibrosis */
'D800' = '57' /* Hereditary hypogammaglobulinemia - Immunity disorders */
'D801' = '57' /* Nonfamilial hypogammaglobulinemia - Immunity disorders */
'D802' = '57' /* Selective deficiency of immunoglobulin A [IgA] - Immunity disorders */
'D803' = '57' /* Selective deficiency of immunoglobulin G [IgG] subclasses - Immunity disorders */
'D804' = '57' /* Selective deficiency of immunoglobulin M [IgM] - Immunity disorders */
'D805' = '57' /* Immunodeficiency with increased immunoglobulin M [IgM] - Immunity disorders */
'D806' = '57' /* Antibody defic w near-norm immunoglob or w hyperimmunoglob - Immunity disorders */
'D807' = '57' /* Transient hypogammaglobulinemia of infancy - Immunity disorders */
'D808' = '57' /* Other immunodeficiencies with predominantly antibody defects - Immunity disorders */
'D809' = '57' /* Immunodeficiency with predominantly antibody defects, unsp - Immunity disorders */
'D810' = '57' /* Severe combined immunodeficiency with reticular dysgenesis - Immunity disorders */
'D811' = '57' /* Severe combined immunodeficiency w low T- and B-cell numbers - Immunity disorders */
'D812' = '57' /* Severe combined immunodef w low or normal B-cell numbers - Immunity disorders */
'D814' = '57' /* Nezelof's syndrome - Immunity disorders */
'D816' = '57' /* Major histocompatibility complex class I deficiency - Immunity disorders */
'D817' = '57' /* Major histocompatibility complex class II deficiency - Immunity disorders */
'D8189' = '57' /* Other combined immunodeficiencies - Immunity disorders */
'D819' = '57' /* Combined immunodeficiency, unspecified - Immunity disorders */
'D820' = '57' /* Wiskott-Aldrich syndrome - Immunity disorders */
'D821' = '57' /* Di George's syndrome - Immunity disorders */
'D822' = '57' /* Immunodeficiency with short-limbed stature - Immunity disorders */
'D823' = '57' /* Immunodef fol heredit defctv response to Epstein-Barr virus - Immunity disorders */
'D824' = '57' /* Hyperimmunoglobulin E [IgE] syndrome - Immunity disorders */
'D828' = '57' /* Immunodeficiency associated with oth major defects - Immunity disorders */
'D829' = '57' /* Immunodeficiency associated with major defect, unspecified - Immunity disorders */
'D830' = '57' /* Com variab immunodef w predom abnlt of B-cell nums & functn - Immunity disorders */
'D831' = '57' /* Com variab immunodef w predom immunoreg T-cell disorders - Immunity disorders */
'D832' = '57' /* Common variable immunodef w autoantibodies to B- or T-cells - Immunity disorders */
'D838' = '57' /* Other common variable immunodeficiencies - Immunity disorders */
'D839' = '57' /* Common variable immunodeficiency, unspecified - Immunity disorders */
'D840' = '57' /* Lymphocyte function antigen-1 [LFA-1] defect - Immunity disorders */
'D841' = '57' /* Defects in the complement system - Immunity disorders */
'D848' = '57' /* Other specified immunodeficiencies - Immunity disorders */
'D849' = '57' /* Immunodeficiency, unspecified - Immunity disorders */
'D860' = '57' /* Sarcoidosis of lung - Immunity disorders */
'D861' = '57' /* Sarcoidosis of lymph nodes - Immunity disorders */
'D862' = '57' /* Sarcoidosis of lung with sarcoidosis of lymph nodes - Immunity disorders */
'D863' = '57' /* Sarcoidosis of skin - Immunity disorders */
'D8682' = '57' /* Multiple cranial nerve palsies in sarcoidosis - Immunity disorders */
'D8683' = '57' /* Sarcoid iridocyclitis - Immunity disorders */
'D8684' = '57' /* Sarcoid pyelonephritis - Immunity disorders */
'D8685' = '57' /* Sarcoid myocarditis - Immunity disorders */
'D8686' = '57' /* Sarcoid arthropathy - Immunity disorders */
'D8687' = '57' /* Sarcoid myositis - Immunity disorders */
'D8689' = '57' /* Sarcoidosis of other sites - Immunity disorders */
'D869' = '57' /* Sarcoidosis, unspecified - Immunity disorders */
'D890' = '57' /* Polyclonal hypergammaglobulinemia - Immunity disorders */
'D891' = '57' /* Cryoglobulinemia - Immunity disorders */
'D892' = '57' /* Hypergammaglobulinemia, unspecified - Immunity disorders */
'D893' = '57' /* Immune reconstitution syndrome - Immunity disorders */
'D8982' = '57' /* Autoimmune lymphoproliferative syndrome [ALPS] - Immunity disorders */
'D8989' = '57' /* Oth disrd involving the immune mechanism, NEC - Immunity disorders */
'D899' = '57' /* Disorder involving the immune mechanism, unspecified - Immunity disorders */
'D813' = '58' /* Adenosine deaminase [ADA] deficiency - Other nutritional; endocrine; and metabolic disorders */
'D815' = '58' /* Purine nucleoside phosphorylase [PNP] deficiency - Other nutritional; endocrine; and metabolic disorders */
'D81810' = '58' /* Biotinidase deficiency - Other nutritional; endocrine; and metabolic disorders */
'E201' = '58' /* Pseudohypoparathyroidism - Other nutritional; endocrine; and metabolic disorders */
'E65' = '58' /* Localized adiposity - Other nutritional; endocrine; and metabolic disorders */
'E6601' = '58' /* Morbid (severe) obesity due to excess calories - Other nutritional; endocrine; and metabolic disorders */
'E6609' = '58' /* Other obesity due to excess calories - Other nutritional; endocrine; and metabolic disorders */
'E661' = '58' /* Drug-induced obesity - Other nutritional; endocrine; and metabolic disorders */
'E662' = '58' /* Morbid (severe) obesity with alveolar hypoventilation - Other nutritional; endocrine; and metabolic disorders */
'E663' = '58' /* Overweight - Other nutritional; endocrine; and metabolic disorders */
'E668' = '58' /* Other obesity - Other nutritional; endocrine; and metabolic disorders */
'E669' = '58' /* Obesity, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E670' = '58' /* Hypervitaminosis A - Other nutritional; endocrine; and metabolic disorders */
'E671' = '58' /* Hypercarotinemia - Other nutritional; endocrine; and metabolic disorders */
'E672' = '58' /* Megavitamin-B6 syndrome - Other nutritional; endocrine; and metabolic disorders */
'E673' = '58' /* Hypervitaminosis D - Other nutritional; endocrine; and metabolic disorders */
'E678' = '58' /* Other specified hyperalimentation - Other nutritional; endocrine; and metabolic disorders */
'E68' = '58' /* Sequelae of hyperalimentation - Other nutritional; endocrine; and metabolic disorders */
'E700' = '58' /* Classical phenylketonuria - Other nutritional; endocrine; and metabolic disorders */
'E701' = '58' /* Other hyperphenylalaninemias - Other nutritional; endocrine; and metabolic disorders */
'E7020' = '58' /* Disorder of tyrosine metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E7021' = '58' /* Tyrosinemia - Other nutritional; endocrine; and metabolic disorders */
'E7029' = '58' /* Other disorders of tyrosine metabolism - Other nutritional; endocrine; and metabolic disorders */
'E7030' = '58' /* Albinism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E70310' = '58' /* X-linked ocular albinism - Other nutritional; endocrine; and metabolic disorders */
'E70311' = '58' /* Autosomal recessive ocular albinism - Other nutritional; endocrine; and metabolic disorders */
'E70318' = '58' /* Other ocular albinism - Other nutritional; endocrine; and metabolic disorders */
'E70319' = '58' /* Ocular albinism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E70320' = '58' /* Tyrosinase negative oculocutaneous albinism - Other nutritional; endocrine; and metabolic disorders */
'E70321' = '58' /* Tyrosinase positive oculocutaneous albinism - Other nutritional; endocrine; and metabolic disorders */
'E70328' = '58' /* Other oculocutaneous albinism - Other nutritional; endocrine; and metabolic disorders */
'E70329' = '58' /* Oculocutaneous albinism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E70330' = '58' /* Chediak-Higashi syndrome - Other nutritional; endocrine; and metabolic disorders */
'E70331' = '58' /* Hermansky-Pudlak syndrome - Other nutritional; endocrine; and metabolic disorders */
'E70338' = '58' /* Other albinism with hematologic abnormality - Other nutritional; endocrine; and metabolic disorders */
'E70339' = '58' /* Albinism with hematologic abnormality, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E7039' = '58' /* Other specified albinism - Other nutritional; endocrine; and metabolic disorders */
'E7040' = '58' /* Disorders of histidine metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E7041' = '58' /* Histidinemia - Other nutritional; endocrine; and metabolic disorders */
'E7049' = '58' /* Other disorders of histidine metabolism - Other nutritional; endocrine; and metabolic disorders */
'E705' = '58' /* Disorders of tryptophan metabolism - Other nutritional; endocrine; and metabolic disorders */
'E708' = '58' /* Other disorders of aromatic amino-acid metabolism - Other nutritional; endocrine; and metabolic disorders */
'E709' = '58' /* Disorder of aromatic amino-acid metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E710' = '58' /* Maple-syrup-urine disease - Other nutritional; endocrine; and metabolic disorders */
'E71110' = '58' /* Isovaleric acidemia - Other nutritional; endocrine; and metabolic disorders */
'E71111' = '58' /* 3-methylglutaconic aciduria - Other nutritional; endocrine; and metabolic disorders */
'E71118' = '58' /* Other branched-chain organic acidurias - Other nutritional; endocrine; and metabolic disorders */
'E71120' = '58' /* Methylmalonic acidemia - Other nutritional; endocrine; and metabolic disorders */
'E71121' = '58' /* Propionic acidemia - Other nutritional; endocrine; and metabolic disorders */
'E71128' = '58' /* Other disorders of propionate metabolism - Other nutritional; endocrine; and metabolic disorders */
'E7119' = '58' /* Other disorders of branched-chain amino-acid metabolism - Other nutritional; endocrine; and metabolic disorders */
'E712' = '58' /* Disorder of branched-chain amino-acid metabolism, unsp - Other nutritional; endocrine; and metabolic disorders */
'E7130' = '58' /* Disorder of fatty-acid metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E71310' = '58' /* Long chain/very long chain acyl CoA dehydrogenase deficiency - Other nutritional; endocrine; and metabolic disorders */
'E71311' = '58' /* Medium chain acyl CoA dehydrogenase deficiency - Other nutritional; endocrine; and metabolic disorders */
'E71312' = '58' /* Short chain acyl CoA dehydrogenase deficiency - Other nutritional; endocrine; and metabolic disorders */
'E71313' = '58' /* Glutaric aciduria type II - Other nutritional; endocrine; and metabolic disorders */
'E71314' = '58' /* Muscle carnitine palmitoyltransferase deficiency - Other nutritional; endocrine; and metabolic disorders */
'E71318' = '58' /* Other disorders of fatty-acid oxidation - Other nutritional; endocrine; and metabolic disorders */
'E7132' = '58' /* Disorders of ketone metabolism - Other nutritional; endocrine; and metabolic disorders */
'E7139' = '58' /* Other disorders of fatty-acid metabolism - Other nutritional; endocrine; and metabolic disorders */
'E7140' = '58' /* Disorder of carnitine metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E7141' = '58' /* Primary carnitine deficiency - Other nutritional; endocrine; and metabolic disorders */
'E7142' = '58' /* Carnitine deficiency due to inborn errors of metabolism - Other nutritional; endocrine; and metabolic disorders */
'E71440' = '58' /* Ruvalcaba-Myhre-Smith syndrome - Other nutritional; endocrine; and metabolic disorders */
'E71448' = '58' /* Other secondary carnitine deficiency - Other nutritional; endocrine; and metabolic disorders */
'E7150' = '58' /* Peroxisomal disorder, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E71510' = '58' /* Zellweger syndrome - Other nutritional; endocrine; and metabolic disorders */
'E71511' = '58' /* Neonatal adrenoleukodystrophy - Other nutritional; endocrine; and metabolic disorders */
'E71518' = '58' /* Other disorders of peroxisome biogenesis - Other nutritional; endocrine; and metabolic disorders */
'E71520' = '58' /* Childhood cerebral X-linked adrenoleukodystrophy - Other nutritional; endocrine; and metabolic disorders */
'E71521' = '58' /* Adolescent X-linked adrenoleukodystrophy - Other nutritional; endocrine; and metabolic disorders */
'E71522' = '58' /* Adrenomyeloneuropathy - Other nutritional; endocrine; and metabolic disorders */
'E71528' = '58' /* Other X-linked adrenoleukodystrophy - Other nutritional; endocrine; and metabolic disorders */
'E71529' = '58' /* X-linked adrenoleukodystrophy, unspecified type - Other nutritional; endocrine; and metabolic disorders */
'E7153' = '58' /* Other group 2 peroxisomal disorders - Other nutritional; endocrine; and metabolic disorders */
'E71540' = '58' /* Rhizomelic chondrodysplasia punctata - Other nutritional; endocrine; and metabolic disorders */
'E71541' = '58' /* Zellweger-like syndrome - Other nutritional; endocrine; and metabolic disorders */
'E71542' = '58' /* Other group 3 peroxisomal disorders - Other nutritional; endocrine; and metabolic disorders */
'E71548' = '58' /* Other peroxisomal disorders - Other nutritional; endocrine; and metabolic disorders */
'E7200' = '58' /* Disorders of amino-acid transport, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E7201' = '58' /* Cystinuria - Other nutritional; endocrine; and metabolic disorders */
'E7202' = '58' /* Hartnup's disease - Other nutritional; endocrine; and metabolic disorders */
'E7203' = '58' /* Lowe's syndrome - Other nutritional; endocrine; and metabolic disorders */
'E7204' = '58' /* Cystinosis - Other nutritional; endocrine; and metabolic disorders */
'E7209' = '58' /* Other disorders of amino-acid transport - Other nutritional; endocrine; and metabolic disorders */
'E7210' = '58' /* Disorders of sulfur-bearing amino-acid metabolism, unsp - Other nutritional; endocrine; and metabolic disorders */
'E7211' = '58' /* Homocystinuria - Other nutritional; endocrine; and metabolic disorders */
'E7212' = '58' /* Methylenetetrahydrofolate reductase deficiency - Other nutritional; endocrine; and metabolic disorders */
'E7219' = '58' /* Other disorders of sulfur-bearing amino-acid metabolism - Other nutritional; endocrine; and metabolic disorders */
'E7220' = '58' /* Disorder of urea cycle metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E7221' = '58' /* Argininemia - Other nutritional; endocrine; and metabolic disorders */
'E7222' = '58' /* Arginosuccinic aciduria - Other nutritional; endocrine; and metabolic disorders */
'E7223' = '58' /* Citrullinemia - Other nutritional; endocrine; and metabolic disorders */
'E7229' = '58' /* Other disorders of urea cycle metabolism - Other nutritional; endocrine; and metabolic disorders */
'E723' = '58' /* Disorders of lysine and hydroxylysine metabolism - Other nutritional; endocrine; and metabolic disorders */
'E724' = '58' /* Disorders of ornithine metabolism - Other nutritional; endocrine; and metabolic disorders */
'E7250' = '58' /* Disorder of glycine metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E7251' = '58' /* Non-ketotic hyperglycinemia - Other nutritional; endocrine; and metabolic disorders */
'E7252' = '58' /* Trimethylaminuria - Other nutritional; endocrine; and metabolic disorders */
'E7253' = '58' /* Hyperoxaluria - Other nutritional; endocrine; and metabolic disorders */
'E7259' = '58' /* Other disorders of glycine metabolism - Other nutritional; endocrine; and metabolic disorders */
'E728' = '58' /* Other specified disorders of amino-acid metabolism - Other nutritional; endocrine; and metabolic disorders */
'E729' = '58' /* Disorder of amino-acid metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E730' = '58' /* Congenital lactase deficiency - Other nutritional; endocrine; and metabolic disorders */
'E731' = '58' /* Secondary lactase deficiency - Other nutritional; endocrine; and metabolic disorders */
'E738' = '58' /* Other lactose intolerance - Other nutritional; endocrine; and metabolic disorders */
'E739' = '58' /* Lactose intolerance, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E7400' = '58' /* Glycogen storage disease, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E7401' = '58' /* von Gierke disease - Other nutritional; endocrine; and metabolic disorders */
'E7402' = '58' /* Pompe disease - Other nutritional; endocrine; and metabolic disorders */
'E7403' = '58' /* Cori disease - Other nutritional; endocrine; and metabolic disorders */
'E7404' = '58' /* McArdle disease - Other nutritional; endocrine; and metabolic disorders */
'E7409' = '58' /* Other glycogen storage disease - Other nutritional; endocrine; and metabolic disorders */
'E7410' = '58' /* Disorder of fructose metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E7411' = '58' /* Essential fructosuria - Other nutritional; endocrine; and metabolic disorders */
'E7412' = '58' /* Hereditary fructose intolerance - Other nutritional; endocrine; and metabolic disorders */
'E7419' = '58' /* Other disorders of fructose metabolism - Other nutritional; endocrine; and metabolic disorders */
'E7420' = '58' /* Disorders of galactose metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E7421' = '58' /* Galactosemia - Other nutritional; endocrine; and metabolic disorders */
'E7429' = '58' /* Other disorders of galactose metabolism - Other nutritional; endocrine; and metabolic disorders */
'E7431' = '58' /* Sucrase-isomaltase deficiency - Other nutritional; endocrine; and metabolic disorders */
'E7439' = '58' /* Other disorders of intestinal carbohydrate absorption - Other nutritional; endocrine; and metabolic disorders */
'E744' = '58' /* Disorders of pyruvate metabolism and gluconeogenesis - Other nutritional; endocrine; and metabolic disorders */
'E748' = '58' /* Other specified disorders of carbohydrate metabolism - Other nutritional; endocrine; and metabolic disorders */
'E749' = '58' /* Disorder of carbohydrate metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E7521' = '58' /* Fabry (-Anderson) disease - Other nutritional; endocrine; and metabolic disorders */
'E7522' = '58' /* Gaucher disease - Other nutritional; endocrine; and metabolic disorders */
'E75240' = '58' /* Niemann-Pick disease type A - Other nutritional; endocrine; and metabolic disorders */
'E75241' = '58' /* Niemann-Pick disease type B - Other nutritional; endocrine; and metabolic disorders */
'E75242' = '58' /* Niemann-Pick disease type C - Other nutritional; endocrine; and metabolic disorders */
'E75243' = '58' /* Niemann-Pick disease type D - Other nutritional; endocrine; and metabolic disorders */
'E75248' = '58' /* Other Niemann-Pick disease - Other nutritional; endocrine; and metabolic disorders */
'E75249' = '58' /* Niemann-Pick disease, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E753' = '58' /* Sphingolipidosis, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E755' = '58' /* Other lipid storage disorders - Other nutritional; endocrine; and metabolic disorders */
'E756' = '58' /* Lipid storage disorder, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E7601' = '58' /* Hurler's syndrome - Other nutritional; endocrine; and metabolic disorders */
'E7602' = '58' /* Hurler-Scheie syndrome - Other nutritional; endocrine; and metabolic disorders */
'E7603' = '58' /* Scheie's syndrome - Other nutritional; endocrine; and metabolic disorders */
'E761' = '58' /* Mucopolysaccharidosis, type II - Other nutritional; endocrine; and metabolic disorders */
'E76210' = '58' /* Morquio A mucopolysaccharidoses - Other nutritional; endocrine; and metabolic disorders */
'E76211' = '58' /* Morquio B mucopolysaccharidoses - Other nutritional; endocrine; and metabolic disorders */
'E76219' = '58' /* Morquio mucopolysaccharidoses, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E7622' = '58' /* Sanfilippo mucopolysaccharidoses - Other nutritional; endocrine; and metabolic disorders */
'E7629' = '58' /* Other mucopolysaccharidoses - Other nutritional; endocrine; and metabolic disorders */
'E763' = '58' /* Mucopolysaccharidosis, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E768' = '58' /* Other disorders of glucosaminoglycan metabolism - Other nutritional; endocrine; and metabolic disorders */
'E769' = '58' /* Glucosaminoglycan metabolism disorder, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E770' = '58' /* Defects in post-translational mod of lysosomal enzymes - Other nutritional; endocrine; and metabolic disorders */
'E771' = '58' /* Defects in glycoprotein degradation - Other nutritional; endocrine; and metabolic disorders */
'E778' = '58' /* Other disorders of glycoprotein metabolism - Other nutritional; endocrine; and metabolic disorders */
'E779' = '58' /* Disorder of glycoprotein metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E786' = '58' /* Lipoprotein deficiency - Other nutritional; endocrine; and metabolic disorders */
'E7870' = '58' /* Disorder of bile acid and cholesterol metabolism, unsp - Other nutritional; endocrine; and metabolic disorders */
'E7879' = '58' /* Other disorders of bile acid and cholesterol metabolism - Other nutritional; endocrine; and metabolic disorders */
'E7881' = '58' /* Lipoid dermatoarthritis - Other nutritional; endocrine; and metabolic disorders */
'E7889' = '58' /* Other lipoprotein metabolism disorders - Other nutritional; endocrine; and metabolic disorders */
'E789' = '58' /* Disorder of lipoprotein metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E790' = '58' /* Hyperuricemia w/o signs of inflam arthrit and tophaceous dis - Other nutritional; endocrine; and metabolic disorders */
'E791' = '58' /* Lesch-Nyhan syndrome - Other nutritional; endocrine; and metabolic disorders */
'E792' = '58' /* Myoadenylate deaminase deficiency - Other nutritional; endocrine; and metabolic disorders */
'E798' = '58' /* Other disorders of purine and pyrimidine metabolism - Other nutritional; endocrine; and metabolic disorders */
'E799' = '58' /* Disorder of purine and pyrimidine metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E800' = '58' /* Hereditary erythropoietic porphyria - Other nutritional; endocrine; and metabolic disorders */
'E801' = '58' /* Porphyria cutanea tarda - Other nutritional; endocrine; and metabolic disorders */
'E8020' = '58' /* Unspecified porphyria - Other nutritional; endocrine; and metabolic disorders */
'E8021' = '58' /* Acute intermittent (hepatic) porphyria - Other nutritional; endocrine; and metabolic disorders */
'E8029' = '58' /* Other porphyria - Other nutritional; endocrine; and metabolic disorders */
'E803' = '58' /* Defects of catalase and peroxidase - Other nutritional; endocrine; and metabolic disorders */
'E804' = '58' /* Gilbert syndrome - Other nutritional; endocrine; and metabolic disorders */
'E805' = '58' /* Crigler-Najjar syndrome - Other nutritional; endocrine; and metabolic disorders */
'E806' = '58' /* Other disorders of bilirubin metabolism - Other nutritional; endocrine; and metabolic disorders */
'E807' = '58' /* Disorder of bilirubin metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E8300' = '58' /* Disorder of copper metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E8301' = '58' /* Wilson's disease - Other nutritional; endocrine; and metabolic disorders */
'E8309' = '58' /* Other disorders of copper metabolism - Other nutritional; endocrine; and metabolic disorders */
'E8310' = '58' /* Disorder of iron metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E83110' = '58' /* Hereditary hemochromatosis - Other nutritional; endocrine; and metabolic disorders */
'E83111' = '58' /* Hemochromatosis due to repeated red blood cell transfusions - Other nutritional; endocrine; and metabolic disorders */
'E83118' = '58' /* Other hemochromatosis - Other nutritional; endocrine; and metabolic disorders */
'E83119' = '58' /* Hemochromatosis, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E8319' = '58' /* Other disorders of iron metabolism - Other nutritional; endocrine; and metabolic disorders */
'E832' = '58' /* Disorders of zinc metabolism - Other nutritional; endocrine; and metabolic disorders */
'E8330' = '58' /* Disorder of phosphorus metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E8331' = '58' /* Familial hypophosphatemia - Other nutritional; endocrine; and metabolic disorders */
'E8332' = '58' /* Hereditary vitamin D-dependent rickets (type 1) (type 2) - Other nutritional; endocrine; and metabolic disorders */
'E8339' = '58' /* Other disorders of phosphorus metabolism - Other nutritional; endocrine; and metabolic disorders */
'E8340' = '58' /* Disorders of magnesium metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E8341' = '58' /* Hypermagnesemia - Other nutritional; endocrine; and metabolic disorders */
'E8342' = '58' /* Hypomagnesemia - Other nutritional; endocrine; and metabolic disorders */
'E8349' = '58' /* Other disorders of magnesium metabolism - Other nutritional; endocrine; and metabolic disorders */
'E8350' = '58' /* Unspecified disorder of calcium metabolism - Other nutritional; endocrine; and metabolic disorders */
'E8351' = '58' /* Hypocalcemia - Other nutritional; endocrine; and metabolic disorders */
'E8352' = '58' /* Hypercalcemia - Other nutritional; endocrine; and metabolic disorders */
'E8359' = '58' /* Other disorders of calcium metabolism - Other nutritional; endocrine; and metabolic disorders */
'E8381' = '58' /* Hungry bone syndrome - Other nutritional; endocrine; and metabolic disorders */
'E8389' = '58' /* Other disorders of mineral metabolism - Other nutritional; endocrine; and metabolic disorders */
'E839' = '58' /* Disorder of mineral metabolism, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E850' = '58' /* Non-neuropathic heredofamilial amyloidosis - Other nutritional; endocrine; and metabolic disorders */
'E851' = '58' /* Neuropathic heredofamilial amyloidosis - Other nutritional; endocrine; and metabolic disorders */
'E852' = '58' /* Heredofamilial amyloidosis, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E853' = '58' /* Secondary systemic amyloidosis - Other nutritional; endocrine; and metabolic disorders */
'E854' = '58' /* Organ-limited amyloidosis - Other nutritional; endocrine; and metabolic disorders */
'E858' = '58' /* Other amyloidosis - Other nutritional; endocrine; and metabolic disorders */
'E859' = '58' /* Amyloidosis, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E8801' = '58' /* Alpha-1-antitrypsin deficiency - Other nutritional; endocrine; and metabolic disorders */
'E8809' = '58' /* Oth disorders of plasma-protein metabolism, NEC - Other nutritional; endocrine; and metabolic disorders */
'E881' = '58' /* Lipodystrophy, not elsewhere classified - Other nutritional; endocrine; and metabolic disorders */
'E882' = '58' /* Lipomatosis, not elsewhere classified - Other nutritional; endocrine; and metabolic disorders */
'E8840' = '58' /* Mitochondrial metabolism disorder, unspecified - Other nutritional; endocrine; and metabolic disorders */
'E8841' = '58' /* MELAS syndrome - Other nutritional; endocrine; and metabolic disorders */
'E8842' = '58' /* MERRF syndrome - Other nutritional; endocrine; and metabolic disorders */
'E8849' = '58' /* Other mitochondrial metabolism disorders - Other nutritional; endocrine; and metabolic disorders */
'E8881' = '58' /* Metabolic syndrome - Other nutritional; endocrine; and metabolic disorders */
'E8889' = '58' /* Other specified metabolic disorders - Other nutritional; endocrine; and metabolic disorders */
'E889' = '58' /* Metabolic disorder, unspecified - Other nutritional; endocrine; and metabolic disorders */
'R620' = '58' /* Delayed milestone in childhood - Other nutritional; endocrine; and metabolic disorders */
'R6250' = '58' /* Unsp lack of expected normal physiol dev in childhood - Other nutritional; endocrine; and metabolic disorders */
'R6251' = '58' /* Failure to thrive (child) - Other nutritional; endocrine; and metabolic disorders */
'R6252' = '58' /* Short stature (child) - Other nutritional; endocrine; and metabolic disorders */
'R6259' = '58' /* Oth lack of expected normal physiol development in childhood - Other nutritional; endocrine; and metabolic disorders */
'R627' = '58' /* Adult failure to thrive - Other nutritional; endocrine; and metabolic disorders */
'R631' = '58' /* Polydipsia - Other nutritional; endocrine; and metabolic disorders */
'R633' = '58' /* Feeding difficulties - Other nutritional; endocrine; and metabolic disorders */
'R634' = '58' /* Abnormal weight loss - Other nutritional; endocrine; and metabolic disorders */
'R635' = '58' /* Abnormal weight gain - Other nutritional; endocrine; and metabolic disorders */
'R636' = '58' /* Underweight - Other nutritional; endocrine; and metabolic disorders */
'R638' = '58' /* Other symptoms and signs concerning food and fluid intake - Other nutritional; endocrine; and metabolic disorders */
'Z8639' = '58' /* Personal history of endo, nutritional and metabolic disease - Other nutritional; endocrine; and metabolic disorders */
'D65' = '62' /* Disseminated intravascular coagulation - Coagulation and hemorrhagic disorders */
'D66' = '62' /* Hereditary factor VIII deficiency - Coagulation and hemorrhagic disorders */
'D67' = '62' /* Hereditary factor IX deficiency - Coagulation and hemorrhagic disorders */
'D680' = '62' /* Von Willebrand's disease - Coagulation and hemorrhagic disorders */
'D681' = '62' /* Hereditary factor XI deficiency - Coagulation and hemorrhagic disorders */
'D682' = '62' /* Hereditary deficiency of other clotting factors - Coagulation and hemorrhagic disorders */
'D68311' = '62' /* Acquired hemophilia - Coagulation and hemorrhagic disorders */
'D68312' = '62' /* Antiphospholipid antibody with hemorrhagic disorder - Coagulation and hemorrhagic disorders */
'D68318' = '62' /* Oth hemorrhagic disord d/t intrns circ anticoag,antib,inhib - Coagulation and hemorrhagic disorders */
'D6832' = '62' /* Hemorrhagic disord d/t extrinsic circulating anticoagulants - Coagulation and hemorrhagic disorders */
'D684' = '62' /* Acquired coagulation factor deficiency - Coagulation and hemorrhagic disorders */
'D6851' = '62' /* Activated protein C resistance - Coagulation and hemorrhagic disorders */
'D6852' = '62' /* Prothrombin gene mutation - Coagulation and hemorrhagic disorders */
'D6859' = '62' /* Other primary thrombophilia - Coagulation and hemorrhagic disorders */
'D6861' = '62' /* Antiphospholipid syndrome - Coagulation and hemorrhagic disorders */
'D6862' = '62' /* Lupus anticoagulant syndrome - Coagulation and hemorrhagic disorders */
'D6869' = '62' /* Other thrombophilia - Coagulation and hemorrhagic disorders */
'D688' = '62' /* Other specified coagulation defects - Coagulation and hemorrhagic disorders */
'D689' = '62' /* Coagulation defect, unspecified - Coagulation and hemorrhagic disorders */
'D690' = '62' /* Allergic purpura - Coagulation and hemorrhagic disorders */
'D691' = '62' /* Qualitative platelet defects - Coagulation and hemorrhagic disorders */
'D692' = '62' /* Other nonthrombocytopenic purpura - Coagulation and hemorrhagic disorders */
'D693' = '62' /* Immune thrombocytopenic purpura - Coagulation and hemorrhagic disorders */
'D6941' = '62' /* Evans syndrome - Coagulation and hemorrhagic disorders */
'D6942' = '62' /* Congenital and hereditary thrombocytopenia purpura - Coagulation and hemorrhagic disorders */
'D6949' = '62' /* Other primary thrombocytopenia - Coagulation and hemorrhagic disorders */
'D6959' = '62' /* Other secondary thrombocytopenia - Coagulation and hemorrhagic disorders */
'D696' = '62' /* Thrombocytopenia, unspecified - Coagulation and hemorrhagic disorders */
'D698' = '62' /* Other specified hemorrhagic conditions - Coagulation and hemorrhagic disorders */
'D699' = '62' /* Hemorrhagic condition, unspecified - Coagulation and hemorrhagic disorders */
'D7582' = '62' /* Heparin induced thrombocytopenia (HIT) - Coagulation and hemorrhagic disorders */
'R233' = '62' /* Spontaneous ecchymoses - Coagulation and hemorrhagic disorders */
'D700' = '63' /* Congenital agranulocytosis - Diseases of white blood cells */
'D701' = '63' /* Agranulocytosis secondary to cancer chemotherapy - Diseases of white blood cells */
'D702' = '63' /* Other drug-induced agranulocytosis - Diseases of white blood cells */
'D703' = '63' /* Neutropenia due to infection - Diseases of white blood cells */
'D704' = '63' /* Cyclic neutropenia - Diseases of white blood cells */
'D708' = '63' /* Other neutropenia - Diseases of white blood cells */
'D709' = '63' /* Neutropenia, unspecified - Diseases of white blood cells */
'D71' = '63' /* Functional disorders of polymorphonuclear neutrophils - Diseases of white blood cells */
'D720' = '63' /* Genetic anomalies of leukocytes - Diseases of white blood cells */
'D721' = '63' /* Eosinophilia - Diseases of white blood cells */
'D72810' = '63' /* Lymphocytopenia - Diseases of white blood cells */
'D72818' = '63' /* Other decreased white blood cell count - Diseases of white blood cells */
'D72819' = '63' /* Decreased white blood cell count, unspecified - Diseases of white blood cells */
'D72820' = '63' /* Lymphocytosis (symptomatic) - Diseases of white blood cells */
'D72821' = '63' /* Monocytosis (symptomatic) - Diseases of white blood cells */
'D72822' = '63' /* Plasmacytosis - Diseases of white blood cells */
'D72823' = '63' /* Leukemoid reaction - Diseases of white blood cells */
'D72824' = '63' /* Basophilia - Diseases of white blood cells */
'D72825' = '63' /* Bandemia - Diseases of white blood cells */
'D72828' = '63' /* Other elevated white blood cell count - Diseases of white blood cells */
'D72829' = '63' /* Elevated white blood cell count, unspecified - Diseases of white blood cells */
'D7289' = '63' /* Other specified disorders of white blood cells - Diseases of white blood cells */
'D729' = '63' /* Disorder of white blood cells, unspecified - Diseases of white blood cells */
'D7381' = '63' /* Neutropenic splenomegaly - Diseases of white blood cells */
'D761' = '63' /* Hemophagocytic lymphohistiocytosis - Diseases of white blood cells */
'D762' = '63' /* Hemophagocytic syndrome, infection-associated - Diseases of white blood cells */
'D763' = '63' /* Other histiocytosis syndromes - Diseases of white blood cells */
'E7500' = '81' /* GM2 gangliosidosis, unspecified - Other hereditary and degenerative nervous system conditions */
'E7501' = '81' /* Sandhoff disease - Other hereditary and degenerative nervous system conditions */
'E7502' = '81' /* Tay-Sachs disease - Other hereditary and degenerative nervous system conditions */
'E7509' = '81' /* Other GM2 gangliosidosis - Other hereditary and degenerative nervous system conditions */
'E7510' = '81' /* Unspecified gangliosidosis - Other hereditary and degenerative nervous system conditions */
'E7511' = '81' /* Mucolipidosis IV - Other hereditary and degenerative nervous system conditions */
'E7519' = '81' /* Other gangliosidosis - Other hereditary and degenerative nervous system conditions */
'E7523' = '81' /* Krabbe disease - Other hereditary and degenerative nervous system conditions */
'E7525' = '81' /* Metachromatic leukodystrophy - Other hereditary and degenerative nervous system conditions */
'E7529' = '81' /* Other sphingolipidosis - Other hereditary and degenerative nervous system conditions */
'E754' = '81' /* Neuronal ceroid lipofuscinosis - Other hereditary and degenerative nervous system conditions */
'G10' = '81' /* Huntington's disease - Other hereditary and degenerative nervous system conditions */
'G110' = '81' /* Congenital nonprogressive ataxia - Other hereditary and degenerative nervous system conditions */
'G111' = '81' /* Early-onset cerebellar ataxia - Other hereditary and degenerative nervous system conditions */
'G112' = '81' /* Late-onset cerebellar ataxia - Other hereditary and degenerative nervous system conditions */
'G113' = '81' /* Cerebellar ataxia with defective DNA repair - Other hereditary and degenerative nervous system conditions */
'G114' = '81' /* Hereditary spastic paraplegia - Other hereditary and degenerative nervous system conditions */
'G118' = '81' /* Other hereditary ataxias - Other hereditary and degenerative nervous system conditions */
'G119' = '81' /* Hereditary ataxia, unspecified - Other hereditary and degenerative nervous system conditions */
'G120' = '81' /* Infantile spinal muscular atrophy, type I [Werdnig-Hoffman] - Other hereditary and degenerative nervous system conditions */
'G121' = '81' /* Other inherited spinal muscular atrophy - Other hereditary and degenerative nervous system conditions */
'G1220' = '81' /* Motor neuron disease, unspecified - Other hereditary and degenerative nervous system conditions */
'G1221' = '81' /* Amyotrophic lateral sclerosis - Other hereditary and degenerative nervous system conditions */
'G1222' = '81' /* Progressive bulbar palsy - Other hereditary and degenerative nervous system conditions */
'G1229' = '81' /* Other motor neuron disease - Other hereditary and degenerative nervous system conditions */
'G128' = '81' /* Other spinal muscular atrophies and related syndromes - Other hereditary and degenerative nervous system conditions */
'G129' = '81' /* Spinal muscular atrophy, unspecified - Other hereditary and degenerative nervous system conditions */
'G230' = '81' /* Hallervorden-Spatz disease - Other hereditary and degenerative nervous system conditions */
'G231' = '81' /* Progressive supranuclear ophthalmoplegia - Other hereditary and degenerative nervous system conditions */
'G232' = '81' /* Striatonigral degeneration - Other hereditary and degenerative nervous system conditions */
'G238' = '81' /* Other specified degenerative diseases of basal ganglia - Other hereditary and degenerative nervous system conditions */
'G239' = '81' /* Degenerative disease of basal ganglia, unspecified - Other hereditary and degenerative nervous system conditions */
'G2401' = '81' /* Drug induced subacute dyskinesia - Other hereditary and degenerative nervous system conditions */
'G2402' = '81' /* Drug induced acute dystonia - Other hereditary and degenerative nervous system conditions */
'G2409' = '81' /* Other drug induced dystonia - Other hereditary and degenerative nervous system conditions */
'G241' = '81' /* Genetic torsion dystonia - Other hereditary and degenerative nervous system conditions */
'G242' = '81' /* Idiopathic nonfamilial dystonia - Other hereditary and degenerative nervous system conditions */
'G243' = '81' /* Spasmodic torticollis - Other hereditary and degenerative nervous system conditions */
'G244' = '81' /* Idiopathic orofacial dystonia - Other hereditary and degenerative nervous system conditions */
'G245' = '81' /* Blepharospasm - Other hereditary and degenerative nervous system conditions */
'G248' = '81' /* Other dystonia - Other hereditary and degenerative nervous system conditions */
'G249' = '81' /* Dystonia, unspecified - Other hereditary and degenerative nervous system conditions */
'G250' = '81' /* Essential tremor - Other hereditary and degenerative nervous system conditions */
'G251' = '81' /* Drug-induced tremor - Other hereditary and degenerative nervous system conditions */
'G252' = '81' /* Other specified forms of tremor - Other hereditary and degenerative nervous system conditions */
'G253' = '81' /* Myoclonus - Other hereditary and degenerative nervous system conditions */
'G254' = '81' /* Drug-induced chorea - Other hereditary and degenerative nervous system conditions */
'G255' = '81' /* Other chorea - Other hereditary and degenerative nervous system conditions */
'G2561' = '81' /* Drug induced tics - Other hereditary and degenerative nervous system conditions */
'G2569' = '81' /* Other tics of organic origin - Other hereditary and degenerative nervous system conditions */
'G2570' = '81' /* Drug induced movement disorder, unspecified - Other hereditary and degenerative nervous system conditions */
'G2571' = '81' /* Drug induced akathisia - Other hereditary and degenerative nervous system conditions */
'G2579' = '81' /* Other drug induced movement disorders - Other hereditary and degenerative nervous system conditions */
'G2581' = '81' /* Restless legs syndrome - Other hereditary and degenerative nervous system conditions */
'G2582' = '81' /* Stiff-man syndrome - Other hereditary and degenerative nervous system conditions */
'G2583' = '81' /* Benign shuddering attacks - Other hereditary and degenerative nervous system conditions */
'G2589' = '81' /* Other specified extrapyramidal and movement disorders - Other hereditary and degenerative nervous system conditions */
'G259' = '81' /* Extrapyramidal and movement disorder, unspecified - Other hereditary and degenerative nervous system conditions */
'G26' = '81' /* Extrapyramidal and movement disord in diseases classd elswhr - Other hereditary and degenerative nervous system conditions */
'G312' = '81' /* Degeneration of nervous system due to alcohol - Other hereditary and degenerative nervous system conditions */
'G3181' = '81' /* Alpers disease - Other hereditary and degenerative nervous system conditions */
'G3182' = '81' /* Leigh's disease - Other hereditary and degenerative nervous system conditions */
'G3184' = '81' /* Mild cognitive impairment, so stated - Other hereditary and degenerative nervous system conditions */
'G3185' = '81' /* Corticobasal degeneration - Other hereditary and degenerative nervous system conditions */
'G3189' = '81' /* Other specified degenerative diseases of nervous system - Other hereditary and degenerative nervous system conditions */
'G319' = '81' /* Degenerative disease of nervous system, unspecified - Other hereditary and degenerative nervous system conditions */
'G320' = '81' /* Subac comb degeneration of spinal cord in dis classd elswhr - Other hereditary and degenerative nervous system conditions */
'G3281' = '81' /* Cerebellar ataxia in diseases classified elsewhere - Other hereditary and degenerative nervous system conditions */
'G803' = '81' /* Athetoid cerebral palsy - Other hereditary and degenerative nervous system conditions */
'G9001' = '81' /* Carotid sinus syncope - Other hereditary and degenerative nervous system conditions */
'G9009' = '81' /* Other idiopathic peripheral autonomic neuropathy - Other hereditary and degenerative nervous system conditions */
'G903' = '81' /* Multi-system degeneration of the autonomic nervous system - Other hereditary and degenerative nervous system conditions */
'G904' = '81' /* Autonomic dysreflexia - Other hereditary and degenerative nervous system conditions */
'G990' = '81' /* Autonomic neuropathy in diseases classified elsewhere - Other hereditary and degenerative nervous system conditions */
'G992' = '81' /* Myelopathy in diseases classified elsewhere - Other hereditary and degenerative nervous system conditions */
'G041' = '82' /* Tropical spastic paraplegia - Paralysis */
'G800' = '82' /* Spastic quadriplegic cerebral palsy - Paralysis */
'G801' = '82' /* Spastic diplegic cerebral palsy - Paralysis */
'G802' = '82' /* Spastic hemiplegic cerebral palsy - Paralysis */
'G804' = '82' /* Ataxic cerebral palsy - Paralysis */
'G808' = '82' /* Other cerebral palsy - Paralysis */
'G809' = '82' /* Cerebral palsy, unspecified - Paralysis */
'G8100' = '82' /* Flaccid hemiplegia affecting unspecified side - Paralysis */
'G8101' = '82' /* Flaccid hemiplegia affecting right dominant side - Paralysis */
'G8102' = '82' /* Flaccid hemiplegia affecting left dominant side - Paralysis */
'G8103' = '82' /* Flaccid hemiplegia affecting right nondominant side - Paralysis */
'G8104' = '82' /* Flaccid hemiplegia affecting left nondominant side - Paralysis */
'G8110' = '82' /* Spastic hemiplegia affecting unspecified side - Paralysis */
'G8111' = '82' /* Spastic hemiplegia affecting right dominant side - Paralysis */
'G8112' = '82' /* Spastic hemiplegia affecting left dominant side - Paralysis */
'G8113' = '82' /* Spastic hemiplegia affecting right nondominant side - Paralysis */
'G8114' = '82' /* Spastic hemiplegia affecting left nondominant side - Paralysis */
'G8190' = '82' /* Hemiplegia, unspecified affecting unspecified side - Paralysis */
'G8191' = '82' /* Hemiplegia, unspecified affecting right dominant side - Paralysis */
'G8192' = '82' /* Hemiplegia, unspecified affecting left dominant side - Paralysis */
'G8193' = '82' /* Hemiplegia, unspecified affecting right nondominant side - Paralysis */
'G8194' = '82' /* Hemiplegia, unspecified affecting left nondominant side - Paralysis */
'G8220' = '82' /* Paraplegia, unspecified - Paralysis */
'G8221' = '82' /* Paraplegia, complete - Paralysis */
'G8222' = '82' /* Paraplegia, incomplete - Paralysis */
'G8250' = '82' /* Quadriplegia, unspecified - Paralysis */
'G8251' = '82' /* Quadriplegia, C1-C4 complete - Paralysis */
'G8252' = '82' /* Quadriplegia, C1-C4 incomplete - Paralysis */
'G8253' = '82' /* Quadriplegia, C5-C7 complete - Paralysis */
'G8254' = '82' /* Quadriplegia, C5-C7 incomplete - Paralysis */
'G830' = '82' /* Diplegia of upper limbs - Paralysis */
'G8310' = '82' /* Monoplegia of lower limb affecting unspecified side - Paralysis */
'G8311' = '82' /* Monoplegia of lower limb affecting right dominant side - Paralysis */
'G8312' = '82' /* Monoplegia of lower limb affecting left dominant side - Paralysis */
'G8313' = '82' /* Monoplegia of lower limb affecting right nondominant side - Paralysis */
'G8314' = '82' /* Monoplegia of lower limb affecting left nondominant side - Paralysis */
'G8320' = '82' /* Monoplegia of upper limb affecting unspecified side - Paralysis */
'G8321' = '82' /* Monoplegia of upper limb affecting right dominant side - Paralysis */
'G8322' = '82' /* Monoplegia of upper limb affecting left dominant side - Paralysis */
'G8323' = '82' /* Monoplegia of upper limb affecting right nondominant side - Paralysis */
'G8324' = '82' /* Monoplegia of upper limb affecting left nondominant side - Paralysis */
'G8330' = '82' /* Monoplegia, unspecified affecting unspecified side - Paralysis */
'G8331' = '82' /* Monoplegia, unspecified affecting right dominant side - Paralysis */
'G8332' = '82' /* Monoplegia, unspecified affecting left dominant side - Paralysis */
'G8333' = '82' /* Monoplegia, unspecified affecting right nondominant side - Paralysis */
'G8334' = '82' /* Monoplegia, unspecified affecting left nondominant side - Paralysis */
'G834' = '82' /* Cauda equina syndrome - Paralysis */
'G835' = '82' /* Locked-in state - Paralysis */
'G8381' = '82' /* Brown-Sequard syndrome - Paralysis */
'G8382' = '82' /* Anterior cord syndrome - Paralysis */
'G8383' = '82' /* Posterior cord syndrome - Paralysis */
'G8384' = '82' /* Todd's paralysis (postepileptic) - Paralysis */
'G8389' = '82' /* Other specified paralytic syndromes - Paralysis */
'G839' = '82' /* Paralytic syndrome, unspecified - Paralysis */
'R295' = '82' /* Transient paralysis - Paralysis */
'R532' = '82' /* Functional quadriplegia - Paralysis */
'G40001' = '83' /* Local-rel idio epi w seiz of loc onst, not ntrct, w stat epi - Epilepsy; convulsions */
'G40009' = '83' /* Local-rel idio epi w seiz of loc onst,not ntrct,w/o stat epi - Epilepsy; convulsions */
'G40011' = '83' /* Local-rel idio epi w seiz of loc onset, ntrct, w stat epi - Epilepsy; convulsions */
'G40019' = '83' /* Local-rel idio epi w seiz of loc onset, ntrct, w/o stat epi - Epilepsy; convulsions */
'G40101' = '83' /* Local-rel symptc epi w simp part seiz, not ntrct, w stat epi - Epilepsy; convulsions */
'G40109' = '83' /* Local-rel symptc epi w simp prt seiz,not ntrct, w/o stat epi - Epilepsy; convulsions */
'G40111' = '83' /* Local-rel symptc epi w simple part seiz, ntrct, w stat epi - Epilepsy; convulsions */
'G40119' = '83' /* Local-rel symptc epi w simple part seiz, ntrct, w/o stat epi - Epilepsy; convulsions */
'G40201' = '83' /* Local-rel symptc epi w cmplx prt seiz, not ntrct, w stat epi - Epilepsy; convulsions */
'G40209' = '83' /* Local-rel symptc epi w cmplx prt seiz,not ntrct,w/o stat epi - Epilepsy; convulsions */
'G40211' = '83' /* Local-rel symptc epi w cmplx partial seiz, ntrct, w stat epi - Epilepsy; convulsions */
'G40219' = '83' /* Local-rel symptc epi w cmplx part seiz, ntrct, w/o stat epi - Epilepsy; convulsions */
'G40301' = '83' /* Gen idiopathic epilepsy, not intractable, w stat epi - Epilepsy; convulsions */
'G40309' = '83' /* Gen idiopathic epilepsy, not intractable, w/o stat epi - Epilepsy; convulsions */
'G40311' = '83' /* Generalized idiopathic epilepsy, intractable, w stat epi - Epilepsy; convulsions */
'G40319' = '83' /* Generalized idiopathic epilepsy, intractable, w/o stat epi - Epilepsy; convulsions */
'G40401' = '83' /* Oth generalized epilepsy, not intractable, w stat epi - Epilepsy; convulsions */
'G40409' = '83' /* Oth generalized epilepsy, not intractable, w/o stat epi - Epilepsy; convulsions */
'G40411' = '83' /* Oth generalized epilepsy, intractable, w status epilepticus - Epilepsy; convulsions */
'G40419' = '83' /* Oth generalized epilepsy, intractable, w/o stat epi - Epilepsy; convulsions */
'G40501' = '83' /* Epileptic seiz rel to extrn causes, not ntrct, w stat epi - Epilepsy; convulsions */
'G40509' = '83' /* Epileptic seiz rel to extrn causes, not ntrct, w/o stat epi - Epilepsy; convulsions */
'G40801' = '83' /* Other epilepsy, not intractable, with status epilepticus - Epilepsy; convulsions */
'G40802' = '83' /* Other epilepsy, not intractable, without status epilepticus - Epilepsy; convulsions */
'G40803' = '83' /* Other epilepsy, intractable, with status epilepticus - Epilepsy; convulsions */
'G40804' = '83' /* Other epilepsy, intractable, without status epilepticus - Epilepsy; convulsions */
'G40811' = '83' /* Lennox-Gastaut syndrome, not intractable, w stat epi - Epilepsy; convulsions */
'G40812' = '83' /* Lennox-Gastaut syndrome, not intractable, w/o stat epi - Epilepsy; convulsions */
'G40813' = '83' /* Lennox-Gastaut syndrome, intractable, w status epilepticus - Epilepsy; convulsions */
'G40814' = '83' /* Lennox-Gastaut syndrome, intractable, w/o status epilepticus - Epilepsy; convulsions */
'G40821' = '83' /* Epileptic spasms, not intractable, with status epilepticus - Epilepsy; convulsions */
'G40822' = '83' /* Epileptic spasms, not intractable, w/o status epilepticus - Epilepsy; convulsions */
'G40823' = '83' /* Epileptic spasms, intractable, with status epilepticus - Epilepsy; convulsions */
'G40824' = '83' /* Epileptic spasms, intractable, without status epilepticus - Epilepsy; convulsions */
'G4089' = '83' /* Other seizures - Epilepsy; convulsions */
'G40901' = '83' /* Epilepsy, unsp, not intractable, with status epilepticus - Epilepsy; convulsions */
'G40909' = '83' /* Epilepsy, unsp, not intractable, without status epilepticus - Epilepsy; convulsions */
'G40911' = '83' /* Epilepsy, unspecified, intractable, with status epilepticus - Epilepsy; convulsions */
'G40919' = '83' /* Epilepsy, unsp, intractable, without status epilepticus - Epilepsy; convulsions */
'G40A01' = '83' /* Absence epileptic syndrome, not intractable, w stat epi - Epilepsy; convulsions */
'G40A09' = '83' /* Absence epileptic syndrome, not intractable, w/o stat epi - Epilepsy; convulsions */
'G40A11' = '83' /* Absence epileptic syndrome, intractable, w stat epi - Epilepsy; convulsions */
'G40A19' = '83' /* Absence epileptic syndrome, intractable, w/o stat epi - Epilepsy; convulsions */
'G40B01' = '83' /* Juvenile myoclonic epilepsy, not intractable, w stat epi - Epilepsy; convulsions */
'G40B09' = '83' /* Juvenile myoclonic epilepsy, not intractable, w/o stat epi - Epilepsy; convulsions */
'G40B11' = '83' /* Juvenile myoclonic epilepsy, intractable, w stat epi - Epilepsy; convulsions */
'G40B19' = '83' /* Juvenile myoclonic epilepsy, intractable, w/o stat epi - Epilepsy; convulsions */
'R5600' = '83' /* Simple febrile convulsions - Epilepsy; convulsions */
'R5601' = '83' /* Complex febrile convulsions - Epilepsy; convulsions */
'R561' = '83' /* Post traumatic seizures - Epilepsy; convulsions */
'R569' = '83' /* Unspecified convulsions - Epilepsy; convulsions */
'G931' = '85' /* Anoxic brain damage, not elsewhere classified - Coma; stupor; and brain damage */
'R400' = '85' /* Somnolence - Coma; stupor; and brain damage */
'R401' = '85' /* Stupor - Coma; stupor; and brain damage */
'R4020' = '85' /* Unspecified coma - Coma; stupor; and brain damage */
'R402110' = '85' /* Coma scale, eyes open, never, unspecified time - Coma; stupor; and brain damage */
'R402111' = '85' /* Coma scale, eyes open, never, in the field - Coma; stupor; and brain damage */
'R402112' = '85' /* Coma scale, eyes open, never, EMR - Coma; stupor; and brain damage */
'R402113' = '85' /* Coma scale, eyes open, never, at hospital admission - Coma; stupor; and brain damage */
'R402114' = '85' /* Coma scale, eyes open, never, 24+hrs - Coma; stupor; and brain damage */
'R402120' = '85' /* Coma scale, eyes open, to pain, unspecified time - Coma; stupor; and brain damage */
'R402121' = '85' /* Coma scale, eyes open, to pain, in the field - Coma; stupor; and brain damage */
'R402122' = '85' /* Coma scale, eyes open, to pain, EMR - Coma; stupor; and brain damage */
'R402123' = '85' /* Coma scale, eyes open, to pain, at hospital admission - Coma; stupor; and brain damage */
'R402124' = '85' /* Coma scale, eyes open, to pain, 24+hrs - Coma; stupor; and brain damage */
'R402130' = '85' /* Coma scale, eyes open, to sound, unspecified time - Coma; stupor; and brain damage */
'R402131' = '85' /* Coma scale, eyes open, to sound, in the field - Coma; stupor; and brain damage */
'R402132' = '85' /* Coma scale, eyes open, to sound, EMR - Coma; stupor; and brain damage */
'R402133' = '85' /* Coma scale, eyes open, to sound, at hospital admission - Coma; stupor; and brain damage */
'R402134' = '85' /* Coma scale, eyes open, to sound, 24+hrs - Coma; stupor; and brain damage */
'R402140' = '85' /* Coma scale, eyes open, spontaneous, unspecified time - Coma; stupor; and brain damage */
'R402141' = '85' /* Coma scale, eyes open, spontaneous, in the field - Coma; stupor; and brain damage */
'R402142' = '85' /* Coma scale, eyes open, spontaneous, EMR - Coma; stupor; and brain damage */
'R402143' = '85' /* Coma scale, eyes open, spontaneous, at hospital admission - Coma; stupor; and brain damage */
'R402144' = '85' /* Coma scale, eyes open, spontaneous, 24+hrs - Coma; stupor; and brain damage */
'R402210' = '85' /* Coma scale, best verbal response, none, unspecified time - Coma; stupor; and brain damage */
'R402211' = '85' /* Coma scale, best verbal response, none, in the field - Coma; stupor; and brain damage */
'R402212' = '85' /* Coma scale, best verbal response, none, EMR - Coma; stupor; and brain damage */
'R402213' = '85' /* Coma scale, best verbal response, none, admit - Coma; stupor; and brain damage */
'R402214' = '85' /* Coma scale, best verbal response, none, 24+hrs - Coma; stupor; and brain damage */
'R402220' = '85' /* Coma scale, best verb, incomprehensible words, unsp time - Coma; stupor; and brain damage */
'R402221' = '85' /* Coma scale, best verb, incomprehensible words, in the field - Coma; stupor; and brain damage */
'R402222' = '85' /* Coma scale, best verb, incomprehensible words, EMR - Coma; stupor; and brain damage */
'R402223' = '85' /* Coma scale, best verb, incomprehensible words, admit - Coma; stupor; and brain damage */
'R402224' = '85' /* Coma scale, best verb, incomprehensible words, 24+hrs - Coma; stupor; and brain damage */
'R402230' = '85' /* Coma scale, best verb, inappropriate words, unsp time - Coma; stupor; and brain damage */
'R402231' = '85' /* Coma scale, best verb, inappropriate words, in the field - Coma; stupor; and brain damage */
'R402232' = '85' /* Coma scale, best verbal response, inappropriate words, EMR - Coma; stupor; and brain damage */
'R402233' = '85' /* Coma scale, best verbal response, inappropriate words, admit - Coma; stupor; and brain damage */
'R402234' = '85' /* Coma scale, best verb, inappropriate words, 24+hrs - Coma; stupor; and brain damage */
'R402240' = '85' /* Coma scale, best verb, confused conversation, unsp time - Coma; stupor; and brain damage */
'R402241' = '85' /* Coma scale, best verb, confused conversation, in the field - Coma; stupor; and brain damage */
'R402242' = '85' /* Coma scale, best verbal response, confused conversation, EMR - Coma; stupor; and brain damage */
'R402243' = '85' /* Coma scale, best verb, confused conversation, admit - Coma; stupor; and brain damage */
'R402244' = '85' /* Coma scale, best verb, confused conversation, 24+hrs - Coma; stupor; and brain damage */
'R402250' = '85' /* Coma scale, best verbal response, oriented, unspecified time - Coma; stupor; and brain damage */
'R402251' = '85' /* Coma scale, best verbal response, oriented, in the field - Coma; stupor; and brain damage */
'R402252' = '85' /* Coma scale, best verbal response, oriented, EMR - Coma; stupor; and brain damage */
'R402253' = '85' /* Coma scale, best verbal response, oriented, admit - Coma; stupor; and brain damage */
'R402254' = '85' /* Coma scale, best verbal response, oriented, 24+hrs - Coma; stupor; and brain damage */
'R402310' = '85' /* Coma scale, best motor response, none, unspecified time - Coma; stupor; and brain damage */
'R402311' = '85' /* Coma scale, best motor response, none, in the field - Coma; stupor; and brain damage */
'R402312' = '85' /* Coma scale, best motor response, none, EMR - Coma; stupor; and brain damage */
'R402313' = '85' /* Coma scale, best motor response, none, at hospital admission - Coma; stupor; and brain damage */
'R402314' = '85' /* Coma scale, best motor response, none, 24+hrs - Coma; stupor; and brain damage */
'R402320' = '85' /* Coma scale, best motor response, extension, unspecified time - Coma; stupor; and brain damage */
'R402321' = '85' /* Coma scale, best motor response, extension, in the field - Coma; stupor; and brain damage */
'R402322' = '85' /* Coma scale, best motor response, extension, EMR - Coma; stupor; and brain damage */
'R402323' = '85' /* Coma scale, best motor response, extension, admit - Coma; stupor; and brain damage */
'R402324' = '85' /* Coma scale, best motor response, extension, 24+hrs - Coma; stupor; and brain damage */
'R402330' = '85' /* Coma scale, best motor response, abnormal, unspecified time - Coma; stupor; and brain damage */
'R402331' = '85' /* Coma scale, best motor response, abnormal, in the field - Coma; stupor; and brain damage */
'R402332' = '85' /* Coma scale, best motor response, abnormal, EMR - Coma; stupor; and brain damage */
'R402333' = '85' /* Coma scale, best motor response, abnormal, admit - Coma; stupor; and brain damage */
'R402334' = '85' /* Coma scale, best motor response, abnormal, 24+hrs - Coma; stupor; and brain damage */
'R402340' = '85' /* Coma scale, best motor, flexion withdrawal, unsp time - Coma; stupor; and brain damage */
'R402341' = '85' /* Coma scale, best motor, flexion withdrawal, in the field - Coma; stupor; and brain damage */
'R402342' = '85' /* Coma scale, best motor response, flexion withdrawal, EMR - Coma; stupor; and brain damage */
'R402343' = '85' /* Coma scale, best motor response, flexion withdrawal, admit - Coma; stupor; and brain damage */
'R402344' = '85' /* Coma scale, best motor response, flexion withdrawal, 24+hrs - Coma; stupor; and brain damage */
'R402350' = '85' /* Coma scale, best motor response, localizes pain, unsp time - Coma; stupor; and brain damage */
'R402351' = '85' /* Coma scale, best motor, localizes pain, in the field - Coma; stupor; and brain damage */
'R402352' = '85' /* Coma scale, best motor response, localizes pain, EMR - Coma; stupor; and brain damage */
'R402353' = '85' /* Coma scale, best motor response, localizes pain, admit - Coma; stupor; and brain damage */
'R402354' = '85' /* Coma scale, best motor response, localizes pain, 24+hrs - Coma; stupor; and brain damage */
'R402360' = '85' /* Coma scale, best motor response, obeys commands, unsp time - Coma; stupor; and brain damage */
'R402361' = '85' /* Coma scale, best motor, obeys commands, in the field - Coma; stupor; and brain damage */
'R402362' = '85' /* Coma scale, best motor response, obeys commands, EMR - Coma; stupor; and brain damage */
'R402363' = '85' /* Coma scale, best motor response, obeys commands, admit - Coma; stupor; and brain damage */
'R402364' = '85' /* Coma scale, best motor response, obeys commands, 24+hrs - Coma; stupor; and brain damage */
'R40241' = '85' /* Glasgow coma scale score 13-15 - Coma; stupor; and brain damage */
'R40242' = '85' /* Glasgow coma scale score 9-12 - Coma; stupor; and brain damage */
'R40243' = '85' /* Glasgow coma scale score 3-8 - Coma; stupor; and brain damage */
'R40244' = '85' /* Oth coma,w/o Glasgow coma scale score,or w/part score report - Coma; stupor; and brain damage */
'R403' = '85' /* Persistent vegetative state - Coma; stupor; and brain damage */
'B2701' = '95' /* Gammaherpesviral mononucleosis with polyneuropathy - Other nervous system disorders */
'B2711' = '95' /* Cytomegaloviral mononucleosis with polyneuropathy - Other nervous system disorders */
'B2781' = '95' /* Other infectious mononucleosis with polyneuropathy - Other nervous system disorders */
'B2791' = '95' /* Infectious mononucleosis, unspecified with polyneuropathy - Other nervous system disorders */
'G08' = '95' /* Intracranial and intraspinal phlebitis and thrombophlebitis - Other nervous system disorders */
'G130' = '95' /* Paraneoplastic neuromyopathy and neuropathy - Other nervous system disorders */
'G131' = '95' /* Oth systemic atrophy aff cnsl in neoplastic disease - Other nervous system disorders */
'G132' = '95' /* Systemic atrophy primarily affecting the cnsl in myxedema - Other nervous system disorders */
'G138' = '95' /* Systemic atrophy aff cnsl in oth diseases classd elswhr - Other nervous system disorders */
'G210' = '95' /* Malignant neuroleptic syndrome - Other nervous system disorders */
'G2111' = '95' /* Neuroleptic induced parkinsonism - Other nervous system disorders */
'G2119' = '95' /* Other drug induced secondary parkinsonism - Other nervous system disorders */
'G212' = '95' /* Secondary parkinsonism due to other external agents - Other nervous system disorders */
'G213' = '95' /* Postencephalitic parkinsonism - Other nervous system disorders */
'G214' = '95' /* Vascular parkinsonism - Other nervous system disorders */
'G218' = '95' /* Other secondary parkinsonism - Other nervous system disorders */
'G219' = '95' /* Secondary parkinsonism, unspecified - Other nervous system disorders */
'G360' = '95' /* Neuromyelitis optica [Devic] - Other nervous system disorders */
'G361' = '95' /* Acute and subacute hemorrhagic leukoencephalitis [Hurst] - Other nervous system disorders */
'G368' = '95' /* Other specified acute disseminated demyelination - Other nervous system disorders */
'G369' = '95' /* Acute disseminated demyelination, unspecified - Other nervous system disorders */
'G370' = '95' /* Diffuse sclerosis of central nervous system - Other nervous system disorders */
'G371' = '95' /* Central demyelination of corpus callosum - Other nervous system disorders */
'G372' = '95' /* Central pontine myelinolysis - Other nervous system disorders */
'G373' = '95' /* Acute transverse myelitis in demyelinating disease of cnsl - Other nervous system disorders */
'G374' = '95' /* Subacute necrotizing myelitis of central nervous system - Other nervous system disorders */
'G375' = '95' /* Concentric sclerosis [Balo] of central nervous system - Other nervous system disorders */
'G378' = '95' /* Oth demyelinating diseases of central nervous system - Other nervous system disorders */
'G379' = '95' /* Demyelinating disease of central nervous system, unspecified - Other nervous system disorders */
'G4720' = '95' /* Circadian rhythm sleep disorder, unspecified type - Other nervous system disorders */
'G4721' = '95' /* Circadian rhythm sleep disorder, delayed sleep phase type - Other nervous system disorders */
'G4722' = '95' /* Circadian rhythm sleep disorder, advanced sleep phase type - Other nervous system disorders */
'G4723' = '95' /* Circadian rhythm sleep disorder, irregular sleep wake type - Other nervous system disorders */
'G4724' = '95' /* Circadian rhythm sleep disorder, free running type - Other nervous system disorders */
'G4725' = '95' /* Circadian rhythm sleep disorder, jet lag type - Other nervous system disorders */
'G4726' = '95' /* Circadian rhythm sleep disorder, shift work type - Other nervous system disorders */
'G4727' = '95' /* Circadian rhythm sleep disorder in conditions classd elswhr - Other nervous system disorders */
'G4729' = '95' /* Other circadian rhythm sleep disorder - Other nervous system disorders */
'G47411' = '95' /* Narcolepsy with cataplexy - Other nervous system disorders */
'G47419' = '95' /* Narcolepsy without cataplexy - Other nervous system disorders */
'G47421' = '95' /* Narcolepsy in conditions classified elsewhere with cataplexy - Other nervous system disorders */
'G47429' = '95' /* Narcolepsy in conditions classified elsewhere w/o cataplexy - Other nervous system disorders */
'G4763' = '95' /* Sleep related bruxism - Other nervous system disorders */
'G500' = '95' /* Trigeminal neuralgia - Other nervous system disorders */
'G501' = '95' /* Atypical facial pain - Other nervous system disorders */
'G508' = '95' /* Other disorders of trigeminal nerve - Other nervous system disorders */
'G509' = '95' /* Disorder of trigeminal nerve, unspecified - Other nervous system disorders */
'G510' = '95' /* Bell's palsy - Other nervous system disorders */
'G511' = '95' /* Geniculate ganglionitis - Other nervous system disorders */
'G512' = '95' /* Melkersson's syndrome - Other nervous system disorders */
'G513' = '95' /* Clonic hemifacial spasm - Other nervous system disorders */
'G514' = '95' /* Facial myokymia - Other nervous system disorders */
'G518' = '95' /* Other disorders of facial nerve - Other nervous system disorders */
'G519' = '95' /* Disorder of facial nerve, unspecified - Other nervous system disorders */
'G520' = '95' /* Disorders of olfactory nerve - Other nervous system disorders */
'G521' = '95' /* Disorders of glossopharyngeal nerve - Other nervous system disorders */
'G522' = '95' /* Disorders of vagus nerve - Other nervous system disorders */
'G523' = '95' /* Disorders of hypoglossal nerve - Other nervous system disorders */
'G527' = '95' /* Disorders of multiple cranial nerves - Other nervous system disorders */
'G528' = '95' /* Disorders of other specified cranial nerves - Other nervous system disorders */
'G529' = '95' /* Cranial nerve disorder, unspecified - Other nervous system disorders */
'G53' = '95' /* Cranial nerve disorders in diseases classified elsewhere - Other nervous system disorders */
'G540' = '95' /* Brachial plexus disorders - Other nervous system disorders */
'G541' = '95' /* Lumbosacral plexus disorders - Other nervous system disorders */
'G542' = '95' /* Cervical root disorders, not elsewhere classified - Other nervous system disorders */
'G543' = '95' /* Thoracic root disorders, not elsewhere classified - Other nervous system disorders */
'G544' = '95' /* Lumbosacral root disorders, not elsewhere classified - Other nervous system disorders */
'G545' = '95' /* Neuralgic amyotrophy - Other nervous system disorders */
'G546' = '95' /* Phantom limb syndrome with pain - Other nervous system disorders */
'G547' = '95' /* Phantom limb syndrome without pain - Other nervous system disorders */
'G548' = '95' /* Other nerve root and plexus disorders - Other nervous system disorders */
'G549' = '95' /* Nerve root and plexus disorder, unspecified - Other nervous system disorders */
'G55' = '95' /* Nerve root and plexus compressions in diseases classd elswhr - Other nervous system disorders */
'G5600' = '95' /* Carpal tunnel syndrome, unspecified upper limb - Other nervous system disorders */
'G5601' = '95' /* Carpal tunnel syndrome, right upper limb - Other nervous system disorders */
'G5602' = '95' /* Carpal tunnel syndrome, left upper limb - Other nervous system disorders */
'G5610' = '95' /* Other lesions of median nerve, unspecified upper limb - Other nervous system disorders */
'G5611' = '95' /* Other lesions of median nerve, right upper limb - Other nervous system disorders */
'G5612' = '95' /* Other lesions of median nerve, left upper limb - Other nervous system disorders */
'G5620' = '95' /* Lesion of ulnar nerve, unspecified upper limb - Other nervous system disorders */
'G5621' = '95' /* Lesion of ulnar nerve, right upper limb - Other nervous system disorders */
'G5622' = '95' /* Lesion of ulnar nerve, left upper limb - Other nervous system disorders */
'G5630' = '95' /* Lesion of radial nerve, unspecified upper limb - Other nervous system disorders */
'G5631' = '95' /* Lesion of radial nerve, right upper limb - Other nervous system disorders */
'G5632' = '95' /* Lesion of radial nerve, left upper limb - Other nervous system disorders */
'G5640' = '95' /* Causalgia of unspecified upper limb - Other nervous system disorders */
'G5641' = '95' /* Causalgia of right upper limb - Other nervous system disorders */
'G5642' = '95' /* Causalgia of left upper limb - Other nervous system disorders */
'G5680' = '95' /* Other specified mononeuropathies of unspecified upper limb - Other nervous system disorders */
'G5681' = '95' /* Other specified mononeuropathies of right upper limb - Other nervous system disorders */
'G5682' = '95' /* Other specified mononeuropathies of left upper limb - Other nervous system disorders */
'G5690' = '95' /* Unspecified mononeuropathy of unspecified upper limb - Other nervous system disorders */
'G5691' = '95' /* Unspecified mononeuropathy of right upper limb - Other nervous system disorders */
'G5692' = '95' /* Unspecified mononeuropathy of left upper limb - Other nervous system disorders */
'G5700' = '95' /* Lesion of sciatic nerve, unspecified lower limb - Other nervous system disorders */
'G5701' = '95' /* Lesion of sciatic nerve, right lower limb - Other nervous system disorders */
'G5702' = '95' /* Lesion of sciatic nerve, left lower limb - Other nervous system disorders */
'G5710' = '95' /* Meralgia paresthetica, unspecified lower limb - Other nervous system disorders */
'G5711' = '95' /* Meralgia paresthetica, right lower limb - Other nervous system disorders */
'G5712' = '95' /* Meralgia paresthetica, left lower limb - Other nervous system disorders */
'G5720' = '95' /* Lesion of femoral nerve, unspecified lower limb - Other nervous system disorders */
'G5721' = '95' /* Lesion of femoral nerve, right lower limb - Other nervous system disorders */
'G5722' = '95' /* Lesion of femoral nerve, left lower limb - Other nervous system disorders */
'G5730' = '95' /* Lesion of lateral popliteal nerve, unspecified lower limb - Other nervous system disorders */
'G5731' = '95' /* Lesion of lateral popliteal nerve, right lower limb - Other nervous system disorders */
'G5732' = '95' /* Lesion of lateral popliteal nerve, left lower limb - Other nervous system disorders */
'G5740' = '95' /* Lesion of medial popliteal nerve, unspecified lower limb - Other nervous system disorders */
'G5741' = '95' /* Lesion of medial popliteal nerve, right lower limb - Other nervous system disorders */
'G5742' = '95' /* Lesion of medial popliteal nerve, left lower limb - Other nervous system disorders */
'G5750' = '95' /* Tarsal tunnel syndrome, unspecified lower limb - Other nervous system disorders */
'G5751' = '95' /* Tarsal tunnel syndrome, right lower limb - Other nervous system disorders */
'G5752' = '95' /* Tarsal tunnel syndrome, left lower limb - Other nervous system disorders */
'G5760' = '95' /* Lesion of plantar nerve, unspecified lower limb - Other nervous system disorders */
'G5761' = '95' /* Lesion of plantar nerve, right lower limb - Other nervous system disorders */
'G5762' = '95' /* Lesion of plantar nerve, left lower limb - Other nervous system disorders */
'G5770' = '95' /* Causalgia of unspecified lower limb - Other nervous system disorders */
'G5771' = '95' /* Causalgia of right lower limb - Other nervous system disorders */
'G5772' = '95' /* Causalgia of left lower limb - Other nervous system disorders */
'G5780' = '95' /* Other specified mononeuropathies of unspecified lower limb - Other nervous system disorders */
'G5781' = '95' /* Other specified mononeuropathies of right lower limb - Other nervous system disorders */
'G5782' = '95' /* Other specified mononeuropathies of left lower limb - Other nervous system disorders */
'G5790' = '95' /* Unspecified mononeuropathy of unspecified lower limb - Other nervous system disorders */
'G5791' = '95' /* Unspecified mononeuropathy of right lower limb - Other nervous system disorders */
'G5792' = '95' /* Unspecified mononeuropathy of left lower limb - Other nervous system disorders */
'G580' = '95' /* Intercostal neuropathy - Other nervous system disorders */
'G587' = '95' /* Mononeuritis multiplex - Other nervous system disorders */
'G588' = '95' /* Other specified mononeuropathies - Other nervous system disorders */
'G589' = '95' /* Mononeuropathy, unspecified - Other nervous system disorders */
'G59' = '95' /* Mononeuropathy in diseases classified elsewhere - Other nervous system disorders */
'G600' = '95' /* Hereditary motor and sensory neuropathy - Other nervous system disorders */
'G601' = '95' /* Refsum's disease - Other nervous system disorders */
'G602' = '95' /* Neuropathy in association with hereditary ataxia - Other nervous system disorders */
'G603' = '95' /* Idiopathic progressive neuropathy - Other nervous system disorders */
'G608' = '95' /* Other hereditary and idiopathic neuropathies - Other nervous system disorders */
'G609' = '95' /* Hereditary and idiopathic neuropathy, unspecified - Other nervous system disorders */
'G610' = '95' /* Guillain-Barre syndrome - Other nervous system disorders */
'G611' = '95' /* Serum neuropathy - Other nervous system disorders */
'G6181' = '95' /* Chronic inflammatory demyelinating polyneuritis - Other nervous system disorders */
'G6189' = '95' /* Other inflammatory polyneuropathies - Other nervous system disorders */
'G619' = '95' /* Inflammatory polyneuropathy, unspecified - Other nervous system disorders */
'G620' = '95' /* Drug-induced polyneuropathy - Other nervous system disorders */
'G622' = '95' /* Polyneuropathy due to other toxic agents - Other nervous system disorders */
'G6281' = '95' /* Critical illness polyneuropathy - Other nervous system disorders */
'G6282' = '95' /* Radiation-induced polyneuropathy - Other nervous system disorders */
'G6289' = '95' /* Other specified polyneuropathies - Other nervous system disorders */
'G629' = '95' /* Polyneuropathy, unspecified - Other nervous system disorders */
'G63' = '95' /* Polyneuropathy in diseases classified elsewhere - Other nervous system disorders */
'G64' = '95' /* Other disorders of peripheral nervous system - Other nervous system disorders */
'G650' = '95' /* Sequelae of Guillain-Barre syndrome - Other nervous system disorders */
'G651' = '95' /* Sequelae of other inflammatory polyneuropathy - Other nervous system disorders */
'G652' = '95' /* Sequelae of toxic polyneuropathy - Other nervous system disorders */
'G7000' = '95' /* Myasthenia gravis without (acute) exacerbation - Other nervous system disorders */
'G7001' = '95' /* Myasthenia gravis with (acute) exacerbation - Other nervous system disorders */
'G701' = '95' /* Toxic myoneural disorders - Other nervous system disorders */
'G702' = '95' /* Congenital and developmental myasthenia - Other nervous system disorders */
'G7080' = '95' /* Lambert-Eaton syndrome, unspecified - Other nervous system disorders */
'G7081' = '95' /* Lambert-Eaton syndrome in disease classified elsewhere - Other nervous system disorders */
'G7089' = '95' /* Other specified myoneural disorders - Other nervous system disorders */
'G709' = '95' /* Myoneural disorder, unspecified - Other nervous system disorders */
'G710' = '95' /* Muscular dystrophy - Other nervous system disorders */
'G7111' = '95' /* Myotonic muscular dystrophy - Other nervous system disorders */
'G7112' = '95' /* Myotonia congenita - Other nervous system disorders */
'G7113' = '95' /* Myotonic chondrodystrophy - Other nervous system disorders */
'G7114' = '95' /* Drug induced myotonia - Other nervous system disorders */
'G7119' = '95' /* Other specified myotonic disorders - Other nervous system disorders */
'G712' = '95' /* Congenital myopathies - Other nervous system disorders */
'G713' = '95' /* Mitochondrial myopathy, not elsewhere classified - Other nervous system disorders */
'G718' = '95' /* Other primary disorders of muscles - Other nervous system disorders */
'G719' = '95' /* Primary disorder of muscle, unspecified - Other nervous system disorders */
'G720' = '95' /* Drug-induced myopathy - Other nervous system disorders */
'G721' = '95' /* Alcoholic myopathy - Other nervous system disorders */
'G722' = '95' /* Myopathy due to other toxic agents - Other nervous system disorders */
'G723' = '95' /* Periodic paralysis - Other nervous system disorders */
'G7241' = '95' /* Inclusion body myositis [IBM] - Other nervous system disorders */
'G7249' = '95' /* Oth inflammatory and immune myopathies, NEC - Other nervous system disorders */
'G7281' = '95' /* Critical illness myopathy - Other nervous system disorders */
'G7289' = '95' /* Other specified myopathies - Other nervous system disorders */
'G729' = '95' /* Myopathy, unspecified - Other nervous system disorders */
'G731' = '95' /* Lambert-Eaton syndrome in neoplastic disease - Other nervous system disorders */
'G733' = '95' /* Myasthenic syndromes in other diseases classified elsewhere - Other nervous system disorders */
'G737' = '95' /* Myopathy in diseases classified elsewhere - Other nervous system disorders */
'G890' = '95' /* Central pain syndrome - Other nervous system disorders */
'G8911' = '95' /* Acute pain due to trauma - Other nervous system disorders */
'G8912' = '95' /* Acute post-thoracotomy pain - Other nervous system disorders */
'G8918' = '95' /* Other acute postprocedural pain - Other nervous system disorders */
'G8921' = '95' /* Chronic pain due to trauma - Other nervous system disorders */
'G8922' = '95' /* Chronic post-thoracotomy pain - Other nervous system disorders */
'G8928' = '95' /* Other chronic postprocedural pain - Other nervous system disorders */
'G8929' = '95' /* Other chronic pain - Other nervous system disorders */
'G893' = '95' /* Neoplasm related pain (acute) (chronic) - Other nervous system disorders */
'G894' = '95' /* Chronic pain syndrome - Other nervous system disorders */
'G902' = '95' /* Horner's syndrome - Other nervous system disorders */
'G9050' = '95' /* Complex regional pain syndrome I, unspecified - Other nervous system disorders */
'G90511' = '95' /* Complex regional pain syndrome I of right upper limb - Other nervous system disorders */
'G90512' = '95' /* Complex regional pain syndrome I of left upper limb - Other nervous system disorders */
'G90513' = '95' /* Complex regional pain syndrome I of upper limb, bilateral - Other nervous system disorders */
'G90519' = '95' /* Complex regional pain syndrome I of unspecified upper limb - Other nervous system disorders */
'G90521' = '95' /* Complex regional pain syndrome I of right lower limb - Other nervous system disorders */
'G90522' = '95' /* Complex regional pain syndrome I of left lower limb - Other nervous system disorders */
'G90523' = '95' /* Complex regional pain syndrome I of lower limb, bilateral - Other nervous system disorders */
'G90529' = '95' /* Complex regional pain syndrome I of unspecified lower limb - Other nervous system disorders */
'G9059' = '95' /* Complex regional pain syndrome I of other specified site - Other nervous system disorders */
'G908' = '95' /* Other disorders of autonomic nervous system - Other nervous system disorders */
'G909' = '95' /* Disorder of the autonomic nervous system, unspecified - Other nervous system disorders */
'G910' = '95' /* Communicating hydrocephalus - Other nervous system disorders */
'G911' = '95' /* Obstructive hydrocephalus - Other nervous system disorders */
'G912' = '95' /* (Idiopathic) normal pressure hydrocephalus - Other nervous system disorders */
'G913' = '95' /* Post-traumatic hydrocephalus, unspecified - Other nervous system disorders */
'G914' = '95' /* Hydrocephalus in diseases classified elsewhere - Other nervous system disorders */
'G918' = '95' /* Other hydrocephalus - Other nervous system disorders */
'G919' = '95' /* Hydrocephalus, unspecified - Other nervous system disorders */
'G92' = '95' /* Toxic encephalopathy - Other nervous system disorders */
'G930' = '95' /* Cerebral cysts - Other nervous system disorders */
'G932' = '95' /* Benign intracranial hypertension - Other nervous system disorders */
'G9340' = '95' /* Encephalopathy, unspecified - Other nervous system disorders */
'G9341' = '95' /* Metabolic encephalopathy - Other nervous system disorders */
'G9349' = '95' /* Other encephalopathy - Other nervous system disorders */
'G935' = '95' /* Compression of brain - Other nervous system disorders */
'G936' = '95' /* Cerebral edema - Other nervous system disorders */
'G937' = '95' /* Reye's syndrome - Other nervous system disorders */
'G9381' = '95' /* Temporal sclerosis - Other nervous system disorders */
'G9382' = '95' /* Brain death - Other nervous system disorders */
'G9389' = '95' /* Other specified disorders of brain - Other nervous system disorders */
'G939' = '95' /* Disorder of brain, unspecified - Other nervous system disorders */
'G94' = '95' /* Other disorders of brain in diseases classified elsewhere - Other nervous system disorders */
'G950' = '95' /* Syringomyelia and syringobulbia - Other nervous system disorders */
'G9511' = '95' /* Acute infarction of spinal cord (embolic) (nonembolic) - Other nervous system disorders */
'G9519' = '95' /* Other vascular myelopathies - Other nervous system disorders */
'G9520' = '95' /* Unspecified cord compression - Other nervous system disorders */
'G9529' = '95' /* Other cord compression - Other nervous system disorders */
'G9581' = '95' /* Conus medullaris syndrome - Other nervous system disorders */
'G9589' = '95' /* Other specified diseases of spinal cord - Other nervous system disorders */
'G959' = '95' /* Disease of spinal cord, unspecified - Other nervous system disorders */
'G960' = '95' /* Cerebrospinal fluid leak - Other nervous system disorders */
'G9612' = '95' /* Meningeal adhesions (cerebral) (spinal) - Other nervous system disorders */
'G9619' = '95' /* Other disorders of meninges, not elsewhere classified - Other nervous system disorders */
'G968' = '95' /* Other specified disorders of central nervous system - Other nervous system disorders */
'G969' = '95' /* Disorder of central nervous system, unspecified - Other nervous system disorders */
'G980' = '95' /* Neurogenic arthritis, not elsewhere classified - Other nervous system disorders */
'G988' = '95' /* Other disorders of nervous system - Other nervous system disorders */
'G998' = '95' /* Oth disrd of nervous system in diseases classified elsewhere - Other nervous system disorders */
'J1081' = '95' /* Influenza due to oth ident influenza virus w encephalopathy - Other nervous system disorders */
'R200' = '95' /* Anesthesia of skin - Other nervous system disorders */
'R201' = '95' /* Hypoesthesia of skin - Other nervous system disorders */
'R202' = '95' /* Paresthesia of skin - Other nervous system disorders */
'R203' = '95' /* Hyperesthesia - Other nervous system disorders */
'R208' = '95' /* Other disturbances of skin sensation - Other nervous system disorders */
'R209' = '95' /* Unspecified disturbances of skin sensation - Other nervous system disorders */
'R250' = '95' /* Abnormal head movements - Other nervous system disorders */
'R251' = '95' /* Tremor, unspecified - Other nervous system disorders */
'R253' = '95' /* Fasciculation - Other nervous system disorders */
'R258' = '95' /* Other abnormal involuntary movements - Other nervous system disorders */
'R259' = '95' /* Unspecified abnormal involuntary movements - Other nervous system disorders */
'R260' = '95' /* Ataxic gait - Other nervous system disorders */
'R261' = '95' /* Paralytic gait - Other nervous system disorders */
'R262' = '95' /* Difficulty in walking, not elsewhere classified - Other nervous system disorders */
'R2681' = '95' /* Unsteadiness on feet - Other nervous system disorders */
'R2689' = '95' /* Other abnormalities of gait and mobility - Other nervous system disorders */
'R269' = '95' /* Unspecified abnormalities of gait and mobility - Other nervous system disorders */
'R270' = '95' /* Ataxia, unspecified - Other nervous system disorders */
'R278' = '95' /* Other lack of coordination - Other nervous system disorders */
'R279' = '95' /* Unspecified lack of coordination - Other nervous system disorders */
'R290' = '95' /* Tetany - Other nervous system disorders */
'R292' = '95' /* Abnormal reflex - Other nervous system disorders */
'R414' = '95' /* Neurologic neglect syndrome - Other nervous system disorders */
'R41840' = '95' /* Attention and concentration deficit - Other nervous system disorders */
'R41841' = '95' /* Cognitive communication deficit - Other nervous system disorders */
'R41842' = '95' /* Visuospatial deficit - Other nervous system disorders */
'R41843' = '95' /* Psychomotor deficit - Other nervous system disorders */
'R41844' = '95' /* Frontal lobe and executive function deficit - Other nervous system disorders */
'R4189' = '95' /* Oth symptoms and signs w cognitive functions and awareness - Other nervous system disorders */
'R430' = '95' /* Anosmia - Other nervous system disorders */
'R431' = '95' /* Parosmia - Other nervous system disorders */
'R432' = '95' /* Parageusia - Other nervous system disorders */
'R438' = '95' /* Other disturbances of smell and taste - Other nervous system disorders */
'R439' = '95' /* Unspecified disturbances of smell and taste - Other nervous system disorders */
'R4701' = '95' /* Aphasia - Other nervous system disorders */
'R4702' = '95' /* Dysphasia - Other nervous system disorders */
'R471' = '95' /* Dysarthria and anarthria - Other nervous system disorders */
'R4781' = '95' /* Slurred speech - Other nervous system disorders */
'R4782' = '95' /* Fluency disorder in conditions classified elsewhere - Other nervous system disorders */
'R4789' = '95' /* Other speech disturbances - Other nervous system disorders */
'R479' = '95' /* Unspecified speech disturbances - Other nervous system disorders */
'R481' = '95' /* Agnosia - Other nervous system disorders */
'R482' = '95' /* Apraxia - Other nervous system disorders */
'R488' = '95' /* Other symbolic dysfunctions - Other nervous system disorders */
'R489' = '95' /* Unspecified symbolic dysfunctions - Other nervous system disorders */
'R900' = '95' /* Intcrn space-occupying lesion found on dx imaging of cnsl - Other nervous system disorders */
'R9082' = '95' /* White matter disease, unspecified - Other nervous system disorders */
'Z462' = '95' /* Encntr for fit/adjst of dev rel to nrv sys and specl senses - Other nervous system disorders */
'Z86011' = '95' /* Personal history of benign neoplasm of the brain - Other nervous system disorders */
'Z8661' = '95' /* Personal history of infections of the central nervous system - Other nervous system disorders */
'Z8669' = '95' /* Personal history of dis of the nervous sys and sense organs - Other nervous system disorders */
'Z982' = '95' /* Presence of cerebrospinal fluid drainage device - Other nervous system disorders */
'I050' = '96' /* Rheumatic mitral stenosis - Heart valve disorders */
'I051' = '96' /* Rheumatic mitral insufficiency - Heart valve disorders */
'I052' = '96' /* Rheumatic mitral stenosis with insufficiency - Heart valve disorders */
'I058' = '96' /* Other rheumatic mitral valve diseases - Heart valve disorders */
'I059' = '96' /* Rheumatic mitral valve disease, unspecified - Heart valve disorders */
'I060' = '96' /* Rheumatic aortic stenosis - Heart valve disorders */
'I061' = '96' /* Rheumatic aortic insufficiency - Heart valve disorders */
'I062' = '96' /* Rheumatic aortic stenosis with insufficiency - Heart valve disorders */
'I068' = '96' /* Other rheumatic aortic valve diseases - Heart valve disorders */
'I069' = '96' /* Rheumatic aortic valve disease, unspecified - Heart valve disorders */
'I070' = '96' /* Rheumatic tricuspid stenosis - Heart valve disorders */
'I071' = '96' /* Rheumatic tricuspid insufficiency - Heart valve disorders */
'I072' = '96' /* Rheumatic tricuspid stenosis and insufficiency - Heart valve disorders */
'I078' = '96' /* Other rheumatic tricuspid valve diseases - Heart valve disorders */
'I079' = '96' /* Rheumatic tricuspid valve disease, unspecified - Heart valve disorders */
'I080' = '96' /* Rheumatic disorders of both mitral and aortic valves - Heart valve disorders */
'I081' = '96' /* Rheumatic disorders of both mitral and tricuspid valves - Heart valve disorders */
'I082' = '96' /* Rheumatic disorders of both aortic and tricuspid valves - Heart valve disorders */
'I083' = '96' /* Comb rheumatic disord of mitral, aortic and tricuspid valves - Heart valve disorders */
'I088' = '96' /* Other rheumatic multiple valve diseases - Heart valve disorders */
'I089' = '96' /* Rheumatic multiple valve disease, unspecified - Heart valve disorders */
'I091' = '96' /* Rheumatic diseases of endocardium, valve unspecified - Heart valve disorders */
'I0989' = '96' /* Other specified rheumatic heart diseases - Heart valve disorders */
'I340' = '96' /* Nonrheumatic mitral (valve) insufficiency - Heart valve disorders */
'I341' = '96' /* Nonrheumatic mitral (valve) prolapse - Heart valve disorders */
'I342' = '96' /* Nonrheumatic mitral (valve) stenosis - Heart valve disorders */
'I348' = '96' /* Other nonrheumatic mitral valve disorders - Heart valve disorders */
'I349' = '96' /* Nonrheumatic mitral valve disorder, unspecified - Heart valve disorders */
'I350' = '96' /* Nonrheumatic aortic (valve) stenosis - Heart valve disorders */
'I351' = '96' /* Nonrheumatic aortic (valve) insufficiency - Heart valve disorders */
'I352' = '96' /* Nonrheumatic aortic (valve) stenosis with insufficiency - Heart valve disorders */
'I358' = '96' /* Other nonrheumatic aortic valve disorders - Heart valve disorders */
'I359' = '96' /* Nonrheumatic aortic valve disorder, unspecified - Heart valve disorders */
'I360' = '96' /* Nonrheumatic tricuspid (valve) stenosis - Heart valve disorders */
'I361' = '96' /* Nonrheumatic tricuspid (valve) insufficiency - Heart valve disorders */
'I362' = '96' /* Nonrheumatic tricuspid (valve) stenosis with insufficiency - Heart valve disorders */
'I368' = '96' /* Other nonrheumatic tricuspid valve disorders - Heart valve disorders */
'I369' = '96' /* Nonrheumatic tricuspid valve disorder, unspecified - Heart valve disorders */
'I370' = '96' /* Nonrheumatic pulmonary valve stenosis - Heart valve disorders */
'I371' = '96' /* Nonrheumatic pulmonary valve insufficiency - Heart valve disorders */
'I372' = '96' /* Nonrheumatic pulmonary valve stenosis with insufficiency - Heart valve disorders */
'I378' = '96' /* Other nonrheumatic pulmonary valve disorders - Heart valve disorders */
'I379' = '96' /* Nonrheumatic pulmonary valve disorder, unspecified - Heart valve disorders */
'R008' = '96' /* Other abnormalities of heart beat - Heart valve disorders */
'R009' = '96' /* Unspecified abnormalities of heart beat - Heart valve disorders */
'R010' = '96' /* Benign and innocent cardiac murmurs - Heart valve disorders */
'R011' = '96' /* Cardiac murmur, unspecified - Heart valve disorders */
'R012' = '96' /* Other cardiac sounds - Heart valve disorders */
'Z952' = '96' /* Presence of prosthetic heart valve - Heart valve disorders */
'Z953' = '96' /* Presence of xenogenic heart valve - Heart valve disorders */
'Z954' = '96' /* Presence of other heart-valve replacement - Heart valve disorders */
'I10' = '98' /* Essential (primary) hypertension - Essential hypertension */
'I2601' = '103' /* Septic pulmonary embolism with acute cor pulmonale - Pulmonary heart disease */
'I2602' = '103' /* Saddle embolus of pulmonary artery with acute cor pulmonale - Pulmonary heart disease */
'I2609' = '103' /* Other pulmonary embolism with acute cor pulmonale - Pulmonary heart disease */
'I2690' = '103' /* Septic pulmonary embolism without acute cor pulmonale - Pulmonary heart disease */
'I2692' = '103' /* Saddle embolus of pulmonary artery w/o acute cor pulmonale - Pulmonary heart disease */
'I2699' = '103' /* Other pulmonary embolism without acute cor pulmonale - Pulmonary heart disease */
'I270' = '103' /* Primary pulmonary hypertension - Pulmonary heart disease */
'I271' = '103' /* Kyphoscoliotic heart disease - Pulmonary heart disease */
'I272' = '103' /* Other secondary pulmonary hypertension - Pulmonary heart disease */
'I2781' = '103' /* Cor pulmonale (chronic) - Pulmonary heart disease */
'I2782' = '103' /* Chronic pulmonary embolism - Pulmonary heart disease */
'I2789' = '103' /* Other specified pulmonary heart diseases - Pulmonary heart disease */
'I279' = '103' /* Pulmonary heart disease, unspecified - Pulmonary heart disease */
'I280' = '103' /* Arteriovenous fistula of pulmonary vessels - Pulmonary heart disease */
'I281' = '103' /* Aneurysm of pulmonary artery - Pulmonary heart disease */
'I288' = '103' /* Other diseases of pulmonary vessels - Pulmonary heart disease */
'I289' = '103' /* Disease of pulmonary vessels, unspecified - Pulmonary heart disease */
'Z86711' = '103' /* Personal history of pulmonary embolism - Pulmonary heart disease */
'I440' = '105' /* Atrioventricular block, first degree - Conduction disorders */
'I441' = '105' /* Atrioventricular block, second degree - Conduction disorders */
'I442' = '105' /* Atrioventricular block, complete - Conduction disorders */
'I4430' = '105' /* Unspecified atrioventricular block - Conduction disorders */
'I4439' = '105' /* Other atrioventricular block - Conduction disorders */
'I444' = '105' /* Left anterior fascicular block - Conduction disorders */
'I445' = '105' /* Left posterior fascicular block - Conduction disorders */
'I4460' = '105' /* Unspecified fascicular block - Conduction disorders */
'I4469' = '105' /* Other fascicular block - Conduction disorders */
'I447' = '105' /* Left bundle-branch block, unspecified - Conduction disorders */
'I450' = '105' /* Right fascicular block - Conduction disorders */
'I4510' = '105' /* Unspecified right bundle-branch block - Conduction disorders */
'I4519' = '105' /* Other right bundle-branch block - Conduction disorders */
'I452' = '105' /* Bifascicular block - Conduction disorders */
'I453' = '105' /* Trifascicular block - Conduction disorders */
'I454' = '105' /* Nonspecific intraventricular block - Conduction disorders */
'I455' = '105' /* Other specified heart block - Conduction disorders */
'I456' = '105' /* Pre-excitation syndrome - Conduction disorders */
'I4581' = '105' /* Long QT syndrome - Conduction disorders */
'I4589' = '105' /* Other specified conduction disorders - Conduction disorders */
'I459' = '105' /* Conduction disorder, unspecified - Conduction disorders */
'Z45010' = '105' /* Encntr for checking and test of card pacemaker pulse gnrtr - Conduction disorders */
'Z45018' = '105' /* Encounter for adjust and mgmt oth prt cardiac pacemaker - Conduction disorders */
'Z4502' = '105' /* Encntr for adjust and mgmt of automatic implntbl card defib - Conduction disorders */
'Z4509' = '105' /* Encounter for adjustment and management of cardiac device - Conduction disorders */
'Z950' = '105' /* Presence of cardiac pacemaker - Conduction disorders */
'Z95810' = '105' /* Presence of automatic (implantable) cardiac defibrillator - Conduction disorders */
'I470' = '106' /* Re-entry ventricular arrhythmia - Cardiac dysrhythmias */
'I471' = '106' /* Supraventricular tachycardia - Cardiac dysrhythmias */
'I472' = '106' /* Ventricular tachycardia - Cardiac dysrhythmias */
'I479' = '106' /* Paroxysmal tachycardia, unspecified - Cardiac dysrhythmias */
'I480' = '106' /* Paroxysmal atrial fibrillation - Cardiac dysrhythmias */
'I481' = '106' /* Persistent atrial fibrillation - Cardiac dysrhythmias */
'I482' = '106' /* Chronic atrial fibrillation - Cardiac dysrhythmias */
'I483' = '106' /* Typical atrial flutter - Cardiac dysrhythmias */
'I484' = '106' /* Atypical atrial flutter - Cardiac dysrhythmias */
'I4891' = '106' /* Unspecified atrial fibrillation - Cardiac dysrhythmias */
'I4892' = '106' /* Unspecified atrial flutter - Cardiac dysrhythmias */
'I491' = '106' /* Atrial premature depolarization - Cardiac dysrhythmias */
'I492' = '106' /* Junctional premature depolarization - Cardiac dysrhythmias */
'I493' = '106' /* Ventricular premature depolarization - Cardiac dysrhythmias */
'I4940' = '106' /* Unspecified premature depolarization - Cardiac dysrhythmias */
'I4949' = '106' /* Other premature depolarization - Cardiac dysrhythmias */
'I495' = '106' /* Sick sinus syndrome - Cardiac dysrhythmias */
'I498' = '106' /* Other specified cardiac arrhythmias - Cardiac dysrhythmias */
'I499' = '106' /* Cardiac arrhythmia, unspecified - Cardiac dysrhythmias */
'R000' = '106' /* Tachycardia, unspecified - Cardiac dysrhythmias */
'R001' = '106' /* Bradycardia, unspecified - Cardiac dysrhythmias */
'R002' = '106' /* Palpitations - Cardiac dysrhythmias */
'I0981' = '108' /* Rheumatic heart failure - Congestive heart failure; nonhypertensive */
'I501' = '108' /* Left ventricular failure - Congestive heart failure; nonhypertensive */
'I5020' = '108' /* Unspecified systolic (congestive) heart failure - Congestive heart failure; nonhypertensive */
'I5021' = '108' /* Acute systolic (congestive) heart failure - Congestive heart failure; nonhypertensive */
'I5022' = '108' /* Chronic systolic (congestive) heart failure - Congestive heart failure; nonhypertensive */
'I5023' = '108' /* Acute on chronic systolic (congestive) heart failure - Congestive heart failure; nonhypertensive */
'I5030' = '108' /* Unspecified diastolic (congestive) heart failure - Congestive heart failure; nonhypertensive */
'I5031' = '108' /* Acute diastolic (congestive) heart failure - Congestive heart failure; nonhypertensive */
'I5032' = '108' /* Chronic diastolic (congestive) heart failure - Congestive heart failure; nonhypertensive */
'I5033' = '108' /* Acute on chronic diastolic (congestive) heart failure - Congestive heart failure; nonhypertensive */
'I5040' = '108' /* Unsp combined systolic and diastolic (congestive) hrt fail - Congestive heart failure; nonhypertensive */
'I5041' = '108' /* Acute combined systolic and diastolic (congestive) hrt fail - Congestive heart failure; nonhypertensive */
'I5042' = '108' /* Chronic combined systolic and diastolic hrt fail - Congestive heart failure; nonhypertensive */
'I5043' = '108' /* Acute on chronic combined systolic and diastolic hrt fail - Congestive heart failure; nonhypertensive */
'I509' = '108' /* Heart failure, unspecified - Congestive heart failure; nonhypertensive */
'J182' = '133' /* Hypostatic pneumonia, unspecified organism - Other lower respiratory disease */
'J22' = '133' /* Unspecified acute lower respiratory infection - Other lower respiratory disease */
'J810' = '133' /* Acute pulmonary edema - Other lower respiratory disease */
'J811' = '133' /* Chronic pulmonary edema - Other lower respiratory disease */
'J82' = '133' /* Pulmonary eosinophilia, not elsewhere classified - Other lower respiratory disease */
'J8401' = '133' /* Alveolar proteinosis - Other lower respiratory disease */
'J8402' = '133' /* Pulmonary alveolar microlithiasis - Other lower respiratory disease */
'J8403' = '133' /* Idiopathic pulmonary hemosiderosis - Other lower respiratory disease */
'J8409' = '133' /* Other alveolar and parieto-alveolar conditions - Other lower respiratory disease */
'J8410' = '133' /* Pulmonary fibrosis, unspecified - Other lower respiratory disease */
'J84111' = '133' /* Idiopathic interstitial pneumonia, not otherwise specified - Other lower respiratory disease */
'J84112' = '133' /* Idiopathic pulmonary fibrosis - Other lower respiratory disease */
'J84113' = '133' /* Idiopathic non-specific interstitial pneumonitis - Other lower respiratory disease */
'J84114' = '133' /* Acute interstitial pneumonitis - Other lower respiratory disease */
'J84115' = '133' /* Respiratory bronchiolitis interstitial lung disease - Other lower respiratory disease */
'J84116' = '133' /* Cryptogenic organizing pneumonia - Other lower respiratory disease */
'J84117' = '133' /* Desquamative interstitial pneumonia - Other lower respiratory disease */
'J8417' = '133' /* Oth interstit pulmon dis w fibrosis in dis classd elswhr - Other lower respiratory disease */
'J842' = '133' /* Lymphoid interstitial pneumonia - Other lower respiratory disease */
'J8481' = '133' /* Lymphangioleiomyomatosis - Other lower respiratory disease */
'J8482' = '133' /* Adult pulmonary Langerhans cell histiocytosis - Other lower respiratory disease */
'J8483' = '133' /* Surfactant mutations of the lung - Other lower respiratory disease */
'J84841' = '133' /* Neuroendocrine cell hyperplasia of infancy - Other lower respiratory disease */
'J84842' = '133' /* Pulmonary interstitial glycogenosis - Other lower respiratory disease */
'J84843' = '133' /* Alveolar capillary dysplasia with vein misalignment - Other lower respiratory disease */
'J84848' = '133' /* Other interstitial  lung diseases of childhood - Other lower respiratory disease */
'J8489' = '133' /* Other specified interstitial pulmonary diseases - Other lower respiratory disease */
'J849' = '133' /* Interstitial pulmonary disease, unspecified - Other lower respiratory disease */
'J853' = '133' /* Abscess of mediastinum - Other lower respiratory disease */
'J984' = '133' /* Other disorders of lung - Other lower respiratory disease */
'J986' = '133' /* Disorders of diaphragm - Other lower respiratory disease */
'J988' = '133' /* Other specified respiratory disorders - Other lower respiratory disease */
'J989' = '133' /* Respiratory disorder, unspecified - Other lower respiratory disease */
'J99' = '133' /* Respiratory disorders in diseases classified elsewhere - Other lower respiratory disease */
'R042' = '133' /* Hemoptysis - Other lower respiratory disease */
'R0481' = '133' /* Acute idiopathic pulmonary hemorrhage in infants - Other lower respiratory disease */
'R0489' = '133' /* Hemorrhage from other sites in respiratory passages - Other lower respiratory disease */
'R049' = '133' /* Hemorrhage from respiratory passages, unspecified - Other lower respiratory disease */
'R05' = '133' /* Cough - Other lower respiratory disease */
'R0600' = '133' /* Dyspnea, unspecified - Other lower respiratory disease */
'R0601' = '133' /* Orthopnea - Other lower respiratory disease */
'R0602' = '133' /* Shortness of breath - Other lower respiratory disease */
'R0609' = '133' /* Other forms of dyspnea - Other lower respiratory disease */
'R062' = '133' /* Wheezing - Other lower respiratory disease */
'R063' = '133' /* Periodic breathing - Other lower respiratory disease */
'R064' = '133' /* Hyperventilation - Other lower respiratory disease */
'R066' = '133' /* Hiccough - Other lower respiratory disease */
'R0681' = '133' /* Apnea, not elsewhere classified - Other lower respiratory disease */
'R0682' = '133' /* Tachypnea, not elsewhere classified - Other lower respiratory disease */
'R0683' = '133' /* Snoring - Other lower respiratory disease */
'R0689' = '133' /* Other abnormalities of breathing - Other lower respiratory disease */
'R069' = '133' /* Unspecified abnormalities of breathing - Other lower respiratory disease */
'R071' = '133' /* Chest pain on breathing - Other lower respiratory disease */
'R0781' = '133' /* Pleurodynia - Other lower respiratory disease */
'R0902' = '133' /* Hypoxemia - Other lower respiratory disease */
'R093' = '133' /* Abnormal sputum - Other lower respiratory disease */
'R230' = '133' /* Cyanosis - Other lower respiratory disease */
'R911' = '133' /* Solitary pulmonary nodule - Other lower respiratory disease */
'Z8701' = '133' /* Personal history of pneumonia (recurrent) - Other lower respiratory disease */
'Z8709' = '133' /* Personal history of other diseases of the respiratory system - Other lower respiratory disease */
'Z942' = '133' /* Lung transplant status - Other lower respiratory disease */
'B5731' = '138' /* Megaesophagus in Chagas' disease - Esophageal disorders */
'I8500' = '138' /* Esophageal varices without bleeding - Esophageal disorders */
'I8510' = '138' /* Secondary esophageal varices without bleeding - Esophageal disorders */
'K200' = '138' /* Eosinophilic esophagitis - Esophageal disorders */
'K208' = '138' /* Other esophagitis - Esophageal disorders */
'K209' = '138' /* Esophagitis, unspecified - Esophageal disorders */
'K210' = '138' /* Gastro-esophageal reflux disease with esophagitis - Esophageal disorders */
'K219' = '138' /* Gastro-esophageal reflux disease without esophagitis - Esophageal disorders */
'K220' = '138' /* Achalasia of cardia - Esophageal disorders */
'K2210' = '138' /* Ulcer of esophagus without bleeding - Esophageal disorders */
'K2211' = '138' /* Ulcer of esophagus with bleeding - Esophageal disorders */
'K222' = '138' /* Esophageal obstruction - Esophageal disorders */
'K223' = '138' /* Perforation of esophagus - Esophageal disorders */
'K224' = '138' /* Dyskinesia of esophagus - Esophageal disorders */
'K225' = '138' /* Diverticulum of esophagus, acquired - Esophageal disorders */
'K226' = '138' /* Gastro-esophageal laceration-hemorrhage syndrome - Esophageal disorders */
'K2270' = '138' /* Barrett's esophagus without dysplasia - Esophageal disorders */
'K22710' = '138' /* Barrett's esophagus with low grade dysplasia - Esophageal disorders */
'K22711' = '138' /* Barrett's esophagus with high grade dysplasia - Esophageal disorders */
'K22719' = '138' /* Barrett's esophagus with dysplasia, unspecified - Esophageal disorders */
'K228' = '138' /* Other specified diseases of esophagus - Esophageal disorders */
'K229' = '138' /* Disease of esophagus, unspecified - Esophageal disorders */
'K23' = '138' /* Disorders of esophagus in diseases classified elsewhere - Esophageal disorders */
'K7041' = '151' /* Alcoholic hepatic failure with coma - Other liver diseases */
'K7110' = '151' /* Toxic liver disease with hepatic necrosis, without coma - Other liver diseases */
'K7111' = '151' /* Toxic liver disease with hepatic necrosis, with coma - Other liver diseases */
'K718' = '151' /* Toxic liver disease with other disorders of liver - Other liver diseases */
'K719' = '151' /* Toxic liver disease, unspecified - Other liver diseases */
'K7200' = '151' /* Acute and subacute hepatic failure without coma - Other liver diseases */
'K7201' = '151' /* Acute and subacute hepatic failure with coma - Other liver diseases */
'K7210' = '151' /* Chronic hepatic failure without coma - Other liver diseases */
'K7211' = '151' /* Chronic hepatic failure with coma - Other liver diseases */
'K7290' = '151' /* Hepatic failure, unspecified without coma - Other liver diseases */
'K7291' = '151' /* Hepatic failure, unspecified with coma - Other liver diseases */
'K740' = '151' /* Hepatic fibrosis - Other liver diseases */
'K741' = '151' /* Hepatic sclerosis - Other liver diseases */
'K742' = '151' /* Hepatic fibrosis with hepatic sclerosis - Other liver diseases */
'K743' = '151' /* Primary biliary cirrhosis - Other liver diseases */
'K744' = '151' /* Secondary biliary cirrhosis - Other liver diseases */
'K745' = '151' /* Biliary cirrhosis, unspecified - Other liver diseases */
'K7460' = '151' /* Unspecified cirrhosis of liver - Other liver diseases */
'K7469' = '151' /* Other cirrhosis of liver - Other liver diseases */
'K750' = '151' /* Abscess of liver - Other liver diseases */
'K751' = '151' /* Phlebitis of portal vein - Other liver diseases */
'K7589' = '151' /* Other specified inflammatory liver diseases - Other liver diseases */
'K759' = '151' /* Inflammatory liver disease, unspecified - Other liver diseases */
'K760' = '151' /* Fatty (change of) liver, not elsewhere classified - Other liver diseases */
'K761' = '151' /* Chronic passive congestion of liver - Other liver diseases */
'K762' = '151' /* Central hemorrhagic necrosis of liver - Other liver diseases */
'K763' = '151' /* Infarction of liver - Other liver diseases */
'K765' = '151' /* Hepatic veno-occlusive disease - Other liver diseases */
'K766' = '151' /* Portal hypertension - Other liver diseases */
'K767' = '151' /* Hepatorenal syndrome - Other liver diseases */
'K7681' = '151' /* Hepatopulmonary syndrome - Other liver diseases */
'K7689' = '151' /* Other specified diseases of liver - Other liver diseases */
'K769' = '151' /* Liver disease, unspecified - Other liver diseases */
'K77' = '151' /* Liver disorders in diseases classified elsewhere - Other liver diseases */
'R160' = '151' /* Hepatomegaly, not elsewhere classified - Other liver diseases */
'R162' = '151' /* Hepatomegaly with splenomegaly, not elsewhere classified - Other liver diseases */
'R17' = '151' /* Unspecified jaundice - Other liver diseases */
'R740' = '151' /* Nonspec elev of levels of transamns & lactic acid dehydrgnse - Other liver diseases */
'R748' = '151' /* Abnormal levels of other serum enzymes - Other liver diseases */
'R749' = '151' /* Abnormal serum enzyme level, unspecified - Other liver diseases */
'Z944' = '151' /* Liver transplant status - Other liver diseases */
'N181' = '158' /* Chronic kidney disease, stage 1 - Chronic kidney disease */
'N182' = '158' /* Chronic kidney disease, stage 2 (mild) - Chronic kidney disease */
'N183' = '158' /* Chronic kidney disease, stage 3 (moderate) - Chronic kidney disease */
'N184' = '158' /* Chronic kidney disease, stage 4 (severe) - Chronic kidney disease */
'N185' = '158' /* Chronic kidney disease, stage 5 - Chronic kidney disease */
'N186' = '158' /* End stage renal disease - Chronic kidney disease */
'N189' = '158' /* Chronic kidney disease, unspecified - Chronic kidney disease */
'R880' = '158' /* Cloudy (hemodialysis) (peritoneal) dialysis effluent - Chronic kidney disease */
'Z4901' = '158' /* Encounter for fit/adjst of extracorporeal dialysis catheter - Chronic kidney disease */
'Z4902' = '158' /* Encounter for fit/adjst of peritoneal dialysis catheter - Chronic kidney disease */
'Z4931' = '158' /* Encounter for adequacy testing for hemodialysis - Chronic kidney disease */
'Z4932' = '158' /* Encounter for adequacy testing for peritoneal dialysis - Chronic kidney disease */
'Z9115' = '158' /* Patient's noncompliance with renal dialysis - Chronic kidney disease */
'Z940' = '158' /* Kidney transplant status - Chronic kidney disease */
'Z992' = '158' /* Dependence on renal dialysis - Chronic kidney disease */
'N131' = '161' /* Hydronephrosis w ureteral stricture, NEC - Other diseases of kidney and ureters */
'N132' = '161' /* Hydronephrosis with renal and ureteral calculous obstruction - Other diseases of kidney and ureters */
'N1330' = '161' /* Unspecified hydronephrosis - Other diseases of kidney and ureters */
'N1339' = '161' /* Other hydronephrosis - Other diseases of kidney and ureters */
'N134' = '161' /* Hydroureter - Other diseases of kidney and ureters */
'N135' = '161' /* Crossing vessel and stricture of ureter w/o hydronephrosis - Other diseases of kidney and ureters */
'N1370' = '161' /* Vesicoureteral-reflux, unspecified - Other diseases of kidney and ureters */
'N1371' = '161' /* Vesicoureteral-reflux without reflux nephropathy - Other diseases of kidney and ureters */
'N13721' = '161' /* Vesicoureter-reflux w reflux neuropath w/o hydrourt, unil - Other diseases of kidney and ureters */
'N13722' = '161' /* Vesicoureter-reflux w reflux neuropath w/o hydrourt, bi - Other diseases of kidney and ureters */
'N13729' = '161' /* Vesicoureter-reflux w reflux nephropathy w/o hydrourt, unsp - Other diseases of kidney and ureters */
'N13731' = '161' /* Vesicoureter-reflux w reflux neuropath w hydrourt, unil - Other diseases of kidney and ureters */
'N13732' = '161' /* Vesicoureter-reflux w reflux neuropath w hydrourt, bilateral - Other diseases of kidney and ureters */
'N13739' = '161' /* Vesicoureter-reflux w reflux nephropathy w hydroureter, unsp - Other diseases of kidney and ureters */
'N138' = '161' /* Other obstructive and reflux uropathy - Other diseases of kidney and ureters */
'N159' = '161' /* Renal tubulo-interstitial disease, unspecified - Other diseases of kidney and ureters */
'N16' = '161' /* Renal tubulo-interstitial disord in diseases classd elswhr - Other diseases of kidney and ureters */
'N250' = '161' /* Renal osteodystrophy - Other diseases of kidney and ureters */
'N251' = '161' /* Nephrogenic diabetes insipidus - Other diseases of kidney and ureters */
'N2581' = '161' /* Secondary hyperparathyroidism of renal origin - Other diseases of kidney and ureters */
'N2589' = '161' /* Oth disorders resulting from impaired renal tubular function - Other diseases of kidney and ureters */
'N259' = '161' /* Disorder rslt from impaired renal tubular function, unsp - Other diseases of kidney and ureters */
'N270' = '161' /* Small kidney, unilateral - Other diseases of kidney and ureters */
'N271' = '161' /* Small kidney, bilateral - Other diseases of kidney and ureters */
'N279' = '161' /* Small kidney, unspecified - Other diseases of kidney and ureters */
'N280' = '161' /* Ischemia and infarction of kidney - Other diseases of kidney and ureters */
'N281' = '161' /* Cyst of kidney, acquired - Other diseases of kidney and ureters */
'N2881' = '161' /* Hypertrophy of kidney - Other diseases of kidney and ureters */
'N2882' = '161' /* Megaloureter - Other diseases of kidney and ureters */
'N2883' = '161' /* Nephroptosis - Other diseases of kidney and ureters */
'N2884' = '161' /* Pyelitis cystica - Other diseases of kidney and ureters */
'N2885' = '161' /* Pyeloureteritis cystica - Other diseases of kidney and ureters */
'N2886' = '161' /* Ureteritis cystica - Other diseases of kidney and ureters */
'N2889' = '161' /* Other specified disorders of kidney and ureter - Other diseases of kidney and ureters */
'N289' = '161' /* Disorder of kidney and ureter, unspecified - Other diseases of kidney and ureters */
'N29' = '161' /* Oth disorders of kidney and ureter in diseases classd elswhr - Other diseases of kidney and ureters */
'R802' = '161' /* Orthostatic proteinuria, unspecified - Other diseases of kidney and ureters */
'Z87441' = '161' /* Personal history of nephrotic syndrome - Other diseases of kidney and ureters */
'M300' = '210' /* Polyarteritis nodosa - Systemic lupus erythematosus and connective tissue disorders */
'M301' = '210' /* Polyarteritis with lung involvement [Churg-Strauss] - Systemic lupus erythematosus and connective tissue disorders */
'M302' = '210' /* Juvenile polyarteritis - Systemic lupus erythematosus and connective tissue disorders */
'M303' = '210' /* Mucocutaneous lymph node syndrome [Kawasaki] - Systemic lupus erythematosus and connective tissue disorders */
'M308' = '210' /* Other conditions related to polyarteritis nodosa - Systemic lupus erythematosus and connective tissue disorders */
'M311' = '210' /* Thrombotic microangiopathy - Systemic lupus erythematosus and connective tissue disorders */
'M312' = '210' /* Lethal midline granuloma - Systemic lupus erythematosus and connective tissue disorders */
'M3130' = '210' /* Wegener's granulomatosis without renal involvement - Systemic lupus erythematosus and connective tissue disorders */
'M3131' = '210' /* Wegener's granulomatosis with renal involvement - Systemic lupus erythematosus and connective tissue disorders */
'M314' = '210' /* Aortic arch syndrome [Takayasu] - Systemic lupus erythematosus and connective tissue disorders */
'M315' = '210' /* Giant cell arteritis with polymyalgia rheumatica - Systemic lupus erythematosus and connective tissue disorders */
'M316' = '210' /* Other giant cell arteritis - Systemic lupus erythematosus and connective tissue disorders */
'M317' = '210' /* Microscopic polyangiitis - Systemic lupus erythematosus and connective tissue disorders */
'M318' = '210' /* Other specified necrotizing vasculopathies - Systemic lupus erythematosus and connective tissue disorders */
'M319' = '210' /* Necrotizing vasculopathy, unspecified - Systemic lupus erythematosus and connective tissue disorders */
'M320' = '210' /* Drug-induced systemic lupus erythematosus - Systemic lupus erythematosus and connective tissue disorders */
'M3210' = '210' /* Systemic lupus erythematosus, organ or system involv unsp - Systemic lupus erythematosus and connective tissue disorders */
'M3211' = '210' /* Endocarditis in systemic lupus erythematosus - Systemic lupus erythematosus and connective tissue disorders */
'M3212' = '210' /* Pericarditis in systemic lupus erythematosus - Systemic lupus erythematosus and connective tissue disorders */
'M3213' = '210' /* Lung involvement in systemic lupus erythematosus - Systemic lupus erythematosus and connective tissue disorders */
'M3214' = '210' /* Glomerular disease in systemic lupus erythematosus - Systemic lupus erythematosus and connective tissue disorders */
'M3215' = '210' /* Tubulo-interstitial neuropath in sys lupus erythematosus - Systemic lupus erythematosus and connective tissue disorders */
'M3219' = '210' /* Oth organ or system involv in systemic lupus erythematosus - Systemic lupus erythematosus and connective tissue disorders */
'M328' = '210' /* Other forms of systemic lupus erythematosus - Systemic lupus erythematosus and connective tissue disorders */
'M329' = '210' /* Systemic lupus erythematosus, unspecified - Systemic lupus erythematosus and connective tissue disorders */
'M3300' = '210' /* Juvenile dermatopolymyositis, organ involvement unspecified - Systemic lupus erythematosus and connective tissue disorders */
'M3301' = '210' /* Juvenile dermatopolymyositis with respiratory involvement - Systemic lupus erythematosus and connective tissue disorders */
'M3302' = '210' /* Juvenile dermatopolymyositis with myopathy - Systemic lupus erythematosus and connective tissue disorders */
'M3309' = '210' /* Juvenile dermatopolymyositis with other organ involvement - Systemic lupus erythematosus and connective tissue disorders */
'M3310' = '210' /* Other dermatopolymyositis, organ involvement unspecified - Systemic lupus erythematosus and connective tissue disorders */
'M3311' = '210' /* Other dermatopolymyositis with respiratory involvement - Systemic lupus erythematosus and connective tissue disorders */
'M3312' = '210' /* Other dermatopolymyositis with myopathy - Systemic lupus erythematosus and connective tissue disorders */
'M3319' = '210' /* Other dermatopolymyositis with other organ involvement - Systemic lupus erythematosus and connective tissue disorders */
'M3320' = '210' /* Polymyositis, organ involvement unspecified - Systemic lupus erythematosus and connective tissue disorders */
'M3321' = '210' /* Polymyositis with respiratory involvement - Systemic lupus erythematosus and connective tissue disorders */
'M3322' = '210' /* Polymyositis with myopathy - Systemic lupus erythematosus and connective tissue disorders */
'M3329' = '210' /* Polymyositis with other organ involvement - Systemic lupus erythematosus and connective tissue disorders */
'M3390' = '210' /* Dermatopolymyositis, unsp, organ involvement unspecified - Systemic lupus erythematosus and connective tissue disorders */
'M3391' = '210' /* Dermatopolymyositis, unsp with respiratory involvement - Systemic lupus erythematosus and connective tissue disorders */
'M3392' = '210' /* Dermatopolymyositis, unspecified with myopathy - Systemic lupus erythematosus and connective tissue disorders */
'M3399' = '210' /* Dermatopolymyositis, unsp with other organ involvement - Systemic lupus erythematosus and connective tissue disorders */
'M340' = '210' /* Progressive systemic sclerosis - Systemic lupus erythematosus and connective tissue disorders */
'M341' = '210' /* CR(E)ST syndrome - Systemic lupus erythematosus and connective tissue disorders */
'M342' = '210' /* Systemic sclerosis induced by drug and chemical - Systemic lupus erythematosus and connective tissue disorders */
'M3481' = '210' /* Systemic sclerosis with lung involvement - Systemic lupus erythematosus and connective tissue disorders */
'M3482' = '210' /* Systemic sclerosis with myopathy - Systemic lupus erythematosus and connective tissue disorders */
'M3483' = '210' /* Systemic sclerosis with polyneuropathy - Systemic lupus erythematosus and connective tissue disorders */
'M3489' = '210' /* Other systemic sclerosis - Systemic lupus erythematosus and connective tissue disorders */
'M349' = '210' /* Systemic sclerosis, unspecified - Systemic lupus erythematosus and connective tissue disorders */
'M3500' = '210' /* Sicca syndrome, unspecified - Systemic lupus erythematosus and connective tissue disorders */
'M3501' = '210' /* Sicca syndrome with keratoconjunctivitis - Systemic lupus erythematosus and connective tissue disorders */
'M3502' = '210' /* Sicca syndrome with lung involvement - Systemic lupus erythematosus and connective tissue disorders */
'M3503' = '210' /* Sicca syndrome with myopathy - Systemic lupus erythematosus and connective tissue disorders */
'M3504' = '210' /* Sicca syndrome with tubulo-interstitial nephropathy - Systemic lupus erythematosus and connective tissue disorders */
'M3509' = '210' /* Sicca syndrome with other organ involvement - Systemic lupus erythematosus and connective tissue disorders */
'M351' = '210' /* Other overlap syndromes - Systemic lupus erythematosus and connective tissue disorders */
'M355' = '210' /* Multifocal fibrosclerosis - Systemic lupus erythematosus and connective tissue disorders */
'M359' = '210' /* Systemic involvement of connective tissue, unspecified - Systemic lupus erythematosus and connective tissue disorders */
'M360' = '210' /* Dermato(poly)myositis in neoplastic disease - Systemic lupus erythematosus and connective tissue disorders */
'M368' = '210' /* Systemic disord of conn tiss in oth diseases classd elswhr - Systemic lupus erythematosus and connective tissue disorders */
'G4762' = '211' /* Sleep related leg cramps - Other connective tissue disease */
'K6812' = '211' /* Psoas muscle abscess - Other connective tissue disease */
'M2420' = '211' /* Disorder of ligament, unspecified site - Other connective tissue disease */
'M24211' = '211' /* Disorder of ligament, right shoulder - Other connective tissue disease */
'M24212' = '211' /* Disorder of ligament, left shoulder - Other connective tissue disease */
'M24219' = '211' /* Disorder of ligament, unspecified shoulder - Other connective tissue disease */
'M24221' = '211' /* Disorder of ligament, right elbow - Other connective tissue disease */
'M24222' = '211' /* Disorder of ligament, left elbow - Other connective tissue disease */
'M24229' = '211' /* Disorder of ligament, unspecified elbow - Other connective tissue disease */
'M24231' = '211' /* Disorder of ligament, right wrist - Other connective tissue disease */
'M24232' = '211' /* Disorder of ligament, left wrist - Other connective tissue disease */
'M24239' = '211' /* Disorder of ligament, unspecified wrist - Other connective tissue disease */
'M24241' = '211' /* Disorder of ligament, right hand - Other connective tissue disease */
'M24242' = '211' /* Disorder of ligament, left hand - Other connective tissue disease */
'M24249' = '211' /* Disorder of ligament, unspecified hand - Other connective tissue disease */
'M24251' = '211' /* Disorder of ligament, right hip - Other connective tissue disease */
'M24252' = '211' /* Disorder of ligament, left hip - Other connective tissue disease */
'M24259' = '211' /* Disorder of ligament, unspecified hip - Other connective tissue disease */
'M24271' = '211' /* Disorder of ligament, right ankle - Other connective tissue disease */
'M24272' = '211' /* Disorder of ligament, left ankle - Other connective tissue disease */
'M24273' = '211' /* Disorder of ligament, unspecified ankle - Other connective tissue disease */
'M24274' = '211' /* Disorder of ligament, right foot - Other connective tissue disease */
'M24275' = '211' /* Disorder of ligament, left foot - Other connective tissue disease */
'M24276' = '211' /* Disorder of ligament, unspecified foot - Other connective tissue disease */
'M2428' = '211' /* Disorder of ligament, vertebrae - Other connective tissue disease */
'M352' = '211' /* Behcet's disease - Other connective tissue disease */
'M353' = '211' /* Polymyalgia rheumatica - Other connective tissue disease */
'M354' = '211' /* Diffuse (eosinophilic) fasciitis - Other connective tissue disease */
'M356' = '211' /* Relapsing panniculitis [Weber-Christian] - Other connective tissue disease */
'M357' = '211' /* Hypermobility syndrome - Other connective tissue disease */
'M358' = '211' /* Other specified systemic involvement of connective tissue - Other connective tissue disease */
'M60000' = '211' /* Infective myositis, unspecified right arm - Other connective tissue disease */
'M60001' = '211' /* Infective myositis, unspecified left arm - Other connective tissue disease */
'M60002' = '211' /* Infective myositis, unspecified arm - Other connective tissue disease */
'M60003' = '211' /* Infective myositis, unspecified right leg - Other connective tissue disease */
'M60004' = '211' /* Infective myositis, unspecified left leg - Other connective tissue disease */
'M60005' = '211' /* Infective myositis, unspecified leg - Other connective tissue disease */
'M60009' = '211' /* Infective myositis, unspecified site - Other connective tissue disease */
'M60011' = '211' /* Infective myositis, right shoulder - Other connective tissue disease */
'M60012' = '211' /* Infective myositis, left shoulder - Other connective tissue disease */
'M60019' = '211' /* Infective myositis, unspecified shoulder - Other connective tissue disease */
'M60021' = '211' /* Infective myositis, right upper arm - Other connective tissue disease */
'M60022' = '211' /* Infective myositis, left upper arm - Other connective tissue disease */
'M60029' = '211' /* Infective myositis, unspecified upper arm - Other connective tissue disease */
'M60031' = '211' /* Infective myositis, right forearm - Other connective tissue disease */
'M60032' = '211' /* Infective myositis, left forearm - Other connective tissue disease */
'M60039' = '211' /* Infective myositis, unspecified forearm - Other connective tissue disease */
'M60041' = '211' /* Infective myositis, right hand - Other connective tissue disease */
'M60042' = '211' /* Infective myositis, left hand - Other connective tissue disease */
'M60043' = '211' /* Infective myositis, unspecified hand - Other connective tissue disease */
'M60044' = '211' /* Infective myositis, right finger(s) - Other connective tissue disease */
'M60045' = '211' /* Infective myositis, left finger(s) - Other connective tissue disease */
'M60046' = '211' /* Infective myositis, unspecified finger(s) - Other connective tissue disease */
'M60051' = '211' /* Infective myositis, right thigh - Other connective tissue disease */
'M60052' = '211' /* Infective myositis, left thigh - Other connective tissue disease */
'M60059' = '211' /* Infective myositis, unspecified thigh - Other connective tissue disease */
'M60061' = '211' /* Infective myositis, right lower leg - Other connective tissue disease */
'M60062' = '211' /* Infective myositis, left lower leg - Other connective tissue disease */
'M60069' = '211' /* Infective myositis, unspecified lower leg - Other connective tissue disease */
'M60070' = '211' /* Infective myositis, right ankle - Other connective tissue disease */
'M60071' = '211' /* Infective myositis, left ankle - Other connective tissue disease */
'M60072' = '211' /* Infective myositis, unspecified ankle - Other connective tissue disease */
'M60073' = '211' /* Infective myositis, right foot - Other connective tissue disease */
'M60074' = '211' /* Infective myositis, left foot - Other connective tissue disease */
'M60075' = '211' /* Infective myositis, unspecified foot - Other connective tissue disease */
'M60076' = '211' /* Infective myositis, right toe(s) - Other connective tissue disease */
'M60077' = '211' /* Infective myositis, left toe(s) - Other connective tissue disease */
'M60078' = '211' /* Infective myositis, unspecified toe(s) - Other connective tissue disease */
'M6008' = '211' /* Infective myositis, other site - Other connective tissue disease */
'M6009' = '211' /* Infective myositis, multiple sites - Other connective tissue disease */
'M6010' = '211' /* Interstitial myositis of unspecified site - Other connective tissue disease */
'M60111' = '211' /* Interstitial myositis, right shoulder - Other connective tissue disease */
'M60112' = '211' /* Interstitial myositis, left shoulder - Other connective tissue disease */
'M60119' = '211' /* Interstitial myositis, unspecified shoulder - Other connective tissue disease */
'M60121' = '211' /* Interstitial myositis, right upper arm - Other connective tissue disease */
'M60122' = '211' /* Interstitial myositis, left upper arm - Other connective tissue disease */
'M60129' = '211' /* Interstitial myositis, unspecified upper arm - Other connective tissue disease */
'M60131' = '211' /* Interstitial myositis, right forearm - Other connective tissue disease */
'M60132' = '211' /* Interstitial myositis, left forearm - Other connective tissue disease */
'M60139' = '211' /* Interstitial myositis, unspecified forearm - Other connective tissue disease */
'M60141' = '211' /* Interstitial myositis, right hand - Other connective tissue disease */
'M60142' = '211' /* Interstitial myositis, left hand - Other connective tissue disease */
'M60149' = '211' /* Interstitial myositis, unspecified hand - Other connective tissue disease */
'M60151' = '211' /* Interstitial myositis, right thigh - Other connective tissue disease */
'M60152' = '211' /* Interstitial myositis, left thigh - Other connective tissue disease */
'M60159' = '211' /* Interstitial myositis, unspecified thigh - Other connective tissue disease */
'M60161' = '211' /* Interstitial myositis, right lower leg - Other connective tissue disease */
'M60162' = '211' /* Interstitial myositis, left lower leg - Other connective tissue disease */
'M60169' = '211' /* Interstitial myositis, unspecified lower leg - Other connective tissue disease */
'M60171' = '211' /* Interstitial myositis, right ankle and foot - Other connective tissue disease */
'M60172' = '211' /* Interstitial myositis, left ankle and foot - Other connective tissue disease */
'M60179' = '211' /* Interstitial myositis, unspecified ankle and foot - Other connective tissue disease */
'M6018' = '211' /* Interstitial myositis, other site - Other connective tissue disease */
'M6019' = '211' /* Interstitial myositis, multiple sites - Other connective tissue disease */
'M6020' = '211' /* Foreign body granuloma of soft tissue, NEC, unsp site - Other connective tissue disease */
'M60211' = '211' /* Foreign body granuloma of soft tissue, NEC, right shoulder - Other connective tissue disease */
'M60212' = '211' /* Foreign body granuloma of soft tissue, NEC, left shoulder - Other connective tissue disease */
'M60219' = '211' /* Foreign body granuloma of soft tissue, NEC, unsp shoulder - Other connective tissue disease */
'M60221' = '211' /* Foreign body granuloma of soft tissue, NEC, right upper arm - Other connective tissue disease */
'M60222' = '211' /* Foreign body granuloma of soft tissue, NEC, left upper arm - Other connective tissue disease */
'M60229' = '211' /* Foreign body granuloma of soft tissue, NEC, unsp upper arm - Other connective tissue disease */
'M60231' = '211' /* Foreign body granuloma of soft tissue, NEC, right forearm - Other connective tissue disease */
'M60232' = '211' /* Foreign body granuloma of soft tissue, NEC, left forearm - Other connective tissue disease */
'M60239' = '211' /* Foreign body granuloma of soft tissue, NEC, unsp forearm - Other connective tissue disease */
'M60241' = '211' /* Foreign body granuloma of soft tissue, NEC, right hand - Other connective tissue disease */
'M60242' = '211' /* Foreign body granuloma of soft tissue, NEC, left hand - Other connective tissue disease */
'M60249' = '211' /* Foreign body granuloma of soft tissue, NEC, unsp hand - Other connective tissue disease */
'M60251' = '211' /* Foreign body granuloma of soft tissue, NEC, right thigh - Other connective tissue disease */
'M60252' = '211' /* Foreign body granuloma of soft tissue, NEC, left thigh - Other connective tissue disease */
'M60259' = '211' /* Foreign body granuloma of soft tissue, NEC, unsp thigh - Other connective tissue disease */
'M60261' = '211' /* Foreign body granuloma of soft tissue, NEC, right lower leg - Other connective tissue disease */
'M60262' = '211' /* Foreign body granuloma of soft tissue, NEC, left lower leg - Other connective tissue disease */
'M60269' = '211' /* Foreign body granuloma of soft tissue, NEC, unsp lower leg - Other connective tissue disease */
'M60271' = '211' /* Foreign body granuloma of soft tissue, NEC, right ank/ft - Other connective tissue disease */
'M60272' = '211' /* Foreign body granuloma of soft tissue, NEC, left ank/ft - Other connective tissue disease */
'M60279' = '211' /* Foreign body granuloma of soft tissue, NEC, unsp ank/ft - Other connective tissue disease */
'M6028' = '211' /* Foreign body granuloma of soft tissue, NEC, oth site - Other connective tissue disease */
'M6080' = '211' /* Other myositis, unspecified site - Other connective tissue disease */
'M60811' = '211' /* Other myositis, right shoulder - Other connective tissue disease */
'M60812' = '211' /* Other myositis, left shoulder - Other connective tissue disease */
'M60819' = '211' /* Other myositis, unspecified shoulder - Other connective tissue disease */
'M60821' = '211' /* Other myositis, right upper arm - Other connective tissue disease */
'M60822' = '211' /* Other myositis, left upper arm - Other connective tissue disease */
'M60829' = '211' /* Other myositis, unspecified upper arm - Other connective tissue disease */
'M60831' = '211' /* Other myositis, right forearm - Other connective tissue disease */
'M60832' = '211' /* Other myositis, left forearm - Other connective tissue disease */
'M60839' = '211' /* Other myositis, unspecified forearm - Other connective tissue disease */
'M60841' = '211' /* Other myositis, right hand - Other connective tissue disease */
'M60842' = '211' /* Other myositis, left hand - Other connective tissue disease */
'M60849' = '211' /* Other myositis, unspecified hand - Other connective tissue disease */
'M60851' = '211' /* Other myositis, right thigh - Other connective tissue disease */
'M60852' = '211' /* Other myositis, left thigh - Other connective tissue disease */
'M60859' = '211' /* Other myositis, unspecified thigh - Other connective tissue disease */
'M60861' = '211' /* Other myositis, right lower leg - Other connective tissue disease */
'M60862' = '211' /* Other myositis, left lower leg - Other connective tissue disease */
'M60869' = '211' /* Other myositis, unspecified lower leg - Other connective tissue disease */
'M60871' = '211' /* Other myositis, right ankle and foot - Other connective tissue disease */
'M60872' = '211' /* Other myositis, left ankle and foot - Other connective tissue disease */
'M60879' = '211' /* Other myositis, unspecified ankle and foot - Other connective tissue disease */
'M6088' = '211' /* Other myositis, other site - Other connective tissue disease */
'M6089' = '211' /* Other myositis, multiple sites - Other connective tissue disease */
'M609' = '211' /* Myositis, unspecified - Other connective tissue disease */
'M6100' = '211' /* Myositis ossificans traumatica, unspecified site - Other connective tissue disease */
'M61011' = '211' /* Myositis ossificans traumatica, right shoulder - Other connective tissue disease */
'M61012' = '211' /* Myositis ossificans traumatica, left shoulder - Other connective tissue disease */
'M61019' = '211' /* Myositis ossificans traumatica, unspecified shoulder - Other connective tissue disease */
'M61021' = '211' /* Myositis ossificans traumatica, right upper arm - Other connective tissue disease */
'M61022' = '211' /* Myositis ossificans traumatica, left upper arm - Other connective tissue disease */
'M61029' = '211' /* Myositis ossificans traumatica, unspecified upper arm - Other connective tissue disease */
'M61031' = '211' /* Myositis ossificans traumatica, right forearm - Other connective tissue disease */
'M61032' = '211' /* Myositis ossificans traumatica, left forearm - Other connective tissue disease */
'M61039' = '211' /* Myositis ossificans traumatica, unspecified forearm - Other connective tissue disease */
'M61041' = '211' /* Myositis ossificans traumatica, right hand - Other connective tissue disease */
'M61042' = '211' /* Myositis ossificans traumatica, left hand - Other connective tissue disease */
'M61049' = '211' /* Myositis ossificans traumatica, unspecified hand - Other connective tissue disease */
'M61051' = '211' /* Myositis ossificans traumatica, right thigh - Other connective tissue disease */
'M61052' = '211' /* Myositis ossificans traumatica, left thigh - Other connective tissue disease */
'M61059' = '211' /* Myositis ossificans traumatica, unspecified thigh - Other connective tissue disease */
'M61061' = '211' /* Myositis ossificans traumatica, right lower leg - Other connective tissue disease */
'M61062' = '211' /* Myositis ossificans traumatica, left lower leg - Other connective tissue disease */
'M61069' = '211' /* Myositis ossificans traumatica, unspecified lower leg - Other connective tissue disease */
'M61071' = '211' /* Myositis ossificans traumatica, right ankle and foot - Other connective tissue disease */
'M61072' = '211' /* Myositis ossificans traumatica, left ankle and foot - Other connective tissue disease */
'M61079' = '211' /* Myositis ossificans traumatica, unspecified ankle and foot - Other connective tissue disease */
'M6108' = '211' /* Myositis ossificans traumatica, other site - Other connective tissue disease */
'M6109' = '211' /* Myositis ossificans traumatica, multiple sites - Other connective tissue disease */
'M6110' = '211' /* Myositis ossificans progressiva, unspecified site - Other connective tissue disease */
'M61111' = '211' /* Myositis ossificans progressiva, right shoulder - Other connective tissue disease */
'M61112' = '211' /* Myositis ossificans progressiva, left shoulder - Other connective tissue disease */
'M61119' = '211' /* Myositis ossificans progressiva, unspecified shoulder - Other connective tissue disease */
'M61121' = '211' /* Myositis ossificans progressiva, right upper arm - Other connective tissue disease */
'M61122' = '211' /* Myositis ossificans progressiva, left upper arm - Other connective tissue disease */
'M61129' = '211' /* Myositis ossificans progressiva, unspecified arm - Other connective tissue disease */
'M61131' = '211' /* Myositis ossificans progressiva, right forearm - Other connective tissue disease */
'M61132' = '211' /* Myositis ossificans progressiva, left forearm - Other connective tissue disease */
'M61139' = '211' /* Myositis ossificans progressiva, unspecified forearm - Other connective tissue disease */
'M61141' = '211' /* Myositis ossificans progressiva, right hand - Other connective tissue disease */
'M61142' = '211' /* Myositis ossificans progressiva, left hand - Other connective tissue disease */
'M61143' = '211' /* Myositis ossificans progressiva, unspecified hand - Other connective tissue disease */
'M61144' = '211' /* Myositis ossificans progressiva, right finger(s) - Other connective tissue disease */
'M61145' = '211' /* Myositis ossificans progressiva, left finger(s) - Other connective tissue disease */
'M61146' = '211' /* Myositis ossificans progressiva, unspecified finger(s) - Other connective tissue disease */
'M61151' = '211' /* Myositis ossificans progressiva, right thigh - Other connective tissue disease */
'M61152' = '211' /* Myositis ossificans progressiva, left thigh - Other connective tissue disease */
'M61159' = '211' /* Myositis ossificans progressiva, unspecified thigh - Other connective tissue disease */
'M61161' = '211' /* Myositis ossificans progressiva, right lower leg - Other connective tissue disease */
'M61162' = '211' /* Myositis ossificans progressiva, left lower leg - Other connective tissue disease */
'M61169' = '211' /* Myositis ossificans progressiva, unspecified lower leg - Other connective tissue disease */
'M61171' = '211' /* Myositis ossificans progressiva, right ankle - Other connective tissue disease */
'M61172' = '211' /* Myositis ossificans progressiva, left ankle - Other connective tissue disease */
'M61173' = '211' /* Myositis ossificans progressiva, unspecified ankle - Other connective tissue disease */
'M61174' = '211' /* Myositis ossificans progressiva, right foot - Other connective tissue disease */
'M61175' = '211' /* Myositis ossificans progressiva, left foot - Other connective tissue disease */
'M61176' = '211' /* Myositis ossificans progressiva, unspecified foot - Other connective tissue disease */
'M61177' = '211' /* Myositis ossificans progressiva, right toe(s) - Other connective tissue disease */
'M61178' = '211' /* Myositis ossificans progressiva, left toe(s) - Other connective tissue disease */
'M61179' = '211' /* Myositis ossificans progressiva, unspecified toe(s) - Other connective tissue disease */
'M6118' = '211' /* Myositis ossificans progressiva, other site - Other connective tissue disease */
'M6119' = '211' /* Myositis ossificans progressiva, multiple sites - Other connective tissue disease */
'M6120' = '211' /* Paralytic calcifcn and ossification of muscle, unsp site - Other connective tissue disease */
'M61211' = '211' /* Paralytic calcifcn and ossification of muscle, r shoulder - Other connective tissue disease */
'M61212' = '211' /* Paralytic calcifcn and ossification of muscle, left shoulder - Other connective tissue disease */
'M61219' = '211' /* Paralytic calcifcn and ossification of muscle, unsp shoulder - Other connective tissue disease */
'M61221' = '211' /* Paralytic calcification and ossification of muscle, r up arm - Other connective tissue disease */
'M61222' = '211' /* Paralytic calcification and ossification of muscle, l up arm - Other connective tissue disease */
'M61229' = '211' /* Paralytic calcifcn and ossifictn of muscle, unsp upper arm - Other connective tissue disease */
'M61231' = '211' /* Paralytic calcifcn and ossification of muscle, right forearm - Other connective tissue disease */
'M61232' = '211' /* Paralytic calcifcn and ossification of muscle, left forearm - Other connective tissue disease */
'M61239' = '211' /* Paralytic calcifcn and ossification of muscle, unsp forearm - Other connective tissue disease */
'M61241' = '211' /* Paralytic calcifcn and ossification of muscle, right hand - Other connective tissue disease */
'M61242' = '211' /* Paralytic calcifcn and ossification of muscle, left hand - Other connective tissue disease */
'M61249' = '211' /* Paralytic calcifcn and ossification of muscle, unsp hand - Other connective tissue disease */
'M61251' = '211' /* Paralytic calcifcn and ossification of muscle, right thigh - Other connective tissue disease */
'M61252' = '211' /* Paralytic calcifcn and ossification of muscle, left thigh - Other connective tissue disease */
'M61259' = '211' /* Paralytic calcifcn and ossification of muscle, unsp thigh - Other connective tissue disease */
'M61261' = '211' /* Paralytic calcifcn and ossification of muscle, r low leg - Other connective tissue disease */
'M61262' = '211' /* Paralytic calcifcn and ossification of muscle, l low leg - Other connective tissue disease */
'M61269' = '211' /* Paralytic calcifcn and ossifictn of muscle, unsp lower leg - Other connective tissue disease */
'M61271' = '211' /* Paralytic calcifcn and ossification of muscle, right ank/ft - Other connective tissue disease */
'M61272' = '211' /* Paralytic calcifcn and ossification of muscle, left ank/ft - Other connective tissue disease */
'M61279' = '211' /* Paralytic calcifcn and ossification of muscle, unsp ank/ft - Other connective tissue disease */
'M6128' = '211' /* Paralytic calcification and ossification of muscle, oth site - Other connective tissue disease */
'M6129' = '211' /* Paralytic calcifcn and ossifictn of muscle, multiple sites - Other connective tissue disease */
'M6130' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, unsp site - Other connective tissue disease */
'M61311' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, r shoulder - Other connective tissue disease */
'M61312' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, l shoulder - Other connective tissue disease */
'M61319' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, unsp shldr - Other connective tissue disease */
'M61321' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, r up arm - Other connective tissue disease */
'M61322' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, l up arm - Other connective tissue disease */
'M61329' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, unsp up arm - Other connective tissue disease */
'M61331' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, r forearm - Other connective tissue disease */
'M61332' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, l forearm - Other connective tissue disease */
'M61339' = '211' /* Calcifcn and ossifictn of musc assoc w burns, unsp forearm - Other connective tissue disease */
'M61341' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, right hand - Other connective tissue disease */
'M61342' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, left hand - Other connective tissue disease */
'M61349' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, unsp hand - Other connective tissue disease */
'M61351' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, right thigh - Other connective tissue disease */
'M61352' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, left thigh - Other connective tissue disease */
'M61359' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, unsp thigh - Other connective tissue disease */
'M61361' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, r low leg - Other connective tissue disease */
'M61362' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, l low leg - Other connective tissue disease */
'M61369' = '211' /* Calcifcn and ossifictn of musc assoc w burns, unsp low leg - Other connective tissue disease */
'M61371' = '211' /* Calcifcn and ossifictn of musc assoc w burns, right ank/ft - Other connective tissue disease */
'M61372' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, left ank/ft - Other connective tissue disease */
'M61379' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, unsp ank/ft - Other connective tissue disease */
'M6138' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, oth site - Other connective tissue disease */
'M6139' = '211' /* Calcifcn and ossifictn of muscles assoc w burns, mult sites - Other connective tissue disease */
'M6140' = '211' /* Other calcification of muscle, unspecified site - Other connective tissue disease */
'M61411' = '211' /* Other calcification of muscle, right shoulder - Other connective tissue disease */
'M61412' = '211' /* Other calcification of muscle, left shoulder - Other connective tissue disease */
'M61419' = '211' /* Other calcification of muscle, unspecified shoulder - Other connective tissue disease */
'M61421' = '211' /* Other calcification of muscle, right upper arm - Other connective tissue disease */
'M61422' = '211' /* Other calcification of muscle, left upper arm - Other connective tissue disease */
'M61429' = '211' /* Other calcification of muscle, unspecified upper arm - Other connective tissue disease */
'M61431' = '211' /* Other calcification of muscle, right forearm - Other connective tissue disease */
'M61432' = '211' /* Other calcification of muscle, left forearm - Other connective tissue disease */
'M61439' = '211' /* Other calcification of muscle, unspecified forearm - Other connective tissue disease */
'M61441' = '211' /* Other calcification of muscle, right hand - Other connective tissue disease */
'M61442' = '211' /* Other calcification of muscle, left hand - Other connective tissue disease */
'M61449' = '211' /* Other calcification of muscle, unspecified hand - Other connective tissue disease */
'M61451' = '211' /* Other calcification of muscle, right thigh - Other connective tissue disease */
'M61452' = '211' /* Other calcification of muscle, left thigh - Other connective tissue disease */
'M61459' = '211' /* Other calcification of muscle, unspecified thigh - Other connective tissue disease */
'M61461' = '211' /* Other calcification of muscle, right lower leg - Other connective tissue disease */
'M61462' = '211' /* Other calcification of muscle, left lower leg - Other connective tissue disease */
'M61469' = '211' /* Other calcification of muscle, unspecified lower leg - Other connective tissue disease */
'M61471' = '211' /* Other calcification of muscle, right ankle and foot - Other connective tissue disease */
'M61472' = '211' /* Other calcification of muscle, left ankle and foot - Other connective tissue disease */
'M61479' = '211' /* Other calcification of muscle, unspecified ankle and foot - Other connective tissue disease */
'M6148' = '211' /* Other calcification of muscle, other site - Other connective tissue disease */
'M6149' = '211' /* Other calcification of muscle, multiple sites - Other connective tissue disease */
'M6150' = '211' /* Other ossification of muscle, unspecified site - Other connective tissue disease */
'M61511' = '211' /* Other ossification of muscle, right shoulder - Other connective tissue disease */
'M61512' = '211' /* Other ossification of muscle, left shoulder - Other connective tissue disease */
'M61519' = '211' /* Other ossification of muscle, unspecified shoulder - Other connective tissue disease */
'M61521' = '211' /* Other ossification of muscle, right upper arm - Other connective tissue disease */
'M61522' = '211' /* Other ossification of muscle, left upper arm - Other connective tissue disease */
'M61529' = '211' /* Other ossification of muscle, unspecified upper arm - Other connective tissue disease */
'M61531' = '211' /* Other ossification of muscle, right forearm - Other connective tissue disease */
'M61532' = '211' /* Other ossification of muscle, left forearm - Other connective tissue disease */
'M61539' = '211' /* Other ossification of muscle, unspecified forearm - Other connective tissue disease */
'M61541' = '211' /* Other ossification of muscle, right hand - Other connective tissue disease */
'M61542' = '211' /* Other ossification of muscle, left hand - Other connective tissue disease */
'M61549' = '211' /* Other ossification of muscle, unspecified hand - Other connective tissue disease */
'M61551' = '211' /* Other ossification of muscle, right thigh - Other connective tissue disease */
'M61552' = '211' /* Other ossification of muscle, left thigh - Other connective tissue disease */
'M61559' = '211' /* Other ossification of muscle, unspecified thigh - Other connective tissue disease */
'M61561' = '211' /* Other ossification of muscle, right lower leg - Other connective tissue disease */
'M61562' = '211' /* Other ossification of muscle, left lower leg - Other connective tissue disease */
'M61569' = '211' /* Other ossification of muscle, unspecified lower leg - Other connective tissue disease */
'M61571' = '211' /* Other ossification of muscle, right ankle and foot - Other connective tissue disease */
'M61572' = '211' /* Other ossification of muscle, left ankle and foot - Other connective tissue disease */
'M61579' = '211' /* Other ossification of muscle, unspecified ankle and foot - Other connective tissue disease */
'M6158' = '211' /* Other ossification of muscle, other site - Other connective tissue disease */
'M6159' = '211' /* Other ossification of muscle, multiple sites - Other connective tissue disease */
'M619' = '211' /* Calcification and ossification of muscle, unspecified - Other connective tissue disease */
'M6200' = '211' /* Separation of muscle (nontraumatic), unspecified site - Other connective tissue disease */
'M62011' = '211' /* Separation of muscle (nontraumatic), right shoulder - Other connective tissue disease */
'M62012' = '211' /* Separation of muscle (nontraumatic), left shoulder - Other connective tissue disease */
'M62019' = '211' /* Separation of muscle (nontraumatic), unspecified shoulder - Other connective tissue disease */
'M62021' = '211' /* Separation of muscle (nontraumatic), right upper arm - Other connective tissue disease */
'M62022' = '211' /* Separation of muscle (nontraumatic), left upper arm - Other connective tissue disease */
'M62029' = '211' /* Separation of muscle (nontraumatic), unspecified upper arm - Other connective tissue disease */
'M62031' = '211' /* Separation of muscle (nontraumatic), right forearm - Other connective tissue disease */
'M62032' = '211' /* Separation of muscle (nontraumatic), left forearm - Other connective tissue disease */
'M62039' = '211' /* Separation of muscle (nontraumatic), unspecified forearm - Other connective tissue disease */
'M62041' = '211' /* Separation of muscle (nontraumatic), right hand - Other connective tissue disease */
'M62042' = '211' /* Separation of muscle (nontraumatic), left hand - Other connective tissue disease */
'M62049' = '211' /* Separation of muscle (nontraumatic), unspecified hand - Other connective tissue disease */
'M62051' = '211' /* Separation of muscle (nontraumatic), right thigh - Other connective tissue disease */
'M62052' = '211' /* Separation of muscle (nontraumatic), left thigh - Other connective tissue disease */
'M62059' = '211' /* Separation of muscle (nontraumatic), unspecified thigh - Other connective tissue disease */
'M62061' = '211' /* Separation of muscle (nontraumatic), right lower leg - Other connective tissue disease */
'M62062' = '211' /* Separation of muscle (nontraumatic), left lower leg - Other connective tissue disease */
'M62069' = '211' /* Separation of muscle (nontraumatic), unspecified lower leg - Other connective tissue disease */
'M62071' = '211' /* Separation of muscle (nontraumatic), right ankle and foot - Other connective tissue disease */
'M62072' = '211' /* Separation of muscle (nontraumatic), left ankle and foot - Other connective tissue disease */
'M62079' = '211' /* Separation of muscle (nontraumatic), unsp ankle and foot - Other connective tissue disease */
'M6208' = '211' /* Separation of muscle (nontraumatic), other site - Other connective tissue disease */
'M6210' = '211' /* Other rupture of muscle (nontraumatic), unspecified site - Other connective tissue disease */
'M62111' = '211' /* Other rupture of muscle (nontraumatic), right shoulder - Other connective tissue disease */
'M62112' = '211' /* Other rupture of muscle (nontraumatic), left shoulder - Other connective tissue disease */
'M62119' = '211' /* Other rupture of muscle (nontraumatic), unspecified shoulder - Other connective tissue disease */
'M62121' = '211' /* Other rupture of muscle (nontraumatic), right upper arm - Other connective tissue disease */
'M62122' = '211' /* Other rupture of muscle (nontraumatic), left upper arm - Other connective tissue disease */
'M62129' = '211' /* Other rupture of muscle (nontraumatic), unsp upper arm - Other connective tissue disease */
'M62131' = '211' /* Other rupture of muscle (nontraumatic), right forearm - Other connective tissue disease */
'M62132' = '211' /* Other rupture of muscle (nontraumatic), left forearm - Other connective tissue disease */
'M62139' = '211' /* Other rupture of muscle (nontraumatic), unspecified forearm - Other connective tissue disease */
'M62141' = '211' /* Other rupture of muscle (nontraumatic), right hand - Other connective tissue disease */
'M62142' = '211' /* Other rupture of muscle (nontraumatic), left hand - Other connective tissue disease */
'M62149' = '211' /* Other rupture of muscle (nontraumatic), unspecified hand - Other connective tissue disease */
'M62151' = '211' /* Other rupture of muscle (nontraumatic), right thigh - Other connective tissue disease */
'M62152' = '211' /* Other rupture of muscle (nontraumatic), left thigh - Other connective tissue disease */
'M62159' = '211' /* Other rupture of muscle (nontraumatic), unspecified thigh - Other connective tissue disease */
'M62161' = '211' /* Other rupture of muscle (nontraumatic), right lower leg - Other connective tissue disease */
'M62162' = '211' /* Other rupture of muscle (nontraumatic), left lower leg - Other connective tissue disease */
'M62169' = '211' /* Other rupture of muscle (nontraumatic), unsp lower leg - Other connective tissue disease */
'M62171' = '211' /* Other rupture of muscle (nontraumatic), right ankle and foot - Other connective tissue disease */
'M62172' = '211' /* Other rupture of muscle (nontraumatic), left ankle and foot - Other connective tissue disease */
'M62179' = '211' /* Other rupture of muscle (nontraumatic), unsp ankle and foot - Other connective tissue disease */
'M6218' = '211' /* Other rupture of muscle (nontraumatic), other site - Other connective tissue disease */
'M6220' = '211' /* Nontraumatic ischemic infarction of muscle, unspecified site - Other connective tissue disease */
'M62211' = '211' /* Nontraumatic ischemic infarction of muscle, right shoulder - Other connective tissue disease */
'M62212' = '211' /* Nontraumatic ischemic infarction of muscle, left shoulder - Other connective tissue disease */
'M62219' = '211' /* Nontraumatic ischemic infarction of muscle, unsp shoulder - Other connective tissue disease */
'M62221' = '211' /* Nontraumatic ischemic infarction of muscle, right upper arm - Other connective tissue disease */
'M62222' = '211' /* Nontraumatic ischemic infarction of muscle, left upper arm - Other connective tissue disease */
'M62229' = '211' /* Nontraumatic ischemic infarction of muscle, unsp upper arm - Other connective tissue disease */
'M62231' = '211' /* Nontraumatic ischemic infarction of muscle, right forearm - Other connective tissue disease */
'M62232' = '211' /* Nontraumatic ischemic infarction of muscle, left forearm - Other connective tissue disease */
'M62239' = '211' /* Nontraumatic ischemic infarction of muscle, unsp forearm - Other connective tissue disease */
'M62241' = '211' /* Nontraumatic ischemic infarction of muscle, right hand - Other connective tissue disease */
'M62242' = '211' /* Nontraumatic ischemic infarction of muscle, left hand - Other connective tissue disease */
'M62249' = '211' /* Nontraumatic ischemic infarction of muscle, unspecified hand - Other connective tissue disease */
'M62251' = '211' /* Nontraumatic ischemic infarction of muscle, right thigh - Other connective tissue disease */
'M62252' = '211' /* Nontraumatic ischemic infarction of muscle, left thigh - Other connective tissue disease */
'M62259' = '211' /* Nontraumatic ischemic infarction of muscle, unsp thigh - Other connective tissue disease */
'M62261' = '211' /* Nontraumatic ischemic infarction of muscle, right lower leg - Other connective tissue disease */
'M62262' = '211' /* Nontraumatic ischemic infarction of muscle, left lower leg - Other connective tissue disease */
'M62269' = '211' /* Nontraumatic ischemic infarction of muscle, unsp lower leg - Other connective tissue disease */
'M62271' = '211' /* Nontraumatic ischemic infarction of muscle, right ank/ft - Other connective tissue disease */
'M62272' = '211' /* Nontraumatic ischemic infarction of muscle, left ank/ft - Other connective tissue disease */
'M62279' = '211' /* Nontraumatic ischemic infarction of muscle, unsp ank/ft - Other connective tissue disease */
'M6228' = '211' /* Nontraumatic ischemic infarction of muscle, other site - Other connective tissue disease */
'M623' = '211' /* Immobility syndrome (paraplegic) - Other connective tissue disease */
'M6240' = '211' /* Contracture of muscle, unspecified site - Other connective tissue disease */
'M62411' = '211' /* Contracture of muscle, right shoulder - Other connective tissue disease */
'M62412' = '211' /* Contracture of muscle, left shoulder - Other connective tissue disease */
'M62419' = '211' /* Contracture of muscle, unspecified shoulder - Other connective tissue disease */
'M62421' = '211' /* Contracture of muscle, right upper arm - Other connective tissue disease */
'M62422' = '211' /* Contracture of muscle, left upper arm - Other connective tissue disease */
'M62429' = '211' /* Contracture of muscle, unspecified upper arm - Other connective tissue disease */
'M62431' = '211' /* Contracture of muscle, right forearm - Other connective tissue disease */
'M62432' = '211' /* Contracture of muscle, left forearm - Other connective tissue disease */
'M62439' = '211' /* Contracture of muscle, unspecified forearm - Other connective tissue disease */
'M62441' = '211' /* Contracture of muscle, right hand - Other connective tissue disease */
'M62442' = '211' /* Contracture of muscle, left hand - Other connective tissue disease */
'M62449' = '211' /* Contracture of muscle, unspecified hand - Other connective tissue disease */
'M62451' = '211' /* Contracture of muscle, right thigh - Other connective tissue disease */
'M62452' = '211' /* Contracture of muscle, left thigh - Other connective tissue disease */
'M62459' = '211' /* Contracture of muscle, unspecified thigh - Other connective tissue disease */
'M62461' = '211' /* Contracture of muscle, right lower leg - Other connective tissue disease */
'M62462' = '211' /* Contracture of muscle, left lower leg - Other connective tissue disease */
'M62469' = '211' /* Contracture of muscle, unspecified lower leg - Other connective tissue disease */
'M62471' = '211' /* Contracture of muscle, right ankle and foot - Other connective tissue disease */
'M62472' = '211' /* Contracture of muscle, left ankle and foot - Other connective tissue disease */
'M62479' = '211' /* Contracture of muscle, unspecified ankle and foot - Other connective tissue disease */
'M6248' = '211' /* Contracture of muscle, other site - Other connective tissue disease */
'M6249' = '211' /* Contracture of muscle, multiple sites - Other connective tissue disease */
'M6250' = '211' /* Muscle wasting and atrophy, NEC, unsp site - Other connective tissue disease */
'M62511' = '211' /* Muscle wasting and atrophy, NEC, right shoulder - Other connective tissue disease */
'M62512' = '211' /* Muscle wasting and atrophy, NEC, left shoulder - Other connective tissue disease */
'M62519' = '211' /* Muscle wasting and atrophy, NEC, unsp shoulder - Other connective tissue disease */
'M62521' = '211' /* Muscle wasting and atrophy, NEC, right upper arm - Other connective tissue disease */
'M62522' = '211' /* Muscle wasting and atrophy, NEC, left upper arm - Other connective tissue disease */
'M62529' = '211' /* Muscle wasting and atrophy, NEC, unsp upper arm - Other connective tissue disease */
'M62531' = '211' /* Muscle wasting and atrophy, NEC, right forearm - Other connective tissue disease */
'M62532' = '211' /* Muscle wasting and atrophy, NEC, left forearm - Other connective tissue disease */
'M62539' = '211' /* Muscle wasting and atrophy, NEC, unsp forearm - Other connective tissue disease */
'M62541' = '211' /* Muscle wasting and atrophy, NEC, right hand - Other connective tissue disease */
'M62542' = '211' /* Muscle wasting and atrophy, NEC, left hand - Other connective tissue disease */
'M62549' = '211' /* Muscle wasting and atrophy, NEC, unsp hand - Other connective tissue disease */
'M62551' = '211' /* Muscle wasting and atrophy, NEC, right thigh - Other connective tissue disease */
'M62552' = '211' /* Muscle wasting and atrophy, NEC, left thigh - Other connective tissue disease */
'M62559' = '211' /* Muscle wasting and atrophy, NEC, unsp thigh - Other connective tissue disease */
'M62561' = '211' /* Muscle wasting and atrophy, NEC, right lower leg - Other connective tissue disease */
'M62562' = '211' /* Muscle wasting and atrophy, NEC, left lower leg - Other connective tissue disease */
'M62569' = '211' /* Muscle wasting and atrophy, NEC, unsp lower leg - Other connective tissue disease */
'M62571' = '211' /* Muscle wasting and atrophy, NEC, right ankle and foot - Other connective tissue disease */
'M62572' = '211' /* Muscle wasting and atrophy, NEC, left ankle and foot - Other connective tissue disease */
'M62579' = '211' /* Muscle wasting and atrophy, NEC, unsp ankle and foot - Other connective tissue disease */
'M6258' = '211' /* Muscle wasting and atrophy, NEC, oth site - Other connective tissue disease */
'M6259' = '211' /* Muscle wasting and atrophy, NEC, multiple sites - Other connective tissue disease */
'M6281' = '211' /* Muscle weakness (generalized) - Other connective tissue disease */
'M6282' = '211' /* Rhabdomyolysis - Other connective tissue disease */
'M62831' = '211' /* Muscle spasm of calf - Other connective tissue disease */
'M62838' = '211' /* Other muscle spasm - Other connective tissue disease */
'M6289' = '211' /* Other specified disorders of muscle - Other connective tissue disease */
'M629' = '211' /* Disorder of muscle, unspecified - Other connective tissue disease */
'M6380' = '211' /* Disorders of muscle in diseases classd elswhr, unsp site - Other connective tissue disease */
'M63811' = '211' /* Disorders of muscle in diseases classd elswhr, r shoulder - Other connective tissue disease */
'M63812' = '211' /* Disorders of muscle in diseases classd elswhr, left shoulder - Other connective tissue disease */
'M63819' = '211' /* Disorders of muscle in diseases classd elswhr, unsp shoulder - Other connective tissue disease */
'M63821' = '211' /* Disorders of muscle in diseases classd elswhr, r up arm - Other connective tissue disease */
'M63822' = '211' /* Disorders of muscle in diseases classd elswhr, l up arm - Other connective tissue disease */
'M63829' = '211' /* Disord of muscle in diseases classd elswhr, unsp upper arm - Other connective tissue disease */
'M63831' = '211' /* Disorders of muscle in diseases classd elswhr, right forearm - Other connective tissue disease */
'M63832' = '211' /* Disorders of muscle in diseases classd elswhr, left forearm - Other connective tissue disease */
'M63839' = '211' /* Disorders of muscle in diseases classd elswhr, unsp forearm - Other connective tissue disease */
'M63841' = '211' /* Disorders of muscle in diseases classd elswhr, right hand - Other connective tissue disease */
'M63842' = '211' /* Disorders of muscle in diseases classd elswhr, left hand - Other connective tissue disease */
'M63849' = '211' /* Disorders of muscle in diseases classd elswhr, unsp hand - Other connective tissue disease */
'M63851' = '211' /* Disorders of muscle in diseases classd elswhr, right thigh - Other connective tissue disease */
'M63852' = '211' /* Disorders of muscle in diseases classd elswhr, left thigh - Other connective tissue disease */
'M63859' = '211' /* Disorders of muscle in diseases classd elswhr, unsp thigh - Other connective tissue disease */
'M63861' = '211' /* Disorders of muscle in diseases classd elswhr, r low leg - Other connective tissue disease */
'M63862' = '211' /* Disorders of muscle in diseases classd elswhr, l low leg - Other connective tissue disease */
'M63869' = '211' /* Disord of muscle in diseases classd elswhr, unsp lower leg - Other connective tissue disease */
'M63871' = '211' /* Disorders of muscle in diseases classd elswhr, right ank/ft - Other connective tissue disease */
'M63872' = '211' /* Disorders of muscle in diseases classd elswhr, left ank/ft - Other connective tissue disease */
'M63879' = '211' /* Disorders of muscle in diseases classd elswhr, unsp ank/ft - Other connective tissue disease */
'M6388' = '211' /* Disorders of muscle in diseases classd elswhr, oth site - Other connective tissue disease */
'M6389' = '211' /* Disord of muscle in diseases classd elswhr, multiple sites - Other connective tissue disease */
'M6500' = '211' /* Abscess of tendon sheath, unspecified site - Other connective tissue disease */
'M65011' = '211' /* Abscess of tendon sheath, right shoulder - Other connective tissue disease */
'M65012' = '211' /* Abscess of tendon sheath, left shoulder - Other connective tissue disease */
'M65019' = '211' /* Abscess of tendon sheath, unspecified shoulder - Other connective tissue disease */
'M65021' = '211' /* Abscess of tendon sheath, right upper arm - Other connective tissue disease */
'M65022' = '211' /* Abscess of tendon sheath, left upper arm - Other connective tissue disease */
'M65029' = '211' /* Abscess of tendon sheath, unspecified upper arm - Other connective tissue disease */
'M65031' = '211' /* Abscess of tendon sheath, right forearm - Other connective tissue disease */
'M65032' = '211' /* Abscess of tendon sheath, left forearm - Other connective tissue disease */
'M65039' = '211' /* Abscess of tendon sheath, unspecified forearm - Other connective tissue disease */
'M65041' = '211' /* Abscess of tendon sheath, right hand - Other connective tissue disease */
'M65042' = '211' /* Abscess of tendon sheath, left hand - Other connective tissue disease */
'M65049' = '211' /* Abscess of tendon sheath, unspecified hand - Other connective tissue disease */
'M65051' = '211' /* Abscess of tendon sheath, right thigh - Other connective tissue disease */
'M65052' = '211' /* Abscess of tendon sheath, left thigh - Other connective tissue disease */
'M65059' = '211' /* Abscess of tendon sheath, unspecified thigh - Other connective tissue disease */
'M65061' = '211' /* Abscess of tendon sheath, right lower leg - Other connective tissue disease */
'M65062' = '211' /* Abscess of tendon sheath, left lower leg - Other connective tissue disease */
'M65069' = '211' /* Abscess of tendon sheath, unspecified lower leg - Other connective tissue disease */
'M65071' = '211' /* Abscess of tendon sheath, right ankle and foot - Other connective tissue disease */
'M65072' = '211' /* Abscess of tendon sheath, left ankle and foot - Other connective tissue disease */
'M65079' = '211' /* Abscess of tendon sheath, unspecified ankle and foot - Other connective tissue disease */
'M6508' = '211' /* Abscess of tendon sheath, other site - Other connective tissue disease */
'M6510' = '211' /* Other infective (teno)synovitis, unspecified site - Other connective tissue disease */
'M65111' = '211' /* Other infective (teno)synovitis, right shoulder - Other connective tissue disease */
'M65112' = '211' /* Other infective (teno)synovitis, left shoulder - Other connective tissue disease */
'M65119' = '211' /* Other infective (teno)synovitis, unspecified shoulder - Other connective tissue disease */
'M65121' = '211' /* Other infective (teno)synovitis, right elbow - Other connective tissue disease */
'M65122' = '211' /* Other infective (teno)synovitis, left elbow - Other connective tissue disease */
'M65129' = '211' /* Other infective (teno)synovitis, unspecified elbow - Other connective tissue disease */
'M65131' = '211' /* Other infective (teno)synovitis, right wrist - Other connective tissue disease */
'M65132' = '211' /* Other infective (teno)synovitis, left wrist - Other connective tissue disease */
'M65139' = '211' /* Other infective (teno)synovitis, unspecified wrist - Other connective tissue disease */
'M65141' = '211' /* Other infective (teno)synovitis, right hand - Other connective tissue disease */
'M65142' = '211' /* Other infective (teno)synovitis, left hand - Other connective tissue disease */
'M65149' = '211' /* Other infective (teno)synovitis, unspecified hand - Other connective tissue disease */
'M65151' = '211' /* Other infective (teno)synovitis, right hip - Other connective tissue disease */
'M65152' = '211' /* Other infective (teno)synovitis, left hip - Other connective tissue disease */
'M65159' = '211' /* Other infective (teno)synovitis, unspecified hip - Other connective tissue disease */
'M65161' = '211' /* Other infective (teno)synovitis, right knee - Other connective tissue disease */
'M65162' = '211' /* Other infective (teno)synovitis, left knee - Other connective tissue disease */
'M65169' = '211' /* Other infective (teno)synovitis, unspecified knee - Other connective tissue disease */
'M65171' = '211' /* Other infective (teno)synovitis, right ankle and foot - Other connective tissue disease */
'M65172' = '211' /* Other infective (teno)synovitis, left ankle and foot - Other connective tissue disease */
'M65179' = '211' /* Other infective (teno)synovitis, unspecified ankle and foot - Other connective tissue disease */
'M6518' = '211' /* Other infective (teno)synovitis, other site - Other connective tissue disease */
'M6519' = '211' /* Other infective (teno)synovitis, multiple sites - Other connective tissue disease */
'M6520' = '211' /* Calcific tendinitis, unspecified site - Other connective tissue disease */
'M65221' = '211' /* Calcific tendinitis, right upper arm - Other connective tissue disease */
'M65222' = '211' /* Calcific tendinitis, left upper arm - Other connective tissue disease */
'M65229' = '211' /* Calcific tendinitis, unspecified upper arm - Other connective tissue disease */
'M65231' = '211' /* Calcific tendinitis, right forearm - Other connective tissue disease */
'M65232' = '211' /* Calcific tendinitis, left forearm - Other connective tissue disease */
'M65239' = '211' /* Calcific tendinitis, unspecified forearm - Other connective tissue disease */
'M65241' = '211' /* Calcific tendinitis, right hand - Other connective tissue disease */
'M65242' = '211' /* Calcific tendinitis, left hand - Other connective tissue disease */
'M65249' = '211' /* Calcific tendinitis, unspecified hand - Other connective tissue disease */
'M65251' = '211' /* Calcific tendinitis, right thigh - Other connective tissue disease */
'M65252' = '211' /* Calcific tendinitis, left thigh - Other connective tissue disease */
'M65259' = '211' /* Calcific tendinitis, unspecified thigh - Other connective tissue disease */
'M65261' = '211' /* Calcific tendinitis, right lower leg - Other connective tissue disease */
'M65262' = '211' /* Calcific tendinitis, left lower leg - Other connective tissue disease */
'M65269' = '211' /* Calcific tendinitis, unspecified lower leg - Other connective tissue disease */
'M65271' = '211' /* Calcific tendinitis, right ankle and foot - Other connective tissue disease */
'M65272' = '211' /* Calcific tendinitis, left ankle and foot - Other connective tissue disease */
'M65279' = '211' /* Calcific tendinitis, unspecified ankle and foot - Other connective tissue disease */
'M6528' = '211' /* Calcific tendinitis, other site - Other connective tissue disease */
'M6529' = '211' /* Calcific tendinitis, multiple sites - Other connective tissue disease */
'M6530' = '211' /* Trigger finger, unspecified finger - Other connective tissue disease */
'M65311' = '211' /* Trigger thumb, right thumb - Other connective tissue disease */
'M65312' = '211' /* Trigger thumb, left thumb - Other connective tissue disease */
'M65319' = '211' /* Trigger thumb, unspecified thumb - Other connective tissue disease */
'M65321' = '211' /* Trigger finger, right index finger - Other connective tissue disease */
'M65322' = '211' /* Trigger finger, left index finger - Other connective tissue disease */
'M65329' = '211' /* Trigger finger, unspecified index finger - Other connective tissue disease */
'M65331' = '211' /* Trigger finger, right middle finger - Other connective tissue disease */
'M65332' = '211' /* Trigger finger, left middle finger - Other connective tissue disease */
'M65339' = '211' /* Trigger finger, unspecified middle finger - Other connective tissue disease */
'M65341' = '211' /* Trigger finger, right ring finger - Other connective tissue disease */
'M65342' = '211' /* Trigger finger, left ring finger - Other connective tissue disease */
'M65349' = '211' /* Trigger finger, unspecified ring finger - Other connective tissue disease */
'M65351' = '211' /* Trigger finger, right little finger - Other connective tissue disease */
'M65352' = '211' /* Trigger finger, left little finger - Other connective tissue disease */
'M65359' = '211' /* Trigger finger, unspecified little finger - Other connective tissue disease */
'M654' = '211' /* Radial styloid tenosynovitis [de Quervain] - Other connective tissue disease */
'M6580' = '211' /* Other synovitis and tenosynovitis, unspecified site - Other connective tissue disease */
'M65811' = '211' /* Other synovitis and tenosynovitis, right shoulder - Other connective tissue disease */
'M65812' = '211' /* Other synovitis and tenosynovitis, left shoulder - Other connective tissue disease */
'M65819' = '211' /* Other synovitis and tenosynovitis, unspecified shoulder - Other connective tissue disease */
'M65821' = '211' /* Other synovitis and tenosynovitis, right upper arm - Other connective tissue disease */
'M65822' = '211' /* Other synovitis and tenosynovitis, left upper arm - Other connective tissue disease */
'M65829' = '211' /* Other synovitis and tenosynovitis, unspecified upper arm - Other connective tissue disease */
'M65831' = '211' /* Other synovitis and tenosynovitis, right forearm - Other connective tissue disease */
'M65832' = '211' /* Other synovitis and tenosynovitis, left forearm - Other connective tissue disease */
'M65839' = '211' /* Other synovitis and tenosynovitis, unspecified forearm - Other connective tissue disease */
'M65841' = '211' /* Other synovitis and tenosynovitis, right hand - Other connective tissue disease */
'M65842' = '211' /* Other synovitis and tenosynovitis, left hand - Other connective tissue disease */
'M65849' = '211' /* Other synovitis and tenosynovitis, unspecified hand - Other connective tissue disease */
'M65851' = '211' /* Other synovitis and tenosynovitis, right thigh - Other connective tissue disease */
'M65852' = '211' /* Other synovitis and tenosynovitis, left thigh - Other connective tissue disease */
'M65859' = '211' /* Other synovitis and tenosynovitis, unspecified thigh - Other connective tissue disease */
'M65861' = '211' /* Other synovitis and tenosynovitis, right lower leg - Other connective tissue disease */
'M65862' = '211' /* Other synovitis and tenosynovitis, left lower leg - Other connective tissue disease */
'M65869' = '211' /* Other synovitis and tenosynovitis, unspecified lower leg - Other connective tissue disease */
'M65871' = '211' /* Other synovitis and tenosynovitis, right ankle and foot - Other connective tissue disease */
'M65872' = '211' /* Other synovitis and tenosynovitis, left ankle and foot - Other connective tissue disease */
'M65879' = '211' /* Other synovitis and tenosynovitis, unsp ankle and foot - Other connective tissue disease */
'M6588' = '211' /* Other synovitis and tenosynovitis, other site - Other connective tissue disease */
'M6589' = '211' /* Other synovitis and tenosynovitis, multiple sites - Other connective tissue disease */
'M659' = '211' /* Synovitis and tenosynovitis, unspecified - Other connective tissue disease */
'M660' = '211' /* Rupture of popliteal cyst - Other connective tissue disease */
'M6610' = '211' /* Rupture of synovium, unspecified joint - Other connective tissue disease */
'M66111' = '211' /* Rupture of synovium, right shoulder - Other connective tissue disease */
'M66112' = '211' /* Rupture of synovium, left shoulder - Other connective tissue disease */
'M66119' = '211' /* Rupture of synovium, unspecified shoulder - Other connective tissue disease */
'M66121' = '211' /* Rupture of synovium, right elbow - Other connective tissue disease */
'M66122' = '211' /* Rupture of synovium, left elbow - Other connective tissue disease */
'M66129' = '211' /* Rupture of synovium, unspecified elbow - Other connective tissue disease */
'M66131' = '211' /* Rupture of synovium, right wrist - Other connective tissue disease */
'M66132' = '211' /* Rupture of synovium, left wrist - Other connective tissue disease */
'M66139' = '211' /* Rupture of synovium, unspecified wrist - Other connective tissue disease */
'M66141' = '211' /* Rupture of synovium, right hand - Other connective tissue disease */
'M66142' = '211' /* Rupture of synovium, left hand - Other connective tissue disease */
'M66143' = '211' /* Rupture of synovium, unspecified hand - Other connective tissue disease */
'M66144' = '211' /* Rupture of synovium, right finger(s) - Other connective tissue disease */
'M66145' = '211' /* Rupture of synovium, left finger(s) - Other connective tissue disease */
'M66146' = '211' /* Rupture of synovium, unspecified finger(s) - Other connective tissue disease */
'M66151' = '211' /* Rupture of synovium, right hip - Other connective tissue disease */
'M66152' = '211' /* Rupture of synovium, left hip - Other connective tissue disease */
'M66159' = '211' /* Rupture of synovium, unspecified hip - Other connective tissue disease */
'M66171' = '211' /* Rupture of synovium, right ankle - Other connective tissue disease */
'M66172' = '211' /* Rupture of synovium, left ankle - Other connective tissue disease */
'M66173' = '211' /* Rupture of synovium, unspecified ankle - Other connective tissue disease */
'M66174' = '211' /* Rupture of synovium, right foot - Other connective tissue disease */
'M66175' = '211' /* Rupture of synovium, left foot - Other connective tissue disease */
'M66176' = '211' /* Rupture of synovium, unspecified foot - Other connective tissue disease */
'M66177' = '211' /* Rupture of synovium, right toe(s) - Other connective tissue disease */
'M66178' = '211' /* Rupture of synovium, left toe(s) - Other connective tissue disease */
'M66179' = '211' /* Rupture of synovium, unspecified toe(s) - Other connective tissue disease */
'M6618' = '211' /* Rupture of synovium, other site - Other connective tissue disease */
'M6620' = '211' /* Spontaneous rupture of extensor tendons, unspecified site - Other connective tissue disease */
'M66211' = '211' /* Spontaneous rupture of extensor tendons, right shoulder - Other connective tissue disease */
'M66212' = '211' /* Spontaneous rupture of extensor tendons, left shoulder - Other connective tissue disease */
'M66219' = '211' /* Spontaneous rupture of extensor tendons, unsp shoulder - Other connective tissue disease */
'M66221' = '211' /* Spontaneous rupture of extensor tendons, right upper arm - Other connective tissue disease */
'M66222' = '211' /* Spontaneous rupture of extensor tendons, left upper arm - Other connective tissue disease */
'M66229' = '211' /* Spontaneous rupture of extensor tendons, unsp upper arm - Other connective tissue disease */
'M66231' = '211' /* Spontaneous rupture of extensor tendons, right forearm - Other connective tissue disease */
'M66232' = '211' /* Spontaneous rupture of extensor tendons, left forearm - Other connective tissue disease */
'M66239' = '211' /* Spontaneous rupture of extensor tendons, unspecified forearm - Other connective tissue disease */
'M66241' = '211' /* Spontaneous rupture of extensor tendons, right hand - Other connective tissue disease */
'M66242' = '211' /* Spontaneous rupture of extensor tendons, left hand - Other connective tissue disease */
'M66249' = '211' /* Spontaneous rupture of extensor tendons, unspecified hand - Other connective tissue disease */
'M66251' = '211' /* Spontaneous rupture of extensor tendons, right thigh - Other connective tissue disease */
'M66252' = '211' /* Spontaneous rupture of extensor tendons, left thigh - Other connective tissue disease */
'M66259' = '211' /* Spontaneous rupture of extensor tendons, unspecified thigh - Other connective tissue disease */
'M66261' = '211' /* Spontaneous rupture of extensor tendons, right lower leg - Other connective tissue disease */
'M66262' = '211' /* Spontaneous rupture of extensor tendons, left lower leg - Other connective tissue disease */
'M66269' = '211' /* Spontaneous rupture of extensor tendons, unsp lower leg - Other connective tissue disease */
'M66271' = '211' /* Spontaneous rupture of extensor tendons, right ank/ft - Other connective tissue disease */
'M66272' = '211' /* Spontaneous rupture of extensor tendons, left ankle and foot - Other connective tissue disease */
'M66279' = '211' /* Spontaneous rupture of extensor tendons, unsp ankle and foot - Other connective tissue disease */
'M6628' = '211' /* Spontaneous rupture of extensor tendons, other site - Other connective tissue disease */
'M6629' = '211' /* Spontaneous rupture of extensor tendons, multiple sites - Other connective tissue disease */
'M6630' = '211' /* Spontaneous rupture of flexor tendons, unspecified site - Other connective tissue disease */
'M66311' = '211' /* Spontaneous rupture of flexor tendons, right shoulder - Other connective tissue disease */
'M66312' = '211' /* Spontaneous rupture of flexor tendons, left shoulder - Other connective tissue disease */
'M66319' = '211' /* Spontaneous rupture of flexor tendons, unspecified shoulder - Other connective tissue disease */
'M66321' = '211' /* Spontaneous rupture of flexor tendons, right upper arm - Other connective tissue disease */
'M66322' = '211' /* Spontaneous rupture of flexor tendons, left upper arm - Other connective tissue disease */
'M66329' = '211' /* Spontaneous rupture of flexor tendons, unspecified upper arm - Other connective tissue disease */
'M66331' = '211' /* Spontaneous rupture of flexor tendons, right forearm - Other connective tissue disease */
'M66332' = '211' /* Spontaneous rupture of flexor tendons, left forearm - Other connective tissue disease */
'M66339' = '211' /* Spontaneous rupture of flexor tendons, unspecified forearm - Other connective tissue disease */
'M66341' = '211' /* Spontaneous rupture of flexor tendons, right hand - Other connective tissue disease */
'M66342' = '211' /* Spontaneous rupture of flexor tendons, left hand - Other connective tissue disease */
'M66349' = '211' /* Spontaneous rupture of flexor tendons, unspecified hand - Other connective tissue disease */
'M66351' = '211' /* Spontaneous rupture of flexor tendons, right thigh - Other connective tissue disease */
'M66352' = '211' /* Spontaneous rupture of flexor tendons, left thigh - Other connective tissue disease */
'M66359' = '211' /* Spontaneous rupture of flexor tendons, unspecified thigh - Other connective tissue disease */
'M66361' = '211' /* Spontaneous rupture of flexor tendons, right lower leg - Other connective tissue disease */
'M66362' = '211' /* Spontaneous rupture of flexor tendons, left lower leg - Other connective tissue disease */
'M66369' = '211' /* Spontaneous rupture of flexor tendons, unspecified lower leg - Other connective tissue disease */
'M66371' = '211' /* Spontaneous rupture of flexor tendons, right ankle and foot - Other connective tissue disease */
'M66372' = '211' /* Spontaneous rupture of flexor tendons, left ankle and foot - Other connective tissue disease */
'M66379' = '211' /* Spontaneous rupture of flexor tendons, unsp ankle and foot - Other connective tissue disease */
'M6638' = '211' /* Spontaneous rupture of flexor tendons, other site - Other connective tissue disease */
'M6639' = '211' /* Spontaneous rupture of flexor tendons, multiple sites - Other connective tissue disease */
'M6680' = '211' /* Spontaneous rupture of other tendons, unspecified site - Other connective tissue disease */
'M66811' = '211' /* Spontaneous rupture of other tendons, right shoulder - Other connective tissue disease */
'M66812' = '211' /* Spontaneous rupture of other tendons, left shoulder - Other connective tissue disease */
'M66819' = '211' /* Spontaneous rupture of other tendons, unspecified shoulder - Other connective tissue disease */
'M66821' = '211' /* Spontaneous rupture of other tendons, right upper arm - Other connective tissue disease */
'M66822' = '211' /* Spontaneous rupture of other tendons, left upper arm - Other connective tissue disease */
'M66829' = '211' /* Spontaneous rupture of other tendons, unspecified upper arm - Other connective tissue disease */
'M66831' = '211' /* Spontaneous rupture of other tendons, right forearm - Other connective tissue disease */
'M66832' = '211' /* Spontaneous rupture of other tendons, left forearm - Other connective tissue disease */
'M66839' = '211' /* Spontaneous rupture of other tendons, unspecified forearm - Other connective tissue disease */
'M66841' = '211' /* Spontaneous rupture of other tendons, right hand - Other connective tissue disease */
'M66842' = '211' /* Spontaneous rupture of other tendons, left hand - Other connective tissue disease */
'M66849' = '211' /* Spontaneous rupture of other tendons, unspecified hand - Other connective tissue disease */
'M66851' = '211' /* Spontaneous rupture of other tendons, right thigh - Other connective tissue disease */
'M66852' = '211' /* Spontaneous rupture of other tendons, left thigh - Other connective tissue disease */
'M66859' = '211' /* Spontaneous rupture of other tendons, unspecified thigh - Other connective tissue disease */
'M66861' = '211' /* Spontaneous rupture of other tendons, right lower leg - Other connective tissue disease */
'M66862' = '211' /* Spontaneous rupture of other tendons, left lower leg - Other connective tissue disease */
'M66869' = '211' /* Spontaneous rupture of other tendons, unspecified lower leg - Other connective tissue disease */
'M66871' = '211' /* Spontaneous rupture of other tendons, right ankle and foot - Other connective tissue disease */
'M66872' = '211' /* Spontaneous rupture of other tendons, left ankle and foot - Other connective tissue disease */
'M66879' = '211' /* Spontaneous rupture of other tendons, unsp ankle and foot - Other connective tissue disease */
'M6688' = '211' /* Spontaneous rupture of other tendons, other - Other connective tissue disease */
'M6689' = '211' /* Spontaneous rupture of other tendons, multiple sites - Other connective tissue disease */
'M669' = '211' /* Spontaneous rupture of unspecified tendon - Other connective tissue disease */
'M6700' = '211' /* Short Achilles tendon (acquired), unspecified ankle - Other connective tissue disease */
'M6701' = '211' /* Short Achilles tendon (acquired), right ankle - Other connective tissue disease */
'M6702' = '211' /* Short Achilles tendon (acquired), left ankle - Other connective tissue disease */
'M6720' = '211' /* Synovial hypertrophy, not elsewhere classified, unsp site - Other connective tissue disease */
'M67211' = '211' /* Synovial hypertrophy, NEC, right shoulder - Other connective tissue disease */
'M67212' = '211' /* Synovial hypertrophy, NEC, left shoulder - Other connective tissue disease */
'M67219' = '211' /* Synovial hypertrophy, NEC, unsp shoulder - Other connective tissue disease */
'M67221' = '211' /* Synovial hypertrophy, NEC, right upper arm - Other connective tissue disease */
'M67222' = '211' /* Synovial hypertrophy, NEC, left upper arm - Other connective tissue disease */
'M67229' = '211' /* Synovial hypertrophy, NEC, unsp upper arm - Other connective tissue disease */
'M67231' = '211' /* Synovial hypertrophy, NEC, right forearm - Other connective tissue disease */
'M67232' = '211' /* Synovial hypertrophy, not elsewhere classified, left forearm - Other connective tissue disease */
'M67239' = '211' /* Synovial hypertrophy, not elsewhere classified, unsp forearm - Other connective tissue disease */
'M67241' = '211' /* Synovial hypertrophy, not elsewhere classified, right hand - Other connective tissue disease */
'M67242' = '211' /* Synovial hypertrophy, not elsewhere classified, left hand - Other connective tissue disease */
'M67249' = '211' /* Synovial hypertrophy, not elsewhere classified, unsp hand - Other connective tissue disease */
'M67251' = '211' /* Synovial hypertrophy, not elsewhere classified, right thigh - Other connective tissue disease */
'M67252' = '211' /* Synovial hypertrophy, not elsewhere classified, left thigh - Other connective tissue disease */
'M67259' = '211' /* Synovial hypertrophy, not elsewhere classified, unsp thigh - Other connective tissue disease */
'M67261' = '211' /* Synovial hypertrophy, NEC, right lower leg - Other connective tissue disease */
'M67262' = '211' /* Synovial hypertrophy, NEC, left lower leg - Other connective tissue disease */
'M67269' = '211' /* Synovial hypertrophy, NEC, unsp lower leg - Other connective tissue disease */
'M67271' = '211' /* Synovial hypertrophy, NEC, right ankle and foot - Other connective tissue disease */
'M67272' = '211' /* Synovial hypertrophy, NEC, left ankle and foot - Other connective tissue disease */
'M67279' = '211' /* Synovial hypertrophy, NEC, unsp ankle and foot - Other connective tissue disease */
'M6728' = '211' /* Synovial hypertrophy, not elsewhere classified, other site - Other connective tissue disease */
'M6729' = '211' /* Synovial hypertrophy, NEC, multiple sites - Other connective tissue disease */
'M6730' = '211' /* Transient synovitis, unspecified site - Other connective tissue disease */
'M67311' = '211' /* Transient synovitis, right shoulder - Other connective tissue disease */
'M67312' = '211' /* Transient synovitis, left shoulder - Other connective tissue disease */
'M67319' = '211' /* Transient synovitis, unspecified shoulder - Other connective tissue disease */
'M67321' = '211' /* Transient synovitis, right elbow - Other connective tissue disease */
'M67322' = '211' /* Transient synovitis, left elbow - Other connective tissue disease */
'M67329' = '211' /* Transient synovitis, unspecified elbow - Other connective tissue disease */
'M67331' = '211' /* Transient synovitis, right wrist - Other connective tissue disease */
'M67332' = '211' /* Transient synovitis, left wrist - Other connective tissue disease */
'M67339' = '211' /* Transient synovitis, unspecified wrist - Other connective tissue disease */
'M67341' = '211' /* Transient synovitis, right hand - Other connective tissue disease */
'M67342' = '211' /* Transient synovitis, left hand - Other connective tissue disease */
'M67349' = '211' /* Transient synovitis, unspecified hand - Other connective tissue disease */
'M67351' = '211' /* Transient synovitis, right hip - Other connective tissue disease */
'M67352' = '211' /* Transient synovitis, left hip - Other connective tissue disease */
'M67359' = '211' /* Transient synovitis, unspecified hip - Other connective tissue disease */
'M67361' = '211' /* Transient synovitis, right knee - Other connective tissue disease */
'M67362' = '211' /* Transient synovitis, left knee - Other connective tissue disease */
'M67369' = '211' /* Transient synovitis, unspecified knee - Other connective tissue disease */
'M67371' = '211' /* Transient synovitis, right ankle and foot - Other connective tissue disease */
'M67372' = '211' /* Transient synovitis, left ankle and foot - Other connective tissue disease */
'M67379' = '211' /* Transient synovitis, unspecified ankle and foot - Other connective tissue disease */
'M6738' = '211' /* Transient synovitis, other site - Other connective tissue disease */
'M6739' = '211' /* Transient synovitis, multiple sites - Other connective tissue disease */
'M6740' = '211' /* Ganglion, unspecified site - Other connective tissue disease */
'M67411' = '211' /* Ganglion, right shoulder - Other connective tissue disease */
'M67412' = '211' /* Ganglion, left shoulder - Other connective tissue disease */
'M67419' = '211' /* Ganglion, unspecified shoulder - Other connective tissue disease */
'M67421' = '211' /* Ganglion, right elbow - Other connective tissue disease */
'M67422' = '211' /* Ganglion, left elbow - Other connective tissue disease */
'M67429' = '211' /* Ganglion, unspecified elbow - Other connective tissue disease */
'M67431' = '211' /* Ganglion, right wrist - Other connective tissue disease */
'M67432' = '211' /* Ganglion, left wrist - Other connective tissue disease */
'M67439' = '211' /* Ganglion, unspecified wrist - Other connective tissue disease */
'M67441' = '211' /* Ganglion, right hand - Other connective tissue disease */
'M67442' = '211' /* Ganglion, left hand - Other connective tissue disease */
'M67449' = '211' /* Ganglion, unspecified hand - Other connective tissue disease */
'M67451' = '211' /* Ganglion, right hip - Other connective tissue disease */
'M67452' = '211' /* Ganglion, left hip - Other connective tissue disease */
'M67459' = '211' /* Ganglion, unspecified hip - Other connective tissue disease */
'M67461' = '211' /* Ganglion, right knee - Other connective tissue disease */
'M67462' = '211' /* Ganglion, left knee - Other connective tissue disease */
'M67469' = '211' /* Ganglion, unspecified knee - Other connective tissue disease */
'M67471' = '211' /* Ganglion, right ankle and foot - Other connective tissue disease */
'M67472' = '211' /* Ganglion, left ankle and foot - Other connective tissue disease */
'M67479' = '211' /* Ganglion, unspecified ankle and foot - Other connective tissue disease */
'M6748' = '211' /* Ganglion, other site - Other connective tissue disease */
'M6749' = '211' /* Ganglion, multiple sites - Other connective tissue disease */
'M6750' = '211' /* Plica syndrome, unspecified knee - Other connective tissue disease */
'M6751' = '211' /* Plica syndrome, right knee - Other connective tissue disease */
'M6752' = '211' /* Plica syndrome, left knee - Other connective tissue disease */
'M6780' = '211' /* Oth disrd of synovium and tendon, unspecified site - Other connective tissue disease */
'M67811' = '211' /* Other specified disorders of synovium, right shoulder - Other connective tissue disease */
'M67812' = '211' /* Other specified disorders of synovium, left shoulder - Other connective tissue disease */
'M67813' = '211' /* Other specified disorders of tendon, right shoulder - Other connective tissue disease */
'M67814' = '211' /* Other specified disorders of tendon, left shoulder - Other connective tissue disease */
'M67819' = '211' /* Oth disrd of synovium and tendon, unspecified shoulder - Other connective tissue disease */
'M67821' = '211' /* Other specified disorders of synovium, right elbow - Other connective tissue disease */
'M67822' = '211' /* Other specified disorders of synovium, left elbow - Other connective tissue disease */
'M67823' = '211' /* Other specified disorders of tendon, right elbow - Other connective tissue disease */
'M67824' = '211' /* Other specified disorders of tendon, left elbow - Other connective tissue disease */
'M67829' = '211' /* Oth disrd of synovium and tendon, unspecified elbow - Other connective tissue disease */
'M67831' = '211' /* Other specified disorders of synovium, right wrist - Other connective tissue disease */
'M67832' = '211' /* Other specified disorders of synovium, left wrist - Other connective tissue disease */
'M67833' = '211' /* Other specified disorders of tendon, right wrist - Other connective tissue disease */
'M67834' = '211' /* Other specified disorders of tendon, left wrist - Other connective tissue disease */
'M67839' = '211' /* Oth disrd of synovium and tendon, unspecified forearm - Other connective tissue disease */
'M67841' = '211' /* Other specified disorders of synovium, right hand - Other connective tissue disease */
'M67842' = '211' /* Other specified disorders of synovium, left hand - Other connective tissue disease */
'M67843' = '211' /* Other specified disorders of tendon, right hand - Other connective tissue disease */
'M67844' = '211' /* Other specified disorders of tendon, left hand - Other connective tissue disease */
'M67849' = '211' /* Oth disrd of synovium and tendon, unspecified hand - Other connective tissue disease */
'M67851' = '211' /* Other specified disorders of synovium, right hip - Other connective tissue disease */
'M67852' = '211' /* Other specified disorders of synovium, left hip - Other connective tissue disease */
'M67853' = '211' /* Other specified disorders of tendon, right hip - Other connective tissue disease */
'M67854' = '211' /* Other specified disorders of tendon, left hip - Other connective tissue disease */
'M67859' = '211' /* Oth disrd of synovium and tendon, unspecified hip - Other connective tissue disease */
'M67861' = '211' /* Other specified disorders of synovium, right knee - Other connective tissue disease */
'M67862' = '211' /* Other specified disorders of synovium, left knee - Other connective tissue disease */
'M67863' = '211' /* Other specified disorders of tendon, right knee - Other connective tissue disease */
'M67864' = '211' /* Other specified disorders of tendon, left knee - Other connective tissue disease */
'M67869' = '211' /* Oth disrd of synovium and tendon, unspecified knee - Other connective tissue disease */
'M67871' = '211' /* Other specified disorders of synovium, right ankle and foot - Other connective tissue disease */
'M67872' = '211' /* Other specified disorders of synovium, left ankle and foot - Other connective tissue disease */
'M67873' = '211' /* Other specified disorders of tendon, right ankle and foot - Other connective tissue disease */
'M67874' = '211' /* Other specified disorders of tendon, left ankle and foot - Other connective tissue disease */
'M67879' = '211' /* Oth disrd of synovium and tendon, unspecified ankle and foot - Other connective tissue disease */
'M6788' = '211' /* Other specified disorders of synovium and tendon, other site - Other connective tissue disease */
'M6789' = '211' /* Oth disrd of synovium and tendon, multiple sites - Other connective tissue disease */
'M6790' = '211' /* Unsp disorder of synovium and tendon, unspecified site - Other connective tissue disease */
'M67911' = '211' /* Unspecified disorder of synovium and tendon, right shoulder - Other connective tissue disease */
'M67912' = '211' /* Unspecified disorder of synovium and tendon, left shoulder - Other connective tissue disease */
'M67919' = '211' /* Unsp disorder of synovium and tendon, unspecified shoulder - Other connective tissue disease */
'M67921' = '211' /* Unspecified disorder of synovium and tendon, right upper arm - Other connective tissue disease */
'M67922' = '211' /* Unspecified disorder of synovium and tendon, left upper arm - Other connective tissue disease */
'M67929' = '211' /* Unsp disorder of synovium and tendon, unspecified upper arm - Other connective tissue disease */
'M67931' = '211' /* Unspecified disorder of synovium and tendon, right forearm - Other connective tissue disease */
'M67932' = '211' /* Unspecified disorder of synovium and tendon, left forearm - Other connective tissue disease */
'M67939' = '211' /* Unsp disorder of synovium and tendon, unspecified forearm - Other connective tissue disease */
'M67941' = '211' /* Unspecified disorder of synovium and tendon, right hand - Other connective tissue disease */
'M67942' = '211' /* Unspecified disorder of synovium and tendon, left hand - Other connective tissue disease */
'M67949' = '211' /* Unsp disorder of synovium and tendon, unspecified hand - Other connective tissue disease */
'M67951' = '211' /* Unspecified disorder of synovium and tendon, right thigh - Other connective tissue disease */
'M67952' = '211' /* Unspecified disorder of synovium and tendon, left thigh - Other connective tissue disease */
'M67959' = '211' /* Unsp disorder of synovium and tendon, unspecified thigh - Other connective tissue disease */
'M67961' = '211' /* Unspecified disorder of synovium and tendon, right lower leg - Other connective tissue disease */
'M67962' = '211' /* Unspecified disorder of synovium and tendon, left lower leg - Other connective tissue disease */
'M67969' = '211' /* Unsp disorder of synovium and tendon, unspecified lower leg - Other connective tissue disease */
'M67971' = '211' /* Unsp disorder of synovium and tendon, right ankle and foot - Other connective tissue disease */
'M67972' = '211' /* Unsp disorder of synovium and tendon, left ankle and foot - Other connective tissue disease */
'M67979' = '211' /* Unsp disorder of synovium and tendon, unsp ankle and foot - Other connective tissue disease */
'M6798' = '211' /* Unspecified disorder of synovium and tendon, other site - Other connective tissue disease */
'M6799' = '211' /* Unspecified disorder of synovium and tendon, multiple sites - Other connective tissue disease */
'M70031' = '211' /* Crepitant synovitis (acute) (chronic), right wrist - Other connective tissue disease */
'M70032' = '211' /* Crepitant synovitis (acute) (chronic), left wrist - Other connective tissue disease */
'M70039' = '211' /* Crepitant synovitis (acute) (chronic), unspecified wrist - Other connective tissue disease */
'M70041' = '211' /* Crepitant synovitis (acute) (chronic), right hand - Other connective tissue disease */
'M70042' = '211' /* Crepitant synovitis (acute) (chronic), left hand - Other connective tissue disease */
'M70049' = '211' /* Crepitant synovitis (acute) (chronic), unspecified hand - Other connective tissue disease */
'M7010' = '211' /* Bursitis, unspecified hand - Other connective tissue disease */
'M7011' = '211' /* Bursitis, right hand - Other connective tissue disease */
'M7012' = '211' /* Bursitis, left hand - Other connective tissue disease */
'M7020' = '211' /* Olecranon bursitis, unspecified elbow - Other connective tissue disease */
'M7021' = '211' /* Olecranon bursitis, right elbow - Other connective tissue disease */
'M7022' = '211' /* Olecranon bursitis, left elbow - Other connective tissue disease */
'M7030' = '211' /* Other bursitis of elbow, unspecified elbow - Other connective tissue disease */
'M7031' = '211' /* Other bursitis of elbow, right elbow - Other connective tissue disease */
'M7032' = '211' /* Other bursitis of elbow, left elbow - Other connective tissue disease */
'M7040' = '211' /* Prepatellar bursitis, unspecified knee - Other connective tissue disease */
'M7041' = '211' /* Prepatellar bursitis, right knee - Other connective tissue disease */
'M7042' = '211' /* Prepatellar bursitis, left knee - Other connective tissue disease */
'M7050' = '211' /* Other bursitis of knee, unspecified knee - Other connective tissue disease */
'M7051' = '211' /* Other bursitis of knee, right knee - Other connective tissue disease */
'M7052' = '211' /* Other bursitis of knee, left knee - Other connective tissue disease */
'M7060' = '211' /* Trochanteric bursitis, unspecified hip - Other connective tissue disease */
'M7061' = '211' /* Trochanteric bursitis, right hip - Other connective tissue disease */
'M7062' = '211' /* Trochanteric bursitis, left hip - Other connective tissue disease */
'M7070' = '211' /* Other bursitis of hip, unspecified hip - Other connective tissue disease */
'M7071' = '211' /* Other bursitis of hip, right hip - Other connective tissue disease */
'M7072' = '211' /* Other bursitis of hip, left hip - Other connective tissue disease */
'M7080' = '211' /* Oth soft tissue disord related to use/pressure of unsp site - Other connective tissue disease */
'M70811' = '211' /* Oth soft tissue disord related to use/pressure, r shoulder - Other connective tissue disease */
'M70812' = '211' /* Oth soft tissue disord related to use/pressure, l shoulder - Other connective tissue disease */
'M70819' = '211' /* Oth soft tissue disord related to use/pressure, unsp shldr - Other connective tissue disease */
'M70821' = '211' /* Oth soft tissue disorders related to use/pressure, r up arm - Other connective tissue disease */
'M70822' = '211' /* Oth soft tissue disorders related to use/pressure, l up arm - Other connective tissue disease */
'M70829' = '211' /* Oth soft tissue disord rel to use/pressure, unsp upper arms - Other connective tissue disease */
'M70831' = '211' /* Oth soft tissue disorders related to use/pressure, r forearm - Other connective tissue disease */
'M70832' = '211' /* Oth soft tissue disorders related to use/pressure, l forearm - Other connective tissue disease */
'M70839' = '211' /* Oth soft tissue disord related to use/pressure, unsp forearm - Other connective tissue disease */
'M70841' = '211' /* Oth soft tissue disorders related to use/pressure, r hand - Other connective tissue disease */
'M70842' = '211' /* Oth soft tissue disorders related to use/pressure, left hand - Other connective tissue disease */
'M70849' = '211' /* Oth soft tissue disorders related to use/pressure, unsp hand - Other connective tissue disease */
'M70851' = '211' /* Oth soft tissue disord related to use/pressure, right thigh - Other connective tissue disease */
'M70852' = '211' /* Oth soft tissue disord related to use/pressure, left thigh - Other connective tissue disease */
'M70859' = '211' /* Oth soft tissue disord related to use/pressure, unsp thigh - Other connective tissue disease */
'M70861' = '211' /* Oth soft tissue disorders related to use/pressure, r low leg - Other connective tissue disease */
'M70862' = '211' /* Oth soft tissue disorders related to use/pressure, l low leg - Other connective tissue disease */
'M70869' = '211' /* Oth soft tissue disorders related to use/pressure, unsp leg - Other connective tissue disease */
'M70871' = '211' /* Oth soft tissue disord related to use/pressure, right ank/ft - Other connective tissue disease */
'M70872' = '211' /* Oth soft tissue disord related to use/pressure, left ank/ft - Other connective tissue disease */
'M70879' = '211' /* Oth soft tissue disord related to use/pressure, unsp ank/ft - Other connective tissue disease */
'M7088' = '211' /* Oth soft tissue disorders related to use/pressure oth site - Other connective tissue disease */
'M7089' = '211' /* Oth soft tissue disord related to use/pressure mult sites - Other connective tissue disease */
'M7090' = '211' /* Unsp soft tissue disord related to use/pressure of unsp site - Other connective tissue disease */
'M70911' = '211' /* Unsp soft tissue disord related to use/pressure, r shoulder - Other connective tissue disease */
'M70912' = '211' /* Unsp soft tissue disord related to use/pressure, l shoulder - Other connective tissue disease */
'M70919' = '211' /* Unsp soft tissue disord related to use/pressure, unsp shldr - Other connective tissue disease */
'M70921' = '211' /* Unsp soft tissue disorder related to use/pressure, r up arm - Other connective tissue disease */
'M70922' = '211' /* Unsp soft tissue disorder related to use/pressure, l up arm - Other connective tissue disease */
'M70929' = '211' /* Unsp soft tissue disord related to use/pressure, unsp up arm - Other connective tissue disease */
'M70931' = '211' /* Unsp soft tissue disorder related to use/pressure, r forearm - Other connective tissue disease */
'M70932' = '211' /* Unsp soft tissue disorder related to use/pressure, l forearm - Other connective tissue disease */
'M70939' = '211' /* Unsp soft tissue disord rel to use/pressure, unsp forearm - Other connective tissue disease */
'M70941' = '211' /* Unsp soft tissue disorder related to use/pressure, r hand - Other connective tissue disease */
'M70942' = '211' /* Unsp soft tissue disorder related to use/pressure, left hand - Other connective tissue disease */
'M70949' = '211' /* Unsp soft tissue disorder related to use/pressure, unsp hand - Other connective tissue disease */
'M70951' = '211' /* Unsp soft tissue disord related to use/pressure, right thigh - Other connective tissue disease */
'M70952' = '211' /* Unsp soft tissue disord related to use/pressure, left thigh - Other connective tissue disease */
'M70959' = '211' /* Unsp soft tissue disord related to use/pressure, unsp thigh - Other connective tissue disease */
'M70961' = '211' /* Unsp soft tissue disorder related to use/pressure, r low leg - Other connective tissue disease */
'M70962' = '211' /* Unsp soft tissue disorder related to use/pressure, l low leg - Other connective tissue disease */
'M70969' = '211' /* Unsp soft tissue disord rel to use/pressure, unsp low leg - Other connective tissue disease */
'M70971' = '211' /* Unsp soft tissue disord rel to use/pressure, right ank/ft - Other connective tissue disease */
'M70972' = '211' /* Unsp soft tissue disord related to use/pressure, left ank/ft - Other connective tissue disease */
'M70979' = '211' /* Unsp soft tissue disord related to use/pressure, unsp ank/ft - Other connective tissue disease */
'M7098' = '211' /* Unsp soft tissue disorder related to use/pressure oth - Other connective tissue disease */
'M7099' = '211' /* Unsp soft tissue disord related to use/pressure mult sites - Other connective tissue disease */
'M7100' = '211' /* Abscess of bursa, unspecified site - Other connective tissue disease */
'M71011' = '211' /* Abscess of bursa, right shoulder - Other connective tissue disease */
'M71012' = '211' /* Abscess of bursa, left shoulder - Other connective tissue disease */
'M71019' = '211' /* Abscess of bursa, unspecified shoulder - Other connective tissue disease */
'M71021' = '211' /* Abscess of bursa, right elbow - Other connective tissue disease */
'M71022' = '211' /* Abscess of bursa, left elbow - Other connective tissue disease */
'M71029' = '211' /* Abscess of bursa, unspecified elbow - Other connective tissue disease */
'M71031' = '211' /* Abscess of bursa, right wrist - Other connective tissue disease */
'M71032' = '211' /* Abscess of bursa, left wrist - Other connective tissue disease */
'M71039' = '211' /* Abscess of bursa, unspecified wrist - Other connective tissue disease */
'M71041' = '211' /* Abscess of bursa, right hand - Other connective tissue disease */
'M71042' = '211' /* Abscess of bursa, left hand - Other connective tissue disease */
'M71049' = '211' /* Abscess of bursa, unspecified hand - Other connective tissue disease */
'M71051' = '211' /* Abscess of bursa, right hip - Other connective tissue disease */
'M71052' = '211' /* Abscess of bursa, left hip - Other connective tissue disease */
'M71059' = '211' /* Abscess of bursa, unspecified hip - Other connective tissue disease */
'M71061' = '211' /* Abscess of bursa, right knee - Other connective tissue disease */
'M71062' = '211' /* Abscess of bursa, left knee - Other connective tissue disease */
'M71069' = '211' /* Abscess of bursa, unspecified knee - Other connective tissue disease */
'M71071' = '211' /* Abscess of bursa, right ankle and foot - Other connective tissue disease */
'M71072' = '211' /* Abscess of bursa, left ankle and foot - Other connective tissue disease */
'M71079' = '211' /* Abscess of bursa, unspecified ankle and foot - Other connective tissue disease */
'M7108' = '211' /* Abscess of bursa, other site - Other connective tissue disease */
'M7109' = '211' /* Abscess of bursa, multiple sites - Other connective tissue disease */
'M7110' = '211' /* Other infective bursitis, unspecified site - Other connective tissue disease */
'M71111' = '211' /* Other infective bursitis, right shoulder - Other connective tissue disease */
'M71112' = '211' /* Other infective bursitis, left shoulder - Other connective tissue disease */
'M71119' = '211' /* Other infective bursitis, unspecified shoulder - Other connective tissue disease */
'M71121' = '211' /* Other infective bursitis, right elbow - Other connective tissue disease */
'M71122' = '211' /* Other infective bursitis, left elbow - Other connective tissue disease */
'M71129' = '211' /* Other infective bursitis, unspecified elbow - Other connective tissue disease */
'M71131' = '211' /* Other infective bursitis, right wrist - Other connective tissue disease */
'M71132' = '211' /* Other infective bursitis, left wrist - Other connective tissue disease */
'M71139' = '211' /* Other infective bursitis, unspecified wrist - Other connective tissue disease */
'M71141' = '211' /* Other infective bursitis, right hand - Other connective tissue disease */
'M71142' = '211' /* Other infective bursitis, left hand - Other connective tissue disease */
'M71149' = '211' /* Other infective bursitis, unspecified hand - Other connective tissue disease */
'M71151' = '211' /* Other infective bursitis, right hip - Other connective tissue disease */
'M71152' = '211' /* Other infective bursitis, left hip - Other connective tissue disease */
'M71159' = '211' /* Other infective bursitis, unspecified hip - Other connective tissue disease */
'M71161' = '211' /* Other infective bursitis, right knee - Other connective tissue disease */
'M71162' = '211' /* Other infective bursitis, left knee - Other connective tissue disease */
'M71169' = '211' /* Other infective bursitis, unspecified knee - Other connective tissue disease */
'M71171' = '211' /* Other infective bursitis, right ankle and foot - Other connective tissue disease */
'M71172' = '211' /* Other infective bursitis, left ankle and foot - Other connective tissue disease */
'M71179' = '211' /* Other infective bursitis, unspecified ankle and foot - Other connective tissue disease */
'M7118' = '211' /* Other infective bursitis, other site - Other connective tissue disease */
'M7119' = '211' /* Other infective bursitis, multiple sites - Other connective tissue disease */
'M7120' = '211' /* Synovial cyst of popliteal space [Baker], unspecified knee - Other connective tissue disease */
'M7121' = '211' /* Synovial cyst of popliteal space [Baker], right knee - Other connective tissue disease */
'M7122' = '211' /* Synovial cyst of popliteal space [Baker], left knee - Other connective tissue disease */
'M7130' = '211' /* Other bursal cyst, unspecified site - Other connective tissue disease */
'M71311' = '211' /* Other bursal cyst, right shoulder - Other connective tissue disease */
'M71312' = '211' /* Other bursal cyst, left shoulder - Other connective tissue disease */
'M71319' = '211' /* Other bursal cyst, unspecified shoulder - Other connective tissue disease */
'M71321' = '211' /* Other bursal cyst, right elbow - Other connective tissue disease */
'M71322' = '211' /* Other bursal cyst, left elbow - Other connective tissue disease */
'M71329' = '211' /* Other bursal cyst, unspecified elbow - Other connective tissue disease */
'M71331' = '211' /* Other bursal cyst, right wrist - Other connective tissue disease */
'M71332' = '211' /* Other bursal cyst, left wrist - Other connective tissue disease */
'M71339' = '211' /* Other bursal cyst, unspecified wrist - Other connective tissue disease */
'M71341' = '211' /* Other bursal cyst, right hand - Other connective tissue disease */
'M71342' = '211' /* Other bursal cyst, left hand - Other connective tissue disease */
'M71349' = '211' /* Other bursal cyst, unspecified hand - Other connective tissue disease */
'M71351' = '211' /* Other bursal cyst, right hip - Other connective tissue disease */
'M71352' = '211' /* Other bursal cyst, left hip - Other connective tissue disease */
'M71359' = '211' /* Other bursal cyst, unspecified hip - Other connective tissue disease */
'M71371' = '211' /* Other bursal cyst, right ankle and foot - Other connective tissue disease */
'M71372' = '211' /* Other bursal cyst, left ankle and foot - Other connective tissue disease */
'M71379' = '211' /* Other bursal cyst, unspecified ankle and foot - Other connective tissue disease */
'M7138' = '211' /* Other bursal cyst, other site - Other connective tissue disease */
'M7139' = '211' /* Other bursal cyst, multiple sites - Other connective tissue disease */
'M7140' = '211' /* Calcium deposit in bursa, unspecified site - Other connective tissue disease */
'M71421' = '211' /* Calcium deposit in bursa, right elbow - Other connective tissue disease */
'M71422' = '211' /* Calcium deposit in bursa, left elbow - Other connective tissue disease */
'M71429' = '211' /* Calcium deposit in bursa, unspecified elbow - Other connective tissue disease */
'M71431' = '211' /* Calcium deposit in bursa, right wrist - Other connective tissue disease */
'M71432' = '211' /* Calcium deposit in bursa, left wrist - Other connective tissue disease */
'M71439' = '211' /* Calcium deposit in bursa, unspecified wrist - Other connective tissue disease */
'M71441' = '211' /* Calcium deposit in bursa, right hand - Other connective tissue disease */
'M71442' = '211' /* Calcium deposit in bursa, left hand - Other connective tissue disease */
'M71449' = '211' /* Calcium deposit in bursa, unspecified hand - Other connective tissue disease */
'M71451' = '211' /* Calcium deposit in bursa, right hip - Other connective tissue disease */
'M71452' = '211' /* Calcium deposit in bursa, left hip - Other connective tissue disease */
'M71459' = '211' /* Calcium deposit in bursa, unspecified hip - Other connective tissue disease */
'M71461' = '211' /* Calcium deposit in bursa, right knee - Other connective tissue disease */
'M71462' = '211' /* Calcium deposit in bursa, left knee - Other connective tissue disease */
'M71469' = '211' /* Calcium deposit in bursa, unspecified knee - Other connective tissue disease */
'M71471' = '211' /* Calcium deposit in bursa, right ankle and foot - Other connective tissue disease */
'M71472' = '211' /* Calcium deposit in bursa, left ankle and foot - Other connective tissue disease */
'M71479' = '211' /* Calcium deposit in bursa, unspecified ankle and foot - Other connective tissue disease */
'M7148' = '211' /* Calcium deposit in bursa, other site - Other connective tissue disease */
'M7149' = '211' /* Calcium deposit in bursa, multiple sites - Other connective tissue disease */
'M7150' = '211' /* Other bursitis, not elsewhere classified, unspecified site - Other connective tissue disease */
'M71521' = '211' /* Other bursitis, not elsewhere classified, right elbow - Other connective tissue disease */
'M71522' = '211' /* Other bursitis, not elsewhere classified, left elbow - Other connective tissue disease */
'M71529' = '211' /* Other bursitis, not elsewhere classified, unspecified elbow - Other connective tissue disease */
'M71531' = '211' /* Other bursitis, not elsewhere classified, right wrist - Other connective tissue disease */
'M71532' = '211' /* Other bursitis, not elsewhere classified, left wrist - Other connective tissue disease */
'M71539' = '211' /* Other bursitis, not elsewhere classified, unspecified wrist - Other connective tissue disease */
'M71541' = '211' /* Other bursitis, not elsewhere classified, right hand - Other connective tissue disease */
'M71542' = '211' /* Other bursitis, not elsewhere classified, left hand - Other connective tissue disease */
'M71549' = '211' /* Other bursitis, not elsewhere classified, unspecified hand - Other connective tissue disease */
'M71551' = '211' /* Other bursitis, not elsewhere classified, right hip - Other connective tissue disease */
'M71552' = '211' /* Other bursitis, not elsewhere classified, left hip - Other connective tissue disease */
'M71559' = '211' /* Other bursitis, not elsewhere classified, unspecified hip - Other connective tissue disease */
'M71561' = '211' /* Other bursitis, not elsewhere classified, right knee - Other connective tissue disease */
'M71562' = '211' /* Other bursitis, not elsewhere classified, left knee - Other connective tissue disease */
'M71569' = '211' /* Other bursitis, not elsewhere classified, unspecified knee - Other connective tissue disease */
'M71571' = '211' /* Oth bursitis, not elsewhere classified, right ankle and foot - Other connective tissue disease */
'M71572' = '211' /* Oth bursitis, not elsewhere classified, left ankle and foot - Other connective tissue disease */
'M71579' = '211' /* Oth bursitis, not elsewhere classified, unsp ankle and foot - Other connective tissue disease */
'M7158' = '211' /* Other bursitis, not elsewhere classified, other site - Other connective tissue disease */
'M7180' = '211' /* Other specified bursopathies, unspecified site - Other connective tissue disease */
'M71811' = '211' /* Other specified bursopathies, right shoulder - Other connective tissue disease */
'M71812' = '211' /* Other specified bursopathies, left shoulder - Other connective tissue disease */
'M71819' = '211' /* Other specified bursopathies, unspecified shoulder - Other connective tissue disease */
'M71821' = '211' /* Other specified bursopathies, right elbow - Other connective tissue disease */
'M71822' = '211' /* Other specified bursopathies, left elbow - Other connective tissue disease */
'M71829' = '211' /* Other specified bursopathies, unspecified elbow - Other connective tissue disease */
'M71831' = '211' /* Other specified bursopathies, right wrist - Other connective tissue disease */
'M71832' = '211' /* Other specified bursopathies, left wrist - Other connective tissue disease */
'M71839' = '211' /* Other specified bursopathies, unspecified wrist - Other connective tissue disease */
'M71841' = '211' /* Other specified bursopathies, right hand - Other connective tissue disease */
'M71842' = '211' /* Other specified bursopathies, left hand - Other connective tissue disease */
'M71849' = '211' /* Other specified bursopathies, unspecified hand - Other connective tissue disease */
'M71851' = '211' /* Other specified bursopathies, right hip - Other connective tissue disease */
'M71852' = '211' /* Other specified bursopathies, left hip - Other connective tissue disease */
'M71859' = '211' /* Other specified bursopathies, unspecified hip - Other connective tissue disease */
'M71861' = '211' /* Other specified bursopathies, right knee - Other connective tissue disease */
'M71862' = '211' /* Other specified bursopathies, left knee - Other connective tissue disease */
'M71869' = '211' /* Other specified bursopathies, unspecified knee - Other connective tissue disease */
'M71871' = '211' /* Other specified bursopathies, right ankle and foot - Other connective tissue disease */
'M71872' = '211' /* Other specified bursopathies, left ankle and foot - Other connective tissue disease */
'M71879' = '211' /* Other specified bursopathies, unspecified ankle and foot - Other connective tissue disease */
'M7188' = '211' /* Other specified bursopathies, other site - Other connective tissue disease */
'M7189' = '211' /* Other specified bursopathies, multiple sites - Other connective tissue disease */
'M719' = '211' /* Bursopathy, unspecified - Other connective tissue disease */
'M720' = '211' /* Palmar fascial fibromatosis [Dupuytren] - Other connective tissue disease */
'M721' = '211' /* Knuckle pads - Other connective tissue disease */
'M722' = '211' /* Plantar fascial fibromatosis - Other connective tissue disease */
'M724' = '211' /* Pseudosarcomatous fibromatosis - Other connective tissue disease */
'M726' = '211' /* Necrotizing fasciitis - Other connective tissue disease */
'M728' = '211' /* Other fibroblastic disorders - Other connective tissue disease */
'M729' = '211' /* Fibroblastic disorder, unspecified - Other connective tissue disease */
'M7500' = '211' /* Adhesive capsulitis of unspecified shoulder - Other connective tissue disease */
'M7501' = '211' /* Adhesive capsulitis of right shoulder - Other connective tissue disease */
'M7502' = '211' /* Adhesive capsulitis of left shoulder - Other connective tissue disease */
'M75100' = '211' /* Unsp rotatr-cuff tear/ruptr of unsp shoulder, not trauma - Other connective tissue disease */
'M75101' = '211' /* Unsp rotatr-cuff tear/ruptr of right shoulder, not trauma - Other connective tissue disease */
'M75102' = '211' /* Unsp rotatr-cuff tear/ruptr of left shoulder, not trauma - Other connective tissue disease */
'M75110' = '211' /* Incmpl rotatr-cuff tear/ruptr of unsp shoulder, not trauma - Other connective tissue disease */
'M75111' = '211' /* Incomplete rotatr-cuff tear/ruptr of r shoulder, not trauma - Other connective tissue disease */
'M75112' = '211' /* Incomplete rotatr-cuff tear/ruptr of l shoulder, not trauma - Other connective tissue disease */
'M75120' = '211' /* Complete rotatr-cuff tear/ruptr of unsp shoulder, not trauma - Other connective tissue disease */
'M75121' = '211' /* Complete rotatr-cuff tear/ruptr of r shoulder, not trauma - Other connective tissue disease */
'M75122' = '211' /* Complete rotatr-cuff tear/ruptr of left shoulder, not trauma - Other connective tissue disease */
'M7520' = '211' /* Bicipital tendinitis, unspecified shoulder - Other connective tissue disease */
'M7521' = '211' /* Bicipital tendinitis, right shoulder - Other connective tissue disease */
'M7522' = '211' /* Bicipital tendinitis, left shoulder - Other connective tissue disease */
'M7530' = '211' /* Calcific tendinitis of unspecified shoulder - Other connective tissue disease */
'M7531' = '211' /* Calcific tendinitis of right shoulder - Other connective tissue disease */
'M7532' = '211' /* Calcific tendinitis of left shoulder - Other connective tissue disease */
'M7540' = '211' /* Impingement syndrome of unspecified shoulder - Other connective tissue disease */
'M7541' = '211' /* Impingement syndrome of right shoulder - Other connective tissue disease */
'M7542' = '211' /* Impingement syndrome of left shoulder - Other connective tissue disease */
'M7550' = '211' /* Bursitis of unspecified shoulder - Other connective tissue disease */
'M7551' = '211' /* Bursitis of right shoulder - Other connective tissue disease */
'M7552' = '211' /* Bursitis of left shoulder - Other connective tissue disease */
'M7580' = '211' /* Other shoulder lesions, unspecified shoulder - Other connective tissue disease */
'M7581' = '211' /* Other shoulder lesions, right shoulder - Other connective tissue disease */
'M7582' = '211' /* Other shoulder lesions, left shoulder - Other connective tissue disease */
'M7590' = '211' /* Shoulder lesion, unspecified, unspecified shoulder - Other connective tissue disease */
'M7591' = '211' /* Shoulder lesion, unspecified, right shoulder - Other connective tissue disease */
'M7592' = '211' /* Shoulder lesion, unspecified, left shoulder - Other connective tissue disease */
'M7600' = '211' /* Gluteal tendinitis, unspecified hip - Other connective tissue disease */
'M7601' = '211' /* Gluteal tendinitis, right hip - Other connective tissue disease */
'M7602' = '211' /* Gluteal tendinitis, left hip - Other connective tissue disease */
'M7610' = '211' /* Psoas tendinitis, unspecified hip - Other connective tissue disease */
'M7611' = '211' /* Psoas tendinitis, right hip - Other connective tissue disease */
'M7612' = '211' /* Psoas tendinitis, left hip - Other connective tissue disease */
'M7620' = '211' /* Iliac crest spur, unspecified hip - Other connective tissue disease */
'M7621' = '211' /* Iliac crest spur, right hip - Other connective tissue disease */
'M7622' = '211' /* Iliac crest spur, left hip - Other connective tissue disease */
'M7630' = '211' /* Iliotibial band syndrome, unspecified leg - Other connective tissue disease */
'M7631' = '211' /* Iliotibial band syndrome, right leg - Other connective tissue disease */
'M7632' = '211' /* Iliotibial band syndrome, left leg - Other connective tissue disease */
'M7640' = '211' /* Tibial collateral bursitis, unspecified leg - Other connective tissue disease */
'M7641' = '211' /* Tibial collateral bursitis [Pellegrini-Stieda], right leg - Other connective tissue disease */
'M7642' = '211' /* Tibial collateral bursitis [Pellegrini-Stieda], left leg - Other connective tissue disease */
'M7650' = '211' /* Patellar tendinitis, unspecified knee - Other connective tissue disease */
'M7651' = '211' /* Patellar tendinitis, right knee - Other connective tissue disease */
'M7652' = '211' /* Patellar tendinitis, left knee - Other connective tissue disease */
'M7660' = '211' /* Achilles tendinitis, unspecified leg - Other connective tissue disease */
'M7661' = '211' /* Achilles tendinitis, right leg - Other connective tissue disease */
'M7662' = '211' /* Achilles tendinitis, left leg - Other connective tissue disease */
'M7670' = '211' /* Peroneal tendinitis, unspecified leg - Other connective tissue disease */
'M7671' = '211' /* Peroneal tendinitis, right leg - Other connective tissue disease */
'M7672' = '211' /* Peroneal tendinitis, left leg - Other connective tissue disease */
'M76811' = '211' /* Anterior tibial syndrome, right leg - Other connective tissue disease */
'M76812' = '211' /* Anterior tibial syndrome, left leg - Other connective tissue disease */
'M76819' = '211' /* Anterior tibial syndrome, unspecified leg - Other connective tissue disease */
'M76821' = '211' /* Posterior tibial tendinitis, right leg - Other connective tissue disease */
'M76822' = '211' /* Posterior tibial tendinitis, left leg - Other connective tissue disease */
'M76829' = '211' /* Posterior tibial tendinitis, unspecified leg - Other connective tissue disease */
'M76891' = '211' /* Oth enthesopathies of right lower limb, excluding foot - Other connective tissue disease */
'M76892' = '211' /* Oth enthesopathies of left lower limb, excluding foot - Other connective tissue disease */
'M76899' = '211' /* Oth enthesopathies of unspecified lower limb, excluding foot - Other connective tissue disease */
'M769' = '211' /* Unspecified enthesopathy, lower limb, excluding foot - Other connective tissue disease */
'M7700' = '211' /* Medial epicondylitis, unspecified elbow - Other connective tissue disease */
'M7701' = '211' /* Medial epicondylitis, right elbow - Other connective tissue disease */
'M7702' = '211' /* Medial epicondylitis, left elbow - Other connective tissue disease */
'M7710' = '211' /* Lateral epicondylitis, unspecified elbow - Other connective tissue disease */
'M7711' = '211' /* Lateral epicondylitis, right elbow - Other connective tissue disease */
'M7712' = '211' /* Lateral epicondylitis, left elbow - Other connective tissue disease */
'M7720' = '211' /* Periarthritis, unspecified wrist - Other connective tissue disease */
'M7721' = '211' /* Periarthritis, right wrist - Other connective tissue disease */
'M7722' = '211' /* Periarthritis, left wrist - Other connective tissue disease */
'M7730' = '211' /* Calcaneal spur, unspecified foot - Other connective tissue disease */
'M7731' = '211' /* Calcaneal spur, right foot - Other connective tissue disease */
'M7732' = '211' /* Calcaneal spur, left foot - Other connective tissue disease */
'M7740' = '211' /* Metatarsalgia, unspecified foot - Other connective tissue disease */
'M7741' = '211' /* Metatarsalgia, right foot - Other connective tissue disease */
'M7742' = '211' /* Metatarsalgia, left foot - Other connective tissue disease */
'M7750' = '211' /* Other enthesopathy of unspecified foot - Other connective tissue disease */
'M7751' = '211' /* Other enthesopathy of right foot - Other connective tissue disease */
'M7752' = '211' /* Other enthesopathy of left foot - Other connective tissue disease */
'M778' = '211' /* Other enthesopathies, not elsewhere classified - Other connective tissue disease */
'M779' = '211' /* Enthesopathy, unspecified - Other connective tissue disease */
'M790' = '211' /* Rheumatism, unspecified - Other connective tissue disease */
'M791' = '211' /* Myalgia - Other connective tissue disease */
'M792' = '211' /* Neuralgia and neuritis, unspecified - Other connective tissue disease */
'M793' = '211' /* Panniculitis, unspecified - Other connective tissue disease */
'M794' = '211' /* Hypertrophy of (infrapatellar) fat pad - Other connective tissue disease */
'M795' = '211' /* Residual foreign body in soft tissue - Other connective tissue disease */
'M79601' = '211' /* Pain in right arm - Other connective tissue disease */
'M79602' = '211' /* Pain in left arm - Other connective tissue disease */
'M79603' = '211' /* Pain in arm, unspecified - Other connective tissue disease */
'M79604' = '211' /* Pain in right leg - Other connective tissue disease */
'M79605' = '211' /* Pain in left leg - Other connective tissue disease */
'M79606' = '211' /* Pain in leg, unspecified - Other connective tissue disease */
'M79609' = '211' /* Pain in unspecified limb - Other connective tissue disease */
'M79621' = '211' /* Pain in right upper arm - Other connective tissue disease */
'M79622' = '211' /* Pain in left upper arm - Other connective tissue disease */
'M79629' = '211' /* Pain in unspecified upper arm - Other connective tissue disease */
'M79631' = '211' /* Pain in right forearm - Other connective tissue disease */
'M79632' = '211' /* Pain in left forearm - Other connective tissue disease */
'M79639' = '211' /* Pain in unspecified forearm - Other connective tissue disease */
'M79641' = '211' /* Pain in right hand - Other connective tissue disease */
'M79642' = '211' /* Pain in left hand - Other connective tissue disease */
'M79643' = '211' /* Pain in unspecified hand - Other connective tissue disease */
'M79644' = '211' /* Pain in right finger(s) - Other connective tissue disease */
'M79645' = '211' /* Pain in left finger(s) - Other connective tissue disease */
'M79646' = '211' /* Pain in unspecified finger(s) - Other connective tissue disease */
'M79651' = '211' /* Pain in right thigh - Other connective tissue disease */
'M79652' = '211' /* Pain in left thigh - Other connective tissue disease */
'M79659' = '211' /* Pain in unspecified thigh - Other connective tissue disease */
'M79661' = '211' /* Pain in right lower leg - Other connective tissue disease */
'M79662' = '211' /* Pain in left lower leg - Other connective tissue disease */
'M79669' = '211' /* Pain in unspecified lower leg - Other connective tissue disease */
'M79671' = '211' /* Pain in right foot - Other connective tissue disease */
'M79672' = '211' /* Pain in left foot - Other connective tissue disease */
'M79673' = '211' /* Pain in unspecified foot - Other connective tissue disease */
'M79674' = '211' /* Pain in right toe(s) - Other connective tissue disease */
'M79675' = '211' /* Pain in left toe(s) - Other connective tissue disease */
'M79676' = '211' /* Pain in unspecified toe(s) - Other connective tissue disease */
'M797' = '211' /* Fibromyalgia - Other connective tissue disease */
'M7981' = '211' /* Nontraumatic hematoma of soft tissue - Other connective tissue disease */
'M7989' = '211' /* Other specified soft tissue disorders - Other connective tissue disease */
'M799' = '211' /* Soft tissue disorder, unspecified - Other connective tissue disease */
'M79A11' = '211' /* Nontraumatic compartment syndrome of right upper extremity - Other connective tissue disease */
'M79A12' = '211' /* Nontraumatic compartment syndrome of left upper extremity - Other connective tissue disease */
'M79A19' = '211' /* Nontraumatic compartment syndrome of unsp upper extremity - Other connective tissue disease */
'M79A21' = '211' /* Nontraumatic compartment syndrome of right lower extremity - Other connective tissue disease */
'M79A22' = '211' /* Nontraumatic compartment syndrome of left lower extremity - Other connective tissue disease */
'M79A29' = '211' /* Nontraumatic compartment syndrome of unsp lower extremity - Other connective tissue disease */
'M79A3' = '211' /* Nontraumatic compartment syndrome of abdomen - Other connective tissue disease */
'M79A9' = '211' /* Nontraumatic compartment syndrome of other sites - Other connective tissue disease */
'R252' = '211' /* Cramp and spasm - Other connective tissue disease */
'R293' = '211' /* Abnormal posture - Other connective tissue disease */
'R296' = '211' /* Repeated falls - Other connective tissue disease */
'R29810' = '211' /* Facial weakness - Other connective tissue disease */
'R29818' = '211' /* Other symptoms and signs involving the nervous system - Other connective tissue disease */
'R29890' = '211' /* Loss of height - Other connective tissue disease */
'R29898' = '211' /* Oth symptoms and signs involving the musculoskeletal system - Other connective tissue disease */
'R2990' = '211' /* Unspecified symptoms and signs involving the nervous system - Other connective tissue disease */
'R2991' = '211' /* Unsp symptoms and signs involving the musculoskeletal system - Other connective tissue disease */
'Z8739' = '211' /* Personal history of diseases of the ms sys and conn tiss - Other connective tissue disease */
'Z9660' = '211' /* Presence of unspecified orthopedic joint implant - Other connective tissue disease */
'Z96611' = '211' /* Presence of right artificial shoulder joint - Other connective tissue disease */
'Z96612' = '211' /* Presence of left artificial shoulder joint - Other connective tissue disease */
'Z96619' = '211' /* Presence of unspecified artificial shoulder joint - Other connective tissue disease */
'Z96621' = '211' /* Presence of right artificial elbow joint - Other connective tissue disease */
'Z96622' = '211' /* Presence of left artificial elbow joint - Other connective tissue disease */
'Z96629' = '211' /* Presence of unspecified artificial elbow joint - Other connective tissue disease */
'Z96631' = '211' /* Presence of right artificial wrist joint - Other connective tissue disease */
'Z96632' = '211' /* Presence of left artificial wrist joint - Other connective tissue disease */
'Z96639' = '211' /* Presence of unspecified artificial wrist joint - Other connective tissue disease */
'Z96641' = '211' /* Presence of right artificial hip joint - Other connective tissue disease */
'Z96642' = '211' /* Presence of left artificial hip joint - Other connective tissue disease */
'Z96643' = '211' /* Presence of artificial hip joint, bilateral - Other connective tissue disease */
'Z96649' = '211' /* Presence of unspecified artificial hip joint - Other connective tissue disease */
'Z96651' = '211' /* Presence of right artificial knee joint - Other connective tissue disease */
'Z96652' = '211' /* Presence of left artificial knee joint - Other connective tissue disease */
'Z96653' = '211' /* Presence of artificial knee joint, bilateral - Other connective tissue disease */
'Z96659' = '211' /* Presence of unspecified artificial knee joint - Other connective tissue disease */
'Z96661' = '211' /* Presence of right artificial ankle joint - Other connective tissue disease */
'Z96662' = '211' /* Presence of left artificial ankle joint - Other connective tissue disease */
'Z96669' = '211' /* Presence of unspecified artificial ankle joint - Other connective tissue disease */
'Z96691' = '211' /* Finger-joint replacement of right hand - Other connective tissue disease */
'Z96692' = '211' /* Finger-joint replacement of left hand - Other connective tissue disease */
'Z96693' = '211' /* Finger-joint replacement, bilateral - Other connective tissue disease */
'Z96698' = '211' /* Presence of other orthopedic joint implants - Other connective tissue disease */
'Z967' = '211' /* Presence of other bone and tendon implants - Other connective tissue disease */
'Z9710' = '211' /* Presence of artificial limb (complete) (partial), unsp - Other connective tissue disease */
'Z9711' = '211' /* Presence of artificial right arm (complete) (partial) - Other connective tissue disease */
'Z9712' = '211' /* Presence of artificial left arm (complete) (partial) - Other connective tissue disease */
'Z9713' = '211' /* Presence of artificial right leg (complete) (partial) - Other connective tissue disease */
'Z9714' = '211' /* Presence of artificial left leg (complete) (partial) - Other connective tissue disease */
'Z9715' = '211' /* Presence of artificial arms, bilateral (complete) (partial) - Other connective tissue disease */
'Z9716' = '211' /* Presence of artificial legs, bilateral (complete) (partial) - Other connective tissue disease */
'Z981' = '211' /* Arthrodesis status - Other connective tissue disease */
'P293' = '213' /* Persistent fetal circulation - Cardiac and circulatory congenital anomalies */
'Q200' = '213' /* Common arterial trunk - Cardiac and circulatory congenital anomalies */
'Q201' = '213' /* Double outlet right ventricle - Cardiac and circulatory congenital anomalies */
'Q202' = '213' /* Double outlet left ventricle - Cardiac and circulatory congenital anomalies */
'Q203' = '213' /* Discordant ventriculoarterial connection - Cardiac and circulatory congenital anomalies */
'Q204' = '213' /* Double inlet ventricle - Cardiac and circulatory congenital anomalies */
'Q205' = '213' /* Discordant atrioventricular connection - Cardiac and circulatory congenital anomalies */
'Q206' = '213' /* Isomerism of atrial appendages - Cardiac and circulatory congenital anomalies */
'Q208' = '213' /* Oth congenital malform of cardiac chambers and connections - Cardiac and circulatory congenital anomalies */
'Q209' = '213' /* Congenital malform of cardiac chambers and connections, unsp - Cardiac and circulatory congenital anomalies */
'Q210' = '213' /* Ventricular septal defect - Cardiac and circulatory congenital anomalies */
'Q211' = '213' /* Atrial septal defect - Cardiac and circulatory congenital anomalies */
'Q212' = '213' /* Atrioventricular septal defect - Cardiac and circulatory congenital anomalies */
'Q213' = '213' /* Tetralogy of Fallot - Cardiac and circulatory congenital anomalies */
'Q214' = '213' /* Aortopulmonary septal defect - Cardiac and circulatory congenital anomalies */
'Q218' = '213' /* Other congenital malformations of cardiac septa - Cardiac and circulatory congenital anomalies */
'Q219' = '213' /* Congenital malformation of cardiac septum, unspecified - Cardiac and circulatory congenital anomalies */
'Q220' = '213' /* Pulmonary valve atresia - Cardiac and circulatory congenital anomalies */
'Q221' = '213' /* Congenital pulmonary valve stenosis - Cardiac and circulatory congenital anomalies */
'Q222' = '213' /* Congenital pulmonary valve insufficiency - Cardiac and circulatory congenital anomalies */
'Q223' = '213' /* Other congenital malformations of pulmonary valve - Cardiac and circulatory congenital anomalies */
'Q224' = '213' /* Congenital tricuspid stenosis - Cardiac and circulatory congenital anomalies */
'Q225' = '213' /* Ebstein's anomaly - Cardiac and circulatory congenital anomalies */
'Q226' = '213' /* Hypoplastic right heart syndrome - Cardiac and circulatory congenital anomalies */
'Q228' = '213' /* Other congenital malformations of tricuspid valve - Cardiac and circulatory congenital anomalies */
'Q229' = '213' /* Congenital malformation of tricuspid valve, unspecified - Cardiac and circulatory congenital anomalies */
'Q230' = '213' /* Congenital stenosis of aortic valve - Cardiac and circulatory congenital anomalies */
'Q231' = '213' /* Congenital insufficiency of aortic valve - Cardiac and circulatory congenital anomalies */
'Q232' = '213' /* Congenital mitral stenosis - Cardiac and circulatory congenital anomalies */
'Q233' = '213' /* Congenital mitral insufficiency - Cardiac and circulatory congenital anomalies */
'Q234' = '213' /* Hypoplastic left heart syndrome - Cardiac and circulatory congenital anomalies */
'Q238' = '213' /* Other congenital malformations of aortic and mitral valves - Cardiac and circulatory congenital anomalies */
'Q239' = '213' /* Congenital malformation of aortic and mitral valves, unsp - Cardiac and circulatory congenital anomalies */
'Q240' = '213' /* Dextrocardia - Cardiac and circulatory congenital anomalies */
'Q241' = '213' /* Levocardia - Cardiac and circulatory congenital anomalies */
'Q242' = '213' /* Cor triatriatum - Cardiac and circulatory congenital anomalies */
'Q243' = '213' /* Pulmonary infundibular stenosis - Cardiac and circulatory congenital anomalies */
'Q244' = '213' /* Congenital subaortic stenosis - Cardiac and circulatory congenital anomalies */
'Q245' = '213' /* Malformation of coronary vessels - Cardiac and circulatory congenital anomalies */
'Q246' = '213' /* Congenital heart block - Cardiac and circulatory congenital anomalies */
'Q248' = '213' /* Other specified congenital malformations of heart - Cardiac and circulatory congenital anomalies */
'Q249' = '213' /* Congenital malformation of heart, unspecified - Cardiac and circulatory congenital anomalies */
'Q250' = '213' /* Patent ductus arteriosus - Cardiac and circulatory congenital anomalies */
'Q251' = '213' /* Coarctation of aorta - Cardiac and circulatory congenital anomalies */
'Q252' = '213' /* Atresia of aorta - Cardiac and circulatory congenital anomalies */
'Q253' = '213' /* Supravalvular aortic stenosis - Cardiac and circulatory congenital anomalies */
'Q254' = '213' /* Other congenital malformations of aorta - Cardiac and circulatory congenital anomalies */
'Q255' = '213' /* Atresia of pulmonary artery - Cardiac and circulatory congenital anomalies */
'Q256' = '213' /* Stenosis of pulmonary artery - Cardiac and circulatory congenital anomalies */
'Q2571' = '213' /* Coarctation of pulmonary artery - Cardiac and circulatory congenital anomalies */
'Q2572' = '213' /* Congenital pulmonary arteriovenous malformation - Cardiac and circulatory congenital anomalies */
'Q2579' = '213' /* Other congenital malformations of pulmonary artery - Cardiac and circulatory congenital anomalies */
'Q258' = '213' /* Other congenital malformations of other great arteries - Cardiac and circulatory congenital anomalies */
'Q259' = '213' /* Congenital malformation of great arteries, unspecified - Cardiac and circulatory congenital anomalies */
'Q260' = '213' /* Congenital stenosis of vena cava - Cardiac and circulatory congenital anomalies */
'Q261' = '213' /* Persistent left superior vena cava - Cardiac and circulatory congenital anomalies */
'Q262' = '213' /* Total anomalous pulmonary venous connection - Cardiac and circulatory congenital anomalies */
'Q263' = '213' /* Partial anomalous pulmonary venous connection - Cardiac and circulatory congenital anomalies */
'Q264' = '213' /* Anomalous pulmonary venous connection, unspecified - Cardiac and circulatory congenital anomalies */
'Q265' = '213' /* Anomalous portal venous connection - Cardiac and circulatory congenital anomalies */
'Q266' = '213' /* Portal vein-hepatic artery fistula - Cardiac and circulatory congenital anomalies */
'Q268' = '213' /* Other congenital malformations of great veins - Cardiac and circulatory congenital anomalies */
'Q269' = '213' /* Congenital malformation of great vein, unspecified - Cardiac and circulatory congenital anomalies */
'Q270' = '213' /* Congenital absence and hypoplasia of umbilical artery - Cardiac and circulatory congenital anomalies */
'Q271' = '213' /* Congenital renal artery stenosis - Cardiac and circulatory congenital anomalies */
'Q272' = '213' /* Other congenital malformations of renal artery - Cardiac and circulatory congenital anomalies */
'Q2730' = '213' /* Arteriovenous malformation, site unspecified - Cardiac and circulatory congenital anomalies */
'Q2731' = '213' /* Arteriovenous malformation of vessel of upper limb - Cardiac and circulatory congenital anomalies */
'Q2732' = '213' /* Arteriovenous malformation of vessel of lower limb - Cardiac and circulatory congenital anomalies */
'Q2733' = '213' /* Arteriovenous malformation of digestive system vessel - Cardiac and circulatory congenital anomalies */
'Q2734' = '213' /* Arteriovenous malformation of renal vessel - Cardiac and circulatory congenital anomalies */
'Q2739' = '213' /* Arteriovenous malformation, other site - Cardiac and circulatory congenital anomalies */
'Q274' = '213' /* Congenital phlebectasia - Cardiac and circulatory congenital anomalies */
'Q278' = '213' /* Oth congenital malformations of peripheral vascular system - Cardiac and circulatory congenital anomalies */
'Q279' = '213' /* Congenital malformation of peripheral vascular system, unsp - Cardiac and circulatory congenital anomalies */
'Q280' = '213' /* Arteriovenous malformation of precerebral vessels - Cardiac and circulatory congenital anomalies */
'Q281' = '213' /* Other malformations of precerebral vessels - Cardiac and circulatory congenital anomalies */
'Q282' = '213' /* Arteriovenous malformation of cerebral vessels - Cardiac and circulatory congenital anomalies */
'Q283' = '213' /* Other malformations of cerebral vessels - Cardiac and circulatory congenital anomalies */
'Q288' = '213' /* Oth congenital malformations of circulatory system - Cardiac and circulatory congenital anomalies */
'Q289' = '213' /* Congenital malformation of circulatory system, unspecified - Cardiac and circulatory congenital anomalies */
'Z8774' = '213' /* Personal history of congenital malform of heart and circ sys - Cardiac and circulatory congenital anomalies */
'Q380' = '214' /* Congenital malformations of lips, not elsewhere classified - Digestive congenital anomalies */
'Q381' = '214' /* Ankyloglossia - Digestive congenital anomalies */
'Q382' = '214' /* Macroglossia - Digestive congenital anomalies */
'Q383' = '214' /* Other congenital malformations of tongue - Digestive congenital anomalies */
'Q384' = '214' /* Congenital malformations of salivary glands and ducts - Digestive congenital anomalies */
'Q385' = '214' /* Congenital malformations of palate, not elsewhere classified - Digestive congenital anomalies */
'Q386' = '214' /* Other congenital malformations of mouth - Digestive congenital anomalies */
'Q387' = '214' /* Congenital pharyngeal pouch - Digestive congenital anomalies */
'Q388' = '214' /* Other congenital malformations of pharynx - Digestive congenital anomalies */
'Q390' = '214' /* Atresia of esophagus without fistula - Digestive congenital anomalies */
'Q391' = '214' /* Atresia of esophagus with tracheo-esophageal fistula - Digestive congenital anomalies */
'Q392' = '214' /* Congenital tracheo-esophageal fistula without atresia - Digestive congenital anomalies */
'Q393' = '214' /* Congenital stenosis and stricture of esophagus - Digestive congenital anomalies */
'Q394' = '214' /* Esophageal web - Digestive congenital anomalies */
'Q395' = '214' /* Congenital dilatation of esophagus - Digestive congenital anomalies */
'Q396' = '214' /* Congenital diverticulum of esophagus - Digestive congenital anomalies */
'Q398' = '214' /* Other congenital malformations of esophagus - Digestive congenital anomalies */
'Q399' = '214' /* Congenital malformation of esophagus, unspecified - Digestive congenital anomalies */
'Q400' = '214' /* Congenital hypertrophic pyloric stenosis - Digestive congenital anomalies */
'Q401' = '214' /* Congenital hiatus hernia - Digestive congenital anomalies */
'Q402' = '214' /* Other specified congenital malformations of stomach - Digestive congenital anomalies */
'Q403' = '214' /* Congenital malformation of stomach, unspecified - Digestive congenital anomalies */
'Q408' = '214' /* Oth congenital malformations of upper alimentary tract - Digestive congenital anomalies */
'Q409' = '214' /* Congenital malformation of upper alimentary tract, unsp - Digestive congenital anomalies */
'Q410' = '214' /* Congenital absence, atresia and stenosis of duodenum - Digestive congenital anomalies */
'Q411' = '214' /* Congenital absence, atresia and stenosis of jejunum - Digestive congenital anomalies */
'Q412' = '214' /* Congenital absence, atresia and stenosis of ileum - Digestive congenital anomalies */
'Q418' = '214' /* Congenital absence, atresia and stenosis of prt sm int - Digestive congenital anomalies */
'Q419' = '214' /* Congen absence, atresia and stenosis of sm int, part unsp - Digestive congenital anomalies */
'Q420' = '214' /* Congenital absence, atresia and stenosis of rectum w fistula - Digestive congenital anomalies */
'Q421' = '214' /* Congen absence, atresia and stenosis of rectum w/o fistula - Digestive congenital anomalies */
'Q422' = '214' /* Congenital absence, atresia and stenosis of anus w fistula - Digestive congenital anomalies */
'Q423' = '214' /* Congenital absence, atresia and stenosis of anus w/o fistula - Digestive congenital anomalies */
'Q428' = '214' /* Congenital absence, atresia and stenosis of prt lg int - Digestive congenital anomalies */
'Q429' = '214' /* Congen absence, atresia and stenosis of lg int, part unsp - Digestive congenital anomalies */
'Q430' = '214' /* Meckel's diverticulum (displaced) (hypertrophic) - Digestive congenital anomalies */
'Q431' = '214' /* Hirschsprung's disease - Digestive congenital anomalies */
'Q432' = '214' /* Other congenital functional disorders of colon - Digestive congenital anomalies */
'Q433' = '214' /* Congenital malformations of intestinal fixation - Digestive congenital anomalies */
'Q434' = '214' /* Duplication of intestine - Digestive congenital anomalies */
'Q435' = '214' /* Ectopic anus - Digestive congenital anomalies */
'Q436' = '214' /* Congenital fistula of rectum and anus - Digestive congenital anomalies */
'Q437' = '214' /* Persistent cloaca - Digestive congenital anomalies */
'Q438' = '214' /* Other specified congenital malformations of intestine - Digestive congenital anomalies */
'Q439' = '214' /* Congenital malformation of intestine, unspecified - Digestive congenital anomalies */
'Q440' = '214' /* Agenesis, aplasia and hypoplasia of gallbladder - Digestive congenital anomalies */
'Q441' = '214' /* Other congenital malformations of gallbladder - Digestive congenital anomalies */
'Q442' = '214' /* Atresia of bile ducts - Digestive congenital anomalies */
'Q443' = '214' /* Congenital stenosis and stricture of bile ducts - Digestive congenital anomalies */
'Q444' = '214' /* Choledochal cyst - Digestive congenital anomalies */
'Q445' = '214' /* Other congenital malformations of bile ducts - Digestive congenital anomalies */
'Q446' = '214' /* Cystic disease of liver - Digestive congenital anomalies */
'Q447' = '214' /* Other congenital malformations of liver - Digestive congenital anomalies */
'Q450' = '214' /* Agenesis, aplasia and hypoplasia of pancreas - Digestive congenital anomalies */
'Q451' = '214' /* Annular pancreas - Digestive congenital anomalies */
'Q452' = '214' /* Congenital pancreatic cyst - Digestive congenital anomalies */
'Q453' = '214' /* Oth congenital malformations of pancreas and pancreatic duct - Digestive congenital anomalies */
'Q458' = '214' /* Other specified congenital malformations of digestive system - Digestive congenital anomalies */
'Q459' = '214' /* Congenital malformation of digestive system, unspecified - Digestive congenital anomalies */
'Z87738' = '214' /* Personal history of congenital malform of dgstv sys - Digestive congenital anomalies */
'Q5001' = '215' /* Congenital absence of ovary, unilateral - Genitourinary congenital anomalies */
'Q5002' = '215' /* Congenital absence of ovary, bilateral - Genitourinary congenital anomalies */
'Q501' = '215' /* Developmental ovarian cyst - Genitourinary congenital anomalies */
'Q502' = '215' /* Congenital torsion of ovary - Genitourinary congenital anomalies */
'Q5031' = '215' /* Accessory ovary - Genitourinary congenital anomalies */
'Q5032' = '215' /* Ovarian streak - Genitourinary congenital anomalies */
'Q5039' = '215' /* Other congenital malformation of ovary - Genitourinary congenital anomalies */
'Q504' = '215' /* Embryonic cyst of fallopian tube - Genitourinary congenital anomalies */
'Q505' = '215' /* Embryonic cyst of broad ligament - Genitourinary congenital anomalies */
'Q506' = '215' /* Oth congenital malformations of fallop and broad ligament - Genitourinary congenital anomalies */
'Q510' = '215' /* Agenesis and aplasia of uterus - Genitourinary congenital anomalies */
'Q5110' = '215' /* Doubling of uterus w doubling of cervix and vagina w/o obst - Genitourinary congenital anomalies */
'Q5111' = '215' /* Doubling of uterus w doubling of cervix and vagina w obst - Genitourinary congenital anomalies */
'Q512' = '215' /* Other doubling of uterus - Genitourinary congenital anomalies */
'Q513' = '215' /* Bicornate uterus - Genitourinary congenital anomalies */
'Q514' = '215' /* Unicornate uterus - Genitourinary congenital anomalies */
'Q515' = '215' /* Agenesis and aplasia of cervix - Genitourinary congenital anomalies */
'Q516' = '215' /* Embryonic cyst of cervix - Genitourinary congenital anomalies */
'Q517' = '215' /* Congen fistulae betw uterus and digestive and urinary tracts - Genitourinary congenital anomalies */
'Q51810' = '215' /* Arcuate uterus - Genitourinary congenital anomalies */
'Q51811' = '215' /* Hypoplasia of uterus - Genitourinary congenital anomalies */
'Q51818' = '215' /* Other congenital malformations of uterus - Genitourinary congenital anomalies */
'Q51820' = '215' /* Cervical duplication - Genitourinary congenital anomalies */
'Q51821' = '215' /* Hypoplasia of cervix - Genitourinary congenital anomalies */
'Q51828' = '215' /* Other congenital malformations of cervix - Genitourinary congenital anomalies */
'Q519' = '215' /* Congenital malformation of uterus and cervix, unspecified - Genitourinary congenital anomalies */
'Q520' = '215' /* Congenital absence of vagina - Genitourinary congenital anomalies */
'Q5210' = '215' /* Doubling of vagina, unspecified - Genitourinary congenital anomalies */
'Q5211' = '215' /* Transverse vaginal septum - Genitourinary congenital anomalies */
'Q5212' = '215' /* Longitudinal vaginal septum - Genitourinary congenital anomalies */
'Q522' = '215' /* Congenital rectovaginal fistula - Genitourinary congenital anomalies */
'Q523' = '215' /* Imperforate hymen - Genitourinary congenital anomalies */
'Q524' = '215' /* Other congenital malformations of vagina - Genitourinary congenital anomalies */
'Q525' = '215' /* Fusion of labia - Genitourinary congenital anomalies */
'Q526' = '215' /* Congenital malformation of clitoris - Genitourinary congenital anomalies */
'Q5270' = '215' /* Unspecified congenital malformations of vulva - Genitourinary congenital anomalies */
'Q5271' = '215' /* Congenital absence of vulva - Genitourinary congenital anomalies */
'Q5279' = '215' /* Other congenital malformations of vulva - Genitourinary congenital anomalies */
'Q528' = '215' /* Other specified congenital malformations of female genitalia - Genitourinary congenital anomalies */
'Q529' = '215' /* Congenital malformation of female genitalia, unspecified - Genitourinary congenital anomalies */
'Q5300' = '215' /* Ectopic testis, unspecified - Genitourinary congenital anomalies */
'Q5301' = '215' /* Ectopic testis, unilateral - Genitourinary congenital anomalies */
'Q5302' = '215' /* Ectopic testes, bilateral - Genitourinary congenital anomalies */
'Q5310' = '215' /* Unspecified undescended testicle, unilateral - Genitourinary congenital anomalies */
'Q5311' = '215' /* Abdominal testis, unilateral - Genitourinary congenital anomalies */
'Q5312' = '215' /* Ectopic perineal testis, unilateral - Genitourinary congenital anomalies */
'Q5320' = '215' /* Undescended testicle, unspecified, bilateral - Genitourinary congenital anomalies */
'Q5321' = '215' /* Abdominal testis, bilateral - Genitourinary congenital anomalies */
'Q5322' = '215' /* Ectopic perineal testis, bilateral - Genitourinary congenital anomalies */
'Q539' = '215' /* Undescended testicle, unspecified - Genitourinary congenital anomalies */
'Q540' = '215' /* Hypospadias, balanic - Genitourinary congenital anomalies */
'Q541' = '215' /* Hypospadias, penile - Genitourinary congenital anomalies */
'Q542' = '215' /* Hypospadias, penoscrotal - Genitourinary congenital anomalies */
'Q543' = '215' /* Hypospadias, perineal - Genitourinary congenital anomalies */
'Q544' = '215' /* Congenital chordee - Genitourinary congenital anomalies */
'Q548' = '215' /* Other hypospadias - Genitourinary congenital anomalies */
'Q549' = '215' /* Hypospadias, unspecified - Genitourinary congenital anomalies */
'Q550' = '215' /* Absence and aplasia of testis - Genitourinary congenital anomalies */
'Q551' = '215' /* Hypoplasia of testis and scrotum - Genitourinary congenital anomalies */
'Q5520' = '215' /* Unspecified congenital malformations of testis and scrotum - Genitourinary congenital anomalies */
'Q5521' = '215' /* Polyorchism - Genitourinary congenital anomalies */
'Q5522' = '215' /* Retractile testis - Genitourinary congenital anomalies */
'Q5523' = '215' /* Scrotal transposition - Genitourinary congenital anomalies */
'Q5529' = '215' /* Other congenital malformations of testis and scrotum - Genitourinary congenital anomalies */
'Q553' = '215' /* Atresia of vas deferens - Genitourinary congenital anomalies */
'Q554' = '215' /* Oth congen malform of vas def,epidid, semnl vescl & prostate - Genitourinary congenital anomalies */
'Q555' = '215' /* Congenital absence and aplasia of penis - Genitourinary congenital anomalies */
'Q5561' = '215' /* Curvature of penis (lateral) - Genitourinary congenital anomalies */
'Q5562' = '215' /* Hypoplasia of penis - Genitourinary congenital anomalies */
'Q5563' = '215' /* Congenital torsion of penis - Genitourinary congenital anomalies */
'Q5564' = '215' /* Hidden penis - Genitourinary congenital anomalies */
'Q5569' = '215' /* Other congenital malformation of penis - Genitourinary congenital anomalies */
'Q557' = '215' /* Congenital vasocutaneous fistula - Genitourinary congenital anomalies */
'Q558' = '215' /* Oth congenital malformations of male genital organs - Genitourinary congenital anomalies */
'Q559' = '215' /* Congenital malformation of male genital organ, unspecified - Genitourinary congenital anomalies */
'Q560' = '215' /* Hermaphroditism, not elsewhere classified - Genitourinary congenital anomalies */
'Q561' = '215' /* Male pseudohermaphroditism, not elsewhere classified - Genitourinary congenital anomalies */
'Q562' = '215' /* Female pseudohermaphroditism, not elsewhere classified - Genitourinary congenital anomalies */
'Q563' = '215' /* Pseudohermaphroditism, unspecified - Genitourinary congenital anomalies */
'Q564' = '215' /* Indeterminate sex, unspecified - Genitourinary congenital anomalies */
'Q600' = '215' /* Renal agenesis, unilateral - Genitourinary congenital anomalies */
'Q601' = '215' /* Renal agenesis, bilateral - Genitourinary congenital anomalies */
'Q602' = '215' /* Renal agenesis, unspecified - Genitourinary congenital anomalies */
'Q603' = '215' /* Renal hypoplasia, unilateral - Genitourinary congenital anomalies */
'Q604' = '215' /* Renal hypoplasia, bilateral - Genitourinary congenital anomalies */
'Q605' = '215' /* Renal hypoplasia, unspecified - Genitourinary congenital anomalies */
'Q606' = '215' /* Potter's syndrome - Genitourinary congenital anomalies */
'Q6100' = '215' /* Congenital renal cyst, unspecified - Genitourinary congenital anomalies */
'Q6101' = '215' /* Congenital single renal cyst - Genitourinary congenital anomalies */
'Q6102' = '215' /* Congenital multiple renal cysts - Genitourinary congenital anomalies */
'Q6111' = '215' /* Cystic dilatation of collecting ducts - Genitourinary congenital anomalies */
'Q6119' = '215' /* Other polycystic kidney, infantile type - Genitourinary congenital anomalies */
'Q612' = '215' /* Polycystic kidney, adult type - Genitourinary congenital anomalies */
'Q613' = '215' /* Polycystic kidney, unspecified - Genitourinary congenital anomalies */
'Q614' = '215' /* Renal dysplasia - Genitourinary congenital anomalies */
'Q615' = '215' /* Medullary cystic kidney - Genitourinary congenital anomalies */
'Q618' = '215' /* Other cystic kidney diseases - Genitourinary congenital anomalies */
'Q619' = '215' /* Cystic kidney disease, unspecified - Genitourinary congenital anomalies */
'Q620' = '215' /* Congenital hydronephrosis - Genitourinary congenital anomalies */
'Q6210' = '215' /* Congenital occlusion of ureter, unspecified - Genitourinary congenital anomalies */
'Q6211' = '215' /* Congenital occlusion of ureteropelvic junction - Genitourinary congenital anomalies */
'Q6212' = '215' /* Congenital occlusion of ureterovesical orifice - Genitourinary congenital anomalies */
'Q622' = '215' /* Congenital megaureter - Genitourinary congenital anomalies */
'Q6231' = '215' /* Congenital ureterocele, orthotopic - Genitourinary congenital anomalies */
'Q6232' = '215' /* Cecoureterocele - Genitourinary congenital anomalies */
'Q6239' = '215' /* Other obstructive defects of renal pelvis and ureter - Genitourinary congenital anomalies */
'Q624' = '215' /* Agenesis of ureter - Genitourinary congenital anomalies */
'Q625' = '215' /* Duplication of ureter - Genitourinary congenital anomalies */
'Q6260' = '215' /* Malposition of ureter, unspecified - Genitourinary congenital anomalies */
'Q6261' = '215' /* Deviation of ureter - Genitourinary congenital anomalies */
'Q6262' = '215' /* Displacement of ureter - Genitourinary congenital anomalies */
'Q6263' = '215' /* Anomalous implantation of ureter - Genitourinary congenital anomalies */
'Q6269' = '215' /* Other malposition of ureter - Genitourinary congenital anomalies */
'Q627' = '215' /* Congenital vesico-uretero-renal reflux - Genitourinary congenital anomalies */
'Q628' = '215' /* Other congenital malformations of ureter - Genitourinary congenital anomalies */
'Q630' = '215' /* Accessory kidney - Genitourinary congenital anomalies */
'Q631' = '215' /* Lobulated, fused and horseshoe kidney - Genitourinary congenital anomalies */
'Q632' = '215' /* Ectopic kidney - Genitourinary congenital anomalies */
'Q633' = '215' /* Hyperplastic and giant kidney - Genitourinary congenital anomalies */
'Q638' = '215' /* Other specified congenital malformations of kidney - Genitourinary congenital anomalies */
'Q639' = '215' /* Congenital malformation of kidney, unspecified - Genitourinary congenital anomalies */
'Q640' = '215' /* Epispadias - Genitourinary congenital anomalies */
'Q6410' = '215' /* Exstrophy of urinary bladder, unspecified - Genitourinary congenital anomalies */
'Q6411' = '215' /* Supravesical fissure of urinary bladder - Genitourinary congenital anomalies */
'Q6412' = '215' /* Cloacal extrophy of urinary bladder - Genitourinary congenital anomalies */
'Q6419' = '215' /* Other exstrophy of urinary bladder - Genitourinary congenital anomalies */
'Q642' = '215' /* Congenital posterior urethral valves - Genitourinary congenital anomalies */
'Q6431' = '215' /* Congenital bladder neck obstruction - Genitourinary congenital anomalies */
'Q6432' = '215' /* Congenital stricture of urethra - Genitourinary congenital anomalies */
'Q6433' = '215' /* Congenital stricture of urinary meatus - Genitourinary congenital anomalies */
'Q6439' = '215' /* Other atresia and stenosis of urethra and bladder neck - Genitourinary congenital anomalies */
'Q644' = '215' /* Malformation of urachus - Genitourinary congenital anomalies */
'Q645' = '215' /* Congenital absence of bladder and urethra - Genitourinary congenital anomalies */
'Q646' = '215' /* Congenital diverticulum of bladder - Genitourinary congenital anomalies */
'Q6470' = '215' /* Unspecified congenital malformation of bladder and urethra - Genitourinary congenital anomalies */
'Q6471' = '215' /* Congenital prolapse of urethra - Genitourinary congenital anomalies */
'Q6472' = '215' /* Congenital prolapse of urinary meatus - Genitourinary congenital anomalies */
'Q6473' = '215' /* Congenital urethrorectal fistula - Genitourinary congenital anomalies */
'Q6474' = '215' /* Double urethra - Genitourinary congenital anomalies */
'Q6475' = '215' /* Double urinary meatus - Genitourinary congenital anomalies */
'Q6479' = '215' /* Other congenital malformations of bladder and urethra - Genitourinary congenital anomalies */
'Q648' = '215' /* Other specified congenital malformations of urinary system - Genitourinary congenital anomalies */
'Q649' = '215' /* Congenital malformation of urinary system, unspecified - Genitourinary congenital anomalies */
'Z87710' = '215' /* Personal history of (corrected) hypospadias - Genitourinary congenital anomalies */
'Z87718' = '215' /* Personal history of (corrected) congenital malform of GU sys - Genitourinary congenital anomalies */
'G901' = '216' /* Familial dysautonomia [Riley-Day] - Nervous system congenital anomalies */
'Q000' = '216' /* Anencephaly - Nervous system congenital anomalies */
'Q001' = '216' /* Craniorachischisis - Nervous system congenital anomalies */
'Q002' = '216' /* Iniencephaly - Nervous system congenital anomalies */
'Q010' = '216' /* Frontal encephalocele - Nervous system congenital anomalies */
'Q011' = '216' /* Nasofrontal encephalocele - Nervous system congenital anomalies */
'Q012' = '216' /* Occipital encephalocele - Nervous system congenital anomalies */
'Q018' = '216' /* Encephalocele of other sites - Nervous system congenital anomalies */
'Q019' = '216' /* Encephalocele, unspecified - Nervous system congenital anomalies */
'Q02' = '216' /* Microcephaly - Nervous system congenital anomalies */
'Q030' = '216' /* Malformations of aqueduct of Sylvius - Nervous system congenital anomalies */
'Q031' = '216' /* Atresia of foramina of Magendie and Luschka - Nervous system congenital anomalies */
'Q038' = '216' /* Other congenital hydrocephalus - Nervous system congenital anomalies */
'Q039' = '216' /* Congenital hydrocephalus, unspecified - Nervous system congenital anomalies */
'Q040' = '216' /* Congenital malformations of corpus callosum - Nervous system congenital anomalies */
'Q041' = '216' /* Arhinencephaly - Nervous system congenital anomalies */
'Q042' = '216' /* Holoprosencephaly - Nervous system congenital anomalies */
'Q043' = '216' /* Other reduction deformities of brain - Nervous system congenital anomalies */
'Q044' = '216' /* Septo-optic dysplasia of brain - Nervous system congenital anomalies */
'Q045' = '216' /* Megalencephaly - Nervous system congenital anomalies */
'Q046' = '216' /* Congenital cerebral cysts - Nervous system congenital anomalies */
'Q048' = '216' /* Other specified congenital malformations of brain - Nervous system congenital anomalies */
'Q049' = '216' /* Congenital malformation of brain, unspecified - Nervous system congenital anomalies */
'Q050' = '216' /* Cervical spina bifida with hydrocephalus - Nervous system congenital anomalies */
'Q051' = '216' /* Thoracic spina bifida with hydrocephalus - Nervous system congenital anomalies */
'Q052' = '216' /* Lumbar spina bifida with hydrocephalus - Nervous system congenital anomalies */
'Q053' = '216' /* Sacral spina bifida with hydrocephalus - Nervous system congenital anomalies */
'Q054' = '216' /* Unspecified spina bifida with hydrocephalus - Nervous system congenital anomalies */
'Q055' = '216' /* Cervical spina bifida without hydrocephalus - Nervous system congenital anomalies */
'Q056' = '216' /* Thoracic spina bifida without hydrocephalus - Nervous system congenital anomalies */
'Q057' = '216' /* Lumbar spina bifida without hydrocephalus - Nervous system congenital anomalies */
'Q058' = '216' /* Sacral spina bifida without hydrocephalus - Nervous system congenital anomalies */
'Q059' = '216' /* Spina bifida, unspecified - Nervous system congenital anomalies */
'Q060' = '216' /* Amyelia - Nervous system congenital anomalies */
'Q061' = '216' /* Hypoplasia and dysplasia of spinal cord - Nervous system congenital anomalies */
'Q062' = '216' /* Diastematomyelia - Nervous system congenital anomalies */
'Q063' = '216' /* Other congenital cauda equina malformations - Nervous system congenital anomalies */
'Q064' = '216' /* Hydromyelia - Nervous system congenital anomalies */
'Q068' = '216' /* Other specified congenital malformations of spinal cord - Nervous system congenital anomalies */
'Q069' = '216' /* Congenital malformation of spinal cord, unspecified - Nervous system congenital anomalies */
'Q0700' = '216' /* Arnold-Chiari syndrome without spina bifida or hydrocephalus - Nervous system congenital anomalies */
'Q0701' = '216' /* Arnold-Chiari syndrome with spina bifida - Nervous system congenital anomalies */
'Q0702' = '216' /* Arnold-Chiari syndrome with hydrocephalus - Nervous system congenital anomalies */
'Q0703' = '216' /* Arnold-Chiari syndrome with spina bifida and hydrocephalus - Nervous system congenital anomalies */
'Q078' = '216' /* Other specified congenital malformations of nervous system - Nervous system congenital anomalies */
'Q079' = '216' /* Congenital malformation of nervous system, unspecified - Nervous system congenital anomalies */
'Q8500' = '216' /* Neurofibromatosis, unspecified - Nervous system congenital anomalies */
'Q8501' = '216' /* Neurofibromatosis, type 1 - Nervous system congenital anomalies */
'Q8502' = '216' /* Neurofibromatosis, type 2 - Nervous system congenital anomalies */
'Q8503' = '216' /* Schwannomatosis - Nervous system congenital anomalies */
'Q8509' = '216' /* Other neurofibromatosis - Nervous system congenital anomalies */
'Z87728' = '216' /* Prsnl hx of congen malform of nervous sys and sense organs - Nervous system congenital anomalies */
'E7871' = '217' /* Barth syndrome - Other congenital anomalies */
'E7872' = '217' /* Smith-Lemli-Opitz syndrome - Other congenital anomalies */
'Q100' = '217' /* Congenital ptosis - Other congenital anomalies */
'Q101' = '217' /* Congenital ectropion - Other congenital anomalies */
'Q102' = '217' /* Congenital entropion - Other congenital anomalies */
'Q103' = '217' /* Other congenital malformations of eyelid - Other congenital anomalies */
'Q104' = '217' /* Absence and agenesis of lacrimal apparatus - Other congenital anomalies */
'Q105' = '217' /* Congenital stenosis and stricture of lacrimal duct - Other congenital anomalies */
'Q106' = '217' /* Other congenital malformations of lacrimal apparatus - Other congenital anomalies */
'Q107' = '217' /* Congenital malformation of orbit - Other congenital anomalies */
'Q110' = '217' /* Cystic eyeball - Other congenital anomalies */
'Q111' = '217' /* Other anophthalmos - Other congenital anomalies */
'Q112' = '217' /* Microphthalmos - Other congenital anomalies */
'Q113' = '217' /* Macrophthalmos - Other congenital anomalies */
'Q120' = '217' /* Congenital cataract - Other congenital anomalies */
'Q121' = '217' /* Congenital displaced lens - Other congenital anomalies */
'Q122' = '217' /* Coloboma of lens - Other congenital anomalies */
'Q123' = '217' /* Congenital aphakia - Other congenital anomalies */
'Q124' = '217' /* Spherophakia - Other congenital anomalies */
'Q128' = '217' /* Other congenital lens malformations - Other congenital anomalies */
'Q129' = '217' /* Congenital lens malformation, unspecified - Other congenital anomalies */
'Q130' = '217' /* Coloboma of iris - Other congenital anomalies */
'Q131' = '217' /* Absence of iris - Other congenital anomalies */
'Q132' = '217' /* Other congenital malformations of iris - Other congenital anomalies */
'Q133' = '217' /* Congenital corneal opacity - Other congenital anomalies */
'Q134' = '217' /* Other congenital corneal malformations - Other congenital anomalies */
'Q135' = '217' /* Blue sclera - Other congenital anomalies */
'Q1381' = '217' /* Rieger's anomaly - Other congenital anomalies */
'Q1389' = '217' /* Other congenital malformations of anterior segment of eye - Other congenital anomalies */
'Q139' = '217' /* Congenital malformation of anterior segment of eye, unsp - Other congenital anomalies */
'Q140' = '217' /* Congenital malformation of vitreous humor - Other congenital anomalies */
'Q141' = '217' /* Congenital malformation of retina - Other congenital anomalies */
'Q142' = '217' /* Congenital malformation of optic disc - Other congenital anomalies */
'Q143' = '217' /* Congenital malformation of choroid - Other congenital anomalies */
'Q148' = '217' /* Other congenital malformations of posterior segment of eye - Other congenital anomalies */
'Q149' = '217' /* Congenital malformation of posterior segment of eye, unsp - Other congenital anomalies */
'Q150' = '217' /* Congenital glaucoma - Other congenital anomalies */
'Q158' = '217' /* Other specified congenital malformations of eye - Other congenital anomalies */
'Q159' = '217' /* Congenital malformation of eye, unspecified - Other congenital anomalies */
'Q160' = '217' /* Congenital absence of (ear) auricle - Other congenital anomalies */
'Q161' = '217' /* Congenital absence, atresia and stricture of auditory canal - Other congenital anomalies */
'Q162' = '217' /* Absence of eustachian tube - Other congenital anomalies */
'Q163' = '217' /* Congenital malformation of ear ossicles - Other congenital anomalies */
'Q164' = '217' /* Other congenital malformations of middle ear - Other congenital anomalies */
'Q165' = '217' /* Congenital malformation of inner ear - Other congenital anomalies */
'Q169' = '217' /* Congen malform of ear causing impairment of hearing, unsp - Other congenital anomalies */
'Q170' = '217' /* Accessory auricle - Other congenital anomalies */
'Q171' = '217' /* Macrotia - Other congenital anomalies */
'Q172' = '217' /* Microtia - Other congenital anomalies */
'Q173' = '217' /* Other misshapen ear - Other congenital anomalies */
'Q174' = '217' /* Misplaced ear - Other congenital anomalies */
'Q175' = '217' /* Prominent ear - Other congenital anomalies */
'Q178' = '217' /* Other specified congenital malformations of ear - Other congenital anomalies */
'Q179' = '217' /* Congenital malformation of ear, unspecified - Other congenital anomalies */
'Q180' = '217' /* Sinus, fistula and cyst of branchial cleft - Other congenital anomalies */
'Q181' = '217' /* Preauricular sinus and cyst - Other congenital anomalies */
'Q182' = '217' /* Other branchial cleft malformations - Other congenital anomalies */
'Q183' = '217' /* Webbing of neck - Other congenital anomalies */
'Q184' = '217' /* Macrostomia - Other congenital anomalies */
'Q185' = '217' /* Microstomia - Other congenital anomalies */
'Q186' = '217' /* Macrocheilia - Other congenital anomalies */
'Q187' = '217' /* Microcheilia - Other congenital anomalies */
'Q188' = '217' /* Other specified congenital malformations of face and neck - Other congenital anomalies */
'Q189' = '217' /* Congenital malformation of face and neck, unspecified - Other congenital anomalies */
'Q300' = '217' /* Choanal atresia - Other congenital anomalies */
'Q301' = '217' /* Agenesis and underdevelopment of nose - Other congenital anomalies */
'Q302' = '217' /* Fissured, notched and cleft nose - Other congenital anomalies */
'Q303' = '217' /* Congenital perforated nasal septum - Other congenital anomalies */
'Q308' = '217' /* Other congenital malformations of nose - Other congenital anomalies */
'Q309' = '217' /* Congenital malformation of nose, unspecified - Other congenital anomalies */
'Q310' = '217' /* Web of larynx - Other congenital anomalies */
'Q311' = '217' /* Congenital subglottic stenosis - Other congenital anomalies */
'Q312' = '217' /* Laryngeal hypoplasia - Other congenital anomalies */
'Q313' = '217' /* Laryngocele - Other congenital anomalies */
'Q315' = '217' /* Congenital laryngomalacia - Other congenital anomalies */
'Q318' = '217' /* Other congenital malformations of larynx - Other congenital anomalies */
'Q319' = '217' /* Congenital malformation of larynx, unspecified - Other congenital anomalies */
'Q320' = '217' /* Congenital tracheomalacia - Other congenital anomalies */
'Q321' = '217' /* Other congenital malformations of trachea - Other congenital anomalies */
'Q322' = '217' /* Congenital bronchomalacia - Other congenital anomalies */
'Q323' = '217' /* Congenital stenosis of bronchus - Other congenital anomalies */
'Q324' = '217' /* Other congenital malformations of bronchus - Other congenital anomalies */
'Q330' = '217' /* Congenital cystic lung - Other congenital anomalies */
'Q331' = '217' /* Accessory lobe of lung - Other congenital anomalies */
'Q332' = '217' /* Sequestration of lung - Other congenital anomalies */
'Q333' = '217' /* Agenesis of lung - Other congenital anomalies */
'Q334' = '217' /* Congenital bronchiectasis - Other congenital anomalies */
'Q335' = '217' /* Ectopic tissue in lung - Other congenital anomalies */
'Q336' = '217' /* Congenital hypoplasia and dysplasia of lung - Other congenital anomalies */
'Q338' = '217' /* Other congenital malformations of lung - Other congenital anomalies */
'Q339' = '217' /* Congenital malformation of lung, unspecified - Other congenital anomalies */
'Q340' = '217' /* Anomaly of pleura - Other congenital anomalies */
'Q341' = '217' /* Congenital cyst of mediastinum - Other congenital anomalies */
'Q348' = '217' /* Oth congenital malformations of respiratory system - Other congenital anomalies */
'Q349' = '217' /* Congenital malformation of respiratory system, unspecified - Other congenital anomalies */
'Q351' = '217' /* Cleft hard palate - Other congenital anomalies */
'Q353' = '217' /* Cleft soft palate - Other congenital anomalies */
'Q355' = '217' /* Cleft hard palate with cleft soft palate - Other congenital anomalies */
'Q357' = '217' /* Cleft uvula - Other congenital anomalies */
'Q359' = '217' /* Cleft palate, unspecified - Other congenital anomalies */
'Q360' = '217' /* Cleft lip, bilateral - Other congenital anomalies */
'Q361' = '217' /* Cleft lip, median - Other congenital anomalies */
'Q369' = '217' /* Cleft lip, unilateral - Other congenital anomalies */
'Q370' = '217' /* Cleft hard palate with bilateral cleft lip - Other congenital anomalies */
'Q371' = '217' /* Cleft hard palate with unilateral cleft lip - Other congenital anomalies */
'Q372' = '217' /* Cleft soft palate with bilateral cleft lip - Other congenital anomalies */
'Q373' = '217' /* Cleft soft palate with unilateral cleft lip - Other congenital anomalies */
'Q374' = '217' /* Cleft hard and soft palate with bilateral cleft lip - Other congenital anomalies */
'Q375' = '217' /* Cleft hard and soft palate with unilateral cleft lip - Other congenital anomalies */
'Q378' = '217' /* Unspecified cleft palate with bilateral cleft lip - Other congenital anomalies */
'Q379' = '217' /* Unspecified cleft palate with unilateral cleft lip - Other congenital anomalies */
'Q6500' = '217' /* Congenital dislocation of unspecified hip, unilateral - Other congenital anomalies */
'Q6501' = '217' /* Congenital dislocation of right hip, unilateral - Other congenital anomalies */
'Q6502' = '217' /* Congenital dislocation of left hip, unilateral - Other congenital anomalies */
'Q651' = '217' /* Congenital dislocation of hip, bilateral - Other congenital anomalies */
'Q652' = '217' /* Congenital dislocation of hip, unspecified - Other congenital anomalies */
'Q6530' = '217' /* Congenital partial dislocation of unsp hip, unilateral - Other congenital anomalies */
'Q6531' = '217' /* Congenital partial dislocation of right hip, unilateral - Other congenital anomalies */
'Q6532' = '217' /* Congenital partial dislocation of left hip, unilateral - Other congenital anomalies */
'Q654' = '217' /* Congenital partial dislocation of hip, bilateral - Other congenital anomalies */
'Q655' = '217' /* Congenital partial dislocation of hip, unspecified - Other congenital anomalies */
'Q656' = '217' /* Congenital unstable hip - Other congenital anomalies */
'Q6581' = '217' /* Congenital coxa valga - Other congenital anomalies */
'Q6582' = '217' /* Congenital coxa vara - Other congenital anomalies */
'Q6589' = '217' /* Other specified congenital deformities of hip - Other congenital anomalies */
'Q659' = '217' /* Congenital deformity of hip, unspecified - Other congenital anomalies */
'Q660' = '217' /* Congenital talipes equinovarus - Other congenital anomalies */
'Q661' = '217' /* Congenital talipes calcaneovarus - Other congenital anomalies */
'Q662' = '217' /* Congenital metatarsus (primus) varus - Other congenital anomalies */
'Q663' = '217' /* Other congenital varus deformities of feet - Other congenital anomalies */
'Q664' = '217' /* Congenital talipes calcaneovalgus - Other congenital anomalies */
'Q6650' = '217' /* Congenital pes planus, unspecified foot - Other congenital anomalies */
'Q6651' = '217' /* Congenital pes planus, right foot - Other congenital anomalies */
'Q6652' = '217' /* Congenital pes planus, left foot - Other congenital anomalies */
'Q666' = '217' /* Other congenital valgus deformities of feet - Other congenital anomalies */
'Q667' = '217' /* Congenital pes cavus - Other congenital anomalies */
'Q6680' = '217' /* Congenital vertical talus deformity, unspecified foot - Other congenital anomalies */
'Q6681' = '217' /* Congenital vertical talus deformity, right foot - Other congenital anomalies */
'Q6682' = '217' /* Congenital vertical talus deformity, left foot - Other congenital anomalies */
'Q6689' = '217' /* Other  specified congenital deformities of feet - Other congenital anomalies */
'Q669' = '217' /* Congenital deformity of feet, unspecified - Other congenital anomalies */
'Q670' = '217' /* Congenital facial asymmetry - Other congenital anomalies */
'Q671' = '217' /* Congenital compression facies - Other congenital anomalies */
'Q672' = '217' /* Dolichocephaly - Other congenital anomalies */
'Q673' = '217' /* Plagiocephaly - Other congenital anomalies */
'Q674' = '217' /* Other congenital deformities of skull, face and jaw - Other congenital anomalies */
'Q675' = '217' /* Congenital deformity of spine - Other congenital anomalies */
'Q676' = '217' /* Pectus excavatum - Other congenital anomalies */
'Q677' = '217' /* Pectus carinatum - Other congenital anomalies */
'Q678' = '217' /* Other congenital deformities of chest - Other congenital anomalies */
'Q680' = '217' /* Congenital deformity of sternocleidomastoid muscle - Other congenital anomalies */
'Q681' = '217' /* Congenital deformity of finger(s) and hand - Other congenital anomalies */
'Q682' = '217' /* Congenital deformity of knee - Other congenital anomalies */
'Q683' = '217' /* Congenital bowing of femur - Other congenital anomalies */
'Q684' = '217' /* Congenital bowing of tibia and fibula - Other congenital anomalies */
'Q685' = '217' /* Congenital bowing of long bones of leg, unspecified - Other congenital anomalies */
'Q686' = '217' /* Discoid meniscus - Other congenital anomalies */
'Q688' = '217' /* Other specified congenital musculoskeletal deformities - Other congenital anomalies */
'Q690' = '217' /* Accessory finger(s) - Other congenital anomalies */
'Q691' = '217' /* Accessory thumb(s) - Other congenital anomalies */
'Q692' = '217' /* Accessory toe(s) - Other congenital anomalies */
'Q699' = '217' /* Polydactyly, unspecified - Other congenital anomalies */
'Q7000' = '217' /* Fused fingers, unspecified hand - Other congenital anomalies */
'Q7001' = '217' /* Fused fingers, right hand - Other congenital anomalies */
'Q7002' = '217' /* Fused fingers, left hand - Other congenital anomalies */
'Q7003' = '217' /* Fused fingers, bilateral - Other congenital anomalies */
'Q7010' = '217' /* Webbed fingers, unspecified hand - Other congenital anomalies */
'Q7011' = '217' /* Webbed fingers, right hand - Other congenital anomalies */
'Q7012' = '217' /* Webbed fingers, left hand - Other congenital anomalies */
'Q7013' = '217' /* Webbed fingers, bilateral - Other congenital anomalies */
'Q7020' = '217' /* Fused toes, unspecified foot - Other congenital anomalies */
'Q7021' = '217' /* Fused toes, right foot - Other congenital anomalies */
'Q7022' = '217' /* Fused toes, left foot - Other congenital anomalies */
'Q7023' = '217' /* Fused toes, bilateral - Other congenital anomalies */
'Q7030' = '217' /* Webbed toes, unspecified foot - Other congenital anomalies */
'Q7031' = '217' /* Webbed toes, right foot - Other congenital anomalies */
'Q7032' = '217' /* Webbed toes, left foot - Other congenital anomalies */
'Q7033' = '217' /* Webbed toes, bilateral - Other congenital anomalies */
'Q704' = '217' /* Polysyndactyly, unspecified - Other congenital anomalies */
'Q709' = '217' /* Syndactyly, unspecified - Other congenital anomalies */
'Q7100' = '217' /* Congenital complete absence of unspecified upper limb - Other congenital anomalies */
'Q7101' = '217' /* Congenital complete absence of right upper limb - Other congenital anomalies */
'Q7102' = '217' /* Congenital complete absence of left upper limb - Other congenital anomalies */
'Q7103' = '217' /* Congenital complete absence of upper limb, bilateral - Other congenital anomalies */
'Q7110' = '217' /* Congen absence of unsp upper arm and forearm w hand present - Other congenital anomalies */
'Q7111' = '217' /* Congenital absence of r up arm and forearm w hand present - Other congenital anomalies */
'Q7112' = '217' /* Congenital absence of l up arm and forearm w hand present - Other congenital anomalies */
'Q7113' = '217' /* Congen absence of upper arm and forearm w hand present, bi - Other congenital anomalies */
'Q7120' = '217' /* Congenital absence of both forearm and hand, unsp upper limb - Other congenital anomalies */
'Q7121' = '217' /* Congen absence of both forearm and hand, right upper limb - Other congenital anomalies */
'Q7122' = '217' /* Congenital absence of both forearm and hand, left upper limb - Other congenital anomalies */
'Q7123' = '217' /* Congenital absence of both forearm and hand, bilateral - Other congenital anomalies */
'Q7130' = '217' /* Congenital absence of unspecified hand and finger - Other congenital anomalies */
'Q7131' = '217' /* Congenital absence of right hand and finger - Other congenital anomalies */
'Q7132' = '217' /* Congenital absence of left hand and finger - Other congenital anomalies */
'Q7133' = '217' /* Congenital absence of hand and finger, bilateral - Other congenital anomalies */
'Q7140' = '217' /* Longitudinal reduction defect of unspecified radius - Other congenital anomalies */
'Q7141' = '217' /* Longitudinal reduction defect of right radius - Other congenital anomalies */
'Q7142' = '217' /* Longitudinal reduction defect of left radius - Other congenital anomalies */
'Q7143' = '217' /* Longitudinal reduction defect of radius, bilateral - Other congenital anomalies */
'Q7150' = '217' /* Longitudinal reduction defect of unspecified ulna - Other congenital anomalies */
'Q7151' = '217' /* Longitudinal reduction defect of right ulna - Other congenital anomalies */
'Q7152' = '217' /* Longitudinal reduction defect of left ulna - Other congenital anomalies */
'Q7153' = '217' /* Longitudinal reduction defect of ulna, bilateral - Other congenital anomalies */
'Q7160' = '217' /* Lobster-claw hand, unspecified hand - Other congenital anomalies */
'Q7161' = '217' /* Lobster-claw right hand - Other congenital anomalies */
'Q7162' = '217' /* Lobster-claw left hand - Other congenital anomalies */
'Q7163' = '217' /* Lobster-claw hand, bilateral - Other congenital anomalies */
'Q71811' = '217' /* Congenital shortening of right upper limb - Other congenital anomalies */
'Q71812' = '217' /* Congenital shortening of left upper limb - Other congenital anomalies */
'Q71813' = '217' /* Congenital shortening of upper limb, bilateral - Other congenital anomalies */
'Q71819' = '217' /* Congenital shortening of unspecified upper limb - Other congenital anomalies */
'Q71891' = '217' /* Other reduction defects of right upper limb - Other congenital anomalies */
'Q71892' = '217' /* Other reduction defects of left upper limb - Other congenital anomalies */
'Q71893' = '217' /* Other reduction defects of upper limb, bilateral - Other congenital anomalies */
'Q71899' = '217' /* Other reduction defects of unspecified upper limb - Other congenital anomalies */
'Q7190' = '217' /* Unspecified reduction defect of unspecified upper limb - Other congenital anomalies */
'Q7191' = '217' /* Unspecified reduction defect of right upper limb - Other congenital anomalies */
'Q7192' = '217' /* Unspecified reduction defect of left upper limb - Other congenital anomalies */
'Q7193' = '217' /* Unspecified reduction defect of upper limb, bilateral - Other congenital anomalies */
'Q7200' = '217' /* Congenital complete absence of unspecified lower limb - Other congenital anomalies */
'Q7201' = '217' /* Congenital complete absence of right lower limb - Other congenital anomalies */
'Q7202' = '217' /* Congenital complete absence of left lower limb - Other congenital anomalies */
'Q7203' = '217' /* Congenital complete absence of lower limb, bilateral - Other congenital anomalies */
'Q7210' = '217' /* Congen absence of unsp thigh and lower leg w foot present - Other congenital anomalies */
'Q7211' = '217' /* Congen absence of right thigh and lower leg w foot present - Other congenital anomalies */
'Q7212' = '217' /* Congen absence of left thigh and lower leg w foot present - Other congenital anomalies */
'Q7213' = '217' /* Congen absence of thigh and lower leg w foot present, bi - Other congenital anomalies */
'Q7220' = '217' /* Congen absence of both lower leg and foot, unsp lower limb - Other congenital anomalies */
'Q7221' = '217' /* Congen absence of both lower leg and foot, right lower limb - Other congenital anomalies */
'Q7222' = '217' /* Congen absence of both lower leg and foot, left lower limb - Other congenital anomalies */
'Q7223' = '217' /* Congenital absence of both lower leg and foot, bilateral - Other congenital anomalies */
'Q7230' = '217' /* Congenital absence of unspecified foot and toe(s) - Other congenital anomalies */
'Q7231' = '217' /* Congenital absence of right foot and toe(s) - Other congenital anomalies */
'Q7232' = '217' /* Congenital absence of left foot and toe(s) - Other congenital anomalies */
'Q7233' = '217' /* Congenital absence of foot and toe(s), bilateral - Other congenital anomalies */
'Q7240' = '217' /* Longitudinal reduction defect of unspecified femur - Other congenital anomalies */
'Q7241' = '217' /* Longitudinal reduction defect of right femur - Other congenital anomalies */
'Q7242' = '217' /* Longitudinal reduction defect of left femur - Other congenital anomalies */
'Q7243' = '217' /* Longitudinal reduction defect of femur, bilateral - Other congenital anomalies */
'Q7250' = '217' /* Longitudinal reduction defect of unspecified tibia - Other congenital anomalies */
'Q7251' = '217' /* Longitudinal reduction defect of right tibia - Other congenital anomalies */
'Q7252' = '217' /* Longitudinal reduction defect of left tibia - Other congenital anomalies */
'Q7253' = '217' /* Longitudinal reduction defect of tibia, bilateral - Other congenital anomalies */
'Q7260' = '217' /* Longitudinal reduction defect of unspecified fibula - Other congenital anomalies */
'Q7261' = '217' /* Longitudinal reduction defect of right fibula - Other congenital anomalies */
'Q7262' = '217' /* Longitudinal reduction defect of left fibula - Other congenital anomalies */
'Q7263' = '217' /* Longitudinal reduction defect of fibula, bilateral - Other congenital anomalies */
'Q7270' = '217' /* Split foot, unspecified lower limb - Other congenital anomalies */
'Q7271' = '217' /* Split foot, right lower limb - Other congenital anomalies */
'Q7272' = '217' /* Split foot, left lower limb - Other congenital anomalies */
'Q7273' = '217' /* Split foot, bilateral - Other congenital anomalies */
'Q72811' = '217' /* Congenital shortening of right lower limb - Other congenital anomalies */
'Q72812' = '217' /* Congenital shortening of left lower limb - Other congenital anomalies */
'Q72813' = '217' /* Congenital shortening of lower limb, bilateral - Other congenital anomalies */
'Q72819' = '217' /* Congenital shortening of unspecified lower limb - Other congenital anomalies */
'Q72891' = '217' /* Other reduction defects of right lower limb - Other congenital anomalies */
'Q72892' = '217' /* Other reduction defects of left lower limb - Other congenital anomalies */
'Q72893' = '217' /* Other reduction defects of lower limb, bilateral - Other congenital anomalies */
'Q72899' = '217' /* Other reduction defects of unspecified lower limb - Other congenital anomalies */
'Q7290' = '217' /* Unspecified reduction defect of unspecified lower limb - Other congenital anomalies */
'Q7291' = '217' /* Unspecified reduction defect of right lower limb - Other congenital anomalies */
'Q7292' = '217' /* Unspecified reduction defect of left lower limb - Other congenital anomalies */
'Q7293' = '217' /* Unspecified reduction defect of lower limb, bilateral - Other congenital anomalies */
'Q730' = '217' /* Congenital absence of unspecified limb(s) - Other congenital anomalies */
'Q731' = '217' /* Phocomelia, unspecified limb(s) - Other congenital anomalies */
'Q738' = '217' /* Other reduction defects of unspecified limb(s) - Other congenital anomalies */
'Q740' = '217' /* Oth congen malform of upper limb(s), inc shoulder girdle - Other congenital anomalies */
'Q741' = '217' /* Congenital malformation of knee - Other congenital anomalies */
'Q742' = '217' /* Oth congen malform of lower limb(s), including pelvic girdle - Other congenital anomalies */
'Q743' = '217' /* Arthrogryposis multiplex congenita - Other congenital anomalies */
'Q748' = '217' /* Other specified congenital malformations of limb(s) - Other congenital anomalies */
'Q749' = '217' /* Unspecified congenital malformation of limb(s) - Other congenital anomalies */
'Q750' = '217' /* Craniosynostosis - Other congenital anomalies */
'Q751' = '217' /* Craniofacial dysostosis - Other congenital anomalies */
'Q752' = '217' /* Hypertelorism - Other congenital anomalies */
'Q753' = '217' /* Macrocephaly - Other congenital anomalies */
'Q754' = '217' /* Mandibulofacial dysostosis - Other congenital anomalies */
'Q755' = '217' /* Oculomandibular dysostosis - Other congenital anomalies */
'Q758' = '217' /* Oth congenital malformations of skull and face bones - Other congenital anomalies */
'Q759' = '217' /* Congenital malformation of skull and face bones, unspecified - Other congenital anomalies */
'Q760' = '217' /* Spina bifida occulta - Other congenital anomalies */
'Q761' = '217' /* Klippel-Feil syndrome - Other congenital anomalies */
'Q762' = '217' /* Congenital spondylolisthesis - Other congenital anomalies */
'Q763' = '217' /* Congenital scoliosis due to congenital bony malformation - Other congenital anomalies */
'Q76411' = '217' /* Congenital kyphosis, occipito-atlanto-axial region - Other congenital anomalies */
'Q76412' = '217' /* Congenital kyphosis, cervical region - Other congenital anomalies */
'Q76413' = '217' /* Congenital kyphosis, cervicothoracic region - Other congenital anomalies */
'Q76414' = '217' /* Congenital kyphosis, thoracic region - Other congenital anomalies */
'Q76415' = '217' /* Congenital kyphosis, thoracolumbar region - Other congenital anomalies */
'Q76419' = '217' /* Congenital kyphosis, unspecified region - Other congenital anomalies */
'Q76425' = '217' /* Congenital lordosis, thoracolumbar region - Other congenital anomalies */
'Q76426' = '217' /* Congenital lordosis, lumbar region - Other congenital anomalies */
'Q76427' = '217' /* Congenital lordosis, lumbosacral region - Other congenital anomalies */
'Q76428' = '217' /* Congenital lordosis, sacral and sacrococcygeal region - Other congenital anomalies */
'Q76429' = '217' /* Congenital lordosis, unspecified region - Other congenital anomalies */
'Q7649' = '217' /* Oth congenital malform of spine, not associated w scoliosis - Other congenital anomalies */
'Q765' = '217' /* Cervical rib - Other congenital anomalies */
'Q766' = '217' /* Other congenital malformations of ribs - Other congenital anomalies */
'Q767' = '217' /* Congenital malformation of sternum - Other congenital anomalies */
'Q768' = '217' /* Other congenital malformations of bony thorax - Other congenital anomalies */
'Q769' = '217' /* Congenital malformation of bony thorax, unspecified - Other congenital anomalies */
'Q770' = '217' /* Achondrogenesis - Other congenital anomalies */
'Q771' = '217' /* Thanatophoric short stature - Other congenital anomalies */
'Q772' = '217' /* Short rib syndrome - Other congenital anomalies */
'Q773' = '217' /* Chondrodysplasia punctata - Other congenital anomalies */
'Q774' = '217' /* Achondroplasia - Other congenital anomalies */
'Q775' = '217' /* Diastrophic dysplasia - Other congenital anomalies */
'Q776' = '217' /* Chondroectodermal dysplasia - Other congenital anomalies */
'Q777' = '217' /* Spondyloepiphyseal dysplasia - Other congenital anomalies */
'Q778' = '217' /* Oth osteochndrdys w defct of growth of tublr bones and spine - Other congenital anomalies */
'Q779' = '217' /* Osteochndrdys w defct of grth of tublr bones and spine, unsp - Other congenital anomalies */
'Q780' = '217' /* Osteogenesis imperfecta - Other congenital anomalies */
'Q781' = '217' /* Polyostotic fibrous dysplasia - Other congenital anomalies */
'Q782' = '217' /* Osteopetrosis - Other congenital anomalies */
'Q783' = '217' /* Progressive diaphyseal dysplasia - Other congenital anomalies */
'Q784' = '217' /* Enchondromatosis - Other congenital anomalies */
'Q785' = '217' /* Metaphyseal dysplasia - Other congenital anomalies */
'Q786' = '217' /* Multiple congenital exostoses - Other congenital anomalies */
'Q788' = '217' /* Other specified osteochondrodysplasias - Other congenital anomalies */
'Q789' = '217' /* Osteochondrodysplasia, unspecified - Other congenital anomalies */
'Q790' = '217' /* Congenital diaphragmatic hernia - Other congenital anomalies */
'Q791' = '217' /* Other congenital malformations of diaphragm - Other congenital anomalies */
'Q792' = '217' /* Exomphalos - Other congenital anomalies */
'Q793' = '217' /* Gastroschisis - Other congenital anomalies */
'Q794' = '217' /* Prune belly syndrome - Other congenital anomalies */
'Q7951' = '217' /* Congenital hernia of bladder - Other congenital anomalies */
'Q7959' = '217' /* Other congenital malformations of abdominal wall - Other congenital anomalies */
'Q796' = '217' /* Ehlers-Danlos syndrome - Other congenital anomalies */
'Q798' = '217' /* Other congenital malformations of musculoskeletal system - Other congenital anomalies */
'Q799' = '217' /* Congenital malformation of musculoskeletal system, unsp - Other congenital anomalies */
'Q800' = '217' /* Ichthyosis vulgaris - Other congenital anomalies */
'Q801' = '217' /* X-linked ichthyosis - Other congenital anomalies */
'Q802' = '217' /* Lamellar ichthyosis - Other congenital anomalies */
'Q803' = '217' /* Congenital bullous ichthyosiform erythroderma - Other congenital anomalies */
'Q804' = '217' /* Harlequin fetus - Other congenital anomalies */
'Q808' = '217' /* Other congenital ichthyosis - Other congenital anomalies */
'Q809' = '217' /* Congenital ichthyosis, unspecified - Other congenital anomalies */
'Q810' = '217' /* Epidermolysis bullosa simplex - Other congenital anomalies */
'Q811' = '217' /* Epidermolysis bullosa letalis - Other congenital anomalies */
'Q812' = '217' /* Epidermolysis bullosa dystrophica - Other congenital anomalies */
'Q818' = '217' /* Other epidermolysis bullosa - Other congenital anomalies */
'Q819' = '217' /* Epidermolysis bullosa, unspecified - Other congenital anomalies */
'Q820' = '217' /* Hereditary lymphedema - Other congenital anomalies */
'Q821' = '217' /* Xeroderma pigmentosum - Other congenital anomalies */
'Q822' = '217' /* Mastocytosis - Other congenital anomalies */
'Q823' = '217' /* Incontinentia pigmenti - Other congenital anomalies */
'Q824' = '217' /* Ectodermal dysplasia (anhidrotic) - Other congenital anomalies */
'Q825' = '217' /* Congenital non-neoplastic nevus - Other congenital anomalies */
'Q828' = '217' /* Other specified congenital malformations of skin - Other congenital anomalies */
'Q829' = '217' /* Congenital malformation of skin, unspecified - Other congenital anomalies */
'Q830' = '217' /* Congenital absence of breast with absent nipple - Other congenital anomalies */
'Q831' = '217' /* Accessory breast - Other congenital anomalies */
'Q832' = '217' /* Absent nipple - Other congenital anomalies */
'Q833' = '217' /* Accessory nipple - Other congenital anomalies */
'Q838' = '217' /* Other congenital malformations of breast - Other congenital anomalies */
'Q839' = '217' /* Congenital malformation of breast, unspecified - Other congenital anomalies */
'Q840' = '217' /* Congenital alopecia - Other congenital anomalies */
'Q841' = '217' /* Congenital morphological disturbances of hair, NEC - Other congenital anomalies */
'Q842' = '217' /* Other congenital malformations of hair - Other congenital anomalies */
'Q843' = '217' /* Anonychia - Other congenital anomalies */
'Q844' = '217' /* Congenital leukonychia - Other congenital anomalies */
'Q845' = '217' /* Enlarged and hypertrophic nails - Other congenital anomalies */
'Q846' = '217' /* Other congenital malformations of nails - Other congenital anomalies */
'Q848' = '217' /* Other specified congenital malformations of integument - Other congenital anomalies */
'Q849' = '217' /* Congenital malformation of integument, unspecified - Other congenital anomalies */
'Q851' = '217' /* Tuberous sclerosis - Other congenital anomalies */
'Q858' = '217' /* Other phakomatoses, not elsewhere classified - Other congenital anomalies */
'Q859' = '217' /* Phakomatosis, unspecified - Other congenital anomalies */
'Q870' = '217' /* Congen malform syndromes predom affecting facial appearance - Other congenital anomalies */
'Q871' = '217' /* Congenital malform syndromes predom assoc w short stature - Other congenital anomalies */
'Q872' = '217' /* Congenital malformation syndromes predom involving limbs - Other congenital anomalies */
'Q873' = '217' /* Congenital malformation syndromes involving early overgrowth - Other congenital anomalies */
'Q8740' = '217' /* Marfan's syndrome, unspecified - Other congenital anomalies */
'Q87410' = '217' /* Marfan's syndrome with aortic dilation - Other congenital anomalies */
'Q87418' = '217' /* Marfan's syndrome with other cardiovascular manifestations - Other congenital anomalies */
'Q8742' = '217' /* Marfan's syndrome with ocular manifestations - Other congenital anomalies */
'Q8743' = '217' /* Marfan's syndrome with skeletal manifestation - Other congenital anomalies */
'Q875' = '217' /* Oth congenital malformation syndromes w oth skeletal changes - Other congenital anomalies */
'Q8781' = '217' /* Alport syndrome - Other congenital anomalies */
'Q8789' = '217' /* Oth congenital malformation syndromes, NEC - Other congenital anomalies */
'Q8901' = '217' /* Asplenia (congenital) - Other congenital anomalies */
'Q8909' = '217' /* Congenital malformations of spleen - Other congenital anomalies */
'Q891' = '217' /* Congenital malformations of adrenal gland - Other congenital anomalies */
'Q892' = '217' /* Congenital malformations of other endocrine glands - Other congenital anomalies */
'Q893' = '217' /* Situs inversus - Other congenital anomalies */
'Q894' = '217' /* Conjoined twins - Other congenital anomalies */
'Q897' = '217' /* Multiple congenital malformations, not elsewhere classified - Other congenital anomalies */
'Q898' = '217' /* Other specified congenital malformations - Other congenital anomalies */
'Q899' = '217' /* Congenital malformation, unspecified - Other congenital anomalies */
'Q900' = '217' /* Trisomy 21, nonmosaicism (meiotic nondisjunction) - Other congenital anomalies */
'Q901' = '217' /* Trisomy 21, mosaicism (mitotic nondisjunction) - Other congenital anomalies */
'Q902' = '217' /* Trisomy 21, translocation - Other congenital anomalies */
'Q909' = '217' /* Down syndrome, unspecified - Other congenital anomalies */
'Q910' = '217' /* Trisomy 18, nonmosaicism (meiotic nondisjunction) - Other congenital anomalies */
'Q911' = '217' /* Trisomy 18, mosaicism (mitotic nondisjunction) - Other congenital anomalies */
'Q912' = '217' /* Trisomy 18, translocation - Other congenital anomalies */
'Q913' = '217' /* Trisomy 18, unspecified - Other congenital anomalies */
'Q914' = '217' /* Trisomy 13, nonmosaicism (meiotic nondisjunction) - Other congenital anomalies */
'Q915' = '217' /* Trisomy 13, mosaicism (mitotic nondisjunction) - Other congenital anomalies */
'Q916' = '217' /* Trisomy 13, translocation - Other congenital anomalies */
'Q917' = '217' /* Trisomy 13, unspecified - Other congenital anomalies */
'Q920' = '217' /* Whole chromosome trisomy, nonmosaic (meiotic nondisjunction) - Other congenital anomalies */
'Q921' = '217' /* Whole chromosome trisomy, mosaicism (mitotic nondisjunction) - Other congenital anomalies */
'Q922' = '217' /* Partial trisomy - Other congenital anomalies */
'Q925' = '217' /* Duplications with other complex rearrangements - Other congenital anomalies */
'Q9261' = '217' /* Marker chromosomes in normal individual - Other congenital anomalies */
'Q9262' = '217' /* Marker chromosomes in abnormal individual - Other congenital anomalies */
'Q927' = '217' /* Triploidy and polyploidy - Other congenital anomalies */
'Q928' = '217' /* Other specified trisomies and partial trisomies of autosomes - Other congenital anomalies */
'Q929' = '217' /* Trisomy and partial trisomy of autosomes, unspecified - Other congenital anomalies */
'Q930' = '217' /* Whole chromosome monosomy,nonmosaic (meiotic nondisjunction) - Other congenital anomalies */
'Q931' = '217' /* Whole chromosome monosomy, mosaic (mitotic nondisjunction) - Other congenital anomalies */
'Q932' = '217' /* Chromosome replaced with ring, dicentric or isochromosome - Other congenital anomalies */
'Q933' = '217' /* Deletion of short arm of chromosome 4 - Other congenital anomalies */
'Q934' = '217' /* Deletion of short arm of chromosome 5 - Other congenital anomalies */
'Q935' = '217' /* Other deletions of part of a chromosome - Other congenital anomalies */
'Q937' = '217' /* Deletions with other complex rearrangements - Other congenital anomalies */
'Q9381' = '217' /* Velo-cardio-facial syndrome - Other congenital anomalies */
'Q9388' = '217' /* Other microdeletions - Other congenital anomalies */
'Q9389' = '217' /* Other deletions from the autosomes - Other congenital anomalies */
'Q939' = '217' /* Deletion from autosomes, unspecified - Other congenital anomalies */
'Q950' = '217' /* Balanced translocation and insertion in normal individual - Other congenital anomalies */
'Q951' = '217' /* Chromosome inversion in normal individual - Other congenital anomalies */
'Q952' = '217' /* Balanced autosomal rearrangement in abnormal individual - Other congenital anomalies */
'Q953' = '217' /* Balanced sex/autosomal rearrangement in abnormal individual - Other congenital anomalies */
'Q955' = '217' /* Individual with autosomal fragile site - Other congenital anomalies */
'Q958' = '217' /* Other balanced rearrangements and structural markers - Other congenital anomalies */
'Q959' = '217' /* Balanced rearrangement and structural marker, unspecified - Other congenital anomalies */
'Q960' = '217' /* Karyotype 45, X - Other congenital anomalies */
'Q961' = '217' /* Karyotype 46, X iso (Xq) - Other congenital anomalies */
'Q962' = '217' /* Karyotype 46, X w abnormal sex chromosome, except iso (Xq) - Other congenital anomalies */
'Q963' = '217' /* Mosaicism, 45, X/46, XX or XY - Other congenital anomalies */
'Q964' = '217' /* Mosaic, 45, X/other cell line(s) w abnormal sex chromosome - Other congenital anomalies */
'Q968' = '217' /* Other variants of Turner's syndrome - Other congenital anomalies */
'Q969' = '217' /* Turner's syndrome, unspecified - Other congenital anomalies */
'Q970' = '217' /* Karyotype 47, XXX - Other congenital anomalies */
'Q971' = '217' /* Female with more than three X chromosomes - Other congenital anomalies */
'Q972' = '217' /* Mosaicism, lines with various numbers of X chromosomes - Other congenital anomalies */
'Q973' = '217' /* Female with 46, XY karyotype - Other congenital anomalies */
'Q978' = '217' /* Oth sex chromosome abnormalities, female phenotype - Other congenital anomalies */
'Q979' = '217' /* Sex chromosome abnormality, female phenotype, unspecified - Other congenital anomalies */
'Q980' = '217' /* Klinefelter syndrome karyotype 47, XXY - Other congenital anomalies */
'Q981' = '217' /* Klinefelter syndrome, male with more than two X chromosomes - Other congenital anomalies */
'Q983' = '217' /* Other male with 46, XX karyotype - Other congenital anomalies */
'Q984' = '217' /* Klinefelter syndrome, unspecified - Other congenital anomalies */
'Q985' = '217' /* Karyotype 47, XYY - Other congenital anomalies */
'Q986' = '217' /* Male with structurally abnormal sex chromosome - Other congenital anomalies */
'Q987' = '217' /* Male with sex chromosome mosaicism - Other congenital anomalies */
'Q988' = '217' /* Other specified sex chromosome abnormalities, male phenotype - Other congenital anomalies */
'Q989' = '217' /* Sex chromosome abnormality, male phenotype, unspecified - Other congenital anomalies */
'Q990' = '217' /* Chimera 46, XX/46, XY - Other congenital anomalies */
'Q991' = '217' /* 46, XX true hermaphrodite - Other congenital anomalies */
'Q992' = '217' /* Fragile X chromosome - Other congenital anomalies */
'Q998' = '217' /* Other specified chromosome abnormalities - Other congenital anomalies */
'Q999' = '217' /* Chromosomal abnormality, unspecified - Other congenital anomalies */
'Z87720' = '217' /* Personal history of (corrected) congenital malform of eye - Other congenital anomalies */
'Z87721' = '217' /* Personal history of (corrected) congenital malform of ear - Other congenital anomalies */
'Z87730' = '217' /* Personal history of (corrected) cleft lip and palate - Other congenital anomalies */
'Z8775' = '217' /* Personal history of congenital malform of resp sys - Other congenital anomalies */
'Z8776' = '217' /* Prsnl hx of congen malform of integument, limbs and ms sys - Other congenital anomalies */
'Z87790' = '217' /* Personal history of congenital malform of face and neck - Other congenital anomalies */
'Z87798' = '217' /* Personal history of oth (corrected) congenital malformations - Other congenital anomalies */
'P0500' = '219' /* Newborn light for gestational age, unspecified weight - Short gestation; low birth weight; and fetal growth retardation */
'P0501' = '219' /* Newborn light for gestational age, less than 500 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0502' = '219' /* Newborn light for gestational age, 500-749 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0503' = '219' /* Newborn light for gestational age, 750-999 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0504' = '219' /* Newborn light for gestational age, 1000-1249 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0505' = '219' /* Newborn light for gestational age, 1250-1499 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0506' = '219' /* Newborn light for gestational age, 1500-1749 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0507' = '219' /* Newborn light for gestational age, 1750-1999 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0508' = '219' /* Newborn light for gestational age, 2000-2499 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0510' = '219' /* Newborn small for gestational age, unspecified weight - Short gestation; low birth weight; and fetal growth retardation */
'P0511' = '219' /* Newborn small for gestational age, less than 500 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0512' = '219' /* Newborn small for gestational age, 500-749 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0513' = '219' /* Newborn small for gestational age, 750-999 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0514' = '219' /* Newborn small for gestational age, 1000-1249 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0515' = '219' /* Newborn small for gestational age, 1250-1499 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0516' = '219' /* Newborn small for gestational age, 1500-1749 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0517' = '219' /* Newborn small for gestational age, 1750-1999 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0518' = '219' /* Newborn small for gestational age, 2000-2499 grams - Short gestation; low birth weight; and fetal growth retardation */
'P052' = '219' /* NB aff by fetal malnut not light or small for gestatnl age - Short gestation; low birth weight; and fetal growth retardation */
'P059' = '219' /* Newborn affected by slow intrauterine growth, unspecified - Short gestation; low birth weight; and fetal growth retardation */
'P0700' = '219' /* Extremely low birth weight newborn, unspecified weight - Short gestation; low birth weight; and fetal growth retardation */
'P0701' = '219' /* Extremely low birth weight newborn, less than 500 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0702' = '219' /* Extremely low birth weight newborn, 500-749 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0703' = '219' /* Extremely low birth weight newborn, 750-999 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0710' = '219' /* Other low birth weight newborn, unspecified weight - Short gestation; low birth weight; and fetal growth retardation */
'P0714' = '219' /* Other low birth weight newborn, 1000-1249 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0715' = '219' /* Other low birth weight newborn, 1250-1499 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0716' = '219' /* Other low birth weight newborn, 1500-1749 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0717' = '219' /* Other low birth weight newborn, 1750-1999 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0718' = '219' /* Other low birth weight newborn, 2000-2499 grams - Short gestation; low birth weight; and fetal growth retardation */
'P0720' = '219' /* Extreme immaturity of newborn, unsp weeks of gestation - Short gestation; low birth weight; and fetal growth retardation */
'P0721' = '219' /* Extreme immaturity of NB, gestatnl age < 23 completed weeks - Short gestation; low birth weight; and fetal growth retardation */
'P0722' = '219' /* Extreme immaturity of NB, gestatnl age 23 completed weeks - Short gestation; low birth weight; and fetal growth retardation */
'P0723' = '219' /* Extreme immaturity of NB, gestatnl age 24 completed weeks - Short gestation; low birth weight; and fetal growth retardation */
'P0724' = '219' /* Extreme immaturity of NB, gestatnl age 25 completed weeks - Short gestation; low birth weight; and fetal growth retardation */
'P0725' = '219' /* Extreme immaturity of NB, gestatnl age 26 completed weeks - Short gestation; low birth weight; and fetal growth retardation */
'P0726' = '219' /* Extreme immaturity of NB, gestatnl age 27 completed weeks - Short gestation; low birth weight; and fetal growth retardation */
'P0730' = '219' /* Preterm newborn, unspecified weeks of gestation - Short gestation; low birth weight; and fetal growth retardation */
'P0731' = '219' /* Preterm newborn, gestational age 28 completed weeks - Short gestation; low birth weight; and fetal growth retardation */
'P0732' = '219' /* Preterm newborn, gestational age 29 completed weeks - Short gestation; low birth weight; and fetal growth retardation */
'P0733' = '219' /* Preterm newborn, gestational age 30 completed weeks - Short gestation; low birth weight; and fetal growth retardation */
'P0734' = '219' /* Preterm newborn, gestational age 31 completed weeks - Short gestation; low birth weight; and fetal growth retardation */
'P0735' = '219' /* Preterm newborn, gestational age 32 completed weeks - Short gestation; low birth weight; and fetal growth retardation */
'P0736' = '219' /* Preterm newborn, gestational age 33 completed weeks - Short gestation; low birth weight; and fetal growth retardation */
'P0737' = '219' /* Preterm newborn, gestational age 34 completed weeks - Short gestation; low birth weight; and fetal growth retardation */
'P0738' = '219' /* Preterm newborn, gestational age 35 completed weeks - Short gestation; low birth weight; and fetal growth retardation */
'P0739' = '219' /* Preterm newborn, gestational age 36 completed weeks - Short gestation; low birth weight; and fetal growth retardation */
'P220' = '221' /* Respiratory distress syndrome of newborn - Respiratory distress syndrome */
'A33' = '224' /* Tetanus neonatorum - Other perinatal conditions */
'A4851' = '224' /* Infant botulism - Other perinatal conditions */
'P000' = '224' /* Newborn affected by maternal hypertensive disorders - Other perinatal conditions */
'P001' = '224' /* Newborn aff by maternal renal and urinary tract diseases - Other perinatal conditions */
'P002' = '224' /* Newborn affected by maternal infec/parastc diseases - Other perinatal conditions */
'P003' = '224' /* Newborn affected by oth maternal circ and resp diseases - Other perinatal conditions */
'P004' = '224' /* Newborn affected by maternal nutritional disorders - Other perinatal conditions */
'P005' = '224' /* Newborn (suspected to be) affected by maternal injury - Other perinatal conditions */
'P006' = '224' /* Newborn affected by surgical procedure on mother - Other perinatal conditions */
'P007' = '224' /* Newborn affected by oth medical procedures on mother, NEC - Other perinatal conditions */
'P0081' = '224' /* Newborn affected by periodontal disease in mother - Other perinatal conditions */
'P0089' = '224' /* Newborn affected by oth maternal conditions - Other perinatal conditions */
'P009' = '224' /* Newborn affected by unsp maternal condition - Other perinatal conditions */
'P010' = '224' /* Newborn (suspected to be) affected by incompetent cervix - Other perinatal conditions */
'P011' = '224' /* Newborn (suspected to be) affected by premature ROM - Other perinatal conditions */
'P012' = '224' /* Newborn (suspected to be) affected by oligohydramnios - Other perinatal conditions */
'P013' = '224' /* Newborn (suspected to be) affected by polyhydramnios - Other perinatal conditions */
'P014' = '224' /* Newborn (suspected to be) affected by ectopic pregnancy - Other perinatal conditions */
'P015' = '224' /* Newborn (suspected to be) affected by multiple pregnancy - Other perinatal conditions */
'P016' = '224' /* Newborn (suspected to be) affected by maternal death - Other perinatal conditions */
'P017' = '224' /* Newborn affected by malpresentation before labor - Other perinatal conditions */
'P018' = '224' /* Newborn affected by oth maternal complications of pregnancy - Other perinatal conditions */
'P019' = '224' /* Newborn affected by maternal complication of pregnancy, unsp - Other perinatal conditions */
'P020' = '224' /* Newborn (suspected to be) affected by placenta previa - Other perinatal conditions */
'P021' = '224' /* Newborn affected by oth placental separation and hemorrhage - Other perinatal conditions */
'P0220' = '224' /* Newborn aff by unsp morpholog and functn abnlt of placenta - Other perinatal conditions */
'P0229' = '224' /* Newborn aff by oth morpholog and functn abnlt of placenta - Other perinatal conditions */
'P023' = '224' /* Newborn affected by placental transfusion syndromes - Other perinatal conditions */
'P024' = '224' /* Newborn (suspected to be) affected by prolapsed cord - Other perinatal conditions */
'P025' = '224' /* Newborn affected by oth compression of umbilical cord - Other perinatal conditions */
'P0260' = '224' /* Newborn affected by unsp conditions of umbilical cord - Other perinatal conditions */
'P0269' = '224' /* Newborn affected by oth conditions of umbilical cord - Other perinatal conditions */
'P027' = '224' /* Newborn (suspected to be) affected by chorioamnionitis - Other perinatal conditions */
'P028' = '224' /* Newborn affected by oth abnormalities of membranes - Other perinatal conditions */
'P029' = '224' /* Newborn affected by abnormality of membranes, unsp - Other perinatal conditions */
'P030' = '224' /* Newborn affected by breech delivery and extraction - Other perinatal conditions */
'P031' = '224' /* NB aff by oth malpresent, malpos & disproprtn dur labr & del - Other perinatal conditions */
'P032' = '224' /* Newborn (suspected to be) affected by forceps delivery - Other perinatal conditions */
'P033' = '224' /* Newborn affected by delivery by vacuum extractor - Other perinatal conditions */
'P034' = '224' /* Newborn (suspected to be) affected by Cesarean delivery - Other perinatal conditions */
'P035' = '224' /* Newborn (suspected to be) affected by precipitate delivery - Other perinatal conditions */
'P036' = '224' /* Newborn affected by abnormal uterine contractions - Other perinatal conditions */
'P03810' = '224' /* NB aff by abnlt in fetl heart rate or rhythm bef onset labor - Other perinatal conditions */
'P03811' = '224' /* NB aff by abnlt in fetal heart rate or rhythm during labor - Other perinatal conditions */
'P03819' = '224' /* NB aff by abnlt in fetl hrt rate or rhythm, unsp time onset - Other perinatal conditions */
'P0382' = '224' /* Meconium passage during delivery - Other perinatal conditions */
'P0389' = '224' /* Newborn affected by oth complications of labor and delivery - Other perinatal conditions */
'P039' = '224' /* Newborn affected by complication of labor and delivery, unsp - Other perinatal conditions */
'P040' = '224' /* NB aff by matern anesth and analgesia in preg, labor and del - Other perinatal conditions */
'P041' = '224' /* Newborn affected by oth maternal medication - Other perinatal conditions */
'P042' = '224' /* Newborn affected by maternal use of tobacco - Other perinatal conditions */
'P045' = '224' /* Newborn aff by maternal use of nutritional chemical substnc - Other perinatal conditions */
'P046' = '224' /* Newborn aff by maternal exposure to environ chemical substnc - Other perinatal conditions */
'P048' = '224' /* Newborn affected by oth maternal noxious substances - Other perinatal conditions */
'P049' = '224' /* Newborn affected by maternal noxious substance, unsp - Other perinatal conditions */
'P080' = '224' /* Exceptionally large newborn baby - Other perinatal conditions */
'P081' = '224' /* Other heavy for gestational age newborn - Other perinatal conditions */
'P0821' = '224' /* Post-term newborn - Other perinatal conditions */
'P0822' = '224' /* Prolonged gestation of newborn - Other perinatal conditions */
'P09' = '224' /* Abnormal findings on neonatal screening - Other perinatal conditions */
'P221' = '224' /* Transient tachypnea of newborn - Other perinatal conditions */
'P228' = '224' /* Other respiratory distress of newborn - Other perinatal conditions */
'P229' = '224' /* Respiratory distress of newborn, unspecified - Other perinatal conditions */
'P230' = '224' /* Congenital pneumonia due to viral agent - Other perinatal conditions */
'P231' = '224' /* Congenital pneumonia due to Chlamydia - Other perinatal conditions */
'P232' = '224' /* Congenital pneumonia due to staphylococcus - Other perinatal conditions */
'P233' = '224' /* Congenital pneumonia due to streptococcus, group B - Other perinatal conditions */
'P234' = '224' /* Congenital pneumonia due to Escherichia coli - Other perinatal conditions */
'P235' = '224' /* Congenital pneumonia due to Pseudomonas - Other perinatal conditions */
'P236' = '224' /* Congenital pneumonia due to other bacterial agents - Other perinatal conditions */
'P238' = '224' /* Congenital pneumonia due to other organisms - Other perinatal conditions */
'P239' = '224' /* Congenital pneumonia, unspecified - Other perinatal conditions */
'P2400' = '224' /* Meconium aspiration without respiratory symptoms - Other perinatal conditions */
'P2401' = '224' /* Meconium aspiration with respiratory symptoms - Other perinatal conditions */
'P2410' = '224' /* Neonatal aspirat of amnio fluid and mucus w/o resp symp - Other perinatal conditions */
'P2411' = '224' /* Neonatal aspirat of amnio fluid and mucus w resp symp - Other perinatal conditions */
'P2420' = '224' /* Neonatal aspiration of blood without respiratory symptoms - Other perinatal conditions */
'P2421' = '224' /* Neonatal aspiration of blood with respiratory symptoms - Other perinatal conditions */
'P2430' = '224' /* Neonatal aspirat of milk and regurgitated food w/o resp symp - Other perinatal conditions */
'P2431' = '224' /* Neonatal aspirat of milk and regurgitated food w resp symp - Other perinatal conditions */
'P2480' = '224' /* Other neonatal aspiration without respiratory symptoms - Other perinatal conditions */
'P2481' = '224' /* Other neonatal aspiration with respiratory symptoms - Other perinatal conditions */
'P249' = '224' /* Neonatal aspiration, unspecified - Other perinatal conditions */
'P250' = '224' /* Interstitial emphysema originating in the perinatal period - Other perinatal conditions */
'P251' = '224' /* Pneumothorax originating in the perinatal period - Other perinatal conditions */
'P252' = '224' /* Pneumomediastinum originating in the perinatal period - Other perinatal conditions */
'P253' = '224' /* Pneumopericardium originating in the perinatal period - Other perinatal conditions */
'P258' = '224' /* Oth cond rel to interstit emphysema origin in perinat period - Other perinatal conditions */
'P260' = '224' /* Tracheobronchial hemorrhage origin in the perinatal period - Other perinatal conditions */
'P261' = '224' /* Massive pulmonary hemorrhage origin in the perinatal period - Other perinatal conditions */
'P268' = '224' /* Oth pulmonary hemorrhages origin in the perinatal period - Other perinatal conditions */
'P269' = '224' /* Unsp pulmonary hemorrhage origin in the perinatal period - Other perinatal conditions */
'P270' = '224' /* Wilson-Mikity syndrome - Other perinatal conditions */
'P271' = '224' /* Bronchopulmonary dysplasia origin in the perinatal period - Other perinatal conditions */
'P278' = '224' /* Oth chronic resp diseases origin in the perinatal period - Other perinatal conditions */
'P279' = '224' /* Unsp chronic resp disease origin in the perinatal period - Other perinatal conditions */
'P280' = '224' /* Primary atelectasis of newborn - Other perinatal conditions */
'P2810' = '224' /* Unspecified atelectasis of newborn - Other perinatal conditions */
'P2811' = '224' /* Resorption atelectasis without respiratory distress syndrome - Other perinatal conditions */
'P2819' = '224' /* Other atelectasis of newborn - Other perinatal conditions */
'P282' = '224' /* Cyanotic attacks of newborn - Other perinatal conditions */
'P283' = '224' /* Primary sleep apnea of newborn - Other perinatal conditions */
'P284' = '224' /* Other apnea of newborn - Other perinatal conditions */
'P285' = '224' /* Respiratory failure of newborn - Other perinatal conditions */
'P2881' = '224' /* Respiratory arrest of newborn - Other perinatal conditions */
'P2889' = '224' /* Other specified respiratory conditions of newborn - Other perinatal conditions */
'P289' = '224' /* Respiratory condition of newborn, unspecified - Other perinatal conditions */
'P290' = '224' /* Neonatal cardiac failure - Other perinatal conditions */
'P2911' = '224' /* Neonatal tachycardia - Other perinatal conditions */
'P2912' = '224' /* Neonatal bradycardia - Other perinatal conditions */
'P292' = '224' /* Neonatal hypertension - Other perinatal conditions */
'P294' = '224' /* Transient myocardial ischemia in newborn - Other perinatal conditions */
'P2981' = '224' /* Cardiac arrest of newborn - Other perinatal conditions */
'P2989' = '224' /* Oth cardiovasc disorders originating in the perinatal period - Other perinatal conditions */
'P299' = '224' /* Cardiovasc disorder origin in the perinatal period, unsp - Other perinatal conditions */
'P350' = '224' /* Congenital rubella syndrome - Other perinatal conditions */
'P351' = '224' /* Congenital cytomegalovirus infection - Other perinatal conditions */
'P352' = '224' /* Congenital herpesviral [herpes simplex] infection - Other perinatal conditions */
'P353' = '224' /* Congenital viral hepatitis - Other perinatal conditions */
'P358' = '224' /* Other congenital viral diseases - Other perinatal conditions */
'P359' = '224' /* Congenital viral disease, unspecified - Other perinatal conditions */
'P370' = '224' /* Congenital tuberculosis - Other perinatal conditions */
'P371' = '224' /* Congenital toxoplasmosis - Other perinatal conditions */
'P372' = '224' /* Neonatal (disseminated) listeriosis - Other perinatal conditions */
'P373' = '224' /* Congenital falciparum malaria - Other perinatal conditions */
'P374' = '224' /* Other congenital malaria - Other perinatal conditions */
'P375' = '224' /* Neonatal candidiasis - Other perinatal conditions */
'P378' = '224' /* Other specified congenital infectious and parasitic diseases - Other perinatal conditions */
'P379' = '224' /* Congenital infectious or parasitic disease, unspecified - Other perinatal conditions */
'P381' = '224' /* Omphalitis with mild hemorrhage - Other perinatal conditions */
'P389' = '224' /* Omphalitis without hemorrhage - Other perinatal conditions */
'P390' = '224' /* Neonatal infective mastitis - Other perinatal conditions */
'P391' = '224' /* Neonatal conjunctivitis and dacryocystitis - Other perinatal conditions */
'P392' = '224' /* Intra-amniotic infection affecting newborn, NEC - Other perinatal conditions */
'P393' = '224' /* Neonatal urinary tract infection - Other perinatal conditions */
'P394' = '224' /* Neonatal skin infection - Other perinatal conditions */
'P398' = '224' /* Other specified infections specific to the perinatal period - Other perinatal conditions */
'P399' = '224' /* Infection specific to the perinatal period, unspecified - Other perinatal conditions */
'P500' = '224' /* Newborn aff by uterin (fetal) blood loss from vasa previa - Other perinatal conditions */
'P501' = '224' /* Newborn aff by uterin (fetal) blood loss from ruptured cord - Other perinatal conditions */
'P502' = '224' /* Newborn affected by uterin (fetal) blood loss from placenta - Other perinatal conditions */
'P503' = '224' /* Newborn affected by hemorrhage into co-twin - Other perinatal conditions */
'P504' = '224' /* Newborn affected by hemorrhage into maternal circulation - Other perinatal conditions */
'P505' = '224' /* NB aff by uterin blood loss from cut end of co-twin's cord - Other perinatal conditions */
'P508' = '224' /* Newborn affected by other intrauterine (fetal) blood loss - Other perinatal conditions */
'P509' = '224' /* Newborn affected by intrauterine (fetal) blood loss, unsp - Other perinatal conditions */
'P510' = '224' /* Massive umbilical hemorrhage of newborn - Other perinatal conditions */
'P518' = '224' /* Other umbilical hemorrhages of newborn - Other perinatal conditions */
'P519' = '224' /* Umbilical hemorrhage of newborn, unspecified - Other perinatal conditions */
'P520' = '224' /* Intraventricular hemorrhage, grade 1, of newborn - Other perinatal conditions */
'P521' = '224' /* Intraventricular hemorrhage, grade 2, of newborn - Other perinatal conditions */
'P5221' = '224' /* Intraventricular hemorrhage, grade 3, of newborn - Other perinatal conditions */
'P5222' = '224' /* Intraventricular hemorrhage, grade 4, of newborn - Other perinatal conditions */
'P523' = '224' /* Unsp intraventricular (nontraumatic) hemorrhage of newborn - Other perinatal conditions */
'P524' = '224' /* Intracerebral (nontraumatic) hemorrhage of newborn - Other perinatal conditions */
'P525' = '224' /* Subarachnoid (nontraumatic) hemorrhage of newborn - Other perinatal conditions */
'P526' = '224' /* Cerebellar and posterior fossa hemorrhage of newborn - Other perinatal conditions */
'P528' = '224' /* Other intracranial (nontraumatic) hemorrhages of newborn - Other perinatal conditions */
'P529' = '224' /* Intracranial (nontraumatic) hemorrhage of newborn, unsp - Other perinatal conditions */
'P53' = '224' /* Hemorrhagic disease of newborn - Other perinatal conditions */
'P540' = '224' /* Neonatal hematemesis - Other perinatal conditions */
'P541' = '224' /* Neonatal melena - Other perinatal conditions */
'P542' = '224' /* Neonatal rectal hemorrhage - Other perinatal conditions */
'P543' = '224' /* Other neonatal gastrointestinal hemorrhage - Other perinatal conditions */
'P544' = '224' /* Neonatal adrenal hemorrhage - Other perinatal conditions */
'P545' = '224' /* Neonatal cutaneous hemorrhage - Other perinatal conditions */
'P546' = '224' /* Neonatal vaginal hemorrhage - Other perinatal conditions */
'P548' = '224' /* Other specified neonatal hemorrhages - Other perinatal conditions */
'P549' = '224' /* Neonatal hemorrhage, unspecified - Other perinatal conditions */
'P60' = '224' /* Disseminated intravascular coagulation of newborn - Other perinatal conditions */
'P610' = '224' /* Transient neonatal thrombocytopenia - Other perinatal conditions */
'P611' = '224' /* Polycythemia neonatorum - Other perinatal conditions */
'P612' = '224' /* Anemia of prematurity - Other perinatal conditions */
'P613' = '224' /* Congenital anemia from fetal blood loss - Other perinatal conditions */
'P614' = '224' /* Other congenital anemias, not elsewhere classified - Other perinatal conditions */
'P615' = '224' /* Transient neonatal neutropenia - Other perinatal conditions */
'P616' = '224' /* Other transient neonatal disorders of coagulation - Other perinatal conditions */
'P618' = '224' /* Other specified perinatal hematological disorders - Other perinatal conditions */
'P619' = '224' /* Perinatal hematological disorder, unspecified - Other perinatal conditions */
'P700' = '224' /* Syndrome of infant of mother with gestational diabetes - Other perinatal conditions */
'P701' = '224' /* Syndrome of infant of a diabetic mother - Other perinatal conditions */
'P702' = '224' /* Neonatal diabetes mellitus - Other perinatal conditions */
'P703' = '224' /* Iatrogenic neonatal hypoglycemia - Other perinatal conditions */
'P704' = '224' /* Other neonatal hypoglycemia - Other perinatal conditions */
'P708' = '224' /* Oth transitory disorders of carbohydrate metab of newborn - Other perinatal conditions */
'P709' = '224' /* Transitory disorder of carbohydrate metab of newborn, unsp - Other perinatal conditions */
'P710' = '224' /* Cow's milk hypocalcemia in newborn - Other perinatal conditions */
'P711' = '224' /* Other neonatal hypocalcemia - Other perinatal conditions */
'P712' = '224' /* Neonatal hypomagnesemia - Other perinatal conditions */
'P713' = '224' /* Neonatal tetany without calcium or magnesium deficiency - Other perinatal conditions */
'P714' = '224' /* Transitory neonatal hypoparathyroidism - Other perinatal conditions */
'P718' = '224' /* Oth transitory neonatal disord of calcium & magnesium metab - Other perinatal conditions */
'P719' = '224' /* Transitory neonatal disord of calcium & magnesium metab,unsp - Other perinatal conditions */
'P720' = '224' /* Neonatal goiter, not elsewhere classified - Other perinatal conditions */
'P721' = '224' /* Transitory neonatal hyperthyroidism - Other perinatal conditions */
'P722' = '224' /* Oth transitory neonatal disorders of thyroid function, NEC - Other perinatal conditions */
'P728' = '224' /* Other specified transitory neonatal endocrine disorders - Other perinatal conditions */
'P729' = '224' /* Transitory neonatal endocrine disorder, unspecified - Other perinatal conditions */
'P740' = '224' /* Late metabolic acidosis of newborn - Other perinatal conditions */
'P741' = '224' /* Dehydration of newborn - Other perinatal conditions */
'P742' = '224' /* Disturbances of sodium balance of newborn - Other perinatal conditions */
'P743' = '224' /* Disturbances of potassium balance of newborn - Other perinatal conditions */
'P744' = '224' /* Other transitory electrolyte disturbances of newborn - Other perinatal conditions */
'P745' = '224' /* Transitory tyrosinemia of newborn - Other perinatal conditions */
'P746' = '224' /* Transitory hyperammonemia of newborn - Other perinatal conditions */
'P748' = '224' /* Other transitory metabolic disturbances of newborn - Other perinatal conditions */
'P749' = '224' /* Transitory metabolic disturbance of newborn, unspecified - Other perinatal conditions */
'P760' = '224' /* Meconium plug syndrome - Other perinatal conditions */
'P761' = '224' /* Transitory ileus of newborn - Other perinatal conditions */
'P762' = '224' /* Intestinal obstruction due to inspissated milk - Other perinatal conditions */
'P768' = '224' /* Other specified intestinal obstruction of newborn - Other perinatal conditions */
'P769' = '224' /* Intestinal obstruction of newborn, unspecified - Other perinatal conditions */
'P771' = '224' /* Stage 1 necrotizing enterocolitis in newborn - Other perinatal conditions */
'P772' = '224' /* Stage 2 necrotizing enterocolitis in newborn - Other perinatal conditions */
'P773' = '224' /* Stage 3 necrotizing enterocolitis in newborn - Other perinatal conditions */
'P779' = '224' /* Necrotizing enterocolitis in newborn, unspecified - Other perinatal conditions */
'P780' = '224' /* Perinatal intestinal perforation - Other perinatal conditions */
'P781' = '224' /* Other neonatal peritonitis - Other perinatal conditions */
'P782' = '224' /* Neonatal hematemesis and melena d/t swallowed matern blood - Other perinatal conditions */
'P783' = '224' /* Noninfective neonatal diarrhea - Other perinatal conditions */
'P7881' = '224' /* Congenital cirrhosis (of liver) - Other perinatal conditions */
'P7882' = '224' /* Peptic ulcer of newborn - Other perinatal conditions */
'P7883' = '224' /* Newborn esophageal reflux - Other perinatal conditions */
'P7889' = '224' /* Other specified perinatal digestive system disorders - Other perinatal conditions */
'P789' = '224' /* Perinatal digestive system disorder, unspecified - Other perinatal conditions */
'P800' = '224' /* Cold injury syndrome - Other perinatal conditions */
'P808' = '224' /* Other hypothermia of newborn - Other perinatal conditions */
'P809' = '224' /* Hypothermia of newborn, unspecified - Other perinatal conditions */
'P810' = '224' /* Environmental hyperthermia of newborn - Other perinatal conditions */
'P818' = '224' /* Oth disturbances of temperature regulation of newborn - Other perinatal conditions */
'P819' = '224' /* Disturbance of temperature regulation of newborn, unsp - Other perinatal conditions */
'P830' = '224' /* Sclerema neonatorum - Other perinatal conditions */
'P831' = '224' /* Neonatal erythema toxicum - Other perinatal conditions */
'P832' = '224' /* Hydrops fetalis not due to hemolytic disease - Other perinatal conditions */
'P8330' = '224' /* Unspecified edema specific to newborn - Other perinatal conditions */
'P8339' = '224' /* Other edema specific to newborn - Other perinatal conditions */
'P834' = '224' /* Breast engorgement of newborn - Other perinatal conditions */
'P835' = '224' /* Congenital hydrocele - Other perinatal conditions */
'P836' = '224' /* Umbilical polyp of newborn - Other perinatal conditions */
'P838' = '224' /* Other specified conditions of integument specific to newborn - Other perinatal conditions */
'P839' = '224' /* Condition of the integument specific to newborn, unspecified - Other perinatal conditions */
'P84' = '224' /* Other problems with newborn - Other perinatal conditions */
'P90' = '224' /* Convulsions of newborn - Other perinatal conditions */
'P910' = '224' /* Neonatal cerebral ischemia - Other perinatal conditions */
'P911' = '224' /* Acquired periventricular cysts of newborn - Other perinatal conditions */
'P912' = '224' /* Neonatal cerebral leukomalacia - Other perinatal conditions */
'P913' = '224' /* Neonatal cerebral irritability - Other perinatal conditions */
'P914' = '224' /* Neonatal cerebral depression - Other perinatal conditions */
'P915' = '224' /* Neonatal coma - Other perinatal conditions */
'P918' = '224' /* Other specified disturbances of cerebral status of newborn - Other perinatal conditions */
'P919' = '224' /* Disturbance of cerebral status of newborn, unspecified - Other perinatal conditions */
'P9201' = '224' /* Bilious vomiting of newborn - Other perinatal conditions */
'P9209' = '224' /* Other vomiting of newborn - Other perinatal conditions */
'P921' = '224' /* Regurgitation and rumination of newborn - Other perinatal conditions */
'P922' = '224' /* Slow feeding of newborn - Other perinatal conditions */
'P923' = '224' /* Underfeeding of newborn - Other perinatal conditions */
'P924' = '224' /* Overfeeding of newborn - Other perinatal conditions */
'P925' = '224' /* Neonatal difficulty in feeding at breast - Other perinatal conditions */
'P926' = '224' /* Failure to thrive in newborn - Other perinatal conditions */
'P928' = '224' /* Other feeding problems of newborn - Other perinatal conditions */
'P929' = '224' /* Feeding problem of newborn, unspecified - Other perinatal conditions */
'P930' = '224' /* Grey baby syndrome - Other perinatal conditions */
'P938' = '224' /* Oth reactions and intoxications d/t drugs administered to NB - Other perinatal conditions */
'P940' = '224' /* Transient neonatal myasthenia gravis - Other perinatal conditions */
'P941' = '224' /* Congenital hypertonia - Other perinatal conditions */
'P942' = '224' /* Congenital hypotonia - Other perinatal conditions */
'P948' = '224' /* Other disorders of muscle tone of newborn - Other perinatal conditions */
'P949' = '224' /* Disorder of muscle tone of newborn, unspecified - Other perinatal conditions */
'P95' = '224' /* Stillbirth - Other perinatal conditions */
'P960' = '224' /* Congenital renal failure - Other perinatal conditions */
'P963' = '224' /* Wide cranial sutures of newborn - Other perinatal conditions */
'P965' = '224' /* Comp to newborn due to (fetal) intrauterine procedure - Other perinatal conditions */
'P9681' = '224' /* Expsr to (environmental) tobacco smoke in the perinat period - Other perinatal conditions */
'P9682' = '224' /* Delayed separation of umbilical cord - Other perinatal conditions */
'P9683' = '224' /* Meconium staining - Other perinatal conditions */
'P9689' = '224' /* Oth conditions originating in the perinatal period - Other perinatal conditions */
'P969' = '224' /* Condition originating in the perinatal period, unspecified - Other perinatal conditions */
'Q861' = '224' /* Fetal hydantoin syndrome - Other perinatal conditions */
'Q862' = '224' /* Dysmorphism due to warfarin - Other perinatal conditions */
'Q868' = '224' /* Oth congen malform syndromes due to known exogenous causes - Other perinatal conditions */
'R1083' = '224' /* Colic - Other perinatal conditions */
'R6811' = '224' /* Excessive crying of infant (baby) - Other perinatal conditions */
'R6812' = '224' /* Fussy infant (baby) - Other perinatal conditions */
'R6813' = '224' /* Apparent life threatening event in infant (ALTE) - Other perinatal conditions */
'R6819' = '224' /* Other nonspecific symptoms peculiar to infancy - Other perinatal conditions */
'Z412' = '224' /* Encounter for routine and ritual male circumcision - Other perinatal conditions */
'G9611' = '227' /* Dural tear - Spinal cord injury */
'S140XXA' = '227' /* Concussion and edema of cervical spinal cord, init encntr - Spinal cord injury */
'S140XXD' = '227' /* Concussion and edema of cervical spinal cord, subs encntr - Spinal cord injury */
'S140XXS' = '227' /* Concussion and edema of cervical spinal cord, sequela - Spinal cord injury */
'S14101A' = '227' /* Unsp injury at C1 level of cervical spinal cord, init encntr - Spinal cord injury */
'S14101D' = '227' /* Unsp injury at C1 level of cervical spinal cord, subs encntr - Spinal cord injury */
'S14101S' = '227' /* Unsp injury at C1 level of cervical spinal cord, sequela - Spinal cord injury */
'S14102A' = '227' /* Unsp injury at C2 level of cervical spinal cord, init encntr - Spinal cord injury */
'S14102D' = '227' /* Unsp injury at C2 level of cervical spinal cord, subs encntr - Spinal cord injury */
'S14102S' = '227' /* Unsp injury at C2 level of cervical spinal cord, sequela - Spinal cord injury */
'S14103A' = '227' /* Unsp injury at C3 level of cervical spinal cord, init encntr - Spinal cord injury */
'S14103D' = '227' /* Unsp injury at C3 level of cervical spinal cord, subs encntr - Spinal cord injury */
'S14103S' = '227' /* Unsp injury at C3 level of cervical spinal cord, sequela - Spinal cord injury */
'S14104A' = '227' /* Unsp injury at C4 level of cervical spinal cord, init encntr - Spinal cord injury */
'S14104D' = '227' /* Unsp injury at C4 level of cervical spinal cord, subs encntr - Spinal cord injury */
'S14104S' = '227' /* Unsp injury at C4 level of cervical spinal cord, sequela - Spinal cord injury */
'S14105A' = '227' /* Unsp injury at C5 level of cervical spinal cord, init encntr - Spinal cord injury */
'S14105D' = '227' /* Unsp injury at C5 level of cervical spinal cord, subs encntr - Spinal cord injury */
'S14105S' = '227' /* Unsp injury at C5 level of cervical spinal cord, sequela - Spinal cord injury */
'S14106A' = '227' /* Unsp injury at C6 level of cervical spinal cord, init encntr - Spinal cord injury */
'S14106D' = '227' /* Unsp injury at C6 level of cervical spinal cord, subs encntr - Spinal cord injury */
'S14106S' = '227' /* Unsp injury at C6 level of cervical spinal cord, sequela - Spinal cord injury */
'S14107A' = '227' /* Unsp injury at C7 level of cervical spinal cord, init encntr - Spinal cord injury */
'S14107D' = '227' /* Unsp injury at C7 level of cervical spinal cord, subs encntr - Spinal cord injury */
'S14107S' = '227' /* Unsp injury at C7 level of cervical spinal cord, sequela - Spinal cord injury */
'S14108A' = '227' /* Unsp injury at C8 level of cervical spinal cord, init encntr - Spinal cord injury */
'S14108D' = '227' /* Unsp injury at C8 level of cervical spinal cord, subs encntr - Spinal cord injury */
'S14108S' = '227' /* Unsp injury at C8 level of cervical spinal cord, sequela - Spinal cord injury */
'S14109A' = '227' /* Unsp injury at unsp level of cervical spinal cord, init - Spinal cord injury */
'S14109D' = '227' /* Unsp injury at unsp level of cervical spinal cord, subs - Spinal cord injury */
'S14109S' = '227' /* Unsp injury at unsp level of cervical spinal cord, sequela - Spinal cord injury */
'S14111A' = '227' /* Complete lesion at C1 level of cervical spinal cord, init - Spinal cord injury */
'S14111D' = '227' /* Complete lesion at C1 level of cervical spinal cord, subs - Spinal cord injury */
'S14111S' = '227' /* Complete lesion at C1 level of cervical spinal cord, sequela - Spinal cord injury */
'S14112A' = '227' /* Complete lesion at C2 level of cervical spinal cord, init - Spinal cord injury */
'S14112D' = '227' /* Complete lesion at C2 level of cervical spinal cord, subs - Spinal cord injury */
'S14112S' = '227' /* Complete lesion at C2 level of cervical spinal cord, sequela - Spinal cord injury */
'S14113A' = '227' /* Complete lesion at C3 level of cervical spinal cord, init - Spinal cord injury */
'S14113D' = '227' /* Complete lesion at C3 level of cervical spinal cord, subs - Spinal cord injury */
'S14113S' = '227' /* Complete lesion at C3 level of cervical spinal cord, sequela - Spinal cord injury */
'S14114A' = '227' /* Complete lesion at C4 level of cervical spinal cord, init - Spinal cord injury */
'S14114D' = '227' /* Complete lesion at C4 level of cervical spinal cord, subs - Spinal cord injury */
'S14114S' = '227' /* Complete lesion at C4 level of cervical spinal cord, sequela - Spinal cord injury */
'S14115A' = '227' /* Complete lesion at C5 level of cervical spinal cord, init - Spinal cord injury */
'S14115D' = '227' /* Complete lesion at C5 level of cervical spinal cord, subs - Spinal cord injury */
'S14115S' = '227' /* Complete lesion at C5 level of cervical spinal cord, sequela - Spinal cord injury */
'S14116A' = '227' /* Complete lesion at C6 level of cervical spinal cord, init - Spinal cord injury */
'S14116D' = '227' /* Complete lesion at C6 level of cervical spinal cord, subs - Spinal cord injury */
'S14116S' = '227' /* Complete lesion at C6 level of cervical spinal cord, sequela - Spinal cord injury */
'S14117A' = '227' /* Complete lesion at C7 level of cervical spinal cord, init - Spinal cord injury */
'S14117D' = '227' /* Complete lesion at C7 level of cervical spinal cord, subs - Spinal cord injury */
'S14117S' = '227' /* Complete lesion at C7 level of cervical spinal cord, sequela - Spinal cord injury */
'S14118A' = '227' /* Complete lesion at C8 level of cervical spinal cord, init - Spinal cord injury */
'S14118D' = '227' /* Complete lesion at C8 level of cervical spinal cord, subs - Spinal cord injury */
'S14118S' = '227' /* Complete lesion at C8 level of cervical spinal cord, sequela - Spinal cord injury */
'S14119A' = '227' /* Complete lesion at unsp level of cervical spinal cord, init - Spinal cord injury */
'S14119D' = '227' /* Complete lesion at unsp level of cervical spinal cord, subs - Spinal cord injury */
'S14119S' = '227' /* Complete lesion at unsp level of cerv spinal cord, sequela - Spinal cord injury */
'S14121A' = '227' /* Central cord syndrome at C1, init - Spinal cord injury */
'S14121D' = '227' /* Central cord syndrome at C1, subs - Spinal cord injury */
'S14121S' = '227' /* Central cord syndrome at C1, sequela - Spinal cord injury */
'S14122A' = '227' /* Central cord syndrome at C2, init - Spinal cord injury */
'S14122D' = '227' /* Central cord syndrome at C2, subs - Spinal cord injury */
'S14122S' = '227' /* Central cord syndrome at C2, sequela - Spinal cord injury */
'S14123A' = '227' /* Central cord syndrome at C3, init - Spinal cord injury */
'S14123D' = '227' /* Central cord syndrome at C3, subs - Spinal cord injury */
'S14123S' = '227' /* Central cord syndrome at C3, sequela - Spinal cord injury */
'S14124A' = '227' /* Central cord syndrome at C4, init - Spinal cord injury */
'S14124D' = '227' /* Central cord syndrome at C4, subs - Spinal cord injury */
'S14124S' = '227' /* Central cord syndrome at C4, sequela - Spinal cord injury */
'S14125A' = '227' /* Central cord syndrome at C5, init - Spinal cord injury */
'S14125D' = '227' /* Central cord syndrome at C5, subs - Spinal cord injury */
'S14125S' = '227' /* Central cord syndrome at C5, sequela - Spinal cord injury */
'S14126A' = '227' /* Central cord syndrome at C6, init - Spinal cord injury */
'S14126D' = '227' /* Central cord syndrome at C6, subs - Spinal cord injury */
'S14126S' = '227' /* Central cord syndrome at C6, sequela - Spinal cord injury */
'S14127A' = '227' /* Central cord syndrome at C7, init - Spinal cord injury */
'S14127D' = '227' /* Central cord syndrome at C7, subs - Spinal cord injury */
'S14127S' = '227' /* Central cord syndrome at C7, sequela - Spinal cord injury */
'S14128A' = '227' /* Central cord syndrome at C8, init - Spinal cord injury */
'S14128D' = '227' /* Central cord syndrome at C8, subs - Spinal cord injury */
'S14128S' = '227' /* Central cord syndrome at C8, sequela - Spinal cord injury */
'S14129A' = '227' /* Central cord synd at unsp level of cerv spinal cord, init - Spinal cord injury */
'S14129D' = '227' /* Central cord synd at unsp level of cerv spinal cord, subs - Spinal cord injury */
'S14129S' = '227' /* Central cord synd at unsp level of cerv spinal cord, sequela - Spinal cord injury */
'S14131A' = '227' /* Anterior cord syndrome at C1, init - Spinal cord injury */
'S14131D' = '227' /* Anterior cord syndrome at C1, subs - Spinal cord injury */
'S14131S' = '227' /* Anterior cord syndrome at C1, sequela - Spinal cord injury */
'S14132A' = '227' /* Anterior cord syndrome at C2, init - Spinal cord injury */
'S14132D' = '227' /* Anterior cord syndrome at C2, subs - Spinal cord injury */
'S14132S' = '227' /* Anterior cord syndrome at C2, sequela - Spinal cord injury */
'S14133A' = '227' /* Anterior cord syndrome at C3, init - Spinal cord injury */
'S14133D' = '227' /* Anterior cord syndrome at C3, subs - Spinal cord injury */
'S14133S' = '227' /* Anterior cord syndrome at C3, sequela - Spinal cord injury */
'S14134A' = '227' /* Anterior cord syndrome at C4, init - Spinal cord injury */
'S14134D' = '227' /* Anterior cord syndrome at C4, subs - Spinal cord injury */
'S14134S' = '227' /* Anterior cord syndrome at C4, sequela - Spinal cord injury */
'S14135A' = '227' /* Anterior cord syndrome at C5, init - Spinal cord injury */
'S14135D' = '227' /* Anterior cord syndrome at C5, subs - Spinal cord injury */
'S14135S' = '227' /* Anterior cord syndrome at C5, sequela - Spinal cord injury */
'S14136A' = '227' /* Anterior cord syndrome at C6, init - Spinal cord injury */
'S14136D' = '227' /* Anterior cord syndrome at C6, subs - Spinal cord injury */
'S14136S' = '227' /* Anterior cord syndrome at C6, sequela - Spinal cord injury */
'S14137A' = '227' /* Anterior cord syndrome at C7, init - Spinal cord injury */
'S14137D' = '227' /* Anterior cord syndrome at C7, subs - Spinal cord injury */
'S14137S' = '227' /* Anterior cord syndrome at C7, sequela - Spinal cord injury */
'S14138A' = '227' /* Anterior cord syndrome at C8, init - Spinal cord injury */
'S14138D' = '227' /* Anterior cord syndrome at C8, subs - Spinal cord injury */
'S14138S' = '227' /* Anterior cord syndrome at C8, sequela - Spinal cord injury */
'S14139A' = '227' /* Ant cord syndrome at unsp level of cerv spinal cord, init - Spinal cord injury */
'S14139D' = '227' /* Ant cord syndrome at unsp level of cerv spinal cord, subs - Spinal cord injury */
'S14139S' = '227' /* Ant cord syndrome at unsp level of cerv spinal cord, sequela - Spinal cord injury */
'S14141A' = '227' /* Brown-Sequard syndrome at C1, init - Spinal cord injury */
'S14141D' = '227' /* Brown-Sequard syndrome at C1, subs - Spinal cord injury */
'S14141S' = '227' /* Brown-Sequard syndrome at C1, sequela - Spinal cord injury */
'S14142A' = '227' /* Brown-Sequard syndrome at C2, init - Spinal cord injury */
'S14142D' = '227' /* Brown-Sequard syndrome at C2, subs - Spinal cord injury */
'S14142S' = '227' /* Brown-Sequard syndrome at C2, sequela - Spinal cord injury */
'S14143A' = '227' /* Brown-Sequard syndrome at C3, init - Spinal cord injury */
'S14143D' = '227' /* Brown-Sequard syndrome at C3, subs - Spinal cord injury */
'S14143S' = '227' /* Brown-Sequard syndrome at C3, sequela - Spinal cord injury */
'S14144A' = '227' /* Brown-Sequard syndrome at C4, init - Spinal cord injury */
'S14144D' = '227' /* Brown-Sequard syndrome at C4, subs - Spinal cord injury */
'S14144S' = '227' /* Brown-Sequard syndrome at C4, sequela - Spinal cord injury */
'S14145A' = '227' /* Brown-Sequard syndrome at C5, init - Spinal cord injury */
'S14145D' = '227' /* Brown-Sequard syndrome at C5, subs - Spinal cord injury */
'S14145S' = '227' /* Brown-Sequard syndrome at C5, sequela - Spinal cord injury */
'S14146A' = '227' /* Brown-Sequard syndrome at C6, init - Spinal cord injury */
'S14146D' = '227' /* Brown-Sequard syndrome at C6, subs - Spinal cord injury */
'S14146S' = '227' /* Brown-Sequard syndrome at C6, sequela - Spinal cord injury */
'S14147A' = '227' /* Brown-Sequard syndrome at C7, init - Spinal cord injury */
'S14147D' = '227' /* Brown-Sequard syndrome at C7, subs - Spinal cord injury */
'S14147S' = '227' /* Brown-Sequard syndrome at C7, sequela - Spinal cord injury */
'S14148A' = '227' /* Brown-Sequard syndrome at C8, init - Spinal cord injury */
'S14148D' = '227' /* Brown-Sequard syndrome at C8, subs - Spinal cord injury */
'S14148S' = '227' /* Brown-Sequard syndrome at C8, sequela - Spinal cord injury */
'S14149A' = '227' /* Brown-Sequard synd at unsp level of cerv spinal cord, init - Spinal cord injury */
'S14149D' = '227' /* Brown-Sequard synd at unsp level of cerv spinal cord, subs - Spinal cord injury */
'S14149S' = '227' /* Brown-Sequard synd at unsp level of cerv spinal cord, sqla - Spinal cord injury */
'S14151A' = '227' /* Oth incomplete lesion at C1, init - Spinal cord injury */
'S14151D' = '227' /* Oth incomplete lesion at C1, subs - Spinal cord injury */
'S14151S' = '227' /* Oth incomplete lesion at C1, sequela - Spinal cord injury */
'S14152A' = '227' /* Oth incomplete lesion at C2, init - Spinal cord injury */
'S14152D' = '227' /* Oth incomplete lesion at C2, subs - Spinal cord injury */
'S14152S' = '227' /* Oth incomplete lesion at C2, sequela - Spinal cord injury */
'S14153A' = '227' /* Oth incomplete lesion at C3, init - Spinal cord injury */
'S14153D' = '227' /* Oth incomplete lesion at C3, subs - Spinal cord injury */
'S14153S' = '227' /* Oth incomplete lesion at C3, sequela - Spinal cord injury */
'S14154A' = '227' /* Oth incomplete lesion at C4, init - Spinal cord injury */
'S14154D' = '227' /* Oth incomplete lesion at C4, subs - Spinal cord injury */
'S14154S' = '227' /* Oth incomplete lesion at C4, sequela - Spinal cord injury */
'S14155A' = '227' /* Oth incomplete lesion at C5, init - Spinal cord injury */
'S14155D' = '227' /* Oth incomplete lesion at C5, subs - Spinal cord injury */
'S14155S' = '227' /* Oth incomplete lesion at C5, sequela - Spinal cord injury */
'S14156A' = '227' /* Oth incomplete lesion at C6, init - Spinal cord injury */
'S14156D' = '227' /* Oth incomplete lesion at C6, subs - Spinal cord injury */
'S14156S' = '227' /* Oth incomplete lesion at C6, sequela - Spinal cord injury */
'S14157A' = '227' /* Oth incomplete lesion at C7, init - Spinal cord injury */
'S14157D' = '227' /* Oth incomplete lesion at C7, subs - Spinal cord injury */
'S14157S' = '227' /* Oth incomplete lesion at C7, sequela - Spinal cord injury */
'S14158A' = '227' /* Oth incomplete lesion at C8, init - Spinal cord injury */
'S14158D' = '227' /* Oth incomplete lesion at C8, subs - Spinal cord injury */
'S14158S' = '227' /* Oth incomplete lesion at C8, sequela - Spinal cord injury */
'S14159A' = '227' /* Oth incmpl lesion at unsp level of cerv spinal cord, init - Spinal cord injury */
'S14159D' = '227' /* Oth incmpl lesion at unsp level of cerv spinal cord, subs - Spinal cord injury */
'S14159S' = '227' /* Oth incmpl lesion at unsp level of cerv spinal cord, sequela - Spinal cord injury */
'S240XXA' = '227' /* Concussion and edema of thoracic spinal cord, init encntr - Spinal cord injury */
'S240XXD' = '227' /* Concussion and edema of thoracic spinal cord, subs encntr - Spinal cord injury */
'S240XXS' = '227' /* Concussion and edema of thoracic spinal cord, sequela - Spinal cord injury */
'S24101A' = '227' /* Unsp injury at T1 level of thoracic spinal cord, init encntr - Spinal cord injury */
'S24101D' = '227' /* Unsp injury at T1 level of thoracic spinal cord, subs encntr - Spinal cord injury */
'S24101S' = '227' /* Unsp injury at T1 level of thoracic spinal cord, sequela - Spinal cord injury */
'S24102A' = '227' /* Unsp injury at T2-T6 level of thoracic spinal cord, init - Spinal cord injury */
'S24102D' = '227' /* Unsp injury at T2-T6 level of thoracic spinal cord, subs - Spinal cord injury */
'S24102S' = '227' /* Unsp injury at T2-T6 level of thoracic spinal cord, sequela - Spinal cord injury */
'S24103A' = '227' /* Unsp injury at T7-T10 level of thoracic spinal cord, init - Spinal cord injury */
'S24103D' = '227' /* Unsp injury at T7-T10 level of thoracic spinal cord, subs - Spinal cord injury */
'S24103S' = '227' /* Unsp injury at T7-T10 level of thoracic spinal cord, sequela - Spinal cord injury */
'S24104A' = '227' /* Unsp injury at T11-T12 level of thoracic spinal cord, init - Spinal cord injury */
'S24104D' = '227' /* Unsp injury at T11-T12 level of thoracic spinal cord, subs - Spinal cord injury */
'S24104S' = '227' /* Unsp injury at T11-T12, sequela - Spinal cord injury */
'S24109A' = '227' /* Unsp injury at unsp level of thoracic spinal cord, init - Spinal cord injury */
'S24109D' = '227' /* Unsp injury at unsp level of thoracic spinal cord, subs - Spinal cord injury */
'S24109S' = '227' /* Unsp injury at unsp level of thoracic spinal cord, sequela - Spinal cord injury */
'S24111A' = '227' /* Complete lesion at T1 level of thoracic spinal cord, init - Spinal cord injury */
'S24111D' = '227' /* Complete lesion at T1 level of thoracic spinal cord, subs - Spinal cord injury */
'S24111S' = '227' /* Complete lesion at T1 level of thoracic spinal cord, sequela - Spinal cord injury */
'S24112A' = '227' /* Complete lesion at T2-T6 level of thoracic spinal cord, init - Spinal cord injury */
'S24112D' = '227' /* Complete lesion at T2-T6 level of thoracic spinal cord, subs - Spinal cord injury */
'S24112S' = '227' /* Complete lesion at T2-T6, sequela - Spinal cord injury */
'S24113A' = '227' /* Complete lesion at T7-T10, init - Spinal cord injury */
'S24113D' = '227' /* Complete lesion at T7-T10, subs - Spinal cord injury */
'S24113S' = '227' /* Complete lesion at T7-T10, sequela - Spinal cord injury */
'S24114A' = '227' /* Complete lesion at T11-T12, init - Spinal cord injury */
'S24114D' = '227' /* Complete lesion at T11-T12, subs - Spinal cord injury */
'S24114S' = '227' /* Complete lesion at T11-T12, sequela - Spinal cord injury */
'S24119A' = '227' /* Complete lesion at unsp level of thoracic spinal cord, init - Spinal cord injury */
'S24119D' = '227' /* Complete lesion at unsp level of thoracic spinal cord, subs - Spinal cord injury */
'S24119S' = '227' /* Complete lesion at unsp level of thor spinal cord, sequela - Spinal cord injury */
'S24131A' = '227' /* Anterior cord syndrome at T1, init - Spinal cord injury */
'S24131D' = '227' /* Anterior cord syndrome at T1, subs - Spinal cord injury */
'S24131S' = '227' /* Anterior cord syndrome at T1, sequela - Spinal cord injury */
'S24132A' = '227' /* Anterior cord syndrome at T2-T6, init - Spinal cord injury */
'S24132D' = '227' /* Anterior cord syndrome at T2-T6, subs - Spinal cord injury */
'S24132S' = '227' /* Anterior cord syndrome at T2-T6, sequela - Spinal cord injury */
'S24133A' = '227' /* Anterior cord syndrome at T7-T10, init - Spinal cord injury */
'S24133D' = '227' /* Anterior cord syndrome at T7-T10, subs - Spinal cord injury */
'S24133S' = '227' /* Anterior cord syndrome at T7-T10, sequela - Spinal cord injury */
'S24134A' = '227' /* Anterior cord syndrome at T11-T12, init - Spinal cord injury */
'S24134D' = '227' /* Anterior cord syndrome at T11-T12, subs - Spinal cord injury */
'S24134S' = '227' /* Anterior cord syndrome at T11-T12, sequela - Spinal cord injury */
'S24139A' = '227' /* Ant cord syndrome at unsp level of thor spinal cord, init - Spinal cord injury */
'S24139D' = '227' /* Ant cord syndrome at unsp level of thor spinal cord, subs - Spinal cord injury */
'S24139S' = '227' /* Ant cord syndrome at unsp level of thor spinal cord, sequela - Spinal cord injury */
'S24141A' = '227' /* Brown-Sequard syndrome at T1, init - Spinal cord injury */
'S24141D' = '227' /* Brown-Sequard syndrome at T1, subs - Spinal cord injury */
'S24141S' = '227' /* Brown-Sequard syndrome at T1, sequela - Spinal cord injury */
'S24142A' = '227' /* Brown-Sequard syndrome at T2-T6, init - Spinal cord injury */
'S24142D' = '227' /* Brown-Sequard syndrome at T2-T6, subs - Spinal cord injury */
'S24142S' = '227' /* Brown-Sequard syndrome at T2-T6, sequela - Spinal cord injury */
'S24143A' = '227' /* Brown-Sequard syndrome at T7-T10, init - Spinal cord injury */
'S24143D' = '227' /* Brown-Sequard syndrome at T7-T10, subs - Spinal cord injury */
'S24143S' = '227' /* Brown-Sequard syndrome at T7-T10, sequela - Spinal cord injury */
'S24144A' = '227' /* Brown-Sequard syndrome at T11-T12, init - Spinal cord injury */
'S24144D' = '227' /* Brown-Sequard syndrome at T11-T12, subs - Spinal cord injury */
'S24144S' = '227' /* Brown-Sequard syndrome at T11-T12, sequela - Spinal cord injury */
'S24149A' = '227' /* Brown-Sequard synd at unsp level of thor spinal cord, init - Spinal cord injury */
'S24149D' = '227' /* Brown-Sequard synd at unsp level of thor spinal cord, subs - Spinal cord injury */
'S24149S' = '227' /* Brown-Sequard synd at unsp level of thor spinal cord, sqla - Spinal cord injury */
'S24151A' = '227' /* Oth incomplete lesion at T1, init - Spinal cord injury */
'S24151D' = '227' /* Oth incomplete lesion at T1, subs - Spinal cord injury */
'S24151S' = '227' /* Oth incomplete lesion at T1, sequela - Spinal cord injury */
'S24152A' = '227' /* Oth incomplete lesion at T2-T6, init - Spinal cord injury */
'S24152D' = '227' /* Oth incomplete lesion at T2-T6, subs - Spinal cord injury */
'S24152S' = '227' /* Oth incomplete lesion at T2-T6, sequela - Spinal cord injury */
'S24153A' = '227' /* Oth incomplete lesion at T7-T10, init - Spinal cord injury */
'S24153D' = '227' /* Oth incomplete lesion at T7-T10, subs - Spinal cord injury */
'S24153S' = '227' /* Oth incomplete lesion at T7-T10, sequela - Spinal cord injury */
'S24154A' = '227' /* Oth incomplete lesion at T11-T12, init - Spinal cord injury */
'S24154D' = '227' /* Oth incomplete lesion at T11-T12, subs - Spinal cord injury */
'S24154S' = '227' /* Oth incomplete lesion at T11-T12, sequela - Spinal cord injury */
'S24159A' = '227' /* Oth incmpl lesion at unsp level of thor spinal cord, init - Spinal cord injury */
'S24159D' = '227' /* Oth incmpl lesion at unsp level of thor spinal cord, subs - Spinal cord injury */
'S24159S' = '227' /* Oth incmpl lesion at unsp level of thor spinal cord, sequela - Spinal cord injury */
'S3401XA' = '227' /* Concussion and edema of lumbar spinal cord, init encntr - Spinal cord injury */
'S3401XD' = '227' /* Concussion and edema of lumbar spinal cord, subs encntr - Spinal cord injury */
'S3401XS' = '227' /* Concussion and edema of lumbar spinal cord, sequela - Spinal cord injury */
'S3402XA' = '227' /* Concussion and edema of sacral spinal cord, init encntr - Spinal cord injury */
'S3402XD' = '227' /* Concussion and edema of sacral spinal cord, subs encntr - Spinal cord injury */
'S3402XS' = '227' /* Concussion and edema of sacral spinal cord, sequela - Spinal cord injury */
'S34101A' = '227' /* Unsp injury to L1 level of lumbar spinal cord, init encntr - Spinal cord injury */
'S34101D' = '227' /* Unsp injury to L1 level of lumbar spinal cord, subs encntr - Spinal cord injury */
'S34101S' = '227' /* Unsp injury to L1 level of lumbar spinal cord, sequela - Spinal cord injury */
'S34102A' = '227' /* Unsp injury to L2 level of lumbar spinal cord, init encntr - Spinal cord injury */
'S34102D' = '227' /* Unsp injury to L2 level of lumbar spinal cord, subs encntr - Spinal cord injury */
'S34102S' = '227' /* Unsp injury to L2 level of lumbar spinal cord, sequela - Spinal cord injury */
'S34103A' = '227' /* Unsp injury to L3 level of lumbar spinal cord, init encntr - Spinal cord injury */
'S34103D' = '227' /* Unsp injury to L3 level of lumbar spinal cord, subs encntr - Spinal cord injury */
'S34103S' = '227' /* Unsp injury to L3 level of lumbar spinal cord, sequela - Spinal cord injury */
'S34104A' = '227' /* Unsp injury to L4 level of lumbar spinal cord, init encntr - Spinal cord injury */
'S34104D' = '227' /* Unsp injury to L4 level of lumbar spinal cord, subs encntr - Spinal cord injury */
'S34104S' = '227' /* Unsp injury to L4 level of lumbar spinal cord, sequela - Spinal cord injury */
'S34105A' = '227' /* Unsp injury to L5 level of lumbar spinal cord, init encntr - Spinal cord injury */
'S34105D' = '227' /* Unsp injury to L5 level of lumbar spinal cord, subs encntr - Spinal cord injury */
'S34105S' = '227' /* Unsp injury to L5 level of lumbar spinal cord, sequela - Spinal cord injury */
'S34109A' = '227' /* Unsp injury to unsp level of lumbar spinal cord, init encntr - Spinal cord injury */
'S34109D' = '227' /* Unsp injury to unsp level of lumbar spinal cord, subs encntr - Spinal cord injury */
'S34109S' = '227' /* Unsp injury to unsp level of lumbar spinal cord, sequela - Spinal cord injury */
'S34111A' = '227' /* Complete lesion of L1 level of lumbar spinal cord, init - Spinal cord injury */
'S34111D' = '227' /* Complete lesion of L1 level of lumbar spinal cord, subs - Spinal cord injury */
'S34111S' = '227' /* Complete lesion of L1 level of lumbar spinal cord, sequela - Spinal cord injury */
'S34112A' = '227' /* Complete lesion of L2 level of lumbar spinal cord, init - Spinal cord injury */
'S34112D' = '227' /* Complete lesion of L2 level of lumbar spinal cord, subs - Spinal cord injury */
'S34112S' = '227' /* Complete lesion of L2 level of lumbar spinal cord, sequela - Spinal cord injury */
'S34113A' = '227' /* Complete lesion of L3 level of lumbar spinal cord, init - Spinal cord injury */
'S34113D' = '227' /* Complete lesion of L3 level of lumbar spinal cord, subs - Spinal cord injury */
'S34113S' = '227' /* Complete lesion of L3 level of lumbar spinal cord, sequela - Spinal cord injury */
'S34114A' = '227' /* Complete lesion of L4 level of lumbar spinal cord, init - Spinal cord injury */
'S34114D' = '227' /* Complete lesion of L4 level of lumbar spinal cord, subs - Spinal cord injury */
'S34114S' = '227' /* Complete lesion of L4 level of lumbar spinal cord, sequela - Spinal cord injury */
'S34115A' = '227' /* Complete lesion of L5 level of lumbar spinal cord, init - Spinal cord injury */
'S34115D' = '227' /* Complete lesion of L5 level of lumbar spinal cord, subs - Spinal cord injury */
'S34115S' = '227' /* Complete lesion of L5 level of lumbar spinal cord, sequela - Spinal cord injury */
'S34119A' = '227' /* Complete lesion of unsp level of lumbar spinal cord, init - Spinal cord injury */
'S34119D' = '227' /* Complete lesion of unsp level of lumbar spinal cord, subs - Spinal cord injury */
'S34119S' = '227' /* Complete lesion of unsp level of lumbar spinal cord, sequela - Spinal cord injury */
'S34121A' = '227' /* Incomplete lesion of L1 level of lumbar spinal cord, init - Spinal cord injury */
'S34121D' = '227' /* Incomplete lesion of L1 level of lumbar spinal cord, subs - Spinal cord injury */
'S34121S' = '227' /* Incomplete lesion of L1 level of lumbar spinal cord, sequela - Spinal cord injury */
'S34122A' = '227' /* Incomplete lesion of L2 level of lumbar spinal cord, init - Spinal cord injury */
'S34122D' = '227' /* Incomplete lesion of L2 level of lumbar spinal cord, subs - Spinal cord injury */
'S34122S' = '227' /* Incomplete lesion of L2 level of lumbar spinal cord, sequela - Spinal cord injury */
'S34123A' = '227' /* Incomplete lesion of L3 level of lumbar spinal cord, init - Spinal cord injury */
'S34123D' = '227' /* Incomplete lesion of L3 level of lumbar spinal cord, subs - Spinal cord injury */
'S34123S' = '227' /* Incomplete lesion of L3 level of lumbar spinal cord, sequela - Spinal cord injury */
'S34124A' = '227' /* Incomplete lesion of L4 level of lumbar spinal cord, init - Spinal cord injury */
'S34124D' = '227' /* Incomplete lesion of L4 level of lumbar spinal cord, subs - Spinal cord injury */
'S34124S' = '227' /* Incomplete lesion of L4 level of lumbar spinal cord, sequela - Spinal cord injury */
'S34125A' = '227' /* Incomplete lesion of L5 level of lumbar spinal cord, init - Spinal cord injury */
'S34125D' = '227' /* Incomplete lesion of L5 level of lumbar spinal cord, subs - Spinal cord injury */
'S34125S' = '227' /* Incomplete lesion of L5 level of lumbar spinal cord, sequela - Spinal cord injury */
'S34129A' = '227' /* Incomplete lesion of unsp level of lumbar spinal cord, init - Spinal cord injury */
'S34129D' = '227' /* Incomplete lesion of unsp level of lumbar spinal cord, subs - Spinal cord injury */
'S34129S' = '227' /* Incomplete lesion of unsp level of lum spinal cord, sequela - Spinal cord injury */
'S34131A' = '227' /* Complete lesion of sacral spinal cord, initial encounter - Spinal cord injury */
'S34131D' = '227' /* Complete lesion of sacral spinal cord, subsequent encounter - Spinal cord injury */
'S34131S' = '227' /* Complete lesion of sacral spinal cord, sequela - Spinal cord injury */
'S34132A' = '227' /* Incomplete lesion of sacral spinal cord, initial encounter - Spinal cord injury */
'S34132D' = '227' /* Incomplete lesion of sacral spinal cord, subs encntr - Spinal cord injury */
'S34132S' = '227' /* Incomplete lesion of sacral spinal cord, sequela - Spinal cord injury */
'S34139A' = '227' /* Unspecified injury to sacral spinal cord, initial encounter - Spinal cord injury */
'S34139D' = '227' /* Unspecified injury to sacral spinal cord, subs encntr - Spinal cord injury */
'S34139S' = '227' /* Unspecified injury to sacral spinal cord, sequela - Spinal cord injury */
'F70' = '654' /* Mild intellectual disabilities - Developmental disorders */
'F71' = '654' /* Moderate intellectual disabilities - Developmental disorders */
'F72' = '654' /* Severe intellectual disabilities - Developmental disorders */
'F73' = '654' /* Profound intellectual disabilities - Developmental disorders */
'F78' = '654' /* Other intellectual disabilities - Developmental disorders */
'F79' = '654' /* Unspecified intellectual disabilities - Developmental disorders */
'F800' = '654' /* Phonological disorder - Developmental disorders */
'F801' = '654' /* Expressive language disorder - Developmental disorders */
'F802' = '654' /* Mixed receptive-expressive language disorder - Developmental disorders */
'F804' = '654' /* Speech and language development delay due to hearing loss - Developmental disorders */
'F8081' = '654' /* Childhood onset fluency disorder - Developmental disorders */
'F8089' = '654' /* Other developmental disorders of speech and language - Developmental disorders */
'F809' = '654' /* Developmental disorder of speech and language, unspecified - Developmental disorders */
'F810' = '654' /* Specific reading disorder - Developmental disorders */
'F812' = '654' /* Mathematics disorder - Developmental disorders */
'F8181' = '654' /* Disorder of written expression - Developmental disorders */
'F8189' = '654' /* Other developmental disorders of scholastic skills - Developmental disorders */
'F819' = '654' /* Developmental disorder of scholastic skills, unspecified - Developmental disorders */
'F82' = '654' /* Specific developmental disorder of motor function - Developmental disorders */
'F88' = '654' /* Other disorders of psychological development - Developmental disorders */
'F89' = '654' /* Unspecified disorder of psychological development - Developmental disorders */
'F985' = '654' /* Adult onset fluency disorder - Developmental disorders */
'R4183' = '654' /* Borderline intellectual functioning - Developmental disorders */
'R480' = '654' /* Dyslexia and alexia - Developmental disorders */
'F1110' = '661' /* Opioid abuse, uncomplicated - Substance-related disorders */
'F11120' = '661' /* Opioid abuse with intoxication, uncomplicated - Substance-related disorders */
'F11121' = '661' /* Opioid abuse with intoxication delirium - Substance-related disorders */
'F11122' = '661' /* Opioid abuse with intoxication with perceptual disturbance - Substance-related disorders */
'F11129' = '661' /* Opioid abuse with intoxication, unspecified - Substance-related disorders */
'F1114' = '661' /* Opioid abuse with opioid-induced mood disorder - Substance-related disorders */
'F11150' = '661' /* Opioid abuse w opioid-induced psychotic disorder w delusions - Substance-related disorders */
'F11151' = '661' /* Opioid abuse w opioid-induced psychotic disorder w hallucin - Substance-related disorders */
'F11159' = '661' /* Opioid abuse with opioid-induced psychotic disorder, unsp - Substance-related disorders */
'F11181' = '661' /* Opioid abuse with opioid-induced sexual dysfunction - Substance-related disorders */
'F11182' = '661' /* Opioid abuse with opioid-induced sleep disorder - Substance-related disorders */
'F11188' = '661' /* Opioid abuse with other opioid-induced disorder - Substance-related disorders */
'F1119' = '661' /* Opioid abuse with unspecified opioid-induced disorder - Substance-related disorders */
'F1120' = '661' /* Opioid dependence, uncomplicated - Substance-related disorders */
'F1121' = '661' /* Opioid dependence, in remission - Substance-related disorders */
'F11220' = '661' /* Opioid dependence with intoxication, uncomplicated - Substance-related disorders */
'F11221' = '661' /* Opioid dependence with intoxication delirium - Substance-related disorders */
'F11222' = '661' /* Opioid dependence w intoxication with perceptual disturbance - Substance-related disorders */
'F11229' = '661' /* Opioid dependence with intoxication, unspecified - Substance-related disorders */
'F1123' = '661' /* Opioid dependence with withdrawal - Substance-related disorders */
'F1124' = '661' /* Opioid dependence with opioid-induced mood disorder - Substance-related disorders */
'F11250' = '661' /* Opioid depend w opioid-induc psychotic disorder w delusions - Substance-related disorders */
'F11251' = '661' /* Opioid depend w opioid-induc psychotic disorder w hallucin - Substance-related disorders */
'F11259' = '661' /* Opioid dependence w opioid-induced psychotic disorder, unsp - Substance-related disorders */
'F11281' = '661' /* Opioid dependence with opioid-induced sexual dysfunction - Substance-related disorders */
'F11282' = '661' /* Opioid dependence with opioid-induced sleep disorder - Substance-related disorders */
'F11288' = '661' /* Opioid dependence with other opioid-induced disorder - Substance-related disorders */
'F1129' = '661' /* Opioid dependence with unspecified opioid-induced disorder - Substance-related disorders */
'F1190' = '661' /* Opioid use, unspecified, uncomplicated - Substance-related disorders */
'F11920' = '661' /* Opioid use, unspecified with intoxication, uncomplicated - Substance-related disorders */
'F11921' = '661' /* Opioid use, unspecified with intoxication delirium - Substance-related disorders */
'F11922' = '661' /* Opioid use, unsp w intoxication with perceptual disturbance - Substance-related disorders */
'F11929' = '661' /* Opioid use, unspecified with intoxication, unspecified - Substance-related disorders */
'F1193' = '661' /* Opioid use, unspecified with withdrawal - Substance-related disorders */
'F1194' = '661' /* Opioid use, unspecified with opioid-induced mood disorder - Substance-related disorders */
'F11950' = '661' /* Opioid use, unsp w opioid-induc psych disorder w delusions - Substance-related disorders */
'F11951' = '661' /* Opioid use, unsp w opioid-induc psych disorder w hallucin - Substance-related disorders */
'F11959' = '661' /* Opioid use, unsp w opioid-induced psychotic disorder, unsp - Substance-related disorders */
'F11981' = '661' /* Opioid use, unsp with opioid-induced sexual dysfunction - Substance-related disorders */
'F11982' = '661' /* Opioid use, unspecified with opioid-induced sleep disorder - Substance-related disorders */
'F11988' = '661' /* Opioid use, unspecified with other opioid-induced disorder - Substance-related disorders */
'F1199' = '661' /* Opioid use, unsp with unspecified opioid-induced disorder - Substance-related disorders */
'F1210' = '661' /* Cannabis abuse, uncomplicated - Substance-related disorders */
'F12120' = '661' /* Cannabis abuse with intoxication, uncomplicated - Substance-related disorders */
'F12121' = '661' /* Cannabis abuse with intoxication delirium - Substance-related disorders */
'F12122' = '661' /* Cannabis abuse with intoxication with perceptual disturbance - Substance-related disorders */
'F12129' = '661' /* Cannabis abuse with intoxication, unspecified - Substance-related disorders */
'F12150' = '661' /* Cannabis abuse with psychotic disorder with delusions - Substance-related disorders */
'F12151' = '661' /* Cannabis abuse with psychotic disorder with hallucinations - Substance-related disorders */
'F12159' = '661' /* Cannabis abuse with psychotic disorder, unspecified - Substance-related disorders */
'F12180' = '661' /* Cannabis abuse with cannabis-induced anxiety disorder - Substance-related disorders */
'F12188' = '661' /* Cannabis abuse with other cannabis-induced disorder - Substance-related disorders */
'F1219' = '661' /* Cannabis abuse with unspecified cannabis-induced disorder - Substance-related disorders */
'F1220' = '661' /* Cannabis dependence, uncomplicated - Substance-related disorders */
'F1221' = '661' /* Cannabis dependence, in remission - Substance-related disorders */
'F12220' = '661' /* Cannabis dependence with intoxication, uncomplicated - Substance-related disorders */
'F12221' = '661' /* Cannabis dependence with intoxication delirium - Substance-related disorders */
'F12222' = '661' /* Cannabis dependence w intoxication w perceptual disturbance - Substance-related disorders */
'F12229' = '661' /* Cannabis dependence with intoxication, unspecified - Substance-related disorders */
'F12250' = '661' /* Cannabis dependence with psychotic disorder with delusions - Substance-related disorders */
'F12251' = '661' /* Cannabis dependence w psychotic disorder with hallucinations - Substance-related disorders */
'F12259' = '661' /* Cannabis dependence with psychotic disorder, unspecified - Substance-related disorders */
'F12280' = '661' /* Cannabis dependence with cannabis-induced anxiety disorder - Substance-related disorders */
'F12288' = '661' /* Cannabis dependence with other cannabis-induced disorder - Substance-related disorders */
'F1229' = '661' /* Cannabis dependence with unsp cannabis-induced disorder - Substance-related disorders */
'F1290' = '661' /* Cannabis use, unspecified, uncomplicated - Substance-related disorders */
'F12920' = '661' /* Cannabis use, unspecified with intoxication, uncomplicated - Substance-related disorders */
'F12921' = '661' /* Cannabis use, unspecified with intoxication delirium - Substance-related disorders */
'F12922' = '661' /* Cannabis use, unsp w intoxication w perceptual disturbance - Substance-related disorders */
'F12929' = '661' /* Cannabis use, unspecified with intoxication, unspecified - Substance-related disorders */
'F12950' = '661' /* Cannabis use, unsp with psychotic disorder with delusions - Substance-related disorders */
'F12951' = '661' /* Cannabis use, unsp w psychotic disorder with hallucinations - Substance-related disorders */
'F12959' = '661' /* Cannabis use, unsp with psychotic disorder, unspecified - Substance-related disorders */
'F12980' = '661' /* Cannabis use, unspecified with anxiety disorder - Substance-related disorders */
'F12988' = '661' /* Cannabis use, unsp with other cannabis-induced disorder - Substance-related disorders */
'F1299' = '661' /* Cannabis use, unsp with unsp cannabis-induced disorder - Substance-related disorders */
'F1310' = '661' /* Sedative, hypnotic or anxiolytic abuse, uncomplicated - Substance-related disorders */
'F13120' = '661' /* Sedatv/hyp/anxiolytc abuse w intoxication, uncomplicated - Substance-related disorders */
'F13121' = '661' /* Sedatv/hyp/anxiolytc abuse w intoxication delirium - Substance-related disorders */
'F13129' = '661' /* Sedative, hypnotic or anxiolytic abuse w intoxication, unsp - Substance-related disorders */
'F1314' = '661' /* Sedative, hypnotic or anxiolytic abuse w mood disorder - Substance-related disorders */
'F13150' = '661' /* Sedatv/hyp/anxiolytc abuse w psychotic disorder w delusions - Substance-related disorders */
'F13151' = '661' /* Sedatv/hyp/anxiolytc abuse w psychotic disorder w hallucin - Substance-related disorders */
'F13159' = '661' /* Sedatv/hyp/anxiolytc abuse w psychotic disorder, unsp - Substance-related disorders */
'F13180' = '661' /* Sedative, hypnotic or anxiolytic abuse w anxiety disorder - Substance-related disorders */
'F13181' = '661' /* Sedative, hypnotic or anxiolytic abuse w sexual dysfunction - Substance-related disorders */
'F13182' = '661' /* Sedative, hypnotic or anxiolytic abuse w sleep disorder - Substance-related disorders */
'F13188' = '661' /* Sedative, hypnotic or anxiolytic abuse w oth disorder - Substance-related disorders */
'F1319' = '661' /* Sedative, hypnotic or anxiolytic abuse w unsp disorder - Substance-related disorders */
'F1320' = '661' /* Sedative, hypnotic or anxiolytic dependence, uncomplicated - Substance-related disorders */
'F1321' = '661' /* Sedative, hypnotic or anxiolytic dependence, in remission - Substance-related disorders */
'F13220' = '661' /* Sedatv/hyp/anxiolytc dependence w intoxication, uncomp - Substance-related disorders */
'F13221' = '661' /* Sedatv/hyp/anxiolytc dependence w intoxication delirium - Substance-related disorders */
'F13229' = '661' /* Sedatv/hyp/anxiolytc dependence w intoxication, unsp - Substance-related disorders */
'F13230' = '661' /* Sedatv/hyp/anxiolytc dependence w withdrawal, uncomplicated - Substance-related disorders */
'F13231' = '661' /* Sedatv/hyp/anxiolytc dependence w withdrawal delirium - Substance-related disorders */
'F13232' = '661' /* Sedatv/hyp/anxiolytc depend w w/drawal w perceptual disturb - Substance-related disorders */
'F13239' = '661' /* Sedatv/hyp/anxiolytc dependence w withdrawal, unsp - Substance-related disorders */
'F1324' = '661' /* Sedative, hypnotic or anxiolytic dependence w mood disorder - Substance-related disorders */
'F13250' = '661' /* Sedatv/hyp/anxiolytc depend w psychotic disorder w delusions - Substance-related disorders */
'F13251' = '661' /* Sedatv/hyp/anxiolytc depend w psychotic disorder w hallucin - Substance-related disorders */
'F13259' = '661' /* Sedatv/hyp/anxiolytc dependence w psychotic disorder, unsp - Substance-related disorders */
'F1326' = '661' /* Sedatv/hyp/anxiolytc depend w persisting amnestic disorder - Substance-related disorders */
'F1327' = '661' /* Sedatv/hyp/anxiolytc dependence w persisting dementia - Substance-related disorders */
'F13280' = '661' /* Sedatv/hyp/anxiolytc dependence w anxiety disorder - Substance-related disorders */
'F13281' = '661' /* Sedatv/hyp/anxiolytc dependence w sexual dysfunction - Substance-related disorders */
'F13282' = '661' /* Sedative, hypnotic or anxiolytic dependence w sleep disorder - Substance-related disorders */
'F13288' = '661' /* Sedative, hypnotic or anxiolytic dependence w oth disorder - Substance-related disorders */
'F1329' = '661' /* Sedative, hypnotic or anxiolytic dependence w unsp disorder - Substance-related disorders */
'F1390' = '661' /* Sedative, hypnotic, or anxiolytic use, unsp, uncomplicated - Substance-related disorders */
'F13920' = '661' /* Sedatv/hyp/anxiolytc use, unsp w intoxication, uncomplicated - Substance-related disorders */
'F13921' = '661' /* Sedatv/hyp/anxiolytc use, unsp w intoxication delirium - Substance-related disorders */
'F13929' = '661' /* Sedatv/hyp/anxiolytc use, unsp w intoxication, unsp - Substance-related disorders */
'F13930' = '661' /* Sedatv/hyp/anxiolytc use, unsp w withdrawal, uncomplicated - Substance-related disorders */
'F13931' = '661' /* Sedatv/hyp/anxiolytc use, unsp w withdrawal delirium - Substance-related disorders */
'F13932' = '661' /* Sedatv/hyp/anxiolytc use, unsp w w/drawal w perceptl disturb - Substance-related disorders */
'F13939' = '661' /* Sedatv/hyp/anxiolytc use, unsp w withdrawal, unsp - Substance-related disorders */
'F1394' = '661' /* Sedative, hypnotic or anxiolytic use, unsp w mood disorder - Substance-related disorders */
'F13950' = '661' /* Sedatv/hyp/anxiolytc use, unsp w psych disorder w delusions - Substance-related disorders */
'F13951' = '661' /* Sedatv/hyp/anxiolytc use, unsp w psych disorder w hallucin - Substance-related disorders */
'F13959' = '661' /* Sedatv/hyp/anxiolytc use, unsp w psychotic disorder, unsp - Substance-related disorders */
'F1396' = '661' /* Sedatv/hyp/anxiolytc use, unsp w persist amnestic disorder - Substance-related disorders */
'F1397' = '661' /* Sedatv/hyp/anxiolytc use, unsp w persisting dementia - Substance-related disorders */
'F13980' = '661' /* Sedatv/hyp/anxiolytc use, unsp w anxiety disorder - Substance-related disorders */
'F13981' = '661' /* Sedatv/hyp/anxiolytc use, unsp w sexual dysfunction - Substance-related disorders */
'F13982' = '661' /* Sedative, hypnotic or anxiolytic use, unsp w sleep disorder - Substance-related disorders */
'F13988' = '661' /* Sedative, hypnotic or anxiolytic use, unsp w oth disorder - Substance-related disorders */
'F1399' = '661' /* Sedative, hypnotic or anxiolytic use, unsp w unsp disorder - Substance-related disorders */
'F1410' = '661' /* Cocaine abuse, uncomplicated - Substance-related disorders */
'F14120' = '661' /* Cocaine abuse with intoxication, uncomplicated - Substance-related disorders */
'F14121' = '661' /* Cocaine abuse with intoxication with delirium - Substance-related disorders */
'F14122' = '661' /* Cocaine abuse with intoxication with perceptual disturbance - Substance-related disorders */
'F14129' = '661' /* Cocaine abuse with intoxication, unspecified - Substance-related disorders */
'F1414' = '661' /* Cocaine abuse with cocaine-induced mood disorder - Substance-related disorders */
'F14150' = '661' /* Cocaine abuse w cocaine-induc psychotic disorder w delusions - Substance-related disorders */
'F14151' = '661' /* Cocaine abuse w cocaine-induc psychotic disorder w hallucin - Substance-related disorders */
'F14159' = '661' /* Cocaine abuse with cocaine-induced psychotic disorder, unsp - Substance-related disorders */
'F14180' = '661' /* Cocaine abuse with cocaine-induced anxiety disorder - Substance-related disorders */
'F14181' = '661' /* Cocaine abuse with cocaine-induced sexual dysfunction - Substance-related disorders */
'F14182' = '661' /* Cocaine abuse with cocaine-induced sleep disorder - Substance-related disorders */
'F14188' = '661' /* Cocaine abuse with other cocaine-induced disorder - Substance-related disorders */
'F1419' = '661' /* Cocaine abuse with unspecified cocaine-induced disorder - Substance-related disorders */
'F1420' = '661' /* Cocaine dependence, uncomplicated - Substance-related disorders */
'F1421' = '661' /* Cocaine dependence, in remission - Substance-related disorders */
'F14220' = '661' /* Cocaine dependence with intoxication, uncomplicated - Substance-related disorders */
'F14221' = '661' /* Cocaine dependence with intoxication delirium - Substance-related disorders */
'F14222' = '661' /* Cocaine dependence w intoxication w perceptual disturbance - Substance-related disorders */
'F14229' = '661' /* Cocaine dependence with intoxication, unspecified - Substance-related disorders */
'F1423' = '661' /* Cocaine dependence with withdrawal - Substance-related disorders */
'F1424' = '661' /* Cocaine dependence with cocaine-induced mood disorder - Substance-related disorders */
'F14250' = '661' /* Cocaine depend w cocaine-induc psych disorder w delusions - Substance-related disorders */
'F14251' = '661' /* Cocaine depend w cocaine-induc psychotic disorder w hallucin - Substance-related disorders */
'F14259' = '661' /* Cocaine dependence w cocaine-induc psychotic disorder, unsp - Substance-related disorders */
'F14280' = '661' /* Cocaine dependence with cocaine-induced anxiety disorder - Substance-related disorders */
'F14281' = '661' /* Cocaine dependence with cocaine-induced sexual dysfunction - Substance-related disorders */
'F14282' = '661' /* Cocaine dependence with cocaine-induced sleep disorder - Substance-related disorders */
'F14288' = '661' /* Cocaine dependence with other cocaine-induced disorder - Substance-related disorders */
'F1429' = '661' /* Cocaine dependence with unspecified cocaine-induced disorder - Substance-related disorders */
'F1490' = '661' /* Cocaine use, unspecified, uncomplicated - Substance-related disorders */
'F14920' = '661' /* Cocaine use, unspecified with intoxication, uncomplicated - Substance-related disorders */
'F14921' = '661' /* Cocaine use, unspecified with intoxication delirium - Substance-related disorders */
'F14922' = '661' /* Cocaine use, unsp w intoxication with perceptual disturbance - Substance-related disorders */
'F14929' = '661' /* Cocaine use, unspecified with intoxication, unspecified - Substance-related disorders */
'F1494' = '661' /* Cocaine use, unspecified with cocaine-induced mood disorder - Substance-related disorders */
'F14950' = '661' /* Cocaine use, unsp w cocaine-induc psych disorder w delusions - Substance-related disorders */
'F14951' = '661' /* Cocaine use, unsp w cocaine-induc psych disorder w hallucin - Substance-related disorders */
'F14959' = '661' /* Cocaine use, unsp w cocaine-induced psychotic disorder, unsp - Substance-related disorders */
'F14980' = '661' /* Cocaine use, unsp with cocaine-induced anxiety disorder - Substance-related disorders */
'F14981' = '661' /* Cocaine use, unsp with cocaine-induced sexual dysfunction - Substance-related disorders */
'F14982' = '661' /* Cocaine use, unspecified with cocaine-induced sleep disorder - Substance-related disorders */
'F14988' = '661' /* Cocaine use, unspecified with other cocaine-induced disorder - Substance-related disorders */
'F1499' = '661' /* Cocaine use, unsp with unspecified cocaine-induced disorder - Substance-related disorders */
'F1510' = '661' /* Other stimulant abuse, uncomplicated - Substance-related disorders */
'F15120' = '661' /* Other stimulant abuse with intoxication, uncomplicated - Substance-related disorders */
'F15121' = '661' /* Other stimulant abuse with intoxication delirium - Substance-related disorders */
'F15122' = '661' /* Oth stimulant abuse w intoxication w perceptual disturbance - Substance-related disorders */
'F15129' = '661' /* Other stimulant abuse with intoxication, unspecified - Substance-related disorders */
'F1514' = '661' /* Other stimulant abuse with stimulant-induced mood disorder - Substance-related disorders */
'F15150' = '661' /* Oth stimulant abuse w stim-induce psych disorder w delusions - Substance-related disorders */
'F15151' = '661' /* Oth stimulant abuse w stim-induce psych disorder w hallucin - Substance-related disorders */
'F15159' = '661' /* Oth stimulant abuse w stim-induce psychotic disorder, unsp - Substance-related disorders */
'F15180' = '661' /* Oth stimulant abuse with stimulant-induced anxiety disorder - Substance-related disorders */
'F15181' = '661' /* Oth stimulant abuse w stimulant-induced sexual dysfunction - Substance-related disorders */
'F15182' = '661' /* Other stimulant abuse with stimulant-induced sleep disorder - Substance-related disorders */
'F15188' = '661' /* Other stimulant abuse with other stimulant-induced disorder - Substance-related disorders */
'F1519' = '661' /* Other stimulant abuse with unsp stimulant-induced disorder - Substance-related disorders */
'F1520' = '661' /* Other stimulant dependence, uncomplicated - Substance-related disorders */
'F1521' = '661' /* Other stimulant dependence, in remission - Substance-related disorders */
'F15220' = '661' /* Other stimulant dependence with intoxication, uncomplicated - Substance-related disorders */
'F15221' = '661' /* Other stimulant dependence with intoxication delirium - Substance-related disorders */
'F15222' = '661' /* Oth stimulant dependence w intox w perceptual disturbance - Substance-related disorders */
'F15229' = '661' /* Other stimulant dependence with intoxication, unspecified - Substance-related disorders */
'F1523' = '661' /* Other stimulant dependence with withdrawal - Substance-related disorders */
'F1524' = '661' /* Oth stimulant dependence w stimulant-induced mood disorder - Substance-related disorders */
'F15250' = '661' /* Oth stim depend w stim-induce psych disorder w delusions - Substance-related disorders */
'F15251' = '661' /* Oth stimulant depend w stim-induce psych disorder w hallucin - Substance-related disorders */
'F15259' = '661' /* Oth stimulant depend w stim-induce psychotic disorder, unsp - Substance-related disorders */
'F15280' = '661' /* Oth stimulant dependence w stim-induce anxiety disorder - Substance-related disorders */
'F15281' = '661' /* Oth stimulant dependence w stim-induce sexual dysfunction - Substance-related disorders */
'F15282' = '661' /* Oth stimulant dependence w stimulant-induced sleep disorder - Substance-related disorders */
'F15288' = '661' /* Oth stimulant dependence with oth stimulant-induced disorder - Substance-related disorders */
'F1529' = '661' /* Oth stimulant dependence w unsp stimulant-induced disorder - Substance-related disorders */
'F1590' = '661' /* Other stimulant use, unspecified, uncomplicated - Substance-related disorders */
'F15920' = '661' /* Other stimulant use, unsp with intoxication, uncomplicated - Substance-related disorders */
'F15921' = '661' /* Other stimulant use, unspecified with intoxication delirium - Substance-related disorders */
'F15922' = '661' /* Oth stimulant use, unsp w intox w perceptual disturbance - Substance-related disorders */
'F15929' = '661' /* Other stimulant use, unsp with intoxication, unspecified - Substance-related disorders */
'F1593' = '661' /* Other stimulant use, unspecified with withdrawal - Substance-related disorders */
'F1594' = '661' /* Oth stimulant use, unsp with stimulant-induced mood disorder - Substance-related disorders */
'F15950' = '661' /* Oth stim use, unsp w stim-induce psych disorder w delusions - Substance-related disorders */
'F15951' = '661' /* Oth stim use, unsp w stim-induce psych disorder w hallucin - Substance-related disorders */
'F15959' = '661' /* Oth stimulant use, unsp w stim-induce psych disorder, unsp - Substance-related disorders */
'F15980' = '661' /* Oth stimulant use, unsp w stimulant-induced anxiety disorder - Substance-related disorders */
'F15981' = '661' /* Oth stimulant use, unsp w stim-induce sexual dysfunction - Substance-related disorders */
'F15982' = '661' /* Oth stimulant use, unsp w stimulant-induced sleep disorder - Substance-related disorders */
'F15988' = '661' /* Oth stimulant use, unsp with oth stimulant-induced disorder - Substance-related disorders */
'F1599' = '661' /* Oth stimulant use, unsp with unsp stimulant-induced disorder - Substance-related disorders */
'F1610' = '661' /* Hallucinogen abuse, uncomplicated - Substance-related disorders */
'F16120' = '661' /* Hallucinogen abuse with intoxication, uncomplicated - Substance-related disorders */
'F16121' = '661' /* Hallucinogen abuse with intoxication with delirium - Substance-related disorders */
'F16122' = '661' /* Hallucinogen abuse w intoxication w perceptual disturbance - Substance-related disorders */
'F16129' = '661' /* Hallucinogen abuse with intoxication, unspecified - Substance-related disorders */
'F1614' = '661' /* Hallucinogen abuse with hallucinogen-induced mood disorder - Substance-related disorders */
'F16150' = '661' /* Hallucinogen abuse w psychotic disorder w delusions - Substance-related disorders */
'F16151' = '661' /* Hallucinogen abuse w psychotic disorder w hallucinations - Substance-related disorders */
'F16159' = '661' /* Hallucinogen abuse w psychotic disorder, unsp - Substance-related disorders */
'F16180' = '661' /* Hallucinogen abuse w hallucinogen-induced anxiety disorder - Substance-related disorders */
'F16183' = '661' /* Hallucign abuse w hallucign persisting perception disorder - Substance-related disorders */
'F16188' = '661' /* Hallucinogen abuse with other hallucinogen-induced disorder - Substance-related disorders */
'F1619' = '661' /* Hallucinogen abuse with unsp hallucinogen-induced disorder - Substance-related disorders */
'F1620' = '661' /* Hallucinogen dependence, uncomplicated - Substance-related disorders */
'F1621' = '661' /* Hallucinogen dependence, in remission - Substance-related disorders */
'F16220' = '661' /* Hallucinogen dependence with intoxication, uncomplicated - Substance-related disorders */
'F16221' = '661' /* Hallucinogen dependence with intoxication with delirium - Substance-related disorders */
'F16229' = '661' /* Hallucinogen dependence with intoxication, unspecified - Substance-related disorders */
'F1624' = '661' /* Hallucinogen dependence w hallucinogen-induced mood disorder - Substance-related disorders */
'F16250' = '661' /* Hallucinogen dependence w psychotic disorder w delusions - Substance-related disorders */
'F16251' = '661' /* Hallucinogen dependence w psychotic disorder w hallucin - Substance-related disorders */
'F16259' = '661' /* Hallucinogen dependence w psychotic disorder, unsp - Substance-related disorders */
'F16280' = '661' /* Hallucinogen dependence w anxiety disorder - Substance-related disorders */
'F16283' = '661' /* Hallucign depend w hallucign persisting perception disorder - Substance-related disorders */
'F16288' = '661' /* Hallucinogen dependence w oth hallucinogen-induced disorder - Substance-related disorders */
'F1629' = '661' /* Hallucinogen dependence w unsp hallucinogen-induced disorder - Substance-related disorders */
'F1690' = '661' /* Hallucinogen use, unspecified, uncomplicated - Substance-related disorders */
'F16920' = '661' /* Hallucinogen use, unsp with intoxication, uncomplicated - Substance-related disorders */
'F16921' = '661' /* Hallucinogen use, unsp with intoxication with delirium - Substance-related disorders */
'F16929' = '661' /* Hallucinogen use, unspecified with intoxication, unspecified - Substance-related disorders */
'F1694' = '661' /* Hallucinogen use, unsp w hallucinogen-induced mood disorder - Substance-related disorders */
'F16950' = '661' /* Hallucinogen use, unsp w psychotic disorder w delusions - Substance-related disorders */
'F16951' = '661' /* Hallucinogen use, unsp w psychotic disorder w hallucinations - Substance-related disorders */
'F16959' = '661' /* Hallucinogen use, unsp w psychotic disorder, unsp - Substance-related disorders */
'F16980' = '661' /* Hallucinogen use, unsp w anxiety disorder - Substance-related disorders */
'F16983' = '661' /* Hallucign use, unsp w hallucign persist perception disorder - Substance-related disorders */
'F16988' = '661' /* Hallucinogen use, unsp w oth hallucinogen-induced disorder - Substance-related disorders */
'F1699' = '661' /* Hallucinogen use, unsp w unsp hallucinogen-induced disorder - Substance-related disorders */
'F17200' = '661' /* Nicotine dependence, unspecified, uncomplicated - Substance-related disorders */
'F17201' = '661' /* Nicotine dependence, unspecified, in remission - Substance-related disorders */
'F17203' = '661' /* Nicotine dependence unspecified, with withdrawal - Substance-related disorders */
'F17208' = '661' /* Nicotine dependence, unsp, w oth nicotine-induced disorders - Substance-related disorders */
'F17209' = '661' /* Nicotine dependence, unsp, w unsp nicotine-induced disorders - Substance-related disorders */
'F17210' = '661' /* Nicotine dependence, cigarettes, uncomplicated - Substance-related disorders */
'F17211' = '661' /* Nicotine dependence, cigarettes, in remission - Substance-related disorders */
'F17213' = '661' /* Nicotine dependence, cigarettes, with withdrawal - Substance-related disorders */
'F17218' = '661' /* Nicotine dependence, cigarettes, w oth disorders - Substance-related disorders */
'F17219' = '661' /* Nicotine dependence, cigarettes, w unsp disorders - Substance-related disorders */
'F17220' = '661' /* Nicotine dependence, chewing tobacco, uncomplicated - Substance-related disorders */
'F17221' = '661' /* Nicotine dependence, chewing tobacco, in remission - Substance-related disorders */
'F17223' = '661' /* Nicotine dependence, chewing tobacco, with withdrawal - Substance-related disorders */
'F17228' = '661' /* Nicotine dependence, chewing tobacco, w oth disorders - Substance-related disorders */
'F17229' = '661' /* Nicotine dependence, chewing tobacco, w unsp disorders - Substance-related disorders */
'F17290' = '661' /* Nicotine dependence, other tobacco product, uncomplicated - Substance-related disorders */
'F17291' = '661' /* Nicotine dependence, other tobacco product, in remission - Substance-related disorders */
'F17293' = '661' /* Nicotine dependence, other tobacco product, with withdrawal - Substance-related disorders */
'F17298' = '661' /* Nicotine dependence, oth tobacco product, w oth disorders - Substance-related disorders */
'F17299' = '661' /* Nicotine dependence, oth tobacco product, w unsp disorders - Substance-related disorders */
'F1810' = '661' /* Inhalant abuse, uncomplicated - Substance-related disorders */
'F18120' = '661' /* Inhalant abuse with intoxication, uncomplicated - Substance-related disorders */
'F18121' = '661' /* Inhalant abuse with intoxication delirium - Substance-related disorders */
'F18129' = '661' /* Inhalant abuse with intoxication, unspecified - Substance-related disorders */
'F1814' = '661' /* Inhalant abuse with inhalant-induced mood disorder - Substance-related disorders */
'F18150' = '661' /* Inhalant abuse w inhalnt-induce psych disorder w delusions - Substance-related disorders */
'F18151' = '661' /* Inhalant abuse w inhalnt-induce psych disorder w hallucin - Substance-related disorders */
'F18159' = '661' /* Inhalant abuse w inhalant-induced psychotic disorder, unsp - Substance-related disorders */
'F1817' = '661' /* Inhalant abuse with inhalant-induced dementia - Substance-related disorders */
'F18180' = '661' /* Inhalant abuse with inhalant-induced anxiety disorder - Substance-related disorders */
'F18188' = '661' /* Inhalant abuse with other inhalant-induced disorder - Substance-related disorders */
'F1819' = '661' /* Inhalant abuse with unspecified inhalant-induced disorder - Substance-related disorders */
'F1820' = '661' /* Inhalant dependence, uncomplicated - Substance-related disorders */
'F1821' = '661' /* Inhalant dependence, in remission - Substance-related disorders */
'F18220' = '661' /* Inhalant dependence with intoxication, uncomplicated - Substance-related disorders */
'F18221' = '661' /* Inhalant dependence with intoxication delirium - Substance-related disorders */
'F18229' = '661' /* Inhalant dependence with intoxication, unspecified - Substance-related disorders */
'F1824' = '661' /* Inhalant dependence with inhalant-induced mood disorder - Substance-related disorders */
'F18250' = '661' /* Inhalant depend w inhalnt-induce psych disorder w delusions - Substance-related disorders */
'F18251' = '661' /* Inhalant depend w inhalnt-induce psych disorder w hallucin - Substance-related disorders */
'F18259' = '661' /* Inhalant depend w inhalnt-induce psychotic disorder, unsp - Substance-related disorders */
'F1827' = '661' /* Inhalant dependence with inhalant-induced dementia - Substance-related disorders */
'F18280' = '661' /* Inhalant dependence with inhalant-induced anxiety disorder - Substance-related disorders */
'F18288' = '661' /* Inhalant dependence with other inhalant-induced disorder - Substance-related disorders */
'F1829' = '661' /* Inhalant dependence with unsp inhalant-induced disorder - Substance-related disorders */
'F1890' = '661' /* Inhalant use, unspecified, uncomplicated - Substance-related disorders */
'F18920' = '661' /* Inhalant use, unspecified with intoxication, uncomplicated - Substance-related disorders */
'F18921' = '661' /* Inhalant use, unspecified with intoxication with delirium - Substance-related disorders */
'F18929' = '661' /* Inhalant use, unspecified with intoxication, unspecified - Substance-related disorders */
'F1894' = '661' /* Inhalant use, unsp with inhalant-induced mood disorder - Substance-related disorders */
'F18950' = '661' /* Inhalant use, unsp w inhalnt-induce psych disord w delusions - Substance-related disorders */
'F18951' = '661' /* Inhalant use, unsp w inhalnt-induce psych disord w hallucin - Substance-related disorders */
'F18959' = '661' /* Inhalant use, unsp w inhalnt-induce psychotic disorder, unsp - Substance-related disorders */
'F1897' = '661' /* Inhalant use, unsp with inhalant-induced persisting dementia - Substance-related disorders */
'F18980' = '661' /* Inhalant use, unsp with inhalant-induced anxiety disorder - Substance-related disorders */
'F18988' = '661' /* Inhalant use, unsp with other inhalant-induced disorder - Substance-related disorders */
'F1899' = '661' /* Inhalant use, unsp with unsp inhalant-induced disorder - Substance-related disorders */
'F1910' = '661' /* Other psychoactive substance abuse, uncomplicated - Substance-related disorders */
'F19120' = '661' /* Oth psychoactive substance abuse w intoxication, uncomp - Substance-related disorders */
'F19121' = '661' /* Oth psychoactive substance abuse with intoxication delirium - Substance-related disorders */
'F19122' = '661' /* Oth psychoactv substance abuse w intox w perceptual disturb - Substance-related disorders */
'F19129' = '661' /* Other psychoactive substance abuse with intoxication, unsp - Substance-related disorders */
'F1914' = '661' /* Oth psychoactive substance abuse w mood disorder - Substance-related disorders */
'F19150' = '661' /* Oth psychoactv substance abuse w psych disorder w delusions - Substance-related disorders */
'F19151' = '661' /* Oth psychoactv substance abuse w psych disorder w hallucin - Substance-related disorders */
'F19159' = '661' /* Oth psychoactive substance abuse w psychotic disorder, unsp - Substance-related disorders */
'F1916' = '661' /* Oth psychoactv substance abuse w persist amnestic disorder - Substance-related disorders */
'F1917' = '661' /* Oth psychoactive substance abuse w persisting dementia - Substance-related disorders */
'F19180' = '661' /* Oth psychoactive substance abuse w anxiety disorder - Substance-related disorders */
'F19181' = '661' /* Oth psychoactive substance abuse w sexual dysfunction - Substance-related disorders */
'F19182' = '661' /* Oth psychoactive substance abuse w sleep disorder - Substance-related disorders */
'F19188' = '661' /* Oth psychoactive substance abuse w oth disorder - Substance-related disorders */
'F1919' = '661' /* Oth psychoactive substance abuse w unsp disorder - Substance-related disorders */
'F1920' = '661' /* Other psychoactive substance dependence, uncomplicated - Substance-related disorders */
'F1921' = '661' /* Other psychoactive substance dependence, in remission - Substance-related disorders */
'F19220' = '661' /* Oth psychoactive substance dependence w intoxication, uncomp - Substance-related disorders */
'F19221' = '661' /* Oth psychoactive substance dependence w intox delirium - Substance-related disorders */
'F19222' = '661' /* Oth psychoactv substance depend w intox w perceptual disturb - Substance-related disorders */
'F19229' = '661' /* Oth psychoactive substance dependence w intoxication, unsp - Substance-related disorders */
'F19230' = '661' /* Oth psychoactive substance dependence w withdrawal, uncomp - Substance-related disorders */
'F19231' = '661' /* Oth psychoactive substance dependence w withdrawal delirium - Substance-related disorders */
'F19232' = '661' /* Oth psychoactv sub depend w w/drawal w perceptl disturb - Substance-related disorders */
'F19239' = '661' /* Oth psychoactive substance dependence with withdrawal, unsp - Substance-related disorders */
'F1924' = '661' /* Oth psychoactive substance dependence w mood disorder - Substance-related disorders */
'F19250' = '661' /* Oth psychoactv substance depend w psych disorder w delusions - Substance-related disorders */
'F19251' = '661' /* Oth psychoactv substance depend w psych disorder w hallucin - Substance-related disorders */
'F19259' = '661' /* Oth psychoactv substance depend w psychotic disorder, unsp - Substance-related disorders */
'F1926' = '661' /* Oth psychoactv substance depend w persist amnestic disorder - Substance-related disorders */
'F1927' = '661' /* Oth psychoactive substance dependence w persisting dementia - Substance-related disorders */
'F19280' = '661' /* Oth psychoactive substance dependence w anxiety disorder - Substance-related disorders */
'F19281' = '661' /* Oth psychoactive substance dependence w sexual dysfunction - Substance-related disorders */
'F19282' = '661' /* Oth psychoactive substance dependence w sleep disorder - Substance-related disorders */
'F19288' = '661' /* Oth psychoactive substance dependence w oth disorder - Substance-related disorders */
'F1929' = '661' /* Oth psychoactive substance dependence w unsp disorder - Substance-related disorders */
'F1990' = '661' /* Other psychoactive substance use, unspecified, uncomplicated - Substance-related disorders */
'F19920' = '661' /* Oth psychoactive substance use, unsp w intoxication, uncomp - Substance-related disorders */
'F19921' = '661' /* Oth psychoactive substance use, unsp w intox w delirium - Substance-related disorders */
'F19922' = '661' /* Oth psychoactv sub use, unsp w intox w perceptl disturb - Substance-related disorders */
'F19929' = '661' /* Oth psychoactive substance use, unsp with intoxication, unsp - Substance-related disorders */
'F19930' = '661' /* Oth psychoactive substance use, unsp w withdrawal, uncomp - Substance-related disorders */
'F19931' = '661' /* Oth psychoactive substance use, unsp w withdrawal delirium - Substance-related disorders */
'F19932' = '661' /* Oth psychoactv sub use, unsp w w/drawal w perceptl disturb - Substance-related disorders */
'F19939' = '661' /* Other psychoactive substance use, unsp with withdrawal, unsp - Substance-related disorders */
'F1994' = '661' /* Oth psychoactive substance use, unsp w mood disorder - Substance-related disorders */
'F19950' = '661' /* Oth psychoactv sub use, unsp w psych disorder w delusions - Substance-related disorders */
'F19951' = '661' /* Oth psychoactv sub use, unsp w psych disorder w hallucin - Substance-related disorders */
'F19959' = '661' /* Oth psychoactv substance use, unsp w psych disorder, unsp - Substance-related disorders */
'F1996' = '661' /* Oth psychoactv sub use, unsp w persist amnestic disorder - Substance-related disorders */
'F1997' = '661' /* Oth psychoactive substance use, unsp w persisting dementia - Substance-related disorders */
'F19980' = '661' /* Oth psychoactive substance use, unsp w anxiety disorder - Substance-related disorders */
'F19981' = '661' /* Oth psychoactive substance use, unsp w sexual dysfunction - Substance-related disorders */
'F19982' = '661' /* Oth psychoactive substance use, unsp w sleep disorder - Substance-related disorders */
'F19988' = '661' /* Oth psychoactive substance use, unsp w oth disorder - Substance-related disorders */
'F1999' = '661' /* Oth psychoactive substance use, unsp w unsp disorder - Substance-related disorders */
'F550' = '661' /* Abuse of antacids - Substance-related disorders */
'F551' = '661' /* Abuse of herbal or folk remedies - Substance-related disorders */
'F552' = '661' /* Abuse of laxatives - Substance-related disorders */
'F553' = '661' /* Abuse of steroids or hormones - Substance-related disorders */
'F554' = '661' /* Abuse of vitamins - Substance-related disorders */
'F558' = '661' /* Abuse of other non-psychoactive substances - Substance-related disorders */
'O355XX0' = '661' /* Maternal care for (suspected) damage to fetus by drugs, unsp - Substance-related disorders */
'O355XX1' = '661' /* Maternal care for damage to fetus by drugs, fetus 1 - Substance-related disorders */
'O355XX2' = '661' /* Maternal care for damage to fetus by drugs, fetus 2 - Substance-related disorders */
'O355XX3' = '661' /* Maternal care for damage to fetus by drugs, fetus 3 - Substance-related disorders */
'O355XX4' = '661' /* Maternal care for damage to fetus by drugs, fetus 4 - Substance-related disorders */
'O355XX5' = '661' /* Maternal care for damage to fetus by drugs, fetus 5 - Substance-related disorders */
'O355XX9' = '661' /* Maternal care for (suspected) damage to fetus by drugs, oth - Substance-related disorders */
'O99320' = '661' /* Drug use complicating pregnancy, unspecified trimester - Substance-related disorders */
'O99321' = '661' /* Drug use complicating pregnancy, first trimester - Substance-related disorders */
'O99322' = '661' /* Drug use complicating pregnancy, second trimester - Substance-related disorders */
'O99323' = '661' /* Drug use complicating pregnancy, third trimester - Substance-related disorders */
'O99324' = '661' /* Drug use complicating childbirth - Substance-related disorders */
'O99325' = '661' /* Drug use complicating the puerperium - Substance-related disorders */
'P0441' = '661' /* Newborn affected by maternal use of cocaine - Substance-related disorders */
'P0449' = '661' /* Newborn affected by maternal use of drugs of addiction - Substance-related disorders */
'P961' = '661' /* Neonatal w/drawal symp from matern use of drugs of addiction - Substance-related disorders */
'P962' = '661' /* Withdrawal symptoms from therapeutic use of drugs in newborn - Substance-related disorders */
'T400X1A' = '661' /* Poisoning by opium, accidental (unintentional), init encntr - Substance-related disorders */
'T400X1D' = '661' /* Poisoning by opium, accidental (unintentional), subs encntr - Substance-related disorders */
'T400X1S' = '661' /* Poisoning by opium, accidental (unintentional), sequela - Substance-related disorders */
'T400X3A' = '661' /* Poisoning by opium, assault, initial encounter - Substance-related disorders */
'T400X3D' = '661' /* Poisoning by opium, assault, subsequent encounter - Substance-related disorders */
'T400X3S' = '661' /* Poisoning by opium, assault, sequela - Substance-related disorders */
'T400X4A' = '661' /* Poisoning by opium, undetermined, initial encounter - Substance-related disorders */
'T400X4D' = '661' /* Poisoning by opium, undetermined, subsequent encounter - Substance-related disorders */
'T400X4S' = '661' /* Poisoning by opium, undetermined, sequela - Substance-related disorders */
'T400X5A' = '661' /* Adverse effect of opium, initial encounter - Substance-related disorders */
'T400X5D' = '661' /* Adverse effect of opium, subsequent encounter - Substance-related disorders */
'T400X5S' = '661' /* Adverse effect of opium, sequela - Substance-related disorders */
'T400X6A' = '661' /* Underdosing of opium, initial encounter - Substance-related disorders */
'T400X6D' = '661' /* Underdosing of opium, subsequent encounter - Substance-related disorders */
'T400X6S' = '661' /* Underdosing of opium, sequela - Substance-related disorders */
'T401X1A' = '661' /* Poisoning by heroin, accidental (unintentional), init encntr - Substance-related disorders */
'T401X1D' = '661' /* Poisoning by heroin, accidental (unintentional), subs encntr - Substance-related disorders */
'T401X1S' = '661' /* Poisoning by heroin, accidental (unintentional), sequela - Substance-related disorders */
'T401X3A' = '661' /* Poisoning by heroin, assault, initial encounter - Substance-related disorders */
'T401X3D' = '661' /* Poisoning by heroin, assault, subsequent encounter - Substance-related disorders */
'T401X3S' = '661' /* Poisoning by heroin, assault, sequela - Substance-related disorders */
'T401X4A' = '661' /* Poisoning by heroin, undetermined, initial encounter - Substance-related disorders */
'T401X4D' = '661' /* Poisoning by heroin, undetermined, subsequent encounter - Substance-related disorders */
'T401X4S' = '661' /* Poisoning by heroin, undetermined, sequela - Substance-related disorders */
'T401X5A' = '661' /* Adverse effect of heroin initial encounter - Substance-related disorders */
'T401X5D' = '661' /* Adverse effect of heroin subsequent encounter - Substance-related disorders */
'T401X5S' = '661' /* Adverse effect of heroin sequela - Substance-related disorders */
'T405X1A' = '661' /* Poisoning by cocaine, accidental (unintentional), init - Substance-related disorders */
'T405X1D' = '661' /* Poisoning by cocaine, accidental (unintentional), subs - Substance-related disorders */
'T405X1S' = '661' /* Poisoning by cocaine, accidental (unintentional), sequela - Substance-related disorders */
'T405X3A' = '661' /* Poisoning by cocaine, assault, initial encounter - Substance-related disorders */
'T405X3D' = '661' /* Poisoning by cocaine, assault, subsequent encounter - Substance-related disorders */
'T405X3S' = '661' /* Poisoning by cocaine, assault, sequela - Substance-related disorders */
'T405X4A' = '661' /* Poisoning by cocaine, undetermined, initial encounter - Substance-related disorders */
'T405X4D' = '661' /* Poisoning by cocaine, undetermined, subsequent encounter - Substance-related disorders */
'T405X4S' = '661' /* Poisoning by cocaine, undetermined, sequela - Substance-related disorders */
'T405X5A' = '661' /* Adverse effect of cocaine, initial encounter - Substance-related disorders */
'T405X5D' = '661' /* Adverse effect of cocaine, subsequent encounter - Substance-related disorders */
'T405X5S' = '661' /* Adverse effect of cocaine, sequela - Substance-related disorders */
'T405X6A' = '661' /* Underdosing of cocaine, initial encounter - Substance-related disorders */
'T405X6D' = '661' /* Underdosing of cocaine, subsequent encounter - Substance-related disorders */
'T405X6S' = '661' /* Underdosing of cocaine, sequela - Substance-related disorders */
'T407X1A' = '661' /* Poisoning by cannabis (derivatives), accidental, init - Substance-related disorders */
'T407X1D' = '661' /* Poisoning by cannabis (derivatives), accidental, subs - Substance-related disorders */
'T407X1S' = '661' /* Poisoning by cannabis (derivatives), accidental, sequela - Substance-related disorders */
'T407X3A' = '661' /* Poisoning by cannabis (derivatives), assault, init encntr - Substance-related disorders */
'T407X3D' = '661' /* Poisoning by cannabis (derivatives), assault, subs encntr - Substance-related disorders */
'T407X3S' = '661' /* Poisoning by cannabis (derivatives), assault, sequela - Substance-related disorders */
'T407X4A' = '661' /* Poisoning by cannabis (derivatives), undetermined, init - Substance-related disorders */
'T407X4D' = '661' /* Poisoning by cannabis (derivatives), undetermined, subs - Substance-related disorders */
'T407X4S' = '661' /* Poisoning by cannabis (derivatives), undetermined, sequela - Substance-related disorders */
'T407X5A' = '661' /* Adverse effect of cannabis (derivatives), initial encounter - Substance-related disorders */
'T407X5D' = '661' /* Adverse effect of cannabis (derivatives), subs encntr - Substance-related disorders */
'T407X5S' = '661' /* Adverse effect of cannabis (derivatives), sequela - Substance-related disorders */
'T407X6A' = '661' /* Underdosing of cannabis (derivatives), initial encounter - Substance-related disorders */
'T407X6D' = '661' /* Underdosing of cannabis (derivatives), subsequent encounter - Substance-related disorders */
'T407X6S' = '661' /* Underdosing of cannabis (derivatives), sequela - Substance-related disorders */
'T408X1A' = '661' /* Poisoning by lysergide, accidental (unintentional), init - Substance-related disorders */
'T408X1D' = '661' /* Poisoning by lysergide, accidental (unintentional), subs - Substance-related disorders */
'T408X1S' = '661' /* Poisoning by lysergide, accidental (unintentional), sequela - Substance-related disorders */
'T408X3A' = '661' /* Poisoning by lysergide [LSD], assault, initial encounter - Substance-related disorders */
'T408X3D' = '661' /* Poisoning by lysergide [LSD], assault, subsequent encounter - Substance-related disorders */
'T408X3S' = '661' /* Poisoning by lysergide [LSD], assault, sequela - Substance-related disorders */
'T408X4A' = '661' /* Poisoning by lysergide, undetermined, initial encounter - Substance-related disorders */
'T408X4D' = '661' /* Poisoning by lysergide, undetermined, subsequent encounter - Substance-related disorders */
'T408X4S' = '661' /* Poisoning by lysergide [LSD], undetermined, sequela - Substance-related disorders */
'T408X5A' = '661' /* Adverse effect of lysergide [LSD] initial encounter - Substance-related disorders */
'T408X5D' = '661' /* Adverse effect of lysergide [LSD] subsequent encounter - Substance-related disorders */
'T408X5S' = '661' /* Adverse effect of lysergide [LSD] sequela - Substance-related disorders */
'T40901A' = '661' /* Poisoning by unsp psychodyslept, accidental, init - Substance-related disorders */
'T40901D' = '661' /* Poisoning by unsp psychodyslept, accidental, subs - Substance-related disorders */
'T40901S' = '661' /* Poisoning by unsp psychodyslept, accidental, sequela - Substance-related disorders */
'T40903A' = '661' /* Poisoning by unsp psychodysleptics, assault, init encntr - Substance-related disorders */
'T40903D' = '661' /* Poisoning by unsp psychodysleptics, assault, subs encntr - Substance-related disorders */
'T40903S' = '661' /* Poisoning by unspecified psychodysleptics, assault, sequela - Substance-related disorders */
'T40904A' = '661' /* Poisoning by unsp psychodysleptics, undetermined, init - Substance-related disorders */
'T40904D' = '661' /* Poisoning by unsp psychodysleptics, undetermined, subs - Substance-related disorders */
'T40904S' = '661' /* Poisoning by unsp psychodysleptics, undetermined, sequela - Substance-related disorders */
'T40905A' = '661' /* Adverse effect of unspecified psychodysleptics, init encntr - Substance-related disorders */
'T40905D' = '661' /* Adverse effect of unspecified psychodysleptics, subs encntr - Substance-related disorders */
'T40905S' = '661' /* Adverse effect of unspecified psychodysleptics, sequela - Substance-related disorders */
'T40906A' = '661' /* Underdosing of unspecified psychodysleptics, init encntr - Substance-related disorders */
'T40906D' = '661' /* Underdosing of unspecified psychodysleptics, subs encntr - Substance-related disorders */
'T40906S' = '661' /* Underdosing of unspecified psychodysleptics, sequela - Substance-related disorders */
'T40991A' = '661' /* Poisoning by oth psychodyslept, accidental, init - Substance-related disorders */
'T40991D' = '661' /* Poisoning by oth psychodyslept, accidental, subs - Substance-related disorders */
'T40991S' = '661' /* Poisoning by oth psychodyslept, accidental, sequela - Substance-related disorders */
'T40993A' = '661' /* Poisoning by other psychodysleptics, assault, init encntr - Substance-related disorders */
'T40993D' = '661' /* Poisoning by other psychodysleptics, assault, subs encntr - Substance-related disorders */
'T40993S' = '661' /* Poisoning by other psychodysleptics, assault, sequela - Substance-related disorders */
'T40994A' = '661' /* Poisoning by oth psychodysleptics, undetermined, init encntr - Substance-related disorders */
'T40994D' = '661' /* Poisoning by oth psychodysleptics, undetermined, subs encntr - Substance-related disorders */
'T40994S' = '661' /* Poisoning by other psychodysleptics, undetermined, sequela - Substance-related disorders */
'T40995A' = '661' /* Adverse effect of other psychodysleptics, initial encounter - Substance-related disorders */
'T40995D' = '661' /* Adverse effect of other psychodysleptics, subs encntr - Substance-related disorders */
'T40995S' = '661' /* Adverse effect of other psychodysleptics, sequela - Substance-related disorders */
'T40996A' = '661' /* Underdosing of other psychodysleptics, initial encounter - Substance-related disorders */
'T40996D' = '661' /* Underdosing of other psychodysleptics, subsequent encounter - Substance-related disorders */
'T40996S' = '661' /* Underdosing of other psychodysleptics, sequela - Substance-related disorders */
OTHER   = 0;










/*============================================================================;
*  TITLE:
*          FORMAT LIBRARY FOR PEDIATRIC QUALITY INDICATORS
*
*  DESCRIPTION:
*          COVARIATE FORMAT LIB FOR AHRQ PEDIATRIC QUALITY INDICATORS;
*
*          >>>  VERSION 4.5, APRIL 2013 <<<
*
*============================================================================;*/

 /* -------------------------------------------------------------- */
 /* ---   FORMATS FOR SEX STRATIFIERS                          --- */
 /* ---   ONE FORMAT PER QI.  NUMBER REFERS TO N VARIABLE IN   --- */
 /* ---   MODULE 3. ASSIGNS ARRAY INDEX FOR SEX CATEGORIES     --- */
 /* -------------------------------------------------------------- */

Value S1PDFMT
1 = 0
2 = 0
other = 0 
; 

Value S2PDFMT
1 = 0
2 = 0
other = 0 
; 

Value S5PDFMT
1 = 0
2 = 0
other = 0 
; 

Value S6PDFMT
1 = 0
2 = 0
other = 0 
; 

Value S8PDFMT
1 = 0
2 = 0
other = 0 
; 

Value S9PDFMT
1 = 0
2 = 0
other = 0 
; 

Value S10PDFMT
1 = 0
2 = 0
other = 0 
; 

Value S11PDFMT
1 = 0
2 = 0
other = 0 
; 

Value S12PDFMT
1 = 0
2 = 0
other = 0 
; 

Value S70PDFMT
1 = 0
2 = 0
other = 0 
; 

Value S71PDFMT
1 = 0
2 = 1
other = 0 
; 

Value S72PDFMT
1 = 0
2 = 0
other = 0 
; 

 /* -------------------------------------------------------------- */
 /* ---   FORMATS FOR AGE (YEARS) STRATIFIERS                  --- */
 /* ---   ONE FORMAT PER QI.  NUMBER REFERS TO N VARIABLE IN   --- */
 /* ---   MODULE 3. ASSIGNS ARRAY INDEX FOR AGE CATEGORIES     --- */
 /* -------------------------------------------------------------- */

Value P1PDFMT
2 = 0
3 = 0
4 = 0
5 = 0
other = 0 
; 

Value P2PDFMT
2 = 0
3 = 0
4 = 2
5 = 1
other = 0 
; 

Value P5PDFMT
2 = 2
3 = 2
4 = 2
5 = 1
other = 0 
; 

Value P6PDFMT
2 = 0
3 = 0
4 = 0
5 = 0
other = 0 
; 

Value P8PDFMT
2 = 0
3 = 0
4 = 0
5 = 0
other = 0 
; 

Value P9PDFMT
2 = 4
3 = 3
4 = 2
5 = 1
other = 0 
; 

Value P10PDFMT
2 = 0
3 = 3
4 = 2
5 = 1
other = 0 
; 

Value P11PDFMT
2 = 0
3 = 0
4 = 0
5 = 0
other = 0 
; 

Value P12PDFMT
2 = 1
3 = 2
4 = 3
5 = 4
other = 0 
; 

Value P70PDFMT
2 = 0
3 = 0
4 = 0
5 = 0
other = 0 
; 

Value P71PDFMT
2 = 0
3 = 0
4 = 0
5 = 0
other = 0 
; 

Value P72PDFMT
2 = 0
3 = 0
4 = 0
5 = 0
other = 0 
; 

 /* -------------------------------------------------------------- */
 /* ---   FORMATS FOR AGE (DAYS) STRATIFIERS                   --- */
 /* ---   ONE FORMAT PER QI.  NUMBER REFERS TO N VARIABLE IN   --- */
 /* ---   MODULE 3. ASSIGNS ARRAY INDEX FOR AGE CATEGORIES     --- */
 /* -------------------------------------------------------------- */

Value A1PDFMT
2 = 0
3 = 0
4 = 0
other = 0 
; 

Value A2PDFMT
2 = 0
3 = 0
4 = 0
other = 0 
; 

Value A5PDFMT
2 = 0
3 = 0
4 = 0
other = 0 
; 

Value A6PDFMT
1 = 3
2 = 2
3 = 2
4 = 1
other = 0 
; 

Value A8PDFMT
2 = 0
3 = 0
4 = 0
other = 0 
; 

Value A9PDFMT
2 = 5
3 = 5
4 = 5
other = 0 
; 

Value A10PDFMT
2 = 0
3 = 0
4 = 0
other = 0 
; 

Value A11PDFMT
2 = 0
3 = 0
4 = 0
other = 0 
; 

Value A12PDFMT
2 = 0
3 = 0
4 = 0
other = 0 
; 

Value A70PDFMT
2 = 0
3 = 0
4 = 0
other = 0 
; 

Value A71PDFMT
2 = 0
3 = 0
4 = 0
other = 0 
; 

Value A72PDFMT
2 = 0
3 = 0
4 = 0
other = 0 
; 

 /* -------------------------------------------------------------- */
 /* ---   FORMATS FOR BIRTHWEIGHT STRATIFIERS                  --- */
 /* ---   ONE FORMAT PER QI.  NUMBER REFERS TO N VARIABLE IN   --- */
 /* ---   MODULE 3. ASSIGNS ARRAY INDEX FOR BWHT CATEGORIES    --- */
 /* -------------------------------------------------------------- */

Value B1PDFMT
0 = 0
1 = 0
2 = 0
3 = 0
4 = 0
5 = 0
6 = 0
7 = 0
8 = 0
9 = 0
other = 0 
; 

Value B2PDFMT
0 = 0
1 = 0
2 = 0
3 = 0
4 = 0
5 = 0
6 = 0
7 = 0
8 = 0
9 = 0
other = 0 
; 

Value B5PDFMT
0 = 0
1 = 0
2 = 0
3 = 0
4 = 0
5 = 0
6 = 0
7 = 0
8 = 0
9 = 0
other = 0 
; 

Value B6PDFMT
0 = 0
1 = 4
2 = 4
3 = 4
4 = 4
5 = 4
6 = 4
7 = 4
8 = 4
9 = 0
other = 0 
; 

Value B8PDFMT
0 = 0
1 = 0
2 = 0
3 = 0
4 = 0
5 = 0
6 = 0
7 = 0
8 = 0
9 = 0
other = 0 
; 

Value B9PDFMT
0 = 0
1 = 0
2 = 0
3 = 0
4 = 0
5 = 0
6 = 0
7 = 0
8 = 0
9 = 0
other = 0 
; 

Value B10PDFMT
0 = 0
1 = 0
2 = 0
3 = 0
4 = 0
5 = 0
6 = 0
7 = 0
8 = 0
9 = 0
other = 0 
; 

Value B11PDFMT
0 = 0
1 = 0
2 = 0
3 = 0
4 = 0
5 = 0
6 = 0
7 = 0
8 = 0
9 = 0
other = 0 
; 

Value B12PDFMT
0 = 0
1 = 7
2 = 7
3 = 6
4 = 5
5 = 0
6 = 0
7 = 0
8 = 0
9 = 0
other = 0 
; 

Value B70PDFMT
0 = 0
1 = 0
2 = 0
3 = 0
4 = 0
5 = 0
6 = 0
7 = 0
8 = 0
9 = 0
other = 0 
; 

Value B71PDFMT
0 = 0
1 = 9
2 = 9
3 = 8
4 = 7
5 = 6
6 = 5
7 = 4
8 = 3
9 = 2
other = 0 
; 

Value B72PDFMT
0 = 0
1 = 3
2 = 3
3 = 2
4 = 1
5 = 1
6 = 1
7 = 1
8 = 1
9 = 1
other = 0 
; 


 /* -------------------------------------------------------------- */
 /* ---   FORMATS FOR DRG STRATIFIERS                          --- */
 /* ---   MODULE 3. ASSIGNS ARRAY INDEX FOR DRG CATEGORIES     --- */
 /* -------------------------------------------------------------- */

 /* CMS-DRG (V24) TO MODIFIED DRG */;

VALUE $DRGFMT
001 = 0103
002 = 0103
003 = 0103
004 = 0104
005 = 0107
006 = 0108
007 = 0108
008 = 0108
009 = 0109
010 = 0110
011 = 0110
012 = 0111
013 = 0112
014 = 0114
015 = 0115
016 = 0117
017 = 0117
018 = 0118
019 = 0118
020 = 0198
021 = 0119
022 = 0120
023 = 0121
024 = 0198
025 = 0198
026 = 0128
027 = 0122
028 = 0123
029 = 0123
030 = 0123
031 = 0124
032 = 0124
033 = 0124
034 = 0125
035 = 0125
036 = 0203
037 = 0201
038 = 0203
039 = 0203
040 = 0202
041 = 0202
042 = 0203
043 = 0206
044 = 0204
045 = 0205
046 = 0206
047 = 0206
048 = 0206
049 = 0301
050 = 0305
051 = 0305
052 = 0302
053 = 0303
054 = 0303
055 = 0302
056 = 0302
057 = 0302
058 = 0302
059 = 0302
060 = 0302
061 = 0302
062 = 0302
063 = 0302
064 = 0306
065 = 0307
066 = 0308
067 = 0309
068 = 0309
069 = 0309
070 = 0309
071 = 0309
072 = 0310
073 = 0310
074 = 0310
075 = 0401
076 = 0402
077 = 0402
078 = 0403
079 = 0404
080 = 0404
081 = 0404
082 = 0405
083 = 0406
084 = 0406
085 = 0407
086 = 0407
087 = 0408
088 = 0409
089 = 0410
090 = 0410
091 = 0410
092 = 0411
093 = 0411
094 = 0412
095 = 0412
096 = 0413
097 = 0413
098 = 0413
099 = 0414
100 = 0414
101 = 0415
102 = 0415
103 = 7701
104 = 0503
105 = 0503
106 = 0506
107 = 0507
108 = 0505
109 = 0507
110 = 0508
111 = 0508
112 = 0599
113 = 0509
114 = 0515
115 = 0510
116 = 0510
117 = 0517
118 = 0516
119 = 0518
120 = 0519
121 = 0520
122 = 0520
123 = 0521
124 = 0522
125 = 0522
126 = 0523
127 = 0524
128 = 0525
129 = 0526
130 = 0527
131 = 0527
132 = 0528
133 = 0528
134 = 0529
135 = 0530
136 = 0530
137 = 0530
138 = 0531
139 = 0531
140 = 0532
141 = 0533
142 = 0533
143 = 0534
144 = 0535
145 = 0535
146 = 0603
147 = 0603
148 = 0602
149 = 0602
150 = 0604
151 = 0604
152 = 0607
153 = 0607
154 = 0601
155 = 0601
156 = 0601
157 = 0608
158 = 0608
159 = 0610
160 = 0610
161 = 0609
162 = 0609
163 = 0609
164 = 0605
165 = 0605
166 = 0606
167 = 0606
168 = 0304
169 = 0304
170 = 0611
171 = 0611
172 = 0614
173 = 0614
174 = 0615
175 = 0615
176 = 0616
177 = 0617
178 = 0617
179 = 0618
180 = 0619
181 = 0619
182 = 0620
183 = 0620
184 = 0620
185 = 0311
186 = 0311
187 = 0311
188 = 0621
189 = 0621
190 = 0621
191 = 0701
192 = 0701
193 = 0702
194 = 0702
195 = 0703
196 = 0703
197 = 0704
198 = 0704
199 = 0706
200 = 0706
201 = 0707
202 = 0708
203 = 0709
204 = 0710
205 = 0711
206 = 0711
207 = 0712
208 = 0712
209 = 0899
210 = 0811
211 = 0811
212 = 0811
213 = 0809
214 = 0899
215 = 0899
216 = 0810
217 = 0805
218 = 0816
219 = 0816
220 = 0816
221 = 0899
222 = 0899
223 = 0822
224 = 0824
225 = 0820
226 = 0819
227 = 0819
228 = 0821
229 = 0825
230 = 0818
231 = 0899
232 = 0823
233 = 0826
234 = 0826
235 = 0827
236 = 0828
237 = 0829
238 = 0830
239 = 0831
240 = 0832
241 = 0832
242 = 0833
243 = 0834
244 = 0835
245 = 0835
246 = 0835
247 = 0836
248 = 0837
249 = 0838
250 = 0839
251 = 0839
252 = 0839
253 = 0839
254 = 0839
255 = 0839
256 = 0840
257 = 0904
258 = 0904
259 = 0905
260 = 0905
261 = 0905
262 = 0905
263 = 0901
264 = 0901
265 = 0902
266 = 0902
267 = 0903
268 = 0903
269 = 0903
270 = 0903
271 = 0906
272 = 0907
273 = 0907
274 = 0908
275 = 0908
276 = 0909
277 = 0910
278 = 0910
279 = 0910
280 = 0911
281 = 0911
282 = 0911
283 = 0912
284 = 0912
285 = 1002
286 = 1001
287 = 1004
288 = 1003
289 = 1005
290 = 1005
291 = 1005
292 = 1006
293 = 1006
294 = 1007
295 = 1007
296 = 1008
297 = 1008
298 = 1008
299 = 1009
300 = 1010
301 = 1010
302 = 1101
303 = 1103
304 = 1104
305 = 1104
306 = 1106
307 = 1106
308 = 1105
309 = 1105
310 = 1107
311 = 1107
312 = 1108
313 = 1108
314 = 1108
315 = 1109
316 = 1110
317 = 1111
318 = 1112
319 = 1112
320 = 1113
321 = 1113
322 = 1113
323 = 1114
324 = 1115
325 = 1116
326 = 1116
327 = 1116
328 = 1117
329 = 1117
330 = 1117
331 = 1118
332 = 1118
333 = 1118
334 = 1201
335 = 1201
336 = 1204
337 = 1204
338 = 1203
339 = 1203
340 = 1203
341 = 1202
342 = 1206
343 = 1206
344 = 1205
345 = 1206
346 = 1207
347 = 1207
348 = 1208
349 = 1208
350 = 1209
351 = 1210
352 = 1210
353 = 1301
354 = 1303
355 = 1303
356 = 1307
357 = 1302
358 = 1304
359 = 1304
360 = 1306
361 = 1305
362 = 1305
363 = 1305
364 = 1305
365 = 1308
366 = 1309
367 = 1309
368 = 1310
369 = 1311
370 = 1401
371 = 1401
372 = 1405
373 = 1405
374 = 1402
375 = 1402
376 = 1406
377 = 1403
378 = 1407
379 = 1408
380 = 1409
381 = 1404
382 = 1410
383 = 1411
384 = 1412
385 = 1501
386 = 1502
387 = 1503
388 = 1504
389 = 1505
390 = 1506
391 = 1507
392 = 1601
393 = 1601
394 = 1602
395 = 1604
396 = 1604
397 = 1605
398 = 1606
399 = 1606
400 = 1799
401 = 1708
402 = 1708
403 = 1713
404 = 1713
405 = 1711
406 = 1709
407 = 1709
408 = 1710
409 = 1716
410 = 1715
411 = 1714
412 = 1714
413 = 1714
414 = 1714
415 = 1899
416 = 1898
417 = 1807
418 = 1803
419 = 1804
420 = 1804
421 = 1805
422 = 1805
423 = 1806
424 = 1909
425 = 1910
426 = 1911
427 = 1912
428 = 1913
429 = 1914
430 = 1915
431 = 1916
432 = 1917
433 = 2018
434 = 2098
435 = 2098
436 = 2098
437 = 2098
438 = 2098
439 = 2102
440 = 2101
441 = 2103
442 = 2104
443 = 2104
444 = 2105
445 = 2105
446 = 2105
447 = 2106
448 = 2106
449 = 2107
450 = 2107
451 = 2107
452 = 2108
453 = 2108
454 = 2109
455 = 2109
456 = 2298
457 = 2298
458 = 2298
459 = 2298
460 = 2298
461 = 2301
462 = 2302
463 = 2303
464 = 2303
465 = 2304
466 = 2304
467 = 2305
468 = 8899
469 = 8898
470 = 8898
471 = 0804
472 = 2299
473 = 1711
474 = 0198
475 = 0416
476 = 8899
477 = 8899
478 = 0514
479 = 0514
480 = 7702
481 = 7705
482 = 7799
483 = 7799
484 = 2406
485 = 2407
486 = 2408
487 = 2409
488 = 2501
489 = 2502
490 = 2503
491 = 0812
492 = 1712
493 = 0705
494 = 0705
495 = 7703
496 = 0801
497 = 0803
498 = 0803
499 = 0815
500 = 0815
501 = 0813
502 = 0813
503 = 0814
504 = 2210
505 = 2212
506 = 2201
507 = 2201
508 = 2213
509 = 2213
510 = 2214
511 = 2214
512 = 7704
513 = 7706
514 = 0599
515 = 0504
516 = 0502
517 = 0502
518 = 0513
519 = 0808
520 = 0808
521 = 2019
522 = 2019
523 = 2020
524 = 0116
525 = 0501
526 = 0511
527 = 0511
528 = 0101
529 = 0105
530 = 0105
531 = 0104
532 = 0104
533 = 0107
534 = 0107
535 = 0504
536 = 0504
537 = 0817
538 = 0817
539 = 1707
540 = 1707
541 = 7799
542 = 7799
543 = 0102
544 = 0807
545 = 0806
546 = 0802
547 = 0507
548 = 0507
549 = 0507
550 = 0507
551 = 0510
552 = 0510
553 = 0514
554 = 0514
555 = 0502
556 = 0502
557 = 0511
558 = 0511
559 = 0113
560 = 0126
561 = 0127
562 = 0128
563 = 0128
564 = 0129
565 = 0416
566 = 0416
567 = 0601
568 = 0601
569 = 0602
570 = 0602
571 = 0612
572 = 0613
573 = 1102
574 = 1603
575 = 1807
576 = 1808
577 = 0106
578 = 1801
579 = 1802
OTHER = 8898
;

 /* MS-DRG (V25) TO MODIFIED DRG */;

VALUE $DRGF2T
001 = 7701
002 = 7701
003 = 7799
004 = 7799
005 = 7702
006 = 7702
007 = 7703
008 = 7704
009 = 7705
010 = 7706
011 = 7799
012 = 7799
013 = 7799
014 = 7705
015 = 7705
016 = 7705
017 = 7705
020 = 0101
021 = 0101
022 = 0101
023 = 0102
024 = 0102
025 = 0103
026 = 0103
027 = 0103
028 = 0104
029 = 0104
030 = 0104
031 = 0105
032 = 0105
033 = 0105
034 = 0106
035 = 0106
036 = 0106
037 = 0107
038 = 0107
039 = 0107
040 = 0108
041 = 0108
042 = 0108
052 = 0109
053 = 0109
054 = 0110
055 = 0110
056 = 0111
057 = 0111
058 = 0112
059 = 0112
060 = 0112
061 = 0113
062 = 0113
063 = 0113
064 = 0114
065 = 0114
066 = 0114
067 = 0115
068 = 0115
069 = 0116
070 = 0117
071 = 0117
072 = 0117
073 = 0118
074 = 0118
075 = 0119
076 = 0119
077 = 0120
078 = 0120
079 = 0120
080 = 0121
081 = 0121
082 = 0122
083 = 0122
084 = 0122
085 = 0123
086 = 0123
087 = 0123
088 = 0124
089 = 0124
090 = 0124
091 = 0125
092 = 0125
093 = 0125
094 = 0126
095 = 0126
096 = 0126
097 = 0127
098 = 0127
099 = 0127
100 = 0128
101 = 0128
102 = 0129
103 = 0129
113 = 0201
114 = 0201
115 = 0202
116 = 0203
117 = 0203
121 = 0204
122 = 0204
123 = 0205
124 = 0206
125 = 0206
129 = 0301
130 = 0301
131 = 0301
132 = 0301
133 = 0302
134 = 0302
135 = 0303
136 = 0303
137 = 0304
138 = 0304
139 = 0305
146 = 0306
147 = 0306
148 = 0306
149 = 0307
150 = 0308
151 = 0308
152 = 0309
153 = 0309
154 = 0310
155 = 0310
156 = 0310
157 = 0311
158 = 0311
159 = 0311
163 = 0401
164 = 0401
165 = 0401
166 = 0402
167 = 0402
168 = 0402
175 = 0403
176 = 0403
177 = 0404
178 = 0404
179 = 0404
180 = 0405
181 = 0405
182 = 0405
183 = 0406
184 = 0406
185 = 0406
186 = 0407
187 = 0407
188 = 0407
189 = 0408
190 = 0409
191 = 0409
192 = 0409
193 = 0410
194 = 0410
195 = 0410
196 = 0411
197 = 0411
198 = 0411
199 = 0412
200 = 0412
201 = 0412
202 = 0413
203 = 0413
204 = 0414
205 = 0415
206 = 0415
207 = 0416
208 = 0416
215 = 0501
216 = 0503
217 = 0503
218 = 0503
219 = 0503
220 = 0503
221 = 0503
222 = 0504
223 = 0504
224 = 0504
225 = 0504
226 = 0504
227 = 0504
228 = 0505
229 = 0505
230 = 0505
231 = 0506
232 = 0506
233 = 0507
234 = 0507
235 = 0507
236 = 0507
237 = 0508
238 = 0508
239 = 0509
240 = 0509
241 = 0509
242 = 0510
243 = 0510
244 = 0510
245 = 0510
246 = 0511
247 = 0511
248 = 0502
249 = 0502
250 = 0513
251 = 0513
252 = 0514
253 = 0514
254 = 0514
255 = 0515
256 = 0515
257 = 0515
258 = 0516
259 = 0516
260 = 0517
261 = 0517
262 = 0517
263 = 0518
264 = 0519
265 = 0510
266 = 0540
267 = 0540
280 = 0520
281 = 0520
282 = 0520
283 = 0521
284 = 0521
285 = 0521
286 = 0522
287 = 0522
288 = 0523
289 = 0523
290 = 0523
291 = 0524
292 = 0524
293 = 0524
294 = 0525
295 = 0525
296 = 0526
297 = 0526
298 = 0526
299 = 0527
300 = 0527
301 = 0527
302 = 0528
303 = 0528
304 = 0529
305 = 0529
306 = 0530
307 = 0530
308 = 0531
309 = 0531
310 = 0531
311 = 0532
312 = 0533
313 = 0534
314 = 0535
315 = 0535
316 = 0535
326 = 0601
327 = 0601
328 = 0601
329 = 0602
330 = 0602
331 = 0602
332 = 0603
333 = 0603
334 = 0603
335 = 0604
336 = 0604
337 = 0604
338 = 0605
339 = 0605
340 = 0605
341 = 0606
342 = 0606
343 = 0606
344 = 0607
345 = 0607
346 = 0607
347 = 0608
348 = 0608
349 = 0608
350 = 0609
351 = 0609
352 = 0609
353 = 0610
354 = 0610
355 = 0610
356 = 0611
357 = 0611
358 = 0611
368 = 0612
369 = 0612
370 = 0612
371 = 0613
372 = 0613
373 = 0613
374 = 0614
375 = 0614
376 = 0614
377 = 0615
378 = 0615
379 = 0615
380 = 0616
381 = 0616
382 = 0616
383 = 0617
384 = 0617
385 = 0618
386 = 0618
387 = 0618
388 = 0619
389 = 0619
390 = 0619
391 = 0620
392 = 0620
393 = 0621
394 = 0621
395 = 0621
405 = 0701
406 = 0701
407 = 0701
408 = 0702
409 = 0702
410 = 0702
411 = 0703
412 = 0703
413 = 0703
414 = 0704
415 = 0704
416 = 0704
417 = 0705
418 = 0705
419 = 0705
420 = 0706
421 = 0706
422 = 0706
423 = 0707
424 = 0707
425 = 0707
432 = 0708
433 = 0708
434 = 0708
435 = 0709
436 = 0709
437 = 0709
438 = 0710
439 = 0710
440 = 0710
441 = 0711
442 = 0711
443 = 0711
444 = 0712
445 = 0712
446 = 0712
453 = 0801
454 = 0801
455 = 0801
456 = 0802
457 = 0802
458 = 0802
459 = 0803
460 = 0803
461 = 0804
462 = 0804
463 = 0805
464 = 0805
465 = 0805
466 = 0806
467 = 0806
468 = 0806
469 = 0807
470 = 0807
471 = 0808
472 = 0808
473 = 0808
474 = 0809
475 = 0809
476 = 0809
477 = 0810
478 = 0810
479 = 0810
480 = 0811
481 = 0811
482 = 0811
483 = 0812
484 = 0812
485 = 0813
486 = 0813
487 = 0813
488 = 0814
489 = 0814
490 = 0815
491 = 0815
492 = 0816
493 = 0816
494 = 0816
495 = 0817
496 = 0817
497 = 0817
498 = 0818
499 = 0818
500 = 0819
501 = 0819
502 = 0819
503 = 0820
504 = 0820
505 = 0820
506 = 0821
507 = 0822
508 = 0822
509 = 0823
510 = 0824
511 = 0824
512 = 0824
513 = 0825
514 = 0825
515 = 0826
516 = 0826
517 = 0826
518 = 0815
519 = 0815
520 = 0815
533 = 0827
534 = 0827
535 = 0828
536 = 0828
537 = 0829
538 = 0829
539 = 0830
540 = 0830
541 = 0830
542 = 0831
543 = 0831
544 = 0831
545 = 0832
546 = 0832
547 = 0832
548 = 0833
549 = 0833
550 = 0833
551 = 0834
552 = 0834
553 = 0835
554 = 0835
555 = 0836
556 = 0836
557 = 0837
558 = 0837
559 = 0838
560 = 0838
561 = 0838
562 = 0839
563 = 0839
564 = 0840
565 = 0840
566 = 0840
570 = 0901
571 = 0901
572 = 0901
573 = 0901
574 = 0901
575 = 0901
576 = 0902
577 = 0902
578 = 0902
579 = 0903
580 = 0903
581 = 0903
582 = 0904
583 = 0904
584 = 0905
585 = 0905
592 = 0906
593 = 0906
594 = 0906
595 = 0907
596 = 0907
597 = 0908
598 = 0908
599 = 0908
600 = 0909
601 = 0909
602 = 0910
603 = 0910
604 = 0911
605 = 0911
606 = 0912
607 = 0912
614 = 1001
615 = 1001
616 = 1002
617 = 1002
618 = 1002
619 = 1003
620 = 1003
621 = 1003
622 = 1004
623 = 1004
624 = 1004
625 = 1005
626 = 1005
627 = 1005
628 = 1006
629 = 1006
630 = 1006
637 = 1007
638 = 1007
639 = 1007
640 = 1008
641 = 1008
642 = 1009
643 = 1010
644 = 1010
645 = 1010
652 = 1101
653 = 1102
654 = 1102
655 = 1102
656 = 1103
657 = 1103
658 = 1103
659 = 1104
660 = 1104
661 = 1104
662 = 1105
663 = 1105
664 = 1105
665 = 1106
666 = 1106
667 = 1106
668 = 1107
669 = 1107
670 = 1107
671 = 1108
672 = 1108
673 = 1109
674 = 1109
675 = 1109
682 = 1110
683 = 1110
684 = 1110
685 = 1111
686 = 1112
687 = 1112
688 = 1112
689 = 1113
690 = 1113
691 = 1114
692 = 1114
693 = 1115
694 = 1115
695 = 1116
696 = 1116
697 = 1117
698 = 1118
699 = 1118
700 = 1118
707 = 1201
708 = 1201
709 = 1202
710 = 1202
711 = 1203
712 = 1203
713 = 1204
714 = 1204
715 = 1205
716 = 1205
717 = 1206
718 = 1206
722 = 1207
723 = 1207
724 = 1207
725 = 1208
726 = 1208
727 = 1209
728 = 1209
729 = 1210
730 = 1210
734 = 1301
735 = 1301
736 = 1302
737 = 1302
738 = 1302
739 = 1303
740 = 1303
741 = 1303
742 = 1304
743 = 1304
744 = 1305
745 = 1305
746 = 1306
747 = 1306
748 = 1307
749 = 1308
750 = 1308
754 = 1309
755 = 1309
756 = 1309
757 = 1310
758 = 1310
759 = 1310
760 = 1311
761 = 1311
765 = 1401
766 = 1401
767 = 1402
768 = 1402
769 = 1403
770 = 1404
774 = 1405
775 = 1405
776 = 1406
777 = 1407
778 = 1408
779 = 1409
780 = 1410
781 = 1411
782 = 1412
789 = 1501
790 = 1502
791 = 1503
792 = 1504
793 = 1505
794 = 1506
795 = 1507
799 = 1601
800 = 1601
801 = 1601
802 = 1602
803 = 1602
804 = 1602
808 = 1603
809 = 1603
810 = 1603
811 = 1604
812 = 1604
813 = 1605
814 = 1606
815 = 1606
816 = 1606
820 = 1707
821 = 1707
822 = 1707
823 = 1708
824 = 1708
825 = 1708
826 = 1709
827 = 1709
828 = 1709
829 = 1710
830 = 1710
834 = 1711
835 = 1711
836 = 1711
837 = 1712
838 = 1712
839 = 1712
840 = 1713
841 = 1713
842 = 1713
843 = 1714
844 = 1714
845 = 1714
846 = 1715
847 = 1715
848 = 1715
849 = 1716
853 = 1801
854 = 1801
855 = 1801
856 = 1802
857 = 1802
858 = 1802
862 = 1803
863 = 1803
864 = 1804
865 = 1805
866 = 1805
867 = 1806
868 = 1806
869 = 1806
870 = 1807
871 = 1808
872 = 1808
876 = 1909
880 = 1910
881 = 1911
882 = 1912
883 = 1913
884 = 1914
885 = 1915
886 = 1916
887 = 1917
894 = 2018
895 = 2019
896 = 2020
897 = 2020
901 = 2101
902 = 2101
903 = 2101
904 = 2102
905 = 2102
906 = 2103
907 = 2104
908 = 2104
909 = 2104
913 = 2105
914 = 2105
915 = 2106
916 = 2106
917 = 2107
918 = 2107
919 = 2108
920 = 2108
921 = 2108
922 = 2109
923 = 2109
927 = 2210
928 = 2201
929 = 2201
933 = 2212
934 = 2213
935 = 2214
939 = 2301
940 = 2301
941 = 2301
945 = 2302
946 = 2302
947 = 2303
948 = 2303
949 = 2304
950 = 2304
951 = 2305
955 = 2406
956 = 2407
957 = 2408
958 = 2408
959 = 2408
963 = 2409
964 = 2409
965 = 2409
969 = 2501
970 = 2501
974 = 2502
975 = 2502
976 = 2502
977 = 2503
981 = 8899
982 = 8899
983 = 8899
984 = 8899
985 = 8899
986 = 8899
987 = 8899
988 = 8899
989 = 8899
998 = 8898
999 = 8898
OTHER = 8898
;

 /* -------------------------------------------------------------- */
 /* ---   FORMATS FOR DRG STRATIFIERS                         --- */
 /* ---   ONE FORMAT PER QI.  NUMBER REFERS TO N VARIABLE IN   --- */
 /* ---   MODULE 3. ASSIGNS ARRAY INDEX FOR DRG CATEGORIES    --- */
 /* -------------------------------------------------------------- */

Value D1PDFMT
1505 = 0
other = -1 
; 

Value D2PDFMT
other = -1 
; 

Value D5PDFMT
other = -1 
; 

Value D6PDFMT
503 = 0
505 = 0
508 = 0
other = -1 
; 

Value D8PDFMT
other = -1 
; 

Value D9PDFMT
802 = 6
other = -1 
; 

Value D10PDFMT
other = -1 
; 

Value D11PDFMT
other = -1 
; 

Value D12PDFMT
416 = 8
602 = 9
1711 = 10
7705 = 11
other = -1 
; 

Value D70PDFMT
other = -1 
; 

Value D71PDFMT
416 = 10
505 = 11
508 = 12
530 = 0
other = -1 
; 

Value D72PDFMT
1501 = 4
other = -1 
; 

 /* CMS-DRG (V24) TO MDC */;

VALUE MDCFMT
001 = 01
002 = 01
003 = 01
004 = 01
005 = 01
006 = 01
007 = 01
008 = 01
009 = 01
010 = 01
011 = 01
012 = 01
013 = 01
014 = 01
015 = 01
016 = 01
017 = 01
018 = 01
019 = 01
020 = 01
021 = 01
022 = 01
023 = 01
024 = 01
025 = 01
026 = 01
027 = 01
028 = 01
029 = 01
030 = 01
031 = 01
032 = 01
033 = 01
034 = 01
035 = 01
036 = 02
037 = 02
038 = 02
039 = 02
040 = 02
041 = 02
042 = 02
043 = 02
044 = 02
045 = 02
046 = 02
047 = 02
048 = 02
049 = 03
050 = 03
051 = 03
052 = 03
053 = 03
054 = 03
055 = 03
056 = 03
057 = 03
058 = 03
059 = 03
060 = 03
061 = 03
062 = 03
063 = 03
064 = 03
065 = 03
066 = 03
067 = 03
068 = 03
069 = 03
070 = 03
071 = 03
072 = 03
073 = 03
074 = 03
075 = 04
076 = 04
077 = 04
078 = 04
079 = 04
080 = 04
081 = 04
082 = 04
083 = 04
084 = 04
085 = 04
086 = 04
087 = 04
088 = 04
089 = 04
090 = 04
091 = 04
092 = 04
093 = 04
094 = 04
095 = 04
096 = 04
097 = 04
098 = 04
099 = 04
100 = 04
101 = 04
102 = 04
103 = 77
104 = 05
105 = 05
106 = 05
107 = 05
108 = 05
109 = 05
110 = 05
111 = 05
112 = 05
113 = 05
114 = 05
115 = 05
116 = 05
117 = 05
118 = 05
119 = 05
120 = 05
121 = 05
122 = 05
123 = 05
124 = 05
125 = 05
126 = 05
127 = 05
128 = 05
129 = 05
130 = 05
131 = 05
132 = 05
133 = 05
134 = 05
135 = 05
136 = 05
137 = 05
138 = 05
139 = 05
140 = 05
141 = 05
142 = 05
143 = 05
144 = 05
145 = 05
146 = 06
147 = 06
148 = 06
149 = 06
150 = 06
151 = 06
152 = 06
153 = 06
154 = 06
155 = 06
156 = 06
157 = 06
158 = 06
159 = 06
160 = 06
161 = 06
162 = 06
163 = 06
164 = 06
165 = 06
166 = 06
167 = 06
168 = 03
169 = 03
170 = 06
171 = 06
172 = 06
173 = 06
174 = 06
175 = 06
176 = 06
177 = 06
178 = 06
179 = 06
180 = 06
181 = 06
182 = 06
183 = 06
184 = 06
185 = 03
186 = 03
187 = 03
188 = 06
189 = 06
190 = 06
191 = 07
192 = 07
193 = 07
194 = 07
195 = 07
196 = 07
197 = 07
198 = 07
199 = 07
200 = 07
201 = 07
202 = 07
203 = 07
204 = 07
205 = 07
206 = 07
207 = 07
208 = 07
209 = 08
210 = 08
211 = 08
212 = 08
213 = 08
214 = 08
215 = 08
216 = 08
217 = 08
218 = 08
219 = 08
220 = 08
221 = 08
222 = 08
223 = 08
224 = 08
225 = 08
226 = 08
227 = 08
228 = 08
229 = 08
230 = 08
231 = 08
232 = 08
233 = 08
234 = 08
235 = 08
236 = 08
237 = 08
238 = 08
239 = 08
240 = 08
241 = 08
242 = 08
243 = 08
244 = 08
245 = 08
246 = 08
247 = 08
248 = 08
249 = 08
250 = 08
251 = 08
252 = 08
253 = 08
254 = 08
255 = 08
256 = 08
257 = 09
258 = 09
259 = 09
260 = 09
261 = 09
262 = 09
263 = 09
264 = 09
265 = 09
266 = 09
267 = 09
268 = 09
269 = 09
270 = 09
271 = 09
272 = 09
273 = 09
274 = 09
275 = 09
276 = 09
277 = 09
278 = 09
279 = 09
280 = 09
281 = 09
282 = 09
283 = 09
284 = 09
285 = 10
286 = 10
287 = 10
288 = 10
289 = 10
290 = 10
291 = 10
292 = 10
293 = 10
294 = 10
295 = 10
296 = 10
297 = 10
298 = 10
299 = 10
300 = 10
301 = 10
302 = 11
303 = 11
304 = 11
305 = 11
306 = 11
307 = 11
308 = 11
309 = 11
310 = 11
311 = 11
312 = 11
313 = 11
314 = 11
315 = 11
316 = 11
317 = 11
318 = 11
319 = 11
320 = 11
321 = 11
322 = 11
323 = 11
324 = 11
325 = 11
326 = 11
327 = 11
328 = 11
329 = 11
330 = 11
331 = 11
332 = 11
333 = 11
334 = 12
335 = 12
336 = 12
337 = 12
338 = 12
339 = 12
340 = 12
341 = 12
342 = 12
343 = 12
344 = 12
345 = 12
346 = 12
347 = 12
348 = 12
349 = 12
350 = 12
351 = 12
352 = 12
353 = 13
354 = 13
355 = 13
356 = 13
357 = 13
358 = 13
359 = 13
360 = 13
361 = 13
362 = 13
363 = 13
364 = 13
365 = 13
366 = 13
367 = 13
368 = 13
369 = 13
370 = 14
371 = 14
372 = 14
373 = 14
374 = 14
375 = 14
376 = 14
377 = 14
378 = 14
379 = 14
380 = 14
381 = 14
382 = 14
383 = 14
384 = 14
385 = 15
386 = 15
387 = 15
388 = 15
389 = 15
390 = 15
391 = 15
392 = 16
393 = 16
394 = 16
395 = 16
396 = 16
397 = 16
398 = 16
399 = 16
400 = 17
401 = 17
402 = 17
403 = 17
404 = 17
405 = 17
406 = 17
407 = 17
408 = 17
409 = 17
410 = 17
411 = 17
412 = 17
413 = 17
414 = 17
415 = 18
416 = 18
417 = 18
418 = 18
419 = 18
420 = 18
421 = 18
422 = 18
423 = 18
424 = 19
425 = 19
426 = 19
427 = 19
428 = 19
429 = 19
430 = 19
431 = 19
432 = 19
433 = 20
434 = 20
435 = 20
436 = 20
437 = 20
438 = 20
439 = 21
440 = 21
441 = 21
442 = 21
443 = 21
444 = 21
445 = 21
446 = 21
447 = 21
448 = 21
449 = 21
450 = 21
451 = 21
452 = 21
453 = 21
454 = 21
455 = 21
456 = 22
457 = 22
458 = 22
459 = 22
460 = 22
461 = 23
462 = 23
463 = 23
464 = 23
465 = 23
466 = 23
467 = 23
468 = 88
469 = 88
470 = 88
471 = 08
472 = 22
473 = 17
474 = 04
475 = 04
476 = 88
477 = 88
478 = 05
479 = 05
480 = 77
481 = 77
482 = 77
483 = 77
484 = 24
485 = 24
486 = 24
487 = 24
488 = 25
489 = 25
490 = 25
491 = 08
492 = 17
493 = 07
494 = 07
495 = 77
496 = 08
497 = 08
498 = 08
499 = 08
500 = 08
501 = 08
502 = 08
503 = 08
504 = 22
505 = 22
506 = 22
507 = 22
508 = 22
509 = 22
510 = 22
511 = 22
512 = 77
513 = 77
514 = 05
515 = 05
516 = 05
517 = 05
518 = 05
519 = 08
520 = 08
521 = 20
522 = 20
523 = 20
524 = 01
525 = 05
526 = 05
527 = 05
528 = 01
529 = 01
530 = 01
531 = 01
532 = 01
533 = 01
534 = 01
535 = 05
536 = 05
537 = 08
538 = 08
539 = 17
540 = 17
541 = 77
542 = 77
543 = 01
544 = 08
545 = 08
546 = 08
547 = 05
548 = 05
549 = 05
550 = 05
551 = 05
552 = 05
553 = 05
554 = 05
555 = 05
556 = 05
557 = 05
558 = 05
559 = 01
560 = 01
561 = 01
562 = 01
563 = 01
564 = 01
565 = 04
566 = 04
567 = 06
568 = 06
569 = 06
570 = 06
571 = 06
572 = 06
573 = 11
574 = 16
575 = 18
576 = 18
577 = 01
578 = 18
579 = 18
OTHER  = 99 ;

 /* MS-DRG (V25) TO MDC */;

VALUE MDCF2T
001 = 77
002 = 77
003 = 77
004 = 77
005 = 77
006 = 77
007 = 77
008 = 77
009 = 77
010 = 77
011 = 77
012 = 77
013 = 77
014 = 77   
015 = 77  
016 = 77
017 = 77
020 = 01
021 = 01
022 = 01
023 = 01
024 = 01
025 = 01
026 = 01
027 = 01
028 = 01
029 = 01
030 = 01
031 = 01
032 = 01
033 = 01
034 = 01
035 = 01
036 = 01
037 = 01
038 = 01
039 = 01
040 = 01
041 = 01
042 = 01
052 = 01
053 = 01
054 = 01
055 = 01
056 = 01
057 = 01
058 = 01
059 = 01
060 = 01
061 = 01
062 = 01
063 = 01
064 = 01
065 = 01
066 = 01
067 = 01
068 = 01
069 = 01
070 = 01
071 = 01
072 = 01
073 = 01
074 = 01
075 = 01
076 = 01
077 = 01
078 = 01
079 = 01
080 = 01
081 = 01
082 = 01
083 = 01
084 = 01
085 = 01
086 = 01
087 = 01
088 = 01
089 = 01
090 = 01
091 = 01
092 = 01
093 = 01
094 = 01
095 = 01
096 = 01
097 = 01
098 = 01
099 = 01
100 = 01
101 = 01
102 = 01
103 = 01
113 = 02
114 = 02
115 = 02
116 = 02
117 = 02
121 = 02
122 = 02
123 = 02
124 = 02
125 = 02
129 = 03
130 = 03
131 = 03
132 = 03
133 = 03
134 = 03
135 = 03
136 = 03
137 = 03
138 = 03
139 = 03
146 = 03
147 = 03
148 = 03
149 = 03
150 = 03
151 = 03
152 = 03
153 = 03
154 = 03
155 = 03
156 = 03
157 = 03
158 = 03
159 = 03
163 = 04
164 = 04
165 = 04
166 = 04
167 = 04
168 = 04
175 = 04
176 = 04
177 = 04
178 = 04
179 = 04
180 = 04
181 = 04
182 = 04
183 = 04
184 = 04
185 = 04
186 = 04
187 = 04
188 = 04
189 = 04
190 = 04
191 = 04
192 = 04
193 = 04
194 = 04
195 = 04
196 = 04
197 = 04
198 = 04
199 = 04
200 = 04
201 = 04
202 = 04
203 = 04
204 = 04
205 = 04
206 = 04
207 = 04
208 = 04
215 = 05
216 = 05
217 = 05
218 = 05
219 = 05
220 = 05
221 = 05
222 = 05
223 = 05
224 = 05
225 = 05
226 = 05
227 = 05
228 = 05
229 = 05
230 = 05
231 = 05
232 = 05
233 = 05
234 = 05
235 = 05
236 = 05
237 = 05
238 = 05
239 = 05
240 = 05
241 = 05
242 = 05
243 = 05
244 = 05
245 = 05
246 = 05
247 = 05
248 = 05
249 = 05
250 = 05
251 = 05
252 = 05
253 = 05
254 = 05
255 = 05
256 = 05
257 = 05
258 = 05
259 = 05
260 = 05
261 = 05
262 = 05
263 = 05
264 = 05
265 = 05
266 = 05
267 = 05
280 = 05
281 = 05
282 = 05
283 = 05
284 = 05
285 = 05
286 = 05
287 = 05
288 = 05
289 = 05
290 = 05
291 = 05
292 = 05
293 = 05
294 = 05
295 = 05
296 = 05
297 = 05
298 = 05
299 = 05
300 = 05
301 = 05
302 = 05
303 = 05
304 = 05
305 = 05
306 = 05
307 = 05
308 = 05
309 = 05
310 = 05
311 = 05
312 = 05
313 = 05
314 = 05
315 = 05
316 = 05
326 = 06
327 = 06
328 = 06
329 = 06
330 = 06
331 = 06
332 = 06
333 = 06
334 = 06
335 = 06
336 = 06
337 = 06
338 = 06
339 = 06
340 = 06
341 = 06
342 = 06
343 = 06
344 = 06
345 = 06
346 = 06
347 = 06
348 = 06
349 = 06
350 = 06
351 = 06
352 = 06
353 = 06
354 = 06
355 = 06
356 = 06
357 = 06
358 = 06
368 = 06
369 = 06
370 = 06
371 = 06
372 = 06
373 = 06
374 = 06
375 = 06
376 = 06
377 = 06
378 = 06
379 = 06
380 = 06
381 = 06
382 = 06
383 = 06
384 = 06
385 = 06
386 = 06
387 = 06
388 = 06
389 = 06
390 = 06
391 = 06
392 = 06
393 = 06
394 = 06
395 = 06
405 = 07
406 = 07
407 = 07
408 = 07
409 = 07
410 = 07
411 = 07
412 = 07
413 = 07
414 = 07
415 = 07
416 = 07
417 = 07
418 = 07
419 = 07
420 = 07
421 = 07
422 = 07
423 = 07
424 = 07
425 = 07
432 = 07
433 = 07
434 = 07
435 = 07
436 = 07
437 = 07
438 = 07
439 = 07
440 = 07
441 = 07
442 = 07
443 = 07
444 = 07
445 = 07
446 = 07
453 = 08
454 = 08
455 = 08
456 = 08
457 = 08
458 = 08
459 = 08
460 = 08
461 = 08
462 = 08
463 = 08
464 = 08
465 = 08
466 = 08
467 = 08
468 = 08
469 = 08
470 = 08
471 = 08
472 = 08
473 = 08
474 = 08
475 = 08
476 = 08
477 = 08
478 = 08
479 = 08
480 = 08
481 = 08
482 = 08
483 = 08
484 = 08
485 = 08
486 = 08
487 = 08
488 = 08
489 = 08
490 = 08
491 = 08
492 = 08
493 = 08
494 = 08
495 = 08
496 = 08
497 = 08
498 = 08
499 = 08
500 = 08
501 = 08
502 = 08
503 = 08
504 = 08
505 = 08
506 = 08
507 = 08
508 = 08
509 = 08
510 = 08
511 = 08
512 = 08
513 = 08
514 = 08
515 = 08
516 = 08
517 = 08
518 = 08
519 = 08
520 = 08
533 = 08
534 = 08
535 = 08
536 = 08
537 = 08
538 = 08
539 = 08
540 = 08
541 = 08
542 = 08
543 = 08
544 = 08
545 = 08
546 = 08
547 = 08
548 = 08
549 = 08
550 = 08
551 = 08
552 = 08
553 = 08
554 = 08
555 = 08
556 = 08
557 = 08
558 = 08
559 = 08
560 = 08
561 = 08
562 = 08
563 = 08
564 = 08
565 = 08
566 = 08
570 = 09
571 = 09
572 = 09
573 = 09
574 = 09
575 = 09
576 = 09
577 = 09
578 = 09
579 = 09
580 = 09
581 = 09
582 = 09
583 = 09
584 = 09
585 = 09
592 = 09
593 = 09
594 = 09
595 = 09
596 = 09
597 = 09
598 = 09
599 = 09
600 = 09
601 = 09
602 = 09
603 = 09
604 = 09
605 = 09
606 = 09
607 = 09
614 = 10
615 = 10
616 = 10
617 = 10
618 = 10
619 = 10
620 = 10
621 = 10
622 = 10
623 = 10
624 = 10
625 = 10
626 = 10
627 = 10
628 = 10
629 = 10
630 = 10
637 = 10
638 = 10
639 = 10
640 = 10
641 = 10
642 = 10
643 = 10
644 = 10
645 = 10
652 = 11
653 = 11
654 = 11
655 = 11
656 = 11
657 = 11
658 = 11
659 = 11
660 = 11
661 = 11
662 = 11
663 = 11
664 = 11
665 = 11
666 = 11
667 = 11
668 = 11
669 = 11
670 = 11
671 = 11
672 = 11
673 = 11
674 = 11
675 = 11
682 = 11
683 = 11
684 = 11
685 = 11
686 = 11
687 = 11
688 = 11
689 = 11
690 = 11
691 = 11
692 = 11
693 = 11
694 = 11
695 = 11
696 = 11
697 = 11
698 = 11
699 = 11
700 = 11
707 = 12
708 = 12
709 = 12
710 = 12
711 = 12
712 = 12
713 = 12
714 = 12
715 = 12
716 = 12
717 = 12
718 = 12
722 = 12
723 = 12
724 = 12
725 = 12
726 = 12
727 = 12
728 = 12
729 = 12
730 = 12
734 = 13
735 = 13
736 = 13
737 = 13
738 = 13
739 = 13
740 = 13
741 = 13
742 = 13
743 = 13
744 = 13
745 = 13
746 = 13
747 = 13
748 = 13
749 = 13
750 = 13
754 = 13
755 = 13
756 = 13
757 = 13
758 = 13
759 = 13
760 = 13
761 = 13
765 = 14
766 = 14
767 = 14
768 = 14
769 = 14
770 = 14
774 = 14
775 = 14
776 = 14
777 = 14
778 = 14
779 = 14
780 = 14
781 = 14
782 = 14
789 = 15
790 = 15
791 = 15
792 = 15
793 = 15
794 = 15
795 = 15
799 = 16
800 = 16
801 = 16
802 = 16
803 = 16
804 = 16
808 = 16
809 = 16
810 = 16
811 = 16
812 = 16
813 = 16
814 = 16
815 = 16
816 = 16
820 = 17
821 = 17
822 = 17
823 = 17
824 = 17
825 = 17
826 = 17
827 = 17
828 = 17
829 = 17
830 = 17
834 = 17
835 = 17
836 = 17
837 = 17
838 = 17
839 = 17
840 = 17
841 = 17
842 = 17
843 = 17
844 = 17
845 = 17
846 = 17
847 = 17
848 = 17
849 = 17
853 = 18
854 = 18
855 = 18
856 = 18
857 = 18
858 = 18
862 = 18
863 = 18
864 = 18
865 = 18
866 = 18
867 = 18
868 = 18
869 = 18
870 = 18
871 = 18
872 = 18
876 = 19
880 = 19
881 = 19
882 = 19
883 = 19
884 = 19
885 = 19
886 = 19
887 = 19
894 = 20
895 = 20
896 = 20
897 = 20
901 = 21
902 = 21
903 = 21
904 = 21
905 = 21
906 = 21
907 = 21
908 = 21
909 = 21
913 = 21
914 = 21
915 = 21
916 = 21
917 = 21
918 = 21
919 = 21
920 = 21
921 = 21
922 = 21
923 = 21
927 = 22
928 = 22
929 = 22
933 = 22
934 = 22
935 = 22
939 = 23
940 = 23
941 = 23
945 = 23
946 = 23
947 = 23
948 = 23
949 = 23
950 = 23
951 = 23
955 = 24
956 = 24
957 = 24
958 = 24
959 = 24
963 = 24
964 = 24
965 = 24
969 = 25
970 = 25
974 = 25
975 = 25
976 = 25
977 = 25
981 = 88
982 = 88
983 = 88
984 = 88
985 = 88
986 = 88
987 = 88
988 = 88
989 = 88
998 = 88
999 = 88
OTHER  = 99 ;

 /* -------------------------------------------------------------- */
 /* ---   FORMATS FOR MDC STRATIFIERS                         --- */
 /* ---   ONE FORMAT PER QI.  NUMBER REFERS TO N VARIABLE IN   --- */
 /* ---   MODULE 3. ASSIGNS ARRAY INDEX FOR MDC CATEGORIES    --- */
 /* -------------------------------------------------------------- */

Value M1PDFMT
5 = 1
6 = 2
8 = 3
11 = 4
15 = 5
other = 6 
; 

Value M2PDFMT
1 = 3
other = 0 
; 

Value M5PDFMT
other = 0 
; 

Value M6PDFMT
other = 0 
; 

Value M8PDFMT
5 = 1
other = 0 
; 

Value M9PDFMT
1 = 7
other = 0 
; 

Value M10PDFMT
other = 0 
; 

Value M11PDFMT
other = 0 
; 

Value M12PDFMT
5 = 12
7 = 13
10 = 14
15 = 15
16 = 16
other = 0 
; 

Value M70PDFMT
other = 0 
; 

Value M71PDFMT
1 = 13
4 = 14
5 = 15
6 = 16
15 = 17
other = 18 
; 

Value M72PDFMT
other = 0 
; 


 /* --------------------------------------------------------------- */
 /* ---   POVERTY DECILE                                          - */
 /* --------------------------------------------------------------- */

VALUE $POVCAT
"01001" = 3
"01003" = 4
"01005" = 10
"01007" = 9
"01009" = 6
"01011" = 10
"01013" = 10
"01015" = 10
"01017" = 10
"01019" = 9
"01021" = 9
"01023" = 9
"01025" = 10
"01027" = 9
"01029" = 8
"01031" = 9
"01033" = 9
"01035" = 10
"01037" = 8
"01039" = 9
"01041" = 9
"01043" = 9
"01045" = 8
"01047" = 10
"01049" = 9
"01051" = 4
"01053" = 10
"01055" = 8
"01057" = 10
"01059" = 9
"01061" = 9
"01063" = 10
"01065" = 10
"01067" = 8
"01069" = 8
"01071" = 9
"01073" = 8
"01075" = 9
"01077" = 7
"01079" = 8
"01081" = 9
"01083" = 4
"01085" = 10
"01087" = 10
"01089" = 4
"01091" = 10
"01093" = 9
"01095" = 9
"01097" = 9
"01099" = 10
"01101" = 9
"01103" = 4
"01105" = 10
"01107" = 10
"01109" = 10
"01111" = 10
"01113" = 10
"01115" = 4
"01117" = 2
"01119" = 10
"01121" = 10
"01123" = 8
"01125" = 8
"01127" = 10
"01129" = 9
"01131" = 10
"01133" = 9
"02013" = 7
"02016" = 2
"02020" = 2
"02050" = 10
"02060" = 2
"02068" = 1
"02070" = 9
"02090" = 2
"02100" = 3
"02105" = 9
"02110" = 1
"02122" = 2
"02130" = 2
"02150" = 2
"02164" = 8
"02170" = 3
"02180" = 10
"02185" = 3
"02188" = 9
"02195" = 2
"02198" = 7
"02220" = 2
"02230" = 1
"02240" = 4
"02261" = 2
"02270" = 10
"02275" = 4
"02282" = 5
"02290" = 10
"04001" = 10
"04003" = 8
"04005" = 10
"04007" = 9
"04009" = 9
"04011" = 4
"04012" = 10
"04013" = 6
"04015" = 8
"04017" = 10
"04019" = 8
"04021" = 5
"04023" = 10
"04025" = 8
"04027" = 9
"05001" = 8
"05003" = 9
"05005" = 6
"05007" = 2
"05009" = 6
"05011" = 10
"05013" = 6
"05015" = 6
"05017" = 10
"05019" = 10
"05021" = 8
"05023" = 8
"05025" = 6
"05027" = 10
"05029" = 8
"05031" = 9
"05033" = 6
"05035" = 10
"05037" = 9
"05039" = 9
"05041" = 10
"05043" = 9
"05045" = 5
"05047" = 7
"05049" = 8
"05051" = 9
"05053" = 3
"05055" = 7
"05057" = 10
"05059" = 5
"05061" = 9
"05063" = 8
"05065" = 8
"05067" = 10
"05069" = 10
"05071" = 10
"05073" = 10
"05075" = 10
"05077" = 10
"05079" = 10
"05081" = 8
"05083" = 7
"05085" = 4
"05087" = 9
"05089" = 9
"05091" = 9
"05093" = 10
"05095" = 10
"05097" = 10
"05099" = 10
"05101" = 10
"05103" = 9
"05105" = 7
"05107" = 10
"05109" = 9
"05111" = 10
"05113" = 10
"05115" = 8
"05117" = 8
"05119" = 7
"05121" = 9
"05123" = 10
"05125" = 2
"05127" = 10
"05129" = 10
"05131" = 9
"05133" = 10
"05135" = 10
"05137" = 10
"05139" = 9
"05141" = 9
"05143" = 9
"05145" = 6
"05147" = 10
"05149" = 9
"06001" = 4
"06003" = 7
"06005" = 4
"06007" = 9
"06009" = 3
"06011" = 5
"06013" = 2
"06015" = 10
"06017" = 2
"06019" = 10
"06021" = 7
"06023" = 8
"06025" = 10
"06027" = 4
"06029" = 9
"06031" = 10
"06033" = 9
"06035" = 7
"06037" = 7
"06039" = 9
"06041" = 2
"06043" = 5
"06045" = 9
"06047" = 10
"06049" = 10
"06051" = 3
"06053" = 7
"06055" = 2
"06057" = 3
"06059" = 3
"06061" = 2
"06063" = 5
"06065" = 6
"06067" = 6
"06069" = 4
"06071" = 8
"06073" = 5
"06075" = 4
"06077" = 8
"06079" = 5
"06081" = 1
"06083" = 8
"06085" = 2
"06087" = 5
"06089" = 8
"06091" = 4
"06093" = 9
"06095" = 3
"06097" = 4
"06099" = 9
"06101" = 7
"06103" = 9
"06105" = 8
"06107" = 10
"06109" = 5
"06111" = 3
"06113" = 6
"06115" = 9
"08001" = 4
"08003" = 9
"08005" = 3
"08007" = 4
"08009" = 7
"08011" = 10
"08013" = 4
"08014" = 1
"08015" = 4
"08017" = 3
"08019" = 2
"08021" = 9
"08023" = 10
"08025" = 10
"08027" = 5
"08029" = 6
"08031" = 9
"08033" = 4
"08035" = 1
"08037" = 2
"08039" = 1
"08041" = 4
"08043" = 6
"08045" = 2
"08047" = 1
"08049" = 2
"08051" = 5
"08053" = 2
"08055" = 10
"08057" = 5
"08059" = 2
"08061" = 4
"08063" = 4
"08065" = 5
"08067" = 3
"08069" = 4
"08071" = 8
"08073" = 8
"08075" = 6
"08077" = 5
"08079" = 3
"08081" = 2
"08083" = 7
"08085" = 5
"08087" = 5
"08089" = 9
"08091" = 2
"08093" = 2
"08095" = 3
"08097" = 1
"08099" = 10
"08101" = 9
"08103" = 2
"08105" = 7
"08107" = 1
"08109" = 10
"08111" = 6
"08113" = 2
"08115" = 4
"08117" = 2
"08119" = 2
"08121" = 4
"08123" = 5
"08125" = 4
"09001" = 2
"09003" = 3
"09005" = 1
"09007" = 1
"09009" = 3
"09011" = 2
"09013" = 1
"09015" = 3
"10001" = 3
"10003" = 3
"10005" = 4
"11001" = 8
"12001" = 10
"12003" = 7
"12005" = 5
"12007" = 9
"12009" = 4
"12011" = 5
"12013" = 10
"12015" = 4
"12017" = 7
"12019" = 3
"12021" = 6
"12023" = 9
"12027" = 10
"12029" = 10
"12031" = 6
"12033" = 9
"12035" = 6
"12037" = 10
"12039" = 10
"12041" = 9
"12043" = 9
"12045" = 9
"12047" = 10
"12049" = 10
"12051" = 10
"12053" = 5
"12055" = 9
"12057" = 6
"12059" = 10
"12061" = 5
"12063" = 8
"12065" = 8
"12067" = 10
"12069" = 4
"12071" = 7
"12073" = 10
"12075" = 10
"12077" = 10
"12079" = 10
"12081" = 5
"12083" = 9
"12085" = 3
"12086" = 9
"12087" = 4
"12089" = 3
"12091" = 4
"12093" = 10
"12095" = 6
"12097" = 6
"12099" = 5
"12101" = 5
"12103" = 5
"12105" = 7
"12107" = 10
"12109" = 3
"12111" = 7
"12113" = 4
"12115" = 4
"12117" = 3
"12119" = 4
"12121" = 9
"12123" = 9
"12125" = 10
"12127" = 6
"12129" = 4
"12131" = 6
"12133" = 9
"13001" = 10
"13003" = 10
"13005" = 10
"13007" = 10
"13009" = 10
"13011" = 6
"13013" = 3
"13015" = 5
"13017" = 10
"13019" = 9
"13021" = 10
"13023" = 9
"13025" = 10
"13027" = 10
"13029" = 3
"13031" = 10
"13033" = 10
"13035" = 6
"13037" = 10
"13039" = 6
"13043" = 10
"13045" = 8
"13047" = 5
"13049" = 10
"13051" = 9
"13053" = 8
"13055" = 9
"13057" = 2
"13059" = 10
"13061" = 10
"13063" = 10
"13065" = 10
"13067" = 4
"13069" = 10
"13071" = 10
"13073" = 2
"13075" = 10
"13077" = 3
"13079" = 8
"13081" = 10
"13083" = 6
"13085" = 4
"13087" = 10
"13089" = 9
"13091" = 10
"13093" = 10
"13095" = 10
"13097" = 4
"13099" = 10
"13101" = 10
"13103" = 2
"13105" = 10
"13107" = 10
"13109" = 10
"13111" = 8
"13113" = 1
"13115" = 9
"13117" = 1
"13119" = 8
"13121" = 8
"13123" = 6
"13125" = 8
"13127" = 8
"13129" = 8
"13131" = 10
"13133" = 10
"13135" = 4
"13137" = 7
"13139" = 8
"13141" = 10
"13143" = 9
"13145" = 2
"13147" = 10
"13149" = 9
"13151" = 2
"13153" = 4
"13155" = 10
"13157" = 6
"13159" = 8
"13161" = 10
"13163" = 10
"13165" = 10
"13167" = 10
"13169" = 3
"13171" = 8
"13173" = 10
"13175" = 9
"13177" = 2
"13179" = 10
"13181" = 9
"13183" = 9
"13185" = 10
"13187" = 6
"13189" = 9
"13191" = 8
"13193" = 10
"13195" = 7
"13197" = 10
"13199" = 9
"13201" = 10
"13205" = 10
"13207" = 5
"13209" = 10
"13211" = 6
"13213" = 9
"13215" = 9
"13217" = 5
"13219" = 1
"13221" = 6
"13223" = 2
"13225" = 10
"13227" = 4
"13229" = 8
"13231" = 4
"13233" = 10
"13235" = 9
"13237" = 8
"13239" = 10
"13241" = 8
"13243" = 10
"13245" = 10
"13247" = 5
"13249" = 7
"13251" = 10
"13253" = 10
"13255" = 10
"13257" = 10
"13259" = 10
"13261" = 10
"13263" = 9
"13265" = 10
"13267" = 10
"13269" = 10
"13271" = 10
"13273" = 10
"13275" = 10
"13277" = 10
"13279" = 10
"13281" = 5
"13283" = 10
"13285" = 9
"13287" = 10
"13289" = 10
"13291" = 9
"13293" = 9
"13295" = 8
"13297" = 5
"13299" = 10
"13301" = 10
"13303" = 10
"13305" = 10
"13307" = 10
"13309" = 10
"13311" = 7
"13313" = 9
"13315" = 10
"13317" = 10
"13319" = 9
"13321" = 9
"15001" = 8
"15003" = 2
"15005" = 1
"15007" = 3
"15009" = 3
"16001" = 4
"16003" = 6
"16005" = 6
"16007" = 6
"16009" = 7
"16011" = 6
"16013" = 2
"16015" = 5
"16017" = 7
"16019" = 4
"16021" = 7
"16023" = 6
"16025" = 3
"16027" = 9
"16029" = 3
"16031" = 6
"16033" = 8
"16035" = 5
"16037" = 5
"16039" = 6
"16041" = 2
"16043" = 6
"16045" = 7
"16047" = 6
"16049" = 8
"16051" = 3
"16053" = 9
"16055" = 5
"16057" = 7
"16059" = 9
"16061" = 8
"16063" = 4
"16065" = 10
"16067" = 6
"16069" = 4
"16071" = 4
"16073" = 10
"16075" = 7
"16077" = 6
"16079" = 9
"16081" = 3
"16083" = 6
"16085" = 4
"16087" = 7
"17001" = 4
"17003" = 10
"17005" = 4
"17007" = 3
"17009" = 5
"17011" = 3
"17013" = 3
"17015" = 4
"17017" = 4
"17019" = 9
"17021" = 5
"17023" = 3
"17025" = 5
"17027" = 2
"17029" = 8
"17031" = 7
"17033" = 5
"17035" = 3
"17037" = 5
"17039" = 3
"17041" = 3
"17043" = 1
"17045" = 5
"17047" = 3
"17049" = 2
"17051" = 6
"17053" = 3
"17055" = 9
"17057" = 5
"17059" = 8
"17061" = 5
"17063" = 1
"17065" = 5
"17067" = 4
"17069" = 9
"17071" = 3
"17073" = 3
"17075" = 3
"17077" = 10
"17079" = 3
"17081" = 7
"17083" = 2
"17085" = 2
"17087" = 5
"17089" = 3
"17091" = 5
"17093" = 1
"17095" = 6
"17097" = 2
"17099" = 4
"17101" = 7
"17103" = 3
"17105" = 3
"17107" = 3
"17109" = 10
"17111" = 1
"17113" = 3
"17115" = 8
"17117" = 4
"17119" = 5
"17121" = 6
"17123" = 3
"17125" = 5
"17127" = 6
"17129" = 2
"17131" = 3
"17133" = 1
"17135" = 5
"17137" = 5
"17139" = 2
"17141" = 3
"17143" = 5
"17145" = 7
"17147" = 1
"17149" = 7
"17151" = 8
"17153" = 9
"17155" = 1
"17157" = 4
"17159" = 5
"17161" = 4
"17163" = 6
"17165" = 9
"17167" = 4
"17169" = 3
"17171" = 2
"17173" = 3
"17175" = 2
"17177" = 4
"17179" = 2
"17181" = 8
"17183" = 9
"17185" = 5
"17187" = 4
"17189" = 1
"17191" = 5
"17193" = 7
"17195" = 3
"17197" = 2
"17199" = 6
"17201" = 7
"17203" = 1
"18001" = 7
"18003" = 4
"18005" = 4
"18007" = 3
"18009" = 6
"18011" = 1
"18013" = 4
"18015" = 2
"18017" = 6
"18019" = 4
"18021" = 4
"18023" = 4
"18025" = 7
"18027" = 5
"18029" = 2
"18031" = 4
"18033" = 2
"18035" = 10
"18037" = 2
"18039" = 7
"18041" = 7
"18043" = 4
"18045" = 4
"18047" = 3
"18049" = 4
"18051" = 3
"18053" = 9
"18055" = 4
"18057" = 1
"18059" = 2
"18061" = 3
"18063" = 1
"18065" = 6
"18067" = 6
"18069" = 3
"18071" = 4
"18073" = 2
"18075" = 6
"18077" = 6
"18079" = 5
"18081" = 2
"18083" = 7
"18085" = 4
"18087" = 6
"18089" = 8
"18091" = 6
"18093" = 5
"18095" = 8
"18097" = 9
"18099" = 4
"18101" = 4
"18103" = 7
"18105" = 10
"18107" = 5
"18109" = 3
"18111" = 3
"18113" = 4
"18115" = 2
"18117" = 7
"18119" = 4
"18121" = 6
"18123" = 4
"18125" = 5
"18127" = 3
"18129" = 2
"18131" = 5
"18133" = 4
"18135" = 6
"18137" = 2
"18139" = 4
"18141" = 6
"18143" = 8
"18145" = 4
"18147" = 3
"18149" = 8
"18151" = 4
"18153" = 5
"18155" = 6
"18157" = 9
"18159" = 2
"18161" = 4
"18163" = 7
"18165" = 4
"18167" = 9
"18169" = 3
"18171" = 2
"18173" = 1
"18175" = 6
"18177" = 9
"18179" = 2
"18181" = 3
"18183" = 2
"19001" = 2
"19003" = 4
"19005" = 4
"19007" = 6
"19009" = 2
"19011" = 2
"19013" = 8
"19015" = 2
"19017" = 1
"19019" = 3
"19021" = 4
"19023" = 2
"19025" = 3
"19027" = 2
"19029" = 4
"19031" = 1
"19033" = 4
"19035" = 2
"19037" = 2
"19039" = 4
"19041" = 3
"19043" = 3
"19045" = 4
"19047" = 4
"19049" = 1
"19051" = 7
"19053" = 9
"19055" = 2
"19057" = 5
"19059" = 2
"19061" = 2
"19063" = 4
"19065" = 4
"19067" = 5
"19069" = 3
"19071" = 3
"19073" = 3
"19075" = 1
"19077" = 2
"19079" = 3
"19081" = 2
"19083" = 3
"19085" = 2
"19087" = 5
"19089" = 4
"19091" = 2
"19093" = 4
"19095" = 1
"19097" = 4
"19099" = 4
"19101" = 5
"19103" = 7
"19105" = 3
"19107" = 4
"19109" = 2
"19111" = 6
"19113" = 2
"19115" = 3
"19117" = 7
"19119" = 2
"19121" = 2
"19123" = 6
"19125" = 2
"19127" = 4
"19129" = 2
"19131" = 3
"19133" = 3
"19135" = 4
"19137" = 5
"19139" = 4
"19141" = 3
"19143" = 2
"19145" = 5
"19147" = 3
"19149" = 1
"19151" = 3
"19153" = 3
"19155" = 5
"19157" = 3
"19159" = 8
"19161" = 2
"19163" = 5
"19165" = 2
"19167" = 1
"19169" = 8
"19171" = 3
"19173" = 4
"19175" = 5
"19177" = 6
"19179" = 7
"19181" = 1
"19183" = 2
"19185" = 7
"19187" = 6
"19189" = 2
"19191" = 2
"19193" = 5
"19195" = 3
"19197" = 3
"20001" = 8
"20003" = 5
"20005" = 6
"20007" = 4
"20009" = 4
"20011" = 6
"20013" = 8
"20015" = 2
"20017" = 4
"20019" = 6
"20021" = 9
"20023" = 4
"20025" = 3
"20027" = 3
"20029" = 6
"20031" = 2
"20033" = 2
"20035" = 7
"20037" = 9
"20039" = 4
"20041" = 3
"20043" = 4
"20045" = 6
"20047" = 3
"20049" = 8
"20051" = 4
"20053" = 3
"20055" = 5
"20057" = 4
"20059" = 3
"20061" = 3
"20063" = 3
"20065" = 4
"20067" = 3
"20069" = 1
"20071" = 2
"20073" = 7
"20075" = 4
"20077" = 5
"20079" = 2
"20081" = 3
"20083" = 2
"20085" = 3
"20087" = 2
"20089" = 4
"20091" = 1
"20093" = 3
"20095" = 4
"20097" = 4
"20099" = 7
"20101" = 2
"20103" = 2
"20105" = 4
"20107" = 5
"20109" = 2
"20111" = 9
"20113" = 2
"20115" = 3
"20117" = 3
"20119" = 2
"20121" = 2
"20123" = 2
"20125" = 8
"20127" = 3
"20129" = 3
"20131" = 2
"20133" = 6
"20135" = 2
"20137" = 4
"20139" = 3
"20141" = 5
"20143" = 2
"20145" = 4
"20147" = 3
"20149" = 2
"20151" = 3
"20153" = 4
"20155" = 5
"20157" = 3
"20159" = 4
"20161" = 9
"20163" = 5
"20165" = 4
"20167" = 6
"20169" = 7
"20171" = 2
"20173" = 5
"20175" = 6
"20177" = 7
"20179" = 4
"20181" = 7
"20183" = 4
"20185" = 4
"20187" = 3
"20189" = 3
"20191" = 3
"20193" = 2
"20195" = 3
"20197" = 2
"20199" = 4
"20201" = 2
"20203" = 4
"20205" = 6
"20207" = 7
"20209" = 10
"21001" = 10
"21003" = 10
"21005" = 4
"21007" = 5
"21009" = 9
"21011" = 10
"21013" = 10
"21015" = 2
"21017" = 7
"21019" = 9
"21021" = 6
"21023" = 6
"21025" = 10
"21027" = 9
"21029" = 2
"21031" = 9
"21033" = 8
"21035" = 7
"21037" = 4
"21039" = 5
"21041" = 10
"21043" = 10
"21045" = 10
"21047" = 9
"21049" = 7
"21051" = 10
"21053" = 10
"21055" = 9
"21057" = 10
"21059" = 6
"21061" = 8
"21063" = 10
"21065" = 10
"21067" = 9
"21069" = 8
"21071" = 10
"21073" = 6
"21075" = 10
"21077" = 6
"21079" = 7
"21081" = 8
"21083" = 9
"21085" = 9
"21087" = 9
"21089" = 7
"21091" = 5
"21093" = 5
"21095" = 10
"21097" = 6
"21099" = 10
"21101" = 6
"21103" = 7
"21105" = 8
"21107" = 9
"21109" = 10
"21111" = 7
"21113" = 6
"21115" = 10
"21117" = 4
"21119" = 10
"21121" = 10
"21123" = 9
"21125" = 10
"21127" = 9
"21129" = 10
"21131" = 10
"21133" = 10
"21135" = 10
"21137" = 10
"21139" = 5
"21141" = 8
"21143" = 5
"21145" = 6
"21147" = 10
"21149" = 6
"21151" = 9
"21153" = 10
"21155" = 8
"21157" = 3
"21159" = 10
"21161" = 9
"21163" = 5
"21165" = 10
"21167" = 5
"21169" = 9
"21171" = 10
"21173" = 9
"21175" = 10
"21177" = 9
"21179" = 6
"21181" = 8
"21183" = 9
"21185" = 1
"21187" = 6
"21189" = 10
"21191" = 5
"21193" = 10
"21195" = 10
"21197" = 10
"21199" = 9
"21201" = 10
"21203" = 10
"21205" = 10
"21207" = 10
"21209" = 3
"21211" = 3
"21213" = 6
"21215" = 2
"21217" = 10
"21219" = 10
"21221" = 5
"21223" = 5
"21225" = 9
"21227" = 9
"21229" = 6
"21231" = 10
"21233" = 5
"21235" = 10
"21237" = 10
"21239" = 3
"22001" = 9
"22003" = 9
"22005" = 4
"22007" = 6
"22009" = 9
"22011" = 6
"22013" = 10
"22015" = 4
"22017" = 9
"22019" = 7
"22021" = 9
"22023" = 4
"22025" = 10
"22027" = 10
"22029" = 10
"22031" = 9
"22033" = 7
"22035" = 10
"22037" = 8
"22039" = 10
"22041" = 10
"22043" = 9
"22045" = 10
"22047" = 10
"22049" = 9
"22051" = 6
"22053" = 9
"22055" = 8
"22057" = 4
"22059" = 5
"22061" = 10
"22063" = 4
"22065" = 10
"22067" = 10
"22069" = 10
"22071" = 10
"22073" = 9
"22075" = 5
"22077" = 7
"22079" = 9
"22081" = 10
"22083" = 10
"22085" = 10
"22087" = 8
"22089" = 4
"22091" = 10
"22093" = 6
"22095" = 8
"22097" = 10
"22099" = 10
"22101" = 9
"22103" = 2
"22105" = 9
"22107" = 10
"22109" = 8
"22111" = 9
"22113" = 7
"22115" = 4
"22117" = 10
"22119" = 10
"22121" = 5
"22123" = 10
"22125" = 9
"22127" = 10
"23001" = 5
"23003" = 4
"23005" = 2
"23007" = 6
"23009" = 4
"23011" = 3
"23013" = 4
"23015" = 4
"23017" = 5
"23019" = 6
"23021" = 6
"23023" = 2
"23025" = 8
"23027" = 6
"23029" = 9
"23031" = 2
"24001" = 7
"24003" = 1
"24005" = 1
"24009" = 1
"24011" = 4
"24013" = 1
"24015" = 2
"24017" = 1
"24019" = 6
"24021" = 1
"24023" = 5
"24025" = 1
"24027" = 1
"24029" = 5
"24031" = 1
"24033" = 2
"24035" = 1
"24037" = 1
"24039" = 9
"24041" = 2
"24043" = 3
"24045" = 6
"24047" = 2
"24510" = 10
"25001" = 3
"25003" = 4
"25005" = 4
"25007" = 2
"25009" = 2
"25011" = 3
"25013" = 7
"25015" = 4
"25017" = 1
"25019" = 1
"25021" = 1
"25023" = 2
"25025" = 10
"25027" = 3
"26001" = 6
"26003" = 5
"26005" = 5
"26007" = 7
"26009" = 5
"26011" = 8
"26013" = 4
"26015" = 3
"26017" = 6
"26019" = 4
"26021" = 7
"26023" = 9
"26025" = 7
"26027" = 6
"26029" = 4
"26031" = 7
"26033" = 9
"26035" = 10
"26037" = 2
"26039" = 6
"26041" = 5
"26043" = 4
"26045" = 3
"26047" = 3
"26049" = 9
"26051" = 9
"26053" = 9
"26055" = 3
"26057" = 8
"26059" = 7
"26061" = 9
"26063" = 6
"26065" = 9
"26067" = 6
"26069" = 9
"26071" = 5
"26073" = 10
"26075" = 9
"26077" = 9
"26079" = 8
"26081" = 6
"26083" = 4
"26085" = 10
"26087" = 4
"26089" = 2
"26091" = 5
"26093" = 1
"26095" = 9
"26097" = 5
"26099" = 4
"26101" = 7
"26103" = 5
"26105" = 6
"26107" = 10
"26109" = 4
"26111" = 3
"26113" = 6
"26115" = 3
"26117" = 9
"26119" = 9
"26121" = 9
"26123" = 9
"26125" = 2
"26127" = 10
"26129" = 9
"26131" = 6
"26133" = 9
"26135" = 9
"26137" = 4
"26139" = 3
"26141" = 5
"26143" = 10
"26145" = 8
"26147" = 6
"26149" = 6
"26151" = 7
"26153" = 6
"26155" = 5
"26157" = 6
"26159" = 6
"26161" = 4
"26163" = 10
"26165" = 7
"27001" = 4
"27003" = 1
"27005" = 5
"27007" = 9
"27009" = 2
"27011" = 3
"27013" = 7
"27015" = 2
"27017" = 3
"27019" = 1
"27021" = 6
"27023" = 2
"27025" = 1
"27027" = 4
"27029" = 6
"27031" = 2
"27033" = 3
"27035" = 3
"27037" = 1
"27039" = 1
"27041" = 2
"27043" = 3
"27045" = 4
"27047" = 3
"27049" = 2
"27051" = 3
"27053" = 4
"27055" = 2
"27057" = 4
"27059" = 2
"27061" = 3
"27063" = 2
"27065" = 4
"27067" = 5
"27069" = 2
"27071" = 5
"27073" = 2
"27075" = 3
"27077" = 3
"27079" = 2
"27081" = 2
"27083" = 3
"27085" = 2
"27087" = 10
"27089" = 1
"27091" = 3
"27093" = 2
"27095" = 4
"27097" = 4
"27099" = 3
"27101" = 2
"27103" = 2
"27105" = 4
"27107" = 3
"27109" = 2
"27111" = 4
"27113" = 3
"27115" = 5
"27117" = 2
"27119" = 3
"27121" = 2
"27123" = 7
"27125" = 3
"27127" = 2
"27129" = 3
"27131" = 2
"27133" = 3
"27135" = 2
"27137" = 7
"27139" = 1
"27141" = 1
"27143" = 2
"27145" = 3
"27147" = 2
"27149" = 3
"27151" = 2
"27153" = 7
"27155" = 4
"27157" = 1
"27159" = 7
"27161" = 2
"27163" = 1
"27165" = 3
"27167" = 2
"27169" = 6
"27171" = 1
"27173" = 4
"28001" = 10
"28003" = 10
"28005" = 10
"28007" = 10
"28009" = 10
"28011" = 10
"28013" = 10
"28015" = 8
"28017" = 10
"28019" = 10
"28021" = 10
"28023" = 10
"28025" = 10
"28027" = 10
"28029" = 10
"28031" = 9
"28033" = 2
"28035" = 10
"28037" = 9
"28039" = 7
"28041" = 10
"28043" = 9
"28045" = 8
"28047" = 9
"28049" = 10
"28051" = 10
"28053" = 10
"28055" = 10
"28057" = 7
"28059" = 6
"28061" = 9
"28063" = 10
"28065" = 10
"28067" = 10
"28069" = 10
"28071" = 9
"28073" = 6
"28075" = 10
"28077" = 8
"28079" = 10
"28081" = 9
"28083" = 10
"28085" = 8
"28087" = 10
"28089" = 4
"28091" = 10
"28093" = 10
"28095" = 10
"28097" = 10
"28099" = 10
"28101" = 9
"28103" = 10
"28105" = 10
"28107" = 10
"28109" = 9
"28111" = 9
"28113" = 10
"28115" = 6
"28117" = 10
"28119" = 10
"28121" = 4
"28123" = 9
"28125" = 10
"28127" = 10
"28129" = 9
"28131" = 9
"28133" = 10
"28135" = 10
"28137" = 8
"28139" = 9
"28141" = 8
"28143" = 10
"28145" = 9
"28147" = 10
"28149" = 10
"28151" = 10
"28153" = 10
"28155" = 10
"28157" = 10
"28159" = 10
"28161" = 10
"28163" = 10
"29001" = 10
"29003" = 2
"29005" = 3
"29007" = 8
"29009" = 8
"29011" = 7
"29013" = 6
"29015" = 9
"29017" = 9
"29019" = 8
"29021" = 6
"29023" = 9
"29025" = 4
"29027" = 4
"29029" = 5
"29031" = 5
"29033" = 6
"29035" = 10
"29037" = 2
"29039" = 9
"29041" = 4
"29043" = 3
"29045" = 6
"29047" = 2
"29049" = 3
"29051" = 3
"29053" = 6
"29055" = 9
"29057" = 9
"29059" = 9
"29061" = 7
"29063" = 6
"29065" = 9
"29067" = 10
"29069" = 10
"29071" = 4
"29073" = 4
"29075" = 5
"29077" = 9
"29079" = 7
"29081" = 7
"29083" = 7
"29085" = 10
"29087" = 5
"29089" = 6
"29091" = 9
"29093" = 9
"29095" = 7
"29097" = 7
"29099" = 3
"29101" = 8
"29103" = 9
"29105" = 8
"29107" = 3
"29109" = 6
"29111" = 6
"29113" = 3
"29115" = 6
"29117" = 6
"29119" = 9
"29121" = 5
"29123" = 9
"29125" = 5
"29127" = 6
"29129" = 5
"29131" = 9
"29133" = 10
"29135" = 5
"29137" = 5
"29139" = 7
"29141" = 9
"29143" = 10
"29145" = 6
"29147" = 7
"29149" = 10
"29151" = 3
"29153" = 10
"29155" = 10
"29157" = 4
"29159" = 8
"29161" = 8
"29163" = 8
"29165" = 1
"29167" = 9
"29169" = 5
"29171" = 9
"29173" = 3
"29175" = 8
"29177" = 3
"29179" = 10
"29181" = 10
"29183" = 1
"29185" = 9
"29186" = 3
"29187" = 8
"29189" = 2
"29195" = 9
"29197" = 9
"29199" = 6
"29201" = 8
"29203" = 10
"29205" = 5
"29207" = 8
"29209" = 6
"29211" = 7
"29213" = 9
"29215" = 10
"29217" = 8
"29219" = 4
"29221" = 10
"29223" = 10
"29225" = 7
"29227" = 6
"29229" = 10
"29510" = 10
"30001" = 8
"30003" = 10
"30005" = 10
"30007" = 3
"30009" = 4
"30011" = 8
"30013" = 4
"30015" = 6
"30017" = 5
"30019" = 3
"30021" = 4
"30023" = 8
"30025" = 2
"30027" = 4
"30029" = 5
"30031" = 4
"30033" = 9
"30035" = 10
"30037" = 9
"30039" = 5
"30041" = 8
"30043" = 2
"30045" = 6
"30047" = 9
"30049" = 3
"30051" = 10
"30053" = 8
"30055" = 6
"30057" = 3
"30059" = 9
"30061" = 8
"30063" = 5
"30065" = 8
"30067" = 4
"30069" = 9
"30071" = 6
"30073" = 9
"30075" = 5
"30077" = 9
"30079" = 6
"30081" = 7
"30083" = 3
"30085" = 10
"30087" = 8
"30089" = 9
"30091" = 3
"30093" = 8
"30095" = 2
"30097" = 3
"30099" = 6
"30101" = 6
"30103" = 3
"30105" = 4
"30107" = 7
"30109" = 4
"30111" = 4
"31001" = 4
"31003" = 4
"31005" = 3
"31007" = 4
"31009" = 9
"31011" = 2
"31013" = 4
"31015" = 4
"31017" = 4
"31019" = 4
"31021" = 3
"31023" = 2
"31025" = 1
"31027" = 3
"31029" = 3
"31031" = 4
"31033" = 2
"31035" = 3
"31037" = 3
"31039" = 2
"31041" = 4
"31043" = 4
"31045" = 7
"31047" = 3
"31049" = 4
"31051" = 2
"31053" = 4
"31055" = 5
"31057" = 4
"31059" = 2
"31061" = 5
"31063" = 4
"31065" = 4
"31067" = 4
"31069" = 6
"31071" = 5
"31073" = 2
"31075" = 3
"31077" = 4
"31079" = 4
"31081" = 2
"31083" = 3
"31085" = 7
"31087" = 5
"31089" = 4
"31091" = 2
"31093" = 2
"31095" = 4
"31097" = 3
"31099" = 2
"31101" = 3
"31103" = 10
"31105" = 4
"31107" = 5
"31109" = 5
"31111" = 3
"31113" = 4
"31115" = 7
"31117" = 8
"31119" = 4
"31121" = 3
"31123" = 6
"31125" = 4
"31127" = 4
"31129" = 4
"31131" = 2
"31133" = 4
"31135" = 2
"31137" = 2
"31139" = 2
"31141" = 2
"31143" = 2
"31145" = 4
"31147" = 4
"31149" = 6
"31151" = 2
"31153" = 1
"31155" = 2
"31157" = 6
"31159" = 1
"31161" = 7
"31163" = 4
"31165" = 7
"31167" = 2
"31169" = 3
"31171" = 6
"31173" = 10
"31175" = 5
"31177" = 1
"31179" = 5
"31181" = 3
"31183" = 9
"31185" = 2
"32001" = 3
"32003" = 5
"32005" = 2
"32007" = 2
"32009" = 5
"32011" = 2
"32013" = 2
"32015" = 2
"32017" = 4
"32019" = 3
"32021" = 6
"32023" = 8
"32027" = 8
"32029" = 2
"32031" = 5
"32033" = 4
"32510" = 5
"33001" = 2
"33003" = 2
"33005" = 3
"33007" = 5
"33009" = 3
"33011" = 1
"33013" = 2
"33015" = 1
"33017" = 3
"33019" = 3
"34001" = 4
"34003" = 1
"34005" = 1
"34007" = 3
"34009" = 3
"34011" = 7
"34013" = 6
"34015" = 1
"34017" = 6
"34019" = 1
"34021" = 3
"34023" = 1
"34025" = 1
"34027" = 1
"34029" = 3
"34031" = 6
"34033" = 3
"34035" = 1
"34037" = 1
"34039" = 3
"34041" = 1
"35001" = 6
"35003" = 10
"35005" = 10
"35006" = 10
"35007" = 8
"35009" = 9
"35011" = 8
"35013" = 10
"35015" = 6
"35017" = 8
"35019" = 10
"35021" = 4
"35023" = 10
"35025" = 8
"35027" = 8
"35028" = 1
"35029" = 10
"35031" = 10
"35033" = 10
"35035" = 10
"35037" = 10
"35039" = 9
"35041" = 10
"35043" = 4
"35045" = 10
"35047" = 10
"35049" = 6
"35051" = 9
"35053" = 10
"35055" = 8
"35057" = 10
"35059" = 8
"35061" = 10
"36001" = 4
"36003" = 7
"36005" = 10
"36007" = 6
"36009" = 5
"36011" = 4
"36013" = 7
"36015" = 6
"36017" = 6
"36019" = 6
"36021" = 2
"36023" = 5
"36025" = 3
"36027" = 1
"36029" = 5
"36031" = 4
"36033" = 6
"36035" = 6
"36037" = 3
"36039" = 7
"36041" = 2
"36043" = 6
"36045" = 5
"36047" = 10
"36049" = 5
"36051" = 4
"36053" = 3
"36055" = 5
"36057" = 7
"36059" = 1
"36061" = 6
"36063" = 4
"36065" = 5
"36067" = 4
"36069" = 2
"36071" = 3
"36073" = 4
"36075" = 7
"36077" = 6
"36079" = 1
"36081" = 5
"36083" = 4
"36085" = 3
"36087" = 3
"36089" = 7
"36091" = 1
"36093" = 3
"36095" = 4
"36097" = 4
"36099" = 4
"36101" = 5
"36103" = 1
"36105" = 9
"36107" = 2
"36109" = 9
"36111" = 4
"36113" = 2
"36115" = 3
"36117" = 2
"36119" = 2
"36121" = 3
"36123" = 7
"37001" = 8
"37003" = 7
"37005" = 10
"37007" = 10
"37009" = 9
"37011" = 10
"37013" = 9
"37015" = 10
"37017" = 10
"37019" = 6
"37021" = 7
"37023" = 8
"37025" = 4
"37027" = 8
"37029" = 2
"37031" = 4
"37033" = 9
"37035" = 5
"37037" = 5
"37039" = 8
"37041" = 9
"37043" = 8
"37045" = 9
"37047" = 10
"37049" = 7
"37051" = 8
"37053" = 3
"37055" = 3
"37057" = 7
"37059" = 4
"37061" = 10
"37063" = 8
"37065" = 10
"37067" = 6
"37069" = 6
"37071" = 9
"37073" = 7
"37075" = 10
"37077" = 6
"37079" = 10
"37081" = 8
"37083" = 10
"37085" = 6
"37087" = 5
"37089" = 6
"37091" = 10
"37093" = 8
"37095" = 10
"37097" = 4
"37099" = 9
"37101" = 6
"37103" = 8
"37105" = 8
"37107" = 10
"37109" = 5
"37111" = 9
"37113" = 9
"37115" = 9
"37117" = 10
"37119" = 6
"37121" = 8
"37123" = 10
"37125" = 6
"37127" = 6
"37129" = 8
"37131" = 10
"37133" = 6
"37135" = 7
"37137" = 5
"37139" = 10
"37141" = 6
"37143" = 6
"37145" = 6
"37147" = 9
"37149" = 5
"37151" = 8
"37153" = 10
"37155" = 10
"37157" = 8
"37159" = 9
"37161" = 10
"37163" = 9
"37165" = 10
"37167" = 6
"37169" = 5
"37171" = 9
"37173" = 8
"37175" = 6
"37177" = 10
"37179" = 2
"37181" = 10
"37183" = 3
"37185" = 10
"37187" = 10
"37189" = 10
"37191" = 9
"37193" = 9
"37195" = 10
"37197" = 5
"37199" = 9
"38001" = 2
"38003" = 3
"38005" = 10
"38007" = 3
"38009" = 3
"38011" = 1
"38013" = 2
"38015" = 2
"38017" = 3
"38019" = 2
"38021" = 3
"38023" = 2
"38025" = 3
"38027" = 3
"38029" = 6
"38031" = 2
"38033" = 4
"38035" = 6
"38037" = 8
"38039" = 2
"38041" = 3
"38043" = 6
"38045" = 2
"38047" = 5
"38049" = 4
"38051" = 5
"38053" = 4
"38055" = 2
"38057" = 1
"38059" = 2
"38061" = 4
"38063" = 2
"38065" = 3
"38067" = 2
"38069" = 4
"38071" = 4
"38073" = 2
"38075" = 2
"38077" = 3
"38079" = 10
"38081" = 2
"38083" = 7
"38085" = 10
"38087" = 4
"38089" = 2
"38091" = 1
"38093" = 3
"38095" = 2
"38097" = 2
"38099" = 3
"38101" = 4
"38103" = 4
"38105" = 2
"39001" = 10
"39003" = 8
"39005" = 6
"39007" = 6
"39009" = 10
"39011" = 2
"39013" = 6
"39015" = 4
"39017" = 4
"39019" = 6
"39021" = 4
"39023" = 9
"39025" = 2
"39027" = 6
"39029" = 8
"39031" = 9
"39033" = 6
"39035" = 8
"39037" = 3
"39039" = 3
"39041" = 1
"39043" = 5
"39045" = 3
"39047" = 6
"39049" = 8
"39051" = 3
"39053" = 8
"39055" = 1
"39057" = 4
"39059" = 9
"39061" = 8
"39063" = 3
"39065" = 7
"39067" = 8
"39069" = 4
"39071" = 8
"39073" = 6
"39075" = 6
"39077" = 4
"39079" = 10
"39081" = 8
"39083" = 6
"39085" = 2
"39087" = 9
"39089" = 4
"39091" = 7
"39093" = 5
"39095" = 9
"39097" = 5
"39099" = 7
"39101" = 9
"39103" = 1
"39105" = 10
"39107" = 2
"39109" = 3
"39111" = 7
"39113" = 8
"39115" = 9
"39117" = 4
"39119" = 8
"39121" = 7
"39123" = 2
"39125" = 4
"39127" = 9
"39129" = 4
"39131" = 10
"39133" = 5
"39135" = 3
"39137" = 2
"39139" = 5
"39141" = 9
"39143" = 3
"39145" = 10
"39147" = 5
"39149" = 3
"39151" = 5
"39153" = 6
"39155" = 8
"39157" = 5
"39159" = 1
"39161" = 4
"39163" = 10
"39165" = 1
"39167" = 6
"39169" = 4
"39171" = 3
"39173" = 4
"39175" = 2
"40001" = 10
"40003" = 7
"40005" = 10
"40007" = 3
"40009" = 8
"40011" = 9
"40013" = 7
"40015" = 9
"40017" = 2
"40019" = 8
"40021" = 10
"40023" = 10
"40025" = 8
"40027" = 4
"40029" = 9
"40031" = 7
"40033" = 6
"40035" = 6
"40037" = 6
"40039" = 7
"40041" = 10
"40043" = 4
"40045" = 4
"40047" = 6
"40049" = 6
"40051" = 6
"40053" = 4
"40055" = 10
"40057" = 10
"40059" = 3
"40061" = 7
"40063" = 10
"40065" = 7
"40067" = 9
"40069" = 10
"40071" = 7
"40073" = 3
"40075" = 10
"40077" = 8
"40079" = 10
"40081" = 5
"40083" = 5
"40085" = 6
"40087" = 3
"40089" = 10
"40091" = 9
"40093" = 4
"40095" = 8
"40097" = 8
"40099" = 5
"40101" = 9
"40103" = 5
"40105" = 6
"40107" = 10
"40109" = 7
"40111" = 9
"40113" = 6
"40115" = 9
"40117" = 6
"40119" = 9
"40121" = 8
"40123" = 8
"40125" = 8
"40127" = 10
"40129" = 4
"40131" = 2
"40133" = 10
"40135" = 9
"40137" = 6
"40139" = 3
"40141" = 10
"40143" = 6
"40145" = 3
"40147" = 4
"40149" = 5
"40151" = 6
"40153" = 5
"41001" = 9
"41003" = 9
"41005" = 2
"41007" = 5
"41009" = 4
"41011" = 9
"41013" = 7
"41015" = 6
"41017" = 5
"41019" = 9
"41021" = 3
"41023" = 6
"41025" = 9
"41027" = 4
"41029" = 6
"41031" = 9
"41033" = 8
"41035" = 7
"41037" = 9
"41039" = 9
"41041" = 7
"41043" = 8
"41045" = 10
"41047" = 8
"41049" = 6
"41051" = 8
"41053" = 6
"41055" = 4
"41057" = 6
"41059" = 6
"41061" = 6
"41063" = 6
"41065" = 6
"41067" = 2
"41069" = 9
"41071" = 5
"42001" = 2
"42003" = 3
"42005" = 4
"42007" = 5
"42009" = 4
"42011" = 4
"42013" = 4
"42015" = 3
"42017" = 1
"42019" = 2
"42021" = 4
"42023" = 5
"42025" = 3
"42027" = 8
"42029" = 1
"42031" = 8
"42033" = 7
"42035" = 7
"42037" = 6
"42039" = 9
"42041" = 1
"42043" = 4
"42045" = 2
"42047" = 3
"42049" = 7
"42051" = 9
"42053" = 9
"42055" = 2
"42057" = 2
"42059" = 8
"42061" = 5
"42063" = 8
"42065" = 6
"42067" = 3
"42069" = 4
"42071" = 2
"42073" = 6
"42075" = 3
"42077" = 4
"42079" = 6
"42081" = 7
"42083" = 5
"42085" = 6
"42087" = 6
"42089" = 4
"42091" = 1
"42093" = 3
"42095" = 2
"42097" = 5
"42099" = 2
"42101" = 10
"42103" = 2
"42105" = 5
"42107" = 4
"42109" = 4
"42111" = 5
"42113" = 5
"42115" = 4
"42117" = 6
"42119" = 4
"42121" = 6
"42123" = 4
"42125" = 2
"42127" = 4
"42129" = 2
"42131" = 4
"42133" = 2
"44001" = 1
"44003" = 2
"44005" = 2
"44007" = 7
"44009" = 2
"45001" = 9
"45003" = 8
"45005" = 10
"45007" = 8
"45009" = 10
"45011" = 10
"45013" = 4
"45015" = 5
"45017" = 8
"45019" = 8
"45021" = 9
"45023" = 9
"45025" = 10
"45027" = 10
"45029" = 10
"45031" = 10
"45033" = 10
"45035" = 3
"45037" = 9
"45039" = 10
"45041" = 9
"45043" = 9
"45045" = 6
"45047" = 10
"45049" = 10
"45051" = 9
"45053" = 10
"45055" = 8
"45057" = 9
"45059" = 9
"45061" = 10
"45063" = 4
"45065" = 9
"45067" = 10
"45069" = 10
"45071" = 8
"45073" = 4
"45075" = 10
"45077" = 7
"45079" = 7
"45081" = 9
"45083" = 7
"45085" = 9
"45087" = 8
"45089" = 10
"45091" = 4
"46003" = 3
"46005" = 5
"46007" = 10
"46009" = 7
"46011" = 7
"46013" = 2
"46015" = 5
"46017" = 10
"46019" = 5
"46021" = 3
"46023" = 10
"46025" = 5
"46027" = 9
"46029" = 3
"46031" = 10
"46033" = 3
"46035" = 5
"46037" = 7
"46039" = 2
"46041" = 10
"46043" = 5
"46045" = 3
"46047" = 6
"46049" = 5
"46051" = 3
"46053" = 10
"46055" = 4
"46057" = 3
"46059" = 3
"46061" = 2
"46063" = 7
"46065" = 2
"46067" = 3
"46069" = 4
"46071" = 10
"46073" = 6
"46075" = 6
"46077" = 2
"46079" = 3
"46081" = 4
"46083" = 1
"46085" = 10
"46087" = 2
"46089" = 8
"46091" = 5
"46093" = 2
"46095" = 10
"46097" = 4
"46099" = 3
"46101" = 3
"46103" = 5
"46105" = 7
"46107" = 3
"46109" = 9
"46111" = 5
"46113" = 10
"46115" = 4
"46117" = 2
"46119" = 2
"46121" = 10
"46123" = 9
"46125" = 3
"46127" = 1
"46129" = 6
"46135" = 3
"46137" = 10
"47001" = 6
"47003" = 10
"47005" = 9
"47007" = 10
"47009" = 4
"47011" = 9
"47013" = 10
"47015" = 6
"47017" = 9
"47019" = 10
"47021" = 3
"47023" = 9
"47025" = 10
"47027" = 10
"47029" = 10
"47031" = 10
"47033" = 9
"47035" = 8
"47037" = 9
"47039" = 9
"47041" = 9
"47043" = 4
"47045" = 9
"47047" = 4
"47049" = 10
"47051" = 5
"47053" = 8
"47055" = 7
"47057" = 9
"47059" = 10
"47061" = 10
"47063" = 9
"47065" = 6
"47067" = 10
"47069" = 10
"47071" = 10
"47073" = 8
"47075" = 10
"47077" = 7
"47079" = 9
"47081" = 9
"47083" = 9
"47085" = 6
"47087" = 9
"47089" = 6
"47091" = 10
"47093" = 4
"47095" = 10
"47097" = 10
"47099" = 8
"47101" = 9
"47103" = 7
"47105" = 5
"47107" = 8
"47109" = 10
"47111" = 10
"47113" = 9
"47115" = 10
"47117" = 5
"47119" = 5
"47121" = 10
"47123" = 9
"47125" = 6
"47127" = 4
"47129" = 9
"47131" = 6
"47133" = 9
"47135" = 9
"47137" = 9
"47139" = 9
"47141" = 9
"47143" = 9
"47145" = 5
"47147" = 4
"47149" = 5
"47151" = 10
"47153" = 10
"47155" = 6
"47157" = 9
"47159" = 9
"47161" = 5
"47163" = 8
"47165" = 4
"47167" = 4
"47169" = 7
"47171" = 7
"47173" = 10
"47175" = 10
"47177" = 10
"47179" = 8
"47181" = 10
"47183" = 9
"47185" = 9
"47187" = 1
"47189" = 2
"48001" = 9
"48003" = 4
"48005" = 8
"48007" = 10
"48009" = 3
"48011" = 2
"48013" = 9
"48015" = 4
"48017" = 8
"48019" = 6
"48021" = 6
"48023" = 9
"48025" = 10
"48027" = 4
"48029" = 7
"48031" = 4
"48033" = 3
"48035" = 7
"48037" = 8
"48039" = 3
"48041" = 10
"48043" = 5
"48045" = 6
"48047" = 10
"48049" = 8
"48051" = 5
"48053" = 5
"48055" = 7
"48057" = 9
"48059" = 6
"48061" = 10
"48063" = 9
"48065" = 2
"48067" = 9
"48069" = 9
"48071" = 2
"48073" = 10
"48075" = 10
"48077" = 4
"48079" = 9
"48081" = 4
"48083" = 9
"48085" = 1
"48087" = 8
"48089" = 7
"48091" = 3
"48093" = 9
"48095" = 10
"48097" = 5
"48099" = 4
"48101" = 9
"48103" = 2
"48105" = 6
"48107" = 10
"48109" = 10
"48111" = 5
"48113" = 8
"48115" = 10
"48117" = 9
"48119" = 8
"48121" = 1
"48123" = 9
"48125" = 9
"48127" = 10
"48129" = 7
"48131" = 10
"48133" = 10
"48135" = 9
"48137" = 10
"48139" = 3
"48141" = 10
"48143" = 8
"48145" = 10
"48147" = 6
"48149" = 4
"48151" = 5
"48153" = 9
"48155" = 8
"48157" = 2
"48159" = 6
"48161" = 8
"48163" = 10
"48165" = 8
"48167" = 4
"48169" = 10
"48171" = 3
"48173" = 3
"48175" = 6
"48177" = 9
"48179" = 6
"48181" = 5
"48183" = 9
"48185" = 9
"48187" = 3
"48189" = 9
"48191" = 10
"48193" = 6
"48195" = 5
"48197" = 10
"48199" = 4
"48201" = 8
"48203" = 7
"48205" = 3
"48207" = 10
"48209" = 5
"48211" = 2
"48213" = 9
"48215" = 10
"48217" = 7
"48219" = 9
"48221" = 3
"48223" = 8
"48225" = 10
"48227" = 10
"48229" = 10
"48231" = 9
"48233" = 4
"48235" = 2
"48237" = 6
"48239" = 6
"48241" = 9
"48243" = 3
"48245" = 10
"48247" = 10
"48249" = 9
"48251" = 4
"48253" = 10
"48255" = 10
"48257" = 3
"48259" = 2
"48261" = 4
"48263" = 3
"48265" = 6
"48267" = 8
"48269" = 4
"48271" = 10
"48273" = 10
"48275" = 10
"48277" = 8
"48279" = 9
"48281" = 5
"48283" = 10
"48285" = 5
"48287" = 4
"48289" = 5
"48291" = 8
"48293" = 8
"48295" = 4
"48297" = 8
"48299" = 6
"48301" = 5
"48303" = 9
"48305" = 9
"48307" = 10
"48309" = 9
"48311" = 3
"48313" = 10
"48315" = 10
"48317" = 6
"48319" = 6
"48321" = 9
"48323" = 10
"48325" = 7
"48327" = 10
"48329" = 5
"48331" = 9
"48333" = 7
"48335" = 10
"48337" = 6
"48339" = 3
"48341" = 5
"48343" = 9
"48345" = 6
"48347" = 10
"48349" = 9
"48351" = 10
"48353" = 9
"48355" = 9
"48357" = 4
"48359" = 4
"48361" = 5
"48363" = 7
"48365" = 4
"48367" = 3
"48369" = 5
"48371" = 10
"48373" = 9
"48375" = 10
"48377" = 10
"48379" = 6
"48381" = 2
"48383" = 3
"48385" = 10
"48387" = 9
"48389" = 10
"48391" = 8
"48393" = 1
"48395" = 9
"48397" = 1
"48399" = 10
"48401" = 5
"48403" = 8
"48405" = 9
"48407" = 9
"48409" = 9
"48411" = 9
"48413" = 7
"48415" = 7
"48417" = 4
"48419" = 10
"48421" = 4
"48423" = 5
"48425" = 2
"48427" = 10
"48429" = 9
"48431" = 4
"48433" = 7
"48435" = 4
"48437" = 9
"48439" = 5
"48441" = 8
"48443" = 5
"48445" = 10
"48447" = 5
"48449" = 9
"48451" = 9
"48453" = 8
"48455" = 8
"48457" = 9
"48459" = 6
"48461" = 6
"48463" = 10
"48465" = 10
"48467" = 7
"48469" = 8
"48471" = 10
"48473" = 9
"48475" = 9
"48477" = 5
"48479" = 10
"48481" = 9
"48483" = 4
"48485" = 7
"48487" = 7
"48489" = 10
"48491" = 1
"48493" = 3
"48495" = 6
"48497" = 4
"48499" = 8
"48501" = 5
"48503" = 7
"48505" = 10
"48507" = 10
"49001" = 4
"49003" = 2
"49005" = 6
"49007" = 5
"49009" = 1
"49011" = 1
"49013" = 4
"49015" = 3
"49017" = 3
"49019" = 5
"49021" = 9
"49023" = 3
"49025" = 4
"49027" = 4
"49029" = 1
"49031" = 9
"49033" = 3
"49035" = 4
"49037" = 10
"49039" = 8
"49041" = 5
"49043" = 1
"49045" = 2
"49047" = 5
"49049" = 5
"49051" = 1
"49053" = 5
"49055" = 5
"49057" = 4
"50001" = 3
"50003" = 4
"50005" = 6
"50007" = 3
"50009" = 7
"50011" = 3
"50013" = 1
"50015" = 4
"50017" = 4
"50019" = 7
"50021" = 4
"50023" = 3
"50025" = 4
"50027" = 2
"51001" = 9
"51003" = 2
"51005" = 4
"51007" = 3
"51009" = 4
"51011" = 4
"51013" = 1
"51015" = 2
"51017" = 3
"51019" = 2
"51021" = 5
"51023" = 1
"51025" = 9
"51027" = 10
"51029" = 9
"51031" = 5
"51033" = 3
"51035" = 7
"51036" = 3
"51037" = 8
"51041" = 1
"51043" = 2
"51045" = 3
"51047" = 2
"51049" = 7
"51051" = 10
"51053" = 3
"51057" = 4
"51059" = 1
"51061" = 1
"51063" = 4
"51065" = 1
"51067" = 5
"51069" = 1
"51071" = 4
"51073" = 2
"51075" = 1
"51077" = 7
"51079" = 2
"51081" = 10
"51083" = 9
"51085" = 1
"51087" = 2
"51089" = 8
"51091" = 5
"51093" = 2
"51095" = 1
"51097" = 4
"51099" = 1
"51101" = 1
"51103" = 4
"51105" = 10
"51107" = 1
"51109" = 2
"51111" = 9
"51113" = 3
"51115" = 2
"51117" = 9
"51119" = 4
"51121" = 9
"51125" = 4
"51127" = 1
"51131" = 8
"51133" = 5
"51135" = 9
"51137" = 2
"51139" = 6
"51141" = 8
"51143" = 6
"51145" = 1
"51147" = 10
"51149" = 2
"51153" = 1
"51155" = 6
"51157" = 2
"51159" = 8
"51161" = 1
"51163" = 3
"51165" = 3
"51167" = 8
"51169" = 8
"51171" = 3
"51173" = 8
"51175" = 6
"51177" = 1
"51179" = 1
"51181" = 3
"51183" = 10
"51185" = 8
"51187" = 2
"51191" = 4
"51193" = 5
"51195" = 10
"51197" = 6
"51199" = 1
"51510" = 2
"51515" = 7
"51520" = 9
"51530" = 6
"51540" = 9
"51550" = 1
"51570" = 2
"51580" = 6
"51590" = 10
"51595" = 10
"51600" = 1
"51610" = 1
"51620" = 9
"51630" = 9
"51640" = 10
"51650" = 4
"51660" = 10
"51670" = 7
"51678" = 9
"51680" = 10
"51683" = 3
"51685" = 2
"51690" = 10
"51700" = 5
"51710" = 8
"51720" = 9
"51730" = 10
"51735" = 1
"51740" = 8
"51750" = 10
"51760" = 10
"51770" = 10
"51775" = 2
"51790" = 5
"51800" = 3
"51810" = 1
"51820" = 7
"51830" = 8
"51840" = 9
"53001" = 9
"53003" = 6
"53005" = 4
"53007" = 5
"53009" = 6
"53011" = 4
"53013" = 5
"53015" = 9
"53017" = 5
"53019" = 9
"53021" = 8
"53023" = 4
"53025" = 9
"53027" = 8
"53029" = 2
"53031" = 3
"53033" = 3
"53035" = 3
"53037" = 9
"53039" = 7
"53041" = 6
"53043" = 3
"53045" = 6
"53047" = 10
"53049" = 8
"53051" = 8
"53053" = 3
"53055" = 3
"53057" = 4
"53059" = 4
"53061" = 2
"53063" = 5
"53065" = 7
"53067" = 2
"53069" = 4
"53071" = 6
"53073" = 5
"53075" = 10
"53077" = 10
"54001" = 7
"54003" = 4
"54005" = 8
"54007" = 10
"54009" = 5
"54011" = 10
"54013" = 10
"54015" = 10
"54017" = 9
"54019" = 10
"54021" = 10
"54023" = 7
"54025" = 9
"54027" = 8
"54029" = 4
"54031" = 6
"54033" = 7
"54035" = 9
"54037" = 3
"54039" = 5
"54041" = 9
"54043" = 10
"54045" = 10
"54047" = 10
"54049" = 7
"54051" = 7
"54053" = 8
"54055" = 9
"54057" = 6
"54059" = 10
"54061" = 10
"54063" = 8
"54065" = 4
"54067" = 8
"54069" = 6
"54071" = 5
"54073" = 5
"54075" = 9
"54077" = 6
"54079" = 2
"54081" = 8
"54083" = 8
"54085" = 9
"54087" = 10
"54089" = 10
"54091" = 9
"54093" = 9
"54095" = 6
"54097" = 10
"54099" = 9
"54101" = 10
"54103" = 8
"54105" = 10
"54107" = 6
"54109" = 9
"55001" = 5
"55003" = 8
"55005" = 4
"55007" = 5
"55009" = 2
"55011" = 3
"55013" = 5
"55015" = 1
"55017" = 3
"55019" = 5
"55021" = 2
"55023" = 5
"55025" = 3
"55027" = 2
"55029" = 3
"55031" = 5
"55033" = 5
"55035" = 6
"55037" = 4
"55039" = 3
"55041" = 7
"55043" = 6
"55045" = 2
"55047" = 4
"55049" = 2
"55051" = 7
"55053" = 8
"55055" = 3
"55057" = 6
"55059" = 4
"55061" = 2
"55063" = 4
"55065" = 3
"55067" = 4
"55069" = 3
"55071" = 3
"55073" = 3
"55075" = 5
"55077" = 3
"55078" = 10
"55079" = 10
"55081" = 6
"55083" = 4
"55085" = 3
"55087" = 2
"55089" = 1
"55091" = 3
"55093" = 3
"55095" = 4
"55097" = 3
"55099" = 4
"55101" = 5
"55103" = 5
"55105" = 4
"55107" = 9
"55109" = 1
"55111" = 3
"55113" = 8
"55115" = 4
"55117" = 2
"55119" = 4
"55121" = 4
"55123" = 5
"55125" = 5
"55127" = 4
"55129" = 5
"55131" = 1
"55133" = 1
"55135" = 3
"55137" = 5
"55139" = 3
"55141" = 3
"56001" = 9
"56003" = 4
"56005" = 1
"56007" = 4
"56009" = 2
"56011" = 1
"56013" = 4
"56015" = 7
"56017" = 3
"56019" = 2
"56021" = 3
"56023" = 2
"56025" = 2
"56027" = 5
"56029" = 3
"56031" = 3
"56033" = 2
"56035" = 1
"56037" = 2
"56039" = 2
"56041" = 3
"56043" = 3
"56045" = 2
"02201" = 7
"02232" = 5
"02280" = 3
OTHER = 0
;



 /* -----------------------------------------------------------------------------  */
 /*  CCS 653 - Delirium, dementia, and amnestic and other cognitive disorders   -  */
 /* ------------------------------------------------------------------------------ */


/*VALUE $CCS653*/

VALUE $DGNEUID
"F0150"  =  "1"  /* Vascular dementia without behavioral disturbance */
"F0151"  =  "1"  /* Vascular dementia with behavioral disturbance */
"F0280"  =  "1"  /* Dementia in other diseases classified elsewhere without behavioral disturbance */
"F0281"  =  "1"  /* Dementia in other diseases classified elsewhere with behavioral disturbance */
"F0390"  =  "1"  /* Unspecified dementia without behavioral disturbance */
"F04"  =  "1"  /* Amnestic disorder due to known physiological condition */
"F05"  =  "1"  /* Delirium due to known physiological condition */
"F060"  =  "1"  /* Psychotic disorder with hallucinations due to known physiological condition */
"F061"  =  "1"  /* Catatonic disorder due to known physiological condition */
"F068"  =  "1"  /* Other specified mental disorders due to known physiological condition */
"F070"  =  "1"  /* Personality change due to known physiological condition */
"F0789"  =  "1"  /* Other personality and behavioral disorders due to known physiological condition */
"F079"  =  "1"  /* Unspecified personality and behavioral disorder due to known physiological condition */
"F09"  =  "1"  /* Unspecified mental disorder due to known physiological condition */
"F482"  =  "1"  /* Pseudobulbar affect */
"G300"  =  "1"  /* Alzheimer's disease with early onset */
"G301"  =  "1"  /* Alzheimer's disease with late onset */
"G308"  =  "1"  /* Other Alzheimer's disease */
"G309"  =  "1"  /* Alzheimer's disease, unspecified */
"G3101"  =  "1"  /* Pick's disease */
"G3109"  =  "1"  /* Other frontotemporal dementia */
"G311"  =  "1"  /* Senile degeneration of brain, not elsewhere classified */
"G3183"  =  "1"  /* Dementia with Lewy bodies */
"G3185"  =  "1"  /* Corticobasal degeneration */
"G9382"  =  "1"  /* Brain death */
OTHER  = '0' ;



/* ---------------------------------------- */
 /* Esophageal Cancer Resection (PRESOP)     */
 /*   Volume Indicator / IQI #1              */
 /*   Mortality (post-op) Indicator / IQI #8 */
 /* ---------------------------------------- */

 /* Esophageal resection */
 /* ICD-10-CM Procedure Codes: */

VALUE $PRESOPP
"0D11074"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D11076"  =  "1"  /* Bypass Upper Esophagus to Stomach with Autologous Tissue Substitute, Open Approach */
"0D11079"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1107A"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1107B"  =  "1"  /* Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D110J4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Synthetic Substitute, Open Approach */
"0D110J6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Synthetic Substitute, Open Approach */
"0D110J9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D110JA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D110JB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D110K4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D110K6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach */
"0D110K9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D110KA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D110KB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D110Z4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous, Open Approach */
"0D110Z6"  =  "1"  /* Bypass Upper Esophagus to Stomach, Open Approach */
"0D110Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Open Approach */
"0D110ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Open Approach */
"0D110ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Open Approach */
"0D113J4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D11474"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D11476"  =  "1"  /* Bypass Upper Esophagus to Stomach with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D11479"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1147A"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1147B"  =  "1"  /* Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114J4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114J6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114J9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114JA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114JB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D114K4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114K6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114K9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114KA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114KB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D114Z4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous, Percutaneous Endoscopic Approach */
"0D114Z6"  =  "1"  /* Bypass Upper Esophagus to Stomach, Percutaneous Endoscopic Approach */
"0D114Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D114ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D114ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D11874"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D11876"  =  "1"  /* Bypass Upper Esophagus to Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D11879"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1187A"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1187B"  =  "1"  /* Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118J4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118J6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118J9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118JA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118JB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118K4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118K6"  =  "1"  /* Bypass Upper Esophagus to Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118K9"  =  "1"  /* Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118KA"  =  "1"  /* Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118KB"  =  "1"  /* Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D118Z4"  =  "1"  /* Bypass Upper Esophagus to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D118Z6"  =  "1"  /* Bypass Upper Esophagus to Stomach, Via Natural or Artificial Opening Endoscopic */
"0D118Z9"  =  "1"  /* Bypass Upper Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D118ZA"  =  "1"  /* Bypass Upper Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D118ZB"  =  "1"  /* Bypass Upper Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D12074"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D12076"  =  "1"  /* Bypass Middle Esophagus to Stomach with Autologous Tissue Substitute, Open Approach */
"0D12079"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1207A"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1207B"  =  "1"  /* Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D120J4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Synthetic Substitute, Open Approach */
"0D120J6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Synthetic Substitute, Open Approach */
"0D120J9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D120JA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D120JB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D120K4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D120K6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach */
"0D120K9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D120KA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D120KB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D120Z4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous, Open Approach */
"0D120Z6"  =  "1"  /* Bypass Middle Esophagus to Stomach, Open Approach */
"0D120Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Open Approach */
"0D120ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Open Approach */
"0D120ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Open Approach */
"0D123J4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D12474"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D12476"  =  "1"  /* Bypass Middle Esophagus to Stomach with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D12479"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1247A"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1247B"  =  "1"  /* Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124J4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124J6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124J9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124JA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124JB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D124K4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124K6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124K9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124KA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124KB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D124Z4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous, Percutaneous Endoscopic Approach */
"0D124Z6"  =  "1"  /* Bypass Middle Esophagus to Stomach, Percutaneous Endoscopic Approach */
"0D124Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D124ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D124ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D12874"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D12876"  =  "1"  /* Bypass Middle Esophagus to Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D12879"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1287A"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1287B"  =  "1"  /* Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128J4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128J6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128J9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128JA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128JB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128K4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128K6"  =  "1"  /* Bypass Middle Esophagus to Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128K9"  =  "1"  /* Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128KA"  =  "1"  /* Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128KB"  =  "1"  /* Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D128Z4"  =  "1"  /* Bypass Middle Esophagus to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D128Z6"  =  "1"  /* Bypass Middle Esophagus to Stomach, Via Natural or Artificial Opening Endoscopic */
"0D128Z9"  =  "1"  /* Bypass Middle Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D128ZA"  =  "1"  /* Bypass Middle Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D128ZB"  =  "1"  /* Bypass Middle Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D13074"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D13076"  =  "1"  /* Bypass Lower Esophagus to Stomach with Autologous Tissue Substitute, Open Approach */
"0D13079"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1307A"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1307B"  =  "1"  /* Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D130J4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Synthetic Substitute, Open Approach */
"0D130J6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Synthetic Substitute, Open Approach */
"0D130J9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D130JA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D130JB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D130K4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D130K6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach */
"0D130K9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D130KA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D130KB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D130Z4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous, Open Approach */
"0D130Z6"  =  "1"  /* Bypass Lower Esophagus to Stomach, Open Approach */
"0D130Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Open Approach */
"0D130ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Open Approach */
"0D130ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Open Approach */
"0D133J4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D13474"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D13476"  =  "1"  /* Bypass Lower Esophagus to Stomach with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D13479"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1347A"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1347B"  =  "1"  /* Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134J4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134J6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134J9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134JA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134JB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D134K4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134K6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134K9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134KA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134KB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D134Z4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous, Percutaneous Endoscopic Approach */
"0D134Z6"  =  "1"  /* Bypass Lower Esophagus to Stomach, Percutaneous Endoscopic Approach */
"0D134Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D134ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D134ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D13874"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D13876"  =  "1"  /* Bypass Lower Esophagus to Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D13879"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1387A"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1387B"  =  "1"  /* Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138J4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138J6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138J9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138JA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138JB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138K4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138K6"  =  "1"  /* Bypass Lower Esophagus to Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138K9"  =  "1"  /* Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138KA"  =  "1"  /* Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138KB"  =  "1"  /* Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D138Z4"  =  "1"  /* Bypass Lower Esophagus to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D138Z6"  =  "1"  /* Bypass Lower Esophagus to Stomach, Via Natural or Artificial Opening Endoscopic */
"0D138Z9"  =  "1"  /* Bypass Lower Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D138ZA"  =  "1"  /* Bypass Lower Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D138ZB"  =  "1"  /* Bypass Lower Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0D15074"  =  "1"  /* Bypass Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach */
"0D15076"  =  "1"  /* Bypass Esophagus to Stomach with Autologous Tissue Substitute, Open Approach */
"0D15079"  =  "1"  /* Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach */
"0D1507A"  =  "1"  /* Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach */
"0D1507B"  =  "1"  /* Bypass Esophagus to Ileum with Autologous Tissue Substitute, Open Approach */
"0D150J4"  =  "1"  /* Bypass Esophagus to Cutaneous with Synthetic Substitute, Open Approach */
"0D150J6"  =  "1"  /* Bypass Esophagus to Stomach with Synthetic Substitute, Open Approach */
"0D150J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synthetic Substitute, Open Approach */
"0D150JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synthetic Substitute, Open Approach */
"0D150JB"  =  "1"  /* Bypass Esophagus to Ileum with Synthetic Substitute, Open Approach */
"0D150K4"  =  "1"  /* Bypass Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach */
"0D150K6"  =  "1"  /* Bypass Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach */
"0D150K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach */
"0D150KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach */
"0D150KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach */
"0D150Z4"  =  "1"  /* Bypass Esophagus to Cutaneous, Open Approach */
"0D150Z6"  =  "1"  /* Bypass Esophagus to Stomach, Open Approach */
"0D150Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Open Approach */
"0D150ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Open Approach */
"0D150ZB"  =  "1"  /* Bypass Esophagus to Ileum, Open Approach */
"0D153J4"  =  "1"  /* Bypass Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Approach */
"0D15474"  =  "1"  /* Bypass Esophagus to Cutaneous with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D15476"  =  "1"  /* Bypass Esophagus to Stomach with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D15479"  =  "1"  /* Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1547A"  =  "1"  /* Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D1547B"  =  "1"  /* Bypass Esophagus to Ileum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154J4"  =  "1"  /* Bypass Esophagus to Cutaneous with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154J6"  =  "1"  /* Bypass Esophagus to Stomach with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154JB"  =  "1"  /* Bypass Esophagus to Ileum with Synthetic Substitute, Percutaneous Endoscopic Approach */
"0D154K4"  =  "1"  /* Bypass Esophagus to Cutaneous with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154K6"  =  "1"  /* Bypass Esophagus to Stomach with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach */
"0D154Z4"  =  "1"  /* Bypass Esophagus to Cutaneous, Percutaneous Endoscopic Approach */
"0D154Z6"  =  "1"  /* Bypass Esophagus to Stomach, Percutaneous Endoscopic Approach */
"0D154Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Percutaneous Endoscopic Approach */
"0D154ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Percutaneous Endoscopic Approach */
"0D154ZB"  =  "1"  /* Bypass Esophagus to Ileum, Percutaneous Endoscopic Approach */
"0D15874"  =  "1"  /* Bypass Esophagus to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D15876"  =  "1"  /* Bypass Esophagus to Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D15879"  =  "1"  /* Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1587A"  =  "1"  /* Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D1587B"  =  "1"  /* Bypass Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158J4"  =  "1"  /* Bypass Esophagus to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158J6"  =  "1"  /* Bypass Esophagus to Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158J9"  =  "1"  /* Bypass Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158JA"  =  "1"  /* Bypass Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158JB"  =  "1"  /* Bypass Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158K4"  =  "1"  /* Bypass Esophagus to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158K6"  =  "1"  /* Bypass Esophagus to Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158K9"  =  "1"  /* Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158KA"  =  "1"  /* Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158KB"  =  "1"  /* Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic */
"0D158Z4"  =  "1"  /* Bypass Esophagus to Cutaneous, Via Natural or Artificial Opening Endoscopic */
"0D158Z6"  =  "1"  /* Bypass Esophagus to Stomach, Via Natural or Artificial Opening Endoscopic */
"0D158Z9"  =  "1"  /* Bypass Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic */
"0D158ZA"  =  "1"  /* Bypass Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic */
"0D158ZB"  =  "1"  /* Bypass Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic */
"0DB10ZZ"  =  "1"  /* Excision of Upper Esophagus, Open Approach */
"0DB13ZZ"  =  "1"  /* Excision of Upper Esophagus, Percutaneous Approach */
"0DB17ZZ"  =  "1"  /* Excision of Upper Esophagus, Via Natural or Artificial Opening */
"0DB20ZZ"  =  "1"  /* Excision of Middle Esophagus, Open Approach */
"0DB23ZZ"  =  "1"  /* Excision of Middle Esophagus, Percutaneous Approach */
"0DB27ZZ"  =  "1"  /* Excision of Middle Esophagus, Via Natural or Artificial Opening */
"0DB30ZZ"  =  "1"  /* Excision of Lower Esophagus, Open Approach */
"0DB33ZZ"  =  "1"  /* Excision of Lower Esophagus, Percutaneous Approach */
"0DB37ZZ"  =  "1"  /* Excision of Lower Esophagus, Via Natural or Artificial Opening */
"0DB50ZZ"  =  "1"  /* Excision of Esophagus, Open Approach */
"0DB53ZZ"  =  "1"  /* Excision of Esophagus, Percutaneous Approach */
"0DB57ZZ"  =  "1"  /* Excision of Esophagus, Via Natural or Artificial Opening */
"0DT10ZZ"  =  "1"  /* Resection of Upper Esophagus, Open Approach */
"0DT14ZZ"  =  "1"  /* Resection of Upper Esophagus, Percutaneous Endoscopic Approach */
"0DT17ZZ"  =  "1"  /* Resection of Upper Esophagus, Via Natural or Artificial Opening */
"0DT18ZZ"  =  "1"  /* Resection of Upper Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT20ZZ"  =  "1"  /* Resection of Middle Esophagus, Open Approach */
"0DT24ZZ"  =  "1"  /* Resection of Middle Esophagus, Percutaneous Endoscopic Approach */
"0DT27ZZ"  =  "1"  /* Resection of Middle Esophagus, Via Natural or Artificial Opening */
"0DT28ZZ"  =  "1"  /* Resection of Middle Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT30ZZ"  =  "1"  /* Resection of Lower Esophagus, Open Approach */
"0DT34ZZ"  =  "1"  /* Resection of Lower Esophagus, Percutaneous Endoscopic Approach */
"0DT37ZZ"  =  "1"  /* Resection of Lower Esophagus, Via Natural or Artificial Opening */
"0DT38ZZ"  =  "1"  /* Resection of Lower Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DT50ZZ"  =  "1"  /* Resection of Esophagus, Open Approach */
"0DT54ZZ"  =  "1"  /* Resection of Esophagus, Percutaneous Endoscopic Approach */
"0DT57ZZ"  =  "1"  /* Resection of Esophagus, Via Natural or Artificial Opening */
"0DT58ZZ"  =  "1"  /* Resection of Esophagus, Via Natural or Artificial Opening Endoscopic */
"0DX60Z5"  =  "1"  /* Transfer Stomach to Esophagus, Open Approach */
"0DX64Z5"  =  "1"  /* Transfer Stomach to Esophagus, Percutaneous Endoscopic Approach */
"0DX80Z5"  =  "1"  /* Transfer Small Intestine to Esophagus, Open Approach */
"0DX84Z5"  =  "1"  /* Transfer Small Intestine to Esophagus, Percutaneous Endoscopic Approach */
"0DXE0Z5"  =  "1"  /* Transfer Large Intestine to Esophagus, Open Approach */
"0DXE4Z5"  =  "1"  /* Transfer Large Intestine to Esophagus, Percutaneous Endoscopic Approach */
OTHER  = '0' ;

 /* Esophageal resection */
 /* ICD-10-CM Procedure Codes: */

VALUE $PRESO2P
"0DT60ZZ"  =  "1"  /* Resection of Stomach, Open Approach */
"0DT64ZZ"  =  "1"  /* Resection of Stomach, Percutaneous Endoscopic Approach */
"0DT67ZZ"  =  "1"  /* Resection of Stomach, Via Natural or Artificial Opening */
"0DT68ZZ"  =  "1"  /* Resection of Stomach, Via Natural or Artificial Opening Endoscopic */
OTHER  = '0' ;

VALUE $POAXMPT 
'B900'
,'B901'
,'B902'
,'B908'
,'B909'
,'B91'
,'B92'
,'B940'
,'B941'
,'B942'
,'B948'
,'B949'
,'E640'
,'E641'
,'E642'
,'E643'
,'E648'
,'E649'
,'I252'
,'I6900'
,'I6901'
,'I69020'
,'I69021'
,'I69022'
,'I69023'
,'I69028'
,'I69031'
,'I69032'
,'I69033'
,'I69034'
,'I69039'
,'I69041'
,'I69042'
,'I69043'
,'I69044'
,'I69049'
,'I69051'
,'I69052'
,'I69053'
,'I69054'
,'I69059'
,'I69061'
,'I69062'
,'I69063'
,'I69064'
,'I69065'
,'I69069'
,'I69090'
,'I69091'
,'I69092'
,'I69093'
,'I69098'
,'I6910'
,'I6911'
,'I69120'
,'I69121'
,'I69122'
,'I69123'
,'I69128'
,'I69131'
,'I69132'
,'I69133'
,'I69134'
,'I69139'
,'I69141'
,'I69142'
,'I69143'
,'I69144'
,'I69149'
,'I69151'
,'I69152'
,'I69153'
,'I69154'
,'I69159'
,'I69161'
,'I69162'
,'I69163'
,'I69164'
,'I69165'
,'I69169'
,'I69190'
,'I69191'
,'I69192'
,'I69193'
,'I69198'
,'I6920'
,'I6921'
,'I69220'
,'I69221'
,'I69222'
,'I69223'
,'I69228'
,'I69231'
,'I69232'
,'I69233'
,'I69234'
,'I69239'
,'I69241'
,'I69242'
,'I69243'
,'I69244'
,'I69249'
,'I69251'
,'I69252'
,'I69253'
,'I69254'
,'I69259'
,'I69261'
,'I69262'
,'I69263'
,'I69264'
,'I69265'
,'I69269'
,'I69290'
,'I69291'
,'I69292'
,'I69293'
,'I69298'
,'I6930'
,'I6931'
,'I69320'
,'I69321'
,'I69322'
,'I69323'
,'I69328'
,'I69331'
,'I69332'
,'I69333'
,'I69334'
,'I69339'
,'I69341'
,'I69342'
,'I69343'
,'I69344'
,'I69349'
,'I69351'
,'I69352'
,'I69353'
,'I69354'
,'I69359'
,'I69361'
,'I69362'
,'I69363'
,'I69364'
,'I69365'
,'I69369'
,'I69390'
,'I69391'
,'I69392'
,'I69393'
,'I69398'
,'I6980'
,'I6981'
,'I69820'
,'I69821'
,'I69822'
,'I69823'
,'I69828'
,'I69831'
,'I69832'
,'I69833'
,'I69834'
,'I69839'
,'I69841'
,'I69842'
,'I69843'
,'I69844'
,'I69849'
,'I69851'
,'I69852'
,'I69853'
,'I69854'
,'I69859'
,'I69861'
,'I69862'
,'I69863'
,'I69864'
,'I69865'
,'I69869'
,'I69890'
,'I69891'
,'I69892'
,'I69893'
,'I69898'
,'I6990'
,'I6991'
,'I69920'
,'I69921'
,'I69922'
,'I69923'
,'I69928'
,'I69931'
,'I69932'
,'I69933'
,'I69934'
,'I69939'
,'I69941'
,'I69942'
,'I69943'
,'I69944'
,'I69949'
,'I69951'
,'I69952'
,'I69953'
,'I69954'
,'I69959'
,'I69961'
,'I69962'
,'I69963'
,'I69964'
,'I69965'
,'I69969'
,'I69990'
,'I69991'
,'I69992'
,'I69993'
,'I69998'
,'O0900'
,'O0901'
,'O0902'
,'O0903'
,'O0910'
,'O0911'
,'O0912'
,'O0913'
,'O09211'
,'O09212'
,'O09213'
,'O09219'
,'O09291'
,'O09292'
,'O09293'
,'O09299'
,'O0930'
,'O0931'
,'O0932'
,'O0933'
,'O0940'
,'O0941'
,'O0942'
,'O0943'
,'O09511'
,'O09512'
,'O09513'
,'O09519'
,'O09521'
,'O09522'
,'O09523'
,'O09529'
,'O09611'
,'O09612'
,'O09613'
,'O09619'
,'O09621'
,'O09622'
,'O09623'
,'O09629'
,'O0970'
,'O0971'
,'O0972'
,'O0973'
,'O09811'
,'O09812'
,'O09813'
,'O09819'
,'O09821'
,'O09822'
,'O09823'
,'O09829'
,'O09891'
,'O09892'
,'O09893'
,'O09899'
,'O0990'
,'O0991'
,'O0992'
,'O0993'
,'O665'
,'O80'
,'O94'
,'P000'
,'P001'
,'P002'
,'P003'
,'P004'
,'P005'
,'P006'
,'P007'
,'P0081'
,'P0089'
,'P009'
,'Q000'
,'Q001'
,'Q002'
,'Q010'
,'Q011'
,'Q012'
,'Q018'
,'Q019'
,'Q02'
,'Q030'
,'Q031'
,'Q038'
,'Q039'
,'Q040'
,'Q041'
,'Q042'
,'Q043'
,'Q044'
,'Q045'
,'Q046'
,'Q048'
,'Q049'
,'Q050'
,'Q051'
,'Q052'
,'Q053'
,'Q054'
,'Q055'
,'Q056'
,'Q057'
,'Q058'
,'Q059'
,'Q060'
,'Q061'
,'Q062'
,'Q063'
,'Q064'
,'Q068'
,'Q069'
,'Q0700'
,'Q0701'
,'Q0702'
,'Q0703'
,'Q078'
,'Q079'
,'Q100'
,'Q101'
,'Q102'
,'Q103'
,'Q104'
,'Q105'
,'Q106'
,'Q107'
,'Q110'
,'Q111'
,'Q112'
,'Q113'
,'Q120'
,'Q121'
,'Q122'
,'Q123'
,'Q124'
,'Q128'
,'Q129'
,'Q130'
,'Q131'
,'Q132'
,'Q133'
,'Q134'
,'Q135'
,'Q1381'
,'Q1389'
,'Q139'
,'Q140'
,'Q141'
,'Q142'
,'Q143'
,'Q148'
,'Q149'
,'Q150'
,'Q158'
,'Q159'
,'Q160'
,'Q161'
,'Q162'
,'Q163'
,'Q164'
,'Q165'
,'Q169'
,'Q170'
,'Q171'
,'Q172'
,'Q173'
,'Q174'
,'Q175'
,'Q178'
,'Q179'
,'Q180'
,'Q181'
,'Q182'
,'Q183'
,'Q184'
,'Q185'
,'Q186'
,'Q187'
,'Q188'
,'Q189'
,'Q200'
,'Q201'
,'Q202'
,'Q203'
,'Q204'
,'Q205'
,'Q206'
,'Q208'
,'Q209'
,'Q210'
,'Q211'
,'Q212'
,'Q213'
,'Q214'
,'Q218'
,'Q219'
,'Q220'
,'Q221'
,'Q222'
,'Q223'
,'Q224'
,'Q225'
,'Q226'
,'Q228'
,'Q229'
,'Q230'
,'Q231'
,'Q232'
,'Q233'
,'Q234'
,'Q238'
,'Q239'
,'Q240'
,'Q241'
,'Q242'
,'Q243'
,'Q244'
,'Q245'
,'Q246'
,'Q248'
,'Q249'
,'Q250'
,'Q251'
,'Q252'
,'Q253'
,'Q254'
,'Q255'
,'Q256'
,'Q2571'
,'Q2572'
,'Q2579'
,'Q258'
,'Q259'
,'Q260'
,'Q261'
,'Q262'
,'Q263'
,'Q264'
,'Q265'
,'Q266'
,'Q268'
,'Q269'
,'Q270'
,'Q271'
,'Q272'
,'Q2730'
,'Q2731'
,'Q2732'
,'Q2733'
,'Q2734'
,'Q2739'
,'Q274'
,'Q278'
,'Q279'
,'Q280'
,'Q281'
,'Q282'
,'Q283'
,'Q288'
,'Q289'
,'Q300'
,'Q301'
,'Q302'
,'Q303'
,'Q308'
,'Q309'
,'Q310'
,'Q311'
,'Q312'
,'Q313'
,'Q315'
,'Q318'
,'Q319'
,'Q320'
,'Q321'
,'Q322'
,'Q323'
,'Q324'
,'Q330'
,'Q331'
,'Q332'
,'Q333'
,'Q334'
,'Q335'
,'Q336'
,'Q338'
,'Q339'
,'Q340'
,'Q341'
,'Q348'
,'Q349'
,'Q351'
,'Q353'
,'Q355'
,'Q357'
,'Q359'
,'Q360'
,'Q361'
,'Q369'
,'Q370'
,'Q371'
,'Q372'
,'Q373'
,'Q374'
,'Q375'
,'Q378'
,'Q379'
,'Q380'
,'Q381'
,'Q382'
,'Q383'
,'Q384'
,'Q385'
,'Q386'
,'Q387'
,'Q388'
,'Q390'
,'Q391'
,'Q392'
,'Q393'
,'Q394'
,'Q395'
,'Q396'
,'Q398'
,'Q399'
,'Q400'
,'Q401'
,'Q402'
,'Q403'
,'Q408'
,'Q409'
,'Q410'
,'Q411'
,'Q412'
,'Q418'
,'Q419'
,'Q420'
,'Q421'
,'Q422'
,'Q423'
,'Q428'
,'Q429'
,'Q430'
,'Q431'
,'Q432'
,'Q433'
,'Q434'
,'Q435'
,'Q436'
,'Q437'
,'Q438'
,'Q439'
,'Q440'
,'Q441'
,'Q442'
,'Q443'
,'Q444'
,'Q445'
,'Q446'
,'Q447'
,'Q450'
,'Q451'
,'Q452'
,'Q453'
,'Q458'
,'Q459'
,'Q5001'
,'Q5002'
,'Q501'
,'Q502'
,'Q5031'
,'Q5032'
,'Q5039'
,'Q504'
,'Q505'
,'Q506'
,'Q510'
,'Q5110'
,'Q5111'
,'Q512'
,'Q513'
,'Q514'
,'Q515'
,'Q516'
,'Q517'
,'Q51810'
,'Q51811'
,'Q51818'
,'Q51820'
,'Q51821'
,'Q51828'
,'Q519'
,'Q520'
,'Q5210'
,'Q5211'
,'Q5212'
,'Q522'
,'Q523'
,'Q524'
,'Q525'
,'Q526'
,'Q5270'
,'Q5271'
,'Q5279'
,'Q528'
,'Q529'
,'Q5300'
,'Q5301'
,'Q5302'
,'Q5310'
,'Q5311'
,'Q5312'
,'Q5320'
,'Q5321'
,'Q5322'
,'Q539'
,'Q540'
,'Q541'
,'Q542'
,'Q543'
,'Q544'
,'Q548'
,'Q549'
,'Q550'
,'Q551'
,'Q5520'
,'Q5521'
,'Q5522'
,'Q5523'
,'Q5529'
,'Q553'
,'Q554'
,'Q555'
,'Q5561'
,'Q5562'
,'Q5563'
,'Q5564'
,'Q5569'
,'Q557'
,'Q558'
,'Q559'
,'Q560'
,'Q561'
,'Q562'
,'Q563'
,'Q564'
,'Q600'
,'Q601'
,'Q602'
,'Q603'
,'Q604'
,'Q605'
,'Q606'
,'Q6100'
,'Q6101'
,'Q6102'
,'Q6111'
,'Q6119'
,'Q612'
,'Q613'
,'Q614'
,'Q615'
,'Q618'
,'Q619'
,'Q620'
,'Q6210'
,'Q6211'
,'Q6212'
,'Q622'
,'Q6231'
,'Q6232'
,'Q6239'
,'Q624'
,'Q625'
,'Q6260'
,'Q6261'
,'Q6262'
,'Q6263'
,'Q6269'
,'Q627'
,'Q628'
,'Q630'
,'Q631'
,'Q632'
,'Q633'
,'Q638'
,'Q639'
,'Q640'
,'Q6410'
,'Q6411'
,'Q6412'
,'Q6419'
,'Q642'
,'Q6431'
,'Q6432'
,'Q6433'
,'Q6439'
,'Q644'
,'Q645'
,'Q646'
,'Q6470'
,'Q6471'
,'Q6472'
,'Q6473'
,'Q6474'
,'Q6475'
,'Q6479'
,'Q648'
,'Q649'
,'Q6500'
,'Q6501'
,'Q6502'
,'Q651'
,'Q652'
,'Q6530'
,'Q6531'
,'Q6532'
,'Q654'
,'Q655'
,'Q656'
,'Q6581'
,'Q6582'
,'Q6589'
,'Q659'
,'Q660'
,'Q661'
,'Q662'
,'Q663'
,'Q664'
,'Q6650'
,'Q6651'
,'Q6652'
,'Q666'
,'Q667'
,'Q6680'
,'Q6681'
,'Q6682'
,'Q6689'
,'Q669'
,'Q670'
,'Q671'
,'Q672'
,'Q673'
,'Q674'
,'Q675'
,'Q676'
,'Q677'
,'Q678'
,'Q680'
,'Q681'
,'Q682'
,'Q683'
,'Q684'
,'Q685'
,'Q686'
,'Q688'
,'Q690'
,'Q691'
,'Q692'
,'Q699'
,'Q7000'
,'Q7001'
,'Q7002'
,'Q7003'
,'Q7010'
,'Q7011'
,'Q7012'
,'Q7013'
,'Q7020'
,'Q7021'
,'Q7022'
,'Q7023'
,'Q7030'
,'Q7031'
,'Q7032'
,'Q7033'
,'Q704'
,'Q709'
,'Q7100'
,'Q7101'
,'Q7102'
,'Q7103'
,'Q7110'
,'Q7111'
,'Q7112'
,'Q7113'
,'Q7120'
,'Q7121'
,'Q7122'
,'Q7123'
,'Q7130'
,'Q7131'
,'Q7132'
,'Q7133'
,'Q7140'
,'Q7141'
,'Q7142'
,'Q7143'
,'Q7150'
,'Q7151'
,'Q7152'
,'Q7153'
,'Q7160'
,'Q7161'
,'Q7162'
,'Q7163'
,'Q71811'
,'Q71812'
,'Q71813'
,'Q71819'
,'Q71891'
,'Q71892'
,'Q71893'
,'Q71899'
,'Q7190'
,'Q7191'
,'Q7192'
,'Q7193'
,'Q7200'
,'Q7201'
,'Q7202'
,'Q7203'
,'Q7210'
,'Q7211'
,'Q7212'
,'Q7213'
,'Q7220'
,'Q7221'
,'Q7222'
,'Q7223'
,'Q7230'
,'Q7231'
,'Q7232'
,'Q7233'
,'Q7240'
,'Q7241'
,'Q7242'
,'Q7243'
,'Q7250'
,'Q7251'
,'Q7252'
,'Q7253'
,'Q7260'
,'Q7261'
,'Q7262'
,'Q7263'
,'Q7270'
,'Q7271'
,'Q7272'
,'Q7273'
,'Q72811'
,'Q72812'
,'Q72813'
,'Q72819'
,'Q72891'
,'Q72892'
,'Q72893'
,'Q72899'
,'Q7290'
,'Q7291'
,'Q7292'
,'Q7293'
,'Q730'
,'Q731'
,'Q738'
,'Q740'
,'Q741'
,'Q742'
,'Q743'
,'Q748'
,'Q749'
,'Q750'
,'Q751'
,'Q752'
,'Q753'
,'Q754'
,'Q755'
,'Q758'
,'Q759'
,'Q760'
,'Q761'
,'Q762'
,'Q763'
,'Q76411'
,'Q76412'
,'Q76413'
,'Q76414'
,'Q76415'
,'Q76419'
,'Q76425'
,'Q76426'
,'Q76427'
,'Q76428'
,'Q76429'
,'Q7649'
,'Q765'
,'Q766'
,'Q767'
,'Q768'
,'Q769'
,'Q770'
,'Q771'
,'Q772'
,'Q773'
,'Q774'
,'Q775'
,'Q776'
,'Q777'
,'Q778'
,'Q779'
,'Q780'
,'Q781'
,'Q782'
,'Q783'
,'Q784'
,'Q785'
,'Q786'
,'Q788'
,'Q789'
,'Q790'
,'Q791'
,'Q792'
,'Q793'
,'Q794'
,'Q7951'
,'Q7959'
,'Q796'
,'Q798'
,'Q799'
,'Q800'
,'Q801'
,'Q802'
,'Q803'
,'Q804'
,'Q808'
,'Q809'
,'Q810'
,'Q811'
,'Q812'
,'Q818'
,'Q819'
,'Q820'
,'Q821'
,'Q822'
,'Q823'
,'Q824'
,'Q825'
,'Q828'
,'Q829'
,'Q830'
,'Q831'
,'Q832'
,'Q833'
,'Q838'
,'Q839'
,'Q840'
,'Q841'
,'Q842'
,'Q843'
,'Q844'
,'Q845'
,'Q846'
,'Q848'
,'Q849'
,'Q8500'
,'Q8501'
,'Q8502'
,'Q8503'
,'Q8509'
,'Q851'
,'Q858'
,'Q859'
,'Q860'
,'Q861'
,'Q862'
,'Q868'
,'Q870'
,'Q871'
,'Q872'
,'Q873'
,'Q8740'
,'Q87410'
,'Q87418'
,'Q8742'
,'Q8743'
,'Q875'
,'Q8781'
,'Q8789'
,'Q8901'
,'Q8909'
,'Q891'
,'Q892'
,'Q893'
,'Q894'
,'Q897'
,'Q898'
,'Q899'
,'Q900'
,'Q901'
,'Q902'
,'Q909'
,'Q910'
,'Q911'
,'Q912'
,'Q913'
,'Q914'
,'Q915'
,'Q916'
,'Q917'
,'Q920'
,'Q921'
,'Q922'
,'Q925'
,'Q9261'
,'Q9262'
,'Q927'
,'Q928'
,'Q929'
,'Q930'
,'Q931'
,'Q932'
,'Q933'
,'Q934'
,'Q935'
,'Q937'
,'Q9381'
,'Q9388'
,'Q9389'
,'Q939'
,'Q950'
,'Q951'
,'Q952'
,'Q953'
,'Q955'
,'Q958'
,'Q959'
,'Q960'
,'Q961'
,'Q962'
,'Q963'
,'Q964'
,'Q968'
,'Q969'
,'Q970'
,'Q971'
,'Q972'
,'Q973'
,'Q978'
,'Q979'
,'Q980'
,'Q981'
,'Q983'
,'Q984'
,'Q985'
,'Q986'
,'Q987'
,'Q988'
,'Q989'
,'Q990'
,'Q991'
,'Q992'
,'Q998'
,'Q999'
,'S0000XD'
,'S0000XS'
,'S0001XD'
,'S0001XS'
,'S0002XD'
,'S0002XS'
,'S0003XD'
,'S0003XS'
,'S0004XD'
,'S0004XS'
,'S0005XD'
,'S0005XS'
,'S0006XD'
,'S0006XS'
,'S0007XD'
,'S0007XS'
,'S0010XD'
,'S0010XS'
,'S0011XD'
,'S0011XS'
,'S0012XD'
,'S0012XS'
,'S00201D'
,'S00201S'
,'S00202D'
,'S00202S'
,'S00209D'
,'S00209S'
,'S00211D'
,'S00211S'
,'S00212D'
,'S00212S'
,'S00219D'
,'S00219S'
,'S00221D'
,'S00221S'
,'S00222D'
,'S00222S'
,'S00229D'
,'S00229S'
,'S00241D'
,'S00241S'
,'S00242D'
,'S00242S'
,'S00249D'
,'S00249S'
,'S00251D'
,'S00251S'
,'S00252D'
,'S00252S'
,'S00259D'
,'S00259S'
,'S00261D'
,'S00261S'
,'S00262D'
,'S00262S'
,'S00269D'
,'S00269S'
,'S00271D'
,'S00271S'
,'S00272D'
,'S00272S'
,'S00279D'
,'S00279S'
,'S0030XD'
,'S0030XS'
,'S0031XD'
,'S0031XS'
,'S0032XD'
,'S0032XS'
,'S0033XD'
,'S0033XS'
,'S0034XD'
,'S0034XS'
,'S0035XD'
,'S0035XS'
,'S0036XD'
,'S0036XS'
,'S0037XD'
,'S0037XS'
,'S00401D'
,'S00401S'
,'S00402D'
,'S00402S'
,'S00409D'
,'S00409S'
,'S00411D'
,'S00411S'
,'S00412D'
,'S00412S'
,'S00419D'
,'S00419S'
,'S00421D'
,'S00421S'
,'S00422D'
,'S00422S'
,'S00429D'
,'S00429S'
,'S00431D'
,'S00431S'
,'S00432D'
,'S00432S'
,'S00439D'
,'S00439S'
,'S00441D'
,'S00441S'
,'S00442D'
,'S00442S'
,'S00449D'
,'S00449S'
,'S00451D'
,'S00451S'
,'S00452D'
,'S00452S'
,'S00459D'
,'S00459S'
,'S00461D'
,'S00461S'
,'S00462D'
,'S00462S'
,'S00469D'
,'S00469S'
,'S00471D'
,'S00471S'
,'S00472D'
,'S00472S'
,'S00479D'
,'S00479S'
,'S00501D'
,'S00501S'
,'S00502D'
,'S00502S'
,'S00511D'
,'S00511S'
,'S00512D'
,'S00512S'
,'S00521D'
,'S00521S'
,'S00522D'
,'S00522S'
,'S00531D'
,'S00531S'
,'S00532D'
,'S00532S'
,'S00541D'
,'S00541S'
,'S00542D'
,'S00542S'
,'S00551D'
,'S00551S'
,'S00552D'
,'S00552S'
,'S00561D'
,'S00561S'
,'S00562D'
,'S00562S'
,'S00571D'
,'S00571S'
,'S00572D'
,'S00572S'
,'S0080XD'
,'S0080XS'
,'S0081XD'
,'S0081XS'
,'S0082XD'
,'S0082XS'
,'S0083XD'
,'S0083XS'
,'S0084XD'
,'S0084XS'
,'S0085XD'
,'S0085XS'
,'S0086XD'
,'S0086XS'
,'S0087XD'
,'S0087XS'
,'S0090XD'
,'S0090XS'
,'S0091XD'
,'S0091XS'
,'S0092XD'
,'S0092XS'
,'S0093XD'
,'S0093XS'
,'S0094XD'
,'S0094XS'
,'S0095XD'
,'S0095XS'
,'S0096XD'
,'S0096XS'
,'S0097XD'
,'S0097XS'
,'S0100XD'
,'S0100XS'
,'S0101XD'
,'S0101XS'
,'S0102XD'
,'S0102XS'
,'S0103XD'
,'S0103XS'
,'S0104XD'
,'S0104XS'
,'S0105XD'
,'S0105XS'
,'S01101D'
,'S01101S'
,'S01102D'
,'S01102S'
,'S01109D'
,'S01109S'
,'S01111D'
,'S01111S'
,'S01112D'
,'S01112S'
,'S01119D'
,'S01119S'
,'S01121D'
,'S01121S'
,'S01122D'
,'S01122S'
,'S01129D'
,'S01129S'
,'S01131D'
,'S01131S'
,'S01132D'
,'S01132S'
,'S01139D'
,'S01139S'
,'S01141D'
,'S01141S'
,'S01142D'
,'S01142S'
,'S01149D'
,'S01149S'
,'S01151D'
,'S01151S'
,'S01152D'
,'S01152S'
,'S01159D'
,'S01159S'
,'S0120XD'
,'S0120XS'
,'S0121XD'
,'S0121XS'
,'S0122XD'
,'S0122XS'
,'S0123XD'
,'S0123XS'
,'S0124XD'
,'S0124XS'
,'S0125XD'
,'S0125XS'
,'S01301D'
,'S01301S'
,'S01302D'
,'S01302S'
,'S01309D'
,'S01309S'
,'S01311D'
,'S01311S'
,'S01312D'
,'S01312S'
,'S01319D'
,'S01319S'
,'S01321D'
,'S01321S'
,'S01322D'
,'S01322S'
,'S01329D'
,'S01329S'
,'S01331D'
,'S01331S'
,'S01332D'
,'S01332S'
,'S01339D'
,'S01339S'
,'S01341D'
,'S01341S'
,'S01342D'
,'S01342S'
,'S01349D'
,'S01349S'
,'S01351D'
,'S01351S'
,'S01352D'
,'S01352S'
,'S01359D'
,'S01359S'
,'S01401D'
,'S01401S'
,'S01402D'
,'S01402S'
,'S01409D'
,'S01409S'
,'S01411D'
,'S01411S'
,'S01412D'
,'S01412S'
,'S01419D'
,'S01419S'
,'S01421D'
,'S01421S'
,'S01422D'
,'S01422S'
,'S01429D'
,'S01429S'
,'S01431D'
,'S01431S'
,'S01432D'
,'S01432S'
,'S01439D'
,'S01439S'
,'S01441D'
,'S01441S'
,'S01442D'
,'S01442S'
,'S01449D'
,'S01449S'
,'S01451D'
,'S01451S'
,'S01452D'
,'S01452S'
,'S01459D'
,'S01459S'
,'S01501D'
,'S01501S'
,'S01502D'
,'S01502S'
,'S01511D'
,'S01511S'
,'S01512D'
,'S01512S'
,'S01521D'
,'S01521S'
,'S01522D'
,'S01522S'
,'S01531D'
,'S01531S'
,'S01532D'
,'S01532S'
,'S01541D'
,'S01541S'
,'S01542D'
,'S01542S'
,'S01551D'
,'S01551S'
,'S01552D'
,'S01552S'
,'S0180XD'
,'S0180XS'
,'S0181XD'
,'S0181XS'
,'S0182XD'
,'S0182XS'
,'S0183XD'
,'S0183XS'
,'S0184XD'
,'S0184XS'
,'S0185XD'
,'S0185XS'
,'S0190XD'
,'S0190XS'
,'S0191XD'
,'S0191XS'
,'S0192XD'
,'S0192XS'
,'S0193XD'
,'S0193XS'
,'S0194XD'
,'S0194XS'
,'S0195XD'
,'S0195XS'
,'S020XXD'
,'S020XXG'
,'S020XXK'
,'S020XXS'
,'S0210XD'
,'S0210XG'
,'S0210XK'
,'S0210XS'
,'S02110D'
,'S02110G'
,'S02110K'
,'S02110S'
,'S02111D'
,'S02111G'
,'S02111K'
,'S02111S'
,'S02112D'
,'S02112G'
,'S02112K'
,'S02112S'
,'S02113D'
,'S02113G'
,'S02113K'
,'S02113S'
,'S02118D'
,'S02118G'
,'S02118K'
,'S02118S'
,'S02119D'
,'S02119G'
,'S02119K'
,'S02119S'
,'S0219XD'
,'S0219XG'
,'S0219XK'
,'S0219XS'
,'S022XXD'
,'S022XXG'
,'S022XXK'
,'S022XXS'
,'S023XXD'
,'S023XXG'
,'S023XXK'
,'S023XXS'
,'S02400D'
,'S02400G'
,'S02400K'
,'S02400S'
,'S02401D'
,'S02401G'
,'S02401K'
,'S02401S'
,'S02402D'
,'S02402G'
,'S02402K'
,'S02402S'
,'S02411D'
,'S02411G'
,'S02411K'
,'S02411S'
,'S02412D'
,'S02412G'
,'S02412K'
,'S02412S'
,'S02413D'
,'S02413G'
,'S02413K'
,'S02413S'
,'S0242XD'
,'S0242XG'
,'S0242XK'
,'S0242XS'
,'S025XXD'
,'S025XXG'
,'S025XXK'
,'S025XXS'
,'S02600D'
,'S02600G'
,'S02600K'
,'S02600S'
,'S02609D'
,'S02609G'
,'S02609K'
,'S02609S'
,'S0261XD'
,'S0261XG'
,'S0261XK'
,'S0261XS'
,'S0262XD'
,'S0262XG'
,'S0262XK'
,'S0262XS'
,'S0263XD'
,'S0263XG'
,'S0263XK'
,'S0263XS'
,'S0264XD'
,'S0264XG'
,'S0264XK'
,'S0264XS'
,'S0265XD'
,'S0265XG'
,'S0265XK'
,'S0265XS'
,'S0266XD'
,'S0266XG'
,'S0266XK'
,'S0266XS'
,'S0267XD'
,'S0267XG'
,'S0267XK'
,'S0267XS'
,'S0269XD'
,'S0269XG'
,'S0269XK'
,'S0269XS'
,'S028XXD'
,'S028XXG'
,'S028XXK'
,'S028XXS'
,'S0291XD'
,'S0291XG'
,'S0291XK'
,'S0291XS'
,'S0292XD'
,'S0292XG'
,'S0292XK'
,'S0292XS'
,'S030XXD'
,'S030XXS'
,'S031XXD'
,'S031XXS'
,'S032XXD'
,'S032XXS'
,'S034XXD'
,'S034XXS'
,'S038XXD'
,'S038XXS'
,'S039XXD'
,'S039XXS'
,'S04011D'
,'S04011S'
,'S04012D'
,'S04012S'
,'S04019D'
,'S04019S'
,'S0402XD'
,'S0402XS'
,'S04031D'
,'S04031S'
,'S04032D'
,'S04032S'
,'S04039D'
,'S04039S'
,'S04041D'
,'S04041S'
,'S04042D'
,'S04042S'
,'S04049D'
,'S04049S'
,'S0410XD'
,'S0410XS'
,'S0411XD'
,'S0411XS'
,'S0412XD'
,'S0412XS'
,'S0420XD'
,'S0420XS'
,'S0421XD'
,'S0421XS'
,'S0422XD'
,'S0422XS'
,'S0430XD'
,'S0430XS'
,'S0431XD'
,'S0431XS'
,'S0432XD'
,'S0432XS'
,'S0440XD'
,'S0440XS'
,'S0441XD'
,'S0441XS'
,'S0442XD'
,'S0442XS'
,'S0450XD'
,'S0450XS'
,'S0451XD'
,'S0451XS'
,'S0452XD'
,'S0452XS'
,'S0460XD'
,'S0460XS'
,'S0461XD'
,'S0461XS'
,'S0462XD'
,'S0462XS'
,'S0470XD'
,'S0470XS'
,'S0471XD'
,'S0471XS'
,'S0472XD'
,'S0472XS'
,'S04811D'
,'S04811S'
,'S04812D'
,'S04812S'
,'S04819D'
,'S04819S'
,'S04891D'
,'S04891S'
,'S04892D'
,'S04892S'
,'S04899D'
,'S04899S'
,'S049XXD'
,'S049XXS'
,'S0500XD'
,'S0500XS'
,'S0501XD'
,'S0501XS'
,'S0502XD'
,'S0502XS'
,'S0510XD'
,'S0510XS'
,'S0511XD'
,'S0511XS'
,'S0512XD'
,'S0512XS'
,'S0520XD'
,'S0520XS'
,'S0521XD'
,'S0521XS'
,'S0522XD'
,'S0522XS'
,'S0530XD'
,'S0530XS'
,'S0531XD'
,'S0531XS'
,'S0532XD'
,'S0532XS'
,'S0540XD'
,'S0540XS'
,'S0541XD'
,'S0541XS'
,'S0542XD'
,'S0542XS'
,'S0550XD'
,'S0550XS'
,'S0551XD'
,'S0551XS'
,'S0552XD'
,'S0552XS'
,'S0560XD'
,'S0560XS'
,'S0561XD'
,'S0561XS'
,'S0562XD'
,'S0562XS'
,'S0570XD'
,'S0570XS'
,'S0571XD'
,'S0571XS'
,'S0572XD'
,'S0572XS'
,'S058X1D'
,'S058X1S'
,'S058X2D'
,'S058X2S'
,'S058X9D'
,'S058X9S'
,'S0590XD'
,'S0590XS'
,'S0591XD'
,'S0591XS'
,'S0592XD'
,'S0592XS'
,'S060X0D'
,'S060X0S'
,'S060X1D'
,'S060X1S'
,'S060X2D'
,'S060X2S'
,'S060X3D'
,'S060X3S'
,'S060X4D'
,'S060X4S'
,'S060X5D'
,'S060X5S'
,'S060X6D'
,'S060X6S'
,'S060X7D'
,'S060X7S'
,'S060X8D'
,'S060X8S'
,'S060X9D'
,'S060X9S'
,'S061X0D'
,'S061X0S'
,'S061X1D'
,'S061X1S'
,'S061X2D'
,'S061X2S'
,'S061X3D'
,'S061X3S'
,'S061X4D'
,'S061X4S'
,'S061X5D'
,'S061X5S'
,'S061X6D'
,'S061X6S'
,'S061X7D'
,'S061X7S'
,'S061X8D'
,'S061X8S'
,'S061X9D'
,'S061X9S'
,'S062X0D'
,'S062X0S'
,'S062X1D'
,'S062X1S'
,'S062X2D'
,'S062X2S'
,'S062X3D'
,'S062X3S'
,'S062X4D'
,'S062X4S'
,'S062X5D'
,'S062X5S'
,'S062X6D'
,'S062X6S'
,'S062X7D'
,'S062X7S'
,'S062X8D'
,'S062X8S'
,'S062X9D'
,'S062X9S'
,'S06300D'
,'S06300S'
,'S06301D'
,'S06301S'
,'S06302D'
,'S06302S'
,'S06303D'
,'S06303S'
,'S06304D'
,'S06304S'
,'S06305D'
,'S06305S'
,'S06306D'
,'S06306S'
,'S06307D'
,'S06307S'
,'S06308D'
,'S06308S'
,'S06309D'
,'S06309S'
,'S06310D'
,'S06310S'
,'S06311D'
,'S06311S'
,'S06312D'
,'S06312S'
,'S06313D'
,'S06313S'
,'S06314D'
,'S06314S'
,'S06315D'
,'S06315S'
,'S06316D'
,'S06316S'
,'S06317D'
,'S06317S'
,'S06318D'
,'S06318S'
,'S06319D'
,'S06319S'
,'S06320D'
,'S06320S'
,'S06321D'
,'S06321S'
,'S06322D'
,'S06322S'
,'S06323D'
,'S06323S'
,'S06324D'
,'S06324S'
,'S06325D'
,'S06325S'
,'S06326D'
,'S06326S'
,'S06327D'
,'S06327S'
,'S06328D'
,'S06328S'
,'S06329D'
,'S06329S'
,'S06330D'
,'S06330S'
,'S06331D'
,'S06331S'
,'S06332D'
,'S06332S'
,'S06333D'
,'S06333S'
,'S06334D'
,'S06334S'
,'S06335D'
,'S06335S'
,'S06336D'
,'S06336S'
,'S06337D'
,'S06337S'
,'S06338D'
,'S06338S'
,'S06339D'
,'S06339S'
,'S06340D'
,'S06340S'
,'S06341D'
,'S06341S'
,'S06342D'
,'S06342S'
,'S06343D'
,'S06343S'
,'S06344D'
,'S06344S'
,'S06345D'
,'S06345S'
,'S06346D'
,'S06346S'
,'S06347D'
,'S06347S'
,'S06348D'
,'S06348S'
,'S06349D'
,'S06349S'
,'S06350D'
,'S06350S'
,'S06351D'
,'S06351S'
,'S06352D'
,'S06352S'
,'S06353D'
,'S06353S'
,'S06354D'
,'S06354S'
,'S06355D'
,'S06355S'
,'S06356D'
,'S06356S'
,'S06357D'
,'S06357S'
,'S06358D'
,'S06358S'
,'S06359D'
,'S06359S'
,'S06360D'
,'S06360S'
,'S06361D'
,'S06361S'
,'S06362D'
,'S06362S'
,'S06363D'
,'S06363S'
,'S06364D'
,'S06364S'
,'S06365D'
,'S06365S'
,'S06366D'
,'S06366S'
,'S06367D'
,'S06367S'
,'S06368D'
,'S06368S'
,'S06369D'
,'S06369S'
,'S06370D'
,'S06370S'
,'S06371D'
,'S06371S'
,'S06372D'
,'S06372S'
,'S06373D'
,'S06373S'
,'S06374D'
,'S06374S'
,'S06375D'
,'S06375S'
,'S06376D'
,'S06376S'
,'S06377D'
,'S06377S'
,'S06378D'
,'S06378S'
,'S06379D'
,'S06379S'
,'S06380D'
,'S06380S'
,'S06381D'
,'S06381S'
,'S06382D'
,'S06382S'
,'S06383D'
,'S06383S'
,'S06384D'
,'S06384S'
,'S06385D'
,'S06385S'
,'S06386D'
,'S06386S'
,'S06387D'
,'S06387S'
,'S06388D'
,'S06388S'
,'S06389D'
,'S06389S'
,'S064X0D'
,'S064X0S'
,'S064X1D'
,'S064X1S'
,'S064X2D'
,'S064X2S'
,'S064X3D'
,'S064X3S'
,'S064X4D'
,'S064X4S'
,'S064X5D'
,'S064X5S'
,'S064X6D'
,'S064X6S'
,'S064X7D'
,'S064X7S'
,'S064X8D'
,'S064X8S'
,'S064X9D'
,'S064X9S'
,'S065X0D'
,'S065X0S'
,'S065X1D'
,'S065X1S'
,'S065X2D'
,'S065X2S'
,'S065X3D'
,'S065X3S'
,'S065X4D'
,'S065X4S'
,'S065X5D'
,'S065X5S'
,'S065X6D'
,'S065X6S'
,'S065X7D'
,'S065X7S'
,'S065X8D'
,'S065X8S'
,'S065X9D'
,'S065X9S'
,'S066X0D'
,'S066X0S'
,'S066X1D'
,'S066X1S'
,'S066X2D'
,'S066X2S'
,'S066X3D'
,'S066X3S'
,'S066X4D'
,'S066X4S'
,'S066X5D'
,'S066X5S'
,'S066X6D'
,'S066X6S'
,'S066X7D'
,'S066X7S'
,'S066X8D'
,'S066X8S'
,'S066X9D'
,'S066X9S'
,'S06810D'
,'S06810S'
,'S06811D'
,'S06811S'
,'S06812D'
,'S06812S'
,'S06813D'
,'S06813S'
,'S06814D'
,'S06814S'
,'S06815D'
,'S06815S'
,'S06816D'
,'S06816S'
,'S06817D'
,'S06817S'
,'S06818D'
,'S06818S'
,'S06819D'
,'S06819S'
,'S06820D'
,'S06820S'
,'S06821D'
,'S06821S'
,'S06822D'
,'S06822S'
,'S06823D'
,'S06823S'
,'S06824D'
,'S06824S'
,'S06825D'
,'S06825S'
,'S06826D'
,'S06826S'
,'S06827D'
,'S06827S'
,'S06828D'
,'S06828S'
,'S06829D'
,'S06829S'
,'S06890D'
,'S06890S'
,'S06891D'
,'S06891S'
,'S06892D'
,'S06892S'
,'S06893D'
,'S06893S'
,'S06894D'
,'S06894S'
,'S06895D'
,'S06895S'
,'S06896D'
,'S06896S'
,'S06897D'
,'S06897S'
,'S06898D'
,'S06898S'
,'S06899D'
,'S06899S'
,'S069X0D'
,'S069X0S'
,'S069X1D'
,'S069X1S'
,'S069X2D'
,'S069X2S'
,'S069X3D'
,'S069X3S'
,'S069X4D'
,'S069X4S'
,'S069X5D'
,'S069X5S'
,'S069X6D'
,'S069X6S'
,'S069X7D'
,'S069X7S'
,'S069X8D'
,'S069X8S'
,'S069X9D'
,'S069X9S'
,'S070XXD'
,'S070XXS'
,'S071XXD'
,'S071XXS'
,'S078XXD'
,'S078XXS'
,'S079XXD'
,'S079XXS'
,'S080XXD'
,'S080XXS'
,'S08111D'
,'S08111S'
,'S08112D'
,'S08112S'
,'S08119D'
,'S08119S'
,'S08121D'
,'S08121S'
,'S08122D'
,'S08122S'
,'S08129D'
,'S08129S'
,'S08811D'
,'S08811S'
,'S08812D'
,'S08812S'
,'S0889XD'
,'S0889XS'
,'S090XXD'
,'S090XXS'
,'S0910XD'
,'S0910XS'
,'S0911XD'
,'S0911XS'
,'S0912XD'
,'S0912XS'
,'S0919XD'
,'S0919XS'
,'S0920XD'
,'S0920XS'
,'S0921XD'
,'S0921XS'
,'S0922XD'
,'S0922XS'
,'S09301D'
,'S09301S'
,'S09302D'
,'S09302S'
,'S09309D'
,'S09309S'
,'S09311D'
,'S09311S'
,'S09312D'
,'S09312S'
,'S09313D'
,'S09313S'
,'S09319D'
,'S09319S'
,'S09391D'
,'S09391S'
,'S09392D'
,'S09392S'
,'S09399D'
,'S09399S'
,'S098XXD'
,'S098XXS'
,'S0990XD'
,'S0990XS'
,'S0991XD'
,'S0991XS'
,'S0992XD'
,'S0992XS'
,'S0993XD'
,'S0993XS'
,'S100XXD'
,'S100XXS'
,'S1010XD'
,'S1010XS'
,'S1011XD'
,'S1011XS'
,'S1012XD'
,'S1012XS'
,'S1014XD'
,'S1014XS'
,'S1015XD'
,'S1015XS'
,'S1016XD'
,'S1016XS'
,'S1017XD'
,'S1017XS'
,'S1080XD'
,'S1080XS'
,'S1081XD'
,'S1081XS'
,'S1082XD'
,'S1082XS'
,'S1083XD'
,'S1083XS'
,'S1084XD'
,'S1084XS'
,'S1085XD'
,'S1085XS'
,'S1086XD'
,'S1086XS'
,'S1087XD'
,'S1087XS'
,'S1090XD'
,'S1090XS'
,'S1091XD'
,'S1091XS'
,'S1092XD'
,'S1092XS'
,'S1093XD'
,'S1093XS'
,'S1094XD'
,'S1094XS'
,'S1095XD'
,'S1095XS'
,'S1096XD'
,'S1096XS'
,'S1097XD'
,'S1097XS'
,'S11011D'
,'S11011S'
,'S11012D'
,'S11012S'
,'S11013D'
,'S11013S'
,'S11014D'
,'S11014S'
,'S11015D'
,'S11015S'
,'S11019D'
,'S11019S'
,'S11021D'
,'S11021S'
,'S11022D'
,'S11022S'
,'S11023D'
,'S11023S'
,'S11024D'
,'S11024S'
,'S11025D'
,'S11025S'
,'S11029D'
,'S11029S'
,'S11031D'
,'S11031S'
,'S11032D'
,'S11032S'
,'S11033D'
,'S11033S'
,'S11034D'
,'S11034S'
,'S11035D'
,'S11035S'
,'S11039D'
,'S11039S'
,'S1110XD'
,'S1110XS'
,'S1111XD'
,'S1111XS'
,'S1112XD'
,'S1112XS'
,'S1113XD'
,'S1113XS'
,'S1114XD'
,'S1114XS'
,'S1115XD'
,'S1115XS'
,'S1120XD'
,'S1120XS'
,'S1121XD'
,'S1121XS'
,'S1122XD'
,'S1122XS'
,'S1123XD'
,'S1123XS'
,'S1124XD'
,'S1124XS'
,'S1125XD'
,'S1125XS'
,'S1180XD'
,'S1180XS'
,'S1181XD'
,'S1181XS'
,'S1182XD'
,'S1182XS'
,'S1183XD'
,'S1183XS'
,'S1184XD'
,'S1184XS'
,'S1185XD'
,'S1185XS'
,'S1189XD'
,'S1189XS'
,'S1190XD'
,'S1190XS'
,'S1191XD'
,'S1191XS'
,'S1192XD'
,'S1192XS'
,'S1193XD'
,'S1193XS'
,'S1194XD'
,'S1194XS'
,'S1195XD'
,'S1195XS'
,'S12000D'
,'S12000G'
,'S12000K'
,'S12000S'
,'S12001D'
,'S12001G'
,'S12001K'
,'S12001S'
,'S1201XD'
,'S1201XG'
,'S1201XK'
,'S1201XS'
,'S1202XD'
,'S1202XG'
,'S1202XK'
,'S1202XS'
,'S12030D'
,'S12030G'
,'S12030K'
,'S12030S'
,'S12031D'
,'S12031G'
,'S12031K'
,'S12031S'
,'S12040D'
,'S12040G'
,'S12040K'
,'S12040S'
,'S12041D'
,'S12041G'
,'S12041K'
,'S12041S'
,'S12090D'
,'S12090G'
,'S12090K'
,'S12090S'
,'S12091D'
,'S12091G'
,'S12091K'
,'S12091S'
,'S12100D'
,'S12100G'
,'S12100K'
,'S12100S'
,'S12101D'
,'S12101G'
,'S12101K'
,'S12101S'
,'S12110D'
,'S12110G'
,'S12110K'
,'S12110S'
,'S12111D'
,'S12111G'
,'S12111K'
,'S12111S'
,'S12112D'
,'S12112G'
,'S12112K'
,'S12112S'
,'S12120D'
,'S12120G'
,'S12120K'
,'S12120S'
,'S12121D'
,'S12121G'
,'S12121K'
,'S12121S'
,'S12130D'
,'S12130G'
,'S12130K'
,'S12130S'
,'S12131D'
,'S12131G'
,'S12131K'
,'S12131S'
,'S1214XD'
,'S1214XG'
,'S1214XK'
,'S1214XS'
,'S12150D'
,'S12150G'
,'S12150K'
,'S12150S'
,'S12151D'
,'S12151G'
,'S12151K'
,'S12151S'
,'S12190D'
,'S12190G'
,'S12190K'
,'S12190S'
,'S12191D'
,'S12191G'
,'S12191K'
,'S12191S'
,'S12200D'
,'S12200G'
,'S12200K'
,'S12200S'
,'S12201D'
,'S12201G'
,'S12201K'
,'S12201S'
,'S12230D'
,'S12230G'
,'S12230K'
,'S12230S'
,'S12231D'
,'S12231G'
,'S12231K'
,'S12231S'
,'S1224XD'
,'S1224XG'
,'S1224XK'
,'S1224XS'
,'S12250D'
,'S12250G'
,'S12250K'
,'S12250S'
,'S12251D'
,'S12251G'
,'S12251K'
,'S12251S'
,'S12290D'
,'S12290G'
,'S12290K'
,'S12290S'
,'S12291D'
,'S12291G'
,'S12291K'
,'S12291S'
,'S12300D'
,'S12300G'
,'S12300K'
,'S12300S'
,'S12301D'
,'S12301G'
,'S12301K'
,'S12301S'
,'S12330D'
,'S12330G'
,'S12330K'
,'S12330S'
,'S12331D'
,'S12331G'
,'S12331K'
,'S12331S'
,'S1234XD'
,'S1234XG'
,'S1234XK'
,'S1234XS'
,'S12350D'
,'S12350G'
,'S12350K'
,'S12350S'
,'S12351D'
,'S12351G'
,'S12351K'
,'S12351S'
,'S12390D'
,'S12390G'
,'S12390K'
,'S12390S'
,'S12391D'
,'S12391G'
,'S12391K'
,'S12391S'
,'S12400D'
,'S12400G'
,'S12400K'
,'S12400S'
,'S12401D'
,'S12401G'
,'S12401K'
,'S12401S'
,'S12430D'
,'S12430G'
,'S12430K'
,'S12430S'
,'S12431D'
,'S12431G'
,'S12431K'
,'S12431S'
,'S1244XD'
,'S1244XG'
,'S1244XK'
,'S1244XS'
,'S12450D'
,'S12450G'
,'S12450K'
,'S12450S'
,'S12451D'
,'S12451G'
,'S12451K'
,'S12451S'
,'S12490D'
,'S12490G'
,'S12490K'
,'S12490S'
,'S12491D'
,'S12491G'
,'S12491K'
,'S12491S'
,'S12500D'
,'S12500G'
,'S12500K'
,'S12500S'
,'S12501D'
,'S12501G'
,'S12501K'
,'S12501S'
,'S12530D'
,'S12530G'
,'S12530K'
,'S12530S'
,'S12531D'
,'S12531G'
,'S12531K'
,'S12531S'
,'S1254XD'
,'S1254XG'
,'S1254XK'
,'S1254XS'
,'S12550D'
,'S12550G'
,'S12550K'
,'S12550S'
,'S12551D'
,'S12551G'
,'S12551K'
,'S12551S'
,'S12590D'
,'S12590G'
,'S12590K'
,'S12590S'
,'S12591D'
,'S12591G'
,'S12591K'
,'S12591S'
,'S12600D'
,'S12600G'
,'S12600K'
,'S12600S'
,'S12601D'
,'S12601G'
,'S12601K'
,'S12601S'
,'S12630D'
,'S12630G'
,'S12630K'
,'S12630S'
,'S12631D'
,'S12631G'
,'S12631K'
,'S12631S'
,'S1264XD'
,'S1264XG'
,'S1264XK'
,'S1264XS'
,'S12650D'
,'S12650G'
,'S12650K'
,'S12650S'
,'S12651D'
,'S12651G'
,'S12651K'
,'S12651S'
,'S12690D'
,'S12690G'
,'S12690K'
,'S12690S'
,'S12691D'
,'S12691G'
,'S12691K'
,'S12691S'
,'S128XXD'
,'S128XXS'
,'S129XXD'
,'S129XXS'
,'S130XXD'
,'S130XXS'
,'S13100D'
,'S13100S'
,'S13101D'
,'S13101S'
,'S13110D'
,'S13110S'
,'S13111D'
,'S13111S'
,'S13120D'
,'S13120S'
,'S13121D'
,'S13121S'
,'S13130D'
,'S13130S'
,'S13131D'
,'S13131S'
,'S13140D'
,'S13140S'
,'S13141D'
,'S13141S'
,'S13150D'
,'S13150S'
,'S13151D'
,'S13151S'
,'S13160D'
,'S13160S'
,'S13161D'
,'S13161S'
,'S13170D'
,'S13170S'
,'S13171D'
,'S13171S'
,'S13180D'
,'S13180S'
,'S13181D'
,'S13181S'
,'S1320XD'
,'S1320XS'
,'S1329XD'
,'S1329XS'
,'S134XXD'
,'S134XXS'
,'S135XXD'
,'S135XXS'
,'S138XXD'
,'S138XXS'
,'S139XXD'
,'S139XXS'
,'S140XXD'
,'S140XXS'
,'S14101D'
,'S14101S'
,'S14102D'
,'S14102S'
,'S14103D'
,'S14103S'
,'S14104D'
,'S14104S'
,'S14105D'
,'S14105S'
,'S14106D'
,'S14106S'
,'S14107D'
,'S14107S'
,'S14108D'
,'S14108S'
,'S14109D'
,'S14109S'
,'S14111D'
,'S14111S'
,'S14112D'
,'S14112S'
,'S14113D'
,'S14113S'
,'S14114D'
,'S14114S'
,'S14115D'
,'S14115S'
,'S14116D'
,'S14116S'
,'S14117D'
,'S14117S'
,'S14118D'
,'S14118S'
,'S14119D'
,'S14119S'
,'S14121D'
,'S14121S'
,'S14122D'
,'S14122S'
,'S14123D'
,'S14123S'
,'S14124D'
,'S14124S'
,'S14125D'
,'S14125S'
,'S14126D'
,'S14126S'
,'S14127D'
,'S14127S'
,'S14128D'
,'S14128S'
,'S14129D'
,'S14129S'
,'S14131D'
,'S14131S'
,'S14132D'
,'S14132S'
,'S14133D'
,'S14133S'
,'S14134D'
,'S14134S'
,'S14135D'
,'S14135S'
,'S14136D'
,'S14136S'
,'S14137D'
,'S14137S'
,'S14138D'
,'S14138S'
,'S14139D'
,'S14139S'
,'S14141D'
,'S14141S'
,'S14142D'
,'S14142S'
,'S14143D'
,'S14143S'
,'S14144D'
,'S14144S'
,'S14145D'
,'S14145S'
,'S14146D'
,'S14146S'
,'S14147D'
,'S14147S'
,'S14148D'
,'S14148S'
,'S14149D'
,'S14149S'
,'S14151D'
,'S14151S'
,'S14152D'
,'S14152S'
,'S14153D'
,'S14153S'
,'S14154D'
,'S14154S'
,'S14155D'
,'S14155S'
,'S14156D'
,'S14156S'
,'S14157D'
,'S14157S'
,'S14158D'
,'S14158S'
,'S14159D'
,'S14159S'
,'S142XXD'
,'S142XXS'
,'S143XXD'
,'S143XXS'
,'S144XXD'
,'S144XXS'
,'S145XXD'
,'S145XXS'
,'S148XXD'
,'S148XXS'
,'S149XXD'
,'S149XXS'
,'S15001D'
,'S15001S'
,'S15002D'
,'S15002S'
,'S15009D'
,'S15009S'
,'S15011D'
,'S15011S'
,'S15012D'
,'S15012S'
,'S15019D'
,'S15019S'
,'S15021D'
,'S15021S'
,'S15022D'
,'S15022S'
,'S15029D'
,'S15029S'
,'S15091D'
,'S15091S'
,'S15092D'
,'S15092S'
,'S15099D'
,'S15099S'
,'S15101D'
,'S15101S'
,'S15102D'
,'S15102S'
,'S15109D'
,'S15109S'
,'S15111D'
,'S15111S'
,'S15112D'
,'S15112S'
,'S15119D'
,'S15119S'
,'S15121D'
,'S15121S'
,'S15122D'
,'S15122S'
,'S15129D'
,'S15129S'
,'S15191D'
,'S15191S'
,'S15192D'
,'S15192S'
,'S15199D'
,'S15199S'
,'S15201D'
,'S15201S'
,'S15202D'
,'S15202S'
,'S15209D'
,'S15209S'
,'S15211D'
,'S15211S'
,'S15212D'
,'S15212S'
,'S15219D'
,'S15219S'
,'S15221D'
,'S15221S'
,'S15222D'
,'S15222S'
,'S15229D'
,'S15229S'
,'S15291D'
,'S15291S'
,'S15292D'
,'S15292S'
,'S15299D'
,'S15299S'
,'S15301D'
,'S15301S'
,'S15302D'
,'S15302S'
,'S15309D'
,'S15309S'
,'S15311D'
,'S15311S'
,'S15312D'
,'S15312S'
,'S15319D'
,'S15319S'
,'S15321D'
,'S15321S'
,'S15322D'
,'S15322S'
,'S15329D'
,'S15329S'
,'S15391D'
,'S15391S'
,'S15392D'
,'S15392S'
,'S15399D'
,'S15399S'
,'S158XXD'
,'S158XXS'
,'S159XXD'
,'S159XXS'
,'S161XXD'
,'S161XXS'
,'S162XXD'
,'S162XXS'
,'S168XXD'
,'S168XXS'
,'S169XXD'
,'S169XXS'
,'S170XXD'
,'S170XXS'
,'S178XXD'
,'S178XXS'
,'S179XXD'
,'S179XXS'
,'S1980XD'
,'S1980XS'
,'S1981XD'
,'S1981XS'
,'S1982XD'
,'S1982XS'
,'S1983XD'
,'S1983XS'
,'S1984XD'
,'S1984XS'
,'S1985XD'
,'S1985XS'
,'S1989XD'
,'S1989XS'
,'S199XXD'
,'S199XXS'
,'S2000XD'
,'S2000XS'
,'S2001XD'
,'S2001XS'
,'S2002XD'
,'S2002XS'
,'S20101D'
,'S20101S'
,'S20102D'
,'S20102S'
,'S20109D'
,'S20109S'
,'S20111D'
,'S20111S'
,'S20112D'
,'S20112S'
,'S20119D'
,'S20119S'
,'S20121D'
,'S20121S'
,'S20122D'
,'S20122S'
,'S20129D'
,'S20129S'
,'S20141D'
,'S20141S'
,'S20142D'
,'S20142S'
,'S20149D'
,'S20149S'
,'S20151D'
,'S20151S'
,'S20152D'
,'S20152S'
,'S20159D'
,'S20159S'
,'S20161D'
,'S20161S'
,'S20162D'
,'S20162S'
,'S20169D'
,'S20169S'
,'S20171D'
,'S20171S'
,'S20172D'
,'S20172S'
,'S20179D'
,'S20179S'
,'S2020XD'
,'S2020XS'
,'S20211D'
,'S20211S'
,'S20212D'
,'S20212S'
,'S20219D'
,'S20219S'
,'S20221D'
,'S20221S'
,'S20222D'
,'S20222S'
,'S20229D'
,'S20229S'
,'S20301D'
,'S20301S'
,'S20302D'
,'S20302S'
,'S20309D'
,'S20309S'
,'S20311D'
,'S20311S'
,'S20312D'
,'S20312S'
,'S20319D'
,'S20319S'
,'S20321D'
,'S20321S'
,'S20322D'
,'S20322S'
,'S20329D'
,'S20329S'
,'S20341D'
,'S20341S'
,'S20342D'
,'S20342S'
,'S20349D'
,'S20349S'
,'S20351D'
,'S20351S'
,'S20352D'
,'S20352S'
,'S20359D'
,'S20359S'
,'S20361D'
,'S20361S'
,'S20362D'
,'S20362S'
,'S20369D'
,'S20369S'
,'S20371D'
,'S20371S'
,'S20372D'
,'S20372S'
,'S20379D'
,'S20379S'
,'S20401D'
,'S20401S'
,'S20402D'
,'S20402S'
,'S20409D'
,'S20409S'
,'S20411D'
,'S20411S'
,'S20412D'
,'S20412S'
,'S20419D'
,'S20419S'
,'S20421D'
,'S20421S'
,'S20422D'
,'S20422S'
,'S20429D'
,'S20429S'
,'S20441D'
,'S20441S'
,'S20442D'
,'S20442S'
,'S20449D'
,'S20449S'
,'S20451D'
,'S20451S'
,'S20452D'
,'S20452S'
,'S20459D'
,'S20459S'
,'S20461D'
,'S20461S'
,'S20462D'
,'S20462S'
,'S20469D'
,'S20469S'
,'S20471D'
,'S20471S'
,'S20472D'
,'S20472S'
,'S20479D'
,'S20479S'
,'S2090XD'
,'S2090XS'
,'S2091XD'
,'S2091XS'
,'S2092XD'
,'S2092XS'
,'S2094XD'
,'S2094XS'
,'S2095XD'
,'S2095XS'
,'S2096XD'
,'S2096XS'
,'S2097XD'
,'S2097XS'
,'S21001D'
,'S21001S'
,'S21002D'
,'S21002S'
,'S21009D'
,'S21009S'
,'S21011D'
,'S21011S'
,'S21012D'
,'S21012S'
,'S21019D'
,'S21019S'
,'S21021D'
,'S21021S'
,'S21022D'
,'S21022S'
,'S21029D'
,'S21029S'
,'S21031D'
,'S21031S'
,'S21032D'
,'S21032S'
,'S21039D'
,'S21039S'
,'S21041D'
,'S21041S'
,'S21042D'
,'S21042S'
,'S21049D'
,'S21049S'
,'S21051D'
,'S21051S'
,'S21052D'
,'S21052S'
,'S21059D'
,'S21059S'
,'S21101D'
,'S21101S'
,'S21102D'
,'S21102S'
,'S21109D'
,'S21109S'
,'S21111D'
,'S21111S'
,'S21112D'
,'S21112S'
,'S21119D'
,'S21119S'
,'S21121D'
,'S21121S'
,'S21122D'
,'S21122S'
,'S21129D'
,'S21129S'
,'S21131D'
,'S21131S'
,'S21132D'
,'S21132S'
,'S21139D'
,'S21139S'
,'S21141D'
,'S21141S'
,'S21142D'
,'S21142S'
,'S21149D'
,'S21149S'
,'S21151D'
,'S21151S'
,'S21152D'
,'S21152S'
,'S21159D'
,'S21159S'
,'S21201D'
,'S21201S'
,'S21202D'
,'S21202S'
,'S21209D'
,'S21209S'
,'S21211D'
,'S21211S'
,'S21212D'
,'S21212S'
,'S21219D'
,'S21219S'
,'S21221D'
,'S21221S'
,'S21222D'
,'S21222S'
,'S21229D'
,'S21229S'
,'S21231D'
,'S21231S'
,'S21232D'
,'S21232S'
,'S21239D'
,'S21239S'
,'S21241D'
,'S21241S'
,'S21242D'
,'S21242S'
,'S21249D'
,'S21249S'
,'S21251D'
,'S21251S'
,'S21252D'
,'S21252S'
,'S21259D'
,'S21259S'
,'S21301D'
,'S21301S'
,'S21302D'
,'S21302S'
,'S21309D'
,'S21309S'
,'S21311D'
,'S21311S'
,'S21312D'
,'S21312S'
,'S21319D'
,'S21319S'
,'S21321D'
,'S21321S'
,'S21322D'
,'S21322S'
,'S21329D'
,'S21329S'
,'S21331D'
,'S21331S'
,'S21332D'
,'S21332S'
,'S21339D'
,'S21339S'
,'S21341D'
,'S21341S'
,'S21342D'
,'S21342S'
,'S21349D'
,'S21349S'
,'S21351D'
,'S21351S'
,'S21352D'
,'S21352S'
,'S21359D'
,'S21359S'
,'S21401D'
,'S21401S'
,'S21402D'
,'S21402S'
,'S21409D'
,'S21409S'
,'S21411D'
,'S21411S'
,'S21412D'
,'S21412S'
,'S21419D'
,'S21419S'
,'S21421D'
,'S21421S'
,'S21422D'
,'S21422S'
,'S21429D'
,'S21429S'
,'S21431D'
,'S21431S'
,'S21432D'
,'S21432S'
,'S21439D'
,'S21439S'
,'S21441D'
,'S21441S'
,'S21442D'
,'S21442S'
,'S21449D'
,'S21449S'
,'S21451D'
,'S21451S'
,'S21452D'
,'S21452S'
,'S21459D'
,'S21459S'
,'S2190XD'
,'S2190XS'
,'S2191XD'
,'S2191XS'
,'S2192XD'
,'S2192XS'
,'S2193XD'
,'S2193XS'
,'S2194XD'
,'S2194XS'
,'S2195XD'
,'S2195XS'
,'S22000D'
,'S22000G'
,'S22000K'
,'S22000S'
,'S22001D'
,'S22001G'
,'S22001K'
,'S22001S'
,'S22002D'
,'S22002G'
,'S22002K'
,'S22002S'
,'S22008D'
,'S22008G'
,'S22008K'
,'S22008S'
,'S22009D'
,'S22009G'
,'S22009K'
,'S22009S'
,'S22010D'
,'S22010G'
,'S22010K'
,'S22010S'
,'S22011D'
,'S22011G'
,'S22011K'
,'S22011S'
,'S22012D'
,'S22012G'
,'S22012K'
,'S22012S'
,'S22018D'
,'S22018G'
,'S22018K'
,'S22018S'
,'S22019D'
,'S22019G'
,'S22019K'
,'S22019S'
,'S22020D'
,'S22020G'
,'S22020K'
,'S22020S'
,'S22021D'
,'S22021G'
,'S22021K'
,'S22021S'
,'S22022D'
,'S22022G'
,'S22022K'
,'S22022S'
,'S22028D'
,'S22028G'
,'S22028K'
,'S22028S'
,'S22029D'
,'S22029G'
,'S22029K'
,'S22029S'
,'S22030D'
,'S22030G'
,'S22030K'
,'S22030S'
,'S22031D'
,'S22031G'
,'S22031K'
,'S22031S'
,'S22032D'
,'S22032G'
,'S22032K'
,'S22032S'
,'S22038D'
,'S22038G'
,'S22038K'
,'S22038S'
,'S22039D'
,'S22039G'
,'S22039K'
,'S22039S'
,'S22040D'
,'S22040G'
,'S22040K'
,'S22040S'
,'S22041D'
,'S22041G'
,'S22041K'
,'S22041S'
,'S22042D'
,'S22042G'
,'S22042K'
,'S22042S'
,'S22048D'
,'S22048G'
,'S22048K'
,'S22048S'
,'S22049D'
,'S22049G'
,'S22049K'
,'S22049S'
,'S22050D'
,'S22050G'
,'S22050K'
,'S22050S'
,'S22051D'
,'S22051G'
,'S22051K'
,'S22051S'
,'S22052D'
,'S22052G'
,'S22052K'
,'S22052S'
,'S22058D'
,'S22058G'
,'S22058K'
,'S22058S'
,'S22059D'
,'S22059G'
,'S22059K'
,'S22059S'
,'S22060D'
,'S22060G'
,'S22060K'
,'S22060S'
,'S22061D'
,'S22061G'
,'S22061K'
,'S22061S'
,'S22062D'
,'S22062G'
,'S22062K'
,'S22062S'
,'S22068D'
,'S22068G'
,'S22068K'
,'S22068S'
,'S22069D'
,'S22069G'
,'S22069K'
,'S22069S'
,'S22070D'
,'S22070G'
,'S22070K'
,'S22070S'
,'S22071D'
,'S22071G'
,'S22071K'
,'S22071S'
,'S22072D'
,'S22072G'
,'S22072K'
,'S22072S'
,'S22078D'
,'S22078G'
,'S22078K'
,'S22078S'
,'S22079D'
,'S22079G'
,'S22079K'
,'S22079S'
,'S22080D'
,'S22080G'
,'S22080K'
,'S22080S'
,'S22081D'
,'S22081G'
,'S22081K'
,'S22081S'
,'S22082D'
,'S22082G'
,'S22082K'
,'S22082S'
,'S22088D'
,'S22088G'
,'S22088K'
,'S22088S'
,'S22089D'
,'S22089G'
,'S22089K'
,'S22089S'
,'S2220XD'
,'S2220XG'
,'S2220XK'
,'S2220XS'
,'S2221XD'
,'S2221XG'
,'S2221XK'
,'S2221XS'
,'S2222XD'
,'S2222XG'
,'S2222XK'
,'S2222XS'
,'S2223XD'
,'S2223XG'
,'S2223XK'
,'S2223XS'
,'S2224XD'
,'S2224XG'
,'S2224XK'
,'S2224XS'
,'S2231XD'
,'S2231XG'
,'S2231XK'
,'S2231XS'
,'S2232XD'
,'S2232XG'
,'S2232XK'
,'S2232XS'
,'S2239XD'
,'S2239XG'
,'S2239XK'
,'S2239XS'
,'S2241XD'
,'S2241XG'
,'S2241XK'
,'S2241XS'
,'S2242XD'
,'S2242XG'
,'S2242XK'
,'S2242XS'
,'S2243XD'
,'S2243XG'
,'S2243XK'
,'S2243XS'
,'S2249XD'
,'S2249XG'
,'S2249XK'
,'S2249XS'
,'S225XXD'
,'S225XXG'
,'S225XXK'
,'S225XXS'
,'S229XXD'
,'S229XXG'
,'S229XXK'
,'S229XXS'
,'S230XXD'
,'S230XXS'
,'S23100D'
,'S23100S'
,'S23101D'
,'S23101S'
,'S23110D'
,'S23110S'
,'S23111D'
,'S23111S'
,'S23120D'
,'S23120S'
,'S23121D'
,'S23121S'
,'S23122D'
,'S23122S'
,'S23123D'
,'S23123S'
,'S23130D'
,'S23130S'
,'S23131D'
,'S23131S'
,'S23132D'
,'S23132S'
,'S23133D'
,'S23133S'
,'S23140D'
,'S23140S'
,'S23141D'
,'S23141S'
,'S23142D'
,'S23142S'
,'S23143D'
,'S23143S'
,'S23150D'
,'S23150S'
,'S23151D'
,'S23151S'
,'S23152D'
,'S23152S'
,'S23153D'
,'S23153S'
,'S23160D'
,'S23160S'
,'S23161D'
,'S23161S'
,'S23162D'
,'S23162S'
,'S23163D'
,'S23163S'
,'S23170D'
,'S23170S'
,'S23171D'
,'S23171S'
,'S2320XD'
,'S2320XS'
,'S2329XD'
,'S2329XS'
,'S233XXD'
,'S233XXS'
,'S2341XD'
,'S2341XS'
,'S23420D'
,'S23420S'
,'S23421D'
,'S23421S'
,'S23428D'
,'S23428S'
,'S23429D'
,'S23429S'
,'S238XXD'
,'S238XXS'
,'S239XXD'
,'S239XXS'
,'S240XXD'
,'S240XXS'
,'S24101D'
,'S24101S'
,'S24102D'
,'S24102S'
,'S24103D'
,'S24103S'
,'S24104D'
,'S24104S'
,'S24109D'
,'S24109S'
,'S24111D'
,'S24111S'
,'S24112D'
,'S24112S'
,'S24113D'
,'S24113S'
,'S24114D'
,'S24114S'
,'S24119D'
,'S24119S'
,'S24131D'
,'S24131S'
,'S24132D'
,'S24132S'
,'S24133D'
,'S24133S'
,'S24134D'
,'S24134S'
,'S24139D'
,'S24139S'
,'S24141D'
,'S24141S'
,'S24142D'
,'S24142S'
,'S24143D'
,'S24143S'
,'S24144D'
,'S24144S'
,'S24149D'
,'S24149S'
,'S24151D'
,'S24151S'
,'S24152D'
,'S24152S'
,'S24153D'
,'S24153S'
,'S24154D'
,'S24154S'
,'S24159D'
,'S24159S'
,'S242XXD'
,'S242XXS'
,'S243XXD'
,'S243XXS'
,'S244XXD'
,'S244XXS'
,'S248XXD'
,'S248XXS'
,'S249XXD'
,'S249XXS'
,'S2500XD'
,'S2500XS'
,'S2501XD'
,'S2501XS'
,'S2502XD'
,'S2502XS'
,'S2509XD'
,'S2509XS'
,'S25101D'
,'S25101S'
,'S25102D'
,'S25102S'
,'S25109D'
,'S25109S'
,'S25111D'
,'S25111S'
,'S25112D'
,'S25112S'
,'S25119D'
,'S25119S'
,'S25121D'
,'S25121S'
,'S25122D'
,'S25122S'
,'S25129D'
,'S25129S'
,'S25191D'
,'S25191S'
,'S25192D'
,'S25192S'
,'S25199D'
,'S25199S'
,'S2520XD'
,'S2520XS'
,'S2521XD'
,'S2521XS'
,'S2522XD'
,'S2522XS'
,'S2529XD'
,'S2529XS'
,'S25301D'
,'S25301S'
,'S25302D'
,'S25302S'
,'S25309D'
,'S25309S'
,'S25311D'
,'S25311S'
,'S25312D'
,'S25312S'
,'S25319D'
,'S25319S'
,'S25321D'
,'S25321S'
,'S25322D'
,'S25322S'
,'S25329D'
,'S25329S'
,'S25391D'
,'S25391S'
,'S25392D'
,'S25392S'
,'S25399D'
,'S25399S'
,'S25401D'
,'S25401S'
,'S25402D'
,'S25402S'
,'S25409D'
,'S25409S'
,'S25411D'
,'S25411S'
,'S25412D'
,'S25412S'
,'S25419D'
,'S25419S'
,'S25421D'
,'S25421S'
,'S25422D'
,'S25422S'
,'S25429D'
,'S25429S'
,'S25491D'
,'S25491S'
,'S25492D'
,'S25492S'
,'S25499D'
,'S25499S'
,'S25501D'
,'S25501S'
,'S25502D'
,'S25502S'
,'S25509D'
,'S25509S'
,'S25511D'
,'S25511S'
,'S25512D'
,'S25512S'
,'S25519D'
,'S25519S'
,'S25591D'
,'S25591S'
,'S25592D'
,'S25592S'
,'S25599D'
,'S25599S'
,'S25801D'
,'S25801S'
,'S25802D'
,'S25802S'
,'S25809D'
,'S25809S'
,'S25811D'
,'S25811S'
,'S25812D'
,'S25812S'
,'S25819D'
,'S25819S'
,'S25891D'
,'S25891S'
,'S25892D'
,'S25892S'
,'S25899D'
,'S25899S'
,'S2590XD'
,'S2590XS'
,'S2591XD'
,'S2591XS'
,'S2599XD'
,'S2599XS'
,'S2600XD'
,'S2600XS'
,'S2601XD'
,'S2601XS'
,'S26020D'
,'S26020S'
,'S26021D'
,'S26021S'
,'S26022D'
,'S26022S'
,'S2609XD'
,'S2609XS'
,'S2610XD'
,'S2610XS'
,'S2611XD'
,'S2611XS'
,'S2612XD'
,'S2612XS'
,'S2619XD'
,'S2619XS'
,'S2690XD'
,'S2690XS'
,'S2691XD'
,'S2691XS'
,'S2692XD'
,'S2692XS'
,'S2699XD'
,'S2699XS'
,'S270XXD'
,'S270XXS'
,'S271XXD'
,'S271XXS'
,'S272XXD'
,'S272XXS'
,'S27301D'
,'S27301S'
,'S27302D'
,'S27302S'
,'S27309D'
,'S27309S'
,'S27311D'
,'S27311S'
,'S27312D'
,'S27312S'
,'S27319D'
,'S27319S'
,'S27321D'
,'S27321S'
,'S27322D'
,'S27322S'
,'S27329D'
,'S27329S'
,'S27331D'
,'S27331S'
,'S27332D'
,'S27332S'
,'S27339D'
,'S27339S'
,'S27391D'
,'S27391S'
,'S27392D'
,'S27392S'
,'S27399D'
,'S27399S'
,'S27401D'
,'S27401S'
,'S27402D'
,'S27402S'
,'S27409D'
,'S27409S'
,'S27411D'
,'S27411S'
,'S27412D'
,'S27412S'
,'S27419D'
,'S27419S'
,'S27421D'
,'S27421S'
,'S27422D'
,'S27422S'
,'S27429D'
,'S27429S'
,'S27431D'
,'S27431S'
,'S27432D'
,'S27432S'
,'S27439D'
,'S27439S'
,'S27491D'
,'S27491S'
,'S27492D'
,'S27492S'
,'S27499D'
,'S27499S'
,'S2750XD'
,'S2750XS'
,'S2751XD'
,'S2751XS'
,'S2752XD'
,'S2752XS'
,'S2753XD'
,'S2753XS'
,'S2759XD'
,'S2759XS'
,'S2760XD'
,'S2760XS'
,'S2763XD'
,'S2763XS'
,'S2769XD'
,'S2769XS'
,'S27802D'
,'S27802S'
,'S27803D'
,'S27803S'
,'S27808D'
,'S27808S'
,'S27809D'
,'S27809S'
,'S27812D'
,'S27812S'
,'S27813D'
,'S27813S'
,'S27818D'
,'S27818S'
,'S27819D'
,'S27819S'
,'S27892D'
,'S27892S'
,'S27893D'
,'S27893S'
,'S27898D'
,'S27898S'
,'S27899D'
,'S27899S'
,'S279XXD'
,'S279XXS'
,'S280XXD'
,'S280XXS'
,'S281XXD'
,'S281XXS'
,'S28211D'
,'S28211S'
,'S28212D'
,'S28212S'
,'S28219D'
,'S28219S'
,'S28221D'
,'S28221S'
,'S28222D'
,'S28222S'
,'S28229D'
,'S28229S'
,'S29001D'
,'S29001S'
,'S29002D'
,'S29002S'
,'S29009D'
,'S29009S'
,'S29011D'
,'S29011S'
,'S29012D'
,'S29012S'
,'S29019D'
,'S29019S'
,'S29021D'
,'S29021S'
,'S29022D'
,'S29022S'
,'S29029D'
,'S29029S'
,'S29091D'
,'S29091S'
,'S29092D'
,'S29092S'
,'S29099D'
,'S29099S'
,'S298XXD'
,'S298XXS'
,'S299XXD'
,'S299XXS'
,'S300XXD'
,'S300XXS'
,'S301XXD'
,'S301XXS'
,'S30201D'
,'S30201S'
,'S30202D'
,'S30202S'
,'S3021XD'
,'S3021XS'
,'S3022XD'
,'S3022XS'
,'S3023XD'
,'S3023XS'
,'S303XXD'
,'S303XXS'
,'S30810D'
,'S30810S'
,'S30811D'
,'S30811S'
,'S30812D'
,'S30812S'
,'S30813D'
,'S30813S'
,'S30814D'
,'S30814S'
,'S30815D'
,'S30815S'
,'S30816D'
,'S30816S'
,'S30817D'
,'S30817S'
,'S30820D'
,'S30820S'
,'S30821D'
,'S30821S'
,'S30822D'
,'S30822S'
,'S30823D'
,'S30823S'
,'S30824D'
,'S30824S'
,'S30825D'
,'S30825S'
,'S30826D'
,'S30826S'
,'S30827D'
,'S30827S'
,'S30840D'
,'S30840S'
,'S30841D'
,'S30841S'
,'S30842D'
,'S30842S'
,'S30843D'
,'S30843S'
,'S30844D'
,'S30844S'
,'S30845D'
,'S30845S'
,'S30846D'
,'S30846S'
,'S30850D'
,'S30850S'
,'S30851D'
,'S30851S'
,'S30852D'
,'S30852S'
,'S30853D'
,'S30853S'
,'S30854D'
,'S30854S'
,'S30855D'
,'S30855S'
,'S30856D'
,'S30856S'
,'S30857D'
,'S30857S'
,'S30860D'
,'S30860S'
,'S30861D'
,'S30861S'
,'S30862D'
,'S30862S'
,'S30863D'
,'S30863S'
,'S30864D'
,'S30864S'
,'S30865D'
,'S30865S'
,'S30866D'
,'S30866S'
,'S30867D'
,'S30867S'
,'S30870D'
,'S30870S'
,'S30871D'
,'S30871S'
,'S30872D'
,'S30872S'
,'S30873D'
,'S30873S'
,'S30874D'
,'S30874S'
,'S30875D'
,'S30875S'
,'S30876D'
,'S30876S'
,'S30877D'
,'S30877S'
,'S3091XD'
,'S3091XS'
,'S3092XD'
,'S3092XS'
,'S3093XD'
,'S3093XS'
,'S3094XD'
,'S3094XS'
,'S3095XD'
,'S3095XS'
,'S3096XD'
,'S3096XS'
,'S3097XD'
,'S3097XS'
,'S3098XD'
,'S3098XS'
,'S31000D'
,'S31000S'
,'S31001D'
,'S31001S'
,'S31010D'
,'S31010S'
,'S31011D'
,'S31011S'
,'S31020D'
,'S31020S'
,'S31021D'
,'S31021S'
,'S31030D'
,'S31030S'
,'S31031D'
,'S31031S'
,'S31040D'
,'S31040S'
,'S31041D'
,'S31041S'
,'S31050D'
,'S31050S'
,'S31051D'
,'S31051S'
,'S31100D'
,'S31100S'
,'S31101D'
,'S31101S'
,'S31102D'
,'S31102S'
,'S31103D'
,'S31103S'
,'S31104D'
,'S31104S'
,'S31105D'
,'S31105S'
,'S31109D'
,'S31109S'
,'S31110D'
,'S31110S'
,'S31111D'
,'S31111S'
,'S31112D'
,'S31112S'
,'S31113D'
,'S31113S'
,'S31114D'
,'S31114S'
,'S31115D'
,'S31115S'
,'S31119D'
,'S31119S'
,'S31120D'
,'S31120S'
,'S31121D'
,'S31121S'
,'S31122D'
,'S31122S'
,'S31123D'
,'S31123S'
,'S31124D'
,'S31124S'
,'S31125D'
,'S31125S'
,'S31129D'
,'S31129S'
,'S31130D'
,'S31130S'
,'S31131D'
,'S31131S'
,'S31132D'
,'S31132S'
,'S31133D'
,'S31133S'
,'S31134D'
,'S31134S'
,'S31135D'
,'S31135S'
,'S31139D'
,'S31139S'
,'S31140D'
,'S31140S'
,'S31141D'
,'S31141S'
,'S31142D'
,'S31142S'
,'S31143D'
,'S31143S'
,'S31144D'
,'S31144S'
,'S31145D'
,'S31145S'
,'S31149D'
,'S31149S'
,'S31150D'
,'S31150S'
,'S31151D'
,'S31151S'
,'S31152D'
,'S31152S'
,'S31153D'
,'S31153S'
,'S31154D'
,'S31154S'
,'S31155D'
,'S31155S'
,'S31159D'
,'S31159S'
,'S3120XD'
,'S3120XS'
,'S3121XD'
,'S3121XS'
,'S3122XD'
,'S3122XS'
,'S3123XD'
,'S3123XS'
,'S3124XD'
,'S3124XS'
,'S3125XD'
,'S3125XS'
,'S3130XD'
,'S3130XS'
,'S3131XD'
,'S3131XS'
,'S3132XD'
,'S3132XS'
,'S3133XD'
,'S3133XS'
,'S3134XD'
,'S3134XS'
,'S3135XD'
,'S3135XS'
,'S3140XD'
,'S3140XS'
,'S3141XD'
,'S3141XS'
,'S3142XD'
,'S3142XS'
,'S3143XD'
,'S3143XS'
,'S3144XD'
,'S3144XS'
,'S3145XD'
,'S3145XS'
,'S31501D'
,'S31501S'
,'S31502D'
,'S31502S'
,'S31511D'
,'S31511S'
,'S31512D'
,'S31512S'
,'S31521D'
,'S31521S'
,'S31522D'
,'S31522S'
,'S31531D'
,'S31531S'
,'S31532D'
,'S31532S'
,'S31541D'
,'S31541S'
,'S31542D'
,'S31542S'
,'S31551D'
,'S31551S'
,'S31552D'
,'S31552S'
,'S31600D'
,'S31600S'
,'S31601D'
,'S31601S'
,'S31602D'
,'S31602S'
,'S31603D'
,'S31603S'
,'S31604D'
,'S31604S'
,'S31605D'
,'S31605S'
,'S31609D'
,'S31609S'
,'S31610D'
,'S31610S'
,'S31611D'
,'S31611S'
,'S31612D'
,'S31612S'
,'S31613D'
,'S31613S'
,'S31614D'
,'S31614S'
,'S31615D'
,'S31615S'
,'S31619D'
,'S31619S'
,'S31620D'
,'S31620S'
,'S31621D'
,'S31621S'
,'S31622D'
,'S31622S'
,'S31623D'
,'S31623S'
,'S31624D'
,'S31624S'
,'S31625D'
,'S31625S'
,'S31629D'
,'S31629S'
,'S31630D'
,'S31630S'
,'S31631D'
,'S31631S'
,'S31632D'
,'S31632S'
,'S31633D'
,'S31633S'
,'S31634D'
,'S31634S'
,'S31635D'
,'S31635S'
,'S31639D'
,'S31639S'
,'S31640D'
,'S31640S'
,'S31641D'
,'S31641S'
,'S31642D'
,'S31642S'
,'S31643D'
,'S31643S'
,'S31644D'
,'S31644S'
,'S31645D'
,'S31645S'
,'S31649D'
,'S31649S'
,'S31650D'
,'S31650S'
,'S31651D'
,'S31651S'
,'S31652D'
,'S31652S'
,'S31653D'
,'S31653S'
,'S31654D'
,'S31654S'
,'S31655D'
,'S31655S'
,'S31659D'
,'S31659S'
,'S31801D'
,'S31801S'
,'S31802D'
,'S31802S'
,'S31803D'
,'S31803S'
,'S31804D'
,'S31804S'
,'S31805D'
,'S31805S'
,'S31809D'
,'S31809S'
,'S31811D'
,'S31811S'
,'S31812D'
,'S31812S'
,'S31813D'
,'S31813S'
,'S31814D'
,'S31814S'
,'S31815D'
,'S31815S'
,'S31819D'
,'S31819S'
,'S31821D'
,'S31821S'
,'S31822D'
,'S31822S'
,'S31823D'
,'S31823S'
,'S31824D'
,'S31824S'
,'S31825D'
,'S31825S'
,'S31829D'
,'S31829S'
,'S31831D'
,'S31831S'
,'S31832D'
,'S31832S'
,'S31833D'
,'S31833S'
,'S31834D'
,'S31834S'
,'S31835D'
,'S31835S'
,'S31839D'
,'S31839S'
,'S32000D'
,'S32000G'
,'S32000K'
,'S32000S'
,'S32001D'
,'S32001G'
,'S32001K'
,'S32001S'
,'S32002D'
,'S32002G'
,'S32002K'
,'S32002S'
,'S32008D'
,'S32008G'
,'S32008K'
,'S32008S'
,'S32009D'
,'S32009G'
,'S32009K'
,'S32009S'
,'S32010D'
,'S32010G'
,'S32010K'
,'S32010S'
,'S32011D'
,'S32011G'
,'S32011K'
,'S32011S'
,'S32012D'
,'S32012G'
,'S32012K'
,'S32012S'
,'S32018D'
,'S32018G'
,'S32018K'
,'S32018S'
,'S32019D'
,'S32019G'
,'S32019K'
,'S32019S'
,'S32020D'
,'S32020G'
,'S32020K'
,'S32020S'
,'S32021D'
,'S32021G'
,'S32021K'
,'S32021S'
,'S32022D'
,'S32022G'
,'S32022K'
,'S32022S'
,'S32028D'
,'S32028G'
,'S32028K'
,'S32028S'
,'S32029D'
,'S32029G'
,'S32029K'
,'S32029S'
,'S32030D'
,'S32030G'
,'S32030K'
,'S32030S'
,'S32031D'
,'S32031G'
,'S32031K'
,'S32031S'
,'S32032D'
,'S32032G'
,'S32032K'
,'S32032S'
,'S32038D'
,'S32038G'
,'S32038K'
,'S32038S'
,'S32039D'
,'S32039G'
,'S32039K'
,'S32039S'
,'S32040D'
,'S32040G'
,'S32040K'
,'S32040S'
,'S32041D'
,'S32041G'
,'S32041K'
,'S32041S'
,'S32042D'
,'S32042G'
,'S32042K'
,'S32042S'
,'S32048D'
,'S32048G'
,'S32048K'
,'S32048S'
,'S32049D'
,'S32049G'
,'S32049K'
,'S32049S'
,'S32050D'
,'S32050G'
,'S32050K'
,'S32050S'
,'S32051D'
,'S32051G'
,'S32051K'
,'S32051S'
,'S32052D'
,'S32052G'
,'S32052K'
,'S32052S'
,'S32058D'
,'S32058G'
,'S32058K'
,'S32058S'
,'S32059D'
,'S32059G'
,'S32059K'
,'S32059S'
,'S3210XD'
,'S3210XG'
,'S3210XK'
,'S3210XS'
,'S32110D'
,'S32110G'
,'S32110K'
,'S32110S'
,'S32111D'
,'S32111G'
,'S32111K'
,'S32111S'
,'S32112D'
,'S32112G'
,'S32112K'
,'S32112S'
,'S32119D'
,'S32119G'
,'S32119K'
,'S32119S'
,'S32120D'
,'S32120G'
,'S32120K'
,'S32120S'
,'S32121D'
,'S32121G'
,'S32121K'
,'S32121S'
,'S32122D'
,'S32122G'
,'S32122K'
,'S32122S'
,'S32129D'
,'S32129G'
,'S32129K'
,'S32129S'
,'S32130D'
,'S32130G'
,'S32130K'
,'S32130S'
,'S32131D'
,'S32131G'
,'S32131K'
,'S32131S'
,'S32132D'
,'S32132G'
,'S32132K'
,'S32132S'
,'S32139D'
,'S32139G'
,'S32139K'
,'S32139S'
,'S3214XD'
,'S3214XG'
,'S3214XK'
,'S3214XS'
,'S3215XD'
,'S3215XG'
,'S3215XK'
,'S3215XS'
,'S3216XD'
,'S3216XG'
,'S3216XK'
,'S3216XS'
,'S3217XD'
,'S3217XG'
,'S3217XK'
,'S3217XS'
,'S3219XD'
,'S3219XG'
,'S3219XK'
,'S3219XS'
,'S322XXD'
,'S322XXG'
,'S322XXK'
,'S322XXS'
,'S32301D'
,'S32301G'
,'S32301K'
,'S32301S'
,'S32302D'
,'S32302G'
,'S32302K'
,'S32302S'
,'S32309D'
,'S32309G'
,'S32309K'
,'S32309S'
,'S32311D'
,'S32311G'
,'S32311K'
,'S32311S'
,'S32312D'
,'S32312G'
,'S32312K'
,'S32312S'
,'S32313D'
,'S32313G'
,'S32313K'
,'S32313S'
,'S32314D'
,'S32314G'
,'S32314K'
,'S32314S'
,'S32315D'
,'S32315G'
,'S32315K'
,'S32315S'
,'S32316D'
,'S32316G'
,'S32316K'
,'S32316S'
,'S32391D'
,'S32391G'
,'S32391K'
,'S32391S'
,'S32392D'
,'S32392G'
,'S32392K'
,'S32392S'
,'S32399D'
,'S32399G'
,'S32399K'
,'S32399S'
,'S32401D'
,'S32401G'
,'S32401K'
,'S32401S'
,'S32402D'
,'S32402G'
,'S32402K'
,'S32402S'
,'S32409D'
,'S32409G'
,'S32409K'
,'S32409S'
,'S32411D'
,'S32411G'
,'S32411K'
,'S32411S'
,'S32412D'
,'S32412G'
,'S32412K'
,'S32412S'
,'S32413D'
,'S32413G'
,'S32413K'
,'S32413S'
,'S32414D'
,'S32414G'
,'S32414K'
,'S32414S'
,'S32415D'
,'S32415G'
,'S32415K'
,'S32415S'
,'S32416D'
,'S32416G'
,'S32416K'
,'S32416S'
,'S32421D'
,'S32421G'
,'S32421K'
,'S32421S'
,'S32422D'
,'S32422G'
,'S32422K'
,'S32422S'
,'S32423D'
,'S32423G'
,'S32423K'
,'S32423S'
,'S32424D'
,'S32424G'
,'S32424K'
,'S32424S'
,'S32425D'
,'S32425G'
,'S32425K'
,'S32425S'
,'S32426D'
,'S32426G'
,'S32426K'
,'S32426S'
,'S32431D'
,'S32431G'
,'S32431K'
,'S32431S'
,'S32432D'
,'S32432G'
,'S32432K'
,'S32432S'
,'S32433D'
,'S32433G'
,'S32433K'
,'S32433S'
,'S32434D'
,'S32434G'
,'S32434K'
,'S32434S'
,'S32435D'
,'S32435G'
,'S32435K'
,'S32435S'
,'S32436D'
,'S32436G'
,'S32436K'
,'S32436S'
,'S32441D'
,'S32441G'
,'S32441K'
,'S32441S'
,'S32442D'
,'S32442G'
,'S32442K'
,'S32442S'
,'S32443D'
,'S32443G'
,'S32443K'
,'S32443S'
,'S32444D'
,'S32444G'
,'S32444K'
,'S32444S'
,'S32445D'
,'S32445G'
,'S32445K'
,'S32445S'
,'S32446D'
,'S32446G'
,'S32446K'
,'S32446S'
,'S32451D'
,'S32451G'
,'S32451K'
,'S32451S'
,'S32452D'
,'S32452G'
,'S32452K'
,'S32452S'
,'S32453D'
,'S32453G'
,'S32453K'
,'S32453S'
,'S32454D'
,'S32454G'
,'S32454K'
,'S32454S'
,'S32455D'
,'S32455G'
,'S32455K'
,'S32455S'
,'S32456D'
,'S32456G'
,'S32456K'
,'S32456S'
,'S32461D'
,'S32461G'
,'S32461K'
,'S32461S'
,'S32462D'
,'S32462G'
,'S32462K'
,'S32462S'
,'S32463D'
,'S32463G'
,'S32463K'
,'S32463S'
,'S32464D'
,'S32464G'
,'S32464K'
,'S32464S'
,'S32465D'
,'S32465G'
,'S32465K'
,'S32465S'
,'S32466D'
,'S32466G'
,'S32466K'
,'S32466S'
,'S32471D'
,'S32471G'
,'S32471K'
,'S32471S'
,'S32472D'
,'S32472G'
,'S32472K'
,'S32472S'
,'S32473D'
,'S32473G'
,'S32473K'
,'S32473S'
,'S32474D'
,'S32474G'
,'S32474K'
,'S32474S'
,'S32475D'
,'S32475G'
,'S32475K'
,'S32475S'
,'S32476D'
,'S32476G'
,'S32476K'
,'S32476S'
,'S32481D'
,'S32481G'
,'S32481K'
,'S32481S'
,'S32482D'
,'S32482G'
,'S32482K'
,'S32482S'
,'S32483D'
,'S32483G'
,'S32483K'
,'S32483S'
,'S32484D'
,'S32484G'
,'S32484K'
,'S32484S'
,'S32485D'
,'S32485G'
,'S32485K'
,'S32485S'
,'S32486D'
,'S32486G'
,'S32486K'
,'S32486S'
,'S32491D'
,'S32491G'
,'S32491K'
,'S32491S'
,'S32492D'
,'S32492G'
,'S32492K'
,'S32492S'
,'S32499D'
,'S32499G'
,'S32499K'
,'S32499S'
,'S32501D'
,'S32501G'
,'S32501K'
,'S32501S'
,'S32502D'
,'S32502G'
,'S32502K'
,'S32502S'
,'S32509D'
,'S32509G'
,'S32509K'
,'S32509S'
,'S32511D'
,'S32511G'
,'S32511K'
,'S32511S'
,'S32512D'
,'S32512G'
,'S32512K'
,'S32512S'
,'S32519D'
,'S32519G'
,'S32519K'
,'S32519S'
,'S32591D'
,'S32591G'
,'S32591K'
,'S32591S'
,'S32592D'
,'S32592G'
,'S32592K'
,'S32592S'
,'S32599D'
,'S32599G'
,'S32599K'
,'S32599S'
,'S32601D'
,'S32601G'
,'S32601K'
,'S32601S'
,'S32602D'
,'S32602G'
,'S32602K'
,'S32602S'
,'S32609D'
,'S32609G'
,'S32609K'
,'S32609S'
,'S32611D'
,'S32611G'
,'S32611K'
,'S32611S'
,'S32612D'
,'S32612G'
,'S32612K'
,'S32612S'
,'S32613D'
,'S32613G'
,'S32613K'
,'S32613S'
,'S32614D'
,'S32614G'
,'S32614K'
,'S32614S'
,'S32615D'
,'S32615G'
,'S32615K'
,'S32615S'
,'S32616D'
,'S32616G'
,'S32616K'
,'S32616S'
,'S32691D'
,'S32691G'
,'S32691K'
,'S32691S'
,'S32692D'
,'S32692G'
,'S32692K'
,'S32692S'
,'S32699D'
,'S32699G'
,'S32699K'
,'S32699S'
,'S32810D'
,'S32810G'
,'S32810K'
,'S32810S'
,'S32811D'
,'S32811G'
,'S32811K'
,'S32811S'
,'S3282XD'
,'S3282XG'
,'S3282XK'
,'S3282XS'
,'S3289XD'
,'S3289XG'
,'S3289XK'
,'S3289XS'
,'S329XXD'
,'S329XXG'
,'S329XXK'
,'S329XXS'
,'S330XXD'
,'S330XXS'
,'S33100D'
,'S33100S'
,'S33101D'
,'S33101S'
,'S33110D'
,'S33110S'
,'S33111D'
,'S33111S'
,'S33120D'
,'S33120S'
,'S33121D'
,'S33121S'
,'S33130D'
,'S33130S'
,'S33131D'
,'S33131S'
,'S33140D'
,'S33140S'
,'S33141D'
,'S33141S'
,'S332XXD'
,'S332XXS'
,'S3330XD'
,'S3330XS'
,'S3339XD'
,'S3339XS'
,'S334XXD'
,'S334XXS'
,'S335XXD'
,'S335XXS'
,'S336XXD'
,'S336XXS'
,'S338XXD'
,'S338XXS'
,'S339XXD'
,'S339XXS'
,'S3401XD'
,'S3401XS'
,'S3402XD'
,'S3402XS'
,'S34101D'
,'S34101S'
,'S34102D'
,'S34102S'
,'S34103D'
,'S34103S'
,'S34104D'
,'S34104S'
,'S34105D'
,'S34105S'
,'S34109D'
,'S34109S'
,'S34111D'
,'S34111S'
,'S34112D'
,'S34112S'
,'S34113D'
,'S34113S'
,'S34114D'
,'S34114S'
,'S34115D'
,'S34115S'
,'S34119D'
,'S34119S'
,'S34121D'
,'S34121S'
,'S34122D'
,'S34122S'
,'S34123D'
,'S34123S'
,'S34124D'
,'S34124S'
,'S34125D'
,'S34125S'
,'S34129D'
,'S34129S'
,'S34131D'
,'S34131S'
,'S34132D'
,'S34132S'
,'S34139D'
,'S34139S'
,'S3421XD'
,'S3421XS'
,'S3422XD'
,'S3422XS'
,'S343XXD'
,'S343XXS'
,'S344XXD'
,'S344XXS'
,'S345XXD'
,'S345XXS'
,'S346XXD'
,'S346XXS'
,'S348XXD'
,'S348XXS'
,'S349XXD'
,'S349XXS'
,'S3500XD'
,'S3500XS'
,'S3501XD'
,'S3501XS'
,'S3502XD'
,'S3502XS'
,'S3509XD'
,'S3509XS'
,'S3510XD'
,'S3510XS'
,'S3511XD'
,'S3511XS'
,'S3512XD'
,'S3512XS'
,'S3519XD'
,'S3519XS'
,'S35211D'
,'S35211S'
,'S35212D'
,'S35212S'
,'S35218D'
,'S35218S'
,'S35219D'
,'S35219S'
,'S35221D'
,'S35221S'
,'S35222D'
,'S35222S'
,'S35228D'
,'S35228S'
,'S35229D'
,'S35229S'
,'S35231D'
,'S35231S'
,'S35232D'
,'S35232S'
,'S35238D'
,'S35238S'
,'S35239D'
,'S35239S'
,'S35291D'
,'S35291S'
,'S35292D'
,'S35292S'
,'S35298D'
,'S35298S'
,'S35299D'
,'S35299S'
,'S35311D'
,'S35311S'
,'S35318D'
,'S35318S'
,'S35319D'
,'S35319S'
,'S35321D'
,'S35321S'
,'S35328D'
,'S35328S'
,'S35329D'
,'S35329S'
,'S35331D'
,'S35331S'
,'S35338D'
,'S35338S'
,'S35339D'
,'S35339S'
,'S35341D'
,'S35341S'
,'S35348D'
,'S35348S'
,'S35349D'
,'S35349S'
,'S35401D'
,'S35401S'
,'S35402D'
,'S35402S'
,'S35403D'
,'S35403S'
,'S35404D'
,'S35404S'
,'S35405D'
,'S35405S'
,'S35406D'
,'S35406S'
,'S35411D'
,'S35411S'
,'S35412D'
,'S35412S'
,'S35413D'
,'S35413S'
,'S35414D'
,'S35414S'
,'S35415D'
,'S35415S'
,'S35416D'
,'S35416S'
,'S35491D'
,'S35491S'
,'S35492D'
,'S35492S'
,'S35493D'
,'S35493S'
,'S35494D'
,'S35494S'
,'S35495D'
,'S35495S'
,'S35496D'
,'S35496S'
,'S3550XD'
,'S3550XS'
,'S35511D'
,'S35511S'
,'S35512D'
,'S35512S'
,'S35513D'
,'S35513S'
,'S35514D'
,'S35514S'
,'S35515D'
,'S35515S'
,'S35516D'
,'S35516S'
,'S35531D'
,'S35531S'
,'S35532D'
,'S35532S'
,'S35533D'
,'S35533S'
,'S35534D'
,'S35534S'
,'S35535D'
,'S35535S'
,'S35536D'
,'S35536S'
,'S3559XD'
,'S3559XS'
,'S358X1D'
,'S358X1S'
,'S358X8D'
,'S358X8S'
,'S358X9D'
,'S358X9S'
,'S3590XD'
,'S3590XS'
,'S3591XD'
,'S3591XS'
,'S3599XD'
,'S3599XS'
,'S3600XD'
,'S3600XS'
,'S36020D'
,'S36020S'
,'S36021D'
,'S36021S'
,'S36029D'
,'S36029S'
,'S36030D'
,'S36030S'
,'S36031D'
,'S36031S'
,'S36032D'
,'S36032S'
,'S36039D'
,'S36039S'
,'S3609XD'
,'S3609XS'
,'S36112D'
,'S36112S'
,'S36113D'
,'S36113S'
,'S36114D'
,'S36114S'
,'S36115D'
,'S36115S'
,'S36116D'
,'S36116S'
,'S36118D'
,'S36118S'
,'S36119D'
,'S36119S'
,'S36122D'
,'S36122S'
,'S36123D'
,'S36123S'
,'S36128D'
,'S36128S'
,'S36129D'
,'S36129S'
,'S3613XD'
,'S3613XS'
,'S36200D'
,'S36200S'
,'S36201D'
,'S36201S'
,'S36202D'
,'S36202S'
,'S36209D'
,'S36209S'
,'S36220D'
,'S36220S'
,'S36221D'
,'S36221S'
,'S36222D'
,'S36222S'
,'S36229D'
,'S36229S'
,'S36230D'
,'S36230S'
,'S36231D'
,'S36231S'
,'S36232D'
,'S36232S'
,'S36239D'
,'S36239S'
,'S36240D'
,'S36240S'
,'S36241D'
,'S36241S'
,'S36242D'
,'S36242S'
,'S36249D'
,'S36249S'
,'S36250D'
,'S36250S'
,'S36251D'
,'S36251S'
,'S36252D'
,'S36252S'
,'S36259D'
,'S36259S'
,'S36260D'
,'S36260S'
,'S36261D'
,'S36261S'
,'S36262D'
,'S36262S'
,'S36269D'
,'S36269S'
,'S36290D'
,'S36290S'
,'S36291D'
,'S36291S'
,'S36292D'
,'S36292S'
,'S36299D'
,'S36299S'
,'S3630XD'
,'S3630XS'
,'S3632XD'
,'S3632XS'
,'S3633XD'
,'S3633XS'
,'S3639XD'
,'S3639XS'
,'S36400D'
,'S36400S'
,'S36408D'
,'S36408S'
,'S36409D'
,'S36409S'
,'S36410D'
,'S36410S'
,'S36418D'
,'S36418S'
,'S36419D'
,'S36419S'
,'S36420D'
,'S36420S'
,'S36428D'
,'S36428S'
,'S36429D'
,'S36429S'
,'S36430D'
,'S36430S'
,'S36438D'
,'S36438S'
,'S36439D'
,'S36439S'
,'S36490D'
,'S36490S'
,'S36498D'
,'S36498S'
,'S36499D'
,'S36499S'
,'S36500D'
,'S36500S'
,'S36501D'
,'S36501S'
,'S36502D'
,'S36502S'
,'S36503D'
,'S36503S'
,'S36508D'
,'S36508S'
,'S36509D'
,'S36509S'
,'S36510D'
,'S36510S'
,'S36511D'
,'S36511S'
,'S36512D'
,'S36512S'
,'S36513D'
,'S36513S'
,'S36518D'
,'S36518S'
,'S36519D'
,'S36519S'
,'S36520D'
,'S36520S'
,'S36521D'
,'S36521S'
,'S36522D'
,'S36522S'
,'S36523D'
,'S36523S'
,'S36528D'
,'S36528S'
,'S36529D'
,'S36529S'
,'S36530D'
,'S36530S'
,'S36531D'
,'S36531S'
,'S36532D'
,'S36532S'
,'S36533D'
,'S36533S'
,'S36538D'
,'S36538S'
,'S36539D'
,'S36539S'
,'S36590D'
,'S36590S'
,'S36591D'
,'S36591S'
,'S36592D'
,'S36592S'
,'S36593D'
,'S36593S'
,'S36598D'
,'S36598S'
,'S36599D'
,'S36599S'
,'S3660XD'
,'S3660XS'
,'S3661XD'
,'S3661XS'
,'S3662XD'
,'S3662XS'
,'S3663XD'
,'S3663XS'
,'S3669XD'
,'S3669XS'
,'S3681XD'
,'S3681XS'
,'S36892D'
,'S36892S'
,'S36893D'
,'S36893S'
,'S36898D'
,'S36898S'
,'S36899D'
,'S36899S'
,'S3690XD'
,'S3690XS'
,'S3692XD'
,'S3692XS'
,'S3693XD'
,'S3693XS'
,'S3699XD'
,'S3699XS'
,'S37001D'
,'S37001S'
,'S37002D'
,'S37002S'
,'S37009D'
,'S37009S'
,'S37011D'
,'S37011S'
,'S37012D'
,'S37012S'
,'S37019D'
,'S37019S'
,'S37021D'
,'S37021S'
,'S37022D'
,'S37022S'
,'S37029D'
,'S37029S'
,'S37031D'
,'S37031S'
,'S37032D'
,'S37032S'
,'S37039D'
,'S37039S'
,'S37041D'
,'S37041S'
,'S37042D'
,'S37042S'
,'S37049D'
,'S37049S'
,'S37051D'
,'S37051S'
,'S37052D'
,'S37052S'
,'S37059D'
,'S37059S'
,'S37061D'
,'S37061S'
,'S37062D'
,'S37062S'
,'S37069D'
,'S37069S'
,'S37091D'
,'S37091S'
,'S37092D'
,'S37092S'
,'S37099D'
,'S37099S'
,'S3710XD'
,'S3710XS'
,'S3712XD'
,'S3712XS'
,'S3713XD'
,'S3713XS'
,'S3719XD'
,'S3719XS'
,'S3720XD'
,'S3720XS'
,'S3722XD'
,'S3722XS'
,'S3723XD'
,'S3723XS'
,'S3729XD'
,'S3729XS'
,'S3730XD'
,'S3730XS'
,'S3732XD'
,'S3732XS'
,'S3733XD'
,'S3733XS'
,'S3739XD'
,'S3739XS'
,'S37401D'
,'S37401S'
,'S37402D'
,'S37402S'
,'S37409D'
,'S37409S'
,'S37421D'
,'S37421S'
,'S37422D'
,'S37422S'
,'S37429D'
,'S37429S'
,'S37431D'
,'S37431S'
,'S37432D'
,'S37432S'
,'S37439D'
,'S37439S'
,'S37491D'
,'S37491S'
,'S37492D'
,'S37492S'
,'S37499D'
,'S37499S'
,'S37501D'
,'S37501S'
,'S37502D'
,'S37502S'
,'S37509D'
,'S37509S'
,'S37511D'
,'S37511S'
,'S37512D'
,'S37512S'
,'S37519D'
,'S37519S'
,'S37521D'
,'S37521S'
,'S37522D'
,'S37522S'
,'S37529D'
,'S37529S'
,'S37531D'
,'S37531S'
,'S37532D'
,'S37532S'
,'S37539D'
,'S37539S'
,'S37591D'
,'S37591S'
,'S37592D'
,'S37592S'
,'S37599D'
,'S37599S'
,'S3760XD'
,'S3760XS'
,'S3762XD'
,'S3762XS'
,'S3763XD'
,'S3763XS'
,'S3769XD'
,'S3769XS'
,'S37812D'
,'S37812S'
,'S37813D'
,'S37813S'
,'S37818D'
,'S37818S'
,'S37819D'
,'S37819S'
,'S37822D'
,'S37822S'
,'S37823D'
,'S37823S'
,'S37828D'
,'S37828S'
,'S37829D'
,'S37829S'
,'S37892D'
,'S37892S'
,'S37893D'
,'S37893S'
,'S37898D'
,'S37898S'
,'S37899D'
,'S37899S'
,'S3790XD'
,'S3790XS'
,'S3792XD'
,'S3792XS'
,'S3793XD'
,'S3793XS'
,'S3799XD'
,'S3799XS'
,'S38001D'
,'S38001S'
,'S38002D'
,'S38002S'
,'S3801XD'
,'S3801XS'
,'S3802XD'
,'S3802XS'
,'S3803XD'
,'S3803XS'
,'S381XXD'
,'S381XXS'
,'S38211D'
,'S38211S'
,'S38212D'
,'S38212S'
,'S38221D'
,'S38221S'
,'S38222D'
,'S38222S'
,'S38231D'
,'S38231S'
,'S38232D'
,'S38232S'
,'S383XXD'
,'S383XXS'
,'S39001D'
,'S39001S'
,'S39002D'
,'S39002S'
,'S39003D'
,'S39003S'
,'S39011D'
,'S39011S'
,'S39012D'
,'S39012S'
,'S39013D'
,'S39013S'
,'S39021D'
,'S39021S'
,'S39022D'
,'S39022S'
,'S39023D'
,'S39023S'
,'S39091D'
,'S39091S'
,'S39092D'
,'S39092S'
,'S39093D'
,'S39093S'
,'S3981XD'
,'S3981XS'
,'S3982XD'
,'S3982XS'
,'S3983XD'
,'S3983XS'
,'S39840D'
,'S39840S'
,'S39848D'
,'S39848S'
,'S3991XD'
,'S3991XS'
,'S3992XD'
,'S3992XS'
,'S3993XD'
,'S3993XS'
,'S3994XD'
,'S3994XS'
,'S40011D'
,'S40011S'
,'S40012D'
,'S40012S'
,'S40019D'
,'S40019S'
,'S40021D'
,'S40021S'
,'S40022D'
,'S40022S'
,'S40029D'
,'S40029S'
,'S40211D'
,'S40211S'
,'S40212D'
,'S40212S'
,'S40219D'
,'S40219S'
,'S40221D'
,'S40221S'
,'S40222D'
,'S40222S'
,'S40229D'
,'S40229S'
,'S40241D'
,'S40241S'
,'S40242D'
,'S40242S'
,'S40249D'
,'S40249S'
,'S40251D'
,'S40251S'
,'S40252D'
,'S40252S'
,'S40259D'
,'S40259S'
,'S40261D'
,'S40261S'
,'S40262D'
,'S40262S'
,'S40269D'
,'S40269S'
,'S40271D'
,'S40271S'
,'S40272D'
,'S40272S'
,'S40279D'
,'S40279S'
,'S40811D'
,'S40811S'
,'S40812D'
,'S40812S'
,'S40819D'
,'S40819S'
,'S40821D'
,'S40821S'
,'S40822D'
,'S40822S'
,'S40829D'
,'S40829S'
,'S40841D'
,'S40841S'
,'S40842D'
,'S40842S'
,'S40849D'
,'S40849S'
,'S40851D'
,'S40851S'
,'S40852D'
,'S40852S'
,'S40859D'
,'S40859S'
,'S40861D'
,'S40861S'
,'S40862D'
,'S40862S'
,'S40869D'
,'S40869S'
,'S40871D'
,'S40871S'
,'S40872D'
,'S40872S'
,'S40879D'
,'S40879S'
,'S40911D'
,'S40911S'
,'S40912D'
,'S40912S'
,'S40919D'
,'S40919S'
,'S40921D'
,'S40921S'
,'S40922D'
,'S40922S'
,'S40929D'
,'S40929S'
,'S41001D'
,'S41001S'
,'S41002D'
,'S41002S'
,'S41009D'
,'S41009S'
,'S41011D'
,'S41011S'
,'S41012D'
,'S41012S'
,'S41019D'
,'S41019S'
,'S41021D'
,'S41021S'
,'S41022D'
,'S41022S'
,'S41029D'
,'S41029S'
,'S41031D'
,'S41031S'
,'S41032D'
,'S41032S'
,'S41039D'
,'S41039S'
,'S41041D'
,'S41041S'
,'S41042D'
,'S41042S'
,'S41049D'
,'S41049S'
,'S41051D'
,'S41051S'
,'S41052D'
,'S41052S'
,'S41059D'
,'S41059S'
,'S41101D'
,'S41101S'
,'S41102D'
,'S41102S'
,'S41109D'
,'S41109S'
,'S41111D'
,'S41111S'
,'S41112D'
,'S41112S'
,'S41119D'
,'S41119S'
,'S41121D'
,'S41121S'
,'S41122D'
,'S41122S'
,'S41129D'
,'S41129S'
,'S41131D'
,'S41131S'
,'S41132D'
,'S41132S'
,'S41139D'
,'S41139S'
,'S41141D'
,'S41141S'
,'S41142D'
,'S41142S'
,'S41149D'
,'S41149S'
,'S41151D'
,'S41151S'
,'S41152D'
,'S41152S'
,'S41159D'
,'S41159S'
,'S42001D'
,'S42001G'
,'S42001K'
,'S42001P'
,'S42001S'
,'S42002D'
,'S42002G'
,'S42002K'
,'S42002P'
,'S42002S'
,'S42009D'
,'S42009G'
,'S42009K'
,'S42009P'
,'S42009S'
,'S42011D'
,'S42011G'
,'S42011K'
,'S42011P'
,'S42011S'
,'S42012D'
,'S42012G'
,'S42012K'
,'S42012P'
,'S42012S'
,'S42013D'
,'S42013G'
,'S42013K'
,'S42013P'
,'S42013S'
,'S42014D'
,'S42014G'
,'S42014K'
,'S42014P'
,'S42014S'
,'S42015D'
,'S42015G'
,'S42015K'
,'S42015P'
,'S42015S'
,'S42016D'
,'S42016G'
,'S42016K'
,'S42016P'
,'S42016S'
,'S42017D'
,'S42017G'
,'S42017K'
,'S42017P'
,'S42017S'
,'S42018D'
,'S42018G'
,'S42018K'
,'S42018P'
,'S42018S'
,'S42019D'
,'S42019G'
,'S42019K'
,'S42019P'
,'S42019S'
,'S42021D'
,'S42021G'
,'S42021K'
,'S42021P'
,'S42021S'
,'S42022D'
,'S42022G'
,'S42022K'
,'S42022P'
,'S42022S'
,'S42023D'
,'S42023G'
,'S42023K'
,'S42023P'
,'S42023S'
,'S42024D'
,'S42024G'
,'S42024K'
,'S42024P'
,'S42024S'
,'S42025D'
,'S42025G'
,'S42025K'
,'S42025P'
,'S42025S'
,'S42026D'
,'S42026G'
,'S42026K'
,'S42026P'
,'S42026S'
,'S42031D'
,'S42031G'
,'S42031K'
,'S42031P'
,'S42031S'
,'S42032D'
,'S42032G'
,'S42032K'
,'S42032P'
,'S42032S'
,'S42033D'
,'S42033G'
,'S42033K'
,'S42033P'
,'S42033S'
,'S42034D'
,'S42034G'
,'S42034K'
,'S42034P'
,'S42034S'
,'S42035D'
,'S42035G'
,'S42035K'
,'S42035P'
,'S42035S'
,'S42036D'
,'S42036G'
,'S42036K'
,'S42036P'
,'S42036S'
,'S42101D'
,'S42101G'
,'S42101K'
,'S42101P'
,'S42101S'
,'S42102D'
,'S42102G'
,'S42102K'
,'S42102P'
,'S42102S'
,'S42109D'
,'S42109G'
,'S42109K'
,'S42109P'
,'S42109S'
,'S42111D'
,'S42111G'
,'S42111K'
,'S42111P'
,'S42111S'
,'S42112D'
,'S42112G'
,'S42112K'
,'S42112P'
,'S42112S'
,'S42113D'
,'S42113G'
,'S42113K'
,'S42113P'
,'S42113S'
,'S42114D'
,'S42114G'
,'S42114K'
,'S42114P'
,'S42114S'
,'S42115D'
,'S42115G'
,'S42115K'
,'S42115P'
,'S42115S'
,'S42116D'
,'S42116G'
,'S42116K'
,'S42116P'
,'S42116S'
,'S42121D'
,'S42121G'
,'S42121K'
,'S42121P'
,'S42121S'
,'S42122D'
,'S42122G'
,'S42122K'
,'S42122P'
,'S42122S'
,'S42123D'
,'S42123G'
,'S42123K'
,'S42123P'
,'S42123S'
,'S42124D'
,'S42124G'
,'S42124K'
,'S42124P'
,'S42124S'
,'S42125D'
,'S42125G'
,'S42125K'
,'S42125P'
,'S42125S'
,'S42126D'
,'S42126G'
,'S42126K'
,'S42126P'
,'S42126S'
,'S42131D'
,'S42131G'
,'S42131K'
,'S42131P'
,'S42131S'
,'S42132D'
,'S42132G'
,'S42132K'
,'S42132P'
,'S42132S'
,'S42133D'
,'S42133G'
,'S42133K'
,'S42133P'
,'S42133S'
,'S42134D'
,'S42134G'
,'S42134K'
,'S42134P'
,'S42134S'
,'S42135D'
,'S42135G'
,'S42135K'
,'S42135P'
,'S42135S'
,'S42136D'
,'S42136G'
,'S42136K'
,'S42136P'
,'S42136S'
,'S42141D'
,'S42141G'
,'S42141K'
,'S42141P'
,'S42141S'
,'S42142D'
,'S42142G'
,'S42142K'
,'S42142P'
,'S42142S'
,'S42143D'
,'S42143G'
,'S42143K'
,'S42143P'
,'S42143S'
,'S42144D'
,'S42144G'
,'S42144K'
,'S42144P'
,'S42144S'
,'S42145D'
,'S42145G'
,'S42145K'
,'S42145P'
,'S42145S'
,'S42146D'
,'S42146G'
,'S42146K'
,'S42146P'
,'S42146S'
,'S42151D'
,'S42151G'
,'S42151K'
,'S42151P'
,'S42151S'
,'S42152D'
,'S42152G'
,'S42152K'
,'S42152P'
,'S42152S'
,'S42153D'
,'S42153G'
,'S42153K'
,'S42153P'
,'S42153S'
,'S42154D'
,'S42154G'
,'S42154K'
,'S42154P'
,'S42154S'
,'S42155D'
,'S42155G'
,'S42155K'
,'S42155P'
,'S42155S'
,'S42156D'
,'S42156G'
,'S42156K'
,'S42156P'
,'S42156S'
,'S42191D'
,'S42191G'
,'S42191K'
,'S42191P'
,'S42191S'
,'S42192D'
,'S42192G'
,'S42192K'
,'S42192P'
,'S42192S'
,'S42199D'
,'S42199G'
,'S42199K'
,'S42199P'
,'S42199S'
,'S42201D'
,'S42201G'
,'S42201K'
,'S42201P'
,'S42201S'
,'S42202D'
,'S42202G'
,'S42202K'
,'S42202P'
,'S42202S'
,'S42209D'
,'S42209G'
,'S42209K'
,'S42209P'
,'S42209S'
,'S42211D'
,'S42211G'
,'S42211K'
,'S42211P'
,'S42211S'
,'S42212D'
,'S42212G'
,'S42212K'
,'S42212P'
,'S42212S'
,'S42213D'
,'S42213G'
,'S42213K'
,'S42213P'
,'S42213S'
,'S42214D'
,'S42214G'
,'S42214K'
,'S42214P'
,'S42214S'
,'S42215D'
,'S42215G'
,'S42215K'
,'S42215P'
,'S42215S'
,'S42216D'
,'S42216G'
,'S42216K'
,'S42216P'
,'S42216S'
,'S42221D'
,'S42221G'
,'S42221K'
,'S42221P'
,'S42221S'
,'S42222D'
,'S42222G'
,'S42222K'
,'S42222P'
,'S42222S'
,'S42223D'
,'S42223G'
,'S42223K'
,'S42223P'
,'S42223S'
,'S42224D'
,'S42224G'
,'S42224K'
,'S42224P'
,'S42224S'
,'S42225D'
,'S42225G'
,'S42225K'
,'S42225P'
,'S42225S'
,'S42226D'
,'S42226G'
,'S42226K'
,'S42226P'
,'S42226S'
,'S42231D'
,'S42231G'
,'S42231K'
,'S42231P'
,'S42231S'
,'S42232D'
,'S42232G'
,'S42232K'
,'S42232P'
,'S42232S'
,'S42239D'
,'S42239G'
,'S42239K'
,'S42239P'
,'S42239S'
,'S42241D'
,'S42241G'
,'S42241K'
,'S42241P'
,'S42241S'
,'S42242D'
,'S42242G'
,'S42242K'
,'S42242P'
,'S42242S'
,'S42249D'
,'S42249G'
,'S42249K'
,'S42249P'
,'S42249S'
,'S42251D'
,'S42251G'
,'S42251K'
,'S42251P'
,'S42251S'
,'S42252D'
,'S42252G'
,'S42252K'
,'S42252P'
,'S42252S'
,'S42253D'
,'S42253G'
,'S42253K'
,'S42253P'
,'S42253S'
,'S42254D'
,'S42254G'
,'S42254K'
,'S42254P'
,'S42254S'
,'S42255D'
,'S42255G'
,'S42255K'
,'S42255P'
,'S42255S'
,'S42256D'
,'S42256G'
,'S42256K'
,'S42256P'
,'S42256S'
,'S42261D'
,'S42261G'
,'S42261K'
,'S42261P'
,'S42261S'
,'S42262D'
,'S42262G'
,'S42262K'
,'S42262P'
,'S42262S'
,'S42263D'
,'S42263G'
,'S42263K'
,'S42263P'
,'S42263S'
,'S42264D'
,'S42264G'
,'S42264K'
,'S42264P'
,'S42264S'
,'S42265D'
,'S42265G'
,'S42265K'
,'S42265P'
,'S42265S'
,'S42266D'
,'S42266G'
,'S42266K'
,'S42266P'
,'S42266S'
,'S42271D'
,'S42271G'
,'S42271K'
,'S42271P'
,'S42271S'
,'S42272D'
,'S42272G'
,'S42272K'
,'S42272P'
,'S42272S'
,'S42279D'
,'S42279G'
,'S42279K'
,'S42279P'
,'S42279S'
,'S42291D'
,'S42291G'
,'S42291K'
,'S42291P'
,'S42291S'
,'S42292D'
,'S42292G'
,'S42292K'
,'S42292P'
,'S42292S'
,'S42293D'
,'S42293G'
,'S42293K'
,'S42293P'
,'S42293S'
,'S42294D'
,'S42294G'
,'S42294K'
,'S42294P'
,'S42294S'
,'S42295D'
,'S42295G'
,'S42295K'
,'S42295P'
,'S42295S'
,'S42296D'
,'S42296G'
,'S42296K'
,'S42296P'
,'S42296S'
,'S42301D'
,'S42301G'
,'S42301K'
,'S42301P'
,'S42301S'
,'S42302D'
,'S42302G'
,'S42302K'
,'S42302P'
,'S42302S'
,'S42309D'
,'S42309G'
,'S42309K'
,'S42309P'
,'S42309S'
,'S42311D'
,'S42311G'
,'S42311K'
,'S42311P'
,'S42311S'
,'S42312D'
,'S42312G'
,'S42312K'
,'S42312P'
,'S42312S'
,'S42319D'
,'S42319G'
,'S42319K'
,'S42319P'
,'S42319S'
,'S42321D'
,'S42321G'
,'S42321K'
,'S42321P'
,'S42321S'
,'S42322D'
,'S42322G'
,'S42322K'
,'S42322P'
,'S42322S'
,'S42323D'
,'S42323G'
,'S42323K'
,'S42323P'
,'S42323S'
,'S42324D'
,'S42324G'
,'S42324K'
,'S42324P'
,'S42324S'
,'S42325D'
,'S42325G'
,'S42325K'
,'S42325P'
,'S42325S'
,'S42326D'
,'S42326G'
,'S42326K'
,'S42326P'
,'S42326S'
,'S42331D'
,'S42331G'
,'S42331K'
,'S42331P'
,'S42331S'
,'S42332D'
,'S42332G'
,'S42332K'
,'S42332P'
,'S42332S'
,'S42333D'
,'S42333G'
,'S42333K'
,'S42333P'
,'S42333S'
,'S42334D'
,'S42334G'
,'S42334K'
,'S42334P'
,'S42334S'
,'S42335D'
,'S42335G'
,'S42335K'
,'S42335P'
,'S42335S'
,'S42336D'
,'S42336G'
,'S42336K'
,'S42336P'
,'S42336S'
,'S42341D'
,'S42341G'
,'S42341K'
,'S42341P'
,'S42341S'
,'S42342D'
,'S42342G'
,'S42342K'
,'S42342P'
,'S42342S'
,'S42343D'
,'S42343G'
,'S42343K'
,'S42343P'
,'S42343S'
,'S42344D'
,'S42344G'
,'S42344K'
,'S42344P'
,'S42344S'
,'S42345D'
,'S42345G'
,'S42345K'
,'S42345P'
,'S42345S'
,'S42346D'
,'S42346G'
,'S42346K'
,'S42346P'
,'S42346S'
,'S42351D'
,'S42351G'
,'S42351K'
,'S42351P'
,'S42351S'
,'S42352D'
,'S42352G'
,'S42352K'
,'S42352P'
,'S42352S'
,'S42353D'
,'S42353G'
,'S42353K'
,'S42353P'
,'S42353S'
,'S42354D'
,'S42354G'
,'S42354K'
,'S42354P'
,'S42354S'
,'S42355D'
,'S42355G'
,'S42355K'
,'S42355P'
,'S42355S'
,'S42356D'
,'S42356G'
,'S42356K'
,'S42356P'
,'S42356S'
,'S42361D'
,'S42361G'
,'S42361K'
,'S42361P'
,'S42361S'
,'S42362D'
,'S42362G'
,'S42362K'
,'S42362P'
,'S42362S'
,'S42363D'
,'S42363G'
,'S42363K'
,'S42363P'
,'S42363S'
,'S42364D'
,'S42364G'
,'S42364K'
,'S42364P'
,'S42364S'
,'S42365D'
,'S42365G'
,'S42365K'
,'S42365P'
,'S42365S'
,'S42366D'
,'S42366G'
,'S42366K'
,'S42366P'
,'S42366S'
,'S42391D'
,'S42391G'
,'S42391K'
,'S42391P'
,'S42391S'
,'S42392D'
,'S42392G'
,'S42392K'
,'S42392P'
,'S42392S'
,'S42399D'
,'S42399G'
,'S42399K'
,'S42399P'
,'S42399S'
,'S42401D'
,'S42401G'
,'S42401K'
,'S42401P'
,'S42401S'
,'S42402D'
,'S42402G'
,'S42402K'
,'S42402P'
,'S42402S'
,'S42409D'
,'S42409G'
,'S42409K'
,'S42409P'
,'S42409S'
,'S42411D'
,'S42411G'
,'S42411K'
,'S42411P'
,'S42411S'
,'S42412D'
,'S42412G'
,'S42412K'
,'S42412P'
,'S42412S'
,'S42413D'
,'S42413G'
,'S42413K'
,'S42413P'
,'S42413S'
,'S42414D'
,'S42414G'
,'S42414K'
,'S42414P'
,'S42414S'
,'S42415D'
,'S42415G'
,'S42415K'
,'S42415P'
,'S42415S'
,'S42416D'
,'S42416G'
,'S42416K'
,'S42416P'
,'S42416S'
,'S42421D'
,'S42421G'
,'S42421K'
,'S42421P'
,'S42421S'
,'S42422D'
,'S42422G'
,'S42422K'
,'S42422P'
,'S42422S'
,'S42423D'
,'S42423G'
,'S42423K'
,'S42423P'
,'S42423S'
,'S42424D'
,'S42424G'
,'S42424K'
,'S42424P'
,'S42424S'
,'S42425D'
,'S42425G'
,'S42425K'
,'S42425P'
,'S42425S'
,'S42426D'
,'S42426G'
,'S42426K'
,'S42426P'
,'S42426S'
,'S42431D'
,'S42431G'
,'S42431K'
,'S42431P'
,'S42431S'
,'S42432D'
,'S42432G'
,'S42432K'
,'S42432P'
,'S42432S'
,'S42433D'
,'S42433G'
,'S42433K'
,'S42433P'
,'S42433S'
,'S42434D'
,'S42434G'
,'S42434K'
,'S42434P'
,'S42434S'
,'S42435D'
,'S42435G'
,'S42435K'
,'S42435P'
,'S42435S'
,'S42436D'
,'S42436G'
,'S42436K'
,'S42436P'
,'S42436S'
,'S42441D'
,'S42441G'
,'S42441K'
,'S42441P'
,'S42441S'
,'S42442D'
,'S42442G'
,'S42442K'
,'S42442P'
,'S42442S'
,'S42443D'
,'S42443G'
,'S42443K'
,'S42443P'
,'S42443S'
,'S42444D'
,'S42444G'
,'S42444K'
,'S42444P'
,'S42444S'
,'S42445D'
,'S42445G'
,'S42445K'
,'S42445P'
,'S42445S'
,'S42446D'
,'S42446G'
,'S42446K'
,'S42446P'
,'S42446S'
,'S42447D'
,'S42447G'
,'S42447K'
,'S42447P'
,'S42447S'
,'S42448D'
,'S42448G'
,'S42448K'
,'S42448P'
,'S42448S'
,'S42449D'
,'S42449G'
,'S42449K'
,'S42449P'
,'S42449S'
,'S42451D'
,'S42451G'
,'S42451K'
,'S42451P'
,'S42451S'
,'S42452D'
,'S42452G'
,'S42452K'
,'S42452P'
,'S42452S'
,'S42453D'
,'S42453G'
,'S42453K'
,'S42453P'
,'S42453S'
,'S42454D'
,'S42454G'
,'S42454K'
,'S42454P'
,'S42454S'
,'S42455D'
,'S42455G'
,'S42455K'
,'S42455P'
,'S42455S'
,'S42456D'
,'S42456G'
,'S42456K'
,'S42456P'
,'S42456S'
,'S42461D'
,'S42461G'
,'S42461K'
,'S42461P'
,'S42461S'
,'S42462D'
,'S42462G'
,'S42462K'
,'S42462P'
,'S42462S'
,'S42463D'
,'S42463G'
,'S42463K'
,'S42463P'
,'S42463S'
,'S42464D'
,'S42464G'
,'S42464K'
,'S42464P'
,'S42464S'
,'S42465D'
,'S42465G'
,'S42465K'
,'S42465P'
,'S42465S'
,'S42466D'
,'S42466G'
,'S42466K'
,'S42466P'
,'S42466S'
,'S42471D'
,'S42471G'
,'S42471K'
,'S42471P'
,'S42471S'
,'S42472D'
,'S42472G'
,'S42472K'
,'S42472P'
,'S42472S'
,'S42473D'
,'S42473G'
,'S42473K'
,'S42473P'
,'S42473S'
,'S42474D'
,'S42474G'
,'S42474K'
,'S42474P'
,'S42474S'
,'S42475D'
,'S42475G'
,'S42475K'
,'S42475P'
,'S42475S'
,'S42476D'
,'S42476G'
,'S42476K'
,'S42476P'
,'S42476S'
,'S42481D'
,'S42481G'
,'S42481K'
,'S42481P'
,'S42481S'
,'S42482D'
,'S42482G'
,'S42482K'
,'S42482P'
,'S42482S'
,'S42489D'
,'S42489G'
,'S42489K'
,'S42489P'
,'S42489S'
,'S42491D'
,'S42491G'
,'S42491K'
,'S42491P'
,'S42491S'
,'S42492D'
,'S42492G'
,'S42492K'
,'S42492P'
,'S42492S'
,'S42493D'
,'S42493G'
,'S42493K'
,'S42493P'
,'S42493S'
,'S42494D'
,'S42494G'
,'S42494K'
,'S42494P'
,'S42494S'
,'S42495D'
,'S42495G'
,'S42495K'
,'S42495P'
,'S42495S'
,'S42496D'
,'S42496G'
,'S42496K'
,'S42496P'
,'S42496S'
,'S4290XD'
,'S4290XG'
,'S4290XK'
,'S4290XP'
,'S4290XS'
,'S4291XD'
,'S4291XG'
,'S4291XK'
,'S4291XP'
,'S4291XS'
,'S4292XD'
,'S4292XG'
,'S4292XK'
,'S4292XP'
,'S4292XS'
,'S43001D'
,'S43001S'
,'S43002D'
,'S43002S'
,'S43003D'
,'S43003S'
,'S43004D'
,'S43004S'
,'S43005D'
,'S43005S'
,'S43006D'
,'S43006S'
,'S43011D'
,'S43011S'
,'S43012D'
,'S43012S'
,'S43013D'
,'S43013S'
,'S43014D'
,'S43014S'
,'S43015D'
,'S43015S'
,'S43016D'
,'S43016S'
,'S43021D'
,'S43021S'
,'S43022D'
,'S43022S'
,'S43023D'
,'S43023S'
,'S43024D'
,'S43024S'
,'S43025D'
,'S43025S'
,'S43026D'
,'S43026S'
,'S43031D'
,'S43031S'
,'S43032D'
,'S43032S'
,'S43033D'
,'S43033S'
,'S43034D'
,'S43034S'
,'S43035D'
,'S43035S'
,'S43036D'
,'S43036S'
,'S43081D'
,'S43081S'
,'S43082D'
,'S43082S'
,'S43083D'
,'S43083S'
,'S43084D'
,'S43084S'
,'S43085D'
,'S43085S'
,'S43086D'
,'S43086S'
,'S43101D'
,'S43101S'
,'S43102D'
,'S43102S'
,'S43109D'
,'S43109S'
,'S43111D'
,'S43111S'
,'S43112D'
,'S43112S'
,'S43119D'
,'S43119S'
,'S43121D'
,'S43121S'
,'S43122D'
,'S43122S'
,'S43129D'
,'S43129S'
,'S43131D'
,'S43131S'
,'S43132D'
,'S43132S'
,'S43139D'
,'S43139S'
,'S43141D'
,'S43141S'
,'S43142D'
,'S43142S'
,'S43149D'
,'S43149S'
,'S43151D'
,'S43151S'
,'S43152D'
,'S43152S'
,'S43159D'
,'S43159S'
,'S43201D'
,'S43201S'
,'S43202D'
,'S43202S'
,'S43203D'
,'S43203S'
,'S43204D'
,'S43204S'
,'S43205D'
,'S43205S'
,'S43206D'
,'S43206S'
,'S43211D'
,'S43211S'
,'S43212D'
,'S43212S'
,'S43213D'
,'S43213S'
,'S43214D'
,'S43214S'
,'S43215D'
,'S43215S'
,'S43216D'
,'S43216S'
,'S43221D'
,'S43221S'
,'S43222D'
,'S43222S'
,'S43223D'
,'S43223S'
,'S43224D'
,'S43224S'
,'S43225D'
,'S43225S'
,'S43226D'
,'S43226S'
,'S43301D'
,'S43301S'
,'S43302D'
,'S43302S'
,'S43303D'
,'S43303S'
,'S43304D'
,'S43304S'
,'S43305D'
,'S43305S'
,'S43306D'
,'S43306S'
,'S43311D'
,'S43311S'
,'S43312D'
,'S43312S'
,'S43313D'
,'S43313S'
,'S43314D'
,'S43314S'
,'S43315D'
,'S43315S'
,'S43316D'
,'S43316S'
,'S43391D'
,'S43391S'
,'S43392D'
,'S43392S'
,'S43393D'
,'S43393S'
,'S43394D'
,'S43394S'
,'S43395D'
,'S43395S'
,'S43396D'
,'S43396S'
,'S43401D'
,'S43401S'
,'S43402D'
,'S43402S'
,'S43409D'
,'S43409S'
,'S43411D'
,'S43411S'
,'S43412D'
,'S43412S'
,'S43419D'
,'S43419S'
,'S43421D'
,'S43421S'
,'S43422D'
,'S43422S'
,'S43429D'
,'S43429S'
,'S43431D'
,'S43431S'
,'S43432D'
,'S43432S'
,'S43439D'
,'S43439S'
,'S43491D'
,'S43491S'
,'S43492D'
,'S43492S'
,'S43499D'
,'S43499S'
,'S4350XD'
,'S4350XS'
,'S4351XD'
,'S4351XS'
,'S4352XD'
,'S4352XS'
,'S4360XD'
,'S4360XS'
,'S4361XD'
,'S4361XS'
,'S4362XD'
,'S4362XS'
,'S4380XD'
,'S4380XS'
,'S4381XD'
,'S4381XS'
,'S4382XD'
,'S4382XS'
,'S4390XD'
,'S4390XS'
,'S4391XD'
,'S4391XS'
,'S4392XD'
,'S4392XS'
,'S4400XD'
,'S4400XS'
,'S4401XD'
,'S4401XS'
,'S4402XD'
,'S4402XS'
,'S4410XD'
,'S4410XS'
,'S4411XD'
,'S4411XS'
,'S4412XD'
,'S4412XS'
,'S4420XD'
,'S4420XS'
,'S4421XD'
,'S4421XS'
,'S4422XD'
,'S4422XS'
,'S4430XD'
,'S4430XS'
,'S4431XD'
,'S4431XS'
,'S4432XD'
,'S4432XS'
,'S4440XD'
,'S4440XS'
,'S4441XD'
,'S4441XS'
,'S4442XD'
,'S4442XS'
,'S4450XD'
,'S4450XS'
,'S4451XD'
,'S4451XS'
,'S4452XD'
,'S4452XS'
,'S448X1D'
,'S448X1S'
,'S448X2D'
,'S448X2S'
,'S448X9D'
,'S448X9S'
,'S4490XD'
,'S4490XS'
,'S4491XD'
,'S4491XS'
,'S4492XD'
,'S4492XS'
,'S45001D'
,'S45001S'
,'S45002D'
,'S45002S'
,'S45009D'
,'S45009S'
,'S45011D'
,'S45011S'
,'S45012D'
,'S45012S'
,'S45019D'
,'S45019S'
,'S45091D'
,'S45091S'
,'S45092D'
,'S45092S'
,'S45099D'
,'S45099S'
,'S45101D'
,'S45101S'
,'S45102D'
,'S45102S'
,'S45109D'
,'S45109S'
,'S45111D'
,'S45111S'
,'S45112D'
,'S45112S'
,'S45119D'
,'S45119S'
,'S45191D'
,'S45191S'
,'S45192D'
,'S45192S'
,'S45199D'
,'S45199S'
,'S45201D'
,'S45201S'
,'S45202D'
,'S45202S'
,'S45209D'
,'S45209S'
,'S45211D'
,'S45211S'
,'S45212D'
,'S45212S'
,'S45219D'
,'S45219S'
,'S45291D'
,'S45291S'
,'S45292D'
,'S45292S'
,'S45299D'
,'S45299S'
,'S45301D'
,'S45301S'
,'S45302D'
,'S45302S'
,'S45309D'
,'S45309S'
,'S45311D'
,'S45311S'
,'S45312D'
,'S45312S'
,'S45319D'
,'S45319S'
,'S45391D'
,'S45391S'
,'S45392D'
,'S45392S'
,'S45399D'
,'S45399S'
,'S45801D'
,'S45801S'
,'S45802D'
,'S45802S'
,'S45809D'
,'S45809S'
,'S45811D'
,'S45811S'
,'S45812D'
,'S45812S'
,'S45819D'
,'S45819S'
,'S45891D'
,'S45891S'
,'S45892D'
,'S45892S'
,'S45899D'
,'S45899S'
,'S45901D'
,'S45901S'
,'S45902D'
,'S45902S'
,'S45909D'
,'S45909S'
,'S45911D'
,'S45911S'
,'S45912D'
,'S45912S'
,'S45919D'
,'S45919S'
,'S45991D'
,'S45991S'
,'S45992D'
,'S45992S'
,'S45999D'
,'S45999S'
,'S46001D'
,'S46001S'
,'S46002D'
,'S46002S'
,'S46009D'
,'S46009S'
,'S46011D'
,'S46011S'
,'S46012D'
,'S46012S'
,'S46019D'
,'S46019S'
,'S46021D'
,'S46021S'
,'S46022D'
,'S46022S'
,'S46029D'
,'S46029S'
,'S46091D'
,'S46091S'
,'S46092D'
,'S46092S'
,'S46099D'
,'S46099S'
,'S46101D'
,'S46101S'
,'S46102D'
,'S46102S'
,'S46109D'
,'S46109S'
,'S46111D'
,'S46111S'
,'S46112D'
,'S46112S'
,'S46119D'
,'S46119S'
,'S46121D'
,'S46121S'
,'S46122D'
,'S46122S'
,'S46129D'
,'S46129S'
,'S46191D'
,'S46191S'
,'S46192D'
,'S46192S'
,'S46199D'
,'S46199S'
,'S46201D'
,'S46201S'
,'S46202D'
,'S46202S'
,'S46209D'
,'S46209S'
,'S46211D'
,'S46211S'
,'S46212D'
,'S46212S'
,'S46219D'
,'S46219S'
,'S46221D'
,'S46221S'
,'S46222D'
,'S46222S'
,'S46229D'
,'S46229S'
,'S46291D'
,'S46291S'
,'S46292D'
,'S46292S'
,'S46299D'
,'S46299S'
,'S46301D'
,'S46301S'
,'S46302D'
,'S46302S'
,'S46309D'
,'S46309S'
,'S46311D'
,'S46311S'
,'S46312D'
,'S46312S'
,'S46319D'
,'S46319S'
,'S46321D'
,'S46321S'
,'S46322D'
,'S46322S'
,'S46329D'
,'S46329S'
,'S46391D'
,'S46391S'
,'S46392D'
,'S46392S'
,'S46399D'
,'S46399S'
,'S46801D'
,'S46801S'
,'S46802D'
,'S46802S'
,'S46809D'
,'S46809S'
,'S46811D'
,'S46811S'
,'S46812D'
,'S46812S'
,'S46819D'
,'S46819S'
,'S46821D'
,'S46821S'
,'S46822D'
,'S46822S'
,'S46829D'
,'S46829S'
,'S46891D'
,'S46891S'
,'S46892D'
,'S46892S'
,'S46899D'
,'S46899S'
,'S46901D'
,'S46901S'
,'S46902D'
,'S46902S'
,'S46909D'
,'S46909S'
,'S46911D'
,'S46911S'
,'S46912D'
,'S46912S'
,'S46919D'
,'S46919S'
,'S46921D'
,'S46921S'
,'S46922D'
,'S46922S'
,'S46929D'
,'S46929S'
,'S46991D'
,'S46991S'
,'S46992D'
,'S46992S'
,'S46999D'
,'S46999S'
,'S471XXD'
,'S471XXS'
,'S472XXD'
,'S472XXS'
,'S479XXD'
,'S479XXS'
,'S48011D'
,'S48011S'
,'S48012D'
,'S48012S'
,'S48019D'
,'S48019S'
,'S48021D'
,'S48021S'
,'S48022D'
,'S48022S'
,'S48029D'
,'S48029S'
,'S48111D'
,'S48111S'
,'S48112D'
,'S48112S'
,'S48119D'
,'S48119S'
,'S48121D'
,'S48121S'
,'S48122D'
,'S48122S'
,'S48129D'
,'S48129S'
,'S48911D'
,'S48911S'
,'S48912D'
,'S48912S'
,'S48919D'
,'S48919S'
,'S48921D'
,'S48921S'
,'S48922D'
,'S48922S'
,'S48929D'
,'S48929S'
,'S49001D'
,'S49001G'
,'S49001K'
,'S49001P'
,'S49001S'
,'S49002D'
,'S49002G'
,'S49002K'
,'S49002P'
,'S49002S'
,'S49009D'
,'S49009G'
,'S49009K'
,'S49009P'
,'S49009S'
,'S49011D'
,'S49011G'
,'S49011K'
,'S49011P'
,'S49011S'
,'S49012D'
,'S49012G'
,'S49012K'
,'S49012P'
,'S49012S'
,'S49019D'
,'S49019G'
,'S49019K'
,'S49019P'
,'S49019S'
,'S49021D'
,'S49021G'
,'S49021K'
,'S49021P'
,'S49021S'
,'S49022D'
,'S49022G'
,'S49022K'
,'S49022P'
,'S49022S'
,'S49029D'
,'S49029G'
,'S49029K'
,'S49029P'
,'S49029S'
,'S49031D'
,'S49031G'
,'S49031K'
,'S49031P'
,'S49031S'
,'S49032D'
,'S49032G'
,'S49032K'
,'S49032P'
,'S49032S'
,'S49039D'
,'S49039G'
,'S49039K'
,'S49039P'
,'S49039S'
,'S49041D'
,'S49041G'
,'S49041K'
,'S49041P'
,'S49041S'
,'S49042D'
,'S49042G'
,'S49042K'
,'S49042P'
,'S49042S'
,'S49049D'
,'S49049G'
,'S49049K'
,'S49049P'
,'S49049S'
,'S49091D'
,'S49091G'
,'S49091K'
,'S49091P'
,'S49091S'
,'S49092D'
,'S49092G'
,'S49092K'
,'S49092P'
,'S49092S'
,'S49099D'
,'S49099G'
,'S49099K'
,'S49099P'
,'S49099S'
,'S49101D'
,'S49101G'
,'S49101K'
,'S49101P'
,'S49101S'
,'S49102D'
,'S49102G'
,'S49102K'
,'S49102P'
,'S49102S'
,'S49109D'
,'S49109G'
,'S49109K'
,'S49109P'
,'S49109S'
,'S49111D'
,'S49111G'
,'S49111K'
,'S49111P'
,'S49111S'
,'S49112D'
,'S49112G'
,'S49112K'
,'S49112P'
,'S49112S'
,'S49119D'
,'S49119G'
,'S49119K'
,'S49119P'
,'S49119S'
,'S49121D'
,'S49121G'
,'S49121K'
,'S49121P'
,'S49121S'
,'S49122D'
,'S49122G'
,'S49122K'
,'S49122P'
,'S49122S'
,'S49129D'
,'S49129G'
,'S49129K'
,'S49129P'
,'S49129S'
,'S49131D'
,'S49131G'
,'S49131K'
,'S49131P'
,'S49131S'
,'S49132D'
,'S49132G'
,'S49132K'
,'S49132P'
,'S49132S'
,'S49139D'
,'S49139G'
,'S49139K'
,'S49139P'
,'S49139S'
,'S49141D'
,'S49141G'
,'S49141K'
,'S49141P'
,'S49141S'
,'S49142D'
,'S49142G'
,'S49142K'
,'S49142P'
,'S49142S'
,'S49149D'
,'S49149G'
,'S49149K'
,'S49149P'
,'S49149S'
,'S49191D'
,'S49191G'
,'S49191K'
,'S49191P'
,'S49191S'
,'S49192D'
,'S49192G'
,'S49192K'
,'S49192P'
,'S49192S'
,'S49199D'
,'S49199G'
,'S49199K'
,'S49199P'
,'S49199S'
,'S4980XD'
,'S4980XS'
,'S4981XD'
,'S4981XS'
,'S4982XD'
,'S4982XS'
,'S4990XD'
,'S4990XS'
,'S4991XD'
,'S4991XS'
,'S4992XD'
,'S4992XS'
,'S5000XD'
,'S5000XS'
,'S5001XD'
,'S5001XS'
,'S5002XD'
,'S5002XS'
,'S5010XD'
,'S5010XS'
,'S5011XD'
,'S5011XS'
,'S5012XD'
,'S5012XS'
,'S50311D'
,'S50311S'
,'S50312D'
,'S50312S'
,'S50319D'
,'S50319S'
,'S50321D'
,'S50321S'
,'S50322D'
,'S50322S'
,'S50329D'
,'S50329S'
,'S50341D'
,'S50341S'
,'S50342D'
,'S50342S'
,'S50349D'
,'S50349S'
,'S50351D'
,'S50351S'
,'S50352D'
,'S50352S'
,'S50359D'
,'S50359S'
,'S50361D'
,'S50361S'
,'S50362D'
,'S50362S'
,'S50369D'
,'S50369S'
,'S50371D'
,'S50371S'
,'S50372D'
,'S50372S'
,'S50379D'
,'S50379S'
,'S50811D'
,'S50811S'
,'S50812D'
,'S50812S'
,'S50819D'
,'S50819S'
,'S50821D'
,'S50821S'
,'S50822D'
,'S50822S'
,'S50829D'
,'S50829S'
,'S50841D'
,'S50841S'
,'S50842D'
,'S50842S'
,'S50849D'
,'S50849S'
,'S50851D'
,'S50851S'
,'S50852D'
,'S50852S'
,'S50859D'
,'S50859S'
,'S50861D'
,'S50861S'
,'S50862D'
,'S50862S'
,'S50869D'
,'S50869S'
,'S50871D'
,'S50871S'
,'S50872D'
,'S50872S'
,'S50879D'
,'S50879S'
,'S50901D'
,'S50901S'
,'S50902D'
,'S50902S'
,'S50909D'
,'S50909S'
,'S50911D'
,'S50911S'
,'S50912D'
,'S50912S'
,'S50919D'
,'S50919S'
,'S51001D'
,'S51001S'
,'S51002D'
,'S51002S'
,'S51009D'
,'S51009S'
,'S51011D'
,'S51011S'
,'S51012D'
,'S51012S'
,'S51019D'
,'S51019S'
,'S51021D'
,'S51021S'
,'S51022D'
,'S51022S'
,'S51029D'
,'S51029S'
,'S51031D'
,'S51031S'
,'S51032D'
,'S51032S'
,'S51039D'
,'S51039S'
,'S51041D'
,'S51041S'
,'S51042D'
,'S51042S'
,'S51049D'
,'S51049S'
,'S51051D'
,'S51051S'
,'S51052D'
,'S51052S'
,'S51059D'
,'S51059S'
,'S51801D'
,'S51801S'
,'S51802D'
,'S51802S'
,'S51809D'
,'S51809S'
,'S51811D'
,'S51811S'
,'S51812D'
,'S51812S'
,'S51819D'
,'S51819S'
,'S51821D'
,'S51821S'
,'S51822D'
,'S51822S'
,'S51829D'
,'S51829S'
,'S51831D'
,'S51831S'
,'S51832D'
,'S51832S'
,'S51839D'
,'S51839S'
,'S51841D'
,'S51841S'
,'S51842D'
,'S51842S'
,'S51849D'
,'S51849S'
,'S51851D'
,'S51851S'
,'S51852D'
,'S51852S'
,'S51859D'
,'S51859S'
,'S52001C'
,'S52001D'
,'S52001E'
,'S52001F'
,'S52001G'
,'S52001H'
,'S52001J'
,'S52001K'
,'S52001M'
,'S52001N'
,'S52001P'
,'S52001Q'
,'S52001R'
,'S52001S'
,'S52002C'
,'S52002D'
,'S52002E'
,'S52002F'
,'S52002G'
,'S52002H'
,'S52002J'
,'S52002K'
,'S52002M'
,'S52002N'
,'S52002P'
,'S52002Q'
,'S52002R'
,'S52002S'
,'S52009C'
,'S52009D'
,'S52009E'
,'S52009F'
,'S52009G'
,'S52009H'
,'S52009J'
,'S52009K'
,'S52009M'
,'S52009N'
,'S52009P'
,'S52009Q'
,'S52009R'
,'S52009S'
,'S52011D'
,'S52011G'
,'S52011K'
,'S52011P'
,'S52011S'
,'S52012D'
,'S52012G'
,'S52012K'
,'S52012P'
,'S52012S'
,'S52019D'
,'S52019G'
,'S52019K'
,'S52019P'
,'S52019S'
,'S52021C'
,'S52021D'
,'S52021E'
,'S52021F'
,'S52021G'
,'S52021H'
,'S52021J'
,'S52021K'
,'S52021M'
,'S52021N'
,'S52021P'
,'S52021Q'
,'S52021R'
,'S52021S'
,'S52022C'
,'S52022D'
,'S52022E'
,'S52022F'
,'S52022G'
,'S52022H'
,'S52022J'
,'S52022K'
,'S52022M'
,'S52022N'
,'S52022P'
,'S52022Q'
,'S52022R'
,'S52022S'
,'S52023C'
,'S52023D'
,'S52023E'
,'S52023F'
,'S52023G'
,'S52023H'
,'S52023J'
,'S52023K'
,'S52023M'
,'S52023N'
,'S52023P'
,'S52023Q'
,'S52023R'
,'S52023S'
,'S52024C'
,'S52024D'
,'S52024E'
,'S52024F'
,'S52024G'
,'S52024H'
,'S52024J'
,'S52024K'
,'S52024M'
,'S52024N'
,'S52024P'
,'S52024Q'
,'S52024R'
,'S52024S'
,'S52025C'
,'S52025D'
,'S52025E'
,'S52025F'
,'S52025G'
,'S52025H'
,'S52025J'
,'S52025K'
,'S52025M'
,'S52025N'
,'S52025P'
,'S52025Q'
,'S52025R'
,'S52025S'
,'S52026C'
,'S52026D'
,'S52026E'
,'S52026F'
,'S52026G'
,'S52026H'
,'S52026J'
,'S52026K'
,'S52026M'
,'S52026N'
,'S52026P'
,'S52026Q'
,'S52026R'
,'S52026S'
,'S52031C'
,'S52031D'
,'S52031E'
,'S52031F'
,'S52031G'
,'S52031H'
,'S52031J'
,'S52031K'
,'S52031M'
,'S52031N'
,'S52031P'
,'S52031Q'
,'S52031R'
,'S52031S'
,'S52032C'
,'S52032D'
,'S52032E'
,'S52032F'
,'S52032G'
,'S52032H'
,'S52032J'
,'S52032K'
,'S52032M'
,'S52032N'
,'S52032P'
,'S52032Q'
,'S52032R'
,'S52032S'
,'S52033C'
,'S52033D'
,'S52033E'
,'S52033F'
,'S52033G'
,'S52033H'
,'S52033J'
,'S52033K'
,'S52033M'
,'S52033N'
,'S52033P'
,'S52033Q'
,'S52033R'
,'S52033S'
,'S52034C'
,'S52034D'
,'S52034E'
,'S52034F'
,'S52034G'
,'S52034H'
,'S52034J'
,'S52034K'
,'S52034M'
,'S52034N'
,'S52034P'
,'S52034Q'
,'S52034R'
,'S52034S'
,'S52035C'
,'S52035D'
,'S52035E'
,'S52035F'
,'S52035G'
,'S52035H'
,'S52035J'
,'S52035K'
,'S52035M'
,'S52035N'
,'S52035P'
,'S52035Q'
,'S52035R'
,'S52035S'
,'S52036C'
,'S52036D'
,'S52036E'
,'S52036F'
,'S52036G'
,'S52036H'
,'S52036J'
,'S52036K'
,'S52036M'
,'S52036N'
,'S52036P'
,'S52036Q'
,'S52036R'
,'S52036S'
,'S52041C'
,'S52041D'
,'S52041E'
,'S52041F'
,'S52041G'
,'S52041H'
,'S52041J'
,'S52041K'
,'S52041M'
,'S52041N'
,'S52041P'
,'S52041Q'
,'S52041R'
,'S52041S'
,'S52042C'
,'S52042D'
,'S52042E'
,'S52042F'
,'S52042G'
,'S52042H'
,'S52042J'
,'S52042K'
,'S52042M'
,'S52042N'
,'S52042P'
,'S52042Q'
,'S52042R'
,'S52042S'
,'S52043C'
,'S52043D'
,'S52043E'
,'S52043F'
,'S52043G'
,'S52043H'
,'S52043J'
,'S52043K'
,'S52043M'
,'S52043N'
,'S52043P'
,'S52043Q'
,'S52043R'
,'S52043S'
,'S52044C'
,'S52044D'
,'S52044E'
,'S52044F'
,'S52044G'
,'S52044H'
,'S52044J'
,'S52044K'
,'S52044M'
,'S52044N'
,'S52044P'
,'S52044Q'
,'S52044R'
,'S52044S'
,'S52045C'
,'S52045D'
,'S52045E'
,'S52045F'
,'S52045G'
,'S52045H'
,'S52045J'
,'S52045K'
,'S52045M'
,'S52045N'
,'S52045P'
,'S52045Q'
,'S52045R'
,'S52045S'
,'S52046C'
,'S52046D'
,'S52046E'
,'S52046F'
,'S52046G'
,'S52046H'
,'S52046J'
,'S52046K'
,'S52046M'
,'S52046N'
,'S52046P'
,'S52046Q'
,'S52046R'
,'S52046S'
,'S52091C'
,'S52091D'
,'S52091E'
,'S52091F'
,'S52091G'
,'S52091H'
,'S52091J'
,'S52091K'
,'S52091M'
,'S52091N'
,'S52091P'
,'S52091Q'
,'S52091R'
,'S52091S'
,'S52092C'
,'S52092D'
,'S52092E'
,'S52092F'
,'S52092G'
,'S52092H'
,'S52092J'
,'S52092K'
,'S52092M'
,'S52092N'
,'S52092P'
,'S52092Q'
,'S52092R'
,'S52092S'
,'S52099C'
,'S52099D'
,'S52099E'
,'S52099F'
,'S52099G'
,'S52099H'
,'S52099J'
,'S52099K'
,'S52099M'
,'S52099N'
,'S52099P'
,'S52099Q'
,'S52099R'
,'S52099S'
,'S52101C'
,'S52101D'
,'S52101E'
,'S52101F'
,'S52101G'
,'S52101H'
,'S52101J'
,'S52101K'
,'S52101M'
,'S52101N'
,'S52101P'
,'S52101Q'
,'S52101R'
,'S52101S'
,'S52102C'
,'S52102D'
,'S52102E'
,'S52102F'
,'S52102G'
,'S52102H'
,'S52102J'
,'S52102K'
,'S52102M'
,'S52102N'
,'S52102P'
,'S52102Q'
,'S52102R'
,'S52102S'
,'S52109C'
,'S52109D'
,'S52109E'
,'S52109F'
,'S52109G'
,'S52109H'
,'S52109J'
,'S52109K'
,'S52109M'
,'S52109N'
,'S52109P'
,'S52109Q'
,'S52109R'
,'S52109S'
,'S52111D'
,'S52111G'
,'S52111K'
,'S52111P'
,'S52111S'
,'S52112D'
,'S52112G'
,'S52112K'
,'S52112P'
,'S52112S'
,'S52119D'
,'S52119G'
,'S52119K'
,'S52119P'
,'S52119S'
,'S52121C'
,'S52121D'
,'S52121E'
,'S52121F'
,'S52121G'
,'S52121H'
,'S52121J'
,'S52121K'
,'S52121M'
,'S52121N'
,'S52121P'
,'S52121Q'
,'S52121R'
,'S52121S'
,'S52122C'
,'S52122D'
,'S52122E'
,'S52122F'
,'S52122G'
,'S52122H'
,'S52122J'
,'S52122K'
,'S52122M'
,'S52122N'
,'S52122P'
,'S52122Q'
,'S52122R'
,'S52122S'
,'S52123C'
,'S52123D'
,'S52123E'
,'S52123F'
,'S52123G'
,'S52123H'
,'S52123J'
,'S52123K'
,'S52123M'
,'S52123N'
,'S52123P'
,'S52123Q'
,'S52123R'
,'S52123S'
,'S52124C'
,'S52124D'
,'S52124E'
,'S52124F'
,'S52124G'
,'S52124H'
,'S52124J'
,'S52124K'
,'S52124M'
,'S52124N'
,'S52124P'
,'S52124Q'
,'S52124R'
,'S52124S'
,'S52125C'
,'S52125D'
,'S52125E'
,'S52125F'
,'S52125G'
,'S52125H'
,'S52125J'
,'S52125K'
,'S52125M'
,'S52125N'
,'S52125P'
,'S52125Q'
,'S52125R'
,'S52125S'
,'S52126C'
,'S52126D'
,'S52126E'
,'S52126F'
,'S52126G'
,'S52126H'
,'S52126J'
,'S52126K'
,'S52126M'
,'S52126N'
,'S52126P'
,'S52126Q'
,'S52126R'
,'S52126S'
,'S52131C'
,'S52131D'
,'S52131E'
,'S52131F'
,'S52131G'
,'S52131H'
,'S52131J'
,'S52131K'
,'S52131M'
,'S52131N'
,'S52131P'
,'S52131Q'
,'S52131R'
,'S52131S'
,'S52132C'
,'S52132D'
,'S52132E'
,'S52132F'
,'S52132G'
,'S52132H'
,'S52132J'
,'S52132K'
,'S52132M'
,'S52132N'
,'S52132P'
,'S52132Q'
,'S52132R'
,'S52132S'
,'S52133C'
,'S52133D'
,'S52133E'
,'S52133F'
,'S52133G'
,'S52133H'
,'S52133J'
,'S52133K'
,'S52133M'
,'S52133N'
,'S52133P'
,'S52133Q'
,'S52133R'
,'S52133S'
,'S52134C'
,'S52134D'
,'S52134E'
,'S52134F'
,'S52134G'
,'S52134H'
,'S52134J'
,'S52134K'
,'S52134M'
,'S52134N'
,'S52134P'
,'S52134Q'
,'S52134R'
,'S52134S'
,'S52135C'
,'S52135D'
,'S52135E'
,'S52135F'
,'S52135G'
,'S52135H'
,'S52135J'
,'S52135K'
,'S52135M'
,'S52135N'
,'S52135P'
,'S52135Q'
,'S52135R'
,'S52135S'
,'S52136C'
,'S52136D'
,'S52136E'
,'S52136F'
,'S52136G'
,'S52136H'
,'S52136J'
,'S52136K'
,'S52136M'
,'S52136N'
,'S52136P'
,'S52136Q'
,'S52136R'
,'S52136S'
,'S52181C'
,'S52181D'
,'S52181E'
,'S52181F'
,'S52181G'
,'S52181H'
,'S52181J'
,'S52181K'
,'S52181M'
,'S52181N'
,'S52181P'
,'S52181Q'
,'S52181R'
,'S52181S'
,'S52182C'
,'S52182D'
,'S52182E'
,'S52182F'
,'S52182G'
,'S52182H'
,'S52182J'
,'S52182K'
,'S52182M'
,'S52182N'
,'S52182P'
,'S52182Q'
,'S52182R'
,'S52182S'
,'S52189C'
,'S52189D'
,'S52189E'
,'S52189F'
,'S52189G'
,'S52189H'
,'S52189J'
,'S52189K'
,'S52189M'
,'S52189N'
,'S52189P'
,'S52189Q'
,'S52189R'
,'S52189S'
,'S52201C'
,'S52201D'
,'S52201E'
,'S52201F'
,'S52201G'
,'S52201H'
,'S52201J'
,'S52201K'
,'S52201M'
,'S52201N'
,'S52201P'
,'S52201Q'
,'S52201R'
,'S52201S'
,'S52202C'
,'S52202D'
,'S52202E'
,'S52202F'
,'S52202G'
,'S52202H'
,'S52202J'
,'S52202K'
,'S52202M'
,'S52202N'
,'S52202P'
,'S52202Q'
,'S52202R'
,'S52202S'
,'S52209C'
,'S52209D'
,'S52209E'
,'S52209F'
,'S52209G'
,'S52209H'
,'S52209J'
,'S52209K'
,'S52209M'
,'S52209N'
,'S52209P'
,'S52209Q'
,'S52209R'
,'S52209S'
,'S52211D'
,'S52211G'
,'S52211K'
,'S52211P'
,'S52211S'
,'S52212D'
,'S52212G'
,'S52212K'
,'S52212P'
,'S52212S'
,'S52219D'
,'S52219G'
,'S52219K'
,'S52219P'
,'S52219S'
,'S52221C'
,'S52221D'
,'S52221E'
,'S52221F'
,'S52221G'
,'S52221H'
,'S52221J'
,'S52221K'
,'S52221M'
,'S52221N'
,'S52221P'
,'S52221Q'
,'S52221R'
,'S52221S'
,'S52222C'
,'S52222D'
,'S52222E'
,'S52222F'
,'S52222G'
,'S52222H'
,'S52222J'
,'S52222K'
,'S52222M'
,'S52222N'
,'S52222P'
,'S52222Q'
,'S52222R'
,'S52222S'
,'S52223C'
,'S52223D'
,'S52223E'
,'S52223F'
,'S52223G'
,'S52223H'
,'S52223J'
,'S52223K'
,'S52223M'
,'S52223N'
,'S52223P'
,'S52223Q'
,'S52223R'
,'S52223S'
,'S52224C'
,'S52224D'
,'S52224E'
,'S52224F'
,'S52224G'
,'S52224H'
,'S52224J'
,'S52224K'
,'S52224M'
,'S52224N'
,'S52224P'
,'S52224Q'
,'S52224R'
,'S52224S'
,'S52225C'
,'S52225D'
,'S52225E'
,'S52225F'
,'S52225G'
,'S52225H'
,'S52225J'
,'S52225K'
,'S52225M'
,'S52225N'
,'S52225P'
,'S52225Q'
,'S52225R'
,'S52225S'
,'S52226C'
,'S52226D'
,'S52226E'
,'S52226F'
,'S52226G'
,'S52226H'
,'S52226J'
,'S52226K'
,'S52226M'
,'S52226N'
,'S52226P'
,'S52226Q'
,'S52226R'
,'S52226S'
,'S52231C'
,'S52231D'
,'S52231E'
,'S52231F'
,'S52231G'
,'S52231H'
,'S52231J'
,'S52231K'
,'S52231M'
,'S52231N'
,'S52231P'
,'S52231Q'
,'S52231R'
,'S52231S'
,'S52232C'
,'S52232D'
,'S52232E'
,'S52232F'
,'S52232G'
,'S52232H'
,'S52232J'
,'S52232K'
,'S52232M'
,'S52232N'
,'S52232P'
,'S52232Q'
,'S52232R'
,'S52232S'
,'S52233C'
,'S52233D'
,'S52233E'
,'S52233F'
,'S52233G'
,'S52233H'
,'S52233J'
,'S52233K'
,'S52233M'
,'S52233N'
,'S52233P'
,'S52233Q'
,'S52233R'
,'S52233S'
,'S52234C'
,'S52234D'
,'S52234E'
,'S52234F'
,'S52234G'
,'S52234H'
,'S52234J'
,'S52234K'
,'S52234M'
,'S52234N'
,'S52234P'
,'S52234Q'
,'S52234R'
,'S52234S'
,'S52235C'
,'S52235D'
,'S52235E'
,'S52235F'
,'S52235G'
,'S52235H'
,'S52235J'
,'S52235K'
,'S52235M'
,'S52235N'
,'S52235P'
,'S52235Q'
,'S52235R'
,'S52235S'
,'S52236C'
,'S52236D'
,'S52236E'
,'S52236F'
,'S52236G'
,'S52236H'
,'S52236J'
,'S52236K'
,'S52236M'
,'S52236N'
,'S52236P'
,'S52236Q'
,'S52236R'
,'S52236S'
,'S52241C'
,'S52241D'
,'S52241E'
,'S52241F'
,'S52241G'
,'S52241H'
,'S52241J'
,'S52241K'
,'S52241M'
,'S52241N'
,'S52241P'
,'S52241Q'
,'S52241R'
,'S52241S'
,'S52242C'
,'S52242D'
,'S52242E'
,'S52242F'
,'S52242G'
,'S52242H'
,'S52242J'
,'S52242K'
,'S52242M'
,'S52242N'
,'S52242P'
,'S52242Q'
,'S52242R'
,'S52242S'
,'S52243C'
,'S52243D'
,'S52243E'
,'S52243F'
,'S52243G'
,'S52243H'
,'S52243J'
,'S52243K'
,'S52243M'
,'S52243N'
,'S52243P'
,'S52243Q'
,'S52243R'
,'S52243S'
,'S52244C'
,'S52244D'
,'S52244E'
,'S52244F'
,'S52244G'
,'S52244H'
,'S52244J'
,'S52244K'
,'S52244M'
,'S52244N'
,'S52244P'
,'S52244Q'
,'S52244R'
,'S52244S'
,'S52245C'
,'S52245D'
,'S52245E'
,'S52245F'
,'S52245G'
,'S52245H'
,'S52245J'
,'S52245K'
,'S52245M'
,'S52245N'
,'S52245P'
,'S52245Q'
,'S52245R'
,'S52245S'
,'S52246C'
,'S52246D'
,'S52246E'
,'S52246F'
,'S52246G'
,'S52246H'
,'S52246J'
,'S52246K'
,'S52246M'
,'S52246N'
,'S52246P'
,'S52246Q'
,'S52246R'
,'S52246S'
,'S52251C'
,'S52251D'
,'S52251E'
,'S52251F'
,'S52251G'
,'S52251H'
,'S52251J'
,'S52251K'
,'S52251M'
,'S52251N'
,'S52251P'
,'S52251Q'
,'S52251R'
,'S52251S'
,'S52252C'
,'S52252D'
,'S52252E'
,'S52252F'
,'S52252G'
,'S52252H'
,'S52252J'
,'S52252K'
,'S52252M'
,'S52252N'
,'S52252P'
,'S52252Q'
,'S52252R'
,'S52252S'
,'S52253C'
,'S52253D'
,'S52253E'
,'S52253F'
,'S52253G'
,'S52253H'
,'S52253J'
,'S52253K'
,'S52253M'
,'S52253N'
,'S52253P'
,'S52253Q'
,'S52253R'
,'S52253S'
,'S52254C'
,'S52254D'
,'S52254E'
,'S52254F'
,'S52254G'
,'S52254H'
,'S52254J'
,'S52254K'
,'S52254M'
,'S52254N'
,'S52254P'
,'S52254Q'
,'S52254R'
,'S52254S'
,'S52255C'
,'S52255D'
,'S52255E'
,'S52255F'
,'S52255G'
,'S52255H'
,'S52255J'
,'S52255K'
,'S52255M'
,'S52255N'
,'S52255P'
,'S52255Q'
,'S52255R'
,'S52255S'
,'S52256C'
,'S52256D'
,'S52256E'
,'S52256F'
,'S52256G'
,'S52256H'
,'S52256J'
,'S52256K'
,'S52256M'
,'S52256N'
,'S52256P'
,'S52256Q'
,'S52256R'
,'S52256S'
,'S52261C'
,'S52261D'
,'S52261E'
,'S52261F'
,'S52261G'
,'S52261H'
,'S52261J'
,'S52261K'
,'S52261M'
,'S52261N'
,'S52261P'
,'S52261Q'
,'S52261R'
,'S52261S'
,'S52262C'
,'S52262D'
,'S52262E'
,'S52262F'
,'S52262G'
,'S52262H'
,'S52262J'
,'S52262K'
,'S52262M'
,'S52262N'
,'S52262P'
,'S52262Q'
,'S52262R'
,'S52262S'
,'S52263C'
,'S52263D'
,'S52263E'
,'S52263F'
,'S52263G'
,'S52263H'
,'S52263J'
,'S52263K'
,'S52263M'
,'S52263N'
,'S52263P'
,'S52263Q'
,'S52263R'
,'S52263S'
,'S52264C'
,'S52264D'
,'S52264E'
,'S52264F'
,'S52264G'
,'S52264H'
,'S52264J'
,'S52264K'
,'S52264M'
,'S52264N'
,'S52264P'
,'S52264Q'
,'S52264R'
,'S52264S'
,'S52265C'
,'S52265D'
,'S52265E'
,'S52265F'
,'S52265G'
,'S52265H'
,'S52265J'
,'S52265K'
,'S52265M'
,'S52265N'
,'S52265P'
,'S52265Q'
,'S52265R'
,'S52265S'
,'S52266C'
,'S52266D'
,'S52266E'
,'S52266F'
,'S52266G'
,'S52266H'
,'S52266J'
,'S52266K'
,'S52266M'
,'S52266N'
,'S52266P'
,'S52266Q'
,'S52266R'
,'S52266S'
,'S52271C'
,'S52271D'
,'S52271E'
,'S52271F'
,'S52271G'
,'S52271H'
,'S52271J'
,'S52271K'
,'S52271M'
,'S52271N'
,'S52271P'
,'S52271Q'
,'S52271R'
,'S52271S'
,'S52272C'
,'S52272D'
,'S52272E'
,'S52272F'
,'S52272G'
,'S52272H'
,'S52272J'
,'S52272K'
,'S52272M'
,'S52272N'
,'S52272P'
,'S52272Q'
,'S52272R'
,'S52272S'
,'S52279C'
,'S52279D'
,'S52279E'
,'S52279F'
,'S52279G'
,'S52279H'
,'S52279J'
,'S52279K'
,'S52279M'
,'S52279N'
,'S52279P'
,'S52279Q'
,'S52279R'
,'S52279S'
,'S52281C'
,'S52281D'
,'S52281E'
,'S52281F'
,'S52281G'
,'S52281H'
,'S52281J'
,'S52281K'
,'S52281M'
,'S52281N'
,'S52281P'
,'S52281Q'
,'S52281R'
,'S52281S'
,'S52282C'
,'S52282D'
,'S52282E'
,'S52282F'
,'S52282G'
,'S52282H'
,'S52282J'
,'S52282K'
,'S52282M'
,'S52282N'
,'S52282P'
,'S52282Q'
,'S52282R'
,'S52282S'
,'S52283C'
,'S52283D'
,'S52283E'
,'S52283F'
,'S52283G'
,'S52283H'
,'S52283J'
,'S52283K'
,'S52283M'
,'S52283N'
,'S52283P'
,'S52283Q'
,'S52283R'
,'S52283S'
,'S52291C'
,'S52291D'
,'S52291E'
,'S52291F'
,'S52291G'
,'S52291H'
,'S52291J'
,'S52291K'
,'S52291M'
,'S52291N'
,'S52291P'
,'S52291Q'
,'S52291R'
,'S52291S'
,'S52292C'
,'S52292D'
,'S52292E'
,'S52292F'
,'S52292G'
,'S52292H'
,'S52292J'
,'S52292K'
,'S52292M'
,'S52292N'
,'S52292P'
,'S52292Q'
,'S52292R'
,'S52292S'
,'S52299C'
,'S52299D'
,'S52299E'
,'S52299F'
,'S52299G'
,'S52299H'
,'S52299J'
,'S52299K'
,'S52299M'
,'S52299N'
,'S52299P'
,'S52299Q'
,'S52299R'
,'S52299S'
,'S52301C'
,'S52301D'
,'S52301E'
,'S52301F'
,'S52301G'
,'S52301H'
,'S52301J'
,'S52301K'
,'S52301M'
,'S52301N'
,'S52301P'
,'S52301Q'
,'S52301R'
,'S52301S'
,'S52302C'
,'S52302D'
,'S52302E'
,'S52302F'
,'S52302G'
,'S52302H'
,'S52302J'
,'S52302K'
,'S52302M'
,'S52302N'
,'S52302P'
,'S52302Q'
,'S52302R'
,'S52302S'
,'S52309C'
,'S52309D'
,'S52309E'
,'S52309F'
,'S52309G'
,'S52309H'
,'S52309J'
,'S52309K'
,'S52309M'
,'S52309N'
,'S52309P'
,'S52309Q'
,'S52309R'
,'S52309S'
,'S52311D'
,'S52311G'
,'S52311K'
,'S52311P'
,'S52311S'
,'S52312D'
,'S52312G'
,'S52312K'
,'S52312P'
,'S52312S'
,'S52319D'
,'S52319G'
,'S52319K'
,'S52319P'
,'S52319S'
,'S52321C'
,'S52321D'
,'S52321E'
,'S52321F'
,'S52321G'
,'S52321H'
,'S52321J'
,'S52321K'
,'S52321M'
,'S52321N'
,'S52321P'
,'S52321Q'
,'S52321R'
,'S52321S'
,'S52322C'
,'S52322D'
,'S52322E'
,'S52322F'
,'S52322G'
,'S52322H'
,'S52322J'
,'S52322K'
,'S52322M'
,'S52322N'
,'S52322P'
,'S52322Q'
,'S52322R'
,'S52322S'
,'S52323C'
,'S52323D'
,'S52323E'
,'S52323F'
,'S52323G'
,'S52323H'
,'S52323J'
,'S52323K'
,'S52323M'
,'S52323N'
,'S52323P'
,'S52323Q'
,'S52323R'
,'S52323S'
,'S52324C'
,'S52324D'
,'S52324E'
,'S52324F'
,'S52324G'
,'S52324H'
,'S52324J'
,'S52324K'
,'S52324M'
,'S52324N'
,'S52324P'
,'S52324Q'
,'S52324R'
,'S52324S'
,'S52325C'
,'S52325D'
,'S52325E'
,'S52325F'
,'S52325G'
,'S52325H'
,'S52325J'
,'S52325K'
,'S52325M'
,'S52325N'
,'S52325P'
,'S52325Q'
,'S52325R'
,'S52325S'
,'S52326C'
,'S52326D'
,'S52326E'
,'S52326F'
,'S52326G'
,'S52326H'
,'S52326J'
,'S52326K'
,'S52326M'
,'S52326N'
,'S52326P'
,'S52326Q'
,'S52326R'
,'S52326S'
,'S52331C'
,'S52331D'
,'S52331E'
,'S52331F'
,'S52331G'
,'S52331H'
,'S52331J'
,'S52331K'
,'S52331M'
,'S52331N'
,'S52331P'
,'S52331Q'
,'S52331R'
,'S52331S'
,'S52332C'
,'S52332D'
,'S52332E'
,'S52332F'
,'S52332G'
,'S52332H'
,'S52332J'
,'S52332K'
,'S52332M'
,'S52332N'
,'S52332P'
,'S52332Q'
,'S52332R'
,'S52332S'
,'S52333C'
,'S52333D'
,'S52333E'
,'S52333F'
,'S52333G'
,'S52333H'
,'S52333J'
,'S52333K'
,'S52333M'
,'S52333N'
,'S52333P'
,'S52333Q'
,'S52333R'
,'S52333S'
,'S52334C'
,'S52334D'
,'S52334E'
,'S52334F'
,'S52334G'
,'S52334H'
,'S52334J'
,'S52334K'
,'S52334M'
,'S52334N'
,'S52334P'
,'S52334Q'
,'S52334R'
,'S52334S'
,'S52335C'
,'S52335D'
,'S52335E'
,'S52335F'
,'S52335G'
,'S52335H'
,'S52335J'
,'S52335K'
,'S52335M'
,'S52335N'
,'S52335P'
,'S52335Q'
,'S52335R'
,'S52335S'
,'S52336C'
,'S52336D'
,'S52336E'
,'S52336F'
,'S52336G'
,'S52336H'
,'S52336J'
,'S52336K'
,'S52336M'
,'S52336N'
,'S52336P'
,'S52336Q'
,'S52336R'
,'S52336S'
,'S52341C'
,'S52341D'
,'S52341E'
,'S52341F'
,'S52341G'
,'S52341H'
,'S52341J'
,'S52341K'
,'S52341M'
,'S52341N'
,'S52341P'
,'S52341Q'
,'S52341R'
,'S52341S'
,'S52342C'
,'S52342D'
,'S52342E'
,'S52342F'
,'S52342G'
,'S52342H'
,'S52342J'
,'S52342K'
,'S52342M'
,'S52342N'
,'S52342P'
,'S52342Q'
,'S52342R'
,'S52342S'
,'S52343C'
,'S52343D'
,'S52343E'
,'S52343F'
,'S52343G'
,'S52343H'
,'S52343J'
,'S52343K'
,'S52343M'
,'S52343N'
,'S52343P'
,'S52343Q'
,'S52343R'
,'S52343S'
,'S52344C'
,'S52344D'
,'S52344E'
,'S52344F'
,'S52344G'
,'S52344H'
,'S52344J'
,'S52344K'
,'S52344M'
,'S52344N'
,'S52344P'
,'S52344Q'
,'S52344R'
,'S52344S'
,'S52345C'
,'S52345D'
,'S52345E'
,'S52345F'
,'S52345G'
,'S52345H'
,'S52345J'
,'S52345K'
,'S52345M'
,'S52345N'
,'S52345P'
,'S52345Q'
,'S52345R'
,'S52345S'
,'S52346C'
,'S52346D'
,'S52346E'
,'S52346F'
,'S52346G'
,'S52346H'
,'S52346J'
,'S52346K'
,'S52346M'
,'S52346N'
,'S52346P'
,'S52346Q'
,'S52346R'
,'S52346S'
,'S52351C'
,'S52351D'
,'S52351E'
,'S52351F'
,'S52351G'
,'S52351H'
,'S52351J'
,'S52351K'
,'S52351M'
,'S52351N'
,'S52351P'
,'S52351Q'
,'S52351R'
,'S52351S'
,'S52352C'
,'S52352D'
,'S52352E'
,'S52352F'
,'S52352G'
,'S52352H'
,'S52352J'
,'S52352K'
,'S52352M'
,'S52352N'
,'S52352P'
,'S52352Q'
,'S52352R'
,'S52352S'
,'S52353C'
,'S52353D'
,'S52353E'
,'S52353F'
,'S52353G'
,'S52353H'
,'S52353J'
,'S52353K'
,'S52353M'
,'S52353N'
,'S52353P'
,'S52353Q'
,'S52353R'
,'S52353S'
,'S52354C'
,'S52354D'
,'S52354E'
,'S52354F'
,'S52354G'
,'S52354H'
,'S52354J'
,'S52354K'
,'S52354M'
,'S52354N'
,'S52354P'
,'S52354Q'
,'S52354R'
,'S52354S'
,'S52355C'
,'S52355D'
,'S52355E'
,'S52355F'
,'S52355G'
,'S52355H'
,'S52355J'
,'S52355K'
,'S52355M'
,'S52355N'
,'S52355P'
,'S52355Q'
,'S52355R'
,'S52355S'
,'S52356C'
,'S52356D'
,'S52356E'
,'S52356F'
,'S52356G'
,'S52356H'
,'S52356J'
,'S52356K'
,'S52356M'
,'S52356N'
,'S52356P'
,'S52356Q'
,'S52356R'
,'S52356S'
,'S52361C'
,'S52361D'
,'S52361E'
,'S52361F'
,'S52361G'
,'S52361H'
,'S52361J'
,'S52361K'
,'S52361M'
,'S52361N'
,'S52361P'
,'S52361Q'
,'S52361R'
,'S52361S'
,'S52362C'
,'S52362D'
,'S52362E'
,'S52362F'
,'S52362G'
,'S52362H'
,'S52362J'
,'S52362K'
,'S52362M'
,'S52362N'
,'S52362P'
,'S52362Q'
,'S52362R'
,'S52362S'
,'S52363C'
,'S52363D'
,'S52363E'
,'S52363F'
,'S52363G'
,'S52363H'
,'S52363J'
,'S52363K'
,'S52363M'
,'S52363N'
,'S52363P'
,'S52363Q'
,'S52363R'
,'S52363S'
,'S52364C'
,'S52364D'
,'S52364E'
,'S52364F'
,'S52364G'
,'S52364H'
,'S52364J'
,'S52364K'
,'S52364M'
,'S52364N'
,'S52364P'
,'S52364Q'
,'S52364R'
,'S52364S'
,'S52365C'
,'S52365D'
,'S52365E'
,'S52365F'
,'S52365G'
,'S52365H'
,'S52365J'
,'S52365K'
,'S52365M'
,'S52365N'
,'S52365P'
,'S52365Q'
,'S52365R'
,'S52365S'
,'S52366C'
,'S52366D'
,'S52366E'
,'S52366F'
,'S52366G'
,'S52366H'
,'S52366J'
,'S52366K'
,'S52366M'
,'S52366N'
,'S52366P'
,'S52366Q'
,'S52366R'
,'S52366S'
,'S52371C'
,'S52371D'
,'S52371E'
,'S52371F'
,'S52371G'
,'S52371H'
,'S52371J'
,'S52371K'
,'S52371M'
,'S52371N'
,'S52371P'
,'S52371Q'
,'S52371R'
,'S52371S'
,'S52372C'
,'S52372D'
,'S52372E'
,'S52372F'
,'S52372G'
,'S52372H'
,'S52372J'
,'S52372K'
,'S52372M'
,'S52372N'
,'S52372P'
,'S52372Q'
,'S52372R'
,'S52372S'
,'S52379C'
,'S52379D'
,'S52379E'
,'S52379F'
,'S52379G'
,'S52379H'
,'S52379J'
,'S52379K'
,'S52379M'
,'S52379N'
,'S52379P'
,'S52379Q'
,'S52379R'
,'S52379S'
,'S52381C'
,'S52381D'
,'S52381E'
,'S52381F'
,'S52381G'
,'S52381H'
,'S52381J'
,'S52381K'
,'S52381M'
,'S52381N'
,'S52381P'
,'S52381Q'
,'S52381R'
,'S52381S'
,'S52382C'
,'S52382D'
,'S52382E'
,'S52382F'
,'S52382G'
,'S52382H'
,'S52382J'
,'S52382K'
,'S52382M'
,'S52382N'
,'S52382P'
,'S52382Q'
,'S52382R'
,'S52382S'
,'S52389C'
,'S52389D'
,'S52389E'
,'S52389F'
,'S52389G'
,'S52389H'
,'S52389J'
,'S52389K'
,'S52389M'
,'S52389N'
,'S52389P'
,'S52389Q'
,'S52389R'
,'S52389S'
,'S52391C'
,'S52391D'
,'S52391E'
,'S52391F'
,'S52391G'
,'S52391H'
,'S52391J'
,'S52391K'
,'S52391M'
,'S52391N'
,'S52391P'
,'S52391Q'
,'S52391R'
,'S52391S'
,'S52392C'
,'S52392D'
,'S52392E'
,'S52392F'
,'S52392G'
,'S52392H'
,'S52392J'
,'S52392K'
,'S52392M'
,'S52392N'
,'S52392P'
,'S52392Q'
,'S52392R'
,'S52392S'
,'S52399C'
,'S52399D'
,'S52399E'
,'S52399F'
,'S52399G'
,'S52399H'
,'S52399J'
,'S52399K'
,'S52399M'
,'S52399N'
,'S52399P'
,'S52399Q'
,'S52399R'
,'S52399S'
,'S52501C'
,'S52501D'
,'S52501E'
,'S52501F'
,'S52501G'
,'S52501H'
,'S52501J'
,'S52501K'
,'S52501M'
,'S52501N'
,'S52501P'
,'S52501Q'
,'S52501R'
,'S52501S'
,'S52502C'
,'S52502D'
,'S52502E'
,'S52502F'
,'S52502G'
,'S52502H'
,'S52502J'
,'S52502K'
,'S52502M'
,'S52502N'
,'S52502P'
,'S52502Q'
,'S52502R'
,'S52502S'
,'S52509C'
,'S52509D'
,'S52509E'
,'S52509F'
,'S52509G'
,'S52509H'
,'S52509J'
,'S52509K'
,'S52509M'
,'S52509N'
,'S52509P'
,'S52509Q'
,'S52509R'
,'S52509S'
,'S52511C'
,'S52511D'
,'S52511E'
,'S52511F'
,'S52511G'
,'S52511H'
,'S52511J'
,'S52511K'
,'S52511M'
,'S52511N'
,'S52511P'
,'S52511Q'
,'S52511R'
,'S52511S'
,'S52512C'
,'S52512D'
,'S52512E'
,'S52512F'
,'S52512G'
,'S52512H'
,'S52512J'
,'S52512K'
,'S52512M'
,'S52512N'
,'S52512P'
,'S52512Q'
,'S52512R'
,'S52512S'
,'S52513C'
,'S52513D'
,'S52513E'
,'S52513F'
,'S52513G'
,'S52513H'
,'S52513J'
,'S52513K'
,'S52513M'
,'S52513N'
,'S52513P'
,'S52513Q'
,'S52513R'
,'S52513S'
,'S52514C'
,'S52514D'
,'S52514E'
,'S52514F'
,'S52514G'
,'S52514H'
,'S52514J'
,'S52514K'
,'S52514M'
,'S52514N'
,'S52514P'
,'S52514Q'
,'S52514R'
,'S52514S'
,'S52515C'
,'S52515D'
,'S52515E'
,'S52515F'
,'S52515G'
,'S52515H'
,'S52515J'
,'S52515K'
,'S52515M'
,'S52515N'
,'S52515P'
,'S52515Q'
,'S52515R'
,'S52515S'
,'S52516C'
,'S52516D'
,'S52516E'
,'S52516F'
,'S52516G'
,'S52516H'
,'S52516J'
,'S52516K'
,'S52516M'
,'S52516N'
,'S52516P'
,'S52516Q'
,'S52516R'
,'S52516S'
,'S52521D'
,'S52521G'
,'S52521K'
,'S52521P'
,'S52521S'
,'S52522D'
,'S52522G'
,'S52522K'
,'S52522P'
,'S52522S'
,'S52529D'
,'S52529G'
,'S52529K'
,'S52529P'
,'S52529S'
,'S52531C'
,'S52531D'
,'S52531E'
,'S52531F'
,'S52531G'
,'S52531H'
,'S52531J'
,'S52531K'
,'S52531M'
,'S52531N'
,'S52531P'
,'S52531Q'
,'S52531R'
,'S52531S'
,'S52532C'
,'S52532D'
,'S52532E'
,'S52532F'
,'S52532G'
,'S52532H'
,'S52532J'
,'S52532K'
,'S52532M'
,'S52532N'
,'S52532P'
,'S52532Q'
,'S52532R'
,'S52532S'
,'S52539C'
,'S52539D'
,'S52539E'
,'S52539F'
,'S52539G'
,'S52539H'
,'S52539J'
,'S52539K'
,'S52539M'
,'S52539N'
,'S52539P'
,'S52539Q'
,'S52539R'
,'S52539S'
,'S52541C'
,'S52541D'
,'S52541E'
,'S52541F'
,'S52541G'
,'S52541H'
,'S52541J'
,'S52541K'
,'S52541M'
,'S52541N'
,'S52541P'
,'S52541Q'
,'S52541R'
,'S52541S'
,'S52542C'
,'S52542D'
,'S52542E'
,'S52542F'
,'S52542G'
,'S52542H'
,'S52542J'
,'S52542K'
,'S52542M'
,'S52542N'
,'S52542P'
,'S52542Q'
,'S52542R'
,'S52542S'
,'S52549C'
,'S52549D'
,'S52549E'
,'S52549F'
,'S52549G'
,'S52549H'
,'S52549J'
,'S52549K'
,'S52549M'
,'S52549N'
,'S52549P'
,'S52549Q'
,'S52549R'
,'S52549S'
,'S52551C'
,'S52551D'
,'S52551E'
,'S52551F'
,'S52551G'
,'S52551H'
,'S52551J'
,'S52551K'
,'S52551M'
,'S52551N'
,'S52551P'
,'S52551Q'
,'S52551R'
,'S52551S'
,'S52552C'
,'S52552D'
,'S52552E'
,'S52552F'
,'S52552G'
,'S52552H'
,'S52552J'
,'S52552K'
,'S52552M'
,'S52552N'
,'S52552P'
,'S52552Q'
,'S52552R'
,'S52552S'
,'S52559C'
,'S52559D'
,'S52559E'
,'S52559F'
,'S52559G'
,'S52559H'
,'S52559J'
,'S52559K'
,'S52559M'
,'S52559N'
,'S52559P'
,'S52559Q'
,'S52559R'
,'S52559S'
,'S52561C'
,'S52561D'
,'S52561E'
,'S52561F'
,'S52561G'
,'S52561H'
,'S52561J'
,'S52561K'
,'S52561M'
,'S52561N'
,'S52561P'
,'S52561Q'
,'S52561R'
,'S52561S'
,'S52562C'
,'S52562D'
,'S52562E'
,'S52562F'
,'S52562G'
,'S52562H'
,'S52562J'
,'S52562K'
,'S52562M'
,'S52562N'
,'S52562P'
,'S52562Q'
,'S52562R'
,'S52562S'
,'S52569C'
,'S52569D'
,'S52569E'
,'S52569F'
,'S52569G'
,'S52569H'
,'S52569J'
,'S52569K'
,'S52569M'
,'S52569N'
,'S52569P'
,'S52569Q'
,'S52569R'
,'S52569S'
,'S52571C'
,'S52571D'
,'S52571E'
,'S52571F'
,'S52571G'
,'S52571H'
,'S52571J'
,'S52571K'
,'S52571M'
,'S52571N'
,'S52571P'
,'S52571Q'
,'S52571R'
,'S52571S'
,'S52572C'
,'S52572D'
,'S52572E'
,'S52572F'
,'S52572G'
,'S52572H'
,'S52572J'
,'S52572K'
,'S52572M'
,'S52572N'
,'S52572P'
,'S52572Q'
,'S52572R'
,'S52572S'
,'S52579C'
,'S52579D'
,'S52579E'
,'S52579F'
,'S52579G'
,'S52579H'
,'S52579J'
,'S52579K'
,'S52579M'
,'S52579N'
,'S52579P'
,'S52579Q'
,'S52579R'
,'S52579S'
,'S52591C'
,'S52591D'
,'S52591E'
,'S52591F'
,'S52591G'
,'S52591H'
,'S52591J'
,'S52591K'
,'S52591M'
,'S52591N'
,'S52591P'
,'S52591Q'
,'S52591R'
,'S52591S'
,'S52592C'
,'S52592D'
,'S52592E'
,'S52592F'
,'S52592G'
,'S52592H'
,'S52592J'
,'S52592K'
,'S52592M'
,'S52592N'
,'S52592P'
,'S52592Q'
,'S52592R'
,'S52592S'
,'S52599C'
,'S52599D'
,'S52599E'
,'S52599F'
,'S52599G'
,'S52599H'
,'S52599J'
,'S52599K'
,'S52599M'
,'S52599N'
,'S52599P'
,'S52599Q'
,'S52599R'
,'S52599S'
,'S52601C'
,'S52601D'
,'S52601E'
,'S52601F'
,'S52601G'
,'S52601H'
,'S52601J'
,'S52601K'
,'S52601M'
,'S52601N'
,'S52601P'
,'S52601Q'
,'S52601R'
,'S52601S'
,'S52602C'
,'S52602D'
,'S52602E'
,'S52602F'
,'S52602G'
,'S52602H'
,'S52602J'
,'S52602K'
,'S52602M'
,'S52602N'
,'S52602P'
,'S52602Q'
,'S52602R'
,'S52602S'
,'S52609C'
,'S52609D'
,'S52609E'
,'S52609F'
,'S52609G'
,'S52609H'
,'S52609J'
,'S52609K'
,'S52609M'
,'S52609N'
,'S52609P'
,'S52609Q'
,'S52609R'
,'S52609S'
,'S52611C'
,'S52611D'
,'S52611E'
,'S52611F'
,'S52611G'
,'S52611H'
,'S52611J'
,'S52611K'
,'S52611M'
,'S52611N'
,'S52611P'
,'S52611Q'
,'S52611R'
,'S52611S'
,'S52612C'
,'S52612D'
,'S52612E'
,'S52612F'
,'S52612G'
,'S52612H'
,'S52612J'
,'S52612K'
,'S52612M'
,'S52612N'
,'S52612P'
,'S52612Q'
,'S52612R'
,'S52612S'
,'S52613C'
,'S52613D'
,'S52613E'
,'S52613F'
,'S52613G'
,'S52613H'
,'S52613J'
,'S52613K'
,'S52613M'
,'S52613N'
,'S52613P'
,'S52613Q'
,'S52613R'
,'S52613S'
,'S52614C'
,'S52614D'
,'S52614E'
,'S52614F'
,'S52614G'
,'S52614H'
,'S52614J'
,'S52614K'
,'S52614M'
,'S52614N'
,'S52614P'
,'S52614Q'
,'S52614R'
,'S52614S'
,'S52615C'
,'S52615D'
,'S52615E'
,'S52615F'
,'S52615G'
,'S52615H'
,'S52615J'
,'S52615K'
,'S52615M'
,'S52615N'
,'S52615P'
,'S52615Q'
,'S52615R'
,'S52615S'
,'S52616C'
,'S52616D'
,'S52616E'
,'S52616F'
,'S52616G'
,'S52616H'
,'S52616J'
,'S52616K'
,'S52616M'
,'S52616N'
,'S52616P'
,'S52616Q'
,'S52616R'
,'S52616S'
,'S52621D'
,'S52621G'
,'S52621K'
,'S52621P'
,'S52621S'
,'S52622D'
,'S52622G'
,'S52622K'
,'S52622P'
,'S52622S'
,'S52629D'
,'S52629G'
,'S52629K'
,'S52629P'
,'S52629S'
,'S52691C'
,'S52691D'
,'S52691E'
,'S52691F'
,'S52691G'
,'S52691H'
,'S52691J'
,'S52691K'
,'S52691M'
,'S52691N'
,'S52691P'
,'S52691Q'
,'S52691R'
,'S52691S'
,'S52692C'
,'S52692D'
,'S52692E'
,'S52692F'
,'S52692G'
,'S52692H'
,'S52692J'
,'S52692K'
,'S52692M'
,'S52692N'
,'S52692P'
,'S52692Q'
,'S52692R'
,'S52692S'
,'S52699C'
,'S52699D'
,'S52699E'
,'S52699F'
,'S52699G'
,'S52699H'
,'S52699J'
,'S52699K'
,'S52699M'
,'S52699N'
,'S52699P'
,'S52699Q'
,'S52699R'
,'S52699S'
,'S5290XC'
,'S5290XD'
,'S5290XE'
,'S5290XF'
,'S5290XG'
,'S5290XH'
,'S5290XJ'
,'S5290XK'
,'S5290XM'
,'S5290XN'
,'S5290XP'
,'S5290XQ'
,'S5290XR'
,'S5290XS'
,'S5291XC'
,'S5291XD'
,'S5291XE'
,'S5291XF'
,'S5291XG'
,'S5291XH'
,'S5291XJ'
,'S5291XK'
,'S5291XM'
,'S5291XN'
,'S5291XP'
,'S5291XQ'
,'S5291XR'
,'S5291XS'
,'S5292XC'
,'S5292XD'
,'S5292XE'
,'S5292XF'
,'S5292XG'
,'S5292XH'
,'S5292XJ'
,'S5292XK'
,'S5292XM'
,'S5292XN'
,'S5292XP'
,'S5292XQ'
,'S5292XR'
,'S5292XS'
,'S53001D'
,'S53001S'
,'S53002D'
,'S53002S'
,'S53003D'
,'S53003S'
,'S53004D'
,'S53004S'
,'S53005D'
,'S53005S'
,'S53006D'
,'S53006S'
,'S53011D'
,'S53011S'
,'S53012D'
,'S53012S'
,'S53013D'
,'S53013S'
,'S53014D'
,'S53014S'
,'S53015D'
,'S53015S'
,'S53016D'
,'S53016S'
,'S53021D'
,'S53021S'
,'S53022D'
,'S53022S'
,'S53023D'
,'S53023S'
,'S53024D'
,'S53024S'
,'S53025D'
,'S53025S'
,'S53026D'
,'S53026S'
,'S53031D'
,'S53031S'
,'S53032D'
,'S53032S'
,'S53033D'
,'S53033S'
,'S53091D'
,'S53091S'
,'S53092D'
,'S53092S'
,'S53093D'
,'S53093S'
,'S53094D'
,'S53094S'
,'S53095D'
,'S53095S'
,'S53096D'
,'S53096S'
,'S53101D'
,'S53101S'
,'S53102D'
,'S53102S'
,'S53103D'
,'S53103S'
,'S53104D'
,'S53104S'
,'S53105D'
,'S53105S'
,'S53106D'
,'S53106S'
,'S53111D'
,'S53111S'
,'S53112D'
,'S53112S'
,'S53113D'
,'S53113S'
,'S53114D'
,'S53114S'
,'S53115D'
,'S53115S'
,'S53116D'
,'S53116S'
,'S53121D'
,'S53121S'
,'S53122D'
,'S53122S'
,'S53123D'
,'S53123S'
,'S53124D'
,'S53124S'
,'S53125D'
,'S53125S'
,'S53126D'
,'S53126S'
,'S53131D'
,'S53131S'
,'S53132D'
,'S53132S'
,'S53133D'
,'S53133S'
,'S53134D'
,'S53134S'
,'S53135D'
,'S53135S'
,'S53136D'
,'S53136S'
,'S53141D'
,'S53141S'
,'S53142D'
,'S53142S'
,'S53143D'
,'S53143S'
,'S53144D'
,'S53144S'
,'S53145D'
,'S53145S'
,'S53146D'
,'S53146S'
,'S53191D'
,'S53191S'
,'S53192D'
,'S53192S'
,'S53193D'
,'S53193S'
,'S53194D'
,'S53194S'
,'S53195D'
,'S53195S'
,'S53196D'
,'S53196S'
,'S5320XD'
,'S5320XS'
,'S5321XD'
,'S5321XS'
,'S5322XD'
,'S5322XS'
,'S5330XD'
,'S5330XS'
,'S5331XD'
,'S5331XS'
,'S5332XD'
,'S5332XS'
,'S53401D'
,'S53401S'
,'S53402D'
,'S53402S'
,'S53409D'
,'S53409S'
,'S53411D'
,'S53411S'
,'S53412D'
,'S53412S'
,'S53419D'
,'S53419S'
,'S53421D'
,'S53421S'
,'S53422D'
,'S53422S'
,'S53429D'
,'S53429S'
,'S53431D'
,'S53431S'
,'S53432D'
,'S53432S'
,'S53439D'
,'S53439S'
,'S53441D'
,'S53441S'
,'S53442D'
,'S53442S'
,'S53449D'
,'S53449S'
,'S53491D'
,'S53491S'
,'S53492D'
,'S53492S'
,'S53499D'
,'S53499S'
,'S5400XD'
,'S5400XS'
,'S5401XD'
,'S5401XS'
,'S5402XD'
,'S5402XS'
,'S5410XD'
,'S5410XS'
,'S5411XD'
,'S5411XS'
,'S5412XD'
,'S5412XS'
,'S5420XD'
,'S5420XS'
,'S5421XD'
,'S5421XS'
,'S5422XD'
,'S5422XS'
,'S5430XD'
,'S5430XS'
,'S5431XD'
,'S5431XS'
,'S5432XD'
,'S5432XS'
,'S548X1D'
,'S548X1S'
,'S548X2D'
,'S548X2S'
,'S548X9D'
,'S548X9S'
,'S5490XD'
,'S5490XS'
,'S5491XD'
,'S5491XS'
,'S5492XD'
,'S5492XS'
,'S55001D'
,'S55001S'
,'S55002D'
,'S55002S'
,'S55009D'
,'S55009S'
,'S55011D'
,'S55011S'
,'S55012D'
,'S55012S'
,'S55019D'
,'S55019S'
,'S55091D'
,'S55091S'
,'S55092D'
,'S55092S'
,'S55099D'
,'S55099S'
,'S55101D'
,'S55101S'
,'S55102D'
,'S55102S'
,'S55109D'
,'S55109S'
,'S55111D'
,'S55111S'
,'S55112D'
,'S55112S'
,'S55119D'
,'S55119S'
,'S55191D'
,'S55191S'
,'S55192D'
,'S55192S'
,'S55199D'
,'S55199S'
,'S55201D'
,'S55201S'
,'S55202D'
,'S55202S'
,'S55209D'
,'S55209S'
,'S55211D'
,'S55211S'
,'S55212D'
,'S55212S'
,'S55219D'
,'S55219S'
,'S55291D'
,'S55291S'
,'S55292D'
,'S55292S'
,'S55299D'
,'S55299S'
,'S55801D'
,'S55801S'
,'S55802D'
,'S55802S'
,'S55809D'
,'S55809S'
,'S55811D'
,'S55811S'
,'S55812D'
,'S55812S'
,'S55819D'
,'S55819S'
,'S55891D'
,'S55891S'
,'S55892D'
,'S55892S'
,'S55899D'
,'S55899S'
,'S55901D'
,'S55901S'
,'S55902D'
,'S55902S'
,'S55909D'
,'S55909S'
,'S55911D'
,'S55911S'
,'S55912D'
,'S55912S'
,'S55919D'
,'S55919S'
,'S55991D'
,'S55991S'
,'S55992D'
,'S55992S'
,'S55999D'
,'S55999S'
,'S56001D'
,'S56001S'
,'S56002D'
,'S56002S'
,'S56009D'
,'S56009S'
,'S56011D'
,'S56011S'
,'S56012D'
,'S56012S'
,'S56019D'
,'S56019S'
,'S56021D'
,'S56021S'
,'S56022D'
,'S56022S'
,'S56029D'
,'S56029S'
,'S56091D'
,'S56091S'
,'S56092D'
,'S56092S'
,'S56099D'
,'S56099S'
,'S56101D'
,'S56101S'
,'S56102D'
,'S56102S'
,'S56103D'
,'S56103S'
,'S56104D'
,'S56104S'
,'S56105D'
,'S56105S'
,'S56106D'
,'S56106S'
,'S56107D'
,'S56107S'
,'S56108D'
,'S56108S'
,'S56109D'
,'S56109S'
,'S56111D'
,'S56111S'
,'S56112D'
,'S56112S'
,'S56113D'
,'S56113S'
,'S56114D'
,'S56114S'
,'S56115D'
,'S56115S'
,'S56116D'
,'S56116S'
,'S56117D'
,'S56117S'
,'S56118D'
,'S56118S'
,'S56119D'
,'S56119S'
,'S56121D'
,'S56121S'
,'S56122D'
,'S56122S'
,'S56123D'
,'S56123S'
,'S56124D'
,'S56124S'
,'S56125D'
,'S56125S'
,'S56126D'
,'S56126S'
,'S56127D'
,'S56127S'
,'S56128D'
,'S56128S'
,'S56129D'
,'S56129S'
,'S56191D'
,'S56191S'
,'S56192D'
,'S56192S'
,'S56193D'
,'S56193S'
,'S56194D'
,'S56194S'
,'S56195D'
,'S56195S'
,'S56196D'
,'S56196S'
,'S56197D'
,'S56197S'
,'S56198D'
,'S56198S'
,'S56199D'
,'S56199S'
,'S56201D'
,'S56201S'
,'S56202D'
,'S56202S'
,'S56209D'
,'S56209S'
,'S56211D'
,'S56211S'
,'S56212D'
,'S56212S'
,'S56219D'
,'S56219S'
,'S56221D'
,'S56221S'
,'S56222D'
,'S56222S'
,'S56229D'
,'S56229S'
,'S56291D'
,'S56291S'
,'S56292D'
,'S56292S'
,'S56299D'
,'S56299S'
,'S56301D'
,'S56301S'
,'S56302D'
,'S56302S'
,'S56309D'
,'S56309S'
,'S56311D'
,'S56311S'
,'S56312D'
,'S56312S'
,'S56319D'
,'S56319S'
,'S56321D'
,'S56321S'
,'S56322D'
,'S56322S'
,'S56329D'
,'S56329S'
,'S56391D'
,'S56391S'
,'S56392D'
,'S56392S'
,'S56399D'
,'S56399S'
,'S56401D'
,'S56401S'
,'S56402D'
,'S56402S'
,'S56403D'
,'S56403S'
,'S56404D'
,'S56404S'
,'S56405D'
,'S56405S'
,'S56406D'
,'S56406S'
,'S56407D'
,'S56407S'
,'S56408D'
,'S56408S'
,'S56409D'
,'S56409S'
,'S56411D'
,'S56411S'
,'S56412D'
,'S56412S'
,'S56413D'
,'S56413S'
,'S56414D'
,'S56414S'
,'S56415D'
,'S56415S'
,'S56416D'
,'S56416S'
,'S56417D'
,'S56417S'
,'S56418D'
,'S56418S'
,'S56419D'
,'S56419S'
,'S56421D'
,'S56421S'
,'S56422D'
,'S56422S'
,'S56423D'
,'S56423S'
,'S56424D'
,'S56424S'
,'S56425D'
,'S56425S'
,'S56426D'
,'S56426S'
,'S56427D'
,'S56427S'
,'S56428D'
,'S56428S'
,'S56429D'
,'S56429S'
,'S56491D'
,'S56491S'
,'S56492D'
,'S56492S'
,'S56493D'
,'S56493S'
,'S56494D'
,'S56494S'
,'S56495D'
,'S56495S'
,'S56496D'
,'S56496S'
,'S56497D'
,'S56497S'
,'S56498D'
,'S56498S'
,'S56499D'
,'S56499S'
,'S56501D'
,'S56501S'
,'S56502D'
,'S56502S'
,'S56509D'
,'S56509S'
,'S56511D'
,'S56511S'
,'S56512D'
,'S56512S'
,'S56519D'
,'S56519S'
,'S56521D'
,'S56521S'
,'S56522D'
,'S56522S'
,'S56529D'
,'S56529S'
,'S56591D'
,'S56591S'
,'S56592D'
,'S56592S'
,'S56599D'
,'S56599S'
,'S56801D'
,'S56801S'
,'S56802D'
,'S56802S'
,'S56809D'
,'S56809S'
,'S56811D'
,'S56811S'
,'S56812D'
,'S56812S'
,'S56819D'
,'S56819S'
,'S56821D'
,'S56821S'
,'S56822D'
,'S56822S'
,'S56829D'
,'S56829S'
,'S56891D'
,'S56891S'
,'S56892D'
,'S56892S'
,'S56899D'
,'S56899S'
,'S56901D'
,'S56901S'
,'S56902D'
,'S56902S'
,'S56909D'
,'S56909S'
,'S56911D'
,'S56911S'
,'S56912D'
,'S56912S'
,'S56919D'
,'S56919S'
,'S56921D'
,'S56921S'
,'S56922D'
,'S56922S'
,'S56929D'
,'S56929S'
,'S56991D'
,'S56991S'
,'S56992D'
,'S56992S'
,'S56999D'
,'S56999S'
,'S5700XD'
,'S5700XS'
,'S5701XD'
,'S5701XS'
,'S5702XD'
,'S5702XS'
,'S5780XD'
,'S5780XS'
,'S5781XD'
,'S5781XS'
,'S5782XD'
,'S5782XS'
,'S58011D'
,'S58011S'
,'S58012D'
,'S58012S'
,'S58019D'
,'S58019S'
,'S58021D'
,'S58021S'
,'S58022D'
,'S58022S'
,'S58029D'
,'S58029S'
,'S58111D'
,'S58111S'
,'S58112D'
,'S58112S'
,'S58119D'
,'S58119S'
,'S58121D'
,'S58121S'
,'S58122D'
,'S58122S'
,'S58129D'
,'S58129S'
,'S58911D'
,'S58911S'
,'S58912D'
,'S58912S'
,'S58919D'
,'S58919S'
,'S58921D'
,'S58921S'
,'S58922D'
,'S58922S'
,'S58929D'
,'S58929S'
,'S59001D'
,'S59001G'
,'S59001K'
,'S59001P'
,'S59001S'
,'S59002D'
,'S59002G'
,'S59002K'
,'S59002P'
,'S59002S'
,'S59009D'
,'S59009G'
,'S59009K'
,'S59009P'
,'S59009S'
,'S59011D'
,'S59011G'
,'S59011K'
,'S59011P'
,'S59011S'
,'S59012D'
,'S59012G'
,'S59012K'
,'S59012P'
,'S59012S'
,'S59019D'
,'S59019G'
,'S59019K'
,'S59019P'
,'S59019S'
,'S59021D'
,'S59021G'
,'S59021K'
,'S59021P'
,'S59021S'
,'S59022D'
,'S59022G'
,'S59022K'
,'S59022P'
,'S59022S'
,'S59029D'
,'S59029G'
,'S59029K'
,'S59029P'
,'S59029S'
,'S59031D'
,'S59031G'
,'S59031K'
,'S59031P'
,'S59031S'
,'S59032D'
,'S59032G'
,'S59032K'
,'S59032P'
,'S59032S'
,'S59039D'
,'S59039G'
,'S59039K'
,'S59039P'
,'S59039S'
,'S59041D'
,'S59041G'
,'S59041K'
,'S59041P'
,'S59041S'
,'S59042D'
,'S59042G'
,'S59042K'
,'S59042P'
,'S59042S'
,'S59049D'
,'S59049G'
,'S59049K'
,'S59049P'
,'S59049S'
,'S59091D'
,'S59091G'
,'S59091K'
,'S59091P'
,'S59091S'
,'S59092D'
,'S59092G'
,'S59092K'
,'S59092P'
,'S59092S'
,'S59099D'
,'S59099G'
,'S59099K'
,'S59099P'
,'S59099S'
,'S59101D'
,'S59101G'
,'S59101K'
,'S59101P'
,'S59101S'
,'S59102D'
,'S59102G'
,'S59102K'
,'S59102P'
,'S59102S'
,'S59109D'
,'S59109G'
,'S59109K'
,'S59109P'
,'S59109S'
,'S59111D'
,'S59111G'
,'S59111K'
,'S59111P'
,'S59111S'
,'S59112D'
,'S59112G'
,'S59112K'
,'S59112P'
,'S59112S'
,'S59119D'
,'S59119G'
,'S59119K'
,'S59119P'
,'S59119S'
,'S59121D'
,'S59121G'
,'S59121K'
,'S59121P'
,'S59121S'
,'S59122D'
,'S59122G'
,'S59122K'
,'S59122P'
,'S59122S'
,'S59129D'
,'S59129G'
,'S59129K'
,'S59129P'
,'S59129S'
,'S59131D'
,'S59131G'
,'S59131K'
,'S59131P'
,'S59131S'
,'S59132D'
,'S59132G'
,'S59132K'
,'S59132P'
,'S59132S'
,'S59139D'
,'S59139G'
,'S59139K'
,'S59139P'
,'S59139S'
,'S59141D'
,'S59141G'
,'S59141K'
,'S59141P'
,'S59141S'
,'S59142D'
,'S59142G'
,'S59142K'
,'S59142P'
,'S59142S'
,'S59149D'
,'S59149G'
,'S59149K'
,'S59149P'
,'S59149S'
,'S59191D'
,'S59191G'
,'S59191K'
,'S59191P'
,'S59191S'
,'S59192D'
,'S59192G'
,'S59192K'
,'S59192P'
,'S59192S'
,'S59199D'
,'S59199G'
,'S59199K'
,'S59199P'
,'S59199S'
,'S59201D'
,'S59201G'
,'S59201K'
,'S59201P'
,'S59201S'
,'S59202D'
,'S59202G'
,'S59202K'
,'S59202P'
,'S59202S'
,'S59209D'
,'S59209G'
,'S59209K'
,'S59209P'
,'S59209S'
,'S59211D'
,'S59211G'
,'S59211K'
,'S59211P'
,'S59211S'
,'S59212D'
,'S59212G'
,'S59212K'
,'S59212P'
,'S59212S'
,'S59219D'
,'S59219G'
,'S59219K'
,'S59219P'
,'S59219S'
,'S59221D'
,'S59221G'
,'S59221K'
,'S59221P'
,'S59221S'
,'S59222D'
,'S59222G'
,'S59222K'
,'S59222P'
,'S59222S'
,'S59229D'
,'S59229G'
,'S59229K'
,'S59229P'
,'S59229S'
,'S59231D'
,'S59231G'
,'S59231K'
,'S59231P'
,'S59231S'
,'S59232D'
,'S59232G'
,'S59232K'
,'S59232P'
,'S59232S'
,'S59239D'
,'S59239G'
,'S59239K'
,'S59239P'
,'S59239S'
,'S59241D'
,'S59241G'
,'S59241K'
,'S59241P'
,'S59241S'
,'S59242D'
,'S59242G'
,'S59242K'
,'S59242P'
,'S59242S'
,'S59249D'
,'S59249G'
,'S59249K'
,'S59249P'
,'S59249S'
,'S59291D'
,'S59291G'
,'S59291K'
,'S59291P'
,'S59291S'
,'S59292D'
,'S59292G'
,'S59292K'
,'S59292P'
,'S59292S'
,'S59299D'
,'S59299G'
,'S59299K'
,'S59299P'
,'S59299S'
,'S59801D'
,'S59801S'
,'S59802D'
,'S59802S'
,'S59809D'
,'S59809S'
,'S59811D'
,'S59811S'
,'S59812D'
,'S59812S'
,'S59819D'
,'S59819S'
,'S59901D'
,'S59901S'
,'S59902D'
,'S59902S'
,'S59909D'
,'S59909S'
,'S59911D'
,'S59911S'
,'S59912D'
,'S59912S'
,'S59919D'
,'S59919S'
,'S6000XD'
,'S6000XS'
,'S60011D'
,'S60011S'
,'S60012D'
,'S60012S'
,'S60019D'
,'S60019S'
,'S60021D'
,'S60021S'
,'S60022D'
,'S60022S'
,'S60029D'
,'S60029S'
,'S60031D'
,'S60031S'
,'S60032D'
,'S60032S'
,'S60039D'
,'S60039S'
,'S60041D'
,'S60041S'
,'S60042D'
,'S60042S'
,'S60049D'
,'S60049S'
,'S60051D'
,'S60051S'
,'S60052D'
,'S60052S'
,'S60059D'
,'S60059S'
,'S6010XD'
,'S6010XS'
,'S60111D'
,'S60111S'
,'S60112D'
,'S60112S'
,'S60119D'
,'S60119S'
,'S60121D'
,'S60121S'
,'S60122D'
,'S60122S'
,'S60129D'
,'S60129S'
,'S60131D'
,'S60131S'
,'S60132D'
,'S60132S'
,'S60139D'
,'S60139S'
,'S60141D'
,'S60141S'
,'S60142D'
,'S60142S'
,'S60149D'
,'S60149S'
,'S60151D'
,'S60151S'
,'S60152D'
,'S60152S'
,'S60159D'
,'S60159S'
,'S60211D'
,'S60211S'
,'S60212D'
,'S60212S'
,'S60219D'
,'S60219S'
,'S60221D'
,'S60221S'
,'S60222D'
,'S60222S'
,'S60229D'
,'S60229S'
,'S60311D'
,'S60311S'
,'S60312D'
,'S60312S'
,'S60319D'
,'S60319S'
,'S60321D'
,'S60321S'
,'S60322D'
,'S60322S'
,'S60329D'
,'S60329S'
,'S60341D'
,'S60341S'
,'S60342D'
,'S60342S'
,'S60349D'
,'S60349S'
,'S60351D'
,'S60351S'
,'S60352D'
,'S60352S'
,'S60359D'
,'S60359S'
,'S60361D'
,'S60361S'
,'S60362D'
,'S60362S'
,'S60369D'
,'S60369S'
,'S60371D'
,'S60371S'
,'S60372D'
,'S60372S'
,'S60379D'
,'S60379S'
,'S60391D'
,'S60391S'
,'S60392D'
,'S60392S'
,'S60399D'
,'S60399S'
,'S60410D'
,'S60410S'
,'S60411D'
,'S60411S'
,'S60412D'
,'S60412S'
,'S60413D'
,'S60413S'
,'S60414D'
,'S60414S'
,'S60415D'
,'S60415S'
,'S60416D'
,'S60416S'
,'S60417D'
,'S60417S'
,'S60418D'
,'S60418S'
,'S60419D'
,'S60419S'
,'S60420D'
,'S60420S'
,'S60421D'
,'S60421S'
,'S60422D'
,'S60422S'
,'S60423D'
,'S60423S'
,'S60424D'
,'S60424S'
,'S60425D'
,'S60425S'
,'S60426D'
,'S60426S'
,'S60427D'
,'S60427S'
,'S60428D'
,'S60428S'
,'S60429D'
,'S60429S'
,'S60440D'
,'S60440S'
,'S60441D'
,'S60441S'
,'S60442D'
,'S60442S'
,'S60443D'
,'S60443S'
,'S60444D'
,'S60444S'
,'S60445D'
,'S60445S'
,'S60446D'
,'S60446S'
,'S60447D'
,'S60447S'
,'S60448D'
,'S60448S'
,'S60449D'
,'S60449S'
,'S60450D'
,'S60450S'
,'S60451D'
,'S60451S'
,'S60452D'
,'S60452S'
,'S60453D'
,'S60453S'
,'S60454D'
,'S60454S'
,'S60455D'
,'S60455S'
,'S60456D'
,'S60456S'
,'S60457D'
,'S60457S'
,'S60458D'
,'S60458S'
,'S60459D'
,'S60459S'
,'S60460D'
,'S60460S'
,'S60461D'
,'S60461S'
,'S60462D'
,'S60462S'
,'S60463D'
,'S60463S'
,'S60464D'
,'S60464S'
,'S60465D'
,'S60465S'
,'S60466D'
,'S60466S'
,'S60467D'
,'S60467S'
,'S60468D'
,'S60468S'
,'S60469D'
,'S60469S'
,'S60470D'
,'S60470S'
,'S60471D'
,'S60471S'
,'S60472D'
,'S60472S'
,'S60473D'
,'S60473S'
,'S60474D'
,'S60474S'
,'S60475D'
,'S60475S'
,'S60476D'
,'S60476S'
,'S60477D'
,'S60477S'
,'S60478D'
,'S60478S'
,'S60479D'
,'S60479S'
,'S60511D'
,'S60511S'
,'S60512D'
,'S60512S'
,'S60519D'
,'S60519S'
,'S60521D'
,'S60521S'
,'S60522D'
,'S60522S'
,'S60529D'
,'S60529S'
,'S60541D'
,'S60541S'
,'S60542D'
,'S60542S'
,'S60549D'
,'S60549S'
,'S60551D'
,'S60551S'
,'S60552D'
,'S60552S'
,'S60559D'
,'S60559S'
,'S60561D'
,'S60561S'
,'S60562D'
,'S60562S'
,'S60569D'
,'S60569S'
,'S60571D'
,'S60571S'
,'S60572D'
,'S60572S'
,'S60579D'
,'S60579S'
,'S60811D'
,'S60811S'
,'S60812D'
,'S60812S'
,'S60819D'
,'S60819S'
,'S60821D'
,'S60821S'
,'S60822D'
,'S60822S'
,'S60829D'
,'S60829S'
,'S60841D'
,'S60841S'
,'S60842D'
,'S60842S'
,'S60849D'
,'S60849S'
,'S60851D'
,'S60851S'
,'S60852D'
,'S60852S'
,'S60859D'
,'S60859S'
,'S60861D'
,'S60861S'
,'S60862D'
,'S60862S'
,'S60869D'
,'S60869S'
,'S60871D'
,'S60871S'
,'S60872D'
,'S60872S'
,'S60879D'
,'S60879S'
,'S60911D'
,'S60911S'
,'S60912D'
,'S60912S'
,'S60919D'
,'S60919S'
,'S60921D'
,'S60921S'
,'S60922D'
,'S60922S'
,'S60929D'
,'S60929S'
,'S60931D'
,'S60931S'
,'S60932D'
,'S60932S'
,'S60939D'
,'S60939S'
,'S60940D'
,'S60940S'
,'S60941D'
,'S60941S'
,'S60942D'
,'S60942S'
,'S60943D'
,'S60943S'
,'S60944D'
,'S60944S'
,'S60945D'
,'S60945S'
,'S60946D'
,'S60946S'
,'S60947D'
,'S60947S'
,'S60948D'
,'S60948S'
,'S60949D'
,'S60949S'
,'S61001D'
,'S61001S'
,'S61002D'
,'S61002S'
,'S61009D'
,'S61009S'
,'S61011D'
,'S61011S'
,'S61012D'
,'S61012S'
,'S61019D'
,'S61019S'
,'S61021D'
,'S61021S'
,'S61022D'
,'S61022S'
,'S61029D'
,'S61029S'
,'S61031D'
,'S61031S'
,'S61032D'
,'S61032S'
,'S61039D'
,'S61039S'
,'S61041D'
,'S61041S'
,'S61042D'
,'S61042S'
,'S61049D'
,'S61049S'
,'S61051D'
,'S61051S'
,'S61052D'
,'S61052S'
,'S61059D'
,'S61059S'
,'S61101D'
,'S61101S'
,'S61102D'
,'S61102S'
,'S61109D'
,'S61109S'
,'S61111D'
,'S61111S'
,'S61112D'
,'S61112S'
,'S61119D'
,'S61119S'
,'S61121D'
,'S61121S'
,'S61122D'
,'S61122S'
,'S61129D'
,'S61129S'
,'S61131D'
,'S61131S'
,'S61132D'
,'S61132S'
,'S61139D'
,'S61139S'
,'S61141D'
,'S61141S'
,'S61142D'
,'S61142S'
,'S61149D'
,'S61149S'
,'S61151D'
,'S61151S'
,'S61152D'
,'S61152S'
,'S61159D'
,'S61159S'
,'S61200D'
,'S61200S'
,'S61201D'
,'S61201S'
,'S61202D'
,'S61202S'
,'S61203D'
,'S61203S'
,'S61204D'
,'S61204S'
,'S61205D'
,'S61205S'
,'S61206D'
,'S61206S'
,'S61207D'
,'S61207S'
,'S61208D'
,'S61208S'
,'S61209D'
,'S61209S'
,'S61210D'
,'S61210S'
,'S61211D'
,'S61211S'
,'S61212D'
,'S61212S'
,'S61213D'
,'S61213S'
,'S61214D'
,'S61214S'
,'S61215D'
,'S61215S'
,'S61216D'
,'S61216S'
,'S61217D'
,'S61217S'
,'S61218D'
,'S61218S'
,'S61219D'
,'S61219S'
,'S61220D'
,'S61220S'
,'S61221D'
,'S61221S'
,'S61222D'
,'S61222S'
,'S61223D'
,'S61223S'
,'S61224D'
,'S61224S'
,'S61225D'
,'S61225S'
,'S61226D'
,'S61226S'
,'S61227D'
,'S61227S'
,'S61228D'
,'S61228S'
,'S61229D'
,'S61229S'
,'S61230D'
,'S61230S'
,'S61231D'
,'S61231S'
,'S61232D'
,'S61232S'
,'S61233D'
,'S61233S'
,'S61234D'
,'S61234S'
,'S61235D'
,'S61235S'
,'S61236D'
,'S61236S'
,'S61237D'
,'S61237S'
,'S61238D'
,'S61238S'
,'S61239D'
,'S61239S'
,'S61240D'
,'S61240S'
,'S61241D'
,'S61241S'
,'S61242D'
,'S61242S'
,'S61243D'
,'S61243S'
,'S61244D'
,'S61244S'
,'S61245D'
,'S61245S'
,'S61246D'
,'S61246S'
,'S61247D'
,'S61247S'
,'S61248D'
,'S61248S'
,'S61249D'
,'S61249S'
,'S61250D'
,'S61250S'
,'S61251D'
,'S61251S'
,'S61252D'
,'S61252S'
,'S61253D'
,'S61253S'
,'S61254D'
,'S61254S'
,'S61255D'
,'S61255S'
,'S61256D'
,'S61256S'
,'S61257D'
,'S61257S'
,'S61258D'
,'S61258S'
,'S61259D'
,'S61259S'
,'S61300D'
,'S61300S'
,'S61301D'
,'S61301S'
,'S61302D'
,'S61302S'
,'S61303D'
,'S61303S'
,'S61304D'
,'S61304S'
,'S61305D'
,'S61305S'
,'S61306D'
,'S61306S'
,'S61307D'
,'S61307S'
,'S61308D'
,'S61308S'
,'S61309D'
,'S61309S'
,'S61310D'
,'S61310S'
,'S61311D'
,'S61311S'
,'S61312D'
,'S61312S'
,'S61313D'
,'S61313S'
,'S61314D'
,'S61314S'
,'S61315D'
,'S61315S'
,'S61316D'
,'S61316S'
,'S61317D'
,'S61317S'
,'S61318D'
,'S61318S'
,'S61319D'
,'S61319S'
,'S61320D'
,'S61320S'
,'S61321D'
,'S61321S'
,'S61322D'
,'S61322S'
,'S61323D'
,'S61323S'
,'S61324D'
,'S61324S'
,'S61325D'
,'S61325S'
,'S61326D'
,'S61326S'
,'S61327D'
,'S61327S'
,'S61328D'
,'S61328S'
,'S61329D'
,'S61329S'
,'S61330D'
,'S61330S'
,'S61331D'
,'S61331S'
,'S61332D'
,'S61332S'
,'S61333D'
,'S61333S'
,'S61334D'
,'S61334S'
,'S61335D'
,'S61335S'
,'S61336D'
,'S61336S'
,'S61337D'
,'S61337S'
,'S61338D'
,'S61338S'
,'S61339D'
,'S61339S'
,'S61340D'
,'S61340S'
,'S61341D'
,'S61341S'
,'S61342D'
,'S61342S'
,'S61343D'
,'S61343S'
,'S61344D'
,'S61344S'
,'S61345D'
,'S61345S'
,'S61346D'
,'S61346S'
,'S61347D'
,'S61347S'
,'S61348D'
,'S61348S'
,'S61349D'
,'S61349S'
,'S61350D'
,'S61350S'
,'S61351D'
,'S61351S'
,'S61352D'
,'S61352S'
,'S61353D'
,'S61353S'
,'S61354D'
,'S61354S'
,'S61355D'
,'S61355S'
,'S61356D'
,'S61356S'
,'S61357D'
,'S61357S'
,'S61358D'
,'S61358S'
,'S61359D'
,'S61359S'
,'S61401D'
,'S61401S'
,'S61402D'
,'S61402S'
,'S61409D'
,'S61409S'
,'S61411D'
,'S61411S'
,'S61412D'
,'S61412S'
,'S61419D'
,'S61419S'
,'S61421D'
,'S61421S'
,'S61422D'
,'S61422S'
,'S61429D'
,'S61429S'
,'S61431D'
,'S61431S'
,'S61432D'
,'S61432S'
,'S61439D'
,'S61439S'
,'S61441D'
,'S61441S'
,'S61442D'
,'S61442S'
,'S61449D'
,'S61449S'
,'S61451D'
,'S61451S'
,'S61452D'
,'S61452S'
,'S61459D'
,'S61459S'
,'S61501D'
,'S61501S'
,'S61502D'
,'S61502S'
,'S61509D'
,'S61509S'
,'S61511D'
,'S61511S'
,'S61512D'
,'S61512S'
,'S61519D'
,'S61519S'
,'S61521D'
,'S61521S'
,'S61522D'
,'S61522S'
,'S61529D'
,'S61529S'
,'S61531D'
,'S61531S'
,'S61532D'
,'S61532S'
,'S61539D'
,'S61539S'
,'S61541D'
,'S61541S'
,'S61542D'
,'S61542S'
,'S61549D'
,'S61549S'
,'S61551D'
,'S61551S'
,'S61552D'
,'S61552S'
,'S61559D'
,'S61559S'
,'S62001D'
,'S62001G'
,'S62001K'
,'S62001P'
,'S62001S'
,'S62002D'
,'S62002G'
,'S62002K'
,'S62002P'
,'S62002S'
,'S62009D'
,'S62009G'
,'S62009K'
,'S62009P'
,'S62009S'
,'S62011D'
,'S62011G'
,'S62011K'
,'S62011P'
,'S62011S'
,'S62012D'
,'S62012G'
,'S62012K'
,'S62012P'
,'S62012S'
,'S62013D'
,'S62013G'
,'S62013K'
,'S62013P'
,'S62013S'
,'S62014D'
,'S62014G'
,'S62014K'
,'S62014P'
,'S62014S'
,'S62015D'
,'S62015G'
,'S62015K'
,'S62015P'
,'S62015S'
,'S62016D'
,'S62016G'
,'S62016K'
,'S62016P'
,'S62016S'
,'S62021D'
,'S62021G'
,'S62021K'
,'S62021P'
,'S62021S'
,'S62022D'
,'S62022G'
,'S62022K'
,'S62022P'
,'S62022S'
,'S62023D'
,'S62023G'
,'S62023K'
,'S62023P'
,'S62023S'
,'S62024D'
,'S62024G'
,'S62024K'
,'S62024P'
,'S62024S'
,'S62025D'
,'S62025G'
,'S62025K'
,'S62025P'
,'S62025S'
,'S62026D'
,'S62026G'
,'S62026K'
,'S62026P'
,'S62026S'
,'S62031D'
,'S62031G'
,'S62031K'
,'S62031P'
,'S62031S'
,'S62032D'
,'S62032G'
,'S62032K'
,'S62032P'
,'S62032S'
,'S62033D'
,'S62033G'
,'S62033K'
,'S62033P'
,'S62033S'
,'S62034D'
,'S62034G'
,'S62034K'
,'S62034P'
,'S62034S'
,'S62035D'
,'S62035G'
,'S62035K'
,'S62035P'
,'S62035S'
,'S62036D'
,'S62036G'
,'S62036K'
,'S62036P'
,'S62036S'
,'S62101D'
,'S62101G'
,'S62101K'
,'S62101P'
,'S62101S'
,'S62102D'
,'S62102G'
,'S62102K'
,'S62102P'
,'S62102S'
,'S62109D'
,'S62109G'
,'S62109K'
,'S62109P'
,'S62109S'
,'S62111D'
,'S62111G'
,'S62111K'
,'S62111P'
,'S62111S'
,'S62112D'
,'S62112G'
,'S62112K'
,'S62112P'
,'S62112S'
,'S62113D'
,'S62113G'
,'S62113K'
,'S62113P'
,'S62113S'
,'S62114D'
,'S62114G'
,'S62114K'
,'S62114P'
,'S62114S'
,'S62115D'
,'S62115G'
,'S62115K'
,'S62115P'
,'S62115S'
,'S62116D'
,'S62116G'
,'S62116K'
,'S62116P'
,'S62116S'
,'S62121D'
,'S62121G'
,'S62121K'
,'S62121P'
,'S62121S'
,'S62122D'
,'S62122G'
,'S62122K'
,'S62122P'
,'S62122S'
,'S62123D'
,'S62123G'
,'S62123K'
,'S62123P'
,'S62123S'
,'S62124D'
,'S62124G'
,'S62124K'
,'S62124P'
,'S62124S'
,'S62125D'
,'S62125G'
,'S62125K'
,'S62125P'
,'S62125S'
,'S62126D'
,'S62126G'
,'S62126K'
,'S62126P'
,'S62126S'
,'S62131D'
,'S62131G'
,'S62131K'
,'S62131P'
,'S62131S'
,'S62132D'
,'S62132G'
,'S62132K'
,'S62132P'
,'S62132S'
,'S62133D'
,'S62133G'
,'S62133K'
,'S62133P'
,'S62133S'
,'S62134D'
,'S62134G'
,'S62134K'
,'S62134P'
,'S62134S'
,'S62135D'
,'S62135G'
,'S62135K'
,'S62135P'
,'S62135S'
,'S62136D'
,'S62136G'
,'S62136K'
,'S62136P'
,'S62136S'
,'S62141D'
,'S62141G'
,'S62141K'
,'S62141P'
,'S62141S'
,'S62142D'
,'S62142G'
,'S62142K'
,'S62142P'
,'S62142S'
,'S62143D'
,'S62143G'
,'S62143K'
,'S62143P'
,'S62143S'
,'S62144D'
,'S62144G'
,'S62144K'
,'S62144P'
,'S62144S'
,'S62145D'
,'S62145G'
,'S62145K'
,'S62145P'
,'S62145S'
,'S62146D'
,'S62146G'
,'S62146K'
,'S62146P'
,'S62146S'
,'S62151D'
,'S62151G'
,'S62151K'
,'S62151P'
,'S62151S'
,'S62152D'
,'S62152G'
,'S62152K'
,'S62152P'
,'S62152S'
,'S62153D'
,'S62153G'
,'S62153K'
,'S62153P'
,'S62153S'
,'S62154D'
,'S62154G'
,'S62154K'
,'S62154P'
,'S62154S'
,'S62155D'
,'S62155G'
,'S62155K'
,'S62155P'
,'S62155S'
,'S62156D'
,'S62156G'
,'S62156K'
,'S62156P'
,'S62156S'
,'S62161D'
,'S62161G'
,'S62161K'
,'S62161P'
,'S62161S'
,'S62162D'
,'S62162G'
,'S62162K'
,'S62162P'
,'S62162S'
,'S62163D'
,'S62163G'
,'S62163K'
,'S62163P'
,'S62163S'
,'S62164D'
,'S62164G'
,'S62164K'
,'S62164P'
,'S62164S'
,'S62165D'
,'S62165G'
,'S62165K'
,'S62165P'
,'S62165S'
,'S62166D'
,'S62166G'
,'S62166K'
,'S62166P'
,'S62166S'
,'S62171D'
,'S62171G'
,'S62171K'
,'S62171P'
,'S62171S'
,'S62172D'
,'S62172G'
,'S62172K'
,'S62172P'
,'S62172S'
,'S62173D'
,'S62173G'
,'S62173K'
,'S62173P'
,'S62173S'
,'S62174D'
,'S62174G'
,'S62174K'
,'S62174P'
,'S62174S'
,'S62175D'
,'S62175G'
,'S62175K'
,'S62175P'
,'S62175S'
,'S62176D'
,'S62176G'
,'S62176K'
,'S62176P'
,'S62176S'
,'S62181D'
,'S62181G'
,'S62181K'
,'S62181P'
,'S62181S'
,'S62182D'
,'S62182G'
,'S62182K'
,'S62182P'
,'S62182S'
,'S62183D'
,'S62183G'
,'S62183K'
,'S62183P'
,'S62183S'
,'S62184D'
,'S62184G'
,'S62184K'
,'S62184P'
,'S62184S'
,'S62185D'
,'S62185G'
,'S62185K'
,'S62185P'
,'S62185S'
,'S62186D'
,'S62186G'
,'S62186K'
,'S62186P'
,'S62186S'
,'S62201D'
,'S62201G'
,'S62201K'
,'S62201P'
,'S62201S'
,'S62202D'
,'S62202G'
,'S62202K'
,'S62202P'
,'S62202S'
,'S62209D'
,'S62209G'
,'S62209K'
,'S62209P'
,'S62209S'
,'S62211D'
,'S62211G'
,'S62211K'
,'S62211P'
,'S62211S'
,'S62212D'
,'S62212G'
,'S62212K'
,'S62212P'
,'S62212S'
,'S62213D'
,'S62213G'
,'S62213K'
,'S62213P'
,'S62213S'
,'S62221D'
,'S62221G'
,'S62221K'
,'S62221P'
,'S62221S'
,'S62222D'
,'S62222G'
,'S62222K'
,'S62222P'
,'S62222S'
,'S62223D'
,'S62223G'
,'S62223K'
,'S62223P'
,'S62223S'
,'S62224D'
,'S62224G'
,'S62224K'
,'S62224P'
,'S62224S'
,'S62225D'
,'S62225G'
,'S62225K'
,'S62225P'
,'S62225S'
,'S62226D'
,'S62226G'
,'S62226K'
,'S62226P'
,'S62226S'
,'S62231D'
,'S62231G'
,'S62231K'
,'S62231P'
,'S62231S'
,'S62232D'
,'S62232G'
,'S62232K'
,'S62232P'
,'S62232S'
,'S62233D'
,'S62233G'
,'S62233K'
,'S62233P'
,'S62233S'
,'S62234D'
,'S62234G'
,'S62234K'
,'S62234P'
,'S62234S'
,'S62235D'
,'S62235G'
,'S62235K'
,'S62235P'
,'S62235S'
,'S62236D'
,'S62236G'
,'S62236K'
,'S62236P'
,'S62236S'
,'S62241D'
,'S62241G'
,'S62241K'
,'S62241P'
,'S62241S'
,'S62242D'
,'S62242G'
,'S62242K'
,'S62242P'
,'S62242S'
,'S62243D'
,'S62243G'
,'S62243K'
,'S62243P'
,'S62243S'
,'S62244D'
,'S62244G'
,'S62244K'
,'S62244P'
,'S62244S'
,'S62245D'
,'S62245G'
,'S62245K'
,'S62245P'
,'S62245S'
,'S62246D'
,'S62246G'
,'S62246K'
,'S62246P'
,'S62246S'
,'S62251D'
,'S62251G'
,'S62251K'
,'S62251P'
,'S62251S'
,'S62252D'
,'S62252G'
,'S62252K'
,'S62252P'
,'S62252S'
,'S62253D'
,'S62253G'
,'S62253K'
,'S62253P'
,'S62253S'
,'S62254D'
,'S62254G'
,'S62254K'
,'S62254P'
,'S62254S'
,'S62255D'
,'S62255G'
,'S62255K'
,'S62255P'
,'S62255S'
,'S62256D'
,'S62256G'
,'S62256K'
,'S62256P'
,'S62256S'
,'S62291D'
,'S62291G'
,'S62291K'
,'S62291P'
,'S62291S'
,'S62292D'
,'S62292G'
,'S62292K'
,'S62292P'
,'S62292S'
,'S62299D'
,'S62299G'
,'S62299K'
,'S62299P'
,'S62299S'
,'S62300D'
,'S62300G'
,'S62300K'
,'S62300P'
,'S62300S'
,'S62301D'
,'S62301G'
,'S62301K'
,'S62301P'
,'S62301S'
,'S62302D'
,'S62302G'
,'S62302K'
,'S62302P'
,'S62302S'
,'S62303D'
,'S62303G'
,'S62303K'
,'S62303P'
,'S62303S'
,'S62304D'
,'S62304G'
,'S62304K'
,'S62304P'
,'S62304S'
,'S62305D'
,'S62305G'
,'S62305K'
,'S62305P'
,'S62305S'
,'S62306D'
,'S62306G'
,'S62306K'
,'S62306P'
,'S62306S'
,'S62307D'
,'S62307G'
,'S62307K'
,'S62307P'
,'S62307S'
,'S62308D'
,'S62308G'
,'S62308K'
,'S62308P'
,'S62308S'
,'S62309D'
,'S62309G'
,'S62309K'
,'S62309P'
,'S62309S'
,'S62310D'
,'S62310G'
,'S62310K'
,'S62310P'
,'S62310S'
,'S62311D'
,'S62311G'
,'S62311K'
,'S62311P'
,'S62311S'
,'S62312D'
,'S62312G'
,'S62312K'
,'S62312P'
,'S62312S'
,'S62313D'
,'S62313G'
,'S62313K'
,'S62313P'
,'S62313S'
,'S62314D'
,'S62314G'
,'S62314K'
,'S62314P'
,'S62314S'
,'S62315D'
,'S62315G'
,'S62315K'
,'S62315P'
,'S62315S'
,'S62316D'
,'S62316G'
,'S62316K'
,'S62316P'
,'S62316S'
,'S62317D'
,'S62317G'
,'S62317K'
,'S62317P'
,'S62317S'
,'S62318D'
,'S62318G'
,'S62318K'
,'S62318P'
,'S62318S'
,'S62319D'
,'S62319G'
,'S62319K'
,'S62319P'
,'S62319S'
,'S62320D'
,'S62320G'
,'S62320K'
,'S62320P'
,'S62320S'
,'S62321D'
,'S62321G'
,'S62321K'
,'S62321P'
,'S62321S'
,'S62322D'
,'S62322G'
,'S62322K'
,'S62322P'
,'S62322S'
,'S62323D'
,'S62323G'
,'S62323K'
,'S62323P'
,'S62323S'
,'S62324D'
,'S62324G'
,'S62324K'
,'S62324P'
,'S62324S'
,'S62325D'
,'S62325G'
,'S62325K'
,'S62325P'
,'S62325S'
,'S62326D'
,'S62326G'
,'S62326K'
,'S62326P'
,'S62326S'
,'S62327D'
,'S62327G'
,'S62327K'
,'S62327P'
,'S62327S'
,'S62328D'
,'S62328G'
,'S62328K'
,'S62328P'
,'S62328S'
,'S62329D'
,'S62329G'
,'S62329K'
,'S62329P'
,'S62329S'
,'S62330D'
,'S62330G'
,'S62330K'
,'S62330P'
,'S62330S'
,'S62331D'
,'S62331G'
,'S62331K'
,'S62331P'
,'S62331S'
,'S62332D'
,'S62332G'
,'S62332K'
,'S62332P'
,'S62332S'
,'S62333D'
,'S62333G'
,'S62333K'
,'S62333P'
,'S62333S'
,'S62334D'
,'S62334G'
,'S62334K'
,'S62334P'
,'S62334S'
,'S62335D'
,'S62335G'
,'S62335K'
,'S62335P'
,'S62335S'
,'S62336D'
,'S62336G'
,'S62336K'
,'S62336P'
,'S62336S'
,'S62337D'
,'S62337G'
,'S62337K'
,'S62337P'
,'S62337S'
,'S62338D'
,'S62338G'
,'S62338K'
,'S62338P'
,'S62338S'
,'S62339D'
,'S62339G'
,'S62339K'
,'S62339P'
,'S62339S'
,'S62340D'
,'S62340G'
,'S62340K'
,'S62340P'
,'S62340S'
,'S62341D'
,'S62341G'
,'S62341K'
,'S62341P'
,'S62341S'
,'S62342D'
,'S62342G'
,'S62342K'
,'S62342P'
,'S62342S'
,'S62343D'
,'S62343G'
,'S62343K'
,'S62343P'
,'S62343S'
,'S62344D'
,'S62344G'
,'S62344K'
,'S62344P'
,'S62344S'
,'S62345D'
,'S62345G'
,'S62345K'
,'S62345P'
,'S62345S'
,'S62346D'
,'S62346G'
,'S62346K'
,'S62346P'
,'S62346S'
,'S62347D'
,'S62347G'
,'S62347K'
,'S62347P'
,'S62347S'
,'S62348D'
,'S62348G'
,'S62348K'
,'S62348P'
,'S62348S'
,'S62349D'
,'S62349G'
,'S62349K'
,'S62349P'
,'S62349S'
,'S62350D'
,'S62350G'
,'S62350K'
,'S62350P'
,'S62350S'
,'S62351D'
,'S62351G'
,'S62351K'
,'S62351P'
,'S62351S'
,'S62352D'
,'S62352G'
,'S62352K'
,'S62352P'
,'S62352S'
,'S62353D'
,'S62353G'
,'S62353K'
,'S62353P'
,'S62353S'
,'S62354D'
,'S62354G'
,'S62354K'
,'S62354P'
,'S62354S'
,'S62355D'
,'S62355G'
,'S62355K'
,'S62355P'
,'S62355S'
,'S62356D'
,'S62356G'
,'S62356K'
,'S62356P'
,'S62356S'
,'S62357D'
,'S62357G'
,'S62357K'
,'S62357P'
,'S62357S'
,'S62358D'
,'S62358G'
,'S62358K'
,'S62358P'
,'S62358S'
,'S62359D'
,'S62359G'
,'S62359K'
,'S62359P'
,'S62359S'
,'S62360D'
,'S62360G'
,'S62360K'
,'S62360P'
,'S62360S'
,'S62361D'
,'S62361G'
,'S62361K'
,'S62361P'
,'S62361S'
,'S62362D'
,'S62362G'
,'S62362K'
,'S62362P'
,'S62362S'
,'S62363D'
,'S62363G'
,'S62363K'
,'S62363P'
,'S62363S'
,'S62364D'
,'S62364G'
,'S62364K'
,'S62364P'
,'S62364S'
,'S62365D'
,'S62365G'
,'S62365K'
,'S62365P'
,'S62365S'
,'S62366D'
,'S62366G'
,'S62366K'
,'S62366P'
,'S62366S'
,'S62367D'
,'S62367G'
,'S62367K'
,'S62367P'
,'S62367S'
,'S62368D'
,'S62368G'
,'S62368K'
,'S62368P'
,'S62368S'
,'S62369D'
,'S62369G'
,'S62369K'
,'S62369P'
,'S62369S'
,'S62390D'
,'S62390G'
,'S62390K'
,'S62390P'
,'S62390S'
,'S62391D'
,'S62391G'
,'S62391K'
,'S62391P'
,'S62391S'
,'S62392D'
,'S62392G'
,'S62392K'
,'S62392P'
,'S62392S'
,'S62393D'
,'S62393G'
,'S62393K'
,'S62393P'
,'S62393S'
,'S62394D'
,'S62394G'
,'S62394K'
,'S62394P'
,'S62394S'
,'S62395D'
,'S62395G'
,'S62395K'
,'S62395P'
,'S62395S'
,'S62396D'
,'S62396G'
,'S62396K'
,'S62396P'
,'S62396S'
,'S62397D'
,'S62397G'
,'S62397K'
,'S62397P'
,'S62397S'
,'S62398D'
,'S62398G'
,'S62398K'
,'S62398P'
,'S62398S'
,'S62399D'
,'S62399G'
,'S62399K'
,'S62399P'
,'S62399S'
,'S62501D'
,'S62501G'
,'S62501K'
,'S62501P'
,'S62501S'
,'S62502D'
,'S62502G'
,'S62502K'
,'S62502P'
,'S62502S'
,'S62509D'
,'S62509G'
,'S62509K'
,'S62509P'
,'S62509S'
,'S62511D'
,'S62511G'
,'S62511K'
,'S62511P'
,'S62511S'
,'S62512D'
,'S62512G'
,'S62512K'
,'S62512P'
,'S62512S'
,'S62513D'
,'S62513G'
,'S62513K'
,'S62513P'
,'S62513S'
,'S62514D'
,'S62514G'
,'S62514K'
,'S62514P'
,'S62514S'
,'S62515D'
,'S62515G'
,'S62515K'
,'S62515P'
,'S62515S'
,'S62516D'
,'S62516G'
,'S62516K'
,'S62516P'
,'S62516S'
,'S62521D'
,'S62521G'
,'S62521K'
,'S62521P'
,'S62521S'
,'S62522D'
,'S62522G'
,'S62522K'
,'S62522P'
,'S62522S'
,'S62523D'
,'S62523G'
,'S62523K'
,'S62523P'
,'S62523S'
,'S62524D'
,'S62524G'
,'S62524K'
,'S62524P'
,'S62524S'
,'S62525D'
,'S62525G'
,'S62525K'
,'S62525P'
,'S62525S'
,'S62526D'
,'S62526G'
,'S62526K'
,'S62526P'
,'S62526S'
,'S62600D'
,'S62600G'
,'S62600K'
,'S62600P'
,'S62600S'
,'S62601D'
,'S62601G'
,'S62601K'
,'S62601P'
,'S62601S'
,'S62602D'
,'S62602G'
,'S62602K'
,'S62602P'
,'S62602S'
,'S62603D'
,'S62603G'
,'S62603K'
,'S62603P'
,'S62603S'
,'S62604D'
,'S62604G'
,'S62604K'
,'S62604P'
,'S62604S'
,'S62605D'
,'S62605G'
,'S62605K'
,'S62605P'
,'S62605S'
,'S62606D'
,'S62606G'
,'S62606K'
,'S62606P'
,'S62606S'
,'S62607D'
,'S62607G'
,'S62607K'
,'S62607P'
,'S62607S'
,'S62608D'
,'S62608G'
,'S62608K'
,'S62608P'
,'S62608S'
,'S62609D'
,'S62609G'
,'S62609K'
,'S62609P'
,'S62609S'
,'S62610D'
,'S62610G'
,'S62610K'
,'S62610P'
,'S62610S'
,'S62611D'
,'S62611G'
,'S62611K'
,'S62611P'
,'S62611S'
,'S62612D'
,'S62612G'
,'S62612K'
,'S62612P'
,'S62612S'
,'S62613D'
,'S62613G'
,'S62613K'
,'S62613P'
,'S62613S'
,'S62614D'
,'S62614G'
,'S62614K'
,'S62614P'
,'S62614S'
,'S62615D'
,'S62615G'
,'S62615K'
,'S62615P'
,'S62615S'
,'S62616D'
,'S62616G'
,'S62616K'
,'S62616P'
,'S62616S'
,'S62617D'
,'S62617G'
,'S62617K'
,'S62617P'
,'S62617S'
,'S62618D'
,'S62618G'
,'S62618K'
,'S62618P'
,'S62618S'
,'S62619D'
,'S62619G'
,'S62619K'
,'S62619P'
,'S62619S'
,'S62620D'
,'S62620G'
,'S62620K'
,'S62620P'
,'S62620S'
,'S62621D'
,'S62621G'
,'S62621K'
,'S62621P'
,'S62621S'
,'S62622D'
,'S62622G'
,'S62622K'
,'S62622P'
,'S62622S'
,'S62623D'
,'S62623G'
,'S62623K'
,'S62623P'
,'S62623S'
,'S62624D'
,'S62624G'
,'S62624K'
,'S62624P'
,'S62624S'
,'S62625D'
,'S62625G'
,'S62625K'
,'S62625P'
,'S62625S'
,'S62626D'
,'S62626G'
,'S62626K'
,'S62626P'
,'S62626S'
,'S62627D'
,'S62627G'
,'S62627K'
,'S62627P'
,'S62627S'
,'S62628D'
,'S62628G'
,'S62628K'
,'S62628P'
,'S62628S'
,'S62629D'
,'S62629G'
,'S62629K'
,'S62629P'
,'S62629S'
,'S62630D'
,'S62630G'
,'S62630K'
,'S62630P'
,'S62630S'
,'S62631D'
,'S62631G'
,'S62631K'
,'S62631P'
,'S62631S'
,'S62632D'
,'S62632G'
,'S62632K'
,'S62632P'
,'S62632S'
,'S62633D'
,'S62633G'
,'S62633K'
,'S62633P'
,'S62633S'
,'S62634D'
,'S62634G'
,'S62634K'
,'S62634P'
,'S62634S'
,'S62635D'
,'S62635G'
,'S62635K'
,'S62635P'
,'S62635S'
,'S62636D'
,'S62636G'
,'S62636K'
,'S62636P'
,'S62636S'
,'S62637D'
,'S62637G'
,'S62637K'
,'S62637P'
,'S62637S'
,'S62638D'
,'S62638G'
,'S62638K'
,'S62638P'
,'S62638S'
,'S62639D'
,'S62639G'
,'S62639K'
,'S62639P'
,'S62639S'
,'S62640D'
,'S62640G'
,'S62640K'
,'S62640P'
,'S62640S'
,'S62641D'
,'S62641G'
,'S62641K'
,'S62641P'
,'S62641S'
,'S62642D'
,'S62642G'
,'S62642K'
,'S62642P'
,'S62642S'
,'S62643D'
,'S62643G'
,'S62643K'
,'S62643P'
,'S62643S'
,'S62644D'
,'S62644G'
,'S62644K'
,'S62644P'
,'S62644S'
,'S62645D'
,'S62645G'
,'S62645K'
,'S62645P'
,'S62645S'
,'S62646D'
,'S62646G'
,'S62646K'
,'S62646P'
,'S62646S'
,'S62647D'
,'S62647G'
,'S62647K'
,'S62647P'
,'S62647S'
,'S62648D'
,'S62648G'
,'S62648K'
,'S62648P'
,'S62648S'
,'S62649D'
,'S62649G'
,'S62649K'
,'S62649P'
,'S62649S'
,'S62650D'
,'S62650G'
,'S62650K'
,'S62650P'
,'S62650S'
,'S62651D'
,'S62651G'
,'S62651K'
,'S62651P'
,'S62651S'
,'S62652D'
,'S62652G'
,'S62652K'
,'S62652P'
,'S62652S'
,'S62653D'
,'S62653G'
,'S62653K'
,'S62653P'
,'S62653S'
,'S62654D'
,'S62654G'
,'S62654K'
,'S62654P'
,'S62654S'
,'S62655D'
,'S62655G'
,'S62655K'
,'S62655P'
,'S62655S'
,'S62656D'
,'S62656G'
,'S62656K'
,'S62656P'
,'S62656S'
,'S62657D'
,'S62657G'
,'S62657K'
,'S62657P'
,'S62657S'
,'S62658D'
,'S62658G'
,'S62658K'
,'S62658P'
,'S62658S'
,'S62659D'
,'S62659G'
,'S62659K'
,'S62659P'
,'S62659S'
,'S62660D'
,'S62660G'
,'S62660K'
,'S62660P'
,'S62660S'
,'S62661D'
,'S62661G'
,'S62661K'
,'S62661P'
,'S62661S'
,'S62662D'
,'S62662G'
,'S62662K'
,'S62662P'
,'S62662S'
,'S62663D'
,'S62663G'
,'S62663K'
,'S62663P'
,'S62663S'
,'S62664D'
,'S62664G'
,'S62664K'
,'S62664P'
,'S62664S'
,'S62665D'
,'S62665G'
,'S62665K'
,'S62665P'
,'S62665S'
,'S62666D'
,'S62666G'
,'S62666K'
,'S62666P'
,'S62666S'
,'S62667D'
,'S62667G'
,'S62667K'
,'S62667P'
,'S62667S'
,'S62668D'
,'S62668G'
,'S62668K'
,'S62668P'
,'S62668S'
,'S62669D'
,'S62669G'
,'S62669K'
,'S62669P'
,'S62669S'
,'S6290XD'
,'S6290XG'
,'S6290XK'
,'S6290XP'
,'S6290XS'
,'S6291XD'
,'S6291XG'
,'S6291XK'
,'S6291XP'
,'S6291XS'
,'S6292XD'
,'S6292XG'
,'S6292XK'
,'S6292XP'
,'S6292XS'
,'S63001D'
,'S63001S'
,'S63002D'
,'S63002S'
,'S63003D'
,'S63003S'
,'S63004D'
,'S63004S'
,'S63005D'
,'S63005S'
,'S63006D'
,'S63006S'
,'S63011D'
,'S63011S'
,'S63012D'
,'S63012S'
,'S63013D'
,'S63013S'
,'S63014D'
,'S63014S'
,'S63015D'
,'S63015S'
,'S63016D'
,'S63016S'
,'S63021D'
,'S63021S'
,'S63022D'
,'S63022S'
,'S63023D'
,'S63023S'
,'S63024D'
,'S63024S'
,'S63025D'
,'S63025S'
,'S63026D'
,'S63026S'
,'S63031D'
,'S63031S'
,'S63032D'
,'S63032S'
,'S63033D'
,'S63033S'
,'S63034D'
,'S63034S'
,'S63035D'
,'S63035S'
,'S63036D'
,'S63036S'
,'S63041D'
,'S63041S'
,'S63042D'
,'S63042S'
,'S63043D'
,'S63043S'
,'S63044D'
,'S63044S'
,'S63045D'
,'S63045S'
,'S63046D'
,'S63046S'
,'S63051D'
,'S63051S'
,'S63052D'
,'S63052S'
,'S63053D'
,'S63053S'
,'S63054D'
,'S63054S'
,'S63055D'
,'S63055S'
,'S63056D'
,'S63056S'
,'S63061D'
,'S63061S'
,'S63062D'
,'S63062S'
,'S63063D'
,'S63063S'
,'S63064D'
,'S63064S'
,'S63065D'
,'S63065S'
,'S63066D'
,'S63066S'
,'S63071D'
,'S63071S'
,'S63072D'
,'S63072S'
,'S63073D'
,'S63073S'
,'S63074D'
,'S63074S'
,'S63075D'
,'S63075S'
,'S63076D'
,'S63076S'
,'S63091D'
,'S63091S'
,'S63092D'
,'S63092S'
,'S63093D'
,'S63093S'
,'S63094D'
,'S63094S'
,'S63095D'
,'S63095S'
,'S63096D'
,'S63096S'
,'S63101D'
,'S63101S'
,'S63102D'
,'S63102S'
,'S63103D'
,'S63103S'
,'S63104D'
,'S63104S'
,'S63105D'
,'S63105S'
,'S63106D'
,'S63106S'
,'S63111D'
,'S63111S'
,'S63112D'
,'S63112S'
,'S63113D'
,'S63113S'
,'S63114D'
,'S63114S'
,'S63115D'
,'S63115S'
,'S63116D'
,'S63116S'
,'S63121D'
,'S63121S'
,'S63122D'
,'S63122S'
,'S63123D'
,'S63123S'
,'S63124D'
,'S63124S'
,'S63125D'
,'S63125S'
,'S63126D'
,'S63126S'
,'S63131D'
,'S63131S'
,'S63132D'
,'S63132S'
,'S63133D'
,'S63133S'
,'S63134D'
,'S63134S'
,'S63135D'
,'S63135S'
,'S63136D'
,'S63136S'
,'S63141D'
,'S63141S'
,'S63142D'
,'S63142S'
,'S63143D'
,'S63143S'
,'S63144D'
,'S63144S'
,'S63145D'
,'S63145S'
,'S63146D'
,'S63146S'
,'S63200D'
,'S63200S'
,'S63201D'
,'S63201S'
,'S63202D'
,'S63202S'
,'S63203D'
,'S63203S'
,'S63204D'
,'S63204S'
,'S63205D'
,'S63205S'
,'S63206D'
,'S63206S'
,'S63207D'
,'S63207S'
,'S63208D'
,'S63208S'
,'S63209D'
,'S63209S'
,'S63210D'
,'S63210S'
,'S63211D'
,'S63211S'
,'S63212D'
,'S63212S'
,'S63213D'
,'S63213S'
,'S63214D'
,'S63214S'
,'S63215D'
,'S63215S'
,'S63216D'
,'S63216S'
,'S63217D'
,'S63217S'
,'S63218D'
,'S63218S'
,'S63219D'
,'S63219S'
,'S63220D'
,'S63220S'
,'S63221D'
,'S63221S'
,'S63222D'
,'S63222S'
,'S63223D'
,'S63223S'
,'S63224D'
,'S63224S'
,'S63225D'
,'S63225S'
,'S63226D'
,'S63226S'
,'S63227D'
,'S63227S'
,'S63228D'
,'S63228S'
,'S63229D'
,'S63229S'
,'S63230D'
,'S63230S'
,'S63231D'
,'S63231S'
,'S63232D'
,'S63232S'
,'S63233D'
,'S63233S'
,'S63234D'
,'S63234S'
,'S63235D'
,'S63235S'
,'S63236D'
,'S63236S'
,'S63237D'
,'S63237S'
,'S63238D'
,'S63238S'
,'S63239D'
,'S63239S'
,'S63240D'
,'S63240S'
,'S63241D'
,'S63241S'
,'S63242D'
,'S63242S'
,'S63243D'
,'S63243S'
,'S63244D'
,'S63244S'
,'S63245D'
,'S63245S'
,'S63246D'
,'S63246S'
,'S63247D'
,'S63247S'
,'S63248D'
,'S63248S'
,'S63249D'
,'S63249S'
,'S63250D'
,'S63250S'
,'S63251D'
,'S63251S'
,'S63252D'
,'S63252S'
,'S63253D'
,'S63253S'
,'S63254D'
,'S63254S'
,'S63255D'
,'S63255S'
,'S63256D'
,'S63256S'
,'S63257D'
,'S63257S'
,'S63258D'
,'S63258S'
,'S63259D'
,'S63259S'
,'S63260D'
,'S63260S'
,'S63261D'
,'S63261S'
,'S63262D'
,'S63262S'
,'S63263D'
,'S63263S'
,'S63264D'
,'S63264S'
,'S63265D'
,'S63265S'
,'S63266D'
,'S63266S'
,'S63267D'
,'S63267S'
,'S63268D'
,'S63268S'
,'S63269D'
,'S63269S'
,'S63270D'
,'S63270S'
,'S63271D'
,'S63271S'
,'S63272D'
,'S63272S'
,'S63273D'
,'S63273S'
,'S63274D'
,'S63274S'
,'S63275D'
,'S63275S'
,'S63276D'
,'S63276S'
,'S63277D'
,'S63277S'
,'S63278D'
,'S63278S'
,'S63279D'
,'S63279S'
,'S63280D'
,'S63280S'
,'S63281D'
,'S63281S'
,'S63282D'
,'S63282S'
,'S63283D'
,'S63283S'
,'S63284D'
,'S63284S'
,'S63285D'
,'S63285S'
,'S63286D'
,'S63286S'
,'S63287D'
,'S63287S'
,'S63288D'
,'S63288S'
,'S63289D'
,'S63289S'
,'S63290D'
,'S63290S'
,'S63291D'
,'S63291S'
,'S63292D'
,'S63292S'
,'S63293D'
,'S63293S'
,'S63294D'
,'S63294S'
,'S63295D'
,'S63295S'
,'S63296D'
,'S63296S'
,'S63297D'
,'S63297S'
,'S63298D'
,'S63298S'
,'S63299D'
,'S63299S'
,'S63301D'
,'S63301S'
,'S63302D'
,'S63302S'
,'S63309D'
,'S63309S'
,'S63311D'
,'S63311S'
,'S63312D'
,'S63312S'
,'S63319D'
,'S63319S'
,'S63321D'
,'S63321S'
,'S63322D'
,'S63322S'
,'S63329D'
,'S63329S'
,'S63331D'
,'S63331S'
,'S63332D'
,'S63332S'
,'S63339D'
,'S63339S'
,'S63391D'
,'S63391S'
,'S63392D'
,'S63392S'
,'S63399D'
,'S63399S'
,'S63400D'
,'S63400S'
,'S63401D'
,'S63401S'
,'S63402D'
,'S63402S'
,'S63403D'
,'S63403S'
,'S63404D'
,'S63404S'
,'S63405D'
,'S63405S'
,'S63406D'
,'S63406S'
,'S63407D'
,'S63407S'
,'S63408D'
,'S63408S'
,'S63409D'
,'S63409S'
,'S63410D'
,'S63410S'
,'S63411D'
,'S63411S'
,'S63412D'
,'S63412S'
,'S63413D'
,'S63413S'
,'S63414D'
,'S63414S'
,'S63415D'
,'S63415S'
,'S63416D'
,'S63416S'
,'S63417D'
,'S63417S'
,'S63418D'
,'S63418S'
,'S63419D'
,'S63419S'
,'S63420D'
,'S63420S'
,'S63421D'
,'S63421S'
,'S63422D'
,'S63422S'
,'S63423D'
,'S63423S'
,'S63424D'
,'S63424S'
,'S63425D'
,'S63425S'
,'S63426D'
,'S63426S'
,'S63427D'
,'S63427S'
,'S63428D'
,'S63428S'
,'S63429D'
,'S63429S'
,'S63430D'
,'S63430S'
,'S63431D'
,'S63431S'
,'S63432D'
,'S63432S'
,'S63433D'
,'S63433S'
,'S63434D'
,'S63434S'
,'S63435D'
,'S63435S'
,'S63436D'
,'S63436S'
,'S63437D'
,'S63437S'
,'S63438D'
,'S63438S'
,'S63439D'
,'S63439S'
,'S63490D'
,'S63490S'
,'S63491D'
,'S63491S'
,'S63492D'
,'S63492S'
,'S63493D'
,'S63493S'
,'S63494D'
,'S63494S'
,'S63495D'
,'S63495S'
,'S63496D'
,'S63496S'
,'S63497D'
,'S63497S'
,'S63498D'
,'S63498S'
,'S63499D'
,'S63499S'
,'S63501D'
,'S63501S'
,'S63502D'
,'S63502S'
,'S63509D'
,'S63509S'
,'S63511D'
,'S63511S'
,'S63512D'
,'S63512S'
,'S63519D'
,'S63519S'
,'S63521D'
,'S63521S'
,'S63522D'
,'S63522S'
,'S63529D'
,'S63529S'
,'S63591D'
,'S63591S'
,'S63592D'
,'S63592S'
,'S63599D'
,'S63599S'
,'S63601D'
,'S63601S'
,'S63602D'
,'S63602S'
,'S63609D'
,'S63609S'
,'S63610D'
,'S63610S'
,'S63611D'
,'S63611S'
,'S63612D'
,'S63612S'
,'S63613D'
,'S63613S'
,'S63614D'
,'S63614S'
,'S63615D'
,'S63615S'
,'S63616D'
,'S63616S'
,'S63617D'
,'S63617S'
,'S63618D'
,'S63618S'
,'S63619D'
,'S63619S'
,'S63621D'
,'S63621S'
,'S63622D'
,'S63622S'
,'S63629D'
,'S63629S'
,'S63630D'
,'S63630S'
,'S63631D'
,'S63631S'
,'S63632D'
,'S63632S'
,'S63633D'
,'S63633S'
,'S63634D'
,'S63634S'
,'S63635D'
,'S63635S'
,'S63636D'
,'S63636S'
,'S63637D'
,'S63637S'
,'S63638D'
,'S63638S'
,'S63639D'
,'S63639S'
,'S63641D'
,'S63641S'
,'S63642D'
,'S63642S'
,'S63649D'
,'S63649S'
,'S63650D'
,'S63650S'
,'S63651D'
,'S63651S'
,'S63652D'
,'S63652S'
,'S63653D'
,'S63653S'
,'S63654D'
,'S63654S'
,'S63655D'
,'S63655S'
,'S63656D'
,'S63656S'
,'S63657D'
,'S63657S'
,'S63658D'
,'S63658S'
,'S63659D'
,'S63659S'
,'S63681D'
,'S63681S'
,'S63682D'
,'S63682S'
,'S63689D'
,'S63689S'
,'S63690D'
,'S63690S'
,'S63691D'
,'S63691S'
,'S63692D'
,'S63692S'
,'S63693D'
,'S63693S'
,'S63694D'
,'S63694S'
,'S63695D'
,'S63695S'
,'S63696D'
,'S63696S'
,'S63697D'
,'S63697S'
,'S63698D'
,'S63698S'
,'S63699D'
,'S63699S'
,'S638X1D'
,'S638X1S'
,'S638X2D'
,'S638X2S'
,'S638X9D'
,'S638X9S'
,'S6390XD'
,'S6390XS'
,'S6391XD'
,'S6391XS'
,'S6392XD'
,'S6392XS'
,'S6400XD'
,'S6400XS'
,'S6401XD'
,'S6401XS'
,'S6402XD'
,'S6402XS'
,'S6410XD'
,'S6410XS'
,'S6411XD'
,'S6411XS'
,'S6412XD'
,'S6412XS'
,'S6420XD'
,'S6420XS'
,'S6421XD'
,'S6421XS'
,'S6422XD'
,'S6422XS'
,'S6430XD'
,'S6430XS'
,'S6431XD'
,'S6431XS'
,'S6432XD'
,'S6432XS'
,'S6440XD'
,'S6440XS'
,'S64490D'
,'S64490S'
,'S64491D'
,'S64491S'
,'S64492D'
,'S64492S'
,'S64493D'
,'S64493S'
,'S64494D'
,'S64494S'
,'S64495D'
,'S64495S'
,'S64496D'
,'S64496S'
,'S64497D'
,'S64497S'
,'S64498D'
,'S64498S'
,'S648X1D'
,'S648X1S'
,'S648X2D'
,'S648X2S'
,'S648X9D'
,'S648X9S'
,'S6490XD'
,'S6490XS'
,'S6491XD'
,'S6491XS'
,'S6492XD'
,'S6492XS'
,'S65001D'
,'S65001S'
,'S65002D'
,'S65002S'
,'S65009D'
,'S65009S'
,'S65011D'
,'S65011S'
,'S65012D'
,'S65012S'
,'S65019D'
,'S65019S'
,'S65091D'
,'S65091S'
,'S65092D'
,'S65092S'
,'S65099D'
,'S65099S'
,'S65101D'
,'S65101S'
,'S65102D'
,'S65102S'
,'S65109D'
,'S65109S'
,'S65111D'
,'S65111S'
,'S65112D'
,'S65112S'
,'S65119D'
,'S65119S'
,'S65191D'
,'S65191S'
,'S65192D'
,'S65192S'
,'S65199D'
,'S65199S'
,'S65201D'
,'S65201S'
,'S65202D'
,'S65202S'
,'S65209D'
,'S65209S'
,'S65211D'
,'S65211S'
,'S65212D'
,'S65212S'
,'S65219D'
,'S65219S'
,'S65291D'
,'S65291S'
,'S65292D'
,'S65292S'
,'S65299D'
,'S65299S'
,'S65301D'
,'S65301S'
,'S65302D'
,'S65302S'
,'S65309D'
,'S65309S'
,'S65311D'
,'S65311S'
,'S65312D'
,'S65312S'
,'S65319D'
,'S65319S'
,'S65391D'
,'S65391S'
,'S65392D'
,'S65392S'
,'S65399D'
,'S65399S'
,'S65401D'
,'S65401S'
,'S65402D'
,'S65402S'
,'S65409D'
,'S65409S'
,'S65411D'
,'S65411S'
,'S65412D'
,'S65412S'
,'S65419D'
,'S65419S'
,'S65491D'
,'S65491S'
,'S65492D'
,'S65492S'
,'S65499D'
,'S65499S'
,'S65500D'
,'S65500S'
,'S65501D'
,'S65501S'
,'S65502D'
,'S65502S'
,'S65503D'
,'S65503S'
,'S65504D'
,'S65504S'
,'S65505D'
,'S65505S'
,'S65506D'
,'S65506S'
,'S65507D'
,'S65507S'
,'S65508D'
,'S65508S'
,'S65509D'
,'S65509S'
,'S65510D'
,'S65510S'
,'S65511D'
,'S65511S'
,'S65512D'
,'S65512S'
,'S65513D'
,'S65513S'
,'S65514D'
,'S65514S'
,'S65515D'
,'S65515S'
,'S65516D'
,'S65516S'
,'S65517D'
,'S65517S'
,'S65518D'
,'S65518S'
,'S65519D'
,'S65519S'
,'S65590D'
,'S65590S'
,'S65591D'
,'S65591S'
,'S65592D'
,'S65592S'
,'S65593D'
,'S65593S'
,'S65594D'
,'S65594S'
,'S65595D'
,'S65595S'
,'S65596D'
,'S65596S'
,'S65597D'
,'S65597S'
,'S65598D'
,'S65598S'
,'S65599D'
,'S65599S'
,'S65801D'
,'S65801S'
,'S65802D'
,'S65802S'
,'S65809D'
,'S65809S'
,'S65811D'
,'S65811S'
,'S65812D'
,'S65812S'
,'S65819D'
,'S65819S'
,'S65891D'
,'S65891S'
,'S65892D'
,'S65892S'
,'S65899D'
,'S65899S'
,'S65901D'
,'S65901S'
,'S65902D'
,'S65902S'
,'S65909D'
,'S65909S'
,'S65911D'
,'S65911S'
,'S65912D'
,'S65912S'
,'S65919D'
,'S65919S'
,'S65991D'
,'S65991S'
,'S65992D'
,'S65992S'
,'S65999D'
,'S65999S'
,'S66001D'
,'S66001S'
,'S66002D'
,'S66002S'
,'S66009D'
,'S66009S'
,'S66011D'
,'S66011S'
,'S66012D'
,'S66012S'
,'S66019D'
,'S66019S'
,'S66021D'
,'S66021S'
,'S66022D'
,'S66022S'
,'S66029D'
,'S66029S'
,'S66091D'
,'S66091S'
,'S66092D'
,'S66092S'
,'S66099D'
,'S66099S'
,'S66100D'
,'S66100S'
,'S66101D'
,'S66101S'
,'S66102D'
,'S66102S'
,'S66103D'
,'S66103S'
,'S66104D'
,'S66104S'
,'S66105D'
,'S66105S'
,'S66106D'
,'S66106S'
,'S66107D'
,'S66107S'
,'S66108D'
,'S66108S'
,'S66109D'
,'S66109S'
,'S66110D'
,'S66110S'
,'S66111D'
,'S66111S'
,'S66112D'
,'S66112S'
,'S66113D'
,'S66113S'
,'S66114D'
,'S66114S'
,'S66115D'
,'S66115S'
,'S66116D'
,'S66116S'
,'S66117D'
,'S66117S'
,'S66118D'
,'S66118S'
,'S66119D'
,'S66119S'
,'S66120D'
,'S66120S'
,'S66121D'
,'S66121S'
,'S66122D'
,'S66122S'
,'S66123D'
,'S66123S'
,'S66124D'
,'S66124S'
,'S66125D'
,'S66125S'
,'S66126D'
,'S66126S'
,'S66127D'
,'S66127S'
,'S66128D'
,'S66128S'
,'S66129D'
,'S66129S'
,'S66190D'
,'S66190S'
,'S66191D'
,'S66191S'
,'S66192D'
,'S66192S'
,'S66193D'
,'S66193S'
,'S66194D'
,'S66194S'
,'S66195D'
,'S66195S'
,'S66196D'
,'S66196S'
,'S66197D'
,'S66197S'
,'S66198D'
,'S66198S'
,'S66199D'
,'S66199S'
,'S66201D'
,'S66201S'
,'S66202D'
,'S66202S'
,'S66209D'
,'S66209S'
,'S66211D'
,'S66211S'
,'S66212D'
,'S66212S'
,'S66219D'
,'S66219S'
,'S66221D'
,'S66221S'
,'S66222D'
,'S66222S'
,'S66229D'
,'S66229S'
,'S66291D'
,'S66291S'
,'S66292D'
,'S66292S'
,'S66299D'
,'S66299S'
,'S66300D'
,'S66300S'
,'S66301D'
,'S66301S'
,'S66302D'
,'S66302S'
,'S66303D'
,'S66303S'
,'S66304D'
,'S66304S'
,'S66305D'
,'S66305S'
,'S66306D'
,'S66306S'
,'S66307D'
,'S66307S'
,'S66308D'
,'S66308S'
,'S66309D'
,'S66309S'
,'S66310D'
,'S66310S'
,'S66311D'
,'S66311S'
,'S66312D'
,'S66312S'
,'S66313D'
,'S66313S'
,'S66314D'
,'S66314S'
,'S66315D'
,'S66315S'
,'S66316D'
,'S66316S'
,'S66317D'
,'S66317S'
,'S66318D'
,'S66318S'
,'S66319D'
,'S66319S'
,'S66320D'
,'S66320S'
,'S66321D'
,'S66321S'
,'S66322D'
,'S66322S'
,'S66323D'
,'S66323S'
,'S66324D'
,'S66324S'
,'S66325D'
,'S66325S'
,'S66326D'
,'S66326S'
,'S66327D'
,'S66327S'
,'S66328D'
,'S66328S'
,'S66329D'
,'S66329S'
,'S66390D'
,'S66390S'
,'S66391D'
,'S66391S'
,'S66392D'
,'S66392S'
,'S66393D'
,'S66393S'
,'S66394D'
,'S66394S'
,'S66395D'
,'S66395S'
,'S66396D'
,'S66396S'
,'S66397D'
,'S66397S'
,'S66398D'
,'S66398S'
,'S66399D'
,'S66399S'
,'S66401D'
,'S66401S'
,'S66402D'
,'S66402S'
,'S66409D'
,'S66409S'
,'S66411D'
,'S66411S'
,'S66412D'
,'S66412S'
,'S66419D'
,'S66419S'
,'S66421D'
,'S66421S'
,'S66422D'
,'S66422S'
,'S66429D'
,'S66429S'
,'S66491D'
,'S66491S'
,'S66492D'
,'S66492S'
,'S66499D'
,'S66499S'
,'S66500D'
,'S66500S'
,'S66501D'
,'S66501S'
,'S66502D'
,'S66502S'
,'S66503D'
,'S66503S'
,'S66504D'
,'S66504S'
,'S66505D'
,'S66505S'
,'S66506D'
,'S66506S'
,'S66507D'
,'S66507S'
,'S66508D'
,'S66508S'
,'S66509D'
,'S66509S'
,'S66510D'
,'S66510S'
,'S66511D'
,'S66511S'
,'S66512D'
,'S66512S'
,'S66513D'
,'S66513S'
,'S66514D'
,'S66514S'
,'S66515D'
,'S66515S'
,'S66516D'
,'S66516S'
,'S66517D'
,'S66517S'
,'S66518D'
,'S66518S'
,'S66519D'
,'S66519S'
,'S66520D'
,'S66520S'
,'S66521D'
,'S66521S'
,'S66522D'
,'S66522S'
,'S66523D'
,'S66523S'
,'S66524D'
,'S66524S'
,'S66525D'
,'S66525S'
,'S66526D'
,'S66526S'
,'S66527D'
,'S66527S'
,'S66528D'
,'S66528S'
,'S66529D'
,'S66529S'
,'S66590D'
,'S66590S'
,'S66591D'
,'S66591S'
,'S66592D'
,'S66592S'
,'S66593D'
,'S66593S'
,'S66594D'
,'S66594S'
,'S66595D'
,'S66595S'
,'S66596D'
,'S66596S'
,'S66597D'
,'S66597S'
,'S66598D'
,'S66598S'
,'S66599D'
,'S66599S'
,'S66801D'
,'S66801S'
,'S66802D'
,'S66802S'
,'S66809D'
,'S66809S'
,'S66811D'
,'S66811S'
,'S66812D'
,'S66812S'
,'S66819D'
,'S66819S'
,'S66821D'
,'S66821S'
,'S66822D'
,'S66822S'
,'S66829D'
,'S66829S'
,'S66891D'
,'S66891S'
,'S66892D'
,'S66892S'
,'S66899D'
,'S66899S'
,'S66901D'
,'S66901S'
,'S66902D'
,'S66902S'
,'S66909D'
,'S66909S'
,'S66911D'
,'S66911S'
,'S66912D'
,'S66912S'
,'S66919D'
,'S66919S'
,'S66921D'
,'S66921S'
,'S66922D'
,'S66922S'
,'S66929D'
,'S66929S'
,'S66991D'
,'S66991S'
,'S66992D'
,'S66992S'
,'S66999D'
,'S66999S'
,'S6700XD'
,'S6700XS'
,'S6701XD'
,'S6701XS'
,'S6702XD'
,'S6702XS'
,'S6710XD'
,'S6710XS'
,'S67190D'
,'S67190S'
,'S67191D'
,'S67191S'
,'S67192D'
,'S67192S'
,'S67193D'
,'S67193S'
,'S67194D'
,'S67194S'
,'S67195D'
,'S67195S'
,'S67196D'
,'S67196S'
,'S67197D'
,'S67197S'
,'S67198D'
,'S67198S'
,'S6720XD'
,'S6720XS'
,'S6721XD'
,'S6721XS'
,'S6722XD'
,'S6722XS'
,'S6730XD'
,'S6730XS'
,'S6731XD'
,'S6731XS'
,'S6732XD'
,'S6732XS'
,'S6740XD'
,'S6740XS'
,'S6741XD'
,'S6741XS'
,'S6742XD'
,'S6742XS'
,'S6790XD'
,'S6790XS'
,'S6791XD'
,'S6791XS'
,'S6792XD'
,'S6792XS'
,'S68011D'
,'S68011S'
,'S68012D'
,'S68012S'
,'S68019D'
,'S68019S'
,'S68021D'
,'S68021S'
,'S68022D'
,'S68022S'
,'S68029D'
,'S68029S'
,'S68110D'
,'S68110S'
,'S68111D'
,'S68111S'
,'S68112D'
,'S68112S'
,'S68113D'
,'S68113S'
,'S68114D'
,'S68114S'
,'S68115D'
,'S68115S'
,'S68116D'
,'S68116S'
,'S68117D'
,'S68117S'
,'S68118D'
,'S68118S'
,'S68119D'
,'S68119S'
,'S68120D'
,'S68120S'
,'S68121D'
,'S68121S'
,'S68122D'
,'S68122S'
,'S68123D'
,'S68123S'
,'S68124D'
,'S68124S'
,'S68125D'
,'S68125S'
,'S68126D'
,'S68126S'
,'S68127D'
,'S68127S'
,'S68128D'
,'S68128S'
,'S68129D'
,'S68129S'
,'S68411D'
,'S68411S'
,'S68412D'
,'S68412S'
,'S68419D'
,'S68419S'
,'S68421D'
,'S68421S'
,'S68422D'
,'S68422S'
,'S68429D'
,'S68429S'
,'S68511D'
,'S68511S'
,'S68512D'
,'S68512S'
,'S68519D'
,'S68519S'
,'S68521D'
,'S68521S'
,'S68522D'
,'S68522S'
,'S68529D'
,'S68529S'
,'S68610D'
,'S68610S'
,'S68611D'
,'S68611S'
,'S68612D'
,'S68612S'
,'S68613D'
,'S68613S'
,'S68614D'
,'S68614S'
,'S68615D'
,'S68615S'
,'S68616D'
,'S68616S'
,'S68617D'
,'S68617S'
,'S68618D'
,'S68618S'
,'S68619D'
,'S68619S'
,'S68620D'
,'S68620S'
,'S68621D'
,'S68621S'
,'S68622D'
,'S68622S'
,'S68623D'
,'S68623S'
,'S68624D'
,'S68624S'
,'S68625D'
,'S68625S'
,'S68626D'
,'S68626S'
,'S68627D'
,'S68627S'
,'S68628D'
,'S68628S'
,'S68629D'
,'S68629S'
,'S68711D'
,'S68711S'
,'S68712D'
,'S68712S'
,'S68719D'
,'S68719S'
,'S68721D'
,'S68721S'
,'S68722D'
,'S68722S'
,'S68729D'
,'S68729S'
,'S6980XD'
,'S6980XS'
,'S6981XD'
,'S6981XS'
,'S6982XD'
,'S6982XS'
,'S6990XD'
,'S6990XS'
,'S6991XD'
,'S6991XS'
,'S6992XD'
,'S6992XS'
,'S7000XD'
,'S7000XS'
,'S7001XD'
,'S7001XS'
,'S7002XD'
,'S7002XS'
,'S7010XD'
,'S7010XS'
,'S7011XD'
,'S7011XS'
,'S7012XD'
,'S7012XS'
,'S70211D'
,'S70211S'
,'S70212D'
,'S70212S'
,'S70219D'
,'S70219S'
,'S70221D'
,'S70221S'
,'S70222D'
,'S70222S'
,'S70229D'
,'S70229S'
,'S70241D'
,'S70241S'
,'S70242D'
,'S70242S'
,'S70249D'
,'S70249S'
,'S70251D'
,'S70251S'
,'S70252D'
,'S70252S'
,'S70259D'
,'S70259S'
,'S70261D'
,'S70261S'
,'S70262D'
,'S70262S'
,'S70269D'
,'S70269S'
,'S70271D'
,'S70271S'
,'S70272D'
,'S70272S'
,'S70279D'
,'S70279S'
,'S70311D'
,'S70311S'
,'S70312D'
,'S70312S'
,'S70319D'
,'S70319S'
,'S70321D'
,'S70321S'
,'S70322D'
,'S70322S'
,'S70329D'
,'S70329S'
,'S70341D'
,'S70341S'
,'S70342D'
,'S70342S'
,'S70349D'
,'S70349S'
,'S70351D'
,'S70351S'
,'S70352D'
,'S70352S'
,'S70359D'
,'S70359S'
,'S70361D'
,'S70361S'
,'S70362D'
,'S70362S'
,'S70369D'
,'S70369S'
,'S70371D'
,'S70371S'
,'S70372D'
,'S70372S'
,'S70379D'
,'S70379S'
,'S70911D'
,'S70911S'
,'S70912D'
,'S70912S'
,'S70919D'
,'S70919S'
,'S70921D'
,'S70921S'
,'S70922D'
,'S70922S'
,'S70929D'
,'S70929S'
,'S71001D'
,'S71001S'
,'S71002D'
,'S71002S'
,'S71009D'
,'S71009S'
,'S71011D'
,'S71011S'
,'S71012D'
,'S71012S'
,'S71019D'
,'S71019S'
,'S71021D'
,'S71021S'
,'S71022D'
,'S71022S'
,'S71029D'
,'S71029S'
,'S71031D'
,'S71031S'
,'S71032D'
,'S71032S'
,'S71039D'
,'S71039S'
,'S71041D'
,'S71041S'
,'S71042D'
,'S71042S'
,'S71049D'
,'S71049S'
,'S71051D'
,'S71051S'
,'S71052D'
,'S71052S'
,'S71059D'
,'S71059S'
,'S71101D'
,'S71101S'
,'S71102D'
,'S71102S'
,'S71109D'
,'S71109S'
,'S71111D'
,'S71111S'
,'S71112D'
,'S71112S'
,'S71119D'
,'S71119S'
,'S71121D'
,'S71121S'
,'S71122D'
,'S71122S'
,'S71129D'
,'S71129S'
,'S71131D'
,'S71131S'
,'S71132D'
,'S71132S'
,'S71139D'
,'S71139S'
,'S71141D'
,'S71141S'
,'S71142D'
,'S71142S'
,'S71149D'
,'S71149S'
,'S71151D'
,'S71151S'
,'S71152D'
,'S71152S'
,'S71159D'
,'S71159S'
,'S72001C'
,'S72001D'
,'S72001E'
,'S72001F'
,'S72001G'
,'S72001H'
,'S72001J'
,'S72001K'
,'S72001M'
,'S72001N'
,'S72001P'
,'S72001Q'
,'S72001R'
,'S72001S'
,'S72002C'
,'S72002D'
,'S72002E'
,'S72002F'
,'S72002G'
,'S72002H'
,'S72002J'
,'S72002K'
,'S72002M'
,'S72002N'
,'S72002P'
,'S72002Q'
,'S72002R'
,'S72002S'
,'S72009C'
,'S72009D'
,'S72009E'
,'S72009F'
,'S72009G'
,'S72009H'
,'S72009J'
,'S72009K'
,'S72009M'
,'S72009N'
,'S72009P'
,'S72009Q'
,'S72009R'
,'S72009S'
,'S72011C'
,'S72011D'
,'S72011E'
,'S72011F'
,'S72011G'
,'S72011H'
,'S72011J'
,'S72011K'
,'S72011M'
,'S72011N'
,'S72011P'
,'S72011Q'
,'S72011R'
,'S72011S'
,'S72012C'
,'S72012D'
,'S72012E'
,'S72012F'
,'S72012G'
,'S72012H'
,'S72012J'
,'S72012K'
,'S72012M'
,'S72012N'
,'S72012P'
,'S72012Q'
,'S72012R'
,'S72012S'
,'S72019C'
,'S72019D'
,'S72019E'
,'S72019F'
,'S72019G'
,'S72019H'
,'S72019J'
,'S72019K'
,'S72019M'
,'S72019N'
,'S72019P'
,'S72019Q'
,'S72019R'
,'S72019S'
,'S72021C'
,'S72021D'
,'S72021E'
,'S72021F'
,'S72021G'
,'S72021H'
,'S72021J'
,'S72021K'
,'S72021M'
,'S72021N'
,'S72021P'
,'S72021Q'
,'S72021R'
,'S72021S'
,'S72022C'
,'S72022D'
,'S72022E'
,'S72022F'
,'S72022G'
,'S72022H'
,'S72022J'
,'S72022K'
,'S72022M'
,'S72022N'
,'S72022P'
,'S72022Q'
,'S72022R'
,'S72022S'
,'S72023C'
,'S72023D'
,'S72023E'
,'S72023F'
,'S72023G'
,'S72023H'
,'S72023J'
,'S72023K'
,'S72023M'
,'S72023N'
,'S72023P'
,'S72023Q'
,'S72023R'
,'S72023S'
,'S72024C'
,'S72024D'
,'S72024E'
,'S72024F'
,'S72024G'
,'S72024H'
,'S72024J'
,'S72024K'
,'S72024M'
,'S72024N'
,'S72024P'
,'S72024Q'
,'S72024R'
,'S72024S'
,'S72025C'
,'S72025D'
,'S72025E'
,'S72025F'
,'S72025G'
,'S72025H'
,'S72025J'
,'S72025K'
,'S72025M'
,'S72025N'
,'S72025P'
,'S72025Q'
,'S72025R'
,'S72025S'
,'S72026C'
,'S72026D'
,'S72026E'
,'S72026F'
,'S72026G'
,'S72026H'
,'S72026J'
,'S72026K'
,'S72026M'
,'S72026N'
,'S72026P'
,'S72026Q'
,'S72026R'
,'S72026S'
,'S72031C'
,'S72031D'
,'S72031E'
,'S72031F'
,'S72031G'
,'S72031H'
,'S72031J'
,'S72031K'
,'S72031M'
,'S72031N'
,'S72031P'
,'S72031Q'
,'S72031R'
,'S72031S'
,'S72032C'
,'S72032D'
,'S72032E'
,'S72032F'
,'S72032G'
,'S72032H'
,'S72032J'
,'S72032K'
,'S72032M'
,'S72032N'
,'S72032P'
,'S72032Q'
,'S72032R'
,'S72032S'
,'S72033C'
,'S72033D'
,'S72033E'
,'S72033F'
,'S72033G'
,'S72033H'
,'S72033J'
,'S72033K'
,'S72033M'
,'S72033N'
,'S72033P'
,'S72033Q'
,'S72033R'
,'S72033S'
,'S72034C'
,'S72034D'
,'S72034E'
,'S72034F'
,'S72034G'
,'S72034H'
,'S72034J'
,'S72034K'
,'S72034M'
,'S72034N'
,'S72034P'
,'S72034Q'
,'S72034R'
,'S72034S'
,'S72035C'
,'S72035D'
,'S72035E'
,'S72035F'
,'S72035G'
,'S72035H'
,'S72035J'
,'S72035K'
,'S72035M'
,'S72035N'
,'S72035P'
,'S72035Q'
,'S72035R'
,'S72035S'
,'S72036C'
,'S72036D'
,'S72036E'
,'S72036F'
,'S72036G'
,'S72036H'
,'S72036J'
,'S72036K'
,'S72036M'
,'S72036N'
,'S72036P'
,'S72036Q'
,'S72036R'
,'S72036S'
,'S72041C'
,'S72041D'
,'S72041E'
,'S72041F'
,'S72041G'
,'S72041H'
,'S72041J'
,'S72041K'
,'S72041M'
,'S72041N'
,'S72041P'
,'S72041Q'
,'S72041R'
,'S72041S'
,'S72042C'
,'S72042D'
,'S72042E'
,'S72042F'
,'S72042G'
,'S72042H'
,'S72042J'
,'S72042K'
,'S72042M'
,'S72042N'
,'S72042P'
,'S72042Q'
,'S72042R'
,'S72042S'
,'S72043C'
,'S72043D'
,'S72043E'
,'S72043F'
,'S72043G'
,'S72043H'
,'S72043J'
,'S72043K'
,'S72043M'
,'S72043N'
,'S72043P'
,'S72043Q'
,'S72043R'
,'S72043S'
,'S72044C'
,'S72044D'
,'S72044E'
,'S72044F'
,'S72044G'
,'S72044H'
,'S72044J'
,'S72044K'
,'S72044M'
,'S72044N'
,'S72044P'
,'S72044Q'
,'S72044R'
,'S72044S'
,'S72045C'
,'S72045D'
,'S72045E'
,'S72045F'
,'S72045G'
,'S72045H'
,'S72045J'
,'S72045K'
,'S72045M'
,'S72045N'
,'S72045P'
,'S72045Q'
,'S72045R'
,'S72045S'
,'S72046C'
,'S72046D'
,'S72046E'
,'S72046F'
,'S72046G'
,'S72046H'
,'S72046J'
,'S72046K'
,'S72046M'
,'S72046N'
,'S72046P'
,'S72046Q'
,'S72046R'
,'S72046S'
,'S72051C'
,'S72051D'
,'S72051E'
,'S72051F'
,'S72051G'
,'S72051H'
,'S72051J'
,'S72051K'
,'S72051M'
,'S72051N'
,'S72051P'
,'S72051Q'
,'S72051R'
,'S72051S'
,'S72052C'
,'S72052D'
,'S72052E'
,'S72052F'
,'S72052G'
,'S72052H'
,'S72052J'
,'S72052K'
,'S72052M'
,'S72052N'
,'S72052P'
,'S72052Q'
,'S72052R'
,'S72052S'
,'S72059C'
,'S72059D'
,'S72059E'
,'S72059F'
,'S72059G'
,'S72059H'
,'S72059J'
,'S72059K'
,'S72059M'
,'S72059N'
,'S72059P'
,'S72059Q'
,'S72059R'
,'S72059S'
,'S72061C'
,'S72061D'
,'S72061E'
,'S72061F'
,'S72061G'
,'S72061H'
,'S72061J'
,'S72061K'
,'S72061M'
,'S72061N'
,'S72061P'
,'S72061Q'
,'S72061R'
,'S72061S'
,'S72062C'
,'S72062D'
,'S72062E'
,'S72062F'
,'S72062G'
,'S72062H'
,'S72062J'
,'S72062K'
,'S72062M'
,'S72062N'
,'S72062P'
,'S72062Q'
,'S72062R'
,'S72062S'
,'S72063C'
,'S72063D'
,'S72063E'
,'S72063F'
,'S72063G'
,'S72063H'
,'S72063J'
,'S72063K'
,'S72063M'
,'S72063N'
,'S72063P'
,'S72063Q'
,'S72063R'
,'S72063S'
,'S72064C'
,'S72064D'
,'S72064E'
,'S72064F'
,'S72064G'
,'S72064H'
,'S72064J'
,'S72064K'
,'S72064M'
,'S72064N'
,'S72064P'
,'S72064Q'
,'S72064R'
,'S72064S'
,'S72065C'
,'S72065D'
,'S72065E'
,'S72065F'
,'S72065G'
,'S72065H'
,'S72065J'
,'S72065K'
,'S72065M'
,'S72065N'
,'S72065P'
,'S72065Q'
,'S72065R'
,'S72065S'
,'S72066C'
,'S72066D'
,'S72066E'
,'S72066F'
,'S72066G'
,'S72066H'
,'S72066J'
,'S72066K'
,'S72066M'
,'S72066N'
,'S72066P'
,'S72066Q'
,'S72066R'
,'S72066S'
,'S72091C'
,'S72091D'
,'S72091E'
,'S72091F'
,'S72091G'
,'S72091H'
,'S72091J'
,'S72091K'
,'S72091M'
,'S72091N'
,'S72091P'
,'S72091Q'
,'S72091R'
,'S72091S'
,'S72092C'
,'S72092D'
,'S72092E'
,'S72092F'
,'S72092G'
,'S72092H'
,'S72092J'
,'S72092K'
,'S72092M'
,'S72092N'
,'S72092P'
,'S72092Q'
,'S72092R'
,'S72092S'
,'S72099C'
,'S72099D'
,'S72099E'
,'S72099F'
,'S72099G'
,'S72099H'
,'S72099J'
,'S72099K'
,'S72099M'
,'S72099N'
,'S72099P'
,'S72099Q'
,'S72099R'
,'S72099S'
,'S72101C'
,'S72101D'
,'S72101E'
,'S72101F'
,'S72101G'
,'S72101H'
,'S72101J'
,'S72101K'
,'S72101M'
,'S72101N'
,'S72101P'
,'S72101Q'
,'S72101R'
,'S72101S'
,'S72102C'
,'S72102D'
,'S72102E'
,'S72102F'
,'S72102G'
,'S72102H'
,'S72102J'
,'S72102K'
,'S72102M'
,'S72102N'
,'S72102P'
,'S72102Q'
,'S72102R'
,'S72102S'
,'S72109C'
,'S72109D'
,'S72109E'
,'S72109F'
,'S72109G'
,'S72109H'
,'S72109J'
,'S72109K'
,'S72109M'
,'S72109N'
,'S72109P'
,'S72109Q'
,'S72109R'
,'S72109S'
,'S72111C'
,'S72111D'
,'S72111E'
,'S72111F'
,'S72111G'
,'S72111H'
,'S72111J'
,'S72111K'
,'S72111M'
,'S72111N'
,'S72111P'
,'S72111Q'
,'S72111R'
,'S72111S'
,'S72112C'
,'S72112D'
,'S72112E'
,'S72112F'
,'S72112G'
,'S72112H'
,'S72112J'
,'S72112K'
,'S72112M'
,'S72112N'
,'S72112P'
,'S72112Q'
,'S72112R'
,'S72112S'
,'S72113C'
,'S72113D'
,'S72113E'
,'S72113F'
,'S72113G'
,'S72113H'
,'S72113J'
,'S72113K'
,'S72113M'
,'S72113N'
,'S72113P'
,'S72113Q'
,'S72113R'
,'S72113S'
,'S72114C'
,'S72114D'
,'S72114E'
,'S72114F'
,'S72114G'
,'S72114H'
,'S72114J'
,'S72114K'
,'S72114M'
,'S72114N'
,'S72114P'
,'S72114Q'
,'S72114R'
,'S72114S'
,'S72115C'
,'S72115D'
,'S72115E'
,'S72115F'
,'S72115G'
,'S72115H'
,'S72115J'
,'S72115K'
,'S72115M'
,'S72115N'
,'S72115P'
,'S72115Q'
,'S72115R'
,'S72115S'
,'S72116C'
,'S72116D'
,'S72116E'
,'S72116F'
,'S72116G'
,'S72116H'
,'S72116J'
,'S72116K'
,'S72116M'
,'S72116N'
,'S72116P'
,'S72116Q'
,'S72116R'
,'S72116S'
,'S72121C'
,'S72121D'
,'S72121E'
,'S72121F'
,'S72121G'
,'S72121H'
,'S72121J'
,'S72121K'
,'S72121M'
,'S72121N'
,'S72121P'
,'S72121Q'
,'S72121R'
,'S72121S'
,'S72122C'
,'S72122D'
,'S72122E'
,'S72122F'
,'S72122G'
,'S72122H'
,'S72122J'
,'S72122K'
,'S72122M'
,'S72122N'
,'S72122P'
,'S72122Q'
,'S72122R'
,'S72122S'
,'S72123C'
,'S72123D'
,'S72123E'
,'S72123F'
,'S72123G'
,'S72123H'
,'S72123J'
,'S72123K'
,'S72123M'
,'S72123N'
,'S72123P'
,'S72123Q'
,'S72123R'
,'S72123S'
,'S72124C'
,'S72124D'
,'S72124E'
,'S72124F'
,'S72124G'
,'S72124H'
,'S72124J'
,'S72124K'
,'S72124M'
,'S72124N'
,'S72124P'
,'S72124Q'
,'S72124R'
,'S72124S'
,'S72125C'
,'S72125D'
,'S72125E'
,'S72125F'
,'S72125G'
,'S72125H'
,'S72125J'
,'S72125K'
,'S72125M'
,'S72125N'
,'S72125P'
,'S72125Q'
,'S72125R'
,'S72125S'
,'S72126C'
,'S72126D'
,'S72126E'
,'S72126F'
,'S72126G'
,'S72126H'
,'S72126J'
,'S72126K'
,'S72126M'
,'S72126N'
,'S72126P'
,'S72126Q'
,'S72126R'
,'S72126S'
,'S72131C'
,'S72131D'
,'S72131E'
,'S72131F'
,'S72131G'
,'S72131H'
,'S72131J'
,'S72131K'
,'S72131M'
,'S72131N'
,'S72131P'
,'S72131Q'
,'S72131R'
,'S72131S'
,'S72132C'
,'S72132D'
,'S72132E'
,'S72132F'
,'S72132G'
,'S72132H'
,'S72132J'
,'S72132K'
,'S72132M'
,'S72132N'
,'S72132P'
,'S72132Q'
,'S72132R'
,'S72132S'
,'S72133C'
,'S72133D'
,'S72133E'
,'S72133F'
,'S72133G'
,'S72133H'
,'S72133J'
,'S72133K'
,'S72133M'
,'S72133N'
,'S72133P'
,'S72133Q'
,'S72133R'
,'S72133S'
,'S72134C'
,'S72134D'
,'S72134E'
,'S72134F'
,'S72134G'
,'S72134H'
,'S72134J'
,'S72134K'
,'S72134M'
,'S72134N'
,'S72134P'
,'S72134Q'
,'S72134R'
,'S72134S'
,'S72135C'
,'S72135D'
,'S72135E'
,'S72135F'
,'S72135G'
,'S72135H'
,'S72135J'
,'S72135K'
,'S72135M'
,'S72135N'
,'S72135P'
,'S72135Q'
,'S72135R'
,'S72135S'
,'S72136C'
,'S72136D'
,'S72136E'
,'S72136F'
,'S72136G'
,'S72136H'
,'S72136J'
,'S72136K'
,'S72136M'
,'S72136N'
,'S72136P'
,'S72136Q'
,'S72136R'
,'S72136S'
,'S72141C'
,'S72141D'
,'S72141E'
,'S72141F'
,'S72141G'
,'S72141H'
,'S72141J'
,'S72141K'
,'S72141M'
,'S72141N'
,'S72141P'
,'S72141Q'
,'S72141R'
,'S72141S'
,'S72142C'
,'S72142D'
,'S72142E'
,'S72142F'
,'S72142G'
,'S72142H'
,'S72142J'
,'S72142K'
,'S72142M'
,'S72142N'
,'S72142P'
,'S72142Q'
,'S72142R'
,'S72142S'
,'S72143C'
,'S72143D'
,'S72143E'
,'S72143F'
,'S72143G'
,'S72143H'
,'S72143J'
,'S72143K'
,'S72143M'
,'S72143N'
,'S72143P'
,'S72143Q'
,'S72143R'
,'S72143S'
,'S72144C'
,'S72144D'
,'S72144E'
,'S72144F'
,'S72144G'
,'S72144H'
,'S72144J'
,'S72144K'
,'S72144M'
,'S72144N'
,'S72144P'
,'S72144Q'
,'S72144R'
,'S72144S'
,'S72145C'
,'S72145D'
,'S72145E'
,'S72145F'
,'S72145G'
,'S72145H'
,'S72145J'
,'S72145K'
,'S72145M'
,'S72145N'
,'S72145P'
,'S72145Q'
,'S72145R'
,'S72145S'
,'S72146C'
,'S72146D'
,'S72146E'
,'S72146F'
,'S72146G'
,'S72146H'
,'S72146J'
,'S72146K'
,'S72146M'
,'S72146N'
,'S72146P'
,'S72146Q'
,'S72146R'
,'S72146S'
,'S7221XC'
,'S7221XD'
,'S7221XE'
,'S7221XF'
,'S7221XG'
,'S7221XH'
,'S7221XJ'
,'S7221XK'
,'S7221XM'
,'S7221XN'
,'S7221XP'
,'S7221XQ'
,'S7221XR'
,'S7221XS'
,'S7222XC'
,'S7222XD'
,'S7222XE'
,'S7222XF'
,'S7222XG'
,'S7222XH'
,'S7222XJ'
,'S7222XK'
,'S7222XM'
,'S7222XN'
,'S7222XP'
,'S7222XQ'
,'S7222XR'
,'S7222XS'
,'S7223XC'
,'S7223XD'
,'S7223XE'
,'S7223XF'
,'S7223XG'
,'S7223XH'
,'S7223XJ'
,'S7223XK'
,'S7223XM'
,'S7223XN'
,'S7223XP'
,'S7223XQ'
,'S7223XR'
,'S7223XS'
,'S7224XC'
,'S7224XD'
,'S7224XE'
,'S7224XF'
,'S7224XG'
,'S7224XH'
,'S7224XJ'
,'S7224XK'
,'S7224XM'
,'S7224XN'
,'S7224XP'
,'S7224XQ'
,'S7224XR'
,'S7224XS'
,'S7225XC'
,'S7225XD'
,'S7225XE'
,'S7225XF'
,'S7225XG'
,'S7225XH'
,'S7225XJ'
,'S7225XK'
,'S7225XM'
,'S7225XN'
,'S7225XP'
,'S7225XQ'
,'S7225XR'
,'S7225XS'
,'S7226XC'
,'S7226XD'
,'S7226XE'
,'S7226XF'
,'S7226XG'
,'S7226XH'
,'S7226XJ'
,'S7226XK'
,'S7226XM'
,'S7226XN'
,'S7226XP'
,'S7226XQ'
,'S7226XR'
,'S7226XS'
,'S72301C'
,'S72301D'
,'S72301E'
,'S72301F'
,'S72301G'
,'S72301H'
,'S72301J'
,'S72301K'
,'S72301M'
,'S72301N'
,'S72301P'
,'S72301Q'
,'S72301R'
,'S72301S'
,'S72302C'
,'S72302D'
,'S72302E'
,'S72302F'
,'S72302G'
,'S72302H'
,'S72302J'
,'S72302K'
,'S72302M'
,'S72302N'
,'S72302P'
,'S72302Q'
,'S72302R'
,'S72302S'
,'S72309C'
,'S72309D'
,'S72309E'
,'S72309F'
,'S72309G'
,'S72309H'
,'S72309J'
,'S72309K'
,'S72309M'
,'S72309N'
,'S72309P'
,'S72309Q'
,'S72309R'
,'S72309S'
,'S72321C'
,'S72321D'
,'S72321E'
,'S72321F'
,'S72321G'
,'S72321H'
,'S72321J'
,'S72321K'
,'S72321M'
,'S72321N'
,'S72321P'
,'S72321Q'
,'S72321R'
,'S72321S'
,'S72322C'
,'S72322D'
,'S72322E'
,'S72322F'
,'S72322G'
,'S72322H'
,'S72322J'
,'S72322K'
,'S72322M'
,'S72322N'
,'S72322P'
,'S72322Q'
,'S72322R'
,'S72322S'
,'S72323C'
,'S72323D'
,'S72323E'
,'S72323F'
,'S72323G'
,'S72323H'
,'S72323J'
,'S72323K'
,'S72323M'
,'S72323N'
,'S72323P'
,'S72323Q'
,'S72323R'
,'S72323S'
,'S72324C'
,'S72324D'
,'S72324E'
,'S72324F'
,'S72324G'
,'S72324H'
,'S72324J'
,'S72324K'
,'S72324M'
,'S72324N'
,'S72324P'
,'S72324Q'
,'S72324R'
,'S72324S'
,'S72325C'
,'S72325D'
,'S72325E'
,'S72325F'
,'S72325G'
,'S72325H'
,'S72325J'
,'S72325K'
,'S72325M'
,'S72325N'
,'S72325P'
,'S72325Q'
,'S72325R'
,'S72325S'
,'S72326C'
,'S72326D'
,'S72326E'
,'S72326F'
,'S72326G'
,'S72326H'
,'S72326J'
,'S72326K'
,'S72326M'
,'S72326N'
,'S72326P'
,'S72326Q'
,'S72326R'
,'S72326S'
,'S72331C'
,'S72331D'
,'S72331E'
,'S72331F'
,'S72331G'
,'S72331H'
,'S72331J'
,'S72331K'
,'S72331M'
,'S72331N'
,'S72331P'
,'S72331Q'
,'S72331R'
,'S72331S'
,'S72332C'
,'S72332D'
,'S72332E'
,'S72332F'
,'S72332G'
,'S72332H'
,'S72332J'
,'S72332K'
,'S72332M'
,'S72332N'
,'S72332P'
,'S72332Q'
,'S72332R'
,'S72332S'
,'S72333C'
,'S72333D'
,'S72333E'
,'S72333F'
,'S72333G'
,'S72333H'
,'S72333J'
,'S72333K'
,'S72333M'
,'S72333N'
,'S72333P'
,'S72333Q'
,'S72333R'
,'S72333S'
,'S72334C'
,'S72334D'
,'S72334E'
,'S72334F'
,'S72334G'
,'S72334H'
,'S72334J'
,'S72334K'
,'S72334M'
,'S72334N'
,'S72334P'
,'S72334Q'
,'S72334R'
,'S72334S'
,'S72335C'
,'S72335D'
,'S72335E'
,'S72335F'
,'S72335G'
,'S72335H'
,'S72335J'
,'S72335K'
,'S72335M'
,'S72335N'
,'S72335P'
,'S72335Q'
,'S72335R'
,'S72335S'
,'S72336C'
,'S72336D'
,'S72336E'
,'S72336F'
,'S72336G'
,'S72336H'
,'S72336J'
,'S72336K'
,'S72336M'
,'S72336N'
,'S72336P'
,'S72336Q'
,'S72336R'
,'S72336S'
,'S72341C'
,'S72341D'
,'S72341E'
,'S72341F'
,'S72341G'
,'S72341H'
,'S72341J'
,'S72341K'
,'S72341M'
,'S72341N'
,'S72341P'
,'S72341Q'
,'S72341R'
,'S72341S'
,'S72342C'
,'S72342D'
,'S72342E'
,'S72342F'
,'S72342G'
,'S72342H'
,'S72342J'
,'S72342K'
,'S72342M'
,'S72342N'
,'S72342P'
,'S72342Q'
,'S72342R'
,'S72342S'
,'S72343C'
,'S72343D'
,'S72343E'
,'S72343F'
,'S72343G'
,'S72343H'
,'S72343J'
,'S72343K'
,'S72343M'
,'S72343N'
,'S72343P'
,'S72343Q'
,'S72343R'
,'S72343S'
,'S72344C'
,'S72344D'
,'S72344E'
,'S72344F'
,'S72344G'
,'S72344H'
,'S72344J'
,'S72344K'
,'S72344M'
,'S72344N'
,'S72344P'
,'S72344Q'
,'S72344R'
,'S72344S'
,'S72345C'
,'S72345D'
,'S72345E'
,'S72345F'
,'S72345G'
,'S72345H'
,'S72345J'
,'S72345K'
,'S72345M'
,'S72345N'
,'S72345P'
,'S72345Q'
,'S72345R'
,'S72345S'
,'S72346C'
,'S72346D'
,'S72346E'
,'S72346F'
,'S72346G'
,'S72346H'
,'S72346J'
,'S72346K'
,'S72346M'
,'S72346N'
,'S72346P'
,'S72346Q'
,'S72346R'
,'S72346S'
,'S72351C'
,'S72351D'
,'S72351E'
,'S72351F'
,'S72351G'
,'S72351H'
,'S72351J'
,'S72351K'
,'S72351M'
,'S72351N'
,'S72351P'
,'S72351Q'
,'S72351R'
,'S72351S'
,'S72352C'
,'S72352D'
,'S72352E'
,'S72352F'
,'S72352G'
,'S72352H'
,'S72352J'
,'S72352K'
,'S72352M'
,'S72352N'
,'S72352P'
,'S72352Q'
,'S72352R'
,'S72352S'
,'S72353C'
,'S72353D'
,'S72353E'
,'S72353F'
,'S72353G'
,'S72353H'
,'S72353J'
,'S72353K'
,'S72353M'
,'S72353N'
,'S72353P'
,'S72353Q'
,'S72353R'
,'S72353S'
,'S72354C'
,'S72354D'
,'S72354E'
,'S72354F'
,'S72354G'
,'S72354H'
,'S72354J'
,'S72354K'
,'S72354M'
,'S72354N'
,'S72354P'
,'S72354Q'
,'S72354R'
,'S72354S'
,'S72355C'
,'S72355D'
,'S72355E'
,'S72355F'
,'S72355G'
,'S72355H'
,'S72355J'
,'S72355K'
,'S72355M'
,'S72355N'
,'S72355P'
,'S72355Q'
,'S72355R'
,'S72355S'
,'S72356C'
,'S72356D'
,'S72356E'
,'S72356F'
,'S72356G'
,'S72356H'
,'S72356J'
,'S72356K'
,'S72356M'
,'S72356N'
,'S72356P'
,'S72356Q'
,'S72356R'
,'S72356S'
,'S72361C'
,'S72361D'
,'S72361E'
,'S72361F'
,'S72361G'
,'S72361H'
,'S72361J'
,'S72361K'
,'S72361M'
,'S72361N'
,'S72361P'
,'S72361Q'
,'S72361R'
,'S72361S'
,'S72362C'
,'S72362D'
,'S72362E'
,'S72362F'
,'S72362G'
,'S72362H'
,'S72362J'
,'S72362K'
,'S72362M'
,'S72362N'
,'S72362P'
,'S72362Q'
,'S72362R'
,'S72362S'
,'S72363C'
,'S72363D'
,'S72363E'
,'S72363F'
,'S72363G'
,'S72363H'
,'S72363J'
,'S72363K'
,'S72363M'
,'S72363N'
,'S72363P'
,'S72363Q'
,'S72363R'
,'S72363S'
,'S72364C'
,'S72364D'
,'S72364E'
,'S72364F'
,'S72364G'
,'S72364H'
,'S72364J'
,'S72364K'
,'S72364M'
,'S72364N'
,'S72364P'
,'S72364Q'
,'S72364R'
,'S72364S'
,'S72365C'
,'S72365D'
,'S72365E'
,'S72365F'
,'S72365G'
,'S72365H'
,'S72365J'
,'S72365K'
,'S72365M'
,'S72365N'
,'S72365P'
,'S72365Q'
,'S72365R'
,'S72365S'
,'S72366C'
,'S72366D'
,'S72366E'
,'S72366F'
,'S72366G'
,'S72366H'
,'S72366J'
,'S72366K'
,'S72366M'
,'S72366N'
,'S72366P'
,'S72366Q'
,'S72366R'
,'S72366S'
,'S72391C'
,'S72391D'
,'S72391E'
,'S72391F'
,'S72391G'
,'S72391H'
,'S72391J'
,'S72391K'
,'S72391M'
,'S72391N'
,'S72391P'
,'S72391Q'
,'S72391R'
,'S72391S'
,'S72392C'
,'S72392D'
,'S72392E'
,'S72392F'
,'S72392G'
,'S72392H'
,'S72392J'
,'S72392K'
,'S72392M'
,'S72392N'
,'S72392P'
,'S72392Q'
,'S72392R'
,'S72392S'
,'S72399C'
,'S72399D'
,'S72399E'
,'S72399F'
,'S72399G'
,'S72399H'
,'S72399J'
,'S72399K'
,'S72399M'
,'S72399N'
,'S72399P'
,'S72399Q'
,'S72399R'
,'S72399S'
,'S72401C'
,'S72401D'
,'S72401E'
,'S72401F'
,'S72401G'
,'S72401H'
,'S72401J'
,'S72401K'
,'S72401M'
,'S72401N'
,'S72401P'
,'S72401Q'
,'S72401R'
,'S72401S'
,'S72402C'
,'S72402D'
,'S72402E'
,'S72402F'
,'S72402G'
,'S72402H'
,'S72402J'
,'S72402K'
,'S72402M'
,'S72402N'
,'S72402P'
,'S72402Q'
,'S72402R'
,'S72402S'
,'S72409C'
,'S72409D'
,'S72409E'
,'S72409F'
,'S72409G'
,'S72409H'
,'S72409J'
,'S72409K'
,'S72409M'
,'S72409N'
,'S72409P'
,'S72409Q'
,'S72409R'
,'S72409S'
,'S72411C'
,'S72411D'
,'S72411E'
,'S72411F'
,'S72411G'
,'S72411H'
,'S72411J'
,'S72411K'
,'S72411M'
,'S72411N'
,'S72411P'
,'S72411Q'
,'S72411R'
,'S72411S'
,'S72412C'
,'S72412D'
,'S72412E'
,'S72412F'
,'S72412G'
,'S72412H'
,'S72412J'
,'S72412K'
,'S72412M'
,'S72412N'
,'S72412P'
,'S72412Q'
,'S72412R'
,'S72412S'
,'S72413C'
,'S72413D'
,'S72413E'
,'S72413F'
,'S72413G'
,'S72413H'
,'S72413J'
,'S72413K'
,'S72413M'
,'S72413N'
,'S72413P'
,'S72413Q'
,'S72413R'
,'S72413S'
,'S72414C'
,'S72414D'
,'S72414E'
,'S72414F'
,'S72414G'
,'S72414H'
,'S72414J'
,'S72414K'
,'S72414M'
,'S72414N'
,'S72414P'
,'S72414Q'
,'S72414R'
,'S72414S'
,'S72415C'
,'S72415D'
,'S72415E'
,'S72415F'
,'S72415G'
,'S72415H'
,'S72415J'
,'S72415K'
,'S72415M'
,'S72415N'
,'S72415P'
,'S72415Q'
,'S72415R'
,'S72415S'
,'S72416C'
,'S72416D'
,'S72416E'
,'S72416F'
,'S72416G'
,'S72416H'
,'S72416J'
,'S72416K'
,'S72416M'
,'S72416N'
,'S72416P'
,'S72416Q'
,'S72416R'
,'S72416S'
,'S72421C'
,'S72421D'
,'S72421E'
,'S72421F'
,'S72421G'
,'S72421H'
,'S72421J'
,'S72421K'
,'S72421M'
,'S72421N'
,'S72421P'
,'S72421Q'
,'S72421R'
,'S72421S'
,'S72422C'
,'S72422D'
,'S72422E'
,'S72422F'
,'S72422G'
,'S72422H'
,'S72422J'
,'S72422K'
,'S72422M'
,'S72422N'
,'S72422P'
,'S72422Q'
,'S72422R'
,'S72422S'
,'S72423C'
,'S72423D'
,'S72423E'
,'S72423F'
,'S72423G'
,'S72423H'
,'S72423J'
,'S72423K'
,'S72423M'
,'S72423N'
,'S72423P'
,'S72423Q'
,'S72423R'
,'S72423S'
,'S72424C'
,'S72424D'
,'S72424E'
,'S72424F'
,'S72424G'
,'S72424H'
,'S72424J'
,'S72424K'
,'S72424M'
,'S72424N'
,'S72424P'
,'S72424Q'
,'S72424R'
,'S72424S'
,'S72425C'
,'S72425D'
,'S72425E'
,'S72425F'
,'S72425G'
,'S72425H'
,'S72425J'
,'S72425K'
,'S72425M'
,'S72425N'
,'S72425P'
,'S72425Q'
,'S72425R'
,'S72425S'
,'S72426C'
,'S72426D'
,'S72426E'
,'S72426F'
,'S72426G'
,'S72426H'
,'S72426J'
,'S72426K'
,'S72426M'
,'S72426N'
,'S72426P'
,'S72426Q'
,'S72426R'
,'S72426S'
,'S72431C'
,'S72431D'
,'S72431E'
,'S72431F'
,'S72431G'
,'S72431H'
,'S72431J'
,'S72431K'
,'S72431M'
,'S72431N'
,'S72431P'
,'S72431Q'
,'S72431R'
,'S72431S'
,'S72432C'
,'S72432D'
,'S72432E'
,'S72432F'
,'S72432G'
,'S72432H'
,'S72432J'
,'S72432K'
,'S72432M'
,'S72432N'
,'S72432P'
,'S72432Q'
,'S72432R'
,'S72432S'
,'S72433C'
,'S72433D'
,'S72433E'
,'S72433F'
,'S72433G'
,'S72433H'
,'S72433J'
,'S72433K'
,'S72433M'
,'S72433N'
,'S72433P'
,'S72433Q'
,'S72433R'
,'S72433S'
,'S72434C'
,'S72434D'
,'S72434E'
,'S72434F'
,'S72434G'
,'S72434H'
,'S72434J'
,'S72434K'
,'S72434M'
,'S72434N'
,'S72434P'
,'S72434Q'
,'S72434R'
,'S72434S'
,'S72435C'
,'S72435D'
,'S72435E'
,'S72435F'
,'S72435G'
,'S72435H'
,'S72435J'
,'S72435K'
,'S72435M'
,'S72435N'
,'S72435P'
,'S72435Q'
,'S72435R'
,'S72435S'
,'S72436C'
,'S72436D'
,'S72436E'
,'S72436F'
,'S72436G'
,'S72436H'
,'S72436J'
,'S72436K'
,'S72436M'
,'S72436N'
,'S72436P'
,'S72436Q'
,'S72436R'
,'S72436S'
,'S72441C'
,'S72441D'
,'S72441E'
,'S72441F'
,'S72441G'
,'S72441H'
,'S72441J'
,'S72441K'
,'S72441M'
,'S72441N'
,'S72441P'
,'S72441Q'
,'S72441R'
,'S72441S'
,'S72442C'
,'S72442D'
,'S72442E'
,'S72442F'
,'S72442G'
,'S72442H'
,'S72442J'
,'S72442K'
,'S72442M'
,'S72442N'
,'S72442P'
,'S72442Q'
,'S72442R'
,'S72442S'
,'S72443C'
,'S72443D'
,'S72443E'
,'S72443F'
,'S72443G'
,'S72443H'
,'S72443J'
,'S72443K'
,'S72443M'
,'S72443N'
,'S72443P'
,'S72443Q'
,'S72443R'
,'S72443S'
,'S72444C'
,'S72444D'
,'S72444E'
,'S72444F'
,'S72444G'
,'S72444H'
,'S72444J'
,'S72444K'
,'S72444M'
,'S72444N'
,'S72444P'
,'S72444Q'
,'S72444R'
,'S72444S'
,'S72445C'
,'S72445D'
,'S72445E'
,'S72445F'
,'S72445G'
,'S72445H'
,'S72445J'
,'S72445K'
,'S72445M'
,'S72445N'
,'S72445P'
,'S72445Q'
,'S72445R'
,'S72445S'
,'S72446C'
,'S72446D'
,'S72446E'
,'S72446F'
,'S72446G'
,'S72446H'
,'S72446J'
,'S72446K'
,'S72446M'
,'S72446N'
,'S72446P'
,'S72446Q'
,'S72446R'
,'S72446S'
,'S72451C'
,'S72451D'
,'S72451E'
,'S72451F'
,'S72451G'
,'S72451H'
,'S72451J'
,'S72451K'
,'S72451M'
,'S72451N'
,'S72451P'
,'S72451Q'
,'S72451R'
,'S72451S'
,'S72452C'
,'S72452D'
,'S72452E'
,'S72452F'
,'S72452G'
,'S72452H'
,'S72452J'
,'S72452K'
,'S72452M'
,'S72452N'
,'S72452P'
,'S72452Q'
,'S72452R'
,'S72452S'
,'S72453C'
,'S72453D'
,'S72453E'
,'S72453F'
,'S72453G'
,'S72453H'
,'S72453J'
,'S72453K'
,'S72453M'
,'S72453N'
,'S72453P'
,'S72453Q'
,'S72453R'
,'S72453S'
,'S72454C'
,'S72454D'
,'S72454E'
,'S72454F'
,'S72454G'
,'S72454H'
,'S72454J'
,'S72454K'
,'S72454M'
,'S72454N'
,'S72454P'
,'S72454Q'
,'S72454R'
,'S72454S'
,'S72455C'
,'S72455D'
,'S72455E'
,'S72455F'
,'S72455G'
,'S72455H'
,'S72455J'
,'S72455K'
,'S72455M'
,'S72455N'
,'S72455P'
,'S72455Q'
,'S72455R'
,'S72455S'
,'S72456C'
,'S72456D'
,'S72456E'
,'S72456F'
,'S72456G'
,'S72456H'
,'S72456J'
,'S72456K'
,'S72456M'
,'S72456N'
,'S72456P'
,'S72456Q'
,'S72456R'
,'S72456S'
,'S72461C'
,'S72461D'
,'S72461E'
,'S72461F'
,'S72461G'
,'S72461H'
,'S72461J'
,'S72461K'
,'S72461M'
,'S72461N'
,'S72461P'
,'S72461Q'
,'S72461R'
,'S72461S'
,'S72462C'
,'S72462D'
,'S72462E'
,'S72462F'
,'S72462G'
,'S72462H'
,'S72462J'
,'S72462K'
,'S72462M'
,'S72462N'
,'S72462P'
,'S72462Q'
,'S72462R'
,'S72462S'
,'S72463C'
,'S72463D'
,'S72463E'
,'S72463F'
,'S72463G'
,'S72463H'
,'S72463J'
,'S72463K'
,'S72463M'
,'S72463N'
,'S72463P'
,'S72463Q'
,'S72463R'
,'S72463S'
,'S72464C'
,'S72464D'
,'S72464E'
,'S72464F'
,'S72464G'
,'S72464H'
,'S72464J'
,'S72464K'
,'S72464M'
,'S72464N'
,'S72464P'
,'S72464Q'
,'S72464R'
,'S72464S'
,'S72465C'
,'S72465D'
,'S72465E'
,'S72465F'
,'S72465G'
,'S72465H'
,'S72465J'
,'S72465K'
,'S72465M'
,'S72465N'
,'S72465P'
,'S72465Q'
,'S72465R'
,'S72465S'
,'S72466C'
,'S72466D'
,'S72466E'
,'S72466F'
,'S72466G'
,'S72466H'
,'S72466J'
,'S72466K'
,'S72466M'
,'S72466N'
,'S72466P'
,'S72466Q'
,'S72466R'
,'S72466S'
,'S72471D'
,'S72471G'
,'S72471K'
,'S72471P'
,'S72471S'
,'S72472D'
,'S72472G'
,'S72472K'
,'S72472P'
,'S72472S'
,'S72479D'
,'S72479G'
,'S72479K'
,'S72479P'
,'S72479S'
,'S72491C'
,'S72491D'
,'S72491E'
,'S72491F'
,'S72491G'
,'S72491H'
,'S72491J'
,'S72491K'
,'S72491M'
,'S72491N'
,'S72491P'
,'S72491Q'
,'S72491R'
,'S72491S'
,'S72492C'
,'S72492D'
,'S72492E'
,'S72492F'
,'S72492G'
,'S72492H'
,'S72492J'
,'S72492K'
,'S72492M'
,'S72492N'
,'S72492P'
,'S72492Q'
,'S72492R'
,'S72492S'
,'S72499C'
,'S72499D'
,'S72499E'
,'S72499F'
,'S72499G'
,'S72499H'
,'S72499J'
,'S72499K'
,'S72499M'
,'S72499N'
,'S72499P'
,'S72499Q'
,'S72499R'
,'S72499S'
,'S728X1C'
,'S728X1D'
,'S728X1E'
,'S728X1F'
,'S728X1G'
,'S728X1H'
,'S728X1J'
,'S728X1K'
,'S728X1M'
,'S728X1N'
,'S728X1P'
,'S728X1Q'
,'S728X1R'
,'S728X1S'
,'S728X2C'
,'S728X2D'
,'S728X2E'
,'S728X2F'
,'S728X2G'
,'S728X2H'
,'S728X2J'
,'S728X2K'
,'S728X2M'
,'S728X2N'
,'S728X2P'
,'S728X2Q'
,'S728X2R'
,'S728X2S'
,'S728X9C'
,'S728X9D'
,'S728X9E'
,'S728X9F'
,'S728X9G'
,'S728X9H'
,'S728X9J'
,'S728X9K'
,'S728X9M'
,'S728X9N'
,'S728X9P'
,'S728X9Q'
,'S728X9R'
,'S728X9S'
,'S7290XC'
,'S7290XD'
,'S7290XE'
,'S7290XF'
,'S7290XG'
,'S7290XH'
,'S7290XJ'
,'S7290XK'
,'S7290XM'
,'S7290XN'
,'S7290XP'
,'S7290XQ'
,'S7290XR'
,'S7290XS'
,'S7291XC'
,'S7291XD'
,'S7291XE'
,'S7291XF'
,'S7291XG'
,'S7291XH'
,'S7291XJ'
,'S7291XK'
,'S7291XM'
,'S7291XN'
,'S7291XP'
,'S7291XQ'
,'S7291XR'
,'S7291XS'
,'S7292XC'
,'S7292XD'
,'S7292XE'
,'S7292XF'
,'S7292XG'
,'S7292XH'
,'S7292XJ'
,'S7292XK'
,'S7292XM'
,'S7292XN'
,'S7292XP'
,'S7292XQ'
,'S7292XR'
,'S7292XS'
,'S73001D'
,'S73001S'
,'S73002D'
,'S73002S'
,'S73003D'
,'S73003S'
,'S73004D'
,'S73004S'
,'S73005D'
,'S73005S'
,'S73006D'
,'S73006S'
,'S73011D'
,'S73011S'
,'S73012D'
,'S73012S'
,'S73013D'
,'S73013S'
,'S73014D'
,'S73014S'
,'S73015D'
,'S73015S'
,'S73016D'
,'S73016S'
,'S73021D'
,'S73021S'
,'S73022D'
,'S73022S'
,'S73023D'
,'S73023S'
,'S73024D'
,'S73024S'
,'S73025D'
,'S73025S'
,'S73026D'
,'S73026S'
,'S73031D'
,'S73031S'
,'S73032D'
,'S73032S'
,'S73033D'
,'S73033S'
,'S73034D'
,'S73034S'
,'S73035D'
,'S73035S'
,'S73036D'
,'S73036S'
,'S73041D'
,'S73041S'
,'S73042D'
,'S73042S'
,'S73043D'
,'S73043S'
,'S73044D'
,'S73044S'
,'S73045D'
,'S73045S'
,'S73046D'
,'S73046S'
,'S73101D'
,'S73101S'
,'S73102D'
,'S73102S'
,'S73109D'
,'S73109S'
,'S73111D'
,'S73111S'
,'S73112D'
,'S73112S'
,'S73119D'
,'S73119S'
,'S73121D'
,'S73121S'
,'S73122D'
,'S73122S'
,'S73129D'
,'S73129S'
,'S73191D'
,'S73191S'
,'S73192D'
,'S73192S'
,'S73199D'
,'S73199S'
,'S7400XD'
,'S7400XS'
,'S7401XD'
,'S7401XS'
,'S7402XD'
,'S7402XS'
,'S7410XD'
,'S7410XS'
,'S7411XD'
,'S7411XS'
,'S7412XD'
,'S7412XS'
,'S7420XD'
,'S7420XS'
,'S7421XD'
,'S7421XS'
,'S7422XD'
,'S7422XS'
,'S748X1D'
,'S748X1S'
,'S748X2D'
,'S748X2S'
,'S748X9D'
,'S748X9S'
,'S7490XD'
,'S7490XS'
,'S7491XD'
,'S7491XS'
,'S7492XD'
,'S7492XS'
,'S75001D'
,'S75001S'
,'S75002D'
,'S75002S'
,'S75009D'
,'S75009S'
,'S75011D'
,'S75011S'
,'S75012D'
,'S75012S'
,'S75019D'
,'S75019S'
,'S75021D'
,'S75021S'
,'S75022D'
,'S75022S'
,'S75029D'
,'S75029S'
,'S75091D'
,'S75091S'
,'S75092D'
,'S75092S'
,'S75099D'
,'S75099S'
,'S75101D'
,'S75101S'
,'S75102D'
,'S75102S'
,'S75109D'
,'S75109S'
,'S75111D'
,'S75111S'
,'S75112D'
,'S75112S'
,'S75119D'
,'S75119S'
,'S75121D'
,'S75121S'
,'S75122D'
,'S75122S'
,'S75129D'
,'S75129S'
,'S75191D'
,'S75191S'
,'S75192D'
,'S75192S'
,'S75199D'
,'S75199S'
,'S75201D'
,'S75201S'
,'S75202D'
,'S75202S'
,'S75209D'
,'S75209S'
,'S75211D'
,'S75211S'
,'S75212D'
,'S75212S'
,'S75219D'
,'S75219S'
,'S75221D'
,'S75221S'
,'S75222D'
,'S75222S'
,'S75229D'
,'S75229S'
,'S75291D'
,'S75291S'
,'S75292D'
,'S75292S'
,'S75299D'
,'S75299S'
,'S75801D'
,'S75801S'
,'S75802D'
,'S75802S'
,'S75809D'
,'S75809S'
,'S75811D'
,'S75811S'
,'S75812D'
,'S75812S'
,'S75819D'
,'S75819S'
,'S75891D'
,'S75891S'
,'S75892D'
,'S75892S'
,'S75899D'
,'S75899S'
,'S75901D'
,'S75901S'
,'S75902D'
,'S75902S'
,'S75909D'
,'S75909S'
,'S75911D'
,'S75911S'
,'S75912D'
,'S75912S'
,'S75919D'
,'S75919S'
,'S75991D'
,'S75991S'
,'S75992D'
,'S75992S'
,'S75999D'
,'S75999S'
,'S76001D'
,'S76001S'
,'S76002D'
,'S76002S'
,'S76009D'
,'S76009S'
,'S76011D'
,'S76011S'
,'S76012D'
,'S76012S'
,'S76019D'
,'S76019S'
,'S76021D'
,'S76021S'
,'S76022D'
,'S76022S'
,'S76029D'
,'S76029S'
,'S76091D'
,'S76091S'
,'S76092D'
,'S76092S'
,'S76099D'
,'S76099S'
,'S76101D'
,'S76101S'
,'S76102D'
,'S76102S'
,'S76109D'
,'S76109S'
,'S76111D'
,'S76111S'
,'S76112D'
,'S76112S'
,'S76119D'
,'S76119S'
,'S76121D'
,'S76121S'
,'S76122D'
,'S76122S'
,'S76129D'
,'S76129S'
,'S76191D'
,'S76191S'
,'S76192D'
,'S76192S'
,'S76199D'
,'S76199S'
,'S76201D'
,'S76201S'
,'S76202D'
,'S76202S'
,'S76209D'
,'S76209S'
,'S76211D'
,'S76211S'
,'S76212D'
,'S76212S'
,'S76219D'
,'S76219S'
,'S76221D'
,'S76221S'
,'S76222D'
,'S76222S'
,'S76229D'
,'S76229S'
,'S76291D'
,'S76291S'
,'S76292D'
,'S76292S'
,'S76299D'
,'S76299S'
,'S76301D'
,'S76301S'
,'S76302D'
,'S76302S'
,'S76309D'
,'S76309S'
,'S76311D'
,'S76311S'
,'S76312D'
,'S76312S'
,'S76319D'
,'S76319S'
,'S76321D'
,'S76321S'
,'S76322D'
,'S76322S'
,'S76329D'
,'S76329S'
,'S76391D'
,'S76391S'
,'S76392D'
,'S76392S'
,'S76399D'
,'S76399S'
,'S76801D'
,'S76801S'
,'S76802D'
,'S76802S'
,'S76809D'
,'S76809S'
,'S76811D'
,'S76811S'
,'S76812D'
,'S76812S'
,'S76819D'
,'S76819S'
,'S76821D'
,'S76821S'
,'S76822D'
,'S76822S'
,'S76829D'
,'S76829S'
,'S76891D'
,'S76891S'
,'S76892D'
,'S76892S'
,'S76899D'
,'S76899S'
,'S76901D'
,'S76901S'
,'S76902D'
,'S76902S'
,'S76909D'
,'S76909S'
,'S76911D'
,'S76911S'
,'S76912D'
,'S76912S'
,'S76919D'
,'S76919S'
,'S76921D'
,'S76921S'
,'S76922D'
,'S76922S'
,'S76929D'
,'S76929S'
,'S76991D'
,'S76991S'
,'S76992D'
,'S76992S'
,'S76999D'
,'S76999S'
,'S7700XD'
,'S7700XS'
,'S7701XD'
,'S7701XS'
,'S7702XD'
,'S7702XS'
,'S7710XD'
,'S7710XS'
,'S7711XD'
,'S7711XS'
,'S7712XD'
,'S7712XS'
,'S7720XD'
,'S7720XS'
,'S7721XD'
,'S7721XS'
,'S7722XD'
,'S7722XS'
,'S78011D'
,'S78011S'
,'S78012D'
,'S78012S'
,'S78019D'
,'S78019S'
,'S78021D'
,'S78021S'
,'S78022D'
,'S78022S'
,'S78029D'
,'S78029S'
,'S78111D'
,'S78111S'
,'S78112D'
,'S78112S'
,'S78119D'
,'S78119S'
,'S78121D'
,'S78121S'
,'S78122D'
,'S78122S'
,'S78129D'
,'S78129S'
,'S78911D'
,'S78911S'
,'S78912D'
,'S78912S'
,'S78919D'
,'S78919S'
,'S78921D'
,'S78921S'
,'S78922D'
,'S78922S'
,'S78929D'
,'S78929S'
,'S79001D'
,'S79001G'
,'S79001K'
,'S79001P'
,'S79001S'
,'S79002D'
,'S79002G'
,'S79002K'
,'S79002P'
,'S79002S'
,'S79009D'
,'S79009G'
,'S79009K'
,'S79009P'
,'S79009S'
,'S79011D'
,'S79011G'
,'S79011K'
,'S79011P'
,'S79011S'
,'S79012D'
,'S79012G'
,'S79012K'
,'S79012P'
,'S79012S'
,'S79019D'
,'S79019G'
,'S79019K'
,'S79019P'
,'S79019S'
,'S79091D'
,'S79091G'
,'S79091K'
,'S79091P'
,'S79091S'
,'S79092D'
,'S79092G'
,'S79092K'
,'S79092P'
,'S79092S'
,'S79099D'
,'S79099G'
,'S79099K'
,'S79099P'
,'S79099S'
,'S79101D'
,'S79101G'
,'S79101K'
,'S79101P'
,'S79101S'
,'S79102D'
,'S79102G'
,'S79102K'
,'S79102P'
,'S79102S'
,'S79109D'
,'S79109G'
,'S79109K'
,'S79109P'
,'S79109S'
,'S79111D'
,'S79111G'
,'S79111K'
,'S79111P'
,'S79111S'
,'S79112D'
,'S79112G'
,'S79112K'
,'S79112P'
,'S79112S'
,'S79119D'
,'S79119G'
,'S79119K'
,'S79119P'
,'S79119S'
,'S79121D'
,'S79121G'
,'S79121K'
,'S79121P'
,'S79121S'
,'S79122D'
,'S79122G'
,'S79122K'
,'S79122P'
,'S79122S'
,'S79129D'
,'S79129G'
,'S79129K'
,'S79129P'
,'S79129S'
,'S79131D'
,'S79131G'
,'S79131K'
,'S79131P'
,'S79131S'
,'S79132D'
,'S79132G'
,'S79132K'
,'S79132P'
,'S79132S'
,'S79139D'
,'S79139G'
,'S79139K'
,'S79139P'
,'S79139S'
,'S79141D'
,'S79141G'
,'S79141K'
,'S79141P'
,'S79141S'
,'S79142D'
,'S79142G'
,'S79142K'
,'S79142P'
,'S79142S'
,'S79149D'
,'S79149G'
,'S79149K'
,'S79149P'
,'S79149S'
,'S79191D'
,'S79191G'
,'S79191K'
,'S79191P'
,'S79191S'
,'S79192D'
,'S79192G'
,'S79192K'
,'S79192P'
,'S79192S'
,'S79199D'
,'S79199G'
,'S79199K'
,'S79199P'
,'S79199S'
,'S79811D'
,'S79811S'
,'S79812D'
,'S79812S'
,'S79819D'
,'S79819S'
,'S79821D'
,'S79821S'
,'S79822D'
,'S79822S'
,'S79829D'
,'S79829S'
,'S79911D'
,'S79911S'
,'S79912D'
,'S79912S'
,'S79919D'
,'S79919S'
,'S79921D'
,'S79921S'
,'S79922D'
,'S79922S'
,'S79929D'
,'S79929S'
,'S8000XD'
,'S8000XS'
,'S8001XD'
,'S8001XS'
,'S8002XD'
,'S8002XS'
,'S8010XD'
,'S8010XS'
,'S8011XD'
,'S8011XS'
,'S8012XD'
,'S8012XS'
,'S80211D'
,'S80211S'
,'S80212D'
,'S80212S'
,'S80219D'
,'S80219S'
,'S80221D'
,'S80221S'
,'S80222D'
,'S80222S'
,'S80229D'
,'S80229S'
,'S80241D'
,'S80241S'
,'S80242D'
,'S80242S'
,'S80249D'
,'S80249S'
,'S80251D'
,'S80251S'
,'S80252D'
,'S80252S'
,'S80259D'
,'S80259S'
,'S80261D'
,'S80261S'
,'S80262D'
,'S80262S'
,'S80269D'
,'S80269S'
,'S80271D'
,'S80271S'
,'S80272D'
,'S80272S'
,'S80279D'
,'S80279S'
,'S80811D'
,'S80811S'
,'S80812D'
,'S80812S'
,'S80819D'
,'S80819S'
,'S80821D'
,'S80821S'
,'S80822D'
,'S80822S'
,'S80829D'
,'S80829S'
,'S80841D'
,'S80841S'
,'S80842D'
,'S80842S'
,'S80849D'
,'S80849S'
,'S80851D'
,'S80851S'
,'S80852D'
,'S80852S'
,'S80859D'
,'S80859S'
,'S80861D'
,'S80861S'
,'S80862D'
,'S80862S'
,'S80869D'
,'S80869S'
,'S80871D'
,'S80871S'
,'S80872D'
,'S80872S'
,'S80879D'
,'S80879S'
,'S80911D'
,'S80911S'
,'S80912D'
,'S80912S'
,'S80919D'
,'S80919S'
,'S80921D'
,'S80921S'
,'S80922D'
,'S80922S'
,'S80929D'
,'S80929S'
,'S81001D'
,'S81001S'
,'S81002D'
,'S81002S'
,'S81009D'
,'S81009S'
,'S81011D'
,'S81011S'
,'S81012D'
,'S81012S'
,'S81019D'
,'S81019S'
,'S81021D'
,'S81021S'
,'S81022D'
,'S81022S'
,'S81029D'
,'S81029S'
,'S81031D'
,'S81031S'
,'S81032D'
,'S81032S'
,'S81039D'
,'S81039S'
,'S81041D'
,'S81041S'
,'S81042D'
,'S81042S'
,'S81049D'
,'S81049S'
,'S81051D'
,'S81051S'
,'S81052D'
,'S81052S'
,'S81059D'
,'S81059S'
,'S81801D'
,'S81801S'
,'S81802D'
,'S81802S'
,'S81809D'
,'S81809S'
,'S81811D'
,'S81811S'
,'S81812D'
,'S81812S'
,'S81819D'
,'S81819S'
,'S81821D'
,'S81821S'
,'S81822D'
,'S81822S'
,'S81829D'
,'S81829S'
,'S81831D'
,'S81831S'
,'S81832D'
,'S81832S'
,'S81839D'
,'S81839S'
,'S81841D'
,'S81841S'
,'S81842D'
,'S81842S'
,'S81849D'
,'S81849S'
,'S81851D'
,'S81851S'
,'S81852D'
,'S81852S'
,'S81859D'
,'S81859S'
,'S82001C'
,'S82001D'
,'S82001E'
,'S82001F'
,'S82001G'
,'S82001H'
,'S82001J'
,'S82001K'
,'S82001M'
,'S82001N'
,'S82001P'
,'S82001Q'
,'S82001R'
,'S82001S'
,'S82002C'
,'S82002D'
,'S82002E'
,'S82002F'
,'S82002G'
,'S82002H'
,'S82002J'
,'S82002K'
,'S82002M'
,'S82002N'
,'S82002P'
,'S82002Q'
,'S82002R'
,'S82002S'
,'S82009C'
,'S82009D'
,'S82009E'
,'S82009F'
,'S82009G'
,'S82009H'
,'S82009J'
,'S82009K'
,'S82009M'
,'S82009N'
,'S82009P'
,'S82009Q'
,'S82009R'
,'S82009S'
,'S82011C'
,'S82011D'
,'S82011E'
,'S82011F'
,'S82011G'
,'S82011H'
,'S82011J'
,'S82011K'
,'S82011M'
,'S82011N'
,'S82011P'
,'S82011Q'
,'S82011R'
,'S82011S'
,'S82012C'
,'S82012D'
,'S82012E'
,'S82012F'
,'S82012G'
,'S82012H'
,'S82012J'
,'S82012K'
,'S82012M'
,'S82012N'
,'S82012P'
,'S82012Q'
,'S82012R'
,'S82012S'
,'S82013C'
,'S82013D'
,'S82013E'
,'S82013F'
,'S82013G'
,'S82013H'
,'S82013J'
,'S82013K'
,'S82013M'
,'S82013N'
,'S82013P'
,'S82013Q'
,'S82013R'
,'S82013S'
,'S82014C'
,'S82014D'
,'S82014E'
,'S82014F'
,'S82014G'
,'S82014H'
,'S82014J'
,'S82014K'
,'S82014M'
,'S82014N'
,'S82014P'
,'S82014Q'
,'S82014R'
,'S82014S'
,'S82015C'
,'S82015D'
,'S82015E'
,'S82015F'
,'S82015G'
,'S82015H'
,'S82015J'
,'S82015K'
,'S82015M'
,'S82015N'
,'S82015P'
,'S82015Q'
,'S82015R'
,'S82015S'
,'S82016C'
,'S82016D'
,'S82016E'
,'S82016F'
,'S82016G'
,'S82016H'
,'S82016J'
,'S82016K'
,'S82016M'
,'S82016N'
,'S82016P'
,'S82016Q'
,'S82016R'
,'S82016S'
,'S82021C'
,'S82021D'
,'S82021E'
,'S82021F'
,'S82021G'
,'S82021H'
,'S82021J'
,'S82021K'
,'S82021M'
,'S82021N'
,'S82021P'
,'S82021Q'
,'S82021R'
,'S82021S'
,'S82022C'
,'S82022D'
,'S82022E'
,'S82022F'
,'S82022G'
,'S82022H'
,'S82022J'
,'S82022K'
,'S82022M'
,'S82022N'
,'S82022P'
,'S82022Q'
,'S82022R'
,'S82022S'
,'S82023C'
,'S82023D'
,'S82023E'
,'S82023F'
,'S82023G'
,'S82023H'
,'S82023J'
,'S82023K'
,'S82023M'
,'S82023N'
,'S82023P'
,'S82023Q'
,'S82023R'
,'S82023S'
,'S82024C'
,'S82024D'
,'S82024E'
,'S82024F'
,'S82024G'
,'S82024H'
,'S82024J'
,'S82024K'
,'S82024M'
,'S82024N'
,'S82024P'
,'S82024Q'
,'S82024R'
,'S82024S'
,'S82025C'
,'S82025D'
,'S82025E'
,'S82025F'
,'S82025G'
,'S82025H'
,'S82025J'
,'S82025K'
,'S82025M'
,'S82025N'
,'S82025P'
,'S82025Q'
,'S82025R'
,'S82025S'
,'S82026C'
,'S82026D'
,'S82026E'
,'S82026F'
,'S82026G'
,'S82026H'
,'S82026J'
,'S82026K'
,'S82026M'
,'S82026N'
,'S82026P'
,'S82026Q'
,'S82026R'
,'S82026S'
,'S82031C'
,'S82031D'
,'S82031E'
,'S82031F'
,'S82031G'
,'S82031H'
,'S82031J'
,'S82031K'
,'S82031M'
,'S82031N'
,'S82031P'
,'S82031Q'
,'S82031R'
,'S82031S'
,'S82032C'
,'S82032D'
,'S82032E'
,'S82032F'
,'S82032G'
,'S82032H'
,'S82032J'
,'S82032K'
,'S82032M'
,'S82032N'
,'S82032P'
,'S82032Q'
,'S82032R'
,'S82032S'
,'S82033C'
,'S82033D'
,'S82033E'
,'S82033F'
,'S82033G'
,'S82033H'
,'S82033J'
,'S82033K'
,'S82033M'
,'S82033N'
,'S82033P'
,'S82033Q'
,'S82033R'
,'S82033S'
,'S82034C'
,'S82034D'
,'S82034E'
,'S82034F'
,'S82034G'
,'S82034H'
,'S82034J'
,'S82034K'
,'S82034M'
,'S82034N'
,'S82034P'
,'S82034Q'
,'S82034R'
,'S82034S'
,'S82035C'
,'S82035D'
,'S82035E'
,'S82035F'
,'S82035G'
,'S82035H'
,'S82035J'
,'S82035K'
,'S82035M'
,'S82035N'
,'S82035P'
,'S82035Q'
,'S82035R'
,'S82035S'
,'S82036C'
,'S82036D'
,'S82036E'
,'S82036F'
,'S82036G'
,'S82036H'
,'S82036J'
,'S82036K'
,'S82036M'
,'S82036N'
,'S82036P'
,'S82036Q'
,'S82036R'
,'S82036S'
,'S82041C'
,'S82041D'
,'S82041E'
,'S82041F'
,'S82041G'
,'S82041H'
,'S82041J'
,'S82041K'
,'S82041M'
,'S82041N'
,'S82041P'
,'S82041Q'
,'S82041R'
,'S82041S'
,'S82042C'
,'S82042D'
,'S82042E'
,'S82042F'
,'S82042G'
,'S82042H'
,'S82042J'
,'S82042K'
,'S82042M'
,'S82042N'
,'S82042P'
,'S82042Q'
,'S82042R'
,'S82042S'
,'S82043C'
,'S82043D'
,'S82043E'
,'S82043F'
,'S82043G'
,'S82043H'
,'S82043J'
,'S82043K'
,'S82043M'
,'S82043N'
,'S82043P'
,'S82043Q'
,'S82043R'
,'S82043S'
,'S82044C'
,'S82044D'
,'S82044E'
,'S82044F'
,'S82044G'
,'S82044H'
,'S82044J'
,'S82044K'
,'S82044M'
,'S82044N'
,'S82044P'
,'S82044Q'
,'S82044R'
,'S82044S'
,'S82045C'
,'S82045D'
,'S82045E'
,'S82045F'
,'S82045G'
,'S82045H'
,'S82045J'
,'S82045K'
,'S82045M'
,'S82045N'
,'S82045P'
,'S82045Q'
,'S82045R'
,'S82045S'
,'S82046C'
,'S82046D'
,'S82046E'
,'S82046F'
,'S82046G'
,'S82046H'
,'S82046J'
,'S82046K'
,'S82046M'
,'S82046N'
,'S82046P'
,'S82046Q'
,'S82046R'
,'S82046S'
,'S82091C'
,'S82091D'
,'S82091E'
,'S82091F'
,'S82091G'
,'S82091H'
,'S82091J'
,'S82091K'
,'S82091M'
,'S82091N'
,'S82091P'
,'S82091Q'
,'S82091R'
,'S82091S'
,'S82092C'
,'S82092D'
,'S82092E'
,'S82092F'
,'S82092G'
,'S82092H'
,'S82092J'
,'S82092K'
,'S82092M'
,'S82092N'
,'S82092P'
,'S82092Q'
,'S82092R'
,'S82092S'
,'S82099C'
,'S82099D'
,'S82099E'
,'S82099F'
,'S82099G'
,'S82099H'
,'S82099J'
,'S82099K'
,'S82099M'
,'S82099N'
,'S82099P'
,'S82099Q'
,'S82099R'
,'S82099S'
,'S82101C'
,'S82101D'
,'S82101E'
,'S82101F'
,'S82101G'
,'S82101H'
,'S82101J'
,'S82101K'
,'S82101M'
,'S82101N'
,'S82101P'
,'S82101Q'
,'S82101R'
,'S82101S'
,'S82102C'
,'S82102D'
,'S82102E'
,'S82102F'
,'S82102G'
,'S82102H'
,'S82102J'
,'S82102K'
,'S82102M'
,'S82102N'
,'S82102P'
,'S82102Q'
,'S82102R'
,'S82102S'
,'S82109C'
,'S82109D'
,'S82109E'
,'S82109F'
,'S82109G'
,'S82109H'
,'S82109J'
,'S82109K'
,'S82109M'
,'S82109N'
,'S82109P'
,'S82109Q'
,'S82109R'
,'S82109S'
,'S82111C'
,'S82111D'
,'S82111E'
,'S82111F'
,'S82111G'
,'S82111H'
,'S82111J'
,'S82111K'
,'S82111M'
,'S82111N'
,'S82111P'
,'S82111Q'
,'S82111R'
,'S82111S'
,'S82112C'
,'S82112D'
,'S82112E'
,'S82112F'
,'S82112G'
,'S82112H'
,'S82112J'
,'S82112K'
,'S82112M'
,'S82112N'
,'S82112P'
,'S82112Q'
,'S82112R'
,'S82112S'
,'S82113C'
,'S82113D'
,'S82113E'
,'S82113F'
,'S82113G'
,'S82113H'
,'S82113J'
,'S82113K'
,'S82113M'
,'S82113N'
,'S82113P'
,'S82113Q'
,'S82113R'
,'S82113S'
,'S82114C'
,'S82114D'
,'S82114E'
,'S82114F'
,'S82114G'
,'S82114H'
,'S82114J'
,'S82114K'
,'S82114M'
,'S82114N'
,'S82114P'
,'S82114Q'
,'S82114R'
,'S82114S'
,'S82115C'
,'S82115D'
,'S82115E'
,'S82115F'
,'S82115G'
,'S82115H'
,'S82115J'
,'S82115K'
,'S82115M'
,'S82115N'
,'S82115P'
,'S82115Q'
,'S82115R'
,'S82115S'
,'S82116C'
,'S82116D'
,'S82116E'
,'S82116F'
,'S82116G'
,'S82116H'
,'S82116J'
,'S82116K'
,'S82116M'
,'S82116N'
,'S82116P'
,'S82116Q'
,'S82116R'
,'S82116S'
,'S82121C'
,'S82121D'
,'S82121E'
,'S82121F'
,'S82121G'
,'S82121H'
,'S82121J'
,'S82121K'
,'S82121M'
,'S82121N'
,'S82121P'
,'S82121Q'
,'S82121R'
,'S82121S'
,'S82122C'
,'S82122D'
,'S82122E'
,'S82122F'
,'S82122G'
,'S82122H'
,'S82122J'
,'S82122K'
,'S82122M'
,'S82122N'
,'S82122P'
,'S82122Q'
,'S82122R'
,'S82122S'
,'S82123C'
,'S82123D'
,'S82123E'
,'S82123F'
,'S82123G'
,'S82123H'
,'S82123J'
,'S82123K'
,'S82123M'
,'S82123N'
,'S82123P'
,'S82123Q'
,'S82123R'
,'S82123S'
,'S82124C'
,'S82124D'
,'S82124E'
,'S82124F'
,'S82124G'
,'S82124H'
,'S82124J'
,'S82124K'
,'S82124M'
,'S82124N'
,'S82124P'
,'S82124Q'
,'S82124R'
,'S82124S'
,'S82125C'
,'S82125D'
,'S82125E'
,'S82125F'
,'S82125G'
,'S82125H'
,'S82125J'
,'S82125K'
,'S82125M'
,'S82125N'
,'S82125P'
,'S82125Q'
,'S82125R'
,'S82125S'
,'S82126C'
,'S82126D'
,'S82126E'
,'S82126F'
,'S82126G'
,'S82126H'
,'S82126J'
,'S82126K'
,'S82126M'
,'S82126N'
,'S82126P'
,'S82126Q'
,'S82126R'
,'S82126S'
,'S82131C'
,'S82131D'
,'S82131E'
,'S82131F'
,'S82131G'
,'S82131H'
,'S82131J'
,'S82131K'
,'S82131M'
,'S82131N'
,'S82131P'
,'S82131Q'
,'S82131R'
,'S82131S'
,'S82132C'
,'S82132D'
,'S82132E'
,'S82132F'
,'S82132G'
,'S82132H'
,'S82132J'
,'S82132K'
,'S82132M'
,'S82132N'
,'S82132P'
,'S82132Q'
,'S82132R'
,'S82132S'
,'S82133C'
,'S82133D'
,'S82133E'
,'S82133F'
,'S82133G'
,'S82133H'
,'S82133J'
,'S82133K'
,'S82133M'
,'S82133N'
,'S82133P'
,'S82133Q'
,'S82133R'
,'S82133S'
,'S82134C'
,'S82134D'
,'S82134E'
,'S82134F'
,'S82134G'
,'S82134H'
,'S82134J'
,'S82134K'
,'S82134M'
,'S82134N'
,'S82134P'
,'S82134Q'
,'S82134R'
,'S82134S'
,'S82135C'
,'S82135D'
,'S82135E'
,'S82135F'
,'S82135G'
,'S82135H'
,'S82135J'
,'S82135K'
,'S82135M'
,'S82135N'
,'S82135P'
,'S82135Q'
,'S82135R'
,'S82135S'
,'S82136C'
,'S82136D'
,'S82136E'
,'S82136F'
,'S82136G'
,'S82136H'
,'S82136J'
,'S82136K'
,'S82136M'
,'S82136N'
,'S82136P'
,'S82136Q'
,'S82136R'
,'S82136S'
,'S82141C'
,'S82141D'
,'S82141E'
,'S82141F'
,'S82141G'
,'S82141H'
,'S82141J'
,'S82141K'
,'S82141M'
,'S82141N'
,'S82141P'
,'S82141Q'
,'S82141R'
,'S82141S'
,'S82142C'
,'S82142D'
,'S82142E'
,'S82142F'
,'S82142G'
,'S82142H'
,'S82142J'
,'S82142K'
,'S82142M'
,'S82142N'
,'S82142P'
,'S82142Q'
,'S82142R'
,'S82142S'
,'S82143C'
,'S82143D'
,'S82143E'
,'S82143F'
,'S82143G'
,'S82143H'
,'S82143J'
,'S82143K'
,'S82143M'
,'S82143N'
,'S82143P'
,'S82143Q'
,'S82143R'
,'S82143S'
,'S82144C'
,'S82144D'
,'S82144E'
,'S82144F'
,'S82144G'
,'S82144H'
,'S82144J'
,'S82144K'
,'S82144M'
,'S82144N'
,'S82144P'
,'S82144Q'
,'S82144R'
,'S82144S'
,'S82145C'
,'S82145D'
,'S82145E'
,'S82145F'
,'S82145G'
,'S82145H'
,'S82145J'
,'S82145K'
,'S82145M'
,'S82145N'
,'S82145P'
,'S82145Q'
,'S82145R'
,'S82145S'
,'S82146C'
,'S82146D'
,'S82146E'
,'S82146F'
,'S82146G'
,'S82146H'
,'S82146J'
,'S82146K'
,'S82146M'
,'S82146N'
,'S82146P'
,'S82146Q'
,'S82146R'
,'S82146S'
,'S82151C'
,'S82151D'
,'S82151E'
,'S82151F'
,'S82151G'
,'S82151H'
,'S82151J'
,'S82151K'
,'S82151M'
,'S82151N'
,'S82151P'
,'S82151Q'
,'S82151R'
,'S82151S'
,'S82152C'
,'S82152D'
,'S82152E'
,'S82152F'
,'S82152G'
,'S82152H'
,'S82152J'
,'S82152K'
,'S82152M'
,'S82152N'
,'S82152P'
,'S82152Q'
,'S82152R'
,'S82152S'
,'S82153C'
,'S82153D'
,'S82153E'
,'S82153F'
,'S82153G'
,'S82153H'
,'S82153J'
,'S82153K'
,'S82153M'
,'S82153N'
,'S82153P'
,'S82153Q'
,'S82153R'
,'S82153S'
,'S82154C'
,'S82154D'
,'S82154E'
,'S82154F'
,'S82154G'
,'S82154H'
,'S82154J'
,'S82154K'
,'S82154M'
,'S82154N'
,'S82154P'
,'S82154Q'
,'S82154R'
,'S82154S'
,'S82155C'
,'S82155D'
,'S82155E'
,'S82155F'
,'S82155G'
,'S82155H'
,'S82155J'
,'S82155K'
,'S82155M'
,'S82155N'
,'S82155P'
,'S82155Q'
,'S82155R'
,'S82155S'
,'S82156C'
,'S82156D'
,'S82156E'
,'S82156F'
,'S82156G'
,'S82156H'
,'S82156J'
,'S82156K'
,'S82156M'
,'S82156N'
,'S82156P'
,'S82156Q'
,'S82156R'
,'S82156S'
,'S82161D'
,'S82161G'
,'S82161K'
,'S82161P'
,'S82161S'
,'S82162D'
,'S82162G'
,'S82162K'
,'S82162P'
,'S82162S'
,'S82169D'
,'S82169G'
,'S82169K'
,'S82169P'
,'S82169S'
,'S82191C'
,'S82191D'
,'S82191E'
,'S82191F'
,'S82191G'
,'S82191H'
,'S82191J'
,'S82191K'
,'S82191M'
,'S82191N'
,'S82191P'
,'S82191Q'
,'S82191R'
,'S82191S'
,'S82192C'
,'S82192D'
,'S82192E'
,'S82192F'
,'S82192G'
,'S82192H'
,'S82192J'
,'S82192K'
,'S82192M'
,'S82192N'
,'S82192P'
,'S82192Q'
,'S82192R'
,'S82192S'
,'S82199C'
,'S82199D'
,'S82199E'
,'S82199F'
,'S82199G'
,'S82199H'
,'S82199J'
,'S82199K'
,'S82199M'
,'S82199N'
,'S82199P'
,'S82199Q'
,'S82199R'
,'S82199S'
,'S82201C'
,'S82201D'
,'S82201E'
,'S82201F'
,'S82201G'
,'S82201H'
,'S82201J'
,'S82201K'
,'S82201M'
,'S82201N'
,'S82201P'
,'S82201Q'
,'S82201R'
,'S82201S'
,'S82202C'
,'S82202D'
,'S82202E'
,'S82202F'
,'S82202G'
,'S82202H'
,'S82202J'
,'S82202K'
,'S82202M'
,'S82202N'
,'S82202P'
,'S82202Q'
,'S82202R'
,'S82202S'
,'S82209C'
,'S82209D'
,'S82209E'
,'S82209F'
,'S82209G'
,'S82209H'
,'S82209J'
,'S82209K'
,'S82209M'
,'S82209N'
,'S82209P'
,'S82209Q'
,'S82209R'
,'S82209S'
,'S82221C'
,'S82221D'
,'S82221E'
,'S82221F'
,'S82221G'
,'S82221H'
,'S82221J'
,'S82221K'
,'S82221M'
,'S82221N'
,'S82221P'
,'S82221Q'
,'S82221R'
,'S82221S'
,'S82222C'
,'S82222D'
,'S82222E'
,'S82222F'
,'S82222G'
,'S82222H'
,'S82222J'
,'S82222K'
,'S82222M'
,'S82222N'
,'S82222P'
,'S82222Q'
,'S82222R'
,'S82222S'
,'S82223C'
,'S82223D'
,'S82223E'
,'S82223F'
,'S82223G'
,'S82223H'
,'S82223J'
,'S82223K'
,'S82223M'
,'S82223N'
,'S82223P'
,'S82223Q'
,'S82223R'
,'S82223S'
,'S82224C'
,'S82224D'
,'S82224E'
,'S82224F'
,'S82224G'
,'S82224H'
,'S82224J'
,'S82224K'
,'S82224M'
,'S82224N'
,'S82224P'
,'S82224Q'
,'S82224R'
,'S82224S'
,'S82225C'
,'S82225D'
,'S82225E'
,'S82225F'
,'S82225G'
,'S82225H'
,'S82225J'
,'S82225K'
,'S82225M'
,'S82225N'
,'S82225P'
,'S82225Q'
,'S82225R'
,'S82225S'
,'S82226C'
,'S82226D'
,'S82226E'
,'S82226F'
,'S82226G'
,'S82226H'
,'S82226J'
,'S82226K'
,'S82226M'
,'S82226N'
,'S82226P'
,'S82226Q'
,'S82226R'
,'S82226S'
,'S82231C'
,'S82231D'
,'S82231E'
,'S82231F'
,'S82231G'
,'S82231H'
,'S82231J'
,'S82231K'
,'S82231M'
,'S82231N'
,'S82231P'
,'S82231Q'
,'S82231R'
,'S82231S'
,'S82232C'
,'S82232D'
,'S82232E'
,'S82232F'
,'S82232G'
,'S82232H'
,'S82232J'
,'S82232K'
,'S82232M'
,'S82232N'
,'S82232P'
,'S82232Q'
,'S82232R'
,'S82232S'
,'S82233C'
,'S82233D'
,'S82233E'
,'S82233F'
,'S82233G'
,'S82233H'
,'S82233J'
,'S82233K'
,'S82233M'
,'S82233N'
,'S82233P'
,'S82233Q'
,'S82233R'
,'S82233S'
,'S82234C'
,'S82234D'
,'S82234E'
,'S82234F'
,'S82234G'
,'S82234H'
,'S82234J'
,'S82234K'
,'S82234M'
,'S82234N'
,'S82234P'
,'S82234Q'
,'S82234R'
,'S82234S'
,'S82235C'
,'S82235D'
,'S82235E'
,'S82235F'
,'S82235G'
,'S82235H'
,'S82235J'
,'S82235K'
,'S82235M'
,'S82235N'
,'S82235P'
,'S82235Q'
,'S82235R'
,'S82235S'
,'S82236C'
,'S82236D'
,'S82236E'
,'S82236F'
,'S82236G'
,'S82236H'
,'S82236J'
,'S82236K'
,'S82236M'
,'S82236N'
,'S82236P'
,'S82236Q'
,'S82236R'
,'S82236S'
,'S82241C'
,'S82241D'
,'S82241E'
,'S82241F'
,'S82241G'
,'S82241H'
,'S82241J'
,'S82241K'
,'S82241M'
,'S82241N'
,'S82241P'
,'S82241Q'
,'S82241R'
,'S82241S'
,'S82242C'
,'S82242D'
,'S82242E'
,'S82242F'
,'S82242G'
,'S82242H'
,'S82242J'
,'S82242K'
,'S82242M'
,'S82242N'
,'S82242P'
,'S82242Q'
,'S82242R'
,'S82242S'
,'S82243C'
,'S82243D'
,'S82243E'
,'S82243F'
,'S82243G'
,'S82243H'
,'S82243J'
,'S82243K'
,'S82243M'
,'S82243N'
,'S82243P'
,'S82243Q'
,'S82243R'
,'S82243S'
,'S82244C'
,'S82244D'
,'S82244E'
,'S82244F'
,'S82244G'
,'S82244H'
,'S82244J'
,'S82244K'
,'S82244M'
,'S82244N'
,'S82244P'
,'S82244Q'
,'S82244R'
,'S82244S'
,'S82245C'
,'S82245D'
,'S82245E'
,'S82245F'
,'S82245G'
,'S82245H'
,'S82245J'
,'S82245K'
,'S82245M'
,'S82245N'
,'S82245P'
,'S82245Q'
,'S82245R'
,'S82245S'
,'S82246C'
,'S82246D'
,'S82246E'
,'S82246F'
,'S82246G'
,'S82246H'
,'S82246J'
,'S82246K'
,'S82246M'
,'S82246N'
,'S82246P'
,'S82246Q'
,'S82246R'
,'S82246S'
,'S82251C'
,'S82251D'
,'S82251E'
,'S82251F'
,'S82251G'
,'S82251H'
,'S82251J'
,'S82251K'
,'S82251M'
,'S82251N'
,'S82251P'
,'S82251Q'
,'S82251R'
,'S82251S'
,'S82252C'
,'S82252D'
,'S82252E'
,'S82252F'
,'S82252G'
,'S82252H'
,'S82252J'
,'S82252K'
,'S82252M'
,'S82252N'
,'S82252P'
,'S82252Q'
,'S82252R'
,'S82252S'
,'S82253C'
,'S82253D'
,'S82253E'
,'S82253F'
,'S82253G'
,'S82253H'
,'S82253J'
,'S82253K'
,'S82253M'
,'S82253N'
,'S82253P'
,'S82253Q'
,'S82253R'
,'S82253S'
,'S82254C'
,'S82254D'
,'S82254E'
,'S82254F'
,'S82254G'
,'S82254H'
,'S82254J'
,'S82254K'
,'S82254M'
,'S82254N'
,'S82254P'
,'S82254Q'
,'S82254R'
,'S82254S'
,'S82255C'
,'S82255D'
,'S82255E'
,'S82255F'
,'S82255G'
,'S82255H'
,'S82255J'
,'S82255K'
,'S82255M'
,'S82255N'
,'S82255P'
,'S82255Q'
,'S82255R'
,'S82255S'
,'S82256C'
,'S82256D'
,'S82256E'
,'S82256F'
,'S82256G'
,'S82256H'
,'S82256J'
,'S82256K'
,'S82256M'
,'S82256N'
,'S82256P'
,'S82256Q'
,'S82256R'
,'S82256S'
,'S82261C'
,'S82261D'
,'S82261E'
,'S82261F'
,'S82261G'
,'S82261H'
,'S82261J'
,'S82261K'
,'S82261M'
,'S82261N'
,'S82261P'
,'S82261Q'
,'S82261R'
,'S82261S'
,'S82262C'
,'S82262D'
,'S82262E'
,'S82262F'
,'S82262G'
,'S82262H'
,'S82262J'
,'S82262K'
,'S82262M'
,'S82262N'
,'S82262P'
,'S82262Q'
,'S82262R'
,'S82262S'
,'S82263C'
,'S82263D'
,'S82263E'
,'S82263F'
,'S82263G'
,'S82263H'
,'S82263J'
,'S82263K'
,'S82263M'
,'S82263N'
,'S82263P'
,'S82263Q'
,'S82263R'
,'S82263S'
,'S82264C'
,'S82264D'
,'S82264E'
,'S82264F'
,'S82264G'
,'S82264H'
,'S82264J'
,'S82264K'
,'S82264M'
,'S82264N'
,'S82264P'
,'S82264Q'
,'S82264R'
,'S82264S'
,'S82265C'
,'S82265D'
,'S82265E'
,'S82265F'
,'S82265G'
,'S82265H'
,'S82265J'
,'S82265K'
,'S82265M'
,'S82265N'
,'S82265P'
,'S82265Q'
,'S82265R'
,'S82265S'
,'S82266C'
,'S82266D'
,'S82266E'
,'S82266F'
,'S82266G'
,'S82266H'
,'S82266J'
,'S82266K'
,'S82266M'
,'S82266N'
,'S82266P'
,'S82266Q'
,'S82266R'
,'S82266S'
,'S82291C'
,'S82291D'
,'S82291E'
,'S82291F'
,'S82291G'
,'S82291H'
,'S82291J'
,'S82291K'
,'S82291M'
,'S82291N'
,'S82291P'
,'S82291Q'
,'S82291R'
,'S82291S'
,'S82292C'
,'S82292D'
,'S82292E'
,'S82292F'
,'S82292G'
,'S82292H'
,'S82292J'
,'S82292K'
,'S82292M'
,'S82292N'
,'S82292P'
,'S82292Q'
,'S82292R'
,'S82292S'
,'S82299C'
,'S82299D'
,'S82299E'
,'S82299F'
,'S82299G'
,'S82299H'
,'S82299J'
,'S82299K'
,'S82299M'
,'S82299N'
,'S82299P'
,'S82299Q'
,'S82299R'
,'S82299S'
,'S82301C'
,'S82301D'
,'S82301E'
,'S82301F'
,'S82301G'
,'S82301H'
,'S82301J'
,'S82301K'
,'S82301M'
,'S82301N'
,'S82301P'
,'S82301Q'
,'S82301R'
,'S82301S'
,'S82302C'
,'S82302D'
,'S82302E'
,'S82302F'
,'S82302G'
,'S82302H'
,'S82302J'
,'S82302K'
,'S82302M'
,'S82302N'
,'S82302P'
,'S82302Q'
,'S82302R'
,'S82302S'
,'S82309C'
,'S82309D'
,'S82309E'
,'S82309F'
,'S82309G'
,'S82309H'
,'S82309J'
,'S82309K'
,'S82309M'
,'S82309N'
,'S82309P'
,'S82309Q'
,'S82309R'
,'S82309S'
,'S82311D'
,'S82311G'
,'S82311K'
,'S82311P'
,'S82311S'
,'S82312D'
,'S82312G'
,'S82312K'
,'S82312P'
,'S82312S'
,'S82319D'
,'S82319G'
,'S82319K'
,'S82319P'
,'S82319S'
,'S82391C'
,'S82391D'
,'S82391E'
,'S82391F'
,'S82391G'
,'S82391H'
,'S82391J'
,'S82391K'
,'S82391M'
,'S82391N'
,'S82391P'
,'S82391Q'
,'S82391R'
,'S82391S'
,'S82392C'
,'S82392D'
,'S82392E'
,'S82392F'
,'S82392G'
,'S82392H'
,'S82392J'
,'S82392K'
,'S82392M'
,'S82392N'
,'S82392P'
,'S82392Q'
,'S82392R'
,'S82392S'
,'S82399C'
,'S82399D'
,'S82399E'
,'S82399F'
,'S82399G'
,'S82399H'
,'S82399J'
,'S82399K'
,'S82399M'
,'S82399N'
,'S82399P'
,'S82399Q'
,'S82399R'
,'S82399S'
,'S82401C'
,'S82401D'
,'S82401E'
,'S82401F'
,'S82401G'
,'S82401H'
,'S82401J'
,'S82401K'
,'S82401M'
,'S82401N'
,'S82401P'
,'S82401Q'
,'S82401R'
,'S82401S'
,'S82402C'
,'S82402D'
,'S82402E'
,'S82402F'
,'S82402G'
,'S82402H'
,'S82402J'
,'S82402K'
,'S82402M'
,'S82402N'
,'S82402P'
,'S82402Q'
,'S82402R'
,'S82402S'
,'S82409C'
,'S82409D'
,'S82409E'
,'S82409F'
,'S82409G'
,'S82409H'
,'S82409J'
,'S82409K'
,'S82409M'
,'S82409N'
,'S82409P'
,'S82409Q'
,'S82409R'
,'S82409S'
,'S82421C'
,'S82421D'
,'S82421E'
,'S82421F'
,'S82421G'
,'S82421H'
,'S82421J'
,'S82421K'
,'S82421M'
,'S82421N'
,'S82421P'
,'S82421Q'
,'S82421R'
,'S82421S'
,'S82422C'
,'S82422D'
,'S82422E'
,'S82422F'
,'S82422G'
,'S82422H'
,'S82422J'
,'S82422K'
,'S82422M'
,'S82422N'
,'S82422P'
,'S82422Q'
,'S82422R'
,'S82422S'
,'S82423C'
,'S82423D'
,'S82423E'
,'S82423F'
,'S82423G'
,'S82423H'
,'S82423J'
,'S82423K'
,'S82423M'
,'S82423N'
,'S82423P'
,'S82423Q'
,'S82423R'
,'S82423S'
,'S82424C'
,'S82424D'
,'S82424E'
,'S82424F'
,'S82424G'
,'S82424H'
,'S82424J'
,'S82424K'
,'S82424M'
,'S82424N'
,'S82424P'
,'S82424Q'
,'S82424R'
,'S82424S'
,'S82425C'
,'S82425D'
,'S82425E'
,'S82425F'
,'S82425G'
,'S82425H'
,'S82425J'
,'S82425K'
,'S82425M'
,'S82425N'
,'S82425P'
,'S82425Q'
,'S82425R'
,'S82425S'
,'S82426C'
,'S82426D'
,'S82426E'
,'S82426F'
,'S82426G'
,'S82426H'
,'S82426J'
,'S82426K'
,'S82426M'
,'S82426N'
,'S82426P'
,'S82426Q'
,'S82426R'
,'S82426S'
,'S82431C'
,'S82431D'
,'S82431E'
,'S82431F'
,'S82431G'
,'S82431H'
,'S82431J'
,'S82431K'
,'S82431M'
,'S82431N'
,'S82431P'
,'S82431Q'
,'S82431R'
,'S82431S'
,'S82432C'
,'S82432D'
,'S82432E'
,'S82432F'
,'S82432G'
,'S82432H'
,'S82432J'
,'S82432K'
,'S82432M'
,'S82432N'
,'S82432P'
,'S82432Q'
,'S82432R'
,'S82432S'
,'S82433C'
,'S82433D'
,'S82433E'
,'S82433F'
,'S82433G'
,'S82433H'
,'S82433J'
,'S82433K'
,'S82433M'
,'S82433N'
,'S82433P'
,'S82433Q'
,'S82433R'
,'S82433S'
,'S82434C'
,'S82434D'
,'S82434E'
,'S82434F'
,'S82434G'
,'S82434H'
,'S82434J'
,'S82434K'
,'S82434M'
,'S82434N'
,'S82434P'
,'S82434Q'
,'S82434R'
,'S82434S'
,'S82435C'
,'S82435D'
,'S82435E'
,'S82435F'
,'S82435G'
,'S82435H'
,'S82435J'
,'S82435K'
,'S82435M'
,'S82435N'
,'S82435P'
,'S82435Q'
,'S82435R'
,'S82435S'
,'S82436C'
,'S82436D'
,'S82436E'
,'S82436F'
,'S82436G'
,'S82436H'
,'S82436J'
,'S82436K'
,'S82436M'
,'S82436N'
,'S82436P'
,'S82436Q'
,'S82436R'
,'S82436S'
,'S82441C'
,'S82441D'
,'S82441E'
,'S82441F'
,'S82441G'
,'S82441H'
,'S82441J'
,'S82441K'
,'S82441M'
,'S82441N'
,'S82441P'
,'S82441Q'
,'S82441R'
,'S82441S'
,'S82442C'
,'S82442D'
,'S82442E'
,'S82442F'
,'S82442G'
,'S82442H'
,'S82442J'
,'S82442K'
,'S82442M'
,'S82442N'
,'S82442P'
,'S82442Q'
,'S82442R'
,'S82442S'
,'S82443C'
,'S82443D'
,'S82443E'
,'S82443F'
,'S82443G'
,'S82443H'
,'S82443J'
,'S82443K'
,'S82443M'
,'S82443N'
,'S82443P'
,'S82443Q'
,'S82443R'
,'S82443S'
,'S82444C'
,'S82444D'
,'S82444E'
,'S82444F'
,'S82444G'
,'S82444H'
,'S82444J'
,'S82444K'
,'S82444M'
,'S82444N'
,'S82444P'
,'S82444Q'
,'S82444R'
,'S82444S'
,'S82445C'
,'S82445D'
,'S82445E'
,'S82445F'
,'S82445G'
,'S82445H'
,'S82445J'
,'S82445K'
,'S82445M'
,'S82445N'
,'S82445P'
,'S82445Q'
,'S82445R'
,'S82445S'
,'S82446C'
,'S82446D'
,'S82446E'
,'S82446F'
,'S82446G'
,'S82446H'
,'S82446J'
,'S82446K'
,'S82446M'
,'S82446N'
,'S82446P'
,'S82446Q'
,'S82446R'
,'S82446S'
,'S82451C'
,'S82451D'
,'S82451E'
,'S82451F'
,'S82451G'
,'S82451H'
,'S82451J'
,'S82451K'
,'S82451M'
,'S82451N'
,'S82451P'
,'S82451Q'
,'S82451R'
,'S82451S'
,'S82452C'
,'S82452D'
,'S82452E'
,'S82452F'
,'S82452G'
,'S82452H'
,'S82452J'
,'S82452K'
,'S82452M'
,'S82452N'
,'S82452P'
,'S82452Q'
,'S82452R'
,'S82452S'
,'S82453C'
,'S82453D'
,'S82453E'
,'S82453F'
,'S82453G'
,'S82453H'
,'S82453J'
,'S82453K'
,'S82453M'
,'S82453N'
,'S82453P'
,'S82453Q'
,'S82453R'
,'S82453S'
,'S82454C'
,'S82454D'
,'S82454E'
,'S82454F'
,'S82454G'
,'S82454H'
,'S82454J'
,'S82454K'
,'S82454M'
,'S82454N'
,'S82454P'
,'S82454Q'
,'S82454R'
,'S82454S'
,'S82455C'
,'S82455D'
,'S82455E'
,'S82455F'
,'S82455G'
,'S82455H'
,'S82455J'
,'S82455K'
,'S82455M'
,'S82455N'
,'S82455P'
,'S82455Q'
,'S82455R'
,'S82455S'
,'S82456C'
,'S82456D'
,'S82456E'
,'S82456F'
,'S82456G'
,'S82456H'
,'S82456J'
,'S82456K'
,'S82456M'
,'S82456N'
,'S82456P'
,'S82456Q'
,'S82456R'
,'S82456S'
,'S82461C'
,'S82461D'
,'S82461E'
,'S82461F'
,'S82461G'
,'S82461H'
,'S82461J'
,'S82461K'
,'S82461M'
,'S82461N'
,'S82461P'
,'S82461Q'
,'S82461R'
,'S82461S'
,'S82462C'
,'S82462D'
,'S82462E'
,'S82462F'
,'S82462G'
,'S82462H'
,'S82462J'
,'S82462K'
,'S82462M'
,'S82462N'
,'S82462P'
,'S82462Q'
,'S82462R'
,'S82462S'
,'S82463C'
,'S82463D'
,'S82463E'
,'S82463F'
,'S82463G'
,'S82463H'
,'S82463J'
,'S82463K'
,'S82463M'
,'S82463N'
,'S82463P'
,'S82463Q'
,'S82463R'
,'S82463S'
,'S82464C'
,'S82464D'
,'S82464E'
,'S82464F'
,'S82464G'
,'S82464H'
,'S82464J'
,'S82464K'
,'S82464M'
,'S82464N'
,'S82464P'
,'S82464Q'
,'S82464R'
,'S82464S'
,'S82465C'
,'S82465D'
,'S82465E'
,'S82465F'
,'S82465G'
,'S82465H'
,'S82465J'
,'S82465K'
,'S82465M'
,'S82465N'
,'S82465P'
,'S82465Q'
,'S82465R'
,'S82465S'
,'S82466C'
,'S82466D'
,'S82466E'
,'S82466F'
,'S82466G'
,'S82466H'
,'S82466J'
,'S82466K'
,'S82466M'
,'S82466N'
,'S82466P'
,'S82466Q'
,'S82466R'
,'S82466S'
,'S82491C'
,'S82491D'
,'S82491E'
,'S82491F'
,'S82491G'
,'S82491H'
,'S82491J'
,'S82491K'
,'S82491M'
,'S82491N'
,'S82491P'
,'S82491Q'
,'S82491R'
,'S82491S'
,'S82492C'
,'S82492D'
,'S82492E'
,'S82492F'
,'S82492G'
,'S82492H'
,'S82492J'
,'S82492K'
,'S82492M'
,'S82492N'
,'S82492P'
,'S82492Q'
,'S82492R'
,'S82492S'
,'S82499C'
,'S82499D'
,'S82499E'
,'S82499F'
,'S82499G'
,'S82499H'
,'S82499J'
,'S82499K'
,'S82499M'
,'S82499N'
,'S82499P'
,'S82499Q'
,'S82499R'
,'S82499S'
,'S8251XC'
,'S8251XD'
,'S8251XE'
,'S8251XF'
,'S8251XG'
,'S8251XH'
,'S8251XJ'
,'S8251XK'
,'S8251XM'
,'S8251XN'
,'S8251XP'
,'S8251XQ'
,'S8251XR'
,'S8251XS'
,'S8252XC'
,'S8252XD'
,'S8252XE'
,'S8252XF'
,'S8252XG'
,'S8252XH'
,'S8252XJ'
,'S8252XK'
,'S8252XM'
,'S8252XN'
,'S8252XP'
,'S8252XQ'
,'S8252XR'
,'S8252XS'
,'S8253XC'
,'S8253XD'
,'S8253XE'
,'S8253XF'
,'S8253XG'
,'S8253XH'
,'S8253XJ'
,'S8253XK'
,'S8253XM'
,'S8253XN'
,'S8253XP'
,'S8253XQ'
,'S8253XR'
,'S8253XS'
,'S8254XC'
,'S8254XD'
,'S8254XE'
,'S8254XF'
,'S8254XG'
,'S8254XH'
,'S8254XJ'
,'S8254XK'
,'S8254XM'
,'S8254XN'
,'S8254XP'
,'S8254XQ'
,'S8254XR'
,'S8254XS'
,'S8255XC'
,'S8255XD'
,'S8255XE'
,'S8255XF'
,'S8255XG'
,'S8255XH'
,'S8255XJ'
,'S8255XK'
,'S8255XM'
,'S8255XN'
,'S8255XP'
,'S8255XQ'
,'S8255XR'
,'S8255XS'
,'S8256XC'
,'S8256XD'
,'S8256XE'
,'S8256XF'
,'S8256XG'
,'S8256XH'
,'S8256XJ'
,'S8256XK'
,'S8256XM'
,'S8256XN'
,'S8256XP'
,'S8256XQ'
,'S8256XR'
,'S8256XS'
,'S8261XC'
,'S8261XD'
,'S8261XE'
,'S8261XF'
,'S8261XG'
,'S8261XH'
,'S8261XJ'
,'S8261XK'
,'S8261XM'
,'S8261XN'
,'S8261XP'
,'S8261XQ'
,'S8261XR'
,'S8261XS'
,'S8262XC'
,'S8262XD'
,'S8262XE'
,'S8262XF'
,'S8262XG'
,'S8262XH'
,'S8262XJ'
,'S8262XK'
,'S8262XM'
,'S8262XN'
,'S8262XP'
,'S8262XQ'
,'S8262XR'
,'S8262XS'
,'S8263XC'
,'S8263XD'
,'S8263XE'
,'S8263XF'
,'S8263XG'
,'S8263XH'
,'S8263XJ'
,'S8263XK'
,'S8263XM'
,'S8263XN'
,'S8263XP'
,'S8263XQ'
,'S8263XR'
,'S8263XS'
,'S8264XC'
,'S8264XD'
,'S8264XE'
,'S8264XF'
,'S8264XG'
,'S8264XH'
,'S8264XJ'
,'S8264XK'
,'S8264XM'
,'S8264XN'
,'S8264XP'
,'S8264XQ'
,'S8264XR'
,'S8264XS'
,'S8265XC'
,'S8265XD'
,'S8265XE'
,'S8265XF'
,'S8265XG'
,'S8265XH'
,'S8265XJ'
,'S8265XK'
,'S8265XM'
,'S8265XN'
,'S8265XP'
,'S8265XQ'
,'S8265XR'
,'S8265XS'
,'S8266XC'
,'S8266XD'
,'S8266XE'
,'S8266XF'
,'S8266XG'
,'S8266XH'
,'S8266XJ'
,'S8266XK'
,'S8266XM'
,'S8266XN'
,'S8266XP'
,'S8266XQ'
,'S8266XR'
,'S8266XS'
,'S82811D'
,'S82811G'
,'S82811K'
,'S82811P'
,'S82811S'
,'S82812D'
,'S82812G'
,'S82812K'
,'S82812P'
,'S82812S'
,'S82819D'
,'S82819G'
,'S82819K'
,'S82819P'
,'S82819S'
,'S82821D'
,'S82821G'
,'S82821K'
,'S82821P'
,'S82821S'
,'S82822D'
,'S82822G'
,'S82822K'
,'S82822P'
,'S82822S'
,'S82829D'
,'S82829G'
,'S82829K'
,'S82829P'
,'S82829S'
,'S82831C'
,'S82831D'
,'S82831E'
,'S82831F'
,'S82831G'
,'S82831H'
,'S82831J'
,'S82831K'
,'S82831M'
,'S82831N'
,'S82831P'
,'S82831Q'
,'S82831R'
,'S82831S'
,'S82832C'
,'S82832D'
,'S82832E'
,'S82832F'
,'S82832G'
,'S82832H'
,'S82832J'
,'S82832K'
,'S82832M'
,'S82832N'
,'S82832P'
,'S82832Q'
,'S82832R'
,'S82832S'
,'S82839C'
,'S82839D'
,'S82839E'
,'S82839F'
,'S82839G'
,'S82839H'
,'S82839J'
,'S82839K'
,'S82839M'
,'S82839N'
,'S82839P'
,'S82839Q'
,'S82839R'
,'S82839S'
,'S82841C'
,'S82841D'
,'S82841E'
,'S82841F'
,'S82841G'
,'S82841H'
,'S82841J'
,'S82841K'
,'S82841M'
,'S82841N'
,'S82841P'
,'S82841Q'
,'S82841R'
,'S82841S'
,'S82842C'
,'S82842D'
,'S82842E'
,'S82842F'
,'S82842G'
,'S82842H'
,'S82842J'
,'S82842K'
,'S82842M'
,'S82842N'
,'S82842P'
,'S82842Q'
,'S82842R'
,'S82842S'
,'S82843C'
,'S82843D'
,'S82843E'
,'S82843F'
,'S82843G'
,'S82843H'
,'S82843J'
,'S82843K'
,'S82843M'
,'S82843N'
,'S82843P'
,'S82843Q'
,'S82843R'
,'S82843S'
,'S82844C'
,'S82844D'
,'S82844E'
,'S82844F'
,'S82844G'
,'S82844H'
,'S82844J'
,'S82844K'
,'S82844M'
,'S82844N'
,'S82844P'
,'S82844Q'
,'S82844R'
,'S82844S'
,'S82845C'
,'S82845D'
,'S82845E'
,'S82845F'
,'S82845G'
,'S82845H'
,'S82845J'
,'S82845K'
,'S82845M'
,'S82845N'
,'S82845P'
,'S82845Q'
,'S82845R'
,'S82845S'
,'S82846C'
,'S82846D'
,'S82846E'
,'S82846F'
,'S82846G'
,'S82846H'
,'S82846J'
,'S82846K'
,'S82846M'
,'S82846N'
,'S82846P'
,'S82846Q'
,'S82846R'
,'S82846S'
,'S82851C'
,'S82851D'
,'S82851E'
,'S82851F'
,'S82851G'
,'S82851H'
,'S82851J'
,'S82851K'
,'S82851M'
,'S82851N'
,'S82851P'
,'S82851Q'
,'S82851R'
,'S82851S'
,'S82852C'
,'S82852D'
,'S82852E'
,'S82852F'
,'S82852G'
,'S82852H'
,'S82852J'
,'S82852K'
,'S82852M'
,'S82852N'
,'S82852P'
,'S82852Q'
,'S82852R'
,'S82852S'
,'S82853C'
,'S82853D'
,'S82853E'
,'S82853F'
,'S82853G'
,'S82853H'
,'S82853J'
,'S82853K'
,'S82853M'
,'S82853N'
,'S82853P'
,'S82853Q'
,'S82853R'
,'S82853S'
,'S82854C'
,'S82854D'
,'S82854E'
,'S82854F'
,'S82854G'
,'S82854H'
,'S82854J'
,'S82854K'
,'S82854M'
,'S82854N'
,'S82854P'
,'S82854Q'
,'S82854R'
,'S82854S'
,'S82855C'
,'S82855D'
,'S82855E'
,'S82855F'
,'S82855G'
,'S82855H'
,'S82855J'
,'S82855K'
,'S82855M'
,'S82855N'
,'S82855P'
,'S82855Q'
,'S82855R'
,'S82855S'
,'S82856C'
,'S82856D'
,'S82856E'
,'S82856F'
,'S82856G'
,'S82856H'
,'S82856J'
,'S82856K'
,'S82856M'
,'S82856N'
,'S82856P'
,'S82856Q'
,'S82856R'
,'S82856S'
,'S82861C'
,'S82861D'
,'S82861E'
,'S82861F'
,'S82861G'
,'S82861H'
,'S82861J'
,'S82861K'
,'S82861M'
,'S82861N'
,'S82861P'
,'S82861Q'
,'S82861R'
,'S82861S'
,'S82862C'
,'S82862D'
,'S82862E'
,'S82862F'
,'S82862G'
,'S82862H'
,'S82862J'
,'S82862K'
,'S82862M'
,'S82862N'
,'S82862P'
,'S82862Q'
,'S82862R'
,'S82862S'
,'S82863C'
,'S82863D'
,'S82863E'
,'S82863F'
,'S82863G'
,'S82863H'
,'S82863J'
,'S82863K'
,'S82863M'
,'S82863N'
,'S82863P'
,'S82863Q'
,'S82863R'
,'S82863S'
,'S82864C'
,'S82864D'
,'S82864E'
,'S82864F'
,'S82864G'
,'S82864H'
,'S82864J'
,'S82864K'
,'S82864M'
,'S82864N'
,'S82864P'
,'S82864Q'
,'S82864R'
,'S82864S'
,'S82865C'
,'S82865D'
,'S82865E'
,'S82865F'
,'S82865G'
,'S82865H'
,'S82865J'
,'S82865K'
,'S82865M'
,'S82865N'
,'S82865P'
,'S82865Q'
,'S82865R'
,'S82865S'
,'S82866C'
,'S82866D'
,'S82866E'
,'S82866F'
,'S82866G'
,'S82866H'
,'S82866J'
,'S82866K'
,'S82866M'
,'S82866N'
,'S82866P'
,'S82866Q'
,'S82866R'
,'S82866S'
,'S82871C'
,'S82871D'
,'S82871E'
,'S82871F'
,'S82871G'
,'S82871H'
,'S82871J'
,'S82871K'
,'S82871M'
,'S82871N'
,'S82871P'
,'S82871Q'
,'S82871R'
,'S82871S'
,'S82872C'
,'S82872D'
,'S82872E'
,'S82872F'
,'S82872G'
,'S82872H'
,'S82872J'
,'S82872K'
,'S82872M'
,'S82872N'
,'S82872P'
,'S82872Q'
,'S82872R'
,'S82872S'
,'S82873C'
,'S82873D'
,'S82873E'
,'S82873F'
,'S82873G'
,'S82873H'
,'S82873J'
,'S82873K'
,'S82873M'
,'S82873N'
,'S82873P'
,'S82873Q'
,'S82873R'
,'S82873S'
,'S82874C'
,'S82874D'
,'S82874E'
,'S82874F'
,'S82874G'
,'S82874H'
,'S82874J'
,'S82874K'
,'S82874M'
,'S82874N'
,'S82874P'
,'S82874Q'
,'S82874R'
,'S82874S'
,'S82875C'
,'S82875D'
,'S82875E'
,'S82875F'
,'S82875G'
,'S82875H'
,'S82875J'
,'S82875K'
,'S82875M'
,'S82875N'
,'S82875P'
,'S82875Q'
,'S82875R'
,'S82875S'
,'S82876C'
,'S82876D'
,'S82876E'
,'S82876F'
,'S82876G'
,'S82876H'
,'S82876J'
,'S82876K'
,'S82876M'
,'S82876N'
,'S82876P'
,'S82876Q'
,'S82876R'
,'S82876S'
,'S82891C'
,'S82891D'
,'S82891E'
,'S82891F'
,'S82891G'
,'S82891H'
,'S82891J'
,'S82891K'
,'S82891M'
,'S82891N'
,'S82891P'
,'S82891Q'
,'S82891R'
,'S82891S'
,'S82892C'
,'S82892D'
,'S82892E'
,'S82892F'
,'S82892G'
,'S82892H'
,'S82892J'
,'S82892K'
,'S82892M'
,'S82892N'
,'S82892P'
,'S82892Q'
,'S82892R'
,'S82892S'
,'S82899C'
,'S82899D'
,'S82899E'
,'S82899F'
,'S82899G'
,'S82899H'
,'S82899J'
,'S82899K'
,'S82899M'
,'S82899N'
,'S82899P'
,'S82899Q'
,'S82899R'
,'S82899S'
,'S8290XC'
,'S8290XD'
,'S8290XE'
,'S8290XF'
,'S8290XG'
,'S8290XH'
,'S8290XJ'
,'S8290XK'
,'S8290XM'
,'S8290XN'
,'S8290XP'
,'S8290XQ'
,'S8290XR'
,'S8290XS'
,'S8291XC'
,'S8291XD'
,'S8291XE'
,'S8291XF'
,'S8291XG'
,'S8291XH'
,'S8291XJ'
,'S8291XK'
,'S8291XM'
,'S8291XN'
,'S8291XP'
,'S8291XQ'
,'S8291XR'
,'S8291XS'
,'S8292XC'
,'S8292XD'
,'S8292XE'
,'S8292XF'
,'S8292XG'
,'S8292XH'
,'S8292XJ'
,'S8292XK'
,'S8292XM'
,'S8292XN'
,'S8292XP'
,'S8292XQ'
,'S8292XR'
,'S8292XS'
,'S83001D'
,'S83001S'
,'S83002D'
,'S83002S'
,'S83003D'
,'S83003S'
,'S83004D'
,'S83004S'
,'S83005D'
,'S83005S'
,'S83006D'
,'S83006S'
,'S83011D'
,'S83011S'
,'S83012D'
,'S83012S'
,'S83013D'
,'S83013S'
,'S83014D'
,'S83014S'
,'S83015D'
,'S83015S'
,'S83016D'
,'S83016S'
,'S83091D'
,'S83091S'
,'S83092D'
,'S83092S'
,'S83093D'
,'S83093S'
,'S83094D'
,'S83094S'
,'S83095D'
,'S83095S'
,'S83096D'
,'S83096S'
,'S83101D'
,'S83101S'
,'S83102D'
,'S83102S'
,'S83103D'
,'S83103S'
,'S83104D'
,'S83104S'
,'S83105D'
,'S83105S'
,'S83106D'
,'S83106S'
,'S83111D'
,'S83111S'
,'S83112D'
,'S83112S'
,'S83113D'
,'S83113S'
,'S83114D'
,'S83114S'
,'S83115D'
,'S83115S'
,'S83116D'
,'S83116S'
,'S83121D'
,'S83121S'
,'S83122D'
,'S83122S'
,'S83123D'
,'S83123S'
,'S83124D'
,'S83124S'
,'S83125D'
,'S83125S'
,'S83126D'
,'S83126S'
,'S83131D'
,'S83131S'
,'S83132D'
,'S83132S'
,'S83133D'
,'S83133S'
,'S83134D'
,'S83134S'
,'S83135D'
,'S83135S'
,'S83136D'
,'S83136S'
,'S83141D'
,'S83141S'
,'S83142D'
,'S83142S'
,'S83143D'
,'S83143S'
,'S83144D'
,'S83144S'
,'S83145D'
,'S83145S'
,'S83146D'
,'S83146S'
,'S83191D'
,'S83191S'
,'S83192D'
,'S83192S'
,'S83193D'
,'S83193S'
,'S83194D'
,'S83194S'
,'S83195D'
,'S83195S'
,'S83196D'
,'S83196S'
,'S83200D'
,'S83200S'
,'S83201D'
,'S83201S'
,'S83202D'
,'S83202S'
,'S83203D'
,'S83203S'
,'S83204D'
,'S83204S'
,'S83205D'
,'S83205S'
,'S83206D'
,'S83206S'
,'S83207D'
,'S83207S'
,'S83209D'
,'S83209S'
,'S83211D'
,'S83211S'
,'S83212D'
,'S83212S'
,'S83219D'
,'S83219S'
,'S83221D'
,'S83221S'
,'S83222D'
,'S83222S'
,'S83229D'
,'S83229S'
,'S83231D'
,'S83231S'
,'S83232D'
,'S83232S'
,'S83239D'
,'S83239S'
,'S83241D'
,'S83241S'
,'S83242D'
,'S83242S'
,'S83249D'
,'S83249S'
,'S83251D'
,'S83251S'
,'S83252D'
,'S83252S'
,'S83259D'
,'S83259S'
,'S83261D'
,'S83261S'
,'S83262D'
,'S83262S'
,'S83269D'
,'S83269S'
,'S83271D'
,'S83271S'
,'S83272D'
,'S83272S'
,'S83279D'
,'S83279S'
,'S83281D'
,'S83281S'
,'S83282D'
,'S83282S'
,'S83289D'
,'S83289S'
,'S8330XD'
,'S8330XS'
,'S8331XD'
,'S8331XS'
,'S8332XD'
,'S8332XS'
,'S83401D'
,'S83401S'
,'S83402D'
,'S83402S'
,'S83409D'
,'S83409S'
,'S83411D'
,'S83411S'
,'S83412D'
,'S83412S'
,'S83419D'
,'S83419S'
,'S83421D'
,'S83421S'
,'S83422D'
,'S83422S'
,'S83429D'
,'S83429S'
,'S83501D'
,'S83501S'
,'S83502D'
,'S83502S'
,'S83509D'
,'S83509S'
,'S83511D'
,'S83511S'
,'S83512D'
,'S83512S'
,'S83519D'
,'S83519S'
,'S83521D'
,'S83521S'
,'S83522D'
,'S83522S'
,'S83529D'
,'S83529S'
,'S8360XD'
,'S8360XS'
,'S8361XD'
,'S8361XS'
,'S8362XD'
,'S8362XS'
,'S838X1D'
,'S838X1S'
,'S838X2D'
,'S838X2S'
,'S838X9D'
,'S838X9S'
,'S8390XD'
,'S8390XS'
,'S8391XD'
,'S8391XS'
,'S8392XD'
,'S8392XS'
,'S8400XD'
,'S8400XS'
,'S8401XD'
,'S8401XS'
,'S8402XD'
,'S8402XS'
,'S8410XD'
,'S8410XS'
,'S8411XD'
,'S8411XS'
,'S8412XD'
,'S8412XS'
,'S8420XD'
,'S8420XS'
,'S8421XD'
,'S8421XS'
,'S8422XD'
,'S8422XS'
,'S84801D'
,'S84801S'
,'S84802D'
,'S84802S'
,'S84809D'
,'S84809S'
,'S8490XD'
,'S8490XS'
,'S8491XD'
,'S8491XS'
,'S8492XD'
,'S8492XS'
,'S85001D'
,'S85001S'
,'S85002D'
,'S85002S'
,'S85009D'
,'S85009S'
,'S85011D'
,'S85011S'
,'S85012D'
,'S85012S'
,'S85019D'
,'S85019S'
,'S85091D'
,'S85091S'
,'S85092D'
,'S85092S'
,'S85099D'
,'S85099S'
,'S85101D'
,'S85101S'
,'S85102D'
,'S85102S'
,'S85109D'
,'S85109S'
,'S85111D'
,'S85111S'
,'S85112D'
,'S85112S'
,'S85119D'
,'S85119S'
,'S85121D'
,'S85121S'
,'S85122D'
,'S85122S'
,'S85129D'
,'S85129S'
,'S85131D'
,'S85131S'
,'S85132D'
,'S85132S'
,'S85139D'
,'S85139S'
,'S85141D'
,'S85141S'
,'S85142D'
,'S85142S'
,'S85149D'
,'S85149S'
,'S85151D'
,'S85151S'
,'S85152D'
,'S85152S'
,'S85159D'
,'S85159S'
,'S85161D'
,'S85161S'
,'S85162D'
,'S85162S'
,'S85169D'
,'S85169S'
,'S85171D'
,'S85171S'
,'S85172D'
,'S85172S'
,'S85179D'
,'S85179S'
,'S85181D'
,'S85181S'
,'S85182D'
,'S85182S'
,'S85189D'
,'S85189S'
,'S85201D'
,'S85201S'
,'S85202D'
,'S85202S'
,'S85209D'
,'S85209S'
,'S85211D'
,'S85211S'
,'S85212D'
,'S85212S'
,'S85219D'
,'S85219S'
,'S85291D'
,'S85291S'
,'S85292D'
,'S85292S'
,'S85299D'
,'S85299S'
,'S85301D'
,'S85301S'
,'S85302D'
,'S85302S'
,'S85309D'
,'S85309S'
,'S85311D'
,'S85311S'
,'S85312D'
,'S85312S'
,'S85319D'
,'S85319S'
,'S85391D'
,'S85391S'
,'S85392D'
,'S85392S'
,'S85399D'
,'S85399S'
,'S85401D'
,'S85401S'
,'S85402D'
,'S85402S'
,'S85409D'
,'S85409S'
,'S85411D'
,'S85411S'
,'S85412D'
,'S85412S'
,'S85419D'
,'S85419S'
,'S85491D'
,'S85491S'
,'S85492D'
,'S85492S'
,'S85499D'
,'S85499S'
,'S85501D'
,'S85501S'
,'S85502D'
,'S85502S'
,'S85509D'
,'S85509S'
,'S85511D'
,'S85511S'
,'S85512D'
,'S85512S'
,'S85519D'
,'S85519S'
,'S85591D'
,'S85591S'
,'S85592D'
,'S85592S'
,'S85599D'
,'S85599S'
,'S85801D'
,'S85801S'
,'S85802D'
,'S85802S'
,'S85809D'
,'S85809S'
,'S85811D'
,'S85811S'
,'S85812D'
,'S85812S'
,'S85819D'
,'S85819S'
,'S85891D'
,'S85891S'
,'S85892D'
,'S85892S'
,'S85899D'
,'S85899S'
,'S85901D'
,'S85901S'
,'S85902D'
,'S85902S'
,'S85909D'
,'S85909S'
,'S85911D'
,'S85911S'
,'S85912D'
,'S85912S'
,'S85919D'
,'S85919S'
,'S85991D'
,'S85991S'
,'S85992D'
,'S85992S'
,'S85999D'
,'S85999S'
,'S86001D'
,'S86001S'
,'S86002D'
,'S86002S'
,'S86009D'
,'S86009S'
,'S86011D'
,'S86011S'
,'S86012D'
,'S86012S'
,'S86019D'
,'S86019S'
,'S86021D'
,'S86021S'
,'S86022D'
,'S86022S'
,'S86029D'
,'S86029S'
,'S86091D'
,'S86091S'
,'S86092D'
,'S86092S'
,'S86099D'
,'S86099S'
,'S86101D'
,'S86101S'
,'S86102D'
,'S86102S'
,'S86109D'
,'S86109S'
,'S86111D'
,'S86111S'
,'S86112D'
,'S86112S'
,'S86119D'
,'S86119S'
,'S86121D'
,'S86121S'
,'S86122D'
,'S86122S'
,'S86129D'
,'S86129S'
,'S86191D'
,'S86191S'
,'S86192D'
,'S86192S'
,'S86199D'
,'S86199S'
,'S86201D'
,'S86201S'
,'S86202D'
,'S86202S'
,'S86209D'
,'S86209S'
,'S86211D'
,'S86211S'
,'S86212D'
,'S86212S'
,'S86219D'
,'S86219S'
,'S86221D'
,'S86221S'
,'S86222D'
,'S86222S'
,'S86229D'
,'S86229S'
,'S86291D'
,'S86291S'
,'S86292D'
,'S86292S'
,'S86299D'
,'S86299S'
,'S86301D'
,'S86301S'
,'S86302D'
,'S86302S'
,'S86309D'
,'S86309S'
,'S86311D'
,'S86311S'
,'S86312D'
,'S86312S'
,'S86319D'
,'S86319S'
,'S86321D'
,'S86321S'
,'S86322D'
,'S86322S'
,'S86329D'
,'S86329S'
,'S86391D'
,'S86391S'
,'S86392D'
,'S86392S'
,'S86399D'
,'S86399S'
,'S86801D'
,'S86801S'
,'S86802D'
,'S86802S'
,'S86809D'
,'S86809S'
,'S86811D'
,'S86811S'
,'S86812D'
,'S86812S'
,'S86819D'
,'S86819S'
,'S86821D'
,'S86821S'
,'S86822D'
,'S86822S'
,'S86829D'
,'S86829S'
,'S86891D'
,'S86891S'
,'S86892D'
,'S86892S'
,'S86899D'
,'S86899S'
,'S86901D'
,'S86901S'
,'S86902D'
,'S86902S'
,'S86909D'
,'S86909S'
,'S86911D'
,'S86911S'
,'S86912D'
,'S86912S'
,'S86919D'
,'S86919S'
,'S86921D'
,'S86921S'
,'S86922D'
,'S86922S'
,'S86929D'
,'S86929S'
,'S86991D'
,'S86991S'
,'S86992D'
,'S86992S'
,'S86999D'
,'S86999S'
,'S8700XD'
,'S8700XS'
,'S8701XD'
,'S8701XS'
,'S8702XD'
,'S8702XS'
,'S8780XD'
,'S8780XS'
,'S8781XD'
,'S8781XS'
,'S8782XD'
,'S8782XS'
,'S88011D'
,'S88011S'
,'S88012D'
,'S88012S'
,'S88019D'
,'S88019S'
,'S88021D'
,'S88021S'
,'S88022D'
,'S88022S'
,'S88029D'
,'S88029S'
,'S88111D'
,'S88111S'
,'S88112D'
,'S88112S'
,'S88119D'
,'S88119S'
,'S88121D'
,'S88121S'
,'S88122D'
,'S88122S'
,'S88129D'
,'S88129S'
,'S88911D'
,'S88911S'
,'S88912D'
,'S88912S'
,'S88919D'
,'S88919S'
,'S88921D'
,'S88921S'
,'S88922D'
,'S88922S'
,'S88929D'
,'S88929S'
,'S89001D'
,'S89001G'
,'S89001K'
,'S89001P'
,'S89001S'
,'S89002D'
,'S89002G'
,'S89002K'
,'S89002P'
,'S89002S'
,'S89009D'
,'S89009G'
,'S89009K'
,'S89009P'
,'S89009S'
,'S89011D'
,'S89011G'
,'S89011K'
,'S89011P'
,'S89011S'
,'S89012D'
,'S89012G'
,'S89012K'
,'S89012P'
,'S89012S'
,'S89019D'
,'S89019G'
,'S89019K'
,'S89019P'
,'S89019S'
,'S89021D'
,'S89021G'
,'S89021K'
,'S89021P'
,'S89021S'
,'S89022D'
,'S89022G'
,'S89022K'
,'S89022P'
,'S89022S'
,'S89029D'
,'S89029G'
,'S89029K'
,'S89029P'
,'S89029S'
,'S89031D'
,'S89031G'
,'S89031K'
,'S89031P'
,'S89031S'
,'S89032D'
,'S89032G'
,'S89032K'
,'S89032P'
,'S89032S'
,'S89039D'
,'S89039G'
,'S89039K'
,'S89039P'
,'S89039S'
,'S89041D'
,'S89041G'
,'S89041K'
,'S89041P'
,'S89041S'
,'S89042D'
,'S89042G'
,'S89042K'
,'S89042P'
,'S89042S'
,'S89049D'
,'S89049G'
,'S89049K'
,'S89049P'
,'S89049S'
,'S89091D'
,'S89091G'
,'S89091K'
,'S89091P'
,'S89091S'
,'S89092D'
,'S89092G'
,'S89092K'
,'S89092P'
,'S89092S'
,'S89099D'
,'S89099G'
,'S89099K'
,'S89099P'
,'S89099S'
,'S89101D'
,'S89101G'
,'S89101K'
,'S89101P'
,'S89101S'
,'S89102D'
,'S89102G'
,'S89102K'
,'S89102P'
,'S89102S'
,'S89109D'
,'S89109G'
,'S89109K'
,'S89109P'
,'S89109S'
,'S89111D'
,'S89111G'
,'S89111K'
,'S89111P'
,'S89111S'
,'S89112D'
,'S89112G'
,'S89112K'
,'S89112P'
,'S89112S'
,'S89119D'
,'S89119G'
,'S89119K'
,'S89119P'
,'S89119S'
,'S89121D'
,'S89121G'
,'S89121K'
,'S89121P'
,'S89121S'
,'S89122D'
,'S89122G'
,'S89122K'
,'S89122P'
,'S89122S'
,'S89129D'
,'S89129G'
,'S89129K'
,'S89129P'
,'S89129S'
,'S89131D'
,'S89131G'
,'S89131K'
,'S89131P'
,'S89131S'
,'S89132D'
,'S89132G'
,'S89132K'
,'S89132P'
,'S89132S'
,'S89139D'
,'S89139G'
,'S89139K'
,'S89139P'
,'S89139S'
,'S89141D'
,'S89141G'
,'S89141K'
,'S89141P'
,'S89141S'
,'S89142D'
,'S89142G'
,'S89142K'
,'S89142P'
,'S89142S'
,'S89149D'
,'S89149G'
,'S89149K'
,'S89149P'
,'S89149S'
,'S89191D'
,'S89191G'
,'S89191K'
,'S89191P'
,'S89191S'
,'S89192D'
,'S89192G'
,'S89192K'
,'S89192P'
,'S89192S'
,'S89199D'
,'S89199G'
,'S89199K'
,'S89199P'
,'S89199S'
,'S89201D'
,'S89201G'
,'S89201K'
,'S89201P'
,'S89201S'
,'S89202D'
,'S89202G'
,'S89202K'
,'S89202P'
,'S89202S'
,'S89209D'
,'S89209G'
,'S89209K'
,'S89209P'
,'S89209S'
,'S89211D'
,'S89211G'
,'S89211K'
,'S89211P'
,'S89211S'
,'S89212D'
,'S89212G'
,'S89212K'
,'S89212P'
,'S89212S'
,'S89219D'
,'S89219G'
,'S89219K'
,'S89219P'
,'S89219S'
,'S89221D'
,'S89221G'
,'S89221K'
,'S89221P'
,'S89221S'
,'S89222D'
,'S89222G'
,'S89222K'
,'S89222P'
,'S89222S'
,'S89229D'
,'S89229G'
,'S89229K'
,'S89229P'
,'S89229S'
,'S89291D'
,'S89291G'
,'S89291K'
,'S89291P'
,'S89291S'
,'S89292D'
,'S89292G'
,'S89292K'
,'S89292P'
,'S89292S'
,'S89299D'
,'S89299G'
,'S89299K'
,'S89299P'
,'S89299S'
,'S89301D'
,'S89301G'
,'S89301K'
,'S89301P'
,'S89301S'
,'S89302D'
,'S89302G'
,'S89302K'
,'S89302P'
,'S89302S'
,'S89309D'
,'S89309G'
,'S89309K'
,'S89309P'
,'S89309S'
,'S89311D'
,'S89311G'
,'S89311K'
,'S89311P'
,'S89311S'
,'S89312D'
,'S89312G'
,'S89312K'
,'S89312P'
,'S89312S'
,'S89319D'
,'S89319G'
,'S89319K'
,'S89319P'
,'S89319S'
,'S89321D'
,'S89321G'
,'S89321K'
,'S89321P'
,'S89321S'
,'S89322D'
,'S89322G'
,'S89322K'
,'S89322P'
,'S89322S'
,'S89329D'
,'S89329G'
,'S89329K'
,'S89329P'
,'S89329S'
,'S89391D'
,'S89391G'
,'S89391K'
,'S89391P'
,'S89391S'
,'S89392D'
,'S89392G'
,'S89392K'
,'S89392P'
,'S89392S'
,'S89399D'
,'S89399G'
,'S89399K'
,'S89399P'
,'S89399S'
,'S8980XD'
,'S8980XS'
,'S8981XD'
,'S8981XS'
,'S8982XD'
,'S8982XS'
,'S8990XD'
,'S8990XS'
,'S8991XD'
,'S8991XS'
,'S8992XD'
,'S8992XS'
,'S9000XD'
,'S9000XS'
,'S9001XD'
,'S9001XS'
,'S9002XD'
,'S9002XS'
,'S90111D'
,'S90111S'
,'S90112D'
,'S90112S'
,'S90119D'
,'S90119S'
,'S90121D'
,'S90121S'
,'S90122D'
,'S90122S'
,'S90129D'
,'S90129S'
,'S90211D'
,'S90211S'
,'S90212D'
,'S90212S'
,'S90219D'
,'S90219S'
,'S90221D'
,'S90221S'
,'S90222D'
,'S90222S'
,'S90229D'
,'S90229S'
,'S9030XD'
,'S9030XS'
,'S9031XD'
,'S9031XS'
,'S9032XD'
,'S9032XS'
,'S90411D'
,'S90411S'
,'S90412D'
,'S90412S'
,'S90413D'
,'S90413S'
,'S90414D'
,'S90414S'
,'S90415D'
,'S90415S'
,'S90416D'
,'S90416S'
,'S90421D'
,'S90421S'
,'S90422D'
,'S90422S'
,'S90423D'
,'S90423S'
,'S90424D'
,'S90424S'
,'S90425D'
,'S90425S'
,'S90426D'
,'S90426S'
,'S90441D'
,'S90441S'
,'S90442D'
,'S90442S'
,'S90443D'
,'S90443S'
,'S90444D'
,'S90444S'
,'S90445D'
,'S90445S'
,'S90446D'
,'S90446S'
,'S90451D'
,'S90451S'
,'S90452D'
,'S90452S'
,'S90453D'
,'S90453S'
,'S90454D'
,'S90454S'
,'S90455D'
,'S90455S'
,'S90456D'
,'S90456S'
,'S90461D'
,'S90461S'
,'S90462D'
,'S90462S'
,'S90463D'
,'S90463S'
,'S90464D'
,'S90464S'
,'S90465D'
,'S90465S'
,'S90466D'
,'S90466S'
,'S90471D'
,'S90471S'
,'S90472D'
,'S90472S'
,'S90473D'
,'S90473S'
,'S90474D'
,'S90474S'
,'S90475D'
,'S90475S'
,'S90476D'
,'S90476S'
,'S90511D'
,'S90511S'
,'S90512D'
,'S90512S'
,'S90519D'
,'S90519S'
,'S90521D'
,'S90521S'
,'S90522D'
,'S90522S'
,'S90529D'
,'S90529S'
,'S90541D'
,'S90541S'
,'S90542D'
,'S90542S'
,'S90549D'
,'S90549S'
,'S90551D'
,'S90551S'
,'S90552D'
,'S90552S'
,'S90559D'
,'S90559S'
,'S90561D'
,'S90561S'
,'S90562D'
,'S90562S'
,'S90569D'
,'S90569S'
,'S90571D'
,'S90571S'
,'S90572D'
,'S90572S'
,'S90579D'
,'S90579S'
,'S90811D'
,'S90811S'
,'S90812D'
,'S90812S'
,'S90819D'
,'S90819S'
,'S90821D'
,'S90821S'
,'S90822D'
,'S90822S'
,'S90829D'
,'S90829S'
,'S90841D'
,'S90841S'
,'S90842D'
,'S90842S'
,'S90849D'
,'S90849S'
,'S90851D'
,'S90851S'
,'S90852D'
,'S90852S'
,'S90859D'
,'S90859S'
,'S90861D'
,'S90861S'
,'S90862D'
,'S90862S'
,'S90869D'
,'S90869S'
,'S90871D'
,'S90871S'
,'S90872D'
,'S90872S'
,'S90879D'
,'S90879S'
,'S90911D'
,'S90911S'
,'S90912D'
,'S90912S'
,'S90919D'
,'S90919S'
,'S90921D'
,'S90921S'
,'S90922D'
,'S90922S'
,'S90929D'
,'S90929S'
,'S90931D'
,'S90931S'
,'S90932D'
,'S90932S'
,'S90933D'
,'S90933S'
,'S90934D'
,'S90934S'
,'S90935D'
,'S90935S'
,'S90936D'
,'S90936S'
,'S91001D'
,'S91001S'
,'S91002D'
,'S91002S'
,'S91009D'
,'S91009S'
,'S91011D'
,'S91011S'
,'S91012D'
,'S91012S'
,'S91019D'
,'S91019S'
,'S91021D'
,'S91021S'
,'S91022D'
,'S91022S'
,'S91029D'
,'S91029S'
,'S91031D'
,'S91031S'
,'S91032D'
,'S91032S'
,'S91039D'
,'S91039S'
,'S91041D'
,'S91041S'
,'S91042D'
,'S91042S'
,'S91049D'
,'S91049S'
,'S91051D'
,'S91051S'
,'S91052D'
,'S91052S'
,'S91059D'
,'S91059S'
,'S91101D'
,'S91101S'
,'S91102D'
,'S91102S'
,'S91103D'
,'S91103S'
,'S91104D'
,'S91104S'
,'S91105D'
,'S91105S'
,'S91106D'
,'S91106S'
,'S91109D'
,'S91109S'
,'S91111D'
,'S91111S'
,'S91112D'
,'S91112S'
,'S91113D'
,'S91113S'
,'S91114D'
,'S91114S'
,'S91115D'
,'S91115S'
,'S91116D'
,'S91116S'
,'S91119D'
,'S91119S'
,'S91121D'
,'S91121S'
,'S91122D'
,'S91122S'
,'S91123D'
,'S91123S'
,'S91124D'
,'S91124S'
,'S91125D'
,'S91125S'
,'S91126D'
,'S91126S'
,'S91129D'
,'S91129S'
,'S91131D'
,'S91131S'
,'S91132D'
,'S91132S'
,'S91133D'
,'S91133S'
,'S91134D'
,'S91134S'
,'S91135D'
,'S91135S'
,'S91136D'
,'S91136S'
,'S91139D'
,'S91139S'
,'S91141D'
,'S91141S'
,'S91142D'
,'S91142S'
,'S91143D'
,'S91143S'
,'S91144D'
,'S91144S'
,'S91145D'
,'S91145S'
,'S91146D'
,'S91146S'
,'S91149D'
,'S91149S'
,'S91151D'
,'S91151S'
,'S91152D'
,'S91152S'
,'S91153D'
,'S91153S'
,'S91154D'
,'S91154S'
,'S91155D'
,'S91155S'
,'S91156D'
,'S91156S'
,'S91159D'
,'S91159S'
,'S91201D'
,'S91201S'
,'S91202D'
,'S91202S'
,'S91203D'
,'S91203S'
,'S91204D'
,'S91204S'
,'S91205D'
,'S91205S'
,'S91206D'
,'S91206S'
,'S91209D'
,'S91209S'
,'S91211D'
,'S91211S'
,'S91212D'
,'S91212S'
,'S91213D'
,'S91213S'
,'S91214D'
,'S91214S'
,'S91215D'
,'S91215S'
,'S91216D'
,'S91216S'
,'S91219D'
,'S91219S'
,'S91221D'
,'S91221S'
,'S91222D'
,'S91222S'
,'S91223D'
,'S91223S'
,'S91224D'
,'S91224S'
,'S91225D'
,'S91225S'
,'S91226D'
,'S91226S'
,'S91229D'
,'S91229S'
,'S91231D'
,'S91231S'
,'S91232D'
,'S91232S'
,'S91233D'
,'S91233S'
,'S91234D'
,'S91234S'
,'S91235D'
,'S91235S'
,'S91236D'
,'S91236S'
,'S91239D'
,'S91239S'
,'S91241D'
,'S91241S'
,'S91242D'
,'S91242S'
,'S91243D'
,'S91243S'
,'S91244D'
,'S91244S'
,'S91245D'
,'S91245S'
,'S91246D'
,'S91246S'
,'S91249D'
,'S91249S'
,'S91251D'
,'S91251S'
,'S91252D'
,'S91252S'
,'S91253D'
,'S91253S'
,'S91254D'
,'S91254S'
,'S91255D'
,'S91255S'
,'S91256D'
,'S91256S'
,'S91259D'
,'S91259S'
,'S91301D'
,'S91301S'
,'S91302D'
,'S91302S'
,'S91309D'
,'S91309S'
,'S91311D'
,'S91311S'
,'S91312D'
,'S91312S'
,'S91319D'
,'S91319S'
,'S91321D'
,'S91321S'
,'S91322D'
,'S91322S'
,'S91329D'
,'S91329S'
,'S91331D'
,'S91331S'
,'S91332D'
,'S91332S'
,'S91339D'
,'S91339S'
,'S91341D'
,'S91341S'
,'S91342D'
,'S91342S'
,'S91349D'
,'S91349S'
,'S91351D'
,'S91351S'
,'S91352D'
,'S91352S'
,'S91359D'
,'S91359S'
,'S92001D'
,'S92001G'
,'S92001K'
,'S92001P'
,'S92001S'
,'S92002D'
,'S92002G'
,'S92002K'
,'S92002P'
,'S92002S'
,'S92009D'
,'S92009G'
,'S92009K'
,'S92009P'
,'S92009S'
,'S92011D'
,'S92011G'
,'S92011K'
,'S92011P'
,'S92011S'
,'S92012D'
,'S92012G'
,'S92012K'
,'S92012P'
,'S92012S'
,'S92013D'
,'S92013G'
,'S92013K'
,'S92013P'
,'S92013S'
,'S92014D'
,'S92014G'
,'S92014K'
,'S92014P'
,'S92014S'
,'S92015D'
,'S92015G'
,'S92015K'
,'S92015P'
,'S92015S'
,'S92016D'
,'S92016G'
,'S92016K'
,'S92016P'
,'S92016S'
,'S92021D'
,'S92021G'
,'S92021K'
,'S92021P'
,'S92021S'
,'S92022D'
,'S92022G'
,'S92022K'
,'S92022P'
,'S92022S'
,'S92023D'
,'S92023G'
,'S92023K'
,'S92023P'
,'S92023S'
,'S92024D'
,'S92024G'
,'S92024K'
,'S92024P'
,'S92024S'
,'S92025D'
,'S92025G'
,'S92025K'
,'S92025P'
,'S92025S'
,'S92026D'
,'S92026G'
,'S92026K'
,'S92026P'
,'S92026S'
,'S92031D'
,'S92031G'
,'S92031K'
,'S92031P'
,'S92031S'
,'S92032D'
,'S92032G'
,'S92032K'
,'S92032P'
,'S92032S'
,'S92033D'
,'S92033G'
,'S92033K'
,'S92033P'
,'S92033S'
,'S92034D'
,'S92034G'
,'S92034K'
,'S92034P'
,'S92034S'
,'S92035D'
,'S92035G'
,'S92035K'
,'S92035P'
,'S92035S'
,'S92036D'
,'S92036G'
,'S92036K'
,'S92036P'
,'S92036S'
,'S92041D'
,'S92041G'
,'S92041K'
,'S92041P'
,'S92041S'
,'S92042D'
,'S92042G'
,'S92042K'
,'S92042P'
,'S92042S'
,'S92043D'
,'S92043G'
,'S92043K'
,'S92043P'
,'S92043S'
,'S92044D'
,'S92044G'
,'S92044K'
,'S92044P'
,'S92044S'
,'S92045D'
,'S92045G'
,'S92045K'
,'S92045P'
,'S92045S'
,'S92046D'
,'S92046G'
,'S92046K'
,'S92046P'
,'S92046S'
,'S92051D'
,'S92051G'
,'S92051K'
,'S92051P'
,'S92051S'
,'S92052D'
,'S92052G'
,'S92052K'
,'S92052P'
,'S92052S'
,'S92053D'
,'S92053G'
,'S92053K'
,'S92053P'
,'S92053S'
,'S92054D'
,'S92054G'
,'S92054K'
,'S92054P'
,'S92054S'
,'S92055D'
,'S92055G'
,'S92055K'
,'S92055P'
,'S92055S'
,'S92056D'
,'S92056G'
,'S92056K'
,'S92056P'
,'S92056S'
,'S92061D'
,'S92061G'
,'S92061K'
,'S92061P'
,'S92061S'
,'S92062D'
,'S92062G'
,'S92062K'
,'S92062P'
,'S92062S'
,'S92063D'
,'S92063G'
,'S92063K'
,'S92063P'
,'S92063S'
,'S92064D'
,'S92064G'
,'S92064K'
,'S92064P'
,'S92064S'
,'S92065D'
,'S92065G'
,'S92065K'
,'S92065P'
,'S92065S'
,'S92066D'
,'S92066G'
,'S92066K'
,'S92066P'
,'S92066S'
,'S92101D'
,'S92101G'
,'S92101K'
,'S92101P'
,'S92101S'
,'S92102D'
,'S92102G'
,'S92102K'
,'S92102P'
,'S92102S'
,'S92109D'
,'S92109G'
,'S92109K'
,'S92109P'
,'S92109S'
,'S92111D'
,'S92111G'
,'S92111K'
,'S92111P'
,'S92111S'
,'S92112D'
,'S92112G'
,'S92112K'
,'S92112P'
,'S92112S'
,'S92113D'
,'S92113G'
,'S92113K'
,'S92113P'
,'S92113S'
,'S92114D'
,'S92114G'
,'S92114K'
,'S92114P'
,'S92114S'
,'S92115D'
,'S92115G'
,'S92115K'
,'S92115P'
,'S92115S'
,'S92116D'
,'S92116G'
,'S92116K'
,'S92116P'
,'S92116S'
,'S92121D'
,'S92121G'
,'S92121K'
,'S92121P'
,'S92121S'
,'S92122D'
,'S92122G'
,'S92122K'
,'S92122P'
,'S92122S'
,'S92123D'
,'S92123G'
,'S92123K'
,'S92123P'
,'S92123S'
,'S92124D'
,'S92124G'
,'S92124K'
,'S92124P'
,'S92124S'
,'S92125D'
,'S92125G'
,'S92125K'
,'S92125P'
,'S92125S'
,'S92126D'
,'S92126G'
,'S92126K'
,'S92126P'
,'S92126S'
,'S92131D'
,'S92131G'
,'S92131K'
,'S92131P'
,'S92131S'
,'S92132D'
,'S92132G'
,'S92132K'
,'S92132P'
,'S92132S'
,'S92133D'
,'S92133G'
,'S92133K'
,'S92133P'
,'S92133S'
,'S92134D'
,'S92134G'
,'S92134K'
,'S92134P'
,'S92134S'
,'S92135D'
,'S92135G'
,'S92135K'
,'S92135P'
,'S92135S'
,'S92136D'
,'S92136G'
,'S92136K'
,'S92136P'
,'S92136S'
,'S92141D'
,'S92141G'
,'S92141K'
,'S92141P'
,'S92141S'
,'S92142D'
,'S92142G'
,'S92142K'
,'S92142P'
,'S92142S'
,'S92143D'
,'S92143G'
,'S92143K'
,'S92143P'
,'S92143S'
,'S92144D'
,'S92144G'
,'S92144K'
,'S92144P'
,'S92144S'
,'S92145D'
,'S92145G'
,'S92145K'
,'S92145P'
,'S92145S'
,'S92146D'
,'S92146G'
,'S92146K'
,'S92146P'
,'S92146S'
,'S92151D'
,'S92151G'
,'S92151K'
,'S92151P'
,'S92151S'
,'S92152D'
,'S92152G'
,'S92152K'
,'S92152P'
,'S92152S'
,'S92153D'
,'S92153G'
,'S92153K'
,'S92153P'
,'S92153S'
,'S92154D'
,'S92154G'
,'S92154K'
,'S92154P'
,'S92154S'
,'S92155D'
,'S92155G'
,'S92155K'
,'S92155P'
,'S92155S'
,'S92156D'
,'S92156G'
,'S92156K'
,'S92156P'
,'S92156S'
,'S92191D'
,'S92191G'
,'S92191K'
,'S92191P'
,'S92191S'
,'S92192D'
,'S92192G'
,'S92192K'
,'S92192P'
,'S92192S'
,'S92199D'
,'S92199G'
,'S92199K'
,'S92199P'
,'S92199S'
,'S92201D'
,'S92201G'
,'S92201K'
,'S92201P'
,'S92201S'
,'S92202D'
,'S92202G'
,'S92202K'
,'S92202P'
,'S92202S'
,'S92209D'
,'S92209G'
,'S92209K'
,'S92209P'
,'S92209S'
,'S92211D'
,'S92211G'
,'S92211K'
,'S92211P'
,'S92211S'
,'S92212D'
,'S92212G'
,'S92212K'
,'S92212P'
,'S92212S'
,'S92213D'
,'S92213G'
,'S92213K'
,'S92213P'
,'S92213S'
,'S92214D'
,'S92214G'
,'S92214K'
,'S92214P'
,'S92214S'
,'S92215D'
,'S92215G'
,'S92215K'
,'S92215P'
,'S92215S'
,'S92216D'
,'S92216G'
,'S92216K'
,'S92216P'
,'S92216S'
,'S92221D'
,'S92221G'
,'S92221K'
,'S92221P'
,'S92221S'
,'S92222D'
,'S92222G'
,'S92222K'
,'S92222P'
,'S92222S'
,'S92223D'
,'S92223G'
,'S92223K'
,'S92223P'
,'S92223S'
,'S92224D'
,'S92224G'
,'S92224K'
,'S92224P'
,'S92224S'
,'S92225D'
,'S92225G'
,'S92225K'
,'S92225P'
,'S92225S'
,'S92226D'
,'S92226G'
,'S92226K'
,'S92226P'
,'S92226S'
,'S92231D'
,'S92231G'
,'S92231K'
,'S92231P'
,'S92231S'
,'S92232D'
,'S92232G'
,'S92232K'
,'S92232P'
,'S92232S'
,'S92233D'
,'S92233G'
,'S92233K'
,'S92233P'
,'S92233S'
,'S92234D'
,'S92234G'
,'S92234K'
,'S92234P'
,'S92234S'
,'S92235D'
,'S92235G'
,'S92235K'
,'S92235P'
,'S92235S'
,'S92236D'
,'S92236G'
,'S92236K'
,'S92236P'
,'S92236S'
,'S92241D'
,'S92241G'
,'S92241K'
,'S92241P'
,'S92241S'
,'S92242D'
,'S92242G'
,'S92242K'
,'S92242P'
,'S92242S'
,'S92243D'
,'S92243G'
,'S92243K'
,'S92243P'
,'S92243S'
,'S92244D'
,'S92244G'
,'S92244K'
,'S92244P'
,'S92244S'
,'S92245D'
,'S92245G'
,'S92245K'
,'S92245P'
,'S92245S'
,'S92246D'
,'S92246G'
,'S92246K'
,'S92246P'
,'S92246S'
,'S92251D'
,'S92251G'
,'S92251K'
,'S92251P'
,'S92251S'
,'S92252D'
,'S92252G'
,'S92252K'
,'S92252P'
,'S92252S'
,'S92253D'
,'S92253G'
,'S92253K'
,'S92253P'
,'S92253S'
,'S92254D'
,'S92254G'
,'S92254K'
,'S92254P'
,'S92254S'
,'S92255D'
,'S92255G'
,'S92255K'
,'S92255P'
,'S92255S'
,'S92256D'
,'S92256G'
,'S92256K'
,'S92256P'
,'S92256S'
,'S92301D'
,'S92301G'
,'S92301K'
,'S92301P'
,'S92301S'
,'S92302D'
,'S92302G'
,'S92302K'
,'S92302P'
,'S92302S'
,'S92309D'
,'S92309G'
,'S92309K'
,'S92309P'
,'S92309S'
,'S92311D'
,'S92311G'
,'S92311K'
,'S92311P'
,'S92311S'
,'S92312D'
,'S92312G'
,'S92312K'
,'S92312P'
,'S92312S'
,'S92313D'
,'S92313G'
,'S92313K'
,'S92313P'
,'S92313S'
,'S92314D'
,'S92314G'
,'S92314K'
,'S92314P'
,'S92314S'
,'S92315D'
,'S92315G'
,'S92315K'
,'S92315P'
,'S92315S'
,'S92316D'
,'S92316G'
,'S92316K'
,'S92316P'
,'S92316S'
,'S92321D'
,'S92321G'
,'S92321K'
,'S92321P'
,'S92321S'
,'S92322D'
,'S92322G'
,'S92322K'
,'S92322P'
,'S92322S'
,'S92323D'
,'S92323G'
,'S92323K'
,'S92323P'
,'S92323S'
,'S92324D'
,'S92324G'
,'S92324K'
,'S92324P'
,'S92324S'
,'S92325D'
,'S92325G'
,'S92325K'
,'S92325P'
,'S92325S'
,'S92326D'
,'S92326G'
,'S92326K'
,'S92326P'
,'S92326S'
,'S92331D'
,'S92331G'
,'S92331K'
,'S92331P'
,'S92331S'
,'S92332D'
,'S92332G'
,'S92332K'
,'S92332P'
,'S92332S'
,'S92333D'
,'S92333G'
,'S92333K'
,'S92333P'
,'S92333S'
,'S92334D'
,'S92334G'
,'S92334K'
,'S92334P'
,'S92334S'
,'S92335D'
,'S92335G'
,'S92335K'
,'S92335P'
,'S92335S'
,'S92336D'
,'S92336G'
,'S92336K'
,'S92336P'
,'S92336S'
,'S92341D'
,'S92341G'
,'S92341K'
,'S92341P'
,'S92341S'
,'S92342D'
,'S92342G'
,'S92342K'
,'S92342P'
,'S92342S'
,'S92343D'
,'S92343G'
,'S92343K'
,'S92343P'
,'S92343S'
,'S92344D'
,'S92344G'
,'S92344K'
,'S92344P'
,'S92344S'
,'S92345D'
,'S92345G'
,'S92345K'
,'S92345P'
,'S92345S'
,'S92346D'
,'S92346G'
,'S92346K'
,'S92346P'
,'S92346S'
,'S92351D'
,'S92351G'
,'S92351K'
,'S92351P'
,'S92351S'
,'S92352D'
,'S92352G'
,'S92352K'
,'S92352P'
,'S92352S'
,'S92353D'
,'S92353G'
,'S92353K'
,'S92353P'
,'S92353S'
,'S92354D'
,'S92354G'
,'S92354K'
,'S92354P'
,'S92354S'
,'S92355D'
,'S92355G'
,'S92355K'
,'S92355P'
,'S92355S'
,'S92356D'
,'S92356G'
,'S92356K'
,'S92356P'
,'S92356S'
,'S92401D'
,'S92401G'
,'S92401K'
,'S92401P'
,'S92401S'
,'S92402D'
,'S92402G'
,'S92402K'
,'S92402P'
,'S92402S'
,'S92403D'
,'S92403G'
,'S92403K'
,'S92403P'
,'S92403S'
,'S92404D'
,'S92404G'
,'S92404K'
,'S92404P'
,'S92404S'
,'S92405D'
,'S92405G'
,'S92405K'
,'S92405P'
,'S92405S'
,'S92406D'
,'S92406G'
,'S92406K'
,'S92406P'
,'S92406S'
,'S92411D'
,'S92411G'
,'S92411K'
,'S92411P'
,'S92411S'
,'S92412D'
,'S92412G'
,'S92412K'
,'S92412P'
,'S92412S'
,'S92413D'
,'S92413G'
,'S92413K'
,'S92413P'
,'S92413S'
,'S92414D'
,'S92414G'
,'S92414K'
,'S92414P'
,'S92414S'
,'S92415D'
,'S92415G'
,'S92415K'
,'S92415P'
,'S92415S'
,'S92416D'
,'S92416G'
,'S92416K'
,'S92416P'
,'S92416S'
,'S92421D'
,'S92421G'
,'S92421K'
,'S92421P'
,'S92421S'
,'S92422D'
,'S92422G'
,'S92422K'
,'S92422P'
,'S92422S'
,'S92423D'
,'S92423G'
,'S92423K'
,'S92423P'
,'S92423S'
,'S92424D'
,'S92424G'
,'S92424K'
,'S92424P'
,'S92424S'
,'S92425D'
,'S92425G'
,'S92425K'
,'S92425P'
,'S92425S'
,'S92426D'
,'S92426G'
,'S92426K'
,'S92426P'
,'S92426S'
,'S92491D'
,'S92491G'
,'S92491K'
,'S92491P'
,'S92491S'
,'S92492D'
,'S92492G'
,'S92492K'
,'S92492P'
,'S92492S'
,'S92499D'
,'S92499G'
,'S92499K'
,'S92499P'
,'S92499S'
,'S92501D'
,'S92501G'
,'S92501K'
,'S92501P'
,'S92501S'
,'S92502D'
,'S92502G'
,'S92502K'
,'S92502P'
,'S92502S'
,'S92503D'
,'S92503G'
,'S92503K'
,'S92503P'
,'S92503S'
,'S92504D'
,'S92504G'
,'S92504K'
,'S92504P'
,'S92504S'
,'S92505D'
,'S92505G'
,'S92505K'
,'S92505P'
,'S92505S'
,'S92506D'
,'S92506G'
,'S92506K'
,'S92506P'
,'S92506S'
,'S92511D'
,'S92511G'
,'S92511K'
,'S92511P'
,'S92511S'
,'S92512D'
,'S92512G'
,'S92512K'
,'S92512P'
,'S92512S'
,'S92513D'
,'S92513G'
,'S92513K'
,'S92513P'
,'S92513S'
,'S92514D'
,'S92514G'
,'S92514K'
,'S92514P'
,'S92514S'
,'S92515D'
,'S92515G'
,'S92515K'
,'S92515P'
,'S92515S'
,'S92516D'
,'S92516G'
,'S92516K'
,'S92516P'
,'S92516S'
,'S92521D'
,'S92521G'
,'S92521K'
,'S92521P'
,'S92521S'
,'S92522D'
,'S92522G'
,'S92522K'
,'S92522P'
,'S92522S'
,'S92523D'
,'S92523G'
,'S92523K'
,'S92523P'
,'S92523S'
,'S92524D'
,'S92524G'
,'S92524K'
,'S92524P'
,'S92524S'
,'S92525D'
,'S92525G'
,'S92525K'
,'S92525P'
,'S92525S'
,'S92526D'
,'S92526G'
,'S92526K'
,'S92526P'
,'S92526S'
,'S92531D'
,'S92531G'
,'S92531K'
,'S92531P'
,'S92531S'
,'S92532D'
,'S92532G'
,'S92532K'
,'S92532P'
,'S92532S'
,'S92533D'
,'S92533G'
,'S92533K'
,'S92533P'
,'S92533S'
,'S92534D'
,'S92534G'
,'S92534K'
,'S92534P'
,'S92534S'
,'S92535D'
,'S92535G'
,'S92535K'
,'S92535P'
,'S92535S'
,'S92536D'
,'S92536G'
,'S92536K'
,'S92536P'
,'S92536S'
,'S92591D'
,'S92591G'
,'S92591K'
,'S92591P'
,'S92591S'
,'S92592D'
,'S92592G'
,'S92592K'
,'S92592P'
,'S92592S'
,'S92599D'
,'S92599G'
,'S92599K'
,'S92599P'
,'S92599S'
,'S92901D'
,'S92901G'
,'S92901K'
,'S92901P'
,'S92901S'
,'S92902D'
,'S92902G'
,'S92902K'
,'S92902P'
,'S92902S'
,'S92909D'
,'S92909G'
,'S92909K'
,'S92909P'
,'S92909S'
,'S92911D'
,'S92911G'
,'S92911K'
,'S92911P'
,'S92911S'
,'S92912D'
,'S92912G'
,'S92912K'
,'S92912P'
,'S92912S'
,'S92919D'
,'S92919G'
,'S92919K'
,'S92919P'
,'S92919S'
,'S9301XD'
,'S9301XS'
,'S9302XD'
,'S9302XS'
,'S9303XD'
,'S9303XS'
,'S9304XD'
,'S9304XS'
,'S9305XD'
,'S9305XS'
,'S9306XD'
,'S9306XS'
,'S93101D'
,'S93101S'
,'S93102D'
,'S93102S'
,'S93103D'
,'S93103S'
,'S93104D'
,'S93104S'
,'S93105D'
,'S93105S'
,'S93106D'
,'S93106S'
,'S93111D'
,'S93111S'
,'S93112D'
,'S93112S'
,'S93113D'
,'S93113S'
,'S93114D'
,'S93114S'
,'S93115D'
,'S93115S'
,'S93116D'
,'S93116S'
,'S93119D'
,'S93119S'
,'S93121D'
,'S93121S'
,'S93122D'
,'S93122S'
,'S93123D'
,'S93123S'
,'S93124D'
,'S93124S'
,'S93125D'
,'S93125S'
,'S93126D'
,'S93126S'
,'S93129D'
,'S93129S'
,'S93131D'
,'S93131S'
,'S93132D'
,'S93132S'
,'S93133D'
,'S93133S'
,'S93134D'
,'S93134S'
,'S93135D'
,'S93135S'
,'S93136D'
,'S93136S'
,'S93139D'
,'S93139S'
,'S93141D'
,'S93141S'
,'S93142D'
,'S93142S'
,'S93143D'
,'S93143S'
,'S93144D'
,'S93144S'
,'S93145D'
,'S93145S'
,'S93146D'
,'S93146S'
,'S93149D'
,'S93149S'
,'S93301D'
,'S93301S'
,'S93302D'
,'S93302S'
,'S93303D'
,'S93303S'
,'S93304D'
,'S93304S'
,'S93305D'
,'S93305S'
,'S93306D'
,'S93306S'
,'S93311D'
,'S93311S'
,'S93312D'
,'S93312S'
,'S93313D'
,'S93313S'
,'S93314D'
,'S93314S'
,'S93315D'
,'S93315S'
,'S93316D'
,'S93316S'
,'S93321D'
,'S93321S'
,'S93322D'
,'S93322S'
,'S93323D'
,'S93323S'
,'S93324D'
,'S93324S'
,'S93325D'
,'S93325S'
,'S93326D'
,'S93326S'
,'S93331D'
,'S93331S'
,'S93332D'
,'S93332S'
,'S93333D'
,'S93333S'
,'S93334D'
,'S93334S'
,'S93335D'
,'S93335S'
,'S93336D'
,'S93336S'
,'S93401D'
,'S93401S'
,'S93402D'
,'S93402S'
,'S93409D'
,'S93409S'
,'S93411D'
,'S93411S'
,'S93412D'
,'S93412S'
,'S93419D'
,'S93419S'
,'S93421D'
,'S93421S'
,'S93422D'
,'S93422S'
,'S93429D'
,'S93429S'
,'S93431D'
,'S93431S'
,'S93432D'
,'S93432S'
,'S93439D'
,'S93439S'
,'S93491D'
,'S93491S'
,'S93492D'
,'S93492S'
,'S93499D'
,'S93499S'
,'S93501D'
,'S93501S'
,'S93502D'
,'S93502S'
,'S93503D'
,'S93503S'
,'S93504D'
,'S93504S'
,'S93505D'
,'S93505S'
,'S93506D'
,'S93506S'
,'S93509D'
,'S93509S'
,'S93511D'
,'S93511S'
,'S93512D'
,'S93512S'
,'S93513D'
,'S93513S'
,'S93514D'
,'S93514S'
,'S93515D'
,'S93515S'
,'S93516D'
,'S93516S'
,'S93519D'
,'S93519S'
,'S93521D'
,'S93521S'
,'S93522D'
,'S93522S'
,'S93523D'
,'S93523S'
,'S93524D'
,'S93524S'
,'S93525D'
,'S93525S'
,'S93526D'
,'S93526S'
,'S93529D'
,'S93529S'
,'S93601D'
,'S93601S'
,'S93602D'
,'S93602S'
,'S93609D'
,'S93609S'
,'S93611D'
,'S93611S'
,'S93612D'
,'S93612S'
,'S93619D'
,'S93619S'
,'S93621D'
,'S93621S'
,'S93622D'
,'S93622S'
,'S93629D'
,'S93629S'
,'S93691D'
,'S93691S'
,'S93692D'
,'S93692S'
,'S93699D'
,'S93699S'
,'S9400XD'
,'S9400XS'
,'S9401XD'
,'S9401XS'
,'S9402XD'
,'S9402XS'
,'S9410XD'
,'S9410XS'
,'S9411XD'
,'S9411XS'
,'S9412XD'
,'S9412XS'
,'S9420XD'
,'S9420XS'
,'S9421XD'
,'S9421XS'
,'S9422XD'
,'S9422XS'
,'S9430XD'
,'S9430XS'
,'S9431XD'
,'S9431XS'
,'S9432XD'
,'S9432XS'
,'S948X1D'
,'S948X1S'
,'S948X2D'
,'S948X2S'
,'S948X9D'
,'S948X9S'
,'S9490XD'
,'S9490XS'
,'S9491XD'
,'S9491XS'
,'S9492XD'
,'S9492XS'
,'S95001D'
,'S95001S'
,'S95002D'
,'S95002S'
,'S95009D'
,'S95009S'
,'S95011D'
,'S95011S'
,'S95012D'
,'S95012S'
,'S95019D'
,'S95019S'
,'S95091D'
,'S95091S'
,'S95092D'
,'S95092S'
,'S95099D'
,'S95099S'
,'S95101D'
,'S95101S'
,'S95102D'
,'S95102S'
,'S95109D'
,'S95109S'
,'S95111D'
,'S95111S'
,'S95112D'
,'S95112S'
,'S95119D'
,'S95119S'
,'S95191D'
,'S95191S'
,'S95192D'
,'S95192S'
,'S95199D'
,'S95199S'
,'S95201D'
,'S95201S'
,'S95202D'
,'S95202S'
,'S95209D'
,'S95209S'
,'S95211D'
,'S95211S'
,'S95212D'
,'S95212S'
,'S95219D'
,'S95219S'
,'S95291D'
,'S95291S'
,'S95292D'
,'S95292S'
,'S95299D'
,'S95299S'
,'S95801D'
,'S95801S'
,'S95802D'
,'S95802S'
,'S95809D'
,'S95809S'
,'S95811D'
,'S95811S'
,'S95812D'
,'S95812S'
,'S95819D'
,'S95819S'
,'S95891D'
,'S95891S'
,'S95892D'
,'S95892S'
,'S95899D'
,'S95899S'
,'S95901D'
,'S95901S'
,'S95902D'
,'S95902S'
,'S95909D'
,'S95909S'
,'S95911D'
,'S95911S'
,'S95912D'
,'S95912S'
,'S95919D'
,'S95919S'
,'S95991D'
,'S95991S'
,'S95992D'
,'S95992S'
,'S95999D'
,'S95999S'
,'S96001D'
,'S96001S'
,'S96002D'
,'S96002S'
,'S96009D'
,'S96009S'
,'S96011D'
,'S96011S'
,'S96012D'
,'S96012S'
,'S96019D'
,'S96019S'
,'S96021D'
,'S96021S'
,'S96022D'
,'S96022S'
,'S96029D'
,'S96029S'
,'S96091D'
,'S96091S'
,'S96092D'
,'S96092S'
,'S96099D'
,'S96099S'
,'S96101D'
,'S96101S'
,'S96102D'
,'S96102S'
,'S96109D'
,'S96109S'
,'S96111D'
,'S96111S'
,'S96112D'
,'S96112S'
,'S96119D'
,'S96119S'
,'S96121D'
,'S96121S'
,'S96122D'
,'S96122S'
,'S96129D'
,'S96129S'
,'S96191D'
,'S96191S'
,'S96192D'
,'S96192S'
,'S96199D'
,'S96199S'
,'S96201D'
,'S96201S'
,'S96202D'
,'S96202S'
,'S96209D'
,'S96209S'
,'S96211D'
,'S96211S'
,'S96212D'
,'S96212S'
,'S96219D'
,'S96219S'
,'S96221D'
,'S96221S'
,'S96222D'
,'S96222S'
,'S96229D'
,'S96229S'
,'S96291D'
,'S96291S'
,'S96292D'
,'S96292S'
,'S96299D'
,'S96299S'
,'S96801D'
,'S96801S'
,'S96802D'
,'S96802S'
,'S96809D'
,'S96809S'
,'S96811D'
,'S96811S'
,'S96812D'
,'S96812S'
,'S96819D'
,'S96819S'
,'S96821D'
,'S96821S'
,'S96822D'
,'S96822S'
,'S96829D'
,'S96829S'
,'S96891D'
,'S96891S'
,'S96892D'
,'S96892S'
,'S96899D'
,'S96899S'
,'S96901D'
,'S96901S'
,'S96902D'
,'S96902S'
,'S96909D'
,'S96909S'
,'S96911D'
,'S96911S'
,'S96912D'
,'S96912S'
,'S96919D'
,'S96919S'
,'S96921D'
,'S96921S'
,'S96922D'
,'S96922S'
,'S96929D'
,'S96929S'
,'S96991D'
,'S96991S'
,'S96992D'
,'S96992S'
,'S96999D'
,'S96999S'
,'S9700XD'
,'S9700XS'
,'S9701XD'
,'S9701XS'
,'S9702XD'
,'S9702XS'
,'S97101D'
,'S97101S'
,'S97102D'
,'S97102S'
,'S97109D'
,'S97109S'
,'S97111D'
,'S97111S'
,'S97112D'
,'S97112S'
,'S97119D'
,'S97119S'
,'S97121D'
,'S97121S'
,'S97122D'
,'S97122S'
,'S97129D'
,'S97129S'
,'S9780XD'
,'S9780XS'
,'S9781XD'
,'S9781XS'
,'S9782XD'
,'S9782XS'
,'S98011D'
,'S98011S'
,'S98012D'
,'S98012S'
,'S98019D'
,'S98019S'
,'S98021D'
,'S98021S'
,'S98022D'
,'S98022S'
,'S98029D'
,'S98029S'
,'S98111D'
,'S98111S'
,'S98112D'
,'S98112S'
,'S98119D'
,'S98119S'
,'S98121D'
,'S98121S'
,'S98122D'
,'S98122S'
,'S98129D'
,'S98129S'
,'S98131D'
,'S98131S'
,'S98132D'
,'S98132S'
,'S98139D'
,'S98139S'
,'S98141D'
,'S98141S'
,'S98142D'
,'S98142S'
,'S98149D'
,'S98149S'
,'S98211D'
,'S98211S'
,'S98212D'
,'S98212S'
,'S98219D'
,'S98219S'
,'S98221D'
,'S98221S'
,'S98222D'
,'S98222S'
,'S98229D'
,'S98229S'
,'S98311D'
,'S98311S'
,'S98312D'
,'S98312S'
,'S98319D'
,'S98319S'
,'S98321D'
,'S98321S'
,'S98322D'
,'S98322S'
,'S98329D'
,'S98329S'
,'S98911D'
,'S98911S'
,'S98912D'
,'S98912S'
,'S98919D'
,'S98919S'
,'S98921D'
,'S98921S'
,'S98922D'
,'S98922S'
,'S98929D'
,'S98929S'
,'S99811D'
,'S99811S'
,'S99812D'
,'S99812S'
,'S99819D'
,'S99819S'
,'S99821D'
,'S99821S'
,'S99822D'
,'S99822S'
,'S99829D'
,'S99829S'
,'S99911D'
,'S99911S'
,'S99912D'
,'S99912S'
,'S99919D'
,'S99919S'
,'S99921D'
,'S99921S'
,'S99922D'
,'S99922S'
,'S99929D'
,'S99929S'
,'T07'
,'T148'
,'T1490'
,'T1491'
,'T1500XD'
,'T1500XS'
,'T1501XD'
,'T1501XS'
,'T1502XD'
,'T1502XS'
,'T1510XD'
,'T1510XS'
,'T1511XD'
,'T1511XS'
,'T1512XD'
,'T1512XS'
,'T1580XD'
,'T1580XS'
,'T1581XD'
,'T1581XS'
,'T1582XD'
,'T1582XS'
,'T1590XD'
,'T1590XS'
,'T1591XD'
,'T1591XS'
,'T1592XD'
,'T1592XS'
,'T161XXD'
,'T161XXS'
,'T162XXD'
,'T162XXS'
,'T169XXD'
,'T169XXS'
,'T170XXD'
,'T170XXS'
,'T171XXD'
,'T171XXS'
,'T17200D'
,'T17200S'
,'T17208D'
,'T17208S'
,'T17210D'
,'T17210S'
,'T17218D'
,'T17218S'
,'T17220D'
,'T17220S'
,'T17228D'
,'T17228S'
,'T17290D'
,'T17290S'
,'T17298D'
,'T17298S'
,'T17300D'
,'T17300S'
,'T17308D'
,'T17308S'
,'T17310D'
,'T17310S'
,'T17318D'
,'T17318S'
,'T17320D'
,'T17320S'
,'T17328D'
,'T17328S'
,'T17390D'
,'T17390S'
,'T17398D'
,'T17398S'
,'T17400D'
,'T17400S'
,'T17408D'
,'T17408S'
,'T17410D'
,'T17410S'
,'T17418D'
,'T17418S'
,'T17420D'
,'T17420S'
,'T17428D'
,'T17428S'
,'T17490D'
,'T17490S'
,'T17498D'
,'T17498S'
,'T17500D'
,'T17500S'
,'T17508D'
,'T17508S'
,'T17510D'
,'T17510S'
,'T17518D'
,'T17518S'
,'T17520D'
,'T17520S'
,'T17528D'
,'T17528S'
,'T17590D'
,'T17590S'
,'T17598D'
,'T17598S'
,'T17800D'
,'T17800S'
,'T17808D'
,'T17808S'
,'T17810D'
,'T17810S'
,'T17818D'
,'T17818S'
,'T17820D'
,'T17820S'
,'T17828D'
,'T17828S'
,'T17890D'
,'T17890S'
,'T17898D'
,'T17898S'
,'T17900D'
,'T17900S'
,'T17908D'
,'T17908S'
,'T17910D'
,'T17910S'
,'T17918D'
,'T17918S'
,'T17920D'
,'T17920S'
,'T17928D'
,'T17928S'
,'T17990D'
,'T17990S'
,'T17998D'
,'T17998S'
,'T180XXD'
,'T180XXS'
,'T18100D'
,'T18100S'
,'T18108D'
,'T18108S'
,'T18110D'
,'T18110S'
,'T18118D'
,'T18118S'
,'T18120D'
,'T18120S'
,'T18128D'
,'T18128S'
,'T18190D'
,'T18190S'
,'T18198D'
,'T18198S'
,'T182XXD'
,'T182XXS'
,'T183XXD'
,'T183XXS'
,'T184XXD'
,'T184XXS'
,'T185XXD'
,'T185XXS'
,'T188XXD'
,'T188XXS'
,'T189XXD'
,'T189XXS'
,'T190XXD'
,'T190XXS'
,'T191XXD'
,'T191XXS'
,'T192XXD'
,'T192XXS'
,'T193XXD'
,'T193XXS'
,'T194XXD'
,'T194XXS'
,'T198XXD'
,'T198XXS'
,'T199XXD'
,'T199XXS'
,'T2000XD'
,'T2000XS'
,'T20011D'
,'T20011S'
,'T20012D'
,'T20012S'
,'T20019D'
,'T20019S'
,'T2002XD'
,'T2002XS'
,'T2003XD'
,'T2003XS'
,'T2004XD'
,'T2004XS'
,'T2005XD'
,'T2005XS'
,'T2006XD'
,'T2006XS'
,'T2007XD'
,'T2007XS'
,'T2009XD'
,'T2009XS'
,'T2010XD'
,'T2010XS'
,'T20111D'
,'T20111S'
,'T20112D'
,'T20112S'
,'T20119D'
,'T20119S'
,'T2012XD'
,'T2012XS'
,'T2013XD'
,'T2013XS'
,'T2014XD'
,'T2014XS'
,'T2015XD'
,'T2015XS'
,'T2016XD'
,'T2016XS'
,'T2017XD'
,'T2017XS'
,'T2019XD'
,'T2019XS'
,'T2020XD'
,'T2020XS'
,'T20211D'
,'T20211S'
,'T20212D'
,'T20212S'
,'T20219D'
,'T20219S'
,'T2022XD'
,'T2022XS'
,'T2023XD'
,'T2023XS'
,'T2024XD'
,'T2024XS'
,'T2025XD'
,'T2025XS'
,'T2026XD'
,'T2026XS'
,'T2027XD'
,'T2027XS'
,'T2029XD'
,'T2029XS'
,'T2030XD'
,'T2030XS'
,'T20311D'
,'T20311S'
,'T20312D'
,'T20312S'
,'T20319D'
,'T20319S'
,'T2032XD'
,'T2032XS'
,'T2033XD'
,'T2033XS'
,'T2034XD'
,'T2034XS'
,'T2035XD'
,'T2035XS'
,'T2036XD'
,'T2036XS'
,'T2037XD'
,'T2037XS'
,'T2039XD'
,'T2039XS'
,'T2040XD'
,'T2040XS'
,'T20411D'
,'T20411S'
,'T20412D'
,'T20412S'
,'T20419D'
,'T20419S'
,'T2042XD'
,'T2042XS'
,'T2043XD'
,'T2043XS'
,'T2044XD'
,'T2044XS'
,'T2045XD'
,'T2045XS'
,'T2046XD'
,'T2046XS'
,'T2047XD'
,'T2047XS'
,'T2049XD'
,'T2049XS'
,'T2050XD'
,'T2050XS'
,'T20511D'
,'T20511S'
,'T20512D'
,'T20512S'
,'T20519D'
,'T20519S'
,'T2052XD'
,'T2052XS'
,'T2053XD'
,'T2053XS'
,'T2054XD'
,'T2054XS'
,'T2055XD'
,'T2055XS'
,'T2056XD'
,'T2056XS'
,'T2057XD'
,'T2057XS'
,'T2059XD'
,'T2059XS'
,'T2060XD'
,'T2060XS'
,'T20611D'
,'T20611S'
,'T20612D'
,'T20612S'
,'T20619D'
,'T20619S'
,'T2062XD'
,'T2062XS'
,'T2063XD'
,'T2063XS'
,'T2064XD'
,'T2064XS'
,'T2065XD'
,'T2065XS'
,'T2066XD'
,'T2066XS'
,'T2067XD'
,'T2067XS'
,'T2069XD'
,'T2069XS'
,'T2070XD'
,'T2070XS'
,'T20711D'
,'T20711S'
,'T20712D'
,'T20712S'
,'T20719D'
,'T20719S'
,'T2072XD'
,'T2072XS'
,'T2073XD'
,'T2073XS'
,'T2074XD'
,'T2074XS'
,'T2075XD'
,'T2075XS'
,'T2076XD'
,'T2076XS'
,'T2077XD'
,'T2077XS'
,'T2079XD'
,'T2079XS'
,'T2100XD'
,'T2100XS'
,'T2101XD'
,'T2101XS'
,'T2102XD'
,'T2102XS'
,'T2103XD'
,'T2103XS'
,'T2104XD'
,'T2104XS'
,'T2105XD'
,'T2105XS'
,'T2106XD'
,'T2106XS'
,'T2107XD'
,'T2107XS'
,'T2109XD'
,'T2109XS'
,'T2110XD'
,'T2110XS'
,'T2111XD'
,'T2111XS'
,'T2112XD'
,'T2112XS'
,'T2113XD'
,'T2113XS'
,'T2114XD'
,'T2114XS'
,'T2115XD'
,'T2115XS'
,'T2116XD'
,'T2116XS'
,'T2117XD'
,'T2117XS'
,'T2119XD'
,'T2119XS'
,'T2120XD'
,'T2120XS'
,'T2121XD'
,'T2121XS'
,'T2122XD'
,'T2122XS'
,'T2123XD'
,'T2123XS'
,'T2124XD'
,'T2124XS'
,'T2125XD'
,'T2125XS'
,'T2126XD'
,'T2126XS'
,'T2127XD'
,'T2127XS'
,'T2129XD'
,'T2129XS'
,'T2130XD'
,'T2130XS'
,'T2131XD'
,'T2131XS'
,'T2132XD'
,'T2132XS'
,'T2133XD'
,'T2133XS'
,'T2134XD'
,'T2134XS'
,'T2135XD'
,'T2135XS'
,'T2136XD'
,'T2136XS'
,'T2137XD'
,'T2137XS'
,'T2139XD'
,'T2139XS'
,'T2140XD'
,'T2140XS'
,'T2141XD'
,'T2141XS'
,'T2142XD'
,'T2142XS'
,'T2143XD'
,'T2143XS'
,'T2144XD'
,'T2144XS'
,'T2145XD'
,'T2145XS'
,'T2146XD'
,'T2146XS'
,'T2147XD'
,'T2147XS'
,'T2149XD'
,'T2149XS'
,'T2150XD'
,'T2150XS'
,'T2151XD'
,'T2151XS'
,'T2152XD'
,'T2152XS'
,'T2153XD'
,'T2153XS'
,'T2154XD'
,'T2154XS'
,'T2155XD'
,'T2155XS'
,'T2156XD'
,'T2156XS'
,'T2157XD'
,'T2157XS'
,'T2159XD'
,'T2159XS'
,'T2160XD'
,'T2160XS'
,'T2161XD'
,'T2161XS'
,'T2162XD'
,'T2162XS'
,'T2163XD'
,'T2163XS'
,'T2164XD'
,'T2164XS'
,'T2165XD'
,'T2165XS'
,'T2166XD'
,'T2166XS'
,'T2167XD'
,'T2167XS'
,'T2169XD'
,'T2169XS'
,'T2170XD'
,'T2170XS'
,'T2171XD'
,'T2171XS'
,'T2172XD'
,'T2172XS'
,'T2173XD'
,'T2173XS'
,'T2174XD'
,'T2174XS'
,'T2175XD'
,'T2175XS'
,'T2176XD'
,'T2176XS'
,'T2177XD'
,'T2177XS'
,'T2179XD'
,'T2179XS'
,'T2200XD'
,'T2200XS'
,'T22011D'
,'T22011S'
,'T22012D'
,'T22012S'
,'T22019D'
,'T22019S'
,'T22021D'
,'T22021S'
,'T22022D'
,'T22022S'
,'T22029D'
,'T22029S'
,'T22031D'
,'T22031S'
,'T22032D'
,'T22032S'
,'T22039D'
,'T22039S'
,'T22041D'
,'T22041S'
,'T22042D'
,'T22042S'
,'T22049D'
,'T22049S'
,'T22051D'
,'T22051S'
,'T22052D'
,'T22052S'
,'T22059D'
,'T22059S'
,'T22061D'
,'T22061S'
,'T22062D'
,'T22062S'
,'T22069D'
,'T22069S'
,'T22091D'
,'T22091S'
,'T22092D'
,'T22092S'
,'T22099D'
,'T22099S'
,'T2210XD'
,'T2210XS'
,'T22111D'
,'T22111S'
,'T22112D'
,'T22112S'
,'T22119D'
,'T22119S'
,'T22121D'
,'T22121S'
,'T22122D'
,'T22122S'
,'T22129D'
,'T22129S'
,'T22131D'
,'T22131S'
,'T22132D'
,'T22132S'
,'T22139D'
,'T22139S'
,'T22141D'
,'T22141S'
,'T22142D'
,'T22142S'
,'T22149D'
,'T22149S'
,'T22151D'
,'T22151S'
,'T22152D'
,'T22152S'
,'T22159D'
,'T22159S'
,'T22161D'
,'T22161S'
,'T22162D'
,'T22162S'
,'T22169D'
,'T22169S'
,'T22191D'
,'T22191S'
,'T22192D'
,'T22192S'
,'T22199D'
,'T22199S'
,'T2220XD'
,'T2220XS'
,'T22211D'
,'T22211S'
,'T22212D'
,'T22212S'
,'T22219D'
,'T22219S'
,'T22221D'
,'T22221S'
,'T22222D'
,'T22222S'
,'T22229D'
,'T22229S'
,'T22231D'
,'T22231S'
,'T22232D'
,'T22232S'
,'T22239D'
,'T22239S'
,'T22241D'
,'T22241S'
,'T22242D'
,'T22242S'
,'T22249D'
,'T22249S'
,'T22251D'
,'T22251S'
,'T22252D'
,'T22252S'
,'T22259D'
,'T22259S'
,'T22261D'
,'T22261S'
,'T22262D'
,'T22262S'
,'T22269D'
,'T22269S'
,'T22291D'
,'T22291S'
,'T22292D'
,'T22292S'
,'T22299D'
,'T22299S'
,'T2230XD'
,'T2230XS'
,'T22311D'
,'T22311S'
,'T22312D'
,'T22312S'
,'T22319D'
,'T22319S'
,'T22321D'
,'T22321S'
,'T22322D'
,'T22322S'
,'T22329D'
,'T22329S'
,'T22331D'
,'T22331S'
,'T22332D'
,'T22332S'
,'T22339D'
,'T22339S'
,'T22341D'
,'T22341S'
,'T22342D'
,'T22342S'
,'T22349D'
,'T22349S'
,'T22351D'
,'T22351S'
,'T22352D'
,'T22352S'
,'T22359D'
,'T22359S'
,'T22361D'
,'T22361S'
,'T22362D'
,'T22362S'
,'T22369D'
,'T22369S'
,'T22391D'
,'T22391S'
,'T22392D'
,'T22392S'
,'T22399D'
,'T22399S'
,'T2240XD'
,'T2240XS'
,'T22411D'
,'T22411S'
,'T22412D'
,'T22412S'
,'T22419D'
,'T22419S'
,'T22421D'
,'T22421S'
,'T22422D'
,'T22422S'
,'T22429D'
,'T22429S'
,'T22431D'
,'T22431S'
,'T22432D'
,'T22432S'
,'T22439D'
,'T22439S'
,'T22441D'
,'T22441S'
,'T22442D'
,'T22442S'
,'T22449D'
,'T22449S'
,'T22451D'
,'T22451S'
,'T22452D'
,'T22452S'
,'T22459D'
,'T22459S'
,'T22461D'
,'T22461S'
,'T22462D'
,'T22462S'
,'T22469D'
,'T22469S'
,'T22491D'
,'T22491S'
,'T22492D'
,'T22492S'
,'T22499D'
,'T22499S'
,'T2250XD'
,'T2250XS'
,'T22511D'
,'T22511S'
,'T22512D'
,'T22512S'
,'T22519D'
,'T22519S'
,'T22521D'
,'T22521S'
,'T22522D'
,'T22522S'
,'T22529D'
,'T22529S'
,'T22531D'
,'T22531S'
,'T22532D'
,'T22532S'
,'T22539D'
,'T22539S'
,'T22541D'
,'T22541S'
,'T22542D'
,'T22542S'
,'T22549D'
,'T22549S'
,'T22551D'
,'T22551S'
,'T22552D'
,'T22552S'
,'T22559D'
,'T22559S'
,'T22561D'
,'T22561S'
,'T22562D'
,'T22562S'
,'T22569D'
,'T22569S'
,'T22591D'
,'T22591S'
,'T22592D'
,'T22592S'
,'T22599D'
,'T22599S'
,'T2260XD'
,'T2260XS'
,'T22611D'
,'T22611S'
,'T22612D'
,'T22612S'
,'T22619D'
,'T22619S'
,'T22621D'
,'T22621S'
,'T22622D'
,'T22622S'
,'T22629D'
,'T22629S'
,'T22631D'
,'T22631S'
,'T22632D'
,'T22632S'
,'T22639D'
,'T22639S'
,'T22641D'
,'T22641S'
,'T22642D'
,'T22642S'
,'T22649D'
,'T22649S'
,'T22651D'
,'T22651S'
,'T22652D'
,'T22652S'
,'T22659D'
,'T22659S'
,'T22661D'
,'T22661S'
,'T22662D'
,'T22662S'
,'T22669D'
,'T22669S'
,'T22691D'
,'T22691S'
,'T22692D'
,'T22692S'
,'T22699D'
,'T22699S'
,'T2270XD'
,'T2270XS'
,'T22711D'
,'T22711S'
,'T22712D'
,'T22712S'
,'T22719D'
,'T22719S'
,'T22721D'
,'T22721S'
,'T22722D'
,'T22722S'
,'T22729D'
,'T22729S'
,'T22731D'
,'T22731S'
,'T22732D'
,'T22732S'
,'T22739D'
,'T22739S'
,'T22741D'
,'T22741S'
,'T22742D'
,'T22742S'
,'T22749D'
,'T22749S'
,'T22751D'
,'T22751S'
,'T22752D'
,'T22752S'
,'T22759D'
,'T22759S'
,'T22761D'
,'T22761S'
,'T22762D'
,'T22762S'
,'T22769D'
,'T22769S'
,'T22791D'
,'T22791S'
,'T22792D'
,'T22792S'
,'T22799D'
,'T22799S'
,'T23001D'
,'T23001S'
,'T23002D'
,'T23002S'
,'T23009D'
,'T23009S'
,'T23011D'
,'T23011S'
,'T23012D'
,'T23012S'
,'T23019D'
,'T23019S'
,'T23021D'
,'T23021S'
,'T23022D'
,'T23022S'
,'T23029D'
,'T23029S'
,'T23031D'
,'T23031S'
,'T23032D'
,'T23032S'
,'T23039D'
,'T23039S'
,'T23041D'
,'T23041S'
,'T23042D'
,'T23042S'
,'T23049D'
,'T23049S'
,'T23051D'
,'T23051S'
,'T23052D'
,'T23052S'
,'T23059D'
,'T23059S'
,'T23061D'
,'T23061S'
,'T23062D'
,'T23062S'
,'T23069D'
,'T23069S'
,'T23071D'
,'T23071S'
,'T23072D'
,'T23072S'
,'T23079D'
,'T23079S'
,'T23091D'
,'T23091S'
,'T23092D'
,'T23092S'
,'T23099D'
,'T23099S'
,'T23101D'
,'T23101S'
,'T23102D'
,'T23102S'
,'T23109D'
,'T23109S'
,'T23111D'
,'T23111S'
,'T23112D'
,'T23112S'
,'T23119D'
,'T23119S'
,'T23121D'
,'T23121S'
,'T23122D'
,'T23122S'
,'T23129D'
,'T23129S'
,'T23131D'
,'T23131S'
,'T23132D'
,'T23132S'
,'T23139D'
,'T23139S'
,'T23141D'
,'T23141S'
,'T23142D'
,'T23142S'
,'T23149D'
,'T23149S'
,'T23151D'
,'T23151S'
,'T23152D'
,'T23152S'
,'T23159D'
,'T23159S'
,'T23161D'
,'T23161S'
,'T23162D'
,'T23162S'
,'T23169D'
,'T23169S'
,'T23171D'
,'T23171S'
,'T23172D'
,'T23172S'
,'T23179D'
,'T23179S'
,'T23191D'
,'T23191S'
,'T23192D'
,'T23192S'
,'T23199D'
,'T23199S'
,'T23201D'
,'T23201S'
,'T23202D'
,'T23202S'
,'T23209D'
,'T23209S'
,'T23211D'
,'T23211S'
,'T23212D'
,'T23212S'
,'T23219D'
,'T23219S'
,'T23221D'
,'T23221S'
,'T23222D'
,'T23222S'
,'T23229D'
,'T23229S'
,'T23231D'
,'T23231S'
,'T23232D'
,'T23232S'
,'T23239D'
,'T23239S'
,'T23241D'
,'T23241S'
,'T23242D'
,'T23242S'
,'T23249D'
,'T23249S'
,'T23251D'
,'T23251S'
,'T23252D'
,'T23252S'
,'T23259D'
,'T23259S'
,'T23261D'
,'T23261S'
,'T23262D'
,'T23262S'
,'T23269D'
,'T23269S'
,'T23271D'
,'T23271S'
,'T23272D'
,'T23272S'
,'T23279D'
,'T23279S'
,'T23291D'
,'T23291S'
,'T23292D'
,'T23292S'
,'T23299D'
,'T23299S'
,'T23301D'
,'T23301S'
,'T23302D'
,'T23302S'
,'T23309D'
,'T23309S'
,'T23311D'
,'T23311S'
,'T23312D'
,'T23312S'
,'T23319D'
,'T23319S'
,'T23321D'
,'T23321S'
,'T23322D'
,'T23322S'
,'T23329D'
,'T23329S'
,'T23331D'
,'T23331S'
,'T23332D'
,'T23332S'
,'T23339D'
,'T23339S'
,'T23341D'
,'T23341S'
,'T23342D'
,'T23342S'
,'T23349D'
,'T23349S'
,'T23351D'
,'T23351S'
,'T23352D'
,'T23352S'
,'T23359D'
,'T23359S'
,'T23361D'
,'T23361S'
,'T23362D'
,'T23362S'
,'T23369D'
,'T23369S'
,'T23371D'
,'T23371S'
,'T23372D'
,'T23372S'
,'T23379D'
,'T23379S'
,'T23391D'
,'T23391S'
,'T23392D'
,'T23392S'
,'T23399D'
,'T23399S'
,'T23401D'
,'T23401S'
,'T23402D'
,'T23402S'
,'T23409D'
,'T23409S'
,'T23411D'
,'T23411S'
,'T23412D'
,'T23412S'
,'T23419D'
,'T23419S'
,'T23421D'
,'T23421S'
,'T23422D'
,'T23422S'
,'T23429D'
,'T23429S'
,'T23431D'
,'T23431S'
,'T23432D'
,'T23432S'
,'T23439D'
,'T23439S'
,'T23441D'
,'T23441S'
,'T23442D'
,'T23442S'
,'T23449D'
,'T23449S'
,'T23451D'
,'T23451S'
,'T23452D'
,'T23452S'
,'T23459D'
,'T23459S'
,'T23461D'
,'T23461S'
,'T23462D'
,'T23462S'
,'T23469D'
,'T23469S'
,'T23471D'
,'T23471S'
,'T23472D'
,'T23472S'
,'T23479D'
,'T23479S'
,'T23491D'
,'T23491S'
,'T23492D'
,'T23492S'
,'T23499D'
,'T23499S'
,'T23501D'
,'T23501S'
,'T23502D'
,'T23502S'
,'T23509D'
,'T23509S'
,'T23511D'
,'T23511S'
,'T23512D'
,'T23512S'
,'T23519D'
,'T23519S'
,'T23521D'
,'T23521S'
,'T23522D'
,'T23522S'
,'T23529D'
,'T23529S'
,'T23531D'
,'T23531S'
,'T23532D'
,'T23532S'
,'T23539D'
,'T23539S'
,'T23541D'
,'T23541S'
,'T23542D'
,'T23542S'
,'T23549D'
,'T23549S'
,'T23551D'
,'T23551S'
,'T23552D'
,'T23552S'
,'T23559D'
,'T23559S'
,'T23561D'
,'T23561S'
,'T23562D'
,'T23562S'
,'T23569D'
,'T23569S'
,'T23571D'
,'T23571S'
,'T23572D'
,'T23572S'
,'T23579D'
,'T23579S'
,'T23591D'
,'T23591S'
,'T23592D'
,'T23592S'
,'T23599D'
,'T23599S'
,'T23601D'
,'T23601S'
,'T23602D'
,'T23602S'
,'T23609D'
,'T23609S'
,'T23611D'
,'T23611S'
,'T23612D'
,'T23612S'
,'T23619D'
,'T23619S'
,'T23621D'
,'T23621S'
,'T23622D'
,'T23622S'
,'T23629D'
,'T23629S'
,'T23631D'
,'T23631S'
,'T23632D'
,'T23632S'
,'T23639D'
,'T23639S'
,'T23641D'
,'T23641S'
,'T23642D'
,'T23642S'
,'T23649D'
,'T23649S'
,'T23651D'
,'T23651S'
,'T23652D'
,'T23652S'
,'T23659D'
,'T23659S'
,'T23661D'
,'T23661S'
,'T23662D'
,'T23662S'
,'T23669D'
,'T23669S'
,'T23671D'
,'T23671S'
,'T23672D'
,'T23672S'
,'T23679D'
,'T23679S'
,'T23691D'
,'T23691S'
,'T23692D'
,'T23692S'
,'T23699D'
,'T23699S'
,'T23701D'
,'T23701S'
,'T23702D'
,'T23702S'
,'T23709D'
,'T23709S'
,'T23711D'
,'T23711S'
,'T23712D'
,'T23712S'
,'T23719D'
,'T23719S'
,'T23721D'
,'T23721S'
,'T23722D'
,'T23722S'
,'T23729D'
,'T23729S'
,'T23731D'
,'T23731S'
,'T23732D'
,'T23732S'
,'T23739D'
,'T23739S'
,'T23741D'
,'T23741S'
,'T23742D'
,'T23742S'
,'T23749D'
,'T23749S'
,'T23751D'
,'T23751S'
,'T23752D'
,'T23752S'
,'T23759D'
,'T23759S'
,'T23761D'
,'T23761S'
,'T23762D'
,'T23762S'
,'T23769D'
,'T23769S'
,'T23771D'
,'T23771S'
,'T23772D'
,'T23772S'
,'T23779D'
,'T23779S'
,'T23791D'
,'T23791S'
,'T23792D'
,'T23792S'
,'T23799D'
,'T23799S'
,'T24001D'
,'T24001S'
,'T24002D'
,'T24002S'
,'T24009D'
,'T24009S'
,'T24011D'
,'T24011S'
,'T24012D'
,'T24012S'
,'T24019D'
,'T24019S'
,'T24021D'
,'T24021S'
,'T24022D'
,'T24022S'
,'T24029D'
,'T24029S'
,'T24031D'
,'T24031S'
,'T24032D'
,'T24032S'
,'T24039D'
,'T24039S'
,'T24091D'
,'T24091S'
,'T24092D'
,'T24092S'
,'T24099D'
,'T24099S'
,'T24101D'
,'T24101S'
,'T24102D'
,'T24102S'
,'T24109D'
,'T24109S'
,'T24111D'
,'T24111S'
,'T24112D'
,'T24112S'
,'T24119D'
,'T24119S'
,'T24121D'
,'T24121S'
,'T24122D'
,'T24122S'
,'T24129D'
,'T24129S'
,'T24131D'
,'T24131S'
,'T24132D'
,'T24132S'
,'T24139D'
,'T24139S'
,'T24191D'
,'T24191S'
,'T24192D'
,'T24192S'
,'T24199D'
,'T24199S'
,'T24201D'
,'T24201S'
,'T24202D'
,'T24202S'
,'T24209D'
,'T24209S'
,'T24211D'
,'T24211S'
,'T24212D'
,'T24212S'
,'T24219D'
,'T24219S'
,'T24221D'
,'T24221S'
,'T24222D'
,'T24222S'
,'T24229D'
,'T24229S'
,'T24231D'
,'T24231S'
,'T24232D'
,'T24232S'
,'T24239D'
,'T24239S'
,'T24291D'
,'T24291S'
,'T24292D'
,'T24292S'
,'T24299D'
,'T24299S'
,'T24301D'
,'T24301S'
,'T24302D'
,'T24302S'
,'T24309D'
,'T24309S'
,'T24311D'
,'T24311S'
,'T24312D'
,'T24312S'
,'T24319D'
,'T24319S'
,'T24321D'
,'T24321S'
,'T24322D'
,'T24322S'
,'T24329D'
,'T24329S'
,'T24331D'
,'T24331S'
,'T24332D'
,'T24332S'
,'T24339D'
,'T24339S'
,'T24391D'
,'T24391S'
,'T24392D'
,'T24392S'
,'T24399D'
,'T24399S'
,'T24401D'
,'T24401S'
,'T24402D'
,'T24402S'
,'T24409D'
,'T24409S'
,'T24411D'
,'T24411S'
,'T24412D'
,'T24412S'
,'T24419D'
,'T24419S'
,'T24421D'
,'T24421S'
,'T24422D'
,'T24422S'
,'T24429D'
,'T24429S'
,'T24431D'
,'T24431S'
,'T24432D'
,'T24432S'
,'T24439D'
,'T24439S'
,'T24491D'
,'T24491S'
,'T24492D'
,'T24492S'
,'T24499D'
,'T24499S'
,'T24501D'
,'T24501S'
,'T24502D'
,'T24502S'
,'T24509D'
,'T24509S'
,'T24511D'
,'T24511S'
,'T24512D'
,'T24512S'
,'T24519D'
,'T24519S'
,'T24521D'
,'T24521S'
,'T24522D'
,'T24522S'
,'T24529D'
,'T24529S'
,'T24531D'
,'T24531S'
,'T24532D'
,'T24532S'
,'T24539D'
,'T24539S'
,'T24591D'
,'T24591S'
,'T24592D'
,'T24592S'
,'T24599D'
,'T24599S'
,'T24601D'
,'T24601S'
,'T24602D'
,'T24602S'
,'T24609D'
,'T24609S'
,'T24611D'
,'T24611S'
,'T24612D'
,'T24612S'
,'T24619D'
,'T24619S'
,'T24621D'
,'T24621S'
,'T24622D'
,'T24622S'
,'T24629D'
,'T24629S'
,'T24631D'
,'T24631S'
,'T24632D'
,'T24632S'
,'T24639D'
,'T24639S'
,'T24691D'
,'T24691S'
,'T24692D'
,'T24692S'
,'T24699D'
,'T24699S'
,'T24701D'
,'T24701S'
,'T24702D'
,'T24702S'
,'T24709D'
,'T24709S'
,'T24711D'
,'T24711S'
,'T24712D'
,'T24712S'
,'T24719D'
,'T24719S'
,'T24721D'
,'T24721S'
,'T24722D'
,'T24722S'
,'T24729D'
,'T24729S'
,'T24731D'
,'T24731S'
,'T24732D'
,'T24732S'
,'T24739D'
,'T24739S'
,'T24791D'
,'T24791S'
,'T24792D'
,'T24792S'
,'T24799D'
,'T24799S'
,'T25011D'
,'T25011S'
,'T25012D'
,'T25012S'
,'T25019D'
,'T25019S'
,'T25021D'
,'T25021S'
,'T25022D'
,'T25022S'
,'T25029D'
,'T25029S'
,'T25031D'
,'T25031S'
,'T25032D'
,'T25032S'
,'T25039D'
,'T25039S'
,'T25091D'
,'T25091S'
,'T25092D'
,'T25092S'
,'T25099D'
,'T25099S'
,'T25111D'
,'T25111S'
,'T25112D'
,'T25112S'
,'T25119D'
,'T25119S'
,'T25121D'
,'T25121S'
,'T25122D'
,'T25122S'
,'T25129D'
,'T25129S'
,'T25131D'
,'T25131S'
,'T25132D'
,'T25132S'
,'T25139D'
,'T25139S'
,'T25191D'
,'T25191S'
,'T25192D'
,'T25192S'
,'T25199D'
,'T25199S'
,'T25211D'
,'T25211S'
,'T25212D'
,'T25212S'
,'T25219D'
,'T25219S'
,'T25221D'
,'T25221S'
,'T25222D'
,'T25222S'
,'T25229D'
,'T25229S'
,'T25231D'
,'T25231S'
,'T25232D'
,'T25232S'
,'T25239D'
,'T25239S'
,'T25291D'
,'T25291S'
,'T25292D'
,'T25292S'
,'T25299D'
,'T25299S'
,'T25311D'
,'T25311S'
,'T25312D'
,'T25312S'
,'T25319D'
,'T25319S'
,'T25321D'
,'T25321S'
,'T25322D'
,'T25322S'
,'T25329D'
,'T25329S'
,'T25331D'
,'T25331S'
,'T25332D'
,'T25332S'
,'T25339D'
,'T25339S'
,'T25391D'
,'T25391S'
,'T25392D'
,'T25392S'
,'T25399D'
,'T25399S'
,'T25411D'
,'T25411S'
,'T25412D'
,'T25412S'
,'T25419D'
,'T25419S'
,'T25421D'
,'T25421S'
,'T25422D'
,'T25422S'
,'T25429D'
,'T25429S'
,'T25431D'
,'T25431S'
,'T25432D'
,'T25432S'
,'T25439D'
,'T25439S'
,'T25491D'
,'T25491S'
,'T25492D'
,'T25492S'
,'T25499D'
,'T25499S'
,'T25511D'
,'T25511S'
,'T25512D'
,'T25512S'
,'T25519D'
,'T25519S'
,'T25521D'
,'T25521S'
,'T25522D'
,'T25522S'
,'T25529D'
,'T25529S'
,'T25531D'
,'T25531S'
,'T25532D'
,'T25532S'
,'T25539D'
,'T25539S'
,'T25591D'
,'T25591S'
,'T25592D'
,'T25592S'
,'T25599D'
,'T25599S'
,'T25611D'
,'T25611S'
,'T25612D'
,'T25612S'
,'T25619D'
,'T25619S'
,'T25621D'
,'T25621S'
,'T25622D'
,'T25622S'
,'T25629D'
,'T25629S'
,'T25631D'
,'T25631S'
,'T25632D'
,'T25632S'
,'T25639D'
,'T25639S'
,'T25691D'
,'T25691S'
,'T25692D'
,'T25692S'
,'T25699D'
,'T25699S'
,'T25711D'
,'T25711S'
,'T25712D'
,'T25712S'
,'T25719D'
,'T25719S'
,'T25721D'
,'T25721S'
,'T25722D'
,'T25722S'
,'T25729D'
,'T25729S'
,'T25731D'
,'T25731S'
,'T25732D'
,'T25732S'
,'T25739D'
,'T25739S'
,'T25791D'
,'T25791S'
,'T25792D'
,'T25792S'
,'T25799D'
,'T25799S'
,'T2600XD'
,'T2600XS'
,'T2601XD'
,'T2601XS'
,'T2602XD'
,'T2602XS'
,'T2610XD'
,'T2610XS'
,'T2611XD'
,'T2611XS'
,'T2612XD'
,'T2612XS'
,'T2620XD'
,'T2620XS'
,'T2621XD'
,'T2621XS'
,'T2622XD'
,'T2622XS'
,'T2630XD'
,'T2630XS'
,'T2631XD'
,'T2631XS'
,'T2632XD'
,'T2632XS'
,'T2640XD'
,'T2640XS'
,'T2641XD'
,'T2641XS'
,'T2642XD'
,'T2642XS'
,'T2650XD'
,'T2650XS'
,'T2651XD'
,'T2651XS'
,'T2652XD'
,'T2652XS'
,'T2660XD'
,'T2660XS'
,'T2661XD'
,'T2661XS'
,'T2662XD'
,'T2662XS'
,'T2670XD'
,'T2670XS'
,'T2671XD'
,'T2671XS'
,'T2672XD'
,'T2672XS'
,'T2680XD'
,'T2680XS'
,'T2681XD'
,'T2681XS'
,'T2682XD'
,'T2682XS'
,'T2690XD'
,'T2690XS'
,'T2691XD'
,'T2691XS'
,'T2692XD'
,'T2692XS'
,'T270XXD'
,'T270XXS'
,'T271XXD'
,'T271XXS'
,'T272XXD'
,'T272XXS'
,'T273XXD'
,'T273XXS'
,'T274XXD'
,'T274XXS'
,'T275XXD'
,'T275XXS'
,'T276XXD'
,'T276XXS'
,'T277XXD'
,'T277XXS'
,'T280XXD'
,'T280XXS'
,'T281XXD'
,'T281XXS'
,'T282XXD'
,'T282XXS'
,'T283XXD'
,'T283XXS'
,'T2840XD'
,'T2840XS'
,'T28411D'
,'T28411S'
,'T28412D'
,'T28412S'
,'T28419D'
,'T28419S'
,'T2849XD'
,'T2849XS'
,'T285XXD'
,'T285XXS'
,'T286XXD'
,'T286XXS'
,'T287XXD'
,'T287XXS'
,'T288XXD'
,'T288XXS'
,'T2890XD'
,'T2890XS'
,'T28911D'
,'T28911S'
,'T28912D'
,'T28912S'
,'T28919D'
,'T28919S'
,'T2899XD'
,'T2899XS'
,'T300'
,'T304'
,'T310'
,'T3110'
,'T3111'
,'T3120'
,'T3121'
,'T3122'
,'T3130'
,'T3131'
,'T3132'
,'T3133'
,'T3140'
,'T3141'
,'T3142'
,'T3143'
,'T3144'
,'T3150'
,'T3151'
,'T3152'
,'T3153'
,'T3154'
,'T3155'
,'T3160'
,'T3161'
,'T3162'
,'T3163'
,'T3164'
,'T3165'
,'T3166'
,'T3170'
,'T3171'
,'T3172'
,'T3173'
,'T3174'
,'T3175'
,'T3176'
,'T3177'
,'T3180'
,'T3181'
,'T3182'
,'T3183'
,'T3184'
,'T3185'
,'T3186'
,'T3187'
,'T3188'
,'T3190'
,'T3191'
,'T3192'
,'T3193'
,'T3194'
,'T3195'
,'T3196'
,'T3197'
,'T3198'
,'T3199'
,'T320'
,'T3210'
,'T3211'
,'T3220'
,'T3221'
,'T3222'
,'T3230'
,'T3231'
,'T3232'
,'T3233'
,'T3240'
,'T3241'
,'T3242'
,'T3243'
,'T3244'
,'T3250'
,'T3251'
,'T3252'
,'T3253'
,'T3254'
,'T3255'
,'T3260'
,'T3261'
,'T3262'
,'T3263'
,'T3264'
,'T3265'
,'T3266'
,'T3270'
,'T3271'
,'T3272'
,'T3273'
,'T3274'
,'T3275'
,'T3276'
,'T3277'
,'T3280'
,'T3281'
,'T3282'
,'T3283'
,'T3284'
,'T3285'
,'T3286'
,'T3287'
,'T3288'
,'T3290'
,'T3291'
,'T3292'
,'T3293'
,'T3294'
,'T3295'
,'T3296'
,'T3297'
,'T3298'
,'T3299'
,'T33011D'
,'T33011S'
,'T33012D'
,'T33012S'
,'T33019D'
,'T33019S'
,'T3302XD'
,'T3302XS'
,'T3309XD'
,'T3309XS'
,'T331XXD'
,'T331XXS'
,'T332XXD'
,'T332XXS'
,'T333XXD'
,'T333XXS'
,'T3340XD'
,'T3340XS'
,'T3341XD'
,'T3341XS'
,'T3342XD'
,'T3342XS'
,'T33511D'
,'T33511S'
,'T33512D'
,'T33512S'
,'T33519D'
,'T33519S'
,'T33521D'
,'T33521S'
,'T33522D'
,'T33522S'
,'T33529D'
,'T33529S'
,'T33531D'
,'T33531S'
,'T33532D'
,'T33532S'
,'T33539D'
,'T33539S'
,'T3360XD'
,'T3360XS'
,'T3361XD'
,'T3361XS'
,'T3362XD'
,'T3362XS'
,'T3370XD'
,'T3370XS'
,'T3371XD'
,'T3371XS'
,'T3372XD'
,'T3372XS'
,'T33811D'
,'T33811S'
,'T33812D'
,'T33812S'
,'T33819D'
,'T33819S'
,'T33821D'
,'T33821S'
,'T33822D'
,'T33822S'
,'T33829D'
,'T33829S'
,'T33831D'
,'T33831S'
,'T33832D'
,'T33832S'
,'T33839D'
,'T33839S'
,'T3390XD'
,'T3390XS'
,'T3399XD'
,'T3399XS'
,'T34011D'
,'T34011S'
,'T34012D'
,'T34012S'
,'T34019D'
,'T34019S'
,'T3402XD'
,'T3402XS'
,'T3409XD'
,'T3409XS'
,'T341XXD'
,'T341XXS'
,'T342XXD'
,'T342XXS'
,'T343XXD'
,'T343XXS'
,'T3440XD'
,'T3440XS'
,'T3441XD'
,'T3441XS'
,'T3442XD'
,'T3442XS'
,'T34511D'
,'T34511S'
,'T34512D'
,'T34512S'
,'T34519D'
,'T34519S'
,'T34521D'
,'T34521S'
,'T34522D'
,'T34522S'
,'T34529D'
,'T34529S'
,'T34531D'
,'T34531S'
,'T34532D'
,'T34532S'
,'T34539D'
,'T34539S'
,'T3460XD'
,'T3460XS'
,'T3461XD'
,'T3461XS'
,'T3462XD'
,'T3462XS'
,'T3470XD'
,'T3470XS'
,'T3471XD'
,'T3471XS'
,'T3472XD'
,'T3472XS'
,'T34811D'
,'T34811S'
,'T34812D'
,'T34812S'
,'T34819D'
,'T34819S'
,'T34821D'
,'T34821S'
,'T34822D'
,'T34822S'
,'T34829D'
,'T34829S'
,'T34831D'
,'T34831S'
,'T34832D'
,'T34832S'
,'T34839D'
,'T34839S'
,'T3490XD'
,'T3490XS'
,'T3499XD'
,'T3499XS'
,'T360X1D'
,'T360X1S'
,'T360X2D'
,'T360X2S'
,'T360X3D'
,'T360X3S'
,'T360X4D'
,'T360X4S'
,'T360X5D'
,'T360X5S'
,'T360X6D'
,'T360X6S'
,'T361X1D'
,'T361X1S'
,'T361X2D'
,'T361X2S'
,'T361X3D'
,'T361X3S'
,'T361X4D'
,'T361X4S'
,'T361X5D'
,'T361X5S'
,'T361X6D'
,'T361X6S'
,'T362X1D'
,'T362X1S'
,'T362X2D'
,'T362X2S'
,'T362X3D'
,'T362X3S'
,'T362X4D'
,'T362X4S'
,'T362X5D'
,'T362X5S'
,'T362X6D'
,'T362X6S'
,'T363X1D'
,'T363X1S'
,'T363X2D'
,'T363X2S'
,'T363X3D'
,'T363X3S'
,'T363X4D'
,'T363X4S'
,'T363X5D'
,'T363X5S'
,'T363X6D'
,'T363X6S'
,'T364X1D'
,'T364X1S'
,'T364X2D'
,'T364X2S'
,'T364X3D'
,'T364X3S'
,'T364X4D'
,'T364X4S'
,'T364X5D'
,'T364X5S'
,'T364X6D'
,'T364X6S'
,'T365X1D'
,'T365X1S'
,'T365X2D'
,'T365X2S'
,'T365X3D'
,'T365X3S'
,'T365X4D'
,'T365X4S'
,'T365X5D'
,'T365X5S'
,'T365X6D'
,'T365X6S'
,'T366X1D'
,'T366X1S'
,'T366X2D'
,'T366X2S'
,'T366X3D'
,'T366X3S'
,'T366X4D'
,'T366X4S'
,'T366X5D'
,'T366X5S'
,'T366X6D'
,'T366X6S'
,'T367X1D'
,'T367X1S'
,'T367X2D'
,'T367X2S'
,'T367X3D'
,'T367X3S'
,'T367X4D'
,'T367X4S'
,'T367X5D'
,'T367X5S'
,'T367X6D'
,'T367X6S'
,'T368X1D'
,'T368X1S'
,'T368X2D'
,'T368X2S'
,'T368X3D'
,'T368X3S'
,'T368X4D'
,'T368X4S'
,'T368X5D'
,'T368X5S'
,'T368X6D'
,'T368X6S'
,'T3691XD'
,'T3691XS'
,'T3692XD'
,'T3692XS'
,'T3693XD'
,'T3693XS'
,'T3694XD'
,'T3694XS'
,'T3695XD'
,'T3695XS'
,'T3696XD'
,'T3696XS'
,'T370X1D'
,'T370X1S'
,'T370X2D'
,'T370X2S'
,'T370X3D'
,'T370X3S'
,'T370X4D'
,'T370X4S'
,'T370X5D'
,'T370X5S'
,'T370X6D'
,'T370X6S'
,'T371X1D'
,'T371X1S'
,'T371X2D'
,'T371X2S'
,'T371X3D'
,'T371X3S'
,'T371X4D'
,'T371X4S'
,'T371X5D'
,'T371X5S'
,'T371X6D'
,'T371X6S'
,'T372X1D'
,'T372X1S'
,'T372X2D'
,'T372X2S'
,'T372X3D'
,'T372X3S'
,'T372X4D'
,'T372X4S'
,'T372X5D'
,'T372X5S'
,'T372X6D'
,'T372X6S'
,'T373X1D'
,'T373X1S'
,'T373X2D'
,'T373X2S'
,'T373X3D'
,'T373X3S'
,'T373X4D'
,'T373X4S'
,'T373X5D'
,'T373X5S'
,'T373X6D'
,'T373X6S'
,'T374X1D'
,'T374X1S'
,'T374X2D'
,'T374X2S'
,'T374X3D'
,'T374X3S'
,'T374X4D'
,'T374X4S'
,'T374X5D'
,'T374X5S'
,'T374X6D'
,'T374X6S'
,'T375X1D'
,'T375X1S'
,'T375X2D'
,'T375X2S'
,'T375X3D'
,'T375X3S'
,'T375X4D'
,'T375X4S'
,'T375X5D'
,'T375X5S'
,'T375X6D'
,'T375X6S'
,'T378X1D'
,'T378X1S'
,'T378X2D'
,'T378X2S'
,'T378X3D'
,'T378X3S'
,'T378X4D'
,'T378X4S'
,'T378X5D'
,'T378X5S'
,'T378X6D'
,'T378X6S'
,'T3791XD'
,'T3791XS'
,'T3792XD'
,'T3792XS'
,'T3793XD'
,'T3793XS'
,'T3794XD'
,'T3794XS'
,'T3795XD'
,'T3795XS'
,'T3796XD'
,'T3796XS'
,'T380X1D'
,'T380X1S'
,'T380X2D'
,'T380X2S'
,'T380X3D'
,'T380X3S'
,'T380X4D'
,'T380X4S'
,'T380X5D'
,'T380X5S'
,'T380X6D'
,'T380X6S'
,'T381X1D'
,'T381X1S'
,'T381X2D'
,'T381X2S'
,'T381X3D'
,'T381X3S'
,'T381X4D'
,'T381X4S'
,'T381X5D'
,'T381X5S'
,'T381X6D'
,'T381X6S'
,'T382X1D'
,'T382X1S'
,'T382X2D'
,'T382X2S'
,'T382X3D'
,'T382X3S'
,'T382X4D'
,'T382X4S'
,'T382X5D'
,'T382X5S'
,'T382X6D'
,'T382X6S'
,'T383X1D'
,'T383X1S'
,'T383X2D'
,'T383X2S'
,'T383X3D'
,'T383X3S'
,'T383X4D'
,'T383X4S'
,'T383X5D'
,'T383X5S'
,'T383X6D'
,'T383X6S'
,'T384X1D'
,'T384X1S'
,'T384X2D'
,'T384X2S'
,'T384X3D'
,'T384X3S'
,'T384X4D'
,'T384X4S'
,'T384X5D'
,'T384X5S'
,'T384X6D'
,'T384X6S'
,'T385X1D'
,'T385X1S'
,'T385X2D'
,'T385X2S'
,'T385X3D'
,'T385X3S'
,'T385X4D'
,'T385X4S'
,'T385X5D'
,'T385X5S'
,'T385X6D'
,'T385X6S'
,'T386X1D'
,'T386X1S'
,'T386X2D'
,'T386X2S'
,'T386X3D'
,'T386X3S'
,'T386X4D'
,'T386X4S'
,'T386X5D'
,'T386X5S'
,'T386X6D'
,'T386X6S'
,'T387X1D'
,'T387X1S'
,'T387X2D'
,'T387X2S'
,'T387X3D'
,'T387X3S'
,'T387X4D'
,'T387X4S'
,'T387X5D'
,'T387X5S'
,'T387X6D'
,'T387X6S'
,'T38801D'
,'T38801S'
,'T38802D'
,'T38802S'
,'T38803D'
,'T38803S'
,'T38804D'
,'T38804S'
,'T38805D'
,'T38805S'
,'T38806D'
,'T38806S'
,'T38811D'
,'T38811S'
,'T38812D'
,'T38812S'
,'T38813D'
,'T38813S'
,'T38814D'
,'T38814S'
,'T38815D'
,'T38815S'
,'T38816D'
,'T38816S'
,'T38891D'
,'T38891S'
,'T38892D'
,'T38892S'
,'T38893D'
,'T38893S'
,'T38894D'
,'T38894S'
,'T38895D'
,'T38895S'
,'T38896D'
,'T38896S'
,'T38901D'
,'T38901S'
,'T38902D'
,'T38902S'
,'T38903D'
,'T38903S'
,'T38904D'
,'T38904S'
,'T38905D'
,'T38905S'
,'T38906D'
,'T38906S'
,'T38991D'
,'T38991S'
,'T38992D'
,'T38992S'
,'T38993D'
,'T38993S'
,'T38994D'
,'T38994S'
,'T38995D'
,'T38995S'
,'T38996D'
,'T38996S'
,'T39011D'
,'T39011S'
,'T39012D'
,'T39012S'
,'T39013D'
,'T39013S'
,'T39014D'
,'T39014S'
,'T39015D'
,'T39015S'
,'T39016D'
,'T39016S'
,'T39091D'
,'T39091S'
,'T39092D'
,'T39092S'
,'T39093D'
,'T39093S'
,'T39094D'
,'T39094S'
,'T39095D'
,'T39095S'
,'T39096D'
,'T39096S'
,'T391X1D'
,'T391X1S'
,'T391X2D'
,'T391X2S'
,'T391X3D'
,'T391X3S'
,'T391X4D'
,'T391X4S'
,'T391X5D'
,'T391X5S'
,'T391X6D'
,'T391X6S'
,'T392X1D'
,'T392X1S'
,'T392X2D'
,'T392X2S'
,'T392X3D'
,'T392X3S'
,'T392X4D'
,'T392X4S'
,'T392X5D'
,'T392X5S'
,'T392X6D'
,'T392X6S'
,'T39311D'
,'T39311S'
,'T39312D'
,'T39312S'
,'T39313D'
,'T39313S'
,'T39314D'
,'T39314S'
,'T39315D'
,'T39315S'
,'T39316D'
,'T39316S'
,'T39391D'
,'T39391S'
,'T39392D'
,'T39392S'
,'T39393D'
,'T39393S'
,'T39394D'
,'T39394S'
,'T39395D'
,'T39395S'
,'T39396D'
,'T39396S'
,'T394X1D'
,'T394X1S'
,'T394X2D'
,'T394X2S'
,'T394X3D'
,'T394X3S'
,'T394X4D'
,'T394X4S'
,'T394X5D'
,'T394X5S'
,'T394X6D'
,'T394X6S'
,'T398X1D'
,'T398X1S'
,'T398X2D'
,'T398X2S'
,'T398X3D'
,'T398X3S'
,'T398X4D'
,'T398X4S'
,'T398X5D'
,'T398X5S'
,'T398X6D'
,'T398X6S'
,'T3991XD'
,'T3991XS'
,'T3992XD'
,'T3992XS'
,'T3993XD'
,'T3993XS'
,'T3994XD'
,'T3994XS'
,'T3995XD'
,'T3995XS'
,'T3996XD'
,'T3996XS'
,'T400X1D'
,'T400X1S'
,'T400X2D'
,'T400X2S'
,'T400X3D'
,'T400X3S'
,'T400X4D'
,'T400X4S'
,'T400X5D'
,'T400X5S'
,'T400X6D'
,'T400X6S'
,'T401X1D'
,'T401X1S'
,'T401X2D'
,'T401X2S'
,'T401X3D'
,'T401X3S'
,'T401X4D'
,'T401X4S'
,'T401X5D'
,'T401X5S'
,'T402X1D'
,'T402X1S'
,'T402X2D'
,'T402X2S'
,'T402X3D'
,'T402X3S'
,'T402X4D'
,'T402X4S'
,'T402X5D'
,'T402X5S'
,'T402X6D'
,'T402X6S'
,'T403X1D'
,'T403X1S'
,'T403X2D'
,'T403X2S'
,'T403X3D'
,'T403X3S'
,'T403X4D'
,'T403X4S'
,'T403X5D'
,'T403X5S'
,'T403X6D'
,'T403X6S'
,'T404X1D'
,'T404X1S'
,'T404X2D'
,'T404X2S'
,'T404X3D'
,'T404X3S'
,'T404X4D'
,'T404X4S'
,'T404X5D'
,'T404X5S'
,'T404X6D'
,'T404X6S'
,'T405X1D'
,'T405X1S'
,'T405X2D'
,'T405X2S'
,'T405X3D'
,'T405X3S'
,'T405X4D'
,'T405X4S'
,'T405X5D'
,'T405X5S'
,'T405X6D'
,'T405X6S'
,'T40601D'
,'T40601S'
,'T40602D'
,'T40602S'
,'T40603D'
,'T40603S'
,'T40604D'
,'T40604S'
,'T40605D'
,'T40605S'
,'T40606D'
,'T40606S'
,'T40691D'
,'T40691S'
,'T40692D'
,'T40692S'
,'T40693D'
,'T40693S'
,'T40694D'
,'T40694S'
,'T40695D'
,'T40695S'
,'T40696D'
,'T40696S'
,'T407X1D'
,'T407X1S'
,'T407X2D'
,'T407X2S'
,'T407X3D'
,'T407X3S'
,'T407X4D'
,'T407X4S'
,'T407X5D'
,'T407X5S'
,'T407X6D'
,'T407X6S'
,'T408X1D'
,'T408X1S'
,'T408X2D'
,'T408X2S'
,'T408X3D'
,'T408X3S'
,'T408X4D'
,'T408X4S'
,'T408X5D'
,'T408X5S'
,'T40901D'
,'T40901S'
,'T40902D'
,'T40902S'
,'T40903D'
,'T40903S'
,'T40904D'
,'T40904S'
,'T40905D'
,'T40905S'
,'T40906D'
,'T40906S'
,'T40991D'
,'T40991S'
,'T40992D'
,'T40992S'
,'T40993D'
,'T40993S'
,'T40994D'
,'T40994S'
,'T40995D'
,'T40995S'
,'T40996D'
,'T40996S'
,'T410X1D'
,'T410X1S'
,'T410X2D'
,'T410X2S'
,'T410X3D'
,'T410X3S'
,'T410X4D'
,'T410X4S'
,'T410X5D'
,'T410X5S'
,'T410X6D'
,'T410X6S'
,'T411X1D'
,'T411X1S'
,'T411X2D'
,'T411X2S'
,'T411X3D'
,'T411X3S'
,'T411X4D'
,'T411X4S'
,'T411X5D'
,'T411X5S'
,'T411X6D'
,'T411X6S'
,'T41201D'
,'T41201S'
,'T41202D'
,'T41202S'
,'T41203D'
,'T41203S'
,'T41204D'
,'T41204S'
,'T41205D'
,'T41205S'
,'T41206D'
,'T41206S'
,'T41291D'
,'T41291S'
,'T41292D'
,'T41292S'
,'T41293D'
,'T41293S'
,'T41294D'
,'T41294S'
,'T41295D'
,'T41295S'
,'T41296D'
,'T41296S'
,'T413X1D'
,'T413X1S'
,'T413X2D'
,'T413X2S'
,'T413X3D'
,'T413X3S'
,'T413X4D'
,'T413X4S'
,'T413X5D'
,'T413X5S'
,'T413X6D'
,'T413X6S'
,'T4141XD'
,'T4141XS'
,'T4142XD'
,'T4142XS'
,'T4143XD'
,'T4143XS'
,'T4144XD'
,'T4144XS'
,'T4145XD'
,'T4145XS'
,'T4146XD'
,'T4146XS'
,'T415X1D'
,'T415X1S'
,'T415X2D'
,'T415X2S'
,'T415X3D'
,'T415X3S'
,'T415X4D'
,'T415X4S'
,'T415X5D'
,'T415X5S'
,'T415X6D'
,'T415X6S'
,'T420X1D'
,'T420X1S'
,'T420X2D'
,'T420X2S'
,'T420X3D'
,'T420X3S'
,'T420X4D'
,'T420X4S'
,'T420X5D'
,'T420X5S'
,'T420X6D'
,'T420X6S'
,'T421X1D'
,'T421X1S'
,'T421X2D'
,'T421X2S'
,'T421X3D'
,'T421X3S'
,'T421X4D'
,'T421X4S'
,'T421X5D'
,'T421X5S'
,'T421X6D'
,'T421X6S'
,'T422X1D'
,'T422X1S'
,'T422X2D'
,'T422X2S'
,'T422X3D'
,'T422X3S'
,'T422X4D'
,'T422X4S'
,'T422X5D'
,'T422X5S'
,'T422X6D'
,'T422X6S'
,'T423X1D'
,'T423X1S'
,'T423X2D'
,'T423X2S'
,'T423X3D'
,'T423X3S'
,'T423X4D'
,'T423X4S'
,'T423X5D'
,'T423X5S'
,'T423X6D'
,'T423X6S'
,'T424X1D'
,'T424X1S'
,'T424X2D'
,'T424X2S'
,'T424X3D'
,'T424X3S'
,'T424X4D'
,'T424X4S'
,'T424X5D'
,'T424X5S'
,'T424X6D'
,'T424X6S'
,'T425X1D'
,'T425X1S'
,'T425X2D'
,'T425X2S'
,'T425X3D'
,'T425X3S'
,'T425X4D'
,'T425X4S'
,'T425X5D'
,'T425X5S'
,'T425X6D'
,'T425X6S'
,'T426X1D'
,'T426X1S'
,'T426X2D'
,'T426X2S'
,'T426X3D'
,'T426X3S'
,'T426X4D'
,'T426X4S'
,'T426X5D'
,'T426X5S'
,'T426X6D'
,'T426X6S'
,'T4271XD'
,'T4271XS'
,'T4272XD'
,'T4272XS'
,'T4273XD'
,'T4273XS'
,'T4274XD'
,'T4274XS'
,'T4275XD'
,'T4275XS'
,'T4276XD'
,'T4276XS'
,'T428X1D'
,'T428X1S'
,'T428X2D'
,'T428X2S'
,'T428X3D'
,'T428X3S'
,'T428X4D'
,'T428X4S'
,'T428X5D'
,'T428X5S'
,'T428X6D'
,'T428X6S'
,'T43011D'
,'T43011S'
,'T43012D'
,'T43012S'
,'T43013D'
,'T43013S'
,'T43014D'
,'T43014S'
,'T43015D'
,'T43015S'
,'T43016D'
,'T43016S'
,'T43021D'
,'T43021S'
,'T43022D'
,'T43022S'
,'T43023D'
,'T43023S'
,'T43024D'
,'T43024S'
,'T43025D'
,'T43025S'
,'T43026D'
,'T43026S'
,'T431X1D'
,'T431X1S'
,'T431X2D'
,'T431X2S'
,'T431X3D'
,'T431X3S'
,'T431X4D'
,'T431X4S'
,'T431X5D'
,'T431X5S'
,'T431X6D'
,'T431X6S'
,'T43201D'
,'T43201S'
,'T43202D'
,'T43202S'
,'T43203D'
,'T43203S'
,'T43204D'
,'T43204S'
,'T43205D'
,'T43205S'
,'T43206D'
,'T43206S'
,'T43211D'
,'T43211S'
,'T43212D'
,'T43212S'
,'T43213D'
,'T43213S'
,'T43214D'
,'T43214S'
,'T43215D'
,'T43215S'
,'T43216D'
,'T43216S'
,'T43221D'
,'T43221S'
,'T43222D'
,'T43222S'
,'T43223D'
,'T43223S'
,'T43224D'
,'T43224S'
,'T43225D'
,'T43225S'
,'T43226D'
,'T43226S'
,'T43291D'
,'T43291S'
,'T43292D'
,'T43292S'
,'T43293D'
,'T43293S'
,'T43294D'
,'T43294S'
,'T43295D'
,'T43295S'
,'T43296D'
,'T43296S'
,'T433X1D'
,'T433X1S'
,'T433X2D'
,'T433X2S'
,'T433X3D'
,'T433X3S'
,'T433X4D'
,'T433X4S'
,'T433X5D'
,'T433X5S'
,'T433X6D'
,'T433X6S'
,'T434X1D'
,'T434X1S'
,'T434X2D'
,'T434X2S'
,'T434X3D'
,'T434X3S'
,'T434X4D'
,'T434X4S'
,'T434X5D'
,'T434X5S'
,'T434X6D'
,'T434X6S'
,'T43501D'
,'T43501S'
,'T43502D'
,'T43502S'
,'T43503D'
,'T43503S'
,'T43504D'
,'T43504S'
,'T43505D'
,'T43505S'
,'T43506D'
,'T43506S'
,'T43591D'
,'T43591S'
,'T43592D'
,'T43592S'
,'T43593D'
,'T43593S'
,'T43594D'
,'T43594S'
,'T43595D'
,'T43595S'
,'T43596D'
,'T43596S'
,'T43601D'
,'T43601S'
,'T43602D'
,'T43602S'
,'T43603D'
,'T43603S'
,'T43604D'
,'T43604S'
,'T43605D'
,'T43605S'
,'T43606D'
,'T43606S'
,'T43611D'
,'T43611S'
,'T43612D'
,'T43612S'
,'T43613D'
,'T43613S'
,'T43614D'
,'T43614S'
,'T43615D'
,'T43615S'
,'T43616D'
,'T43616S'
,'T43621D'
,'T43621S'
,'T43622D'
,'T43622S'
,'T43623D'
,'T43623S'
,'T43624D'
,'T43624S'
,'T43625D'
,'T43625S'
,'T43626D'
,'T43626S'
,'T43631D'
,'T43631S'
,'T43632D'
,'T43632S'
,'T43633D'
,'T43633S'
,'T43634D'
,'T43634S'
,'T43635D'
,'T43635S'
,'T43636D'
,'T43636S'
,'T43691D'
,'T43691S'
,'T43692D'
,'T43692S'
,'T43693D'
,'T43693S'
,'T43694D'
,'T43694S'
,'T43695D'
,'T43695S'
,'T43696D'
,'T43696S'
,'T438X1D'
,'T438X1S'
,'T438X2D'
,'T438X2S'
,'T438X3D'
,'T438X3S'
,'T438X4D'
,'T438X4S'
,'T438X5D'
,'T438X5S'
,'T438X6D'
,'T438X6S'
,'T4391XD'
,'T4391XS'
,'T4392XD'
,'T4392XS'
,'T4393XD'
,'T4393XS'
,'T4394XD'
,'T4394XS'
,'T4395XD'
,'T4395XS'
,'T4396XD'
,'T4396XS'
,'T440X1D'
,'T440X1S'
,'T440X2D'
,'T440X2S'
,'T440X3D'
,'T440X3S'
,'T440X4D'
,'T440X4S'
,'T440X5D'
,'T440X5S'
,'T440X6D'
,'T440X6S'
,'T441X1D'
,'T441X1S'
,'T441X2D'
,'T441X2S'
,'T441X3D'
,'T441X3S'
,'T441X4D'
,'T441X4S'
,'T441X5D'
,'T441X5S'
,'T441X6D'
,'T441X6S'
,'T442X1D'
,'T442X1S'
,'T442X2D'
,'T442X2S'
,'T442X3D'
,'T442X3S'
,'T442X4D'
,'T442X4S'
,'T442X5D'
,'T442X5S'
,'T442X6D'
,'T442X6S'
,'T443X1D'
,'T443X1S'
,'T443X2D'
,'T443X2S'
,'T443X3D'
,'T443X3S'
,'T443X4D'
,'T443X4S'
,'T443X5D'
,'T443X5S'
,'T443X6D'
,'T443X6S'
,'T444X1D'
,'T444X1S'
,'T444X2D'
,'T444X2S'
,'T444X3D'
,'T444X3S'
,'T444X4D'
,'T444X4S'
,'T444X5D'
,'T444X5S'
,'T444X6D'
,'T444X6S'
,'T445X1D'
,'T445X1S'
,'T445X2D'
,'T445X2S'
,'T445X3D'
,'T445X3S'
,'T445X4D'
,'T445X4S'
,'T445X5D'
,'T445X5S'
,'T445X6D'
,'T445X6S'
,'T446X1D'
,'T446X1S'
,'T446X2D'
,'T446X2S'
,'T446X3D'
,'T446X3S'
,'T446X4D'
,'T446X4S'
,'T446X5D'
,'T446X5S'
,'T446X6D'
,'T446X6S'
,'T447X1D'
,'T447X1S'
,'T447X2D'
,'T447X2S'
,'T447X3D'
,'T447X3S'
,'T447X4D'
,'T447X4S'
,'T447X5D'
,'T447X5S'
,'T447X6D'
,'T447X6S'
,'T448X1D'
,'T448X1S'
,'T448X2D'
,'T448X2S'
,'T448X3D'
,'T448X3S'
,'T448X4D'
,'T448X4S'
,'T448X5D'
,'T448X5S'
,'T448X6D'
,'T448X6S'
,'T44901D'
,'T44901S'
,'T44902D'
,'T44902S'
,'T44903D'
,'T44903S'
,'T44904D'
,'T44904S'
,'T44905D'
,'T44905S'
,'T44906D'
,'T44906S'
,'T44991D'
,'T44991S'
,'T44992D'
,'T44992S'
,'T44993D'
,'T44993S'
,'T44994D'
,'T44994S'
,'T44995D'
,'T44995S'
,'T44996D'
,'T44996S'
,'T450X1D'
,'T450X1S'
,'T450X2D'
,'T450X2S'
,'T450X3D'
,'T450X3S'
,'T450X4D'
,'T450X4S'
,'T450X5D'
,'T450X5S'
,'T450X6D'
,'T450X6S'
,'T451X1D'
,'T451X1S'
,'T451X2D'
,'T451X2S'
,'T451X3D'
,'T451X3S'
,'T451X4D'
,'T451X4S'
,'T451X5D'
,'T451X5S'
,'T451X6D'
,'T451X6S'
,'T452X1D'
,'T452X1S'
,'T452X2D'
,'T452X2S'
,'T452X3D'
,'T452X3S'
,'T452X4D'
,'T452X4S'
,'T452X5D'
,'T452X5S'
,'T452X6D'
,'T452X6S'
,'T453X1D'
,'T453X1S'
,'T453X2D'
,'T453X2S'
,'T453X3D'
,'T453X3S'
,'T453X4D'
,'T453X4S'
,'T453X5D'
,'T453X5S'
,'T453X6D'
,'T453X6S'
,'T454X1D'
,'T454X1S'
,'T454X2D'
,'T454X2S'
,'T454X3D'
,'T454X3S'
,'T454X4D'
,'T454X4S'
,'T454X5D'
,'T454X5S'
,'T454X6D'
,'T454X6S'
,'T45511D'
,'T45511S'
,'T45512D'
,'T45512S'
,'T45513D'
,'T45513S'
,'T45514D'
,'T45514S'
,'T45515D'
,'T45515S'
,'T45516D'
,'T45516S'
,'T45521D'
,'T45521S'
,'T45522D'
,'T45522S'
,'T45523D'
,'T45523S'
,'T45524D'
,'T45524S'
,'T45525D'
,'T45525S'
,'T45526D'
,'T45526S'
,'T45601D'
,'T45601S'
,'T45602D'
,'T45602S'
,'T45603D'
,'T45603S'
,'T45604D'
,'T45604S'
,'T45605D'
,'T45605S'
,'T45606D'
,'T45606S'
,'T45611D'
,'T45611S'
,'T45612D'
,'T45612S'
,'T45613D'
,'T45613S'
,'T45614D'
,'T45614S'
,'T45615D'
,'T45615S'
,'T45616D'
,'T45616S'
,'T45621D'
,'T45621S'
,'T45622D'
,'T45622S'
,'T45623D'
,'T45623S'
,'T45624D'
,'T45624S'
,'T45625D'
,'T45625S'
,'T45626D'
,'T45626S'
,'T45691D'
,'T45691S'
,'T45692D'
,'T45692S'
,'T45693D'
,'T45693S'
,'T45694D'
,'T45694S'
,'T45695D'
,'T45695S'
,'T45696D'
,'T45696S'
,'T457X1D'
,'T457X1S'
,'T457X2D'
,'T457X2S'
,'T457X3D'
,'T457X3S'
,'T457X4D'
,'T457X4S'
,'T457X5D'
,'T457X5S'
,'T457X6D'
,'T457X6S'
,'T458X1D'
,'T458X1S'
,'T458X2D'
,'T458X2S'
,'T458X3D'
,'T458X3S'
,'T458X4D'
,'T458X4S'
,'T458X5D'
,'T458X5S'
,'T458X6D'
,'T458X6S'
,'T4591XD'
,'T4591XS'
,'T4592XD'
,'T4592XS'
,'T4593XD'
,'T4593XS'
,'T4594XD'
,'T4594XS'
,'T4595XD'
,'T4595XS'
,'T4596XD'
,'T4596XS'
,'T460X1D'
,'T460X1S'
,'T460X2D'
,'T460X2S'
,'T460X3D'
,'T460X3S'
,'T460X4D'
,'T460X4S'
,'T460X5D'
,'T460X5S'
,'T460X6D'
,'T460X6S'
,'T461X1D'
,'T461X1S'
,'T461X2D'
,'T461X2S'
,'T461X3D'
,'T461X3S'
,'T461X4D'
,'T461X4S'
,'T461X5D'
,'T461X5S'
,'T461X6D'
,'T461X6S'
,'T462X1D'
,'T462X1S'
,'T462X2D'
,'T462X2S'
,'T462X3D'
,'T462X3S'
,'T462X4D'
,'T462X4S'
,'T462X5D'
,'T462X5S'
,'T462X6D'
,'T462X6S'
,'T463X1D'
,'T463X1S'
,'T463X2D'
,'T463X2S'
,'T463X3D'
,'T463X3S'
,'T463X4D'
,'T463X4S'
,'T463X5D'
,'T463X5S'
,'T463X6D'
,'T463X6S'
,'T464X1D'
,'T464X1S'
,'T464X2D'
,'T464X2S'
,'T464X3D'
,'T464X3S'
,'T464X4D'
,'T464X4S'
,'T464X5D'
,'T464X5S'
,'T464X6D'
,'T464X6S'
,'T465X1D'
,'T465X1S'
,'T465X2D'
,'T465X2S'
,'T465X3D'
,'T465X3S'
,'T465X4D'
,'T465X4S'
,'T465X5D'
,'T465X5S'
,'T465X6D'
,'T465X6S'
,'T466X1D'
,'T466X1S'
,'T466X2D'
,'T466X2S'
,'T466X3D'
,'T466X3S'
,'T466X4D'
,'T466X4S'
,'T466X5D'
,'T466X5S'
,'T466X6D'
,'T466X6S'
,'T467X1D'
,'T467X1S'
,'T467X2D'
,'T467X2S'
,'T467X3D'
,'T467X3S'
,'T467X4D'
,'T467X4S'
,'T467X5D'
,'T467X5S'
,'T467X6D'
,'T467X6S'
,'T468X1D'
,'T468X1S'
,'T468X2D'
,'T468X2S'
,'T468X3D'
,'T468X3S'
,'T468X4D'
,'T468X4S'
,'T468X5D'
,'T468X5S'
,'T468X6D'
,'T468X6S'
,'T46901D'
,'T46901S'
,'T46902D'
,'T46902S'
,'T46903D'
,'T46903S'
,'T46904D'
,'T46904S'
,'T46905D'
,'T46905S'
,'T46906D'
,'T46906S'
,'T46991D'
,'T46991S'
,'T46992D'
,'T46992S'
,'T46993D'
,'T46993S'
,'T46994D'
,'T46994S'
,'T46995D'
,'T46995S'
,'T46996D'
,'T46996S'
,'T470X1D'
,'T470X1S'
,'T470X2D'
,'T470X2S'
,'T470X3D'
,'T470X3S'
,'T470X4D'
,'T470X4S'
,'T470X5D'
,'T470X5S'
,'T470X6D'
,'T470X6S'
,'T471X1D'
,'T471X1S'
,'T471X2D'
,'T471X2S'
,'T471X3D'
,'T471X3S'
,'T471X4D'
,'T471X4S'
,'T471X5D'
,'T471X5S'
,'T471X6D'
,'T471X6S'
,'T472X1D'
,'T472X1S'
,'T472X2D'
,'T472X2S'
,'T472X3D'
,'T472X3S'
,'T472X4D'
,'T472X4S'
,'T472X5D'
,'T472X5S'
,'T472X6D'
,'T472X6S'
,'T473X1D'
,'T473X1S'
,'T473X2D'
,'T473X2S'
,'T473X3D'
,'T473X3S'
,'T473X4D'
,'T473X4S'
,'T473X5D'
,'T473X5S'
,'T473X6D'
,'T473X6S'
,'T474X1D'
,'T474X1S'
,'T474X2D'
,'T474X2S'
,'T474X3D'
,'T474X3S'
,'T474X4D'
,'T474X4S'
,'T474X5D'
,'T474X5S'
,'T474X6D'
,'T474X6S'
,'T475X1D'
,'T475X1S'
,'T475X2D'
,'T475X2S'
,'T475X3D'
,'T475X3S'
,'T475X4D'
,'T475X4S'
,'T475X5D'
,'T475X5S'
,'T475X6D'
,'T475X6S'
,'T476X1D'
,'T476X1S'
,'T476X2D'
,'T476X2S'
,'T476X3D'
,'T476X3S'
,'T476X4D'
,'T476X4S'
,'T476X5D'
,'T476X5S'
,'T476X6D'
,'T476X6S'
,'T477X1D'
,'T477X1S'
,'T477X2D'
,'T477X2S'
,'T477X3D'
,'T477X3S'
,'T477X4D'
,'T477X4S'
,'T477X5D'
,'T477X5S'
,'T477X6D'
,'T477X6S'
,'T478X1D'
,'T478X1S'
,'T478X2D'
,'T478X2S'
,'T478X3D'
,'T478X3S'
,'T478X4D'
,'T478X4S'
,'T478X5D'
,'T478X5S'
,'T478X6D'
,'T478X6S'
,'T4791XD'
,'T4791XS'
,'T4792XD'
,'T4792XS'
,'T4793XD'
,'T4793XS'
,'T4794XD'
,'T4794XS'
,'T4795XD'
,'T4795XS'
,'T4796XD'
,'T4796XS'
,'T480X1D'
,'T480X1S'
,'T480X2D'
,'T480X2S'
,'T480X3D'
,'T480X3S'
,'T480X4D'
,'T480X4S'
,'T480X5D'
,'T480X5S'
,'T480X6D'
,'T480X6S'
,'T481X1D'
,'T481X1S'
,'T481X2D'
,'T481X2S'
,'T481X3D'
,'T481X3S'
,'T481X4D'
,'T481X4S'
,'T481X5D'
,'T481X5S'
,'T481X6D'
,'T481X6S'
,'T48201D'
,'T48201S'
,'T48202D'
,'T48202S'
,'T48203D'
,'T48203S'
,'T48204D'
,'T48204S'
,'T48205D'
,'T48205S'
,'T48206D'
,'T48206S'
,'T48291D'
,'T48291S'
,'T48292D'
,'T48292S'
,'T48293D'
,'T48293S'
,'T48294D'
,'T48294S'
,'T48295D'
,'T48295S'
,'T48296D'
,'T48296S'
,'T483X1D'
,'T483X1S'
,'T483X2D'
,'T483X2S'
,'T483X3D'
,'T483X3S'
,'T483X4D'
,'T483X4S'
,'T483X5D'
,'T483X5S'
,'T483X6D'
,'T483X6S'
,'T484X1D'
,'T484X1S'
,'T484X2D'
,'T484X2S'
,'T484X3D'
,'T484X3S'
,'T484X4D'
,'T484X4S'
,'T484X5D'
,'T484X5S'
,'T484X6D'
,'T484X6S'
,'T485X1D'
,'T485X1S'
,'T485X2D'
,'T485X2S'
,'T485X3D'
,'T485X3S'
,'T485X4D'
,'T485X4S'
,'T485X5D'
,'T485X5S'
,'T485X6D'
,'T485X6S'
,'T486X1D'
,'T486X1S'
,'T486X2D'
,'T486X2S'
,'T486X3D'
,'T486X3S'
,'T486X4D'
,'T486X4S'
,'T486X5D'
,'T486X5S'
,'T486X6D'
,'T486X6S'
,'T48901D'
,'T48901S'
,'T48902D'
,'T48902S'
,'T48903D'
,'T48903S'
,'T48904D'
,'T48904S'
,'T48905D'
,'T48905S'
,'T48906D'
,'T48906S'
,'T48991D'
,'T48991S'
,'T48992D'
,'T48992S'
,'T48993D'
,'T48993S'
,'T48994D'
,'T48994S'
,'T48995D'
,'T48995S'
,'T48996D'
,'T48996S'
,'T490X1D'
,'T490X1S'
,'T490X2D'
,'T490X2S'
,'T490X3D'
,'T490X3S'
,'T490X4D'
,'T490X4S'
,'T490X5D'
,'T490X5S'
,'T490X6D'
,'T490X6S'
,'T491X1D'
,'T491X1S'
,'T491X2D'
,'T491X2S'
,'T491X3D'
,'T491X3S'
,'T491X4D'
,'T491X4S'
,'T491X5D'
,'T491X5S'
,'T491X6D'
,'T491X6S'
,'T492X1D'
,'T492X1S'
,'T492X2D'
,'T492X2S'
,'T492X3D'
,'T492X3S'
,'T492X4D'
,'T492X4S'
,'T492X5D'
,'T492X5S'
,'T492X6D'
,'T492X6S'
,'T493X1D'
,'T493X1S'
,'T493X2D'
,'T493X2S'
,'T493X3D'
,'T493X3S'
,'T493X4D'
,'T493X4S'
,'T493X5D'
,'T493X5S'
,'T493X6D'
,'T493X6S'
,'T494X1D'
,'T494X1S'
,'T494X2D'
,'T494X2S'
,'T494X3D'
,'T494X3S'
,'T494X4D'
,'T494X4S'
,'T494X5D'
,'T494X5S'
,'T494X6D'
,'T494X6S'
,'T495X1D'
,'T495X1S'
,'T495X2D'
,'T495X2S'
,'T495X3D'
,'T495X3S'
,'T495X4D'
,'T495X4S'
,'T495X5D'
,'T495X5S'
,'T495X6D'
,'T495X6S'
,'T496X1D'
,'T496X1S'
,'T496X2D'
,'T496X2S'
,'T496X3D'
,'T496X3S'
,'T496X4D'
,'T496X4S'
,'T496X5D'
,'T496X5S'
,'T496X6D'
,'T496X6S'
,'T497X1D'
,'T497X1S'
,'T497X2D'
,'T497X2S'
,'T497X3D'
,'T497X3S'
,'T497X4D'
,'T497X4S'
,'T497X5D'
,'T497X5S'
,'T497X6D'
,'T497X6S'
,'T498X1D'
,'T498X1S'
,'T498X2D'
,'T498X2S'
,'T498X3D'
,'T498X3S'
,'T498X4D'
,'T498X4S'
,'T498X5D'
,'T498X5S'
,'T498X6D'
,'T498X6S'
,'T4991XD'
,'T4991XS'
,'T4992XD'
,'T4992XS'
,'T4993XD'
,'T4993XS'
,'T4994XD'
,'T4994XS'
,'T4995XD'
,'T4995XS'
,'T4996XD'
,'T4996XS'
,'T500X1D'
,'T500X1S'
,'T500X2D'
,'T500X2S'
,'T500X3D'
,'T500X3S'
,'T500X4D'
,'T500X4S'
,'T500X5D'
,'T500X5S'
,'T500X6D'
,'T500X6S'
,'T501X1D'
,'T501X1S'
,'T501X2D'
,'T501X2S'
,'T501X3D'
,'T501X3S'
,'T501X4D'
,'T501X4S'
,'T501X5D'
,'T501X5S'
,'T501X6D'
,'T501X6S'
,'T502X1D'
,'T502X1S'
,'T502X2D'
,'T502X2S'
,'T502X3D'
,'T502X3S'
,'T502X4D'
,'T502X4S'
,'T502X5D'
,'T502X5S'
,'T502X6D'
,'T502X6S'
,'T503X1D'
,'T503X1S'
,'T503X2D'
,'T503X2S'
,'T503X3D'
,'T503X3S'
,'T503X4D'
,'T503X4S'
,'T503X5D'
,'T503X5S'
,'T503X6D'
,'T503X6S'
,'T504X1D'
,'T504X1S'
,'T504X2D'
,'T504X2S'
,'T504X3D'
,'T504X3S'
,'T504X4D'
,'T504X4S'
,'T504X5D'
,'T504X5S'
,'T504X6D'
,'T504X6S'
,'T505X1D'
,'T505X1S'
,'T505X2D'
,'T505X2S'
,'T505X3D'
,'T505X3S'
,'T505X4D'
,'T505X4S'
,'T505X5D'
,'T505X5S'
,'T505X6D'
,'T505X6S'
,'T506X1D'
,'T506X1S'
,'T506X2D'
,'T506X2S'
,'T506X3D'
,'T506X3S'
,'T506X4D'
,'T506X4S'
,'T506X5D'
,'T506X5S'
,'T506X6D'
,'T506X6S'
,'T507X1D'
,'T507X1S'
,'T507X2D'
,'T507X2S'
,'T507X3D'
,'T507X3S'
,'T507X4D'
,'T507X4S'
,'T507X5D'
,'T507X5S'
,'T507X6D'
,'T507X6S'
,'T508X1D'
,'T508X1S'
,'T508X2D'
,'T508X2S'
,'T508X3D'
,'T508X3S'
,'T508X4D'
,'T508X4S'
,'T508X5D'
,'T508X5S'
,'T508X6D'
,'T508X6S'
,'T50901D'
,'T50901S'
,'T50902D'
,'T50902S'
,'T50903D'
,'T50903S'
,'T50904D'
,'T50904S'
,'T50905D'
,'T50905S'
,'T50906D'
,'T50906S'
,'T50991D'
,'T50991S'
,'T50992D'
,'T50992S'
,'T50993D'
,'T50993S'
,'T50994D'
,'T50994S'
,'T50995D'
,'T50995S'
,'T50996D'
,'T50996S'
,'T50A11D'
,'T50A11S'
,'T50A12D'
,'T50A12S'
,'T50A13D'
,'T50A13S'
,'T50A14D'
,'T50A14S'
,'T50A15D'
,'T50A15S'
,'T50A16D'
,'T50A16S'
,'T50A21D'
,'T50A21S'
,'T50A22D'
,'T50A22S'
,'T50A23D'
,'T50A23S'
,'T50A24D'
,'T50A24S'
,'T50A25D'
,'T50A25S'
,'T50A26D'
,'T50A26S'
,'T50A91D'
,'T50A91S'
,'T50A92D'
,'T50A92S'
,'T50A93D'
,'T50A93S'
,'T50A94D'
,'T50A94S'
,'T50A95D'
,'T50A95S'
,'T50A96D'
,'T50A96S'
,'T50B11D'
,'T50B11S'
,'T50B12D'
,'T50B12S'
,'T50B13D'
,'T50B13S'
,'T50B14D'
,'T50B14S'
,'T50B15D'
,'T50B15S'
,'T50B16D'
,'T50B16S'
,'T50B91D'
,'T50B91S'
,'T50B92D'
,'T50B92S'
,'T50B93D'
,'T50B93S'
,'T50B94D'
,'T50B94S'
,'T50B95D'
,'T50B95S'
,'T50B96D'
,'T50B96S'
,'T50Z11D'
,'T50Z11S'
,'T50Z12D'
,'T50Z12S'
,'T50Z13D'
,'T50Z13S'
,'T50Z14D'
,'T50Z14S'
,'T50Z15D'
,'T50Z15S'
,'T50Z16D'
,'T50Z16S'
,'T50Z91D'
,'T50Z91S'
,'T50Z92D'
,'T50Z92S'
,'T50Z93D'
,'T50Z93S'
,'T50Z94D'
,'T50Z94S'
,'T50Z95D'
,'T50Z95S'
,'T50Z96D'
,'T50Z96S'
,'T510X1D'
,'T510X1S'
,'T510X2D'
,'T510X2S'
,'T510X3D'
,'T510X3S'
,'T510X4D'
,'T510X4S'
,'T511X1D'
,'T511X1S'
,'T511X2D'
,'T511X2S'
,'T511X3D'
,'T511X3S'
,'T511X4D'
,'T511X4S'
,'T512X1D'
,'T512X1S'
,'T512X2D'
,'T512X2S'
,'T512X3D'
,'T512X3S'
,'T512X4D'
,'T512X4S'
,'T513X1D'
,'T513X1S'
,'T513X2D'
,'T513X2S'
,'T513X3D'
,'T513X3S'
,'T513X4D'
,'T513X4S'
,'T518X1D'
,'T518X1S'
,'T518X2D'
,'T518X2S'
,'T518X3D'
,'T518X3S'
,'T518X4D'
,'T518X4S'
,'T5191XD'
,'T5191XS'
,'T5192XD'
,'T5192XS'
,'T5193XD'
,'T5193XS'
,'T5194XD'
,'T5194XS'
,'T520X1D'
,'T520X1S'
,'T520X2D'
,'T520X2S'
,'T520X3D'
,'T520X3S'
,'T520X4D'
,'T520X4S'
,'T521X1D'
,'T521X1S'
,'T521X2D'
,'T521X2S'
,'T521X3D'
,'T521X3S'
,'T521X4D'
,'T521X4S'
,'T522X1D'
,'T522X1S'
,'T522X2D'
,'T522X2S'
,'T522X3D'
,'T522X3S'
,'T522X4D'
,'T522X4S'
,'T523X1D'
,'T523X1S'
,'T523X2D'
,'T523X2S'
,'T523X3D'
,'T523X3S'
,'T523X4D'
,'T523X4S'
,'T524X1D'
,'T524X1S'
,'T524X2D'
,'T524X2S'
,'T524X3D'
,'T524X3S'
,'T524X4D'
,'T524X4S'
,'T528X1D'
,'T528X1S'
,'T528X2D'
,'T528X2S'
,'T528X3D'
,'T528X3S'
,'T528X4D'
,'T528X4S'
,'T5291XD'
,'T5291XS'
,'T5292XD'
,'T5292XS'
,'T5293XD'
,'T5293XS'
,'T5294XD'
,'T5294XS'
,'T530X1D'
,'T530X1S'
,'T530X2D'
,'T530X2S'
,'T530X3D'
,'T530X3S'
,'T530X4D'
,'T530X4S'
,'T531X1D'
,'T531X1S'
,'T531X2D'
,'T531X2S'
,'T531X3D'
,'T531X3S'
,'T531X4D'
,'T531X4S'
,'T532X1D'
,'T532X1S'
,'T532X2D'
,'T532X2S'
,'T532X3D'
,'T532X3S'
,'T532X4D'
,'T532X4S'
,'T533X1D'
,'T533X1S'
,'T533X2D'
,'T533X2S'
,'T533X3D'
,'T533X3S'
,'T533X4D'
,'T533X4S'
,'T534X1D'
,'T534X1S'
,'T534X2D'
,'T534X2S'
,'T534X3D'
,'T534X3S'
,'T534X4D'
,'T534X4S'
,'T535X1D'
,'T535X1S'
,'T535X2D'
,'T535X2S'
,'T535X3D'
,'T535X3S'
,'T535X4D'
,'T535X4S'
,'T536X1D'
,'T536X1S'
,'T536X2D'
,'T536X2S'
,'T536X3D'
,'T536X3S'
,'T536X4D'
,'T536X4S'
,'T537X1D'
,'T537X1S'
,'T537X2D'
,'T537X2S'
,'T537X3D'
,'T537X3S'
,'T537X4D'
,'T537X4S'
,'T5391XD'
,'T5391XS'
,'T5392XD'
,'T5392XS'
,'T5393XD'
,'T5393XS'
,'T5394XD'
,'T5394XS'
,'T540X1D'
,'T540X1S'
,'T540X2D'
,'T540X2S'
,'T540X3D'
,'T540X3S'
,'T540X4D'
,'T540X4S'
,'T541X1D'
,'T541X1S'
,'T541X2D'
,'T541X2S'
,'T541X3D'
,'T541X3S'
,'T541X4D'
,'T541X4S'
,'T542X1D'
,'T542X1S'
,'T542X2D'
,'T542X2S'
,'T542X3D'
,'T542X3S'
,'T542X4D'
,'T542X4S'
,'T543X1D'
,'T543X1S'
,'T543X2D'
,'T543X2S'
,'T543X3D'
,'T543X3S'
,'T543X4D'
,'T543X4S'
,'T5491XD'
,'T5491XS'
,'T5492XD'
,'T5492XS'
,'T5493XD'
,'T5493XS'
,'T5494XD'
,'T5494XS'
,'T550X1D'
,'T550X1S'
,'T550X2D'
,'T550X2S'
,'T550X3D'
,'T550X3S'
,'T550X4D'
,'T550X4S'
,'T551X1D'
,'T551X1S'
,'T551X2D'
,'T551X2S'
,'T551X3D'
,'T551X3S'
,'T551X4D'
,'T551X4S'
,'T560X1D'
,'T560X1S'
,'T560X2D'
,'T560X2S'
,'T560X3D'
,'T560X3S'
,'T560X4D'
,'T560X4S'
,'T561X1D'
,'T561X1S'
,'T561X2D'
,'T561X2S'
,'T561X3D'
,'T561X3S'
,'T561X4D'
,'T561X4S'
,'T562X1D'
,'T562X1S'
,'T562X2D'
,'T562X2S'
,'T562X3D'
,'T562X3S'
,'T562X4D'
,'T562X4S'
,'T563X1D'
,'T563X1S'
,'T563X2D'
,'T563X2S'
,'T563X3D'
,'T563X3S'
,'T563X4D'
,'T563X4S'
,'T564X1D'
,'T564X1S'
,'T564X2D'
,'T564X2S'
,'T564X3D'
,'T564X3S'
,'T564X4D'
,'T564X4S'
,'T565X1D'
,'T565X1S'
,'T565X2D'
,'T565X2S'
,'T565X3D'
,'T565X3S'
,'T565X4D'
,'T565X4S'
,'T566X1D'
,'T566X1S'
,'T566X2D'
,'T566X2S'
,'T566X3D'
,'T566X3S'
,'T566X4D'
,'T566X4S'
,'T567X1D'
,'T567X1S'
,'T567X2D'
,'T567X2S'
,'T567X3D'
,'T567X3S'
,'T567X4D'
,'T567X4S'
,'T56811D'
,'T56811S'
,'T56812D'
,'T56812S'
,'T56813D'
,'T56813S'
,'T56814D'
,'T56814S'
,'T56891D'
,'T56891S'
,'T56892D'
,'T56892S'
,'T56893D'
,'T56893S'
,'T56894D'
,'T56894S'
,'T5691XD'
,'T5691XS'
,'T5692XD'
,'T5692XS'
,'T5693XD'
,'T5693XS'
,'T5694XD'
,'T5694XS'
,'T570X1D'
,'T570X1S'
,'T570X2D'
,'T570X2S'
,'T570X3D'
,'T570X3S'
,'T570X4D'
,'T570X4S'
,'T571X1D'
,'T571X1S'
,'T571X2D'
,'T571X2S'
,'T571X3D'
,'T571X3S'
,'T571X4D'
,'T571X4S'
,'T572X1D'
,'T572X1S'
,'T572X2D'
,'T572X2S'
,'T572X3D'
,'T572X3S'
,'T572X4D'
,'T572X4S'
,'T573X1D'
,'T573X1S'
,'T573X2D'
,'T573X2S'
,'T573X3D'
,'T573X3S'
,'T573X4D'
,'T573X4S'
,'T578X1D'
,'T578X1S'
,'T578X2D'
,'T578X2S'
,'T578X3D'
,'T578X3S'
,'T578X4D'
,'T578X4S'
,'T5791XD'
,'T5791XS'
,'T5792XD'
,'T5792XS'
,'T5793XD'
,'T5793XS'
,'T5794XD'
,'T5794XS'
,'T5801XD'
,'T5801XS'
,'T5802XD'
,'T5802XS'
,'T5803XD'
,'T5803XS'
,'T5804XD'
,'T5804XS'
,'T5811XD'
,'T5811XS'
,'T5812XD'
,'T5812XS'
,'T5813XD'
,'T5813XS'
,'T5814XD'
,'T5814XS'
,'T582X1D'
,'T582X1S'
,'T582X2D'
,'T582X2S'
,'T582X3D'
,'T582X3S'
,'T582X4D'
,'T582X4S'
,'T588X1D'
,'T588X1S'
,'T588X2D'
,'T588X2S'
,'T588X3D'
,'T588X3S'
,'T588X4D'
,'T588X4S'
,'T5891XD'
,'T5891XS'
,'T5892XD'
,'T5892XS'
,'T5893XD'
,'T5893XS'
,'T5894XD'
,'T5894XS'
,'T590X1D'
,'T590X1S'
,'T590X2D'
,'T590X2S'
,'T590X3D'
,'T590X3S'
,'T590X4D'
,'T590X4S'
,'T591X1D'
,'T591X1S'
,'T591X2D'
,'T591X2S'
,'T591X3D'
,'T591X3S'
,'T591X4D'
,'T591X4S'
,'T592X1D'
,'T592X1S'
,'T592X2D'
,'T592X2S'
,'T592X3D'
,'T592X3S'
,'T592X4D'
,'T592X4S'
,'T593X1D'
,'T593X1S'
,'T593X2D'
,'T593X2S'
,'T593X3D'
,'T593X3S'
,'T593X4D'
,'T593X4S'
,'T594X1D'
,'T594X1S'
,'T594X2D'
,'T594X2S'
,'T594X3D'
,'T594X3S'
,'T594X4D'
,'T594X4S'
,'T595X1D'
,'T595X1S'
,'T595X2D'
,'T595X2S'
,'T595X3D'
,'T595X3S'
,'T595X4D'
,'T595X4S'
,'T596X1D'
,'T596X1S'
,'T596X2D'
,'T596X2S'
,'T596X3D'
,'T596X3S'
,'T596X4D'
,'T596X4S'
,'T597X1D'
,'T597X1S'
,'T597X2D'
,'T597X2S'
,'T597X3D'
,'T597X3S'
,'T597X4D'
,'T597X4S'
,'T59811D'
,'T59811S'
,'T59812D'
,'T59812S'
,'T59813D'
,'T59813S'
,'T59814D'
,'T59814S'
,'T59891D'
,'T59891S'
,'T59892D'
,'T59892S'
,'T59893D'
,'T59893S'
,'T59894D'
,'T59894S'
,'T5991XD'
,'T5991XS'
,'T5992XD'
,'T5992XS'
,'T5993XD'
,'T5993XS'
,'T5994XD'
,'T5994XS'
,'T600X1D'
,'T600X1S'
,'T600X2D'
,'T600X2S'
,'T600X3D'
,'T600X3S'
,'T600X4D'
,'T600X4S'
,'T601X1D'
,'T601X1S'
,'T601X2D'
,'T601X2S'
,'T601X3D'
,'T601X3S'
,'T601X4D'
,'T601X4S'
,'T602X1D'
,'T602X1S'
,'T602X2D'
,'T602X2S'
,'T602X3D'
,'T602X3S'
,'T602X4D'
,'T602X4S'
,'T603X1D'
,'T603X1S'
,'T603X2D'
,'T603X2S'
,'T603X3D'
,'T603X3S'
,'T603X4D'
,'T603X4S'
,'T604X1D'
,'T604X1S'
,'T604X2D'
,'T604X2S'
,'T604X3D'
,'T604X3S'
,'T604X4D'
,'T604X4S'
,'T608X1D'
,'T608X1S'
,'T608X2D'
,'T608X2S'
,'T608X3D'
,'T608X3S'
,'T608X4D'
,'T608X4S'
,'T6091XD'
,'T6091XS'
,'T6092XD'
,'T6092XS'
,'T6093XD'
,'T6093XS'
,'T6094XD'
,'T6094XS'
,'T6101XD'
,'T6101XS'
,'T6102XD'
,'T6102XS'
,'T6103XD'
,'T6103XS'
,'T6104XD'
,'T6104XS'
,'T6111XD'
,'T6111XS'
,'T6112XD'
,'T6112XS'
,'T6113XD'
,'T6113XS'
,'T6114XD'
,'T6114XS'
,'T61771D'
,'T61771S'
,'T61772D'
,'T61772S'
,'T61773D'
,'T61773S'
,'T61774D'
,'T61774S'
,'T61781D'
,'T61781S'
,'T61782D'
,'T61782S'
,'T61783D'
,'T61783S'
,'T61784D'
,'T61784S'
,'T618X1D'
,'T618X1S'
,'T618X2D'
,'T618X2S'
,'T618X3D'
,'T618X3S'
,'T618X4D'
,'T618X4S'
,'T6191XD'
,'T6191XS'
,'T6192XD'
,'T6192XS'
,'T6193XD'
,'T6193XS'
,'T6194XD'
,'T6194XS'
,'T620X1D'
,'T620X1S'
,'T620X2D'
,'T620X2S'
,'T620X3D'
,'T620X3S'
,'T620X4D'
,'T620X4S'
,'T621X1D'
,'T621X1S'
,'T621X2D'
,'T621X2S'
,'T621X3D'
,'T621X3S'
,'T621X4D'
,'T621X4S'
,'T622X1D'
,'T622X1S'
,'T622X2D'
,'T622X2S'
,'T622X3D'
,'T622X3S'
,'T622X4D'
,'T622X4S'
,'T628X1D'
,'T628X1S'
,'T628X2D'
,'T628X2S'
,'T628X3D'
,'T628X3S'
,'T628X4D'
,'T628X4S'
,'T6291XD'
,'T6291XS'
,'T6292XD'
,'T6292XS'
,'T6293XD'
,'T6293XS'
,'T6294XD'
,'T6294XS'
,'T63001D'
,'T63001S'
,'T63002D'
,'T63002S'
,'T63003D'
,'T63003S'
,'T63004D'
,'T63004S'
,'T63011D'
,'T63011S'
,'T63012D'
,'T63012S'
,'T63013D'
,'T63013S'
,'T63014D'
,'T63014S'
,'T63021D'
,'T63021S'
,'T63022D'
,'T63022S'
,'T63023D'
,'T63023S'
,'T63024D'
,'T63024S'
,'T63031D'
,'T63031S'
,'T63032D'
,'T63032S'
,'T63033D'
,'T63033S'
,'T63034D'
,'T63034S'
,'T63041D'
,'T63041S'
,'T63042D'
,'T63042S'
,'T63043D'
,'T63043S'
,'T63044D'
,'T63044S'
,'T63061D'
,'T63061S'
,'T63062D'
,'T63062S'
,'T63063D'
,'T63063S'
,'T63064D'
,'T63064S'
,'T63071D'
,'T63071S'
,'T63072D'
,'T63072S'
,'T63073D'
,'T63073S'
,'T63074D'
,'T63074S'
,'T63081D'
,'T63081S'
,'T63082D'
,'T63082S'
,'T63083D'
,'T63083S'
,'T63084D'
,'T63084S'
,'T63091D'
,'T63091S'
,'T63092D'
,'T63092S'
,'T63093D'
,'T63093S'
,'T63094D'
,'T63094S'
,'T63111D'
,'T63111S'
,'T63112D'
,'T63112S'
,'T63113D'
,'T63113S'
,'T63114D'
,'T63114S'
,'T63121D'
,'T63121S'
,'T63122D'
,'T63122S'
,'T63123D'
,'T63123S'
,'T63124D'
,'T63124S'
,'T63191D'
,'T63191S'
,'T63192D'
,'T63192S'
,'T63193D'
,'T63193S'
,'T63194D'
,'T63194S'
,'T632X1D'
,'T632X1S'
,'T632X2D'
,'T632X2S'
,'T632X3D'
,'T632X3S'
,'T632X4D'
,'T632X4S'
,'T63301D'
,'T63301S'
,'T63302D'
,'T63302S'
,'T63303D'
,'T63303S'
,'T63304D'
,'T63304S'
,'T63311D'
,'T63311S'
,'T63312D'
,'T63312S'
,'T63313D'
,'T63313S'
,'T63314D'
,'T63314S'
,'T63321D'
,'T63321S'
,'T63322D'
,'T63322S'
,'T63323D'
,'T63323S'
,'T63324D'
,'T63324S'
,'T63331D'
,'T63331S'
,'T63332D'
,'T63332S'
,'T63333D'
,'T63333S'
,'T63334D'
,'T63334S'
,'T63391D'
,'T63391S'
,'T63392D'
,'T63392S'
,'T63393D'
,'T63393S'
,'T63394D'
,'T63394S'
,'T63411D'
,'T63411S'
,'T63412D'
,'T63412S'
,'T63413D'
,'T63413S'
,'T63414D'
,'T63414S'
,'T63421D'
,'T63421S'
,'T63422D'
,'T63422S'
,'T63423D'
,'T63423S'
,'T63424D'
,'T63424S'
,'T63431D'
,'T63431S'
,'T63432D'
,'T63432S'
,'T63433D'
,'T63433S'
,'T63434D'
,'T63434S'
,'T63441D'
,'T63441S'
,'T63442D'
,'T63442S'
,'T63443D'
,'T63443S'
,'T63444D'
,'T63444S'
,'T63451D'
,'T63451S'
,'T63452D'
,'T63452S'
,'T63453D'
,'T63453S'
,'T63454D'
,'T63454S'
,'T63461D'
,'T63461S'
,'T63462D'
,'T63462S'
,'T63463D'
,'T63463S'
,'T63464D'
,'T63464S'
,'T63481D'
,'T63481S'
,'T63482D'
,'T63482S'
,'T63483D'
,'T63483S'
,'T63484D'
,'T63484S'
,'T63511D'
,'T63511S'
,'T63512D'
,'T63512S'
,'T63513D'
,'T63513S'
,'T63514D'
,'T63514S'
,'T63591D'
,'T63591S'
,'T63592D'
,'T63592S'
,'T63593D'
,'T63593S'
,'T63594D'
,'T63594S'
,'T63611D'
,'T63611S'
,'T63612D'
,'T63612S'
,'T63613D'
,'T63613S'
,'T63614D'
,'T63614S'
,'T63621D'
,'T63621S'
,'T63622D'
,'T63622S'
,'T63623D'
,'T63623S'
,'T63624D'
,'T63624S'
,'T63631D'
,'T63631S'
,'T63632D'
,'T63632S'
,'T63633D'
,'T63633S'
,'T63634D'
,'T63634S'
,'T63691D'
,'T63691S'
,'T63692D'
,'T63692S'
,'T63693D'
,'T63693S'
,'T63694D'
,'T63694S'
,'T63711D'
,'T63711S'
,'T63712D'
,'T63712S'
,'T63713D'
,'T63713S'
,'T63714D'
,'T63714S'
,'T63791D'
,'T63791S'
,'T63792D'
,'T63792S'
,'T63793D'
,'T63793S'
,'T63794D'
,'T63794S'
,'T63811D'
,'T63811S'
,'T63812D'
,'T63812S'
,'T63813D'
,'T63813S'
,'T63814D'
,'T63814S'
,'T63821D'
,'T63821S'
,'T63822D'
,'T63822S'
,'T63823D'
,'T63823S'
,'T63824D'
,'T63824S'
,'T63831D'
,'T63831S'
,'T63832D'
,'T63832S'
,'T63833D'
,'T63833S'
,'T63834D'
,'T63834S'
,'T63891D'
,'T63891S'
,'T63892D'
,'T63892S'
,'T63893D'
,'T63893S'
,'T63894D'
,'T63894S'
,'T6391XD'
,'T6391XS'
,'T6392XD'
,'T6392XS'
,'T6393XD'
,'T6393XS'
,'T6394XD'
,'T6394XS'
,'T6401XD'
,'T6401XS'
,'T6402XD'
,'T6402XS'
,'T6403XD'
,'T6403XS'
,'T6404XD'
,'T6404XS'
,'T6481XD'
,'T6481XS'
,'T6482XD'
,'T6482XS'
,'T6483XD'
,'T6483XS'
,'T6484XD'
,'T6484XS'
,'T650X1D'
,'T650X1S'
,'T650X2D'
,'T650X2S'
,'T650X3D'
,'T650X3S'
,'T650X4D'
,'T650X4S'
,'T651X1D'
,'T651X1S'
,'T651X2D'
,'T651X2S'
,'T651X3D'
,'T651X3S'
,'T651X4D'
,'T651X4S'
,'T65211D'
,'T65211S'
,'T65212D'
,'T65212S'
,'T65213D'
,'T65213S'
,'T65214D'
,'T65214S'
,'T65221D'
,'T65221S'
,'T65222D'
,'T65222S'
,'T65223D'
,'T65223S'
,'T65224D'
,'T65224S'
,'T65291D'
,'T65291S'
,'T65292D'
,'T65292S'
,'T65293D'
,'T65293S'
,'T65294D'
,'T65294S'
,'T653X1D'
,'T653X1S'
,'T653X2D'
,'T653X2S'
,'T653X3D'
,'T653X3S'
,'T653X4D'
,'T653X4S'
,'T654X1D'
,'T654X1S'
,'T654X2D'
,'T654X2S'
,'T654X3D'
,'T654X3S'
,'T654X4D'
,'T654X4S'
,'T655X1D'
,'T655X1S'
,'T655X2D'
,'T655X2S'
,'T655X3D'
,'T655X3S'
,'T655X4D'
,'T655X4S'
,'T656X1D'
,'T656X1S'
,'T656X2D'
,'T656X2S'
,'T656X3D'
,'T656X3S'
,'T656X4D'
,'T656X4S'
,'T65811D'
,'T65811S'
,'T65812D'
,'T65812S'
,'T65813D'
,'T65813S'
,'T65814D'
,'T65814S'
,'T65821D'
,'T65821S'
,'T65822D'
,'T65822S'
,'T65823D'
,'T65823S'
,'T65824D'
,'T65824S'
,'T65831D'
,'T65831S'
,'T65832D'
,'T65832S'
,'T65833D'
,'T65833S'
,'T65834D'
,'T65834S'
,'T65891D'
,'T65891S'
,'T65892D'
,'T65892S'
,'T65893D'
,'T65893S'
,'T65894D'
,'T65894S'
,'T6591XD'
,'T6591XS'
,'T6592XD'
,'T6592XS'
,'T6593XD'
,'T6593XS'
,'T6594XD'
,'T6594XS'
,'T66XXXD'
,'T66XXXS'
,'T670XXD'
,'T670XXS'
,'T671XXD'
,'T671XXS'
,'T672XXD'
,'T672XXS'
,'T673XXD'
,'T673XXS'
,'T674XXD'
,'T674XXS'
,'T675XXD'
,'T675XXS'
,'T676XXD'
,'T676XXS'
,'T677XXD'
,'T677XXS'
,'T678XXD'
,'T678XXS'
,'T679XXD'
,'T679XXS'
,'T68XXXD'
,'T68XXXS'
,'T69011D'
,'T69011S'
,'T69012D'
,'T69012S'
,'T69019D'
,'T69019S'
,'T69021D'
,'T69021S'
,'T69022D'
,'T69022S'
,'T69029D'
,'T69029S'
,'T691XXD'
,'T691XXS'
,'T698XXD'
,'T698XXS'
,'T699XXD'
,'T699XXS'
,'T700XXD'
,'T700XXS'
,'T701XXD'
,'T701XXS'
,'T7020XD'
,'T7020XS'
,'T7029XD'
,'T7029XS'
,'T703XXD'
,'T703XXS'
,'T704XXD'
,'T704XXS'
,'T708XXD'
,'T708XXS'
,'T709XXD'
,'T709XXS'
,'T71111D'
,'T71111S'
,'T71112D'
,'T71112S'
,'T71113D'
,'T71113S'
,'T71114D'
,'T71114S'
,'T71121D'
,'T71121S'
,'T71122D'
,'T71122S'
,'T71123D'
,'T71123S'
,'T71124D'
,'T71124S'
,'T71131D'
,'T71131S'
,'T71132D'
,'T71132S'
,'T71133D'
,'T71133S'
,'T71134D'
,'T71134S'
,'T71141D'
,'T71141S'
,'T71143D'
,'T71143S'
,'T71144D'
,'T71144S'
,'T71151D'
,'T71151S'
,'T71152D'
,'T71152S'
,'T71153D'
,'T71153S'
,'T71154D'
,'T71154S'
,'T71161D'
,'T71161S'
,'T71162D'
,'T71162S'
,'T71163D'
,'T71163S'
,'T71164D'
,'T71164S'
,'T71191D'
,'T71191S'
,'T71192D'
,'T71192S'
,'T71193D'
,'T71193S'
,'T71194D'
,'T71194S'
,'T7120XD'
,'T7120XS'
,'T7121XD'
,'T7121XS'
,'T71221D'
,'T71221S'
,'T71222D'
,'T71222S'
,'T71223D'
,'T71223S'
,'T71224D'
,'T71224S'
,'T71231D'
,'T71231S'
,'T71232D'
,'T71232S'
,'T71233D'
,'T71233S'
,'T71234D'
,'T71234S'
,'T7129XD'
,'T7129XS'
,'T719XXD'
,'T719XXS'
,'T730XXD'
,'T730XXS'
,'T731XXD'
,'T731XXS'
,'T732XXD'
,'T732XXS'
,'T733XXD'
,'T733XXS'
,'T738XXD'
,'T738XXS'
,'T739XXD'
,'T739XXS'
,'T7401XD'
,'T7401XS'
,'T7402XD'
,'T7402XS'
,'T7411XD'
,'T7411XS'
,'T7412XD'
,'T7412XS'
,'T7421XD'
,'T7421XS'
,'T7422XD'
,'T7422XS'
,'T7431XD'
,'T7431XS'
,'T7432XD'
,'T7432XS'
,'T744XXD'
,'T744XXS'
,'T7491XD'
,'T7491XS'
,'T7492XD'
,'T7492XS'
,'T7500XD'
,'T7500XS'
,'T7501XD'
,'T7501XS'
,'T7509XD'
,'T7509XS'
,'T751XXD'
,'T751XXS'
,'T7520XD'
,'T7520XS'
,'T7521XD'
,'T7521XS'
,'T7522XD'
,'T7522XS'
,'T7523XD'
,'T7523XS'
,'T7529XD'
,'T7529XS'
,'T753XXD'
,'T753XXS'
,'T754XXD'
,'T754XXS'
,'T7581XD'
,'T7581XS'
,'T7582XD'
,'T7582XS'
,'T7589XD'
,'T7589XS'
,'T7601XD'
,'T7601XS'
,'T7602XD'
,'T7602XS'
,'T7611XD'
,'T7611XS'
,'T7612XD'
,'T7612XS'
,'T7621XD'
,'T7621XS'
,'T7622XD'
,'T7622XS'
,'T7631XD'
,'T7631XS'
,'T7632XD'
,'T7632XS'
,'T7691XD'
,'T7691XS'
,'T7692XD'
,'T7692XS'
,'T7800XD'
,'T7800XS'
,'T7801XD'
,'T7801XS'
,'T7802XD'
,'T7802XS'
,'T7803XD'
,'T7803XS'
,'T7804XD'
,'T7804XS'
,'T7805XD'
,'T7805XS'
,'T7806XD'
,'T7806XS'
,'T7807XD'
,'T7807XS'
,'T7808XD'
,'T7808XS'
,'T7809XD'
,'T7809XS'
,'T781XXD'
,'T781XXS'
,'T782XXD'
,'T782XXS'
,'T783XXD'
,'T783XXS'
,'T7840XD'
,'T7840XS'
,'T7841XD'
,'T7841XS'
,'T7849XD'
,'T7849XS'
,'T788XXD'
,'T788XXS'
,'T790XXD'
,'T790XXS'
,'T791XXD'
,'T791XXS'
,'T792XXD'
,'T792XXS'
,'T794XXD'
,'T794XXS'
,'T795XXD'
,'T795XXS'
,'T796XXD'
,'T796XXS'
,'T797XXD'
,'T797XXS'
,'T798XXD'
,'T798XXS'
,'T799XXD'
,'T799XXS'
,'T79A0XD'
,'T79A0XS'
,'T79A11D'
,'T79A11S'
,'T79A12D'
,'T79A12S'
,'T79A19D'
,'T79A19S'
,'T79A21D'
,'T79A21S'
,'T79A22D'
,'T79A22S'
,'T79A29D'
,'T79A29S'
,'T79A3XD'
,'T79A3XS'
,'T79A9XD'
,'T79A9XS'
,'T800XXD'
,'T800XXS'
,'T801XXD'
,'T801XXS'
,'T80211D'
,'T80211S'
,'T80212D'
,'T80212S'
,'T80218D'
,'T80218S'
,'T80219D'
,'T80219S'
,'T8022XD'
,'T8022XS'
,'T8029XD'
,'T8029XS'
,'T8030XD'
,'T8030XS'
,'T80310D'
,'T80310S'
,'T80311D'
,'T80311S'
,'T80319D'
,'T80319S'
,'T8039XD'
,'T8039XS'
,'T8040XD'
,'T8040XS'
,'T80410D'
,'T80410S'
,'T80411D'
,'T80411S'
,'T80419D'
,'T80419S'
,'T8049XD'
,'T8049XS'
,'T8051XD'
,'T8051XS'
,'T8052XD'
,'T8052XS'
,'T8059XD'
,'T8059XS'
,'T8061XD'
,'T8061XS'
,'T8062XD'
,'T8062XS'
,'T8069XD'
,'T8069XS'
,'T80810D'
,'T80810S'
,'T80818D'
,'T80818S'
,'T8089XD'
,'T8089XS'
,'T8090XD'
,'T8090XS'
,'T80910D'
,'T80910S'
,'T80911D'
,'T80911S'
,'T80919D'
,'T80919S'
,'T8092XD'
,'T8092XS'
,'T80A0XD'
,'T80A0XS'
,'T80A10D'
,'T80A10S'
,'T80A11D'
,'T80A11S'
,'T80A19D'
,'T80A19S'
,'T80A9XD'
,'T80A9XS'
,'T8110XD'
,'T8110XS'
,'T8111XD'
,'T8111XS'
,'T8112XD'
,'T8112XS'
,'T8119XD'
,'T8119XS'
,'T8130XD'
,'T8130XS'
,'T8131XD'
,'T8131XS'
,'T8132XD'
,'T8132XS'
,'T8133XD'
,'T8133XS'
,'T814XXD'
,'T814XXS'
,'T81500D'
,'T81500S'
,'T81501D'
,'T81501S'
,'T81502D'
,'T81502S'
,'T81503D'
,'T81503S'
,'T81504D'
,'T81504S'
,'T81505D'
,'T81505S'
,'T81506D'
,'T81506S'
,'T81507D'
,'T81507S'
,'T81508D'
,'T81508S'
,'T81509D'
,'T81509S'
,'T81510D'
,'T81510S'
,'T81511D'
,'T81511S'
,'T81512D'
,'T81512S'
,'T81513D'
,'T81513S'
,'T81514D'
,'T81514S'
,'T81515D'
,'T81515S'
,'T81516D'
,'T81516S'
,'T81517D'
,'T81517S'
,'T81518D'
,'T81518S'
,'T81519D'
,'T81519S'
,'T81520D'
,'T81520S'
,'T81521D'
,'T81521S'
,'T81522D'
,'T81522S'
,'T81523D'
,'T81523S'
,'T81524D'
,'T81524S'
,'T81525D'
,'T81525S'
,'T81526D'
,'T81526S'
,'T81527D'
,'T81527S'
,'T81528D'
,'T81528S'
,'T81529D'
,'T81529S'
,'T81530D'
,'T81530S'
,'T81531D'
,'T81531S'
,'T81532D'
,'T81532S'
,'T81533D'
,'T81533S'
,'T81534D'
,'T81534S'
,'T81535D'
,'T81535S'
,'T81536D'
,'T81536S'
,'T81537D'
,'T81537S'
,'T81538D'
,'T81538S'
,'T81539D'
,'T81539S'
,'T81590D'
,'T81590S'
,'T81591D'
,'T81591S'
,'T81592D'
,'T81592S'
,'T81593D'
,'T81593S'
,'T81594D'
,'T81594S'
,'T81595D'
,'T81595S'
,'T81596D'
,'T81596S'
,'T81597D'
,'T81597S'
,'T81598D'
,'T81598S'
,'T81599D'
,'T81599S'
,'T8160XD'
,'T8160XS'
,'T8161XD'
,'T8161XS'
,'T8169XD'
,'T8169XS'
,'T81710D'
,'T81710S'
,'T81711D'
,'T81711S'
,'T81718D'
,'T81718S'
,'T81719D'
,'T81719S'
,'T8172XD'
,'T8172XS'
,'T8181XD'
,'T8181XS'
,'T8182XD'
,'T8182XS'
,'T8183XD'
,'T8183XS'
,'T8189XD'
,'T8189XS'
,'T819XXD'
,'T819XXS'
,'T8201XD'
,'T8201XS'
,'T8202XD'
,'T8202XS'
,'T8203XD'
,'T8203XS'
,'T8209XD'
,'T8209XS'
,'T82110D'
,'T82110S'
,'T82111D'
,'T82111S'
,'T82118D'
,'T82118S'
,'T82119D'
,'T82119S'
,'T82120D'
,'T82120S'
,'T82121D'
,'T82121S'
,'T82128D'
,'T82128S'
,'T82129D'
,'T82129S'
,'T82190D'
,'T82190S'
,'T82191D'
,'T82191S'
,'T82198D'
,'T82198S'
,'T82199D'
,'T82199S'
,'T82211D'
,'T82211S'
,'T82212D'
,'T82212S'
,'T82213D'
,'T82213S'
,'T82218D'
,'T82218S'
,'T82221D'
,'T82221S'
,'T82222D'
,'T82222S'
,'T82223D'
,'T82223S'
,'T82228D'
,'T82228S'
,'T82310D'
,'T82310S'
,'T82311D'
,'T82311S'
,'T82312D'
,'T82312S'
,'T82318D'
,'T82318S'
,'T82319D'
,'T82319S'
,'T82320D'
,'T82320S'
,'T82321D'
,'T82321S'
,'T82322D'
,'T82322S'
,'T82328D'
,'T82328S'
,'T82329D'
,'T82329S'
,'T82330D'
,'T82330S'
,'T82331D'
,'T82331S'
,'T82332D'
,'T82332S'
,'T82338D'
,'T82338S'
,'T82339D'
,'T82339S'
,'T82390D'
,'T82390S'
,'T82391D'
,'T82391S'
,'T82392D'
,'T82392S'
,'T82398D'
,'T82398S'
,'T82399D'
,'T82399S'
,'T8241XD'
,'T8241XS'
,'T8242XD'
,'T8242XS'
,'T8243XD'
,'T8243XS'
,'T8249XD'
,'T8249XS'
,'T82510D'
,'T82510S'
,'T82511D'
,'T82511S'
,'T82512D'
,'T82512S'
,'T82513D'
,'T82513S'
,'T82514D'
,'T82514S'
,'T82515D'
,'T82515S'
,'T82518D'
,'T82518S'
,'T82519D'
,'T82519S'
,'T82520D'
,'T82520S'
,'T82521D'
,'T82521S'
,'T82522D'
,'T82522S'
,'T82523D'
,'T82523S'
,'T82524D'
,'T82524S'
,'T82525D'
,'T82525S'
,'T82528D'
,'T82528S'
,'T82529D'
,'T82529S'
,'T82530D'
,'T82530S'
,'T82531D'
,'T82531S'
,'T82532D'
,'T82532S'
,'T82533D'
,'T82533S'
,'T82534D'
,'T82534S'
,'T82535D'
,'T82535S'
,'T82538D'
,'T82538S'
,'T82539D'
,'T82539S'
,'T82590D'
,'T82590S'
,'T82591D'
,'T82591S'
,'T82592D'
,'T82592S'
,'T82593D'
,'T82593S'
,'T82594D'
,'T82594S'
,'T82595D'
,'T82595S'
,'T82598D'
,'T82598S'
,'T82599D'
,'T82599S'
,'T826XXD'
,'T826XXS'
,'T827XXD'
,'T827XXS'
,'T82817D'
,'T82817S'
,'T82818D'
,'T82818S'
,'T82827D'
,'T82827S'
,'T82828D'
,'T82828S'
,'T82837D'
,'T82837S'
,'T82838D'
,'T82838S'
,'T82847D'
,'T82847S'
,'T82848D'
,'T82848S'
,'T82857D'
,'T82857S'
,'T82858D'
,'T82858S'
,'T82867D'
,'T82867S'
,'T82868D'
,'T82868S'
,'T82897D'
,'T82897S'
,'T82898D'
,'T82898S'
,'T829XXD'
,'T829XXS'
,'T83010D'
,'T83010S'
,'T83018D'
,'T83018S'
,'T83020D'
,'T83020S'
,'T83028D'
,'T83028S'
,'T83030D'
,'T83030S'
,'T83038D'
,'T83038S'
,'T83090D'
,'T83090S'
,'T83098D'
,'T83098S'
,'T83110D'
,'T83110S'
,'T83111D'
,'T83111S'
,'T83112D'
,'T83112S'
,'T83118D'
,'T83118S'
,'T83120D'
,'T83120S'
,'T83121D'
,'T83121S'
,'T83122D'
,'T83122S'
,'T83128D'
,'T83128S'
,'T83190D'
,'T83190S'
,'T83191D'
,'T83191S'
,'T83192D'
,'T83192S'
,'T83198D'
,'T83198S'
,'T8321XD'
,'T8321XS'
,'T8322XD'
,'T8322XS'
,'T8323XD'
,'T8323XS'
,'T8329XD'
,'T8329XS'
,'T8331XD'
,'T8331XS'
,'T8332XD'
,'T8332XS'
,'T8339XD'
,'T8339XS'
,'T83410D'
,'T83410S'
,'T83418D'
,'T83418S'
,'T83420D'
,'T83420S'
,'T83428D'
,'T83428S'
,'T83490D'
,'T83490S'
,'T83498D'
,'T83498S'
,'T8351XD'
,'T8351XS'
,'T8359XD'
,'T8359XS'
,'T836XXD'
,'T836XXS'
,'T83711D'
,'T83711S'
,'T83718D'
,'T83718S'
,'T83721D'
,'T83721S'
,'T83728D'
,'T83728S'
,'T8381XD'
,'T8381XS'
,'T8382XD'
,'T8382XS'
,'T8383XD'
,'T8383XS'
,'T8384XD'
,'T8384XS'
,'T8385XD'
,'T8385XS'
,'T8386XD'
,'T8386XS'
,'T8389XD'
,'T8389XS'
,'T839XXD'
,'T839XXS'
,'T84010D'
,'T84010S'
,'T84011D'
,'T84011S'
,'T84012D'
,'T84012S'
,'T84013D'
,'T84013S'
,'T84018D'
,'T84018S'
,'T84019D'
,'T84019S'
,'T84020D'
,'T84020S'
,'T84021D'
,'T84021S'
,'T84022D'
,'T84022S'
,'T84023D'
,'T84023S'
,'T84028D'
,'T84028S'
,'T84029D'
,'T84029S'
,'T84030D'
,'T84030S'
,'T84031D'
,'T84031S'
,'T84032D'
,'T84032S'
,'T84033D'
,'T84033S'
,'T84038D'
,'T84038S'
,'T84039D'
,'T84039S'
,'T84040D'
,'T84040S'
,'T84041D'
,'T84041S'
,'T84042D'
,'T84042S'
,'T84043D'
,'T84043S'
,'T84048D'
,'T84048S'
,'T84049D'
,'T84049S'
,'T84050D'
,'T84050S'
,'T84051D'
,'T84051S'
,'T84052D'
,'T84052S'
,'T84053D'
,'T84053S'
,'T84058D'
,'T84058S'
,'T84059D'
,'T84059S'
,'T84060D'
,'T84060S'
,'T84061D'
,'T84061S'
,'T84062D'
,'T84062S'
,'T84063D'
,'T84063S'
,'T84068D'
,'T84068S'
,'T84069D'
,'T84069S'
,'T84090D'
,'T84090S'
,'T84091D'
,'T84091S'
,'T84092D'
,'T84092S'
,'T84093D'
,'T84093S'
,'T84098D'
,'T84098S'
,'T84099D'
,'T84099S'
,'T84110D'
,'T84110S'
,'T84111D'
,'T84111S'
,'T84112D'
,'T84112S'
,'T84113D'
,'T84113S'
,'T84114D'
,'T84114S'
,'T84115D'
,'T84115S'
,'T84116D'
,'T84116S'
,'T84117D'
,'T84117S'
,'T84119D'
,'T84119S'
,'T84120D'
,'T84120S'
,'T84121D'
,'T84121S'
,'T84122D'
,'T84122S'
,'T84123D'
,'T84123S'
,'T84124D'
,'T84124S'
,'T84125D'
,'T84125S'
,'T84126D'
,'T84126S'
,'T84127D'
,'T84127S'
,'T84129D'
,'T84129S'
,'T84190D'
,'T84190S'
,'T84191D'
,'T84191S'
,'T84192D'
,'T84192S'
,'T84193D'
,'T84193S'
,'T84194D'
,'T84194S'
,'T84195D'
,'T84195S'
,'T84196D'
,'T84196S'
,'T84197D'
,'T84197S'
,'T84199D'
,'T84199S'
,'T84210D'
,'T84210S'
,'T84213D'
,'T84213S'
,'T84216D'
,'T84216S'
,'T84218D'
,'T84218S'
,'T84220D'
,'T84220S'
,'T84223D'
,'T84223S'
,'T84226D'
,'T84226S'
,'T84228D'
,'T84228S'
,'T84290D'
,'T84290S'
,'T84293D'
,'T84293S'
,'T84296D'
,'T84296S'
,'T84298D'
,'T84298S'
,'T84310D'
,'T84310S'
,'T84318D'
,'T84318S'
,'T84320D'
,'T84320S'
,'T84328D'
,'T84328S'
,'T84390D'
,'T84390S'
,'T84398D'
,'T84398S'
,'T84410D'
,'T84410S'
,'T84418D'
,'T84418S'
,'T84420D'
,'T84420S'
,'T84428D'
,'T84428S'
,'T84490D'
,'T84490S'
,'T84498D'
,'T84498S'
,'T8450XD'
,'T8450XS'
,'T8451XD'
,'T8451XS'
,'T8452XD'
,'T8452XS'
,'T8453XD'
,'T8453XS'
,'T8454XD'
,'T8454XS'
,'T8459XD'
,'T8459XS'
,'T8460XD'
,'T8460XS'
,'T84610D'
,'T84610S'
,'T84611D'
,'T84611S'
,'T84612D'
,'T84612S'
,'T84613D'
,'T84613S'
,'T84614D'
,'T84614S'
,'T84615D'
,'T84615S'
,'T84619D'
,'T84619S'
,'T84620D'
,'T84620S'
,'T84621D'
,'T84621S'
,'T84622D'
,'T84622S'
,'T84623D'
,'T84623S'
,'T84624D'
,'T84624S'
,'T84625D'
,'T84625S'
,'T84629D'
,'T84629S'
,'T8463XD'
,'T8463XS'
,'T8469XD'
,'T8469XS'
,'T847XXD'
,'T847XXS'
,'T8481XD'
,'T8481XS'
,'T8482XD'
,'T8482XS'
,'T8483XD'
,'T8483XS'
,'T8484XD'
,'T8484XS'
,'T8485XD'
,'T8485XS'
,'T8486XD'
,'T8486XS'
,'T8489XD'
,'T8489XS'
,'T849XXD'
,'T849XXS'
,'T8501XD'
,'T8501XS'
,'T8502XD'
,'T8502XS'
,'T8503XD'
,'T8503XS'
,'T8509XD'
,'T8509XS'
,'T85110D'
,'T85110S'
,'T85111D'
,'T85111S'
,'T85112D'
,'T85112S'
,'T85118D'
,'T85118S'
,'T85120D'
,'T85120S'
,'T85121D'
,'T85121S'
,'T85122D'
,'T85122S'
,'T85128D'
,'T85128S'
,'T85190D'
,'T85190S'
,'T85191D'
,'T85191S'
,'T85192D'
,'T85192S'
,'T85199D'
,'T85199S'
,'T8521XD'
,'T8521XS'
,'T8522XD'
,'T8522XS'
,'T8529XD'
,'T8529XS'
,'T85310D'
,'T85310S'
,'T85311D'
,'T85311S'
,'T85318D'
,'T85318S'
,'T85320D'
,'T85320S'
,'T85321D'
,'T85321S'
,'T85328D'
,'T85328S'
,'T85390D'
,'T85390S'
,'T85391D'
,'T85391S'
,'T85398D'
,'T85398S'
,'T8541XD'
,'T8541XS'
,'T8542XD'
,'T8542XS'
,'T8543XD'
,'T8543XS'
,'T8544XD'
,'T8544XS'
,'T8549XD'
,'T8549XS'
,'T85510D'
,'T85510S'
,'T85511D'
,'T85511S'
,'T85518D'
,'T85518S'
,'T85520D'
,'T85520S'
,'T85521D'
,'T85521S'
,'T85528D'
,'T85528S'
,'T85590D'
,'T85590S'
,'T85591D'
,'T85591S'
,'T85598D'
,'T85598S'
,'T85610D'
,'T85610S'
,'T85611D'
,'T85611S'
,'T85612D'
,'T85612S'
,'T85613D'
,'T85613S'
,'T85614D'
,'T85614S'
,'T85618D'
,'T85618S'
,'T85620D'
,'T85620S'
,'T85621D'
,'T85621S'
,'T85622D'
,'T85622S'
,'T85623D'
,'T85623S'
,'T85624D'
,'T85624S'
,'T85628D'
,'T85628S'
,'T85630D'
,'T85630S'
,'T85631D'
,'T85631S'
,'T85633D'
,'T85633S'
,'T85638D'
,'T85638S'
,'T85690D'
,'T85690S'
,'T85691D'
,'T85691S'
,'T85692D'
,'T85692S'
,'T85693D'
,'T85693S'
,'T85694D'
,'T85694S'
,'T85698D'
,'T85698S'
,'T8571XD'
,'T8571XS'
,'T8572XD'
,'T8572XS'
,'T8579XD'
,'T8579XS'
,'T8581XD'
,'T8581XS'
,'T8582XD'
,'T8582XS'
,'T8583XD'
,'T8583XS'
,'T8584XD'
,'T8584XS'
,'T8585XD'
,'T8585XS'
,'T8586XD'
,'T8586XS'
,'T8589XD'
,'T8589XS'
,'T859XXD'
,'T859XXS'
,'T8600'
,'T8601'
,'T8602'
,'T8603'
,'T8609'
,'T8610'
,'T8611'
,'T8612'
,'T8613'
,'T8619'
,'T8620'
,'T8621'
,'T8622'
,'T8623'
,'T86290'
,'T86298'
,'T8630'
,'T8631'
,'T8632'
,'T8633'
,'T8639'
,'T8640'
,'T8641'
,'T8642'
,'T8643'
,'T8649'
,'T865'
,'T86810'
,'T86811'
,'T86812'
,'T86818'
,'T86819'
,'T86820'
,'T86821'
,'T86822'
,'T86828'
,'T86829'
,'T86830'
,'T86831'
,'T86832'
,'T86838'
,'T86839'
,'T86840'
,'T86841'
,'T86842'
,'T86848'
,'T86849'
,'T86850'
,'T86851'
,'T86852'
,'T86858'
,'T86859'
,'T86890'
,'T86891'
,'T86892'
,'T86898'
,'T86899'
,'T8690'
,'T8691'
,'T8692'
,'T8693'
,'T8699'
,'T870X1'
,'T870X2'
,'T870X9'
,'T871X1'
,'T871X2'
,'T871X9'
,'T872'
,'T8730'
,'T8731'
,'T8732'
,'T8733'
,'T8734'
,'T8740'
,'T8741'
,'T8742'
,'T8743'
,'T8744'
,'T8750'
,'T8751'
,'T8752'
,'T8753'
,'T8754'
,'T8781'
,'T8789'
,'T879'
,'T880XXD'
,'T880XXS'
,'T881XXD'
,'T881XXS'
,'T882XXD'
,'T882XXS'
,'T883XXD'
,'T883XXS'
,'T884XXD'
,'T884XXS'
,'T8851XD'
,'T8851XS'
,'T8852XD'
,'T8852XS'
,'T8859XD'
,'T8859XS'
,'T886XXD'
,'T886XXS'
,'T887XXD'
,'T887XXS'
,'T888XXD'
,'T888XXS'
,'T889XXD'
,'T889XXS'
,'V0001XD'
,'V0001XS'
,'V0002XD'
,'V0002XS'
,'V0009XD'
,'V0009XS'
,'V00111D'
,'V00111S'
,'V00112D'
,'V00112S'
,'V00118D'
,'V00118S'
,'V00121D'
,'V00121S'
,'V00122D'
,'V00122S'
,'V00128D'
,'V00128S'
,'V00131D'
,'V00131S'
,'V00132D'
,'V00132S'
,'V00138D'
,'V00138S'
,'V00141D'
,'V00141S'
,'V00142D'
,'V00142S'
,'V00148D'
,'V00148S'
,'V00151D'
,'V00151S'
,'V00152D'
,'V00152S'
,'V00158D'
,'V00158S'
,'V00181D'
,'V00181S'
,'V00182D'
,'V00182S'
,'V00188D'
,'V00188S'
,'V00211D'
,'V00211S'
,'V00212D'
,'V00212S'
,'V00218D'
,'V00218S'
,'V00221D'
,'V00221S'
,'V00222D'
,'V00222S'
,'V00228D'
,'V00228S'
,'V00281D'
,'V00281S'
,'V00282D'
,'V00282S'
,'V00288D'
,'V00288S'
,'V00311D'
,'V00311S'
,'V00312D'
,'V00312S'
,'V00318D'
,'V00318S'
,'V00321D'
,'V00321S'
,'V00322D'
,'V00322S'
,'V00328D'
,'V00328S'
,'V00381D'
,'V00381S'
,'V00382D'
,'V00382S'
,'V00388D'
,'V00388S'
,'V00811D'
,'V00811S'
,'V00812D'
,'V00812S'
,'V00818D'
,'V00818S'
,'V00821D'
,'V00821S'
,'V00822D'
,'V00822S'
,'V00828D'
,'V00828S'
,'V00831D'
,'V00831S'
,'V00832D'
,'V00832S'
,'V00838D'
,'V00838S'
,'V00891D'
,'V00891S'
,'V00892D'
,'V00892S'
,'V00898D'
,'V00898S'
,'V0100XD'
,'V0100XS'
,'V0101XD'
,'V0101XS'
,'V0102XD'
,'V0102XS'
,'V0109XD'
,'V0109XS'
,'V0110XD'
,'V0110XS'
,'V0111XD'
,'V0111XS'
,'V0112XD'
,'V0112XS'
,'V0119XD'
,'V0119XS'
,'V0190XD'
,'V0190XS'
,'V0191XD'
,'V0191XS'
,'V0192XD'
,'V0192XS'
,'V0199XD'
,'V0199XS'
,'V0200XD'
,'V0200XS'
,'V0201XD'
,'V0201XS'
,'V0202XD'
,'V0202XS'
,'V0209XD'
,'V0209XS'
,'V0210XD'
,'V0210XS'
,'V0211XD'
,'V0211XS'
,'V0212XD'
,'V0212XS'
,'V0219XD'
,'V0219XS'
,'V0290XD'
,'V0290XS'
,'V0291XD'
,'V0291XS'
,'V0292XD'
,'V0292XS'
,'V0299XD'
,'V0299XS'
,'V0300XD'
,'V0300XS'
,'V0301XD'
,'V0301XS'
,'V0302XD'
,'V0302XS'
,'V0309XD'
,'V0309XS'
,'V0310XD'
,'V0310XS'
,'V0311XD'
,'V0311XS'
,'V0312XD'
,'V0312XS'
,'V0319XD'
,'V0319XS'
,'V0390XD'
,'V0390XS'
,'V0391XD'
,'V0391XS'
,'V0392XD'
,'V0392XS'
,'V0399XD'
,'V0399XS'
,'V0400XD'
,'V0400XS'
,'V0401XD'
,'V0401XS'
,'V0402XD'
,'V0402XS'
,'V0409XD'
,'V0409XS'
,'V0410XD'
,'V0410XS'
,'V0411XD'
,'V0411XS'
,'V0412XD'
,'V0412XS'
,'V0419XD'
,'V0419XS'
,'V0490XD'
,'V0490XS'
,'V0491XD'
,'V0491XS'
,'V0492XD'
,'V0492XS'
,'V0499XD'
,'V0499XS'
,'V0500XD'
,'V0500XS'
,'V0501XD'
,'V0501XS'
,'V0502XD'
,'V0502XS'
,'V0509XD'
,'V0509XS'
,'V0510XD'
,'V0510XS'
,'V0511XD'
,'V0511XS'
,'V0512XD'
,'V0512XS'
,'V0519XD'
,'V0519XS'
,'V0590XD'
,'V0590XS'
,'V0591XD'
,'V0591XS'
,'V0592XD'
,'V0592XS'
,'V0599XD'
,'V0599XS'
,'V0600XD'
,'V0600XS'
,'V0601XD'
,'V0601XS'
,'V0602XD'
,'V0602XS'
,'V0609XD'
,'V0609XS'
,'V0610XD'
,'V0610XS'
,'V0611XD'
,'V0611XS'
,'V0612XD'
,'V0612XS'
,'V0619XD'
,'V0619XS'
,'V0690XD'
,'V0690XS'
,'V0691XD'
,'V0691XS'
,'V0692XD'
,'V0692XS'
,'V0699XD'
,'V0699XS'
,'V0900XD'
,'V0900XS'
,'V0901XD'
,'V0901XS'
,'V0909XD'
,'V0909XS'
,'V091XXD'
,'V091XXS'
,'V0920XD'
,'V0920XS'
,'V0921XD'
,'V0921XS'
,'V0929XD'
,'V0929XS'
,'V093XXD'
,'V093XXS'
,'V099XXD'
,'V099XXS'
,'V100XXD'
,'V100XXS'
,'V101XXD'
,'V101XXS'
,'V102XXD'
,'V102XXS'
,'V103XXD'
,'V103XXS'
,'V104XXD'
,'V104XXS'
,'V105XXD'
,'V105XXS'
,'V109XXD'
,'V109XXS'
,'V110XXD'
,'V110XXS'
,'V111XXD'
,'V111XXS'
,'V112XXD'
,'V112XXS'
,'V113XXD'
,'V113XXS'
,'V114XXD'
,'V114XXS'
,'V115XXD'
,'V115XXS'
,'V119XXD'
,'V119XXS'
,'V120XXD'
,'V120XXS'
,'V121XXD'
,'V121XXS'
,'V122XXD'
,'V122XXS'
,'V123XXD'
,'V123XXS'
,'V124XXD'
,'V124XXS'
,'V125XXD'
,'V125XXS'
,'V129XXD'
,'V129XXS'
,'V130XXD'
,'V130XXS'
,'V131XXD'
,'V131XXS'
,'V132XXD'
,'V132XXS'
,'V133XXD'
,'V133XXS'
,'V134XXD'
,'V134XXS'
,'V135XXD'
,'V135XXS'
,'V139XXD'
,'V139XXS'
,'V140XXD'
,'V140XXS'
,'V141XXD'
,'V141XXS'
,'V142XXD'
,'V142XXS'
,'V143XXD'
,'V143XXS'
,'V144XXD'
,'V144XXS'
,'V145XXD'
,'V145XXS'
,'V149XXD'
,'V149XXS'
,'V150XXD'
,'V150XXS'
,'V151XXD'
,'V151XXS'
,'V152XXD'
,'V152XXS'
,'V153XXD'
,'V153XXS'
,'V154XXD'
,'V154XXS'
,'V155XXD'
,'V155XXS'
,'V159XXD'
,'V159XXS'
,'V160XXD'
,'V160XXS'
,'V161XXD'
,'V161XXS'
,'V162XXD'
,'V162XXS'
,'V163XXD'
,'V163XXS'
,'V164XXD'
,'V164XXS'
,'V165XXD'
,'V165XXS'
,'V169XXD'
,'V169XXS'
,'V170XXD'
,'V170XXS'
,'V171XXD'
,'V171XXS'
,'V172XXD'
,'V172XXS'
,'V173XXD'
,'V173XXS'
,'V174XXD'
,'V174XXS'
,'V175XXD'
,'V175XXS'
,'V179XXD'
,'V179XXS'
,'V180XXD'
,'V180XXS'
,'V181XXD'
,'V181XXS'
,'V182XXD'
,'V182XXS'
,'V183XXD'
,'V183XXS'
,'V184XXD'
,'V184XXS'
,'V185XXD'
,'V185XXS'
,'V189XXD'
,'V189XXS'
,'V1900XD'
,'V1900XS'
,'V1909XD'
,'V1909XS'
,'V1910XD'
,'V1910XS'
,'V1919XD'
,'V1919XS'
,'V1920XD'
,'V1920XS'
,'V1929XD'
,'V1929XS'
,'V193XXD'
,'V193XXS'
,'V1940XD'
,'V1940XS'
,'V1949XD'
,'V1949XS'
,'V1950XD'
,'V1950XS'
,'V1959XD'
,'V1959XS'
,'V1960XD'
,'V1960XS'
,'V1969XD'
,'V1969XS'
,'V1981XD'
,'V1981XS'
,'V1988XD'
,'V1988XS'
,'V199XXD'
,'V199XXS'
,'V200XXD'
,'V200XXS'
,'V201XXD'
,'V201XXS'
,'V202XXD'
,'V202XXS'
,'V203XXD'
,'V203XXS'
,'V204XXD'
,'V204XXS'
,'V205XXD'
,'V205XXS'
,'V209XXD'
,'V209XXS'
,'V210XXD'
,'V210XXS'
,'V211XXD'
,'V211XXS'
,'V212XXD'
,'V212XXS'
,'V213XXD'
,'V213XXS'
,'V214XXD'
,'V214XXS'
,'V215XXD'
,'V215XXS'
,'V219XXD'
,'V219XXS'
,'V220XXD'
,'V220XXS'
,'V221XXD'
,'V221XXS'
,'V222XXD'
,'V222XXS'
,'V223XXD'
,'V223XXS'
,'V224XXD'
,'V224XXS'
,'V225XXD'
,'V225XXS'
,'V229XXD'
,'V229XXS'
,'V230XXD'
,'V230XXS'
,'V231XXD'
,'V231XXS'
,'V232XXD'
,'V232XXS'
,'V233XXD'
,'V233XXS'
,'V234XXD'
,'V234XXS'
,'V235XXD'
,'V235XXS'
,'V239XXD'
,'V239XXS'
,'V240XXD'
,'V240XXS'
,'V241XXD'
,'V241XXS'
,'V242XXD'
,'V242XXS'
,'V243XXD'
,'V243XXS'
,'V244XXD'
,'V244XXS'
,'V245XXD'
,'V245XXS'
,'V249XXD'
,'V249XXS'
,'V250XXD'
,'V250XXS'
,'V251XXD'
,'V251XXS'
,'V252XXD'
,'V252XXS'
,'V253XXD'
,'V253XXS'
,'V254XXD'
,'V254XXS'
,'V255XXD'
,'V255XXS'
,'V259XXD'
,'V259XXS'
,'V260XXD'
,'V260XXS'
,'V261XXD'
,'V261XXS'
,'V262XXD'
,'V262XXS'
,'V263XXD'
,'V263XXS'
,'V264XXD'
,'V264XXS'
,'V265XXD'
,'V265XXS'
,'V269XXD'
,'V269XXS'
,'V270XXD'
,'V270XXS'
,'V271XXD'
,'V271XXS'
,'V272XXD'
,'V272XXS'
,'V273XXD'
,'V273XXS'
,'V274XXD'
,'V274XXS'
,'V275XXD'
,'V275XXS'
,'V279XXD'
,'V279XXS'
,'V280XXD'
,'V280XXS'
,'V281XXD'
,'V281XXS'
,'V282XXD'
,'V282XXS'
,'V283XXD'
,'V283XXS'
,'V284XXD'
,'V284XXS'
,'V285XXD'
,'V285XXS'
,'V289XXD'
,'V289XXS'
,'V2900XD'
,'V2900XS'
,'V2909XD'
,'V2909XS'
,'V2910XD'
,'V2910XS'
,'V2919XD'
,'V2919XS'
,'V2920XD'
,'V2920XS'
,'V2929XD'
,'V2929XS'
,'V293XXD'
,'V293XXS'
,'V2940XD'
,'V2940XS'
,'V2949XD'
,'V2949XS'
,'V2950XD'
,'V2950XS'
,'V2959XD'
,'V2959XS'
,'V2960XD'
,'V2960XS'
,'V2969XD'
,'V2969XS'
,'V2981XD'
,'V2981XS'
,'V2988XD'
,'V2988XS'
,'V299XXD'
,'V299XXS'
,'V300XXD'
,'V300XXS'
,'V301XXD'
,'V301XXS'
,'V302XXD'
,'V302XXS'
,'V303XXD'
,'V303XXS'
,'V304XXD'
,'V304XXS'
,'V305XXD'
,'V305XXS'
,'V306XXD'
,'V306XXS'
,'V307XXD'
,'V307XXS'
,'V309XXD'
,'V309XXS'
,'V310XXD'
,'V310XXS'
,'V311XXD'
,'V311XXS'
,'V312XXD'
,'V312XXS'
,'V313XXD'
,'V313XXS'
,'V314XXD'
,'V314XXS'
,'V315XXD'
,'V315XXS'
,'V316XXD'
,'V316XXS'
,'V317XXD'
,'V317XXS'
,'V319XXD'
,'V319XXS'
,'V320XXD'
,'V320XXS'
,'V321XXD'
,'V321XXS'
,'V322XXD'
,'V322XXS'
,'V323XXD'
,'V323XXS'
,'V324XXD'
,'V324XXS'
,'V325XXD'
,'V325XXS'
,'V326XXD'
,'V326XXS'
,'V327XXD'
,'V327XXS'
,'V329XXD'
,'V329XXS'
,'V330XXD'
,'V330XXS'
,'V331XXD'
,'V331XXS'
,'V332XXD'
,'V332XXS'
,'V333XXD'
,'V333XXS'
,'V334XXD'
,'V334XXS'
,'V335XXD'
,'V335XXS'
,'V336XXD'
,'V336XXS'
,'V337XXD'
,'V337XXS'
,'V339XXD'
,'V339XXS'
,'V340XXD'
,'V340XXS'
,'V341XXD'
,'V341XXS'
,'V342XXD'
,'V342XXS'
,'V343XXD'
,'V343XXS'
,'V344XXD'
,'V344XXS'
,'V345XXD'
,'V345XXS'
,'V346XXD'
,'V346XXS'
,'V347XXD'
,'V347XXS'
,'V349XXD'
,'V349XXS'
,'V350XXD'
,'V350XXS'
,'V351XXD'
,'V351XXS'
,'V352XXD'
,'V352XXS'
,'V353XXD'
,'V353XXS'
,'V354XXD'
,'V354XXS'
,'V355XXD'
,'V355XXS'
,'V356XXD'
,'V356XXS'
,'V357XXD'
,'V357XXS'
,'V359XXD'
,'V359XXS'
,'V360XXD'
,'V360XXS'
,'V361XXD'
,'V361XXS'
,'V362XXD'
,'V362XXS'
,'V363XXD'
,'V363XXS'
,'V364XXD'
,'V364XXS'
,'V365XXD'
,'V365XXS'
,'V366XXD'
,'V366XXS'
,'V367XXD'
,'V367XXS'
,'V369XXD'
,'V369XXS'
,'V370XXD'
,'V370XXS'
,'V371XXD'
,'V371XXS'
,'V372XXD'
,'V372XXS'
,'V373XXD'
,'V373XXS'
,'V374XXD'
,'V374XXS'
,'V375XXD'
,'V375XXS'
,'V376XXD'
,'V376XXS'
,'V377XXD'
,'V377XXS'
,'V379XXD'
,'V379XXS'
,'V380XXD'
,'V380XXS'
,'V381XXD'
,'V381XXS'
,'V382XXD'
,'V382XXS'
,'V383XXD'
,'V383XXS'
,'V384XXD'
,'V384XXS'
,'V385XXD'
,'V385XXS'
,'V386XXD'
,'V386XXS'
,'V387XXD'
,'V387XXS'
,'V389XXD'
,'V389XXS'
,'V3900XD'
,'V3900XS'
,'V3909XD'
,'V3909XS'
,'V3910XD'
,'V3910XS'
,'V3919XD'
,'V3919XS'
,'V3920XD'
,'V3920XS'
,'V3929XD'
,'V3929XS'
,'V393XXD'
,'V393XXS'
,'V3940XD'
,'V3940XS'
,'V3949XD'
,'V3949XS'
,'V3950XD'
,'V3950XS'
,'V3959XD'
,'V3959XS'
,'V3960XD'
,'V3960XS'
,'V3969XD'
,'V3969XS'
,'V3981XD'
,'V3981XS'
,'V3989XD'
,'V3989XS'
,'V399XXD'
,'V399XXS'
,'V400XXD'
,'V400XXS'
,'V401XXD'
,'V401XXS'
,'V402XXD'
,'V402XXS'
,'V403XXD'
,'V403XXS'
,'V404XXD'
,'V404XXS'
,'V405XXD'
,'V405XXS'
,'V406XXD'
,'V406XXS'
,'V407XXD'
,'V407XXS'
,'V409XXD'
,'V409XXS'
,'V410XXD'
,'V410XXS'
,'V411XXD'
,'V411XXS'
,'V412XXD'
,'V412XXS'
,'V413XXD'
,'V413XXS'
,'V414XXD'
,'V414XXS'
,'V415XXD'
,'V415XXS'
,'V416XXD'
,'V416XXS'
,'V417XXD'
,'V417XXS'
,'V419XXD'
,'V419XXS'
,'V420XXD'
,'V420XXS'
,'V421XXD'
,'V421XXS'
,'V422XXD'
,'V422XXS'
,'V423XXD'
,'V423XXS'
,'V424XXD'
,'V424XXS'
,'V425XXD'
,'V425XXS'
,'V426XXD'
,'V426XXS'
,'V427XXD'
,'V427XXS'
,'V429XXD'
,'V429XXS'
,'V4301XD'
,'V4301XS'
,'V4302XD'
,'V4302XS'
,'V4303XD'
,'V4303XS'
,'V4304XD'
,'V4304XS'
,'V4311XD'
,'V4311XS'
,'V4312XD'
,'V4312XS'
,'V4313XD'
,'V4313XS'
,'V4314XD'
,'V4314XS'
,'V4321XD'
,'V4321XS'
,'V4322XD'
,'V4322XS'
,'V4323XD'
,'V4323XS'
,'V4324XD'
,'V4324XS'
,'V4331XD'
,'V4331XS'
,'V4332XD'
,'V4332XS'
,'V4333XD'
,'V4333XS'
,'V4334XD'
,'V4334XS'
,'V4341XD'
,'V4341XS'
,'V4342XD'
,'V4342XS'
,'V4343XD'
,'V4343XS'
,'V4344XD'
,'V4344XS'
,'V4351XD'
,'V4351XS'
,'V4352XD'
,'V4352XS'
,'V4353XD'
,'V4353XS'
,'V4354XD'
,'V4354XS'
,'V4361XD'
,'V4361XS'
,'V4362XD'
,'V4362XS'
,'V4363XD'
,'V4363XS'
,'V4364XD'
,'V4364XS'
,'V4371XD'
,'V4371XS'
,'V4372XD'
,'V4372XS'
,'V4373XD'
,'V4373XS'
,'V4374XD'
,'V4374XS'
,'V4391XD'
,'V4391XS'
,'V4392XD'
,'V4392XS'
,'V4393XD'
,'V4393XS'
,'V4394XD'
,'V4394XS'
,'V440XXD'
,'V440XXS'
,'V441XXD'
,'V441XXS'
,'V442XXD'
,'V442XXS'
,'V443XXD'
,'V443XXS'
,'V444XXD'
,'V444XXS'
,'V445XXD'
,'V445XXS'
,'V446XXD'
,'V446XXS'
,'V447XXD'
,'V447XXS'
,'V449XXD'
,'V449XXS'
,'V450XXD'
,'V450XXS'
,'V451XXD'
,'V451XXS'
,'V452XXD'
,'V452XXS'
,'V453XXD'
,'V453XXS'
,'V454XXD'
,'V454XXS'
,'V455XXD'
,'V455XXS'
,'V456XXD'
,'V456XXS'
,'V457XXD'
,'V457XXS'
,'V459XXD'
,'V459XXS'
,'V460XXD'
,'V460XXS'
,'V461XXD'
,'V461XXS'
,'V462XXD'
,'V462XXS'
,'V463XXD'
,'V463XXS'
,'V464XXD'
,'V464XXS'
,'V465XXD'
,'V465XXS'
,'V466XXD'
,'V466XXS'
,'V467XXD'
,'V467XXS'
,'V469XXD'
,'V469XXS'
,'V4701XD'
,'V4701XS'
,'V4702XD'
,'V4702XS'
,'V4711XD'
,'V4711XS'
,'V4712XD'
,'V4712XS'
,'V472XXD'
,'V472XXS'
,'V4731XD'
,'V4731XS'
,'V4732XD'
,'V4732XS'
,'V474XXD'
,'V474XXS'
,'V4751XD'
,'V4751XS'
,'V4752XD'
,'V4752XS'
,'V4761XD'
,'V4761XS'
,'V4762XD'
,'V4762XS'
,'V477XXD'
,'V477XXS'
,'V4791XD'
,'V4791XS'
,'V4792XD'
,'V4792XS'
,'V480XXD'
,'V480XXS'
,'V481XXD'
,'V481XXS'
,'V482XXD'
,'V482XXS'
,'V483XXD'
,'V483XXS'
,'V484XXD'
,'V484XXS'
,'V485XXD'
,'V485XXS'
,'V486XXD'
,'V486XXS'
,'V487XXD'
,'V487XXS'
,'V489XXD'
,'V489XXS'
,'V4900XD'
,'V4900XS'
,'V4909XD'
,'V4909XS'
,'V4910XD'
,'V4910XS'
,'V4919XD'
,'V4919XS'
,'V4920XD'
,'V4920XS'
,'V4929XD'
,'V4929XS'
,'V493XXD'
,'V493XXS'
,'V4940XD'
,'V4940XS'
,'V4949XD'
,'V4949XS'
,'V4950XD'
,'V4950XS'
,'V4959XD'
,'V4959XS'
,'V4960XD'
,'V4960XS'
,'V4969XD'
,'V4969XS'
,'V4981XD'
,'V4981XS'
,'V4988XD'
,'V4988XS'
,'V499XXD'
,'V499XXS'
,'V500XXD'
,'V500XXS'
,'V501XXD'
,'V501XXS'
,'V502XXD'
,'V502XXS'
,'V503XXD'
,'V503XXS'
,'V504XXD'
,'V504XXS'
,'V505XXD'
,'V505XXS'
,'V506XXD'
,'V506XXS'
,'V507XXD'
,'V507XXS'
,'V509XXD'
,'V509XXS'
,'V510XXD'
,'V510XXS'
,'V511XXD'
,'V511XXS'
,'V512XXD'
,'V512XXS'
,'V513XXD'
,'V513XXS'
,'V514XXD'
,'V514XXS'
,'V515XXD'
,'V515XXS'
,'V516XXD'
,'V516XXS'
,'V517XXD'
,'V517XXS'
,'V519XXD'
,'V519XXS'
,'V520XXD'
,'V520XXS'
,'V521XXD'
,'V521XXS'
,'V522XXD'
,'V522XXS'
,'V523XXD'
,'V523XXS'
,'V524XXD'
,'V524XXS'
,'V525XXD'
,'V525XXS'
,'V526XXD'
,'V526XXS'
,'V527XXD'
,'V527XXS'
,'V529XXD'
,'V529XXS'
,'V530XXD'
,'V530XXS'
,'V531XXD'
,'V531XXS'
,'V532XXD'
,'V532XXS'
,'V533XXD'
,'V533XXS'
,'V534XXD'
,'V534XXS'
,'V535XXD'
,'V535XXS'
,'V536XXD'
,'V536XXS'
,'V537XXD'
,'V537XXS'
,'V539XXD'
,'V539XXS'
,'V540XXD'
,'V540XXS'
,'V541XXD'
,'V541XXS'
,'V542XXD'
,'V542XXS'
,'V543XXD'
,'V543XXS'
,'V544XXD'
,'V544XXS'
,'V545XXD'
,'V545XXS'
,'V546XXD'
,'V546XXS'
,'V547XXD'
,'V547XXS'
,'V549XXD'
,'V549XXS'
,'V550XXD'
,'V550XXS'
,'V551XXD'
,'V551XXS'
,'V552XXD'
,'V552XXS'
,'V553XXD'
,'V553XXS'
,'V554XXD'
,'V554XXS'
,'V555XXD'
,'V555XXS'
,'V556XXD'
,'V556XXS'
,'V557XXD'
,'V557XXS'
,'V559XXD'
,'V559XXS'
,'V560XXD'
,'V560XXS'
,'V561XXD'
,'V561XXS'
,'V562XXD'
,'V562XXS'
,'V563XXD'
,'V563XXS'
,'V564XXD'
,'V564XXS'
,'V565XXD'
,'V565XXS'
,'V566XXD'
,'V566XXS'
,'V567XXD'
,'V567XXS'
,'V569XXD'
,'V569XXS'
,'V570XXD'
,'V570XXS'
,'V571XXD'
,'V571XXS'
,'V572XXD'
,'V572XXS'
,'V573XXD'
,'V573XXS'
,'V574XXD'
,'V574XXS'
,'V575XXD'
,'V575XXS'
,'V576XXD'
,'V576XXS'
,'V577XXD'
,'V577XXS'
,'V579XXD'
,'V579XXS'
,'V580XXD'
,'V580XXS'
,'V581XXD'
,'V581XXS'
,'V582XXD'
,'V582XXS'
,'V583XXD'
,'V583XXS'
,'V584XXD'
,'V584XXS'
,'V585XXD'
,'V585XXS'
,'V586XXD'
,'V586XXS'
,'V587XXD'
,'V587XXS'
,'V589XXD'
,'V589XXS'
,'V5900XD'
,'V5900XS'
,'V5909XD'
,'V5909XS'
,'V5910XD'
,'V5910XS'
,'V5919XD'
,'V5919XS'
,'V5920XD'
,'V5920XS'
,'V5929XD'
,'V5929XS'
,'V593XXD'
,'V593XXS'
,'V5940XD'
,'V5940XS'
,'V5949XD'
,'V5949XS'
,'V5950XD'
,'V5950XS'
,'V5959XD'
,'V5959XS'
,'V5960XD'
,'V5960XS'
,'V5969XD'
,'V5969XS'
,'V5981XD'
,'V5981XS'
,'V5988XD'
,'V5988XS'
,'V599XXD'
,'V599XXS'
,'V600XXD'
,'V600XXS'
,'V601XXD'
,'V601XXS'
,'V602XXD'
,'V602XXS'
,'V603XXD'
,'V603XXS'
,'V604XXD'
,'V604XXS'
,'V605XXD'
,'V605XXS'
,'V606XXD'
,'V606XXS'
,'V607XXD'
,'V607XXS'
,'V609XXD'
,'V609XXS'
,'V610XXD'
,'V610XXS'
,'V611XXD'
,'V611XXS'
,'V612XXD'
,'V612XXS'
,'V613XXD'
,'V613XXS'
,'V614XXD'
,'V614XXS'
,'V615XXD'
,'V615XXS'
,'V616XXD'
,'V616XXS'
,'V617XXD'
,'V617XXS'
,'V619XXD'
,'V619XXS'
,'V620XXD'
,'V620XXS'
,'V621XXD'
,'V621XXS'
,'V622XXD'
,'V622XXS'
,'V623XXD'
,'V623XXS'
,'V624XXD'
,'V624XXS'
,'V625XXD'
,'V625XXS'
,'V626XXD'
,'V626XXS'
,'V627XXD'
,'V627XXS'
,'V629XXD'
,'V629XXS'
,'V630XXD'
,'V630XXS'
,'V631XXD'
,'V631XXS'
,'V632XXD'
,'V632XXS'
,'V633XXD'
,'V633XXS'
,'V634XXD'
,'V634XXS'
,'V635XXD'
,'V635XXS'
,'V636XXD'
,'V636XXS'
,'V637XXD'
,'V637XXS'
,'V639XXD'
,'V639XXS'
,'V640XXD'
,'V640XXS'
,'V641XXD'
,'V641XXS'
,'V642XXD'
,'V642XXS'
,'V643XXD'
,'V643XXS'
,'V644XXD'
,'V644XXS'
,'V645XXD'
,'V645XXS'
,'V646XXD'
,'V646XXS'
,'V647XXD'
,'V647XXS'
,'V649XXD'
,'V649XXS'
,'V650XXD'
,'V650XXS'
,'V651XXD'
,'V651XXS'
,'V652XXD'
,'V652XXS'
,'V653XXD'
,'V653XXS'
,'V654XXD'
,'V654XXS'
,'V655XXD'
,'V655XXS'
,'V656XXD'
,'V656XXS'
,'V657XXD'
,'V657XXS'
,'V659XXD'
,'V659XXS'
,'V660XXD'
,'V660XXS'
,'V661XXD'
,'V661XXS'
,'V662XXD'
,'V662XXS'
,'V663XXD'
,'V663XXS'
,'V664XXD'
,'V664XXS'
,'V665XXD'
,'V665XXS'
,'V666XXD'
,'V666XXS'
,'V667XXD'
,'V667XXS'
,'V669XXD'
,'V669XXS'
,'V670XXD'
,'V670XXS'
,'V671XXD'
,'V671XXS'
,'V672XXD'
,'V672XXS'
,'V673XXD'
,'V673XXS'
,'V674XXD'
,'V674XXS'
,'V675XXD'
,'V675XXS'
,'V676XXD'
,'V676XXS'
,'V677XXD'
,'V677XXS'
,'V679XXD'
,'V679XXS'
,'V680XXD'
,'V680XXS'
,'V681XXD'
,'V681XXS'
,'V682XXD'
,'V682XXS'
,'V683XXD'
,'V683XXS'
,'V684XXD'
,'V684XXS'
,'V685XXD'
,'V685XXS'
,'V686XXD'
,'V686XXS'
,'V687XXD'
,'V687XXS'
,'V689XXD'
,'V689XXS'
,'V6900XD'
,'V6900XS'
,'V6909XD'
,'V6909XS'
,'V6910XD'
,'V6910XS'
,'V6919XD'
,'V6919XS'
,'V6920XD'
,'V6920XS'
,'V6929XD'
,'V6929XS'
,'V693XXD'
,'V693XXS'
,'V6940XD'
,'V6940XS'
,'V6949XD'
,'V6949XS'
,'V6950XD'
,'V6950XS'
,'V6959XD'
,'V6959XS'
,'V6960XD'
,'V6960XS'
,'V6969XD'
,'V6969XS'
,'V6981XD'
,'V6981XS'
,'V6988XD'
,'V6988XS'
,'V699XXD'
,'V699XXS'
,'V700XXD'
,'V700XXS'
,'V701XXD'
,'V701XXS'
,'V702XXD'
,'V702XXS'
,'V703XXD'
,'V703XXS'
,'V704XXD'
,'V704XXS'
,'V705XXD'
,'V705XXS'
,'V706XXD'
,'V706XXS'
,'V707XXD'
,'V707XXS'
,'V709XXD'
,'V709XXS'
,'V710XXD'
,'V710XXS'
,'V711XXD'
,'V711XXS'
,'V712XXD'
,'V712XXS'
,'V713XXD'
,'V713XXS'
,'V714XXD'
,'V714XXS'
,'V715XXD'
,'V715XXS'
,'V716XXD'
,'V716XXS'
,'V717XXD'
,'V717XXS'
,'V719XXD'
,'V719XXS'
,'V720XXD'
,'V720XXS'
,'V721XXD'
,'V721XXS'
,'V722XXD'
,'V722XXS'
,'V723XXD'
,'V723XXS'
,'V724XXD'
,'V724XXS'
,'V725XXD'
,'V725XXS'
,'V726XXD'
,'V726XXS'
,'V727XXD'
,'V727XXS'
,'V729XXD'
,'V729XXS'
,'V730XXD'
,'V730XXS'
,'V731XXD'
,'V731XXS'
,'V732XXD'
,'V732XXS'
,'V733XXD'
,'V733XXS'
,'V734XXD'
,'V734XXS'
,'V735XXD'
,'V735XXS'
,'V736XXD'
,'V736XXS'
,'V737XXD'
,'V737XXS'
,'V739XXD'
,'V739XXS'
,'V740XXD'
,'V740XXS'
,'V741XXD'
,'V741XXS'
,'V742XXD'
,'V742XXS'
,'V743XXD'
,'V743XXS'
,'V744XXD'
,'V744XXS'
,'V745XXD'
,'V745XXS'
,'V746XXD'
,'V746XXS'
,'V747XXD'
,'V747XXS'
,'V749XXD'
,'V749XXS'
,'V750XXD'
,'V750XXS'
,'V751XXD'
,'V751XXS'
,'V752XXD'
,'V752XXS'
,'V753XXD'
,'V753XXS'
,'V754XXD'
,'V754XXS'
,'V755XXD'
,'V755XXS'
,'V756XXD'
,'V756XXS'
,'V757XXD'
,'V757XXS'
,'V759XXD'
,'V759XXS'
,'V760XXD'
,'V760XXS'
,'V761XXD'
,'V761XXS'
,'V762XXD'
,'V762XXS'
,'V763XXD'
,'V763XXS'
,'V764XXD'
,'V764XXS'
,'V765XXD'
,'V765XXS'
,'V766XXD'
,'V766XXS'
,'V767XXD'
,'V767XXS'
,'V769XXD'
,'V769XXS'
,'V770XXD'
,'V770XXS'
,'V771XXD'
,'V771XXS'
,'V772XXD'
,'V772XXS'
,'V773XXD'
,'V773XXS'
,'V774XXD'
,'V774XXS'
,'V775XXD'
,'V775XXS'
,'V776XXD'
,'V776XXS'
,'V777XXD'
,'V777XXS'
,'V779XXD'
,'V779XXS'
,'V780XXD'
,'V780XXS'
,'V781XXD'
,'V781XXS'
,'V782XXD'
,'V782XXS'
,'V783XXD'
,'V783XXS'
,'V784XXD'
,'V784XXS'
,'V785XXD'
,'V785XXS'
,'V786XXD'
,'V786XXS'
,'V787XXD'
,'V787XXS'
,'V789XXD'
,'V789XXS'
,'V7900XD'
,'V7900XS'
,'V7909XD'
,'V7909XS'
,'V7910XD'
,'V7910XS'
,'V7919XD'
,'V7919XS'
,'V7920XD'
,'V7920XS'
,'V7929XD'
,'V7929XS'
,'V793XXD'
,'V793XXS'
,'V7940XD'
,'V7940XS'
,'V7949XD'
,'V7949XS'
,'V7950XD'
,'V7950XS'
,'V7959XD'
,'V7959XS'
,'V7960XD'
,'V7960XS'
,'V7969XD'
,'V7969XS'
,'V7981XD'
,'V7981XS'
,'V7988XD'
,'V7988XS'
,'V799XXD'
,'V799XXS'
,'V80010D'
,'V80010S'
,'V80018D'
,'V80018S'
,'V8002XD'
,'V8002XS'
,'V8011XD'
,'V8011XS'
,'V8012XD'
,'V8012XS'
,'V8021XD'
,'V8021XS'
,'V8022XD'
,'V8022XS'
,'V8031XD'
,'V8031XS'
,'V8032XD'
,'V8032XS'
,'V8041XD'
,'V8041XS'
,'V8042XD'
,'V8042XS'
,'V8051XD'
,'V8051XS'
,'V8052XD'
,'V8052XS'
,'V8061XD'
,'V8061XS'
,'V8062XD'
,'V8062XS'
,'V80710D'
,'V80710S'
,'V80711D'
,'V80711S'
,'V80720D'
,'V80720S'
,'V80721D'
,'V80721S'
,'V80730D'
,'V80730S'
,'V80731D'
,'V80731S'
,'V80790D'
,'V80790S'
,'V80791D'
,'V80791S'
,'V8081XD'
,'V8081XS'
,'V8082XD'
,'V8082XS'
,'V80910D'
,'V80910S'
,'V80918D'
,'V80918S'
,'V80919D'
,'V80919S'
,'V80920D'
,'V80920S'
,'V80928D'
,'V80928S'
,'V80929D'
,'V80929S'
,'V810XXD'
,'V810XXS'
,'V811XXD'
,'V811XXS'
,'V812XXD'
,'V812XXS'
,'V813XXD'
,'V813XXS'
,'V814XXD'
,'V814XXS'
,'V815XXD'
,'V815XXS'
,'V816XXD'
,'V816XXS'
,'V817XXD'
,'V817XXS'
,'V8181XD'
,'V8181XS'
,'V8182XD'
,'V8182XS'
,'V8183XD'
,'V8183XS'
,'V8189XD'
,'V8189XS'
,'V819XXD'
,'V819XXS'
,'V820XXD'
,'V820XXS'
,'V821XXD'
,'V821XXS'
,'V822XXD'
,'V822XXS'
,'V823XXD'
,'V823XXS'
,'V824XXD'
,'V824XXS'
,'V825XXD'
,'V825XXS'
,'V826XXD'
,'V826XXS'
,'V827XXD'
,'V827XXS'
,'V828XXD'
,'V828XXS'
,'V829XXD'
,'V829XXS'
,'V830XXD'
,'V830XXS'
,'V831XXD'
,'V831XXS'
,'V832XXD'
,'V832XXS'
,'V833XXD'
,'V833XXS'
,'V834XXD'
,'V834XXS'
,'V835XXD'
,'V835XXS'
,'V836XXD'
,'V836XXS'
,'V837XXD'
,'V837XXS'
,'V839XXD'
,'V839XXS'
,'V840XXD'
,'V840XXS'
,'V841XXD'
,'V841XXS'
,'V842XXD'
,'V842XXS'
,'V843XXD'
,'V843XXS'
,'V844XXD'
,'V844XXS'
,'V845XXD'
,'V845XXS'
,'V846XXD'
,'V846XXS'
,'V847XXD'
,'V847XXS'
,'V849XXD'
,'V849XXS'
,'V850XXD'
,'V850XXS'
,'V851XXD'
,'V851XXS'
,'V852XXD'
,'V852XXS'
,'V853XXD'
,'V853XXS'
,'V854XXD'
,'V854XXS'
,'V855XXD'
,'V855XXS'
,'V856XXD'
,'V856XXS'
,'V857XXD'
,'V857XXS'
,'V859XXD'
,'V859XXS'
,'V8601XD'
,'V8601XS'
,'V8602XD'
,'V8602XS'
,'V8603XD'
,'V8603XS'
,'V8604XD'
,'V8604XS'
,'V8609XD'
,'V8609XS'
,'V8611XD'
,'V8611XS'
,'V8612XD'
,'V8612XS'
,'V8613XD'
,'V8613XS'
,'V8614XD'
,'V8614XS'
,'V8619XD'
,'V8619XS'
,'V8621XD'
,'V8621XS'
,'V8622XD'
,'V8622XS'
,'V8623XD'
,'V8623XS'
,'V8624XD'
,'V8624XS'
,'V8629XD'
,'V8629XS'
,'V8631XD'
,'V8631XS'
,'V8632XD'
,'V8632XS'
,'V8633XD'
,'V8633XS'
,'V8634XD'
,'V8634XS'
,'V8639XD'
,'V8639XS'
,'V8641XD'
,'V8641XS'
,'V8642XD'
,'V8642XS'
,'V8643XD'
,'V8643XS'
,'V8644XD'
,'V8644XS'
,'V8649XD'
,'V8649XS'
,'V8651XD'
,'V8651XS'
,'V8652XD'
,'V8652XS'
,'V8653XD'
,'V8653XS'
,'V8654XD'
,'V8654XS'
,'V8659XD'
,'V8659XS'
,'V8661XD'
,'V8661XS'
,'V8662XD'
,'V8662XS'
,'V8663XD'
,'V8663XS'
,'V8664XD'
,'V8664XS'
,'V8669XD'
,'V8669XS'
,'V8671XD'
,'V8671XS'
,'V8672XD'
,'V8672XS'
,'V8673XD'
,'V8673XS'
,'V8674XD'
,'V8674XS'
,'V8679XD'
,'V8679XS'
,'V8691XD'
,'V8691XS'
,'V8692XD'
,'V8692XS'
,'V8693XD'
,'V8693XS'
,'V8694XD'
,'V8694XS'
,'V8699XD'
,'V8699XS'
,'V870XXD'
,'V870XXS'
,'V871XXD'
,'V871XXS'
,'V872XXD'
,'V872XXS'
,'V873XXD'
,'V873XXS'
,'V874XXD'
,'V874XXS'
,'V875XXD'
,'V875XXS'
,'V876XXD'
,'V876XXS'
,'V877XXD'
,'V877XXS'
,'V878XXD'
,'V878XXS'
,'V879XXD'
,'V879XXS'
,'V880XXD'
,'V880XXS'
,'V881XXD'
,'V881XXS'
,'V882XXD'
,'V882XXS'
,'V883XXD'
,'V883XXS'
,'V884XXD'
,'V884XXS'
,'V885XXD'
,'V885XXS'
,'V886XXD'
,'V886XXS'
,'V887XXD'
,'V887XXS'
,'V888XXD'
,'V888XXS'
,'V889XXD'
,'V889XXS'
,'V890XXD'
,'V890XXS'
,'V891XXD'
,'V891XXS'
,'V892XXD'
,'V892XXS'
,'V893XXD'
,'V893XXS'
,'V899XXD'
,'V899XXS'
,'V9000XD'
,'V9000XS'
,'V9001XD'
,'V9001XS'
,'V9002XD'
,'V9002XS'
,'V9003XD'
,'V9003XS'
,'V9004XD'
,'V9004XS'
,'V9005XD'
,'V9005XS'
,'V9006XD'
,'V9006XS'
,'V9008XD'
,'V9008XS'
,'V9009XD'
,'V9009XS'
,'V9010XD'
,'V9010XS'
,'V9011XD'
,'V9011XS'
,'V9012XD'
,'V9012XS'
,'V9013XD'
,'V9013XS'
,'V9014XD'
,'V9014XS'
,'V9015XD'
,'V9015XS'
,'V9016XD'
,'V9016XS'
,'V9018XD'
,'V9018XS'
,'V9019XD'
,'V9019XS'
,'V9020XD'
,'V9020XS'
,'V9021XD'
,'V9021XS'
,'V9022XD'
,'V9022XS'
,'V9023XD'
,'V9023XS'
,'V9024XD'
,'V9024XS'
,'V9025XD'
,'V9025XS'
,'V9026XD'
,'V9026XS'
,'V9027XD'
,'V9027XS'
,'V9028XD'
,'V9028XS'
,'V9029XD'
,'V9029XS'
,'V9030XD'
,'V9030XS'
,'V9031XD'
,'V9031XS'
,'V9032XD'
,'V9032XS'
,'V9033XD'
,'V9033XS'
,'V9034XD'
,'V9034XS'
,'V9035XD'
,'V9035XS'
,'V9036XD'
,'V9036XS'
,'V9037XD'
,'V9037XS'
,'V9038XD'
,'V9038XS'
,'V9039XD'
,'V9039XS'
,'V9080XD'
,'V9080XS'
,'V9081XD'
,'V9081XS'
,'V9082XD'
,'V9082XS'
,'V9083XD'
,'V9083XS'
,'V9084XD'
,'V9084XS'
,'V9085XD'
,'V9085XS'
,'V9086XD'
,'V9086XS'
,'V9087XD'
,'V9087XS'
,'V9088XD'
,'V9088XS'
,'V9089XD'
,'V9089XS'
,'V9100XD'
,'V9100XS'
,'V9101XD'
,'V9101XS'
,'V9102XD'
,'V9102XS'
,'V9103XD'
,'V9103XS'
,'V9104XD'
,'V9104XS'
,'V9105XD'
,'V9105XS'
,'V9106XD'
,'V9106XS'
,'V9107XD'
,'V9107XS'
,'V9108XD'
,'V9108XS'
,'V9109XD'
,'V9109XS'
,'V9110XD'
,'V9110XS'
,'V9111XD'
,'V9111XS'
,'V9112XD'
,'V9112XS'
,'V9113XD'
,'V9113XS'
,'V9114XD'
,'V9114XS'
,'V9115XD'
,'V9115XS'
,'V9116XD'
,'V9116XS'
,'V9118XD'
,'V9118XS'
,'V9119XD'
,'V9119XS'
,'V9120XD'
,'V9120XS'
,'V9121XD'
,'V9121XS'
,'V9122XD'
,'V9122XS'
,'V9123XD'
,'V9123XS'
,'V9124XD'
,'V9124XS'
,'V9125XD'
,'V9125XS'
,'V9126XD'
,'V9126XS'
,'V9129XD'
,'V9129XS'
,'V9130XD'
,'V9130XS'
,'V9131XD'
,'V9131XS'
,'V9132XD'
,'V9132XS'
,'V9133XD'
,'V9133XS'
,'V9134XD'
,'V9134XS'
,'V9135XD'
,'V9135XS'
,'V9136XD'
,'V9136XS'
,'V9137XD'
,'V9137XS'
,'V9138XD'
,'V9138XS'
,'V9139XD'
,'V9139XS'
,'V9180XD'
,'V9180XS'
,'V9181XD'
,'V9181XS'
,'V9182XD'
,'V9182XS'
,'V9183XD'
,'V9183XS'
,'V9184XD'
,'V9184XS'
,'V9185XD'
,'V9185XS'
,'V9186XD'
,'V9186XS'
,'V9187XD'
,'V9187XS'
,'V9188XD'
,'V9188XS'
,'V9189XD'
,'V9189XS'
,'V9200XD'
,'V9200XS'
,'V9201XD'
,'V9201XS'
,'V9202XD'
,'V9202XS'
,'V9203XD'
,'V9203XS'
,'V9204XD'
,'V9204XS'
,'V9205XD'
,'V9205XS'
,'V9206XD'
,'V9206XS'
,'V9207XD'
,'V9207XS'
,'V9208XD'
,'V9208XS'
,'V9209XD'
,'V9209XS'
,'V9210XD'
,'V9210XS'
,'V9211XD'
,'V9211XS'
,'V9212XD'
,'V9212XS'
,'V9213XD'
,'V9213XS'
,'V9214XD'
,'V9214XS'
,'V9215XD'
,'V9215XS'
,'V9216XD'
,'V9216XS'
,'V9219XD'
,'V9219XS'
,'V9220XD'
,'V9220XS'
,'V9221XD'
,'V9221XS'
,'V9222XD'
,'V9222XS'
,'V9223XD'
,'V9223XS'
,'V9224XD'
,'V9224XS'
,'V9225XD'
,'V9225XS'
,'V9226XD'
,'V9226XS'
,'V9227XD'
,'V9227XS'
,'V9228XD'
,'V9228XS'
,'V9229XD'
,'V9229XS'
,'V9300XD'
,'V9300XS'
,'V9301XD'
,'V9301XS'
,'V9302XD'
,'V9302XS'
,'V9303XD'
,'V9303XS'
,'V9304XD'
,'V9304XS'
,'V9309XD'
,'V9309XS'
,'V9310XD'
,'V9310XS'
,'V9311XD'
,'V9311XS'
,'V9312XD'
,'V9312XS'
,'V9313XD'
,'V9313XS'
,'V9314XD'
,'V9314XS'
,'V9319XD'
,'V9319XS'
,'V9320XD'
,'V9320XS'
,'V9321XD'
,'V9321XS'
,'V9322XD'
,'V9322XS'
,'V9323XD'
,'V9323XS'
,'V9324XD'
,'V9324XS'
,'V9329XD'
,'V9329XS'
,'V9330XD'
,'V9330XS'
,'V9331XD'
,'V9331XS'
,'V9332XD'
,'V9332XS'
,'V9333XD'
,'V9333XS'
,'V9334XD'
,'V9334XS'
,'V9335XD'
,'V9335XS'
,'V9336XD'
,'V9336XS'
,'V9338XD'
,'V9338XS'
,'V9339XD'
,'V9339XS'
,'V9340XD'
,'V9340XS'
,'V9341XD'
,'V9341XS'
,'V9342XD'
,'V9342XS'
,'V9343XD'
,'V9343XS'
,'V9344XD'
,'V9344XS'
,'V9348XD'
,'V9348XS'
,'V9349XD'
,'V9349XS'
,'V9350XD'
,'V9350XS'
,'V9351XD'
,'V9351XS'
,'V9352XD'
,'V9352XS'
,'V9353XD'
,'V9353XS'
,'V9354XD'
,'V9354XS'
,'V9359XD'
,'V9359XS'
,'V9360XD'
,'V9360XS'
,'V9361XD'
,'V9361XS'
,'V9362XD'
,'V9362XS'
,'V9363XD'
,'V9363XS'
,'V9364XD'
,'V9364XS'
,'V9369XD'
,'V9369XS'
,'V9380XD'
,'V9380XS'
,'V9381XD'
,'V9381XS'
,'V9382XD'
,'V9382XS'
,'V9383XD'
,'V9383XS'
,'V9384XD'
,'V9384XS'
,'V9385XD'
,'V9385XS'
,'V9386XD'
,'V9386XS'
,'V9387XD'
,'V9387XS'
,'V9388XD'
,'V9388XS'
,'V9389XD'
,'V9389XS'
,'V940XXD'
,'V940XXS'
,'V9411XD'
,'V9411XS'
,'V9412XD'
,'V9412XS'
,'V9421XD'
,'V9421XS'
,'V9422XD'
,'V9422XS'
,'V9431XD'
,'V9431XS'
,'V9432XD'
,'V9432XS'
,'V944XXD'
,'V944XXS'
,'V94810D'
,'V94810S'
,'V94811D'
,'V94811S'
,'V94818D'
,'V94818S'
,'V9489XD'
,'V9489XS'
,'V949XXD'
,'V949XXS'
,'V9500XD'
,'V9500XS'
,'V9501XD'
,'V9501XS'
,'V9502XD'
,'V9502XS'
,'V9503XD'
,'V9503XS'
,'V9504XD'
,'V9504XS'
,'V9505XD'
,'V9505XS'
,'V9509XD'
,'V9509XS'
,'V9510XD'
,'V9510XS'
,'V9511XD'
,'V9511XS'
,'V9512XD'
,'V9512XS'
,'V9513XD'
,'V9513XS'
,'V9514XD'
,'V9514XS'
,'V9515XD'
,'V9515XS'
,'V9519XD'
,'V9519XS'
,'V9520XD'
,'V9520XS'
,'V9521XD'
,'V9521XS'
,'V9522XD'
,'V9522XS'
,'V9523XD'
,'V9523XS'
,'V9524XD'
,'V9524XS'
,'V9525XD'
,'V9525XS'
,'V9529XD'
,'V9529XS'
,'V9530XD'
,'V9530XS'
,'V9531XD'
,'V9531XS'
,'V9532XD'
,'V9532XS'
,'V9533XD'
,'V9533XS'
,'V9534XD'
,'V9534XS'
,'V9535XD'
,'V9535XS'
,'V9539XD'
,'V9539XS'
,'V9540XD'
,'V9540XS'
,'V9541XD'
,'V9541XS'
,'V9542XD'
,'V9542XS'
,'V9543XD'
,'V9543XS'
,'V9544XD'
,'V9544XS'
,'V9545XD'
,'V9545XS'
,'V9549XD'
,'V9549XS'
,'V958XXD'
,'V958XXS'
,'V959XXD'
,'V959XXS'
,'V9600XD'
,'V9600XS'
,'V9601XD'
,'V9601XS'
,'V9602XD'
,'V9602XS'
,'V9603XD'
,'V9603XS'
,'V9604XD'
,'V9604XS'
,'V9605XD'
,'V9605XS'
,'V9609XD'
,'V9609XS'
,'V9610XD'
,'V9610XS'
,'V9611XD'
,'V9611XS'
,'V9612XD'
,'V9612XS'
,'V9613XD'
,'V9613XS'
,'V9614XD'
,'V9614XS'
,'V9615XD'
,'V9615XS'
,'V9619XD'
,'V9619XS'
,'V9620XD'
,'V9620XS'
,'V9621XD'
,'V9621XS'
,'V9622XD'
,'V9622XS'
,'V9623XD'
,'V9623XS'
,'V9624XD'
,'V9624XS'
,'V9625XD'
,'V9625XS'
,'V9629XD'
,'V9629XS'
,'V968XXD'
,'V968XXS'
,'V969XXD'
,'V969XXS'
,'V970XXD'
,'V970XXS'
,'V971XXD'
,'V971XXS'
,'V9721XD'
,'V9721XS'
,'V9722XD'
,'V9722XS'
,'V9729XD'
,'V9729XS'
,'V9731XD'
,'V9731XS'
,'V9732XD'
,'V9732XS'
,'V9733XD'
,'V9733XS'
,'V9739XD'
,'V9739XS'
,'V97810D'
,'V97810S'
,'V97811D'
,'V97811S'
,'V97818D'
,'V97818S'
,'V9789XD'
,'V9789XS'
,'V980XXD'
,'V980XXS'
,'V981XXD'
,'V981XXS'
,'V982XXD'
,'V982XXS'
,'V983XXD'
,'V983XXS'
,'V988XXD'
,'V988XXS'
,'V99XXXD'
,'V99XXXS'
,'W090XXD'
,'W090XXS'
,'W091XXD'
,'W091XXS'
,'W092XXD'
,'W092XXS'
,'W098XXD'
,'W098XXS'
,'W14XXXD'
,'W14XXXS'
,'W15XXXD'
,'W15XXXS'
,'W170XXD'
,'W170XXS'
,'W171XXD'
,'W171XXS'
,'W1801XD'
,'W1801XS'
,'W2100XD'
,'W2100XS'
,'W2101XD'
,'W2101XS'
,'W2102XD'
,'W2102XS'
,'W2103XD'
,'W2103XS'
,'W2104XD'
,'W2104XS'
,'W2105XD'
,'W2105XS'
,'W2106XD'
,'W2106XS'
,'W2107XD'
,'W2107XS'
,'W2109XD'
,'W2109XS'
,'W2111XD'
,'W2111XS'
,'W2112XD'
,'W2112XS'
,'W2113XD'
,'W2113XS'
,'W2119XD'
,'W2119XS'
,'W21210D'
,'W21210S'
,'W21211D'
,'W21211S'
,'W21220D'
,'W21220S'
,'W21221D'
,'W21221S'
,'W2131XD'
,'W2131XS'
,'W2132XD'
,'W2132XS'
,'W2139XD'
,'W2139XS'
,'W214XXD'
,'W214XXS'
,'W2181XD'
,'W2181XS'
,'W2189XD'
,'W2189XS'
,'W219XXD'
,'W219XXS'
,'W300XXD'
,'W300XXS'
,'W301XXD'
,'W301XXS'
,'W302XXD'
,'W302XXS'
,'W303XXD'
,'W303XXS'
,'W3081XD'
,'W3081XS'
,'W3089XD'
,'W3089XS'
,'W309XXD'
,'W309XXS'
,'W310XXD'
,'W310XXS'
,'W311XXD'
,'W311XXS'
,'W312XXD'
,'W312XXS'
,'W313XXD'
,'W313XXS'
,'W3181XD'
,'W3181XS'
,'W3182XD'
,'W3182XS'
,'W3183XD'
,'W3183XS'
,'W3189XD'
,'W3189XS'
,'W319XXD'
,'W319XXS'
,'W320XXD'
,'W320XXS'
,'W321XXD'
,'W321XXS'
,'W3300XD'
,'W3300XS'
,'W3301XD'
,'W3301XS'
,'W3302XD'
,'W3302XS'
,'W3303XD'
,'W3303XS'
,'W3309XD'
,'W3309XS'
,'W3310XD'
,'W3310XS'
,'W3311XD'
,'W3311XS'
,'W3312XD'
,'W3312XS'
,'W3313XD'
,'W3313XS'
,'W3319XD'
,'W3319XS'
,'W3400XD'
,'W3400XS'
,'W34010D'
,'W34010S'
,'W34011D'
,'W34011S'
,'W34018D'
,'W34018S'
,'W3409XD'
,'W3409XS'
,'W3410XD'
,'W3410XS'
,'W34110D'
,'W34110S'
,'W34111D'
,'W34111S'
,'W34118D'
,'W34118S'
,'W3419XD'
,'W3419XS'
,'W35XXXD'
,'W35XXXS'
,'W361XXD'
,'W361XXS'
,'W362XXD'
,'W362XXS'
,'W363XXD'
,'W363XXS'
,'W368XXD'
,'W368XXS'
,'W369XXD'
,'W369XXS'
,'W370XXD'
,'W370XXS'
,'W378XXD'
,'W378XXS'
,'W38XXXD'
,'W38XXXS'
,'W39XXXD'
,'W39XXXS'
,'W400XXD'
,'W400XXS'
,'W401XXD'
,'W401XXS'
,'W408XXD'
,'W408XXS'
,'W409XXD'
,'W409XXS'
,'W52XXXD'
,'W52XXXS'
,'W5601XD'
,'W5601XS'
,'W5602XD'
,'W5602XS'
,'W5609XD'
,'W5609XS'
,'W5611XD'
,'W5611XS'
,'W5612XD'
,'W5612XS'
,'W5619XD'
,'W5619XS'
,'W5621XD'
,'W5621XS'
,'W5622XD'
,'W5622XS'
,'W5629XD'
,'W5629XS'
,'W5631XD'
,'W5631XS'
,'W5632XD'
,'W5632XS'
,'W5639XD'
,'W5639XS'
,'W5641XD'
,'W5641XS'
,'W5642XD'
,'W5642XS'
,'W5649XD'
,'W5649XS'
,'W5651XD'
,'W5651XS'
,'W5652XD'
,'W5652XS'
,'W5659XD'
,'W5659XS'
,'W5681XD'
,'W5681XS'
,'W5682XD'
,'W5682XS'
,'W5689XD'
,'W5689XS'
,'W5801XD'
,'W5801XS'
,'W5802XD'
,'W5802XS'
,'W5803XD'
,'W5803XS'
,'W5809XD'
,'W5809XS'
,'W5811XD'
,'W5811XS'
,'W5812XD'
,'W5812XS'
,'W5813XD'
,'W5813XS'
,'W5819XD'
,'W5819XS'
,'W6101XD'
,'W6101XS'
,'W6102XD'
,'W6102XS'
,'W6109XD'
,'W6109XS'
,'W6111XD'
,'W6111XS'
,'W6112XD'
,'W6112XS'
,'W6119XD'
,'W6119XS'
,'W6121XD'
,'W6121XS'
,'W6122XD'
,'W6122XS'
,'W6129XD'
,'W6129XS'
,'W6132XD'
,'W6132XS'
,'W6133XD'
,'W6133XS'
,'W6139XD'
,'W6139XS'
,'W6142XD'
,'W6142XS'
,'W6143XD'
,'W6143XS'
,'W6149XD'
,'W6149XS'
,'W6151XD'
,'W6151XS'
,'W6152XD'
,'W6152XS'
,'W6159XD'
,'W6159XS'
,'W6161XD'
,'W6161XS'
,'W6162XD'
,'W6162XS'
,'W6169XD'
,'W6169XS'
,'W6191XD'
,'W6191XS'
,'W6192XD'
,'W6192XS'
,'W6199XD'
,'W6199XS'
,'W620XXD'
,'W620XXS'
,'W621XXD'
,'W621XXS'
,'W629XXD'
,'W629XXS'
,'W890XXD'
,'W890XXS'
,'W891XXD'
,'W891XXS'
,'W898XXD'
,'W898XXS'
,'W899XXD'
,'W899XXS'
,'X020XXD'
,'X020XXS'
,'X021XXD'
,'X021XXS'
,'X022XXD'
,'X022XXS'
,'X023XXD'
,'X023XXS'
,'X024XXD'
,'X024XXS'
,'X025XXD'
,'X025XXS'
,'X028XXD'
,'X028XXS'
,'X030XXD'
,'X030XXS'
,'X031XXD'
,'X031XXS'
,'X033XXD'
,'X033XXS'
,'X034XXD'
,'X034XXS'
,'X038XXD'
,'X038XXS'
,'X04XXXD'
,'X04XXXS'
,'X52XXXD'
,'X52XXXS'
,'X710XXD'
,'X710XXS'
,'X711XXD'
,'X711XXS'
,'X712XXD'
,'X712XXS'
,'X713XXD'
,'X713XXS'
,'X718XXD'
,'X718XXS'
,'X719XXD'
,'X719XXS'
,'X72XXXD'
,'X72XXXS'
,'X730XXD'
,'X730XXS'
,'X731XXD'
,'X731XXS'
,'X732XXD'
,'X732XXS'
,'X738XXD'
,'X738XXS'
,'X739XXD'
,'X739XXS'
,'X7401XD'
,'X7401XS'
,'X7402XD'
,'X7402XS'
,'X7409XD'
,'X7409XS'
,'X748XXD'
,'X748XXS'
,'X749XXD'
,'X749XXS'
,'X75XXXD'
,'X75XXXS'
,'X76XXXD'
,'X76XXXS'
,'X770XXD'
,'X770XXS'
,'X771XXD'
,'X771XXS'
,'X772XXD'
,'X772XXS'
,'X773XXD'
,'X773XXS'
,'X778XXD'
,'X778XXS'
,'X779XXD'
,'X779XXS'
,'X810XXD'
,'X810XXS'
,'X811XXD'
,'X811XXS'
,'X818XXD'
,'X818XXS'
,'X820XXD'
,'X820XXS'
,'X821XXD'
,'X821XXS'
,'X822XXD'
,'X822XXS'
,'X828XXD'
,'X828XXS'
,'X830XXD'
,'X830XXS'
,'X831XXD'
,'X831XXS'
,'X832XXD'
,'X832XXS'
,'X838XXD'
,'X838XXS'
,'Y030XXD'
,'Y030XXS'
,'Y038XXD'
,'Y038XXS'
,'Y0701'
,'Y0702'
,'Y0703'
,'Y0704'
,'Y0711'
,'Y0712'
,'Y0713'
,'Y0714'
,'Y07410'
,'Y07411'
,'Y07420'
,'Y07421'
,'Y07430'
,'Y07432'
,'Y07433'
,'Y07434'
,'Y07435'
,'Y07436'
,'Y07490'
,'Y07491'
,'Y07499'
,'Y0750'
,'Y07510'
,'Y07511'
,'Y07512'
,'Y07513'
,'Y07519'
,'Y07521'
,'Y07528'
,'Y07529'
,'Y0753'
,'Y0759'
,'Y079'
,'Y0881XD'
,'Y0881XS'
,'Y0889XD'
,'Y0889XS'
,'Y210XXD'
,'Y210XXS'
,'Y211XXD'
,'Y211XXS'
,'Y212XXD'
,'Y212XXS'
,'Y213XXD'
,'Y213XXS'
,'Y214XXD'
,'Y214XXS'
,'Y218XXD'
,'Y218XXS'
,'Y219XXD'
,'Y219XXS'
,'Y22XXXD'
,'Y22XXXS'
,'Y230XXD'
,'Y230XXS'
,'Y231XXD'
,'Y231XXS'
,'Y232XXD'
,'Y232XXS'
,'Y233XXD'
,'Y233XXS'
,'Y238XXD'
,'Y238XXS'
,'Y239XXD'
,'Y239XXS'
,'Y240XXD'
,'Y240XXS'
,'Y248XXD'
,'Y248XXS'
,'Y249XXD'
,'Y249XXS'
,'Y30XXXD'
,'Y30XXXS'
,'Y32XXXD'
,'Y32XXXS'
,'Y36000D'
,'Y36000S'
,'Y36001D'
,'Y36001S'
,'Y36010D'
,'Y36010S'
,'Y36011D'
,'Y36011S'
,'Y36020D'
,'Y36020S'
,'Y36021D'
,'Y36021S'
,'Y36030D'
,'Y36030S'
,'Y36031D'
,'Y36031S'
,'Y36040D'
,'Y36040S'
,'Y36041D'
,'Y36041S'
,'Y36050D'
,'Y36050S'
,'Y36051D'
,'Y36051S'
,'Y36090D'
,'Y36090S'
,'Y36091D'
,'Y36091S'
,'Y36100D'
,'Y36100S'
,'Y36101D'
,'Y36101S'
,'Y36110D'
,'Y36110S'
,'Y36111D'
,'Y36111S'
,'Y36120D'
,'Y36120S'
,'Y36121D'
,'Y36121S'
,'Y36130D'
,'Y36130S'
,'Y36131D'
,'Y36131S'
,'Y36140D'
,'Y36140S'
,'Y36141D'
,'Y36141S'
,'Y36190D'
,'Y36190S'
,'Y36191D'
,'Y36191S'
,'Y36200D'
,'Y36200S'
,'Y36201D'
,'Y36201S'
,'Y36210D'
,'Y36210S'
,'Y36211D'
,'Y36211S'
,'Y36220D'
,'Y36220S'
,'Y36221D'
,'Y36221S'
,'Y36230D'
,'Y36230S'
,'Y36231D'
,'Y36231S'
,'Y36240D'
,'Y36240S'
,'Y36241D'
,'Y36241S'
,'Y36250D'
,'Y36250S'
,'Y36251D'
,'Y36251S'
,'Y36260D'
,'Y36260S'
,'Y36261D'
,'Y36261S'
,'Y36270D'
,'Y36270S'
,'Y36271D'
,'Y36271S'
,'Y36290D'
,'Y36290S'
,'Y36291D'
,'Y36291S'
,'Y36300D'
,'Y36300S'
,'Y36301D'
,'Y36301S'
,'Y36310D'
,'Y36310S'
,'Y36311D'
,'Y36311S'
,'Y36320D'
,'Y36320S'
,'Y36321D'
,'Y36321S'
,'Y36330D'
,'Y36330S'
,'Y36331D'
,'Y36331S'
,'Y36390D'
,'Y36390S'
,'Y36391D'
,'Y36391S'
,'Y36410D'
,'Y36410S'
,'Y36411D'
,'Y36411S'
,'Y36420D'
,'Y36420S'
,'Y36421D'
,'Y36421S'
,'Y36430D'
,'Y36430S'
,'Y36431D'
,'Y36431S'
,'Y36440D'
,'Y36440S'
,'Y36441D'
,'Y36441S'
,'Y36450D'
,'Y36450S'
,'Y36451D'
,'Y36451S'
,'Y36460D'
,'Y36460S'
,'Y36461D'
,'Y36461S'
,'Y36470D'
,'Y36470S'
,'Y36471D'
,'Y36471S'
,'Y36490D'
,'Y36490S'
,'Y36491D'
,'Y36491S'
,'Y36500D'
,'Y36500S'
,'Y36501D'
,'Y36501S'
,'Y36510D'
,'Y36510S'
,'Y36511D'
,'Y36511S'
,'Y36520D'
,'Y36520S'
,'Y36521D'
,'Y36521S'
,'Y36530D'
,'Y36530S'
,'Y36531D'
,'Y36531S'
,'Y36540D'
,'Y36540S'
,'Y36541D'
,'Y36541S'
,'Y36590D'
,'Y36590S'
,'Y36591D'
,'Y36591S'
,'Y366X0D'
,'Y366X0S'
,'Y366X1D'
,'Y366X1S'
,'Y367X0D'
,'Y367X0S'
,'Y367X1D'
,'Y367X1S'
,'Y36810D'
,'Y36810S'
,'Y36811D'
,'Y36811S'
,'Y36820D'
,'Y36820S'
,'Y36821D'
,'Y36821S'
,'Y36880D'
,'Y36880S'
,'Y36881D'
,'Y36881S'
,'Y36890D'
,'Y36890S'
,'Y36891D'
,'Y36891S'
,'Y3690XD'
,'Y3690XS'
,'Y3691XD'
,'Y3691XS'
,'Y3692XD'
,'Y3692XS'
,'Y37000D'
,'Y37000S'
,'Y37001D'
,'Y37001S'
,'Y37010D'
,'Y37010S'
,'Y37011D'
,'Y37011S'
,'Y37020D'
,'Y37020S'
,'Y37021D'
,'Y37021S'
,'Y37030D'
,'Y37030S'
,'Y37031D'
,'Y37031S'
,'Y37040D'
,'Y37040S'
,'Y37041D'
,'Y37041S'
,'Y37050D'
,'Y37050S'
,'Y37051D'
,'Y37051S'
,'Y37090D'
,'Y37090S'
,'Y37091D'
,'Y37091S'
,'Y37100D'
,'Y37100S'
,'Y37101D'
,'Y37101S'
,'Y37110D'
,'Y37110S'
,'Y37111D'
,'Y37111S'
,'Y37120D'
,'Y37120S'
,'Y37121D'
,'Y37121S'
,'Y37130D'
,'Y37130S'
,'Y37131D'
,'Y37131S'
,'Y37140D'
,'Y37140S'
,'Y37141D'
,'Y37141S'
,'Y37190D'
,'Y37190S'
,'Y37191D'
,'Y37191S'
,'Y37200D'
,'Y37200S'
,'Y37201D'
,'Y37201S'
,'Y37210D'
,'Y37210S'
,'Y37211D'
,'Y37211S'
,'Y37220D'
,'Y37220S'
,'Y37221D'
,'Y37221S'
,'Y37230D'
,'Y37230S'
,'Y37231D'
,'Y37231S'
,'Y37240D'
,'Y37240S'
,'Y37241D'
,'Y37241S'
,'Y37250D'
,'Y37250S'
,'Y37251D'
,'Y37251S'
,'Y37260D'
,'Y37260S'
,'Y37261D'
,'Y37261S'
,'Y37270D'
,'Y37270S'
,'Y37271D'
,'Y37271S'
,'Y37290D'
,'Y37290S'
,'Y37291D'
,'Y37291S'
,'Y37300D'
,'Y37300S'
,'Y37301D'
,'Y37301S'
,'Y37310D'
,'Y37310S'
,'Y37311D'
,'Y37311S'
,'Y37320D'
,'Y37320S'
,'Y37321D'
,'Y37321S'
,'Y37330D'
,'Y37330S'
,'Y37331D'
,'Y37331S'
,'Y37390D'
,'Y37390S'
,'Y37391D'
,'Y37391S'
,'Y37410D'
,'Y37410S'
,'Y37411D'
,'Y37411S'
,'Y37420D'
,'Y37420S'
,'Y37421D'
,'Y37421S'
,'Y37430D'
,'Y37430S'
,'Y37431D'
,'Y37431S'
,'Y37440D'
,'Y37440S'
,'Y37441D'
,'Y37441S'
,'Y37450D'
,'Y37450S'
,'Y37451D'
,'Y37451S'
,'Y37460D'
,'Y37460S'
,'Y37461D'
,'Y37461S'
,'Y37470D'
,'Y37470S'
,'Y37471D'
,'Y37471S'
,'Y37490D'
,'Y37490S'
,'Y37491D'
,'Y37491S'
,'Y37500D'
,'Y37500S'
,'Y37501D'
,'Y37501S'
,'Y37510D'
,'Y37510S'
,'Y37511D'
,'Y37511S'
,'Y37520D'
,'Y37520S'
,'Y37521D'
,'Y37521S'
,'Y37530D'
,'Y37530S'
,'Y37531D'
,'Y37531S'
,'Y37540D'
,'Y37540S'
,'Y37541D'
,'Y37541S'
,'Y37590D'
,'Y37590S'
,'Y37591D'
,'Y37591S'
,'Y376X0D'
,'Y376X0S'
,'Y376X1D'
,'Y376X1S'
,'Y377X0D'
,'Y377X0S'
,'Y377X1D'
,'Y377X1S'
,'Y3790XD'
,'Y3790XS'
,'Y3791XD'
,'Y3791XS'
,'Y3792XD'
,'Y3792XS'
,'Y92000'
,'Y92001'
,'Y92002'
,'Y92003'
,'Y92007'
,'Y92008'
,'Y92009'
,'Y92010'
,'Y92011'
,'Y92012'
,'Y92013'
,'Y92014'
,'Y92015'
,'Y92016'
,'Y92017'
,'Y92018'
,'Y92019'
,'Y92020'
,'Y92021'
,'Y92022'
,'Y92023'
,'Y92024'
,'Y92025'
,'Y92026'
,'Y92027'
,'Y92028'
,'Y92029'
,'Y92030'
,'Y92031'
,'Y92032'
,'Y92038'
,'Y92039'
,'Y92040'
,'Y92041'
,'Y92042'
,'Y92043'
,'Y92044'
,'Y92045'
,'Y92046'
,'Y92048'
,'Y92049'
,'Y92090'
,'Y92091'
,'Y92092'
,'Y92093'
,'Y92094'
,'Y92095'
,'Y92096'
,'Y92098'
,'Y92099'
,'Y9210'
,'Y92110'
,'Y92111'
,'Y92112'
,'Y92113'
,'Y92114'
,'Y92115'
,'Y92116'
,'Y92118'
,'Y92119'
,'Y92120'
,'Y92121'
,'Y92122'
,'Y92123'
,'Y92124'
,'Y92125'
,'Y92126'
,'Y92128'
,'Y92129'
,'Y92130'
,'Y92131'
,'Y92133'
,'Y92135'
,'Y92136'
,'Y92137'
,'Y92138'
,'Y92139'
,'Y92140'
,'Y92141'
,'Y92142'
,'Y92143'
,'Y92146'
,'Y92147'
,'Y92148'
,'Y92149'
,'Y92150'
,'Y92151'
,'Y92152'
,'Y92153'
,'Y92154'
,'Y92155'
,'Y92156'
,'Y92157'
,'Y92158'
,'Y92159'
,'Y92160'
,'Y92161'
,'Y92162'
,'Y92163'
,'Y92168'
,'Y92169'
,'Y92190'
,'Y92191'
,'Y92192'
,'Y92193'
,'Y92194'
,'Y92195'
,'Y92196'
,'Y92197'
,'Y92198'
,'Y92199'
,'Y92210'
,'Y92211'
,'Y92212'
,'Y92213'
,'Y92214'
,'Y92215'
,'Y92218'
,'Y92219'
,'Y9222'
,'Y92230'
,'Y92231'
,'Y92232'
,'Y92233'
,'Y92234'
,'Y92238'
,'Y92239'
,'Y92240'
,'Y92241'
,'Y92242'
,'Y92243'
,'Y92248'
,'Y92250'
,'Y92251'
,'Y92252'
,'Y92253'
,'Y92254'
,'Y92258'
,'Y9226'
,'Y9229'
,'Y92310'
,'Y92311'
,'Y92312'
,'Y92318'
,'Y92320'
,'Y92321'
,'Y92322'
,'Y92328'
,'Y92330'
,'Y92331'
,'Y9234'
,'Y9239'
,'Y92410'
,'Y92411'
,'Y92412'
,'Y92413'
,'Y92414'
,'Y92415'
,'Y92480'
,'Y92481'
,'Y92482'
,'Y92488'
,'Y92510'
,'Y92511'
,'Y92512'
,'Y92513'
,'Y92520'
,'Y92521'
,'Y92522'
,'Y92523'
,'Y92524'
,'Y92530'
,'Y92531'
,'Y92532'
,'Y92538'
,'Y9259'
,'Y9261'
,'Y9262'
,'Y9263'
,'Y9264'
,'Y9265'
,'Y9269'
,'Y9271'
,'Y9272'
,'Y9273'
,'Y9274'
,'Y9279'
,'Y92810'
,'Y92811'
,'Y92812'
,'Y92813'
,'Y92814'
,'Y92815'
,'Y92816'
,'Y92818'
,'Y92820'
,'Y92821'
,'Y92828'
,'Y92830'
,'Y92831'
,'Y92832'
,'Y92833'
,'Y92834'
,'Y92838'
,'Y9284'
,'Y9285'
,'Y9286'
,'Y9289'
,'Y929'
,'Y9301'
,'Y9302'
,'Y9311'
,'Y9312'
,'Y9313'
,'Y9314'
,'Y9315'
,'Y9316'
,'Y9317'
,'Y9318'
,'Y9319'
,'Y9321'
,'Y9322'
,'Y9323'
,'Y9324'
,'Y9329'
,'Y9331'
,'Y9332'
,'Y9333'
,'Y9334'
,'Y9335'
,'Y9339'
,'Y9341'
,'Y9342'
,'Y9343'
,'Y9344'
,'Y9345'
,'Y9349'
,'Y9351'
,'Y9352'
,'Y9353'
,'Y9354'
,'Y9355'
,'Y9356'
,'Y9357'
,'Y9359'
,'Y9361'
,'Y9362'
,'Y9363'
,'Y9364'
,'Y9365'
,'Y9366'
,'Y9367'
,'Y9368'
,'Y9369'
,'Y936A'
,'Y9371'
,'Y9372'
,'Y9373'
,'Y9374'
,'Y9375'
,'Y9379'
,'Y9381'
,'Y9382'
,'Y9383'
,'Y9384'
,'Y9389'
,'Y939'
,'Y93A1'
,'Y93A2'
,'Y93A3'
,'Y93A4'
,'Y93A5'
,'Y93A6'
,'Y93A9'
,'Y93B1'
,'Y93B2'
,'Y93B3'
,'Y93B4'
,'Y93B9'
,'Y93C1'
,'Y93C2'
,'Y93C9'
,'Y93D1'
,'Y93D2'
,'Y93D3'
,'Y93D9'
,'Y93E1'
,'Y93E2'
,'Y93E3'
,'Y93E4'
,'Y93E5'
,'Y93E6'
,'Y93E8'
,'Y93E9'
,'Y93F1'
,'Y93F2'
,'Y93F9'
,'Y93G1'
,'Y93G2'
,'Y93G3'
,'Y93G9'
,'Y93H1'
,'Y93H2'
,'Y93H3'
,'Y93H9'
,'Y93I1'
,'Y93I9'
,'Y93J1'
,'Y93J2'
,'Y93J3'
,'Y93J4'
,'Y93K1'
,'Y93K2'
,'Y93K3'
,'Y93K9'
,'Y990'
,'Y991'
,'Y992'
,'Y998'
,'Y999'
,'Z0000'
,'Z0001'
,'Z00110'
,'Z00111'
,'Z00121'
,'Z00129'
,'Z002'
,'Z003'
,'Z005'
,'Z006'
,'Z0070'
,'Z0071'
,'Z008'
,'Z0100'
,'Z0101'
,'Z0110'
,'Z01110'
,'Z01118'
,'Z0112'
,'Z0120'
,'Z0121'
,'Z0130'
,'Z0131'
,'Z01411'
,'Z01419'
,'Z0142'
,'Z01810'
,'Z01811'
,'Z01812'
,'Z01818'
,'Z0182'
,'Z0183'
,'Z0184'
,'Z0189'
,'Z020'
,'Z021'
,'Z022'
,'Z023'
,'Z024'
,'Z025'
,'Z026'
,'Z0271'
,'Z0279'
,'Z0281'
,'Z0282'
,'Z0283'
,'Z0289'
,'Z029'
,'Z036'
,'Z0371'
,'Z0372'
,'Z0373'
,'Z0374'
,'Z0375'
,'Z0379'
,'Z03810'
,'Z03818'
,'Z0389'
,'Z08'
,'Z09'
,'Z110'
,'Z111'
,'Z112'
,'Z113'
,'Z114'
,'Z1151'
,'Z1159'
,'Z116'
,'Z118'
,'Z119'
,'Z120'
,'Z1210'
,'Z1211'
,'Z1212'
,'Z1213'
,'Z122'
,'Z1231'
,'Z1239'
,'Z124'
,'Z125'
,'Z126'
,'Z1271'
,'Z1272'
,'Z1273'
,'Z1279'
,'Z1281'
,'Z1282'
,'Z1283'
,'Z1289'
,'Z129'
,'Z130'
,'Z131'
,'Z1321'
,'Z13220'
,'Z13228'
,'Z1329'
,'Z134'
,'Z135'
,'Z136'
,'Z1371'
,'Z1379'
,'Z13810'
,'Z13811'
,'Z13818'
,'Z13820'
,'Z13828'
,'Z1383'
,'Z1384'
,'Z13850'
,'Z13858'
,'Z1388'
,'Z1389'
,'Z139'
,'Z1401'
,'Z1402'
,'Z141'
,'Z148'
,'Z1501'
,'Z1502'
,'Z1503'
,'Z1504'
,'Z1509'
,'Z1581'
,'Z1589'
,'Z170'
,'Z171'
,'Z1801'
,'Z1809'
,'Z1810'
,'Z1811'
,'Z1812'
,'Z182'
,'Z1831'
,'Z1832'
,'Z1833'
,'Z1839'
,'Z1881'
,'Z1883'
,'Z1889'
,'Z189'
,'Z220'
,'Z221'
,'Z222'
,'Z2231'
,'Z22321'
,'Z22322'
,'Z22330'
,'Z22338'
,'Z2239'
,'Z224'
,'Z2250'
,'Z2251'
,'Z2252'
,'Z2259'
,'Z226'
,'Z228'
,'Z229'
,'Z23'
,'Z2801'
,'Z2802'
,'Z2803'
,'Z2804'
,'Z2809'
,'Z281'
,'Z2820'
,'Z2821'
,'Z2829'
,'Z283'
,'Z2881'
,'Z2882'
,'Z2889'
,'Z289'
,'Z30011'
,'Z30012'
,'Z30013'
,'Z30014'
,'Z30018'
,'Z30019'
,'Z3002'
,'Z3009'
,'Z302'
,'Z3040'
,'Z3041'
,'Z3042'
,'Z30430'
,'Z30431'
,'Z30432'
,'Z30433'
,'Z3049'
,'Z308'
,'Z309'
,'Z310'
,'Z3141'
,'Z3142'
,'Z31430'
,'Z31438'
,'Z31440'
,'Z31441'
,'Z31448'
,'Z3149'
,'Z315'
,'Z3161'
,'Z3162'
,'Z3169'
,'Z3181'
,'Z3182'
,'Z3183'
,'Z3184'
,'Z3189'
,'Z319'
,'Z3400'
,'Z3401'
,'Z3402'
,'Z3403'
,'Z3480'
,'Z3481'
,'Z3482'
,'Z3483'
,'Z3490'
,'Z3491'
,'Z3492'
,'Z3493'
,'Z36'
,'Z370'
,'Z371'
,'Z372'
,'Z373'
,'Z374'
,'Z3750'
,'Z3751'
,'Z3752'
,'Z3753'
,'Z3754'
,'Z3759'
,'Z3760'
,'Z3761'
,'Z3762'
,'Z3763'
,'Z3764'
,'Z3769'
,'Z377'
,'Z379'
,'Z3800'
,'Z3801'
,'Z381'
,'Z382'
,'Z3830'
,'Z3831'
,'Z384'
,'Z385'
,'Z3861'
,'Z3862'
,'Z3863'
,'Z3864'
,'Z3865'
,'Z3866'
,'Z3868'
,'Z3869'
,'Z387'
,'Z388'
,'Z390'
,'Z391'
,'Z392'
,'Z411'
,'Z412'
,'Z413'
,'Z418'
,'Z419'
,'Z421'
,'Z428'
,'Z430'
,'Z431'
,'Z432'
,'Z433'
,'Z434'
,'Z435'
,'Z436'
,'Z437'
,'Z438'
,'Z439'
,'Z44001'
,'Z44002'
,'Z44009'
,'Z44011'
,'Z44012'
,'Z44019'
,'Z44021'
,'Z44022'
,'Z44029'
,'Z44101'
,'Z44102'
,'Z44109'
,'Z44111'
,'Z44112'
,'Z44119'
,'Z44121'
,'Z44122'
,'Z44129'
,'Z4420'
,'Z4421'
,'Z4422'
,'Z4430'
,'Z4431'
,'Z4432'
,'Z448'
,'Z449'
,'Z45010'
,'Z45018'
,'Z4502'
,'Z4509'
,'Z451'
,'Z452'
,'Z4531'
,'Z45320'
,'Z45321'
,'Z45328'
,'Z4541'
,'Z4542'
,'Z4549'
,'Z45811'
,'Z45812'
,'Z45819'
,'Z4582'
,'Z4589'
,'Z459'
,'Z460'
,'Z461'
,'Z462'
,'Z463'
,'Z464'
,'Z4651'
,'Z4659'
,'Z466'
,'Z4681'
,'Z4682'
,'Z4689'
,'Z469'
,'Z4781'
,'Z4782'
,'Z4789'
,'Z4901'
,'Z4902'
,'Z4931'
,'Z4932'
,'Z510'
,'Z5111'
,'Z5112'
,'Z515'
,'Z5181'
,'Z5189'
,'Z52000'
,'Z52001'
,'Z52008'
,'Z52010'
,'Z52011'
,'Z52018'
,'Z52090'
,'Z52091'
,'Z52098'
,'Z5210'
,'Z5211'
,'Z5219'
,'Z5220'
,'Z5221'
,'Z5229'
,'Z523'
,'Z524'
,'Z525'
,'Z526'
,'Z52810'
,'Z52811'
,'Z52812'
,'Z52813'
,'Z52819'
,'Z5289'
,'Z529'
,'Z590'
,'Z591'
,'Z592'
,'Z593'
,'Z594'
,'Z595'
,'Z596'
,'Z597'
,'Z598'
,'Z599'
,'Z630'
,'Z631'
,'Z6331'
,'Z6332'
,'Z634'
,'Z635'
,'Z636'
,'Z6371'
,'Z6372'
,'Z6379'
,'Z638'
,'Z639'
,'Z650'
,'Z651'
,'Z652'
,'Z653'
,'Z654'
,'Z655'
,'Z658'
,'Z659'
,'Z6710'
,'Z6711'
,'Z6720'
,'Z6721'
,'Z6730'
,'Z6731'
,'Z6740'
,'Z6741'
,'Z6790'
,'Z6791'
,'Z681'
,'Z6820'
,'Z6821'
,'Z6822'
,'Z6823'
,'Z6824'
,'Z6825'
,'Z6826'
,'Z6827'
,'Z6828'
,'Z6829'
,'Z6830'
,'Z6831'
,'Z6832'
,'Z6833'
,'Z6834'
,'Z6835'
,'Z6836'
,'Z6837'
,'Z6838'
,'Z6839'
,'Z6841'
,'Z6842'
,'Z6843'
,'Z6844'
,'Z6845'
,'Z6851'
,'Z6852'
,'Z6853'
,'Z6854'
,'Z720'
,'Z723'
,'Z724'
,'Z7251'
,'Z7252'
,'Z7253'
,'Z726'
,'Z72810'
,'Z72811'
,'Z72820'
,'Z72821'
,'Z7289'
,'Z729'
,'Z7401'
,'Z760'
,'Z761'
,'Z762'
,'Z763'
,'Z764'
,'Z765'
,'Z7681'
,'Z7682'
,'Z7689'
,'Z77110'
,'Z77111'
,'Z77112'
,'Z77118'
,'Z77120'
,'Z77121'
,'Z77122'
,'Z77123'
,'Z77128'
,'Z780'
,'Z781'
,'Z789'
,'Z7901'
,'Z7902'
,'Z791'
,'Z792'
,'Z793'
,'Z794'
,'Z7951'
,'Z7952'
,'Z79810'
,'Z79811'
,'Z79818'
,'Z7982'
,'Z7983'
,'Z79890'
,'Z79891'
,'Z79899'
,'Z800'
,'Z801'
,'Z802'
,'Z803'
,'Z8041'
,'Z8042'
,'Z8043'
,'Z8049'
,'Z8051'
,'Z8052'
,'Z8059'
,'Z806'
,'Z807'
,'Z808'
,'Z809'
,'Z810'
,'Z811'
,'Z812'
,'Z813'
,'Z814'
,'Z818'
,'Z820'
,'Z821'
,'Z822'
,'Z823'
,'Z8241'
,'Z8249'
,'Z825'
,'Z8261'
,'Z8262'
,'Z8269'
,'Z8271'
,'Z8279'
,'Z828'
,'Z830'
,'Z831'
,'Z832'
,'Z833'
,'Z8341'
,'Z8349'
,'Z83511'
,'Z83518'
,'Z8352'
,'Z836'
,'Z8371'
,'Z8379'
,'Z840'
,'Z841'
,'Z842'
,'Z843'
,'Z8481'
,'Z8489'
,'Z8500'
,'Z8501'
,'Z85020'
,'Z85028'
,'Z85030'
,'Z85038'
,'Z85040'
,'Z85048'
,'Z8505'
,'Z85060'
,'Z85068'
,'Z8507'
,'Z8509'
,'Z85110'
,'Z85118'
,'Z8512'
,'Z8520'
,'Z8521'
,'Z8522'
,'Z85230'
,'Z85238'
,'Z8529'
,'Z853'
,'Z8540'
,'Z8541'
,'Z8542'
,'Z8543'
,'Z8544'
,'Z8545'
,'Z8546'
,'Z8547'
,'Z8548'
,'Z8549'
,'Z8550'
,'Z8551'
,'Z85520'
,'Z85528'
,'Z8553'
,'Z8554'
,'Z8559'
,'Z856'
,'Z8571'
,'Z8572'
,'Z8579'
,'Z85810'
,'Z85818'
,'Z85819'
,'Z85820'
,'Z85821'
,'Z85828'
,'Z85830'
,'Z85831'
,'Z85840'
,'Z85841'
,'Z85848'
,'Z85850'
,'Z85858'
,'Z8589'
,'Z859'
,'Z86000'
,'Z86001'
,'Z86008'
,'Z86010'
,'Z86011'
,'Z86012'
,'Z86018'
,'Z8603'
,'Z8611'
,'Z8612'
,'Z8613'
,'Z8614'
,'Z8619'
,'Z862'
,'Z8631'
,'Z8632'
,'Z8639'
,'Z8651'
,'Z8659'
,'Z8661'
,'Z8669'
,'Z86711'
,'Z86718'
,'Z8672'
,'Z8673'
,'Z8674'
,'Z8679'
,'Z8701'
,'Z8709'
,'Z8711'
,'Z8719'
,'Z872'
,'Z87310'
,'Z87311'
,'Z87312'
,'Z8739'
,'Z87410'
,'Z87411'
,'Z87412'
,'Z8742'
,'Z87430'
,'Z87438'
,'Z87440'
,'Z87441'
,'Z87442'
,'Z87448'
,'Z8751'
,'Z8759'
,'Z87710'
,'Z87718'
,'Z87720'
,'Z87721'
,'Z87728'
,'Z87730'
,'Z87738'
,'Z8774'
,'Z8775'
,'Z8776'
,'Z87790'
,'Z87798'
,'Z8781'
,'Z87820'
,'Z87821'
,'Z87828'
,'Z87890'
,'Z87891'
,'Z87892'
,'Z87898'
,'Z880'
,'Z881'
,'Z882'
,'Z883'
,'Z884'
,'Z885'
,'Z886'
,'Z887'
,'Z888'
,'Z889'
,'Z89011'
,'Z89012'
,'Z89019'
,'Z89021'
,'Z89022'
,'Z89029'
,'Z89111'
,'Z89112'
,'Z89119'
,'Z89121'
,'Z89122'
,'Z89129'
,'Z89201'
,'Z89202'
,'Z89209'
,'Z89211'
,'Z89212'
,'Z89219'
,'Z89221'
,'Z89222'
,'Z89229'
,'Z89231'
,'Z89232'
,'Z89239'
,'Z89411'
,'Z89412'
,'Z89419'
,'Z89421'
,'Z89422'
,'Z89429'
,'Z89431'
,'Z89432'
,'Z89439'
,'Z89441'
,'Z89442'
,'Z89449'
,'Z89511'
,'Z89512'
,'Z89519'
,'Z89521'
,'Z89522'
,'Z89529'
,'Z89611'
,'Z89612'
,'Z89619'
,'Z89621'
,'Z89622'
,'Z89629'
,'Z899'
,'Z90710'
,'Z91010'
,'Z91011'
,'Z91012'
,'Z91013'
,'Z91018'
,'Z9102'
,'Z91030'
,'Z91038'
,'Z91040'
,'Z91041'
,'Z91048'
,'Z9109'
,'Z91410'
,'Z91411'
,'Z91412'
,'Z91419'
,'Z9149'
,'Z915'
,'Z9181'
,'Z9182'
,'Z9183'
,'Z9189'
,'Z920'
,'Z9221'
,'Z9222'
,'Z9223'
,'Z92240'
,'Z92241'
,'Z9225'
,'Z9229'
,'Z923'
,'Z9281'
,'Z9282'
,'Z9283'
,'Z9289'
,'Z930'
,'Z931'
,'Z932'
,'Z933'
,'Z934'
,'Z9350'
,'Z9351'
,'Z9352'
,'Z9359'
,'Z936'
,'Z938'
,'Z939'
,'Z940'
,'Z941'
,'Z942'
,'Z943'
,'Z944'
,'Z945'
,'Z946'
,'Z947'
,'Z9481'
,'Z9482'
,'Z9483'
,'Z9484'
,'Z9489'
,'Z949'
,'Z950'
,'Z951'
,'Z952'
,'Z953'
,'Z954'
,'Z955'
,'Z95810'
,'Z95811'
,'Z95812'
,'Z95818'
,'Z95820'
,'Z95828'
,'Z959'
,'Z970'
,'Z9710'
,'Z9711'
,'Z9712'
,'Z9713'
,'Z9714'
,'Z9715'
,'Z9716'
,'Z972'
,'Z973'
,'Z974'
,'Z975'
,'Z978'
,'Z980'
,'Z981'
,'Z982'
,'Z983'
,'Z9841'
,'Z9842'
,'Z9849'
,'Z9851'
,'Z9852'
,'Z9861'
,'Z9862'
,'Z98810'
,'Z98811'
,'Z98818'
,'Z9882'
,'Z9883'
,'Z9884'
,'Z9885'
,'Z9886'
,'Z98870'
,'Z98871'
,'Z9889'
,'Z990'
,'Z9911'
,'Z9912'
,'Z992'
,'Z993'
,'Z9981'
,'Z9989'='1'
OTHER='0'
; 

RUN;
